/**
 * Edited by Sergey Zhdanov
 * In IE hasCustomToString always return true, known bug https://github.com/angular/angular.js/issues/16207,
 * Object.prototype.toString gets changed after angular toString being set, therefore hasCustomToString always true.
 * To Fix this, line: "toString = Object.prototype.toString;" at 36499
 */

/**
 * @license AngularJS v1.8.0
 * (c) 2010-2020 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function(window) {'use strict';

/* exported
  minErrConfig,
  errorHandlingConfig,
  isValidObjectMaxDepth
*/

var minErrConfig = {
  objectMaxDepth: 5,
  urlErrorParamsEnabled: true
};

/**
 * @ngdoc function
 * @name angular.errorHandlingConfig
 * @module ng
 * @kind function
 *
 * @description
 * Configure several aspects of error handling in AngularJS if used as a setter or return the
 * current configuration if used as a getter. The following options are supported:
 *
 * - **objectMaxDepth**: The maximum depth to which objects are traversed when stringified for error messages.
 *
 * Omitted or undefined options will leave the corresponding configuration values unchanged.
 *
 * @param {Object=} config - The configuration object. May only contain the options that need to be
 *     updated. Supported keys:
 *
 * * `objectMaxDepth`  **{Number}** - The max depth for stringifying objects. Setting to a
 *   non-positive or non-numeric value, removes the max depth limit.
 *   Default: 5
 *
 * * `urlErrorParamsEnabled`  **{Boolean}** - Specifies whether the generated error url will
 *   contain the parameters of the thrown error. Disabling the parameters can be useful if the
 *   generated error url is very long.
 *
 *   Default: true. When used without argument, it returns the current value.
 */
function errorHandlingConfig(config) {
  if (isObject(config)) {
    if (isDefined(config.objectMaxDepth)) {
      minErrConfig.objectMaxDepth = isValidObjectMaxDepth(config.objectMaxDepth) ? config.objectMaxDepth : NaN;
    }
    if (isDefined(config.urlErrorParamsEnabled) && isBoolean(config.urlErrorParamsEnabled)) {
      minErrConfig.urlErrorParamsEnabled = config.urlErrorParamsEnabled;
    }
  } else {
    return minErrConfig;
  }
}

/**
 * @private
 * @param {Number} maxDepth
 * @return {boolean}
 */
function isValidObjectMaxDepth(maxDepth) {
  return isNumber(maxDepth) && maxDepth > 0;
}


/**
 * @description
 *
 * This object provides a utility for producing rich Error messages within
 * AngularJS. It can be called as follows:
 *
 * var exampleMinErr = minErr('example');
 * throw exampleMinErr('one', 'This {0} is {1}', foo, bar);
 *
 * The above creates an instance of minErr in the example namespace. The
 * resulting error will have a namespaced error code of example.one.  The
 * resulting error will replace {0} with the value of foo, and {1} with the
 * value of bar. The object is not restricted in the number of arguments it can
 * take.
 *
 * If fewer arguments are specified than necessary for interpolation, the extra
 * interpolation markers will be preserved in the final string.
 *
 * Since data will be parsed statically during a build step, some restrictions
 * are applied with respect to how minErr instances are created and called.
 * Instances should have names of the form namespaceMinErr for a minErr created
 * using minErr('namespace'). Error codes, namespaces and template strings
 * should all be static strings, not variables or general expressions.
 *
 * @param {string} module The namespace to use for the new minErr instance.
 * @param {function} ErrorConstructor Custom error constructor to be instantiated when returning
 *   error from returned function, for cases when a particular type of error is useful.
 * @returns {function(code:string, template:string, ...templateArgs): Error} minErr instance
 */

function minErr(module, ErrorConstructor) {
  ErrorConstructor = ErrorConstructor || Error;

  var url = 'https://errors.angularjs.org/1.8.0/';
  var regex = url.replace('.', '\\.') + '[\\s\\S]*';
  var errRegExp = new RegExp(regex, 'g');

  return function() {
    var code = arguments[0],
      template = arguments[1],
      message = '[' + (module ? module + ':' : '') + code + '] ',
      templateArgs = sliceArgs(arguments, 2).map(function(arg) {
        return toDebugString(arg, minErrConfig.objectMaxDepth);
      }),
      paramPrefix, i;

    // A minErr message has two parts: the message itself and the url that contains the
    // encoded message.
    // The message's parameters can contain other error messages which also include error urls.
    // To prevent the messages from getting too long, we strip the error urls from the parameters.

    message += template.replace(/\{\d+\}/g, function(match) {
      var index = +match.slice(1, -1);

      if (index < templateArgs.length) {
        return templateArgs[index].replace(errRegExp, '');
      }

      return match;
    });

    message += '\n' + url + (module ? module + '/' : '') + code;

    if (minErrConfig.urlErrorParamsEnabled) {
      for (i = 0, paramPrefix = '?'; i < templateArgs.length; i++, paramPrefix = '&') {
        message += paramPrefix + 'p' + i + '=' + encodeURIComponent(templateArgs[i]);
      }
    }

    return new ErrorConstructor(message);
  };
}

/* We need to tell ESLint what variables are being exported */
/* exported
  angular,
  msie,
  jqLite,
  jQuery,
  slice,
  splice,
  push,
  toString,
  minErrConfig,
  errorHandlingConfig,
  isValidObjectMaxDepth,
  ngMinErr,
  angularModule,
  uid,
  REGEX_STRING_REGEXP,
  VALIDITY_STATE_PROPERTY,

  lowercase,
  uppercase,
  nodeName_,
  isArrayLike,
  forEach,
  forEachSorted,
  reverseParams,
  nextUid,
  setHashKey,
  extend,
  toInt,
  inherit,
  merge,
  noop,
  identity,
  valueFn,
  isUndefined,
  isDefined,
  isObject,
  isBlankObject,
  isString,
  isNumber,
  isNumberNaN,
  isDate,
  isError,
  isArray,
  isFunction,
  isRegExp,
  isWindow,
  isScope,
  isFile,
  isFormData,
  isBlob,
  isBoolean,
  isPromiseLike,
  trim,
  escapeForRegexp,
  isElement,
  makeMap,
  includes,
  arrayRemove,
  copy,
  simpleCompare,
  equals,
  csp,
  jq,
  concat,
  sliceArgs,
  bind,
  toJsonReplacer,
  toJson,
  fromJson,
  convertTimezoneToLocal,
  timezoneToOffset,
  addDateMinutes,
  startingTag,
  tryDecodeURIComponent,
  parseKeyValue,
  toKeyValue,
  encodeUriSegment,
  encodeUriQuery,
  angularInit,
  bootstrap,
  getTestability,
  snake_case,
  bindJQuery,
  assertArg,
  assertArgFn,
  assertNotHasOwnProperty,
  getter,
  getBlockNodes,
  hasOwnProperty,
  createMap,
  stringify,
  UNSAFE_restoreLegacyJqLiteXHTMLReplacement,

  NODE_TYPE_ELEMENT,
  NODE_TYPE_ATTRIBUTE,
  NODE_TYPE_TEXT,
  NODE_TYPE_COMMENT,
  NODE_TYPE_DOCUMENT,
  NODE_TYPE_DOCUMENT_FRAGMENT
*/

////////////////////////////////////

/**
 * @ngdoc module
 * @name ng
 * @module ng
 * @installation
 * @description
 *
 * The ng module is loaded by default when an AngularJS application is started. The module itself
 * contains the essential components for an AngularJS application to function. The table below
 * lists a high level breakdown of each of the services/factories, filters, directives and testing
 * components available within this core module.
 *
 */

var REGEX_STRING_REGEXP = /^\/(.+)\/([a-z]*)$/;

// The name of a form control's ValidityState property.
// This is used so that it's possible for internal tests to create mock ValidityStates.
var VALIDITY_STATE_PROPERTY = 'validity';


var hasOwnProperty = Object.prototype.hasOwnProperty;

/**
 * @private
 *
 * @description Converts the specified string to lowercase.
 * @param {string} string String to be converted to lowercase.
 * @returns {string} Lowercased string.
 */
var lowercase = function(string) {return isString(string) ? string.toLowerCase() : string;};

/**
 * @private
 *
 * @description Converts the specified string to uppercase.
 * @param {string} string String to be converted to uppercase.
 * @returns {string} Uppercased string.
 */
var uppercase = function(string) {return isString(string) ? string.toUpperCase() : string;};


var
    msie,             // holds major version number for IE, or NaN if UA is not IE.
    jqLite,           // delay binding since jQuery could be loaded after us.
    jQuery,           // delay binding
    slice             = [].slice,
    splice            = [].splice,
    push              = [].push,
    toString          = Object.prototype.toString,
    getPrototypeOf    = Object.getPrototypeOf,
    ngMinErr          = minErr('ng'),

    /** @name angular */
    angular           = window.angular || (window.angular = {}),
    angularModule,
    uid               = 0;

// Support: IE 9-11 only
/**
 * documentMode is an IE-only property
 * http://msdn.microsoft.com/en-us/library/ie/cc196988(v=vs.85).aspx
 */
msie = window.document.documentMode;


/**
 * @private
 * @param {*} obj
 * @return {boolean} Returns true if `obj` is an array or array-like object (NodeList, Arguments,
 *                   String ...)
 */
function isArrayLike(obj) {

  // `null`, `undefined` and `window` are not array-like
  if (obj == null || isWindow(obj)) return false;

  // arrays, strings and jQuery/jqLite objects are array like
  // * jqLite is either the jQuery or jqLite constructor function
  // * we have to check the existence of jqLite first as this method is called
  //   via the forEach method when constructing the jqLite object in the first place
  if (isArray(obj) || isString(obj) || (jqLite && obj instanceof jqLite)) return true;

  // Support: iOS 8.2 (not reproducible in simulator)
  // "length" in obj used to prevent JIT error (gh-11508)
  var length = 'length' in Object(obj) && obj.length;

  // NodeList objects (with `item` method) and
  // other objects with suitable length characteristics are array-like
  return isNumber(length) && (length >= 0 && (length - 1) in obj || typeof obj.item === 'function');

}

/**
 * @ngdoc function
 * @name angular.forEach
 * @module ng
 * @kind function
 *
 * @description
 * Invokes the `iterator` function once for each item in `obj` collection, which can be either an
 * object or an array. The `iterator` function is invoked with `iterator(value, key, obj)`, where `value`
 * is the value of an object property or an array element, `key` is the object property key or
 * array element index and obj is the `obj` itself. Specifying a `context` for the function is optional.
 *
 * It is worth noting that `.forEach` does not iterate over inherited properties because it filters
 * using the `hasOwnProperty` method.
 *
 * Unlike ES262's
 * [Array.prototype.forEach](http://www.ecma-international.org/ecma-262/5.1/#sec-15.4.4.18),
 * providing 'undefined' or 'null' values for `obj` will not throw a TypeError, but rather just
 * return the value provided.
 *
   ```js
     var values = {name: 'misko', gender: 'male'};
     var log = [];
     angular.forEach(values, function(value, key) {
       this.push(key + ': ' + value);
     }, log);
     expect(log).toEqual(['name: misko', 'gender: male']);
   ```
 *
 * @param {Object|Array} obj Object to iterate over.
 * @param {Function} iterator Iterator function.
 * @param {Object=} context Object to become context (`this`) for the iterator function.
 * @returns {Object|Array} Reference to `obj`.
 */

function forEach(obj, iterator, context) {
  var key, length;
  if (obj) {
    if (isFunction(obj)) {
      for (key in obj) {
        if (key !== 'prototype' && key !== 'length' && key !== 'name' && obj.hasOwnProperty(key)) {
          iterator.call(context, obj[key], key, obj);
        }
      }
    } else if (isArray(obj) || isArrayLike(obj)) {
      var isPrimitive = typeof obj !== 'object';
      for (key = 0, length = obj.length; key < length; key++) {
        if (isPrimitive || key in obj) {
          iterator.call(context, obj[key], key, obj);
        }
      }
    } else if (obj.forEach && obj.forEach !== forEach) {
        obj.forEach(iterator, context, obj);
    } else if (isBlankObject(obj)) {
      // createMap() fast path --- Safe to avoid hasOwnProperty check because prototype chain is empty
      for (key in obj) {
        iterator.call(context, obj[key], key, obj);
      }
    } else if (typeof obj.hasOwnProperty === 'function') {
      // Slow path for objects inheriting Object.prototype, hasOwnProperty check needed
      for (key in obj) {
        if (obj.hasOwnProperty(key)) {
          iterator.call(context, obj[key], key, obj);
        }
      }
    } else {
      // Slow path for objects which do not have a method `hasOwnProperty`
      for (key in obj) {
        if (hasOwnProperty.call(obj, key)) {
          iterator.call(context, obj[key], key, obj);
        }
      }
    }
  }
  return obj;
}

function forEachSorted(obj, iterator, context) {
  var keys = Object.keys(obj).sort();
  for (var i = 0; i < keys.length; i++) {
    iterator.call(context, obj[keys[i]], keys[i]);
  }
  return keys;
}


/**
 * when using forEach the params are value, key, but it is often useful to have key, value.
 * @param {function(string, *)} iteratorFn
 * @returns {function(*, string)}
 */
function reverseParams(iteratorFn) {
  return function(value, key) {iteratorFn(key, value);};
}

/**
 * A consistent way of creating unique IDs in angular.
 *
 * Using simple numbers allows us to generate 28.6 million unique ids per second for 10 years before
 * we hit number precision issues in JavaScript.
 *
 * Math.pow(2,53) / 60 / 60 / 24 / 365 / 10 = 28.6M
 *
 * @returns {number} an unique alpha-numeric string
 */
function nextUid() {
  return ++uid;
}


/**
 * Set or clear the hashkey for an object.
 * @param obj object
 * @param h the hashkey (!truthy to delete the hashkey)
 */
function setHashKey(obj, h) {
  if (h) {
    obj.$$hashKey = h;
  } else {
    delete obj.$$hashKey;
  }
}


function baseExtend(dst, objs, deep) {
  var h = dst.$$hashKey;

  for (var i = 0, ii = objs.length; i < ii; ++i) {
    var obj = objs[i];
    if (!isObject(obj) && !isFunction(obj)) continue;
    var keys = Object.keys(obj);
    for (var j = 0, jj = keys.length; j < jj; j++) {
      var key = keys[j];
      var src = obj[key];

      if (deep && isObject(src)) {
        if (isDate(src)) {
          dst[key] = new Date(src.valueOf());
        } else if (isRegExp(src)) {
          dst[key] = new RegExp(src);
        } else if (src.nodeName) {
          dst[key] = src.cloneNode(true);
        } else if (isElement(src)) {
          dst[key] = src.clone();
        } else {
          if (key !== '__proto__') {
            if (!isObject(dst[key])) dst[key] = isArray(src) ? [] : {};
            baseExtend(dst[key], [src], true);
          }
        }
      } else {
        dst[key] = src;
      }
    }
  }

  setHashKey(dst, h);
  return dst;
}

/**
 * @ngdoc function
 * @name angular.extend
 * @module ng
 * @kind function
 *
 * @description
 * Extends the destination object `dst` by copying own enumerable properties from the `src` object(s)
 * to `dst`. You can specify multiple `src` objects. If you want to preserve original objects, you can do so
 * by passing an empty object as the target: `var object = angular.extend({}, object1, object2)`.
 *
 * **Note:** Keep in mind that `angular.extend` does not support recursive merge (deep copy). Use
 * {@link angular.merge} for this.
 *
 * @param {Object} dst Destination object.
 * @param {...Object} src Source object(s).
 * @returns {Object} Reference to `dst`.
 */
function extend(dst) {
  return baseExtend(dst, slice.call(arguments, 1), false);
}


/**
* @ngdoc function
* @name angular.merge
* @module ng
* @kind function
*
* @description
* Deeply extends the destination object `dst` by copying own enumerable properties from the `src` object(s)
* to `dst`. You can specify multiple `src` objects. If you want to preserve original objects, you can do so
* by passing an empty object as the target: `var object = angular.merge({}, object1, object2)`.
*
* Unlike {@link angular.extend extend()}, `merge()` recursively descends into object properties of source
* objects, performing a deep copy.
*
* @deprecated
* sinceVersion="1.6.5"
* This function is deprecated, but will not be removed in the 1.x lifecycle.
* There are edge cases (see {@link angular.merge#known-issues known issues}) that are not
* supported by this function. We suggest using another, similar library for all-purpose merging,
* such as [lodash's merge()](https://lodash.com/docs/4.17.4#merge).
*
* @knownIssue
* This is a list of (known) object types that are not handled correctly by this function:
* - [`Blob`](https://developer.mozilla.org/docs/Web/API/Blob)
* - [`MediaStream`](https://developer.mozilla.org/docs/Web/API/MediaStream)
* - [`CanvasGradient`](https://developer.mozilla.org/docs/Web/API/CanvasGradient)
* - AngularJS {@link $rootScope.Scope scopes};
*
* `angular.merge` also does not support merging objects with circular references.
*
* @param {Object} dst Destination object.
* @param {...Object} src Source object(s).
* @returns {Object} Reference to `dst`.
*/
function merge(dst) {
  return baseExtend(dst, slice.call(arguments, 1), true);
}



function toInt(str) {
  return parseInt(str, 10);
}

var isNumberNaN = Number.isNaN || function isNumberNaN(num) {
  // eslint-disable-next-line no-self-compare
  return num !== num;
};


function inherit(parent, extra) {
  return extend(Object.create(parent), extra);
}

/**
 * @ngdoc function
 * @name angular.noop
 * @module ng
 * @kind function
 *
 * @description
 * A function that performs no operations. This function can be useful when writing code in the
 * functional style.
   ```js
     function foo(callback) {
       var result = calculateResult();
       (callback || angular.noop)(result);
     }
   ```
 */
function noop() {}
noop.$inject = [];


/**
 * @ngdoc function
 * @name angular.identity
 * @module ng
 * @kind function
 *
 * @description
 * A function that returns its first argument. This function is useful when writing code in the
 * functional style.
 *
   ```js
   function transformer(transformationFn, value) {
     return (transformationFn || angular.identity)(value);
   };

   // E.g.
   function getResult(fn, input) {
     return (fn || angular.identity)(input);
   };

   getResult(function(n) { return n * 2; }, 21);   // returns 42
   getResult(null, 21);                            // returns 21
   getResult(undefined, 21);                       // returns 21
   ```
 *
 * @param {*} value to be returned.
 * @returns {*} the value passed in.
 */
function identity($) {return $;}
identity.$inject = [];


function valueFn(value) {return function valueRef() {return value;};}

function hasCustomToString(obj) {
  return isFunction(obj.toString) && obj.toString !== toString;
}


/**
 * @ngdoc function
 * @name angular.isUndefined
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is undefined.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is undefined.
 */
function isUndefined(value) {return typeof value === 'undefined';}


/**
 * @ngdoc function
 * @name angular.isDefined
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is defined.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is defined.
 */
function isDefined(value) {return typeof value !== 'undefined';}


/**
 * @ngdoc function
 * @name angular.isObject
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is an `Object`. Unlike `typeof` in JavaScript, `null`s are not
 * considered to be objects. Note that JavaScript arrays are objects.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is an `Object` but not `null`.
 */
function isObject(value) {
  // http://jsperf.com/isobject4
  return value !== null && typeof value === 'object';
}


/**
 * Determine if a value is an object with a null prototype
 *
 * @returns {boolean} True if `value` is an `Object` with a null prototype
 */
function isBlankObject(value) {
  return value !== null && typeof value === 'object' && !getPrototypeOf(value);
}


/**
 * @ngdoc function
 * @name angular.isString
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `String`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `String`.
 */
function isString(value) {return typeof value === 'string';}


/**
 * @ngdoc function
 * @name angular.isNumber
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `Number`.
 *
 * This includes the "special" numbers `NaN`, `+Infinity` and `-Infinity`.
 *
 * If you wish to exclude these then you can use the native
 * [`isFinite'](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/isFinite)
 * method.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Number`.
 */
function isNumber(value) {return typeof value === 'number';}


/**
 * @ngdoc function
 * @name angular.isDate
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a value is a date.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Date`.
 */
function isDate(value) {
  return toString.call(value) === '[object Date]';
}


/**
 * @ngdoc function
 * @name angular.isArray
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is an `Array`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is an `Array`.
 */
function isArray(arr) {
  return Array.isArray(arr) || arr instanceof Array;
}

/**
 * @description
 * Determines if a reference is an `Error`.
 * Loosely based on https://www.npmjs.com/package/iserror
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is an `Error`.
 */
function isError(value) {
  var tag = toString.call(value);
  switch (tag) {
    case '[object Error]': return true;
    case '[object Exception]': return true;
    case '[object DOMException]': return true;
    default: return value instanceof Error;
  }
}

/**
 * @ngdoc function
 * @name angular.isFunction
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `Function`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Function`.
 */
function isFunction(value) {return typeof value === 'function';}


/**
 * Determines if a value is a regular expression object.
 *
 * @private
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `RegExp`.
 */
function isRegExp(value) {
  return toString.call(value) === '[object RegExp]';
}


/**
 * Checks if `obj` is a window object.
 *
 * @private
 * @param {*} obj Object to check
 * @returns {boolean} True if `obj` is a window obj.
 */
function isWindow(obj) {
  return obj && obj.window === obj;
}


function isScope(obj) {
  return obj && obj.$evalAsync && obj.$watch;
}


function isFile(obj) {
  return toString.call(obj) === '[object File]';
}


function isFormData(obj) {
  return toString.call(obj) === '[object FormData]';
}


function isBlob(obj) {
  return toString.call(obj) === '[object Blob]';
}


function isBoolean(value) {
  return typeof value === 'boolean';
}


function isPromiseLike(obj) {
  return obj && isFunction(obj.then);
}


var TYPED_ARRAY_REGEXP = /^\[object (?:Uint8|Uint8Clamped|Uint16|Uint32|Int8|Int16|Int32|Float32|Float64)Array]$/;
function isTypedArray(value) {
  return value && isNumber(value.length) && TYPED_ARRAY_REGEXP.test(toString.call(value));
}

function isArrayBuffer(obj) {
  return toString.call(obj) === '[object ArrayBuffer]';
}


var trim = function(value) {
  return isString(value) ? value.trim() : value;
};

// Copied from:
// http://docs.closure-library.googlecode.com/git/local_closure_goog_string_string.js.source.html#line1021
// Prereq: s is a string.
var escapeForRegexp = function(s) {
  return s
    .replace(/([-()[\]{}+?*.$^|,:#<!\\])/g, '\\$1')
    // eslint-disable-next-line no-control-regex
    .replace(/\x08/g, '\\x08');
};


/**
 * @ngdoc function
 * @name angular.isElement
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a DOM element (or wrapped jQuery element).
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a DOM element (or wrapped jQuery element).
 */
function isElement(node) {
  return !!(node &&
    (node.nodeName  // We are a direct element.
    || (node.prop && node.attr && node.find)));  // We have an on and find method part of jQuery API.
}

/**
 * @param str 'key1,key2,...'
 * @returns {object} in the form of {key1:true, key2:true, ...}
 */
function makeMap(str) {
  var obj = {}, items = str.split(','), i;
  for (i = 0; i < items.length; i++) {
    obj[items[i]] = true;
  }
  return obj;
}


function nodeName_(element) {
  return lowercase(element.nodeName || (element[0] && element[0].nodeName));
}

function includes(array, obj) {
  return Array.prototype.indexOf.call(array, obj) !== -1;
}

function arrayRemove(array, value) {
  var index = array.indexOf(value);
  if (index >= 0) {
    array.splice(index, 1);
  }
  return index;
}

/**
 * @ngdoc function
 * @name angular.copy
 * @module ng
 * @kind function
 *
 * @description
 * Creates a deep copy of `source`, which should be an object or an array. This functions is used
 * internally, mostly in the change-detection code. It is not intended as an all-purpose copy
 * function, and has several limitations (see below).
 *
 * * If no destination is supplied, a copy of the object or array is created.
 * * If a destination is provided, all of its elements (for arrays) or properties (for objects)
 *   are deleted and then all elements/properties from the source are copied to it.
 * * If `source` is not an object or array (inc. `null` and `undefined`), `source` is returned.
 * * If `source` is identical to `destination` an exception will be thrown.
 *
 * <br />
 *
 * <div class="alert alert-warning">
 *   Only enumerable properties are taken into account. Non-enumerable properties (both on `source`
 *   and on `destination`) will be ignored.
 * </div>
 *
 * <div class="alert alert-warning">
 *   `angular.copy` does not check if destination and source are of the same type. It's the
 *   developer's responsibility to make sure they are compatible.
 * </div>
 *
 * @knownIssue
 * This is a non-exhaustive list of object types / features that are not handled correctly by
 * `angular.copy`. Note that since this functions is used by the change detection code, this
 * means binding or watching objects of these types (or that include these types) might not work
 * correctly.
 * - [`File`](https://developer.mozilla.org/docs/Web/API/File)
 * - [`Map`](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Map)
 * - [`ImageData`](https://developer.mozilla.org/docs/Web/API/ImageData)
 * - [`MediaStream`](https://developer.mozilla.org/docs/Web/API/MediaStream)
 * - [`Set`](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Set)
 * - [`WeakMap`](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/WeakMap)
 * - [`getter`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/get)/
 *   [`setter`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/set)
 *
 * @param {*} source The source that will be used to make a copy. Can be any type, including
 *     primitives, `null`, and `undefined`.
 * @param {(Object|Array)=} destination Destination into which the source is copied. If provided,
 *     must be of the same type as `source`.
 * @returns {*} The copy or updated `destination`, if `destination` was specified.
 *
 * @example
  <example module="copyExample" name="angular-copy">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <form novalidate class="simple-form">
          <label>Name: <input type="text" ng-model="user.name" /></label><br />
          <label>Age:  <input type="number" ng-model="user.age" /></label><br />
          Gender: <label><input type="radio" ng-model="user.gender" value="male" />male</label>
                  <label><input type="radio" ng-model="user.gender" value="female" />female</label><br />
          <button ng-click="reset()">RESET</button>
          <button ng-click="update(user)">SAVE</button>
        </form>
        <pre>form = {{user | json}}</pre>
        <pre>leader = {{leader | json}}</pre>
      </div>
    </file>
    <file name="script.js">
      // Module: copyExample
      angular.
        module('copyExample', []).
        controller('ExampleController', ['$scope', function($scope) {
          $scope.leader = {};

          $scope.reset = function() {
            // Example with 1 argument
            $scope.user = angular.copy($scope.leader);
          };

          $scope.update = function(user) {
            // Example with 2 arguments
            angular.copy(user, $scope.leader);
          };

          $scope.reset();
        }]);
    </file>
  </example>
 */
function copy(source, destination, maxDepth) {
  var stackSource = [];
  var stackDest = [];
  maxDepth = isValidObjectMaxDepth(maxDepth) ? maxDepth : NaN;

  if (destination) {
    if (isTypedArray(destination) || isArrayBuffer(destination)) {
      throw ngMinErr('cpta', 'Can\'t copy! TypedArray destination cannot be mutated.');
    }
    if (source === destination) {
      throw ngMinErr('cpi', 'Can\'t copy! Source and destination are identical.');
    }

    // Empty the destination object
    if (isArray(destination)) {
      destination.length = 0;
    } else {
      forEach(destination, function(value, key) {
        if (key !== '$$hashKey') {
          delete destination[key];
        }
      });
    }

    stackSource.push(source);
    stackDest.push(destination);
    return copyRecurse(source, destination, maxDepth);
  }

  return copyElement(source, maxDepth);

  function copyRecurse(source, destination, maxDepth) {
    maxDepth--;
    if (maxDepth < 0) {
      return '...';
    }
    var h = destination.$$hashKey;
    var key;
    if (isArray(source)) {
      for (var i = 0, ii = source.length; i < ii; i++) {
        destination.push(copyElement(source[i], maxDepth));
      }
    } else if (isBlankObject(source)) {
      // createMap() fast path --- Safe to avoid hasOwnProperty check because prototype chain is empty
      for (key in source) {
        destination[key] = copyElement(source[key], maxDepth);
      }
    } else if (source && typeof source.hasOwnProperty === 'function') {
      // Slow path, which must rely on hasOwnProperty
      for (key in source) {
        if (source.hasOwnProperty(key)) {
          destination[key] = copyElement(source[key], maxDepth);
        }
      }
    } else {
      // Slowest path --- hasOwnProperty can't be called as a method
      for (key in source) {
        if (hasOwnProperty.call(source, key)) {
          destination[key] = copyElement(source[key], maxDepth);
        }
      }
    }
    setHashKey(destination, h);
    return destination;
  }

  function copyElement(source, maxDepth) {
    // Simple values
    if (!isObject(source)) {
      return source;
    }

    // Already copied values
    var index = stackSource.indexOf(source);
    if (index !== -1) {
      return stackDest[index];
    }

    if (isWindow(source) || isScope(source)) {
      throw ngMinErr('cpws',
        'Can\'t copy! Making copies of Window or Scope instances is not supported.');
    }

    var needsRecurse = false;
    var destination = copyType(source);

    if (destination === undefined) {
      destination = isArray(source) ? [] : Object.create(getPrototypeOf(source));
      needsRecurse = true;
    }

    stackSource.push(source);
    stackDest.push(destination);

    return needsRecurse
      ? copyRecurse(source, destination, maxDepth)
      : destination;
  }

  function copyType(source) {
    switch (toString.call(source)) {
      case '[object Int8Array]':
      case '[object Int16Array]':
      case '[object Int32Array]':
      case '[object Float32Array]':
      case '[object Float64Array]':
      case '[object Uint8Array]':
      case '[object Uint8ClampedArray]':
      case '[object Uint16Array]':
      case '[object Uint32Array]':
        return new source.constructor(copyElement(source.buffer), source.byteOffset, source.length);

      case '[object ArrayBuffer]':
        // Support: IE10
        if (!source.slice) {
          // If we're in this case we know the environment supports ArrayBuffer
          /* eslint-disable no-undef */
          var copied = new ArrayBuffer(source.byteLength);
          new Uint8Array(copied).set(new Uint8Array(source));
          /* eslint-enable */
          return copied;
        }
        return source.slice(0);

      case '[object Boolean]':
      case '[object Number]':
      case '[object String]':
      case '[object Date]':
        return new source.constructor(source.valueOf());

      case '[object RegExp]':
        var re = new RegExp(source.source, source.toString().match(/[^/]*$/)[0]);
        re.lastIndex = source.lastIndex;
        return re;

      case '[object Blob]':
        return new source.constructor([source], {type: source.type});
    }

    if (isFunction(source.cloneNode)) {
      return source.cloneNode(true);
    }
  }
}


// eslint-disable-next-line no-self-compare
function simpleCompare(a, b) { return a === b || (a !== a && b !== b); }


/**
 * @ngdoc function
 * @name angular.equals
 * @module ng
 * @kind function
 *
 * @description
 * Determines if two objects or two values are equivalent. Supports value types, regular
 * expressions, arrays and objects.
 *
 * Two objects or values are considered equivalent if at least one of the following is true:
 *
 * * Both objects or values pass `===` comparison.
 * * Both objects or values are of the same type and all of their properties are equal by
 *   comparing them with `angular.equals`.
 * * Both values are NaN. (In JavaScript, NaN == NaN => false. But we consider two NaN as equal)
 * * Both values represent the same regular expression (In JavaScript,
 *   /abc/ == /abc/ => false. But we consider two regular expressions as equal when their textual
 *   representation matches).
 *
 * During a property comparison, properties of `function` type and properties with names
 * that begin with `$` are ignored.
 *
 * Scope and DOMWindow objects are being compared only by identify (`===`).
 *
 * @param {*} o1 Object or value to compare.
 * @param {*} o2 Object or value to compare.
 * @returns {boolean} True if arguments are equal.
 *
 * @example
   <example module="equalsExample" name="equalsExample">
     <file name="index.html">
      <div ng-controller="ExampleController">
        <form novalidate>
          <h3>User 1</h3>
          Name: <input type="text" ng-model="user1.name">
          Age: <input type="number" ng-model="user1.age">

          <h3>User 2</h3>
          Name: <input type="text" ng-model="user2.name">
          Age: <input type="number" ng-model="user2.age">

          <div>
            <br/>
            <input type="button" value="Compare" ng-click="compare()">
          </div>
          User 1: <pre>{{user1 | json}}</pre>
          User 2: <pre>{{user2 | json}}</pre>
          Equal: <pre>{{result}}</pre>
        </form>
      </div>
    </file>
    <file name="script.js">
        angular.module('equalsExample', []).controller('ExampleController', ['$scope', function($scope) {
          $scope.user1 = {};
          $scope.user2 = {};
          $scope.compare = function() {
            $scope.result = angular.equals($scope.user1, $scope.user2);
          };
        }]);
    </file>
  </example>
 */
function equals(o1, o2) {
  if (o1 === o2) return true;
  if (o1 === null || o2 === null) return false;
  // eslint-disable-next-line no-self-compare
  if (o1 !== o1 && o2 !== o2) return true; // NaN === NaN
  var t1 = typeof o1, t2 = typeof o2, length, key, keySet;
  if (t1 === t2 && t1 === 'object') {
    if (isArray(o1)) {
      if (!isArray(o2)) return false;
      if ((length = o1.length) === o2.length) {
        for (key = 0; key < length; key++) {
          if (!equals(o1[key], o2[key])) return false;
        }
        return true;
      }
    } else if (isDate(o1)) {
      if (!isDate(o2)) return false;
      return simpleCompare(o1.getTime(), o2.getTime());
    } else if (isRegExp(o1)) {
      if (!isRegExp(o2)) return false;
      return o1.toString() === o2.toString();
    } else {
      if (isScope(o1) || isScope(o2) || isWindow(o1) || isWindow(o2) ||
        isArray(o2) || isDate(o2) || isRegExp(o2)) return false;
      keySet = createMap();
      for (key in o1) {
        if (key.charAt(0) === '$' || isFunction(o1[key])) continue;
        if (!equals(o1[key], o2[key])) return false;
        keySet[key] = true;
      }
      for (key in o2) {
        if (!(key in keySet) &&
            key.charAt(0) !== '$' &&
            isDefined(o2[key]) &&
            !isFunction(o2[key])) return false;
      }
      return true;
    }
  }
  return false;
}

var csp = function() {
  if (!isDefined(csp.rules)) {


    var ngCspElement = (window.document.querySelector('[ng-csp]') ||
                    window.document.querySelector('[data-ng-csp]'));

    if (ngCspElement) {
      var ngCspAttribute = ngCspElement.getAttribute('ng-csp') ||
                    ngCspElement.getAttribute('data-ng-csp');
      csp.rules = {
        noUnsafeEval: !ngCspAttribute || (ngCspAttribute.indexOf('no-unsafe-eval') !== -1),
        noInlineStyle: !ngCspAttribute || (ngCspAttribute.indexOf('no-inline-style') !== -1)
      };
    } else {
      csp.rules = {
        noUnsafeEval: noUnsafeEval(),
        noInlineStyle: false
      };
    }
  }

  return csp.rules;

  function noUnsafeEval() {
    try {
      // eslint-disable-next-line no-new, no-new-func
      new Function('');
      return false;
    } catch (e) {
      return true;
    }
  }
};

/**
 * @ngdoc directive
 * @module ng
 * @name ngJq
 *
 * @element ANY
 * @param {string=} ngJq the name of the library available under `window`
 * to be used for angular.element
 * @description
 * Use this directive to force the angular.element library.  This should be
 * used to force either jqLite by leaving ng-jq blank or setting the name of
 * the jquery variable under window (eg. jQuery).
 *
 * Since AngularJS looks for this directive when it is loaded (doesn't wait for the
 * DOMContentLoaded event), it must be placed on an element that comes before the script
 * which loads angular. Also, only the first instance of `ng-jq` will be used and all
 * others ignored.
 *
 * @example
 * This example shows how to force jqLite using the `ngJq` directive to the `html` tag.
 ```html
 <!doctype html>
 <html ng-app ng-jq>
 ...
 ...
 </html>
 ```
 * @example
 * This example shows how to use a jQuery based library of a different name.
 * The library name must be available at the top most 'window'.
 ```html
 <!doctype html>
 <html ng-app ng-jq="jQueryLib">
 ...
 ...
 </html>
 ```
 */
var jq = function() {
  if (isDefined(jq.name_)) return jq.name_;
  var el;
  var i, ii = ngAttrPrefixes.length, prefix, name;
  for (i = 0; i < ii; ++i) {
    prefix = ngAttrPrefixes[i];
    el = window.document.querySelector('[' + prefix.replace(':', '\\:') + 'jq]');
    if (el) {
      name = el.getAttribute(prefix + 'jq');
      break;
    }
  }

  return (jq.name_ = name);
};

function concat(array1, array2, index) {
  return array1.concat(slice.call(array2, index));
}

function sliceArgs(args, startIndex) {
  return slice.call(args, startIndex || 0);
}


/**
 * @ngdoc function
 * @name angular.bind
 * @module ng
 * @kind function
 *
 * @description
 * Returns a function which calls function `fn` bound to `self` (`self` becomes the `this` for
 * `fn`). You can supply optional `args` that are prebound to the function. This feature is also
 * known as [partial application](http://en.wikipedia.org/wiki/Partial_application), as
 * distinguished from [function currying](http://en.wikipedia.org/wiki/Currying#Contrast_with_partial_function_application).
 *
 * @param {Object} self Context which `fn` should be evaluated in.
 * @param {function()} fn Function to be bound.
 * @param {...*} args Optional arguments to be prebound to the `fn` function call.
 * @returns {function()} Function that wraps the `fn` with all the specified bindings.
 */
function bind(self, fn) {
  var curryArgs = arguments.length > 2 ? sliceArgs(arguments, 2) : [];
  if (isFunction(fn) && !(fn instanceof RegExp)) {
    return curryArgs.length
      ? function() {
          return arguments.length
            ? fn.apply(self, concat(curryArgs, arguments, 0))
            : fn.apply(self, curryArgs);
        }
      : function() {
          return arguments.length
            ? fn.apply(self, arguments)
            : fn.call(self);
        };
  } else {
    // In IE, native methods are not functions so they cannot be bound (note: they don't need to be).
    return fn;
  }
}


function toJsonReplacer(key, value) {
  var val = value;

  if (typeof key === 'string' && key.charAt(0) === '$' && key.charAt(1) === '$') {
    val = undefined;
  } else if (isWindow(value)) {
    val = '$WINDOW';
  } else if (value &&  window.document === value) {
    val = '$DOCUMENT';
  } else if (isScope(value)) {
    val = '$SCOPE';
  }

  return val;
}


/**
 * @ngdoc function
 * @name angular.toJson
 * @module ng
 * @kind function
 *
 * @description
 * Serializes input into a JSON-formatted string. Properties with leading $$ characters will be
 * stripped since AngularJS uses this notation internally.
 *
 * @param {Object|Array|Date|string|number|boolean} obj Input to be serialized into JSON.
 * @param {boolean|number} [pretty=2] If set to true, the JSON output will contain newlines and whitespace.
 *    If set to an integer, the JSON output will contain that many spaces per indentation.
 * @returns {string|undefined} JSON-ified string representing `obj`.
 * @knownIssue
 *
 * The Safari browser throws a `RangeError` instead of returning `null` when it tries to stringify a `Date`
 * object with an invalid date value. The only reliable way to prevent this is to monkeypatch the
 * `Date.prototype.toJSON` method as follows:
 *
 * ```
 * var _DatetoJSON = Date.prototype.toJSON;
 * Date.prototype.toJSON = function() {
 *   try {
 *     return _DatetoJSON.call(this);
 *   } catch(e) {
 *     if (e instanceof RangeError) {
 *       return null;
 *     }
 *     throw e;
 *   }
 * };
 * ```
 *
 * See https://github.com/angular/angular.js/pull/14221 for more information.
 */
function toJson(obj, pretty) {
  if (isUndefined(obj)) return undefined;
  if (!isNumber(pretty)) {
    pretty = pretty ? 2 : null;
  }
  return JSON.stringify(obj, toJsonReplacer, pretty);
}


/**
 * @ngdoc function
 * @name angular.fromJson
 * @module ng
 * @kind function
 *
 * @description
 * Deserializes a JSON string.
 *
 * @param {string} json JSON string to deserialize.
 * @returns {Object|Array|string|number} Deserialized JSON string.
 */
function fromJson(json) {
  return isString(json)
      ? JSON.parse(json)
      : json;
}


var ALL_COLONS = /:/g;
function timezoneToOffset(timezone, fallback) {
  // Support: IE 9-11 only, Edge 13-15+
  // IE/Edge do not "understand" colon (`:`) in timezone
  timezone = timezone.replace(ALL_COLONS, '');
  var requestedTimezoneOffset = Date.parse('Jan 01, 1970 00:00:00 ' + timezone) / 60000;
  return isNumberNaN(requestedTimezoneOffset) ? fallback : requestedTimezoneOffset;
}


function addDateMinutes(date, minutes) {
  date = new Date(date.getTime());
  date.setMinutes(date.getMinutes() + minutes);
  return date;
}


function convertTimezoneToLocal(date, timezone, reverse) {
  reverse = reverse ? -1 : 1;
  var dateTimezoneOffset = date.getTimezoneOffset();
  var timezoneOffset = timezoneToOffset(timezone, dateTimezoneOffset);
  return addDateMinutes(date, reverse * (timezoneOffset - dateTimezoneOffset));
}


/**
 * @returns {string} Returns the string representation of the element.
 */
function startingTag(element) {
  element = jqLite(element).clone().empty();
  var elemHtml = jqLite('<div></div>').append(element).html();
  try {
    return element[0].nodeType === NODE_TYPE_TEXT ? lowercase(elemHtml) :
        elemHtml.
          match(/^(<[^>]+>)/)[1].
          replace(/^<([\w-]+)/, function(match, nodeName) {return '<' + lowercase(nodeName);});
  } catch (e) {
    return lowercase(elemHtml);
  }

}


/////////////////////////////////////////////////

/**
 * Tries to decode the URI component without throwing an exception.
 *
 * @private
 * @param str value potential URI component to check.
 * @returns {boolean} True if `value` can be decoded
 * with the decodeURIComponent function.
 */
function tryDecodeURIComponent(value) {
  try {
    return decodeURIComponent(value);
  } catch (e) {
    // Ignore any invalid uri component.
  }
}


/**
 * Parses an escaped url query string into key-value pairs.
 * @returns {Object.<string,boolean|Array>}
 */
function parseKeyValue(/**string*/keyValue) {
  var obj = {};
  forEach((keyValue || '').split('&'), function(keyValue) {
    var splitPoint, key, val;
    if (keyValue) {
      key = keyValue = keyValue.replace(/\+/g,'%20');
      splitPoint = keyValue.indexOf('=');
      if (splitPoint !== -1) {
        key = keyValue.substring(0, splitPoint);
        val = keyValue.substring(splitPoint + 1);
      }
      key = tryDecodeURIComponent(key);
      if (isDefined(key)) {
        val = isDefined(val) ? tryDecodeURIComponent(val) : true;
        if (!hasOwnProperty.call(obj, key)) {
          obj[key] = val;
        } else if (isArray(obj[key])) {
          obj[key].push(val);
        } else {
          obj[key] = [obj[key],val];
        }
      }
    }
  });
  return obj;
}

function toKeyValue(obj) {
  var parts = [];
  forEach(obj, function(value, key) {
    if (isArray(value)) {
      forEach(value, function(arrayValue) {
        parts.push(encodeUriQuery(key, true) +
                   (arrayValue === true ? '' : '=' + encodeUriQuery(arrayValue, true)));
      });
    } else {
    parts.push(encodeUriQuery(key, true) +
               (value === true ? '' : '=' + encodeUriQuery(value, true)));
    }
  });
  return parts.length ? parts.join('&') : '';
}


/**
 * We need our custom method because encodeURIComponent is too aggressive and doesn't follow
 * http://www.ietf.org/rfc/rfc3986.txt with regards to the character set (pchar) allowed in path
 * segments:
 *    segment       = *pchar
 *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
 *    pct-encoded   = "%" HEXDIG HEXDIG
 *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
 *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
 *                     / "*" / "+" / "," / ";" / "="
 */
function encodeUriSegment(val) {
  return encodeUriQuery(val, true).
             replace(/%26/gi, '&').
             replace(/%3D/gi, '=').
             replace(/%2B/gi, '+');
}


/**
 * This method is intended for encoding *key* or *value* parts of query component. We need a custom
 * method because encodeURIComponent is too aggressive and encodes stuff that doesn't have to be
 * encoded per http://tools.ietf.org/html/rfc3986:
 *    query         = *( pchar / "/" / "?" )
 *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
 *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
 *    pct-encoded   = "%" HEXDIG HEXDIG
 *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
 *                     / "*" / "+" / "," / ";" / "="
 */
function encodeUriQuery(val, pctEncodeSpaces) {
  return encodeURIComponent(val).
             replace(/%40/gi, '@').
             replace(/%3A/gi, ':').
             replace(/%24/g, '$').
             replace(/%2C/gi, ',').
             replace(/%3B/gi, ';').
             replace(/%20/g, (pctEncodeSpaces ? '%20' : '+'));
}

var ngAttrPrefixes = ['ng-', 'data-ng-', 'ng:', 'x-ng-'];

function getNgAttribute(element, ngAttr) {
  var attr, i, ii = ngAttrPrefixes.length;
  for (i = 0; i < ii; ++i) {
    attr = ngAttrPrefixes[i] + ngAttr;
    if (isString(attr = element.getAttribute(attr))) {
      return attr;
    }
  }
  return null;
}

function allowAutoBootstrap(document) {
  var script = document.currentScript;

  if (!script) {
    // Support: IE 9-11 only
    // IE does not have `document.currentScript`
    return true;
  }

  // If the `currentScript` property has been clobbered just return false, since this indicates a probable attack
  if (!(script instanceof window.HTMLScriptElement || script instanceof window.SVGScriptElement)) {
    return false;
  }

  var attributes = script.attributes;
  var srcs = [attributes.getNamedItem('src'), attributes.getNamedItem('href'), attributes.getNamedItem('xlink:href')];

  return srcs.every(function(src) {
    if (!src) {
      return true;
    }
    if (!src.value) {
      return false;
    }

    var link = document.createElement('a');
    link.href = src.value;

    if (document.location.origin === link.origin) {
      // Same-origin resources are always allowed, even for non-whitelisted schemes.
      return true;
    }
    // Disabled bootstrapping unless angular.js was loaded from a known scheme used on the web.
    // This is to prevent angular.js bundled with browser extensions from being used to bypass the
    // content security policy in web pages and other browser extensions.
    switch (link.protocol) {
      case 'http:':
      case 'https:':
      case 'ftp:':
      case 'blob:':
      case 'file:':
      case 'data:':
        return true;
      default:
        return false;
    }
  });
}

// Cached as it has to run during loading so that document.currentScript is available.
var isAutoBootstrapAllowed = allowAutoBootstrap(window.document);

/**
 * @ngdoc directive
 * @name ngApp
 * @module ng
 *
 * @element ANY
 * @param {angular.Module} ngApp an optional application
 *   {@link angular.module module} name to load.
 * @param {boolean=} ngStrictDi if this attribute is present on the app element, the injector will be
 *   created in "strict-di" mode. This means that the application will fail to invoke functions which
 *   do not use explicit function annotation (and are thus unsuitable for minification), as described
 *   in {@link guide/di the Dependency Injection guide}, and useful debugging info will assist in
 *   tracking down the root of these bugs.
 *
 * @description
 *
 * Use this directive to **auto-bootstrap** an AngularJS application. The `ngApp` directive
 * designates the **root element** of the application and is typically placed near the root element
 * of the page - e.g. on the `<body>` or `<html>` tags.
 *
 * There are a few things to keep in mind when using `ngApp`:
 * - only one AngularJS application can be auto-bootstrapped per HTML document. The first `ngApp`
 *   found in the document will be used to define the root element to auto-bootstrap as an
 *   application. To run multiple applications in an HTML document you must manually bootstrap them using
 *   {@link angular.bootstrap} instead.
 * - AngularJS applications cannot be nested within each other.
 * - Do not use a directive that uses {@link ng.$compile#transclusion transclusion} on the same element as `ngApp`.
 *   This includes directives such as {@link ng.ngIf `ngIf`}, {@link ng.ngInclude `ngInclude`} and
 *   {@link ngRoute.ngView `ngView`}.
 *   Doing this misplaces the app {@link ng.$rootElement `$rootElement`} and the app's {@link auto.$injector injector},
 *   causing animations to stop working and making the injector inaccessible from outside the app.
 *
 * You can specify an **AngularJS module** to be used as the root module for the application.  This
 * module will be loaded into the {@link auto.$injector} when the application is bootstrapped. It
 * should contain the application code needed or have dependencies on other modules that will
 * contain the code. See {@link angular.module} for more information.
 *
 * In the example below if the `ngApp` directive were not placed on the `html` element then the
 * document would not be compiled, the `AppController` would not be instantiated and the `{{ a+b }}`
 * would not be resolved to `3`.
 *
 * @example
 *
 * ### Simple Usage
 *
 * `ngApp` is the easiest, and most common way to bootstrap an application.
 *
 <example module="ngAppDemo" name="ng-app">
   <file name="index.html">
   <div ng-controller="ngAppDemoController">
     I can add: {{a}} + {{b}} =  {{ a+b }}
   </div>
   </file>
   <file name="script.js">
   angular.module('ngAppDemo', []).controller('ngAppDemoController', function($scope) {
     $scope.a = 1;
     $scope.b = 2;
   });
   </file>
 </example>
 *
 * @example
 *
 * ### With `ngStrictDi`
 *
 * Using `ngStrictDi`, you would see something like this:
 *
 <example ng-app-included="true" name="strict-di">
   <file name="index.html">
   <div ng-app="ngAppStrictDemo" ng-strict-di>
       <div ng-controller="GoodController1">
           I can add: {{a}} + {{b}} =  {{ a+b }}

           <p>This renders because the controller does not fail to
              instantiate, by using explicit annotation style (see
              script.js for details)
           </p>
       </div>

       <div ng-controller="GoodController2">
           Name: <input ng-model="name"><br />
           Hello, {{name}}!

           <p>This renders because the controller does not fail to
              instantiate, by using explicit annotation style
              (see script.js for details)
           </p>
       </div>

       <div ng-controller="BadController">
           I can add: {{a}} + {{b}} =  {{ a+b }}

           <p>The controller could not be instantiated, due to relying
              on automatic function annotations (which are disabled in
              strict mode). As such, the content of this section is not
              interpolated, and there should be an error in your web console.
           </p>
       </div>
   </div>
   </file>
   <file name="script.js">
   angular.module('ngAppStrictDemo', [])
     // BadController will fail to instantiate, due to relying on automatic function annotation,
     // rather than an explicit annotation
     .controller('BadController', function($scope) {
       $scope.a = 1;
       $scope.b = 2;
     })
     // Unlike BadController, GoodController1 and GoodController2 will not fail to be instantiated,
     // due to using explicit annotations using the array style and $inject property, respectively.
     .controller('GoodController1', ['$scope', function($scope) {
       $scope.a = 1;
       $scope.b = 2;
     }])
     .controller('GoodController2', GoodController2);
     function GoodController2($scope) {
       $scope.name = 'World';
     }
     GoodController2.$inject = ['$scope'];
   </file>
   <file name="style.css">
   div[ng-controller] {
       margin-bottom: 1em;
       -webkit-border-radius: 4px;
       border-radius: 4px;
       border: 1px solid;
       padding: .5em;
   }
   div[ng-controller^=Good] {
       border-color: #d6e9c6;
       background-color: #dff0d8;
       color: #3c763d;
   }
   div[ng-controller^=Bad] {
       border-color: #ebccd1;
       background-color: #f2dede;
       color: #a94442;
       margin-bottom: 0;
   }
   </file>
 </example>
 */
function angularInit(element, bootstrap) {
  var appElement,
      module,
      config = {};

  // The element `element` has priority over any other element.
  forEach(ngAttrPrefixes, function(prefix) {
    var name = prefix + 'app';

    if (!appElement && element.hasAttribute && element.hasAttribute(name)) {
      appElement = element;
      module = element.getAttribute(name);
    }
  });
  forEach(ngAttrPrefixes, function(prefix) {
    var name = prefix + 'app';
    var candidate;

    if (!appElement && (candidate = element.querySelector('[' + name.replace(':', '\\:') + ']'))) {
      appElement = candidate;
      module = candidate.getAttribute(name);
    }
  });
  if (appElement) {
    if (!isAutoBootstrapAllowed) {
      window.console.error('AngularJS: disabling automatic bootstrap. <script> protocol indicates ' +
          'an extension, document.location.href does not match.');
      return;
    }
    config.strictDi = getNgAttribute(appElement, 'strict-di') !== null;
    bootstrap(appElement, module ? [module] : [], config);
  }
}

/**
 * @ngdoc function
 * @name angular.bootstrap
 * @module ng
 * @description
 * Use this function to manually start up AngularJS application.
 *
 * For more information, see the {@link guide/bootstrap Bootstrap guide}.
 *
 * AngularJS will detect if it has been loaded into the browser more than once and only allow the
 * first loaded script to be bootstrapped and will report a warning to the browser console for
 * each of the subsequent scripts. This prevents strange results in applications, where otherwise
 * multiple instances of AngularJS try to work on the DOM.
 *
 * <div class="alert alert-warning">
 * **Note:** Protractor based end-to-end tests cannot use this function to bootstrap manually.
 * They must use {@link ng.directive:ngApp ngApp}.
 * </div>
 *
 * <div class="alert alert-warning">
 * **Note:** Do not bootstrap the app on an element with a directive that uses {@link ng.$compile#transclusion transclusion},
 * such as {@link ng.ngIf `ngIf`}, {@link ng.ngInclude `ngInclude`} and {@link ngRoute.ngView `ngView`}.
 * Doing this misplaces the app {@link ng.$rootElement `$rootElement`} and the app's {@link auto.$injector injector},
 * causing animations to stop working and making the injector inaccessible from outside the app.
 * </div>
 *
 * ```html
 * <!doctype html>
 * <html>
 * <body>
 * <div ng-controller="WelcomeController">
 *   {{greeting}}
 * </div>
 *
 * <script src="angular.js"></script>
 * <script>
 *   var app = angular.module('demo', [])
 *   .controller('WelcomeController', function($scope) {
 *       $scope.greeting = 'Welcome!';
 *   });
 *   angular.bootstrap(document, ['demo']);
 * </script>
 * </body>
 * </html>
 * ```
 *
 * @param {DOMElement} element DOM element which is the root of AngularJS application.
 * @param {Array<String|Function|Array>=} modules an array of modules to load into the application.
 *     Each item in the array should be the name of a predefined module or a (DI annotated)
 *     function that will be invoked by the injector as a `config` block.
 *     See: {@link angular.module modules}
 * @param {Object=} config an object for defining configuration options for the application. The
 *     following keys are supported:
 *
 * * `strictDi` - disable automatic function annotation for the application. This is meant to
 *   assist in finding bugs which break minified code. Defaults to `false`.
 *
 * @returns {auto.$injector} Returns the newly created injector for this app.
 */
function bootstrap(element, modules, config) {
  if (!isObject(config)) config = {};
  var defaultConfig = {
    strictDi: false
  };
  config = extend(defaultConfig, config);
  var doBootstrap = function() {
    element = jqLite(element);

    if (element.injector()) {
      var tag = (element[0] === window.document) ? 'document' : startingTag(element);
      // Encode angle brackets to prevent input from being sanitized to empty string #8683.
      throw ngMinErr(
          'btstrpd',
          'App already bootstrapped with this element \'{0}\'',
          tag.replace(/</,'&lt;').replace(/>/,'&gt;'));
    }

    modules = modules || [];
    modules.unshift(['$provide', function($provide) {
      $provide.value('$rootElement', element);
    }]);

    if (config.debugInfoEnabled) {
      // Pushing so that this overrides `debugInfoEnabled` setting defined in user's `modules`.
      modules.push(['$compileProvider', function($compileProvider) {
        $compileProvider.debugInfoEnabled(true);
      }]);
    }

    modules.unshift('ng');
    var injector = createInjector(modules, config.strictDi);
    injector.invoke(['$rootScope', '$rootElement', '$compile', '$injector',
       function bootstrapApply(scope, element, compile, injector) {
        scope.$apply(function() {
          element.data('$injector', injector);
          compile(element)(scope);
        });
      }]
    );
    return injector;
  };

  var NG_ENABLE_DEBUG_INFO = /^NG_ENABLE_DEBUG_INFO!/;
  var NG_DEFER_BOOTSTRAP = /^NG_DEFER_BOOTSTRAP!/;

  if (window && NG_ENABLE_DEBUG_INFO.test(window.name)) {
    config.debugInfoEnabled = true;
    window.name = window.name.replace(NG_ENABLE_DEBUG_INFO, '');
  }

  if (window && !NG_DEFER_BOOTSTRAP.test(window.name)) {
    return doBootstrap();
  }

  window.name = window.name.replace(NG_DEFER_BOOTSTRAP, '');
  angular.resumeBootstrap = function(extraModules) {
    forEach(extraModules, function(module) {
      modules.push(module);
    });
    return doBootstrap();
  };

  if (isFunction(angular.resumeDeferredBootstrap)) {
    angular.resumeDeferredBootstrap();
  }
}

/**
 * @ngdoc function
 * @name angular.reloadWithDebugInfo
 * @module ng
 * @description
 * Use this function to reload the current application with debug information turned on.
 * This takes precedence over a call to `$compileProvider.debugInfoEnabled(false)`.
 *
 * See {@link ng.$compileProvider#debugInfoEnabled} for more.
 */
function reloadWithDebugInfo() {
  window.name = 'NG_ENABLE_DEBUG_INFO!' + window.name;
  window.location.reload();
}

/**
 * @name angular.getTestability
 * @module ng
 * @description
 * Get the testability service for the instance of AngularJS on the given
 * element.
 * @param {DOMElement} element DOM element which is the root of AngularJS application.
 */
function getTestability(rootElement) {
  var injector = angular.element(rootElement).injector();
  if (!injector) {
    throw ngMinErr('test',
      'no injector found for element argument to getTestability');
  }
  return injector.get('$$testability');
}

var SNAKE_CASE_REGEXP = /[A-Z]/g;
function snake_case(name, separator) {
  separator = separator || '_';
  return name.replace(SNAKE_CASE_REGEXP, function(letter, pos) {
    return (pos ? separator : '') + letter.toLowerCase();
  });
}

var bindJQueryFired = false;
function bindJQuery() {
  var originalCleanData;

  if (bindJQueryFired) {
    return;
  }

  // bind to jQuery if present;
  var jqName = jq();
  jQuery = isUndefined(jqName) ? window.jQuery :   // use jQuery (if present)
           !jqName             ? undefined     :   // use jqLite
                                 window[jqName];   // use jQuery specified by `ngJq`

  // Use jQuery if it exists with proper functionality, otherwise default to us.
  // AngularJS 1.2+ requires jQuery 1.7+ for on()/off() support.
  // AngularJS 1.3+ technically requires at least jQuery 2.1+ but it may work with older
  // versions. It will not work for sure with jQuery <1.7, though.
  if (jQuery && jQuery.fn.on) {
    jqLite = jQuery;
    extend(jQuery.fn, {
      scope: JQLitePrototype.scope,
      isolateScope: JQLitePrototype.isolateScope,
      controller: /** @type {?} */ (JQLitePrototype).controller,
      injector: JQLitePrototype.injector,
      inheritedData: JQLitePrototype.inheritedData
    });
  } else {
    jqLite = JQLite;
  }

  // All nodes removed from the DOM via various jqLite/jQuery APIs like .remove()
  // are passed through jqLite/jQuery.cleanData. Monkey-patch this method to fire
  // the $destroy event on all removed nodes.
  originalCleanData = jqLite.cleanData;
  jqLite.cleanData = function(elems) {
    var events;
    for (var i = 0, elem; (elem = elems[i]) != null; i++) {
      events = (jqLite._data(elem) || {}).events;
      if (events && events.$destroy) {
        jqLite(elem).triggerHandler('$destroy');
      }
    }
    originalCleanData(elems);
  };

  angular.element = jqLite;

  // Prevent double-proxying.
  bindJQueryFired = true;
}

/**
 * @ngdoc function
 * @name angular.UNSAFE_restoreLegacyJqLiteXHTMLReplacement
 * @module ng
 * @kind function
 *
 * @description
 * Restores the pre-1.8 behavior of jqLite that turns XHTML-like strings like
 * `<div /><span />` to `<div></div><span></span>` instead of `<div><span></span></div>`.
 * The new behavior is a security fix. Thus, if you need to call this function, please try to adjust
 * your code for this change and remove your use of this function as soon as possible.

 * Note that this only patches jqLite. If you use jQuery 3.5.0 or newer, please read the
 * [jQuery 3.5 upgrade guide](https://jquery.com/upgrade-guide/3.5/) for more details
 * about the workarounds.
 */
function UNSAFE_restoreLegacyJqLiteXHTMLReplacement() {
  JQLite.legacyXHTMLReplacement = true;
}

/**
 * throw error if the argument is falsy.
 */
function assertArg(arg, name, reason) {
  if (!arg) {
    throw ngMinErr('areq', 'Argument \'{0}\' is {1}', (name || '?'), (reason || 'required'));
  }
  return arg;
}

function assertArgFn(arg, name, acceptArrayAnnotation) {
  if (acceptArrayAnnotation && isArray(arg)) {
      arg = arg[arg.length - 1];
  }

  assertArg(isFunction(arg), name, 'not a function, got ' +
      (arg && typeof arg === 'object' ? arg.constructor.name || 'Object' : typeof arg));
  return arg;
}

/**
 * throw error if the name given is hasOwnProperty
 * @param  {String} name    the name to test
 * @param  {String} context the context in which the name is used, such as module or directive
 */
function assertNotHasOwnProperty(name, context) {
  if (name === 'hasOwnProperty') {
    throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
  }
}

/**
 * Return the value accessible from the object by path. Any undefined traversals are ignored
 * @param {Object} obj starting object
 * @param {String} path path to traverse
 * @param {boolean} [bindFnToScope=true]
 * @returns {Object} value as accessible by path
 */
//TODO(misko): this function needs to be removed
function getter(obj, path, bindFnToScope) {
  if (!path) return obj;
  var keys = path.split('.');
  var key;
  var lastInstance = obj;
  var len = keys.length;

  for (var i = 0; i < len; i++) {
    key = keys[i];
    if (obj) {
      obj = (lastInstance = obj)[key];
    }
  }
  if (!bindFnToScope && isFunction(obj)) {
    return bind(lastInstance, obj);
  }
  return obj;
}

/**
 * Return the DOM siblings between the first and last node in the given array.
 * @param {Array} array like object
 * @returns {Array} the inputted object or a jqLite collection containing the nodes
 */
function getBlockNodes(nodes) {
  // TODO(perf): update `nodes` instead of creating a new object?
  var node = nodes[0];
  var endNode = nodes[nodes.length - 1];
  var blockNodes;

  for (var i = 1; node !== endNode && (node = node.nextSibling); i++) {
    if (blockNodes || nodes[i] !== node) {
      if (!blockNodes) {
        blockNodes = jqLite(slice.call(nodes, 0, i));
      }
      blockNodes.push(node);
    }
  }

  return blockNodes || nodes;
}


/**
 * Creates a new object without a prototype. This object is useful for lookup without having to
 * guard against prototypically inherited properties via hasOwnProperty.
 *
 * Related micro-benchmarks:
 * - http://jsperf.com/object-create2
 * - http://jsperf.com/proto-map-lookup/2
 * - http://jsperf.com/for-in-vs-object-keys2
 *
 * @returns {Object}
 */
function createMap() {
  return Object.create(null);
}

function stringify(value) {
  if (value == null) { // null || undefined
    return '';
  }
  switch (typeof value) {
    case 'string':
      break;
    case 'number':
      value = '' + value;
      break;
    default:
      if (hasCustomToString(value) && !isArray(value) && !isDate(value)) {
        value = value.toString();
      } else {
        value = toJson(value);
      }
  }

  return value;
}

var NODE_TYPE_ELEMENT = 1;
var NODE_TYPE_ATTRIBUTE = 2;
var NODE_TYPE_TEXT = 3;
var NODE_TYPE_COMMENT = 8;
var NODE_TYPE_DOCUMENT = 9;
var NODE_TYPE_DOCUMENT_FRAGMENT = 11;

/**
 * @ngdoc type
 * @name angular.Module
 * @module ng
 * @description
 *
 * Interface for configuring AngularJS {@link angular.module modules}.
 */

function setupModuleLoader(window) {

  var $injectorMinErr = minErr('$injector');
  var ngMinErr = minErr('ng');

  function ensure(obj, name, factory) {
    return obj[name] || (obj[name] = factory());
  }

  var angular = ensure(window, 'angular', Object);

  // We need to expose `angular.$$minErr` to modules such as `ngResource` that reference it during bootstrap
  angular.$$minErr = angular.$$minErr || minErr;

  return ensure(angular, 'module', function() {
    /** @type {Object.<string, angular.Module>} */
    var modules = {};

    /**
     * @ngdoc function
     * @name angular.module
     * @module ng
     * @description
     *
     * The `angular.module` is a global place for creating, registering and retrieving AngularJS
     * modules.
     * All modules (AngularJS core or 3rd party) that should be available to an application must be
     * registered using this mechanism.
     *
     * Passing one argument retrieves an existing {@link angular.Module},
     * whereas passing more than one argument creates a new {@link angular.Module}
     *
     *
     * # Module
     *
     * A module is a collection of services, directives, controllers, filters, and configuration information.
     * `angular.module` is used to configure the {@link auto.$injector $injector}.
     *
     * ```js
     * // Create a new module
     * var myModule = angular.module('myModule', []);
     *
     * // register a new service
     * myModule.value('appName', 'MyCoolApp');
     *
     * // configure existing services inside initialization blocks.
     * myModule.config(['$locationProvider', function($locationProvider) {
     *   // Configure existing providers
     *   $locationProvider.hashPrefix('!');
     * }]);
     * ```
     *
     * Then you can create an injector and load your modules like this:
     *
     * ```js
     * var injector = angular.injector(['ng', 'myModule'])
     * ```
     *
     * However it's more likely that you'll just use
     * {@link ng.directive:ngApp ngApp} or
     * {@link angular.bootstrap} to simplify this process for you.
     *
     * @param {!string} name The name of the module to create or retrieve.
     * @param {!Array.<string>=} requires If specified then new module is being created. If
     *        unspecified then the module is being retrieved for further configuration.
     * @param {Function=} configFn Optional configuration function for the module. Same as
     *        {@link angular.Module#config Module#config()}.
     * @returns {angular.Module} new module with the {@link angular.Module} api.
     */
    return function module(name, requires, configFn) {

      var info = {};

      var assertNotHasOwnProperty = function(name, context) {
        if (name === 'hasOwnProperty') {
          throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
        }
      };

      assertNotHasOwnProperty(name, 'module');
      if (requires && modules.hasOwnProperty(name)) {
        modules[name] = null;
      }
      return ensure(modules, name, function() {
        if (!requires) {
          throw $injectorMinErr('nomod', 'Module \'{0}\' is not available! You either misspelled ' +
             'the module name or forgot to load it. If registering a module ensure that you ' +
             'specify the dependencies as the second argument.', name);
        }

        /** @type {!Array.<Array.<*>>} */
        var invokeQueue = [];

        /** @type {!Array.<Function>} */
        var configBlocks = [];

        /** @type {!Array.<Function>} */
        var runBlocks = [];

        var config = invokeLater('$injector', 'invoke', 'push', configBlocks);

        /** @type {angular.Module} */
        var moduleInstance = {
          // Private state
          _invokeQueue: invokeQueue,
          _configBlocks: configBlocks,
          _runBlocks: runBlocks,

          /**
           * @ngdoc method
           * @name angular.Module#info
           * @module ng
           *
           * @param {Object=} info Information about the module
           * @returns {Object|Module} The current info object for this module if called as a getter,
           *                          or `this` if called as a setter.
           *
           * @description
           * Read and write custom information about this module.
           * For example you could put the version of the module in here.
           *
           * ```js
           * angular.module('myModule', []).info({ version: '1.0.0' });
           * ```
           *
           * The version could then be read back out by accessing the module elsewhere:
           *
           * ```
           * var version = angular.module('myModule').info().version;
           * ```
           *
           * You can also retrieve this information during runtime via the
           * {@link $injector#modules `$injector.modules`} property:
           *
           * ```js
           * var version = $injector.modules['myModule'].info().version;
           * ```
           */
          info: function(value) {
            if (isDefined(value)) {
              if (!isObject(value)) throw ngMinErr('aobj', 'Argument \'{0}\' must be an object', 'value');
              info = value;
              return this;
            }
            return info;
          },

          /**
           * @ngdoc property
           * @name angular.Module#requires
           * @module ng
           *
           * @description
           * Holds the list of modules which the injector will load before the current module is
           * loaded.
           */
          requires: requires,

          /**
           * @ngdoc property
           * @name angular.Module#name
           * @module ng
           *
           * @description
           * Name of the module.
           */
          name: name,


          /**
           * @ngdoc method
           * @name angular.Module#provider
           * @module ng
           * @param {string} name service name
           * @param {Function} providerType Construction function for creating new instance of the
           *                                service.
           * @description
           * See {@link auto.$provide#provider $provide.provider()}.
           */
          provider: invokeLaterAndSetModuleName('$provide', 'provider'),

          /**
           * @ngdoc method
           * @name angular.Module#factory
           * @module ng
           * @param {string} name service name
           * @param {Function} providerFunction Function for creating new instance of the service.
           * @description
           * See {@link auto.$provide#factory $provide.factory()}.
           */
          factory: invokeLaterAndSetModuleName('$provide', 'factory'),

          /**
           * @ngdoc method
           * @name angular.Module#service
           * @module ng
           * @param {string} name service name
           * @param {Function} constructor A constructor function that will be instantiated.
           * @description
           * See {@link auto.$provide#service $provide.service()}.
           */
          service: invokeLaterAndSetModuleName('$provide', 'service'),

          /**
           * @ngdoc method
           * @name angular.Module#value
           * @module ng
           * @param {string} name service name
           * @param {*} object Service instance object.
           * @description
           * See {@link auto.$provide#value $provide.value()}.
           */
          value: invokeLater('$provide', 'value'),

          /**
           * @ngdoc method
           * @name angular.Module#constant
           * @module ng
           * @param {string} name constant name
           * @param {*} object Constant value.
           * @description
           * Because the constants are fixed, they get applied before other provide methods.
           * See {@link auto.$provide#constant $provide.constant()}.
           */
          constant: invokeLater('$provide', 'constant', 'unshift'),

           /**
           * @ngdoc method
           * @name angular.Module#decorator
           * @module ng
           * @param {string} name The name of the service to decorate.
           * @param {Function} decorFn This function will be invoked when the service needs to be
           *                           instantiated and should return the decorated service instance.
           * @description
           * See {@link auto.$provide#decorator $provide.decorator()}.
           */
          decorator: invokeLaterAndSetModuleName('$provide', 'decorator', configBlocks),

          /**
           * @ngdoc method
           * @name angular.Module#animation
           * @module ng
           * @param {string} name animation name
           * @param {Function} animationFactory Factory function for creating new instance of an
           *                                    animation.
           * @description
           *
           * **NOTE**: animations take effect only if the **ngAnimate** module is loaded.
           *
           *
           * Defines an animation hook that can be later used with
           * {@link $animate $animate} service and directives that use this service.
           *
           * ```js
           * module.animation('.animation-name', function($inject1, $inject2) {
           *   return {
           *     eventName : function(element, done) {
           *       //code to run the animation
           *       //once complete, then run done()
           *       return function cancellationFunction(element) {
           *         //code to cancel the animation
           *       }
           *     }
           *   }
           * })
           * ```
           *
           * See {@link ng.$animateProvider#register $animateProvider.register()} and
           * {@link ngAnimate ngAnimate module} for more information.
           */
          animation: invokeLaterAndSetModuleName('$animateProvider', 'register'),

          /**
           * @ngdoc method
           * @name angular.Module#filter
           * @module ng
           * @param {string} name Filter name - this must be a valid AngularJS expression identifier
           * @param {Function} filterFactory Factory function for creating new instance of filter.
           * @description
           * See {@link ng.$filterProvider#register $filterProvider.register()}.
           *
           * <div class="alert alert-warning">
           * **Note:** Filter names must be valid AngularJS {@link expression} identifiers, such as `uppercase` or `orderBy`.
           * Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
           * your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
           * (`myapp_subsection_filterx`).
           * </div>
           */
          filter: invokeLaterAndSetModuleName('$filterProvider', 'register'),

          /**
           * @ngdoc method
           * @name angular.Module#controller
           * @module ng
           * @param {string|Object} name Controller name, or an object map of controllers where the
           *    keys are the names and the values are the constructors.
           * @param {Function} constructor Controller constructor function.
           * @description
           * See {@link ng.$controllerProvider#register $controllerProvider.register()}.
           */
          controller: invokeLaterAndSetModuleName('$controllerProvider', 'register'),

          /**
           * @ngdoc method
           * @name angular.Module#directive
           * @module ng
           * @param {string|Object} name Directive name, or an object map of directives where the
           *    keys are the names and the values are the factories.
           * @param {Function} directiveFactory Factory function for creating new instance of
           * directives.
           * @description
           * See {@link ng.$compileProvider#directive $compileProvider.directive()}.
           */
          directive: invokeLaterAndSetModuleName('$compileProvider', 'directive'),

          /**
           * @ngdoc method
           * @name angular.Module#component
           * @module ng
           * @param {string|Object} name Name of the component in camelCase (i.e. `myComp` which will match `<my-comp>`),
           *    or an object map of components where the keys are the names and the values are the component definition objects.
           * @param {Object} options Component definition object (a simplified
           *    {@link ng.$compile#directive-definition-object directive definition object})
           *
           * @description
           * See {@link ng.$compileProvider#component $compileProvider.component()}.
           */
          component: invokeLaterAndSetModuleName('$compileProvider', 'component'),

          /**
           * @ngdoc method
           * @name angular.Module#config
           * @module ng
           * @param {Function} configFn Execute this function on module load. Useful for service
           *    configuration.
           * @description
           * Use this method to configure services by injecting their
           * {@link angular.Module#provider `providers`}, e.g. for adding routes to the
           * {@link ngRoute.$routeProvider $routeProvider}.
           *
           * Note that you can only inject {@link angular.Module#provider `providers`} and
           * {@link angular.Module#constant `constants`} into this function.
           *
           * For more about how to configure services, see
           * {@link providers#provider-recipe Provider Recipe}.
           */
          config: config,

          /**
           * @ngdoc method
           * @name angular.Module#run
           * @module ng
           * @param {Function} initializationFn Execute this function after injector creation.
           *    Useful for application initialization.
           * @description
           * Use this method to register work which should be performed when the injector is done
           * loading all modules.
           */
          run: function(block) {
            runBlocks.push(block);
            return this;
          }
        };

        if (configFn) {
          config(configFn);
        }

        return moduleInstance;

        /**
         * @param {string} provider
         * @param {string} method
         * @param {String=} insertMethod
         * @returns {angular.Module}
         */
        function invokeLater(provider, method, insertMethod, queue) {
          if (!queue) queue = invokeQueue;
          return function() {
            queue[insertMethod || 'push']([provider, method, arguments]);
            return moduleInstance;
          };
        }

        /**
         * @param {string} provider
         * @param {string} method
         * @returns {angular.Module}
         */
        function invokeLaterAndSetModuleName(provider, method, queue) {
          if (!queue) queue = invokeQueue;
          return function(recipeName, factoryFunction) {
            if (factoryFunction && isFunction(factoryFunction)) factoryFunction.$$moduleName = name;
            queue.push([provider, method, arguments]);
            return moduleInstance;
          };
        }
      });
    };
  });

}

/* global shallowCopy: true */

/**
 * Creates a shallow copy of an object, an array or a primitive.
 *
 * Assumes that there are no proto properties for objects.
 */
function shallowCopy(src, dst) {
  if (isArray(src)) {
    dst = dst || [];

    for (var i = 0, ii = src.length; i < ii; i++) {
      dst[i] = src[i];
    }
  } else if (isObject(src)) {
    dst = dst || {};

    for (var key in src) {
      if (!(key.charAt(0) === '$' && key.charAt(1) === '$')) {
        dst[key] = src[key];
      }
    }
  }

  return dst || src;
}

/* exported toDebugString */

function serializeObject(obj, maxDepth) {
  var seen = [];

  // There is no direct way to stringify object until reaching a specific depth
  // and a very deep object can cause a performance issue, so we copy the object
  // based on this specific depth and then stringify it.
  if (isValidObjectMaxDepth(maxDepth)) {
    // This file is also included in `angular-loader`, so `copy()` might not always be available in
    // the closure. Therefore, it is lazily retrieved as `angular.copy()` when needed.
    obj = angular.copy(obj, null, maxDepth);
  }
  return JSON.stringify(obj, function(key, val) {
    val = toJsonReplacer(key, val);
    if (isObject(val)) {

      if (seen.indexOf(val) >= 0) return '...';

      seen.push(val);
    }
    return val;
  });
}

function toDebugString(obj, maxDepth) {
  if (typeof obj === 'function') {
    return obj.toString().replace(/ \{[\s\S]*$/, '');
  } else if (isUndefined(obj)) {
    return 'undefined';
  } else if (typeof obj !== 'string') {
    return serializeObject(obj, maxDepth);
  }
  return obj;
}

/* global angularModule: true,
  version: true,

  $CompileProvider,

  htmlAnchorDirective,
  inputDirective,
  hiddenInputBrowserCacheDirective,
  formDirective,
  scriptDirective,
  selectDirective,
  optionDirective,
  ngBindDirective,
  ngBindHtmlDirective,
  ngBindTemplateDirective,
  ngClassDirective,
  ngClassEvenDirective,
  ngClassOddDirective,
  ngCloakDirective,
  ngControllerDirective,
  ngFormDirective,
  ngHideDirective,
  ngIfDirective,
  ngIncludeDirective,
  ngIncludeFillContentDirective,
  ngInitDirective,
  ngNonBindableDirective,
  ngPluralizeDirective,
  ngRefDirective,
  ngRepeatDirective,
  ngShowDirective,
  ngStyleDirective,
  ngSwitchDirective,
  ngSwitchWhenDirective,
  ngSwitchDefaultDirective,
  ngOptionsDirective,
  ngTranscludeDirective,
  ngModelDirective,
  ngListDirective,
  ngChangeDirective,
  patternDirective,
  patternDirective,
  requiredDirective,
  requiredDirective,
  minlengthDirective,
  minlengthDirective,
  maxlengthDirective,
  maxlengthDirective,
  ngValueDirective,
  ngModelOptionsDirective,
  ngAttributeAliasDirectives,
  ngEventDirectives,

  $AnchorScrollProvider,
  $AnimateProvider,
  $CoreAnimateCssProvider,
  $$CoreAnimateJsProvider,
  $$CoreAnimateQueueProvider,
  $$AnimateRunnerFactoryProvider,
  $$AnimateAsyncRunFactoryProvider,
  $BrowserProvider,
  $CacheFactoryProvider,
  $ControllerProvider,
  $DateProvider,
  $DocumentProvider,
  $$IsDocumentHiddenProvider,
  $ExceptionHandlerProvider,
  $FilterProvider,
  $$ForceReflowProvider,
  $InterpolateProvider,
  $$IntervalFactoryProvider,
  $IntervalProvider,
  $HttpProvider,
  $HttpParamSerializerProvider,
  $HttpParamSerializerJQLikeProvider,
  $HttpBackendProvider,
  $xhrFactoryProvider,
  $jsonpCallbacksProvider,
  $LocationProvider,
  $LogProvider,
  $$MapProvider,
  $ParseProvider,
  $RootScopeProvider,
  $QProvider,
  $$QProvider,
  $$SanitizeUriProvider,
  $SceProvider,
  $SceDelegateProvider,
  $SnifferProvider,
  $$TaskTrackerFactoryProvider,
  $TemplateCacheProvider,
  $TemplateRequestProvider,
  $$TestabilityProvider,
  $TimeoutProvider,
  $$RAFProvider,
  $WindowProvider,
  $$jqLiteProvider,
  $$CookieReaderProvider
*/


/**
 * @ngdoc object
 * @name angular.version
 * @module ng
 * @description
 * An object that contains information about the current AngularJS version.
 *
 * This object has the following properties:
 *
 * - `full` – `{string}` – Full version string, such as "0.9.18".
 * - `major` – `{number}` – Major version number, such as "0".
 * - `minor` – `{number}` – Minor version number, such as "9".
 * - `dot` – `{number}` – Dot version number, such as "18".
 * - `codeName` – `{string}` – Code name of the release, such as "jiggling-armfat".
 */
var version = {
  // These placeholder strings will be replaced by grunt's `build` task.
  // They need to be double- or single-quoted.
  full: '1.8.0',
  major: 1,
  minor: 8,
  dot: 0,
  codeName: 'nested-vaccination'
};


function publishExternalAPI(angular) {
  extend(angular, {
    'errorHandlingConfig': errorHandlingConfig,
    'bootstrap': bootstrap,
    'copy': copy,
    'extend': extend,
    'merge': merge,
    'equals': equals,
    'element': jqLite,
    'forEach': forEach,
    'injector': createInjector,
    'noop': noop,
    'bind': bind,
    'toJson': toJson,
    'fromJson': fromJson,
    'identity': identity,
    'isUndefined': isUndefined,
    'isDefined': isDefined,
    'isString': isString,
    'isFunction': isFunction,
    'isObject': isObject,
    'isNumber': isNumber,
    'isElement': isElement,
    'isArray': isArray,
    'version': version,
    'isDate': isDate,
    'callbacks': {$$counter: 0},
    'getTestability': getTestability,
    'reloadWithDebugInfo': reloadWithDebugInfo,
    'UNSAFE_restoreLegacyJqLiteXHTMLReplacement': UNSAFE_restoreLegacyJqLiteXHTMLReplacement,
    '$$minErr': minErr,
    '$$csp': csp,
    '$$encodeUriSegment': encodeUriSegment,
    '$$encodeUriQuery': encodeUriQuery,
    '$$lowercase': lowercase,
    '$$stringify': stringify,
    '$$uppercase': uppercase
  });

  angularModule = setupModuleLoader(window);

  angularModule('ng', ['ngLocale'], ['$provide',
    function ngModule($provide) {
      // $$sanitizeUriProvider needs to be before $compileProvider as it is used by it.
      $provide.provider({
        $$sanitizeUri: $$SanitizeUriProvider
      });
      $provide.provider('$compile', $CompileProvider).
        directive({
            a: htmlAnchorDirective,
            input: inputDirective,
            textarea: inputDirective,
            form: formDirective,
            script: scriptDirective,
            select: selectDirective,
            option: optionDirective,
            ngBind: ngBindDirective,
            ngBindHtml: ngBindHtmlDirective,
            ngBindTemplate: ngBindTemplateDirective,
            ngClass: ngClassDirective,
            ngClassEven: ngClassEvenDirective,
            ngClassOdd: ngClassOddDirective,
            ngCloak: ngCloakDirective,
            ngController: ngControllerDirective,
            ngForm: ngFormDirective,
            ngHide: ngHideDirective,
            ngIf: ngIfDirective,
            ngInclude: ngIncludeDirective,
            ngInit: ngInitDirective,
            ngNonBindable: ngNonBindableDirective,
            ngPluralize: ngPluralizeDirective,
            ngRef: ngRefDirective,
            ngRepeat: ngRepeatDirective,
            ngShow: ngShowDirective,
            ngStyle: ngStyleDirective,
            ngSwitch: ngSwitchDirective,
            ngSwitchWhen: ngSwitchWhenDirective,
            ngSwitchDefault: ngSwitchDefaultDirective,
            ngOptions: ngOptionsDirective,
            ngTransclude: ngTranscludeDirective,
            ngModel: ngModelDirective,
            ngList: ngListDirective,
            ngChange: ngChangeDirective,
            pattern: patternDirective,
            ngPattern: patternDirective,
            required: requiredDirective,
            ngRequired: requiredDirective,
            minlength: minlengthDirective,
            ngMinlength: minlengthDirective,
            maxlength: maxlengthDirective,
            ngMaxlength: maxlengthDirective,
            ngValue: ngValueDirective,
            ngModelOptions: ngModelOptionsDirective
        }).
        directive({
          ngInclude: ngIncludeFillContentDirective,
          input: hiddenInputBrowserCacheDirective
        }).
        directive(ngAttributeAliasDirectives).
        directive(ngEventDirectives);
      $provide.provider({
        $anchorScroll: $AnchorScrollProvider,
        $animate: $AnimateProvider,
        $animateCss: $CoreAnimateCssProvider,
        $$animateJs: $$CoreAnimateJsProvider,
        $$animateQueue: $$CoreAnimateQueueProvider,
        $$AnimateRunner: $$AnimateRunnerFactoryProvider,
        $$animateAsyncRun: $$AnimateAsyncRunFactoryProvider,
        $browser: $BrowserProvider,
        $cacheFactory: $CacheFactoryProvider,
        $controller: $ControllerProvider,
        $document: $DocumentProvider,
        $$isDocumentHidden: $$IsDocumentHiddenProvider,
        $exceptionHandler: $ExceptionHandlerProvider,
        $filter: $FilterProvider,
        $$forceReflow: $$ForceReflowProvider,
        $interpolate: $InterpolateProvider,
        $interval: $IntervalProvider,
        $$intervalFactory: $$IntervalFactoryProvider,
        $http: $HttpProvider,
        $httpParamSerializer: $HttpParamSerializerProvider,
        $httpParamSerializerJQLike: $HttpParamSerializerJQLikeProvider,
        $httpBackend: $HttpBackendProvider,
        $xhrFactory: $xhrFactoryProvider,
        $jsonpCallbacks: $jsonpCallbacksProvider,
        $location: $LocationProvider,
        $log: $LogProvider,
        $parse: $ParseProvider,
        $rootScope: $RootScopeProvider,
        $q: $QProvider,
        $$q: $$QProvider,
        $sce: $SceProvider,
        $sceDelegate: $SceDelegateProvider,
        $sniffer: $SnifferProvider,
        $$taskTrackerFactory: $$TaskTrackerFactoryProvider,
        $templateCache: $TemplateCacheProvider,
        $templateRequest: $TemplateRequestProvider,
        $$testability: $$TestabilityProvider,
        $timeout: $TimeoutProvider,
        $window: $WindowProvider,
        $$rAF: $$RAFProvider,
        $$jqLite: $$jqLiteProvider,
        $$Map: $$MapProvider,
        $$cookieReader: $$CookieReaderProvider
      });
    }
  ])
  .info({ angularVersion: '1.8.0' });
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* global
  JQLitePrototype: true,
  BOOLEAN_ATTR: true,
  ALIASED_ATTR: true
*/

//////////////////////////////////
//JQLite
//////////////////////////////////

/**
 * @ngdoc function
 * @name angular.element
 * @module ng
 * @kind function
 *
 * @description
 * Wraps a raw DOM element or HTML string as a [jQuery](http://jquery.com) element.
 *
 * If jQuery is available, `angular.element` is an alias for the
 * [jQuery](http://api.jquery.com/jQuery/) function. If jQuery is not available, `angular.element`
 * delegates to AngularJS's built-in subset of jQuery, called "jQuery lite" or **jqLite**.
 *
 * jqLite is a tiny, API-compatible subset of jQuery that allows
 * AngularJS to manipulate the DOM in a cross-browser compatible way. jqLite implements only the most
 * commonly needed functionality with the goal of having a very small footprint.
 *
 * To use `jQuery`, simply ensure it is loaded before the `angular.js` file. You can also use the
 * {@link ngJq `ngJq`} directive to specify that jqlite should be used over jQuery, or to use a
 * specific version of jQuery if multiple versions exist on the page.
 *
 * <div class="alert alert-info">**Note:** All element references in AngularJS are always wrapped with jQuery or
 * jqLite (such as the element argument in a directive's compile / link function). They are never raw DOM references.</div>
 *
 * <div class="alert alert-warning">**Note:** Keep in mind that this function will not find elements
 * by tag name / CSS selector. For lookups by tag name, try instead `angular.element(document).find(...)`
 * or `$document.find()`, or use the standard DOM APIs, e.g. `document.querySelectorAll()`.</div>
 *
 * ## AngularJS's jqLite
 * jqLite provides only the following jQuery methods:
 *
 * - [`addClass()`](http://api.jquery.com/addClass/) - Does not support a function as first argument
 * - [`after()`](http://api.jquery.com/after/)
 * - [`append()`](http://api.jquery.com/append/) - Contrary to jQuery, this doesn't clone elements
 *   so will not work correctly when invoked on a jqLite object containing more than one DOM node
 * - [`attr()`](http://api.jquery.com/attr/) - Does not support functions as parameters
 * - [`bind()`](http://api.jquery.com/bind/) (_deprecated_, use [`on()`](http://api.jquery.com/on/)) - Does not support namespaces, selectors or eventData
 * - [`children()`](http://api.jquery.com/children/) - Does not support selectors
 * - [`clone()`](http://api.jquery.com/clone/)
 * - [`contents()`](http://api.jquery.com/contents/)
 * - [`css()`](http://api.jquery.com/css/) - Only retrieves inline-styles, does not call `getComputedStyle()`.
 *   As a setter, does not convert numbers to strings or append 'px', and also does not have automatic property prefixing.
 * - [`data()`](http://api.jquery.com/data/)
 * - [`detach()`](http://api.jquery.com/detach/)
 * - [`empty()`](http://api.jquery.com/empty/)
 * - [`eq()`](http://api.jquery.com/eq/)
 * - [`find()`](http://api.jquery.com/find/) - Limited to lookups by tag name
 * - [`hasClass()`](http://api.jquery.com/hasClass/)
 * - [`html()`](http://api.jquery.com/html/)
 * - [`next()`](http://api.jquery.com/next/) - Does not support selectors
 * - [`on()`](http://api.jquery.com/on/) - Does not support namespaces, selectors or eventData
 * - [`off()`](http://api.jquery.com/off/) - Does not support namespaces, selectors or event object as parameter
 * - [`one()`](http://api.jquery.com/one/) - Does not support namespaces or selectors
 * - [`parent()`](http://api.jquery.com/parent/) - Does not support selectors
 * - [`prepend()`](http://api.jquery.com/prepend/)
 * - [`prop()`](http://api.jquery.com/prop/)
 * - [`ready()`](http://api.jquery.com/ready/) (_deprecated_, use `angular.element(callback)` instead of `angular.element(document).ready(callback)`)
 * - [`remove()`](http://api.jquery.com/remove/)
 * - [`removeAttr()`](http://api.jquery.com/removeAttr/) - Does not support multiple attributes
 * - [`removeClass()`](http://api.jquery.com/removeClass/) - Does not support a function as first argument
 * - [`removeData()`](http://api.jquery.com/removeData/)
 * - [`replaceWith()`](http://api.jquery.com/replaceWith/)
 * - [`text()`](http://api.jquery.com/text/)
 * - [`toggleClass()`](http://api.jquery.com/toggleClass/) - Does not support a function as first argument
 * - [`triggerHandler()`](http://api.jquery.com/triggerHandler/) - Passes a dummy event object to handlers
 * - [`unbind()`](http://api.jquery.com/unbind/) (_deprecated_, use [`off()`](http://api.jquery.com/off/)) - Does not support namespaces or event object as parameter
 * - [`val()`](http://api.jquery.com/val/)
 * - [`wrap()`](http://api.jquery.com/wrap/)
 *
 * jqLite also provides a method restoring pre-1.8 insecure treatment of XHTML-like tags.
 * This legacy behavior turns input like `<div /><span />` to `<div></div><span></span>`
 * instead of `<div><span></span></div>` like version 1.8 & newer do. To restore it, invoke:
 * ```js
 * angular.UNSAFE_restoreLegacyJqLiteXHTMLReplacement();
 * ```
 * Note that this only patches jqLite. If you use jQuery 3.5.0 or newer, please read the
 * [jQuery 3.5 upgrade guide](https://jquery.com/upgrade-guide/3.5/) for more details
 * about the workarounds.
 *
 * ## jQuery/jqLite Extras
 * AngularJS also provides the following additional methods and events to both jQuery and jqLite:
 *
 * ### Events
 * - `$destroy` - AngularJS intercepts all jqLite/jQuery's DOM destruction apis and fires this event
 *    on all DOM nodes being removed.  This can be used to clean up any 3rd party bindings to the DOM
 *    element before it is removed.
 *
 * ### Methods
 * - `controller(name)` - retrieves the controller of the current element or its parent. By default
 *   retrieves controller associated with the `ngController` directive. If `name` is provided as
 *   camelCase directive name, then the controller for this directive will be retrieved (e.g.
 *   `'ngModel'`).
 * - `injector()` - retrieves the injector of the current element or its parent.
 * - `scope()` - retrieves the {@link ng.$rootScope.Scope scope} of the current
 *   element or its parent. Requires {@link guide/production#disabling-debug-data Debug Data} to
 *   be enabled.
 * - `isolateScope()` - retrieves an isolate {@link ng.$rootScope.Scope scope} if one is attached directly to the
 *   current element. This getter should be used only on elements that contain a directive which starts a new isolate
 *   scope. Calling `scope()` on this element always returns the original non-isolate scope.
 *   Requires {@link guide/production#disabling-debug-data Debug Data} to be enabled.
 * - `inheritedData()` - same as `data()`, but walks up the DOM until a value is found or the top
 *   parent element is reached.
 *
 * @knownIssue You cannot spy on `angular.element` if you are using Jasmine version 1.x. See
 * https://github.com/angular/angular.js/issues/14251 for more information.
 *
 * @param {string|DOMElement} element HTML string or DOMElement to be wrapped into jQuery.
 * @returns {Object} jQuery object.
 */

JQLite.expando = 'ng339';

var jqCache = JQLite.cache = {},
    jqId = 1;

/*
 * !!! This is an undocumented "private" function !!!
 */
JQLite._data = function(node) {
  //jQuery always returns an object on cache miss
  return this.cache[node[this.expando]] || {};
};

function jqNextId() { return ++jqId; }


var DASH_LOWERCASE_REGEXP = /-([a-z])/g;
var MS_HACK_REGEXP = /^-ms-/;
var MOUSE_EVENT_MAP = { mouseleave: 'mouseout', mouseenter: 'mouseover' };
var jqLiteMinErr = minErr('jqLite');

/**
 * Converts kebab-case to camelCase.
 * There is also a special case for the ms prefix starting with a lowercase letter.
 * @param name Name to normalize
 */
function cssKebabToCamel(name) {
    return kebabToCamel(name.replace(MS_HACK_REGEXP, 'ms-'));
}

function fnCamelCaseReplace(all, letter) {
  return letter.toUpperCase();
}

/**
 * Converts kebab-case to camelCase.
 * @param name Name to normalize
 */
function kebabToCamel(name) {
  return name
    .replace(DASH_LOWERCASE_REGEXP, fnCamelCaseReplace);
}

var SINGLE_TAG_REGEXP = /^<([\w-]+)\s*\/?>(?:<\/\1>|)$/;
var HTML_REGEXP = /<|&#?\w+;/;
var TAG_NAME_REGEXP = /<([\w:-]+)/;
var XHTML_TAG_REGEXP = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:-]+)[^>]*)\/>/gi;

// Table parts need to be wrapped with `<table>` or they're
// stripped to their contents when put in a div.
// XHTML parsers do not magically insert elements in the
// same way that tag soup parsers do, so we cannot shorten
// this by omitting <tbody> or other required elements.
var wrapMap = {
  thead: ['table'],
  col: ['colgroup', 'table'],
  tr: ['tbody', 'table'],
  td: ['tr', 'tbody', 'table']
};

wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

// Support: IE <10 only
// IE 9 requires an option wrapper & it needs to have the whole table structure
// set up in advance; assigning `"<td></td>"` to `tr.innerHTML` doesn't work, etc.
var wrapMapIE9 = {
  option: [1, '<select multiple="multiple">', '</select>'],
  _default: [0, '', '']
};

for (var key in wrapMap) {
  var wrapMapValueClosing = wrapMap[key];
  var wrapMapValue = wrapMapValueClosing.slice().reverse();
  wrapMapIE9[key] = [wrapMapValue.length, '<' + wrapMapValue.join('><') + '>', '</' + wrapMapValueClosing.join('></') + '>'];
}

wrapMapIE9.optgroup = wrapMapIE9.option;

function jqLiteIsTextNode(html) {
  return !HTML_REGEXP.test(html);
}

function jqLiteAcceptsData(node) {
  // The window object can accept data but has no nodeType
  // Otherwise we are only interested in elements (1) and documents (9)
  var nodeType = node.nodeType;
  return nodeType === NODE_TYPE_ELEMENT || !nodeType || nodeType === NODE_TYPE_DOCUMENT;
}

function jqLiteHasData(node) {
  for (var key in jqCache[node.ng339]) {
    return true;
  }
  return false;
}

function jqLiteBuildFragment(html, context) {
  var tmp, tag, wrap, finalHtml,
      fragment = context.createDocumentFragment(),
      nodes = [], i;

  if (jqLiteIsTextNode(html)) {
    // Convert non-html into a text node
    nodes.push(context.createTextNode(html));
  } else {
    // Convert html into DOM nodes
    tmp = fragment.appendChild(context.createElement('div'));
    tag = (TAG_NAME_REGEXP.exec(html) || ['', ''])[1].toLowerCase();
    finalHtml = JQLite.legacyXHTMLReplacement ?
      html.replace(XHTML_TAG_REGEXP, '<$1></$2>') :
      html;

    if (msie < 10) {
      wrap = wrapMapIE9[tag] || wrapMapIE9._default;
      tmp.innerHTML = wrap[1] + finalHtml + wrap[2];

      // Descend through wrappers to the right content
      i = wrap[0];
      while (i--) {
        tmp = tmp.firstChild;
      }
    } else {
      wrap = wrapMap[tag] || [];

      // Create wrappers & descend into them
      i = wrap.length;
      while (--i > -1) {
        tmp.appendChild(window.document.createElement(wrap[i]));
        tmp = tmp.firstChild;
      }

      tmp.innerHTML = finalHtml;
    }

    nodes = concat(nodes, tmp.childNodes);

    tmp = fragment.firstChild;
    tmp.textContent = '';
  }

  // Remove wrapper from fragment
  fragment.textContent = '';
  fragment.innerHTML = ''; // Clear inner HTML
  forEach(nodes, function(node) {
    fragment.appendChild(node);
  });

  return fragment;
}

function jqLiteParseHTML(html, context) {
  context = context || window.document;
  var parsed;

  if ((parsed = SINGLE_TAG_REGEXP.exec(html))) {
    return [context.createElement(parsed[1])];
  }

  if ((parsed = jqLiteBuildFragment(html, context))) {
    return parsed.childNodes;
  }

  return [];
}

function jqLiteWrapNode(node, wrapper) {
  var parent = node.parentNode;

  if (parent) {
    parent.replaceChild(wrapper, node);
  }

  wrapper.appendChild(node);
}


// IE9-11 has no method "contains" in SVG element and in Node.prototype. Bug #10259.
var jqLiteContains = window.Node.prototype.contains || /** @this */ function(arg) {
  // eslint-disable-next-line no-bitwise
  return !!(this.compareDocumentPosition(arg) & 16);
};

/////////////////////////////////////////////
function JQLite(element) {
  if (element instanceof JQLite) {
    return element;
  }

  var argIsString;

  if (isString(element)) {
    element = trim(element);
    argIsString = true;
  }
  if (!(this instanceof JQLite)) {
    if (argIsString && element.charAt(0) !== '<') {
      throw jqLiteMinErr('nosel', 'Looking up elements via selectors is not supported by jqLite! See: http://docs.angularjs.org/api/angular.element');
    }
    return new JQLite(element);
  }

  if (argIsString) {
    jqLiteAddNodes(this, jqLiteParseHTML(element));
  } else if (isFunction(element)) {
    jqLiteReady(element);
  } else {
    jqLiteAddNodes(this, element);
  }
}

function jqLiteClone(element) {
  return element.cloneNode(true);
}

function jqLiteDealoc(element, onlyDescendants) {
  if (!onlyDescendants && jqLiteAcceptsData(element)) jqLite.cleanData([element]);

  if (element.querySelectorAll) {
    jqLite.cleanData(element.querySelectorAll('*'));
  }
}

function isEmptyObject(obj) {
  var name;

  for (name in obj) {
    return false;
  }
  return true;
}

function removeIfEmptyData(element) {
  var expandoId = element.ng339;
  var expandoStore = expandoId && jqCache[expandoId];

  var events = expandoStore && expandoStore.events;
  var data = expandoStore && expandoStore.data;

  if ((!data || isEmptyObject(data)) && (!events || isEmptyObject(events))) {
    delete jqCache[expandoId];
    element.ng339 = undefined; // don't delete DOM expandos. IE and Chrome don't like it
  }
}

function jqLiteOff(element, type, fn, unsupported) {
  if (isDefined(unsupported)) throw jqLiteMinErr('offargs', 'jqLite#off() does not support the `selector` argument');

  var expandoStore = jqLiteExpandoStore(element);
  var events = expandoStore && expandoStore.events;
  var handle = expandoStore && expandoStore.handle;

  if (!handle) return; //no listeners registered

  if (!type) {
    for (type in events) {
      if (type !== '$destroy') {
        element.removeEventListener(type, handle);
      }
      delete events[type];
    }
  } else {

    var removeHandler = function(type) {
      var listenerFns = events[type];
      if (isDefined(fn)) {
        arrayRemove(listenerFns || [], fn);
      }
      if (!(isDefined(fn) && listenerFns && listenerFns.length > 0)) {
        element.removeEventListener(type, handle);
        delete events[type];
      }
    };

    forEach(type.split(' '), function(type) {
      removeHandler(type);
      if (MOUSE_EVENT_MAP[type]) {
        removeHandler(MOUSE_EVENT_MAP[type]);
      }
    });
  }

  removeIfEmptyData(element);
}

function jqLiteRemoveData(element, name) {
  var expandoId = element.ng339;
  var expandoStore = expandoId && jqCache[expandoId];

  if (expandoStore) {
    if (name) {
      delete expandoStore.data[name];
    } else {
      expandoStore.data = {};
    }

    removeIfEmptyData(element);
  }
}


function jqLiteExpandoStore(element, createIfNecessary) {
  var expandoId = element.ng339,
      expandoStore = expandoId && jqCache[expandoId];

  if (createIfNecessary && !expandoStore) {
    element.ng339 = expandoId = jqNextId();
    expandoStore = jqCache[expandoId] = {events: {}, data: {}, handle: undefined};
  }

  return expandoStore;
}


function jqLiteData(element, key, value) {
  if (jqLiteAcceptsData(element)) {
    var prop;

    var isSimpleSetter = isDefined(value);
    var isSimpleGetter = !isSimpleSetter && key && !isObject(key);
    var massGetter = !key;
    var expandoStore = jqLiteExpandoStore(element, !isSimpleGetter);
    var data = expandoStore && expandoStore.data;

    if (isSimpleSetter) { // data('key', value)
      data[kebabToCamel(key)] = value;
    } else {
      if (massGetter) {  // data()
        return data;
      } else {
        if (isSimpleGetter) { // data('key')
          // don't force creation of expandoStore if it doesn't exist yet
          return data && data[kebabToCamel(key)];
        } else { // mass-setter: data({key1: val1, key2: val2})
          for (prop in key) {
            data[kebabToCamel(prop)] = key[prop];
          }
        }
      }
    }
  }
}

function jqLiteHasClass(element, selector) {
  if (!element.getAttribute) return false;
  return ((' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ').
      indexOf(' ' + selector + ' ') > -1);
}

function jqLiteRemoveClass(element, cssClasses) {
  if (cssClasses && element.setAttribute) {
    var existingClasses = (' ' + (element.getAttribute('class') || '') + ' ')
                            .replace(/[\n\t]/g, ' ');
    var newClasses = existingClasses;

    forEach(cssClasses.split(' '), function(cssClass) {
      cssClass = trim(cssClass);
      newClasses = newClasses.replace(' ' + cssClass + ' ', ' ');
    });

    if (newClasses !== existingClasses) {
      element.setAttribute('class', trim(newClasses));
    }
  }
}

function jqLiteAddClass(element, cssClasses) {
  if (cssClasses && element.setAttribute) {
    var existingClasses = (' ' + (element.getAttribute('class') || '') + ' ')
                            .replace(/[\n\t]/g, ' ');
    var newClasses = existingClasses;

    forEach(cssClasses.split(' '), function(cssClass) {
      cssClass = trim(cssClass);
      if (newClasses.indexOf(' ' + cssClass + ' ') === -1) {
        newClasses += cssClass + ' ';
      }
    });

    if (newClasses !== existingClasses) {
      element.setAttribute('class', trim(newClasses));
    }
  }
}


function jqLiteAddNodes(root, elements) {
  // THIS CODE IS VERY HOT. Don't make changes without benchmarking.

  if (elements) {

    // if a Node (the most common case)
    if (elements.nodeType) {
      root[root.length++] = elements;
    } else {
      var length = elements.length;

      // if an Array or NodeList and not a Window
      if (typeof length === 'number' && elements.window !== elements) {
        if (length) {
          for (var i = 0; i < length; i++) {
            root[root.length++] = elements[i];
          }
        }
      } else {
        root[root.length++] = elements;
      }
    }
  }
}


function jqLiteController(element, name) {
  return jqLiteInheritedData(element, '$' + (name || 'ngController') + 'Controller');
}

function jqLiteInheritedData(element, name, value) {
  // if element is the document object work with the html element instead
  // this makes $(document).scope() possible
  if (element.nodeType === NODE_TYPE_DOCUMENT) {
    element = element.documentElement;
  }
  var names = isArray(name) ? name : [name];

  while (element) {
    for (var i = 0, ii = names.length; i < ii; i++) {
      if (isDefined(value = jqLite.data(element, names[i]))) return value;
    }

    // If dealing with a document fragment node with a host element, and no parent, use the host
    // element as the parent. This enables directives within a Shadow DOM or polyfilled Shadow DOM
    // to lookup parent controllers.
    element = element.parentNode || (element.nodeType === NODE_TYPE_DOCUMENT_FRAGMENT && element.host);
  }
}

function jqLiteEmpty(element) {
  jqLiteDealoc(element, true);
  while (element.firstChild) {
    element.removeChild(element.firstChild);
  }
}

function jqLiteRemove(element, keepData) {
  if (!keepData) jqLiteDealoc(element);
  var parent = element.parentNode;
  if (parent) parent.removeChild(element);
}


function jqLiteDocumentLoaded(action, win) {
  win = win || window;
  if (win.document.readyState === 'complete') {
    // Force the action to be run async for consistent behavior
    // from the action's point of view
    // i.e. it will definitely not be in a $apply
    win.setTimeout(action);
  } else {
    // No need to unbind this handler as load is only ever called once
    jqLite(win).on('load', action);
  }
}

function jqLiteReady(fn) {
  function trigger() {
    window.document.removeEventListener('DOMContentLoaded', trigger);
    window.removeEventListener('load', trigger);
    fn();
  }

  // check if document is already loaded
  if (window.document.readyState === 'complete') {
    window.setTimeout(fn);
  } else {
    // We can not use jqLite since we are not done loading and jQuery could be loaded later.

    // Works for modern browsers and IE9
    window.document.addEventListener('DOMContentLoaded', trigger);

    // Fallback to window.onload for others
    window.addEventListener('load', trigger);
  }
}

//////////////////////////////////////////
// Functions which are declared directly.
//////////////////////////////////////////
var JQLitePrototype = JQLite.prototype = {
  ready: jqLiteReady,
  toString: function() {
    var value = [];
    forEach(this, function(e) { value.push('' + e);});
    return '[' + value.join(', ') + ']';
  },

  eq: function(index) {
      return (index >= 0) ? jqLite(this[index]) : jqLite(this[this.length + index]);
  },

  length: 0,
  push: push,
  sort: [].sort,
  splice: [].splice
};

//////////////////////////////////////////
// Functions iterating getter/setters.
// these functions return self on setter and
// value on get.
//////////////////////////////////////////
var BOOLEAN_ATTR = {};
forEach('multiple,selected,checked,disabled,readOnly,required,open'.split(','), function(value) {
  BOOLEAN_ATTR[lowercase(value)] = value;
});
var BOOLEAN_ELEMENTS = {};
forEach('input,select,option,textarea,button,form,details'.split(','), function(value) {
  BOOLEAN_ELEMENTS[value] = true;
});
var ALIASED_ATTR = {
  'ngMinlength': 'minlength',
  'ngMaxlength': 'maxlength',
  'ngMin': 'min',
  'ngMax': 'max',
  'ngPattern': 'pattern',
  'ngStep': 'step'
};

function getBooleanAttrName(element, name) {
  // check dom last since we will most likely fail on name
  var booleanAttr = BOOLEAN_ATTR[name.toLowerCase()];

  // booleanAttr is here twice to minimize DOM access
  return booleanAttr && BOOLEAN_ELEMENTS[nodeName_(element)] && booleanAttr;
}

function getAliasedAttrName(name) {
  return ALIASED_ATTR[name];
}

forEach({
  data: jqLiteData,
  removeData: jqLiteRemoveData,
  hasData: jqLiteHasData,
  cleanData: function jqLiteCleanData(nodes) {
    for (var i = 0, ii = nodes.length; i < ii; i++) {
      jqLiteRemoveData(nodes[i]);
      jqLiteOff(nodes[i]);
    }
  }
}, function(fn, name) {
  JQLite[name] = fn;
});

forEach({
  data: jqLiteData,
  inheritedData: jqLiteInheritedData,

  scope: function(element) {
    // Can't use jqLiteData here directly so we stay compatible with jQuery!
    return jqLite.data(element, '$scope') || jqLiteInheritedData(element.parentNode || element, ['$isolateScope', '$scope']);
  },

  isolateScope: function(element) {
    // Can't use jqLiteData here directly so we stay compatible with jQuery!
    return jqLite.data(element, '$isolateScope') || jqLite.data(element, '$isolateScopeNoTemplate');
  },

  controller: jqLiteController,

  injector: function(element) {
    return jqLiteInheritedData(element, '$injector');
  },

  removeAttr: function(element, name) {
    element.removeAttribute(name);
  },

  hasClass: jqLiteHasClass,

  css: function(element, name, value) {
    name = cssKebabToCamel(name);

    if (isDefined(value)) {
      element.style[name] = value;
    } else {
      return element.style[name];
    }
  },

  attr: function(element, name, value) {
    var ret;
    var nodeType = element.nodeType;
    if (nodeType === NODE_TYPE_TEXT || nodeType === NODE_TYPE_ATTRIBUTE || nodeType === NODE_TYPE_COMMENT ||
      !element.getAttribute) {
      return;
    }

    var lowercasedName = lowercase(name);
    var isBooleanAttr = BOOLEAN_ATTR[lowercasedName];

    if (isDefined(value)) {
      // setter

      if (value === null || (value === false && isBooleanAttr)) {
        element.removeAttribute(name);
      } else {
        element.setAttribute(name, isBooleanAttr ? lowercasedName : value);
      }
    } else {
      // getter

      ret = element.getAttribute(name);

      if (isBooleanAttr && ret !== null) {
        ret = lowercasedName;
      }
      // Normalize non-existing attributes to undefined (as jQuery).
      return ret === null ? undefined : ret;
    }
  },

  prop: function(element, name, value) {
    if (isDefined(value)) {
      element[name] = value;
    } else {
      return element[name];
    }
  },

  text: (function() {
    getText.$dv = '';
    return getText;

    function getText(element, value) {
      if (isUndefined(value)) {
        var nodeType = element.nodeType;
        return (nodeType === NODE_TYPE_ELEMENT || nodeType === NODE_TYPE_TEXT) ? element.textContent : '';
      }
      element.textContent = value;
    }
  })(),

  val: function(element, value) {
    if (isUndefined(value)) {
      if (element.multiple && nodeName_(element) === 'select') {
        var result = [];
        forEach(element.options, function(option) {
          if (option.selected) {
            result.push(option.value || option.text);
          }
        });
        return result;
      }
      return element.value;
    }
    element.value = value;
  },

  html: function(element, value) {
    if (isUndefined(value)) {
      return element.innerHTML;
    }
    jqLiteDealoc(element, true);
    element.innerHTML = value;
  },

  empty: jqLiteEmpty
}, function(fn, name) {
  /**
   * Properties: writes return selection, reads return first value
   */
  JQLite.prototype[name] = function(arg1, arg2) {
    var i, key;
    var nodeCount = this.length;

    // jqLiteHasClass has only two arguments, but is a getter-only fn, so we need to special-case it
    // in a way that survives minification.
    // jqLiteEmpty takes no arguments but is a setter.
    if (fn !== jqLiteEmpty &&
        (isUndefined((fn.length === 2 && (fn !== jqLiteHasClass && fn !== jqLiteController)) ? arg1 : arg2))) {
      if (isObject(arg1)) {

        // we are a write, but the object properties are the key/values
        for (i = 0; i < nodeCount; i++) {
          if (fn === jqLiteData) {
            // data() takes the whole object in jQuery
            fn(this[i], arg1);
          } else {
            for (key in arg1) {
              fn(this[i], key, arg1[key]);
            }
          }
        }
        // return self for chaining
        return this;
      } else {
        // we are a read, so read the first child.
        // TODO: do we still need this?
        var value = fn.$dv;
        // Only if we have $dv do we iterate over all, otherwise it is just the first element.
        var jj = (isUndefined(value)) ? Math.min(nodeCount, 1) : nodeCount;
        for (var j = 0; j < jj; j++) {
          var nodeValue = fn(this[j], arg1, arg2);
          value = value ? value + nodeValue : nodeValue;
        }
        return value;
      }
    } else {
      // we are a write, so apply to all children
      for (i = 0; i < nodeCount; i++) {
        fn(this[i], arg1, arg2);
      }
      // return self for chaining
      return this;
    }
  };
});

function createEventHandler(element, events) {
  var eventHandler = function(event, type) {
    // jQuery specific api
    event.isDefaultPrevented = function() {
      return event.defaultPrevented;
    };

    var eventFns = events[type || event.type];
    var eventFnsLength = eventFns ? eventFns.length : 0;

    if (!eventFnsLength) return;

    if (isUndefined(event.immediatePropagationStopped)) {
      var originalStopImmediatePropagation = event.stopImmediatePropagation;
      event.stopImmediatePropagation = function() {
        event.immediatePropagationStopped = true;

        if (event.stopPropagation) {
          event.stopPropagation();
        }

        if (originalStopImmediatePropagation) {
          originalStopImmediatePropagation.call(event);
        }
      };
    }

    event.isImmediatePropagationStopped = function() {
      return event.immediatePropagationStopped === true;
    };

    // Some events have special handlers that wrap the real handler
    var handlerWrapper = eventFns.specialHandlerWrapper || defaultHandlerWrapper;

    // Copy event handlers in case event handlers array is modified during execution.
    if ((eventFnsLength > 1)) {
      eventFns = shallowCopy(eventFns);
    }

    for (var i = 0; i < eventFnsLength; i++) {
      if (!event.isImmediatePropagationStopped()) {
        handlerWrapper(element, event, eventFns[i]);
      }
    }
  };

  // TODO: this is a hack for angularMocks/clearDataCache that makes it possible to deregister all
  //       events on `element`
  eventHandler.elem = element;
  return eventHandler;
}

function defaultHandlerWrapper(element, event, handler) {
  handler.call(element, event);
}

function specialMouseHandlerWrapper(target, event, handler) {
  // Refer to jQuery's implementation of mouseenter & mouseleave
  // Read about mouseenter and mouseleave:
  // http://www.quirksmode.org/js/events_mouse.html#link8
  var related = event.relatedTarget;
  // For mousenter/leave call the handler if related is outside the target.
  // NB: No relatedTarget if the mouse left/entered the browser window
  if (!related || (related !== target && !jqLiteContains.call(target, related))) {
    handler.call(target, event);
  }
}

//////////////////////////////////////////
// Functions iterating traversal.
// These functions chain results into a single
// selector.
//////////////////////////////////////////
forEach({
  removeData: jqLiteRemoveData,

  on: function jqLiteOn(element, type, fn, unsupported) {
    if (isDefined(unsupported)) throw jqLiteMinErr('onargs', 'jqLite#on() does not support the `selector` or `eventData` parameters');

    // Do not add event handlers to non-elements because they will not be cleaned up.
    if (!jqLiteAcceptsData(element)) {
      return;
    }

    var expandoStore = jqLiteExpandoStore(element, true);
    var events = expandoStore.events;
    var handle = expandoStore.handle;

    if (!handle) {
      handle = expandoStore.handle = createEventHandler(element, events);
    }

    // http://jsperf.com/string-indexof-vs-split
    var types = type.indexOf(' ') >= 0 ? type.split(' ') : [type];
    var i = types.length;

    var addHandler = function(type, specialHandlerWrapper, noEventListener) {
      var eventFns = events[type];

      if (!eventFns) {
        eventFns = events[type] = [];
        eventFns.specialHandlerWrapper = specialHandlerWrapper;
        if (type !== '$destroy' && !noEventListener) {
          element.addEventListener(type, handle);
        }
      }

      eventFns.push(fn);
    };

    while (i--) {
      type = types[i];
      if (MOUSE_EVENT_MAP[type]) {
        addHandler(MOUSE_EVENT_MAP[type], specialMouseHandlerWrapper);
        addHandler(type, undefined, true);
      } else {
        addHandler(type);
      }
    }
  },

  off: jqLiteOff,

  one: function(element, type, fn) {
    element = jqLite(element);

    //add the listener twice so that when it is called
    //you can remove the original function and still be
    //able to call element.off(ev, fn) normally
    element.on(type, function onFn() {
      element.off(type, fn);
      element.off(type, onFn);
    });
    element.on(type, fn);
  },

  replaceWith: function(element, replaceNode) {
    var index, parent = element.parentNode;
    jqLiteDealoc(element);
    forEach(new JQLite(replaceNode), function(node) {
      if (index) {
        parent.insertBefore(node, index.nextSibling);
      } else {
        parent.replaceChild(node, element);
      }
      index = node;
    });
  },

  children: function(element) {
    var children = [];
    forEach(element.childNodes, function(element) {
      if (element.nodeType === NODE_TYPE_ELEMENT) {
        children.push(element);
      }
    });
    return children;
  },

  contents: function(element) {
    return element.contentDocument || element.childNodes || [];
  },

  append: function(element, node) {
    var nodeType = element.nodeType;
    if (nodeType !== NODE_TYPE_ELEMENT && nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT) return;

    node = new JQLite(node);

    for (var i = 0, ii = node.length; i < ii; i++) {
      var child = node[i];
      element.appendChild(child);
    }
  },

  prepend: function(element, node) {
    if (element.nodeType === NODE_TYPE_ELEMENT) {
      var index = element.firstChild;
      forEach(new JQLite(node), function(child) {
        element.insertBefore(child, index);
      });
    }
  },

  wrap: function(element, wrapNode) {
    jqLiteWrapNode(element, jqLite(wrapNode).eq(0).clone()[0]);
  },

  remove: jqLiteRemove,

  detach: function(element) {
    jqLiteRemove(element, true);
  },

  after: function(element, newElement) {
    var index = element, parent = element.parentNode;

    if (parent) {
      newElement = new JQLite(newElement);

      for (var i = 0, ii = newElement.length; i < ii; i++) {
        var node = newElement[i];
        parent.insertBefore(node, index.nextSibling);
        index = node;
      }
    }
  },

  addClass: jqLiteAddClass,
  removeClass: jqLiteRemoveClass,

  toggleClass: function(element, selector, condition) {
    if (selector) {
      forEach(selector.split(' '), function(className) {
        var classCondition = condition;
        if (isUndefined(classCondition)) {
          classCondition = !jqLiteHasClass(element, className);
        }
        (classCondition ? jqLiteAddClass : jqLiteRemoveClass)(element, className);
      });
    }
  },

  parent: function(element) {
    var parent = element.parentNode;
    return parent && parent.nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT ? parent : null;
  },

  next: function(element) {
    return element.nextElementSibling;
  },

  find: function(element, selector) {
    if (element.getElementsByTagName) {
      return element.getElementsByTagName(selector);
    } else {
      return [];
    }
  },

  clone: jqLiteClone,

  triggerHandler: function(element, event, extraParameters) {

    var dummyEvent, eventFnsCopy, handlerArgs;
    var eventName = event.type || event;
    var expandoStore = jqLiteExpandoStore(element);
    var events = expandoStore && expandoStore.events;
    var eventFns = events && events[eventName];

    if (eventFns) {
      // Create a dummy event to pass to the handlers
      dummyEvent = {
        preventDefault: function() { this.defaultPrevented = true; },
        isDefaultPrevented: function() { return this.defaultPrevented === true; },
        stopImmediatePropagation: function() { this.immediatePropagationStopped = true; },
        isImmediatePropagationStopped: function() { return this.immediatePropagationStopped === true; },
        stopPropagation: noop,
        type: eventName,
        target: element
      };

      // If a custom event was provided then extend our dummy event with it
      if (event.type) {
        dummyEvent = extend(dummyEvent, event);
      }

      // Copy event handlers in case event handlers array is modified during execution.
      eventFnsCopy = shallowCopy(eventFns);
      handlerArgs = extraParameters ? [dummyEvent].concat(extraParameters) : [dummyEvent];

      forEach(eventFnsCopy, function(fn) {
        if (!dummyEvent.isImmediatePropagationStopped()) {
          fn.apply(element, handlerArgs);
        }
      });
    }
  }
}, function(fn, name) {
  /**
   * chaining functions
   */
  JQLite.prototype[name] = function(arg1, arg2, arg3) {
    var value;

    for (var i = 0, ii = this.length; i < ii; i++) {
      if (isUndefined(value)) {
        value = fn(this[i], arg1, arg2, arg3);
        if (isDefined(value)) {
          // any function which returns a value needs to be wrapped
          value = jqLite(value);
        }
      } else {
        jqLiteAddNodes(value, fn(this[i], arg1, arg2, arg3));
      }
    }
    return isDefined(value) ? value : this;
  };
});

// bind legacy bind/unbind to on/off
JQLite.prototype.bind = JQLite.prototype.on;
JQLite.prototype.unbind = JQLite.prototype.off;


// Provider for private $$jqLite service
/** @this */
function $$jqLiteProvider() {
  this.$get = function $$jqLite() {
    return extend(JQLite, {
      hasClass: function(node, classes) {
        if (node.attr) node = node[0];
        return jqLiteHasClass(node, classes);
      },
      addClass: function(node, classes) {
        if (node.attr) node = node[0];
        return jqLiteAddClass(node, classes);
      },
      removeClass: function(node, classes) {
        if (node.attr) node = node[0];
        return jqLiteRemoveClass(node, classes);
      }
    });
  };
}

/**
 * Computes a hash of an 'obj'.
 * Hash of a:
 *  string is string
 *  number is number as string
 *  object is either result of calling $$hashKey function on the object or uniquely generated id,
 *         that is also assigned to the $$hashKey property of the object.
 *
 * @param obj
 * @returns {string} hash string such that the same input will have the same hash string.
 *         The resulting string key is in 'type:hashKey' format.
 */
function hashKey(obj, nextUidFn) {
  var key = obj && obj.$$hashKey;

  if (key) {
    if (typeof key === 'function') {
      key = obj.$$hashKey();
    }
    return key;
  }

  var objType = typeof obj;
  if (objType === 'function' || (objType === 'object' && obj !== null)) {
    key = obj.$$hashKey = objType + ':' + (nextUidFn || nextUid)();
  } else {
    key = objType + ':' + obj;
  }

  return key;
}

// A minimal ES2015 Map implementation.
// Should be bug/feature equivalent to the native implementations of supported browsers
// (for the features required in Angular).
// See https://kangax.github.io/compat-table/es6/#test-Map
var nanKey = Object.create(null);
function NgMapShim() {
  this._keys = [];
  this._values = [];
  this._lastKey = NaN;
  this._lastIndex = -1;
}
NgMapShim.prototype = {
  _idx: function(key) {
    if (key !== this._lastKey) {
      this._lastKey = key;
      this._lastIndex = this._keys.indexOf(key);
    }
    return this._lastIndex;
  },
  _transformKey: function(key) {
    return isNumberNaN(key) ? nanKey : key;
  },
  get: function(key) {
    key = this._transformKey(key);
    var idx = this._idx(key);
    if (idx !== -1) {
      return this._values[idx];
    }
  },
  has: function(key) {
    key = this._transformKey(key);
    var idx = this._idx(key);
    return idx !== -1;
  },
  set: function(key, value) {
    key = this._transformKey(key);
    var idx = this._idx(key);
    if (idx === -1) {
      idx = this._lastIndex = this._keys.length;
    }
    this._keys[idx] = key;
    this._values[idx] = value;

    // Support: IE11
    // Do not `return this` to simulate the partial IE11 implementation
  },
  delete: function(key) {
    key = this._transformKey(key);
    var idx = this._idx(key);
    if (idx === -1) {
      return false;
    }
    this._keys.splice(idx, 1);
    this._values.splice(idx, 1);
    this._lastKey = NaN;
    this._lastIndex = -1;
    return true;
  }
};

// For now, always use `NgMapShim`, even if `window.Map` is available. Some native implementations
// are still buggy (often in subtle ways) and can cause hard-to-debug failures. When native `Map`
// implementations get more stable, we can reconsider switching to `window.Map` (when available).
var NgMap = NgMapShim;

var $$MapProvider = [/** @this */function() {
  this.$get = [function() {
    return NgMap;
  }];
}];

/**
 * @ngdoc function
 * @module ng
 * @name angular.injector
 * @kind function
 *
 * @description
 * Creates an injector object that can be used for retrieving services as well as for
 * dependency injection (see {@link guide/di dependency injection}).
 *
 * @param {Array.<string|Function>} modules A list of module functions or their aliases. See
 *     {@link angular.module}. The `ng` module must be explicitly added.
 * @param {boolean=} [strictDi=false] Whether the injector should be in strict mode, which
 *     disallows argument name annotation inference.
 * @returns {injector} Injector object. See {@link auto.$injector $injector}.
 *
 * @example
 * Typical usage
 * ```js
 *   // create an injector
 *   var $injector = angular.injector(['ng']);
 *
 *   // use the injector to kick off your application
 *   // use the type inference to auto inject arguments, or use implicit injection
 *   $injector.invoke(function($rootScope, $compile, $document) {
 *     $compile($document)($rootScope);
 *     $rootScope.$digest();
 *   });
 * ```
 *
 * Sometimes you want to get access to the injector of a currently running AngularJS app
 * from outside AngularJS. Perhaps, you want to inject and compile some markup after the
 * application has been bootstrapped. You can do this using the extra `injector()` added
 * to JQuery/jqLite elements. See {@link angular.element}.
 *
 * *This is fairly rare but could be the case if a third party library is injecting the
 * markup.*
 *
 * In the following example a new block of HTML containing a `ng-controller`
 * directive is added to the end of the document body by JQuery. We then compile and link
 * it into the current AngularJS scope.
 *
 * ```js
 * var $div = $('<div ng-controller="MyCtrl">{{content.label}}</div>');
 * $(document.body).append($div);
 *
 * angular.element(document).injector().invoke(function($compile) {
 *   var scope = angular.element($div).scope();
 *   $compile($div)(scope);
 * });
 * ```
 */


/**
 * @ngdoc module
 * @name auto
 * @installation
 * @description
 *
 * Implicit module which gets automatically added to each {@link auto.$injector $injector}.
 */

var ARROW_ARG = /^([^(]+?)=>/;
var FN_ARGS = /^[^(]*\(\s*([^)]*)\)/m;
var FN_ARG_SPLIT = /,/;
var FN_ARG = /^\s*(_?)(\S+?)\1\s*$/;
var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
var $injectorMinErr = minErr('$injector');

function stringifyFn(fn) {
  return Function.prototype.toString.call(fn);
}

function extractArgs(fn) {
  var fnText = stringifyFn(fn).replace(STRIP_COMMENTS, ''),
      args = fnText.match(ARROW_ARG) || fnText.match(FN_ARGS);
  return args;
}

function anonFn(fn) {
  // For anonymous functions, showing at the very least the function signature can help in
  // debugging.
  var args = extractArgs(fn);
  if (args) {
    return 'function(' + (args[1] || '').replace(/[\s\r\n]+/, ' ') + ')';
  }
  return 'fn';
}

function annotate(fn, strictDi, name) {
  var $inject,
      argDecl,
      last;

  if (typeof fn === 'function') {
    if (!($inject = fn.$inject)) {
      $inject = [];
      if (fn.length) {
        if (strictDi) {
          if (!isString(name) || !name) {
            name = fn.name || anonFn(fn);
          }
          throw $injectorMinErr('strictdi',
            '{0} is not using explicit annotation and cannot be invoked in strict mode', name);
        }
        argDecl = extractArgs(fn);
        forEach(argDecl[1].split(FN_ARG_SPLIT), function(arg) {
          arg.replace(FN_ARG, function(all, underscore, name) {
            $inject.push(name);
          });
        });
      }
      fn.$inject = $inject;
    }
  } else if (isArray(fn)) {
    last = fn.length - 1;
    assertArgFn(fn[last], 'fn');
    $inject = fn.slice(0, last);
  } else {
    assertArgFn(fn, 'fn', true);
  }
  return $inject;
}

///////////////////////////////////////

/**
 * @ngdoc service
 * @name $injector
 *
 * @description
 *
 * `$injector` is used to retrieve object instances as defined by
 * {@link auto.$provide provider}, instantiate types, invoke methods,
 * and load modules.
 *
 * The following always holds true:
 *
 * ```js
 *   var $injector = angular.injector();
 *   expect($injector.get('$injector')).toBe($injector);
 *   expect($injector.invoke(function($injector) {
 *     return $injector;
 *   })).toBe($injector);
 * ```
 *
 * ## Injection Function Annotation
 *
 * JavaScript does not have annotations, and annotations are needed for dependency injection. The
 * following are all valid ways of annotating function with injection arguments and are equivalent.
 *
 * ```js
 *   // inferred (only works if code not minified/obfuscated)
 *   $injector.invoke(function(serviceA){});
 *
 *   // annotated
 *   function explicit(serviceA) {};
 *   explicit.$inject = ['serviceA'];
 *   $injector.invoke(explicit);
 *
 *   // inline
 *   $injector.invoke(['serviceA', function(serviceA){}]);
 * ```
 *
 * ### Inference
 *
 * In JavaScript calling `toString()` on a function returns the function definition. The definition
 * can then be parsed and the function arguments can be extracted. This method of discovering
 * annotations is disallowed when the injector is in strict mode.
 * *NOTE:* This does not work with minification, and obfuscation tools since these tools change the
 * argument names.
 *
 * ### `$inject` Annotation
 * By adding an `$inject` property onto a function the injection parameters can be specified.
 *
 * ### Inline
 * As an array of injection names, where the last item in the array is the function to call.
 */

/**
 * @ngdoc property
 * @name $injector#modules
 * @type {Object}
 * @description
 * A hash containing all the modules that have been loaded into the
 * $injector.
 *
 * You can use this property to find out information about a module via the
 * {@link angular.Module#info `myModule.info(...)`} method.
 *
 * For example:
 *
 * ```
 * var info = $injector.modules['ngAnimate'].info();
 * ```
 *
 * **Do not use this property to attempt to modify the modules after the application
 * has been bootstrapped.**
 */


/**
 * @ngdoc method
 * @name $injector#get
 *
 * @description
 * Return an instance of the service.
 *
 * @param {string} name The name of the instance to retrieve.
 * @param {string=} caller An optional string to provide the origin of the function call for error messages.
 * @return {*} The instance.
 */

/**
 * @ngdoc method
 * @name $injector#invoke
 *
 * @description
 * Invoke the method and supply the method arguments from the `$injector`.
 *
 * @param {Function|Array.<string|Function>} fn The injectable function to invoke. Function parameters are
 *   injected according to the {@link guide/di $inject Annotation} rules.
 * @param {Object=} self The `this` for the invoked method.
 * @param {Object=} locals Optional object. If preset then any argument names are read from this
 *                         object first, before the `$injector` is consulted.
 * @returns {*} the value returned by the invoked `fn` function.
 */

/**
 * @ngdoc method
 * @name $injector#has
 *
 * @description
 * Allows the user to query if the particular service exists.
 *
 * @param {string} name Name of the service to query.
 * @returns {boolean} `true` if injector has given service.
 */

/**
 * @ngdoc method
 * @name $injector#instantiate
 * @description
 * Create a new instance of JS type. The method takes a constructor function, invokes the new
 * operator, and supplies all of the arguments to the constructor function as specified by the
 * constructor annotation.
 *
 * @param {Function} Type Annotated constructor function.
 * @param {Object=} locals Optional object. If preset then any argument names are read from this
 * object first, before the `$injector` is consulted.
 * @returns {Object} new instance of `Type`.
 */

/**
 * @ngdoc method
 * @name $injector#annotate
 *
 * @description
 * Returns an array of service names which the function is requesting for injection. This API is
 * used by the injector to determine which services need to be injected into the function when the
 * function is invoked. There are three ways in which the function can be annotated with the needed
 * dependencies.
 *
 * #### Argument names
 *
 * The simplest form is to extract the dependencies from the arguments of the function. This is done
 * by converting the function into a string using `toString()` method and extracting the argument
 * names.
 * ```js
 *   // Given
 *   function MyController($scope, $route) {
 *     // ...
 *   }
 *
 *   // Then
 *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
 * ```
 *
 * You can disallow this method by using strict injection mode.
 *
 * This method does not work with code minification / obfuscation. For this reason the following
 * annotation strategies are supported.
 *
 * #### The `$inject` property
 *
 * If a function has an `$inject` property and its value is an array of strings, then the strings
 * represent names of services to be injected into the function.
 * ```js
 *   // Given
 *   var MyController = function(obfuscatedScope, obfuscatedRoute) {
 *     // ...
 *   }
 *   // Define function dependencies
 *   MyController['$inject'] = ['$scope', '$route'];
 *
 *   // Then
 *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
 * ```
 *
 * #### The array notation
 *
 * It is often desirable to inline Injected functions and that's when setting the `$inject` property
 * is very inconvenient. In these situations using the array notation to specify the dependencies in
 * a way that survives minification is a better choice:
 *
 * ```js
 *   // We wish to write this (not minification / obfuscation safe)
 *   injector.invoke(function($compile, $rootScope) {
 *     // ...
 *   });
 *
 *   // We are forced to write break inlining
 *   var tmpFn = function(obfuscatedCompile, obfuscatedRootScope) {
 *     // ...
 *   };
 *   tmpFn.$inject = ['$compile', '$rootScope'];
 *   injector.invoke(tmpFn);
 *
 *   // To better support inline function the inline annotation is supported
 *   injector.invoke(['$compile', '$rootScope', function(obfCompile, obfRootScope) {
 *     // ...
 *   }]);
 *
 *   // Therefore
 *   expect(injector.annotate(
 *      ['$compile', '$rootScope', function(obfus_$compile, obfus_$rootScope) {}])
 *    ).toEqual(['$compile', '$rootScope']);
 * ```
 *
 * @param {Function|Array.<string|Function>} fn Function for which dependent service names need to
 * be retrieved as described above.
 *
 * @param {boolean=} [strictDi=false] Disallow argument name annotation inference.
 *
 * @returns {Array.<string>} The names of the services which the function requires.
 */
/**
 * @ngdoc method
 * @name $injector#loadNewModules
 *
 * @description
 *
 * **This is a dangerous API, which you use at your own risk!**
 *
 * Add the specified modules to the current injector.
 *
 * This method will add each of the injectables to the injector and execute all of the config and run
 * blocks for each module passed to the method.
 *
 * If a module has already been loaded into the injector then it will not be loaded again.
 *
 * * The application developer is responsible for loading the code containing the modules; and for
 * ensuring that lazy scripts are not downloaded and executed more often that desired.
 * * Previously compiled HTML will not be affected by newly loaded directives, filters and components.
 * * Modules cannot be unloaded.
 *
 * You can use {@link $injector#modules `$injector.modules`} to check whether a module has been loaded
 * into the injector, which may indicate whether the script has been executed already.
 *
 * @example
 * Here is an example of loading a bundle of modules, with a utility method called `getScript`:
 *
 * ```javascript
 * app.factory('loadModule', function($injector) {
 *   return function loadModule(moduleName, bundleUrl) {
 *     return getScript(bundleUrl).then(function() { $injector.loadNewModules([moduleName]); });
 *   };
 * })
 * ```
 *
 * @param {Array<String|Function|Array>=} mods an array of modules to load into the application.
 *     Each item in the array should be the name of a predefined module or a (DI annotated)
 *     function that will be invoked by the injector as a `config` block.
 *     See: {@link angular.module modules}
 */


/**
 * @ngdoc service
 * @name $provide
 *
 * @description
 *
 * The {@link auto.$provide $provide} service has a number of methods for registering components
 * with the {@link auto.$injector $injector}. Many of these functions are also exposed on
 * {@link angular.Module}.
 *
 * An AngularJS **service** is a singleton object created by a **service factory**.  These **service
 * factories** are functions which, in turn, are created by a **service provider**.
 * The **service providers** are constructor functions. When instantiated they must contain a
 * property called `$get`, which holds the **service factory** function.
 *
 * When you request a service, the {@link auto.$injector $injector} is responsible for finding the
 * correct **service provider**, instantiating it and then calling its `$get` **service factory**
 * function to get the instance of the **service**.
 *
 * Often services have no configuration options and there is no need to add methods to the service
 * provider.  The provider will be no more than a constructor function with a `$get` property. For
 * these cases the {@link auto.$provide $provide} service has additional helper methods to register
 * services without specifying a provider.
 *
 * * {@link auto.$provide#provider provider(name, provider)} - registers a **service provider** with the
 *     {@link auto.$injector $injector}
 * * {@link auto.$provide#constant constant(name, obj)} - registers a value/object that can be accessed by
 *     providers and services.
 * * {@link auto.$provide#value value(name, obj)} - registers a value/object that can only be accessed by
 *     services, not providers.
 * * {@link auto.$provide#factory factory(name, fn)} - registers a service **factory function**
 *     that will be wrapped in a **service provider** object, whose `$get` property will contain the
 *     given factory function.
 * * {@link auto.$provide#service service(name, Fn)} - registers a **constructor function**
 *     that will be wrapped in a **service provider** object, whose `$get` property will instantiate
 *      a new object using the given constructor function.
 * * {@link auto.$provide#decorator decorator(name, decorFn)} - registers a **decorator function** that
 *      will be able to modify or replace the implementation of another service.
 *
 * See the individual methods for more information and examples.
 */

/**
 * @ngdoc method
 * @name $provide#provider
 * @description
 *
 * Register a **provider function** with the {@link auto.$injector $injector}. Provider functions
 * are constructor functions, whose instances are responsible for "providing" a factory for a
 * service.
 *
 * Service provider names start with the name of the service they provide followed by `Provider`.
 * For example, the {@link ng.$log $log} service has a provider called
 * {@link ng.$logProvider $logProvider}.
 *
 * Service provider objects can have additional methods which allow configuration of the provider
 * and its service. Importantly, you can configure what kind of service is created by the `$get`
 * method, or how that service will act. For example, the {@link ng.$logProvider $logProvider} has a
 * method {@link ng.$logProvider#debugEnabled debugEnabled}
 * which lets you specify whether the {@link ng.$log $log} service will log debug messages to the
 * console or not.
 *
 * It is possible to inject other providers into the provider function,
 * but the injected provider must have been defined before the one that requires it.
 *
 * @param {string} name The name of the instance. NOTE: the provider will be available under `name +
                        'Provider'` key.
 * @param {(Object|function())} provider If the provider is:
 *
 *   - `Object`: then it should have a `$get` method. The `$get` method will be invoked using
 *     {@link auto.$injector#invoke $injector.invoke()} when an instance needs to be created.
 *   - `Constructor`: a new instance of the provider will be created using
 *     {@link auto.$injector#instantiate $injector.instantiate()}, then treated as `object`.
 *
 * @returns {Object} registered provider instance

 * @example
 *
 * The following example shows how to create a simple event tracking service and register it using
 * {@link auto.$provide#provider $provide.provider()}.
 *
 * ```js
 *  // Define the eventTracker provider
 *  function EventTrackerProvider() {
 *    var trackingUrl = '/track';
 *
 *    // A provider method for configuring where the tracked events should been saved
 *    this.setTrackingUrl = function(url) {
 *      trackingUrl = url;
 *    };
 *
 *    // The service factory function
 *    this.$get = ['$http', function($http) {
 *      var trackedEvents = {};
 *      return {
 *        // Call this to track an event
 *        event: function(event) {
 *          var count = trackedEvents[event] || 0;
 *          count += 1;
 *          trackedEvents[event] = count;
 *          return count;
 *        },
 *        // Call this to save the tracked events to the trackingUrl
 *        save: function() {
 *          $http.post(trackingUrl, trackedEvents);
 *        }
 *      };
 *    }];
 *  }
 *
 *  describe('eventTracker', function() {
 *    var postSpy;
 *
 *    beforeEach(module(function($provide) {
 *      // Register the eventTracker provider
 *      $provide.provider('eventTracker', EventTrackerProvider);
 *    }));
 *
 *    beforeEach(module(function(eventTrackerProvider) {
 *      // Configure eventTracker provider
 *      eventTrackerProvider.setTrackingUrl('/custom-track');
 *    }));
 *
 *    it('tracks events', inject(function(eventTracker) {
 *      expect(eventTracker.event('login')).toEqual(1);
 *      expect(eventTracker.event('login')).toEqual(2);
 *    }));
 *
 *    it('saves to the tracking url', inject(function(eventTracker, $http) {
 *      postSpy = spyOn($http, 'post');
 *      eventTracker.event('login');
 *      eventTracker.save();
 *      expect(postSpy).toHaveBeenCalled();
 *      expect(postSpy.mostRecentCall.args[0]).not.toEqual('/track');
 *      expect(postSpy.mostRecentCall.args[0]).toEqual('/custom-track');
 *      expect(postSpy.mostRecentCall.args[1]).toEqual({ 'login': 1 });
 *    }));
 *  });
 * ```
 */

/**
 * @ngdoc method
 * @name $provide#factory
 * @description
 *
 * Register a **service factory**, which will be called to return the service instance.
 * This is short for registering a service where its provider consists of only a `$get` property,
 * which is the given service factory function.
 * You should use {@link auto.$provide#factory $provide.factory(getFn)} if you do not need to
 * configure your service in a provider.
 *
 * @param {string} name The name of the instance.
 * @param {Function|Array.<string|Function>} $getFn The injectable $getFn for the instance creation.
 *                      Internally this is a short hand for `$provide.provider(name, {$get: $getFn})`.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here is an example of registering a service
 * ```js
 *   $provide.factory('ping', ['$http', function($http) {
 *     return function ping() {
 *       return $http.send('/ping');
 *     };
 *   }]);
 * ```
 * You would then inject and use this service like this:
 * ```js
 *   someModule.controller('Ctrl', ['ping', function(ping) {
 *     ping();
 *   }]);
 * ```
 */


/**
 * @ngdoc method
 * @name $provide#service
 * @description
 *
 * Register a **service constructor**, which will be invoked with `new` to create the service
 * instance.
 * This is short for registering a service where its provider's `$get` property is a factory
 * function that returns an instance instantiated by the injector from the service constructor
 * function.
 *
 * Internally it looks a bit like this:
 *
 * ```
 * {
 *   $get: function() {
 *     return $injector.instantiate(constructor);
 *   }
 * }
 * ```
 *
 *
 * You should use {@link auto.$provide#service $provide.service(class)} if you define your service
 * as a type/class.
 *
 * @param {string} name The name of the instance.
 * @param {Function|Array.<string|Function>} constructor An injectable class (constructor function)
 *     that will be instantiated.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here is an example of registering a service using
 * {@link auto.$provide#service $provide.service(class)}.
 * ```js
 *   var Ping = function($http) {
 *     this.$http = $http;
 *   };
 *
 *   Ping.$inject = ['$http'];
 *
 *   Ping.prototype.send = function() {
 *     return this.$http.get('/ping');
 *   };
 *   $provide.service('ping', Ping);
 * ```
 * You would then inject and use this service like this:
 * ```js
 *   someModule.controller('Ctrl', ['ping', function(ping) {
 *     ping.send();
 *   }]);
 * ```
 */


/**
 * @ngdoc method
 * @name $provide#value
 * @description
 *
 * Register a **value service** with the {@link auto.$injector $injector}, such as a string, a
 * number, an array, an object or a function. This is short for registering a service where its
 * provider's `$get` property is a factory function that takes no arguments and returns the **value
 * service**. That also means it is not possible to inject other services into a value service.
 *
 * Value services are similar to constant services, except that they cannot be injected into a
 * module configuration function (see {@link angular.Module#config}) but they can be overridden by
 * an AngularJS {@link auto.$provide#decorator decorator}.
 *
 * @param {string} name The name of the instance.
 * @param {*} value The value.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here are some examples of creating value services.
 * ```js
 *   $provide.value('ADMIN_USER', 'admin');
 *
 *   $provide.value('RoleLookup', { admin: 0, writer: 1, reader: 2 });
 *
 *   $provide.value('halfOf', function(value) {
 *     return value / 2;
 *   });
 * ```
 */


/**
 * @ngdoc method
 * @name $provide#constant
 * @description
 *
 * Register a **constant service** with the {@link auto.$injector $injector}, such as a string,
 * a number, an array, an object or a function. Like the {@link auto.$provide#value value}, it is not
 * possible to inject other services into a constant.
 *
 * But unlike {@link auto.$provide#value value}, a constant can be
 * injected into a module configuration function (see {@link angular.Module#config}) and it cannot
 * be overridden by an AngularJS {@link auto.$provide#decorator decorator}.
 *
 * @param {string} name The name of the constant.
 * @param {*} value The constant value.
 * @returns {Object} registered instance
 *
 * @example
 * Here a some examples of creating constants:
 * ```js
 *   $provide.constant('SHARD_HEIGHT', 306);
 *
 *   $provide.constant('MY_COLOURS', ['red', 'blue', 'grey']);
 *
 *   $provide.constant('double', function(value) {
 *     return value * 2;
 *   });
 * ```
 */


/**
 * @ngdoc method
 * @name $provide#decorator
 * @description
 *
 * Register a **decorator function** with the {@link auto.$injector $injector}. A decorator function
 * intercepts the creation of a service, allowing it to override or modify the behavior of the
 * service. The return value of the decorator function may be the original service, or a new service
 * that replaces (or wraps and delegates to) the original service.
 *
 * You can find out more about using decorators in the {@link guide/decorators} guide.
 *
 * @param {string} name The name of the service to decorate.
 * @param {Function|Array.<string|Function>} decorator This function will be invoked when the service needs to be
 *    provided and should return the decorated service instance. The function is called using
 *    the {@link auto.$injector#invoke injector.invoke} method and is therefore fully injectable.
 *    Local injection arguments:
 *
 *    * `$delegate` - The original service instance, which can be replaced, monkey patched, configured,
 *      decorated or delegated to.
 *
 * @example
 * Here we decorate the {@link ng.$log $log} service to convert warnings to errors by intercepting
 * calls to {@link ng.$log#error $log.warn()}.
 * ```js
 *   $provide.decorator('$log', ['$delegate', function($delegate) {
 *     $delegate.warn = $delegate.error;
 *     return $delegate;
 *   }]);
 * ```
 */


function createInjector(modulesToLoad, strictDi) {
  strictDi = (strictDi === true);
  var INSTANTIATING = {},
      providerSuffix = 'Provider',
      path = [],
      loadedModules = new NgMap(),
      providerCache = {
        $provide: {
            provider: supportObject(provider),
            factory: supportObject(factory),
            service: supportObject(service),
            value: supportObject(value),
            constant: supportObject(constant),
            decorator: decorator
          }
      },
      providerInjector = (providerCache.$injector =
          createInternalInjector(providerCache, function(serviceName, caller) {
            if (angular.isString(caller)) {
              path.push(caller);
            }
            throw $injectorMinErr('unpr', 'Unknown provider: {0}', path.join(' <- '));
          })),
      instanceCache = {},
      protoInstanceInjector =
          createInternalInjector(instanceCache, function(serviceName, caller) {
            var provider = providerInjector.get(serviceName + providerSuffix, caller);
            return instanceInjector.invoke(
                provider.$get, provider, undefined, serviceName);
          }),
      instanceInjector = protoInstanceInjector;

  providerCache['$injector' + providerSuffix] = { $get: valueFn(protoInstanceInjector) };
  instanceInjector.modules = providerInjector.modules = createMap();
  var runBlocks = loadModules(modulesToLoad);
  instanceInjector = protoInstanceInjector.get('$injector');
  instanceInjector.strictDi = strictDi;
  forEach(runBlocks, function(fn) { if (fn) instanceInjector.invoke(fn); });

  instanceInjector.loadNewModules = function(mods) {
    forEach(loadModules(mods), function(fn) { if (fn) instanceInjector.invoke(fn); });
  };


  return instanceInjector;

  ////////////////////////////////////
  // $provider
  ////////////////////////////////////

  function supportObject(delegate) {
    return function(key, value) {
      if (isObject(key)) {
        forEach(key, reverseParams(delegate));
      } else {
        return delegate(key, value);
      }
    };
  }

  function provider(name, provider_) {
    assertNotHasOwnProperty(name, 'service');
    if (isFunction(provider_) || isArray(provider_)) {
      provider_ = providerInjector.instantiate(provider_);
    }
    if (!provider_.$get) {
      throw $injectorMinErr('pget', 'Provider \'{0}\' must define $get factory method.', name);
    }
    return (providerCache[name + providerSuffix] = provider_);
  }

  function enforceReturnValue(name, factory) {
    return /** @this */ function enforcedReturnValue() {
      var result = instanceInjector.invoke(factory, this);
      if (isUndefined(result)) {
        throw $injectorMinErr('undef', 'Provider \'{0}\' must return a value from $get factory method.', name);
      }
      return result;
    };
  }

  function factory(name, factoryFn, enforce) {
    return provider(name, {
      $get: enforce !== false ? enforceReturnValue(name, factoryFn) : factoryFn
    });
  }

  function service(name, constructor) {
    return factory(name, ['$injector', function($injector) {
      return $injector.instantiate(constructor);
    }]);
  }

  function value(name, val) { return factory(name, valueFn(val), false); }

  function constant(name, value) {
    assertNotHasOwnProperty(name, 'constant');
    providerCache[name] = value;
    instanceCache[name] = value;
  }

  function decorator(serviceName, decorFn) {
    var origProvider = providerInjector.get(serviceName + providerSuffix),
        orig$get = origProvider.$get;

    origProvider.$get = function() {
      var origInstance = instanceInjector.invoke(orig$get, origProvider);
      return instanceInjector.invoke(decorFn, null, {$delegate: origInstance});
    };
  }

  ////////////////////////////////////
  // Module Loading
  ////////////////////////////////////
  function loadModules(modulesToLoad) {
    assertArg(isUndefined(modulesToLoad) || isArray(modulesToLoad), 'modulesToLoad', 'not an array');
    var runBlocks = [], moduleFn;
    forEach(modulesToLoad, function(module) {
      if (loadedModules.get(module)) return;
      loadedModules.set(module, true);

      function runInvokeQueue(queue) {
        var i, ii;
        for (i = 0, ii = queue.length; i < ii; i++) {
          var invokeArgs = queue[i],
              provider = providerInjector.get(invokeArgs[0]);

          provider[invokeArgs[1]].apply(provider, invokeArgs[2]);
        }
      }

      try {
        if (isString(module)) {
          moduleFn = angularModule(module);
          instanceInjector.modules[module] = moduleFn;
          runBlocks = runBlocks.concat(loadModules(moduleFn.requires)).concat(moduleFn._runBlocks);
          runInvokeQueue(moduleFn._invokeQueue);
          runInvokeQueue(moduleFn._configBlocks);
        } else if (isFunction(module)) {
            runBlocks.push(providerInjector.invoke(module));
        } else if (isArray(module)) {
            runBlocks.push(providerInjector.invoke(module));
        } else {
          assertArgFn(module, 'module');
        }
      } catch (e) {
        if (isArray(module)) {
          module = module[module.length - 1];
        }
        if (e.message && e.stack && e.stack.indexOf(e.message) === -1) {
          // Safari & FF's stack traces don't contain error.message content
          // unlike those of Chrome and IE
          // So if stack doesn't contain message, we create a new string that contains both.
          // Since error.stack is read-only in Safari, I'm overriding e and not e.stack here.
          // eslint-disable-next-line no-ex-assign
          e = e.message + '\n' + e.stack;
        }
        throw $injectorMinErr('modulerr', 'Failed to instantiate module {0} due to:\n{1}',
                  module, e.stack || e.message || e);
      }
    });
    return runBlocks;
  }

  ////////////////////////////////////
  // internal Injector
  ////////////////////////////////////

  function createInternalInjector(cache, factory) {

    function getService(serviceName, caller) {
      if (cache.hasOwnProperty(serviceName)) {
        if (cache[serviceName] === INSTANTIATING) {
          throw $injectorMinErr('cdep', 'Circular dependency found: {0}',
                    serviceName + ' <- ' + path.join(' <- '));
        }
        return cache[serviceName];
      } else {
        try {
          path.unshift(serviceName);
          cache[serviceName] = INSTANTIATING;
          cache[serviceName] = factory(serviceName, caller);
          return cache[serviceName];
        } catch (err) {
          if (cache[serviceName] === INSTANTIATING) {
            delete cache[serviceName];
          }
          throw err;
        } finally {
          path.shift();
        }
      }
    }


    function injectionArgs(fn, locals, serviceName) {
      var args = [],
          $inject = createInjector.$$annotate(fn, strictDi, serviceName);

      for (var i = 0, length = $inject.length; i < length; i++) {
        var key = $inject[i];
        if (typeof key !== 'string') {
          throw $injectorMinErr('itkn',
                  'Incorrect injection token! Expected service name as string, got {0}', key);
        }
        args.push(locals && locals.hasOwnProperty(key) ? locals[key] :
                                                         getService(key, serviceName));
      }
      return args;
    }

    function isClass(func) {
      // Support: IE 9-11 only
      // IE 9-11 do not support classes and IE9 leaks with the code below.
      if (msie || typeof func !== 'function') {
        return false;
      }
      var result = func.$$ngIsClass;
      if (!isBoolean(result)) {
        result = func.$$ngIsClass = /^class\b/.test(stringifyFn(func));
      }
      return result;
    }

    function invoke(fn, self, locals, serviceName) {
      if (typeof locals === 'string') {
        serviceName = locals;
        locals = null;
      }

      var args = injectionArgs(fn, locals, serviceName);
      if (isArray(fn)) {
        fn = fn[fn.length - 1];
      }

      if (!isClass(fn)) {
        // http://jsperf.com/angularjs-invoke-apply-vs-switch
        // #5388
        return fn.apply(self, args);
      } else {
        args.unshift(null);
        return new (Function.prototype.bind.apply(fn, args))();
      }
    }


    function instantiate(Type, locals, serviceName) {
      // Check if Type is annotated and use just the given function at n-1 as parameter
      // e.g. someModule.factory('greeter', ['$window', function(renamed$window) {}]);
      var ctor = (isArray(Type) ? Type[Type.length - 1] : Type);
      var args = injectionArgs(Type, locals, serviceName);
      // Empty object at position 0 is ignored for invocation with `new`, but required.
      args.unshift(null);
      return new (Function.prototype.bind.apply(ctor, args))();
    }


    return {
      invoke: invoke,
      instantiate: instantiate,
      get: getService,
      annotate: createInjector.$$annotate,
      has: function(name) {
        return providerCache.hasOwnProperty(name + providerSuffix) || cache.hasOwnProperty(name);
      }
    };
  }
}

createInjector.$$annotate = annotate;

/**
 * @ngdoc provider
 * @name $anchorScrollProvider
 * @this
 *
 * @description
 * Use `$anchorScrollProvider` to disable automatic scrolling whenever
 * {@link ng.$location#hash $location.hash()} changes.
 */
function $AnchorScrollProvider() {

  var autoScrollingEnabled = true;

  /**
   * @ngdoc method
   * @name $anchorScrollProvider#disableAutoScrolling
   *
   * @description
   * By default, {@link ng.$anchorScroll $anchorScroll()} will automatically detect changes to
   * {@link ng.$location#hash $location.hash()} and scroll to the element matching the new hash.<br />
   * Use this method to disable automatic scrolling.
   *
   * If automatic scrolling is disabled, one must explicitly call
   * {@link ng.$anchorScroll $anchorScroll()} in order to scroll to the element related to the
   * current hash.
   */
  this.disableAutoScrolling = function() {
    autoScrollingEnabled = false;
  };

  /**
   * @ngdoc service
   * @name $anchorScroll
   * @kind function
   * @requires $window
   * @requires $location
   * @requires $rootScope
   *
   * @description
   * When called, it scrolls to the element related to the specified `hash` or (if omitted) to the
   * current value of {@link ng.$location#hash $location.hash()}, according to the rules specified
   * in the
   * [HTML5 spec](http://www.w3.org/html/wg/drafts/html/master/browsers.html#an-indicated-part-of-the-document).
   *
   * It also watches the {@link ng.$location#hash $location.hash()} and automatically scrolls to
   * match any anchor whenever it changes. This can be disabled by calling
   * {@link ng.$anchorScrollProvider#disableAutoScrolling $anchorScrollProvider.disableAutoScrolling()}.
   *
   * Additionally, you can use its {@link ng.$anchorScroll#yOffset yOffset} property to specify a
   * vertical scroll-offset (either fixed or dynamic).
   *
   * @param {string=} hash The hash specifying the element to scroll to. If omitted, the value of
   *                       {@link ng.$location#hash $location.hash()} will be used.
   *
   * @property {(number|function|jqLite)} yOffset
   * If set, specifies a vertical scroll-offset. This is often useful when there are fixed
   * positioned elements at the top of the page, such as navbars, headers etc.
   *
   * `yOffset` can be specified in various ways:
   * - **number**: A fixed number of pixels to be used as offset.<br /><br />
   * - **function**: A getter function called everytime `$anchorScroll()` is executed. Must return
   *   a number representing the offset (in pixels).<br /><br />
   * - **jqLite**: A jqLite/jQuery element to be used for specifying the offset. The distance from
   *   the top of the page to the element's bottom will be used as offset.<br />
   *   **Note**: The element will be taken into account only as long as its `position` is set to
   *   `fixed`. This option is useful, when dealing with responsive navbars/headers that adjust
   *   their height and/or positioning according to the viewport's size.
   *
   * <br />
   * <div class="alert alert-warning">
   * In order for `yOffset` to work properly, scrolling should take place on the document's root and
   * not some child element.
   * </div>
   *
   * @example
     <example module="anchorScrollExample" name="anchor-scroll">
       <file name="index.html">
         <div id="scrollArea" ng-controller="ScrollController">
           <a ng-click="gotoBottom()">Go to bottom</a>
           <a id="bottom"></a> You're at the bottom!
         </div>
       </file>
       <file name="script.js">
         angular.module('anchorScrollExample', [])
           .controller('ScrollController', ['$scope', '$location', '$anchorScroll',
             function($scope, $location, $anchorScroll) {
               $scope.gotoBottom = function() {
                 // set the location.hash to the id of
                 // the element you wish to scroll to.
                 $location.hash('bottom');

                 // call $anchorScroll()
                 $anchorScroll();
               };
             }]);
       </file>
       <file name="style.css">
         #scrollArea {
           height: 280px;
           overflow: auto;
         }

         #bottom {
           display: block;
           margin-top: 2000px;
         }
       </file>
     </example>
   *
   * <hr />
   * The example below illustrates the use of a vertical scroll-offset (specified as a fixed value).
   * See {@link ng.$anchorScroll#yOffset $anchorScroll.yOffset} for more details.
   *
   * @example
     <example module="anchorScrollOffsetExample" name="anchor-scroll-offset">
       <file name="index.html">
         <div class="fixed-header" ng-controller="headerCtrl">
           <a href="" ng-click="gotoAnchor(x)" ng-repeat="x in [1,2,3,4,5]">
             Go to anchor {{x}}
           </a>
         </div>
         <div id="anchor{{x}}" class="anchor" ng-repeat="x in [1,2,3,4,5]">
           Anchor {{x}} of 5
         </div>
       </file>
       <file name="script.js">
         angular.module('anchorScrollOffsetExample', [])
           .run(['$anchorScroll', function($anchorScroll) {
             $anchorScroll.yOffset = 50;   // always scroll by 50 extra pixels
           }])
           .controller('headerCtrl', ['$anchorScroll', '$location', '$scope',
             function($anchorScroll, $location, $scope) {
               $scope.gotoAnchor = function(x) {
                 var newHash = 'anchor' + x;
                 if ($location.hash() !== newHash) {
                   // set the $location.hash to `newHash` and
                   // $anchorScroll will automatically scroll to it
                   $location.hash('anchor' + x);
                 } else {
                   // call $anchorScroll() explicitly,
                   // since $location.hash hasn't changed
                   $anchorScroll();
                 }
               };
             }
           ]);
       </file>
       <file name="style.css">
         body {
           padding-top: 50px;
         }

         .anchor {
           border: 2px dashed DarkOrchid;
           padding: 10px 10px 200px 10px;
         }

         .fixed-header {
           background-color: rgba(0, 0, 0, 0.2);
           height: 50px;
           position: fixed;
           top: 0; left: 0; right: 0;
         }

         .fixed-header > a {
           display: inline-block;
           margin: 5px 15px;
         }
       </file>
     </example>
   */
  this.$get = ['$window', '$location', '$rootScope', function($window, $location, $rootScope) {
    var document = $window.document;

    // Helper function to get first anchor from a NodeList
    // (using `Array#some()` instead of `angular#forEach()` since it's more performant
    //  and working in all supported browsers.)
    function getFirstAnchor(list) {
      var result = null;
      Array.prototype.some.call(list, function(element) {
        if (nodeName_(element) === 'a') {
          result = element;
          return true;
        }
      });
      return result;
    }

    function getYOffset() {

      var offset = scroll.yOffset;

      if (isFunction(offset)) {
        offset = offset();
      } else if (isElement(offset)) {
        var elem = offset[0];
        var style = $window.getComputedStyle(elem);
        if (style.position !== 'fixed') {
          offset = 0;
        } else {
          offset = elem.getBoundingClientRect().bottom;
        }
      } else if (!isNumber(offset)) {
        offset = 0;
      }

      return offset;
    }

    function scrollTo(elem) {
      if (elem) {
        elem.scrollIntoView();

        var offset = getYOffset();

        if (offset) {
          // `offset` is the number of pixels we should scroll UP in order to align `elem` properly.
          // This is true ONLY if the call to `elem.scrollIntoView()` initially aligns `elem` at the
          // top of the viewport.
          //
          // IF the number of pixels from the top of `elem` to the end of the page's content is less
          // than the height of the viewport, then `elem.scrollIntoView()` will align the `elem` some
          // way down the page.
          //
          // This is often the case for elements near the bottom of the page.
          //
          // In such cases we do not need to scroll the whole `offset` up, just the difference between
          // the top of the element and the offset, which is enough to align the top of `elem` at the
          // desired position.
          var elemTop = elem.getBoundingClientRect().top;
          $window.scrollBy(0, elemTop - offset);
        }
      } else {
        $window.scrollTo(0, 0);
      }
    }

    function scroll(hash) {
      // Allow numeric hashes
      hash = isString(hash) ? hash : isNumber(hash) ? hash.toString() : $location.hash();
      var elm;

      // empty hash, scroll to the top of the page
      if (!hash) scrollTo(null);

      // element with given id
      else if ((elm = document.getElementById(hash))) scrollTo(elm);

      // first anchor with given name :-D
      else if ((elm = getFirstAnchor(document.getElementsByName(hash)))) scrollTo(elm);

      // no element and hash === 'top', scroll to the top of the page
      else if (hash === 'top') scrollTo(null);
    }

    // does not scroll when user clicks on anchor link that is currently on
    // (no url change, no $location.hash() change), browser native does scroll
    if (autoScrollingEnabled) {
      $rootScope.$watch(function autoScrollWatch() {return $location.hash();},
        function autoScrollWatchAction(newVal, oldVal) {
          // skip the initial scroll if $location.hash is empty
          if (newVal === oldVal && newVal === '') return;

          jqLiteDocumentLoaded(function() {
            $rootScope.$evalAsync(scroll);
          });
        });
    }

    return scroll;
  }];
}

var $animateMinErr = minErr('$animate');
var ELEMENT_NODE = 1;
var NG_ANIMATE_CLASSNAME = 'ng-animate';

function mergeClasses(a,b) {
  if (!a && !b) return '';
  if (!a) return b;
  if (!b) return a;
  if (isArray(a)) a = a.join(' ');
  if (isArray(b)) b = b.join(' ');
  return a + ' ' + b;
}

function extractElementNode(element) {
  for (var i = 0; i < element.length; i++) {
    var elm = element[i];
    if (elm.nodeType === ELEMENT_NODE) {
      return elm;
    }
  }
}

function splitClasses(classes) {
  if (isString(classes)) {
    classes = classes.split(' ');
  }

  // Use createMap() to prevent class assumptions involving property names in
  // Object.prototype
  var obj = createMap();
  forEach(classes, function(klass) {
    // sometimes the split leaves empty string values
    // incase extra spaces were applied to the options
    if (klass.length) {
      obj[klass] = true;
    }
  });
  return obj;
}

// if any other type of options value besides an Object value is
// passed into the $animate.method() animation then this helper code
// will be run which will ignore it. While this patch is not the
// greatest solution to this, a lot of existing plugins depend on
// $animate to either call the callback (< 1.2) or return a promise
// that can be changed. This helper function ensures that the options
// are wiped clean incase a callback function is provided.
function prepareAnimateOptions(options) {
  return isObject(options)
      ? options
      : {};
}

var $$CoreAnimateJsProvider = /** @this */ function() {
  this.$get = noop;
};

// this is prefixed with Core since it conflicts with
// the animateQueueProvider defined in ngAnimate/animateQueue.js
var $$CoreAnimateQueueProvider = /** @this */ function() {
  var postDigestQueue = new NgMap();
  var postDigestElements = [];

  this.$get = ['$$AnimateRunner', '$rootScope',
       function($$AnimateRunner,   $rootScope) {
    return {
      enabled: noop,
      on: noop,
      off: noop,
      pin: noop,

      push: function(element, event, options, domOperation) {
        if (domOperation) {
          domOperation();
        }

        options = options || {};
        if (options.from) {
          element.css(options.from);
        }
        if (options.to) {
          element.css(options.to);
        }

        if (options.addClass || options.removeClass) {
          addRemoveClassesPostDigest(element, options.addClass, options.removeClass);
        }

        var runner = new $$AnimateRunner();

        // since there are no animations to run the runner needs to be
        // notified that the animation call is complete.
        runner.complete();
        return runner;
      }
    };


    function updateData(data, classes, value) {
      var changed = false;
      if (classes) {
        classes = isString(classes) ? classes.split(' ') :
                  isArray(classes) ? classes : [];
        forEach(classes, function(className) {
          if (className) {
            changed = true;
            data[className] = value;
          }
        });
      }
      return changed;
    }

    function handleCSSClassChanges() {
      forEach(postDigestElements, function(element) {
        var data = postDigestQueue.get(element);
        if (data) {
          var existing = splitClasses(element.attr('class'));
          var toAdd = '';
          var toRemove = '';
          forEach(data, function(status, className) {
            var hasClass = !!existing[className];
            if (status !== hasClass) {
              if (status) {
                toAdd += (toAdd.length ? ' ' : '') + className;
              } else {
                toRemove += (toRemove.length ? ' ' : '') + className;
              }
            }
          });

          forEach(element, function(elm) {
            if (toAdd) {
              jqLiteAddClass(elm, toAdd);
            }
            if (toRemove) {
              jqLiteRemoveClass(elm, toRemove);
            }
          });
          postDigestQueue.delete(element);
        }
      });
      postDigestElements.length = 0;
    }


    function addRemoveClassesPostDigest(element, add, remove) {
      var data = postDigestQueue.get(element) || {};

      var classesAdded = updateData(data, add, true);
      var classesRemoved = updateData(data, remove, false);

      if (classesAdded || classesRemoved) {

        postDigestQueue.set(element, data);
        postDigestElements.push(element);

        if (postDigestElements.length === 1) {
          $rootScope.$$postDigest(handleCSSClassChanges);
        }
      }
    }
  }];
};

/**
 * @ngdoc provider
 * @name $animateProvider
 *
 * @description
 * Default implementation of $animate that doesn't perform any animations, instead just
 * synchronously performs DOM updates and resolves the returned runner promise.
 *
 * In order to enable animations the `ngAnimate` module has to be loaded.
 *
 * To see the functional implementation check out `src/ngAnimate/animate.js`.
 */
var $AnimateProvider = ['$provide', /** @this */ function($provide) {
  var provider = this;
  var classNameFilter = null;
  var customFilter = null;

  this.$$registeredAnimations = Object.create(null);

   /**
   * @ngdoc method
   * @name $animateProvider#register
   *
   * @description
   * Registers a new injectable animation factory function. The factory function produces the
   * animation object which contains callback functions for each event that is expected to be
   * animated.
   *
   *   * `eventFn`: `function(element, ... , doneFunction, options)`
   *   The element to animate, the `doneFunction` and the options fed into the animation. Depending
   *   on the type of animation additional arguments will be injected into the animation function. The
   *   list below explains the function signatures for the different animation methods:
   *
   *   - setClass: function(element, addedClasses, removedClasses, doneFunction, options)
   *   - addClass: function(element, addedClasses, doneFunction, options)
   *   - removeClass: function(element, removedClasses, doneFunction, options)
   *   - enter, leave, move: function(element, doneFunction, options)
   *   - animate: function(element, fromStyles, toStyles, doneFunction, options)
   *
   *   Make sure to trigger the `doneFunction` once the animation is fully complete.
   *
   * ```js
   *   return {
   *     //enter, leave, move signature
   *     eventFn : function(element, done, options) {
   *       //code to run the animation
   *       //once complete, then run done()
   *       return function endFunction(wasCancelled) {
   *         //code to cancel the animation
   *       }
   *     }
   *   }
   * ```
   *
   * @param {string} name The name of the animation (this is what the class-based CSS value will be compared to).
   * @param {Function} factory The factory function that will be executed to return the animation
   *                           object.
   */
  this.register = function(name, factory) {
    if (name && name.charAt(0) !== '.') {
      throw $animateMinErr('notcsel', 'Expecting class selector starting with \'.\' got \'{0}\'.', name);
    }

    var key = name + '-animation';
    provider.$$registeredAnimations[name.substr(1)] = key;
    $provide.factory(key, factory);
  };

  /**
   * @ngdoc method
   * @name $animateProvider#customFilter
   *
   * @description
   * Sets and/or returns the custom filter function that is used to "filter" animations, i.e.
   * determine if an animation is allowed or not. When no filter is specified (the default), no
   * animation will be blocked. Setting the `customFilter` value will only allow animations for
   * which the filter function's return value is truthy.
   *
   * This allows to easily create arbitrarily complex rules for filtering animations, such as
   * allowing specific events only, or enabling animations on specific subtrees of the DOM, etc.
   * Filtering animations can also boost performance for low-powered devices, as well as
   * applications containing a lot of structural operations.
   *
   * <div class="alert alert-success">
   *   **Best Practice:**
   *   Keep the filtering function as lean as possible, because it will be called for each DOM
   *   action (e.g. insertion, removal, class change) performed by "animation-aware" directives.
   *   See {@link guide/animations#which-directives-support-animations- here} for a list of built-in
   *   directives that support animations.
   *   Performing computationally expensive or time-consuming operations on each call of the
   *   filtering function can make your animations sluggish.
   * </div>
   *
   * **Note:** If present, `customFilter` will be checked before
   * {@link $animateProvider#classNameFilter classNameFilter}.
   *
   * @param {Function=} filterFn - The filter function which will be used to filter all animations.
   *   If a falsy value is returned, no animation will be performed. The function will be called
   *   with the following arguments:
   *   - **node** `{DOMElement}` - The DOM element to be animated.
   *   - **event** `{String}` - The name of the animation event (e.g. `enter`, `leave`, `addClass`
   *     etc).
   *   - **options** `{Object}` - A collection of options/styles used for the animation.
   * @return {Function} The current filter function or `null` if there is none set.
   */
  this.customFilter = function(filterFn) {
    if (arguments.length === 1) {
      customFilter = isFunction(filterFn) ? filterFn : null;
    }

    return customFilter;
  };

  /**
   * @ngdoc method
   * @name $animateProvider#classNameFilter
   *
   * @description
   * Sets and/or returns the CSS class regular expression that is checked when performing
   * an animation. Upon bootstrap the classNameFilter value is not set at all and will
   * therefore enable $animate to attempt to perform an animation on any element that is triggered.
   * When setting the `classNameFilter` value, animations will only be performed on elements
   * that successfully match the filter expression. This in turn can boost performance
   * for low-powered devices as well as applications containing a lot of structural operations.
   *
   * **Note:** If present, `classNameFilter` will be checked after
   * {@link $animateProvider#customFilter customFilter}. If `customFilter` is present and returns
   * false, `classNameFilter` will not be checked.
   *
   * @param {RegExp=} expression The className expression which will be checked against all animations
   * @return {RegExp} The current CSS className expression value. If null then there is no expression value
   */
  this.classNameFilter = function(expression) {
    if (arguments.length === 1) {
      classNameFilter = (expression instanceof RegExp) ? expression : null;
      if (classNameFilter) {
        var reservedRegex = new RegExp('[(\\s|\\/)]' + NG_ANIMATE_CLASSNAME + '[(\\s|\\/)]');
        if (reservedRegex.test(classNameFilter.toString())) {
          classNameFilter = null;
          throw $animateMinErr('nongcls', '$animateProvider.classNameFilter(regex) prohibits accepting a regex value which matches/contains the "{0}" CSS class.', NG_ANIMATE_CLASSNAME);
        }
      }
    }
    return classNameFilter;
  };

  this.$get = ['$$animateQueue', function($$animateQueue) {
    function domInsert(element, parentElement, afterElement) {
      // if for some reason the previous element was removed
      // from the dom sometime before this code runs then let's
      // just stick to using the parent element as the anchor
      if (afterElement) {
        var afterNode = extractElementNode(afterElement);
        if (afterNode && !afterNode.parentNode && !afterNode.previousElementSibling) {
          afterElement = null;
        }
      }
      if (afterElement) {
        afterElement.after(element);
      } else {
        parentElement.prepend(element);
      }
    }

    /**
     * @ngdoc service
     * @name $animate
     * @description The $animate service exposes a series of DOM utility methods that provide support
     * for animation hooks. The default behavior is the application of DOM operations, however,
     * when an animation is detected (and animations are enabled), $animate will do the heavy lifting
     * to ensure that animation runs with the triggered DOM operation.
     *
     * By default $animate doesn't trigger any animations. This is because the `ngAnimate` module isn't
     * included and only when it is active then the animation hooks that `$animate` triggers will be
     * functional. Once active then all structural `ng-` directives will trigger animations as they perform
     * their DOM-related operations (enter, leave and move). Other directives such as `ngClass`,
     * `ngShow`, `ngHide` and `ngMessages` also provide support for animations.
     *
     * It is recommended that the`$animate` service is always used when executing DOM-related procedures within directives.
     *
     * To learn more about enabling animation support, click here to visit the
     * {@link ngAnimate ngAnimate module page}.
     */
    return {
      // we don't call it directly since non-existant arguments may
      // be interpreted as null within the sub enabled function

      /**
       *
       * @ngdoc method
       * @name $animate#on
       * @kind function
       * @description Sets up an event listener to fire whenever the animation event (enter, leave, move, etc...)
       *    has fired on the given element or among any of its children. Once the listener is fired, the provided callback
       *    is fired with the following params:
       *
       * ```js
       * $animate.on('enter', container,
       *    function callback(element, phase) {
       *      // cool we detected an enter animation within the container
       *    }
       * );
       * ```
       *
       * <div class="alert alert-warning">
       * **Note**: Generally, the events that are fired correspond 1:1 to `$animate` method names,
       * e.g. {@link ng.$animate#addClass addClass()} will fire `addClass`, and {@link ng.ngClass}
       * will fire `addClass` if classes are added, and `removeClass` if classes are removed.
       * However, there are two exceptions:
       *
       * <ul>
       *   <li>if both an {@link ng.$animate#addClass addClass()} and a
       *   {@link ng.$animate#removeClass removeClass()} action are performed during the same
       *   animation, the event fired will be `setClass`. This is true even for `ngClass`.</li>
       *   <li>an {@link ng.$animate#animate animate()} call that adds and removes classes will fire
       *   the `setClass` event, but if it either removes or adds classes,
       *   it will fire `animate` instead.</li>
       * </ul>
       *
       * </div>
       *
       * @param {string} event the animation event that will be captured (e.g. enter, leave, move, addClass, removeClass, etc...)
       * @param {DOMElement} container the container element that will capture each of the animation events that are fired on itself
       *     as well as among its children
       * @param {Function} callback the callback function that will be fired when the listener is triggered.
       *
       * The arguments present in the callback function are:
       * * `element` - The captured DOM element that the animation was fired on.
       * * `phase` - The phase of the animation. The two possible phases are **start** (when the animation starts) and **close** (when it ends).
       * * `data` - an object with these properties:
       *     * addClass - `{string|null}` - space-separated CSS classes to add to the element
       *     * removeClass - `{string|null}` - space-separated CSS classes to remove from the element
       *     * from - `{Object|null}` - CSS properties & values at the beginning of the animation
       *     * to - `{Object|null}` - CSS properties & values at the end of the animation
       *
       * Note that the callback does not trigger a scope digest. Wrap your call into a
       * {@link $rootScope.Scope#$apply scope.$apply} to propagate changes to the scope.
       */
      on: $$animateQueue.on,

      /**
       *
       * @ngdoc method
       * @name $animate#off
       * @kind function
       * @description Deregisters an event listener based on the event which has been associated with the provided element. This method
       * can be used in three different ways depending on the arguments:
       *
       * ```js
       * // remove all the animation event listeners listening for `enter`
       * $animate.off('enter');
       *
       * // remove listeners for all animation events from the container element
       * $animate.off(container);
       *
       * // remove all the animation event listeners listening for `enter` on the given element and its children
       * $animate.off('enter', container);
       *
       * // remove the event listener function provided by `callback` that is set
       * // to listen for `enter` on the given `container` as well as its children
       * $animate.off('enter', container, callback);
       * ```
       *
       * @param {string|DOMElement} event|container the animation event (e.g. enter, leave, move,
       * addClass, removeClass, etc...), or the container element. If it is the element, all other
       * arguments are ignored.
       * @param {DOMElement=} container the container element the event listener was placed on
       * @param {Function=} callback the callback function that was registered as the listener
       */
      off: $$animateQueue.off,

      /**
       * @ngdoc method
       * @name $animate#pin
       * @kind function
       * @description Associates the provided element with a host parent element to allow the element to be animated even if it exists
       *    outside of the DOM structure of the AngularJS application. By doing so, any animation triggered via `$animate` can be issued on the
       *    element despite being outside the realm of the application or within another application. Say for example if the application
       *    was bootstrapped on an element that is somewhere inside of the `<body>` tag, but we wanted to allow for an element to be situated
       *    as a direct child of `document.body`, then this can be achieved by pinning the element via `$animate.pin(element)`. Keep in mind
       *    that calling `$animate.pin(element, parentElement)` will not actually insert into the DOM anywhere; it will just create the association.
       *
       *    Note that this feature is only active when the `ngAnimate` module is used.
       *
       * @param {DOMElement} element the external element that will be pinned
       * @param {DOMElement} parentElement the host parent element that will be associated with the external element
       */
      pin: $$animateQueue.pin,

      /**
       *
       * @ngdoc method
       * @name $animate#enabled
       * @kind function
       * @description Used to get and set whether animations are enabled or not on the entire application or on an element and its children. This
       * function can be called in four ways:
       *
       * ```js
       * // returns true or false
       * $animate.enabled();
       *
       * // changes the enabled state for all animations
       * $animate.enabled(false);
       * $animate.enabled(true);
       *
       * // returns true or false if animations are enabled for an element
       * $animate.enabled(element);
       *
       * // changes the enabled state for an element and its children
       * $animate.enabled(element, true);
       * $animate.enabled(element, false);
       * ```
       *
       * @param {DOMElement=} element the element that will be considered for checking/setting the enabled state
       * @param {boolean=} enabled whether or not the animations will be enabled for the element
       *
       * @return {boolean} whether or not animations are enabled
       */
      enabled: $$animateQueue.enabled,

      /**
       * @ngdoc method
       * @name $animate#cancel
       * @kind function
       * @description Cancels the provided animation and applies the end state of the animation.
       * Note that this does not cancel the underlying operation, e.g. the setting of classes or
       * adding the element to the DOM.
       *
       * @param {animationRunner} animationRunner An animation runner returned by an $animate function.
       *
       * @example
        <example module="animationExample" deps="angular-animate.js" animations="true" name="animate-cancel">
          <file name="app.js">
            angular.module('animationExample', ['ngAnimate']).component('cancelExample', {
              templateUrl: 'template.html',
              controller: function($element, $animate) {
                this.runner = null;

                this.addClass = function() {
                  this.runner = $animate.addClass($element.find('div'), 'red');
                  var ctrl = this;
                  this.runner.finally(function() {
                    ctrl.runner = null;
                  });
                };

                this.removeClass = function() {
                  this.runner = $animate.removeClass($element.find('div'), 'red');
                  var ctrl = this;
                  this.runner.finally(function() {
                    ctrl.runner = null;
                  });
                };

                this.cancel = function() {
                  $animate.cancel(this.runner);
                };
              }
            });
          </file>
          <file name="template.html">
            <p>
              <button id="add" ng-click="$ctrl.addClass()">Add</button>
              <button ng-click="$ctrl.removeClass()">Remove</button>
              <br>
              <button id="cancel" ng-click="$ctrl.cancel()" ng-disabled="!$ctrl.runner">Cancel</button>
              <br>
              <div id="target">CSS-Animated Text</div>
            </p>
          </file>
          <file name="index.html">
            <cancel-example></cancel-example>
          </file>
          <file name="style.css">
            .red-add, .red-remove {
              transition: all 4s cubic-bezier(0.250, 0.460, 0.450, 0.940);
            }

            .red,
            .red-add.red-add-active {
              color: #FF0000;
              font-size: 40px;
            }

            .red-remove.red-remove-active {
              font-size: 10px;
              color: black;
            }

          </file>
        </example>
       */
      cancel: function(runner) {
        if (runner.cancel) {
          runner.cancel();
        }
      },

      /**
       *
       * @ngdoc method
       * @name $animate#enter
       * @kind function
       * @description Inserts the element into the DOM either after the `after` element (if provided) or
       *   as the first child within the `parent` element and then triggers an animation.
       *   A promise is returned that will be resolved during the next digest once the animation
       *   has completed.
       *
       * @param {DOMElement} element the element which will be inserted into the DOM
       * @param {DOMElement} parent the parent element which will append the element as
       *   a child (so long as the after element is not present)
       * @param {DOMElement=} after the sibling element after which the element will be appended
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      enter: function(element, parent, after, options) {
        parent = parent && jqLite(parent);
        after = after && jqLite(after);
        parent = parent || after.parent();
        domInsert(element, parent, after);
        return $$animateQueue.push(element, 'enter', prepareAnimateOptions(options));
      },

      /**
       *
       * @ngdoc method
       * @name $animate#move
       * @kind function
       * @description Inserts (moves) the element into its new position in the DOM either after
       *   the `after` element (if provided) or as the first child within the `parent` element
       *   and then triggers an animation. A promise is returned that will be resolved
       *   during the next digest once the animation has completed.
       *
       * @param {DOMElement} element the element which will be moved into the new DOM position
       * @param {DOMElement} parent the parent element which will append the element as
       *   a child (so long as the after element is not present)
       * @param {DOMElement=} after the sibling element after which the element will be appended
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      move: function(element, parent, after, options) {
        parent = parent && jqLite(parent);
        after = after && jqLite(after);
        parent = parent || after.parent();
        domInsert(element, parent, after);
        return $$animateQueue.push(element, 'move', prepareAnimateOptions(options));
      },

      /**
       * @ngdoc method
       * @name $animate#leave
       * @kind function
       * @description Triggers an animation and then removes the element from the DOM.
       * When the function is called a promise is returned that will be resolved during the next
       * digest once the animation has completed.
       *
       * @param {DOMElement} element the element which will be removed from the DOM
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      leave: function(element, options) {
        return $$animateQueue.push(element, 'leave', prepareAnimateOptions(options), function() {
          element.remove();
        });
      },

      /**
       * @ngdoc method
       * @name $animate#addClass
       * @kind function
       *
       * @description Triggers an addClass animation surrounding the addition of the provided CSS class(es). Upon
       *   execution, the addClass operation will only be handled after the next digest and it will not trigger an
       *   animation if element already contains the CSS class or if the class is removed at a later step.
       *   Note that class-based animations are treated differently compared to structural animations
       *   (like enter, move and leave) since the CSS classes may be added/removed at different points
       *   depending if CSS or JavaScript animations are used.
       *
       * @param {DOMElement} element the element which the CSS classes will be applied to
       * @param {string} className the CSS class(es) that will be added (multiple classes are separated via spaces)
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} animationRunner the animation runner
       */
      addClass: function(element, className, options) {
        options = prepareAnimateOptions(options);
        options.addClass = mergeClasses(options.addclass, className);
        return $$animateQueue.push(element, 'addClass', options);
      },

      /**
       * @ngdoc method
       * @name $animate#removeClass
       * @kind function
       *
       * @description Triggers a removeClass animation surrounding the removal of the provided CSS class(es). Upon
       *   execution, the removeClass operation will only be handled after the next digest and it will not trigger an
       *   animation if element does not contain the CSS class or if the class is added at a later step.
       *   Note that class-based animations are treated differently compared to structural animations
       *   (like enter, move and leave) since the CSS classes may be added/removed at different points
       *   depending if CSS or JavaScript animations are used.
       *
       * @param {DOMElement} element the element which the CSS classes will be applied to
       * @param {string} className the CSS class(es) that will be removed (multiple classes are separated via spaces)
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      removeClass: function(element, className, options) {
        options = prepareAnimateOptions(options);
        options.removeClass = mergeClasses(options.removeClass, className);
        return $$animateQueue.push(element, 'removeClass', options);
      },

      /**
       * @ngdoc method
       * @name $animate#setClass
       * @kind function
       *
       * @description Performs both the addition and removal of a CSS classes on an element and (during the process)
       *    triggers an animation surrounding the class addition/removal. Much like `$animate.addClass` and
       *    `$animate.removeClass`, `setClass` will only evaluate the classes being added/removed once a digest has
       *    passed. Note that class-based animations are treated differently compared to structural animations
       *    (like enter, move and leave) since the CSS classes may be added/removed at different points
       *    depending if CSS or JavaScript animations are used.
       *
       * @param {DOMElement} element the element which the CSS classes will be applied to
       * @param {string} add the CSS class(es) that will be added (multiple classes are separated via spaces)
       * @param {string} remove the CSS class(es) that will be removed (multiple classes are separated via spaces)
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      setClass: function(element, add, remove, options) {
        options = prepareAnimateOptions(options);
        options.addClass = mergeClasses(options.addClass, add);
        options.removeClass = mergeClasses(options.removeClass, remove);
        return $$animateQueue.push(element, 'setClass', options);
      },

      /**
       * @ngdoc method
       * @name $animate#animate
       * @kind function
       *
       * @description Performs an inline animation on the element which applies the provided to and from CSS styles to the element.
       * If any detected CSS transition, keyframe or JavaScript matches the provided className value, then the animation will take
       * on the provided styles. For example, if a transition animation is set for the given className, then the provided `from` and
       * `to` styles will be applied alongside the given transition. If the CSS style provided in `from` does not have a corresponding
       * style in `to`, the style in `from` is applied immediately, and no animation is run.
       * If a JavaScript animation is detected then the provided styles will be given in as function parameters into the `animate`
       * method (or as part of the `options` parameter):
       *
       * ```js
       * ngModule.animation('.my-inline-animation', function() {
       *   return {
       *     animate : function(element, from, to, done, options) {
       *       //animation
       *       done();
       *     }
       *   }
       * });
       * ```
       *
       * @param {DOMElement} element the element which the CSS styles will be applied to
       * @param {object} from the from (starting) CSS styles that will be applied to the element and across the animation.
       * @param {object} to the to (destination) CSS styles that will be applied to the element and across the animation.
       * @param {string=} className an optional CSS class that will be applied to the element for the duration of the animation. If
       *    this value is left as empty then a CSS class of `ng-inline-animate` will be applied to the element.
       *    (Note that if no animation is detected then this value will not be applied to the element.)
       * @param {object=} options an optional collection of options/styles that will be applied to the element.
       *   The object can have the following properties:
       *
       *   - **addClass** - `{string}` - space-separated CSS classes to add to element
       *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
       *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
       *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
       *
       * @return {Runner} the animation runner
       */
      animate: function(element, from, to, className, options) {
        options = prepareAnimateOptions(options);
        options.from = options.from ? extend(options.from, from) : from;
        options.to   = options.to   ? extend(options.to, to)     : to;

        className = className || 'ng-inline-animate';
        options.tempClasses = mergeClasses(options.tempClasses, className);
        return $$animateQueue.push(element, 'animate', options);
      }
    };
  }];
}];

var $$AnimateAsyncRunFactoryProvider = /** @this */ function() {
  this.$get = ['$$rAF', function($$rAF) {
    var waitQueue = [];

    function waitForTick(fn) {
      waitQueue.push(fn);
      if (waitQueue.length > 1) return;
      $$rAF(function() {
        for (var i = 0; i < waitQueue.length; i++) {
          waitQueue[i]();
        }
        waitQueue = [];
      });
    }

    return function() {
      var passed = false;
      waitForTick(function() {
        passed = true;
      });
      return function(callback) {
        if (passed) {
          callback();
        } else {
          waitForTick(callback);
        }
      };
    };
  }];
};

var $$AnimateRunnerFactoryProvider = /** @this */ function() {
  this.$get = ['$q', '$sniffer', '$$animateAsyncRun', '$$isDocumentHidden', '$timeout',
       function($q,   $sniffer,   $$animateAsyncRun,   $$isDocumentHidden,   $timeout) {

    var INITIAL_STATE = 0;
    var DONE_PENDING_STATE = 1;
    var DONE_COMPLETE_STATE = 2;

    AnimateRunner.chain = function(chain, callback) {
      var index = 0;

      next();
      function next() {
        if (index === chain.length) {
          callback(true);
          return;
        }

        chain[index](function(response) {
          if (response === false) {
            callback(false);
            return;
          }
          index++;
          next();
        });
      }
    };

    AnimateRunner.all = function(runners, callback) {
      var count = 0;
      var status = true;
      forEach(runners, function(runner) {
        runner.done(onProgress);
      });

      function onProgress(response) {
        status = status && response;
        if (++count === runners.length) {
          callback(status);
        }
      }
    };

    function AnimateRunner(host) {
      this.setHost(host);

      var rafTick = $$animateAsyncRun();
      var timeoutTick = function(fn) {
        $timeout(fn, 0, false);
      };

      this._doneCallbacks = [];
      this._tick = function(fn) {
        if ($$isDocumentHidden()) {
          timeoutTick(fn);
        } else {
          rafTick(fn);
        }
      };
      this._state = 0;
    }

    AnimateRunner.prototype = {
      setHost: function(host) {
        this.host = host || {};
      },

      done: function(fn) {
        if (this._state === DONE_COMPLETE_STATE) {
          fn();
        } else {
          this._doneCallbacks.push(fn);
        }
      },

      progress: noop,

      getPromise: function() {
        if (!this.promise) {
          var self = this;
          this.promise = $q(function(resolve, reject) {
            self.done(function(status) {
              if (status === false) {
                reject();
              } else {
                resolve();
              }
            });
          });
        }
        return this.promise;
      },

      then: function(resolveHandler, rejectHandler) {
        return this.getPromise().then(resolveHandler, rejectHandler);
      },

      'catch': function(handler) {
        return this.getPromise()['catch'](handler);
      },

      'finally': function(handler) {
        return this.getPromise()['finally'](handler);
      },

      pause: function() {
        if (this.host.pause) {
          this.host.pause();
        }
      },

      resume: function() {
        if (this.host.resume) {
          this.host.resume();
        }
      },

      end: function() {
        if (this.host.end) {
          this.host.end();
        }
        this._resolve(true);
      },

      cancel: function() {
        if (this.host.cancel) {
          this.host.cancel();
        }
        this._resolve(false);
      },

      complete: function(response) {
        var self = this;
        if (self._state === INITIAL_STATE) {
          self._state = DONE_PENDING_STATE;
          self._tick(function() {
            self._resolve(response);
          });
        }
      },

      _resolve: function(response) {
        if (this._state !== DONE_COMPLETE_STATE) {
          forEach(this._doneCallbacks, function(fn) {
            fn(response);
          });
          this._doneCallbacks.length = 0;
          this._state = DONE_COMPLETE_STATE;
        }
      }
    };

    return AnimateRunner;
  }];
};

/* exported $CoreAnimateCssProvider */

/**
 * @ngdoc service
 * @name $animateCss
 * @kind object
 * @this
 *
 * @description
 * This is the core version of `$animateCss`. By default, only when the `ngAnimate` is included,
 * then the `$animateCss` service will actually perform animations.
 *
 * Click here {@link ngAnimate.$animateCss to read the documentation for $animateCss}.
 */
var $CoreAnimateCssProvider = function() {
  this.$get = ['$$rAF', '$q', '$$AnimateRunner', function($$rAF, $q, $$AnimateRunner) {

    return function(element, initialOptions) {
      // all of the animation functions should create
      // a copy of the options data, however, if a
      // parent service has already created a copy then
      // we should stick to using that
      var options = initialOptions || {};
      if (!options.$$prepared) {
        options = copy(options);
      }

      // there is no point in applying the styles since
      // there is no animation that goes on at all in
      // this version of $animateCss.
      if (options.cleanupStyles) {
        options.from = options.to = null;
      }

      if (options.from) {
        element.css(options.from);
        options.from = null;
      }

      var closed, runner = new $$AnimateRunner();
      return {
        start: run,
        end: run
      };

      function run() {
        $$rAF(function() {
          applyAnimationContents();
          if (!closed) {
            runner.complete();
          }
          closed = true;
        });
        return runner;
      }

      function applyAnimationContents() {
        if (options.addClass) {
          element.addClass(options.addClass);
          options.addClass = null;
        }
        if (options.removeClass) {
          element.removeClass(options.removeClass);
          options.removeClass = null;
        }
        if (options.to) {
          element.css(options.to);
          options.to = null;
        }
      }
    };
  }];
};

/* global getHash: true, stripHash: false */

function getHash(url) {
  var index = url.indexOf('#');
  return index === -1 ? '' : url.substr(index);
}

function trimEmptyHash(url) {
  return url.replace(/#$/, '');
}

/**
 * ! This is a private undocumented service !
 *
 * @name $browser
 * @requires $log
 * @description
 * This object has two goals:
 *
 * - hide all the global state in the browser caused by the window object
 * - abstract away all the browser specific features and inconsistencies
 *
 * For tests we provide {@link ngMock.$browser mock implementation} of the `$browser`
 * service, which can be used for convenient testing of the application without the interaction with
 * the real browser apis.
 */
/**
 * @param {object} window The global window object.
 * @param {object} document jQuery wrapped document.
 * @param {object} $log window.console or an object with the same interface.
 * @param {object} $sniffer $sniffer service
 */
function Browser(window, document, $log, $sniffer, $$taskTrackerFactory) {
  var self = this,
      location = window.location,
      history = window.history,
      setTimeout = window.setTimeout,
      clearTimeout = window.clearTimeout,
      pendingDeferIds = {},
      taskTracker = $$taskTrackerFactory($log);

  self.isMock = false;

  //////////////////////////////////////////////////////////////
  // Task-tracking API
  //////////////////////////////////////////////////////////////

  // TODO(vojta): remove this temporary api
  self.$$completeOutstandingRequest = taskTracker.completeTask;
  self.$$incOutstandingRequestCount = taskTracker.incTaskCount;

  // TODO(vojta): prefix this method with $$ ?
  self.notifyWhenNoOutstandingRequests = taskTracker.notifyWhenNoPendingTasks;

  //////////////////////////////////////////////////////////////
  // URL API
  //////////////////////////////////////////////////////////////

  var cachedState, lastHistoryState,
      lastBrowserUrl = location.href,
      baseElement = document.find('base'),
      pendingLocation = null,
      getCurrentState = !$sniffer.history ? noop : function getCurrentState() {
        try {
          return history.state;
        } catch (e) {
          // MSIE can reportedly throw when there is no state (UNCONFIRMED).
        }
      };

  cacheState();

  /**
   * @name $browser#url
   *
   * @description
   * GETTER:
   * Without any argument, this method just returns current value of `location.href` (with a
   * trailing `#` stripped of if the hash is empty).
   *
   * SETTER:
   * With at least one argument, this method sets url to new value.
   * If html5 history api supported, `pushState`/`replaceState` is used, otherwise
   * `location.href`/`location.replace` is used.
   * Returns its own instance to allow chaining.
   *
   * NOTE: this api is intended for use only by the `$location` service. Please use the
   * {@link ng.$location $location service} to change url.
   *
   * @param {string} url New url (when used as setter)
   * @param {boolean=} replace Should new url replace current history record?
   * @param {object=} state State object to use with `pushState`/`replaceState`
   */
  self.url = function(url, replace, state) {
    // In modern browsers `history.state` is `null` by default; treating it separately
    // from `undefined` would cause `$browser.url('/foo')` to change `history.state`
    // to undefined via `pushState`. Instead, let's change `undefined` to `null` here.
    if (isUndefined(state)) {
      state = null;
    }

    // Android Browser BFCache causes location, history reference to become stale.
    if (location !== window.location) location = window.location;
    if (history !== window.history) history = window.history;

    // setter
    if (url) {
      var sameState = lastHistoryState === state;

      // Normalize the inputted URL
      url = urlResolve(url).href;

      // Don't change anything if previous and current URLs and states match. This also prevents
      // IE<10 from getting into redirect loop when in LocationHashbangInHtml5Url mode.
      // See https://github.com/angular/angular.js/commit/ffb2701
      if (lastBrowserUrl === url && (!$sniffer.history || sameState)) {
        return self;
      }
      var sameBase = lastBrowserUrl && stripHash(lastBrowserUrl) === stripHash(url);
      lastBrowserUrl = url;
      lastHistoryState = state;
      // Don't use history API if only the hash changed
      // due to a bug in IE10/IE11 which leads
      // to not firing a `hashchange` nor `popstate` event
      // in some cases (see #9143).
      if ($sniffer.history && (!sameBase || !sameState)) {
        history[replace ? 'replaceState' : 'pushState'](state, '', url);
        cacheState();
      } else {
        if (!sameBase) {
          pendingLocation = url;
        }
        if (replace) {
          location.replace(url);
        } else if (!sameBase) {
          location.href = url;
        } else {
          location.hash = getHash(url);
        }
        if (location.href !== url) {
          pendingLocation = url;
        }
      }
      if (pendingLocation) {
        pendingLocation = url;
      }
      return self;
    // getter
    } else {
      // - pendingLocation is needed as browsers don't allow to read out
      //   the new location.href if a reload happened or if there is a bug like in iOS 9 (see
      //   https://openradar.appspot.com/22186109).
      return trimEmptyHash(pendingLocation || location.href);
    }
  };

  /**
   * @name $browser#state
   *
   * @description
   * This method is a getter.
   *
   * Return history.state or null if history.state is undefined.
   *
   * @returns {object} state
   */
  self.state = function() {
    return cachedState;
  };

  var urlChangeListeners = [],
      urlChangeInit = false;

  function cacheStateAndFireUrlChange() {
    pendingLocation = null;
    fireStateOrUrlChange();
  }

  // This variable should be used *only* inside the cacheState function.
  var lastCachedState = null;
  function cacheState() {
    // This should be the only place in $browser where `history.state` is read.
    cachedState = getCurrentState();
    cachedState = isUndefined(cachedState) ? null : cachedState;

    // Prevent callbacks fo fire twice if both hashchange & popstate were fired.
    if (equals(cachedState, lastCachedState)) {
      cachedState = lastCachedState;
    }

    lastCachedState = cachedState;
    lastHistoryState = cachedState;
  }

  function fireStateOrUrlChange() {
    var prevLastHistoryState = lastHistoryState;
    cacheState();

    if (lastBrowserUrl === self.url() && prevLastHistoryState === cachedState) {
      return;
    }

    lastBrowserUrl = self.url();
    lastHistoryState = cachedState;
    forEach(urlChangeListeners, function(listener) {
      listener(self.url(), cachedState);
    });
  }

  /**
   * @name $browser#onUrlChange
   *
   * @description
   * Register callback function that will be called, when url changes.
   *
   * It's only called when the url is changed from outside of AngularJS:
   * - user types different url into address bar
   * - user clicks on history (forward/back) button
   * - user clicks on a link
   *
   * It's not called when url is changed by $browser.url() method
   *
   * The listener gets called with new url as parameter.
   *
   * NOTE: this api is intended for use only by the $location service. Please use the
   * {@link ng.$location $location service} to monitor url changes in AngularJS apps.
   *
   * @param {function(string)} listener Listener function to be called when url changes.
   * @return {function(string)} Returns the registered listener fn - handy if the fn is anonymous.
   */
  self.onUrlChange = function(callback) {
    // TODO(vojta): refactor to use node's syntax for events
    if (!urlChangeInit) {
      // We listen on both (hashchange/popstate) when available, as some browsers don't
      // fire popstate when user changes the address bar and don't fire hashchange when url
      // changed by push/replaceState

      // html5 history api - popstate event
      if ($sniffer.history) jqLite(window).on('popstate', cacheStateAndFireUrlChange);
      // hashchange event
      jqLite(window).on('hashchange', cacheStateAndFireUrlChange);

      urlChangeInit = true;
    }

    urlChangeListeners.push(callback);
    return callback;
  };

  /**
   * @private
   * Remove popstate and hashchange handler from window.
   *
   * NOTE: this api is intended for use only by $rootScope.
   */
  self.$$applicationDestroyed = function() {
    jqLite(window).off('hashchange popstate', cacheStateAndFireUrlChange);
  };

  /**
   * Checks whether the url has changed outside of AngularJS.
   * Needs to be exported to be able to check for changes that have been done in sync,
   * as hashchange/popstate events fire in async.
   */
  self.$$checkUrlChange = fireStateOrUrlChange;

  //////////////////////////////////////////////////////////////
  // Misc API
  //////////////////////////////////////////////////////////////

  /**
   * @name $browser#baseHref
   *
   * @description
   * Returns current <base href>
   * (always relative - without domain)
   *
   * @returns {string} The current base href
   */
  self.baseHref = function() {
    var href = baseElement.attr('href');
    return href ? href.replace(/^(https?:)?\/\/[^/]*/, '') : '';
  };

  /**
   * @name $browser#defer
   * @param {function()} fn A function, who's execution should be deferred.
   * @param {number=} [delay=0] Number of milliseconds to defer the function execution.
   * @param {string=} [taskType=DEFAULT_TASK_TYPE] The type of task that is deferred.
   * @returns {*} DeferId that can be used to cancel the task via `$browser.defer.cancel()`.
   *
   * @description
   * Executes a fn asynchronously via `setTimeout(fn, delay)`.
   *
   * Unlike when calling `setTimeout` directly, in test this function is mocked and instead of using
   * `setTimeout` in tests, the fns are queued in an array, which can be programmatically flushed
   * via `$browser.defer.flush()`.
   *
   */
  self.defer = function(fn, delay, taskType) {
    var timeoutId;

    delay = delay || 0;
    taskType = taskType || taskTracker.DEFAULT_TASK_TYPE;

    taskTracker.incTaskCount(taskType);
    timeoutId = setTimeout(function() {
      delete pendingDeferIds[timeoutId];
      taskTracker.completeTask(fn, taskType);
    }, delay);
    pendingDeferIds[timeoutId] = taskType;

    return timeoutId;
  };


  /**
   * @name $browser#defer.cancel
   *
   * @description
   * Cancels a deferred task identified with `deferId`.
   *
   * @param {*} deferId Token returned by the `$browser.defer` function.
   * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
   *                    canceled.
   */
  self.defer.cancel = function(deferId) {
    if (pendingDeferIds.hasOwnProperty(deferId)) {
      var taskType = pendingDeferIds[deferId];
      delete pendingDeferIds[deferId];
      clearTimeout(deferId);
      taskTracker.completeTask(noop, taskType);
      return true;
    }
    return false;
  };

}

/** @this */
function $BrowserProvider() {
  this.$get = ['$window', '$log', '$sniffer', '$document', '$$taskTrackerFactory',
       function($window,   $log,   $sniffer,   $document,   $$taskTrackerFactory) {
    return new Browser($window, $document, $log, $sniffer, $$taskTrackerFactory);
  }];
}

/**
 * @ngdoc service
 * @name $cacheFactory
 * @this
 *
 * @description
 * Factory that constructs {@link $cacheFactory.Cache Cache} objects and gives access to
 * them.
 *
 * ```js
 *
 *  var cache = $cacheFactory('cacheId');
 *  expect($cacheFactory.get('cacheId')).toBe(cache);
 *  expect($cacheFactory.get('noSuchCacheId')).not.toBeDefined();
 *
 *  cache.put("key", "value");
 *  cache.put("another key", "another value");
 *
 *  // We've specified no options on creation
 *  expect(cache.info()).toEqual({id: 'cacheId', size: 2});
 *
 * ```
 *
 *
 * @param {string} cacheId Name or id of the newly created cache.
 * @param {object=} options Options object that specifies the cache behavior. Properties:
 *
 *   - `{number=}` `capacity` — turns the cache into LRU cache.
 *
 * @returns {object} Newly created cache object with the following set of methods:
 *
 * - `{object}` `info()` — Returns id, size, and options of cache.
 * - `{{*}}` `put({string} key, {*} value)` — Puts a new key-value pair into the cache and returns
 *   it.
 * - `{{*}}` `get({string} key)` — Returns cached value for `key` or undefined for cache miss.
 * - `{void}` `remove({string} key)` — Removes a key-value pair from the cache.
 * - `{void}` `removeAll()` — Removes all cached values.
 * - `{void}` `destroy()` — Removes references to this cache from $cacheFactory.
 *
 * @example
   <example module="cacheExampleApp" name="cache-factory">
     <file name="index.html">
       <div ng-controller="CacheController">
         <input ng-model="newCacheKey" placeholder="Key">
         <input ng-model="newCacheValue" placeholder="Value">
         <button ng-click="put(newCacheKey, newCacheValue)">Cache</button>

         <p ng-if="keys.length">Cached Values</p>
         <div ng-repeat="key in keys">
           <span ng-bind="key"></span>
           <span>: </span>
           <b ng-bind="cache.get(key)"></b>
         </div>

         <p>Cache Info</p>
         <div ng-repeat="(key, value) in cache.info()">
           <span ng-bind="key"></span>
           <span>: </span>
           <b ng-bind="value"></b>
         </div>
       </div>
     </file>
     <file name="script.js">
       angular.module('cacheExampleApp', []).
         controller('CacheController', ['$scope', '$cacheFactory', function($scope, $cacheFactory) {
           $scope.keys = [];
           $scope.cache = $cacheFactory('cacheId');
           $scope.put = function(key, value) {
             if (angular.isUndefined($scope.cache.get(key))) {
               $scope.keys.push(key);
             }
             $scope.cache.put(key, angular.isUndefined(value) ? null : value);
           };
         }]);
     </file>
     <file name="style.css">
       p {
         margin: 10px 0 3px;
       }
     </file>
   </example>
 */
function $CacheFactoryProvider() {

  this.$get = function() {
    var caches = {};

    function cacheFactory(cacheId, options) {
      if (cacheId in caches) {
        throw minErr('$cacheFactory')('iid', 'CacheId \'{0}\' is already taken!', cacheId);
      }

      var size = 0,
          stats = extend({}, options, {id: cacheId}),
          data = createMap(),
          capacity = (options && options.capacity) || Number.MAX_VALUE,
          lruHash = createMap(),
          freshEnd = null,
          staleEnd = null;

      /**
       * @ngdoc type
       * @name $cacheFactory.Cache
       *
       * @description
       * A cache object used to store and retrieve data, primarily used by
       * {@link $templateRequest $templateRequest} and the {@link ng.directive:script script}
       * directive to cache templates and other data.
       *
       * ```js
       *  angular.module('superCache')
       *    .factory('superCache', ['$cacheFactory', function($cacheFactory) {
       *      return $cacheFactory('super-cache');
       *    }]);
       * ```
       *
       * Example test:
       *
       * ```js
       *  it('should behave like a cache', inject(function(superCache) {
       *    superCache.put('key', 'value');
       *    superCache.put('another key', 'another value');
       *
       *    expect(superCache.info()).toEqual({
       *      id: 'super-cache',
       *      size: 2
       *    });
       *
       *    superCache.remove('another key');
       *    expect(superCache.get('another key')).toBeUndefined();
       *
       *    superCache.removeAll();
       *    expect(superCache.info()).toEqual({
       *      id: 'super-cache',
       *      size: 0
       *    });
       *  }));
       * ```
       */
      return (caches[cacheId] = {

        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#put
         * @kind function
         *
         * @description
         * Inserts a named entry into the {@link $cacheFactory.Cache Cache} object to be
         * retrieved later, and incrementing the size of the cache if the key was not already
         * present in the cache. If behaving like an LRU cache, it will also remove stale
         * entries from the set.
         *
         * It will not insert undefined values into the cache.
         *
         * @param {string} key the key under which the cached data is stored.
         * @param {*} value the value to store alongside the key. If it is undefined, the key
         *    will not be stored.
         * @returns {*} the value stored.
         */
        put: function(key, value) {
          if (isUndefined(value)) return;
          if (capacity < Number.MAX_VALUE) {
            var lruEntry = lruHash[key] || (lruHash[key] = {key: key});

            refresh(lruEntry);
          }

          if (!(key in data)) size++;
          data[key] = value;

          if (size > capacity) {
            this.remove(staleEnd.key);
          }

          return value;
        },

        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#get
         * @kind function
         *
         * @description
         * Retrieves named data stored in the {@link $cacheFactory.Cache Cache} object.
         *
         * @param {string} key the key of the data to be retrieved
         * @returns {*} the value stored.
         */
        get: function(key) {
          if (capacity < Number.MAX_VALUE) {
            var lruEntry = lruHash[key];

            if (!lruEntry) return;

            refresh(lruEntry);
          }

          return data[key];
        },


        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#remove
         * @kind function
         *
         * @description
         * Removes an entry from the {@link $cacheFactory.Cache Cache} object.
         *
         * @param {string} key the key of the entry to be removed
         */
        remove: function(key) {
          if (capacity < Number.MAX_VALUE) {
            var lruEntry = lruHash[key];

            if (!lruEntry) return;

            if (lruEntry === freshEnd) freshEnd = lruEntry.p;
            if (lruEntry === staleEnd) staleEnd = lruEntry.n;
            link(lruEntry.n,lruEntry.p);

            delete lruHash[key];
          }

          if (!(key in data)) return;

          delete data[key];
          size--;
        },


        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#removeAll
         * @kind function
         *
         * @description
         * Clears the cache object of any entries.
         */
        removeAll: function() {
          data = createMap();
          size = 0;
          lruHash = createMap();
          freshEnd = staleEnd = null;
        },


        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#destroy
         * @kind function
         *
         * @description
         * Destroys the {@link $cacheFactory.Cache Cache} object entirely,
         * removing it from the {@link $cacheFactory $cacheFactory} set.
         */
        destroy: function() {
          data = null;
          stats = null;
          lruHash = null;
          delete caches[cacheId];
        },


        /**
         * @ngdoc method
         * @name $cacheFactory.Cache#info
         * @kind function
         *
         * @description
         * Retrieve information regarding a particular {@link $cacheFactory.Cache Cache}.
         *
         * @returns {object} an object with the following properties:
         *   <ul>
         *     <li>**id**: the id of the cache instance</li>
         *     <li>**size**: the number of entries kept in the cache instance</li>
         *     <li>**...**: any additional properties from the options object when creating the
         *       cache.</li>
         *   </ul>
         */
        info: function() {
          return extend({}, stats, {size: size});
        }
      });


      /**
       * makes the `entry` the freshEnd of the LRU linked list
       */
      function refresh(entry) {
        if (entry !== freshEnd) {
          if (!staleEnd) {
            staleEnd = entry;
          } else if (staleEnd === entry) {
            staleEnd = entry.n;
          }

          link(entry.n, entry.p);
          link(entry, freshEnd);
          freshEnd = entry;
          freshEnd.n = null;
        }
      }


      /**
       * bidirectionally links two entries of the LRU linked list
       */
      function link(nextEntry, prevEntry) {
        if (nextEntry !== prevEntry) {
          if (nextEntry) nextEntry.p = prevEntry; //p stands for previous, 'prev' didn't minify
          if (prevEntry) prevEntry.n = nextEntry; //n stands for next, 'next' didn't minify
        }
      }
    }


  /**
   * @ngdoc method
   * @name $cacheFactory#info
   *
   * @description
   * Get information about all the caches that have been created
   *
   * @returns {Object} - key-value map of `cacheId` to the result of calling `cache#info`
   */
    cacheFactory.info = function() {
      var info = {};
      forEach(caches, function(cache, cacheId) {
        info[cacheId] = cache.info();
      });
      return info;
    };


  /**
   * @ngdoc method
   * @name $cacheFactory#get
   *
   * @description
   * Get access to a cache object by the `cacheId` used when it was created.
   *
   * @param {string} cacheId Name or id of a cache to access.
   * @returns {object} Cache object identified by the cacheId or undefined if no such cache.
   */
    cacheFactory.get = function(cacheId) {
      return caches[cacheId];
    };


    return cacheFactory;
  };
}

/**
 * @ngdoc service
 * @name $templateCache
 * @this
 *
 * @description
 * `$templateCache` is a {@link $cacheFactory.Cache Cache object} created by the
 * {@link ng.$cacheFactory $cacheFactory}.
 *
 * The first time a template is used, it is loaded in the template cache for quick retrieval. You
 * can load templates directly into the cache in a `script` tag, by using {@link $templateRequest},
 * or by consuming the `$templateCache` service directly.
 *
 * Adding via the `script` tag:
 *
 * ```html
 *   <script type="text/ng-template" id="templateId.html">
 *     <p>This is the content of the template</p>
 *   </script>
 * ```
 *
 * **Note:** the `script` tag containing the template does not need to be included in the `head` of
 * the document, but it must be a descendent of the {@link ng.$rootElement $rootElement} (e.g.
 * element with {@link ngApp} attribute), otherwise the template will be ignored.
 *
 * Adding via the `$templateCache` service:
 *
 * ```js
 * var myApp = angular.module('myApp', []);
 * myApp.run(function($templateCache) {
 *   $templateCache.put('templateId.html', 'This is the content of the template');
 * });
 * ```
 *
 * To retrieve the template later, simply use it in your component:
 * ```js
 * myApp.component('myComponent', {
 *    templateUrl: 'templateId.html'
 * });
 * ```
 *
 * or get it via the `$templateCache` service:
 * ```js
 * $templateCache.get('templateId.html')
 * ```
 *
 */
function $TemplateCacheProvider() {
  this.$get = ['$cacheFactory', function($cacheFactory) {
    return $cacheFactory('templates');
  }];
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables like document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* ! VARIABLE/FUNCTION NAMING CONVENTIONS THAT APPLY TO THIS FILE!
 *
 * DOM-related variables:
 *
 * - "node" - DOM Node
 * - "element" - DOM Element or Node
 * - "$node" or "$element" - jqLite-wrapped node or element
 *
 *
 * Compiler related stuff:
 *
 * - "linkFn" - linking fn of a single directive
 * - "nodeLinkFn" - function that aggregates all linking fns for a particular node
 * - "childLinkFn" -  function that aggregates all linking fns for child nodes of a particular node
 * - "compositeLinkFn" - function that aggregates all linking fns for a compilation root (nodeList)
 */


/**
 * @ngdoc service
 * @name $compile
 * @kind function
 *
 * @description
 * Compiles an HTML string or DOM into a template and produces a template function, which
 * can then be used to link {@link ng.$rootScope.Scope `scope`} and the template together.
 *
 * The compilation is a process of walking the DOM tree and matching DOM elements to
 * {@link ng.$compileProvider#directive directives}.
 *
 * <div class="alert alert-warning">
 * **Note:** This document is an in-depth reference of all directive options.
 * For a gentle introduction to directives with examples of common use cases,
 * see the {@link guide/directive directive guide}.
 * </div>
 *
 * ## Comprehensive Directive API
 *
 * There are many different options for a directive.
 *
 * The difference resides in the return value of the factory function.
 * You can either return a {@link $compile#directive-definition-object Directive Definition Object (see below)}
 * that defines the directive properties, or just the `postLink` function (all other properties will have
 * the default values).
 *
 * <div class="alert alert-success">
 * **Best Practice:** It's recommended to use the "directive definition object" form.
 * </div>
 *
 * Here's an example directive declared with a Directive Definition Object:
 *
 * ```js
 *   var myModule = angular.module(...);
 *
 *   myModule.directive('directiveName', function factory(injectables) {
 *     var directiveDefinitionObject = {
 *       {@link $compile#-priority- priority}: 0,
 *       {@link $compile#-template- template}: '<div></div>', // or // function(tElement, tAttrs) { ... },
 *       // or
 *       // {@link $compile#-templateurl- templateUrl}: 'directive.html', // or // function(tElement, tAttrs) { ... },
 *       {@link $compile#-transclude- transclude}: false,
 *       {@link $compile#-restrict- restrict}: 'A',
 *       {@link $compile#-templatenamespace- templateNamespace}: 'html',
 *       {@link $compile#-scope- scope}: false,
 *       {@link $compile#-controller- controller}: function($scope, $element, $attrs, $transclude, otherInjectables) { ... },
 *       {@link $compile#-controlleras- controllerAs}: 'stringIdentifier',
 *       {@link $compile#-bindtocontroller- bindToController}: false,
 *       {@link $compile#-require- require}: 'siblingDirectiveName', // or // ['^parentDirectiveName', '?optionalDirectiveName', '?^optionalParent'],
 *       {@link $compile#-multielement- multiElement}: false,
 *       {@link $compile#-compile- compile}: function compile(tElement, tAttrs, transclude) {
 *         return {
 *            {@link $compile#pre-linking-function pre}: function preLink(scope, iElement, iAttrs, controller) { ... },
 *            {@link $compile#post-linking-function post}: function postLink(scope, iElement, iAttrs, controller) { ... }
 *         }
 *         // or
 *         // return function postLink( ... ) { ... }
 *       },
 *       // or
 *       // {@link $compile#-link- link}: {
 *       //  {@link $compile#pre-linking-function pre}: function preLink(scope, iElement, iAttrs, controller) { ... },
 *       //  {@link $compile#post-linking-function post}: function postLink(scope, iElement, iAttrs, controller) { ... }
 *       // }
 *       // or
 *       // {@link $compile#-link- link}: function postLink( ... ) { ... }
 *     };
 *     return directiveDefinitionObject;
 *   });
 * ```
 *
 * <div class="alert alert-warning">
 * **Note:** Any unspecified options will use the default value. You can see the default values below.
 * </div>
 *
 * Therefore the above can be simplified as:
 *
 * ```js
 *   var myModule = angular.module(...);
 *
 *   myModule.directive('directiveName', function factory(injectables) {
 *     var directiveDefinitionObject = {
 *       link: function postLink(scope, iElement, iAttrs) { ... }
 *     };
 *     return directiveDefinitionObject;
 *     // or
 *     // return function postLink(scope, iElement, iAttrs) { ... }
 *   });
 * ```
 *
 * ### Life-cycle hooks
 * Directive controllers can provide the following methods that are called by AngularJS at points in the life-cycle of the
 * directive:
 * * `$onInit()` - Called on each controller after all the controllers on an element have been constructed and
 *   had their bindings initialized (and before the pre &amp; post linking functions for the directives on
 *   this element). This is a good place to put initialization code for your controller.
 * * `$onChanges(changesObj)` - Called whenever one-way (`<`) or interpolation (`@`) bindings are updated. The
 *   `changesObj` is a hash whose keys are the names of the bound properties that have changed, and the values are an
 *   object of the form `{ currentValue, previousValue, isFirstChange() }`. Use this hook to trigger updates within a
 *   component such as cloning the bound value to prevent accidental mutation of the outer value. Note that this will
 *   also be called when your bindings are initialized.
 * * `$doCheck()` - Called on each turn of the digest cycle. Provides an opportunity to detect and act on
 *   changes. Any actions that you wish to take in response to the changes that you detect must be
 *   invoked from this hook; implementing this has no effect on when `$onChanges` is called. For example, this hook
 *   could be useful if you wish to perform a deep equality check, or to check a Date object, changes to which would not
 *   be detected by AngularJS's change detector and thus not trigger `$onChanges`. This hook is invoked with no arguments;
 *   if detecting changes, you must store the previous value(s) for comparison to the current values.
 * * `$onDestroy()` - Called on a controller when its containing scope is destroyed. Use this hook for releasing
 *   external resources, watches and event handlers. Note that components have their `$onDestroy()` hooks called in
 *   the same order as the `$scope.$broadcast` events are triggered, which is top down. This means that parent
 *   components will have their `$onDestroy()` hook called before child components.
 * * `$postLink()` - Called after this controller's element and its children have been linked. Similar to the post-link
 *   function this hook can be used to set up DOM event handlers and do direct DOM manipulation.
 *   Note that child elements that contain `templateUrl` directives will not have been compiled and linked since
 *   they are waiting for their template to load asynchronously and their own compilation and linking has been
 *   suspended until that occurs.
 *
 * #### Comparison with life-cycle hooks in the new Angular
 * The new Angular also uses life-cycle hooks for its components. While the AngularJS life-cycle hooks are similar there are
 * some differences that you should be aware of, especially when it comes to moving your code from AngularJS to Angular:
 *
 * * AngularJS hooks are prefixed with `$`, such as `$onInit`. Angular hooks are prefixed with `ng`, such as `ngOnInit`.
 * * AngularJS hooks can be defined on the controller prototype or added to the controller inside its constructor.
 *   In Angular you can only define hooks on the prototype of the Component class.
 * * Due to the differences in change-detection, you may get many more calls to `$doCheck` in AngularJS than you would to
 *   `ngDoCheck` in Angular.
 * * Changes to the model inside `$doCheck` will trigger new turns of the digest loop, which will cause the changes to be
 *   propagated throughout the application.
 *   Angular does not allow the `ngDoCheck` hook to trigger a change outside of the component. It will either throw an
 *   error or do nothing depending upon the state of `enableProdMode()`.
 *
 * #### Life-cycle hook examples
 *
 * This example shows how you can check for mutations to a Date object even though the identity of the object
 * has not changed.
 *
 * <example name="doCheckDateExample" module="do-check-module">
 *   <file name="app.js">
 *     angular.module('do-check-module', [])
 *       .component('app', {
 *         template:
 *           'Month: <input ng-model="$ctrl.month" ng-change="$ctrl.updateDate()">' +
 *           'Date: {{ $ctrl.date }}' +
 *           '<test date="$ctrl.date"></test>',
 *         controller: function() {
 *           this.date = new Date();
 *           this.month = this.date.getMonth();
 *           this.updateDate = function() {
 *             this.date.setMonth(this.month);
 *           };
 *         }
 *       })
 *       .component('test', {
 *         bindings: { date: '<' },
 *         template:
 *           '<pre>{{ $ctrl.log | json }}</pre>',
 *         controller: function() {
 *           var previousValue;
 *           this.log = [];
 *           this.$doCheck = function() {
 *             var currentValue = this.date && this.date.valueOf();
 *             if (previousValue !== currentValue) {
 *               this.log.push('doCheck: date mutated: ' + this.date);
 *               previousValue = currentValue;
 *             }
 *           };
 *         }
 *       });
 *   </file>
 *   <file name="index.html">
 *     <app></app>
 *   </file>
 * </example>
 *
 * This example show how you might use `$doCheck` to trigger changes in your component's inputs even if the
 * actual identity of the component doesn't change. (Be aware that cloning and deep equality checks on large
 * arrays or objects can have a negative impact on your application performance.)
 *
 * <example name="doCheckArrayExample" module="do-check-module">
 *   <file name="index.html">
 *     <div ng-init="items = []">
 *       <button ng-click="items.push(items.length)">Add Item</button>
 *       <button ng-click="items = []">Reset Items</button>
 *       <pre>{{ items }}</pre>
 *       <test items="items"></test>
 *     </div>
 *   </file>
 *   <file name="app.js">
 *      angular.module('do-check-module', [])
 *        .component('test', {
 *          bindings: { items: '<' },
 *          template:
 *            '<pre>{{ $ctrl.log | json }}</pre>',
 *          controller: function() {
 *            this.log = [];
 *
 *            this.$doCheck = function() {
 *              if (this.items_ref !== this.items) {
 *                this.log.push('doCheck: items changed');
 *                this.items_ref = this.items;
 *              }
 *              if (!angular.equals(this.items_clone, this.items)) {
 *                this.log.push('doCheck: items mutated');
 *                this.items_clone = angular.copy(this.items);
 *              }
 *            };
 *          }
 *        });
 *   </file>
 * </example>
 *
 *
 * ### Directive Definition Object
 *
 * The directive definition object provides instructions to the {@link ng.$compile
 * compiler}. The attributes are:
 *
 * #### `multiElement`
 * When this property is set to true (default is `false`), the HTML compiler will collect DOM nodes between
 * nodes with the attributes `directive-name-start` and `directive-name-end`, and group them
 * together as the directive elements. It is recommended that this feature be used on directives
 * which are not strictly behavioral (such as {@link ngClick}), and which
 * do not manipulate or replace child nodes (such as {@link ngInclude}).
 *
 * #### `priority`
 * When there are multiple directives defined on a single DOM element, sometimes it
 * is necessary to specify the order in which the directives are applied. The `priority` is used
 * to sort the directives before their `compile` functions get called. Priority is defined as a
 * number. Directives with greater numerical `priority` are compiled first. Pre-link functions
 * are also run in priority order, but post-link functions are run in reverse order. The order
 * of directives with the same priority is undefined. The default priority is `0`.
 *
 * #### `terminal`
 * If set to true then the current `priority` will be the last set of directives
 * which will execute (any directives at the current priority will still execute
 * as the order of execution on same `priority` is undefined). Note that expressions
 * and other directives used in the directive's template will also be excluded from execution.
 *
 * #### `scope`
 * The scope property can be `false`, `true`, or an object:
 *
 * * **`false` (default):** No scope will be created for the directive. The directive will use its
 * parent's scope.
 *
 * * **`true`:** A new child scope that prototypically inherits from its parent will be created for
 * the directive's element. If multiple directives on the same element request a new scope,
 * only one new scope is created.
 *
 * * **`{...}` (an object hash):** A new "isolate" scope is created for the directive's template.
 * The 'isolate' scope differs from normal scope in that it does not prototypically
 * inherit from its parent scope. This is useful when creating reusable components, which should not
 * accidentally read or modify data in the parent scope. Note that an isolate scope
 * directive without a `template` or `templateUrl` will not apply the isolate scope
 * to its children elements.
 *
 * The 'isolate' scope object hash defines a set of local scope properties derived from attributes on the
 * directive's element. These local properties are useful for aliasing values for templates. The keys in
 * the object hash map to the name of the property on the isolate scope; the values define how the property
 * is bound to the parent scope, via matching attributes on the directive's element:
 *
 * * `@` or `@attr` - bind a local scope property to the value of DOM attribute. The result is
 *   always a string since DOM attributes are strings. If no `attr` name is specified then the
 *   attribute name is assumed to be the same as the local name. Given `<my-component
 *   my-attr="hello {{name}}">` and the isolate scope definition `scope: { localName:'@myAttr' }`,
 *   the directive's scope property `localName` will reflect the interpolated value of `hello
 *   {{name}}`. As the `name` attribute changes so will the `localName` property on the directive's
 *   scope. The `name` is read from the parent scope (not the directive's scope).
 *
 * * `=` or `=attr` - set up a bidirectional binding between a local scope property and an expression
 *   passed via the attribute `attr`. The expression is evaluated in the context of the parent scope.
 *   If no `attr` name is specified then the attribute name is assumed to be the same as the local
 *   name. Given `<my-component my-attr="parentModel">` and the isolate scope definition `scope: {
 *   localModel: '=myAttr' }`, the property `localModel` on the directive's scope will reflect the
 *   value of `parentModel` on the parent scope. Changes to `parentModel` will be reflected in
 *   `localModel` and vice versa. If the binding expression is non-assignable, or if the attribute
 *   isn't  optional and doesn't exist, an exception
 *   ({@link error/$compile/nonassign `$compile:nonassign`}) will be thrown upon discovering changes
 *   to the local value, since it will be impossible to sync them back to the parent scope.
 *
 *   By default, the {@link ng.$rootScope.Scope#$watch `$watch`}
 *   method is used for tracking changes, and the equality check is based on object identity.
 *   However, if an object literal or an array literal is passed as the binding expression, the
 *   equality check is done by value (using the {@link angular.equals} function). It's also possible
 *   to watch the evaluated value shallowly with {@link ng.$rootScope.Scope#$watchCollection
 *   `$watchCollection`}: use `=*` or `=*attr`
 *
  * * `<` or `<attr` - set up a one-way (one-directional) binding between a local scope property and an
 *   expression passed via the attribute `attr`. The expression is evaluated in the context of the
 *   parent scope. If no `attr` name is specified then the attribute name is assumed to be the same as the
 *   local name.
 *
 *   For example, given `<my-component my-attr="parentModel">` and directive definition of
 *   `scope: { localModel:'<myAttr' }`, then the isolated scope property `localModel` will reflect the
 *   value of `parentModel` on the parent scope. Any changes to `parentModel` will be reflected
 *   in `localModel`, but changes in `localModel` will not reflect in `parentModel`. There are however
 *   two caveats:
 *     1. one-way binding does not copy the value from the parent to the isolate scope, it simply
 *     sets the same value. That means if your bound value is an object, changes to its properties
 *     in the isolated scope will be reflected in the parent scope (because both reference the same object).
 *     2. one-way binding watches changes to the **identity** of the parent value. That means the
 *     {@link ng.$rootScope.Scope#$watch `$watch`} on the parent value only fires if the reference
 *     to the value has changed. In most cases, this should not be of concern, but can be important
 *     to know if you one-way bind to an object, and then replace that object in the isolated scope.
 *     If you now change a property of the object in your parent scope, the change will not be
 *     propagated to the isolated scope, because the identity of the object on the parent scope
 *     has not changed. Instead you must assign a new object.
 *
 *   One-way binding is useful if you do not plan to propagate changes to your isolated scope bindings
 *   back to the parent. However, it does not make this completely impossible.
 *
 *   By default, the {@link ng.$rootScope.Scope#$watch `$watch`}
 *   method is used for tracking changes, and the equality check is based on object identity.
 *   It's also possible to watch the evaluated value shallowly with
 *   {@link ng.$rootScope.Scope#$watchCollection `$watchCollection`}: use `<*` or `<*attr`
 *
 * * `&` or `&attr` - provides a way to execute an expression in the context of the parent scope. If
 *   no `attr` name is specified then the attribute name is assumed to be the same as the local name.
 *   Given `<my-component my-attr="count = count + value">` and the isolate scope definition `scope: {
 *   localFn:'&myAttr' }`, the isolate scope property `localFn` will point to a function wrapper for
 *   the `count = count + value` expression. Often it's desirable to pass data from the isolated scope
 *   via an expression to the parent scope. This can be done by passing a map of local variable names
 *   and values into the expression wrapper fn. For example, if the expression is `increment(amount)`
 *   then we can specify the amount value by calling the `localFn` as `localFn({amount: 22})`.
 *
 * All 4 kinds of bindings (`@`, `=`, `<`, and `&`) can be made optional by adding `?` to the expression.
 * The marker must come after the mode and before the attribute name.
 * See the {@link error/$compile/iscp Invalid Isolate Scope Definition error} for definition examples.
 * This is useful to refine the interface directives provide.
 * One subtle difference between optional and non-optional happens **when the binding attribute is not
 * set**:
 * - the binding is optional: the property will not be defined
 * - the binding is not optional: the property is defined
 *
 * ```js
 *app.directive('testDir', function() {
    return {
      scope: {
        notoptional: '=',
        optional: '=?',
      },
      bindToController: true,
      controller: function() {
        this.$onInit = function() {
          console.log(this.hasOwnProperty('notoptional')) // true
          console.log(this.hasOwnProperty('optional')) // false
        }
      }
    }
  })
 *```
 *
 *
 * ##### Combining directives with different scope defintions
 *
 * In general it's possible to apply more than one directive to one element, but there might be limitations
 * depending on the type of scope required by the directives. The following points will help explain these limitations.
 * For simplicity only two directives are taken into account, but it is also applicable for several directives:
 *
 * * **no scope** + **no scope** => Two directives which don't require their own scope will use their parent's scope
 * * **child scope** + **no scope** =>  Both directives will share one single child scope
 * * **child scope** + **child scope** =>  Both directives will share one single child scope
 * * **isolated scope** + **no scope** =>  The isolated directive will use it's own created isolated scope. The other directive will use
 * its parent's scope
 * * **isolated scope** + **child scope** =>  **Won't work!** Only one scope can be related to one element. Therefore these directives cannot
 * be applied to the same element.
 * * **isolated scope** + **isolated scope**  =>  **Won't work!** Only one scope can be related to one element. Therefore these directives
 * cannot be applied to the same element.
 *
 *
 * #### `bindToController`
 * This property is used to bind scope properties directly to the controller. It can be either
 * `true` or an object hash with the same format as the `scope` property.
 *
 * When an isolate scope is used for a directive (see above), `bindToController: true` will
 * allow a component to have its properties bound to the controller, rather than to scope.
 *
 * After the controller is instantiated, the initial values of the isolate scope bindings will be bound to the controller
 * properties. You can access these bindings once they have been initialized by providing a controller method called
 * `$onInit`, which is called after all the controllers on an element have been constructed and had their bindings
 * initialized.
 *
 * It is also possible to set `bindToController` to an object hash with the same format as the `scope` property.
 * This will set up the scope bindings to the controller directly. Note that `scope` can still be used
 * to define which kind of scope is created. By default, no scope is created. Use `scope: {}` to create an isolate
 * scope (useful for component directives).
 *
 * If both `bindToController` and `scope` are defined and have object hashes, `bindToController` overrides `scope`.
 *
 *
 * #### `controller`
 * Controller constructor function. The controller is instantiated before the
 * pre-linking phase and can be accessed by other directives (see
 * `require` attribute). This allows the directives to communicate with each other and augment
 * each other's behavior. The controller is injectable (and supports bracket notation) with the following locals:
 *
 * * `$scope` - Current scope associated with the element
 * * `$element` - Current element
 * * `$attrs` - Current attributes object for the element
 * * `$transclude` - A transclude linking function pre-bound to the correct transclusion scope:
 *   `function([scope], cloneLinkingFn, futureParentElement, slotName)`:
 *    * `scope`: (optional) override the scope.
 *    * `cloneLinkingFn`: (optional) argument to create clones of the original transcluded content.
 *    * `futureParentElement` (optional):
 *        * defines the parent to which the `cloneLinkingFn` will add the cloned elements.
 *        * default: `$element.parent()` resp. `$element` for `transclude:'element'` resp. `transclude:true`.
 *        * only needed for transcludes that are allowed to contain non html elements (e.g. SVG elements)
 *          and when the `cloneLinkingFn` is passed,
 *          as those elements need to created and cloned in a special way when they are defined outside their
 *          usual containers (e.g. like `<svg>`).
 *        * See also the `directive.templateNamespace` property.
 *    * `slotName`: (optional) the name of the slot to transclude. If falsy (e.g. `null`, `undefined` or `''`)
 *      then the default transclusion is provided.
 *    The `$transclude` function also has a method on it, `$transclude.isSlotFilled(slotName)`, which returns
 *    `true` if the specified slot contains content (i.e. one or more DOM nodes).
 *
 * #### `require`
 * Require another directive and inject its controller as the fourth argument to the linking function. The
 * `require` property can be a string, an array or an object:
 * * a **string** containing the name of the directive to pass to the linking function
 * * an **array** containing the names of directives to pass to the linking function. The argument passed to the
 * linking function will be an array of controllers in the same order as the names in the `require` property
 * * an **object** whose property values are the names of the directives to pass to the linking function. The argument
 * passed to the linking function will also be an object with matching keys, whose values will hold the corresponding
 * controllers.
 *
 * If the `require` property is an object and `bindToController` is truthy, then the required controllers are
 * bound to the controller using the keys of the `require` property. This binding occurs after all the controllers
 * have been constructed but before `$onInit` is called.
 * If the name of the required controller is the same as the local name (the key), the name can be
 * omitted. For example, `{parentDir: '^^'}` is equivalent to `{parentDir: '^^parentDir'}`.
 * See the {@link $compileProvider#component} helper for an example of how this can be used.
 * If no such required directive(s) can be found, or if the directive does not have a controller, then an error is
 * raised (unless no link function is specified and the required controllers are not being bound to the directive
 * controller, in which case error checking is skipped). The name can be prefixed with:
 *
 * * (no prefix) - Locate the required controller on the current element. Throw an error if not found.
 * * `?` - Attempt to locate the required controller or pass `null` to the `link` fn if not found.
 * * `^` - Locate the required controller by searching the element and its parents. Throw an error if not found.
 * * `^^` - Locate the required controller by searching the element's parents. Throw an error if not found.
 * * `?^` - Attempt to locate the required controller by searching the element and its parents or pass
 *   `null` to the `link` fn if not found.
 * * `?^^` - Attempt to locate the required controller by searching the element's parents, or pass
 *   `null` to the `link` fn if not found.
 *
 *
 * #### `controllerAs`
 * Identifier name for a reference to the controller in the directive's scope.
 * This allows the controller to be referenced from the directive template. This is especially
 * useful when a directive is used as component, i.e. with an `isolate` scope. It's also possible
 * to use it in a directive without an `isolate` / `new` scope, but you need to be aware that the
 * `controllerAs` reference might overwrite a property that already exists on the parent scope.
 *
 *
 * #### `restrict`
 * String of subset of `EACM` which restricts the directive to a specific directive
 * declaration style. If omitted, the defaults (elements and attributes) are used.
 *
 * * `E` - Element name (default): `<my-directive></my-directive>`
 * * `A` - Attribute (default): `<div my-directive="exp"></div>`
 * * `C` - Class: `<div class="my-directive: exp;"></div>`
 * * `M` - Comment: `<!-- directive: my-directive exp -->`
 *
 *
 * #### `templateNamespace`
 * String representing the document type used by the markup in the template.
 * AngularJS needs this information as those elements need to be created and cloned
 * in a special way when they are defined outside their usual containers like `<svg>` and `<math>`.
 *
 * * `html` - All root nodes in the template are HTML. Root nodes may also be
 *   top-level elements such as `<svg>` or `<math>`.
 * * `svg` - The root nodes in the template are SVG elements (excluding `<math>`).
 * * `math` - The root nodes in the template are MathML elements (excluding `<svg>`).
 *
 * If no `templateNamespace` is specified, then the namespace is considered to be `html`.
 *
 * #### `template`
 * HTML markup that may:
 * * Replace the contents of the directive's element (default).
 * * Replace the directive's element itself (if `replace` is true - DEPRECATED).
 * * Wrap the contents of the directive's element (if `transclude` is true).
 *
 * Value may be:
 *
 * * A string. For example `<div red-on-hover>{{delete_str}}</div>`.
 * * A function which takes two arguments `tElement` and `tAttrs` (described in the `compile`
 *   function api below) and returns a string value.
 *
 *
 * #### `templateUrl`
 * This is similar to `template` but the template is loaded from the specified URL, asynchronously.
 *
 * Because template loading is asynchronous the compiler will suspend compilation of directives on that element
 * for later when the template has been resolved.  In the meantime it will continue to compile and link
 * sibling and parent elements as though this element had not contained any directives.
 *
 * The compiler does not suspend the entire compilation to wait for templates to be loaded because this
 * would result in the whole app "stalling" until all templates are loaded asynchronously - even in the
 * case when only one deeply nested directive has `templateUrl`.
 *
 * Template loading is asynchronous even if the template has been preloaded into the {@link $templateCache}.
 *
 * You can specify `templateUrl` as a string representing the URL or as a function which takes two
 * arguments `tElement` and `tAttrs` (described in the `compile` function api below) and returns
 * a string value representing the url.  In either case, the template URL is passed through {@link
 * $sce#getTrustedResourceUrl $sce.getTrustedResourceUrl}.
 *
 *
 * #### `replace`
 * <div class="alert alert-danger">
 * **Note:** `replace` is deprecated in AngularJS and has been removed in the new Angular (v2+).
 * </div>
 *
 * Specifies what the template should replace. Defaults to `false`.
 *
 * * `true` - the template will replace the directive's element.
 * * `false` - the template will replace the contents of the directive's element.
 *
 * The replacement process migrates all of the attributes / classes from the old element to the new
 * one. See the {@link guide/directive#template-expanding-directive
 * Directives Guide} for an example.
 *
 * There are very few scenarios where element replacement is required for the application function,
 * the main one being reusable custom components that are used within SVG contexts
 * (because SVG doesn't work with custom elements in the DOM tree).
 *
 * #### `transclude`
 * Extract the contents of the element where the directive appears and make it available to the directive.
 * The contents are compiled and provided to the directive as a **transclusion function**. See the
 * {@link $compile#transclusion Transclusion} section below.
 *
 *
 * #### `compile`
 *
 * ```js
 *   function compile(tElement, tAttrs, transclude) { ... }
 * ```
 *
 * The compile function deals with transforming the template DOM. Since most directives do not do
 * template transformation, it is not used often. The compile function takes the following arguments:
 *
 *   * `tElement` - template element - The element where the directive has been declared. It is
 *     safe to do template transformation on the element and child elements only.
 *
 *   * `tAttrs` - template attributes - Normalized list of attributes declared on this element shared
 *     between all directive compile functions.
 *
 *   * `transclude` -  [*DEPRECATED*!] A transclude linking function: `function(scope, cloneLinkingFn)`
 *
 * <div class="alert alert-warning">
 * **Note:** The template instance and the link instance may be different objects if the template has
 * been cloned. For this reason it is **not** safe to do anything other than DOM transformations that
 * apply to all cloned DOM nodes within the compile function. Specifically, DOM listener registration
 * should be done in a linking function rather than in a compile function.
 * </div>

 * <div class="alert alert-warning">
 * **Note:** The compile function cannot handle directives that recursively use themselves in their
 * own templates or compile functions. Compiling these directives results in an infinite loop and
 * stack overflow errors.
 *
 * This can be avoided by manually using `$compile` in the postLink function to imperatively compile
 * a directive's template instead of relying on automatic template compilation via `template` or
 * `templateUrl` declaration or manual compilation inside the compile function.
 * </div>
 *
 * <div class="alert alert-danger">
 * **Note:** The `transclude` function that is passed to the compile function is deprecated, as it
 *   e.g. does not know about the right outer scope. Please use the transclude function that is passed
 *   to the link function instead.
 * </div>

 * A compile function can have a return value which can be either a function or an object.
 *
 * * returning a (post-link) function - is equivalent to registering the linking function via the
 *   `link` property of the config object when the compile function is empty.
 *
 * * returning an object with function(s) registered via `pre` and `post` properties - allows you to
 *   control when a linking function should be called during the linking phase. See info about
 *   pre-linking and post-linking functions below.
 *
 *
 * #### `link`
 * This property is used only if the `compile` property is not defined.
 *
 * ```js
 *   function link(scope, iElement, iAttrs, controller, transcludeFn) { ... }
 * ```
 *
 * The link function is responsible for registering DOM listeners as well as updating the DOM. It is
 * executed after the template has been cloned. This is where most of the directive logic will be
 * put.
 *
 *   * `scope` - {@link ng.$rootScope.Scope Scope} - The scope to be used by the
 *     directive for registering {@link ng.$rootScope.Scope#$watch watches}.
 *
 *   * `iElement` - instance element - The element where the directive is to be used. It is safe to
 *     manipulate the children of the element only in `postLink` function since the children have
 *     already been linked.
 *
 *   * `iAttrs` - instance attributes - Normalized list of attributes declared on this element shared
 *     between all directive linking functions.
 *
 *   * `controller` - the directive's required controller instance(s) - Instances are shared
 *     among all directives, which allows the directives to use the controllers as a communication
 *     channel. The exact value depends on the directive's `require` property:
 *       * no controller(s) required: the directive's own controller, or `undefined` if it doesn't have one
 *       * `string`: the controller instance
 *       * `array`: array of controller instances
 *
 *     If a required controller cannot be found, and it is optional, the instance is `null`,
 *     otherwise the {@link error:$compile:ctreq Missing Required Controller} error is thrown.
 *
 *     Note that you can also require the directive's own controller - it will be made available like
 *     any other controller.
 *
 *   * `transcludeFn` - A transclude linking function pre-bound to the correct transclusion scope.
 *     This is the same as the `$transclude` parameter of directive controllers,
 *     see {@link ng.$compile#-controller- the controller section for details}.
 *     `function([scope], cloneLinkingFn, futureParentElement)`.
 *
 * #### Pre-linking function
 *
 * Executed before the child elements are linked. Not safe to do DOM transformation since the
 * compiler linking function will fail to locate the correct elements for linking.
 *
 * #### Post-linking function
 *
 * Executed after the child elements are linked.
 *
 * Note that child elements that contain `templateUrl` directives will not have been compiled
 * and linked since they are waiting for their template to load asynchronously and their own
 * compilation and linking has been suspended until that occurs.
 *
 * It is safe to do DOM transformation in the post-linking function on elements that are not waiting
 * for their async templates to be resolved.
 *
 *
 * ### Transclusion
 *
 * Transclusion is the process of extracting a collection of DOM elements from one part of the DOM and
 * copying them to another part of the DOM, while maintaining their connection to the original AngularJS
 * scope from where they were taken.
 *
 * Transclusion is used (often with {@link ngTransclude}) to insert the
 * original contents of a directive's element into a specified place in the template of the directive.
 * The benefit of transclusion, over simply moving the DOM elements manually, is that the transcluded
 * content has access to the properties on the scope from which it was taken, even if the directive
 * has isolated scope.
 * See the {@link guide/directive#creating-a-directive-that-wraps-other-elements Directives Guide}.
 *
 * This makes it possible for the widget to have private state for its template, while the transcluded
 * content has access to its originating scope.
 *
 * <div class="alert alert-warning">
 * **Note:** When testing an element transclude directive you must not place the directive at the root of the
 * DOM fragment that is being compiled. See {@link guide/unit-testing#testing-transclusion-directives
 * Testing Transclusion Directives}.
 * </div>
 *
 * There are three kinds of transclusion depending upon whether you want to transclude just the contents of the
 * directive's element, the entire element or multiple parts of the element contents:
 *
 * * `true` - transclude the content (i.e. the child nodes) of the directive's element.
 * * `'element'` - transclude the whole of the directive's element including any directives on this
 *   element that are defined at a lower priority than this directive. When used, the `template`
 *   property is ignored.
 * * **`{...}` (an object hash):** - map elements of the content onto transclusion "slots" in the template.
 *
 * **Multi-slot transclusion** is declared by providing an object for the `transclude` property.
 *
 * This object is a map where the keys are the name of the slot to fill and the value is an element selector
 * used to match the HTML to the slot. The element selector should be in normalized form (e.g. `myElement`)
 * and will match the standard element variants (e.g. `my-element`, `my:element`, `data-my-element`, etc).
 *
 * For further information check out the guide on {@link guide/directive#matching-directives Matching Directives}.
 *
 * If the element selector is prefixed with a `?` then that slot is optional.
 *
 * For example, the transclude object `{ slotA: '?myCustomElement' }` maps `<my-custom-element>` elements to
 * the `slotA` slot, which can be accessed via the `$transclude` function or via the {@link ngTransclude} directive.
 *
 * Slots that are not marked as optional (`?`) will trigger a compile time error if there are no matching elements
 * in the transclude content. If you wish to know if an optional slot was filled with content, then you can call
 * `$transclude.isSlotFilled(slotName)` on the transclude function passed to the directive's link function and
 * injectable into the directive's controller.
 *
 *
 * #### Transclusion Functions
 *
 * When a directive requests transclusion, the compiler extracts its contents and provides a **transclusion
 * function** to the directive's `link` function and `controller`. This transclusion function is a special
 * **linking function** that will return the compiled contents linked to a new transclusion scope.
 *
 * <div class="alert alert-info">
 * If you are just using {@link ngTransclude} then you don't need to worry about this function, since
 * ngTransclude will deal with it for us.
 * </div>
 *
 * If you want to manually control the insertion and removal of the transcluded content in your directive
 * then you must use this transclude function. When you call a transclude function it returns a jqLite/JQuery
 * object that contains the compiled DOM, which is linked to the correct transclusion scope.
 *
 * When you call a transclusion function you can pass in a **clone attach function**. This function accepts
 * two parameters, `function(clone, scope) { ... }`, where the `clone` is a fresh compiled copy of your transcluded
 * content and the `scope` is the newly created transclusion scope, which the clone will be linked to.
 *
 * <div class="alert alert-info">
 * **Best Practice**: Always provide a `cloneFn` (clone attach function) when you call a transclude function
 * since you then get a fresh clone of the original DOM and also have access to the new transclusion scope.
 * </div>
 *
 * It is normal practice to attach your transcluded content (`clone`) to the DOM inside your **clone
 * attach function**:
 *
 * ```js
 * var transcludedContent, transclusionScope;
 *
 * $transclude(function(clone, scope) {
 *   element.append(clone);
 *   transcludedContent = clone;
 *   transclusionScope = scope;
 * });
 * ```
 *
 * Later, if you want to remove the transcluded content from your DOM then you should also destroy the
 * associated transclusion scope:
 *
 * ```js
 * transcludedContent.remove();
 * transclusionScope.$destroy();
 * ```
 *
 * <div class="alert alert-info">
 * **Best Practice**: if you intend to add and remove transcluded content manually in your directive
 * (by calling the transclude function to get the DOM and calling `element.remove()` to remove it),
 * then you are also responsible for calling `$destroy` on the transclusion scope.
 * </div>
 *
 * The built-in DOM manipulation directives, such as {@link ngIf}, {@link ngSwitch} and {@link ngRepeat}
 * automatically destroy their transcluded clones as necessary so you do not need to worry about this if
 * you are simply using {@link ngTransclude} to inject the transclusion into your directive.
 *
 *
 * #### Transclusion Scopes
 *
 * When you call a transclude function it returns a DOM fragment that is pre-bound to a **transclusion
 * scope**. This scope is special, in that it is a child of the directive's scope (and so gets destroyed
 * when the directive's scope gets destroyed) but it inherits the properties of the scope from which it
 * was taken.
 *
 * For example consider a directive that uses transclusion and isolated scope. The DOM hierarchy might look
 * like this:
 *
 * ```html
 * <div ng-app>
 *   <div isolate>
 *     <div transclusion>
 *     </div>
 *   </div>
 * </div>
 * ```
 *
 * The `$parent` scope hierarchy will look like this:
 *
   ```
   - $rootScope
     - isolate
       - transclusion
   ```
 *
 * but the scopes will inherit prototypically from different scopes to their `$parent`.
 *
   ```
   - $rootScope
     - transclusion
   - isolate
   ```
 *
 *
 * ### Attributes
 *
 * The {@link ng.$compile.directive.Attributes Attributes} object - passed as a parameter in the
 * `link()` or `compile()` functions. It has a variety of uses.
 *
 * * *Accessing normalized attribute names:* Directives like `ngBind` can be expressed in many ways:
 *   `ng:bind`, `data-ng-bind`, or `x-ng-bind`. The attributes object allows for normalized access
 *   to the attributes.
 *
 * * *Directive inter-communication:* All directives share the same instance of the attributes
 *   object which allows the directives to use the attributes object as inter directive
 *   communication.
 *
 * * *Supports interpolation:* Interpolation attributes are assigned to the attribute object
 *   allowing other directives to read the interpolated value.
 *
 * * *Observing interpolated attributes:* Use `$observe` to observe the value changes of attributes
 *   that contain interpolation (e.g. `src="{{bar}}"`). Not only is this very efficient but it's also
 *   the only way to easily get the actual value because during the linking phase the interpolation
 *   hasn't been evaluated yet and so the value is at this time set to `undefined`.
 *
 * ```js
 * function linkingFn(scope, elm, attrs, ctrl) {
 *   // get the attribute value
 *   console.log(attrs.ngModel);
 *
 *   // change the attribute
 *   attrs.$set('ngModel', 'new value');
 *
 *   // observe changes to interpolated attribute
 *   attrs.$observe('ngModel', function(value) {
 *     console.log('ngModel has changed value to ' + value);
 *   });
 * }
 * ```
 *
 * ## Example
 *
 * <div class="alert alert-warning">
 * **Note**: Typically directives are registered with `module.directive`. The example below is
 * to illustrate how `$compile` works.
 * </div>
 *
 <example module="compileExample" name="compile">
   <file name="index.html">
    <script>
      angular.module('compileExample', [], function($compileProvider) {
        // Configure new 'compile' directive by passing a directive
        // factory function. The factory function injects '$compile'.
        $compileProvider.directive('compile', function($compile) {
          // The directive factory creates a link function.
          return function(scope, element, attrs) {
            scope.$watch(
              function(scope) {
                // Watch the 'compile' expression for changes.
                return scope.$eval(attrs.compile);
              },
              function(value) {
                // When the 'compile' expression changes
                // assign it into the current DOM.
                element.html(value);

                // Compile the new DOM and link it to the current scope.
                // NOTE: we only compile '.childNodes' so that we
                // don't get into an infinite loop compiling ourselves.
                $compile(element.contents())(scope);
              }
            );
          };
        });
      })
      .controller('GreeterController', ['$scope', function($scope) {
        $scope.name = 'AngularJS';
        $scope.html = 'Hello {{name}}';
      }]);
    </script>
    <div ng-controller="GreeterController">
      <input ng-model="name"> <br/>
      <textarea ng-model="html"></textarea> <br/>
      <div compile="html"></div>
    </div>
   </file>
   <file name="protractor.js" type="protractor">
     it('should auto compile', function() {
       var textarea = $('textarea');
       var output = $('div[compile]');
       // The initial state reads 'Hello AngularJS'.
       expect(output.getText()).toBe('Hello AngularJS');
       textarea.clear();
       textarea.sendKeys('{{name}}!');
       expect(output.getText()).toBe('AngularJS!');
     });
   </file>
 </example>

 *
 *
 * @param {string|DOMElement} element Element or HTML string to compile into a template function.
 * @param {function(angular.Scope, cloneAttachFn=)} transclude function available to directives - DEPRECATED.
 *
 * <div class="alert alert-danger">
 * **Note:** Passing a `transclude` function to the $compile function is deprecated, as it
 *   e.g. will not use the right outer scope. Please pass the transclude function as a
 *   `parentBoundTranscludeFn` to the link function instead.
 * </div>
 *
 * @param {number} maxPriority only apply directives lower than given priority (Only effects the
 *                 root element(s), not their children)
 * @returns {function(scope, cloneAttachFn=, options=)} a link function which is used to bind template
 * (a DOM element/tree) to a scope. Where:
 *
 *  * `scope` - A {@link ng.$rootScope.Scope Scope} to bind to.
 *  * `cloneAttachFn` - If `cloneAttachFn` is provided, then the link function will clone the
 *  `template` and call the `cloneAttachFn` function allowing the caller to attach the
 *  cloned elements to the DOM document at the appropriate place. The `cloneAttachFn` is
 *  called as: <br/> `cloneAttachFn(clonedElement, scope)` where:
 *
 *      * `clonedElement` - is a clone of the original `element` passed into the compiler.
 *      * `scope` - is the current scope with which the linking function is working with.
 *
 *  * `options` - An optional object hash with linking options. If `options` is provided, then the following
 *  keys may be used to control linking behavior:
 *
 *      * `parentBoundTranscludeFn` - the transclude function made available to
 *        directives; if given, it will be passed through to the link functions of
 *        directives found in `element` during compilation.
 *      * `transcludeControllers` - an object hash with keys that map controller names
 *        to a hash with the key `instance`, which maps to the controller instance;
 *        if given, it will make the controllers available to directives on the compileNode:
 *        ```
 *        {
 *          parent: {
 *            instance: parentControllerInstance
 *          }
 *        }
 *        ```
 *      * `futureParentElement` - defines the parent to which the `cloneAttachFn` will add
 *        the cloned elements; only needed for transcludes that are allowed to contain non HTML
 *        elements (e.g. SVG elements). See also the `directive.controller` property.
 *
 * Calling the linking function returns the element of the template. It is either the original
 * element passed in, or the clone of the element if the `cloneAttachFn` is provided.
 *
 * After linking the view is not updated until after a call to `$digest`, which typically is done by
 * AngularJS automatically.
 *
 * If you need access to the bound view, there are two ways to do it:
 *
 * - If you are not asking the linking function to clone the template, create the DOM element(s)
 *   before you send them to the compiler and keep this reference around.
 *   ```js
 *     var element = angular.element('<p>{{total}}</p>');
 *     $compile(element)(scope);
 *   ```
 *
 * - if on the other hand, you need the element to be cloned, the view reference from the original
 *   example would not point to the clone, but rather to the original template that was cloned. In
 *   this case, you can access the clone either via the `cloneAttachFn` or the value returned by the
 *   linking function:
 *   ```js
 *     var templateElement = angular.element('<p>{{total}}</p>');
 *     var clonedElement = $compile(templateElement)(scope, function(clonedElement, scope) {
 *       // Attach the clone to DOM document at the right place.
 *     });
 *
 *     // Now we have reference to the cloned DOM via `clonedElement`.
 *     // NOTE: The `clonedElement` returned by the linking function is the same as the
 *     //       `clonedElement` passed to `cloneAttachFn`.
 *   ```
 *
 *
 * For information on how the compiler works, see the
 * {@link guide/compiler AngularJS HTML Compiler} section of the Developer Guide.
 *
 * @knownIssue
 *
 * ### Double Compilation
 *
   Double compilation occurs when an already compiled part of the DOM gets
   compiled again. This is an undesired effect and can lead to misbehaving directives, performance issues,
   and memory leaks. Refer to the Compiler Guide {@link guide/compiler#double-compilation-and-how-to-avoid-it
   section on double compilation} for an in-depth explanation and ways to avoid it.

 * @knownIssue

   ### Issues with `replace: true`
 *
 * <div class="alert alert-danger">
 *   **Note**: {@link $compile#-replace- `replace: true`} is deprecated and not recommended to use,
 *   mainly due to the issues listed here. It has been completely removed in the new Angular.
 * </div>
 *
 * #### Attribute values are not merged
 *
 * When a `replace` directive encounters the same attribute on the original and the replace node,
 * it will simply deduplicate the attribute and join the values with a space or with a `;` in case of
 * the `style` attribute.
 * ```html
 * Original Node: <span class="original" style="color: red;"></span>
 * Replace Template: <span class="replaced" style="background: blue;"></span>
 * Result: <span class="original replaced" style="color: red; background: blue;"></span>
 * ```
 *
 * That means attributes that contain AngularJS expressions will not be merged correctly, e.g.
 * {@link ngShow} or {@link ngClass} will cause a {@link $parse} error:
 *
 * ```html
 * Original Node: <span ng-class="{'something': something}" ng-show="!condition"></span>
 * Replace Template: <span ng-class="{'else': else}" ng-show="otherCondition"></span>
 * Result: <span ng-class="{'something': something} {'else': else}" ng-show="!condition otherCondition"></span>
 * ```
 *
 * See issue [#5695](https://github.com/angular/angular.js/issues/5695).
 *
 * #### Directives are not deduplicated before compilation
 *
 * When the original node and the replace template declare the same directive(s), they will be
 * {@link guide/compiler#double-compilation-and-how-to-avoid-it compiled twice} because the compiler
 * does not deduplicate them. In many cases, this is not noticeable, but e.g. {@link ngModel} will
 * attach `$formatters` and `$parsers` twice.
 *
 * See issue [#2573](https://github.com/angular/angular.js/issues/2573).
 *
 * #### `transclude: element` in the replace template root can have unexpected effects
 *
 * When the replace template has a directive at the root node that uses
 * {@link $compile#-transclude- `transclude: element`}, e.g.
 * {@link ngIf} or {@link ngRepeat}, the DOM structure or scope inheritance can be incorrect.
 * See the following issues:
 *
 * - Incorrect scope on replaced element:
 * [#9837](https://github.com/angular/angular.js/issues/9837)
 * - Different DOM between `template` and `templateUrl`:
 * [#10612](https://github.com/angular/angular.js/issues/14326)
 *
 */

/**
 * @ngdoc directive
 * @name ngProp
 * @restrict A
 * @element ANY
 *
 * @usage
 *
 * ```html
 * <ANY ng-prop-propname="expression">
 * </ANY>
 * ```
 *
 * or with uppercase letters in property (e.g. "propName"):
 *
 *
 * ```html
 * <ANY ng-prop-prop_name="expression">
 * </ANY>
 * ```
 *
 *
 * @description
 * The `ngProp` directive binds an expression to a DOM element property.
 * `ngProp` allows writing to arbitrary properties by including
 * the property name in the attribute, e.g. `ng-prop-value="'my value'"` binds 'my value' to
 * the `value` property.
 *
 * Usually, it's not necessary to write to properties in AngularJS, as the built-in directives
 * handle the most common use cases (instead of the above example, you would use {@link ngValue}).
 *
 * However, [custom elements](https://developer.mozilla.org/docs/Web/Web_Components/Using_custom_elements)
 * often use custom properties to hold data, and `ngProp` can be used to provide input to these
 * custom elements.
 *
 * ## Binding to camelCase properties
 *
 * Since HTML attributes are case-insensitive, camelCase properties like `innerHTML` must be escaped.
 * AngularJS uses the underscore (_) in front of a character to indicate that it is uppercase, so
 * `innerHTML`  must be written as `ng-prop-inner_h_t_m_l="expression"` (Note that this is just an
 * example, and for binding HTML {@link ngBindHtml} should be used.
 *
 * ## Security
 *
 * Binding expressions to arbitrary properties poses a security risk, as  properties like `innerHTML`
 * can insert potentially dangerous HTML into the application, e.g. script tags that execute
 * malicious code.
 * For this reason, `ngProp` applies Strict Contextual Escaping with the {@link ng.$sce $sce service}.
 * This means vulnerable properties require their content to be "trusted", based on the
 * context of the property. For example, the `innerHTML` is in the `HTML` context, and the
 * `iframe.src` property is in the `RESOURCE_URL` context, which requires that values written to
 * this property are trusted as a `RESOURCE_URL`.
 *
 * This can be set explicitly by calling $sce.trustAs(type, value) on the value that is
 * trusted before passing it to the `ng-prop-*` directive. There are exist shorthand methods for
 * each context type in the form of {@link ng.$sce#trustAsResourceUrl $sce.trustAsResourceUrl()} et al.
 *
 * In some cases you can also rely upon automatic sanitization of untrusted values - see below.
 *
 * Based on the context, other options may exist to mark a value as trusted / configure the behavior
 * of {@link ng.$sce}. For example, to restrict the `RESOURCE_URL` context to specific origins, use
 * the {@link $sceDelegateProvider#resourceUrlWhitelist resourceUrlWhitelist()}
 * and {@link $sceDelegateProvider#resourceUrlBlacklist resourceUrlBlacklist()}.
 *
 * {@link ng.$sce#what-trusted-context-types-are-supported- Find out more about the different context types}.
 *
 * ### HTML Sanitization
 *
 * By default, `$sce` will throw an error if it detects untrusted HTML content, and will not bind the
 * content.
 * However, if you include the {@link ngSanitize ngSanitize module}, it will try to sanitize the
 * potentially dangerous HTML, e.g. strip non-whitelisted tags and attributes when binding to
 * `innerHTML`.
 *
 * @example
 * ### Binding to different contexts
 *
 * <example name="ngProp" module="exampleNgProp">
 *   <file name="app.js">
 *     angular.module('exampleNgProp', [])
 *       .component('main', {
 *         templateUrl: 'main.html',
 *         controller: function($sce) {
 *           this.safeContent = '<strong>Safe content</strong>';
 *           this.unsafeContent = '<button onclick="alert(\'Hello XSS!\')">Click for XSS</button>';
 *           this.trustedUnsafeContent = $sce.trustAsHtml(this.unsafeContent);
 *         }
 *       });
 *   </file>
 *   <file name="main.html">
 *     <div>
 *       <div class="prop-unit">
 *         Binding to a property without security context:
 *         <div class="prop-binding" ng-prop-inner_text="$ctrl.safeContent"></div>
 *         <span class="prop-note">innerText</span> (safeContent)
 *       </div>
 *
 *       <div class="prop-unit">
 *         "Safe" content that requires a security context will throw because the contents could potentially be dangerous ...
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.safeContent"></div>
 *         <span class="prop-note">innerHTML</span> (safeContent)
 *       </div>
 *
 *       <div class="prop-unit">
 *         ... so that actually dangerous content cannot be executed:
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.unsafeContent"></div>
 *         <span class="prop-note">innerHTML</span> (unsafeContent)
 *       </div>
 *
 *       <div class="prop-unit">
 *         ... but unsafe Content that has been trusted explicitly works - only do this if you are 100% sure!
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.trustedUnsafeContent"></div>
 *         <span class="prop-note">innerHTML</span> (trustedUnsafeContent)
 *       </div>
 *     </div>
 *   </file>
 *   <file name="index.html">
 *     <main></main>
 *   </file>
 *   <file name="styles.css">
 *     .prop-unit {
 *       margin-bottom: 10px;
 *     }
 *
 *     .prop-binding {
 *       min-height: 30px;
 *       border: 1px solid blue;
 *     }
 *
 *     .prop-note {
 *       font-family: Monospace;
 *     }
 *   </file>
 * </example>
 *
 *
 * @example
 * ### Binding to innerHTML with ngSanitize
 *
 * <example name="ngProp" module="exampleNgProp" deps="angular-sanitize.js">
 *   <file name="app.js">
 *     angular.module('exampleNgProp', ['ngSanitize'])
 *       .component('main', {
 *         templateUrl: 'main.html',
 *         controller: function($sce) {
 *           this.safeContent = '<strong>Safe content</strong>';
 *           this.unsafeContent = '<button onclick="alert(\'Hello XSS!\')">Click for XSS</button>';
 *           this.trustedUnsafeContent = $sce.trustAsHtml(this.unsafeContent);
 *         }
 *       });
 *   </file>
 *   <file name="main.html">
 *     <div>
 *       <div class="prop-unit">
 *         "Safe" content will be sanitized ...
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.safeContent"></div>
 *         <span class="prop-note">innerHTML</span> (safeContent)
 *       </div>
 *
 *       <div class="prop-unit">
 *         ... as will dangerous content:
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.unsafeContent"></div>
 *         <span class="prop-note">innerHTML</span> (unsafeContent)
 *       </div>
 *
 *       <div class="prop-unit">
 *         ... and content that has been trusted explicitly works the same as without ngSanitize:
 *         <div class="prop-binding" ng-prop-inner_h_t_m_l="$ctrl.trustedUnsafeContent"></div>
 *         <span class="prop-note">innerHTML</span> (trustedUnsafeContent)
 *       </div>
 *     </div>
 *   </file>
 *   <file name="index.html">
 *     <main></main>
 *   </file>
 *   <file name="styles.css">
 *     .prop-unit {
 *       margin-bottom: 10px;
 *     }
 *
 *     .prop-binding {
 *       min-height: 30px;
 *       border: 1px solid blue;
 *     }
 *
 *     .prop-note {
 *       font-family: Monospace;
 *     }
 *   </file>
 * </example>
 *
 */

/** @ngdoc directive
 * @name ngOn
 * @restrict A
 * @element ANY
 *
 * @usage
 *
 * ```html
 * <ANY ng-on-eventname="expression">
 * </ANY>
 * ```
 *
 * or with uppercase letters in property (e.g. "eventName"):
 *
 *
 * ```html
 * <ANY ng-on-event_name="expression">
 * </ANY>
 * ```
 *
 * @description
 * The `ngOn` directive adds an event listener to a DOM element via
 * {@link angular.element angular.element().on()}, and evaluates an expression when the event is
 * fired.
 * `ngOn` allows adding listeners for arbitrary events by including
 * the event name in the attribute, e.g. `ng-on-drop="onDrop()"` executes the 'onDrop()' expression
 * when the `drop` event is fired.
 *
 * AngularJS provides specific directives for many events, such as {@link ngClick}, so in most
 * cases it is not necessary to use `ngOn`. However, AngularJS does not support all events
 * (e.g. the `drop` event in the example above), and new events might be introduced in later DOM
 * standards.
 *
 * Another use-case for `ngOn` is listening to
 * [custom events](https://developer.mozilla.org/docs/Web/Guide/Events/Creating_and_triggering_events)
 * fired by
 * [custom elements](https://developer.mozilla.org/docs/Web/Web_Components/Using_custom_elements).
 *
 * ## Binding to camelCase properties
 *
 * Since HTML attributes are case-insensitive, camelCase properties like `myEvent` must be escaped.
 * AngularJS uses the underscore (_) in front of a character to indicate that it is uppercase, so
 * `myEvent` must be written as `ng-on-my_event="expression"`.
 *
 * @example
 * ### Bind to built-in DOM events
 *
 * <example name="ngOn" module="exampleNgOn">
 *   <file name="app.js">
 *     angular.module('exampleNgOn', [])
 *       .component('main', {
 *         templateUrl: 'main.html',
 *         controller: function() {
 *           this.clickCount = 0;
 *           this.mouseoverCount = 0;
 *
 *           this.loadingState = 0;
 *         }
 *       });
 *   </file>
 *   <file name="main.html">
 *     <div>
 *       This is equivalent to `ngClick` and `ngMouseover`:<br>
 *       <button
 *         ng-on-click="$ctrl.clickCount = $ctrl.clickCount + 1"
 *         ng-on-mouseover="$ctrl.mouseoverCount = $ctrl.mouseoverCount + 1">Click or mouseover</button><br>
 *       clickCount: {{$ctrl.clickCount}}<br>
 *       mouseover: {{$ctrl.mouseoverCount}}
 *
 *       <hr>
 *
 *       For the `error` and `load` event on images no built-in AngularJS directives exist:<br>
 *       <img src="thisimagedoesnotexist.png" ng-on-error="$ctrl.loadingState = -1" ng-on-load="$ctrl.loadingState = 1"><br>
 *       <div ng-switch="$ctrl.loadingState">
 *         <span ng-switch-when="0">Image is loading</span>
 *         <span ng-switch-when="-1">Image load error</span>
 *         <span ng-switch-when="1">Image loaded successfully</span>
 *       </div>
 *     </div>
 *   </file>
 *   <file name="index.html">
 *     <main></main>
 *   </file>
 * </example>
 *
 *
 * @example
 * ### Bind to custom DOM events
 *
 * <example name="ngOnCustom" module="exampleNgOn">
 *   <file name="app.js">
 *     angular.module('exampleNgOn', [])
 *       .component('main', {
 *         templateUrl: 'main.html',
 *         controller: function() {
 *           this.eventLog = '';
 *
 *           this.listener = function($event) {
 *             this.eventLog = 'Event with type "' + $event.type + '" fired at ' + $event.detail;
 *           };
 *         }
 *       })
 *       .component('childComponent', {
 *         templateUrl: 'child.html',
 *         controller: function($element) {
 *           this.fireEvent = function() {
 *             var event = new CustomEvent('customtype', { detail: new Date()});
 *
 *             $element[0].dispatchEvent(event);
 *           };
 *         }
 *       });
 *   </file>
 *   <file name="main.html">
 *     <child-component ng-on-customtype="$ctrl.listener($event)"></child-component><br>
 *     <span>Event log: {{$ctrl.eventLog}}</span>
 *   </file>
 *   <file name="child.html">
      <button ng-click="$ctrl.fireEvent()">Fire custom event</button>
 *   </file>
 *   <file name="index.html">
 *     <main></main>
 *   </file>
 * </example>
 */

var $compileMinErr = minErr('$compile');

function UNINITIALIZED_VALUE() {}
var _UNINITIALIZED_VALUE = new UNINITIALIZED_VALUE();

/**
 * @ngdoc provider
 * @name $compileProvider
 *
 * @description
 */
$CompileProvider.$inject = ['$provide', '$$sanitizeUriProvider'];
/** @this */
function $CompileProvider($provide, $$sanitizeUriProvider) {
  var hasDirectives = {},
      Suffix = 'Directive',
      COMMENT_DIRECTIVE_REGEXP = /^\s*directive:\s*([\w-]+)\s+(.*)$/,
      CLASS_DIRECTIVE_REGEXP = /(([\w-]+)(?::([^;]+))?;?)/,
      ALL_OR_NOTHING_ATTRS = makeMap('ngSrc,ngSrcset,src,srcset'),
      REQUIRE_PREFIX_REGEXP = /^(?:(\^\^?)?(\?)?(\^\^?)?)?/;

  // Ref: http://developers.whatwg.org/webappapis.html#event-handler-idl-attributes
  // The assumption is that future DOM event attribute names will begin with
  // 'on' and be composed of only English letters.
  var EVENT_HANDLER_ATTR_REGEXP = /^(on[a-z]+|formaction)$/;
  var bindingCache = createMap();

  function parseIsolateBindings(scope, directiveName, isController) {
    var LOCAL_REGEXP = /^([@&]|[=<](\*?))(\??)\s*([\w$]*)$/;

    var bindings = createMap();

    forEach(scope, function(definition, scopeName) {
      definition = definition.trim();

      if (definition in bindingCache) {
        bindings[scopeName] = bindingCache[definition];
        return;
      }
      var match = definition.match(LOCAL_REGEXP);

      if (!match) {
        throw $compileMinErr('iscp',
            'Invalid {3} for directive \'{0}\'.' +
            ' Definition: {... {1}: \'{2}\' ...}',
            directiveName, scopeName, definition,
            (isController ? 'controller bindings definition' :
            'isolate scope definition'));
      }

      bindings[scopeName] = {
        mode: match[1][0],
        collection: match[2] === '*',
        optional: match[3] === '?',
        attrName: match[4] || scopeName
      };
      if (match[4]) {
        bindingCache[definition] = bindings[scopeName];
      }
    });

    return bindings;
  }

  function parseDirectiveBindings(directive, directiveName) {
    var bindings = {
      isolateScope: null,
      bindToController: null
    };
    if (isObject(directive.scope)) {
      if (directive.bindToController === true) {
        bindings.bindToController = parseIsolateBindings(directive.scope,
                                                         directiveName, true);
        bindings.isolateScope = {};
      } else {
        bindings.isolateScope = parseIsolateBindings(directive.scope,
                                                     directiveName, false);
      }
    }
    if (isObject(directive.bindToController)) {
      bindings.bindToController =
          parseIsolateBindings(directive.bindToController, directiveName, true);
    }
    if (bindings.bindToController && !directive.controller) {
      // There is no controller
      throw $compileMinErr('noctrl',
            'Cannot bind to controller without directive \'{0}\'s controller.',
            directiveName);
    }
    return bindings;
  }

  function assertValidDirectiveName(name) {
    var letter = name.charAt(0);
    if (!letter || letter !== lowercase(letter)) {
      throw $compileMinErr('baddir', 'Directive/Component name \'{0}\' is invalid. The first character must be a lowercase letter', name);
    }
    if (name !== name.trim()) {
      throw $compileMinErr('baddir',
            'Directive/Component name \'{0}\' is invalid. The name should not contain leading or trailing whitespaces',
            name);
    }
  }

  function getDirectiveRequire(directive) {
    var require = directive.require || (directive.controller && directive.name);

    if (!isArray(require) && isObject(require)) {
      forEach(require, function(value, key) {
        var match = value.match(REQUIRE_PREFIX_REGEXP);
        var name = value.substring(match[0].length);
        if (!name) require[key] = match[0] + key;
      });
    }

    return require;
  }

  function getDirectiveRestrict(restrict, name) {
    if (restrict && !(isString(restrict) && /[EACM]/.test(restrict))) {
      throw $compileMinErr('badrestrict',
          'Restrict property \'{0}\' of directive \'{1}\' is invalid',
          restrict,
          name);
    }

    return restrict || 'EA';
  }

  /**
   * @ngdoc method
   * @name $compileProvider#directive
   * @kind function
   *
   * @description
   * Register a new directive with the compiler.
   *
   * @param {string|Object} name Name of the directive in camel-case (i.e. `ngBind` which will match
   *    as `ng-bind`), or an object map of directives where the keys are the names and the values
   *    are the factories.
   * @param {Function|Array} directiveFactory An injectable directive factory function. See the
   *    {@link guide/directive directive guide} and the {@link $compile compile API} for more info.
   * @returns {ng.$compileProvider} Self for chaining.
   */
  this.directive = function registerDirective(name, directiveFactory) {
    assertArg(name, 'name');
    assertNotHasOwnProperty(name, 'directive');
    if (isString(name)) {
      assertValidDirectiveName(name);
      assertArg(directiveFactory, 'directiveFactory');
      if (!hasDirectives.hasOwnProperty(name)) {
        hasDirectives[name] = [];
        $provide.factory(name + Suffix, ['$injector', '$exceptionHandler',
          function($injector, $exceptionHandler) {
            var directives = [];
            forEach(hasDirectives[name], function(directiveFactory, index) {
              try {
                var directive = $injector.invoke(directiveFactory);
                if (isFunction(directive)) {
                  directive = { compile: valueFn(directive) };
                } else if (!directive.compile && directive.link) {
                  directive.compile = valueFn(directive.link);
                }
                directive.priority = directive.priority || 0;
                directive.index = index;
                directive.name = directive.name || name;
                directive.require = getDirectiveRequire(directive);
                directive.restrict = getDirectiveRestrict(directive.restrict, name);
                directive.$$moduleName = directiveFactory.$$moduleName;
                directives.push(directive);
              } catch (e) {
                $exceptionHandler(e);
              }
            });
            return directives;
          }]);
      }
      hasDirectives[name].push(directiveFactory);
    } else {
      forEach(name, reverseParams(registerDirective));
    }
    return this;
  };

  /**
   * @ngdoc method
   * @name $compileProvider#component
   * @module ng
   * @param {string|Object} name Name of the component in camelCase (i.e. `myComp` which will match `<my-comp>`),
   *    or an object map of components where the keys are the names and the values are the component definition objects.
   * @param {Object} options Component definition object (a simplified
   *    {@link ng.$compile#directive-definition-object directive definition object}),
   *    with the following properties (all optional):
   *
   *    - `controller` – `{(string|function()=}` – controller constructor function that should be
   *      associated with newly created scope or the name of a {@link ng.$compile#-controller-
   *      registered controller} if passed as a string. An empty `noop` function by default.
   *    - `controllerAs` – `{string=}` – identifier name for to reference the controller in the component's scope.
   *      If present, the controller will be published to scope under the `controllerAs` name.
   *      If not present, this will default to be `$ctrl`.
   *    - `template` – `{string=|function()=}` – html template as a string or a function that
   *      returns an html template as a string which should be used as the contents of this component.
   *      Empty string by default.
   *
   *      If `template` is a function, then it is {@link auto.$injector#invoke injected} with
   *      the following locals:
   *
   *      - `$element` - Current element
   *      - `$attrs` - Current attributes object for the element
   *
   *    - `templateUrl` – `{string=|function()=}` – path or function that returns a path to an html
   *      template that should be used  as the contents of this component.
   *
   *      If `templateUrl` is a function, then it is {@link auto.$injector#invoke injected} with
   *      the following locals:
   *
   *      - `$element` - Current element
   *      - `$attrs` - Current attributes object for the element
   *
   *    - `bindings` – `{object=}` – defines bindings between DOM attributes and component properties.
   *      Component properties are always bound to the component controller and not to the scope.
   *      See {@link ng.$compile#-bindtocontroller- `bindToController`}.
   *    - `transclude` – `{boolean=}` – whether {@link $compile#transclusion content transclusion} is enabled.
   *      Disabled by default.
   *    - `require` - `{Object<string, string>=}` - requires the controllers of other directives and binds them to
   *      this component's controller. The object keys specify the property names under which the required
   *      controllers (object values) will be bound. See {@link ng.$compile#-require- `require`}.
   *    - `$...` – additional properties to attach to the directive factory function and the controller
   *      constructor function. (This is used by the component router to annotate)
   *
   * @returns {ng.$compileProvider} the compile provider itself, for chaining of function calls.
   * @description
   * Register a **component definition** with the compiler. This is a shorthand for registering a special
   * type of directive, which represents a self-contained UI component in your application. Such components
   * are always isolated (i.e. `scope: {}`) and are always restricted to elements (i.e. `restrict: 'E'`).
   *
   * Component definitions are very simple and do not require as much configuration as defining general
   * directives. Component definitions usually consist only of a template and a controller backing it.
   *
   * In order to make the definition easier, components enforce best practices like use of `controllerAs`,
   * `bindToController`. They always have **isolate scope** and are restricted to elements.
   *
   * Here are a few examples of how you would usually define components:
   *
   * ```js
   *   var myMod = angular.module(...);
   *   myMod.component('myComp', {
   *     template: '<div>My name is {{$ctrl.name}}</div>',
   *     controller: function() {
   *       this.name = 'shahar';
   *     }
   *   });
   *
   *   myMod.component('myComp', {
   *     template: '<div>My name is {{$ctrl.name}}</div>',
   *     bindings: {name: '@'}
   *   });
   *
   *   myMod.component('myComp', {
   *     templateUrl: 'views/my-comp.html',
   *     controller: 'MyCtrl',
   *     controllerAs: 'ctrl',
   *     bindings: {name: '@'}
   *   });
   *
   * ```
   * For more examples, and an in-depth guide, see the {@link guide/component component guide}.
   *
   * <br />
   * See also {@link ng.$compileProvider#directive $compileProvider.directive()}.
   */
  this.component = function registerComponent(name, options) {
    if (!isString(name)) {
      forEach(name, reverseParams(bind(this, registerComponent)));
      return this;
    }

    var controller = options.controller || function() {};

    function factory($injector) {
      function makeInjectable(fn) {
        if (isFunction(fn) || isArray(fn)) {
          return /** @this */ function(tElement, tAttrs) {
            return $injector.invoke(fn, this, {$element: tElement, $attrs: tAttrs});
          };
        } else {
          return fn;
        }
      }

      var template = (!options.template && !options.templateUrl ? '' : options.template);
      var ddo = {
        controller: controller,
        controllerAs: identifierForController(options.controller) || options.controllerAs || '$ctrl',
        template: makeInjectable(template),
        templateUrl: makeInjectable(options.templateUrl),
        transclude: options.transclude,
        scope: {},
        bindToController: options.bindings || {},
        restrict: 'E',
        require: options.require
      };

      // Copy annotations (starting with $) over to the DDO
      forEach(options, function(val, key) {
        if (key.charAt(0) === '$') ddo[key] = val;
      });

      return ddo;
    }

    // TODO(pete) remove the following `forEach` before we release 1.6.0
    // The component-router@0.2.0 looks for the annotations on the controller constructor
    // Nothing in AngularJS looks for annotations on the factory function but we can't remove
    // it from 1.5.x yet.

    // Copy any annotation properties (starting with $) over to the factory and controller constructor functions
    // These could be used by libraries such as the new component router
    forEach(options, function(val, key) {
      if (key.charAt(0) === '$') {
        factory[key] = val;
        // Don't try to copy over annotations to named controller
        if (isFunction(controller)) controller[key] = val;
      }
    });

    factory.$inject = ['$injector'];

    return this.directive(name, factory);
  };


  /**
   * @ngdoc method
   * @name $compileProvider#aHrefSanitizationWhitelist
   * @kind function
   *
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during a[href] sanitization.
   *
   * The sanitization is a security measure aimed at preventing XSS attacks via html links.
   *
   * Any url about to be assigned to a[href] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `aHrefSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
  this.aHrefSanitizationWhitelist = function(regexp) {
    if (isDefined(regexp)) {
      $$sanitizeUriProvider.aHrefSanitizationWhitelist(regexp);
      return this;
    } else {
      return $$sanitizeUriProvider.aHrefSanitizationWhitelist();
    }
  };


  /**
   * @ngdoc method
   * @name $compileProvider#imgSrcSanitizationWhitelist
   * @kind function
   *
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during img[src] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via html links.
   *
   * Any url about to be assigned to img[src] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `imgSrcSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
  this.imgSrcSanitizationWhitelist = function(regexp) {
    if (isDefined(regexp)) {
      $$sanitizeUriProvider.imgSrcSanitizationWhitelist(regexp);
      return this;
    } else {
      return $$sanitizeUriProvider.imgSrcSanitizationWhitelist();
    }
  };

  /**
   * @ngdoc method
   * @name  $compileProvider#debugInfoEnabled
   *
   * @param {boolean=} enabled update the debugInfoEnabled state if provided, otherwise just return the
   * current debugInfoEnabled state
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   *
   * @kind function
   *
   * @description
   * Call this method to enable/disable various debug runtime information in the compiler such as adding
   * binding information and a reference to the current scope on to DOM elements.
   * If enabled, the compiler will add the following to DOM elements that have been bound to the scope
   * * `ng-binding` CSS class
   * * `ng-scope` and `ng-isolated-scope` CSS classes
   * * `$binding` data property containing an array of the binding expressions
   * * Data properties used by the {@link angular.element#methods `scope()`/`isolateScope()` methods} to return
   *   the element's scope.
   * * Placeholder comments will contain information about what directive and binding caused the placeholder.
   *   E.g. `<!-- ngIf: shouldShow() -->`.
   *
   * You may want to disable this in production for a significant performance boost. See
   * {@link guide/production#disabling-debug-data Disabling Debug Data} for more.
   *
   * The default value is true.
   */
  var debugInfoEnabled = true;
  this.debugInfoEnabled = function(enabled) {
    if (isDefined(enabled)) {
      debugInfoEnabled = enabled;
      return this;
    }
    return debugInfoEnabled;
  };

  /**
   * @ngdoc method
   * @name  $compileProvider#strictComponentBindingsEnabled
   *
   * @param {boolean=} enabled update the strictComponentBindingsEnabled state if provided,
   * otherwise return the current strictComponentBindingsEnabled state.
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   *
   * @kind function
   *
   * @description
   * Call this method to enable / disable the strict component bindings check. If enabled, the
   * compiler will enforce that all scope / controller bindings of a
   * {@link $compileProvider#directive directive} / {@link $compileProvider#component component}
   * that are not set as optional with `?`, must be provided when the directive is instantiated.
   * If not provided, the compiler will throw the
   * {@link error/$compile/missingattr $compile:missingattr error}.
   *
   * The default value is false.
   */
  var strictComponentBindingsEnabled = false;
  this.strictComponentBindingsEnabled = function(enabled) {
    if (isDefined(enabled)) {
      strictComponentBindingsEnabled = enabled;
      return this;
    }
    return strictComponentBindingsEnabled;
  };

  var TTL = 10;
  /**
   * @ngdoc method
   * @name $compileProvider#onChangesTtl
   * @description
   *
   * Sets the number of times `$onChanges` hooks can trigger new changes before giving up and
   * assuming that the model is unstable.
   *
   * The current default is 10 iterations.
   *
   * In complex applications it's possible that dependencies between `$onChanges` hooks and bindings will result
   * in several iterations of calls to these hooks. However if an application needs more than the default 10
   * iterations to stabilize then you should investigate what is causing the model to continuously change during
   * the `$onChanges` hook execution.
   *
   * Increasing the TTL could have performance implications, so you should not change it without proper justification.
   *
   * @param {number} limit The number of `$onChanges` hook iterations.
   * @returns {number|object} the current limit (or `this` if called as a setter for chaining)
   */
  this.onChangesTtl = function(value) {
    if (arguments.length) {
      TTL = value;
      return this;
    }
    return TTL;
  };

  var commentDirectivesEnabledConfig = true;
  /**
   * @ngdoc method
   * @name $compileProvider#commentDirectivesEnabled
   * @description
   *
   * It indicates to the compiler
   * whether or not directives on comments should be compiled.
   * Defaults to `true`.
   *
   * Calling this function with false disables the compilation of directives
   * on comments for the whole application.
   * This results in a compilation performance gain,
   * as the compiler doesn't have to check comments when looking for directives.
   * This should however only be used if you are sure that no comment directives are used in
   * the application (including any 3rd party directives).
   *
   * @param {boolean} enabled `false` if the compiler may ignore directives on comments
   * @returns {boolean|object} the current value (or `this` if called as a setter for chaining)
   */
  this.commentDirectivesEnabled = function(value) {
    if (arguments.length) {
      commentDirectivesEnabledConfig = value;
      return this;
    }
    return commentDirectivesEnabledConfig;
  };


  var cssClassDirectivesEnabledConfig = true;
  /**
   * @ngdoc method
   * @name $compileProvider#cssClassDirectivesEnabled
   * @description
   *
   * It indicates to the compiler
   * whether or not directives on element classes should be compiled.
   * Defaults to `true`.
   *
   * Calling this function with false disables the compilation of directives
   * on element classes for the whole application.
   * This results in a compilation performance gain,
   * as the compiler doesn't have to check element classes when looking for directives.
   * This should however only be used if you are sure that no class directives are used in
   * the application (including any 3rd party directives).
   *
   * @param {boolean} enabled `false` if the compiler may ignore directives on element classes
   * @returns {boolean|object} the current value (or `this` if called as a setter for chaining)
   */
  this.cssClassDirectivesEnabled = function(value) {
    if (arguments.length) {
      cssClassDirectivesEnabledConfig = value;
      return this;
    }
    return cssClassDirectivesEnabledConfig;
  };


  /**
   * The security context of DOM Properties.
   * @private
   */
  var PROP_CONTEXTS = createMap();

  /**
   * @ngdoc method
   * @name $compileProvider#addPropertySecurityContext
   * @description
   *
   * Defines the security context for DOM properties bound by ng-prop-*.
   *
   * @param {string} elementName The element name or '*' to match any element.
   * @param {string} propertyName The DOM property name.
   * @param {string} ctx The {@link $sce} security context in which this value is safe for use, e.g. `$sce.URL`
   * @returns {object} `this` for chaining
   */
  this.addPropertySecurityContext = function(elementName, propertyName, ctx) {
    var key = (elementName.toLowerCase() + '|' + propertyName.toLowerCase());

    if (key in PROP_CONTEXTS && PROP_CONTEXTS[key] !== ctx) {
      throw $compileMinErr('ctxoverride', 'Property context \'{0}.{1}\' already set to \'{2}\', cannot override to \'{3}\'.', elementName, propertyName, PROP_CONTEXTS[key], ctx);
    }

    PROP_CONTEXTS[key] = ctx;
    return this;
  };

  /* Default property contexts.
   *
   * Copy of https://github.com/angular/angular/blob/6.0.6/packages/compiler/src/schema/dom_security_schema.ts#L31-L58
   * Changing:
   * - SecurityContext.* => SCE_CONTEXTS/$sce.*
   * - STYLE => CSS
   * - various URL => MEDIA_URL
   * - *|formAction, form|action URL => RESOURCE_URL (like the attribute)
   */
  (function registerNativePropertyContexts() {
    function registerContext(ctx, values) {
      forEach(values, function(v) { PROP_CONTEXTS[v.toLowerCase()] = ctx; });
    }

    registerContext(SCE_CONTEXTS.HTML, [
      'iframe|srcdoc',
      '*|innerHTML',
      '*|outerHTML'
    ]);
    registerContext(SCE_CONTEXTS.CSS, ['*|style']);
    registerContext(SCE_CONTEXTS.URL, [
      'area|href',       'area|ping',
      'a|href',          'a|ping',
      'blockquote|cite',
      'body|background',
      'del|cite',
      'input|src',
      'ins|cite',
      'q|cite'
    ]);
    registerContext(SCE_CONTEXTS.MEDIA_URL, [
      'audio|src',
      'img|src',    'img|srcset',
      'source|src', 'source|srcset',
      'track|src',
      'video|src',  'video|poster'
    ]);
    registerContext(SCE_CONTEXTS.RESOURCE_URL, [
      '*|formAction',
      'applet|code',      'applet|codebase',
      'base|href',
      'embed|src',
      'frame|src',
      'form|action',
      'head|profile',
      'html|manifest',
      'iframe|src',
      'link|href',
      'media|src',
      'object|codebase',  'object|data',
      'script|src'
    ]);
  })();


  this.$get = [
            '$injector', '$interpolate', '$exceptionHandler', '$templateRequest', '$parse',
            '$controller', '$rootScope', '$sce', '$animate',
    function($injector,   $interpolate,   $exceptionHandler,   $templateRequest,   $parse,
             $controller,   $rootScope,   $sce,   $animate) {

    var SIMPLE_ATTR_NAME = /^\w/;
    var specialAttrHolder = window.document.createElement('div');


    var commentDirectivesEnabled = commentDirectivesEnabledConfig;
    var cssClassDirectivesEnabled = cssClassDirectivesEnabledConfig;


    var onChangesTtl = TTL;
    // The onChanges hooks should all be run together in a single digest
    // When changes occur, the call to trigger their hooks will be added to this queue
    var onChangesQueue;

    // This function is called in a $$postDigest to trigger all the onChanges hooks in a single digest
    function flushOnChangesQueue() {
      try {
        if (!(--onChangesTtl)) {
          // We have hit the TTL limit so reset everything
          onChangesQueue = undefined;
          throw $compileMinErr('infchng', '{0} $onChanges() iterations reached. Aborting!\n', TTL);
        }
        // We must run this hook in an apply since the $$postDigest runs outside apply
        $rootScope.$apply(function() {
          for (var i = 0, ii = onChangesQueue.length; i < ii; ++i) {
            try {
              onChangesQueue[i]();
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          // Reset the queue to trigger a new schedule next time there is a change
          onChangesQueue = undefined;
        });
      } finally {
        onChangesTtl++;
      }
    }


    function sanitizeSrcset(value, invokeType) {
      if (!value) {
        return value;
      }
      if (!isString(value)) {
        throw $compileMinErr('srcset', 'Can\'t pass trusted values to `{0}`: "{1}"', invokeType, value.toString());
      }

      // Such values are a bit too complex to handle automatically inside $sce.
      // Instead, we sanitize each of the URIs individually, which works, even dynamically.

      // It's not possible to work around this using `$sce.trustAsMediaUrl`.
      // If you want to programmatically set explicitly trusted unsafe URLs, you should use
      // `$sce.trustAsHtml` on the whole `img` tag and inject it into the DOM using the
      // `ng-bind-html` directive.

      var result = '';

      // first check if there are spaces because it's not the same pattern
      var trimmedSrcset = trim(value);
      //                (   999x   ,|   999w   ,|   ,|,   )
      var srcPattern = /(\s+\d+x\s*,|\s+\d+w\s*,|\s+,|,\s+)/;
      var pattern = /\s/.test(trimmedSrcset) ? srcPattern : /(,)/;

      // split srcset into tuple of uri and descriptor except for the last item
      var rawUris = trimmedSrcset.split(pattern);

      // for each tuples
      var nbrUrisWith2parts = Math.floor(rawUris.length / 2);
      for (var i = 0; i < nbrUrisWith2parts; i++) {
        var innerIdx = i * 2;
        // sanitize the uri
        result += $sce.getTrustedMediaUrl(trim(rawUris[innerIdx]));
        // add the descriptor
        result += ' ' + trim(rawUris[innerIdx + 1]);
      }

      // split the last item into uri and descriptor
      var lastTuple = trim(rawUris[i * 2]).split(/\s/);

      // sanitize the last uri
      result += $sce.getTrustedMediaUrl(trim(lastTuple[0]));

      // and add the last descriptor if any
      if (lastTuple.length === 2) {
        result += (' ' + trim(lastTuple[1]));
      }
      return result;
    }


    function Attributes(element, attributesToCopy) {
      if (attributesToCopy) {
        var keys = Object.keys(attributesToCopy);
        var i, l, key;

        for (i = 0, l = keys.length; i < l; i++) {
          key = keys[i];
          this[key] = attributesToCopy[key];
        }
      } else {
        this.$attr = {};
      }

      this.$$element = element;
    }

    Attributes.prototype = {
      /**
       * @ngdoc method
       * @name $compile.directive.Attributes#$normalize
       * @kind function
       *
       * @description
       * Converts an attribute name (e.g. dash/colon/underscore-delimited string, optionally prefixed with `x-` or
       * `data-`) to its normalized, camelCase form.
       *
       * Also there is special case for Moz prefix starting with upper case letter.
       *
       * For further information check out the guide on {@link guide/directive#matching-directives Matching Directives}
       *
       * @param {string} name Name to normalize
       */
      $normalize: directiveNormalize,


      /**
       * @ngdoc method
       * @name $compile.directive.Attributes#$addClass
       * @kind function
       *
       * @description
       * Adds the CSS class value specified by the classVal parameter to the element. If animations
       * are enabled then an animation will be triggered for the class addition.
       *
       * @param {string} classVal The className value that will be added to the element
       */
      $addClass: function(classVal) {
        if (classVal && classVal.length > 0) {
          $animate.addClass(this.$$element, classVal);
        }
      },

      /**
       * @ngdoc method
       * @name $compile.directive.Attributes#$removeClass
       * @kind function
       *
       * @description
       * Removes the CSS class value specified by the classVal parameter from the element. If
       * animations are enabled then an animation will be triggered for the class removal.
       *
       * @param {string} classVal The className value that will be removed from the element
       */
      $removeClass: function(classVal) {
        if (classVal && classVal.length > 0) {
          $animate.removeClass(this.$$element, classVal);
        }
      },

      /**
       * @ngdoc method
       * @name $compile.directive.Attributes#$updateClass
       * @kind function
       *
       * @description
       * Adds and removes the appropriate CSS class values to the element based on the difference
       * between the new and old CSS class values (specified as newClasses and oldClasses).
       *
       * @param {string} newClasses The current CSS className value
       * @param {string} oldClasses The former CSS className value
       */
      $updateClass: function(newClasses, oldClasses) {
        var toAdd = tokenDifference(newClasses, oldClasses);
        if (toAdd && toAdd.length) {
          $animate.addClass(this.$$element, toAdd);
        }

        var toRemove = tokenDifference(oldClasses, newClasses);
        if (toRemove && toRemove.length) {
          $animate.removeClass(this.$$element, toRemove);
        }
      },

      /**
       * Set a normalized attribute on the element in a way such that all directives
       * can share the attribute. This function properly handles boolean attributes.
       * @param {string} key Normalized key. (ie ngAttribute)
       * @param {string|boolean} value The value to set. If `null` attribute will be deleted.
       * @param {boolean=} writeAttr If false, does not write the value to DOM element attribute.
       *     Defaults to true.
       * @param {string=} attrName Optional none normalized name. Defaults to key.
       */
      $set: function(key, value, writeAttr, attrName) {
        // TODO: decide whether or not to throw an error if "class"
        // is set through this function since it may cause $updateClass to
        // become unstable.

        var node = this.$$element[0],
            booleanKey = getBooleanAttrName(node, key),
            aliasedKey = getAliasedAttrName(key),
            observer = key,
            nodeName;

        if (booleanKey) {
          this.$$element.prop(key, value);
          attrName = booleanKey;
        } else if (aliasedKey) {
          this[aliasedKey] = value;
          observer = aliasedKey;
        }

        this[key] = value;

        // translate normalized key to actual key
        if (attrName) {
          this.$attr[key] = attrName;
        } else {
          attrName = this.$attr[key];
          if (!attrName) {
            this.$attr[key] = attrName = snake_case(key, '-');
          }
        }

        nodeName = nodeName_(this.$$element);

        // Sanitize img[srcset] values.
        if (nodeName === 'img' && key === 'srcset') {
          this[key] = value = sanitizeSrcset(value, '$set(\'srcset\', value)');
        }

        if (writeAttr !== false) {
          if (value === null || isUndefined(value)) {
            this.$$element.removeAttr(attrName);
          } else {
            if (SIMPLE_ATTR_NAME.test(attrName)) {
              // jQuery skips special boolean attrs treatment in XML nodes for
              // historical reasons and hence AngularJS cannot freely call
              // `.attr(attrName, false) with such attributes. To avoid issues
              // in XHTML, call `removeAttr` in such cases instead.
              // See https://github.com/jquery/jquery/issues/4249
              if (booleanKey && value === false) {
                this.$$element.removeAttr(attrName);
              } else {
                this.$$element.attr(attrName, value);
              }
            } else {
              setSpecialAttr(this.$$element[0], attrName, value);
            }
          }
        }

        // fire observers
        var $$observers = this.$$observers;
        if ($$observers) {
          forEach($$observers[observer], function(fn) {
            try {
              fn(value);
            } catch (e) {
              $exceptionHandler(e);
            }
          });
        }
      },


      /**
       * @ngdoc method
       * @name $compile.directive.Attributes#$observe
       * @kind function
       *
       * @description
       * Observes an interpolated attribute.
       *
       * The observer function will be invoked once during the next `$digest` following
       * compilation. The observer is then invoked whenever the interpolated value
       * changes.
       *
       * @param {string} key Normalized key. (ie ngAttribute) .
       * @param {function(interpolatedValue)} fn Function that will be called whenever
                the interpolated value of the attribute changes.
       *        See the {@link guide/interpolation#how-text-and-attribute-bindings-work Interpolation
       *        guide} for more info.
       * @returns {function()} Returns a deregistration function for this observer.
       */
      $observe: function(key, fn) {
        var attrs = this,
            $$observers = (attrs.$$observers || (attrs.$$observers = createMap())),
            listeners = ($$observers[key] || ($$observers[key] = []));

        listeners.push(fn);
        $rootScope.$evalAsync(function() {
          if (!listeners.$$inter && attrs.hasOwnProperty(key) && !isUndefined(attrs[key])) {
            // no one registered attribute interpolation function, so lets call it manually
            fn(attrs[key]);
          }
        });

        return function() {
          arrayRemove(listeners, fn);
        };
      }
    };

    function setSpecialAttr(element, attrName, value) {
      // Attributes names that do not start with letters (such as `(click)`) cannot be set using `setAttribute`
      // so we have to jump through some hoops to get such an attribute
      // https://github.com/angular/angular.js/pull/13318
      specialAttrHolder.innerHTML = '<span ' + attrName + '>';
      var attributes = specialAttrHolder.firstChild.attributes;
      var attribute = attributes[0];
      // We have to remove the attribute from its container element before we can add it to the destination element
      attributes.removeNamedItem(attribute.name);
      attribute.value = value;
      element.attributes.setNamedItem(attribute);
    }

    function safeAddClass($element, className) {
      try {
        $element.addClass(className);
      } catch (e) {
        // ignore, since it means that we are trying to set class on
        // SVG element, where class name is read-only.
      }
    }


    var startSymbol = $interpolate.startSymbol(),
        endSymbol = $interpolate.endSymbol(),
        denormalizeTemplate = (startSymbol === '{{' && endSymbol  === '}}')
            ? identity
            : function denormalizeTemplate(template) {
              return template.replace(/\{\{/g, startSymbol).replace(/}}/g, endSymbol);
        },
        NG_PREFIX_BINDING = /^ng(Attr|Prop|On)([A-Z].*)$/;
    var MULTI_ELEMENT_DIR_RE = /^(.+)Start$/;

    compile.$$addBindingInfo = debugInfoEnabled ? function $$addBindingInfo($element, binding) {
      var bindings = $element.data('$binding') || [];

      if (isArray(binding)) {
        bindings = bindings.concat(binding);
      } else {
        bindings.push(binding);
      }

      $element.data('$binding', bindings);
    } : noop;

    compile.$$addBindingClass = debugInfoEnabled ? function $$addBindingClass($element) {
      safeAddClass($element, 'ng-binding');
    } : noop;

    compile.$$addScopeInfo = debugInfoEnabled ? function $$addScopeInfo($element, scope, isolated, noTemplate) {
      var dataName = isolated ? (noTemplate ? '$isolateScopeNoTemplate' : '$isolateScope') : '$scope';
      $element.data(dataName, scope);
    } : noop;

    compile.$$addScopeClass = debugInfoEnabled ? function $$addScopeClass($element, isolated) {
      safeAddClass($element, isolated ? 'ng-isolate-scope' : 'ng-scope');
    } : noop;

    compile.$$createComment = function(directiveName, comment) {
      var content = '';
      if (debugInfoEnabled) {
        content = ' ' + (directiveName || '') + ': ';
        if (comment) content += comment + ' ';
      }
      return window.document.createComment(content);
    };

    return compile;

    //================================

    function compile($compileNodes, transcludeFn, maxPriority, ignoreDirective,
                        previousCompileContext) {
      if (!($compileNodes instanceof jqLite)) {
        // jquery always rewraps, whereas we need to preserve the original selector so that we can
        // modify it.
        $compileNodes = jqLite($compileNodes);
      }
      var compositeLinkFn =
              compileNodes($compileNodes, transcludeFn, $compileNodes,
                           maxPriority, ignoreDirective, previousCompileContext);
      compile.$$addScopeClass($compileNodes);
      var namespace = null;
      return function publicLinkFn(scope, cloneConnectFn, options) {
        if (!$compileNodes) {
          throw $compileMinErr('multilink', 'This element has already been linked.');
        }
        assertArg(scope, 'scope');

        if (previousCompileContext && previousCompileContext.needsNewScope) {
          // A parent directive did a replace and a directive on this element asked
          // for transclusion, which caused us to lose a layer of element on which
          // we could hold the new transclusion scope, so we will create it manually
          // here.
          scope = scope.$parent.$new();
        }

        options = options || {};
        var parentBoundTranscludeFn = options.parentBoundTranscludeFn,
          transcludeControllers = options.transcludeControllers,
          futureParentElement = options.futureParentElement;

        // When `parentBoundTranscludeFn` is passed, it is a
        // `controllersBoundTransclude` function (it was previously passed
        // as `transclude` to directive.link) so we must unwrap it to get
        // its `boundTranscludeFn`
        if (parentBoundTranscludeFn && parentBoundTranscludeFn.$$boundTransclude) {
          parentBoundTranscludeFn = parentBoundTranscludeFn.$$boundTransclude;
        }

        if (!namespace) {
          namespace = detectNamespaceForChildElements(futureParentElement);
        }
        var $linkNode;
        if (namespace !== 'html') {
          // When using a directive with replace:true and templateUrl the $compileNodes
          // (or a child element inside of them)
          // might change, so we need to recreate the namespace adapted compileNodes
          // for call to the link function.
          // Note: This will already clone the nodes...
          $linkNode = jqLite(
            wrapTemplate(namespace, jqLite('<div></div>').append($compileNodes).html())
          );
        } else if (cloneConnectFn) {
          // important!!: we must call our jqLite.clone() since the jQuery one is trying to be smart
          // and sometimes changes the structure of the DOM.
          $linkNode = JQLitePrototype.clone.call($compileNodes);
        } else {
          $linkNode = $compileNodes;
        }

        if (transcludeControllers) {
          for (var controllerName in transcludeControllers) {
            $linkNode.data('$' + controllerName + 'Controller', transcludeControllers[controllerName].instance);
          }
        }

        compile.$$addScopeInfo($linkNode, scope);

        if (cloneConnectFn) cloneConnectFn($linkNode, scope);
        if (compositeLinkFn) compositeLinkFn(scope, $linkNode, $linkNode, parentBoundTranscludeFn);

        if (!cloneConnectFn) {
          $compileNodes = compositeLinkFn = null;
        }
        return $linkNode;
      };
    }

    function detectNamespaceForChildElements(parentElement) {
      // TODO: Make this detect MathML as well...
      var node = parentElement && parentElement[0];
      if (!node) {
        return 'html';
      } else {
        return nodeName_(node) !== 'foreignobject' && toString.call(node).match(/SVG/) ? 'svg' : 'html';
      }
    }

    /**
     * Compile function matches each node in nodeList against the directives. Once all directives
     * for a particular node are collected their compile functions are executed. The compile
     * functions return values - the linking functions - are combined into a composite linking
     * function, which is the a linking function for the node.
     *
     * @param {NodeList} nodeList an array of nodes or NodeList to compile
     * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
     *        scope argument is auto-generated to the new child of the transcluded parent scope.
     * @param {DOMElement=} $rootElement If the nodeList is the root of the compilation tree then
     *        the rootElement must be set the jqLite collection of the compile root. This is
     *        needed so that the jqLite collection items can be replaced with widgets.
     * @param {number=} maxPriority Max directive priority.
     * @returns {Function} A composite linking function of all of the matched directives or null.
     */
    function compileNodes(nodeList, transcludeFn, $rootElement, maxPriority, ignoreDirective,
                            previousCompileContext) {
      var linkFns = [],
          // `nodeList` can be either an element's `.childNodes` (live NodeList)
          // or a jqLite/jQuery collection or an array
          notLiveList = isArray(nodeList) || (nodeList instanceof jqLite),
          attrs, directives, nodeLinkFn, childNodes, childLinkFn, linkFnFound, nodeLinkFnFound;


      for (var i = 0; i < nodeList.length; i++) {
        attrs = new Attributes();

        // Support: IE 11 only
        // Workaround for #11781 and #14924
        if (msie === 11) {
          mergeConsecutiveTextNodes(nodeList, i, notLiveList);
        }

        // We must always refer to `nodeList[i]` hereafter,
        // since the nodes can be replaced underneath us.
        directives = collectDirectives(nodeList[i], [], attrs, i === 0 ? maxPriority : undefined,
                                        ignoreDirective);

        nodeLinkFn = (directives.length)
            ? applyDirectivesToNode(directives, nodeList[i], attrs, transcludeFn, $rootElement,
                                      null, [], [], previousCompileContext)
            : null;

        if (nodeLinkFn && nodeLinkFn.scope) {
          compile.$$addScopeClass(attrs.$$element);
        }

        childLinkFn = (nodeLinkFn && nodeLinkFn.terminal ||
                      !(childNodes = nodeList[i].childNodes) ||
                      !childNodes.length)
            ? null
            : compileNodes(childNodes,
                 nodeLinkFn ? (
                  (nodeLinkFn.transcludeOnThisElement || !nodeLinkFn.templateOnThisElement)
                     && nodeLinkFn.transclude) : transcludeFn);

        if (nodeLinkFn || childLinkFn) {
          linkFns.push(i, nodeLinkFn, childLinkFn);
          linkFnFound = true;
          nodeLinkFnFound = nodeLinkFnFound || nodeLinkFn;
        }

        //use the previous context only for the first element in the virtual group
        previousCompileContext = null;
      }

      // return a linking function if we have found anything, null otherwise
      return linkFnFound ? compositeLinkFn : null;

      function compositeLinkFn(scope, nodeList, $rootElement, parentBoundTranscludeFn) {
        var nodeLinkFn, childLinkFn, node, childScope, i, ii, idx, childBoundTranscludeFn;
        var stableNodeList;


        if (nodeLinkFnFound) {
          // copy nodeList so that if a nodeLinkFn removes or adds an element at this DOM level our
          // offsets don't get screwed up
          var nodeListLength = nodeList.length;
          stableNodeList = new Array(nodeListLength);

          // create a sparse array by only copying the elements which have a linkFn
          for (i = 0; i < linkFns.length; i += 3) {
            idx = linkFns[i];
            stableNodeList[idx] = nodeList[idx];
          }
        } else {
          stableNodeList = nodeList;
        }

        for (i = 0, ii = linkFns.length; i < ii;) {
          node = stableNodeList[linkFns[i++]];
          nodeLinkFn = linkFns[i++];
          childLinkFn = linkFns[i++];

          if (nodeLinkFn) {
            if (nodeLinkFn.scope) {
              childScope = scope.$new();
              compile.$$addScopeInfo(jqLite(node), childScope);
            } else {
              childScope = scope;
            }

            if (nodeLinkFn.transcludeOnThisElement) {
              childBoundTranscludeFn = createBoundTranscludeFn(
                  scope, nodeLinkFn.transclude, parentBoundTranscludeFn);

            } else if (!nodeLinkFn.templateOnThisElement && parentBoundTranscludeFn) {
              childBoundTranscludeFn = parentBoundTranscludeFn;

            } else if (!parentBoundTranscludeFn && transcludeFn) {
              childBoundTranscludeFn = createBoundTranscludeFn(scope, transcludeFn);

            } else {
              childBoundTranscludeFn = null;
            }

            nodeLinkFn(childLinkFn, childScope, node, $rootElement, childBoundTranscludeFn);

          } else if (childLinkFn) {
            childLinkFn(scope, node.childNodes, undefined, parentBoundTranscludeFn);
          }
        }
      }
    }

    function mergeConsecutiveTextNodes(nodeList, idx, notLiveList) {
      var node = nodeList[idx];
      var parent = node.parentNode;
      var sibling;

      if (node.nodeType !== NODE_TYPE_TEXT) {
        return;
      }

      while (true) {
        sibling = parent ? node.nextSibling : nodeList[idx + 1];
        if (!sibling || sibling.nodeType !== NODE_TYPE_TEXT) {
          break;
        }

        node.nodeValue = node.nodeValue + sibling.nodeValue;

        if (sibling.parentNode) {
          sibling.parentNode.removeChild(sibling);
        }
        if (notLiveList && sibling === nodeList[idx + 1]) {
          nodeList.splice(idx + 1, 1);
        }
      }
    }

    function createBoundTranscludeFn(scope, transcludeFn, previousBoundTranscludeFn) {
      function boundTranscludeFn(transcludedScope, cloneFn, controllers, futureParentElement, containingScope) {

        if (!transcludedScope) {
          transcludedScope = scope.$new(false, containingScope);
          transcludedScope.$$transcluded = true;
        }

        return transcludeFn(transcludedScope, cloneFn, {
          parentBoundTranscludeFn: previousBoundTranscludeFn,
          transcludeControllers: controllers,
          futureParentElement: futureParentElement
        });
      }

      // We need  to attach the transclusion slots onto the `boundTranscludeFn`
      // so that they are available inside the `controllersBoundTransclude` function
      var boundSlots = boundTranscludeFn.$$slots = createMap();
      for (var slotName in transcludeFn.$$slots) {
        if (transcludeFn.$$slots[slotName]) {
          boundSlots[slotName] = createBoundTranscludeFn(scope, transcludeFn.$$slots[slotName], previousBoundTranscludeFn);
        } else {
          boundSlots[slotName] = null;
        }
      }

      return boundTranscludeFn;
    }

    /**
     * Looks for directives on the given node and adds them to the directive collection which is
     * sorted.
     *
     * @param node Node to search.
     * @param directives An array to which the directives are added to. This array is sorted before
     *        the function returns.
     * @param attrs The shared attrs object which is used to populate the normalized attributes.
     * @param {number=} maxPriority Max directive priority.
     */
    function collectDirectives(node, directives, attrs, maxPriority, ignoreDirective) {
      var nodeType = node.nodeType,
          attrsMap = attrs.$attr,
          match,
          nodeName,
          className;

      switch (nodeType) {
        case NODE_TYPE_ELEMENT: /* Element */

          nodeName = nodeName_(node);

          // use the node name: <directive>
          addDirective(directives,
              directiveNormalize(nodeName), 'E', maxPriority, ignoreDirective);

          // iterate over the attributes
          for (var attr, name, nName, value, ngPrefixMatch, nAttrs = node.attributes,
                   j = 0, jj = nAttrs && nAttrs.length; j < jj; j++) {
            var attrStartName = false;
            var attrEndName = false;

            var isNgAttr = false, isNgProp = false, isNgEvent = false;
            var multiElementMatch;

            attr = nAttrs[j];
            name = attr.name;
            value = attr.value;

            nName = directiveNormalize(name.toLowerCase());

            // Support ng-attr-*, ng-prop-* and ng-on-*
            if ((ngPrefixMatch = nName.match(NG_PREFIX_BINDING))) {
              isNgAttr = ngPrefixMatch[1] === 'Attr';
              isNgProp = ngPrefixMatch[1] === 'Prop';
              isNgEvent = ngPrefixMatch[1] === 'On';

              // Normalize the non-prefixed name
              name = name.replace(PREFIX_REGEXP, '')
                .toLowerCase()
                .substr(4 + ngPrefixMatch[1].length).replace(/_(.)/g, function(match, letter) {
                  return letter.toUpperCase();
                });

            // Support *-start / *-end multi element directives
            } else if ((multiElementMatch = nName.match(MULTI_ELEMENT_DIR_RE)) && directiveIsMultiElement(multiElementMatch[1])) {
              attrStartName = name;
              attrEndName = name.substr(0, name.length - 5) + 'end';
              name = name.substr(0, name.length - 6);
            }

            if (isNgProp || isNgEvent) {
              attrs[nName] = value;
              attrsMap[nName] = attr.name;

              if (isNgProp) {
                addPropertyDirective(node, directives, nName, name);
              } else {
                addEventDirective(directives, nName, name);
              }
            } else {
              // Update nName for cases where a prefix was removed
              // NOTE: the .toLowerCase() is unnecessary and causes https://github.com/angular/angular.js/issues/16624 for ng-attr-*
              nName = directiveNormalize(name.toLowerCase());
              attrsMap[nName] = name;

              if (isNgAttr || !attrs.hasOwnProperty(nName)) {
                attrs[nName] = value;
                if (getBooleanAttrName(node, nName)) {
                  attrs[nName] = true; // presence means true
                }
              }

              addAttrInterpolateDirective(node, directives, value, nName, isNgAttr);
              addDirective(directives, nName, 'A', maxPriority, ignoreDirective, attrStartName,
                            attrEndName);
            }
          }

          if (nodeName === 'input' && node.getAttribute('type') === 'hidden') {
            // Hidden input elements can have strange behaviour when navigating back to the page
            // This tells the browser not to try to cache and reinstate previous values
            node.setAttribute('autocomplete', 'off');
          }

          // use class as directive
          if (!cssClassDirectivesEnabled) break;
          className = node.className;
          if (isObject(className)) {
              // Maybe SVGAnimatedString
              className = className.animVal;
          }
          if (isString(className) && className !== '') {
            while ((match = CLASS_DIRECTIVE_REGEXP.exec(className))) {
              nName = directiveNormalize(match[2]);
              if (addDirective(directives, nName, 'C', maxPriority, ignoreDirective)) {
                attrs[nName] = trim(match[3]);
              }
              className = className.substr(match.index + match[0].length);
            }
          }
          break;
        case NODE_TYPE_TEXT: /* Text Node */
          addTextInterpolateDirective(directives, node.nodeValue);
          break;
        case NODE_TYPE_COMMENT: /* Comment */
          if (!commentDirectivesEnabled) break;
          collectCommentDirectives(node, directives, attrs, maxPriority, ignoreDirective);
          break;
      }

      directives.sort(byPriority);
      return directives;
    }

    function collectCommentDirectives(node, directives, attrs, maxPriority, ignoreDirective) {
      // function created because of performance, try/catch disables
      // the optimization of the whole function #14848
      try {
        var match = COMMENT_DIRECTIVE_REGEXP.exec(node.nodeValue);
        if (match) {
          var nName = directiveNormalize(match[1]);
          if (addDirective(directives, nName, 'M', maxPriority, ignoreDirective)) {
            attrs[nName] = trim(match[2]);
          }
        }
      } catch (e) {
        // turns out that under some circumstances IE9 throws errors when one attempts to read
        // comment's node value.
        // Just ignore it and continue. (Can't seem to reproduce in test case.)
      }
    }

    /**
     * Given a node with a directive-start it collects all of the siblings until it finds
     * directive-end.
     * @param node
     * @param attrStart
     * @param attrEnd
     * @returns {*}
     */
    function groupScan(node, attrStart, attrEnd) {
      var nodes = [];
      var depth = 0;
      if (attrStart && node.hasAttribute && node.hasAttribute(attrStart)) {
        do {
          if (!node) {
            throw $compileMinErr('uterdir',
                      'Unterminated attribute, found \'{0}\' but no matching \'{1}\' found.',
                      attrStart, attrEnd);
          }
          if (node.nodeType === NODE_TYPE_ELEMENT) {
            if (node.hasAttribute(attrStart)) depth++;
            if (node.hasAttribute(attrEnd)) depth--;
          }
          nodes.push(node);
          node = node.nextSibling;
        } while (depth > 0);
      } else {
        nodes.push(node);
      }

      return jqLite(nodes);
    }

    /**
     * Wrapper for linking function which converts normal linking function into a grouped
     * linking function.
     * @param linkFn
     * @param attrStart
     * @param attrEnd
     * @returns {Function}
     */
    function groupElementsLinkFnWrapper(linkFn, attrStart, attrEnd) {
      return function groupedElementsLink(scope, element, attrs, controllers, transcludeFn) {
        element = groupScan(element[0], attrStart, attrEnd);
        return linkFn(scope, element, attrs, controllers, transcludeFn);
      };
    }

    /**
     * A function generator that is used to support both eager and lazy compilation
     * linking function.
     * @param eager
     * @param $compileNodes
     * @param transcludeFn
     * @param maxPriority
     * @param ignoreDirective
     * @param previousCompileContext
     * @returns {Function}
     */
    function compilationGenerator(eager, $compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext) {
      var compiled;

      if (eager) {
        return compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext);
      }
      return /** @this */ function lazyCompilation() {
        if (!compiled) {
          compiled = compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext);

          // Null out all of these references in order to make them eligible for garbage collection
          // since this is a potentially long lived closure
          $compileNodes = transcludeFn = previousCompileContext = null;
        }
        return compiled.apply(this, arguments);
      };
    }

    /**
     * Once the directives have been collected, their compile functions are executed. This method
     * is responsible for inlining directive templates as well as terminating the application
     * of the directives if the terminal directive has been reached.
     *
     * @param {Array} directives Array of collected directives to execute their compile function.
     *        this needs to be pre-sorted by priority order.
     * @param {Node} compileNode The raw DOM node to apply the compile functions to
     * @param {Object} templateAttrs The shared attribute function
     * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
     *                                                  scope argument is auto-generated to the new
     *                                                  child of the transcluded parent scope.
     * @param {JQLite} jqCollection If we are working on the root of the compile tree then this
     *                              argument has the root jqLite array so that we can replace nodes
     *                              on it.
     * @param {Object=} originalReplaceDirective An optional directive that will be ignored when
     *                                           compiling the transclusion.
     * @param {Array.<Function>} preLinkFns
     * @param {Array.<Function>} postLinkFns
     * @param {Object} previousCompileContext Context used for previous compilation of the current
     *                                        node
     * @returns {Function} linkFn
     */
    function applyDirectivesToNode(directives, compileNode, templateAttrs, transcludeFn,
                                   jqCollection, originalReplaceDirective, preLinkFns, postLinkFns,
                                   previousCompileContext) {
      previousCompileContext = previousCompileContext || {};

      var terminalPriority = -Number.MAX_VALUE,
          newScopeDirective = previousCompileContext.newScopeDirective,
          controllerDirectives = previousCompileContext.controllerDirectives,
          newIsolateScopeDirective = previousCompileContext.newIsolateScopeDirective,
          templateDirective = previousCompileContext.templateDirective,
          nonTlbTranscludeDirective = previousCompileContext.nonTlbTranscludeDirective,
          hasTranscludeDirective = false,
          hasTemplate = false,
          hasElementTranscludeDirective = previousCompileContext.hasElementTranscludeDirective,
          $compileNode = templateAttrs.$$element = jqLite(compileNode),
          directive,
          directiveName,
          $template,
          replaceDirective = originalReplaceDirective,
          childTranscludeFn = transcludeFn,
          linkFn,
          didScanForMultipleTransclusion = false,
          mightHaveMultipleTransclusionError = false,
          directiveValue;

      // executes all directives on the current element
      for (var i = 0, ii = directives.length; i < ii; i++) {
        directive = directives[i];
        var attrStart = directive.$$start;
        var attrEnd = directive.$$end;

        // collect multiblock sections
        if (attrStart) {
          $compileNode = groupScan(compileNode, attrStart, attrEnd);
        }
        $template = undefined;

        if (terminalPriority > directive.priority) {
          break; // prevent further processing of directives
        }

        directiveValue = directive.scope;

        if (directiveValue) {

          // skip the check for directives with async templates, we'll check the derived sync
          // directive when the template arrives
          if (!directive.templateUrl) {
            if (isObject(directiveValue)) {
              // This directive is trying to add an isolated scope.
              // Check that there is no scope of any kind already
              assertNoDuplicate('new/isolated scope', newIsolateScopeDirective || newScopeDirective,
                                directive, $compileNode);
              newIsolateScopeDirective = directive;
            } else {
              // This directive is trying to add a child scope.
              // Check that there is no isolated scope already
              assertNoDuplicate('new/isolated scope', newIsolateScopeDirective, directive,
                                $compileNode);
            }
          }

          newScopeDirective = newScopeDirective || directive;
        }

        directiveName = directive.name;

        // If we encounter a condition that can result in transclusion on the directive,
        // then scan ahead in the remaining directives for others that may cause a multiple
        // transclusion error to be thrown during the compilation process.  If a matching directive
        // is found, then we know that when we encounter a transcluded directive, we need to eagerly
        // compile the `transclude` function rather than doing it lazily in order to throw
        // exceptions at the correct time
        if (!didScanForMultipleTransclusion && ((directive.replace && (directive.templateUrl || directive.template))
            || (directive.transclude && !directive.$$tlb))) {
                var candidateDirective;

                for (var scanningIndex = i + 1; (candidateDirective = directives[scanningIndex++]);) {
                    if ((candidateDirective.transclude && !candidateDirective.$$tlb)
                        || (candidateDirective.replace && (candidateDirective.templateUrl || candidateDirective.template))) {
                        mightHaveMultipleTransclusionError = true;
                        break;
                    }
                }

                didScanForMultipleTransclusion = true;
        }

        if (!directive.templateUrl && directive.controller) {
          controllerDirectives = controllerDirectives || createMap();
          assertNoDuplicate('\'' + directiveName + '\' controller',
              controllerDirectives[directiveName], directive, $compileNode);
          controllerDirectives[directiveName] = directive;
        }

        directiveValue = directive.transclude;

        if (directiveValue) {
          hasTranscludeDirective = true;

          // Special case ngIf and ngRepeat so that we don't complain about duplicate transclusion.
          // This option should only be used by directives that know how to safely handle element transclusion,
          // where the transcluded nodes are added or replaced after linking.
          if (!directive.$$tlb) {
            assertNoDuplicate('transclusion', nonTlbTranscludeDirective, directive, $compileNode);
            nonTlbTranscludeDirective = directive;
          }

          if (directiveValue === 'element') {
            hasElementTranscludeDirective = true;
            terminalPriority = directive.priority;
            $template = $compileNode;
            $compileNode = templateAttrs.$$element =
                jqLite(compile.$$createComment(directiveName, templateAttrs[directiveName]));
            compileNode = $compileNode[0];
            replaceWith(jqCollection, sliceArgs($template), compileNode);

            childTranscludeFn = compilationGenerator(mightHaveMultipleTransclusionError, $template, transcludeFn, terminalPriority,
                                        replaceDirective && replaceDirective.name, {
                                          // Don't pass in:
                                          // - controllerDirectives - otherwise we'll create duplicates controllers
                                          // - newIsolateScopeDirective or templateDirective - combining templates with
                                          //   element transclusion doesn't make sense.
                                          //
                                          // We need only nonTlbTranscludeDirective so that we prevent putting transclusion
                                          // on the same element more than once.
                                          nonTlbTranscludeDirective: nonTlbTranscludeDirective
                                        });
          } else {

            var slots = createMap();

            if (!isObject(directiveValue)) {
              $template = jqLite(jqLiteClone(compileNode)).contents();
            } else {

              // We have transclusion slots,
              // collect them up, compile them and store their transclusion functions
              $template = window.document.createDocumentFragment();

              var slotMap = createMap();
              var filledSlots = createMap();

              // Parse the element selectors
              forEach(directiveValue, function(elementSelector, slotName) {
                // If an element selector starts with a ? then it is optional
                var optional = (elementSelector.charAt(0) === '?');
                elementSelector = optional ? elementSelector.substring(1) : elementSelector;

                slotMap[elementSelector] = slotName;

                // We explicitly assign `null` since this implies that a slot was defined but not filled.
                // Later when calling boundTransclusion functions with a slot name we only error if the
                // slot is `undefined`
                slots[slotName] = null;

                // filledSlots contains `true` for all slots that are either optional or have been
                // filled. This is used to check that we have not missed any required slots
                filledSlots[slotName] = optional;
              });

              // Add the matching elements into their slot
              forEach($compileNode.contents(), function(node) {
                var slotName = slotMap[directiveNormalize(nodeName_(node))];
                if (slotName) {
                  filledSlots[slotName] = true;
                  slots[slotName] = slots[slotName] || window.document.createDocumentFragment();
                  slots[slotName].appendChild(node);
                } else {
                  $template.appendChild(node);
                }
              });

              // Check for required slots that were not filled
              forEach(filledSlots, function(filled, slotName) {
                if (!filled) {
                  throw $compileMinErr('reqslot', 'Required transclusion slot `{0}` was not filled.', slotName);
                }
              });

              for (var slotName in slots) {
                if (slots[slotName]) {
                  // Only define a transclusion function if the slot was filled
                  var slotCompileNodes = jqLite(slots[slotName].childNodes);
                  slots[slotName] = compilationGenerator(mightHaveMultipleTransclusionError, slotCompileNodes, transcludeFn);
                }
              }

              $template = jqLite($template.childNodes);
            }

            $compileNode.empty(); // clear contents
            childTranscludeFn = compilationGenerator(mightHaveMultipleTransclusionError, $template, transcludeFn, undefined,
                undefined, { needsNewScope: directive.$$isolateScope || directive.$$newScope});
            childTranscludeFn.$$slots = slots;
          }
        }

        if (directive.template) {
          hasTemplate = true;
          assertNoDuplicate('template', templateDirective, directive, $compileNode);
          templateDirective = directive;

          directiveValue = (isFunction(directive.template))
              ? directive.template($compileNode, templateAttrs)
              : directive.template;

          directiveValue = denormalizeTemplate(directiveValue);

          if (directive.replace) {
            replaceDirective = directive;
            if (jqLiteIsTextNode(directiveValue)) {
              $template = [];
            } else {
              $template = removeComments(wrapTemplate(directive.templateNamespace, trim(directiveValue)));
            }
            compileNode = $template[0];

            if ($template.length !== 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
              throw $compileMinErr('tplrt',
                  'Template for directive \'{0}\' must have exactly one root element. {1}',
                  directiveName, '');
            }

            replaceWith(jqCollection, $compileNode, compileNode);

            var newTemplateAttrs = {$attr: {}};

            // combine directives from the original node and from the template:
            // - take the array of directives for this element
            // - split it into two parts, those that already applied (processed) and those that weren't (unprocessed)
            // - collect directives from the template and sort them by priority
            // - combine directives as: processed + template + unprocessed
            var templateDirectives = collectDirectives(compileNode, [], newTemplateAttrs);
            var unprocessedDirectives = directives.splice(i + 1, directives.length - (i + 1));

            if (newIsolateScopeDirective || newScopeDirective) {
              // The original directive caused the current element to be replaced but this element
              // also needs to have a new scope, so we need to tell the template directives
              // that they would need to get their scope from further up, if they require transclusion
              markDirectiveScope(templateDirectives, newIsolateScopeDirective, newScopeDirective);
            }
            directives = directives.concat(templateDirectives).concat(unprocessedDirectives);
            mergeTemplateAttributes(templateAttrs, newTemplateAttrs);

            ii = directives.length;
          } else {
            $compileNode.html(directiveValue);
          }
        }

        if (directive.templateUrl) {
          hasTemplate = true;
          assertNoDuplicate('template', templateDirective, directive, $compileNode);
          templateDirective = directive;

          if (directive.replace) {
            replaceDirective = directive;
          }

          // eslint-disable-next-line no-func-assign
          nodeLinkFn = compileTemplateUrl(directives.splice(i, directives.length - i), $compileNode,
              templateAttrs, jqCollection, hasTranscludeDirective && childTranscludeFn, preLinkFns, postLinkFns, {
                controllerDirectives: controllerDirectives,
                newScopeDirective: (newScopeDirective !== directive) && newScopeDirective,
                newIsolateScopeDirective: newIsolateScopeDirective,
                templateDirective: templateDirective,
                nonTlbTranscludeDirective: nonTlbTranscludeDirective
              });
          ii = directives.length;
        } else if (directive.compile) {
          try {
            linkFn = directive.compile($compileNode, templateAttrs, childTranscludeFn);
            var context = directive.$$originalDirective || directive;
            if (isFunction(linkFn)) {
              addLinkFns(null, bind(context, linkFn), attrStart, attrEnd);
            } else if (linkFn) {
              addLinkFns(bind(context, linkFn.pre), bind(context, linkFn.post), attrStart, attrEnd);
            }
          } catch (e) {
            $exceptionHandler(e, startingTag($compileNode));
          }
        }

        if (directive.terminal) {
          nodeLinkFn.terminal = true;
          terminalPriority = Math.max(terminalPriority, directive.priority);
        }

      }

      nodeLinkFn.scope = newScopeDirective && newScopeDirective.scope === true;
      nodeLinkFn.transcludeOnThisElement = hasTranscludeDirective;
      nodeLinkFn.templateOnThisElement = hasTemplate;
      nodeLinkFn.transclude = childTranscludeFn;

      previousCompileContext.hasElementTranscludeDirective = hasElementTranscludeDirective;

      // might be normal or delayed nodeLinkFn depending on if templateUrl is present
      return nodeLinkFn;

      ////////////////////

      function addLinkFns(pre, post, attrStart, attrEnd) {
        if (pre) {
          if (attrStart) pre = groupElementsLinkFnWrapper(pre, attrStart, attrEnd);
          pre.require = directive.require;
          pre.directiveName = directiveName;
          if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
            pre = cloneAndAnnotateFn(pre, {isolateScope: true});
          }
          preLinkFns.push(pre);
        }
        if (post) {
          if (attrStart) post = groupElementsLinkFnWrapper(post, attrStart, attrEnd);
          post.require = directive.require;
          post.directiveName = directiveName;
          if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
            post = cloneAndAnnotateFn(post, {isolateScope: true});
          }
          postLinkFns.push(post);
        }
      }

      function nodeLinkFn(childLinkFn, scope, linkNode, $rootElement, boundTranscludeFn) {
        var i, ii, linkFn, isolateScope, controllerScope, elementControllers, transcludeFn, $element,
            attrs, scopeBindingInfo;

        if (compileNode === linkNode) {
          attrs = templateAttrs;
          $element = templateAttrs.$$element;
        } else {
          $element = jqLite(linkNode);
          attrs = new Attributes($element, templateAttrs);
        }

        controllerScope = scope;
        if (newIsolateScopeDirective) {
          isolateScope = scope.$new(true);
        } else if (newScopeDirective) {
          controllerScope = scope.$parent;
        }

        if (boundTranscludeFn) {
          // track `boundTranscludeFn` so it can be unwrapped if `transcludeFn`
          // is later passed as `parentBoundTranscludeFn` to `publicLinkFn`
          transcludeFn = controllersBoundTransclude;
          transcludeFn.$$boundTransclude = boundTranscludeFn;
          // expose the slots on the `$transclude` function
          transcludeFn.isSlotFilled = function(slotName) {
            return !!boundTranscludeFn.$$slots[slotName];
          };
        }

        if (controllerDirectives) {
          elementControllers = setupControllers($element, attrs, transcludeFn, controllerDirectives, isolateScope, scope, newIsolateScopeDirective);
        }

        if (newIsolateScopeDirective) {
          // Initialize isolate scope bindings for new isolate scope directive.
          compile.$$addScopeInfo($element, isolateScope, true, !(templateDirective && (templateDirective === newIsolateScopeDirective ||
              templateDirective === newIsolateScopeDirective.$$originalDirective)));
          compile.$$addScopeClass($element, true);
          isolateScope.$$isolateBindings =
              newIsolateScopeDirective.$$isolateBindings;
          scopeBindingInfo = initializeDirectiveBindings(scope, attrs, isolateScope,
                                        isolateScope.$$isolateBindings,
                                        newIsolateScopeDirective);
          if (scopeBindingInfo.removeWatches) {
            isolateScope.$on('$destroy', scopeBindingInfo.removeWatches);
          }
        }

        // Initialize bindToController bindings
        for (var name in elementControllers) {
          var controllerDirective = controllerDirectives[name];
          var controller = elementControllers[name];
          var bindings = controllerDirective.$$bindings.bindToController;

          controller.instance = controller();
          $element.data('$' + controllerDirective.name + 'Controller', controller.instance);
          controller.bindingInfo =
            initializeDirectiveBindings(controllerScope, attrs, controller.instance, bindings, controllerDirective);
          }

        // Bind the required controllers to the controller, if `require` is an object and `bindToController` is truthy
        forEach(controllerDirectives, function(controllerDirective, name) {
          var require = controllerDirective.require;
          if (controllerDirective.bindToController && !isArray(require) && isObject(require)) {
            extend(elementControllers[name].instance, getControllers(name, require, $element, elementControllers));
          }
        });

        // Handle the init and destroy lifecycle hooks on all controllers that have them
        forEach(elementControllers, function(controller) {
          var controllerInstance = controller.instance;
          if (isFunction(controllerInstance.$onChanges)) {
            try {
              controllerInstance.$onChanges(controller.bindingInfo.initialChanges);
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          if (isFunction(controllerInstance.$onInit)) {
            try {
              controllerInstance.$onInit();
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          if (isFunction(controllerInstance.$doCheck)) {
            controllerScope.$watch(function() { controllerInstance.$doCheck(); });
            controllerInstance.$doCheck();
          }
          if (isFunction(controllerInstance.$onDestroy)) {
            controllerScope.$on('$destroy', function callOnDestroyHook() {
              controllerInstance.$onDestroy();
            });
          }
        });

        // PRELINKING
        for (i = 0, ii = preLinkFns.length; i < ii; i++) {
          linkFn = preLinkFns[i];
          invokeLinkFn(linkFn,
              linkFn.isolateScope ? isolateScope : scope,
              $element,
              attrs,
              linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers),
              transcludeFn
          );
        }

        // RECURSION
        // We only pass the isolate scope, if the isolate directive has a template,
        // otherwise the child elements do not belong to the isolate directive.
        var scopeToChild = scope;
        if (newIsolateScopeDirective && (newIsolateScopeDirective.template || newIsolateScopeDirective.templateUrl === null)) {
          scopeToChild = isolateScope;
        }
        if (childLinkFn) {
          childLinkFn(scopeToChild, linkNode.childNodes, undefined, boundTranscludeFn);
        }

        // POSTLINKING
        for (i = postLinkFns.length - 1; i >= 0; i--) {
          linkFn = postLinkFns[i];
          invokeLinkFn(linkFn,
              linkFn.isolateScope ? isolateScope : scope,
              $element,
              attrs,
              linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers),
              transcludeFn
          );
        }

        // Trigger $postLink lifecycle hooks
        forEach(elementControllers, function(controller) {
          var controllerInstance = controller.instance;
          if (isFunction(controllerInstance.$postLink)) {
            controllerInstance.$postLink();
          }
        });

        // This is the function that is injected as `$transclude`.
        // Note: all arguments are optional!
        function controllersBoundTransclude(scope, cloneAttachFn, futureParentElement, slotName) {
          var transcludeControllers;
          // No scope passed in:
          if (!isScope(scope)) {
            slotName = futureParentElement;
            futureParentElement = cloneAttachFn;
            cloneAttachFn = scope;
            scope = undefined;
          }

          if (hasElementTranscludeDirective) {
            transcludeControllers = elementControllers;
          }
          if (!futureParentElement) {
            futureParentElement = hasElementTranscludeDirective ? $element.parent() : $element;
          }
          if (slotName) {
            // slotTranscludeFn can be one of three things:
            //  * a transclude function - a filled slot
            //  * `null` - an optional slot that was not filled
            //  * `undefined` - a slot that was not declared (i.e. invalid)
            var slotTranscludeFn = boundTranscludeFn.$$slots[slotName];
            if (slotTranscludeFn) {
              return slotTranscludeFn(scope, cloneAttachFn, transcludeControllers, futureParentElement, scopeToChild);
            } else if (isUndefined(slotTranscludeFn)) {
              throw $compileMinErr('noslot',
               'No parent directive that requires a transclusion with slot name "{0}". ' +
               'Element: {1}',
               slotName, startingTag($element));
            }
          } else {
            return boundTranscludeFn(scope, cloneAttachFn, transcludeControllers, futureParentElement, scopeToChild);
          }
        }
      }
    }

    function getControllers(directiveName, require, $element, elementControllers) {
      var value;

      if (isString(require)) {
        var match = require.match(REQUIRE_PREFIX_REGEXP);
        var name = require.substring(match[0].length);
        var inheritType = match[1] || match[3];
        var optional = match[2] === '?';

        //If only parents then start at the parent element
        if (inheritType === '^^') {
          $element = $element.parent();
        //Otherwise attempt getting the controller from elementControllers in case
        //the element is transcluded (and has no data) and to avoid .data if possible
        } else {
          value = elementControllers && elementControllers[name];
          value = value && value.instance;
        }

        if (!value) {
          var dataName = '$' + name + 'Controller';

          if (inheritType === '^^' && $element[0] && $element[0].nodeType === NODE_TYPE_DOCUMENT) {
            // inheritedData() uses the documentElement when it finds the document, so we would
            // require from the element itself.
            value = null;
          } else {
            value = inheritType ? $element.inheritedData(dataName) : $element.data(dataName);
          }
        }

        if (!value && !optional) {
          throw $compileMinErr('ctreq',
              'Controller \'{0}\', required by directive \'{1}\', can\'t be found!',
              name, directiveName);
        }
      } else if (isArray(require)) {
        value = [];
        for (var i = 0, ii = require.length; i < ii; i++) {
          value[i] = getControllers(directiveName, require[i], $element, elementControllers);
        }
      } else if (isObject(require)) {
        value = {};
        forEach(require, function(controller, property) {
          value[property] = getControllers(directiveName, controller, $element, elementControllers);
        });
      }

      return value || null;
    }

    function setupControllers($element, attrs, transcludeFn, controllerDirectives, isolateScope, scope, newIsolateScopeDirective) {
      var elementControllers = createMap();
      for (var controllerKey in controllerDirectives) {
        var directive = controllerDirectives[controllerKey];
        var locals = {
          $scope: directive === newIsolateScopeDirective || directive.$$isolateScope ? isolateScope : scope,
          $element: $element,
          $attrs: attrs,
          $transclude: transcludeFn
        };

        var controller = directive.controller;
        if (controller === '@') {
          controller = attrs[directive.name];
        }

        var controllerInstance = $controller(controller, locals, true, directive.controllerAs);

        // For directives with element transclusion the element is a comment.
        // In this case .data will not attach any data.
        // Instead, we save the controllers for the element in a local hash and attach to .data
        // later, once we have the actual element.
        elementControllers[directive.name] = controllerInstance;
        $element.data('$' + directive.name + 'Controller', controllerInstance.instance);
      }
      return elementControllers;
    }

    // Depending upon the context in which a directive finds itself it might need to have a new isolated
    // or child scope created. For instance:
    // * if the directive has been pulled into a template because another directive with a higher priority
    // asked for element transclusion
    // * if the directive itself asks for transclusion but it is at the root of a template and the original
    // element was replaced. See https://github.com/angular/angular.js/issues/12936
    function markDirectiveScope(directives, isolateScope, newScope) {
      for (var j = 0, jj = directives.length; j < jj; j++) {
        directives[j] = inherit(directives[j], {$$isolateScope: isolateScope, $$newScope: newScope});
      }
    }

    /**
     * looks up the directive and decorates it with exception handling and proper parameters. We
     * call this the boundDirective.
     *
     * @param {string} name name of the directive to look up.
     * @param {string} location The directive must be found in specific format.
     *   String containing any of theses characters:
     *
     *   * `E`: element name
     *   * `A': attribute
     *   * `C`: class
     *   * `M`: comment
     * @returns {boolean} true if directive was added.
     */
    function addDirective(tDirectives, name, location, maxPriority, ignoreDirective, startAttrName,
                          endAttrName) {
      if (name === ignoreDirective) return null;
      var match = null;
      if (hasDirectives.hasOwnProperty(name)) {
        for (var directive, directives = $injector.get(name + Suffix),
            i = 0, ii = directives.length; i < ii; i++) {
          directive = directives[i];
          if ((isUndefined(maxPriority) || maxPriority > directive.priority) &&
               directive.restrict.indexOf(location) !== -1) {
            if (startAttrName) {
              directive = inherit(directive, {$$start: startAttrName, $$end: endAttrName});
            }
            if (!directive.$$bindings) {
              var bindings = directive.$$bindings =
                  parseDirectiveBindings(directive, directive.name);
              if (isObject(bindings.isolateScope)) {
                directive.$$isolateBindings = bindings.isolateScope;
              }
            }
            tDirectives.push(directive);
            match = directive;
          }
        }
      }
      return match;
    }


    /**
     * looks up the directive and returns true if it is a multi-element directive,
     * and therefore requires DOM nodes between -start and -end markers to be grouped
     * together.
     *
     * @param {string} name name of the directive to look up.
     * @returns true if directive was registered as multi-element.
     */
    function directiveIsMultiElement(name) {
      if (hasDirectives.hasOwnProperty(name)) {
        for (var directive, directives = $injector.get(name + Suffix),
            i = 0, ii = directives.length; i < ii; i++) {
          directive = directives[i];
          if (directive.multiElement) {
            return true;
          }
        }
      }
      return false;
    }

    /**
     * When the element is replaced with HTML template then the new attributes
     * on the template need to be merged with the existing attributes in the DOM.
     * The desired effect is to have both of the attributes present.
     *
     * @param {object} dst destination attributes (original DOM)
     * @param {object} src source attributes (from the directive template)
     */
    function mergeTemplateAttributes(dst, src) {
      var srcAttr = src.$attr,
          dstAttr = dst.$attr;

      // reapply the old attributes to the new element
      forEach(dst, function(value, key) {
        if (key.charAt(0) !== '$') {
          if (src[key] && src[key] !== value) {
            if (value.length) {
              value += (key === 'style' ? ';' : ' ') + src[key];
            } else {
              value = src[key];
            }
          }
          dst.$set(key, value, true, srcAttr[key]);
        }
      });

      // copy the new attributes on the old attrs object
      forEach(src, function(value, key) {
        // Check if we already set this attribute in the loop above.
        // `dst` will never contain hasOwnProperty as DOM parser won't let it.
        // You will get an "InvalidCharacterError: DOM Exception 5" error if you
        // have an attribute like "has-own-property" or "data-has-own-property", etc.
        if (!dst.hasOwnProperty(key) && key.charAt(0) !== '$') {
          dst[key] = value;

          if (key !== 'class' && key !== 'style') {
            dstAttr[key] = srcAttr[key];
          }
        }
      });
    }


    function compileTemplateUrl(directives, $compileNode, tAttrs,
        $rootElement, childTranscludeFn, preLinkFns, postLinkFns, previousCompileContext) {
      var linkQueue = [],
          afterTemplateNodeLinkFn,
          afterTemplateChildLinkFn,
          beforeTemplateCompileNode = $compileNode[0],
          origAsyncDirective = directives.shift(),
          derivedSyncDirective = inherit(origAsyncDirective, {
            templateUrl: null, transclude: null, replace: null, $$originalDirective: origAsyncDirective
          }),
          templateUrl = (isFunction(origAsyncDirective.templateUrl))
              ? origAsyncDirective.templateUrl($compileNode, tAttrs)
              : origAsyncDirective.templateUrl,
          templateNamespace = origAsyncDirective.templateNamespace;

      $compileNode.empty();

      $templateRequest(templateUrl)
        .then(function(content) {
          var compileNode, tempTemplateAttrs, $template, childBoundTranscludeFn;

          content = denormalizeTemplate(content);

          if (origAsyncDirective.replace) {
            if (jqLiteIsTextNode(content)) {
              $template = [];
            } else {
              $template = removeComments(wrapTemplate(templateNamespace, trim(content)));
            }
            compileNode = $template[0];

            if ($template.length !== 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
              throw $compileMinErr('tplrt',
                  'Template for directive \'{0}\' must have exactly one root element. {1}',
                  origAsyncDirective.name, templateUrl);
            }

            tempTemplateAttrs = {$attr: {}};
            replaceWith($rootElement, $compileNode, compileNode);
            var templateDirectives = collectDirectives(compileNode, [], tempTemplateAttrs);

            if (isObject(origAsyncDirective.scope)) {
              // the original directive that caused the template to be loaded async required
              // an isolate scope
              markDirectiveScope(templateDirectives, true);
            }
            directives = templateDirectives.concat(directives);
            mergeTemplateAttributes(tAttrs, tempTemplateAttrs);
          } else {
            compileNode = beforeTemplateCompileNode;
            $compileNode.html(content);
          }

          directives.unshift(derivedSyncDirective);

          afterTemplateNodeLinkFn = applyDirectivesToNode(directives, compileNode, tAttrs,
              childTranscludeFn, $compileNode, origAsyncDirective, preLinkFns, postLinkFns,
              previousCompileContext);
          forEach($rootElement, function(node, i) {
            if (node === compileNode) {
              $rootElement[i] = $compileNode[0];
            }
          });
          afterTemplateChildLinkFn = compileNodes($compileNode[0].childNodes, childTranscludeFn);

          while (linkQueue.length) {
            var scope = linkQueue.shift(),
                beforeTemplateLinkNode = linkQueue.shift(),
                linkRootElement = linkQueue.shift(),
                boundTranscludeFn = linkQueue.shift(),
                linkNode = $compileNode[0];

            if (scope.$$destroyed) continue;

            if (beforeTemplateLinkNode !== beforeTemplateCompileNode) {
              var oldClasses = beforeTemplateLinkNode.className;

              if (!(previousCompileContext.hasElementTranscludeDirective &&
                  origAsyncDirective.replace)) {
                // it was cloned therefore we have to clone as well.
                linkNode = jqLiteClone(compileNode);
              }
              replaceWith(linkRootElement, jqLite(beforeTemplateLinkNode), linkNode);

              // Copy in CSS classes from original node
              safeAddClass(jqLite(linkNode), oldClasses);
            }
            if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
              childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
            } else {
              childBoundTranscludeFn = boundTranscludeFn;
            }
            afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, linkNode, $rootElement,
              childBoundTranscludeFn);
          }
          linkQueue = null;
        }).catch(function(error) {
          if (isError(error)) {
            $exceptionHandler(error);
          }
        });

      return function delayedNodeLinkFn(ignoreChildLinkFn, scope, node, rootElement, boundTranscludeFn) {
        var childBoundTranscludeFn = boundTranscludeFn;
        if (scope.$$destroyed) return;
        if (linkQueue) {
          linkQueue.push(scope,
                         node,
                         rootElement,
                         childBoundTranscludeFn);
        } else {
          if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
            childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
          }
          afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, node, rootElement, childBoundTranscludeFn);
        }
      };
    }


    /**
     * Sorting function for bound directives.
     */
    function byPriority(a, b) {
      var diff = b.priority - a.priority;
      if (diff !== 0) return diff;
      if (a.name !== b.name) return (a.name < b.name) ? -1 : 1;
      return a.index - b.index;
    }

    function assertNoDuplicate(what, previousDirective, directive, element) {

      function wrapModuleNameIfDefined(moduleName) {
        return moduleName ?
          (' (module: ' + moduleName + ')') :
          '';
      }

      if (previousDirective) {
        throw $compileMinErr('multidir', 'Multiple directives [{0}{1}, {2}{3}] asking for {4} on: {5}',
            previousDirective.name, wrapModuleNameIfDefined(previousDirective.$$moduleName),
            directive.name, wrapModuleNameIfDefined(directive.$$moduleName), what, startingTag(element));
      }
    }


    function addTextInterpolateDirective(directives, text) {
      var interpolateFn = $interpolate(text, true);
      if (interpolateFn) {
        directives.push({
          priority: 0,
          compile: function textInterpolateCompileFn(templateNode) {
            var templateNodeParent = templateNode.parent(),
                hasCompileParent = !!templateNodeParent.length;

            // When transcluding a template that has bindings in the root
            // we don't have a parent and thus need to add the class during linking fn.
            if (hasCompileParent) compile.$$addBindingClass(templateNodeParent);

            return function textInterpolateLinkFn(scope, node) {
              var parent = node.parent();
              if (!hasCompileParent) compile.$$addBindingClass(parent);
              compile.$$addBindingInfo(parent, interpolateFn.expressions);
              scope.$watch(interpolateFn, function interpolateFnWatchAction(value) {
                node[0].nodeValue = value;
              });
            };
          }
        });
      }
    }


    function wrapTemplate(type, template) {
      type = lowercase(type || 'html');
      switch (type) {
      case 'svg':
      case 'math':
        var wrapper = window.document.createElement('div');
        wrapper.innerHTML = '<' + type + '>' + template + '</' + type + '>';
        return wrapper.childNodes[0].childNodes;
      default:
        return template;
      }
    }


    function getTrustedAttrContext(nodeName, attrNormalizedName) {
      if (attrNormalizedName === 'srcdoc') {
        return $sce.HTML;
      }
      // All nodes with src attributes require a RESOURCE_URL value, except for
      // img and various html5 media nodes, which require the MEDIA_URL context.
      if (attrNormalizedName === 'src' || attrNormalizedName === 'ngSrc') {
        if (['img', 'video', 'audio', 'source', 'track'].indexOf(nodeName) === -1) {
          return $sce.RESOURCE_URL;
        }
        return $sce.MEDIA_URL;
      } else if (attrNormalizedName === 'xlinkHref') {
        // Some xlink:href are okay, most aren't
        if (nodeName === 'image') return $sce.MEDIA_URL;
        if (nodeName === 'a') return $sce.URL;
        return $sce.RESOURCE_URL;
      } else if (
          // Formaction
          (nodeName === 'form' && attrNormalizedName === 'action') ||
          // If relative URLs can go where they are not expected to, then
          // all sorts of trust issues can arise.
          (nodeName === 'base' && attrNormalizedName === 'href') ||
          // links can be stylesheets or imports, which can run script in the current origin
          (nodeName === 'link' && attrNormalizedName === 'href')
      ) {
        return $sce.RESOURCE_URL;
      } else if (nodeName === 'a' && (attrNormalizedName === 'href' ||
                                 attrNormalizedName === 'ngHref')) {
        return $sce.URL;
      }
    }

    function getTrustedPropContext(nodeName, propNormalizedName) {
      var prop = propNormalizedName.toLowerCase();
      return PROP_CONTEXTS[nodeName + '|' + prop] || PROP_CONTEXTS['*|' + prop];
    }

    function sanitizeSrcsetPropertyValue(value) {
      return sanitizeSrcset($sce.valueOf(value), 'ng-prop-srcset');
    }
    function addPropertyDirective(node, directives, attrName, propName) {
      if (EVENT_HANDLER_ATTR_REGEXP.test(propName)) {
        throw $compileMinErr('nodomevents', 'Property bindings for HTML DOM event properties are disallowed');
      }

      var nodeName = nodeName_(node);
      var trustedContext = getTrustedPropContext(nodeName, propName);

      var sanitizer = identity;
      // Sanitize img[srcset] + source[srcset] values.
      if (propName === 'srcset' && (nodeName === 'img' || nodeName === 'source')) {
        sanitizer = sanitizeSrcsetPropertyValue;
      } else if (trustedContext) {
        sanitizer = $sce.getTrusted.bind($sce, trustedContext);
      }

      directives.push({
        priority: 100,
        compile: function ngPropCompileFn(_, attr) {
          var ngPropGetter = $parse(attr[attrName]);
          var ngPropWatch = $parse(attr[attrName], function sceValueOf(val) {
            // Unwrap the value to compare the actual inner safe value, not the wrapper object.
            return $sce.valueOf(val);
          });

          return {
            pre: function ngPropPreLinkFn(scope, $element) {
              function applyPropValue() {
                var propValue = ngPropGetter(scope);
                $element[0][propName] = sanitizer(propValue);
              }

              applyPropValue();
              scope.$watch(ngPropWatch, applyPropValue);
            }
          };
        }
      });
    }

    function addEventDirective(directives, attrName, eventName) {
      directives.push(
        createEventDirective($parse, $rootScope, $exceptionHandler, attrName, eventName, /*forceAsync=*/false)
      );
    }

    function addAttrInterpolateDirective(node, directives, value, name, isNgAttr) {
      var nodeName = nodeName_(node);
      var trustedContext = getTrustedAttrContext(nodeName, name);
      var mustHaveExpression = !isNgAttr;
      var allOrNothing = ALL_OR_NOTHING_ATTRS[name] || isNgAttr;

      var interpolateFn = $interpolate(value, mustHaveExpression, trustedContext, allOrNothing);

      // no interpolation found -> ignore
      if (!interpolateFn) return;

      if (name === 'multiple' && nodeName === 'select') {
        throw $compileMinErr('selmulti',
            'Binding to the \'multiple\' attribute is not supported. Element: {0}',
            startingTag(node));
      }

      if (EVENT_HANDLER_ATTR_REGEXP.test(name)) {
        throw $compileMinErr('nodomevents', 'Interpolations for HTML DOM event attributes are disallowed');
      }

      directives.push({
        priority: 100,
        compile: function() {
            return {
              pre: function attrInterpolatePreLinkFn(scope, element, attr) {
                var $$observers = (attr.$$observers || (attr.$$observers = createMap()));

                // If the attribute has changed since last $interpolate()ed
                var newValue = attr[name];
                if (newValue !== value) {
                  // we need to interpolate again since the attribute value has been updated
                  // (e.g. by another directive's compile function)
                  // ensure unset/empty values make interpolateFn falsy
                  interpolateFn = newValue && $interpolate(newValue, true, trustedContext, allOrNothing);
                  value = newValue;
                }

                // if attribute was updated so that there is no interpolation going on we don't want to
                // register any observers
                if (!interpolateFn) return;

                // initialize attr object so that it's ready in case we need the value for isolate
                // scope initialization, otherwise the value would not be available from isolate
                // directive's linking fn during linking phase
                attr[name] = interpolateFn(scope);

                ($$observers[name] || ($$observers[name] = [])).$$inter = true;
                (attr.$$observers && attr.$$observers[name].$$scope || scope).
                  $watch(interpolateFn, function interpolateFnWatchAction(newValue, oldValue) {
                    //special case for class attribute addition + removal
                    //so that class changes can tap into the animation
                    //hooks provided by the $animate service. Be sure to
                    //skip animations when the first digest occurs (when
                    //both the new and the old values are the same) since
                    //the CSS classes are the non-interpolated values
                    if (name === 'class' && newValue !== oldValue) {
                      attr.$updateClass(newValue, oldValue);
                    } else {
                      attr.$set(name, newValue);
                    }
                  });
              }
            };
          }
      });
    }


    /**
     * This is a special jqLite.replaceWith, which can replace items which
     * have no parents, provided that the containing jqLite collection is provided.
     *
     * @param {JqLite=} $rootElement The root of the compile tree. Used so that we can replace nodes
     *                               in the root of the tree.
     * @param {JqLite} elementsToRemove The jqLite element which we are going to replace. We keep
     *                                  the shell, but replace its DOM node reference.
     * @param {Node} newNode The new DOM node.
     */
    function replaceWith($rootElement, elementsToRemove, newNode) {
      var firstElementToRemove = elementsToRemove[0],
          removeCount = elementsToRemove.length,
          parent = firstElementToRemove.parentNode,
          i, ii;

      if ($rootElement) {
        for (i = 0, ii = $rootElement.length; i < ii; i++) {
          if ($rootElement[i] === firstElementToRemove) {
            $rootElement[i++] = newNode;
            for (var j = i, j2 = j + removeCount - 1,
                     jj = $rootElement.length;
                 j < jj; j++, j2++) {
              if (j2 < jj) {
                $rootElement[j] = $rootElement[j2];
              } else {
                delete $rootElement[j];
              }
            }
            $rootElement.length -= removeCount - 1;

            // If the replaced element is also the jQuery .context then replace it
            // .context is a deprecated jQuery api, so we should set it only when jQuery set it
            // http://api.jquery.com/context/
            if ($rootElement.context === firstElementToRemove) {
              $rootElement.context = newNode;
            }
            break;
          }
        }
      }

      if (parent) {
        parent.replaceChild(newNode, firstElementToRemove);
      }

      // Append all the `elementsToRemove` to a fragment. This will...
      // - remove them from the DOM
      // - allow them to still be traversed with .nextSibling
      // - allow a single fragment.qSA to fetch all elements being removed
      var fragment = window.document.createDocumentFragment();
      for (i = 0; i < removeCount; i++) {
        fragment.appendChild(elementsToRemove[i]);
      }

      if (jqLite.hasData(firstElementToRemove)) {
        // Copy over user data (that includes AngularJS's $scope etc.). Don't copy private
        // data here because there's no public interface in jQuery to do that and copying over
        // event listeners (which is the main use of private data) wouldn't work anyway.
        jqLite.data(newNode, jqLite.data(firstElementToRemove));

        // Remove $destroy event listeners from `firstElementToRemove`
        jqLite(firstElementToRemove).off('$destroy');
      }

      // Cleanup any data/listeners on the elements and children.
      // This includes invoking the $destroy event on any elements with listeners.
      jqLite.cleanData(fragment.querySelectorAll('*'));

      // Update the jqLite collection to only contain the `newNode`
      for (i = 1; i < removeCount; i++) {
        delete elementsToRemove[i];
      }
      elementsToRemove[0] = newNode;
      elementsToRemove.length = 1;
    }


    function cloneAndAnnotateFn(fn, annotation) {
      return extend(function() { return fn.apply(null, arguments); }, fn, annotation);
    }


    function invokeLinkFn(linkFn, scope, $element, attrs, controllers, transcludeFn) {
      try {
        linkFn(scope, $element, attrs, controllers, transcludeFn);
      } catch (e) {
        $exceptionHandler(e, startingTag($element));
      }
    }

    function strictBindingsCheck(attrName, directiveName) {
      if (strictComponentBindingsEnabled) {
        throw $compileMinErr('missingattr',
          'Attribute \'{0}\' of \'{1}\' is non-optional and must be set!',
          attrName, directiveName);
      }
    }

    // Set up $watches for isolate scope and controller bindings.
    function initializeDirectiveBindings(scope, attrs, destination, bindings, directive) {
      var removeWatchCollection = [];
      var initialChanges = {};
      var changes;

      forEach(bindings, function initializeBinding(definition, scopeName) {
        var attrName = definition.attrName,
        optional = definition.optional,
        mode = definition.mode, // @, =, <, or &
        lastValue,
        parentGet, parentSet, compare, removeWatch;

        switch (mode) {

          case '@':
            if (!optional && !hasOwnProperty.call(attrs, attrName)) {
              strictBindingsCheck(attrName, directive.name);
              destination[scopeName] = attrs[attrName] = undefined;

            }
            removeWatch = attrs.$observe(attrName, function(value) {
              if (isString(value) || isBoolean(value)) {
                var oldValue = destination[scopeName];
                recordChanges(scopeName, value, oldValue);
                destination[scopeName] = value;
              }
            });
            attrs.$$observers[attrName].$$scope = scope;
            lastValue = attrs[attrName];
            if (isString(lastValue)) {
              // If the attribute has been provided then we trigger an interpolation to ensure
              // the value is there for use in the link fn
              destination[scopeName] = $interpolate(lastValue)(scope);
            } else if (isBoolean(lastValue)) {
              // If the attributes is one of the BOOLEAN_ATTR then AngularJS will have converted
              // the value to boolean rather than a string, so we special case this situation
              destination[scopeName] = lastValue;
            }
            initialChanges[scopeName] = new SimpleChange(_UNINITIALIZED_VALUE, destination[scopeName]);
            removeWatchCollection.push(removeWatch);
            break;

          case '=':
            if (!hasOwnProperty.call(attrs, attrName)) {
              if (optional) break;
              strictBindingsCheck(attrName, directive.name);
              attrs[attrName] = undefined;
            }
            if (optional && !attrs[attrName]) break;

            parentGet = $parse(attrs[attrName]);
            if (parentGet.literal) {
              compare = equals;
            } else {
              compare = simpleCompare;
            }
            parentSet = parentGet.assign || function() {
              // reset the change, or we will throw this exception on every $digest
              lastValue = destination[scopeName] = parentGet(scope);
              throw $compileMinErr('nonassign',
                  'Expression \'{0}\' in attribute \'{1}\' used with directive \'{2}\' is non-assignable!',
                  attrs[attrName], attrName, directive.name);
            };
            lastValue = destination[scopeName] = parentGet(scope);
            var parentValueWatch = function parentValueWatch(parentValue) {
              if (!compare(parentValue, destination[scopeName])) {
                // we are out of sync and need to copy
                if (!compare(parentValue, lastValue)) {
                  // parent changed and it has precedence
                  destination[scopeName] = parentValue;
                } else {
                  // if the parent can be assigned then do so
                  parentSet(scope, parentValue = destination[scopeName]);
                }
              }
              lastValue = parentValue;
              return lastValue;
            };
            parentValueWatch.$stateful = true;
            if (definition.collection) {
              removeWatch = scope.$watchCollection(attrs[attrName], parentValueWatch);
            } else {
              removeWatch = scope.$watch($parse(attrs[attrName], parentValueWatch), null, parentGet.literal);
            }
            removeWatchCollection.push(removeWatch);
            break;

          case '<':
            if (!hasOwnProperty.call(attrs, attrName)) {
              if (optional) break;
              strictBindingsCheck(attrName, directive.name);
              attrs[attrName] = undefined;
            }
            if (optional && !attrs[attrName]) break;

            parentGet = $parse(attrs[attrName]);
            var isLiteral = parentGet.literal;

            var initialValue = destination[scopeName] = parentGet(scope);
            initialChanges[scopeName] = new SimpleChange(_UNINITIALIZED_VALUE, destination[scopeName]);

            removeWatch = scope[definition.collection ? '$watchCollection' : '$watch'](parentGet, function parentValueWatchAction(newValue, oldValue) {
              if (oldValue === newValue) {
                if (oldValue === initialValue || (isLiteral && equals(oldValue, initialValue))) {
                  return;
                }
                oldValue = initialValue;
              }
              recordChanges(scopeName, newValue, oldValue);
              destination[scopeName] = newValue;
            });

            removeWatchCollection.push(removeWatch);
            break;

          case '&':
            if (!optional && !hasOwnProperty.call(attrs, attrName)) {
              strictBindingsCheck(attrName, directive.name);
            }
            // Don't assign Object.prototype method to scope
            parentGet = attrs.hasOwnProperty(attrName) ? $parse(attrs[attrName]) : noop;

            // Don't assign noop to destination if expression is not valid
            if (parentGet === noop && optional) break;

            destination[scopeName] = function(locals) {
              return parentGet(scope, locals);
            };
            break;
        }
      });

      function recordChanges(key, currentValue, previousValue) {
        if (isFunction(destination.$onChanges) && !simpleCompare(currentValue, previousValue)) {
          // If we have not already scheduled the top level onChangesQueue handler then do so now
          if (!onChangesQueue) {
            scope.$$postDigest(flushOnChangesQueue);
            onChangesQueue = [];
          }
          // If we have not already queued a trigger of onChanges for this controller then do so now
          if (!changes) {
            changes = {};
            onChangesQueue.push(triggerOnChangesHook);
          }
          // If the has been a change on this property already then we need to reuse the previous value
          if (changes[key]) {
            previousValue = changes[key].previousValue;
          }
          // Store this change
          changes[key] = new SimpleChange(previousValue, currentValue);
        }
      }

      function triggerOnChangesHook() {
        destination.$onChanges(changes);
        // Now clear the changes so that we schedule onChanges when more changes arrive
        changes = undefined;
      }

      return {
        initialChanges: initialChanges,
        removeWatches: removeWatchCollection.length && function removeWatches() {
          for (var i = 0, ii = removeWatchCollection.length; i < ii; ++i) {
            removeWatchCollection[i]();
          }
        }
      };
    }
  }];
}

function SimpleChange(previous, current) {
  this.previousValue = previous;
  this.currentValue = current;
}
SimpleChange.prototype.isFirstChange = function() { return this.previousValue === _UNINITIALIZED_VALUE; };


var PREFIX_REGEXP = /^((?:x|data)[:\-_])/i;
var SPECIAL_CHARS_REGEXP = /[:\-_]+(.)/g;

/**
 * Converts all accepted directives format into proper directive name.
 * @param name Name to normalize
 */
function directiveNormalize(name) {
  return name
    .replace(PREFIX_REGEXP, '')
    .replace(SPECIAL_CHARS_REGEXP, function(_, letter, offset) {
      return offset ? letter.toUpperCase() : letter;
    });
}

/**
 * @ngdoc type
 * @name $compile.directive.Attributes
 *
 * @description
 * A shared object between directive compile / linking functions which contains normalized DOM
 * element attributes. The values reflect current binding state `{{ }}`. The normalization is
 * needed since all of these are treated as equivalent in AngularJS:
 *
 * ```
 *    <span ng:bind="a" ng-bind="a" data-ng-bind="a" x-ng-bind="a">
 * ```
 */

/**
 * @ngdoc property
 * @name $compile.directive.Attributes#$attr
 *
 * @description
 * A map of DOM element attribute names to the normalized name. This is
 * needed to do reverse lookup from normalized name back to actual name.
 */


/**
 * @ngdoc method
 * @name $compile.directive.Attributes#$set
 * @kind function
 *
 * @description
 * Set DOM element attribute value.
 *
 *
 * @param {string} name Normalized element attribute name of the property to modify. The name is
 *          reverse-translated using the {@link ng.$compile.directive.Attributes#$attr $attr}
 *          property to the original name.
 * @param {string} value Value to set the attribute to. The value can be an interpolated string.
 */



/**
 * Closure compiler type information
 */

function nodesetLinkingFn(
  /* angular.Scope */ scope,
  /* NodeList */ nodeList,
  /* Element */ rootElement,
  /* function(Function) */ boundTranscludeFn
) {}

function directiveLinkingFn(
  /* nodesetLinkingFn */ nodesetLinkingFn,
  /* angular.Scope */ scope,
  /* Node */ node,
  /* Element */ rootElement,
  /* function(Function) */ boundTranscludeFn
) {}

function tokenDifference(str1, str2) {
  var values = '',
      tokens1 = str1.split(/\s+/),
      tokens2 = str2.split(/\s+/);

  outer:
  for (var i = 0; i < tokens1.length; i++) {
    var token = tokens1[i];
    for (var j = 0; j < tokens2.length; j++) {
      if (token === tokens2[j]) continue outer;
    }
    values += (values.length > 0 ? ' ' : '') + token;
  }
  return values;
}

function removeComments(jqNodes) {
  jqNodes = jqLite(jqNodes);
  var i = jqNodes.length;

  if (i <= 1) {
    return jqNodes;
  }

  while (i--) {
    var node = jqNodes[i];
    if (node.nodeType === NODE_TYPE_COMMENT ||
       (node.nodeType === NODE_TYPE_TEXT && node.nodeValue.trim() === '')) {
         splice.call(jqNodes, i, 1);
    }
  }
  return jqNodes;
}

var $controllerMinErr = minErr('$controller');


var CNTRL_REG = /^(\S+)(\s+as\s+([\w$]+))?$/;
function identifierForController(controller, ident) {
  if (ident && isString(ident)) return ident;
  if (isString(controller)) {
    var match = CNTRL_REG.exec(controller);
    if (match) return match[3];
  }
}


/**
 * @ngdoc provider
 * @name $controllerProvider
 * @this
 *
 * @description
 * The {@link ng.$controller $controller service} is used by AngularJS to create new
 * controllers.
 *
 * This provider allows controller registration via the
 * {@link ng.$controllerProvider#register register} method.
 */
function $ControllerProvider() {
  var controllers = {};

  /**
   * @ngdoc method
   * @name $controllerProvider#has
   * @param {string} name Controller name to check.
   */
  this.has = function(name) {
    return controllers.hasOwnProperty(name);
  };

  /**
   * @ngdoc method
   * @name $controllerProvider#register
   * @param {string|Object} name Controller name, or an object map of controllers where the keys are
   *    the names and the values are the constructors.
   * @param {Function|Array} constructor Controller constructor fn (optionally decorated with DI
   *    annotations in the array notation).
   */
  this.register = function(name, constructor) {
    assertNotHasOwnProperty(name, 'controller');
    if (isObject(name)) {
      extend(controllers, name);
    } else {
      controllers[name] = constructor;
    }
  };

  this.$get = ['$injector', function($injector) {

    /**
     * @ngdoc service
     * @name $controller
     * @requires $injector
     *
     * @param {Function|string} constructor If called with a function then it's considered to be the
     *    controller constructor function. Otherwise it's considered to be a string which is used
     *    to retrieve the controller constructor using the following steps:
     *
     *    * check if a controller with given name is registered via `$controllerProvider`
     *    * check if evaluating the string on the current scope returns a constructor
     *
     *    The string can use the `controller as property` syntax, where the controller instance is published
     *    as the specified property on the `scope`; the `scope` must be injected into `locals` param for this
     *    to work correctly.
     *
     * @param {Object} locals Injection locals for Controller.
     * @return {Object} Instance of given controller.
     *
     * @description
     * `$controller` service is responsible for instantiating controllers.
     *
     * It's just a simple call to {@link auto.$injector $injector}, but extracted into
     * a service, so that one can override this service with [BC version](https://gist.github.com/1649788).
     */
    return function $controller(expression, locals, later, ident) {
      // PRIVATE API:
      //   param `later` --- indicates that the controller's constructor is invoked at a later time.
      //                     If true, $controller will allocate the object with the correct
      //                     prototype chain, but will not invoke the controller until a returned
      //                     callback is invoked.
      //   param `ident` --- An optional label which overrides the label parsed from the controller
      //                     expression, if any.
      var instance, match, constructor, identifier;
      later = later === true;
      if (ident && isString(ident)) {
        identifier = ident;
      }

      if (isString(expression)) {
        match = expression.match(CNTRL_REG);
        if (!match) {
          throw $controllerMinErr('ctrlfmt',
            'Badly formed controller string \'{0}\'. ' +
            'Must match `__name__ as __id__` or `__name__`.', expression);
        }
        constructor = match[1];
        identifier = identifier || match[3];
        expression = controllers.hasOwnProperty(constructor)
            ? controllers[constructor]
            : getter(locals.$scope, constructor, true);

        if (!expression) {
          throw $controllerMinErr('ctrlreg',
            'The controller with the name \'{0}\' is not registered.', constructor);
        }

        assertArgFn(expression, constructor, true);
      }

      if (later) {
        // Instantiate controller later:
        // This machinery is used to create an instance of the object before calling the
        // controller's constructor itself.
        //
        // This allows properties to be added to the controller before the constructor is
        // invoked. Primarily, this is used for isolate scope bindings in $compile.
        //
        // This feature is not intended for use by applications, and is thus not documented
        // publicly.
        // Object creation: http://jsperf.com/create-constructor/2
        var controllerPrototype = (isArray(expression) ?
          expression[expression.length - 1] : expression).prototype;
        instance = Object.create(controllerPrototype || null);

        if (identifier) {
          addIdentifier(locals, identifier, instance, constructor || expression.name);
        }

        return extend(function $controllerInit() {
          var result = $injector.invoke(expression, instance, locals, constructor);
          if (result !== instance && (isObject(result) || isFunction(result))) {
            instance = result;
            if (identifier) {
              // If result changed, re-assign controllerAs value to scope.
              addIdentifier(locals, identifier, instance, constructor || expression.name);
            }
          }
          return instance;
        }, {
          instance: instance,
          identifier: identifier
        });
      }

      instance = $injector.instantiate(expression, locals, constructor);

      if (identifier) {
        addIdentifier(locals, identifier, instance, constructor || expression.name);
      }

      return instance;
    };

    function addIdentifier(locals, identifier, instance, name) {
      if (!(locals && isObject(locals.$scope))) {
        throw minErr('$controller')('noscp',
          'Cannot export controller \'{0}\' as \'{1}\'! No $scope object provided via `locals`.',
          name, identifier);
      }

      locals.$scope[identifier] = instance;
    }
  }];
}

/**
 * @ngdoc service
 * @name $document
 * @requires $window
 * @this
 *
 * @description
 * A {@link angular.element jQuery or jqLite} wrapper for the browser's `window.document` object.
 *
 * @example
   <example module="documentExample" name="document">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <p>$document title: <b ng-bind="title"></b></p>
         <p>window.document title: <b ng-bind="windowTitle"></b></p>
       </div>
     </file>
     <file name="script.js">
       angular.module('documentExample', [])
         .controller('ExampleController', ['$scope', '$document', function($scope, $document) {
           $scope.title = $document[0].title;
           $scope.windowTitle = angular.element(window.document)[0].title;
         }]);
     </file>
   </example>
 */
function $DocumentProvider() {
  this.$get = ['$window', function(window) {
    return jqLite(window.document);
  }];
}


/**
 * @private
 * @this
 * Listens for document visibility change and makes the current status accessible.
 */
function $$IsDocumentHiddenProvider() {
  this.$get = ['$document', '$rootScope', function($document, $rootScope) {
    var doc = $document[0];
    var hidden = doc && doc.hidden;

    $document.on('visibilitychange', changeListener);

    $rootScope.$on('$destroy', function() {
      $document.off('visibilitychange', changeListener);
    });

    function changeListener() {
      hidden = doc.hidden;
    }

    return function() {
      return hidden;
    };
  }];
}

/**
 * @ngdoc service
 * @name $exceptionHandler
 * @requires ng.$log
 * @this
 *
 * @description
 * Any uncaught exception in AngularJS expressions is delegated to this service.
 * The default implementation simply delegates to `$log.error` which logs it into
 * the browser console.
 *
 * In unit tests, if `angular-mocks.js` is loaded, this service is overridden by
 * {@link ngMock.$exceptionHandler mock $exceptionHandler} which aids in testing.
 *
 * ## Example:
 *
 * The example below will overwrite the default `$exceptionHandler` in order to (a) log uncaught
 * errors to the backend for later inspection by the developers and (b) to use `$log.warn()` instead
 * of `$log.error()`.
 *
 * ```js
 *   angular.
 *     module('exceptionOverwrite', []).
 *     factory('$exceptionHandler', ['$log', 'logErrorsToBackend', function($log, logErrorsToBackend) {
 *       return function myExceptionHandler(exception, cause) {
 *         logErrorsToBackend(exception, cause);
 *         $log.warn(exception, cause);
 *       };
 *     }]);
 * ```
 *
 * <hr />
 * Note, that code executed in event-listeners (even those registered using jqLite's `on`/`bind`
 * methods) does not delegate exceptions to the {@link ng.$exceptionHandler $exceptionHandler}
 * (unless executed during a digest).
 *
 * If you wish, you can manually delegate exceptions, e.g.
 * `try { ... } catch(e) { $exceptionHandler(e); }`
 *
 * @param {Error} exception Exception associated with the error.
 * @param {string=} cause Optional information about the context in which
 *       the error was thrown.
 *
 */
function $ExceptionHandlerProvider() {
  this.$get = ['$log', function($log) {
    return function(exception, cause) {
      $log.error.apply($log, arguments);
    };
  }];
}

var $$ForceReflowProvider = /** @this */ function() {
  this.$get = ['$document', function($document) {
    return function(domNode) {
      //the line below will force the browser to perform a repaint so
      //that all the animated elements within the animation frame will
      //be properly updated and drawn on screen. This is required to
      //ensure that the preparation animation is properly flushed so that
      //the active state picks up from there. DO NOT REMOVE THIS LINE.
      //DO NOT OPTIMIZE THIS LINE. THE MINIFIER WILL REMOVE IT OTHERWISE WHICH
      //WILL RESULT IN AN UNPREDICTABLE BUG THAT IS VERY HARD TO TRACK DOWN AND
      //WILL TAKE YEARS AWAY FROM YOUR LIFE.
      if (domNode) {
        if (!domNode.nodeType && domNode instanceof jqLite) {
          domNode = domNode[0];
        }
      } else {
        domNode = $document[0].body;
      }
      return domNode.offsetWidth + 1;
    };
  }];
};

var APPLICATION_JSON = 'application/json';
var CONTENT_TYPE_APPLICATION_JSON = {'Content-Type': APPLICATION_JSON + ';charset=utf-8'};
var JSON_START = /^\[|^\{(?!\{)/;
var JSON_ENDS = {
  '[': /]$/,
  '{': /}$/
};
var JSON_PROTECTION_PREFIX = /^\)]\}',?\n/;
var $httpMinErr = minErr('$http');

function serializeValue(v) {
  if (isObject(v)) {
    return isDate(v) ? v.toISOString() : toJson(v);
  }
  return v;
}


/** @this */
function $HttpParamSerializerProvider() {
  /**
   * @ngdoc service
   * @name $httpParamSerializer
   * @description
   *
   * Default {@link $http `$http`} params serializer that converts objects to strings
   * according to the following rules:
   *
   * * `{'foo': 'bar'}` results in `foo=bar`
   * * `{'foo': Date.now()}` results in `foo=2015-04-01T09%3A50%3A49.262Z` (`toISOString()` and encoded representation of a Date object)
   * * `{'foo': ['bar', 'baz']}` results in `foo=bar&foo=baz` (repeated key for each array element)
   * * `{'foo': {'bar':'baz'}}` results in `foo=%7B%22bar%22%3A%22baz%22%7D` (stringified and encoded representation of an object)
   *
   * Note that serializer will sort the request parameters alphabetically.
   */

  this.$get = function() {
    return function ngParamSerializer(params) {
      if (!params) return '';
      var parts = [];
      forEachSorted(params, function(value, key) {
        if (value === null || isUndefined(value) || isFunction(value)) return;
        if (isArray(value)) {
          forEach(value, function(v) {
            parts.push(encodeUriQuery(key)  + '=' + encodeUriQuery(serializeValue(v)));
          });
        } else {
          parts.push(encodeUriQuery(key) + '=' + encodeUriQuery(serializeValue(value)));
        }
      });

      return parts.join('&');
    };
  };
}

/** @this */
function $HttpParamSerializerJQLikeProvider() {
  /**
   * @ngdoc service
   * @name $httpParamSerializerJQLike
   *
   * @description
   *
   * Alternative {@link $http `$http`} params serializer that follows
   * jQuery's [`param()`](http://api.jquery.com/jquery.param/) method logic.
   * The serializer will also sort the params alphabetically.
   *
   * To use it for serializing `$http` request parameters, set it as the `paramSerializer` property:
   *
   * ```js
   * $http({
   *   url: myUrl,
   *   method: 'GET',
   *   params: myParams,
   *   paramSerializer: '$httpParamSerializerJQLike'
   * });
   * ```
   *
   * It is also possible to set it as the default `paramSerializer` in the
   * {@link $httpProvider#defaults `$httpProvider`}.
   *
   * Additionally, you can inject the serializer and use it explicitly, for example to serialize
   * form data for submission:
   *
   * ```js
   * .controller(function($http, $httpParamSerializerJQLike) {
   *   //...
   *
   *   $http({
   *     url: myUrl,
   *     method: 'POST',
   *     data: $httpParamSerializerJQLike(myData),
   *     headers: {
   *       'Content-Type': 'application/x-www-form-urlencoded'
   *     }
   *   });
   *
   * });
   * ```
   *
   */
  this.$get = function() {
    return function jQueryLikeParamSerializer(params) {
      if (!params) return '';
      var parts = [];
      serialize(params, '', true);
      return parts.join('&');

      function serialize(toSerialize, prefix, topLevel) {
        if (isArray(toSerialize)) {
          forEach(toSerialize, function(value, index) {
            serialize(value, prefix + '[' + (isObject(value) ? index : '') + ']');
          });
        } else if (isObject(toSerialize) && !isDate(toSerialize)) {
          forEachSorted(toSerialize, function(value, key) {
            serialize(value, prefix +
                (topLevel ? '' : '[') +
                key +
                (topLevel ? '' : ']'));
          });
        } else {
          if (isFunction(toSerialize)) {
            toSerialize = toSerialize();
          }
          parts.push(encodeUriQuery(prefix) + '=' +
              (toSerialize == null ? '' : encodeUriQuery(serializeValue(toSerialize))));
        }
      }
    };
  };
}

function defaultHttpResponseTransform(data, headers) {
  if (isString(data)) {
    // Strip json vulnerability protection prefix and trim whitespace
    var tempData = data.replace(JSON_PROTECTION_PREFIX, '').trim();

    if (tempData) {
      var contentType = headers('Content-Type');
      var hasJsonContentType = contentType && (contentType.indexOf(APPLICATION_JSON) === 0);

      if (hasJsonContentType || isJsonLike(tempData)) {
        try {
          data = fromJson(tempData);
        } catch (e) {
          if (!hasJsonContentType) {
            return data;
          }
          throw $httpMinErr('baddata', 'Data must be a valid JSON object. Received: "{0}". ' +
          'Parse error: "{1}"', data, e);
        }
      }
    }
  }

  return data;
}

function isJsonLike(str) {
    var jsonStart = str.match(JSON_START);
    return jsonStart && JSON_ENDS[jsonStart[0]].test(str);
}

/**
 * Parse headers into key value object
 *
 * @param {string} headers Raw headers as a string
 * @returns {Object} Parsed headers as key value object
 */
function parseHeaders(headers) {
  var parsed = createMap(), i;

  function fillInParsed(key, val) {
    if (key) {
      parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
    }
  }

  if (isString(headers)) {
    forEach(headers.split('\n'), function(line) {
      i = line.indexOf(':');
      fillInParsed(lowercase(trim(line.substr(0, i))), trim(line.substr(i + 1)));
    });
  } else if (isObject(headers)) {
    forEach(headers, function(headerVal, headerKey) {
      fillInParsed(lowercase(headerKey), trim(headerVal));
    });
  }

  return parsed;
}


/**
 * Returns a function that provides access to parsed headers.
 *
 * Headers are lazy parsed when first requested.
 * @see parseHeaders
 *
 * @param {(string|Object)} headers Headers to provide access to.
 * @returns {function(string=)} Returns a getter function which if called with:
 *
 *   - if called with an argument returns a single header value or null
 *   - if called with no arguments returns an object containing all headers.
 */
function headersGetter(headers) {
  var headersObj;

  return function(name) {
    if (!headersObj) headersObj =  parseHeaders(headers);

    if (name) {
      var value = headersObj[lowercase(name)];
      if (value === undefined) {
        value = null;
      }
      return value;
    }

    return headersObj;
  };
}


/**
 * Chain all given functions
 *
 * This function is used for both request and response transforming
 *
 * @param {*} data Data to transform.
 * @param {function(string=)} headers HTTP headers getter fn.
 * @param {number} status HTTP status code of the response.
 * @param {(Function|Array.<Function>)} fns Function or an array of functions.
 * @returns {*} Transformed data.
 */
function transformData(data, headers, status, fns) {
  if (isFunction(fns)) {
    return fns(data, headers, status);
  }

  forEach(fns, function(fn) {
    data = fn(data, headers, status);
  });

  return data;
}


function isSuccess(status) {
  return 200 <= status && status < 300;
}


/**
 * @ngdoc provider
 * @name $httpProvider
 * @this
 *
 * @description
 * Use `$httpProvider` to change the default behavior of the {@link ng.$http $http} service.
 */
function $HttpProvider() {
  /**
   * @ngdoc property
   * @name $httpProvider#defaults
   * @description
   *
   * Object containing default values for all {@link ng.$http $http} requests.
   *
   * - **`defaults.cache`** - {boolean|Object} - A boolean value or object created with
   * {@link ng.$cacheFactory `$cacheFactory`} to enable or disable caching of HTTP responses
   * by default. See {@link $http#caching $http Caching} for more information.
   *
   * - **`defaults.headers`** - {Object} - Default headers for all $http requests.
   * Refer to {@link ng.$http#setting-http-headers $http} for documentation on
   * setting default headers.
   *     - **`defaults.headers.common`**
   *     - **`defaults.headers.post`**
   *     - **`defaults.headers.put`**
   *     - **`defaults.headers.patch`**
   *
   * - **`defaults.jsonpCallbackParam`** - `{string}` - the name of the query parameter that passes the name of the
   * callback in a JSONP request. The value of this parameter will be replaced with the expression generated by the
   * {@link $jsonpCallbacks} service. Defaults to `'callback'`.
   *
   * - **`defaults.paramSerializer`** - `{string|function(Object<string,string>):string}` - A function
   *  used to the prepare string representation of request parameters (specified as an object).
   *  If specified as string, it is interpreted as a function registered with the {@link auto.$injector $injector}.
   *  Defaults to {@link ng.$httpParamSerializer $httpParamSerializer}.
   *
   * - **`defaults.transformRequest`** -
   * `{Array<function(data, headersGetter)>|function(data, headersGetter)}` -
   * An array of functions (or a single function) which are applied to the request data.
   * By default, this is an array with one request transformation function:
   *
   *   - If the `data` property of the request configuration object contains an object, serialize it
   *     into JSON format.
   *
   * - **`defaults.transformResponse`** -
   * `{Array<function(data, headersGetter, status)>|function(data, headersGetter, status)}` -
   * An array of functions (or a single function) which are applied to the response data. By default,
   * this is an array which applies one response transformation function that does two things:
   *
   *  - If XSRF prefix is detected, strip it
   *    (see {@link ng.$http#security-considerations Security Considerations in the $http docs}).
   *  - If the `Content-Type` is `application/json` or the response looks like JSON,
   *    deserialize it using a JSON parser.
   *
   * - **`defaults.xsrfCookieName`** - {string} - Name of cookie containing the XSRF token.
   * Defaults value is `'XSRF-TOKEN'`.
   *
   * - **`defaults.xsrfHeaderName`** - {string} - Name of HTTP header to populate with the
   * XSRF token. Defaults value is `'X-XSRF-TOKEN'`.
   *
   */
  var defaults = this.defaults = {
    // transform incoming response data
    transformResponse: [defaultHttpResponseTransform],

    // transform outgoing request data
    transformRequest: [function(d) {
      return isObject(d) && !isFile(d) && !isBlob(d) && !isFormData(d) ? toJson(d) : d;
    }],

    // default headers
    headers: {
      common: {
        'Accept': 'application/json, text/plain, */*'
      },
      post:   shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
      put:    shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
      patch:  shallowCopy(CONTENT_TYPE_APPLICATION_JSON)
    },

    xsrfCookieName: 'XSRF-TOKEN',
    xsrfHeaderName: 'X-XSRF-TOKEN',

    paramSerializer: '$httpParamSerializer',

    jsonpCallbackParam: 'callback'
  };

  var useApplyAsync = false;
  /**
   * @ngdoc method
   * @name $httpProvider#useApplyAsync
   * @description
   *
   * Configure $http service to combine processing of multiple http responses received at around
   * the same time via {@link ng.$rootScope.Scope#$applyAsync $rootScope.$applyAsync}. This can result in
   * significant performance improvement for bigger applications that make many HTTP requests
   * concurrently (common during application bootstrap).
   *
   * Defaults to false. If no value is specified, returns the current configured value.
   *
   * @param {boolean=} value If true, when requests are loaded, they will schedule a deferred
   *    "apply" on the next tick, giving time for subsequent requests in a roughly ~10ms window
   *    to load and share the same digest cycle.
   *
   * @returns {boolean|Object} If a value is specified, returns the $httpProvider for chaining.
   *    otherwise, returns the current configured value.
   */
  this.useApplyAsync = function(value) {
    if (isDefined(value)) {
      useApplyAsync = !!value;
      return this;
    }
    return useApplyAsync;
  };

  /**
   * @ngdoc property
   * @name $httpProvider#interceptors
   * @description
   *
   * Array containing service factories for all synchronous or asynchronous {@link ng.$http $http}
   * pre-processing of request or postprocessing of responses.
   *
   * These service factories are ordered by request, i.e. they are applied in the same order as the
   * array, on request, but reverse order, on response.
   *
   * {@link ng.$http#interceptors Interceptors detailed info}
   */
  var interceptorFactories = this.interceptors = [];

  /**
   * @ngdoc property
   * @name $httpProvider#xsrfWhitelistedOrigins
   * @description
   *
   * Array containing URLs whose origins are trusted to receive the XSRF token. See the
   * {@link ng.$http#security-considerations Security Considerations} sections for more details on
   * XSRF.
   *
   * **Note:** An "origin" consists of the [URI scheme](https://en.wikipedia.org/wiki/URI_scheme),
   * the [hostname](https://en.wikipedia.org/wiki/Hostname) and the
   * [port number](https://en.wikipedia.org/wiki/Port_(computer_networking). For `http:` and
   * `https:`, the port number can be omitted if using th default ports (80 and 443 respectively).
   * Examples: `http://example.com`, `https://api.example.com:9876`
   *
   * <div class="alert alert-warning">
   *   It is not possible to whitelist specific URLs/paths. The `path`, `query` and `fragment` parts
   *   of a URL will be ignored. For example, `https://foo.com/path/bar?query=baz#fragment` will be
   *   treated as `https://foo.com`, meaning that **all** requests to URLs starting with
   *   `https://foo.com/` will include the XSRF token.
   * </div>
   *
   * @example
   *
   * ```js
   * // App served from `https://example.com/`.
   * angular.
   *   module('xsrfWhitelistedOriginsExample', []).
   *   config(['$httpProvider', function($httpProvider) {
   *     $httpProvider.xsrfWhitelistedOrigins.push('https://api.example.com');
   *   }]).
   *   run(['$http', function($http) {
   *     // The XSRF token will be sent.
   *     $http.get('https://api.example.com/preferences').then(...);
   *
   *     // The XSRF token will NOT be sent.
   *     $http.get('https://stats.example.com/activity').then(...);
   *   }]);
   * ```
   */
  var xsrfWhitelistedOrigins = this.xsrfWhitelistedOrigins = [];

  this.$get = ['$browser', '$httpBackend', '$$cookieReader', '$cacheFactory', '$rootScope', '$q', '$injector', '$sce',
      function($browser, $httpBackend, $$cookieReader, $cacheFactory, $rootScope, $q, $injector, $sce) {

    var defaultCache = $cacheFactory('$http');

    /**
     * Make sure that default param serializer is exposed as a function
     */
    defaults.paramSerializer = isString(defaults.paramSerializer) ?
      $injector.get(defaults.paramSerializer) : defaults.paramSerializer;

    /**
     * Interceptors stored in reverse order. Inner interceptors before outer interceptors.
     * The reversal is needed so that we can build up the interception chain around the
     * server request.
     */
    var reversedInterceptors = [];

    forEach(interceptorFactories, function(interceptorFactory) {
      reversedInterceptors.unshift(isString(interceptorFactory)
          ? $injector.get(interceptorFactory) : $injector.invoke(interceptorFactory));
    });

    /**
     * A function to check request URLs against a list of allowed origins.
     */
    var urlIsAllowedOrigin = urlIsAllowedOriginFactory(xsrfWhitelistedOrigins);

    /**
     * @ngdoc service
     * @kind function
     * @name $http
     * @requires ng.$httpBackend
     * @requires $cacheFactory
     * @requires $rootScope
     * @requires $q
     * @requires $injector
     *
     * @description
     * The `$http` service is a core AngularJS service that facilitates communication with the remote
     * HTTP servers via the browser's [XMLHttpRequest](https://developer.mozilla.org/en/xmlhttprequest)
     * object or via [JSONP](http://en.wikipedia.org/wiki/JSONP).
     *
     * For unit testing applications that use `$http` service, see
     * {@link ngMock.$httpBackend $httpBackend mock}.
     *
     * For a higher level of abstraction, please check out the {@link ngResource.$resource
     * $resource} service.
     *
     * The $http API is based on the {@link ng.$q deferred/promise APIs} exposed by
     * the $q service. While for simple usage patterns this doesn't matter much, for advanced usage
     * it is important to familiarize yourself with these APIs and the guarantees they provide.
     *
     *
     * ## General usage
     * The `$http` service is a function which takes a single argument — a {@link $http#usage configuration object} —
     * that is used to generate an HTTP request and returns  a {@link ng.$q promise} that is
     * resolved (request success) or rejected (request failure) with a
     * {@link ng.$http#$http-returns response} object.
     *
     * ```js
     *   // Simple GET request example:
     *   $http({
     *     method: 'GET',
     *     url: '/someUrl'
     *   }).then(function successCallback(response) {
     *       // this callback will be called asynchronously
     *       // when the response is available
     *     }, function errorCallback(response) {
     *       // called asynchronously if an error occurs
     *       // or server returns response with an error status.
     *     });
     * ```
     *
     *
     * ## Shortcut methods
     *
     * Shortcut methods are also available. All shortcut methods require passing in the URL, and
     * request data must be passed in for POST/PUT requests. An optional config can be passed as the
     * last argument.
     *
     * ```js
     *   $http.get('/someUrl', config).then(successCallback, errorCallback);
     *   $http.post('/someUrl', data, config).then(successCallback, errorCallback);
     * ```
     *
     * Complete list of shortcut methods:
     *
     * - {@link ng.$http#get $http.get}
     * - {@link ng.$http#head $http.head}
     * - {@link ng.$http#post $http.post}
     * - {@link ng.$http#put $http.put}
     * - {@link ng.$http#delete $http.delete}
     * - {@link ng.$http#jsonp $http.jsonp}
     * - {@link ng.$http#patch $http.patch}
     *
     *
     * ## Writing Unit Tests that use $http
     * When unit testing (using {@link ngMock ngMock}), it is necessary to call
     * {@link ngMock.$httpBackend#flush $httpBackend.flush()} to flush each pending
     * request using trained responses.
     *
     * ```
     * $httpBackend.expectGET(...);
     * $http.get(...);
     * $httpBackend.flush();
     * ```
     *
     * ## Setting HTTP Headers
     *
     * The $http service will automatically add certain HTTP headers to all requests. These defaults
     * can be fully configured by accessing the `$httpProvider.defaults.headers` configuration
     * object, which currently contains this default configuration:
     *
     * - `$httpProvider.defaults.headers.common` (headers that are common for all requests):
     *   - <code>Accept: application/json, text/plain, \*&#65279;/&#65279;\*</code>
     * - `$httpProvider.defaults.headers.post`: (header defaults for POST requests)
     *   - `Content-Type: application/json`
     * - `$httpProvider.defaults.headers.put` (header defaults for PUT requests)
     *   - `Content-Type: application/json`
     *
     * To add or overwrite these defaults, simply add or remove a property from these configuration
     * objects. To add headers for an HTTP method other than POST or PUT, simply add a new object
     * with the lowercased HTTP method name as the key, e.g.
     * `$httpProvider.defaults.headers.get = { 'My-Header' : 'value' }`.
     *
     * The defaults can also be set at runtime via the `$http.defaults` object in the same
     * fashion. For example:
     *
     * ```
     * module.run(function($http) {
     *   $http.defaults.headers.common.Authorization = 'Basic YmVlcDpib29w';
     * });
     * ```
     *
     * In addition, you can supply a `headers` property in the config object passed when
     * calling `$http(config)`, which overrides the defaults without changing them globally.
     *
     * To explicitly remove a header automatically added via $httpProvider.defaults.headers on a per request basis,
     * Use the `headers` property, setting the desired header to `undefined`. For example:
     *
     * ```js
     * var req = {
     *  method: 'POST',
     *  url: 'http://example.com',
     *  headers: {
     *    'Content-Type': undefined
     *  },
     *  data: { test: 'test' }
     * }
     *
     * $http(req).then(function(){...}, function(){...});
     * ```
     *
     * ## Transforming Requests and Responses
     *
     * Both requests and responses can be transformed using transformation functions: `transformRequest`
     * and `transformResponse`. These properties can be a single function that returns
     * the transformed value (`function(data, headersGetter, status)`) or an array of such transformation functions,
     * which allows you to `push` or `unshift` a new transformation function into the transformation chain.
     *
     * <div class="alert alert-warning">
     * **Note:** AngularJS does not make a copy of the `data` parameter before it is passed into the `transformRequest` pipeline.
     * That means changes to the properties of `data` are not local to the transform function (since Javascript passes objects by reference).
     * For example, when calling `$http.get(url, $scope.myObject)`, modifications to the object's properties in a transformRequest
     * function will be reflected on the scope and in any templates where the object is data-bound.
     * To prevent this, transform functions should have no side-effects.
     * If you need to modify properties, it is recommended to make a copy of the data, or create new object to return.
     * </div>
     *
     * ### Default Transformations
     *
     * The `$httpProvider` provider and `$http` service expose `defaults.transformRequest` and
     * `defaults.transformResponse` properties. If a request does not provide its own transformations
     * then these will be applied.
     *
     * You can augment or replace the default transformations by modifying these properties by adding to or
     * replacing the array.
     *
     * AngularJS provides the following default transformations:
     *
     * Request transformations (`$httpProvider.defaults.transformRequest` and `$http.defaults.transformRequest`) is
     * an array with one function that does the following:
     *
     * - If the `data` property of the request configuration object contains an object, serialize it
     *   into JSON format.
     *
     * Response transformations (`$httpProvider.defaults.transformResponse` and `$http.defaults.transformResponse`) is
     * an array with one function that does the following:
     *
     *  - If XSRF prefix is detected, strip it (see Security Considerations section below).
     *  - If the `Content-Type` is `application/json` or the response looks like JSON,
   *      deserialize it using a JSON parser.
     *
     *
     * ### Overriding the Default Transformations Per Request
     *
     * If you wish to override the request/response transformations only for a single request then provide
     * `transformRequest` and/or `transformResponse` properties on the configuration object passed
     * into `$http`.
     *
     * Note that if you provide these properties on the config object the default transformations will be
     * overwritten. If you wish to augment the default transformations then you must include them in your
     * local transformation array.
     *
     * The following code demonstrates adding a new response transformation to be run after the default response
     * transformations have been run.
     *
     * ```js
     * function appendTransform(defaults, transform) {
     *
     *   // We can't guarantee that the default transformation is an array
     *   defaults = angular.isArray(defaults) ? defaults : [defaults];
     *
     *   // Append the new transformation to the defaults
     *   return defaults.concat(transform);
     * }
     *
     * $http({
     *   url: '...',
     *   method: 'GET',
     *   transformResponse: appendTransform($http.defaults.transformResponse, function(value) {
     *     return doTransform(value);
     *   })
     * });
     * ```
     *
     *
     * ## Caching
     *
     * {@link ng.$http `$http`} responses are not cached by default. To enable caching, you must
     * set the config.cache value or the default cache value to TRUE or to a cache object (created
     * with {@link ng.$cacheFactory `$cacheFactory`}). If defined, the value of config.cache takes
     * precedence over the default cache value.
     *
     * In order to:
     *   * cache all responses - set the default cache value to TRUE or to a cache object
     *   * cache a specific response - set config.cache value to TRUE or to a cache object
     *
     * If caching is enabled, but neither the default cache nor config.cache are set to a cache object,
     * then the default `$cacheFactory("$http")` object is used.
     *
     * The default cache value can be set by updating the
     * {@link ng.$http#defaults `$http.defaults.cache`} property or the
     * {@link $httpProvider#defaults `$httpProvider.defaults.cache`} property.
     *
     * When caching is enabled, {@link ng.$http `$http`} stores the response from the server using
     * the relevant cache object. The next time the same request is made, the response is returned
     * from the cache without sending a request to the server.
     *
     * Take note that:
     *
     *   * Only GET and JSONP requests are cached.
     *   * The cache key is the request URL including search parameters; headers are not considered.
     *   * Cached responses are returned asynchronously, in the same way as responses from the server.
     *   * If multiple identical requests are made using the same cache, which is not yet populated,
     *     one request will be made to the server and remaining requests will return the same response.
     *   * A cache-control header on the response does not affect if or how responses are cached.
     *
     *
     * ## Interceptors
     *
     * Before you start creating interceptors, be sure to understand the
     * {@link ng.$q $q and deferred/promise APIs}.
     *
     * For purposes of global error handling, authentication, or any kind of synchronous or
     * asynchronous pre-processing of request or postprocessing of responses, it is desirable to be
     * able to intercept requests before they are handed to the server and
     * responses before they are handed over to the application code that
     * initiated these requests. The interceptors leverage the {@link ng.$q
     * promise APIs} to fulfill this need for both synchronous and asynchronous pre-processing.
     *
     * The interceptors are service factories that are registered with the `$httpProvider` by
     * adding them to the `$httpProvider.interceptors` array. The factory is called and
     * injected with dependencies (if specified) and returns the interceptor.
     *
     * There are two kinds of interceptors (and two kinds of rejection interceptors):
     *
     *   * `request`: interceptors get called with a http {@link $http#usage config} object. The function is free to
     *     modify the `config` object or create a new one. The function needs to return the `config`
     *     object directly, or a promise containing the `config` or a new `config` object.
     *   * `requestError`: interceptor gets called when a previous interceptor threw an error or
     *     resolved with a rejection.
     *   * `response`: interceptors get called with http `response` object. The function is free to
     *     modify the `response` object or create a new one. The function needs to return the `response`
     *     object directly, or as a promise containing the `response` or a new `response` object.
     *   * `responseError`: interceptor gets called when a previous interceptor threw an error or
     *     resolved with a rejection.
     *
     *
     * ```js
     *   // register the interceptor as a service
     *   $provide.factory('myHttpInterceptor', function($q, dependency1, dependency2) {
     *     return {
     *       // optional method
     *       'request': function(config) {
     *         // do something on success
     *         return config;
     *       },
     *
     *       // optional method
     *      'requestError': function(rejection) {
     *         // do something on error
     *         if (canRecover(rejection)) {
     *           return responseOrNewPromise
     *         }
     *         return $q.reject(rejection);
     *       },
     *
     *
     *
     *       // optional method
     *       'response': function(response) {
     *         // do something on success
     *         return response;
     *       },
     *
     *       // optional method
     *      'responseError': function(rejection) {
     *         // do something on error
     *         if (canRecover(rejection)) {
     *           return responseOrNewPromise
     *         }
     *         return $q.reject(rejection);
     *       }
     *     };
     *   });
     *
     *   $httpProvider.interceptors.push('myHttpInterceptor');
     *
     *
     *   // alternatively, register the interceptor via an anonymous factory
     *   $httpProvider.interceptors.push(function($q, dependency1, dependency2) {
     *     return {
     *      'request': function(config) {
     *          // same as above
     *       },
     *
     *       'response': function(response) {
     *          // same as above
     *       }
     *     };
     *   });
     * ```
     *
     * ## Security Considerations
     *
     * When designing web applications, consider security threats from:
     *
     * - [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
     * - [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery)
     *
     * Both server and the client must cooperate in order to eliminate these threats. AngularJS comes
     * pre-configured with strategies that address these issues, but for this to work backend server
     * cooperation is required.
     *
     * ### JSON Vulnerability Protection
     *
     * A [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
     * allows third party website to turn your JSON resource URL into
     * [JSONP](http://en.wikipedia.org/wiki/JSONP) request under some conditions. To
     * counter this your server can prefix all JSON requests with following string `")]}',\n"`.
     * AngularJS will automatically strip the prefix before processing it as JSON.
     *
     * For example if your server needs to return:
     * ```js
     * ['one','two']
     * ```
     *
     * which is vulnerable to attack, your server can return:
     * ```js
     * )]}',
     * ['one','two']
     * ```
     *
     * AngularJS will strip the prefix, before processing the JSON.
     *
     *
     * ### Cross Site Request Forgery (XSRF) Protection
     *
     * [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery) is an attack technique by
     * which the attacker can trick an authenticated user into unknowingly executing actions on your
     * website. AngularJS provides a mechanism to counter XSRF. When performing XHR requests, the
     * $http service reads a token from a cookie (by default, `XSRF-TOKEN`) and sets it as an HTTP
     * header (by default `X-XSRF-TOKEN`). Since only JavaScript that runs on your domain could read
     * the cookie, your server can be assured that the XHR came from JavaScript running on your
     * domain.
     *
     * To take advantage of this, your server needs to set a token in a JavaScript readable session
     * cookie called `XSRF-TOKEN` on the first HTTP GET request. On subsequent XHR requests the
     * server can verify that the cookie matches the `X-XSRF-TOKEN` HTTP header, and therefore be
     * sure that only JavaScript running on your domain could have sent the request. The token must
     * be unique for each user and must be verifiable by the server (to prevent the JavaScript from
     * making up its own tokens). We recommend that the token is a digest of your site's
     * authentication cookie with a [salt](https://en.wikipedia.org/wiki/Salt_(cryptography&#41;)
     * for added security.
     *
     * The header will &mdash; by default &mdash; **not** be set for cross-domain requests. This
     * prevents unauthorized servers (e.g. malicious or compromised 3rd-party APIs) from gaining
     * access to your users' XSRF tokens and exposing them to Cross Site Request Forgery. If you
     * want to, you can whitelist additional origins to also receive the XSRF token, by adding them
     * to {@link ng.$httpProvider#xsrfWhitelistedOrigins xsrfWhitelistedOrigins}. This might be
     * useful, for example, if your application, served from `example.com`, needs to access your API
     * at `api.example.com`.
     * See {@link ng.$httpProvider#xsrfWhitelistedOrigins $httpProvider.xsrfWhitelistedOrigins} for
     * more details.
     *
     * <div class="alert alert-danger">
     *   **Warning**<br />
     *   Only whitelist origins that you have control over and make sure you understand the
     *   implications of doing so.
     * </div>
     *
     * The name of the cookie and the header can be specified using the `xsrfCookieName` and
     * `xsrfHeaderName` properties of either `$httpProvider.defaults` at config-time,
     * `$http.defaults` at run-time, or the per-request config object.
     *
     * In order to prevent collisions in environments where multiple AngularJS apps share the
     * same domain or subdomain, we recommend that each application uses a unique cookie name.
     *
     *
     * @param {object} config Object describing the request to be made and how it should be
     *    processed. The object has following properties:
     *
     *    - **method** – `{string}` – HTTP method (e.g. 'GET', 'POST', etc)
     *    - **url** – `{string|TrustedObject}` – Absolute or relative URL of the resource that is being requested;
     *      or an object created by a call to `$sce.trustAsResourceUrl(url)`.
     *    - **params** – `{Object.<string|Object>}` – Map of strings or objects which will be serialized
     *      with the `paramSerializer` and appended as GET parameters.
     *    - **data** – `{string|Object}` – Data to be sent as the request message data.
     *    - **headers** – `{Object}` – Map of strings or functions which return strings representing
     *      HTTP headers to send to the server. If the return value of a function is null, the
     *      header will not be sent. Functions accept a config object as an argument.
     *    - **eventHandlers** - `{Object}` - Event listeners to be bound to the XMLHttpRequest object.
     *      To bind events to the XMLHttpRequest upload object, use `uploadEventHandlers`.
     *      The handler will be called in the context of a `$apply` block.
     *    - **uploadEventHandlers** - `{Object}` - Event listeners to be bound to the XMLHttpRequest upload
     *      object. To bind events to the XMLHttpRequest object, use `eventHandlers`.
     *      The handler will be called in the context of a `$apply` block.
     *    - **xsrfHeaderName** – `{string}` – Name of HTTP header to populate with the XSRF token.
     *    - **xsrfCookieName** – `{string}` – Name of cookie containing the XSRF token.
     *    - **transformRequest** –
     *      `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` –
     *      transform function or an array of such functions. The transform function takes the http
     *      request body and headers and returns its transformed (typically serialized) version.
     *      See {@link ng.$http#overriding-the-default-transformations-per-request
     *      Overriding the Default Transformations}
     *    - **transformResponse** –
     *      `{function(data, headersGetter, status)|Array.<function(data, headersGetter, status)>}` –
     *      transform function or an array of such functions. The transform function takes the http
     *      response body, headers and status and returns its transformed (typically deserialized) version.
     *      See {@link ng.$http#overriding-the-default-transformations-per-request
     *      Overriding the Default Transformations}
     *    - **paramSerializer** - `{string|function(Object<string,string>):string}` - A function used to
     *      prepare the string representation of request parameters (specified as an object).
     *      If specified as string, it is interpreted as function registered with the
     *      {@link $injector $injector}, which means you can create your own serializer
     *      by registering it as a {@link auto.$provide#service service}.
     *      The default serializer is the {@link $httpParamSerializer $httpParamSerializer};
     *      alternatively, you can use the {@link $httpParamSerializerJQLike $httpParamSerializerJQLike}
     *    - **cache** – `{boolean|Object}` – A boolean value or object created with
     *      {@link ng.$cacheFactory `$cacheFactory`} to enable or disable caching of the HTTP response.
     *      See {@link $http#caching $http Caching} for more information.
     *    - **timeout** – `{number|Promise}` – timeout in milliseconds, or {@link ng.$q promise}
     *      that should abort the request when resolved.
     *
     *      A numerical timeout or a promise returned from {@link ng.$timeout $timeout}, will set
     *      the `xhrStatus` in the {@link $http#$http-returns response} to "timeout", and any other
     *      resolved promise will set it to "abort", following standard XMLHttpRequest behavior.
     *
     *    - **withCredentials** - `{boolean}` - whether to set the `withCredentials` flag on the
     *      XHR object. See [requests with credentials](https://developer.mozilla.org/docs/Web/HTTP/Access_control_CORS#Requests_with_credentials)
     *      for more information.
     *    - **responseType** - `{string}` - see
     *      [XMLHttpRequest.responseType](https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest#xmlhttprequest-responsetype).
     *
     * @returns {HttpPromise} A {@link ng.$q `Promise}` that will be resolved (request success)
     *   or rejected (request failure) with a response object.
     *
     *   The response object has these properties:
     *
     *   - **data** – `{string|Object}` – The response body transformed with
     *     the transform functions.
     *   - **status** – `{number}` – HTTP status code of the response.
     *   - **headers** – `{function([headerName])}` – Header getter function.
     *   - **config** – `{Object}` – The configuration object that was used
     *     to generate the request.
     *   - **statusText** – `{string}` – HTTP status text of the response.
     *   - **xhrStatus** – `{string}` – Status of the XMLHttpRequest
     *     (`complete`, `error`, `timeout` or `abort`).
     *
     *
     *   A response status code between 200 and 299 is considered a success status
     *   and will result in the success callback being called. Any response status
     *   code outside of that range is considered an error status and will result
     *   in the error callback being called.
     *   Also, status codes less than -1 are normalized to zero. -1 usually means
     *   the request was aborted, e.g. using a `config.timeout`. More information
     *   about the status might be available in the `xhrStatus` property.
     *
     *   Note that if the response is a redirect, XMLHttpRequest will transparently
     *   follow it, meaning that the outcome (success or error) will be determined
     *   by the final response status code.
     *
     *
     * @property {Array.<Object>} pendingRequests Array of config objects for currently pending
     *   requests. This is primarily meant to be used for debugging purposes.
     *
     *
     * @example
<example module="httpExample" name="http-service">
<file name="index.html">
  <div ng-controller="FetchController">
    <select ng-model="method" aria-label="Request method">
      <option>GET</option>
      <option>JSONP</option>
    </select>
    <input type="text" ng-model="url" size="80" aria-label="URL" />
    <button id="fetchbtn" ng-click="fetch()">fetch</button><br>
    <button id="samplegetbtn" ng-click="updateModel('GET', 'http-hello.html')">Sample GET</button>
    <button id="samplejsonpbtn"
      ng-click="updateModel('JSONP',
                    'https://angularjs.org/greet.php?name=Super%20Hero')">
      Sample JSONP
    </button>
    <button id="invalidjsonpbtn"
      ng-click="updateModel('JSONP', 'https://angularjs.org/doesntexist')">
        Invalid JSONP
      </button>
    <pre>http status code: {{status}}</pre>
    <pre>http response data: {{data}}</pre>
  </div>
</file>
<file name="script.js">
  angular.module('httpExample', [])
    .config(['$sceDelegateProvider', function($sceDelegateProvider) {
      // We must whitelist the JSONP endpoint that we are using to show that we trust it
      $sceDelegateProvider.resourceUrlWhitelist([
        'self',
        'https://angularjs.org/**'
      ]);
    }])
    .controller('FetchController', ['$scope', '$http', '$templateCache',
      function($scope, $http, $templateCache) {
        $scope.method = 'GET';
        $scope.url = 'http-hello.html';

        $scope.fetch = function() {
          $scope.code = null;
          $scope.response = null;

          $http({method: $scope.method, url: $scope.url, cache: $templateCache}).
            then(function(response) {
              $scope.status = response.status;
              $scope.data = response.data;
            }, function(response) {
              $scope.data = response.data || 'Request failed';
              $scope.status = response.status;
          });
        };

        $scope.updateModel = function(method, url) {
          $scope.method = method;
          $scope.url = url;
        };
      }]);
</file>
<file name="http-hello.html">
  Hello, $http!
</file>
<file name="protractor.js" type="protractor">
  var status = element(by.binding('status'));
  var data = element(by.binding('data'));
  var fetchBtn = element(by.id('fetchbtn'));
  var sampleGetBtn = element(by.id('samplegetbtn'));
  var invalidJsonpBtn = element(by.id('invalidjsonpbtn'));

  it('should make an xhr GET request', function() {
    sampleGetBtn.click();
    fetchBtn.click();
    expect(status.getText()).toMatch('200');
    expect(data.getText()).toMatch(/Hello, \$http!/);
  });

// Commented out due to flakes. See https://github.com/angular/angular.js/issues/9185
// it('should make a JSONP request to angularjs.org', function() {
//   var sampleJsonpBtn = element(by.id('samplejsonpbtn'));
//   sampleJsonpBtn.click();
//   fetchBtn.click();
//   expect(status.getText()).toMatch('200');
//   expect(data.getText()).toMatch(/Super Hero!/);
// });

  it('should make JSONP request to invalid URL and invoke the error handler',
      function() {
    invalidJsonpBtn.click();
    fetchBtn.click();
    expect(status.getText()).toMatch('0');
    expect(data.getText()).toMatch('Request failed');
  });
</file>
</example>
     */
    function $http(requestConfig) {

      if (!isObject(requestConfig)) {
        throw minErr('$http')('badreq', 'Http request configuration must be an object.  Received: {0}', requestConfig);
      }

      if (!isString($sce.valueOf(requestConfig.url))) {
        throw minErr('$http')('badreq', 'Http request configuration url must be a string or a $sce trusted object.  Received: {0}', requestConfig.url);
      }

      var config = extend({
        method: 'get',
        transformRequest: defaults.transformRequest,
        transformResponse: defaults.transformResponse,
        paramSerializer: defaults.paramSerializer,
        jsonpCallbackParam: defaults.jsonpCallbackParam
      }, requestConfig);

      config.headers = mergeHeaders(requestConfig);
      config.method = uppercase(config.method);
      config.paramSerializer = isString(config.paramSerializer) ?
          $injector.get(config.paramSerializer) : config.paramSerializer;

      $browser.$$incOutstandingRequestCount('$http');

      var requestInterceptors = [];
      var responseInterceptors = [];
      var promise = $q.resolve(config);

      // apply interceptors
      forEach(reversedInterceptors, function(interceptor) {
        if (interceptor.request || interceptor.requestError) {
          requestInterceptors.unshift(interceptor.request, interceptor.requestError);
        }
        if (interceptor.response || interceptor.responseError) {
          responseInterceptors.push(interceptor.response, interceptor.responseError);
        }
      });

      promise = chainInterceptors(promise, requestInterceptors);
      promise = promise.then(serverRequest);
      promise = chainInterceptors(promise, responseInterceptors);
      promise = promise.finally(completeOutstandingRequest);

      return promise;


      function chainInterceptors(promise, interceptors) {
        for (var i = 0, ii = interceptors.length; i < ii;) {
          var thenFn = interceptors[i++];
          var rejectFn = interceptors[i++];

          promise = promise.then(thenFn, rejectFn);
        }

        interceptors.length = 0;

        return promise;
      }

      function completeOutstandingRequest() {
        $browser.$$completeOutstandingRequest(noop, '$http');
      }

      function executeHeaderFns(headers, config) {
        var headerContent, processedHeaders = {};

        forEach(headers, function(headerFn, header) {
          if (isFunction(headerFn)) {
            headerContent = headerFn(config);
            if (headerContent != null) {
              processedHeaders[header] = headerContent;
            }
          } else {
            processedHeaders[header] = headerFn;
          }
        });

        return processedHeaders;
      }

      function mergeHeaders(config) {
        var defHeaders = defaults.headers,
            reqHeaders = extend({}, config.headers),
            defHeaderName, lowercaseDefHeaderName, reqHeaderName;

        defHeaders = extend({}, defHeaders.common, defHeaders[lowercase(config.method)]);

        // using for-in instead of forEach to avoid unnecessary iteration after header has been found
        defaultHeadersIteration:
        for (defHeaderName in defHeaders) {
          lowercaseDefHeaderName = lowercase(defHeaderName);

          for (reqHeaderName in reqHeaders) {
            if (lowercase(reqHeaderName) === lowercaseDefHeaderName) {
              continue defaultHeadersIteration;
            }
          }

          reqHeaders[defHeaderName] = defHeaders[defHeaderName];
        }

        // execute if header value is a function for merged headers
        return executeHeaderFns(reqHeaders, shallowCopy(config));
      }

      function serverRequest(config) {
        var headers = config.headers;
        var reqData = transformData(config.data, headersGetter(headers), undefined, config.transformRequest);

        // strip content-type if data is undefined
        if (isUndefined(reqData)) {
          forEach(headers, function(value, header) {
            if (lowercase(header) === 'content-type') {
              delete headers[header];
            }
          });
        }

        if (isUndefined(config.withCredentials) && !isUndefined(defaults.withCredentials)) {
          config.withCredentials = defaults.withCredentials;
        }

        // send request
        return sendReq(config, reqData).then(transformResponse, transformResponse);
      }

      function transformResponse(response) {
        // make a copy since the response must be cacheable
        var resp = extend({}, response);
        resp.data = transformData(response.data, response.headers, response.status,
                                  config.transformResponse);
        return (isSuccess(response.status))
          ? resp
          : $q.reject(resp);
      }
    }

    $http.pendingRequests = [];

    /**
     * @ngdoc method
     * @name $http#get
     *
     * @description
     * Shortcut method to perform `GET` request.
     *
     * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
     *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */

    /**
     * @ngdoc method
     * @name $http#delete
     *
     * @description
     * Shortcut method to perform `DELETE` request.
     *
     * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
     *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */

    /**
     * @ngdoc method
     * @name $http#head
     *
     * @description
     * Shortcut method to perform `HEAD` request.
     *
     * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
     *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */

    /**
     * @ngdoc method
     * @name $http#jsonp
     *
     * @description
     * Shortcut method to perform `JSONP` request.
     *
     * Note that, since JSONP requests are sensitive because the response is given full access to the browser,
     * the url must be declared, via {@link $sce} as a trusted resource URL.
     * You can trust a URL by adding it to the whitelist via
     * {@link $sceDelegateProvider#resourceUrlWhitelist  `$sceDelegateProvider.resourceUrlWhitelist`} or
     * by explicitly trusting the URL via {@link $sce#trustAsResourceUrl `$sce.trustAsResourceUrl(url)`}.
     *
     * You should avoid generating the URL for the JSONP request from user provided data.
     * Provide additional query parameters via `params` property of the `config` parameter, rather than
     * modifying the URL itself.
     *
     * JSONP requests must specify a callback to be used in the response from the server. This callback
     * is passed as a query parameter in the request. You must specify the name of this parameter by
     * setting the `jsonpCallbackParam` property on the request config object.
     *
     * ```
     * $http.jsonp('some/trusted/url', {jsonpCallbackParam: 'callback'})
     * ```
     *
     * You can also specify a default callback parameter name in `$http.defaults.jsonpCallbackParam`.
     * Initially this is set to `'callback'`.
     *
     * <div class="alert alert-danger">
     * You can no longer use the `JSON_CALLBACK` string as a placeholder for specifying where the callback
     * parameter value should go.
     * </div>
     *
     * If you would like to customise where and how the callbacks are stored then try overriding
     * or decorating the {@link $jsonpCallbacks} service.
     *
     * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
     *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */
    createShortMethods('get', 'delete', 'head', 'jsonp');

    /**
     * @ngdoc method
     * @name $http#post
     *
     * @description
     * Shortcut method to perform `POST` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {*} data Request content
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */

    /**
     * @ngdoc method
     * @name $http#put
     *
     * @description
     * Shortcut method to perform `PUT` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {*} data Request content
     * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
     * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
     * See {@link ng.$http#$http-returns `$http()` return value}.
     */

     /**
      * @ngdoc method
      * @name $http#patch
      *
      * @description
      * Shortcut method to perform `PATCH` request.
      *
      * @param {string} url Relative or absolute URL specifying the destination of the request
      * @param {*} data Request content
      * @param {Object=} config Optional configuration object. See {@link ng.$http#$http-arguments `$http()` arguments}.
      * @returns {HttpPromise}  A Promise that will be resolved or rejected with a response object.
      * See {@link ng.$http#$http-returns `$http()` return value}.
      */
    createShortMethodsWithData('post', 'put', 'patch');

        /**
         * @ngdoc property
         * @name $http#defaults
         *
         * @description
         * Runtime equivalent of the `$httpProvider.defaults` property. Allows configuration of
         * default headers, withCredentials as well as request and response transformations.
         *
         * See "Setting HTTP Headers" and "Transforming Requests and Responses" sections above.
         */
    $http.defaults = defaults;


    return $http;


    function createShortMethods(names) {
      forEach(arguments, function(name) {
        $http[name] = function(url, config) {
          return $http(extend({}, config || {}, {
            method: name,
            url: url
          }));
        };
      });
    }


    function createShortMethodsWithData(name) {
      forEach(arguments, function(name) {
        $http[name] = function(url, data, config) {
          return $http(extend({}, config || {}, {
            method: name,
            url: url,
            data: data
          }));
        };
      });
    }


    /**
     * Makes the request.
     *
     * !!! ACCESSES CLOSURE VARS:
     * $httpBackend, defaults, $log, $rootScope, defaultCache, $http.pendingRequests
     */
    function sendReq(config, reqData) {
      var deferred = $q.defer(),
          promise = deferred.promise,
          cache,
          cachedResp,
          reqHeaders = config.headers,
          isJsonp = lowercase(config.method) === 'jsonp',
          url = config.url;

      if (isJsonp) {
        // JSONP is a pretty sensitive operation where we're allowing a script to have full access to
        // our DOM and JS space.  So we require that the URL satisfies SCE.RESOURCE_URL.
        url = $sce.getTrustedResourceUrl(url);
      } else if (!isString(url)) {
        // If it is not a string then the URL must be a $sce trusted object
        url = $sce.valueOf(url);
      }

      url = buildUrl(url, config.paramSerializer(config.params));

      if (isJsonp) {
        // Check the url and add the JSONP callback placeholder
        url = sanitizeJsonpCallbackParam(url, config.jsonpCallbackParam);
      }

      $http.pendingRequests.push(config);
      promise.then(removePendingReq, removePendingReq);

      if ((config.cache || defaults.cache) && config.cache !== false &&
          (config.method === 'GET' || config.method === 'JSONP')) {
        cache = isObject(config.cache) ? config.cache
            : isObject(/** @type {?} */ (defaults).cache)
              ? /** @type {?} */ (defaults).cache
              : defaultCache;
      }

      if (cache) {
        cachedResp = cache.get(url);
        if (isDefined(cachedResp)) {
          if (isPromiseLike(cachedResp)) {
            // cached request has already been sent, but there is no response yet
            cachedResp.then(resolvePromiseWithResult, resolvePromiseWithResult);
          } else {
            // serving from cache
            if (isArray(cachedResp)) {
              resolvePromise(cachedResp[1], cachedResp[0], shallowCopy(cachedResp[2]), cachedResp[3], cachedResp[4]);
            } else {
              resolvePromise(cachedResp, 200, {}, 'OK', 'complete');
            }
          }
        } else {
          // put the promise for the non-transformed response into cache as a placeholder
          cache.put(url, promise);
        }
      }


      // if we won't have the response in cache, set the xsrf headers and
      // send the request to the backend
      if (isUndefined(cachedResp)) {
        var xsrfValue = urlIsAllowedOrigin(config.url)
            ? $$cookieReader()[config.xsrfCookieName || defaults.xsrfCookieName]
            : undefined;
        if (xsrfValue) {
          reqHeaders[(config.xsrfHeaderName || defaults.xsrfHeaderName)] = xsrfValue;
        }

        $httpBackend(config.method, url, reqData, done, reqHeaders, config.timeout,
            config.withCredentials, config.responseType,
            createApplyHandlers(config.eventHandlers),
            createApplyHandlers(config.uploadEventHandlers));
      }

      return promise;

      function createApplyHandlers(eventHandlers) {
        if (eventHandlers) {
          var applyHandlers = {};
          forEach(eventHandlers, function(eventHandler, key) {
            applyHandlers[key] = function(event) {
              if (useApplyAsync) {
                $rootScope.$applyAsync(callEventHandler);
              } else if ($rootScope.$$phase) {
                callEventHandler();
              } else {
                $rootScope.$apply(callEventHandler);
              }

              function callEventHandler() {
                eventHandler(event);
              }
            };
          });
          return applyHandlers;
        }
      }


      /**
       * Callback registered to $httpBackend():
       *  - caches the response if desired
       *  - resolves the raw $http promise
       *  - calls $apply
       */
      function done(status, response, headersString, statusText, xhrStatus) {
        if (cache) {
          if (isSuccess(status)) {
            cache.put(url, [status, response, parseHeaders(headersString), statusText, xhrStatus]);
          } else {
            // remove promise from the cache
            cache.remove(url);
          }
        }

        function resolveHttpPromise() {
          resolvePromise(response, status, headersString, statusText, xhrStatus);
        }

        if (useApplyAsync) {
          $rootScope.$applyAsync(resolveHttpPromise);
        } else {
          resolveHttpPromise();
          if (!$rootScope.$$phase) $rootScope.$apply();
        }
      }


      /**
       * Resolves the raw $http promise.
       */
      function resolvePromise(response, status, headers, statusText, xhrStatus) {
        //status: HTTP response status code, 0, -1 (aborted by timeout / promise)
        status = status >= -1 ? status : 0;

        (isSuccess(status) ? deferred.resolve : deferred.reject)({
          data: response,
          status: status,
          headers: headersGetter(headers),
          config: config,
          statusText: statusText,
          xhrStatus: xhrStatus
        });
      }

      function resolvePromiseWithResult(result) {
        resolvePromise(result.data, result.status, shallowCopy(result.headers()), result.statusText, result.xhrStatus);
      }

      function removePendingReq() {
        var idx = $http.pendingRequests.indexOf(config);
        if (idx !== -1) $http.pendingRequests.splice(idx, 1);
      }
    }


    function buildUrl(url, serializedParams) {
      if (serializedParams.length > 0) {
        url += ((url.indexOf('?') === -1) ? '?' : '&') + serializedParams;
      }
      return url;
    }

    function sanitizeJsonpCallbackParam(url, cbKey) {
      var parts = url.split('?');
      if (parts.length > 2) {
        // Throw if the url contains more than one `?` query indicator
        throw $httpMinErr('badjsonp', 'Illegal use more than one "?", in url, "{1}"', url);
      }
      var params = parseKeyValue(parts[1]);
      forEach(params, function(value, key) {
        if (value === 'JSON_CALLBACK') {
          // Throw if the url already contains a reference to JSON_CALLBACK
          throw $httpMinErr('badjsonp', 'Illegal use of JSON_CALLBACK in url, "{0}"', url);
        }
        if (key === cbKey) {
          // Throw if the callback param was already provided
          throw $httpMinErr('badjsonp', 'Illegal use of callback param, "{0}", in url, "{1}"', cbKey, url);
        }
      });

      // Add in the JSON_CALLBACK callback param value
      url += ((url.indexOf('?') === -1) ? '?' : '&') + cbKey + '=JSON_CALLBACK';

      return url;
    }
  }];
}

/**
 * @ngdoc service
 * @name $xhrFactory
 * @this
 *
 * @description
 * Factory function used to create XMLHttpRequest objects.
 *
 * Replace or decorate this service to create your own custom XMLHttpRequest objects.
 *
 * ```
 * angular.module('myApp', [])
 * .factory('$xhrFactory', function() {
 *   return function createXhr(method, url) {
 *     return new window.XMLHttpRequest({mozSystem: true});
 *   };
 * });
 * ```
 *
 * @param {string} method HTTP method of the request (GET, POST, PUT, ..)
 * @param {string} url URL of the request.
 */
function $xhrFactoryProvider() {
  this.$get = function() {
    return function createXhr() {
      return new window.XMLHttpRequest();
    };
  };
}

/**
 * @ngdoc service
 * @name $httpBackend
 * @requires $jsonpCallbacks
 * @requires $document
 * @requires $xhrFactory
 * @this
 *
 * @description
 * HTTP backend used by the {@link ng.$http service} that delegates to
 * XMLHttpRequest object or JSONP and deals with browser incompatibilities.
 *
 * You should never need to use this service directly, instead use the higher-level abstractions:
 * {@link ng.$http $http} or {@link ngResource.$resource $resource}.
 *
 * During testing this implementation is swapped with {@link ngMock.$httpBackend mock
 * $httpBackend} which can be trained with responses.
 */
function $HttpBackendProvider() {
  this.$get = ['$browser', '$jsonpCallbacks', '$document', '$xhrFactory', function($browser, $jsonpCallbacks, $document, $xhrFactory) {
    return createHttpBackend($browser, $xhrFactory, $browser.defer, $jsonpCallbacks, $document[0]);
  }];
}

function createHttpBackend($browser, createXhr, $browserDefer, callbacks, rawDocument) {
  // TODO(vojta): fix the signature
  return function(method, url, post, callback, headers, timeout, withCredentials, responseType, eventHandlers, uploadEventHandlers) {
    url = url || $browser.url();

    if (lowercase(method) === 'jsonp') {
      var callbackPath = callbacks.createCallback(url);
      var jsonpDone = jsonpReq(url, callbackPath, function(status, text) {
        // jsonpReq only ever sets status to 200 (OK), 404 (ERROR) or -1 (WAITING)
        var response = (status === 200) && callbacks.getResponse(callbackPath);
        completeRequest(callback, status, response, '', text, 'complete');
        callbacks.removeCallback(callbackPath);
      });
    } else {

      var xhr = createXhr(method, url);
      var abortedByTimeout = false;

      xhr.open(method, url, true);
      forEach(headers, function(value, key) {
        if (isDefined(value)) {
            xhr.setRequestHeader(key, value);
        }
      });

      xhr.onload = function requestLoaded() {
        var statusText = xhr.statusText || '';

        // responseText is the old-school way of retrieving response (supported by IE9)
        // response/responseType properties were introduced in XHR Level2 spec (supported by IE10)
        var response = ('response' in xhr) ? xhr.response : xhr.responseText;

        // normalize IE9 bug (http://bugs.jquery.com/ticket/1450)
        var status = xhr.status === 1223 ? 204 : xhr.status;

        // fix status code when it is 0 (0 status is undocumented).
        // Occurs when accessing file resources or on Android 4.1 stock browser
        // while retrieving files from application cache.
        if (status === 0) {
          status = response ? 200 : urlResolve(url).protocol === 'file' ? 404 : 0;
        }

        completeRequest(callback,
            status,
            response,
            xhr.getAllResponseHeaders(),
            statusText,
            'complete');
      };

      var requestError = function() {
        // The response is always empty
        // See https://xhr.spec.whatwg.org/#request-error-steps and https://fetch.spec.whatwg.org/#concept-network-error
        completeRequest(callback, -1, null, null, '', 'error');
      };

      var requestAborted = function() {
        completeRequest(callback, -1, null, null, '', abortedByTimeout ? 'timeout' : 'abort');
      };

      var requestTimeout = function() {
        // The response is always empty
        // See https://xhr.spec.whatwg.org/#request-error-steps and https://fetch.spec.whatwg.org/#concept-network-error
        completeRequest(callback, -1, null, null, '', 'timeout');
      };

      xhr.onerror = requestError;
      xhr.ontimeout = requestTimeout;
      xhr.onabort = requestAborted;

      forEach(eventHandlers, function(value, key) {
        xhr.addEventListener(key, value);
      });

      forEach(uploadEventHandlers, function(value, key) {
        xhr.upload.addEventListener(key, value);
      });

      if (withCredentials) {
        xhr.withCredentials = true;
      }

      if (responseType) {
        try {
          xhr.responseType = responseType;
        } catch (e) {
          // WebKit added support for the json responseType value on 09/03/2013
          // https://bugs.webkit.org/show_bug.cgi?id=73648. Versions of Safari prior to 7 are
          // known to throw when setting the value "json" as the response type. Other older
          // browsers implementing the responseType
          //
          // The json response type can be ignored if not supported, because JSON payloads are
          // parsed on the client-side regardless.
          if (responseType !== 'json') {
            throw e;
          }
        }
      }

      xhr.send(isUndefined(post) ? null : post);
    }

    // Since we are using xhr.abort() when a request times out, we have to set a flag that
    // indicates to requestAborted if the request timed out or was aborted.
    //
    // http.timeout = numerical timeout   timeout
    // http.timeout = $timeout            timeout
    // http.timeout = promise             abort
    // xhr.abort()                        abort (The xhr object is normally inaccessible, but
    //                                    can be exposed with the xhrFactory)
    if (timeout > 0) {
      var timeoutId = $browserDefer(function() {
        timeoutRequest('timeout');
      }, timeout);
    } else if (isPromiseLike(timeout)) {
      timeout.then(function() {
        timeoutRequest(isDefined(timeout.$$timeoutId) ? 'timeout' : 'abort');
      });
    }

    function timeoutRequest(reason) {
      abortedByTimeout = reason === 'timeout';
      if (jsonpDone) {
        jsonpDone();
      }
      if (xhr) {
        xhr.abort();
      }
    }

    function completeRequest(callback, status, response, headersString, statusText, xhrStatus) {
      // cancel timeout and subsequent timeout promise resolution
      if (isDefined(timeoutId)) {
        $browserDefer.cancel(timeoutId);
      }
      jsonpDone = xhr = null;

      callback(status, response, headersString, statusText, xhrStatus);
    }
  };

  function jsonpReq(url, callbackPath, done) {
    url = url.replace('JSON_CALLBACK', callbackPath);
    // we can't use jQuery/jqLite here because jQuery does crazy stuff with script elements, e.g.:
    // - fetches local scripts via XHR and evals them
    // - adds and immediately removes script elements from the document
    var script = rawDocument.createElement('script'), callback = null;
    script.type = 'text/javascript';
    script.src = url;
    script.async = true;

    callback = function(event) {
      script.removeEventListener('load', callback);
      script.removeEventListener('error', callback);
      rawDocument.body.removeChild(script);
      script = null;
      var status = -1;
      var text = 'unknown';

      if (event) {
        if (event.type === 'load' && !callbacks.wasCalled(callbackPath)) {
          event = { type: 'error' };
        }
        text = event.type;
        status = event.type === 'error' ? 404 : 200;
      }

      if (done) {
        done(status, text);
      }
    };

    script.addEventListener('load', callback);
    script.addEventListener('error', callback);
    rawDocument.body.appendChild(script);
    return callback;
  }
}

var $interpolateMinErr = angular.$interpolateMinErr = minErr('$interpolate');
$interpolateMinErr.throwNoconcat = function(text) {
  throw $interpolateMinErr('noconcat',
      'Error while interpolating: {0}\nStrict Contextual Escaping disallows ' +
      'interpolations that concatenate multiple expressions when a trusted value is ' +
      'required.  See http://docs.angularjs.org/api/ng.$sce', text);
};

$interpolateMinErr.interr = function(text, err) {
  return $interpolateMinErr('interr', 'Can\'t interpolate: {0}\n{1}', text, err.toString());
};

/**
 * @ngdoc provider
 * @name $interpolateProvider
 * @this
 *
 * @description
 *
 * Used for configuring the interpolation markup. Defaults to `{{` and `}}`.
 *
 * <div class="alert alert-danger">
 * This feature is sometimes used to mix different markup languages, e.g. to wrap an AngularJS
 * template within a Python Jinja template (or any other template language). Mixing templating
 * languages is **very dangerous**. The embedding template language will not safely escape AngularJS
 * expressions, so any user-controlled values in the template will cause Cross Site Scripting (XSS)
 * security bugs!
 * </div>
 *
 * @example
<example name="custom-interpolation-markup" module="customInterpolationApp">
<file name="index.html">
<script>
  var customInterpolationApp = angular.module('customInterpolationApp', []);

  customInterpolationApp.config(function($interpolateProvider) {
    $interpolateProvider.startSymbol('//');
    $interpolateProvider.endSymbol('//');
  });


  customInterpolationApp.controller('DemoController', function() {
      this.label = "This binding is brought you by // interpolation symbols.";
  });
</script>
<div ng-controller="DemoController as demo">
    //demo.label//
</div>
</file>
<file name="protractor.js" type="protractor">
  it('should interpolate binding with custom symbols', function() {
    expect(element(by.binding('demo.label')).getText()).toBe('This binding is brought you by // interpolation symbols.');
  });
</file>
</example>
 */
function $InterpolateProvider() {
  var startSymbol = '{{';
  var endSymbol = '}}';

  /**
   * @ngdoc method
   * @name $interpolateProvider#startSymbol
   * @description
   * Symbol to denote start of expression in the interpolated string. Defaults to `{{`.
   *
   * @param {string=} value new value to set the starting symbol to.
   * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
   */
  this.startSymbol = function(value) {
    if (value) {
      startSymbol = value;
      return this;
    }
    return startSymbol;
  };

  /**
   * @ngdoc method
   * @name $interpolateProvider#endSymbol
   * @description
   * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
   *
   * @param {string=} value new value to set the ending symbol to.
   * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
   */
  this.endSymbol = function(value) {
    if (value) {
      endSymbol = value;
      return this;
    }
    return endSymbol;
  };


  this.$get = ['$parse', '$exceptionHandler', '$sce', function($parse, $exceptionHandler, $sce) {
    var startSymbolLength = startSymbol.length,
        endSymbolLength = endSymbol.length,
        escapedStartRegexp = new RegExp(startSymbol.replace(/./g, escape), 'g'),
        escapedEndRegexp = new RegExp(endSymbol.replace(/./g, escape), 'g');

    function escape(ch) {
      return '\\\\\\' + ch;
    }

    function unescapeText(text) {
      return text.replace(escapedStartRegexp, startSymbol).
        replace(escapedEndRegexp, endSymbol);
    }

    // TODO: this is the same as the constantWatchDelegate in parse.js
    function constantWatchDelegate(scope, listener, objectEquality, constantInterp) {
      var unwatch = scope.$watch(function constantInterpolateWatch(scope) {
        unwatch();
        return constantInterp(scope);
      }, listener, objectEquality);
      return unwatch;
    }

    /**
     * @ngdoc service
     * @name $interpolate
     * @kind function
     *
     * @requires $parse
     * @requires $sce
     *
     * @description
     *
     * Compiles a string with markup into an interpolation function. This service is used by the
     * HTML {@link ng.$compile $compile} service for data binding. See
     * {@link ng.$interpolateProvider $interpolateProvider} for configuring the
     * interpolation markup.
     *
     *
     * ```js
     *   var $interpolate = ...; // injected
     *   var exp = $interpolate('Hello {{name | uppercase}}!');
     *   expect(exp({name:'AngularJS'})).toEqual('Hello ANGULARJS!');
     * ```
     *
     * `$interpolate` takes an optional fourth argument, `allOrNothing`. If `allOrNothing` is
     * `true`, the interpolation function will return `undefined` unless all embedded expressions
     * evaluate to a value other than `undefined`.
     *
     * ```js
     *   var $interpolate = ...; // injected
     *   var context = {greeting: 'Hello', name: undefined };
     *
     *   // default "forgiving" mode
     *   var exp = $interpolate('{{greeting}} {{name}}!');
     *   expect(exp(context)).toEqual('Hello !');
     *
     *   // "allOrNothing" mode
     *   exp = $interpolate('{{greeting}} {{name}}!', false, null, true);
     *   expect(exp(context)).toBeUndefined();
     *   context.name = 'AngularJS';
     *   expect(exp(context)).toEqual('Hello AngularJS!');
     * ```
     *
     * `allOrNothing` is useful for interpolating URLs. `ngSrc` and `ngSrcset` use this behavior.
     *
     * #### Escaped Interpolation
     * $interpolate provides a mechanism for escaping interpolation markers. Start and end markers
     * can be escaped by preceding each of their characters with a REVERSE SOLIDUS U+005C (backslash).
     * It will be rendered as a regular start/end marker, and will not be interpreted as an expression
     * or binding.
     *
     * This enables web-servers to prevent script injection attacks and defacing attacks, to some
     * degree, while also enabling code examples to work without relying on the
     * {@link ng.directive:ngNonBindable ngNonBindable} directive.
     *
     * **For security purposes, it is strongly encouraged that web servers escape user-supplied data,
     * replacing angle brackets (&lt;, &gt;) with &amp;lt; and &amp;gt; respectively, and replacing all
     * interpolation start/end markers with their escaped counterparts.**
     *
     * Escaped interpolation markers are only replaced with the actual interpolation markers in rendered
     * output when the $interpolate service processes the text. So, for HTML elements interpolated
     * by {@link ng.$compile $compile}, or otherwise interpolated with the `mustHaveExpression` parameter
     * set to `true`, the interpolated text must contain an unescaped interpolation expression. As such,
     * this is typically useful only when user-data is used in rendering a template from the server, or
     * when otherwise untrusted data is used by a directive.
     *
     * <example name="interpolation">
     *  <file name="index.html">
     *    <div ng-init="username='A user'">
     *      <p ng-init="apptitle='Escaping demo'">{{apptitle}}: \{\{ username = "defaced value"; \}\}
     *        </p>
     *      <p><strong>{{username}}</strong> attempts to inject code which will deface the
     *        application, but fails to accomplish their task, because the server has correctly
     *        escaped the interpolation start/end markers with REVERSE SOLIDUS U+005C (backslash)
     *        characters.</p>
     *      <p>Instead, the result of the attempted script injection is visible, and can be removed
     *        from the database by an administrator.</p>
     *    </div>
     *  </file>
     * </example>
     *
     * @knownIssue
     * It is currently not possible for an interpolated expression to contain the interpolation end
     * symbol. For example, `{{ '}}' }}` will be incorrectly interpreted as `{{ ' }}` + `' }}`, i.e.
     * an interpolated expression consisting of a single-quote (`'`) and the `' }}` string.
     *
     * @knownIssue
     * All directives and components must use the standard `{{` `}}` interpolation symbols
     * in their templates. If you change the application interpolation symbols the {@link $compile}
     * service will attempt to denormalize the standard symbols to the custom symbols.
     * The denormalization process is not clever enough to know not to replace instances of the standard
     * symbols where they would not normally be treated as interpolation symbols. For example in the following
     * code snippet the closing braces of the literal object will get incorrectly denormalized:
     *
     * ```
     * <div data-context='{"context":{"id":3,"type":"page"}}">
     * ```
     *
     * The workaround is to ensure that such instances are separated by whitespace:
     * ```
     * <div data-context='{"context":{"id":3,"type":"page"} }">
     * ```
     *
     * See https://github.com/angular/angular.js/pull/14610#issuecomment-219401099 for more information.
     *
     * @param {string} text The text with markup to interpolate.
     * @param {boolean=} mustHaveExpression if set to true then the interpolation string must have
     *    embedded expression in order to return an interpolation function. Strings with no
     *    embedded expression will return null for the interpolation function.
     * @param {string=} trustedContext when provided, the returned function passes the interpolated
     *    result through {@link ng.$sce#getTrusted $sce.getTrusted(interpolatedResult,
     *    trustedContext)} before returning it.  Refer to the {@link ng.$sce $sce} service that
     *    provides Strict Contextual Escaping for details.
     * @param {boolean=} allOrNothing if `true`, then the returned function returns undefined
     *    unless all embedded expressions evaluate to a value other than `undefined`.
     * @returns {function(context)} an interpolation function which is used to compute the
     *    interpolated string. The function has these parameters:
     *
     * - `context`: evaluation context for all expressions embedded in the interpolated text
     */
    function $interpolate(text, mustHaveExpression, trustedContext, allOrNothing) {
      var contextAllowsConcatenation = trustedContext === $sce.URL || trustedContext === $sce.MEDIA_URL;

      // Provide a quick exit and simplified result function for text with no interpolation
      if (!text.length || text.indexOf(startSymbol) === -1) {
        if (mustHaveExpression) return;

        var unescapedText = unescapeText(text);
        if (contextAllowsConcatenation) {
          unescapedText = $sce.getTrusted(trustedContext, unescapedText);
        }
        var constantInterp = valueFn(unescapedText);
        constantInterp.exp = text;
        constantInterp.expressions = [];
        constantInterp.$$watchDelegate = constantWatchDelegate;

        return constantInterp;
      }

      allOrNothing = !!allOrNothing;
      var startIndex,
          endIndex,
          index = 0,
          expressions = [],
          parseFns,
          textLength = text.length,
          exp,
          concat = [],
          expressionPositions = [],
          singleExpression;


      while (index < textLength) {
        if (((startIndex = text.indexOf(startSymbol, index)) !== -1) &&
             ((endIndex = text.indexOf(endSymbol, startIndex + startSymbolLength)) !== -1)) {
          if (index !== startIndex) {
            concat.push(unescapeText(text.substring(index, startIndex)));
          }
          exp = text.substring(startIndex + startSymbolLength, endIndex);
          expressions.push(exp);
          index = endIndex + endSymbolLength;
          expressionPositions.push(concat.length);
          concat.push(''); // Placeholder that will get replaced with the evaluated expression.
        } else {
          // we did not find an interpolation, so we have to add the remainder to the separators array
          if (index !== textLength) {
            concat.push(unescapeText(text.substring(index)));
          }
          break;
        }
      }

      singleExpression = concat.length === 1 && expressionPositions.length === 1;
      // Intercept expression if we need to stringify concatenated inputs, which may be SCE trusted
      // objects rather than simple strings
      // (we don't modify the expression if the input consists of only a single trusted input)
      var interceptor = contextAllowsConcatenation && singleExpression ? undefined : parseStringifyInterceptor;
      parseFns = expressions.map(function(exp) { return $parse(exp, interceptor); });

      // Concatenating expressions makes it hard to reason about whether some combination of
      // concatenated values are unsafe to use and could easily lead to XSS.  By requiring that a
      // single expression be used for some $sce-managed secure contexts (RESOURCE_URLs mostly),
      // we ensure that the value that's used is assigned or constructed by some JS code somewhere
      // that is more testable or make it obvious that you bound the value to some user controlled
      // value.  This helps reduce the load when auditing for XSS issues.

      // Note that URL and MEDIA_URL $sce contexts do not need this, since `$sce` can sanitize the values
      // passed to it. In that case, `$sce.getTrusted` will be called on either the single expression
      // or on the overall concatenated string (losing trusted types used in the mix, by design).
      // Both these methods will sanitize plain strings. Also, HTML could be included, but since it's
      // only used in srcdoc attributes, this would not be very useful.

      if (!mustHaveExpression || expressions.length) {
        var compute = function(values) {
          for (var i = 0, ii = expressions.length; i < ii; i++) {
            if (allOrNothing && isUndefined(values[i])) return;
            concat[expressionPositions[i]] = values[i];
          }

          if (contextAllowsConcatenation) {
            // If `singleExpression` then `concat[0]` might be a "trusted" value or `null`, rather than a string
            return $sce.getTrusted(trustedContext, singleExpression ? concat[0] : concat.join(''));
          } else if (trustedContext && concat.length > 1) {
            // This context does not allow more than one part, e.g. expr + string or exp + exp.
            $interpolateMinErr.throwNoconcat(text);
          }
          // In an unprivileged context or only one part: just concatenate and return.
          return concat.join('');
        };

        return extend(function interpolationFn(context) {
            var i = 0;
            var ii = expressions.length;
            var values = new Array(ii);

            try {
              for (; i < ii; i++) {
                values[i] = parseFns[i](context);
              }

              return compute(values);
            } catch (err) {
              $exceptionHandler($interpolateMinErr.interr(text, err));
            }

          }, {
          // all of these properties are undocumented for now
          exp: text, //just for compatibility with regular watchers created via $watch
          expressions: expressions,
          $$watchDelegate: function(scope, listener) {
            var lastValue;
            return scope.$watchGroup(parseFns, /** @this */ function interpolateFnWatcher(values, oldValues) {
              var currValue = compute(values);
              listener.call(this, currValue, values !== oldValues ? lastValue : currValue, scope);
              lastValue = currValue;
            });
          }
        });
      }

      function parseStringifyInterceptor(value) {
        try {
          // In concatenable contexts, getTrusted comes at the end, to avoid sanitizing individual
          // parts of a full URL. We don't care about losing the trustedness here.
          // In non-concatenable contexts, where there is only one expression, this interceptor is
          // not applied to the expression.
          value = (trustedContext && !contextAllowsConcatenation) ?
                    $sce.getTrusted(trustedContext, value) :
                    $sce.valueOf(value);
          return allOrNothing && !isDefined(value) ? value : stringify(value);
        } catch (err) {
          $exceptionHandler($interpolateMinErr.interr(text, err));
        }
      }
    }


    /**
     * @ngdoc method
     * @name $interpolate#startSymbol
     * @description
     * Symbol to denote the start of expression in the interpolated string. Defaults to `{{`.
     *
     * Use {@link ng.$interpolateProvider#startSymbol `$interpolateProvider.startSymbol`} to change
     * the symbol.
     *
     * @returns {string} start symbol.
     */
    $interpolate.startSymbol = function() {
      return startSymbol;
    };


    /**
     * @ngdoc method
     * @name $interpolate#endSymbol
     * @description
     * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
     *
     * Use {@link ng.$interpolateProvider#endSymbol `$interpolateProvider.endSymbol`} to change
     * the symbol.
     *
     * @returns {string} end symbol.
     */
    $interpolate.endSymbol = function() {
      return endSymbol;
    };

    return $interpolate;
  }];
}

var $intervalMinErr = minErr('$interval');

/** @this */
function $IntervalProvider() {
  this.$get = ['$$intervalFactory', '$window',
       function($$intervalFactory,   $window) {
    var intervals = {};
    var setIntervalFn = function(tick, delay, deferred) {
      var id = $window.setInterval(tick, delay);
      intervals[id] = deferred;
      return id;
    };
    var clearIntervalFn = function(id) {
      $window.clearInterval(id);
      delete intervals[id];
    };

    /**
     * @ngdoc service
     * @name $interval
     *
     * @description
     * AngularJS's wrapper for `window.setInterval`. The `fn` function is executed every `delay`
     * milliseconds.
     *
     * The return value of registering an interval function is a promise. This promise will be
     * notified upon each tick of the interval, and will be resolved after `count` iterations, or
     * run indefinitely if `count` is not defined. The value of the notification will be the
     * number of iterations that have run.
     * To cancel an interval, call `$interval.cancel(promise)`.
     *
     * In tests you can use {@link ngMock.$interval#flush `$interval.flush(millis)`} to
     * move forward by `millis` milliseconds and trigger any functions scheduled to run in that
     * time.
     *
     * <div class="alert alert-warning">
     * **Note**: Intervals created by this service must be explicitly destroyed when you are finished
     * with them.  In particular they are not automatically destroyed when a controller's scope or a
     * directive's element are destroyed.
     * You should take this into consideration and make sure to always cancel the interval at the
     * appropriate moment.  See the example below for more details on how and when to do this.
     * </div>
     *
     * @param {function()} fn A function that should be called repeatedly. If no additional arguments
     *   are passed (see below), the function is called with the current iteration count.
     * @param {number} delay Number of milliseconds between each function call.
     * @param {number=} [count=0] Number of times to repeat. If not set, or 0, will repeat
     *   indefinitely.
     * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
     *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
     * @param {...*=} Pass additional parameters to the executed function.
     * @returns {promise} A promise which will be notified on each iteration. It will resolve once all iterations of the interval complete.
     *
     * @example
     * <example module="intervalExample" name="interval-service">
     * <file name="index.html">
     *   <script>
     *     angular.module('intervalExample', [])
     *       .controller('ExampleController', ['$scope', '$interval',
     *         function($scope, $interval) {
     *           $scope.format = 'M/d/yy h:mm:ss a';
     *           $scope.blood_1 = 100;
     *           $scope.blood_2 = 120;
     *
     *           var stop;
     *           $scope.fight = function() {
     *             // Don't start a new fight if we are already fighting
     *             if ( angular.isDefined(stop) ) return;
     *
     *             stop = $interval(function() {
     *               if ($scope.blood_1 > 0 && $scope.blood_2 > 0) {
     *                 $scope.blood_1 = $scope.blood_1 - 3;
     *                 $scope.blood_2 = $scope.blood_2 - 4;
     *               } else {
     *                 $scope.stopFight();
     *               }
     *             }, 100);
     *           };
     *
     *           $scope.stopFight = function() {
     *             if (angular.isDefined(stop)) {
     *               $interval.cancel(stop);
     *               stop = undefined;
     *             }
     *           };
     *
     *           $scope.resetFight = function() {
     *             $scope.blood_1 = 100;
     *             $scope.blood_2 = 120;
     *           };
     *
     *           $scope.$on('$destroy', function() {
     *             // Make sure that the interval is destroyed too
     *             $scope.stopFight();
     *           });
     *         }])
     *       // Register the 'myCurrentTime' directive factory method.
     *       // We inject $interval and dateFilter service since the factory method is DI.
     *       .directive('myCurrentTime', ['$interval', 'dateFilter',
     *         function($interval, dateFilter) {
     *           // return the directive link function. (compile function not needed)
     *           return function(scope, element, attrs) {
     *             var format,  // date format
     *                 stopTime; // so that we can cancel the time updates
     *
     *             // used to update the UI
     *             function updateTime() {
     *               element.text(dateFilter(new Date(), format));
     *             }
     *
     *             // watch the expression, and update the UI on change.
     *             scope.$watch(attrs.myCurrentTime, function(value) {
     *               format = value;
     *               updateTime();
     *             });
     *
     *             stopTime = $interval(updateTime, 1000);
     *
     *             // listen on DOM destroy (removal) event, and cancel the next UI update
     *             // to prevent updating time after the DOM element was removed.
     *             element.on('$destroy', function() {
     *               $interval.cancel(stopTime);
     *             });
     *           }
     *         }]);
     *   </script>
     *
     *   <div>
     *     <div ng-controller="ExampleController">
     *       <label>Date format: <input ng-model="format"></label> <hr/>
     *       Current time is: <span my-current-time="format"></span>
     *       <hr/>
     *       Blood 1 : <font color='red'>{{blood_1}}</font>
     *       Blood 2 : <font color='red'>{{blood_2}}</font>
     *       <button type="button" data-ng-click="fight()">Fight</button>
     *       <button type="button" data-ng-click="stopFight()">StopFight</button>
     *       <button type="button" data-ng-click="resetFight()">resetFight</button>
     *     </div>
     *   </div>
     *
     * </file>
     * </example>
     */
    var interval = $$intervalFactory(setIntervalFn, clearIntervalFn);

    /**
     * @ngdoc method
     * @name $interval#cancel
     *
     * @description
     * Cancels a task associated with the `promise`.
     *
     * @param {Promise=} promise returned by the `$interval` function.
     * @returns {boolean} Returns `true` if the task was successfully canceled.
     */
    interval.cancel = function(promise) {
      if (!promise) return false;

      if (!promise.hasOwnProperty('$$intervalId')) {
        throw $intervalMinErr('badprom',
            '`$interval.cancel()` called with a promise that was not generated by `$interval()`.');
      }

      if (!intervals.hasOwnProperty(promise.$$intervalId)) return false;

      var id = promise.$$intervalId;
      var deferred = intervals[id];

      // Interval cancels should not report an unhandled promise.
      markQExceptionHandled(deferred.promise);
      deferred.reject('canceled');
      clearIntervalFn(id);

      return true;
    };

    return interval;
  }];
}

/** @this */
function $$IntervalFactoryProvider() {
  this.$get = ['$browser', '$q', '$$q', '$rootScope',
       function($browser,   $q,   $$q,   $rootScope) {
    return function intervalFactory(setIntervalFn, clearIntervalFn) {
      return function intervalFn(fn, delay, count, invokeApply) {
        var hasParams = arguments.length > 4,
            args = hasParams ? sliceArgs(arguments, 4) : [],
            iteration = 0,
            skipApply = isDefined(invokeApply) && !invokeApply,
            deferred = (skipApply ? $$q : $q).defer(),
            promise = deferred.promise;

        count = isDefined(count) ? count : 0;

        function callback() {
          if (!hasParams) {
            fn(iteration);
          } else {
            fn.apply(null, args);
          }
        }

        function tick() {
          if (skipApply) {
            $browser.defer(callback);
          } else {
            $rootScope.$evalAsync(callback);
          }
          deferred.notify(iteration++);

          if (count > 0 && iteration >= count) {
            deferred.resolve(iteration);
            clearIntervalFn(promise.$$intervalId);
          }

          if (!skipApply) $rootScope.$apply();
        }

        promise.$$intervalId = setIntervalFn(tick, delay, deferred, skipApply);

        return promise;
      };
    };
  }];
}

/**
 * @ngdoc service
 * @name $jsonpCallbacks
 * @requires $window
 * @description
 * This service handles the lifecycle of callbacks to handle JSONP requests.
 * Override this service if you wish to customise where the callbacks are stored and
 * how they vary compared to the requested url.
 */
var $jsonpCallbacksProvider = /** @this */ function() {
  this.$get = function() {
    var callbacks = angular.callbacks;
    var callbackMap = {};

    function createCallback(callbackId) {
      var callback = function(data) {
        callback.data = data;
        callback.called = true;
      };
      callback.id = callbackId;
      return callback;
    }

    return {
      /**
       * @ngdoc method
       * @name $jsonpCallbacks#createCallback
       * @param {string} url the url of the JSONP request
       * @returns {string} the callback path to send to the server as part of the JSONP request
       * @description
       * {@link $httpBackend} calls this method to create a callback and get hold of the path to the callback
       * to pass to the server, which will be used to call the callback with its payload in the JSONP response.
       */
      createCallback: function(url) {
        var callbackId = '_' + (callbacks.$$counter++).toString(36);
        var callbackPath = 'angular.callbacks.' + callbackId;
        var callback = createCallback(callbackId);
        callbackMap[callbackPath] = callbacks[callbackId] = callback;
        return callbackPath;
      },
      /**
       * @ngdoc method
       * @name $jsonpCallbacks#wasCalled
       * @param {string} callbackPath the path to the callback that was sent in the JSONP request
       * @returns {boolean} whether the callback has been called, as a result of the JSONP response
       * @description
       * {@link $httpBackend} calls this method to find out whether the JSONP response actually called the
       * callback that was passed in the request.
       */
      wasCalled: function(callbackPath) {
        return callbackMap[callbackPath].called;
      },
      /**
       * @ngdoc method
       * @name $jsonpCallbacks#getResponse
       * @param {string} callbackPath the path to the callback that was sent in the JSONP request
       * @returns {*} the data received from the response via the registered callback
       * @description
       * {@link $httpBackend} calls this method to get hold of the data that was provided to the callback
       * in the JSONP response.
       */
      getResponse: function(callbackPath) {
        return callbackMap[callbackPath].data;
      },
      /**
       * @ngdoc method
       * @name $jsonpCallbacks#removeCallback
       * @param {string} callbackPath the path to the callback that was sent in the JSONP request
       * @description
       * {@link $httpBackend} calls this method to remove the callback after the JSONP request has
       * completed or timed-out.
       */
      removeCallback: function(callbackPath) {
        var callback = callbackMap[callbackPath];
        delete callbacks[callback.id];
        delete callbackMap[callbackPath];
      }
    };
  };
};

/**
 * @ngdoc service
 * @name $locale
 *
 * @description
 * $locale service provides localization rules for various AngularJS components. As of right now the
 * only public api is:
 *
 * * `id` – `{string}` – locale id formatted as `languageId-countryId` (e.g. `en-us`)
 */

/* global stripHash: true */

var PATH_MATCH = /^([^?#]*)(\?([^#]*))?(#(.*))?$/,
    DEFAULT_PORTS = {'http': 80, 'https': 443, 'ftp': 21};
var $locationMinErr = minErr('$location');


/**
 * Encode path using encodeUriSegment, ignoring forward slashes
 *
 * @param {string} path Path to encode
 * @returns {string}
 */
function encodePath(path) {
  var segments = path.split('/'),
      i = segments.length;

  while (i--) {
    // decode forward slashes to prevent them from being double encoded
    segments[i] = encodeUriSegment(segments[i].replace(/%2F/g, '/'));
  }

  return segments.join('/');
}

function decodePath(path, html5Mode) {
  var segments = path.split('/'),
      i = segments.length;

  while (i--) {
    segments[i] = decodeURIComponent(segments[i]);
    if (html5Mode) {
      // encode forward slashes to prevent them from being mistaken for path separators
      segments[i] = segments[i].replace(/\//g, '%2F');
    }
  }

  return segments.join('/');
}

function normalizePath(pathValue, searchValue, hashValue) {
  var search = toKeyValue(searchValue),
    hash = hashValue ? '#' + encodeUriSegment(hashValue) : '',
    path = encodePath(pathValue);

  return path + (search ? '?' + search : '') + hash;
}

function parseAbsoluteUrl(absoluteUrl, locationObj) {
  var parsedUrl = urlResolve(absoluteUrl);

  locationObj.$$protocol = parsedUrl.protocol;
  locationObj.$$host = parsedUrl.hostname;
  locationObj.$$port = toInt(parsedUrl.port) || DEFAULT_PORTS[parsedUrl.protocol] || null;
}

var DOUBLE_SLASH_REGEX = /^\s*[\\/]{2,}/;
function parseAppUrl(url, locationObj, html5Mode) {

  if (DOUBLE_SLASH_REGEX.test(url)) {
    throw $locationMinErr('badpath', 'Invalid url "{0}".', url);
  }

  var prefixed = (url.charAt(0) !== '/');
  if (prefixed) {
    url = '/' + url;
  }
  var match = urlResolve(url);
  var path = prefixed && match.pathname.charAt(0) === '/' ? match.pathname.substring(1) : match.pathname;
  locationObj.$$path = decodePath(path, html5Mode);
  locationObj.$$search = parseKeyValue(match.search);
  locationObj.$$hash = decodeURIComponent(match.hash);

  // make sure path starts with '/';
  if (locationObj.$$path && locationObj.$$path.charAt(0) !== '/') {
    locationObj.$$path = '/' + locationObj.$$path;
  }
}

function startsWith(str, search) {
  return str.slice(0, search.length) === search;
}

/**
 *
 * @param {string} base
 * @param {string} url
 * @returns {string} returns text from `url` after `base` or `undefined` if it does not begin with
 *                   the expected string.
 */
function stripBaseUrl(base, url) {
  if (startsWith(url, base)) {
    return url.substr(base.length);
  }
}

function stripHash(url) {
  var index = url.indexOf('#');
  return index === -1 ? url : url.substr(0, index);
}

function stripFile(url) {
  return url.substr(0, stripHash(url).lastIndexOf('/') + 1);
}

/* return the server only (scheme://host:port) */
function serverBase(url) {
  return url.substring(0, url.indexOf('/', url.indexOf('//') + 2));
}


/**
 * LocationHtml5Url represents a URL
 * This object is exposed as $location service when HTML5 mode is enabled and supported
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} basePrefix URL path prefix
 */
function LocationHtml5Url(appBase, appBaseNoFile, basePrefix) {
  this.$$html5 = true;
  basePrefix = basePrefix || '';
  parseAbsoluteUrl(appBase, this);


  /**
   * Parse given HTML5 (regular) URL string into properties
   * @param {string} url HTML5 URL
   * @private
   */
  this.$$parse = function(url) {
    var pathUrl = stripBaseUrl(appBaseNoFile, url);
    if (!isString(pathUrl)) {
      throw $locationMinErr('ipthprfx', 'Invalid url "{0}", missing path prefix "{1}".', url,
          appBaseNoFile);
    }

    parseAppUrl(pathUrl, this, true);

    if (!this.$$path) {
      this.$$path = '/';
    }

    this.$$compose();
  };

  this.$$normalizeUrl = function(url) {
    return appBaseNoFile + url.substr(1); // first char is always '/'
  };

  this.$$parseLinkUrl = function(url, relHref) {
    if (relHref && relHref[0] === '#') {
      // special case for links to hash fragments:
      // keep the old url and only replace the hash fragment
      this.hash(relHref.slice(1));
      return true;
    }
    var appUrl, prevAppUrl;
    var rewrittenUrl;


    if (isDefined(appUrl = stripBaseUrl(appBase, url))) {
      prevAppUrl = appUrl;
      if (basePrefix && isDefined(appUrl = stripBaseUrl(basePrefix, appUrl))) {
        rewrittenUrl = appBaseNoFile + (stripBaseUrl('/', appUrl) || appUrl);
      } else {
        rewrittenUrl = appBase + prevAppUrl;
      }
    } else if (isDefined(appUrl = stripBaseUrl(appBaseNoFile, url))) {
      rewrittenUrl = appBaseNoFile + appUrl;
    } else if (appBaseNoFile === url + '/') {
      rewrittenUrl = appBaseNoFile;
    }
    if (rewrittenUrl) {
      this.$$parse(rewrittenUrl);
    }
    return !!rewrittenUrl;
  };
}


/**
 * LocationHashbangUrl represents URL
 * This object is exposed as $location service when developer doesn't opt into html5 mode.
 * It also serves as the base class for html5 mode fallback on legacy browsers.
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} hashPrefix hashbang prefix
 */
function LocationHashbangUrl(appBase, appBaseNoFile, hashPrefix) {

  parseAbsoluteUrl(appBase, this);


  /**
   * Parse given hashbang URL into properties
   * @param {string} url Hashbang URL
   * @private
   */
  this.$$parse = function(url) {
    var withoutBaseUrl = stripBaseUrl(appBase, url) || stripBaseUrl(appBaseNoFile, url);
    var withoutHashUrl;

    if (!isUndefined(withoutBaseUrl) && withoutBaseUrl.charAt(0) === '#') {

      // The rest of the URL starts with a hash so we have
      // got either a hashbang path or a plain hash fragment
      withoutHashUrl = stripBaseUrl(hashPrefix, withoutBaseUrl);
      if (isUndefined(withoutHashUrl)) {
        // There was no hashbang prefix so we just have a hash fragment
        withoutHashUrl = withoutBaseUrl;
      }

    } else {
      // There was no hashbang path nor hash fragment:
      // If we are in HTML5 mode we use what is left as the path;
      // Otherwise we ignore what is left
      if (this.$$html5) {
        withoutHashUrl = withoutBaseUrl;
      } else {
        withoutHashUrl = '';
        if (isUndefined(withoutBaseUrl)) {
          appBase = url;
          /** @type {?} */ (this).replace();
        }
      }
    }

    parseAppUrl(withoutHashUrl, this, false);

    this.$$path = removeWindowsDriveName(this.$$path, withoutHashUrl, appBase);

    this.$$compose();

    /*
     * In Windows, on an anchor node on documents loaded from
     * the filesystem, the browser will return a pathname
     * prefixed with the drive name ('/C:/path') when a
     * pathname without a drive is set:
     *  * a.setAttribute('href', '/foo')
     *   * a.pathname === '/C:/foo' //true
     *
     * Inside of AngularJS, we're always using pathnames that
     * do not include drive names for routing.
     */
    function removeWindowsDriveName(path, url, base) {
      /*
      Matches paths for file protocol on windows,
      such as /C:/foo/bar, and captures only /foo/bar.
      */
      var windowsFilePathExp = /^\/[A-Z]:(\/.*)/;

      var firstPathSegmentMatch;

      //Get the relative path from the input URL.
      if (startsWith(url, base)) {
        url = url.replace(base, '');
      }

      // The input URL intentionally contains a first path segment that ends with a colon.
      if (windowsFilePathExp.exec(url)) {
        return path;
      }

      firstPathSegmentMatch = windowsFilePathExp.exec(path);
      return firstPathSegmentMatch ? firstPathSegmentMatch[1] : path;
    }
  };

  this.$$normalizeUrl = function(url) {
    return appBase + (url ? hashPrefix + url : '');
  };

  this.$$parseLinkUrl = function(url, relHref) {
    if (stripHash(appBase) === stripHash(url)) {
      this.$$parse(url);
      return true;
    }
    return false;
  };
}


/**
 * LocationHashbangUrl represents URL
 * This object is exposed as $location service when html5 history api is enabled but the browser
 * does not support it.
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} hashPrefix hashbang prefix
 */
function LocationHashbangInHtml5Url(appBase, appBaseNoFile, hashPrefix) {
  this.$$html5 = true;
  LocationHashbangUrl.apply(this, arguments);

  this.$$parseLinkUrl = function(url, relHref) {
    if (relHref && relHref[0] === '#') {
      // special case for links to hash fragments:
      // keep the old url and only replace the hash fragment
      this.hash(relHref.slice(1));
      return true;
    }

    var rewrittenUrl;
    var appUrl;

    if (appBase === stripHash(url)) {
      rewrittenUrl = url;
    } else if ((appUrl = stripBaseUrl(appBaseNoFile, url))) {
      rewrittenUrl = appBase + hashPrefix + appUrl;
    } else if (appBaseNoFile === url + '/') {
      rewrittenUrl = appBaseNoFile;
    }
    if (rewrittenUrl) {
      this.$$parse(rewrittenUrl);
    }
    return !!rewrittenUrl;
  };

  this.$$normalizeUrl = function(url) {
    // include hashPrefix in $$absUrl when $$url is empty so IE9 does not reload page because of removal of '#'
    return appBase + hashPrefix + url;
  };
}


var locationPrototype = {

  /**
   * Ensure absolute URL is initialized.
   * @private
   */
  $$absUrl:'',

  /**
   * Are we in html5 mode?
   * @private
   */
  $$html5: false,

  /**
   * Has any change been replacing?
   * @private
   */
  $$replace: false,

  /**
   * Compose url and update `url` and `absUrl` property
   * @private
   */
  $$compose: function() {
    this.$$url = normalizePath(this.$$path, this.$$search, this.$$hash);
    this.$$absUrl = this.$$normalizeUrl(this.$$url);
    this.$$urlUpdatedByLocation = true;
  },

  /**
   * @ngdoc method
   * @name $location#absUrl
   *
   * @description
   * This method is getter only.
   *
   * Return full URL representation with all segments encoded according to rules specified in
   * [RFC 3986](http://www.ietf.org/rfc/rfc3986.txt).
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var absUrl = $location.absUrl();
   * // => "http://example.com/#/some/path?foo=bar&baz=xoxo"
   * ```
   *
   * @return {string} full URL
   */
  absUrl: locationGetter('$$absUrl'),

  /**
   * @ngdoc method
   * @name $location#url
   *
   * @description
   * This method is getter / setter.
   *
   * Return URL (e.g. `/path?a=b#hash`) when called without any parameter.
   *
   * Change path, search and hash, when called with parameter and return `$location`.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var url = $location.url();
   * // => "/some/path?foo=bar&baz=xoxo"
   * ```
   *
   * @param {string=} url New URL without base prefix (e.g. `/path?a=b#hash`)
   * @return {string} url
   */
  url: function(url) {
    if (isUndefined(url)) {
      return this.$$url;
    }

    var match = PATH_MATCH.exec(url);
    if (match[1] || url === '') this.path(decodeURIComponent(match[1]));
    if (match[2] || match[1] || url === '') this.search(match[3] || '');
    this.hash(match[5] || '');

    return this;
  },

  /**
   * @ngdoc method
   * @name $location#protocol
   *
   * @description
   * This method is getter only.
   *
   * Return protocol of current URL.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var protocol = $location.protocol();
   * // => "http"
   * ```
   *
   * @return {string} protocol of current URL
   */
  protocol: locationGetter('$$protocol'),

  /**
   * @ngdoc method
   * @name $location#host
   *
   * @description
   * This method is getter only.
   *
   * Return host of current URL.
   *
   * Note: compared to the non-AngularJS version `location.host` which returns `hostname:port`, this returns the `hostname` portion only.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var host = $location.host();
   * // => "example.com"
   *
   * // given URL http://user:password@example.com:8080/#/some/path?foo=bar&baz=xoxo
   * host = $location.host();
   * // => "example.com"
   * host = location.host;
   * // => "example.com:8080"
   * ```
   *
   * @return {string} host of current URL.
   */
  host: locationGetter('$$host'),

  /**
   * @ngdoc method
   * @name $location#port
   *
   * @description
   * This method is getter only.
   *
   * Return port of current URL.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var port = $location.port();
   * // => 80
   * ```
   *
   * @return {Number} port
   */
  port: locationGetter('$$port'),

  /**
   * @ngdoc method
   * @name $location#path
   *
   * @description
   * This method is getter / setter.
   *
   * Return path of current URL when called without any parameter.
   *
   * Change path when called with parameter and return `$location`.
   *
   * Note: Path should always begin with forward slash (/), this method will add the forward slash
   * if it is missing.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var path = $location.path();
   * // => "/some/path"
   * ```
   *
   * @param {(string|number)=} path New path
   * @return {(string|object)} path if called with no parameters, or `$location` if called with a parameter
   */
  path: locationGetterSetter('$$path', function(path) {
    path = path !== null ? path.toString() : '';
    return path.charAt(0) === '/' ? path : '/' + path;
  }),

  /**
   * @ngdoc method
   * @name $location#search
   *
   * @description
   * This method is getter / setter.
   *
   * Return search part (as object) of current URL when called without any parameter.
   *
   * Change search part when called with parameter and return `$location`.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
   * var searchObject = $location.search();
   * // => {foo: 'bar', baz: 'xoxo'}
   *
   * // set foo to 'yipee'
   * $location.search('foo', 'yipee');
   * // $location.search() => {foo: 'yipee', baz: 'xoxo'}
   * ```
   *
   * @param {string|Object.<string>|Object.<Array.<string>>} search New search params - string or
   * hash object.
   *
   * When called with a single argument the method acts as a setter, setting the `search` component
   * of `$location` to the specified value.
   *
   * If the argument is a hash object containing an array of values, these values will be encoded
   * as duplicate search parameters in the URL.
   *
   * @param {(string|Number|Array<string>|boolean)=} paramValue If `search` is a string or number, then `paramValue`
   * will override only a single search property.
   *
   * If `paramValue` is an array, it will override the property of the `search` component of
   * `$location` specified via the first argument.
   *
   * If `paramValue` is `null`, the property specified via the first argument will be deleted.
   *
   * If `paramValue` is `true`, the property specified via the first argument will be added with no
   * value nor trailing equal sign.
   *
   * @return {Object} If called with no arguments returns the parsed `search` object. If called with
   * one or more arguments returns `$location` object itself.
   */
  search: function(search, paramValue) {
    switch (arguments.length) {
      case 0:
        return this.$$search;
      case 1:
        if (isString(search) || isNumber(search)) {
          search = search.toString();
          this.$$search = parseKeyValue(search);
        } else if (isObject(search)) {
          search = copy(search, {});
          // remove object undefined or null properties
          forEach(search, function(value, key) {
            if (value == null) delete search[key];
          });

          this.$$search = search;
        } else {
          throw $locationMinErr('isrcharg',
              'The first argument of the `$location#search()` call must be a string or an object.');
        }
        break;
      default:
        if (isUndefined(paramValue) || paramValue === null) {
          delete this.$$search[search];
        } else {
          this.$$search[search] = paramValue;
        }
    }

    this.$$compose();
    return this;
  },

  /**
   * @ngdoc method
   * @name $location#hash
   *
   * @description
   * This method is getter / setter.
   *
   * Returns the hash fragment when called without any parameters.
   *
   * Changes the hash fragment when called with a parameter and returns `$location`.
   *
   *
   * ```js
   * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo#hashValue
   * var hash = $location.hash();
   * // => "hashValue"
   * ```
   *
   * @param {(string|number)=} hash New hash fragment
   * @return {string} hash
   */
  hash: locationGetterSetter('$$hash', function(hash) {
    return hash !== null ? hash.toString() : '';
  }),

  /**
   * @ngdoc method
   * @name $location#replace
   *
   * @description
   * If called, all changes to $location during the current `$digest` will replace the current history
   * record, instead of adding a new one.
   */
  replace: function() {
    this.$$replace = true;
    return this;
  }
};

forEach([LocationHashbangInHtml5Url, LocationHashbangUrl, LocationHtml5Url], function(Location) {
  Location.prototype = Object.create(locationPrototype);

  /**
   * @ngdoc method
   * @name $location#state
   *
   * @description
   * This method is getter / setter.
   *
   * Return the history state object when called without any parameter.
   *
   * Change the history state object when called with one parameter and return `$location`.
   * The state object is later passed to `pushState` or `replaceState`.
   *
   * NOTE: This method is supported only in HTML5 mode and only in browsers supporting
   * the HTML5 History API (i.e. methods `pushState` and `replaceState`). If you need to support
   * older browsers (like IE9 or Android < 4.0), don't use this method.
   *
   * @param {object=} state State object for pushState or replaceState
   * @return {object} state
   */
  Location.prototype.state = function(state) {
    if (!arguments.length) {
      return this.$$state;
    }

    if (Location !== LocationHtml5Url || !this.$$html5) {
      throw $locationMinErr('nostate', 'History API state support is available only ' +
        'in HTML5 mode and only in browsers supporting HTML5 History API');
    }
    // The user might modify `stateObject` after invoking `$location.state(stateObject)`
    // but we're changing the $$state reference to $browser.state() during the $digest
    // so the modification window is narrow.
    this.$$state = isUndefined(state) ? null : state;
    this.$$urlUpdatedByLocation = true;

    return this;
  };
});


function locationGetter(property) {
  return /** @this */ function() {
    return this[property];
  };
}


function locationGetterSetter(property, preprocess) {
  return /** @this */ function(value) {
    if (isUndefined(value)) {
      return this[property];
    }

    this[property] = preprocess(value);
    this.$$compose();

    return this;
  };
}


/**
 * @ngdoc service
 * @name $location
 *
 * @requires $rootElement
 *
 * @description
 * The $location service parses the URL in the browser address bar (based on the
 * [window.location](https://developer.mozilla.org/en/window.location)) and makes the URL
 * available to your application. Changes to the URL in the address bar are reflected into
 * $location service and changes to $location are reflected into the browser address bar.
 *
 * **The $location service:**
 *
 * - Exposes the current URL in the browser address bar, so you can
 *   - Watch and observe the URL.
 *   - Change the URL.
 * - Synchronizes the URL with the browser when the user
 *   - Changes the address bar.
 *   - Clicks the back or forward button (or clicks a History link).
 *   - Clicks on a link.
 * - Represents the URL object as a set of methods (protocol, host, port, path, search, hash).
 *
 * For more information see {@link guide/$location Developer Guide: Using $location}
 */

/**
 * @ngdoc provider
 * @name $locationProvider
 * @this
 *
 * @description
 * Use the `$locationProvider` to configure how the application deep linking paths are stored.
 */
function $LocationProvider() {
  var hashPrefix = '!',
      html5Mode = {
        enabled: false,
        requireBase: true,
        rewriteLinks: true
      };

  /**
   * @ngdoc method
   * @name $locationProvider#hashPrefix
   * @description
   * The default value for the prefix is `'!'`.
   * @param {string=} prefix Prefix for hash part (containing path and search)
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   */
  this.hashPrefix = function(prefix) {
    if (isDefined(prefix)) {
      hashPrefix = prefix;
      return this;
    } else {
      return hashPrefix;
    }
  };

  /**
   * @ngdoc method
   * @name $locationProvider#html5Mode
   * @description
   * @param {(boolean|Object)=} mode If boolean, sets `html5Mode.enabled` to value.
   *   If object, sets `enabled`, `requireBase` and `rewriteLinks` to respective values. Supported
   *   properties:
   *   - **enabled** – `{boolean}` – (default: false) If true, will rely on `history.pushState` to
   *     change urls where supported. Will fall back to hash-prefixed paths in browsers that do not
   *     support `pushState`.
   *   - **requireBase** - `{boolean}` - (default: `true`) When html5Mode is enabled, specifies
   *     whether or not a <base> tag is required to be present. If `enabled` and `requireBase` are
   *     true, and a base tag is not present, an error will be thrown when `$location` is injected.
   *     See the {@link guide/$location $location guide for more information}
   *   - **rewriteLinks** - `{boolean|string}` - (default: `true`) When html5Mode is enabled,
   *     enables/disables URL rewriting for relative links. If set to a string, URL rewriting will
   *     only happen on links with an attribute that matches the given string. For example, if set
   *     to `'internal-link'`, then the URL will only be rewritten for `<a internal-link>` links.
   *     Note that [attribute name normalization](guide/directive#normalization) does not apply
   *     here, so `'internalLink'` will **not** match `'internal-link'`.
   *
   * @returns {Object} html5Mode object if used as getter or itself (chaining) if used as setter
   */
  this.html5Mode = function(mode) {
    if (isBoolean(mode)) {
      html5Mode.enabled = mode;
      return this;
    } else if (isObject(mode)) {

      if (isBoolean(mode.enabled)) {
        html5Mode.enabled = mode.enabled;
      }

      if (isBoolean(mode.requireBase)) {
        html5Mode.requireBase = mode.requireBase;
      }

      if (isBoolean(mode.rewriteLinks) || isString(mode.rewriteLinks)) {
        html5Mode.rewriteLinks = mode.rewriteLinks;
      }

      return this;
    } else {
      return html5Mode;
    }
  };

  /**
   * @ngdoc event
   * @name $location#$locationChangeStart
   * @eventType broadcast on root scope
   * @description
   * Broadcasted before a URL will change.
   *
   * This change can be prevented by calling
   * `preventDefault` method of the event. See {@link ng.$rootScope.Scope#$on} for more
   * details about event object. Upon successful change
   * {@link ng.$location#$locationChangeSuccess $locationChangeSuccess} is fired.
   *
   * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
   * the browser supports the HTML5 History API.
   *
   * @param {Object} angularEvent Synthetic event object.
   * @param {string} newUrl New URL
   * @param {string=} oldUrl URL that was before it was changed.
   * @param {string=} newState New history state object
   * @param {string=} oldState History state object that was before it was changed.
   */

  /**
   * @ngdoc event
   * @name $location#$locationChangeSuccess
   * @eventType broadcast on root scope
   * @description
   * Broadcasted after a URL was changed.
   *
   * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
   * the browser supports the HTML5 History API.
   *
   * @param {Object} angularEvent Synthetic event object.
   * @param {string} newUrl New URL
   * @param {string=} oldUrl URL that was before it was changed.
   * @param {string=} newState New history state object
   * @param {string=} oldState History state object that was before it was changed.
   */

  this.$get = ['$rootScope', '$browser', '$sniffer', '$rootElement', '$window',
      function($rootScope, $browser, $sniffer, $rootElement, $window) {
    var $location,
        LocationMode,
        baseHref = $browser.baseHref(), // if base[href] is undefined, it defaults to ''
        initialUrl = $browser.url(),
        appBase;

    if (html5Mode.enabled) {
      if (!baseHref && html5Mode.requireBase) {
        throw $locationMinErr('nobase',
          '$location in HTML5 mode requires a <base> tag to be present!');
      }
      appBase = serverBase(initialUrl) + (baseHref || '/');
      LocationMode = $sniffer.history ? LocationHtml5Url : LocationHashbangInHtml5Url;
    } else {
      appBase = stripHash(initialUrl);
      LocationMode = LocationHashbangUrl;
    }
    var appBaseNoFile = stripFile(appBase);

    $location = new LocationMode(appBase, appBaseNoFile, '#' + hashPrefix);
    $location.$$parseLinkUrl(initialUrl, initialUrl);

    $location.$$state = $browser.state();

    var IGNORE_URI_REGEXP = /^\s*(javascript|mailto):/i;

    // Determine if two URLs are equal despite potentially having different encoding/normalizing
    //  such as $location.absUrl() vs $browser.url()
    // See https://github.com/angular/angular.js/issues/16592
    function urlsEqual(a, b) {
      return a === b || urlResolve(a).href === urlResolve(b).href;
    }

    function setBrowserUrlWithFallback(url, replace, state) {
      var oldUrl = $location.url();
      var oldState = $location.$$state;
      try {
        $browser.url(url, replace, state);

        // Make sure $location.state() returns referentially identical (not just deeply equal)
        // state object; this makes possible quick checking if the state changed in the digest
        // loop. Checking deep equality would be too expensive.
        $location.$$state = $browser.state();
      } catch (e) {
        // Restore old values if pushState fails
        $location.url(oldUrl);
        $location.$$state = oldState;

        throw e;
      }
    }

    $rootElement.on('click', function(event) {
      var rewriteLinks = html5Mode.rewriteLinks;
      // TODO(vojta): rewrite link when opening in new tab/window (in legacy browser)
      // currently we open nice url link and redirect then

      if (!rewriteLinks || event.ctrlKey || event.metaKey || event.shiftKey || event.which === 2 || event.button === 2) return;

      var elm = jqLite(event.target);

      // traverse the DOM up to find first A tag
      while (nodeName_(elm[0]) !== 'a') {
        // ignore rewriting if no A tag (reached root element, or no parent - removed from document)
        if (elm[0] === $rootElement[0] || !(elm = elm.parent())[0]) return;
      }

      if (isString(rewriteLinks) && isUndefined(elm.attr(rewriteLinks))) return;

      var absHref = elm.prop('href');
      // get the actual href attribute - see
      // http://msdn.microsoft.com/en-us/library/ie/dd347148(v=vs.85).aspx
      var relHref = elm.attr('href') || elm.attr('xlink:href');

      if (isObject(absHref) && absHref.toString() === '[object SVGAnimatedString]') {
        // SVGAnimatedString.animVal should be identical to SVGAnimatedString.baseVal, unless during
        // an animation.
        absHref = urlResolve(absHref.animVal).href;
      }

      // Ignore when url is started with javascript: or mailto:
      if (IGNORE_URI_REGEXP.test(absHref)) return;

      if (absHref && !elm.attr('target') && !event.isDefaultPrevented()) {
        if ($location.$$parseLinkUrl(absHref, relHref)) {
          // We do a preventDefault for all urls that are part of the AngularJS application,
          // in html5mode and also without, so that we are able to abort navigation without
          // getting double entries in the location history.
          event.preventDefault();
          // update location manually
          if ($location.absUrl() !== $browser.url()) {
            $rootScope.$apply();
          }
        }
      }
    });


    // rewrite hashbang url <> html5 url
    if ($location.absUrl() !== initialUrl) {
      $browser.url($location.absUrl(), true);
    }

    var initializing = true;

    // update $location when $browser url changes
    $browser.onUrlChange(function(newUrl, newState) {

      if (!startsWith(newUrl, appBaseNoFile)) {
        // If we are navigating outside of the app then force a reload
        $window.location.href = newUrl;
        return;
      }

      $rootScope.$evalAsync(function() {
        var oldUrl = $location.absUrl();
        var oldState = $location.$$state;
        var defaultPrevented;
        $location.$$parse(newUrl);
        $location.$$state = newState;

        defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl,
            newState, oldState).defaultPrevented;

        // if the location was changed by a `$locationChangeStart` handler then stop
        // processing this location change
        if ($location.absUrl() !== newUrl) return;

        if (defaultPrevented) {
          $location.$$parse(oldUrl);
          $location.$$state = oldState;
          setBrowserUrlWithFallback(oldUrl, false, oldState);
        } else {
          initializing = false;
          afterLocationChange(oldUrl, oldState);
        }
      });
      if (!$rootScope.$$phase) $rootScope.$digest();
    });

    // update browser
    $rootScope.$watch(function $locationWatch() {
      if (initializing || $location.$$urlUpdatedByLocation) {
        $location.$$urlUpdatedByLocation = false;

        var oldUrl = $browser.url();
        var newUrl = $location.absUrl();
        var oldState = $browser.state();
        var currentReplace = $location.$$replace;
        var urlOrStateChanged = !urlsEqual(oldUrl, newUrl) ||
          ($location.$$html5 && $sniffer.history && oldState !== $location.$$state);

        if (initializing || urlOrStateChanged) {
          initializing = false;

          $rootScope.$evalAsync(function() {
            var newUrl = $location.absUrl();
            var defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl,
                $location.$$state, oldState).defaultPrevented;

            // if the location was changed by a `$locationChangeStart` handler then stop
            // processing this location change
            if ($location.absUrl() !== newUrl) return;

            if (defaultPrevented) {
              $location.$$parse(oldUrl);
              $location.$$state = oldState;
            } else {
              if (urlOrStateChanged) {
                setBrowserUrlWithFallback(newUrl, currentReplace,
                                          oldState === $location.$$state ? null : $location.$$state);
              }
              afterLocationChange(oldUrl, oldState);
            }
          });
        }
      }

      $location.$$replace = false;

      // we don't need to return anything because $evalAsync will make the digest loop dirty when
      // there is a change
    });

    return $location;

    function afterLocationChange(oldUrl, oldState) {
      $rootScope.$broadcast('$locationChangeSuccess', $location.absUrl(), oldUrl,
        $location.$$state, oldState);
    }
}];
}

/**
 * @ngdoc service
 * @name $log
 * @requires $window
 *
 * @description
 * Simple service for logging. Default implementation safely writes the message
 * into the browser's console (if present).
 *
 * The main purpose of this service is to simplify debugging and troubleshooting.
 *
 * To reveal the location of the calls to `$log` in the JavaScript console,
 * you can "blackbox" the AngularJS source in your browser:
 *
 * [Mozilla description of blackboxing](https://developer.mozilla.org/en-US/docs/Tools/Debugger/How_to/Black_box_a_source).
 * [Chrome description of blackboxing](https://developer.chrome.com/devtools/docs/blackboxing).
 *
 * Note: Not all browsers support blackboxing.
 *
 * The default is to log `debug` messages. You can use
 * {@link ng.$logProvider ng.$logProvider#debugEnabled} to change this.
 *
 * @example
   <example module="logExample" name="log-service">
     <file name="script.js">
       angular.module('logExample', [])
         .controller('LogController', ['$scope', '$log', function($scope, $log) {
           $scope.$log = $log;
           $scope.message = 'Hello World!';
         }]);
     </file>
     <file name="index.html">
       <div ng-controller="LogController">
         <p>Reload this page with open console, enter text and hit the log button...</p>
         <label>Message:
         <input type="text" ng-model="message" /></label>
         <button ng-click="$log.log(message)">log</button>
         <button ng-click="$log.warn(message)">warn</button>
         <button ng-click="$log.info(message)">info</button>
         <button ng-click="$log.error(message)">error</button>
         <button ng-click="$log.debug(message)">debug</button>
       </div>
     </file>
   </example>
 */

/**
 * @ngdoc provider
 * @name $logProvider
 * @this
 *
 * @description
 * Use the `$logProvider` to configure how the application logs messages
 */
function $LogProvider() {
  var debug = true,
      self = this;

  /**
   * @ngdoc method
   * @name $logProvider#debugEnabled
   * @description
   * @param {boolean=} flag enable or disable debug level messages
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   */
  this.debugEnabled = function(flag) {
    if (isDefined(flag)) {
      debug = flag;
      return this;
    } else {
      return debug;
    }
  };

  this.$get = ['$window', function($window) {
    // Support: IE 9-11, Edge 12-14+
    // IE/Edge display errors in such a way that it requires the user to click in 4 places
    // to see the stack trace. There is no way to feature-detect it so there's a chance
    // of the user agent sniffing to go wrong but since it's only about logging, this shouldn't
    // break apps. Other browsers display errors in a sensible way and some of them map stack
    // traces along source maps if available so it makes sense to let browsers display it
    // as they want.
    var formatStackTrace = msie || /\bEdge\//.test($window.navigator && $window.navigator.userAgent);

    return {
      /**
       * @ngdoc method
       * @name $log#log
       *
       * @description
       * Write a log message
       */
      log: consoleLog('log'),

      /**
       * @ngdoc method
       * @name $log#info
       *
       * @description
       * Write an information message
       */
      info: consoleLog('info'),

      /**
       * @ngdoc method
       * @name $log#warn
       *
       * @description
       * Write a warning message
       */
      warn: consoleLog('warn'),

      /**
       * @ngdoc method
       * @name $log#error
       *
       * @description
       * Write an error message
       */
      error: consoleLog('error'),

      /**
       * @ngdoc method
       * @name $log#debug
       *
       * @description
       * Write a debug message
       */
      debug: (function() {
        var fn = consoleLog('debug');

        return function() {
          if (debug) {
            fn.apply(self, arguments);
          }
        };
      })()
    };

    function formatError(arg) {
      if (isError(arg)) {
        if (arg.stack && formatStackTrace) {
          arg = (arg.message && arg.stack.indexOf(arg.message) === -1)
              ? 'Error: ' + arg.message + '\n' + arg.stack
              : arg.stack;
        } else if (arg.sourceURL) {
          arg = arg.message + '\n' + arg.sourceURL + ':' + arg.line;
        }
      }
      return arg;
    }

    function consoleLog(type) {
      var console = $window.console || {},
          logFn = console[type] || console.log || noop;

      return function() {
        var args = [];
        forEach(arguments, function(arg) {
          args.push(formatError(arg));
        });
        // Support: IE 9 only
        // console methods don't inherit from Function.prototype in IE 9 so we can't
        // call `logFn.apply(console, args)` directly.
        return Function.prototype.apply.call(logFn, console, args);
      };
    }
  }];
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

var $parseMinErr = minErr('$parse');

var objectValueOf = {}.constructor.prototype.valueOf;

// Sandboxing AngularJS Expressions
// ------------------------------
// AngularJS expressions are no longer sandboxed. So it is now even easier to access arbitrary JS code by
// various means such as obtaining a reference to native JS functions like the Function constructor.
//
// As an example, consider the following AngularJS expression:
//
//   {}.toString.constructor('alert("evil JS code")')
//
// It is important to realize that if you create an expression from a string that contains user provided
// content then it is possible that your application contains a security vulnerability to an XSS style attack.
//
// See https://docs.angularjs.org/guide/security


function getStringValue(name) {
  // Property names must be strings. This means that non-string objects cannot be used
  // as keys in an object. Any non-string object, including a number, is typecasted
  // into a string via the toString method.
  // -- MDN, https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Operators/Property_accessors#Property_names
  //
  // So, to ensure that we are checking the same `name` that JavaScript would use, we cast it
  // to a string. It's not always possible. If `name` is an object and its `toString` method is
  // 'broken' (doesn't return a string, isn't a function, etc.), an error will be thrown:
  //
  // TypeError: Cannot convert object to primitive value
  //
  // For performance reasons, we don't catch this error here and allow it to propagate up the call
  // stack. Note that you'll get the same error in JavaScript if you try to access a property using
  // such a 'broken' object as a key.
  return name + '';
}


var OPERATORS = createMap();
forEach('+ - * / % === !== == != < > <= >= && || ! = |'.split(' '), function(operator) { OPERATORS[operator] = true; });
var ESCAPE = {'n':'\n', 'f':'\f', 'r':'\r', 't':'\t', 'v':'\v', '\'':'\'', '"':'"'};


/////////////////////////////////////////


/**
 * @constructor
 */
var Lexer = function Lexer(options) {
  this.options = options;
};

Lexer.prototype = {
  constructor: Lexer,

  lex: function(text) {
    this.text = text;
    this.index = 0;
    this.tokens = [];

    while (this.index < this.text.length) {
      var ch = this.text.charAt(this.index);
      if (ch === '"' || ch === '\'') {
        this.readString(ch);
      } else if (this.isNumber(ch) || ch === '.' && this.isNumber(this.peek())) {
        this.readNumber();
      } else if (this.isIdentifierStart(this.peekMultichar())) {
        this.readIdent();
      } else if (this.is(ch, '(){}[].,;:?')) {
        this.tokens.push({index: this.index, text: ch});
        this.index++;
      } else if (this.isWhitespace(ch)) {
        this.index++;
      } else {
        var ch2 = ch + this.peek();
        var ch3 = ch2 + this.peek(2);
        var op1 = OPERATORS[ch];
        var op2 = OPERATORS[ch2];
        var op3 = OPERATORS[ch3];
        if (op1 || op2 || op3) {
          var token = op3 ? ch3 : (op2 ? ch2 : ch);
          this.tokens.push({index: this.index, text: token, operator: true});
          this.index += token.length;
        } else {
          this.throwError('Unexpected next character ', this.index, this.index + 1);
        }
      }
    }
    return this.tokens;
  },

  is: function(ch, chars) {
    return chars.indexOf(ch) !== -1;
  },

  peek: function(i) {
    var num = i || 1;
    return (this.index + num < this.text.length) ? this.text.charAt(this.index + num) : false;
  },

  isNumber: function(ch) {
    return ('0' <= ch && ch <= '9') && typeof ch === 'string';
  },

  isWhitespace: function(ch) {
    // IE treats non-breaking space as \u00A0
    return (ch === ' ' || ch === '\r' || ch === '\t' ||
            ch === '\n' || ch === '\v' || ch === '\u00A0');
  },

  isIdentifierStart: function(ch) {
    return this.options.isIdentifierStart ?
        this.options.isIdentifierStart(ch, this.codePointAt(ch)) :
        this.isValidIdentifierStart(ch);
  },

  isValidIdentifierStart: function(ch) {
    return ('a' <= ch && ch <= 'z' ||
            'A' <= ch && ch <= 'Z' ||
            '_' === ch || ch === '$');
  },

  isIdentifierContinue: function(ch) {
    return this.options.isIdentifierContinue ?
        this.options.isIdentifierContinue(ch, this.codePointAt(ch)) :
        this.isValidIdentifierContinue(ch);
  },

  isValidIdentifierContinue: function(ch, cp) {
    return this.isValidIdentifierStart(ch, cp) || this.isNumber(ch);
  },

  codePointAt: function(ch) {
    if (ch.length === 1) return ch.charCodeAt(0);
    // eslint-disable-next-line no-bitwise
    return (ch.charCodeAt(0) << 10) + ch.charCodeAt(1) - 0x35FDC00;
  },

  peekMultichar: function() {
    var ch = this.text.charAt(this.index);
    var peek = this.peek();
    if (!peek) {
      return ch;
    }
    var cp1 = ch.charCodeAt(0);
    var cp2 = peek.charCodeAt(0);
    if (cp1 >= 0xD800 && cp1 <= 0xDBFF && cp2 >= 0xDC00 && cp2 <= 0xDFFF) {
      return ch + peek;
    }
    return ch;
  },

  isExpOperator: function(ch) {
    return (ch === '-' || ch === '+' || this.isNumber(ch));
  },

  throwError: function(error, start, end) {
    end = end || this.index;
    var colStr = (isDefined(start)
            ? 's ' + start +  '-' + this.index + ' [' + this.text.substring(start, end) + ']'
            : ' ' + end);
    throw $parseMinErr('lexerr', 'Lexer Error: {0} at column{1} in expression [{2}].',
        error, colStr, this.text);
  },

  readNumber: function() {
    var number = '';
    var start = this.index;
    while (this.index < this.text.length) {
      var ch = lowercase(this.text.charAt(this.index));
      if (ch === '.' || this.isNumber(ch)) {
        number += ch;
      } else {
        var peekCh = this.peek();
        if (ch === 'e' && this.isExpOperator(peekCh)) {
          number += ch;
        } else if (this.isExpOperator(ch) &&
            peekCh && this.isNumber(peekCh) &&
            number.charAt(number.length - 1) === 'e') {
          number += ch;
        } else if (this.isExpOperator(ch) &&
            (!peekCh || !this.isNumber(peekCh)) &&
            number.charAt(number.length - 1) === 'e') {
          this.throwError('Invalid exponent');
        } else {
          break;
        }
      }
      this.index++;
    }
    this.tokens.push({
      index: start,
      text: number,
      constant: true,
      value: Number(number)
    });
  },

  readIdent: function() {
    var start = this.index;
    this.index += this.peekMultichar().length;
    while (this.index < this.text.length) {
      var ch = this.peekMultichar();
      if (!this.isIdentifierContinue(ch)) {
        break;
      }
      this.index += ch.length;
    }
    this.tokens.push({
      index: start,
      text: this.text.slice(start, this.index),
      identifier: true
    });
  },

  readString: function(quote) {
    var start = this.index;
    this.index++;
    var string = '';
    var rawString = quote;
    var escape = false;
    while (this.index < this.text.length) {
      var ch = this.text.charAt(this.index);
      rawString += ch;
      if (escape) {
        if (ch === 'u') {
          var hex = this.text.substring(this.index + 1, this.index + 5);
          if (!hex.match(/[\da-f]{4}/i)) {
            this.throwError('Invalid unicode escape [\\u' + hex + ']');
          }
          this.index += 4;
          string += String.fromCharCode(parseInt(hex, 16));
        } else {
          var rep = ESCAPE[ch];
          string = string + (rep || ch);
        }
        escape = false;
      } else if (ch === '\\') {
        escape = true;
      } else if (ch === quote) {
        this.index++;
        this.tokens.push({
          index: start,
          text: rawString,
          constant: true,
          value: string
        });
        return;
      } else {
        string += ch;
      }
      this.index++;
    }
    this.throwError('Unterminated quote', start);
  }
};

var AST = function AST(lexer, options) {
  this.lexer = lexer;
  this.options = options;
};

AST.Program = 'Program';
AST.ExpressionStatement = 'ExpressionStatement';
AST.AssignmentExpression = 'AssignmentExpression';
AST.ConditionalExpression = 'ConditionalExpression';
AST.LogicalExpression = 'LogicalExpression';
AST.BinaryExpression = 'BinaryExpression';
AST.UnaryExpression = 'UnaryExpression';
AST.CallExpression = 'CallExpression';
AST.MemberExpression = 'MemberExpression';
AST.Identifier = 'Identifier';
AST.Literal = 'Literal';
AST.ArrayExpression = 'ArrayExpression';
AST.Property = 'Property';
AST.ObjectExpression = 'ObjectExpression';
AST.ThisExpression = 'ThisExpression';
AST.LocalsExpression = 'LocalsExpression';

// Internal use only
AST.NGValueParameter = 'NGValueParameter';

AST.prototype = {
  ast: function(text) {
    this.text = text;
    this.tokens = this.lexer.lex(text);

    var value = this.program();

    if (this.tokens.length !== 0) {
      this.throwError('is an unexpected token', this.tokens[0]);
    }

    return value;
  },

  program: function() {
    var body = [];
    while (true) {
      if (this.tokens.length > 0 && !this.peek('}', ')', ';', ']'))
        body.push(this.expressionStatement());
      if (!this.expect(';')) {
        return { type: AST.Program, body: body};
      }
    }
  },

  expressionStatement: function() {
    return { type: AST.ExpressionStatement, expression: this.filterChain() };
  },

  filterChain: function() {
    var left = this.expression();
    while (this.expect('|')) {
      left = this.filter(left);
    }
    return left;
  },

  expression: function() {
    return this.assignment();
  },

  assignment: function() {
    var result = this.ternary();
    if (this.expect('=')) {
      if (!isAssignable(result)) {
        throw $parseMinErr('lval', 'Trying to assign a value to a non l-value');
      }

      result = { type: AST.AssignmentExpression, left: result, right: this.assignment(), operator: '='};
    }
    return result;
  },

  ternary: function() {
    var test = this.logicalOR();
    var alternate;
    var consequent;
    if (this.expect('?')) {
      alternate = this.expression();
      if (this.consume(':')) {
        consequent = this.expression();
        return { type: AST.ConditionalExpression, test: test, alternate: alternate, consequent: consequent};
      }
    }
    return test;
  },

  logicalOR: function() {
    var left = this.logicalAND();
    while (this.expect('||')) {
      left = { type: AST.LogicalExpression, operator: '||', left: left, right: this.logicalAND() };
    }
    return left;
  },

  logicalAND: function() {
    var left = this.equality();
    while (this.expect('&&')) {
      left = { type: AST.LogicalExpression, operator: '&&', left: left, right: this.equality()};
    }
    return left;
  },

  equality: function() {
    var left = this.relational();
    var token;
    while ((token = this.expect('==','!=','===','!=='))) {
      left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.relational() };
    }
    return left;
  },

  relational: function() {
    var left = this.additive();
    var token;
    while ((token = this.expect('<', '>', '<=', '>='))) {
      left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.additive() };
    }
    return left;
  },

  additive: function() {
    var left = this.multiplicative();
    var token;
    while ((token = this.expect('+','-'))) {
      left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.multiplicative() };
    }
    return left;
  },

  multiplicative: function() {
    var left = this.unary();
    var token;
    while ((token = this.expect('*','/','%'))) {
      left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.unary() };
    }
    return left;
  },

  unary: function() {
    var token;
    if ((token = this.expect('+', '-', '!'))) {
      return { type: AST.UnaryExpression, operator: token.text, prefix: true, argument: this.unary() };
    } else {
      return this.primary();
    }
  },

  primary: function() {
    var primary;
    if (this.expect('(')) {
      primary = this.filterChain();
      this.consume(')');
    } else if (this.expect('[')) {
      primary = this.arrayDeclaration();
    } else if (this.expect('{')) {
      primary = this.object();
    } else if (this.selfReferential.hasOwnProperty(this.peek().text)) {
      primary = copy(this.selfReferential[this.consume().text]);
    } else if (this.options.literals.hasOwnProperty(this.peek().text)) {
      primary = { type: AST.Literal, value: this.options.literals[this.consume().text]};
    } else if (this.peek().identifier) {
      primary = this.identifier();
    } else if (this.peek().constant) {
      primary = this.constant();
    } else {
      this.throwError('not a primary expression', this.peek());
    }

    var next;
    while ((next = this.expect('(', '[', '.'))) {
      if (next.text === '(') {
        primary = {type: AST.CallExpression, callee: primary, arguments: this.parseArguments() };
        this.consume(')');
      } else if (next.text === '[') {
        primary = { type: AST.MemberExpression, object: primary, property: this.expression(), computed: true };
        this.consume(']');
      } else if (next.text === '.') {
        primary = { type: AST.MemberExpression, object: primary, property: this.identifier(), computed: false };
      } else {
        this.throwError('IMPOSSIBLE');
      }
    }
    return primary;
  },

  filter: function(baseExpression) {
    var args = [baseExpression];
    var result = {type: AST.CallExpression, callee: this.identifier(), arguments: args, filter: true};

    while (this.expect(':')) {
      args.push(this.expression());
    }

    return result;
  },

  parseArguments: function() {
    var args = [];
    if (this.peekToken().text !== ')') {
      do {
        args.push(this.filterChain());
      } while (this.expect(','));
    }
    return args;
  },

  identifier: function() {
    var token = this.consume();
    if (!token.identifier) {
      this.throwError('is not a valid identifier', token);
    }
    return { type: AST.Identifier, name: token.text };
  },

  constant: function() {
    // TODO check that it is a constant
    return { type: AST.Literal, value: this.consume().value };
  },

  arrayDeclaration: function() {
    var elements = [];
    if (this.peekToken().text !== ']') {
      do {
        if (this.peek(']')) {
          // Support trailing commas per ES5.1.
          break;
        }
        elements.push(this.expression());
      } while (this.expect(','));
    }
    this.consume(']');

    return { type: AST.ArrayExpression, elements: elements };
  },

  object: function() {
    var properties = [], property;
    if (this.peekToken().text !== '}') {
      do {
        if (this.peek('}')) {
          // Support trailing commas per ES5.1.
          break;
        }
        property = {type: AST.Property, kind: 'init'};
        if (this.peek().constant) {
          property.key = this.constant();
          property.computed = false;
          this.consume(':');
          property.value = this.expression();
        } else if (this.peek().identifier) {
          property.key = this.identifier();
          property.computed = false;
          if (this.peek(':')) {
            this.consume(':');
            property.value = this.expression();
          } else {
            property.value = property.key;
          }
        } else if (this.peek('[')) {
          this.consume('[');
          property.key = this.expression();
          this.consume(']');
          property.computed = true;
          this.consume(':');
          property.value = this.expression();
        } else {
          this.throwError('invalid key', this.peek());
        }
        properties.push(property);
      } while (this.expect(','));
    }
    this.consume('}');

    return {type: AST.ObjectExpression, properties: properties };
  },

  throwError: function(msg, token) {
    throw $parseMinErr('syntax',
        'Syntax Error: Token \'{0}\' {1} at column {2} of the expression [{3}] starting at [{4}].',
          token.text, msg, (token.index + 1), this.text, this.text.substring(token.index));
  },

  consume: function(e1) {
    if (this.tokens.length === 0) {
      throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
    }

    var token = this.expect(e1);
    if (!token) {
      this.throwError('is unexpected, expecting [' + e1 + ']', this.peek());
    }
    return token;
  },

  peekToken: function() {
    if (this.tokens.length === 0) {
      throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
    }
    return this.tokens[0];
  },

  peek: function(e1, e2, e3, e4) {
    return this.peekAhead(0, e1, e2, e3, e4);
  },

  peekAhead: function(i, e1, e2, e3, e4) {
    if (this.tokens.length > i) {
      var token = this.tokens[i];
      var t = token.text;
      if (t === e1 || t === e2 || t === e3 || t === e4 ||
          (!e1 && !e2 && !e3 && !e4)) {
        return token;
      }
    }
    return false;
  },

  expect: function(e1, e2, e3, e4) {
    var token = this.peek(e1, e2, e3, e4);
    if (token) {
      this.tokens.shift();
      return token;
    }
    return false;
  },

  selfReferential: {
    'this': {type: AST.ThisExpression },
    '$locals': {type: AST.LocalsExpression }
  }
};

function ifDefined(v, d) {
  return typeof v !== 'undefined' ? v : d;
}

function plusFn(l, r) {
  if (typeof l === 'undefined') return r;
  if (typeof r === 'undefined') return l;
  return l + r;
}

function isStateless($filter, filterName) {
  var fn = $filter(filterName);
  return !fn.$stateful;
}

var PURITY_ABSOLUTE = 1;
var PURITY_RELATIVE = 2;

// Detect nodes which could depend on non-shallow state of objects
function isPure(node, parentIsPure) {
  switch (node.type) {
    // Computed members might invoke a stateful toString()
    case AST.MemberExpression:
      if (node.computed) {
        return false;
      }
      break;

    // Unary always convert to primative
    case AST.UnaryExpression:
      return PURITY_ABSOLUTE;

    // The binary + operator can invoke a stateful toString().
    case AST.BinaryExpression:
      return node.operator !== '+' ? PURITY_ABSOLUTE : false;

    // Functions / filters probably read state from within objects
    case AST.CallExpression:
      return false;
  }

  return (undefined === parentIsPure) ? PURITY_RELATIVE : parentIsPure;
}

function findConstantAndWatchExpressions(ast, $filter, parentIsPure) {
  var allConstants;
  var argsToWatch;
  var isStatelessFilter;

  var astIsPure = ast.isPure = isPure(ast, parentIsPure);

  switch (ast.type) {
  case AST.Program:
    allConstants = true;
    forEach(ast.body, function(expr) {
      findConstantAndWatchExpressions(expr.expression, $filter, astIsPure);
      allConstants = allConstants && expr.expression.constant;
    });
    ast.constant = allConstants;
    break;
  case AST.Literal:
    ast.constant = true;
    ast.toWatch = [];
    break;
  case AST.UnaryExpression:
    findConstantAndWatchExpressions(ast.argument, $filter, astIsPure);
    ast.constant = ast.argument.constant;
    ast.toWatch = ast.argument.toWatch;
    break;
  case AST.BinaryExpression:
    findConstantAndWatchExpressions(ast.left, $filter, astIsPure);
    findConstantAndWatchExpressions(ast.right, $filter, astIsPure);
    ast.constant = ast.left.constant && ast.right.constant;
    ast.toWatch = ast.left.toWatch.concat(ast.right.toWatch);
    break;
  case AST.LogicalExpression:
    findConstantAndWatchExpressions(ast.left, $filter, astIsPure);
    findConstantAndWatchExpressions(ast.right, $filter, astIsPure);
    ast.constant = ast.left.constant && ast.right.constant;
    ast.toWatch = ast.constant ? [] : [ast];
    break;
  case AST.ConditionalExpression:
    findConstantAndWatchExpressions(ast.test, $filter, astIsPure);
    findConstantAndWatchExpressions(ast.alternate, $filter, astIsPure);
    findConstantAndWatchExpressions(ast.consequent, $filter, astIsPure);
    ast.constant = ast.test.constant && ast.alternate.constant && ast.consequent.constant;
    ast.toWatch = ast.constant ? [] : [ast];
    break;
  case AST.Identifier:
    ast.constant = false;
    ast.toWatch = [ast];
    break;
  case AST.MemberExpression:
    findConstantAndWatchExpressions(ast.object, $filter, astIsPure);
    if (ast.computed) {
      findConstantAndWatchExpressions(ast.property, $filter, astIsPure);
    }
    ast.constant = ast.object.constant && (!ast.computed || ast.property.constant);
    ast.toWatch = ast.constant ? [] : [ast];
    break;
  case AST.CallExpression:
    isStatelessFilter = ast.filter ? isStateless($filter, ast.callee.name) : false;
    allConstants = isStatelessFilter;
    argsToWatch = [];
    forEach(ast.arguments, function(expr) {
      findConstantAndWatchExpressions(expr, $filter, astIsPure);
      allConstants = allConstants && expr.constant;
      argsToWatch.push.apply(argsToWatch, expr.toWatch);
    });
    ast.constant = allConstants;
    ast.toWatch = isStatelessFilter ? argsToWatch : [ast];
    break;
  case AST.AssignmentExpression:
    findConstantAndWatchExpressions(ast.left, $filter, astIsPure);
    findConstantAndWatchExpressions(ast.right, $filter, astIsPure);
    ast.constant = ast.left.constant && ast.right.constant;
    ast.toWatch = [ast];
    break;
  case AST.ArrayExpression:
    allConstants = true;
    argsToWatch = [];
    forEach(ast.elements, function(expr) {
      findConstantAndWatchExpressions(expr, $filter, astIsPure);
      allConstants = allConstants && expr.constant;
      argsToWatch.push.apply(argsToWatch, expr.toWatch);
    });
    ast.constant = allConstants;
    ast.toWatch = argsToWatch;
    break;
  case AST.ObjectExpression:
    allConstants = true;
    argsToWatch = [];
    forEach(ast.properties, function(property) {
      findConstantAndWatchExpressions(property.value, $filter, astIsPure);
      allConstants = allConstants && property.value.constant;
      argsToWatch.push.apply(argsToWatch, property.value.toWatch);
      if (property.computed) {
        //`{[key]: value}` implicitly does `key.toString()` which may be non-pure
        findConstantAndWatchExpressions(property.key, $filter, /*parentIsPure=*/false);
        allConstants = allConstants && property.key.constant;
        argsToWatch.push.apply(argsToWatch, property.key.toWatch);
      }
    });
    ast.constant = allConstants;
    ast.toWatch = argsToWatch;
    break;
  case AST.ThisExpression:
    ast.constant = false;
    ast.toWatch = [];
    break;
  case AST.LocalsExpression:
    ast.constant = false;
    ast.toWatch = [];
    break;
  }
}

function getInputs(body) {
  if (body.length !== 1) return;
  var lastExpression = body[0].expression;
  var candidate = lastExpression.toWatch;
  if (candidate.length !== 1) return candidate;
  return candidate[0] !== lastExpression ? candidate : undefined;
}

function isAssignable(ast) {
  return ast.type === AST.Identifier || ast.type === AST.MemberExpression;
}

function assignableAST(ast) {
  if (ast.body.length === 1 && isAssignable(ast.body[0].expression)) {
    return {type: AST.AssignmentExpression, left: ast.body[0].expression, right: {type: AST.NGValueParameter}, operator: '='};
  }
}

function isLiteral(ast) {
  return ast.body.length === 0 ||
      ast.body.length === 1 && (
      ast.body[0].expression.type === AST.Literal ||
      ast.body[0].expression.type === AST.ArrayExpression ||
      ast.body[0].expression.type === AST.ObjectExpression);
}

function isConstant(ast) {
  return ast.constant;
}

function ASTCompiler($filter) {
  this.$filter = $filter;
}

ASTCompiler.prototype = {
  compile: function(ast) {
    var self = this;
    this.state = {
      nextId: 0,
      filters: {},
      fn: {vars: [], body: [], own: {}},
      assign: {vars: [], body: [], own: {}},
      inputs: []
    };
    findConstantAndWatchExpressions(ast, self.$filter);
    var extra = '';
    var assignable;
    this.stage = 'assign';
    if ((assignable = assignableAST(ast))) {
      this.state.computing = 'assign';
      var result = this.nextId();
      this.recurse(assignable, result);
      this.return_(result);
      extra = 'fn.assign=' + this.generateFunction('assign', 's,v,l');
    }
    var toWatch = getInputs(ast.body);
    self.stage = 'inputs';
    forEach(toWatch, function(watch, key) {
      var fnKey = 'fn' + key;
      self.state[fnKey] = {vars: [], body: [], own: {}};
      self.state.computing = fnKey;
      var intoId = self.nextId();
      self.recurse(watch, intoId);
      self.return_(intoId);
      self.state.inputs.push({name: fnKey, isPure: watch.isPure});
      watch.watchId = key;
    });
    this.state.computing = 'fn';
    this.stage = 'main';
    this.recurse(ast);
    var fnString =
      // The build and minification steps remove the string "use strict" from the code, but this is done using a regex.
      // This is a workaround for this until we do a better job at only removing the prefix only when we should.
      '"' + this.USE + ' ' + this.STRICT + '";\n' +
      this.filterPrefix() +
      'var fn=' + this.generateFunction('fn', 's,l,a,i') +
      extra +
      this.watchFns() +
      'return fn;';

    // eslint-disable-next-line no-new-func
    var fn = (new Function('$filter',
        'getStringValue',
        'ifDefined',
        'plus',
        fnString))(
          this.$filter,
          getStringValue,
          ifDefined,
          plusFn);
    this.state = this.stage = undefined;
    return fn;
  },

  USE: 'use',

  STRICT: 'strict',

  watchFns: function() {
    var result = [];
    var inputs = this.state.inputs;
    var self = this;
    forEach(inputs, function(input) {
      result.push('var ' + input.name + '=' + self.generateFunction(input.name, 's'));
      if (input.isPure) {
        result.push(input.name, '.isPure=' + JSON.stringify(input.isPure) + ';');
      }
    });
    if (inputs.length) {
      result.push('fn.inputs=[' + inputs.map(function(i) { return i.name; }).join(',') + '];');
    }
    return result.join('');
  },

  generateFunction: function(name, params) {
    return 'function(' + params + '){' +
        this.varsPrefix(name) +
        this.body(name) +
        '};';
  },

  filterPrefix: function() {
    var parts = [];
    var self = this;
    forEach(this.state.filters, function(id, filter) {
      parts.push(id + '=$filter(' + self.escape(filter) + ')');
    });
    if (parts.length) return 'var ' + parts.join(',') + ';';
    return '';
  },

  varsPrefix: function(section) {
    return this.state[section].vars.length ? 'var ' + this.state[section].vars.join(',') + ';' : '';
  },

  body: function(section) {
    return this.state[section].body.join('');
  },

  recurse: function(ast, intoId, nameId, recursionFn, create, skipWatchIdCheck) {
    var left, right, self = this, args, expression, computed;
    recursionFn = recursionFn || noop;
    if (!skipWatchIdCheck && isDefined(ast.watchId)) {
      intoId = intoId || this.nextId();
      this.if_('i',
        this.lazyAssign(intoId, this.computedMember('i', ast.watchId)),
        this.lazyRecurse(ast, intoId, nameId, recursionFn, create, true)
      );
      return;
    }
    switch (ast.type) {
    case AST.Program:
      forEach(ast.body, function(expression, pos) {
        self.recurse(expression.expression, undefined, undefined, function(expr) { right = expr; });
        if (pos !== ast.body.length - 1) {
          self.current().body.push(right, ';');
        } else {
          self.return_(right);
        }
      });
      break;
    case AST.Literal:
      expression = this.escape(ast.value);
      this.assign(intoId, expression);
      recursionFn(intoId || expression);
      break;
    case AST.UnaryExpression:
      this.recurse(ast.argument, undefined, undefined, function(expr) { right = expr; });
      expression = ast.operator + '(' + this.ifDefined(right, 0) + ')';
      this.assign(intoId, expression);
      recursionFn(expression);
      break;
    case AST.BinaryExpression:
      this.recurse(ast.left, undefined, undefined, function(expr) { left = expr; });
      this.recurse(ast.right, undefined, undefined, function(expr) { right = expr; });
      if (ast.operator === '+') {
        expression = this.plus(left, right);
      } else if (ast.operator === '-') {
        expression = this.ifDefined(left, 0) + ast.operator + this.ifDefined(right, 0);
      } else {
        expression = '(' + left + ')' + ast.operator + '(' + right + ')';
      }
      this.assign(intoId, expression);
      recursionFn(expression);
      break;
    case AST.LogicalExpression:
      intoId = intoId || this.nextId();
      self.recurse(ast.left, intoId);
      self.if_(ast.operator === '&&' ? intoId : self.not(intoId), self.lazyRecurse(ast.right, intoId));
      recursionFn(intoId);
      break;
    case AST.ConditionalExpression:
      intoId = intoId || this.nextId();
      self.recurse(ast.test, intoId);
      self.if_(intoId, self.lazyRecurse(ast.alternate, intoId), self.lazyRecurse(ast.consequent, intoId));
      recursionFn(intoId);
      break;
    case AST.Identifier:
      intoId = intoId || this.nextId();
      if (nameId) {
        nameId.context = self.stage === 'inputs' ? 's' : this.assign(this.nextId(), this.getHasOwnProperty('l', ast.name) + '?l:s');
        nameId.computed = false;
        nameId.name = ast.name;
      }
      self.if_(self.stage === 'inputs' || self.not(self.getHasOwnProperty('l', ast.name)),
        function() {
          self.if_(self.stage === 'inputs' || 's', function() {
            if (create && create !== 1) {
              self.if_(
                self.isNull(self.nonComputedMember('s', ast.name)),
                self.lazyAssign(self.nonComputedMember('s', ast.name), '{}'));
            }
            self.assign(intoId, self.nonComputedMember('s', ast.name));
          });
        }, intoId && self.lazyAssign(intoId, self.nonComputedMember('l', ast.name))
        );
      recursionFn(intoId);
      break;
    case AST.MemberExpression:
      left = nameId && (nameId.context = this.nextId()) || this.nextId();
      intoId = intoId || this.nextId();
      self.recurse(ast.object, left, undefined, function() {
        self.if_(self.notNull(left), function() {
          if (ast.computed) {
            right = self.nextId();
            self.recurse(ast.property, right);
            self.getStringValue(right);
            if (create && create !== 1) {
              self.if_(self.not(self.computedMember(left, right)), self.lazyAssign(self.computedMember(left, right), '{}'));
            }
            expression = self.computedMember(left, right);
            self.assign(intoId, expression);
            if (nameId) {
              nameId.computed = true;
              nameId.name = right;
            }
          } else {
            if (create && create !== 1) {
              self.if_(self.isNull(self.nonComputedMember(left, ast.property.name)), self.lazyAssign(self.nonComputedMember(left, ast.property.name), '{}'));
            }
            expression = self.nonComputedMember(left, ast.property.name);
            self.assign(intoId, expression);
            if (nameId) {
              nameId.computed = false;
              nameId.name = ast.property.name;
            }
          }
        }, function() {
          self.assign(intoId, 'undefined');
        });
        recursionFn(intoId);
      }, !!create);
      break;
    case AST.CallExpression:
      intoId = intoId || this.nextId();
      if (ast.filter) {
        right = self.filter(ast.callee.name);
        args = [];
        forEach(ast.arguments, function(expr) {
          var argument = self.nextId();
          self.recurse(expr, argument);
          args.push(argument);
        });
        expression = right + '(' + args.join(',') + ')';
        self.assign(intoId, expression);
        recursionFn(intoId);
      } else {
        right = self.nextId();
        left = {};
        args = [];
        self.recurse(ast.callee, right, left, function() {
          self.if_(self.notNull(right), function() {
            forEach(ast.arguments, function(expr) {
              self.recurse(expr, ast.constant ? undefined : self.nextId(), undefined, function(argument) {
                args.push(argument);
              });
            });
            if (left.name) {
              expression = self.member(left.context, left.name, left.computed) + '(' + args.join(',') + ')';
            } else {
              expression = right + '(' + args.join(',') + ')';
            }
            self.assign(intoId, expression);
          }, function() {
            self.assign(intoId, 'undefined');
          });
          recursionFn(intoId);
        });
      }
      break;
    case AST.AssignmentExpression:
      right = this.nextId();
      left = {};
      this.recurse(ast.left, undefined, left, function() {
        self.if_(self.notNull(left.context), function() {
          self.recurse(ast.right, right);
          expression = self.member(left.context, left.name, left.computed) + ast.operator + right;
          self.assign(intoId, expression);
          recursionFn(intoId || expression);
        });
      }, 1);
      break;
    case AST.ArrayExpression:
      args = [];
      forEach(ast.elements, function(expr) {
        self.recurse(expr, ast.constant ? undefined : self.nextId(), undefined, function(argument) {
          args.push(argument);
        });
      });
      expression = '[' + args.join(',') + ']';
      this.assign(intoId, expression);
      recursionFn(intoId || expression);
      break;
    case AST.ObjectExpression:
      args = [];
      computed = false;
      forEach(ast.properties, function(property) {
        if (property.computed) {
          computed = true;
        }
      });
      if (computed) {
        intoId = intoId || this.nextId();
        this.assign(intoId, '{}');
        forEach(ast.properties, function(property) {
          if (property.computed) {
            left = self.nextId();
            self.recurse(property.key, left);
          } else {
            left = property.key.type === AST.Identifier ?
                       property.key.name :
                       ('' + property.key.value);
          }
          right = self.nextId();
          self.recurse(property.value, right);
          self.assign(self.member(intoId, left, property.computed), right);
        });
      } else {
        forEach(ast.properties, function(property) {
          self.recurse(property.value, ast.constant ? undefined : self.nextId(), undefined, function(expr) {
            args.push(self.escape(
                property.key.type === AST.Identifier ? property.key.name :
                  ('' + property.key.value)) +
                ':' + expr);
          });
        });
        expression = '{' + args.join(',') + '}';
        this.assign(intoId, expression);
      }
      recursionFn(intoId || expression);
      break;
    case AST.ThisExpression:
      this.assign(intoId, 's');
      recursionFn(intoId || 's');
      break;
    case AST.LocalsExpression:
      this.assign(intoId, 'l');
      recursionFn(intoId || 'l');
      break;
    case AST.NGValueParameter:
      this.assign(intoId, 'v');
      recursionFn(intoId || 'v');
      break;
    }
  },

  getHasOwnProperty: function(element, property) {
    var key = element + '.' + property;
    var own = this.current().own;
    if (!own.hasOwnProperty(key)) {
      own[key] = this.nextId(false, element + '&&(' + this.escape(property) + ' in ' + element + ')');
    }
    return own[key];
  },

  assign: function(id, value) {
    if (!id) return;
    this.current().body.push(id, '=', value, ';');
    return id;
  },

  filter: function(filterName) {
    if (!this.state.filters.hasOwnProperty(filterName)) {
      this.state.filters[filterName] = this.nextId(true);
    }
    return this.state.filters[filterName];
  },

  ifDefined: function(id, defaultValue) {
    return 'ifDefined(' + id + ',' + this.escape(defaultValue) + ')';
  },

  plus: function(left, right) {
    return 'plus(' + left + ',' + right + ')';
  },

  return_: function(id) {
    this.current().body.push('return ', id, ';');
  },

  if_: function(test, alternate, consequent) {
    if (test === true) {
      alternate();
    } else {
      var body = this.current().body;
      body.push('if(', test, '){');
      alternate();
      body.push('}');
      if (consequent) {
        body.push('else{');
        consequent();
        body.push('}');
      }
    }
  },

  not: function(expression) {
    return '!(' + expression + ')';
  },

  isNull: function(expression) {
    return expression + '==null';
  },

  notNull: function(expression) {
    return expression + '!=null';
  },

  nonComputedMember: function(left, right) {
    var SAFE_IDENTIFIER = /^[$_a-zA-Z][$_a-zA-Z0-9]*$/;
    var UNSAFE_CHARACTERS = /[^$_a-zA-Z0-9]/g;
    if (SAFE_IDENTIFIER.test(right)) {
      return left + '.' + right;
    } else {
      return left  + '["' + right.replace(UNSAFE_CHARACTERS, this.stringEscapeFn) + '"]';
    }
  },

  computedMember: function(left, right) {
    return left + '[' + right + ']';
  },

  member: function(left, right, computed) {
    if (computed) return this.computedMember(left, right);
    return this.nonComputedMember(left, right);
  },

  getStringValue: function(item) {
    this.assign(item, 'getStringValue(' + item + ')');
  },

  lazyRecurse: function(ast, intoId, nameId, recursionFn, create, skipWatchIdCheck) {
    var self = this;
    return function() {
      self.recurse(ast, intoId, nameId, recursionFn, create, skipWatchIdCheck);
    };
  },

  lazyAssign: function(id, value) {
    var self = this;
    return function() {
      self.assign(id, value);
    };
  },

  stringEscapeRegex: /[^ a-zA-Z0-9]/g,

  stringEscapeFn: function(c) {
    return '\\u' + ('0000' + c.charCodeAt(0).toString(16)).slice(-4);
  },

  escape: function(value) {
    if (isString(value)) return '\'' + value.replace(this.stringEscapeRegex, this.stringEscapeFn) + '\'';
    if (isNumber(value)) return value.toString();
    if (value === true) return 'true';
    if (value === false) return 'false';
    if (value === null) return 'null';
    if (typeof value === 'undefined') return 'undefined';

    throw $parseMinErr('esc', 'IMPOSSIBLE');
  },

  nextId: function(skip, init) {
    var id = 'v' + (this.state.nextId++);
    if (!skip) {
      this.current().vars.push(id + (init ? '=' + init : ''));
    }
    return id;
  },

  current: function() {
    return this.state[this.state.computing];
  }
};


function ASTInterpreter($filter) {
  this.$filter = $filter;
}

ASTInterpreter.prototype = {
  compile: function(ast) {
    var self = this;
    findConstantAndWatchExpressions(ast, self.$filter);
    var assignable;
    var assign;
    if ((assignable = assignableAST(ast))) {
      assign = this.recurse(assignable);
    }
    var toWatch = getInputs(ast.body);
    var inputs;
    if (toWatch) {
      inputs = [];
      forEach(toWatch, function(watch, key) {
        var input = self.recurse(watch);
        input.isPure = watch.isPure;
        watch.input = input;
        inputs.push(input);
        watch.watchId = key;
      });
    }
    var expressions = [];
    forEach(ast.body, function(expression) {
      expressions.push(self.recurse(expression.expression));
    });
    var fn = ast.body.length === 0 ? noop :
             ast.body.length === 1 ? expressions[0] :
             function(scope, locals) {
               var lastValue;
               forEach(expressions, function(exp) {
                 lastValue = exp(scope, locals);
               });
               return lastValue;
             };
    if (assign) {
      fn.assign = function(scope, value, locals) {
        return assign(scope, locals, value);
      };
    }
    if (inputs) {
      fn.inputs = inputs;
    }
    return fn;
  },

  recurse: function(ast, context, create) {
    var left, right, self = this, args;
    if (ast.input) {
      return this.inputs(ast.input, ast.watchId);
    }
    switch (ast.type) {
    case AST.Literal:
      return this.value(ast.value, context);
    case AST.UnaryExpression:
      right = this.recurse(ast.argument);
      return this['unary' + ast.operator](right, context);
    case AST.BinaryExpression:
      left = this.recurse(ast.left);
      right = this.recurse(ast.right);
      return this['binary' + ast.operator](left, right, context);
    case AST.LogicalExpression:
      left = this.recurse(ast.left);
      right = this.recurse(ast.right);
      return this['binary' + ast.operator](left, right, context);
    case AST.ConditionalExpression:
      return this['ternary?:'](
        this.recurse(ast.test),
        this.recurse(ast.alternate),
        this.recurse(ast.consequent),
        context
      );
    case AST.Identifier:
      return self.identifier(ast.name, context, create);
    case AST.MemberExpression:
      left = this.recurse(ast.object, false, !!create);
      if (!ast.computed) {
        right = ast.property.name;
      }
      if (ast.computed) right = this.recurse(ast.property);
      return ast.computed ?
        this.computedMember(left, right, context, create) :
        this.nonComputedMember(left, right, context, create);
    case AST.CallExpression:
      args = [];
      forEach(ast.arguments, function(expr) {
        args.push(self.recurse(expr));
      });
      if (ast.filter) right = this.$filter(ast.callee.name);
      if (!ast.filter) right = this.recurse(ast.callee, true);
      return ast.filter ?
        function(scope, locals, assign, inputs) {
          var values = [];
          for (var i = 0; i < args.length; ++i) {
            values.push(args[i](scope, locals, assign, inputs));
          }
          var value = right.apply(undefined, values, inputs);
          return context ? {context: undefined, name: undefined, value: value} : value;
        } :
        function(scope, locals, assign, inputs) {
          var rhs = right(scope, locals, assign, inputs);
          var value;
          if (rhs.value != null) {
            var values = [];
            for (var i = 0; i < args.length; ++i) {
              values.push(args[i](scope, locals, assign, inputs));
            }
            value = rhs.value.apply(rhs.context, values);
          }
          return context ? {value: value} : value;
        };
    case AST.AssignmentExpression:
      left = this.recurse(ast.left, true, 1);
      right = this.recurse(ast.right);
      return function(scope, locals, assign, inputs) {
        var lhs = left(scope, locals, assign, inputs);
        var rhs = right(scope, locals, assign, inputs);
        lhs.context[lhs.name] = rhs;
        return context ? {value: rhs} : rhs;
      };
    case AST.ArrayExpression:
      args = [];
      forEach(ast.elements, function(expr) {
        args.push(self.recurse(expr));
      });
      return function(scope, locals, assign, inputs) {
        var value = [];
        for (var i = 0; i < args.length; ++i) {
          value.push(args[i](scope, locals, assign, inputs));
        }
        return context ? {value: value} : value;
      };
    case AST.ObjectExpression:
      args = [];
      forEach(ast.properties, function(property) {
        if (property.computed) {
          args.push({key: self.recurse(property.key),
                     computed: true,
                     value: self.recurse(property.value)
          });
        } else {
          args.push({key: property.key.type === AST.Identifier ?
                          property.key.name :
                          ('' + property.key.value),
                     computed: false,
                     value: self.recurse(property.value)
          });
        }
      });
      return function(scope, locals, assign, inputs) {
        var value = {};
        for (var i = 0; i < args.length; ++i) {
          if (args[i].computed) {
            value[args[i].key(scope, locals, assign, inputs)] = args[i].value(scope, locals, assign, inputs);
          } else {
            value[args[i].key] = args[i].value(scope, locals, assign, inputs);
          }
        }
        return context ? {value: value} : value;
      };
    case AST.ThisExpression:
      return function(scope) {
        return context ? {value: scope} : scope;
      };
    case AST.LocalsExpression:
      return function(scope, locals) {
        return context ? {value: locals} : locals;
      };
    case AST.NGValueParameter:
      return function(scope, locals, assign) {
        return context ? {value: assign} : assign;
      };
    }
  },

  'unary+': function(argument, context) {
    return function(scope, locals, assign, inputs) {
      var arg = argument(scope, locals, assign, inputs);
      if (isDefined(arg)) {
        arg = +arg;
      } else {
        arg = 0;
      }
      return context ? {value: arg} : arg;
    };
  },
  'unary-': function(argument, context) {
    return function(scope, locals, assign, inputs) {
      var arg = argument(scope, locals, assign, inputs);
      if (isDefined(arg)) {
        arg = -arg;
      } else {
        arg = -0;
      }
      return context ? {value: arg} : arg;
    };
  },
  'unary!': function(argument, context) {
    return function(scope, locals, assign, inputs) {
      var arg = !argument(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary+': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var lhs = left(scope, locals, assign, inputs);
      var rhs = right(scope, locals, assign, inputs);
      var arg = plusFn(lhs, rhs);
      return context ? {value: arg} : arg;
    };
  },
  'binary-': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var lhs = left(scope, locals, assign, inputs);
      var rhs = right(scope, locals, assign, inputs);
      var arg = (isDefined(lhs) ? lhs : 0) - (isDefined(rhs) ? rhs : 0);
      return context ? {value: arg} : arg;
    };
  },
  'binary*': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) * right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary/': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) / right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary%': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) % right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary===': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) === right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary!==': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) !== right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary==': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      // eslint-disable-next-line eqeqeq
      var arg = left(scope, locals, assign, inputs) == right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary!=': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      // eslint-disable-next-line eqeqeq
      var arg = left(scope, locals, assign, inputs) != right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary<': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) < right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary>': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) > right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary<=': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) <= right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary>=': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) >= right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary&&': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) && right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'binary||': function(left, right, context) {
    return function(scope, locals, assign, inputs) {
      var arg = left(scope, locals, assign, inputs) || right(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  'ternary?:': function(test, alternate, consequent, context) {
    return function(scope, locals, assign, inputs) {
      var arg = test(scope, locals, assign, inputs) ? alternate(scope, locals, assign, inputs) : consequent(scope, locals, assign, inputs);
      return context ? {value: arg} : arg;
    };
  },
  value: function(value, context) {
    return function() { return context ? {context: undefined, name: undefined, value: value} : value; };
  },
  identifier: function(name, context, create) {
    return function(scope, locals, assign, inputs) {
      var base = locals && (name in locals) ? locals : scope;
      if (create && create !== 1 && base && base[name] == null) {
        base[name] = {};
      }
      var value = base ? base[name] : undefined;
      if (context) {
        return {context: base, name: name, value: value};
      } else {
        return value;
      }
    };
  },
  computedMember: function(left, right, context, create) {
    return function(scope, locals, assign, inputs) {
      var lhs = left(scope, locals, assign, inputs);
      var rhs;
      var value;
      if (lhs != null) {
        rhs = right(scope, locals, assign, inputs);
        rhs = getStringValue(rhs);
        if (create && create !== 1) {
          if (lhs && !(lhs[rhs])) {
            lhs[rhs] = {};
          }
        }
        value = lhs[rhs];
      }
      if (context) {
        return {context: lhs, name: rhs, value: value};
      } else {
        return value;
      }
    };
  },
  nonComputedMember: function(left, right, context, create) {
    return function(scope, locals, assign, inputs) {
      var lhs = left(scope, locals, assign, inputs);
      if (create && create !== 1) {
        if (lhs && lhs[right] == null) {
          lhs[right] = {};
        }
      }
      var value = lhs != null ? lhs[right] : undefined;
      if (context) {
        return {context: lhs, name: right, value: value};
      } else {
        return value;
      }
    };
  },
  inputs: function(input, watchId) {
    return function(scope, value, locals, inputs) {
      if (inputs) return inputs[watchId];
      return input(scope, value, locals);
    };
  }
};

/**
 * @constructor
 */
function Parser(lexer, $filter, options) {
  this.ast = new AST(lexer, options);
  this.astCompiler = options.csp ? new ASTInterpreter($filter) :
                                   new ASTCompiler($filter);
}

Parser.prototype = {
  constructor: Parser,

  parse: function(text) {
    var ast = this.getAst(text);
    var fn = this.astCompiler.compile(ast.ast);
    fn.literal = isLiteral(ast.ast);
    fn.constant = isConstant(ast.ast);
    fn.oneTime = ast.oneTime;
    return fn;
  },

  getAst: function(exp) {
    var oneTime = false;
    exp = exp.trim();

    if (exp.charAt(0) === ':' && exp.charAt(1) === ':') {
      oneTime = true;
      exp = exp.substring(2);
    }
    return {
      ast: this.ast.ast(exp),
      oneTime: oneTime
    };
  }
};

function getValueOf(value) {
  return isFunction(value.valueOf) ? value.valueOf() : objectValueOf.call(value);
}

///////////////////////////////////

/**
 * @ngdoc service
 * @name $parse
 * @kind function
 *
 * @description
 *
 * Converts AngularJS {@link guide/expression expression} into a function.
 *
 * ```js
 *   var getter = $parse('user.name');
 *   var setter = getter.assign;
 *   var context = {user:{name:'AngularJS'}};
 *   var locals = {user:{name:'local'}};
 *
 *   expect(getter(context)).toEqual('AngularJS');
 *   setter(context, 'newValue');
 *   expect(context.user.name).toEqual('newValue');
 *   expect(getter(context, locals)).toEqual('local');
 * ```
 *
 *
 * @param {string} expression String expression to compile.
 * @returns {function(context, locals)} a function which represents the compiled expression:
 *
 *    * `context` – `{object}` – an object against which any expressions embedded in the strings
 *      are evaluated against (typically a scope object).
 *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
 *      `context`.
 *
 *    The returned function also has the following properties:
 *      * `literal` – `{boolean}` – whether the expression's top-level node is a JavaScript
 *        literal.
 *      * `constant` – `{boolean}` – whether the expression is made entirely of JavaScript
 *        constant literals.
 *      * `assign` – `{?function(context, value)}` – if the expression is assignable, this will be
 *        set to a function to change its value on the given context.
 *
 */


/**
 * @ngdoc provider
 * @name $parseProvider
 * @this
 *
 * @description
 * `$parseProvider` can be used for configuring the default behavior of the {@link ng.$parse $parse}
 *  service.
 */
function $ParseProvider() {
  var cache = createMap();
  var literals = {
    'true': true,
    'false': false,
    'null': null,
    'undefined': undefined
  };
  var identStart, identContinue;

  /**
   * @ngdoc method
   * @name $parseProvider#addLiteral
   * @description
   *
   * Configure $parse service to add literal values that will be present as literal at expressions.
   *
   * @param {string} literalName Token for the literal value. The literal name value must be a valid literal name.
   * @param {*} literalValue Value for this literal. All literal values must be primitives or `undefined`.
   *
   **/
  this.addLiteral = function(literalName, literalValue) {
    literals[literalName] = literalValue;
  };

 /**
  * @ngdoc method
  * @name $parseProvider#setIdentifierFns
  *
  * @description
  *
  * Allows defining the set of characters that are allowed in AngularJS expressions. The function
  * `identifierStart` will get called to know if a given character is a valid character to be the
  * first character for an identifier. The function `identifierContinue` will get called to know if
  * a given character is a valid character to be a follow-up identifier character. The functions
  * `identifierStart` and `identifierContinue` will receive as arguments the single character to be
  * identifier and the character code point. These arguments will be `string` and `numeric`. Keep in
  * mind that the `string` parameter can be two characters long depending on the character
  * representation. It is expected for the function to return `true` or `false`, whether that
  * character is allowed or not.
  *
  * Since this function will be called extensively, keep the implementation of these functions fast,
  * as the performance of these functions have a direct impact on the expressions parsing speed.
  *
  * @param {function=} identifierStart The function that will decide whether the given character is
  *   a valid identifier start character.
  * @param {function=} identifierContinue The function that will decide whether the given character is
  *   a valid identifier continue character.
  */
  this.setIdentifierFns = function(identifierStart, identifierContinue) {
    identStart = identifierStart;
    identContinue = identifierContinue;
    return this;
  };

  this.$get = ['$filter', function($filter) {
    var noUnsafeEval = csp().noUnsafeEval;
    var $parseOptions = {
          csp: noUnsafeEval,
          literals: copy(literals),
          isIdentifierStart: isFunction(identStart) && identStart,
          isIdentifierContinue: isFunction(identContinue) && identContinue
        };
    $parse.$$getAst = $$getAst;
    return $parse;

    function $parse(exp, interceptorFn) {
      var parsedExpression, cacheKey;

      switch (typeof exp) {
        case 'string':
          exp = exp.trim();
          cacheKey = exp;

          parsedExpression = cache[cacheKey];

          if (!parsedExpression) {
            var lexer = new Lexer($parseOptions);
            var parser = new Parser(lexer, $filter, $parseOptions);
            parsedExpression = parser.parse(exp);

            cache[cacheKey] = addWatchDelegate(parsedExpression);
          }
          return addInterceptor(parsedExpression, interceptorFn);

        case 'function':
          return addInterceptor(exp, interceptorFn);

        default:
          return addInterceptor(noop, interceptorFn);
      }
    }

    function $$getAst(exp) {
      var lexer = new Lexer($parseOptions);
      var parser = new Parser(lexer, $filter, $parseOptions);
      return parser.getAst(exp).ast;
    }

    function expressionInputDirtyCheck(newValue, oldValueOfValue, compareObjectIdentity) {

      if (newValue == null || oldValueOfValue == null) { // null/undefined
        return newValue === oldValueOfValue;
      }

      if (typeof newValue === 'object') {

        // attempt to convert the value to a primitive type
        // TODO(docs): add a note to docs that by implementing valueOf even objects and arrays can
        //             be cheaply dirty-checked
        newValue = getValueOf(newValue);

        if (typeof newValue === 'object' && !compareObjectIdentity) {
          // objects/arrays are not supported - deep-watching them would be too expensive
          return false;
        }

        // fall-through to the primitive equality check
      }

      //Primitive or NaN
      // eslint-disable-next-line no-self-compare
      return newValue === oldValueOfValue || (newValue !== newValue && oldValueOfValue !== oldValueOfValue);
    }

    function inputsWatchDelegate(scope, listener, objectEquality, parsedExpression, prettyPrintExpression) {
      var inputExpressions = parsedExpression.inputs;
      var lastResult;

      if (inputExpressions.length === 1) {
        var oldInputValueOf = expressionInputDirtyCheck; // init to something unique so that equals check fails
        inputExpressions = inputExpressions[0];
        return scope.$watch(function expressionInputWatch(scope) {
          var newInputValue = inputExpressions(scope);
          if (!expressionInputDirtyCheck(newInputValue, oldInputValueOf, inputExpressions.isPure)) {
            lastResult = parsedExpression(scope, undefined, undefined, [newInputValue]);
            oldInputValueOf = newInputValue && getValueOf(newInputValue);
          }
          return lastResult;
        }, listener, objectEquality, prettyPrintExpression);
      }

      var oldInputValueOfValues = [];
      var oldInputValues = [];
      for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
        oldInputValueOfValues[i] = expressionInputDirtyCheck; // init to something unique so that equals check fails
        oldInputValues[i] = null;
      }

      return scope.$watch(function expressionInputsWatch(scope) {
        var changed = false;

        for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
          var newInputValue = inputExpressions[i](scope);
          if (changed || (changed = !expressionInputDirtyCheck(newInputValue, oldInputValueOfValues[i], inputExpressions[i].isPure))) {
            oldInputValues[i] = newInputValue;
            oldInputValueOfValues[i] = newInputValue && getValueOf(newInputValue);
          }
        }

        if (changed) {
          lastResult = parsedExpression(scope, undefined, undefined, oldInputValues);
        }

        return lastResult;
      }, listener, objectEquality, prettyPrintExpression);
    }

    function oneTimeWatchDelegate(scope, listener, objectEquality, parsedExpression, prettyPrintExpression) {
      var isDone = parsedExpression.literal ? isAllDefined : isDefined;
      var unwatch, lastValue;

      var exp = parsedExpression.$$intercepted || parsedExpression;
      var post = parsedExpression.$$interceptor || identity;

      var useInputs = parsedExpression.inputs && !exp.inputs;

      // Propogate the literal/inputs/constant attributes
      // ... but not oneTime since we are handling it
      oneTimeWatch.literal = parsedExpression.literal;
      oneTimeWatch.constant = parsedExpression.constant;
      oneTimeWatch.inputs = parsedExpression.inputs;

      // Allow other delegates to run on this wrapped expression
      addWatchDelegate(oneTimeWatch);

      unwatch = scope.$watch(oneTimeWatch, listener, objectEquality, prettyPrintExpression);

      return unwatch;

      function unwatchIfDone() {
        if (isDone(lastValue)) {
          unwatch();
        }
      }

      function oneTimeWatch(scope, locals, assign, inputs) {
        lastValue = useInputs && inputs ? inputs[0] : exp(scope, locals, assign, inputs);
        if (isDone(lastValue)) {
          scope.$$postDigest(unwatchIfDone);
        }
        return post(lastValue);
      }
    }

    function isAllDefined(value) {
      var allDefined = true;
      forEach(value, function(val) {
        if (!isDefined(val)) allDefined = false;
      });
      return allDefined;
    }

    function constantWatchDelegate(scope, listener, objectEquality, parsedExpression) {
      var unwatch = scope.$watch(function constantWatch(scope) {
        unwatch();
        return parsedExpression(scope);
      }, listener, objectEquality);
      return unwatch;
    }

    function addWatchDelegate(parsedExpression) {
      if (parsedExpression.constant) {
        parsedExpression.$$watchDelegate = constantWatchDelegate;
      } else if (parsedExpression.oneTime) {
        parsedExpression.$$watchDelegate = oneTimeWatchDelegate;
      } else if (parsedExpression.inputs) {
        parsedExpression.$$watchDelegate = inputsWatchDelegate;
      }

      return parsedExpression;
    }

    function chainInterceptors(first, second) {
      function chainedInterceptor(value) {
        return second(first(value));
      }
      chainedInterceptor.$stateful = first.$stateful || second.$stateful;
      chainedInterceptor.$$pure = first.$$pure && second.$$pure;

      return chainedInterceptor;
    }

    function addInterceptor(parsedExpression, interceptorFn) {
      if (!interceptorFn) return parsedExpression;

      // Extract any existing interceptors out of the parsedExpression
      // to ensure the original parsedExpression is always the $$intercepted
      if (parsedExpression.$$interceptor) {
        interceptorFn = chainInterceptors(parsedExpression.$$interceptor, interceptorFn);
        parsedExpression = parsedExpression.$$intercepted;
      }

      var useInputs = false;

      var fn = function interceptedExpression(scope, locals, assign, inputs) {
        var value = useInputs && inputs ? inputs[0] : parsedExpression(scope, locals, assign, inputs);
        return interceptorFn(value);
      };

      // Maintain references to the interceptor/intercepted
      fn.$$intercepted = parsedExpression;
      fn.$$interceptor = interceptorFn;

      // Propogate the literal/oneTime/constant attributes
      fn.literal = parsedExpression.literal;
      fn.oneTime = parsedExpression.oneTime;
      fn.constant = parsedExpression.constant;

      // Treat the interceptor like filters.
      // If it is not $stateful then only watch its inputs.
      // If the expression itself has no inputs then use the full expression as an input.
      if (!interceptorFn.$stateful) {
        useInputs = !parsedExpression.inputs;
        fn.inputs = parsedExpression.inputs ? parsedExpression.inputs : [parsedExpression];

        if (!interceptorFn.$$pure) {
          fn.inputs = fn.inputs.map(function(e) {
              // Remove the isPure flag of inputs when it is not absolute because they are now wrapped in a
              // non-pure interceptor function.
              if (e.isPure === PURITY_RELATIVE) {
                return function depurifier(s) { return e(s); };
              }
              return e;
            });
        }
      }

      return addWatchDelegate(fn);
    }
  }];
}

/**
 * @ngdoc service
 * @name $q
 * @requires $rootScope
 *
 * @description
 * A service that helps you run functions asynchronously, and use their return values (or exceptions)
 * when they are done processing.
 *
 * This is a [Promises/A+](https://promisesaplus.com/)-compliant implementation of promises/deferred
 * objects inspired by [Kris Kowal's Q](https://github.com/kriskowal/q).
 *
 * $q can be used in two fashions --- one which is more similar to Kris Kowal's Q or jQuery's Deferred
 * implementations, and the other which resembles ES6 (ES2015) promises to some degree.
 *
 * ## $q constructor
 *
 * The streamlined ES6 style promise is essentially just using $q as a constructor which takes a `resolver`
 * function as the first argument. This is similar to the native Promise implementation from ES6,
 * see [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise).
 *
 * While the constructor-style use is supported, not all of the supporting methods from ES6 promises are
 * available yet.
 *
 * It can be used like so:
 *
 * ```js
 *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
 *   // are available in the current lexical scope (they could have been injected or passed in).
 *
 *   function asyncGreet(name) {
 *     // perform some asynchronous operation, resolve or reject the promise when appropriate.
 *     return $q(function(resolve, reject) {
 *       setTimeout(function() {
 *         if (okToGreet(name)) {
 *           resolve('Hello, ' + name + '!');
 *         } else {
 *           reject('Greeting ' + name + ' is not allowed.');
 *         }
 *       }, 1000);
 *     });
 *   }
 *
 *   var promise = asyncGreet('Robin Hood');
 *   promise.then(function(greeting) {
 *     alert('Success: ' + greeting);
 *   }, function(reason) {
 *     alert('Failed: ' + reason);
 *   });
 * ```
 *
 * Note: progress/notify callbacks are not currently supported via the ES6-style interface.
 *
 * Note: unlike ES6 behavior, an exception thrown in the constructor function will NOT implicitly reject the promise.
 *
 * However, the more traditional CommonJS-style usage is still available, and documented below.
 *
 * [The CommonJS Promise proposal](http://wiki.commonjs.org/wiki/Promises) describes a promise as an
 * interface for interacting with an object that represents the result of an action that is
 * performed asynchronously, and may or may not be finished at any given point in time.
 *
 * From the perspective of dealing with error handling, deferred and promise APIs are to
 * asynchronous programming what `try`, `catch` and `throw` keywords are to synchronous programming.
 *
 * ```js
 *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
 *   // are available in the current lexical scope (they could have been injected or passed in).
 *
 *   function asyncGreet(name) {
 *     var deferred = $q.defer();
 *
 *     setTimeout(function() {
 *       deferred.notify('About to greet ' + name + '.');
 *
 *       if (okToGreet(name)) {
 *         deferred.resolve('Hello, ' + name + '!');
 *       } else {
 *         deferred.reject('Greeting ' + name + ' is not allowed.');
 *       }
 *     }, 1000);
 *
 *     return deferred.promise;
 *   }
 *
 *   var promise = asyncGreet('Robin Hood');
 *   promise.then(function(greeting) {
 *     alert('Success: ' + greeting);
 *   }, function(reason) {
 *     alert('Failed: ' + reason);
 *   }, function(update) {
 *     alert('Got notification: ' + update);
 *   });
 * ```
 *
 * At first it might not be obvious why this extra complexity is worth the trouble. The payoff
 * comes in the way of guarantees that promise and deferred APIs make, see
 * https://github.com/kriskowal/uncommonjs/blob/master/promises/specification.md.
 *
 * Additionally the promise api allows for composition that is very hard to do with the
 * traditional callback ([CPS](http://en.wikipedia.org/wiki/Continuation-passing_style)) approach.
 * For more on this please see the [Q documentation](https://github.com/kriskowal/q) especially the
 * section on serial or parallel joining of promises.
 *
 * ## The Deferred API
 *
 * A new instance of deferred is constructed by calling `$q.defer()`.
 *
 * The purpose of the deferred object is to expose the associated Promise instance as well as APIs
 * that can be used for signaling the successful or unsuccessful completion, as well as the status
 * of the task.
 *
 * **Methods**
 *
 * - `resolve(value)` – resolves the derived promise with the `value`. If the value is a rejection
 *   constructed via `$q.reject`, the promise will be rejected instead.
 * - `reject(reason)` – rejects the derived promise with the `reason`. This is equivalent to
 *   resolving it with a rejection constructed via `$q.reject`.
 * - `notify(value)` - provides updates on the status of the promise's execution. This may be called
 *   multiple times before the promise is either resolved or rejected.
 *
 * **Properties**
 *
 * - promise – `{Promise}` – promise object associated with this deferred.
 *
 *
 * ## The Promise API
 *
 * A new promise instance is created when a deferred instance is created and can be retrieved by
 * calling `deferred.promise`.
 *
 * The purpose of the promise object is to allow for interested parties to get access to the result
 * of the deferred task when it completes.
 *
 * **Methods**
 *
 * - `then(successCallback, [errorCallback], [notifyCallback])` – regardless of when the promise was or
 *   will be resolved or rejected, `then` calls one of the success or error callbacks asynchronously
 *   as soon as the result is available. The callbacks are called with a single argument: the result
 *   or rejection reason. Additionally, the notify callback may be called zero or more times to
 *   provide a progress indication, before the promise is resolved or rejected.
 *
 *   This method *returns a new promise* which is resolved or rejected via the return value of the
 *   `successCallback`, `errorCallback` (unless that value is a promise, in which case it is resolved
 *   with the value which is resolved in that promise using
 *   [promise chaining](http://www.html5rocks.com/en/tutorials/es6/promises/#toc-promises-queues)).
 *   It also notifies via the return value of the `notifyCallback` method. The promise cannot be
 *   resolved or rejected from the notifyCallback method. The errorCallback and notifyCallback
 *   arguments are optional.
 *
 * - `catch(errorCallback)` – shorthand for `promise.then(null, errorCallback)`
 *
 * - `finally(callback, notifyCallback)` – allows you to observe either the fulfillment or rejection of a promise,
 *   but to do so without modifying the final value. This is useful to release resources or do some
 *   clean-up that needs to be done whether the promise was rejected or resolved. See the [full
 *   specification](https://github.com/kriskowal/q/wiki/API-Reference#promisefinallycallback) for
 *   more information.
 *
 * ## Chaining promises
 *
 * Because calling the `then` method of a promise returns a new derived promise, it is easily
 * possible to create a chain of promises:
 *
 * ```js
 *   promiseB = promiseA.then(function(result) {
 *     return result + 1;
 *   });
 *
 *   // promiseB will be resolved immediately after promiseA is resolved and its value
 *   // will be the result of promiseA incremented by 1
 * ```
 *
 * It is possible to create chains of any length and since a promise can be resolved with another
 * promise (which will defer its resolution further), it is possible to pause/defer resolution of
 * the promises at any point in the chain. This makes it possible to implement powerful APIs like
 * $http's response interceptors.
 *
 *
 * ## Differences between Kris Kowal's Q and $q
 *
 *  There are two main differences:
 *
 * - $q is integrated with the {@link ng.$rootScope.Scope} Scope model observation
 *   mechanism in AngularJS, which means faster propagation of resolution or rejection into your
 *   models and avoiding unnecessary browser repaints, which would result in flickering UI.
 * - Q has many more features than $q, but that comes at a cost of bytes. $q is tiny, but contains
 *   all the important functionality needed for common async tasks.
 *
 * ## Testing
 *
 *  ```js
 *    it('should simulate promise', inject(function($q, $rootScope) {
 *      var deferred = $q.defer();
 *      var promise = deferred.promise;
 *      var resolvedValue;
 *
 *      promise.then(function(value) { resolvedValue = value; });
 *      expect(resolvedValue).toBeUndefined();
 *
 *      // Simulate resolving of promise
 *      deferred.resolve(123);
 *      // Note that the 'then' function does not get called synchronously.
 *      // This is because we want the promise API to always be async, whether or not
 *      // it got called synchronously or asynchronously.
 *      expect(resolvedValue).toBeUndefined();
 *
 *      // Propagate promise resolution to 'then' functions using $apply().
 *      $rootScope.$apply();
 *      expect(resolvedValue).toEqual(123);
 *    }));
 *  ```
 *
 * @param {function(function, function)} resolver Function which is responsible for resolving or
 *   rejecting the newly created promise. The first parameter is a function which resolves the
 *   promise, the second parameter is a function which rejects the promise.
 *
 * @returns {Promise} The newly created promise.
 */
/**
 * @ngdoc provider
 * @name $qProvider
 * @this
 *
 * @description
 */
function $QProvider() {
  var errorOnUnhandledRejections = true;
  this.$get = ['$rootScope', '$exceptionHandler', function($rootScope, $exceptionHandler) {
    return qFactory(function(callback) {
      $rootScope.$evalAsync(callback);
    }, $exceptionHandler, errorOnUnhandledRejections);
  }];

  /**
   * @ngdoc method
   * @name $qProvider#errorOnUnhandledRejections
   * @kind function
   *
   * @description
   * Retrieves or overrides whether to generate an error when a rejected promise is not handled.
   * This feature is enabled by default.
   *
   * @param {boolean=} value Whether to generate an error when a rejected promise is not handled.
   * @returns {boolean|ng.$qProvider} Current value when called without a new value or self for
   *    chaining otherwise.
   */
  this.errorOnUnhandledRejections = function(value) {
    if (isDefined(value)) {
      errorOnUnhandledRejections = value;
      return this;
    } else {
      return errorOnUnhandledRejections;
    }
  };
}

/** @this */
function $$QProvider() {
  var errorOnUnhandledRejections = true;
  this.$get = ['$browser', '$exceptionHandler', function($browser, $exceptionHandler) {
    return qFactory(function(callback) {
      $browser.defer(callback);
    }, $exceptionHandler, errorOnUnhandledRejections);
  }];

  this.errorOnUnhandledRejections = function(value) {
    if (isDefined(value)) {
      errorOnUnhandledRejections = value;
      return this;
    } else {
      return errorOnUnhandledRejections;
    }
  };
}

/**
 * Constructs a promise manager.
 *
 * @param {function(function)} nextTick Function for executing functions in the next turn.
 * @param {function(...*)} exceptionHandler Function into which unexpected exceptions are passed for
 *     debugging purposes.
 * @param {boolean=} errorOnUnhandledRejections Whether an error should be generated on unhandled
 *     promises rejections.
 * @returns {object} Promise manager.
 */
function qFactory(nextTick, exceptionHandler, errorOnUnhandledRejections) {
  var $qMinErr = minErr('$q', TypeError);
  var queueSize = 0;
  var checkQueue = [];

  /**
   * @ngdoc method
   * @name ng.$q#defer
   * @kind function
   *
   * @description
   * Creates a `Deferred` object which represents a task which will finish in the future.
   *
   * @returns {Deferred} Returns a new instance of deferred.
   */
  function defer() {
    return new Deferred();
  }

  function Deferred() {
    var promise = this.promise = new Promise();
    //Non prototype methods necessary to support unbound execution :/
    this.resolve = function(val) { resolvePromise(promise, val); };
    this.reject = function(reason) { rejectPromise(promise, reason); };
    this.notify = function(progress) { notifyPromise(promise, progress); };
  }


  function Promise() {
    this.$$state = { status: 0 };
  }

  extend(Promise.prototype, {
    then: function(onFulfilled, onRejected, progressBack) {
      if (isUndefined(onFulfilled) && isUndefined(onRejected) && isUndefined(progressBack)) {
        return this;
      }
      var result = new Promise();

      this.$$state.pending = this.$$state.pending || [];
      this.$$state.pending.push([result, onFulfilled, onRejected, progressBack]);
      if (this.$$state.status > 0) scheduleProcessQueue(this.$$state);

      return result;
    },

    'catch': function(callback) {
      return this.then(null, callback);
    },

    'finally': function(callback, progressBack) {
      return this.then(function(value) {
        return handleCallback(value, resolve, callback);
      }, function(error) {
        return handleCallback(error, reject, callback);
      }, progressBack);
    }
  });

  function processQueue(state) {
    var fn, promise, pending;

    pending = state.pending;
    state.processScheduled = false;
    state.pending = undefined;
    try {
      for (var i = 0, ii = pending.length; i < ii; ++i) {
        markQStateExceptionHandled(state);
        promise = pending[i][0];
        fn = pending[i][state.status];
        try {
          if (isFunction(fn)) {
            resolvePromise(promise, fn(state.value));
          } else if (state.status === 1) {
            resolvePromise(promise, state.value);
          } else {
            rejectPromise(promise, state.value);
          }
        } catch (e) {
          rejectPromise(promise, e);
          // This error is explicitly marked for being passed to the $exceptionHandler
          if (e && e.$$passToExceptionHandler === true) {
            exceptionHandler(e);
          }
        }
      }
    } finally {
      --queueSize;
      if (errorOnUnhandledRejections && queueSize === 0) {
        nextTick(processChecks);
      }
    }
  }

  function processChecks() {
    // eslint-disable-next-line no-unmodified-loop-condition
    while (!queueSize && checkQueue.length) {
      var toCheck = checkQueue.shift();
      if (!isStateExceptionHandled(toCheck)) {
        markQStateExceptionHandled(toCheck);
        var errorMessage = 'Possibly unhandled rejection: ' + toDebugString(toCheck.value);
        if (isError(toCheck.value)) {
          exceptionHandler(toCheck.value, errorMessage);
        } else {
          exceptionHandler(errorMessage);
        }
      }
    }
  }

  function scheduleProcessQueue(state) {
    if (errorOnUnhandledRejections && !state.pending && state.status === 2 && !isStateExceptionHandled(state)) {
      if (queueSize === 0 && checkQueue.length === 0) {
        nextTick(processChecks);
      }
      checkQueue.push(state);
    }
    if (state.processScheduled || !state.pending) return;
    state.processScheduled = true;
    ++queueSize;
    nextTick(function() { processQueue(state); });
  }

  function resolvePromise(promise, val) {
    if (promise.$$state.status) return;
    if (val === promise) {
      $$reject(promise, $qMinErr(
        'qcycle',
        'Expected promise to be resolved with value other than itself \'{0}\'',
        val));
    } else {
      $$resolve(promise, val);
    }

  }

  function $$resolve(promise, val) {
    var then;
    var done = false;
    try {
      if (isObject(val) || isFunction(val)) then = val.then;
      if (isFunction(then)) {
        promise.$$state.status = -1;
        then.call(val, doResolve, doReject, doNotify);
      } else {
        promise.$$state.value = val;
        promise.$$state.status = 1;
        scheduleProcessQueue(promise.$$state);
      }
    } catch (e) {
      doReject(e);
    }

    function doResolve(val) {
      if (done) return;
      done = true;
      $$resolve(promise, val);
    }
    function doReject(val) {
      if (done) return;
      done = true;
      $$reject(promise, val);
    }
    function doNotify(progress) {
      notifyPromise(promise, progress);
    }
  }

  function rejectPromise(promise, reason) {
    if (promise.$$state.status) return;
    $$reject(promise, reason);
  }

  function $$reject(promise, reason) {
    promise.$$state.value = reason;
    promise.$$state.status = 2;
    scheduleProcessQueue(promise.$$state);
  }

  function notifyPromise(promise, progress) {
    var callbacks = promise.$$state.pending;

    if ((promise.$$state.status <= 0) && callbacks && callbacks.length) {
      nextTick(function() {
        var callback, result;
        for (var i = 0, ii = callbacks.length; i < ii; i++) {
          result = callbacks[i][0];
          callback = callbacks[i][3];
          try {
            notifyPromise(result, isFunction(callback) ? callback(progress) : progress);
          } catch (e) {
            exceptionHandler(e);
          }
        }
      });
    }
  }

  /**
   * @ngdoc method
   * @name $q#reject
   * @kind function
   *
   * @description
   * Creates a promise that is resolved as rejected with the specified `reason`. This api should be
   * used to forward rejection in a chain of promises. If you are dealing with the last promise in
   * a promise chain, you don't need to worry about it.
   *
   * When comparing deferreds/promises to the familiar behavior of try/catch/throw, think of
   * `reject` as the `throw` keyword in JavaScript. This also means that if you "catch" an error via
   * a promise error callback and you want to forward the error to the promise derived from the
   * current promise, you have to "rethrow" the error by returning a rejection constructed via
   * `reject`.
   *
   * ```js
   *   promiseB = promiseA.then(function(result) {
   *     // success: do something and resolve promiseB
   *     //          with the old or a new result
   *     return result;
   *   }, function(reason) {
   *     // error: handle the error if possible and
   *     //        resolve promiseB with newPromiseOrValue,
   *     //        otherwise forward the rejection to promiseB
   *     if (canHandle(reason)) {
   *      // handle the error and recover
   *      return newPromiseOrValue;
   *     }
   *     return $q.reject(reason);
   *   });
   * ```
   *
   * @param {*} reason Constant, message, exception or an object representing the rejection reason.
   * @returns {Promise} Returns a promise that was already resolved as rejected with the `reason`.
   */
  function reject(reason) {
    var result = new Promise();
    rejectPromise(result, reason);
    return result;
  }

  function handleCallback(value, resolver, callback) {
    var callbackOutput = null;
    try {
      if (isFunction(callback)) callbackOutput = callback();
    } catch (e) {
      return reject(e);
    }
    if (isPromiseLike(callbackOutput)) {
      return callbackOutput.then(function() {
        return resolver(value);
      }, reject);
    } else {
      return resolver(value);
    }
  }

  /**
   * @ngdoc method
   * @name $q#when
   * @kind function
   *
   * @description
   * Wraps an object that might be a value or a (3rd party) then-able promise into a $q promise.
   * This is useful when you are dealing with an object that might or might not be a promise, or if
   * the promise comes from a source that can't be trusted.
   *
   * @param {*} value Value or a promise
   * @param {Function=} successCallback
   * @param {Function=} errorCallback
   * @param {Function=} progressCallback
   * @returns {Promise} Returns a promise of the passed value or promise
   */


  function when(value, callback, errback, progressBack) {
    var result = new Promise();
    resolvePromise(result, value);
    return result.then(callback, errback, progressBack);
  }

  /**
   * @ngdoc method
   * @name $q#resolve
   * @kind function
   *
   * @description
   * Alias of {@link ng.$q#when when} to maintain naming consistency with ES6.
   *
   * @param {*} value Value or a promise
   * @param {Function=} successCallback
   * @param {Function=} errorCallback
   * @param {Function=} progressCallback
   * @returns {Promise} Returns a promise of the passed value or promise
   */
  var resolve = when;

  /**
   * @ngdoc method
   * @name $q#all
   * @kind function
   *
   * @description
   * Combines multiple promises into a single promise that is resolved when all of the input
   * promises are resolved.
   *
   * @param {Array.<Promise>|Object.<Promise>} promises An array or hash of promises.
   * @returns {Promise} Returns a single promise that will be resolved with an array/hash of values,
   *   each value corresponding to the promise at the same index/key in the `promises` array/hash.
   *   If any of the promises is resolved with a rejection, this resulting promise will be rejected
   *   with the same rejection value.
   */

  function all(promises) {
    var result = new Promise(),
        counter = 0,
        results = isArray(promises) ? [] : {};

    forEach(promises, function(promise, key) {
      counter++;
      when(promise).then(function(value) {
        results[key] = value;
        if (!(--counter)) resolvePromise(result, results);
      }, function(reason) {
        rejectPromise(result, reason);
      });
    });

    if (counter === 0) {
      resolvePromise(result, results);
    }

    return result;
  }

  /**
   * @ngdoc method
   * @name $q#race
   * @kind function
   *
   * @description
   * Returns a promise that resolves or rejects as soon as one of those promises
   * resolves or rejects, with the value or reason from that promise.
   *
   * @param {Array.<Promise>|Object.<Promise>} promises An array or hash of promises.
   * @returns {Promise} a promise that resolves or rejects as soon as one of the `promises`
   * resolves or rejects, with the value or reason from that promise.
   */

  function race(promises) {
    var deferred = defer();

    forEach(promises, function(promise) {
      when(promise).then(deferred.resolve, deferred.reject);
    });

    return deferred.promise;
  }

  function $Q(resolver) {
    if (!isFunction(resolver)) {
      throw $qMinErr('norslvr', 'Expected resolverFn, got \'{0}\'', resolver);
    }

    var promise = new Promise();

    function resolveFn(value) {
      resolvePromise(promise, value);
    }

    function rejectFn(reason) {
      rejectPromise(promise, reason);
    }

    resolver(resolveFn, rejectFn);

    return promise;
  }

  // Let's make the instanceof operator work for promises, so that
  // `new $q(fn) instanceof $q` would evaluate to true.
  $Q.prototype = Promise.prototype;

  $Q.defer = defer;
  $Q.reject = reject;
  $Q.when = when;
  $Q.resolve = resolve;
  $Q.all = all;
  $Q.race = race;

  return $Q;
}

function isStateExceptionHandled(state) {
  return !!state.pur;
}
function markQStateExceptionHandled(state) {
  state.pur = true;
}
function markQExceptionHandled(q) {
  // Built-in `$q` promises will always have a `$$state` property. This check is to allow
  // overwriting `$q` with a different promise library (e.g. Bluebird + angular-bluebird-promises).
  // (Currently, this is the only method that might be called with a promise, even if it is not
  // created by the built-in `$q`.)
  if (q.$$state) {
    markQStateExceptionHandled(q.$$state);
  }
}

/** @this */
function $$RAFProvider() { //rAF
  this.$get = ['$window', '$timeout', function($window, $timeout) {
    var requestAnimationFrame = $window.requestAnimationFrame ||
                                $window.webkitRequestAnimationFrame;

    var cancelAnimationFrame = $window.cancelAnimationFrame ||
                               $window.webkitCancelAnimationFrame ||
                               $window.webkitCancelRequestAnimationFrame;

    var rafSupported = !!requestAnimationFrame;
    var raf = rafSupported
      ? function(fn) {
          var id = requestAnimationFrame(fn);
          return function() {
            cancelAnimationFrame(id);
          };
        }
      : function(fn) {
          var timer = $timeout(fn, 16.66, false); // 1000 / 60 = 16.666
          return function() {
            $timeout.cancel(timer);
          };
        };

    raf.supported = rafSupported;

    return raf;
  }];
}

/**
 * DESIGN NOTES
 *
 * The design decisions behind the scope are heavily favored for speed and memory consumption.
 *
 * The typical use of scope is to watch the expressions, which most of the time return the same
 * value as last time so we optimize the operation.
 *
 * Closures construction is expensive in terms of speed as well as memory:
 *   - No closures, instead use prototypical inheritance for API
 *   - Internal state needs to be stored on scope directly, which means that private state is
 *     exposed as $$____ properties
 *
 * Loop operations are optimized by using while(count--) { ... }
 *   - This means that in order to keep the same order of execution as addition we have to add
 *     items to the array at the beginning (unshift) instead of at the end (push)
 *
 * Child scopes are created and removed often
 *   - Using an array would be slow since inserts in the middle are expensive; so we use linked lists
 *
 * There are fewer watches than observers. This is why you don't want the observer to be implemented
 * in the same way as watch. Watch requires return of the initialization function which is expensive
 * to construct.
 */


/**
 * @ngdoc provider
 * @name $rootScopeProvider
 * @description
 *
 * Provider for the $rootScope service.
 */

/**
 * @ngdoc method
 * @name $rootScopeProvider#digestTtl
 * @description
 *
 * Sets the number of `$digest` iterations the scope should attempt to execute before giving up and
 * assuming that the model is unstable.
 *
 * The current default is 10 iterations.
 *
 * In complex applications it's possible that the dependencies between `$watch`s will result in
 * several digest iterations. However if an application needs more than the default 10 digest
 * iterations for its model to stabilize then you should investigate what is causing the model to
 * continuously change during the digest.
 *
 * Increasing the TTL could have performance implications, so you should not change it without
 * proper justification.
 *
 * @param {number} limit The number of digest iterations.
 */


/**
 * @ngdoc service
 * @name $rootScope
 * @this
 *
 * @description
 *
 * Every application has a single root {@link ng.$rootScope.Scope scope}.
 * All other scopes are descendant scopes of the root scope. Scopes provide separation
 * between the model and the view, via a mechanism for watching the model for changes.
 * They also provide event emission/broadcast and subscription facility. See the
 * {@link guide/scope developer guide on scopes}.
 */
function $RootScopeProvider() {
  var TTL = 10;
  var $rootScopeMinErr = minErr('$rootScope');
  var lastDirtyWatch = null;
  var applyAsyncId = null;

  this.digestTtl = function(value) {
    if (arguments.length) {
      TTL = value;
    }
    return TTL;
  };

  function createChildScopeClass(parent) {
    function ChildScope() {
      this.$$watchers = this.$$nextSibling =
          this.$$childHead = this.$$childTail = null;
      this.$$listeners = {};
      this.$$listenerCount = {};
      this.$$watchersCount = 0;
      this.$id = nextUid();
      this.$$ChildScope = null;
      this.$$suspended = false;
    }
    ChildScope.prototype = parent;
    return ChildScope;
  }

  this.$get = ['$exceptionHandler', '$parse', '$browser',
      function($exceptionHandler, $parse, $browser) {

    function destroyChildScope($event) {
        $event.currentScope.$$destroyed = true;
    }

    function cleanUpScope($scope) {

      // Support: IE 9 only
      if (msie === 9) {
        // There is a memory leak in IE9 if all child scopes are not disconnected
        // completely when a scope is destroyed. So this code will recurse up through
        // all this scopes children
        //
        // See issue https://github.com/angular/angular.js/issues/10706
        if ($scope.$$childHead) {
          cleanUpScope($scope.$$childHead);
        }
        if ($scope.$$nextSibling) {
          cleanUpScope($scope.$$nextSibling);
        }
      }

      // The code below works around IE9 and V8's memory leaks
      //
      // See:
      // - https://code.google.com/p/v8/issues/detail?id=2073#c26
      // - https://github.com/angular/angular.js/issues/6794#issuecomment-38648909
      // - https://github.com/angular/angular.js/issues/1313#issuecomment-10378451

      $scope.$parent = $scope.$$nextSibling = $scope.$$prevSibling = $scope.$$childHead =
          $scope.$$childTail = $scope.$root = $scope.$$watchers = null;
    }

    /**
     * @ngdoc type
     * @name $rootScope.Scope
     *
     * @description
     * A root scope can be retrieved using the {@link ng.$rootScope $rootScope} key from the
     * {@link auto.$injector $injector}. Child scopes are created using the
     * {@link ng.$rootScope.Scope#$new $new()} method. (Most scopes are created automatically when
     * compiled HTML template is executed.) See also the {@link guide/scope Scopes guide} for
     * an in-depth introduction and usage examples.
     *
     *
     * ## Inheritance
     * A scope can inherit from a parent scope, as in this example:
     * ```js
         var parent = $rootScope;
         var child = parent.$new();

         parent.salutation = "Hello";
         expect(child.salutation).toEqual('Hello');

         child.salutation = "Welcome";
         expect(child.salutation).toEqual('Welcome');
         expect(parent.salutation).toEqual('Hello');
     * ```
     *
     * When interacting with `Scope` in tests, additional helper methods are available on the
     * instances of `Scope` type. See {@link ngMock.$rootScope.Scope ngMock Scope} for additional
     * details.
     *
     *
     * @param {Object.<string, function()>=} providers Map of service factory which need to be
     *                                       provided for the current scope. Defaults to {@link ng}.
     * @param {Object.<string, *>=} instanceCache Provides pre-instantiated services which should
     *                              append/override services provided by `providers`. This is handy
     *                              when unit-testing and having the need to override a default
     *                              service.
     * @returns {Object} Newly created scope.
     *
     */
    function Scope() {
      this.$id = nextUid();
      this.$$phase = this.$parent = this.$$watchers =
                     this.$$nextSibling = this.$$prevSibling =
                     this.$$childHead = this.$$childTail = null;
      this.$root = this;
      this.$$destroyed = false;
      this.$$suspended = false;
      this.$$listeners = {};
      this.$$listenerCount = {};
      this.$$watchersCount = 0;
      this.$$isolateBindings = null;
    }

    /**
     * @ngdoc property
     * @name $rootScope.Scope#$id
     *
     * @description
     * Unique scope ID (monotonically increasing) useful for debugging.
     */

     /**
      * @ngdoc property
      * @name $rootScope.Scope#$parent
      *
      * @description
      * Reference to the parent scope.
      */

      /**
       * @ngdoc property
       * @name $rootScope.Scope#$root
       *
       * @description
       * Reference to the root scope.
       */

    Scope.prototype = {
      constructor: Scope,
      /**
       * @ngdoc method
       * @name $rootScope.Scope#$new
       * @kind function
       *
       * @description
       * Creates a new child {@link ng.$rootScope.Scope scope}.
       *
       * The parent scope will propagate the {@link ng.$rootScope.Scope#$digest $digest()} event.
       * The scope can be removed from the scope hierarchy using {@link ng.$rootScope.Scope#$destroy $destroy()}.
       *
       * {@link ng.$rootScope.Scope#$destroy $destroy()} must be called on a scope when it is
       * desired for the scope and its child scopes to be permanently detached from the parent and
       * thus stop participating in model change detection and listener notification by invoking.
       *
       * @param {boolean} isolate If true, then the scope does not prototypically inherit from the
       *         parent scope. The scope is isolated, as it can not see parent scope properties.
       *         When creating widgets, it is useful for the widget to not accidentally read parent
       *         state.
       *
       * @param {Scope} [parent=this] The {@link ng.$rootScope.Scope `Scope`} that will be the `$parent`
       *                              of the newly created scope. Defaults to `this` scope if not provided.
       *                              This is used when creating a transclude scope to correctly place it
       *                              in the scope hierarchy while maintaining the correct prototypical
       *                              inheritance.
       *
       * @returns {Object} The newly created child scope.
       *
       */
      $new: function(isolate, parent) {
        var child;

        parent = parent || this;

        if (isolate) {
          child = new Scope();
          child.$root = this.$root;
        } else {
          // Only create a child scope class if somebody asks for one,
          // but cache it to allow the VM to optimize lookups.
          if (!this.$$ChildScope) {
            this.$$ChildScope = createChildScopeClass(this);
          }
          child = new this.$$ChildScope();
        }
        child.$parent = parent;
        child.$$prevSibling = parent.$$childTail;
        if (parent.$$childHead) {
          parent.$$childTail.$$nextSibling = child;
          parent.$$childTail = child;
        } else {
          parent.$$childHead = parent.$$childTail = child;
        }

        // When the new scope is not isolated or we inherit from `this`, and
        // the parent scope is destroyed, the property `$$destroyed` is inherited
        // prototypically. In all other cases, this property needs to be set
        // when the parent scope is destroyed.
        // The listener needs to be added after the parent is set
        if (isolate || parent !== this) child.$on('$destroy', destroyChildScope);

        return child;
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$watch
       * @kind function
       *
       * @description
       * Registers a `listener` callback to be executed whenever the `watchExpression` changes.
       *
       * - The `watchExpression` is called on every call to {@link ng.$rootScope.Scope#$digest
       *   $digest()} and should return the value that will be watched. (`watchExpression` should not change
       *   its value when executed multiple times with the same input because it may be executed multiple
       *   times by {@link ng.$rootScope.Scope#$digest $digest()}. That is, `watchExpression` should be
       *   [idempotent](http://en.wikipedia.org/wiki/Idempotence).)
       * - The `listener` is called only when the value from the current `watchExpression` and the
       *   previous call to `watchExpression` are not equal (with the exception of the initial run,
       *   see below). Inequality is determined according to reference inequality,
       *   [strict comparison](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Comparison_Operators)
       *    via the `!==` Javascript operator, unless `objectEquality == true`
       *   (see next point)
       * - When `objectEquality == true`, inequality of the `watchExpression` is determined
       *   according to the {@link angular.equals} function. To save the value of the object for
       *   later comparison, the {@link angular.copy} function is used. This therefore means that
       *   watching complex objects will have adverse memory and performance implications.
       * - This should not be used to watch for changes in objects that are (or contain)
       *   [File](https://developer.mozilla.org/docs/Web/API/File) objects due to limitations with {@link angular.copy `angular.copy`}.
       * - The watch `listener` may change the model, which may trigger other `listener`s to fire.
       *   This is achieved by rerunning the watchers until no changes are detected. The rerun
       *   iteration limit is 10 to prevent an infinite loop deadlock.
       *
       *
       * If you want to be notified whenever {@link ng.$rootScope.Scope#$digest $digest} is called,
       * you can register a `watchExpression` function with no `listener`. (Be prepared for
       * multiple calls to your `watchExpression` because it will execute multiple times in a
       * single {@link ng.$rootScope.Scope#$digest $digest} cycle if a change is detected.)
       *
       * After a watcher is registered with the scope, the `listener` fn is called asynchronously
       * (via {@link ng.$rootScope.Scope#$evalAsync $evalAsync}) to initialize the
       * watcher. In rare cases, this is undesirable because the listener is called when the result
       * of `watchExpression` didn't change. To detect this scenario within the `listener` fn, you
       * can compare the `newVal` and `oldVal`. If these two values are identical (`===`) then the
       * listener was called due to initialization.
       *
       *
       *
       * @example
       * ```js
           // let's assume that scope was dependency injected as the $rootScope
           var scope = $rootScope;
           scope.name = 'misko';
           scope.counter = 0;

           expect(scope.counter).toEqual(0);
           scope.$watch('name', function(newValue, oldValue) {
             scope.counter = scope.counter + 1;
           });
           expect(scope.counter).toEqual(0);

           scope.$digest();
           // the listener is always called during the first $digest loop after it was registered
           expect(scope.counter).toEqual(1);

           scope.$digest();
           // but now it will not be called unless the value changes
           expect(scope.counter).toEqual(1);

           scope.name = 'adam';
           scope.$digest();
           expect(scope.counter).toEqual(2);



           // Using a function as a watchExpression
           var food;
           scope.foodCounter = 0;
           expect(scope.foodCounter).toEqual(0);
           scope.$watch(
             // This function returns the value being watched. It is called for each turn of the $digest loop
             function() { return food; },
             // This is the change listener, called when the value returned from the above function changes
             function(newValue, oldValue) {
               if ( newValue !== oldValue ) {
                 // Only increment the counter if the value changed
                 scope.foodCounter = scope.foodCounter + 1;
               }
             }
           );
           // No digest has been run so the counter will be zero
           expect(scope.foodCounter).toEqual(0);

           // Run the digest but since food has not changed count will still be zero
           scope.$digest();
           expect(scope.foodCounter).toEqual(0);

           // Update food and run digest.  Now the counter will increment
           food = 'cheeseburger';
           scope.$digest();
           expect(scope.foodCounter).toEqual(1);

       * ```
       *
       *
       *
       * @param {(function()|string)} watchExpression Expression that is evaluated on each
       *    {@link ng.$rootScope.Scope#$digest $digest} cycle. A change in the return value triggers
       *    a call to the `listener`.
       *
       *    - `string`: Evaluated as {@link guide/expression expression}
       *    - `function(scope)`: called with current `scope` as a parameter.
       * @param {function(newVal, oldVal, scope)} listener Callback called whenever the value
       *    of `watchExpression` changes.
       *
       *    - `newVal` contains the current value of the `watchExpression`
       *    - `oldVal` contains the previous value of the `watchExpression`
       *    - `scope` refers to the current scope
       * @param {boolean=} [objectEquality=false] Compare for object equality using {@link angular.equals} instead of
       *     comparing for reference equality.
       * @returns {function()} Returns a deregistration function for this listener.
       */
      $watch: function(watchExp, listener, objectEquality, prettyPrintExpression) {
        var get = $parse(watchExp);
        var fn = isFunction(listener) ? listener : noop;

        if (get.$$watchDelegate) {
          return get.$$watchDelegate(this, fn, objectEquality, get, watchExp);
        }
        var scope = this,
            array = scope.$$watchers,
            watcher = {
              fn: fn,
              last: initWatchVal,
              get: get,
              exp: prettyPrintExpression || watchExp,
              eq: !!objectEquality
            };

        lastDirtyWatch = null;

        if (!array) {
          array = scope.$$watchers = [];
          array.$$digestWatchIndex = -1;
        }
        // we use unshift since we use a while loop in $digest for speed.
        // the while loop reads in reverse order.
        array.unshift(watcher);
        array.$$digestWatchIndex++;
        incrementWatchersCount(this, 1);

        return function deregisterWatch() {
          var index = arrayRemove(array, watcher);
          if (index >= 0) {
            incrementWatchersCount(scope, -1);
            if (index < array.$$digestWatchIndex) {
              array.$$digestWatchIndex--;
            }
          }
          lastDirtyWatch = null;
        };
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$watchGroup
       * @kind function
       *
       * @description
       * A variant of {@link ng.$rootScope.Scope#$watch $watch()} where it watches an array of `watchExpressions`.
       * If any one expression in the collection changes the `listener` is executed.
       *
       * - The items in the `watchExpressions` array are observed via the standard `$watch` operation. Their return
       *   values are examined for changes on every call to `$digest`.
       * - The `listener` is called whenever any expression in the `watchExpressions` array changes.
       *
       * @param {Array.<string|Function(scope)>} watchExpressions Array of expressions that will be individually
       * watched using {@link ng.$rootScope.Scope#$watch $watch()}
       *
       * @param {function(newValues, oldValues, scope)} listener Callback called whenever the return value of any
       *    expression in `watchExpressions` changes
       *    The `newValues` array contains the current values of the `watchExpressions`, with the indexes matching
       *    those of `watchExpression`
       *    and the `oldValues` array contains the previous values of the `watchExpressions`, with the indexes matching
       *    those of `watchExpression`
       *    The `scope` refers to the current scope.
       * @returns {function()} Returns a de-registration function for all listeners.
       */
      $watchGroup: function(watchExpressions, listener) {
        var oldValues = new Array(watchExpressions.length);
        var newValues = new Array(watchExpressions.length);
        var deregisterFns = [];
        var self = this;
        var changeReactionScheduled = false;
        var firstRun = true;

        if (!watchExpressions.length) {
          // No expressions means we call the listener ASAP
          var shouldCall = true;
          self.$evalAsync(function() {
            if (shouldCall) listener(newValues, newValues, self);
          });
          return function deregisterWatchGroup() {
            shouldCall = false;
          };
        }

        if (watchExpressions.length === 1) {
          // Special case size of one
          return this.$watch(watchExpressions[0], function watchGroupAction(value, oldValue, scope) {
            newValues[0] = value;
            oldValues[0] = oldValue;
            listener(newValues, (value === oldValue) ? newValues : oldValues, scope);
          });
        }

        forEach(watchExpressions, function(expr, i) {
          var unwatchFn = self.$watch(expr, function watchGroupSubAction(value) {
            newValues[i] = value;
            if (!changeReactionScheduled) {
              changeReactionScheduled = true;
              self.$evalAsync(watchGroupAction);
            }
          });
          deregisterFns.push(unwatchFn);
        });

        function watchGroupAction() {
          changeReactionScheduled = false;

          try {
            if (firstRun) {
              firstRun = false;
              listener(newValues, newValues, self);
            } else {
              listener(newValues, oldValues, self);
            }
          } finally {
            for (var i = 0; i < watchExpressions.length; i++) {
              oldValues[i] = newValues[i];
            }
          }
        }

        return function deregisterWatchGroup() {
          while (deregisterFns.length) {
            deregisterFns.shift()();
          }
        };
      },


      /**
       * @ngdoc method
       * @name $rootScope.Scope#$watchCollection
       * @kind function
       *
       * @description
       * Shallow watches the properties of an object and fires whenever any of the properties change
       * (for arrays, this implies watching the array items; for object maps, this implies watching
       * the properties). If a change is detected, the `listener` callback is fired.
       *
       * - The `obj` collection is observed via standard $watch operation and is examined on every
       *   call to $digest() to see if any items have been added, removed, or moved.
       * - The `listener` is called whenever anything within the `obj` has changed. Examples include
       *   adding, removing, and moving items belonging to an object or array.
       *
       *
       * @example
       * ```js
          $scope.names = ['igor', 'matias', 'misko', 'james'];
          $scope.dataCount = 4;

          $scope.$watchCollection('names', function(newNames, oldNames) {
            $scope.dataCount = newNames.length;
          });

          expect($scope.dataCount).toEqual(4);
          $scope.$digest();

          //still at 4 ... no changes
          expect($scope.dataCount).toEqual(4);

          $scope.names.pop();
          $scope.$digest();

          //now there's been a change
          expect($scope.dataCount).toEqual(3);
       * ```
       *
       *
       * @param {string|function(scope)} obj Evaluated as {@link guide/expression expression}. The
       *    expression value should evaluate to an object or an array which is observed on each
       *    {@link ng.$rootScope.Scope#$digest $digest} cycle. Any shallow change within the
       *    collection will trigger a call to the `listener`.
       *
       * @param {function(newCollection, oldCollection, scope)} listener a callback function called
       *    when a change is detected.
       *    - The `newCollection` object is the newly modified data obtained from the `obj` expression
       *    - The `oldCollection` object is a copy of the former collection data.
       *      Due to performance considerations, the`oldCollection` value is computed only if the
       *      `listener` function declares two or more arguments.
       *    - The `scope` argument refers to the current scope.
       *
       * @returns {function()} Returns a de-registration function for this listener. When the
       *    de-registration function is executed, the internal watch operation is terminated.
       */
      $watchCollection: function(obj, listener) {
        // Mark the interceptor as
        // ... $$pure when literal since the instance will change when any input changes
        $watchCollectionInterceptor.$$pure = $parse(obj).literal;
        // ... $stateful when non-literal since we must read the state of the collection
        $watchCollectionInterceptor.$stateful = !$watchCollectionInterceptor.$$pure;

        var self = this;
        // the current value, updated on each dirty-check run
        var newValue;
        // a shallow copy of the newValue from the last dirty-check run,
        // updated to match newValue during dirty-check run
        var oldValue;
        // a shallow copy of the newValue from when the last change happened
        var veryOldValue;
        // only track veryOldValue if the listener is asking for it
        var trackVeryOldValue = (listener.length > 1);
        var changeDetected = 0;
        var changeDetector = $parse(obj, $watchCollectionInterceptor);
        var internalArray = [];
        var internalObject = {};
        var initRun = true;
        var oldLength = 0;

        function $watchCollectionInterceptor(_value) {
          newValue = _value;
          var newLength, key, bothNaN, newItem, oldItem;

          // If the new value is undefined, then return undefined as the watch may be a one-time watch
          if (isUndefined(newValue)) return;

          if (!isObject(newValue)) { // if primitive
            if (oldValue !== newValue) {
              oldValue = newValue;
              changeDetected++;
            }
          } else if (isArrayLike(newValue)) {
            if (oldValue !== internalArray) {
              // we are transitioning from something which was not an array into array.
              oldValue = internalArray;
              oldLength = oldValue.length = 0;
              changeDetected++;
            }

            newLength = newValue.length;

            if (oldLength !== newLength) {
              // if lengths do not match we need to trigger change notification
              changeDetected++;
              oldValue.length = oldLength = newLength;
            }
            // copy the items to oldValue and look for changes.
            for (var i = 0; i < newLength; i++) {
              oldItem = oldValue[i];
              newItem = newValue[i];

              // eslint-disable-next-line no-self-compare
              bothNaN = (oldItem !== oldItem) && (newItem !== newItem);
              if (!bothNaN && (oldItem !== newItem)) {
                changeDetected++;
                oldValue[i] = newItem;
              }
            }
          } else {
            if (oldValue !== internalObject) {
              // we are transitioning from something which was not an object into object.
              oldValue = internalObject = {};
              oldLength = 0;
              changeDetected++;
            }
            // copy the items to oldValue and look for changes.
            newLength = 0;
            for (key in newValue) {
              if (hasOwnProperty.call(newValue, key)) {
                newLength++;
                newItem = newValue[key];
                oldItem = oldValue[key];

                if (key in oldValue) {
                  // eslint-disable-next-line no-self-compare
                  bothNaN = (oldItem !== oldItem) && (newItem !== newItem);
                  if (!bothNaN && (oldItem !== newItem)) {
                    changeDetected++;
                    oldValue[key] = newItem;
                  }
                } else {
                  oldLength++;
                  oldValue[key] = newItem;
                  changeDetected++;
                }
              }
            }
            if (oldLength > newLength) {
              // we used to have more keys, need to find them and destroy them.
              changeDetected++;
              for (key in oldValue) {
                if (!hasOwnProperty.call(newValue, key)) {
                  oldLength--;
                  delete oldValue[key];
                }
              }
            }
          }
          return changeDetected;
        }

        function $watchCollectionAction() {
          if (initRun) {
            initRun = false;
            listener(newValue, newValue, self);
          } else {
            listener(newValue, veryOldValue, self);
          }

          // make a copy for the next time a collection is changed
          if (trackVeryOldValue) {
            if (!isObject(newValue)) {
              //primitive
              veryOldValue = newValue;
            } else if (isArrayLike(newValue)) {
              veryOldValue = new Array(newValue.length);
              for (var i = 0; i < newValue.length; i++) {
                veryOldValue[i] = newValue[i];
              }
            } else { // if object
              veryOldValue = {};
              for (var key in newValue) {
                if (hasOwnProperty.call(newValue, key)) {
                  veryOldValue[key] = newValue[key];
                }
              }
            }
          }
        }

        return this.$watch(changeDetector, $watchCollectionAction);
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$digest
       * @kind function
       *
       * @description
       * Processes all of the {@link ng.$rootScope.Scope#$watch watchers} of the current scope and
       * its children. Because a {@link ng.$rootScope.Scope#$watch watcher}'s listener can change
       * the model, the `$digest()` keeps calling the {@link ng.$rootScope.Scope#$watch watchers}
       * until no more listeners are firing. This means that it is possible to get into an infinite
       * loop. This function will throw `'Maximum iteration limit exceeded.'` if the number of
       * iterations exceeds 10.
       *
       * Usually, you don't call `$digest()` directly in
       * {@link ng.directive:ngController controllers} or in
       * {@link ng.$compileProvider#directive directives}.
       * Instead, you should call {@link ng.$rootScope.Scope#$apply $apply()} (typically from within
       * a {@link ng.$compileProvider#directive directive}), which will force a `$digest()`.
       *
       * If you want to be notified whenever `$digest()` is called,
       * you can register a `watchExpression` function with
       * {@link ng.$rootScope.Scope#$watch $watch()} with no `listener`.
       *
       * In unit tests, you may need to call `$digest()` to simulate the scope life cycle.
       *
       * @example
       * ```js
           var scope = ...;
           scope.name = 'misko';
           scope.counter = 0;

           expect(scope.counter).toEqual(0);
           scope.$watch('name', function(newValue, oldValue) {
             scope.counter = scope.counter + 1;
           });
           expect(scope.counter).toEqual(0);

           scope.$digest();
           // the listener is always called during the first $digest loop after it was registered
           expect(scope.counter).toEqual(1);

           scope.$digest();
           // but now it will not be called unless the value changes
           expect(scope.counter).toEqual(1);

           scope.name = 'adam';
           scope.$digest();
           expect(scope.counter).toEqual(2);
       * ```
       *
       */
      $digest: function() {
        var watch, value, last, fn, get,
            watchers,
            dirty, ttl = TTL,
            next, current, target = asyncQueue.length ? $rootScope : this,
            watchLog = [],
            logIdx, asyncTask;

        beginPhase('$digest');
        // Check for changes to browser url that happened in sync before the call to $digest
        $browser.$$checkUrlChange();

        if (this === $rootScope && applyAsyncId !== null) {
          // If this is the root scope, and $applyAsync has scheduled a deferred $apply(), then
          // cancel the scheduled $apply and flush the queue of expressions to be evaluated.
          $browser.defer.cancel(applyAsyncId);
          flushApplyAsync();
        }

        lastDirtyWatch = null;

        do { // "while dirty" loop
          dirty = false;
          current = target;

          // It's safe for asyncQueuePosition to be a local variable here because this loop can't
          // be reentered recursively. Calling $digest from a function passed to $evalAsync would
          // lead to a '$digest already in progress' error.
          for (var asyncQueuePosition = 0; asyncQueuePosition < asyncQueue.length; asyncQueuePosition++) {
            try {
              asyncTask = asyncQueue[asyncQueuePosition];
              fn = asyncTask.fn;
              fn(asyncTask.scope, asyncTask.locals);
            } catch (e) {
              $exceptionHandler(e);
            }
            lastDirtyWatch = null;
          }
          asyncQueue.length = 0;

          traverseScopesLoop:
          do { // "traverse the scopes" loop
            if ((watchers = !current.$$suspended && current.$$watchers)) {
              // process our watches
              watchers.$$digestWatchIndex = watchers.length;
              while (watchers.$$digestWatchIndex--) {
                try {
                  watch = watchers[watchers.$$digestWatchIndex];
                  // Most common watches are on primitives, in which case we can short
                  // circuit it with === operator, only when === fails do we use .equals
                  if (watch) {
                    get = watch.get;
                    if ((value = get(current)) !== (last = watch.last) &&
                        !(watch.eq
                            ? equals(value, last)
                            : (isNumberNaN(value) && isNumberNaN(last)))) {
                      dirty = true;
                      lastDirtyWatch = watch;
                      watch.last = watch.eq ? copy(value, null) : value;
                      fn = watch.fn;
                      fn(value, ((last === initWatchVal) ? value : last), current);
                      if (ttl < 5) {
                        logIdx = 4 - ttl;
                        if (!watchLog[logIdx]) watchLog[logIdx] = [];
                        watchLog[logIdx].push({
                          msg: isFunction(watch.exp) ? 'fn: ' + (watch.exp.name || watch.exp.toString()) : watch.exp,
                          newVal: value,
                          oldVal: last
                        });
                      }
                    } else if (watch === lastDirtyWatch) {
                      // If the most recently dirty watcher is now clean, short circuit since the remaining watchers
                      // have already been tested.
                      dirty = false;
                      break traverseScopesLoop;
                    }
                  }
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
            }

            // Insanity Warning: scope depth-first traversal
            // yes, this code is a bit crazy, but it works and we have tests to prove it!
            // this piece should be kept in sync with the traversal in $broadcast
            // (though it differs due to having the extra check for $$suspended and does not
            // check $$listenerCount)
            if (!(next = ((!current.$$suspended && current.$$watchersCount && current.$$childHead) ||
                (current !== target && current.$$nextSibling)))) {
              while (current !== target && !(next = current.$$nextSibling)) {
                current = current.$parent;
              }
            }
          } while ((current = next));

          // `break traverseScopesLoop;` takes us to here

          if ((dirty || asyncQueue.length) && !(ttl--)) {
            clearPhase();
            throw $rootScopeMinErr('infdig',
                '{0} $digest() iterations reached. Aborting!\n' +
                'Watchers fired in the last 5 iterations: {1}',
                TTL, watchLog);
          }

        } while (dirty || asyncQueue.length);

        clearPhase();

        // postDigestQueuePosition isn't local here because this loop can be reentered recursively.
        while (postDigestQueuePosition < postDigestQueue.length) {
          try {
            postDigestQueue[postDigestQueuePosition++]();
          } catch (e) {
            $exceptionHandler(e);
          }
        }
        postDigestQueue.length = postDigestQueuePosition = 0;

        // Check for changes to browser url that happened during the $digest
        // (for which no event is fired; e.g. via `history.pushState()`)
        $browser.$$checkUrlChange();
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$suspend
       * @kind function
       *
       * @description
       * Suspend watchers of this scope subtree so that they will not be invoked during digest.
       *
       * This can be used to optimize your application when you know that running those watchers
       * is redundant.
       *
       * **Warning**
       *
       * Suspending scopes from the digest cycle can have unwanted and difficult to debug results.
       * Only use this approach if you are confident that you know what you are doing and have
       * ample tests to ensure that bindings get updated as you expect.
       *
       * Some of the things to consider are:
       *
       * * Any external event on a directive/component will not trigger a digest while the hosting
       *   scope is suspended - even if the event handler calls `$apply()` or `$rootScope.$digest()`.
       * * Transcluded content exists on a scope that inherits from outside a directive but exists
       *   as a child of the directive's containing scope. If the containing scope is suspended the
       *   transcluded scope will also be suspended, even if the scope from which the transcluded
       *   scope inherits is not suspended.
       * * Multiple directives trying to manage the suspended status of a scope can confuse each other:
       *    * A call to `$suspend()` on an already suspended scope is a no-op.
       *    * A call to `$resume()` on a non-suspended scope is a no-op.
       *    * If two directives suspend a scope, then one of them resumes the scope, the scope will no
       *      longer be suspended. This could result in the other directive believing a scope to be
       *      suspended when it is not.
       * * If a parent scope is suspended then all its descendants will be also excluded from future
       *   digests whether or not they have been suspended themselves. Note that this also applies to
       *   isolate child scopes.
       * * Calling `$digest()` directly on a descendant of a suspended scope will still run the watchers
       *   for that scope and its descendants. When digesting we only check whether the current scope is
       *   locally suspended, rather than checking whether it has a suspended ancestor.
       * * Calling `$resume()` on a scope that has a suspended ancestor will not cause the scope to be
       *   included in future digests until all its ancestors have been resumed.
       * * Resolved promises, e.g. from explicit `$q` deferreds and `$http` calls, trigger `$apply()`
       *   against the `$rootScope` and so will still trigger a global digest even if the promise was
       *   initiated by a component that lives on a suspended scope.
       */
      $suspend: function() {
        this.$$suspended = true;
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$isSuspended
       * @kind function
       *
       * @description
       * Call this method to determine if this scope has been explicitly suspended. It will not
       * tell you whether an ancestor has been suspended.
       * To determine if this scope will be excluded from a digest triggered at the $rootScope,
       * for example, you must check all its ancestors:
       *
       * ```
       * function isExcludedFromDigest(scope) {
       *   while(scope) {
       *     if (scope.$isSuspended()) return true;
       *     scope = scope.$parent;
       *   }
       *   return false;
       * ```
       *
       * Be aware that a scope may not be included in digests if it has a suspended ancestor,
       * even if `$isSuspended()` returns false.
       *
       * @returns true if the current scope has been suspended.
       */
      $isSuspended: function() {
        return this.$$suspended;
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$resume
       * @kind function
       *
       * @description
       * Resume watchers of this scope subtree in case it was suspended.
       *
       * See {@link $rootScope.Scope#$suspend} for information about the dangers of using this approach.
       */
      $resume: function() {
        this.$$suspended = false;
      },

      /**
       * @ngdoc event
       * @name $rootScope.Scope#$destroy
       * @eventType broadcast on scope being destroyed
       *
       * @description
       * Broadcasted when a scope and its children are being destroyed.
       *
       * Note that, in AngularJS, there is also a `$destroy` jQuery event, which can be used to
       * clean up DOM bindings before an element is removed from the DOM.
       */

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$destroy
       * @kind function
       *
       * @description
       * Removes the current scope (and all of its children) from the parent scope. Removal implies
       * that calls to {@link ng.$rootScope.Scope#$digest $digest()} will no longer
       * propagate to the current scope and its children. Removal also implies that the current
       * scope is eligible for garbage collection.
       *
       * The `$destroy()` is usually used by directives such as
       * {@link ng.directive:ngRepeat ngRepeat} for managing the
       * unrolling of the loop.
       *
       * Just before a scope is destroyed, a `$destroy` event is broadcasted on this scope.
       * Application code can register a `$destroy` event handler that will give it a chance to
       * perform any necessary cleanup.
       *
       * Note that, in AngularJS, there is also a `$destroy` jQuery event, which can be used to
       * clean up DOM bindings before an element is removed from the DOM.
       */
      $destroy: function() {
        // We can't destroy a scope that has been already destroyed.
        if (this.$$destroyed) return;
        var parent = this.$parent;

        this.$broadcast('$destroy');
        this.$$destroyed = true;

        if (this === $rootScope) {
          //Remove handlers attached to window when $rootScope is removed
          $browser.$$applicationDestroyed();
        }

        incrementWatchersCount(this, -this.$$watchersCount);
        for (var eventName in this.$$listenerCount) {
          decrementListenerCount(this, this.$$listenerCount[eventName], eventName);
        }

        // sever all the references to parent scopes (after this cleanup, the current scope should
        // not be retained by any of our references and should be eligible for garbage collection)
        if (parent && parent.$$childHead === this) parent.$$childHead = this.$$nextSibling;
        if (parent && parent.$$childTail === this) parent.$$childTail = this.$$prevSibling;
        if (this.$$prevSibling) this.$$prevSibling.$$nextSibling = this.$$nextSibling;
        if (this.$$nextSibling) this.$$nextSibling.$$prevSibling = this.$$prevSibling;

        // Disable listeners, watchers and apply/digest methods
        this.$destroy = this.$digest = this.$apply = this.$evalAsync = this.$applyAsync = noop;
        this.$on = this.$watch = this.$watchGroup = function() { return noop; };
        this.$$listeners = {};

        // Disconnect the next sibling to prevent `cleanUpScope` destroying those too
        this.$$nextSibling = null;
        cleanUpScope(this);
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$eval
       * @kind function
       *
       * @description
       * Executes the `expression` on the current scope and returns the result. Any exceptions in
       * the expression are propagated (uncaught). This is useful when evaluating AngularJS
       * expressions.
       *
       * @example
       * ```js
           var scope = ng.$rootScope.Scope();
           scope.a = 1;
           scope.b = 2;

           expect(scope.$eval('a+b')).toEqual(3);
           expect(scope.$eval(function(scope){ return scope.a + scope.b; })).toEqual(3);
       * ```
       *
       * @param {(string|function())=} expression An AngularJS expression to be executed.
       *
       *    - `string`: execute using the rules as defined in  {@link guide/expression expression}.
       *    - `function(scope)`: execute the function with the current `scope` parameter.
       *
       * @param {(object)=} locals Local variables object, useful for overriding values in scope.
       * @returns {*} The result of evaluating the expression.
       */
      $eval: function(expr, locals) {
        return $parse(expr)(this, locals);
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$evalAsync
       * @kind function
       *
       * @description
       * Executes the expression on the current scope at a later point in time.
       *
       * The `$evalAsync` makes no guarantees as to when the `expression` will be executed, only
       * that:
       *
       *   - it will execute after the function that scheduled the evaluation (preferably before DOM
       *     rendering).
       *   - at least one {@link ng.$rootScope.Scope#$digest $digest cycle} will be performed after
       *     `expression` execution.
       *
       * Any exceptions from the execution of the expression are forwarded to the
       * {@link ng.$exceptionHandler $exceptionHandler} service.
       *
       * __Note:__ if this function is called outside of a `$digest` cycle, a new `$digest` cycle
       * will be scheduled. However, it is encouraged to always call code that changes the model
       * from within an `$apply` call. That includes code evaluated via `$evalAsync`.
       *
       * @param {(string|function())=} expression An AngularJS expression to be executed.
       *
       *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
       *    - `function(scope)`: execute the function with the current `scope` parameter.
       *
       * @param {(object)=} locals Local variables object, useful for overriding values in scope.
       */
      $evalAsync: function(expr, locals) {
        // if we are outside of an $digest loop and this is the first time we are scheduling async
        // task also schedule async auto-flush
        if (!$rootScope.$$phase && !asyncQueue.length) {
          $browser.defer(function() {
            if (asyncQueue.length) {
              $rootScope.$digest();
            }
          }, null, '$evalAsync');
        }

        asyncQueue.push({scope: this, fn: $parse(expr), locals: locals});
      },

      $$postDigest: function(fn) {
        postDigestQueue.push(fn);
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$apply
       * @kind function
       *
       * @description
       * `$apply()` is used to execute an expression in AngularJS from outside of the AngularJS
       * framework. (For example from browser DOM events, setTimeout, XHR or third party libraries).
       * Because we are calling into the AngularJS framework we need to perform proper scope life
       * cycle of {@link ng.$exceptionHandler exception handling},
       * {@link ng.$rootScope.Scope#$digest executing watches}.
       *
       * **Life cycle: Pseudo-Code of `$apply()`**
       *
       * ```js
           function $apply(expr) {
             try {
               return $eval(expr);
             } catch (e) {
               $exceptionHandler(e);
             } finally {
               $root.$digest();
             }
           }
       * ```
       *
       *
       * Scope's `$apply()` method transitions through the following stages:
       *
       * 1. The {@link guide/expression expression} is executed using the
       *    {@link ng.$rootScope.Scope#$eval $eval()} method.
       * 2. Any exceptions from the execution of the expression are forwarded to the
       *    {@link ng.$exceptionHandler $exceptionHandler} service.
       * 3. The {@link ng.$rootScope.Scope#$watch watch} listeners are fired immediately after the
       *    expression was executed using the {@link ng.$rootScope.Scope#$digest $digest()} method.
       *
       *
       * @param {(string|function())=} exp An AngularJS expression to be executed.
       *
       *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
       *    - `function(scope)`: execute the function with current `scope` parameter.
       *
       * @returns {*} The result of evaluating the expression.
       */
      $apply: function(expr) {
        try {
          beginPhase('$apply');
          try {
            return this.$eval(expr);
          } finally {
            clearPhase();
          }
        } catch (e) {
          $exceptionHandler(e);
        } finally {
          try {
            $rootScope.$digest();
          } catch (e) {
            $exceptionHandler(e);
            // eslint-disable-next-line no-unsafe-finally
            throw e;
          }
        }
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$applyAsync
       * @kind function
       *
       * @description
       * Schedule the invocation of $apply to occur at a later time. The actual time difference
       * varies across browsers, but is typically around ~10 milliseconds.
       *
       * This can be used to queue up multiple expressions which need to be evaluated in the same
       * digest.
       *
       * @param {(string|function())=} exp An AngularJS expression to be executed.
       *
       *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
       *    - `function(scope)`: execute the function with current `scope` parameter.
       */
      $applyAsync: function(expr) {
        var scope = this;
        if (expr) {
          applyAsyncQueue.push($applyAsyncExpression);
        }
        expr = $parse(expr);
        scheduleApplyAsync();

        function $applyAsyncExpression() {
          scope.$eval(expr);
        }
      },

      /**
       * @ngdoc method
       * @name $rootScope.Scope#$on
       * @kind function
       *
       * @description
       * Listens on events of a given type. See {@link ng.$rootScope.Scope#$emit $emit} for
       * discussion of event life cycle.
       *
       * The event listener function format is: `function(event, args...)`. The `event` object
       * passed into the listener has the following attributes:
       *
       *   - `targetScope` - `{Scope}`: the scope on which the event was `$emit`-ed or
       *     `$broadcast`-ed.
       *   - `currentScope` - `{Scope}`: the scope that is currently handling the event. Once the
       *     event propagates through the scope hierarchy, this property is set to null.
       *   - `name` - `{string}`: name of the event.
       *   - `stopPropagation` - `{function=}`: calling `stopPropagation` function will cancel
       *     further event propagation (available only for events that were `$emit`-ed).
       *   - `preventDefault` - `{function}`: calling `preventDefault` sets `defaultPrevented` flag
       *     to true.
       *   - `defaultPrevented` - `{boolean}`: true if `preventDefault` was called.
       *
       * @param {string} name Event name to listen on.
       * @param {function(event, ...args)} listener Function to call when the event is emitted.
       * @returns {function()} Returns a deregistration function for this listener.
       */
      $on: function(name, listener) {
        var namedListeners = this.$$listeners[name];
        if (!namedListeners) {
          this.$$listeners[name] = namedListeners = [];
        }
        namedListeners.push(listener);

        var current = this;
        do {
          if (!current.$$listenerCount[name]) {
            current.$$listenerCount[name] = 0;
          }
          current.$$listenerCount[name]++;
        } while ((current = current.$parent));

        var self = this;
        return function() {
          var indexOfListener = namedListeners.indexOf(listener);
          if (indexOfListener !== -1) {
            // Use delete in the hope of the browser deallocating the memory for the array entry,
            // while not shifting the array indexes of other listeners.
            // See issue https://github.com/angular/angular.js/issues/16135
            delete namedListeners[indexOfListener];
            decrementListenerCount(self, 1, name);
          }
        };
      },


      /**
       * @ngdoc method
       * @name $rootScope.Scope#$emit
       * @kind function
       *
       * @description
       * Dispatches an event `name` upwards through the scope hierarchy notifying the
       * registered {@link ng.$rootScope.Scope#$on} listeners.
       *
       * The event life cycle starts at the scope on which `$emit` was called. All
       * {@link ng.$rootScope.Scope#$on listeners} listening for `name` event on this scope get
       * notified. Afterwards, the event traverses upwards toward the root scope and calls all
       * registered listeners along the way. The event will stop propagating if one of the listeners
       * cancels it.
       *
       * Any exception emitted from the {@link ng.$rootScope.Scope#$on listeners} will be passed
       * onto the {@link ng.$exceptionHandler $exceptionHandler} service.
       *
       * @param {string} name Event name to emit.
       * @param {...*} args Optional one or more arguments which will be passed onto the event listeners.
       * @return {Object} Event object (see {@link ng.$rootScope.Scope#$on}).
       */
      $emit: function(name, args) {
        var empty = [],
            namedListeners,
            scope = this,
            stopPropagation = false,
            event = {
              name: name,
              targetScope: scope,
              stopPropagation: function() {stopPropagation = true;},
              preventDefault: function() {
                event.defaultPrevented = true;
              },
              defaultPrevented: false
            },
            listenerArgs = concat([event], arguments, 1),
            i, length;

        do {
          namedListeners = scope.$$listeners[name] || empty;
          event.currentScope = scope;
          for (i = 0, length = namedListeners.length; i < length; i++) {

            // if listeners were deregistered, defragment the array
            if (!namedListeners[i]) {
              namedListeners.splice(i, 1);
              i--;
              length--;
              continue;
            }
            try {
              //allow all listeners attached to the current scope to run
              namedListeners[i].apply(null, listenerArgs);
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          //if any listener on the current scope stops propagation, prevent bubbling
          if (stopPropagation) {
            break;
          }
          //traverse upwards
          scope = scope.$parent;
        } while (scope);

        event.currentScope = null;

        return event;
      },


      /**
       * @ngdoc method
       * @name $rootScope.Scope#$broadcast
       * @kind function
       *
       * @description
       * Dispatches an event `name` downwards to all child scopes (and their children) notifying the
       * registered {@link ng.$rootScope.Scope#$on} listeners.
       *
       * The event life cycle starts at the scope on which `$broadcast` was called. All
       * {@link ng.$rootScope.Scope#$on listeners} listening for `name` event on this scope get
       * notified. Afterwards, the event propagates to all direct and indirect scopes of the current
       * scope and calls all registered listeners along the way. The event cannot be canceled.
       *
       * Any exception emitted from the {@link ng.$rootScope.Scope#$on listeners} will be passed
       * onto the {@link ng.$exceptionHandler $exceptionHandler} service.
       *
       * @param {string} name Event name to broadcast.
       * @param {...*} args Optional one or more arguments which will be passed onto the event listeners.
       * @return {Object} Event object, see {@link ng.$rootScope.Scope#$on}
       */
      $broadcast: function(name, args) {
        var target = this,
            current = target,
            next = target,
            event = {
              name: name,
              targetScope: target,
              preventDefault: function() {
                event.defaultPrevented = true;
              },
              defaultPrevented: false
            };

        if (!target.$$listenerCount[name]) return event;

        var listenerArgs = concat([event], arguments, 1),
            listeners, i, length;

        //down while you can, then up and next sibling or up and next sibling until back at root
        while ((current = next)) {
          event.currentScope = current;
          listeners = current.$$listeners[name] || [];
          for (i = 0, length = listeners.length; i < length; i++) {
            // if listeners were deregistered, defragment the array
            if (!listeners[i]) {
              listeners.splice(i, 1);
              i--;
              length--;
              continue;
            }

            try {
              listeners[i].apply(null, listenerArgs);
            } catch (e) {
              $exceptionHandler(e);
            }
          }

          // Insanity Warning: scope depth-first traversal
          // yes, this code is a bit crazy, but it works and we have tests to prove it!
          // this piece should be kept in sync with the traversal in $digest
          // (though it differs due to having the extra check for $$listenerCount and
          // does not check $$suspended)
          if (!(next = ((current.$$listenerCount[name] && current.$$childHead) ||
              (current !== target && current.$$nextSibling)))) {
            while (current !== target && !(next = current.$$nextSibling)) {
              current = current.$parent;
            }
          }
        }

        event.currentScope = null;
        return event;
      }
    };

    var $rootScope = new Scope();

    //The internal queues. Expose them on the $rootScope for debugging/testing purposes.
    var asyncQueue = $rootScope.$$asyncQueue = [];
    var postDigestQueue = $rootScope.$$postDigestQueue = [];
    var applyAsyncQueue = $rootScope.$$applyAsyncQueue = [];

    var postDigestQueuePosition = 0;

    return $rootScope;


    function beginPhase(phase) {
      if ($rootScope.$$phase) {
        throw $rootScopeMinErr('inprog', '{0} already in progress', $rootScope.$$phase);
      }

      $rootScope.$$phase = phase;
    }

    function clearPhase() {
      $rootScope.$$phase = null;
    }

    function incrementWatchersCount(current, count) {
      do {
        current.$$watchersCount += count;
      } while ((current = current.$parent));
    }

    function decrementListenerCount(current, count, name) {
      do {
        current.$$listenerCount[name] -= count;

        if (current.$$listenerCount[name] === 0) {
          delete current.$$listenerCount[name];
        }
      } while ((current = current.$parent));
    }

    /**
     * function used as an initial value for watchers.
     * because it's unique we can easily tell it apart from other values
     */
    function initWatchVal() {}

    function flushApplyAsync() {
      while (applyAsyncQueue.length) {
        try {
          applyAsyncQueue.shift()();
        } catch (e) {
          $exceptionHandler(e);
        }
      }
      applyAsyncId = null;
    }

    function scheduleApplyAsync() {
      if (applyAsyncId === null) {
        applyAsyncId = $browser.defer(function() {
          $rootScope.$apply(flushApplyAsync);
        }, null, '$applyAsync');
      }
    }
  }];
}

/**
 * @ngdoc service
 * @name $rootElement
 *
 * @description
 * The root element of AngularJS application. This is either the element where {@link
 * ng.directive:ngApp ngApp} was declared or the element passed into
 * {@link angular.bootstrap}. The element represents the root element of application. It is also the
 * location where the application's {@link auto.$injector $injector} service gets
 * published, and can be retrieved using `$rootElement.injector()`.
 */


// the implementation is in angular.bootstrap

/**
 * @this
 * @description
 * Private service to sanitize uris for links and images. Used by $compile and $sanitize.
 */
function $$SanitizeUriProvider() {

  var aHrefSanitizationWhitelist = /^\s*(https?|s?ftp|mailto|tel|file):/,
    imgSrcSanitizationWhitelist = /^\s*((https?|ftp|file|blob):|data:image\/)/;

  /**
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during a[href] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via HTML anchor links.
   *
   * Any url due to be assigned to an `a[href]` attribute via interpolation is marked as requiring
   * the $sce.URL security context. When interpolation occurs a call is made to `$sce.trustAsUrl(url)`
   * which in turn may call `$$sanitizeUri(url, isMedia)` to sanitize the potentially malicious URL.
   *
   * If the URL matches the `aHrefSanitizationWhitelist` regular expression, it is returned unchanged.
   *
   * If there is no match the URL is returned prefixed with `'unsafe:'` to ensure that when it is written
   * to the DOM it is inactive and potentially malicious code will not be executed.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
  this.aHrefSanitizationWhitelist = function(regexp) {
    if (isDefined(regexp)) {
      aHrefSanitizationWhitelist = regexp;
      return this;
    }
    return aHrefSanitizationWhitelist;
  };


  /**
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during img[src] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via HTML image src links.
   *
   * Any URL due to be assigned to an `img[src]` attribute via interpolation is marked as requiring
   * the $sce.MEDIA_URL security context. When interpolation occurs a call is made to
   * `$sce.trustAsMediaUrl(url)` which in turn may call `$$sanitizeUri(url, isMedia)` to sanitize
   * the potentially malicious URL.
   *
   * If the URL matches the `aImgSanitizationWhitelist` regular expression, it is returned unchanged.
   *
   * If there is no match the URL is returned prefixed with `'unsafe:'` to ensure that when it is written
   * to the DOM it is inactive and potentially malicious code will not be executed.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
  this.imgSrcSanitizationWhitelist = function(regexp) {
    if (isDefined(regexp)) {
      imgSrcSanitizationWhitelist = regexp;
      return this;
    }
    return imgSrcSanitizationWhitelist;
  };

  this.$get = function() {
    return function sanitizeUri(uri, isMediaUrl) {
      // if (!uri) return uri;
      var regex = isMediaUrl ? imgSrcSanitizationWhitelist : aHrefSanitizationWhitelist;
      var normalizedVal = urlResolve(uri && uri.trim()).href;
      if (normalizedVal !== '' && !normalizedVal.match(regex)) {
        return 'unsafe:' + normalizedVal;
      }
      return uri;
    };
  };
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* exported $SceProvider, $SceDelegateProvider */

var $sceMinErr = minErr('$sce');

var SCE_CONTEXTS = {
  // HTML is used when there's HTML rendered (e.g. ng-bind-html, iframe srcdoc binding).
  HTML: 'html',

  // Style statements or stylesheets. Currently unused in AngularJS.
  CSS: 'css',

  // An URL used in a context where it refers to the source of media, which are not expected to be run
  // as scripts, such as an image, audio, video, etc.
  MEDIA_URL: 'mediaUrl',

  // An URL used in a context where it does not refer to a resource that loads code.
  // A value that can be trusted as a URL can also trusted as a MEDIA_URL.
  URL: 'url',

  // RESOURCE_URL is a subtype of URL used where the referred-to resource could be interpreted as
  // code. (e.g. ng-include, script src binding, templateUrl)
  // A value that can be trusted as a RESOURCE_URL, can also trusted as a URL and a MEDIA_URL.
  RESOURCE_URL: 'resourceUrl',

  // Script. Currently unused in AngularJS.
  JS: 'js'
};

// Helper functions follow.

var UNDERSCORE_LOWERCASE_REGEXP = /_([a-z])/g;

function snakeToCamel(name) {
  return name
    .replace(UNDERSCORE_LOWERCASE_REGEXP, fnCamelCaseReplace);
}

function adjustMatcher(matcher) {
  if (matcher === 'self') {
    return matcher;
  } else if (isString(matcher)) {
    // Strings match exactly except for 2 wildcards - '*' and '**'.
    // '*' matches any character except those from the set ':/.?&'.
    // '**' matches any character (like .* in a RegExp).
    // More than 2 *'s raises an error as it's ill defined.
    if (matcher.indexOf('***') > -1) {
      throw $sceMinErr('iwcard',
          'Illegal sequence *** in string matcher.  String: {0}', matcher);
    }
    matcher = escapeForRegexp(matcher).
                  replace(/\\\*\\\*/g, '.*').
                  replace(/\\\*/g, '[^:/.?&;]*');
    return new RegExp('^' + matcher + '$');
  } else if (isRegExp(matcher)) {
    // The only other type of matcher allowed is a Regexp.
    // Match entire URL / disallow partial matches.
    // Flags are reset (i.e. no global, ignoreCase or multiline)
    return new RegExp('^' + matcher.source + '$');
  } else {
    throw $sceMinErr('imatcher',
        'Matchers may only be "self", string patterns or RegExp objects');
  }
}


function adjustMatchers(matchers) {
  var adjustedMatchers = [];
  if (isDefined(matchers)) {
    forEach(matchers, function(matcher) {
      adjustedMatchers.push(adjustMatcher(matcher));
    });
  }
  return adjustedMatchers;
}


/**
 * @ngdoc service
 * @name $sceDelegate
 * @kind function
 *
 * @description
 *
 * `$sceDelegate` is a service that is used by the `$sce` service to provide {@link ng.$sce Strict
 * Contextual Escaping (SCE)} services to AngularJS.
 *
 * For an overview of this service and the functionnality it provides in AngularJS, see the main
 * page for {@link ng.$sce SCE}. The current page is targeted for developers who need to alter how
 * SCE works in their application, which shouldn't be needed in most cases.
 *
 * <div class="alert alert-danger">
 * AngularJS strongly relies on contextual escaping for the security of bindings: disabling or
 * modifying this might cause cross site scripting (XSS) vulnerabilities. For libraries owners,
 * changes to this service will also influence users, so be extra careful and document your changes.
 * </div>
 *
 * Typically, you would configure or override the {@link ng.$sceDelegate $sceDelegate} instead of
 * the `$sce` service to customize the way Strict Contextual Escaping works in AngularJS.  This is
 * because, while the `$sce` provides numerous shorthand methods, etc., you really only need to
 * override 3 core functions (`trustAs`, `getTrusted` and `valueOf`) to replace the way things
 * work because `$sce` delegates to `$sceDelegate` for these operations.
 *
 * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} to configure this service.
 *
 * The default instance of `$sceDelegate` should work out of the box with little pain.  While you
 * can override it completely to change the behavior of `$sce`, the common case would
 * involve configuring the {@link ng.$sceDelegateProvider $sceDelegateProvider} instead by setting
 * your own whitelists and blacklists for trusting URLs used for loading AngularJS resources such as
 * templates.  Refer {@link ng.$sceDelegateProvider#resourceUrlWhitelist
 * $sceDelegateProvider.resourceUrlWhitelist} and {@link
 * ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist}
 */

/**
 * @ngdoc provider
 * @name $sceDelegateProvider
 * @this
 *
 * @description
 *
 * The `$sceDelegateProvider` provider allows developers to configure the {@link ng.$sceDelegate
 * $sceDelegate service}, used as a delegate for {@link ng.$sce Strict Contextual Escaping (SCE)}.
 *
 * The `$sceDelegateProvider` allows one to get/set the whitelists and blacklists used to ensure
 * that the URLs used for sourcing AngularJS templates and other script-running URLs are safe (all
 * places that use the `$sce.RESOURCE_URL` context). See
 * {@link ng.$sceDelegateProvider#resourceUrlWhitelist $sceDelegateProvider.resourceUrlWhitelist}
 * and
 * {@link ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist},
 *
 * For the general details about this service in AngularJS, read the main page for {@link ng.$sce
 * Strict Contextual Escaping (SCE)}.
 *
 * **Example**:  Consider the following case. <a name="example"></a>
 *
 * - your app is hosted at url `http://myapp.example.com/`
 * - but some of your templates are hosted on other domains you control such as
 *   `http://srv01.assets.example.com/`, `http://srv02.assets.example.com/`, etc.
 * - and you have an open redirect at `http://myapp.example.com/clickThru?...`.
 *
 * Here is what a secure configuration for this scenario might look like:
 *
 * ```
 *  angular.module('myApp', []).config(function($sceDelegateProvider) {
 *    $sceDelegateProvider.resourceUrlWhitelist([
 *      // Allow same origin resource loads.
 *      'self',
 *      // Allow loading from our assets domain.  Notice the difference between * and **.
 *      'http://srv*.assets.example.com/**'
 *    ]);
 *
 *    // The blacklist overrides the whitelist so the open redirect here is blocked.
 *    $sceDelegateProvider.resourceUrlBlacklist([
 *      'http://myapp.example.com/clickThru**'
 *    ]);
 *  });
 * ```
 * Note that an empty whitelist will block every resource URL from being loaded, and will require
 * you to manually mark each one as trusted with `$sce.trustAsResourceUrl`. However, templates
 * requested by {@link ng.$templateRequest $templateRequest} that are present in
 * {@link ng.$templateCache $templateCache} will not go through this check. If you have a mechanism
 * to populate your templates in that cache at config time, then it is a good idea to remove 'self'
 * from that whitelist. This helps to mitigate the security impact of certain types of issues, like
 * for instance attacker-controlled `ng-includes`.
 */

function $SceDelegateProvider() {
  this.SCE_CONTEXTS = SCE_CONTEXTS;

  // Resource URLs can also be trusted by policy.
  var resourceUrlWhitelist = ['self'],
      resourceUrlBlacklist = [];

  /**
   * @ngdoc method
   * @name $sceDelegateProvider#resourceUrlWhitelist
   * @kind function
   *
   * @param {Array=} whitelist When provided, replaces the resourceUrlWhitelist with the value
   *     provided.  This must be an array or null.  A snapshot of this array is used so further
   *     changes to the array are ignored.
   *     Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
   *     allowed in this array.
   *
   * @return {Array} The currently set whitelist array.
   *
   * @description
   * Sets/Gets the whitelist of trusted resource URLs.
   *
   * The **default value** when no whitelist has been explicitly set is `['self']` allowing only
   * same origin resource requests.
   *
   * <div class="alert alert-warning">
   * **Note:** the default whitelist of 'self' is not recommended if your app shares its origin
   * with other apps! It is a good idea to limit it to only your application's directory.
   * </div>
   */
  this.resourceUrlWhitelist = function(value) {
    if (arguments.length) {
      resourceUrlWhitelist = adjustMatchers(value);
    }
    return resourceUrlWhitelist;
  };

  /**
   * @ngdoc method
   * @name $sceDelegateProvider#resourceUrlBlacklist
   * @kind function
   *
   * @param {Array=} blacklist When provided, replaces the resourceUrlBlacklist with the value
   *     provided.  This must be an array or null.  A snapshot of this array is used so further
   *     changes to the array are ignored.</p><p>
   *     Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
   *     allowed in this array.</p><p>
   *     The typical usage for the blacklist is to **block
   *     [open redirects](http://cwe.mitre.org/data/definitions/601.html)** served by your domain as
   *     these would otherwise be trusted but actually return content from the redirected domain.
   *     </p><p>
   *     Finally, **the blacklist overrides the whitelist** and has the final say.
   *
   * @return {Array} The currently set blacklist array.
   *
   * @description
   * Sets/Gets the blacklist of trusted resource URLs.
   *
   * The **default value** when no whitelist has been explicitly set is the empty array (i.e. there
   * is no blacklist.)
   */

  this.resourceUrlBlacklist = function(value) {
    if (arguments.length) {
      resourceUrlBlacklist = adjustMatchers(value);
    }
    return resourceUrlBlacklist;
  };

  this.$get = ['$injector', '$$sanitizeUri', function($injector, $$sanitizeUri) {

    var htmlSanitizer = function htmlSanitizer(html) {
      throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
    };

    if ($injector.has('$sanitize')) {
      htmlSanitizer = $injector.get('$sanitize');
    }


    function matchUrl(matcher, parsedUrl) {
      if (matcher === 'self') {
        return urlIsSameOrigin(parsedUrl) || urlIsSameOriginAsBaseUrl(parsedUrl);
      } else {
        // definitely a regex.  See adjustMatchers()
        return !!matcher.exec(parsedUrl.href);
      }
    }

    function isResourceUrlAllowedByPolicy(url) {
      var parsedUrl = urlResolve(url.toString());
      var i, n, allowed = false;
      // Ensure that at least one item from the whitelist allows this url.
      for (i = 0, n = resourceUrlWhitelist.length; i < n; i++) {
        if (matchUrl(resourceUrlWhitelist[i], parsedUrl)) {
          allowed = true;
          break;
        }
      }
      if (allowed) {
        // Ensure that no item from the blacklist blocked this url.
        for (i = 0, n = resourceUrlBlacklist.length; i < n; i++) {
          if (matchUrl(resourceUrlBlacklist[i], parsedUrl)) {
            allowed = false;
            break;
          }
        }
      }
      return allowed;
    }

    function generateHolderType(Base) {
      var holderType = function TrustedValueHolderType(trustedValue) {
        this.$$unwrapTrustedValue = function() {
          return trustedValue;
        };
      };
      if (Base) {
        holderType.prototype = new Base();
      }
      holderType.prototype.valueOf = function sceValueOf() {
        return this.$$unwrapTrustedValue();
      };
      holderType.prototype.toString = function sceToString() {
        return this.$$unwrapTrustedValue().toString();
      };
      return holderType;
    }

    var trustedValueHolderBase = generateHolderType(),
        byType = {};

    byType[SCE_CONTEXTS.HTML] = generateHolderType(trustedValueHolderBase);
    byType[SCE_CONTEXTS.CSS] = generateHolderType(trustedValueHolderBase);
    byType[SCE_CONTEXTS.MEDIA_URL] = generateHolderType(trustedValueHolderBase);
    byType[SCE_CONTEXTS.URL] = generateHolderType(byType[SCE_CONTEXTS.MEDIA_URL]);
    byType[SCE_CONTEXTS.JS] = generateHolderType(trustedValueHolderBase);
    byType[SCE_CONTEXTS.RESOURCE_URL] = generateHolderType(byType[SCE_CONTEXTS.URL]);

    /**
     * @ngdoc method
     * @name $sceDelegate#trustAs
     *
     * @description
     * Returns a trusted representation of the parameter for the specified context. This trusted
     * object will later on be used as-is, without any security check, by bindings or directives
     * that require this security context.
     * For instance, marking a string as trusted for the `$sce.HTML` context will entirely bypass
     * the potential `$sanitize` call in corresponding `$sce.HTML` bindings or directives, such as
     * `ng-bind-html`. Note that in most cases you won't need to call this function: if you have the
     * sanitizer loaded, passing the value itself will render all the HTML that does not pose a
     * security risk.
     *
     * See {@link ng.$sceDelegate#getTrusted getTrusted} for the function that will consume those
     * trusted values, and {@link ng.$sce $sce} for general documentation about strict contextual
     * escaping.
     *
     * @param {string} type The context in which this value is safe for use, e.g. `$sce.URL`,
     *     `$sce.RESOURCE_URL`, `$sce.HTML`, `$sce.JS` or `$sce.CSS`.
     *
     * @param {*} value The value that should be considered trusted.
     * @return {*} A trusted representation of value, that can be used in the given context.
     */
    function trustAs(type, trustedValue) {
      var Constructor = (byType.hasOwnProperty(type) ? byType[type] : null);
      if (!Constructor) {
        throw $sceMinErr('icontext',
            'Attempted to trust a value in invalid context. Context: {0}; Value: {1}',
            type, trustedValue);
      }
      if (trustedValue === null || isUndefined(trustedValue) || trustedValue === '') {
        return trustedValue;
      }
      // All the current contexts in SCE_CONTEXTS happen to be strings.  In order to avoid trusting
      // mutable objects, we ensure here that the value passed in is actually a string.
      if (typeof trustedValue !== 'string') {
        throw $sceMinErr('itype',
            'Attempted to trust a non-string value in a content requiring a string: Context: {0}',
            type);
      }
      return new Constructor(trustedValue);
    }

    /**
     * @ngdoc method
     * @name $sceDelegate#valueOf
     *
     * @description
     * If the passed parameter had been returned by a prior call to {@link ng.$sceDelegate#trustAs
     * `$sceDelegate.trustAs`}, returns the value that had been passed to {@link
     * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}.
     *
     * If the passed parameter is not a value that had been returned by {@link
     * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}, it must be returned as-is.
     *
     * @param {*} value The result of a prior {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}
     *     call or anything else.
     * @return {*} The `value` that was originally provided to {@link ng.$sceDelegate#trustAs
     *     `$sceDelegate.trustAs`} if `value` is the result of such a call.  Otherwise, returns
     *     `value` unchanged.
     */
    function valueOf(maybeTrusted) {
      if (maybeTrusted instanceof trustedValueHolderBase) {
        return maybeTrusted.$$unwrapTrustedValue();
      } else {
        return maybeTrusted;
      }
    }

    /**
     * @ngdoc method
     * @name $sceDelegate#getTrusted
     *
     * @description
     * Given an object and a security context in which to assign it, returns a value that's safe to
     * use in this context, which was represented by the parameter. To do so, this function either
     * unwraps the safe type it has been given (for instance, a {@link ng.$sceDelegate#trustAs
     * `$sceDelegate.trustAs`} result), or it might try to sanitize the value given, depending on
     * the context and sanitizer availablility.
     *
     * The contexts that can be sanitized are $sce.MEDIA_URL, $sce.URL and $sce.HTML. The first two are available
     * by default, and the third one relies on the `$sanitize` service (which may be loaded through
     * the `ngSanitize` module). Furthermore, for $sce.RESOURCE_URL context, a plain string may be
     * accepted if the resource url policy defined by {@link ng.$sceDelegateProvider#resourceUrlWhitelist
     * `$sceDelegateProvider.resourceUrlWhitelist`} and {@link ng.$sceDelegateProvider#resourceUrlBlacklist
     * `$sceDelegateProvider.resourceUrlBlacklist`} accepts that resource.
     *
     * This function will throw if the safe type isn't appropriate for this context, or if the
     * value given cannot be accepted in the context (which might be caused by sanitization not
     * being available, or the value not being recognized as safe).
     *
     * <div class="alert alert-danger">
     * Disabling auto-escaping is extremely dangerous, it usually creates a Cross Site Scripting
     * (XSS) vulnerability in your application.
     * </div>
     *
     * @param {string} type The context in which this value is to be used (such as `$sce.HTML`).
     * @param {*} maybeTrusted The result of a prior {@link ng.$sceDelegate#trustAs
     *     `$sceDelegate.trustAs`} call, or anything else (which will not be considered trusted.)
     * @return {*} A version of the value that's safe to use in the given context, or throws an
     *     exception if this is impossible.
     */
    function getTrusted(type, maybeTrusted) {
      if (maybeTrusted === null || isUndefined(maybeTrusted) || maybeTrusted === '') {
        return maybeTrusted;
      }
      var constructor = (byType.hasOwnProperty(type) ? byType[type] : null);
      // If maybeTrusted is a trusted class instance or subclass instance, then unwrap and return
      // as-is.
      if (constructor && maybeTrusted instanceof constructor) {
        return maybeTrusted.$$unwrapTrustedValue();
      }

      // If maybeTrusted is a trusted class instance but not of the correct trusted type
      // then unwrap it and allow it to pass through to the rest of the checks
      if (isFunction(maybeTrusted.$$unwrapTrustedValue)) {
        maybeTrusted = maybeTrusted.$$unwrapTrustedValue();
      }

      // If we get here, then we will either sanitize the value or throw an exception.
      if (type === SCE_CONTEXTS.MEDIA_URL || type === SCE_CONTEXTS.URL) {
        // we attempt to sanitize non-resource URLs
        return $$sanitizeUri(maybeTrusted.toString(), type === SCE_CONTEXTS.MEDIA_URL);
      } else if (type === SCE_CONTEXTS.RESOURCE_URL) {
        if (isResourceUrlAllowedByPolicy(maybeTrusted)) {
          return maybeTrusted;
        } else {
          throw $sceMinErr('insecurl',
              'Blocked loading resource from url not allowed by $sceDelegate policy.  URL: {0}',
              maybeTrusted.toString());
        }
      } else if (type === SCE_CONTEXTS.HTML) {
        // htmlSanitizer throws its own error when no sanitizer is available.
        return htmlSanitizer(maybeTrusted);
      }
      // Default error when the $sce service has no way to make the input safe.
      throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
    }

    return { trustAs: trustAs,
             getTrusted: getTrusted,
             valueOf: valueOf };
  }];
}


/**
 * @ngdoc provider
 * @name $sceProvider
 * @this
 *
 * @description
 *
 * The $sceProvider provider allows developers to configure the {@link ng.$sce $sce} service.
 * -   enable/disable Strict Contextual Escaping (SCE) in a module
 * -   override the default implementation with a custom delegate
 *
 * Read more about {@link ng.$sce Strict Contextual Escaping (SCE)}.
 */

/**
 * @ngdoc service
 * @name $sce
 * @kind function
 *
 * @description
 *
 * `$sce` is a service that provides Strict Contextual Escaping services to AngularJS.
 *
 * ## Strict Contextual Escaping
 *
 * Strict Contextual Escaping (SCE) is a mode in which AngularJS constrains bindings to only render
 * trusted values. Its goal is to assist in writing code in a way that (a) is secure by default, and
 * (b) makes auditing for security vulnerabilities such as XSS, clickjacking, etc. a lot easier.
 *
 * ### Overview
 *
 * To systematically block XSS security bugs, AngularJS treats all values as untrusted by default in
 * HTML or sensitive URL bindings. When binding untrusted values, AngularJS will automatically
 * run security checks on them (sanitizations, whitelists, depending on context), or throw when it
 * cannot guarantee the security of the result. That behavior depends strongly on contexts: HTML
 * can be sanitized, but template URLs cannot, for instance.
 *
 * To illustrate this, consider the `ng-bind-html` directive. It renders its value directly as HTML:
 * we call that the *context*. When given an untrusted input, AngularJS will attempt to sanitize it
 * before rendering if a sanitizer is available, and throw otherwise. To bypass sanitization and
 * render the input as-is, you will need to mark it as trusted for that context before attempting
 * to bind it.
 *
 * As of version 1.2, AngularJS ships with SCE enabled by default.
 *
 * ### In practice
 *
 * Here's an example of a binding in a privileged context:
 *
 * ```
 * <input ng-model="userHtml" aria-label="User input">
 * <div ng-bind-html="userHtml"></div>
 * ```
 *
 * Notice that `ng-bind-html` is bound to `userHtml` controlled by the user.  With SCE
 * disabled, this application allows the user to render arbitrary HTML into the DIV, which would
 * be an XSS security bug. In a more realistic example, one may be rendering user comments, blog
 * articles, etc. via bindings. (HTML is just one example of a context where rendering user
 * controlled input creates security vulnerabilities.)
 *
 * For the case of HTML, you might use a library, either on the client side, or on the server side,
 * to sanitize unsafe HTML before binding to the value and rendering it in the document.
 *
 * How would you ensure that every place that used these types of bindings was bound to a value that
 * was sanitized by your library (or returned as safe for rendering by your server?)  How can you
 * ensure that you didn't accidentally delete the line that sanitized the value, or renamed some
 * properties/fields and forgot to update the binding to the sanitized value?
 *
 * To be secure by default, AngularJS makes sure bindings go through that sanitization, or
 * any similar validation process, unless there's a good reason to trust the given value in this
 * context.  That trust is formalized with a function call. This means that as a developer, you
 * can assume all untrusted bindings are safe. Then, to audit your code for binding security issues,
 * you just need to ensure the values you mark as trusted indeed are safe - because they were
 * received from your server, sanitized by your library, etc. You can organize your codebase to
 * help with this - perhaps allowing only the files in a specific directory to do this.
 * Ensuring that the internal API exposed by that code doesn't markup arbitrary values as safe then
 * becomes a more manageable task.
 *
 * In the case of AngularJS' SCE service, one uses {@link ng.$sce#trustAs $sce.trustAs}
 * (and shorthand methods such as {@link ng.$sce#trustAsHtml $sce.trustAsHtml}, etc.) to
 * build the trusted versions of your values.
 *
 * ### How does it work?
 *
 * In privileged contexts, directives and code will bind to the result of {@link ng.$sce#getTrusted
 * $sce.getTrusted(context, value)} rather than to the value directly.  Think of this function as
 * a way to enforce the required security context in your data sink. Directives use {@link
 * ng.$sce#parseAs $sce.parseAs} rather than `$parse` to watch attribute bindings, which performs
 * the {@link ng.$sce#getTrusted $sce.getTrusted} behind the scenes on non-constant literals. Also,
 * when binding without directives, AngularJS will understand the context of your bindings
 * automatically.
 *
 * As an example, {@link ng.directive:ngBindHtml ngBindHtml} uses {@link
 * ng.$sce#parseAsHtml $sce.parseAsHtml(binding expression)}.  Here's the actual code (slightly
 * simplified):
 *
 * ```
 * var ngBindHtmlDirective = ['$sce', function($sce) {
 *   return function(scope, element, attr) {
 *     scope.$watch($sce.parseAsHtml(attr.ngBindHtml), function(value) {
 *       element.html(value || '');
 *     });
 *   };
 * }];
 * ```
 *
 * ### Impact on loading templates
 *
 * This applies both to the {@link ng.directive:ngInclude `ng-include`} directive as well as
 * `templateUrl`'s specified by {@link guide/directive directives}.
 *
 * By default, AngularJS only loads templates from the same domain and protocol as the application
 * document.  This is done by calling {@link ng.$sce#getTrustedResourceUrl
 * $sce.getTrustedResourceUrl} on the template URL.  To load templates from other domains and/or
 * protocols, you may either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist
 * them} or {@link ng.$sce#trustAsResourceUrl wrap it} into a trusted value.
 *
 * *Please note*:
 * The browser's
 * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
 * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
 * policy apply in addition to this and may further restrict whether the template is successfully
 * loaded.  This means that without the right CORS policy, loading templates from a different domain
 * won't work on all browsers.  Also, loading templates from `file://` URL does not work on some
 * browsers.
 *
 * ### This feels like too much overhead
 *
 * It's important to remember that SCE only applies to interpolation expressions.
 *
 * If your expressions are constant literals, they're automatically trusted and you don't need to
 * call `$sce.trustAs` on them (e.g.
 * `<div ng-bind-html="'<b>implicitly trusted</b>'"></div>`) just works (remember to include the
 * `ngSanitize` module). The `$sceDelegate` will also use the `$sanitize` service if it is available
 * when binding untrusted values to `$sce.HTML` context.
 * AngularJS provides an implementation in `angular-sanitize.js`, and if you
 * wish to use it, you will also need to depend on the {@link ngSanitize `ngSanitize`} module in
 * your application.
 *
 * The included {@link ng.$sceDelegate $sceDelegate} comes with sane defaults to allow you to load
 * templates in `ng-include` from your application's domain without having to even know about SCE.
 * It blocks loading templates from other domains or loading templates over http from an https
 * served document.  You can change these by setting your own custom {@link
 * ng.$sceDelegateProvider#resourceUrlWhitelist whitelists} and {@link
 * ng.$sceDelegateProvider#resourceUrlBlacklist blacklists} for matching such URLs.
 *
 * This significantly reduces the overhead.  It is far easier to pay the small overhead and have an
 * application that's secure and can be audited to verify that with much more ease than bolting
 * security onto an application later.
 *
 * <a name="contexts"></a>
 * ### What trusted context types are supported?
 *
 * | Context             | Notes          |
 * |---------------------|----------------|
 * | `$sce.HTML`         | For HTML that's safe to source into the application.  The {@link ng.directive:ngBindHtml ngBindHtml} directive uses this context for bindings. If an unsafe value is encountered and the {@link ngSanitize $sanitize} module is present this will sanitize the value instead of throwing an error. |
 * | `$sce.CSS`          | For CSS that's safe to source into the application.  Currently unused.  Feel free to use it in your own directives. |
 * | `$sce.MEDIA_URL`    | For URLs that are safe to render as media. Is automatically converted from string by sanitizing when needed. |
 * | `$sce.URL`          | For URLs that are safe to follow as links. Is automatically converted from string by sanitizing when needed. Note that `$sce.URL` makes a stronger statement about the URL than `$sce.MEDIA_URL` does and therefore contexts requiring values trusted for `$sce.URL` can be used anywhere that values trusted for `$sce.MEDIA_URL` are required.|
 * | `$sce.RESOURCE_URL` | For URLs that are not only safe to follow as links, but whose contents are also safe to include in your application.  Examples include `ng-include`, `src` / `ngSrc` bindings for tags other than `IMG` (e.g. `IFRAME`, `OBJECT`, etc.)  <br><br>Note that `$sce.RESOURCE_URL` makes a stronger statement about the URL than `$sce.URL` or `$sce.MEDIA_URL` do and therefore contexts requiring values trusted for `$sce.RESOURCE_URL` can be used anywhere that values trusted for `$sce.URL` or `$sce.MEDIA_URL` are required. <br><br> The {@link $sceDelegateProvider#resourceUrlWhitelist $sceDelegateProvider#resourceUrlWhitelist()} and {@link $sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider#resourceUrlBlacklist()} can be used to restrict trusted origins for `RESOURCE_URL` |
 * | `$sce.JS`           | For JavaScript that is safe to execute in your application's context.  Currently unused.  Feel free to use it in your own directives. |
 *
 *
 * <div class="alert alert-warning">
 * Be aware that, before AngularJS 1.7.0, `a[href]` and `img[src]` used to sanitize their
 * interpolated values directly rather than rely upon {@link ng.$sce#getTrusted `$sce.getTrusted`}.
 *
 * **As of 1.7.0, this is no longer the case.**
 *
 * Now such interpolations are marked as requiring `$sce.URL` (for `a[href]`) or `$sce.MEDIA_URL`
 * (for `img[src]`), so that the sanitization happens (via `$sce.getTrusted...`) when the `$interpolate`
 * service evaluates the expressions.
 * </div>
 *
 * There are no CSS or JS context bindings in AngularJS currently, so their corresponding `$sce.trustAs`
 * functions aren't useful yet. This might evolve.
 *
 * ### Format of items in {@link ng.$sceDelegateProvider#resourceUrlWhitelist resourceUrlWhitelist}/{@link ng.$sceDelegateProvider#resourceUrlBlacklist Blacklist} <a name="resourceUrlPatternItem"></a>
 *
 *  Each element in these arrays must be one of the following:
 *
 *  - **'self'**
 *    - The special **string**, `'self'`, can be used to match against all URLs of the **same
 *      domain** as the application document using the **same protocol**.
 *  - **String** (except the special value `'self'`)
 *    - The string is matched against the full *normalized / absolute URL* of the resource
 *      being tested (substring matches are not good enough.)
 *    - There are exactly **two wildcard sequences** - `*` and `**`.  All other characters
 *      match themselves.
 *    - `*`: matches zero or more occurrences of any character other than one of the following 6
 *      characters: '`:`', '`/`', '`.`', '`?`', '`&`' and '`;`'.  It's a useful wildcard for use
 *      in a whitelist.
 *    - `**`: matches zero or more occurrences of *any* character.  As such, it's not
 *      appropriate for use in a scheme, domain, etc. as it would match too much.  (e.g.
 *      http://**.example.com/ would match http://evil.com/?ignore=.example.com/ and that might
 *      not have been the intention.)  Its usage at the very end of the path is ok.  (e.g.
 *      http://foo.example.com/templates/**).
 *  - **RegExp** (*see caveat below*)
 *    - *Caveat*:  While regular expressions are powerful and offer great flexibility,  their syntax
 *      (and all the inevitable escaping) makes them *harder to maintain*.  It's easy to
 *      accidentally introduce a bug when one updates a complex expression (imho, all regexes should
 *      have good test coverage).  For instance, the use of `.` in the regex is correct only in a
 *      small number of cases.  A `.` character in the regex used when matching the scheme or a
 *      subdomain could be matched against a `:` or literal `.` that was likely not intended.   It
 *      is highly recommended to use the string patterns and only fall back to regular expressions
 *      as a last resort.
 *    - The regular expression must be an instance of RegExp (i.e. not a string.)  It is
 *      matched against the **entire** *normalized / absolute URL* of the resource being tested
 *      (even when the RegExp did not have the `^` and `$` codes.)  In addition, any flags
 *      present on the RegExp (such as multiline, global, ignoreCase) are ignored.
 *    - If you are generating your JavaScript from some other templating engine (not
 *      recommended, e.g. in issue [#4006](https://github.com/angular/angular.js/issues/4006)),
 *      remember to escape your regular expression (and be aware that you might need more than
 *      one level of escaping depending on your templating engine and the way you interpolated
 *      the value.)  Do make use of your platform's escaping mechanism as it might be good
 *      enough before coding your own.  E.g. Ruby has
 *      [Regexp.escape(str)](http://www.ruby-doc.org/core-2.0.0/Regexp.html#method-c-escape)
 *      and Python has [re.escape](http://docs.python.org/library/re.html#re.escape).
 *      Javascript lacks a similar built in function for escaping.  Take a look at Google
 *      Closure library's [goog.string.regExpEscape(s)](
 *      http://docs.closure-library.googlecode.com/git/closure_goog_string_string.js.source.html#line962).
 *
 * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} for an example.
 *
 * ### Show me an example using SCE.
 *
 * <example module="mySceApp" deps="angular-sanitize.js" name="sce-service">
 * <file name="index.html">
 *   <div ng-controller="AppController as myCtrl">
 *     <i ng-bind-html="myCtrl.explicitlyTrustedHtml" id="explicitlyTrustedHtml"></i><br><br>
 *     <b>User comments</b><br>
 *     By default, HTML that isn't explicitly trusted (e.g. Alice's comment) is sanitized when
 *     $sanitize is available.  If $sanitize isn't available, this results in an error instead of an
 *     exploit.
 *     <div class="well">
 *       <div ng-repeat="userComment in myCtrl.userComments">
 *         <b>{{userComment.name}}</b>:
 *         <span ng-bind-html="userComment.htmlComment" class="htmlComment"></span>
 *         <br>
 *       </div>
 *     </div>
 *   </div>
 * </file>
 *
 * <file name="script.js">
 *   angular.module('mySceApp', ['ngSanitize'])
 *     .controller('AppController', ['$http', '$templateCache', '$sce',
 *       function AppController($http, $templateCache, $sce) {
 *         var self = this;
 *         $http.get('test_data.json', {cache: $templateCache}).then(function(response) {
 *           self.userComments = response.data;
 *         });
 *         self.explicitlyTrustedHtml = $sce.trustAsHtml(
 *             '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
 *             'sanitization.&quot;">Hover over this text.</span>');
 *       }]);
 * </file>
 *
 * <file name="test_data.json">
 * [
 *   { "name": "Alice",
 *     "htmlComment":
 *         "<span onmouseover='this.textContent=\"PWN3D!\"'>Is <i>anyone</i> reading this?</span>"
 *   },
 *   { "name": "Bob",
 *     "htmlComment": "<i>Yes!</i>  Am I the only other one?"
 *   }
 * ]
 * </file>
 *
 * <file name="protractor.js" type="protractor">
 *   describe('SCE doc demo', function() {
 *     it('should sanitize untrusted values', function() {
 *       expect(element.all(by.css('.htmlComment')).first().getAttribute('innerHTML'))
 *           .toBe('<span>Is <i>anyone</i> reading this?</span>');
 *     });
 *
 *     it('should NOT sanitize explicitly trusted values', function() {
 *       expect(element(by.id('explicitlyTrustedHtml')).getAttribute('innerHTML')).toBe(
 *           '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
 *           'sanitization.&quot;">Hover over this text.</span>');
 *     });
 *   });
 * </file>
 * </example>
 *
 *
 *
 * ## Can I disable SCE completely?
 *
 * Yes, you can.  However, this is strongly discouraged.  SCE gives you a lot of security benefits
 * for little coding overhead.  It will be much harder to take an SCE disabled application and
 * either secure it on your own or enable SCE at a later stage.  It might make sense to disable SCE
 * for cases where you have a lot of existing code that was written before SCE was introduced and
 * you're migrating them a module at a time. Also do note that this is an app-wide setting, so if
 * you are writing a library, you will cause security bugs applications using it.
 *
 * That said, here's how you can completely disable SCE:
 *
 * ```
 * angular.module('myAppWithSceDisabledmyApp', []).config(function($sceProvider) {
 *   // Completely disable SCE.  For demonstration purposes only!
 *   // Do not use in new projects or libraries.
 *   $sceProvider.enabled(false);
 * });
 * ```
 *
 */

function $SceProvider() {
  var enabled = true;

  /**
   * @ngdoc method
   * @name $sceProvider#enabled
   * @kind function
   *
   * @param {boolean=} value If provided, then enables/disables SCE application-wide.
   * @return {boolean} True if SCE is enabled, false otherwise.
   *
   * @description
   * Enables/disables SCE and returns the current value.
   */
  this.enabled = function(value) {
    if (arguments.length) {
      enabled = !!value;
    }
    return enabled;
  };


  /* Design notes on the default implementation for SCE.
   *
   * The API contract for the SCE delegate
   * -------------------------------------
   * The SCE delegate object must provide the following 3 methods:
   *
   * - trustAs(contextEnum, value)
   *     This method is used to tell the SCE service that the provided value is OK to use in the
   *     contexts specified by contextEnum.  It must return an object that will be accepted by
   *     getTrusted() for a compatible contextEnum and return this value.
   *
   * - valueOf(value)
   *     For values that were not produced by trustAs(), return them as is.  For values that were
   *     produced by trustAs(), return the corresponding input value to trustAs.  Basically, if
   *     trustAs is wrapping the given values into some type, this operation unwraps it when given
   *     such a value.
   *
   * - getTrusted(contextEnum, value)
   *     This function should return the value that is safe to use in the context specified by
   *     contextEnum or throw and exception otherwise.
   *
   * NOTE: This contract deliberately does NOT state that values returned by trustAs() must be
   * opaque or wrapped in some holder object.  That happens to be an implementation detail.  For
   * instance, an implementation could maintain a registry of all trusted objects by context.  In
   * such a case, trustAs() would return the same object that was passed in.  getTrusted() would
   * return the same object passed in if it was found in the registry under a compatible context or
   * throw an exception otherwise.  An implementation might only wrap values some of the time based
   * on some criteria.  getTrusted() might return a value and not throw an exception for special
   * constants or objects even if not wrapped.  All such implementations fulfill this contract.
   *
   *
   * A note on the inheritance model for SCE contexts
   * ------------------------------------------------
   * I've used inheritance and made RESOURCE_URL wrapped types a subtype of URL wrapped types.  This
   * is purely an implementation details.
   *
   * The contract is simply this:
   *
   *     getTrusted($sce.RESOURCE_URL, value) succeeding implies that getTrusted($sce.URL, value)
   *     will also succeed.
   *
   * Inheritance happens to capture this in a natural way. In some future, we may not use
   * inheritance anymore. That is OK because no code outside of sce.js and sceSpecs.js would need to
   * be aware of this detail.
   */

  this.$get = ['$parse', '$sceDelegate', function(
                $parse,   $sceDelegate) {
    // Support: IE 9-11 only
    // Prereq: Ensure that we're not running in IE<11 quirks mode.  In that mode, IE < 11 allow
    // the "expression(javascript expression)" syntax which is insecure.
    if (enabled && msie < 8) {
      throw $sceMinErr('iequirks',
        'Strict Contextual Escaping does not support Internet Explorer version < 11 in quirks ' +
        'mode.  You can fix this by adding the text <!doctype html> to the top of your HTML ' +
        'document.  See http://docs.angularjs.org/api/ng.$sce for more information.');
    }

    var sce = shallowCopy(SCE_CONTEXTS);

    /**
     * @ngdoc method
     * @name $sce#isEnabled
     * @kind function
     *
     * @return {Boolean} True if SCE is enabled, false otherwise.  If you want to set the value, you
     *     have to do it at module config time on {@link ng.$sceProvider $sceProvider}.
     *
     * @description
     * Returns a boolean indicating if SCE is enabled.
     */
    sce.isEnabled = function() {
      return enabled;
    };
    sce.trustAs = $sceDelegate.trustAs;
    sce.getTrusted = $sceDelegate.getTrusted;
    sce.valueOf = $sceDelegate.valueOf;

    if (!enabled) {
      sce.trustAs = sce.getTrusted = function(type, value) { return value; };
      sce.valueOf = identity;
    }

    /**
     * @ngdoc method
     * @name $sce#parseAs
     *
     * @description
     * Converts AngularJS {@link guide/expression expression} into a function.  This is like {@link
     * ng.$parse $parse} and is identical when the expression is a literal constant.  Otherwise, it
     * wraps the expression in a call to {@link ng.$sce#getTrusted $sce.getTrusted(*type*,
     * *result*)}
     *
     * @param {string} type The SCE context in which this result will be used.
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */
    sce.parseAs = function sceParseAs(type, expr) {
      var parsed = $parse(expr);
      if (parsed.literal && parsed.constant) {
        return parsed;
      } else {
        return $parse(expr, function(value) {
          return sce.getTrusted(type, value);
        });
      }
    };

    /**
     * @ngdoc method
     * @name $sce#trustAs
     *
     * @description
     * Delegates to {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}. As such, returns a
     * wrapped object that represents your value, and the trust you have in its safety for the given
     * context. AngularJS can then use that value as-is in bindings of the specified secure context.
     * This is used in bindings for `ng-bind-html`, `ng-include`, and most `src` attribute
     * interpolations. See {@link ng.$sce $sce} for strict contextual escaping.
     *
     * @param {string} type The context in which this value is safe for use, e.g. `$sce.URL`,
     *     `$sce.RESOURCE_URL`, `$sce.HTML`, `$sce.JS` or `$sce.CSS`.
     *
     * @param {*} value The value that that should be considered trusted.
     * @return {*} A wrapped version of value that can be used as a trusted variant of your `value`
     *     in the context you specified.
     */

    /**
     * @ngdoc method
     * @name $sce#trustAsHtml
     *
     * @description
     * Shorthand method.  `$sce.trustAsHtml(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.HTML, value)`}
     *
     * @param {*} value The value to mark as trusted for `$sce.HTML` context.
     * @return {*} A wrapped version of value that can be used as a trusted variant of your `value`
     *     in `$sce.HTML` context (like `ng-bind-html`).
     */

    /**
     * @ngdoc method
     * @name $sce#trustAsCss
     *
     * @description
     * Shorthand method.  `$sce.trustAsCss(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.CSS, value)`}
     *
     * @param {*} value The value to mark as trusted for `$sce.CSS` context.
     * @return {*} A wrapped version of value that can be used as a trusted variant
     *     of your `value` in `$sce.CSS` context. This context is currently unused, so there are
     *     almost no reasons to use this function so far.
     */

    /**
     * @ngdoc method
     * @name $sce#trustAsUrl
     *
     * @description
     * Shorthand method.  `$sce.trustAsUrl(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.URL, value)`}
     *
     * @param {*} value The value to mark as trusted for `$sce.URL` context.
     * @return {*} A wrapped version of value that can be used as a trusted variant of your `value`
     *     in `$sce.URL` context. That context is currently unused, so there are almost no reasons
     *     to use this function so far.
     */

    /**
     * @ngdoc method
     * @name $sce#trustAsResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.trustAsResourceUrl(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.RESOURCE_URL, value)`}
     *
     * @param {*} value The value to mark as trusted for `$sce.RESOURCE_URL` context.
     * @return {*} A wrapped version of value that can be used as a trusted variant of your `value`
     *     in `$sce.RESOURCE_URL` context (template URLs in `ng-include`, most `src` attribute
     *     bindings, ...)
     */

    /**
     * @ngdoc method
     * @name $sce#trustAsJs
     *
     * @description
     * Shorthand method.  `$sce.trustAsJs(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.JS, value)`}
     *
     * @param {*} value The value to mark as trusted for `$sce.JS` context.
     * @return {*} A wrapped version of value that can be used as a trusted variant of your `value`
     *     in `$sce.JS` context. That context is currently unused, so there are almost no reasons to
     *     use this function so far.
     */

    /**
     * @ngdoc method
     * @name $sce#getTrusted
     *
     * @description
     * Delegates to {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted`}.  As such,
     * takes any input, and either returns a value that's safe to use in the specified context,
     * or throws an exception. This function is aware of trusted values created by the `trustAs`
     * function and its shorthands, and when contexts are appropriate, returns the unwrapped value
     * as-is. Finally, this function can also throw when there is no way to turn `maybeTrusted` in a
     * safe value (e.g., no sanitization is available or possible.)
     *
     * @param {string} type The context in which this value is to be used.
     * @param {*} maybeTrusted The result of a prior {@link ng.$sce#trustAs
     *     `$sce.trustAs`} call, or anything else (which will not be considered trusted.)
     * @return {*} A version of the value that's safe to use in the given context, or throws an
     *     exception if this is impossible.
     */

    /**
     * @ngdoc method
     * @name $sce#getTrustedHtml
     *
     * @description
     * Shorthand method.  `$sce.getTrustedHtml(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.HTML, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @return {*} The return value of `$sce.getTrusted($sce.HTML, value)`
     */

    /**
     * @ngdoc method
     * @name $sce#getTrustedCss
     *
     * @description
     * Shorthand method.  `$sce.getTrustedCss(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.CSS, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @return {*} The return value of `$sce.getTrusted($sce.CSS, value)`
     */

    /**
     * @ngdoc method
     * @name $sce#getTrustedUrl
     *
     * @description
     * Shorthand method.  `$sce.getTrustedUrl(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.URL, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @return {*} The return value of `$sce.getTrusted($sce.URL, value)`
     */

    /**
     * @ngdoc method
     * @name $sce#getTrustedResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.getTrustedResourceUrl(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.RESOURCE_URL, value)`}
     *
     * @param {*} value The value to pass to `$sceDelegate.getTrusted`.
     * @return {*} The return value of `$sce.getTrusted($sce.RESOURCE_URL, value)`
     */

    /**
     * @ngdoc method
     * @name $sce#getTrustedJs
     *
     * @description
     * Shorthand method.  `$sce.getTrustedJs(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.JS, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @return {*} The return value of `$sce.getTrusted($sce.JS, value)`
     */

    /**
     * @ngdoc method
     * @name $sce#parseAsHtml
     *
     * @description
     * Shorthand method.  `$sce.parseAsHtml(expression string)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.HTML, value)`}
     *
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */

    /**
     * @ngdoc method
     * @name $sce#parseAsCss
     *
     * @description
     * Shorthand method.  `$sce.parseAsCss(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.CSS, value)`}
     *
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */

    /**
     * @ngdoc method
     * @name $sce#parseAsUrl
     *
     * @description
     * Shorthand method.  `$sce.parseAsUrl(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.URL, value)`}
     *
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */

    /**
     * @ngdoc method
     * @name $sce#parseAsResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.parseAsResourceUrl(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.RESOURCE_URL, value)`}
     *
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */

    /**
     * @ngdoc method
     * @name $sce#parseAsJs
     *
     * @description
     * Shorthand method.  `$sce.parseAsJs(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.JS, value)`}
     *
     * @param {string} expression String expression to compile.
     * @return {function(context, locals)} A function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the
     *      strings are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values
     *      in `context`.
     */

    // Shorthand delegations.
    var parse = sce.parseAs,
        getTrusted = sce.getTrusted,
        trustAs = sce.trustAs;

    forEach(SCE_CONTEXTS, function(enumValue, name) {
      var lName = lowercase(name);
      sce[snakeToCamel('parse_as_' + lName)] = function(expr) {
        return parse(enumValue, expr);
      };
      sce[snakeToCamel('get_trusted_' + lName)] = function(value) {
        return getTrusted(enumValue, value);
      };
      sce[snakeToCamel('trust_as_' + lName)] = function(value) {
        return trustAs(enumValue, value);
      };
    });

    return sce;
  }];
}

/* exported $SnifferProvider */

/**
 * !!! This is an undocumented "private" service !!!
 *
 * @name $sniffer
 * @requires $window
 * @requires $document
 * @this
 *
 * @property {boolean} history Does the browser support html5 history api ?
 * @property {boolean} transitions Does the browser support CSS transition events ?
 * @property {boolean} animations Does the browser support CSS animation events ?
 *
 * @description
 * This is very simple implementation of testing browser's features.
 */
function $SnifferProvider() {
  this.$get = ['$window', '$document', function($window, $document) {
    var eventSupport = {},
        // Chrome Packaged Apps are not allowed to access `history.pushState`.
        // If not sandboxed, they can be detected by the presence of `chrome.app.runtime`
        // (see https://developer.chrome.com/apps/api_index). If sandboxed, they can be detected by
        // the presence of an extension runtime ID and the absence of other Chrome runtime APIs
        // (see https://developer.chrome.com/apps/manifest/sandbox).
        // (NW.js apps have access to Chrome APIs, but do support `history`.)
        isNw = $window.nw && $window.nw.process,
        isChromePackagedApp =
            !isNw &&
            $window.chrome &&
            ($window.chrome.app && $window.chrome.app.runtime ||
                !$window.chrome.app && $window.chrome.runtime && $window.chrome.runtime.id),
        hasHistoryPushState = !isChromePackagedApp && $window.history && $window.history.pushState,
        android =
          toInt((/android (\d+)/.exec(lowercase(($window.navigator || {}).userAgent)) || [])[1]),
        boxee = /Boxee/i.test(($window.navigator || {}).userAgent),
        document = $document[0] || {},
        bodyStyle = document.body && document.body.style,
        transitions = false,
        animations = false;

    if (bodyStyle) {
      // Support: Android <5, Blackberry Browser 10, default Chrome in Android 4.4.x
      // Mentioned browsers need a -webkit- prefix for transitions & animations.
      transitions = !!('transition' in bodyStyle || 'webkitTransition' in bodyStyle);
      animations = !!('animation' in bodyStyle || 'webkitAnimation' in bodyStyle);
    }


    return {
      // Android has history.pushState, but it does not update location correctly
      // so let's not use the history API at all.
      // http://code.google.com/p/android/issues/detail?id=17471
      // https://github.com/angular/angular.js/issues/904

      // older webkit browser (533.9) on Boxee box has exactly the same problem as Android has
      // so let's not use the history API also
      // We are purposefully using `!(android < 4)` to cover the case when `android` is undefined
      history: !!(hasHistoryPushState && !(android < 4) && !boxee),
      hasEvent: function(event) {
        // Support: IE 9-11 only
        // IE9 implements 'input' event it's so fubared that we rather pretend that it doesn't have
        // it. In particular the event is not fired when backspace or delete key are pressed or
        // when cut operation is performed.
        // IE10+ implements 'input' event but it erroneously fires under various situations,
        // e.g. when placeholder changes, or a form is focused.
        if (event === 'input' && msie) return false;

        if (isUndefined(eventSupport[event])) {
          var divElm = document.createElement('div');
          eventSupport[event] = 'on' + event in divElm;
        }

        return eventSupport[event];
      },
      csp: csp(),
      transitions: transitions,
      animations: animations,
      android: android
    };
  }];
}

/**
 * ! This is a private undocumented service !
 *
 * @name $$taskTrackerFactory
 * @description
 * A function to create `TaskTracker` instances.
 *
 * A `TaskTracker` can keep track of pending tasks (grouped by type) and can notify interested
 * parties when all pending tasks (or tasks of a specific type) have been completed.
 *
 * @param {$log} log - A logger instance (such as `$log`). Used to log error during callback
 *     execution.
 *
 * @this
 */
function $$TaskTrackerFactoryProvider() {
  this.$get = valueFn(function(log) { return new TaskTracker(log); });
}

function TaskTracker(log) {
  var self = this;
  var taskCounts = {};
  var taskCallbacks = [];

  var ALL_TASKS_TYPE = self.ALL_TASKS_TYPE = '$$all$$';
  var DEFAULT_TASK_TYPE = self.DEFAULT_TASK_TYPE = '$$default$$';

  /**
   * Execute the specified function and decrement the appropriate `taskCounts` counter.
   * If the counter reaches 0, all corresponding `taskCallbacks` are executed.
   *
   * @param {Function} fn - The function to execute.
   * @param {string=} [taskType=DEFAULT_TASK_TYPE] - The type of task that is being completed.
   */
  self.completeTask = completeTask;

  /**
   * Increase the task count for the specified task type (or the default task type if non is
   * specified).
   *
   * @param {string=} [taskType=DEFAULT_TASK_TYPE] - The type of task whose count will be increased.
   */
  self.incTaskCount = incTaskCount;

  /**
   * Execute the specified callback when all pending tasks have been completed.
   *
   * If there are no pending tasks, the callback is executed immediately. You can optionally limit
   * the tasks that will be waited for to a specific type, by passing a `taskType`.
   *
   * @param {function} callback - The function to call when there are no pending tasks.
   * @param {string=} [taskType=ALL_TASKS_TYPE] - The type of tasks that will be waited for.
   */
  self.notifyWhenNoPendingTasks = notifyWhenNoPendingTasks;

  function completeTask(fn, taskType) {
    taskType = taskType || DEFAULT_TASK_TYPE;

    try {
      fn();
    } finally {
      decTaskCount(taskType);

      var countForType = taskCounts[taskType];
      var countForAll = taskCounts[ALL_TASKS_TYPE];

      // If at least one of the queues (`ALL_TASKS_TYPE` or `taskType`) is empty, run callbacks.
      if (!countForAll || !countForType) {
        var getNextCallback = !countForAll ? getLastCallback : getLastCallbackForType;
        var nextCb;

        while ((nextCb = getNextCallback(taskType))) {
          try {
            nextCb();
          } catch (e) {
            log.error(e);
          }
        }
      }
    }
  }

  function decTaskCount(taskType) {
    taskType = taskType || DEFAULT_TASK_TYPE;
    if (taskCounts[taskType]) {
      taskCounts[taskType]--;
      taskCounts[ALL_TASKS_TYPE]--;
    }
  }

  function getLastCallback() {
    var cbInfo = taskCallbacks.pop();
    return cbInfo && cbInfo.cb;
  }

  function getLastCallbackForType(taskType) {
    for (var i = taskCallbacks.length - 1; i >= 0; --i) {
      var cbInfo = taskCallbacks[i];
      if (cbInfo.type === taskType) {
        taskCallbacks.splice(i, 1);
        return cbInfo.cb;
      }
    }
  }

  function incTaskCount(taskType) {
    taskType = taskType || DEFAULT_TASK_TYPE;
    taskCounts[taskType] = (taskCounts[taskType] || 0) + 1;
    taskCounts[ALL_TASKS_TYPE] = (taskCounts[ALL_TASKS_TYPE] || 0) + 1;
  }

  function notifyWhenNoPendingTasks(callback, taskType) {
    taskType = taskType || ALL_TASKS_TYPE;
    if (!taskCounts[taskType]) {
      callback();
    } else {
      taskCallbacks.push({type: taskType, cb: callback});
    }
  }
}

var $templateRequestMinErr = minErr('$templateRequest');

/**
 * @ngdoc provider
 * @name $templateRequestProvider
 * @this
 *
 * @description
 * Used to configure the options passed to the {@link $http} service when making a template request.
 *
 * For example, it can be used for specifying the "Accept" header that is sent to the server, when
 * requesting a template.
 */
function $TemplateRequestProvider() {

  var httpOptions;

  /**
   * @ngdoc method
   * @name $templateRequestProvider#httpOptions
   * @description
   * The options to be passed to the {@link $http} service when making the request.
   * You can use this to override options such as the "Accept" header for template requests.
   *
   * The {@link $templateRequest} will set the `cache` and the `transformResponse` properties of the
   * options if not overridden here.
   *
   * @param {string=} value new value for the {@link $http} options.
   * @returns {string|self} Returns the {@link $http} options when used as getter and self if used as setter.
   */
  this.httpOptions = function(val) {
    if (val) {
      httpOptions = val;
      return this;
    }
    return httpOptions;
  };

  /**
   * @ngdoc service
   * @name $templateRequest
   *
   * @description
   * The `$templateRequest` service runs security checks then downloads the provided template using
   * `$http` and, upon success, stores the contents inside of `$templateCache`. If the HTTP request
   * fails or the response data of the HTTP request is empty, a `$compile` error will be thrown (the
   * exception can be thwarted by setting the 2nd parameter of the function to true). Note that the
   * contents of `$templateCache` are trusted, so the call to `$sce.getTrustedUrl(tpl)` is omitted
   * when `tpl` is of type string and `$templateCache` has the matching entry.
   *
   * If you want to pass custom options to the `$http` service, such as setting the Accept header you
   * can configure this via {@link $templateRequestProvider#httpOptions}.
   *
   * `$templateRequest` is used internally by {@link $compile}, {@link ngRoute.$route}, and directives such
   * as {@link ngInclude} to download and cache templates.
   *
   * 3rd party modules should use `$templateRequest` if their services or directives are loading
   * templates.
   *
   * @param {string|TrustedResourceUrl} tpl The HTTP request template URL
   * @param {boolean=} ignoreRequestError Whether or not to ignore the exception when the request fails or the template is empty
   *
   * @return {Promise} a promise for the HTTP response data of the given URL.
   *
   * @property {number} totalPendingRequests total amount of pending template requests being downloaded.
   */
  this.$get = ['$exceptionHandler', '$templateCache', '$http', '$q', '$sce',
    function($exceptionHandler, $templateCache, $http, $q, $sce) {

      function handleRequestFn(tpl, ignoreRequestError) {
        handleRequestFn.totalPendingRequests++;

        // We consider the template cache holds only trusted templates, so
        // there's no need to go through whitelisting again for keys that already
        // are included in there. This also makes AngularJS accept any script
        // directive, no matter its name. However, we still need to unwrap trusted
        // types.
        if (!isString(tpl) || isUndefined($templateCache.get(tpl))) {
          tpl = $sce.getTrustedResourceUrl(tpl);
        }

        var transformResponse = $http.defaults && $http.defaults.transformResponse;

        if (isArray(transformResponse)) {
          transformResponse = transformResponse.filter(function(transformer) {
            return transformer !== defaultHttpResponseTransform;
          });
        } else if (transformResponse === defaultHttpResponseTransform) {
          transformResponse = null;
        }

        return $http.get(tpl, extend({
            cache: $templateCache,
            transformResponse: transformResponse
          }, httpOptions))
          .finally(function() {
            handleRequestFn.totalPendingRequests--;
          })
          .then(function(response) {
            return $templateCache.put(tpl, response.data);
          }, handleError);

        function handleError(resp) {
          if (!ignoreRequestError) {
            resp = $templateRequestMinErr('tpload',
                'Failed to load template: {0} (HTTP status: {1} {2})',
                tpl, resp.status, resp.statusText);

            $exceptionHandler(resp);
          }

          return $q.reject(resp);
        }
      }

      handleRequestFn.totalPendingRequests = 0;

      return handleRequestFn;
    }
  ];
}

/** @this */
function $$TestabilityProvider() {
  this.$get = ['$rootScope', '$browser', '$location',
       function($rootScope,   $browser,   $location) {

    /**
     * @name $testability
     *
     * @description
     * The private $$testability service provides a collection of methods for use when debugging
     * or by automated test and debugging tools.
     */
    var testability = {};

    /**
     * @name $$testability#findBindings
     *
     * @description
     * Returns an array of elements that are bound (via ng-bind or {{}})
     * to expressions matching the input.
     *
     * @param {Element} element The element root to search from.
     * @param {string} expression The binding expression to match.
     * @param {boolean} opt_exactMatch If true, only returns exact matches
     *     for the expression. Filters and whitespace are ignored.
     */
    testability.findBindings = function(element, expression, opt_exactMatch) {
      var bindings = element.getElementsByClassName('ng-binding');
      var matches = [];
      forEach(bindings, function(binding) {
        var dataBinding = angular.element(binding).data('$binding');
        if (dataBinding) {
          forEach(dataBinding, function(bindingName) {
            if (opt_exactMatch) {
              var matcher = new RegExp('(^|\\s)' + escapeForRegexp(expression) + '(\\s|\\||$)');
              if (matcher.test(bindingName)) {
                matches.push(binding);
              }
            } else {
              if (bindingName.indexOf(expression) !== -1) {
                matches.push(binding);
              }
            }
          });
        }
      });
      return matches;
    };

    /**
     * @name $$testability#findModels
     *
     * @description
     * Returns an array of elements that are two-way found via ng-model to
     * expressions matching the input.
     *
     * @param {Element} element The element root to search from.
     * @param {string} expression The model expression to match.
     * @param {boolean} opt_exactMatch If true, only returns exact matches
     *     for the expression.
     */
    testability.findModels = function(element, expression, opt_exactMatch) {
      var prefixes = ['ng-', 'data-ng-', 'ng\\:'];
      for (var p = 0; p < prefixes.length; ++p) {
        var attributeEquals = opt_exactMatch ? '=' : '*=';
        var selector = '[' + prefixes[p] + 'model' + attributeEquals + '"' + expression + '"]';
        var elements = element.querySelectorAll(selector);
        if (elements.length) {
          return elements;
        }
      }
    };

    /**
     * @name $$testability#getLocation
     *
     * @description
     * Shortcut for getting the location in a browser agnostic way. Returns
     *     the path, search, and hash. (e.g. /path?a=b#hash)
     */
    testability.getLocation = function() {
      return $location.url();
    };

    /**
     * @name $$testability#setLocation
     *
     * @description
     * Shortcut for navigating to a location without doing a full page reload.
     *
     * @param {string} url The location url (path, search and hash,
     *     e.g. /path?a=b#hash) to go to.
     */
    testability.setLocation = function(url) {
      if (url !== $location.url()) {
        $location.url(url);
        $rootScope.$digest();
      }
    };

    /**
     * @name $$testability#whenStable
     *
     * @description
     * Calls the callback when all pending tasks are completed.
     *
     * Types of tasks waited for include:
     * - Pending timeouts (via {@link $timeout}).
     * - Pending HTTP requests (via {@link $http}).
     * - In-progress route transitions (via {@link $route}).
     * - Pending tasks scheduled via {@link $rootScope#$applyAsync}.
     * - Pending tasks scheduled via {@link $rootScope#$evalAsync}.
     *   These include tasks scheduled via `$evalAsync()` indirectly (such as {@link $q} promises).
     *
     * @param {function} callback
     */
    testability.whenStable = function(callback) {
      $browser.notifyWhenNoOutstandingRequests(callback);
    };

    return testability;
  }];
}

var $timeoutMinErr = minErr('$timeout');

/** @this */
function $TimeoutProvider() {
  this.$get = ['$rootScope', '$browser', '$q', '$$q', '$exceptionHandler',
       function($rootScope,   $browser,   $q,   $$q,   $exceptionHandler) {

    var deferreds = {};


    /**
     * @ngdoc service
     * @name $timeout
     *
     * @description
     * AngularJS's wrapper for `window.setTimeout`. The `fn` function is wrapped into a try/catch
     * block and delegates any exceptions to
     * {@link ng.$exceptionHandler $exceptionHandler} service.
     *
     * The return value of calling `$timeout` is a promise, which will be resolved when
     * the delay has passed and the timeout function, if provided, is executed.
     *
     * To cancel a timeout request, call `$timeout.cancel(promise)`.
     *
     * In tests you can use {@link ngMock.$timeout `$timeout.flush()`} to
     * synchronously flush the queue of deferred functions.
     *
     * If you only want a promise that will be resolved after some specified delay
     * then you can call `$timeout` without the `fn` function.
     *
     * @param {function()=} fn A function, whose execution should be delayed.
     * @param {number=} [delay=0] Delay in milliseconds.
     * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
     *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
     * @param {...*=} Pass additional parameters to the executed function.
     * @returns {Promise} Promise that will be resolved when the timeout is reached. The promise
     *   will be resolved with the return value of the `fn` function.
     *
     */
    function timeout(fn, delay, invokeApply) {
      if (!isFunction(fn)) {
        invokeApply = delay;
        delay = fn;
        fn = noop;
      }

      var args = sliceArgs(arguments, 3),
          skipApply = (isDefined(invokeApply) && !invokeApply),
          deferred = (skipApply ? $$q : $q).defer(),
          promise = deferred.promise,
          timeoutId;

      timeoutId = $browser.defer(function() {
        try {
          deferred.resolve(fn.apply(null, args));
        } catch (e) {
          deferred.reject(e);
          $exceptionHandler(e);
        } finally {
          delete deferreds[promise.$$timeoutId];
        }

        if (!skipApply) $rootScope.$apply();
      }, delay, '$timeout');

      promise.$$timeoutId = timeoutId;
      deferreds[timeoutId] = deferred;

      return promise;
    }


    /**
     * @ngdoc method
     * @name $timeout#cancel
     *
     * @description
     * Cancels a task associated with the `promise`. As a result of this, the promise will be
     * resolved with a rejection.
     *
     * @param {Promise=} promise Promise returned by the `$timeout` function.
     * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
     *   canceled.
     */
    timeout.cancel = function(promise) {
      if (!promise) return false;

      if (!promise.hasOwnProperty('$$timeoutId')) {
        throw $timeoutMinErr('badprom',
            '`$timeout.cancel()` called with a promise that was not generated by `$timeout()`.');
      }

      if (!deferreds.hasOwnProperty(promise.$$timeoutId)) return false;

      var id = promise.$$timeoutId;
      var deferred = deferreds[id];

      // Timeout cancels should not report an unhandled promise.
      markQExceptionHandled(deferred.promise);
      deferred.reject('canceled');
      delete deferreds[id];

      return $browser.defer.cancel(id);
    };

    return timeout;
  }];
}

// NOTE:  The usage of window and document instead of $window and $document here is
// deliberate.  This service depends on the specific behavior of anchor nodes created by the
// browser (resolving and parsing URLs) that is unlikely to be provided by mock objects and
// cause us to break tests.  In addition, when the browser resolves a URL for XHR, it
// doesn't know about mocked locations and resolves URLs to the real document - which is
// exactly the behavior needed here.  There is little value is mocking these out for this
// service.
var urlParsingNode = window.document.createElement('a');
var originUrl = urlResolve(window.location.href);
var baseUrlParsingNode;

urlParsingNode.href = 'http://[::1]';

// Support: IE 9-11 only, Edge 16-17 only (fixed in 18 Preview)
// IE/Edge don't wrap IPv6 addresses' hostnames in square brackets
// when parsed out of an anchor element.
var ipv6InBrackets = urlParsingNode.hostname === '[::1]';

/**
 *
 * Implementation Notes for non-IE browsers
 * ----------------------------------------
 * Assigning a URL to the href property of an anchor DOM node, even one attached to the DOM,
 * results both in the normalizing and parsing of the URL.  Normalizing means that a relative
 * URL will be resolved into an absolute URL in the context of the application document.
 * Parsing means that the anchor node's host, hostname, protocol, port, pathname and related
 * properties are all populated to reflect the normalized URL.  This approach has wide
 * compatibility - Safari 1+, Mozilla 1+ etc.  See
 * http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
 *
 * Implementation Notes for IE
 * ---------------------------
 * IE <= 10 normalizes the URL when assigned to the anchor node similar to the other
 * browsers.  However, the parsed components will not be set if the URL assigned did not specify
 * them.  (e.g. if you assign a.href = "foo", then a.protocol, a.host, etc. will be empty.)  We
 * work around that by performing the parsing in a 2nd step by taking a previously normalized
 * URL (e.g. by assigning to a.href) and assigning it a.href again.  This correctly populates the
 * properties such as protocol, hostname, port, etc.
 *
 * References:
 *   http://developer.mozilla.org/en-US/docs/Web/API/HTMLAnchorElement
 *   http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
 *   http://url.spec.whatwg.org/#urlutils
 *   https://github.com/angular/angular.js/pull/2902
 *   http://james.padolsey.com/javascript/parsing-urls-with-the-dom/
 *
 * @kind function
 * @param {string|object} url The URL to be parsed. If `url` is not a string, it will be returned
 *     unchanged.
 * @description Normalizes and parses a URL.
 * @returns {object} Returns the normalized URL as a dictionary.
 *
 *   | member name   | Description                                                            |
 *   |---------------|------------------------------------------------------------------------|
 *   | href          | A normalized version of the provided URL if it was not an absolute URL |
 *   | protocol      | The protocol without the trailing colon                                |
 *   | host          | The host and port (if the port is non-default) of the normalizedUrl    |
 *   | search        | The search params, minus the question mark                             |
 *   | hash          | The hash string, minus the hash symbol                                 |
 *   | hostname      | The hostname                                                           |
 *   | port          | The port, without ":"                                                  |
 *   | pathname      | The pathname, beginning with "/"                                       |
 *
 */
function urlResolve(url) {
  if (!isString(url)) return url;

  var href = url;

  // Support: IE 9-11 only
  if (msie) {
    // Normalize before parse.  Refer Implementation Notes on why this is
    // done in two steps on IE.
    urlParsingNode.setAttribute('href', href);
    href = urlParsingNode.href;
  }

  urlParsingNode.setAttribute('href', href);

  var hostname = urlParsingNode.hostname;

  if (!ipv6InBrackets && hostname.indexOf(':') > -1) {
    hostname = '[' + hostname + ']';
  }

  return {
    href: urlParsingNode.href,
    protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
    host: urlParsingNode.host,
    search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
    hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
    hostname: hostname,
    port: urlParsingNode.port,
    pathname: (urlParsingNode.pathname.charAt(0) === '/')
      ? urlParsingNode.pathname
      : '/' + urlParsingNode.pathname
  };
}

/**
 * Parse a request URL and determine whether this is a same-origin request as the application
 * document.
 *
 * @param {string|object} requestUrl The url of the request as a string that will be resolved
 * or a parsed URL object.
 * @returns {boolean} Whether the request is for the same origin as the application document.
 */
function urlIsSameOrigin(requestUrl) {
  return urlsAreSameOrigin(requestUrl, originUrl);
}

/**
 * Parse a request URL and determine whether it is same-origin as the current document base URL.
 *
 * Note: The base URL is usually the same as the document location (`location.href`) but can
 * be overriden by using the `<base>` tag.
 *
 * @param {string|object} requestUrl The url of the request as a string that will be resolved
 * or a parsed URL object.
 * @returns {boolean} Whether the URL is same-origin as the document base URL.
 */
function urlIsSameOriginAsBaseUrl(requestUrl) {
  return urlsAreSameOrigin(requestUrl, getBaseUrl());
}

/**
 * Create a function that can check a URL's origin against a list of allowed/whitelisted origins.
 * The current location's origin is implicitly trusted.
 *
 * @param {string[]} whitelistedOriginUrls - A list of URLs (strings), whose origins are trusted.
 *
 * @returns {Function} - A function that receives a URL (string or parsed URL object) and returns
 *     whether it is of an allowed origin.
 */
function urlIsAllowedOriginFactory(whitelistedOriginUrls) {
  var parsedAllowedOriginUrls = [originUrl].concat(whitelistedOriginUrls.map(urlResolve));

  /**
   * Check whether the specified URL (string or parsed URL object) has an origin that is allowed
   * based on a list of whitelisted-origin URLs. The current location's origin is implicitly
   * trusted.
   *
   * @param {string|Object} requestUrl - The URL to be checked (provided as a string that will be
   *     resolved or a parsed URL object).
   *
   * @returns {boolean} - Whether the specified URL is of an allowed origin.
   */
  return function urlIsAllowedOrigin(requestUrl) {
    var parsedUrl = urlResolve(requestUrl);
    return parsedAllowedOriginUrls.some(urlsAreSameOrigin.bind(null, parsedUrl));
  };
}

/**
 * Determine if two URLs share the same origin.
 *
 * @param {string|Object} url1 - First URL to compare as a string or a normalized URL in the form of
 *     a dictionary object returned by `urlResolve()`.
 * @param {string|object} url2 - Second URL to compare as a string or a normalized URL in the form
 *     of a dictionary object returned by `urlResolve()`.
 *
 * @returns {boolean} - True if both URLs have the same origin, and false otherwise.
 */
function urlsAreSameOrigin(url1, url2) {
  url1 = urlResolve(url1);
  url2 = urlResolve(url2);

  return (url1.protocol === url2.protocol &&
          url1.host === url2.host);
}

/**
 * Returns the current document base URL.
 * @returns {string}
 */
function getBaseUrl() {
  if (window.document.baseURI) {
    return window.document.baseURI;
  }

  // `document.baseURI` is available everywhere except IE
  if (!baseUrlParsingNode) {
    baseUrlParsingNode = window.document.createElement('a');
    baseUrlParsingNode.href = '.';

    // Work-around for IE bug described in Implementation Notes. The fix in `urlResolve()` is not
    // suitable here because we need to track changes to the base URL.
    baseUrlParsingNode = baseUrlParsingNode.cloneNode(false);
  }
  return baseUrlParsingNode.href;
}

/**
 * @ngdoc service
 * @name $window
 * @this
 *
 * @description
 * A reference to the browser's `window` object. While `window`
 * is globally available in JavaScript, it causes testability problems, because
 * it is a global variable. In AngularJS we always refer to it through the
 * `$window` service, so it may be overridden, removed or mocked for testing.
 *
 * Expressions, like the one defined for the `ngClick` directive in the example
 * below, are evaluated with respect to the current scope.  Therefore, there is
 * no risk of inadvertently coding in a dependency on a global value in such an
 * expression.
 *
 * @example
   <example module="windowExample" name="window-service">
     <file name="index.html">
       <script>
         angular.module('windowExample', [])
           .controller('ExampleController', ['$scope', '$window', function($scope, $window) {
             $scope.greeting = 'Hello, World!';
             $scope.doGreeting = function(greeting) {
               $window.alert(greeting);
             };
           }]);
       </script>
       <div ng-controller="ExampleController">
         <input type="text" ng-model="greeting" aria-label="greeting" />
         <button ng-click="doGreeting(greeting)">ALERT</button>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
      it('should display the greeting in the input box', function() {
       element(by.model('greeting')).sendKeys('Hello, E2E Tests');
       // If we click the button it will block the test runner
       // element(':button').click();
      });
     </file>
   </example>
 */
function $WindowProvider() {
  this.$get = valueFn(window);
}

/**
 * @name $$cookieReader
 * @requires $document
 *
 * @description
 * This is a private service for reading cookies used by $http and ngCookies
 *
 * @return {Object} a key/value map of the current cookies
 */
function $$CookieReader($document) {
  var rawDocument = $document[0] || {};
  var lastCookies = {};
  var lastCookieString = '';

  function safeGetCookie(rawDocument) {
    try {
      return rawDocument.cookie || '';
    } catch (e) {
      return '';
    }
  }

  function safeDecodeURIComponent(str) {
    try {
      return decodeURIComponent(str);
    } catch (e) {
      return str;
    }
  }

  return function() {
    var cookieArray, cookie, i, index, name;
    var currentCookieString = safeGetCookie(rawDocument);

    if (currentCookieString !== lastCookieString) {
      lastCookieString = currentCookieString;
      cookieArray = lastCookieString.split('; ');
      lastCookies = {};

      for (i = 0; i < cookieArray.length; i++) {
        cookie = cookieArray[i];
        index = cookie.indexOf('=');
        if (index > 0) { //ignore nameless cookies
          name = safeDecodeURIComponent(cookie.substring(0, index));
          // the first value that is seen for a cookie is the most
          // specific one.  values for the same cookie name that
          // follow are for less specific paths.
          if (isUndefined(lastCookies[name])) {
            lastCookies[name] = safeDecodeURIComponent(cookie.substring(index + 1));
          }
        }
      }
    }
    return lastCookies;
  };
}

$$CookieReader.$inject = ['$document'];

/** @this */
function $$CookieReaderProvider() {
  this.$get = $$CookieReader;
}

/* global currencyFilter: true,
 dateFilter: true,
 filterFilter: true,
 jsonFilter: true,
 limitToFilter: true,
 lowercaseFilter: true,
 numberFilter: true,
 orderByFilter: true,
 uppercaseFilter: true,
 */

/**
 * @ngdoc provider
 * @name $filterProvider
 * @description
 *
 * Filters are just functions which transform input to an output. However filters need to be
 * Dependency Injected. To achieve this a filter definition consists of a factory function which is
 * annotated with dependencies and is responsible for creating a filter function.
 *
 * <div class="alert alert-warning">
 * **Note:** Filter names must be valid AngularJS {@link expression} identifiers, such as `uppercase` or `orderBy`.
 * Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
 * your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
 * (`myapp_subsection_filterx`).
 * </div>
 *
 * ```js
 *   // Filter registration
 *   function MyModule($provide, $filterProvider) {
 *     // create a service to demonstrate injection (not always needed)
 *     $provide.value('greet', function(name){
 *       return 'Hello ' + name + '!';
 *     });
 *
 *     // register a filter factory which uses the
 *     // greet service to demonstrate DI.
 *     $filterProvider.register('greet', function(greet){
 *       // return the filter function which uses the greet service
 *       // to generate salutation
 *       return function(text) {
 *         // filters need to be forgiving so check input validity
 *         return text && greet(text) || text;
 *       };
 *     });
 *   }
 * ```
 *
 * The filter function is registered with the `$injector` under the filter name suffix with
 * `Filter`.
 *
 * ```js
 *   it('should be the same instance', inject(
 *     function($filterProvider) {
 *       $filterProvider.register('reverse', function(){
 *         return ...;
 *       });
 *     },
 *     function($filter, reverseFilter) {
 *       expect($filter('reverse')).toBe(reverseFilter);
 *     });
 * ```
 *
 *
 * For more information about how AngularJS filters work, and how to create your own filters, see
 * {@link guide/filter Filters} in the AngularJS Developer Guide.
 */

/**
 * @ngdoc service
 * @name $filter
 * @kind function
 * @description
 * Filters are used for formatting data displayed to the user.
 *
 * They can be used in view templates, controllers or services. AngularJS comes
 * with a collection of [built-in filters](api/ng/filter), but it is easy to
 * define your own as well.
 *
 * The general syntax in templates is as follows:
 *
 * ```html
 * {{ expression [| filter_name[:parameter_value] ... ] }}
 * ```
 *
 * @param {String} name Name of the filter function to retrieve
 * @return {Function} the filter function
 * @example
   <example name="$filter" module="filterExample">
     <file name="index.html">
       <div ng-controller="MainCtrl">
        <h3>{{ originalText }}</h3>
        <h3>{{ filteredText }}</h3>
       </div>
     </file>

     <file name="script.js">
      angular.module('filterExample', [])
      .controller('MainCtrl', function($scope, $filter) {
        $scope.originalText = 'hello';
        $scope.filteredText = $filter('uppercase')($scope.originalText);
      });
     </file>
   </example>
  */
$FilterProvider.$inject = ['$provide'];
/** @this */
function $FilterProvider($provide) {
  var suffix = 'Filter';

  /**
   * @ngdoc method
   * @name $filterProvider#register
   * @param {string|Object} name Name of the filter function, or an object map of filters where
   *    the keys are the filter names and the values are the filter factories.
   *
   *    <div class="alert alert-warning">
   *    **Note:** Filter names must be valid AngularJS {@link expression} identifiers, such as `uppercase` or `orderBy`.
   *    Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
   *    your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
   *    (`myapp_subsection_filterx`).
   *    </div>
    * @param {Function} factory If the first argument was a string, a factory function for the filter to be registered.
   * @returns {Object} Registered filter instance, or if a map of filters was provided then a map
   *    of the registered filter instances.
   */
  function register(name, factory) {
    if (isObject(name)) {
      var filters = {};
      forEach(name, function(filter, key) {
        filters[key] = register(key, filter);
      });
      return filters;
    } else {
      return $provide.factory(name + suffix, factory);
    }
  }
  this.register = register;

  this.$get = ['$injector', function($injector) {
    return function(name) {
      return $injector.get(name + suffix);
    };
  }];

  ////////////////////////////////////////

  /* global
    currencyFilter: false,
    dateFilter: false,
    filterFilter: false,
    jsonFilter: false,
    limitToFilter: false,
    lowercaseFilter: false,
    numberFilter: false,
    orderByFilter: false,
    uppercaseFilter: false
  */

  register('currency', currencyFilter);
  register('date', dateFilter);
  register('filter', filterFilter);
  register('json', jsonFilter);
  register('limitTo', limitToFilter);
  register('lowercase', lowercaseFilter);
  register('number', numberFilter);
  register('orderBy', orderByFilter);
  register('uppercase', uppercaseFilter);
}

/**
 * @ngdoc filter
 * @name filter
 * @kind function
 *
 * @description
 * Selects a subset of items from `array` and returns it as a new array.
 *
 * @param {Array} array The source array.
 * <div class="alert alert-info">
 *   **Note**: If the array contains objects that reference themselves, filtering is not possible.
 * </div>
 * @param {string|Object|function()} expression The predicate to be used for selecting items from
 *   `array`.
 *
 *   Can be one of:
 *
 *   - `string`: The string is used for matching against the contents of the `array`. All strings or
 *     objects with string properties in `array` that match this string will be returned. This also
 *     applies to nested object properties.
 *     The predicate can be negated by prefixing the string with `!`.
 *
 *   - `Object`: A pattern object can be used to filter specific properties on objects contained
 *     by `array`. For example `{name:"M", phone:"1"}` predicate will return an array of items
 *     which have property `name` containing "M" and property `phone` containing "1". A special
 *     property name (`$` by default) can be used (e.g. as in `{$: "text"}`) to accept a match
 *     against any property of the object or its nested object properties. That's equivalent to the
 *     simple substring match with a `string` as described above. The special property name can be
 *     overwritten, using the `anyPropertyKey` parameter.
 *     The predicate can be negated by prefixing the string with `!`.
 *     For example `{name: "!M"}` predicate will return an array of items which have property `name`
 *     not containing "M".
 *
 *     Note that a named property will match properties on the same level only, while the special
 *     `$` property will match properties on the same level or deeper. E.g. an array item like
 *     `{name: {first: 'John', last: 'Doe'}}` will **not** be matched by `{name: 'John'}`, but
 *     **will** be matched by `{$: 'John'}`.
 *
 *   - `function(value, index, array)`: A predicate function can be used to write arbitrary filters.
 *     The function is called for each element of the array, with the element, its index, and
 *     the entire array itself as arguments.
 *
 *     The final result is an array of those elements that the predicate returned true for.
 *
 * @param {function(actual, expected)|true|false} [comparator] Comparator which is used in
 *     determining if values retrieved using `expression` (when it is not a function) should be
 *     considered a match based on the expected value (from the filter expression) and actual
 *     value (from the object in the array).
 *
 *   Can be one of:
 *
 *   - `function(actual, expected)`:
 *     The function will be given the object value and the predicate value to compare and
 *     should return true if both values should be considered equal.
 *
 *   - `true`: A shorthand for `function(actual, expected) { return angular.equals(actual, expected)}`.
 *     This is essentially strict comparison of expected and actual.
 *
 *   - `false`: A short hand for a function which will look for a substring match in a case
 *     insensitive way. Primitive values are converted to strings. Objects are not compared against
 *     primitives, unless they have a custom `toString` method (e.g. `Date` objects).
 *
 *
 *   Defaults to `false`.
 *
 * @param {string} [anyPropertyKey] The special property name that matches against any property.
 *     By default `$`.
 *
 * @example
   <example name="filter-filter">
     <file name="index.html">
       <div ng-init="friends = [{name:'John', phone:'555-1276'},
                                {name:'Mary', phone:'800-BIG-MARY'},
                                {name:'Mike', phone:'555-4321'},
                                {name:'Adam', phone:'555-5678'},
                                {name:'Julie', phone:'555-8765'},
                                {name:'Juliette', phone:'555-5678'}]"></div>

       <label>Search: <input ng-model="searchText"></label>
       <table id="searchTextResults">
         <tr><th>Name</th><th>Phone</th></tr>
         <tr ng-repeat="friend in friends | filter:searchText">
           <td>{{friend.name}}</td>
           <td>{{friend.phone}}</td>
         </tr>
       </table>
       <hr>
       <label>Any: <input ng-model="search.$"></label> <br>
       <label>Name only <input ng-model="search.name"></label><br>
       <label>Phone only <input ng-model="search.phone"></label><br>
       <label>Equality <input type="checkbox" ng-model="strict"></label><br>
       <table id="searchObjResults">
         <tr><th>Name</th><th>Phone</th></tr>
         <tr ng-repeat="friendObj in friends | filter:search:strict">
           <td>{{friendObj.name}}</td>
           <td>{{friendObj.phone}}</td>
         </tr>
       </table>
     </file>
     <file name="protractor.js" type="protractor">
       var expectFriendNames = function(expectedNames, key) {
         element.all(by.repeater(key + ' in friends').column(key + '.name')).then(function(arr) {
           arr.forEach(function(wd, i) {
             expect(wd.getText()).toMatch(expectedNames[i]);
           });
         });
       };

       it('should search across all fields when filtering with a string', function() {
         var searchText = element(by.model('searchText'));
         searchText.clear();
         searchText.sendKeys('m');
         expectFriendNames(['Mary', 'Mike', 'Adam'], 'friend');

         searchText.clear();
         searchText.sendKeys('76');
         expectFriendNames(['John', 'Julie'], 'friend');
       });

       it('should search in specific fields when filtering with a predicate object', function() {
         var searchAny = element(by.model('search.$'));
         searchAny.clear();
         searchAny.sendKeys('i');
         expectFriendNames(['Mary', 'Mike', 'Julie', 'Juliette'], 'friendObj');
       });
       it('should use a equal comparison when comparator is true', function() {
         var searchName = element(by.model('search.name'));
         var strict = element(by.model('strict'));
         searchName.clear();
         searchName.sendKeys('Julie');
         strict.click();
         expectFriendNames(['Julie'], 'friendObj');
       });
     </file>
   </example>
 */

function filterFilter() {
  return function(array, expression, comparator, anyPropertyKey) {
    if (!isArrayLike(array)) {
      if (array == null) {
        return array;
      } else {
        throw minErr('filter')('notarray', 'Expected array but received: {0}', array);
      }
    }

    anyPropertyKey = anyPropertyKey || '$';
    var expressionType = getTypeForFilter(expression);
    var predicateFn;
    var matchAgainstAnyProp;

    switch (expressionType) {
      case 'function':
        predicateFn = expression;
        break;
      case 'boolean':
      case 'null':
      case 'number':
      case 'string':
        matchAgainstAnyProp = true;
        // falls through
      case 'object':
        predicateFn = createPredicateFn(expression, comparator, anyPropertyKey, matchAgainstAnyProp);
        break;
      default:
        return array;
    }

    return Array.prototype.filter.call(array, predicateFn);
  };
}

// Helper functions for `filterFilter`
function createPredicateFn(expression, comparator, anyPropertyKey, matchAgainstAnyProp) {
  var shouldMatchPrimitives = isObject(expression) && (anyPropertyKey in expression);
  var predicateFn;

  if (comparator === true) {
    comparator = equals;
  } else if (!isFunction(comparator)) {
    comparator = function(actual, expected) {
      if (isUndefined(actual)) {
        // No substring matching against `undefined`
        return false;
      }
      if ((actual === null) || (expected === null)) {
        // No substring matching against `null`; only match against `null`
        return actual === expected;
      }
      if (isObject(expected) || (isObject(actual) && !hasCustomToString(actual))) {
        // Should not compare primitives against objects, unless they have custom `toString` method
        return false;
      }

      actual = lowercase('' + actual);
      expected = lowercase('' + expected);
      return actual.indexOf(expected) !== -1;
    };
  }

  predicateFn = function(item) {
    if (shouldMatchPrimitives && !isObject(item)) {
      return deepCompare(item, expression[anyPropertyKey], comparator, anyPropertyKey, false);
    }
    return deepCompare(item, expression, comparator, anyPropertyKey, matchAgainstAnyProp);
  };

  return predicateFn;
}

function deepCompare(actual, expected, comparator, anyPropertyKey, matchAgainstAnyProp, dontMatchWholeObject) {
  var actualType = getTypeForFilter(actual);
  var expectedType = getTypeForFilter(expected);

  if ((expectedType === 'string') && (expected.charAt(0) === '!')) {
    return !deepCompare(actual, expected.substring(1), comparator, anyPropertyKey, matchAgainstAnyProp);
  } else if (isArray(actual)) {
    // In case `actual` is an array, consider it a match
    // if ANY of it's items matches `expected`
    return actual.some(function(item) {
      return deepCompare(item, expected, comparator, anyPropertyKey, matchAgainstAnyProp);
    });
  }

  switch (actualType) {
    case 'object':
      var key;
      if (matchAgainstAnyProp) {
        for (key in actual) {
          // Under certain, rare, circumstances, key may not be a string and `charAt` will be undefined
          // See: https://github.com/angular/angular.js/issues/15644
          if (key.charAt && (key.charAt(0) !== '$') &&
              deepCompare(actual[key], expected, comparator, anyPropertyKey, true)) {
            return true;
          }
        }
        return dontMatchWholeObject ? false : deepCompare(actual, expected, comparator, anyPropertyKey, false);
      } else if (expectedType === 'object') {
        for (key in expected) {
          var expectedVal = expected[key];
          if (isFunction(expectedVal) || isUndefined(expectedVal)) {
            continue;
          }

          var matchAnyProperty = key === anyPropertyKey;
          var actualVal = matchAnyProperty ? actual : actual[key];
          if (!deepCompare(actualVal, expectedVal, comparator, anyPropertyKey, matchAnyProperty, matchAnyProperty)) {
            return false;
          }
        }
        return true;
      } else {
        return comparator(actual, expected);
      }
    case 'function':
      return false;
    default:
      return comparator(actual, expected);
  }
}

// Used for easily differentiating between `null` and actual `object`
function getTypeForFilter(val) {
  return (val === null) ? 'null' : typeof val;
}

var MAX_DIGITS = 22;
var DECIMAL_SEP = '.';
var ZERO_CHAR = '0';

/**
 * @ngdoc filter
 * @name currency
 * @kind function
 *
 * @description
 * Formats a number as a currency (ie $1,234.56). When no currency symbol is provided, default
 * symbol for current locale is used.
 *
 * @param {number} amount Input to filter.
 * @param {string=} symbol Currency symbol or identifier to be displayed.
 * @param {number=} fractionSize Number of decimal places to round the amount to, defaults to default max fraction size for current locale
 * @returns {string} Formatted number.
 *
 *
 * @example
   <example module="currencyExample" name="currency-filter">
     <file name="index.html">
       <script>
         angular.module('currencyExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.amount = 1234.56;
           }]);
       </script>
       <div ng-controller="ExampleController">
         <input type="number" ng-model="amount" aria-label="amount"> <br>
         default currency symbol ($): <span id="currency-default">{{amount | currency}}</span><br>
         custom currency identifier (USD$): <span id="currency-custom">{{amount | currency:"USD$"}}</span><br>
         no fractions (0): <span id="currency-no-fractions">{{amount | currency:"USD$":0}}</span>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should init with 1234.56', function() {
         expect(element(by.id('currency-default')).getText()).toBe('$1,234.56');
         expect(element(by.id('currency-custom')).getText()).toBe('USD$1,234.56');
         expect(element(by.id('currency-no-fractions')).getText()).toBe('USD$1,235');
       });
       it('should update', function() {
         if (browser.params.browser === 'safari') {
           // Safari does not understand the minus key. See
           // https://github.com/angular/protractor/issues/481
           return;
         }
         element(by.model('amount')).clear();
         element(by.model('amount')).sendKeys('-1234');
         expect(element(by.id('currency-default')).getText()).toBe('-$1,234.00');
         expect(element(by.id('currency-custom')).getText()).toBe('-USD$1,234.00');
         expect(element(by.id('currency-no-fractions')).getText()).toBe('-USD$1,234');
       });
     </file>
   </example>
 */
currencyFilter.$inject = ['$locale'];
function currencyFilter($locale) {
  var formats = $locale.NUMBER_FORMATS;
  return function(amount, currencySymbol, fractionSize) {
    if (isUndefined(currencySymbol)) {
      currencySymbol = formats.CURRENCY_SYM;
    }

    if (isUndefined(fractionSize)) {
      fractionSize = formats.PATTERNS[1].maxFrac;
    }

    // If the currency symbol is empty, trim whitespace around the symbol
    var currencySymbolRe = !currencySymbol ? /\s*\u00A4\s*/g : /\u00A4/g;

    // if null or undefined pass it through
    return (amount == null)
        ? amount
        : formatNumber(amount, formats.PATTERNS[1], formats.GROUP_SEP, formats.DECIMAL_SEP, fractionSize).
            replace(currencySymbolRe, currencySymbol);
  };
}

/**
 * @ngdoc filter
 * @name number
 * @kind function
 *
 * @description
 * Formats a number as text.
 *
 * If the input is null or undefined, it will just be returned.
 * If the input is infinite (Infinity or -Infinity), the Infinity symbol '∞' or '-∞' is returned, respectively.
 * If the input is not a number an empty string is returned.
 *
 *
 * @param {number|string} number Number to format.
 * @param {(number|string)=} fractionSize Number of decimal places to round the number to.
 * If this is not provided then the fraction size is computed from the current locale's number
 * formatting pattern. In the case of the default locale, it will be 3.
 * @returns {string} Number rounded to `fractionSize` appropriately formatted based on the current
 *                   locale (e.g., in the en_US locale it will have "." as the decimal separator and
 *                   include "," group separators after each third digit).
 *
 * @example
   <example module="numberFilterExample" name="number-filter">
     <file name="index.html">
       <script>
         angular.module('numberFilterExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.val = 1234.56789;
           }]);
       </script>
       <div ng-controller="ExampleController">
         <label>Enter number: <input ng-model='val'></label><br>
         Default formatting: <span id='number-default'>{{val | number}}</span><br>
         No fractions: <span>{{val | number:0}}</span><br>
         Negative number: <span>{{-val | number:4}}</span>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should format numbers', function() {
         expect(element(by.id('number-default')).getText()).toBe('1,234.568');
         expect(element(by.binding('val | number:0')).getText()).toBe('1,235');
         expect(element(by.binding('-val | number:4')).getText()).toBe('-1,234.5679');
       });

       it('should update', function() {
         element(by.model('val')).clear();
         element(by.model('val')).sendKeys('3374.333');
         expect(element(by.id('number-default')).getText()).toBe('3,374.333');
         expect(element(by.binding('val | number:0')).getText()).toBe('3,374');
         expect(element(by.binding('-val | number:4')).getText()).toBe('-3,374.3330');
      });
     </file>
   </example>
 */
numberFilter.$inject = ['$locale'];
function numberFilter($locale) {
  var formats = $locale.NUMBER_FORMATS;
  return function(number, fractionSize) {

    // if null or undefined pass it through
    return (number == null)
        ? number
        : formatNumber(number, formats.PATTERNS[0], formats.GROUP_SEP, formats.DECIMAL_SEP,
                       fractionSize);
  };
}

/**
 * Parse a number (as a string) into three components that can be used
 * for formatting the number.
 *
 * (Significant bits of this parse algorithm came from https://github.com/MikeMcl/big.js/)
 *
 * @param  {string} numStr The number to parse
 * @return {object} An object describing this number, containing the following keys:
 *  - d : an array of digits containing leading zeros as necessary
 *  - i : the number of the digits in `d` that are to the left of the decimal point
 *  - e : the exponent for numbers that would need more than `MAX_DIGITS` digits in `d`
 *
 */
function parse(numStr) {
  var exponent = 0, digits, numberOfIntegerDigits;
  var i, j, zeros;

  // Decimal point?
  if ((numberOfIntegerDigits = numStr.indexOf(DECIMAL_SEP)) > -1) {
    numStr = numStr.replace(DECIMAL_SEP, '');
  }

  // Exponential form?
  if ((i = numStr.search(/e/i)) > 0) {
    // Work out the exponent.
    if (numberOfIntegerDigits < 0) numberOfIntegerDigits = i;
    numberOfIntegerDigits += +numStr.slice(i + 1);
    numStr = numStr.substring(0, i);
  } else if (numberOfIntegerDigits < 0) {
    // There was no decimal point or exponent so it is an integer.
    numberOfIntegerDigits = numStr.length;
  }

  // Count the number of leading zeros.
  for (i = 0; numStr.charAt(i) === ZERO_CHAR; i++) { /* empty */ }

  if (i === (zeros = numStr.length)) {
    // The digits are all zero.
    digits = [0];
    numberOfIntegerDigits = 1;
  } else {
    // Count the number of trailing zeros
    zeros--;
    while (numStr.charAt(zeros) === ZERO_CHAR) zeros--;

    // Trailing zeros are insignificant so ignore them
    numberOfIntegerDigits -= i;
    digits = [];
    // Convert string to array of digits without leading/trailing zeros.
    for (j = 0; i <= zeros; i++, j++) {
      digits[j] = +numStr.charAt(i);
    }
  }

  // If the number overflows the maximum allowed digits then use an exponent.
  if (numberOfIntegerDigits > MAX_DIGITS) {
    digits = digits.splice(0, MAX_DIGITS - 1);
    exponent = numberOfIntegerDigits - 1;
    numberOfIntegerDigits = 1;
  }

  return { d: digits, e: exponent, i: numberOfIntegerDigits };
}

/**
 * Round the parsed number to the specified number of decimal places
 * This function changed the parsedNumber in-place
 */
function roundNumber(parsedNumber, fractionSize, minFrac, maxFrac) {
    var digits = parsedNumber.d;
    var fractionLen = digits.length - parsedNumber.i;

    // determine fractionSize if it is not specified; `+fractionSize` converts it to a number
    fractionSize = (isUndefined(fractionSize)) ? Math.min(Math.max(minFrac, fractionLen), maxFrac) : +fractionSize;

    // The index of the digit to where rounding is to occur
    var roundAt = fractionSize + parsedNumber.i;
    var digit = digits[roundAt];

    if (roundAt > 0) {
      // Drop fractional digits beyond `roundAt`
      digits.splice(Math.max(parsedNumber.i, roundAt));

      // Set non-fractional digits beyond `roundAt` to 0
      for (var j = roundAt; j < digits.length; j++) {
        digits[j] = 0;
      }
    } else {
      // We rounded to zero so reset the parsedNumber
      fractionLen = Math.max(0, fractionLen);
      parsedNumber.i = 1;
      digits.length = Math.max(1, roundAt = fractionSize + 1);
      digits[0] = 0;
      for (var i = 1; i < roundAt; i++) digits[i] = 0;
    }

    if (digit >= 5) {
      if (roundAt - 1 < 0) {
        for (var k = 0; k > roundAt; k--) {
          digits.unshift(0);
          parsedNumber.i++;
        }
        digits.unshift(1);
        parsedNumber.i++;
      } else {
        digits[roundAt - 1]++;
      }
    }

    // Pad out with zeros to get the required fraction length
    for (; fractionLen < Math.max(0, fractionSize); fractionLen++) digits.push(0);


    // Do any carrying, e.g. a digit was rounded up to 10
    var carry = digits.reduceRight(function(carry, d, i, digits) {
      d = d + carry;
      digits[i] = d % 10;
      return Math.floor(d / 10);
    }, 0);
    if (carry) {
      digits.unshift(carry);
      parsedNumber.i++;
    }
}

/**
 * Format a number into a string
 * @param  {number} number       The number to format
 * @param  {{
 *           minFrac, // the minimum number of digits required in the fraction part of the number
 *           maxFrac, // the maximum number of digits required in the fraction part of the number
 *           gSize,   // number of digits in each group of separated digits
 *           lgSize,  // number of digits in the last group of digits before the decimal separator
 *           negPre,  // the string to go in front of a negative number (e.g. `-` or `(`))
 *           posPre,  // the string to go in front of a positive number
 *           negSuf,  // the string to go after a negative number (e.g. `)`)
 *           posSuf   // the string to go after a positive number
 *         }} pattern
 * @param  {string} groupSep     The string to separate groups of number (e.g. `,`)
 * @param  {string} decimalSep   The string to act as the decimal separator (e.g. `.`)
 * @param  {[type]} fractionSize The size of the fractional part of the number
 * @return {string}              The number formatted as a string
 */
function formatNumber(number, pattern, groupSep, decimalSep, fractionSize) {

  if (!(isString(number) || isNumber(number)) || isNaN(number)) return '';

  var isInfinity = !isFinite(number);
  var isZero = false;
  var numStr = Math.abs(number) + '',
      formattedText = '',
      parsedNumber;

  if (isInfinity) {
    formattedText = '\u221e';
  } else {
    parsedNumber = parse(numStr);

    roundNumber(parsedNumber, fractionSize, pattern.minFrac, pattern.maxFrac);

    var digits = parsedNumber.d;
    var integerLen = parsedNumber.i;
    var exponent = parsedNumber.e;
    var decimals = [];
    isZero = digits.reduce(function(isZero, d) { return isZero && !d; }, true);

    // pad zeros for small numbers
    while (integerLen < 0) {
      digits.unshift(0);
      integerLen++;
    }

    // extract decimals digits
    if (integerLen > 0) {
      decimals = digits.splice(integerLen, digits.length);
    } else {
      decimals = digits;
      digits = [0];
    }

    // format the integer digits with grouping separators
    var groups = [];
    if (digits.length >= pattern.lgSize) {
      groups.unshift(digits.splice(-pattern.lgSize, digits.length).join(''));
    }
    while (digits.length > pattern.gSize) {
      groups.unshift(digits.splice(-pattern.gSize, digits.length).join(''));
    }
    if (digits.length) {
      groups.unshift(digits.join(''));
    }
    formattedText = groups.join(groupSep);

    // append the decimal digits
    if (decimals.length) {
      formattedText += decimalSep + decimals.join('');
    }

    if (exponent) {
      formattedText += 'e+' + exponent;
    }
  }
  if (number < 0 && !isZero) {
    return pattern.negPre + formattedText + pattern.negSuf;
  } else {
    return pattern.posPre + formattedText + pattern.posSuf;
  }
}

function padNumber(num, digits, trim, negWrap) {
  var neg = '';
  if (num < 0 || (negWrap && num <= 0)) {
    if (negWrap) {
      num = -num + 1;
    } else {
      num = -num;
      neg = '-';
    }
  }
  num = '' + num;
  while (num.length < digits) num = ZERO_CHAR + num;
  if (trim) {
    num = num.substr(num.length - digits);
  }
  return neg + num;
}


function dateGetter(name, size, offset, trim, negWrap) {
  offset = offset || 0;
  return function(date) {
    var value = date['get' + name]();
    if (offset > 0 || value > -offset) {
      value += offset;
    }
    if (value === 0 && offset === -12) value = 12;
    return padNumber(value, size, trim, negWrap);
  };
}

function dateStrGetter(name, shortForm, standAlone) {
  return function(date, formats) {
    var value = date['get' + name]();
    var propPrefix = (standAlone ? 'STANDALONE' : '') + (shortForm ? 'SHORT' : '');
    var get = uppercase(propPrefix + name);

    return formats[get][value];
  };
}

function timeZoneGetter(date, formats, offset) {
  var zone = -1 * offset;
  var paddedZone = (zone >= 0) ? '+' : '';

  paddedZone += padNumber(Math[zone > 0 ? 'floor' : 'ceil'](zone / 60), 2) +
                padNumber(Math.abs(zone % 60), 2);

  return paddedZone;
}

function getFirstThursdayOfYear(year) {
    // 0 = index of January
    var dayOfWeekOnFirst = (new Date(year, 0, 1)).getDay();
    // 4 = index of Thursday (+1 to account for 1st = 5)
    // 11 = index of *next* Thursday (+1 account for 1st = 12)
    return new Date(year, 0, ((dayOfWeekOnFirst <= 4) ? 5 : 12) - dayOfWeekOnFirst);
}

function getThursdayThisWeek(datetime) {
    return new Date(datetime.getFullYear(), datetime.getMonth(),
      // 4 = index of Thursday
      datetime.getDate() + (4 - datetime.getDay()));
}

function weekGetter(size) {
   return function(date) {
      var firstThurs = getFirstThursdayOfYear(date.getFullYear()),
         thisThurs = getThursdayThisWeek(date);

      var diff = +thisThurs - +firstThurs,
         result = 1 + Math.round(diff / 6.048e8); // 6.048e8 ms per week

      return padNumber(result, size);
   };
}

function ampmGetter(date, formats) {
  return date.getHours() < 12 ? formats.AMPMS[0] : formats.AMPMS[1];
}

function eraGetter(date, formats) {
  return date.getFullYear() <= 0 ? formats.ERAS[0] : formats.ERAS[1];
}

function longEraGetter(date, formats) {
  return date.getFullYear() <= 0 ? formats.ERANAMES[0] : formats.ERANAMES[1];
}

var DATE_FORMATS = {
  yyyy: dateGetter('FullYear', 4, 0, false, true),
    yy: dateGetter('FullYear', 2, 0, true, true),
     y: dateGetter('FullYear', 1, 0, false, true),
  MMMM: dateStrGetter('Month'),
   MMM: dateStrGetter('Month', true),
    MM: dateGetter('Month', 2, 1),
     M: dateGetter('Month', 1, 1),
  LLLL: dateStrGetter('Month', false, true),
    dd: dateGetter('Date', 2),
     d: dateGetter('Date', 1),
    HH: dateGetter('Hours', 2),
     H: dateGetter('Hours', 1),
    hh: dateGetter('Hours', 2, -12),
     h: dateGetter('Hours', 1, -12),
    mm: dateGetter('Minutes', 2),
     m: dateGetter('Minutes', 1),
    ss: dateGetter('Seconds', 2),
     s: dateGetter('Seconds', 1),
     // while ISO 8601 requires fractions to be prefixed with `.` or `,`
     // we can be just safely rely on using `sss` since we currently don't support single or two digit fractions
   sss: dateGetter('Milliseconds', 3),
  EEEE: dateStrGetter('Day'),
   EEE: dateStrGetter('Day', true),
     a: ampmGetter,
     Z: timeZoneGetter,
    ww: weekGetter(2),
     w: weekGetter(1),
     G: eraGetter,
     GG: eraGetter,
     GGG: eraGetter,
     GGGG: longEraGetter
};

var DATE_FORMATS_SPLIT = /((?:[^yMLdHhmsaZEwG']+)|(?:'(?:[^']|'')*')|(?:E+|y+|M+|L+|d+|H+|h+|m+|s+|a|Z|G+|w+))([\s\S]*)/,
    NUMBER_STRING = /^-?\d+$/;

/**
 * @ngdoc filter
 * @name date
 * @kind function
 *
 * @description
 *   Formats `date` to a string based on the requested `format`.
 *
 *   `format` string can be composed of the following elements:
 *
 *   * `'yyyy'`: 4 digit representation of year (e.g. AD 1 => 0001, AD 2010 => 2010)
 *   * `'yy'`: 2 digit representation of year, padded (00-99). (e.g. AD 2001 => 01, AD 2010 => 10)
 *   * `'y'`: 1 digit representation of year, e.g. (AD 1 => 1, AD 199 => 199)
 *   * `'MMMM'`: Month in year (January-December)
 *   * `'MMM'`: Month in year (Jan-Dec)
 *   * `'MM'`: Month in year, padded (01-12)
 *   * `'M'`: Month in year (1-12)
 *   * `'LLLL'`: Stand-alone month in year (January-December)
 *   * `'dd'`: Day in month, padded (01-31)
 *   * `'d'`: Day in month (1-31)
 *   * `'EEEE'`: Day in Week,(Sunday-Saturday)
 *   * `'EEE'`: Day in Week, (Sun-Sat)
 *   * `'HH'`: Hour in day, padded (00-23)
 *   * `'H'`: Hour in day (0-23)
 *   * `'hh'`: Hour in AM/PM, padded (01-12)
 *   * `'h'`: Hour in AM/PM, (1-12)
 *   * `'mm'`: Minute in hour, padded (00-59)
 *   * `'m'`: Minute in hour (0-59)
 *   * `'ss'`: Second in minute, padded (00-59)
 *   * `'s'`: Second in minute (0-59)
 *   * `'sss'`: Millisecond in second, padded (000-999)
 *   * `'a'`: AM/PM marker
 *   * `'Z'`: 4 digit (+sign) representation of the timezone offset (-1200-+1200)
 *   * `'ww'`: Week of year, padded (00-53). Week 01 is the week with the first Thursday of the year
 *   * `'w'`: Week of year (0-53). Week 1 is the week with the first Thursday of the year
 *   * `'G'`, `'GG'`, `'GGG'`: The abbreviated form of the era string (e.g. 'AD')
 *   * `'GGGG'`: The long form of the era string (e.g. 'Anno Domini')
 *
 *   `format` string can also be one of the following predefined
 *   {@link guide/i18n localizable formats}:
 *
 *   * `'medium'`: equivalent to `'MMM d, y h:mm:ss a'` for en_US locale
 *     (e.g. Sep 3, 2010 12:05:08 PM)
 *   * `'short'`: equivalent to `'M/d/yy h:mm a'` for en_US  locale (e.g. 9/3/10 12:05 PM)
 *   * `'fullDate'`: equivalent to `'EEEE, MMMM d, y'` for en_US  locale
 *     (e.g. Friday, September 3, 2010)
 *   * `'longDate'`: equivalent to `'MMMM d, y'` for en_US  locale (e.g. September 3, 2010)
 *   * `'mediumDate'`: equivalent to `'MMM d, y'` for en_US  locale (e.g. Sep 3, 2010)
 *   * `'shortDate'`: equivalent to `'M/d/yy'` for en_US locale (e.g. 9/3/10)
 *   * `'mediumTime'`: equivalent to `'h:mm:ss a'` for en_US locale (e.g. 12:05:08 PM)
 *   * `'shortTime'`: equivalent to `'h:mm a'` for en_US locale (e.g. 12:05 PM)
 *
 *   `format` string can contain literal values. These need to be escaped by surrounding with single quotes (e.g.
 *   `"h 'in the morning'"`). In order to output a single quote, escape it - i.e., two single quotes in a sequence
 *   (e.g. `"h 'o''clock'"`).
 *
 *   Any other characters in the `format` string will be output as-is.
 *
 * @param {(Date|number|string)} date Date to format either as Date object, milliseconds (string or
 *    number) or various ISO 8601 datetime string formats (e.g. yyyy-MM-ddTHH:mm:ss.sssZ and its
 *    shorter versions like yyyy-MM-ddTHH:mmZ, yyyy-MM-dd or yyyyMMddTHHmmssZ). If no timezone is
 *    specified in the string input, the time is considered to be in the local timezone.
 * @param {string=} format Formatting rules (see Description). If not specified,
 *    `mediumDate` is used.
 * @param {string=} timezone Timezone to be used for formatting. It understands UTC/GMT and the
 *    continental US time zone abbreviations, but for general use, use a time zone offset, for
 *    example, `'+0430'` (4 hours, 30 minutes east of the Greenwich meridian)
 *    If not specified, the timezone of the browser will be used.
 * @returns {string} Formatted string or the input if input is not recognized as date/millis.
 *
 * @example
   <example name="filter-date">
     <file name="index.html">
       <span ng-non-bindable>{{1288323623006 | date:'medium'}}</span>:
           <span>{{1288323623006 | date:'medium'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span>:
          <span>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:'MM/dd/yyyy @ h:mma'}}</span>:
          <span>{{'1288323623006' | date:'MM/dd/yyyy @ h:mma'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:"MM/dd/yyyy 'at' h:mma"}}</span>:
          <span>{{'1288323623006' | date:"MM/dd/yyyy 'at' h:mma"}}</span><br>
     </file>
     <file name="protractor.js" type="protractor">
       it('should format date', function() {
         expect(element(by.binding("1288323623006 | date:'medium'")).getText()).
            toMatch(/Oct 2\d, 2010 \d{1,2}:\d{2}:\d{2} (AM|PM)/);
         expect(element(by.binding("1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'")).getText()).
            toMatch(/2010-10-2\d \d{2}:\d{2}:\d{2} (-|\+)?\d{4}/);
         expect(element(by.binding("'1288323623006' | date:'MM/dd/yyyy @ h:mma'")).getText()).
            toMatch(/10\/2\d\/2010 @ \d{1,2}:\d{2}(AM|PM)/);
         expect(element(by.binding("'1288323623006' | date:\"MM/dd/yyyy 'at' h:mma\"")).getText()).
            toMatch(/10\/2\d\/2010 at \d{1,2}:\d{2}(AM|PM)/);
       });
     </file>
   </example>
 */
dateFilter.$inject = ['$locale'];
function dateFilter($locale) {


  var R_ISO8601_STR = /^(\d{4})-?(\d\d)-?(\d\d)(?:T(\d\d)(?::?(\d\d)(?::?(\d\d)(?:\.(\d+))?)?)?(Z|([+-])(\d\d):?(\d\d))?)?$/;
                     // 1        2       3         4          5          6          7          8  9     10      11
  function jsonStringToDate(string) {
    var match;
    if ((match = string.match(R_ISO8601_STR))) {
      var date = new Date(0),
          tzHour = 0,
          tzMin  = 0,
          dateSetter = match[8] ? date.setUTCFullYear : date.setFullYear,
          timeSetter = match[8] ? date.setUTCHours : date.setHours;

      if (match[9]) {
        tzHour = toInt(match[9] + match[10]);
        tzMin = toInt(match[9] + match[11]);
      }
      dateSetter.call(date, toInt(match[1]), toInt(match[2]) - 1, toInt(match[3]));
      var h = toInt(match[4] || 0) - tzHour;
      var m = toInt(match[5] || 0) - tzMin;
      var s = toInt(match[6] || 0);
      var ms = Math.round(parseFloat('0.' + (match[7] || 0)) * 1000);
      timeSetter.call(date, h, m, s, ms);
      return date;
    }
    return string;
  }


  return function(date, format, timezone) {
    var text = '',
        parts = [],
        fn, match;

    format = format || 'mediumDate';
    format = $locale.DATETIME_FORMATS[format] || format;
    if (isString(date)) {
      date = NUMBER_STRING.test(date) ? toInt(date) : jsonStringToDate(date);
    }

    if (isNumber(date)) {
      date = new Date(date);
    }

    if (!isDate(date) || !isFinite(date.getTime())) {
      return date;
    }

    while (format) {
      match = DATE_FORMATS_SPLIT.exec(format);
      if (match) {
        parts = concat(parts, match, 1);
        format = parts.pop();
      } else {
        parts.push(format);
        format = null;
      }
    }

    var dateTimezoneOffset = date.getTimezoneOffset();
    if (timezone) {
      dateTimezoneOffset = timezoneToOffset(timezone, dateTimezoneOffset);
      date = convertTimezoneToLocal(date, timezone, true);
    }
    forEach(parts, function(value) {
      fn = DATE_FORMATS[value];
      text += fn ? fn(date, $locale.DATETIME_FORMATS, dateTimezoneOffset)
                 : value === '\'\'' ? '\'' : value.replace(/(^'|'$)/g, '').replace(/''/g, '\'');
    });

    return text;
  };
}


/**
 * @ngdoc filter
 * @name json
 * @kind function
 *
 * @description
 *   Allows you to convert a JavaScript object into JSON string.
 *
 *   This filter is mostly useful for debugging. When using the double curly {{value}} notation
 *   the binding is automatically converted to JSON.
 *
 * @param {*} object Any JavaScript object (including arrays and primitive types) to filter.
 * @param {number=} spacing The number of spaces to use per indentation, defaults to 2.
 * @returns {string} JSON string.
 *
 *
 * @example
   <example name="filter-json">
     <file name="index.html">
       <pre id="default-spacing">{{ {'name':'value'} | json }}</pre>
       <pre id="custom-spacing">{{ {'name':'value'} | json:4 }}</pre>
     </file>
     <file name="protractor.js" type="protractor">
       it('should jsonify filtered objects', function() {
         expect(element(by.id('default-spacing')).getText()).toMatch(/\{\n {2}"name": ?"value"\n}/);
         expect(element(by.id('custom-spacing')).getText()).toMatch(/\{\n {4}"name": ?"value"\n}/);
       });
     </file>
   </example>
 *
 */
function jsonFilter() {
  return function(object, spacing) {
    if (isUndefined(spacing)) {
        spacing = 2;
    }
    return toJson(object, spacing);
  };
}


/**
 * @ngdoc filter
 * @name lowercase
 * @kind function
 * @description
 * Converts string to lowercase.
 *
 * See the {@link ng.uppercase uppercase filter documentation} for a functionally identical example.
 *
 * @see angular.lowercase
 */
var lowercaseFilter = valueFn(lowercase);


/**
 * @ngdoc filter
 * @name uppercase
 * @kind function
 * @description
 * Converts string to uppercase.
 * @example
   <example module="uppercaseFilterExample" name="filter-uppercase">
     <file name="index.html">
       <script>
         angular.module('uppercaseFilterExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.title = 'This is a title';
           }]);
       </script>
       <div ng-controller="ExampleController">
         <!-- This title should be formatted normally -->
         <h1>{{title}}</h1>
         <!-- This title should be capitalized -->
         <h1>{{title | uppercase}}</h1>
       </div>
     </file>
   </example>
 */
var uppercaseFilter = valueFn(uppercase);

/**
 * @ngdoc filter
 * @name limitTo
 * @kind function
 *
 * @description
 * Creates a new array or string containing only a specified number of elements. The elements are
 * taken from either the beginning or the end of the source array, string or number, as specified by
 * the value and sign (positive or negative) of `limit`. Other array-like objects are also supported
 * (e.g. array subclasses, NodeLists, jqLite/jQuery collections etc). If a number is used as input,
 * it is converted to a string.
 *
 * @param {Array|ArrayLike|string|number} input - Array/array-like, string or number to be limited.
 * @param {string|number} limit - The length of the returned array or string. If the `limit` number
 *     is positive, `limit` number of items from the beginning of the source array/string are copied.
 *     If the number is negative, `limit` number  of items from the end of the source array/string
 *     are copied. The `limit` will be trimmed if it exceeds `array.length`. If `limit` is undefined,
 *     the input will be returned unchanged.
 * @param {(string|number)=} begin - Index at which to begin limitation. As a negative index,
 *     `begin` indicates an offset from the end of `input`. Defaults to `0`.
 * @returns {Array|string} A new sub-array or substring of length `limit` or less if the input had
 *     less than `limit` elements.
 *
 * @example
   <example module="limitToExample" name="limit-to-filter">
     <file name="index.html">
       <script>
         angular.module('limitToExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.numbers = [1,2,3,4,5,6,7,8,9];
             $scope.letters = "abcdefghi";
             $scope.longNumber = 2345432342;
             $scope.numLimit = 3;
             $scope.letterLimit = 3;
             $scope.longNumberLimit = 3;
           }]);
       </script>
       <div ng-controller="ExampleController">
         <label>
            Limit {{numbers}} to:
            <input type="number" step="1" ng-model="numLimit">
         </label>
         <p>Output numbers: {{ numbers | limitTo:numLimit }}</p>
         <label>
            Limit {{letters}} to:
            <input type="number" step="1" ng-model="letterLimit">
         </label>
         <p>Output letters: {{ letters | limitTo:letterLimit }}</p>
         <label>
            Limit {{longNumber}} to:
            <input type="number" step="1" ng-model="longNumberLimit">
         </label>
         <p>Output long number: {{ longNumber | limitTo:longNumberLimit }}</p>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       var numLimitInput = element(by.model('numLimit'));
       var letterLimitInput = element(by.model('letterLimit'));
       var longNumberLimitInput = element(by.model('longNumberLimit'));
       var limitedNumbers = element(by.binding('numbers | limitTo:numLimit'));
       var limitedLetters = element(by.binding('letters | limitTo:letterLimit'));
       var limitedLongNumber = element(by.binding('longNumber | limitTo:longNumberLimit'));

       it('should limit the number array to first three items', function() {
         expect(numLimitInput.getAttribute('value')).toBe('3');
         expect(letterLimitInput.getAttribute('value')).toBe('3');
         expect(longNumberLimitInput.getAttribute('value')).toBe('3');
         expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3]');
         expect(limitedLetters.getText()).toEqual('Output letters: abc');
         expect(limitedLongNumber.getText()).toEqual('Output long number: 234');
       });

       // There is a bug in safari and protractor that doesn't like the minus key
       // it('should update the output when -3 is entered', function() {
       //   numLimitInput.clear();
       //   numLimitInput.sendKeys('-3');
       //   letterLimitInput.clear();
       //   letterLimitInput.sendKeys('-3');
       //   longNumberLimitInput.clear();
       //   longNumberLimitInput.sendKeys('-3');
       //   expect(limitedNumbers.getText()).toEqual('Output numbers: [7,8,9]');
       //   expect(limitedLetters.getText()).toEqual('Output letters: ghi');
       //   expect(limitedLongNumber.getText()).toEqual('Output long number: 342');
       // });

       it('should not exceed the maximum size of input array', function() {
         numLimitInput.clear();
         numLimitInput.sendKeys('100');
         letterLimitInput.clear();
         letterLimitInput.sendKeys('100');
         longNumberLimitInput.clear();
         longNumberLimitInput.sendKeys('100');
         expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3,4,5,6,7,8,9]');
         expect(limitedLetters.getText()).toEqual('Output letters: abcdefghi');
         expect(limitedLongNumber.getText()).toEqual('Output long number: 2345432342');
       });
     </file>
   </example>
*/
function limitToFilter() {
  return function(input, limit, begin) {
    if (Math.abs(Number(limit)) === Infinity) {
      limit = Number(limit);
    } else {
      limit = toInt(limit);
    }
    if (isNumberNaN(limit)) return input;

    if (isNumber(input)) input = input.toString();
    if (!isArrayLike(input)) return input;

    begin = (!begin || isNaN(begin)) ? 0 : toInt(begin);
    begin = (begin < 0) ? Math.max(0, input.length + begin) : begin;

    if (limit >= 0) {
      return sliceFn(input, begin, begin + limit);
    } else {
      if (begin === 0) {
        return sliceFn(input, limit, input.length);
      } else {
        return sliceFn(input, Math.max(0, begin + limit), begin);
      }
    }
  };
}

function sliceFn(input, begin, end) {
  if (isString(input)) return input.slice(begin, end);

  return slice.call(input, begin, end);
}

/**
 * @ngdoc filter
 * @name orderBy
 * @kind function
 *
 * @description
 * Returns an array containing the items from the specified `collection`, ordered by a `comparator`
 * function based on the values computed using the `expression` predicate.
 *
 * For example, `[{id: 'foo'}, {id: 'bar'}] | orderBy:'id'` would result in
 * `[{id: 'bar'}, {id: 'foo'}]`.
 *
 * The `collection` can be an Array or array-like object (e.g. NodeList, jQuery object, TypedArray,
 * String, etc).
 *
 * The `expression` can be a single predicate, or a list of predicates each serving as a tie-breaker
 * for the preceding one. The `expression` is evaluated against each item and the output is used
 * for comparing with other items.
 *
 * You can change the sorting order by setting `reverse` to `true`. By default, items are sorted in
 * ascending order.
 *
 * The comparison is done using the `comparator` function. If none is specified, a default, built-in
 * comparator is used (see below for details - in a nutshell, it compares numbers numerically and
 * strings alphabetically).
 *
 * ### Under the hood
 *
 * Ordering the specified `collection` happens in two phases:
 *
 * 1. All items are passed through the predicate (or predicates), and the returned values are saved
 *    along with their type (`string`, `number` etc). For example, an item `{label: 'foo'}`, passed
 *    through a predicate that extracts the value of the `label` property, would be transformed to:
 *    ```
 *    {
 *      value: 'foo',
 *      type: 'string',
 *      index: ...
 *    }
 *    ```
 *    **Note:** `null` values use `'null'` as their type.
 * 2. The comparator function is used to sort the items, based on the derived values, types and
 *    indices.
 *
 * If you use a custom comparator, it will be called with pairs of objects of the form
 * `{value: ..., type: '...', index: ...}` and is expected to return `0` if the objects are equal
 * (as far as the comparator is concerned), `-1` if the 1st one should be ranked higher than the
 * second, or `1` otherwise.
 *
 * In order to ensure that the sorting will be deterministic across platforms, if none of the
 * specified predicates can distinguish between two items, `orderBy` will automatically introduce a
 * dummy predicate that returns the item's index as `value`.
 * (If you are using a custom comparator, make sure it can handle this predicate as well.)
 *
 * If a custom comparator still can't distinguish between two items, then they will be sorted based
 * on their index using the built-in comparator.
 *
 * Finally, in an attempt to simplify things, if a predicate returns an object as the extracted
 * value for an item, `orderBy` will try to convert that object to a primitive value, before passing
 * it to the comparator. The following rules govern the conversion:
 *
 * 1. If the object has a `valueOf()` method that returns a primitive, its return value will be
 *    used instead.<br />
 *    (If the object has a `valueOf()` method that returns another object, then the returned object
 *    will be used in subsequent steps.)
 * 2. If the object has a custom `toString()` method (i.e. not the one inherited from `Object`) that
 *    returns a primitive, its return value will be used instead.<br />
 *    (If the object has a `toString()` method that returns another object, then the returned object
 *    will be used in subsequent steps.)
 * 3. No conversion; the object itself is used.
 *
 * ### The default comparator
 *
 * The default, built-in comparator should be sufficient for most usecases. In short, it compares
 * numbers numerically, strings alphabetically (and case-insensitively), for objects falls back to
 * using their index in the original collection, sorts values of different types by type and puts
 * `undefined` and `null` values at the end of the sorted list.
 *
 * More specifically, it follows these steps to determine the relative order of items:
 *
 * 1. If the compared values are of different types:
 *    - If one of the values is undefined, consider it "greater than" the other.
 *    - Else if one of the values is null, consider it "greater than" the other.
 *    - Else compare the types themselves alphabetically.
 * 2. If both values are of type `string`, compare them alphabetically in a case- and
 *    locale-insensitive way.
 * 3. If both values are objects, compare their indices instead.
 * 4. Otherwise, return:
 *    -  `0`, if the values are equal (by strict equality comparison, i.e. using `===`).
 *    - `-1`, if the 1st value is "less than" the 2nd value (compared using the `<` operator).
 *    -  `1`, otherwise.
 *
 * **Note:** If you notice numbers not being sorted as expected, make sure they are actually being
 *           saved as numbers and not strings.
 * **Note:** For the purpose of sorting, `null` and `undefined` are considered "greater than"
 *           any other value (with undefined "greater than" null). This effectively means that `null`
 *           and `undefined` values end up at the end of a list sorted in ascending order.
 * **Note:** `null` values use `'null'` as their type to be able to distinguish them from objects.
 *
 * @param {Array|ArrayLike} collection - The collection (array or array-like object) to sort.
 * @param {(Function|string|Array.<Function|string>)=} expression - A predicate (or list of
 *    predicates) to be used by the comparator to determine the order of elements.
 *
 *    Can be one of:
 *
 *    - `Function`: A getter function. This function will be called with each item as argument and
 *      the return value will be used for sorting.
 *    - `string`: An AngularJS expression. This expression will be evaluated against each item and the
 *      result will be used for sorting. For example, use `'label'` to sort by a property called
 *      `label` or `'label.substring(0, 3)'` to sort by the first 3 characters of the `label`
 *      property.<br />
 *      (The result of a constant expression is interpreted as a property name to be used for
 *      comparison. For example, use `'"special name"'` (note the extra pair of quotes) to sort by a
 *      property called `special name`.)<br />
 *      An expression can be optionally prefixed with `+` or `-` to control the sorting direction,
 *      ascending or descending. For example, `'+label'` or `'-label'`. If no property is provided,
 *      (e.g. `'+'` or `'-'`), the collection element itself is used in comparisons.
 *    - `Array`: An array of function and/or string predicates. If a predicate cannot determine the
 *      relative order of two items, the next predicate is used as a tie-breaker.
 *
 * **Note:** If the predicate is missing or empty then it defaults to `'+'`.
 *
 * @param {boolean=} reverse - If `true`, reverse the sorting order.
 * @param {(Function)=} comparator - The comparator function used to determine the relative order of
 *    value pairs. If omitted, the built-in comparator will be used.
 *
 * @returns {Array} - The sorted array.
 *
 *
 * @example
 * ### Ordering a table with `ngRepeat`
 *
 * The example below demonstrates a simple {@link ngRepeat ngRepeat}, where the data is sorted by
 * age in descending order (expression is set to `'-age'`). The `comparator` is not set, which means
 * it defaults to the built-in comparator.
 *
   <example name="orderBy-static" module="orderByExample1">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <table class="friends">
           <tr>
             <th>Name</th>
             <th>Phone Number</th>
             <th>Age</th>
           </tr>
           <tr ng-repeat="friend in friends | orderBy:'-age'">
             <td>{{friend.name}}</td>
             <td>{{friend.phone}}</td>
             <td>{{friend.age}}</td>
           </tr>
         </table>
       </div>
     </file>
     <file name="script.js">
       angular.module('orderByExample1', [])
         .controller('ExampleController', ['$scope', function($scope) {
           $scope.friends = [
             {name: 'John',   phone: '555-1212',  age: 10},
             {name: 'Mary',   phone: '555-9876',  age: 19},
             {name: 'Mike',   phone: '555-4321',  age: 21},
             {name: 'Adam',   phone: '555-5678',  age: 35},
             {name: 'Julie',  phone: '555-8765',  age: 29}
           ];
         }]);
     </file>
     <file name="style.css">
       .friends {
         border-collapse: collapse;
       }

       .friends th {
         border-bottom: 1px solid;
       }
       .friends td, .friends th {
         border-left: 1px solid;
         padding: 5px 10px;
       }
       .friends td:first-child, .friends th:first-child {
         border-left: none;
       }
     </file>
     <file name="protractor.js" type="protractor">
       // Element locators
       var names = element.all(by.repeater('friends').column('friend.name'));

       it('should sort friends by age in reverse order', function() {
         expect(names.get(0).getText()).toBe('Adam');
         expect(names.get(1).getText()).toBe('Julie');
         expect(names.get(2).getText()).toBe('Mike');
         expect(names.get(3).getText()).toBe('Mary');
         expect(names.get(4).getText()).toBe('John');
       });
     </file>
   </example>
 * <hr />
 *
 * @example
 * ### Changing parameters dynamically
 *
 * All parameters can be changed dynamically. The next example shows how you can make the columns of
 * a table sortable, by binding the `expression` and `reverse` parameters to scope properties.
 *
   <example name="orderBy-dynamic" module="orderByExample2">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <pre>Sort by = {{propertyName}}; reverse = {{reverse}}</pre>
         <hr/>
         <button ng-click="propertyName = null; reverse = false">Set to unsorted</button>
         <hr/>
         <table class="friends">
           <tr>
             <th>
               <button ng-click="sortBy('name')">Name</button>
               <span class="sortorder" ng-show="propertyName === 'name'" ng-class="{reverse: reverse}"></span>
             </th>
             <th>
               <button ng-click="sortBy('phone')">Phone Number</button>
               <span class="sortorder" ng-show="propertyName === 'phone'" ng-class="{reverse: reverse}"></span>
             </th>
             <th>
               <button ng-click="sortBy('age')">Age</button>
               <span class="sortorder" ng-show="propertyName === 'age'" ng-class="{reverse: reverse}"></span>
             </th>
           </tr>
           <tr ng-repeat="friend in friends | orderBy:propertyName:reverse">
             <td>{{friend.name}}</td>
             <td>{{friend.phone}}</td>
             <td>{{friend.age}}</td>
           </tr>
         </table>
       </div>
     </file>
     <file name="script.js">
       angular.module('orderByExample2', [])
         .controller('ExampleController', ['$scope', function($scope) {
           var friends = [
             {name: 'John',   phone: '555-1212',  age: 10},
             {name: 'Mary',   phone: '555-9876',  age: 19},
             {name: 'Mike',   phone: '555-4321',  age: 21},
             {name: 'Adam',   phone: '555-5678',  age: 35},
             {name: 'Julie',  phone: '555-8765',  age: 29}
           ];

           $scope.propertyName = 'age';
           $scope.reverse = true;
           $scope.friends = friends;

           $scope.sortBy = function(propertyName) {
             $scope.reverse = ($scope.propertyName === propertyName) ? !$scope.reverse : false;
             $scope.propertyName = propertyName;
           };
         }]);
     </file>
     <file name="style.css">
       .friends {
         border-collapse: collapse;
       }

       .friends th {
         border-bottom: 1px solid;
       }
       .friends td, .friends th {
         border-left: 1px solid;
         padding: 5px 10px;
       }
       .friends td:first-child, .friends th:first-child {
         border-left: none;
       }

       .sortorder:after {
         content: '\25b2';   // BLACK UP-POINTING TRIANGLE
       }
       .sortorder.reverse:after {
         content: '\25bc';   // BLACK DOWN-POINTING TRIANGLE
       }
     </file>
     <file name="protractor.js" type="protractor">
       // Element locators
       var unsortButton = element(by.partialButtonText('unsorted'));
       var nameHeader = element(by.partialButtonText('Name'));
       var phoneHeader = element(by.partialButtonText('Phone'));
       var ageHeader = element(by.partialButtonText('Age'));
       var firstName = element(by.repeater('friends').column('friend.name').row(0));
       var lastName = element(by.repeater('friends').column('friend.name').row(4));

       it('should sort friends by some property, when clicking on the column header', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         phoneHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Mary');

         nameHeader.click();
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('Mike');

         ageHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Adam');
       });

       it('should sort friends in reverse order, when clicking on the same column', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         ageHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Adam');

         ageHeader.click();
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');
       });

       it('should restore the original order, when clicking "Set to unsorted"', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         unsortButton.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Julie');
       });
     </file>
   </example>
 * <hr />
 *
 * @example
 * ### Using `orderBy` inside a controller
 *
 * It is also possible to call the `orderBy` filter manually, by injecting `orderByFilter`, and
 * calling it with the desired parameters. (Alternatively, you could inject the `$filter` factory
 * and retrieve the `orderBy` filter with `$filter('orderBy')`.)
 *
   <example name="orderBy-call-manually" module="orderByExample3">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <pre>Sort by = {{propertyName}}; reverse = {{reverse}}</pre>
         <hr/>
         <button ng-click="sortBy(null)">Set to unsorted</button>
         <hr/>
         <table class="friends">
           <tr>
             <th>
               <button ng-click="sortBy('name')">Name</button>
               <span class="sortorder" ng-show="propertyName === 'name'" ng-class="{reverse: reverse}"></span>
             </th>
             <th>
               <button ng-click="sortBy('phone')">Phone Number</button>
               <span class="sortorder" ng-show="propertyName === 'phone'" ng-class="{reverse: reverse}"></span>
             </th>
             <th>
               <button ng-click="sortBy('age')">Age</button>
               <span class="sortorder" ng-show="propertyName === 'age'" ng-class="{reverse: reverse}"></span>
             </th>
           </tr>
           <tr ng-repeat="friend in friends">
             <td>{{friend.name}}</td>
             <td>{{friend.phone}}</td>
             <td>{{friend.age}}</td>
           </tr>
         </table>
       </div>
     </file>
     <file name="script.js">
       angular.module('orderByExample3', [])
         .controller('ExampleController', ['$scope', 'orderByFilter', function($scope, orderBy) {
           var friends = [
             {name: 'John',   phone: '555-1212',  age: 10},
             {name: 'Mary',   phone: '555-9876',  age: 19},
             {name: 'Mike',   phone: '555-4321',  age: 21},
             {name: 'Adam',   phone: '555-5678',  age: 35},
             {name: 'Julie',  phone: '555-8765',  age: 29}
           ];

           $scope.propertyName = 'age';
           $scope.reverse = true;
           $scope.friends = orderBy(friends, $scope.propertyName, $scope.reverse);

           $scope.sortBy = function(propertyName) {
             $scope.reverse = (propertyName !== null && $scope.propertyName === propertyName)
                 ? !$scope.reverse : false;
             $scope.propertyName = propertyName;
             $scope.friends = orderBy(friends, $scope.propertyName, $scope.reverse);
           };
         }]);
     </file>
     <file name="style.css">
       .friends {
         border-collapse: collapse;
       }

       .friends th {
         border-bottom: 1px solid;
       }
       .friends td, .friends th {
         border-left: 1px solid;
         padding: 5px 10px;
       }
       .friends td:first-child, .friends th:first-child {
         border-left: none;
       }

       .sortorder:after {
         content: '\25b2';   // BLACK UP-POINTING TRIANGLE
       }
       .sortorder.reverse:after {
         content: '\25bc';   // BLACK DOWN-POINTING TRIANGLE
       }
     </file>
     <file name="protractor.js" type="protractor">
       // Element locators
       var unsortButton = element(by.partialButtonText('unsorted'));
       var nameHeader = element(by.partialButtonText('Name'));
       var phoneHeader = element(by.partialButtonText('Phone'));
       var ageHeader = element(by.partialButtonText('Age'));
       var firstName = element(by.repeater('friends').column('friend.name').row(0));
       var lastName = element(by.repeater('friends').column('friend.name').row(4));

       it('should sort friends by some property, when clicking on the column header', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         phoneHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Mary');

         nameHeader.click();
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('Mike');

         ageHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Adam');
       });

       it('should sort friends in reverse order, when clicking on the same column', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         ageHeader.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Adam');

         ageHeader.click();
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');
       });

       it('should restore the original order, when clicking "Set to unsorted"', function() {
         expect(firstName.getText()).toBe('Adam');
         expect(lastName.getText()).toBe('John');

         unsortButton.click();
         expect(firstName.getText()).toBe('John');
         expect(lastName.getText()).toBe('Julie');
       });
     </file>
   </example>
 * <hr />
 *
 * @example
 * ### Using a custom comparator
 *
 * If you have very specific requirements about the way items are sorted, you can pass your own
 * comparator function. For example, you might need to compare some strings in a locale-sensitive
 * way. (When specifying a custom comparator, you also need to pass a value for the `reverse`
 * argument - passing `false` retains the default sorting order, i.e. ascending.)
 *
   <example name="orderBy-custom-comparator" module="orderByExample4">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <div class="friends-container custom-comparator">
           <h3>Locale-sensitive Comparator</h3>
           <table class="friends">
             <tr>
               <th>Name</th>
               <th>Favorite Letter</th>
             </tr>
             <tr ng-repeat="friend in friends | orderBy:'favoriteLetter':false:localeSensitiveComparator">
               <td>{{friend.name}}</td>
               <td>{{friend.favoriteLetter}}</td>
             </tr>
           </table>
         </div>
         <div class="friends-container default-comparator">
           <h3>Default Comparator</h3>
           <table class="friends">
             <tr>
               <th>Name</th>
               <th>Favorite Letter</th>
             </tr>
             <tr ng-repeat="friend in friends | orderBy:'favoriteLetter'">
               <td>{{friend.name}}</td>
               <td>{{friend.favoriteLetter}}</td>
             </tr>
           </table>
         </div>
       </div>
     </file>
     <file name="script.js">
       angular.module('orderByExample4', [])
         .controller('ExampleController', ['$scope', function($scope) {
           $scope.friends = [
             {name: 'John',   favoriteLetter: 'Ä'},
             {name: 'Mary',   favoriteLetter: 'Ü'},
             {name: 'Mike',   favoriteLetter: 'Ö'},
             {name: 'Adam',   favoriteLetter: 'H'},
             {name: 'Julie',  favoriteLetter: 'Z'}
           ];

           $scope.localeSensitiveComparator = function(v1, v2) {
             // If we don't get strings, just compare by index
             if (v1.type !== 'string' || v2.type !== 'string') {
               return (v1.index < v2.index) ? -1 : 1;
             }

             // Compare strings alphabetically, taking locale into account
             return v1.value.localeCompare(v2.value);
           };
         }]);
     </file>
     <file name="style.css">
       .friends-container {
         display: inline-block;
         margin: 0 30px;
       }

       .friends {
         border-collapse: collapse;
       }

       .friends th {
         border-bottom: 1px solid;
       }
       .friends td, .friends th {
         border-left: 1px solid;
         padding: 5px 10px;
       }
       .friends td:first-child, .friends th:first-child {
         border-left: none;
       }
     </file>
     <file name="protractor.js" type="protractor">
       // Element locators
       var container = element(by.css('.custom-comparator'));
       var names = container.all(by.repeater('friends').column('friend.name'));

       it('should sort friends by favorite letter (in correct alphabetical order)', function() {
         expect(names.get(0).getText()).toBe('John');
         expect(names.get(1).getText()).toBe('Adam');
         expect(names.get(2).getText()).toBe('Mike');
         expect(names.get(3).getText()).toBe('Mary');
         expect(names.get(4).getText()).toBe('Julie');
       });
     </file>
   </example>
 *
 */
orderByFilter.$inject = ['$parse'];
function orderByFilter($parse) {
  return function(array, sortPredicate, reverseOrder, compareFn) {

    if (array == null) return array;
    if (!isArrayLike(array)) {
      throw minErr('orderBy')('notarray', 'Expected array but received: {0}', array);
    }

    if (!isArray(sortPredicate)) { sortPredicate = [sortPredicate]; }
    if (sortPredicate.length === 0) { sortPredicate = ['+']; }

    var predicates = processPredicates(sortPredicate);

    var descending = reverseOrder ? -1 : 1;

    // Define the `compare()` function. Use a default comparator if none is specified.
    var compare = isFunction(compareFn) ? compareFn : defaultCompare;

    // The next three lines are a version of a Swartzian Transform idiom from Perl
    // (sometimes called the Decorate-Sort-Undecorate idiom)
    // See https://en.wikipedia.org/wiki/Schwartzian_transform
    var compareValues = Array.prototype.map.call(array, getComparisonObject);
    compareValues.sort(doComparison);
    array = compareValues.map(function(item) { return item.value; });

    return array;

    function getComparisonObject(value, index) {
      // NOTE: We are adding an extra `tieBreaker` value based on the element's index.
      // This will be used to keep the sort stable when none of the input predicates can
      // distinguish between two elements.
      return {
        value: value,
        tieBreaker: {value: index, type: 'number', index: index},
        predicateValues: predicates.map(function(predicate) {
          return getPredicateValue(predicate.get(value), index);
        })
      };
    }

    function doComparison(v1, v2) {
      for (var i = 0, ii = predicates.length; i < ii; i++) {
        var result = compare(v1.predicateValues[i], v2.predicateValues[i]);
        if (result) {
          return result * predicates[i].descending * descending;
        }
      }

      return (compare(v1.tieBreaker, v2.tieBreaker) || defaultCompare(v1.tieBreaker, v2.tieBreaker)) * descending;
    }
  };

  function processPredicates(sortPredicates) {
    return sortPredicates.map(function(predicate) {
      var descending = 1, get = identity;

      if (isFunction(predicate)) {
        get = predicate;
      } else if (isString(predicate)) {
        if ((predicate.charAt(0) === '+' || predicate.charAt(0) === '-')) {
          descending = predicate.charAt(0) === '-' ? -1 : 1;
          predicate = predicate.substring(1);
        }
        if (predicate !== '') {
          get = $parse(predicate);
          if (get.constant) {
            var key = get();
            get = function(value) { return value[key]; };
          }
        }
      }
      return {get: get, descending: descending};
    });
  }

  function isPrimitive(value) {
    switch (typeof value) {
      case 'number': /* falls through */
      case 'boolean': /* falls through */
      case 'string':
        return true;
      default:
        return false;
    }
  }

  function objectValue(value) {
    // If `valueOf` is a valid function use that
    if (isFunction(value.valueOf)) {
      value = value.valueOf();
      if (isPrimitive(value)) return value;
    }
    // If `toString` is a valid function and not the one from `Object.prototype` use that
    if (hasCustomToString(value)) {
      value = value.toString();
      if (isPrimitive(value)) return value;
    }

    return value;
  }

  function getPredicateValue(value, index) {
    var type = typeof value;
    if (value === null) {
      type = 'null';
    } else if (type === 'object') {
      value = objectValue(value);
    }
    return {value: value, type: type, index: index};
  }

  function defaultCompare(v1, v2) {
    var result = 0;
    var type1 = v1.type;
    var type2 = v2.type;

    if (type1 === type2) {
      var value1 = v1.value;
      var value2 = v2.value;

      if (type1 === 'string') {
        // Compare strings case-insensitively
        value1 = value1.toLowerCase();
        value2 = value2.toLowerCase();
      } else if (type1 === 'object') {
        // For basic objects, use the position of the object
        // in the collection instead of the value
        if (isObject(value1)) value1 = v1.index;
        if (isObject(value2)) value2 = v2.index;
      }

      if (value1 !== value2) {
        result = value1 < value2 ? -1 : 1;
      }
    } else {
      result = (type1 === 'undefined') ? 1 :
        (type2 === 'undefined') ? -1 :
        (type1 === 'null') ? 1 :
        (type2 === 'null') ? -1 :
        (type1 < type2) ? -1 : 1;
    }

    return result;
  }
}

function ngDirective(directive) {
  if (isFunction(directive)) {
    directive = {
      link: directive
    };
  }
  directive.restrict = directive.restrict || 'AC';
  return valueFn(directive);
}

/**
 * @ngdoc directive
 * @name a
 * @restrict E
 *
 * @description
 * Modifies the default behavior of the html a tag so that the default action is prevented when
 * the href attribute is empty.
 *
 * For dynamically creating `href` attributes for a tags, see the {@link ng.ngHref `ngHref`} directive.
 */
var htmlAnchorDirective = valueFn({
  restrict: 'E',
  compile: function(element, attr) {
    if (!attr.href && !attr.xlinkHref) {
      return function(scope, element) {
        // If the linked element is not an anchor tag anymore, do nothing
        if (element[0].nodeName.toLowerCase() !== 'a') return;

        // SVGAElement does not use the href attribute, but rather the 'xlinkHref' attribute.
        var href = toString.call(element.prop('href')) === '[object SVGAnimatedString]' ?
                   'xlink:href' : 'href';
        element.on('click', function(event) {
          // if we have no href url, then don't navigate anywhere.
          if (!element.attr(href)) {
            event.preventDefault();
          }
        });
      };
    }
  }
});

/**
 * @ngdoc directive
 * @name ngHref
 * @restrict A
 * @priority 99
 *
 * @description
 * Using AngularJS markup like `{{hash}}` in an href attribute will
 * make the link go to the wrong URL if the user clicks it before
 * AngularJS has a chance to replace the `{{hash}}` markup with its
 * value. Until AngularJS replaces the markup the link will be broken
 * and will most likely return a 404 error. The `ngHref` directive
 * solves this problem.
 *
 * The wrong way to write it:
 * ```html
 * <a href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <a ng-href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
 * ```
 *
 * @element A
 * @param {template} ngHref any string which can contain `{{}}` markup.
 *
 * @example
 * This example shows various combinations of `href`, `ng-href` and `ng-click` attributes
 * in links and their different behaviors:
    <example name="ng-href">
      <file name="index.html">
        <input ng-model="value" /><br />
        <a id="link-1" href ng-click="value = 1">link 1</a> (link, don't reload)<br />
        <a id="link-2" href="" ng-click="value = 2">link 2</a> (link, don't reload)<br />
        <a id="link-3" ng-href="/{{'123'}}">link 3</a> (link, reload!)<br />
        <a id="link-4" href="" name="xx" ng-click="value = 4">anchor</a> (link, don't reload)<br />
        <a id="link-5" name="xxx" ng-click="value = 5">anchor</a> (no link)<br />
        <a id="link-6" ng-href="{{value}}">link</a> (link, change location)
      </file>
      <file name="protractor.js" type="protractor">
        it('should execute ng-click but not reload when href without value', function() {
          element(by.id('link-1')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('1');
          expect(element(by.id('link-1')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click but not reload when href empty string', function() {
          element(by.id('link-2')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('2');
          expect(element(by.id('link-2')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click and change url when ng-href specified', function() {
          expect(element(by.id('link-3')).getAttribute('href')).toMatch(/\/123$/);

          element(by.id('link-3')).click();

          // At this point, we navigate away from an AngularJS page, so we need
          // to use browser.driver to get the base webdriver.

          browser.wait(function() {
            return browser.driver.getCurrentUrl().then(function(url) {
              return url.match(/\/123$/);
            });
          }, 5000, 'page should navigate to /123');
        });

        it('should execute ng-click but not reload when href empty string and name specified', function() {
          element(by.id('link-4')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('4');
          expect(element(by.id('link-4')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click but not reload when no href but name specified', function() {
          element(by.id('link-5')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('5');
          expect(element(by.id('link-5')).getAttribute('href')).toBe(null);
        });

        it('should only change url when only ng-href', function() {
          element(by.model('value')).clear();
          element(by.model('value')).sendKeys('6');
          expect(element(by.id('link-6')).getAttribute('href')).toMatch(/\/6$/);

          element(by.id('link-6')).click();

          // At this point, we navigate away from an AngularJS page, so we need
          // to use browser.driver to get the base webdriver.
          browser.wait(function() {
            return browser.driver.getCurrentUrl().then(function(url) {
              return url.match(/\/6$/);
            });
          }, 5000, 'page should navigate to /6');
        });
      </file>
    </example>
 */

/**
 * @ngdoc directive
 * @name ngSrc
 * @restrict A
 * @priority 99
 *
 * @description
 * Using AngularJS markup like `{{hash}}` in a `src` attribute doesn't
 * work right: The browser will fetch from the URL with the literal
 * text `{{hash}}` until AngularJS replaces the expression inside
 * `{{hash}}`. The `ngSrc` directive solves this problem.
 *
 * The buggy way to write it:
 * ```html
 * <img src="http://www.gravatar.com/avatar/{{hash}}" alt="Description"/>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <img ng-src="http://www.gravatar.com/avatar/{{hash}}" alt="Description" />
 * ```
 *
 * @element IMG
 * @param {template} ngSrc any string which can contain `{{}}` markup.
 */

/**
 * @ngdoc directive
 * @name ngSrcset
 * @restrict A
 * @priority 99
 *
 * @description
 * Using AngularJS markup like `{{hash}}` in a `srcset` attribute doesn't
 * work right: The browser will fetch from the URL with the literal
 * text `{{hash}}` until AngularJS replaces the expression inside
 * `{{hash}}`. The `ngSrcset` directive solves this problem.
 *
 * The buggy way to write it:
 * ```html
 * <img srcset="http://www.gravatar.com/avatar/{{hash}} 2x" alt="Description"/>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <img ng-srcset="http://www.gravatar.com/avatar/{{hash}} 2x" alt="Description" />
 * ```
 *
 * @element IMG
 * @param {template} ngSrcset any string which can contain `{{}}` markup.
 */

/**
 * @ngdoc directive
 * @name ngDisabled
 * @restrict A
 * @priority 100
 *
 * @description
 *
 * This directive sets the `disabled` attribute on the element (typically a form control,
 * e.g. `input`, `button`, `select` etc.) if the
 * {@link guide/expression expression} inside `ngDisabled` evaluates to truthy.
 *
 * A special directive is necessary because we cannot use interpolation inside the `disabled`
 * attribute. See the {@link guide/interpolation interpolation guide} for more info.
 *
 * @example
    <example name="ng-disabled">
      <file name="index.html">
        <label>Click me to toggle: <input type="checkbox" ng-model="checked"></label><br/>
        <button ng-model="button" ng-disabled="checked">Button</button>
      </file>
      <file name="protractor.js" type="protractor">
        it('should toggle button', function() {
          expect(element(by.css('button')).getAttribute('disabled')).toBeFalsy();
          element(by.model('checked')).click();
          expect(element(by.css('button')).getAttribute('disabled')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @param {expression} ngDisabled If the {@link guide/expression expression} is truthy,
 *     then the `disabled` attribute will be set on the element
 */


/**
 * @ngdoc directive
 * @name ngChecked
 * @restrict A
 * @priority 100
 *
 * @description
 * Sets the `checked` attribute on the element, if the expression inside `ngChecked` is truthy.
 *
 * Note that this directive should not be used together with {@link ngModel `ngModel`},
 * as this can lead to unexpected behavior.
 *
 * A special directive is necessary because we cannot use interpolation inside the `checked`
 * attribute. See the {@link guide/interpolation interpolation guide} for more info.
 *
 * @example
    <example name="ng-checked">
      <file name="index.html">
        <label>Check me to check both: <input type="checkbox" ng-model="leader"></label><br/>
        <input id="checkFollower" type="checkbox" ng-checked="leader" aria-label="Follower input">
      </file>
      <file name="protractor.js" type="protractor">
        it('should check both checkBoxes', function() {
          expect(element(by.id('checkFollower')).getAttribute('checked')).toBeFalsy();
          element(by.model('leader')).click();
          expect(element(by.id('checkFollower')).getAttribute('checked')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element INPUT
 * @param {expression} ngChecked If the {@link guide/expression expression} is truthy,
 *     then the `checked` attribute will be set on the element
 */


/**
 * @ngdoc directive
 * @name ngReadonly
 * @restrict A
 * @priority 100
 *
 * @description
 *
 * Sets the `readonly` attribute on the element, if the expression inside `ngReadonly` is truthy.
 * Note that `readonly` applies only to `input` elements with specific types. [See the input docs on
 * MDN](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#attr-readonly) for more information.
 *
 * A special directive is necessary because we cannot use interpolation inside the `readonly`
 * attribute. See the {@link guide/interpolation interpolation guide} for more info.
 *
 * @example
    <example name="ng-readonly">
      <file name="index.html">
        <label>Check me to make text readonly: <input type="checkbox" ng-model="checked"></label><br/>
        <input type="text" ng-readonly="checked" value="I'm AngularJS" aria-label="Readonly field" />
      </file>
      <file name="protractor.js" type="protractor">
        it('should toggle readonly attr', function() {
          expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeFalsy();
          element(by.model('checked')).click();
          expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element INPUT
 * @param {expression} ngReadonly If the {@link guide/expression expression} is truthy,
 *     then special attribute "readonly" will be set on the element
 */


/**
 * @ngdoc directive
 * @name ngSelected
 * @restrict A
 * @priority 100
 *
 * @description
 *
 * Sets the `selected` attribute on the element, if the expression inside `ngSelected` is truthy.
 *
 * A special directive is necessary because we cannot use interpolation inside the `selected`
 * attribute. See the {@link guide/interpolation interpolation guide} for more info.
 *
 * <div class="alert alert-warning">
 *   **Note:** `ngSelected` does not interact with the `select` and `ngModel` directives, it only
 *   sets the `selected` attribute on the element. If you are using `ngModel` on the select, you
 *   should not use `ngSelected` on the options, as `ngModel` will set the select value and
 *   selected options.
 * </div>
 *
 * @example
    <example name="ng-selected">
      <file name="index.html">
        <label>Check me to select: <input type="checkbox" ng-model="selected"></label><br/>
        <select aria-label="ngSelected demo">
          <option>Hello!</option>
          <option id="greet" ng-selected="selected">Greetings!</option>
        </select>
      </file>
      <file name="protractor.js" type="protractor">
        it('should select Greetings!', function() {
          expect(element(by.id('greet')).getAttribute('selected')).toBeFalsy();
          element(by.model('selected')).click();
          expect(element(by.id('greet')).getAttribute('selected')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element OPTION
 * @param {expression} ngSelected If the {@link guide/expression expression} is truthy,
 *     then special attribute "selected" will be set on the element
 */

/**
 * @ngdoc directive
 * @name ngOpen
 * @restrict A
 * @priority 100
 *
 * @description
 *
 * Sets the `open` attribute on the element, if the expression inside `ngOpen` is truthy.
 *
 * A special directive is necessary because we cannot use interpolation inside the `open`
 * attribute. See the {@link guide/interpolation interpolation guide} for more info.
 *
 * ## A note about browser compatibility
 *
 * Internet Explorer and Edge do not support the `details` element, it is
 * recommended to use {@link ng.ngShow} and {@link ng.ngHide} instead.
 *
 * @example
     <example name="ng-open">
       <file name="index.html">
         <label>Toggle details: <input type="checkbox" ng-model="open"></label><br/>
         <details id="details" ng-open="open">
            <summary>List</summary>
            <ul>
              <li>Apple</li>
              <li>Orange</li>
              <li>Durian</li>
            </ul>
         </details>
       </file>
       <file name="protractor.js" type="protractor">
         it('should toggle open', function() {
           expect(element(by.id('details')).getAttribute('open')).toBeFalsy();
           element(by.model('open')).click();
           expect(element(by.id('details')).getAttribute('open')).toBeTruthy();
         });
       </file>
     </example>
 *
 * @element DETAILS
 * @param {expression} ngOpen If the {@link guide/expression expression} is truthy,
 *     then special attribute "open" will be set on the element
 */

var ngAttributeAliasDirectives = {};

// boolean attrs are evaluated
forEach(BOOLEAN_ATTR, function(propName, attrName) {
  // binding to multiple is not supported
  if (propName === 'multiple') return;

  function defaultLinkFn(scope, element, attr) {
    scope.$watch(attr[normalized], function ngBooleanAttrWatchAction(value) {
      attr.$set(attrName, !!value);
    });
  }

  var normalized = directiveNormalize('ng-' + attrName);
  var linkFn = defaultLinkFn;

  if (propName === 'checked') {
    linkFn = function(scope, element, attr) {
      // ensuring ngChecked doesn't interfere with ngModel when both are set on the same input
      if (attr.ngModel !== attr[normalized]) {
        defaultLinkFn(scope, element, attr);
      }
    };
  }

  ngAttributeAliasDirectives[normalized] = function() {
    return {
      restrict: 'A',
      priority: 100,
      link: linkFn
    };
  };
});

// aliased input attrs are evaluated
forEach(ALIASED_ATTR, function(htmlAttr, ngAttr) {
  ngAttributeAliasDirectives[ngAttr] = function() {
    return {
      priority: 100,
      link: function(scope, element, attr) {
        //special case ngPattern when a literal regular expression value
        //is used as the expression (this way we don't have to watch anything).
        if (ngAttr === 'ngPattern' && attr.ngPattern.charAt(0) === '/') {
          var match = attr.ngPattern.match(REGEX_STRING_REGEXP);
          if (match) {
            attr.$set('ngPattern', new RegExp(match[1], match[2]));
            return;
          }
        }

        scope.$watch(attr[ngAttr], function ngAttrAliasWatchAction(value) {
          attr.$set(ngAttr, value);
        });
      }
    };
  };
});

// ng-src, ng-srcset, ng-href are interpolated
forEach(['src', 'srcset', 'href'], function(attrName) {
  var normalized = directiveNormalize('ng-' + attrName);
  ngAttributeAliasDirectives[normalized] = ['$sce', function($sce) {
    return {
      priority: 99, // it needs to run after the attributes are interpolated
      link: function(scope, element, attr) {
        var propName = attrName,
            name = attrName;

        if (attrName === 'href' &&
            toString.call(element.prop('href')) === '[object SVGAnimatedString]') {
          name = 'xlinkHref';
          attr.$attr[name] = 'xlink:href';
          propName = null;
        }

        // We need to sanitize the url at least once, in case it is a constant
        // non-interpolated attribute.
        attr.$set(normalized, $sce.getTrustedMediaUrl(attr[normalized]));

        attr.$observe(normalized, function(value) {
          if (!value) {
            if (attrName === 'href') {
              attr.$set(name, null);
            }
            return;
          }

          attr.$set(name, value);

          // Support: IE 9-11 only
          // On IE, if "ng:src" directive declaration is used and "src" attribute doesn't exist
          // then calling element.setAttribute('src', 'foo') doesn't do anything, so we need
          // to set the property as well to achieve the desired effect.
          // We use attr[attrName] value since $set might have sanitized the url.
          if (msie && propName) element.prop(propName, attr[name]);
        });
      }
    };
  }];
});

/* global -nullFormCtrl, -PENDING_CLASS, -SUBMITTED_CLASS
 */
var nullFormCtrl = {
  $addControl: noop,
  $getControls: valueFn([]),
  $$renameControl: nullFormRenameControl,
  $removeControl: noop,
  $setValidity: noop,
  $setDirty: noop,
  $setPristine: noop,
  $setSubmitted: noop,
  $$setSubmitted: noop
},
PENDING_CLASS = 'ng-pending',
SUBMITTED_CLASS = 'ng-submitted';

function nullFormRenameControl(control, name) {
  control.$name = name;
}

/**
 * @ngdoc type
 * @name form.FormController
 *
 * @property {boolean} $pristine True if user has not interacted with the form yet.
 * @property {boolean} $dirty True if user has already interacted with the form.
 * @property {boolean} $valid True if all of the containing forms and controls are valid.
 * @property {boolean} $invalid True if at least one containing control or form is invalid.
 * @property {boolean} $submitted True if user has submitted the form even if its invalid.
 *
 * @property {Object} $pending An object hash, containing references to controls or forms with
 *  pending validators, where:
 *
 *  - keys are validations tokens (error names).
 *  - values are arrays of controls or forms that have a pending validator for the given error name.
 *
 * See {@link form.FormController#$error $error} for a list of built-in validation tokens.
 *
 * @property {Object} $error An object hash, containing references to controls or forms with failing
 *  validators, where:
 *
 *  - keys are validation tokens (error names),
 *  - values are arrays of controls or forms that have a failing validator for the given error name.
 *
 *  Built-in validation tokens:
 *  - `email`
 *  - `max`
 *  - `maxlength`
 *  - `min`
 *  - `minlength`
 *  - `number`
 *  - `pattern`
 *  - `required`
 *  - `url`
 *  - `date`
 *  - `datetimelocal`
 *  - `time`
 *  - `week`
 *  - `month`
 *
 * @description
 * `FormController` keeps track of all its controls and nested forms as well as the state of them,
 * such as being valid/invalid or dirty/pristine.
 *
 * Each {@link ng.directive:form form} directive creates an instance
 * of `FormController`.
 *
 */
//asks for $scope to fool the BC controller module
FormController.$inject = ['$element', '$attrs', '$scope', '$animate', '$interpolate'];
function FormController($element, $attrs, $scope, $animate, $interpolate) {
  this.$$controls = [];

  // init state
  this.$error = {};
  this.$$success = {};
  this.$pending = undefined;
  this.$name = $interpolate($attrs.name || $attrs.ngForm || '')($scope);
  this.$dirty = false;
  this.$pristine = true;
  this.$valid = true;
  this.$invalid = false;
  this.$submitted = false;
  this.$$parentForm = nullFormCtrl;

  this.$$element = $element;
  this.$$animate = $animate;

  setupValidity(this);
}

FormController.prototype = {
  /**
   * @ngdoc method
   * @name form.FormController#$rollbackViewValue
   *
   * @description
   * Rollback all form controls pending updates to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. This method is typically needed by the reset button of
   * a form that uses `ng-model-options` to pend updates.
   */
  $rollbackViewValue: function() {
    forEach(this.$$controls, function(control) {
      control.$rollbackViewValue();
    });
  },

  /**
   * @ngdoc method
   * @name form.FormController#$commitViewValue
   *
   * @description
   * Commit all form controls pending updates to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. This method is rarely needed as `NgModelController`
   * usually handles calling this in response to input events.
   */
  $commitViewValue: function() {
    forEach(this.$$controls, function(control) {
      control.$commitViewValue();
    });
  },

  /**
   * @ngdoc method
   * @name form.FormController#$addControl
   * @param {object} control control object, either a {@link form.FormController} or an
   * {@link ngModel.NgModelController}
   *
   * @description
   * Register a control with the form. Input elements using ngModelController do this automatically
   * when they are linked.
   *
   * Note that the current state of the control will not be reflected on the new parent form. This
   * is not an issue with normal use, as freshly compiled and linked controls are in a `$pristine`
   * state.
   *
   * However, if the method is used programmatically, for example by adding dynamically created controls,
   * or controls that have been previously removed without destroying their corresponding DOM element,
   * it's the developers responsibility to make sure the current state propagates to the parent form.
   *
   * For example, if an input control is added that is already `$dirty` and has `$error` properties,
   * calling `$setDirty()` and `$validate()` afterwards will propagate the state to the parent form.
   */
  $addControl: function(control) {
    // Breaking change - before, inputs whose name was "hasOwnProperty" were quietly ignored
    // and not added to the scope.  Now we throw an error.
    assertNotHasOwnProperty(control.$name, 'input');
    this.$$controls.push(control);

    if (control.$name) {
      this[control.$name] = control;
    }

    control.$$parentForm = this;
  },

  /**
   * @ngdoc method
   * @name form.FormController#$getControls
   * @returns {Array} the controls that are currently part of this form
   *
   * @description
   * This method returns a **shallow copy** of the controls that are currently part of this form.
   * The controls can be instances of {@link form.FormController `FormController`}
   * ({@link ngForm "child-forms"}) and of {@link ngModel.NgModelController `NgModelController`}.
   * If you need access to the controls of child-forms, you have to call `$getControls()`
   * recursively on them.
   * This can be used for example to iterate over all controls to validate them.
   *
   * The controls can be accessed normally, but adding to, or removing controls from the array has
   * no effect on the form. Instead, use {@link form.FormController#$addControl `$addControl()`} and
   * {@link form.FormController#$removeControl `$removeControl()`} for this use-case.
   * Likewise, adding a control to, or removing a control from the form is not reflected
   * in the shallow copy. That means you should get a fresh copy from `$getControls()` every time
   * you need access to the controls.
   */
  $getControls: function() {
    return shallowCopy(this.$$controls);
  },

  // Private API: rename a form control
  $$renameControl: function(control, newName) {
    var oldName = control.$name;

    if (this[oldName] === control) {
      delete this[oldName];
    }
    this[newName] = control;
    control.$name = newName;
  },

  /**
   * @ngdoc method
   * @name form.FormController#$removeControl
   * @param {object} control control object, either a {@link form.FormController} or an
   * {@link ngModel.NgModelController}
   *
   * @description
   * Deregister a control from the form.
   *
   * Input elements using ngModelController do this automatically when they are destroyed.
   *
   * Note that only the removed control's validation state (`$errors`etc.) will be removed from the
   * form. `$dirty`, `$submitted` states will not be changed, because the expected behavior can be
   * different from case to case. For example, removing the only `$dirty` control from a form may or
   * may not mean that the form is still `$dirty`.
   */
  $removeControl: function(control) {
    if (control.$name && this[control.$name] === control) {
      delete this[control.$name];
    }
    forEach(this.$pending, function(value, name) {
      // eslint-disable-next-line no-invalid-this
      this.$setValidity(name, null, control);
    }, this);
    forEach(this.$error, function(value, name) {
      // eslint-disable-next-line no-invalid-this
      this.$setValidity(name, null, control);
    }, this);
    forEach(this.$$success, function(value, name) {
      // eslint-disable-next-line no-invalid-this
      this.$setValidity(name, null, control);
    }, this);

    arrayRemove(this.$$controls, control);
    control.$$parentForm = nullFormCtrl;
  },

  /**
   * @ngdoc method
   * @name form.FormController#$setDirty
   *
   * @description
   * Sets the form to a dirty state.
   *
   * This method can be called to add the 'ng-dirty' class and set the form to a dirty
   * state (ng-dirty class). This method will also propagate to parent forms.
   */
  $setDirty: function() {
    this.$$animate.removeClass(this.$$element, PRISTINE_CLASS);
    this.$$animate.addClass(this.$$element, DIRTY_CLASS);
    this.$dirty = true;
    this.$pristine = false;
    this.$$parentForm.$setDirty();
  },

  /**
   * @ngdoc method
   * @name form.FormController#$setPristine
   *
   * @description
   * Sets the form to its pristine state.
   *
   * This method sets the form's `$pristine` state to true, the `$dirty` state to false, removes
   * the `ng-dirty` class and adds the `ng-pristine` class. Additionally, it sets the `$submitted`
   * state to false.
   *
   * This method will also propagate to all the controls contained in this form.
   *
   * Setting a form back to a pristine state is often useful when we want to 'reuse' a form after
   * saving or resetting it.
   */
  $setPristine: function() {
    this.$$animate.setClass(this.$$element, PRISTINE_CLASS, DIRTY_CLASS + ' ' + SUBMITTED_CLASS);
    this.$dirty = false;
    this.$pristine = true;
    this.$submitted = false;
    forEach(this.$$controls, function(control) {
      control.$setPristine();
    });
  },

  /**
   * @ngdoc method
   * @name form.FormController#$setUntouched
   *
   * @description
   * Sets the form to its untouched state.
   *
   * This method can be called to remove the 'ng-touched' class and set the form controls to their
   * untouched state (ng-untouched class).
   *
   * Setting a form controls back to their untouched state is often useful when setting the form
   * back to its pristine state.
   */
  $setUntouched: function() {
    forEach(this.$$controls, function(control) {
      control.$setUntouched();
    });
  },

  /**
   * @ngdoc method
   * @name form.FormController#$setSubmitted
   *
   * @description
   * Sets the form to its `$submitted` state. This will also set `$submitted` on all child and
   * parent forms of the form.
   */
  $setSubmitted: function() {
    var rootForm = this;
    while (rootForm.$$parentForm && (rootForm.$$parentForm !== nullFormCtrl)) {
      rootForm = rootForm.$$parentForm;
    }
    rootForm.$$setSubmitted();
  },

  $$setSubmitted: function() {
    this.$$animate.addClass(this.$$element, SUBMITTED_CLASS);
    this.$submitted = true;
    forEach(this.$$controls, function(control) {
      if (control.$$setSubmitted) {
        control.$$setSubmitted();
      }
    });
  }
};

/**
 * @ngdoc method
 * @name form.FormController#$setValidity
 *
 * @description
 * Change the validity state of the form, and notify the parent form (if any).
 *
 * Application developers will rarely need to call this method directly. It is used internally, by
 * {@link ngModel.NgModelController#$setValidity NgModelController.$setValidity()}, to propagate a
 * control's validity state to the parent `FormController`.
 *
 * @param {string} validationErrorKey Name of the validator. The `validationErrorKey` will be
 *        assigned to either `$error[validationErrorKey]` or `$pending[validationErrorKey]` (for
 *        unfulfilled `$asyncValidators`), so that it is available for data-binding. The
 *        `validationErrorKey` should be in camelCase and will get converted into dash-case for
 *        class name. Example: `myError` will result in `ng-valid-my-error` and
 *        `ng-invalid-my-error` classes and can be bound to as `{{ someForm.$error.myError }}`.
 * @param {boolean} isValid Whether the current state is valid (true), invalid (false), pending
 *        (undefined),  or skipped (null). Pending is used for unfulfilled `$asyncValidators`.
 *        Skipped is used by AngularJS when validators do not run because of parse errors and when
 *        `$asyncValidators` do not run because any of the `$validators` failed.
 * @param {NgModelController | FormController} controller - The controller whose validity state is
 *        triggering the change.
 */
addSetValidityMethod({
  clazz: FormController,
  set: function(object, property, controller) {
    var list = object[property];
    if (!list) {
      object[property] = [controller];
    } else {
      var index = list.indexOf(controller);
      if (index === -1) {
        list.push(controller);
      }
    }
  },
  unset: function(object, property, controller) {
    var list = object[property];
    if (!list) {
      return;
    }
    arrayRemove(list, controller);
    if (list.length === 0) {
      delete object[property];
    }
  }
});

/**
 * @ngdoc directive
 * @name ngForm
 * @restrict EAC
 *
 * @description
 * Helper directive that makes it possible to create control groups inside a
 * {@link ng.directive:form `form`} directive.
 * These "child forms" can be used, for example, to determine the validity of a sub-group of
 * controls.
 *
 * <div class="alert alert-danger">
 * **Note**: `ngForm` cannot be used as a replacement for `<form>`, because it lacks its
 * [built-in HTML functionality](https://html.spec.whatwg.org/#the-form-element).
 * Specifically, you cannot submit `ngForm` like a `<form>` tag. That means,
 * you cannot send data to the server with `ngForm`, or integrate it with
 * {@link ng.directive:ngSubmit `ngSubmit`}.
 * </div>
 *
 * @param {string=} ngForm|name Name of the form. If specified, the form controller will
 *                              be published into the related scope, under this name.
 *
 */

 /**
 * @ngdoc directive
 * @name form
 * @restrict E
 *
 * @description
 * Directive that instantiates
 * {@link form.FormController FormController}.
 *
 * If the `name` attribute is specified, the form controller is published onto the current scope under
 * this name.
 *
 * ## Alias: {@link ng.directive:ngForm `ngForm`}
 *
 * In AngularJS, forms can be nested. This means that the outer form is valid when all of the child
 * forms are valid as well. However, browsers do not allow nesting of `<form>` elements, so
 * AngularJS provides the {@link ng.directive:ngForm `ngForm`} directive, which behaves identically to
 * `form` but can be nested. Nested forms can be useful, for example, if the validity of a sub-group
 * of controls needs to be determined.
 *
 * ## CSS classes
 *  - `ng-valid` is set if the form is valid.
 *  - `ng-invalid` is set if the form is invalid.
 *  - `ng-pending` is set if the form is pending.
 *  - `ng-pristine` is set if the form is pristine.
 *  - `ng-dirty` is set if the form is dirty.
 *  - `ng-submitted` is set if the form was submitted.
 *
 * Keep in mind that ngAnimate can detect each of these classes when added and removed.
 *
 *
 * ## Submitting a form and preventing the default action
 *
 * Since the role of forms in client-side AngularJS applications is different than in classical
 * roundtrip apps, it is desirable for the browser not to translate the form submission into a full
 * page reload that sends the data to the server. Instead some javascript logic should be triggered
 * to handle the form submission in an application-specific way.
 *
 * For this reason, AngularJS prevents the default action (form submission to the server) unless the
 * `<form>` element has an `action` attribute specified.
 *
 * You can use one of the following two ways to specify what javascript method should be called when
 * a form is submitted:
 *
 * - {@link ng.directive:ngSubmit ngSubmit} directive on the form element
 * - {@link ng.directive:ngClick ngClick} directive on the first
  *  button or input field of type submit (input[type=submit])
 *
 * To prevent double execution of the handler, use only one of the {@link ng.directive:ngSubmit ngSubmit}
 * or {@link ng.directive:ngClick ngClick} directives.
 * This is because of the following form submission rules in the HTML specification:
 *
 * - If a form has only one input field then hitting enter in this field triggers form submit
 * (`ngSubmit`)
 * - if a form has 2+ input fields and no buttons or input[type=submit] then hitting enter
 * doesn't trigger submit
 * - if a form has one or more input fields and one or more buttons or input[type=submit] then
 * hitting enter in any of the input fields will trigger the click handler on the *first* button or
 * input[type=submit] (`ngClick`) *and* a submit handler on the enclosing form (`ngSubmit`)
 *
 * Any pending `ngModelOptions` changes will take place immediately when an enclosing form is
 * submitted. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
 * to have access to the updated model.
 *
 * @animations
 * Animations in ngForm are triggered when any of the associated CSS classes are added and removed.
 * These classes are: `.ng-pristine`, `.ng-dirty`, `.ng-invalid` and `.ng-valid` as well as any
 * other validations that are performed within the form. Animations in ngForm are similar to how
 * they work in ngClass and animations can be hooked into using CSS transitions, keyframes as well
 * as JS animations.
 *
 * The following example shows a simple way to utilize CSS transitions to style a form element
 * that has been rendered as invalid after it has been validated:
 *
 * <pre>
 * //be sure to include ngAnimate as a module to hook into more
 * //advanced animations
 * .my-form {
 *   transition:0.5s linear all;
 *   background: white;
 * }
 * .my-form.ng-invalid {
 *   background: red;
 *   color:white;
 * }
 * </pre>
 *
 * @example
    <example name="ng-form" deps="angular-animate.js" animations="true" fixBase="true" module="formExample">
      <file name="index.html">
       <script>
         angular.module('formExample', [])
           .controller('FormController', ['$scope', function($scope) {
             $scope.userType = 'guest';
           }]);
       </script>
       <style>
        .my-form {
          transition:all linear 0.5s;
          background: transparent;
        }
        .my-form.ng-invalid {
          background: red;
        }
       </style>
       <form name="myForm" ng-controller="FormController" class="my-form">
         userType: <input name="input" ng-model="userType" required>
         <span class="error" ng-show="myForm.input.$error.required">Required!</span><br>
         <code>userType = {{userType}}</code><br>
         <code>myForm.input.$valid = {{myForm.input.$valid}}</code><br>
         <code>myForm.input.$error = {{myForm.input.$error}}</code><br>
         <code>myForm.$valid = {{myForm.$valid}}</code><br>
         <code>myForm.$error.required = {{!!myForm.$error.required}}</code><br>
        </form>
      </file>
      <file name="protractor.js" type="protractor">
        it('should initialize to model', function() {
          var userType = element(by.binding('userType'));
          var valid = element(by.binding('myForm.input.$valid'));

          expect(userType.getText()).toContain('guest');
          expect(valid.getText()).toContain('true');
        });

        it('should be invalid if empty', function() {
          var userType = element(by.binding('userType'));
          var valid = element(by.binding('myForm.input.$valid'));
          var userInput = element(by.model('userType'));

          userInput.clear();
          userInput.sendKeys('');

          expect(userType.getText()).toEqual('userType =');
          expect(valid.getText()).toContain('false');
        });
      </file>
    </example>
 *
 * @param {string=} name Name of the form. If specified, the form controller will be published into
 *                       related scope, under this name.
 */
var formDirectiveFactory = function(isNgForm) {
  return ['$timeout', '$parse', function($timeout, $parse) {
    var formDirective = {
      name: 'form',
      restrict: isNgForm ? 'EAC' : 'E',
      require: ['form', '^^?form'], //first is the form's own ctrl, second is an optional parent form
      controller: FormController,
      compile: function ngFormCompile(formElement, attr) {
        // Setup initial state of the control
        formElement.addClass(PRISTINE_CLASS).addClass(VALID_CLASS);

        var nameAttr = attr.name ? 'name' : (isNgForm && attr.ngForm ? 'ngForm' : false);

        return {
          pre: function ngFormPreLink(scope, formElement, attr, ctrls) {
            var controller = ctrls[0];

            // if `action` attr is not present on the form, prevent the default action (submission)
            if (!('action' in attr)) {
              // we can't use jq events because if a form is destroyed during submission the default
              // action is not prevented. see #1238
              //
              // IE 9 is not affected because it doesn't fire a submit event and try to do a full
              // page reload if the form was destroyed by submission of the form via a click handler
              // on a button in the form. Looks like an IE9 specific bug.
              var handleFormSubmission = function(event) {
                scope.$apply(function() {
                  controller.$commitViewValue();
                  controller.$setSubmitted();
                });

                event.preventDefault();
              };

              formElement[0].addEventListener('submit', handleFormSubmission);

              // unregister the preventDefault listener so that we don't not leak memory but in a
              // way that will achieve the prevention of the default action.
              formElement.on('$destroy', function() {
                $timeout(function() {
                  formElement[0].removeEventListener('submit', handleFormSubmission);
                }, 0, false);
              });
            }

            var parentFormCtrl = ctrls[1] || controller.$$parentForm;
            parentFormCtrl.$addControl(controller);

            var setter = nameAttr ? getSetter(controller.$name) : noop;

            if (nameAttr) {
              setter(scope, controller);
              attr.$observe(nameAttr, function(newValue) {
                if (controller.$name === newValue) return;
                setter(scope, undefined);
                controller.$$parentForm.$$renameControl(controller, newValue);
                setter = getSetter(controller.$name);
                setter(scope, controller);
              });
            }
            formElement.on('$destroy', function() {
              controller.$$parentForm.$removeControl(controller);
              setter(scope, undefined);
              extend(controller, nullFormCtrl); //stop propagating child destruction handlers upwards
            });
          }
        };
      }
    };

    return formDirective;

    function getSetter(expression) {
      if (expression === '') {
        //create an assignable expression, so forms with an empty name can be renamed later
        return $parse('this[""]').assign;
      }
      return $parse(expression).assign || noop;
    }
  }];
};

var formDirective = formDirectiveFactory();
var ngFormDirective = formDirectiveFactory(true);



// helper methods
function setupValidity(instance) {
  instance.$$classCache = {};
  instance.$$classCache[INVALID_CLASS] = !(instance.$$classCache[VALID_CLASS] = instance.$$element.hasClass(VALID_CLASS));
}
function addSetValidityMethod(context) {
  var clazz = context.clazz,
      set = context.set,
      unset = context.unset;

  clazz.prototype.$setValidity = function(validationErrorKey, state, controller) {
    if (isUndefined(state)) {
      createAndSet(this, '$pending', validationErrorKey, controller);
    } else {
      unsetAndCleanup(this, '$pending', validationErrorKey, controller);
    }
    if (!isBoolean(state)) {
      unset(this.$error, validationErrorKey, controller);
      unset(this.$$success, validationErrorKey, controller);
    } else {
      if (state) {
        unset(this.$error, validationErrorKey, controller);
        set(this.$$success, validationErrorKey, controller);
      } else {
        set(this.$error, validationErrorKey, controller);
        unset(this.$$success, validationErrorKey, controller);
      }
    }
    if (this.$pending) {
      cachedToggleClass(this, PENDING_CLASS, true);
      this.$valid = this.$invalid = undefined;
      toggleValidationCss(this, '', null);
    } else {
      cachedToggleClass(this, PENDING_CLASS, false);
      this.$valid = isObjectEmpty(this.$error);
      this.$invalid = !this.$valid;
      toggleValidationCss(this, '', this.$valid);
    }

    // re-read the state as the set/unset methods could have
    // combined state in this.$error[validationError] (used for forms),
    // where setting/unsetting only increments/decrements the value,
    // and does not replace it.
    var combinedState;
    if (this.$pending && this.$pending[validationErrorKey]) {
      combinedState = undefined;
    } else if (this.$error[validationErrorKey]) {
      combinedState = false;
    } else if (this.$$success[validationErrorKey]) {
      combinedState = true;
    } else {
      combinedState = null;
    }

    toggleValidationCss(this, validationErrorKey, combinedState);
    this.$$parentForm.$setValidity(validationErrorKey, combinedState, this);
  };

  function createAndSet(ctrl, name, value, controller) {
    if (!ctrl[name]) {
      ctrl[name] = {};
    }
    set(ctrl[name], value, controller);
  }

  function unsetAndCleanup(ctrl, name, value, controller) {
    if (ctrl[name]) {
      unset(ctrl[name], value, controller);
    }
    if (isObjectEmpty(ctrl[name])) {
      ctrl[name] = undefined;
    }
  }

  function cachedToggleClass(ctrl, className, switchValue) {
    if (switchValue && !ctrl.$$classCache[className]) {
      ctrl.$$animate.addClass(ctrl.$$element, className);
      ctrl.$$classCache[className] = true;
    } else if (!switchValue && ctrl.$$classCache[className]) {
      ctrl.$$animate.removeClass(ctrl.$$element, className);
      ctrl.$$classCache[className] = false;
    }
  }

  function toggleValidationCss(ctrl, validationErrorKey, isValid) {
    validationErrorKey = validationErrorKey ? '-' + snake_case(validationErrorKey, '-') : '';

    cachedToggleClass(ctrl, VALID_CLASS + validationErrorKey, isValid === true);
    cachedToggleClass(ctrl, INVALID_CLASS + validationErrorKey, isValid === false);
  }
}

function isObjectEmpty(obj) {
  if (obj) {
    for (var prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        return false;
      }
    }
  }
  return true;
}

/* global
  VALID_CLASS: false,
  INVALID_CLASS: false,
  PRISTINE_CLASS: false,
  DIRTY_CLASS: false,
  ngModelMinErr: false
*/

// Regex code was initially obtained from SO prior to modification: https://stackoverflow.com/questions/3143070/javascript-regex-iso-datetime#answer-3143231
var ISO_DATE_REGEXP = /^\d{4,}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+(?:[+-][0-2]\d:[0-5]\d|Z)$/;
// See valid URLs in RFC3987 (http://tools.ietf.org/html/rfc3987)
// Note: We are being more lenient, because browsers are too.
//   1. Scheme
//   2. Slashes
//   3. Username
//   4. Password
//   5. Hostname
//   6. Port
//   7. Path
//   8. Query
//   9. Fragment
//                 1111111111111111 222   333333    44444        55555555555555555555555     666     77777777     8888888     999
var URL_REGEXP = /^[a-z][a-z\d.+-]*:\/*(?:[^:@]+(?::[^@]+)?@)?(?:[^\s:/?#]+|\[[a-f\d:]+])(?::\d+)?(?:\/[^?#]*)?(?:\?[^#]*)?(?:#.*)?$/i;
// eslint-disable-next-line max-len
var EMAIL_REGEXP = /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/;
var NUMBER_REGEXP = /^\s*(-|\+)?(\d+|(\d*(\.\d*)))([eE][+-]?\d+)?\s*$/;
var DATE_REGEXP = /^(\d{4,})-(\d{2})-(\d{2})$/;
var DATETIMELOCAL_REGEXP = /^(\d{4,})-(\d\d)-(\d\d)T(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;
var WEEK_REGEXP = /^(\d{4,})-W(\d\d)$/;
var MONTH_REGEXP = /^(\d{4,})-(\d\d)$/;
var TIME_REGEXP = /^(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;

var PARTIAL_VALIDATION_EVENTS = 'keydown wheel mousedown';
var PARTIAL_VALIDATION_TYPES = createMap();
forEach('date,datetime-local,month,time,week'.split(','), function(type) {
  PARTIAL_VALIDATION_TYPES[type] = true;
});

var inputType = {

  /**
   * @ngdoc input
   * @name input[text]
   *
   * @description
   * Standard HTML text input with AngularJS data binding, inherited by most of the `input` elements.
   *
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} required Adds `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
   *    minlength.
   * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
   *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
   *    any length.
   * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
   *    that contains the regular expression body that will be converted to a regular expression
   *    as in the ngPattern directive.
   * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
   *    does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
   *    If the expression evaluates to a RegExp object, then this is used directly.
   *    If the expression evaluates to a string, then it will be converted to a RegExp
   *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
   *    `new RegExp('^abc$')`.<br />
   *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
   *    start at the index of the last search's match, thus not taking the whole input value into
   *    account.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   * @param {boolean=} [ngTrim=true] If set to false AngularJS will not automatically trim the input.
   *    This parameter is ignored for input[type=password] controls, which will never trim the
   *    input.
   *
   * @example
      <example name="text-input-directive" module="textInputExample">
        <file name="index.html">
         <script>
           angular.module('textInputExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.example = {
                 text: 'guest',
                 word: /^\s*\w*\s*$/
               };
             }]);
         </script>
         <form name="myForm" ng-controller="ExampleController">
           <label>Single word:
             <input type="text" name="input" ng-model="example.text"
                    ng-pattern="example.word" required ng-trim="false">
           </label>
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
               Required!</span>
             <span class="error" ng-show="myForm.input.$error.pattern">
               Single word only!</span>
           </div>
           <code>text = {{example.text}}</code><br/>
           <code>myForm.input.$valid = {{myForm.input.$valid}}</code><br/>
           <code>myForm.input.$error = {{myForm.input.$error}}</code><br/>
           <code>myForm.$valid = {{myForm.$valid}}</code><br/>
           <code>myForm.$error.required = {{!!myForm.$error.required}}</code><br/>
          </form>
        </file>
        <file name="protractor.js" type="protractor">
          var text = element(by.binding('example.text'));
          var valid = element(by.binding('myForm.input.$valid'));
          var input = element(by.model('example.text'));

          it('should initialize to model', function() {
            expect(text.getText()).toContain('guest');
            expect(valid.getText()).toContain('true');
          });

          it('should be invalid if empty', function() {
            input.clear();
            input.sendKeys('');

            expect(text.getText()).toEqual('text =');
            expect(valid.getText()).toContain('false');
          });

          it('should be invalid if multi word', function() {
            input.clear();
            input.sendKeys('hello world');

            expect(valid.getText()).toContain('false');
          });
        </file>
      </example>
   */
  'text': textInputType,

    /**
     * @ngdoc input
     * @name input[date]
     *
     * @description
     * Input with date validation and transformation. In browsers that do not yet support
     * the HTML5 date input, a text element will be used. In that case, text must be entered in a valid ISO-8601
     * date format (yyyy-MM-dd), for example: `2009-01-06`. Since many
     * modern browsers do not yet support this input type, it is important to provide cues to users on the
     * expected input format via a placeholder or label.
     *
     * The model must always be a Date object, otherwise AngularJS will throw an error.
     * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
     *
     * The timezone to be used to read/write the `Date` instance in the model can be defined using
     * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
     *
     * @param {string} ngModel Assignable AngularJS expression to data-bind to.
     * @param {string=} name Property name of the form under which the control is published.
     * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`. This must be a
     *   valid ISO date string (yyyy-MM-dd). You can also use interpolation inside this attribute
     *   (e.g. `min="{{minDate | date:'yyyy-MM-dd'}}"`). Note that `min` will also add native HTML5
     *   constraint validation.
     * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`. This must be
     *   a valid ISO date string (yyyy-MM-dd). You can also use interpolation inside this attribute
     *   (e.g. `max="{{maxDate | date:'yyyy-MM-dd'}}"`). Note that `max` will also add native HTML5
     *   constraint validation.
     * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO date string
     *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
     * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO date string
     *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
     * @param {string=} required Sets `required` validation error key if the value is not entered.
     * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
     *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
     *    `required` when you want to data-bind to the `required` attribute.
     * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
     *    interaction with the input element.
     *
     * @example
     <example name="date-input-directive" module="dateInputExample">
     <file name="index.html">
       <script>
          angular.module('dateInputExample', [])
            .controller('DateController', ['$scope', function($scope) {
              $scope.example = {
                value: new Date(2013, 9, 22)
              };
            }]);
       </script>
       <form name="myForm" ng-controller="DateController as dateCtrl">
          <label for="exampleInput">Pick a date in 2013:</label>
          <input type="date" id="exampleInput" name="input" ng-model="example.value"
              placeholder="yyyy-MM-dd" min="2013-01-01" max="2013-12-31" required />
          <div role="alert">
            <span class="error" ng-show="myForm.input.$error.required">
                Required!</span>
            <span class="error" ng-show="myForm.input.$error.date">
                Not a valid date!</span>
           </div>
           <tt>value = {{example.value | date: "yyyy-MM-dd"}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
       </form>
     </file>
     <file name="protractor.js" type="protractor">
        var value = element(by.binding('example.value | date: "yyyy-MM-dd"'));
        var valid = element(by.binding('myForm.input.$valid'));

        // currently protractor/webdriver does not support
        // sending keys to all known HTML5 input controls
        // for various browsers (see https://github.com/angular/protractor/issues/562).
        function setInput(val) {
          // set the value of the element and force validation.
          var scr = "var ipt = document.getElementById('exampleInput'); " +
          "ipt.value = '" + val + "';" +
          "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
          browser.executeScript(scr);
        }

        it('should initialize to model', function() {
          expect(value.getText()).toContain('2013-10-22');
          expect(valid.getText()).toContain('myForm.input.$valid = true');
        });

        it('should be invalid if empty', function() {
          setInput('');
          expect(value.getText()).toEqual('value =');
          expect(valid.getText()).toContain('myForm.input.$valid = false');
        });

        it('should be invalid if over max', function() {
          setInput('2015-01-01');
          expect(value.getText()).toContain('');
          expect(valid.getText()).toContain('myForm.input.$valid = false');
        });
     </file>
     </example>
     */
  'date': createDateInputType('date', DATE_REGEXP,
         createDateParser(DATE_REGEXP, ['yyyy', 'MM', 'dd']),
         'yyyy-MM-dd'),

   /**
    * @ngdoc input
    * @name input[datetime-local]
    *
    * @description
    * Input with datetime validation and transformation. In browsers that do not yet support
    * the HTML5 date input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
    * local datetime format (yyyy-MM-ddTHH:mm:ss), for example: `2010-12-28T14:57:00`.
    *
    * The model must always be a Date object, otherwise AngularJS will throw an error.
    * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
    *
    * The timezone to be used to read/write the `Date` instance in the model can be defined using
    * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
    *
    * The format of the displayed time can be adjusted with the
    * {@link ng.directive:ngModelOptions#ngModelOptions-arguments ngModelOptions} `timeSecondsFormat`
    * and `timeStripZeroSeconds`.
    *
    * @param {string} ngModel Assignable AngularJS expression to data-bind to.
    * @param {string=} name Property name of the form under which the control is published.
    * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
    *   This must be a valid ISO datetime format (yyyy-MM-ddTHH:mm:ss). You can also use interpolation
    *   inside this attribute (e.g. `min="{{minDatetimeLocal | date:'yyyy-MM-ddTHH:mm:ss'}}"`).
    *   Note that `min` will also add native HTML5 constraint validation.
    * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
    *   This must be a valid ISO datetime format (yyyy-MM-ddTHH:mm:ss). You can also use interpolation
    *   inside this attribute (e.g. `max="{{maxDatetimeLocal | date:'yyyy-MM-ddTHH:mm:ss'}}"`).
    *   Note that `max` will also add native HTML5 constraint validation.
    * @param {(date|string)=} ngMin Sets the `min` validation error key to the Date / ISO datetime string
    *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
    * @param {(date|string)=} ngMax Sets the `max` validation error key to the Date / ISO datetime string
    *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
    * @param {string=} required Sets `required` validation error key if the value is not entered.
    * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
    *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
    *    `required` when you want to data-bind to the `required` attribute.
    * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
    *    interaction with the input element.
    *
    * @example
    <example name="datetimelocal-input-directive" module="dateExample">
    <file name="index.html">
      <script>
        angular.module('dateExample', [])
          .controller('DateController', ['$scope', function($scope) {
            $scope.example = {
              value: new Date(2010, 11, 28, 14, 57)
            };
          }]);
      </script>
      <form name="myForm" ng-controller="DateController as dateCtrl">
        <label for="exampleInput">Pick a date between in 2013:</label>
        <input type="datetime-local" id="exampleInput" name="input" ng-model="example.value"
            placeholder="yyyy-MM-ddTHH:mm:ss" min="2001-01-01T00:00:00" max="2013-12-31T00:00:00" required />
        <div role="alert">
          <span class="error" ng-show="myForm.input.$error.required">
              Required!</span>
          <span class="error" ng-show="myForm.input.$error.datetimelocal">
              Not a valid date!</span>
        </div>
        <tt>value = {{example.value | date: "yyyy-MM-ddTHH:mm:ss"}}</tt><br/>
        <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
        <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
        <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
        <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
      </form>
    </file>
    <file name="protractor.js" type="protractor">
      var value = element(by.binding('example.value | date: "yyyy-MM-ddTHH:mm:ss"'));
      var valid = element(by.binding('myForm.input.$valid'));

      // currently protractor/webdriver does not support
      // sending keys to all known HTML5 input controls
      // for various browsers (https://github.com/angular/protractor/issues/562).
      function setInput(val) {
        // set the value of the element and force validation.
        var scr = "var ipt = document.getElementById('exampleInput'); " +
        "ipt.value = '" + val + "';" +
        "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
        browser.executeScript(scr);
      }

      it('should initialize to model', function() {
        expect(value.getText()).toContain('2010-12-28T14:57:00');
        expect(valid.getText()).toContain('myForm.input.$valid = true');
      });

      it('should be invalid if empty', function() {
        setInput('');
        expect(value.getText()).toEqual('value =');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });

      it('should be invalid if over max', function() {
        setInput('2015-01-01T23:59:00');
        expect(value.getText()).toContain('');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });
    </file>
    </example>
    */
  'datetime-local': createDateInputType('datetimelocal', DATETIMELOCAL_REGEXP,
      createDateParser(DATETIMELOCAL_REGEXP, ['yyyy', 'MM', 'dd', 'HH', 'mm', 'ss', 'sss']),
      'yyyy-MM-ddTHH:mm:ss.sss'),

  /**
   * @ngdoc input
   * @name input[time]
   *
   * @description
   * Input with time validation and transformation. In browsers that do not yet support
   * the HTML5 time input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
   * local time format (HH:mm:ss), for example: `14:57:00`. Model must be a Date object. This binding will always output a
   * Date object to the model of January 1, 1970, or local date `new Date(1970, 0, 1, HH, mm, ss)`.
   *
   * The model must always be a Date object, otherwise AngularJS will throw an error.
   * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
   *
   * The timezone to be used to read/write the `Date` instance in the model can be defined using
   * {@link ng.directive:ngModelOptions#ngModelOptions-arguments ngModelOptions}. By default,
   * this is the timezone of the browser.
   *
   * The format of the displayed time can be adjusted with the
   * {@link ng.directive:ngModelOptions#ngModelOptions-arguments ngModelOptions} `timeSecondsFormat`
   * and `timeStripZeroSeconds`.
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
   *   This must be a valid ISO time format (HH:mm:ss). You can also use interpolation inside this
   *   attribute (e.g. `min="{{minTime | date:'HH:mm:ss'}}"`). Note that `min` will also add
   *   native HTML5 constraint validation.
   * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
   *   This must be a valid ISO time format (HH:mm:ss). You can also use interpolation inside this
   *   attribute (e.g. `max="{{maxTime | date:'HH:mm:ss'}}"`). Note that `max` will also add
   *   native HTML5 constraint validation.
   * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO time string the
   *   `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
   * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO time string the
   *   `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
   * @param {string=} required Sets `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
   <example name="time-input-directive" module="timeExample">
   <file name="index.html">
     <script>
      angular.module('timeExample', [])
        .controller('DateController', ['$scope', function($scope) {
          $scope.example = {
            value: new Date(1970, 0, 1, 14, 57, 0)
          };
        }]);
     </script>
     <form name="myForm" ng-controller="DateController as dateCtrl">
        <label for="exampleInput">Pick a time between 8am and 5pm:</label>
        <input type="time" id="exampleInput" name="input" ng-model="example.value"
            placeholder="HH:mm:ss" min="08:00:00" max="17:00:00" required />
        <div role="alert">
          <span class="error" ng-show="myForm.input.$error.required">
              Required!</span>
          <span class="error" ng-show="myForm.input.$error.time">
              Not a valid date!</span>
        </div>
        <tt>value = {{example.value | date: "HH:mm:ss"}}</tt><br/>
        <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
        <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
        <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
        <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
     </form>
   </file>
   <file name="protractor.js" type="protractor">
      var value = element(by.binding('example.value | date: "HH:mm:ss"'));
      var valid = element(by.binding('myForm.input.$valid'));

      // currently protractor/webdriver does not support
      // sending keys to all known HTML5 input controls
      // for various browsers (https://github.com/angular/protractor/issues/562).
      function setInput(val) {
        // set the value of the element and force validation.
        var scr = "var ipt = document.getElementById('exampleInput'); " +
        "ipt.value = '" + val + "';" +
        "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
        browser.executeScript(scr);
      }

      it('should initialize to model', function() {
        expect(value.getText()).toContain('14:57:00');
        expect(valid.getText()).toContain('myForm.input.$valid = true');
      });

      it('should be invalid if empty', function() {
        setInput('');
        expect(value.getText()).toEqual('value =');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });

      it('should be invalid if over max', function() {
        setInput('23:59:00');
        expect(value.getText()).toContain('');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });
   </file>
   </example>
   */
  'time': createDateInputType('time', TIME_REGEXP,
      createDateParser(TIME_REGEXP, ['HH', 'mm', 'ss', 'sss']),
     'HH:mm:ss.sss'),

   /**
    * @ngdoc input
    * @name input[week]
    *
    * @description
    * Input with week-of-the-year validation and transformation to Date. In browsers that do not yet support
    * the HTML5 week input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
    * week format (yyyy-W##), for example: `2013-W02`.
    *
    * The model must always be a Date object, otherwise AngularJS will throw an error.
    * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
    *
    * The value of the resulting Date object will be set to Thursday at 00:00:00 of the requested week,
    * due to ISO-8601 week numbering standards. Information on ISO's system for numbering the weeks of the
    * year can be found at: https://en.wikipedia.org/wiki/ISO_8601#Week_dates
    *
    * The timezone to be used to read/write the `Date` instance in the model can be defined using
    * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
    *
    * @param {string} ngModel Assignable AngularJS expression to data-bind to.
    * @param {string=} name Property name of the form under which the control is published.
    * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
    *   This must be a valid ISO week format (yyyy-W##). You can also use interpolation inside this
    *   attribute (e.g. `min="{{minWeek | date:'yyyy-Www'}}"`). Note that `min` will also add
    *   native HTML5 constraint validation.
    * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
    *   This must be a valid ISO week format (yyyy-W##). You can also use interpolation inside this
    *   attribute (e.g. `max="{{maxWeek | date:'yyyy-Www'}}"`). Note that `max` will also add
    *   native HTML5 constraint validation.
    * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO week string
    *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
    * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO week string
    *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
    * @param {string=} required Sets `required` validation error key if the value is not entered.
    * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
    *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
    *    `required` when you want to data-bind to the `required` attribute.
    * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
    *    interaction with the input element.
    *
    * @example
    <example name="week-input-directive" module="weekExample">
    <file name="index.html">
      <script>
      angular.module('weekExample', [])
        .controller('DateController', ['$scope', function($scope) {
          $scope.example = {
            value: new Date(2013, 0, 3)
          };
        }]);
      </script>
      <form name="myForm" ng-controller="DateController as dateCtrl">
        <label>Pick a date between in 2013:
          <input id="exampleInput" type="week" name="input" ng-model="example.value"
                 placeholder="YYYY-W##" min="2012-W32"
                 max="2013-W52" required />
        </label>
        <div role="alert">
          <span class="error" ng-show="myForm.input.$error.required">
              Required!</span>
          <span class="error" ng-show="myForm.input.$error.week">
              Not a valid date!</span>
        </div>
        <tt>value = {{example.value | date: "yyyy-Www"}}</tt><br/>
        <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
        <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
        <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
        <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
      </form>
    </file>
    <file name="protractor.js" type="protractor">
      var value = element(by.binding('example.value | date: "yyyy-Www"'));
      var valid = element(by.binding('myForm.input.$valid'));

      // currently protractor/webdriver does not support
      // sending keys to all known HTML5 input controls
      // for various browsers (https://github.com/angular/protractor/issues/562).
      function setInput(val) {
        // set the value of the element and force validation.
        var scr = "var ipt = document.getElementById('exampleInput'); " +
        "ipt.value = '" + val + "';" +
        "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
        browser.executeScript(scr);
      }

      it('should initialize to model', function() {
        expect(value.getText()).toContain('2013-W01');
        expect(valid.getText()).toContain('myForm.input.$valid = true');
      });

      it('should be invalid if empty', function() {
        setInput('');
        expect(value.getText()).toEqual('value =');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });

      it('should be invalid if over max', function() {
        setInput('2015-W01');
        expect(value.getText()).toContain('');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });
    </file>
    </example>
    */
  'week': createDateInputType('week', WEEK_REGEXP, weekParser, 'yyyy-Www'),

  /**
   * @ngdoc input
   * @name input[month]
   *
   * @description
   * Input with month validation and transformation. In browsers that do not yet support
   * the HTML5 month input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
   * month format (yyyy-MM), for example: `2009-01`.
   *
   * The model must always be a Date object, otherwise AngularJS will throw an error.
   * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
   * If the model is not set to the first of the month, the next view to model update will set it
   * to the first of the month.
   *
   * The timezone to be used to read/write the `Date` instance in the model can be defined using
   * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
   *   This must be a valid ISO month format (yyyy-MM). You can also use interpolation inside this
   *   attribute (e.g. `min="{{minMonth | date:'yyyy-MM'}}"`). Note that `min` will also add
   *   native HTML5 constraint validation.
   * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
   *   This must be a valid ISO month format (yyyy-MM). You can also use interpolation inside this
   *   attribute (e.g. `max="{{maxMonth | date:'yyyy-MM'}}"`). Note that `max` will also add
   *   native HTML5 constraint validation.
   * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO week string
   *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
   * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO week string
   *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.

   * @param {string=} required Sets `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
   <example name="month-input-directive" module="monthExample">
   <file name="index.html">
     <script>
      angular.module('monthExample', [])
        .controller('DateController', ['$scope', function($scope) {
          $scope.example = {
            value: new Date(2013, 9, 1)
          };
        }]);
     </script>
     <form name="myForm" ng-controller="DateController as dateCtrl">
       <label for="exampleInput">Pick a month in 2013:</label>
       <input id="exampleInput" type="month" name="input" ng-model="example.value"
          placeholder="yyyy-MM" min="2013-01" max="2013-12" required />
       <div role="alert">
         <span class="error" ng-show="myForm.input.$error.required">
            Required!</span>
         <span class="error" ng-show="myForm.input.$error.month">
            Not a valid month!</span>
       </div>
       <tt>value = {{example.value | date: "yyyy-MM"}}</tt><br/>
       <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
       <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
       <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
       <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
     </form>
   </file>
   <file name="protractor.js" type="protractor">
      var value = element(by.binding('example.value | date: "yyyy-MM"'));
      var valid = element(by.binding('myForm.input.$valid'));

      // currently protractor/webdriver does not support
      // sending keys to all known HTML5 input controls
      // for various browsers (https://github.com/angular/protractor/issues/562).
      function setInput(val) {
        // set the value of the element and force validation.
        var scr = "var ipt = document.getElementById('exampleInput'); " +
        "ipt.value = '" + val + "';" +
        "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
        browser.executeScript(scr);
      }

      it('should initialize to model', function() {
        expect(value.getText()).toContain('2013-10');
        expect(valid.getText()).toContain('myForm.input.$valid = true');
      });

      it('should be invalid if empty', function() {
        setInput('');
        expect(value.getText()).toEqual('value =');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });

      it('should be invalid if over max', function() {
        setInput('2015-01');
        expect(value.getText()).toContain('');
        expect(valid.getText()).toContain('myForm.input.$valid = false');
      });
   </file>
   </example>
   */
  'month': createDateInputType('month', MONTH_REGEXP,
     createDateParser(MONTH_REGEXP, ['yyyy', 'MM']),
     'yyyy-MM'),

  /**
   * @ngdoc input
   * @name input[number]
   *
   * @description
   * Text input with number validation and transformation. Sets the `number` validation
   * error if not a valid number.
   *
   * <div class="alert alert-warning">
   * The model must always be of type `number` otherwise AngularJS will throw an error.
   * Be aware that a string containing a number is not enough. See the {@link ngModel:numfmt}
   * error docs for more information and an example of how to convert your model if necessary.
   * </div>
   *
   *
   *
   * @knownIssue
   *
   * ### HTML5 constraint validation and `allowInvalid`
   *
   * In browsers that follow the
   * [HTML5 specification](https://html.spec.whatwg.org/multipage/forms.html#number-state-%28type=number%29),
   * `input[number]` does not work as expected with {@link ngModelOptions `ngModelOptions.allowInvalid`}.
   * If a non-number is entered in the input, the browser will report the value as an empty string,
   * which means the view / model values in `ngModel` and subsequently the scope value
   * will also be an empty string.
   *
   * @knownIssue
   *
   * ### Large numbers and `step` validation
   *
   * The `step` validation will not work correctly for very large numbers (e.g. 9999999999) due to
   * Javascript's arithmetic limitations. If you need to handle large numbers, purpose-built
   * libraries (e.g. https://github.com/MikeMcl/big.js/), can be included into AngularJS by
   * {@link guide/forms#modifying-built-in-validators overwriting the validators}
   * for `number` and / or `step`, or by {@link guide/forms#custom-validation applying custom validators}
   * to an `input[text]` element. The source for `input[number]` type can be used as a starting
   * point for both implementations.
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
   *    Can be interpolated.
   * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
   *    Can be interpolated.
   * @param {string=} ngMin Like `min`, sets the `min` validation error key if the value entered is less than `ngMin`,
   *    but does not trigger HTML5 native validation. Takes an expression.
   * @param {string=} ngMax Like `max`, sets the `max` validation error key if the value entered is greater than `ngMax`,
   *    but does not trigger HTML5 native validation. Takes an expression.
   * @param {string=} step Sets the `step` validation error key if the value entered does not fit the `step` constraint.
   *    Can be interpolated.
   * @param {string=} ngStep Like `step`, sets the `step` validation error key if the value entered does not fit the `ngStep` constraint,
   *    but does not trigger HTML5 native validation. Takes an expression.
   * @param {string=} required Sets `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
   *    minlength.
   * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
   *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
   *    any length.
   * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
   *    that contains the regular expression body that will be converted to a regular expression
   *    as in the ngPattern directive.
   * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
   *    does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
   *    If the expression evaluates to a RegExp object, then this is used directly.
   *    If the expression evaluates to a string, then it will be converted to a RegExp
   *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
   *    `new RegExp('^abc$')`.<br />
   *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
   *    start at the index of the last search's match, thus not taking the whole input value into
   *    account.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
      <example name="number-input-directive" module="numberExample">
        <file name="index.html">
         <script>
           angular.module('numberExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.example = {
                 value: 12
               };
             }]);
         </script>
         <form name="myForm" ng-controller="ExampleController">
           <label>Number:
             <input type="number" name="input" ng-model="example.value"
                    min="0" max="99" required>
          </label>
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
               Required!</span>
             <span class="error" ng-show="myForm.input.$error.number">
               Not valid number!</span>
           </div>
           <tt>value = {{example.value}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
          </form>
        </file>
        <file name="protractor.js" type="protractor">
          var value = element(by.binding('example.value'));
          var valid = element(by.binding('myForm.input.$valid'));
          var input = element(by.model('example.value'));

          it('should initialize to model', function() {
            expect(value.getText()).toContain('12');
            expect(valid.getText()).toContain('true');
          });

          it('should be invalid if empty', function() {
            input.clear();
            input.sendKeys('');
            expect(value.getText()).toEqual('value =');
            expect(valid.getText()).toContain('false');
          });

          it('should be invalid if over max', function() {
            input.clear();
            input.sendKeys('123');
            expect(value.getText()).toEqual('value =');
            expect(valid.getText()).toContain('false');
          });
        </file>
      </example>
   */
  'number': numberInputType,


  /**
   * @ngdoc input
   * @name input[url]
   *
   * @description
   * Text input with URL validation. Sets the `url` validation error key if the content is not a
   * valid URL.
   *
   * <div class="alert alert-warning">
   * **Note:** `input[url]` uses a regex to validate urls that is derived from the regex
   * used in Chromium. If you need stricter validation, you can use `ng-pattern` or modify
   * the built-in validators (see the {@link guide/forms Forms guide})
   * </div>
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} required Sets `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
   *    minlength.
   * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
   *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
   *    any length.
   * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
   *    that contains the regular expression body that will be converted to a regular expression
   *    as in the ngPattern directive.
   * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
   *    does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
   *    If the expression evaluates to a RegExp object, then this is used directly.
   *    If the expression evaluates to a string, then it will be converted to a RegExp
   *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
   *    `new RegExp('^abc$')`.<br />
   *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
   *    start at the index of the last search's match, thus not taking the whole input value into
   *    account.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
      <example name="url-input-directive" module="urlExample">
        <file name="index.html">
         <script>
           angular.module('urlExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.url = {
                 text: 'http://google.com'
               };
             }]);
         </script>
         <form name="myForm" ng-controller="ExampleController">
           <label>URL:
             <input type="url" name="input" ng-model="url.text" required>
           <label>
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
               Required!</span>
             <span class="error" ng-show="myForm.input.$error.url">
               Not valid url!</span>
           </div>
           <tt>text = {{url.text}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
           <tt>myForm.$error.url = {{!!myForm.$error.url}}</tt><br/>
          </form>
        </file>
        <file name="protractor.js" type="protractor">
          var text = element(by.binding('url.text'));
          var valid = element(by.binding('myForm.input.$valid'));
          var input = element(by.model('url.text'));

          it('should initialize to model', function() {
            expect(text.getText()).toContain('http://google.com');
            expect(valid.getText()).toContain('true');
          });

          it('should be invalid if empty', function() {
            input.clear();
            input.sendKeys('');

            expect(text.getText()).toEqual('text =');
            expect(valid.getText()).toContain('false');
          });

          it('should be invalid if not url', function() {
            input.clear();
            input.sendKeys('box');

            expect(valid.getText()).toContain('false');
          });
        </file>
      </example>
   */
  'url': urlInputType,


  /**
   * @ngdoc input
   * @name input[email]
   *
   * @description
   * Text input with email validation. Sets the `email` validation error key if not a valid email
   * address.
   *
   * <div class="alert alert-warning">
   * **Note:** `input[email]` uses a regex to validate email addresses that is derived from the regex
   * used in Chromium, which may not fulfill your app's requirements.
   * If you need stricter (e.g. requiring a top-level domain), or more relaxed validation
   * (e.g. allowing IPv6 address literals) you can use `ng-pattern` or
   * modify the built-in validators (see the {@link guide/forms Forms guide}).
   * </div>
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} required Sets `required` validation error key if the value is not entered.
   * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
   *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
   *    `required` when you want to data-bind to the `required` attribute.
   * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
   *    minlength.
   * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
   *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
   *    any length.
   * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
   *    that contains the regular expression body that will be converted to a regular expression
   *    as in the ngPattern directive.
   * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
   *    does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
   *    If the expression evaluates to a RegExp object, then this is used directly.
   *    If the expression evaluates to a string, then it will be converted to a RegExp
   *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
   *    `new RegExp('^abc$')`.<br />
   *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
   *    start at the index of the last search's match, thus not taking the whole input value into
   *    account.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
      <example name="email-input-directive" module="emailExample">
        <file name="index.html">
         <script>
           angular.module('emailExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.email = {
                 text: 'me@example.com'
               };
             }]);
         </script>
           <form name="myForm" ng-controller="ExampleController">
             <label>Email:
               <input type="email" name="input" ng-model="email.text" required>
             </label>
             <div role="alert">
               <span class="error" ng-show="myForm.input.$error.required">
                 Required!</span>
               <span class="error" ng-show="myForm.input.$error.email">
                 Not valid email!</span>
             </div>
             <tt>text = {{email.text}}</tt><br/>
             <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
             <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
             <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
             <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
             <tt>myForm.$error.email = {{!!myForm.$error.email}}</tt><br/>
           </form>
         </file>
        <file name="protractor.js" type="protractor">
          var text = element(by.binding('email.text'));
          var valid = element(by.binding('myForm.input.$valid'));
          var input = element(by.model('email.text'));

          it('should initialize to model', function() {
            expect(text.getText()).toContain('me@example.com');
            expect(valid.getText()).toContain('true');
          });

          it('should be invalid if empty', function() {
            input.clear();
            input.sendKeys('');
            expect(text.getText()).toEqual('text =');
            expect(valid.getText()).toContain('false');
          });

          it('should be invalid if not email', function() {
            input.clear();
            input.sendKeys('xxx');

            expect(valid.getText()).toContain('false');
          });
        </file>
      </example>
   */
  'email': emailInputType,


  /**
   * @ngdoc input
   * @name input[radio]
   *
   * @description
   * HTML radio button.
   *
   * **Note:**<br>
   * All inputs controlled by {@link ngModel ngModel} (including those of type `radio`) will use the
   * value of their `name` attribute to determine the property under which their
   * {@link ngModel.NgModelController NgModelController} will be published on the parent
   * {@link form.FormController FormController}. Thus, if you use the same `name` for multiple
   * inputs of a form (e.g. a group of radio inputs), only _one_ `NgModelController` will be
   * published on the parent `FormController` under that name. The rest of the controllers will
   * continue to work as expected, but you won't be able to access them as properties on the parent
   * `FormController`.
   *
   * <div class="alert alert-info">
   *   <p>
   *     In plain HTML forms, the `name` attribute is used to identify groups of radio inputs, so
   *     that the browser can manage their state (checked/unchecked) based on the state of other
   *     inputs in the same group.
   *   </p>
   *   <p>
   *     In AngularJS forms, this is not necessary. The input's state will be updated based on the
   *     value of the underlying model data.
   *   </p>
   * </div>
   *
   * <div class="alert alert-success">
   *   If you omit the `name` attribute on a radio input, `ngModel` will automatically assign it a
   *   unique name.
   * </div>
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string} value The value to which the `ngModel` expression should be set when selected.
   *    Note that `value` only supports `string` values, i.e. the scope model needs to be a string,
   *    too. Use `ngValue` if you need complex models (`number`, `object`, ...).
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   * @param {string} ngValue AngularJS expression to which `ngModel` will be be set when the radio
   *    is selected. Should be used instead of the `value` attribute if you need
   *    a non-string `ngModel` (`boolean`, `array`, ...).
   *
   * @example
      <example name="radio-input-directive" module="radioExample">
        <file name="index.html">
         <script>
           angular.module('radioExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.color = {
                 name: 'blue'
               };
               $scope.specialValue = {
                 "id": "12345",
                 "value": "green"
               };
             }]);
         </script>
         <form name="myForm" ng-controller="ExampleController">
           <label>
             <input type="radio" ng-model="color.name" value="red">
             Red
           </label><br/>
           <label>
             <input type="radio" ng-model="color.name" ng-value="specialValue">
             Green
           </label><br/>
           <label>
             <input type="radio" ng-model="color.name" value="blue">
             Blue
           </label><br/>
           <tt>color = {{color.name | json}}</tt><br/>
          </form>
          Note that `ng-value="specialValue"` sets radio item's value to be the value of `$scope.specialValue`.
        </file>
        <file name="protractor.js" type="protractor">
          it('should change state', function() {
            var inputs = element.all(by.model('color.name'));
            var color = element(by.binding('color.name'));

            expect(color.getText()).toContain('blue');

            inputs.get(0).click();
            expect(color.getText()).toContain('red');

            inputs.get(1).click();
            expect(color.getText()).toContain('green');
          });
        </file>
      </example>
   */
  'radio': radioInputType,

  /**
   * @ngdoc input
   * @name input[range]
   *
   * @description
   * Native range input with validation and transformation.
   *
   * The model for the range input must always be a `Number`.
   *
   * IE9 and other browsers that do not support the `range` type fall back
   * to a text input without any default values for `min`, `max` and `step`. Model binding,
   * validation and number parsing are nevertheless supported.
   *
   * Browsers that support range (latest Chrome, Safari, Firefox, Edge) treat `input[range]`
   * in a way that never allows the input to hold an invalid value. That means:
   * - any non-numerical value is set to `(max + min) / 2`.
   * - any numerical value that is less than the current min val, or greater than the current max val
   * is set to the min / max val respectively.
   * - additionally, the current `step` is respected, so the nearest value that satisfies a step
   * is used.
   *
   * See the [HTML Spec on input[type=range]](https://www.w3.org/TR/html5/forms.html#range-state-(type=range))
   * for more info.
   *
   * This has the following consequences for AngularJS:
   *
   * Since the element value should always reflect the current model value, a range input
   * will set the bound ngModel expression to the value that the browser has set for the
   * input element. For example, in the following input `<input type="range" ng-model="model.value">`,
   * if the application sets `model.value = null`, the browser will set the input to `'50'`.
   * AngularJS will then set the model to `50`, to prevent input and model value being out of sync.
   *
   * That means the model for range will immediately be set to `50` after `ngModel` has been
   * initialized. It also means a range input can never have the required error.
   *
   * This does not only affect changes to the model value, but also to the values of the `min`,
   * `max`, and `step` attributes. When these change in a way that will cause the browser to modify
   * the input value, AngularJS will also update the model value.
   *
   * Automatic value adjustment also means that a range input element can never have the `required`,
   * `min`, or `max` errors.
   *
   * However, `step` is currently only fully implemented by Firefox. Other browsers have problems
   * when the step value changes dynamically - they do not adjust the element value correctly, but
   * instead may set the `stepMismatch` error. If that's the case, the AngularJS will set the `step`
   * error on the input, and set the model to `undefined`.
   *
   * Note that `input[range]` is not compatible with`ngMax`, `ngMin`, and `ngStep`, because they do
   * not set the `min` and `max` attributes, which means that the browser won't automatically adjust
   * the input value based on their values, and will always assume min = 0, max = 100, and step = 1.
   *
   * @param {string}  ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {string=} min Sets the `min` validation to ensure that the value entered is greater
   *                  than `min`. Can be interpolated.
   * @param {string=} max Sets the `max` validation to ensure that the value entered is less than `max`.
   *                  Can be interpolated.
   * @param {string=} step Sets the `step` validation to ensure that the value entered matches the `step`
   *                  Can be interpolated.
   * @param {expression=} ngChange AngularJS expression to be executed when the ngModel value changes due
   *                      to user interaction with the input element.
   * @param {expression=} ngChecked If the expression is truthy, then the `checked` attribute will be set on the
   *                      element. **Note** : `ngChecked` should not be used alongside `ngModel`.
   *                      Checkout {@link ng.directive:ngChecked ngChecked} for usage.
   *
   * @example
      <example name="range-input-directive" module="rangeExample">
        <file name="index.html">
          <script>
            angular.module('rangeExample', [])
              .controller('ExampleController', ['$scope', function($scope) {
                $scope.value = 75;
                $scope.min = 10;
                $scope.max = 90;
              }]);
          </script>
          <form name="myForm" ng-controller="ExampleController">

            Model as range: <input type="range" name="range" ng-model="value" min="{{min}}"  max="{{max}}">
            <hr>
            Model as number: <input type="number" ng-model="value"><br>
            Min: <input type="number" ng-model="min"><br>
            Max: <input type="number" ng-model="max"><br>
            value = <code>{{value}}</code><br/>
            myForm.range.$valid = <code>{{myForm.range.$valid}}</code><br/>
            myForm.range.$error = <code>{{myForm.range.$error}}</code>
          </form>
        </file>
      </example>

   * ## Range Input with ngMin & ngMax attributes

   * @example
      <example name="range-input-directive-ng" module="rangeExample">
        <file name="index.html">
          <script>
            angular.module('rangeExample', [])
              .controller('ExampleController', ['$scope', function($scope) {
                $scope.value = 75;
                $scope.min = 10;
                $scope.max = 90;
              }]);
          </script>
          <form name="myForm" ng-controller="ExampleController">
            Model as range: <input type="range" name="range" ng-model="value" ng-min="min" ng-max="max">
            <hr>
            Model as number: <input type="number" ng-model="value"><br>
            Min: <input type="number" ng-model="min"><br>
            Max: <input type="number" ng-model="max"><br>
            value = <code>{{value}}</code><br/>
            myForm.range.$valid = <code>{{myForm.range.$valid}}</code><br/>
            myForm.range.$error = <code>{{myForm.range.$error}}</code>
          </form>
        </file>
      </example>

   */
  'range': rangeInputType,

  /**
   * @ngdoc input
   * @name input[checkbox]
   *
   * @description
   * HTML checkbox.
   *
   * @param {string} ngModel Assignable AngularJS expression to data-bind to.
   * @param {string=} name Property name of the form under which the control is published.
   * @param {expression=} ngTrueValue The value to which the expression should be set when selected.
   * @param {expression=} ngFalseValue The value to which the expression should be set when not selected.
   * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
   *    interaction with the input element.
   *
   * @example
      <example name="checkbox-input-directive" module="checkboxExample">
        <file name="index.html">
         <script>
           angular.module('checkboxExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.checkboxModel = {
                value1 : true,
                value2 : 'YES'
              };
             }]);
         </script>
         <form name="myForm" ng-controller="ExampleController">
           <label>Value1:
             <input type="checkbox" ng-model="checkboxModel.value1">
           </label><br/>
           <label>Value2:
             <input type="checkbox" ng-model="checkboxModel.value2"
                    ng-true-value="'YES'" ng-false-value="'NO'">
            </label><br/>
           <tt>value1 = {{checkboxModel.value1}}</tt><br/>
           <tt>value2 = {{checkboxModel.value2}}</tt><br/>
          </form>
        </file>
        <file name="protractor.js" type="protractor">
          it('should change state', function() {
            var value1 = element(by.binding('checkboxModel.value1'));
            var value2 = element(by.binding('checkboxModel.value2'));

            expect(value1.getText()).toContain('true');
            expect(value2.getText()).toContain('YES');

            element(by.model('checkboxModel.value1')).click();
            element(by.model('checkboxModel.value2')).click();

            expect(value1.getText()).toContain('false');
            expect(value2.getText()).toContain('NO');
          });
        </file>
      </example>
   */
  'checkbox': checkboxInputType,

  'hidden': noop,
  'button': noop,
  'submit': noop,
  'reset': noop,
  'file': noop
};

function stringBasedInputType(ctrl) {
  ctrl.$formatters.push(function(value) {
    return ctrl.$isEmpty(value) ? value : value.toString();
  });
}

function textInputType(scope, element, attr, ctrl, $sniffer, $browser) {
  baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
  stringBasedInputType(ctrl);
}

function baseInputType(scope, element, attr, ctrl, $sniffer, $browser) {
  var type = lowercase(element[0].type);

  // In composition mode, users are still inputting intermediate text buffer,
  // hold the listener until composition is done.
  // More about composition events: https://developer.mozilla.org/en-US/docs/Web/API/CompositionEvent
  if (!$sniffer.android) {
    var composing = false;

    element.on('compositionstart', function() {
      composing = true;
    });

    // Support: IE9+
    element.on('compositionupdate', function(ev) {
      // End composition when ev.data is empty string on 'compositionupdate' event.
      // When the input de-focusses (e.g. by clicking away), IE triggers 'compositionupdate'
      // instead of 'compositionend'.
      if (isUndefined(ev.data) || ev.data === '') {
        composing = false;
      }
    });

    element.on('compositionend', function() {
      composing = false;
      listener();
    });
  }

  var timeout;

  var listener = function(ev) {
    if (timeout) {
      $browser.defer.cancel(timeout);
      timeout = null;
    }
    if (composing) return;
    var value = element.val(),
        event = ev && ev.type;

    // By default we will trim the value
    // If the attribute ng-trim exists we will avoid trimming
    // If input type is 'password', the value is never trimmed
    if (type !== 'password' && (!attr.ngTrim || attr.ngTrim !== 'false')) {
      value = trim(value);
    }

    // If a control is suffering from bad input (due to native validators), browsers discard its
    // value, so it may be necessary to revalidate (by calling $setViewValue again) even if the
    // control's value is the same empty value twice in a row.
    if (ctrl.$viewValue !== value || (value === '' && ctrl.$$hasNativeValidators)) {
      ctrl.$setViewValue(value, event);
    }
  };

  // if the browser does support "input" event, we are fine - except on IE9 which doesn't fire the
  // input event on backspace, delete or cut
  if ($sniffer.hasEvent('input')) {
    element.on('input', listener);
  } else {
    var deferListener = function(ev, input, origValue) {
      if (!timeout) {
        timeout = $browser.defer(function() {
          timeout = null;
          if (!input || input.value !== origValue) {
            listener(ev);
          }
        });
      }
    };

    element.on('keydown', /** @this */ function(event) {
      var key = event.keyCode;

      // ignore
      //    command            modifiers                   arrows
      if (key === 91 || (15 < key && key < 19) || (37 <= key && key <= 40)) return;

      deferListener(event, this, this.value);
    });

    // if user modifies input value using context menu in IE, we need "paste", "cut" and "drop" events to catch it
    if ($sniffer.hasEvent('paste')) {
      element.on('paste cut drop', deferListener);
    }
  }

  // if user paste into input using mouse on older browser
  // or form autocomplete on newer browser, we need "change" event to catch it
  element.on('change', listener);

  // Some native input types (date-family) have the ability to change validity without
  // firing any input/change events.
  // For these event types, when native validators are present and the browser supports the type,
  // check for validity changes on various DOM events.
  if (PARTIAL_VALIDATION_TYPES[type] && ctrl.$$hasNativeValidators && type === attr.type) {
    element.on(PARTIAL_VALIDATION_EVENTS, /** @this */ function(ev) {
      if (!timeout) {
        var validity = this[VALIDITY_STATE_PROPERTY];
        var origBadInput = validity.badInput;
        var origTypeMismatch = validity.typeMismatch;
        timeout = $browser.defer(function() {
          timeout = null;
          if (validity.badInput !== origBadInput || validity.typeMismatch !== origTypeMismatch) {
            listener(ev);
          }
        });
      }
    });
  }

  ctrl.$render = function() {
    // Workaround for Firefox validation #12102.
    var value = ctrl.$isEmpty(ctrl.$viewValue) ? '' : ctrl.$viewValue;
    if (element.val() !== value) {
      element.val(value);
    }
  };
}

function weekParser(isoWeek, existingDate) {
  if (isDate(isoWeek)) {
    return isoWeek;
  }

  if (isString(isoWeek)) {
    WEEK_REGEXP.lastIndex = 0;
    var parts = WEEK_REGEXP.exec(isoWeek);
    if (parts) {
      var year = +parts[1],
          week = +parts[2],
          hours = 0,
          minutes = 0,
          seconds = 0,
          milliseconds = 0,
          firstThurs = getFirstThursdayOfYear(year),
          addDays = (week - 1) * 7;

      if (existingDate) {
        hours = existingDate.getHours();
        minutes = existingDate.getMinutes();
        seconds = existingDate.getSeconds();
        milliseconds = existingDate.getMilliseconds();
      }

      return new Date(year, 0, firstThurs.getDate() + addDays, hours, minutes, seconds, milliseconds);
    }
  }

  return NaN;
}

function createDateParser(regexp, mapping) {
  return function(iso, previousDate) {
    var parts, map;

    if (isDate(iso)) {
      return iso;
    }

    if (isString(iso)) {
      // When a date is JSON'ified to wraps itself inside of an extra
      // set of double quotes. This makes the date parsing code unable
      // to match the date string and parse it as a date.
      if (iso.charAt(0) === '"' && iso.charAt(iso.length - 1) === '"') {
        iso = iso.substring(1, iso.length - 1);
      }
      if (ISO_DATE_REGEXP.test(iso)) {
        return new Date(iso);
      }
      regexp.lastIndex = 0;
      parts = regexp.exec(iso);

      if (parts) {
        parts.shift();
        if (previousDate) {
          map = {
            yyyy: previousDate.getFullYear(),
            MM: previousDate.getMonth() + 1,
            dd: previousDate.getDate(),
            HH: previousDate.getHours(),
            mm: previousDate.getMinutes(),
            ss: previousDate.getSeconds(),
            sss: previousDate.getMilliseconds() / 1000
          };
        } else {
          map = { yyyy: 1970, MM: 1, dd: 1, HH: 0, mm: 0, ss: 0, sss: 0 };
        }

        forEach(parts, function(part, index) {
          if (index < mapping.length) {
            map[mapping[index]] = +part;
          }
        });

        var date = new Date(map.yyyy, map.MM - 1, map.dd, map.HH, map.mm, map.ss || 0, map.sss * 1000 || 0);
        if (map.yyyy < 100) {
          // In the constructor, 2-digit years map to 1900-1999.
          // Use `setFullYear()` to set the correct year.
          date.setFullYear(map.yyyy);
        }

        return date;
      }
    }

    return NaN;
  };
}

function createDateInputType(type, regexp, parseDate, format) {
  return function dynamicDateInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter, $parse) {
    badInputChecker(scope, element, attr, ctrl, type);
    baseInputType(scope, element, attr, ctrl, $sniffer, $browser);

    var isTimeType = type === 'time' || type === 'datetimelocal';
    var previousDate;
    var previousTimezone;

    ctrl.$parsers.push(function(value) {
      if (ctrl.$isEmpty(value)) return null;

      if (regexp.test(value)) {
        // Note: We cannot read ctrl.$modelValue, as there might be a different
        // parser/formatter in the processing chain so that the model
        // contains some different data format!
        return parseDateAndConvertTimeZoneToLocal(value, previousDate);
      }
      ctrl.$$parserName = type;
      return undefined;
    });

    ctrl.$formatters.push(function(value) {
      if (value && !isDate(value)) {
        throw ngModelMinErr('datefmt', 'Expected `{0}` to be a date', value);
      }
      if (isValidDate(value)) {
        previousDate = value;
        var timezone = ctrl.$options.getOption('timezone');

        if (timezone) {
          previousTimezone = timezone;
          previousDate = convertTimezoneToLocal(previousDate, timezone, true);
        }

        return formatter(value, timezone);
      } else {
        previousDate = null;
        previousTimezone = null;
        return '';
      }
    });

    if (isDefined(attr.min) || attr.ngMin) {
      var minVal = attr.min || $parse(attr.ngMin)(scope);
      var parsedMinVal = parseObservedDateValue(minVal);

      ctrl.$validators.min = function(value) {
        return !isValidDate(value) || isUndefined(parsedMinVal) || parseDate(value) >= parsedMinVal;
      };
      attr.$observe('min', function(val) {
        if (val !== minVal) {
          parsedMinVal = parseObservedDateValue(val);
          minVal = val;
          ctrl.$validate();
        }
      });
    }

    if (isDefined(attr.max) || attr.ngMax) {
      var maxVal = attr.max || $parse(attr.ngMax)(scope);
      var parsedMaxVal = parseObservedDateValue(maxVal);

      ctrl.$validators.max = function(value) {
        return !isValidDate(value) || isUndefined(parsedMaxVal) || parseDate(value) <= parsedMaxVal;
      };
      attr.$observe('max', function(val) {
        if (val !== maxVal) {
          parsedMaxVal = parseObservedDateValue(val);
          maxVal = val;
          ctrl.$validate();
        }
      });
    }

    function isValidDate(value) {
      // Invalid Date: getTime() returns NaN
      return value && !(value.getTime && value.getTime() !== value.getTime());
    }

    function parseObservedDateValue(val) {
      return isDefined(val) && !isDate(val) ? parseDateAndConvertTimeZoneToLocal(val) || undefined : val;
    }

    function parseDateAndConvertTimeZoneToLocal(value, previousDate) {
      var timezone = ctrl.$options.getOption('timezone');

      if (previousTimezone && previousTimezone !== timezone) {
        // If the timezone has changed, adjust the previousDate to the default timezone
        // so that the new date is converted with the correct timezone offset
        previousDate = addDateMinutes(previousDate, timezoneToOffset(previousTimezone));
      }

      var parsedDate = parseDate(value, previousDate);

      if (!isNaN(parsedDate) && timezone) {
        parsedDate = convertTimezoneToLocal(parsedDate, timezone);
      }
      return parsedDate;
    }

    function formatter(value, timezone) {
      var targetFormat = format;

      if (isTimeType && isString(ctrl.$options.getOption('timeSecondsFormat'))) {
        targetFormat = format
          .replace('ss.sss', ctrl.$options.getOption('timeSecondsFormat'))
          .replace(/:$/, '');
      }

      var formatted =  $filter('date')(value, targetFormat, timezone);

      if (isTimeType && ctrl.$options.getOption('timeStripZeroSeconds')) {
        formatted = formatted.replace(/(?::00)?(?:\.000)?$/, '');
      }

      return formatted;
    }
  };
}

function badInputChecker(scope, element, attr, ctrl, parserName) {
  var node = element[0];
  var nativeValidation = ctrl.$$hasNativeValidators = isObject(node.validity);
  if (nativeValidation) {
    ctrl.$parsers.push(function(value) {
      var validity = element.prop(VALIDITY_STATE_PROPERTY) || {};
      if (validity.badInput || validity.typeMismatch) {
        ctrl.$$parserName = parserName;
        return undefined;
      }

      return value;
    });
  }
}

function numberFormatterParser(ctrl) {
  ctrl.$parsers.push(function(value) {
    if (ctrl.$isEmpty(value))      return null;
    if (NUMBER_REGEXP.test(value)) return parseFloat(value);

    ctrl.$$parserName = 'number';
    return undefined;
  });

  ctrl.$formatters.push(function(value) {
    if (!ctrl.$isEmpty(value)) {
      if (!isNumber(value)) {
        throw ngModelMinErr('numfmt', 'Expected `{0}` to be a number', value);
      }
      value = value.toString();
    }
    return value;
  });
}

function parseNumberAttrVal(val) {
  if (isDefined(val) && !isNumber(val)) {
    val = parseFloat(val);
  }
  return !isNumberNaN(val) ? val : undefined;
}

function isNumberInteger(num) {
  // See http://stackoverflow.com/questions/14636536/how-to-check-if-a-variable-is-an-integer-in-javascript#14794066
  // (minus the assumption that `num` is a number)

  // eslint-disable-next-line no-bitwise
  return (num | 0) === num;
}

function countDecimals(num) {
  var numString = num.toString();
  var decimalSymbolIndex = numString.indexOf('.');

  if (decimalSymbolIndex === -1) {
    if (-1 < num && num < 1) {
      // It may be in the exponential notation format (`1e-X`)
      var match = /e-(\d+)$/.exec(numString);

      if (match) {
        return Number(match[1]);
      }
    }

    return 0;
  }

  return numString.length - decimalSymbolIndex - 1;
}

function isValidForStep(viewValue, stepBase, step) {
  // At this point `stepBase` and `step` are expected to be non-NaN values
  // and `viewValue` is expected to be a valid stringified number.
  var value = Number(viewValue);

  var isNonIntegerValue = !isNumberInteger(value);
  var isNonIntegerStepBase = !isNumberInteger(stepBase);
  var isNonIntegerStep = !isNumberInteger(step);

  // Due to limitations in Floating Point Arithmetic (e.g. `0.3 - 0.2 !== 0.1` or
  // `0.5 % 0.1 !== 0`), we need to convert all numbers to integers.
  if (isNonIntegerValue || isNonIntegerStepBase || isNonIntegerStep) {
    var valueDecimals = isNonIntegerValue ? countDecimals(value) : 0;
    var stepBaseDecimals = isNonIntegerStepBase ? countDecimals(stepBase) : 0;
    var stepDecimals = isNonIntegerStep ? countDecimals(step) : 0;

    var decimalCount = Math.max(valueDecimals, stepBaseDecimals, stepDecimals);
    var multiplier = Math.pow(10, decimalCount);

    value = value * multiplier;
    stepBase = stepBase * multiplier;
    step = step * multiplier;

    if (isNonIntegerValue) value = Math.round(value);
    if (isNonIntegerStepBase) stepBase = Math.round(stepBase);
    if (isNonIntegerStep) step = Math.round(step);
  }

  return (value - stepBase) % step === 0;
}

function numberInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter, $parse) {
  badInputChecker(scope, element, attr, ctrl, 'number');
  numberFormatterParser(ctrl);
  baseInputType(scope, element, attr, ctrl, $sniffer, $browser);

  var parsedMinVal;

  if (isDefined(attr.min) || attr.ngMin) {
    var minVal = attr.min || $parse(attr.ngMin)(scope);
    parsedMinVal = parseNumberAttrVal(minVal);

    ctrl.$validators.min = function(modelValue, viewValue) {
      return ctrl.$isEmpty(viewValue) || isUndefined(parsedMinVal) || viewValue >= parsedMinVal;
    };

    attr.$observe('min', function(val) {
      if (val !== minVal) {
        parsedMinVal = parseNumberAttrVal(val);
        minVal = val;
        // TODO(matsko): implement validateLater to reduce number of validations
        ctrl.$validate();
      }
    });
  }

  if (isDefined(attr.max) || attr.ngMax) {
    var maxVal = attr.max || $parse(attr.ngMax)(scope);
    var parsedMaxVal = parseNumberAttrVal(maxVal);

    ctrl.$validators.max = function(modelValue, viewValue) {
      return ctrl.$isEmpty(viewValue) || isUndefined(parsedMaxVal) || viewValue <= parsedMaxVal;
    };

    attr.$observe('max', function(val) {
      if (val !== maxVal) {
        parsedMaxVal = parseNumberAttrVal(val);
        maxVal = val;
        // TODO(matsko): implement validateLater to reduce number of validations
        ctrl.$validate();
      }
    });
  }

  if (isDefined(attr.step) || attr.ngStep) {
    var stepVal = attr.step || $parse(attr.ngStep)(scope);
    var parsedStepVal = parseNumberAttrVal(stepVal);

    ctrl.$validators.step = function(modelValue, viewValue) {
      return ctrl.$isEmpty(viewValue) || isUndefined(parsedStepVal) ||
        isValidForStep(viewValue, parsedMinVal || 0, parsedStepVal);
    };

    attr.$observe('step', function(val) {
      // TODO(matsko): implement validateLater to reduce number of validations
      if (val !== stepVal) {
        parsedStepVal = parseNumberAttrVal(val);
        stepVal = val;
        ctrl.$validate();
      }

    });

  }
}

function rangeInputType(scope, element, attr, ctrl, $sniffer, $browser) {
  badInputChecker(scope, element, attr, ctrl, 'range');
  numberFormatterParser(ctrl);
  baseInputType(scope, element, attr, ctrl, $sniffer, $browser);

  var supportsRange = ctrl.$$hasNativeValidators && element[0].type === 'range',
      minVal = supportsRange ? 0 : undefined,
      maxVal = supportsRange ? 100 : undefined,
      stepVal = supportsRange ? 1 : undefined,
      validity = element[0].validity,
      hasMinAttr = isDefined(attr.min),
      hasMaxAttr = isDefined(attr.max),
      hasStepAttr = isDefined(attr.step);

  var originalRender = ctrl.$render;

  ctrl.$render = supportsRange && isDefined(validity.rangeUnderflow) && isDefined(validity.rangeOverflow) ?
    //Browsers that implement range will set these values automatically, but reading the adjusted values after
    //$render would cause the min / max validators to be applied with the wrong value
    function rangeRender() {
      originalRender();
      ctrl.$setViewValue(element.val());
    } :
    originalRender;

  if (hasMinAttr) {
    minVal = parseNumberAttrVal(attr.min);

    ctrl.$validators.min = supportsRange ?
      // Since all browsers set the input to a valid value, we don't need to check validity
      function noopMinValidator() { return true; } :
      // non-support browsers validate the min val
      function minValidator(modelValue, viewValue) {
        return ctrl.$isEmpty(viewValue) || isUndefined(minVal) || viewValue >= minVal;
      };

    setInitialValueAndObserver('min', minChange);
  }

  if (hasMaxAttr) {
    maxVal = parseNumberAttrVal(attr.max);

    ctrl.$validators.max = supportsRange ?
      // Since all browsers set the input to a valid value, we don't need to check validity
      function noopMaxValidator() { return true; } :
      // non-support browsers validate the max val
      function maxValidator(modelValue, viewValue) {
        return ctrl.$isEmpty(viewValue) || isUndefined(maxVal) || viewValue <= maxVal;
      };

    setInitialValueAndObserver('max', maxChange);
  }

  if (hasStepAttr) {
    stepVal = parseNumberAttrVal(attr.step);

    ctrl.$validators.step = supportsRange ?
      function nativeStepValidator() {
        // Currently, only FF implements the spec on step change correctly (i.e. adjusting the
        // input element value to a valid value). It's possible that other browsers set the stepMismatch
        // validity error instead, so we can at least report an error in that case.
        return !validity.stepMismatch;
      } :
      // ngStep doesn't set the setp attr, so the browser doesn't adjust the input value as setting step would
      function stepValidator(modelValue, viewValue) {
        return ctrl.$isEmpty(viewValue) || isUndefined(stepVal) ||
               isValidForStep(viewValue, minVal || 0, stepVal);
      };

    setInitialValueAndObserver('step', stepChange);
  }

  function setInitialValueAndObserver(htmlAttrName, changeFn) {
    // interpolated attributes set the attribute value only after a digest, but we need the
    // attribute value when the input is first rendered, so that the browser can adjust the
    // input value based on the min/max value
    element.attr(htmlAttrName, attr[htmlAttrName]);
    var oldVal = attr[htmlAttrName];
    attr.$observe(htmlAttrName, function wrappedObserver(val) {
      if (val !== oldVal) {
        oldVal = val;
        changeFn(val);
      }
    });
  }

  function minChange(val) {
    minVal = parseNumberAttrVal(val);
    // ignore changes before model is initialized
    if (isNumberNaN(ctrl.$modelValue)) {
      return;
    }

    if (supportsRange) {
      var elVal = element.val();
      // IE11 doesn't set the el val correctly if the minVal is greater than the element value
      if (minVal > elVal) {
        elVal = minVal;
        element.val(elVal);
      }
      ctrl.$setViewValue(elVal);
    } else {
      // TODO(matsko): implement validateLater to reduce number of validations
      ctrl.$validate();
    }
  }

  function maxChange(val) {
    maxVal = parseNumberAttrVal(val);
    // ignore changes before model is initialized
    if (isNumberNaN(ctrl.$modelValue)) {
      return;
    }

    if (supportsRange) {
      var elVal = element.val();
      // IE11 doesn't set the el val correctly if the maxVal is less than the element value
      if (maxVal < elVal) {
        element.val(maxVal);
        // IE11 and Chrome don't set the value to the minVal when max < min
        elVal = maxVal < minVal ? minVal : maxVal;
      }
      ctrl.$setViewValue(elVal);
    } else {
      // TODO(matsko): implement validateLater to reduce number of validations
      ctrl.$validate();
    }
  }

  function stepChange(val) {
    stepVal = parseNumberAttrVal(val);
    // ignore changes before model is initialized
    if (isNumberNaN(ctrl.$modelValue)) {
      return;
    }

    // Some browsers don't adjust the input value correctly, but set the stepMismatch error
    if (!supportsRange) {
      // TODO(matsko): implement validateLater to reduce number of validations
      ctrl.$validate();
    } else if (ctrl.$viewValue !== element.val()) {
      ctrl.$setViewValue(element.val());
    }
  }
}

function urlInputType(scope, element, attr, ctrl, $sniffer, $browser) {
  // Note: no badInputChecker here by purpose as `url` is only a validation
  // in browsers, i.e. we can always read out input.value even if it is not valid!
  baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
  stringBasedInputType(ctrl);

  ctrl.$validators.url = function(modelValue, viewValue) {
    var value = modelValue || viewValue;
    return ctrl.$isEmpty(value) || URL_REGEXP.test(value);
  };
}

function emailInputType(scope, element, attr, ctrl, $sniffer, $browser) {
  // Note: no badInputChecker here by purpose as `url` is only a validation
  // in browsers, i.e. we can always read out input.value even if it is not valid!
  baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
  stringBasedInputType(ctrl);

  ctrl.$validators.email = function(modelValue, viewValue) {
    var value = modelValue || viewValue;
    return ctrl.$isEmpty(value) || EMAIL_REGEXP.test(value);
  };
}

function radioInputType(scope, element, attr, ctrl) {
  var doTrim = !attr.ngTrim || trim(attr.ngTrim) !== 'false';
  // make the name unique, if not defined
  if (isUndefined(attr.name)) {
    element.attr('name', nextUid());
  }

  var listener = function(ev) {
    var value;
    if (element[0].checked) {
      value = attr.value;
      if (doTrim) {
        value = trim(value);
      }
      ctrl.$setViewValue(value, ev && ev.type);
    }
  };

  element.on('change', listener);

  ctrl.$render = function() {
    var value = attr.value;
    if (doTrim) {
      value = trim(value);
    }
    element[0].checked = (value === ctrl.$viewValue);
  };

  attr.$observe('value', ctrl.$render);
}

function parseConstantExpr($parse, context, name, expression, fallback) {
  var parseFn;
  if (isDefined(expression)) {
    parseFn = $parse(expression);
    if (!parseFn.constant) {
      throw ngModelMinErr('constexpr', 'Expected constant expression for `{0}`, but saw ' +
                                   '`{1}`.', name, expression);
    }
    return parseFn(context);
  }
  return fallback;
}

function checkboxInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter, $parse) {
  var trueValue = parseConstantExpr($parse, scope, 'ngTrueValue', attr.ngTrueValue, true);
  var falseValue = parseConstantExpr($parse, scope, 'ngFalseValue', attr.ngFalseValue, false);

  var listener = function(ev) {
    ctrl.$setViewValue(element[0].checked, ev && ev.type);
  };

  element.on('change', listener);

  ctrl.$render = function() {
    element[0].checked = ctrl.$viewValue;
  };

  // Override the standard `$isEmpty` because the $viewValue of an empty checkbox is always set to `false`
  // This is because of the parser below, which compares the `$modelValue` with `trueValue` to convert
  // it to a boolean.
  ctrl.$isEmpty = function(value) {
    return value === false;
  };

  ctrl.$formatters.push(function(value) {
    return equals(value, trueValue);
  });

  ctrl.$parsers.push(function(value) {
    return value ? trueValue : falseValue;
  });
}


/**
 * @ngdoc directive
 * @name textarea
 * @restrict E
 *
 * @description
 * HTML textarea element control with AngularJS data-binding. The data-binding and validation
 * properties of this element are exactly the same as those of the
 * {@link ng.directive:input input element}.
 *
 * @param {string} ngModel Assignable AngularJS expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required Sets `required` validation error key if the value is not entered.
 * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
 *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
 *    `required` when you want to data-bind to the `required` attribute.
 * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
 *    minlength.
 * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
 *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
 *    length.
 * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
 *    does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
 *    If the expression evaluates to a RegExp object, then this is used directly.
 *    If the expression evaluates to a string, then it will be converted to a RegExp
 *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
 *    `new RegExp('^abc$')`.<br />
 *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
 *    start at the index of the last search's match, thus not taking the whole input value into
 *    account.
 * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
 *    interaction with the input element.
 * @param {boolean=} [ngTrim=true] If set to false AngularJS will not automatically trim the input.
 *
 * @knownIssue
 *
 * When specifying the `placeholder` attribute of `<textarea>`, Internet Explorer will temporarily
 * insert the placeholder value as the textarea's content. If the placeholder value contains
 * interpolation (`{{ ... }}`), an error will be logged in the console when AngularJS tries to update
 * the value of the by-then-removed text node. This doesn't affect the functionality of the
 * textarea, but can be undesirable.
 *
 * You can work around this Internet Explorer issue by using `ng-attr-placeholder` instead of
 * `placeholder` on textareas, whenever you need interpolation in the placeholder value. You can
 * find more details on `ngAttr` in the
 * [Interpolation](guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes) section of the
 * Developer Guide.
 */


/**
 * @ngdoc directive
 * @name input
 * @restrict E
 *
 * @description
 * HTML input element control. When used together with {@link ngModel `ngModel`}, it provides data-binding,
 * input state control, and validation.
 * Input control follows HTML5 input types and polyfills the HTML5 validation behavior for older browsers.
 *
 * <div class="alert alert-warning">
 * **Note:** Not every feature offered is available for all input types.
 * Specifically, data binding and event handling via `ng-model` is unsupported for `input[file]`.
 * </div>
 *
 * @param {string} ngModel Assignable AngularJS expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required Sets `required` validation error key if the value is not entered.
 * @param {boolean=} ngRequired Sets `required` attribute if set to true
 * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
 *    minlength.
 * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
 *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
 *    length.
 * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
 *    value does not match a RegExp found by evaluating the AngularJS expression given in the attribute value.
 *    If the expression evaluates to a RegExp object, then this is used directly.
 *    If the expression evaluates to a string, then it will be converted to a RegExp
 *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
 *    `new RegExp('^abc$')`.<br />
 *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
 *    start at the index of the last search's match, thus not taking the whole input value into
 *    account.
 * @param {string=} ngChange AngularJS expression to be executed when input changes due to user
 *    interaction with the input element.
 * @param {boolean=} [ngTrim=true] If set to false AngularJS will not automatically trim the input.
 *    This parameter is ignored for input[type=password] controls, which will never trim the
 *    input.
 *
 * @example
    <example name="input-directive" module="inputExample">
      <file name="index.html">
       <script>
          angular.module('inputExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.user = {name: 'guest', last: 'visitor'};
            }]);
       </script>
       <div ng-controller="ExampleController">
         <form name="myForm">
           <label>
              User name:
              <input type="text" name="userName" ng-model="user.name" required>
           </label>
           <div role="alert">
             <span class="error" ng-show="myForm.userName.$error.required">
              Required!</span>
           </div>
           <label>
              Last name:
              <input type="text" name="lastName" ng-model="user.last"
              ng-minlength="3" ng-maxlength="10">
           </label>
           <div role="alert">
             <span class="error" ng-show="myForm.lastName.$error.minlength">
               Too short!</span>
             <span class="error" ng-show="myForm.lastName.$error.maxlength">
               Too long!</span>
           </div>
         </form>
         <hr>
         <tt>user = {{user}}</tt><br/>
         <tt>myForm.userName.$valid = {{myForm.userName.$valid}}</tt><br/>
         <tt>myForm.userName.$error = {{myForm.userName.$error}}</tt><br/>
         <tt>myForm.lastName.$valid = {{myForm.lastName.$valid}}</tt><br/>
         <tt>myForm.lastName.$error = {{myForm.lastName.$error}}</tt><br/>
         <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
         <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         <tt>myForm.$error.minlength = {{!!myForm.$error.minlength}}</tt><br/>
         <tt>myForm.$error.maxlength = {{!!myForm.$error.maxlength}}</tt><br/>
       </div>
      </file>
      <file name="protractor.js" type="protractor">
        var user = element(by.exactBinding('user'));
        var userNameValid = element(by.binding('myForm.userName.$valid'));
        var lastNameValid = element(by.binding('myForm.lastName.$valid'));
        var lastNameError = element(by.binding('myForm.lastName.$error'));
        var formValid = element(by.binding('myForm.$valid'));
        var userNameInput = element(by.model('user.name'));
        var userLastInput = element(by.model('user.last'));

        it('should initialize to model', function() {
          expect(user.getText()).toContain('{"name":"guest","last":"visitor"}');
          expect(userNameValid.getText()).toContain('true');
          expect(formValid.getText()).toContain('true');
        });

        it('should be invalid if empty when required', function() {
          userNameInput.clear();
          userNameInput.sendKeys('');

          expect(user.getText()).toContain('{"last":"visitor"}');
          expect(userNameValid.getText()).toContain('false');
          expect(formValid.getText()).toContain('false');
        });

        it('should be valid if empty when min length is set', function() {
          userLastInput.clear();
          userLastInput.sendKeys('');

          expect(user.getText()).toContain('{"name":"guest","last":""}');
          expect(lastNameValid.getText()).toContain('true');
          expect(formValid.getText()).toContain('true');
        });

        it('should be invalid if less than required min length', function() {
          userLastInput.clear();
          userLastInput.sendKeys('xx');

          expect(user.getText()).toContain('{"name":"guest"}');
          expect(lastNameValid.getText()).toContain('false');
          expect(lastNameError.getText()).toContain('minlength');
          expect(formValid.getText()).toContain('false');
        });

        it('should be invalid if longer than max length', function() {
          userLastInput.clear();
          userLastInput.sendKeys('some ridiculously long name');

          expect(user.getText()).toContain('{"name":"guest"}');
          expect(lastNameValid.getText()).toContain('false');
          expect(lastNameError.getText()).toContain('maxlength');
          expect(formValid.getText()).toContain('false');
        });
      </file>
    </example>
 */
var inputDirective = ['$browser', '$sniffer', '$filter', '$parse',
    function($browser, $sniffer, $filter, $parse) {
  return {
    restrict: 'E',
    require: ['?ngModel'],
    link: {
      pre: function(scope, element, attr, ctrls) {
        if (ctrls[0]) {
          (inputType[lowercase(attr.type)] || inputType.text)(scope, element, attr, ctrls[0], $sniffer,
                                                              $browser, $filter, $parse);
        }
      }
    }
  };
}];


var hiddenInputBrowserCacheDirective = function() {
  var valueProperty = {
    configurable: true,
    enumerable: false,
    get: function() {
      return this.getAttribute('value') || '';
    },
    set: function(val) {
      this.setAttribute('value', val);
    }
  };

  return {
    restrict: 'E',
    priority: 200,
    compile: function(_, attr) {
      if (lowercase(attr.type) !== 'hidden') {
        return;
      }

      return {
        pre: function(scope, element, attr, ctrls) {
          var node = element[0];

          // Support: Edge
          // Moving the DOM around prevents autofillling
          if (node.parentNode) {
            node.parentNode.insertBefore(node, node.nextSibling);
          }

          // Support: FF, IE
          // Avoiding direct assignment to .value prevents autofillling
          if (Object.defineProperty) {
            Object.defineProperty(node, 'value', valueProperty);
          }
        }
      };
    }
  };
};



var CONSTANT_VALUE_REGEXP = /^(true|false|\d+)$/;
/**
 * @ngdoc directive
 * @name ngValue
 * @restrict A
 * @priority 100
 *
 * @description
 * Binds the given expression to the value of the element.
 *
 * It is mainly used on {@link input[radio] `input[radio]`} and option elements,
 * so that when the element is selected, the {@link ngModel `ngModel`} of that element (or its
 * {@link select `select`} parent element) is set to the bound value. It is especially useful
 * for dynamically generated lists using {@link ngRepeat `ngRepeat`}, as shown below.
 *
 * It can also be used to achieve one-way binding of a given expression to an input element
 * such as an `input[text]` or a `textarea`, when that element does not use ngModel.
 *
 * @element ANY
 * @param {string=} ngValue AngularJS expression, whose value will be bound to the `value` attribute
 * and `value` property of the element.
 *
 * @example
    <example name="ngValue-directive" module="valueExample">
      <file name="index.html">
       <script>
          angular.module('valueExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.names = ['pizza', 'unicorns', 'robots'];
              $scope.my = { favorite: 'unicorns' };
            }]);
       </script>
        <form ng-controller="ExampleController">
          <h2>Which is your favorite?</h2>
            <label ng-repeat="name in names" for="{{name}}">
              {{name}}
              <input type="radio"
                     ng-model="my.favorite"
                     ng-value="name"
                     id="{{name}}"
                     name="favorite">
            </label>
          <div>You chose {{my.favorite}}</div>
        </form>
      </file>
      <file name="protractor.js" type="protractor">
        var favorite = element(by.binding('my.favorite'));

        it('should initialize to model', function() {
          expect(favorite.getText()).toContain('unicorns');
        });
        it('should bind the values to the inputs', function() {
          element.all(by.model('my.favorite')).get(0).click();
          expect(favorite.getText()).toContain('pizza');
        });
      </file>
    </example>
 */
var ngValueDirective = function() {
  /**
   *  inputs use the value attribute as their default value if the value property is not set.
   *  Once the value property has been set (by adding input), it will not react to changes to
   *  the value attribute anymore. Setting both attribute and property fixes this behavior, and
   *  makes it possible to use ngValue as a sort of one-way bind.
   */
  function updateElementValue(element, attr, value) {
    // Support: IE9 only
    // In IE9 values are converted to string (e.g. `input.value = null` results in `input.value === 'null'`).
    var propValue = isDefined(value) ? value : (msie === 9) ? '' : null;
    element.prop('value', propValue);
    attr.$set('value', value);
  }

  return {
    restrict: 'A',
    priority: 100,
    compile: function(tpl, tplAttr) {
      if (CONSTANT_VALUE_REGEXP.test(tplAttr.ngValue)) {
        return function ngValueConstantLink(scope, elm, attr) {
          var value = scope.$eval(attr.ngValue);
          updateElementValue(elm, attr, value);
        };
      } else {
        return function ngValueLink(scope, elm, attr) {
          scope.$watch(attr.ngValue, function valueWatchAction(value) {
            updateElementValue(elm, attr, value);
          });
        };
      }
    }
  };
};

/**
 * @ngdoc directive
 * @name ngBind
 * @restrict AC
 *
 * @description
 * The `ngBind` attribute tells AngularJS to replace the text content of the specified HTML element
 * with the value of a given expression, and to update the text content when the value of that
 * expression changes.
 *
 * Typically, you don't use `ngBind` directly, but instead you use the double curly markup like
 * `{{ expression }}` which is similar but less verbose.
 *
 * It is preferable to use `ngBind` instead of `{{ expression }}` if a template is momentarily
 * displayed by the browser in its raw state before AngularJS compiles it. Since `ngBind` is an
 * element attribute, it makes the bindings invisible to the user while the page is loading.
 *
 * An alternative solution to this problem would be using the
 * {@link ng.directive:ngCloak ngCloak} directive.
 *
 *
 * @element ANY
 * @param {expression} ngBind {@link guide/expression Expression} to evaluate.
 *
 * @example
 * Enter a name in the Live Preview text box; the greeting below the text box changes instantly.
   <example module="bindExample" name="ng-bind">
     <file name="index.html">
       <script>
         angular.module('bindExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.name = 'Whirled';
           }]);
       </script>
       <div ng-controller="ExampleController">
         <label>Enter name: <input type="text" ng-model="name"></label><br>
         Hello <span ng-bind="name"></span>!
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-bind', function() {
         var nameInput = element(by.model('name'));

         expect(element(by.binding('name')).getText()).toBe('Whirled');
         nameInput.clear();
         nameInput.sendKeys('world');
         expect(element(by.binding('name')).getText()).toBe('world');
       });
     </file>
   </example>
 */
var ngBindDirective = ['$compile', function($compile) {
  return {
    restrict: 'AC',
    compile: function ngBindCompile(templateElement) {
      $compile.$$addBindingClass(templateElement);
      return function ngBindLink(scope, element, attr) {
        $compile.$$addBindingInfo(element, attr.ngBind);
        element = element[0];
        scope.$watch(attr.ngBind, function ngBindWatchAction(value) {
          element.textContent = stringify(value);
        });
      };
    }
  };
}];


/**
 * @ngdoc directive
 * @name ngBindTemplate
 *
 * @description
 * The `ngBindTemplate` directive specifies that the element
 * text content should be replaced with the interpolation of the template
 * in the `ngBindTemplate` attribute.
 * Unlike `ngBind`, the `ngBindTemplate` can contain multiple `{{` `}}`
 * expressions. This directive is needed since some HTML elements
 * (such as TITLE and OPTION) cannot contain SPAN elements.
 *
 * @element ANY
 * @param {string} ngBindTemplate template of form
 *   <tt>{{</tt> <tt>expression</tt> <tt>}}</tt> to eval.
 *
 * @example
 * Try it here: enter text in text box and watch the greeting change.
   <example module="bindExample" name="ng-bind-template">
     <file name="index.html">
       <script>
         angular.module('bindExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.salutation = 'Hello';
             $scope.name = 'World';
           }]);
       </script>
       <div ng-controller="ExampleController">
        <label>Salutation: <input type="text" ng-model="salutation"></label><br>
        <label>Name: <input type="text" ng-model="name"></label><br>
        <pre ng-bind-template="{{salutation}} {{name}}!"></pre>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-bind', function() {
         var salutationElem = element(by.binding('salutation'));
         var salutationInput = element(by.model('salutation'));
         var nameInput = element(by.model('name'));

         expect(salutationElem.getText()).toBe('Hello World!');

         salutationInput.clear();
         salutationInput.sendKeys('Greetings');
         nameInput.clear();
         nameInput.sendKeys('user');

         expect(salutationElem.getText()).toBe('Greetings user!');
       });
     </file>
   </example>
 */
var ngBindTemplateDirective = ['$interpolate', '$compile', function($interpolate, $compile) {
  return {
    compile: function ngBindTemplateCompile(templateElement) {
      $compile.$$addBindingClass(templateElement);
      return function ngBindTemplateLink(scope, element, attr) {
        var interpolateFn = $interpolate(element.attr(attr.$attr.ngBindTemplate));
        $compile.$$addBindingInfo(element, interpolateFn.expressions);
        element = element[0];
        attr.$observe('ngBindTemplate', function(value) {
          element.textContent = isUndefined(value) ? '' : value;
        });
      };
    }
  };
}];


/**
 * @ngdoc directive
 * @name ngBindHtml
 *
 * @description
 * Evaluates the expression and inserts the resulting HTML into the element in a secure way. By default,
 * the resulting HTML content will be sanitized using the {@link ngSanitize.$sanitize $sanitize} service.
 * To utilize this functionality, ensure that `$sanitize` is available, for example, by including {@link
 * ngSanitize} in your module's dependencies (not in core AngularJS). In order to use {@link ngSanitize}
 * in your module's dependencies, you need to include "angular-sanitize.js" in your application.
 *
 * You may also bypass sanitization for values you know are safe. To do so, bind to
 * an explicitly trusted value via {@link ng.$sce#trustAsHtml $sce.trustAsHtml}.  See the example
 * under {@link ng.$sce#show-me-an-example-using-sce- Strict Contextual Escaping (SCE)}.
 *
 * Note: If a `$sanitize` service is unavailable and the bound value isn't explicitly trusted, you
 * will have an exception (instead of an exploit.)
 *
 * @element ANY
 * @param {expression} ngBindHtml {@link guide/expression Expression} to evaluate.
 *
 * @example

   <example module="bindHtmlExample" deps="angular-sanitize.js" name="ng-bind-html">
     <file name="index.html">
       <div ng-controller="ExampleController">
        <p ng-bind-html="myHTML"></p>
       </div>
     </file>

     <file name="script.js">
       angular.module('bindHtmlExample', ['ngSanitize'])
         .controller('ExampleController', ['$scope', function($scope) {
           $scope.myHTML =
              'I am an <code>HTML</code>string with ' +
              '<a href="#">links!</a> and other <em>stuff</em>';
         }]);
     </file>

     <file name="protractor.js" type="protractor">
       it('should check ng-bind-html', function() {
         expect(element(by.binding('myHTML')).getText()).toBe(
             'I am an HTMLstring with links! and other stuff');
       });
     </file>
   </example>
 */
var ngBindHtmlDirective = ['$sce', '$parse', '$compile', function($sce, $parse, $compile) {
  return {
    restrict: 'A',
    compile: function ngBindHtmlCompile(tElement, tAttrs) {
      var ngBindHtmlGetter = $parse(tAttrs.ngBindHtml);
      var ngBindHtmlWatch = $parse(tAttrs.ngBindHtml, function sceValueOf(val) {
        // Unwrap the value to compare the actual inner safe value, not the wrapper object.
        return $sce.valueOf(val);
      });
      $compile.$$addBindingClass(tElement);

      return function ngBindHtmlLink(scope, element, attr) {
        $compile.$$addBindingInfo(element, attr.ngBindHtml);

        scope.$watch(ngBindHtmlWatch, function ngBindHtmlWatchAction() {
          // The watched value is the unwrapped value. To avoid re-escaping, use the direct getter.
          var value = ngBindHtmlGetter(scope);
          element.html($sce.getTrustedHtml(value) || '');
        });
      };
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngChange
 * @restrict A
 *
 * @description
 * Evaluate the given expression when the user changes the input.
 * The expression is evaluated immediately, unlike the JavaScript onchange event
 * which only triggers at the end of a change (usually, when the user leaves the
 * form element or presses the return key).
 *
 * The `ngChange` expression is only evaluated when a change in the input value causes
 * a new value to be committed to the model.
 *
 * It will not be evaluated:
 * * if the value returned from the `$parsers` transformation pipeline has not changed
 * * if the input has continued to be invalid since the model will stay `null`
 * * if the model is changed programmatically and not by a change to the input value
 *
 *
 * Note, this directive requires `ngModel` to be present.
 *
 * @element ANY
 * @param {expression} ngChange {@link guide/expression Expression} to evaluate upon change
 * in input value.
 *
 * @example
 * <example name="ngChange-directive" module="changeExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('changeExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.counter = 0;
 *           $scope.change = function() {
 *             $scope.counter++;
 *           };
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <input type="checkbox" ng-model="confirmed" ng-change="change()" id="ng-change-example1" />
 *       <input type="checkbox" ng-model="confirmed" id="ng-change-example2" />
 *       <label for="ng-change-example2">Confirmed</label><br />
 *       <tt>debug = {{confirmed}}</tt><br/>
 *       <tt>counter = {{counter}}</tt><br/>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     var counter = element(by.binding('counter'));
 *     var debug = element(by.binding('confirmed'));
 *
 *     it('should evaluate the expression if changing from view', function() {
 *       expect(counter.getText()).toContain('0');
 *
 *       element(by.id('ng-change-example1')).click();
 *
 *       expect(counter.getText()).toContain('1');
 *       expect(debug.getText()).toContain('true');
 *     });
 *
 *     it('should not evaluate the expression if changing from model', function() {
 *       element(by.id('ng-change-example2')).click();

 *       expect(counter.getText()).toContain('0');
 *       expect(debug.getText()).toContain('true');
 *     });
 *   </file>
 * </example>
 */
var ngChangeDirective = valueFn({
  restrict: 'A',
  require: 'ngModel',
  link: function(scope, element, attr, ctrl) {
    ctrl.$viewChangeListeners.push(function() {
      scope.$eval(attr.ngChange);
    });
  }
});

/* exported
  ngClassDirective,
  ngClassEvenDirective,
  ngClassOddDirective
*/

function classDirective(name, selector) {
  name = 'ngClass' + name;
  var indexWatchExpression;

  return ['$parse', function($parse) {
    return {
      restrict: 'AC',
      link: function(scope, element, attr) {
        var classCounts = element.data('$classCounts');
        var oldModulo = true;
        var oldClassString;

        if (!classCounts) {
          // Use createMap() to prevent class assumptions involving property
          // names in Object.prototype
          classCounts = createMap();
          element.data('$classCounts', classCounts);
        }

        if (name !== 'ngClass') {
          if (!indexWatchExpression) {
            indexWatchExpression = $parse('$index', function moduloTwo($index) {
              // eslint-disable-next-line no-bitwise
              return $index & 1;
            });
          }

          scope.$watch(indexWatchExpression, ngClassIndexWatchAction);
        }

        scope.$watch($parse(attr[name], toClassString), ngClassWatchAction);

        function addClasses(classString) {
          classString = digestClassCounts(split(classString), 1);
          attr.$addClass(classString);
        }

        function removeClasses(classString) {
          classString = digestClassCounts(split(classString), -1);
          attr.$removeClass(classString);
        }

        function updateClasses(oldClassString, newClassString) {
          var oldClassArray = split(oldClassString);
          var newClassArray = split(newClassString);

          var toRemoveArray = arrayDifference(oldClassArray, newClassArray);
          var toAddArray = arrayDifference(newClassArray, oldClassArray);

          var toRemoveString = digestClassCounts(toRemoveArray, -1);
          var toAddString = digestClassCounts(toAddArray, 1);

          attr.$addClass(toAddString);
          attr.$removeClass(toRemoveString);
        }

        function digestClassCounts(classArray, count) {
          var classesToUpdate = [];

          forEach(classArray, function(className) {
            if (count > 0 || classCounts[className]) {
              classCounts[className] = (classCounts[className] || 0) + count;
              if (classCounts[className] === +(count > 0)) {
                classesToUpdate.push(className);
              }
            }
          });

          return classesToUpdate.join(' ');
        }

        function ngClassIndexWatchAction(newModulo) {
          // This watch-action should run before the `ngClassWatchAction()`, thus it
          // adds/removes `oldClassString`. If the `ngClass` expression has changed as well, the
          // `ngClassWatchAction()` will update the classes.
          if (newModulo === selector) {
            addClasses(oldClassString);
          } else {
            removeClasses(oldClassString);
          }

          oldModulo = newModulo;
        }

        function ngClassWatchAction(newClassString) {
          if (oldModulo === selector) {
            updateClasses(oldClassString, newClassString);
          }

          oldClassString = newClassString;
        }
      }
    };
  }];

  // Helpers
  function arrayDifference(tokens1, tokens2) {
    if (!tokens1 || !tokens1.length) return [];
    if (!tokens2 || !tokens2.length) return tokens1;

    var values = [];

    outer:
    for (var i = 0; i < tokens1.length; i++) {
      var token = tokens1[i];
      for (var j = 0; j < tokens2.length; j++) {
        if (token === tokens2[j]) continue outer;
      }
      values.push(token);
    }

    return values;
  }

  function split(classString) {
    return classString && classString.split(' ');
  }

  function toClassString(classValue) {
    if (!classValue) return classValue;

    var classString = classValue;

    if (isArray(classValue)) {
      classString = classValue.map(toClassString).join(' ');
    } else if (isObject(classValue)) {
      classString = Object.keys(classValue).
        filter(function(key) { return classValue[key]; }).
        join(' ');
    } else if (!isString(classValue)) {
      classString = classValue + '';
    }

    return classString;
  }
}

/**
 * @ngdoc directive
 * @name ngClass
 * @restrict AC
 * @element ANY
 *
 * @description
 * The `ngClass` directive allows you to dynamically set CSS classes on an HTML element by databinding
 * an expression that represents all classes to be added.
 *
 * The directive operates in three different ways, depending on which of three types the expression
 * evaluates to:
 *
 * 1. If the expression evaluates to a string, the string should be one or more space-delimited class
 * names.
 *
 * 2. If the expression evaluates to an object, then for each key-value pair of the
 * object with a truthy value the corresponding key is used as a class name.
 *
 * 3. If the expression evaluates to an array, each element of the array should either be a string as in
 * type 1 or an object as in type 2. This means that you can mix strings and objects together in an array
 * to give you more control over what CSS classes appear. See the code below for an example of this.
 *
 *
 * The directive won't add duplicate classes if a particular class was already set.
 *
 * When the expression changes, the previously added classes are removed and only then are the
 * new classes added.
 *
 * @knownIssue
 * You should not use {@link guide/interpolation interpolation} in the value of the `class`
 * attribute, when using the `ngClass` directive on the same element.
 * See {@link guide/interpolation#known-issues here} for more info.
 *
 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#addClass addClass}       | just before the class is applied to the element   |
 * | {@link ng.$animate#removeClass removeClass} | just before the class is removed from the element |
 * | {@link ng.$animate#setClass setClass} | just before classes are added and classes are removed from the element at the same time |
 *
 * ### ngClass and pre-existing CSS3 Transitions/Animations
   The ngClass directive still supports CSS3 Transitions/Animations even if they do not follow the ngAnimate CSS naming structure.
   Upon animation ngAnimate will apply supplementary CSS classes to track the start and end of an animation, but this will not hinder
   any pre-existing CSS transitions already on the element. To get an idea of what happens during a class-based animation, be sure
   to view the step by step details of {@link $animate#addClass $animate.addClass} and
   {@link $animate#removeClass $animate.removeClass}.
 *
 * @param {expression} ngClass {@link guide/expression Expression} to eval. The result
 *   of the evaluation can be a string representing space delimited class
 *   names, an array, or a map of class names to boolean values. In the case of a map, the
 *   names of the properties whose values are truthy will be added as css classes to the
 *   element.
 *
 * @example
 * ### Basic
   <example name="ng-class">
     <file name="index.html">
       <p ng-class="{strike: deleted, bold: important, 'has-error': error}">Map Syntax Example</p>
       <label>
          <input type="checkbox" ng-model="deleted">
          deleted (apply "strike" class)
       </label><br>
       <label>
          <input type="checkbox" ng-model="important">
          important (apply "bold" class)
       </label><br>
       <label>
          <input type="checkbox" ng-model="error">
          error (apply "has-error" class)
       </label>
       <hr>
       <p ng-class="style">Using String Syntax</p>
       <input type="text" ng-model="style"
              placeholder="Type: bold strike red" aria-label="Type: bold strike red">
       <hr>
       <p ng-class="[style1, style2, style3]">Using Array Syntax</p>
       <input ng-model="style1"
              placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red"><br>
       <input ng-model="style2"
              placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red 2"><br>
       <input ng-model="style3"
              placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red 3"><br>
       <hr>
       <p ng-class="[style4, {orange: warning}]">Using Array and Map Syntax</p>
       <input ng-model="style4" placeholder="Type: bold, strike" aria-label="Type: bold, strike"><br>
       <label><input type="checkbox" ng-model="warning"> warning (apply "orange" class)</label>
     </file>
     <file name="style.css">
       .strike {
           text-decoration: line-through;
       }
       .bold {
           font-weight: bold;
       }
       .red {
           color: red;
       }
       .has-error {
           color: red;
           background-color: yellow;
       }
       .orange {
           color: orange;
       }
     </file>
     <file name="protractor.js" type="protractor">
       var ps = element.all(by.css('p'));

       it('should let you toggle the class', function() {

         expect(ps.first().getAttribute('class')).not.toMatch(/bold/);
         expect(ps.first().getAttribute('class')).not.toMatch(/has-error/);

         element(by.model('important')).click();
         expect(ps.first().getAttribute('class')).toMatch(/bold/);

         element(by.model('error')).click();
         expect(ps.first().getAttribute('class')).toMatch(/has-error/);
       });

       it('should let you toggle string example', function() {
         expect(ps.get(1).getAttribute('class')).toBe('');
         element(by.model('style')).clear();
         element(by.model('style')).sendKeys('red');
         expect(ps.get(1).getAttribute('class')).toBe('red');
       });

       it('array example should have 3 classes', function() {
         expect(ps.get(2).getAttribute('class')).toBe('');
         element(by.model('style1')).sendKeys('bold');
         element(by.model('style2')).sendKeys('strike');
         element(by.model('style3')).sendKeys('red');
         expect(ps.get(2).getAttribute('class')).toBe('bold strike red');
       });

       it('array with map example should have 2 classes', function() {
         expect(ps.last().getAttribute('class')).toBe('');
         element(by.model('style4')).sendKeys('bold');
         element(by.model('warning')).click();
         expect(ps.last().getAttribute('class')).toBe('bold orange');
       });
     </file>
   </example>

   @example
   ### Animations

   The example below demonstrates how to perform animations using ngClass.

   <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-class">
     <file name="index.html">
      <input id="setbtn" type="button" value="set" ng-click="myVar='my-class'">
      <input id="clearbtn" type="button" value="clear" ng-click="myVar=''">
      <br>
      <span class="base-class" ng-class="myVar">Sample Text</span>
     </file>
     <file name="style.css">
       .base-class {
         transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
       }

       .base-class.my-class {
         color: red;
         font-size:3em;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class', function() {
         expect(element(by.css('.base-class')).getAttribute('class')).not.
           toMatch(/my-class/);

         element(by.id('setbtn')).click();

         expect(element(by.css('.base-class')).getAttribute('class')).
           toMatch(/my-class/);

         element(by.id('clearbtn')).click();

         expect(element(by.css('.base-class')).getAttribute('class')).not.
           toMatch(/my-class/);
       });
     </file>
   </example>
 */
var ngClassDirective = classDirective('', true);

/**
 * @ngdoc directive
 * @name ngClassOdd
 * @restrict AC
 *
 * @description
 * The `ngClassOdd` and `ngClassEven` directives work exactly as
 * {@link ng.directive:ngClass ngClass}, except they work in
 * conjunction with `ngRepeat` and take effect only on odd (even) rows.
 *
 * This directive can be applied only within the scope of an
 * {@link ng.directive:ngRepeat ngRepeat}.
 *
 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#addClass addClass}       | just before the class is applied to the element   |
 * | {@link ng.$animate#removeClass removeClass} | just before the class is removed from the element |
 *
 * @element ANY
 * @param {expression} ngClassOdd {@link guide/expression Expression} to eval. The result
 *   of the evaluation can be a string representing space delimited class names or an array.
 *
 * @example
   <example name="ng-class-odd">
     <file name="index.html">
        <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
          <li ng-repeat="name in names">
           <span ng-class-odd="'odd'" ng-class-even="'even'">
             {{name}}
           </span>
          </li>
        </ol>
     </file>
     <file name="style.css">
       .odd {
         color: red;
       }
       .even {
         color: blue;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class-odd and ng-class-even', function() {
         expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
           toMatch(/odd/);
         expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
           toMatch(/even/);
       });
     </file>
   </example>
 *
 * <hr />
 * @example
 * An example on how to implement animations using `ngClassOdd`:
 *
   <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-class-odd-animate">
     <file name="index.html">
       <div ng-init="items=['Item 3', 'Item 2', 'Item 1', 'Item 0']">
         <button ng-click="items.unshift('Item ' + items.length)">Add item</button>
         <hr />
         <table>
           <tr ng-repeat="item in items" ng-class-odd="'odd'">
             <td>{{ item }}</td>
           </tr>
         </table>
       </div>
     </file>
     <file name="style.css">
       .odd {
         background: rgba(255, 255, 0, 0.25);
       }

       .odd-add, .odd-remove {
         transition: 1.5s;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should add new entries to the beginning of the list', function() {
         var button = element(by.buttonText('Add item'));
         var rows = element.all(by.repeater('item in items'));

         expect(rows.count()).toBe(4);
         expect(rows.get(0).getText()).toBe('Item 3');
         expect(rows.get(1).getText()).toBe('Item 2');

         button.click();

         expect(rows.count()).toBe(5);
         expect(rows.get(0).getText()).toBe('Item 4');
         expect(rows.get(1).getText()).toBe('Item 3');
       });

       it('should add odd class to odd entries', function() {
         var button = element(by.buttonText('Add item'));
         var rows = element.all(by.repeater('item in items'));

         expect(rows.get(0).getAttribute('class')).toMatch(/odd/);
         expect(rows.get(1).getAttribute('class')).not.toMatch(/odd/);

         button.click();

         expect(rows.get(0).getAttribute('class')).toMatch(/odd/);
         expect(rows.get(1).getAttribute('class')).not.toMatch(/odd/);
       });
     </file>
   </example>
 */
var ngClassOddDirective = classDirective('Odd', 0);

/**
 * @ngdoc directive
 * @name ngClassEven
 * @restrict AC
 *
 * @description
 * The `ngClassOdd` and `ngClassEven` directives work exactly as
 * {@link ng.directive:ngClass ngClass}, except they work in
 * conjunction with `ngRepeat` and take effect only on odd (even) rows.
 *
 * This directive can be applied only within the scope of an
 * {@link ng.directive:ngRepeat ngRepeat}.
 *
 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#addClass addClass}       | just before the class is applied to the element   |
 * | {@link ng.$animate#removeClass removeClass} | just before the class is removed from the element |
 *
 * @element ANY
 * @param {expression} ngClassEven {@link guide/expression Expression} to eval. The
 *   result of the evaluation can be a string representing space delimited class names or an array.
 *
 * @example
   <example name="ng-class-even">
     <file name="index.html">
        <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
          <li ng-repeat="name in names">
           <span ng-class-odd="'odd'" ng-class-even="'even'">
             {{name}} &nbsp; &nbsp; &nbsp;
           </span>
          </li>
        </ol>
     </file>
     <file name="style.css">
       .odd {
         color: red;
       }
       .even {
         color: blue;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class-odd and ng-class-even', function() {
         expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
           toMatch(/odd/);
         expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
           toMatch(/even/);
       });
     </file>
   </example>
 *
 * <hr />
 * @example
 * An example on how to implement animations using `ngClassEven`:
 *
   <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-class-even-animate">
     <file name="index.html">
       <div ng-init="items=['Item 3', 'Item 2', 'Item 1', 'Item 0']">
         <button ng-click="items.unshift('Item ' + items.length)">Add item</button>
         <hr />
         <table>
           <tr ng-repeat="item in items" ng-class-even="'even'">
             <td>{{ item }}</td>
           </tr>
         </table>
       </div>
     </file>
     <file name="style.css">
       .even {
         background: rgba(255, 255, 0, 0.25);
       }

       .even-add, .even-remove {
         transition: 1.5s;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should add new entries to the beginning of the list', function() {
         var button = element(by.buttonText('Add item'));
         var rows = element.all(by.repeater('item in items'));

         expect(rows.count()).toBe(4);
         expect(rows.get(0).getText()).toBe('Item 3');
         expect(rows.get(1).getText()).toBe('Item 2');

         button.click();

         expect(rows.count()).toBe(5);
         expect(rows.get(0).getText()).toBe('Item 4');
         expect(rows.get(1).getText()).toBe('Item 3');
       });

       it('should add even class to even entries', function() {
         var button = element(by.buttonText('Add item'));
         var rows = element.all(by.repeater('item in items'));

         expect(rows.get(0).getAttribute('class')).not.toMatch(/even/);
         expect(rows.get(1).getAttribute('class')).toMatch(/even/);

         button.click();

         expect(rows.get(0).getAttribute('class')).not.toMatch(/even/);
         expect(rows.get(1).getAttribute('class')).toMatch(/even/);
       });
     </file>
   </example>
 */
var ngClassEvenDirective = classDirective('Even', 1);

/**
 * @ngdoc directive
 * @name ngCloak
 * @restrict AC
 *
 * @description
 * The `ngCloak` directive is used to prevent the AngularJS html template from being briefly
 * displayed by the browser in its raw (uncompiled) form while your application is loading. Use this
 * directive to avoid the undesirable flicker effect caused by the html template display.
 *
 * The directive can be applied to the `<body>` element, but the preferred usage is to apply
 * multiple `ngCloak` directives to small portions of the page to permit progressive rendering
 * of the browser view.
 *
 * `ngCloak` works in cooperation with the following css rule embedded within `angular.js` and
 * `angular.min.js`.
 * For CSP mode please add `angular-csp.css` to your html file (see {@link ng.directive:ngCsp ngCsp}).
 *
 * ```css
 * [ng\:cloak], [ng-cloak], [data-ng-cloak], [x-ng-cloak], .ng-cloak, .x-ng-cloak {
 *   display: none !important;
 * }
 * ```
 *
 * When this css rule is loaded by the browser, all html elements (including their children) that
 * are tagged with the `ngCloak` directive are hidden. When AngularJS encounters this directive
 * during the compilation of the template it deletes the `ngCloak` element attribute, making
 * the compiled element visible.
 *
 * For the best result, the `angular.js` script must be loaded in the head section of the html
 * document; alternatively, the css rule above must be included in the external stylesheet of the
 * application.
 *
 * @element ANY
 *
 * @example
   <example name="ng-cloak">
     <file name="index.html">
        <div id="template1" ng-cloak>{{ 'hello' }}</div>
        <div id="template2" class="ng-cloak">{{ 'world' }}</div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should remove the template directive and css class', function() {
         expect($('#template1').getAttribute('ng-cloak')).
           toBeNull();
         expect($('#template2').getAttribute('ng-cloak')).
           toBeNull();
       });
     </file>
   </example>
 *
 */
var ngCloakDirective = ngDirective({
  compile: function(element, attr) {
    attr.$set('ngCloak', undefined);
    element.removeClass('ng-cloak');
  }
});

/**
 * @ngdoc directive
 * @name ngController
 *
 * @description
 * The `ngController` directive attaches a controller class to the view. This is a key aspect of how angular
 * supports the principles behind the Model-View-Controller design pattern.
 *
 * MVC components in angular:
 *
 * * Model — Models are the properties of a scope; scopes are attached to the DOM where scope properties
 *   are accessed through bindings.
 * * View — The template (HTML with data bindings) that is rendered into the View.
 * * Controller — The `ngController` directive specifies a Controller class; the class contains business
 *   logic behind the application to decorate the scope with functions and values
 *
 * Note that you can also attach controllers to the DOM by declaring it in a route definition
 * via the {@link ngRoute.$route $route} service. A common mistake is to declare the controller
 * again using `ng-controller` in the template itself.  This will cause the controller to be attached
 * and executed twice.
 *
 * @element ANY
 * @scope
 * @priority 500
 * @param {expression} ngController Name of a constructor function registered with the current
 * {@link ng.$controllerProvider $controllerProvider} or an {@link guide/expression expression}
 * that on the current scope evaluates to a constructor function.
 *
 * The controller instance can be published into a scope property by specifying
 * `ng-controller="as propertyName"`.
 *
 * @example
 * Here is a simple form for editing user contact information. Adding, removing, clearing, and
 * greeting are methods declared on the controller (see source tab). These methods can
 * easily be called from the AngularJS markup. Any changes to the data are automatically reflected
 * in the View without the need for a manual update.
 *
 * Two different declaration styles are included below:
 *
 * * one binds methods and properties directly onto the controller using `this`:
 * `ng-controller="SettingsController1 as settings"`
 * * one injects `$scope` into the controller:
 * `ng-controller="SettingsController2"`
 *
 * The second option is more common in the AngularJS community, and is generally used in boilerplates
 * and in this guide. However, there are advantages to binding properties directly to the controller
 * and avoiding scope.
 *
 * * Using `controller as` makes it obvious which controller you are accessing in the template when
 * multiple controllers apply to an element.
 * * If you are writing your controllers as classes you have easier access to the properties and
 * methods, which will appear on the scope, from inside the controller code.
 * * Since there is always a `.` in the bindings, you don't have to worry about prototypal
 * inheritance masking primitives.
 *
 * This example demonstrates the `controller as` syntax.
 *
 * <example name="ngControllerAs" module="controllerAsExample">
 *   <file name="index.html">
 *    <div id="ctrl-as-exmpl" ng-controller="SettingsController1 as settings">
 *      <label>Name: <input type="text" ng-model="settings.name"/></label>
 *      <button ng-click="settings.greet()">greet</button><br/>
 *      Contact:
 *      <ul>
 *        <li ng-repeat="contact in settings.contacts">
 *          <select ng-model="contact.type" aria-label="Contact method" id="select_{{$index}}">
 *             <option>phone</option>
 *             <option>email</option>
 *          </select>
 *          <input type="text" ng-model="contact.value" aria-labelledby="select_{{$index}}" />
 *          <button ng-click="settings.clearContact(contact)">clear</button>
 *          <button ng-click="settings.removeContact(contact)" aria-label="Remove">X</button>
 *        </li>
 *        <li><button ng-click="settings.addContact()">add</button></li>
 *     </ul>
 *    </div>
 *   </file>
 *   <file name="app.js">
 *    angular.module('controllerAsExample', [])
 *      .controller('SettingsController1', SettingsController1);
 *
 *    function SettingsController1() {
 *      this.name = 'John Smith';
 *      this.contacts = [
 *        {type: 'phone', value: '408 555 1212'},
 *        {type: 'email', value: 'john.smith@example.org'}
 *      ];
 *    }
 *
 *    SettingsController1.prototype.greet = function() {
 *      alert(this.name);
 *    };
 *
 *    SettingsController1.prototype.addContact = function() {
 *      this.contacts.push({type: 'email', value: 'yourname@example.org'});
 *    };
 *
 *    SettingsController1.prototype.removeContact = function(contactToRemove) {
 *     var index = this.contacts.indexOf(contactToRemove);
 *      this.contacts.splice(index, 1);
 *    };
 *
 *    SettingsController1.prototype.clearContact = function(contact) {
 *      contact.type = 'phone';
 *      contact.value = '';
 *    };
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     it('should check controller as', function() {
 *       var container = element(by.id('ctrl-as-exmpl'));
 *         expect(container.element(by.model('settings.name'))
 *           .getAttribute('value')).toBe('John Smith');
 *
 *       var firstRepeat =
 *           container.element(by.repeater('contact in settings.contacts').row(0));
 *       var secondRepeat =
 *           container.element(by.repeater('contact in settings.contacts').row(1));
 *
 *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('408 555 1212');
 *
 *       expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('john.smith@example.org');
 *
 *       firstRepeat.element(by.buttonText('clear')).click();
 *
 *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('');
 *
 *       container.element(by.buttonText('add')).click();
 *
 *       expect(container.element(by.repeater('contact in settings.contacts').row(2))
 *           .element(by.model('contact.value'))
 *           .getAttribute('value'))
 *           .toBe('yourname@example.org');
 *     });
 *   </file>
 * </example>
 *
 * This example demonstrates the "attach to `$scope`" style of controller.
 *
 * <example name="ngController" module="controllerExample">
 *  <file name="index.html">
 *   <div id="ctrl-exmpl" ng-controller="SettingsController2">
 *     <label>Name: <input type="text" ng-model="name"/></label>
 *     <button ng-click="greet()">greet</button><br/>
 *     Contact:
 *     <ul>
 *       <li ng-repeat="contact in contacts">
 *         <select ng-model="contact.type" id="select_{{$index}}">
 *            <option>phone</option>
 *            <option>email</option>
 *         </select>
 *         <input type="text" ng-model="contact.value" aria-labelledby="select_{{$index}}" />
 *         <button ng-click="clearContact(contact)">clear</button>
 *         <button ng-click="removeContact(contact)">X</button>
 *       </li>
 *       <li>[ <button ng-click="addContact()">add</button> ]</li>
 *    </ul>
 *   </div>
 *  </file>
 *  <file name="app.js">
 *   angular.module('controllerExample', [])
 *     .controller('SettingsController2', ['$scope', SettingsController2]);
 *
 *   function SettingsController2($scope) {
 *     $scope.name = 'John Smith';
 *     $scope.contacts = [
 *       {type:'phone', value:'408 555 1212'},
 *       {type:'email', value:'john.smith@example.org'}
 *     ];
 *
 *     $scope.greet = function() {
 *       alert($scope.name);
 *     };
 *
 *     $scope.addContact = function() {
 *       $scope.contacts.push({type:'email', value:'yourname@example.org'});
 *     };
 *
 *     $scope.removeContact = function(contactToRemove) {
 *       var index = $scope.contacts.indexOf(contactToRemove);
 *       $scope.contacts.splice(index, 1);
 *     };
 *
 *     $scope.clearContact = function(contact) {
 *       contact.type = 'phone';
 *       contact.value = '';
 *     };
 *   }
 *  </file>
 *  <file name="protractor.js" type="protractor">
 *    it('should check controller', function() {
 *      var container = element(by.id('ctrl-exmpl'));
 *
 *      expect(container.element(by.model('name'))
 *          .getAttribute('value')).toBe('John Smith');
 *
 *      var firstRepeat =
 *          container.element(by.repeater('contact in contacts').row(0));
 *      var secondRepeat =
 *          container.element(by.repeater('contact in contacts').row(1));
 *
 *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('408 555 1212');
 *      expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('john.smith@example.org');
 *
 *      firstRepeat.element(by.buttonText('clear')).click();
 *
 *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('');
 *
 *      container.element(by.buttonText('add')).click();
 *
 *      expect(container.element(by.repeater('contact in contacts').row(2))
 *          .element(by.model('contact.value'))
 *          .getAttribute('value'))
 *          .toBe('yourname@example.org');
 *    });
 *  </file>
 *</example>

 */
var ngControllerDirective = [function() {
  return {
    restrict: 'A',
    scope: true,
    controller: '@',
    priority: 500
  };
}];

/**
 * @ngdoc directive
 * @name ngCsp
 *
 * @restrict A
 * @element ANY
 * @description
 *
 * AngularJS has some features that can conflict with certain restrictions that are applied when using
 * [CSP (Content Security Policy)](https://developer.mozilla.org/en/Security/CSP) rules.
 *
 * If you intend to implement CSP with these rules then you must tell AngularJS not to use these
 * features.
 *
 * This is necessary when developing things like Google Chrome Extensions or Universal Windows Apps.
 *
 *
 * The following default rules in CSP affect AngularJS:
 *
 * * The use of `eval()`, `Function(string)` and similar functions to dynamically create and execute
 * code from strings is forbidden. AngularJS makes use of this in the {@link $parse} service to
 * provide a 30% increase in the speed of evaluating AngularJS expressions. (This CSP rule can be
 * disabled with the CSP keyword `unsafe-eval`, but it is generally not recommended as it would
 * weaken the protections offered by CSP.)
 *
 * * The use of inline resources, such as inline `<script>` and `<style>` elements, are forbidden.
 * This prevents apps from injecting custom styles directly into the document. AngularJS makes use of
 * this to include some CSS rules (e.g. {@link ngCloak} and {@link ngHide}). To make these
 * directives work when a CSP rule is blocking inline styles, you must link to the `angular-csp.css`
 * in your HTML manually. (This CSP rule can be disabled with the CSP keyword `unsafe-inline`, but
 * it is generally not recommended as it would weaken the protections offered by CSP.)
 *
 * If you do not provide `ngCsp` then AngularJS tries to autodetect if CSP is blocking dynamic code
 * creation from strings (e.g., `unsafe-eval` not specified in CSP header) and automatically
 * deactivates this feature in the {@link $parse} service. This autodetection, however, triggers a
 * CSP error to be logged in the console:
 *
 * ```
 * Refused to evaluate a string as JavaScript because 'unsafe-eval' is not an allowed source of
 * script in the following Content Security Policy directive: "default-src 'self'". Note that
 * 'script-src' was not explicitly set, so 'default-src' is used as a fallback.
 * ```
 *
 * This error is harmless but annoying. To prevent the error from showing up, put the `ngCsp`
 * directive on an element of the HTML document that appears before the `<script>` tag that loads
 * the `angular.js` file.
 *
 * *Note: This directive is only available in the `ng-csp` and `data-ng-csp` attribute form.*
 *
 * You can specify which of the CSP related AngularJS features should be deactivated by providing
 * a value for the `ng-csp` attribute. The options are as follows:
 *
 * * no-inline-style: this stops AngularJS from injecting CSS styles into the DOM
 *
 * * no-unsafe-eval: this stops AngularJS from optimizing $parse with unsafe eval of strings
 *
 * You can use these values in the following combinations:
 *
 *
 * * No declaration means that AngularJS will assume that you can do inline styles, but it will do
 * a runtime check for unsafe-eval. E.g. `<body>`. This is backwardly compatible with previous
 * versions of AngularJS.
 *
 * * A simple `ng-csp` (or `data-ng-csp`) attribute will tell AngularJS to deactivate both inline
 * styles and unsafe eval. E.g. `<body ng-csp>`. This is backwardly compatible with previous
 * versions of AngularJS.
 *
 * * Specifying only `no-unsafe-eval` tells AngularJS that we must not use eval, but that we can
 * inject inline styles. E.g. `<body ng-csp="no-unsafe-eval">`.
 *
 * * Specifying only `no-inline-style` tells AngularJS that we must not inject styles, but that we can
 * run eval - no automatic check for unsafe eval will occur. E.g. `<body ng-csp="no-inline-style">`
 *
 * * Specifying both `no-unsafe-eval` and `no-inline-style` tells AngularJS that we must not inject
 * styles nor use eval, which is the same as an empty: ng-csp.
 * E.g.`<body ng-csp="no-inline-style;no-unsafe-eval">`
 *
 * @example
 *
 * This example shows how to apply the `ngCsp` directive to the `html` tag.
   ```html
     <!doctype html>
     <html ng-app ng-csp>
     ...
     ...
     </html>
   ```

  <!-- Note: the `.csp` suffix in the example name triggers CSP mode in our http server! -->
  <example name="example.csp" module="cspExample" ng-csp="true">
    <file name="index.html">
      <div ng-controller="MainController as ctrl">
        <div>
          <button ng-click="ctrl.inc()" id="inc">Increment</button>
          <span id="counter">
            {{ctrl.counter}}
          </span>
        </div>

        <div>
          <button ng-click="ctrl.evil()" id="evil">Evil</button>
          <span id="evilError">
            {{ctrl.evilError}}
          </span>
        </div>
      </div>
    </file>
    <file name="script.js">
       angular.module('cspExample', [])
         .controller('MainController', function MainController() {
            this.counter = 0;
            this.inc = function() {
              this.counter++;
            };
            this.evil = function() {
              try {
                eval('1+2'); // eslint-disable-line no-eval
              } catch (e) {
                this.evilError = e.message;
              }
            };
          });
    </file>
    <file name="protractor.js" type="protractor">
      var util, webdriver;

      var incBtn = element(by.id('inc'));
      var counter = element(by.id('counter'));
      var evilBtn = element(by.id('evil'));
      var evilError = element(by.id('evilError'));

      function getAndClearSevereErrors() {
        return browser.manage().logs().get('browser').then(function(browserLog) {
          return browserLog.filter(function(logEntry) {
            return logEntry.level.value > webdriver.logging.Level.WARNING.value;
          });
        });
      }

      function clearErrors() {
        getAndClearSevereErrors();
      }

      function expectNoErrors() {
        getAndClearSevereErrors().then(function(filteredLog) {
          expect(filteredLog.length).toEqual(0);
          if (filteredLog.length) {
            console.log('browser console errors: ' + util.inspect(filteredLog));
          }
        });
      }

      function expectError(regex) {
        getAndClearSevereErrors().then(function(filteredLog) {
          var found = false;
          filteredLog.forEach(function(log) {
            if (log.message.match(regex)) {
              found = true;
            }
          });
          if (!found) {
            throw new Error('expected an error that matches ' + regex);
          }
        });
      }

      beforeEach(function() {
        util = require('util');
        webdriver = require('selenium-webdriver');
      });

      // For now, we only test on Chrome,
      // as Safari does not load the page with Protractor's injected scripts,
      // and Firefox webdriver always disables content security policy (#6358)
      if (browser.params.browser !== 'chrome') {
        return;
      }

      it('should not report errors when the page is loaded', function() {
        // clear errors so we are not dependent on previous tests
        clearErrors();
        // Need to reload the page as the page is already loaded when
        // we come here
        browser.driver.getCurrentUrl().then(function(url) {
          browser.get(url);
        });
        expectNoErrors();
      });

      it('should evaluate expressions', function() {
        expect(counter.getText()).toEqual('0');
        incBtn.click();
        expect(counter.getText()).toEqual('1');
        expectNoErrors();
      });

      it('should throw and report an error when using "eval"', function() {
        evilBtn.click();
        expect(evilError.getText()).toMatch(/Content Security Policy/);
        expectError(/Content Security Policy/);
      });
    </file>
  </example>
  */

// `ngCsp` is not implemented as a proper directive any more, because we need it be processed while
// we bootstrap the app (before `$parse` is instantiated). For this reason, we just have the `csp()`
// fn that looks for the `ng-csp` attribute anywhere in the current doc.

/**
 * @ngdoc directive
 * @name ngClick
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * The ngClick directive allows you to specify custom behavior when
 * an element is clicked.
 *
 * @param {expression} ngClick {@link guide/expression Expression} to evaluate upon
 * click. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-click">
     <file name="index.html">
      <button ng-click="count = count + 1" ng-init="count=0">
        Increment
      </button>
      <span>
        count: {{count}}
      </span>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-click', function() {
         expect(element(by.binding('count')).getText()).toMatch('0');
         element(by.css('button')).click();
         expect(element(by.binding('count')).getText()).toMatch('1');
       });
     </file>
   </example>
 */
/*
 * A collection of directives that allows creation of custom event handlers that are defined as
 * AngularJS expressions and are compiled and executed within the current scope.
 */
var ngEventDirectives = {};

// For events that might fire synchronously during DOM manipulation
// we need to execute their event handlers asynchronously using $evalAsync,
// so that they are not executed in an inconsistent state.
var forceAsyncEvents = {
  'blur': true,
  'focus': true
};
forEach(
  'click dblclick mousedown mouseup mouseover mouseout mousemove mouseenter mouseleave keydown keyup keypress submit focus blur copy cut paste'.split(' '),
  function(eventName) {
    var directiveName = directiveNormalize('ng-' + eventName);
    ngEventDirectives[directiveName] = ['$parse', '$rootScope', '$exceptionHandler', function($parse, $rootScope, $exceptionHandler) {
      return createEventDirective($parse, $rootScope, $exceptionHandler, directiveName, eventName, forceAsyncEvents[eventName]);
    }];
  }
);

function createEventDirective($parse, $rootScope, $exceptionHandler, directiveName, eventName, forceAsync) {
  return {
    restrict: 'A',
    compile: function($element, attr) {
      // NOTE:
      // We expose the powerful `$event` object on the scope that provides access to the Window,
      // etc. This is OK, because expressions are not sandboxed any more (and the expression
      // sandbox was never meant to be a security feature anyway).
      var fn = $parse(attr[directiveName]);
      return function ngEventHandler(scope, element) {
        element.on(eventName, function(event) {
          var callback = function() {
            fn(scope, {$event: event});
          };

          if (!$rootScope.$$phase) {
            scope.$apply(callback);
          } else if (forceAsync) {
            scope.$evalAsync(callback);
          } else {
            try {
              callback();
            } catch (error) {
              $exceptionHandler(error);
            }
          }
        });
      };
    }
  };
}

/**
 * @ngdoc directive
 * @name ngDblclick
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * The `ngDblclick` directive allows you to specify custom behavior on a dblclick event.
 *
 * @param {expression} ngDblclick {@link guide/expression Expression} to evaluate upon
 * a dblclick. (The Event object is available as `$event`)
 *
 * @example
   <example name="ng-dblclick">
     <file name="index.html">
      <button ng-dblclick="count = count + 1" ng-init="count=0">
        Increment (on double click)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngMousedown
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * The ngMousedown directive allows you to specify custom behavior on mousedown event.
 *
 * @param {expression} ngMousedown {@link guide/expression Expression} to evaluate upon
 * mousedown. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mousedown">
     <file name="index.html">
      <button ng-mousedown="count = count + 1" ng-init="count=0">
        Increment (on mouse down)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngMouseup
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on mouseup event.
 *
 * @param {expression} ngMouseup {@link guide/expression Expression} to evaluate upon
 * mouseup. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mouseup">
     <file name="index.html">
      <button ng-mouseup="count = count + 1" ng-init="count=0">
        Increment (on mouse up)
      </button>
      count: {{count}}
     </file>
   </example>
 */

/**
 * @ngdoc directive
 * @name ngMouseover
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on mouseover event.
 *
 * @param {expression} ngMouseover {@link guide/expression Expression} to evaluate upon
 * mouseover. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mouseover">
     <file name="index.html">
      <button ng-mouseover="count = count + 1" ng-init="count=0">
        Increment (when mouse is over)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngMouseenter
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on mouseenter event.
 *
 * @param {expression} ngMouseenter {@link guide/expression Expression} to evaluate upon
 * mouseenter. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mouseenter">
     <file name="index.html">
      <button ng-mouseenter="count = count + 1" ng-init="count=0">
        Increment (when mouse enters)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngMouseleave
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on mouseleave event.
 *
 * @param {expression} ngMouseleave {@link guide/expression Expression} to evaluate upon
 * mouseleave. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mouseleave">
     <file name="index.html">
      <button ng-mouseleave="count = count + 1" ng-init="count=0">
        Increment (when mouse leaves)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngMousemove
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on mousemove event.
 *
 * @param {expression} ngMousemove {@link guide/expression Expression} to evaluate upon
 * mousemove. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-mousemove">
     <file name="index.html">
      <button ng-mousemove="count = count + 1" ng-init="count=0">
        Increment (when mouse moves)
      </button>
      count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngKeydown
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on keydown event.
 *
 * @param {expression} ngKeydown {@link guide/expression Expression} to evaluate upon
 * keydown. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example name="ng-keydown">
     <file name="index.html">
      <input ng-keydown="count = count + 1" ng-init="count=0">
      key down count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngKeyup
 * @restrict A
 * @element ANY
 * @priority 0
 *
 * @description
 * Specify custom behavior on keyup event.
 *
 * @param {expression} ngKeyup {@link guide/expression Expression} to evaluate upon
 * keyup. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example name="ng-keyup">
     <file name="index.html">
       <p>Typing in the input box below updates the key count</p>
       <input ng-keyup="count = count + 1" ng-init="count=0"> key up count: {{count}}

       <p>Typing in the input box below updates the keycode</p>
       <input ng-keyup="event=$event">
       <p>event keyCode: {{ event.keyCode }}</p>
       <p>event altKey: {{ event.altKey }}</p>
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngKeypress
 * @restrict A
 * @element ANY
 *
 * @description
 * Specify custom behavior on keypress event.
 *
 * @param {expression} ngKeypress {@link guide/expression Expression} to evaluate upon
 * keypress. ({@link guide/expression#-event- Event object is available as `$event`}
 * and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example name="ng-keypress">
     <file name="index.html">
      <input ng-keypress="count = count + 1" ng-init="count=0">
      key press count: {{count}}
     </file>
   </example>
 */


/**
 * @ngdoc directive
 * @name ngSubmit
 * @restrict A
 * @element form
 * @priority 0
 *
 * @description
 * Enables binding AngularJS expressions to onsubmit events.
 *
 * Additionally it prevents the default action (which for form means sending the request to the
 * server and reloading the current page), but only if the form does not contain `action`,
 * `data-action`, or `x-action` attributes.
 *
 * <div class="alert alert-warning">
 * **Warning:** Be careful not to cause "double-submission" by using both the `ngClick` and
 * `ngSubmit` handlers together. See the
 * {@link form#submitting-a-form-and-preventing-the-default-action `form` directive documentation}
 * for a detailed discussion of when `ngSubmit` may be triggered.
 * </div>
 *
 * @param {expression} ngSubmit {@link guide/expression Expression} to eval.
 * ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example module="submitExample" name="ng-submit">
     <file name="index.html">
      <script>
        angular.module('submitExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.list = [];
            $scope.text = 'hello';
            $scope.submit = function() {
              if ($scope.text) {
                $scope.list.push(this.text);
                $scope.text = '';
              }
            };
          }]);
      </script>
      <form ng-submit="submit()" ng-controller="ExampleController">
        Enter text and hit enter:
        <input type="text" ng-model="text" name="text" />
        <input type="submit" id="submit" value="Submit" />
        <pre>list={{list}}</pre>
      </form>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-submit', function() {
         expect(element(by.binding('list')).getText()).toBe('list=[]');
         element(by.css('#submit')).click();
         expect(element(by.binding('list')).getText()).toContain('hello');
         expect(element(by.model('text')).getAttribute('value')).toBe('');
       });
       it('should ignore empty strings', function() {
         expect(element(by.binding('list')).getText()).toBe('list=[]');
         element(by.css('#submit')).click();
         element(by.css('#submit')).click();
         expect(element(by.binding('list')).getText()).toContain('hello');
        });
     </file>
   </example>
 */

/**
 * @ngdoc directive
 * @name ngFocus
 * @restrict A
 * @element window, input, select, textarea, a
 * @priority 0
 *
 * @description
 * Specify custom behavior on focus event.
 *
 * Note: As the `focus` event is executed synchronously when calling `input.focus()`
 * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
 * during an `$apply` to ensure a consistent state.
 *
 * @param {expression} ngFocus {@link guide/expression Expression} to evaluate upon
 * focus. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
 * See {@link ng.directive:ngClick ngClick}
 */

/**
 * @ngdoc directive
 * @name ngBlur
 * @restrict A
 * @element window, input, select, textarea, a
 * @priority 0
 *
 * @description
 * Specify custom behavior on blur event.
 *
 * A [blur event](https://developer.mozilla.org/en-US/docs/Web/Events/blur) fires when
 * an element has lost focus.
 *
 * Note: As the `blur` event is executed synchronously also during DOM manipulations
 * (e.g. removing a focussed input),
 * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
 * during an `$apply` to ensure a consistent state.
 *
 * @param {expression} ngBlur {@link guide/expression Expression} to evaluate upon
 * blur. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
 * See {@link ng.directive:ngClick ngClick}
 */

/**
 * @ngdoc directive
 * @name ngCopy
 * @restrict A
 * @element window, input, select, textarea, a
 * @priority 0
 *
 * @description
 * Specify custom behavior on copy event.
 *
 * @param {expression} ngCopy {@link guide/expression Expression} to evaluate upon
 * copy. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-copy">
     <file name="index.html">
      <input ng-copy="copied=true" ng-init="copied=false; value='copy me'" ng-model="value">
      copied: {{copied}}
     </file>
   </example>
 */

/**
 * @ngdoc directive
 * @name ngCut
 * @restrict A
 * @element window, input, select, textarea, a
 * @priority 0
 *
 * @description
 * Specify custom behavior on cut event.
 *
 * @param {expression} ngCut {@link guide/expression Expression} to evaluate upon
 * cut. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-cut">
     <file name="index.html">
      <input ng-cut="cut=true" ng-init="cut=false; value='cut me'" ng-model="value">
      cut: {{cut}}
     </file>
   </example>
 */

/**
 * @ngdoc directive
 * @name ngPaste
 * @restrict A
 * @element window, input, select, textarea, a
 * @priority 0
 *
 * @description
 * Specify custom behavior on paste event.
 *
 * @param {expression} ngPaste {@link guide/expression Expression} to evaluate upon
 * paste. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example name="ng-paste">
     <file name="index.html">
      <input ng-paste="paste=true" ng-init="paste=false" placeholder='paste here'>
      pasted: {{paste}}
     </file>
   </example>
 */

/**
 * @ngdoc directive
 * @name ngIf
 * @restrict A
 * @multiElement
 *
 * @description
 * The `ngIf` directive removes or recreates a portion of the DOM tree based on an
 * {expression}. If the expression assigned to `ngIf` evaluates to a false
 * value then the element is removed from the DOM, otherwise a clone of the
 * element is reinserted into the DOM.
 *
 * `ngIf` differs from `ngShow` and `ngHide` in that `ngIf` completely removes and recreates the
 * element in the DOM rather than changing its visibility via the `display` css property.  A common
 * case when this difference is significant is when using css selectors that rely on an element's
 * position within the DOM, such as the `:first-child` or `:last-child` pseudo-classes.
 *
 * Note that when an element is removed using `ngIf` its scope is destroyed and a new scope
 * is created when the element is restored.  The scope created within `ngIf` inherits from
 * its parent scope using
 * [prototypal inheritance](https://github.com/angular/angular.js/wiki/Understanding-Scopes#javascript-prototypal-inheritance).
 * An important implication of this is if `ngModel` is used within `ngIf` to bind to
 * a javascript primitive defined in the parent scope. In this case any modifications made to the
 * variable within the child scope will override (hide) the value in the parent scope.
 *
 * Also, `ngIf` recreates elements using their compiled state. An example of this behavior
 * is if an element's class attribute is directly modified after it's compiled, using something like
 * jQuery's `.addClass()` method, and the element is later removed. When `ngIf` recreates the element
 * the added class will be lost because the original compiled state is used to regenerate the element.
 *
 * Additionally, you can provide animations via the `ngAnimate` module to animate the `enter`
 * and `leave` effects.
 *
 * @animations
 * | Animation                        | Occurs                               |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#enter enter}  | just after the `ngIf` contents change and a new DOM element is created and injected into the `ngIf` container |
 * | {@link ng.$animate#leave leave}  | just before the `ngIf` contents are removed from the DOM |
 *
 * @element ANY
 * @scope
 * @priority 600
 * @param {expression} ngIf If the {@link guide/expression expression} is falsy then
 *     the element is removed from the DOM tree. If it is truthy a copy of the compiled
 *     element is added to the DOM tree.
 *
 * @example
  <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-if">
    <file name="index.html">
      <label>Click me: <input type="checkbox" ng-model="checked" ng-init="checked=true" /></label><br/>
      Show when checked:
      <span ng-if="checked" class="animate-if">
        This is removed when the checkbox is unchecked.
      </span>
    </file>
    <file name="animations.css">
      .animate-if {
        background:white;
        border:1px solid black;
        padding:10px;
      }

      .animate-if.ng-enter, .animate-if.ng-leave {
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
      }

      .animate-if.ng-enter,
      .animate-if.ng-leave.ng-leave-active {
        opacity:0;
      }

      .animate-if.ng-leave,
      .animate-if.ng-enter.ng-enter-active {
        opacity:1;
      }
    </file>
  </example>
 */
var ngIfDirective = ['$animate', '$compile', function($animate, $compile) {
  return {
    multiElement: true,
    transclude: 'element',
    priority: 600,
    terminal: true,
    restrict: 'A',
    $$tlb: true,
    link: function($scope, $element, $attr, ctrl, $transclude) {
        var block, childScope, previousElements;
        $scope.$watch($attr.ngIf, function ngIfWatchAction(value) {

          if (value) {
            if (!childScope) {
              $transclude(function(clone, newScope) {
                childScope = newScope;
                clone[clone.length++] = $compile.$$createComment('end ngIf', $attr.ngIf);
                // Note: We only need the first/last node of the cloned nodes.
                // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                // by a directive with templateUrl when its template arrives.
                block = {
                  clone: clone
                };
                $animate.enter(clone, $element.parent(), $element);
              });
            }
          } else {
            if (previousElements) {
              previousElements.remove();
              previousElements = null;
            }
            if (childScope) {
              childScope.$destroy();
              childScope = null;
            }
            if (block) {
              previousElements = getBlockNodes(block.clone);
              $animate.leave(previousElements).done(function(response) {
                if (response !== false) previousElements = null;
              });
              block = null;
            }
          }
        });
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngInclude
 * @restrict ECA
 * @scope
 * @priority -400
 *
 * @description
 * Fetches, compiles and includes an external HTML fragment.
 *
 * By default, the template URL is restricted to the same domain and protocol as the
 * application document. This is done by calling {@link $sce#getTrustedResourceUrl
 * $sce.getTrustedResourceUrl} on it. To load templates from other domains or protocols
 * you may either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist them} or
 * {@link $sce#trustAsResourceUrl wrap them} as trusted values. Refer to AngularJS's {@link
 * ng.$sce Strict Contextual Escaping}.
 *
 * In addition, the browser's
 * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
 * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
 * policy may further restrict whether the template is successfully loaded.
 * For example, `ngInclude` won't work for cross-domain requests on all browsers and for `file://`
 * access on some browsers.
 *
 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#enter enter}  | when the expression changes, on the new include |
 * | {@link ng.$animate#leave leave}  | when the expression changes, on the old include |
 *
 * The enter and leave animation occur concurrently.
 *
 * @param {string} ngInclude|src AngularJS expression evaluating to URL. If the source is a string constant,
 *                 make sure you wrap it in **single** quotes, e.g. `src="'myPartialTemplate.html'"`.
 * @param {string=} onload Expression to evaluate when a new partial is loaded.
 *                  <div class="alert alert-warning">
 *                  **Note:** When using onload on SVG elements in IE11, the browser will try to call
 *                  a function with the name on the window element, which will usually throw a
 *                  "function is undefined" error. To fix this, you can instead use `data-onload` or a
 *                  different form that {@link guide/directive#normalization matches} `onload`.
 *                  </div>
   *
 * @param {string=} autoscroll Whether `ngInclude` should call {@link ng.$anchorScroll
 *                  $anchorScroll} to scroll the viewport after the content is loaded.
 *
 *                  - If the attribute is not set, disable scrolling.
 *                  - If the attribute is set without value, enable scrolling.
 *                  - Otherwise enable scrolling only if the expression evaluates to truthy value.
 *
 * @example
  <example module="includeExample" deps="angular-animate.js" animations="true" name="ng-include">
    <file name="index.html">
     <div ng-controller="ExampleController">
       <select ng-model="template" ng-options="t.name for t in templates">
        <option value="">(blank)</option>
       </select>
       url of the template: <code>{{template.url}}</code>
       <hr/>
       <div class="slide-animate-container">
         <div class="slide-animate" ng-include="template.url"></div>
       </div>
     </div>
    </file>
    <file name="script.js">
      angular.module('includeExample', ['ngAnimate'])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.templates =
            [{ name: 'template1.html', url: 'template1.html'},
             { name: 'template2.html', url: 'template2.html'}];
          $scope.template = $scope.templates[0];
        }]);
     </file>
    <file name="template1.html">
      Content of template1.html
    </file>
    <file name="template2.html">
      Content of template2.html
    </file>
    <file name="animations.css">
      .slide-animate-container {
        position:relative;
        background:white;
        border:1px solid black;
        height:40px;
        overflow:hidden;
      }

      .slide-animate {
        padding:10px;
      }

      .slide-animate.ng-enter, .slide-animate.ng-leave {
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;

        position:absolute;
        top:0;
        left:0;
        right:0;
        bottom:0;
        display:block;
        padding:10px;
      }

      .slide-animate.ng-enter {
        top:-50px;
      }
      .slide-animate.ng-enter.ng-enter-active {
        top:0;
      }

      .slide-animate.ng-leave {
        top:0;
      }
      .slide-animate.ng-leave.ng-leave-active {
        top:50px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var templateSelect = element(by.model('template'));
      var includeElem = element(by.css('[ng-include]'));

      it('should load template1.html', function() {
        expect(includeElem.getText()).toMatch(/Content of template1.html/);
      });

      it('should load template2.html', function() {
        if (browser.params.browser === 'firefox') {
          // Firefox can't handle using selects
          // See https://github.com/angular/protractor/issues/480
          return;
        }
        templateSelect.click();
        templateSelect.all(by.css('option')).get(2).click();
        expect(includeElem.getText()).toMatch(/Content of template2.html/);
      });

      it('should change to blank', function() {
        if (browser.params.browser === 'firefox') {
          // Firefox can't handle using selects
          return;
        }
        templateSelect.click();
        templateSelect.all(by.css('option')).get(0).click();
        expect(includeElem.isPresent()).toBe(false);
      });
    </file>
  </example>
 */


/**
 * @ngdoc event
 * @name ngInclude#$includeContentRequested
 * @eventType emit on the scope ngInclude was declared in
 * @description
 * Emitted every time the ngInclude content is requested.
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */


/**
 * @ngdoc event
 * @name ngInclude#$includeContentLoaded
 * @eventType emit on the current ngInclude scope
 * @description
 * Emitted every time the ngInclude content is reloaded.
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */


/**
 * @ngdoc event
 * @name ngInclude#$includeContentError
 * @eventType emit on the scope ngInclude was declared in
 * @description
 * Emitted when a template HTTP request yields an erroneous response (status < 200 || status > 299)
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */
var ngIncludeDirective = ['$templateRequest', '$anchorScroll', '$animate',
                  function($templateRequest,   $anchorScroll,   $animate) {
  return {
    restrict: 'ECA',
    priority: 400,
    terminal: true,
    transclude: 'element',
    controller: angular.noop,
    compile: function(element, attr) {
      var srcExp = attr.ngInclude || attr.src,
          onloadExp = attr.onload || '',
          autoScrollExp = attr.autoscroll;

      return function(scope, $element, $attr, ctrl, $transclude) {
        var changeCounter = 0,
            currentScope,
            previousElement,
            currentElement;

        var cleanupLastIncludeContent = function() {
          if (previousElement) {
            previousElement.remove();
            previousElement = null;
          }
          if (currentScope) {
            currentScope.$destroy();
            currentScope = null;
          }
          if (currentElement) {
            $animate.leave(currentElement).done(function(response) {
              if (response !== false) previousElement = null;
            });
            previousElement = currentElement;
            currentElement = null;
          }
        };

        scope.$watch(srcExp, function ngIncludeWatchAction(src) {
          var afterAnimation = function(response) {
            if (response !== false && isDefined(autoScrollExp) &&
              (!autoScrollExp || scope.$eval(autoScrollExp))) {
                $anchorScroll();
            }
          };
          var thisChangeId = ++changeCounter;

          if (src) {
            //set the 2nd param to true to ignore the template request error so that the inner
            //contents and scope can be cleaned up.
            $templateRequest(src, true).then(function(response) {
              if (scope.$$destroyed) return;

              if (thisChangeId !== changeCounter) return;
              var newScope = scope.$new();
              ctrl.template = response;

              // Note: This will also link all children of ng-include that were contained in the original
              // html. If that content contains controllers, ... they could pollute/change the scope.
              // However, using ng-include on an element with additional content does not make sense...
              // Note: We can't remove them in the cloneAttchFn of $transclude as that
              // function is called before linking the content, which would apply child
              // directives to non existing elements.
              var clone = $transclude(newScope, function(clone) {
                cleanupLastIncludeContent();
                $animate.enter(clone, null, $element).done(afterAnimation);
              });

              currentScope = newScope;
              currentElement = clone;

              currentScope.$emit('$includeContentLoaded', src);
              scope.$eval(onloadExp);
            }, function() {
              if (scope.$$destroyed) return;

              if (thisChangeId === changeCounter) {
                cleanupLastIncludeContent();
                scope.$emit('$includeContentError', src);
              }
            });
            scope.$emit('$includeContentRequested', src);
          } else {
            cleanupLastIncludeContent();
            ctrl.template = null;
          }
        });
      };
    }
  };
}];

// This directive is called during the $transclude call of the first `ngInclude` directive.
// It will replace and compile the content of the element with the loaded template.
// We need this directive so that the element content is already filled when
// the link function of another directive on the same element as ngInclude
// is called.
var ngIncludeFillContentDirective = ['$compile',
  function($compile) {
    return {
      restrict: 'ECA',
      priority: -400,
      require: 'ngInclude',
      link: function(scope, $element, $attr, ctrl) {
        if (toString.call($element[0]).match(/SVG/)) {
          // WebKit: https://bugs.webkit.org/show_bug.cgi?id=135698 --- SVG elements do not
          // support innerHTML, so detect this here and try to generate the contents
          // specially.
          $element.empty();
          $compile(jqLiteBuildFragment(ctrl.template, window.document).childNodes)(scope,
              function namespaceAdaptedClone(clone) {
            $element.append(clone);
          }, {futureParentElement: $element});
          return;
        }

        $element.html(ctrl.template);
        $compile($element.contents())(scope);
      }
    };
  }];

/**
 * @ngdoc directive
 * @name ngInit
 * @restrict AC
 * @priority 450
 * @element ANY
 *
 * @param {expression} ngInit {@link guide/expression Expression} to eval.
 *
 * @description
 * The `ngInit` directive allows you to evaluate an expression in the
 * current scope.
 *
 * <div class="alert alert-danger">
 * This directive can be abused to add unnecessary amounts of logic into your templates.
 * There are only a few appropriate uses of `ngInit`:
 * <ul>
 *   <li>aliasing special properties of {@link ng.directive:ngRepeat `ngRepeat`},
 *     as seen in the demo below.</li>
 *   <li>initializing data during development, or for examples, as seen throughout these docs.</li>
 *   <li>injecting data via server side scripting.</li>
 * </ul>
 *
 * Besides these few cases, you should use {@link guide/component Components} or
 * {@link guide/controller Controllers} rather than `ngInit` to initialize values on a scope.
 * </div>
 *
 * <div class="alert alert-warning">
 * **Note**: If you have assignment in `ngInit` along with a {@link ng.$filter `filter`}, make
 * sure you have parentheses to ensure correct operator precedence:
 * <pre class="prettyprint">
 * `<div ng-init="test1 = ($index | toString)"></div>`
 * </pre>
 * </div>
 *
 * @example
   <example module="initExample" name="ng-init">
     <file name="index.html">
   <script>
     angular.module('initExample', [])
       .controller('ExampleController', ['$scope', function($scope) {
         $scope.list = [['a', 'b'], ['c', 'd']];
       }]);
   </script>
   <div ng-controller="ExampleController">
     <div ng-repeat="innerList in list" ng-init="outerIndex = $index">
       <div ng-repeat="value in innerList" ng-init="innerIndex = $index">
          <span class="example-init">list[ {{outerIndex}} ][ {{innerIndex}} ] = {{value}};</span>
       </div>
     </div>
   </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should alias index positions', function() {
         var elements = element.all(by.css('.example-init'));
         expect(elements.get(0).getText()).toBe('list[ 0 ][ 0 ] = a;');
         expect(elements.get(1).getText()).toBe('list[ 0 ][ 1 ] = b;');
         expect(elements.get(2).getText()).toBe('list[ 1 ][ 0 ] = c;');
         expect(elements.get(3).getText()).toBe('list[ 1 ][ 1 ] = d;');
       });
     </file>
   </example>
 */
var ngInitDirective = ngDirective({
  priority: 450,
  compile: function() {
    return {
      pre: function(scope, element, attrs) {
        scope.$eval(attrs.ngInit);
      }
    };
  }
});

/**
 * @ngdoc directive
 * @name ngList
 * @restrict A
 * @priority 100
 *
 * @param {string=} ngList optional delimiter that should be used to split the value.
 *
 * @description
 * Text input that converts between a delimited string and an array of strings. The default
 * delimiter is a comma followed by a space - equivalent to `ng-list=", "`. You can specify a custom
 * delimiter as the value of the `ngList` attribute - for example, `ng-list=" | "`.
 *
 * The behaviour of the directive is affected by the use of the `ngTrim` attribute.
 * * If `ngTrim` is set to `"false"` then whitespace around both the separator and each
 *   list item is respected. This implies that the user of the directive is responsible for
 *   dealing with whitespace but also allows you to use whitespace as a delimiter, such as a
 *   tab or newline character.
 * * Otherwise whitespace around the delimiter is ignored when splitting (although it is respected
 *   when joining the list items back together) and whitespace around each list item is stripped
 *   before it is added to the model.
 *
 * @example
 * ### Validation
 *
 * <example name="ngList-directive" module="listExample">
 *   <file name="app.js">
 *      angular.module('listExample', [])
 *        .controller('ExampleController', ['$scope', function($scope) {
 *          $scope.names = ['morpheus', 'neo', 'trinity'];
 *        }]);
 *   </file>
 *   <file name="index.html">
 *    <form name="myForm" ng-controller="ExampleController">
 *      <label>List: <input name="namesInput" ng-model="names" ng-list required></label>
 *      <span role="alert">
 *        <span class="error" ng-show="myForm.namesInput.$error.required">
 *        Required!</span>
 *      </span>
 *      <br>
 *      <tt>names = {{names}}</tt><br/>
 *      <tt>myForm.namesInput.$valid = {{myForm.namesInput.$valid}}</tt><br/>
 *      <tt>myForm.namesInput.$error = {{myForm.namesInput.$error}}</tt><br/>
 *      <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
 *      <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
 *     </form>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     var listInput = element(by.model('names'));
 *     var names = element(by.exactBinding('names'));
 *     var valid = element(by.binding('myForm.namesInput.$valid'));
 *     var error = element(by.css('span.error'));
 *
 *     it('should initialize to model', function() {
 *       expect(names.getText()).toContain('["morpheus","neo","trinity"]');
 *       expect(valid.getText()).toContain('true');
 *       expect(error.getCssValue('display')).toBe('none');
 *     });
 *
 *     it('should be invalid if empty', function() {
 *       listInput.clear();
 *       listInput.sendKeys('');
 *
 *       expect(names.getText()).toContain('');
 *       expect(valid.getText()).toContain('false');
 *       expect(error.getCssValue('display')).not.toBe('none');
 *     });
 *   </file>
 * </example>
 *
 * @example
 * ### Splitting on newline
 *
 * <example name="ngList-directive-newlines">
 *   <file name="index.html">
 *    <textarea ng-model="list" ng-list="&#10;" ng-trim="false"></textarea>
 *    <pre>{{ list | json }}</pre>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     it("should split the text by newlines", function() {
 *       var listInput = element(by.model('list'));
 *       var output = element(by.binding('list | json'));
 *       listInput.sendKeys('abc\ndef\nghi');
 *       expect(output.getText()).toContain('[\n  "abc",\n  "def",\n  "ghi"\n]');
 *     });
 *   </file>
 * </example>
 *
 */
var ngListDirective = function() {
  return {
    restrict: 'A',
    priority: 100,
    require: 'ngModel',
    link: function(scope, element, attr, ctrl) {
      var ngList = attr.ngList || ', ';
      var trimValues = attr.ngTrim !== 'false';
      var separator = trimValues ? trim(ngList) : ngList;

      var parse = function(viewValue) {
        // If the viewValue is invalid (say required but empty) it will be `undefined`
        if (isUndefined(viewValue)) return;

        var list = [];

        if (viewValue) {
          forEach(viewValue.split(separator), function(value) {
            if (value) list.push(trimValues ? trim(value) : value);
          });
        }

        return list;
      };

      ctrl.$parsers.push(parse);
      ctrl.$formatters.push(function(value) {
        if (isArray(value)) {
          return value.join(ngList);
        }

        return undefined;
      });

      // Override the standard $isEmpty because an empty array means the input is empty.
      ctrl.$isEmpty = function(value) {
        return !value || !value.length;
      };
    }
  };
};

/* global VALID_CLASS: true,
  INVALID_CLASS: true,
  PRISTINE_CLASS: true,
  DIRTY_CLASS: true,
  UNTOUCHED_CLASS: true,
  TOUCHED_CLASS: true,
  PENDING_CLASS: true,
  addSetValidityMethod: true,
  setupValidity: true,
  defaultModelOptions: false
*/


var VALID_CLASS = 'ng-valid',
    INVALID_CLASS = 'ng-invalid',
    PRISTINE_CLASS = 'ng-pristine',
    DIRTY_CLASS = 'ng-dirty',
    UNTOUCHED_CLASS = 'ng-untouched',
    TOUCHED_CLASS = 'ng-touched',
    EMPTY_CLASS = 'ng-empty',
    NOT_EMPTY_CLASS = 'ng-not-empty';

var ngModelMinErr = minErr('ngModel');

/**
 * @ngdoc type
 * @name ngModel.NgModelController
 * @property {*} $viewValue The actual value from the control's view. For `input` elements, this is a
 * String. See {@link ngModel.NgModelController#$setViewValue} for information about when the $viewValue
 * is set.
 *
 * @property {*} $modelValue The value in the model that the control is bound to.
 *
 * @property {Array.<Function>} $parsers Array of functions to execute, as a pipeline, whenever
 *  the control updates the ngModelController with a new {@link ngModel.NgModelController#$viewValue
    `$viewValue`} from the DOM, usually via user input.
    See {@link ngModel.NgModelController#$setViewValue `$setViewValue()`} for a detailed lifecycle explanation.
    Note that the `$parsers` are not called when the bound ngModel expression changes programmatically.

  The functions are called in array order, each passing
    its return value through to the next. The last return value is forwarded to the
    {@link ngModel.NgModelController#$validators `$validators`} collection.

  Parsers are used to sanitize / convert the {@link ngModel.NgModelController#$viewValue
    `$viewValue`}.

  Returning `undefined` from a parser means a parse error occurred. In that case,
    no {@link ngModel.NgModelController#$validators `$validators`} will run and the `ngModel`
    will be set to `undefined` unless {@link ngModelOptions `ngModelOptions.allowInvalid`}
    is set to `true`. The parse error is stored in `ngModel.$error.parse`.

  This simple example shows a parser that would convert text input value to lowercase:
 * ```js
 * function parse(value) {
 *   if (value) {
 *     return value.toLowerCase();
 *   }
 * }
 * ngModelController.$parsers.push(parse);
 * ```

 *
 * @property {Array.<Function>} $formatters Array of functions to execute, as a pipeline, whenever
    the bound ngModel expression changes programmatically. The `$formatters` are not called when the
    value of the control is changed by user interaction.

  Formatters are used to format / convert the {@link ngModel.NgModelController#$modelValue
    `$modelValue`} for display in the control.

  The functions are called in reverse array order, each passing the value through to the
    next. The last return value is used as the actual DOM value.

  This simple example shows a formatter that would convert the model value to uppercase:

 * ```js
 * function format(value) {
 *   if (value) {
 *     return value.toUpperCase();
 *   }
 * }
 * ngModel.$formatters.push(format);
 * ```
 *
 * @property {Object.<string, function>} $validators A collection of validators that are applied
 *      whenever the model value changes. The key value within the object refers to the name of the
 *      validator while the function refers to the validation operation. The validation operation is
 *      provided with the model value as an argument and must return a true or false value depending
 *      on the response of that validation.
 *
 * ```js
 * ngModel.$validators.validCharacters = function(modelValue, viewValue) {
 *   var value = modelValue || viewValue;
 *   return /[0-9]+/.test(value) &&
 *          /[a-z]+/.test(value) &&
 *          /[A-Z]+/.test(value) &&
 *          /\W+/.test(value);
 * };
 * ```
 *
 * @property {Object.<string, function>} $asyncValidators A collection of validations that are expected to
 *      perform an asynchronous validation (e.g. a HTTP request). The validation function that is provided
 *      is expected to return a promise when it is run during the model validation process. Once the promise
 *      is delivered then the validation status will be set to true when fulfilled and false when rejected.
 *      When the asynchronous validators are triggered, each of the validators will run in parallel and the model
 *      value will only be updated once all validators have been fulfilled. As long as an asynchronous validator
 *      is unfulfilled, its key will be added to the controllers `$pending` property. Also, all asynchronous validators
 *      will only run once all synchronous validators have passed.
 *
 * Please note that if $http is used then it is important that the server returns a success HTTP response code
 * in order to fulfill the validation and a status level of `4xx` in order to reject the validation.
 *
 * ```js
 * ngModel.$asyncValidators.uniqueUsername = function(modelValue, viewValue) {
 *   var value = modelValue || viewValue;
 *
 *   // Lookup user by username
 *   return $http.get('/api/users/' + value).
 *      then(function resolved() {
 *        //username exists, this means validation fails
 *        return $q.reject('exists');
 *      }, function rejected() {
 *        //username does not exist, therefore this validation passes
 *        return true;
 *      });
 * };
 * ```
 *
 * @property {Array.<Function>} $viewChangeListeners Array of functions to execute whenever
 *     a change to {@link ngModel.NgModelController#$viewValue `$viewValue`} has caused a change
 *     to {@link ngModel.NgModelController#$modelValue `$modelValue`}.
 *     It is called with no arguments, and its return value is ignored.
 *     This can be used in place of additional $watches against the model value.
 *
 * @property {Object} $error An object hash with all failing validator ids as keys.
 * @property {Object} $pending An object hash with all pending validator ids as keys.
 *
 * @property {boolean} $untouched True if control has not lost focus yet.
 * @property {boolean} $touched True if control has lost focus.
 * @property {boolean} $pristine True if user has not interacted with the control yet.
 * @property {boolean} $dirty True if user has already interacted with the control.
 * @property {boolean} $valid True if there is no error.
 * @property {boolean} $invalid True if at least one error on the control.
 * @property {string} $name The name attribute of the control.
 *
 * @description
 *
 * `NgModelController` provides API for the {@link ngModel `ngModel`} directive.
 * The controller contains services for data-binding, validation, CSS updates, and value formatting
 * and parsing. It purposefully does not contain any logic which deals with DOM rendering or
 * listening to DOM events.
 * Such DOM related logic should be provided by other directives which make use of
 * `NgModelController` for data-binding to control elements.
 * AngularJS provides this DOM logic for most {@link input `input`} elements.
 * At the end of this page you can find a {@link ngModel.NgModelController#custom-control-example
 * custom control example} that uses `ngModelController` to bind to `contenteditable` elements.
 *
 * @example
 * ### Custom Control Example
 * This example shows how to use `NgModelController` with a custom control to achieve
 * data-binding. Notice how different directives (`contenteditable`, `ng-model`, and `required`)
 * collaborate together to achieve the desired result.
 *
 * `contenteditable` is an HTML5 attribute, which tells the browser to let the element
 * contents be edited in place by the user.
 *
 * We are using the {@link ng.service:$sce $sce} service here and include the {@link ngSanitize $sanitize}
 * module to automatically remove "bad" content like inline event listener (e.g. `<span onclick="...">`).
 * However, as we are using `$sce` the model can still decide to provide unsafe content if it marks
 * that content using the `$sce` service.
 *
 * <example name="NgModelController" module="customControl" deps="angular-sanitize.js">
    <file name="style.css">
      [contenteditable] {
        border: 1px solid black;
        background-color: white;
        min-height: 20px;
      }

      .ng-invalid {
        border: 1px solid red;
      }

    </file>
    <file name="script.js">
      angular.module('customControl', ['ngSanitize']).
        directive('contenteditable', ['$sce', function($sce) {
          return {
            restrict: 'A', // only activate on element attribute
            require: '?ngModel', // get a hold of NgModelController
            link: function(scope, element, attrs, ngModel) {
              if (!ngModel) return; // do nothing if no ng-model

              // Specify how UI should be updated
              ngModel.$render = function() {
                element.html($sce.getTrustedHtml(ngModel.$viewValue || ''));
              };

              // Listen for change events to enable binding
              element.on('blur keyup change', function() {
                scope.$evalAsync(read);
              });
              read(); // initialize

              // Write data to the model
              function read() {
                var html = element.html();
                // When we clear the content editable the browser leaves a <br> behind
                // If strip-br attribute is provided then we strip this out
                if (attrs.stripBr && html === '<br>') {
                  html = '';
                }
                ngModel.$setViewValue(html);
              }
            }
          };
        }]);
    </file>
    <file name="index.html">
      <form name="myForm">
       <div contenteditable
            name="myWidget" ng-model="userContent"
            strip-br="true"
            required>Change me!</div>
        <span ng-show="myForm.myWidget.$error.required">Required!</span>
       <hr>
       <textarea ng-model="userContent" aria-label="Dynamic textarea"></textarea>
      </form>
    </file>
    <file name="protractor.js" type="protractor">
    it('should data-bind and become invalid', function() {
      if (browser.params.browser === 'safari' || browser.params.browser === 'firefox') {
        // SafariDriver can't handle contenteditable
        // and Firefox driver can't clear contenteditables very well
        return;
      }
      var contentEditable = element(by.css('[contenteditable]'));
      var content = 'Change me!';

      expect(contentEditable.getText()).toEqual(content);

      contentEditable.clear();
      contentEditable.sendKeys(protractor.Key.BACK_SPACE);
      expect(contentEditable.getText()).toEqual('');
      expect(contentEditable.getAttribute('class')).toMatch(/ng-invalid-required/);
    });
    </file>
 * </example>
 *
 *
 */
NgModelController.$inject = ['$scope', '$exceptionHandler', '$attrs', '$element', '$parse', '$animate', '$timeout', '$q', '$interpolate'];
function NgModelController($scope, $exceptionHandler, $attr, $element, $parse, $animate, $timeout, $q, $interpolate) {
  this.$viewValue = Number.NaN;
  this.$modelValue = Number.NaN;
  this.$$rawModelValue = undefined; // stores the parsed modelValue / model set from scope regardless of validity.
  this.$validators = {};
  this.$asyncValidators = {};
  this.$parsers = [];
  this.$formatters = [];
  this.$viewChangeListeners = [];
  this.$untouched = true;
  this.$touched = false;
  this.$pristine = true;
  this.$dirty = false;
  this.$valid = true;
  this.$invalid = false;
  this.$error = {}; // keep invalid keys here
  this.$$success = {}; // keep valid keys here
  this.$pending = undefined; // keep pending keys here
  this.$name = $interpolate($attr.name || '', false)($scope);
  this.$$parentForm = nullFormCtrl;
  this.$options = defaultModelOptions;
  this.$$updateEvents = '';
  // Attach the correct context to the event handler function for updateOn
  this.$$updateEventHandler = this.$$updateEventHandler.bind(this);

  this.$$parsedNgModel = $parse($attr.ngModel);
  this.$$parsedNgModelAssign = this.$$parsedNgModel.assign;
  this.$$ngModelGet = this.$$parsedNgModel;
  this.$$ngModelSet = this.$$parsedNgModelAssign;
  this.$$pendingDebounce = null;
  this.$$parserValid = undefined;
  this.$$parserName = 'parse';

  this.$$currentValidationRunId = 0;

  this.$$scope = $scope;
  this.$$rootScope = $scope.$root;
  this.$$attr = $attr;
  this.$$element = $element;
  this.$$animate = $animate;
  this.$$timeout = $timeout;
  this.$$parse = $parse;
  this.$$q = $q;
  this.$$exceptionHandler = $exceptionHandler;

  setupValidity(this);
  setupModelWatcher(this);
}

NgModelController.prototype = {
  $$initGetterSetters: function() {
    if (this.$options.getOption('getterSetter')) {
      var invokeModelGetter = this.$$parse(this.$$attr.ngModel + '()'),
          invokeModelSetter = this.$$parse(this.$$attr.ngModel + '($$$p)');

      this.$$ngModelGet = function($scope) {
        var modelValue = this.$$parsedNgModel($scope);
        if (isFunction(modelValue)) {
          modelValue = invokeModelGetter($scope);
        }
        return modelValue;
      };
      this.$$ngModelSet = function($scope, newValue) {
        if (isFunction(this.$$parsedNgModel($scope))) {
          invokeModelSetter($scope, {$$$p: newValue});
        } else {
          this.$$parsedNgModelAssign($scope, newValue);
        }
      };
    } else if (!this.$$parsedNgModel.assign) {
      throw ngModelMinErr('nonassign', 'Expression \'{0}\' is non-assignable. Element: {1}',
          this.$$attr.ngModel, startingTag(this.$$element));
    }
  },


  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$render
   *
   * @description
   * Called when the view needs to be updated. It is expected that the user of the ng-model
   * directive will implement this method.
   *
   * The `$render()` method is invoked in the following situations:
   *
   * * `$rollbackViewValue()` is called.  If we are rolling back the view value to the last
   *   committed value then `$render()` is called to update the input control.
   * * The value referenced by `ng-model` is changed programmatically and both the `$modelValue` and
   *   the `$viewValue` are different from last time.
   *
   * Since `ng-model` does not do a deep watch, `$render()` is only invoked if the values of
   * `$modelValue` and `$viewValue` are actually different from their previous values. If `$modelValue`
   * or `$viewValue` are objects (rather than a string or number) then `$render()` will not be
   * invoked if you only change a property on the objects.
   */
  $render: noop,

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$isEmpty
   *
   * @description
   * This is called when we need to determine if the value of an input is empty.
   *
   * For instance, the required directive does this to work out if the input has data or not.
   *
   * The default `$isEmpty` function checks whether the value is `undefined`, `''`, `null` or `NaN`.
   *
   * You can override this for input directives whose concept of being empty is different from the
   * default. The `checkboxInputType` directive does this because in its case a value of `false`
   * implies empty.
   *
   * @param {*} value The value of the input to check for emptiness.
   * @returns {boolean} True if `value` is "empty".
   */
  $isEmpty: function(value) {
    // eslint-disable-next-line no-self-compare
    return isUndefined(value) || value === '' || value === null || value !== value;
  },

  $$updateEmptyClasses: function(value) {
    if (this.$isEmpty(value)) {
      this.$$animate.removeClass(this.$$element, NOT_EMPTY_CLASS);
      this.$$animate.addClass(this.$$element, EMPTY_CLASS);
    } else {
      this.$$animate.removeClass(this.$$element, EMPTY_CLASS);
      this.$$animate.addClass(this.$$element, NOT_EMPTY_CLASS);
    }
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setPristine
   *
   * @description
   * Sets the control to its pristine state.
   *
   * This method can be called to remove the `ng-dirty` class and set the control to its pristine
   * state (`ng-pristine` class). A model is considered to be pristine when the control
   * has not been changed from when first compiled.
   */
  $setPristine: function() {
    this.$dirty = false;
    this.$pristine = true;
    this.$$animate.removeClass(this.$$element, DIRTY_CLASS);
    this.$$animate.addClass(this.$$element, PRISTINE_CLASS);
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setDirty
   *
   * @description
   * Sets the control to its dirty state.
   *
   * This method can be called to remove the `ng-pristine` class and set the control to its dirty
   * state (`ng-dirty` class). A model is considered to be dirty when the control has been changed
   * from when first compiled.
   */
  $setDirty: function() {
    this.$dirty = true;
    this.$pristine = false;
    this.$$animate.removeClass(this.$$element, PRISTINE_CLASS);
    this.$$animate.addClass(this.$$element, DIRTY_CLASS);
    this.$$parentForm.$setDirty();
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setUntouched
   *
   * @description
   * Sets the control to its untouched state.
   *
   * This method can be called to remove the `ng-touched` class and set the control to its
   * untouched state (`ng-untouched` class). Upon compilation, a model is set as untouched
   * by default, however this function can be used to restore that state if the model has
   * already been touched by the user.
   */
  $setUntouched: function() {
    this.$touched = false;
    this.$untouched = true;
    this.$$animate.setClass(this.$$element, UNTOUCHED_CLASS, TOUCHED_CLASS);
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setTouched
   *
   * @description
   * Sets the control to its touched state.
   *
   * This method can be called to remove the `ng-untouched` class and set the control to its
   * touched state (`ng-touched` class). A model is considered to be touched when the user has
   * first focused the control element and then shifted focus away from the control (blur event).
   */
  $setTouched: function() {
    this.$touched = true;
    this.$untouched = false;
    this.$$animate.setClass(this.$$element, TOUCHED_CLASS, UNTOUCHED_CLASS);
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$rollbackViewValue
   *
   * @description
   * Cancel an update and reset the input element's value to prevent an update to the `$modelValue`,
   * which may be caused by a pending debounced event or because the input is waiting for some
   * future event.
   *
   * If you have an input that uses `ng-model-options` to set up debounced updates or updates that
   * depend on special events such as `blur`, there can be a period when the `$viewValue` is out of
   * sync with the ngModel's `$modelValue`.
   *
   * In this case, you can use `$rollbackViewValue()` to manually cancel the debounced / future update
   * and reset the input to the last committed view value.
   *
   * It is also possible that you run into difficulties if you try to update the ngModel's `$modelValue`
   * programmatically before these debounced/future events have resolved/occurred, because AngularJS's
   * dirty checking mechanism is not able to tell whether the model has actually changed or not.
   *
   * The `$rollbackViewValue()` method should be called before programmatically changing the model of an
   * input which may have such events pending. This is important in order to make sure that the
   * input field will be updated with the new model value and any pending operations are cancelled.
   *
   * @example
   * <example name="ng-model-cancel-update" module="cancel-update-example">
   *   <file name="app.js">
   *     angular.module('cancel-update-example', [])
   *
   *     .controller('CancelUpdateController', ['$scope', function($scope) {
   *       $scope.model = {value1: '', value2: ''};
   *
   *       $scope.setEmpty = function(e, value, rollback) {
   *         if (e.keyCode === 27) {
   *           e.preventDefault();
   *           if (rollback) {
   *             $scope.myForm[value].$rollbackViewValue();
   *           }
   *           $scope.model[value] = '';
   *         }
   *       };
   *     }]);
   *   </file>
   *   <file name="index.html">
   *     <div ng-controller="CancelUpdateController">
   *       <p>Both of these inputs are only updated if they are blurred. Hitting escape should
   *       empty them. Follow these steps and observe the difference:</p>
   *       <ol>
   *         <li>Type something in the input. You will see that the model is not yet updated</li>
   *         <li>Press the Escape key.
   *           <ol>
   *             <li> In the first example, nothing happens, because the model is already '', and no
   *             update is detected. If you blur the input, the model will be set to the current view.
   *             </li>
   *             <li> In the second example, the pending update is cancelled, and the input is set back
   *             to the last committed view value (''). Blurring the input does nothing.
   *             </li>
   *           </ol>
   *         </li>
   *       </ol>
   *
   *       <form name="myForm" ng-model-options="{ updateOn: 'blur' }">
   *         <div>
   *           <p id="inputDescription1">Without $rollbackViewValue():</p>
   *           <input name="value1" aria-describedby="inputDescription1" ng-model="model.value1"
   *                  ng-keydown="setEmpty($event, 'value1')">
   *           value1: "{{ model.value1 }}"
   *         </div>
   *
   *         <div>
   *           <p id="inputDescription2">With $rollbackViewValue():</p>
   *           <input name="value2" aria-describedby="inputDescription2" ng-model="model.value2"
   *                  ng-keydown="setEmpty($event, 'value2', true)">
   *           value2: "{{ model.value2 }}"
   *         </div>
   *       </form>
   *     </div>
   *   </file>
       <file name="style.css">
          div {
            display: table-cell;
          }
          div:nth-child(1) {
            padding-right: 30px;
          }

        </file>
   * </example>
   */
  $rollbackViewValue: function() {
    this.$$timeout.cancel(this.$$pendingDebounce);
    this.$viewValue = this.$$lastCommittedViewValue;
    this.$render();
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$validate
   *
   * @description
   * Runs each of the registered validators (first synchronous validators and then
   * asynchronous validators).
   * If the validity changes to invalid, the model will be set to `undefined`,
   * unless {@link ngModelOptions `ngModelOptions.allowInvalid`} is `true`.
   * If the validity changes to valid, it will set the model to the last available valid
   * `$modelValue`, i.e. either the last parsed value or the last value set from the scope.
   */
  $validate: function() {

    // ignore $validate before model is initialized
    if (isNumberNaN(this.$modelValue)) {
      return;
    }

    var viewValue = this.$$lastCommittedViewValue;
    // Note: we use the $$rawModelValue as $modelValue might have been
    // set to undefined during a view -> model update that found validation
    // errors. We can't parse the view here, since that could change
    // the model although neither viewValue nor the model on the scope changed
    var modelValue = this.$$rawModelValue;

    var prevValid = this.$valid;
    var prevModelValue = this.$modelValue;

    var allowInvalid = this.$options.getOption('allowInvalid');

    var that = this;
    this.$$runValidators(modelValue, viewValue, function(allValid) {
      // If there was no change in validity, don't update the model
      // This prevents changing an invalid modelValue to undefined
      if (!allowInvalid && prevValid !== allValid) {
        // Note: Don't check this.$valid here, as we could have
        // external validators (e.g. calculated on the server),
        // that just call $setValidity and need the model value
        // to calculate their validity.
        that.$modelValue = allValid ? modelValue : undefined;

        if (that.$modelValue !== prevModelValue) {
          that.$$writeModelToScope();
        }
      }
    });
  },

  $$runValidators: function(modelValue, viewValue, doneCallback) {
    this.$$currentValidationRunId++;
    var localValidationRunId = this.$$currentValidationRunId;
    var that = this;

    // check parser error
    if (!processParseErrors()) {
      validationDone(false);
      return;
    }
    if (!processSyncValidators()) {
      validationDone(false);
      return;
    }
    processAsyncValidators();

    function processParseErrors() {
      var errorKey = that.$$parserName;

      if (isUndefined(that.$$parserValid)) {
        setValidity(errorKey, null);
      } else {
        if (!that.$$parserValid) {
          forEach(that.$validators, function(v, name) {
            setValidity(name, null);
          });
          forEach(that.$asyncValidators, function(v, name) {
            setValidity(name, null);
          });
        }

        // Set the parse error last, to prevent unsetting it, should a $validators key == parserName
        setValidity(errorKey, that.$$parserValid);
        return that.$$parserValid;
      }
      return true;
    }

    function processSyncValidators() {
      var syncValidatorsValid = true;
      forEach(that.$validators, function(validator, name) {
        var result = Boolean(validator(modelValue, viewValue));
        syncValidatorsValid = syncValidatorsValid && result;
        setValidity(name, result);
      });
      if (!syncValidatorsValid) {
        forEach(that.$asyncValidators, function(v, name) {
          setValidity(name, null);
        });
        return false;
      }
      return true;
    }

    function processAsyncValidators() {
      var validatorPromises = [];
      var allValid = true;
      forEach(that.$asyncValidators, function(validator, name) {
        var promise = validator(modelValue, viewValue);
        if (!isPromiseLike(promise)) {
          throw ngModelMinErr('nopromise',
            'Expected asynchronous validator to return a promise but got \'{0}\' instead.', promise);
        }
        setValidity(name, undefined);
        validatorPromises.push(promise.then(function() {
          setValidity(name, true);
        }, function() {
          allValid = false;
          setValidity(name, false);
        }));
      });
      if (!validatorPromises.length) {
        validationDone(true);
      } else {
        that.$$q.all(validatorPromises).then(function() {
          validationDone(allValid);
        }, noop);
      }
    }

    function setValidity(name, isValid) {
      if (localValidationRunId === that.$$currentValidationRunId) {
        that.$setValidity(name, isValid);
      }
    }

    function validationDone(allValid) {
      if (localValidationRunId === that.$$currentValidationRunId) {

        doneCallback(allValid);
      }
    }
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$commitViewValue
   *
   * @description
   * Commit a pending update to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. this method is rarely needed as `NgModelController`
   * usually handles calling this in response to input events.
   */
  $commitViewValue: function() {
    var viewValue = this.$viewValue;

    this.$$timeout.cancel(this.$$pendingDebounce);

    // If the view value has not changed then we should just exit, except in the case where there is
    // a native validator on the element. In this case the validation state may have changed even though
    // the viewValue has stayed empty.
    if (this.$$lastCommittedViewValue === viewValue && (viewValue !== '' || !this.$$hasNativeValidators)) {
      return;
    }
    this.$$updateEmptyClasses(viewValue);
    this.$$lastCommittedViewValue = viewValue;

    // change to dirty
    if (this.$pristine) {
      this.$setDirty();
    }
    this.$$parseAndValidate();
  },

  $$parseAndValidate: function() {
    var viewValue = this.$$lastCommittedViewValue;
    var modelValue = viewValue;
    var that = this;

    this.$$parserValid = isUndefined(modelValue) ? undefined : true;

    // Reset any previous parse error
    this.$setValidity(this.$$parserName, null);
    this.$$parserName = 'parse';

    if (this.$$parserValid) {
      for (var i = 0; i < this.$parsers.length; i++) {
        modelValue = this.$parsers[i](modelValue);
        if (isUndefined(modelValue)) {
          this.$$parserValid = false;
          break;
        }
      }
    }
    if (isNumberNaN(this.$modelValue)) {
      // this.$modelValue has not been touched yet...
      this.$modelValue = this.$$ngModelGet(this.$$scope);
    }
    var prevModelValue = this.$modelValue;
    var allowInvalid = this.$options.getOption('allowInvalid');
    this.$$rawModelValue = modelValue;

    if (allowInvalid) {
      this.$modelValue = modelValue;
      writeToModelIfNeeded();
    }

    // Pass the $$lastCommittedViewValue here, because the cached viewValue might be out of date.
    // This can happen if e.g. $setViewValue is called from inside a parser
    this.$$runValidators(modelValue, this.$$lastCommittedViewValue, function(allValid) {
      if (!allowInvalid) {
        // Note: Don't check this.$valid here, as we could have
        // external validators (e.g. calculated on the server),
        // that just call $setValidity and need the model value
        // to calculate their validity.
        that.$modelValue = allValid ? modelValue : undefined;
        writeToModelIfNeeded();
      }
    });

    function writeToModelIfNeeded() {
      if (that.$modelValue !== prevModelValue) {
        that.$$writeModelToScope();
      }
    }
  },

  $$writeModelToScope: function() {
    this.$$ngModelSet(this.$$scope, this.$modelValue);
    forEach(this.$viewChangeListeners, function(listener) {
      try {
        listener();
      } catch (e) {
        // eslint-disable-next-line no-invalid-this
        this.$$exceptionHandler(e);
      }
    }, this);
  },

  /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setViewValue
   *
   * @description
   * Update the view value.
   *
   * This method should be called when a control wants to change the view value; typically,
   * this is done from within a DOM event handler. For example, the {@link ng.directive:input input}
   * directive calls it when the value of the input changes and {@link ng.directive:select select}
   * calls it when an option is selected.
   *
   * When `$setViewValue` is called, the new `value` will be staged for committing through the `$parsers`
   * and `$validators` pipelines. If there are no special {@link ngModelOptions} specified then the staged
   * value is sent directly for processing through the `$parsers` pipeline. After this, the `$validators` and
   * `$asyncValidators` are called and the value is applied to `$modelValue`.
   * Finally, the value is set to the **expression** specified in the `ng-model` attribute and
   * all the registered change listeners, in the `$viewChangeListeners` list are called.
   *
   * In case the {@link ng.directive:ngModelOptions ngModelOptions} directive is used with `updateOn`
   * and the `default` trigger is not listed, all those actions will remain pending until one of the
   * `updateOn` events is triggered on the DOM element.
   * All these actions will be debounced if the {@link ng.directive:ngModelOptions ngModelOptions}
   * directive is used with a custom debounce for this particular event.
   * Note that a `$digest` is only triggered once the `updateOn` events are fired, or if `debounce`
   * is specified, once the timer runs out.
   *
   * When used with standard inputs, the view value will always be a string (which is in some cases
   * parsed into another type, such as a `Date` object for `input[date]`.)
   * However, custom controls might also pass objects to this method. In this case, we should make
   * a copy of the object before passing it to `$setViewValue`. This is because `ngModel` does not
   * perform a deep watch of objects, it only looks for a change of identity. If you only change
   * the property of the object then ngModel will not realize that the object has changed and
   * will not invoke the `$parsers` and `$validators` pipelines. For this reason, you should
   * not change properties of the copy once it has been passed to `$setViewValue`.
   * Otherwise you may cause the model value on the scope to change incorrectly.
   *
   * <div class="alert alert-info">
   * In any case, the value passed to the method should always reflect the current value
   * of the control. For example, if you are calling `$setViewValue` for an input element,
   * you should pass the input DOM value. Otherwise, the control and the scope model become
   * out of sync. It's also important to note that `$setViewValue` does not call `$render` or change
   * the control's DOM value in any way. If we want to change the control's DOM value
   * programmatically, we should update the `ngModel` scope expression. Its new value will be
   * picked up by the model controller, which will run it through the `$formatters`, `$render` it
   * to update the DOM, and finally call `$validate` on it.
   * </div>
   *
   * @param {*} value value from the view.
   * @param {string} trigger Event that triggered the update.
   */
  $setViewValue: function(value, trigger) {
    this.$viewValue = value;
    if (this.$options.getOption('updateOnDefault')) {
      this.$$debounceViewValueCommit(trigger);
    }
  },

  $$debounceViewValueCommit: function(trigger) {
    var debounceDelay = this.$options.getOption('debounce');

    if (isNumber(debounceDelay[trigger])) {
      debounceDelay = debounceDelay[trigger];
    } else if (isNumber(debounceDelay['default']) &&
      this.$options.getOption('updateOn').indexOf(trigger) === -1
    ) {
      debounceDelay = debounceDelay['default'];
    } else if (isNumber(debounceDelay['*'])) {
      debounceDelay = debounceDelay['*'];
    }

    this.$$timeout.cancel(this.$$pendingDebounce);
    var that = this;
    if (debounceDelay > 0) { // this fails if debounceDelay is an object
      this.$$pendingDebounce = this.$$timeout(function() {
        that.$commitViewValue();
      }, debounceDelay);
    } else if (this.$$rootScope.$$phase) {
      this.$commitViewValue();
    } else {
      this.$$scope.$apply(function() {
        that.$commitViewValue();
      });
    }
  },

  /**
   * @ngdoc method
   *
   * @name ngModel.NgModelController#$overrideModelOptions
   *
   * @description
   *
   * Override the current model options settings programmatically.
   *
   * The previous `ModelOptions` value will not be modified. Instead, a
   * new `ModelOptions` object will inherit from the previous one overriding
   * or inheriting settings that are defined in the given parameter.
   *
   * See {@link ngModelOptions} for information about what options can be specified
   * and how model option inheritance works.
   *
   * <div class="alert alert-warning">
   * **Note:** this function only affects the options set on the `ngModelController`,
   * and not the options on the {@link ngModelOptions} directive from which they might have been
   * obtained initially.
   * </div>
   *
   * <div class="alert alert-danger">
   * **Note:** it is not possible to override the `getterSetter` option.
   * </div>
   *
   * @param {Object} options a hash of settings to override the previous options
   *
   */
  $overrideModelOptions: function(options) {
    this.$options = this.$options.createChild(options);
    this.$$setUpdateOnEvents();
  },

  /**
   * @ngdoc method
   *
   * @name  ngModel.NgModelController#$processModelValue

   * @description
   *
   * Runs the model -> view pipeline on the current
   * {@link ngModel.NgModelController#$modelValue $modelValue}.
   *
   * The following actions are performed by this method:
   *
   * - the `$modelValue` is run through the {@link ngModel.NgModelController#$formatters $formatters}
   * and the result is set to the {@link ngModel.NgModelController#$viewValue $viewValue}
   * - the `ng-empty` or `ng-not-empty` class is set on the element
   * - if the `$viewValue` has changed:
   *   - {@link ngModel.NgModelController#$render $render} is called on the control
   *   - the {@link ngModel.NgModelController#$validators $validators} are run and
   *   the validation status is set.
   *
   * This method is called by ngModel internally when the bound scope value changes.
   * Application developers usually do not have to call this function themselves.
   *
   * This function can be used when the `$viewValue` or the rendered DOM value are not correctly
   * formatted and the `$modelValue` must be run through the `$formatters` again.
   *
   * @example
   * Consider a text input with an autocomplete list (for fruit), where the items are
   * objects with a name and an id.
   * A user enters `ap` and then selects `Apricot` from the list.
   * Based on this, the autocomplete widget will call `$setViewValue({name: 'Apricot', id: 443})`,
   * but the rendered value will still be `ap`.
   * The widget can then call `ctrl.$processModelValue()` to run the model -> view
   * pipeline again, which formats the object to the string `Apricot`,
   * then updates the `$viewValue`, and finally renders it in the DOM.
   *
   * <example module="inputExample" name="ng-model-process">
     <file name="index.html">
      <div ng-controller="inputController" style="display: flex;">
        <div style="margin-right: 30px;">
          Search Fruit:
          <basic-autocomplete items="items" on-select="selectedFruit = item"></basic-autocomplete>
        </div>
        <div>
          Model:<br>
          <pre>{{selectedFruit | json}}</pre>
        </div>
      </div>
     </file>
     <file name="app.js">
      angular.module('inputExample', [])
        .controller('inputController', function($scope) {
          $scope.items = [
            {name: 'Apricot', id: 443},
            {name: 'Clementine', id: 972},
            {name: 'Durian', id: 169},
            {name: 'Jackfruit', id: 982},
            {name: 'Strawberry', id: 863}
          ];
        })
        .component('basicAutocomplete', {
          bindings: {
            items: '<',
            onSelect: '&'
          },
          templateUrl: 'autocomplete.html',
          controller: function($element, $scope) {
            var that = this;
            var ngModel;

            that.$postLink = function() {
              ngModel = $element.find('input').controller('ngModel');

              ngModel.$formatters.push(function(value) {
                return (value && value.name) || value;
              });

              ngModel.$parsers.push(function(value) {
                var match = value;
                for (var i = 0; i < that.items.length; i++) {
                  if (that.items[i].name === value) {
                    match = that.items[i];
                    break;
                  }
                }

                return match;
              });
            };

            that.selectItem = function(item) {
              ngModel.$setViewValue(item);
              ngModel.$processModelValue();
              that.onSelect({item: item});
            };
          }
        });
     </file>
     <file name="autocomplete.html">
       <div>
         <input type="search" ng-model="$ctrl.searchTerm" />
         <ul>
           <li ng-repeat="item in $ctrl.items | filter:$ctrl.searchTerm">
             <button ng-click="$ctrl.selectItem(item)">{{ item.name }}</button>
           </li>
         </ul>
       </div>
     </file>
   * </example>
   *
   */
  $processModelValue: function() {
    var viewValue = this.$$format();

    if (this.$viewValue !== viewValue) {
      this.$$updateEmptyClasses(viewValue);
      this.$viewValue = this.$$lastCommittedViewValue = viewValue;
      this.$render();
      // It is possible that model and view value have been updated during render
      this.$$runValidators(this.$modelValue, this.$viewValue, noop);
    }
  },

  /**
   * This method is called internally to run the $formatters on the $modelValue
   */
  $$format: function() {
    var formatters = this.$formatters,
        idx = formatters.length;

    var viewValue = this.$modelValue;
    while (idx--) {
      viewValue = formatters[idx](viewValue);
    }

    return viewValue;
  },

  /**
   * This method is called internally when the bound scope value changes.
   */
  $$setModelValue: function(modelValue) {
    this.$modelValue = this.$$rawModelValue = modelValue;
    this.$$parserValid = undefined;
    this.$processModelValue();
  },

  $$setUpdateOnEvents: function() {
    if (this.$$updateEvents) {
      this.$$element.off(this.$$updateEvents, this.$$updateEventHandler);
    }

    this.$$updateEvents = this.$options.getOption('updateOn');
    if (this.$$updateEvents) {
      this.$$element.on(this.$$updateEvents, this.$$updateEventHandler);
    }
  },

  $$updateEventHandler: function(ev) {
    this.$$debounceViewValueCommit(ev && ev.type);
  }
};

function setupModelWatcher(ctrl) {
  // model -> value
  // Note: we cannot use a normal scope.$watch as we want to detect the following:
  // 1. scope value is 'a'
  // 2. user enters 'b'
  // 3. ng-change kicks in and reverts scope value to 'a'
  //    -> scope value did not change since the last digest as
  //       ng-change executes in apply phase
  // 4. view should be changed back to 'a'
  ctrl.$$scope.$watch(function ngModelWatch(scope) {
    var modelValue = ctrl.$$ngModelGet(scope);

    // if scope model value and ngModel value are out of sync
    // This cannot be moved to the action function, because it would not catch the
    // case where the model is changed in the ngChange function or the model setter
    if (modelValue !== ctrl.$modelValue &&
      // checks for NaN is needed to allow setting the model to NaN when there's an asyncValidator
      // eslint-disable-next-line no-self-compare
      (ctrl.$modelValue === ctrl.$modelValue || modelValue === modelValue)
    ) {
      ctrl.$$setModelValue(modelValue);
    }

    return modelValue;
  });
}

/**
 * @ngdoc method
 * @name ngModel.NgModelController#$setValidity
 *
 * @description
 * Change the validity state, and notify the form.
 *
 * This method can be called within $parsers/$formatters or a custom validation implementation.
 * However, in most cases it should be sufficient to use the `ngModel.$validators` and
 * `ngModel.$asyncValidators` collections which will call `$setValidity` automatically.
 *
 * @param {string} validationErrorKey Name of the validator. The `validationErrorKey` will be assigned
 *        to either `$error[validationErrorKey]` or `$pending[validationErrorKey]`
 *        (for unfulfilled `$asyncValidators`), so that it is available for data-binding.
 *        The `validationErrorKey` should be in camelCase and will get converted into dash-case
 *        for class name. Example: `myError` will result in `ng-valid-my-error` and `ng-invalid-my-error`
 *        classes and can be bound to as `{{ someForm.someControl.$error.myError }}`.
 * @param {boolean} isValid Whether the current state is valid (true), invalid (false), pending (undefined),
 *                          or skipped (null). Pending is used for unfulfilled `$asyncValidators`.
 *                          Skipped is used by AngularJS when validators do not run because of parse errors and
 *                          when `$asyncValidators` do not run because any of the `$validators` failed.
 */
addSetValidityMethod({
  clazz: NgModelController,
  set: function(object, property) {
    object[property] = true;
  },
  unset: function(object, property) {
    delete object[property];
  }
});


/**
 * @ngdoc directive
 * @name ngModel
 * @restrict A
 * @priority 1
 * @param {expression} ngModel assignable {@link guide/expression Expression} to bind to.
 *
 * @description
 * The `ngModel` directive binds an `input`,`select`, `textarea` (or custom form control) to a
 * property on the scope using {@link ngModel.NgModelController NgModelController},
 * which is created and exposed by this directive.
 *
 * `ngModel` is responsible for:
 *
 * - Binding the view into the model, which other directives such as `input`, `textarea` or `select`
 *   require.
 * - Providing validation behavior (i.e. required, number, email, url).
 * - Keeping the state of the control (valid/invalid, dirty/pristine, touched/untouched, validation errors).
 * - Setting related css classes on the element (`ng-valid`, `ng-invalid`, `ng-dirty`, `ng-pristine`, `ng-touched`,
 *   `ng-untouched`, `ng-empty`, `ng-not-empty`) including animations.
 * - Registering the control with its parent {@link ng.directive:form form}.
 *
 * Note: `ngModel` will try to bind to the property given by evaluating the expression on the
 * current scope. If the property doesn't already exist on this scope, it will be created
 * implicitly and added to the scope.
 *
 * For best practices on using `ngModel`, see:
 *
 *  - [Understanding Scopes](https://github.com/angular/angular.js/wiki/Understanding-Scopes)
 *
 * For basic examples, how to use `ngModel`, see:
 *
 *  - {@link ng.directive:input input}
 *    - {@link input[text] text}
 *    - {@link input[checkbox] checkbox}
 *    - {@link input[radio] radio}
 *    - {@link input[number] number}
 *    - {@link input[email] email}
 *    - {@link input[url] url}
 *    - {@link input[date] date}
 *    - {@link input[datetime-local] datetime-local}
 *    - {@link input[time] time}
 *    - {@link input[month] month}
 *    - {@link input[week] week}
 *  - {@link ng.directive:select select}
 *  - {@link ng.directive:textarea textarea}
 *
 * ## Complex Models (objects or collections)
 *
 * By default, `ngModel` watches the model by reference, not value. This is important to know when
 * binding inputs to models that are objects (e.g. `Date`) or collections (e.g. arrays). If only properties of the
 * object or collection change, `ngModel` will not be notified and so the input will not be  re-rendered.
 *
 * The model must be assigned an entirely new object or collection before a re-rendering will occur.
 *
 * Some directives have options that will cause them to use a custom `$watchCollection` on the model expression
 * - for example, `ngOptions` will do so when a `track by` clause is included in the comprehension expression or
 * if the select is given the `multiple` attribute.
 *
 * The `$watchCollection()` method only does a shallow comparison, meaning that changing properties deeper than the
 * first level of the object (or only changing the properties of an item in the collection if it's an array) will still
 * not trigger a re-rendering of the model.
 *
 * ## CSS classes
 * The following CSS classes are added and removed on the associated input/select/textarea element
 * depending on the validity of the model.
 *
 *  - `ng-valid`: the model is valid
 *  - `ng-invalid`: the model is invalid
 *  - `ng-valid-[key]`: for each valid key added by `$setValidity`
 *  - `ng-invalid-[key]`: for each invalid key added by `$setValidity`
 *  - `ng-pristine`: the control hasn't been interacted with yet
 *  - `ng-dirty`: the control has been interacted with
 *  - `ng-touched`: the control has been blurred
 *  - `ng-untouched`: the control hasn't been blurred
 *  - `ng-pending`: any `$asyncValidators` are unfulfilled
 *  - `ng-empty`: the view does not contain a value or the value is deemed "empty", as defined
 *     by the {@link ngModel.NgModelController#$isEmpty} method
 *  - `ng-not-empty`: the view contains a non-empty value
 *
 * Keep in mind that ngAnimate can detect each of these classes when added and removed.
 *
 * @animations
 * Animations within models are triggered when any of the associated CSS classes are added and removed
 * on the input element which is attached to the model. These classes include: `.ng-pristine`, `.ng-dirty`,
 * `.ng-invalid` and `.ng-valid` as well as any other validations that are performed on the model itself.
 * The animations that are triggered within ngModel are similar to how they work in ngClass and
 * animations can be hooked into using CSS transitions, keyframes as well as JS animations.
 *
 * The following example shows a simple way to utilize CSS transitions to style an input element
 * that has been rendered as invalid after it has been validated:
 *
 * <pre>
 * //be sure to include ngAnimate as a module to hook into more
 * //advanced animations
 * .my-input {
 *   transition:0.5s linear all;
 *   background: white;
 * }
 * .my-input.ng-invalid {
 *   background: red;
 *   color:white;
 * }
 * </pre>
 *
 * @example
 * ### Basic Usage
 * <example deps="angular-animate.js" animations="true" fixBase="true" module="inputExample" name="ng-model">
     <file name="index.html">
       <script>
        angular.module('inputExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.val = '1';
          }]);
       </script>
       <style>
         .my-input {
           transition:all linear 0.5s;
           background: transparent;
         }
         .my-input.ng-invalid {
           color:white;
           background: red;
         }
       </style>
       <p id="inputDescription">
        Update input to see transitions when valid/invalid.
        Integer is a valid value.
       </p>
       <form name="testForm" ng-controller="ExampleController">
         <input ng-model="val" ng-pattern="/^\d+$/" name="anim" class="my-input"
                aria-describedby="inputDescription" />
       </form>
     </file>
 * </example>
 *
 * @example
 * ### Binding to a getter/setter
 *
 * Sometimes it's helpful to bind `ngModel` to a getter/setter function.  A getter/setter is a
 * function that returns a representation of the model when called with zero arguments, and sets
 * the internal state of a model when called with an argument. It's sometimes useful to use this
 * for models that have an internal representation that's different from what the model exposes
 * to the view.
 *
 * <div class="alert alert-success">
 * **Best Practice:** It's best to keep getters fast because AngularJS is likely to call them more
 * frequently than other parts of your code.
 * </div>
 *
 * You use this behavior by adding `ng-model-options="{ getterSetter: true }"` to an element that
 * has `ng-model` attached to it. You can also add `ng-model-options="{ getterSetter: true }"` to
 * a `<form>`, which will enable this behavior for all `<input>`s within it. See
 * {@link ng.directive:ngModelOptions `ngModelOptions`} for more.
 *
 * The following example shows how to use `ngModel` with a getter/setter:
 *
 * @example
 * <example name="ngModel-getter-setter" module="getterSetterExample">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <form name="userForm">
           <label>Name:
             <input type="text" name="userName"
                    ng-model="user.name"
                    ng-model-options="{ getterSetter: true }" />
           </label>
         </form>
         <pre>user.name = <span ng-bind="user.name()"></span></pre>
       </div>
     </file>
     <file name="app.js">
       angular.module('getterSetterExample', [])
         .controller('ExampleController', ['$scope', function($scope) {
           var _name = 'Brian';
           $scope.user = {
             name: function(newName) {
              // Note that newName can be undefined for two reasons:
              // 1. Because it is called as a getter and thus called with no arguments
              // 2. Because the property should actually be set to undefined. This happens e.g. if the
              //    input is invalid
              return arguments.length ? (_name = newName) : _name;
             }
           };
         }]);
     </file>
 * </example>
 */
var ngModelDirective = ['$rootScope', function($rootScope) {
  return {
    restrict: 'A',
    require: ['ngModel', '^?form', '^?ngModelOptions'],
    controller: NgModelController,
    // Prelink needs to run before any input directive
    // so that we can set the NgModelOptions in NgModelController
    // before anyone else uses it.
    priority: 1,
    compile: function ngModelCompile(element) {
      // Setup initial state of the control
      element.addClass(PRISTINE_CLASS).addClass(UNTOUCHED_CLASS).addClass(VALID_CLASS);

      return {
        pre: function ngModelPreLink(scope, element, attr, ctrls) {
          var modelCtrl = ctrls[0],
              formCtrl = ctrls[1] || modelCtrl.$$parentForm,
              optionsCtrl = ctrls[2];

          if (optionsCtrl) {
            modelCtrl.$options = optionsCtrl.$options;
          }

          modelCtrl.$$initGetterSetters();

          // notify others, especially parent forms
          formCtrl.$addControl(modelCtrl);

          attr.$observe('name', function(newValue) {
            if (modelCtrl.$name !== newValue) {
              modelCtrl.$$parentForm.$$renameControl(modelCtrl, newValue);
            }
          });

          scope.$on('$destroy', function() {
            modelCtrl.$$parentForm.$removeControl(modelCtrl);
          });
        },
        post: function ngModelPostLink(scope, element, attr, ctrls) {
          var modelCtrl = ctrls[0];
          modelCtrl.$$setUpdateOnEvents();

          function setTouched() {
            modelCtrl.$setTouched();
          }

          element.on('blur', function() {
            if (modelCtrl.$touched) return;

            if ($rootScope.$$phase) {
              scope.$evalAsync(setTouched);
            } else {
              scope.$apply(setTouched);
            }
          });
        }
      };
    }
  };
}];

/* exported defaultModelOptions */
var defaultModelOptions;
var DEFAULT_REGEXP = /(\s+|^)default(\s+|$)/;

/**
 * @ngdoc type
 * @name ModelOptions
 * @description
 * A container for the options set by the {@link ngModelOptions} directive
 */
function ModelOptions(options) {
  this.$$options = options;
}

ModelOptions.prototype = {

  /**
   * @ngdoc method
   * @name ModelOptions#getOption
   * @param {string} name the name of the option to retrieve
   * @returns {*} the value of the option
   * @description
   * Returns the value of the given option
   */
  getOption: function(name) {
    return this.$$options[name];
  },

  /**
   * @ngdoc method
   * @name ModelOptions#createChild
   * @param {Object} options a hash of options for the new child that will override the parent's options
   * @return {ModelOptions} a new `ModelOptions` object initialized with the given options.
   */
  createChild: function(options) {
    var inheritAll = false;

    // make a shallow copy
    options = extend({}, options);

    // Inherit options from the parent if specified by the value `"$inherit"`
    forEach(options, /** @this */ function(option, key) {
      if (option === '$inherit') {
        if (key === '*') {
          inheritAll = true;
        } else {
          options[key] = this.$$options[key];
          // `updateOn` is special so we must also inherit the `updateOnDefault` option
          if (key === 'updateOn') {
            options.updateOnDefault = this.$$options.updateOnDefault;
          }
        }
      } else {
        if (key === 'updateOn') {
          // If the `updateOn` property contains the `default` event then we have to remove
          // it from the event list and set the `updateOnDefault` flag.
          options.updateOnDefault = false;
          options[key] = trim(option.replace(DEFAULT_REGEXP, function() {
            options.updateOnDefault = true;
            return ' ';
          }));
        }
      }
    }, this);

    if (inheritAll) {
      // We have a property of the form: `"*": "$inherit"`
      delete options['*'];
      defaults(options, this.$$options);
    }

    // Finally add in any missing defaults
    defaults(options, defaultModelOptions.$$options);

    return new ModelOptions(options);
  }
};


defaultModelOptions = new ModelOptions({
  updateOn: '',
  updateOnDefault: true,
  debounce: 0,
  getterSetter: false,
  allowInvalid: false,
  timezone: null
});


/**
 * @ngdoc directive
 * @name ngModelOptions
 * @restrict A
 * @priority 10
 *
 * @description
 * This directive allows you to modify the behaviour of {@link ngModel} directives within your
 * application. You can specify an `ngModelOptions` directive on any element. All {@link ngModel}
 * directives will use the options of their nearest `ngModelOptions` ancestor.
 *
 * The `ngModelOptions` settings are found by evaluating the value of the attribute directive as
 * an AngularJS expression. This expression should evaluate to an object, whose properties contain
 * the settings. For example: `<div ng-model-options="{ debounce: 100 }"`.
 *
 * ## Inheriting Options
 *
 * You can specify that an `ngModelOptions` setting should be inherited from a parent `ngModelOptions`
 * directive by giving it the value of `"$inherit"`.
 * Then it will inherit that setting from the first `ngModelOptions` directive found by traversing up the
 * DOM tree. If there is no ancestor element containing an `ngModelOptions` directive then default settings
 * will be used.
 *
 * For example given the following fragment of HTML
 *
 *
 * ```html
 * <div ng-model-options="{ allowInvalid: true, debounce: 200 }">
 *   <form ng-model-options="{ updateOn: 'blur', allowInvalid: '$inherit' }">
 *     <input ng-model-options="{ updateOn: 'default', allowInvalid: '$inherit' }" />
 *   </form>
 * </div>
 * ```
 *
 * the `input` element will have the following settings
 *
 * ```js
 * { allowInvalid: true, updateOn: 'default', debounce: 0 }
 * ```
 *
 * Notice that the `debounce` setting was not inherited and used the default value instead.
 *
 * You can specify that all undefined settings are automatically inherited from an ancestor by
 * including a property with key of `"*"` and value of `"$inherit"`.
 *
 * For example given the following fragment of HTML
 *
 *
 * ```html
 * <div ng-model-options="{ allowInvalid: true, debounce: 200 }">
 *   <form ng-model-options="{ updateOn: 'blur', "*": '$inherit' }">
 *     <input ng-model-options="{ updateOn: 'default', "*": '$inherit' }" />
 *   </form>
 * </div>
 * ```
 *
 * the `input` element will have the following settings
 *
 * ```js
 * { allowInvalid: true, updateOn: 'default', debounce: 200 }
 * ```
 *
 * Notice that the `debounce` setting now inherits the value from the outer `<div>` element.
 *
 * If you are creating a reusable component then you should be careful when using `"*": "$inherit"`
 * since you may inadvertently inherit a setting in the future that changes the behavior of your component.
 *
 *
 * ## Triggering and debouncing model updates
 *
 * The `updateOn` and `debounce` properties allow you to specify a custom list of events that will
 * trigger a model update and/or a debouncing delay so that the actual update only takes place when
 * a timer expires; this timer will be reset after another change takes place.
 *
 * Given the nature of `ngModelOptions`, the value displayed inside input fields in the view might
 * be different from the value in the actual model. This means that if you update the model you
 * should also invoke {@link ngModel.NgModelController#$rollbackViewValue} on the relevant input field in
 * order to make sure it is synchronized with the model and that any debounced action is canceled.
 *
 * The easiest way to reference the control's {@link ngModel.NgModelController#$rollbackViewValue}
 * method is by making sure the input is placed inside a form that has a `name` attribute. This is
 * important because `form` controllers are published to the related scope under the name in their
 * `name` attribute.
 *
 * Any pending changes will take place immediately when an enclosing form is submitted via the
 * `submit` event. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
 * to have access to the updated model.
 *
 * ### Overriding immediate updates
 *
 * The following example shows how to override immediate updates. Changes on the inputs within the
 * form will update the model only when the control loses focus (blur event). If `escape` key is
 * pressed while the input field is focused, the value is reset to the value in the current model.
 *
 * <example name="ngModelOptions-directive-blur" module="optionsExample">
 *   <file name="index.html">
 *     <div ng-controller="ExampleController">
 *       <form name="userForm">
 *         <label>
 *           Name:
 *           <input type="text" name="userName"
 *                  ng-model="user.name"
 *                  ng-model-options="{ updateOn: 'blur' }"
 *                  ng-keyup="cancel($event)" />
 *         </label><br />
 *         <label>
 *           Other data:
 *           <input type="text" ng-model="user.data" />
 *         </label><br />
 *       </form>
 *       <pre>user.name = <span ng-bind="user.name"></span></pre>
 *     </div>
 *   </file>
 *   <file name="app.js">
 *     angular.module('optionsExample', [])
 *       .controller('ExampleController', ['$scope', function($scope) {
 *         $scope.user = { name: 'say', data: '' };
 *
 *         $scope.cancel = function(e) {
 *           if (e.keyCode === 27) {
 *             $scope.userForm.userName.$rollbackViewValue();
 *           }
 *         };
 *       }]);
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     var model = element(by.binding('user.name'));
 *     var input = element(by.model('user.name'));
 *     var other = element(by.model('user.data'));
 *
 *     it('should allow custom events', function() {
 *       input.sendKeys(' hello');
 *       input.click();
 *       expect(model.getText()).toEqual('say');
 *       other.click();
 *       expect(model.getText()).toEqual('say hello');
 *     });
 *
 *     it('should $rollbackViewValue when model changes', function() {
 *       input.sendKeys(' hello');
 *       expect(input.getAttribute('value')).toEqual('say hello');
 *       input.sendKeys(protractor.Key.ESCAPE);
 *       expect(input.getAttribute('value')).toEqual('say');
 *       other.click();
 *       expect(model.getText()).toEqual('say');
 *     });
 *   </file>
 * </example>
 *
 * ### Debouncing updates
 *
 * The next example shows how to debounce model changes. Model will be updated only 1 sec after last change.
 * If the `Clear` button is pressed, any debounced action is canceled and the value becomes empty.
 *
 * <example name="ngModelOptions-directive-debounce" module="optionsExample">
 *   <file name="index.html">
 *     <div ng-controller="ExampleController">
 *       <form name="userForm">
 *         Name:
 *         <input type="text" name="userName"
 *                ng-model="user.name"
 *                ng-model-options="{ debounce: 1000 }" />
 *         <button ng-click="userForm.userName.$rollbackViewValue(); user.name=''">Clear</button><br />
 *       </form>
 *       <pre>user.name = <span ng-bind="user.name"></span></pre>
 *     </div>
 *   </file>
 *   <file name="app.js">
 *     angular.module('optionsExample', [])
 *       .controller('ExampleController', ['$scope', function($scope) {
 *         $scope.user = { name: 'say' };
 *       }]);
 *   </file>
 * </example>
 *
 * ### Default events, extra triggers, and catch-all debounce values
 *
 * This example shows the relationship between "default" update events and
 * additional `updateOn` triggers.
 *
 * `default` events are those that are bound to the control, and when fired, update the `$viewValue`
 * via {@link ngModel.NgModelController#$setViewValue $setViewValue}. Every event that is not listed
 * in `updateOn` is considered a "default" event, since different control types have different
 * default events.
 *
 * The control in this example updates by "default", "click", and "blur", with different `debounce`
 * values. You can see that "click" doesn't have an individual `debounce` value -
 * therefore it uses the `*` debounce value.
 *
 * There is also a button that calls {@link ngModel.NgModelController#$setViewValue $setViewValue}
 * directly with a "custom" event. Since "custom" is not defined in the `updateOn` list,
 * it is considered a "default" event and will update the
 * control if "default" is defined in `updateOn`, and will receive the "default" debounce value.
 * Note that this is just to illustrate how custom controls would possibly call `$setViewValue`.
 *
 * You can change the `updateOn` and `debounce` configuration to test different scenarios. This
 * is done with {@link ngModel.NgModelController#$overrideModelOptions $overrideModelOptions}.
 *
  <example name="ngModelOptions-advanced" module="optionsExample">
    <file name="index.html">
       <model-update-demo></model-update-demo>
    </file>
    <file name="app.js">
      angular.module('optionsExample', [])
        .component('modelUpdateDemo', {
          templateUrl: 'template.html',
          controller: function() {
            this.name = 'Chinua';

            this.options = {
              updateOn: 'default blur click',
              debounce: {
                default: 2000,
                blur: 0,
                '*': 1000
              }
            };

            this.updateEvents = function() {
              var eventList = this.options.updateOn.split(' ');
              eventList.push('*');
              var events = {};

              for (var i = 0; i < eventList.length; i++) {
                events[eventList[i]] = this.options.debounce[eventList[i]];
              }

              this.events = events;
            };

            this.updateOptions = function() {
              var options = angular.extend(this.options, {
                updateOn: Object.keys(this.events).join(' ').replace('*', ''),
                debounce: this.events
              });

              this.form.input.$overrideModelOptions(options);
            };

            // Initialize the event form
            this.updateEvents();
          }
        });
    </file>
    <file name="template.html">
      <form name="$ctrl.form">
        Input: <input type="text" name="input" ng-model="$ctrl.name" ng-model-options="$ctrl.options" />
      </form>
      Model: <tt>{{$ctrl.name}}</tt>
      <hr>
      <button ng-click="$ctrl.form.input.$setViewValue('some value', 'custom')">Trigger setViewValue with 'some value' and 'custom' event</button>

      <hr>
      <form ng-submit="$ctrl.updateOptions()">
        <b>updateOn</b><br>
        <input type="text" ng-model="$ctrl.options.updateOn" ng-change="$ctrl.updateEvents()" ng-model-options="{debounce: 500}">

        <table>
          <tr>
            <th>Option</th>
            <th>Debounce value</th>
          </tr>
          <tr ng-repeat="(key, value) in $ctrl.events">
            <td>{{key}}</td>
            <td><input type="number" ng-model="$ctrl.events[key]" /></td>
          </tr>
        </table>

        <br>
        <input type="submit" value="Update options">
      </form>
    </file>
  </example>
 *
 *
 * ## Model updates and validation
 *
 * The default behaviour in `ngModel` is that the model value is set to `undefined` when the
 * validation determines that the value is invalid. By setting the `allowInvalid` property to true,
 * the model will still be updated even if the value is invalid.
 *
 *
 * ## Connecting to the scope
 *
 * By setting the `getterSetter` property to true you are telling ngModel that the `ngModel` expression
 * on the scope refers to a "getter/setter" function rather than the value itself.
 *
 * The following example shows how to bind to getter/setters:
 *
 * <example name="ngModelOptions-directive-getter-setter" module="getterSetterExample">
 *   <file name="index.html">
 *     <div ng-controller="ExampleController">
 *       <form name="userForm">
 *         <label>
 *           Name:
 *           <input type="text" name="userName"
 *                  ng-model="user.name"
 *                  ng-model-options="{ getterSetter: true }" />
 *         </label>
 *       </form>
 *       <pre>user.name = <span ng-bind="user.name()"></span></pre>
 *     </div>
 *   </file>
 *   <file name="app.js">
 *     angular.module('getterSetterExample', [])
 *       .controller('ExampleController', ['$scope', function($scope) {
 *         var _name = 'Brian';
 *         $scope.user = {
 *           name: function(newName) {
 *             return angular.isDefined(newName) ? (_name = newName) : _name;
 *           }
 *         };
 *       }]);
 *   </file>
 * </example>
 *
 *
 * ## Programmatically changing options
 *
 * The `ngModelOptions` expression is only evaluated once when the directive is linked; it is not
 * watched for changes. However, it is possible to override the options on a single
 * {@link ngModel.NgModelController} instance with
 * {@link ngModel.NgModelController#$overrideModelOptions `NgModelController#$overrideModelOptions()`}.
 * See also the example for
 * {@link ngModelOptions#default-events-extra-triggers-and-catch-all-debounce-values
 * Default events, extra triggers, and catch-all debounce values}.
 *
 *
 * ## Specifying timezones
 *
 * You can specify the timezone that date/time input directives expect by providing its name in the
 * `timezone` property.
 *
 *
 * ## Formatting the value of `time` and `datetime-local`
 *
 * With the options `timeSecondsFormat` and `timeStripZeroSeconds` it is possible to adjust the value
 * that is displayed in the control. Note that browsers may apply their own formatting
 * in the user interface.
 *
   <example name="ngModelOptions-time-format" module="timeExample">
     <file name="index.html">
       <time-example></time-example>
     </file>
     <file name="script.js">
        angular.module('timeExample', [])
          .component('timeExample', {
            templateUrl: 'timeExample.html',
            controller: function() {
              this.time = new Date(1970, 0, 1, 14, 57, 0);

              this.options = {
                timeSecondsFormat: 'ss',
                timeStripZeroSeconds: true
              };

              this.optionChange = function() {
                this.timeForm.timeFormatted.$overrideModelOptions(this.options);
                this.time = new Date(this.time);
              };
            }
          });
     </file>
     <file name="timeExample.html">
       <form name="$ctrl.timeForm">
         <strong>Default</strong>:
         <input type="time" ng-model="$ctrl.time" step="any" /><br>
         <strong>With options</strong>:
         <input type="time" name="timeFormatted" ng-model="$ctrl.time" step="any" ng-model-options="$ctrl.options" />
         <br>

         Options:<br>
         <code>timeSecondsFormat</code>:
         <input
           type="text"
           ng-model="$ctrl.options.timeSecondsFormat"
           ng-change="$ctrl.optionChange()">
         <br>
         <code>timeStripZeroSeconds</code>:
         <input
           type="checkbox"
           ng-model="$ctrl.options.timeStripZeroSeconds"
           ng-change="$ctrl.optionChange()">
        </form>
      </file>
 *  </example>
 *
 * @param {Object} ngModelOptions options to apply to {@link ngModel} directives on this element and
 *   and its descendents.
 *
 * **General options**:
 *
 *   - `updateOn`: string specifying which event should the input be bound to. You can set several
 *     events using an space delimited list. There is a special event called `default` that
 *     matches the default events belonging to the control. These are the events that are bound to
 *     the control, and when fired, update the `$viewValue` via `$setViewValue`.
 *
 *     `ngModelOptions` considers every event that is not listed in `updateOn` a "default" event,
 *     since different control types use different default events.
 *
 *     See also the section {@link ngModelOptions#triggering-and-debouncing-model-updates
 *     Triggering and debouncing model updates}.
 *
 *   - `debounce`: integer value which contains the debounce model update value in milliseconds. A
 *     value of 0 triggers an immediate update. If an object is supplied instead, you can specify a
 *     custom value for each event. For example:
 *     ```
 *     ng-model-options="{
 *       updateOn: 'default blur',
 *       debounce: { 'default': 500, 'blur': 0 }
 *     }"
 *     ```
 *     You can use the `*` key to specify a debounce value that applies to all events that are not
 *     specifically listed. In the following example, `mouseup` would have a debounce delay of 1000:
 *     ```
 *     ng-model-options="{
 *       updateOn: 'default blur mouseup',
 *       debounce: { 'default': 500, 'blur': 0, '*': 1000 }
 *     }"
 *     ```
 *   - `allowInvalid`: boolean value which indicates that the model can be set with values that did
 *     not validate correctly instead of the default behavior of setting the model to undefined.
 *   - `getterSetter`: boolean value which determines whether or not to treat functions bound to
 *     `ngModel` as getters/setters.
 *
 *
 *  **Input-type specific options**:
 *
 *   - `timezone`: Defines the timezone to be used to read/write the `Date` instance in the model for
 *     `<input type="date" />`, `<input type="time" />`, ... . It understands UTC/GMT and the
 *     continental US time zone abbreviations, but for general use, use a time zone offset, for
 *     example, `'+0430'` (4 hours, 30 minutes east of the Greenwich meridian)
 *     If not specified, the timezone of the browser will be used.
 *     Note that changing the timezone will have no effect on the current date, and is only applied after
 *     the next input / model change.
 *
 *   - `timeSecondsFormat`: Defines if the `time` and `datetime-local` types should show seconds and
 *     milliseconds. The option follows the format string of {@link date date filter}.
 *     By default, the options is `undefined` which is equal to `'ss.sss'` (seconds and milliseconds).
 *     The other options are `'ss'` (strips milliseconds), and `''` (empty string), which strips both
 *     seconds and milliseconds.
 *     Note that browsers that support `time` and `datetime-local` require the hour and minutes
 *     part of the time string, and may show the value differently in the user interface.
 *     {@link ngModelOptions#formatting-the-value-of-time-and-datetime-local- See the example}.
 *
 *   - `timeStripZeroSeconds`: Defines if the `time` and `datetime-local` types should strip the
 *     seconds and milliseconds from the formatted value if they are zero. This option is applied
 *     after `timeSecondsFormat`.
 *     This option can be used to make the formatting consistent over different browsers, as some
 *     browsers with support for `time` will natively hide the milliseconds and
 *     seconds if they are zero, but others won't, and browsers that don't implement these input
 *     types will always show the full string.
 *     {@link ngModelOptions#formatting-the-value-of-time-and-datetime-local- See the example}.
 *
 */
var ngModelOptionsDirective = function() {
  NgModelOptionsController.$inject = ['$attrs', '$scope'];
  function NgModelOptionsController($attrs, $scope) {
    this.$$attrs = $attrs;
    this.$$scope = $scope;
  }
  NgModelOptionsController.prototype = {
    $onInit: function() {
      var parentOptions = this.parentCtrl ? this.parentCtrl.$options : defaultModelOptions;
      var modelOptionsDefinition = this.$$scope.$eval(this.$$attrs.ngModelOptions);

      this.$options = parentOptions.createChild(modelOptionsDefinition);
    }
  };

  return {
    restrict: 'A',
    // ngModelOptions needs to run before ngModel and input directives
    priority: 10,
    require: {parentCtrl: '?^^ngModelOptions'},
    bindToController: true,
    controller: NgModelOptionsController
  };
};


// shallow copy over values from `src` that are not already specified on `dst`
function defaults(dst, src) {
  forEach(src, function(value, key) {
    if (!isDefined(dst[key])) {
      dst[key] = value;
    }
  });
}

/**
 * @ngdoc directive
 * @name ngNonBindable
 * @restrict AC
 * @priority 1000
 * @element ANY
 *
 * @description
 * The `ngNonBindable` directive tells AngularJS not to compile or bind the contents of the current
 * DOM element, including directives on the element itself that have a lower priority than
 * `ngNonBindable`. This is useful if the element contains what appears to be AngularJS directives
 * and bindings but which should be ignored by AngularJS. This could be the case if you have a site
 * that displays snippets of code, for instance.
 *
 * @example
 * In this example there are two locations where a simple interpolation binding (`{{}}`) is present,
 * but the one wrapped in `ngNonBindable` is left alone.
 *
  <example name="ng-non-bindable">
    <file name="index.html">
      <div>Normal: {{1 + 2}}</div>
      <div ng-non-bindable>Ignored: {{1 + 2}}</div>
    </file>
    <file name="protractor.js" type="protractor">
     it('should check ng-non-bindable', function() {
       expect(element(by.binding('1 + 2')).getText()).toContain('3');
       expect(element.all(by.css('div')).last().getText()).toMatch(/1 \+ 2/);
     });
    </file>
  </example>
 */
var ngNonBindableDirective = ngDirective({ terminal: true, priority: 1000 });

/* exported ngOptionsDirective */

/* global jqLiteRemove */

var ngOptionsMinErr = minErr('ngOptions');

/**
 * @ngdoc directive
 * @name ngOptions
 * @restrict A
 *
 * @description
 *
 * The `ngOptions` attribute can be used to dynamically generate a list of `<option>`
 * elements for the `<select>` element using the array or object obtained by evaluating the
 * `ngOptions` comprehension expression.
 *
 * In many cases, {@link ng.directive:ngRepeat ngRepeat} can be used on `<option>` elements instead of
 * `ngOptions` to achieve a similar result. However, `ngOptions` provides some benefits:
 * - more flexibility in how the `<select>`'s model is assigned via the `select` **`as`** part of the
 * comprehension expression
 * - reduced memory consumption by not creating a new scope for each repeated instance
 * - increased render speed by creating the options in a documentFragment instead of individually
 *
 * When an item in the `<select>` menu is selected, the array element or object property
 * represented by the selected option will be bound to the model identified by the `ngModel`
 * directive.
 *
 * Optionally, a single hard-coded `<option>` element, with the value set to an empty string, can
 * be nested into the `<select>` element. This element will then represent the `null` or "not selected"
 * option. See example below for demonstration.
 *
 * ## Complex Models (objects or collections)
 *
 * By default, `ngModel` watches the model by reference, not value. This is important to know when
 * binding the select to a model that is an object or a collection.
 *
 * One issue occurs if you want to preselect an option. For example, if you set
 * the model to an object that is equal to an object in your collection, `ngOptions` won't be able to set the selection,
 * because the objects are not identical. So by default, you should always reference the item in your collection
 * for preselections, e.g.: `$scope.selected = $scope.collection[3]`.
 *
 * Another solution is to use a `track by` clause, because then `ngOptions` will track the identity
 * of the item not by reference, but by the result of the `track by` expression. For example, if your
 * collection items have an id property, you would `track by item.id`.
 *
 * A different issue with objects or collections is that ngModel won't detect if an object property or
 * a collection item changes. For that reason, `ngOptions` additionally watches the model using
 * `$watchCollection`, when the expression contains a `track by` clause or the the select has the `multiple` attribute.
 * This allows ngOptions to trigger a re-rendering of the options even if the actual object/collection
 * has not changed identity, but only a property on the object or an item in the collection changes.
 *
 * Note that `$watchCollection` does a shallow comparison of the properties of the object (or the items in the collection
 * if the model is an array). This means that changing a property deeper than the first level inside the
 * object/collection will not trigger a re-rendering.
 *
 * ## `select` **`as`**
 *
 * Using `select` **`as`** will bind the result of the `select` expression to the model, but
 * the value of the `<select>` and `<option>` html elements will be either the index (for array data sources)
 * or property name (for object data sources) of the value within the collection. If a **`track by`** expression
 * is used, the result of that expression will be set as the value of the `option` and `select` elements.
 *
 *
 * ### `select` **`as`** and **`track by`**
 *
 * <div class="alert alert-warning">
 * Be careful when using `select` **`as`** and **`track by`** in the same expression.
 * </div>
 *
 * Given this array of items on the $scope:
 *
 * ```js
 * $scope.items = [{
 *   id: 1,
 *   label: 'aLabel',
 *   subItem: { name: 'aSubItem' }
 * }, {
 *   id: 2,
 *   label: 'bLabel',
 *   subItem: { name: 'bSubItem' }
 * }];
 * ```
 *
 * This will work:
 *
 * ```html
 * <select ng-options="item as item.label for item in items track by item.id" ng-model="selected"></select>
 * ```
 * ```js
 * $scope.selected = $scope.items[0];
 * ```
 *
 * but this will not work:
 *
 * ```html
 * <select ng-options="item.subItem as item.label for item in items track by item.id" ng-model="selected"></select>
 * ```
 * ```js
 * $scope.selected = $scope.items[0].subItem;
 * ```
 *
 * In both examples, the **`track by`** expression is applied successfully to each `item` in the
 * `items` array. Because the selected option has been set programmatically in the controller, the
 * **`track by`** expression is also applied to the `ngModel` value. In the first example, the
 * `ngModel` value is `items[0]` and the **`track by`** expression evaluates to `items[0].id` with
 * no issue. In the second example, the `ngModel` value is `items[0].subItem` and the **`track by`**
 * expression evaluates to `items[0].subItem.id` (which is undefined). As a result, the model value
 * is not matched against any `<option>` and the `<select>` appears as having no selected value.
 *
 *
 * @param {string} ngModel Assignable AngularJS expression to data-bind to.
 * @param {comprehension_expression} ngOptions in one of the following forms:
 *
 *   * for array data sources:
 *     * `label` **`for`** `value` **`in`** `array`
 *     * `select` **`as`** `label` **`for`** `value` **`in`** `array`
 *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array`
 *     * `label` **`disable when`** `disable` **`for`** `value` **`in`** `array`
 *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array` **`track by`** `trackexpr`
 *     * `label` **`disable when`** `disable` **`for`** `value` **`in`** `array` **`track by`** `trackexpr`
 *     * `label` **`for`** `value` **`in`** `array` | orderBy:`orderexpr` **`track by`** `trackexpr`
 *        (for including a filter with `track by`)
 *   * for object data sources:
 *     * `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
 *     * `select` **`as`** `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
 *     * `label` **`group by`** `group` **`for (`**`key`**`,`** `value`**`) in`** `object`
 *     * `label` **`disable when`** `disable` **`for (`**`key`**`,`** `value`**`) in`** `object`
 *     * `select` **`as`** `label` **`group by`** `group`
 *         **`for` `(`**`key`**`,`** `value`**`) in`** `object`
 *     * `select` **`as`** `label` **`disable when`** `disable`
 *         **`for` `(`**`key`**`,`** `value`**`) in`** `object`
 *
 * Where:
 *
 *   * `array` / `object`: an expression which evaluates to an array / object to iterate over.
 *   * `value`: local variable which will refer to each item in the `array` or each property value
 *      of `object` during iteration.
 *   * `key`: local variable which will refer to a property name in `object` during iteration.
 *   * `label`: The result of this expression will be the label for `<option>` element. The
 *     `expression` will most likely refer to the `value` variable (e.g. `value.propertyName`).
 *   * `select`: The result of this expression will be bound to the model of the parent `<select>`
 *      element. If not specified, `select` expression will default to `value`.
 *   * `group`: The result of this expression will be used to group options using the `<optgroup>`
 *      DOM element.
 *   * `disable`: The result of this expression will be used to disable the rendered `<option>`
 *      element. Return `true` to disable.
 *   * `trackexpr`: Used when working with an array of objects. The result of this expression will be
 *      used to identify the objects in the array. The `trackexpr` will most likely refer to the
 *     `value` variable (e.g. `value.propertyName`). With this the selection is preserved
 *      even when the options are recreated (e.g. reloaded from the server).
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required The control is considered valid only if value is entered.
 * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
 *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
 *    `required` when you want to data-bind to the `required` attribute.
 * @param {string=} ngAttrSize sets the size of the select element dynamically. Uses the
 * {@link guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes ngAttr} directive.
 *
 * @example
    <example module="selectExample" name="select">
      <file name="index.html">
        <script>
        angular.module('selectExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.colors = [
              {name:'black', shade:'dark'},
              {name:'white', shade:'light', notAnOption: true},
              {name:'red', shade:'dark'},
              {name:'blue', shade:'dark', notAnOption: true},
              {name:'yellow', shade:'light', notAnOption: false}
            ];
            $scope.myColor = $scope.colors[2]; // red
          }]);
        </script>
        <div ng-controller="ExampleController">
          <ul>
            <li ng-repeat="color in colors">
              <label>Name: <input ng-model="color.name"></label>
              <label><input type="checkbox" ng-model="color.notAnOption"> Disabled?</label>
              <button ng-click="colors.splice($index, 1)" aria-label="Remove">X</button>
            </li>
            <li>
              <button ng-click="colors.push({})">add</button>
            </li>
          </ul>
          <hr/>
          <label>Color (null not allowed):
            <select ng-model="myColor" ng-options="color.name for color in colors"></select>
          </label><br/>
          <label>Color (null allowed):
          <span  class="nullable">
            <select ng-model="myColor" ng-options="color.name for color in colors">
              <option value="">-- choose color --</option>
            </select>
          </span></label><br/>

          <label>Color grouped by shade:
            <select ng-model="myColor" ng-options="color.name group by color.shade for color in colors">
            </select>
          </label><br/>

          <label>Color grouped by shade, with some disabled:
            <select ng-model="myColor"
                  ng-options="color.name group by color.shade disable when color.notAnOption for color in colors">
            </select>
          </label><br/>



          Select <button ng-click="myColor = { name:'not in list', shade: 'other' }">bogus</button>.
          <br/>
          <hr/>
          Currently selected: {{ {selected_color:myColor} }}
          <div style="border:solid 1px black; height:20px"
               ng-style="{'background-color':myColor.name}">
          </div>
        </div>
      </file>
      <file name="protractor.js" type="protractor">
         it('should check ng-options', function() {
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('red');
           element.all(by.model('myColor')).first().click();
           element.all(by.css('select[ng-model="myColor"] option')).first().click();
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('black');
           element(by.css('.nullable select[ng-model="myColor"]')).click();
           element.all(by.css('.nullable select[ng-model="myColor"] option')).first().click();
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('null');
         });
      </file>
    </example>
 */

/* eslint-disable max-len */
//                     //00001111111111000000000002222222222000000000000000000000333333333300000000000000000000000004444444444400000000000005555555555555000000000666666666666600000007777777777777000000000000000888888888800000000000000000009999999999
var NG_OPTIONS_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+group\s+by\s+([\s\S]+?))?(?:\s+disable\s+when\s+([\s\S]+?))?\s+for\s+(?:([$\w][$\w]*)|(?:\(\s*([$\w][$\w]*)\s*,\s*([$\w][$\w]*)\s*\)))\s+in\s+([\s\S]+?)(?:\s+track\s+by\s+([\s\S]+?))?$/;
                        // 1: value expression (valueFn)
                        // 2: label expression (displayFn)
                        // 3: group by expression (groupByFn)
                        // 4: disable when expression (disableWhenFn)
                        // 5: array item variable name
                        // 6: object item key variable name
                        // 7: object item value variable name
                        // 8: collection expression
                        // 9: track by expression
/* eslint-enable */


var ngOptionsDirective = ['$compile', '$document', '$parse', function($compile, $document, $parse) {

  function parseOptionsExpression(optionsExp, selectElement, scope) {

    var match = optionsExp.match(NG_OPTIONS_REGEXP);
    if (!(match)) {
      throw ngOptionsMinErr('iexp',
        'Expected expression in form of ' +
        '\'_select_ (as _label_)? for (_key_,)?_value_ in _collection_\'' +
        ' but got \'{0}\'. Element: {1}',
        optionsExp, startingTag(selectElement));
    }

    // Extract the parts from the ngOptions expression

    // The variable name for the value of the item in the collection
    var valueName = match[5] || match[7];
    // The variable name for the key of the item in the collection
    var keyName = match[6];

    // An expression that generates the viewValue for an option if there is a label expression
    var selectAs = / as /.test(match[0]) && match[1];
    // An expression that is used to track the id of each object in the options collection
    var trackBy = match[9];
    // An expression that generates the viewValue for an option if there is no label expression
    var valueFn = $parse(match[2] ? match[1] : valueName);
    var selectAsFn = selectAs && $parse(selectAs);
    var viewValueFn = selectAsFn || valueFn;
    var trackByFn = trackBy && $parse(trackBy);

    // Get the value by which we are going to track the option
    // if we have a trackFn then use that (passing scope and locals)
    // otherwise just hash the given viewValue
    var getTrackByValueFn = trackBy ?
                              function(value, locals) { return trackByFn(scope, locals); } :
                              function getHashOfValue(value) { return hashKey(value); };
    var getTrackByValue = function(value, key) {
      return getTrackByValueFn(value, getLocals(value, key));
    };

    var displayFn = $parse(match[2] || match[1]);
    var groupByFn = $parse(match[3] || '');
    var disableWhenFn = $parse(match[4] || '');
    var valuesFn = $parse(match[8]);

    var locals = {};
    var getLocals = keyName ? function(value, key) {
      locals[keyName] = key;
      locals[valueName] = value;
      return locals;
    } : function(value) {
      locals[valueName] = value;
      return locals;
    };


    function Option(selectValue, viewValue, label, group, disabled) {
      this.selectValue = selectValue;
      this.viewValue = viewValue;
      this.label = label;
      this.group = group;
      this.disabled = disabled;
    }

    function getOptionValuesKeys(optionValues) {
      var optionValuesKeys;

      if (!keyName && isArrayLike(optionValues)) {
        optionValuesKeys = optionValues;
      } else {
        // if object, extract keys, in enumeration order, unsorted
        optionValuesKeys = [];
        for (var itemKey in optionValues) {
          if (optionValues.hasOwnProperty(itemKey) && itemKey.charAt(0) !== '$') {
            optionValuesKeys.push(itemKey);
          }
        }
      }
      return optionValuesKeys;
    }

    return {
      trackBy: trackBy,
      getTrackByValue: getTrackByValue,
      getWatchables: $parse(valuesFn, function(optionValues) {
        // Create a collection of things that we would like to watch (watchedArray)
        // so that they can all be watched using a single $watchCollection
        // that only runs the handler once if anything changes
        var watchedArray = [];
        optionValues = optionValues || [];

        var optionValuesKeys = getOptionValuesKeys(optionValues);
        var optionValuesLength = optionValuesKeys.length;
        for (var index = 0; index < optionValuesLength; index++) {
          var key = (optionValues === optionValuesKeys) ? index : optionValuesKeys[index];
          var value = optionValues[key];

          var locals = getLocals(value, key);
          var selectValue = getTrackByValueFn(value, locals);
          watchedArray.push(selectValue);

          // Only need to watch the displayFn if there is a specific label expression
          if (match[2] || match[1]) {
            var label = displayFn(scope, locals);
            watchedArray.push(label);
          }

          // Only need to watch the disableWhenFn if there is a specific disable expression
          if (match[4]) {
            var disableWhen = disableWhenFn(scope, locals);
            watchedArray.push(disableWhen);
          }
        }
        return watchedArray;
      }),

      getOptions: function() {

        var optionItems = [];
        var selectValueMap = {};

        // The option values were already computed in the `getWatchables` fn,
        // which must have been called to trigger `getOptions`
        var optionValues = valuesFn(scope) || [];
        var optionValuesKeys = getOptionValuesKeys(optionValues);
        var optionValuesLength = optionValuesKeys.length;

        for (var index = 0; index < optionValuesLength; index++) {
          var key = (optionValues === optionValuesKeys) ? index : optionValuesKeys[index];
          var value = optionValues[key];
          var locals = getLocals(value, key);
          var viewValue = viewValueFn(scope, locals);
          var selectValue = getTrackByValueFn(viewValue, locals);
          var label = displayFn(scope, locals);
          var group = groupByFn(scope, locals);
          var disabled = disableWhenFn(scope, locals);
          var optionItem = new Option(selectValue, viewValue, label, group, disabled);

          optionItems.push(optionItem);
          selectValueMap[selectValue] = optionItem;
        }

        return {
          items: optionItems,
          selectValueMap: selectValueMap,
          getOptionFromViewValue: function(value) {
            return selectValueMap[getTrackByValue(value)];
          },
          getViewValueFromOption: function(option) {
            // If the viewValue could be an object that may be mutated by the application,
            // we need to make a copy and not return the reference to the value on the option.
            return trackBy ? copy(option.viewValue) : option.viewValue;
          }
        };
      }
    };
  }


  // Support: IE 9 only
  // We can't just jqLite('<option>') since jqLite is not smart enough
  // to create it in <select> and IE barfs otherwise.
  var optionTemplate = window.document.createElement('option'),
      optGroupTemplate = window.document.createElement('optgroup');

    function ngOptionsPostLink(scope, selectElement, attr, ctrls) {

      var selectCtrl = ctrls[0];
      var ngModelCtrl = ctrls[1];
      var multiple = attr.multiple;

      // The emptyOption allows the application developer to provide their own custom "empty"
      // option when the viewValue does not match any of the option values.
      for (var i = 0, children = selectElement.children(), ii = children.length; i < ii; i++) {
        if (children[i].value === '') {
          selectCtrl.hasEmptyOption = true;
          selectCtrl.emptyOption = children.eq(i);
          break;
        }
      }

      // The empty option will be compiled and rendered before we first generate the options
      selectElement.empty();

      var providedEmptyOption = !!selectCtrl.emptyOption;

      var unknownOption = jqLite(optionTemplate.cloneNode(false));
      unknownOption.val('?');

      var options;
      var ngOptions = parseOptionsExpression(attr.ngOptions, selectElement, scope);
      // This stores the newly created options before they are appended to the select.
      // Since the contents are removed from the fragment when it is appended,
      // we only need to create it once.
      var listFragment = $document[0].createDocumentFragment();

      // Overwrite the implementation. ngOptions doesn't use hashes
      selectCtrl.generateUnknownOptionValue = function(val) {
        return '?';
      };

      // Update the controller methods for multiple selectable options
      if (!multiple) {

        selectCtrl.writeValue = function writeNgOptionsValue(value) {
          // The options might not be defined yet when ngModel tries to render
          if (!options) return;

          var selectedOption = selectElement[0].options[selectElement[0].selectedIndex];
          var option = options.getOptionFromViewValue(value);

          // Make sure to remove the selected attribute from the previously selected option
          // Otherwise, screen readers might get confused
          if (selectedOption) selectedOption.removeAttribute('selected');

          if (option) {
            // Don't update the option when it is already selected.
            // For example, the browser will select the first option by default. In that case,
            // most properties are set automatically - except the `selected` attribute, which we
            // set always

            if (selectElement[0].value !== option.selectValue) {
              selectCtrl.removeUnknownOption();

              selectElement[0].value = option.selectValue;
              option.element.selected = true;
            }

            option.element.setAttribute('selected', 'selected');
          } else {
            selectCtrl.selectUnknownOrEmptyOption(value);
          }
        };

        selectCtrl.readValue = function readNgOptionsValue() {

          var selectedOption = options.selectValueMap[selectElement.val()];

          if (selectedOption && !selectedOption.disabled) {
            selectCtrl.unselectEmptyOption();
            selectCtrl.removeUnknownOption();
            return options.getViewValueFromOption(selectedOption);
          }
          return null;
        };

        // If we are using `track by` then we must watch the tracked value on the model
        // since ngModel only watches for object identity change
        // FIXME: When a user selects an option, this watch will fire needlessly
        if (ngOptions.trackBy) {
          scope.$watch(
            function() { return ngOptions.getTrackByValue(ngModelCtrl.$viewValue); },
            function() { ngModelCtrl.$render(); }
          );
        }

      } else {

        selectCtrl.writeValue = function writeNgOptionsMultiple(values) {
          // The options might not be defined yet when ngModel tries to render
          if (!options) return;

          // Only set `<option>.selected` if necessary, in order to prevent some browsers from
          // scrolling to `<option>` elements that are outside the `<select>` element's viewport.
          var selectedOptions = values && values.map(getAndUpdateSelectedOption) || [];

          options.items.forEach(function(option) {
            if (option.element.selected && !includes(selectedOptions, option)) {
              option.element.selected = false;
            }
          });
        };


        selectCtrl.readValue = function readNgOptionsMultiple() {
          var selectedValues = selectElement.val() || [],
              selections = [];

          forEach(selectedValues, function(value) {
            var option = options.selectValueMap[value];
            if (option && !option.disabled) selections.push(options.getViewValueFromOption(option));
          });

          return selections;
        };

        // If we are using `track by` then we must watch these tracked values on the model
        // since ngModel only watches for object identity change
        if (ngOptions.trackBy) {

          scope.$watchCollection(function() {
            if (isArray(ngModelCtrl.$viewValue)) {
              return ngModelCtrl.$viewValue.map(function(value) {
                return ngOptions.getTrackByValue(value);
              });
            }
          }, function() {
            ngModelCtrl.$render();
          });

        }
      }

      if (providedEmptyOption) {

        // compile the element since there might be bindings in it
        $compile(selectCtrl.emptyOption)(scope);

        selectElement.prepend(selectCtrl.emptyOption);

        if (selectCtrl.emptyOption[0].nodeType === NODE_TYPE_COMMENT) {
          // This means the empty option has currently no actual DOM node, probably because
          // it has been modified by a transclusion directive.
          selectCtrl.hasEmptyOption = false;

          // Redefine the registerOption function, which will catch
          // options that are added by ngIf etc. (rendering of the node is async because of
          // lazy transclusion)
          selectCtrl.registerOption = function(optionScope, optionEl) {
            if (optionEl.val() === '') {
              selectCtrl.hasEmptyOption = true;
              selectCtrl.emptyOption = optionEl;
              selectCtrl.emptyOption.removeClass('ng-scope');
              // This ensures the new empty option is selected if previously no option was selected
              ngModelCtrl.$render();

              optionEl.on('$destroy', function() {
                var needsRerender = selectCtrl.$isEmptyOptionSelected();

                selectCtrl.hasEmptyOption = false;
                selectCtrl.emptyOption = undefined;

                if (needsRerender) ngModelCtrl.$render();
              });
            }
          };

        } else {
          // remove the class, which is added automatically because we recompile the element and it
          // becomes the compilation root
          selectCtrl.emptyOption.removeClass('ng-scope');
        }

      }

      // We will re-render the option elements if the option values or labels change
      scope.$watchCollection(ngOptions.getWatchables, updateOptions);

      // ------------------------------------------------------------------ //

      function addOptionElement(option, parent) {
        var optionElement = optionTemplate.cloneNode(false);
        parent.appendChild(optionElement);
        updateOptionElement(option, optionElement);
      }

      function getAndUpdateSelectedOption(viewValue) {
        var option = options.getOptionFromViewValue(viewValue);
        var element = option && option.element;

        if (element && !element.selected) element.selected = true;

        return option;
      }

      function updateOptionElement(option, element) {
        option.element = element;
        element.disabled = option.disabled;
        // Support: IE 11 only, Edge 12-13 only
        // NOTE: The label must be set before the value, otherwise IE 11 & Edge create unresponsive
        // selects in certain circumstances when multiple selects are next to each other and display
        // the option list in listbox style, i.e. the select is [multiple], or specifies a [size].
        // See https://github.com/angular/angular.js/issues/11314 for more info.
        // This is unfortunately untestable with unit / e2e tests
        if (option.label !== element.label) {
          element.label = option.label;
          element.textContent = option.label;
        }
        element.value = option.selectValue;
      }

      function updateOptions() {
        var previousValue = options && selectCtrl.readValue();

        // We must remove all current options, but cannot simply set innerHTML = null
        // since the providedEmptyOption might have an ngIf on it that inserts comments which we
        // must preserve.
        // Instead, iterate over the current option elements and remove them or their optgroup
        // parents
        if (options) {

          for (var i = options.items.length - 1; i >= 0; i--) {
            var option = options.items[i];
            if (isDefined(option.group)) {
              jqLiteRemove(option.element.parentNode);
            } else {
              jqLiteRemove(option.element);
            }
          }
        }

        options = ngOptions.getOptions();

        var groupElementMap = {};

        options.items.forEach(function addOption(option) {
          var groupElement;

          if (isDefined(option.group)) {

            // This option is to live in a group
            // See if we have already created this group
            groupElement = groupElementMap[option.group];

            if (!groupElement) {

              groupElement = optGroupTemplate.cloneNode(false);
              listFragment.appendChild(groupElement);

              // Update the label on the group element
              // "null" is special cased because of Safari
              groupElement.label = option.group === null ? 'null' : option.group;

              // Store it for use later
              groupElementMap[option.group] = groupElement;
            }

            addOptionElement(option, groupElement);

          } else {

            // This option is not in a group
            addOptionElement(option, listFragment);
          }
        });

        selectElement[0].appendChild(listFragment);

        ngModelCtrl.$render();

        // Check to see if the value has changed due to the update to the options
        if (!ngModelCtrl.$isEmpty(previousValue)) {
          var nextValue = selectCtrl.readValue();
          var isNotPrimitive = ngOptions.trackBy || multiple;
          if (isNotPrimitive ? !equals(previousValue, nextValue) : previousValue !== nextValue) {
            ngModelCtrl.$setViewValue(nextValue);
            ngModelCtrl.$render();
          }
        }
      }
  }

  return {
    restrict: 'A',
    terminal: true,
    require: ['select', 'ngModel'],
    link: {
      pre: function ngOptionsPreLink(scope, selectElement, attr, ctrls) {
        // Deactivate the SelectController.register method to prevent
        // option directives from accidentally registering themselves
        // (and unwanted $destroy handlers etc.)
        ctrls[0].registerOption = noop;
      },
      post: ngOptionsPostLink
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngPluralize
 * @restrict EA
 *
 * @description
 * `ngPluralize` is a directive that displays messages according to en-US localization rules.
 * These rules are bundled with angular.js, but can be overridden
 * (see {@link guide/i18n AngularJS i18n} dev guide). You configure ngPluralize directive
 * by specifying the mappings between
 * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
 * and the strings to be displayed.
 *
 * ## Plural categories and explicit number rules
 * There are two
 * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
 * in AngularJS's default en-US locale: "one" and "other".
 *
 * While a plural category may match many numbers (for example, in en-US locale, "other" can match
 * any number that is not 1), an explicit number rule can only match one number. For example, the
 * explicit number rule for "3" matches the number 3. There are examples of plural categories
 * and explicit number rules throughout the rest of this documentation.
 *
 * ## Configuring ngPluralize
 * You configure ngPluralize by providing 2 attributes: `count` and `when`.
 * You can also provide an optional attribute, `offset`.
 *
 * The value of the `count` attribute can be either a string or an {@link guide/expression
 * AngularJS expression}; these are evaluated on the current scope for its bound value.
 *
 * The `when` attribute specifies the mappings between plural categories and the actual
 * string to be displayed. The value of the attribute should be a JSON object.
 *
 * The following example shows how to configure ngPluralize:
 *
 * ```html
 * <ng-pluralize count="personCount"
                 when="{'0': 'Nobody is viewing.',
 *                      'one': '1 person is viewing.',
 *                      'other': '{} people are viewing.'}">
 * </ng-pluralize>
 *```
 *
 * In the example, `"0: Nobody is viewing."` is an explicit number rule. If you did not
 * specify this rule, 0 would be matched to the "other" category and "0 people are viewing"
 * would be shown instead of "Nobody is viewing". You can specify an explicit number rule for
 * other numbers, for example 12, so that instead of showing "12 people are viewing", you can
 * show "a dozen people are viewing".
 *
 * You can use a set of closed braces (`{}`) as a placeholder for the number that you want substituted
 * into pluralized strings. In the previous example, AngularJS will replace `{}` with
 * <span ng-non-bindable>`{{personCount}}`</span>. The closed braces `{}` is a placeholder
 * for <span ng-non-bindable>{{numberExpression}}</span>.
 *
 * If no rule is defined for a category, then an empty string is displayed and a warning is generated.
 * Note that some locales define more categories than `one` and `other`. For example, fr-fr defines `few` and `many`.
 *
 * ## Configuring ngPluralize with offset
 * The `offset` attribute allows further customization of pluralized text, which can result in
 * a better user experience. For example, instead of the message "4 people are viewing this document",
 * you might display "John, Kate and 2 others are viewing this document".
 * The offset attribute allows you to offset a number by any desired value.
 * Let's take a look at an example:
 *
 * ```html
 * <ng-pluralize count="personCount" offset=2
 *               when="{'0': 'Nobody is viewing.',
 *                      '1': '{{person1}} is viewing.',
 *                      '2': '{{person1}} and {{person2}} are viewing.',
 *                      'one': '{{person1}}, {{person2}} and one other person are viewing.',
 *                      'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
 * </ng-pluralize>
 * ```
 *
 * Notice that we are still using two plural categories(one, other), but we added
 * three explicit number rules 0, 1 and 2.
 * When one person, perhaps John, views the document, "John is viewing" will be shown.
 * When three people view the document, no explicit number rule is found, so
 * an offset of 2 is taken off 3, and AngularJS uses 1 to decide the plural category.
 * In this case, plural category 'one' is matched and "John, Mary and one other person are viewing"
 * is shown.
 *
 * Note that when you specify offsets, you must provide explicit number rules for
 * numbers from 0 up to and including the offset. If you use an offset of 3, for example,
 * you must provide explicit number rules for 0, 1, 2 and 3. You must also provide plural strings for
 * plural categories "one" and "other".
 *
 * @param {string|expression} count The variable to be bound to.
 * @param {string} when The mapping between plural category to its corresponding strings.
 * @param {number=} offset Offset to deduct from the total number.
 *
 * @example
    <example module="pluralizeExample" name="ng-pluralize">
      <file name="index.html">
        <script>
          angular.module('pluralizeExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.person1 = 'Igor';
              $scope.person2 = 'Misko';
              $scope.personCount = 1;
            }]);
        </script>
        <div ng-controller="ExampleController">
          <label>Person 1:<input type="text" ng-model="person1" value="Igor" /></label><br/>
          <label>Person 2:<input type="text" ng-model="person2" value="Misko" /></label><br/>
          <label>Number of People:<input type="text" ng-model="personCount" value="1" /></label><br/>

          <!--- Example with simple pluralization rules for en locale --->
          Without Offset:
          <ng-pluralize count="personCount"
                        when="{'0': 'Nobody is viewing.',
                               'one': '1 person is viewing.',
                               'other': '{} people are viewing.'}">
          </ng-pluralize><br>

          <!--- Example with offset --->
          With Offset(2):
          <ng-pluralize count="personCount" offset=2
                        when="{'0': 'Nobody is viewing.',
                               '1': '{{person1}} is viewing.',
                               '2': '{{person1}} and {{person2}} are viewing.',
                               'one': '{{person1}}, {{person2}} and one other person are viewing.',
                               'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
          </ng-pluralize>
        </div>
      </file>
      <file name="protractor.js" type="protractor">
        it('should show correct pluralized string', function() {
          var withoutOffset = element.all(by.css('ng-pluralize')).get(0);
          var withOffset = element.all(by.css('ng-pluralize')).get(1);
          var countInput = element(by.model('personCount'));

          expect(withoutOffset.getText()).toEqual('1 person is viewing.');
          expect(withOffset.getText()).toEqual('Igor is viewing.');

          countInput.clear();
          countInput.sendKeys('0');

          expect(withoutOffset.getText()).toEqual('Nobody is viewing.');
          expect(withOffset.getText()).toEqual('Nobody is viewing.');

          countInput.clear();
          countInput.sendKeys('2');

          expect(withoutOffset.getText()).toEqual('2 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor and Misko are viewing.');

          countInput.clear();
          countInput.sendKeys('3');

          expect(withoutOffset.getText()).toEqual('3 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor, Misko and one other person are viewing.');

          countInput.clear();
          countInput.sendKeys('4');

          expect(withoutOffset.getText()).toEqual('4 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor, Misko and 2 other people are viewing.');
        });
        it('should show data-bound names', function() {
          var withOffset = element.all(by.css('ng-pluralize')).get(1);
          var personCount = element(by.model('personCount'));
          var person1 = element(by.model('person1'));
          var person2 = element(by.model('person2'));
          personCount.clear();
          personCount.sendKeys('4');
          person1.clear();
          person1.sendKeys('Di');
          person2.clear();
          person2.sendKeys('Vojta');
          expect(withOffset.getText()).toEqual('Di, Vojta and 2 other people are viewing.');
        });
      </file>
    </example>
 */
var ngPluralizeDirective = ['$locale', '$interpolate', '$log', function($locale, $interpolate, $log) {
  var BRACE = /{}/g,
      IS_WHEN = /^when(Minus)?(.+)$/;

  return {
    link: function(scope, element, attr) {
      var numberExp = attr.count,
          whenExp = attr.$attr.when && element.attr(attr.$attr.when), // we have {{}} in attrs
          offset = attr.offset || 0,
          whens = scope.$eval(whenExp) || {},
          whensExpFns = {},
          startSymbol = $interpolate.startSymbol(),
          endSymbol = $interpolate.endSymbol(),
          braceReplacement = startSymbol + numberExp + '-' + offset + endSymbol,
          watchRemover = angular.noop,
          lastCount;

      forEach(attr, function(expression, attributeName) {
        var tmpMatch = IS_WHEN.exec(attributeName);
        if (tmpMatch) {
          var whenKey = (tmpMatch[1] ? '-' : '') + lowercase(tmpMatch[2]);
          whens[whenKey] = element.attr(attr.$attr[attributeName]);
        }
      });
      forEach(whens, function(expression, key) {
        whensExpFns[key] = $interpolate(expression.replace(BRACE, braceReplacement));

      });

      scope.$watch(numberExp, function ngPluralizeWatchAction(newVal) {
        var count = parseFloat(newVal);
        var countIsNaN = isNumberNaN(count);

        if (!countIsNaN && !(count in whens)) {
          // If an explicit number rule such as 1, 2, 3... is defined, just use it.
          // Otherwise, check it against pluralization rules in $locale service.
          count = $locale.pluralCat(count - offset);
        }

        // If both `count` and `lastCount` are NaN, we don't need to re-register a watch.
        // In JS `NaN !== NaN`, so we have to explicitly check.
        if ((count !== lastCount) && !(countIsNaN && isNumberNaN(lastCount))) {
          watchRemover();
          var whenExpFn = whensExpFns[count];
          if (isUndefined(whenExpFn)) {
            if (newVal != null) {
              $log.debug('ngPluralize: no rule defined for \'' + count + '\' in ' + whenExp);
            }
            watchRemover = noop;
            updateElementText();
          } else {
            watchRemover = scope.$watch(whenExpFn, updateElementText);
          }
          lastCount = count;
        }
      });

      function updateElementText(newText) {
        element.text(newText || '');
      }
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngRef
 * @restrict A
 *
 * @description
 * The `ngRef` attribute tells AngularJS to assign the controller of a component (or a directive)
 * to the given property in the current scope. It is also possible to add the jqlite-wrapped DOM
 * element to the scope.
 *
 * If the element with `ngRef` is destroyed `null` is assigned to the property.
 *
 * Note that if you want to assign from a child into the parent scope, you must initialize the
 * target property on the parent scope, otherwise `ngRef` will assign on the child scope.
 * This commonly happens when assigning elements or components wrapped in {@link ngIf} or
 * {@link ngRepeat}. See the second example below.
 *
 *
 * @element ANY
 * @param {string} ngRef property name - A valid AngularJS expression identifier to which the
 *                       controller or jqlite-wrapped DOM element will be bound.
 * @param {string=} ngRefRead read value - The name of a directive (or component) on this element,
 *                            or the special string `$element`. If a name is provided, `ngRef` will
 *                            assign the matching controller. If `$element` is provided, the element
 *                            itself is assigned (even if a controller is available).
 *
 *
 * @example
 * ### Simple toggle
 * This example shows how the controller of the component toggle
 * is reused in the template through the scope to use its logic.
 * <example name="ng-ref-component" module="myApp">
 *   <file name="index.html">
 *     <my-toggle ng-ref="myToggle"></my-toggle>
 *     <button ng-click="myToggle.toggle()">Toggle</button>
 *     <div ng-show="myToggle.isOpen()">
 *       You are using a component in the same template to show it.
 *     </div>
 *   </file>
 *   <file name="index.js">
 *     angular.module('myApp', [])
 *     .component('myToggle', {
 *       controller: function ToggleController() {
 *         var opened = false;
 *         this.isOpen = function() { return opened; };
 *         this.toggle = function() { opened = !opened; };
 *       }
 *     });
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *      it('should publish the toggle into the scope', function() {
 *        var toggle = element(by.buttonText('Toggle'));
 *        expect(toggle.evaluate('myToggle.isOpen()')).toEqual(false);
 *        toggle.click();
 *        expect(toggle.evaluate('myToggle.isOpen()')).toEqual(true);
 *      });
 *   </file>
 * </example>
 *
 * @example
 * ### ngRef inside scopes
 * This example shows how `ngRef` works with child scopes. The `ngRepeat`-ed `myWrapper` components
 * are assigned to the scope of `myRoot`, because the `toggles` property has been initialized.
 * The repeated `myToggle` components are published to the child scopes created by `ngRepeat`.
 * `ngIf` behaves similarly - the assignment of `myToggle` happens in the `ngIf` child scope,
 * because the target property has not been initialized on the `myRoot` component controller.
 *
 * <example name="ng-ref-scopes" module="myApp">
 *   <file name="index.html">
 *     <my-root></my-root>
 *   </file>
 *   <file name="index.js">
 *     angular.module('myApp', [])
 *     .component('myRoot', {
 *       templateUrl: 'root.html',
 *       controller: function() {
 *         this.wrappers = []; // initialize the array so that the wrappers are assigned into the parent scope
 *       }
 *     })
 *     .component('myToggle', {
 *       template: '<strong>myToggle</strong><button ng-click="$ctrl.toggle()" ng-transclude></button>',
 *       transclude: true,
 *       controller: function ToggleController() {
 *         var opened = false;
 *         this.isOpen = function() { return opened; };
 *         this.toggle = function() { opened = !opened; };
 *       }
 *     })
 *     .component('myWrapper', {
 *       transclude: true,
 *       template: '<strong>myWrapper</strong>' +
 *         '<div>ngRepeatToggle.isOpen(): {{$ctrl.ngRepeatToggle.isOpen() | json}}</div>' +
 *         '<my-toggle ng-ref="$ctrl.ngRepeatToggle"><ng-transclude></ng-transclude></my-toggle>'
 *     });
 *   </file>
 *   <file name="root.html">
 *     <strong>myRoot</strong>
 *     <my-toggle ng-ref="$ctrl.outerToggle">Outer Toggle</my-toggle>
 *     <div>outerToggle.isOpen(): {{$ctrl.outerToggle.isOpen() | json}}</div>
 *     <div><em>wrappers assigned to root</em><br>
 *     <div ng-repeat="wrapper in $ctrl.wrappers">
 *       wrapper.ngRepeatToggle.isOpen(): {{wrapper.ngRepeatToggle.isOpen() | json}}
 *     </div>
 *
 *     <ul>
 *       <li ng-repeat="(index, value) in [1,2,3]">
 *         <strong>ngRepeat</strong>
 *         <div>outerToggle.isOpen(): {{$ctrl.outerToggle.isOpen() | json}}</div>
 *         <my-wrapper ng-ref="$ctrl.wrappers[index]">ngRepeat Toggle {{$index + 1}}</my-wrapper>
 *       </li>
 *     </ul>
 *
 *     <div>ngIfToggle.isOpen(): {{ngIfToggle.isOpen()}} // This is always undefined because it's
 *       assigned to the child scope created by ngIf.
 *     </div>
 *     <div ng-if="true">
          <strong>ngIf</strong>
 *        <my-toggle ng-ref="ngIfToggle">ngIf Toggle</my-toggle>
 *        <div>ngIfToggle.isOpen(): {{ngIfToggle.isOpen() | json}}</div>
 *        <div>outerToggle.isOpen(): {{$ctrl.outerToggle.isOpen() | json}}</div>
 *     </div>
 *   </file>
 *   <file name="styles.css">
 *     ul {
 *       list-style: none;
 *       padding-left: 0;
 *     }
 *
 *     li[ng-repeat] {
 *       background: lightgreen;
 *       padding: 8px;
 *       margin: 8px;
 *     }
 *
 *     [ng-if] {
 *       background: lightgrey;
 *       padding: 8px;
 *     }
 *
 *     my-root {
 *       background: lightgoldenrodyellow;
 *       padding: 8px;
 *       display: block;
 *     }
 *
 *     my-wrapper {
 *       background: lightsalmon;
 *       padding: 8px;
 *       display: block;
 *     }
 *
 *     my-toggle {
 *       background: lightblue;
 *       padding: 8px;
 *       display: block;
 *     }
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *      var OuterToggle = function() {
 *        this.toggle = function() {
 *          element(by.buttonText('Outer Toggle')).click();
 *        };
 *        this.isOpen = function() {
 *          return element.all(by.binding('outerToggle.isOpen()')).first().getText();
 *        };
 *      };
 *      var NgRepeatToggle = function(i) {
 *        var parent = element.all(by.repeater('(index, value) in [1,2,3]')).get(i - 1);
 *        this.toggle = function() {
 *          element(by.buttonText('ngRepeat Toggle ' + i)).click();
 *        };
 *        this.isOpen = function() {
 *          return parent.element(by.binding('ngRepeatToggle.isOpen() | json')).getText();
 *        };
 *        this.isOuterOpen = function() {
 *          return parent.element(by.binding('outerToggle.isOpen() | json')).getText();
 *        };
 *      };
 *      var NgRepeatToggles = function() {
 *        var toggles = [1,2,3].map(function(i) { return new NgRepeatToggle(i); });
 *        this.forEach = function(fn) {
 *          toggles.forEach(fn);
 *        };
 *        this.isOuterOpen = function(i) {
 *          return toggles[i - 1].isOuterOpen();
 *        };
 *      };
 *      var NgIfToggle = function() {
 *        var parent = element(by.css('[ng-if]'));
 *        this.toggle = function() {
 *          element(by.buttonText('ngIf Toggle')).click();
 *        };
 *        this.isOpen = function() {
 *          return by.binding('ngIfToggle.isOpen() | json').getText();
 *        };
 *        this.isOuterOpen = function() {
 *          return parent.element(by.binding('outerToggle.isOpen() | json')).getText();
 *        };
 *      };
 *
 *      it('should toggle the outer toggle', function() {
 *        var outerToggle = new OuterToggle();
 *        expect(outerToggle.isOpen()).toEqual('outerToggle.isOpen(): false');
 *        outerToggle.toggle();
 *        expect(outerToggle.isOpen()).toEqual('outerToggle.isOpen(): true');
 *      });
 *
 *      it('should toggle all outer toggles', function() {
 *        var outerToggle = new OuterToggle();
 *        var repeatToggles = new NgRepeatToggles();
 *        var ifToggle = new NgIfToggle();
 *        expect(outerToggle.isOpen()).toEqual('outerToggle.isOpen(): false');
 *        expect(repeatToggles.isOuterOpen(1)).toEqual('outerToggle.isOpen(): false');
 *        expect(repeatToggles.isOuterOpen(2)).toEqual('outerToggle.isOpen(): false');
 *        expect(repeatToggles.isOuterOpen(3)).toEqual('outerToggle.isOpen(): false');
 *        expect(ifToggle.isOuterOpen()).toEqual('outerToggle.isOpen(): false');
 *        outerToggle.toggle();
 *        expect(outerToggle.isOpen()).toEqual('outerToggle.isOpen(): true');
 *        expect(repeatToggles.isOuterOpen(1)).toEqual('outerToggle.isOpen(): true');
 *        expect(repeatToggles.isOuterOpen(2)).toEqual('outerToggle.isOpen(): true');
 *        expect(repeatToggles.isOuterOpen(3)).toEqual('outerToggle.isOpen(): true');
 *        expect(ifToggle.isOuterOpen()).toEqual('outerToggle.isOpen(): true');
 *      });
 *
 *      it('should toggle each repeat iteration separately', function() {
 *        var repeatToggles = new NgRepeatToggles();
 *
 *        repeatToggles.forEach(function(repeatToggle) {
 *          expect(repeatToggle.isOpen()).toEqual('ngRepeatToggle.isOpen(): false');
 *          expect(repeatToggle.isOuterOpen()).toEqual('outerToggle.isOpen(): false');
 *          repeatToggle.toggle();
 *          expect(repeatToggle.isOpen()).toEqual('ngRepeatToggle.isOpen(): true');
 *          expect(repeatToggle.isOuterOpen()).toEqual('outerToggle.isOpen(): false');
 *        });
 *      });
 *   </file>
 * </example>
 *
 */

var ngRefMinErr = minErr('ngRef');

var ngRefDirective = ['$parse', function($parse) {
  return {
    priority: -1, // Needed for compatibility with element transclusion on the same element
    restrict: 'A',
    compile: function(tElement, tAttrs) {
      // Get the expected controller name, converts <data-some-thing> into "someThing"
      var controllerName = directiveNormalize(nodeName_(tElement));

      // Get the expression for value binding
      var getter = $parse(tAttrs.ngRef);
      var setter = getter.assign || function() {
        throw ngRefMinErr('nonassign', 'Expression in ngRef="{0}" is non-assignable!', tAttrs.ngRef);
      };

      return function(scope, element, attrs) {
        var refValue;

        if (attrs.hasOwnProperty('ngRefRead')) {
          if (attrs.ngRefRead === '$element') {
            refValue = element;
          } else {
            refValue = element.data('$' + attrs.ngRefRead + 'Controller');

            if (!refValue) {
              throw ngRefMinErr(
                'noctrl',
                'The controller for ngRefRead="{0}" could not be found on ngRef="{1}"',
                attrs.ngRefRead,
                tAttrs.ngRef
              );
            }
          }
        } else {
          refValue = element.data('$' + controllerName + 'Controller');
        }

        refValue = refValue || element;

        setter(scope, refValue);

        // when the element is removed, remove it (nullify it)
        element.on('$destroy', function() {
          // only remove it if value has not changed,
          // because animations (and other procedures) may duplicate elements
          if (getter(scope) === refValue) {
            setter(scope, null);
          }
        });
      };
    }
  };
}];

/* exported ngRepeatDirective */

/**
 * @ngdoc directive
 * @name ngRepeat
 * @multiElement
 * @restrict A
 *
 * @description
 * The `ngRepeat` directive instantiates a template once per item from a collection. Each template
 * instance gets its own scope, where the given loop variable is set to the current collection item,
 * and `$index` is set to the item index or key.
 *
 * Special properties are exposed on the local scope of each template instance, including:
 *
 * | Variable  | Type            | Details                                                                     |
 * |-----------|-----------------|-----------------------------------------------------------------------------|
 * | `$index`  | {@type number}  | iterator offset of the repeated element (0..length-1)                       |
 * | `$first`  | {@type boolean} | true if the repeated element is first in the iterator.                      |
 * | `$middle` | {@type boolean} | true if the repeated element is between the first and last in the iterator. |
 * | `$last`   | {@type boolean} | true if the repeated element is last in the iterator.                       |
 * | `$even`   | {@type boolean} | true if the iterator position `$index` is even (otherwise false).           |
 * | `$odd`    | {@type boolean} | true if the iterator position `$index` is odd (otherwise false).            |
 *
 * <div class="alert alert-info">
 *   Creating aliases for these properties is possible with {@link ng.directive:ngInit `ngInit`}.
 *   This may be useful when, for instance, nesting ngRepeats.
 * </div>
 *
 *
 * ## Iterating over object properties
 *
 * It is possible to get `ngRepeat` to iterate over the properties of an object using the following
 * syntax:
 *
 * ```js
 * <div ng-repeat="(key, value) in myObj"> ... </div>
 * ```
 *
 * However, there are a few limitations compared to array iteration:
 *
 * - The JavaScript specification does not define the order of keys
 *   returned for an object, so AngularJS relies on the order returned by the browser
 *   when running `for key in myObj`. Browsers generally follow the strategy of providing
 *   keys in the order in which they were defined, although there are exceptions when keys are deleted
 *   and reinstated. See the
 *   [MDN page on `delete` for more info](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/delete#Cross-browser_notes).
 *
 * - `ngRepeat` will silently *ignore* object keys starting with `$`, because
 *   it's a prefix used by AngularJS for public (`$`) and private (`$$`) properties.
 *
 * - The built-in filters {@link ng.orderBy orderBy} and {@link ng.filter filter} do not work with
 *   objects, and will throw an error if used with one.
 *
 * If you are hitting any of these limitations, the recommended workaround is to convert your object into an array
 * that is sorted into the order that you prefer before providing it to `ngRepeat`. You could
 * do this with a filter such as [toArrayFilter](http://ngmodules.org/modules/angular-toArrayFilter)
 * or implement a `$watch` on the object yourself.
 *
 *
 * ## Tracking and Duplicates
 *
 * `ngRepeat` uses {@link $rootScope.Scope#$watchCollection $watchCollection} to detect changes in
 * the collection. When a change happens, `ngRepeat` then makes the corresponding changes to the DOM:
 *
 * * When an item is added, a new instance of the template is added to the DOM.
 * * When an item is removed, its template instance is removed from the DOM.
 * * When items are reordered, their respective templates are reordered in the DOM.
 *
 * To minimize creation of DOM elements, `ngRepeat` uses a function
 * to "keep track" of all items in the collection and their corresponding DOM elements.
 * For example, if an item is added to the collection, `ngRepeat` will know that all other items
 * already have DOM elements, and will not re-render them.
 *
 * All different types of tracking functions, their syntax, and their support for duplicate
 * items in collections can be found in the
 * {@link ngRepeat#ngRepeat-arguments ngRepeat expression description}.
 *
 * <div class="alert alert-success">
 * **Best Practice:** If you are working with objects that have a unique identifier property, you
 * should track by this identifier instead of the object instance,
 * e.g. `item in items track by item.id`.
 * Should you reload your data later, `ngRepeat` will not have to rebuild the DOM elements for items
 * it has already rendered, even if the JavaScript objects in the collection have been substituted
 * for new ones. For large collections, this significantly improves rendering performance.
 * </div>
 *
 * ### Effects of DOM Element re-use
 *
 * When DOM elements are re-used, ngRepeat updates the scope for the element, which will
 * automatically update any active bindings on the template. However, other
 * functionality will not be updated, because the element is not re-created:
 *
 * - Directives are not re-compiled
 * - {@link guide/expression#one-time-binding one-time expressions} on the repeated template are not
 * updated if they have stabilized.
 *
 * The above affects all kinds of element re-use due to tracking, but may be especially visible
 * when tracking by `$index` due to the way ngRepeat re-uses elements.
 *
 * The following example shows the effects of different actions with tracking:

  <example module="ngRepeat" name="ngRepeat-tracking" deps="angular-animate.js" animations="true">
    <file name="script.js">
      angular.module('ngRepeat', ['ngAnimate']).controller('repeatController', function($scope) {
        var friends = [
          {name:'John', age:25},
          {name:'Mary', age:40},
          {name:'Peter', age:85}
        ];

        $scope.removeFirst = function() {
          $scope.friends.shift();
        };

        $scope.updateAge = function() {
          $scope.friends.forEach(function(el) {
            el.age = el.age + 5;
          });
        };

        $scope.copy = function() {
          $scope.friends = angular.copy($scope.friends);
        };

        $scope.reset = function() {
          $scope.friends = angular.copy(friends);
        };

        $scope.reset();
      });
    </file>
    <file name="index.html">
      <div ng-controller="repeatController">
        <ol>
          <li>When you click "Update Age", only the first list updates the age, because all others have
          a one-time binding on the age property. If you then click "Copy", the current friend list
          is copied, and now the second list updates the age, because the identity of the collection items
          has changed and the list must be re-rendered. The 3rd and 4th list stay the same, because all the
          items are already known according to their tracking functions.
          </li>
          <li>When you click "Remove First", the 4th list has the wrong age on both remaining items. This is
          due to tracking by $index: when the first collection item is removed, ngRepeat reuses the first
          DOM element for the new first collection item, and so on. Since the age property is one-time
          bound, the value remains from the collection item which was previously at this index.
          </li>
        </ol>

        <button ng-click="removeFirst()">Remove First</button>
        <button ng-click="updateAge()">Update Age</button>
        <button ng-click="copy()">Copy</button>
        <br><button ng-click="reset()">Reset List</button>
        <br>
        <code>track by $id(friend)</code> (default):
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends">
            {{friend.name}} is {{friend.age}} years old.
          </li>
        </ul>
        <code>track by $id(friend)</code> (default), with age one-time binding:
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends">
            {{friend.name}} is {{::friend.age}} years old.
          </li>
        </ul>
        <code>track by friend.name</code>, with age one-time binding:
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends track by friend.name">
            {{friend.name}}  is {{::friend.age}} years old.
          </li>
        </ul>
        <code>track by $index</code>, with age one-time binding:
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends track by $index">
            {{friend.name}} is {{::friend.age}} years old.
          </li>
        </ul>
      </div>
    </file>
    <file name="animations.css">
      .example-animate-container {
        background:white;
        border:1px solid black;
        list-style:none;
        margin:0;
        padding:0 10px;
      }

      .animate-repeat {
        line-height:30px;
        list-style:none;
        box-sizing:border-box;
      }

      .animate-repeat.ng-move,
      .animate-repeat.ng-enter,
      .animate-repeat.ng-leave {
        transition:all linear 0.5s;
      }

      .animate-repeat.ng-leave.ng-leave-active,
      .animate-repeat.ng-move,
      .animate-repeat.ng-enter {
        opacity:0;
        max-height:0;
      }

      .animate-repeat.ng-leave,
      .animate-repeat.ng-move.ng-move-active,
      .animate-repeat.ng-enter.ng-enter-active {
        opacity:1;
        max-height:30px;
      }
    </file>
  </example>

 *
 * ## Special repeat start and end points
 * To repeat a series of elements instead of just one parent element, ngRepeat (as well as other ng directives) supports extending
 * the range of the repeater by defining explicit start and end points by using **ng-repeat-start** and **ng-repeat-end** respectively.
 * The **ng-repeat-start** directive works the same as **ng-repeat**, but will repeat all the HTML code (including the tag it's defined on)
 * up to and including the ending HTML tag where **ng-repeat-end** is placed.
 *
 * The example below makes use of this feature:
 * ```html
 *   <header ng-repeat-start="item in items">
 *     Header {{ item }}
 *   </header>
 *   <div class="body">
 *     Body {{ item }}
 *   </div>
 *   <footer ng-repeat-end>
 *     Footer {{ item }}
 *   </footer>
 * ```
 *
 * And with an input of {@type ['A','B']} for the items variable in the example above, the output will evaluate to:
 * ```html
 *   <header>
 *     Header A
 *   </header>
 *   <div class="body">
 *     Body A
 *   </div>
 *   <footer>
 *     Footer A
 *   </footer>
 *   <header>
 *     Header B
 *   </header>
 *   <div class="body">
 *     Body B
 *   </div>
 *   <footer>
 *     Footer B
 *   </footer>
 * ```
 *
 * The custom start and end points for ngRepeat also support all other HTML directive syntax flavors provided in AngularJS (such
 * as **data-ng-repeat-start**, **x-ng-repeat-start** and **ng:repeat-start**).
 *
 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#enter enter} | when a new item is added to the list or when an item is revealed after a filter |
 * | {@link ng.$animate#leave leave} | when an item is removed from the list or when an item is filtered out |
 * | {@link ng.$animate#move move } | when an adjacent item is filtered out causing a reorder or when the item contents are reordered |
 *
 * See the example below for defining CSS animations with ngRepeat.
 *
 * @element ANY
 * @scope
 * @priority 1000
 * @param {repeat_expression} ngRepeat The expression indicating how to enumerate a collection. These
 *   formats are currently supported:
 *
 *   * `variable in expression` – where variable is the user defined loop variable and `expression`
 *     is a scope expression giving the collection to enumerate.
 *
 *     For example: `album in artist.albums`.
 *
 *   * `(key, value) in expression` – where `key` and `value` can be any user defined identifiers,
 *     and `expression` is the scope expression giving the collection to enumerate.
 *
 *     For example: `(name, age) in {'adam':10, 'amalie':12}`.
 *
 *   * `variable in expression track by tracking_expression` – You can also provide an optional tracking expression
 *     which can be used to associate the objects in the collection with the DOM elements. If no tracking expression
 *     is specified, ng-repeat associates elements by identity. It is an error to have
 *     more than one tracking expression value resolve to the same key. (This would mean that two distinct objects are
 *     mapped to the same DOM element, which is not possible.)
 *
 *     *Default tracking: $id()*: `item in items` is equivalent to `item in items track by $id(item)`.
 *     This implies that the DOM elements will be associated by item identity in the collection.
 *
 *     The built-in `$id()` function can be used to assign a unique
 *     `$$hashKey` property to each item in the collection. This property is then used as a key to associated DOM elements
 *     with the corresponding item in the collection by identity. Moving the same object would move
 *     the DOM element in the same way in the DOM.
 *     Note that the default id function does not support duplicate primitive values (`number`, `string`),
 *     but supports duplictae non-primitive values (`object`) that are *equal* in shape.
 *
 *     *Custom Expression*: It is possible to use any AngularJS expression to compute the tracking
 *     id, for example with a function, or using a property on the collection items.
 *     `item in items track by item.id` is a typical pattern when the items have a unique identifier,
 *     e.g. database id. In this case the object identity does not matter. Two objects are considered
 *     equivalent as long as their `id` property is same.
 *     Tracking by unique identifier is the most performant way and should be used whenever possible.
 *
 *     *$index*: This special property tracks the collection items by their index, and
 *     re-uses the DOM elements that match that index, e.g. `item in items track by $index`. This can
 *     be used for a performance improvement if no unique identfier is available and the identity of
 *     the collection items cannot be easily computed. It also allows duplicates.
 *
 *     <div class="alert alert-warning">
 *       <strong>Note:</strong> Re-using DOM elements can have unforeseen effects. Read the
 *       {@link ngRepeat#tracking-and-duplicates section on tracking and duplicates} for
 *       more info.
 *     </div>
 *
 *     <div class="alert alert-warning">
 *       <strong>Note:</strong> the `track by` expression must come last - after any filters, and the alias expression:
 *       `item in items | filter:searchText as results  track by item.id`
 *     </div>
 *
 *   * `variable in expression as alias_expression` – You can also provide an optional alias expression which will then store the
 *     intermediate results of the repeater after the filters have been applied. Typically this is used to render a special message
 *     when a filter is active on the repeater, but the filtered result set is empty.
 *
 *     For example: `item in items | filter:x as results` will store the fragment of the repeated items as `results`, but only after
 *     the items have been processed through the filter.
 *
 *     Please note that `as [variable name]` is not an operator but rather a part of ngRepeat
 *     micro-syntax so it can be used only after all filters (and not as operator, inside an expression).
 *
 *     For example: `item in items | filter : x | orderBy : order | limitTo : limit as results track by item.id` .
 *
 * @example
 * This example uses `ngRepeat` to display a list of people. A filter is used to restrict the displayed
 * results by name or by age. New (entering) and removed (leaving) items are animated.
  <example module="ngRepeat" name="ngRepeat" deps="angular-animate.js" animations="true">
    <file name="index.html">
      <div ng-controller="repeatController">
        I have {{friends.length}} friends. They are:
        <input type="search" ng-model="q" placeholder="filter friends..." aria-label="filter friends" />
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends | filter:q as results track by friend.name">
            [{{$index + 1}}] {{friend.name}} who is {{friend.age}} years old.
          </li>
          <li class="animate-repeat" ng-if="results.length === 0">
            <strong>No results found...</strong>
          </li>
        </ul>
      </div>
    </file>
    <file name="script.js">
      angular.module('ngRepeat', ['ngAnimate']).controller('repeatController', function($scope) {
        $scope.friends = [
          {name:'John', age:25, gender:'boy'},
          {name:'Jessie', age:30, gender:'girl'},
          {name:'Johanna', age:28, gender:'girl'},
          {name:'Joy', age:15, gender:'girl'},
          {name:'Mary', age:28, gender:'girl'},
          {name:'Peter', age:95, gender:'boy'},
          {name:'Sebastian', age:50, gender:'boy'},
          {name:'Erika', age:27, gender:'girl'},
          {name:'Patrick', age:40, gender:'boy'},
          {name:'Samantha', age:60, gender:'girl'}
        ];
      });
    </file>
    <file name="animations.css">
      .example-animate-container {
        background:white;
        border:1px solid black;
        list-style:none;
        margin:0;
        padding:0 10px;
      }

      .animate-repeat {
        line-height:30px;
        list-style:none;
        box-sizing:border-box;
      }

      .animate-repeat.ng-move,
      .animate-repeat.ng-enter,
      .animate-repeat.ng-leave {
        transition:all linear 0.5s;
      }

      .animate-repeat.ng-leave.ng-leave-active,
      .animate-repeat.ng-move,
      .animate-repeat.ng-enter {
        opacity:0;
        max-height:0;
      }

      .animate-repeat.ng-leave,
      .animate-repeat.ng-move.ng-move-active,
      .animate-repeat.ng-enter.ng-enter-active {
        opacity:1;
        max-height:30px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var friends = element.all(by.repeater('friend in friends'));

      it('should render initial data set', function() {
        expect(friends.count()).toBe(10);
        expect(friends.get(0).getText()).toEqual('[1] John who is 25 years old.');
        expect(friends.get(1).getText()).toEqual('[2] Jessie who is 30 years old.');
        expect(friends.last().getText()).toEqual('[10] Samantha who is 60 years old.');
        expect(element(by.binding('friends.length')).getText())
            .toMatch("I have 10 friends. They are:");
      });

       it('should update repeater when filter predicate changes', function() {
         expect(friends.count()).toBe(10);

         element(by.model('q')).sendKeys('ma');

         expect(friends.count()).toBe(2);
         expect(friends.get(0).getText()).toEqual('[1] Mary who is 28 years old.');
         expect(friends.last().getText()).toEqual('[2] Samantha who is 60 years old.');
       });
      </file>
    </example>
 */
var ngRepeatDirective = ['$parse', '$animate', '$compile', function($parse, $animate, $compile) {
  var NG_REMOVED = '$$NG_REMOVED';
  var ngRepeatMinErr = minErr('ngRepeat');

  var updateScope = function(scope, index, valueIdentifier, value, keyIdentifier, key, arrayLength) {
    // TODO(perf): generate setters to shave off ~40ms or 1-1.5%
    scope[valueIdentifier] = value;
    if (keyIdentifier) scope[keyIdentifier] = key;
    scope.$index = index;
    scope.$first = (index === 0);
    scope.$last = (index === (arrayLength - 1));
    scope.$middle = !(scope.$first || scope.$last);
    // eslint-disable-next-line no-bitwise
    scope.$odd = !(scope.$even = (index & 1) === 0);
  };

  var getBlockStart = function(block) {
    return block.clone[0];
  };

  var getBlockEnd = function(block) {
    return block.clone[block.clone.length - 1];
  };

  var trackByIdArrayFn = function($scope, key, value) {
    return hashKey(value);
  };

  var trackByIdObjFn = function($scope, key) {
    return key;
  };

  return {
    restrict: 'A',
    multiElement: true,
    transclude: 'element',
    priority: 1000,
    terminal: true,
    $$tlb: true,
    compile: function ngRepeatCompile($element, $attr) {
      var expression = $attr.ngRepeat;
      var ngRepeatEndComment = $compile.$$createComment('end ngRepeat', expression);

      var match = expression.match(/^\s*([\s\S]+?)\s+in\s+([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+track\s+by\s+([\s\S]+?))?\s*$/);

      if (!match) {
        throw ngRepeatMinErr('iexp', 'Expected expression in form of \'_item_ in _collection_[ track by _id_]\' but got \'{0}\'.',
            expression);
      }

      var lhs = match[1];
      var rhs = match[2];
      var aliasAs = match[3];
      var trackByExp = match[4];

      match = lhs.match(/^(?:(\s*[$\w]+)|\(\s*([$\w]+)\s*,\s*([$\w]+)\s*\))$/);

      if (!match) {
        throw ngRepeatMinErr('iidexp', '\'_item_\' in \'_item_ in _collection_\' should be an identifier or \'(_key_, _value_)\' expression, but got \'{0}\'.',
            lhs);
      }
      var valueIdentifier = match[3] || match[1];
      var keyIdentifier = match[2];

      if (aliasAs && (!/^[$a-zA-Z_][$a-zA-Z0-9_]*$/.test(aliasAs) ||
          /^(null|undefined|this|\$index|\$first|\$middle|\$last|\$even|\$odd|\$parent|\$root|\$id)$/.test(aliasAs))) {
        throw ngRepeatMinErr('badident', 'alias \'{0}\' is invalid --- must be a valid JS identifier which is not a reserved name.',
          aliasAs);
      }

      var trackByIdExpFn;

      if (trackByExp) {
        var hashFnLocals = {$id: hashKey};
        var trackByExpGetter = $parse(trackByExp);

        trackByIdExpFn = function($scope, key, value, index) {
          // assign key, value, and $index to the locals so that they can be used in hash functions
          if (keyIdentifier) hashFnLocals[keyIdentifier] = key;
          hashFnLocals[valueIdentifier] = value;
          hashFnLocals.$index = index;
          return trackByExpGetter($scope, hashFnLocals);
        };
      }

      return function ngRepeatLink($scope, $element, $attr, ctrl, $transclude) {

        // Store a list of elements from previous run. This is a hash where key is the item from the
        // iterator, and the value is objects with following properties.
        //   - scope: bound scope
        //   - clone: previous element.
        //   - index: position
        //
        // We are using no-proto object so that we don't need to guard against inherited props via
        // hasOwnProperty.
        var lastBlockMap = createMap();

        //watch props
        $scope.$watchCollection(rhs, function ngRepeatAction(collection) {
          var index, length,
              previousNode = $element[0],     // node that cloned nodes should be inserted after
                                              // initialized to the comment node anchor
              nextNode,
              // Same as lastBlockMap but it has the current state. It will become the
              // lastBlockMap on the next iteration.
              nextBlockMap = createMap(),
              collectionLength,
              key, value, // key/value of iteration
              trackById,
              trackByIdFn,
              collectionKeys,
              block,       // last object information {scope, element, id}
              nextBlockOrder,
              elementsToRemove;

          if (aliasAs) {
            $scope[aliasAs] = collection;
          }

          if (isArrayLike(collection)) {
            collectionKeys = collection;
            trackByIdFn = trackByIdExpFn || trackByIdArrayFn;
          } else {
            trackByIdFn = trackByIdExpFn || trackByIdObjFn;
            // if object, extract keys, in enumeration order, unsorted
            collectionKeys = [];
            for (var itemKey in collection) {
              if (hasOwnProperty.call(collection, itemKey) && itemKey.charAt(0) !== '$') {
                collectionKeys.push(itemKey);
              }
            }
          }

          collectionLength = collectionKeys.length;
          nextBlockOrder = new Array(collectionLength);

          // locate existing items
          for (index = 0; index < collectionLength; index++) {
            key = (collection === collectionKeys) ? index : collectionKeys[index];
            value = collection[key];
            trackById = trackByIdFn($scope, key, value, index);
            if (lastBlockMap[trackById]) {
              // found previously seen block
              block = lastBlockMap[trackById];
              delete lastBlockMap[trackById];
              nextBlockMap[trackById] = block;
              nextBlockOrder[index] = block;
            } else if (nextBlockMap[trackById]) {
              // if collision detected. restore lastBlockMap and throw an error
              forEach(nextBlockOrder, function(block) {
                if (block && block.scope) lastBlockMap[block.id] = block;
              });
              throw ngRepeatMinErr('dupes',
                  'Duplicates in a repeater are not allowed. Use \'track by\' expression to specify unique keys. Repeater: {0}, Duplicate key: {1}, Duplicate value: {2}',
                  expression, trackById, value);
            } else {
              // new never before seen block
              nextBlockOrder[index] = {id: trackById, scope: undefined, clone: undefined};
              nextBlockMap[trackById] = true;
            }
          }

          // Clear the value property from the hashFnLocals object to prevent a reference to the last value
          // being leaked into the ngRepeatCompile function scope
          if (hashFnLocals) {
            hashFnLocals[valueIdentifier] = undefined;
          }

          // remove leftover items
          for (var blockKey in lastBlockMap) {
            block = lastBlockMap[blockKey];
            elementsToRemove = getBlockNodes(block.clone);
            $animate.leave(elementsToRemove);
            if (elementsToRemove[0].parentNode) {
              // if the element was not removed yet because of pending animation, mark it as deleted
              // so that we can ignore it later
              for (index = 0, length = elementsToRemove.length; index < length; index++) {
                elementsToRemove[index][NG_REMOVED] = true;
              }
            }
            block.scope.$destroy();
          }

          // we are not using forEach for perf reasons (trying to avoid #call)
          for (index = 0; index < collectionLength; index++) {
            key = (collection === collectionKeys) ? index : collectionKeys[index];
            value = collection[key];
            block = nextBlockOrder[index];

            if (block.scope) {
              // if we have already seen this object, then we need to reuse the
              // associated scope/element

              nextNode = previousNode;

              // skip nodes that are already pending removal via leave animation
              do {
                nextNode = nextNode.nextSibling;
              } while (nextNode && nextNode[NG_REMOVED]);

              if (getBlockStart(block) !== nextNode) {
                // existing item which got moved
                $animate.move(getBlockNodes(block.clone), null, previousNode);
              }
              previousNode = getBlockEnd(block);
              updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
            } else {
              // new item which we don't know about
              $transclude(function ngRepeatTransclude(clone, scope) {
                block.scope = scope;
                // http://jsperf.com/clone-vs-createcomment
                var endNode = ngRepeatEndComment.cloneNode(false);
                clone[clone.length++] = endNode;

                $animate.enter(clone, null, previousNode);
                previousNode = endNode;
                // Note: We only need the first/last node of the cloned nodes.
                // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                // by a directive with templateUrl when its template arrives.
                block.clone = clone;
                nextBlockMap[block.id] = block;
                updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
              });
            }
          }
          lastBlockMap = nextBlockMap;
        });
      };
    }
  };
}];

var NG_HIDE_CLASS = 'ng-hide';
var NG_HIDE_IN_PROGRESS_CLASS = 'ng-hide-animate';
/**
 * @ngdoc directive
 * @name ngShow
 * @multiElement
 *
 * @description
 * The `ngShow` directive shows or hides the given HTML element based on the expression provided to
 * the `ngShow` attribute.
 *
 * The element is shown or hidden by removing or adding the `.ng-hide` CSS class onto the element.
 * The `.ng-hide` CSS class is predefined in AngularJS and sets the display style to none (using an
 * `!important` flag). For CSP mode please add `angular-csp.css` to your HTML file (see
 * {@link ng.directive:ngCsp ngCsp}).
 *
 * ```html
 * <!-- when $scope.myValue is truthy (element is visible) -->
 * <div ng-show="myValue"></div>
 *
 * <!-- when $scope.myValue is falsy (element is hidden) -->
 * <div ng-show="myValue" class="ng-hide"></div>
 * ```
 *
 * When the `ngShow` expression evaluates to a falsy value then the `.ng-hide` CSS class is added
 * to the class attribute on the element causing it to become hidden. When truthy, the `.ng-hide`
 * CSS class is removed from the element causing the element not to appear hidden.
 *
 * ## Why is `!important` used?
 *
 * You may be wondering why `!important` is used for the `.ng-hide` CSS class. This is because the
 * `.ng-hide` selector can be easily overridden by heavier selectors. For example, something as
 * simple as changing the display style on a HTML list item would make hidden elements appear
 * visible. This also becomes a bigger issue when dealing with CSS frameworks.
 *
 * By using `!important`, the show and hide behavior will work as expected despite any clash between
 * CSS selector specificity (when `!important` isn't used with any conflicting styles). If a
 * developer chooses to override the styling to change how to hide an element then it is just a
 * matter of using `!important` in their own CSS code.
 *
 * ### Overriding `.ng-hide`
 *
 * By default, the `.ng-hide` class will style the element with `display: none !important`. If you
 * wish to change the hide behavior with `ngShow`/`ngHide`, you can simply overwrite the styles for
 * the `.ng-hide` CSS class. Note that the selector that needs to be used is actually
 * `.ng-hide:not(.ng-hide-animate)` to cope with extra animation classes that can be added.
 *
 * ```css
 * .ng-hide:not(.ng-hide-animate) {
 *   /&#42; These are just alternative ways of hiding an element &#42;/
 *   display: block!important;
 *   position: absolute;
 *   top: -9999px;
 *   left: -9999px;
 * }
 * ```
 *
 * By default you don't need to override anything in CSS and the animations will work around the
 * display style.
 *
 * @animations
 * | Animation                                           | Occurs                                                                                                        |
 * |-----------------------------------------------------|---------------------------------------------------------------------------------------------------------------|
 * | {@link $animate#addClass addClass} `.ng-hide`       | After the `ngShow` expression evaluates to a non truthy value and just before the contents are set to hidden. |
 * | {@link $animate#removeClass removeClass} `.ng-hide` | After the `ngShow` expression evaluates to a truthy value and just before contents are set to visible.        |
 *
 * Animations in `ngShow`/`ngHide` work with the show and hide events that are triggered when the
 * directive expression is true and false. This system works like the animation system present with
 * `ngClass` except that you must also include the `!important` flag to override the display
 * property so that the elements are not actually hidden during the animation.
 *
 * ```css
 * /&#42; A working example can be found at the bottom of this page. &#42;/
 * .my-element.ng-hide-add, .my-element.ng-hide-remove {
 *   transition: all 0.5s linear;
 * }
 *
 * .my-element.ng-hide-add { ... }
 * .my-element.ng-hide-add.ng-hide-add-active { ... }
 * .my-element.ng-hide-remove { ... }
 * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
 * ```
 *
 * Keep in mind that, as of AngularJS version 1.3, there is no need to change the display property
 * to block during animation states - ngAnimate will automatically handle the style toggling for you.
 *
 * @element ANY
 * @param {expression} ngShow If the {@link guide/expression expression} is truthy/falsy then the
 *                            element is shown/hidden respectively.
 *
 * @example
 * A simple example, animating the element's opacity:
 *
  <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-show-simple">
    <file name="index.html">
      Show: <input type="checkbox" ng-model="checked" aria-label="Toggle ngShow"><br />
      <div class="check-element animate-show-hide" ng-show="checked">
        I show up when your checkbox is checked.
      </div>
    </file>
    <file name="animations.css">
      .animate-show-hide.ng-hide {
        opacity: 0;
      }

      .animate-show-hide.ng-hide-add,
      .animate-show-hide.ng-hide-remove {
        transition: all linear 0.5s;
      }

      .check-element {
        border: 1px solid black;
        opacity: 1;
        padding: 10px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      it('should check ngShow', function() {
        var checkbox = element(by.model('checked'));
        var checkElem = element(by.css('.check-element'));

        expect(checkElem.isDisplayed()).toBe(false);
        checkbox.click();
        expect(checkElem.isDisplayed()).toBe(true);
      });
    </file>
  </example>
 *
 * <hr />
 * @example
 * A more complex example, featuring different show/hide animations:
 *
  <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-show-complex">
    <file name="index.html">
      Show: <input type="checkbox" ng-model="checked" aria-label="Toggle ngShow"><br />
      <div class="check-element funky-show-hide" ng-show="checked">
        I show up when your checkbox is checked.
      </div>
    </file>
    <file name="animations.css">
      body {
        overflow: hidden;
        perspective: 1000px;
      }

      .funky-show-hide.ng-hide-add {
        transform: rotateZ(0);
        transform-origin: right;
        transition: all 0.5s ease-in-out;
      }

      .funky-show-hide.ng-hide-add.ng-hide-add-active {
        transform: rotateZ(-135deg);
      }

      .funky-show-hide.ng-hide-remove {
        transform: rotateY(90deg);
        transform-origin: left;
        transition: all 0.5s ease;
      }

      .funky-show-hide.ng-hide-remove.ng-hide-remove-active {
        transform: rotateY(0);
      }

      .check-element {
        border: 1px solid black;
        opacity: 1;
        padding: 10px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      it('should check ngShow', function() {
        var checkbox = element(by.model('checked'));
        var checkElem = element(by.css('.check-element'));

        expect(checkElem.isDisplayed()).toBe(false);
        checkbox.click();
        expect(checkElem.isDisplayed()).toBe(true);
      });
    </file>
  </example>
 *
 * @knownIssue
 *
 * ### Flickering when using ngShow to toggle between elements
 *
 * When using {@link ngShow} and / or {@link ngHide} to toggle between elements, it can
 * happen that both the element to show and the element to hide are visible for a very short time.
 *
 * This usually happens when the {@link ngAnimate ngAnimate module} is included, but no actual animations
 * are defined for {@link ngShow} / {@link ngHide}. Internet Explorer is affected more often than
 * other browsers.
 *
 * There are several way to mitigate this problem:
 *
 * - {@link guide/animations#how-to-selectively-enable-disable-and-skip-animations Disable animations on the affected elements}.
 * - Use {@link ngIf} or {@link ngSwitch} instead of {@link ngShow} / {@link ngHide}.
 * - Use the special CSS selector `ng-hide.ng-hide-animate` to set `{display: none}` or similar on the affected elements.
 * - Use `ng-class="{'ng-hide': expression}` instead of instead of {@link ngShow} / {@link ngHide}.
 * - Define an animation on the affected elements.
 */
var ngShowDirective = ['$animate', function($animate) {
  return {
    restrict: 'A',
    multiElement: true,
    link: function(scope, element, attr) {
      scope.$watch(attr.ngShow, function ngShowWatchAction(value) {
        // we're adding a temporary, animation-specific class for ng-hide since this way
        // we can control when the element is actually displayed on screen without having
        // to have a global/greedy CSS selector that breaks when other animations are run.
        // Read: https://github.com/angular/angular.js/issues/9103#issuecomment-58335845
        $animate[value ? 'removeClass' : 'addClass'](element, NG_HIDE_CLASS, {
          tempClasses: NG_HIDE_IN_PROGRESS_CLASS
        });
      });
    }
  };
}];


/**
 * @ngdoc directive
 * @name ngHide
 * @multiElement
 *
 * @description
 * The `ngHide` directive shows or hides the given HTML element based on the expression provided to
 * the `ngHide` attribute.
 *
 * The element is shown or hidden by removing or adding the `.ng-hide` CSS class onto the element.
 * The `.ng-hide` CSS class is predefined in AngularJS and sets the display style to none (using an
 * `!important` flag). For CSP mode please add `angular-csp.css` to your HTML file (see
 * {@link ng.directive:ngCsp ngCsp}).
 *
 * ```html
 * <!-- when $scope.myValue is truthy (element is hidden) -->
 * <div ng-hide="myValue" class="ng-hide"></div>
 *
 * <!-- when $scope.myValue is falsy (element is visible) -->
 * <div ng-hide="myValue"></div>
 * ```
 *
 * When the `ngHide` expression evaluates to a truthy value then the `.ng-hide` CSS class is added
 * to the class attribute on the element causing it to become hidden. When falsy, the `.ng-hide`
 * CSS class is removed from the element causing the element not to appear hidden.
 *
 * ## Why is `!important` used?
 *
 * You may be wondering why `!important` is used for the `.ng-hide` CSS class. This is because the
 * `.ng-hide` selector can be easily overridden by heavier selectors. For example, something as
 * simple as changing the display style on a HTML list item would make hidden elements appear
 * visible. This also becomes a bigger issue when dealing with CSS frameworks.
 *
 * By using `!important`, the show and hide behavior will work as expected despite any clash between
 * CSS selector specificity (when `!important` isn't used with any conflicting styles). If a
 * developer chooses to override the styling to change how to hide an element then it is just a
 * matter of using `!important` in their own CSS code.
 *
 * ### Overriding `.ng-hide`
 *
 * By default, the `.ng-hide` class will style the element with `display: none !important`. If you
 * wish to change the hide behavior with `ngShow`/`ngHide`, you can simply overwrite the styles for
 * the `.ng-hide` CSS class. Note that the selector that needs to be used is actually
 * `.ng-hide:not(.ng-hide-animate)` to cope with extra animation classes that can be added.
 *
 * ```css
 * .ng-hide:not(.ng-hide-animate) {
 *   /&#42; These are just alternative ways of hiding an element &#42;/
 *   display: block!important;
 *   position: absolute;
 *   top: -9999px;
 *   left: -9999px;
 * }
 * ```
 *
 * By default you don't need to override in CSS anything and the animations will work around the
 * display style.
 *
 * @animations
 * | Animation                                           | Occurs                                                                                                     |
 * |-----------------------------------------------------|------------------------------------------------------------------------------------------------------------|
 * | {@link $animate#addClass addClass} `.ng-hide`       | After the `ngHide` expression evaluates to a truthy value and just before the contents are set to hidden.  |
 * | {@link $animate#removeClass removeClass} `.ng-hide` | After the `ngHide` expression evaluates to a non truthy value and just before contents are set to visible. |
 *
 * Animations in `ngShow`/`ngHide` work with the show and hide events that are triggered when the
 * directive expression is true and false. This system works like the animation system present with
 * `ngClass` except that you must also include the `!important` flag to override the display
 * property so that the elements are not actually hidden during the animation.
 *
 * ```css
 * /&#42; A working example can be found at the bottom of this page. &#42;/
 * .my-element.ng-hide-add, .my-element.ng-hide-remove {
 *   transition: all 0.5s linear;
 * }
 *
 * .my-element.ng-hide-add { ... }
 * .my-element.ng-hide-add.ng-hide-add-active { ... }
 * .my-element.ng-hide-remove { ... }
 * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
 * ```
 *
 * Keep in mind that, as of AngularJS version 1.3, there is no need to change the display property
 * to block during animation states - ngAnimate will automatically handle the style toggling for you.
 *
 * @element ANY
 * @param {expression} ngHide If the {@link guide/expression expression} is truthy/falsy then the
 *                            element is hidden/shown respectively.
 *
 * @example
 * A simple example, animating the element's opacity:
 *
  <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-hide-simple">
    <file name="index.html">
      Hide: <input type="checkbox" ng-model="checked" aria-label="Toggle ngHide"><br />
      <div class="check-element animate-show-hide" ng-hide="checked">
        I hide when your checkbox is checked.
      </div>
    </file>
    <file name="animations.css">
      .animate-show-hide.ng-hide {
        opacity: 0;
      }

      .animate-show-hide.ng-hide-add,
      .animate-show-hide.ng-hide-remove {
        transition: all linear 0.5s;
      }

      .check-element {
        border: 1px solid black;
        opacity: 1;
        padding: 10px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      it('should check ngHide', function() {
        var checkbox = element(by.model('checked'));
        var checkElem = element(by.css('.check-element'));

        expect(checkElem.isDisplayed()).toBe(true);
        checkbox.click();
        expect(checkElem.isDisplayed()).toBe(false);
      });
    </file>
  </example>
 *
 * <hr />
 * @example
 * A more complex example, featuring different show/hide animations:
 *
  <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-hide-complex">
    <file name="index.html">
      Hide: <input type="checkbox" ng-model="checked" aria-label="Toggle ngHide"><br />
      <div class="check-element funky-show-hide" ng-hide="checked">
        I hide when your checkbox is checked.
      </div>
    </file>
    <file name="animations.css">
      body {
        overflow: hidden;
        perspective: 1000px;
      }

      .funky-show-hide.ng-hide-add {
        transform: rotateZ(0);
        transform-origin: right;
        transition: all 0.5s ease-in-out;
      }

      .funky-show-hide.ng-hide-add.ng-hide-add-active {
        transform: rotateZ(-135deg);
      }

      .funky-show-hide.ng-hide-remove {
        transform: rotateY(90deg);
        transform-origin: left;
        transition: all 0.5s ease;
      }

      .funky-show-hide.ng-hide-remove.ng-hide-remove-active {
        transform: rotateY(0);
      }

      .check-element {
        border: 1px solid black;
        opacity: 1;
        padding: 10px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      it('should check ngHide', function() {
        var checkbox = element(by.model('checked'));
        var checkElem = element(by.css('.check-element'));

        expect(checkElem.isDisplayed()).toBe(true);
        checkbox.click();
        expect(checkElem.isDisplayed()).toBe(false);
      });
    </file>
  </example>
 *
 * @knownIssue
 *
 * ### Flickering when using ngHide to toggle between elements
 *
 * When using {@link ngShow} and / or {@link ngHide} to toggle between elements, it can
 * happen that both the element to show and the element to hide are visible for a very short time.
 *
 * This usually happens when the {@link ngAnimate ngAnimate module} is included, but no actual animations
 * are defined for {@link ngShow} / {@link ngHide}. Internet Explorer is affected more often than
 * other browsers.
 *
 * There are several way to mitigate this problem:
 *
 * - {@link guide/animations#how-to-selectively-enable-disable-and-skip-animations Disable animations on the affected elements}.
 * - Use {@link ngIf} or {@link ngSwitch} instead of {@link ngShow} / {@link ngHide}.
 * - Use the special CSS selector `ng-hide.ng-hide-animate` to set `{display: none}` or similar on the affected elements.
 * - Use `ng-class="{'ng-hide': expression}` instead of instead of {@link ngShow} / {@link ngHide}.
 * - Define an animation on the affected elements.
 */
var ngHideDirective = ['$animate', function($animate) {
  return {
    restrict: 'A',
    multiElement: true,
    link: function(scope, element, attr) {
      scope.$watch(attr.ngHide, function ngHideWatchAction(value) {
        // The comment inside of the ngShowDirective explains why we add and
        // remove a temporary class for the show/hide animation
        $animate[value ? 'addClass' : 'removeClass'](element,NG_HIDE_CLASS, {
          tempClasses: NG_HIDE_IN_PROGRESS_CLASS
        });
      });
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngStyle
 * @restrict AC
 *
 * @description
 * The `ngStyle` directive allows you to set CSS style on an HTML element conditionally.
 *
 * @knownIssue
 * You should not use {@link guide/interpolation interpolation} in the value of the `style`
 * attribute, when using the `ngStyle` directive on the same element.
 * See {@link guide/interpolation#known-issues here} for more info.
 *
 * @element ANY
 * @param {expression} ngStyle
 *
 * {@link guide/expression Expression} which evals to an
 * object whose keys are CSS style names and values are corresponding values for those CSS
 * keys.
 *
 * Since some CSS style names are not valid keys for an object, they must be quoted.
 * See the 'background-color' style in the example below.
 *
 * @example
   <example name="ng-style">
     <file name="index.html">
        <input type="button" value="set color" ng-click="myStyle={color:'red'}">
        <input type="button" value="set background" ng-click="myStyle={'background-color':'blue'}">
        <input type="button" value="clear" ng-click="myStyle={}">
        <br/>
        <span ng-style="myStyle">Sample Text</span>
        <pre>myStyle={{myStyle}}</pre>
     </file>
     <file name="style.css">
       span {
         color: black;
       }
     </file>
     <file name="protractor.js" type="protractor">
       var colorSpan = element(by.css('span'));

       it('should check ng-style', function() {
         expect(colorSpan.getCssValue('color')).toMatch(/rgba\(0, 0, 0, 1\)|rgb\(0, 0, 0\)/);
         element(by.css('input[value=\'set color\']')).click();
         expect(colorSpan.getCssValue('color')).toMatch(/rgba\(255, 0, 0, 1\)|rgb\(255, 0, 0\)/);
         element(by.css('input[value=clear]')).click();
         expect(colorSpan.getCssValue('color')).toMatch(/rgba\(0, 0, 0, 1\)|rgb\(0, 0, 0\)/);
       });
     </file>
   </example>
 */
var ngStyleDirective = ngDirective(function(scope, element, attr) {
  scope.$watchCollection(attr.ngStyle, function ngStyleWatchAction(newStyles, oldStyles) {
    if (oldStyles && (newStyles !== oldStyles)) {
      forEach(oldStyles, function(val, style) { element.css(style, ''); });
    }
    if (newStyles) element.css(newStyles);
  });
});

/**
 * @ngdoc directive
 * @name ngSwitch
 * @restrict EA
 *
 * @description
 * The `ngSwitch` directive is used to conditionally swap DOM structure on your template based on a scope expression.
 * Elements within `ngSwitch` but without `ngSwitchWhen` or `ngSwitchDefault` directives will be preserved at the location
 * as specified in the template.
 *
 * The directive itself works similar to ngInclude, however, instead of downloading template code (or loading it
 * from the template cache), `ngSwitch` simply chooses one of the nested elements and makes it visible based on which element
 * matches the value obtained from the evaluated expression. In other words, you define a container element
 * (where you place the directive), place an expression on the **`on="..."` attribute**
 * (or the **`ng-switch="..."` attribute**), define any inner elements inside of the directive and place
 * a when attribute per element. The when attribute is used to inform ngSwitch which element to display when the on
 * expression is evaluated. If a matching expression is not found via a when attribute then an element with the default
 * attribute is displayed.
 *
 * <div class="alert alert-info">
 * Be aware that the attribute values to match against cannot be expressions. They are interpreted
 * as literal string values to match against.
 * For example, **`ng-switch-when="someVal"`** will match against the string `"someVal"` not against the
 * value of the expression `$scope.someVal`.
 * </div>

 * @animations
 * | Animation                        | Occurs                              |
 * |----------------------------------|-------------------------------------|
 * | {@link ng.$animate#enter enter}  | after the ngSwitch contents change and the matched child element is placed inside the container |
 * | {@link ng.$animate#leave leave}  | after the ngSwitch contents change and just before the former contents are removed from the DOM |
 *
 * @usage
 *
 * ```
 * <ANY ng-switch="expression">
 *   <ANY ng-switch-when="matchValue1">...</ANY>
 *   <ANY ng-switch-when="matchValue2">...</ANY>
 *   <ANY ng-switch-default>...</ANY>
 * </ANY>
 * ```
 *
 *
 * @scope
 * @priority 1200
 * @param {*} ngSwitch|on expression to match against <code>ng-switch-when</code>.
 * On child elements add:
 *
 * * `ngSwitchWhen`: the case statement to match against. If match then this
 *   case will be displayed. If the same match appears multiple times, all the
 *   elements will be displayed. It is possible to associate multiple values to
 *   the same `ngSwitchWhen` by defining the optional attribute
 *   `ngSwitchWhenSeparator`. The separator will be used to split the value of
 *   the `ngSwitchWhen` attribute into multiple tokens, and the element will show
 *   if any of the `ngSwitch` evaluates to any of these tokens.
 * * `ngSwitchDefault`: the default case when no other case match. If there
 *   are multiple default cases, all of them will be displayed when no other
 *   case match.
 *
 *
 * @example
  <example module="switchExample" deps="angular-animate.js" animations="true" name="ng-switch">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <select ng-model="selection" ng-options="item for item in items">
        </select>
        <code>selection={{selection}}</code>
        <hr/>
        <div class="animate-switch-container"
          ng-switch on="selection">
            <div class="animate-switch" ng-switch-when="settings|options" ng-switch-when-separator="|">Settings Div</div>
            <div class="animate-switch" ng-switch-when="home">Home Span</div>
            <div class="animate-switch" ng-switch-default>default</div>
        </div>
      </div>
    </file>
    <file name="script.js">
      angular.module('switchExample', ['ngAnimate'])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.items = ['settings', 'home', 'options', 'other'];
          $scope.selection = $scope.items[0];
        }]);
    </file>
    <file name="animations.css">
      .animate-switch-container {
        position:relative;
        background:white;
        border:1px solid black;
        height:40px;
        overflow:hidden;
      }

      .animate-switch {
        padding:10px;
      }

      .animate-switch.ng-animate {
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;

        position:absolute;
        top:0;
        left:0;
        right:0;
        bottom:0;
      }

      .animate-switch.ng-leave.ng-leave-active,
      .animate-switch.ng-enter {
        top:-50px;
      }
      .animate-switch.ng-leave,
      .animate-switch.ng-enter.ng-enter-active {
        top:0;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var switchElem = element(by.css('[ng-switch]'));
      var select = element(by.model('selection'));

      it('should start in settings', function() {
        expect(switchElem.getText()).toMatch(/Settings Div/);
      });
      it('should change to home', function() {
        select.all(by.css('option')).get(1).click();
        expect(switchElem.getText()).toMatch(/Home Span/);
      });
      it('should change to settings via "options"', function() {
        select.all(by.css('option')).get(2).click();
        expect(switchElem.getText()).toMatch(/Settings Div/);
      });
      it('should select default', function() {
        select.all(by.css('option')).get(3).click();
        expect(switchElem.getText()).toMatch(/default/);
      });
    </file>
  </example>
 */
var ngSwitchDirective = ['$animate', '$compile', function($animate, $compile) {
  return {
    require: 'ngSwitch',

    // asks for $scope to fool the BC controller module
    controller: ['$scope', function NgSwitchController() {
     this.cases = {};
    }],
    link: function(scope, element, attr, ngSwitchController) {
      var watchExpr = attr.ngSwitch || attr.on,
          selectedTranscludes = [],
          selectedElements = [],
          previousLeaveAnimations = [],
          selectedScopes = [];

      var spliceFactory = function(array, index) {
          return function(response) {
            if (response !== false) array.splice(index, 1);
          };
      };

      scope.$watch(watchExpr, function ngSwitchWatchAction(value) {
        var i, ii;

        // Start with the last, in case the array is modified during the loop
        while (previousLeaveAnimations.length) {
          $animate.cancel(previousLeaveAnimations.pop());
        }

        for (i = 0, ii = selectedScopes.length; i < ii; ++i) {
          var selected = getBlockNodes(selectedElements[i].clone);
          selectedScopes[i].$destroy();
          var runner = previousLeaveAnimations[i] = $animate.leave(selected);
          runner.done(spliceFactory(previousLeaveAnimations, i));
        }

        selectedElements.length = 0;
        selectedScopes.length = 0;

        if ((selectedTranscludes = ngSwitchController.cases['!' + value] || ngSwitchController.cases['?'])) {
          forEach(selectedTranscludes, function(selectedTransclude) {
            selectedTransclude.transclude(function(caseElement, selectedScope) {
              selectedScopes.push(selectedScope);
              var anchor = selectedTransclude.element;
              caseElement[caseElement.length++] = $compile.$$createComment('end ngSwitchWhen');
              var block = { clone: caseElement };

              selectedElements.push(block);
              $animate.enter(caseElement, anchor.parent(), anchor);
            });
          });
        }
      });
    }
  };
}];

var ngSwitchWhenDirective = ngDirective({
  transclude: 'element',
  priority: 1200,
  require: '^ngSwitch',
  multiElement: true,
  link: function(scope, element, attrs, ctrl, $transclude) {

    var cases = attrs.ngSwitchWhen.split(attrs.ngSwitchWhenSeparator).sort().filter(
      // Filter duplicate cases
      function(element, index, array) { return array[index - 1] !== element; }
    );

    forEach(cases, function(whenCase) {
      ctrl.cases['!' + whenCase] = (ctrl.cases['!' + whenCase] || []);
      ctrl.cases['!' + whenCase].push({ transclude: $transclude, element: element });
    });
  }
});

var ngSwitchDefaultDirective = ngDirective({
  transclude: 'element',
  priority: 1200,
  require: '^ngSwitch',
  multiElement: true,
  link: function(scope, element, attr, ctrl, $transclude) {
    ctrl.cases['?'] = (ctrl.cases['?'] || []);
    ctrl.cases['?'].push({ transclude: $transclude, element: element });
   }
});

/**
 * @ngdoc directive
 * @name ngTransclude
 * @restrict EAC
 *
 * @description
 * Directive that marks the insertion point for the transcluded DOM of the nearest parent directive that uses transclusion.
 *
 * You can specify that you want to insert a named transclusion slot, instead of the default slot, by providing the slot name
 * as the value of the `ng-transclude` or `ng-transclude-slot` attribute.
 *
 * If the transcluded content is not empty (i.e. contains one or more DOM nodes, including whitespace text nodes), any existing
 * content of this element will be removed before the transcluded content is inserted.
 * If the transcluded content is empty (or only whitespace), the existing content is left intact. This lets you provide fallback
 * content in the case that no transcluded content is provided.
 *
 * @element ANY
 *
 * @param {string} ngTransclude|ngTranscludeSlot the name of the slot to insert at this point. If this is not provided, is empty
 *                                               or its value is the same as the name of the attribute then the default slot is used.
 *
 * @example
 * ### Basic transclusion
 * This example demonstrates basic transclusion of content into a component directive.
 * <example name="simpleTranscludeExample" module="transcludeExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('transcludeExample', [])
 *        .directive('pane', function(){
 *           return {
 *             restrict: 'E',
 *             transclude: true,
 *             scope: { title:'@' },
 *             template: '<div style="border: 1px solid black;">' +
 *                         '<div style="background-color: gray">{{title}}</div>' +
 *                         '<ng-transclude></ng-transclude>' +
 *                       '</div>'
 *           };
 *       })
 *       .controller('ExampleController', ['$scope', function($scope) {
 *         $scope.title = 'Lorem Ipsum';
 *         $scope.text = 'Neque porro quisquam est qui dolorem ipsum quia dolor...';
 *       }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <input ng-model="title" aria-label="title"> <br/>
 *       <textarea ng-model="text" aria-label="text"></textarea> <br/>
 *       <pane title="{{title}}"><span>{{text}}</span></pane>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *      it('should have transcluded', function() {
 *        var titleElement = element(by.model('title'));
 *        titleElement.clear();
 *        titleElement.sendKeys('TITLE');
 *        var textElement = element(by.model('text'));
 *        textElement.clear();
 *        textElement.sendKeys('TEXT');
 *        expect(element(by.binding('title')).getText()).toEqual('TITLE');
 *        expect(element(by.binding('text')).getText()).toEqual('TEXT');
 *      });
 *   </file>
 * </example>
 *
 * @example
 * ### Transclude fallback content
 * This example shows how to use `NgTransclude` with fallback content, that
 * is displayed if no transcluded content is provided.
 *
 * <example module="transcludeFallbackContentExample" name="ng-transclude">
 * <file name="index.html">
 * <script>
 * angular.module('transcludeFallbackContentExample', [])
 * .directive('myButton', function(){
 *             return {
 *               restrict: 'E',
 *               transclude: true,
 *               scope: true,
 *               template: '<button style="cursor: pointer;">' +
 *                           '<ng-transclude>' +
 *                             '<b style="color: red;">Button1</b>' +
 *                           '</ng-transclude>' +
 *                         '</button>'
 *             };
 *         });
 * </script>
 * <!-- fallback button content -->
 * <my-button id="fallback"></my-button>
 * <!-- modified button content -->
 * <my-button id="modified">
 *   <i style="color: green;">Button2</i>
 * </my-button>
 * </file>
 * <file name="protractor.js" type="protractor">
 * it('should have different transclude element content', function() {
 *          expect(element(by.id('fallback')).getText()).toBe('Button1');
 *          expect(element(by.id('modified')).getText()).toBe('Button2');
 *        });
 * </file>
 * </example>
 *
 * @example
 * ### Multi-slot transclusion
 * This example demonstrates using multi-slot transclusion in a component directive.
 * <example name="multiSlotTranscludeExample" module="multiSlotTranscludeExample">
 *   <file name="index.html">
 *    <style>
 *      .title, .footer {
 *        background-color: gray
 *      }
 *    </style>
 *    <div ng-controller="ExampleController">
 *      <input ng-model="title" aria-label="title"> <br/>
 *      <textarea ng-model="text" aria-label="text"></textarea> <br/>
 *      <pane>
 *        <pane-title><a ng-href="{{link}}">{{title}}</a></pane-title>
 *        <pane-body><p>{{text}}</p></pane-body>
 *      </pane>
 *    </div>
 *   </file>
 *   <file name="app.js">
 *    angular.module('multiSlotTranscludeExample', [])
 *     .directive('pane', function() {
 *        return {
 *          restrict: 'E',
 *          transclude: {
 *            'title': '?paneTitle',
 *            'body': 'paneBody',
 *            'footer': '?paneFooter'
 *          },
 *          template: '<div style="border: 1px solid black;">' +
 *                      '<div class="title" ng-transclude="title">Fallback Title</div>' +
 *                      '<div ng-transclude="body"></div>' +
 *                      '<div class="footer" ng-transclude="footer">Fallback Footer</div>' +
 *                    '</div>'
 *        };
 *    })
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.title = 'Lorem Ipsum';
 *      $scope.link = 'https://google.com';
 *      $scope.text = 'Neque porro quisquam est qui dolorem ipsum quia dolor...';
 *    }]);
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *      it('should have transcluded the title and the body', function() {
 *        var titleElement = element(by.model('title'));
 *        titleElement.clear();
 *        titleElement.sendKeys('TITLE');
 *        var textElement = element(by.model('text'));
 *        textElement.clear();
 *        textElement.sendKeys('TEXT');
 *        expect(element(by.css('.title')).getText()).toEqual('TITLE');
 *        expect(element(by.binding('text')).getText()).toEqual('TEXT');
 *        expect(element(by.css('.footer')).getText()).toEqual('Fallback Footer');
 *      });
 *   </file>
 * </example>
 */
var ngTranscludeMinErr = minErr('ngTransclude');
var ngTranscludeDirective = ['$compile', function($compile) {
  return {
    restrict: 'EAC',
    compile: function ngTranscludeCompile(tElement) {

      // Remove and cache any original content to act as a fallback
      var fallbackLinkFn = $compile(tElement.contents());
      tElement.empty();

      return function ngTranscludePostLink($scope, $element, $attrs, controller, $transclude) {

        if (!$transclude) {
          throw ngTranscludeMinErr('orphan',
          'Illegal use of ngTransclude directive in the template! ' +
          'No parent directive that requires a transclusion found. ' +
          'Element: {0}',
          startingTag($element));
        }


        // If the attribute is of the form: `ng-transclude="ng-transclude"` then treat it like the default
        if ($attrs.ngTransclude === $attrs.$attr.ngTransclude) {
          $attrs.ngTransclude = '';
        }
        var slotName = $attrs.ngTransclude || $attrs.ngTranscludeSlot;

        // If the slot is required and no transclusion content is provided then this call will throw an error
        $transclude(ngTranscludeCloneAttachFn, null, slotName);

        // If the slot is optional and no transclusion content is provided then use the fallback content
        if (slotName && !$transclude.isSlotFilled(slotName)) {
          useFallbackContent();
        }

        function ngTranscludeCloneAttachFn(clone, transcludedScope) {
          if (clone.length && notWhitespace(clone)) {
            $element.append(clone);
          } else {
            useFallbackContent();
            // There is nothing linked against the transcluded scope since no content was available,
            // so it should be safe to clean up the generated scope.
            transcludedScope.$destroy();
          }
        }

        function useFallbackContent() {
          // Since this is the fallback content rather than the transcluded content,
          // we link against the scope of this directive rather than the transcluded scope
          fallbackLinkFn($scope, function(clone) {
            $element.append(clone);
          });
        }

        function notWhitespace(nodes) {
          for (var i = 0, ii = nodes.length; i < ii; i++) {
            var node = nodes[i];
            if (node.nodeType !== NODE_TYPE_TEXT || node.nodeValue.trim()) {
              return true;
            }
          }
        }
      };
    }
  };
}];

/**
 * @ngdoc directive
 * @name script
 * @restrict E
 *
 * @description
 * Load the content of a `<script>` element into {@link ng.$templateCache `$templateCache`}, so that the
 * template can be used by {@link ng.directive:ngInclude `ngInclude`},
 * {@link ngRoute.directive:ngView `ngView`}, or {@link guide/directive directives}. The type of the
 * `<script>` element must be specified as `text/ng-template`, and a cache name for the template must be
 * assigned through the element's `id`, which can then be used as a directive's `templateUrl`.
 *
 * @param {string} type Must be set to `'text/ng-template'`.
 * @param {string} id Cache name of the template.
 *
 * @example
  <example  name="script-tag">
    <file name="index.html">
      <script type="text/ng-template" id="/tpl.html">
        Content of the template.
      </script>

      <a ng-click="currentTpl='/tpl.html'" id="tpl-link">Load inlined template</a>
      <div id="tpl-content" ng-include src="currentTpl"></div>
    </file>
    <file name="protractor.js" type="protractor">
      it('should load template defined inside script tag', function() {
        element(by.css('#tpl-link')).click();
        expect(element(by.css('#tpl-content')).getText()).toMatch(/Content of the template/);
      });
    </file>
  </example>
 */
var scriptDirective = ['$templateCache', function($templateCache) {
  return {
    restrict: 'E',
    terminal: true,
    compile: function(element, attr) {
      if (attr.type === 'text/ng-template') {
        var templateUrl = attr.id,
            text = element[0].text;

        $templateCache.put(templateUrl, text);
      }
    }
  };
}];

/* exported selectDirective, optionDirective */

var noopNgModelController = { $setViewValue: noop, $render: noop };

function setOptionSelectedStatus(optionEl, value) {
  optionEl.prop('selected', value);
  /**
   * When unselecting an option, setting the property to null / false should be enough
   * However, screenreaders might react to the selected attribute instead, see
   * https://github.com/angular/angular.js/issues/14419
   * Note: "selected" is a boolean attr and will be removed when the "value" arg in attr() is false
   * or null
   */
  optionEl.attr('selected', value);
}

/**
 * @ngdoc type
 * @name  select.SelectController
 *
 * @description
 * The controller for the {@link ng.select select} directive. The controller exposes
 * a few utility methods that can be used to augment the behavior of a regular or an
 * {@link ng.ngOptions ngOptions} select element.
 *
 * @example
 * ### Set a custom error when the unknown option is selected
 *
 * This example sets a custom error "unknownValue" on the ngModelController
 * when the select element's unknown option is selected, i.e. when the model is set to a value
 * that is not matched by any option.
 *
 * <example name="select-unknown-value-error" module="staticSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="testSelect"> Single select: </label><br>
 *     <select name="testSelect" ng-model="selected" unknown-value-error>
 *       <option value="option-1">Option 1</option>
 *       <option value="option-2">Option 2</option>
 *     </select><br>
 *     <span class="error" ng-if="myForm.testSelect.$error.unknownValue">
 *       Error: The current model doesn't match any option</span><br>
 *
 *     <button ng-click="forceUnknownOption()">Force unknown option</button><br>
 *   </form>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('staticSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.selected = null;
 *
 *      $scope.forceUnknownOption = function() {
 *        $scope.selected = 'nonsense';
 *      };
 *   }])
 *   .directive('unknownValueError', function() {
 *     return {
 *       require: ['ngModel', 'select'],
 *       link: function(scope, element, attrs, ctrls) {
 *         var ngModelCtrl = ctrls[0];
 *         var selectCtrl = ctrls[1];
 *
 *         ngModelCtrl.$validators.unknownValue = function(modelValue, viewValue) {
 *           if (selectCtrl.$isUnknownOptionSelected()) {
 *             return false;
 *           }
 *
 *           return true;
 *         };
 *       }
 *
 *     };
 *   });
 * </file>
 *</example>
 *
 *
 * @example
 * ### Set the "required" error when the unknown option is selected.
 *
 * By default, the "required" error on the ngModelController is only set on a required select
 * when the empty option is selected. This example adds a custom directive that also sets the
 * error when the unknown option is selected.
 *
 * <example name="select-unknown-value-required" module="staticSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="testSelect"> Select: </label><br>
 *     <select name="testSelect" ng-model="selected" required unknown-value-required>
 *       <option value="option-1">Option 1</option>
 *       <option value="option-2">Option 2</option>
 *     </select><br>
 *     <span class="error" ng-if="myForm.testSelect.$error.required">Error: Please select a value</span><br>
 *
 *     <button ng-click="forceUnknownOption()">Force unknown option</button><br>
 *   </form>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('staticSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.selected = null;
 *
 *      $scope.forceUnknownOption = function() {
 *        $scope.selected = 'nonsense';
 *      };
 *   }])
 *   .directive('unknownValueRequired', function() {
 *     return {
 *       priority: 1, // This directive must run after the required directive has added its validator
 *       require: ['ngModel', 'select'],
 *       link: function(scope, element, attrs, ctrls) {
 *         var ngModelCtrl = ctrls[0];
 *         var selectCtrl = ctrls[1];
 *
 *         var originalRequiredValidator = ngModelCtrl.$validators.required;
 *
 *         ngModelCtrl.$validators.required = function() {
 *           if (attrs.required && selectCtrl.$isUnknownOptionSelected()) {
 *             return false;
 *           }
 *
 *           return originalRequiredValidator.apply(this, arguments);
 *         };
 *       }
 *     };
 *   });
 * </file>
 * <file name="protractor.js" type="protractor">
 *  it('should show the error message when the unknown option is selected', function() {

      var error = element(by.className('error'));

      expect(error.getText()).toBe('Error: Please select a value');

      element(by.cssContainingText('option', 'Option 1')).click();

      expect(error.isPresent()).toBe(false);

      element(by.tagName('button')).click();

      expect(error.getText()).toBe('Error: Please select a value');
    });
 * </file>
 *</example>
 *
 *
 */
var SelectController =
        ['$element', '$scope', /** @this */ function($element, $scope) {

  var self = this,
      optionsMap = new NgMap();

  self.selectValueMap = {}; // Keys are the hashed values, values the original values

  // If the ngModel doesn't get provided then provide a dummy noop version to prevent errors
  self.ngModelCtrl = noopNgModelController;
  self.multiple = false;

  // The "unknown" option is one that is prepended to the list if the viewValue
  // does not match any of the options. When it is rendered the value of the unknown
  // option is '? XXX ?' where XXX is the hashKey of the value that is not known.
  //
  // Support: IE 9 only
  // We can't just jqLite('<option>') since jqLite is not smart enough
  // to create it in <select> and IE barfs otherwise.
  self.unknownOption = jqLite(window.document.createElement('option'));

  // The empty option is an option with the value '' that the application developer can
  // provide inside the select. It is always selectable and indicates that a "null" selection has
  // been made by the user.
  // If the select has an empty option, and the model of the select is set to "undefined" or "null",
  // the empty option is selected.
  // If the model is set to a different unmatched value, the unknown option is rendered and
  // selected, i.e both are present, because a "null" selection and an unknown value are different.
  self.hasEmptyOption = false;
  self.emptyOption = undefined;

  self.renderUnknownOption = function(val) {
    var unknownVal = self.generateUnknownOptionValue(val);
    self.unknownOption.val(unknownVal);
    $element.prepend(self.unknownOption);
    setOptionSelectedStatus(self.unknownOption, true);
    $element.val(unknownVal);
  };

  self.updateUnknownOption = function(val) {
    var unknownVal = self.generateUnknownOptionValue(val);
    self.unknownOption.val(unknownVal);
    setOptionSelectedStatus(self.unknownOption, true);
    $element.val(unknownVal);
  };

  self.generateUnknownOptionValue = function(val) {
    return '? ' + hashKey(val) + ' ?';
  };

  self.removeUnknownOption = function() {
    if (self.unknownOption.parent()) self.unknownOption.remove();
  };

  self.selectEmptyOption = function() {
    if (self.emptyOption) {
      $element.val('');
      setOptionSelectedStatus(self.emptyOption, true);
    }
  };

  self.unselectEmptyOption = function() {
    if (self.hasEmptyOption) {
      setOptionSelectedStatus(self.emptyOption, false);
    }
  };

  $scope.$on('$destroy', function() {
    // disable unknown option so that we don't do work when the whole select is being destroyed
    self.renderUnknownOption = noop;
  });

  // Read the value of the select control, the implementation of this changes depending
  // upon whether the select can have multiple values and whether ngOptions is at work.
  self.readValue = function readSingleValue() {
    var val = $element.val();
    // ngValue added option values are stored in the selectValueMap, normal interpolations are not
    var realVal = val in self.selectValueMap ? self.selectValueMap[val] : val;

    if (self.hasOption(realVal)) {
      return realVal;
    }

    return null;
  };


  // Write the value to the select control, the implementation of this changes depending
  // upon whether the select can have multiple values and whether ngOptions is at work.
  self.writeValue = function writeSingleValue(value) {
    // Make sure to remove the selected attribute from the previously selected option
    // Otherwise, screen readers might get confused
    var currentlySelectedOption = $element[0].options[$element[0].selectedIndex];
    if (currentlySelectedOption) setOptionSelectedStatus(jqLite(currentlySelectedOption), false);

    if (self.hasOption(value)) {
      self.removeUnknownOption();

      var hashedVal = hashKey(value);
      $element.val(hashedVal in self.selectValueMap ? hashedVal : value);

      // Set selected attribute and property on selected option for screen readers
      var selectedOption = $element[0].options[$element[0].selectedIndex];
      setOptionSelectedStatus(jqLite(selectedOption), true);
    } else {
      self.selectUnknownOrEmptyOption(value);
    }
  };


  // Tell the select control that an option, with the given value, has been added
  self.addOption = function(value, element) {
    // Skip comment nodes, as they only pollute the `optionsMap`
    if (element[0].nodeType === NODE_TYPE_COMMENT) return;

    assertNotHasOwnProperty(value, '"option value"');
    if (value === '') {
      self.hasEmptyOption = true;
      self.emptyOption = element;
    }
    var count = optionsMap.get(value) || 0;
    optionsMap.set(value, count + 1);
    // Only render at the end of a digest. This improves render performance when many options
    // are added during a digest and ensures all relevant options are correctly marked as selected
    scheduleRender();
  };

  // Tell the select control that an option, with the given value, has been removed
  self.removeOption = function(value) {
    var count = optionsMap.get(value);
    if (count) {
      if (count === 1) {
        optionsMap.delete(value);
        if (value === '') {
          self.hasEmptyOption = false;
          self.emptyOption = undefined;
        }
      } else {
        optionsMap.set(value, count - 1);
      }
    }
  };

  // Check whether the select control has an option matching the given value
  self.hasOption = function(value) {
    return !!optionsMap.get(value);
  };

  /**
   * @ngdoc method
   * @name select.SelectController#$hasEmptyOption
   *
   * @description
   *
   * Returns `true` if the select element currently has an empty option
   * element, i.e. an option that signifies that the select is empty / the selection is null.
   *
   */
  self.$hasEmptyOption = function() {
    return self.hasEmptyOption;
  };

  /**
   * @ngdoc method
   * @name select.SelectController#$isUnknownOptionSelected
   *
   * @description
   *
   * Returns `true` if the select element's unknown option is selected. The unknown option is added
   * and automatically selected whenever the select model doesn't match any option.
   *
   */
  self.$isUnknownOptionSelected = function() {
    // Presence of the unknown option means it is selected
    return $element[0].options[0] === self.unknownOption[0];
  };

  /**
   * @ngdoc method
   * @name select.SelectController#$isEmptyOptionSelected
   *
   * @description
   *
   * Returns `true` if the select element has an empty option and this empty option is currently
   * selected. Returns `false` if the select element has no empty option or it is not selected.
   *
   */
  self.$isEmptyOptionSelected = function() {
    return self.hasEmptyOption && $element[0].options[$element[0].selectedIndex] === self.emptyOption[0];
  };

  self.selectUnknownOrEmptyOption = function(value) {
    if (value == null && self.emptyOption) {
      self.removeUnknownOption();
      self.selectEmptyOption();
    } else if (self.unknownOption.parent().length) {
      self.updateUnknownOption(value);
    } else {
      self.renderUnknownOption(value);
    }
  };

  var renderScheduled = false;
  function scheduleRender() {
    if (renderScheduled) return;
    renderScheduled = true;
    $scope.$$postDigest(function() {
      renderScheduled = false;
      self.ngModelCtrl.$render();
    });
  }

  var updateScheduled = false;
  function scheduleViewValueUpdate(renderAfter) {
    if (updateScheduled) return;

    updateScheduled = true;

    $scope.$$postDigest(function() {
      if ($scope.$$destroyed) return;

      updateScheduled = false;
      self.ngModelCtrl.$setViewValue(self.readValue());
      if (renderAfter) self.ngModelCtrl.$render();
    });
  }


  self.registerOption = function(optionScope, optionElement, optionAttrs, interpolateValueFn, interpolateTextFn) {

    if (optionAttrs.$attr.ngValue) {
      // The value attribute is set by ngValue
      var oldVal, hashedVal;
      optionAttrs.$observe('value', function valueAttributeObserveAction(newVal) {

        var removal;
        var previouslySelected = optionElement.prop('selected');

        if (isDefined(hashedVal)) {
          self.removeOption(oldVal);
          delete self.selectValueMap[hashedVal];
          removal = true;
        }

        hashedVal = hashKey(newVal);
        oldVal = newVal;
        self.selectValueMap[hashedVal] = newVal;
        self.addOption(newVal, optionElement);
        // Set the attribute directly instead of using optionAttrs.$set - this stops the observer
        // from firing a second time. Other $observers on value will also get the result of the
        // ngValue expression, not the hashed value
        optionElement.attr('value', hashedVal);

        if (removal && previouslySelected) {
          scheduleViewValueUpdate();
        }

      });
    } else if (interpolateValueFn) {
      // The value attribute is interpolated
      optionAttrs.$observe('value', function valueAttributeObserveAction(newVal) {
        // This method is overwritten in ngOptions and has side-effects!
        self.readValue();

        var removal;
        var previouslySelected = optionElement.prop('selected');

        if (isDefined(oldVal)) {
          self.removeOption(oldVal);
          removal = true;
        }
        oldVal = newVal;
        self.addOption(newVal, optionElement);

        if (removal && previouslySelected) {
          scheduleViewValueUpdate();
        }
      });
    } else if (interpolateTextFn) {
      // The text content is interpolated
      optionScope.$watch(interpolateTextFn, function interpolateWatchAction(newVal, oldVal) {
        optionAttrs.$set('value', newVal);
        var previouslySelected = optionElement.prop('selected');
        if (oldVal !== newVal) {
          self.removeOption(oldVal);
        }
        self.addOption(newVal, optionElement);

        if (oldVal && previouslySelected) {
          scheduleViewValueUpdate();
        }
      });
    } else {
      // The value attribute is static
      self.addOption(optionAttrs.value, optionElement);
    }


    optionAttrs.$observe('disabled', function(newVal) {

      // Since model updates will also select disabled options (like ngOptions),
      // we only have to handle options becoming disabled, not enabled

      if (newVal === 'true' || newVal && optionElement.prop('selected')) {
        if (self.multiple) {
          scheduleViewValueUpdate(true);
        } else {
          self.ngModelCtrl.$setViewValue(null);
          self.ngModelCtrl.$render();
        }
      }
    });

    optionElement.on('$destroy', function() {
      var currentValue = self.readValue();
      var removeValue = optionAttrs.value;

      self.removeOption(removeValue);
      scheduleRender();

      if (self.multiple && currentValue && currentValue.indexOf(removeValue) !== -1 ||
          currentValue === removeValue
      ) {
        // When multiple (selected) options are destroyed at the same time, we don't want
        // to run a model update for each of them. Instead, run a single update in the $$postDigest
        scheduleViewValueUpdate(true);
      }
    });
  };
}];

/**
 * @ngdoc directive
 * @name select
 * @restrict E
 *
 * @description
 * HTML `select` element with AngularJS data-binding.
 *
 * The `select` directive is used together with {@link ngModel `ngModel`} to provide data-binding
 * between the scope and the `<select>` control (including setting default values).
 * It also handles dynamic `<option>` elements, which can be added using the {@link ngRepeat `ngRepeat}` or
 * {@link ngOptions `ngOptions`} directives.
 *
 * When an item in the `<select>` menu is selected, the value of the selected option will be bound
 * to the model identified by the `ngModel` directive. With static or repeated options, this is
 * the content of the `value` attribute or the textContent of the `<option>`, if the value attribute is missing.
 * Value and textContent can be interpolated.
 *
 * The {@link select.SelectController select controller} exposes utility functions that can be used
 * to manipulate the select's behavior.
 *
 * ## Matching model and option values
 *
 * In general, the match between the model and an option is evaluated by strictly comparing the model
 * value against the value of the available options.
 *
 * If you are setting the option value with the option's `value` attribute, or textContent, the
 * value will always be a `string` which means that the model value must also be a string.
 * Otherwise the `select` directive cannot match them correctly.
 *
 * To bind the model to a non-string value, you can use one of the following strategies:
 * - the {@link ng.ngOptions `ngOptions`} directive
 *   ({@link ng.select#using-select-with-ngoptions-and-setting-a-default-value})
 * - the {@link ng.ngValue `ngValue`} directive, which allows arbitrary expressions to be
 *   option values ({@link ng.select#using-ngvalue-to-bind-the-model-to-an-array-of-objects Example})
 * - model $parsers / $formatters to convert the string value
 *   ({@link ng.select#binding-select-to-a-non-string-value-via-ngmodel-parsing-formatting Example})
 *
 * If the viewValue of `ngModel` does not match any of the options, then the control
 * will automatically add an "unknown" option, which it then removes when the mismatch is resolved.
 *
 * Optionally, a single hard-coded `<option>` element, with the value set to an empty string, can
 * be nested into the `<select>` element. This element will then represent the `null` or "not selected"
 * option. See example below for demonstration.
 *
 * ## Choosing between `ngRepeat` and `ngOptions`
 *
 * In many cases, `ngRepeat` can be used on `<option>` elements instead of {@link ng.directive:ngOptions
 * ngOptions} to achieve a similar result. However, `ngOptions` provides some benefits:
 * - more flexibility in how the `<select>`'s model is assigned via the `select` **`as`** part of the
 * comprehension expression
 * - reduced memory consumption by not creating a new scope for each repeated instance
 * - increased render speed by creating the options in a documentFragment instead of individually
 *
 * Specifically, select with repeated options slows down significantly starting at 2000 options in
 * Chrome and Internet Explorer / Edge.
 *
 *
 * @param {string} ngModel Assignable AngularJS expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} multiple Allows multiple options to be selected. The selected values will be
 *     bound to the model as an array.
 * @param {string=} required Sets `required` validation error key if the value is not entered.
 * @param {string=} ngRequired Adds required attribute and required validation constraint to
 * the element when the ngRequired expression evaluates to true. Use ngRequired instead of required
 * when you want to data-bind to the required attribute.
 * @param {string=} ngChange AngularJS expression to be executed when selected option(s) changes due to user
 *    interaction with the select element.
 * @param {string=} ngOptions sets the options that the select is populated with and defines what is
 * set on the model on selection. See {@link ngOptions `ngOptions`}.
 * @param {string=} ngAttrSize sets the size of the select element dynamically. Uses the
 * {@link guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes ngAttr} directive.
 *
 *
 * @example
 * ### Simple `select` elements with static options
 *
 * <example name="static-select" module="staticSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="singleSelect"> Single select: </label><br>
 *     <select name="singleSelect" ng-model="data.singleSelect">
 *       <option value="option-1">Option 1</option>
 *       <option value="option-2">Option 2</option>
 *     </select><br>
 *
 *     <label for="singleSelect"> Single select with "not selected" option and dynamic option values: </label><br>
 *     <select name="singleSelect" id="singleSelect" ng-model="data.singleSelect">
 *       <option value="">---Please select---</option> <!-- not selected / blank option -->
 *       <option value="{{data.option1}}">Option 1</option> <!-- interpolation -->
 *       <option value="option-2">Option 2</option>
 *     </select><br>
 *     <button ng-click="forceUnknownOption()">Force unknown option</button><br>
 *     <tt>singleSelect = {{data.singleSelect}}</tt>
 *
 *     <hr>
 *     <label for="multipleSelect"> Multiple select: </label><br>
 *     <select name="multipleSelect" id="multipleSelect" ng-model="data.multipleSelect" multiple>
 *       <option value="option-1">Option 1</option>
 *       <option value="option-2">Option 2</option>
 *       <option value="option-3">Option 3</option>
 *     </select><br>
 *     <tt>multipleSelect = {{data.multipleSelect}}</tt><br/>
 *   </form>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('staticSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.data = {
 *       singleSelect: null,
 *       multipleSelect: [],
 *       option1: 'option-1'
 *      };
 *
 *      $scope.forceUnknownOption = function() {
 *        $scope.data.singleSelect = 'nonsense';
 *      };
 *   }]);
 * </file>
 *</example>
 *
 * @example
 * ### Using `ngRepeat` to generate `select` options
 * <example name="select-ngrepeat" module="ngrepeatSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="repeatSelect"> Repeat select: </label>
 *     <select name="repeatSelect" id="repeatSelect" ng-model="data.model">
 *       <option ng-repeat="option in data.availableOptions" value="{{option.id}}">{{option.name}}</option>
 *     </select>
 *   </form>
 *   <hr>
 *   <tt>model = {{data.model}}</tt><br/>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('ngrepeatSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.data = {
 *       model: null,
 *       availableOptions: [
 *         {id: '1', name: 'Option A'},
 *         {id: '2', name: 'Option B'},
 *         {id: '3', name: 'Option C'}
 *       ]
 *      };
 *   }]);
 * </file>
 *</example>
 *
 * @example
 * ### Using `ngValue` to bind the model to an array of objects
 * <example name="select-ngvalue" module="ngvalueSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="ngvalueselect"> ngvalue select: </label>
 *     <select size="6" name="ngvalueselect" ng-model="data.model" multiple>
 *       <option ng-repeat="option in data.availableOptions" ng-value="option.value">{{option.name}}</option>
 *     </select>
 *   </form>
 *   <hr>
 *   <pre>model = {{data.model | json}}</pre><br/>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('ngvalueSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.data = {
 *       model: null,
 *       availableOptions: [
           {value: 'myString', name: 'string'},
           {value: 1, name: 'integer'},
           {value: true, name: 'boolean'},
           {value: null, name: 'null'},
           {value: {prop: 'value'}, name: 'object'},
           {value: ['a'], name: 'array'}
 *       ]
 *      };
 *   }]);
 * </file>
 *</example>
 *
 * @example
 * ### Using `select` with `ngOptions` and setting a default value
 * See the {@link ngOptions ngOptions documentation} for more `ngOptions` usage examples.
 *
 * <example name="select-with-default-values" module="defaultValueSelect">
 * <file name="index.html">
 * <div ng-controller="ExampleController">
 *   <form name="myForm">
 *     <label for="mySelect">Make a choice:</label>
 *     <select name="mySelect" id="mySelect"
 *       ng-options="option.name for option in data.availableOptions track by option.id"
 *       ng-model="data.selectedOption"></select>
 *   </form>
 *   <hr>
 *   <tt>option = {{data.selectedOption}}</tt><br/>
 * </div>
 * </file>
 * <file name="app.js">
 *  angular.module('defaultValueSelect', [])
 *    .controller('ExampleController', ['$scope', function($scope) {
 *      $scope.data = {
 *       availableOptions: [
 *         {id: '1', name: 'Option A'},
 *         {id: '2', name: 'Option B'},
 *         {id: '3', name: 'Option C'}
 *       ],
 *       selectedOption: {id: '3', name: 'Option C'} //This sets the default value of the select in the ui
 *       };
 *   }]);
 * </file>
 *</example>
 *
 * @example
 * ### Binding `select` to a non-string value via `ngModel` parsing / formatting
 *
 * <example name="select-with-non-string-options" module="nonStringSelect">
 *   <file name="index.html">
 *     <select ng-model="model.id" convert-to-number>
 *       <option value="0">Zero</option>
 *       <option value="1">One</option>
 *       <option value="2">Two</option>
 *     </select>
 *     {{ model }}
 *   </file>
 *   <file name="app.js">
 *     angular.module('nonStringSelect', [])
 *       .run(function($rootScope) {
 *         $rootScope.model = { id: 2 };
 *       })
 *       .directive('convertToNumber', function() {
 *         return {
 *           require: 'ngModel',
 *           link: function(scope, element, attrs, ngModel) {
 *             ngModel.$parsers.push(function(val) {
 *               return parseInt(val, 10);
 *             });
 *             ngModel.$formatters.push(function(val) {
 *               return '' + val;
 *             });
 *           }
 *         };
 *       });
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     it('should initialize to model', function() {
 *       expect(element(by.model('model.id')).$('option:checked').getText()).toEqual('Two');
 *     });
 *   </file>
 * </example>
 *
 */
var selectDirective = function() {

  return {
    restrict: 'E',
    require: ['select', '?ngModel'],
    controller: SelectController,
    priority: 1,
    link: {
      pre: selectPreLink,
      post: selectPostLink
    }
  };

  function selectPreLink(scope, element, attr, ctrls) {

      var selectCtrl = ctrls[0];
      var ngModelCtrl = ctrls[1];

      // if ngModel is not defined, we don't need to do anything but set the registerOption
      // function to noop, so options don't get added internally
      if (!ngModelCtrl) {
        selectCtrl.registerOption = noop;
        return;
      }


      selectCtrl.ngModelCtrl = ngModelCtrl;

      // When the selected item(s) changes we delegate getting the value of the select control
      // to the `readValue` method, which can be changed if the select can have multiple
      // selected values or if the options are being generated by `ngOptions`
      element.on('change', function() {
        selectCtrl.removeUnknownOption();
        scope.$apply(function() {
          ngModelCtrl.$setViewValue(selectCtrl.readValue());
        });
      });

      // If the select allows multiple values then we need to modify how we read and write
      // values from and to the control; also what it means for the value to be empty and
      // we have to add an extra watch since ngModel doesn't work well with arrays - it
      // doesn't trigger rendering if only an item in the array changes.
      if (attr.multiple) {
        selectCtrl.multiple = true;

        // Read value now needs to check each option to see if it is selected
        selectCtrl.readValue = function readMultipleValue() {
          var array = [];
          forEach(element.find('option'), function(option) {
            if (option.selected && !option.disabled) {
              var val = option.value;
              array.push(val in selectCtrl.selectValueMap ? selectCtrl.selectValueMap[val] : val);
            }
          });
          return array;
        };

        // Write value now needs to set the selected property of each matching option
        selectCtrl.writeValue = function writeMultipleValue(value) {
          forEach(element.find('option'), function(option) {
            var shouldBeSelected = !!value && (includes(value, option.value) ||
                                               includes(value, selectCtrl.selectValueMap[option.value]));
            var currentlySelected = option.selected;

            // Support: IE 9-11 only, Edge 12-15+
            // In IE and Edge adding options to the selection via shift+click/UP/DOWN
            // will de-select already selected options if "selected" on those options was set
            // more than once (i.e. when the options were already selected)
            // So we only modify the selected property if necessary.
            // Note: this behavior cannot be replicated via unit tests because it only shows in the
            // actual user interface.
            if (shouldBeSelected !== currentlySelected) {
              setOptionSelectedStatus(jqLite(option), shouldBeSelected);
            }

          });
        };

        // we have to do it on each watch since ngModel watches reference, but
        // we need to work of an array, so we need to see if anything was inserted/removed
        var lastView, lastViewRef = NaN;
        scope.$watch(function selectMultipleWatch() {
          if (lastViewRef === ngModelCtrl.$viewValue && !equals(lastView, ngModelCtrl.$viewValue)) {
            lastView = shallowCopy(ngModelCtrl.$viewValue);
            ngModelCtrl.$render();
          }
          lastViewRef = ngModelCtrl.$viewValue;
        });

        // If we are a multiple select then value is now a collection
        // so the meaning of $isEmpty changes
        ngModelCtrl.$isEmpty = function(value) {
          return !value || value.length === 0;
        };

      }
    }

    function selectPostLink(scope, element, attrs, ctrls) {
      // if ngModel is not defined, we don't need to do anything
      var ngModelCtrl = ctrls[1];
      if (!ngModelCtrl) return;

      var selectCtrl = ctrls[0];

      // We delegate rendering to the `writeValue` method, which can be changed
      // if the select can have multiple selected values or if the options are being
      // generated by `ngOptions`.
      // This must be done in the postLink fn to prevent $render to be called before
      // all nodes have been linked correctly.
      ngModelCtrl.$render = function() {
        selectCtrl.writeValue(ngModelCtrl.$viewValue);
      };
    }
};


// The option directive is purely designed to communicate the existence (or lack of)
// of dynamically created (and destroyed) option elements to their containing select
// directive via its controller.
var optionDirective = ['$interpolate', function($interpolate) {
  return {
    restrict: 'E',
    priority: 100,
    compile: function(element, attr) {
      var interpolateValueFn, interpolateTextFn;

      if (isDefined(attr.ngValue)) {
        // Will be handled by registerOption
      } else if (isDefined(attr.value)) {
        // If the value attribute is defined, check if it contains an interpolation
        interpolateValueFn = $interpolate(attr.value, true);
      } else {
        // If the value attribute is not defined then we fall back to the
        // text content of the option element, which may be interpolated
        interpolateTextFn = $interpolate(element.text(), true);
        if (!interpolateTextFn) {
          attr.$set('value', element.text());
        }
      }

      return function(scope, element, attr) {
        // This is an optimization over using ^^ since we don't want to have to search
        // all the way to the root of the DOM for every single option element
        var selectCtrlName = '$selectController',
            parent = element.parent(),
            selectCtrl = parent.data(selectCtrlName) ||
              parent.parent().data(selectCtrlName); // in case we are in optgroup

        if (selectCtrl) {
          selectCtrl.registerOption(scope, element, attr, interpolateValueFn, interpolateTextFn);
        }
      };
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngRequired
 * @restrict A
 *
 * @param {expression} ngRequired AngularJS expression. If it evaluates to `true`, it sets the
 *                                `required` attribute to the element and adds the `required`
 *                                {@link ngModel.NgModelController#$validators `validator`}.
 *
 * @description
 *
 * ngRequired adds the required {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
 * It is most often used for {@link input `input`} and {@link select `select`} controls, but can also be
 * applied to custom controls.
 *
 * The directive sets the `required` attribute on the element if the AngularJS expression inside
 * `ngRequired` evaluates to true. A special directive for setting `required` is necessary because we
 * cannot use interpolation inside `required`. See the {@link guide/interpolation interpolation guide}
 * for more info.
 *
 * The validator will set the `required` error key to true if the `required` attribute is set and
 * calling {@link ngModel.NgModelController#$isEmpty `NgModelController.$isEmpty`} with the
 * {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`} returns `true`. For example, the
 * `$isEmpty()` implementation for `input[text]` checks the length of the `$viewValue`. When developing
 * custom controls, `$isEmpty()` can be overwritten to account for a $viewValue that is not string-based.
 *
 * @example
 * <example name="ngRequiredDirective" module="ngRequiredExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('ngRequiredExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.required = true;
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <form name="form">
 *         <label for="required">Toggle required: </label>
 *         <input type="checkbox" ng-model="required" id="required" />
 *         <br>
 *         <label for="input">This input must be filled if `required` is true: </label>
 *         <input type="text" ng-model="model" id="input" name="input" ng-required="required" /><br>
 *         <hr>
 *         required error set? = <code>{{form.input.$error.required}}</code><br>
 *         model = <code>{{model}}</code>
 *       </form>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
       var required = element(by.binding('form.input.$error.required'));
       var model = element(by.binding('model'));
       var input = element(by.id('input'));

       it('should set the required error', function() {
         expect(required.getText()).toContain('true');

         input.sendKeys('123');
         expect(required.getText()).not.toContain('true');
         expect(model.getText()).toContain('123');
       });
 *   </file>
 * </example>
 */
var requiredDirective = ['$parse', function($parse) {
  return {
    restrict: 'A',
    require: '?ngModel',
    link: function(scope, elm, attr, ctrl) {
      if (!ctrl) return;
      // For boolean attributes like required, presence means true
      var value = attr.hasOwnProperty('required') || $parse(attr.ngRequired)(scope);

      if (!attr.ngRequired) {
        // force truthy in case we are on non input element
        // (input elements do this automatically for boolean attributes like required)
        attr.required = true;
      }

      ctrl.$validators.required = function(modelValue, viewValue) {
        return !value || !ctrl.$isEmpty(viewValue);
      };

      attr.$observe('required', function(newVal) {

        if (value !== newVal) {
          value = newVal;
          ctrl.$validate();
        }
      });
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngPattern
 * @restrict A
 *
 * @param {expression|RegExp} ngPattern AngularJS expression that must evaluate to a `RegExp` or a `String`
 *                                      parsable into a `RegExp`, or a `RegExp` literal. See above for
 *                                      more details.
 *
 * @description
 *
 * ngPattern adds the pattern {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
 * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
 *
 * The validator sets the `pattern` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
 * does not match a RegExp which is obtained from the `ngPattern` attribute value:
 * - the value is an AngularJS expression:
 *   - If the expression evaluates to a RegExp object, then this is used directly.
 *   - If the expression evaluates to a string, then it will be converted to a RegExp after wrapping it
 *     in `^` and `$` characters. For instance, `"abc"` will be converted to `new RegExp('^abc$')`.
 * - If the value is a RegExp literal, e.g. `ngPattern="/^\d+$/"`, it is used directly.
 *
 * <div class="alert alert-info">
 * **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
 * start at the index of the last search's match, thus not taking the whole input value into
 * account.
 * </div>
 *
 * <div class="alert alert-info">
 * **Note:** This directive is also added when the plain `pattern` attribute is used, with two
 * differences:
 * <ol>
 *   <li>
 *     `ngPattern` does not set the `pattern` attribute and therefore HTML5 constraint validation is
 *     not available.
 *   </li>
 *   <li>
 *     The `ngPattern` attribute must be an expression, while the `pattern` value must be
 *     interpolated.
 *   </li>
 * </ol>
 * </div>
 *
 * @example
 * <example name="ngPatternDirective" module="ngPatternExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('ngPatternExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.regex = '\\d+';
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <form name="form">
 *         <label for="regex">Set a pattern (regex string): </label>
 *         <input type="text" ng-model="regex" id="regex" />
 *         <br>
 *         <label for="input">This input is restricted by the current pattern: </label>
 *         <input type="text" ng-model="model" id="input" name="input" ng-pattern="regex" /><br>
 *         <hr>
 *         input valid? = <code>{{form.input.$valid}}</code><br>
 *         model = <code>{{model}}</code>
 *       </form>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
       var model = element(by.binding('model'));
       var input = element(by.id('input'));

       it('should validate the input with the default pattern', function() {
         input.sendKeys('aaa');
         expect(model.getText()).not.toContain('aaa');

         input.clear().then(function() {
           input.sendKeys('123');
           expect(model.getText()).toContain('123');
         });
       });
 *   </file>
 * </example>
 */
var patternDirective = ['$parse', function($parse) {
  return {
    restrict: 'A',
    require: '?ngModel',
    compile: function(tElm, tAttr) {
      var patternExp;
      var parseFn;

      if (tAttr.ngPattern) {
        patternExp = tAttr.ngPattern;

        // ngPattern might be a scope expression, or an inlined regex, which is not parsable.
        // We get value of the attribute here, so we can compare the old and the new value
        // in the observer to avoid unnecessary validations
        if (tAttr.ngPattern.charAt(0) === '/' && REGEX_STRING_REGEXP.test(tAttr.ngPattern)) {
          parseFn = function() { return tAttr.ngPattern; };
        } else {
          parseFn = $parse(tAttr.ngPattern);
        }
      }

      return function(scope, elm, attr, ctrl) {
        if (!ctrl) return;

        var attrVal = attr.pattern;

        if (attr.ngPattern) {
          attrVal = parseFn(scope);
        } else {
          patternExp = attr.pattern;
        }

        var regexp = parsePatternAttr(attrVal, patternExp, elm);

        attr.$observe('pattern', function(newVal) {
          var oldRegexp = regexp;

          regexp = parsePatternAttr(newVal, patternExp, elm);

          if ((oldRegexp && oldRegexp.toString()) !== (regexp && regexp.toString())) {
            ctrl.$validate();
          }
        });

        ctrl.$validators.pattern = function(modelValue, viewValue) {
          // HTML5 pattern constraint validates the input value, so we validate the viewValue
          return ctrl.$isEmpty(viewValue) || isUndefined(regexp) || regexp.test(viewValue);
        };
      };
    }

  };
}];

/**
 * @ngdoc directive
 * @name ngMaxlength
 * @restrict A
 *
 * @param {expression} ngMaxlength AngularJS expression that must evaluate to a `Number` or `String`
 *                                 parsable into a `Number`. Used as value for the `maxlength`
 *                                 {@link ngModel.NgModelController#$validators validator}.
 *
 * @description
 *
 * ngMaxlength adds the maxlength {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
 * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
 *
 * The validator sets the `maxlength` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
 * is longer than the integer obtained by evaluating the AngularJS expression given in the
 * `ngMaxlength` attribute value.
 *
 * <div class="alert alert-info">
 * **Note:** This directive is also added when the plain `maxlength` attribute is used, with two
 * differences:
 * <ol>
 *   <li>
 *     `ngMaxlength` does not set the `maxlength` attribute and therefore HTML5 constraint
 *     validation is not available.
 *   </li>
 *   <li>
 *     The `ngMaxlength` attribute must be an expression, while the `maxlength` value must be
 *     interpolated.
 *   </li>
 * </ol>
 * </div>
 *
 * @example
 * <example name="ngMaxlengthDirective" module="ngMaxlengthExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('ngMaxlengthExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.maxlength = 5;
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <form name="form">
 *         <label for="maxlength">Set a maxlength: </label>
 *         <input type="number" ng-model="maxlength" id="maxlength" />
 *         <br>
 *         <label for="input">This input is restricted by the current maxlength: </label>
 *         <input type="text" ng-model="model" id="input" name="input" ng-maxlength="maxlength" /><br>
 *         <hr>
 *         input valid? = <code>{{form.input.$valid}}</code><br>
 *         model = <code>{{model}}</code>
 *       </form>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
       var model = element(by.binding('model'));
       var input = element(by.id('input'));

       it('should validate the input with the default maxlength', function() {
         input.sendKeys('abcdef');
         expect(model.getText()).not.toContain('abcdef');

         input.clear().then(function() {
           input.sendKeys('abcde');
           expect(model.getText()).toContain('abcde');
         });
       });
 *   </file>
 * </example>
 */
var maxlengthDirective = ['$parse', function($parse) {
  return {
    restrict: 'A',
    require: '?ngModel',
    link: function(scope, elm, attr, ctrl) {
      if (!ctrl) return;

      var maxlength = attr.maxlength || $parse(attr.ngMaxlength)(scope);
      var maxlengthParsed = parseLength(maxlength);

      attr.$observe('maxlength', function(value) {
        if (maxlength !== value) {
          maxlengthParsed = parseLength(value);
          maxlength = value;
          ctrl.$validate();
        }
      });
      ctrl.$validators.maxlength = function(modelValue, viewValue) {
        return (maxlengthParsed < 0) || ctrl.$isEmpty(viewValue) || (viewValue.length <= maxlengthParsed);
      };
    }
  };
}];

/**
 * @ngdoc directive
 * @name ngMinlength
 * @restrict A
 *
 * @param {expression} ngMinlength AngularJS expression that must evaluate to a `Number` or `String`
 *                                 parsable into a `Number`. Used as value for the `minlength`
 *                                 {@link ngModel.NgModelController#$validators validator}.
 *
 * @description
 *
 * ngMinlength adds the minlength {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
 * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
 *
 * The validator sets the `minlength` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
 * is shorter than the integer obtained by evaluating the AngularJS expression given in the
 * `ngMinlength` attribute value.
 *
 * <div class="alert alert-info">
 * **Note:** This directive is also added when the plain `minlength` attribute is used, with two
 * differences:
 * <ol>
 *   <li>
 *     `ngMinlength` does not set the `minlength` attribute and therefore HTML5 constraint
 *     validation is not available.
 *   </li>
 *   <li>
 *     The `ngMinlength` value must be an expression, while the `minlength` value must be
 *     interpolated.
 *   </li>
 * </ol>
 * </div>
 *
 * @example
 * <example name="ngMinlengthDirective" module="ngMinlengthExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('ngMinlengthExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.minlength = 3;
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <form name="form">
 *         <label for="minlength">Set a minlength: </label>
 *         <input type="number" ng-model="minlength" id="minlength" />
 *         <br>
 *         <label for="input">This input is restricted by the current minlength: </label>
 *         <input type="text" ng-model="model" id="input" name="input" ng-minlength="minlength" /><br>
 *         <hr>
 *         input valid? = <code>{{form.input.$valid}}</code><br>
 *         model = <code>{{model}}</code>
 *       </form>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
       var model = element(by.binding('model'));
       var input = element(by.id('input'));

       it('should validate the input with the default minlength', function() {
         input.sendKeys('ab');
         expect(model.getText()).not.toContain('ab');

         input.sendKeys('abc');
         expect(model.getText()).toContain('abc');
       });
 *   </file>
 * </example>
 */
var minlengthDirective = ['$parse', function($parse) {
  return {
    restrict: 'A',
    require: '?ngModel',
    link: function(scope, elm, attr, ctrl) {
      if (!ctrl) return;

      var minlength = attr.minlength || $parse(attr.ngMinlength)(scope);
      var minlengthParsed = parseLength(minlength) || -1;

      attr.$observe('minlength', function(value) {
        if (minlength !== value) {
          minlengthParsed = parseLength(value) || -1;
          minlength = value;
          ctrl.$validate();
        }

      });
      ctrl.$validators.minlength = function(modelValue, viewValue) {
        return ctrl.$isEmpty(viewValue) || viewValue.length >= minlengthParsed;
      };
    }
  };
}];


function parsePatternAttr(regex, patternExp, elm) {
  if (!regex) return undefined;

  if (isString(regex)) {
    regex = new RegExp('^' + regex + '$');
  }

  if (!regex.test) {
    throw minErr('ngPattern')('noregexp',
      'Expected {0} to be a RegExp but was {1}. Element: {2}', patternExp,
      regex, startingTag(elm));
  }

  return regex;
}

function parseLength(val) {
  var intVal = toInt(val);
  return isNumberNaN(intVal) ? -1 : intVal;
}

if (window.angular.bootstrap) {
  // AngularJS is already loaded, so we can return here...
  if (window.console) {
    console.log('WARNING: Tried to load AngularJS more than once.');
  }
  return;
}

// try to bind to jquery now so that one can write jqLite(fn)
// but we will rebind on bootstrap again.
bindJQuery();

publishExternalAPI(angular);

angular.module("ngLocale", [], ["$provide", function($provide) {
var PLURAL_CATEGORY = {ZERO: "zero", ONE: "one", TWO: "two", FEW: "few", MANY: "many", OTHER: "other"};
function getDecimals(n) {
  n = n + '';
  var i = n.indexOf('.');
  return (i == -1) ? 0 : n.length - i - 1;
}

function getVF(n, opt_precision) {
  var v = opt_precision;

  if (undefined === v) {
    v = Math.min(getDecimals(n), 3);
  }

  var base = Math.pow(10, v);
  var f = ((n * base) | 0) % base;
  return {v: v, f: f};
}

$provide.value("$locale", {
  "DATETIME_FORMATS": {
    "AMPMS": [
      "AM",
      "PM"
    ],
    "DAY": [
      "Sunday",
      "Monday",
      "Tuesday",
      "Wednesday",
      "Thursday",
      "Friday",
      "Saturday"
    ],
    "ERANAMES": [
      "Before Christ",
      "Anno Domini"
    ],
    "ERAS": [
      "BC",
      "AD"
    ],
    "FIRSTDAYOFWEEK": 6,
    "MONTH": [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
    ],
    "SHORTDAY": [
      "Sun",
      "Mon",
      "Tue",
      "Wed",
      "Thu",
      "Fri",
      "Sat"
    ],
    "SHORTMONTH": [
      "Jan",
      "Feb",
      "Mar",
      "Apr",
      "May",
      "Jun",
      "Jul",
      "Aug",
      "Sep",
      "Oct",
      "Nov",
      "Dec"
    ],
    "STANDALONEMONTH": [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
    ],
    "WEEKENDRANGE": [
      5,
      6
    ],
    "fullDate": "EEEE, MMMM d, y",
    "longDate": "MMMM d, y",
    "medium": "MMM d, y h:mm:ss a",
    "mediumDate": "MMM d, y",
    "mediumTime": "h:mm:ss a",
    "short": "M/d/yy h:mm a",
    "shortDate": "M/d/yy",
    "shortTime": "h:mm a"
  },
  "NUMBER_FORMATS": {
    "CURRENCY_SYM": "$",
    "DECIMAL_SEP": ".",
    "GROUP_SEP": ",",
    "PATTERNS": [
      {
        "gSize": 3,
        "lgSize": 3,
        "maxFrac": 3,
        "minFrac": 0,
        "minInt": 1,
        "negPre": "-",
        "negSuf": "",
        "posPre": "",
        "posSuf": ""
      },
      {
        "gSize": 3,
        "lgSize": 3,
        "maxFrac": 2,
        "minFrac": 2,
        "minInt": 1,
        "negPre": "-\u00a4",
        "negSuf": "",
        "posPre": "\u00a4",
        "posSuf": ""
      }
    ]
  },
  "id": "en-us",
  "localeID": "en_US",
  "pluralCat": function(n, opt_precision) {  var i = n | 0;  var vf = getVF(n, opt_precision);  if (i == 1 && vf.v == 0) {    return PLURAL_CATEGORY.ONE;  }  return PLURAL_CATEGORY.OTHER;}
});
}]);

  jqLite(function() {
    angularInit(window.document, bootstrap);
    toString = Object.prototype.toString;
  });

})(window);

!window.angular.$$csp().noInlineStyle && window.angular.element(document.head).prepend(window.angular.element('<style>').text('@charset "UTF-8";[ng\\:cloak],[ng-cloak],[data-ng-cloak],[x-ng-cloak],.ng-cloak,.x-ng-cloak,.ng-hide:not(.ng-hide-animate){display:none !important;}ng\\:form{display:block;}.ng-animate-shim{visibility:hidden;}.ng-anchor{position:absolute;}'));
/**
 * @license AngularJS v1.8.0
 * (c) 2010-2020 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function(window, angular) {'use strict';

var ELEMENT_NODE = 1;
var COMMENT_NODE = 8;

var ADD_CLASS_SUFFIX = '-add';
var REMOVE_CLASS_SUFFIX = '-remove';
var EVENT_CLASS_PREFIX = 'ng-';
var ACTIVE_CLASS_SUFFIX = '-active';
var PREPARE_CLASS_SUFFIX = '-prepare';

var NG_ANIMATE_CLASSNAME = 'ng-animate';
var NG_ANIMATE_CHILDREN_DATA = '$$ngAnimateChildren';

// Detect proper transitionend/animationend event names.
var CSS_PREFIX = '', TRANSITION_PROP, TRANSITIONEND_EVENT, ANIMATION_PROP, ANIMATIONEND_EVENT;

// If unprefixed events are not supported but webkit-prefixed are, use the latter.
// Otherwise, just use W3C names, browsers not supporting them at all will just ignore them.
// Note: Chrome implements `window.onwebkitanimationend` and doesn't implement `window.onanimationend`
// but at the same time dispatches the `animationend` event and not `webkitAnimationEnd`.
// Register both events in case `window.onanimationend` is not supported because of that,
// do the same for `transitionend` as Safari is likely to exhibit similar behavior.
// Also, the only modern browser that uses vendor prefixes for transitions/keyframes is webkit
// therefore there is no reason to test anymore for other vendor prefixes:
// http://caniuse.com/#search=transition
if ((window.ontransitionend === undefined) && (window.onwebkittransitionend !== undefined)) {
  CSS_PREFIX = '-webkit-';
  TRANSITION_PROP = 'WebkitTransition';
  TRANSITIONEND_EVENT = 'webkitTransitionEnd transitionend';
} else {
  TRANSITION_PROP = 'transition';
  TRANSITIONEND_EVENT = 'transitionend';
}

if ((window.onanimationend === undefined) && (window.onwebkitanimationend !== undefined)) {
  CSS_PREFIX = '-webkit-';
  ANIMATION_PROP = 'WebkitAnimation';
  ANIMATIONEND_EVENT = 'webkitAnimationEnd animationend';
} else {
  ANIMATION_PROP = 'animation';
  ANIMATIONEND_EVENT = 'animationend';
}

var DURATION_KEY = 'Duration';
var PROPERTY_KEY = 'Property';
var DELAY_KEY = 'Delay';
var TIMING_KEY = 'TimingFunction';
var ANIMATION_ITERATION_COUNT_KEY = 'IterationCount';
var ANIMATION_PLAYSTATE_KEY = 'PlayState';
var SAFE_FAST_FORWARD_DURATION_VALUE = 9999;

var ANIMATION_DELAY_PROP = ANIMATION_PROP + DELAY_KEY;
var ANIMATION_DURATION_PROP = ANIMATION_PROP + DURATION_KEY;
var TRANSITION_DELAY_PROP = TRANSITION_PROP + DELAY_KEY;
var TRANSITION_DURATION_PROP = TRANSITION_PROP + DURATION_KEY;

var ngMinErr = angular.$$minErr('ng');
function assertArg(arg, name, reason) {
  if (!arg) {
    throw ngMinErr('areq', 'Argument \'{0}\' is {1}', (name || '?'), (reason || 'required'));
  }
  return arg;
}

function mergeClasses(a,b) {
  if (!a && !b) return '';
  if (!a) return b;
  if (!b) return a;
  if (isArray(a)) a = a.join(' ');
  if (isArray(b)) b = b.join(' ');
  return a + ' ' + b;
}

function packageStyles(options) {
  var styles = {};
  if (options && (options.to || options.from)) {
    styles.to = options.to;
    styles.from = options.from;
  }
  return styles;
}

function pendClasses(classes, fix, isPrefix) {
  var className = '';
  classes = isArray(classes)
      ? classes
      : classes && isString(classes) && classes.length
          ? classes.split(/\s+/)
          : [];
  forEach(classes, function(klass, i) {
    if (klass && klass.length > 0) {
      className += (i > 0) ? ' ' : '';
      className += isPrefix ? fix + klass
                            : klass + fix;
    }
  });
  return className;
}

function removeFromArray(arr, val) {
  var index = arr.indexOf(val);
  if (val >= 0) {
    arr.splice(index, 1);
  }
}

function stripCommentsFromElement(element) {
  if (element instanceof jqLite) {
    switch (element.length) {
      case 0:
        return element;

      case 1:
        // there is no point of stripping anything if the element
        // is the only element within the jqLite wrapper.
        // (it's important that we retain the element instance.)
        if (element[0].nodeType === ELEMENT_NODE) {
          return element;
        }
        break;

      default:
        return jqLite(extractElementNode(element));
    }
  }

  if (element.nodeType === ELEMENT_NODE) {
    return jqLite(element);
  }
}

function extractElementNode(element) {
  if (!element[0]) return element;
  for (var i = 0; i < element.length; i++) {
    var elm = element[i];
    if (elm.nodeType === ELEMENT_NODE) {
      return elm;
    }
  }
}

function $$addClass($$jqLite, element, className) {
  forEach(element, function(elm) {
    $$jqLite.addClass(elm, className);
  });
}

function $$removeClass($$jqLite, element, className) {
  forEach(element, function(elm) {
    $$jqLite.removeClass(elm, className);
  });
}

function applyAnimationClassesFactory($$jqLite) {
  return function(element, options) {
    if (options.addClass) {
      $$addClass($$jqLite, element, options.addClass);
      options.addClass = null;
    }
    if (options.removeClass) {
      $$removeClass($$jqLite, element, options.removeClass);
      options.removeClass = null;
    }
  };
}

function prepareAnimationOptions(options) {
  options = options || {};
  if (!options.$$prepared) {
    var domOperation = options.domOperation || noop;
    options.domOperation = function() {
      options.$$domOperationFired = true;
      domOperation();
      domOperation = noop;
    };
    options.$$prepared = true;
  }
  return options;
}

function applyAnimationStyles(element, options) {
  applyAnimationFromStyles(element, options);
  applyAnimationToStyles(element, options);
}

function applyAnimationFromStyles(element, options) {
  if (options.from) {
    element.css(options.from);
    options.from = null;
  }
}

function applyAnimationToStyles(element, options) {
  if (options.to) {
    element.css(options.to);
    options.to = null;
  }
}

function mergeAnimationDetails(element, oldAnimation, newAnimation) {
  var target = oldAnimation.options || {};
  var newOptions = newAnimation.options || {};

  var toAdd = (target.addClass || '') + ' ' + (newOptions.addClass || '');
  var toRemove = (target.removeClass || '') + ' ' + (newOptions.removeClass || '');
  var classes = resolveElementClasses(element.attr('class'), toAdd, toRemove);

  if (newOptions.preparationClasses) {
    target.preparationClasses = concatWithSpace(newOptions.preparationClasses, target.preparationClasses);
    delete newOptions.preparationClasses;
  }

  // noop is basically when there is no callback; otherwise something has been set
  var realDomOperation = target.domOperation !== noop ? target.domOperation : null;

  extend(target, newOptions);

  // TODO(matsko or sreeramu): proper fix is to maintain all animation callback in array and call at last,but now only leave has the callback so no issue with this.
  if (realDomOperation) {
    target.domOperation = realDomOperation;
  }

  if (classes.addClass) {
    target.addClass = classes.addClass;
  } else {
    target.addClass = null;
  }

  if (classes.removeClass) {
    target.removeClass = classes.removeClass;
  } else {
    target.removeClass = null;
  }

  oldAnimation.addClass = target.addClass;
  oldAnimation.removeClass = target.removeClass;

  return target;
}

function resolveElementClasses(existing, toAdd, toRemove) {
  var ADD_CLASS = 1;
  var REMOVE_CLASS = -1;

  var flags = {};
  existing = splitClassesToLookup(existing);

  toAdd = splitClassesToLookup(toAdd);
  forEach(toAdd, function(value, key) {
    flags[key] = ADD_CLASS;
  });

  toRemove = splitClassesToLookup(toRemove);
  forEach(toRemove, function(value, key) {
    flags[key] = flags[key] === ADD_CLASS ? null : REMOVE_CLASS;
  });

  var classes = {
    addClass: '',
    removeClass: ''
  };

  forEach(flags, function(val, klass) {
    var prop, allow;
    if (val === ADD_CLASS) {
      prop = 'addClass';
      allow = !existing[klass] || existing[klass + REMOVE_CLASS_SUFFIX];
    } else if (val === REMOVE_CLASS) {
      prop = 'removeClass';
      allow = existing[klass] || existing[klass + ADD_CLASS_SUFFIX];
    }
    if (allow) {
      if (classes[prop].length) {
        classes[prop] += ' ';
      }
      classes[prop] += klass;
    }
  });

  function splitClassesToLookup(classes) {
    if (isString(classes)) {
      classes = classes.split(' ');
    }

    var obj = {};
    forEach(classes, function(klass) {
      // sometimes the split leaves empty string values
      // incase extra spaces were applied to the options
      if (klass.length) {
        obj[klass] = true;
      }
    });
    return obj;
  }

  return classes;
}

function getDomNode(element) {
  return (element instanceof jqLite) ? element[0] : element;
}

function applyGeneratedPreparationClasses($$jqLite, element, event, options) {
  var classes = '';
  if (event) {
    classes = pendClasses(event, EVENT_CLASS_PREFIX, true);
  }
  if (options.addClass) {
    classes = concatWithSpace(classes, pendClasses(options.addClass, ADD_CLASS_SUFFIX));
  }
  if (options.removeClass) {
    classes = concatWithSpace(classes, pendClasses(options.removeClass, REMOVE_CLASS_SUFFIX));
  }
  if (classes.length) {
    options.preparationClasses = classes;
    element.addClass(classes);
  }
}

function clearGeneratedClasses(element, options) {
  if (options.preparationClasses) {
    element.removeClass(options.preparationClasses);
    options.preparationClasses = null;
  }
  if (options.activeClasses) {
    element.removeClass(options.activeClasses);
    options.activeClasses = null;
  }
}

function blockKeyframeAnimations(node, applyBlock) {
  var value = applyBlock ? 'paused' : '';
  var key = ANIMATION_PROP + ANIMATION_PLAYSTATE_KEY;
  applyInlineStyle(node, [key, value]);
  return [key, value];
}

function applyInlineStyle(node, styleTuple) {
  var prop = styleTuple[0];
  var value = styleTuple[1];
  node.style[prop] = value;
}

function concatWithSpace(a,b) {
  if (!a) return b;
  if (!b) return a;
  return a + ' ' + b;
}

var helpers = {
  blockTransitions: function(node, duration) {
    // we use a negative delay value since it performs blocking
    // yet it doesn't kill any existing transitions running on the
    // same element which makes this safe for class-based animations
    var value = duration ? '-' + duration + 's' : '';
    applyInlineStyle(node, [TRANSITION_DELAY_PROP, value]);
    return [TRANSITION_DELAY_PROP, value];
  }
};

var $$rAFSchedulerFactory = ['$$rAF', function($$rAF) {
  var queue, cancelFn;

  function scheduler(tasks) {
    // we make a copy since RAFScheduler mutates the state
    // of the passed in array variable and this would be difficult
    // to track down on the outside code
    queue = queue.concat(tasks);
    nextTick();
  }

  queue = scheduler.queue = [];

  /* waitUntilQuiet does two things:
   * 1. It will run the FINAL `fn` value only when an uncanceled RAF has passed through
   * 2. It will delay the next wave of tasks from running until the quiet `fn` has run.
   *
   * The motivation here is that animation code can request more time from the scheduler
   * before the next wave runs. This allows for certain DOM properties such as classes to
   * be resolved in time for the next animation to run.
   */
  scheduler.waitUntilQuiet = function(fn) {
    if (cancelFn) cancelFn();

    cancelFn = $$rAF(function() {
      cancelFn = null;
      fn();
      nextTick();
    });
  };

  return scheduler;

  function nextTick() {
    if (!queue.length) return;

    var items = queue.shift();
    for (var i = 0; i < items.length; i++) {
      items[i]();
    }

    if (!cancelFn) {
      $$rAF(function() {
        if (!cancelFn) nextTick();
      });
    }
  }
}];

/**
 * @ngdoc directive
 * @name ngAnimateChildren
 * @restrict AE
 * @element ANY
 *
 * @description
 *
 * ngAnimateChildren allows you to specify that children of this element should animate even if any
 * of the children's parents are currently animating. By default, when an element has an active `enter`, `leave`, or `move`
 * (structural) animation, child elements that also have an active structural animation are not animated.
 *
 * Note that even if `ngAnimateChildren` is set, no child animations will run when the parent element is removed from the DOM (`leave` animation).
 *
 *
 * @param {string} ngAnimateChildren If the value is empty, `true` or `on`,
 *     then child animations are allowed. If the value is `false`, child animations are not allowed.
 *
 * @example
 * <example module="ngAnimateChildren" name="ngAnimateChildren" deps="angular-animate.js" animations="true">
     <file name="index.html">
       <div ng-controller="MainController as main">
         <label>Show container? <input type="checkbox" ng-model="main.enterElement" /></label>
         <label>Animate children? <input type="checkbox" ng-model="main.animateChildren" /></label>
         <hr>
         <div ng-animate-children="{{main.animateChildren}}">
           <div ng-if="main.enterElement" class="container">
             List of items:
             <div ng-repeat="item in [0, 1, 2, 3]" class="item">Item {{item}}</div>
           </div>
         </div>
       </div>
     </file>
     <file name="animations.css">

      .container.ng-enter,
      .container.ng-leave {
        transition: all ease 1.5s;
      }

      .container.ng-enter,
      .container.ng-leave-active {
        opacity: 0;
      }

      .container.ng-leave,
      .container.ng-enter-active {
        opacity: 1;
      }

      .item {
        background: firebrick;
        color: #FFF;
        margin-bottom: 10px;
      }

      .item.ng-enter,
      .item.ng-leave {
        transition: transform 1.5s ease;
      }

      .item.ng-enter {
        transform: translateX(50px);
      }

      .item.ng-enter-active {
        transform: translateX(0);
      }
    </file>
    <file name="script.js">
      angular.module('ngAnimateChildren', ['ngAnimate'])
        .controller('MainController', function MainController() {
          this.animateChildren = false;
          this.enterElement = false;
        });
    </file>
  </example>
 */
var $$AnimateChildrenDirective = ['$interpolate', function($interpolate) {
  return {
    link: function(scope, element, attrs) {
      var val = attrs.ngAnimateChildren;
      if (isString(val) && val.length === 0) { //empty attribute
        element.data(NG_ANIMATE_CHILDREN_DATA, true);
      } else {
        // Interpolate and set the value, so that it is available to
        // animations that run right after compilation
        setData($interpolate(val)(scope));
        attrs.$observe('ngAnimateChildren', setData);
      }

      function setData(value) {
        value = value === 'on' || value === 'true';
        element.data(NG_ANIMATE_CHILDREN_DATA, value);
      }
    }
  };
}];

/* exported $AnimateCssProvider */

var ANIMATE_TIMER_KEY = '$$animateCss';

/**
 * @ngdoc service
 * @name $animateCss
 * @kind object
 *
 * @description
 * The `$animateCss` service is a useful utility to trigger customized CSS-based transitions/keyframes
 * from a JavaScript-based animation or directly from a directive. The purpose of `$animateCss` is NOT
 * to side-step how `$animate` and ngAnimate work, but the goal is to allow pre-existing animations or
 * directives to create more complex animations that can be purely driven using CSS code.
 *
 * Note that only browsers that support CSS transitions and/or keyframe animations are capable of
 * rendering animations triggered via `$animateCss` (bad news for IE9 and lower).
 *
 * ## General Use
 * Once again, `$animateCss` is designed to be used inside of a registered JavaScript animation that
 * is powered by ngAnimate. It is possible to use `$animateCss` directly inside of a directive, however,
 * any automatic control over cancelling animations and/or preventing animations from being run on
 * child elements will not be handled by AngularJS. For this to work as expected, please use `$animate` to
 * trigger the animation and then setup a JavaScript animation that injects `$animateCss` to trigger
 * the CSS animation.
 *
 * The example below shows how we can create a folding animation on an element using `ng-if`:
 *
 * ```html
 * <!-- notice the `fold-animation` CSS class -->
 * <div ng-if="onOff" class="fold-animation">
 *   This element will go BOOM
 * </div>
 * <button ng-click="onOff=true">Fold In</button>
 * ```
 *
 * Now we create the **JavaScript animation** that will trigger the CSS transition:
 *
 * ```js
 * ngModule.animation('.fold-animation', ['$animateCss', function($animateCss) {
 *   return {
 *     enter: function(element, doneFn) {
 *       var height = element[0].offsetHeight;
 *       return $animateCss(element, {
 *         from: { height:'0px' },
 *         to: { height:height + 'px' },
 *         duration: 1 // one second
 *       });
 *     }
 *   }
 * }]);
 * ```
 *
 * ## More Advanced Uses
 *
 * `$animateCss` is the underlying code that ngAnimate uses to power **CSS-based animations** behind the scenes. Therefore CSS hooks
 * like `.ng-EVENT`, `.ng-EVENT-active`, `.ng-EVENT-stagger` are all features that can be triggered using `$animateCss` via JavaScript code.
 *
 * This also means that just about any combination of adding classes, removing classes, setting styles, dynamically setting a keyframe animation,
 * applying a hardcoded duration or delay value, changing the animation easing or applying a stagger animation are all options that work with
 * `$animateCss`. The service itself is smart enough to figure out the combination of options and examine the element styling properties in order
 * to provide a working animation that will run in CSS.
 *
 * The example below showcases a more advanced version of the `.fold-animation` from the example above:
 *
 * ```js
 * ngModule.animation('.fold-animation', ['$animateCss', function($animateCss) {
 *   return {
 *     enter: function(element, doneFn) {
 *       var height = element[0].offsetHeight;
 *       return $animateCss(element, {
 *         addClass: 'red large-text pulse-twice',
 *         easing: 'ease-out',
 *         from: { height:'0px' },
 *         to: { height:height + 'px' },
 *         duration: 1 // one second
 *       });
 *     }
 *   }
 * }]);
 * ```
 *
 * Since we're adding/removing CSS classes then the CSS transition will also pick those up:
 *
 * ```css
 * /&#42; since a hardcoded duration value of 1 was provided in the JavaScript animation code,
 * the CSS classes below will be transitioned despite them being defined as regular CSS classes &#42;/
 * .red { background:red; }
 * .large-text { font-size:20px; }
 *
 * /&#42; we can also use a keyframe animation and $animateCss will make it work alongside the transition &#42;/
 * .pulse-twice {
 *   animation: 0.5s pulse linear 2;
 *   -webkit-animation: 0.5s pulse linear 2;
 * }
 *
 * @keyframes pulse {
 *   from { transform: scale(0.5); }
 *   to { transform: scale(1.5); }
 * }
 *
 * @-webkit-keyframes pulse {
 *   from { -webkit-transform: scale(0.5); }
 *   to { -webkit-transform: scale(1.5); }
 * }
 * ```
 *
 * Given this complex combination of CSS classes, styles and options, `$animateCss` will figure everything out and make the animation happen.
 *
 * ## How the Options are handled
 *
 * `$animateCss` is very versatile and intelligent when it comes to figuring out what configurations to apply to the element to ensure the animation
 * works with the options provided. Say for example we were adding a class that contained a keyframe value and we wanted to also animate some inline
 * styles using the `from` and `to` properties.
 *
 * ```js
 * var animator = $animateCss(element, {
 *   from: { background:'red' },
 *   to: { background:'blue' }
 * });
 * animator.start();
 * ```
 *
 * ```css
 * .rotating-animation {
 *   animation:0.5s rotate linear;
 *   -webkit-animation:0.5s rotate linear;
 * }
 *
 * @keyframes rotate {
 *   from { transform: rotate(0deg); }
 *   to { transform: rotate(360deg); }
 * }
 *
 * @-webkit-keyframes rotate {
 *   from { -webkit-transform: rotate(0deg); }
 *   to { -webkit-transform: rotate(360deg); }
 * }
 * ```
 *
 * The missing pieces here are that we do not have a transition set (within the CSS code nor within the `$animateCss` options) and the duration of the animation is
 * going to be detected from what the keyframe styles on the CSS class are. In this event, `$animateCss` will automatically create an inline transition
 * style matching the duration detected from the keyframe style (which is present in the CSS class that is being added) and then prepare both the transition
 * and keyframe animations to run in parallel on the element. Then when the animation is underway the provided `from` and `to` CSS styles will be applied
 * and spread across the transition and keyframe animation.
 *
 * ## What is returned
 *
 * `$animateCss` works in two stages: a preparation phase and an animation phase. Therefore when `$animateCss` is first called it will NOT actually
 * start the animation. All that is going on here is that the element is being prepared for the animation (which means that the generated CSS classes are
 * added and removed on the element). Once `$animateCss` is called it will return an object with the following properties:
 *
 * ```js
 * var animator = $animateCss(element, { ... });
 * ```
 *
 * Now what do the contents of our `animator` variable look like:
 *
 * ```js
 * {
 *   // starts the animation
 *   start: Function,
 *
 *   // ends (aborts) the animation
 *   end: Function
 * }
 * ```
 *
 * To actually start the animation we need to run `animation.start()` which will then return a promise that we can hook into to detect when the animation ends.
 * If we choose not to run the animation then we MUST run `animation.end()` to perform a cleanup on the element (since some CSS classes and styles may have been
 * applied to the element during the preparation phase). Note that all other properties such as duration, delay, transitions and keyframes are just properties
 * and that changing them will not reconfigure the parameters of the animation.
 *
 * ### runner.done() vs runner.then()
 * It is documented that `animation.start()` will return a promise object and this is true, however, there is also an additional method available on the
 * runner called `.done(callbackFn)`. The done method works the same as `.finally(callbackFn)`, however, it does **not trigger a digest to occur**.
 * Therefore, for performance reasons, it's always best to use `runner.done(callback)` instead of `runner.then()`, `runner.catch()` or `runner.finally()`
 * unless you really need a digest to kick off afterwards.
 *
 * Keep in mind that, to make this easier, ngAnimate has tweaked the JS animations API to recognize when a runner instance is returned from $animateCss
 * (so there is no need to call `runner.done(doneFn)` inside of your JavaScript animation code).
 * Check the {@link ngAnimate.$animateCss#usage animation code above} to see how this works.
 *
 * @param {DOMElement} element the element that will be animated
 * @param {object} options the animation-related options that will be applied during the animation
 *
 * * `event` - The DOM event (e.g. enter, leave, move). When used, a generated CSS class of `ng-EVENT` and `ng-EVENT-active` will be applied
 * to the element during the animation. Multiple events can be provided when spaces are used as a separator. (Note that this will not perform any DOM operation.)
 * * `structural` - Indicates that the `ng-` prefix will be added to the event class. Setting to `false` or omitting will turn `ng-EVENT` and
 * `ng-EVENT-active` in `EVENT` and `EVENT-active`. Unused if `event` is omitted.
 * * `easing` - The CSS easing value that will be applied to the transition or keyframe animation (or both).
 * * `transitionStyle` - The raw CSS transition style that will be used (e.g. `1s linear all`).
 * * `keyframeStyle` - The raw CSS keyframe animation style that will be used (e.g. `1s my_animation linear`).
 * * `from` - The starting CSS styles (a key/value object) that will be applied at the start of the animation.
 * * `to` - The ending CSS styles (a key/value object) that will be applied across the animation via a CSS transition.
 * * `addClass` - A space separated list of CSS classes that will be added to the element and spread across the animation.
 * * `removeClass` - A space separated list of CSS classes that will be removed from the element and spread across the animation.
 * * `duration` - A number value representing the total duration of the transition and/or keyframe (note that a value of 1 is 1000ms). If a value of `0`
 * is provided then the animation will be skipped entirely.
 * * `delay` - A number value representing the total delay of the transition and/or keyframe (note that a value of 1 is 1000ms). If a value of `true` is
 * used then whatever delay value is detected from the CSS classes will be mirrored on the elements styles (e.g. by setting delay true then the style value
 * of the element will be `transition-delay: DETECTED_VALUE`). Using `true` is useful when you want the CSS classes and inline styles to all share the same
 * CSS delay value.
 * * `stagger` - A numeric time value representing the delay between successively animated elements
 * ({@link ngAnimate#css-staggering-animations Click here to learn how CSS-based staggering works in ngAnimate.})
 * * `staggerIndex` - The numeric index representing the stagger item (e.g. a value of 5 is equal to the sixth item in the stagger; therefore when a
 *   `stagger` option value of `0.1` is used then there will be a stagger delay of `600ms`)
 * * `applyClassesEarly` - Whether or not the classes being added or removed will be used when detecting the animation. This is set by `$animate` when enter/leave/move animations are fired to ensure that the CSS classes are resolved in time. (Note that this will prevent any transitions from occurring on the classes being added and removed.)
 * * `cleanupStyles` - Whether or not the provided `from` and `to` styles will be removed once
 *    the animation is closed. This is useful for when the styles are used purely for the sake of
 *    the animation and do not have a lasting visual effect on the element (e.g. a collapse and open animation).
 *    By default this value is set to `false`.
 *
 * @return {object} an object with start and end methods and details about the animation.
 *
 * * `start` - The method to start the animation. This will return a `Promise` when called.
 * * `end` - This method will cancel the animation and remove all applied CSS classes and styles.
 */
var ONE_SECOND = 1000;

var ELAPSED_TIME_MAX_DECIMAL_PLACES = 3;
var CLOSING_TIME_BUFFER = 1.5;

var DETECT_CSS_PROPERTIES = {
  transitionDuration:      TRANSITION_DURATION_PROP,
  transitionDelay:         TRANSITION_DELAY_PROP,
  transitionProperty:      TRANSITION_PROP + PROPERTY_KEY,
  animationDuration:       ANIMATION_DURATION_PROP,
  animationDelay:          ANIMATION_DELAY_PROP,
  animationIterationCount: ANIMATION_PROP + ANIMATION_ITERATION_COUNT_KEY
};

var DETECT_STAGGER_CSS_PROPERTIES = {
  transitionDuration:      TRANSITION_DURATION_PROP,
  transitionDelay:         TRANSITION_DELAY_PROP,
  animationDuration:       ANIMATION_DURATION_PROP,
  animationDelay:          ANIMATION_DELAY_PROP
};

function getCssKeyframeDurationStyle(duration) {
  return [ANIMATION_DURATION_PROP, duration + 's'];
}

function getCssDelayStyle(delay, isKeyframeAnimation) {
  var prop = isKeyframeAnimation ? ANIMATION_DELAY_PROP : TRANSITION_DELAY_PROP;
  return [prop, delay + 's'];
}

function computeCssStyles($window, element, properties) {
  var styles = Object.create(null);
  var detectedStyles = $window.getComputedStyle(element) || {};
  forEach(properties, function(formalStyleName, actualStyleName) {
    var val = detectedStyles[formalStyleName];
    if (val) {
      var c = val.charAt(0);

      // only numerical-based values have a negative sign or digit as the first value
      if (c === '-' || c === '+' || c >= 0) {
        val = parseMaxTime(val);
      }

      // by setting this to null in the event that the delay is not set or is set directly as 0
      // then we can still allow for negative values to be used later on and not mistake this
      // value for being greater than any other negative value.
      if (val === 0) {
        val = null;
      }
      styles[actualStyleName] = val;
    }
  });

  return styles;
}

function parseMaxTime(str) {
  var maxValue = 0;
  var values = str.split(/\s*,\s*/);
  forEach(values, function(value) {
    // it's always safe to consider only second values and omit `ms` values since
    // getComputedStyle will always handle the conversion for us
    if (value.charAt(value.length - 1) === 's') {
      value = value.substring(0, value.length - 1);
    }
    value = parseFloat(value) || 0;
    maxValue = maxValue ? Math.max(value, maxValue) : value;
  });
  return maxValue;
}

function truthyTimingValue(val) {
  return val === 0 || val != null;
}

function getCssTransitionDurationStyle(duration, applyOnlyDuration) {
  var style = TRANSITION_PROP;
  var value = duration + 's';
  if (applyOnlyDuration) {
    style += DURATION_KEY;
  } else {
    value += ' linear all';
  }
  return [style, value];
}

// we do not reassign an already present style value since
// if we detect the style property value again we may be
// detecting styles that were added via the `from` styles.
// We make use of `isDefined` here since an empty string
// or null value (which is what getPropertyValue will return
// for a non-existing style) will still be marked as a valid
// value for the style (a falsy value implies that the style
// is to be removed at the end of the animation). If we had a simple
// "OR" statement then it would not be enough to catch that.
function registerRestorableStyles(backup, node, properties) {
  forEach(properties, function(prop) {
    backup[prop] = isDefined(backup[prop])
        ? backup[prop]
        : node.style.getPropertyValue(prop);
  });
}

var $AnimateCssProvider = ['$animateProvider', /** @this */ function($animateProvider) {

  this.$get = ['$window', '$$jqLite', '$$AnimateRunner', '$timeout', '$$animateCache',
               '$$forceReflow', '$sniffer', '$$rAFScheduler', '$$animateQueue',
       function($window,   $$jqLite,   $$AnimateRunner,   $timeout,   $$animateCache,
                $$forceReflow,   $sniffer,   $$rAFScheduler, $$animateQueue) {

    var applyAnimationClasses = applyAnimationClassesFactory($$jqLite);

    function computeCachedCssStyles(node, className, cacheKey, allowNoDuration, properties) {
      var timings = $$animateCache.get(cacheKey);

      if (!timings) {
        timings = computeCssStyles($window, node, properties);
        if (timings.animationIterationCount === 'infinite') {
          timings.animationIterationCount = 1;
        }
      }

      // if a css animation has no duration we
      // should mark that so that repeated addClass/removeClass calls are skipped
      var hasDuration = allowNoDuration || (timings.transitionDuration > 0 || timings.animationDuration > 0);

      // we keep putting this in multiple times even though the value and the cacheKey are the same
      // because we're keeping an internal tally of how many duplicate animations are detected.
      $$animateCache.put(cacheKey, timings, hasDuration);

      return timings;
    }

    function computeCachedCssStaggerStyles(node, className, cacheKey, properties) {
      var stagger;
      var staggerCacheKey = 'stagger-' + cacheKey;

      // if we have one or more existing matches of matching elements
      // containing the same parent + CSS styles (which is how cacheKey works)
      // then staggering is possible
      if ($$animateCache.count(cacheKey) > 0) {
        stagger = $$animateCache.get(staggerCacheKey);

        if (!stagger) {
          var staggerClassName = pendClasses(className, '-stagger');

          $$jqLite.addClass(node, staggerClassName);

          stagger = computeCssStyles($window, node, properties);

          // force the conversion of a null value to zero incase not set
          stagger.animationDuration = Math.max(stagger.animationDuration, 0);
          stagger.transitionDuration = Math.max(stagger.transitionDuration, 0);

          $$jqLite.removeClass(node, staggerClassName);

          $$animateCache.put(staggerCacheKey, stagger, true);
        }
      }

      return stagger || {};
    }

    var rafWaitQueue = [];
    function waitUntilQuiet(callback) {
      rafWaitQueue.push(callback);
      $$rAFScheduler.waitUntilQuiet(function() {
        $$animateCache.flush();

        // DO NOT REMOVE THIS LINE OR REFACTOR OUT THE `pageWidth` variable.
        // PLEASE EXAMINE THE `$$forceReflow` service to understand why.
        var pageWidth = $$forceReflow();

        // we use a for loop to ensure that if the queue is changed
        // during this looping then it will consider new requests
        for (var i = 0; i < rafWaitQueue.length; i++) {
          rafWaitQueue[i](pageWidth);
        }
        rafWaitQueue.length = 0;
      });
    }

    function computeTimings(node, className, cacheKey, allowNoDuration) {
      var timings = computeCachedCssStyles(node, className, cacheKey, allowNoDuration, DETECT_CSS_PROPERTIES);
      var aD = timings.animationDelay;
      var tD = timings.transitionDelay;
      timings.maxDelay = aD && tD
          ? Math.max(aD, tD)
          : (aD || tD);
      timings.maxDuration = Math.max(
          timings.animationDuration * timings.animationIterationCount,
          timings.transitionDuration);

      return timings;
    }

    return function init(element, initialOptions) {
      // all of the animation functions should create
      // a copy of the options data, however, if a
      // parent service has already created a copy then
      // we should stick to using that
      var options = initialOptions || {};
      if (!options.$$prepared) {
        options = prepareAnimationOptions(copy(options));
      }

      var restoreStyles = {};
      var node = getDomNode(element);
      if (!node
          || !node.parentNode
          || !$$animateQueue.enabled()) {
        return closeAndReturnNoopAnimator();
      }

      var temporaryStyles = [];
      var classes = element.attr('class');
      var styles = packageStyles(options);
      var animationClosed;
      var animationPaused;
      var animationCompleted;
      var runner;
      var runnerHost;
      var maxDelay;
      var maxDelayTime;
      var maxDuration;
      var maxDurationTime;
      var startTime;
      var events = [];

      if (options.duration === 0 || (!$sniffer.animations && !$sniffer.transitions)) {
        return closeAndReturnNoopAnimator();
      }

      var method = options.event && isArray(options.event)
            ? options.event.join(' ')
            : options.event;

      var isStructural = method && options.structural;
      var structuralClassName = '';
      var addRemoveClassName = '';

      if (isStructural) {
        structuralClassName = pendClasses(method, EVENT_CLASS_PREFIX, true);
      } else if (method) {
        structuralClassName = method;
      }

      if (options.addClass) {
        addRemoveClassName += pendClasses(options.addClass, ADD_CLASS_SUFFIX);
      }

      if (options.removeClass) {
        if (addRemoveClassName.length) {
          addRemoveClassName += ' ';
        }
        addRemoveClassName += pendClasses(options.removeClass, REMOVE_CLASS_SUFFIX);
      }

      // there may be a situation where a structural animation is combined together
      // with CSS classes that need to resolve before the animation is computed.
      // However this means that there is no explicit CSS code to block the animation
      // from happening (by setting 0s none in the class name). If this is the case
      // we need to apply the classes before the first rAF so we know to continue if
      // there actually is a detected transition or keyframe animation
      if (options.applyClassesEarly && addRemoveClassName.length) {
        applyAnimationClasses(element, options);
      }

      var preparationClasses = [structuralClassName, addRemoveClassName].join(' ').trim();
      var fullClassName = classes + ' ' + preparationClasses;
      var hasToStyles = styles.to && Object.keys(styles.to).length > 0;
      var containsKeyframeAnimation = (options.keyframeStyle || '').length > 0;

      // there is no way we can trigger an animation if no styles and
      // no classes are being applied which would then trigger a transition,
      // unless there a is raw keyframe value that is applied to the element.
      if (!containsKeyframeAnimation
           && !hasToStyles
           && !preparationClasses) {
        return closeAndReturnNoopAnimator();
      }

      var stagger, cacheKey = $$animateCache.cacheKey(node, method, options.addClass, options.removeClass);
      if ($$animateCache.containsCachedAnimationWithoutDuration(cacheKey)) {
        preparationClasses = null;
        return closeAndReturnNoopAnimator();
      }

      if (options.stagger > 0) {
        var staggerVal = parseFloat(options.stagger);
        stagger = {
          transitionDelay: staggerVal,
          animationDelay: staggerVal,
          transitionDuration: 0,
          animationDuration: 0
        };
      } else {
        stagger = computeCachedCssStaggerStyles(node, preparationClasses, cacheKey, DETECT_STAGGER_CSS_PROPERTIES);
      }

      if (!options.$$skipPreparationClasses) {
        $$jqLite.addClass(element, preparationClasses);
      }

      var applyOnlyDuration;

      if (options.transitionStyle) {
        var transitionStyle = [TRANSITION_PROP, options.transitionStyle];
        applyInlineStyle(node, transitionStyle);
        temporaryStyles.push(transitionStyle);
      }

      if (options.duration >= 0) {
        applyOnlyDuration = node.style[TRANSITION_PROP].length > 0;
        var durationStyle = getCssTransitionDurationStyle(options.duration, applyOnlyDuration);

        // we set the duration so that it will be picked up by getComputedStyle later
        applyInlineStyle(node, durationStyle);
        temporaryStyles.push(durationStyle);
      }

      if (options.keyframeStyle) {
        var keyframeStyle = [ANIMATION_PROP, options.keyframeStyle];
        applyInlineStyle(node, keyframeStyle);
        temporaryStyles.push(keyframeStyle);
      }

      var itemIndex = stagger
          ? options.staggerIndex >= 0
              ? options.staggerIndex
              : $$animateCache.count(cacheKey)
          : 0;

      var isFirst = itemIndex === 0;

      // this is a pre-emptive way of forcing the setup classes to be added and applied INSTANTLY
      // without causing any combination of transitions to kick in. By adding a negative delay value
      // it forces the setup class' transition to end immediately. We later then remove the negative
      // transition delay to allow for the transition to naturally do it's thing. The beauty here is
      // that if there is no transition defined then nothing will happen and this will also allow
      // other transitions to be stacked on top of each other without any chopping them out.
      if (isFirst && !options.skipBlocking) {
        helpers.blockTransitions(node, SAFE_FAST_FORWARD_DURATION_VALUE);
      }

      var timings = computeTimings(node, fullClassName, cacheKey, !isStructural);
      var relativeDelay = timings.maxDelay;
      maxDelay = Math.max(relativeDelay, 0);
      maxDuration = timings.maxDuration;

      var flags = {};
      flags.hasTransitions          = timings.transitionDuration > 0;
      flags.hasAnimations           = timings.animationDuration > 0;
      flags.hasTransitionAll        = flags.hasTransitions && timings.transitionProperty === 'all';
      flags.applyTransitionDuration = hasToStyles && (
                                        (flags.hasTransitions && !flags.hasTransitionAll)
                                         || (flags.hasAnimations && !flags.hasTransitions));
      flags.applyAnimationDuration  = options.duration && flags.hasAnimations;
      flags.applyTransitionDelay    = truthyTimingValue(options.delay) && (flags.applyTransitionDuration || flags.hasTransitions);
      flags.applyAnimationDelay     = truthyTimingValue(options.delay) && flags.hasAnimations;
      flags.recalculateTimingStyles = addRemoveClassName.length > 0;

      if (flags.applyTransitionDuration || flags.applyAnimationDuration) {
        maxDuration = options.duration ? parseFloat(options.duration) : maxDuration;

        if (flags.applyTransitionDuration) {
          flags.hasTransitions = true;
          timings.transitionDuration = maxDuration;
          applyOnlyDuration = node.style[TRANSITION_PROP + PROPERTY_KEY].length > 0;
          temporaryStyles.push(getCssTransitionDurationStyle(maxDuration, applyOnlyDuration));
        }

        if (flags.applyAnimationDuration) {
          flags.hasAnimations = true;
          timings.animationDuration = maxDuration;
          temporaryStyles.push(getCssKeyframeDurationStyle(maxDuration));
        }
      }

      if (maxDuration === 0 && !flags.recalculateTimingStyles) {
        return closeAndReturnNoopAnimator();
      }

      var activeClasses = pendClasses(preparationClasses, ACTIVE_CLASS_SUFFIX);

      if (options.delay != null) {
        var delayStyle;
        if (typeof options.delay !== 'boolean') {
          delayStyle = parseFloat(options.delay);
          // number in options.delay means we have to recalculate the delay for the closing timeout
          maxDelay = Math.max(delayStyle, 0);
        }

        if (flags.applyTransitionDelay) {
          temporaryStyles.push(getCssDelayStyle(delayStyle));
        }

        if (flags.applyAnimationDelay) {
          temporaryStyles.push(getCssDelayStyle(delayStyle, true));
        }
      }

      // we need to recalculate the delay value since we used a pre-emptive negative
      // delay value and the delay value is required for the final event checking. This
      // property will ensure that this will happen after the RAF phase has passed.
      if (options.duration == null && timings.transitionDuration > 0) {
        flags.recalculateTimingStyles = flags.recalculateTimingStyles || isFirst;
      }

      maxDelayTime = maxDelay * ONE_SECOND;
      maxDurationTime = maxDuration * ONE_SECOND;
      if (!options.skipBlocking) {
        flags.blockTransition = timings.transitionDuration > 0;
        flags.blockKeyframeAnimation = timings.animationDuration > 0 &&
                                       stagger.animationDelay > 0 &&
                                       stagger.animationDuration === 0;
      }

      if (options.from) {
        if (options.cleanupStyles) {
          registerRestorableStyles(restoreStyles, node, Object.keys(options.from));
        }
        applyAnimationFromStyles(element, options);
      }

      if (flags.blockTransition || flags.blockKeyframeAnimation) {
        applyBlocking(maxDuration);
      } else if (!options.skipBlocking) {
        helpers.blockTransitions(node, false);
      }

      // TODO(matsko): for 1.5 change this code to have an animator object for better debugging
      return {
        $$willAnimate: true,
        end: endFn,
        start: function() {
          if (animationClosed) return;

          runnerHost = {
            end: endFn,
            cancel: cancelFn,
            resume: null, //this will be set during the start() phase
            pause: null
          };

          runner = new $$AnimateRunner(runnerHost);

          waitUntilQuiet(start);

          // we don't have access to pause/resume the animation
          // since it hasn't run yet. AnimateRunner will therefore
          // set noop functions for resume and pause and they will
          // later be overridden once the animation is triggered
          return runner;
        }
      };

      function endFn() {
        close();
      }

      function cancelFn() {
        close(true);
      }

      function close(rejected) {
        // if the promise has been called already then we shouldn't close
        // the animation again
        if (animationClosed || (animationCompleted && animationPaused)) return;
        animationClosed = true;
        animationPaused = false;

        if (preparationClasses && !options.$$skipPreparationClasses) {
          $$jqLite.removeClass(element, preparationClasses);
        }

        if (activeClasses) {
          $$jqLite.removeClass(element, activeClasses);
        }

        blockKeyframeAnimations(node, false);
        helpers.blockTransitions(node, false);

        forEach(temporaryStyles, function(entry) {
          // There is only one way to remove inline style properties entirely from elements.
          // By using `removeProperty` this works, but we need to convert camel-cased CSS
          // styles down to hyphenated values.
          node.style[entry[0]] = '';
        });

        applyAnimationClasses(element, options);
        applyAnimationStyles(element, options);

        if (Object.keys(restoreStyles).length) {
          forEach(restoreStyles, function(value, prop) {
            if (value) {
              node.style.setProperty(prop, value);
            } else {
              node.style.removeProperty(prop);
            }
          });
        }

        // the reason why we have this option is to allow a synchronous closing callback
        // that is fired as SOON as the animation ends (when the CSS is removed) or if
        // the animation never takes off at all. A good example is a leave animation since
        // the element must be removed just after the animation is over or else the element
        // will appear on screen for one animation frame causing an overbearing flicker.
        if (options.onDone) {
          options.onDone();
        }

        if (events && events.length) {
          // Remove the transitionend / animationend listener(s)
          element.off(events.join(' '), onAnimationProgress);
        }

        //Cancel the fallback closing timeout and remove the timer data
        var animationTimerData = element.data(ANIMATE_TIMER_KEY);
        if (animationTimerData) {
          $timeout.cancel(animationTimerData[0].timer);
          element.removeData(ANIMATE_TIMER_KEY);
        }

        // if the preparation function fails then the promise is not setup
        if (runner) {
          runner.complete(!rejected);
        }
      }

      function applyBlocking(duration) {
        if (flags.blockTransition) {
          helpers.blockTransitions(node, duration);
        }

        if (flags.blockKeyframeAnimation) {
          blockKeyframeAnimations(node, !!duration);
        }
      }

      function closeAndReturnNoopAnimator() {
        runner = new $$AnimateRunner({
          end: endFn,
          cancel: cancelFn
        });

        // should flush the cache animation
        waitUntilQuiet(noop);
        close();

        return {
          $$willAnimate: false,
          start: function() {
            return runner;
          },
          end: endFn
        };
      }

      function onAnimationProgress(event) {
        event.stopPropagation();
        var ev = event.originalEvent || event;

        if (ev.target !== node) {
          // Since TransitionEvent / AnimationEvent bubble up,
          // we have to ignore events by finished child animations
          return;
        }

        // we now always use `Date.now()` due to the recent changes with
        // event.timeStamp in Firefox, Webkit and Chrome (see #13494 for more info)
        var timeStamp = ev.$manualTimeStamp || Date.now();

        /* Firefox (or possibly just Gecko) likes to not round values up
         * when a ms measurement is used for the animation */
        var elapsedTime = parseFloat(ev.elapsedTime.toFixed(ELAPSED_TIME_MAX_DECIMAL_PLACES));

        /* $manualTimeStamp is a mocked timeStamp value which is set
         * within browserTrigger(). This is only here so that tests can
         * mock animations properly. Real events fallback to event.timeStamp,
         * or, if they don't, then a timeStamp is automatically created for them.
         * We're checking to see if the timeStamp surpasses the expected delay,
         * but we're using elapsedTime instead of the timeStamp on the 2nd
         * pre-condition since animationPauseds sometimes close off early */
        if (Math.max(timeStamp - startTime, 0) >= maxDelayTime && elapsedTime >= maxDuration) {
          // we set this flag to ensure that if the transition is paused then, when resumed,
          // the animation will automatically close itself since transitions cannot be paused.
          animationCompleted = true;
          close();
        }
      }

      function start() {
        if (animationClosed) return;
        if (!node.parentNode) {
          close();
          return;
        }

        // even though we only pause keyframe animations here the pause flag
        // will still happen when transitions are used. Only the transition will
        // not be paused since that is not possible. If the animation ends when
        // paused then it will not complete until unpaused or cancelled.
        var playPause = function(playAnimation) {
          if (!animationCompleted) {
            animationPaused = !playAnimation;
            if (timings.animationDuration) {
              var value = blockKeyframeAnimations(node, animationPaused);
              if (animationPaused) {
                temporaryStyles.push(value);
              } else {
                removeFromArray(temporaryStyles, value);
              }
            }
          } else if (animationPaused && playAnimation) {
            animationPaused = false;
            close();
          }
        };

        // checking the stagger duration prevents an accidentally cascade of the CSS delay style
        // being inherited from the parent. If the transition duration is zero then we can safely
        // rely that the delay value is an intentional stagger delay style.
        var maxStagger = itemIndex > 0
                         && ((timings.transitionDuration && stagger.transitionDuration === 0) ||
                            (timings.animationDuration && stagger.animationDuration === 0))
                         && Math.max(stagger.animationDelay, stagger.transitionDelay);
        if (maxStagger) {
          $timeout(triggerAnimationStart,
                   Math.floor(maxStagger * itemIndex * ONE_SECOND),
                   false);
        } else {
          triggerAnimationStart();
        }

        // this will decorate the existing promise runner with pause/resume methods
        runnerHost.resume = function() {
          playPause(true);
        };

        runnerHost.pause = function() {
          playPause(false);
        };

        function triggerAnimationStart() {
          // just incase a stagger animation kicks in when the animation
          // itself was cancelled entirely
          if (animationClosed) return;

          applyBlocking(false);

          forEach(temporaryStyles, function(entry) {
            var key = entry[0];
            var value = entry[1];
            node.style[key] = value;
          });

          applyAnimationClasses(element, options);
          $$jqLite.addClass(element, activeClasses);

          if (flags.recalculateTimingStyles) {
            fullClassName = node.getAttribute('class') + ' ' + preparationClasses;
            cacheKey = $$animateCache.cacheKey(node, method, options.addClass, options.removeClass);

            timings = computeTimings(node, fullClassName, cacheKey, false);
            relativeDelay = timings.maxDelay;
            maxDelay = Math.max(relativeDelay, 0);
            maxDuration = timings.maxDuration;

            if (maxDuration === 0) {
              close();
              return;
            }

            flags.hasTransitions = timings.transitionDuration > 0;
            flags.hasAnimations = timings.animationDuration > 0;
          }

          if (flags.applyAnimationDelay) {
            relativeDelay = typeof options.delay !== 'boolean' && truthyTimingValue(options.delay)
                  ? parseFloat(options.delay)
                  : relativeDelay;

            maxDelay = Math.max(relativeDelay, 0);
            timings.animationDelay = relativeDelay;
            delayStyle = getCssDelayStyle(relativeDelay, true);
            temporaryStyles.push(delayStyle);
            node.style[delayStyle[0]] = delayStyle[1];
          }

          maxDelayTime = maxDelay * ONE_SECOND;
          maxDurationTime = maxDuration * ONE_SECOND;

          if (options.easing) {
            var easeProp, easeVal = options.easing;
            if (flags.hasTransitions) {
              easeProp = TRANSITION_PROP + TIMING_KEY;
              temporaryStyles.push([easeProp, easeVal]);
              node.style[easeProp] = easeVal;
            }
            if (flags.hasAnimations) {
              easeProp = ANIMATION_PROP + TIMING_KEY;
              temporaryStyles.push([easeProp, easeVal]);
              node.style[easeProp] = easeVal;
            }
          }

          if (timings.transitionDuration) {
            events.push(TRANSITIONEND_EVENT);
          }

          if (timings.animationDuration) {
            events.push(ANIMATIONEND_EVENT);
          }

          startTime = Date.now();
          var timerTime = maxDelayTime + CLOSING_TIME_BUFFER * maxDurationTime;
          var endTime = startTime + timerTime;

          var animationsData = element.data(ANIMATE_TIMER_KEY) || [];
          var setupFallbackTimer = true;
          if (animationsData.length) {
            var currentTimerData = animationsData[0];
            setupFallbackTimer = endTime > currentTimerData.expectedEndTime;
            if (setupFallbackTimer) {
              $timeout.cancel(currentTimerData.timer);
            } else {
              animationsData.push(close);
            }
          }

          if (setupFallbackTimer) {
            var timer = $timeout(onAnimationExpired, timerTime, false);
            animationsData[0] = {
              timer: timer,
              expectedEndTime: endTime
            };
            animationsData.push(close);
            element.data(ANIMATE_TIMER_KEY, animationsData);
          }

          if (events.length) {
            element.on(events.join(' '), onAnimationProgress);
          }

          if (options.to) {
            if (options.cleanupStyles) {
              registerRestorableStyles(restoreStyles, node, Object.keys(options.to));
            }
            applyAnimationToStyles(element, options);
          }
        }

        function onAnimationExpired() {
          var animationsData = element.data(ANIMATE_TIMER_KEY);

          // this will be false in the event that the element was
          // removed from the DOM (via a leave animation or something
          // similar)
          if (animationsData) {
            for (var i = 1; i < animationsData.length; i++) {
              animationsData[i]();
            }
            element.removeData(ANIMATE_TIMER_KEY);
          }
        }
      }
    };
  }];
}];

var $$AnimateCssDriverProvider = ['$$animationProvider', /** @this */ function($$animationProvider) {
  $$animationProvider.drivers.push('$$animateCssDriver');

  var NG_ANIMATE_SHIM_CLASS_NAME = 'ng-animate-shim';
  var NG_ANIMATE_ANCHOR_CLASS_NAME = 'ng-anchor';

  var NG_OUT_ANCHOR_CLASS_NAME = 'ng-anchor-out';
  var NG_IN_ANCHOR_CLASS_NAME = 'ng-anchor-in';

  function isDocumentFragment(node) {
    return node.parentNode && node.parentNode.nodeType === 11;
  }

  this.$get = ['$animateCss', '$rootScope', '$$AnimateRunner', '$rootElement', '$sniffer', '$$jqLite', '$document',
       function($animateCss,   $rootScope,   $$AnimateRunner,   $rootElement,   $sniffer,   $$jqLite,   $document) {

    // only browsers that support these properties can render animations
    if (!$sniffer.animations && !$sniffer.transitions) return noop;

    var bodyNode = $document[0].body;
    var rootNode = getDomNode($rootElement);

    var rootBodyElement = jqLite(
      // this is to avoid using something that exists outside of the body
      // we also special case the doc fragment case because our unit test code
      // appends the $rootElement to the body after the app has been bootstrapped
      isDocumentFragment(rootNode) || bodyNode.contains(rootNode) ? rootNode : bodyNode
    );

    return function initDriverFn(animationDetails) {
      return animationDetails.from && animationDetails.to
          ? prepareFromToAnchorAnimation(animationDetails.from,
                                         animationDetails.to,
                                         animationDetails.classes,
                                         animationDetails.anchors)
          : prepareRegularAnimation(animationDetails);
    };

    function filterCssClasses(classes) {
      //remove all the `ng-` stuff
      return classes.replace(/\bng-\S+\b/g, '');
    }

    function getUniqueValues(a, b) {
      if (isString(a)) a = a.split(' ');
      if (isString(b)) b = b.split(' ');
      return a.filter(function(val) {
        return b.indexOf(val) === -1;
      }).join(' ');
    }

    function prepareAnchoredAnimation(classes, outAnchor, inAnchor) {
      var clone = jqLite(getDomNode(outAnchor).cloneNode(true));
      var startingClasses = filterCssClasses(getClassVal(clone));

      outAnchor.addClass(NG_ANIMATE_SHIM_CLASS_NAME);
      inAnchor.addClass(NG_ANIMATE_SHIM_CLASS_NAME);

      clone.addClass(NG_ANIMATE_ANCHOR_CLASS_NAME);

      rootBodyElement.append(clone);

      var animatorIn, animatorOut = prepareOutAnimation();

      // the user may not end up using the `out` animation and
      // only making use of the `in` animation or vice-versa.
      // In either case we should allow this and not assume the
      // animation is over unless both animations are not used.
      if (!animatorOut) {
        animatorIn = prepareInAnimation();
        if (!animatorIn) {
          return end();
        }
      }

      var startingAnimator = animatorOut || animatorIn;

      return {
        start: function() {
          var runner;

          var currentAnimation = startingAnimator.start();
          currentAnimation.done(function() {
            currentAnimation = null;
            if (!animatorIn) {
              animatorIn = prepareInAnimation();
              if (animatorIn) {
                currentAnimation = animatorIn.start();
                currentAnimation.done(function() {
                  currentAnimation = null;
                  end();
                  runner.complete();
                });
                return currentAnimation;
              }
            }
            // in the event that there is no `in` animation
            end();
            runner.complete();
          });

          runner = new $$AnimateRunner({
            end: endFn,
            cancel: endFn
          });

          return runner;

          function endFn() {
            if (currentAnimation) {
              currentAnimation.end();
            }
          }
        }
      };

      function calculateAnchorStyles(anchor) {
        var styles = {};

        var coords = getDomNode(anchor).getBoundingClientRect();

        // we iterate directly since safari messes up and doesn't return
        // all the keys for the coords object when iterated
        forEach(['width','height','top','left'], function(key) {
          var value = coords[key];
          switch (key) {
            case 'top':
              value += bodyNode.scrollTop;
              break;
            case 'left':
              value += bodyNode.scrollLeft;
              break;
          }
          styles[key] = Math.floor(value) + 'px';
        });
        return styles;
      }

      function prepareOutAnimation() {
        var animator = $animateCss(clone, {
          addClass: NG_OUT_ANCHOR_CLASS_NAME,
          delay: true,
          from: calculateAnchorStyles(outAnchor)
        });

        // read the comment within `prepareRegularAnimation` to understand
        // why this check is necessary
        return animator.$$willAnimate ? animator : null;
      }

      function getClassVal(element) {
        return element.attr('class') || '';
      }

      function prepareInAnimation() {
        var endingClasses = filterCssClasses(getClassVal(inAnchor));
        var toAdd = getUniqueValues(endingClasses, startingClasses);
        var toRemove = getUniqueValues(startingClasses, endingClasses);

        var animator = $animateCss(clone, {
          to: calculateAnchorStyles(inAnchor),
          addClass: NG_IN_ANCHOR_CLASS_NAME + ' ' + toAdd,
          removeClass: NG_OUT_ANCHOR_CLASS_NAME + ' ' + toRemove,
          delay: true
        });

        // read the comment within `prepareRegularAnimation` to understand
        // why this check is necessary
        return animator.$$willAnimate ? animator : null;
      }

      function end() {
        clone.remove();
        outAnchor.removeClass(NG_ANIMATE_SHIM_CLASS_NAME);
        inAnchor.removeClass(NG_ANIMATE_SHIM_CLASS_NAME);
      }
    }

    function prepareFromToAnchorAnimation(from, to, classes, anchors) {
      var fromAnimation = prepareRegularAnimation(from, noop);
      var toAnimation = prepareRegularAnimation(to, noop);

      var anchorAnimations = [];
      forEach(anchors, function(anchor) {
        var outElement = anchor['out'];
        var inElement = anchor['in'];
        var animator = prepareAnchoredAnimation(classes, outElement, inElement);
        if (animator) {
          anchorAnimations.push(animator);
        }
      });

      // no point in doing anything when there are no elements to animate
      if (!fromAnimation && !toAnimation && anchorAnimations.length === 0) return;

      return {
        start: function() {
          var animationRunners = [];

          if (fromAnimation) {
            animationRunners.push(fromAnimation.start());
          }

          if (toAnimation) {
            animationRunners.push(toAnimation.start());
          }

          forEach(anchorAnimations, function(animation) {
            animationRunners.push(animation.start());
          });

          var runner = new $$AnimateRunner({
            end: endFn,
            cancel: endFn // CSS-driven animations cannot be cancelled, only ended
          });

          $$AnimateRunner.all(animationRunners, function(status) {
            runner.complete(status);
          });

          return runner;

          function endFn() {
            forEach(animationRunners, function(runner) {
              runner.end();
            });
          }
        }
      };
    }

    function prepareRegularAnimation(animationDetails) {
      var element = animationDetails.element;
      var options = animationDetails.options || {};

      if (animationDetails.structural) {
        options.event = animationDetails.event;
        options.structural = true;
        options.applyClassesEarly = true;

        // we special case the leave animation since we want to ensure that
        // the element is removed as soon as the animation is over. Otherwise
        // a flicker might appear or the element may not be removed at all
        if (animationDetails.event === 'leave') {
          options.onDone = options.domOperation;
        }
      }

      // We assign the preparationClasses as the actual animation event since
      // the internals of $animateCss will just suffix the event token values
      // with `-active` to trigger the animation.
      if (options.preparationClasses) {
        options.event = concatWithSpace(options.event, options.preparationClasses);
      }

      var animator = $animateCss(element, options);

      // the driver lookup code inside of $$animation attempts to spawn a
      // driver one by one until a driver returns a.$$willAnimate animator object.
      // $animateCss will always return an object, however, it will pass in
      // a flag as a hint as to whether an animation was detected or not
      return animator.$$willAnimate ? animator : null;
    }
  }];
}];

// TODO(matsko): use caching here to speed things up for detection
// TODO(matsko): add documentation
//  by the time...

var $$AnimateJsProvider = ['$animateProvider', /** @this */ function($animateProvider) {
  this.$get = ['$injector', '$$AnimateRunner', '$$jqLite',
       function($injector,   $$AnimateRunner,   $$jqLite) {

    var applyAnimationClasses = applyAnimationClassesFactory($$jqLite);
         // $animateJs(element, 'enter');
    return function(element, event, classes, options) {
      var animationClosed = false;

      // the `classes` argument is optional and if it is not used
      // then the classes will be resolved from the element's className
      // property as well as options.addClass/options.removeClass.
      if (arguments.length === 3 && isObject(classes)) {
        options = classes;
        classes = null;
      }

      options = prepareAnimationOptions(options);
      if (!classes) {
        classes = element.attr('class') || '';
        if (options.addClass) {
          classes += ' ' + options.addClass;
        }
        if (options.removeClass) {
          classes += ' ' + options.removeClass;
        }
      }

      var classesToAdd = options.addClass;
      var classesToRemove = options.removeClass;

      // the lookupAnimations function returns a series of animation objects that are
      // matched up with one or more of the CSS classes. These animation objects are
      // defined via the module.animation factory function. If nothing is detected then
      // we don't return anything which then makes $animation query the next driver.
      var animations = lookupAnimations(classes);
      var before, after;
      if (animations.length) {
        var afterFn, beforeFn;
        if (event === 'leave') {
          beforeFn = 'leave';
          afterFn = 'afterLeave'; // TODO(matsko): get rid of this
        } else {
          beforeFn = 'before' + event.charAt(0).toUpperCase() + event.substr(1);
          afterFn = event;
        }

        if (event !== 'enter' && event !== 'move') {
          before = packageAnimations(element, event, options, animations, beforeFn);
        }
        after  = packageAnimations(element, event, options, animations, afterFn);
      }

      // no matching animations
      if (!before && !after) return;

      function applyOptions() {
        options.domOperation();
        applyAnimationClasses(element, options);
      }

      function close() {
        animationClosed = true;
        applyOptions();
        applyAnimationStyles(element, options);
      }

      var runner;

      return {
        $$willAnimate: true,
        end: function() {
          if (runner) {
            runner.end();
          } else {
            close();
            runner = new $$AnimateRunner();
            runner.complete(true);
          }
          return runner;
        },
        start: function() {
          if (runner) {
            return runner;
          }

          runner = new $$AnimateRunner();
          var closeActiveAnimations;
          var chain = [];

          if (before) {
            chain.push(function(fn) {
              closeActiveAnimations = before(fn);
            });
          }

          if (chain.length) {
            chain.push(function(fn) {
              applyOptions();
              fn(true);
            });
          } else {
            applyOptions();
          }

          if (after) {
            chain.push(function(fn) {
              closeActiveAnimations = after(fn);
            });
          }

          runner.setHost({
            end: function() {
              endAnimations();
            },
            cancel: function() {
              endAnimations(true);
            }
          });

          $$AnimateRunner.chain(chain, onComplete);
          return runner;

          function onComplete(success) {
            close(success);
            runner.complete(success);
          }

          function endAnimations(cancelled) {
            if (!animationClosed) {
              (closeActiveAnimations || noop)(cancelled);
              onComplete(cancelled);
            }
          }
        }
      };

      function executeAnimationFn(fn, element, event, options, onDone) {
        var args;
        switch (event) {
          case 'animate':
            args = [element, options.from, options.to, onDone];
            break;

          case 'setClass':
            args = [element, classesToAdd, classesToRemove, onDone];
            break;

          case 'addClass':
            args = [element, classesToAdd, onDone];
            break;

          case 'removeClass':
            args = [element, classesToRemove, onDone];
            break;

          default:
            args = [element, onDone];
            break;
        }

        args.push(options);

        var value = fn.apply(fn, args);
        if (value) {
          if (isFunction(value.start)) {
            value = value.start();
          }

          if (value instanceof $$AnimateRunner) {
            value.done(onDone);
          } else if (isFunction(value)) {
            // optional onEnd / onCancel callback
            return value;
          }
        }

        return noop;
      }

      function groupEventedAnimations(element, event, options, animations, fnName) {
        var operations = [];
        forEach(animations, function(ani) {
          var animation = ani[fnName];
          if (!animation) return;

          // note that all of these animations will run in parallel
          operations.push(function() {
            var runner;
            var endProgressCb;

            var resolved = false;
            var onAnimationComplete = function(rejected) {
              if (!resolved) {
                resolved = true;
                (endProgressCb || noop)(rejected);
                runner.complete(!rejected);
              }
            };

            runner = new $$AnimateRunner({
              end: function() {
                onAnimationComplete();
              },
              cancel: function() {
                onAnimationComplete(true);
              }
            });

            endProgressCb = executeAnimationFn(animation, element, event, options, function(result) {
              var cancelled = result === false;
              onAnimationComplete(cancelled);
            });

            return runner;
          });
        });

        return operations;
      }

      function packageAnimations(element, event, options, animations, fnName) {
        var operations = groupEventedAnimations(element, event, options, animations, fnName);
        if (operations.length === 0) {
          var a, b;
          if (fnName === 'beforeSetClass') {
            a = groupEventedAnimations(element, 'removeClass', options, animations, 'beforeRemoveClass');
            b = groupEventedAnimations(element, 'addClass', options, animations, 'beforeAddClass');
          } else if (fnName === 'setClass') {
            a = groupEventedAnimations(element, 'removeClass', options, animations, 'removeClass');
            b = groupEventedAnimations(element, 'addClass', options, animations, 'addClass');
          }

          if (a) {
            operations = operations.concat(a);
          }
          if (b) {
            operations = operations.concat(b);
          }
        }

        if (operations.length === 0) return;

        // TODO(matsko): add documentation
        return function startAnimation(callback) {
          var runners = [];
          if (operations.length) {
            forEach(operations, function(animateFn) {
              runners.push(animateFn());
            });
          }

          if (runners.length) {
            $$AnimateRunner.all(runners, callback);
          }  else {
            callback();
          }

          return function endFn(reject) {
            forEach(runners, function(runner) {
              if (reject) {
                runner.cancel();
              } else {
                runner.end();
              }
            });
          };
        };
      }
    };

    function lookupAnimations(classes) {
      classes = isArray(classes) ? classes : classes.split(' ');
      var matches = [], flagMap = {};
      for (var i = 0; i < classes.length; i++) {
        var klass = classes[i],
            animationFactory = $animateProvider.$$registeredAnimations[klass];
        if (animationFactory && !flagMap[klass]) {
          matches.push($injector.get(animationFactory));
          flagMap[klass] = true;
        }
      }
      return matches;
    }
  }];
}];

var $$AnimateJsDriverProvider = ['$$animationProvider', /** @this */ function($$animationProvider) {
  $$animationProvider.drivers.push('$$animateJsDriver');
  this.$get = ['$$animateJs', '$$AnimateRunner', function($$animateJs, $$AnimateRunner) {
    return function initDriverFn(animationDetails) {
      if (animationDetails.from && animationDetails.to) {
        var fromAnimation = prepareAnimation(animationDetails.from);
        var toAnimation = prepareAnimation(animationDetails.to);
        if (!fromAnimation && !toAnimation) return;

        return {
          start: function() {
            var animationRunners = [];

            if (fromAnimation) {
              animationRunners.push(fromAnimation.start());
            }

            if (toAnimation) {
              animationRunners.push(toAnimation.start());
            }

            $$AnimateRunner.all(animationRunners, done);

            var runner = new $$AnimateRunner({
              end: endFnFactory(),
              cancel: endFnFactory()
            });

            return runner;

            function endFnFactory() {
              return function() {
                forEach(animationRunners, function(runner) {
                  // at this point we cannot cancel animations for groups just yet. 1.5+
                  runner.end();
                });
              };
            }

            function done(status) {
              runner.complete(status);
            }
          }
        };
      } else {
        return prepareAnimation(animationDetails);
      }
    };

    function prepareAnimation(animationDetails) {
      // TODO(matsko): make sure to check for grouped animations and delegate down to normal animations
      var element = animationDetails.element;
      var event = animationDetails.event;
      var options = animationDetails.options;
      var classes = animationDetails.classes;
      return $$animateJs(element, event, classes, options);
    }
  }];
}];

var NG_ANIMATE_ATTR_NAME = 'data-ng-animate';
var NG_ANIMATE_PIN_DATA = '$ngAnimatePin';
var $$AnimateQueueProvider = ['$animateProvider', /** @this */ function($animateProvider) {
  var PRE_DIGEST_STATE = 1;
  var RUNNING_STATE = 2;
  var ONE_SPACE = ' ';

  var rules = this.rules = {
    skip: [],
    cancel: [],
    join: []
  };

  function getEventData(options) {
    return {
      addClass: options.addClass,
      removeClass: options.removeClass,
      from: options.from,
      to: options.to
    };
  }

  function makeTruthyCssClassMap(classString) {
    if (!classString) {
      return null;
    }

    var keys = classString.split(ONE_SPACE);
    var map = Object.create(null);

    forEach(keys, function(key) {
      map[key] = true;
    });
    return map;
  }

  function hasMatchingClasses(newClassString, currentClassString) {
    if (newClassString && currentClassString) {
      var currentClassMap = makeTruthyCssClassMap(currentClassString);
      return newClassString.split(ONE_SPACE).some(function(className) {
        return currentClassMap[className];
      });
    }
  }

  function isAllowed(ruleType, currentAnimation, previousAnimation) {
    return rules[ruleType].some(function(fn) {
      return fn(currentAnimation, previousAnimation);
    });
  }

  function hasAnimationClasses(animation, and) {
    var a = (animation.addClass || '').length > 0;
    var b = (animation.removeClass || '').length > 0;
    return and ? a && b : a || b;
  }

  rules.join.push(function(newAnimation, currentAnimation) {
    // if the new animation is class-based then we can just tack that on
    return !newAnimation.structural && hasAnimationClasses(newAnimation);
  });

  rules.skip.push(function(newAnimation, currentAnimation) {
    // there is no need to animate anything if no classes are being added and
    // there is no structural animation that will be triggered
    return !newAnimation.structural && !hasAnimationClasses(newAnimation);
  });

  rules.skip.push(function(newAnimation, currentAnimation) {
    // why should we trigger a new structural animation if the element will
    // be removed from the DOM anyway?
    return currentAnimation.event === 'leave' && newAnimation.structural;
  });

  rules.skip.push(function(newAnimation, currentAnimation) {
    // if there is an ongoing current animation then don't even bother running the class-based animation
    return currentAnimation.structural && currentAnimation.state === RUNNING_STATE && !newAnimation.structural;
  });

  rules.cancel.push(function(newAnimation, currentAnimation) {
    // there can never be two structural animations running at the same time
    return currentAnimation.structural && newAnimation.structural;
  });

  rules.cancel.push(function(newAnimation, currentAnimation) {
    // if the previous animation is already running, but the new animation will
    // be triggered, but the new animation is structural
    return currentAnimation.state === RUNNING_STATE && newAnimation.structural;
  });

  rules.cancel.push(function(newAnimation, currentAnimation) {
    // cancel the animation if classes added / removed in both animation cancel each other out,
    // but only if the current animation isn't structural

    if (currentAnimation.structural) return false;

    var nA = newAnimation.addClass;
    var nR = newAnimation.removeClass;
    var cA = currentAnimation.addClass;
    var cR = currentAnimation.removeClass;

    // early detection to save the global CPU shortage :)
    if ((isUndefined(nA) && isUndefined(nR)) || (isUndefined(cA) && isUndefined(cR))) {
      return false;
    }

    return hasMatchingClasses(nA, cR) || hasMatchingClasses(nR, cA);
  });

  this.$get = ['$$rAF', '$rootScope', '$rootElement', '$document', '$$Map',
               '$$animation', '$$AnimateRunner', '$templateRequest', '$$jqLite', '$$forceReflow',
               '$$isDocumentHidden',
       function($$rAF,   $rootScope,   $rootElement,   $document,   $$Map,
                $$animation,   $$AnimateRunner,   $templateRequest,   $$jqLite,   $$forceReflow,
                $$isDocumentHidden) {

    var activeAnimationsLookup = new $$Map();
    var disabledElementsLookup = new $$Map();
    var animationsEnabled = null;

    function removeFromDisabledElementsLookup(evt) {
      disabledElementsLookup.delete(evt.target);
    }

    function postDigestTaskFactory() {
      var postDigestCalled = false;
      return function(fn) {
        // we only issue a call to postDigest before
        // it has first passed. This prevents any callbacks
        // from not firing once the animation has completed
        // since it will be out of the digest cycle.
        if (postDigestCalled) {
          fn();
        } else {
          $rootScope.$$postDigest(function() {
            postDigestCalled = true;
            fn();
          });
        }
      };
    }

    // Wait until all directive and route-related templates are downloaded and
    // compiled. The $templateRequest.totalPendingRequests variable keeps track of
    // all of the remote templates being currently downloaded. If there are no
    // templates currently downloading then the watcher will still fire anyway.
    var deregisterWatch = $rootScope.$watch(
      function() { return $templateRequest.totalPendingRequests === 0; },
      function(isEmpty) {
        if (!isEmpty) return;
        deregisterWatch();

        // Now that all templates have been downloaded, $animate will wait until
        // the post digest queue is empty before enabling animations. By having two
        // calls to $postDigest calls we can ensure that the flag is enabled at the
        // very end of the post digest queue. Since all of the animations in $animate
        // use $postDigest, it's important that the code below executes at the end.
        // This basically means that the page is fully downloaded and compiled before
        // any animations are triggered.
        $rootScope.$$postDigest(function() {
          $rootScope.$$postDigest(function() {
            // we check for null directly in the event that the application already called
            // .enabled() with whatever arguments that it provided it with
            if (animationsEnabled === null) {
              animationsEnabled = true;
            }
          });
        });
      }
    );

    var callbackRegistry = Object.create(null);

    // remember that the `customFilter`/`classNameFilter` are set during the
    // provider/config stage therefore we can optimize here and setup helper functions
    var customFilter = $animateProvider.customFilter();
    var classNameFilter = $animateProvider.classNameFilter();
    var returnTrue = function() { return true; };

    var isAnimatableByFilter = customFilter || returnTrue;
    var isAnimatableClassName = !classNameFilter ? returnTrue : function(node, options) {
      var className = [node.getAttribute('class'), options.addClass, options.removeClass].join(' ');
      return classNameFilter.test(className);
    };

    var applyAnimationClasses = applyAnimationClassesFactory($$jqLite);

    function normalizeAnimationDetails(element, animation) {
      return mergeAnimationDetails(element, animation, {});
    }

    // IE9-11 has no method "contains" in SVG element and in Node.prototype. Bug #10259.
    var contains = window.Node.prototype.contains || /** @this */ function(arg) {
      // eslint-disable-next-line no-bitwise
      return this === arg || !!(this.compareDocumentPosition(arg) & 16);
    };

    function findCallbacks(targetParentNode, targetNode, event) {
      var matches = [];
      var entries = callbackRegistry[event];
      if (entries) {
        forEach(entries, function(entry) {
          if (contains.call(entry.node, targetNode)) {
            matches.push(entry.callback);
          } else if (event === 'leave' && contains.call(entry.node, targetParentNode)) {
            matches.push(entry.callback);
          }
        });
      }

      return matches;
    }

    function filterFromRegistry(list, matchContainer, matchCallback) {
      var containerNode = extractElementNode(matchContainer);
      return list.filter(function(entry) {
        var isMatch = entry.node === containerNode &&
                        (!matchCallback || entry.callback === matchCallback);
        return !isMatch;
      });
    }

    function cleanupEventListeners(phase, node) {
      if (phase === 'close' && !node.parentNode) {
        // If the element is not attached to a parentNode, it has been removed by
        // the domOperation, and we can safely remove the event callbacks
        $animate.off(node);
      }
    }

    var $animate = {
      on: function(event, container, callback) {
        var node = extractElementNode(container);
        callbackRegistry[event] = callbackRegistry[event] || [];
        callbackRegistry[event].push({
          node: node,
          callback: callback
        });

        // Remove the callback when the element is removed from the DOM
        jqLite(container).on('$destroy', function() {
          var animationDetails = activeAnimationsLookup.get(node);

          if (!animationDetails) {
            // If there's an animation ongoing, the callback calling code will remove
            // the event listeners. If we'd remove here, the callbacks would be removed
            // before the animation ends
            $animate.off(event, container, callback);
          }
        });
      },

      off: function(event, container, callback) {
        if (arguments.length === 1 && !isString(arguments[0])) {
          container = arguments[0];
          for (var eventType in callbackRegistry) {
            callbackRegistry[eventType] = filterFromRegistry(callbackRegistry[eventType], container);
          }

          return;
        }

        var entries = callbackRegistry[event];
        if (!entries) return;

        callbackRegistry[event] = arguments.length === 1
            ? null
            : filterFromRegistry(entries, container, callback);
      },

      pin: function(element, parentElement) {
        assertArg(isElement(element), 'element', 'not an element');
        assertArg(isElement(parentElement), 'parentElement', 'not an element');
        element.data(NG_ANIMATE_PIN_DATA, parentElement);
      },

      push: function(element, event, options, domOperation) {
        options = options || {};
        options.domOperation = domOperation;
        return queueAnimation(element, event, options);
      },

      // this method has four signatures:
      //  () - global getter
      //  (bool) - global setter
      //  (element) - element getter
      //  (element, bool) - element setter<F37>
      enabled: function(element, bool) {
        var argCount = arguments.length;

        if (argCount === 0) {
          // () - Global getter
          bool = !!animationsEnabled;
        } else {
          var hasElement = isElement(element);

          if (!hasElement) {
            // (bool) - Global setter
            bool = animationsEnabled = !!element;
          } else {
            var node = getDomNode(element);

            if (argCount === 1) {
              // (element) - Element getter
              bool = !disabledElementsLookup.get(node);
            } else {
              // (element, bool) - Element setter
              if (!disabledElementsLookup.has(node)) {
                // The element is added to the map for the first time.
                // Create a listener to remove it on `$destroy` (to avoid memory leak).
                jqLite(element).on('$destroy', removeFromDisabledElementsLookup);
              }
              disabledElementsLookup.set(node, !bool);
            }
          }
        }

        return bool;
      }
    };

    return $animate;

    function queueAnimation(originalElement, event, initialOptions) {
      // we always make a copy of the options since
      // there should never be any side effects on
      // the input data when running `$animateCss`.
      var options = copy(initialOptions);

      var element = stripCommentsFromElement(originalElement);
      var node = getDomNode(element);
      var parentNode = node && node.parentNode;

      options = prepareAnimationOptions(options);

      // we create a fake runner with a working promise.
      // These methods will become available after the digest has passed
      var runner = new $$AnimateRunner();

      // this is used to trigger callbacks in postDigest mode
      var runInNextPostDigestOrNow = postDigestTaskFactory();

      if (isArray(options.addClass)) {
        options.addClass = options.addClass.join(' ');
      }

      if (options.addClass && !isString(options.addClass)) {
        options.addClass = null;
      }

      if (isArray(options.removeClass)) {
        options.removeClass = options.removeClass.join(' ');
      }

      if (options.removeClass && !isString(options.removeClass)) {
        options.removeClass = null;
      }

      if (options.from && !isObject(options.from)) {
        options.from = null;
      }

      if (options.to && !isObject(options.to)) {
        options.to = null;
      }

      // If animations are hard-disabled for the whole application there is no need to continue.
      // There are also situations where a directive issues an animation for a jqLite wrapper that
      // contains only comment nodes. In this case, there is no way we can perform an animation.
      if (!animationsEnabled ||
          !node ||
          !isAnimatableByFilter(node, event, initialOptions) ||
          !isAnimatableClassName(node, options)) {
        close();
        return runner;
      }

      var isStructural = ['enter', 'move', 'leave'].indexOf(event) >= 0;

      var documentHidden = $$isDocumentHidden();

      // This is a hard disable of all animations the element itself, therefore  there is no need to
      // continue further past this point if not enabled
      // Animations are also disabled if the document is currently hidden (page is not visible
      // to the user), because browsers slow down or do not flush calls to requestAnimationFrame
      var skipAnimations = documentHidden || disabledElementsLookup.get(node);
      var existingAnimation = (!skipAnimations && activeAnimationsLookup.get(node)) || {};
      var hasExistingAnimation = !!existingAnimation.state;

      // there is no point in traversing the same collection of parent ancestors if a followup
      // animation will be run on the same element that already did all that checking work
      if (!skipAnimations && (!hasExistingAnimation || existingAnimation.state !== PRE_DIGEST_STATE)) {
        skipAnimations = !areAnimationsAllowed(node, parentNode, event);
      }

      if (skipAnimations) {
        // Callbacks should fire even if the document is hidden (regression fix for issue #14120)
        if (documentHidden) notifyProgress(runner, event, 'start', getEventData(options));
        close();
        if (documentHidden) notifyProgress(runner, event, 'close', getEventData(options));
        return runner;
      }

      if (isStructural) {
        closeChildAnimations(node);
      }

      var newAnimation = {
        structural: isStructural,
        element: element,
        event: event,
        addClass: options.addClass,
        removeClass: options.removeClass,
        close: close,
        options: options,
        runner: runner
      };

      if (hasExistingAnimation) {
        var skipAnimationFlag = isAllowed('skip', newAnimation, existingAnimation);
        if (skipAnimationFlag) {
          if (existingAnimation.state === RUNNING_STATE) {
            close();
            return runner;
          } else {
            mergeAnimationDetails(element, existingAnimation, newAnimation);
            return existingAnimation.runner;
          }
        }
        var cancelAnimationFlag = isAllowed('cancel', newAnimation, existingAnimation);
        if (cancelAnimationFlag) {
          if (existingAnimation.state === RUNNING_STATE) {
            // this will end the animation right away and it is safe
            // to do so since the animation is already running and the
            // runner callback code will run in async
            existingAnimation.runner.end();
          } else if (existingAnimation.structural) {
            // this means that the animation is queued into a digest, but
            // hasn't started yet. Therefore it is safe to run the close
            // method which will call the runner methods in async.
            existingAnimation.close();
          } else {
            // this will merge the new animation options into existing animation options
            mergeAnimationDetails(element, existingAnimation, newAnimation);

            return existingAnimation.runner;
          }
        } else {
          // a joined animation means that this animation will take over the existing one
          // so an example would involve a leave animation taking over an enter. Then when
          // the postDigest kicks in the enter will be ignored.
          var joinAnimationFlag = isAllowed('join', newAnimation, existingAnimation);
          if (joinAnimationFlag) {
            if (existingAnimation.state === RUNNING_STATE) {
              normalizeAnimationDetails(element, newAnimation);
            } else {
              applyGeneratedPreparationClasses($$jqLite, element, isStructural ? event : null, options);

              event = newAnimation.event = existingAnimation.event;
              options = mergeAnimationDetails(element, existingAnimation, newAnimation);

              //we return the same runner since only the option values of this animation will
              //be fed into the `existingAnimation`.
              return existingAnimation.runner;
            }
          }
        }
      } else {
        // normalization in this case means that it removes redundant CSS classes that
        // already exist (addClass) or do not exist (removeClass) on the element
        normalizeAnimationDetails(element, newAnimation);
      }

      // when the options are merged and cleaned up we may end up not having to do
      // an animation at all, therefore we should check this before issuing a post
      // digest callback. Structural animations will always run no matter what.
      var isValidAnimation = newAnimation.structural;
      if (!isValidAnimation) {
        // animate (from/to) can be quickly checked first, otherwise we check if any classes are present
        isValidAnimation = (newAnimation.event === 'animate' && Object.keys(newAnimation.options.to || {}).length > 0)
                            || hasAnimationClasses(newAnimation);
      }

      if (!isValidAnimation) {
        close();
        clearElementAnimationState(node);
        return runner;
      }

      // the counter keeps track of cancelled animations
      var counter = (existingAnimation.counter || 0) + 1;
      newAnimation.counter = counter;

      markElementAnimationState(node, PRE_DIGEST_STATE, newAnimation);

      $rootScope.$$postDigest(function() {
        // It is possible that the DOM nodes inside `originalElement` have been replaced. This can
        // happen if the animated element is a transcluded clone and also has a `templateUrl`
        // directive on it. Therefore, we must recreate `element` in order to interact with the
        // actual DOM nodes.
        // Note: We still need to use the old `node` for certain things, such as looking up in
        //       HashMaps where it was used as the key.

        element = stripCommentsFromElement(originalElement);

        var animationDetails = activeAnimationsLookup.get(node);
        var animationCancelled = !animationDetails;
        animationDetails = animationDetails || {};

        // if addClass/removeClass is called before something like enter then the
        // registered parent element may not be present. The code below will ensure
        // that a final value for parent element is obtained
        var parentElement = element.parent() || [];

        // animate/structural/class-based animations all have requirements. Otherwise there
        // is no point in performing an animation. The parent node must also be set.
        var isValidAnimation = parentElement.length > 0
                                && (animationDetails.event === 'animate'
                                    || animationDetails.structural
                                    || hasAnimationClasses(animationDetails));

        // this means that the previous animation was cancelled
        // even if the follow-up animation is the same event
        if (animationCancelled || animationDetails.counter !== counter || !isValidAnimation) {
          // if another animation did not take over then we need
          // to make sure that the domOperation and options are
          // handled accordingly
          if (animationCancelled) {
            applyAnimationClasses(element, options);
            applyAnimationStyles(element, options);
          }

          // if the event changed from something like enter to leave then we do
          // it, otherwise if it's the same then the end result will be the same too
          if (animationCancelled || (isStructural && animationDetails.event !== event)) {
            options.domOperation();
            runner.end();
          }

          // in the event that the element animation was not cancelled or a follow-up animation
          // isn't allowed to animate from here then we need to clear the state of the element
          // so that any future animations won't read the expired animation data.
          if (!isValidAnimation) {
            clearElementAnimationState(node);
          }

          return;
        }

        // this combined multiple class to addClass / removeClass into a setClass event
        // so long as a structural event did not take over the animation
        event = !animationDetails.structural && hasAnimationClasses(animationDetails, true)
            ? 'setClass'
            : animationDetails.event;

        markElementAnimationState(node, RUNNING_STATE);
        var realRunner = $$animation(element, event, animationDetails.options);

        // this will update the runner's flow-control events based on
        // the `realRunner` object.
        runner.setHost(realRunner);
        notifyProgress(runner, event, 'start', getEventData(options));

        realRunner.done(function(status) {
          close(!status);
          var animationDetails = activeAnimationsLookup.get(node);
          if (animationDetails && animationDetails.counter === counter) {
            clearElementAnimationState(node);
          }
          notifyProgress(runner, event, 'close', getEventData(options));
        });
      });

      return runner;

      function notifyProgress(runner, event, phase, data) {
        runInNextPostDigestOrNow(function() {
          var callbacks = findCallbacks(parentNode, node, event);
          if (callbacks.length) {
            // do not optimize this call here to RAF because
            // we don't know how heavy the callback code here will
            // be and if this code is buffered then this can
            // lead to a performance regression.
            $$rAF(function() {
              forEach(callbacks, function(callback) {
                callback(element, phase, data);
              });
              cleanupEventListeners(phase, node);
            });
          } else {
            cleanupEventListeners(phase, node);
          }
        });
        runner.progress(event, phase, data);
      }

      function close(reject) {
        clearGeneratedClasses(element, options);
        applyAnimationClasses(element, options);
        applyAnimationStyles(element, options);
        options.domOperation();
        runner.complete(!reject);
      }
    }

    function closeChildAnimations(node) {
      var children = node.querySelectorAll('[' + NG_ANIMATE_ATTR_NAME + ']');
      forEach(children, function(child) {
        var state = parseInt(child.getAttribute(NG_ANIMATE_ATTR_NAME), 10);
        var animationDetails = activeAnimationsLookup.get(child);
        if (animationDetails) {
          switch (state) {
            case RUNNING_STATE:
              animationDetails.runner.end();
              /* falls through */
            case PRE_DIGEST_STATE:
              activeAnimationsLookup.delete(child);
              break;
          }
        }
      });
    }

    function clearElementAnimationState(node) {
      node.removeAttribute(NG_ANIMATE_ATTR_NAME);
      activeAnimationsLookup.delete(node);
    }

    /**
     * This fn returns false if any of the following is true:
     * a) animations on any parent element are disabled, and animations on the element aren't explicitly allowed
     * b) a parent element has an ongoing structural animation, and animateChildren is false
     * c) the element is not a child of the body
     * d) the element is not a child of the $rootElement
     */
    function areAnimationsAllowed(node, parentNode, event) {
      var bodyNode = $document[0].body;
      var rootNode = getDomNode($rootElement);

      var bodyNodeDetected = (node === bodyNode) || node.nodeName === 'HTML';
      var rootNodeDetected = (node === rootNode);
      var parentAnimationDetected = false;
      var elementDisabled = disabledElementsLookup.get(node);
      var animateChildren;

      var parentHost = jqLite.data(node, NG_ANIMATE_PIN_DATA);
      if (parentHost) {
        parentNode = getDomNode(parentHost);
      }

      while (parentNode) {
        if (!rootNodeDetected) {
          // AngularJS doesn't want to attempt to animate elements outside of the application
          // therefore we need to ensure that the rootElement is an ancestor of the current element
          rootNodeDetected = (parentNode === rootNode);
        }

        if (parentNode.nodeType !== ELEMENT_NODE) {
          // no point in inspecting the #document element
          break;
        }

        var details = activeAnimationsLookup.get(parentNode) || {};
        // either an enter, leave or move animation will commence
        // therefore we can't allow any animations to take place
        // but if a parent animation is class-based then that's ok
        if (!parentAnimationDetected) {
          var parentNodeDisabled = disabledElementsLookup.get(parentNode);

          if (parentNodeDisabled === true && elementDisabled !== false) {
            // disable animations if the user hasn't explicitly enabled animations on the
            // current element
            elementDisabled = true;
            // element is disabled via parent element, no need to check anything else
            break;
          } else if (parentNodeDisabled === false) {
            elementDisabled = false;
          }
          parentAnimationDetected = details.structural;
        }

        if (isUndefined(animateChildren) || animateChildren === true) {
          var value = jqLite.data(parentNode, NG_ANIMATE_CHILDREN_DATA);
          if (isDefined(value)) {
            animateChildren = value;
          }
        }

        // there is no need to continue traversing at this point
        if (parentAnimationDetected && animateChildren === false) break;

        if (!bodyNodeDetected) {
          // we also need to ensure that the element is or will be a part of the body element
          // otherwise it is pointless to even issue an animation to be rendered
          bodyNodeDetected = (parentNode === bodyNode);
        }

        if (bodyNodeDetected && rootNodeDetected) {
          // If both body and root have been found, any other checks are pointless,
          // as no animation data should live outside the application
          break;
        }

        if (!rootNodeDetected) {
          // If `rootNode` is not detected, check if `parentNode` is pinned to another element
          parentHost = jqLite.data(parentNode, NG_ANIMATE_PIN_DATA);
          if (parentHost) {
            // The pin target element becomes the next parent element
            parentNode = getDomNode(parentHost);
            continue;
          }
        }

        parentNode = parentNode.parentNode;
      }

      var allowAnimation = (!parentAnimationDetected || animateChildren) && elementDisabled !== true;
      return allowAnimation && rootNodeDetected && bodyNodeDetected;
    }

    function markElementAnimationState(node, state, details) {
      details = details || {};
      details.state = state;

      node.setAttribute(NG_ANIMATE_ATTR_NAME, state);

      var oldValue = activeAnimationsLookup.get(node);
      var newValue = oldValue
          ? extend(oldValue, details)
          : details;
      activeAnimationsLookup.set(node, newValue);
    }
  }];
}];

/** @this */
var $$AnimateCacheProvider = function() {

  var KEY = '$$ngAnimateParentKey';
  var parentCounter = 0;
  var cache = Object.create(null);

  this.$get = [function() {
    return {
      cacheKey: function(node, method, addClass, removeClass) {
        var parentNode = node.parentNode;
        var parentID = parentNode[KEY] || (parentNode[KEY] = ++parentCounter);
        var parts = [parentID, method, node.getAttribute('class')];
        if (addClass) {
          parts.push(addClass);
        }
        if (removeClass) {
          parts.push(removeClass);
        }
        return parts.join(' ');
      },

      containsCachedAnimationWithoutDuration: function(key) {
        var entry = cache[key];

        // nothing cached, so go ahead and animate
        // otherwise it should be a valid animation
        return (entry && !entry.isValid) || false;
      },

      flush: function() {
        cache = Object.create(null);
      },

      count: function(key) {
        var entry = cache[key];
        return entry ? entry.total : 0;
      },

      get: function(key) {
        var entry = cache[key];
        return entry && entry.value;
      },

      put: function(key, value, isValid) {
        if (!cache[key]) {
          cache[key] = { total: 1, value: value, isValid: isValid };
        } else {
          cache[key].total++;
          cache[key].value = value;
        }
      }
    };
  }];
};

/* exported $$AnimationProvider */

var $$AnimationProvider = ['$animateProvider', /** @this */ function($animateProvider) {
  var NG_ANIMATE_REF_ATTR = 'ng-animate-ref';

  var drivers = this.drivers = [];

  var RUNNER_STORAGE_KEY = '$$animationRunner';
  var PREPARE_CLASSES_KEY = '$$animatePrepareClasses';

  function setRunner(element, runner) {
    element.data(RUNNER_STORAGE_KEY, runner);
  }

  function removeRunner(element) {
    element.removeData(RUNNER_STORAGE_KEY);
  }

  function getRunner(element) {
    return element.data(RUNNER_STORAGE_KEY);
  }

  this.$get = ['$$jqLite', '$rootScope', '$injector', '$$AnimateRunner', '$$Map', '$$rAFScheduler', '$$animateCache',
       function($$jqLite,   $rootScope,   $injector,   $$AnimateRunner,   $$Map,   $$rAFScheduler, $$animateCache) {

    var animationQueue = [];
    var applyAnimationClasses = applyAnimationClassesFactory($$jqLite);

    function sortAnimations(animations) {
      var tree = { children: [] };
      var i, lookup = new $$Map();

      // this is done first beforehand so that the map
      // is filled with a list of the elements that will be animated
      for (i = 0; i < animations.length; i++) {
        var animation = animations[i];
        lookup.set(animation.domNode, animations[i] = {
          domNode: animation.domNode,
          element: animation.element,
          fn: animation.fn,
          children: []
        });
      }

      for (i = 0; i < animations.length; i++) {
        processNode(animations[i]);
      }

      return flatten(tree);

      function processNode(entry) {
        if (entry.processed) return entry;
        entry.processed = true;

        var elementNode = entry.domNode;
        var parentNode = elementNode.parentNode;
        lookup.set(elementNode, entry);

        var parentEntry;
        while (parentNode) {
          parentEntry = lookup.get(parentNode);
          if (parentEntry) {
            if (!parentEntry.processed) {
              parentEntry = processNode(parentEntry);
            }
            break;
          }
          parentNode = parentNode.parentNode;
        }

        (parentEntry || tree).children.push(entry);
        return entry;
      }

      function flatten(tree) {
        var result = [];
        var queue = [];
        var i;

        for (i = 0; i < tree.children.length; i++) {
          queue.push(tree.children[i]);
        }

        var remainingLevelEntries = queue.length;
        var nextLevelEntries = 0;
        var row = [];

        for (i = 0; i < queue.length; i++) {
          var entry = queue[i];
          if (remainingLevelEntries <= 0) {
            remainingLevelEntries = nextLevelEntries;
            nextLevelEntries = 0;
            result.push(row);
            row = [];
          }
          row.push(entry);
          entry.children.forEach(function(childEntry) {
            nextLevelEntries++;
            queue.push(childEntry);
          });
          remainingLevelEntries--;
        }

        if (row.length) {
          result.push(row);
        }

        return result;
      }
    }

    // TODO(matsko): document the signature in a better way
    return function(element, event, options) {
      options = prepareAnimationOptions(options);
      var isStructural = ['enter', 'move', 'leave'].indexOf(event) >= 0;

      // there is no animation at the current moment, however
      // these runner methods will get later updated with the
      // methods leading into the driver's end/cancel methods
      // for now they just stop the animation from starting
      var runner = new $$AnimateRunner({
        end: function() { close(); },
        cancel: function() { close(true); }
      });

      if (!drivers.length) {
        close();
        return runner;
      }

      var classes = mergeClasses(element.attr('class'), mergeClasses(options.addClass, options.removeClass));
      var tempClasses = options.tempClasses;
      if (tempClasses) {
        classes += ' ' + tempClasses;
        options.tempClasses = null;
      }

      if (isStructural) {
        element.data(PREPARE_CLASSES_KEY, 'ng-' + event + PREPARE_CLASS_SUFFIX);
      }

      setRunner(element, runner);

      animationQueue.push({
        // this data is used by the postDigest code and passed into
        // the driver step function
        element: element,
        classes: classes,
        event: event,
        structural: isStructural,
        options: options,
        beforeStart: beforeStart,
        close: close
      });

      element.on('$destroy', handleDestroyedElement);

      // we only want there to be one function called within the post digest
      // block. This way we can group animations for all the animations that
      // were apart of the same postDigest flush call.
      if (animationQueue.length > 1) return runner;

      $rootScope.$$postDigest(function() {
        var animations = [];
        forEach(animationQueue, function(entry) {
          // the element was destroyed early on which removed the runner
          // form its storage. This means we can't animate this element
          // at all and it already has been closed due to destruction.
          if (getRunner(entry.element)) {
            animations.push(entry);
          } else {
            entry.close();
          }
        });

        // now any future animations will be in another postDigest
        animationQueue.length = 0;

        var groupedAnimations = groupAnimations(animations);
        var toBeSortedAnimations = [];

        forEach(groupedAnimations, function(animationEntry) {
          var element = animationEntry.from ? animationEntry.from.element : animationEntry.element;
          var extraClasses = options.addClass;

          extraClasses = (extraClasses ? (extraClasses + ' ') : '') + NG_ANIMATE_CLASSNAME;
          var cacheKey = $$animateCache.cacheKey(element[0], animationEntry.event, extraClasses, options.removeClass);

          toBeSortedAnimations.push({
            element: element,
            domNode: getDomNode(element),
            fn: function triggerAnimationStart() {
              var startAnimationFn, closeFn = animationEntry.close;

              // in the event that we've cached the animation status for this element
              // and it's in fact an invalid animation (something that has duration = 0)
              // then we should skip all the heavy work from here on
              if ($$animateCache.containsCachedAnimationWithoutDuration(cacheKey)) {
                closeFn();
                return;
              }

              // it's important that we apply the `ng-animate` CSS class and the
              // temporary classes before we do any driver invoking since these
              // CSS classes may be required for proper CSS detection.
              animationEntry.beforeStart();

              // in the event that the element was removed before the digest runs or
              // during the RAF sequencing then we should not trigger the animation.
              var targetElement = animationEntry.anchors
                  ? (animationEntry.from.element || animationEntry.to.element)
                  : animationEntry.element;

              if (getRunner(targetElement)) {
                var operation = invokeFirstDriver(animationEntry);
                if (operation) {
                  startAnimationFn = operation.start;
                }
              }

              if (!startAnimationFn) {
                closeFn();
              } else {
                var animationRunner = startAnimationFn();
                animationRunner.done(function(status) {
                  closeFn(!status);
                });
                updateAnimationRunners(animationEntry, animationRunner);
              }
            }
          });
        });

        // we need to sort each of the animations in order of parent to child
        // relationships. This ensures that the child classes are applied at the
        // right time.
        var finalAnimations = sortAnimations(toBeSortedAnimations);
        for (var i = 0; i < finalAnimations.length; i++) {
          var innerArray = finalAnimations[i];
          for (var j = 0; j < innerArray.length; j++) {
            var entry = innerArray[j];
            var element = entry.element;

            // the RAFScheduler code only uses functions
            finalAnimations[i][j] = entry.fn;

            // the first row of elements shouldn't have a prepare-class added to them
            // since the elements are at the top of the animation hierarchy and they
            // will be applied without a RAF having to pass...
            if (i === 0) {
              element.removeData(PREPARE_CLASSES_KEY);
              continue;
            }

            var prepareClassName = element.data(PREPARE_CLASSES_KEY);
            if (prepareClassName) {
              $$jqLite.addClass(element, prepareClassName);
            }
          }
        }

        $$rAFScheduler(finalAnimations);
      });

      return runner;

      // TODO(matsko): change to reference nodes
      function getAnchorNodes(node) {
        var SELECTOR = '[' + NG_ANIMATE_REF_ATTR + ']';
        var items = node.hasAttribute(NG_ANIMATE_REF_ATTR)
              ? [node]
              : node.querySelectorAll(SELECTOR);
        var anchors = [];
        forEach(items, function(node) {
          var attr = node.getAttribute(NG_ANIMATE_REF_ATTR);
          if (attr && attr.length) {
            anchors.push(node);
          }
        });
        return anchors;
      }

      function groupAnimations(animations) {
        var preparedAnimations = [];
        var refLookup = {};
        forEach(animations, function(animation, index) {
          var element = animation.element;
          var node = getDomNode(element);
          var event = animation.event;
          var enterOrMove = ['enter', 'move'].indexOf(event) >= 0;
          var anchorNodes = animation.structural ? getAnchorNodes(node) : [];

          if (anchorNodes.length) {
            var direction = enterOrMove ? 'to' : 'from';

            forEach(anchorNodes, function(anchor) {
              var key = anchor.getAttribute(NG_ANIMATE_REF_ATTR);
              refLookup[key] = refLookup[key] || {};
              refLookup[key][direction] = {
                animationID: index,
                element: jqLite(anchor)
              };
            });
          } else {
            preparedAnimations.push(animation);
          }
        });

        var usedIndicesLookup = {};
        var anchorGroups = {};
        forEach(refLookup, function(operations, key) {
          var from = operations.from;
          var to = operations.to;

          if (!from || !to) {
            // only one of these is set therefore we can't have an
            // anchor animation since all three pieces are required
            var index = from ? from.animationID : to.animationID;
            var indexKey = index.toString();
            if (!usedIndicesLookup[indexKey]) {
              usedIndicesLookup[indexKey] = true;
              preparedAnimations.push(animations[index]);
            }
            return;
          }

          var fromAnimation = animations[from.animationID];
          var toAnimation = animations[to.animationID];
          var lookupKey = from.animationID.toString();
          if (!anchorGroups[lookupKey]) {
            var group = anchorGroups[lookupKey] = {
              structural: true,
              beforeStart: function() {
                fromAnimation.beforeStart();
                toAnimation.beforeStart();
              },
              close: function() {
                fromAnimation.close();
                toAnimation.close();
              },
              classes: cssClassesIntersection(fromAnimation.classes, toAnimation.classes),
              from: fromAnimation,
              to: toAnimation,
              anchors: [] // TODO(matsko): change to reference nodes
            };

            // the anchor animations require that the from and to elements both have at least
            // one shared CSS class which effectively marries the two elements together to use
            // the same animation driver and to properly sequence the anchor animation.
            if (group.classes.length) {
              preparedAnimations.push(group);
            } else {
              preparedAnimations.push(fromAnimation);
              preparedAnimations.push(toAnimation);
            }
          }

          anchorGroups[lookupKey].anchors.push({
            'out': from.element, 'in': to.element
          });
        });

        return preparedAnimations;
      }

      function cssClassesIntersection(a,b) {
        a = a.split(' ');
        b = b.split(' ');
        var matches = [];

        for (var i = 0; i < a.length; i++) {
          var aa = a[i];
          if (aa.substring(0,3) === 'ng-') continue;

          for (var j = 0; j < b.length; j++) {
            if (aa === b[j]) {
              matches.push(aa);
              break;
            }
          }
        }

        return matches.join(' ');
      }

      function invokeFirstDriver(animationDetails) {
        // we loop in reverse order since the more general drivers (like CSS and JS)
        // may attempt more elements, but custom drivers are more particular
        for (var i = drivers.length - 1; i >= 0; i--) {
          var driverName = drivers[i];
          var factory = $injector.get(driverName);
          var driver = factory(animationDetails);
          if (driver) {
            return driver;
          }
        }
      }

      function beforeStart() {
        tempClasses = (tempClasses ? (tempClasses + ' ') : '') + NG_ANIMATE_CLASSNAME;
        $$jqLite.addClass(element, tempClasses);

        var prepareClassName = element.data(PREPARE_CLASSES_KEY);
        if (prepareClassName) {
          $$jqLite.removeClass(element, prepareClassName);
          prepareClassName = null;
        }
      }

      function updateAnimationRunners(animation, newRunner) {
        if (animation.from && animation.to) {
          update(animation.from.element);
          update(animation.to.element);
        } else {
          update(animation.element);
        }

        function update(element) {
          var runner = getRunner(element);
          if (runner) runner.setHost(newRunner);
        }
      }

      function handleDestroyedElement() {
        var runner = getRunner(element);
        if (runner && (event !== 'leave' || !options.$$domOperationFired)) {
          runner.end();
        }
      }

      function close(rejected) {
        element.off('$destroy', handleDestroyedElement);
        removeRunner(element);

        applyAnimationClasses(element, options);
        applyAnimationStyles(element, options);
        options.domOperation();

        if (tempClasses) {
          $$jqLite.removeClass(element, tempClasses);
        }

        runner.complete(!rejected);
      }
    };
  }];
}];

/**
 * @ngdoc directive
 * @name ngAnimateSwap
 * @restrict A
 * @scope
 *
 * @description
 *
 * ngAnimateSwap is a animation-oriented directive that allows for the container to
 * be removed and entered in whenever the associated expression changes. A
 * common usecase for this directive is a rotating banner or slider component which
 * contains one image being present at a time. When the active image changes
 * then the old image will perform a `leave` animation and the new element
 * will be inserted via an `enter` animation.
 *
 * @animations
 * | Animation                        | Occurs                               |
 * |----------------------------------|--------------------------------------|
 * | {@link ng.$animate#enter enter}  | when the new element is inserted to the DOM  |
 * | {@link ng.$animate#leave leave}  | when the old element is removed from the DOM |
 *
 * @example
 * <example name="ngAnimateSwap-directive" module="ngAnimateSwapExample"
 *          deps="angular-animate.js"
 *          animations="true" fixBase="true">
 *   <file name="index.html">
 *     <div class="container" ng-controller="AppCtrl">
 *       <div ng-animate-swap="number" class="cell swap-animation" ng-class="colorClass(number)">
 *         {{ number }}
 *       </div>
 *     </div>
 *   </file>
 *   <file name="script.js">
 *     angular.module('ngAnimateSwapExample', ['ngAnimate'])
 *       .controller('AppCtrl', ['$scope', '$interval', function($scope, $interval) {
 *         $scope.number = 0;
 *         $interval(function() {
 *           $scope.number++;
 *         }, 1000);
 *
 *         var colors = ['red','blue','green','yellow','orange'];
 *         $scope.colorClass = function(number) {
 *           return colors[number % colors.length];
 *         };
 *       }]);
 *   </file>
 *  <file name="animations.css">
 *  .container {
 *    height:250px;
 *    width:250px;
 *    position:relative;
 *    overflow:hidden;
 *    border:2px solid black;
 *  }
 *  .container .cell {
 *    font-size:150px;
 *    text-align:center;
 *    line-height:250px;
 *    position:absolute;
 *    top:0;
 *    left:0;
 *    right:0;
 *    border-bottom:2px solid black;
 *  }
 *  .swap-animation.ng-enter, .swap-animation.ng-leave {
 *    transition:0.5s linear all;
 *  }
 *  .swap-animation.ng-enter {
 *    top:-250px;
 *  }
 *  .swap-animation.ng-enter-active {
 *    top:0px;
 *  }
 *  .swap-animation.ng-leave {
 *    top:0px;
 *  }
 *  .swap-animation.ng-leave-active {
 *    top:250px;
 *  }
 *  .red { background:red; }
 *  .green { background:green; }
 *  .blue { background:blue; }
 *  .yellow { background:yellow; }
 *  .orange { background:orange; }
 *  </file>
 * </example>
 */
var ngAnimateSwapDirective = ['$animate', function($animate) {
  return {
    restrict: 'A',
    transclude: 'element',
    terminal: true,
    priority: 550, // We use 550 here to ensure that the directive is caught before others,
                   // but after `ngIf` (at priority 600).
    link: function(scope, $element, attrs, ctrl, $transclude) {
      var previousElement, previousScope;
      scope.$watchCollection(attrs.ngAnimateSwap || attrs['for'], function(value) {
        if (previousElement) {
          $animate.leave(previousElement);
        }
        if (previousScope) {
          previousScope.$destroy();
          previousScope = null;
        }
        if (value || value === 0) {
          $transclude(function(clone, childScope) {
            previousElement = clone;
            previousScope = childScope;
            $animate.enter(clone, null, $element);
          });
        }
      });
    }
  };
}];

/**
 * @ngdoc module
 * @name ngAnimate
 * @description
 *
 * The `ngAnimate` module provides support for CSS-based animations (keyframes and transitions) as well as JavaScript-based animations via
 * callback hooks. Animations are not enabled by default, however, by including `ngAnimate` the animation hooks are enabled for an AngularJS app.
 *
 * ## Usage
 * Simply put, there are two ways to make use of animations when ngAnimate is used: by using **CSS** and **JavaScript**. The former works purely based
 * using CSS (by using matching CSS selectors/styles) and the latter triggers animations that are registered via `module.animation()`. For
 * both CSS and JS animations the sole requirement is to have a matching `CSS class` that exists both in the registered animation and within
 * the HTML element that the animation will be triggered on.
 *
 * ## Directive Support
 * The following directives are "animation aware":
 *
 * | Directive                                                                     | Supported Animations                                                      |
 * |-------------------------------------------------------------------------------|---------------------------------------------------------------------------|
 * | {@link ng.directive:form#animations form / ngForm}                            | add and remove ({@link ng.directive:form#css-classes various classes})    |
 * | {@link ngAnimate.directive:ngAnimateSwap#animations ngAnimateSwap}            | enter and leave                                                           |
 * | {@link ng.directive:ngClass#animations ngClass / {{class&#125;&#8203;&#125;}  | add and remove                                                            |
 * | {@link ng.directive:ngClassEven#animations ngClassEven}                       | add and remove                                                            |
 * | {@link ng.directive:ngClassOdd#animations ngClassOdd}                         | add and remove                                                            |
 * | {@link ng.directive:ngHide#animations ngHide}                                 | add and remove (the `ng-hide` class)                                      |
 * | {@link ng.directive:ngIf#animations ngIf}                                     | enter and leave                                                           |
 * | {@link ng.directive:ngInclude#animations ngInclude}                           | enter and leave                                                           |
 * | {@link module:ngMessages#animations ngMessage / ngMessageExp}                 | enter and leave                                                           |
 * | {@link module:ngMessages#animations ngMessages}                               | add and remove (the `ng-active`/`ng-inactive` classes)                    |
 * | {@link ng.directive:ngModel#animations ngModel}                               | add and remove ({@link ng.directive:ngModel#css-classes various classes}) |
 * | {@link ng.directive:ngRepeat#animations ngRepeat}                             | enter, leave, and move                                                    |
 * | {@link ng.directive:ngShow#animations ngShow}                                 | add and remove (the `ng-hide` class)                                      |
 * | {@link ng.directive:ngSwitch#animations ngSwitch}                             | enter and leave                                                           |
 * | {@link ngRoute.directive:ngView#animations ngView}                            | enter and leave                                                           |
 *
 * (More information can be found by visiting the documentation associated with each directive.)
 *
 * For a full breakdown of the steps involved during each animation event, refer to the
 * {@link ng.$animate `$animate` API docs}.
 *
 * ## CSS-based Animations
 *
 * CSS-based animations with ngAnimate are unique since they require no JavaScript code at all. By using a CSS class that we reference between our HTML
 * and CSS code we can create an animation that will be picked up by AngularJS when an underlying directive performs an operation.
 *
 * The example below shows how an `enter` animation can be made possible on an element using `ng-if`:
 *
 * ```html
 * <div ng-if="bool" class="fade">
 *    Fade me in out
 * </div>
 * <button ng-click="bool=true">Fade In!</button>
 * <button ng-click="bool=false">Fade Out!</button>
 * ```
 *
 * Notice the CSS class **fade**? We can now create the CSS transition code that references this class:
 *
 * ```css
 * /&#42; The starting CSS styles for the enter animation &#42;/
 * .fade.ng-enter {
 *   transition:0.5s linear all;
 *   opacity:0;
 * }
 *
 * /&#42; The finishing CSS styles for the enter animation &#42;/
 * .fade.ng-enter.ng-enter-active {
 *   opacity:1;
 * }
 * ```
 *
 * The key thing to remember here is that, depending on the animation event (which each of the directives above trigger depending on what's going on) two
 * generated CSS classes will be applied to the element; in the example above we have `.ng-enter` and `.ng-enter-active`. For CSS transitions, the transition
 * code **must** be defined within the starting CSS class (in this case `.ng-enter`). The destination class is what the transition will animate towards.
 *
 * If for example we wanted to create animations for `leave` and `move` (ngRepeat triggers move) then we can do so using the same CSS naming conventions:
 *
 * ```css
 * /&#42; now the element will fade out before it is removed from the DOM &#42;/
 * .fade.ng-leave {
 *   transition:0.5s linear all;
 *   opacity:1;
 * }
 * .fade.ng-leave.ng-leave-active {
 *   opacity:0;
 * }
 * ```
 *
 * We can also make use of **CSS Keyframes** by referencing the keyframe animation within the starting CSS class:
 *
 * ```css
 * /&#42; there is no need to define anything inside of the destination
 * CSS class since the keyframe will take charge of the animation &#42;/
 * .fade.ng-leave {
 *   animation: my_fade_animation 0.5s linear;
 *   -webkit-animation: my_fade_animation 0.5s linear;
 * }
 *
 * @keyframes my_fade_animation {
 *   from { opacity:1; }
 *   to { opacity:0; }
 * }
 *
 * @-webkit-keyframes my_fade_animation {
 *   from { opacity:1; }
 *   to { opacity:0; }
 * }
 * ```
 *
 * Feel free also mix transitions and keyframes together as well as any other CSS classes on the same element.
 *
 * ### CSS Class-based Animations
 *
 * Class-based animations (animations that are triggered via `ngClass`, `ngShow`, `ngHide` and some other directives) have a slightly different
 * naming convention. Class-based animations are basic enough that a standard transition or keyframe can be referenced on the class being added
 * and removed.
 *
 * For example if we wanted to do a CSS animation for `ngHide` then we place an animation on the `.ng-hide` CSS class:
 *
 * ```html
 * <div ng-show="bool" class="fade">
 *   Show and hide me
 * </div>
 * <button ng-click="bool=!bool">Toggle</button>
 *
 * <style>
 * .fade.ng-hide {
 *   transition:0.5s linear all;
 *   opacity:0;
 * }
 * </style>
 * ```
 *
 * All that is going on here with ngShow/ngHide behind the scenes is the `.ng-hide` class is added/removed (when the hidden state is valid). Since
 * ngShow and ngHide are animation aware then we can match up a transition and ngAnimate handles the rest.
 *
 * In addition the addition and removal of the CSS class, ngAnimate also provides two helper methods that we can use to further decorate the animation
 * with CSS styles.
 *
 * ```html
 * <div ng-class="{on:onOff}" class="highlight">
 *   Highlight this box
 * </div>
 * <button ng-click="onOff=!onOff">Toggle</button>
 *
 * <style>
 * .highlight {
 *   transition:0.5s linear all;
 * }
 * .highlight.on-add {
 *   background:white;
 * }
 * .highlight.on {
 *   background:yellow;
 * }
 * .highlight.on-remove {
 *   background:black;
 * }
 * </style>
 * ```
 *
 * We can also make use of CSS keyframes by placing them within the CSS classes.
 *
 *
 * ### CSS Staggering Animations
 * A Staggering animation is a collection of animations that are issued with a slight delay in between each successive operation resulting in a
 * curtain-like effect. The ngAnimate module (versions >=1.2) supports staggering animations and the stagger effect can be
 * performed by creating a **ng-EVENT-stagger** CSS class and attaching that class to the base CSS class used for
 * the animation. The style property expected within the stagger class can either be a **transition-delay** or an
 * **animation-delay** property (or both if your animation contains both transitions and keyframe animations).
 *
 * ```css
 * .my-animation.ng-enter {
 *   /&#42; standard transition code &#42;/
 *   transition: 1s linear all;
 *   opacity:0;
 * }
 * .my-animation.ng-enter-stagger {
 *   /&#42; this will have a 100ms delay between each successive leave animation &#42;/
 *   transition-delay: 0.1s;
 *
 *   /&#42; As of 1.4.4, this must always be set: it signals ngAnimate
 *     to not accidentally inherit a delay property from another CSS class &#42;/
 *   transition-duration: 0s;
 *
 *   /&#42; if you are using animations instead of transitions you should configure as follows:
 *     animation-delay: 0.1s;
 *     animation-duration: 0s; &#42;/
 * }
 * .my-animation.ng-enter.ng-enter-active {
 *   /&#42; standard transition styles &#42;/
 *   opacity:1;
 * }
 * ```
 *
 * Staggering animations work by default in ngRepeat (so long as the CSS class is defined). Outside of ngRepeat, to use staggering animations
 * on your own, they can be triggered by firing multiple calls to the same event on $animate. However, the restrictions surrounding this
 * are that each of the elements must have the same CSS className value as well as the same parent element. A stagger operation
 * will also be reset if one or more animation frames have passed since the multiple calls to `$animate` were fired.
 *
 * The following code will issue the **ng-leave-stagger** event on the element provided:
 *
 * ```js
 * var kids = parent.children();
 *
 * $animate.leave(kids[0]); //stagger index=0
 * $animate.leave(kids[1]); //stagger index=1
 * $animate.leave(kids[2]); //stagger index=2
 * $animate.leave(kids[3]); //stagger index=3
 * $animate.leave(kids[4]); //stagger index=4
 *
 * window.requestAnimationFrame(function() {
 *   //stagger has reset itself
 *   $animate.leave(kids[5]); //stagger index=0
 *   $animate.leave(kids[6]); //stagger index=1
 *
 *   $scope.$digest();
 * });
 * ```
 *
 * Stagger animations are currently only supported within CSS-defined animations.
 *
 * ### The `ng-animate` CSS class
 *
 * When ngAnimate is animating an element it will apply the `ng-animate` CSS class to the element for the duration of the animation.
 * This is a temporary CSS class and it will be removed once the animation is over (for both JavaScript and CSS-based animations).
 *
 * Therefore, animations can be applied to an element using this temporary class directly via CSS.
 *
 * ```css
 * .zipper.ng-animate {
 *   transition:0.5s linear all;
 * }
 * .zipper.ng-enter {
 *   opacity:0;
 * }
 * .zipper.ng-enter.ng-enter-active {
 *   opacity:1;
 * }
 * .zipper.ng-leave {
 *   opacity:1;
 * }
 * .zipper.ng-leave.ng-leave-active {
 *   opacity:0;
 * }
 * ```
 *
 * (Note that the `ng-animate` CSS class is reserved and it cannot be applied on an element directly since ngAnimate will always remove
 * the CSS class once an animation has completed.)
 *
 *
 * ### The `ng-[event]-prepare` class
 *
 * This is a special class that can be used to prevent unwanted flickering / flash of content before
 * the actual animation starts. The class is added as soon as an animation is initialized, but removed
 * before the actual animation starts (after waiting for a $digest).
 * It is also only added for *structural* animations (`enter`, `move`, and `leave`).
 *
 * In practice, flickering can appear when nesting elements with structural animations such as `ngIf`
 * into elements that have class-based animations such as `ngClass`.
 *
 * ```html
 * <div ng-class="{red: myProp}">
 *   <div ng-class="{blue: myProp}">
 *     <div class="message" ng-if="myProp"></div>
 *   </div>
 * </div>
 * ```
 *
 * It is possible that during the `enter` animation, the `.message` div will be briefly visible before it starts animating.
 * In that case, you can add styles to the CSS that make sure the element stays hidden before the animation starts:
 *
 * ```css
 * .message.ng-enter-prepare {
 *   opacity: 0;
 * }
 * ```
 *
 * ### Animating between value changes
 *
 * Sometimes you need to animate between different expression states, whose values
 * don't necessary need to be known or referenced in CSS styles.
 * Unless possible with another {@link ngAnimate#directive-support "animation aware" directive},
 * that specific use case can always be covered with {@link ngAnimate.directive:ngAnimateSwap} as
 * can be seen in {@link ngAnimate.directive:ngAnimateSwap#examples this example}.
 *
 * Note that {@link ngAnimate.directive:ngAnimateSwap} is a *structural directive*, which means it
 * creates a new instance of the element (including any other/child directives it may have) and
 * links it to a new scope every time *swap* happens. In some cases this might not be desirable
 * (e.g. for performance reasons, or when you wish to retain internal state on the original
 * element instance).
 *
 * ## JavaScript-based Animations
 *
 * ngAnimate also allows for animations to be consumed by JavaScript code. The approach is similar to CSS-based animations (where there is a shared
 * CSS class that is referenced in our HTML code) but in addition we need to register the JavaScript animation on the module. By making use of the
 * `module.animation()` module function we can register the animation.
 *
 * Let's see an example of a enter/leave animation using `ngRepeat`:
 *
 * ```html
 * <div ng-repeat="item in items" class="slide">
 *   {{ item }}
 * </div>
 * ```
 *
 * See the **slide** CSS class? Let's use that class to define an animation that we'll structure in our module code by using `module.animation`:
 *
 * ```js
 * myModule.animation('.slide', [function() {
 *   return {
 *     // make note that other events (like addClass/removeClass)
 *     // have different function input parameters
 *     enter: function(element, doneFn) {
 *       jQuery(element).fadeIn(1000, doneFn);
 *
 *       // remember to call doneFn so that AngularJS
 *       // knows that the animation has concluded
 *     },
 *
 *     move: function(element, doneFn) {
 *       jQuery(element).fadeIn(1000, doneFn);
 *     },
 *
 *     leave: function(element, doneFn) {
 *       jQuery(element).fadeOut(1000, doneFn);
 *     }
 *   }
 * }]);
 * ```
 *
 * The nice thing about JS-based animations is that we can inject other services and make use of advanced animation libraries such as
 * greensock.js and velocity.js.
 *
 * If our animation code class-based (meaning that something like `ngClass`, `ngHide` and `ngShow` triggers it) then we can still define
 * our animations inside of the same registered animation, however, the function input arguments are a bit different:
 *
 * ```html
 * <div ng-class="color" class="colorful">
 *   this box is moody
 * </div>
 * <button ng-click="color='red'">Change to red</button>
 * <button ng-click="color='blue'">Change to blue</button>
 * <button ng-click="color='green'">Change to green</button>
 * ```
 *
 * ```js
 * myModule.animation('.colorful', [function() {
 *   return {
 *     addClass: function(element, className, doneFn) {
 *       // do some cool animation and call the doneFn
 *     },
 *     removeClass: function(element, className, doneFn) {
 *       // do some cool animation and call the doneFn
 *     },
 *     setClass: function(element, addedClass, removedClass, doneFn) {
 *       // do some cool animation and call the doneFn
 *     }
 *   }
 * }]);
 * ```
 *
 * ## CSS + JS Animations Together
 *
 * AngularJS 1.4 and higher has taken steps to make the amalgamation of CSS and JS animations more flexible. However, unlike earlier versions of AngularJS,
 * defining CSS and JS animations to work off of the same CSS class will not work anymore. Therefore the example below will only result in **JS animations taking
 * charge of the animation**:
 *
 * ```html
 * <div ng-if="bool" class="slide">
 *   Slide in and out
 * </div>
 * ```
 *
 * ```js
 * myModule.animation('.slide', [function() {
 *   return {
 *     enter: function(element, doneFn) {
 *       jQuery(element).slideIn(1000, doneFn);
 *     }
 *   }
 * }]);
 * ```
 *
 * ```css
 * .slide.ng-enter {
 *   transition:0.5s linear all;
 *   transform:translateY(-100px);
 * }
 * .slide.ng-enter.ng-enter-active {
 *   transform:translateY(0);
 * }
 * ```
 *
 * Does this mean that CSS and JS animations cannot be used together? Do JS-based animations always have higher priority? We can make up for the
 * lack of CSS animations by using the `$animateCss` service to trigger our own tweaked-out, CSS-based animations directly from
 * our own JS-based animation code:
 *
 * ```js
 * myModule.animation('.slide', ['$animateCss', function($animateCss) {
 *   return {
 *     enter: function(element) {
*        // this will trigger `.slide.ng-enter` and `.slide.ng-enter-active`.
 *       return $animateCss(element, {
 *         event: 'enter',
 *         structural: true
 *       });
 *     }
 *   }
 * }]);
 * ```
 *
 * The nice thing here is that we can save bandwidth by sticking to our CSS-based animation code and we don't need to rely on a 3rd-party animation framework.
 *
 * The `$animateCss` service is very powerful since we can feed in all kinds of extra properties that will be evaluated and fed into a CSS transition or
 * keyframe animation. For example if we wanted to animate the height of an element while adding and removing classes then we can do so by providing that
 * data into `$animateCss` directly:
 *
 * ```js
 * myModule.animation('.slide', ['$animateCss', function($animateCss) {
 *   return {
 *     enter: function(element) {
 *       return $animateCss(element, {
 *         event: 'enter',
 *         structural: true,
 *         addClass: 'maroon-setting',
 *         from: { height:0 },
 *         to: { height: 200 }
 *       });
 *     }
 *   }
 * }]);
 * ```
 *
 * Now we can fill in the rest via our transition CSS code:
 *
 * ```css
 * /&#42; the transition tells ngAnimate to make the animation happen &#42;/
 * .slide.ng-enter { transition:0.5s linear all; }
 *
 * /&#42; this extra CSS class will be absorbed into the transition
 * since the $animateCss code is adding the class &#42;/
 * .maroon-setting { background:red; }
 * ```
 *
 * And `$animateCss` will figure out the rest. Just make sure to have the `done()` callback fire the `doneFn` function to signal when the animation is over.
 *
 * To learn more about what's possible be sure to visit the {@link ngAnimate.$animateCss $animateCss service}.
 *
 * ## Animation Anchoring (via `ng-animate-ref`)
 *
 * ngAnimate in AngularJS 1.4 comes packed with the ability to cross-animate elements between
 * structural areas of an application (like views) by pairing up elements using an attribute
 * called `ng-animate-ref`.
 *
 * Let's say for example we have two views that are managed by `ng-view` and we want to show
 * that there is a relationship between two components situated in within these views. By using the
 * `ng-animate-ref` attribute we can identify that the two components are paired together and we
 * can then attach an animation, which is triggered when the view changes.
 *
 * Say for example we have the following template code:
 *
 * ```html
 * <!-- index.html -->
 * <div ng-view class="view-animation">
 * </div>
 *
 * <!-- home.html -->
 * <a href="#/banner-page">
 *   <img src="./banner.jpg" class="banner" ng-animate-ref="banner">
 * </a>
 *
 * <!-- banner-page.html -->
 * <img src="./banner.jpg" class="banner" ng-animate-ref="banner">
 * ```
 *
 * Now, when the view changes (once the link is clicked), ngAnimate will examine the
 * HTML contents to see if there is a match reference between any components in the view
 * that is leaving and the view that is entering. It will scan both the view which is being
 * removed (leave) and inserted (enter) to see if there are any paired DOM elements that
 * contain a matching ref value.
 *
 * The two images match since they share the same ref value. ngAnimate will now create a
 * transport element (which is a clone of the first image element) and it will then attempt
 * to animate to the position of the second image element in the next view. For the animation to
 * work a special CSS class called `ng-anchor` will be added to the transported element.
 *
 * We can now attach a transition onto the `.banner.ng-anchor` CSS class and then
 * ngAnimate will handle the entire transition for us as well as the addition and removal of
 * any changes of CSS classes between the elements:
 *
 * ```css
 * .banner.ng-anchor {
 *   /&#42; this animation will last for 1 second since there are
 *          two phases to the animation (an `in` and an `out` phase) &#42;/
 *   transition:0.5s linear all;
 * }
 * ```
 *
 * We also **must** include animations for the views that are being entered and removed
 * (otherwise anchoring wouldn't be possible since the new view would be inserted right away).
 *
 * ```css
 * .view-animation.ng-enter, .view-animation.ng-leave {
 *   transition:0.5s linear all;
 *   position:fixed;
 *   left:0;
 *   top:0;
 *   width:100%;
 * }
 * .view-animation.ng-enter {
 *   transform:translateX(100%);
 * }
 * .view-animation.ng-leave,
 * .view-animation.ng-enter.ng-enter-active {
 *   transform:translateX(0%);
 * }
 * .view-animation.ng-leave.ng-leave-active {
 *   transform:translateX(-100%);
 * }
 * ```
 *
 * Now we can jump back to the anchor animation. When the animation happens, there are two stages that occur:
 * an `out` and an `in` stage. The `out` stage happens first and that is when the element is animated away
 * from its origin. Once that animation is over then the `in` stage occurs which animates the
 * element to its destination. The reason why there are two animations is to give enough time
 * for the enter animation on the new element to be ready.
 *
 * The example above sets up a transition for both the in and out phases, but we can also target the out or
 * in phases directly via `ng-anchor-out` and `ng-anchor-in`.
 *
 * ```css
 * .banner.ng-anchor-out {
 *   transition: 0.5s linear all;
 *
 *   /&#42; the scale will be applied during the out animation,
 *          but will be animated away when the in animation runs &#42;/
 *   transform: scale(1.2);
 * }
 *
 * .banner.ng-anchor-in {
 *   transition: 1s linear all;
 * }
 * ```
 *
 *
 *
 *
 * ### Anchoring Demo
 *
  <example module="anchoringExample"
           name="anchoringExample"
           id="anchoringExample"
           deps="angular-animate.js;angular-route.js"
           animations="true">
    <file name="index.html">
      <a href="#!/">Home</a>
      <hr />
      <div class="view-container">
        <div ng-view class="view"></div>
      </div>
    </file>
    <file name="script.js">
      angular.module('anchoringExample', ['ngAnimate', 'ngRoute'])
        .config(['$routeProvider', function($routeProvider) {
          $routeProvider.when('/', {
            templateUrl: 'home.html',
            controller: 'HomeController as home'
          });
          $routeProvider.when('/profile/:id', {
            templateUrl: 'profile.html',
            controller: 'ProfileController as profile'
          });
        }])
        .run(['$rootScope', function($rootScope) {
          $rootScope.records = [
            { id: 1, title: 'Miss Beulah Roob' },
            { id: 2, title: 'Trent Morissette' },
            { id: 3, title: 'Miss Ava Pouros' },
            { id: 4, title: 'Rod Pouros' },
            { id: 5, title: 'Abdul Rice' },
            { id: 6, title: 'Laurie Rutherford Sr.' },
            { id: 7, title: 'Nakia McLaughlin' },
            { id: 8, title: 'Jordon Blanda DVM' },
            { id: 9, title: 'Rhoda Hand' },
            { id: 10, title: 'Alexandrea Sauer' }
          ];
        }])
        .controller('HomeController', [function() {
          //empty
        }])
        .controller('ProfileController', ['$rootScope', '$routeParams',
            function ProfileController($rootScope, $routeParams) {
          var index = parseInt($routeParams.id, 10);
          var record = $rootScope.records[index - 1];

          this.title = record.title;
          this.id = record.id;
        }]);
    </file>
    <file name="home.html">
      <h2>Welcome to the home page</h1>
      <p>Please click on an element</p>
      <a class="record"
         ng-href="#!/profile/{{ record.id }}"
         ng-animate-ref="{{ record.id }}"
         ng-repeat="record in records">
        {{ record.title }}
      </a>
    </file>
    <file name="profile.html">
      <div class="profile record" ng-animate-ref="{{ profile.id }}">
        {{ profile.title }}
      </div>
    </file>
    <file name="animations.css">
      .record {
        display:block;
        font-size:20px;
      }
      .profile {
        background:black;
        color:white;
        font-size:100px;
      }
      .view-container {
        position:relative;
      }
      .view-container > .view.ng-animate {
        position:absolute;
        top:0;
        left:0;
        width:100%;
        min-height:500px;
      }
      .view.ng-enter, .view.ng-leave,
      .record.ng-anchor {
        transition:0.5s linear all;
      }
      .view.ng-enter {
        transform:translateX(100%);
      }
      .view.ng-enter.ng-enter-active, .view.ng-leave {
        transform:translateX(0%);
      }
      .view.ng-leave.ng-leave-active {
        transform:translateX(-100%);
      }
      .record.ng-anchor-out {
        background:red;
      }
    </file>
  </example>
 *
 * ### How is the element transported?
 *
 * When an anchor animation occurs, ngAnimate will clone the starting element and position it exactly where the starting
 * element is located on screen via absolute positioning. The cloned element will be placed inside of the root element
 * of the application (where ng-app was defined) and all of the CSS classes of the starting element will be applied. The
 * element will then animate into the `out` and `in` animations and will eventually reach the coordinates and match
 * the dimensions of the destination element. During the entire animation a CSS class of `.ng-animate-shim` will be applied
 * to both the starting and destination elements in order to hide them from being visible (the CSS styling for the class
 * is: `visibility:hidden`). Once the anchor reaches its destination then it will be removed and the destination element
 * will become visible since the shim class will be removed.
 *
 * ### How is the morphing handled?
 *
 * CSS Anchoring relies on transitions and keyframes and the internal code is intelligent enough to figure out
 * what CSS classes differ between the starting element and the destination element. These different CSS classes
 * will be added/removed on the anchor element and a transition will be applied (the transition that is provided
 * in the anchor class). Long story short, ngAnimate will figure out what classes to add and remove which will
 * make the transition of the element as smooth and automatic as possible. Be sure to use simple CSS classes that
 * do not rely on DOM nesting structure so that the anchor element appears the same as the starting element (since
 * the cloned element is placed inside of root element which is likely close to the body element).
 *
 * Note that if the root element is on the `<html>` element then the cloned node will be placed inside of body.
 *
 *
 * ## Using $animate in your directive code
 *
 * So far we've explored how to feed in animations into an AngularJS application, but how do we trigger animations within our own directives in our application?
 * By injecting the `$animate` service into our directive code, we can trigger structural and class-based hooks which can then be consumed by animations. Let's
 * imagine we have a greeting box that shows and hides itself when the data changes
 *
 * ```html
 * <greeting-box active="onOrOff">Hi there</greeting-box>
 * ```
 *
 * ```js
 * ngModule.directive('greetingBox', ['$animate', function($animate) {
 *   return function(scope, element, attrs) {
 *     attrs.$observe('active', function(value) {
 *       value ? $animate.addClass(element, 'on') : $animate.removeClass(element, 'on');
 *     });
 *   });
 * }]);
 * ```
 *
 * Now the `on` CSS class is added and removed on the greeting box component. Now if we add a CSS class on top of the greeting box element
 * in our HTML code then we can trigger a CSS or JS animation to happen.
 *
 * ```css
 * /&#42; normally we would create a CSS class to reference on the element &#42;/
 * greeting-box.on { transition:0.5s linear all; background:green; color:white; }
 * ```
 *
 * The `$animate` service contains a variety of other methods like `enter`, `leave`, `animate` and `setClass`. To learn more about what's
 * possible be sure to visit the {@link ng.$animate $animate service API page}.
 *
 *
 * ## Callbacks and Promises
 *
 * When `$animate` is called it returns a promise that can be used to capture when the animation has ended. Therefore if we were to trigger
 * an animation (within our directive code) then we can continue performing directive and scope related activities after the animation has
 * ended by chaining onto the returned promise that animation method returns.
 *
 * ```js
 * // somewhere within the depths of the directive
 * $animate.enter(element, parent).then(function() {
 *   //the animation has completed
 * });
 * ```
 *
 * (Note that earlier versions of AngularJS prior to v1.4 required the promise code to be wrapped using `$scope.$apply(...)`. This is not the case
 * anymore.)
 *
 * In addition to the animation promise, we can also make use of animation-related callbacks within our directives and controller code by registering
 * an event listener using the `$animate` service. Let's say for example that an animation was triggered on our view
 * routing controller to hook into that:
 *
 * ```js
 * ngModule.controller('HomePageController', ['$animate', function($animate) {
 *   $animate.on('enter', ngViewElement, function(element) {
 *     // the animation for this route has completed
 *   }]);
 * }])
 * ```
 *
 * (Note that you will need to trigger a digest within the callback to get AngularJS to notice any scope-related changes.)
 */

var copy;
var extend;
var forEach;
var isArray;
var isDefined;
var isElement;
var isFunction;
var isObject;
var isString;
var isUndefined;
var jqLite;
var noop;

/**
 * @ngdoc service
 * @name $animate
 * @kind object
 *
 * @description
 * The ngAnimate `$animate` service documentation is the same for the core `$animate` service.
 *
 * Click here {@link ng.$animate to learn more about animations with `$animate`}.
 */
angular.module('ngAnimate', [], function initAngularHelpers() {
  // Access helpers from AngularJS core.
  // Do it inside a `config` block to ensure `window.angular` is available.
  noop        = angular.noop;
  copy        = angular.copy;
  extend      = angular.extend;
  jqLite      = angular.element;
  forEach     = angular.forEach;
  isArray     = angular.isArray;
  isString    = angular.isString;
  isObject    = angular.isObject;
  isUndefined = angular.isUndefined;
  isDefined   = angular.isDefined;
  isFunction  = angular.isFunction;
  isElement   = angular.isElement;
})
  .info({ angularVersion: '1.8.0' })
  .directive('ngAnimateSwap', ngAnimateSwapDirective)

  .directive('ngAnimateChildren', $$AnimateChildrenDirective)
  .factory('$$rAFScheduler', $$rAFSchedulerFactory)

  .provider('$$animateQueue', $$AnimateQueueProvider)
  .provider('$$animateCache', $$AnimateCacheProvider)
  .provider('$$animation', $$AnimationProvider)

  .provider('$animateCss', $AnimateCssProvider)
  .provider('$$animateCssDriver', $$AnimateCssDriverProvider)

  .provider('$$animateJs', $$AnimateJsProvider)
  .provider('$$animateJsDriver', $$AnimateJsDriverProvider);


})(window, window.angular);

/**
 * State-based routing for AngularJS
 * @version v0.2.13
 * @link http://angular-ui.github.com/
 * @license MIT License, http://www.opensource.org/licenses/MIT
 */
"undefined"!=typeof module&&"undefined"!=typeof exports&&module.exports===exports&&(module.exports="ui.router"),function(a,b,c){"use strict";function d(a,b){return M(new(M(function(){},{prototype:a})),b)}function e(a){return L(arguments,function(b){b!==a&&L(b,function(b,c){a.hasOwnProperty(c)||(a[c]=b)})}),a}function f(a,b){var c=[];for(var d in a.path){if(a.path[d]!==b.path[d])break;c.push(a.path[d])}return c}function g(a){if(Object.keys)return Object.keys(a);var c=[];return b.forEach(a,function(a,b){c.push(b)}),c}function h(a,b){if(Array.prototype.indexOf)return a.indexOf(b,Number(arguments[2])||0);var c=a.length>>>0,d=Number(arguments[2])||0;for(d=0>d?Math.ceil(d):Math.floor(d),0>d&&(d+=c);c>d;d++)if(d in a&&a[d]===b)return d;return-1}function i(a,b,c,d){var e,i=f(c,d),j={},k=[];for(var l in i)if(i[l].params&&(e=g(i[l].params),e.length))for(var m in e)h(k,e[m])>=0||(k.push(e[m]),j[e[m]]=a[e[m]]);return M({},j,b)}function j(a,b,c){if(!c){c=[];for(var d in a)c.push(d)}for(var e=0;e<c.length;e++){var f=c[e];if(a[f]!=b[f])return!1}return!0}function k(a,b){var c={};return L(a,function(a){c[a]=b[a]}),c}function l(a){var b={},c=Array.prototype.concat.apply(Array.prototype,Array.prototype.slice.call(arguments,1));for(var d in a)-1==h(c,d)&&(b[d]=a[d]);return b}function m(a,b){var c=K(a),d=c?[]:{};return L(a,function(a,e){b(a,e)&&(d[c?d.length:e]=a)}),d}function n(a,b){var c=K(a)?[]:{};return L(a,function(a,d){c[d]=b(a,d)}),c}function o(a,b){var d=1,f=2,i={},j=[],k=i,m=M(a.when(i),{$$promises:i,$$values:i});this.study=function(i){function n(a,c){if(s[c]!==f){if(r.push(c),s[c]===d)throw r.splice(0,h(r,c)),new Error("Cyclic dependency: "+r.join(" -> "));if(s[c]=d,I(a))q.push(c,[function(){return b.get(a)}],j);else{var e=b.annotate(a);L(e,function(a){a!==c&&i.hasOwnProperty(a)&&n(i[a],a)}),q.push(c,a,e)}r.pop(),s[c]=f}}function o(a){return J(a)&&a.then&&a.$$promises}if(!J(i))throw new Error("'invocables' must be an object");var p=g(i||{}),q=[],r=[],s={};return L(i,n),i=r=s=null,function(d,f,g){function h(){--u||(v||e(t,f.$$values),r.$$values=t,r.$$promises=r.$$promises||!0,delete r.$$inheritedValues,n.resolve(t))}function i(a){r.$$failure=a,n.reject(a)}function j(c,e,f){function j(a){l.reject(a),i(a)}function k(){if(!G(r.$$failure))try{l.resolve(b.invoke(e,g,t)),l.promise.then(function(a){t[c]=a,h()},j)}catch(a){j(a)}}var l=a.defer(),m=0;L(f,function(a){s.hasOwnProperty(a)&&!d.hasOwnProperty(a)&&(m++,s[a].then(function(b){t[a]=b,--m||k()},j))}),m||k(),s[c]=l.promise}if(o(d)&&g===c&&(g=f,f=d,d=null),d){if(!J(d))throw new Error("'locals' must be an object")}else d=k;if(f){if(!o(f))throw new Error("'parent' must be a promise returned by $resolve.resolve()")}else f=m;var n=a.defer(),r=n.promise,s=r.$$promises={},t=M({},d),u=1+q.length/3,v=!1;if(G(f.$$failure))return i(f.$$failure),r;f.$$inheritedValues&&e(t,l(f.$$inheritedValues,p)),M(s,f.$$promises),f.$$values?(v=e(t,l(f.$$values,p)),r.$$inheritedValues=l(f.$$values,p),h()):(f.$$inheritedValues&&(r.$$inheritedValues=l(f.$$inheritedValues,p)),f.then(h,i));for(var w=0,x=q.length;x>w;w+=3)d.hasOwnProperty(q[w])?h():j(q[w],q[w+1],q[w+2]);return r}},this.resolve=function(a,b,c,d){return this.study(a)(b,c,d)}}function p(a,b,c){this.fromConfig=function(a,b,c){return G(a.template)?this.fromString(a.template,b):G(a.templateUrl)?this.fromUrl(a.templateUrl,b):G(a.templateProvider)?this.fromProvider(a.templateProvider,b,c):null},this.fromString=function(a,b){return H(a)?a(b):a},this.fromUrl=function(c,d){return H(c)&&(c=c(d)),null==c?null:a.get(c,{cache:b,headers:{Accept:"text/html"}}).then(function(a){return a.data})},this.fromProvider=function(a,b,d){return c.invoke(a,null,d||{params:b})}}function q(a,b,e){function f(b,c,d,e){if(q.push(b),o[b])return o[b];if(!/^\w+(-+\w+)*(?:\[\])?$/.test(b))throw new Error("Invalid parameter name '"+b+"' in pattern '"+a+"'");if(p[b])throw new Error("Duplicate parameter name '"+b+"' in pattern '"+a+"'");return p[b]=new O.Param(b,c,d,e),p[b]}function g(a,b,c){var d=["",""],e=a.replace(/[\\\[\]\^$*+?.()|{}]/g,"\\$&");if(!b)return e;switch(c){case!1:d=["(",")"];break;case!0:d=["?(",")?"];break;default:d=["("+c+"|",")?"]}return e+d[0]+b+d[1]}function h(c,e){var f,g,h,i,j;return f=c[2]||c[3],j=b.params[f],h=a.substring(m,c.index),g=e?c[4]:c[4]||("*"==c[1]?".*":null),i=O.type(g||"string")||d(O.type("string"),{pattern:new RegExp(g)}),{id:f,regexp:g,segment:h,type:i,cfg:j}}b=M({params:{}},J(b)?b:{});var i,j=/([:*])([\w\[\]]+)|\{([\w\[\]]+)(?:\:((?:[^{}\\]+|\\.|\{(?:[^{}\\]+|\\.)*\})+))?\}/g,k=/([:]?)([\w\[\]-]+)|\{([\w\[\]-]+)(?:\:((?:[^{}\\]+|\\.|\{(?:[^{}\\]+|\\.)*\})+))?\}/g,l="^",m=0,n=this.segments=[],o=e?e.params:{},p=this.params=e?e.params.$$new():new O.ParamSet,q=[];this.source=a;for(var r,s,t;(i=j.exec(a))&&(r=h(i,!1),!(r.segment.indexOf("?")>=0));)s=f(r.id,r.type,r.cfg,"path"),l+=g(r.segment,s.type.pattern.source,s.squash),n.push(r.segment),m=j.lastIndex;t=a.substring(m);var u=t.indexOf("?");if(u>=0){var v=this.sourceSearch=t.substring(u);if(t=t.substring(0,u),this.sourcePath=a.substring(0,m+u),v.length>0)for(m=0;i=k.exec(v);)r=h(i,!0),s=f(r.id,r.type,r.cfg,"search"),m=j.lastIndex}else this.sourcePath=a,this.sourceSearch="";l+=g(t)+(b.strict===!1?"/?":"")+"$",n.push(t),this.regexp=new RegExp(l,b.caseInsensitive?"i":c),this.prefix=n[0],this.$$paramNames=q}function r(a){M(this,a)}function s(){function a(a){return null!=a?a.toString().replace(/\//g,"%2F"):a}function e(a){return null!=a?a.toString().replace(/%2F/g,"/"):a}function f(a){return this.pattern.test(a)}function i(){return{strict:t,caseInsensitive:p}}function j(a){return H(a)||K(a)&&H(a[a.length-1])}function k(){for(;x.length;){var a=x.shift();if(a.pattern)throw new Error("You cannot override a type's .pattern at runtime.");b.extend(v[a.name],o.invoke(a.def))}}function l(a){M(this,a||{})}O=this;var o,p=!1,t=!0,u=!1,v={},w=!0,x=[],y={string:{encode:a,decode:e,is:f,pattern:/[^/]*/},"int":{encode:a,decode:function(a){return parseInt(a,10)},is:function(a){return G(a)&&this.decode(a.toString())===a},pattern:/\d+/},bool:{encode:function(a){return a?1:0},decode:function(a){return 0!==parseInt(a,10)},is:function(a){return a===!0||a===!1},pattern:/0|1/},date:{encode:function(a){return this.is(a)?[a.getFullYear(),("0"+(a.getMonth()+1)).slice(-2),("0"+a.getDate()).slice(-2)].join("-"):c},decode:function(a){if(this.is(a))return a;var b=this.capture.exec(a);return b?new Date(b[1],b[2]-1,b[3]):c},is:function(a){return a instanceof Date&&!isNaN(a.valueOf())},equals:function(a,b){return this.is(a)&&this.is(b)&&a.toISOString()===b.toISOString()},pattern:/[0-9]{4}-(?:0[1-9]|1[0-2])-(?:0[1-9]|[1-2][0-9]|3[0-1])/,capture:/([0-9]{4})-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])/},json:{encode:b.toJson,decode:b.fromJson,is:b.isObject,equals:b.equals,pattern:/[^/]*/},any:{encode:b.identity,decode:b.identity,is:b.identity,equals:b.equals,pattern:/.*/}};s.$$getDefaultValue=function(a){if(!j(a.value))return a.value;if(!o)throw new Error("Injectable functions cannot be called at configuration time");return o.invoke(a.value)},this.caseInsensitive=function(a){return G(a)&&(p=a),p},this.strictMode=function(a){return G(a)&&(t=a),t},this.defaultSquashPolicy=function(a){if(!G(a))return u;if(a!==!0&&a!==!1&&!I(a))throw new Error("Invalid squash policy: "+a+". Valid policies: false, true, arbitrary-string");return u=a,a},this.compile=function(a,b){return new q(a,M(i(),b))},this.isMatcher=function(a){if(!J(a))return!1;var b=!0;return L(q.prototype,function(c,d){H(c)&&(b=b&&G(a[d])&&H(a[d]))}),b},this.type=function(a,b,c){if(!G(b))return v[a];if(v.hasOwnProperty(a))throw new Error("A type named '"+a+"' has already been defined.");return v[a]=new r(M({name:a},b)),c&&(x.push({name:a,def:c}),w||k()),this},L(y,function(a,b){v[b]=new r(M({name:b},a))}),v=d(v,{}),this.$get=["$injector",function(a){return o=a,w=!1,k(),L(y,function(a,b){v[b]||(v[b]=new r(a))}),this}],this.Param=function(a,b,d,e){function f(a){var b=J(a)?g(a):[],c=-1===h(b,"value")&&-1===h(b,"type")&&-1===h(b,"squash")&&-1===h(b,"array");return c&&(a={value:a}),a.$$fn=j(a.value)?a.value:function(){return a.value},a}function i(b,c,d){if(b.type&&c)throw new Error("Param '"+a+"' has two type configurations.");return c?c:b.type?b.type instanceof r?b.type:new r(b.type):"config"===d?v.any:v.string}function k(){var b={array:"search"===e?"auto":!1},c=a.match(/\[\]$/)?{array:!0}:{};return M(b,c,d).array}function l(a,b){var c=a.squash;if(!b||c===!1)return!1;if(!G(c)||null==c)return u;if(c===!0||I(c))return c;throw new Error("Invalid squash policy: '"+c+"'. Valid policies: false, true, or arbitrary string")}function p(a,b,d,e){var f,g,i=[{from:"",to:d||b?c:""},{from:null,to:d||b?c:""}];return f=K(a.replace)?a.replace:[],I(e)&&f.push({from:e,to:c}),g=n(f,function(a){return a.from}),m(i,function(a){return-1===h(g,a.from)}).concat(f)}function q(){if(!o)throw new Error("Injectable functions cannot be called at configuration time");return o.invoke(d.$$fn)}function s(a){function b(a){return function(b){return b.from===a}}function c(a){var c=n(m(w.replace,b(a)),function(a){return a.to});return c.length?c[0]:a}return a=c(a),G(a)?w.type.decode(a):q()}function t(){return"{Param:"+a+" "+b+" squash: '"+z+"' optional: "+y+"}"}var w=this;d=f(d),b=i(d,b,e);var x=k();b=x?b.$asArray(x,"search"===e):b,"string"!==b.name||x||"path"!==e||d.value!==c||(d.value="");var y=d.value!==c,z=l(d,y),A=p(d,x,y,z);M(this,{id:a,type:b,location:e,array:x,squash:z,replace:A,isOptional:y,value:s,dynamic:c,config:d,toString:t})},l.prototype={$$new:function(){return d(this,M(new l,{$$parent:this}))},$$keys:function(){for(var a=[],b=[],c=this,d=g(l.prototype);c;)b.push(c),c=c.$$parent;return b.reverse(),L(b,function(b){L(g(b),function(b){-1===h(a,b)&&-1===h(d,b)&&a.push(b)})}),a},$$values:function(a){var b={},c=this;return L(c.$$keys(),function(d){b[d]=c[d].value(a&&a[d])}),b},$$equals:function(a,b){var c=!0,d=this;return L(d.$$keys(),function(e){var f=a&&a[e],g=b&&b[e];d[e].type.equals(f,g)||(c=!1)}),c},$$validates:function(a){var b,c,d,e=!0,f=this;return L(this.$$keys(),function(g){d=f[g],c=a[g],b=!c&&d.isOptional,e=e&&(b||!!d.type.is(c))}),e},$$parent:c},this.ParamSet=l}function t(a,d){function e(a){var b=/^\^((?:\\[^a-zA-Z0-9]|[^\\\[\]\^$*+?.()|{}]+)*)/.exec(a.source);return null!=b?b[1].replace(/\\(.)/g,"$1"):""}function f(a,b){return a.replace(/\$(\$|\d{1,2})/,function(a,c){return b["$"===c?0:Number(c)]})}function g(a,b,c){if(!c)return!1;var d=a.invoke(b,b,{$match:c});return G(d)?d:!0}function h(d,e,f,g){function h(a,b,c){return"/"===p?a:b?p.slice(0,-1)+a:c?p.slice(1)+a:a}function m(a){function b(a){var b=a(f,d);return b?(I(b)&&d.replace().url(b),!0):!1}if(!a||!a.defaultPrevented){var e=o&&d.url()===o;if(o=c,e)return!0;var g,h=j.length;for(g=0;h>g;g++)if(b(j[g]))return;k&&b(k)}}function n(){return i=i||e.$on("$locationChangeSuccess",m)}var o,p=g.baseHref(),q=d.url();return l||n(),{sync:function(){m()},listen:function(){return n()},update:function(a){return a?void(q=d.url()):void(d.url()!==q&&(d.url(q),d.replace()))},push:function(a,b,e){d.url(a.format(b||{})),o=e&&e.$$avoidResync?d.url():c,e&&e.replace&&d.replace()},href:function(c,e,f){if(!c.validates(e))return null;var g=a.html5Mode();b.isObject(g)&&(g=g.enabled);var i=c.format(e);if(f=f||{},g||null===i||(i="#"+a.hashPrefix()+i),i=h(i,g,f.absolute),!f.absolute||!i)return i;var j=!g&&i?"/":"",k=d.port();return k=80===k||443===k?"":":"+k,[d.protocol(),"://",d.host(),k,j,i].join("")}}}var i,j=[],k=null,l=!1;this.rule=function(a){if(!H(a))throw new Error("'rule' must be a function");return j.push(a),this},this.otherwise=function(a){if(I(a)){var b=a;a=function(){return b}}else if(!H(a))throw new Error("'rule' must be a function");return k=a,this},this.when=function(a,b){var c,h=I(b);if(I(a)&&(a=d.compile(a)),!h&&!H(b)&&!K(b))throw new Error("invalid 'handler' in when()");var i={matcher:function(a,b){return h&&(c=d.compile(b),b=["$match",function(a){return c.format(a)}]),M(function(c,d){return g(c,b,a.exec(d.path(),d.search()))},{prefix:I(a.prefix)?a.prefix:""})},regex:function(a,b){if(a.global||a.sticky)throw new Error("when() RegExp must not be global or sticky");return h&&(c=b,b=["$match",function(a){return f(c,a)}]),M(function(c,d){return g(c,b,a.exec(d.path()))},{prefix:e(a)})}},j={matcher:d.isMatcher(a),regex:a instanceof RegExp};for(var k in j)if(j[k])return this.rule(i[k](a,b));throw new Error("invalid 'what' in when()")},this.deferIntercept=function(a){a===c&&(a=!0),l=a},this.$get=h,h.$inject=["$location","$rootScope","$injector","$browser"]}function u(a,e){function f(a){return 0===a.indexOf(".")||0===a.indexOf("^")}function l(a,b){if(!a)return c;var d=I(a),e=d?a:a.name,g=f(e);if(g){if(!b)throw new Error("No reference point given for path '"+e+"'");b=l(b);for(var h=e.split("."),i=0,j=h.length,k=b;j>i;i++)if(""!==h[i]||0!==i){if("^"!==h[i])break;if(!k.parent)throw new Error("Path '"+e+"' not valid for state '"+b.name+"'");k=k.parent}else k=b;h=h.slice(i).join("."),e=k.name+(k.name&&h?".":"")+h}var m=y[e];return!m||!d&&(d||m!==a&&m.self!==a)?c:m}function m(a,b){z[a]||(z[a]=[]),z[a].push(b)}function o(a){for(var b=z[a]||[];b.length;)p(b.shift())}function p(b){b=d(b,{self:b,resolve:b.resolve||{},toString:function(){return this.name}});var c=b.name;if(!I(c)||c.indexOf("@")>=0)throw new Error("State must have a valid name");if(y.hasOwnProperty(c))throw new Error("State '"+c+"'' is already defined");var e=-1!==c.indexOf(".")?c.substring(0,c.lastIndexOf(".")):I(b.parent)?b.parent:J(b.parent)&&I(b.parent.name)?b.parent.name:"";if(e&&!y[e])return m(e,b.self);for(var f in B)H(B[f])&&(b[f]=B[f](b,B.$delegates[f]));return y[c]=b,!b[A]&&b.url&&a.when(b.url,["$match","$stateParams",function(a,c){x.$current.navigable==b&&j(a,c)||x.transitionTo(b,a,{inherit:!0,location:!1})}]),o(c),b}function q(a){return a.indexOf("*")>-1}function r(a){var b=a.split("."),c=x.$current.name.split(".");if("**"===b[0]&&(c=c.slice(h(c,b[1])),c.unshift("**")),"**"===b[b.length-1]&&(c.splice(h(c,b[b.length-2])+1,Number.MAX_VALUE),c.push("**")),b.length!=c.length)return!1;for(var d=0,e=b.length;e>d;d++)"*"===b[d]&&(c[d]="*");return c.join("")===b.join("")}function s(a,b){return I(a)&&!G(b)?B[a]:H(b)&&I(a)?(B[a]&&!B.$delegates[a]&&(B.$delegates[a]=B[a]),B[a]=b,this):this}function t(a,b){return J(a)?b=a:b.name=a,p(b),this}function u(a,e,f,h,m,o,p){function s(b,c,d,f){var g=a.$broadcast("$stateNotFound",b,c,d);if(g.defaultPrevented)return p.update(),B;if(!g.retry)return null;if(f.$retry)return p.update(),C;var h=x.transition=e.when(g.retry);return h.then(function(){return h!==x.transition?u:(b.options.$retry=!0,x.transitionTo(b.to,b.toParams,b.options))},function(){return B}),p.update(),h}function t(a,c,d,g,i,j){var l=d?c:k(a.params.$$keys(),c),n={$stateParams:l};i.resolve=m.resolve(a.resolve,n,i.resolve,a);var o=[i.resolve.then(function(a){i.globals=a})];return g&&o.push(g),L(a.views,function(c,d){var e=c.resolve&&c.resolve!==a.resolve?c.resolve:{};e.$template=[function(){return f.load(d,{view:c,locals:n,params:l,notify:j.notify})||""}],o.push(m.resolve(e,n,i.resolve,a).then(function(f){if(H(c.controllerProvider)||K(c.controllerProvider)){var g=b.extend({},e,n);f.$$controller=h.invoke(c.controllerProvider,null,g)}else f.$$controller=c.controller;f.$$state=a,f.$$controllerAs=c.controllerAs,i[d]=f}))}),e.all(o).then(function(){return i})}var u=e.reject(new Error("transition superseded")),z=e.reject(new Error("transition prevented")),B=e.reject(new Error("transition aborted")),C=e.reject(new Error("transition failed"));return w.locals={resolve:null,globals:{$stateParams:{}}},x={params:{},current:w.self,$current:w,transition:null},x.reload=function(){return x.transitionTo(x.current,o,{reload:!0,inherit:!1,notify:!0})},x.go=function(a,b,c){return x.transitionTo(a,b,M({inherit:!0,relative:x.$current},c))},x.transitionTo=function(b,c,f){c=c||{},f=M({location:!0,inherit:!1,relative:null,notify:!0,reload:!1,$retry:!1},f||{});var g,j=x.$current,m=x.params,n=j.path,q=l(b,f.relative);if(!G(q)){var r={to:b,toParams:c,options:f},y=s(r,j.self,m,f);if(y)return y;if(b=r.to,c=r.toParams,f=r.options,q=l(b,f.relative),!G(q)){if(!f.relative)throw new Error("No such state '"+b+"'");throw new Error("Could not resolve '"+b+"' from state '"+f.relative+"'")}}if(q[A])throw new Error("Cannot transition to abstract state '"+b+"'");if(f.inherit&&(c=i(o,c||{},x.$current,q)),!q.params.$$validates(c))return C;c=q.params.$$values(c),b=q;var B=b.path,D=0,E=B[D],F=w.locals,H=[];if(!f.reload)for(;E&&E===n[D]&&E.ownParams.$$equals(c,m);)F=H[D]=E.locals,D++,E=B[D];if(v(b,j,F,f))return b.self.reloadOnSearch!==!1&&p.update(),x.transition=null,e.when(x.current);if(c=k(b.params.$$keys(),c||{}),f.notify&&a.$broadcast("$stateChangeStart",b.self,c,j.self,m).defaultPrevented)return p.update(),z;for(var I=e.when(F),J=D;J<B.length;J++,E=B[J])F=H[J]=d(F),I=t(E,c,E===b,I,F,f);var K=x.transition=I.then(function(){var d,e,g;if(x.transition!==K)return u;for(d=n.length-1;d>=D;d--)g=n[d],g.self.onExit&&h.invoke(g.self.onExit,g.self,g.locals.globals),g.locals=null;for(d=D;d<B.length;d++)e=B[d],e.locals=H[d],e.self.onEnter&&h.invoke(e.self.onEnter,e.self,e.locals.globals);return x.transition!==K?u:(x.$current=b,x.current=b.self,x.params=c,N(x.params,o),x.transition=null,f.location&&b.navigable&&p.push(b.navigable.url,b.navigable.locals.globals.$stateParams,{$$avoidResync:!0,replace:"replace"===f.location}),f.notify&&a.$broadcast("$stateChangeSuccess",b.self,c,j.self,m),p.update(!0),x.current)},function(d){return x.transition!==K?u:(x.transition=null,g=a.$broadcast("$stateChangeError",b.self,c,j.self,m,d),g.defaultPrevented||p.update(),e.reject(d))});return K},x.is=function(a,b,d){d=M({relative:x.$current},d||{});var e=l(a,d.relative);return G(e)?x.$current!==e?!1:b?j(e.params.$$values(b),o):!0:c},x.includes=function(a,b,d){if(d=M({relative:x.$current},d||{}),I(a)&&q(a)){if(!r(a))return!1;a=x.$current.name}var e=l(a,d.relative);return G(e)?G(x.$current.includes[e.name])?b?j(e.params.$$values(b),o,g(b)):!0:!1:c},x.href=function(a,b,d){d=M({lossy:!0,inherit:!0,absolute:!1,relative:x.$current},d||{});var e=l(a,d.relative);if(!G(e))return null;d.inherit&&(b=i(o,b||{},x.$current,e));var f=e&&d.lossy?e.navigable:e;return f&&f.url!==c&&null!==f.url?p.href(f.url,k(e.params.$$keys(),b||{}),{absolute:d.absolute}):null},x.get=function(a,b){if(0===arguments.length)return n(g(y),function(a){return y[a].self});var c=l(a,b||x.$current);return c&&c.self?c.self:null},x}function v(a,b,c,d){return a!==b||(c!==b.locals||d.reload)&&a.self.reloadOnSearch!==!1?void 0:!0}var w,x,y={},z={},A="abstract",B={parent:function(a){if(G(a.parent)&&a.parent)return l(a.parent);var b=/^(.+)\.[^.]+$/.exec(a.name);return b?l(b[1]):w},data:function(a){return a.parent&&a.parent.data&&(a.data=a.self.data=M({},a.parent.data,a.data)),a.data},url:function(a){var b=a.url,c={params:a.params||{}};if(I(b))return"^"==b.charAt(0)?e.compile(b.substring(1),c):(a.parent.navigable||w).url.concat(b,c);if(!b||e.isMatcher(b))return b;throw new Error("Invalid url '"+b+"' in state '"+a+"'")},navigable:function(a){return a.url?a:a.parent?a.parent.navigable:null},ownParams:function(a){var b=a.url&&a.url.params||new O.ParamSet;return L(a.params||{},function(a,c){b[c]||(b[c]=new O.Param(c,null,a,"config"))}),b},params:function(a){return a.parent&&a.parent.params?M(a.parent.params.$$new(),a.ownParams):new O.ParamSet},views:function(a){var b={};return L(G(a.views)?a.views:{"":a},function(c,d){d.indexOf("@")<0&&(d+="@"+a.parent.name),b[d]=c}),b},path:function(a){return a.parent?a.parent.path.concat(a):[]},includes:function(a){var b=a.parent?M({},a.parent.includes):{};return b[a.name]=!0,b},$delegates:{}};w=p({name:"",url:"^",views:null,"abstract":!0}),w.navigable=null,this.decorator=s,this.state=t,this.$get=u,u.$inject=["$rootScope","$q","$view","$injector","$resolve","$stateParams","$urlRouter","$location","$urlMatcherFactory"]}function v(){function a(a,b){return{load:function(c,d){var e,f={template:null,controller:null,view:null,locals:null,notify:!0,async:!0,params:{}};return d=M(f,d),d.view&&(e=b.fromConfig(d.view,d.params,d.locals)),e&&d.notify&&a.$broadcast("$viewContentLoading",d),e}}}this.$get=a,a.$inject=["$rootScope","$templateFactory"]}function w(){var a=!1;this.useAnchorScroll=function(){a=!0},this.$get=["$anchorScroll","$timeout",function(b,c){return a?b:function(a){c(function(){a[0].scrollIntoView()},0,!1)}}]}function x(a,c,d,e){function f(){return c.has?function(a){return c.has(a)?c.get(a):null}:function(a){try{return c.get(a)}catch(b){return null}}}function g(a,b){var c=function(){return{enter:function(a,b,c){b.after(a),c()},leave:function(a,b){a.remove(),b()}}};if(j)return{enter:function(a,b,c){var d=j.enter(a,null,b,c);d&&d.then&&d.then(c)},leave:function(a,b){var c=j.leave(a,b);c&&c.then&&c.then(b)}};if(i){var d=i&&i(b,a);return{enter:function(a,b,c){d.enter(a,null,b),c()},leave:function(a,b){d.leave(a),b()}}}return c()}var h=f(),i=h("$animator"),j=h("$animate"),k={restrict:"ECA",terminal:!0,priority:400,transclude:"element",compile:function(c,f,h){return function(c,f,i){function j(){l&&(l.remove(),l=null),n&&(n.$destroy(),n=null),m&&(r.leave(m,function(){l=null}),l=m,m=null)}function k(g){var k,l=z(c,i,f,e),s=l&&a.$current&&a.$current.locals[l];if(g||s!==o){k=c.$new(),o=a.$current.locals[l];var t=h(k,function(a){r.enter(a,f,function(){n&&n.$emit("$viewContentAnimationEnded"),(b.isDefined(q)&&!q||c.$eval(q))&&d(a)}),j()});m=t,n=k,n.$emit("$viewContentLoaded"),n.$eval(p)}}var l,m,n,o,p=i.onload||"",q=i.autoscroll,r=g(i,c);c.$on("$stateChangeSuccess",function(){k(!1)}),c.$on("$viewContentLoading",function(){k(!1)}),k(!0)}}};return k}function y(a,b,c,d){return{restrict:"ECA",priority:-400,compile:function(e){var f=e.html();return function(e,g,h){var i=c.$current,j=z(e,h,g,d),k=i&&i.locals[j];if(k){g.data("$uiView",{name:j,state:k.$$state}),g.html(k.$template?k.$template:f);var l=a(g.contents());if(k.$$controller){k.$scope=e;var m=b(k.$$controller,k);k.$$controllerAs&&(e[k.$$controllerAs]=m),g.data("$ngControllerController",m),g.children().data("$ngControllerController",m)}l(e)}}}}}function z(a,b,c,d){var e=d(b.uiView||b.name||"")(a),f=c.inheritedData("$uiView");return e.indexOf("@")>=0?e:e+"@"+(f?f.state.name:"")}function A(a,b){var c,d=a.match(/^\s*({[^}]*})\s*$/);if(d&&(a=b+"("+d[1]+")"),c=a.replace(/\n/g," ").match(/^([^(]+?)\s*(\((.*)\))?$/),!c||4!==c.length)throw new Error("Invalid state ref '"+a+"'");return{state:c[1],paramExpr:c[3]||null}}function B(a){var b=a.parent().inheritedData("$uiView");return b&&b.state&&b.state.name?b.state:void 0}function C(a,c){var d=["location","inherit","reload"];return{restrict:"A",require:["?^uiSrefActive","?^uiSrefActiveEq"],link:function(e,f,g,h){var i=A(g.uiSref,a.current.name),j=null,k=B(f)||a.$current,l=null,m="A"===f.prop("tagName"),n="FORM"===f[0].nodeName,o=n?"action":"href",p=!0,q={relative:k,inherit:!0},r=e.$eval(g.uiSrefOpts)||{};b.forEach(d,function(a){a in r&&(q[a]=r[a])});var s=function(c){if(c&&(j=b.copy(c)),p){l=a.href(i.state,j,q);var d=h[1]||h[0];return d&&d.$$setStateInfo(i.state,j),null===l?(p=!1,!1):void g.$set(o,l)}};i.paramExpr&&(e.$watch(i.paramExpr,function(a){a!==j&&s(a)},!0),j=b.copy(e.$eval(i.paramExpr))),s(),n||f.bind("click",function(b){var d=b.which||b.button;if(!(d>1||b.ctrlKey||b.metaKey||b.shiftKey||f.attr("target"))){var e=c(function(){a.go(i.state,j,q)});b.preventDefault();var g=m&&!l?1:0;b.preventDefault=function(){g--<=0&&c.cancel(e)}}})}}}function D(a,b,c){return{restrict:"A",controller:["$scope","$element","$attrs",function(b,d,e){function f(){g()?d.addClass(j):d.removeClass(j)}function g(){return"undefined"!=typeof e.uiSrefActiveEq?h&&a.is(h.name,i):h&&a.includes(h.name,i)}var h,i,j;j=c(e.uiSrefActiveEq||e.uiSrefActive||"",!1)(b),this.$$setStateInfo=function(b,c){h=a.get(b,B(d)),i=c,f()},b.$on("$stateChangeSuccess",f)}]}}function E(a){var b=function(b){return a.is(b)};return b.$stateful=!0,b}function F(a){var b=function(b){return a.includes(b)};return b.$stateful=!0,b}var G=b.isDefined,H=b.isFunction,I=b.isString,J=b.isObject,K=b.isArray,L=b.forEach,M=b.extend,N=b.copy;b.module("ui.router.util",["ng"]),b.module("ui.router.router",["ui.router.util"]),b.module("ui.router.state",["ui.router.router","ui.router.util"]),b.module("ui.router",["ui.router.state"]),b.module("ui.router.compat",["ui.router"]),o.$inject=["$q","$injector"],b.module("ui.router.util").service("$resolve",o),p.$inject=["$http","$templateCache","$injector"],b.module("ui.router.util").service("$templateFactory",p);var O;q.prototype.concat=function(a,b){var c={caseInsensitive:O.caseInsensitive(),strict:O.strictMode(),squash:O.defaultSquashPolicy()};return new q(this.sourcePath+a+this.sourceSearch,M(c,b),this)},q.prototype.toString=function(){return this.source},q.prototype.exec=function(a,b){function c(a){function b(a){return a.split("").reverse().join("")}function c(a){return a.replace(/\\-/,"-")}var d=b(a).split(/-(?!\\)/),e=n(d,b);return n(e,c).reverse()}var d=this.regexp.exec(a);if(!d)return null;b=b||{};var e,f,g,h=this.parameters(),i=h.length,j=this.segments.length-1,k={};if(j!==d.length-1)throw new Error("Unbalanced capture group in route '"+this.source+"'");for(e=0;j>e;e++){g=h[e];var l=this.params[g],m=d[e+1];for(f=0;f<l.replace;f++)l.replace[f].from===m&&(m=l.replace[f].to);m&&l.array===!0&&(m=c(m)),k[g]=l.value(m)}for(;i>e;e++)g=h[e],k[g]=this.params[g].value(b[g]);return k},q.prototype.parameters=function(a){return G(a)?this.params[a]||null:this.$$paramNames},q.prototype.validates=function(a){return this.params.$$validates(a)},q.prototype.format=function(a){function b(a){return encodeURIComponent(a).replace(/-/g,function(a){return"%5C%"+a.charCodeAt(0).toString(16).toUpperCase()})}a=a||{};var c=this.segments,d=this.parameters(),e=this.params;if(!this.validates(a))return null;var f,g=!1,h=c.length-1,i=d.length,j=c[0];for(f=0;i>f;f++){var k=h>f,l=d[f],m=e[l],o=m.value(a[l]),p=m.isOptional&&m.type.equals(m.value(),o),q=p?m.squash:!1,r=m.type.encode(o);if(k){var s=c[f+1];if(q===!1)null!=r&&(j+=K(r)?n(r,b).join("-"):encodeURIComponent(r)),j+=s;else if(q===!0){var t=j.match(/\/$/)?/\/?(.*)/:/(.*)/;j+=s.match(t)[1]}else I(q)&&(j+=q+s)}else{if(null==r||p&&q!==!1)continue;K(r)||(r=[r]),r=n(r,encodeURIComponent).join("&"+l+"="),j+=(g?"&":"?")+(l+"="+r),g=!0}}return j},r.prototype.is=function(){return!0},r.prototype.encode=function(a){return a},r.prototype.decode=function(a){return a},r.prototype.equals=function(a,b){return a==b},r.prototype.$subPattern=function(){var a=this.pattern.toString();return a.substr(1,a.length-2)},r.prototype.pattern=/.*/,r.prototype.toString=function(){return"{Type:"+this.name+"}"},r.prototype.$asArray=function(a,b){function d(a,b){function d(a,b){return function(){return a[b].apply(a,arguments)}}function e(a){return K(a)?a:G(a)?[a]:[]}function f(a){switch(a.length){case 0:return c;case 1:return"auto"===b?a[0]:a;default:return a}}function g(a){return!a}function h(a,b){return function(c){c=e(c);var d=n(c,a);return b===!0?0===m(d,g).length:f(d)}}function i(a){return function(b,c){var d=e(b),f=e(c);if(d.length!==f.length)return!1;for(var g=0;g<d.length;g++)if(!a(d[g],f[g]))return!1;return!0}}this.encode=h(d(a,"encode")),this.decode=h(d(a,"decode")),this.is=h(d(a,"is"),!0),this.equals=i(d(a,"equals")),this.pattern=a.pattern,this.$arrayMode=b}if(!a)return this;if("auto"===a&&!b)throw new Error("'auto' array mode is for query parameters only");return new d(this,a)},b.module("ui.router.util").provider("$urlMatcherFactory",s),b.module("ui.router.util").run(["$urlMatcherFactory",function(){}]),t.$inject=["$locationProvider","$urlMatcherFactoryProvider"],b.module("ui.router.router").provider("$urlRouter",t),u.$inject=["$urlRouterProvider","$urlMatcherFactoryProvider"],b.module("ui.router.state").value("$stateParams",{}).provider("$state",u),v.$inject=[],b.module("ui.router.state").provider("$view",v),b.module("ui.router.state").provider("$uiViewScroll",w),x.$inject=["$state","$injector","$uiViewScroll","$interpolate"],y.$inject=["$compile","$controller","$state","$interpolate"],b.module("ui.router.state").directive("uiView",x),b.module("ui.router.state").directive("uiView",y),C.$inject=["$state","$timeout"],D.$inject=["$state","$stateParams","$interpolate"],b.module("ui.router.state").directive("uiSref",C).directive("uiSrefActive",D).directive("uiSrefActiveEq",D),E.$inject=["$state"],F.$inject=["$state"],b.module("ui.router.state").filter("isState",E).filter("includedByState",F)}(window,window.angular);
/*
 AngularJS v1.8.0
 (c) 2010-2020 Google, Inc. http://angularjs.org
 License: MIT
*/
(function(s,c){'use strict';function P(c){var h=[];C(h,E).chars(c);return h.join("")}var D=c.$$minErr("$sanitize"),F,h,G,H,I,q,E,J,K,C;c.module("ngSanitize",[]).provider("$sanitize",function(){function f(a,e){return B(a.split(","),e)}function B(a,e){var d={},b;for(b=0;b<a.length;b++)d[e?q(a[b]):a[b]]=!0;return d}function t(a,e){e&&e.length&&h(a,B(e))}function Q(a){for(var e={},d=0,b=a.length;d<b;d++){var k=a[d];e[k.name]=k.value}return e}function L(a){return a.replace(/&/g,"&amp;").replace(z,function(a){var d=
a.charCodeAt(0);a=a.charCodeAt(1);return"&#"+(1024*(d-55296)+(a-56320)+65536)+";"}).replace(u,function(a){return"&#"+a.charCodeAt(0)+";"}).replace(/</g,"&lt;").replace(/>/g,"&gt;")}function A(a){for(;a;){if(a.nodeType===s.Node.ELEMENT_NODE)for(var e=a.attributes,d=0,b=e.length;d<b;d++){var k=e[d],g=k.name.toLowerCase();if("xmlns:ns1"===g||0===g.lastIndexOf("ns1:",0))a.removeAttributeNode(k),d--,b--}(e=a.firstChild)&&A(e);a=v("nextSibling",a)}}function v(a,e){var d=e[a];if(d&&J.call(e,d))throw D("elclob",
e.outerHTML||e.outerText);return d}var y=!1,g=!1;this.$get=["$$sanitizeUri",function(a){y=!0;g&&h(m,l);return function(e){var d=[];K(e,C(d,function(b,d){return!/^unsafe:/.test(a(b,d))}));return d.join("")}}];this.enableSvg=function(a){return I(a)?(g=a,this):g};this.addValidElements=function(a){y||(H(a)&&(a={htmlElements:a}),t(l,a.svgElements),t(r,a.htmlVoidElements),t(m,a.htmlVoidElements),t(m,a.htmlElements));return this};this.addValidAttrs=function(a){y||h(M,B(a,!0));return this};F=c.bind;h=c.extend;
G=c.forEach;H=c.isArray;I=c.isDefined;q=c.$$lowercase;E=c.noop;K=function(a,e){null===a||void 0===a?a="":"string"!==typeof a&&(a=""+a);var d=N(a);if(!d)return"";var b=5;do{if(0===b)throw D("uinput");b--;a=d.innerHTML;d=N(a)}while(a!==d.innerHTML);for(b=d.firstChild;b;){switch(b.nodeType){case 1:e.start(b.nodeName.toLowerCase(),Q(b.attributes));break;case 3:e.chars(b.textContent)}var k;if(!(k=b.firstChild)&&(1===b.nodeType&&e.end(b.nodeName.toLowerCase()),k=v("nextSibling",b),!k))for(;null==k;){b=
v("parentNode",b);if(b===d)break;k=v("nextSibling",b);1===b.nodeType&&e.end(b.nodeName.toLowerCase())}b=k}for(;b=d.firstChild;)d.removeChild(b)};C=function(a,e){var d=!1,b=F(a,a.push);return{start:function(a,g){a=q(a);!d&&w[a]&&(d=a);d||!0!==m[a]||(b("<"),b(a),G(g,function(d,g){var c=q(g),f="img"===a&&"src"===c||"background"===c;!0!==M[c]||!0===O[c]&&!e(d,f)||(b(" "),b(g),b('="'),b(L(d)),b('"'))}),b(">"))},end:function(a){a=q(a);d||!0!==m[a]||!0===r[a]||(b("</"),b(a),b(">"));a==d&&(d=!1)},chars:function(a){d||
b(L(a))}}};J=s.Node.prototype.contains||function(a){return!!(this.compareDocumentPosition(a)&16)};var z=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,u=/([^#-~ |!])/g,r=f("area,br,col,hr,img,wbr"),x=f("colgroup,dd,dt,li,p,tbody,td,tfoot,th,thead,tr"),p=f("rp,rt"),n=h({},p,x),x=h({},x,f("address,article,aside,blockquote,caption,center,del,dir,div,dl,figure,figcaption,footer,h1,h2,h3,h4,h5,h6,header,hgroup,hr,ins,map,menu,nav,ol,pre,section,table,ul")),p=h({},p,f("a,abbr,acronym,b,bdi,bdo,big,br,cite,code,del,dfn,em,font,i,img,ins,kbd,label,map,mark,q,ruby,rp,rt,s,samp,small,span,strike,strong,sub,sup,time,tt,u,var")),
l=f("circle,defs,desc,ellipse,font-face,font-face-name,font-face-src,g,glyph,hkern,image,linearGradient,line,marker,metadata,missing-glyph,mpath,path,polygon,polyline,radialGradient,rect,stop,svg,switch,text,title,tspan"),w=f("script,style"),m=h({},r,x,p,n),O=f("background,cite,href,longdesc,src,xlink:href,xml:base"),n=f("abbr,align,alt,axis,bgcolor,border,cellpadding,cellspacing,class,clear,color,cols,colspan,compact,coords,dir,face,headers,height,hreflang,hspace,ismap,lang,language,nohref,nowrap,rel,rev,rows,rowspan,rules,scope,scrolling,shape,size,span,start,summary,tabindex,target,title,type,valign,value,vspace,width"),
p=f("accent-height,accumulate,additive,alphabetic,arabic-form,ascent,baseProfile,bbox,begin,by,calcMode,cap-height,class,color,color-rendering,content,cx,cy,d,dx,dy,descent,display,dur,end,fill,fill-rule,font-family,font-size,font-stretch,font-style,font-variant,font-weight,from,fx,fy,g1,g2,glyph-name,gradientUnits,hanging,height,horiz-adv-x,horiz-origin-x,ideographic,k,keyPoints,keySplines,keyTimes,lang,marker-end,marker-mid,marker-start,markerHeight,markerUnits,markerWidth,mathematical,max,min,offset,opacity,orient,origin,overline-position,overline-thickness,panose-1,path,pathLength,points,preserveAspectRatio,r,refX,refY,repeatCount,repeatDur,requiredExtensions,requiredFeatures,restart,rotate,rx,ry,slope,stemh,stemv,stop-color,stop-opacity,strikethrough-position,strikethrough-thickness,stroke,stroke-dasharray,stroke-dashoffset,stroke-linecap,stroke-linejoin,stroke-miterlimit,stroke-opacity,stroke-width,systemLanguage,target,text-anchor,to,transform,type,u1,u2,underline-position,underline-thickness,unicode,unicode-range,units-per-em,values,version,viewBox,visibility,width,widths,x,x-height,x1,x2,xlink:actuate,xlink:arcrole,xlink:role,xlink:show,xlink:title,xlink:type,xml:base,xml:lang,xml:space,xmlns,xmlns:xlink,y,y1,y2,zoomAndPan",
!0),M=h({},O,p,n),N=function(a,e){function d(b){b="<remove></remove>"+b;try{var d=(new a.DOMParser).parseFromString(b,"text/html").body;d.firstChild.remove();return d}catch(e){}}function b(a){c.innerHTML=a;e.documentMode&&A(c);return c}var g;if(e&&e.implementation)g=e.implementation.createHTMLDocument("inert");else throw D("noinert");var c=(g.documentElement||g.getDocumentElement()).querySelector("body");c.innerHTML='<svg><g onload="this.parentNode.remove()"></g></svg>';return c.querySelector("svg")?
(c.innerHTML='<svg><p><style><img src="</style><img src=x onerror=alert(1)//">',c.querySelector("svg img")?d:b):function(b){b="<remove></remove>"+b;try{b=encodeURI(b)}catch(d){return}var e=new a.XMLHttpRequest;e.responseType="document";e.open("GET","data:text/html;charset=utf-8,"+b,!1);e.send(null);b=e.response.body;b.firstChild.remove();return b}}(s,s.document)}).info({angularVersion:"1.8.0"});c.module("ngSanitize").filter("linky",["$sanitize",function(f){var h=/((s?ftp|https?):\/\/|(www\.)|(mailto:)?[A-Za-z0-9._%+-]+@)\S*[^\s.;,(){}<>"\u201d\u2019]/i,
t=/^mailto:/i,q=c.$$minErr("linky"),s=c.isDefined,A=c.isFunction,v=c.isObject,y=c.isString;return function(c,z,u){function r(c){c&&l.push(P(c))}function x(c,g){var f,a=p(c);l.push("<a ");for(f in a)l.push(f+'="'+a[f]+'" ');!s(z)||"target"in a||l.push('target="',z,'" ');l.push('href="',c.replace(/"/g,"&quot;"),'">');r(g);l.push("</a>")}if(null==c||""===c)return c;if(!y(c))throw q("notstring",c);for(var p=A(u)?u:v(u)?function(){return u}:function(){return{}},n=c,l=[],w,m;c=n.match(h);)w=c[0],c[2]||
c[4]||(w=(c[3]?"http://":"mailto:")+w),m=c.index,r(n.substr(0,m)),x(w,c[0].replace(t,"")),n=n.substring(m+c[0].length);r(n);return f(l.join(""))}}])})(window,window.angular);
//# sourceMappingURL=angular-sanitize.min.js.map

/*!
 * Copyright 2015 Drifty Co.
 * http://drifty.com/
 *
 * Ionic, v1.3.4
 * A powerful HTML5 mobile app framework.
 * http://ionicframework.com/
 *
 * By @maxlynch, @benjsperry, @adamdbradley <3
 *
 * Licensed under the MIT license. Please see LICENSE for more information.
 *
 */

(function() {

// Create global ionic obj and its namespaces
// build processes may have already created an ionic obj
window.ionic = window.ionic || {};
window.ionic.views = {};
window.ionic.version = '1.3.4';

(function (ionic) {

  ionic.DelegateService = function(methodNames) {

    if (methodNames.indexOf('$getByHandle') > -1) {
      throw new Error("Method '$getByHandle' is implicitly added to each delegate service. Do not list it as a method.");
    }

    function trueFn() { return true; }

    return ['$log', function($log) {

      /*
       * Creates a new object that will have all the methodNames given,
       * and call them on the given the controller instance matching given
       * handle.
       * The reason we don't just let $getByHandle return the controller instance
       * itself is that the controller instance might not exist yet.
       *
       * We want people to be able to do
       * `var instance = $ionicScrollDelegate.$getByHandle('foo')` on controller
       * instantiation, but on controller instantiation a child directive
       * may not have been compiled yet!
       *
       * So this is our way of solving this problem: we create an object
       * that will only try to fetch the controller with given handle
       * once the methods are actually called.
       */
      function DelegateInstance(instances, handle) {
        this._instances = instances;
        this.handle = handle;
      }
      methodNames.forEach(function(methodName) {
        DelegateInstance.prototype[methodName] = instanceMethodCaller(methodName);
      });


      /**
       * The delegate service (eg $ionicNavBarDelegate) is just an instance
       * with a non-defined handle, a couple extra methods for registering
       * and narrowing down to a specific handle.
       */
      function DelegateService() {
        this._instances = [];
      }
      DelegateService.prototype = DelegateInstance.prototype;
      DelegateService.prototype._registerInstance = function(instance, handle, filterFn) {
        var instances = this._instances;
        instance.$$delegateHandle = handle;
        instance.$$filterFn = filterFn || trueFn;
        instances.push(instance);

        return function deregister() {
          var index = instances.indexOf(instance);
          if (index !== -1) {
            instances.splice(index, 1);
          }
        };
      };
      DelegateService.prototype.$getByHandle = function(handle) {
        return new DelegateInstance(this._instances, handle);
      };

      return new DelegateService();

      function instanceMethodCaller(methodName) {
        return function caller() {
          var handle = this.handle;
          var args = arguments;
          var foundInstancesCount = 0;
          var returnValue;

          this._instances.forEach(function(instance) {
            if ((!handle || handle == instance.$$delegateHandle) && instance.$$filterFn(instance)) {
              foundInstancesCount++;
              var ret = instance[methodName].apply(instance, args);
              //Only return the value from the first call
              if (foundInstancesCount === 1) {
                returnValue = ret;
              }
            }
          });

          if (!foundInstancesCount && handle) {
            return $log.warn(
              'Delegate for handle "' + handle + '" could not find a ' +
              'corresponding element with delegate-handle="' + handle + '"! ' +
              methodName + '() was not called!\n' +
              'Possible cause: If you are calling ' + methodName + '() immediately, and ' +
              'your element with delegate-handle="' + handle + '" is a child of your ' +
              'controller, then your element may not be compiled yet. Put a $timeout ' +
              'around your call to ' + methodName + '() and try again.'
            );
          }
          return returnValue;
        };
      }

    }];
  };

})(window.ionic);

(function(window, document, ionic) {

  var readyCallbacks = [];
  var isDomReady = document.readyState === 'complete' || document.readyState === 'interactive';

  function domReady() {
    isDomReady = true;
    for (var x = 0; x < readyCallbacks.length; x++) {
      ionic.requestAnimationFrame(readyCallbacks[x]);
    }
    readyCallbacks = [];
    document.removeEventListener('DOMContentLoaded', domReady);
  }
  if (!isDomReady) {
    document.addEventListener('DOMContentLoaded', domReady);
  }


  // From the man himself, Mr. Paul Irish.
  // The requestAnimationFrame polyfill
  // Put it on window just to preserve its context
  // without having to use .call
  window._rAF = (function() {
    return window.requestAnimationFrame ||
           window.webkitRequestAnimationFrame ||
           window.mozRequestAnimationFrame ||
           function(callback) {
             window.setTimeout(callback, 16);
           };
  })();

  var cancelAnimationFrame = window.cancelAnimationFrame ||
    window.webkitCancelAnimationFrame ||
    window.mozCancelAnimationFrame ||
    window.webkitCancelRequestAnimationFrame;

  /**
  * @ngdoc utility
  * @name ionic.DomUtil
  * @module ionic
  */
  ionic.DomUtil = {
    //Call with proper context
    /**
     * @ngdoc method
     * @name ionic.DomUtil#requestAnimationFrame
     * @alias ionic.requestAnimationFrame
     * @description Calls [requestAnimationFrame](https://developer.mozilla.org/en-US/docs/Web/API/window.requestAnimationFrame), or a polyfill if not available.
     * @param {function} callback The function to call when the next frame
     * happens.
     */
    requestAnimationFrame: function(cb) {
      return window._rAF(cb);
    },

    cancelAnimationFrame: function(requestId) {
      cancelAnimationFrame(requestId);
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#animationFrameThrottle
     * @alias ionic.animationFrameThrottle
     * @description
     * When given a callback, if that callback is called 100 times between
     * animation frames, adding Throttle will make it only run the last of
     * the 100 calls.
     *
     * @param {function} callback a function which will be throttled to
     * requestAnimationFrame
     * @returns {function} A function which will then call the passed in callback.
     * The passed in callback will receive the context the returned function is
     * called with.
     */
    animationFrameThrottle: function(cb) {
      var args, isQueued, context;
      return function() {
        args = arguments;
        context = this;
        if (!isQueued) {
          isQueued = true;
          ionic.requestAnimationFrame(function() {
            cb.apply(context, args);
            isQueued = false;
          });
        }
      };
    },

    contains: function(parentNode, otherNode) {
      var current = otherNode;
      while (current) {
        if (current === parentNode) return true;
        current = current.parentNode;
      }
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#getPositionInParent
     * @description
     * Find an element's scroll offset within its container.
     * @param {DOMElement} element The element to find the offset of.
     * @returns {object} A position object with the following properties:
     *   - `{number}` `left` The left offset of the element.
     *   - `{number}` `top` The top offset of the element.
     */
    getPositionInParent: function(el) {
      return {
        left: el.offsetLeft,
        top: el.offsetTop
      };
    },

    getOffsetTop: function(el) {
      var curtop = 0;
      if (el.offsetParent) {
        do {
          curtop += el.offsetTop;
          el = el.offsetParent;
        } while (el)
        return curtop;
      }
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#ready
     * @description
     * Call a function when the DOM is ready, or if it is already ready
     * call the function immediately.
     * @param {function} callback The function to be called.
     */
    ready: function(cb) {
      if (isDomReady) {
        ionic.requestAnimationFrame(cb);
      } else {
        readyCallbacks.push(cb);
      }
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#getTextBounds
     * @description
     * Get a rect representing the bounds of the given textNode.
     * @param {DOMElement} textNode The textNode to find the bounds of.
     * @returns {object} An object representing the bounds of the node. Properties:
     *   - `{number}` `left` The left position of the textNode.
     *   - `{number}` `right` The right position of the textNode.
     *   - `{number}` `top` The top position of the textNode.
     *   - `{number}` `bottom` The bottom position of the textNode.
     *   - `{number}` `width` The width of the textNode.
     *   - `{number}` `height` The height of the textNode.
     */
    getTextBounds: function(textNode) {
      if (document.createRange) {
        var range = document.createRange();
        range.selectNodeContents(textNode);
        if (range.getBoundingClientRect) {
          var rect = range.getBoundingClientRect();
          if (rect) {
            var sx = window.scrollX;
            var sy = window.scrollY;

            return {
              top: rect.top + sy,
              left: rect.left + sx,
              right: rect.left + sx + rect.width,
              bottom: rect.top + sy + rect.height,
              width: rect.width,
              height: rect.height
            };
          }
        }
      }
      return null;
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#getChildIndex
     * @description
     * Get the first index of a child node within the given element of the
     * specified type.
     * @param {DOMElement} element The element to find the index of.
     * @param {string} type The nodeName to match children of element against.
     * @returns {number} The index, or -1, of a child with nodeName matching type.
     */
    getChildIndex: function(element, type) {
      if (type) {
        var ch = element.parentNode.children;
        var c;
        for (var i = 0, k = 0, j = ch.length; i < j; i++) {
          c = ch[i];
          if (c.nodeName && c.nodeName.toLowerCase() == type) {
            if (c == element) {
              return k;
            }
            k++;
          }
        }
      }
      return Array.prototype.slice.call(element.parentNode.children).indexOf(element);
    },

    /**
     * @private
     */
    swapNodes: function(src, dest) {
      dest.parentNode.insertBefore(src, dest);
    },

    elementIsDescendant: function(el, parent, stopAt) {
      var current = el;
      do {
        if (current === parent) return true;
        current = current.parentNode;
      } while (current && current !== stopAt);
      return false;
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#getParentWithClass
     * @param {DOMElement} element
     * @param {string} className
     * @returns {DOMElement} The closest parent of element matching the
     * className, or null.
     */
    getParentWithClass: function(e, className, depth) {
      depth = depth || 10;
      while (e.parentNode && depth--) {
        if (e.parentNode.classList && e.parentNode.classList.contains(className)) {
          return e.parentNode;
        }
        e = e.parentNode;
      }
      return null;
    },
    /**
     * @ngdoc method
     * @name ionic.DomUtil#getParentOrSelfWithClass
     * @param {DOMElement} element
     * @param {string} className
     * @returns {DOMElement} The closest parent or self matching the
     * className, or null.
     */
    getParentOrSelfWithClass: function(e, className, depth) {
      depth = depth || 10;
      while (e && depth--) {
        if (e.classList && e.classList.contains(className)) {
          return e;
        }
        e = e.parentNode;
      }
      return null;
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#rectContains
     * @param {number} x
     * @param {number} y
     * @param {number} x1
     * @param {number} y1
     * @param {number} x2
     * @param {number} y2
     * @returns {boolean} Whether {x,y} fits within the rectangle defined by
     * {x1,y1,x2,y2}.
     */
    rectContains: function(x, y, x1, y1, x2, y2) {
      if (x < x1 || x > x2) return false;
      if (y < y1 || y > y2) return false;
      return true;
    },

    /**
     * @ngdoc method
     * @name ionic.DomUtil#blurAll
     * @description
     * Blurs any currently focused input element
     * @returns {DOMElement} The element blurred or null
     */
    blurAll: function() {
      if (document.activeElement && document.activeElement != document.body) {
        document.activeElement.blur();
        return document.activeElement;
      }
      return null;
    },

    cachedAttr: function(ele, key, value) {
      ele = ele && ele.length && ele[0] || ele;
      if (ele && ele.setAttribute) {
        var dataKey = '$attr-' + key;
        if (arguments.length > 2) {
          if (ele[dataKey] !== value) {
            ele.setAttribute(key, value);
            ele[dataKey] = value;
          }
        } else if (typeof ele[dataKey] == 'undefined') {
          ele[dataKey] = ele.getAttribute(key);
        }
        return ele[dataKey];
      }
    },

    cachedStyles: function(ele, styles) {
      ele = ele && ele.length && ele[0] || ele;
      if (ele && ele.style) {
        for (var prop in styles) {
          if (ele['$style-' + prop] !== styles[prop]) {
            ele.style[prop] = ele['$style-' + prop] = styles[prop];
          }
        }
      }
    }

  };

  //Shortcuts
  ionic.requestAnimationFrame = ionic.DomUtil.requestAnimationFrame;
  ionic.cancelAnimationFrame = ionic.DomUtil.cancelAnimationFrame;
  ionic.animationFrameThrottle = ionic.DomUtil.animationFrameThrottle;

})(window, document, ionic);

/**
 * ion-events.js
 *
 * Author: Max Lynch <max@drifty.com>
 *
 * Framework events handles various mobile browser events, and
 * detects special events like tap/swipe/etc. and emits them
 * as custom events that can be used in an app.
 *
 * Portions lovingly adapted from github.com/maker/ratchet and github.com/alexgibson/tap.js - thanks guys!
 */

(function(ionic) {

  // Custom event polyfill
  ionic.CustomEvent = (function() {
    if( typeof window.CustomEvent === 'function' ) return CustomEvent;

    var customEvent = function(event, params) {
      var evt;
      params = params || {
        bubbles: false,
        cancelable: false,
        detail: undefined
      };
      try {
        evt = document.createEvent("CustomEvent");
        evt.initCustomEvent(event, params.bubbles, params.cancelable, params.detail);
      } catch (error) {
        // fallback for browsers that don't support createEvent('CustomEvent')
        evt = document.createEvent("Event");
        for (var param in params) {
          evt[param] = params[param];
        }
        evt.initEvent(event, params.bubbles, params.cancelable);
      }
      return evt;
    };
    customEvent.prototype = window.Event.prototype;
    return customEvent;
  })();


  /**
   * @ngdoc utility
   * @name ionic.EventController
   * @module ionic
   */
  ionic.EventController = {
    VIRTUALIZED_EVENTS: ['tap', 'swipe', 'swiperight', 'swipeleft', 'drag', 'hold', 'release'],

    /**
     * @ngdoc method
     * @name ionic.EventController#trigger
     * @alias ionic.trigger
     * @param {string} eventType The event to trigger.
     * @param {object} data The data for the event. Hint: pass in
     * `{target: targetElement}`
     * @param {boolean=} bubbles Whether the event should bubble up the DOM.
     * @param {boolean=} cancelable Whether the event should be cancelable.
     */
    // Trigger a new event
    trigger: function(eventType, data, bubbles, cancelable) {
      var event = new ionic.CustomEvent(eventType, {
        detail: data,
        bubbles: !!bubbles,
        cancelable: !!cancelable
      });

      // Make sure to trigger the event on the given target, or dispatch it from
      // the window if we don't have an event target
      data && data.target && data.target.dispatchEvent && data.target.dispatchEvent(event) || window.dispatchEvent(event);
    },

    /**
     * @ngdoc method
     * @name ionic.EventController#on
     * @alias ionic.on
     * @description Listen to an event on an element.
     * @param {string} type The event to listen for.
     * @param {function} callback The listener to be called.
     * @param {DOMElement} element The element to listen for the event on.
     */
    on: function(type, callback, element) {
      var e = element || window;

      // Bind a gesture if it's a virtual event
      for(var i = 0, j = this.VIRTUALIZED_EVENTS.length; i < j; i++) {
        if(type == this.VIRTUALIZED_EVENTS[i]) {
          var gesture = new ionic.Gesture(element);
          gesture.on(type, callback);
          return gesture;
        }
      }

      // Otherwise bind a normal event
      e.addEventListener(type, callback);
    },

    /**
     * @ngdoc method
     * @name ionic.EventController#off
     * @alias ionic.off
     * @description Remove an event listener.
     * @param {string} type
     * @param {function} callback
     * @param {DOMElement} element
     */
    off: function(type, callback, element) {
      element.removeEventListener(type, callback);
    },

    /**
     * @ngdoc method
     * @name ionic.EventController#onGesture
     * @alias ionic.onGesture
     * @description Add an event listener for a gesture on an element.
     *
     * Available eventTypes (from [hammer.js](http://eightmedia.github.io/hammer.js/)):
     *
     * `hold`, `tap`, `doubletap`, `drag`, `dragstart`, `dragend`, `dragup`, `dragdown`, <br/>
     * `dragleft`, `dragright`, `swipe`, `swipeup`, `swipedown`, `swipeleft`, `swiperight`, <br/>
     * `transform`, `transformstart`, `transformend`, `rotate`, `pinch`, `pinchin`, `pinchout`, <br/>
     * `touch`, `release`
     *
     * @param {string} eventType The gesture event to listen for.
     * @param {function(e)} callback The function to call when the gesture
     * happens.
     * @param {DOMElement} element The angular element to listen for the event on.
     * @param {object} options object.
     * @returns {ionic.Gesture} The gesture object (use this to remove the gesture later on).
     */
    onGesture: function(type, callback, element, options) {
      var gesture = new ionic.Gesture(element, options);
      gesture.on(type, callback);
      return gesture;
    },

    /**
     * @ngdoc method
     * @name ionic.EventController#offGesture
     * @alias ionic.offGesture
     * @description Remove an event listener for a gesture created on an element.
     * @param {ionic.Gesture} gesture The gesture that should be removed.
     * @param {string} eventType The gesture event to remove the listener for.
     * @param {function(e)} callback The listener to remove.

     */
    offGesture: function(gesture, type, callback) {
      gesture && gesture.off(type, callback);
    },

    handlePopState: function() {}
  };


  // Map some convenient top-level functions for event handling
  ionic.on = function() { ionic.EventController.on.apply(ionic.EventController, arguments); };
  ionic.off = function() { ionic.EventController.off.apply(ionic.EventController, arguments); };
  ionic.trigger = ionic.EventController.trigger;//function() { ionic.EventController.trigger.apply(ionic.EventController.trigger, arguments); };
  ionic.onGesture = function() { return ionic.EventController.onGesture.apply(ionic.EventController.onGesture, arguments); };
  ionic.offGesture = function() { return ionic.EventController.offGesture.apply(ionic.EventController.offGesture, arguments); };

})(window.ionic);

/* eslint camelcase:0 */
/**
  * Simple gesture controllers with some common gestures that emit
  * gesture events.
  *
  * Ported from github.com/EightMedia/hammer.js Gestures - thanks!
  */
(function(ionic) {

  /**
   * ionic.Gestures
   * use this to create instances
   * @param   {HTMLElement}   element
   * @param   {Object}        options
   * @returns {ionic.Gestures.Instance}
   * @constructor
   */
  ionic.Gesture = function(element, options) {
    return new ionic.Gestures.Instance(element, options || {});
  };

  ionic.Gestures = {};

  // default settings
  ionic.Gestures.defaults = {
    // add css to the element to prevent the browser from doing
    // its native behavior. this doesnt prevent the scrolling,
    // but cancels the contextmenu, tap highlighting etc
    // set to false to disable this
    stop_browser_behavior: 'disable-user-behavior'
  };

  // detect touchevents
  ionic.Gestures.HAS_POINTEREVENTS = window.navigator.pointerEnabled || window.navigator.msPointerEnabled;
  ionic.Gestures.HAS_TOUCHEVENTS = ('ontouchstart' in window);

  // dont use mouseevents on mobile devices
  ionic.Gestures.MOBILE_REGEX = /mobile|tablet|ip(ad|hone|od)|android|silk/i;
  ionic.Gestures.NO_MOUSEEVENTS = ionic.Gestures.HAS_TOUCHEVENTS && window.navigator.userAgent.match(ionic.Gestures.MOBILE_REGEX);

  // eventtypes per touchevent (start, move, end)
  // are filled by ionic.Gestures.event.determineEventTypes on setup
  ionic.Gestures.EVENT_TYPES = {};

  // direction defines
  ionic.Gestures.DIRECTION_DOWN = 'down';
  ionic.Gestures.DIRECTION_LEFT = 'left';
  ionic.Gestures.DIRECTION_UP = 'up';
  ionic.Gestures.DIRECTION_RIGHT = 'right';

  // pointer type
  ionic.Gestures.POINTER_MOUSE = 'mouse';
  ionic.Gestures.POINTER_TOUCH = 'touch';
  ionic.Gestures.POINTER_PEN = 'pen';

  // touch event defines
  ionic.Gestures.EVENT_START = 'start';
  ionic.Gestures.EVENT_MOVE = 'move';
  ionic.Gestures.EVENT_END = 'end';

  // hammer document where the base events are added at
  ionic.Gestures.DOCUMENT = window.document;

  // plugins namespace
  ionic.Gestures.plugins = {};

  // if the window events are set...
  ionic.Gestures.READY = false;

  /**
   * setup events to detect gestures on the document
   */
  function setup() {
    if(ionic.Gestures.READY) {
      return;
    }

    // find what eventtypes we add listeners to
    ionic.Gestures.event.determineEventTypes();

    // Register all gestures inside ionic.Gestures.gestures
    for(var name in ionic.Gestures.gestures) {
      if(ionic.Gestures.gestures.hasOwnProperty(name)) {
        ionic.Gestures.detection.register(ionic.Gestures.gestures[name]);
      }
    }

    // Add touch events on the document
    ionic.Gestures.event.onTouch(ionic.Gestures.DOCUMENT, ionic.Gestures.EVENT_MOVE, ionic.Gestures.detection.detect);
    ionic.Gestures.event.onTouch(ionic.Gestures.DOCUMENT, ionic.Gestures.EVENT_END, ionic.Gestures.detection.detect);

    // ionic.Gestures is ready...!
    ionic.Gestures.READY = true;
  }

  /**
   * create new hammer instance
   * all methods should return the instance itself, so it is chainable.
   * @param   {HTMLElement}       element
   * @param   {Object}            [options={}]
   * @returns {ionic.Gestures.Instance}
   * @name Gesture.Instance
   * @constructor
   */
  ionic.Gestures.Instance = function(element, options) {
    var self = this;

    // A null element was passed into the instance, which means
    // whatever lookup was done to find this element failed to find it
    // so we can't listen for events on it.
    if(element === null) {
      void 0;
      return this;
    }

    // setup ionic.GesturesJS window events and register all gestures
    // this also sets up the default options
    setup();

    this.element = element;

    // start/stop detection option
    this.enabled = true;

    // merge options
    this.options = ionic.Gestures.utils.extend(
        ionic.Gestures.utils.extend({}, ionic.Gestures.defaults),
        options || {});

    // add some css to the element to prevent the browser from doing its native behavoir
    if(this.options.stop_browser_behavior) {
      ionic.Gestures.utils.stopDefaultBrowserBehavior(this.element, this.options.stop_browser_behavior);
    }

    // start detection on touchstart
    ionic.Gestures.event.onTouch(element, ionic.Gestures.EVENT_START, function(ev) {
      if(self.enabled) {
        ionic.Gestures.detection.startDetect(self, ev);
      }
    });

    // return instance
    return this;
  };


  ionic.Gestures.Instance.prototype = {
    /**
     * bind events to the instance
     * @param   {String}      gesture
     * @param   {Function}    handler
     * @returns {ionic.Gestures.Instance}
     */
    on: function onEvent(gesture, handler){
      var gestures = gesture.split(' ');
      for(var t = 0; t < gestures.length; t++) {
        this.element.addEventListener(gestures[t], handler, false);
      }
      return this;
    },


    /**
     * unbind events to the instance
     * @param   {String}      gesture
     * @param   {Function}    handler
     * @returns {ionic.Gestures.Instance}
     */
    off: function offEvent(gesture, handler){
      var gestures = gesture.split(' ');
      for(var t = 0; t < gestures.length; t++) {
        this.element.removeEventListener(gestures[t], handler, false);
      }
      return this;
    },


    /**
     * trigger gesture event
     * @param   {String}      gesture
     * @param   {Object}      eventData
     * @returns {ionic.Gestures.Instance}
     */
    trigger: function triggerEvent(gesture, eventData){
      // create DOM event
      var event = ionic.Gestures.DOCUMENT.createEvent('Event');
      event.initEvent(gesture, true, true);
      event.gesture = eventData;

      // trigger on the target if it is in the instance element,
      // this is for event delegation tricks
      var element = this.element;
      if(ionic.Gestures.utils.hasParent(eventData.target, element)) {
        element = eventData.target;
      }

      element.dispatchEvent(event);
      return this;
    },


    /**
     * enable of disable hammer.js detection
     * @param   {Boolean}   state
     * @returns {ionic.Gestures.Instance}
     */
    enable: function enable(state) {
      this.enabled = state;
      return this;
    }
  };

  /**
   * this holds the last move event,
   * used to fix empty touchend issue
   * see the onTouch event for an explanation
   * type {Object}
   */
  var last_move_event = null;


  /**
   * when the mouse is hold down, this is true
   * type {Boolean}
   */
  var enable_detect = false;


  /**
   * when touch events have been fired, this is true
   * type {Boolean}
   */
  var touch_triggered = false;


  ionic.Gestures.event = {
    /**
     * simple addEventListener
     * @param   {HTMLElement}   element
     * @param   {String}        type
     * @param   {Function}      handler
     */
    bindDom: function(element, type, handler) {
      var types = type.split(' ');
      for(var t = 0; t < types.length; t++) {
        element.addEventListener(types[t], handler, false);
      }
    },


    /**
     * touch events with mouse fallback
     * @param   {HTMLElement}   element
     * @param   {String}        eventType        like ionic.Gestures.EVENT_MOVE
     * @param   {Function}      handler
     */
    onTouch: function onTouch(element, eventType, handler) {
      var self = this;

      this.bindDom(element, ionic.Gestures.EVENT_TYPES[eventType], function bindDomOnTouch(ev) {
        var sourceEventType = ev.type.toLowerCase();

        // onmouseup, but when touchend has been fired we do nothing.
        // this is for touchdevices which also fire a mouseup on touchend
        if(sourceEventType.match(/mouse/) && touch_triggered) {
          return;
        }

        // mousebutton must be down or a touch event
        else if( sourceEventType.match(/touch/) ||   // touch events are always on screen
          sourceEventType.match(/pointerdown/) || // pointerevents touch
          (sourceEventType.match(/mouse/) && ev.which === 1)   // mouse is pressed
          ){
            enable_detect = true;
          }

        // mouse isn't pressed
        else if(sourceEventType.match(/mouse/) && ev.which !== 1) {
          enable_detect = false;
        }


        // we are in a touch event, set the touch triggered bool to true,
        // this for the conflicts that may occur on ios and android
        if(sourceEventType.match(/touch|pointer/)) {
          touch_triggered = true;
        }

        // count the total touches on the screen
        var count_touches = 0;

        // when touch has been triggered in this detection session
        // and we are now handling a mouse event, we stop that to prevent conflicts
        if(enable_detect) {
          // update pointerevent
          if(ionic.Gestures.HAS_POINTEREVENTS && eventType != ionic.Gestures.EVENT_END) {
            count_touches = ionic.Gestures.PointerEvent.updatePointer(eventType, ev);
          }
          // touch
          else if(sourceEventType.match(/touch/)) {
            count_touches = ev.touches.length;
          }
          // mouse
          else if(!touch_triggered) {
            count_touches = sourceEventType.match(/up/) ? 0 : 1;
          }

          // if we are in a end event, but when we remove one touch and
          // we still have enough, set eventType to move
          if(count_touches > 0 && eventType == ionic.Gestures.EVENT_END) {
            eventType = ionic.Gestures.EVENT_MOVE;
          }
          // no touches, force the end event
          else if(!count_touches) {
            eventType = ionic.Gestures.EVENT_END;
          }

          // store the last move event
          if(count_touches || last_move_event === null) {
            last_move_event = ev;
          }

          // trigger the handler
          handler.call(ionic.Gestures.detection, self.collectEventData(element, eventType, self.getTouchList(last_move_event, eventType), ev));

          // remove pointerevent from list
          if(ionic.Gestures.HAS_POINTEREVENTS && eventType == ionic.Gestures.EVENT_END) {
            count_touches = ionic.Gestures.PointerEvent.updatePointer(eventType, ev);
          }
        }

        //debug(sourceEventType +" "+ eventType);

        // on the end we reset everything
        if(!count_touches) {
          last_move_event = null;
          enable_detect = false;
          touch_triggered = false;
          ionic.Gestures.PointerEvent.reset();
        }
      });
    },


    /**
     * we have different events for each device/browser
     * determine what we need and set them in the ionic.Gestures.EVENT_TYPES constant
     */
    determineEventTypes: function determineEventTypes() {
      // determine the eventtype we want to set
      var types;

      // pointerEvents magic
      if(ionic.Gestures.HAS_POINTEREVENTS) {
        types = ionic.Gestures.PointerEvent.getEvents();
      }
      // on Android, iOS, blackberry, windows mobile we dont want any mouseevents
      else if(ionic.Gestures.NO_MOUSEEVENTS) {
        types = [
          'touchstart',
          'touchmove',
          'touchend touchcancel'];
      }
      // for non pointer events browsers and mixed browsers,
      // like chrome on windows8 touch laptop
      else {
        types = [
          'touchstart mousedown',
          'touchmove mousemove',
          'touchend touchcancel mouseup'];
      }

      ionic.Gestures.EVENT_TYPES[ionic.Gestures.EVENT_START] = types[0];
      ionic.Gestures.EVENT_TYPES[ionic.Gestures.EVENT_MOVE] = types[1];
      ionic.Gestures.EVENT_TYPES[ionic.Gestures.EVENT_END] = types[2];
    },


    /**
     * create touchlist depending on the event
     * @param   {Object}    ev
     * @param   {String}    eventType   used by the fakemultitouch plugin
     */
    getTouchList: function getTouchList(ev/*, eventType*/) {
      // get the fake pointerEvent touchlist
      if(ionic.Gestures.HAS_POINTEREVENTS) {
        return ionic.Gestures.PointerEvent.getTouchList();
      }
      // get the touchlist
      else if(ev.touches) {
        return ev.touches;
      }
      // make fake touchlist from mouse position
      else {
        ev.identifier = 1;
        return [ev];
      }
    },


    /**
     * collect event data for ionic.Gestures js
     * @param   {HTMLElement}   element
     * @param   {String}        eventType        like ionic.Gestures.EVENT_MOVE
     * @param   {Object}        eventData
     */
    collectEventData: function collectEventData(element, eventType, touches, ev) {

      // find out pointerType
      var pointerType = ionic.Gestures.POINTER_TOUCH;
      if(ev.type.match(/mouse/) || ionic.Gestures.PointerEvent.matchType(ionic.Gestures.POINTER_MOUSE, ev)) {
        pointerType = ionic.Gestures.POINTER_MOUSE;
      }

      return {
        center: ionic.Gestures.utils.getCenter(touches),
        timeStamp: new Date().getTime(),
        target: ev.target,
        touches: touches,
        eventType: eventType,
        pointerType: pointerType,
        srcEvent: ev,

        /**
         * prevent the browser default actions
         * mostly used to disable scrolling of the browser
         */
        preventDefault: function() {
          if(this.srcEvent.preventManipulation) {
            this.srcEvent.preventManipulation();
          }

          if(this.srcEvent.preventDefault) {
            // this.srcEvent.preventDefault();
          }
        },

        /**
         * stop bubbling the event up to its parents
         */
        stopPropagation: function() {
          this.srcEvent.stopPropagation();
        },

        /**
         * immediately stop gesture detection
         * might be useful after a swipe was detected
         * @return {*}
         */
        stopDetect: function() {
          return ionic.Gestures.detection.stopDetect();
        }
      };
    }
  };

  ionic.Gestures.PointerEvent = {
    /**
     * holds all pointers
     * type {Object}
     */
    pointers: {},

    /**
     * get a list of pointers
     * @returns {Array}     touchlist
     */
    getTouchList: function() {
      var self = this;
      var touchlist = [];

      // we can use forEach since pointerEvents only is in IE10
      Object.keys(self.pointers).sort().forEach(function(id) {
        touchlist.push(self.pointers[id]);
      });
      return touchlist;
    },

    /**
     * update the position of a pointer
     * @param   {String}   type             ionic.Gestures.EVENT_END
     * @param   {Object}   pointerEvent
     */
    updatePointer: function(type, pointerEvent) {
      if(type == ionic.Gestures.EVENT_END) {
        this.pointers = {};
      }
      else {
        pointerEvent.identifier = pointerEvent.pointerId;
        this.pointers[pointerEvent.pointerId] = pointerEvent;
      }

      return Object.keys(this.pointers).length;
    },

    /**
     * check if ev matches pointertype
     * @param   {String}        pointerType     ionic.Gestures.POINTER_MOUSE
     * @param   {PointerEvent}  ev
     */
    matchType: function(pointerType, ev) {
      if(!ev.pointerType) {
        return false;
      }

      var types = {};
      types[ionic.Gestures.POINTER_MOUSE] = (ev.pointerType == ev.MSPOINTER_TYPE_MOUSE || ev.pointerType == ionic.Gestures.POINTER_MOUSE);
      types[ionic.Gestures.POINTER_TOUCH] = (ev.pointerType == ev.MSPOINTER_TYPE_TOUCH || ev.pointerType == ionic.Gestures.POINTER_TOUCH);
      types[ionic.Gestures.POINTER_PEN] = (ev.pointerType == ev.MSPOINTER_TYPE_PEN || ev.pointerType == ionic.Gestures.POINTER_PEN);
      return types[pointerType];
    },


    /**
     * get events
     */
    getEvents: function() {
      return [
        'pointerdown MSPointerDown',
      'pointermove MSPointerMove',
      'pointerup pointercancel MSPointerUp MSPointerCancel'
        ];
    },

    /**
     * reset the list
     */
    reset: function() {
      this.pointers = {};
    }
  };


  ionic.Gestures.utils = {
    /**
     * extend method,
     * also used for cloning when dest is an empty object
     * @param   {Object}    dest
     * @param   {Object}    src
     * @param	{Boolean}	merge		do a merge
     * @returns {Object}    dest
     */
    extend: function extend(dest, src, merge) {
      for (var key in src) {
        if(dest[key] !== undefined && merge) {
          continue;
        }
        dest[key] = src[key];
      }
      return dest;
    },


    /**
     * find if a node is in the given parent
     * used for event delegation tricks
     * @param   {HTMLElement}   node
     * @param   {HTMLElement}   parent
     * @returns {boolean}       has_parent
     */
    hasParent: function(node, parent) {
      while(node){
        if(node == parent) {
          return true;
        }
        node = node.parentNode;
      }
      return false;
    },


    /**
     * get the center of all the touches
     * @param   {Array}     touches
     * @returns {Object}    center
     */
    getCenter: function getCenter(touches) {
      var valuesX = [], valuesY = [];

      for(var t = 0, len = touches.length; t < len; t++) {
        valuesX.push(touches[t].pageX);
        valuesY.push(touches[t].pageY);
      }

      return {
        pageX: ((Math.min.apply(Math, valuesX) + Math.max.apply(Math, valuesX)) / 2),
          pageY: ((Math.min.apply(Math, valuesY) + Math.max.apply(Math, valuesY)) / 2)
      };
    },


    /**
     * calculate the velocity between two points
     * @param   {Number}    delta_time
     * @param   {Number}    delta_x
     * @param   {Number}    delta_y
     * @returns {Object}    velocity
     */
    getVelocity: function getVelocity(delta_time, delta_x, delta_y) {
      return {
        x: Math.abs(delta_x / delta_time) || 0,
        y: Math.abs(delta_y / delta_time) || 0
      };
    },


    /**
     * calculate the angle between two coordinates
     * @param   {Touch}     touch1
     * @param   {Touch}     touch2
     * @returns {Number}    angle
     */
    getAngle: function getAngle(touch1, touch2) {
      var y = touch2.pageY - touch1.pageY,
      x = touch2.pageX - touch1.pageX;
      return Math.atan2(y, x) * 180 / Math.PI;
    },


    /**
     * angle to direction define
     * @param   {Touch}     touch1
     * @param   {Touch}     touch2
     * @returns {String}    direction constant, like ionic.Gestures.DIRECTION_LEFT
     */
    getDirection: function getDirection(touch1, touch2) {
      var x = Math.abs(touch1.pageX - touch2.pageX),
      y = Math.abs(touch1.pageY - touch2.pageY);

      if(x >= y) {
        return touch1.pageX - touch2.pageX > 0 ? ionic.Gestures.DIRECTION_LEFT : ionic.Gestures.DIRECTION_RIGHT;
      }
      else {
        return touch1.pageY - touch2.pageY > 0 ? ionic.Gestures.DIRECTION_UP : ionic.Gestures.DIRECTION_DOWN;
      }
    },


    /**
     * calculate the distance between two touches
     * @param   {Touch}     touch1
     * @param   {Touch}     touch2
     * @returns {Number}    distance
     */
    getDistance: function getDistance(touch1, touch2) {
      var x = touch2.pageX - touch1.pageX,
      y = touch2.pageY - touch1.pageY;
      return Math.sqrt((x * x) + (y * y));
    },


    /**
     * calculate the scale factor between two touchLists (fingers)
     * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
     * @param   {Array}     start
     * @param   {Array}     end
     * @returns {Number}    scale
     */
    getScale: function getScale(start, end) {
      // need two fingers...
      if(start.length >= 2 && end.length >= 2) {
        return this.getDistance(end[0], end[1]) /
          this.getDistance(start[0], start[1]);
      }
      return 1;
    },


    /**
     * calculate the rotation degrees between two touchLists (fingers)
     * @param   {Array}     start
     * @param   {Array}     end
     * @returns {Number}    rotation
     */
    getRotation: function getRotation(start, end) {
      // need two fingers
      if(start.length >= 2 && end.length >= 2) {
        return this.getAngle(end[1], end[0]) -
          this.getAngle(start[1], start[0]);
      }
      return 0;
    },


    /**
     * boolean if the direction is vertical
     * @param    {String}    direction
     * @returns  {Boolean}   is_vertical
     */
    isVertical: function isVertical(direction) {
      return (direction == ionic.Gestures.DIRECTION_UP || direction == ionic.Gestures.DIRECTION_DOWN);
    },


    /**
     * stop browser default behavior with css class
     * @param   {HtmlElement}   element
     * @param   {Object}        css_class
     */
    stopDefaultBrowserBehavior: function stopDefaultBrowserBehavior(element, css_class) {
      // changed from making many style changes to just adding a preset classname
      // less DOM manipulations, less code, and easier to control in the CSS side of things
      // hammer.js doesn't come with CSS, but ionic does, which is why we prefer this method
      if(element && element.classList) {
        element.classList.add(css_class);
        element.onselectstart = function() {
          return false;
        };
      }
    }
  };


  ionic.Gestures.detection = {
    // contains all registred ionic.Gestures.gestures in the correct order
    gestures: [],

    // data of the current ionic.Gestures.gesture detection session
    current: null,

    // the previous ionic.Gestures.gesture session data
    // is a full clone of the previous gesture.current object
    previous: null,

    // when this becomes true, no gestures are fired
    stopped: false,


    /**
     * start ionic.Gestures.gesture detection
     * @param   {ionic.Gestures.Instance}   inst
     * @param   {Object}            eventData
     */
    startDetect: function startDetect(inst, eventData) {
      // already busy with a ionic.Gestures.gesture detection on an element
      if(this.current) {
        return;
      }

      this.stopped = false;

      this.current = {
        inst: inst, // reference to ionic.GesturesInstance we're working for
        startEvent: ionic.Gestures.utils.extend({}, eventData), // start eventData for distances, timing etc
        lastEvent: false, // last eventData
        name: '' // current gesture we're in/detected, can be 'tap', 'hold' etc
      };

      this.detect(eventData);
    },


    /**
     * ionic.Gestures.gesture detection
     * @param   {Object}    eventData
     */
    detect: function detect(eventData) {
      if(!this.current || this.stopped) {
        return null;
      }

      // extend event data with calculations about scale, distance etc
      eventData = this.extendEventData(eventData);

      // instance options
      var inst_options = this.current.inst.options;

      // call ionic.Gestures.gesture handlers
      for(var g = 0, len = this.gestures.length; g < len; g++) {
        var gesture = this.gestures[g];

        // only when the instance options have enabled this gesture
        if(!this.stopped && inst_options[gesture.name] !== false) {
          // if a handler returns false, we stop with the detection
          if(gesture.handler.call(gesture, eventData, this.current.inst) === false) {
            this.stopDetect();
            break;
          }
        }
      }

      // store as previous event event
      if(this.current) {
        this.current.lastEvent = eventData;
      }

      // endevent, but not the last touch, so dont stop
      if(eventData.eventType == ionic.Gestures.EVENT_END && !eventData.touches.length - 1) {
        this.stopDetect();
      }

      return eventData;
    },


    /**
     * clear the ionic.Gestures.gesture vars
     * this is called on endDetect, but can also be used when a final ionic.Gestures.gesture has been detected
     * to stop other ionic.Gestures.gestures from being fired
     */
    stopDetect: function stopDetect() {
      // clone current data to the store as the previous gesture
      // used for the double tap gesture, since this is an other gesture detect session
      this.previous = ionic.Gestures.utils.extend({}, this.current);

      // reset the current
      this.current = null;

      // stopped!
      this.stopped = true;
    },


    /**
     * extend eventData for ionic.Gestures.gestures
     * @param   {Object}   ev
     * @returns {Object}   ev
     */
    extendEventData: function extendEventData(ev) {
      var startEv = this.current.startEvent;

      // if the touches change, set the new touches over the startEvent touches
      // this because touchevents don't have all the touches on touchstart, or the
      // user must place his fingers at the EXACT same time on the screen, which is not realistic
      // but, sometimes it happens that both fingers are touching at the EXACT same time
      if(startEv && (ev.touches.length != startEv.touches.length || ev.touches === startEv.touches)) {
        // extend 1 level deep to get the touchlist with the touch objects
        startEv.touches = [];
        for(var i = 0, len = ev.touches.length; i < len; i++) {
          startEv.touches.push(ionic.Gestures.utils.extend({}, ev.touches[i]));
        }
      }

      var delta_time = ev.timeStamp - startEv.timeStamp,
          delta_x = ev.center.pageX - startEv.center.pageX,
          delta_y = ev.center.pageY - startEv.center.pageY,
          velocity = ionic.Gestures.utils.getVelocity(delta_time, delta_x, delta_y);

      ionic.Gestures.utils.extend(ev, {
        deltaTime: delta_time,
        deltaX: delta_x,
        deltaY: delta_y,

        velocityX: velocity.x,
        velocityY: velocity.y,

        distance: ionic.Gestures.utils.getDistance(startEv.center, ev.center),
        angle: ionic.Gestures.utils.getAngle(startEv.center, ev.center),
        direction: ionic.Gestures.utils.getDirection(startEv.center, ev.center),

        scale: ionic.Gestures.utils.getScale(startEv.touches, ev.touches),
        rotation: ionic.Gestures.utils.getRotation(startEv.touches, ev.touches),

        startEvent: startEv
      });

      return ev;
    },


    /**
     * register new gesture
     * @param   {Object}    gesture object, see gestures.js for documentation
     * @returns {Array}     gestures
     */
    register: function register(gesture) {
      // add an enable gesture options if there is no given
      var options = gesture.defaults || {};
      if(options[gesture.name] === undefined) {
        options[gesture.name] = true;
      }

      // extend ionic.Gestures default options with the ionic.Gestures.gesture options
      ionic.Gestures.utils.extend(ionic.Gestures.defaults, options, true);

      // set its index
      gesture.index = gesture.index || 1000;

      // add ionic.Gestures.gesture to the list
      this.gestures.push(gesture);

      // sort the list by index
      this.gestures.sort(function(a, b) {
        if (a.index < b.index) {
          return -1;
        }
        if (a.index > b.index) {
          return 1;
        }
        return 0;
      });

      return this.gestures;
    }
  };


  ionic.Gestures.gestures = ionic.Gestures.gestures || {};

  /**
   * Custom gestures
   * ==============================
   *
   * Gesture object
   * --------------------
   * The object structure of a gesture:
   *
   * { name: 'mygesture',
   *   index: 1337,
   *   defaults: {
   *     mygesture_option: true
   *   }
   *   handler: function(type, ev, inst) {
   *     // trigger gesture event
   *     inst.trigger(this.name, ev);
   *   }
   * }

   * @param   {String}    name
   * this should be the name of the gesture, lowercase
   * it is also being used to disable/enable the gesture per instance config.
   *
   * @param   {Number}    [index=1000]
   * the index of the gesture, where it is going to be in the stack of gestures detection
   * like when you build an gesture that depends on the drag gesture, it is a good
   * idea to place it after the index of the drag gesture.
   *
   * @param   {Object}    [defaults={}]
   * the default settings of the gesture. these are added to the instance settings,
   * and can be overruled per instance. you can also add the name of the gesture,
   * but this is also added by default (and set to true).
   *
   * @param   {Function}  handler
   * this handles the gesture detection of your custom gesture and receives the
   * following arguments:
   *
   *      @param  {Object}    eventData
   *      event data containing the following properties:
   *          timeStamp   {Number}        time the event occurred
   *          target      {HTMLElement}   target element
   *          touches     {Array}         touches (fingers, pointers, mouse) on the screen
   *          pointerType {String}        kind of pointer that was used. matches ionic.Gestures.POINTER_MOUSE|TOUCH
   *          center      {Object}        center position of the touches. contains pageX and pageY
   *          deltaTime   {Number}        the total time of the touches in the screen
   *          deltaX      {Number}        the delta on x axis we haved moved
   *          deltaY      {Number}        the delta on y axis we haved moved
   *          velocityX   {Number}        the velocity on the x
   *          velocityY   {Number}        the velocity on y
   *          angle       {Number}        the angle we are moving
   *          direction   {String}        the direction we are moving. matches ionic.Gestures.DIRECTION_UP|DOWN|LEFT|RIGHT
   *          distance    {Number}        the distance we haved moved
   *          scale       {Number}        scaling of the touches, needs 2 touches
   *          rotation    {Number}        rotation of the touches, needs 2 touches *
   *          eventType   {String}        matches ionic.Gestures.EVENT_START|MOVE|END
   *          srcEvent    {Object}        the source event, like TouchStart or MouseDown *
   *          startEvent  {Object}        contains the same properties as above,
   *                                      but from the first touch. this is used to calculate
   *                                      distances, deltaTime, scaling etc
   *
   *      @param  {ionic.Gestures.Instance}    inst
   *      the instance we are doing the detection for. you can get the options from
   *      the inst.options object and trigger the gesture event by calling inst.trigger
   *
   *
   * Handle gestures
   * --------------------
   * inside the handler you can get/set ionic.Gestures.detectionic.current. This is the current
   * detection sessionic. It has the following properties
   *      @param  {String}    name
   *      contains the name of the gesture we have detected. it has not a real function,
   *      only to check in other gestures if something is detected.
   *      like in the drag gesture we set it to 'drag' and in the swipe gesture we can
   *      check if the current gesture is 'drag' by accessing ionic.Gestures.detectionic.current.name
   *
   *      readonly
   *      @param  {ionic.Gestures.Instance}    inst
   *      the instance we do the detection for
   *
   *      readonly
   *      @param  {Object}    startEvent
   *      contains the properties of the first gesture detection in this sessionic.
   *      Used for calculations about timing, distance, etc.
   *
   *      readonly
   *      @param  {Object}    lastEvent
   *      contains all the properties of the last gesture detect in this sessionic.
   *
   * after the gesture detection session has been completed (user has released the screen)
   * the ionic.Gestures.detectionic.current object is copied into ionic.Gestures.detectionic.previous,
   * this is usefull for gestures like doubletap, where you need to know if the
   * previous gesture was a tap
   *
   * options that have been set by the instance can be received by calling inst.options
   *
   * You can trigger a gesture event by calling inst.trigger("mygesture", event).
   * The first param is the name of your gesture, the second the event argument
   *
   *
   * Register gestures
   * --------------------
   * When an gesture is added to the ionic.Gestures.gestures object, it is auto registered
   * at the setup of the first ionic.Gestures instance. You can also call ionic.Gestures.detectionic.register
   * manually and pass your gesture object as a param
   *
   */

  /**
   * Hold
   * Touch stays at the same place for x time
   * events  hold
   */
  ionic.Gestures.gestures.Hold = {
    name: 'hold',
    index: 10,
    defaults: {
      hold_timeout: 500,
      hold_threshold: 9
    },
    timer: null,
    handler: function holdGesture(ev, inst) {
      switch(ev.eventType) {
        case ionic.Gestures.EVENT_START:
          // clear any running timers
          clearTimeout(this.timer);

          // set the gesture so we can check in the timeout if it still is
          ionic.Gestures.detection.current.name = this.name;

          // set timer and if after the timeout it still is hold,
          // we trigger the hold event
          this.timer = setTimeout(function() {
            if(ionic.Gestures.detection.current.name == 'hold') {
              ionic.tap.cancelClick();
              inst.trigger('hold', ev);
            }
          }, inst.options.hold_timeout);
          break;

          // when you move or end we clear the timer
        case ionic.Gestures.EVENT_MOVE:
          if(ev.distance > inst.options.hold_threshold) {
            clearTimeout(this.timer);
          }
          break;

        case ionic.Gestures.EVENT_END:
          clearTimeout(this.timer);
          break;
      }
    }
  };


  /**
   * Tap/DoubleTap
   * Quick touch at a place or double at the same place
   * events  tap, doubletap
   */
  ionic.Gestures.gestures.Tap = {
    name: 'tap',
    index: 100,
    defaults: {
      tap_max_touchtime: 250,
      tap_max_distance: 10,
      tap_always: true,
      doubletap_distance: 20,
      doubletap_interval: 300
    },
    handler: function tapGesture(ev, inst) {
      if(ev.eventType == ionic.Gestures.EVENT_END && ev.srcEvent.type != 'touchcancel') {
        // previous gesture, for the double tap since these are two different gesture detections
        var prev = ionic.Gestures.detection.previous,
        did_doubletap = false;

        // when the touchtime is higher then the max touch time
        // or when the moving distance is too much
        if(ev.deltaTime > inst.options.tap_max_touchtime ||
            ev.distance > inst.options.tap_max_distance) {
              return;
            }

        // check if double tap
        if(prev && prev.name == 'tap' &&
            (ev.timeStamp - prev.lastEvent.timeStamp) < inst.options.doubletap_interval &&
            ev.distance < inst.options.doubletap_distance) {
              inst.trigger('doubletap', ev);
              did_doubletap = true;
            }

        // do a single tap
        if(!did_doubletap || inst.options.tap_always) {
          ionic.Gestures.detection.current.name = 'tap';
          inst.trigger('tap', ev);
        }
      }
    }
  };


  /**
   * Swipe
   * triggers swipe events when the end velocity is above the threshold
   * events  swipe, swipeleft, swiperight, swipeup, swipedown
   */
  ionic.Gestures.gestures.Swipe = {
    name: 'swipe',
    index: 40,
    defaults: {
      // set 0 for unlimited, but this can conflict with transform
      swipe_max_touches: 1,
      swipe_velocity: 0.4
    },
    handler: function swipeGesture(ev, inst) {
      if(ev.eventType == ionic.Gestures.EVENT_END) {
        // max touches
        if(inst.options.swipe_max_touches > 0 &&
            ev.touches.length > inst.options.swipe_max_touches) {
              return;
            }

        // when the distance we moved is too small we skip this gesture
        // or we can be already in dragging
        if(ev.velocityX > inst.options.swipe_velocity ||
            ev.velocityY > inst.options.swipe_velocity) {
              // trigger swipe events
              inst.trigger(this.name, ev);
              inst.trigger(this.name + ev.direction, ev);
            }
      }
    }
  };


  /**
   * Drag
   * Move with x fingers (default 1) around on the page. Blocking the scrolling when
   * moving left and right is a good practice. When all the drag events are blocking
   * you disable scrolling on that area.
   * events  drag, drapleft, dragright, dragup, dragdown
   */
  ionic.Gestures.gestures.Drag = {
    name: 'drag',
    index: 50,
    defaults: {
      drag_min_distance: 10,
      // Set correct_for_drag_min_distance to true to make the starting point of the drag
      // be calculated from where the drag was triggered, not from where the touch started.
      // Useful to avoid a jerk-starting drag, which can make fine-adjustments
      // through dragging difficult, and be visually unappealing.
      correct_for_drag_min_distance: true,
      // set 0 for unlimited, but this can conflict with transform
      drag_max_touches: 1,
      // prevent default browser behavior when dragging occurs
      // be careful with it, it makes the element a blocking element
      // when you are using the drag gesture, it is a good practice to set this true
      drag_block_horizontal: true,
      drag_block_vertical: true,
      // drag_lock_to_axis keeps the drag gesture on the axis that it started on,
      // It disallows vertical directions if the initial direction was horizontal, and vice versa.
      drag_lock_to_axis: false,
      // drag lock only kicks in when distance > drag_lock_min_distance
      // This way, locking occurs only when the distance has become large enough to reliably determine the direction
      drag_lock_min_distance: 25,
      // prevent default if the gesture is going the given direction
      prevent_default_directions: []
    },
    triggered: false,
    handler: function dragGesture(ev, inst) {
      if (ev.srcEvent.type == 'touchstart' || ev.srcEvent.type == 'touchend') {
        this.preventedFirstMove = false;

      } else if (!this.preventedFirstMove && ev.srcEvent.type == 'touchmove') {
        // Prevent gestures that are not intended for this event handler from firing subsequent times
        if (inst.options.prevent_default_directions.length > 0
            && inst.options.prevent_default_directions.indexOf(ev.direction) != -1) {
          ev.srcEvent.preventDefault();
        }
        this.preventedFirstMove = true;
      }

      // current gesture isnt drag, but dragged is true
      // this means an other gesture is busy. now call dragend
      if(ionic.Gestures.detection.current.name != this.name && this.triggered) {
        inst.trigger(this.name + 'end', ev);
        this.triggered = false;
        return;
      }

      // max touches
      if(inst.options.drag_max_touches > 0 &&
          ev.touches.length > inst.options.drag_max_touches) {
            return;
          }

      switch(ev.eventType) {
        case ionic.Gestures.EVENT_START:
          this.triggered = false;
          break;

        case ionic.Gestures.EVENT_MOVE:
          // when the distance we moved is too small we skip this gesture
          // or we can be already in dragging
          if(ev.distance < inst.options.drag_min_distance &&
              ionic.Gestures.detection.current.name != this.name) {
                return;
              }

          // we are dragging!
          if(ionic.Gestures.detection.current.name != this.name) {
            ionic.Gestures.detection.current.name = this.name;
            if (inst.options.correct_for_drag_min_distance) {
              // When a drag is triggered, set the event center to drag_min_distance pixels from the original event center.
              // Without this correction, the dragged distance would jumpstart at drag_min_distance pixels instead of at 0.
              // It might be useful to save the original start point somewhere
              var factor = Math.abs(inst.options.drag_min_distance / ev.distance);
              ionic.Gestures.detection.current.startEvent.center.pageX += ev.deltaX * factor;
              ionic.Gestures.detection.current.startEvent.center.pageY += ev.deltaY * factor;

              // recalculate event data using new start point
              ev = ionic.Gestures.detection.extendEventData(ev);
            }
          }

          // lock drag to axis?
          if(ionic.Gestures.detection.current.lastEvent.drag_locked_to_axis || (inst.options.drag_lock_to_axis && inst.options.drag_lock_min_distance <= ev.distance)) {
            ev.drag_locked_to_axis = true;
          }
          var last_direction = ionic.Gestures.detection.current.lastEvent.direction;
          if(ev.drag_locked_to_axis && last_direction !== ev.direction) {
            // keep direction on the axis that the drag gesture started on
            if(ionic.Gestures.utils.isVertical(last_direction)) {
              ev.direction = (ev.deltaY < 0) ? ionic.Gestures.DIRECTION_UP : ionic.Gestures.DIRECTION_DOWN;
            }
            else {
              ev.direction = (ev.deltaX < 0) ? ionic.Gestures.DIRECTION_LEFT : ionic.Gestures.DIRECTION_RIGHT;
            }
          }

          // first time, trigger dragstart event
          if(!this.triggered) {
            inst.trigger(this.name + 'start', ev);
            this.triggered = true;
          }

          // trigger normal event
          inst.trigger(this.name, ev);

          // direction event, like dragdown
          inst.trigger(this.name + ev.direction, ev);

          // block the browser events
          if( (inst.options.drag_block_vertical && ionic.Gestures.utils.isVertical(ev.direction)) ||
              (inst.options.drag_block_horizontal && !ionic.Gestures.utils.isVertical(ev.direction))) {
                ev.preventDefault();
              }
          break;

        case ionic.Gestures.EVENT_END:
          // trigger dragend
          if(this.triggered) {
            inst.trigger(this.name + 'end', ev);
          }

          this.triggered = false;
          break;
      }
    }
  };


  /**
   * Transform
   * User want to scale or rotate with 2 fingers
   * events  transform, pinch, pinchin, pinchout, rotate
   */
  ionic.Gestures.gestures.Transform = {
    name: 'transform',
    index: 45,
    defaults: {
      // factor, no scale is 1, zoomin is to 0 and zoomout until higher then 1
      transform_min_scale: 0.01,
      // rotation in degrees
      transform_min_rotation: 1,
      // prevent default browser behavior when two touches are on the screen
      // but it makes the element a blocking element
      // when you are using the transform gesture, it is a good practice to set this true
      transform_always_block: false
    },
    triggered: false,
    handler: function transformGesture(ev, inst) {
      // current gesture isnt drag, but dragged is true
      // this means an other gesture is busy. now call dragend
      if(ionic.Gestures.detection.current.name != this.name && this.triggered) {
        inst.trigger(this.name + 'end', ev);
        this.triggered = false;
        return;
      }

      // atleast multitouch
      if(ev.touches.length < 2) {
        return;
      }

      // prevent default when two fingers are on the screen
      if(inst.options.transform_always_block) {
        ev.preventDefault();
      }

      switch(ev.eventType) {
        case ionic.Gestures.EVENT_START:
          this.triggered = false;
          break;

        case ionic.Gestures.EVENT_MOVE:
          var scale_threshold = Math.abs(1 - ev.scale);
          var rotation_threshold = Math.abs(ev.rotation);

          // when the distance we moved is too small we skip this gesture
          // or we can be already in dragging
          if(scale_threshold < inst.options.transform_min_scale &&
              rotation_threshold < inst.options.transform_min_rotation) {
                return;
              }

          // we are transforming!
          ionic.Gestures.detection.current.name = this.name;

          // first time, trigger dragstart event
          if(!this.triggered) {
            inst.trigger(this.name + 'start', ev);
            this.triggered = true;
          }

          inst.trigger(this.name, ev); // basic transform event

          // trigger rotate event
          if(rotation_threshold > inst.options.transform_min_rotation) {
            inst.trigger('rotate', ev);
          }

          // trigger pinch event
          if(scale_threshold > inst.options.transform_min_scale) {
            inst.trigger('pinch', ev);
            inst.trigger('pinch' + ((ev.scale < 1) ? 'in' : 'out'), ev);
          }
          break;

        case ionic.Gestures.EVENT_END:
          // trigger dragend
          if(this.triggered) {
            inst.trigger(this.name + 'end', ev);
          }

          this.triggered = false;
          break;
      }
    }
  };


  /**
   * Touch
   * Called as first, tells the user has touched the screen
   * events  touch
   */
  ionic.Gestures.gestures.Touch = {
    name: 'touch',
    index: -Infinity,
    defaults: {
      // call preventDefault at touchstart, and makes the element blocking by
      // disabling the scrolling of the page, but it improves gestures like
      // transforming and dragging.
      // be careful with using this, it can be very annoying for users to be stuck
      // on the page
      prevent_default: false,

      // disable mouse events, so only touch (or pen!) input triggers events
      prevent_mouseevents: false
    },
    handler: function touchGesture(ev, inst) {
      if(inst.options.prevent_mouseevents && ev.pointerType == ionic.Gestures.POINTER_MOUSE) {
        ev.stopDetect();
        return;
      }

      if(inst.options.prevent_default) {
        ev.preventDefault();
      }

      if(ev.eventType == ionic.Gestures.EVENT_START) {
        inst.trigger(this.name, ev);
      }
    }
  };


  /**
   * Release
   * Called as last, tells the user has released the screen
   * events  release
   */
  ionic.Gestures.gestures.Release = {
    name: 'release',
    index: Infinity,
    handler: function releaseGesture(ev, inst) {
      if(ev.eventType == ionic.Gestures.EVENT_END) {
        inst.trigger(this.name, ev);
      }
    }
  };
})(window.ionic);

(function(window, document, ionic) {

  function getParameterByName(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
    results = regex.exec(location.search);
    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
  }

  var IOS = 'ios';
  var ANDROID = 'android';
  var WINDOWS_PHONE = 'windowsphone';
  var EDGE = 'edge';
  var CROSSWALK = 'crosswalk';
  var requestAnimationFrame = ionic.requestAnimationFrame;

  /**
   * @ngdoc utility
   * @name ionic.Platform
   * @module ionic
   * @description
   * A set of utility methods that can be used to retrieve the device ready state and
   * various other information such as what kind of platform the app is currently installed on.
   *
   * @usage
   * ```js
   * angular.module('PlatformApp', ['ionic'])
   * .controller('PlatformCtrl', function($scope) {
   *
   *   ionic.Platform.ready(function(){
   *     // will execute when device is ready, or immediately if the device is already ready.
   *   });
   *
   *   var deviceInformation = ionic.Platform.device();
   *
   *   var isWebView = ionic.Platform.isWebView();
   *   var isIPad = ionic.Platform.isIPad();
   *   var isIOS = ionic.Platform.isIOS();
   *   var isAndroid = ionic.Platform.isAndroid();
   *   var isWindowsPhone = ionic.Platform.isWindowsPhone();
   *
   *   var currentPlatform = ionic.Platform.platform();
   *   var currentPlatformVersion = ionic.Platform.version();
   *
   *   ionic.Platform.exitApp(); // stops the app
   * });
   * ```
   */
  var self = ionic.Platform = {

    // Put navigator on platform so it can be mocked and set
    // the browser does not allow window.navigator to be set
    navigator: window.navigator,

    /**
     * @ngdoc property
     * @name ionic.Platform#isReady
     * @returns {boolean} Whether the device is ready.
     */
    isReady: false,
    /**
     * @ngdoc property
     * @name ionic.Platform#isFullScreen
     * @returns {boolean} Whether the device is fullscreen.
     */
    isFullScreen: false,
    /**
     * @ngdoc property
     * @name ionic.Platform#platforms
     * @returns {Array(string)} An array of all platforms found.
     */
    platforms: null,
    /**
     * @ngdoc property
     * @name ionic.Platform#grade
     * @returns {string} What grade the current platform is.
     */
    grade: null,
    /**
     * @ngdoc property
     * @name ionic.Platform#ua
     * @returns {string} What User Agent is.
     */
    ua: navigator.userAgent,

    /**
     * @ngdoc method
     * @name ionic.Platform#ready
     * @description
     * Trigger a callback once the device is ready, or immediately
     * if the device is already ready. This method can be run from
     * anywhere and does not need to be wrapped by any additonal methods.
     * When the app is within a WebView (Cordova), it'll fire
     * the callback once the device is ready. If the app is within
     * a web browser, it'll fire the callback after `window.load`.
     * Please remember that Cordova features (Camera, FileSystem, etc) still
     * will not work in a web browser.
     * @param {function} callback The function to call.
     */
    ready: function(cb) {
      // run through tasks to complete now that the device is ready
      if (self.isReady) {
        cb();
      } else {
        // the platform isn't ready yet, add it to this array
        // which will be called once the platform is ready
        readyCallbacks.push(cb);
      }
    },

    /**
     * @private
     */
    detect: function() {
      self._checkPlatforms();

      requestAnimationFrame(function() {
        // only add to the body class if we got platform info
        for (var i = 0; i < self.platforms.length; i++) {
          document.body.classList.add('platform-' + self.platforms[i]);
        }
      });
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#setGrade
     * @description Set the grade of the device: 'a', 'b', or 'c'. 'a' is the best
     * (most css features enabled), 'c' is the worst.  By default, sets the grade
     * depending on the current device.
     * @param {string} grade The new grade to set.
     */
    setGrade: function(grade) {
      var oldGrade = self.grade;
      self.grade = grade;
      requestAnimationFrame(function() {
        if (oldGrade) {
          document.body.classList.remove('grade-' + oldGrade);
        }
        document.body.classList.add('grade-' + grade);
      });
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#device
     * @description Return the current device (given by cordova).
     * @returns {object} The device object.
     */
    device: function() {
      return window.device || {};
    },

    _checkPlatforms: function() {
      self.platforms = [];
      var grade = 'a';

      if (self.isWebView()) {
        self.platforms.push('webview');
        if (!(!window.cordova && !window.PhoneGap && !window.phonegap)) {
          self.platforms.push('cordova');
        } else if (typeof window.forge === 'object') {
          self.platforms.push('trigger');
        }
      } else {
        self.platforms.push('browser');
      }
      if (self.isIPad()) self.platforms.push('ipad');

      var platform = self.platform();
      if (platform) {
        self.platforms.push(platform);

        var version = self.version();
        if (version) {
          var v = version.toString();
          if (v.indexOf('.') > 0) {
            v = v.replace('.', '_');
          } else {
            v += '_0';
          }
          self.platforms.push(platform + v.split('_')[0]);
          self.platforms.push(platform + v);

          if (self.isAndroid() && version < 4.4) {
            grade = (version < 4 ? 'c' : 'b');
          } else if (self.isWindowsPhone()) {
            grade = 'b';
          }
        }
      }

      self.setGrade(grade);
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#isWebView
     * @returns {boolean} Check if we are running within a WebView (such as Cordova).
     */
    isWebView: function() {
      return !(!window.cordova && !window.PhoneGap && !window.phonegap && window.forge !== 'object');
    },
    /**
     * @ngdoc method
     * @name ionic.Platform#isIPad
     * @returns {boolean} Whether we are running on iPad.
     */
    isIPad: function() {
      if (/iPad/i.test(self.navigator.platform)) {
        return true;
      }
      return /iPad/i.test(self.ua);
    },
    /**
     * @ngdoc method
     * @name ionic.Platform#isIOS
     * @returns {boolean} Whether we are running on iOS.
     */
    isIOS: function() {
      return self.is(IOS);
    },
    /**
     * @ngdoc method
     * @name ionic.Platform#isAndroid
     * @returns {boolean} Whether we are running on Android.
     */
    isAndroid: function() {
      return self.is(ANDROID);
    },
    /**
     * @ngdoc method
     * @name ionic.Platform#isWindowsPhone
     * @returns {boolean} Whether we are running on Windows Phone.
     */
    isWindowsPhone: function() {
      return self.is(WINDOWS_PHONE);
    },
    /**
     * @ngdoc method
     * @name ionic.Platform#isEdge
     * @returns {boolean} Whether we are running on MS Edge/Windows 10 (inc. Phone)
     */
    isEdge: function() {
      return self.is(EDGE);
    },

    isCrosswalk: function() {
      return self.is(CROSSWALK);
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#platform
     * @returns {string} The name of the current platform.
     */
    platform: function() {
      // singleton to get the platform name
      if (platformName === null) self.setPlatform(self.device().platform);
      return platformName;
    },

    /**
     * @private
     */
    setPlatform: function(n) {
      if (typeof n != 'undefined' && n !== null && n.length) {
        platformName = n.toLowerCase();
      } else if (getParameterByName('ionicplatform')) {
        platformName = getParameterByName('ionicplatform');
      } else if (self.ua.indexOf('Edge') > -1) {
        platformName = EDGE;
      } else if (self.ua.indexOf('Windows Phone') > -1) {
        platformName = WINDOWS_PHONE;
      } else if (self.ua.indexOf('Android') > 0) {
        platformName = ANDROID;
      } else if (/iPhone|iPad|iPod/.test(self.ua)) {
        platformName = IOS;
      } else {
        platformName = self.navigator.platform && navigator.platform.toLowerCase().split(' ')[0] || '';
      }
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#version
     * @returns {number} The version of the current device platform.
     */
    version: function() {
      // singleton to get the platform version
      if (platformVersion === null) self.setVersion(self.device().version);
      return platformVersion;
    },

    /**
     * @private
     */
    setVersion: function(v) {
      if (typeof v != 'undefined' && v !== null) {
        v = v.split('.');
        v = parseFloat(v[0] + '.' + (v.length > 1 ? v[1] : 0));
        if (!isNaN(v)) {
          platformVersion = v;
          return;
        }
      }

      platformVersion = 0;

      // fallback to user-agent checking
      var pName = self.platform();
      var versionMatch = {
        'android': /Android (\d+).(\d+)?/,
        'ios': /OS (\d+)_(\d+)?/,
        'windowsphone': /Windows Phone (\d+).(\d+)?/
      };
      if (versionMatch[pName]) {
        v = self.ua.match(versionMatch[pName]);
        if (v && v.length > 2) {
          platformVersion = parseFloat(v[1] + '.' + v[2]);
        }
      }
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#is
     * @param {string} Platform name.
     * @returns {boolean} Whether the platform name provided is detected.
     */
    is: function(type) {
      type = type.toLowerCase();
      // check if it has an array of platforms
      if (self.platforms) {
        for (var x = 0; x < self.platforms.length; x++) {
          if (self.platforms[x] === type) return true;
        }
      }
      // exact match
      var pName = self.platform();
      if (pName) {
        return pName === type.toLowerCase();
      }

      // A quick hack for to check userAgent
      return self.ua.toLowerCase().indexOf(type) >= 0;
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#exitApp
     * @description Exit the app.
     */
    exitApp: function() {
      self.ready(function() {
        navigator.app && navigator.app.exitApp && navigator.app.exitApp();
      });
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#showStatusBar
     * @description Shows or hides the device status bar (in Cordova). Requires `ionic plugin add cordova-plugin-statusbar`
     * @param {boolean} shouldShow Whether or not to show the status bar.
     */
    showStatusBar: function(val) {
      // Only useful when run within cordova
      self._showStatusBar = val;
      self.ready(function() {
        // run this only when or if the platform (cordova) is ready
        requestAnimationFrame(function() {
          if (self._showStatusBar) {
            // they do not want it to be full screen
            window.StatusBar && window.StatusBar.show();
            document.body.classList.remove('status-bar-hide');
          } else {
            // it should be full screen
            window.StatusBar && window.StatusBar.hide();
            document.body.classList.add('status-bar-hide');
          }
        });
      });
    },

    /**
     * @ngdoc method
     * @name ionic.Platform#fullScreen
     * @description
     * Sets whether the app is fullscreen or not (in Cordova).
     * @param {boolean=} showFullScreen Whether or not to set the app to fullscreen. Defaults to true. Requires `ionic plugin add cordova-plugin-statusbar`
     * @param {boolean=} showStatusBar Whether or not to show the device's status bar. Defaults to false.
     */
    fullScreen: function(showFullScreen, showStatusBar) {
      // showFullScreen: default is true if no param provided
      self.isFullScreen = (showFullScreen !== false);

      // add/remove the fullscreen classname to the body
      ionic.DomUtil.ready(function() {
        // run this only when or if the DOM is ready
        requestAnimationFrame(function() {
          if (self.isFullScreen) {
            document.body.classList.add('fullscreen');
          } else {
            document.body.classList.remove('fullscreen');
          }
        });
        // showStatusBar: default is false if no param provided
        self.showStatusBar((showStatusBar === true));
      });
    }

  };

  var platformName = null, // just the name, like iOS or Android
  platformVersion = null, // a float of the major and minor, like 7.1
  readyCallbacks = [],
  windowLoadListenderAttached,
  platformReadyTimer = 2000; // How long to wait for platform ready before emitting a warning

  verifyPlatformReady();

  // Warn the user if deviceready did not fire in a reasonable amount of time, and how to fix it.
  function verifyPlatformReady() {
    setTimeout(function() {
      if(!self.isReady && self.isWebView()) {
        void 0;
      }
    }, platformReadyTimer);
  }

  // setup listeners to know when the device is ready to go
  function onWindowLoad() {
    if (self.isWebView()) {
      // the window and scripts are fully loaded, and a cordova/phonegap
      // object exists then let's listen for the deviceready
      document.addEventListener("deviceready", onPlatformReady, false);
    } else {
      // the window and scripts are fully loaded, but the window object doesn't have the
      // cordova/phonegap object, so its just a browser, not a webview wrapped w/ cordova
      onPlatformReady();
    }
    if (windowLoadListenderAttached) {
      window.removeEventListener("load", onWindowLoad, false);
    }
  }
  if (document.readyState === 'complete') {
    onWindowLoad();
  } else {
    windowLoadListenderAttached = true;
    window.addEventListener("load", onWindowLoad, false);
  }

  function onPlatformReady() {
    // the device is all set to go, init our own stuff then fire off our event
    self.isReady = true;
    self.detect();
    for (var x = 0; x < readyCallbacks.length; x++) {
      // fire off all the callbacks that were added before the platform was ready
      readyCallbacks[x]();
    }
    readyCallbacks = [];
    ionic.trigger('platformready', { target: document });

    requestAnimationFrame(function() {
      document.body.classList.add('platform-ready');
    });
  }

})(window, document, ionic);

(function(document, ionic) {
  'use strict';

  // Ionic CSS polyfills
  ionic.CSS = {};
  ionic.CSS.TRANSITION = [];
  ionic.CSS.TRANSFORM = [];

  ionic.EVENTS = {};

  (function() {

    // transform
    var i, keys = ['webkitTransform', 'transform', '-webkit-transform', 'webkit-transform',
                   '-moz-transform', 'moz-transform', 'MozTransform', 'mozTransform', 'msTransform'];

    for (i = 0; i < keys.length; i++) {
      if (document.documentElement.style[keys[i]] !== undefined) {
        ionic.CSS.TRANSFORM = keys[i];
        break;
      }
    }

    // transition
    keys = ['webkitTransition', 'mozTransition', 'msTransition', 'transition'];
    for (i = 0; i < keys.length; i++) {
      if (document.documentElement.style[keys[i]] !== undefined) {
        ionic.CSS.TRANSITION = keys[i];
        break;
      }
    }

    // Fallback in case the keys don't exist at all
    ionic.CSS.TRANSITION = ionic.CSS.TRANSITION || 'transition';

    // The only prefix we care about is webkit for transitions.
    var isWebkit = ionic.CSS.TRANSITION.indexOf('webkit') > -1;

    // transition duration
    ionic.CSS.TRANSITION_DURATION = (isWebkit ? '-webkit-' : '') + 'transition-duration';

    // To be sure transitionend works everywhere, include *both* the webkit and non-webkit events
    ionic.CSS.TRANSITIONEND = (isWebkit ? 'webkitTransitionEnd ' : '') + 'transitionend';
  })();

  (function() {
      var touchStartEvent = 'touchstart';
      var touchMoveEvent = 'touchmove';
      var touchEndEvent = 'touchend';
      var touchCancelEvent = 'touchcancel';

      if (window.navigator.pointerEnabled) {
        touchStartEvent = 'pointerdown';
        touchMoveEvent = 'pointermove';
        touchEndEvent = 'pointerup';
        touchCancelEvent = 'pointercancel';
      } else if (window.navigator.msPointerEnabled) {
        touchStartEvent = 'MSPointerDown';
        touchMoveEvent = 'MSPointerMove';
        touchEndEvent = 'MSPointerUp';
        touchCancelEvent = 'MSPointerCancel';
      }

      ionic.EVENTS.touchstart = touchStartEvent;
      ionic.EVENTS.touchmove = touchMoveEvent;
      ionic.EVENTS.touchend = touchEndEvent;
      ionic.EVENTS.touchcancel = touchCancelEvent;
  })();

  // classList polyfill for them older Androids
  // https://gist.github.com/devongovett/1381839
  if (!("classList" in document.documentElement) && Object.defineProperty && typeof HTMLElement !== 'undefined') {
    Object.defineProperty(HTMLElement.prototype, 'classList', {
      get: function() {
        var self = this;
        function update(fn) {
          return function() {
            var x, classes = self.className.split(/\s+/);

            for (x = 0; x < arguments.length; x++) {
              fn(classes, classes.indexOf(arguments[x]), arguments[x]);
            }

            self.className = classes.join(" ");
          };
        }

        return {
          add: update(function(classes, index, value) {
            ~index || classes.push(value);
          }),

          remove: update(function(classes, index) {
            ~index && classes.splice(index, 1);
          }),

          toggle: update(function(classes, index, value) {
            ~index ? classes.splice(index, 1) : classes.push(value);
          }),

          contains: function(value) {
            return !!~self.className.split(/\s+/).indexOf(value);
          },

          item: function(i) {
            return self.className.split(/\s+/)[i] || null;
          }
        };

      }
    });
  }

})(document, ionic);


/**
 * @ngdoc page
 * @name tap
 * @module ionic
 * @description
 * On touch devices such as a phone or tablet, some browsers implement a 300ms delay between
 * the time the user stops touching the display and the moment the browser executes the
 * click. This delay was initially introduced so the browser can know whether the user wants to
 * double-tap to zoom in on the webpage.  Basically, the browser waits roughly 300ms to see if
 * the user is double-tapping, or just tapping on the display once.
 *
 * Out of the box, Ionic automatically removes the 300ms delay in order to make Ionic apps
 * feel more "native" like. Resultingly, other solutions such as
 * [fastclick](https://github.com/ftlabs/fastclick) and Angular's
 * [ngTouch](https://docs.angularjs.org/api/ngTouch) should not be included, to avoid conflicts.
 *
 * Some browsers already remove the delay with certain settings, such as the CSS property
 * `touch-events: none` or with specific meta tag viewport values. However, each of these
 * browsers still handle clicks differently, such as when to fire off or cancel the event
 * (like scrolling when the target is a button, or holding a button down).
 * For browsers that already remove the 300ms delay, consider Ionic's tap system as a way to
 * normalize how clicks are handled across the various devices so there's an expected response
 * no matter what the device, platform or version. Additionally, Ionic will prevent
 * ghostclicks which even browsers that remove the delay still experience.
 *
 * In some cases, third-party libraries may also be working with touch events which can interfere
 * with the tap system. For example, mapping libraries like Google or Leaflet Maps often implement
 * a touch detection system which conflicts with Ionic's tap system.
 *
 * ### Disabling the tap system
 *
 * To disable the tap for an element and all of its children elements,
 * add the attribute `data-tap-disabled="true"`.
 *
 * ```html
 * <div data-tap-disabled="true">
 *     <div id="google-map"></div>
 * </div>
 * ```
 *
 * ### Additional Notes:
 *
 * - Ionic tap  works with Ionic's JavaScript scrolling
 * - Elements can come and go from the DOM and Ionic tap doesn't keep adding and removing
 *   listeners
 * - No "tap delay" after the first "tap" (you can tap as fast as you want, they all click)
 * - Minimal events listeners, only being added to document
 * - Correct focus in/out on each input type (select, textearea, range) on each platform/device
 * - Shows and hides virtual keyboard correctly for each platform/device
 * - Works with labels surrounding inputs
 * - Does not fire off a click if the user moves the pointer too far
 * - Adds and removes an 'activated' css class
 * - Multiple [unit tests](https://github.com/ionic-team/ionic/blob/1.x/test/unit/utils/tap.unit.js) for each scenario
 *
 */
/*

 IONIC TAP
 ---------------
 - Both touch and mouse events are added to the document.body on DOM ready
 - If a touch event happens, it does not use mouse event listeners
 - On touchend, if the distance between start and end was small, trigger a click
 - In the triggered click event, add a 'isIonicTap' property
 - The triggered click receives the same x,y coordinates as as the end event
 - On document.body click listener (with useCapture=true), only allow clicks with 'isIonicTap'
 - Triggering clicks with mouse events work the same as touch, except with mousedown/mouseup
 - Tapping inputs is disabled during scrolling
*/

var tapDoc; // the element which the listeners are on (document.body)
var tapActiveEle; // the element which is active (probably has focus)
var tapEnabledTouchEvents;
var tapMouseResetTimer;
var tapPointerMoved;
var tapPointerStart;
var tapTouchFocusedInput;
var tapLastTouchTarget;
var tapTouchMoveListener = 'touchmove';

// how much the coordinates can be off between start/end, but still a click
var TAP_RELEASE_TOLERANCE = 12; // default tolerance
var TAP_RELEASE_BUTTON_TOLERANCE = 50; // button elements should have a larger tolerance

var tapEventListeners = {
  'click': tapClickGateKeeper,

  'mousedown': tapMouseDown,
  'mouseup': tapMouseUp,
  'mousemove': tapMouseMove,

  'touchstart': tapTouchStart,
  'touchend': tapTouchEnd,
  'touchcancel': tapTouchCancel,
  'touchmove': tapTouchMove,

  'pointerdown': tapTouchStart,
  'pointerup': tapTouchEnd,
  'pointercancel': tapTouchCancel,
  'pointermove': tapTouchMove,

  'MSPointerDown': tapTouchStart,
  'MSPointerUp': tapTouchEnd,
  'MSPointerCancel': tapTouchCancel,
  'MSPointerMove': tapTouchMove,

  'focusin': tapFocusIn,
  'focusout': tapFocusOut
};

ionic.tap = {

  register: function(ele) {
    tapDoc = ele;

    tapEventListener('click', true, true);
    tapEventListener('mouseup');
    tapEventListener('mousedown');

    if (window.navigator.pointerEnabled) {
      tapEventListener('pointerdown');
      tapEventListener('pointerup');
      tapEventListener('pointercancel');
      tapTouchMoveListener = 'pointermove';

    } else if (window.navigator.msPointerEnabled) {
      tapEventListener('MSPointerDown');
      tapEventListener('MSPointerUp');
      tapEventListener('MSPointerCancel');
      tapTouchMoveListener = 'MSPointerMove';

    } else {
      tapEventListener('touchstart');
      tapEventListener('touchend');
      tapEventListener('touchcancel');
    }

    tapEventListener('focusin');
    tapEventListener('focusout');

    return function() {
      for (var type in tapEventListeners) {
        tapEventListener(type, false);
      }
      tapDoc = null;
      tapActiveEle = null;
      tapEnabledTouchEvents = false;
      tapPointerMoved = false;
      tapPointerStart = null;
    };
  },

  ignoreScrollStart: function(e) {
    return (e.defaultPrevented) ||  // defaultPrevented has been assigned by another component handling the event
           (/^(file|range)$/i).test(e.target.type) ||
           (e.target.dataset ? e.target.dataset.preventScroll : e.target.getAttribute('data-prevent-scroll')) == 'true' || // manually set within an elements attributes
           (!!(/^(object|embed)$/i).test(e.target.tagName)) ||  // flash/movie/object touches should not try to scroll
           ionic.tap.isElementTapDisabled(e.target); // check if this element, or an ancestor, has `data-tap-disabled` attribute
  },

  isTextInput: function(ele) {
    return !!ele &&
           (ele.tagName == 'TEXTAREA' ||
            ele.contentEditable === 'true' ||
            (ele.tagName == 'INPUT' && !(/^(radio|checkbox|range|file|submit|reset|color|image|button)$/i).test(ele.type)));
  },

  isDateInput: function(ele) {
    return !!ele &&
            (ele.tagName == 'INPUT' && (/^(date|time|datetime-local|month|week)$/i).test(ele.type));
  },

  isVideo: function(ele) {
    return !!ele &&
            (ele.tagName == 'VIDEO');
  },

  isKeyboardElement: function(ele) {
    if ( !ionic.Platform.isIOS() || ionic.Platform.isIPad() ) {
      return ionic.tap.isTextInput(ele) && !ionic.tap.isDateInput(ele);
    } else {
      return ionic.tap.isTextInput(ele) || ( !!ele && ele.tagName == "SELECT");
    }
  },

  isLabelWithTextInput: function(ele) {
    var container = tapContainingElement(ele, false);

    return !!container &&
           ionic.tap.isTextInput(tapTargetElement(container));
  },

  containsOrIsTextInput: function(ele) {
    return ionic.tap.isTextInput(ele) || ionic.tap.isLabelWithTextInput(ele);
  },

  cloneFocusedInput: function(container) {
    if (ionic.tap.hasCheckedClone) return;
    ionic.tap.hasCheckedClone = true;

    ionic.requestAnimationFrame(function() {
      var focusInput = container.querySelector(':focus');
      if (ionic.tap.isTextInput(focusInput) && !ionic.tap.isDateInput(focusInput)) {
        var clonedInput = focusInput.cloneNode(true);

        clonedInput.value = focusInput.value;
        clonedInput.classList.add('cloned-text-input');
        clonedInput.readOnly = true;
        if (focusInput.isContentEditable) {
          clonedInput.contentEditable = focusInput.contentEditable;
          clonedInput.innerHTML = focusInput.innerHTML;
        }
        focusInput.parentElement.insertBefore(clonedInput, focusInput);
        focusInput.classList.add('previous-input-focus');

        clonedInput.scrollTop = focusInput.scrollTop;
      }
    });
  },

  hasCheckedClone: false,

  removeClonedInputs: function(container) {
    ionic.tap.hasCheckedClone = false;

    ionic.requestAnimationFrame(function() {
      var clonedInputs = container.querySelectorAll('.cloned-text-input');
      var previousInputFocus = container.querySelectorAll('.previous-input-focus');
      var x;

      for (x = 0; x < clonedInputs.length; x++) {
        clonedInputs[x].parentElement.removeChild(clonedInputs[x]);
      }

      for (x = 0; x < previousInputFocus.length; x++) {
        previousInputFocus[x].classList.remove('previous-input-focus');
        previousInputFocus[x].style.top = '';
        if ( ionic.keyboard.isOpen && !ionic.keyboard.isClosing ) previousInputFocus[x].focus();
      }
    });
  },

  requiresNativeClick: function(ele) {
    if (ionic.Platform.isWindowsPhone() && (ele.tagName == 'A' || ele.tagName == 'BUTTON' || ele.hasAttribute('ng-click') || (ele.tagName == 'INPUT' && (ele.type == 'button' || ele.type == 'submit')))) {
      return true; //Windows Phone edge case, prevent ng-click (and similar) events from firing twice on this platform
    }
    if (!ele || ele.disabled || (/^(file|range)$/i).test(ele.type) || (/^(object|video)$/i).test(ele.tagName) || ionic.tap.isLabelContainingFileInput(ele)) {
      return true;
    }
    return ionic.tap.isElementTapDisabled(ele);
  },

  isLabelContainingFileInput: function(ele) {
    var lbl = tapContainingElement(ele);
    if (lbl.tagName !== 'LABEL') return false;
    var fileInput = lbl.querySelector('input[type=file]');
    if (fileInput && fileInput.disabled === false) return true;
    return false;
  },

  isElementTapDisabled: function(ele) {
    if (ele && ele.nodeType === 1) {
      var element = ele;
      while (element) {
        if (element.getAttribute && element.getAttribute('data-tap-disabled') == 'true') {
          return true;
        }
        element = element.parentElement;
      }
    }
    return false;
  },

  setTolerance: function(releaseTolerance, releaseButtonTolerance) {
    TAP_RELEASE_TOLERANCE = releaseTolerance;
    TAP_RELEASE_BUTTON_TOLERANCE = releaseButtonTolerance;
  },

  cancelClick: function() {
    // used to cancel any simulated clicks which may happen on a touchend/mouseup
    // gestures uses this method within its tap and hold events
    tapPointerMoved = true;
  },

  pointerCoord: function(event) {
    // This method can get coordinates for both a mouse click
    // or a touch depending on the given event
    var c = { x: 0, y: 0 };
    if (event) {
      var touches = event.touches && event.touches.length ? event.touches : [event];
      var e = (event.changedTouches && event.changedTouches[0]) || touches[0];
      if (e) {
        c.x = e.clientX || e.pageX || 0;
        c.y = e.clientY || e.pageY || 0;
      }
    }
    return c;
  }

};

function tapEventListener(type, enable, useCapture) {
  if (enable !== false) {
    tapDoc.addEventListener(type, tapEventListeners[type], useCapture);
  } else {
    tapDoc.removeEventListener(type, tapEventListeners[type]);
  }
}

function tapClick(e) {
  // simulate a normal click by running the element's click method then focus on it
  var container = tapContainingElement(e.target);
  var ele = tapTargetElement(container);

  if (ionic.tap.requiresNativeClick(ele) || tapPointerMoved) return false;

  var c = ionic.tap.pointerCoord(e);

  //console.log('tapClick', e.type, ele.tagName, '('+c.x+','+c.y+')');
  triggerMouseEvent('click', ele, c.x, c.y);

  // if it's an input, focus in on the target, otherwise blur
  tapHandleFocus(ele);
}

function triggerMouseEvent(type, ele, x, y) {
  // using initMouseEvent instead of MouseEvent for our Android friends
  var clickEvent = document.createEvent("MouseEvents");
  clickEvent.initMouseEvent(type, true, true, window, 1, 0, 0, x, y, false, false, false, false, 0, null);
  clickEvent.isIonicTap = true;
  ele.dispatchEvent(clickEvent);
}

function tapClickGateKeeper(e) {
  //console.log('click ' + Date.now() + ' isIonicTap: ' + (e.isIonicTap ? true : false));
  if (e.target.type == 'submit' && e.detail === 0) {
    // do not prevent click if it came from an "Enter" or "Go" keypress submit
    return null;
  }

  // do not allow through any click events that were not created by ionic.tap
  if ((ionic.scroll.isScrolling && ionic.tap.containsOrIsTextInput(e.target)) ||
      (!e.isIonicTap && !ionic.tap.requiresNativeClick(e.target))) {
    //console.log('clickPrevent', e.target.tagName);
    e.stopPropagation();

    if (!ionic.tap.isLabelWithTextInput(e.target)) {
      // labels clicks from native should not preventDefault othersize keyboard will not show on input focus
      e.preventDefault();
    }
    return false;
  }
}

// MOUSE
function tapMouseDown(e) {
  //console.log('mousedown ' + Date.now());
  if (e.isIonicTap || tapIgnoreEvent(e)) return null;

  if (tapEnabledTouchEvents) {
    //console.log('mousedown', 'stop event');
    e.stopPropagation();

    if (!ionic.Platform.isEdge() && (!ionic.tap.isTextInput(e.target) || tapLastTouchTarget !== e.target) &&
      !isSelectOrOption(e.target.tagName) && !e.target.isContentEditable && !ionic.tap.isVideo(e.target)) {
      // If you preventDefault on a text input then you cannot move its text caret/cursor.
      // Allow through only the text input default. However, without preventDefault on an
      // input the 300ms delay can change focus on inputs after the keyboard shows up.
      // The focusin event handles the chance of focus changing after the keyboard shows.
      // Windows Phone - if you preventDefault on a video element then you cannot operate
      // its native controls.
      e.preventDefault();
    }

    return false;
  }

  tapPointerMoved = false;
  tapPointerStart = ionic.tap.pointerCoord(e);

  tapEventListener('mousemove');
  ionic.activator.start(e);
}

function tapMouseUp(e) {
  //console.log("mouseup " + Date.now());
  if (tapEnabledTouchEvents) {
    e.stopPropagation();
    e.preventDefault();
    return false;
  }

  if (tapIgnoreEvent(e) || isSelectOrOption(e.target.tagName)) return false;

  if (!tapHasPointerMoved(e)) {
    tapClick(e);
  }
  tapEventListener('mousemove', false);
  ionic.activator.end();
  tapPointerMoved = false;
}

function tapMouseMove(e) {
  if (tapHasPointerMoved(e)) {
    tapEventListener('mousemove', false);
    ionic.activator.end();
    tapPointerMoved = true;
    return false;
  }
}


// TOUCH
function tapTouchStart(e) {
  //console.log("touchstart " + Date.now());
  if (tapIgnoreEvent(e)) return;

  tapPointerMoved = false;

  tapEnableTouchEvents();
  tapPointerStart = ionic.tap.pointerCoord(e);

  tapEventListener(tapTouchMoveListener);
  ionic.activator.start(e);

  if (ionic.Platform.isIOS() && ionic.tap.isLabelWithTextInput(e.target)) {
    // if the tapped element is a label, which has a child input
    // then preventDefault so iOS doesn't ugly auto scroll to the input
    // but do not prevent default on Android or else you cannot move the text caret
    // and do not prevent default on Android or else no virtual keyboard shows up

    var textInput = tapTargetElement(tapContainingElement(e.target));
    if (textInput !== tapActiveEle) {
      // don't preventDefault on an already focused input or else iOS's text caret isn't usable
      //console.log('Would prevent default here');
      e.preventDefault();
    }
  }
}

function tapTouchEnd(e) {
  //console.log('touchend ' + Date.now());
  if (tapIgnoreEvent(e)) return;

  tapEnableTouchEvents();
  if (!tapHasPointerMoved(e)) {
    tapClick(e);

    if (isSelectOrOption(e.target.tagName)) {
      e.preventDefault();
    }
  }

  tapLastTouchTarget = e.target;
  tapTouchCancel();
}

function tapTouchMove(e) {
  if (tapHasPointerMoved(e)) {
    tapPointerMoved = true;
    tapEventListener(tapTouchMoveListener, false);
    ionic.activator.end();
    return false;
  }
}

function tapTouchCancel() {
  tapEventListener(tapTouchMoveListener, false);
  ionic.activator.end();
  tapPointerMoved = false;
}

function tapEnableTouchEvents() {
  tapEnabledTouchEvents = true;
  clearTimeout(tapMouseResetTimer);
  tapMouseResetTimer = setTimeout(function() {
    tapEnabledTouchEvents = false;
  }, 600);
}

function tapIgnoreEvent(e) {
  if (e.isTapHandled) return true;
  e.isTapHandled = true;

  if(ionic.tap.isElementTapDisabled(e.target)) {
    return true;
  }

  if(e.target.tagName == 'SELECT') {
    return true;
  }

  if (ionic.scroll.isScrolling && ionic.tap.containsOrIsTextInput(e.target)) {
    e.preventDefault();
    return true;
  }
}

function tapHandleFocus(ele) {
  tapTouchFocusedInput = null;

  var triggerFocusIn = false;

  if (ele.tagName == 'SELECT') {
    // trick to force Android options to show up
    triggerMouseEvent('mousedown', ele, 0, 0);
    ele.focus && ele.focus();
    triggerFocusIn = true;

  } else if (tapActiveElement() === ele) {
    // already is the active element and has focus
    triggerFocusIn = true;

  } else if ((/^(input|textarea|ion-label)$/i).test(ele.tagName) || ele.isContentEditable) {
    triggerFocusIn = true;
    ele.focus && ele.focus();
    ele.value = ele.value;
    if (tapEnabledTouchEvents) {
      tapTouchFocusedInput = ele;
    }

  } else {
    tapFocusOutActive();
  }

  if (triggerFocusIn) {
    tapActiveElement(ele);
    ionic.trigger('ionic.focusin', {
      target: ele
    }, true);
  }
}

function tapFocusOutActive() {
  var ele = tapActiveElement();
  if (ele && ((/^(input|textarea|select)$/i).test(ele.tagName) || ele.isContentEditable)) {
    //console.log('tapFocusOutActive', ele.tagName);
    ele.blur();
  }
  tapActiveElement(null);
}

function tapFocusIn(e) {
  //console.log('focusin ' + Date.now());
  // Because a text input doesn't preventDefault (so the caret still works) there's a chance
  // that its mousedown event 300ms later will change the focus to another element after
  // the keyboard shows up.

  if (tapEnabledTouchEvents &&
      ionic.tap.isTextInput(tapActiveElement()) &&
      ionic.tap.isTextInput(tapTouchFocusedInput) &&
      tapTouchFocusedInput !== e.target) {

    // 1) The pointer is from touch events
    // 2) There is an active element which is a text input
    // 3) A text input was just set to be focused on by a touch event
    // 4) A new focus has been set, however the target isn't the one the touch event wanted
    //console.log('focusin', 'tapTouchFocusedInput');
    tapTouchFocusedInput.focus();
    tapTouchFocusedInput = null;
  }
  ionic.scroll.isScrolling = false;
}

function tapFocusOut() {
  //console.log("focusout");
  tapActiveElement(null);
}

function tapActiveElement(ele) {
  if (arguments.length) {
    tapActiveEle = ele;
  }
  return tapActiveEle || document.activeElement;
}

function tapHasPointerMoved(endEvent) {
  if (!endEvent || endEvent.target.nodeType !== 1 || !tapPointerStart || (tapPointerStart.x === 0 && tapPointerStart.y === 0)) {
    return false;
  }
  var endCoordinates = ionic.tap.pointerCoord(endEvent);

  var hasClassList = !!(endEvent.target.classList && endEvent.target.classList.contains &&
    typeof endEvent.target.classList.contains === 'function');
  var releaseTolerance = hasClassList && endEvent.target.classList.contains('button') ?
    TAP_RELEASE_BUTTON_TOLERANCE :
    TAP_RELEASE_TOLERANCE;

  return Math.abs(tapPointerStart.x - endCoordinates.x) > releaseTolerance ||
         Math.abs(tapPointerStart.y - endCoordinates.y) > releaseTolerance;
}

function tapContainingElement(ele, allowSelf) {
  var climbEle = ele;
  for (var x = 0; x < 6; x++) {
    if (!climbEle) break;
    if (climbEle.tagName === 'LABEL') return climbEle;
    climbEle = climbEle.parentElement;
  }
  if (allowSelf !== false) return ele;
}

function tapTargetElement(ele) {
  if (ele && ele.tagName === 'LABEL') {
    if (ele.control) return ele.control;

    // older devices do not support the "control" property
    if (ele.querySelector) {
      var control = ele.querySelector('input,textarea,select');
      if (control) return control;
    }
  }
  return ele;
}

function isSelectOrOption(tagName){
  return (/^(select|option)$/i).test(tagName);
}

ionic.DomUtil.ready(function() {
  var ng = typeof angular !== 'undefined' ? angular : null;
  //do nothing for e2e tests
  if (!ng || (ng && !ng.scenario)) {
    ionic.tap.register(document);
  }
});

(function(document, ionic) {
  'use strict';

  var queueElements = {};   // elements that should get an active state in XX milliseconds
  var activeElements = {};  // elements that are currently active
  var keyId = 0;            // a counter for unique keys for the above ojects
  var ACTIVATED_CLASS = 'activated';

  ionic.activator = {

    start: function(e) {
      var hitX = ionic.tap.pointerCoord(e).x;
      if (hitX > 0 && hitX < 30) {
        return;
      }

      // when an element is touched/clicked, it climbs up a few
      // parents to see if it is an .item or .button element
      ionic.requestAnimationFrame(function() {
        if ((ionic.scroll && ionic.scroll.isScrolling) || ionic.tap.requiresNativeClick(e.target)) return;
        var ele = e.target;
        var eleToActivate;

        for (var x = 0; x < 6; x++) {
          if (!ele || ele.nodeType !== 1) break;
          if (eleToActivate && ele.classList && ele.classList.contains('item')) {
            eleToActivate = ele;
            break;
          }
          if (ele.tagName == 'A' || ele.tagName == 'BUTTON' || ele.hasAttribute('ng-click')) {
            eleToActivate = ele;
            break;
          }
          if (ele.classList && ele.classList.contains('button')) {
            eleToActivate = ele;
            break;
          }
          // no sense climbing past these
          if (ele.tagName == 'ION-CONTENT' || (ele.classList && ele.classList.contains('pane')) || ele.tagName == 'BODY') {
            break;
          }
          ele = ele.parentElement;
        }

        if (eleToActivate) {
          // queue that this element should be set to active
          queueElements[keyId] = eleToActivate;

          // on the next frame, set the queued elements to active
          ionic.requestAnimationFrame(activateElements);

          keyId = (keyId > 29 ? 0 : keyId + 1);
        }

      });
    },

    end: function() {
      // clear out any active/queued elements after XX milliseconds
      setTimeout(clear, 200);
    }

  };

  function clear() {
    // clear out any elements that are queued to be set to active
    queueElements = {};

    // in the next frame, remove the active class from all active elements
    ionic.requestAnimationFrame(deactivateElements);
  }

  function activateElements() {
    // activate all elements in the queue
    for (var key in queueElements) {
      if (queueElements[key]) {
        queueElements[key].classList.add(ACTIVATED_CLASS);
        activeElements[key] = queueElements[key];
      }
    }
    queueElements = {};
  }

  function deactivateElements() {
    if (ionic.transition && ionic.transition.isActive) {
      setTimeout(deactivateElements, 400);
      return;
    }

    for (var key in activeElements) {
      if (activeElements[key]) {
        activeElements[key].classList.remove(ACTIVATED_CLASS);
        delete activeElements[key];
      }
    }
  }

})(document, ionic);

(function(ionic) {
  /* for nextUid function below */
  var nextId = 0;

  /**
   * Various utilities used throughout Ionic
   *
   * Some of these are adopted from underscore.js and backbone.js, both also MIT licensed.
   */
  ionic.Utils = {

    arrayMove: function(arr, oldIndex, newIndex) {
      if (newIndex >= arr.length) {
        var k = newIndex - arr.length;
        while ((k--) + 1) {
          arr.push(undefined);
        }
      }
      arr.splice(newIndex, 0, arr.splice(oldIndex, 1)[0]);
      return arr;
    },

    /**
     * Return a function that will be called with the given context
     */
    proxy: function(func, context) {
      var args = Array.prototype.slice.call(arguments, 2);
      return function() {
        return func.apply(context, args.concat(Array.prototype.slice.call(arguments)));
      };
    },

    /**
     * Only call a function once in the given interval.
     *
     * @param func {Function} the function to call
     * @param wait {int} how long to wait before/after to allow function calls
     * @param immediate {boolean} whether to call immediately or after the wait interval
     */
     debounce: function(func, wait, immediate) {
      var timeout, args, context, timestamp, result;
      return function() {
        context = this;
        args = arguments;
        timestamp = new Date();
        var later = function() {
          var last = (new Date()) - timestamp;
          if (last < wait) {
            timeout = setTimeout(later, wait - last);
          } else {
            timeout = null;
            if (!immediate) result = func.apply(context, args);
          }
        };
        var callNow = immediate && !timeout;
        if (!timeout) {
          timeout = setTimeout(later, wait);
        }
        if (callNow) result = func.apply(context, args);
        return result;
      };
    },

    /**
     * Throttle the given fun, only allowing it to be
     * called at most every `wait` ms.
     */
    throttle: function(func, wait, options) {
      var context, args, result;
      var timeout = null;
      var previous = 0;
      options || (options = {});
      var later = function() {
        previous = options.leading === false ? 0 : Date.now();
        timeout = null;
        result = func.apply(context, args);
      };
      return function() {
        var now = Date.now();
        if (!previous && options.leading === false) previous = now;
        var remaining = wait - (now - previous);
        context = this;
        args = arguments;
        if (remaining <= 0) {
          clearTimeout(timeout);
          timeout = null;
          previous = now;
          result = func.apply(context, args);
        } else if (!timeout && options.trailing !== false) {
          timeout = setTimeout(later, remaining);
        }
        return result;
      };
    },
     // Borrowed from Backbone.js's extend
     // Helper function to correctly set up the prototype chain, for subclasses.
     // Similar to `goog.inherits`, but uses a hash of prototype properties and
     // class properties to be extended.
    inherit: function(protoProps, staticProps) {
      var parent = this;
      var child;

      // The constructor function for the new subclass is either defined by you
      // (the "constructor" property in your `extend` definition), or defaulted
      // by us to simply call the parent's constructor.
      if (protoProps && protoProps.hasOwnProperty('constructor')) {
        child = protoProps.constructor;
      } else {
        child = function() { return parent.apply(this, arguments); };
      }

      // Add static properties to the constructor function, if supplied.
      ionic.extend(child, parent, staticProps);

      // Set the prototype chain to inherit from `parent`, without calling
      // `parent`'s constructor function.
      var Surrogate = function() { this.constructor = child; };
      Surrogate.prototype = parent.prototype;
      child.prototype = new Surrogate();

      // Add prototype properties (instance properties) to the subclass,
      // if supplied.
      if (protoProps) ionic.extend(child.prototype, protoProps);

      // Set a convenience property in case the parent's prototype is needed
      // later.
      child.__super__ = parent.prototype;

      return child;
    },

    // Extend adapted from Underscore.js
    extend: function(obj) {
       var args = Array.prototype.slice.call(arguments, 1);
       for (var i = 0; i < args.length; i++) {
         var source = args[i];
         if (source) {
           for (var prop in source) {
             obj[prop] = source[prop];
           }
         }
       }
       return obj;
    },

    nextUid: function() {
      return 'ion' + (nextId++);
    },

    disconnectScope: function disconnectScope(scope) {
      if (!scope) return;

      if (scope.$root === scope) {
        return; // we can't disconnect the root node;
      }
      var parent = scope.$parent;
      scope.$$disconnected = true;
      scope.$broadcast('$ionic.disconnectScope', scope);

      // See Scope.$destroy
      if (parent.$$childHead === scope) {
        parent.$$childHead = scope.$$nextSibling;
      }
      if (parent.$$childTail === scope) {
        parent.$$childTail = scope.$$prevSibling;
      }
      if (scope.$$prevSibling) {
        scope.$$prevSibling.$$nextSibling = scope.$$nextSibling;
      }
      if (scope.$$nextSibling) {
        scope.$$nextSibling.$$prevSibling = scope.$$prevSibling;
      }
      scope.$$nextSibling = scope.$$prevSibling = null;
    },

    reconnectScope: function reconnectScope(scope) {
      if (!scope) return;

      if (scope.$root === scope) {
        return; // we can't disconnect the root node;
      }
      if (!scope.$$disconnected) {
        return;
      }
      var parent = scope.$parent;
      scope.$$disconnected = false;
      scope.$broadcast('$ionic.reconnectScope', scope);
      // See Scope.$new for this logic...
      scope.$$prevSibling = parent.$$childTail;
      if (parent.$$childHead) {
        parent.$$childTail.$$nextSibling = scope;
        parent.$$childTail = scope;
      } else {
        parent.$$childHead = parent.$$childTail = scope;
      }
    },

    isScopeDisconnected: function(scope) {
      var climbScope = scope;
      while (climbScope) {
        if (climbScope.$$disconnected) return true;
        climbScope = climbScope.$parent;
      }
      return false;
    }
  };

  // Bind a few of the most useful functions to the ionic scope
  ionic.inherit = ionic.Utils.inherit;
  ionic.extend = ionic.Utils.extend;
  ionic.throttle = ionic.Utils.throttle;
  ionic.proxy = ionic.Utils.proxy;
  ionic.debounce = ionic.Utils.debounce;

})(window.ionic);

/**
 * @ngdoc page
 * @name keyboard
 * @module ionic
 * @description
 * On both Android and iOS, Ionic will attempt to prevent the keyboard from
 * obscuring inputs and focusable elements when it appears by scrolling them
 * into view.  In order for this to work, any focusable elements must be within
 * a [Scroll View](http://ionicframework.com/docs/api/directive/ionScroll/)
 * or a directive such as [Content](http://ionicframework.com/docs/api/directive/ionContent/)
 * that has a Scroll View.
 *
 * It will also attempt to prevent the native overflow scrolling on focus,
 * which can cause layout issues such as pushing headers up and out of view.
 *
 * The keyboard fixes work best in conjunction with the
 * [Ionic Keyboard Plugin](https://github.com/ionic-team/ionic-plugins-keyboard),
 * although it will perform reasonably well without.  However, if you are using
 * Cordova there is no reason not to use the plugin.
 *
 * ### Hide when keyboard shows
 *
 * To hide an element when the keyboard is open, add the class `hide-on-keyboard-open`.
 *
 * ```html
 * <div class="hide-on-keyboard-open">
 *   <div id="google-map"></div>
 * </div>
 * ```
 *
 * Note: For performance reasons, elements will not be hidden for 400ms after the start of the `native.keyboardshow` event
 * from the Ionic Keyboard plugin. If you would like them to disappear immediately, you could do something
 * like:
 *
 * ```js
 *   window.addEventListener('native.keyboardshow', function(){
 *     document.body.classList.add('keyboard-open');
 *   });
 * ```
 * This adds the same `keyboard-open` class that is normally added by Ionic 400ms after the keyboard
 * opens. However, bear in mind that adding this class to the body immediately may cause jank in any
 * animations on Android that occur when the keyboard opens (for example, scrolling any obscured inputs into view).
 *
 * ----------
 *
 * ### Plugin Usage
 * Information on using the plugin can be found at
 * [https://github.com/ionic-team/ionic-plugins-keyboard](https://github.com/ionic-team/ionic-plugins-keyboard).
 *
 * ----------
 *
 * ### Android Notes
 * - If your app is running in fullscreen, i.e. you have
 *   `<preference name="Fullscreen" value="true" />` in your `config.xml` file
 *   you will need to set `ionic.Platform.isFullScreen = true` manually.
 *
 * - You can configure the behavior of the web view when the keyboard shows by setting
 *   [android:windowSoftInputMode](http://developer.android.com/reference/android/R.attr.html#windowSoftInputMode)
 *   to either `adjustPan`, `adjustResize` or `adjustNothing` in your app's
 *   activity in `AndroidManifest.xml`. `adjustResize` is the recommended setting
 *   for Ionic, but if for some reason you do use `adjustPan` you will need to
 *   set `ionic.Platform.isFullScreen = true`.
 *
 *   ```xml
 *   <activity android:windowSoftInputMode="adjustResize">
 *
 *   ```
 *
 * ### iOS Notes
 * - If the content of your app (including the header) is being pushed up and
 *   out of view on input focus, try setting `cordova.plugins.Keyboard.disableScroll(true)`.
 *   This does **not** disable scrolling in the Ionic scroll view, rather it
 *   disables the native overflow scrolling that happens automatically as a
 *   result of focusing on inputs below the keyboard.
 *
 */

/**
 * The current viewport height.
 */
var keyboardCurrentViewportHeight = 0;

/**
 * The viewport height when in portrait orientation.
 */
var keyboardPortraitViewportHeight = 0;

/**
 * The viewport height when in landscape orientation.
 */
var keyboardLandscapeViewportHeight = 0;

/**
 * The currently focused input.
 */
var keyboardActiveElement;

/**
 * The previously focused input used to reset keyboard after focusing on a
 * new non-keyboard element
 */
var lastKeyboardActiveElement;

/**
 * The scroll view containing the currently focused input.
 */
var scrollView;

/**
 * Timer for the setInterval that polls window.innerHeight to determine whether
 * the layout has updated for the keyboard showing/hiding.
 */
var waitForResizeTimer;

/**
 * Sometimes when switching inputs or orientations, focusout will fire before
 * focusin, so this timer is for the small setTimeout to determine if we should
 * really focusout/hide the keyboard.
 */
var keyboardFocusOutTimer;

/**
 * on Android, orientationchange will fire before the keyboard plugin notifies
 * the browser that the keyboard will show/is showing, so this flag indicates
 * to nativeShow that there was an orientationChange and we should update
 * the viewport height with an accurate keyboard height value
 */
var wasOrientationChange = false;

/**
 * CSS class added to the body indicating the keyboard is open.
 */
var KEYBOARD_OPEN_CSS = 'keyboard-open';

/**
 * CSS class that indicates a scroll container.
 */
var SCROLL_CONTAINER_CSS = 'scroll-content';

/**
 * Debounced keyboardFocusIn function
 */
var debouncedKeyboardFocusIn = ionic.debounce(keyboardFocusIn, 200, true);

/**
 * Debounced keyboardNativeShow function
 */
var debouncedKeyboardNativeShow = ionic.debounce(keyboardNativeShow, 100, true);

/**
 * Ionic keyboard namespace.
 * @namespace keyboard
 */
ionic.keyboard = {

  /**
   * Whether the keyboard is open or not.
   */
  isOpen: false,

  /**
   * Whether the keyboard is closing or not.
   */
  isClosing: false,

  /**
   * Whether the keyboard is opening or not.
   */
  isOpening: false,

  /**
   * The height of the keyboard in pixels, as reported by the keyboard plugin.
   * If the plugin is not available, calculated as the difference in
   * window.innerHeight after the keyboard has shown.
   */
  height: 0,

  /**
   * Whether the device is in landscape orientation or not.
   */
  isLandscape: false,

  /**
   * Whether the keyboard event listeners have been added or not
   */
  isInitialized: false,

  /**
   * Hide the keyboard, if it is open.
   */
  hide: function() {
    if (keyboardHasPlugin()) {
      cordova.plugins.Keyboard.close();
    }
    keyboardActiveElement && keyboardActiveElement.blur();
  },

  /**
   * An alias for cordova.plugins.Keyboard.show(). If the keyboard plugin
   * is installed, show the keyboard.
   */
  show: function() {
    if (keyboardHasPlugin()) {
      cordova.plugins.Keyboard.show();
    }
  },

  /**
   * Remove all keyboard related event listeners, effectively disabling Ionic's
   * keyboard adjustments.
   */
  disable: function() {
    if (keyboardHasPlugin()) {
      window.removeEventListener('native.keyboardshow', debouncedKeyboardNativeShow );
      window.removeEventListener('native.keyboardhide', keyboardFocusOut);
    } else {
      document.body.removeEventListener('focusout', keyboardFocusOut);
    }

    document.body.removeEventListener('ionic.focusin', debouncedKeyboardFocusIn);
    document.body.removeEventListener('focusin', debouncedKeyboardFocusIn);

    window.removeEventListener('orientationchange', keyboardOrientationChange);

    if ( window.navigator.msPointerEnabled ) {
      document.removeEventListener("MSPointerDown", keyboardInit);
    } else {
      document.removeEventListener('touchstart', keyboardInit);
    }
    ionic.keyboard.isInitialized = false;
  },

  /**
   * Alias for keyboardInit, initialize all keyboard related event listeners.
   */
  enable: function() {
    keyboardInit();
  }
};

// Initialize the viewport height (after ionic.keyboard.height has been
// defined).
keyboardCurrentViewportHeight = getViewportHeight();


                             /* Event handlers */
/* ------------------------------------------------------------------------- */

/**
 * Event handler for first touch event, initializes all event listeners
 * for keyboard related events. Also aliased by ionic.keyboard.enable.
 */
function keyboardInit() {

  if (ionic.keyboard.isInitialized) return;

  if (keyboardHasPlugin()) {
    window.addEventListener('native.keyboardshow', debouncedKeyboardNativeShow);
    window.addEventListener('native.keyboardhide', keyboardFocusOut);
  } else {
    document.body.addEventListener('focusout', keyboardFocusOut);
  }

  document.body.addEventListener('ionic.focusin', debouncedKeyboardFocusIn);
  document.body.addEventListener('focusin', debouncedKeyboardFocusIn);

  if (window.navigator.msPointerEnabled) {
    document.removeEventListener("MSPointerDown", keyboardInit);
  } else {
    document.removeEventListener('touchstart', keyboardInit);
  }

  ionic.keyboard.isInitialized = true;
}

/**
 * Event handler for 'native.keyboardshow' event, sets keyboard.height to the
 * reported height and keyboard.isOpening to true. Then calls
 * keyboardWaitForResize with keyboardShow or keyboardUpdateViewportHeight as
 * the callback depending on whether the event was triggered by a focusin or
 * an orientationchange.
 */
function keyboardNativeShow(e) {
  clearTimeout(keyboardFocusOutTimer);
  //console.log("keyboardNativeShow fired at: " + Date.now());
  //console.log("keyboardNativeshow window.innerHeight: " + window.innerHeight);

  if (!ionic.keyboard.isOpen || ionic.keyboard.isClosing) {
    ionic.keyboard.isOpening = true;
    ionic.keyboard.isClosing = false;
  }

  ionic.keyboard.height = e.keyboardHeight;
  //console.log('nativeshow keyboard height:' + e.keyboardHeight);

  if (wasOrientationChange) {
    keyboardWaitForResize(keyboardUpdateViewportHeight, true);
  } else {
    keyboardWaitForResize(keyboardShow, true);
  }
}

/**
 * Event handler for 'focusin' and 'ionic.focusin' events. Initializes
 * keyboard state (keyboardActiveElement and keyboard.isOpening) for the
 * appropriate adjustments once the window has resized.  If not using the
 * keyboard plugin, calls keyboardWaitForResize with keyboardShow as the
 * callback or keyboardShow right away if the keyboard is already open.  If
 * using the keyboard plugin does nothing and lets keyboardNativeShow handle
 * adjustments with a more accurate keyboard height.
 */
function keyboardFocusIn(e) {
  clearTimeout(keyboardFocusOutTimer);
  //console.log("keyboardFocusIn from: " + e.type + " at: " + Date.now());

  if (!e.target ||
      e.target.readOnly ||
      !ionic.tap.isKeyboardElement(e.target) ||
      !(scrollView = ionic.DomUtil.getParentWithClass(e.target, SCROLL_CONTAINER_CSS))) {
    if (keyboardActiveElement) {
        lastKeyboardActiveElement = keyboardActiveElement;
    }
    keyboardActiveElement = null;
    return;
  }

  keyboardActiveElement = e.target;

  // if using JS scrolling, undo the effects of native overflow scroll so the
  // scroll view is positioned correctly
  if (!scrollView.classList.contains("overflow-scroll")) {
    document.body.scrollTop = 0;
    scrollView.scrollTop = 0;
    ionic.requestAnimationFrame(function(){
      document.body.scrollTop = 0;
      scrollView.scrollTop = 0;
    });

    // any showing part of the document that isn't within the scroll the user
    // could touchmove and cause some ugly changes to the app, so disable
    // any touchmove events while the keyboard is open using e.preventDefault()
    if (window.navigator.msPointerEnabled) {
      document.addEventListener("MSPointerMove", keyboardPreventDefault, false);
    } else {
      document.addEventListener('touchmove', keyboardPreventDefault, false);
    }
  }

  if (!ionic.keyboard.isOpen || ionic.keyboard.isClosing) {
    ionic.keyboard.isOpening = true;
    ionic.keyboard.isClosing = false;
  }

  // attempt to prevent browser from natively scrolling input into view while
  // we are trying to do the same (while we are scrolling) if the user taps the
  // keyboard
  document.addEventListener('keydown', keyboardOnKeyDown, false);



  // if we aren't using the plugin and the keyboard isn't open yet, wait for the
  // window to resize so we can get an accurate estimate of the keyboard size,
  // otherwise we do nothing and let nativeShow call keyboardShow once we have
  // an exact keyboard height
  // if the keyboard is already open, go ahead and scroll the input into view
  // if necessary
  if (!ionic.keyboard.isOpen && !keyboardHasPlugin()) {
    keyboardWaitForResize(keyboardShow, true);

  } else if (ionic.keyboard.isOpen) {
    keyboardShow();
  }
}

/**
 * Event handler for 'focusout' events. Sets keyboard.isClosing to true and
 * calls keyboardWaitForResize with keyboardHide as the callback after a small
 * timeout.
 */
function keyboardFocusOut() {
  clearTimeout(keyboardFocusOutTimer);
  //console.log("keyboardFocusOut fired at: " + Date.now());
  //console.log("keyboardFocusOut event type: " + e.type);

  if (ionic.keyboard.isOpen || ionic.keyboard.isOpening) {
    ionic.keyboard.isClosing = true;
    ionic.keyboard.isOpening = false;
  }

  // Call keyboardHide with a slight delay because sometimes on focus or
  // orientation change focusin is called immediately after, so we give it time
  // to cancel keyboardHide
  keyboardFocusOutTimer = setTimeout(function() {
    ionic.requestAnimationFrame(function() {
      // focusOut during or right after an orientationchange, so we didn't get
      // a chance to update the viewport height yet, do it and keyboardHide
      //console.log("focusOut, wasOrientationChange: " + wasOrientationChange);
      if (wasOrientationChange) {
        keyboardWaitForResize(function(){
          keyboardUpdateViewportHeight();
          keyboardHide();
        }, false);
      } else {
        keyboardWaitForResize(keyboardHide, false);
      }
    });
  }, 50);
}

/**
 * Event handler for 'orientationchange' events. If using the keyboard plugin
 * and the keyboard is open on Android, sets wasOrientationChange to true so
 * nativeShow can update the viewport height with an accurate keyboard height.
 * If the keyboard isn't open or keyboard plugin isn't being used,
 * waits for the window to resize before updating the viewport height.
 *
 * On iOS, where orientationchange fires after the keyboard has already shown,
 * updates the viewport immediately, regardless of if the keyboard is already
 * open.
 */
function keyboardOrientationChange() {
  //console.log("orientationchange fired at: " + Date.now());
  //console.log("orientation was: " + (ionic.keyboard.isLandscape ? "landscape" : "portrait"));

  // toggle orientation
  ionic.keyboard.isLandscape = !ionic.keyboard.isLandscape;
  // //console.log("now orientation is: " + (ionic.keyboard.isLandscape ? "landscape" : "portrait"));

  // no need to wait for resizing on iOS, and orientationchange always fires
  // after the keyboard has opened, so it doesn't matter if it's open or not
  if (ionic.Platform.isIOS()) {
    keyboardUpdateViewportHeight();
  }

  // On Android, if the keyboard isn't open or we aren't using the keyboard
  // plugin, update the viewport height once everything has resized. If the
  // keyboard is open and we are using the keyboard plugin do nothing and let
  // nativeShow handle it using an accurate keyboard height.
  if ( ionic.Platform.isAndroid()) {
    if (!ionic.keyboard.isOpen || !keyboardHasPlugin()) {
      keyboardWaitForResize(keyboardUpdateViewportHeight, false);
    } else {
      wasOrientationChange = true;
    }
  }
}

/**
 * Event handler for 'keydown' event. Tries to prevent browser from natively
 * scrolling an input into view when a user taps the keyboard while we are
 * scrolling the input into view ourselves with JS.
 */
function keyboardOnKeyDown(e) {
  if (ionic.scroll.isScrolling) {
    keyboardPreventDefault(e);
  }
}

/**
 * Event for 'touchmove' or 'MSPointerMove'. Prevents native scrolling on
 * elements outside the scroll view while the keyboard is open.
 */
function keyboardPreventDefault(e) {
  if (e.target.tagName !== 'TEXTAREA') {
    e.preventDefault();
  }
}

                              /* Private API */
/* -------------------------------------------------------------------------- */

/**
 * Polls window.innerHeight until it has updated to an expected value (or
 * sufficient time has passed) before calling the specified callback function.
 * Only necessary for non-fullscreen Android which sometimes reports multiple
 * window.innerHeight values during interim layouts while it is resizing.
 *
 * On iOS, the window.innerHeight will already be updated, but we use the 50ms
 * delay as essentially a timeout so that scroll view adjustments happen after
 * the keyboard has shown so there isn't a white flash from us resizing too
 * quickly.
 *
 * @param {Function} callback the function to call once the window has resized
 * @param {boolean} isOpening whether the resize is from the keyboard opening
 * or not
 */
function keyboardWaitForResize(callback, isOpening) {
  clearInterval(waitForResizeTimer);
  var count = 0;
  var maxCount;
  var initialHeight = getViewportHeight();
  var viewportHeight = initialHeight;

  //console.log("waitForResize initial viewport height: " + viewportHeight);
  //var start = Date.now();
  //console.log("start: " + start);

  // want to fail relatively quickly on modern android devices, since it's much
  // more likely we just have a bad keyboard height
  if (ionic.Platform.isAndroid() && ionic.Platform.version() < 4.4) {
    maxCount = 30;
  } else if (ionic.Platform.isAndroid()) {
    maxCount = 10;
  } else {
    maxCount = 1;
  }

  // poll timer
  waitForResizeTimer = setInterval(function(){
    viewportHeight = getViewportHeight();

    // height hasn't updated yet, try again in 50ms
    // if not using plugin, wait for maxCount to ensure we have waited long enough
    // to get an accurate keyboard height
    if (++count < maxCount &&
        ((!isPortraitViewportHeight(viewportHeight) &&
         !isLandscapeViewportHeight(viewportHeight)) ||
         !ionic.keyboard.height)) {
      return;
    }

    // infer the keyboard height from the resize if not using the keyboard plugin
    if (!keyboardHasPlugin()) {
      ionic.keyboard.height = Math.abs(initialHeight - window.innerHeight);
    }

    // set to true if we were waiting for the keyboard to open
    ionic.keyboard.isOpen = isOpening;

    clearInterval(waitForResizeTimer);
    //var end = Date.now();
    //console.log("waitForResize count: " + count);
    //console.log("end: " + end);
    //console.log("difference: " + ( end - start ) + "ms");

    //console.log("callback: " + callback.name);
    callback();

  }, 50);

  return maxCount; //for tests
}

/**
 * On keyboard close sets keyboard state to closed, resets the scroll view,
 * removes CSS from body indicating keyboard was open, removes any event
 * listeners for when the keyboard is open and on Android blurs the active
 * element (which in some cases will still have focus even if the keyboard
 * is closed and can cause it to reappear on subsequent taps).
 */
function keyboardHide() {
  clearTimeout(keyboardFocusOutTimer);
  //console.log("keyboardHide");

  ionic.keyboard.isOpen = false;
  ionic.keyboard.isClosing = false;

  if (keyboardActiveElement || lastKeyboardActiveElement) {
    ionic.trigger('resetScrollView', {
      target: keyboardActiveElement || lastKeyboardActiveElement
    }, true);
  }

  ionic.requestAnimationFrame(function(){
    document.body.classList.remove(KEYBOARD_OPEN_CSS);
  });

  // the keyboard is gone now, remove the touchmove that disables native scroll
  if (window.navigator.msPointerEnabled) {
    document.removeEventListener("MSPointerMove", keyboardPreventDefault);
  } else {
    document.removeEventListener('touchmove', keyboardPreventDefault);
  }
  document.removeEventListener('keydown', keyboardOnKeyDown);

  if (ionic.Platform.isAndroid()) {
    // on android closing the keyboard with the back/dismiss button won't remove
    // focus and keyboard can re-appear on subsequent taps (like scrolling)
    if (keyboardHasPlugin()) cordova.plugins.Keyboard.close();
    keyboardActiveElement && keyboardActiveElement.blur();
  }

  keyboardActiveElement = null;
  lastKeyboardActiveElement = null;
}

/**
 * On keyboard open sets keyboard state to open, adds CSS to the body
 * indicating the keyboard is open and tells the scroll view to resize and
 * the currently focused input into view if necessary.
 */
function keyboardShow() {

  ionic.keyboard.isOpen = true;
  ionic.keyboard.isOpening = false;

  var details = {
    keyboardHeight: keyboardGetHeight(),
    viewportHeight: keyboardCurrentViewportHeight
  };

  if (keyboardActiveElement) {
    details.target = keyboardActiveElement;

    var elementBounds = keyboardActiveElement.getBoundingClientRect();

    details.elementTop = Math.round(elementBounds.top);
    details.elementBottom = Math.round(elementBounds.bottom);

    details.windowHeight = details.viewportHeight - details.keyboardHeight;
    //console.log("keyboardShow viewportHeight: " + details.viewportHeight +
    //", windowHeight: " + details.windowHeight +
    //", keyboardHeight: " + details.keyboardHeight);

    // figure out if the element is under the keyboard
    details.isElementUnderKeyboard = (details.elementBottom > details.windowHeight);
    //console.log("isUnderKeyboard: " + details.isElementUnderKeyboard);
    //console.log("elementBottom: " + details.elementBottom);

    // send event so the scroll view adjusts
    ionic.trigger('scrollChildIntoView', details, true);
  }

  setTimeout(function(){
    document.body.classList.add(KEYBOARD_OPEN_CSS);
  }, 400);

  return details; //for testing
}

/* eslint no-unused-vars:0 */
function keyboardGetHeight() {
  // check if we already have a keyboard height from the plugin or resize calculations
  if (ionic.keyboard.height) {
    return ionic.keyboard.height;
  }

  if (ionic.Platform.isAndroid()) {
    // should be using the plugin, no way to know how big the keyboard is, so guess
    if ( ionic.Platform.isFullScreen ) {
      return 275;
    }
    // otherwise just calculate it
    var contentHeight = window.innerHeight;
    if (contentHeight < keyboardCurrentViewportHeight) {
      return keyboardCurrentViewportHeight - contentHeight;
    } else {
      return 0;
    }
  }

  // fallback for when it's the webview without the plugin
  // or for just the standard web browser
  // TODO: have these be based on device
  if (ionic.Platform.isIOS()) {
    if (ionic.keyboard.isLandscape) {
      return 206;
    }

    if (!ionic.Platform.isWebView()) {
      return 216;
    }

    return 260;
  }

  // safe guess
  return 275;
}

function isPortraitViewportHeight(viewportHeight) {
  return !!(!ionic.keyboard.isLandscape &&
         keyboardPortraitViewportHeight &&
         (Math.abs(keyboardPortraitViewportHeight - viewportHeight) < 2));
}

function isLandscapeViewportHeight(viewportHeight) {
  return !!(ionic.keyboard.isLandscape &&
         keyboardLandscapeViewportHeight &&
         (Math.abs(keyboardLandscapeViewportHeight - viewportHeight) < 2));
}

function keyboardUpdateViewportHeight() {
  wasOrientationChange = false;
  keyboardCurrentViewportHeight = getViewportHeight();

  if (ionic.keyboard.isLandscape && !keyboardLandscapeViewportHeight) {
    //console.log("saved landscape: " + keyboardCurrentViewportHeight);
    keyboardLandscapeViewportHeight = keyboardCurrentViewportHeight;

  } else if (!ionic.keyboard.isLandscape && !keyboardPortraitViewportHeight) {
    //console.log("saved portrait: " + keyboardCurrentViewportHeight);
    keyboardPortraitViewportHeight = keyboardCurrentViewportHeight;
  }

  if (keyboardActiveElement) {
    ionic.trigger('resetScrollView', {
      target: keyboardActiveElement
    }, true);
  }

  if (ionic.keyboard.isOpen && ionic.tap.isTextInput(keyboardActiveElement)) {
    keyboardShow();
  }
}

function keyboardInitViewportHeight() {
  var viewportHeight = getViewportHeight();
  //console.log("Keyboard init VP: " + viewportHeight + " " + window.innerWidth);
  // can't just use window.innerHeight in case the keyboard is opened immediately
  if ((viewportHeight / window.innerWidth) < 1) {
    ionic.keyboard.isLandscape = true;
  }
  //console.log("ionic.keyboard.isLandscape is: " + ionic.keyboard.isLandscape);

  // initialize or update the current viewport height values
  keyboardCurrentViewportHeight = viewportHeight;
  if (ionic.keyboard.isLandscape && !keyboardLandscapeViewportHeight) {
    keyboardLandscapeViewportHeight = keyboardCurrentViewportHeight;
  } else if (!ionic.keyboard.isLandscape && !keyboardPortraitViewportHeight) {
    keyboardPortraitViewportHeight = keyboardCurrentViewportHeight;
  }
}

function getViewportHeight() {
  var windowHeight = window.innerHeight;
  //console.log('window.innerHeight is: ' + windowHeight);
  //console.log('kb height is: ' + ionic.keyboard.height);
  //console.log('kb isOpen: ' + ionic.keyboard.isOpen);

  //TODO: add iPad undocked/split kb once kb plugin supports it
  // the keyboard overlays the window on Android fullscreen
  if (!(ionic.Platform.isAndroid() && ionic.Platform.isFullScreen) &&
      (ionic.keyboard.isOpen || ionic.keyboard.isOpening) &&
      !ionic.keyboard.isClosing) {

     return windowHeight + keyboardGetHeight();
  }
  return windowHeight;
}

function keyboardHasPlugin() {
  return !!(window.cordova && cordova.plugins && cordova.plugins.Keyboard);
}

ionic.Platform.ready(function() {
  keyboardInitViewportHeight();

  window.addEventListener('orientationchange', keyboardOrientationChange);

  // if orientation changes while app is in background, update on resuming
  /*
  if ( ionic.Platform.isWebView() ) {
    document.addEventListener('resume', keyboardInitViewportHeight);

    if (ionic.Platform.isAndroid()) {
      //TODO: onbackpressed to detect keyboard close without focusout or plugin
    }
  }
  */

  // if orientation changes while app is in background, update on resuming
/*  if ( ionic.Platform.isWebView() ) {
    document.addEventListener('pause', function() {
      window.removeEventListener('orientationchange', keyboardOrientationChange);
    })
    document.addEventListener('resume', function() {
      keyboardInitViewportHeight();
      window.addEventListener('orientationchange', keyboardOrientationChange)
    });
  }*/

  // Android sometimes reports bad innerHeight on window.load
  // try it again in a lil bit to play it safe
  setTimeout(keyboardInitViewportHeight, 999);

  // only initialize the adjustments for the virtual keyboard
  // if a touchstart event happens
  if (window.navigator.msPointerEnabled) {
    document.addEventListener("MSPointerDown", keyboardInit, false);
  } else {
    document.addEventListener('touchstart', keyboardInit, false);
  }
});



var viewportTag;
var viewportProperties = {};

ionic.viewport = {
  orientation: function() {
    // 0 = Portrait
    // 90 = Landscape
    // not using window.orientation because each device has a different implementation
    return (window.innerWidth > window.innerHeight ? 90 : 0);
  }
};

function viewportLoadTag() {
  var x;

  for (x = 0; x < document.head.children.length; x++) {
    if (document.head.children[x].name == 'viewport') {
      viewportTag = document.head.children[x];
      break;
    }
  }

  if (viewportTag) {
    var props = viewportTag.content.toLowerCase().replace(/\s+/g, '').split(',');
    var keyValue;
    for (x = 0; x < props.length; x++) {
      if (props[x]) {
        keyValue = props[x].split('=');
        viewportProperties[ keyValue[0] ] = (keyValue.length > 1 ? keyValue[1] : '_');
      }
    }
    viewportUpdate();
  }
}

function viewportUpdate() {
  // unit tests in viewport.unit.js

  var initWidth = viewportProperties.width;
  var initHeight = viewportProperties.height;
  var p = ionic.Platform;
  var version = p.version();
  var DEVICE_WIDTH = 'device-width';
  var DEVICE_HEIGHT = 'device-height';
  var orientation = ionic.viewport.orientation();

  // Most times we're removing the height and adding the width
  // So this is the default to start with, then modify per platform/version/oreintation
  delete viewportProperties.height;
  viewportProperties.width = DEVICE_WIDTH;

  if (p.isIPad()) {
    // iPad

    if (version > 7) {
      // iPad >= 7.1
      // https://issues.apache.org/jira/browse/CB-4323
      delete viewportProperties.width;

    } else {
      // iPad <= 7.0

      if (p.isWebView()) {
        // iPad <= 7.0 WebView

        if (orientation == 90) {
          // iPad <= 7.0 WebView Landscape
          viewportProperties.height = '0';

        } else if (version == 7) {
          // iPad <= 7.0 WebView Portait
          viewportProperties.height = DEVICE_HEIGHT;
        }
      } else {
        // iPad <= 6.1 Browser
        if (version < 7) {
          viewportProperties.height = '0';
        }
      }
    }

  } else if (p.isIOS()) {
    // iPhone

    if (p.isWebView()) {
      // iPhone WebView

      if (version > 7) {
        // iPhone >= 7.1 WebView
        delete viewportProperties.width;

      } else if (version < 7) {
        // iPhone <= 6.1 WebView
        // if height was set it needs to get removed with this hack for <= 6.1
        if (initHeight) viewportProperties.height = '0';

      } else if (version == 7) {
        //iPhone == 7.0 WebView
        viewportProperties.height = DEVICE_HEIGHT;
      }

    } else {
      // iPhone Browser

      if (version < 7) {
        // iPhone <= 6.1 Browser
        // if height was set it needs to get removed with this hack for <= 6.1
        if (initHeight) viewportProperties.height = '0';
      }
    }

  }

  // only update the viewport tag if there was a change
  if (initWidth !== viewportProperties.width || initHeight !== viewportProperties.height) {
    viewportTagUpdate();
  }
}

function viewportTagUpdate() {
  var key, props = [];
  for (key in viewportProperties) {
    if (viewportProperties[key]) {
      props.push(key + (viewportProperties[key] == '_' ? '' : '=' + viewportProperties[key]));
    }
  }

  viewportTag.content = props.join(', ');
}

ionic.Platform.ready(function() {
  viewportLoadTag();

  window.addEventListener("orientationchange", function() {
    setTimeout(viewportUpdate, 1000);
  }, false);
});

(function(ionic) {
'use strict';
  ionic.views.View = function() {
    this.initialize.apply(this, arguments);
  };

  ionic.views.View.inherit = ionic.inherit;

  ionic.extend(ionic.views.View.prototype, {
    initialize: function() {}
  });

})(window.ionic);

/*
 * Scroller
 * http://github.com/zynga/scroller
 *
 * Copyright 2011, Zynga Inc.
 * Licensed under the MIT License.
 * https://raw.github.com/zynga/scroller/master/MIT-LICENSE.txt
 *
 * Based on the work of: Unify Project (unify-project.org)
 * http://unify-project.org
 * Copyright 2011, Deutsche Telekom AG
 * License: MIT + Apache (V2)
 */

/* jshint eqnull: true */

/**
 * Generic animation class with support for dropped frames both optional easing and duration.
 *
 * Optional duration is useful when the lifetime is defined by another condition than time
 * e.g. speed of an animating object, etc.
 *
 * Dropped frame logic allows to keep using the same updater logic independent from the actual
 * rendering. This eases a lot of cases where it might be pretty complex to break down a state
 * based on the pure time difference.
 */
var zyngaCore = { effect: {} };
(function(global) {
  var time = Date.now || function() {
    return +new Date();
  };
  var desiredFrames = 60;
  var millisecondsPerSecond = 1000;
  var running = {};
  var counter = 1;

  zyngaCore.effect.Animate = {

    /**
     * A requestAnimationFrame wrapper / polyfill.
     *
     * @param callback {Function} The callback to be invoked before the next repaint.
     * @param root {HTMLElement} The root element for the repaint
     */
    requestAnimationFrame: (function() {

      // Check for request animation Frame support
      var requestFrame = global.requestAnimationFrame || global.webkitRequestAnimationFrame || global.mozRequestAnimationFrame || global.oRequestAnimationFrame;
      var isNative = !!requestFrame;

      if (requestFrame && !/requestAnimationFrame\(\)\s*\{\s*\[native code\]\s*\}/i.test(requestFrame.toString())) {
        isNative = false;
      }

      if (isNative) {
        return function(callback, root) {
          requestFrame(callback, root);
        };
      }

      var TARGET_FPS = 60;
      var requests = {};
      var requestCount = 0;
      var rafHandle = 1;
      var intervalHandle = null;
      var lastActive = +new Date();

      return function(callback) {
        var callbackHandle = rafHandle++;

        // Store callback
        requests[callbackHandle] = callback;
        requestCount++;

        // Create timeout at first request
        if (intervalHandle === null) {

          intervalHandle = setInterval(function() {

            var time = +new Date();
            var currentRequests = requests;

            // Reset data structure before executing callbacks
            requests = {};
            requestCount = 0;

            for(var key in currentRequests) {
              if (currentRequests.hasOwnProperty(key)) {
                currentRequests[key](time);
                lastActive = time;
              }
            }

            // Disable the timeout when nothing happens for a certain
            // period of time
            if (time - lastActive > 2500) {
              clearInterval(intervalHandle);
              intervalHandle = null;
            }

          }, 1000 / TARGET_FPS);
        }

        return callbackHandle;
      };

    })(),


    /**
     * Stops the given animation.
     *
     * @param id {Integer} Unique animation ID
     * @return {Boolean} Whether the animation was stopped (aka, was running before)
     */
    stop: function(id) {
      var cleared = running[id] != null;
      if (cleared) {
        running[id] = null;
      }

      return cleared;
    },


    /**
     * Whether the given animation is still running.
     *
     * @param id {Integer} Unique animation ID
     * @return {Boolean} Whether the animation is still running
     */
    isRunning: function(id) {
      return running[id] != null;
    },


    /**
     * Start the animation.
     *
     * @param stepCallback {Function} Pointer to function which is executed on every step.
     *   Signature of the method should be `function(percent, now, virtual) { return continueWithAnimation; }`
     * @param verifyCallback {Function} Executed before every animation step.
     *   Signature of the method should be `function() { return continueWithAnimation; }`
     * @param completedCallback {Function}
     *   Signature of the method should be `function(droppedFrames, finishedAnimation) {}`
     * @param duration {Integer} Milliseconds to run the animation
     * @param easingMethod {Function} Pointer to easing function
     *   Signature of the method should be `function(percent) { return modifiedValue; }`
     * @param root {Element} Render root, when available. Used for internal
     *   usage of requestAnimationFrame.
     * @return {Integer} Identifier of animation. Can be used to stop it any time.
     */
    start: function(stepCallback, verifyCallback, completedCallback, duration, easingMethod, root) {

      var start = time();
      var lastFrame = start;
      var percent = 0;
      var dropCounter = 0;
      var id = counter++;

      if (!root) {
        root = document.body;
      }

      // Compacting running db automatically every few new animations
      if (id % 20 === 0) {
        var newRunning = {};
        for (var usedId in running) {
          newRunning[usedId] = true;
        }
        running = newRunning;
      }

      // This is the internal step method which is called every few milliseconds
      var step = function(virtual) {

        // Normalize virtual value
        var render = virtual !== true;

        // Get current time
        var now = time();

        // Verification is executed before next animation step
        if (!running[id] || (verifyCallback && !verifyCallback(id))) {

          running[id] = null;
          completedCallback && completedCallback(desiredFrames - (dropCounter / ((now - start) / millisecondsPerSecond)), id, false);
          return;

        }

        // For the current rendering to apply let's update omitted steps in memory.
        // This is important to bring internal state variables up-to-date with progress in time.
        if (render) {

          var droppedFrames = Math.round((now - lastFrame) / (millisecondsPerSecond / desiredFrames)) - 1;
          for (var j = 0; j < Math.min(droppedFrames, 4); j++) {
            step(true);
            dropCounter++;
          }

        }

        // Compute percent value
        if (duration) {
          percent = (now - start) / duration;
          if (percent > 1) {
            percent = 1;
          }
        }

        // Execute step callback, then...
        var value = easingMethod ? easingMethod(percent) : percent;
        if ((stepCallback(value, now, render) === false || percent === 1) && render) {
          running[id] = null;
          completedCallback && completedCallback(desiredFrames - (dropCounter / ((now - start) / millisecondsPerSecond)), id, percent === 1 || duration == null);
        } else if (render) {
          lastFrame = now;
          zyngaCore.effect.Animate.requestAnimationFrame(step, root);
        }
      };

      // Mark as running
      running[id] = true;

      // Init first step
      zyngaCore.effect.Animate.requestAnimationFrame(step, root);

      // Return unique animation ID
      return id;
    }
  };
})(window);

/*
 * Scroller
 * http://github.com/zynga/scroller
 *
 * Copyright 2011, Zynga Inc.
 * Licensed under the MIT License.
 * https://raw.github.com/zynga/scroller/master/MIT-LICENSE.txt
 *
 * Based on the work of: Unify Project (unify-project.org)
 * http://unify-project.org
 * Copyright 2011, Deutsche Telekom AG
 * License: MIT + Apache (V2)
 */

(function(ionic) {
  var NOOP = function(){};

  // Easing Equations (c) 2003 Robert Penner, all rights reserved.
  // Open source under the BSD License.

  /**
   * @param pos {Number} position between 0 (start of effect) and 1 (end of effect)
  **/
  var easeOutCubic = function(pos) {
    return (Math.pow((pos - 1), 3) + 1);
  };

  /**
   * @param pos {Number} position between 0 (start of effect) and 1 (end of effect)
  **/
  var easeInOutCubic = function(pos) {
    if ((pos /= 0.5) < 1) {
      return 0.5 * Math.pow(pos, 3);
    }

    return 0.5 * (Math.pow((pos - 2), 3) + 2);
  };


/**
 * ionic.views.Scroll
 * A powerful scroll view with support for bouncing, pull to refresh, and paging.
 * @param   {Object}        options options for the scroll view
 * @class A scroll view system
 * @memberof ionic.views
 */
ionic.views.Scroll = ionic.views.View.inherit({
  initialize: function(options) {
    var self = this;

    self.__container = options.el;
    self.__content = options.el.firstElementChild;

    //Remove any scrollTop attached to these elements; they are virtual scroll now
    //This also stops on-load-scroll-to-window.location.hash that the browser does
    setTimeout(function() {
      if (self.__container && self.__content) {
        self.__container.scrollTop = 0;
        self.__content.scrollTop = 0;
      }
    });

    self.options = {

      /** Disable scrolling on x-axis by default */
      scrollingX: false,
      scrollbarX: true,

      /** Enable scrolling on y-axis */
      scrollingY: true,
      scrollbarY: true,

      startX: 0,
      startY: 0,

      /** The amount to dampen mousewheel events */
      wheelDampen: 6,

      /** The minimum size the scrollbars scale to while scrolling */
      minScrollbarSizeX: 5,
      minScrollbarSizeY: 5,

      /** Scrollbar fading after scrolling */
      scrollbarsFade: true,
      scrollbarFadeDelay: 300,
      /** The initial fade delay when the pane is resized or initialized */
      scrollbarResizeFadeDelay: 1000,

      /** Enable animations for deceleration, snap back, zooming and scrolling */
      animating: true,

      /** duration for animations triggered by scrollTo/zoomTo */
      animationDuration: 250,

      /** The velocity required to make the scroll view "slide" after touchend */
      decelVelocityThreshold: 4,

      /** The velocity required to make the scroll view "slide" after touchend when using paging */
      decelVelocityThresholdPaging: 4,

      /** Enable bouncing (content can be slowly moved outside and jumps back after releasing) */
      bouncing: true,

      /** Enable locking to the main axis if user moves only slightly on one of them at start */
      locking: true,

      /** Enable pagination mode (switching between full page content panes) */
      paging: false,

      /** Enable snapping of content to a configured pixel grid */
      snapping: false,

      /** Enable zooming of content via API, fingers and mouse wheel */
      zooming: false,

      /** Minimum zoom level */
      minZoom: 0.5,

      /** Maximum zoom level */
      maxZoom: 3,

      /** Multiply or decrease scrolling speed **/
      speedMultiplier: 1,

      deceleration: 0.97,

      /** Whether to prevent default on a scroll operation to capture drag events **/
      preventDefault: false,

      /** Callback that is fired on the later of touch end or deceleration end,
        provided that another scrolling action has not begun. Used to know
        when to fade out a scrollbar. */
      scrollingComplete: NOOP,

      /** This configures the amount of change applied to deceleration when reaching boundaries  **/
      penetrationDeceleration: 0.03,

      /** This configures the amount of change applied to acceleration when reaching boundaries  **/
      penetrationAcceleration: 0.08,

      // The ms interval for triggering scroll events
      scrollEventInterval: 10,

      freeze: false,

      getContentWidth: function() {
        return Math.max(self.__content.scrollWidth, self.__content.offsetWidth);
      },
      getContentHeight: function() {
        return Math.max(self.__content.scrollHeight, self.__content.offsetHeight + (self.__content.offsetTop * 2));
      }
    };

    for (var key in options) {
      self.options[key] = options[key];
    }

    self.hintResize = ionic.debounce(function() {
      self.resize();
    }, 1000, true);

    self.onScroll = function() {

      if (!ionic.scroll.isScrolling) {
        setTimeout(self.setScrollStart, 50);
      } else {
        clearTimeout(self.scrollTimer);
        self.scrollTimer = setTimeout(self.setScrollStop, 80);
      }

    };

    self.freeze = function(shouldFreeze) {
      if (arguments.length) {
        self.options.freeze = shouldFreeze;
      }
      return self.options.freeze;
    };

    // We can just use the standard freeze pop in our mouth
    self.freezeShut = self.freeze;

    self.setScrollStart = function() {
      ionic.scroll.isScrolling = Math.abs(ionic.scroll.lastTop - self.__scrollTop) > 1;
      clearTimeout(self.scrollTimer);
      self.scrollTimer = setTimeout(self.setScrollStop, 80);
    };

    self.setScrollStop = function() {
      ionic.scroll.isScrolling = false;
      ionic.scroll.lastTop = self.__scrollTop;
    };

    self.triggerScrollEvent = ionic.throttle(function() {
      self.onScroll();
      ionic.trigger('scroll', {
        scrollTop: self.__scrollTop,
        scrollLeft: self.__scrollLeft,
        target: self.__container
      });
    }, self.options.scrollEventInterval);

    self.triggerScrollEndEvent = function() {
      ionic.trigger('scrollend', {
        scrollTop: self.__scrollTop,
        scrollLeft: self.__scrollLeft,
        target: self.__container
      });
    };

    self.__scrollLeft = self.options.startX;
    self.__scrollTop = self.options.startY;

    // Get the render update function, initialize event handlers,
    // and calculate the size of the scroll container
    self.__callback = self.getRenderFn();
    self.__initEventHandlers();
    self.__createScrollbars();

  },

  run: function() {
    this.resize();

    // Fade them out
    this.__fadeScrollbars('out', this.options.scrollbarResizeFadeDelay);
  },



  /*
  ---------------------------------------------------------------------------
    INTERNAL FIELDS :: STATUS
  ---------------------------------------------------------------------------
  */

  /** Whether only a single finger is used in touch handling */
  __isSingleTouch: false,

  /** Whether a touch event sequence is in progress */
  __isTracking: false,

  /** Whether a deceleration animation went to completion. */
  __didDecelerationComplete: false,

  /**
   * Whether a gesture zoom/rotate event is in progress. Activates when
   * a gesturestart event happens. This has higher priority than dragging.
   */
  __isGesturing: false,

  /**
   * Whether the user has moved by such a distance that we have enabled
   * dragging mode. Hint: It's only enabled after some pixels of movement to
   * not interrupt with clicks etc.
   */
  __isDragging: false,

  /**
   * Not touching and dragging anymore, and smoothly animating the
   * touch sequence using deceleration.
   */
  __isDecelerating: false,

  /**
   * Smoothly animating the currently configured change
   */
  __isAnimating: false,



  /*
  ---------------------------------------------------------------------------
    INTERNAL FIELDS :: DIMENSIONS
  ---------------------------------------------------------------------------
  */

  /** Available outer left position (from document perspective) */
  __clientLeft: 0,

  /** Available outer top position (from document perspective) */
  __clientTop: 0,

  /** Available outer width */
  __clientWidth: 0,

  /** Available outer height */
  __clientHeight: 0,

  /** Outer width of content */
  __contentWidth: 0,

  /** Outer height of content */
  __contentHeight: 0,

  /** Snapping width for content */
  __snapWidth: 100,

  /** Snapping height for content */
  __snapHeight: 100,

  /** Height to assign to refresh area */
  __refreshHeight: null,

  /** Whether the refresh process is enabled when the event is released now */
  __refreshActive: false,

  /** Callback to execute on activation. This is for signalling the user about a refresh is about to happen when he release */
  __refreshActivate: null,

  /** Callback to execute on deactivation. This is for signalling the user about the refresh being cancelled */
  __refreshDeactivate: null,

  /** Callback to execute to start the actual refresh. Call {@link #refreshFinish} when done */
  __refreshStart: null,

  /** Zoom level */
  __zoomLevel: 1,

  /** Scroll position on x-axis */
  __scrollLeft: 0,

  /** Scroll position on y-axis */
  __scrollTop: 0,

  /** Maximum allowed scroll position on x-axis */
  __maxScrollLeft: 0,

  /** Maximum allowed scroll position on y-axis */
  __maxScrollTop: 0,

  /* Scheduled left position (final position when animating) */
  __scheduledLeft: 0,

  /* Scheduled top position (final position when animating) */
  __scheduledTop: 0,

  /* Scheduled zoom level (final scale when animating) */
  __scheduledZoom: 0,



  /*
  ---------------------------------------------------------------------------
    INTERNAL FIELDS :: LAST POSITIONS
  ---------------------------------------------------------------------------
  */

  /** Left position of finger at start */
  __lastTouchLeft: null,

  /** Top position of finger at start */
  __lastTouchTop: null,

  /** Timestamp of last move of finger. Used to limit tracking range for deceleration speed. */
  __lastTouchMove: null,

  /** List of positions, uses three indexes for each state: left, top, timestamp */
  __positions: null,



  /*
  ---------------------------------------------------------------------------
    INTERNAL FIELDS :: DECELERATION SUPPORT
  ---------------------------------------------------------------------------
  */

  /** Minimum left scroll position during deceleration */
  __minDecelerationScrollLeft: null,

  /** Minimum top scroll position during deceleration */
  __minDecelerationScrollTop: null,

  /** Maximum left scroll position during deceleration */
  __maxDecelerationScrollLeft: null,

  /** Maximum top scroll position during deceleration */
  __maxDecelerationScrollTop: null,

  /** Current factor to modify horizontal scroll position with on every step */
  __decelerationVelocityX: null,

  /** Current factor to modify vertical scroll position with on every step */
  __decelerationVelocityY: null,


  /** the browser-specific property to use for transforms */
  __transformProperty: null,
  __perspectiveProperty: null,

  /** scrollbar indicators */
  __indicatorX: null,
  __indicatorY: null,

  /** Timeout for scrollbar fading */
  __scrollbarFadeTimeout: null,

  /** whether we've tried to wait for size already */
  __didWaitForSize: null,
  __sizerTimeout: null,

  __initEventHandlers: function() {
    var self = this;

    // Event Handler
    var container = self.__container;

    // save height when scroll view is shrunk so we don't need to reflow
    var scrollViewOffsetHeight;

    /**
     * Shrink the scroll view when the keyboard is up if necessary and if the
     * focused input is below the bottom of the shrunk scroll view, scroll it
     * into view.
     */
    self.scrollChildIntoView = function(e) {
      //console.log("scrollChildIntoView at: " + Date.now());

      // D
      var scrollBottomOffsetToTop = container.getBoundingClientRect().bottom;
      // D - A
      scrollViewOffsetHeight = container.offsetHeight;
      var alreadyShrunk = self.isShrunkForKeyboard;

      var isModal = container.parentNode.classList.contains('modal');
      // 680px is when the media query for 60% modal width kicks in
      var isInsetModal = isModal && window.innerWidth >= 680;

     /*
      *  _______
      * |---A---| <- top of scroll view
      * |       |
      * |---B---| <- keyboard
      * |   C   | <- input
      * |---D---| <- initial bottom of scroll view
      * |___E___| <- bottom of viewport
      *
      *  All commented calculations relative to the top of the viewport (ie E
      *  is the viewport height, not 0)
      */
      if (!alreadyShrunk) {
        // shrink scrollview so we can actually scroll if the input is hidden
        // if it isn't shrink so we can scroll to inputs under the keyboard
        // inset modals won't shrink on Android on their own when the keyboard appears
        if ( ionic.Platform.isIOS() || ionic.Platform.isFullScreen || isInsetModal ) {
          // if there are things below the scroll view account for them and
          // subtract them from the keyboard height when resizing
          // E - D                         E                         D
          var scrollBottomOffsetToBottom = e.detail.viewportHeight - scrollBottomOffsetToTop;

          // 0 or D - B if D > B           E - B                     E - D
          var keyboardOffset = Math.max(0, e.detail.keyboardHeight - scrollBottomOffsetToBottom);

          ionic.requestAnimationFrame(function(){
            // D - A or B - A if D > B       D - A             max(0, D - B)
            scrollViewOffsetHeight = scrollViewOffsetHeight - keyboardOffset;
            container.style.height = scrollViewOffsetHeight + "px";
            container.style.overflow = "visible";

            //update scroll view
            self.resize();
          });
        }

        self.isShrunkForKeyboard = true;
      }

      /*
       *  _______
       * |---A---| <- top of scroll view
       * |   *   | <- where we want to scroll to
       * |--B-D--| <- keyboard, bottom of scroll view
       * |   C   | <- input
       * |       |
       * |___E___| <- bottom of viewport
       *
       *  All commented calculations relative to the top of the viewport (ie E
       *  is the viewport height, not 0)
       */
      // if the element is positioned under the keyboard scroll it into view
      if (e.detail.isElementUnderKeyboard) {

        ionic.requestAnimationFrame(function(){
          container.scrollTop = 0;
          // update D if we shrunk
          if (self.isShrunkForKeyboard && !alreadyShrunk) {
            scrollBottomOffsetToTop = container.getBoundingClientRect().bottom;
          }

          // middle of the scrollview, this is where we want to scroll to
          // (D - A) / 2
          var scrollMidpointOffset = scrollViewOffsetHeight * 0.5;
          //console.log("container.offsetHeight: " + scrollViewOffsetHeight);

          // middle of the input we want to scroll into view
          // C
          var inputMidpoint = ((e.detail.elementBottom + e.detail.elementTop) / 2);

          // distance from middle of input to the bottom of the scroll view
          // C - D                                C               D
          var inputMidpointOffsetToScrollBottom = inputMidpoint - scrollBottomOffsetToTop;

          //C - D + (D - A)/2          C - D                     (D - A)/ 2
          var scrollTop = inputMidpointOffsetToScrollBottom + scrollMidpointOffset;

          if ( scrollTop > 0) {
            if (ionic.Platform.isIOS()) ionic.tap.cloneFocusedInput(container, self);
            self.scrollBy(0, scrollTop, true);
            self.onScroll();
          }
        });
      }

      // Only the first scrollView parent of the element that broadcasted this event
      // (the active element that needs to be shown) should receive this event
      e.stopPropagation();
    };

    self.resetScrollView = function() {
      //return scrollview to original height once keyboard has hidden
      if ( self.isShrunkForKeyboard ) {
        self.isShrunkForKeyboard = false;
        container.style.height = "";
        container.style.overflow = "";
      }
      self.resize();
    };

    //Broadcasted when keyboard is shown on some platforms.
    //See js/utils/keyboard.js
    container.addEventListener('scrollChildIntoView', self.scrollChildIntoView);

    // Listen on document because container may not have had the last
    // keyboardActiveElement, for example after closing a modal with a focused
    // input and returning to a previously resized scroll view in an ion-content.
    // Since we can only resize scroll views that are currently visible, just resize
    // the current scroll view when the keyboard is closed.
    document.addEventListener('resetScrollView', self.resetScrollView);

    function getEventTouches(e) {
      return e.touches && e.touches.length ? e.touches : [{
        pageX: e.pageX,
        pageY: e.pageY
      }];
    }

    self.touchStart = function(e) {
      self.startCoordinates = ionic.tap.pointerCoord(e);

      if ( ionic.tap.ignoreScrollStart(e) ) {
        return;
      }

      self.__isDown = true;

      if ( ionic.tap.containsOrIsTextInput(e.target) || e.target.tagName === 'SELECT' ) {
        // do not start if the target is a text input
        // if there is a touchmove on this input, then we can start the scroll
        self.__hasStarted = false;
        return;
      }

      self.__isSelectable = true;
      self.__enableScrollY = true;
      self.__hasStarted = true;
      self.doTouchStart(getEventTouches(e), e.timeStamp);
      e.preventDefault();
    };

    self.touchMove = function(e) {
      if (self.options.freeze || !self.__isDown ||
        (!self.__isDown && e.defaultPrevented) ||
        (e.target.tagName === 'TEXTAREA' && e.target.parentElement.querySelector(':focus')) ) {
        return;
      }

      if ( !self.__hasStarted && ( ionic.tap.containsOrIsTextInput(e.target) || e.target.tagName === 'SELECT' ) ) {
        // the target is a text input and scroll has started
        // since the text input doesn't start on touchStart, do it here
        self.__hasStarted = true;
        self.doTouchStart(getEventTouches(e), e.timeStamp);
        e.preventDefault();
        return;
      }

      if (self.startCoordinates) {
        // we have start coordinates, so get this touch move's current coordinates
        var currentCoordinates = ionic.tap.pointerCoord(e);

        if ( self.__isSelectable &&
            ionic.tap.isTextInput(e.target) &&
            Math.abs(self.startCoordinates.x - currentCoordinates.x) > 20 ) {
          // user slid the text input's caret on its x axis, disable any future y scrolling
          self.__enableScrollY = false;
          self.__isSelectable = true;
        }

        if ( self.__enableScrollY && Math.abs(self.startCoordinates.y - currentCoordinates.y) > 10 ) {
          // user scrolled the entire view on the y axis
          // disabled being able to select text on an input
          // hide the input which has focus, and show a cloned one that doesn't have focus
          self.__isSelectable = false;
          ionic.tap.cloneFocusedInput(container, self);
        }
      }

      self.doTouchMove(getEventTouches(e), e.timeStamp, e.scale);
      self.__isDown = true;
    };

    self.touchMoveBubble = function(e) {
      if(self.__isDown && self.options.preventDefault) {
        e.preventDefault();
      }
    };

    self.touchEnd = function(e) {
      if (!self.__isDown) return;

      self.doTouchEnd(e, e.timeStamp);
      self.__isDown = false;
      self.__hasStarted = false;
      self.__isSelectable = true;
      self.__enableScrollY = true;

      if ( !self.__isDragging && !self.__isDecelerating && !self.__isAnimating ) {
        ionic.tap.removeClonedInputs(container, self);
      }
    };

    self.mouseWheel = ionic.animationFrameThrottle(function(e) {
      var scrollParent = ionic.DomUtil.getParentOrSelfWithClass(e.target, 'ionic-scroll');
      if (!self.options.freeze && scrollParent === self.__container) {

        self.hintResize();
        self.scrollBy(
          (e.wheelDeltaX || e.deltaX || 0) / self.options.wheelDampen,
          (-e.wheelDeltaY || e.deltaY || 0) / self.options.wheelDampen
        );

        self.__fadeScrollbars('in');
        clearTimeout(self.__wheelHideBarTimeout);
        self.__wheelHideBarTimeout = setTimeout(function() {
          self.__fadeScrollbars('out');
        }, 100);
      }
    });

    if ('ontouchstart' in window) {
      // Touch Events
      container.addEventListener("touchstart", self.touchStart, false);
      if(self.options.preventDefault) container.addEventListener("touchmove", self.touchMoveBubble, false);
      document.addEventListener("touchmove", self.touchMove, false);
      document.addEventListener("touchend", self.touchEnd, false);
      document.addEventListener("touchcancel", self.touchEnd, false);
      document.addEventListener("wheel", self.mouseWheel, false);

    } else if (window.navigator.pointerEnabled) {
      // Pointer Events
      container.addEventListener("pointerdown", self.touchStart, false);
      if(self.options.preventDefault) container.addEventListener("pointermove", self.touchMoveBubble, false);
      document.addEventListener("pointermove", self.touchMove, false);
      document.addEventListener("pointerup", self.touchEnd, false);
      document.addEventListener("pointercancel", self.touchEnd, false);
      document.addEventListener("wheel", self.mouseWheel, false);

    } else if (window.navigator.msPointerEnabled) {
      // IE10, WP8 (Pointer Events)
      container.addEventListener("MSPointerDown", self.touchStart, false);
      if(self.options.preventDefault) container.addEventListener("MSPointerMove", self.touchMoveBubble, false);
      document.addEventListener("MSPointerMove", self.touchMove, false);
      document.addEventListener("MSPointerUp", self.touchEnd, false);
      document.addEventListener("MSPointerCancel", self.touchEnd, false);
      document.addEventListener("wheel", self.mouseWheel, false);

    } else {
      // Mouse Events
      var mousedown = false;

      self.mouseDown = function(e) {
        if ( ionic.tap.ignoreScrollStart(e) || e.target.tagName === 'SELECT' ) {
          return;
        }
        self.doTouchStart(getEventTouches(e), e.timeStamp);

        if ( !ionic.tap.isTextInput(e.target) ) {
          e.preventDefault();
        }
        mousedown = true;
      };

      self.mouseMove = function(e) {
        if (self.options.freeze || !mousedown || (!mousedown && e.defaultPrevented)) {
          return;
        }

        self.doTouchMove(getEventTouches(e), e.timeStamp);

        mousedown = true;
      };

      self.mouseMoveBubble = function(e) {
        if (mousedown && self.options.preventDefault) {
          e.preventDefault();
        }
      };

      self.mouseUp = function(e) {
        if (!mousedown) {
          return;
        }

        self.doTouchEnd(e, e.timeStamp);

        mousedown = false;
      };

      container.addEventListener("mousedown", self.mouseDown, false);
      if(self.options.preventDefault) container.addEventListener("mousemove", self.mouseMoveBubble, false);
      document.addEventListener("mousemove", self.mouseMove, false);
      document.addEventListener("mouseup", self.mouseUp, false);
      document.addEventListener('mousewheel', self.mouseWheel, false);
      document.addEventListener('wheel', self.mouseWheel, false);
    }
  },

  __cleanup: function() {
    var self = this;
    var container = self.__container;

    container.removeEventListener('touchstart', self.touchStart);
    container.removeEventListener('touchmove', self.touchMoveBubble);
    document.removeEventListener('touchmove', self.touchMove);
    document.removeEventListener('touchend', self.touchEnd);
    document.removeEventListener('touchcancel', self.touchEnd);

    container.removeEventListener("pointerdown", self.touchStart);
    container.removeEventListener("pointermove", self.touchMoveBubble);
    document.removeEventListener("pointermove", self.touchMove);
    document.removeEventListener("pointerup", self.touchEnd);
    document.removeEventListener("pointercancel", self.touchEnd);

    container.removeEventListener("MSPointerDown", self.touchStart);
    container.removeEventListener("MSPointerMove", self.touchMoveBubble);
    document.removeEventListener("MSPointerMove", self.touchMove);
    document.removeEventListener("MSPointerUp", self.touchEnd);
    document.removeEventListener("MSPointerCancel", self.touchEnd);

    container.removeEventListener("mousedown", self.mouseDown);
    container.removeEventListener("mousemove", self.mouseMoveBubble);
    document.removeEventListener("mousemove", self.mouseMove);
    document.removeEventListener("mouseup", self.mouseUp);
    document.removeEventListener('mousewheel', self.mouseWheel);
    document.removeEventListener('wheel', self.mouseWheel);

    container.removeEventListener('scrollChildIntoView', self.scrollChildIntoView);
    document.removeEventListener('resetScrollView', self.resetScrollView);

    ionic.tap.removeClonedInputs(container, self);

    delete self.__container;
    delete self.__content;
    delete self.__indicatorX;
    delete self.__indicatorY;
    delete self.options.el;

    self.__callback = self.scrollChildIntoView = self.resetScrollView = NOOP;

    self.mouseMove = self.mouseDown = self.mouseUp = self.mouseWheel =
      self.touchStart = self.touchMove = self.touchEnd = self.touchCancel = NOOP;

    self.resize = self.scrollTo = self.zoomTo =
      self.__scrollingComplete = NOOP;
    container = null;
  },

  /** Create a scroll bar div with the given direction **/
  __createScrollbar: function(direction) {
    var bar = document.createElement('div'),
      indicator = document.createElement('div');

    indicator.className = 'scroll-bar-indicator scroll-bar-fade-out';

    if (direction == 'h') {
      bar.className = 'scroll-bar scroll-bar-h';
    } else {
      bar.className = 'scroll-bar scroll-bar-v';
    }

    bar.appendChild(indicator);
    return bar;
  },

  __createScrollbars: function() {
    var self = this;
    var indicatorX, indicatorY;

    if (self.options.scrollingX) {
      indicatorX = {
        el: self.__createScrollbar('h'),
        sizeRatio: 1
      };
      indicatorX.indicator = indicatorX.el.children[0];

      if (self.options.scrollbarX) {
        self.__container.appendChild(indicatorX.el);
      }
      self.__indicatorX = indicatorX;
    }

    if (self.options.scrollingY) {
      indicatorY = {
        el: self.__createScrollbar('v'),
        sizeRatio: 1
      };
      indicatorY.indicator = indicatorY.el.children[0];

      if (self.options.scrollbarY) {
        self.__container.appendChild(indicatorY.el);
      }
      self.__indicatorY = indicatorY;
    }
  },

  __resizeScrollbars: function() {
    var self = this;

    // Update horiz bar
    if (self.__indicatorX) {
      var width = Math.max(Math.round(self.__clientWidth * self.__clientWidth / (self.__contentWidth)), 20);
      if (width > self.__contentWidth) {
        width = 0;
      }
      if (width !== self.__indicatorX.size) {
        ionic.requestAnimationFrame(function(){
          self.__indicatorX.indicator.style.width = width + 'px';
        });
      }
      self.__indicatorX.size = width;
      self.__indicatorX.minScale = self.options.minScrollbarSizeX / width;
      self.__indicatorX.maxPos = self.__clientWidth - width;
      self.__indicatorX.sizeRatio = self.__maxScrollLeft ? self.__indicatorX.maxPos / self.__maxScrollLeft : 1;
    }

    // Update vert bar
    if (self.__indicatorY) {
      var height = Math.max(Math.round(self.__clientHeight * self.__clientHeight / (self.__contentHeight)), 20);
      if (height > self.__contentHeight) {
        height = 0;
      }
      if (height !== self.__indicatorY.size) {
        ionic.requestAnimationFrame(function(){
          self.__indicatorY && (self.__indicatorY.indicator.style.height = height + 'px');
        });
      }
      self.__indicatorY.size = height;
      self.__indicatorY.minScale = self.options.minScrollbarSizeY / height;
      self.__indicatorY.maxPos = self.__clientHeight - height;
      self.__indicatorY.sizeRatio = self.__maxScrollTop ? self.__indicatorY.maxPos / self.__maxScrollTop : 1;
    }
  },

  /**
   * Move and scale the scrollbars as the page scrolls.
   */
  __repositionScrollbars: function() {
    var self = this,
        heightScale, widthScale,
        widthDiff, heightDiff,
        x, y,
        xstop = 0, ystop = 0;

    if (self.__indicatorX) {
      // Handle the X scrollbar

      // Don't go all the way to the right if we have a vertical scrollbar as well
      if (self.__indicatorY) xstop = 10;

      x = Math.round(self.__indicatorX.sizeRatio * self.__scrollLeft) || 0;

      // The the difference between the last content X position, and our overscrolled one
      widthDiff = self.__scrollLeft - (self.__maxScrollLeft - xstop);

      if (self.__scrollLeft < 0) {

        widthScale = Math.max(self.__indicatorX.minScale,
            (self.__indicatorX.size - Math.abs(self.__scrollLeft)) / self.__indicatorX.size);

        // Stay at left
        x = 0;

        // Make sure scale is transformed from the left/center origin point
        self.__indicatorX.indicator.style[self.__transformOriginProperty] = 'left center';
      } else if (widthDiff > 0) {

        widthScale = Math.max(self.__indicatorX.minScale,
            (self.__indicatorX.size - widthDiff) / self.__indicatorX.size);

        // Stay at the furthest x for the scrollable viewport
        x = self.__indicatorX.maxPos - xstop;

        // Make sure scale is transformed from the right/center origin point
        self.__indicatorX.indicator.style[self.__transformOriginProperty] = 'right center';

      } else {

        // Normal motion
        x = Math.min(self.__maxScrollLeft, Math.max(0, x));
        widthScale = 1;

      }

      var translate3dX = 'translate3d(' + x + 'px, 0, 0) scaleX(' + widthScale + ')';
      if (self.__indicatorX.transformProp !== translate3dX) {
        self.__indicatorX.indicator.style[self.__transformProperty] = translate3dX;
        self.__indicatorX.transformProp = translate3dX;
      }
    }

    if (self.__indicatorY) {

      y = Math.round(self.__indicatorY.sizeRatio * self.__scrollTop) || 0;

      // Don't go all the way to the right if we have a vertical scrollbar as well
      if (self.__indicatorX) ystop = 10;

      heightDiff = self.__scrollTop - (self.__maxScrollTop - ystop);

      if (self.__scrollTop < 0) {

        heightScale = Math.max(self.__indicatorY.minScale, (self.__indicatorY.size - Math.abs(self.__scrollTop)) / self.__indicatorY.size);

        // Stay at top
        y = 0;

        // Make sure scale is transformed from the center/top origin point
        if (self.__indicatorY.originProp !== 'center top') {
          self.__indicatorY.indicator.style[self.__transformOriginProperty] = 'center top';
          self.__indicatorY.originProp = 'center top';
        }

      } else if (heightDiff > 0) {

        heightScale = Math.max(self.__indicatorY.minScale, (self.__indicatorY.size - heightDiff) / self.__indicatorY.size);

        // Stay at bottom of scrollable viewport
        y = self.__indicatorY.maxPos - ystop;

        // Make sure scale is transformed from the center/bottom origin point
        if (self.__indicatorY.originProp !== 'center bottom') {
          self.__indicatorY.indicator.style[self.__transformOriginProperty] = 'center bottom';
          self.__indicatorY.originProp = 'center bottom';
        }

      } else {

        // Normal motion
        y = Math.min(self.__maxScrollTop, Math.max(0, y));
        heightScale = 1;

      }

      var translate3dY = 'translate3d(0,' + y + 'px, 0) scaleY(' + heightScale + ')';
      if (self.__indicatorY.transformProp !== translate3dY) {
        self.__indicatorY.indicator.style[self.__transformProperty] = translate3dY;
        self.__indicatorY.transformProp = translate3dY;
      }
    }
  },

  __fadeScrollbars: function(direction, delay) {
    var self = this;

    if (!self.options.scrollbarsFade) {
      return;
    }

    var className = 'scroll-bar-fade-out';

    if (self.options.scrollbarsFade === true) {
      clearTimeout(self.__scrollbarFadeTimeout);

      if (direction == 'in') {
        if (self.__indicatorX) { self.__indicatorX.indicator.classList.remove(className); }
        if (self.__indicatorY) { self.__indicatorY.indicator.classList.remove(className); }
      } else {
        self.__scrollbarFadeTimeout = setTimeout(function() {
          if (self.__indicatorX) { self.__indicatorX.indicator.classList.add(className); }
          if (self.__indicatorY) { self.__indicatorY.indicator.classList.add(className); }
        }, delay || self.options.scrollbarFadeDelay);
      }
    }
  },

  __scrollingComplete: function() {
    this.options.scrollingComplete();
    ionic.tap.removeClonedInputs(this.__container, this);
    this.__fadeScrollbars('out');
  },

  resize: function(continueScrolling) {
    var self = this;
    if (!self.__container || !self.options) return;

    // Update Scroller dimensions for changed content
    // Add padding to bottom of content
    self.setDimensions(
      self.__container.clientWidth,
      self.__container.clientHeight,
      self.options.getContentWidth(),
      self.options.getContentHeight(),
      continueScrolling
    );
  },
  /*
  ---------------------------------------------------------------------------
    PUBLIC API
  ---------------------------------------------------------------------------
  */

  getRenderFn: function() {
    var self = this;

    var content = self.__content;

    var docStyle = document.documentElement.style;

    var engine;
    if ('MozAppearance' in docStyle) {
      engine = 'gecko';
    } else if ('WebkitAppearance' in docStyle) {
      engine = 'webkit';
    } else if (typeof navigator.cpuClass === 'string') {
      engine = 'trident';
    }

    var vendorPrefix = {
      trident: 'ms',
      gecko: 'Moz',
      webkit: 'Webkit',
      presto: 'O'
    }[engine];

    var helperElem = document.createElement("div");
    var undef;

    var perspectiveProperty = vendorPrefix + "Perspective";
    var transformProperty = vendorPrefix + "Transform";
    var transformOriginProperty = vendorPrefix + 'TransformOrigin';

    self.__perspectiveProperty = transformProperty;
    self.__transformProperty = transformProperty;
    self.__transformOriginProperty = transformOriginProperty;

    if (helperElem.style[perspectiveProperty] !== undef) {

      return function(left, top, zoom, wasResize) {
        var translate3d = 'translate3d(' + (-left) + 'px,' + (-top) + 'px,0) scale(' + zoom + ')';
        if (translate3d !== self.contentTransform) {
          content.style[transformProperty] = translate3d;
          self.contentTransform = translate3d;
        }
        self.__repositionScrollbars();
        if (!wasResize) {
          self.triggerScrollEvent();
        }
      };

    } else if (helperElem.style[transformProperty] !== undef) {

      return function(left, top, zoom, wasResize) {
        content.style[transformProperty] = 'translate(' + (-left) + 'px,' + (-top) + 'px) scale(' + zoom + ')';
        self.__repositionScrollbars();
        if (!wasResize) {
          self.triggerScrollEvent();
        }
      };

    } else {

      return function(left, top, zoom, wasResize) {
        content.style.marginLeft = left ? (-left / zoom) + 'px' : '';
        content.style.marginTop = top ? (-top / zoom) + 'px' : '';
        content.style.zoom = zoom || '';
        self.__repositionScrollbars();
        if (!wasResize) {
          self.triggerScrollEvent();
        }
      };

    }
  },


  /**
   * Configures the dimensions of the client (outer) and content (inner) elements.
   * Requires the available space for the outer element and the outer size of the inner element.
   * All values which are falsy (null or zero etc.) are ignored and the old value is kept.
   *
   * @param clientWidth {Integer} Inner width of outer element
   * @param clientHeight {Integer} Inner height of outer element
   * @param contentWidth {Integer} Outer width of inner element
   * @param contentHeight {Integer} Outer height of inner element
   */
  setDimensions: function(clientWidth, clientHeight, contentWidth, contentHeight, continueScrolling) {
    var self = this;

    if (!clientWidth && !clientHeight && !contentWidth && !contentHeight) {
      // this scrollview isn't rendered, don't bother
      return;
    }

    // Only update values which are defined
    if (clientWidth === +clientWidth) {
      self.__clientWidth = clientWidth;
    }

    if (clientHeight === +clientHeight) {
      self.__clientHeight = clientHeight;
    }

    if (contentWidth === +contentWidth) {
      self.__contentWidth = contentWidth;
    }

    if (contentHeight === +contentHeight) {
      self.__contentHeight = contentHeight;
    }

    // Refresh maximums
    self.__computeScrollMax();
    self.__resizeScrollbars();

    // Refresh scroll position
    if (!continueScrolling) {
      self.scrollTo(self.__scrollLeft, self.__scrollTop, true, null, true);
    }

  },


  /**
   * Sets the client coordinates in relation to the document.
   *
   * @param left {Integer} Left position of outer element
   * @param top {Integer} Top position of outer element
   */
  setPosition: function(left, top) {
    this.__clientLeft = left || 0;
    this.__clientTop = top || 0;
  },


  /**
   * Configures the snapping (when snapping is active)
   *
   * @param width {Integer} Snapping width
   * @param height {Integer} Snapping height
   */
  setSnapSize: function(width, height) {
    this.__snapWidth = width;
    this.__snapHeight = height;
  },


  /**
   * Activates pull-to-refresh. A special zone on the top of the list to start a list refresh whenever
   * the user event is released during visibility of this zone. This was introduced by some apps on iOS like
   * the official Twitter client.
   *
   * @param height {Integer} Height of pull-to-refresh zone on top of rendered list
   * @param activateCallback {Function} Callback to execute on activation. This is for signalling the user about a refresh is about to happen when he release.
   * @param deactivateCallback {Function} Callback to execute on deactivation. This is for signalling the user about the refresh being cancelled.
   * @param startCallback {Function} Callback to execute to start the real async refresh action. Call {@link #finishPullToRefresh} after finish of refresh.
   * @param showCallback {Function} Callback to execute when the refresher should be shown. This is for showing the refresher during a negative scrollTop.
   * @param hideCallback {Function} Callback to execute when the refresher should be hidden. This is for hiding the refresher when it's behind the nav bar.
   * @param tailCallback {Function} Callback to execute just before the refresher returns to it's original state. This is for zooming out the refresher.
   * @param pullProgressCallback Callback to state the progress while pulling to refresh
   */
  activatePullToRefresh: function(height, refresherMethods) {
    var self = this;

    self.__refreshHeight = height;
    self.__refreshActivate = function() { ionic.requestAnimationFrame(refresherMethods.activate); };
    self.__refreshDeactivate = function() { ionic.requestAnimationFrame(refresherMethods.deactivate); };
    self.__refreshStart = function() { ionic.requestAnimationFrame(refresherMethods.start); };
    self.__refreshShow = function() { ionic.requestAnimationFrame(refresherMethods.show); };
    self.__refreshHide = function() { ionic.requestAnimationFrame(refresherMethods.hide); };
    self.__refreshTail = function() { ionic.requestAnimationFrame(refresherMethods.tail); };
    self.__refreshTailTime = 100;
    self.__minSpinTime = 600;
  },


  /**
   * Starts pull-to-refresh manually.
   */
  triggerPullToRefresh: function() {
    // Use publish instead of scrollTo to allow scrolling to out of boundary position
    // We don't need to normalize scrollLeft, zoomLevel, etc. here because we only y-scrolling when pull-to-refresh is enabled
    this.__publish(this.__scrollLeft, -this.__refreshHeight, this.__zoomLevel, true);

    var d = new Date();
    this.refreshStartTime = d.getTime();

    if (this.__refreshStart) {
      this.__refreshStart();
    }
  },


  /**
   * Signalizes that pull-to-refresh is finished.
   */
  finishPullToRefresh: function() {
    var self = this;
    // delay to make sure the spinner has a chance to spin for a split second before it's dismissed
    var d = new Date();
    var delay = 0;
    if (self.refreshStartTime + self.__minSpinTime > d.getTime()) {
      delay = self.refreshStartTime + self.__minSpinTime - d.getTime();
    }
    setTimeout(function() {
      if (self.__refreshTail) {
        self.__refreshTail();
      }
      setTimeout(function() {
        self.__refreshActive = false;
        if (self.__refreshDeactivate) {
          self.__refreshDeactivate();
        }
        if (self.__refreshHide) {
          self.__refreshHide();
        }

        self.scrollTo(self.__scrollLeft, self.__scrollTop, true);
      }, self.__refreshTailTime);
    }, delay);
  },


  /**
   * Returns the scroll position and zooming values
   *
   * @return {Map} `left` and `top` scroll position and `zoom` level
   */
  getValues: function() {
    return {
      left: this.__scrollLeft,
      top: this.__scrollTop,
      zoom: this.__zoomLevel
    };
  },


  /**
   * Returns the maximum scroll values
   *
   * @return {Map} `left` and `top` maximum scroll values
   */
  getScrollMax: function() {
    return {
      left: this.__maxScrollLeft,
      top: this.__maxScrollTop
    };
  },


  /**
   * Zooms to the given level. Supports optional animation. Zooms
   * the center when no coordinates are given.
   *
   * @param level {Number} Level to zoom to
   * @param animate {Boolean} Whether to use animation
   * @param originLeft {Number} Zoom in at given left coordinate
   * @param originTop {Number} Zoom in at given top coordinate
   */
  zoomTo: function(level, animate, originLeft, originTop) {
    var self = this;

    if (!self.options.zooming) {
      throw new Error("Zooming is not enabled!");
    }

    // Stop deceleration
    if (self.__isDecelerating) {
      zyngaCore.effect.Animate.stop(self.__isDecelerating);
      self.__isDecelerating = false;
    }

    var oldLevel = self.__zoomLevel;

    // Normalize input origin to center of viewport if not defined
    if (originLeft == null) {
      originLeft = self.__clientWidth / 2;
    }

    if (originTop == null) {
      originTop = self.__clientHeight / 2;
    }

    // Limit level according to configuration
    level = Math.max(Math.min(level, self.options.maxZoom), self.options.minZoom);

    // Recompute maximum values while temporary tweaking maximum scroll ranges
    self.__computeScrollMax(level);

    // Recompute left and top coordinates based on new zoom level
    var left = ((originLeft + self.__scrollLeft) * level / oldLevel) - originLeft;
    var top = ((originTop + self.__scrollTop) * level / oldLevel) - originTop;

    // Limit x-axis
    if (left > self.__maxScrollLeft) {
      left = self.__maxScrollLeft;
    } else if (left < 0) {
      left = 0;
    }

    // Limit y-axis
    if (top > self.__maxScrollTop) {
      top = self.__maxScrollTop;
    } else if (top < 0) {
      top = 0;
    }

    // Push values out
    self.__publish(left, top, level, animate);

  },


  /**
   * Zooms the content by the given factor.
   *
   * @param factor {Number} Zoom by given factor
   * @param animate {Boolean} Whether to use animation
   * @param originLeft {Number} Zoom in at given left coordinate
   * @param originTop {Number} Zoom in at given top coordinate
   */
  zoomBy: function(factor, animate, originLeft, originTop) {
    this.zoomTo(this.__zoomLevel * factor, animate, originLeft, originTop);
  },


  /**
   * Scrolls to the given position. Respect limitations and snapping automatically.
   *
   * @param left {Number} Horizontal scroll position, keeps current if value is <code>null</code>
   * @param top {Number} Vertical scroll position, keeps current if value is <code>null</code>
   * @param animate {Boolean} Whether the scrolling should happen using an animation
   * @param zoom {Number} Zoom level to go to
   */
  scrollTo: function(left, top, animate, zoom, wasResize) {
    var self = this;

    // Stop deceleration
    if (self.__isDecelerating) {
      zyngaCore.effect.Animate.stop(self.__isDecelerating);
      self.__isDecelerating = false;
    }

    // Correct coordinates based on new zoom level
    if (zoom != null && zoom !== self.__zoomLevel) {

      if (!self.options.zooming) {
        throw new Error("Zooming is not enabled!");
      }

      left *= zoom;
      top *= zoom;

      // Recompute maximum values while temporary tweaking maximum scroll ranges
      self.__computeScrollMax(zoom);

    } else {

      // Keep zoom when not defined
      zoom = self.__zoomLevel;

    }

    if (!self.options.scrollingX) {

      left = self.__scrollLeft;

    } else {

      if (self.options.paging) {
        left = Math.round(left / self.__clientWidth) * self.__clientWidth;
      } else if (self.options.snapping) {
        left = Math.round(left / self.__snapWidth) * self.__snapWidth;
      }

    }

    if (!self.options.scrollingY) {

      top = self.__scrollTop;

    } else {

      if (self.options.paging) {
        top = Math.round(top / self.__clientHeight) * self.__clientHeight;
      } else if (self.options.snapping) {
        top = Math.round(top / self.__snapHeight) * self.__snapHeight;
      }

    }

    // Limit for allowed ranges
    left = Math.max(Math.min(self.__maxScrollLeft, left), 0);
    top = Math.max(Math.min(self.__maxScrollTop, top), 0);

    // Don't animate when no change detected, still call publish to make sure
    // that rendered position is really in-sync with internal data
    if (left === self.__scrollLeft && top === self.__scrollTop) {
      animate = false;
    }

    // Publish new values
    self.__publish(left, top, zoom, animate, wasResize);

  },


  /**
   * Scroll by the given offset
   *
   * @param left {Number} Scroll x-axis by given offset
   * @param top {Number} Scroll y-axis by given offset
   * @param animate {Boolean} Whether to animate the given change
   */
  scrollBy: function(left, top, animate) {
    var self = this;

    var startLeft = self.__isAnimating ? self.__scheduledLeft : self.__scrollLeft;
    var startTop = self.__isAnimating ? self.__scheduledTop : self.__scrollTop;

    self.scrollTo(startLeft + (left || 0), startTop + (top || 0), animate);
  },



  /*
  ---------------------------------------------------------------------------
    EVENT CALLBACKS
  ---------------------------------------------------------------------------
  */

  /**
   * Mouse wheel handler for zooming support
   */
  doMouseZoom: function(wheelDelta, timeStamp, pageX, pageY) {
    var change = wheelDelta > 0 ? 0.97 : 1.03;
    return this.zoomTo(this.__zoomLevel * change, false, pageX - this.__clientLeft, pageY - this.__clientTop);
  },

  /**
   * Touch start handler for scrolling support
   */
  doTouchStart: function(touches, timeStamp) {
    var self = this;

    // remember if the deceleration was just stopped
    self.__decStopped = !!(self.__isDecelerating || self.__isAnimating);

    self.hintResize();

    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== "number") {
      timeStamp = Date.now();
    }

    // Reset interruptedAnimation flag
    self.__interruptedAnimation = true;

    // Stop deceleration
    if (self.__isDecelerating) {
      zyngaCore.effect.Animate.stop(self.__isDecelerating);
      self.__isDecelerating = false;
      self.__interruptedAnimation = true;
    }

    // Stop animation
    if (self.__isAnimating) {
      zyngaCore.effect.Animate.stop(self.__isAnimating);
      self.__isAnimating = false;
      self.__interruptedAnimation = true;
    }

    // Use center point when dealing with two fingers
    var currentTouchLeft, currentTouchTop;
    var isSingleTouch = touches.length === 1;
    if (isSingleTouch) {
      currentTouchLeft = touches[0].pageX;
      currentTouchTop = touches[0].pageY;
    } else {
      currentTouchLeft = Math.abs(touches[0].pageX + touches[1].pageX) / 2;
      currentTouchTop = Math.abs(touches[0].pageY + touches[1].pageY) / 2;
    }

    // Store initial positions
    self.__initialTouchLeft = currentTouchLeft;
    self.__initialTouchTop = currentTouchTop;

    // Store initial touchList for scale calculation
    self.__initialTouches = touches;

    // Store current zoom level
    self.__zoomLevelStart = self.__zoomLevel;

    // Store initial touch positions
    self.__lastTouchLeft = currentTouchLeft;
    self.__lastTouchTop = currentTouchTop;

    // Store initial move time stamp
    self.__lastTouchMove = timeStamp;

    // Reset initial scale
    self.__lastScale = 1;

    // Reset locking flags
    self.__enableScrollX = !isSingleTouch && self.options.scrollingX;
    self.__enableScrollY = !isSingleTouch && self.options.scrollingY;

    // Reset tracking flag
    self.__isTracking = true;

    // Reset deceleration complete flag
    self.__didDecelerationComplete = false;

    // Dragging starts directly with two fingers, otherwise lazy with an offset
    self.__isDragging = !isSingleTouch;

    // Some features are disabled in multi touch scenarios
    self.__isSingleTouch = isSingleTouch;

    // Clearing data structure
    self.__positions = [];

  },


  /**
   * Touch move handler for scrolling support
   */
  doTouchMove: function(touches, timeStamp, scale) {
    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== "number") {
      timeStamp = Date.now();
    }

    var self = this;

    // Ignore event when tracking is not enabled (event might be outside of element)
    if (!self.__isTracking) {
      return;
    }

    var currentTouchLeft, currentTouchTop;

    // Compute move based around of center of fingers
    if (touches.length === 2) {
      currentTouchLeft = Math.abs(touches[0].pageX + touches[1].pageX) / 2;
      currentTouchTop = Math.abs(touches[0].pageY + touches[1].pageY) / 2;

      // Calculate scale when not present and only when touches are used
      if (!scale && self.options.zooming) {
        scale = self.__getScale(self.__initialTouches, touches);
      }
    } else {
      currentTouchLeft = touches[0].pageX;
      currentTouchTop = touches[0].pageY;
    }

    var positions = self.__positions;

    // Are we already is dragging mode?
    if (self.__isDragging) {
        self.__decStopped = false;

      // Compute move distance
      var moveX = currentTouchLeft - self.__lastTouchLeft;
      var moveY = currentTouchTop - self.__lastTouchTop;

      // Read previous scroll position and zooming
      var scrollLeft = self.__scrollLeft;
      var scrollTop = self.__scrollTop;
      var level = self.__zoomLevel;

      // Work with scaling
      if (scale != null && self.options.zooming) {

        var oldLevel = level;

        // Recompute level based on previous scale and new scale
        level = level / self.__lastScale * scale;

        // Limit level according to configuration
        level = Math.max(Math.min(level, self.options.maxZoom), self.options.minZoom);

        // Only do further compution when change happened
        if (oldLevel !== level) {

          // Compute relative event position to container
          var currentTouchLeftRel = currentTouchLeft - self.__clientLeft;
          var currentTouchTopRel = currentTouchTop - self.__clientTop;

          // Recompute left and top coordinates based on new zoom level
          scrollLeft = ((currentTouchLeftRel + scrollLeft) * level / oldLevel) - currentTouchLeftRel;
          scrollTop = ((currentTouchTopRel + scrollTop) * level / oldLevel) - currentTouchTopRel;

          // Recompute max scroll values
          self.__computeScrollMax(level);

        }
      }

      if (self.__enableScrollX) {

        scrollLeft -= moveX * self.options.speedMultiplier;
        var maxScrollLeft = self.__maxScrollLeft;

        if (scrollLeft > maxScrollLeft || scrollLeft < 0) {

          // Slow down on the edges
          if (self.options.bouncing) {

            scrollLeft += (moveX / 2 * self.options.speedMultiplier);

          } else if (scrollLeft > maxScrollLeft) {

            scrollLeft = maxScrollLeft;

          } else {

            scrollLeft = 0;

          }
        }
      }

      // Compute new vertical scroll position
      if (self.__enableScrollY) {

        scrollTop -= moveY * self.options.speedMultiplier;
        var maxScrollTop = self.__maxScrollTop;

        if (scrollTop > maxScrollTop || scrollTop < 0) {

          // Slow down on the edges
          if (self.options.bouncing || (self.__refreshHeight && scrollTop < 0)) {

            scrollTop += (moveY / 2 * self.options.speedMultiplier);

            // Support pull-to-refresh (only when only y is scrollable)
            if (!self.__enableScrollX && self.__refreshHeight != null) {

              // hide the refresher when it's behind the header bar in case of header transparency
              if (scrollTop < 0) {
                self.__refreshHidden = false;
                self.__refreshShow();
              } else {
                self.__refreshHide();
                self.__refreshHidden = true;
              }

              if (!self.__refreshActive && scrollTop <= -self.__refreshHeight) {

                self.__refreshActive = true;
                if (self.__refreshActivate) {
                  self.__refreshActivate();
                }

              } else if (self.__refreshActive && scrollTop > -self.__refreshHeight) {

                self.__refreshActive = false;
                if (self.__refreshDeactivate) {
                  self.__refreshDeactivate();
                }

              }
            }

          } else if (scrollTop > maxScrollTop) {

            scrollTop = maxScrollTop;

          } else {

            scrollTop = 0;

          }
        } else if (self.__refreshHeight && !self.__refreshHidden) {
          // if a positive scroll value and the refresher is still not hidden, hide it
          self.__refreshHide();
          self.__refreshHidden = true;
        }
      }

      // Keep list from growing infinitely (holding min 10, max 20 measure points)
      if (positions.length > 60) {
        positions.splice(0, 30);
      }

      // Track scroll movement for decleration
      positions.push(scrollLeft, scrollTop, timeStamp);

      // Sync scroll position
      self.__publish(scrollLeft, scrollTop, level);

    // Otherwise figure out whether we are switching into dragging mode now.
    } else {

      var minimumTrackingForScroll = self.options.locking ? 3 : 0;
      var minimumTrackingForDrag = 5;

      var distanceX = Math.abs(currentTouchLeft - self.__initialTouchLeft);
      var distanceY = Math.abs(currentTouchTop - self.__initialTouchTop);

      self.__enableScrollX = self.options.scrollingX && distanceX >= minimumTrackingForScroll;
      self.__enableScrollY = self.options.scrollingY && distanceY >= minimumTrackingForScroll;

      positions.push(self.__scrollLeft, self.__scrollTop, timeStamp);

      self.__isDragging = (self.__enableScrollX || self.__enableScrollY) && (distanceX >= minimumTrackingForDrag || distanceY >= minimumTrackingForDrag);
      if (self.__isDragging) {
        self.__interruptedAnimation = false;
        self.__fadeScrollbars('in');
      }

    }

    // Update last touch positions and time stamp for next event
    self.__lastTouchLeft = currentTouchLeft;
    self.__lastTouchTop = currentTouchTop;
    self.__lastTouchMove = timeStamp;
    self.__lastScale = scale;

  },


  /**
   * Touch end handler for scrolling support
   */
  doTouchEnd: function(e, timeStamp) {
    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== "number") {
      timeStamp = Date.now();
    }

    var self = this;

    // Ignore event when tracking is not enabled (no touchstart event on element)
    // This is required as this listener ('touchmove') sits on the document and not on the element itself.
    if (!self.__isTracking) {
      return;
    }

    // Not touching anymore (when two finger hit the screen there are two touch end events)
    self.__isTracking = false;

    // Be sure to reset the dragging flag now. Here we also detect whether
    // the finger has moved fast enough to switch into a deceleration animation.
    if (self.__isDragging) {

      // Reset dragging flag
      self.__isDragging = false;

      // Start deceleration
      // Verify that the last move detected was in some relevant time frame
      if (self.__isSingleTouch && self.options.animating && (timeStamp - self.__lastTouchMove) <= 100) {

        // Then figure out what the scroll position was about 100ms ago
        var positions = self.__positions;
        var endPos = positions.length - 1;
        var startPos = endPos;

        // Move pointer to position measured 100ms ago
        for (var i = endPos; i > 0 && positions[i] > (self.__lastTouchMove - 100); i -= 3) {
          startPos = i;
        }

        // If start and stop position is identical in a 100ms timeframe,
        // we cannot compute any useful deceleration.
        if (startPos !== endPos) {

          // Compute relative movement between these two points
          var timeOffset = positions[endPos] - positions[startPos];
          var movedLeft = self.__scrollLeft - positions[startPos - 2];
          var movedTop = self.__scrollTop - positions[startPos - 1];

          // Based on 50ms compute the movement to apply for each render step
          self.__decelerationVelocityX = movedLeft / timeOffset * (1000 / 60);
          self.__decelerationVelocityY = movedTop / timeOffset * (1000 / 60);

          // How much velocity is required to start the deceleration
          var minVelocityToStartDeceleration = self.options.paging || self.options.snapping ? self.options.decelVelocityThresholdPaging : self.options.decelVelocityThreshold;

          // Verify that we have enough velocity to start deceleration
          if (Math.abs(self.__decelerationVelocityX) > minVelocityToStartDeceleration || Math.abs(self.__decelerationVelocityY) > minVelocityToStartDeceleration) {

            // Deactivate pull-to-refresh when decelerating
            if (!self.__refreshActive) {
              self.__startDeceleration(timeStamp);
            }
          }
        } else {
          self.__scrollingComplete();
        }
      } else if ((timeStamp - self.__lastTouchMove) > 100) {
        self.__scrollingComplete();
      }

    } else if (self.__decStopped) {
      // the deceleration was stopped
      // user flicked the scroll fast, and stop dragging, then did a touchstart to stop the srolling
      // tell the touchend event code to do nothing, we don't want to actually send a click
      e.isTapHandled = true;
      self.__decStopped = false;
    }

    // If this was a slower move it is per default non decelerated, but this
    // still means that we want snap back to the bounds which is done here.
    // This is placed outside the condition above to improve edge case stability
    // e.g. touchend fired without enabled dragging. This should normally do not
    // have modified the scroll positions or even showed the scrollbars though.
    if (!self.__isDecelerating) {

      if (self.__refreshActive && self.__refreshStart) {

        // Use publish instead of scrollTo to allow scrolling to out of boundary position
        // We don't need to normalize scrollLeft, zoomLevel, etc. here because we only y-scrolling when pull-to-refresh is enabled
        self.__publish(self.__scrollLeft, -self.__refreshHeight, self.__zoomLevel, true);

        var d = new Date();
        self.refreshStartTime = d.getTime();

        if (self.__refreshStart) {
          self.__refreshStart();
        }
        // for iOS-ey style scrolling
        if (!ionic.Platform.isAndroid())self.__startDeceleration();
      } else {

        if (self.__interruptedAnimation || self.__isDragging) {
          self.__scrollingComplete();
        }
        self.scrollTo(self.__scrollLeft, self.__scrollTop, true, self.__zoomLevel);

        // Directly signalize deactivation (nothing todo on refresh?)
        if (self.__refreshActive) {

          self.__refreshActive = false;
          if (self.__refreshDeactivate) {
            self.__refreshDeactivate();
          }

        }
      }
    }

    // Fully cleanup list
    self.__positions.length = 0;

  },



  /*
  ---------------------------------------------------------------------------
    PRIVATE API
  ---------------------------------------------------------------------------
  */

  /**
   * Applies the scroll position to the content element
   *
   * @param left {Number} Left scroll position
   * @param top {Number} Top scroll position
   * @param animate {Boolean} Whether animation should be used to move to the new coordinates
   */
  __publish: function(left, top, zoom, animate, wasResize) {

    var self = this;

    // Remember whether we had an animation, then we try to continue based on the current "drive" of the animation
    var wasAnimating = self.__isAnimating;
    if (wasAnimating) {
      zyngaCore.effect.Animate.stop(wasAnimating);
      self.__isAnimating = false;
    }

    if (animate && self.options.animating) {

      // Keep scheduled positions for scrollBy/zoomBy functionality
      self.__scheduledLeft = left;
      self.__scheduledTop = top;
      self.__scheduledZoom = zoom;

      var oldLeft = self.__scrollLeft;
      var oldTop = self.__scrollTop;
      var oldZoom = self.__zoomLevel;

      var diffLeft = left - oldLeft;
      var diffTop = top - oldTop;
      var diffZoom = zoom - oldZoom;

      var step = function(percent, now, render) {

        if (render) {

          self.__scrollLeft = oldLeft + (diffLeft * percent);
          self.__scrollTop = oldTop + (diffTop * percent);
          self.__zoomLevel = oldZoom + (diffZoom * percent);

          // Push values out
          if (self.__callback) {
            self.__callback(self.__scrollLeft, self.__scrollTop, self.__zoomLevel, wasResize);
          }

        }
      };

      var verify = function(id) {
        return self.__isAnimating === id;
      };

      var completed = function(renderedFramesPerSecond, animationId, wasFinished) {
        if (animationId === self.__isAnimating) {
          self.__isAnimating = false;
        }
        if (self.__didDecelerationComplete || wasFinished) {
          self.__scrollingComplete();
        }

        if (self.options.zooming) {
          self.__computeScrollMax();
        }
      };

      // When continuing based on previous animation we choose an ease-out animation instead of ease-in-out
      self.__isAnimating = zyngaCore.effect.Animate.start(step, verify, completed, self.options.animationDuration, wasAnimating ? easeOutCubic : easeInOutCubic);

    } else {

      self.__scheduledLeft = self.__scrollLeft = left;
      self.__scheduledTop = self.__scrollTop = top;
      self.__scheduledZoom = self.__zoomLevel = zoom;

      // Push values out
      if (self.__callback) {
        self.__callback(left, top, zoom, wasResize);
      }

      // Fix max scroll ranges
      if (self.options.zooming) {
        self.__computeScrollMax();
      }
    }
  },


  /**
   * Recomputes scroll minimum values based on client dimensions and content dimensions.
   */
  __computeScrollMax: function(zoomLevel) {
    var self = this;

    if (zoomLevel == null) {
      zoomLevel = self.__zoomLevel;
    }

    self.__maxScrollLeft = Math.max((self.__contentWidth * zoomLevel) - self.__clientWidth, 0);
    self.__maxScrollTop = Math.max((self.__contentHeight * zoomLevel) - self.__clientHeight, 0);

    if (!self.__didWaitForSize && !self.__maxScrollLeft && !self.__maxScrollTop) {
      self.__didWaitForSize = true;
      self.__waitForSize();
    }
  },


  /**
   * If the scroll view isn't sized correctly on start, wait until we have at least some size
   */
  __waitForSize: function() {
    var self = this;

    clearTimeout(self.__sizerTimeout);

    var sizer = function() {
      self.resize(true);
    };

    sizer();
    self.__sizerTimeout = setTimeout(sizer, 500);
  },

  /*
  ---------------------------------------------------------------------------
    ANIMATION (DECELERATION) SUPPORT
  ---------------------------------------------------------------------------
  */

  /**
   * Called when a touch sequence end and the speed of the finger was high enough
   * to switch into deceleration mode.
   */
  __startDeceleration: function() {
    var self = this;

    if (self.options.paging) {

      var scrollLeft = Math.max(Math.min(self.__scrollLeft, self.__maxScrollLeft), 0);
      var scrollTop = Math.max(Math.min(self.__scrollTop, self.__maxScrollTop), 0);
      var clientWidth = self.__clientWidth;
      var clientHeight = self.__clientHeight;

      // We limit deceleration not to the min/max values of the allowed range, but to the size of the visible client area.
      // Each page should have exactly the size of the client area.
      self.__minDecelerationScrollLeft = Math.floor(scrollLeft / clientWidth) * clientWidth;
      self.__minDecelerationScrollTop = Math.floor(scrollTop / clientHeight) * clientHeight;
      self.__maxDecelerationScrollLeft = Math.ceil(scrollLeft / clientWidth) * clientWidth;
      self.__maxDecelerationScrollTop = Math.ceil(scrollTop / clientHeight) * clientHeight;

    } else {

      self.__minDecelerationScrollLeft = 0;
      self.__minDecelerationScrollTop = 0;
      self.__maxDecelerationScrollLeft = self.__maxScrollLeft;
      self.__maxDecelerationScrollTop = self.__maxScrollTop;
      if (self.__refreshActive) self.__minDecelerationScrollTop = self.__refreshHeight * -1;
    }

    // Wrap class method
    var step = function(percent, now, render) {
      self.__stepThroughDeceleration(render);
    };

    // How much velocity is required to keep the deceleration running
    self.__minVelocityToKeepDecelerating = self.options.snapping ? 4 : 0.1;

    // Detect whether it's still worth to continue animating steps
    // If we are already slow enough to not being user perceivable anymore, we stop the whole process here.
    var verify = function() {
      var shouldContinue = Math.abs(self.__decelerationVelocityX) >= self.__minVelocityToKeepDecelerating ||
        Math.abs(self.__decelerationVelocityY) >= self.__minVelocityToKeepDecelerating;
      if (!shouldContinue) {
        self.__didDecelerationComplete = true;

        //Make sure the scroll values are within the boundaries after a bounce,
        //not below 0 or above maximum
        if (self.options.bouncing && !self.__refreshActive) {
          self.scrollTo(
            Math.min( Math.max(self.__scrollLeft, 0), self.__maxScrollLeft ),
            Math.min( Math.max(self.__scrollTop, 0), self.__maxScrollTop ),
            self.__refreshActive
          );
        }
      }
      return shouldContinue;
    };

    var completed = function() {
      self.__isDecelerating = false;
      if (self.__didDecelerationComplete) {
        self.__scrollingComplete();
      }

      // Animate to grid when snapping is active, otherwise just fix out-of-boundary positions
      if (self.options.paging) {
        self.scrollTo(self.__scrollLeft, self.__scrollTop, self.options.snapping);
      }
    };

    // Start animation and switch on flag
    self.__isDecelerating = zyngaCore.effect.Animate.start(step, verify, completed);

  },


  /**
   * Called on every step of the animation
   *
   * @param inMemory {Boolean} Whether to not render the current step, but keep it in memory only. Used internally only!
   */
  __stepThroughDeceleration: function(render) {
    var self = this;


    //
    // COMPUTE NEXT SCROLL POSITION
    //

    // Add deceleration to scroll position
    var scrollLeft = self.__scrollLeft + self.__decelerationVelocityX;// * self.options.deceleration);
    var scrollTop = self.__scrollTop + self.__decelerationVelocityY;// * self.options.deceleration);


    //
    // HARD LIMIT SCROLL POSITION FOR NON BOUNCING MODE
    //

    if (!self.options.bouncing) {

      var scrollLeftFixed = Math.max(Math.min(self.__maxDecelerationScrollLeft, scrollLeft), self.__minDecelerationScrollLeft);
      if (scrollLeftFixed !== scrollLeft) {
        scrollLeft = scrollLeftFixed;
        self.__decelerationVelocityX = 0;
      }

      var scrollTopFixed = Math.max(Math.min(self.__maxDecelerationScrollTop, scrollTop), self.__minDecelerationScrollTop);
      if (scrollTopFixed !== scrollTop) {
        scrollTop = scrollTopFixed;
        self.__decelerationVelocityY = 0;
      }

    }


    //
    // UPDATE SCROLL POSITION
    //

    if (render) {

      self.__publish(scrollLeft, scrollTop, self.__zoomLevel);

    } else {

      self.__scrollLeft = scrollLeft;
      self.__scrollTop = scrollTop;

    }


    //
    // SLOW DOWN
    //

    // Slow down velocity on every iteration
    if (!self.options.paging) {

      // This is the factor applied to every iteration of the animation
      // to slow down the process. This should emulate natural behavior where
      // objects slow down when the initiator of the movement is removed
      var frictionFactor = self.options.deceleration;

      self.__decelerationVelocityX *= frictionFactor;
      self.__decelerationVelocityY *= frictionFactor;

    }


    //
    // BOUNCING SUPPORT
    //

    if (self.options.bouncing) {

      var scrollOutsideX = 0;
      var scrollOutsideY = 0;

      // This configures the amount of change applied to deceleration/acceleration when reaching boundaries
      var penetrationDeceleration = self.options.penetrationDeceleration;
      var penetrationAcceleration = self.options.penetrationAcceleration;

      // Check limits
      if (scrollLeft < self.__minDecelerationScrollLeft) {
        scrollOutsideX = self.__minDecelerationScrollLeft - scrollLeft;
      } else if (scrollLeft > self.__maxDecelerationScrollLeft) {
        scrollOutsideX = self.__maxDecelerationScrollLeft - scrollLeft;
      }

      if (scrollTop < self.__minDecelerationScrollTop) {
        scrollOutsideY = self.__minDecelerationScrollTop - scrollTop;
      } else if (scrollTop > self.__maxDecelerationScrollTop) {
        scrollOutsideY = self.__maxDecelerationScrollTop - scrollTop;
      }

      // Slow down until slow enough, then flip back to snap position
      if (scrollOutsideX !== 0) {
        var isHeadingOutwardsX = scrollOutsideX * self.__decelerationVelocityX <= self.__minDecelerationScrollLeft;
        if (isHeadingOutwardsX) {
          self.__decelerationVelocityX += scrollOutsideX * penetrationDeceleration;
        }
        var isStoppedX = Math.abs(self.__decelerationVelocityX) <= self.__minVelocityToKeepDecelerating;
        //If we're not heading outwards, or if the above statement got us below minDeceleration, go back towards bounds
        if (!isHeadingOutwardsX || isStoppedX) {
          self.__decelerationVelocityX = scrollOutsideX * penetrationAcceleration;
        }
      }

      if (scrollOutsideY !== 0) {
        var isHeadingOutwardsY = scrollOutsideY * self.__decelerationVelocityY <= self.__minDecelerationScrollTop;
        if (isHeadingOutwardsY) {
          self.__decelerationVelocityY += scrollOutsideY * penetrationDeceleration;
        }
        var isStoppedY = Math.abs(self.__decelerationVelocityY) <= self.__minVelocityToKeepDecelerating;
        //If we're not heading outwards, or if the above statement got us below minDeceleration, go back towards bounds
        if (!isHeadingOutwardsY || isStoppedY) {
          self.__decelerationVelocityY = scrollOutsideY * penetrationAcceleration;
        }
      }
    }
  },


  /**
   * calculate the distance between two touches
   * @param   {Touch}     touch1
   * @param   {Touch}     touch2
   * @returns {Number}    distance
   */
  __getDistance: function getDistance(touch1, touch2) {
    var x = touch2.pageX - touch1.pageX,
    y = touch2.pageY - touch1.pageY;
    return Math.sqrt((x * x) + (y * y));
  },


  /**
   * calculate the scale factor between two touchLists (fingers)
   * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
   * @param   {Array}     start
   * @param   {Array}     end
   * @returns {Number}    scale
   */
  __getScale: function getScale(start, end) {
    // need two fingers...
    if (start.length >= 2 && end.length >= 2) {
      return this.__getDistance(end[0], end[1]) /
        this.__getDistance(start[0], start[1]);
    }
    return 1;
  }
});

ionic.scroll = {
  isScrolling: false,
  lastTop: 0
};

})(ionic);

(function(ionic) {
  var NOOP = function() {};
  var deprecated = function(name) {
    void 0;
  };
  ionic.views.ScrollNative = ionic.views.View.inherit({

    initialize: function(options) {
      var self = this;
      self.__container = self.el = options.el;
      self.__content = options.el.firstElementChild;
      // Whether scrolling is frozen or not
      self.__frozen = false;
      self.isNative = true;

      self.__scrollTop = self.el.scrollTop;
      self.__scrollLeft = self.el.scrollLeft;
      self.__clientHeight = self.__content.clientHeight;
      self.__clientWidth = self.__content.clientWidth;
      self.__maxScrollTop = Math.max((self.__contentHeight) - self.__clientHeight, 0);
      self.__maxScrollLeft = Math.max((self.__contentWidth) - self.__clientWidth, 0);

      if(options.startY >= 0 || options.startX >= 0) {
        ionic.requestAnimationFrame(function() {
          self.__originalContainerHeight = self.el.getBoundingClientRect().height;

          self.el.scrollTop = options.startY || 0;
          self.el.scrollLeft = options.startX || 0;

          self.__scrollTop = self.el.scrollTop;
          self.__scrollLeft = self.el.scrollLeft;
        });
      }

      self.options = {

        freeze: false,

        getContentWidth: function() {
          return Math.max(self.__content.scrollWidth, self.__content.offsetWidth);
        },

        getContentHeight: function() {
          return Math.max(self.__content.scrollHeight, self.__content.offsetHeight + (self.__content.offsetTop * 2));
        }

      };

      for (var key in options) {
        self.options[key] = options[key];
      }

      /**
       * Sets isScrolling to true, and automatically deactivates if not called again in 80ms.
       */
      self.onScroll = function() {
        if (!ionic.scroll.isScrolling) {
          ionic.scroll.isScrolling = true;
        }

        clearTimeout(self.scrollTimer);
        self.scrollTimer = setTimeout(function() {
          ionic.scroll.isScrolling = false;
        }, 80);
      };

      self.freeze = function(shouldFreeze) {
        self.__frozen = shouldFreeze;
      };
      // A more powerful freeze pop that dominates all other freeze pops
      self.freezeShut = function(shouldFreezeShut) {
        self.__frozenShut = shouldFreezeShut;
      };

      self.__initEventHandlers();
    },

    /**  Methods not used in native scrolling */
    __callback: function() { deprecated('__callback'); },
    zoomTo: function() { deprecated('zoomTo'); },
    zoomBy: function() { deprecated('zoomBy'); },
    activatePullToRefresh: function() { deprecated('activatePullToRefresh'); },

    /**
     * Returns the scroll position and zooming values
     *
     * @return {Map} `left` and `top` scroll position and `zoom` level
     */
    resize: function(continueScrolling) {
      var self = this;
      if (!self.__container || !self.options) return;

      // Update Scroller dimensions for changed content
      // Add padding to bottom of content
      self.setDimensions(
        self.__container.clientWidth,
        self.__container.clientHeight,
        self.options.getContentWidth(),
        self.options.getContentHeight(),
        continueScrolling
      );
    },

    /**
     * Initialize the scrollview
     * In native scrolling, this only means we need to gather size information
     */
    run: function() {
      this.resize();
    },

    /**
     * Returns the scroll position and zooming values
     *
     * @return {Map} `left` and `top` scroll position and `zoom` level
     */
    getValues: function() {
      var self = this;
      self.update();
      return {
        left: self.__scrollLeft,
        top: self.__scrollTop,
        zoom: 1
      };
    },

    /**
     * Updates the __scrollLeft and __scrollTop values to el's current value
     */
    update: function() {
      var self = this;
      self.__scrollLeft = self.el.scrollLeft;
      self.__scrollTop = self.el.scrollTop;
    },

    /**
     * Configures the dimensions of the client (outer) and content (inner) elements.
     * Requires the available space for the outer element and the outer size of the inner element.
     * All values which are falsy (null or zero etc.) are ignored and the old value is kept.
     *
     * @param clientWidth {Integer} Inner width of outer element
     * @param clientHeight {Integer} Inner height of outer element
     * @param contentWidth {Integer} Outer width of inner element
     * @param contentHeight {Integer} Outer height of inner element
     */
    setDimensions: function(clientWidth, clientHeight, contentWidth, contentHeight) {
      var self = this;

      if (!clientWidth && !clientHeight && !contentWidth && !contentHeight) {
        // this scrollview isn't rendered, don't bother
        return;
      }

      // Only update values which are defined
      if (clientWidth === +clientWidth) {
        self.__clientWidth = clientWidth;
      }

      if (clientHeight === +clientHeight) {
        self.__clientHeight = clientHeight;
      }

      if (contentWidth === +contentWidth) {
        self.__contentWidth = contentWidth;
      }

      if (contentHeight === +contentHeight) {
        self.__contentHeight = contentHeight;
      }

      // Refresh maximums
      self.__computeScrollMax();
    },

    /**
     * Returns the maximum scroll values
     *
     * @return {Map} `left` and `top` maximum scroll values
     */
    getScrollMax: function() {
      return {
        left: this.__maxScrollLeft,
        top: this.__maxScrollTop
      };
    },

    /**
     * Scrolls by the given amount in px.
     *
     * @param left {Number} Horizontal scroll position, keeps current if value is <code>null</code>
     * @param top {Number} Vertical scroll position, keeps current if value is <code>null</code>
     * @param animate {Boolean} Whether the scrolling should happen using an animation
     */

    scrollBy: function(left, top, animate) {
      var self = this;

      // update scroll vars before refferencing them
      self.update();

      var startLeft = self.__isAnimating ? self.__scheduledLeft : self.__scrollLeft;
      var startTop = self.__isAnimating ? self.__scheduledTop : self.__scrollTop;

      self.scrollTo(startLeft + (left || 0), startTop + (top || 0), animate);
    },

    /**
     * Scrolls to the given position in px.
     *
     * @param left {Number} Horizontal scroll position, keeps current if value is <code>null</code>
     * @param top {Number} Vertical scroll position, keeps current if value is <code>null</code>
     * @param animate {Boolean} Whether the scrolling should happen using an animation
     */
    scrollTo: function(left, top, animate) {
      var self = this;
      if (!animate) {
        self.el.scrollTop = top;
        self.el.scrollLeft = left;
        self.resize();
        return;
      }

      var oldOverflowX = self.el.style.overflowX;
      var oldOverflowY = self.el.style.overflowY;

      clearTimeout(self.__scrollToCleanupTimeout);
      self.__scrollToCleanupTimeout = setTimeout(function() {
        self.el.style.overflowX = oldOverflowX;
        self.el.style.overflowY = oldOverflowY;
      }, 500);

      self.el.style.overflowY = 'hidden';
      self.el.style.overflowX = 'hidden';

      animateScroll(top, left);

      function animateScroll(Y, X) {
        // scroll animation loop w/ easing
        // credit https://gist.github.com/dezinezync/5487119
        var start = Date.now(),
          duration = 250, //milliseconds
          fromY = self.el.scrollTop,
          fromX = self.el.scrollLeft;

        if (fromY === Y && fromX === X) {
          self.el.style.overflowX = oldOverflowX;
          self.el.style.overflowY = oldOverflowY;
          self.resize();
          return; /* Prevent scrolling to the Y point if already there */
        }

        // decelerating to zero velocity
        function easeOutCubic(t) {
          return (--t) * t * t + 1;
        }

        // scroll loop
        function animateScrollStep() {
          var currentTime = Date.now(),
            time = Math.min(1, ((currentTime - start) / duration)),
          // where .5 would be 50% of time on a linear scale easedT gives a
          // fraction based on the easing method
            easedT = easeOutCubic(time);

          if (fromY != Y) {
            self.el.scrollTop = parseInt((easedT * (Y - fromY)) + fromY, 10);
          }
          if (fromX != X) {
            self.el.scrollLeft = parseInt((easedT * (X - fromX)) + fromX, 10);
          }

          if (time < 1) {
            ionic.requestAnimationFrame(animateScrollStep);

          } else {
            // done
            ionic.tap.removeClonedInputs(self.__container, self);
            self.el.style.overflowX = oldOverflowX;
            self.el.style.overflowY = oldOverflowY;
            self.resize();
          }
        }

        // start scroll loop
        ionic.requestAnimationFrame(animateScrollStep);
      }
    },



    /*
     ---------------------------------------------------------------------------
     PRIVATE API
     ---------------------------------------------------------------------------
     */

    /**
     * If the scroll view isn't sized correctly on start, wait until we have at least some size
     */
    __waitForSize: function() {
      var self = this;

      clearTimeout(self.__sizerTimeout);

      var sizer = function() {
        self.resize(true);
      };

      sizer();
      self.__sizerTimeout = setTimeout(sizer, 500);
    },


    /**
     * Recomputes scroll minimum values based on client dimensions and content dimensions.
     */
    __computeScrollMax: function() {
      var self = this;

      self.__maxScrollLeft = Math.max((self.__contentWidth) - self.__clientWidth, 0);
      self.__maxScrollTop = Math.max((self.__contentHeight) - self.__clientHeight, 0);

      if (!self.__didWaitForSize && !self.__maxScrollLeft && !self.__maxScrollTop) {
        self.__didWaitForSize = true;
        self.__waitForSize();
      }
    },

    __initEventHandlers: function() {
      var self = this;

      // Event Handler
      var container = self.__container;
      // save height when scroll view is shrunk so we don't need to reflow
      var scrollViewOffsetHeight;

      var lastKeyboardHeight;

      /**
       * Shrink the scroll view when the keyboard is up if necessary and if the
       * focused input is below the bottom of the shrunk scroll view, scroll it
       * into view.
       */
      self.scrollChildIntoView = function(e) {
        var rect = container.getBoundingClientRect();
        if(!self.__originalContainerHeight) {
          self.__originalContainerHeight = rect.height;
        }

        // D
        //var scrollBottomOffsetToTop = rect.bottom;
        // D - A
        scrollViewOffsetHeight = self.__originalContainerHeight;
        //console.log('Scroll view offset height', scrollViewOffsetHeight);
        //console.dir(container);
        var alreadyShrunk = self.isShrunkForKeyboard;

        var isModal = container.parentNode.classList.contains('modal');
        var isPopover = container.parentNode.classList.contains('popover');
        // 680px is when the media query for 60% modal width kicks in
        var isInsetModal = isModal && window.innerWidth >= 680;

       /*
        *  _______
        * |---A---| <- top of scroll view
        * |       |
        * |---B---| <- keyboard
        * |   C   | <- input
        * |---D---| <- initial bottom of scroll view
        * |___E___| <- bottom of viewport
        *
        *  All commented calculations relative to the top of the viewport (ie E
        *  is the viewport height, not 0)
        */


        var changedKeyboardHeight = lastKeyboardHeight && (lastKeyboardHeight !== e.detail.keyboardHeight);

        if (!alreadyShrunk || changedKeyboardHeight) {
          // shrink scrollview so we can actually scroll if the input is hidden
          // if it isn't shrink so we can scroll to inputs under the keyboard
          // inset modals won't shrink on Android on their own when the keyboard appears
          if ( !isPopover && (ionic.Platform.isIOS() || ionic.Platform.isFullScreen || isInsetModal) ) {
            // if there are things below the scroll view account for them and
            // subtract them from the keyboard height when resizing
            // E - D                         E                         D
            //var scrollBottomOffsetToBottom = e.detail.viewportHeight - scrollBottomOffsetToTop;

            // 0 or D - B if D > B           E - B                     E - D
            //var keyboardOffset = e.detail.keyboardHeight - scrollBottomOffsetToBottom;

            ionic.requestAnimationFrame(function(){
              // D - A or B - A if D > B       D - A             max(0, D - B)
              scrollViewOffsetHeight = Math.max(0, Math.min(self.__originalContainerHeight, self.__originalContainerHeight - (e.detail.keyboardHeight - 43)));//keyboardOffset >= 0 ? scrollViewOffsetHeight - keyboardOffset : scrollViewOffsetHeight + keyboardOffset;

              //console.log('Old container height', self.__originalContainerHeight, 'New container height', scrollViewOffsetHeight, 'Keyboard height', e.detail.keyboardHeight);

              container.style.height = scrollViewOffsetHeight + "px";

              /*
              if (ionic.Platform.isIOS()) {
                // Force redraw to avoid disappearing content
                var disp = container.style.display;
                container.style.display = 'none';
                var trick = container.offsetHeight;
                container.style.display = disp;
              }
              */
              container.classList.add('keyboard-up');
              //update scroll view
              self.resize();
            });
          }

          self.isShrunkForKeyboard = true;
        }

        lastKeyboardHeight = e.detail.keyboardHeight;

        /*
         *  _______
         * |---A---| <- top of scroll view
         * |   *   | <- where we want to scroll to
         * |--B-D--| <- keyboard, bottom of scroll view
         * |   C   | <- input
         * |       |
         * |___E___| <- bottom of viewport
         *
         *  All commented calculations relative to the top of the viewport (ie E
         *  is the viewport height, not 0)
         */
        // if the element is positioned under the keyboard scroll it into view
        if (e.detail.isElementUnderKeyboard) {

          ionic.requestAnimationFrame(function(){
            var pos = ionic.DomUtil.getOffsetTop(e.detail.target);
            setTimeout(function() {
              if (ionic.Platform.isIOS()) {
                ionic.tap.cloneFocusedInput(container, self);
              }
              // Scroll the input into view, with a 100px buffer
              self.scrollTo(0, pos - (rect.top + 100), true);
              self.onScroll();
            }, 32);

            /*
            // update D if we shrunk
            if (self.isShrunkForKeyboard && !alreadyShrunk) {
              scrollBottomOffsetToTop = container.getBoundingClientRect().bottom;
              console.log('Scroll bottom', scrollBottomOffsetToTop);
            }

            // middle of the scrollview, this is where we want to scroll to
            // (D - A) / 2
            var scrollMidpointOffset = scrollViewOffsetHeight * 0.5;
            console.log('Midpoint', scrollMidpointOffset);
            //console.log("container.offsetHeight: " + scrollViewOffsetHeight);

            // middle of the input we want to scroll into view
            // C
            var inputMidpoint = ((e.detail.elementBottom + e.detail.elementTop) / 2);
            console.log('Input midpoint');

            // distance from middle of input to the bottom of the scroll view
            // C - D                                C               D
            var inputMidpointOffsetToScrollBottom = inputMidpoint - scrollBottomOffsetToTop;
            console.log('Input midpoint offset', inputMidpointOffsetToScrollBottom);

            //C - D + (D - A)/2          C - D                     (D - A)/ 2
            var scrollTop = inputMidpointOffsetToScrollBottom + scrollMidpointOffset;
            console.log('Scroll top', scrollTop);

            if ( scrollTop > 0) {
              if (ionic.Platform.isIOS()) {
                //just shrank scroll view, give it some breathing room before scrolling
                setTimeout(function(){
                  ionic.tap.cloneFocusedInput(container, self);
                  self.scrollBy(0, scrollTop, true);
                  self.onScroll();
                }, 32);
              } else {
                self.scrollBy(0, scrollTop, true);
                self.onScroll();
              }
            }
            */
          });
        }

        // Only the first scrollView parent of the element that broadcasted this event
        // (the active element that needs to be shown) should receive this event
        e.stopPropagation();
      };

      self.resetScrollView = function() {
        //return scrollview to original height once keyboard has hidden
        if (self.isShrunkForKeyboard) {
          self.isShrunkForKeyboard = false;
          container.style.height = "";

          /*
          if (ionic.Platform.isIOS()) {
            // Force redraw to avoid disappearing content
            var disp = container.style.display;
            container.style.display = 'none';
            var trick = container.offsetHeight;
            container.style.display = disp;
          }
          */

          self.__originalContainerHeight = container.getBoundingClientRect().height;

          if (ionic.Platform.isIOS()) {
            ionic.requestAnimationFrame(function() {
              container.classList.remove('keyboard-up');
            });
          }

        }
        self.resize();
      };

      self.handleTouchMove = function(e) {
        if (self.__frozenShut) {
          e.preventDefault();
          e.stopPropagation();
          return false;

        } else if ( self.__frozen ){
          e.preventDefault();
          // let it propagate so other events such as drag events can happen,
          // but don't let it actually scroll
          return false;
        }
        return true;
      };

      container.addEventListener('scroll', self.onScroll);

      //Broadcasted when keyboard is shown on some platforms.
      //See js/utils/keyboard.js
      container.addEventListener('scrollChildIntoView', self.scrollChildIntoView);

      container.addEventListener(ionic.EVENTS.touchstart, self.handleTouchMove);
      container.addEventListener(ionic.EVENTS.touchmove, self.handleTouchMove);

      // Listen on document because container may not have had the last
      // keyboardActiveElement, for example after closing a modal with a focused
      // input and returning to a previously resized scroll view in an ion-content.
      // Since we can only resize scroll views that are currently visible, just resize
      // the current scroll view when the keyboard is closed.
      document.addEventListener('resetScrollView', self.resetScrollView);
    },

    __cleanup: function() {
      var self = this;
      var container = self.__container;

      container.removeEventListener('scroll', self.onScroll);
      container.removeEventListener('scrollChildIntoView', self.scrollChildIntoView);

      container.removeEventListener(ionic.EVENTS.touchstart, self.handleTouchMove);
      container.removeEventListener(ionic.EVENTS.touchmove, self.handleTouchMove);

      document.removeEventListener('resetScrollView', self.resetScrollView);

      ionic.tap.removeClonedInputs(container, self);

      delete self.__container;
      delete self.__content;
      delete self.__indicatorX;
      delete self.__indicatorY;
      delete self.options.el;

      self.resize = self.scrollTo = self.onScroll = self.resetScrollView = NOOP;
      self.scrollChildIntoView = NOOP;
      container = null;
    }
  });

})(ionic);

(function(ionic) {
'use strict';

  var ITEM_CLASS = 'item';
  var ITEM_CONTENT_CLASS = 'item-content';
  var ITEM_SLIDING_CLASS = 'item-sliding';
  var ITEM_OPTIONS_CLASS = 'item-options';
  var ITEM_PLACEHOLDER_CLASS = 'item-placeholder';
  var ITEM_REORDERING_CLASS = 'item-reordering';
  var ITEM_REORDER_BTN_CLASS = 'item-reorder';

  var DragOp = function() {};
  DragOp.prototype = {
    start: function(){},
    drag: function(){},
    end: function(){},
    isSameItem: function() {
      return false;
    }
  };

  var SlideDrag = function(opts) {
    this.dragThresholdX = opts.dragThresholdX || 10;
    this.el = opts.el;
    this.item = opts.item;
    this.canSwipe = opts.canSwipe;
  };

  SlideDrag.prototype = new DragOp();

  SlideDrag.prototype.start = function(e) {
    var content, buttons, offsetX, buttonsWidth;

    if (!this.canSwipe()) {
      return;
    }

    if (e.target.classList.contains(ITEM_CONTENT_CLASS)) {
      content = e.target;
    } else if (e.target.classList.contains(ITEM_CLASS)) {
      content = e.target.querySelector('.' + ITEM_CONTENT_CLASS);
    } else {
      content = ionic.DomUtil.getParentWithClass(e.target, ITEM_CONTENT_CLASS);
    }

    // If we don't have a content area as one of our children (or ourselves), skip
    if (!content) {
      return;
    }

    // Make sure we aren't animating as we slide
    content.classList.remove(ITEM_SLIDING_CLASS);

    // Grab the starting X point for the item (for example, so we can tell whether it is open or closed to start)
    offsetX = parseFloat(content.style[ionic.CSS.TRANSFORM].replace('translate3d(', '').split(',')[0]) || 0;

    // Grab the buttons
    buttons = content.parentNode.querySelector('.' + ITEM_OPTIONS_CLASS);
    if (!buttons) {
      return;
    }
    buttons.classList.remove('invisible');

    buttonsWidth = buttons.offsetWidth;

    this._currentDrag = {
      buttons: buttons,
      buttonsWidth: buttonsWidth,
      content: content,
      startOffsetX: offsetX
    };
  };

  /**
   * Check if this is the same item that was previously dragged.
   */
  SlideDrag.prototype.isSameItem = function(op) {
    if (op._lastDrag && this._currentDrag) {
      return this._currentDrag.content == op._lastDrag.content;
    }
    return false;
  };

  SlideDrag.prototype.clean = function(isInstant) {
    var lastDrag = this._lastDrag;

    if (!lastDrag || !lastDrag.content) return;

    lastDrag.content.style[ionic.CSS.TRANSITION] = '';
    lastDrag.content.style[ionic.CSS.TRANSFORM] = '';
    if (isInstant) {
      lastDrag.content.style[ionic.CSS.TRANSITION] = 'none';
      makeInvisible();
      ionic.requestAnimationFrame(function() {
        lastDrag.content.style[ionic.CSS.TRANSITION] = '';
      });
    } else {
      ionic.requestAnimationFrame(function() {
        setTimeout(makeInvisible, 250);
      });
    }
    function makeInvisible() {
      lastDrag.buttons && lastDrag.buttons.classList.add('invisible');
    }
  };

  SlideDrag.prototype.drag = ionic.animationFrameThrottle(function(e) {
    var buttonsWidth;

    // We really aren't dragging
    if (!this._currentDrag) {
      return;
    }

    // Check if we should start dragging. Check if we've dragged past the threshold,
    // or we are starting from the open state.
    if (!this._isDragging &&
        ((Math.abs(e.gesture.deltaX) > this.dragThresholdX) ||
        (Math.abs(this._currentDrag.startOffsetX) > 0))) {
      this._isDragging = true;
    }

    if (this._isDragging) {
      buttonsWidth = this._currentDrag.buttonsWidth;

      // Grab the new X point, capping it at zero
      var newX = Math.min(0, this._currentDrag.startOffsetX + e.gesture.deltaX);

      // If the new X position is past the buttons, we need to slow down the drag (rubber band style)
      if (newX < -buttonsWidth) {
        // Calculate the new X position, capped at the top of the buttons
        newX = Math.min(-buttonsWidth, -buttonsWidth + (((e.gesture.deltaX + buttonsWidth) * 0.4)));
      }

      this._currentDrag.content.$$ionicOptionsOpen = newX !== 0;

      this._currentDrag.content.style[ionic.CSS.TRANSFORM] = 'translate3d(' + newX + 'px, 0, 0)';
      this._currentDrag.content.style[ionic.CSS.TRANSITION] = 'none';
    }
  });

  SlideDrag.prototype.end = function(e, doneCallback) {
    var self = this;

    // There is no drag, just end immediately
    if (!self._currentDrag) {
      doneCallback && doneCallback();
      return;
    }

    // If we are currently dragging, we want to snap back into place
    // The final resting point X will be the width of the exposed buttons
    var restingPoint = -self._currentDrag.buttonsWidth;

    // Check if the drag didn't clear the buttons mid-point
    // and we aren't moving fast enough to swipe open
    if (e.gesture.deltaX > -(self._currentDrag.buttonsWidth / 2)) {

      // If we are going left but too slow, or going right, go back to resting
      if (e.gesture.direction == "left" && Math.abs(e.gesture.velocityX) < 0.3) {
        restingPoint = 0;

      } else if (e.gesture.direction == "right") {
        restingPoint = 0;
      }

    }

    ionic.requestAnimationFrame(function() {
      if (restingPoint === 0) {
        self._currentDrag.content.style[ionic.CSS.TRANSFORM] = '';
        var buttons = self._currentDrag.buttons;
        setTimeout(function() {
          buttons && buttons.classList.add('invisible');
        }, 250);
      } else {
        self._currentDrag.content.style[ionic.CSS.TRANSFORM] = 'translate3d(' + restingPoint + 'px,0,0)';
      }
      self._currentDrag.content.style[ionic.CSS.TRANSITION] = '';


      // Kill the current drag
      if (!self._lastDrag) {
        self._lastDrag = {};
      }
      ionic.extend(self._lastDrag, self._currentDrag);
      if (self._currentDrag) {
        self._currentDrag.buttons = null;
        self._currentDrag.content = null;
      }
      self._currentDrag = null;

      // We are done, notify caller
      doneCallback && doneCallback();
    });
  };

  var ReorderDrag = function(opts) {
    var self = this;

    self.dragThresholdY = opts.dragThresholdY || 0;
    self.onReorder = opts.onReorder;
    self.listEl = opts.listEl;
    self.el = self.item = opts.el;
    self.scrollEl = opts.scrollEl;
    self.scrollView = opts.scrollView;
    // Get the True Top of the list el http://www.quirksmode.org/js/findpos.html
    self.listElTrueTop = 0;
    if (self.listEl.offsetParent) {
      var obj = self.listEl;
      do {
        self.listElTrueTop += obj.offsetTop;
        obj = obj.offsetParent;
      } while (obj);
    }
  };

  ReorderDrag.prototype = new DragOp();

  ReorderDrag.prototype._moveElement = function(e) {
    var y = e.gesture.center.pageY +
      this.scrollView.getValues().top -
      (this._currentDrag.elementHeight / 2) -
      this.listElTrueTop;
    this.el.style[ionic.CSS.TRANSFORM] = 'translate3d(0, ' + y + 'px, 0)';
  };

  ReorderDrag.prototype.deregister = function() {
    this.listEl = this.el = this.scrollEl = this.scrollView = null;
  };

  ReorderDrag.prototype.start = function(e) {

    var startIndex = ionic.DomUtil.getChildIndex(this.el, this.el.nodeName.toLowerCase());
    var elementHeight = this.el.scrollHeight;
    var placeholder = this.el.cloneNode(true);

    placeholder.classList.add(ITEM_PLACEHOLDER_CLASS);

    this.el.parentNode.insertBefore(placeholder, this.el);
    this.el.classList.add(ITEM_REORDERING_CLASS);

    this._currentDrag = {
      elementHeight: elementHeight,
      startIndex: startIndex,
      placeholder: placeholder,
      scrollHeight: scroll,
      list: placeholder.parentNode
    };

    this._moveElement(e);
  };

  ReorderDrag.prototype.drag = ionic.animationFrameThrottle(function(e) {
    // We really aren't dragging
    var self = this;
    if (!this._currentDrag) {
      return;
    }

    var scrollY = 0;
    var pageY = e.gesture.center.pageY;
    var offset = this.listElTrueTop;

    //If we have a scrollView, check scroll boundaries for dragged element and scroll if necessary
    if (this.scrollView) {

      var container = this.scrollView.__container;
      scrollY = this.scrollView.getValues().top;

      var containerTop = container.offsetTop;
      var pixelsPastTop = containerTop - pageY + this._currentDrag.elementHeight / 2;
      var pixelsPastBottom = pageY + this._currentDrag.elementHeight / 2 - containerTop - container.offsetHeight;

      if (e.gesture.deltaY < 0 && pixelsPastTop > 0 && scrollY > 0) {
        this.scrollView.scrollBy(null, -pixelsPastTop);
        //Trigger another drag so the scrolling keeps going
        ionic.requestAnimationFrame(function() {
          self.drag(e);
        });
      }
      if (e.gesture.deltaY > 0 && pixelsPastBottom > 0) {
        if (scrollY < this.scrollView.getScrollMax().top) {
          this.scrollView.scrollBy(null, pixelsPastBottom);
          //Trigger another drag so the scrolling keeps going
          ionic.requestAnimationFrame(function() {
            self.drag(e);
          });
        }
      }
    }

    // Check if we should start dragging. Check if we've dragged past the threshold,
    // or we are starting from the open state.
    if (!this._isDragging && Math.abs(e.gesture.deltaY) > this.dragThresholdY) {
      this._isDragging = true;
    }

    if (this._isDragging) {
      this._moveElement(e);

      this._currentDrag.currentY = scrollY + pageY - offset;

      // this._reorderItems();
    }
  });

  // When an item is dragged, we need to reorder any items for sorting purposes
  ReorderDrag.prototype._getReorderIndex = function() {
    var self = this;

    var siblings = Array.prototype.slice.call(self._currentDrag.placeholder.parentNode.children)
      .filter(function(el) {
        return el.nodeName === self.el.nodeName && el !== self.el;
      });

    var dragOffsetTop = self._currentDrag.currentY;
    var el;
    for (var i = 0, len = siblings.length; i < len; i++) {
      el = siblings[i];
      if (i === len - 1) {
        if (dragOffsetTop > el.offsetTop) {
          return i;
        }
      } else if (i === 0) {
        if (dragOffsetTop < el.offsetTop + el.offsetHeight) {
          return i;
        }
      } else if (dragOffsetTop > el.offsetTop - el.offsetHeight / 2 &&
                 dragOffsetTop < el.offsetTop + el.offsetHeight) {
        return i;
      }
    }
    return self._currentDrag.startIndex;
  };

  ReorderDrag.prototype.end = function(e, doneCallback) {
    if (!this._currentDrag) {
      doneCallback && doneCallback();
      return;
    }

    var placeholder = this._currentDrag.placeholder;
    var finalIndex = this._getReorderIndex();

    // Reposition the element
    this.el.classList.remove(ITEM_REORDERING_CLASS);
    this.el.style[ionic.CSS.TRANSFORM] = '';

    placeholder.parentNode.insertBefore(this.el, placeholder);
    placeholder.parentNode.removeChild(placeholder);

    this.onReorder && this.onReorder(this.el, this._currentDrag.startIndex, finalIndex);

    this._currentDrag = {
      placeholder: null,
      content: null
    };
    this._currentDrag = null;
    doneCallback && doneCallback();
  };



  /**
   * The ListView handles a list of items. It will process drag animations, edit mode,
   * and other operations that are common on mobile lists or table views.
   */
  ionic.views.ListView = ionic.views.View.inherit({
    initialize: function(opts) {
      var self = this;

      opts = ionic.extend({
        onReorder: function() {},
        virtualRemoveThreshold: -200,
        virtualAddThreshold: 200,
        canSwipe: function() {
          return true;
        }
      }, opts);

      ionic.extend(self, opts);

      if (!self.itemHeight && self.listEl) {
        self.itemHeight = self.listEl.children[0] && parseInt(self.listEl.children[0].style.height, 10);
      }

      self.onRefresh = opts.onRefresh || function() {};
      self.onRefreshOpening = opts.onRefreshOpening || function() {};
      self.onRefreshHolding = opts.onRefreshHolding || function() {};

      var gestureOpts = {};
      // don't prevent native scrolling
      if (ionic.DomUtil.getParentOrSelfWithClass(self.el, 'overflow-scroll')) {
        gestureOpts.prevent_default_directions = ['left', 'right'];
      }

      window.ionic.onGesture('release', function(e) {
        self._handleEndDrag(e);
      }, self.el, gestureOpts);

      window.ionic.onGesture('drag', function(e) {
        self._handleDrag(e);
      }, self.el, gestureOpts);
      // Start the drag states
      self._initDrag();
    },

    /**
     * Be sure to cleanup references.
     */
    deregister: function() {
      this.el = this.listEl = this.scrollEl = this.scrollView = null;

      // ensure no scrolls have been left frozen
      if (this.isScrollFreeze) {
        self.scrollView.freeze(false);
      }
    },

    /**
     * Called to tell the list to stop refreshing. This is useful
     * if you are refreshing the list and are done with refreshing.
     */
    stopRefreshing: function() {
      var refresher = this.el.querySelector('.list-refresher');
      refresher.style.height = '0';
    },

    /**
     * If we scrolled and have virtual mode enabled, compute the window
     * of active elements in order to figure out the viewport to render.
     */
    didScroll: function(e) {
      var self = this;

      if (self.isVirtual) {
        var itemHeight = self.itemHeight;

        // Grab the total height of the list
        var scrollHeight = e.target.scrollHeight;

        // Get the viewport height
        var viewportHeight = self.el.parentNode.offsetHeight;

        // High water is the pixel position of the first element to include (everything before
        // that will be removed)
        var highWater = Math.max(0, e.scrollTop + self.virtualRemoveThreshold);

        // Low water is the pixel position of the last element to include (everything after
        // that will be removed)
        var lowWater = Math.min(scrollHeight, Math.abs(e.scrollTop) + viewportHeight + self.virtualAddThreshold);

        // Get the first and last elements in the list based on how many can fit
        // between the pixel range of lowWater and highWater
        var first = parseInt(Math.abs(highWater / itemHeight), 10);
        var last = parseInt(Math.abs(lowWater / itemHeight), 10);

        // Get the items we need to remove
        self._virtualItemsToRemove = Array.prototype.slice.call(self.listEl.children, 0, first);

        self.renderViewport && self.renderViewport(highWater, lowWater, first, last);
      }
    },

    didStopScrolling: function() {
      if (this.isVirtual) {
        for (var i = 0; i < this._virtualItemsToRemove.length; i++) {
          //el.parentNode.removeChild(el);
          this.didHideItem && this.didHideItem(i);
        }
        // Once scrolling stops, check if we need to remove old items

      }
    },

    /**
     * Clear any active drag effects on the list.
     */
    clearDragEffects: function(isInstant) {
      if (this._lastDragOp) {
        this._lastDragOp.clean && this._lastDragOp.clean(isInstant);
        this._lastDragOp.deregister && this._lastDragOp.deregister();
        this._lastDragOp = null;
      }
    },

    _initDrag: function() {
      // Store the last one
      if (this._lastDragOp) {
        this._lastDragOp.deregister && this._lastDragOp.deregister();
      }
      this._lastDragOp = this._dragOp;

      this._dragOp = null;
    },

    // Return the list item from the given target
    _getItem: function(target) {
      while (target) {
        if (target.classList && target.classList.contains(ITEM_CLASS)) {
          return target;
        }
        target = target.parentNode;
      }
      return null;
    },


    _startDrag: function(e) {
      var self = this;

      self._isDragging = false;

      var lastDragOp = self._lastDragOp;
      var item;

      // If we have an open SlideDrag and we're scrolling the list. Clear it.
      if (self._didDragUpOrDown && lastDragOp instanceof SlideDrag) {
          lastDragOp.clean && lastDragOp.clean();
      }

      // Check if this is a reorder drag
      if (ionic.DomUtil.getParentOrSelfWithClass(e.target, ITEM_REORDER_BTN_CLASS) && (e.gesture.direction == 'up' || e.gesture.direction == 'down')) {
        item = self._getItem(e.target);

        if (item) {
          self._dragOp = new ReorderDrag({
            listEl: self.el,
            el: item,
            scrollEl: self.scrollEl,
            scrollView: self.scrollView,
            onReorder: function(el, start, end) {
              self.onReorder && self.onReorder(el, start, end);
            }
          });
          self._dragOp.start(e);
          e.preventDefault();
        }
      }

      // Or check if this is a swipe to the side drag
      else if (!self._didDragUpOrDown && (e.gesture.direction == 'left' || e.gesture.direction == 'right') && Math.abs(e.gesture.deltaX) > 5) {

        // Make sure this is an item with buttons
        item = self._getItem(e.target);
        if (item && item.querySelector('.item-options')) {
          self._dragOp = new SlideDrag({
            el: self.el,
            item: item,
            canSwipe: self.canSwipe
          });
          self._dragOp.start(e);
          e.preventDefault();
          self.isScrollFreeze = self.scrollView.freeze(true);
        }
      }

      // If we had a last drag operation and this is a new one on a different item, clean that last one
      if (lastDragOp && self._dragOp && !self._dragOp.isSameItem(lastDragOp) && e.defaultPrevented) {
        lastDragOp.clean && lastDragOp.clean();
      }
    },


    _handleEndDrag: function(e) {
      var self = this;

      if (self.scrollView) {
        self.isScrollFreeze = self.scrollView.freeze(false);
      }

      self._didDragUpOrDown = false;

      if (!self._dragOp) {
        return;
      }

      self._dragOp.end(e, function() {
        self._initDrag();
      });
    },

    /**
     * Process the drag event to move the item to the left or right.
     */
    _handleDrag: function(e) {
      var self = this;

      if (Math.abs(e.gesture.deltaY) > 5) {
        self._didDragUpOrDown = true;
      }

      // If we get a drag event, make sure we aren't in another drag, then check if we should
      // start one
      if (!self.isDragging && !self._dragOp) {
        self._startDrag(e);
      }

      // No drag still, pass it up
      if (!self._dragOp) {
        return;
      }

      e.gesture.srcEvent.preventDefault();
      self._dragOp.drag(e);
    }

  });

})(ionic);

(function(ionic) {
'use strict';

  ionic.views.Modal = ionic.views.View.inherit({
    initialize: function(opts) {
      opts = ionic.extend({
        focusFirstInput: false,
        unfocusOnHide: true,
        focusFirstDelay: 600,
        backdropClickToClose: true,
        hardwareBackButtonClose: true,
      }, opts);

      ionic.extend(this, opts);

      this.el = opts.el;
    },
    show: function() {
      var self = this;

      if(self.focusFirstInput) {
        // Let any animations run first
        window.setTimeout(function() {
          var input = self.el.querySelector('input, textarea');
          input && input.focus && input.focus();
        }, self.focusFirstDelay);
      }
    },
    hide: function() {
      // Unfocus all elements
      if(this.unfocusOnHide) {
        var inputs = this.el.querySelectorAll('input, textarea');
        // Let any animations run first
        window.setTimeout(function() {
          for(var i = 0; i < inputs.length; i++) {
            inputs[i].blur && inputs[i].blur();
          }
        });
      }
    }
  });

})(ionic);

(function(ionic) {
'use strict';

  /**
   * The side menu view handles one of the side menu's in a Side Menu Controller
   * configuration.
   * It takes a DOM reference to that side menu element.
   */
  ionic.views.SideMenu = ionic.views.View.inherit({
    initialize: function(opts) {
      this.el = opts.el;
      this.isEnabled = (typeof opts.isEnabled === 'undefined') ? true : opts.isEnabled;
      this.setWidth(opts.width);
    },
    getFullWidth: function() {
      return this.width;
    },
    setWidth: function(width) {
      this.width = width;
      this.el.style.width = width + 'px';
    },
    setIsEnabled: function(isEnabled) {
      this.isEnabled = isEnabled;
    },
    bringUp: function() {
      if(this.el.style.zIndex !== '0') {
        this.el.style.zIndex = '0';
      }
    },
    pushDown: function() {
      if(this.el.style.zIndex !== '-1') {
        this.el.style.zIndex = '-1';
      }
    }
  });

  ionic.views.SideMenuContent = ionic.views.View.inherit({
    initialize: function(opts) {
      ionic.extend(this, {
        animationClass: 'menu-animated',
        onDrag: function() {},
        onEndDrag: function() {}
      }, opts);

      ionic.onGesture('drag', ionic.proxy(this._onDrag, this), this.el);
      ionic.onGesture('release', ionic.proxy(this._onEndDrag, this), this.el);
    },
    _onDrag: function(e) {
      this.onDrag && this.onDrag(e);
    },
    _onEndDrag: function(e) {
      this.onEndDrag && this.onEndDrag(e);
    },
    disableAnimation: function() {
      this.el.classList.remove(this.animationClass);
    },
    enableAnimation: function() {
      this.el.classList.add(this.animationClass);
    },
    getTranslateX: function() {
      return parseFloat(this.el.style[ionic.CSS.TRANSFORM].replace('translate3d(', '').split(',')[0]);
    },
    setTranslateX: ionic.animationFrameThrottle(function(x) {
      this.el.style[ionic.CSS.TRANSFORM] = 'translate3d(' + x + 'px, 0, 0)';
    })
  });

})(ionic);

/*
 * Adapted from Swipe.js 2.0
 *
 * Brad Birdsall
 * Copyright 2013, MIT License
 *
*/

(function(ionic) {
'use strict';

ionic.views.Slider = ionic.views.View.inherit({
  initialize: function (options) {
    var slider = this;

    var touchStartEvent, touchMoveEvent, touchEndEvent;
    if (window.navigator.pointerEnabled) {
      touchStartEvent = 'pointerdown';
      touchMoveEvent = 'pointermove';
      touchEndEvent = 'pointerup';
    } else if (window.navigator.msPointerEnabled) {
      touchStartEvent = 'MSPointerDown';
      touchMoveEvent = 'MSPointerMove';
      touchEndEvent = 'MSPointerUp';
    } else {
      touchStartEvent = 'touchstart';
      touchMoveEvent = 'touchmove';
      touchEndEvent = 'touchend';
    }

    var mouseStartEvent = 'mousedown';
    var mouseMoveEvent = 'mousemove';
    var mouseEndEvent = 'mouseup';

    // utilities
    var noop = function() {}; // simple no operation function
    var offloadFn = function(fn) { setTimeout(fn || noop, 0); }; // offload a functions execution

    // check browser capabilities
    var browser = {
      addEventListener: !!window.addEventListener,
      transitions: (function(temp) {
        var props = ['transitionProperty', 'WebkitTransition', 'MozTransition', 'OTransition', 'msTransition'];
        for ( var i in props ) if (temp.style[ props[i] ] !== undefined) return true;
        return false;
      })(document.createElement('swipe'))
    };


    var container = options.el;

    // quit if no root element
    if (!container) return;
    var element = container.children[0];
    var slides, slidePos, width, length;
    options = options || {};
    var index = parseInt(options.startSlide, 10) || 0;
    var speed = options.speed || 300;
    options.continuous = options.continuous !== undefined ? options.continuous : true;

    function setup() {

      // do not setup if the container has no width
      if (!container.offsetWidth) {
        return;
      }

      // cache slides
      slides = element.children;
      length = slides.length;

      // set continuous to false if only one slide
      if (slides.length < 2) options.continuous = false;

      //special case if two slides
      if (browser.transitions && options.continuous && slides.length < 3) {
        element.appendChild(slides[0].cloneNode(true));
        element.appendChild(element.children[1].cloneNode(true));
        slides = element.children;
      }

      // create an array to store current positions of each slide
      slidePos = new Array(slides.length);

      // determine width of each slide
      width = container.offsetWidth || container.getBoundingClientRect().width;

      element.style.width = (slides.length * width) + 'px';

      // stack elements
      var pos = slides.length;
      while(pos--) {

        var slide = slides[pos];

        slide.style.width = width + 'px';
        slide.setAttribute('data-index', pos);

        if (browser.transitions) {
          slide.style.left = (pos * -width) + 'px';
          move(pos, index > pos ? -width : (index < pos ? width : 0), 0);
        }

      }

      // reposition elements before and after index
      if (options.continuous && browser.transitions) {
        move(circle(index - 1), -width, 0);
        move(circle(index + 1), width, 0);
      }

      if (!browser.transitions) element.style.left = (index * -width) + 'px';

      container.style.visibility = 'visible';

      options.slidesChanged && options.slidesChanged();
    }

    function prev(slideSpeed) {

      if (options.continuous) slide(index - 1, slideSpeed);
      else if (index) slide(index - 1, slideSpeed);

    }

    function next(slideSpeed) {

      if (options.continuous) slide(index + 1, slideSpeed);
      else if (index < slides.length - 1) slide(index + 1, slideSpeed);

    }

    function circle(index) {

      // a simple positive modulo using slides.length
      return (slides.length + (index % slides.length)) % slides.length;

    }

    function slide(to, slideSpeed) {

      // do nothing if already on requested slide
      if (index == to) return;

      if (!slides) {
        index = to;
        return;
      }

      if (browser.transitions) {

        var direction = Math.abs(index - to) / (index - to); // 1: backward, -1: forward

        // get the actual position of the slide
        if (options.continuous) {
          var naturalDirection = direction;
          direction = -slidePos[circle(to)] / width;

          // if going forward but to < index, use to = slides.length + to
          // if going backward but to > index, use to = -slides.length + to
          if (direction !== naturalDirection) to = -direction * slides.length + to;

        }

        var diff = Math.abs(index - to) - 1;

        // move all the slides between index and to in the right direction
        while (diff--) move( circle((to > index ? to : index) - diff - 1), width * direction, 0);

        to = circle(to);

        move(index, width * direction, slideSpeed || speed);
        move(to, 0, slideSpeed || speed);

        if (options.continuous) move(circle(to - direction), -(width * direction), 0); // we need to get the next in place

      } else {

        to = circle(to);
        animate(index * -width, to * -width, slideSpeed || speed);
        //no fallback for a circular continuous if the browser does not accept transitions
      }

      index = to;
      offloadFn(options.callback && options.callback(index, slides[index]));
    }

    function move(index, dist, speed) {

      translate(index, dist, speed);
      slidePos[index] = dist;

    }

    function translate(index, dist, speed) {

      var slide = slides[index];
      var style = slide && slide.style;

      if (!style) return;

      style.webkitTransitionDuration =
      style.MozTransitionDuration =
      style.msTransitionDuration =
      style.OTransitionDuration =
      style.transitionDuration = speed + 'ms';

      style.webkitTransform = 'translate(' + dist + 'px,0)' + 'translateZ(0)';
      style.msTransform =
      style.MozTransform =
      style.OTransform = 'translateX(' + dist + 'px)';

    }

    function animate(from, to, speed) {

      // if not an animation, just reposition
      if (!speed) {

        element.style.left = to + 'px';
        return;

      }

      var start = +new Date();

      var timer = setInterval(function() {

        var timeElap = +new Date() - start;

        if (timeElap > speed) {

          element.style.left = to + 'px';

          if (delay) begin();

          options.transitionEnd && options.transitionEnd.call(event, index, slides[index]);

          clearInterval(timer);
          return;

        }

        element.style.left = (( (to - from) * (Math.floor((timeElap / speed) * 100) / 100) ) + from) + 'px';

      }, 4);

    }

    // setup auto slideshow
    var delay = options.auto || 0;
    var interval;

    function begin() {

      interval = setTimeout(next, delay);

    }

    function stop() {

      delay = options.auto || 0;
      clearTimeout(interval);

    }


    // setup initial vars
    var start = {};
    var delta = {};
    var isScrolling;

    // setup event capturing
    var events = {

      handleEvent: function(event) {
        if(!event.touches && event.pageX && event.pageY) {
          event.touches = [{
            pageX: event.pageX,
            pageY: event.pageY
          }];
        }

        switch (event.type) {
          case touchStartEvent: this.start(event); break;
          case mouseStartEvent: this.start(event); break;
          case touchMoveEvent: this.touchmove(event); break;
          case mouseMoveEvent: this.touchmove(event); break;
          case touchEndEvent: offloadFn(this.end(event)); break;
          case mouseEndEvent: offloadFn(this.end(event)); break;
          case 'webkitTransitionEnd':
          case 'msTransitionEnd':
          case 'oTransitionEnd':
          case 'otransitionend':
          case 'transitionend': offloadFn(this.transitionEnd(event)); break;
          case 'resize': offloadFn(setup); break;
        }

        if (options.stopPropagation) event.stopPropagation();

      },
      start: function(event) {

        // prevent to start if there is no valid event
        if (!event.touches) {
          return;
        }

        var touches = event.touches[0];

        // measure start values
        start = {

          // get initial touch coords
          x: touches.pageX,
          y: touches.pageY,

          // store time to determine touch duration
          time: +new Date()

        };

        // used for testing first move event
        isScrolling = undefined;

        // reset delta and end measurements
        delta = {};

        // attach touchmove and touchend listeners
        element.addEventListener(touchMoveEvent, this, false);
        element.addEventListener(mouseMoveEvent, this, false);

        element.addEventListener(touchEndEvent, this, false);
        element.addEventListener(mouseEndEvent, this, false);

        document.addEventListener(touchEndEvent, this, false);
        document.addEventListener(mouseEndEvent, this, false);
      },
      touchmove: function(event) {

        // ensure there is a valid event
        // ensure swiping with one touch and not pinching
        // ensure sliding is enabled
        if (!event.touches ||
            event.touches.length > 1 ||
            event.scale && event.scale !== 1 ||
            slider.slideIsDisabled) {
          return;
        }

        if (options.disableScroll) event.preventDefault();

        var touches = event.touches[0];

        // measure change in x and y
        delta = {
          x: touches.pageX - start.x,
          y: touches.pageY - start.y
        };

        // determine if scrolling test has run - one time test
        if ( typeof isScrolling == 'undefined') {
          isScrolling = !!( isScrolling || Math.abs(delta.x) < Math.abs(delta.y) );
        }

        // if user is not trying to scroll vertically
        if (!isScrolling) {

          // prevent native scrolling
          event.preventDefault();

          // stop slideshow
          stop();

          // increase resistance if first or last slide
          if (options.continuous) { // we don't add resistance at the end

            translate(circle(index - 1), delta.x + slidePos[circle(index - 1)], 0);
            translate(index, delta.x + slidePos[index], 0);
            translate(circle(index + 1), delta.x + slidePos[circle(index + 1)], 0);

          } else {
            // If the slider bounces, do the bounce!
            if(options.bouncing) {
              delta.x =
               delta.x /
                 ( (!index && delta.x > 0 ||         // if first slide and sliding left
                   index == slides.length - 1 &&     // or if last slide and sliding right
                   delta.x < 0                       // and if sliding at all
                 ) ?
                 ( Math.abs(delta.x) / width + 1 )      // determine resistance level
                 : 1 );                                 // no resistance if false
             } else {
               if(width * index - delta.x < 0) {               //We are trying scroll past left boundary
                 delta.x = Math.min(delta.x, width * index);  //Set delta.x so we don't go past left screen
               }
               if(Math.abs(delta.x) > width * (slides.length - index - 1)){         //We are trying to scroll past right bondary
                 delta.x = Math.max( -width * (slides.length - index - 1), delta.x);  //Set delta.x so we don't go past right screen
               }
             }

            // translate 1:1
            translate(index - 1, delta.x + slidePos[index - 1], 0);
            translate(index, delta.x + slidePos[index], 0);
            translate(index + 1, delta.x + slidePos[index + 1], 0);
          }

          options.onDrag && options.onDrag();
        }

      },
      end: function() {

        // measure duration
        var duration = +new Date() - start.time;

        // determine if slide attempt triggers next/prev slide
        var isValidSlide =
              Number(duration) < 250 &&         // if slide duration is less than 250ms
              Math.abs(delta.x) > 20 ||         // and if slide amt is greater than 20px
              Math.abs(delta.x) > width / 2;      // or if slide amt is greater than half the width

        // determine if slide attempt is past start and end
        var isPastBounds = (!index && delta.x > 0) ||      // if first slide and slide amt is greater than 0
              (index == slides.length - 1 && delta.x < 0); // or if last slide and slide amt is less than 0

        if (options.continuous) isPastBounds = false;

        // determine direction of swipe (true:right, false:left)
        var direction = delta.x < 0;

        // if not scrolling vertically
        if (!isScrolling) {

          if (isValidSlide && !isPastBounds) {

            if (direction) {

              if (options.continuous) { // we need to get the next in this direction in place

                move(circle(index - 1), -width, 0);
                move(circle(index + 2), width, 0);

              } else {
                move(index - 1, -width, 0);
              }

              move(index, slidePos[index] - width, speed);
              move(circle(index + 1), slidePos[circle(index + 1)] - width, speed);
              index = circle(index + 1);

            } else {
              if (options.continuous) { // we need to get the next in this direction in place

                move(circle(index + 1), width, 0);
                move(circle(index - 2), -width, 0);

              } else {
                move(index + 1, width, 0);
              }

              move(index, slidePos[index] + width, speed);
              move(circle(index - 1), slidePos[circle(index - 1)] + width, speed);
              index = circle(index - 1);

            }

            options.callback && options.callback(index, slides[index]);

          } else {

            if (options.continuous) {

              move(circle(index - 1), -width, speed);
              move(index, 0, speed);
              move(circle(index + 1), width, speed);

            } else {

              move(index - 1, -width, speed);
              move(index, 0, speed);
              move(index + 1, width, speed);
            }

          }

        }

        // kill touchmove and touchend event listeners until touchstart called again
        element.removeEventListener(touchMoveEvent, events, false);
        element.removeEventListener(mouseMoveEvent, events, false);

        element.removeEventListener(touchEndEvent, events, false);
        element.removeEventListener(mouseEndEvent, events, false);

        document.removeEventListener(touchEndEvent, events, false);
        document.removeEventListener(mouseEndEvent, events, false);

        options.onDragEnd && options.onDragEnd();
      },
      transitionEnd: function(event) {

        if (parseInt(event.target.getAttribute('data-index'), 10) == index) {

          if (delay) begin();

          options.transitionEnd && options.transitionEnd.call(event, index, slides[index]);

        }

      }

    };

    // Public API
    this.update = function() {
      setTimeout(setup);
    };
    this.setup = function() {
      setup();
    };

    this.loop = function(value) {
      if (arguments.length) options.continuous = !!value;
      return options.continuous;
    };

    this.enableSlide = function(shouldEnable) {
      if (arguments.length) {
        this.slideIsDisabled = !shouldEnable;
      }
      return !this.slideIsDisabled;
    };

    this.slide = this.select = function(to, speed) {
      // cancel slideshow
      stop();

      slide(to, speed);
    };

    this.prev = this.previous = function() {
      // cancel slideshow
      stop();

      prev();
    };

    this.next = function() {
      // cancel slideshow
      stop();

      next();
    };

    this.stop = function() {
      // cancel slideshow
      stop();
    };

    this.start = function() {
      begin();
    };

    this.autoPlay = function(newDelay) {
      if (!delay || delay < 0) {
        stop();
      } else {
        delay = newDelay;
        begin();
      }
    };

    this.currentIndex = this.selected = function() {
      // return current index position
      return index;
    };

    this.slidesCount = this.count = function() {
      // return total number of slides
      return length;
    };

    this.kill = function() {
      // cancel slideshow
      stop();

      // reset element
      element.style.width = '';
      element.style.left = '';

      // reset slides so no refs are held on to
      slides && (slides = []);

      // removed event listeners
      if (browser.addEventListener) {

        // remove current event listeners
        element.removeEventListener(touchStartEvent, events, false);
        element.removeEventListener(mouseStartEvent, events, false);
        element.removeEventListener('webkitTransitionEnd', events, false);
        element.removeEventListener('msTransitionEnd', events, false);
        element.removeEventListener('oTransitionEnd', events, false);
        element.removeEventListener('otransitionend', events, false);
        element.removeEventListener('transitionend', events, false);
        window.removeEventListener('resize', events, false);

      }
      else {

        window.onresize = null;

      }
    };

    this.load = function() {
      // trigger setup
      setup();

      // start auto slideshow if applicable
      if (delay) begin();


      // add event listeners
      if (browser.addEventListener) {

        // set touchstart event on element
        element.addEventListener(touchStartEvent, events, false);
        element.addEventListener(mouseStartEvent, events, false);

        if (browser.transitions) {
          element.addEventListener('webkitTransitionEnd', events, false);
          element.addEventListener('msTransitionEnd', events, false);
          element.addEventListener('oTransitionEnd', events, false);
          element.addEventListener('otransitionend', events, false);
          element.addEventListener('transitionend', events, false);
        }

        // set resize event on window
        window.addEventListener('resize', events, false);

      } else {

        window.onresize = function () { setup(); }; // to play nice with old IE

      }
    };

  }
});

})(ionic);

/*eslint space-after-keywords: 0*/

/**
 * Swiper 3.2.7
 * Most modern mobile touch slider and framework with hardware accelerated transitions
 *
 * http://www.idangero.us/swiper/
 *
 * Copyright 2015, Vladimir Kharlampidi
 * The iDangero.us
 * http://www.idangero.us/
 *
 * Licensed under MIT
 *
 * Released on: December 7, 2015
 */
(function () {
    'use strict';
    var $;
    /*===========================
    Swiper
    ===========================*/
    var Swiper = function (container, params, _scope, $compile) {

        if (!(this instanceof Swiper)) return new Swiper(container, params);

        var defaults = {
            direction: 'horizontal',
            touchEventsTarget: 'container',
            initialSlide: 0,
            speed: 300,
            // autoplay
            autoplay: false,
            autoplayDisableOnInteraction: true,
            // To support iOS's swipe-to-go-back gesture (when being used in-app, with UIWebView).
            iOSEdgeSwipeDetection: false,
            iOSEdgeSwipeThreshold: 20,
            // Free mode
            freeMode: false,
            freeModeMomentum: true,
            freeModeMomentumRatio: 1,
            freeModeMomentumBounce: true,
            freeModeMomentumBounceRatio: 1,
            freeModeSticky: false,
            freeModeMinimumVelocity: 0.02,
            // Autoheight
            autoHeight: false,
            // Set wrapper width
            setWrapperSize: false,
            // Virtual Translate
            virtualTranslate: false,
            // Effects
            effect: 'slide', // 'slide' or 'fade' or 'cube' or 'coverflow'
            coverflow: {
                rotate: 50,
                stretch: 0,
                depth: 100,
                modifier: 1,
                slideShadows : true
            },
            cube: {
                slideShadows: true,
                shadow: true,
                shadowOffset: 20,
                shadowScale: 0.94
            },
            fade: {
                crossFade: false
            },
            // Parallax
            parallax: false,
            // Scrollbar
            scrollbar: null,
            scrollbarHide: true,
            scrollbarDraggable: false,
            scrollbarSnapOnRelease: false,
            // Keyboard Mousewheel
            keyboardControl: false,
            mousewheelControl: false,
            mousewheelReleaseOnEdges: false,
            mousewheelInvert: false,
            mousewheelForceToAxis: false,
            mousewheelSensitivity: 1,
            // Hash Navigation
            hashnav: false,
            // Breakpoints
            breakpoints: undefined,
            // Slides grid
            spaceBetween: 0,
            slidesPerView: 1,
            slidesPerColumn: 1,
            slidesPerColumnFill: 'column',
            slidesPerGroup: 1,
            centeredSlides: false,
            slidesOffsetBefore: 0, // in px
            slidesOffsetAfter: 0, // in px
            // Round length
            roundLengths: false,
            // Touches
            touchRatio: 1,
            touchAngle: 45,
            simulateTouch: true,
            shortSwipes: true,
            longSwipes: true,
            longSwipesRatio: 0.5,
            longSwipesMs: 300,
            followFinger: true,
            onlyExternal: false,
            threshold: 0,
            touchMoveStopPropagation: true,
            // Pagination
            pagination: null,
            paginationElement: 'span',
            paginationClickable: false,
            paginationHide: false,
            paginationBulletRender: null,
            // Resistance
            resistance: true,
            resistanceRatio: 0.85,
            // Next/prev buttons
            nextButton: null,
            prevButton: null,
            // Progress
            watchSlidesProgress: false,
            watchSlidesVisibility: false,
            // Cursor
            grabCursor: false,
            // Clicks
            preventClicks: true,
            preventClicksPropagation: true,
            slideToClickedSlide: false,
            // Lazy Loading
            lazyLoading: false,
            lazyLoadingInPrevNext: false,
            lazyLoadingOnTransitionStart: false,
            // Images
            preloadImages: true,
            updateOnImagesReady: true,
            // loop
            loop: false,
            loopAdditionalSlides: 0,
            loopedSlides: null,
            // Control
            control: undefined,
            controlInverse: false,
            controlBy: 'slide', //or 'container'
            // Swiping/no swiping
            allowSwipeToPrev: true,
            allowSwipeToNext: true,
            swipeHandler: null, //'.swipe-handler',
            noSwiping: true,
            noSwipingClass: 'swiper-no-swiping',
            // NS
            slideClass: 'swiper-slide',
            slideActiveClass: 'swiper-slide-active',
            slideVisibleClass: 'swiper-slide-visible',
            slideDuplicateClass: 'swiper-slide-duplicate',
            slideNextClass: 'swiper-slide-next',
            slidePrevClass: 'swiper-slide-prev',
            wrapperClass: 'swiper-wrapper',
            bulletClass: 'swiper-pagination-bullet',
            bulletActiveClass: 'swiper-pagination-bullet-active',
            buttonDisabledClass: 'swiper-button-disabled',
            paginationHiddenClass: 'swiper-pagination-hidden',
            // Observer
            observer: false,
            observeParents: false,
            // Accessibility
            a11y: false,
            prevSlideMessage: 'Previous slide',
            nextSlideMessage: 'Next slide',
            firstSlideMessage: 'This is the first slide',
            lastSlideMessage: 'This is the last slide',
            paginationBulletMessage: 'Go to slide {{index}}',
            // Callbacks
            runCallbacksOnInit: true
            /*
            Callbacks:
            onInit: function (swiper)
            onDestroy: function (swiper)
            onClick: function (swiper, e)
            onTap: function (swiper, e)
            onDoubleTap: function (swiper, e)
            onSliderMove: function (swiper, e)
            onSlideChangeStart: function (swiper)
            onSlideChangeEnd: function (swiper)
            onTransitionStart: function (swiper)
            onTransitionEnd: function (swiper)
            onImagesReady: function (swiper)
            onProgress: function (swiper, progress)
            onTouchStart: function (swiper, e)
            onTouchMove: function (swiper, e)
            onTouchMoveOpposite: function (swiper, e)
            onTouchEnd: function (swiper, e)
            onReachBeginning: function (swiper)
            onReachEnd: function (swiper)
            onSetTransition: function (swiper, duration)
            onSetTranslate: function (swiper, translate)
            onAutoplayStart: function (swiper)
            onAutoplayStop: function (swiper),
            onLazyImageLoad: function (swiper, slide, image)
            onLazyImageReady: function (swiper, slide, image)
            */

        };
        var initialVirtualTranslate = params && params.virtualTranslate;

        params = params || {};
        var originalParams = {};
        for (var param in params) {
            if (typeof params[param] === 'object' && !(params[param].nodeType || params[param] === window || params[param] === document || (typeof Dom7 !== 'undefined' && params[param] instanceof Dom7) || (typeof jQuery !== 'undefined' && params[param] instanceof jQuery))) {
                originalParams[param] = {};
                for (var deepParam in params[param]) {
                    originalParams[param][deepParam] = params[param][deepParam];
                }
            }
            else {
                originalParams[param] = params[param];
            }
        }
        for (var def in defaults) {
            if (typeof params[def] === 'undefined') {
                params[def] = defaults[def];
            }
            else if (typeof params[def] === 'object') {
                for (var deepDef in defaults[def]) {
                    if (typeof params[def][deepDef] === 'undefined') {
                        params[def][deepDef] = defaults[def][deepDef];
                    }
                }
            }
        }

        // Swiper
        var s = this;

        // Params
        s.params = params;
        s.originalParams = originalParams;

        // Classname
        s.classNames = [];
        /*=========================
          Dom Library and plugins
          ===========================*/
        if (typeof $ !== 'undefined' && typeof Dom7 !== 'undefined'){
            $ = Dom7;
        }
        if (typeof $ === 'undefined') {
            if (typeof Dom7 === 'undefined') {
                $ = window.Dom7 || window.Zepto || window.jQuery;
            }
            else {
                $ = Dom7;
            }
            if (!$) return;
        }
        // Export it to Swiper instance
        s.$ = $;

        /*=========================
          Breakpoints
          ===========================*/
        s.currentBreakpoint = undefined;
        s.getActiveBreakpoint = function () {
            //Get breakpoint for window width
            if (!s.params.breakpoints) return false;
            var breakpoint = false;
            var points = [], point;
            for ( point in s.params.breakpoints ) {
                if (s.params.breakpoints.hasOwnProperty(point)) {
                    points.push(point);
                }
            }
            points.sort(function (a, b) {
                return parseInt(a, 10) > parseInt(b, 10);
            });
            for (var i = 0; i < points.length; i++) {
                point = points[i];
                if (point >= window.innerWidth && !breakpoint) {
                    breakpoint = point;
                }
            }
            return breakpoint || 'max';
        };
        s.setBreakpoint = function () {
            //Set breakpoint for window width and update parameters
            var breakpoint = s.getActiveBreakpoint();
            if (breakpoint && s.currentBreakpoint !== breakpoint) {
                var breakPointsParams = breakpoint in s.params.breakpoints ? s.params.breakpoints[breakpoint] : s.originalParams;
                for ( var param in breakPointsParams ) {
                    s.params[param] = breakPointsParams[param];
                }
                s.currentBreakpoint = breakpoint;
            }
        };
        // Set breakpoint on load
        if (s.params.breakpoints) {
            s.setBreakpoint();
        }

        /*=========================
          Preparation - Define Container, Wrapper and Pagination
          ===========================*/
        s.container = $(container);
        if (s.container.length === 0) return;
        if (s.container.length > 1) {
            s.container.each(function () {
                new Swiper(this, params);
            });
            return;
        }

        // Save instance in container HTML Element and in data
        s.container[0].swiper = s;
        s.container.data('swiper', s);

        s.classNames.push('swiper-container-' + s.params.direction);

        if (s.params.freeMode) {
            s.classNames.push('swiper-container-free-mode');
        }
        if (!s.support.flexbox) {
            s.classNames.push('swiper-container-no-flexbox');
            s.params.slidesPerColumn = 1;
        }
        if (s.params.autoHeight) {
            s.classNames.push('swiper-container-autoheight');
        }
        // Enable slides progress when required
        if (s.params.parallax || s.params.watchSlidesVisibility) {
            s.params.watchSlidesProgress = true;
        }
        // Coverflow / 3D
        if (['cube', 'coverflow'].indexOf(s.params.effect) >= 0) {
            if (s.support.transforms3d) {
                s.params.watchSlidesProgress = true;
                s.classNames.push('swiper-container-3d');
            }
            else {
                s.params.effect = 'slide';
            }
        }
        if (s.params.effect !== 'slide') {
            s.classNames.push('swiper-container-' + s.params.effect);
        }
        if (s.params.effect === 'cube') {
            s.params.resistanceRatio = 0;
            s.params.slidesPerView = 1;
            s.params.slidesPerColumn = 1;
            s.params.slidesPerGroup = 1;
            s.params.centeredSlides = false;
            s.params.spaceBetween = 0;
            s.params.virtualTranslate = true;
            s.params.setWrapperSize = false;
        }
        if (s.params.effect === 'fade') {
            s.params.slidesPerView = 1;
            s.params.slidesPerColumn = 1;
            s.params.slidesPerGroup = 1;
            s.params.watchSlidesProgress = true;
            s.params.spaceBetween = 0;
            if (typeof initialVirtualTranslate === 'undefined') {
                s.params.virtualTranslate = true;
            }
        }

        // Grab Cursor
        if (s.params.grabCursor && s.support.touch) {
            s.params.grabCursor = false;
        }

        // Wrapper
        s.wrapper = s.container.children('.' + s.params.wrapperClass);

        // Pagination
        if (s.params.pagination) {
            s.paginationContainer = $(s.params.pagination);
            if (s.params.paginationClickable) {
                s.paginationContainer.addClass('swiper-pagination-clickable');
            }
        }

        // Is Horizontal
        function isH() {
            return s.params.direction === 'horizontal';
        }

        // RTL
        s.rtl = isH() && (s.container[0].dir.toLowerCase() === 'rtl' || s.container.css('direction') === 'rtl');
        if (s.rtl) {
            s.classNames.push('swiper-container-rtl');
        }

        // Wrong RTL support
        if (s.rtl) {
            s.wrongRTL = s.wrapper.css('display') === '-webkit-box';
        }

        // Columns
        if (s.params.slidesPerColumn > 1) {
            s.classNames.push('swiper-container-multirow');
        }

        // Check for Android
        if (s.device.android) {
            s.classNames.push('swiper-container-android');
        }

        // Add classes
        s.container.addClass(s.classNames.join(' '));

        // Translate
        s.translate = 0;

        // Progress
        s.progress = 0;

        // Velocity
        s.velocity = 0;

        /*=========================
          Locks, unlocks
          ===========================*/
        s.lockSwipeToNext = function () {
            s.params.allowSwipeToNext = false;
        };
        s.lockSwipeToPrev = function () {
            s.params.allowSwipeToPrev = false;
        };
        s.lockSwipes = function () {
            s.params.allowSwipeToNext = s.params.allowSwipeToPrev = false;
        };
        s.unlockSwipeToNext = function () {
            s.params.allowSwipeToNext = true;
        };
        s.unlockSwipeToPrev = function () {
            s.params.allowSwipeToPrev = true;
        };
        s.unlockSwipes = function () {
            s.params.allowSwipeToNext = s.params.allowSwipeToPrev = true;
        };

        /*=========================
          Round helper
          ===========================*/
        function round(a) {
            return Math.floor(a);
        }
        /*=========================
          Set grab cursor
          ===========================*/
        if (s.params.grabCursor) {
            s.container[0].style.cursor = 'move';
            s.container[0].style.cursor = '-webkit-grab';
            s.container[0].style.cursor = '-moz-grab';
            s.container[0].style.cursor = 'grab';
        }
        /*=========================
          Update on Images Ready
          ===========================*/
        s.imagesToLoad = [];
        s.imagesLoaded = 0;

        s.loadImage = function (imgElement, src, srcset, checkForComplete, callback) {
            var image;
            function onReady () {
                if (callback) callback();
            }
            if (!imgElement.complete || !checkForComplete) {
                if (src) {
                    image = new window.Image();
                    image.onload = onReady;
                    image.onerror = onReady;
                    if (srcset) {
                        image.srcset = srcset;
                    }
                    if (src) {
                        image.src = src;
                    }
                } else {
                    onReady();
                }

            } else {//image already loaded...
                onReady();
            }
        };
        s.preloadImages = function () {
            s.imagesToLoad = s.container.find('img');
            function _onReady() {
                if (typeof s === 'undefined' || s === null) return;
                if (s.imagesLoaded !== undefined) s.imagesLoaded++;
                if (s.imagesLoaded === s.imagesToLoad.length) {
                    if (s.params.updateOnImagesReady) s.update();
                    s.emit('onImagesReady', s);
                }
            }
            for (var i = 0; i < s.imagesToLoad.length; i++) {
                s.loadImage(s.imagesToLoad[i], (s.imagesToLoad[i].currentSrc || s.imagesToLoad[i].getAttribute('src')), (s.imagesToLoad[i].srcset || s.imagesToLoad[i].getAttribute('srcset')), true, _onReady);
            }
        };

        /*=========================
          Autoplay
          ===========================*/
        s.autoplayTimeoutId = undefined;
        s.autoplaying = false;
        s.autoplayPaused = false;
        function autoplay() {
            s.autoplayTimeoutId = setTimeout(function () {
                if (s.params.loop) {
                    s.fixLoop();
                    s._slideNext();
                }
                else {
                    if (!s.isEnd) {
                        s._slideNext();
                    }
                    else {
                        if (!params.autoplayStopOnLast) {
                            s._slideTo(0);
                        }
                        else {
                            s.stopAutoplay();
                        }
                    }
                }
            }, s.params.autoplay);
        }
        s.startAutoplay = function () {
            if (typeof s.autoplayTimeoutId !== 'undefined') return false;
            if (!s.params.autoplay) return false;
            if (s.autoplaying) return false;
            s.autoplaying = true;
            s.emit('onAutoplayStart', s);
            autoplay();
        };
        s.stopAutoplay = function (internal) {
            if (!s.autoplayTimeoutId) return;
            if (s.autoplayTimeoutId) clearTimeout(s.autoplayTimeoutId);
            s.autoplaying = false;
            s.autoplayTimeoutId = undefined;
            s.emit('onAutoplayStop', s);
        };
        s.pauseAutoplay = function (speed) {
            if (s.autoplayPaused) return;
            if (s.autoplayTimeoutId) clearTimeout(s.autoplayTimeoutId);
            s.autoplayPaused = true;
            if (speed === 0) {
                s.autoplayPaused = false;
                autoplay();
            }
            else {
                s.wrapper.transitionEnd(function () {
                    if (!s) return;
                    s.autoplayPaused = false;
                    if (!s.autoplaying) {
                        s.stopAutoplay();
                    }
                    else {
                        autoplay();
                    }
                });
            }
        };
        /*=========================
          Min/Max Translate
          ===========================*/
        s.minTranslate = function () {
            return (-s.snapGrid[0]);
        };
        s.maxTranslate = function () {
            return (-s.snapGrid[s.snapGrid.length - 1]);
        };
        /*=========================
          Slider/slides sizes
          ===========================*/
        s.updateAutoHeight = function () {
            // Update Height
            var newHeight = s.slides.eq(s.activeIndex)[0].offsetHeight;
            if (newHeight) s.wrapper.css('height', s.slides.eq(s.activeIndex)[0].offsetHeight + 'px');
        };
        s.updateContainerSize = function () {
            var width, height;
            if (typeof s.params.width !== 'undefined') {
                width = s.params.width;
            }
            else {
                width = s.container[0].clientWidth;
            }
            if (typeof s.params.height !== 'undefined') {
                height = s.params.height;
            }
            else {
                height = s.container[0].clientHeight;
            }
            if (width === 0 && isH() || height === 0 && !isH()) {
                return;
            }

            //Subtract paddings
            width = width - parseInt(s.container.css('padding-left'), 10) - parseInt(s.container.css('padding-right'), 10);
            height = height - parseInt(s.container.css('padding-top'), 10) - parseInt(s.container.css('padding-bottom'), 10);

            // Store values
            s.width = width;
            s.height = height;
            s.size = isH() ? s.width : s.height;
        };

        s.updateSlidesSize = function () {
            s.slides = s.wrapper.children('.' + s.params.slideClass);
            s.snapGrid = [];
            s.slidesGrid = [];
            s.slidesSizesGrid = [];

            var spaceBetween = s.params.spaceBetween,
                slidePosition = -s.params.slidesOffsetBefore,
                i,
                prevSlideSize = 0,
                index = 0;
            if (typeof spaceBetween === 'string' && spaceBetween.indexOf('%') >= 0) {
                spaceBetween = parseFloat(spaceBetween.replace('%', '')) / 100 * s.size;
            }

            s.virtualSize = -spaceBetween;
            // reset margins
            if (s.rtl) s.slides.css({marginLeft: '', marginTop: ''});
            else s.slides.css({marginRight: '', marginBottom: ''});

            var slidesNumberEvenToRows;
            if (s.params.slidesPerColumn > 1) {
                if (Math.floor(s.slides.length / s.params.slidesPerColumn) === s.slides.length / s.params.slidesPerColumn) {
                    slidesNumberEvenToRows = s.slides.length;
                }
                else {
                    slidesNumberEvenToRows = Math.ceil(s.slides.length / s.params.slidesPerColumn) * s.params.slidesPerColumn;
                }
                if (s.params.slidesPerView !== 'auto' && s.params.slidesPerColumnFill === 'row') {
                    slidesNumberEvenToRows = Math.max(slidesNumberEvenToRows, s.params.slidesPerView * s.params.slidesPerColumn);
                }
            }

            // Calc slides
            var slideSize;
            var slidesPerColumn = s.params.slidesPerColumn;
            var slidesPerRow = slidesNumberEvenToRows / slidesPerColumn;
            var numFullColumns = slidesPerRow - (s.params.slidesPerColumn * slidesPerRow - s.slides.length);
            for (i = 0; i < s.slides.length; i++) {
                slideSize = 0;
                var slide = s.slides.eq(i);
                if (s.params.slidesPerColumn > 1) {
                    // Set slides order
                    var newSlideOrderIndex;
                    var column, row;
                    if (s.params.slidesPerColumnFill === 'column') {
                        column = Math.floor(i / slidesPerColumn);
                        row = i - column * slidesPerColumn;
                        if (column > numFullColumns || (column === numFullColumns && row === slidesPerColumn-1)) {
                            if (++row >= slidesPerColumn) {
                                row = 0;
                                column++;
                            }
                        }
                        newSlideOrderIndex = column + row * slidesNumberEvenToRows / slidesPerColumn;
                        slide
                            .css({
                                '-webkit-box-ordinal-group': newSlideOrderIndex,
                                '-moz-box-ordinal-group': newSlideOrderIndex,
                                '-ms-flex-order': newSlideOrderIndex,
                                '-webkit-order': newSlideOrderIndex,
                                'order': newSlideOrderIndex
                            });
                    }
                    else {
                        row = Math.floor(i / slidesPerRow);
                        column = i - row * slidesPerRow;
                    }
                    slide
                        .css({
                            'margin-top': (row !== 0 && s.params.spaceBetween) && (s.params.spaceBetween + 'px')
                        })
                        .attr('data-swiper-column', column)
                        .attr('data-swiper-row', row);

                }
                if (slide.css('display') === 'none') continue;
                if (s.params.slidesPerView === 'auto') {
                    slideSize = isH() ? slide.outerWidth(true) : slide.outerHeight(true);
                    if (s.params.roundLengths) slideSize = round(slideSize);
                }
                else {
                    slideSize = (s.size - (s.params.slidesPerView - 1) * spaceBetween) / s.params.slidesPerView;
                    if (s.params.roundLengths) slideSize = round(slideSize);

                    if (isH()) {
                        s.slides[i].style.width = slideSize + 'px';
                    }
                    else {
                        s.slides[i].style.height = slideSize + 'px';
                    }
                }
                s.slides[i].swiperSlideSize = slideSize;
                s.slidesSizesGrid.push(slideSize);


                if (s.params.centeredSlides) {
                    slidePosition = slidePosition + slideSize / 2 + prevSlideSize / 2 + spaceBetween;
                    if (i === 0) slidePosition = slidePosition - s.size / 2 - spaceBetween;
                    if (Math.abs(slidePosition) < 1 / 1000) slidePosition = 0;
                    if ((index) % s.params.slidesPerGroup === 0) s.snapGrid.push(slidePosition);
                    s.slidesGrid.push(slidePosition);
                }
                else {
                    if ((index) % s.params.slidesPerGroup === 0) s.snapGrid.push(slidePosition);
                    s.slidesGrid.push(slidePosition);
                    slidePosition = slidePosition + slideSize + spaceBetween;
                }

                s.virtualSize += slideSize + spaceBetween;

                prevSlideSize = slideSize;

                index ++;
            }
            s.virtualSize = Math.max(s.virtualSize, s.size) + s.params.slidesOffsetAfter;
            var newSlidesGrid;

            if (
                s.rtl && s.wrongRTL && (s.params.effect === 'slide' || s.params.effect === 'coverflow')) {
                s.wrapper.css({width: s.virtualSize + s.params.spaceBetween + 'px'});
            }
            if (!s.support.flexbox || s.params.setWrapperSize) {
                if (isH()) s.wrapper.css({width: s.virtualSize + s.params.spaceBetween + 'px'});
                else s.wrapper.css({height: s.virtualSize + s.params.spaceBetween + 'px'});
            }

            if (s.params.slidesPerColumn > 1) {
                s.virtualSize = (slideSize + s.params.spaceBetween) * slidesNumberEvenToRows;
                s.virtualSize = Math.ceil(s.virtualSize / s.params.slidesPerColumn) - s.params.spaceBetween;
                s.wrapper.css({width: s.virtualSize + s.params.spaceBetween + 'px'});
                if (s.params.centeredSlides) {
                    newSlidesGrid = [];
                    for (i = 0; i < s.snapGrid.length; i++) {
                        if (s.snapGrid[i] < s.virtualSize + s.snapGrid[0]) newSlidesGrid.push(s.snapGrid[i]);
                    }
                    s.snapGrid = newSlidesGrid;
                }
            }

            // Remove last grid elements depending on width
            if (!s.params.centeredSlides) {
                newSlidesGrid = [];
                for (i = 0; i < s.snapGrid.length; i++) {
                    if (s.snapGrid[i] <= s.virtualSize - s.size) {
                        newSlidesGrid.push(s.snapGrid[i]);
                    }
                }
                s.snapGrid = newSlidesGrid;
                if (Math.floor(s.virtualSize - s.size) > Math.floor(s.snapGrid[s.snapGrid.length - 1])) {
                    s.snapGrid.push(s.virtualSize - s.size);
                }
            }
            if (s.snapGrid.length === 0) s.snapGrid = [0];

            if (s.params.spaceBetween !== 0) {
                if (isH()) {
                    if (s.rtl) s.slides.css({marginLeft: spaceBetween + 'px'});
                    else s.slides.css({marginRight: spaceBetween + 'px'});
                }
                else s.slides.css({marginBottom: spaceBetween + 'px'});
            }
            if (s.params.watchSlidesProgress) {
                s.updateSlidesOffset();
            }
        };
        s.updateSlidesOffset = function () {
            for (var i = 0; i < s.slides.length; i++) {
                s.slides[i].swiperSlideOffset = isH() ? s.slides[i].offsetLeft : s.slides[i].offsetTop;
            }
        };

        /*=========================
          Slider/slides progress
          ===========================*/
        s.updateSlidesProgress = function (translate) {
            if (typeof translate === 'undefined') {
                translate = s.translate || 0;
            }
            if (s.slides.length === 0) return;
            if (typeof s.slides[0].swiperSlideOffset === 'undefined') s.updateSlidesOffset();

            var offsetCenter = -translate;
            if (s.rtl) offsetCenter = translate;

            // Visible Slides
            s.slides.removeClass(s.params.slideVisibleClass);
            for (var i = 0; i < s.slides.length; i++) {
                var slide = s.slides[i];
                var slideProgress = (offsetCenter - slide.swiperSlideOffset) / (slide.swiperSlideSize + s.params.spaceBetween);
                if (s.params.watchSlidesVisibility) {
                    var slideBefore = -(offsetCenter - slide.swiperSlideOffset);
                    var slideAfter = slideBefore + s.slidesSizesGrid[i];
                    var isVisible =
                        (slideBefore >= 0 && slideBefore < s.size) ||
                        (slideAfter > 0 && slideAfter <= s.size) ||
                        (slideBefore <= 0 && slideAfter >= s.size);
                    if (isVisible) {
                        s.slides.eq(i).addClass(s.params.slideVisibleClass);
                    }
                }
                slide.progress = s.rtl ? -slideProgress : slideProgress;
            }
        };
        s.updateProgress = function (translate) {
            if (typeof translate === 'undefined') {
                translate = s.translate || 0;
            }
            var translatesDiff = s.maxTranslate() - s.minTranslate();
            var wasBeginning = s.isBeginning;
            var wasEnd = s.isEnd;
            if (translatesDiff === 0) {
                s.progress = 0;
                s.isBeginning = s.isEnd = true;
            }
            else {
                s.progress = (translate - s.minTranslate()) / (translatesDiff);
                s.isBeginning = s.progress <= 0;
                s.isEnd = s.progress >= 1;
            }
            if (s.isBeginning && !wasBeginning) s.emit('onReachBeginning', s);
            if (s.isEnd && !wasEnd) s.emit('onReachEnd', s);

            if (s.params.watchSlidesProgress) s.updateSlidesProgress(translate);
            s.emit('onProgress', s, s.progress);
        };
        s.updateActiveIndex = function () {
            var translate = s.rtl ? s.translate : -s.translate;
            var newActiveIndex, i, snapIndex;
            for (i = 0; i < s.slidesGrid.length; i ++) {
                if (typeof s.slidesGrid[i + 1] !== 'undefined') {
                    if (translate >= s.slidesGrid[i] && translate < s.slidesGrid[i + 1] - (s.slidesGrid[i + 1] - s.slidesGrid[i]) / 2) {
                        newActiveIndex = i;
                    }
                    else if (translate >= s.slidesGrid[i] && translate < s.slidesGrid[i + 1]) {
                        newActiveIndex = i + 1;
                    }
                }
                else {
                    if (translate >= s.slidesGrid[i]) {
                        newActiveIndex = i;
                    }
                }
            }
            // Normalize slideIndex
            if (newActiveIndex < 0 || typeof newActiveIndex === 'undefined') newActiveIndex = 0;
            // for (i = 0; i < s.slidesGrid.length; i++) {
                // if (- translate >= s.slidesGrid[i]) {
                    // newActiveIndex = i;
                // }
            // }
            snapIndex = Math.floor(newActiveIndex / s.params.slidesPerGroup);
            if (snapIndex >= s.snapGrid.length) snapIndex = s.snapGrid.length - 1;

            if (newActiveIndex === s.activeIndex) {
                return;
            }
            s.snapIndex = snapIndex;
            s.previousIndex = s.activeIndex;
            s.activeIndex = newActiveIndex;
            s.updateClasses();
        };

        /*=========================
          Classes
          ===========================*/
        s.updateClasses = function () {
            s.slides.removeClass(s.params.slideActiveClass + ' ' + s.params.slideNextClass + ' ' + s.params.slidePrevClass);
            var activeSlide = s.slides.eq(s.activeIndex);
            // Active classes
            activeSlide.addClass(s.params.slideActiveClass);
            activeSlide.next('.' + s.params.slideClass).addClass(s.params.slideNextClass);
            activeSlide.prev('.' + s.params.slideClass).addClass(s.params.slidePrevClass);

            // Pagination
            if (s.bullets && s.bullets.length > 0) {
                s.bullets.removeClass(s.params.bulletActiveClass);
                var bulletIndex;
                if (s.params.loop) {
                    bulletIndex = Math.ceil(s.activeIndex - s.loopedSlides)/s.params.slidesPerGroup;
                    if (bulletIndex > s.slides.length - 1 - s.loopedSlides * 2) {
                        bulletIndex = bulletIndex - (s.slides.length - s.loopedSlides * 2);
                    }
                    if (bulletIndex > s.bullets.length - 1) bulletIndex = bulletIndex - s.bullets.length;
                }
                else {
                    if (typeof s.snapIndex !== 'undefined') {
                        bulletIndex = s.snapIndex;
                    }
                    else {
                        bulletIndex = s.activeIndex || 0;
                    }
                }
                if (s.paginationContainer.length > 1) {
                    s.bullets.each(function () {
                        if ($(this).index() === bulletIndex) $(this).addClass(s.params.bulletActiveClass);
                    });
                }
                else {
                    s.bullets.eq(bulletIndex).addClass(s.params.bulletActiveClass);
                }
            }

            // Next/active buttons
            if (!s.params.loop) {
                if (s.params.prevButton) {
                    if (s.isBeginning) {
                        $(s.params.prevButton).addClass(s.params.buttonDisabledClass);
                        if (s.params.a11y && s.a11y) s.a11y.disable($(s.params.prevButton));
                    }
                    else {
                        $(s.params.prevButton).removeClass(s.params.buttonDisabledClass);
                        if (s.params.a11y && s.a11y) s.a11y.enable($(s.params.prevButton));
                    }
                }
                if (s.params.nextButton) {
                    if (s.isEnd) {
                        $(s.params.nextButton).addClass(s.params.buttonDisabledClass);
                        if (s.params.a11y && s.a11y) s.a11y.disable($(s.params.nextButton));
                    }
                    else {
                        $(s.params.nextButton).removeClass(s.params.buttonDisabledClass);
                        if (s.params.a11y && s.a11y) s.a11y.enable($(s.params.nextButton));
                    }
                }
            }
        };

        /*=========================
          Pagination
          ===========================*/
        s.updatePagination = function () {
            if (!s.params.pagination) return;
            if (s.paginationContainer && s.paginationContainer.length > 0) {
                var bulletsHTML = '';
                var numberOfBullets = s.params.loop ? Math.ceil((s.slides.length - s.loopedSlides * 2) / s.params.slidesPerGroup) : s.snapGrid.length;
                for (var i = 0; i < numberOfBullets; i++) {
                    if (s.params.paginationBulletRender) {
                        bulletsHTML += s.params.paginationBulletRender(i, s.params.bulletClass);
                    }
                    else {
                        bulletsHTML += '<' + s.params.paginationElement+' class="' + s.params.bulletClass + '"></' + s.params.paginationElement + '>';
                    }
                }
                s.paginationContainer.html(bulletsHTML);
                s.bullets = s.paginationContainer.find('.' + s.params.bulletClass);
                if (s.params.paginationClickable && s.params.a11y && s.a11y) {
                    s.a11y.initPagination();
                }
            }
        };
        /*=========================
          Common update method
          ===========================*/
        s.update = function (updateTranslate) {
            s.updateContainerSize();
            s.updateSlidesSize();
            s.updateProgress();
            s.updatePagination();
            s.updateClasses();
            if (s.params.scrollbar && s.scrollbar) {
                s.scrollbar.set();
            }
            function forceSetTranslate() {
                newTranslate = Math.min(Math.max(s.translate, s.maxTranslate()), s.minTranslate());
                s.setWrapperTranslate(newTranslate);
                s.updateActiveIndex();
                s.updateClasses();
            }
            if (updateTranslate) {
                var translated, newTranslate;
                if (s.controller && s.controller.spline) {
                    s.controller.spline = undefined;
                }
                if (s.params.freeMode) {
                    forceSetTranslate();
                    if (s.params.autoHeight) {
                        s.updateAutoHeight();
                    }
                }
                else {
                    if ((s.params.slidesPerView === 'auto' || s.params.slidesPerView > 1) && s.isEnd && !s.params.centeredSlides) {
                        translated = s.slideTo(s.slides.length - 1, 0, false, true);
                    }
                    else {
                        translated = s.slideTo(s.activeIndex, 0, false, true);
                    }
                    if (!translated) {
                        forceSetTranslate();
                    }
                }
            }
            else if (s.params.autoHeight) {
                s.updateAutoHeight();
            }
        };

        /*=========================
          Resize Handler
          ===========================*/
        s.onResize = function (forceUpdatePagination) {
            //Breakpoints
            if (s.params.breakpoints) {
                s.setBreakpoint();
            }

            // Disable locks on resize
            var allowSwipeToPrev = s.params.allowSwipeToPrev;
            var allowSwipeToNext = s.params.allowSwipeToNext;
            s.params.allowSwipeToPrev = s.params.allowSwipeToNext = true;

            s.updateContainerSize();
            s.updateSlidesSize();
            if (s.params.slidesPerView === 'auto' || s.params.freeMode || forceUpdatePagination) s.updatePagination();
            if (s.params.scrollbar && s.scrollbar) {
                s.scrollbar.set();
            }
            if (s.controller && s.controller.spline) {
                s.controller.spline = undefined;
            }
            if (s.params.freeMode) {
                var newTranslate = Math.min(Math.max(s.translate, s.maxTranslate()), s.minTranslate());
                s.setWrapperTranslate(newTranslate);
                s.updateActiveIndex();
                s.updateClasses();

                if (s.params.autoHeight) {
                    s.updateAutoHeight();
                }
            }
            else {
                s.updateClasses();
                if ((s.params.slidesPerView === 'auto' || s.params.slidesPerView > 1) && s.isEnd && !s.params.centeredSlides) {
                    s.slideTo(s.slides.length - 1, 0, false, true);
                }
                else {
                    s.slideTo(s.activeIndex, 0, false, true);
                }
            }
            // Return locks after resize
            s.params.allowSwipeToPrev = allowSwipeToPrev;
            s.params.allowSwipeToNext = allowSwipeToNext;
        };

        /*=========================
          Events
          ===========================*/

        //Define Touch Events
        var desktopEvents = ['mousedown', 'mousemove', 'mouseup'];
        if (window.navigator.pointerEnabled) desktopEvents = ['pointerdown', 'pointermove', 'pointerup'];
        else if (window.navigator.msPointerEnabled) desktopEvents = ['MSPointerDown', 'MSPointerMove', 'MSPointerUp'];
        s.touchEvents = {
            start : s.support.touch || !s.params.simulateTouch  ? 'touchstart' : desktopEvents[0],
            move : s.support.touch || !s.params.simulateTouch ? 'touchmove' : desktopEvents[1],
            end : s.support.touch || !s.params.simulateTouch ? 'touchend' : desktopEvents[2]
        };


        // WP8 Touch Events Fix
        if (window.navigator.pointerEnabled || window.navigator.msPointerEnabled) {
            (s.params.touchEventsTarget === 'container' ? s.container : s.wrapper).addClass('swiper-wp8-' + s.params.direction);
        }

        // Attach/detach events
        s.initEvents = function (detach) {
            var actionDom = detach ? 'off' : 'on';
            var action = detach ? 'removeEventListener' : 'addEventListener';
            var touchEventsTarget = s.params.touchEventsTarget === 'container' ? s.container[0] : s.wrapper[0];
            var target = s.support.touch ? touchEventsTarget : document;

            var moveCapture = s.params.nested ? true : false;

            //Touch Events
            if (s.browser.ie) {
                touchEventsTarget[action](s.touchEvents.start, s.onTouchStart, false);
                target[action](s.touchEvents.move, s.onTouchMove, moveCapture);
                target[action](s.touchEvents.end, s.onTouchEnd, false);
            }
            else {
                if (s.support.touch) {
                    touchEventsTarget[action](s.touchEvents.start, s.onTouchStart, false);
                    touchEventsTarget[action](s.touchEvents.move, s.onTouchMove, moveCapture);
                    touchEventsTarget[action](s.touchEvents.end, s.onTouchEnd, false);
                }
                if (params.simulateTouch && !s.device.ios && !s.device.android) {
                    touchEventsTarget[action]('mousedown', s.onTouchStart, false);
                    document[action]('mousemove', s.onTouchMove, moveCapture);
                    document[action]('mouseup', s.onTouchEnd, false);
                }
            }
            window[action]('resize', s.onResize);

            // Next, Prev, Index
            if (s.params.nextButton) {
                $(s.params.nextButton)[actionDom]('click', s.onClickNext);
                if (s.params.a11y && s.a11y) $(s.params.nextButton)[actionDom]('keydown', s.a11y.onEnterKey);
            }
            if (s.params.prevButton) {
                $(s.params.prevButton)[actionDom]('click', s.onClickPrev);
                if (s.params.a11y && s.a11y) $(s.params.prevButton)[actionDom]('keydown', s.a11y.onEnterKey);
            }
            if (s.params.pagination && s.params.paginationClickable) {
                $(s.paginationContainer)[actionDom]('click', '.' + s.params.bulletClass, s.onClickIndex);
                if (s.params.a11y && s.a11y) $(s.paginationContainer)[actionDom]('keydown', '.' + s.params.bulletClass, s.a11y.onEnterKey);
            }

            // Prevent Links Clicks
            if (s.params.preventClicks || s.params.preventClicksPropagation) touchEventsTarget[action]('click', s.preventClicks, true);
        };
        s.attachEvents = function (detach) {
            s.initEvents();
        };
        s.detachEvents = function () {
            s.initEvents(true);
        };

        /*=========================
          Handle Clicks
          ===========================*/
        // Prevent Clicks
        s.allowClick = true;
        s.preventClicks = function (e) {
            if (!s.allowClick) {
                if (s.params.preventClicks) e.preventDefault();
                if (s.params.preventClicksPropagation && s.animating) {
                    e.stopPropagation();
                    e.stopImmediatePropagation();
                }
            }
        };
        // Clicks
        s.onClickNext = function (e) {
            e.preventDefault();
            if (s.isEnd && !s.params.loop) return;
            s.slideNext();
        };
        s.onClickPrev = function (e) {
            e.preventDefault();
            if (s.isBeginning && !s.params.loop) return;
            s.slidePrev();
        };
        s.onClickIndex = function (e) {
            e.preventDefault();
            var index = $(this).index() * s.params.slidesPerGroup;
            if (s.params.loop) index = index + s.loopedSlides;
            s.slideTo(index);
        };

        /*=========================
          Handle Touches
          ===========================*/
        function findElementInEvent(e, selector) {
            var el = $(e.target);
            if (!el.is(selector)) {
                if (typeof selector === 'string') {
                    el = el.parents(selector);
                }
                else if (selector.nodeType) {
                    var found;
                    el.parents().each(function (index, _el) {
                        if (_el === selector) found = selector;
                    });
                    if (!found) return undefined;
                    else return selector;
                }
            }
            if (el.length === 0) {
                return undefined;
            }
            return el[0];
        }
        s.updateClickedSlide = function (e) {
            var slide = findElementInEvent(e, '.' + s.params.slideClass);
            var slideFound = false;
            if (slide) {
                for (var i = 0; i < s.slides.length; i++) {
                    if (s.slides[i] === slide) slideFound = true;
                }
            }

            if (slide && slideFound) {
                s.clickedSlide = slide;
                s.clickedIndex = $(slide).index();
            }
            else {
                s.clickedSlide = undefined;
                s.clickedIndex = undefined;
                return;
            }
            if (s.params.slideToClickedSlide && s.clickedIndex !== undefined && s.clickedIndex !== s.activeIndex) {
                var slideToIndex = s.clickedIndex,
                    realIndex,
                    duplicatedSlides;
                if (s.params.loop) {
                    if (s.animating) return;
                    realIndex = $(s.clickedSlide).attr('data-swiper-slide-index');
                    if (s.params.centeredSlides) {
                        if ((slideToIndex < s.loopedSlides - s.params.slidesPerView/2) || (slideToIndex > s.slides.length - s.loopedSlides + s.params.slidesPerView/2)) {
                            s.fixLoop();
                            slideToIndex = s.wrapper.children('.' + s.params.slideClass + '[data-swiper-slide-index="' + realIndex + '"]:not(.swiper-slide-duplicate)').eq(0).index();
                            setTimeout(function () {
                                s.slideTo(slideToIndex);
                            }, 0);
                        }
                        else {
                            s.slideTo(slideToIndex);
                        }
                    }
                    else {
                        if (slideToIndex > s.slides.length - s.params.slidesPerView) {
                            s.fixLoop();
                            slideToIndex = s.wrapper.children('.' + s.params.slideClass + '[data-swiper-slide-index="' + realIndex + '"]:not(.swiper-slide-duplicate)').eq(0).index();
                            setTimeout(function () {
                                s.slideTo(slideToIndex);
                            }, 0);
                        }
                        else {
                            s.slideTo(slideToIndex);
                        }
                    }
                }
                else {
                    s.slideTo(slideToIndex);
                }
            }
        };

        var isTouched,
            isMoved,
            allowTouchCallbacks,
            touchStartTime,
            isScrolling,
            currentTranslate,
            startTranslate,
            allowThresholdMove,
            // Form elements to match
            formElements = 'input, select, textarea, button',
            // Last click time
            lastClickTime = Date.now(), clickTimeout,
            //Velocities
            velocities = [],
            allowMomentumBounce;

        // Animating Flag
        s.animating = false;

        // Touches information
        s.touches = {
            startX: 0,
            startY: 0,
            currentX: 0,
            currentY: 0,
            diff: 0
        };

        // Touch handlers
        var isTouchEvent, startMoving;
        s.onTouchStart = function (e) {
            if (e.originalEvent) e = e.originalEvent;
            isTouchEvent = e.type === 'touchstart';
            if (!isTouchEvent && 'which' in e && e.which === 3) return;
            if (s.params.noSwiping && findElementInEvent(e, '.' + s.params.noSwipingClass)) {
                s.allowClick = true;
                return;
            }
            if (s.params.swipeHandler) {
                if (!findElementInEvent(e, s.params.swipeHandler)) return;
            }

            var startX = s.touches.currentX = e.type === 'touchstart' ? e.targetTouches[0].pageX : e.pageX;
            var startY = s.touches.currentY = e.type === 'touchstart' ? e.targetTouches[0].pageY : e.pageY;

            // Do NOT start if iOS edge swipe is detected. Otherwise iOS app (UIWebView) cannot swipe-to-go-back anymore
            if(s.device.ios && s.params.iOSEdgeSwipeDetection && startX <= s.params.iOSEdgeSwipeThreshold) {
                return;
            }

            isTouched = true;
            isMoved = false;
            allowTouchCallbacks = true;
            isScrolling = undefined;
            startMoving = undefined;
            s.touches.startX = startX;
            s.touches.startY = startY;
            touchStartTime = Date.now();
            s.allowClick = true;
            s.updateContainerSize();
            s.swipeDirection = undefined;
            if (s.params.threshold > 0) allowThresholdMove = false;
            if (e.type !== 'touchstart') {
                var preventDefault = true;
                if ($(e.target).is(formElements)) preventDefault = false;
                if (document.activeElement && $(document.activeElement).is(formElements)) {
                    document.activeElement.blur();
                }
                if (preventDefault) {
                    e.preventDefault();
                }
            }
            s.emit('onTouchStart', s, e);
        };

        s.onTouchMove = function (e) {
            if (e.originalEvent) e = e.originalEvent;
            if (isTouchEvent && e.type === 'mousemove') return;
            if (e.preventedByNestedSwiper) return;
            if (s.params.onlyExternal) {
                // isMoved = true;
                s.allowClick = false;
                if (isTouched) {
                    s.touches.startX = s.touches.currentX = e.type === 'touchmove' ? e.targetTouches[0].pageX : e.pageX;
                    s.touches.startY = s.touches.currentY = e.type === 'touchmove' ? e.targetTouches[0].pageY : e.pageY;
                    touchStartTime = Date.now();
                }
                return;
            }
            if (isTouchEvent && document.activeElement) {
                if (e.target === document.activeElement && $(e.target).is(formElements)) {
                    isMoved = true;
                    s.allowClick = false;
                    return;
                }
            }
            if (allowTouchCallbacks) {
                s.emit('onTouchMove', s, e);
            }
            if (e.targetTouches && e.targetTouches.length > 1) return;

            s.touches.currentX = e.type === 'touchmove' ? e.targetTouches[0].pageX : e.pageX;
            s.touches.currentY = e.type === 'touchmove' ? e.targetTouches[0].pageY : e.pageY;

            if (typeof isScrolling === 'undefined') {
                var touchAngle = Math.atan2(Math.abs(s.touches.currentY - s.touches.startY), Math.abs(s.touches.currentX - s.touches.startX)) * 180 / Math.PI;
                isScrolling = isH() ? touchAngle > s.params.touchAngle : (90 - touchAngle > s.params.touchAngle);
            }
            if (isScrolling) {
                s.emit('onTouchMoveOpposite', s, e);
            }
            if (typeof startMoving === 'undefined' && s.browser.ieTouch) {
                if (s.touches.currentX !== s.touches.startX || s.touches.currentY !== s.touches.startY) {
                    startMoving = true;
                }
            }
            if (!isTouched) return;
            if (isScrolling)  {
                isTouched = false;
                return;
            }
            if (!startMoving && s.browser.ieTouch) {
                return;
            }
            s.allowClick = false;
            s.emit('onSliderMove', s, e);
            e.preventDefault();
            if (s.params.touchMoveStopPropagation && !s.params.nested) {
                e.stopPropagation();
            }

            if (!isMoved) {
                if (params.loop) {
                    s.fixLoop();
                }
                startTranslate = s.getWrapperTranslate();
                s.setWrapperTransition(0);
                if (s.animating) {
                    s.wrapper.trigger('webkitTransitionEnd transitionend oTransitionEnd MSTransitionEnd msTransitionEnd');
                }
                if (s.params.autoplay && s.autoplaying) {
                    if (s.params.autoplayDisableOnInteraction) {
                        s.stopAutoplay();
                    }
                    else {
                        s.pauseAutoplay();
                    }
                }
                allowMomentumBounce = false;
                //Grab Cursor
                if (s.params.grabCursor) {
                    s.container[0].style.cursor = 'move';
                    s.container[0].style.cursor = '-webkit-grabbing';
                    s.container[0].style.cursor = '-moz-grabbin';
                    s.container[0].style.cursor = 'grabbing';
                }
            }
            isMoved = true;

            var diff = s.touches.diff = isH() ? s.touches.currentX - s.touches.startX : s.touches.currentY - s.touches.startY;

            diff = diff * s.params.touchRatio;
            if (s.rtl) diff = -diff;

            s.swipeDirection = diff > 0 ? 'prev' : 'next';
            currentTranslate = diff + startTranslate;

            var disableParentSwiper = true;
            if ((diff > 0 && currentTranslate > s.minTranslate())) {
                disableParentSwiper = false;
                if (s.params.resistance) currentTranslate = s.minTranslate() - 1 + Math.pow(-s.minTranslate() + startTranslate + diff, s.params.resistanceRatio);
            }
            else if (diff < 0 && currentTranslate < s.maxTranslate()) {
                disableParentSwiper = false;
                if (s.params.resistance) currentTranslate = s.maxTranslate() + 1 - Math.pow(s.maxTranslate() - startTranslate - diff, s.params.resistanceRatio);
            }

            if (disableParentSwiper) {
                e.preventedByNestedSwiper = true;
            }

            // Directions locks
            if (!s.params.allowSwipeToNext && s.swipeDirection === 'next' && currentTranslate < startTranslate) {
                currentTranslate = startTranslate;
            }
            if (!s.params.allowSwipeToPrev && s.swipeDirection === 'prev' && currentTranslate > startTranslate) {
                currentTranslate = startTranslate;
            }

            if (!s.params.followFinger) return;

            // Threshold
            if (s.params.threshold > 0) {
                if (Math.abs(diff) > s.params.threshold || allowThresholdMove) {
                    if (!allowThresholdMove) {
                        allowThresholdMove = true;
                        s.touches.startX = s.touches.currentX;
                        s.touches.startY = s.touches.currentY;
                        currentTranslate = startTranslate;
                        s.touches.diff = isH() ? s.touches.currentX - s.touches.startX : s.touches.currentY - s.touches.startY;
                        return;
                    }
                }
                else {
                    currentTranslate = startTranslate;
                    return;
                }
            }
            // Update active index in free mode
            if (s.params.freeMode || s.params.watchSlidesProgress) {
                s.updateActiveIndex();
            }
            if (s.params.freeMode) {
                //Velocity
                if (velocities.length === 0) {
                    velocities.push({
                        position: s.touches[isH() ? 'startX' : 'startY'],
                        time: touchStartTime
                    });
                }
                velocities.push({
                    position: s.touches[isH() ? 'currentX' : 'currentY'],
                    time: (new window.Date()).getTime()
                });
            }
            // Update progress
            s.updateProgress(currentTranslate);
            // Update translate
            s.setWrapperTranslate(currentTranslate);
        };
        s.onTouchEnd = function (e) {
            if (e.originalEvent) e = e.originalEvent;
            if (allowTouchCallbacks) {
                s.emit('onTouchEnd', s, e);
            }
            allowTouchCallbacks = false;
            if (!isTouched) return;
            //Return Grab Cursor
            if (s.params.grabCursor && isMoved && isTouched) {
                s.container[0].style.cursor = 'move';
                s.container[0].style.cursor = '-webkit-grab';
                s.container[0].style.cursor = '-moz-grab';
                s.container[0].style.cursor = 'grab';
            }

            // Time diff
            var touchEndTime = Date.now();
            var timeDiff = touchEndTime - touchStartTime;

            // Tap, doubleTap, Click
            if (s.allowClick) {
                s.updateClickedSlide(e);
                s.emit('onTap', s, e);
                if (timeDiff < 300 && (touchEndTime - lastClickTime) > 300) {
                    if (clickTimeout) clearTimeout(clickTimeout);
                    clickTimeout = setTimeout(function () {
                        if (!s) return;
                        if (s.params.paginationHide && s.paginationContainer.length > 0 && !$(e.target).hasClass(s.params.bulletClass)) {
                            s.paginationContainer.toggleClass(s.params.paginationHiddenClass);
                        }
                        s.emit('onClick', s, e);
                    }, 300);

                }
                if (timeDiff < 300 && (touchEndTime - lastClickTime) < 300) {
                    if (clickTimeout) clearTimeout(clickTimeout);
                    s.emit('onDoubleTap', s, e);
                }
            }

            lastClickTime = Date.now();
            setTimeout(function () {
                if (s) s.allowClick = true;
            }, 0);

            if (!isTouched || !isMoved || !s.swipeDirection || s.touches.diff === 0 || currentTranslate === startTranslate) {
                isTouched = isMoved = false;
                return;
            }
            isTouched = isMoved = false;

            var currentPos;
            if (s.params.followFinger) {
                currentPos = s.rtl ? s.translate : -s.translate;
            }
            else {
                currentPos = -currentTranslate;
            }
            if (s.params.freeMode) {
                if (currentPos < -s.minTranslate()) {
                    s.slideTo(s.activeIndex);
                    return;
                }
                else if (currentPos > -s.maxTranslate()) {
                    if (s.slides.length < s.snapGrid.length) {
                        s.slideTo(s.snapGrid.length - 1);
                    }
                    else {
                        s.slideTo(s.slides.length - 1);
                    }
                    return;
                }

                if (s.params.freeModeMomentum) {
                    if (velocities.length > 1) {
                        var lastMoveEvent = velocities.pop(), velocityEvent = velocities.pop();

                        var distance = lastMoveEvent.position - velocityEvent.position;
                        var time = lastMoveEvent.time - velocityEvent.time;
                        s.velocity = distance / time;
                        s.velocity = s.velocity / 2;
                        if (Math.abs(s.velocity) < s.params.freeModeMinimumVelocity) {
                            s.velocity = 0;
                        }
                        // this implies that the user stopped moving a finger then released.
                        // There would be no events with distance zero, so the last event is stale.
                        if (time > 150 || (new window.Date().getTime() - lastMoveEvent.time) > 300) {
                            s.velocity = 0;
                        }
                    } else {
                        s.velocity = 0;
                    }

                    velocities.length = 0;
                    var momentumDuration = 1000 * s.params.freeModeMomentumRatio;
                    var momentumDistance = s.velocity * momentumDuration;

                    var newPosition = s.translate + momentumDistance;
                    if (s.rtl) newPosition = - newPosition;
                    var doBounce = false;
                    var afterBouncePosition;
                    var bounceAmount = Math.abs(s.velocity) * 20 * s.params.freeModeMomentumBounceRatio;
                    if (newPosition < s.maxTranslate()) {
                        if (s.params.freeModeMomentumBounce) {
                            if (newPosition + s.maxTranslate() < -bounceAmount) {
                                newPosition = s.maxTranslate() - bounceAmount;
                            }
                            afterBouncePosition = s.maxTranslate();
                            doBounce = true;
                            allowMomentumBounce = true;
                        }
                        else {
                            newPosition = s.maxTranslate();
                        }
                    }
                    else if (newPosition > s.minTranslate()) {
                        if (s.params.freeModeMomentumBounce) {
                            if (newPosition - s.minTranslate() > bounceAmount) {
                                newPosition = s.minTranslate() + bounceAmount;
                            }
                            afterBouncePosition = s.minTranslate();
                            doBounce = true;
                            allowMomentumBounce = true;
                        }
                        else {
                            newPosition = s.minTranslate();
                        }
                    }
                    else if (s.params.freeModeSticky) {
                        var j = 0,
                            nextSlide;
                        for (j = 0; j < s.snapGrid.length; j += 1) {
                            if (s.snapGrid[j] > -newPosition) {
                                nextSlide = j;
                                break;
                            }

                        }
                        if (Math.abs(s.snapGrid[nextSlide] - newPosition) < Math.abs(s.snapGrid[nextSlide - 1] - newPosition) || s.swipeDirection === 'next') {
                            newPosition = s.snapGrid[nextSlide];
                        } else {
                            newPosition = s.snapGrid[nextSlide - 1];
                        }
                        if (!s.rtl) newPosition = - newPosition;
                    }
                    //Fix duration
                    if (s.velocity !== 0) {
                        if (s.rtl) {
                            momentumDuration = Math.abs((-newPosition - s.translate) / s.velocity);
                        }
                        else {
                            momentumDuration = Math.abs((newPosition - s.translate) / s.velocity);
                        }
                    }
                    else if (s.params.freeModeSticky) {
                        s.slideReset();
                        return;
                    }

                    if (s.params.freeModeMomentumBounce && doBounce) {
                        s.updateProgress(afterBouncePosition);
                        s.setWrapperTransition(momentumDuration);
                        s.setWrapperTranslate(newPosition);
                        s.onTransitionStart();
                        s.animating = true;
                        s.wrapper.transitionEnd(function () {
                            if (!s || !allowMomentumBounce) return;
                            s.emit('onMomentumBounce', s);

                            s.setWrapperTransition(s.params.speed);
                            s.setWrapperTranslate(afterBouncePosition);
                            s.wrapper.transitionEnd(function () {
                                if (!s) return;
                                s.onTransitionEnd();
                            });
                        });
                    } else if (s.velocity) {
                        s.updateProgress(newPosition);
                        s.setWrapperTransition(momentumDuration);
                        s.setWrapperTranslate(newPosition);
                        s.onTransitionStart();
                        if (!s.animating) {
                            s.animating = true;
                            s.wrapper.transitionEnd(function () {
                                if (!s) return;
                                s.onTransitionEnd();
                            });
                        }

                    } else {
                        s.updateProgress(newPosition);
                    }

                    s.updateActiveIndex();
                }
                if (!s.params.freeModeMomentum || timeDiff >= s.params.longSwipesMs) {
                    s.updateProgress();
                    s.updateActiveIndex();
                }
                return;
            }

            // Find current slide
            var i, stopIndex = 0, groupSize = s.slidesSizesGrid[0];
            for (i = 0; i < s.slidesGrid.length; i += s.params.slidesPerGroup) {
                if (typeof s.slidesGrid[i + s.params.slidesPerGroup] !== 'undefined') {
                    if (currentPos >= s.slidesGrid[i] && currentPos < s.slidesGrid[i + s.params.slidesPerGroup]) {
                        stopIndex = i;
                        groupSize = s.slidesGrid[i + s.params.slidesPerGroup] - s.slidesGrid[i];
                    }
                }
                else {
                    if (currentPos >= s.slidesGrid[i]) {
                        stopIndex = i;
                        groupSize = s.slidesGrid[s.slidesGrid.length - 1] - s.slidesGrid[s.slidesGrid.length - 2];
                    }
                }
            }

            // Find current slide size
            var ratio = (currentPos - s.slidesGrid[stopIndex]) / groupSize;

            if (timeDiff > s.params.longSwipesMs) {
                // Long touches
                if (!s.params.longSwipes) {
                    s.slideTo(s.activeIndex);
                    return;
                }
                if (s.swipeDirection === 'next') {
                    if (ratio >= s.params.longSwipesRatio) s.slideTo(stopIndex + s.params.slidesPerGroup);
                    else s.slideTo(stopIndex);

                }
                if (s.swipeDirection === 'prev') {
                    if (ratio > (1 - s.params.longSwipesRatio)) s.slideTo(stopIndex + s.params.slidesPerGroup);
                    else s.slideTo(stopIndex);
                }
            }
            else {
                // Short swipes
                if (!s.params.shortSwipes) {
                    s.slideTo(s.activeIndex);
                    return;
                }
                if (s.swipeDirection === 'next') {
                    s.slideTo(stopIndex + s.params.slidesPerGroup);

                }
                if (s.swipeDirection === 'prev') {
                    s.slideTo(stopIndex);
                }
            }
        };
        /*=========================
          Transitions
          ===========================*/
        s._slideTo = function (slideIndex, speed) {
            return s.slideTo(slideIndex, speed, true, true);
        };
        s.slideTo = function (slideIndex, speed, runCallbacks, internal) {
            if (typeof runCallbacks === 'undefined') runCallbacks = true;
            if (typeof slideIndex === 'undefined') slideIndex = 0;
            if (slideIndex < 0) slideIndex = 0;
            s.snapIndex = Math.floor(slideIndex / s.params.slidesPerGroup);
            if (s.snapIndex >= s.snapGrid.length) s.snapIndex = s.snapGrid.length - 1;

            var translate = - s.snapGrid[s.snapIndex];
            // Stop autoplay
            if (s.params.autoplay && s.autoplaying) {
                if (internal || !s.params.autoplayDisableOnInteraction) {
                    s.pauseAutoplay(speed);
                }
                else {
                    s.stopAutoplay();
                }
            }
            // Update progress
            s.updateProgress(translate);

            // Normalize slideIndex
            for (var i = 0; i < s.slidesGrid.length; i++) {
                if (- Math.floor(translate * 100) >= Math.floor(s.slidesGrid[i] * 100)) {
                    slideIndex = i;
                }
            }

            // Directions locks
            if (!s.params.allowSwipeToNext && translate < s.translate && translate < s.minTranslate()) {
                return false;
            }
            if (!s.params.allowSwipeToPrev && translate > s.translate && translate > s.maxTranslate()) {
                if ((s.activeIndex || 0) !== slideIndex ) return false;
            }

            // Update Index
            if (typeof speed === 'undefined') speed = s.params.speed;
            s.previousIndex = s.activeIndex || 0;
            s.activeIndex = slideIndex;

            if ((s.rtl && -translate === s.translate) || (!s.rtl && translate === s.translate)) {
                // Update Height
                if (s.params.autoHeight) {
                    s.updateAutoHeight();
                }
                s.updateClasses();
                if (s.params.effect !== 'slide') {
                    s.setWrapperTranslate(translate);
                }
                return false;
            }
            s.updateClasses();
            s.onTransitionStart(runCallbacks);

            if (speed === 0) {
                s.setWrapperTranslate(translate);
                s.setWrapperTransition(0);
                s.onTransitionEnd(runCallbacks);
            }
            else {
                s.setWrapperTranslate(translate);
                s.setWrapperTransition(speed);
                if (!s.animating) {
                    s.animating = true;
                    s.wrapper.transitionEnd(function () {
                        if (!s) return;
                        s.onTransitionEnd(runCallbacks);
                    });
                }

            }

            return true;
        };

        s.onTransitionStart = function (runCallbacks) {
            if (typeof runCallbacks === 'undefined') runCallbacks = true;
            if (s.params.autoHeight) {
                s.updateAutoHeight();
            }
            if (s.lazy) s.lazy.onTransitionStart();
            if (runCallbacks) {
                s.emit('onTransitionStart', s);
                if (s.activeIndex !== s.previousIndex) {
                    s.emit('onSlideChangeStart', s);
                    _scope.$emit("$ionicSlides.slideChangeStart", {
                      slider: s,
                      activeIndex: s.getSlideDataIndex(s.activeIndex),
                      previousIndex: s.getSlideDataIndex(s.previousIndex)
                    });
                    if (s.activeIndex > s.previousIndex) {
                        s.emit('onSlideNextStart', s);
                    }
                    else {
                        s.emit('onSlidePrevStart', s);
                    }
                }

            }
        };
        s.onTransitionEnd = function (runCallbacks) {
            s.animating = false;
            s.setWrapperTransition(0);
            if (typeof runCallbacks === 'undefined') runCallbacks = true;
            if (s.lazy) s.lazy.onTransitionEnd();
            if (runCallbacks) {
                s.emit('onTransitionEnd', s);
                if (s.activeIndex !== s.previousIndex) {
                    s.emit('onSlideChangeEnd', s);
                    _scope.$emit("$ionicSlides.slideChangeEnd", {
                      slider: s,
                      activeIndex: s.getSlideDataIndex(s.activeIndex),
                      previousIndex: s.getSlideDataIndex(s.previousIndex)
                    });
                    if (s.activeIndex > s.previousIndex) {
                        s.emit('onSlideNextEnd', s);
                    }
                    else {
                        s.emit('onSlidePrevEnd', s);
                    }
                }
            }
            if (s.params.hashnav && s.hashnav) {
                s.hashnav.setHash();
            }

        };
        s.slideNext = function (runCallbacks, speed, internal) {
            if (s.params.loop) {
                if (s.animating) return false;
                s.fixLoop();
                var clientLeft = s.container[0].clientLeft;
                return s.slideTo(s.activeIndex + s.params.slidesPerGroup, speed, runCallbacks, internal);
            }
            else return s.slideTo(s.activeIndex + s.params.slidesPerGroup, speed, runCallbacks, internal);
        };
        s._slideNext = function (speed) {
            return s.slideNext(true, speed, true);
        };
        s.slidePrev = function (runCallbacks, speed, internal) {
            if (s.params.loop) {
                if (s.animating) return false;
                s.fixLoop();
                var clientLeft = s.container[0].clientLeft;
                return s.slideTo(s.activeIndex - 1, speed, runCallbacks, internal);
            }
            else return s.slideTo(s.activeIndex - 1, speed, runCallbacks, internal);
        };
        s._slidePrev = function (speed) {
            return s.slidePrev(true, speed, true);
        };
        s.slideReset = function (runCallbacks, speed, internal) {
            return s.slideTo(s.activeIndex, speed, runCallbacks);
        };

        /*=========================
          Translate/transition helpers
          ===========================*/
        s.setWrapperTransition = function (duration, byController) {
            s.wrapper.transition(duration);
            if (s.params.effect !== 'slide' && s.effects[s.params.effect]) {
                s.effects[s.params.effect].setTransition(duration);
            }
            if (s.params.parallax && s.parallax) {
                s.parallax.setTransition(duration);
            }
            if (s.params.scrollbar && s.scrollbar) {
                s.scrollbar.setTransition(duration);
            }
            if (s.params.control && s.controller) {
                s.controller.setTransition(duration, byController);
            }
            s.emit('onSetTransition', s, duration);
        };
        s.setWrapperTranslate = function (translate, updateActiveIndex, byController) {
            var x = 0, y = 0, z = 0;
            if (isH()) {
                x = s.rtl ? -translate : translate;
            }
            else {
                y = translate;
            }

            if (s.params.roundLengths) {
                x = round(x);
                y = round(y);
            }

            if (!s.params.virtualTranslate) {
                if (s.support.transforms3d) s.wrapper.transform('translate3d(' + x + 'px, ' + y + 'px, ' + z + 'px)');
                else s.wrapper.transform('translate(' + x + 'px, ' + y + 'px)');
            }

            s.translate = isH() ? x : y;

            // Check if we need to update progress
            var progress;
            var translatesDiff = s.maxTranslate() - s.minTranslate();
            if (translatesDiff === 0) {
                progress = 0;
            }
            else {
                progress = (translate - s.minTranslate()) / (translatesDiff);
            }
            if (progress !== s.progress) {
                s.updateProgress(translate);
            }

            if (updateActiveIndex) s.updateActiveIndex();
            if (s.params.effect !== 'slide' && s.effects[s.params.effect]) {
                s.effects[s.params.effect].setTranslate(s.translate);
            }
            if (s.params.parallax && s.parallax) {
                s.parallax.setTranslate(s.translate);
            }
            if (s.params.scrollbar && s.scrollbar) {
                s.scrollbar.setTranslate(s.translate);
            }
            if (s.params.control && s.controller) {
                s.controller.setTranslate(s.translate, byController);
            }
            s.emit('onSetTranslate', s, s.translate);
        };

        s.getTranslate = function (el, axis) {
            var matrix, curTransform, curStyle, transformMatrix;

            // automatic axis detection
            if (typeof axis === 'undefined') {
                axis = 'x';
            }

            if (s.params.virtualTranslate) {
                return s.rtl ? -s.translate : s.translate;
            }

            curStyle = window.getComputedStyle(el, null);
            if (window.WebKitCSSMatrix) {
                curTransform = curStyle.transform || curStyle.webkitTransform;
                if (curTransform.split(',').length > 6) {
                    curTransform = curTransform.split(', ').map(function(a){
                        return a.replace(',','.');
                    }).join(', ');
                }
                // Some old versions of Webkit choke when 'none' is passed; pass
                // empty string instead in this case
                transformMatrix = new window.WebKitCSSMatrix(curTransform === 'none' ? '' : curTransform);
            }
            else {
                transformMatrix = curStyle.MozTransform || curStyle.OTransform || curStyle.MsTransform || curStyle.msTransform  || curStyle.transform || curStyle.getPropertyValue('transform').replace('translate(', 'matrix(1, 0, 0, 1,');
                matrix = transformMatrix.toString().split(',');
            }

            if (axis === 'x') {
                //Latest Chrome and webkits Fix
                if (window.WebKitCSSMatrix)
                    curTransform = transformMatrix.m41;
                //Crazy IE10 Matrix
                else if (matrix.length === 16)
                    curTransform = parseFloat(matrix[12]);
                //Normal Browsers
                else
                    curTransform = parseFloat(matrix[4]);
            }
            if (axis === 'y') {
                //Latest Chrome and webkits Fix
                if (window.WebKitCSSMatrix)
                    curTransform = transformMatrix.m42;
                //Crazy IE10 Matrix
                else if (matrix.length === 16)
                    curTransform = parseFloat(matrix[13]);
                //Normal Browsers
                else
                    curTransform = parseFloat(matrix[5]);
            }
            if (s.rtl && curTransform) curTransform = -curTransform;
            return curTransform || 0;
        };
        s.getWrapperTranslate = function (axis) {
            if (typeof axis === 'undefined') {
                axis = isH() ? 'x' : 'y';
            }
            return s.getTranslate(s.wrapper[0], axis);
        };

        /*=========================
          Observer
          ===========================*/
        s.observers = [];
        function initObserver(target, options) {
            options = options || {};
            // create an observer instance
            var ObserverFunc = window.MutationObserver || window.WebkitMutationObserver;
            var observer = new ObserverFunc(function (mutations) {
                mutations.forEach(function (mutation) {
                    s.onResize(true);
                    s.emit('onObserverUpdate', s, mutation);
                });
            });

            observer.observe(target, {
                attributes: typeof options.attributes === 'undefined' ? true : options.attributes,
                childList: typeof options.childList === 'undefined' ? true : options.childList,
                characterData: typeof options.characterData === 'undefined' ? true : options.characterData
            });

            s.observers.push(observer);
        }
        s.initObservers = function () {
            if (s.params.observeParents) {
                var containerParents = s.container.parents();
                for (var i = 0; i < containerParents.length; i++) {
                    initObserver(containerParents[i]);
                }
            }

            // Observe container
            initObserver(s.container[0], {childList: false});

            // Observe wrapper
            initObserver(s.wrapper[0], {attributes: false});
        };
        s.disconnectObservers = function () {
            for (var i = 0; i < s.observers.length; i++) {
                s.observers[i].disconnect();
            }
            s.observers = [];
        };

        s.updateLoop = function(){
          var currentSlide = s.slides.eq(s.activeIndex);
          if ( angular.element(currentSlide).hasClass(s.params.slideDuplicateClass) ){
            // we're on a duplicate, so slide to the non-duplicate
            var swiperSlideIndex = angular.element(currentSlide).attr("data-swiper-slide-index");
            var slides = s.wrapper.children('.' + s.params.slideClass);
            for ( var i = 0; i < slides.length; i++ ){
              if ( !angular.element(slides[i]).hasClass(s.params.slideDuplicateClass) && angular.element(slides[i]).attr("data-swiper-slide-index") === swiperSlideIndex ){
                s.slideTo(i, 0, false, true);
                break;
              }
            }
            // if we needed to switch slides, we did that.  So, now call the createLoop function internally
            setTimeout(function(){
              s.createLoop();
            }, 50);
          }
        }

        s.getSlideDataIndex = function(slideIndex){
          // this is an Ionic custom function
          // Swiper loops utilize duplicate DOM elements for slides when in a loop
          // which means that we cannot rely on the actual slide index for our events
          // because index 0 does not necessarily point to index 0
          // and index n+1 does not necessarily point to the expected piece of data
          // therefore, rather than using the actual slide index we should
          // use the data index that swiper includes as an attribute on the dom elements
          // because this is what will be meaningful to the consumer of our events
          var slide = s.slides.eq(slideIndex);
          var attributeIndex = angular.element(slide).attr("data-swiper-slide-index");
          return parseInt(attributeIndex);
        }

        /*=========================
          Loop
          ===========================*/
        // Create looped slides
        s.createLoop = function () {
          //console.log("Slider create loop method");
            //var toRemove = s.wrapper.children('.' + s.params.slideClass + '.' + s.params.slideDuplicateClass);
            //angular.element(toRemove).remove();
            s.wrapper.children('.' + s.params.slideClass + '.' + s.params.slideDuplicateClass).remove();

            var slides = s.wrapper.children('.' + s.params.slideClass);

            if(s.params.slidesPerView === 'auto' && !s.params.loopedSlides) s.params.loopedSlides = slides.length;

            s.loopedSlides = parseInt(s.params.loopedSlides || s.params.slidesPerView, 10);
            s.loopedSlides = s.loopedSlides + s.params.loopAdditionalSlides;
            if (s.loopedSlides > slides.length) {
                s.loopedSlides = slides.length;
            }

            var prependSlides = [], appendSlides = [], i, scope, newNode;
            slides.each(function (index, el) {
                var slide = $(this);
                if (index < s.loopedSlides) appendSlides.push(el);
                if (index < slides.length && index >= slides.length - s.loopedSlides) prependSlides.push(el);
                slide.attr('data-swiper-slide-index', index);
            });
            for (i = 0; i < appendSlides.length; i++) {

              newNode = angular.element(appendSlides[i]).clone().addClass(s.params.slideDuplicateClass);
              newNode.removeAttr('ng-transclude');
              newNode.removeAttr('ng-repeat');
              scope = angular.element(appendSlides[i]).scope();
              newNode = $compile(newNode)(scope);
              angular.element(s.wrapper).append(newNode);
              //s.wrapper.append($(appendSlides[i].cloneNode(true)).addClass(s.params.slideDuplicateClass));
            }
            for (i = prependSlides.length - 1; i >= 0; i--) {
              //s.wrapper.prepend($(prependSlides[i].cloneNode(true)).addClass(s.params.slideDuplicateClass));

              newNode = angular.element(prependSlides[i]).clone().addClass(s.params.slideDuplicateClass);
              newNode.removeAttr('ng-transclude');
              newNode.removeAttr('ng-repeat');

              scope = angular.element(prependSlides[i]).scope();
              newNode = $compile(newNode)(scope);
              angular.element(s.wrapper).prepend(newNode);
            }
        };
        s.destroyLoop = function () {
            s.wrapper.children('.' + s.params.slideClass + '.' + s.params.slideDuplicateClass).remove();
            s.slides.removeAttr('data-swiper-slide-index');
        };
        s.fixLoop = function () {
            var newIndex;
            //Fix For Negative Oversliding
            if (s.activeIndex < s.loopedSlides) {
                newIndex = s.slides.length - s.loopedSlides * 3 + s.activeIndex;
                newIndex = newIndex + s.loopedSlides;
                s.slideTo(newIndex, 0, false, true);
            }
            //Fix For Positive Oversliding
            else if ((s.params.slidesPerView === 'auto' && s.activeIndex >= s.loopedSlides * 2) || (s.activeIndex > s.slides.length - s.params.slidesPerView * 2)) {
                newIndex = -s.slides.length + s.activeIndex + s.loopedSlides;
                newIndex = newIndex + s.loopedSlides;
                s.slideTo(newIndex, 0, false, true);
            }
        };
        /*=========================
          Append/Prepend/Remove Slides
          ===========================*/
        s.appendSlide = function (slides) {
            if (s.params.loop) {
                s.destroyLoop();
            }
            if (typeof slides === 'object' && slides.length) {
                for (var i = 0; i < slides.length; i++) {
                    if (slides[i]) s.wrapper.append(slides[i]);
                }
            }
            else {
                s.wrapper.append(slides);
            }
            if (s.params.loop) {
                s.createLoop();
            }
            if (!(s.params.observer && s.support.observer)) {
                s.update(true);
            }
        };
        s.prependSlide = function (slides) {
            if (s.params.loop) {
                s.destroyLoop();
            }
            var newActiveIndex = s.activeIndex + 1;
            if (typeof slides === 'object' && slides.length) {
                for (var i = 0; i < slides.length; i++) {
                    if (slides[i]) s.wrapper.prepend(slides[i]);
                }
                newActiveIndex = s.activeIndex + slides.length;
            }
            else {
                s.wrapper.prepend(slides);
            }
            if (s.params.loop) {
                s.createLoop();
            }
            if (!(s.params.observer && s.support.observer)) {
                s.update(true);
            }
            s.slideTo(newActiveIndex, 0, false);
        };
        s.removeSlide = function (slidesIndexes) {
            if (s.params.loop) {
                s.destroyLoop();
                s.slides = s.wrapper.children('.' + s.params.slideClass);
            }
            var newActiveIndex = s.activeIndex,
                indexToRemove;
            if (typeof slidesIndexes === 'object' && slidesIndexes.length) {
                for (var i = 0; i < slidesIndexes.length; i++) {
                    indexToRemove = slidesIndexes[i];
                    if (s.slides[indexToRemove]) s.slides.eq(indexToRemove).remove();
                    if (indexToRemove < newActiveIndex) newActiveIndex--;
                }
                newActiveIndex = Math.max(newActiveIndex, 0);
            }
            else {
                indexToRemove = slidesIndexes;
                if (s.slides[indexToRemove]) s.slides.eq(indexToRemove).remove();
                if (indexToRemove < newActiveIndex) newActiveIndex--;
                newActiveIndex = Math.max(newActiveIndex, 0);
            }

            if (s.params.loop) {
                s.createLoop();
            }

            if (!(s.params.observer && s.support.observer)) {
                s.update(true);
            }
            if (s.params.loop) {
                s.slideTo(newActiveIndex + s.loopedSlides, 0, false);
            }
            else {
                s.slideTo(newActiveIndex, 0, false);
            }

        };
        s.removeAllSlides = function () {
            var slidesIndexes = [];
            for (var i = 0; i < s.slides.length; i++) {
                slidesIndexes.push(i);
            }
            s.removeSlide(slidesIndexes);
        };


        /*=========================
          Effects
          ===========================*/
        s.effects = {
            fade: {
                setTranslate: function () {
                    for (var i = 0; i < s.slides.length; i++) {
                        var slide = s.slides.eq(i);
                        var offset = slide[0].swiperSlideOffset;
                        var tx = -offset;
                        if (!s.params.virtualTranslate) tx = tx - s.translate;
                        var ty = 0;
                        if (!isH()) {
                            ty = tx;
                            tx = 0;
                        }
                        var slideOpacity = s.params.fade.crossFade ?
                                Math.max(1 - Math.abs(slide[0].progress), 0) :
                                1 + Math.min(Math.max(slide[0].progress, -1), 0);
                        slide
                            .css({
                                opacity: slideOpacity
                            })
                            .transform('translate3d(' + tx + 'px, ' + ty + 'px, 0px)');

                    }

                },
                setTransition: function (duration) {
                    s.slides.transition(duration);
                    if (s.params.virtualTranslate && duration !== 0) {
                        var eventTriggered = false;
                        s.slides.transitionEnd(function () {
                            if (eventTriggered) return;
                            if (!s) return;
                            eventTriggered = true;
                            s.animating = false;
                            var triggerEvents = ['webkitTransitionEnd', 'transitionend', 'oTransitionEnd', 'MSTransitionEnd', 'msTransitionEnd'];
                            for (var i = 0; i < triggerEvents.length; i++) {
                                s.wrapper.trigger(triggerEvents[i]);
                            }
                        });
                    }
                }
            },
            cube: {
                setTranslate: function () {
                    var wrapperRotate = 0, cubeShadow;
                    if (s.params.cube.shadow) {
                        if (isH()) {
                            cubeShadow = s.wrapper.find('.swiper-cube-shadow');
                            if (cubeShadow.length === 0) {
                                cubeShadow = $('<div class="swiper-cube-shadow"></div>');
                                s.wrapper.append(cubeShadow);
                            }
                            cubeShadow.css({height: s.width + 'px'});
                        }
                        else {
                            cubeShadow = s.container.find('.swiper-cube-shadow');
                            if (cubeShadow.length === 0) {
                                cubeShadow = $('<div class="swiper-cube-shadow"></div>');
                                s.container.append(cubeShadow);
                            }
                        }
                    }
                    for (var i = 0; i < s.slides.length; i++) {
                        var slide = s.slides.eq(i);
                        var slideAngle = i * 90;
                        var round = Math.floor(slideAngle / 360);
                        if (s.rtl) {
                            slideAngle = -slideAngle;
                            round = Math.floor(-slideAngle / 360);
                        }
                        var progress = Math.max(Math.min(slide[0].progress, 1), -1);
                        var tx = 0, ty = 0, tz = 0;
                        if (i % 4 === 0) {
                            tx = - round * 4 * s.size;
                            tz = 0;
                        }
                        else if ((i - 1) % 4 === 0) {
                            tx = 0;
                            tz = - round * 4 * s.size;
                        }
                        else if ((i - 2) % 4 === 0) {
                            tx = s.size + round * 4 * s.size;
                            tz = s.size;
                        }
                        else if ((i - 3) % 4 === 0) {
                            tx = - s.size;
                            tz = 3 * s.size + s.size * 4 * round;
                        }
                        if (s.rtl) {
                            tx = -tx;
                        }

                        if (!isH()) {
                            ty = tx;
                            tx = 0;
                        }

                        var transform = 'rotateX(' + (isH() ? 0 : -slideAngle) + 'deg) rotateY(' + (isH() ? slideAngle : 0) + 'deg) translate3d(' + tx + 'px, ' + ty + 'px, ' + tz + 'px)';
                        if (progress <= 1 && progress > -1) {
                            wrapperRotate = i * 90 + progress * 90;
                            if (s.rtl) wrapperRotate = -i * 90 - progress * 90;
                        }
                        slide.transform(transform);
                        if (s.params.cube.slideShadows) {
                            //Set shadows
                            var shadowBefore = isH() ? slide.find('.swiper-slide-shadow-left') : slide.find('.swiper-slide-shadow-top');
                            var shadowAfter = isH() ? slide.find('.swiper-slide-shadow-right') : slide.find('.swiper-slide-shadow-bottom');
                            if (shadowBefore.length === 0) {
                                shadowBefore = $('<div class="swiper-slide-shadow-' + (isH() ? 'left' : 'top') + '"></div>');
                                slide.append(shadowBefore);
                            }
                            if (shadowAfter.length === 0) {
                                shadowAfter = $('<div class="swiper-slide-shadow-' + (isH() ? 'right' : 'bottom') + '"></div>');
                                slide.append(shadowAfter);
                            }
                            var shadowOpacity = slide[0].progress;
                            if (shadowBefore.length) shadowBefore[0].style.opacity = -slide[0].progress;
                            if (shadowAfter.length) shadowAfter[0].style.opacity = slide[0].progress;
                        }
                    }
                    s.wrapper.css({
                        '-webkit-transform-origin': '50% 50% -' + (s.size / 2) + 'px',
                        '-moz-transform-origin': '50% 50% -' + (s.size / 2) + 'px',
                        '-ms-transform-origin': '50% 50% -' + (s.size / 2) + 'px',
                        'transform-origin': '50% 50% -' + (s.size / 2) + 'px'
                    });

                    if (s.params.cube.shadow) {
                        if (isH()) {
                            cubeShadow.transform('translate3d(0px, ' + (s.width / 2 + s.params.cube.shadowOffset) + 'px, ' + (-s.width / 2) + 'px) rotateX(90deg) rotateZ(0deg) scale(' + (s.params.cube.shadowScale) + ')');
                        }
                        else {
                            var shadowAngle = Math.abs(wrapperRotate) - Math.floor(Math.abs(wrapperRotate) / 90) * 90;
                            var multiplier = 1.5 - (Math.sin(shadowAngle * 2 * Math.PI / 360) / 2 + Math.cos(shadowAngle * 2 * Math.PI / 360) / 2);
                            var scale1 = s.params.cube.shadowScale,
                                scale2 = s.params.cube.shadowScale / multiplier,
                                offset = s.params.cube.shadowOffset;
                            cubeShadow.transform('scale3d(' + scale1 + ', 1, ' + scale2 + ') translate3d(0px, ' + (s.height / 2 + offset) + 'px, ' + (-s.height / 2 / scale2) + 'px) rotateX(-90deg)');
                        }
                    }
                    var zFactor = (s.isSafari || s.isUiWebView) ? (-s.size / 2) : 0;
                    s.wrapper.transform('translate3d(0px,0,' + zFactor + 'px) rotateX(' + (isH() ? 0 : wrapperRotate) + 'deg) rotateY(' + (isH() ? -wrapperRotate : 0) + 'deg)');
                },
                setTransition: function (duration) {
                    s.slides.transition(duration).find('.swiper-slide-shadow-top, .swiper-slide-shadow-right, .swiper-slide-shadow-bottom, .swiper-slide-shadow-left').transition(duration);
                    if (s.params.cube.shadow && !isH()) {
                        s.container.find('.swiper-cube-shadow').transition(duration);
                    }
                }
            },
            coverflow: {
                setTranslate: function () {
                    var transform = s.translate;
                    var center = isH() ? -transform + s.width / 2 : -transform + s.height / 2;
                    var rotate = isH() ? s.params.coverflow.rotate: -s.params.coverflow.rotate;
                    var translate = s.params.coverflow.depth;
                    //Each slide offset from center
                    for (var i = 0, length = s.slides.length; i < length; i++) {
                        var slide = s.slides.eq(i);
                        var slideSize = s.slidesSizesGrid[i];
                        var slideOffset = slide[0].swiperSlideOffset;
                        var offsetMultiplier = (center - slideOffset - slideSize / 2) / slideSize * s.params.coverflow.modifier;

                        var rotateY = isH() ? rotate * offsetMultiplier : 0;
                        var rotateX = isH() ? 0 : rotate * offsetMultiplier;
                        // var rotateZ = 0
                        var translateZ = -translate * Math.abs(offsetMultiplier);

                        var translateY = isH() ? 0 : s.params.coverflow.stretch * (offsetMultiplier);
                        var translateX = isH() ? s.params.coverflow.stretch * (offsetMultiplier) : 0;

                        //Fix for ultra small values
                        if (Math.abs(translateX) < 0.001) translateX = 0;
                        if (Math.abs(translateY) < 0.001) translateY = 0;
                        if (Math.abs(translateZ) < 0.001) translateZ = 0;
                        if (Math.abs(rotateY) < 0.001) rotateY = 0;
                        if (Math.abs(rotateX) < 0.001) rotateX = 0;

                        var slideTransform = 'translate3d(' + translateX + 'px,' + translateY + 'px,' + translateZ + 'px)  rotateX(' + rotateX + 'deg) rotateY(' + rotateY + 'deg)';

                        slide.transform(slideTransform);
                        slide[0].style.zIndex = -Math.abs(Math.round(offsetMultiplier)) + 1;
                        if (s.params.coverflow.slideShadows) {
                            //Set shadows
                            var shadowBefore = isH() ? slide.find('.swiper-slide-shadow-left') : slide.find('.swiper-slide-shadow-top');
                            var shadowAfter = isH() ? slide.find('.swiper-slide-shadow-right') : slide.find('.swiper-slide-shadow-bottom');
                            if (shadowBefore.length === 0) {
                                shadowBefore = $('<div class="swiper-slide-shadow-' + (isH() ? 'left' : 'top') + '"></div>');
                                slide.append(shadowBefore);
                            }
                            if (shadowAfter.length === 0) {
                                shadowAfter = $('<div class="swiper-slide-shadow-' + (isH() ? 'right' : 'bottom') + '"></div>');
                                slide.append(shadowAfter);
                            }
                            if (shadowBefore.length) shadowBefore[0].style.opacity = offsetMultiplier > 0 ? offsetMultiplier : 0;
                            if (shadowAfter.length) shadowAfter[0].style.opacity = (-offsetMultiplier) > 0 ? -offsetMultiplier : 0;
                        }
                    }

                    //Set correct perspective for IE10
                    if (s.browser.ie) {
                        var ws = s.wrapper[0].style;
                        ws.perspectiveOrigin = center + 'px 50%';
                    }
                },
                setTransition: function (duration) {
                    s.slides.transition(duration).find('.swiper-slide-shadow-top, .swiper-slide-shadow-right, .swiper-slide-shadow-bottom, .swiper-slide-shadow-left').transition(duration);
                }
            }
        };

        /*=========================
          Images Lazy Loading
          ===========================*/
        s.lazy = {
            initialImageLoaded: false,
            loadImageInSlide: function (index, loadInDuplicate) {
                if (typeof index === 'undefined') return;
                if (typeof loadInDuplicate === 'undefined') loadInDuplicate = true;
                if (s.slides.length === 0) return;

                var slide = s.slides.eq(index);
                var img = slide.find('.swiper-lazy:not(.swiper-lazy-loaded):not(.swiper-lazy-loading)');
                if (slide.hasClass('swiper-lazy') && !slide.hasClass('swiper-lazy-loaded') && !slide.hasClass('swiper-lazy-loading')) {
                    img = img.add(slide[0]);
                }
                if (img.length === 0) return;

                img.each(function () {
                    var _img = $(this);
                    _img.addClass('swiper-lazy-loading');
                    var background = _img.attr('data-background');
                    var src = _img.attr('data-src'),
                        srcset = _img.attr('data-srcset');
                    s.loadImage(_img[0], (src || background), srcset, false, function () {
                        if (background) {
                            _img.css('background-image', 'url(' + background + ')');
                            _img.removeAttr('data-background');
                        }
                        else {
                            if (srcset) {
                                _img.attr('srcset', srcset);
                                _img.removeAttr('data-srcset');
                            }
                            if (src) {
                                _img.attr('src', src);
                                _img.removeAttr('data-src');
                            }

                        }

                        _img.addClass('swiper-lazy-loaded').removeClass('swiper-lazy-loading');
                        slide.find('.swiper-lazy-preloader, .preloader').remove();
                        if (s.params.loop && loadInDuplicate) {
                            var slideOriginalIndex = slide.attr('data-swiper-slide-index');
                            if (slide.hasClass(s.params.slideDuplicateClass)) {
                                var originalSlide = s.wrapper.children('[data-swiper-slide-index="' + slideOriginalIndex + '"]:not(.' + s.params.slideDuplicateClass + ')');
                                s.lazy.loadImageInSlide(originalSlide.index(), false);
                            }
                            else {
                                var duplicatedSlide = s.wrapper.children('.' + s.params.slideDuplicateClass + '[data-swiper-slide-index="' + slideOriginalIndex + '"]');
                                s.lazy.loadImageInSlide(duplicatedSlide.index(), false);
                            }
                        }
                        s.emit('onLazyImageReady', s, slide[0], _img[0]);
                    });

                    s.emit('onLazyImageLoad', s, slide[0], _img[0]);
                });

            },
            load: function () {
                var i;
                if (s.params.watchSlidesVisibility) {
                    s.wrapper.children('.' + s.params.slideVisibleClass).each(function () {
                        s.lazy.loadImageInSlide($(this).index());
                    });
                }
                else {
                    if (s.params.slidesPerView > 1) {
                        for (i = s.activeIndex; i < s.activeIndex + s.params.slidesPerView ; i++) {
                            if (s.slides[i]) s.lazy.loadImageInSlide(i);
                        }
                    }
                    else {
                        s.lazy.loadImageInSlide(s.activeIndex);
                    }
                }
                if (s.params.lazyLoadingInPrevNext) {
                    if (s.params.slidesPerView > 1) {
                        // Next Slides
                        for (i = s.activeIndex + s.params.slidesPerView; i < s.activeIndex + s.params.slidesPerView + s.params.slidesPerView; i++) {
                            if (s.slides[i]) s.lazy.loadImageInSlide(i);
                        }
                        // Prev Slides
                        for (i = s.activeIndex - s.params.slidesPerView; i < s.activeIndex ; i++) {
                            if (s.slides[i]) s.lazy.loadImageInSlide(i);
                        }
                    }
                    else {
                        var nextSlide = s.wrapper.children('.' + s.params.slideNextClass);
                        if (nextSlide.length > 0) s.lazy.loadImageInSlide(nextSlide.index());

                        var prevSlide = s.wrapper.children('.' + s.params.slidePrevClass);
                        if (prevSlide.length > 0) s.lazy.loadImageInSlide(prevSlide.index());
                    }
                }
            },
            onTransitionStart: function () {
                if (s.params.lazyLoading) {
                    if (s.params.lazyLoadingOnTransitionStart || (!s.params.lazyLoadingOnTransitionStart && !s.lazy.initialImageLoaded)) {
                        s.lazy.load();
                    }
                }
            },
            onTransitionEnd: function () {
                if (s.params.lazyLoading && !s.params.lazyLoadingOnTransitionStart) {
                    s.lazy.load();
                }
            }
        };


        /*=========================
          Scrollbar
          ===========================*/
        s.scrollbar = {
            isTouched: false,
            setDragPosition: function (e) {
                var sb = s.scrollbar;
                var x = 0, y = 0;
                var translate;
                var pointerPosition = isH() ?
                    ((e.type === 'touchstart' || e.type === 'touchmove') ? e.targetTouches[0].pageX : e.pageX || e.clientX) :
                    ((e.type === 'touchstart' || e.type === 'touchmove') ? e.targetTouches[0].pageY : e.pageY || e.clientY) ;
                var position = (pointerPosition) - sb.track.offset()[isH() ? 'left' : 'top'] - sb.dragSize / 2;
                var positionMin = -s.minTranslate() * sb.moveDivider;
                var positionMax = -s.maxTranslate() * sb.moveDivider;
                if (position < positionMin) {
                    position = positionMin;
                }
                else if (position > positionMax) {
                    position = positionMax;
                }
                position = -position / sb.moveDivider;
                s.updateProgress(position);
                s.setWrapperTranslate(position, true);
            },
            dragStart: function (e) {
                var sb = s.scrollbar;
                sb.isTouched = true;
                e.preventDefault();
                e.stopPropagation();

                sb.setDragPosition(e);
                clearTimeout(sb.dragTimeout);

                sb.track.transition(0);
                if (s.params.scrollbarHide) {
                    sb.track.css('opacity', 1);
                }
                s.wrapper.transition(100);
                sb.drag.transition(100);
                s.emit('onScrollbarDragStart', s);
            },
            dragMove: function (e) {
                var sb = s.scrollbar;
                if (!sb.isTouched) return;
                if (e.preventDefault) e.preventDefault();
                else e.returnValue = false;
                sb.setDragPosition(e);
                s.wrapper.transition(0);
                sb.track.transition(0);
                sb.drag.transition(0);
                s.emit('onScrollbarDragMove', s);
            },
            dragEnd: function (e) {
                var sb = s.scrollbar;
                if (!sb.isTouched) return;
                sb.isTouched = false;
                if (s.params.scrollbarHide) {
                    clearTimeout(sb.dragTimeout);
                    sb.dragTimeout = setTimeout(function () {
                        sb.track.css('opacity', 0);
                        sb.track.transition(400);
                    }, 1000);

                }
                s.emit('onScrollbarDragEnd', s);
                if (s.params.scrollbarSnapOnRelease) {
                    s.slideReset();
                }
            },
            enableDraggable: function () {
                var sb = s.scrollbar;
                var target = s.support.touch ? sb.track : document;
                $(sb.track).on(s.touchEvents.start, sb.dragStart);
                $(target).on(s.touchEvents.move, sb.dragMove);
                $(target).on(s.touchEvents.end, sb.dragEnd);
            },
            disableDraggable: function () {
                var sb = s.scrollbar;
                var target = s.support.touch ? sb.track : document;
                $(sb.track).off(s.touchEvents.start, sb.dragStart);
                $(target).off(s.touchEvents.move, sb.dragMove);
                $(target).off(s.touchEvents.end, sb.dragEnd);
            },
            set: function () {
                if (!s.params.scrollbar) return;
                var sb = s.scrollbar;
                sb.track = $(s.params.scrollbar);
                sb.drag = sb.track.find('.swiper-scrollbar-drag');
                if (sb.drag.length === 0) {
                    sb.drag = $('<div class="swiper-scrollbar-drag"></div>');
                    sb.track.append(sb.drag);
                }
                sb.drag[0].style.width = '';
                sb.drag[0].style.height = '';
                sb.trackSize = isH() ? sb.track[0].offsetWidth : sb.track[0].offsetHeight;

                sb.divider = s.size / s.virtualSize;
                sb.moveDivider = sb.divider * (sb.trackSize / s.size);
                sb.dragSize = sb.trackSize * sb.divider;

                if (isH()) {
                    sb.drag[0].style.width = sb.dragSize + 'px';
                }
                else {
                    sb.drag[0].style.height = sb.dragSize + 'px';
                }

                if (sb.divider >= 1) {
                    sb.track[0].style.display = 'none';
                }
                else {
                    sb.track[0].style.display = '';
                }
                if (s.params.scrollbarHide) {
                    sb.track[0].style.opacity = 0;
                }
            },
            setTranslate: function () {
                if (!s.params.scrollbar) return;
                var diff;
                var sb = s.scrollbar;
                var translate = s.translate || 0;
                var newPos;

                var newSize = sb.dragSize;
                newPos = (sb.trackSize - sb.dragSize) * s.progress;
                if (s.rtl && isH()) {
                    newPos = -newPos;
                    if (newPos > 0) {
                        newSize = sb.dragSize - newPos;
                        newPos = 0;
                    }
                    else if (-newPos + sb.dragSize > sb.trackSize) {
                        newSize = sb.trackSize + newPos;
                    }
                }
                else {
                    if (newPos < 0) {
                        newSize = sb.dragSize + newPos;
                        newPos = 0;
                    }
                    else if (newPos + sb.dragSize > sb.trackSize) {
                        newSize = sb.trackSize - newPos;
                    }
                }
                if (isH()) {
                    if (s.support.transforms3d) {
                        sb.drag.transform('translate3d(' + (newPos) + 'px, 0, 0)');
                    }
                    else {
                        sb.drag.transform('translateX(' + (newPos) + 'px)');
                    }
                    sb.drag[0].style.width = newSize + 'px';
                }
                else {
                    if (s.support.transforms3d) {
                        sb.drag.transform('translate3d(0px, ' + (newPos) + 'px, 0)');
                    }
                    else {
                        sb.drag.transform('translateY(' + (newPos) + 'px)');
                    }
                    sb.drag[0].style.height = newSize + 'px';
                }
                if (s.params.scrollbarHide) {
                    clearTimeout(sb.timeout);
                    sb.track[0].style.opacity = 1;
                    sb.timeout = setTimeout(function () {
                        sb.track[0].style.opacity = 0;
                        sb.track.transition(400);
                    }, 1000);
                }
            },
            setTransition: function (duration) {
                if (!s.params.scrollbar) return;
                s.scrollbar.drag.transition(duration);
            }
        };

        /*=========================
          Controller
          ===========================*/
        s.controller = {
            LinearSpline: function (x, y) {
                this.x = x;
                this.y = y;
                this.lastIndex = x.length - 1;
                // Given an x value (x2), return the expected y2 value:
                // (x1,y1) is the known point before given value,
                // (x3,y3) is the known point after given value.
                var i1, i3;
                var l = this.x.length;

                this.interpolate = function (x2) {
                    if (!x2) return 0;

                    // Get the indexes of x1 and x3 (the array indexes before and after given x2):
                    i3 = binarySearch(this.x, x2);
                    i1 = i3 - 1;

                    // We have our indexes i1 & i3, so we can calculate already:
                    // y2 := ((x2−x1) × (y3−y1)) ÷ (x3−x1) + y1
                    return ((x2 - this.x[i1]) * (this.y[i3] - this.y[i1])) / (this.x[i3] - this.x[i1]) + this.y[i1];
                };

                var binarySearch = (function() {
                    var maxIndex, minIndex, guess;
                    return function(array, val) {
                        minIndex = -1;
                        maxIndex = array.length;
                        while (maxIndex - minIndex > 1)
                            if (array[guess = maxIndex + minIndex >> 1] <= val) {
                                minIndex = guess;
                            } else {
                                maxIndex = guess;
                            }
                        return maxIndex;
                    };
                })();
            },
            //xxx: for now i will just save one spline function to to
            getInterpolateFunction: function(c){
                if(!s.controller.spline) s.controller.spline = s.params.loop ?
                    new s.controller.LinearSpline(s.slidesGrid, c.slidesGrid) :
                    new s.controller.LinearSpline(s.snapGrid, c.snapGrid);
            },
            setTranslate: function (translate, byController) {
               var controlled = s.params.control;
               var multiplier, controlledTranslate;
               function setControlledTranslate(c) {
                    // this will create an Interpolate function based on the snapGrids
                    // x is the Grid of the scrolled scroller and y will be the controlled scroller
                    // it makes sense to create this only once and recall it for the interpolation
                    // the function does a lot of value caching for performance
                    translate = c.rtl && c.params.direction === 'horizontal' ? -s.translate : s.translate;
                    if (s.params.controlBy === 'slide') {
                        s.controller.getInterpolateFunction(c);
                        // i am not sure why the values have to be multiplicated this way, tried to invert the snapGrid
                        // but it did not work out
                        controlledTranslate = -s.controller.spline.interpolate(-translate);
                    }

                    if(!controlledTranslate || s.params.controlBy === 'container'){
                        multiplier = (c.maxTranslate() - c.minTranslate()) / (s.maxTranslate() - s.minTranslate());
                        controlledTranslate = (translate - s.minTranslate()) * multiplier + c.minTranslate();
                    }

                    if (s.params.controlInverse) {
                        controlledTranslate = c.maxTranslate() - controlledTranslate;
                    }
                    c.updateProgress(controlledTranslate);
                    c.setWrapperTranslate(controlledTranslate, false, s);
                    c.updateActiveIndex();
               }
               if (s.isArray(controlled)) {
                   for (var i = 0; i < controlled.length; i++) {
                       if (controlled[i] !== byController && controlled[i] instanceof Swiper) {
                           setControlledTranslate(controlled[i]);
                       }
                   }
               }
               else if (controlled instanceof Swiper && byController !== controlled) {

                   setControlledTranslate(controlled);
               }
            },
            setTransition: function (duration, byController) {
                var controlled = s.params.control;
                var i;
                function setControlledTransition(c) {
                    c.setWrapperTransition(duration, s);
                    if (duration !== 0) {
                        c.onTransitionStart();
                        c.wrapper.transitionEnd(function(){
                            if (!controlled) return;
                            if (c.params.loop && s.params.controlBy === 'slide') {
                                c.fixLoop();
                            }
                            c.onTransitionEnd();

                        });
                    }
                }
                if (s.isArray(controlled)) {
                    for (i = 0; i < controlled.length; i++) {
                        if (controlled[i] !== byController && controlled[i] instanceof Swiper) {
                            setControlledTransition(controlled[i]);
                        }
                    }
                }
                else if (controlled instanceof Swiper && byController !== controlled) {
                    setControlledTransition(controlled);
                }
            }
        };

        /*=========================
          Hash Navigation
          ===========================*/
        s.hashnav = {
            init: function () {
                if (!s.params.hashnav) return;
                s.hashnav.initialized = true;
                var hash = document.location.hash.replace('#', '');
                if (!hash) return;
                var speed = 0;
                for (var i = 0, length = s.slides.length; i < length; i++) {
                    var slide = s.slides.eq(i);
                    var slideHash = slide.attr('data-hash');
                    if (slideHash === hash && !slide.hasClass(s.params.slideDuplicateClass)) {
                        var index = slide.index();
                        s.slideTo(index, speed, s.params.runCallbacksOnInit, true);
                    }
                }
            },
            setHash: function () {
                if (!s.hashnav.initialized || !s.params.hashnav) return;
                document.location.hash = s.slides.eq(s.activeIndex).attr('data-hash') || '';
            }
        };

        /*=========================
          Keyboard Control
          ===========================*/
        function handleKeyboard(e) {
            if (e.originalEvent) e = e.originalEvent; //jquery fix
            var kc = e.keyCode || e.charCode;
            // Directions locks
            if (!s.params.allowSwipeToNext && (isH() && kc === 39 || !isH() && kc === 40)) {
                return false;
            }
            if (!s.params.allowSwipeToPrev && (isH() && kc === 37 || !isH() && kc === 38)) {
                return false;
            }
            if (e.shiftKey || e.altKey || e.ctrlKey || e.metaKey) {
                return;
            }
            if (document.activeElement && document.activeElement.nodeName && (document.activeElement.nodeName.toLowerCase() === 'input' || document.activeElement.nodeName.toLowerCase() === 'textarea')) {
                return;
            }
            if (kc === 37 || kc === 39 || kc === 38 || kc === 40) {
                var inView = false;
                //Check that swiper should be inside of visible area of window
                if (s.container.parents('.swiper-slide').length > 0 && s.container.parents('.swiper-slide-active').length === 0) {
                    return;
                }
                var windowScroll = {
                    left: window.pageXOffset,
                    top: window.pageYOffset
                };
                var windowWidth = window.innerWidth;
                var windowHeight = window.innerHeight;
                var swiperOffset = s.container.offset();
                if (s.rtl) swiperOffset.left = swiperOffset.left - s.container[0].scrollLeft;
                var swiperCoord = [
                    [swiperOffset.left, swiperOffset.top],
                    [swiperOffset.left + s.width, swiperOffset.top],
                    [swiperOffset.left, swiperOffset.top + s.height],
                    [swiperOffset.left + s.width, swiperOffset.top + s.height]
                ];
                for (var i = 0; i < swiperCoord.length; i++) {
                    var point = swiperCoord[i];
                    if (
                        point[0] >= windowScroll.left && point[0] <= windowScroll.left + windowWidth &&
                        point[1] >= windowScroll.top && point[1] <= windowScroll.top + windowHeight
                    ) {
                        inView = true;
                    }

                }
                if (!inView) return;
            }
            if (isH()) {
                if (kc === 37 || kc === 39) {
                    if (e.preventDefault) e.preventDefault();
                    else e.returnValue = false;
                }
                if ((kc === 39 && !s.rtl) || (kc === 37 && s.rtl)) s.slideNext();
                if ((kc === 37 && !s.rtl) || (kc === 39 && s.rtl)) s.slidePrev();
            }
            else {
                if (kc === 38 || kc === 40) {
                    if (e.preventDefault) e.preventDefault();
                    else e.returnValue = false;
                }
                if (kc === 40) s.slideNext();
                if (kc === 38) s.slidePrev();
            }
        }
        s.disableKeyboardControl = function () {
            s.params.keyboardControl = false;
            $(document).off('keydown', handleKeyboard);
        };
        s.enableKeyboardControl = function () {
            s.params.keyboardControl = true;
            $(document).on('keydown', handleKeyboard);
        };


        /*=========================
          Mousewheel Control
          ===========================*/
        s.mousewheel = {
            event: false,
            lastScrollTime: (new window.Date()).getTime()
        };
        if (s.params.mousewheelControl) {
            try {
                new window.WheelEvent('wheel');
                s.mousewheel.event = 'wheel';
            } catch (e) {}

            if (!s.mousewheel.event && document.onmousewheel !== undefined) {
                s.mousewheel.event = 'mousewheel';
            }
            if (!s.mousewheel.event) {
                s.mousewheel.event = 'DOMMouseScroll';
            }
        }
        function handleMousewheel(e) {
            if (e.originalEvent) e = e.originalEvent; //jquery fix
            var we = s.mousewheel.event;
            var delta = 0;
            var rtlFactor = s.rtl ? -1 : 1;
            //Opera & IE
            if (e.detail) delta = -e.detail;
            //WebKits
            else if (we === 'mousewheel') {
                if (s.params.mousewheelForceToAxis) {
                    if (isH()) {
                        if (Math.abs(e.wheelDeltaX) > Math.abs(e.wheelDeltaY)) delta = e.wheelDeltaX * rtlFactor;
                        else return;
                    }
                    else {
                        if (Math.abs(e.wheelDeltaY) > Math.abs(e.wheelDeltaX)) delta = e.wheelDeltaY;
                        else return;
                    }
                }
                else {
                    delta = Math.abs(e.wheelDeltaX) > Math.abs(e.wheelDeltaY) ? - e.wheelDeltaX * rtlFactor : - e.wheelDeltaY;
                }
            }
            //Old FireFox
            else if (we === 'DOMMouseScroll') delta = -e.detail;
            //New FireFox
            else if (we === 'wheel') {
                if (s.params.mousewheelForceToAxis) {
                    if (isH()) {
                        if (Math.abs(e.deltaX) > Math.abs(e.deltaY)) delta = -e.deltaX * rtlFactor;
                        else return;
                    }
                    else {
                        if (Math.abs(e.deltaY) > Math.abs(e.deltaX)) delta = -e.deltaY;
                        else return;
                    }
                }
                else {
                    delta = Math.abs(e.deltaX) > Math.abs(e.deltaY) ? - e.deltaX * rtlFactor : - e.deltaY;
                }
            }
            if (delta === 0) return;

            if (s.params.mousewheelInvert) delta = -delta;

            if (!s.params.freeMode) {
                if ((new window.Date()).getTime() - s.mousewheel.lastScrollTime > 60) {
                    if (delta < 0) {
                        if ((!s.isEnd || s.params.loop) && !s.animating) s.slideNext();
                        else if (s.params.mousewheelReleaseOnEdges) return true;
                    }
                    else {
                        if ((!s.isBeginning || s.params.loop) && !s.animating) s.slidePrev();
                        else if (s.params.mousewheelReleaseOnEdges) return true;
                    }
                }
                s.mousewheel.lastScrollTime = (new window.Date()).getTime();

            }
            else {
                //Freemode or scrollContainer:
                var position = s.getWrapperTranslate() + delta * s.params.mousewheelSensitivity;
                var wasBeginning = s.isBeginning,
                    wasEnd = s.isEnd;

                if (position >= s.minTranslate()) position = s.minTranslate();
                if (position <= s.maxTranslate()) position = s.maxTranslate();

                s.setWrapperTransition(0);
                s.setWrapperTranslate(position);
                s.updateProgress();
                s.updateActiveIndex();

                if (!wasBeginning && s.isBeginning || !wasEnd && s.isEnd) {
                    s.updateClasses();
                }

                if (s.params.freeModeSticky) {
                    clearTimeout(s.mousewheel.timeout);
                    s.mousewheel.timeout = setTimeout(function () {
                        s.slideReset();
                    }, 300);
                }

                // Return page scroll on edge positions
                if (position === 0 || position === s.maxTranslate()) return;
            }
            if (s.params.autoplay) s.stopAutoplay();

            if (e.preventDefault) e.preventDefault();
            else e.returnValue = false;
            return false;
        }
        s.disableMousewheelControl = function () {
            if (!s.mousewheel.event) return false;
            s.container.off(s.mousewheel.event, handleMousewheel);
            return true;
        };

        s.enableMousewheelControl = function () {
            if (!s.mousewheel.event) return false;
            s.container.on(s.mousewheel.event, handleMousewheel);
            return true;
        };


        /*=========================
          Parallax
          ===========================*/
        function setParallaxTransform(el, progress) {
            el = $(el);
            var p, pX, pY;
            var rtlFactor = s.rtl ? -1 : 1;

            p = el.attr('data-swiper-parallax') || '0';
            pX = el.attr('data-swiper-parallax-x');
            pY = el.attr('data-swiper-parallax-y');
            if (pX || pY) {
                pX = pX || '0';
                pY = pY || '0';
            }
            else {
                if (isH()) {
                    pX = p;
                    pY = '0';
                }
                else {
                    pY = p;
                    pX = '0';
                }
            }

            if ((pX).indexOf('%') >= 0) {
                pX = parseInt(pX, 10) * progress * rtlFactor + '%';
            }
            else {
                pX = pX * progress * rtlFactor + 'px' ;
            }
            if ((pY).indexOf('%') >= 0) {
                pY = parseInt(pY, 10) * progress + '%';
            }
            else {
                pY = pY * progress + 'px' ;
            }

            el.transform('translate3d(' + pX + ', ' + pY + ',0px)');
        }
        s.parallax = {
            setTranslate: function () {
                s.container.children('[data-swiper-parallax], [data-swiper-parallax-x], [data-swiper-parallax-y]').each(function(){
                    setParallaxTransform(this, s.progress);

                });
                s.slides.each(function () {
                    var slide = $(this);
                    slide.find('[data-swiper-parallax], [data-swiper-parallax-x], [data-swiper-parallax-y]').each(function () {
                        var progress = Math.min(Math.max(slide[0].progress, -1), 1);
                        setParallaxTransform(this, progress);
                    });
                });
            },
            setTransition: function (duration) {
                if (typeof duration === 'undefined') duration = s.params.speed;
                s.container.find('[data-swiper-parallax], [data-swiper-parallax-x], [data-swiper-parallax-y]').each(function(){
                    var el = $(this);
                    var parallaxDuration = parseInt(el.attr('data-swiper-parallax-duration'), 10) || duration;
                    if (duration === 0) parallaxDuration = 0;
                    el.transition(parallaxDuration);
                });
            }
        };


        /*=========================
          Plugins API. Collect all and init all plugins
          ===========================*/
        s._plugins = [];
        for (var plugin in s.plugins) {
            var p = s.plugins[plugin](s, s.params[plugin]);
            if (p) s._plugins.push(p);
        }
        // Method to call all plugins event/method
        s.callPlugins = function (eventName) {
            for (var i = 0; i < s._plugins.length; i++) {
                if (eventName in s._plugins[i]) {
                    s._plugins[i][eventName](arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
                }
            }
        };

        /*=========================
          Events/Callbacks/Plugins Emitter
          ===========================*/
        function normalizeEventName (eventName) {
            if (eventName.indexOf('on') !== 0) {
                if (eventName[0] !== eventName[0].toUpperCase()) {
                    eventName = 'on' + eventName[0].toUpperCase() + eventName.substring(1);
                }
                else {
                    eventName = 'on' + eventName;
                }
            }
            return eventName;
        }
        s.emitterEventListeners = {

        };
        s.emit = function (eventName) {
            // Trigger callbacks
            if (s.params[eventName]) {
                s.params[eventName](arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
            }
            var i;
            // Trigger events
            if (s.emitterEventListeners[eventName]) {
                for (i = 0; i < s.emitterEventListeners[eventName].length; i++) {
                    s.emitterEventListeners[eventName][i](arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
                }
            }
            // Trigger plugins
            if (s.callPlugins) s.callPlugins(eventName, arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
        };
        s.on = function (eventName, handler) {
            eventName = normalizeEventName(eventName);
            if (!s.emitterEventListeners[eventName]) s.emitterEventListeners[eventName] = [];
            s.emitterEventListeners[eventName].push(handler);
            return s;
        };
        s.off = function (eventName, handler) {
            var i;
            eventName = normalizeEventName(eventName);
            if (typeof handler === 'undefined') {
                // Remove all handlers for such event
                s.emitterEventListeners[eventName] = [];
                return s;
            }
            if (!s.emitterEventListeners[eventName] || s.emitterEventListeners[eventName].length === 0) return;
            for (i = 0; i < s.emitterEventListeners[eventName].length; i++) {
                if(s.emitterEventListeners[eventName][i] === handler) s.emitterEventListeners[eventName].splice(i, 1);
            }
            return s;
        };
        s.once = function (eventName, handler) {
            eventName = normalizeEventName(eventName);
            var _handler = function () {
                handler(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4]);
                s.off(eventName, _handler);
            };
            s.on(eventName, _handler);
            return s;
        };

        // Accessibility tools
        s.a11y = {
            makeFocusable: function ($el) {
                $el.attr('tabIndex', '0');
                return $el;
            },
            addRole: function ($el, role) {
                $el.attr('role', role);
                return $el;
            },

            addLabel: function ($el, label) {
                $el.attr('aria-label', label);
                return $el;
            },

            disable: function ($el) {
                $el.attr('aria-disabled', true);
                return $el;
            },

            enable: function ($el) {
                $el.attr('aria-disabled', false);
                return $el;
            },

            onEnterKey: function (event) {
                if (event.keyCode !== 13) return;
                if ($(event.target).is(s.params.nextButton)) {
                    s.onClickNext(event);
                    if (s.isEnd) {
                        s.a11y.notify(s.params.lastSlideMessage);
                    }
                    else {
                        s.a11y.notify(s.params.nextSlideMessage);
                    }
                }
                else if ($(event.target).is(s.params.prevButton)) {
                    s.onClickPrev(event);
                    if (s.isBeginning) {
                        s.a11y.notify(s.params.firstSlideMessage);
                    }
                    else {
                        s.a11y.notify(s.params.prevSlideMessage);
                    }
                }
                if ($(event.target).is('.' + s.params.bulletClass)) {
                    $(event.target)[0].click();
                }
            },

            liveRegion: $('<span class="swiper-notification" aria-live="assertive" aria-atomic="true"></span>'),

            notify: function (message) {
                var notification = s.a11y.liveRegion;
                if (notification.length === 0) return;
                notification.html('');
                notification.html(message);
            },
            init: function () {
                // Setup accessibility
                if (s.params.nextButton) {
                    var nextButton = $(s.params.nextButton);
                    s.a11y.makeFocusable(nextButton);
                    s.a11y.addRole(nextButton, 'button');
                    s.a11y.addLabel(nextButton, s.params.nextSlideMessage);
                }
                if (s.params.prevButton) {
                    var prevButton = $(s.params.prevButton);
                    s.a11y.makeFocusable(prevButton);
                    s.a11y.addRole(prevButton, 'button');
                    s.a11y.addLabel(prevButton, s.params.prevSlideMessage);
                }

                $(s.container).append(s.a11y.liveRegion);
            },
            initPagination: function () {
                if (s.params.pagination && s.params.paginationClickable && s.bullets && s.bullets.length) {
                    s.bullets.each(function () {
                        var bullet = $(this);
                        s.a11y.makeFocusable(bullet);
                        s.a11y.addRole(bullet, 'button');
                        s.a11y.addLabel(bullet, s.params.paginationBulletMessage.replace(/{{index}}/, bullet.index() + 1));
                    });
                }
            },
            destroy: function () {
                if (s.a11y.liveRegion && s.a11y.liveRegion.length > 0) s.a11y.liveRegion.remove();
            }
        };


        /*=========================
          Init/Destroy
          ===========================*/
        s.init = function () {
            if (s.params.loop) s.createLoop();
            s.updateContainerSize();
            s.updateSlidesSize();
            s.updatePagination();
            if (s.params.scrollbar && s.scrollbar) {
                s.scrollbar.set();
                if (s.params.scrollbarDraggable) {
                    s.scrollbar.enableDraggable();
                }
            }
            if (s.params.effect !== 'slide' && s.effects[s.params.effect]) {
                if (!s.params.loop) s.updateProgress();
                s.effects[s.params.effect].setTranslate();
            }
            if (s.params.loop) {
                s.slideTo(s.params.initialSlide + s.loopedSlides, 0, s.params.runCallbacksOnInit);
            }
            else {
                s.slideTo(s.params.initialSlide, 0, s.params.runCallbacksOnInit);
                if (s.params.initialSlide === 0) {
                    if (s.parallax && s.params.parallax) s.parallax.setTranslate();
                    if (s.lazy && s.params.lazyLoading) {
                        s.lazy.load();
                        s.lazy.initialImageLoaded = true;
                    }
                }
            }
            s.attachEvents();
            if (s.params.observer && s.support.observer) {
                s.initObservers();
            }
            if (s.params.preloadImages && !s.params.lazyLoading) {
                s.preloadImages();
            }
            if (s.params.autoplay) {
                s.startAutoplay();
            }
            if (s.params.keyboardControl) {
                if (s.enableKeyboardControl) s.enableKeyboardControl();
            }
            if (s.params.mousewheelControl) {
                if (s.enableMousewheelControl) s.enableMousewheelControl();
            }
            if (s.params.hashnav) {
                if (s.hashnav) s.hashnav.init();
            }
            if (s.params.a11y && s.a11y) s.a11y.init();
            s.emit('onInit', s);
        };

        // Cleanup dynamic styles
        s.cleanupStyles = function () {
            // Container
            s.container.removeClass(s.classNames.join(' ')).removeAttr('style');

            // Wrapper
            s.wrapper.removeAttr('style');

            // Slides
            if (s.slides && s.slides.length) {
                s.slides
                    .removeClass([
                      s.params.slideVisibleClass,
                      s.params.slideActiveClass,
                      s.params.slideNextClass,
                      s.params.slidePrevClass
                    ].join(' '))
                    .removeAttr('style')
                    .removeAttr('data-swiper-column')
                    .removeAttr('data-swiper-row');
            }

            // Pagination/Bullets
            if (s.paginationContainer && s.paginationContainer.length) {
                s.paginationContainer.removeClass(s.params.paginationHiddenClass);
            }
            if (s.bullets && s.bullets.length) {
                s.bullets.removeClass(s.params.bulletActiveClass);
            }

            // Buttons
            if (s.params.prevButton) $(s.params.prevButton).removeClass(s.params.buttonDisabledClass);
            if (s.params.nextButton) $(s.params.nextButton).removeClass(s.params.buttonDisabledClass);

            // Scrollbar
            if (s.params.scrollbar && s.scrollbar) {
                if (s.scrollbar.track && s.scrollbar.track.length) s.scrollbar.track.removeAttr('style');
                if (s.scrollbar.drag && s.scrollbar.drag.length) s.scrollbar.drag.removeAttr('style');
            }
        };

        // Destroy
        s.destroy = function (deleteInstance, cleanupStyles) {
            // Detach evebts
            s.detachEvents();
            // Stop autoplay
            s.stopAutoplay();
            // Disable draggable
            if (s.params.scrollbar && s.scrollbar) {
                if (s.params.scrollbarDraggable) {
                    s.scrollbar.disableDraggable();
                }
            }
            // Destroy loop
            if (s.params.loop) {
                s.destroyLoop();
            }
            // Cleanup styles
            if (cleanupStyles) {
                s.cleanupStyles();
            }
            // Disconnect observer
            s.disconnectObservers();
            // Disable keyboard/mousewheel
            if (s.params.keyboardControl) {
                if (s.disableKeyboardControl) s.disableKeyboardControl();
            }
            if (s.params.mousewheelControl) {
                if (s.disableMousewheelControl) s.disableMousewheelControl();
            }
            // Disable a11y
            if (s.params.a11y && s.a11y) s.a11y.destroy();
            // Destroy callback
            s.emit('onDestroy');
            // Delete instance
            if (deleteInstance !== false) s = null;
        };

        s.init();



        // Return swiper instance
        return s;
    };


    /*==================================================
        Prototype
    ====================================================*/
    Swiper.prototype = {
        isSafari: (function () {
            var ua = navigator.userAgent.toLowerCase();
            return (ua.indexOf('safari') >= 0 && ua.indexOf('chrome') < 0 && ua.indexOf('android') < 0);
        })(),
        isUiWebView: /(iPhone|iPod|iPad).*AppleWebKit(?!.*Safari)/i.test(navigator.userAgent),
        isArray: function (arr) {
            return Object.prototype.toString.apply(arr) === '[object Array]';
        },
        /*==================================================
        Browser
        ====================================================*/
        browser: {
            ie: window.navigator.pointerEnabled || window.navigator.msPointerEnabled,
            ieTouch: (window.navigator.msPointerEnabled && window.navigator.msMaxTouchPoints > 1) || (window.navigator.pointerEnabled && window.navigator.maxTouchPoints > 1)
        },
        /*==================================================
        Devices
        ====================================================*/
        device: (function () {
            var ua = navigator.userAgent;
            var android = ua.match(/(Android);?[\s\/]+([\d.]+)?/);
            var ipad = ua.match(/(iPad).*OS\s([\d_]+)/);
            var ipod = ua.match(/(iPod)(.*OS\s([\d_]+))?/);
            var iphone = !ipad && ua.match(/(iPhone\sOS)\s([\d_]+)/);
            return {
                ios: ipad || iphone || ipod,
                android: android
            };
        })(),
        /*==================================================
        Feature Detection
        ====================================================*/
        support: {
            touch : (window.Modernizr && Modernizr.touch === true) || (function () {
                return !!(('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch);
            })(),

            transforms3d : (window.Modernizr && Modernizr.csstransforms3d === true) || (function () {
                var div = document.createElement('div').style;
                return ('webkitPerspective' in div || 'MozPerspective' in div || 'OPerspective' in div || 'MsPerspective' in div || 'perspective' in div);
            })(),

            flexbox: (function () {
                var div = document.createElement('div').style;
                var styles = ('alignItems webkitAlignItems webkitBoxAlign msFlexAlign mozBoxAlign webkitFlexDirection msFlexDirection mozBoxDirection mozBoxOrient webkitBoxDirection webkitBoxOrient').split(' ');
                for (var i = 0; i < styles.length; i++) {
                    if (styles[i] in div) return true;
                }
            })(),

            observer: (function () {
                return ('MutationObserver' in window || 'WebkitMutationObserver' in window);
            })()
        },
        /*==================================================
        Plugins
        ====================================================*/
        plugins: {}
    };


    /*===========================
    Dom7 Library
    ===========================*/
    var Dom7 = (function () {
        var Dom7 = function (arr) {
            var _this = this, i = 0;
            // Create array-like object
            for (i = 0; i < arr.length; i++) {
                _this[i] = arr[i];
            }
            _this.length = arr.length;
            // Return collection with methods
            return this;
        };
        var $ = function (selector, context) {
            var arr = [], i = 0;
            if (selector && !context) {
                if (selector instanceof Dom7) {
                    return selector;
                }
            }
            if (selector) {
                // String
                if (typeof selector === 'string') {
                    var els, tempParent, html = selector.trim();
                    if (html.indexOf('<') >= 0 && html.indexOf('>') >= 0) {
                        var toCreate = 'div';
                        if (html.indexOf('<li') === 0) toCreate = 'ul';
                        if (html.indexOf('<tr') === 0) toCreate = 'tbody';
                        if (html.indexOf('<td') === 0 || html.indexOf('<th') === 0) toCreate = 'tr';
                        if (html.indexOf('<tbody') === 0) toCreate = 'table';
                        if (html.indexOf('<option') === 0) toCreate = 'select';
                        tempParent = document.createElement(toCreate);
                        tempParent.innerHTML = selector;
                        for (i = 0; i < tempParent.childNodes.length; i++) {
                            arr.push(tempParent.childNodes[i]);
                        }
                    }
                    else {
                        if (!context && selector[0] === '#' && !selector.match(/[ .<>:~]/)) {
                            // Pure ID selector
                            els = [document.getElementById(selector.split('#')[1])];
                        }
                        else {
                            // Other selectors
                            els = (context || document).querySelectorAll(selector);
                        }
                        for (i = 0; i < els.length; i++) {
                            if (els[i]) arr.push(els[i]);
                        }
                    }
                }
                // Node/element
                else if (selector.nodeType || selector === window || selector === document) {
                    arr.push(selector);
                }
                //Array of elements or instance of Dom
                else if (selector.length > 0 && selector[0].nodeType) {
                    for (i = 0; i < selector.length; i++) {
                        arr.push(selector[i]);
                    }
                }
            }
            return new Dom7(arr);
        };
        Dom7.prototype = {
            // Classes and attriutes
            addClass: function (className) {
                if (typeof className === 'undefined') {
                    return this;
                }
                var classes = className.split(' ');
                for (var i = 0; i < classes.length; i++) {
                    for (var j = 0; j < this.length; j++) {
                        this[j].classList.add(classes[i]);
                    }
                }
                return this;
            },
            removeClass: function (className) {
                var classes = className.split(' ');
                for (var i = 0; i < classes.length; i++) {
                    for (var j = 0; j < this.length; j++) {
                        this[j].classList.remove(classes[i]);
                    }
                }
                return this;
            },
            hasClass: function (className) {
                if (!this[0]) return false;
                else return this[0].classList.contains(className);
            },
            toggleClass: function (className) {
                var classes = className.split(' ');
                for (var i = 0; i < classes.length; i++) {
                    for (var j = 0; j < this.length; j++) {
                        this[j].classList.toggle(classes[i]);
                    }
                }
                return this;
            },
            attr: function (attrs, value) {
                if (arguments.length === 1 && typeof attrs === 'string') {
                    // Get attr
                    if (this[0]) return this[0].getAttribute(attrs);
                    else return undefined;
                }
                else {
                    // Set attrs
                    for (var i = 0; i < this.length; i++) {
                        if (arguments.length === 2) {
                            // String
                            this[i].setAttribute(attrs, value);
                        }
                        else {
                            // Object
                            for (var attrName in attrs) {
                                this[i][attrName] = attrs[attrName];
                                this[i].setAttribute(attrName, attrs[attrName]);
                            }
                        }
                    }
                    return this;
                }
            },
            removeAttr: function (attr) {
                for (var i = 0; i < this.length; i++) {
                    this[i].removeAttribute(attr);
                }
                return this;
            },
            data: function (key, value) {
                if (typeof value === 'undefined') {
                    // Get value
                    if (this[0]) {
                        var dataKey = this[0].getAttribute('data-' + key);
                        if (dataKey) return dataKey;
                        else if (this[0].dom7ElementDataStorage && (key in this[0].dom7ElementDataStorage)) return this[0].dom7ElementDataStorage[key];
                        else return undefined;
                    }
                    else return undefined;
                }
                else {
                    // Set value
                    for (var i = 0; i < this.length; i++) {
                        var el = this[i];
                        if (!el.dom7ElementDataStorage) el.dom7ElementDataStorage = {};
                        el.dom7ElementDataStorage[key] = value;
                    }
                    return this;
                }
            },
            // Transforms
            transform : function (transform) {
                for (var i = 0; i < this.length; i++) {
                    var elStyle = this[i].style;
                    elStyle.webkitTransform = elStyle.MsTransform = elStyle.msTransform = elStyle.MozTransform = elStyle.OTransform = elStyle.transform = transform;
                }
                return this;
            },
            transition: function (duration) {
                if (typeof duration !== 'string') {
                    duration = duration + 'ms';
                }
                for (var i = 0; i < this.length; i++) {
                    var elStyle = this[i].style;
                    elStyle.webkitTransitionDuration = elStyle.MsTransitionDuration = elStyle.msTransitionDuration = elStyle.MozTransitionDuration = elStyle.OTransitionDuration = elStyle.transitionDuration = duration;
                }
                return this;
            },
            //Events
            on: function (eventName, targetSelector, listener, capture) {
                function handleLiveEvent(e) {
                    var target = e.target;
                    if ($(target).is(targetSelector)) listener.call(target, e);
                    else {
                        var parents = $(target).parents();
                        for (var k = 0; k < parents.length; k++) {
                            if ($(parents[k]).is(targetSelector)) listener.call(parents[k], e);
                        }
                    }
                }
                var events = eventName.split(' ');
                var i, j;
                for (i = 0; i < this.length; i++) {
                    if (typeof targetSelector === 'function' || targetSelector === false) {
                        // Usual events
                        if (typeof targetSelector === 'function') {
                            listener = arguments[1];
                            capture = arguments[2] || false;
                        }
                        for (j = 0; j < events.length; j++) {
                            this[i].addEventListener(events[j], listener, capture);
                        }
                    }
                    else {
                        //Live events
                        for (j = 0; j < events.length; j++) {
                            if (!this[i].dom7LiveListeners) this[i].dom7LiveListeners = [];
                            this[i].dom7LiveListeners.push({listener: listener, liveListener: handleLiveEvent});
                            this[i].addEventListener(events[j], handleLiveEvent, capture);
                        }
                    }
                }

                return this;
            },
            off: function (eventName, targetSelector, listener, capture) {
                var events = eventName.split(' ');
                for (var i = 0; i < events.length; i++) {
                    for (var j = 0; j < this.length; j++) {
                        if (typeof targetSelector === 'function' || targetSelector === false) {
                            // Usual events
                            if (typeof targetSelector === 'function') {
                                listener = arguments[1];
                                capture = arguments[2] || false;
                            }
                            this[j].removeEventListener(events[i], listener, capture);
                        }
                        else {
                            // Live event
                            if (this[j].dom7LiveListeners) {
                                for (var k = 0; k < this[j].dom7LiveListeners.length; k++) {
                                    if (this[j].dom7LiveListeners[k].listener === listener) {
                                        this[j].removeEventListener(events[i], this[j].dom7LiveListeners[k].liveListener, capture);
                                    }
                                }
                            }
                        }
                    }
                }
                return this;
            },
            once: function (eventName, targetSelector, listener, capture) {
                var dom = this;
                if (typeof targetSelector === 'function') {
                    targetSelector = false;
                    listener = arguments[1];
                    capture = arguments[2];
                }
                function proxy(e) {
                    listener(e);
                    dom.off(eventName, targetSelector, proxy, capture);
                }
                dom.on(eventName, targetSelector, proxy, capture);
            },
            trigger: function (eventName, eventData) {
                for (var i = 0; i < this.length; i++) {
                    var evt;
                    try {
                        evt = new window.CustomEvent(eventName, {detail: eventData, bubbles: true, cancelable: true});
                    }
                    catch (e) {
                        evt = document.createEvent('Event');
                        evt.initEvent(eventName, true, true);
                        evt.detail = eventData;
                    }
                    this[i].dispatchEvent(evt);
                }
                return this;
            },
            transitionEnd: function (callback) {
                var events = ['webkitTransitionEnd', 'transitionend', 'oTransitionEnd', 'MSTransitionEnd', 'msTransitionEnd'],
                    i, j, dom = this;
                function fireCallBack(e) {
                    /*jshint validthis:true */
                    if (e.target !== this) return;
                    callback.call(this, e);
                    for (i = 0; i < events.length; i++) {
                        dom.off(events[i], fireCallBack);
                    }
                }
                if (callback) {
                    for (i = 0; i < events.length; i++) {
                        dom.on(events[i], fireCallBack);
                    }
                }
                return this;
            },
            // Sizing/Styles
            width: function () {
                if (this[0] === window) {
                    return window.innerWidth;
                }
                else {
                    if (this.length > 0) {
                        return parseFloat(this.css('width'));
                    }
                    else {
                        return null;
                    }
                }
            },
            outerWidth: function (includeMargins) {
                if (this.length > 0) {
                    if (includeMargins)
                        return this[0].offsetWidth + parseFloat(this.css('margin-right')) + parseFloat(this.css('margin-left'));
                    else
                        return this[0].offsetWidth;
                }
                else return null;
            },
            height: function () {
                if (this[0] === window) {
                    return window.innerHeight;
                }
                else {
                    if (this.length > 0) {
                        return parseFloat(this.css('height'));
                    }
                    else {
                        return null;
                    }
                }
            },
            outerHeight: function (includeMargins) {
                if (this.length > 0) {
                    if (includeMargins)
                        return this[0].offsetHeight + parseFloat(this.css('margin-top')) + parseFloat(this.css('margin-bottom'));
                    else
                        return this[0].offsetHeight;
                }
                else return null;
            },
            offset: function () {
                if (this.length > 0) {
                    var el = this[0];
                    var box = el.getBoundingClientRect();
                    var body = document.body;
                    var clientTop  = el.clientTop  || body.clientTop  || 0;
                    var clientLeft = el.clientLeft || body.clientLeft || 0;
                    var scrollTop  = window.pageYOffset || el.scrollTop;
                    var scrollLeft = window.pageXOffset || el.scrollLeft;
                    return {
                        top: box.top  + scrollTop  - clientTop,
                        left: box.left + scrollLeft - clientLeft
                    };
                }
                else {
                    return null;
                }
            },
            css: function (props, value) {
                var i;
                if (arguments.length === 1) {
                    if (typeof props === 'string') {
                        if (this[0]) return window.getComputedStyle(this[0], null).getPropertyValue(props);
                    }
                    else {
                        for (i = 0; i < this.length; i++) {
                            for (var prop in props) {
                                this[i].style[prop] = props[prop];
                            }
                        }
                        return this;
                    }
                }
                if (arguments.length === 2 && typeof props === 'string') {
                    for (i = 0; i < this.length; i++) {
                        this[i].style[props] = value;
                    }
                    return this;
                }
                return this;
            },

            //Dom manipulation
            each: function (callback) {
                for (var i = 0; i < this.length; i++) {
                    callback.call(this[i], i, this[i]);
                }
                return this;
            },
            html: function (html) {
                if (typeof html === 'undefined') {
                    return this[0] ? this[0].innerHTML : undefined;
                }
                else {
                    for (var i = 0; i < this.length; i++) {
                        this[i].innerHTML = html;
                    }
                    return this;
                }
            },
            is: function (selector) {
                if (!this[0]) return false;
                var compareWith, i;
                if (typeof selector === 'string') {
                    var el = this[0];
                    if (el === document) return selector === document;
                    if (el === window) return selector === window;

                    if (el.matches) return el.matches(selector);
                    else if (el.webkitMatchesSelector) return el.webkitMatchesSelector(selector);
                    else if (el.mozMatchesSelector) return el.mozMatchesSelector(selector);
                    else if (el.msMatchesSelector) return el.msMatchesSelector(selector);
                    else {
                        compareWith = $(selector);
                        for (i = 0; i < compareWith.length; i++) {
                            if (compareWith[i] === this[0]) return true;
                        }
                        return false;
                    }
                }
                else if (selector === document) return this[0] === document;
                else if (selector === window) return this[0] === window;
                else {
                    if (selector.nodeType || selector instanceof Dom7) {
                        compareWith = selector.nodeType ? [selector] : selector;
                        for (i = 0; i < compareWith.length; i++) {
                            if (compareWith[i] === this[0]) return true;
                        }
                        return false;
                    }
                    return false;
                }

            },
            index: function () {
                if (this[0]) {
                    var child = this[0];
                    var i = 0;
                    while ((child = child.previousSibling) !== null) {
                        if (child.nodeType === 1) i++;
                    }
                    return i;
                }
                else return undefined;
            },
            eq: function (index) {
                if (typeof index === 'undefined') return this;
                var length = this.length;
                var returnIndex;
                if (index > length - 1) {
                    return new Dom7([]);
                }
                if (index < 0) {
                    returnIndex = length + index;
                    if (returnIndex < 0) return new Dom7([]);
                    else return new Dom7([this[returnIndex]]);
                }
                return new Dom7([this[index]]);
            },
            append: function (newChild) {
                var i, j;
                for (i = 0; i < this.length; i++) {
                    if (typeof newChild === 'string') {
                        var tempDiv = document.createElement('div');
                        tempDiv.innerHTML = newChild;
                        while (tempDiv.firstChild) {
                            this[i].appendChild(tempDiv.firstChild);
                        }
                    }
                    else if (newChild instanceof Dom7) {
                        for (j = 0; j < newChild.length; j++) {
                            this[i].appendChild(newChild[j]);
                        }
                    }
                    else {
                        this[i].appendChild(newChild);
                    }
                }
                return this;
            },
            prepend: function (newChild) {
                var i, j;
                for (i = 0; i < this.length; i++) {
                    if (typeof newChild === 'string') {
                        var tempDiv = document.createElement('div');
                        tempDiv.innerHTML = newChild;
                        for (j = tempDiv.childNodes.length - 1; j >= 0; j--) {
                            this[i].insertBefore(tempDiv.childNodes[j], this[i].childNodes[0]);
                        }
                        // this[i].insertAdjacentHTML('afterbegin', newChild);
                    }
                    else if (newChild instanceof Dom7) {
                        for (j = 0; j < newChild.length; j++) {
                            this[i].insertBefore(newChild[j], this[i].childNodes[0]);
                        }
                    }
                    else {
                        this[i].insertBefore(newChild, this[i].childNodes[0]);
                    }
                }
                return this;
            },
            insertBefore: function (selector) {
                var before = $(selector);
                for (var i = 0; i < this.length; i++) {
                    if (before.length === 1) {
                        before[0].parentNode.insertBefore(this[i], before[0]);
                    }
                    else if (before.length > 1) {
                        for (var j = 0; j < before.length; j++) {
                            before[j].parentNode.insertBefore(this[i].cloneNode(true), before[j]);
                        }
                    }
                }
            },
            insertAfter: function (selector) {
                var after = $(selector);
                for (var i = 0; i < this.length; i++) {
                    if (after.length === 1) {
                        after[0].parentNode.insertBefore(this[i], after[0].nextSibling);
                    }
                    else if (after.length > 1) {
                        for (var j = 0; j < after.length; j++) {
                            after[j].parentNode.insertBefore(this[i].cloneNode(true), after[j].nextSibling);
                        }
                    }
                }
            },
            next: function (selector) {
                if (this.length > 0) {
                    if (selector) {
                        if (this[0].nextElementSibling && $(this[0].nextElementSibling).is(selector)) return new Dom7([this[0].nextElementSibling]);
                        else return new Dom7([]);
                    }
                    else {
                        if (this[0].nextElementSibling) return new Dom7([this[0].nextElementSibling]);
                        else return new Dom7([]);
                    }
                }
                else return new Dom7([]);
            },
            nextAll: function (selector) {
                var nextEls = [];
                var el = this[0];
                if (!el) return new Dom7([]);
                while (el.nextElementSibling) {
                    var next = el.nextElementSibling;
                    if (selector) {
                        if($(next).is(selector)) nextEls.push(next);
                    }
                    else nextEls.push(next);
                    el = next;
                }
                return new Dom7(nextEls);
            },
            prev: function (selector) {
                if (this.length > 0) {
                    if (selector) {
                        if (this[0].previousElementSibling && $(this[0].previousElementSibling).is(selector)) return new Dom7([this[0].previousElementSibling]);
                        else return new Dom7([]);
                    }
                    else {
                        if (this[0].previousElementSibling) return new Dom7([this[0].previousElementSibling]);
                        else return new Dom7([]);
                    }
                }
                else return new Dom7([]);
            },
            prevAll: function (selector) {
                var prevEls = [];
                var el = this[0];
                if (!el) return new Dom7([]);
                while (el.previousElementSibling) {
                    var prev = el.previousElementSibling;
                    if (selector) {
                        if($(prev).is(selector)) prevEls.push(prev);
                    }
                    else prevEls.push(prev);
                    el = prev;
                }
                return new Dom7(prevEls);
            },
            parent: function (selector) {
                var parents = [];
                for (var i = 0; i < this.length; i++) {
                    if (selector) {
                        if ($(this[i].parentNode).is(selector)) parents.push(this[i].parentNode);
                    }
                    else {
                        parents.push(this[i].parentNode);
                    }
                }
                return $($.unique(parents));
            },
            parents: function (selector) {
                var parents = [];
                for (var i = 0; i < this.length; i++) {
                    var parent = this[i].parentNode;
                    while (parent) {
                        if (selector) {
                            if ($(parent).is(selector)) parents.push(parent);
                        }
                        else {
                            parents.push(parent);
                        }
                        parent = parent.parentNode;
                    }
                }
                return $($.unique(parents));
            },
            find : function (selector) {
                var foundElements = [];
                for (var i = 0; i < this.length; i++) {
                    var found = this[i].querySelectorAll(selector);
                    for (var j = 0; j < found.length; j++) {
                        foundElements.push(found[j]);
                    }
                }
                return new Dom7(foundElements);
            },
            children: function (selector) {
                var children = [];
                for (var i = 0; i < this.length; i++) {
                    var childNodes = this[i].childNodes;

                    for (var j = 0; j < childNodes.length; j++) {
                        if (!selector) {
                            if (childNodes[j].nodeType === 1) children.push(childNodes[j]);
                        }
                        else {
                            if (childNodes[j].nodeType === 1 && $(childNodes[j]).is(selector)) children.push(childNodes[j]);
                        }
                    }
                }
                return new Dom7($.unique(children));
            },
            remove: function () {
                for (var i = 0; i < this.length; i++) {
                    if (this[i].parentNode) this[i].parentNode.removeChild(this[i]);
                }
                return this;
            },
            add: function () {
                var dom = this;
                var i, j;
                for (i = 0; i < arguments.length; i++) {
                    var toAdd = $(arguments[i]);
                    for (j = 0; j < toAdd.length; j++) {
                        dom[dom.length] = toAdd[j];
                        dom.length++;
                    }
                }
                return dom;
            }
        };
        $.fn = Dom7.prototype;
        $.unique = function (arr) {
            var unique = [];
            for (var i = 0; i < arr.length; i++) {
                if (unique.indexOf(arr[i]) === -1) unique.push(arr[i]);
            }
            return unique;
        };

        return $;
    })();


    /*===========================
     Get Dom libraries
     ===========================*/
    var swiperDomPlugins = ['jQuery', 'Zepto', 'Dom7'];
    for (var i = 0; i < swiperDomPlugins.length; i++) {
    	if (window[swiperDomPlugins[i]]) {
    		addLibraryPlugin(window[swiperDomPlugins[i]]);
    	}
    }
    // Required DOM Plugins
    var domLib;
    if (typeof Dom7 === 'undefined') {
    	domLib = window.Dom7 || window.Zepto || window.jQuery;
    }
    else {
    	domLib = Dom7;
    }

    /*===========================
    Add .swiper plugin from Dom libraries
    ===========================*/
    function addLibraryPlugin(lib) {
        lib.fn.swiper = function (params) {
            var firstInstance;
            lib(this).each(function () {
                var s = new Swiper(this, params);
                if (!firstInstance) firstInstance = s;
            });
            return firstInstance;
        };
    }

    if (domLib) {
        if (!('transitionEnd' in domLib.fn)) {
            domLib.fn.transitionEnd = function (callback) {
                var events = ['webkitTransitionEnd', 'transitionend', 'oTransitionEnd', 'MSTransitionEnd', 'msTransitionEnd'],
                    i, j, dom = this;
                function fireCallBack(e) {
                    /*jshint validthis:true */
                    if (e.target !== this) return;
                    callback.call(this, e);
                    for (i = 0; i < events.length; i++) {
                        dom.off(events[i], fireCallBack);
                    }
                }
                if (callback) {
                    for (i = 0; i < events.length; i++) {
                        dom.on(events[i], fireCallBack);
                    }
                }
                return this;
            };
        }
        if (!('transform' in domLib.fn)) {
            domLib.fn.transform = function (transform) {
                for (var i = 0; i < this.length; i++) {
                    var elStyle = this[i].style;
                    elStyle.webkitTransform = elStyle.MsTransform = elStyle.msTransform = elStyle.MozTransform = elStyle.OTransform = elStyle.transform = transform;
                }
                return this;
            };
        }
        if (!('transition' in domLib.fn)) {
            domLib.fn.transition = function (duration) {
                if (typeof duration !== 'string') {
                    duration = duration + 'ms';
                }
                for (var i = 0; i < this.length; i++) {
                    var elStyle = this[i].style;
                    elStyle.webkitTransitionDuration = elStyle.MsTransitionDuration = elStyle.msTransitionDuration = elStyle.MozTransitionDuration = elStyle.OTransitionDuration = elStyle.transitionDuration = duration;
                }
                return this;
            };
        }
    }

    ionic.views.Swiper = Swiper;
})();

(function(ionic) {
'use strict';

  ionic.views.Toggle = ionic.views.View.inherit({
    initialize: function(opts) {
      var self = this;

      this.el = opts.el;
      this.checkbox = opts.checkbox;
      this.track = opts.track;
      this.handle = opts.handle;
      this.openPercent = -1;
      this.onChange = opts.onChange || function() {};

      this.triggerThreshold = opts.triggerThreshold || 20;

      this.dragStartHandler = function(e) {
        self.dragStart(e);
      };
      this.dragHandler = function(e) {
        self.drag(e);
      };
      this.holdHandler = function(e) {
        self.hold(e);
      };
      this.releaseHandler = function(e) {
        self.release(e);
      };

      this.dragStartGesture = ionic.onGesture('dragstart', this.dragStartHandler, this.el);
      this.dragGesture = ionic.onGesture('drag', this.dragHandler, this.el);
      this.dragHoldGesture = ionic.onGesture('hold', this.holdHandler, this.el);
      this.dragReleaseGesture = ionic.onGesture('release', this.releaseHandler, this.el);
    },

    destroy: function() {
      ionic.offGesture(this.dragStartGesture, 'dragstart', this.dragStartGesture);
      ionic.offGesture(this.dragGesture, 'drag', this.dragGesture);
      ionic.offGesture(this.dragHoldGesture, 'hold', this.holdHandler);
      ionic.offGesture(this.dragReleaseGesture, 'release', this.releaseHandler);
    },

    tap: function() {
      if(this.el.getAttribute('disabled') !== 'disabled') {
        this.val( !this.checkbox.checked );
      }
    },

    dragStart: function(e) {
      if(this.checkbox.disabled) return;

      this._dragInfo = {
        width: this.el.offsetWidth,
        left: this.el.offsetLeft,
        right: this.el.offsetLeft + this.el.offsetWidth,
        triggerX: this.el.offsetWidth / 2,
        initialState: this.checkbox.checked
      };

      // Stop any parent dragging
      e.gesture.srcEvent.preventDefault();

      // Trigger hold styles
      this.hold(e);
    },

    drag: function(e) {
      var self = this;
      if(!this._dragInfo) { return; }

      // Stop any parent dragging
      e.gesture.srcEvent.preventDefault();

      ionic.requestAnimationFrame(function () {
        if (!self._dragInfo) { return; }

        var px = e.gesture.touches[0].pageX - self._dragInfo.left;
        var mx = self._dragInfo.width - self.triggerThreshold;

        // The initial state was on, so "tend towards" on
        if(self._dragInfo.initialState) {
          if(px < self.triggerThreshold) {
            self.setOpenPercent(0);
          } else if(px > self._dragInfo.triggerX) {
            self.setOpenPercent(100);
          }
        } else {
          // The initial state was off, so "tend towards" off
          if(px < self._dragInfo.triggerX) {
            self.setOpenPercent(0);
          } else if(px > mx) {
            self.setOpenPercent(100);
          }
        }
      });
    },

    endDrag: function() {
      this._dragInfo = null;
    },

    hold: function() {
      this.el.classList.add('dragging');
    },
    release: function(e) {
      this.el.classList.remove('dragging');
      this.endDrag(e);
    },


    setOpenPercent: function(openPercent) {
      // only make a change if the new open percent has changed
      if(this.openPercent < 0 || (openPercent < (this.openPercent - 3) || openPercent > (this.openPercent + 3) ) ) {
        this.openPercent = openPercent;

        if(openPercent === 0) {
          this.val(false);
        } else if(openPercent === 100) {
          this.val(true);
        } else {
          var openPixel = Math.round( (openPercent / 100) * this.track.offsetWidth - (this.handle.offsetWidth) );
          openPixel = (openPixel < 1 ? 0 : openPixel);
          this.handle.style[ionic.CSS.TRANSFORM] = 'translate3d(' + openPixel + 'px,0,0)';
        }
      }
    },

    val: function(value) {
      if(value === true || value === false) {
        if(this.handle.style[ionic.CSS.TRANSFORM] !== "") {
          this.handle.style[ionic.CSS.TRANSFORM] = "";
        }
        this.checkbox.checked = value;
        this.openPercent = (value ? 100 : 0);
        this.onChange && this.onChange();
      }
      return this.checkbox.checked;
    }

  });

})(ionic);

})();
/*!
 * Copyright 2015 Drifty Co.
 * http://drifty.com/
 *
 * Ionic, v1.3.4
 * A powerful HTML5 mobile app framework.
 * http://ionicframework.com/
 *
 * By @maxlynch, @benjsperry, @adamdbradley <3
 *
 * Licensed under the MIT license. Please see LICENSE for more information.
 *
 */

(function() {
/* eslint no-unused-vars:0 */
var IonicModule = angular.module('ionic', ['ngAnimate', 'ngSanitize', 'ui.router', 'ngIOS9UIWebViewPatch']),
  extend = angular.extend,
  forEach = angular.forEach,
  isDefined = angular.isDefined,
  isNumber = angular.isNumber,
  isString = angular.isString,
  jqLite = angular.element,
  noop = angular.noop;

/**
 * @ngdoc service
 * @name $ionicActionSheet
 * @module ionic
 * @description
 * The Action Sheet is a slide-up pane that lets the user choose from a set of options.
 * Dangerous options are highlighted in red and made obvious.
 *
 * There are easy ways to cancel out of the action sheet, such as tapping the backdrop or even
 * hitting escape on the keyboard for desktop testing.
 *
 * ![Action Sheet](http://ionicframework.com.s3.amazonaws.com/docs/controllers/actionSheet.gif)
 *
 * @usage
 * To trigger an Action Sheet in your code, use the $ionicActionSheet service in your angular controllers:
 *
 * ```js
 * angular.module('mySuperApp', ['ionic'])
 * .controller(function($scope, $ionicActionSheet, $timeout) {
 *
 *  // Triggered on a button click, or some other target
 *  $scope.show = function() {
 *
 *    // Show the action sheet
 *    var hideSheet = $ionicActionSheet.show({
 *      buttons: [
 *        { text: '<b>Share</b> This' },
 *        { text: 'Move' }
 *      ],
 *      destructiveText: 'Delete',
 *      titleText: 'Modify your album',
 *      cancelText: 'Cancel',
 *      cancel: function() {
          // add cancel code..
        },
 *      buttonClicked: function(index) {
 *        return true;
 *      }
 *    });
 *
 *    // For example's sake, hide the sheet after two seconds
 *    $timeout(function() {
 *      hideSheet();
 *    }, 2000);
 *
 *  };
 * });
 * ```
 *
 */
IonicModule
.factory('$ionicActionSheet', [
  '$rootScope',
  '$compile',
  '$animate',
  '$timeout',
  '$ionicTemplateLoader',
  '$ionicPlatform',
  '$ionicBody',
  'IONIC_BACK_PRIORITY',
function($rootScope, $compile, $animate, $timeout, $ionicTemplateLoader, $ionicPlatform, $ionicBody, IONIC_BACK_PRIORITY) {

  return {
    show: actionSheet
  };

  /**
   * @ngdoc method
   * @name $ionicActionSheet#show
   * @description
   * Load and return a new action sheet.
   *
   * A new isolated scope will be created for the
   * action sheet and the new element will be appended into the body.
   *
   * @param {object} options The options for this ActionSheet. Properties:
   *
   *  - `[Object]` `buttons` Which buttons to show.  Each button is an object with a `text` field.
   *  - `{string}` `titleText` The title to show on the action sheet.
   *  - `{string=}` `cancelText` the text for a 'cancel' button on the action sheet.
   *  - `{string=}` `destructiveText` The text for a 'danger' on the action sheet.
   *  - `{function=}` `cancel` Called if the cancel button is pressed, the backdrop is tapped or
   *     the hardware back button is pressed.
   *  - `{function=}` `buttonClicked` Called when one of the non-destructive buttons is clicked,
   *     with the index of the button that was clicked and the button object. Return true to close
   *     the action sheet, or false to keep it opened.
   *  - `{function=}` `destructiveButtonClicked` Called when the destructive button is clicked.
   *     Return true to close the action sheet, or false to keep it opened.
   *  -  `{boolean=}` `cancelOnStateChange` Whether to cancel the actionSheet when navigating
   *     to a new state.  Default true.
   *  - `{string}` `cssClass` The custom CSS class name.
   *
   * @returns {function} `hideSheet` A function which, when called, hides & cancels the action sheet.
   */
  function actionSheet(opts) {
    var scope = $rootScope.$new(true);

    extend(scope, {
      cancel: noop,
      destructiveButtonClicked: noop,
      buttonClicked: noop,
      $deregisterBackButton: noop,
      buttons: [],
      cancelOnStateChange: true
    }, opts || {});

    function textForIcon(text) {
      if (text && /icon/.test(text)) {
        scope.$actionSheetHasIcon = true;
      }
    }

    for (var x = 0; x < scope.buttons.length; x++) {
      textForIcon(scope.buttons[x].text);
    }
    textForIcon(scope.cancelText);
    textForIcon(scope.destructiveText);

    // Compile the template
    var element = scope.element = $compile('<ion-action-sheet ng-class="cssClass" buttons="buttons"></ion-action-sheet>')(scope);

    // Grab the sheet element for animation
    var sheetEl = jqLite(element[0].querySelector('.action-sheet-wrapper'));

    var stateChangeListenDone = scope.cancelOnStateChange ?
      $rootScope.$on('$stateChangeSuccess', function() { scope.cancel(); }) :
      noop;

    // removes the actionSheet from the screen
    scope.removeSheet = function(done) {
      if (scope.removed) return;

      scope.removed = true;
      sheetEl.removeClass('action-sheet-up');
      $timeout(function() {
        // wait to remove this due to a 300ms delay native
        // click which would trigging whatever was underneath this
        $ionicBody.removeClass('action-sheet-open');
      }, 400);
      scope.$deregisterBackButton();
      stateChangeListenDone();

      $animate.removeClass(element, 'active').then(function() {
        scope.$destroy();
        element.remove();
        // scope.cancel.$scope is defined near the bottom
        scope.cancel.$scope = sheetEl = null;
        (done || noop)(opts.buttons);
      });
    };

    scope.showSheet = function(done) {
      if (scope.removed) return;

      $ionicBody.append(element)
                .addClass('action-sheet-open');

      $animate.addClass(element, 'active').then(function() {
        if (scope.removed) return;
        (done || noop)();
      });
      $timeout(function() {
        if (scope.removed) return;
        sheetEl.addClass('action-sheet-up');
      }, 20, false);
    };

    // registerBackButtonAction returns a callback to deregister the action
    scope.$deregisterBackButton = $ionicPlatform.registerBackButtonAction(
      function() {
        $timeout(scope.cancel);
      },
      IONIC_BACK_PRIORITY.actionSheet
    );

    // called when the user presses the cancel button
    scope.cancel = function() {
      // after the animation is out, call the cancel callback
      scope.removeSheet(opts.cancel);
    };

    scope.buttonClicked = function(index) {
      // Check if the button click event returned true, which means
      // we can close the action sheet
      if (opts.buttonClicked(index, opts.buttons[index]) === true) {
        scope.removeSheet();
      }
    };

    scope.destructiveButtonClicked = function() {
      // Check if the destructive button click event returned true, which means
      // we can close the action sheet
      if (opts.destructiveButtonClicked() === true) {
        scope.removeSheet();
      }
    };

    scope.showSheet();

    // Expose the scope on $ionicActionSheet's return value for the sake
    // of testing it.
    scope.cancel.$scope = scope;

    return scope.cancel;
  }
}]);


jqLite.prototype.addClass = function(cssClasses) {
  var x, y, cssClass, el, splitClasses, existingClasses;
  if (cssClasses && cssClasses != 'ng-scope' && cssClasses != 'ng-isolate-scope') {
    for (x = 0; x < this.length; x++) {
      el = this[x];
      if (el.setAttribute) {

        if (cssClasses.indexOf(' ') < 0 && el.classList.add) {
          el.classList.add(cssClasses);
        } else {
          existingClasses = (' ' + (el.getAttribute('class') || '') + ' ')
            .replace(/[\n\t]/g, " ");
          splitClasses = cssClasses.split(' ');

          for (y = 0; y < splitClasses.length; y++) {
            cssClass = splitClasses[y].trim();
            if (existingClasses.indexOf(' ' + cssClass + ' ') === -1) {
              existingClasses += cssClass + ' ';
            }
          }
          el.setAttribute('class', existingClasses.trim());
        }
      }
    }
  }
  return this;
};

jqLite.prototype.removeClass = function(cssClasses) {
  var x, y, splitClasses, cssClass, el;
  if (cssClasses) {
    for (x = 0; x < this.length; x++) {
      el = this[x];
      if (el.getAttribute) {
        if (cssClasses.indexOf(' ') < 0 && el.classList.remove) {
          el.classList.remove(cssClasses);
        } else {
          splitClasses = cssClasses.split(' ');

          for (y = 0; y < splitClasses.length; y++) {
            cssClass = splitClasses[y];
            el.setAttribute('class', (
                (" " + (el.getAttribute('class') || '') + " ")
                .replace(/[\n\t]/g, " ")
                .replace(" " + cssClass.trim() + " ", " ")).trim()
            );
          }
        }
      }
    }
  }
  return this;
};

/**
 * @ngdoc service
 * @name $ionicBackdrop
 * @module ionic
 * @description
 * Shows and hides a backdrop over the UI.  Appears behind popups, loading,
 * and other overlays.
 *
 * Often, multiple UI components require a backdrop, but only one backdrop is
 * ever needed in the DOM at a time.
 *
 * Therefore, each component that requires the backdrop to be shown calls
 * `$ionicBackdrop.retain()` when it wants the backdrop, then `$ionicBackdrop.release()`
 * when it is done with the backdrop.
 *
 * For each time `retain` is called, the backdrop will be shown until `release` is called.
 *
 * For example, if `retain` is called three times, the backdrop will be shown until `release`
 * is called three times.
 *
 * **Notes:**
 * - The backdrop service will broadcast 'backdrop.shown' and 'backdrop.hidden' events from the root scope,
 * this is useful for alerting native components not in html.
 *
 * @usage
 *
 * ```js
 * function MyController($scope, $ionicBackdrop, $timeout, $rootScope) {
 *   //Show a backdrop for one second
 *   $scope.action = function() {
 *     $ionicBackdrop.retain();
 *     $timeout(function() {
 *       $ionicBackdrop.release();
 *     }, 1000);
 *   };
 *
 *   // Execute action on backdrop disappearing
 *   $scope.$on('backdrop.hidden', function() {
 *     // Execute action
 *   });
 *
 *   // Execute action on backdrop appearing
 *   $scope.$on('backdrop.shown', function() {
 *     // Execute action
 *   });
 *
 * }
 * ```
 */
IonicModule
.factory('$ionicBackdrop', [
  '$document', '$timeout', '$$rAF', '$rootScope',
function($document, $timeout, $$rAF, $rootScope) {

  var el = jqLite('<div class="backdrop">');
  var backdropHolds = 0;

  $document[0].body.appendChild(el[0]);

  return {
    /**
     * @ngdoc method
     * @name $ionicBackdrop#retain
     * @description Retains the backdrop.
     */
    retain: retain,
    /**
     * @ngdoc method
     * @name $ionicBackdrop#release
     * @description
     * Releases the backdrop.
     */
    release: release,

    getElement: getElement,

    // exposed for testing
    _element: el
  };

  function retain() {
    backdropHolds++;
    if (backdropHolds === 1) {
      el.addClass('visible');
      $rootScope.$broadcast('backdrop.shown');
      $$rAF(function() {
        // If we're still at >0 backdropHolds after async...
        if (backdropHolds >= 1) el.addClass('active');
      });
    }
  }
  function release() {
    if (backdropHolds === 1) {
      el.removeClass('active');
      $rootScope.$broadcast('backdrop.hidden');
      $timeout(function() {
        // If we're still at 0 backdropHolds after async...
        if (backdropHolds === 0) el.removeClass('visible');
      }, 400, false);
    }
    backdropHolds = Math.max(0, backdropHolds - 1);
  }

  function getElement() {
    return el;
  }

}]);

/**
 * @private
 */
IonicModule
.factory('$ionicBind', ['$parse', '$interpolate', function($parse, $interpolate) {
  var LOCAL_REGEXP = /^\s*([@=&])(\??)\s*(\w*)\s*$/;
  return function(scope, attrs, bindDefinition) {
    forEach(bindDefinition || {}, function(definition, scopeName) {
      //Adapted from angular.js $compile
      var match = definition.match(LOCAL_REGEXP) || [],
        attrName = match[3] || scopeName,
        mode = match[1], // @, =, or &
        parentGet,
        unwatch;

      switch (mode) {
        case '@':
          if (!attrs[attrName]) {
            return;
          }
          attrs.$observe(attrName, function(value) {
            scope[scopeName] = value;
          });
          // we trigger an interpolation to ensure
          // the value is there for use immediately
          if (attrs[attrName]) {
            scope[scopeName] = $interpolate(attrs[attrName])(scope);
          }
          break;

        case '=':
          if (!attrs[attrName]) {
            return;
          }
          unwatch = scope.$watch(attrs[attrName], function(value) {
            scope[scopeName] = value;
          });
          //Destroy parent scope watcher when this scope is destroyed
          scope.$on('$destroy', unwatch);
          break;

        case '&':
          /* jshint -W044 */
          if (attrs[attrName] && attrs[attrName].match(RegExp(scopeName + '\(.*?\)'))) {
            throw new Error('& expression binding "' + scopeName + '" looks like it will recursively call "' +
                          attrs[attrName] + '" and cause a stack overflow! Please choose a different scopeName.');
          }
          parentGet = $parse(attrs[attrName]);
          scope[scopeName] = function(locals) {
            return parentGet(scope, locals);
          };
          break;
      }
    });
  };
}]);

/**
 * @ngdoc service
 * @name $ionicBody
 * @module ionic
 * @description An angular utility service to easily and efficiently
 * add and remove CSS classes from the document's body element.
 */
IonicModule
.factory('$ionicBody', ['$document', function($document) {
  return {
    /**
     * @ngdoc method
     * @name $ionicBody#addClass
     * @description Add a class to the document's body element.
     * @param {string} class Each argument will be added to the body element.
     * @returns {$ionicBody} The $ionicBody service so methods can be chained.
     */
    addClass: function() {
      for (var x = 0; x < arguments.length; x++) {
        $document[0].body.classList.add(arguments[x]);
      }
      return this;
    },
    /**
     * @ngdoc method
     * @name $ionicBody#removeClass
     * @description Remove a class from the document's body element.
     * @param {string} class Each argument will be removed from the body element.
     * @returns {$ionicBody} The $ionicBody service so methods can be chained.
     */
    removeClass: function() {
      for (var x = 0; x < arguments.length; x++) {
        $document[0].body.classList.remove(arguments[x]);
      }
      return this;
    },
    /**
     * @ngdoc method
     * @name $ionicBody#enableClass
     * @description Similar to the `add` method, except the first parameter accepts a boolean
     * value determining if the class should be added or removed. Rather than writing user code,
     * such as "if true then add the class, else then remove the class", this method can be
     * given a true or false value which reduces redundant code.
     * @param {boolean} shouldEnableClass A true/false value if the class should be added or removed.
     * @param {string} class Each remaining argument would be added or removed depending on
     * the first argument.
     * @returns {$ionicBody} The $ionicBody service so methods can be chained.
     */
    enableClass: function(shouldEnableClass) {
      var args = Array.prototype.slice.call(arguments).slice(1);
      if (shouldEnableClass) {
        this.addClass.apply(this, args);
      } else {
        this.removeClass.apply(this, args);
      }
      return this;
    },
    /**
     * @ngdoc method
     * @name $ionicBody#append
     * @description Append a child to the document's body.
     * @param {element} element The element to be appended to the body. The passed in element
     * can be either a jqLite element, or a DOM element.
     * @returns {$ionicBody} The $ionicBody service so methods can be chained.
     */
    append: function(ele) {
      $document[0].body.appendChild(ele.length ? ele[0] : ele);
      return this;
    },
    /**
     * @ngdoc method
     * @name $ionicBody#get
     * @description Get the document's body element.
     * @returns {element} Returns the document's body element.
     */
    get: function() {
      return $document[0].body;
    }
  };
}]);

IonicModule
.factory('$ionicClickBlock', [
  '$document',
  '$ionicBody',
  '$timeout',
function($document, $ionicBody, $timeout) {
  var CSS_HIDE = 'click-block-hide';
  var cbEle, fallbackTimer, pendingShow;

  function preventClick(ev) {
    ev.preventDefault();
    ev.stopPropagation();
  }

  function addClickBlock() {
    if (pendingShow) {
      if (cbEle) {
        cbEle.classList.remove(CSS_HIDE);
      } else {
        cbEle = $document[0].createElement('div');
        cbEle.className = 'click-block';
        $ionicBody.append(cbEle);
        cbEle.addEventListener('touchstart', preventClick);
        cbEle.addEventListener('mousedown', preventClick);
      }
      pendingShow = false;
    }
  }

  function removeClickBlock() {
    cbEle && cbEle.classList.add(CSS_HIDE);
  }

  return {
    show: function(autoExpire) {
      pendingShow = true;
      $timeout.cancel(fallbackTimer);
      fallbackTimer = $timeout(this.hide, autoExpire || 310, false);
      addClickBlock();
    },
    hide: function() {
      pendingShow = false;
      $timeout.cancel(fallbackTimer);
      removeClickBlock();
    }
  };
}]);

/**
 * @ngdoc service
 * @name $ionicGesture
 * @module ionic
 * @description An angular service exposing ionic
 * {@link ionic.utility:ionic.EventController}'s gestures.
 */
IonicModule
.factory('$ionicGesture', [function() {
  return {
    /**
     * @ngdoc method
     * @name $ionicGesture#on
     * @description Add an event listener for a gesture on an element. See {@link ionic.utility:ionic.EventController#onGesture}.
     * @param {string} eventType The gesture event to listen for.
     * @param {function(e)} callback The function to call when the gesture
     * happens.
     * @param {element} $element The angular element to listen for the event on.
     * @param {object} options object.
     * @returns {ionic.Gesture} The gesture object (use this to remove the gesture later on).
     */
    on: function(eventType, cb, $element, options) {
      return window.ionic.onGesture(eventType, cb, $element[0], options);
    },
    /**
     * @ngdoc method
     * @name $ionicGesture#off
     * @description Remove an event listener for a gesture on an element. See {@link ionic.utility:ionic.EventController#offGesture}.
     * @param {ionic.Gesture} gesture The gesture that should be removed.
     * @param {string} eventType The gesture event to remove the listener for.
     * @param {function(e)} callback The listener to remove.
     */
    off: function(gesture, eventType, cb) {
      return window.ionic.offGesture(gesture, eventType, cb);
    }
  };
}]);

/**
 * @ngdoc service
 * @name $ionicHistory
 * @module ionic
 * @description
 * $ionicHistory keeps track of views as the user navigates through an app. Similar to the way a
 * browser behaves, an Ionic app is able to keep track of the previous view, the current view, and
 * the forward view (if there is one).  However, a typical web browser only keeps track of one
 * history stack in a linear fashion.
 *
 * Unlike a traditional browser environment, apps and webapps have parallel independent histories,
 * such as with tabs. Should a user navigate few pages deep on one tab, and then switch to a new
 * tab and back, the back button relates not to the previous tab, but to the previous pages
 * visited within _that_ tab.
 *
 * `$ionicHistory` facilitates this parallel history architecture.
 */

IonicModule
.factory('$ionicHistory', [
  '$rootScope',
  '$state',
  '$location',
  '$window',
  '$timeout',
  '$ionicViewSwitcher',
  '$ionicNavViewDelegate',
function($rootScope, $state, $location, $window, $timeout, $ionicViewSwitcher, $ionicNavViewDelegate) {

  // history actions while navigating views
  var ACTION_INITIAL_VIEW = 'initialView';
  var ACTION_NEW_VIEW = 'newView';
  var ACTION_MOVE_BACK = 'moveBack';
  var ACTION_MOVE_FORWARD = 'moveForward';

  // direction of navigation
  var DIRECTION_BACK = 'back';
  var DIRECTION_FORWARD = 'forward';
  var DIRECTION_ENTER = 'enter';
  var DIRECTION_EXIT = 'exit';
  var DIRECTION_SWAP = 'swap';
  var DIRECTION_NONE = 'none';

  var stateChangeCounter = 0;
  var lastStateId, nextViewOptions, deregisterStateChangeListener, nextViewExpireTimer, forcedNav;

  var viewHistory = {
    histories: { root: { historyId: 'root', parentHistoryId: null, stack: [], cursor: -1 } },
    views: {},
    backView: null,
    forwardView: null,
    currentView: null
  };

  var View = function() {};
  View.prototype.initialize = function(data) {
    if (data) {
      for (var name in data) this[name] = data[name];
      return this;
    }
    return null;
  };
  View.prototype.go = function() {

    if (this.stateName) {
      return $state.go(this.stateName, this.stateParams);
    }

    if (this.url && this.url !== $location.url()) {

      if (viewHistory.backView === this) {
        return $window.history.go(-1);
      } else if (viewHistory.forwardView === this) {
        return $window.history.go(1);
      }

      $location.url(this.url);
    }

    return null;
  };
  View.prototype.destroy = function() {
    if (this.scope) {
      this.scope.$destroy && this.scope.$destroy();
      this.scope = null;
    }
  };


  function getViewById(viewId) {
    return (viewId ? viewHistory.views[ viewId ] : null);
  }

  function getBackView(view) {
    return (view ? getViewById(view.backViewId) : null);
  }

  function getForwardView(view) {
    return (view ? getViewById(view.forwardViewId) : null);
  }

  function getHistoryById(historyId) {
    return (historyId ? viewHistory.histories[ historyId ] : null);
  }

  function getHistory(scope) {
    var histObj = getParentHistoryObj(scope);

    if (!viewHistory.histories[ histObj.historyId ]) {
      // this history object exists in parent scope, but doesn't
      // exist in the history data yet
      viewHistory.histories[ histObj.historyId ] = {
        historyId: histObj.historyId,
        parentHistoryId: getParentHistoryObj(histObj.scope.$parent).historyId,
        stack: [],
        cursor: -1
      };
    }
    return getHistoryById(histObj.historyId);
  }

  function getParentHistoryObj(scope) {
    var parentScope = scope;
    while (parentScope) {
      if (parentScope.hasOwnProperty('$historyId')) {
        // this parent scope has a historyId
        return { historyId: parentScope.$historyId, scope: parentScope };
      }
      // nothing found keep climbing up
      parentScope = parentScope.$parent;
    }
    // no history for the parent, use the root
    return { historyId: 'root', scope: $rootScope };
  }

  function setNavViews(viewId) {
    viewHistory.currentView = getViewById(viewId);
    viewHistory.backView = getBackView(viewHistory.currentView);
    viewHistory.forwardView = getForwardView(viewHistory.currentView);
  }

  function getCurrentStateId() {
    var id;
    if ($state && $state.current && $state.current.name) {
      id = $state.current.name;
      if ($state.params) {
        for (var key in $state.params) {
          if ($state.params.hasOwnProperty(key) && $state.params[key]) {
            id += "_" + key + "=" + $state.params[key];
          }
        }
      }
      return id;
    }
    // if something goes wrong make sure its got a unique stateId
    return ionic.Utils.nextUid();
  }

  function getCurrentStateParams() {
    var rtn;
    if ($state && $state.params) {
      for (var key in $state.params) {
        if ($state.params.hasOwnProperty(key)) {
          rtn = rtn || {};
          rtn[key] = $state.params[key];
        }
      }
    }
    return rtn;
  }


  return {

    register: function(parentScope, viewLocals) {

      var currentStateId = getCurrentStateId(),
          hist = getHistory(parentScope),
          currentView = viewHistory.currentView,
          backView = viewHistory.backView,
          forwardView = viewHistory.forwardView,
          viewId = null,
          action = null,
          direction = DIRECTION_NONE,
          historyId = hist.historyId,
          url = $location.url(),
          tmp, x, ele;

      if (lastStateId !== currentStateId) {
        lastStateId = currentStateId;
        stateChangeCounter++;
      }

      if (forcedNav) {
        // we've previously set exactly what to do
        viewId = forcedNav.viewId;
        action = forcedNav.action;
        direction = forcedNav.direction;
        forcedNav = null;

      } else if (backView && backView.stateId === currentStateId) {
        // they went back one, set the old current view as a forward view
        viewId = backView.viewId;
        historyId = backView.historyId;
        action = ACTION_MOVE_BACK;
        if (backView.historyId === currentView.historyId) {
          // went back in the same history
          direction = DIRECTION_BACK;

        } else if (currentView) {
          direction = DIRECTION_EXIT;

          tmp = getHistoryById(backView.historyId);
          if (tmp && tmp.parentHistoryId === currentView.historyId) {
            direction = DIRECTION_ENTER;

          } else {
            tmp = getHistoryById(currentView.historyId);
            if (tmp && tmp.parentHistoryId === hist.parentHistoryId) {
              direction = DIRECTION_SWAP;
            }
          }
        }

      } else if (forwardView && forwardView.stateId === currentStateId) {
        // they went to the forward one, set the forward view to no longer a forward view
        viewId = forwardView.viewId;
        historyId = forwardView.historyId;
        action = ACTION_MOVE_FORWARD;
        if (forwardView.historyId === currentView.historyId) {
          direction = DIRECTION_FORWARD;

        } else if (currentView) {
          direction = DIRECTION_EXIT;

          if (currentView.historyId === hist.parentHistoryId) {
            direction = DIRECTION_ENTER;

          } else {
            tmp = getHistoryById(currentView.historyId);
            if (tmp && tmp.parentHistoryId === hist.parentHistoryId) {
              direction = DIRECTION_SWAP;
            }
          }
        }

        tmp = getParentHistoryObj(parentScope);
        if (forwardView.historyId && tmp.scope) {
          // if a history has already been created by the forward view then make sure it stays the same
          tmp.scope.$historyId = forwardView.historyId;
          historyId = forwardView.historyId;
        }

      } else if (currentView && currentView.historyId !== historyId &&
                hist.cursor > -1 && hist.stack.length > 0 && hist.cursor < hist.stack.length &&
                hist.stack[hist.cursor].stateId === currentStateId) {
        // they just changed to a different history and the history already has views in it
        var switchToView = hist.stack[hist.cursor];
        viewId = switchToView.viewId;
        historyId = switchToView.historyId;
        action = ACTION_MOVE_BACK;
        direction = DIRECTION_SWAP;

        tmp = getHistoryById(currentView.historyId);
        if (tmp && tmp.parentHistoryId === historyId) {
          direction = DIRECTION_EXIT;

        } else {
          tmp = getHistoryById(historyId);
          if (tmp && tmp.parentHistoryId === currentView.historyId) {
            direction = DIRECTION_ENTER;
          }
        }

        // if switching to a different history, and the history of the view we're switching
        // to has an existing back view from a different history than itself, then
        // it's back view would be better represented using the current view as its back view
        tmp = getViewById(switchToView.backViewId);
        if (tmp && switchToView.historyId !== tmp.historyId) {
          // the new view is being removed from it's old position in the history and being placed at the top,
          // so we need to update any views that reference it as a backview, otherwise there will be infinitely loops
          var viewIds = Object.keys(viewHistory.views);
          viewIds.forEach(function(viewId) {
            var view = viewHistory.views[viewId];
            if ((view.backViewId === switchToView.viewId) && (view.historyId !== switchToView.historyId)) {
              view.backViewId = null;
            }
          });

          hist.stack[hist.cursor].backViewId = currentView.viewId;
        }

      } else {

        // create an element from the viewLocals template
        ele = $ionicViewSwitcher.createViewEle(viewLocals);
        if (this.isAbstractEle(ele, viewLocals)) {
          return {
            action: 'abstractView',
            direction: DIRECTION_NONE,
            ele: ele
          };
        }

        // set a new unique viewId
        viewId = ionic.Utils.nextUid();

        if (currentView) {
          // set the forward view if there is a current view (ie: if its not the first view)
          currentView.forwardViewId = viewId;

          action = ACTION_NEW_VIEW;

          // check if there is a new forward view within the same history
          if (forwardView && currentView.stateId !== forwardView.stateId &&
             currentView.historyId === forwardView.historyId) {
            // they navigated to a new view but the stack already has a forward view
            // since its a new view remove any forwards that existed
            tmp = getHistoryById(forwardView.historyId);
            if (tmp) {
              // the forward has a history
              for (x = tmp.stack.length - 1; x >= forwardView.index; x--) {
                // starting from the end destroy all forwards in this history from this point
                var stackItem = tmp.stack[x];
                stackItem && stackItem.destroy && stackItem.destroy();
                tmp.stack.splice(x);
              }
              historyId = forwardView.historyId;
            }
          }

          // its only moving forward if its in the same history
          if (hist.historyId === currentView.historyId) {
            direction = DIRECTION_FORWARD;

          } else if (currentView.historyId !== hist.historyId) {
            // DB: this is a new view in a different tab
            direction = DIRECTION_ENTER;

            tmp = getHistoryById(currentView.historyId);
            if (tmp && tmp.parentHistoryId === hist.parentHistoryId) {
              direction = DIRECTION_SWAP;

            } else {
              tmp = getHistoryById(tmp.parentHistoryId);
              if (tmp && tmp.historyId === hist.historyId) {
                direction = DIRECTION_EXIT;
              }
            }
          }

        } else {
          // there's no current view, so this must be the initial view
          action = ACTION_INITIAL_VIEW;
        }

        if (stateChangeCounter < 2) {
          // views that were spun up on the first load should not animate
          direction = DIRECTION_NONE;
        }

        // add the new view
        viewHistory.views[viewId] = this.createView({
          viewId: viewId,
          index: hist.stack.length,
          historyId: hist.historyId,
          backViewId: (currentView && currentView.viewId ? currentView.viewId : null),
          forwardViewId: null,
          stateId: currentStateId,
          stateName: this.currentStateName(),
          stateParams: getCurrentStateParams(),
          url: url,
          canSwipeBack: canSwipeBack(ele, viewLocals)
        });

        // add the new view to this history's stack
        hist.stack.push(viewHistory.views[viewId]);
      }

      deregisterStateChangeListener && deregisterStateChangeListener();
      $timeout.cancel(nextViewExpireTimer);
      if (nextViewOptions) {
        if (nextViewOptions.disableAnimate) direction = DIRECTION_NONE;
        if (nextViewOptions.disableBack) viewHistory.views[viewId].backViewId = null;
        if (nextViewOptions.historyRoot) {
          for (x = 0; x < hist.stack.length; x++) {
            if (hist.stack[x].viewId === viewId) {
              hist.stack[x].index = 0;
              hist.stack[x].backViewId = hist.stack[x].forwardViewId = null;
            } else {
              delete viewHistory.views[hist.stack[x].viewId];
            }
          }
          hist.stack = [viewHistory.views[viewId]];
        }
        nextViewOptions = null;
      }

      setNavViews(viewId);

      if (viewHistory.backView && historyId == viewHistory.backView.historyId && currentStateId == viewHistory.backView.stateId && url == viewHistory.backView.url) {
        for (x = 0; x < hist.stack.length; x++) {
          if (hist.stack[x].viewId == viewId) {
            action = 'dupNav';
            direction = DIRECTION_NONE;
            if (x > 0) {
              hist.stack[x - 1].forwardViewId = null;
            }
            viewHistory.forwardView = null;
            viewHistory.currentView.index = viewHistory.backView.index;
            viewHistory.currentView.backViewId = viewHistory.backView.backViewId;
            viewHistory.backView = getBackView(viewHistory.backView);
            hist.stack.splice(x, 1);
            break;
          }
        }
      }

      hist.cursor = viewHistory.currentView.index;

      return {
        viewId: viewId,
        action: action,
        direction: direction,
        historyId: historyId,
        enableBack: this.enabledBack(viewHistory.currentView),
        isHistoryRoot: (viewHistory.currentView.index === 0),
        ele: ele
      };
    },

    registerHistory: function(scope) {
      scope.$historyId = ionic.Utils.nextUid();
    },

    createView: function(data) {
      var newView = new View();
      return newView.initialize(data);
    },

    getViewById: getViewById,

    /**
     * @ngdoc method
     * @name $ionicHistory#viewHistory
     * @description The app's view history data, such as all the views and histories, along
     * with how they are ordered and linked together within the navigation stack.
     * @returns {object} Returns an object containing the apps view history data.
     */
    viewHistory: function() {
      return viewHistory;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#currentView
     * @description The app's current view.
     * @returns {object} Returns the current view.
     */
    currentView: function(view) {
      if (arguments.length) {
        viewHistory.currentView = view;
      }
      return viewHistory.currentView;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#currentHistoryId
     * @description The ID of the history stack which is the parent container of the current view.
     * @returns {string} Returns the current history ID.
     */
    currentHistoryId: function() {
      return viewHistory.currentView ? viewHistory.currentView.historyId : null;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#currentTitle
     * @description Gets and sets the current view's title.
     * @param {string=} val The title to update the current view with.
     * @returns {string} Returns the current view's title.
     */
    currentTitle: function(val) {
      if (viewHistory.currentView) {
        if (arguments.length) {
          viewHistory.currentView.title = val;
        }
        return viewHistory.currentView.title;
      }
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#backView
     * @description Returns the view that was before the current view in the history stack.
     * If the user navigated from View A to View B, then View A would be the back view, and
     * View B would be the current view.
     * @returns {object} Returns the back view.
     */
    backView: function(view) {
      if (arguments.length) {
        viewHistory.backView = view;
      }
      return viewHistory.backView;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#backTitle
     * @description Gets the back view's title.
     * @returns {string} Returns the back view's title.
     */
    backTitle: function(view) {
      var backView = (view && getViewById(view.backViewId)) || viewHistory.backView;
      return backView && backView.title;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#forwardView
     * @description Returns the view that was in front of the current view in the history stack.
     * A forward view would exist if the user navigated from View A to View B, then
     * navigated back to View A. At this point then View B would be the forward view, and View
     * A would be the current view.
     * @returns {object} Returns the forward view.
     */
    forwardView: function(view) {
      if (arguments.length) {
        viewHistory.forwardView = view;
      }
      return viewHistory.forwardView;
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#currentStateName
     * @description Returns the current state name.
     * @returns {string}
     */
    currentStateName: function() {
      return ($state && $state.current ? $state.current.name : null);
    },

    isCurrentStateNavView: function(navView) {
      return !!($state && $state.current && $state.current.views && $state.current.views[navView]);
    },

    goToHistoryRoot: function(historyId) {
      if (historyId) {
        var hist = getHistoryById(historyId);
        if (hist && hist.stack.length) {
          if (viewHistory.currentView && viewHistory.currentView.viewId === hist.stack[0].viewId) {
            return;
          }
          forcedNav = {
            viewId: hist.stack[0].viewId,
            action: ACTION_MOVE_BACK,
            direction: DIRECTION_BACK
          };
          hist.stack[0].go();
        }
      }
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#goBack
     * @param {number=} backCount Optional negative integer setting how many views to go
     * back. By default it'll go back one view by using the value `-1`. To go back two
     * views you would use `-2`. If the number goes farther back than the number of views
     * in the current history's stack then it'll go to the first view in the current history's
     * stack. If the number is zero or greater then it'll do nothing. It also does not
     * cross history stacks, meaning it can only go as far back as the current history.
     * @description Navigates the app to the back view, if a back view exists.
     */
    goBack: function(backCount) {
      if (isDefined(backCount) && backCount !== -1) {
        if (backCount > -1) return;

        var currentHistory = viewHistory.histories[this.currentHistoryId()];
        var newCursor = currentHistory.cursor + backCount + 1;
        if (newCursor < 1) {
          newCursor = 1;
        }

        currentHistory.cursor = newCursor;
        setNavViews(currentHistory.stack[newCursor].viewId);

        var cursor = newCursor - 1;
        var clearStateIds = [];
        var fwdView = getViewById(currentHistory.stack[cursor].forwardViewId);
        while (fwdView) {
          clearStateIds.push(fwdView.stateId || fwdView.viewId);
          cursor++;
          if (cursor >= currentHistory.stack.length) break;
          fwdView = getViewById(currentHistory.stack[cursor].forwardViewId);
        }

        var self = this;
        if (clearStateIds.length) {
          $timeout(function() {
            self.clearCache(clearStateIds);
          }, 300);
        }
      }

      viewHistory.backView && viewHistory.backView.go();
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#removeBackView
     * @description Remove the previous view from the history completely, including the
     * cached element and scope (if they exist).
     */
    removeBackView: function() {
      var self = this;
      var currentHistory = viewHistory.histories[this.currentHistoryId()];
      var currentCursor = currentHistory.cursor;

      var currentView = currentHistory.stack[currentCursor];
      var backView = currentHistory.stack[currentCursor - 1];
      var replacementView = currentHistory.stack[currentCursor - 2];

      // fail if we dont have enough views in the history
      if (!backView || !replacementView) {
        return;
      }

      // remove the old backView and the cached element/scope
      currentHistory.stack.splice(currentCursor - 1, 1);
      self.clearCache([backView.viewId]);
      // make the replacementView and currentView point to each other (bypass the old backView)
      currentView.backViewId = replacementView.viewId;
      currentView.index = currentView.index - 1;
      replacementView.forwardViewId = currentView.viewId;
      // update the cursor and set new backView
      viewHistory.backView = replacementView;
      currentHistory.currentCursor += -1;
    },

    enabledBack: function(view) {
      var backView = getBackView(view);
      return !!(backView && backView.historyId === view.historyId);
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#clearHistory
     * @description Clears out the app's entire history, except for the current view.
     */
    clearHistory: function() {
      var
      histories = viewHistory.histories,
      currentView = viewHistory.currentView;

      if (histories) {
        for (var historyId in histories) {

          if (histories[historyId].stack) {
            histories[historyId].stack = [];
            histories[historyId].cursor = -1;
          }

          if (currentView && currentView.historyId === historyId) {
            currentView.backViewId = currentView.forwardViewId = null;
            histories[historyId].stack.push(currentView);
          } else if (histories[historyId].destroy) {
            histories[historyId].destroy();
          }

        }
      }

      for (var viewId in viewHistory.views) {
        if (viewId !== currentView.viewId) {
          delete viewHistory.views[viewId];
        }
      }

      if (currentView) {
        setNavViews(currentView.viewId);
      }
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#clearCache
	 * @return promise
     * @description Removes all cached views within every {@link ionic.directive:ionNavView}.
     * This both removes the view element from the DOM, and destroy it's scope.
     */
    clearCache: function(stateIds) {
      return $timeout(function() {
        $ionicNavViewDelegate._instances.forEach(function(instance) {
          instance.clearCache(stateIds);
        });
      });
    },

    /**
     * @ngdoc method
     * @name $ionicHistory#nextViewOptions
     * @description Sets options for the next view. This method can be useful to override
     * certain view/transition defaults right before a view transition happens. For example,
     * the {@link ionic.directive:menuClose} directive uses this method internally to ensure
     * an animated view transition does not happen when a side menu is open, and also sets
     * the next view as the root of its history stack. After the transition these options
     * are set back to null.
     *
     * Available options:
     *
     * * `disableAnimate`: Do not animate the next transition.
     * * `disableBack`: The next view should forget its back view, and set it to null.
     * * `historyRoot`: The next view should become the root view in its history stack.
     *
     * ```js
     * $ionicHistory.nextViewOptions({
     *   disableAnimate: true,
     *   disableBack: true
     * });
     * ```
     */
    nextViewOptions: function(opts) {
      deregisterStateChangeListener && deregisterStateChangeListener();
      if (arguments.length) {
        $timeout.cancel(nextViewExpireTimer);
        if (opts === null) {
          nextViewOptions = opts;
        } else {
          nextViewOptions = nextViewOptions || {};
          extend(nextViewOptions, opts);
          if (nextViewOptions.expire) {
              deregisterStateChangeListener = $rootScope.$on('$stateChangeSuccess', function() {
                nextViewExpireTimer = $timeout(function() {
                  nextViewOptions = null;
                  }, nextViewOptions.expire);
              });
          }
        }
      }
      return nextViewOptions;
    },

    isAbstractEle: function(ele, viewLocals) {
      if (viewLocals && viewLocals.$$state && viewLocals.$$state.self['abstract']) {
        return true;
      }
      return !!(ele && (isAbstractTag(ele) || isAbstractTag(ele.children())));
    },

    isActiveScope: function(scope) {
      if (!scope) return false;

      var climbScope = scope;
      var currentHistoryId = this.currentHistoryId();
      var foundHistoryId;

      while (climbScope) {
        if (climbScope.$$disconnected) {
          return false;
        }

        if (!foundHistoryId && climbScope.hasOwnProperty('$historyId')) {
          foundHistoryId = true;
        }

        if (currentHistoryId) {
          if (climbScope.hasOwnProperty('$historyId') && currentHistoryId == climbScope.$historyId) {
            return true;
          }
          if (climbScope.hasOwnProperty('$activeHistoryId')) {
            if (currentHistoryId == climbScope.$activeHistoryId) {
              if (climbScope.hasOwnProperty('$historyId')) {
                return true;
              }
              if (!foundHistoryId) {
                return true;
              }
            }
          }
        }

        if (foundHistoryId && climbScope.hasOwnProperty('$activeHistoryId')) {
          foundHistoryId = false;
        }

        climbScope = climbScope.$parent;
      }

      return currentHistoryId ? currentHistoryId == 'root' : true;
    }

  };

  function isAbstractTag(ele) {
    return ele && ele.length && /ion-side-menus|ion-tabs/i.test(ele[0].tagName);
  }

  function canSwipeBack(ele, viewLocals) {
    if (viewLocals && viewLocals.$$state && viewLocals.$$state.self.canSwipeBack === false) {
      return false;
    }
    if (ele && ele.attr('can-swipe-back') === 'false') {
      return false;
    }
    var eleChild = ele.find('ion-view');
    if (eleChild && eleChild.attr('can-swipe-back') === 'false') {
      return false;
    }
    return true;
  }

}])

.run([
  '$rootScope',
  '$state',
  '$location',
  '$document',
  '$ionicPlatform',
  '$ionicHistory',
  'IONIC_BACK_PRIORITY',
function($rootScope, $state, $location, $document, $ionicPlatform, $ionicHistory, IONIC_BACK_PRIORITY) {

  // always reset the keyboard state when change stage
  $rootScope.$on('$ionicView.beforeEnter', function() {
    ionic.keyboard && ionic.keyboard.hide && ionic.keyboard.hide();
  });

  $rootScope.$on('$ionicHistory.change', function(e, data) {
    if (!data) return null;

    var viewHistory = $ionicHistory.viewHistory();

    var hist = (data.historyId ? viewHistory.histories[ data.historyId ] : null);
    if (hist && hist.cursor > -1 && hist.cursor < hist.stack.length) {
      // the history they're going to already exists
      // go to it's last view in its stack
      var view = hist.stack[ hist.cursor ];
      return view.go(data);
    }

    // this history does not have a URL, but it does have a uiSref
    // figure out its URL from the uiSref
    if (!data.url && data.uiSref) {
      data.url = $state.href(data.uiSref);
    }

    if (data.url) {
      // don't let it start with a #, messes with $location.url()
      if (data.url.indexOf('#') === 0) {
        data.url = data.url.replace('#', '');
      }
      if (data.url !== $location.url()) {
        // we've got a good URL, ready GO!
        $location.url(data.url);
      }
    }
  });

  $rootScope.$ionicGoBack = function(backCount) {
    $ionicHistory.goBack(backCount);
  };

  // Set the document title when a new view is shown
  $rootScope.$on('$ionicView.afterEnter', function(ev, data) {
    if (data && data.title) {
      $document[0].title = data.title;
    }
  });

  // Triggered when devices with a hardware back button (Android) is clicked by the user
  // This is a Cordova/Phonegap platform specifc method
  function onHardwareBackButton(e) {
    var backView = $ionicHistory.backView();
    if (backView) {
      // there is a back view, go to it
      backView.go();
    } else {
      // there is no back view, so close the app instead
      ionic.Platform.exitApp();
    }
    e.preventDefault();
    return false;
  }
  $ionicPlatform.registerBackButtonAction(
    onHardwareBackButton,
    IONIC_BACK_PRIORITY.view
  );

}]);

/**
 * @ngdoc provider
 * @name $ionicConfigProvider
 * @module ionic
 * @description
 * Ionic automatically takes platform configurations into account to adjust things like what
 * transition style to use and whether tab icons should show on the top or bottom. For example,
 * iOS will move forward by transitioning the entering view from right to center and the leaving
 * view from center to left. However, Android will transition with the entering view going from
 * bottom to center, covering the previous view, which remains stationary. It should be noted
 * that when a platform is not iOS or Android, then it'll default to iOS. So if you are
 * developing on a desktop browser, it's going to take on iOS default configs.
 *
 * These configs can be changed using the `$ionicConfigProvider` during the configuration phase
 * of your app. Additionally, `$ionicConfig` can also set and get config values during the run
 * phase and within the app itself.
 *
 * By default, all base config variables are set to `'platform'`, which means it'll take on the
 * default config of the platform on which it's running. Config variables can be set at this
 * level so all platforms follow the same setting, rather than its platform config.
 * The following code would set the same config variable for all platforms:
 *
 * ```js
 * $ionicConfigProvider.views.maxCache(10);
 * ```
 *
 * Additionally, each platform can have its own config within the `$ionicConfigProvider.platform`
 * property. The config below would only apply to Android devices.
 *
 * ```js
 * $ionicConfigProvider.platform.android.views.maxCache(5);
 * ```
 *
 * @usage
 * ```js
 * var myApp = angular.module('reallyCoolApp', ['ionic']);
 *
 * myApp.config(function($ionicConfigProvider) {
 *   $ionicConfigProvider.views.maxCache(5);
 *
 *   // note that you can also chain configs
 *   $ionicConfigProvider.backButton.text('Go Back').icon('ion-chevron-left');
 * });
 * ```
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#views.transition
 * @description Animation style when transitioning between views. Default `platform`.
 *
 * @param {string} transition Which style of view transitioning to use.
 *
 * * `platform`: Dynamically choose the correct transition style depending on the platform
 * the app is running from. If the platform is not `ios` or `android` then it will default
 * to `ios`.
 * * `ios`: iOS style transition.
 * * `android`: Android style transition.
 * * `none`: Do not perform animated transitions.
 *
 * @returns {string} value
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#views.maxCache
 * @description  Maximum number of view elements to cache in the DOM. When the max number is
 * exceeded, the view with the longest time period since it was accessed is removed. Views that
 * stay in the DOM cache the view's scope, current state, and scroll position. The scope is
 * disconnected from the `$watch` cycle when it is cached and reconnected when it enters again.
 * When the maximum cache is `0`, the leaving view's element will be removed from the DOM after
 * each view transition, and the next time the same view is shown, it will have to re-compile,
 * attach to the DOM, and link the element again. This disables caching, in effect.
 * @param {number} maxNumber Maximum number of views to retain. Default `10`.
 * @returns {number} How many views Ionic will hold onto until the a view is removed.
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#views.forwardCache
 * @description  By default, when navigating, views that were recently visited are cached, and
 * the same instance data and DOM elements are referenced when navigating back. However, when
 * navigating back in the history, the "forward" views are removed from the cache. If you
 * navigate forward to the same view again, it'll create a new DOM element and controller
 * instance. Basically, any forward views are reset each time. Set this config to `true` to have
 * forward views cached and not reset on each load.
 * @param {boolean} value
 * @returns {boolean}
 */

 /**
  * @ngdoc method
  * @name $ionicConfigProvider#views.swipeBackEnabled
  * @description  By default on iOS devices, swipe to go back functionality is enabled by default.
  * This method can be used to disable it globally, or on a per-view basis.
  * Note: This functionality is only supported on iOS.
  * @param {boolean} value
  * @returns {boolean}
  */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#scrolling.jsScrolling
 * @description  Whether to use JS or Native scrolling. Defaults to native scrolling. Setting this to
 * `true` has the same effect as setting each `ion-content` to have `overflow-scroll='false'`.
 * @param {boolean} value Defaults to `false` as of Ionic 1.2
 * @returns {boolean}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#backButton.icon
 * @description Back button icon.
 * @param {string} value
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#backButton.text
 * @description Back button text.
 * @param {string} value Defaults to `Back`.
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#backButton.previousTitleText
 * @description If the previous title text should become the back button text. This
 * is the default for iOS.
 * @param {boolean} value
 * @returns {boolean}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#form.checkbox
 * @description Checkbox style. Android defaults to `square` and iOS defaults to `circle`.
 * @param {string} value
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#form.toggle
 * @description Toggle item style. Android defaults to `small` and iOS defaults to `large`.
 * @param {string} value
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#spinner.icon
 * @description Default spinner icon to use.
 * @param {string} value Can be: `android`, `ios`, `ios-small`, `bubbles`, `circles`, `crescent`,
 * `dots`, `lines`, `ripple`, or `spiral`.
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#tabs.style
 * @description Tab style. Android defaults to `striped` and iOS defaults to `standard`.
 * @param {string} value Available values include `striped` and `standard`.
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#tabs.position
 * @description Tab position. Android defaults to `top` and iOS defaults to `bottom`.
 * @param {string} value Available values include `top` and `bottom`.
 * @returns {string}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#templates.maxPrefetch
 * @description Sets the maximum number of templates to prefetch from the templateUrls defined in
 * $stateProvider.state. If set to `0`, the user will have to wait
 * for a template to be fetched the first time when navigating to a new page. Default `30`.
 * @param {integer} value Max number of template to prefetch from the templateUrls defined in
 * `$stateProvider.state()`.
 * @returns {integer}
 */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#navBar.alignTitle
 * @description Which side of the navBar to align the title. Default `center`.
 *
 * @param {string} value side of the navBar to align the title.
 *
 * * `platform`: Dynamically choose the correct title style depending on the platform
 * the app is running from. If the platform is `ios`, it will default to `center`.
 * If the platform is `android`, it will default to `left`. If the platform is not
 * `ios` or `android`, it will default to `center`.
 *
 * * `left`: Left align the title in the navBar
 * * `center`: Center align the title in the navBar
 * * `right`: Right align the title in the navBar.
 *
 * @returns {string} value
 */

/**
  * @ngdoc method
  * @name $ionicConfigProvider#navBar.positionPrimaryButtons
  * @description Which side of the navBar to align the primary navBar buttons. Default `left`.
  *
  * @param {string} value side of the navBar to align the primary navBar buttons.
  *
  * * `platform`: Dynamically choose the correct title style depending on the platform
  * the app is running from. If the platform is `ios`, it will default to `left`.
  * If the platform is `android`, it will default to `right`. If the platform is not
  * `ios` or `android`, it will default to `left`.
  *
  * * `left`: Left align the primary navBar buttons in the navBar
  * * `right`: Right align the primary navBar buttons in the navBar.
  *
  * @returns {string} value
  */

/**
 * @ngdoc method
 * @name $ionicConfigProvider#navBar.positionSecondaryButtons
 * @description Which side of the navBar to align the secondary navBar buttons. Default `right`.
 *
 * @param {string} value side of the navBar to align the secondary navBar buttons.
 *
 * * `platform`: Dynamically choose the correct title style depending on the platform
 * the app is running from. If the platform is `ios`, it will default to `right`.
 * If the platform is `android`, it will default to `right`. If the platform is not
 * `ios` or `android`, it will default to `right`.
 *
 * * `left`: Left align the secondary navBar buttons in the navBar
 * * `right`: Right align the secondary navBar buttons in the navBar.
 *
 * @returns {string} value
 */

IonicModule
.provider('$ionicConfig', function() {

  var provider = this;
  provider.platform = {};
  var PLATFORM = 'platform';

  var configProperties = {
    views: {
      maxCache: PLATFORM,
      forwardCache: PLATFORM,
      transition: PLATFORM,
      swipeBackEnabled: PLATFORM,
      swipeBackHitWidth: PLATFORM
    },
    navBar: {
      alignTitle: PLATFORM,
      positionPrimaryButtons: PLATFORM,
      positionSecondaryButtons: PLATFORM,
      transition: PLATFORM
    },
    backButton: {
      icon: PLATFORM,
      text: PLATFORM,
      previousTitleText: PLATFORM
    },
    form: {
      checkbox: PLATFORM,
      toggle: PLATFORM
    },
    scrolling: {
      jsScrolling: PLATFORM
    },
    spinner: {
      icon: PLATFORM
    },
    tabs: {
      style: PLATFORM,
      position: PLATFORM
    },
    templates: {
      maxPrefetch: PLATFORM
    },
    platform: {}
  };
  createConfig(configProperties, provider, '');



  // Default
  // -------------------------
  setPlatformConfig('default', {

    views: {
      maxCache: 10,
      forwardCache: false,
      transition: 'ios',
      swipeBackEnabled: true,
      swipeBackHitWidth: 45
    },

    navBar: {
      alignTitle: 'center',
      positionPrimaryButtons: 'left',
      positionSecondaryButtons: 'right',
      transition: 'view'
    },

    backButton: {
      icon: 'ion-ios-arrow-back',
      text: 'Back',
      previousTitleText: true
    },

    form: {
      checkbox: 'circle',
      toggle: 'large'
    },

    scrolling: {
      jsScrolling: true
    },

    spinner: {
      icon: 'ios'
    },

    tabs: {
      style: 'standard',
      position: 'bottom'
    },

    templates: {
      maxPrefetch: 30
    }

  });



  // iOS (it is the default already)
  // -------------------------
  setPlatformConfig('ios', {});



  // Android
  // -------------------------
  setPlatformConfig('android', {

    views: {
      transition: 'android',
      swipeBackEnabled: false
    },

    navBar: {
      alignTitle: 'left',
      positionPrimaryButtons: 'right',
      positionSecondaryButtons: 'right'
    },

    backButton: {
      icon: 'ion-android-arrow-back',
      text: false,
      previousTitleText: false
    },

    form: {
      checkbox: 'square',
      toggle: 'small'
    },

    spinner: {
      icon: 'android'
    },

    tabs: {
      style: 'striped',
      position: 'top'
    },

    scrolling: {
      jsScrolling: false
    }
  });

  // Windows Phone
  // -------------------------
  setPlatformConfig('windowsphone', {
    //scrolling: {
    //  jsScrolling: false
    //}
    spinner: {
      icon: 'android'
    }
  });


  provider.transitions = {
    views: {},
    navBar: {}
  };


  // iOS Transitions
  // -----------------------
  provider.transitions.views.ios = function(enteringEle, leavingEle, direction, shouldAnimate) {

    function setStyles(ele, opacity, x, boxShadowOpacity) {
      var css = {};
      css[ionic.CSS.TRANSITION_DURATION] = d.shouldAnimate ? '' : 0;
      css.opacity = opacity;
      if (boxShadowOpacity > -1) {
        css.boxShadow = '0 0 10px rgba(0,0,0,' + (d.shouldAnimate ? boxShadowOpacity * 0.45 : 0.3) + ')';
      }
      css[ionic.CSS.TRANSFORM] = 'translate3d(' + x + '%,0,0)';
      ionic.DomUtil.cachedStyles(ele, css);
    }

    var d = {
      run: function(step) {
        if (direction == 'forward') {
          setStyles(enteringEle, 1, (1 - step) * 99, 1 - step); // starting at 98% prevents a flicker
          setStyles(leavingEle, (1 - 0.1 * step), step * -33, -1);

        } else if (direction == 'back') {
          setStyles(enteringEle, (1 - 0.1 * (1 - step)), (1 - step) * -33, -1);
          setStyles(leavingEle, 1, step * 100, 1 - step);

        } else {
          // swap, enter, exit
          setStyles(enteringEle, 1, 0, -1);
          setStyles(leavingEle, 0, 0, -1);
        }
      },
      shouldAnimate: shouldAnimate && (direction == 'forward' || direction == 'back')
    };

    return d;
  };

  provider.transitions.navBar.ios = function(enteringHeaderBar, leavingHeaderBar, direction, shouldAnimate) {

    function setStyles(ctrl, opacity, titleX, backTextX) {
      var css = {};
      css[ionic.CSS.TRANSITION_DURATION] = d.shouldAnimate ? '' : '0ms';
      css.opacity = opacity === 1 ? '' : opacity;

      ctrl.setCss('buttons-left', css);
      ctrl.setCss('buttons-right', css);
      ctrl.setCss('back-button', css);

      css[ionic.CSS.TRANSFORM] = 'translate3d(' + backTextX + 'px,0,0)';
      ctrl.setCss('back-text', css);

      css[ionic.CSS.TRANSFORM] = 'translate3d(' + titleX + 'px,0,0)';
      ctrl.setCss('title', css);
    }

    function enter(ctrlA, ctrlB, step) {
      if (!ctrlA || !ctrlB) return;
      var titleX = (ctrlA.titleTextX() + ctrlA.titleWidth()) * (1 - step);
      var backTextX = (ctrlB && (ctrlB.titleTextX() - ctrlA.backButtonTextLeft()) * (1 - step)) || 0;
      setStyles(ctrlA, step, titleX, backTextX);
    }

    function leave(ctrlA, ctrlB, step) {
      if (!ctrlA || !ctrlB) return;
      var titleX = (-(ctrlA.titleTextX() - ctrlB.backButtonTextLeft()) - (ctrlA.titleLeftRight())) * step;
      setStyles(ctrlA, 1 - step, titleX, 0);
    }

    var d = {
      run: function(step) {
        var enteringHeaderCtrl = enteringHeaderBar.controller();
        var leavingHeaderCtrl = leavingHeaderBar && leavingHeaderBar.controller();
        if (d.direction == 'back') {
          leave(enteringHeaderCtrl, leavingHeaderCtrl, 1 - step);
          enter(leavingHeaderCtrl, enteringHeaderCtrl, 1 - step);
        } else {
          enter(enteringHeaderCtrl, leavingHeaderCtrl, step);
          leave(leavingHeaderCtrl, enteringHeaderCtrl, step);
        }
      },
      direction: direction,
      shouldAnimate: shouldAnimate && (direction == 'forward' || direction == 'back')
    };

    return d;
  };


  // Android Transitions
  // -----------------------

  provider.transitions.views.android = function(enteringEle, leavingEle, direction, shouldAnimate) {
    shouldAnimate = shouldAnimate && (direction == 'forward' || direction == 'back');

    function setStyles(ele, x, opacity) {
      var css = {};
      css[ionic.CSS.TRANSITION_DURATION] = d.shouldAnimate ? '' : 0;
      css[ionic.CSS.TRANSFORM] = 'translate3d(' + x + '%,0,0)';
      css.opacity = opacity;
      ionic.DomUtil.cachedStyles(ele, css);
    }

    var d = {
      run: function(step) {
        if (direction == 'forward') {
          setStyles(enteringEle, (1 - step) * 99, 1); // starting at 98% prevents a flicker
          setStyles(leavingEle, step * -100, 1);

        } else if (direction == 'back') {
          setStyles(enteringEle, (1 - step) * -100, 1);
          setStyles(leavingEle, step * 100, 1);

        } else {
          // swap, enter, exit
          setStyles(enteringEle, 0, 1);
          setStyles(leavingEle, 0, 0);
        }
      },
      shouldAnimate: shouldAnimate
    };

    return d;
  };

  provider.transitions.navBar.android = function(enteringHeaderBar, leavingHeaderBar, direction, shouldAnimate) {

    function setStyles(ctrl, opacity) {
      if (!ctrl) return;
      var css = {};
      css.opacity = opacity === 1 ? '' : opacity;

      ctrl.setCss('buttons-left', css);
      ctrl.setCss('buttons-right', css);
      ctrl.setCss('back-button', css);
      ctrl.setCss('back-text', css);
      ctrl.setCss('title', css);
    }

    return {
      run: function(step) {
        setStyles(enteringHeaderBar.controller(), step);
        setStyles(leavingHeaderBar && leavingHeaderBar.controller(), 1 - step);
      },
      shouldAnimate: shouldAnimate && (direction == 'forward' || direction == 'back')
    };
  };


  // No Transition
  // -----------------------

  provider.transitions.views.none = function(enteringEle, leavingEle) {
    return {
      run: function(step) {
        provider.transitions.views.android(enteringEle, leavingEle, false, false).run(step);
      },
      shouldAnimate: false
    };
  };

  provider.transitions.navBar.none = function(enteringHeaderBar, leavingHeaderBar) {
    return {
      run: function(step) {
        provider.transitions.navBar.ios(enteringHeaderBar, leavingHeaderBar, false, false).run(step);
        provider.transitions.navBar.android(enteringHeaderBar, leavingHeaderBar, false, false).run(step);
      },
      shouldAnimate: false
    };
  };


  // private: used to set platform configs
  function setPlatformConfig(platformName, platformConfigs) {
    configProperties.platform[platformName] = platformConfigs;
    provider.platform[platformName] = {};

    addConfig(configProperties, configProperties.platform[platformName]);

    createConfig(configProperties.platform[platformName], provider.platform[platformName], '');
  }


  // private: used to recursively add new platform configs
  function addConfig(configObj, platformObj) {
    for (var n in configObj) {
      if (n != PLATFORM && configObj.hasOwnProperty(n)) {
        if (angular.isObject(configObj[n])) {
          if (!isDefined(platformObj[n])) {
            platformObj[n] = {};
          }
          addConfig(configObj[n], platformObj[n]);

        } else if (!isDefined(platformObj[n])) {
          platformObj[n] = null;
        }
      }
    }
  }


  // private: create methods for each config to get/set
  function createConfig(configObj, providerObj, platformPath) {
    forEach(configObj, function(value, namespace) {

      if (angular.isObject(configObj[namespace])) {
        // recursively drill down the config object so we can create a method for each one
        providerObj[namespace] = {};
        createConfig(configObj[namespace], providerObj[namespace], platformPath + '.' + namespace);

      } else {
        // create a method for the provider/config methods that will be exposed
        providerObj[namespace] = function(newValue) {
          if (arguments.length) {
            configObj[namespace] = newValue;
            return providerObj;
          }
          if (configObj[namespace] == PLATFORM) {
            // if the config is set to 'platform', then get this config's platform value
            var platformConfig = stringObj(configProperties.platform, ionic.Platform.platform() + platformPath + '.' + namespace);
            if (platformConfig || platformConfig === false) {
              return platformConfig;
            }
            // didnt find a specific platform config, now try the default
            return stringObj(configProperties.platform, 'default' + platformPath + '.' + namespace);
          }
          return configObj[namespace];
        };
      }

    });
  }

  function stringObj(obj, str) {
    str = str.split(".");
    for (var i = 0; i < str.length; i++) {
      if (obj && isDefined(obj[str[i]])) {
        obj = obj[str[i]];
      } else {
        return null;
      }
    }
    return obj;
  }

  provider.setPlatformConfig = setPlatformConfig;


  // private: Service definition for internal Ionic use
  /**
   * @ngdoc service
   * @name $ionicConfig
   * @module ionic
   * @private
   */
  provider.$get = function() {
    return provider;
  };
})
// Fix for URLs in Cordova apps on Windows Phone
// http://blogs.msdn.com/b/msdn_answers/archive/2015/02/10/
// running-cordova-apps-on-windows-and-windows-phone-8-1-using-ionic-angularjs-and-other-frameworks.aspx
.config(['$compileProvider', function($compileProvider) {
  $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|sms|tel|geo|ftp|mailto|file|ghttps?|ms-appx-web|ms-appx|x-wmapp0):/);
  $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|file|content|blob|ms-appx|ms-appx-web|x-wmapp0):|data:image\//);
}]);


var LOADING_TPL =
  '<div class="loading-container">' +
    '<div class="loading">' +
    '</div>' +
  '</div>';

/**
 * @ngdoc service
 * @name $ionicLoading
 * @module ionic
 * @description
 * An overlay that can be used to indicate activity while blocking user
 * interaction.
 *
 * @usage
 * ```js
 * angular.module('LoadingApp', ['ionic'])
 * .controller('LoadingCtrl', function($scope, $ionicLoading) {
 *   $scope.show = function() {
 *     $ionicLoading.show({
 *       template: 'Loading...',
 *       duration: 3000
 *     }).then(function(){
 *        console.log("The loading indicator is now displayed");
 *     });
 *   };
 *   $scope.hide = function(){
 *     $ionicLoading.hide().then(function(){
 *        console.log("The loading indicator is now hidden");
 *     });
 *   };
 * });
 * ```
 */
/**
 * @ngdoc object
 * @name $ionicLoadingConfig
 * @module ionic
 * @description
 * Set the default options to be passed to the {@link ionic.service:$ionicLoading} service.
 *
 * @usage
 * ```js
 * var app = angular.module('myApp', ['ionic'])
 * app.constant('$ionicLoadingConfig', {
 *   template: 'Default Loading Template...'
 * });
 * app.controller('AppCtrl', function($scope, $ionicLoading) {
 *   $scope.showLoading = function() {
 *     //options default to values in $ionicLoadingConfig
 *     $ionicLoading.show().then(function(){
 *        console.log("The loading indicator is now displayed");
 *     });
 *   };
 * });
 * ```
 */
IonicModule
.constant('$ionicLoadingConfig', {
  template: '<ion-spinner></ion-spinner>'
})
.factory('$ionicLoading', [
  '$ionicLoadingConfig',
  '$ionicBody',
  '$ionicTemplateLoader',
  '$ionicBackdrop',
  '$timeout',
  '$q',
  '$log',
  '$compile',
  '$ionicPlatform',
  '$rootScope',
  'IONIC_BACK_PRIORITY',
function($ionicLoadingConfig, $ionicBody, $ionicTemplateLoader, $ionicBackdrop, $timeout, $q, $log, $compile, $ionicPlatform, $rootScope, IONIC_BACK_PRIORITY) {

  var loaderInstance;
  //default values
  var deregisterBackAction = noop;
  var deregisterStateListener1 = noop;
  var deregisterStateListener2 = noop;
  var loadingShowDelay = $q.when();

  return {
    /**
     * @ngdoc method
     * @name $ionicLoading#show
     * @description Shows a loading indicator. If the indicator is already shown,
     * it will set the options given and keep the indicator shown.
     * @returns {promise} A promise which is resolved when the loading indicator is presented.
     * @param {object} opts The options for the loading indicator. Available properties:
     *  - `{string=}` `template` The html content of the indicator.
     *  - `{string=}` `templateUrl` The url of an html template to load as the content of the indicator.
     *  - `{object=}` `scope` The scope to be a child of. Default: creates a child of $rootScope.
     *  - `{boolean=}` `noBackdrop` Whether to hide the backdrop. By default it will be shown.
     *  - `{boolean=}` `hideOnStateChange` Whether to hide the loading spinner when navigating
     *    to a new state. Default false.
     *  - `{number=}` `delay` How many milliseconds to delay showing the indicator. By default there is no delay.
     *  - `{number=}` `duration` How many milliseconds to wait until automatically
     *  hiding the indicator. By default, the indicator will be shown until `.hide()` is called.
     */
    show: showLoader,
    /**
     * @ngdoc method
     * @name $ionicLoading#hide
     * @description Hides the loading indicator, if shown.
     * @returns {promise} A promise which is resolved when the loading indicator is hidden.
     */
    hide: hideLoader,
    /**
     * @private for testing
     */
    _getLoader: getLoader
  };

  function getLoader() {
    if (!loaderInstance) {
      loaderInstance = $ionicTemplateLoader.compile({
        template: LOADING_TPL,
        appendTo: $ionicBody.get()
      })
      .then(function(self) {
        self.show = function(options) {
          var templatePromise = options.templateUrl ?
            $ionicTemplateLoader.load(options.templateUrl) :
            //options.content: deprecated
            $q.when(options.template || options.content || '');

          self.scope = options.scope || self.scope;

          if (!self.isShown) {
            //options.showBackdrop: deprecated
            self.hasBackdrop = !options.noBackdrop && options.showBackdrop !== false;
            if (self.hasBackdrop) {
              $ionicBackdrop.retain();
              $ionicBackdrop.getElement().addClass('backdrop-loading');
            }
          }

          if (options.duration) {
            $timeout.cancel(self.durationTimeout);
            self.durationTimeout = $timeout(
              angular.bind(self, self.hide),
              +options.duration
            );
          }

          deregisterBackAction();
          //Disable hardware back button while loading
          deregisterBackAction = $ionicPlatform.registerBackButtonAction(
            noop,
            IONIC_BACK_PRIORITY.loading
          );

          templatePromise.then(function(html) {
            if (html) {
              var loading = self.element.children();
              loading.html(html);
              $compile(loading.contents())(self.scope);
            }

            //Don't show until template changes
            if (self.isShown) {
              self.element.addClass('visible');
              ionic.requestAnimationFrame(function() {
                if (self.isShown) {
                  self.element.addClass('active');
                  $ionicBody.addClass('loading-active');
                }
              });
            }
          });

          self.isShown = true;
        };
        self.hide = function() {

          deregisterBackAction();
          if (self.isShown) {
            if (self.hasBackdrop) {
              $ionicBackdrop.release();
              $ionicBackdrop.getElement().removeClass('backdrop-loading');
            }
            self.element.removeClass('active');
            $ionicBody.removeClass('loading-active');
            self.element.removeClass('visible');
            ionic.requestAnimationFrame(function() {
              !self.isShown && self.element.removeClass('visible');
            });
          }
          $timeout.cancel(self.durationTimeout);
          self.isShown = false;
          var loading = self.element.children();
          loading.html("");
        };

        return self;
      });
    }
    return loaderInstance;
  }

  function showLoader(options) {
    options = extend({}, $ionicLoadingConfig || {}, options || {});
    // use a default delay of 100 to avoid some issues reported on github
    // https://github.com/ionic-team/ionic/issues/3717
    var delay = options.delay || options.showDelay || 0;

    deregisterStateListener1();
    deregisterStateListener2();
    if (options.hideOnStateChange) {
      deregisterStateListener1 = $rootScope.$on('$stateChangeSuccess', hideLoader);
      deregisterStateListener2 = $rootScope.$on('$stateChangeError', hideLoader);
    }

    //If loading.show() was called previously, cancel it and show with our new options
    //$timeout.cancel(loadingShowDelay);
    // YB 
    if (loadingShowDelay && loadingShowDelay.hasOwnProperty('$$timeoutId')) {
        $timeout.cancel(loadingShowDelay);
    }

    loadingShowDelay = $timeout(noop, delay);
    return loadingShowDelay.then(getLoader).then(function(loader) {
      return loader.show(options);
    });
  }

  function hideLoader() {
    deregisterStateListener1();
    deregisterStateListener2();

    //$timeout.cancel(loadingShowDelay);
    // YB 
    if (loadingShowDelay && loadingShowDelay.hasOwnProperty('$$timeoutId')) {
        $timeout.cancel(loadingShowDelay);
    }

    return getLoader().then(function(loader) {
      return loader.hide();
    });
  }
}]);

/**
 * @ngdoc service
 * @name $ionicModal
 * @module ionic
 * @codepen gblny
 * @description
 *
 * Related: {@link ionic.controller:ionicModal ionicModal controller}.
 *
 * The Modal is a content pane that can go over the user's main view
 * temporarily.  Usually used for making a choice or editing an item.
 *
 * Put the content of the modal inside of an `<ion-modal-view>` element.
 *
 * **Notes:**
 * - A modal will broadcast 'modal.shown', 'modal.hidden', and 'modal.removed' events from its originating
 * scope, passing in itself as an event argument. Both the modal.removed and modal.hidden events are
 * called when the modal is removed.
 *
 * - This example assumes your modal is in your main index file or another template file. If it is in its own
 * template file, remove the script tags and call it by file name.
 *
 * @usage
 * ```html
 * <script id="my-modal.html" type="text/ng-template">
 *   <ion-modal-view>
 *     <ion-header-bar>
 *       <h1 class="title">My Modal title</h1>
 *     </ion-header-bar>
 *     <ion-content>
 *       Hello!
 *     </ion-content>
 *   </ion-modal-view>
 * </script>
 * ```
 * ```js
 * angular.module('testApp', ['ionic'])
 * .controller('MyController', function($scope, $ionicModal) {
 *   $ionicModal.fromTemplateUrl('my-modal.html', {
 *     scope: $scope,
 *     animation: 'slide-in-up'
 *   }).then(function(modal) {
 *     $scope.modal = modal;
 *   });
 *   $scope.openModal = function() {
 *     $scope.modal.show();
 *   };
 *   $scope.closeModal = function() {
 *     $scope.modal.hide();
 *   };
 *   // Cleanup the modal when we're done with it!
 *   $scope.$on('$destroy', function() {
 *     $scope.modal.remove();
 *   });
 *   // Execute action on hide modal
 *   $scope.$on('modal.hidden', function() {
 *     // Execute action
 *   });
 *   // Execute action on remove modal
 *   $scope.$on('modal.removed', function() {
 *     // Execute action
 *   });
 * });
 * ```
 */
IonicModule
.factory('$ionicModal', [
  '$rootScope',
  '$ionicBody',
  '$compile',
  '$timeout',
  '$ionicPlatform',
  '$ionicTemplateLoader',
  '$$q',
  '$log',
  '$ionicClickBlock',
  '$window',
  'IONIC_BACK_PRIORITY',
function($rootScope, $ionicBody, $compile, $timeout, $ionicPlatform, $ionicTemplateLoader, $$q, $log, $ionicClickBlock, $window, IONIC_BACK_PRIORITY) {

  /**
   * @ngdoc controller
   * @name ionicModal
   * @module ionic
   * @description
   * Instantiated by the {@link ionic.service:$ionicModal} service.
   *
   * Be sure to call [remove()](#remove) when you are done with each modal
   * to clean it up and avoid memory leaks.
   *
   * Note: a modal will broadcast 'modal.shown', 'modal.hidden', and 'modal.removed' events from its originating
   * scope, passing in itself as an event argument. Note: both modal.removed and modal.hidden are
   * called when the modal is removed.
   */
  var ModalView = ionic.views.Modal.inherit({
    /**
     * @ngdoc method
     * @name ionicModal#initialize
     * @description Creates a new modal controller instance.
     * @param {object} options An options object with the following properties:
     *  - `{object=}` `scope` The scope to be a child of.
     *    Default: creates a child of $rootScope.
     *  - `{string=}` `animation` The animation to show & hide with.
     *    Default: 'slide-in-up'
     *  - `{boolean=}` `focusFirstInput` Whether to autofocus the first input of
     *    the modal when shown. Will only show the keyboard on iOS, to force the keyboard to show
     *    on Android, please use the [Ionic keyboard plugin](https://github.com/ionic-team/ionic-plugin-keyboard#keyboardshow).
     *    Default: false.
     *  - `{boolean=}` `backdropClickToClose` Whether to close the modal on clicking the backdrop.
     *    Default: true.
     *  - `{boolean=}` `hardwareBackButtonClose` Whether the modal can be closed using the hardware
     *    back button on Android and similar devices.  Default: true.
     */
    initialize: function(opts) {
      ionic.views.Modal.prototype.initialize.call(this, opts);
      this.animation = opts.animation || 'slide-in-up';
    },

    /**
     * @ngdoc method
     * @name ionicModal#show
     * @description Show this modal instance.
     * @returns {promise} A promise which is resolved when the modal is finished animating in.
     */
    show: function(target) {
      var self = this;

      if (self.scope.$$destroyed) {
        $log.error('Cannot call ' + self.viewType + '.show() after remove(). Please create a new ' + self.viewType + ' instance.');
        return $$q.when();
      }

      // on iOS, clicks will sometimes bleed through/ghost click on underlying
      // elements
      $ionicClickBlock.show(600);
      stack.add(self);

      var modalEl = jqLite(self.modalEl);

      self.el.classList.remove('hide');
      $timeout(function() {
        if (!self._isShown) return;
        $ionicBody.addClass(self.viewType + '-open');
      }, 400, false);

      if (!self.el.parentElement) {
        modalEl.addClass(self.animation);
        $ionicBody.append(self.el);
      }

      // if modal was closed while the keyboard was up, reset scroll view on
      // next show since we can only resize it once it's visible
      var scrollCtrl = modalEl.data('$$ionicScrollController');
      scrollCtrl && scrollCtrl.resize();

      if (target && self.positionView) {
        self.positionView(target, modalEl);
        // set up a listener for in case the window size changes

        self._onWindowResize = function() {
          if (self._isShown) self.positionView(target, modalEl);
        };
        ionic.on('resize', self._onWindowResize, window);
      }

      modalEl.addClass('ng-enter active')
             .removeClass('ng-leave ng-leave-active');

      self._isShown = true;
      self._deregisterBackButton = $ionicPlatform.registerBackButtonAction(
        self.hardwareBackButtonClose ? angular.bind(self, self.hide) : noop,
        IONIC_BACK_PRIORITY.modal
      );

      ionic.views.Modal.prototype.show.call(self);

      $timeout(function() {
        if (!self._isShown) return;
        modalEl.addClass('ng-enter-active');
        ionic.trigger('resize');
        self.scope.$parent && self.scope.$parent.$broadcast(self.viewType + '.shown', self);
        self.el.classList.add('active');
        self.scope.$broadcast('$ionicHeader.align');
        self.scope.$broadcast('$ionicFooter.align');
        self.scope.$broadcast('$ionic.modalPresented');
      }, 20);

      return $timeout(function() {
        if (!self._isShown) return;
        self.$el.on('touchmove', function(e) {
          //Don't allow scrolling while open by dragging on backdrop
          var isInScroll = ionic.DomUtil.getParentOrSelfWithClass(e.target, 'scroll');
          if (!isInScroll) {
            e.preventDefault();
          }
        });
        //After animating in, allow hide on backdrop click
        self.$el.on('click', function(e) {
          if (self.backdropClickToClose && e.target === self.el && stack.isHighest(self)) {
            self.hide();
          }
        });
      }, 400);
    },

    /**
     * @ngdoc method
     * @name ionicModal#hide
     * @description Hide this modal instance.
     * @returns {promise} A promise which is resolved when the modal is finished animating out.
     */
    hide: function() {
      var self = this;
      var modalEl = jqLite(self.modalEl);

      // on iOS, clicks will sometimes bleed through/ghost click on underlying
      // elements
      $ionicClickBlock.show(600);
      stack.remove(self);

      self.el.classList.remove('active');
      modalEl.addClass('ng-leave');

      $timeout(function() {
        if (self._isShown) return;
        modalEl.addClass('ng-leave-active')
               .removeClass('ng-enter ng-enter-active active');

        self.scope.$broadcast('$ionic.modalRemoved');
      }, 20, false);

      self.$el.off('click');
      self._isShown = false;
      self.scope.$parent && self.scope.$parent.$broadcast(self.viewType + '.hidden', self);
      self._deregisterBackButton && self._deregisterBackButton();

      ionic.views.Modal.prototype.hide.call(self);

      // clean up event listeners
      if (self.positionView) {
        ionic.off('resize', self._onWindowResize, window);
      }

      return $timeout(function() {
        if (!modalStack.length) {
          $ionicBody.removeClass(self.viewType + '-open');
        }
        self.el.classList.add('hide');
      }, self.hideDelay || 320);
    },

    /**
     * @ngdoc method
     * @name ionicModal#remove
     * @description Remove this modal instance from the DOM and clean up.
     * @returns {promise} A promise which is resolved when the modal is finished animating out.
     */
    remove: function() {
      var self = this,
          deferred, promise;
      self.scope.$parent && self.scope.$parent.$broadcast(self.viewType + '.removed', self);

      // Only hide modal, when it is actually shown!
      // The hide function shows a click-block-div for a split second, because on iOS,
      // clicks will sometimes bleed through/ghost click on underlying elements.
      // However, this will make the app unresponsive for short amount of time.
      // We don't want that, if the modal window is already hidden.
      if (self._isShown) {
        promise = self.hide();
      } else {
        deferred = $$q.defer();
        deferred.resolve();
        promise = deferred.promise;
      }

      return promise.then(function() {
        self.scope.$destroy();
        self.$el.remove();
      });
    },

    /**
     * @ngdoc method
     * @name ionicModal#isShown
     * @returns boolean Whether this modal is currently shown.
     */
    isShown: function() {
      return !!this._isShown;
    }
  });

  var createModal = function(templateString, options) {
    // Create a new scope for the modal
    var scope = options.scope && options.scope.$new() || $rootScope.$new(true);

    options.viewType = options.viewType || 'modal';

    extend(scope, {
      $hasHeader: false,
      $hasSubheader: false,
      $hasFooter: false,
      $hasSubfooter: false,
      $hasTabs: false,
      $hasTabsTop: false
    });

    // Compile the template
    var element = $compile('<ion-' + options.viewType + '>' + templateString + '</ion-' + options.viewType + '>')(scope);

    options.$el = element;
    options.el = element[0];
    options.modalEl = options.el.querySelector('.' + options.viewType);
    var modal = new ModalView(options);

    modal.scope = scope;

    // If this wasn't a defined scope, we can assign the viewType to the isolated scope
    // we created
    if (!options.scope) {
      scope[ options.viewType ] = modal;
    }

    return modal;
  };

  var modalStack = [];
  var stack = {
    add: function(modal) {
      modalStack.push(modal);
    },
    remove: function(modal) {
      var index = modalStack.indexOf(modal);
      if (index > -1 && index < modalStack.length) {
        modalStack.splice(index, 1);
      }
    },
    isHighest: function(modal) {
      var index = modalStack.indexOf(modal);
      return (index > -1 && index === modalStack.length - 1);
    }
  };

  return {
    /**
     * @ngdoc method
     * @name $ionicModal#fromTemplate
     * @param {string} templateString The template string to use as the modal's
     * content.
     * @param {object} options Options to be passed {@link ionic.controller:ionicModal#initialize ionicModal#initialize} method.
     * @returns {object} An instance of an {@link ionic.controller:ionicModal}
     * controller.
     */
    fromTemplate: function(templateString, options) {
      var modal = createModal(templateString, options || {});
      return modal;
    },
    /**
     * @ngdoc method
     * @name $ionicModal#fromTemplateUrl
     * @param {string} templateUrl The url to load the template from.
     * @param {object} options Options to be passed {@link ionic.controller:ionicModal#initialize ionicModal#initialize} method.
     * options object.
     * @returns {promise} A promise that will be resolved with an instance of
     * an {@link ionic.controller:ionicModal} controller.
     */
    fromTemplateUrl: function(url, options, _) {
      var cb;
      //Deprecated: allow a callback as second parameter. Now we return a promise.
      if (angular.isFunction(options)) {
        cb = options;
        options = _;
      }
      return $ionicTemplateLoader.load(url).then(function(templateString) {
        var modal = createModal(templateString, options || {});
        cb && cb(modal);
        return modal;
      });
    },

    stack: stack
  };
}]);


/**
 * @ngdoc service
 * @name $ionicNavBarDelegate
 * @module ionic
 * @description
 * Delegate for controlling the {@link ionic.directive:ionNavBar} directive.
 *
 * @usage
 *
 * ```html
 * <body ng-controller="MyCtrl">
 *   <ion-nav-bar>
 *     <button ng-click="setNavTitle('banana')">
 *       Set title to banana!
 *     </button>
 *   </ion-nav-bar>
 * </body>
 * ```
 * ```js
 * function MyCtrl($scope, $ionicNavBarDelegate) {
 *   $scope.setNavTitle = function(title) {
 *     $ionicNavBarDelegate.title(title);
 *   }
 * }
 * ```
 */
IonicModule
.service('$ionicNavBarDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicNavBarDelegate#align
   * @description Aligns the title with the buttons in a given direction.
   * @param {string=} direction The direction to the align the title text towards.
   * Available: 'left', 'right', 'center'. Default: 'center'.
   */
  'align',
  /**
   * @ngdoc method
   * @name $ionicNavBarDelegate#showBackButton
   * @description
   * Set/get whether the {@link ionic.directive:ionNavBackButton} is shown
   * (if it exists and there is a previous view that can be navigated to).
   * @param {boolean=} show Whether to show the back button.
   * @returns {boolean} Whether the back button is shown.
   */
  'showBackButton',
  /**
   * @ngdoc method
   * @name $ionicNavBarDelegate#showBar
   * @description
   * Set/get whether the {@link ionic.directive:ionNavBar} is shown.
   * @param {boolean} show Whether to show the bar.
   * @returns {boolean} Whether the bar is shown.
   */
  'showBar',
  /**
   * @ngdoc method
   * @name $ionicNavBarDelegate#title
   * @description
   * Set the title for the {@link ionic.directive:ionNavBar}.
   * @param {string} title The new title to show.
   */
  'title',

  // DEPRECATED, as of v1.0.0-beta14 -------
  'changeTitle',
  'setTitle',
  'getTitle',
  'back',
  'getPreviousTitle'
  // END DEPRECATED -------
]));


IonicModule
.service('$ionicNavViewDelegate', ionic.DelegateService([
  'clearCache'
]));



/**
 * @ngdoc service
 * @name $ionicPlatform
 * @module ionic
 * @description
 * An angular abstraction of {@link ionic.utility:ionic.Platform}.
 *
 * Used to detect the current platform, as well as do things like override the
 * Android back button in PhoneGap/Cordova.
 */
IonicModule
.constant('IONIC_BACK_PRIORITY', {
  view: 100,
  sideMenu: 150,
  modal: 200,
  actionSheet: 300,
  popup: 400,
  loading: 500
})
.provider('$ionicPlatform', function() {
  return {
    $get: ['$q', '$ionicScrollDelegate', function($q, $ionicScrollDelegate) {
      var self = {

        /**
         * @ngdoc method
         * @name $ionicPlatform#onHardwareBackButton
         * @description
         * Some platforms have a hardware back button, so this is one way to
         * bind to it.
         * @param {function} callback the callback to trigger when this event occurs
         */
        onHardwareBackButton: function(cb) {
          ionic.Platform.ready(function() {
            document.addEventListener('backbutton', cb, false);
          });
        },

        /**
         * @ngdoc method
         * @name $ionicPlatform#offHardwareBackButton
         * @description
         * Remove an event listener for the backbutton.
         * @param {function} callback The listener function that was
         * originally bound.
         */
        offHardwareBackButton: function(fn) {
          ionic.Platform.ready(function() {
            document.removeEventListener('backbutton', fn);
          });
        },

        /**
         * @ngdoc method
         * @name $ionicPlatform#registerBackButtonAction
         * @description
         * Register a hardware back button action. Only one action will execute
         * when the back button is clicked, so this method decides which of
         * the registered back button actions has the highest priority.
         *
         * For example, if an actionsheet is showing, the back button should
         * close the actionsheet, but it should not also go back a page view
         * or close a modal which may be open.
         *
         * The priorities for the existing back button hooks are as follows:
         *   Return to previous view = 100
         *   Close side menu = 150
         *   Dismiss modal = 200
         *   Close action sheet = 300
         *   Dismiss popup = 400
         *   Dismiss loading overlay = 500
         *
         * Your back button action will override each of the above actions
         * whose priority is less than the priority you provide. For example,
         * an action assigned a priority of 101 will override the 'return to
         * previous view' action, but not any of the other actions.
         *
         * @param {function} callback Called when the back button is pressed,
         * if this listener is the highest priority.
         * @param {number} priority Only the highest priority will execute.
         * @param {*=} actionId The id to assign this action. Default: a
         * random unique id.
         * @returns {function} A function that, when called, will deregister
         * this backButtonAction.
         */
        $backButtonActions: {},
        registerBackButtonAction: function(fn, priority, actionId) {

          if (!self._hasBackButtonHandler) {
            // add a back button listener if one hasn't been setup yet
            self.$backButtonActions = {};
            self.onHardwareBackButton(self.hardwareBackButtonClick);
            self._hasBackButtonHandler = true;
          }

          var action = {
            id: (actionId ? actionId : ionic.Utils.nextUid()),
            priority: (priority ? priority : 0),
            fn: fn
          };
          self.$backButtonActions[action.id] = action;

          // return a function to de-register this back button action
          return function() {
            delete self.$backButtonActions[action.id];
          };
        },

        /**
         * @private
         */
        hardwareBackButtonClick: function(e) {
          // loop through all the registered back button actions
          // and only run the last one of the highest priority
          var priorityAction, actionId;
          for (actionId in self.$backButtonActions) {
            if (!priorityAction || self.$backButtonActions[actionId].priority >= priorityAction.priority) {
              priorityAction = self.$backButtonActions[actionId];
            }
          }
          if (priorityAction) {
            priorityAction.fn(e);
            return priorityAction;
          }
        },

        is: function(type) {
          return ionic.Platform.is(type);
        },

        /**
         * @ngdoc method
         * @name $ionicPlatform#on
         * @description
         * Add Cordova event listeners, such as `pause`, `resume`, `volumedownbutton`, `batterylow`,
         * `offline`, etc. More information about available event types can be found in
         * [Cordova's event documentation](https://cordova.apache.org/docs/en/latest/cordova/events/events.html).
         * @param {string} type Cordova [event type](https://cordova.apache.org/docs/en/latest/cordova/events/events.html).
         * @param {function} callback Called when the Cordova event is fired.
         * @returns {function} Returns a deregistration function to remove the event listener.
         */
        on: function(type, cb) {
          ionic.Platform.ready(function() {
            document.addEventListener(type, cb, false);
          });
          return function() {
            ionic.Platform.ready(function() {
              document.removeEventListener(type, cb);
            });
          };
        },

        /**
         * @ngdoc method
         * @name $ionicPlatform#ready
         * @description
         * Trigger a callback once the device is ready,
         * or immediately if the device is already ready.
         * @param {function=} callback The function to call.
         * @returns {promise} A promise which is resolved when the device is ready.
         */
        ready: function(cb) {
          var q = $q.defer();

          ionic.Platform.ready(function() {

            window.addEventListener('statusTap', function() {
              $ionicScrollDelegate.scrollTop(true);
            });

            q.resolve();
            cb && cb();
          });

          return q.promise;
        }
      };

      return self;
    }]
  };

});

/**
 * @ngdoc service
 * @name $ionicPopover
 * @module ionic
 * @description
 *
 * Related: {@link ionic.controller:ionicPopover ionicPopover controller}.
 *
 * The Popover is a view that floats above an app’s content. Popovers provide an
 * easy way to present or gather information from the user and are
 * commonly used in the following situations:
 *
 * - Show more info about the current view
 * - Select a commonly used tool or configuration
 * - Present a list of actions to perform inside one of your views
 *
 * Put the content of the popover inside of an `<ion-popover-view>` element.
 *
 * @usage
 * ```html
 * <p>
 *   <button ng-click="openPopover($event)">Open Popover</button>
 * </p>
 *
 * <script id="my-popover.html" type="text/ng-template">
 *   <ion-popover-view>
 *     <ion-header-bar>
 *       <h1 class="title">My Popover Title</h1>
 *     </ion-header-bar>
 *     <ion-content>
 *       Hello!
 *     </ion-content>
 *   </ion-popover-view>
 * </script>
 * ```
 * ```js
 * angular.module('testApp', ['ionic'])
 * .controller('MyController', function($scope, $ionicPopover) {
 *
 *   // .fromTemplate() method
 *   var template = '<ion-popover-view><ion-header-bar> <h1 class="title">My Popover Title</h1> </ion-header-bar> <ion-content> Hello! </ion-content></ion-popover-view>';
 *
 *   $scope.popover = $ionicPopover.fromTemplate(template, {
 *     scope: $scope
 *   });
 *
 *   // .fromTemplateUrl() method
 *   $ionicPopover.fromTemplateUrl('my-popover.html', {
 *     scope: $scope
 *   }).then(function(popover) {
 *     $scope.popover = popover;
 *   });
 *
 *
 *   $scope.openPopover = function($event) {
 *     $scope.popover.show($event);
 *   };
 *   $scope.closePopover = function() {
 *     $scope.popover.hide();
 *   };
 *   //Cleanup the popover when we're done with it!
 *   $scope.$on('$destroy', function() {
 *     $scope.popover.remove();
 *   });
 *   // Execute action on hidden popover
 *   $scope.$on('popover.hidden', function() {
 *     // Execute action
 *   });
 *   // Execute action on remove popover
 *   $scope.$on('popover.removed', function() {
 *     // Execute action
 *   });
 * });
 * ```
 */


IonicModule
.factory('$ionicPopover', ['$ionicModal', '$ionicPosition', '$document', '$window',
function($ionicModal, $ionicPosition, $document, $window) {

  var POPOVER_BODY_PADDING = 6;

  var POPOVER_OPTIONS = {
    viewType: 'popover',
    hideDelay: 1,
    animation: 'none',
    positionView: positionView
  };

  function positionView(target, popoverEle) {
    var targetEle = jqLite(target.target || target);
    var buttonOffset = $ionicPosition.offset(targetEle);
    var popoverWidth = popoverEle.prop('offsetWidth');
    var popoverHeight = popoverEle.prop('offsetHeight');
    // Use innerWidth and innerHeight, because clientWidth and clientHeight
    // doesn't work consistently for body on all platforms
    var bodyWidth = $window.innerWidth;
    var bodyHeight = $window.innerHeight;

    var popoverCSS = {
      left: buttonOffset.left + buttonOffset.width / 2 - popoverWidth / 2
    };
    var arrowEle = jqLite(popoverEle[0].querySelector('.popover-arrow'));

    if (popoverCSS.left < POPOVER_BODY_PADDING) {
      popoverCSS.left = POPOVER_BODY_PADDING;
    } else if (popoverCSS.left + popoverWidth + POPOVER_BODY_PADDING > bodyWidth) {
      popoverCSS.left = bodyWidth - popoverWidth - POPOVER_BODY_PADDING;
    }

    // If the popover when popped down stretches past bottom of screen,
    // make it pop up if there's room above
    if (buttonOffset.top + buttonOffset.height + popoverHeight > bodyHeight &&
        buttonOffset.top - popoverHeight > 0) {
      popoverCSS.top = buttonOffset.top - popoverHeight;
      popoverEle.addClass('popover-bottom');
    } else {
      popoverCSS.top = buttonOffset.top + buttonOffset.height;
      popoverEle.removeClass('popover-bottom');
    }

    arrowEle.css({
      left: buttonOffset.left + buttonOffset.width / 2 -
        arrowEle.prop('offsetWidth') / 2 - popoverCSS.left + 'px'
    });

    popoverEle.css({
      top: popoverCSS.top + 'px',
      left: popoverCSS.left + 'px',
      marginLeft: '0',
      opacity: '1'
    });

  }

  /**
   * @ngdoc controller
   * @name ionicPopover
   * @module ionic
   * @description
   * Instantiated by the {@link ionic.service:$ionicPopover} service.
   *
   * Be sure to call [remove()](#remove) when you are done with each popover
   * to clean it up and avoid memory leaks.
   *
   * Note: a popover will broadcast 'popover.shown', 'popover.hidden', and 'popover.removed' events from its originating
   * scope, passing in itself as an event argument. Both the popover.removed and popover.hidden events are
   * called when the popover is removed.
   */

  /**
   * @ngdoc method
   * @name ionicPopover#initialize
   * @description Creates a new popover controller instance.
   * @param {object} options An options object with the following properties:
   *  - `{object=}` `scope` The scope to be a child of.
   *    Default: creates a child of $rootScope.
   *  - `{boolean=}` `focusFirstInput` Whether to autofocus the first input of
   *    the popover when shown.  Default: false.
   *  - `{boolean=}` `backdropClickToClose` Whether to close the popover on clicking the backdrop.
   *    Default: true.
   *  - `{boolean=}` `hardwareBackButtonClose` Whether the popover can be closed using the hardware
   *    back button on Android and similar devices.  Default: true.
   */

  /**
   * @ngdoc method
   * @name ionicPopover#show
   * @description Show this popover instance.
   * @param {$event} $event The $event or target element which the popover should align
   * itself next to.
   * @returns {promise} A promise which is resolved when the popover is finished animating in.
   */

  /**
   * @ngdoc method
   * @name ionicPopover#hide
   * @description Hide this popover instance.
   * @returns {promise} A promise which is resolved when the popover is finished animating out.
   */

  /**
   * @ngdoc method
   * @name ionicPopover#remove
   * @description Remove this popover instance from the DOM and clean up.
   * @returns {promise} A promise which is resolved when the popover is finished animating out.
   */

  /**
   * @ngdoc method
   * @name ionicPopover#isShown
   * @returns boolean Whether this popover is currently shown.
   */

  return {
    /**
     * @ngdoc method
     * @name $ionicPopover#fromTemplate
     * @param {string} templateString The template string to use as the popovers's
     * content.
     * @param {object} options Options to be passed to the initialize method.
     * @returns {object} An instance of an {@link ionic.controller:ionicPopover}
     * controller (ionicPopover is built on top of $ionicPopover).
     */
    fromTemplate: function(templateString, options) {
      return $ionicModal.fromTemplate(templateString, ionic.Utils.extend({}, POPOVER_OPTIONS, options));
    },
    /**
     * @ngdoc method
     * @name $ionicPopover#fromTemplateUrl
     * @param {string} templateUrl The url to load the template from.
     * @param {object} options Options to be passed to the initialize method.
     * @returns {promise} A promise that will be resolved with an instance of
     * an {@link ionic.controller:ionicPopover} controller (ionicPopover is built on top of $ionicPopover).
     */
    fromTemplateUrl: function(url, options) {
      return $ionicModal.fromTemplateUrl(url, ionic.Utils.extend({}, POPOVER_OPTIONS, options));
    }
  };

}]);


var POPUP_TPL =
  '<div class="popup-container" ng-class="cssClass">' +
    '<div class="popup">' +
      '<div class="popup-head">' +
        '<h3 class="popup-title" ng-bind-html="title"></h3>' +
        '<h5 class="popup-sub-title" ng-bind-html="subTitle" ng-if="subTitle"></h5>' +
      '</div>' +
      '<div class="popup-body">' +
      '</div>' +
      '<div class="popup-buttons" ng-show="buttons.length">' +
        '<button ng-repeat="button in buttons" ng-click="$buttonTapped(button, $event)" class="button" ng-class="button.type || \'button-default\'" ng-bind-html="button.text"></button>' +
      '</div>' +
    '</div>' +
  '</div>';

/**
 * @ngdoc service
 * @name $ionicPopup
 * @module ionic
 * @restrict E
 * @codepen zkmhJ
 * @description
 *
 * The Ionic Popup service allows programmatically creating and showing popup
 * windows that require the user to respond in order to continue.
 *
 * The popup system has support for more flexible versions of the built in `alert()`, `prompt()`,
 * and `confirm()` functions that users are used to, in addition to allowing popups with completely
 * custom content and look.
 *
 * An input can be given an `autofocus` attribute so it automatically receives focus when
 * the popup first shows. However, depending on certain use-cases this can cause issues with
 * the tap/click system, which is why Ionic prefers using the `autofocus` attribute as
 * an opt-in feature and not the default.
 *
 * @usage
 * A few basic examples, see below for details about all of the options available.
 *
 * ```js
 *angular.module('mySuperApp', ['ionic'])
 *.controller('PopupCtrl',function($scope, $ionicPopup, $timeout) {
 *
 * // Triggered on a button click, or some other target
 * $scope.showPopup = function() {
 *   $scope.data = {};
 *
 *   // An elaborate, custom popup
 *   var myPopup = $ionicPopup.show({
 *     template: '<input type="password" ng-model="data.wifi">',
 *     title: 'Enter Wi-Fi Password',
 *     subTitle: 'Please use normal things',
 *     scope: $scope,
 *     buttons: [
 *       { text: 'Cancel' },
 *       {
 *         text: '<b>Save</b>',
 *         type: 'button-positive',
 *         onTap: function(e) {
 *           if (!$scope.data.wifi) {
 *             //don't allow the user to close unless he enters wifi password
 *             e.preventDefault();
 *           } else {
 *             return $scope.data.wifi;
 *           }
 *         }
 *       }
 *     ]
 *   });
 *
 *   myPopup.then(function(res) {
 *     console.log('Tapped!', res);
 *   });
 *
 *   $timeout(function() {
 *      myPopup.close(); //close the popup after 3 seconds for some reason
 *   }, 3000);
 *  };
 *
 *  // A confirm dialog
 *  $scope.showConfirm = function() {
 *    var confirmPopup = $ionicPopup.confirm({
 *      title: 'Consume Ice Cream',
 *      template: 'Are you sure you want to eat this ice cream?'
 *    });
 *
 *    confirmPopup.then(function(res) {
 *      if(res) {
 *        console.log('You are sure');
 *      } else {
 *        console.log('You are not sure');
 *      }
 *    });
 *  };
 *
 *  // An alert dialog
 *  $scope.showAlert = function() {
 *    var alertPopup = $ionicPopup.alert({
 *      title: 'Don\'t eat that!',
 *      template: 'It might taste good'
 *    });
 *
 *    alertPopup.then(function(res) {
 *      console.log('Thank you for not eating my delicious ice cream cone');
 *    });
 *  };
 *});
 *```
 */

IonicModule
.factory('$ionicPopup', [
  '$ionicTemplateLoader',
  '$ionicBackdrop',
  '$q',
  '$timeout',
  '$rootScope',
  '$ionicBody',
  '$compile',
  '$ionicPlatform',
  '$ionicModal',
  'IONIC_BACK_PRIORITY',
function($ionicTemplateLoader, $ionicBackdrop, $q, $timeout, $rootScope, $ionicBody, $compile, $ionicPlatform, $ionicModal, IONIC_BACK_PRIORITY) {
  //TODO allow this to be configured
  var config = {
    stackPushDelay: 75
  };
  var popupStack = [];

  var $ionicPopup = {
    /**
     * @ngdoc method
     * @description
     * Show a complex popup. This is the master show function for all popups.
     *
     * A complex popup has a `buttons` array, with each button having a `text` and `type`
     * field, in addition to an `onTap` function.  The `onTap` function, called when
     * the corresponding button on the popup is tapped, will by default close the popup
     * and resolve the popup promise with its return value.  If you wish to prevent the
     * default and keep the popup open on button tap, call `event.preventDefault()` on the
     * passed in tap event.  Details below.
     *
     * @name $ionicPopup#show
     * @param {object} options The options for the new popup, of the form:
     *
     * ```
     * {
     *   title: '', // String. The title of the popup.
     *   cssClass: '', // String, The custom CSS class name
     *   subTitle: '', // String (optional). The sub-title of the popup.
     *   template: '', // String (optional). The html template to place in the popup body.
     *   templateUrl: '', // String (optional). The URL of an html template to place in the popup   body.
     *   scope: null, // Scope (optional). A scope to link to the popup content.
     *   buttons: [{ // Array[Object] (optional). Buttons to place in the popup footer.
     *     text: 'Cancel',
     *     type: 'button-default',
     *     onTap: function(e) {
     *       // e.preventDefault() will stop the popup from closing when tapped.
     *       e.preventDefault();
     *     }
     *   }, {
     *     text: 'OK',
     *     type: 'button-positive',
     *     onTap: function(e) {
     *       // Returning a value will cause the promise to resolve with the given value.
     *       return scope.data.response;
     *     }
     *   }]
     * }
     * ```
     *
     * @returns {object} A promise which is resolved when the popup is closed. Has an additional
     * `close` function, which can be used to programmatically close the popup.
     */
    show: showPopup,

    /**
     * @ngdoc method
     * @name $ionicPopup#alert
     * @description Show a simple alert popup with a message and one button that the user can
     * tap to close the popup.
     *
     * @param {object} options The options for showing the alert, of the form:
     *
     * ```
     * {
     *   title: '', // String. The title of the popup.
     *   cssClass: '', // String, The custom CSS class name
     *   subTitle: '', // String (optional). The sub-title of the popup.
     *   template: '', // String (optional). The html template to place in the popup body.
     *   templateUrl: '', // String (optional). The URL of an html template to place in the popup   body.
     *   okText: '', // String (default: 'OK'). The text of the OK button.
     *   okType: '', // String (default: 'button-positive'). The type of the OK button.
     * }
     * ```
     *
     * @returns {object} A promise which is resolved when the popup is closed. Has one additional
     * function `close`, which can be called with any value to programmatically close the popup
     * with the given value.
     */
    alert: showAlert,

    /**
     * @ngdoc method
     * @name $ionicPopup#confirm
     * @description
     * Show a simple confirm popup with a Cancel and OK button.
     *
     * Resolves the promise with true if the user presses the OK button, and false if the
     * user presses the Cancel button.
     *
     * @param {object} options The options for showing the confirm popup, of the form:
     *
     * ```
     * {
     *   title: '', // String. The title of the popup.
     *   cssClass: '', // String, The custom CSS class name
     *   subTitle: '', // String (optional). The sub-title of the popup.
     *   template: '', // String (optional). The html template to place in the popup body.
     *   templateUrl: '', // String (optional). The URL of an html template to place in the popup   body.
     *   cancelText: '', // String (default: 'Cancel'). The text of the Cancel button.
     *   cancelType: '', // String (default: 'button-default'). The type of the Cancel button.
     *   okText: '', // String (default: 'OK'). The text of the OK button.
     *   okType: '', // String (default: 'button-positive'). The type of the OK button.
     * }
     * ```
     *
     * @returns {object} A promise which is resolved when the popup is closed. Has one additional
     * function `close`, which can be called with any value to programmatically close the popup
     * with the given value.
     */
    confirm: showConfirm,

    /**
     * @ngdoc method
     * @name $ionicPopup#prompt
     * @description Show a simple prompt popup, which has an input, OK button, and Cancel button.
     * Resolves the promise with the value of the input if the user presses OK, and with undefined
     * if the user presses Cancel.
     *
     * ```javascript
     *  $ionicPopup.prompt({
     *    title: 'Password Check',
     *    template: 'Enter your secret password',
     *    inputType: 'password',
     *    inputPlaceholder: 'Your password'
     *  }).then(function(res) {
     *    console.log('Your password is', res);
     *  });
     * ```
     * @param {object} options The options for showing the prompt popup, of the form:
     *
     * ```
     * {
     *   title: '', // String. The title of the popup.
     *   cssClass: '', // String, The custom CSS class name
     *   subTitle: '', // String (optional). The sub-title of the popup.
     *   template: '', // String (optional). The html template to place in the popup body.
     *   templateUrl: '', // String (optional). The URL of an html template to place in the popup body.
     *   inputType: // String (default: 'text'). The type of input to use
     *   defaultText: // String (default: ''). The initial value placed into the input.
     *   maxLength: // Integer (default: null). Specify a maxlength attribute for the input.
     *   inputPlaceholder: // String (default: ''). A placeholder to use for the input.
     *   cancelText: // String (default: 'Cancel'. The text of the Cancel button.
     *   cancelType: // String (default: 'button-default'). The type of the Cancel button.
     *   okText: // String (default: 'OK'). The text of the OK button.
     *   okType: // String (default: 'button-positive'). The type of the OK button.
     * }
     * ```
     *
     * @returns {object} A promise which is resolved when the popup is closed. Has one additional
     * function `close`, which can be called with any value to programmatically close the popup
     * with the given value.
     */
    prompt: showPrompt,
    /**
     * @private for testing
     */
    _createPopup: createPopup,
    _popupStack: popupStack
  };

  return $ionicPopup;

  function createPopup(options) {
    options = extend({
      scope: null,
      title: '',
      buttons: []
    }, options || {});

    var self = {};
    self.scope = (options.scope || $rootScope).$new();
    self.element = jqLite(POPUP_TPL);
    self.responseDeferred = $q.defer();

    $ionicBody.get().appendChild(self.element[0]);
    $compile(self.element)(self.scope);

    extend(self.scope, {
      title: options.title,
      buttons: options.buttons,
      subTitle: options.subTitle,
      cssClass: options.cssClass,
      $buttonTapped: function(button, event) {
        var result = (button.onTap || noop).apply(self, [event]);
        event = event.originalEvent || event; //jquery events

        if (!event.defaultPrevented) {
          self.responseDeferred.resolve(result);
        }
      }
    });

    $q.when(
      options.templateUrl ?
      $ionicTemplateLoader.load(options.templateUrl) :
        (options.template || options.content || '')
    ).then(function(template) {
      var popupBody = jqLite(self.element[0].querySelector('.popup-body'));
      if (template) {
        popupBody.html(template);
        $compile(popupBody.contents())(self.scope);
      } else {
        popupBody.remove();
      }
    });

    self.show = function() {
      if (self.isShown || self.removed) return;

      $ionicModal.stack.add(self);
      self.isShown = true;
      ionic.requestAnimationFrame(function() {
        //if hidden while waiting for raf, don't show
        if (!self.isShown) return;

        self.element.removeClass('popup-hidden');
        self.element.addClass('popup-showing active');
        focusInput(self.element);
      });
    };

    self.hide = function(callback) {
      callback = callback || noop;
      if (!self.isShown) return callback();

      $ionicModal.stack.remove(self);
      self.isShown = false;
      self.element.removeClass('active');
      self.element.addClass('popup-hidden');
      $timeout(callback, 250, false);
    };

    self.remove = function() {
      if (self.removed) return;

      self.hide(function() {
        self.element.remove();
        self.scope.$destroy();
      });

      self.removed = true;
    };

    return self;
  }

  function onHardwareBackButton() {
    var last = popupStack[popupStack.length - 1];
    last && last.responseDeferred.resolve();
  }

  function showPopup(options) {
    var popup = $ionicPopup._createPopup(options);
    var showDelay = 0;

    if (popupStack.length > 0) {
      showDelay = config.stackPushDelay;
      $timeout(popupStack[popupStack.length - 1].hide, showDelay, false);
    } else {
      //Add popup-open & backdrop if this is first popup
      $ionicBody.addClass('popup-open');
      $ionicBackdrop.retain();
      //only show the backdrop on the first popup
      $ionicPopup._backButtonActionDone = $ionicPlatform.registerBackButtonAction(
        onHardwareBackButton,
        IONIC_BACK_PRIORITY.popup
      );
    }

    // Expose a 'close' method on the returned promise
    popup.responseDeferred.promise.close = function popupClose(result) {
      if (!popup.removed) popup.responseDeferred.resolve(result);
    };
    //DEPRECATED: notify the promise with an object with a close method
    popup.responseDeferred.notify({ close: popup.responseDeferred.close });

    doShow();

    return popup.responseDeferred.promise;

    function doShow() {
      popupStack.push(popup);
      $timeout(popup.show, showDelay, false);

      popup.responseDeferred.promise.then(function(result) {
        var index = popupStack.indexOf(popup);
        if (index !== -1) {
          popupStack.splice(index, 1);
        }

        popup.remove();

        if (popupStack.length > 0) {
          popupStack[popupStack.length - 1].show();
        } else {
          $ionicBackdrop.release();
          //Remove popup-open & backdrop if this is last popup
          $timeout(function() {
            // wait to remove this due to a 300ms delay native
            // click which would trigging whatever was underneath this
            if (!popupStack.length) {
              $ionicBody.removeClass('popup-open');
            }
          }, 400, false);
          ($ionicPopup._backButtonActionDone || noop)();
        }


        return result;
      });

    }

  }

  function focusInput(element) {
    var focusOn = element[0].querySelector('[autofocus]');
    if (focusOn) {
      focusOn.focus();
    }
  }

  function showAlert(opts) {
    return showPopup(extend({
      buttons: [{
        text: opts.okText || 'OK',
        type: opts.okType || 'button-positive',
        onTap: function() {
          return true;
        }
      }]
    }, opts || {}));
  }

  function showConfirm(opts) {
    return showPopup(extend({
      buttons: [{
        text: opts.cancelText || 'Cancel',
        type: opts.cancelType || 'button-default',
        onTap: function() { return false; }
      }, {
        text: opts.okText || 'OK',
        type: opts.okType || 'button-positive',
        onTap: function() { return true; }
      }]
    }, opts || {}));
  }

  function showPrompt(opts) {
    var scope = $rootScope.$new(true);
    scope.data = {};
    scope.data.fieldtype = opts.inputType ? opts.inputType : 'text';
    scope.data.response = opts.defaultText ? opts.defaultText : '';
    scope.data.placeholder = opts.inputPlaceholder ? opts.inputPlaceholder : '';
    scope.data.maxlength = opts.maxLength ? parseInt(opts.maxLength) : '';
    var text = '';
    if (opts.template && /<[a-z][\s\S]*>/i.test(opts.template) === false) {
      text = '<span>' + opts.template + '</span>';
      delete opts.template;
    }
    return showPopup(extend({
      template: text + '<input ng-model="data.response" '
        + 'type="{{ data.fieldtype }}"'
        + 'maxlength="{{ data.maxlength }}"'
        + 'placeholder="{{ data.placeholder }}"'
        + '>',
      scope: scope,
      buttons: [{
        text: opts.cancelText || 'Cancel',
        type: opts.cancelType || 'button-default',
        onTap: function() {}
      }, {
        text: opts.okText || 'OK',
        type: opts.okType || 'button-positive',
        onTap: function() {
          return scope.data.response || '';
        }
      }]
    }, opts || {}));
  }
}]);

/**
 * @ngdoc service
 * @name $ionicPosition
 * @module ionic
 * @description
 * A set of utility methods that can be use to retrieve position of DOM elements.
 * It is meant to be used where we need to absolute-position DOM elements in
 * relation to other, existing elements (this is the case for tooltips, popovers, etc.).
 *
 * Adapted from [AngularUI Bootstrap](https://github.com/angular-ui/bootstrap/blob/master/src/position/position.js),
 * ([license](https://github.com/angular-ui/bootstrap/blob/master/LICENSE))
 */
IonicModule
.factory('$ionicPosition', ['$document', '$window', function($document, $window) {

  function getStyle(el, cssprop) {
    if (el.currentStyle) { //IE
      return el.currentStyle[cssprop];
    } else if ($window.getComputedStyle) {
      return $window.getComputedStyle(el)[cssprop];
    }
    // finally try and get inline style
    return el.style[cssprop];
  }

  /**
   * Checks if a given element is statically positioned
   * @param element - raw DOM element
   */
  function isStaticPositioned(element) {
    return (getStyle(element, 'position') || 'static') === 'static';
  }

  /**
   * returns the closest, non-statically positioned parentOffset of a given element
   * @param element
   */
  var parentOffsetEl = function(element) {
    var docDomEl = $document[0];
    var offsetParent = element.offsetParent || docDomEl;
    while (offsetParent && offsetParent !== docDomEl && isStaticPositioned(offsetParent)) {
      offsetParent = offsetParent.offsetParent;
    }
    return offsetParent || docDomEl;
  };

  return {
    /**
     * @ngdoc method
     * @name $ionicPosition#position
     * @description Get the current coordinates of the element, relative to the offset parent.
     * Read-only equivalent of [jQuery's position function](http://api.jquery.com/position/).
     * @param {element} element The element to get the position of.
     * @returns {object} Returns an object containing the properties top, left, width and height.
     */
    position: function(element) {
      var elBCR = this.offset(element);
      var offsetParentBCR = { top: 0, left: 0 };
      var offsetParentEl = parentOffsetEl(element[0]);
      if (offsetParentEl != $document[0]) {
        offsetParentBCR = this.offset(jqLite(offsetParentEl));
        offsetParentBCR.top += offsetParentEl.clientTop - offsetParentEl.scrollTop;
        offsetParentBCR.left += offsetParentEl.clientLeft - offsetParentEl.scrollLeft;
      }

      var boundingClientRect = element[0].getBoundingClientRect();
      return {
        width: boundingClientRect.width || element.prop('offsetWidth'),
        height: boundingClientRect.height || element.prop('offsetHeight'),
        top: elBCR.top - offsetParentBCR.top,
        left: elBCR.left - offsetParentBCR.left
      };
    },

    /**
     * @ngdoc method
     * @name $ionicPosition#offset
     * @description Get the current coordinates of the element, relative to the document.
     * Read-only equivalent of [jQuery's offset function](http://api.jquery.com/offset/).
     * @param {element} element The element to get the offset of.
     * @returns {object} Returns an object containing the properties top, left, width and height.
     */
    offset: function(element) {
      var boundingClientRect = element[0].getBoundingClientRect();
      return {
        width: boundingClientRect.width || element.prop('offsetWidth'),
        height: boundingClientRect.height || element.prop('offsetHeight'),
        top: boundingClientRect.top + ($window.pageYOffset || $document[0].documentElement.scrollTop),
        left: boundingClientRect.left + ($window.pageXOffset || $document[0].documentElement.scrollLeft)
      };
    }

  };
}]);


/**
 * @ngdoc service
 * @name $ionicScrollDelegate
 * @module ionic
 * @description
 * Delegate for controlling scrollViews (created by
 * {@link ionic.directive:ionContent} and
 * {@link ionic.directive:ionScroll} directives).
 *
 * Methods called directly on the $ionicScrollDelegate service will control all scroll
 * views.  Use the {@link ionic.service:$ionicScrollDelegate#$getByHandle $getByHandle}
 * method to control specific scrollViews.
 *
 * @usage
 *
 * ```html
 * <body ng-controller="MainCtrl">
 *   <ion-content>
 *     <button ng-click="scrollTop()">Scroll to Top!</button>
 *   </ion-content>
 * </body>
 * ```
 * ```js
 * function MainCtrl($scope, $ionicScrollDelegate) {
 *   $scope.scrollTop = function() {
 *     $ionicScrollDelegate.scrollTop();
 *   };
 * }
 * ```
 *
 * Example of advanced usage, with two scroll areas using `delegate-handle`
 * for fine control.
 *
 * ```html
 * <body ng-controller="MainCtrl">
 *   <ion-content delegate-handle="mainScroll">
 *     <button ng-click="scrollMainToTop()">
 *       Scroll content to top!
 *     </button>
 *     <ion-scroll delegate-handle="small" style="height: 100px;">
 *       <button ng-click="scrollSmallToTop()">
 *         Scroll small area to top!
 *       </button>
 *     </ion-scroll>
 *   </ion-content>
 * </body>
 * ```
 * ```js
 * function MainCtrl($scope, $ionicScrollDelegate) {
 *   $scope.scrollMainToTop = function() {
 *     $ionicScrollDelegate.$getByHandle('mainScroll').scrollTop();
 *   };
 *   $scope.scrollSmallToTop = function() {
 *     $ionicScrollDelegate.$getByHandle('small').scrollTop();
 *   };
 * }
 * ```
 */
IonicModule
.service('$ionicScrollDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#resize
   * @description Tell the scrollView to recalculate the size of its container.
   */
  'resize',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#scrollTop
   * @param {boolean=} shouldAnimate Whether the scroll should animate.
   */
  'scrollTop',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#scrollBottom
   * @param {boolean=} shouldAnimate Whether the scroll should animate.
   */
  'scrollBottom',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#scrollTo
   * @param {number} left The x-value to scroll to.
   * @param {number} top The y-value to scroll to.
   * @param {boolean=} shouldAnimate Whether the scroll should animate.
   */
  'scrollTo',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#scrollBy
   * @param {number} left The x-offset to scroll by.
   * @param {number} top The y-offset to scroll by.
   * @param {boolean=} shouldAnimate Whether the scroll should animate.
   */
  'scrollBy',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#zoomTo
   * @param {number} level Level to zoom to.
   * @param {boolean=} animate Whether to animate the zoom.
   * @param {number=} originLeft Zoom in at given left coordinate.
   * @param {number=} originTop Zoom in at given top coordinate.
   */
  'zoomTo',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#zoomBy
   * @param {number} factor The factor to zoom by.
   * @param {boolean=} animate Whether to animate the zoom.
   * @param {number=} originLeft Zoom in at given left coordinate.
   * @param {number=} originTop Zoom in at given top coordinate.
   */
  'zoomBy',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#getScrollPosition
   * @returns {object} The scroll position of this view, with the following properties:
   *  - `{number}` `left` The distance the user has scrolled from the left (starts at 0).
   *  - `{number}` `top` The distance the user has scrolled from the top (starts at 0).
   *  - `{number}` `zoom` The current zoom level.
   */
  'getScrollPosition',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#anchorScroll
   * @description Tell the scrollView to scroll to the element with an id
   * matching window.location.hash.
   *
   * If no matching element is found, it will scroll to top.
   *
   * @param {boolean=} shouldAnimate Whether the scroll should animate.
   */
  'anchorScroll',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#freezeScroll
   * @description Does not allow this scroll view to scroll either x or y.
   * @param {boolean=} shouldFreeze Should this scroll view be prevented from scrolling or not.
   * @returns {boolean} If the scroll view is being prevented from scrolling or not.
   */
  'freezeScroll',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#freezeAllScrolls
   * @description Does not allow any of the app's scroll views to scroll either x or y.
   * @param {boolean=} shouldFreeze Should all app scrolls be prevented from scrolling or not.
   */
  'freezeAllScrolls',
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#getScrollView
   * @returns {object} The scrollView associated with this delegate.
   */
  'getScrollView'
  /**
   * @ngdoc method
   * @name $ionicScrollDelegate#$getByHandle
   * @param {string} handle
   * @returns `delegateInstance` A delegate instance that controls only the
   * scrollViews with `delegate-handle` matching the given handle.
   *
   * Example: `$ionicScrollDelegate.$getByHandle('my-handle').scrollTop();`
   */
]));


/**
 * @ngdoc service
 * @name $ionicSideMenuDelegate
 * @module ionic
 *
 * @description
 * Delegate for controlling the {@link ionic.directive:ionSideMenus} directive.
 *
 * Methods called directly on the $ionicSideMenuDelegate service will control all side
 * menus.  Use the {@link ionic.service:$ionicSideMenuDelegate#$getByHandle $getByHandle}
 * method to control specific ionSideMenus instances.
 *
 * @usage
 *
 * ```html
 * <body ng-controller="MainCtrl">
 *   <ion-side-menus>
 *     <ion-side-menu-content>
 *       Content!
 *       <button ng-click="toggleLeftSideMenu()">
 *         Toggle Left Side Menu
 *       </button>
 *     </ion-side-menu-content>
 *     <ion-side-menu side="left">
 *       Left Menu!
 *     <ion-side-menu>
 *   </ion-side-menus>
 * </body>
 * ```
 * ```js
 * function MainCtrl($scope, $ionicSideMenuDelegate) {
 *   $scope.toggleLeftSideMenu = function() {
 *     $ionicSideMenuDelegate.toggleLeft();
 *   };
 * }
 * ```
 */
IonicModule
.service('$ionicSideMenuDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#toggleLeft
   * @description Toggle the left side menu (if it exists).
   * @param {boolean=} isOpen Whether to open or close the menu.
   * Default: Toggles the menu.
   */
  'toggleLeft',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#toggleRight
   * @description Toggle the right side menu (if it exists).
   * @param {boolean=} isOpen Whether to open or close the menu.
   * Default: Toggles the menu.
   */
  'toggleRight',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#getOpenRatio
   * @description Gets the ratio of open amount over menu width. For example, a
   * menu of width 100 that is opened by 50 pixels is 50% opened, and would return
   * a ratio of 0.5.
   *
   * @returns {float} 0 if nothing is open, between 0 and 1 if left menu is
   * opened/opening, and between 0 and -1 if right menu is opened/opening.
   */
  'getOpenRatio',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#isOpen
   * @returns {boolean} Whether either the left or right menu is currently opened.
   */
  'isOpen',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#isOpenLeft
   * @returns {boolean} Whether the left menu is currently opened.
   */
  'isOpenLeft',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#isOpenRight
   * @returns {boolean} Whether the right menu is currently opened.
   */
  'isOpenRight',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#canDragContent
   * @param {boolean=} canDrag Set whether the content can or cannot be dragged to open
   * side menus.
   * @returns {boolean} Whether the content can be dragged to open side menus.
   */
  'canDragContent',
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#edgeDragThreshold
   * @param {boolean|number=} value Set whether the content drag can only start if it is below a certain threshold distance from the edge of the screen. Accepts three different values:
   *  - If a non-zero number is given, that many pixels is used as the maximum allowed distance from the edge that starts dragging the side menu.
   *  - If true is given, the default number of pixels (25) is used as the maximum allowed distance.
   *  - If false or 0 is given, the edge drag threshold is disabled, and dragging from anywhere on the content is allowed.
   * @returns {boolean} Whether the drag can start only from within the edge of screen threshold.
   */
  'edgeDragThreshold'
  /**
   * @ngdoc method
   * @name $ionicSideMenuDelegate#$getByHandle
   * @param {string} handle
   * @returns `delegateInstance` A delegate instance that controls only the
   * {@link ionic.directive:ionSideMenus} directives with `delegate-handle` matching
   * the given handle.
   *
   * Example: `$ionicSideMenuDelegate.$getByHandle('my-handle').toggleLeft();`
   */
]));


/**
 * @ngdoc service
 * @name $ionicSlideBoxDelegate
 * @module ionic
 * @description
 * Delegate that controls the {@link ionic.directive:ionSlideBox} directive.
 *
 * Methods called directly on the $ionicSlideBoxDelegate service will control all slide boxes.  Use the {@link ionic.service:$ionicSlideBoxDelegate#$getByHandle $getByHandle}
 * method to control specific slide box instances.
 *
 * @usage
 *
 * ```html
 * <ion-view>
 *   <ion-slide-box>
 *     <ion-slide>
 *       <div class="box blue">
 *         <button ng-click="nextSlide()">Next slide!</button>
 *       </div>
 *     </ion-slide>
 *     <ion-slide>
 *       <div class="box red">
 *         Slide 2!
 *       </div>
 *     </ion-slide>
 *   </ion-slide-box>
 * </ion-view>
 * ```
 * ```js
 * function MyCtrl($scope, $ionicSlideBoxDelegate) {
 *   $scope.nextSlide = function() {
 *     $ionicSlideBoxDelegate.next();
 *   }
 * }
 * ```
 */
IonicModule
.service('$ionicSlideBoxDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#update
   * @description
   * Update the slidebox (for example if using Angular with ng-repeat,
   * resize it for the elements inside).
   */
  'update',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#slide
   * @param {number} to The index to slide to.
   * @param {number=} speed The number of milliseconds the change should take.
   */
  'slide',
  'select',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#enableSlide
   * @param {boolean=} shouldEnable Whether to enable sliding the slidebox.
   * @returns {boolean} Whether sliding is enabled.
   */
  'enableSlide',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#previous
   * @param {number=} speed The number of milliseconds the change should take.
   * @description Go to the previous slide. Wraps around if at the beginning.
   */
  'previous',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#next
   * @param {number=} speed The number of milliseconds the change should take.
   * @description Go to the next slide. Wraps around if at the end.
   */
  'next',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#stop
   * @description Stop sliding. The slideBox will not move again until
   * explicitly told to do so.
   */
  'stop',
  'autoPlay',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#start
   * @description Start sliding again if the slideBox was stopped.
   */
  'start',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#currentIndex
   * @returns number The index of the current slide.
   */
  'currentIndex',
  'selected',
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#slidesCount
   * @returns number The number of slides there are currently.
   */
  'slidesCount',
  'count',
  'loop'
  /**
   * @ngdoc method
   * @name $ionicSlideBoxDelegate#$getByHandle
   * @param {string} handle
   * @returns `delegateInstance` A delegate instance that controls only the
   * {@link ionic.directive:ionSlideBox} directives with `delegate-handle` matching
   * the given handle.
   *
   * Example: `$ionicSlideBoxDelegate.$getByHandle('my-handle').stop();`
   */
]));


/**
 * @ngdoc service
 * @name $ionicTabsDelegate
 * @module ionic
 *
 * @description
 * Delegate for controlling the {@link ionic.directive:ionTabs} directive.
 *
 * Methods called directly on the $ionicTabsDelegate service will control all ionTabs
 * directives. Use the {@link ionic.service:$ionicTabsDelegate#$getByHandle $getByHandle}
 * method to control specific ionTabs instances.
 *
 * @usage
 *
 * ```html
 * <body ng-controller="MyCtrl">
 *   <ion-tabs>
 *
 *     <ion-tab title="Tab 1">
 *       Hello tab 1!
 *       <button ng-click="selectTabWithIndex(1)">Select tab 2!</button>
 *     </ion-tab>
 *     <ion-tab title="Tab 2">Hello tab 2!</ion-tab>
 *
 *   </ion-tabs>
 * </body>
 * ```
 * ```js
 * function MyCtrl($scope, $ionicTabsDelegate) {
 *   $scope.selectTabWithIndex = function(index) {
 *     $ionicTabsDelegate.select(index);
 *   }
 * }
 * ```
 */
IonicModule
.service('$ionicTabsDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicTabsDelegate#select
   * @description Select the tab matching the given index.
   *
   * @param {number} index Index of the tab to select.
   */
  'select',
  /**
   * @ngdoc method
   * @name $ionicTabsDelegate#selectedIndex
   * @returns `number` The index of the selected tab, or -1.
   */
  'selectedIndex',
  /**
   * @ngdoc method
   * @name $ionicTabsDelegate#showBar
   * @description
   * Set/get whether the {@link ionic.directive:ionTabs} is shown
   * @param {boolean} show Whether to show the bar.
   * @returns {boolean} Whether the bar is shown.
   */
  'showBar'
  /**
   * @ngdoc method
   * @name $ionicTabsDelegate#$getByHandle
   * @param {string} handle
   * @returns `delegateInstance` A delegate instance that controls only the
   * {@link ionic.directive:ionTabs} directives with `delegate-handle` matching
   * the given handle.
   *
   * Example: `$ionicTabsDelegate.$getByHandle('my-handle').select(0);`
   */
]));

// closure to keep things neat
(function() {
  var templatesToCache = [];

/**
 * @ngdoc service
 * @name $ionicTemplateCache
 * @module ionic
 * @description A service that preemptively caches template files to eliminate transition flicker and boost performance.
 * @usage
 * State templates are cached automatically, but you can optionally cache other templates.
 *
 * ```js
 * $ionicTemplateCache('myNgIncludeTemplate.html');
 * ```
 *
 * Optionally disable all preemptive caching with the `$ionicConfigProvider` or individual states by setting `prefetchTemplate`
 * in the `$state` definition
 *
 * ```js
 *   angular.module('myApp', ['ionic'])
 *   .config(function($stateProvider, $ionicConfigProvider) {
 *
 *     // disable preemptive template caching globally
 *     $ionicConfigProvider.templates.prefetch(false);
 *
 *     // disable individual states
 *     $stateProvider
 *       .state('tabs', {
 *         url: "/tab",
 *         abstract: true,
 *         prefetchTemplate: false,
 *         templateUrl: "tabs-templates/tabs.html"
 *       })
 *       .state('tabs.home', {
 *         url: "/home",
 *         views: {
 *           'home-tab': {
 *             prefetchTemplate: false,
 *             templateUrl: "tabs-templates/home.html",
 *             controller: 'HomeTabCtrl'
 *           }
 *         }
 *       });
 *   });
 * ```
 */
IonicModule
.factory('$ionicTemplateCache', [
'$http',
'$templateCache',
'$timeout',
function($http, $templateCache, $timeout) {
  var toCache = templatesToCache,
      hasRun;

  function $ionicTemplateCache(templates) {
    if (typeof templates === 'undefined') {
      return run();
    }
    if (isString(templates)) {
      templates = [templates];
    }
    forEach(templates, function(template) {
      toCache.push(template);
    });
    if (hasRun) {
      run();
    }
  }

  // run through methods - internal method
  function run() {
    var template;
    $ionicTemplateCache._runCount++;

    hasRun = true;
    // ignore if race condition already zeroed out array
    if (toCache.length === 0) return;

    var i = 0;
    while (i < 4 && (template = toCache.pop())) {
      // note that inline templates are ignored by this request
      if (isString(template)) $http.get(template, { cache: $templateCache });
      i++;
    }
    // only preload 3 templates a second
    if (toCache.length) {
      $timeout(run, 1000);
    }
  }

  // exposing for testing
  $ionicTemplateCache._runCount = 0;
  // default method
  return $ionicTemplateCache;
}])

// Intercepts the $stateprovider.state() command to look for templateUrls that can be cached
.config([
'$stateProvider',
'$ionicConfigProvider',
function($stateProvider, $ionicConfigProvider) {
  var stateProviderState = $stateProvider.state;
  $stateProvider.state = function(stateName, definition) {
    // don't even bother if it's disabled. note, another config may run after this, so it's not a catch-all
    if (typeof definition === 'object') {
      var enabled = definition.prefetchTemplate !== false && templatesToCache.length < $ionicConfigProvider.templates.maxPrefetch();
      if (enabled && isString(definition.templateUrl)) templatesToCache.push(definition.templateUrl);
      if (angular.isObject(definition.views)) {
        for (var key in definition.views) {
          enabled = definition.views[key].prefetchTemplate !== false && templatesToCache.length < $ionicConfigProvider.templates.maxPrefetch();
          if (enabled && isString(definition.views[key].templateUrl)) templatesToCache.push(definition.views[key].templateUrl);
        }
      }
    }
    return stateProviderState.call($stateProvider, stateName, definition);
  };
}])

// process the templateUrls collected by the $stateProvider, adding them to the cache
.run(['$ionicTemplateCache', function($ionicTemplateCache) {
  $ionicTemplateCache();
}]);

})();

IonicModule
.factory('$ionicTemplateLoader', [
  '$compile',
  '$controller',
  '$http',
  '$q',
  '$rootScope',
  '$templateCache',
function($compile, $controller, $http, $q, $rootScope, $templateCache) {

  return {
    load: fetchTemplate,
    compile: loadAndCompile
  };

  function fetchTemplate(url) {
    return $http.get(url, {cache: $templateCache})
    .then(function(response) {
      return response.data && response.data.trim();
    });
  }

  function loadAndCompile(options) {
    options = extend({
      template: '',
      templateUrl: '',
      scope: null,
      controller: null,
      locals: {},
      appendTo: null
    }, options || {});

    var templatePromise = options.templateUrl ?
      this.load(options.templateUrl) :
      $q.when(options.template);

    return templatePromise.then(function(template) {
      var controller;
      var scope = options.scope || $rootScope.$new();

      //Incase template doesn't have just one root element, do this
      var element = jqLite('<div>').html(template).contents();

      if (options.controller) {
        controller = $controller(
          options.controller,
          extend(options.locals, {
            $scope: scope
          })
        );
        element.children().data('$ngControllerController', controller);
      }
      if (options.appendTo) {
        jqLite(options.appendTo).append(element);
      }

      $compile(element)(scope);

      return {
        element: element,
        scope: scope
      };
    });
  }

}]);

/**
 * @private
 * DEPRECATED, as of v1.0.0-beta14 -------
 */
IonicModule
.factory('$ionicViewService', ['$ionicHistory', '$log', function($ionicHistory, $log) {

  function warn(oldMethod, newMethod) {
    $log.warn('$ionicViewService' + oldMethod + ' is deprecated, please use $ionicHistory' + newMethod + ' instead: http://ionicframework.com/docs/nightly/api/service/$ionicHistory/');
  }

  warn('', '');

  var methodsMap = {
    getCurrentView: 'currentView',
    getBackView: 'backView',
    getForwardView: 'forwardView',
    getCurrentStateName: 'currentStateName',
    nextViewOptions: 'nextViewOptions',
    clearHistory: 'clearHistory'
  };

  forEach(methodsMap, function(newMethod, oldMethod) {
    methodsMap[oldMethod] = function() {
      warn('.' + oldMethod, '.' + newMethod);
      return $ionicHistory[newMethod].apply(this, arguments);
    };
  });

  return methodsMap;

}]);

/**
 * @private
 * TODO document
 */

IonicModule.factory('$ionicViewSwitcher', [
  '$timeout',
  '$document',
  '$q',
  '$ionicClickBlock',
  '$ionicConfig',
  '$ionicNavBarDelegate',
function($timeout, $document, $q, $ionicClickBlock, $ionicConfig, $ionicNavBarDelegate) {

  var TRANSITIONEND_EVENT = 'webkitTransitionEnd transitionend';
  var DATA_NO_CACHE = '$noCache';
  var DATA_DESTROY_ELE = '$destroyEle';
  var DATA_ELE_IDENTIFIER = '$eleId';
  var DATA_VIEW_ACCESSED = '$accessed';
  var DATA_FALLBACK_TIMER = '$fallbackTimer';
  var DATA_VIEW = '$viewData';
  var NAV_VIEW_ATTR = 'nav-view';
  var VIEW_STATUS_ACTIVE = 'active';
  var VIEW_STATUS_CACHED = 'cached';
  var VIEW_STATUS_STAGED = 'stage';

  var transitionCounter = 0;
  var nextTransition, nextDirection;
  ionic.transition = ionic.transition || {};
  ionic.transition.isActive = false;
  var isActiveTimer;
  var cachedAttr = ionic.DomUtil.cachedAttr;
  var transitionPromises = [];
  var defaultTimeout = 1100;

  var ionicViewSwitcher = {

    create: function(navViewCtrl, viewLocals, enteringView, leavingView, renderStart, renderEnd) {
      // get a reference to an entering/leaving element if they exist
      // loop through to see if the view is already in the navViewElement
      var enteringEle, leavingEle;
      var transitionId = ++transitionCounter;
      var alreadyInDom;

      var switcher = {

        init: function(registerData, callback) {
          ionicViewSwitcher.isTransitioning(true);

          switcher.loadViewElements(registerData);

          switcher.render(registerData, function() {
            callback && callback();
          });
        },

        loadViewElements: function(registerData) {
          var x, l, viewEle;
          var viewElements = navViewCtrl.getViewElements();
          var enteringEleIdentifier = getViewElementIdentifier(viewLocals, enteringView);
          var navViewActiveEleId = navViewCtrl.activeEleId();

          for (x = 0, l = viewElements.length; x < l; x++) {
            viewEle = viewElements.eq(x);

            if (viewEle.data(DATA_ELE_IDENTIFIER) === enteringEleIdentifier) {
              // we found an existing element in the DOM that should be entering the view
              if (viewEle.data(DATA_NO_CACHE)) {
                // the existing element should not be cached, don't use it
                viewEle.data(DATA_ELE_IDENTIFIER, enteringEleIdentifier + ionic.Utils.nextUid());
                viewEle.data(DATA_DESTROY_ELE, true);

              } else {
                enteringEle = viewEle;
              }

            } else if (isDefined(navViewActiveEleId) && viewEle.data(DATA_ELE_IDENTIFIER) === navViewActiveEleId) {
              leavingEle = viewEle;
            }

            if (enteringEle && leavingEle) break;
          }

          alreadyInDom = !!enteringEle;

          if (!alreadyInDom) {
            // still no existing element to use
            // create it using existing template/scope/locals
            enteringEle = registerData.ele || ionicViewSwitcher.createViewEle(viewLocals);

            // existing elements in the DOM are looked up by their state name and state id
            enteringEle.data(DATA_ELE_IDENTIFIER, enteringEleIdentifier);
          }

          if (renderEnd) {
            navViewCtrl.activeEleId(enteringEleIdentifier);
          }

          registerData.ele = null;
        },

        render: function(registerData, callback) {
          if (alreadyInDom) {
            // it was already found in the DOM, just reconnect the scope
            ionic.Utils.reconnectScope(enteringEle.scope());

          } else {
            // the entering element is not already in the DOM
            // set that the entering element should be "staged" and its
            // styles of where this element will go before it hits the DOM
            navViewAttr(enteringEle, VIEW_STATUS_STAGED);

            var enteringData = getTransitionData(viewLocals, enteringEle, registerData.direction, enteringView);
            var transitionFn = $ionicConfig.transitions.views[enteringData.transition] || $ionicConfig.transitions.views.none;
            transitionFn(enteringEle, null, enteringData.direction, true).run(0);

            enteringEle.data(DATA_VIEW, {
              viewId: enteringData.viewId,
              historyId: enteringData.historyId,
              stateName: enteringData.stateName,
              stateParams: enteringData.stateParams
            });

            // if the current state has cache:false
            // or the element has cache-view="false" attribute
            if (viewState(viewLocals).cache === false || viewState(viewLocals).cache === 'false' ||
                enteringEle.attr('cache-view') == 'false' || $ionicConfig.views.maxCache() === 0) {
              enteringEle.data(DATA_NO_CACHE, true);
            }

            // append the entering element to the DOM, create a new scope and run link
            var viewScope = navViewCtrl.appendViewElement(enteringEle, viewLocals);

            delete enteringData.direction;
            delete enteringData.transition;
            viewScope.$emit('$ionicView.loaded', enteringData);
          }

          // update that this view was just accessed
          enteringEle.data(DATA_VIEW_ACCESSED, Date.now());

          callback && callback();
        },

        transition: function(direction, enableBack, allowAnimate) {
          var deferred;
          var enteringData = getTransitionData(viewLocals, enteringEle, direction, enteringView);
          var leavingData = extend(extend({}, enteringData), getViewData(leavingView));
          enteringData.transitionId = leavingData.transitionId = transitionId;
          enteringData.fromCache = !!alreadyInDom;
          enteringData.enableBack = !!enableBack;
          enteringData.renderStart = renderStart;
          enteringData.renderEnd = renderEnd;

          cachedAttr(enteringEle.parent(), 'nav-view-transition', enteringData.transition);
          cachedAttr(enteringEle.parent(), 'nav-view-direction', enteringData.direction);

          // cancel any previous transition complete fallbacks
          $timeout.cancel(enteringEle.data(DATA_FALLBACK_TIMER));

          // get the transition ready and see if it'll animate
          var transitionFn = $ionicConfig.transitions.views[enteringData.transition] || $ionicConfig.transitions.views.none;
          var viewTransition = transitionFn(enteringEle, leavingEle, enteringData.direction,
                                            enteringData.shouldAnimate && allowAnimate && renderEnd);

          if (viewTransition.shouldAnimate) {
            // attach transitionend events (and fallback timer)
            enteringEle.on(TRANSITIONEND_EVENT, completeOnTransitionEnd);
            enteringEle.data(DATA_FALLBACK_TIMER, $timeout(transitionComplete, defaultTimeout));
            $ionicClickBlock.show(defaultTimeout);
          }

          if (renderStart) {
            // notify the views "before" the transition starts
            switcher.emit('before', enteringData, leavingData);

            // stage entering element, opacity 0, no transition duration
            navViewAttr(enteringEle, VIEW_STATUS_STAGED);

            // render the elements in the correct location for their starting point
            viewTransition.run(0);
          }

          if (renderEnd) {
            // create a promise so we can keep track of when all transitions finish
            // only required if this transition should complete
            deferred = $q.defer();
            transitionPromises.push(deferred.promise);
          }

          if (renderStart && renderEnd) {
            // CSS "auto" transitioned, not manually transitioned
            // wait a frame so the styles apply before auto transitioning
            $timeout(function() {
              ionic.requestAnimationFrame(onReflow);
            });
          } else if (!renderEnd) {
            // just the start of a manual transition
            // but it will not render the end of the transition
            navViewAttr(enteringEle, 'entering');
            navViewAttr(leavingEle, 'leaving');

            // return the transition run method so each step can be ran manually
            return {
              run: viewTransition.run,
              cancel: function(shouldAnimate) {
                if (shouldAnimate) {
                  enteringEle.on(TRANSITIONEND_EVENT, cancelOnTransitionEnd);
                  enteringEle.data(DATA_FALLBACK_TIMER, $timeout(cancelTransition, defaultTimeout));
                  $ionicClickBlock.show(defaultTimeout);
                } else {
                  cancelTransition();
                }
                viewTransition.shouldAnimate = shouldAnimate;
                viewTransition.run(0);
                viewTransition = null;
              }
            };

          } else if (renderEnd) {
            // just the end of a manual transition
            // happens after the manual transition has completed
            // and a full history change has happened
            onReflow();
          }


          function onReflow() {
            // remove that we're staging the entering element so it can auto transition
            navViewAttr(enteringEle, viewTransition.shouldAnimate ? 'entering' : VIEW_STATUS_ACTIVE);
            navViewAttr(leavingEle, viewTransition.shouldAnimate ? 'leaving' : VIEW_STATUS_CACHED);

            // start the auto transition and let the CSS take over
            viewTransition.run(1);

            // trigger auto transitions on the associated nav bars
            $ionicNavBarDelegate._instances.forEach(function(instance) {
              instance.triggerTransitionStart(transitionId);
            });

            if (!viewTransition.shouldAnimate) {
              // no animated auto transition
              transitionComplete();
            }
          }

          // Make sure that transitionend events bubbling up from children won't fire
          // transitionComplete. Will only go forward if ev.target == the element listening.
          function completeOnTransitionEnd(ev) {
            if (ev.target !== this) return;
            transitionComplete();
          }
          function transitionComplete() {
            if (transitionComplete.x) return;
            transitionComplete.x = true;

            enteringEle.off(TRANSITIONEND_EVENT, completeOnTransitionEnd);
            $timeout.cancel(enteringEle.data(DATA_FALLBACK_TIMER));
            leavingEle && $timeout.cancel(leavingEle.data(DATA_FALLBACK_TIMER));

            // resolve that this one transition (there could be many w/ nested views)
            deferred && deferred.resolve(navViewCtrl);

            // the most recent transition added has completed and all the active
            // transition promises should be added to the services array of promises
            if (transitionId === transitionCounter) {
              $q.all(transitionPromises).then(ionicViewSwitcher.transitionEnd);

              // emit that the views have finished transitioning
              // each parent nav-view will update which views are active and cached
              switcher.emit('after', enteringData, leavingData);
              switcher.cleanup(enteringData);
            }

            // tell the nav bars that the transition has ended
            $ionicNavBarDelegate._instances.forEach(function(instance) {
              instance.triggerTransitionEnd();
            });


            // remove any references that could cause memory issues
            nextTransition = nextDirection = enteringView = leavingView = enteringEle = leavingEle = null;
          }

          // Make sure that transitionend events bubbling up from children won't fire
          // transitionComplete. Will only go forward if ev.target == the element listening.
          function cancelOnTransitionEnd(ev) {
            if (ev.target !== this) return;
            cancelTransition();
          }
          function cancelTransition() {
            navViewAttr(enteringEle, VIEW_STATUS_CACHED);
            navViewAttr(leavingEle, VIEW_STATUS_ACTIVE);
            enteringEle.off(TRANSITIONEND_EVENT, cancelOnTransitionEnd);
            $timeout.cancel(enteringEle.data(DATA_FALLBACK_TIMER));
            ionicViewSwitcher.transitionEnd([navViewCtrl]);
          }

        },

      emit: function(step, enteringData, leavingData) {
          var enteringScope = getScopeForElement(enteringEle, enteringData);
          var leavingScope = getScopeForElement(leavingEle, leavingData);

          var prefixesAreEqual;

          if ( !enteringData.viewId || enteringData.abstractView ) {
            // it's an abstract view, so treat it accordingly

            // we only get access to the leaving scope once in the transition,
            // so dispatch all events right away if it exists
            if ( leavingScope ) {
              leavingScope.$emit('$ionicView.beforeLeave', leavingData);
              leavingScope.$emit('$ionicView.leave', leavingData);
              leavingScope.$emit('$ionicView.afterLeave', leavingData);
              leavingScope.$broadcast('$ionicParentView.beforeLeave', leavingData);
              leavingScope.$broadcast('$ionicParentView.leave', leavingData);
              leavingScope.$broadcast('$ionicParentView.afterLeave', leavingData);
            }
          }
          else {
            // it's a regular view, so do the normal process
            if (step == 'after') {
              if (enteringScope) {
                enteringScope.$emit('$ionicView.enter', enteringData);
                enteringScope.$broadcast('$ionicParentView.enter', enteringData);
              }

              if (leavingScope) {
                leavingScope.$emit('$ionicView.leave', leavingData);
                leavingScope.$broadcast('$ionicParentView.leave', leavingData);
              }
              else if (enteringScope && leavingData && leavingData.viewId && enteringData.stateName !== leavingData.stateName) {
                // we only want to dispatch this when we are doing a single-tier
                // state change such as changing a tab, so compare the state
                // for the same state-prefix but different suffix
                prefixesAreEqual = compareStatePrefixes(enteringData.stateName, leavingData.stateName);
                if ( prefixesAreEqual ) {
                  enteringScope.$emit('$ionicNavView.leave', leavingData);
                }
              }
            }

            if (enteringScope) {
              enteringScope.$emit('$ionicView.' + step + 'Enter', enteringData);
              enteringScope.$broadcast('$ionicParentView.' + step + 'Enter', enteringData);
            }

            if (leavingScope) {
              leavingScope.$emit('$ionicView.' + step + 'Leave', leavingData);
              leavingScope.$broadcast('$ionicParentView.' + step + 'Leave', leavingData);

            } else if (enteringScope && leavingData && leavingData.viewId && enteringData.stateName !== leavingData.stateName) {
              // we only want to dispatch this when we are doing a single-tier
              // state change such as changing a tab, so compare the state
              // for the same state-prefix but different suffix
              prefixesAreEqual = compareStatePrefixes(enteringData.stateName, leavingData.stateName);
              if ( prefixesAreEqual ) {
                enteringScope.$emit('$ionicNavView.' + step + 'Leave', leavingData);
              }
            }
          }
        },

        cleanup: function(transData) {
          // check if any views should be removed
          if (leavingEle && transData.direction == 'back' && !$ionicConfig.views.forwardCache()) {
            // if they just navigated back we can destroy the forward view
            // do not remove forward views if cacheForwardViews config is true
            destroyViewEle(leavingEle);
          }

          var viewElements = navViewCtrl.getViewElements();
          var viewElementsLength = viewElements.length;
          var x, viewElement;
          var removeOldestAccess = (viewElementsLength - 1) > $ionicConfig.views.maxCache();
          var removableEle;
          var oldestAccess = Date.now();

          for (x = 0; x < viewElementsLength; x++) {
            viewElement = viewElements.eq(x);

            if (removeOldestAccess && viewElement.data(DATA_VIEW_ACCESSED) < oldestAccess) {
              // remember what was the oldest element to be accessed so it can be destroyed
              oldestAccess = viewElement.data(DATA_VIEW_ACCESSED);
              removableEle = viewElements.eq(x);

            } else if (viewElement.data(DATA_DESTROY_ELE) && navViewAttr(viewElement) != VIEW_STATUS_ACTIVE) {
              destroyViewEle(viewElement);
            }
          }

          destroyViewEle(removableEle);

          if (enteringEle.data(DATA_NO_CACHE)) {
            enteringEle.data(DATA_DESTROY_ELE, true);
          }
        },

        enteringEle: function() { return enteringEle; },
        leavingEle: function() { return leavingEle; }

      };

      return switcher;
    },

    transitionEnd: function(navViewCtrls) {
      forEach(navViewCtrls, function(navViewCtrl) {
        navViewCtrl.transitionEnd();
      });

      ionicViewSwitcher.isTransitioning(false);
      $ionicClickBlock.hide();
      transitionPromises = [];
    },

    nextTransition: function(val) {
      nextTransition = val;
    },

    nextDirection: function(val) {
      nextDirection = val;
    },

    isTransitioning: function(val) {
      if (arguments.length) {
        ionic.transition.isActive = !!val;
        $timeout.cancel(isActiveTimer);
        if (val) {
          isActiveTimer = $timeout(function() {
            ionicViewSwitcher.isTransitioning(false);
          }, 999);
        }
      }
      return ionic.transition.isActive;
    },

    createViewEle: function(viewLocals) {
      var containerEle = $document[0].createElement('div');
      if (viewLocals && viewLocals.$template) {
        containerEle.innerHTML = viewLocals.$template;
        if (containerEle.children.length === 1) {
          containerEle.children[0].classList.add('pane');
          if ( viewLocals.$$state && viewLocals.$$state.self && viewLocals.$$state.self['abstract'] ) {
            angular.element(containerEle.children[0]).attr("abstract", "true");
          }
          else {
            if ( viewLocals.$$state && viewLocals.$$state.self ) {
              angular.element(containerEle.children[0]).attr("state", viewLocals.$$state.self.name);
            }

          }
          return jqLite(containerEle.children[0]);
        }
      }
      containerEle.className = "pane";
      return jqLite(containerEle);
    },

    viewEleIsActive: function(viewEle, isActiveAttr) {
      navViewAttr(viewEle, isActiveAttr ? VIEW_STATUS_ACTIVE : VIEW_STATUS_CACHED);
    },

    getTransitionData: getTransitionData,
    navViewAttr: navViewAttr,
    destroyViewEle: destroyViewEle

  };

  return ionicViewSwitcher;


  function getViewElementIdentifier(locals, view) {
    if (viewState(locals)['abstract']) return viewState(locals).name;
    if (view) return view.stateId || view.viewId;
    return ionic.Utils.nextUid();
  }

  function viewState(locals) {
    return locals && locals.$$state && locals.$$state.self || {};
  }

  function getTransitionData(viewLocals, enteringEle, direction, view) {
    // Priority
    // 1) attribute directive on the button/link to this view
    // 2) entering element's attribute
    // 3) entering view's $state config property
    // 4) view registration data
    // 5) global config
    // 6) fallback value

    var state = viewState(viewLocals);
    var viewTransition = nextTransition || cachedAttr(enteringEle, 'view-transition') || state.viewTransition || $ionicConfig.views.transition() || 'ios';
    var navBarTransition = $ionicConfig.navBar.transition();
    direction = nextDirection || cachedAttr(enteringEle, 'view-direction') || state.viewDirection || direction || 'none';

    return extend(getViewData(view), {
      transition: viewTransition,
      navBarTransition: navBarTransition === 'view' ? viewTransition : navBarTransition,
      direction: direction,
      shouldAnimate: (viewTransition !== 'none' && direction !== 'none')
    });
  }

  function getViewData(view) {
    view = view || {};
    return {
      viewId: view.viewId,
      historyId: view.historyId,
      stateId: view.stateId,
      stateName: view.stateName,
      stateParams: view.stateParams
    };
  }

  function navViewAttr(ele, value) {
    if (arguments.length > 1) {
      cachedAttr(ele, NAV_VIEW_ATTR, value);
    } else {
      return cachedAttr(ele, NAV_VIEW_ATTR);
    }
  }

  function destroyViewEle(ele) {
    // we found an element that should be removed
    // destroy its scope, then remove the element
    if (ele && ele.length) {
      var viewScope = ele.scope();
      if (viewScope) {
        viewScope.$emit('$ionicView.unloaded', ele.data(DATA_VIEW));
        viewScope.$destroy();
      }
      ele.remove();
    }
  }

  function compareStatePrefixes(enteringStateName, exitingStateName) {
    var enteringStateSuffixIndex = enteringStateName.lastIndexOf('.');
    var exitingStateSuffixIndex = exitingStateName.lastIndexOf('.');

    // if either of the prefixes are empty, just return false
    if ( enteringStateSuffixIndex < 0 || exitingStateSuffixIndex < 0 ) {
      return false;
    }

    var enteringPrefix = enteringStateName.substring(0, enteringStateSuffixIndex);
    var exitingPrefix = exitingStateName.substring(0, exitingStateSuffixIndex);

    return enteringPrefix === exitingPrefix;
  }

  function getScopeForElement(element, stateData) {
    if ( !element ) {
      return null;
    }
    // check if it's abstract
    var attributeValue = angular.element(element).attr("abstract");
    var stateValue = angular.element(element).attr("state");

    if ( attributeValue !== "true" ) {
      // it's not an abstract view, so make sure the element
      // matches the state.  Due to abstract view weirdness,
      // sometimes it doesn't. If it doesn't, don't dispatch events
      // so leave the scope undefined
      if ( stateValue === stateData.stateName ) {
        return angular.element(element).scope();
      }
      return null;
    }
    else {
      // it is an abstract element, so look for element with the "state" attributeValue
      // set to the name of the stateData state
      var elements = aggregateNavViewChildren(element);
      for ( var i = 0; i < elements.length; i++ ) {
          var state = angular.element(elements[i]).attr("state");
          if ( state === stateData.stateName ) {
            stateData.abstractView = true;
            return angular.element(elements[i]).scope();
          }
      }
      // we didn't find a match, so return null
      return null;
    }
  }

  function aggregateNavViewChildren(element) {
    var aggregate = [];
    var navViews = angular.element(element).find("ion-nav-view");
    for ( var i = 0; i < navViews.length; i++ ) {
      var children = angular.element(navViews[i]).children();
      var childrenAggregated = [];
      for ( var j = 0; j < children.length; j++ ) {
        childrenAggregated = childrenAggregated.concat(children[j]);
      }
      aggregate = aggregate.concat(childrenAggregated);
    }
    return aggregate;
  }

}]);

/**
 * ==================  angular-ios9-uiwebview.patch.js v1.1.1 ==================
 *
 * This patch works around iOS9 UIWebView regression that causes infinite digest
 * errors in Angular.
 *
 * The patch can be applied to Angular 1.2.0 – 1.4.5. Newer versions of Angular
 * have the workaround baked in.
 *
 * To apply this patch load/bundle this file with your application and add a
 * dependency on the "ngIOS9UIWebViewPatch" module to your main app module.
 *
 * For example:
 *
 * ```
 * angular.module('myApp', ['ngRoute'])`
 * ```
 *
 * becomes
 *
 * ```
 * angular.module('myApp', ['ngRoute', 'ngIOS9UIWebViewPatch'])
 * ```
 *
 *
 * More info:
 * - https://openradar.appspot.com/22186109
 * - https://github.com/angular/angular.js/issues/12241
 * - https://github.com/ionic-team/ionic/issues/4082
 *
 *
 * @license AngularJS
 * (c) 2010-2015 Google, Inc. http://angularjs.org
 * License: MIT
 */

angular.module('ngIOS9UIWebViewPatch', ['ng']).config(['$provide', function($provide) {
  'use strict';

  $provide.decorator('$browser', ['$delegate', '$window', function($delegate, $window) {

    if (isIOS9UIWebView($window.navigator.userAgent)) {
      return applyIOS9Shim($delegate);
    }

    return $delegate;

    function isIOS9UIWebView(userAgent) {
      return /(iPhone|iPad|iPod).* OS 9_\d/.test(userAgent) && !/Version\/9\./.test(userAgent);
    }

    function applyIOS9Shim(browser) {
      var pendingLocationUrl = null;
      var originalUrlFn = browser.url;

      browser.url = function() {
        if (arguments.length) {
          pendingLocationUrl = arguments[0];
          return originalUrlFn.apply(browser, arguments);
        }

        return pendingLocationUrl || originalUrlFn.apply(browser, arguments);
      };

      window.addEventListener('popstate', clearPendingLocationUrl, false);
      window.addEventListener('hashchange', clearPendingLocationUrl, false);

      function clearPendingLocationUrl() {
        pendingLocationUrl = null;
      }

      return browser;
    }
  }]);
}]);

/**
 * @private
 * Parts of Ionic requires that $scope data is attached to the element.
 * We do not want to disable adding $scope data to the $element when
 * $compileProvider.debugInfoEnabled(false) is used.
 */
IonicModule.config(['$provide', function($provide) {
  $provide.decorator('$compile', ['$delegate', function($compile) {
     $compile.$$addScopeInfo = function $$addScopeInfo($element, scope, isolated, noTemplate) {
       var dataName = isolated ? (noTemplate ? '$isolateScopeNoTemplate' : '$isolateScope') : '$scope';
       $element.data(dataName, scope);
     };
     return $compile;
  }]);
}]);

/**
 * @private
 */
IonicModule.config([
  '$provide',
function($provide) {
  function $LocationDecorator($location, $timeout) {

    $location.__hash = $location.hash;
    //Fix: when window.location.hash is set, the scrollable area
    //found nearest to body's scrollTop is set to scroll to an element
    //with that ID.
    $location.hash = function(value) {
      if (isDefined(value) && value.length > 0) {
        $timeout(function() {
          var scroll = document.querySelector('.scroll-content');
          if (scroll) {
            scroll.scrollTop = 0;
          }
        }, 0, false);
      }
      return $location.__hash(value);
    };

    return $location;
  }

  $provide.decorator('$location', ['$delegate', '$timeout', $LocationDecorator]);
}]);

IonicModule

.controller('$ionicHeaderBar', [
  '$scope',
  '$element',
  '$attrs',
  '$q',
  '$ionicConfig',
  '$ionicHistory',
function($scope, $element, $attrs, $q, $ionicConfig, $ionicHistory) {
  var TITLE = 'title';
  var BACK_TEXT = 'back-text';
  var BACK_BUTTON = 'back-button';
  var DEFAULT_TITLE = 'default-title';
  var PREVIOUS_TITLE = 'previous-title';
  var HIDE = 'hide';

  var self = this;
  var titleText = '';
  var previousTitleText = '';
  var titleLeft = 0;
  var titleRight = 0;
  var titleCss = '';
  var isBackEnabled = false;
  var isBackShown = true;
  var isNavBackShown = true;
  var isBackElementShown = false;
  var titleTextWidth = 0;


  self.beforeEnter = function(viewData) {
    $scope.$broadcast('$ionicView.beforeEnter', viewData);
  };


  self.title = function(newTitleText) {
    if (arguments.length && newTitleText !== titleText) {
      getEle(TITLE).innerHTML = newTitleText;
      titleText = newTitleText;
      titleTextWidth = 0;
    }
    return titleText;
  };


  self.enableBack = function(shouldEnable, disableReset) {
    // whether or not the back button show be visible, according
    // to the navigation and history
    if (arguments.length) {
      isBackEnabled = shouldEnable;
      if (!disableReset) self.updateBackButton();
    }
    return isBackEnabled;
  };


  self.showBack = function(shouldShow, disableReset) {
    // different from enableBack() because this will always have the back
    // visually hidden if false, even if the history says it should show
    if (arguments.length) {
      isBackShown = shouldShow;
      if (!disableReset) self.updateBackButton();
    }
    return isBackShown;
  };


  self.showNavBack = function(shouldShow) {
    // different from showBack() because this is for the entire nav bar's
    // setting for all of it's child headers. For internal use.
    isNavBackShown = shouldShow;
    self.updateBackButton();
  };


  self.updateBackButton = function() {
    var ele;
    if ((isBackShown && isNavBackShown && isBackEnabled) !== isBackElementShown) {
      isBackElementShown = isBackShown && isNavBackShown && isBackEnabled;
      ele = getEle(BACK_BUTTON);
      ele && ele.classList[ isBackElementShown ? 'remove' : 'add' ](HIDE);
    }

    if (isBackEnabled) {
      ele = ele || getEle(BACK_BUTTON);
      if (ele) {
        if (self.backButtonIcon !== $ionicConfig.backButton.icon()) {
          ele = getEle(BACK_BUTTON + ' .icon');
          if (ele) {
            self.backButtonIcon = $ionicConfig.backButton.icon();
            ele.className = 'icon ' + self.backButtonIcon;
          }
        }

        if (self.backButtonText !== $ionicConfig.backButton.text()) {
          ele = getEle(BACK_BUTTON + ' .back-text');
          if (ele) {
            ele.textContent = self.backButtonText = $ionicConfig.backButton.text();
          }
        }
      }
    }
  };


  self.titleTextWidth = function() {
    var element = getEle(TITLE);
    if ( element ) {
      // If the element has a nav-bar-title, use that instead
      // to calculate the width of the title
      var children = angular.element(element).children();
      for ( var i = 0; i < children.length; i++ ) {
        if ( angular.element(children[i]).hasClass('nav-bar-title') ) {
          element = children[i];
          break;
        }
      }
    }
    var bounds = ionic.DomUtil.getTextBounds(element);
    titleTextWidth = Math.min(bounds && bounds.width || 30);
    return titleTextWidth;
  };


  self.titleWidth = function() {
    var titleWidth = self.titleTextWidth();
    var offsetWidth = getEle(TITLE).offsetWidth;
    if (offsetWidth < titleWidth) {
      titleWidth = offsetWidth + (titleLeft - titleRight - 5);
    }
    return titleWidth;
  };


  self.titleTextX = function() {
    return ($element[0].offsetWidth / 2) - (self.titleWidth() / 2);
  };


  self.titleLeftRight = function() {
    return titleLeft - titleRight;
  };


  self.backButtonTextLeft = function() {
    var offsetLeft = 0;
    var ele = getEle(BACK_TEXT);
    while (ele) {
      offsetLeft += ele.offsetLeft;
      ele = ele.parentElement;
    }
    return offsetLeft;
  };


  self.resetBackButton = function(viewData) {
    if ($ionicConfig.backButton.previousTitleText()) {
      var previousTitleEle = getEle(PREVIOUS_TITLE);
      if (previousTitleEle) {
        previousTitleEle.classList.remove(HIDE);

        var view = (viewData && $ionicHistory.getViewById(viewData.viewId));
        var newPreviousTitleText = $ionicHistory.backTitle(view);

        if (newPreviousTitleText !== previousTitleText) {
          previousTitleText = previousTitleEle.innerHTML = newPreviousTitleText;
        }
      }
      var defaultTitleEle = getEle(DEFAULT_TITLE);
      if (defaultTitleEle) {
        defaultTitleEle.classList.remove(HIDE);
      }
    }
  };


  self.align = function(textAlign) {
    var titleEle = getEle(TITLE);

    textAlign = textAlign || $attrs.alignTitle || $ionicConfig.navBar.alignTitle();

    var widths = self.calcWidths(textAlign, false);

    if (isBackShown && previousTitleText && $ionicConfig.backButton.previousTitleText()) {
      var previousTitleWidths = self.calcWidths(textAlign, true);

      var availableTitleWidth = $element[0].offsetWidth - previousTitleWidths.titleLeft - previousTitleWidths.titleRight;

      if (self.titleTextWidth() <= availableTitleWidth) {
        widths = previousTitleWidths;
      }
    }

    return self.updatePositions(titleEle, widths.titleLeft, widths.titleRight, widths.buttonsLeft, widths.buttonsRight, widths.css, widths.showPrevTitle);
  };


  self.calcWidths = function(textAlign, isPreviousTitle) {
    var titleEle = getEle(TITLE);
    var backBtnEle = getEle(BACK_BUTTON);
    var x, y, z, b, c, d, childSize, bounds;
    var childNodes = $element[0].childNodes;
    var buttonsLeft = 0;
    var buttonsRight = 0;
    var isCountRightOfTitle;
    var updateTitleLeft = 0;
    var updateTitleRight = 0;
    var updateCss = '';
    var backButtonWidth = 0;

    // Compute how wide the left children are
    // Skip all titles (there may still be two titles, one leaving the dom)
    // Once we encounter a titleEle, realize we are now counting the right-buttons, not left
    for (x = 0; x < childNodes.length; x++) {
      c = childNodes[x];

      childSize = 0;
      if (c.nodeType == 1) {
        // element node
        if (c === titleEle) {
          isCountRightOfTitle = true;
          continue;
        }

        if (c.classList.contains(HIDE)) {
          continue;
        }

        if (isBackShown && c === backBtnEle) {

          for (y = 0; y < c.childNodes.length; y++) {
            b = c.childNodes[y];

            if (b.nodeType == 1) {

              if (b.classList.contains(BACK_TEXT)) {
                for (z = 0; z < b.children.length; z++) {
                  d = b.children[z];

                  if (isPreviousTitle) {
                    if (d.classList.contains(DEFAULT_TITLE)) continue;
                    backButtonWidth += d.offsetWidth;
                  } else {
                    if (d.classList.contains(PREVIOUS_TITLE)) continue;
                    backButtonWidth += d.offsetWidth;
                  }
                }

              } else {
                backButtonWidth += b.offsetWidth;
              }

            } else if (b.nodeType == 3 && b.nodeValue.trim()) {
              bounds = ionic.DomUtil.getTextBounds(b);
              backButtonWidth += bounds && bounds.width || 0;
            }

          }
          childSize = backButtonWidth || c.offsetWidth;

        } else {
          // not the title, not the back button, not a hidden element
          childSize = c.offsetWidth;
        }

      } else if (c.nodeType == 3 && c.nodeValue.trim()) {
        // text node
        bounds = ionic.DomUtil.getTextBounds(c);
        childSize = bounds && bounds.width || 0;
      }

      if (isCountRightOfTitle) {
        buttonsRight += childSize;
      } else {
        buttonsLeft += childSize;
      }
    }

    // Size and align the header titleEle based on the sizes of the left and
    // right children, and the desired alignment mode
    if (textAlign == 'left') {
      updateCss = 'title-left';
      if (buttonsLeft) {
        updateTitleLeft = buttonsLeft + 15;
      }
      if (buttonsRight) {
        updateTitleRight = buttonsRight + 15;
      }

    } else if (textAlign == 'right') {
      updateCss = 'title-right';
      if (buttonsLeft) {
        updateTitleLeft = buttonsLeft + 15;
      }
      if (buttonsRight) {
        updateTitleRight = buttonsRight + 15;
      }

    } else {
      // center the default
      var margin = Math.max(buttonsLeft, buttonsRight) + 10;
      if (margin > 10) {
        updateTitleLeft = updateTitleRight = margin;
      }
    }

    return {
      backButtonWidth: backButtonWidth,
      buttonsLeft: buttonsLeft,
      buttonsRight: buttonsRight,
      titleLeft: updateTitleLeft,
      titleRight: updateTitleRight,
      showPrevTitle: isPreviousTitle,
      css: updateCss
    };
  };


  self.updatePositions = function(titleEle, updateTitleLeft, updateTitleRight, buttonsLeft, buttonsRight, updateCss, showPreviousTitle) {
    var deferred = $q.defer();

    // only make DOM updates when there are actual changes
    if (titleEle) {
      if (updateTitleLeft !== titleLeft) {
        titleEle.style.left = updateTitleLeft ? updateTitleLeft + 'px' : '';
        titleLeft = updateTitleLeft;
      }
      if (updateTitleRight !== titleRight) {
        titleEle.style.right = updateTitleRight ? updateTitleRight + 'px' : '';
        titleRight = updateTitleRight;
      }

      if (updateCss !== titleCss) {
        updateCss && titleEle.classList.add(updateCss);
        titleCss && titleEle.classList.remove(titleCss);
        titleCss = updateCss;
      }
    }

    if ($ionicConfig.backButton.previousTitleText()) {
      var prevTitle = getEle(PREVIOUS_TITLE);
      var defaultTitle = getEle(DEFAULT_TITLE);

      prevTitle && prevTitle.classList[ showPreviousTitle ? 'remove' : 'add'](HIDE);
      defaultTitle && defaultTitle.classList[ showPreviousTitle ? 'add' : 'remove'](HIDE);
    }

    ionic.requestAnimationFrame(function() {
      if (titleEle && titleEle.offsetWidth + 10 < titleEle.scrollWidth) {
        var minRight = buttonsRight + 5;
        var testRight = $element[0].offsetWidth - titleLeft - self.titleTextWidth() - 20;
        updateTitleRight = testRight < minRight ? minRight : testRight;
        if (updateTitleRight !== titleRight) {
          titleEle.style.right = updateTitleRight + 'px';
          titleRight = updateTitleRight;
        }
      }
      deferred.resolve();
    });

    return deferred.promise;
  };


  self.setCss = function(elementClassname, css) {
    ionic.DomUtil.cachedStyles(getEle(elementClassname), css);
  };


  var eleCache = {};
  function getEle(className) {
    if (!eleCache[className]) {
      eleCache[className] = $element[0].querySelector('.' + className);
    }
    return eleCache[className];
  }


  $scope.$on('$destroy', function() {
    for (var n in eleCache) eleCache[n] = null;
  });

}]);

IonicModule
.controller('$ionInfiniteScroll', [
  '$scope',
  '$attrs',
  '$element',
  '$timeout',
function($scope, $attrs, $element, $timeout) {
  var self = this;
  self.isLoading = false;

  $scope.icon = function() {
    return isDefined($attrs.icon) ? $attrs.icon : 'ion-load-d';
  };

  $scope.spinner = function() {
    return isDefined($attrs.spinner) ? $attrs.spinner : '';
  };

  $scope.$on('scroll.infiniteScrollComplete', function() {
    finishInfiniteScroll();
  });

  $scope.$on('$destroy', function() {
    if (self.scrollCtrl && self.scrollCtrl.$element) self.scrollCtrl.$element.off('scroll', self.checkBounds);
    if (self.scrollEl && self.scrollEl.removeEventListener) {
      self.scrollEl.removeEventListener('scroll', self.checkBounds);
    }
  });

  // debounce checking infinite scroll events
  self.checkBounds = ionic.Utils.throttle(checkInfiniteBounds, 300);

  function onInfinite() {
    ionic.requestAnimationFrame(function() {
      $element[0].classList.add('active');
    });
    self.isLoading = true;
    $scope.$parent && $scope.$parent.$apply($attrs.onInfinite || '');
  }

  function finishInfiniteScroll() {
    ionic.requestAnimationFrame(function() {
      $element[0].classList.remove('active');
    });
    $timeout(function() {
      if (self.jsScrolling) self.scrollView.resize();
      // only check bounds again immediately if the page isn't cached (scroll el has height)
      if ((self.jsScrolling && self.scrollView.__container && self.scrollView.__container.offsetHeight > 0) ||
      !self.jsScrolling) {
        self.checkBounds();
      }
    }, 30, false);
    self.isLoading = false;
  }

  // check if we've scrolled far enough to trigger an infinite scroll
  function checkInfiniteBounds() {
    if (self.isLoading) return;
    var maxScroll = {};

    if (self.jsScrolling) {
      maxScroll = self.getJSMaxScroll();
      var scrollValues = self.scrollView.getValues();
      if ((maxScroll.left !== -1 && scrollValues.left >= maxScroll.left) ||
        (maxScroll.top !== -1 && scrollValues.top >= maxScroll.top)) {
        onInfinite();
      }
    } else {
      maxScroll = self.getNativeMaxScroll();
      if ((
        maxScroll.left !== -1 &&
        self.scrollEl.scrollLeft >= maxScroll.left - self.scrollEl.clientWidth
        ) || (
        maxScroll.top !== -1 &&
        self.scrollEl.scrollTop >= maxScroll.top - self.scrollEl.clientHeight
        )) {
        onInfinite();
      }
    }
  }

  // determine the threshold at which we should fire an infinite scroll
  // note: this gets processed every scroll event, can it be cached?
  self.getJSMaxScroll = function() {
    var maxValues = self.scrollView.getScrollMax();
    return {
      left: self.scrollView.options.scrollingX ?
        calculateMaxValue(maxValues.left) :
        -1,
      top: self.scrollView.options.scrollingY ?
        calculateMaxValue(maxValues.top) :
        -1
    };
  };

  self.getNativeMaxScroll = function() {
    var maxValues = {
      left: self.scrollEl.scrollWidth,
      top: self.scrollEl.scrollHeight
    };
    var computedStyle = window.getComputedStyle(self.scrollEl) || {};
    return {
      left: maxValues.left &&
        (computedStyle.overflowX === 'scroll' ||
        computedStyle.overflowX === 'auto' ||
        self.scrollEl.style['overflow-x'] === 'scroll') ?
        calculateMaxValue(maxValues.left) : -1,
      top: maxValues.top &&
        (computedStyle.overflowY === 'scroll' ||
        computedStyle.overflowY === 'auto' ||
        self.scrollEl.style['overflow-y'] === 'scroll' ) ?
        calculateMaxValue(maxValues.top) : -1
    };
  };

  // determine pixel refresh distance based on % or value
  function calculateMaxValue(maximum) {
    var distance = ($attrs.distance || '2.5%').trim();
    var isPercent = distance.indexOf('%') !== -1;
    return isPercent ?
    maximum * (1 - parseFloat(distance) / 100) :
    maximum - parseFloat(distance);
  }

  //for testing
  self.__finishInfiniteScroll = finishInfiniteScroll;

}]);

/**
 * @ngdoc service
 * @name $ionicListDelegate
 * @module ionic
 *
 * @description
 * Delegate for controlling the {@link ionic.directive:ionList} directive.
 *
 * Methods called directly on the $ionicListDelegate service will control all lists.
 * Use the {@link ionic.service:$ionicListDelegate#$getByHandle $getByHandle}
 * method to control specific ionList instances.
 *
 * @usage
 * ```html
 * {% raw %}
 * <ion-content ng-controller="MyCtrl">
 *   <button class="button" ng-click="showDeleteButtons()"></button>
 *   <ion-list>
 *     <ion-item ng-repeat="i in items">
 *       Hello, {{i}}!
 *       <ion-delete-button class="ion-minus-circled"></ion-delete-button>
 *     </ion-item>
 *   </ion-list>
 * </ion-content>
 * {% endraw %}
 * ```

 * ```js
 * function MyCtrl($scope, $ionicListDelegate) {
 *   $scope.showDeleteButtons = function() {
 *     $ionicListDelegate.showDelete(true);
 *   };
 * }
 * ```
 */
IonicModule.service('$ionicListDelegate', ionic.DelegateService([
  /**
   * @ngdoc method
   * @name $ionicListDelegate#showReorder
   * @param {boolean=} showReorder Set whether or not this list is showing its reorder buttons.
   * @returns {boolean} Whether the reorder buttons are shown.
   */
  'showReorder',
  /**
   * @ngdoc method
   * @name $ionicListDelegate#showDelete
   * @param {boolean=} showDelete Set whether or not this list is showing its delete buttons.
   * @returns {boolean} Whether the delete buttons are shown.
   */
  'showDelete',
  /**
   * @ngdoc method
   * @name $ionicListDelegate#canSwipeItems
   * @param {boolean=} canSwipeItems Set whether or not this list is able to swipe to show
   * option buttons.
   * @returns {boolean} Whether the list is able to swipe to show option buttons.
   */
  'canSwipeItems',
  /**
   * @ngdoc method
   * @name $ionicListDelegate#closeOptionButtons
   * @description Closes any option buttons on the list that are swiped open.
   */
  'closeOptionButtons'
  /**
   * @ngdoc method
   * @name $ionicListDelegate#$getByHandle
   * @param {string} handle
   * @returns `delegateInstance` A delegate instance that controls only the
   * {@link ionic.directive:ionList} directives with `delegate-handle` matching
   * the given handle.
   *
   * Example: `$ionicListDelegate.$getByHandle('my-handle').showReorder(true);`
   */
]))

.controller('$ionicList', [
  '$scope',
  '$attrs',
  '$ionicListDelegate',
  '$ionicHistory',
function($scope, $attrs, $ionicListDelegate, $ionicHistory) {
  var self = this;
  var isSwipeable = true;
  var isReorderShown = false;
  var isDeleteShown = false;

  var deregisterInstance = $ionicListDelegate._registerInstance(
    self, $attrs.delegateHandle, function() {
      return $ionicHistory.isActiveScope($scope);
    }
  );
  $scope.$on('$destroy', deregisterInstance);

  self.showReorder = function(show) {
    if (arguments.length) {
      isReorderShown = !!show;
    }
    return isReorderShown;
  };

  self.showDelete = function(show) {
    if (arguments.length) {
      isDeleteShown = !!show;
    }
    return isDeleteShown;
  };

  self.canSwipeItems = function(can) {
    if (arguments.length) {
      isSwipeable = !!can;
    }
    return isSwipeable;
  };

  self.closeOptionButtons = function() {
    self.listView && self.listView.clearDragEffects();
  };
}]);

IonicModule

.controller('$ionicNavBar', [
  '$scope',
  '$element',
  '$attrs',
  '$compile',
  '$timeout',
  '$ionicNavBarDelegate',
  '$ionicConfig',
  '$ionicHistory',
function($scope, $element, $attrs, $compile, $timeout, $ionicNavBarDelegate, $ionicConfig, $ionicHistory) {

  var CSS_HIDE = 'hide';
  var DATA_NAV_BAR_CTRL = '$ionNavBarController';
  var PRIMARY_BUTTONS = 'primaryButtons';
  var SECONDARY_BUTTONS = 'secondaryButtons';
  var BACK_BUTTON = 'backButton';
  var ITEM_TYPES = 'primaryButtons secondaryButtons leftButtons rightButtons title'.split(' ');

  var self = this;
  var headerBars = [];
  var navElementHtml = {};
  var isVisible = true;
  var queuedTransitionStart, queuedTransitionEnd, latestTransitionId;

  $element.parent().data(DATA_NAV_BAR_CTRL, self);

  var delegateHandle = $attrs.delegateHandle || 'navBar' + ionic.Utils.nextUid();

  var deregisterInstance = $ionicNavBarDelegate._registerInstance(self, delegateHandle);


  self.init = function() {
    $element.addClass('nav-bar-container');
    ionic.DomUtil.cachedAttr($element, 'nav-bar-transition', $ionicConfig.views.transition());

    // create two nav bar blocks which will trade out which one is shown
    self.createHeaderBar(false);
    self.createHeaderBar(true);

    $scope.$emit('ionNavBar.init', delegateHandle);
  };


  self.createHeaderBar = function(isActive) {
    var containerEle = jqLite('<div class="nav-bar-block">');
    ionic.DomUtil.cachedAttr(containerEle, 'nav-bar', isActive ? 'active' : 'cached');

    var alignTitle = $attrs.alignTitle || $ionicConfig.navBar.alignTitle();
    var headerBarEle = jqLite('<ion-header-bar>').addClass($attrs['class']).attr('align-title', alignTitle);
    if (isDefined($attrs.noTapScroll)) headerBarEle.attr('no-tap-scroll', $attrs.noTapScroll);
    var titleEle = jqLite('<div class="title title-' + alignTitle + '">');
    var navEle = {};
    var lastViewItemEle = {};
    var leftButtonsEle, rightButtonsEle;

    navEle[BACK_BUTTON] = createNavElement(BACK_BUTTON);
    navEle[BACK_BUTTON] && headerBarEle.append(navEle[BACK_BUTTON]);

    // append title in the header, this is the rock to where buttons append
    headerBarEle.append(titleEle);

    forEach(ITEM_TYPES, function(itemType) {
      // create default button elements
      navEle[itemType] = createNavElement(itemType);
      // append and position buttons
      positionItem(navEle[itemType], itemType);
    });

    // add header-item to the root children
    for (var x = 0; x < headerBarEle[0].children.length; x++) {
      headerBarEle[0].children[x].classList.add('header-item');
    }

    // compile header and append to the DOM
    containerEle.append(headerBarEle);
    $element.append($compile(containerEle)($scope.$new()));

    var headerBarCtrl = headerBarEle.data('$ionHeaderBarController');
    headerBarCtrl.backButtonIcon = $ionicConfig.backButton.icon();
    headerBarCtrl.backButtonText = $ionicConfig.backButton.text();

    var headerBarInstance = {
      isActive: isActive,
      title: function(newTitleText) {
        headerBarCtrl.title(newTitleText);
      },
      setItem: function(navBarItemEle, itemType) {
        // first make sure any exiting nav bar item has been removed
        headerBarInstance.removeItem(itemType);

        if (navBarItemEle) {
          if (itemType === 'title') {
            // clear out the text based title
            headerBarInstance.title("");
          }

          // there's a custom nav bar item
          positionItem(navBarItemEle, itemType);

          if (navEle[itemType]) {
            // make sure the default on this itemType is hidden
            navEle[itemType].addClass(CSS_HIDE);
          }
          lastViewItemEle[itemType] = navBarItemEle;

        } else if (navEle[itemType]) {
          // there's a default button for this side and no view button
          navEle[itemType].removeClass(CSS_HIDE);
        }
      },
      removeItem: function(itemType) {
        if (lastViewItemEle[itemType]) {
          lastViewItemEle[itemType].scope().$destroy();
          lastViewItemEle[itemType].remove();
          lastViewItemEle[itemType] = null;
        }
      },
      containerEle: function() {
        return containerEle;
      },
      headerBarEle: function() {
        return headerBarEle;
      },
      afterLeave: function() {
        forEach(ITEM_TYPES, function(itemType) {
          headerBarInstance.removeItem(itemType);
        });
        headerBarCtrl.resetBackButton();
      },
      controller: function() {
        return headerBarCtrl;
      },
      destroy: function() {
        forEach(ITEM_TYPES, function(itemType) {
          headerBarInstance.removeItem(itemType);
        });
        containerEle.scope().$destroy();
        for (var n in navEle) {
          if (navEle[n]) {
            navEle[n].removeData();
            navEle[n] = null;
          }
        }
        leftButtonsEle && leftButtonsEle.removeData();
        rightButtonsEle && rightButtonsEle.removeData();
        titleEle.removeData();
        headerBarEle.removeData();
        containerEle.remove();
        containerEle = headerBarEle = titleEle = leftButtonsEle = rightButtonsEle = null;
      }
    };

    function positionItem(ele, itemType) {
      if (!ele) return;

      if (itemType === 'title') {
        // title element
        titleEle.append(ele);

      } else if (itemType == 'rightButtons' ||
                (itemType == SECONDARY_BUTTONS && $ionicConfig.navBar.positionSecondaryButtons() != 'left') ||
                (itemType == PRIMARY_BUTTONS && $ionicConfig.navBar.positionPrimaryButtons() == 'right')) {
        // right side
        if (!rightButtonsEle) {
          rightButtonsEle = jqLite('<div class="buttons buttons-right">');
          headerBarEle.append(rightButtonsEle);
        }
        if (itemType == SECONDARY_BUTTONS) {
          rightButtonsEle.append(ele);
        } else {
          rightButtonsEle.prepend(ele);
        }

      } else {
        // left side
        if (!leftButtonsEle) {
          leftButtonsEle = jqLite('<div class="buttons buttons-left">');
          if (navEle[BACK_BUTTON]) {
            navEle[BACK_BUTTON].after(leftButtonsEle);
          } else {
            headerBarEle.prepend(leftButtonsEle);
          }
        }
        if (itemType == SECONDARY_BUTTONS) {
          leftButtonsEle.append(ele);
        } else {
          leftButtonsEle.prepend(ele);
        }
      }

    }

    headerBars.push(headerBarInstance);

    return headerBarInstance;
  };


  self.navElement = function(type, html) {
    if (isDefined(html)) {
      navElementHtml[type] = html;
    }
    return navElementHtml[type];
  };


  self.update = function(viewData) {
    var showNavBar = !viewData.hasHeaderBar && viewData.showNavBar;
    viewData.transition = $ionicConfig.views.transition();

    if (!showNavBar) {
      viewData.direction = 'none';
    }

    self.enable(showNavBar);
    var enteringHeaderBar = self.isInitialized ? getOffScreenHeaderBar() : getOnScreenHeaderBar();
    var leavingHeaderBar = self.isInitialized ? getOnScreenHeaderBar() : null;
    var enteringHeaderCtrl = enteringHeaderBar.controller();

    // update if the entering header should show the back button or not
    enteringHeaderCtrl.enableBack(viewData.enableBack, true);
    enteringHeaderCtrl.showBack(viewData.showBack, true);
    enteringHeaderCtrl.updateBackButton();

    // update the entering header bar's title
    self.title(viewData.title, enteringHeaderBar);

    self.showBar(showNavBar);

    // update the nav bar items, depending if the view has their own or not
    if (viewData.navBarItems) {
      forEach(ITEM_TYPES, function(itemType) {
        enteringHeaderBar.setItem(viewData.navBarItems[itemType], itemType);
      });
    }

    // begin transition of entering and leaving header bars
    self.transition(enteringHeaderBar, leavingHeaderBar, viewData);

    self.isInitialized = true;
    navSwipeAttr('');
  };


  self.transition = function(enteringHeaderBar, leavingHeaderBar, viewData) {
    var enteringHeaderBarCtrl = enteringHeaderBar.controller();
    var transitionFn = $ionicConfig.transitions.navBar[viewData.navBarTransition] || $ionicConfig.transitions.navBar.none;
    var transitionId = viewData.transitionId;

    enteringHeaderBarCtrl.beforeEnter(viewData);

    var navBarTransition = transitionFn(enteringHeaderBar, leavingHeaderBar, viewData.direction, viewData.shouldAnimate && self.isInitialized);

    ionic.DomUtil.cachedAttr($element, 'nav-bar-transition', viewData.navBarTransition);
    ionic.DomUtil.cachedAttr($element, 'nav-bar-direction', viewData.direction);

    if (navBarTransition.shouldAnimate && viewData.renderEnd) {
      navBarAttr(enteringHeaderBar, 'stage');
    } else {
      navBarAttr(enteringHeaderBar, 'entering');
      navBarAttr(leavingHeaderBar, 'leaving');
    }

    enteringHeaderBarCtrl.resetBackButton(viewData);

    navBarTransition.run(0);

    self.activeTransition = {
      run: function(step) {
        navBarTransition.shouldAnimate = false;
        navBarTransition.direction = 'back';
        navBarTransition.run(step);
      },
      cancel: function(shouldAnimate, speed, cancelData) {
        navSwipeAttr(speed);
        navBarAttr(leavingHeaderBar, 'active');
        navBarAttr(enteringHeaderBar, 'cached');
        navBarTransition.shouldAnimate = shouldAnimate;
        navBarTransition.run(0);
        self.activeTransition = navBarTransition = null;

        var runApply;
        if (cancelData.showBar !== self.showBar()) {
          self.showBar(cancelData.showBar);
        }
        if (cancelData.showBackButton !== self.showBackButton()) {
          self.showBackButton(cancelData.showBackButton);
        }
        if (runApply) {
          $scope.$apply();
        }
      },
      complete: function(shouldAnimate, speed) {
        navSwipeAttr(speed);
        navBarTransition.shouldAnimate = shouldAnimate;
        navBarTransition.run(1);
        queuedTransitionEnd = transitionEnd;
      }
    };

    $timeout(enteringHeaderBarCtrl.align, 16);

    queuedTransitionStart = function() {
      if (latestTransitionId !== transitionId) return;

      navBarAttr(enteringHeaderBar, 'entering');
      navBarAttr(leavingHeaderBar, 'leaving');

      navBarTransition.run(1);

      queuedTransitionEnd = function() {
        if (latestTransitionId == transitionId || !navBarTransition.shouldAnimate) {
          transitionEnd();
        }
      };

      queuedTransitionStart = null;
    };

    function transitionEnd() {
      for (var x = 0; x < headerBars.length; x++) {
        headerBars[x].isActive = false;
      }
      enteringHeaderBar.isActive = true;

      navBarAttr(enteringHeaderBar, 'active');
      navBarAttr(leavingHeaderBar, 'cached');

      self.activeTransition = navBarTransition = queuedTransitionEnd = null;
    }

    queuedTransitionStart();
  };


  self.triggerTransitionStart = function(triggerTransitionId) {
    latestTransitionId = triggerTransitionId;
    queuedTransitionStart && queuedTransitionStart();
  };


  self.triggerTransitionEnd = function() {
    queuedTransitionEnd && queuedTransitionEnd();
  };


  self.showBar = function(shouldShow) {
    if (arguments.length) {
      self.visibleBar(shouldShow);
      $scope.$parent.$hasHeader = !!shouldShow;
    }
    return !!$scope.$parent.$hasHeader;
  };


  self.visibleBar = function(shouldShow) {
    if (shouldShow && !isVisible) {
      $element.removeClass(CSS_HIDE);
      self.align();
    } else if (!shouldShow && isVisible) {
      $element.addClass(CSS_HIDE);
    }
    isVisible = shouldShow;
  };


  self.enable = function(val) {
    // set primary to show first
    self.visibleBar(val);

    // set non primary to hide second
    for (var x = 0; x < $ionicNavBarDelegate._instances.length; x++) {
      if ($ionicNavBarDelegate._instances[x] !== self) $ionicNavBarDelegate._instances[x].visibleBar(false);
    }
  };


  /**
   * @ngdoc method
   * @name $ionicNavBar#showBackButton
   * @description Show/hide the nav bar back button when there is a
   * back view. If the back button is not possible, for example, the
   * first view in the stack, then this will not force the back button
   * to show.
   */
  self.showBackButton = function(shouldShow) {
    if (arguments.length) {
      for (var x = 0; x < headerBars.length; x++) {
        headerBars[x].controller().showNavBack(!!shouldShow);
      }
      $scope.$isBackButtonShown = !!shouldShow;
    }
    return $scope.$isBackButtonShown;
  };


  /**
   * @ngdoc method
   * @name $ionicNavBar#showActiveBackButton
   * @description Show/hide only the active header bar's back button.
   */
  self.showActiveBackButton = function(shouldShow) {
    var headerBar = getOnScreenHeaderBar();
    if (headerBar) {
      if (arguments.length) {
        return headerBar.controller().showBack(shouldShow);
      }
      return headerBar.controller().showBack();
    }
  };


  self.title = function(newTitleText, headerBar) {
    if (isDefined(newTitleText)) {
      newTitleText = newTitleText || '';
      headerBar = headerBar || getOnScreenHeaderBar();
      headerBar && headerBar.title(newTitleText);
      $scope.$title = newTitleText;
      $ionicHistory.currentTitle(newTitleText);
    }
    return $scope.$title;
  };


  self.align = function(val, headerBar) {
    headerBar = headerBar || getOnScreenHeaderBar();
    headerBar && headerBar.controller().align(val);
  };


  self.hasTabsTop = function(isTabsTop) {
    $element[isTabsTop ? 'addClass' : 'removeClass']('nav-bar-tabs-top');
  };

  self.hasBarSubheader = function(isBarSubheader) {
    $element[isBarSubheader ? 'addClass' : 'removeClass']('nav-bar-has-subheader');
  };

  // DEPRECATED, as of v1.0.0-beta14 -------
  self.changeTitle = function(val) {
    deprecatedWarning('changeTitle(val)', 'title(val)');
    self.title(val);
  };
  self.setTitle = function(val) {
    deprecatedWarning('setTitle(val)', 'title(val)');
    self.title(val);
  };
  self.getTitle = function() {
    deprecatedWarning('getTitle()', 'title()');
    return self.title();
  };
  self.back = function() {
    deprecatedWarning('back()', '$ionicHistory.goBack()');
    $ionicHistory.goBack();
  };
  self.getPreviousTitle = function() {
    deprecatedWarning('getPreviousTitle()', '$ionicHistory.backTitle()');
    $ionicHistory.goBack();
  };
  function deprecatedWarning(oldMethod, newMethod) {
    var warn = console.warn || console.log;
    warn && warn.call(console, 'navBarController.' + oldMethod + ' is deprecated, please use ' + newMethod + ' instead');
  }
  // END DEPRECATED -------


  function createNavElement(type) {
    if (navElementHtml[type]) {
      return jqLite(navElementHtml[type]);
    }
  }


  function getOnScreenHeaderBar() {
    for (var x = 0; x < headerBars.length; x++) {
      if (headerBars[x].isActive) return headerBars[x];
    }
  }


  function getOffScreenHeaderBar() {
    for (var x = 0; x < headerBars.length; x++) {
      if (!headerBars[x].isActive) return headerBars[x];
    }
  }


  function navBarAttr(ctrl, val) {
    ctrl && ionic.DomUtil.cachedAttr(ctrl.containerEle(), 'nav-bar', val);
  }

  function navSwipeAttr(val) {
    ionic.DomUtil.cachedAttr($element, 'nav-swipe', val);
  }


  $scope.$on('$destroy', function() {
    $scope.$parent.$hasHeader = false;
    $element.parent().removeData(DATA_NAV_BAR_CTRL);
    for (var x = 0; x < headerBars.length; x++) {
      headerBars[x].destroy();
    }
    $element.remove();
    $element = headerBars = null;
    deregisterInstance();
  });

}]);

IonicModule
.controller('$ionicNavView', [
  '$scope',
  '$element',
  '$attrs',
  '$compile',
  '$controller',
  '$ionicNavBarDelegate',
  '$ionicNavViewDelegate',
  '$ionicHistory',
  '$ionicViewSwitcher',
  '$ionicConfig',
  '$ionicScrollDelegate',
  '$ionicSideMenuDelegate',
function($scope, $element, $attrs, $compile, $controller, $ionicNavBarDelegate, $ionicNavViewDelegate, $ionicHistory, $ionicViewSwitcher, $ionicConfig, $ionicScrollDelegate, $ionicSideMenuDelegate) {

  var DATA_ELE_IDENTIFIER = '$eleId';
  var DATA_DESTROY_ELE = '$destroyEle';
  var DATA_NO_CACHE = '$noCache';
  var VIEW_STATUS_ACTIVE = 'active';
  var VIEW_STATUS_CACHED = 'cached';

  var self = this;
  var direction;
  var isPrimary = false;
  var navBarDelegate;
  var activeEleId;
  var navViewAttr = $ionicViewSwitcher.navViewAttr;
  var disableRenderStartViewId, disableAnimation;

  self.scope = $scope;
  self.element = $element;

  self.init = function() {
    var navViewName = $attrs.name || '';

    // Find the details of the parent view directive (if any) and use it
    // to derive our own qualified view name, then hang our own details
    // off the DOM so child directives can find it.
    var parent = $element.parent().inheritedData('$uiView');
    var parentViewName = ((parent && parent.state) ? parent.state.name : '');
    if (navViewName.indexOf('@') < 0) navViewName = navViewName + '@' + parentViewName;

    var viewData = { name: navViewName, state: null };
    $element.data('$uiView', viewData);

    var deregisterInstance = $ionicNavViewDelegate._registerInstance(self, $attrs.delegateHandle);
    $scope.$on('$destroy', function() {
      deregisterInstance();

      // ensure no scrolls have been left frozen
      if (self.isSwipeFreeze) {
        $ionicScrollDelegate.freezeAllScrolls(false);
      }
    });

    $scope.$on('$ionicHistory.deselect', self.cacheCleanup);
    $scope.$on('$ionicTabs.top', onTabsTop);
    $scope.$on('$ionicSubheader', onBarSubheader);

    $scope.$on('$ionicTabs.beforeLeave', onTabsLeave);
    $scope.$on('$ionicTabs.afterLeave', onTabsLeave);
    $scope.$on('$ionicTabs.leave', onTabsLeave);

    ionic.Platform.ready(function() {
      if ( ionic.Platform.isWebView() && ionic.Platform.isIOS() ) {
          self.initSwipeBack();
      }
    });

    return viewData;
  };


  self.register = function(viewLocals) {
    var leavingView = extend({}, $ionicHistory.currentView());

    // register that a view is coming in and get info on how it should transition
    var registerData = $ionicHistory.register($scope, viewLocals);

    // update which direction
    self.update(registerData);

    // begin rendering and transitioning
    var enteringView = $ionicHistory.getViewById(registerData.viewId) || {};

    var renderStart = (disableRenderStartViewId !== registerData.viewId);
    self.render(registerData, viewLocals, enteringView, leavingView, renderStart, true);
  };


  self.update = function(registerData) {
    // always reset that this is the primary navView
    isPrimary = true;

    // remember what direction this navView should use
    // this may get updated later by a child navView
    direction = registerData.direction;

    var parentNavViewCtrl = $element.parent().inheritedData('$ionNavViewController');
    if (parentNavViewCtrl) {
      // this navView is nested inside another one
      // update the parent to use this direction and not
      // the other it originally was set to

      // inform the parent navView that it is not the primary navView
      parentNavViewCtrl.isPrimary(false);

      if (direction === 'enter' || direction === 'exit') {
        // they're entering/exiting a history
        // find parent navViewController
        parentNavViewCtrl.direction(direction);

        if (direction === 'enter') {
          // reset the direction so this navView doesn't animate
          // because it's parent will
          direction = 'none';
        }
      }
    }
  };


  self.render = function(registerData, viewLocals, enteringView, leavingView, renderStart, renderEnd) {
    // register the view and figure out where it lives in the various
    // histories and nav stacks, along with how views should enter/leave
    var switcher = $ionicViewSwitcher.create(self, viewLocals, enteringView, leavingView, renderStart, renderEnd);

    // init the rendering of views for this navView directive
    switcher.init(registerData, function() {
      // the view is now compiled, in the dom and linked, now lets transition the views.
      // this uses a callback incase THIS nav-view has a nested nav-view, and after the NESTED
      // nav-view links, the NESTED nav-view would update which direction THIS nav-view should use

      // kick off the transition of views
      switcher.transition(self.direction(), registerData.enableBack, !disableAnimation);

      // reset private vars for next time
      disableRenderStartViewId = disableAnimation = null;
    });

  };


  self.beforeEnter = function(transitionData) {
    if (isPrimary) {
      // only update this nav-view's nav-bar if this is the primary nav-view
      navBarDelegate = transitionData.navBarDelegate;
      var associatedNavBarCtrl = getAssociatedNavBarCtrl();
      associatedNavBarCtrl && associatedNavBarCtrl.update(transitionData);
      navSwipeAttr('');
    }
  };


  self.activeEleId = function(eleId) {
    if (arguments.length) {
      activeEleId = eleId;
    }
    return activeEleId;
  };


  self.transitionEnd = function() {
    var viewElements = $element.children();
    var x, l, viewElement;

    for (x = 0, l = viewElements.length; x < l; x++) {
      viewElement = viewElements.eq(x);

      if (viewElement.data(DATA_ELE_IDENTIFIER) === activeEleId) {
        // this is the active element
        navViewAttr(viewElement, VIEW_STATUS_ACTIVE);

      } else if (navViewAttr(viewElement) === 'leaving' || navViewAttr(viewElement) === VIEW_STATUS_ACTIVE || navViewAttr(viewElement) === VIEW_STATUS_CACHED) {
        // this is a leaving element or was the former active element, or is an cached element
        if (viewElement.data(DATA_DESTROY_ELE) || viewElement.data(DATA_NO_CACHE)) {
          // this element shouldn't stay cached
          $ionicViewSwitcher.destroyViewEle(viewElement);

        } else {
          // keep in the DOM, mark as cached
          navViewAttr(viewElement, VIEW_STATUS_CACHED);

          // disconnect the leaving scope
          ionic.Utils.disconnectScope(viewElement.scope());
        }
      }
    }

    navSwipeAttr('');

    // ensure no scrolls have been left frozen
    if (self.isSwipeFreeze) {
      $ionicScrollDelegate.freezeAllScrolls(false);
    }
  };


  function onTabsLeave(ev, data) {
    var viewElements = $element.children();
    var viewElement, viewScope;

    for (var x = 0, l = viewElements.length; x < l; x++) {
      viewElement = viewElements.eq(x);
      if (navViewAttr(viewElement) == VIEW_STATUS_ACTIVE) {
        viewScope = viewElement.scope();
        viewScope && viewScope.$emit(ev.name.replace('Tabs', 'View'), data);
        viewScope && viewScope.$broadcast(ev.name.replace('Tabs', 'ParentView'), data);
        break;
      }
    }
  }


  self.cacheCleanup = function() {
    var viewElements = $element.children();
    for (var x = 0, l = viewElements.length; x < l; x++) {
      if (viewElements.eq(x).data(DATA_DESTROY_ELE)) {
        $ionicViewSwitcher.destroyViewEle(viewElements.eq(x));
      }
    }
  };


  self.clearCache = function(stateIds) {
    var viewElements = $element.children();
    var viewElement, viewScope, x, l, y, eleIdentifier;

    for (x = 0, l = viewElements.length; x < l; x++) {
      viewElement = viewElements.eq(x);

      if (stateIds) {
        eleIdentifier = viewElement.data(DATA_ELE_IDENTIFIER);

        for (y = 0; y < stateIds.length; y++) {
          if (eleIdentifier === stateIds[y]) {
            $ionicViewSwitcher.destroyViewEle(viewElement);
          }
        }
        continue;
      }

      if (navViewAttr(viewElement) == VIEW_STATUS_CACHED) {
        $ionicViewSwitcher.destroyViewEle(viewElement);

      } else if (navViewAttr(viewElement) == VIEW_STATUS_ACTIVE) {
        viewScope = viewElement.scope();
        viewScope && viewScope.$broadcast('$ionicView.clearCache');
      }

    }
  };


  self.getViewElements = function() {
    return $element.children();
  };


  self.appendViewElement = function(viewEle, viewLocals) {
    // compile the entering element and get the link function
    var linkFn = $compile(viewEle);

    $element.append(viewEle);

    var viewScope = $scope.$new();

    if (viewLocals && viewLocals.$$controller) {
      viewLocals.$scope = viewScope;
      var controller = $controller(viewLocals.$$controller, viewLocals);
      if (viewLocals.$$controllerAs) {
        viewScope[viewLocals.$$controllerAs] = controller;
      }
      $element.children().data('$ngControllerController', controller);
    }

    linkFn(viewScope);

    return viewScope;
  };


  self.title = function(val) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    associatedNavBarCtrl && associatedNavBarCtrl.title(val);
  };


  /**
   * @ngdoc method
   * @name $ionicNavView#enableBackButton
   * @description Enable/disable if the back button can be shown or not. For
   * example, the very first view in the navigation stack would not have a
   * back view, so the back button would be disabled.
   */
  self.enableBackButton = function(shouldEnable) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    associatedNavBarCtrl && associatedNavBarCtrl.enableBackButton(shouldEnable);
  };


  /**
   * @ngdoc method
   * @name $ionicNavView#showBackButton
   * @description Show/hide the nav bar active back button. If the back button
   * is not possible this will not force the back button to show. The
   * `enableBackButton()` method handles if a back button is even possible or not.
   */
  self.showBackButton = function(shouldShow) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    if (associatedNavBarCtrl) {
      if (arguments.length) {
        return associatedNavBarCtrl.showActiveBackButton(shouldShow);
      }
      return associatedNavBarCtrl.showActiveBackButton();
    }
    return true;
  };


  self.showBar = function(val) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    if (associatedNavBarCtrl) {
      if (arguments.length) {
        return associatedNavBarCtrl.showBar(val);
      }
      return associatedNavBarCtrl.showBar();
    }
    return true;
  };


  self.isPrimary = function(val) {
    if (arguments.length) {
      isPrimary = val;
    }
    return isPrimary;
  };


  self.direction = function(val) {
    if (arguments.length) {
      direction = val;
    }
    return direction;
  };


  self.initSwipeBack = function() {
    var swipeBackHitWidth = $ionicConfig.views.swipeBackHitWidth();
    var viewTransition, associatedNavBarCtrl, backView;
    var deregDragStart, deregDrag, deregRelease;
    var windowWidth, startDragX, dragPoints;
    var cancelData = {};

    function onDragStart(ev) {
      if (!isPrimary || !$ionicConfig.views.swipeBackEnabled() || $ionicSideMenuDelegate.isOpenRight() ) return;


      startDragX = getDragX(ev);
      if (startDragX > swipeBackHitWidth) return;

      backView = $ionicHistory.backView();

      var currentView = $ionicHistory.currentView();

      if (!backView || backView.historyId !== currentView.historyId || currentView.canSwipeBack === false) return;

      if (!windowWidth) windowWidth = window.innerWidth;

      self.isSwipeFreeze = $ionicScrollDelegate.freezeAllScrolls(true);

      var registerData = {
        direction: 'back'
      };

      dragPoints = [];

      cancelData = {
        showBar: self.showBar(),
        showBackButton: self.showBackButton()
      };

      var switcher = $ionicViewSwitcher.create(self, registerData, backView, currentView, true, false);
      switcher.loadViewElements(registerData);
      switcher.render(registerData);

      viewTransition = switcher.transition('back', $ionicHistory.enabledBack(backView), true);

      associatedNavBarCtrl = getAssociatedNavBarCtrl();

      deregDrag = ionic.onGesture('drag', onDrag, $element[0]);
      deregRelease = ionic.onGesture('release', onRelease, $element[0]);
    }

    function onDrag(ev) {
      if (isPrimary && viewTransition) {
        var dragX = getDragX(ev);

        dragPoints.push({
          t: Date.now(),
          x: dragX
        });

        if (dragX >= windowWidth - 15) {
          onRelease(ev);

        } else {
          var step = Math.min(Math.max(getSwipeCompletion(dragX), 0), 1);
          viewTransition.run(step);
          associatedNavBarCtrl && associatedNavBarCtrl.activeTransition && associatedNavBarCtrl.activeTransition.run(step);
        }

      }
    }

    function onRelease(ev) {
      if (isPrimary && viewTransition && dragPoints && dragPoints.length > 1) {

        var now = Date.now();
        var releaseX = getDragX(ev);
        var startDrag = dragPoints[dragPoints.length - 1];

        for (var x = dragPoints.length - 2; x >= 0; x--) {
          if (now - startDrag.t > 200) {
            break;
          }
          startDrag = dragPoints[x];
        }

        var isSwipingRight = (releaseX >= dragPoints[dragPoints.length - 2].x);
        var releaseSwipeCompletion = getSwipeCompletion(releaseX);
        var velocity = Math.abs(startDrag.x - releaseX) / (now - startDrag.t);

        // private variables because ui-router has no way to pass custom data using $state.go
        disableRenderStartViewId = backView.viewId;
        disableAnimation = (releaseSwipeCompletion < 0.03 || releaseSwipeCompletion > 0.97);

        if (isSwipingRight && (releaseSwipeCompletion > 0.5 || velocity > 0.1)) {
          // complete view transition on release
          var speed = (velocity > 0.5 || velocity < 0.05 || releaseX > windowWidth - 45) ? 'fast' : 'slow';
          navSwipeAttr(disableAnimation ? '' : speed);
          backView.go();
          associatedNavBarCtrl && associatedNavBarCtrl.activeTransition && associatedNavBarCtrl.activeTransition.complete(!disableAnimation, speed);

        } else {
          // cancel view transition on release
          navSwipeAttr(disableAnimation ? '' : 'fast');
          disableRenderStartViewId = null;
          viewTransition.cancel(!disableAnimation);
          associatedNavBarCtrl && associatedNavBarCtrl.activeTransition && associatedNavBarCtrl.activeTransition.cancel(!disableAnimation, 'fast', cancelData);
          disableAnimation = null;
        }

      }

      ionic.offGesture(deregDrag, 'drag', onDrag);
      ionic.offGesture(deregRelease, 'release', onRelease);

      windowWidth = viewTransition = dragPoints = null;

      self.isSwipeFreeze = $ionicScrollDelegate.freezeAllScrolls(false);
    }

    function getDragX(ev) {
      return ionic.tap.pointerCoord(ev.gesture.srcEvent).x;
    }

    function getSwipeCompletion(dragX) {
      return (dragX - startDragX) / windowWidth;
    }

    deregDragStart = ionic.onGesture('dragstart', onDragStart, $element[0]);

    $scope.$on('$destroy', function() {
      ionic.offGesture(deregDragStart, 'dragstart', onDragStart);
      ionic.offGesture(deregDrag, 'drag', onDrag);
      ionic.offGesture(deregRelease, 'release', onRelease);
      self.element = viewTransition = associatedNavBarCtrl = null;
    });
  };


  function navSwipeAttr(val) {
    ionic.DomUtil.cachedAttr($element, 'nav-swipe', val);
  }


  function onTabsTop(ev, isTabsTop) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    associatedNavBarCtrl && associatedNavBarCtrl.hasTabsTop(isTabsTop);
  }

  function onBarSubheader(ev, isBarSubheader) {
    var associatedNavBarCtrl = getAssociatedNavBarCtrl();
    associatedNavBarCtrl && associatedNavBarCtrl.hasBarSubheader(isBarSubheader);
  }

  function getAssociatedNavBarCtrl() {
    if (navBarDelegate) {
      for (var x = 0; x < $ionicNavBarDelegate._instances.length; x++) {
        if ($ionicNavBarDelegate._instances[x].$$delegateHandle == navBarDelegate) {
          return $ionicNavBarDelegate._instances[x];
        }
      }
    }
    return $element.inheritedData('$ionNavBarController');
  }

}]);

IonicModule
.controller('$ionicRefresher', [
  '$scope',
  '$attrs',
  '$element',
  '$ionicBind',
  '$timeout',
  function($scope, $attrs, $element, $ionicBind, $timeout) {
    var self = this,
        isDragging = false,
        isOverscrolling = false,
        dragOffset = 0,
        lastOverscroll = 0,
        ptrThreshold = 60,
        activated = false,
        scrollTime = 500,
        startY = null,
        deltaY = null,
        canOverscroll = true,
        scrollParent,
        scrollChild;

    if (!isDefined($attrs.pullingIcon)) {
      $attrs.$set('pullingIcon', 'ion-android-arrow-down');
    }

    $scope.showSpinner = !isDefined($attrs.refreshingIcon) && $attrs.spinner != 'none';

    $scope.showIcon = isDefined($attrs.refreshingIcon);

    $ionicBind($scope, $attrs, {
      pullingIcon: '@',
      pullingText: '@',
      refreshingIcon: '@',
      refreshingText: '@',
      spinner: '@',
      disablePullingRotation: '@',
      $onRefresh: '&onRefresh',
      $onPulling: '&onPulling'
    });

    function handleMousedown(e) {
      e.touches = e.touches || [{
        screenX: e.screenX,
        screenY: e.screenY
      }];
      // Mouse needs this
      startY = Math.floor(e.touches[0].screenY);
    }

    function handleTouchstart(e) {
      e.touches = e.touches || [{
        screenX: e.screenX,
        screenY: e.screenY
      }];

      startY = e.touches[0].screenY;
    }

    function handleTouchend() {
      // reset Y
      startY = null;
      // if this wasn't an overscroll, get out immediately
      if (!canOverscroll && !isDragging) {
        return;
      }
      // the user has overscrolled but went back to native scrolling
      if (!isDragging) {
        dragOffset = 0;
        isOverscrolling = false;
        setScrollLock(false);
      } else {
        isDragging = false;
        dragOffset = 0;

        // the user has scroll far enough to trigger a refresh
        if (lastOverscroll > ptrThreshold) {
          start();
          scrollTo(ptrThreshold, scrollTime);

        // the user has overscrolled but not far enough to trigger a refresh
        } else {
          scrollTo(0, scrollTime, deactivate);
          isOverscrolling = false;
        }
      }
    }

    function handleTouchmove(e) {
      e.touches = e.touches || [{
        screenX: e.screenX,
        screenY: e.screenY
      }];

      // Force mouse events to have had a down event first
      if (!startY && e.type == 'mousemove') {
        return;
      }

      // if multitouch or regular scroll event, get out immediately
      if (!canOverscroll || e.touches.length > 1) {
        return;
      }
      //if this is a new drag, keep track of where we start
      if (startY === null) {
        startY = e.touches[0].screenY;
      }

      deltaY = e.touches[0].screenY - startY;

      // how far have we dragged so far?
      // kitkat fix for touchcancel events http://updates.html5rocks.com/2014/05/A-More-Compatible-Smoother-Touch
      // Only do this if we're not on crosswalk
      if (ionic.Platform.isAndroid() && ionic.Platform.version() === 4.4 && !ionic.Platform.isCrosswalk() && scrollParent.scrollTop === 0 && deltaY > 0) {
        isDragging = true;
        e.preventDefault();
      }


      // if we've dragged up and back down in to native scroll territory
      if (deltaY - dragOffset <= 0 || scrollParent.scrollTop !== 0) {

        if (isOverscrolling) {
          isOverscrolling = false;
          setScrollLock(false);
        }

        if (isDragging) {
          nativescroll(scrollParent, deltaY - dragOffset * -1);
        }

        // if we're not at overscroll 0 yet, 0 out
        if (lastOverscroll !== 0) {
          overscroll(0);
        }
        return;

      } else if (deltaY > 0 && scrollParent.scrollTop === 0 && !isOverscrolling) {
        // starting overscroll, but drag started below scrollTop 0, so we need to offset the position
        dragOffset = deltaY;
      }

      // prevent native scroll events while overscrolling
      e.preventDefault();

      // if not overscrolling yet, initiate overscrolling
      if (!isOverscrolling) {
        isOverscrolling = true;
        setScrollLock(true);
      }

      isDragging = true;
      // overscroll according to the user's drag so far
      overscroll((deltaY - dragOffset) / 3);

      // update the icon accordingly
      if (!activated && lastOverscroll > ptrThreshold) {
        activated = true;
        ionic.requestAnimationFrame(activate);

      } else if (activated && lastOverscroll < ptrThreshold) {
        activated = false;
        ionic.requestAnimationFrame(deactivate);
      }
    }

    function handleScroll(e) {
      // canOverscrol is used to greatly simplify the drag handler during normal scrolling
      canOverscroll = (e.target.scrollTop === 0) || isDragging;
    }

    function overscroll(val) {
      scrollChild.style[ionic.CSS.TRANSFORM] = 'translate3d(0px, ' + val + 'px, 0px)';
      lastOverscroll = val;
    }

    function nativescroll(target, newScrollTop) {
      // creates a scroll event that bubbles, can be cancelled, and with its view
      // and detail property initialized to window and 1, respectively
      target.scrollTop = newScrollTop;
      var e = document.createEvent("UIEvents");
      e.initUIEvent("scroll", true, true, window, 1);
      target.dispatchEvent(e);
    }

    function setScrollLock(enabled) {
      // set the scrollbar to be position:fixed in preparation to overscroll
      // or remove it so the app can be natively scrolled
      if (enabled) {
        ionic.requestAnimationFrame(function() {
          scrollChild.classList.add('overscroll');
          show();
        });

      } else {
        ionic.requestAnimationFrame(function() {
          scrollChild.classList.remove('overscroll');
          hide();
          deactivate();
        });
      }
    }

    $scope.$on('scroll.refreshComplete', function() {
      // prevent the complete from firing before the scroll has started
      $timeout(function() {

        ionic.requestAnimationFrame(tail);

        // scroll back to home during tail animation
        scrollTo(0, scrollTime, deactivate);

        // return to native scrolling after tail animation has time to finish
        $timeout(function() {

          if (isOverscrolling) {
            isOverscrolling = false;
            setScrollLock(false);
          }

        }, scrollTime);

      }, scrollTime);
    });

    function scrollTo(Y, duration, callback) {
      // scroll animation loop w/ easing
      // credit https://gist.github.com/dezinezync/5487119
      var start = Date.now(),
          from = lastOverscroll;

      if (from === Y) {
        callback();
        return; /* Prevent scrolling to the Y point if already there */
      }

      // decelerating to zero velocity
      function easeOutCubic(t) {
        return (--t) * t * t + 1;
      }

      // scroll loop
      function scroll() {
        var currentTime = Date.now(),
          time = Math.min(1, ((currentTime - start) / duration)),
          // where .5 would be 50% of time on a linear scale easedT gives a
          // fraction based on the easing method
          easedT = easeOutCubic(time);

        overscroll(Math.floor((easedT * (Y - from)) + from));

        if (time < 1) {
          ionic.requestAnimationFrame(scroll);

        } else {

          if (Y < 5 && Y > -5) {
            isOverscrolling = false;
            setScrollLock(false);
          }

          callback && callback();
        }
      }

      // start scroll loop
      ionic.requestAnimationFrame(scroll);
    }


    var touchStartEvent, touchMoveEvent, touchEndEvent;
    if (window.navigator.pointerEnabled) {
      touchStartEvent = 'pointerdown';
      touchMoveEvent = 'pointermove';
      touchEndEvent = 'pointerup';
    } else if (window.navigator.msPointerEnabled) {
      touchStartEvent = 'MSPointerDown';
      touchMoveEvent = 'MSPointerMove';
      touchEndEvent = 'MSPointerUp';
    } else {
      touchStartEvent = 'touchstart';
      touchMoveEvent = 'touchmove';
      touchEndEvent = 'touchend';
    }

    self.init = function() {
      scrollParent = $element.parent().parent()[0];
      scrollChild = $element.parent()[0];

      if (!scrollParent || !scrollParent.classList.contains('ionic-scroll') ||
        !scrollChild || !scrollChild.classList.contains('scroll')) {
        throw new Error('Refresher must be immediate child of ion-content or ion-scroll');
      }


      ionic.on(touchStartEvent, handleTouchstart, scrollChild);
      ionic.on(touchMoveEvent, handleTouchmove, scrollChild);
      ionic.on(touchEndEvent, handleTouchend, scrollChild);
      ionic.on('mousedown', handleMousedown, scrollChild);
      ionic.on('mousemove', handleTouchmove, scrollChild);
      ionic.on('mouseup', handleTouchend, scrollChild);
      ionic.on('scroll', handleScroll, scrollParent);

      // cleanup when done
      $scope.$on('$destroy', destroy);
    };

    function destroy() {
      if ( scrollChild ) {
        ionic.off(touchStartEvent, handleTouchstart, scrollChild);
        ionic.off(touchMoveEvent, handleTouchmove, scrollChild);
        ionic.off(touchEndEvent, handleTouchend, scrollChild);
        ionic.off('mousedown', handleMousedown, scrollChild);
        ionic.off('mousemove', handleTouchmove, scrollChild);
        ionic.off('mouseup', handleTouchend, scrollChild);
      }
      if ( scrollParent ) {
        ionic.off('scroll', handleScroll, scrollParent);
      }
      scrollParent = null;
      scrollChild = null;
    }

    // DOM manipulation and broadcast methods shared by JS and Native Scrolling
    // getter used by JS Scrolling
    self.getRefresherDomMethods = function() {
      return {
        activate: activate,
        deactivate: deactivate,
        start: start,
        show: show,
        hide: hide,
        tail: tail
      };
    };

    function activate() {
      $element[0].classList.add('active');
      $scope.$onPulling();
    }

    function deactivate() {
      // give tail 150ms to finish
      $timeout(function() {
        // deactivateCallback
        $element.removeClass('active refreshing refreshing-tail');
        if (activated) activated = false;
      }, 150);
    }

    function start() {
      // startCallback
      $element[0].classList.add('refreshing');
      var q = $scope.$onRefresh();

      if (q && q.then) {
        q['finally'](function() {
          $scope.$broadcast('scroll.refreshComplete');
        });
      }
    }

    function show() {
      // showCallback
      $element[0].classList.remove('invisible');
    }

    function hide() {
      // showCallback
      $element[0].classList.add('invisible');
    }

    function tail() {
      // tailCallback
      $element[0].classList.add('refreshing-tail');
    }

    // for testing
    self.__handleTouchmove = handleTouchmove;
    self.__getScrollChild = function() { return scrollChild; };
    self.__getScrollParent = function() { return scrollParent; };
  }
]);

/**
 * @private
 */
IonicModule

.controller('$ionicScroll', [
  '$scope',
  'scrollViewOptions',
  '$timeout',
  '$window',
  '$location',
  '$document',
  '$ionicScrollDelegate',
  '$ionicHistory',
function($scope,
         scrollViewOptions,
         $timeout,
         $window,
         $location,
         $document,
         $ionicScrollDelegate,
         $ionicHistory) {

  var self = this;
  // for testing
  self.__timeout = $timeout;

  self._scrollViewOptions = scrollViewOptions; //for testing
  self.isNative = function() {
    return !!scrollViewOptions.nativeScrolling;
  };

  var element = self.element = scrollViewOptions.el;
  var $element = self.$element = jqLite(element);
  var scrollView;
  if (self.isNative()) {
    scrollView = self.scrollView = new ionic.views.ScrollNative(scrollViewOptions);
  } else {
    scrollView = self.scrollView = new ionic.views.Scroll(scrollViewOptions);
  }


  //Attach self to element as a controller so other directives can require this controller
  //through `require: '$ionicScroll'
  //Also attach to parent so that sibling elements can require this
  ($element.parent().length ? $element.parent() : $element)
    .data('$$ionicScrollController', self);

  var deregisterInstance = $ionicScrollDelegate._registerInstance(
    self, scrollViewOptions.delegateHandle, function() {
      return $ionicHistory.isActiveScope($scope);
    }
  );

  if (!isDefined(scrollViewOptions.bouncing)) {
    ionic.Platform.ready(function() {
      if (scrollView && scrollView.options) {
        scrollView.options.bouncing = true;
        if (ionic.Platform.isAndroid()) {
          // No bouncing by default on Android
          scrollView.options.bouncing = false;
          // Faster scroll decel
          scrollView.options.deceleration = 0.95;
        }
      }
    });
  }

  var resize = angular.bind(scrollView, scrollView.resize);
  angular.element($window).on('resize', resize);

  var scrollFunc = function(e) {
    var detail = (e.originalEvent || e).detail || {};
    $scope.$onScroll && $scope.$onScroll({
      event: e,
      scrollTop: detail.scrollTop || 0,
      scrollLeft: detail.scrollLeft || 0
    });
  };

  $element.on('scroll', scrollFunc);

  $scope.$on('$destroy', function() {
    deregisterInstance();
    scrollView && scrollView.__cleanup && scrollView.__cleanup();
    angular.element($window).off('resize', resize);
    if ( $element ) {
      $element.off('scroll', scrollFunc);
    }
    if ( self._scrollViewOptions ) {
      self._scrollViewOptions.el = null;
    }
    if ( scrollViewOptions ) {
        scrollViewOptions.el = null;
    }

    scrollView = self.scrollView = scrollViewOptions = self._scrollViewOptions = element = self.$element = $element = null;
  });

  $timeout(function() {
    scrollView && scrollView.run && scrollView.run();
  });

  self.getScrollView = function() {
    return scrollView;
  };

  self.getScrollPosition = function() {
    return scrollView.getValues();
  };

  self.resize = function() {
    return $timeout(resize, 0, false).then(function() {
      $element && $element.triggerHandler('scroll-resize');
    });
  };

  self.scrollTop = function(shouldAnimate) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      scrollView.scrollTo(0, 0, !!shouldAnimate);
    });
  };

  self.scrollBottom = function(shouldAnimate) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      var max = scrollView.getScrollMax();
      scrollView.scrollTo(max.left, max.top, !!shouldAnimate);
    });
  };

  self.scrollTo = function(left, top, shouldAnimate) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      scrollView.scrollTo(left, top, !!shouldAnimate);
    });
  };

  self.zoomTo = function(zoom, shouldAnimate, originLeft, originTop) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      scrollView.zoomTo(zoom, !!shouldAnimate, originLeft, originTop);
    });
  };

  self.zoomBy = function(zoom, shouldAnimate, originLeft, originTop) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      scrollView.zoomBy(zoom, !!shouldAnimate, originLeft, originTop);
    });
  };

  self.scrollBy = function(left, top, shouldAnimate) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      scrollView.scrollBy(left, top, !!shouldAnimate);
    });
  };

  self.anchorScroll = function(shouldAnimate) {
    self.resize().then(function() {
      if (!scrollView) {
        return;
      }
      var hash = $location.hash();
      var elm = hash && $document[0].getElementById(hash);
      if (!(hash && elm)) {
        scrollView.scrollTo(0, 0, !!shouldAnimate);
        return;
      }
      var curElm = elm;
      var scrollLeft = 0, scrollTop = 0;
      do {
        if (curElm !== null) scrollLeft += curElm.offsetLeft;
        if (curElm !== null) scrollTop += curElm.offsetTop;
        curElm = curElm.offsetParent;
      } while (curElm.attributes != self.element.attributes && curElm.offsetParent);
      scrollView.scrollTo(scrollLeft, scrollTop, !!shouldAnimate);
    });
  };

  self.freezeScroll = scrollView.freeze;
  self.freezeScrollShut = scrollView.freezeShut;

  self.freezeAllScrolls = function(shouldFreeze) {
    for (var i = 0; i < $ionicScrollDelegate._instances.length; i++) {
      $ionicScrollDelegate._instances[i].freezeScroll(shouldFreeze);
    }
  };


  /**
   * @private
   */
  self._setRefresher = function(refresherScope, refresherElement, refresherMethods) {
    self.refresher = refresherElement;
    var refresherHeight = self.refresher.clientHeight || 60;
    scrollView.activatePullToRefresh(
      refresherHeight,
      refresherMethods
    );
  };

}]);

IonicModule
.controller('$ionicSideMenus', [
  '$scope',
  '$attrs',
  '$ionicSideMenuDelegate',
  '$ionicPlatform',
  '$ionicBody',
  '$ionicHistory',
  '$ionicScrollDelegate',
  'IONIC_BACK_PRIORITY',
  '$rootScope',
function($scope, $attrs, $ionicSideMenuDelegate, $ionicPlatform, $ionicBody, $ionicHistory, $ionicScrollDelegate, IONIC_BACK_PRIORITY, $rootScope) {
  var self = this;
  var rightShowing, leftShowing, isDragging;
  var startX, lastX, offsetX, isAsideExposed;
  var enableMenuWithBackViews = true;

  self.$scope = $scope;

  self.initialize = function(options) {
    self.left = options.left;
    self.right = options.right;
    self.setContent(options.content);
    self.dragThresholdX = options.dragThresholdX || 10;
    $ionicHistory.registerHistory(self.$scope);
  };

  /**
   * Set the content view controller if not passed in the constructor options.
   *
   * @param {object} content
   */
  self.setContent = function(content) {
    if (content) {
      self.content = content;

      self.content.onDrag = function(e) {
        self._handleDrag(e);
      };

      self.content.endDrag = function(e) {
        self._endDrag(e);
      };
    }
  };

  self.isOpenLeft = function() {
    return self.getOpenAmount() > 0;
  };

  self.isOpenRight = function() {
    return self.getOpenAmount() < 0;
  };

  /**
   * Toggle the left menu to open 100%
   */
  self.toggleLeft = function(shouldOpen) {
    if (isAsideExposed || !self.left.isEnabled) return;
    var openAmount = self.getOpenAmount();
    if (arguments.length === 0) {
      shouldOpen = openAmount <= 0;
    }
    self.content.enableAnimation();
    if (!shouldOpen) {
      self.openPercentage(0);
      $rootScope.$emit('$ionicSideMenuClose', 'left');
    } else {
      self.openPercentage(100);
      $rootScope.$emit('$ionicSideMenuOpen', 'left');
    }
  };

  /**
   * Toggle the right menu to open 100%
   */
  self.toggleRight = function(shouldOpen) {
    if (isAsideExposed || !self.right.isEnabled) return;
    var openAmount = self.getOpenAmount();
    if (arguments.length === 0) {
      shouldOpen = openAmount >= 0;
    }
    self.content.enableAnimation();
    if (!shouldOpen) {
      self.openPercentage(0);
      $rootScope.$emit('$ionicSideMenuClose', 'right');
    } else {
      self.openPercentage(-100);
      $rootScope.$emit('$ionicSideMenuOpen', 'right');
    }
  };

  self.toggle = function(side) {
    if (side == 'right') {
      self.toggleRight();
    } else {
      self.toggleLeft();
    }
  };

  /**
   * Close all menus.
   */
  self.close = function() {
    self.openPercentage(0);
    $rootScope.$emit('$ionicSideMenuClose', 'left');
    $rootScope.$emit('$ionicSideMenuClose', 'right');
  };

  /**
   * @return {float} The amount the side menu is open, either positive or negative for left (positive), or right (negative)
   */
  self.getOpenAmount = function() {
    return self.content && self.content.getTranslateX() || 0;
  };

  /**
   * @return {float} The ratio of open amount over menu width. For example, a
   * menu of width 100 open 50 pixels would be open 50% or a ratio of 0.5. Value is negative
   * for right menu.
   */
  self.getOpenRatio = function() {
    var amount = self.getOpenAmount();
    if (amount >= 0) {
      return amount / self.left.width;
    }
    return amount / self.right.width;
  };

  self.isOpen = function() {
    return self.getOpenAmount() !== 0;
  };

  /**
   * @return {float} The percentage of open amount over menu width. For example, a
   * menu of width 100 open 50 pixels would be open 50%. Value is negative
   * for right menu.
   */
  self.getOpenPercentage = function() {
    return self.getOpenRatio() * 100;
  };

  /**
   * Open the menu with a given percentage amount.
   * @param {float} percentage The percentage (positive or negative for left/right) to open the menu.
   */
  self.openPercentage = function(percentage) {
    var p = percentage / 100;

    if (self.left && percentage >= 0) {
      self.openAmount(self.left.width * p);
    } else if (self.right && percentage < 0) {
      self.openAmount(self.right.width * p);
    }

    // add the CSS class "menu-open" if the percentage does not
    // equal 0, otherwise remove the class from the body element
    $ionicBody.enableClass((percentage !== 0), 'menu-open');

    self.content.setCanScroll(percentage == 0);
  };

  /*
  function freezeAllScrolls(shouldFreeze) {
    if (shouldFreeze && !self.isScrollFreeze) {
      $ionicScrollDelegate.freezeAllScrolls(shouldFreeze);

    } else if (!shouldFreeze && self.isScrollFreeze) {
      $ionicScrollDelegate.freezeAllScrolls(false);
    }
    self.isScrollFreeze = shouldFreeze;
  }
  */

  /**
   * Open the menu the given pixel amount.
   * @param {float} amount the pixel amount to open the menu. Positive value for left menu,
   * negative value for right menu (only one menu will be visible at a time).
   */
  self.openAmount = function(amount) {
    var maxLeft = self.left && self.left.width || 0;
    var maxRight = self.right && self.right.width || 0;

    // Check if we can move to that side, depending if the left/right panel is enabled
    if (!(self.left && self.left.isEnabled) && amount > 0) {
      self.content.setTranslateX(0);
      return;
    }

    if (!(self.right && self.right.isEnabled) && amount < 0) {
      self.content.setTranslateX(0);
      return;
    }

    if (leftShowing && amount > maxLeft) {
      self.content.setTranslateX(maxLeft);
      return;
    }

    if (rightShowing && amount < -maxRight) {
      self.content.setTranslateX(-maxRight);
      return;
    }

    self.content.setTranslateX(amount);

    leftShowing = amount > 0;
    rightShowing = amount < 0;

    if (amount > 0) {
      // Push the z-index of the right menu down
      self.right && self.right.pushDown && self.right.pushDown();
      // Bring the z-index of the left menu up
      self.left && self.left.bringUp && self.left.bringUp();
    } else {
      // Bring the z-index of the right menu up
      self.right && self.right.bringUp && self.right.bringUp();
      // Push the z-index of the left menu down
      self.left && self.left.pushDown && self.left.pushDown();
    }
  };

  /**
   * Given an event object, find the final resting position of this side
   * menu. For example, if the user "throws" the content to the right and
   * releases the touch, the left menu should snap open (animated, of course).
   *
   * @param {Event} e the gesture event to use for snapping
   */
  self.snapToRest = function(e) {
    // We want to animate at the end of this
    self.content.enableAnimation();
    isDragging = false;

    // Check how much the panel is open after the drag, and
    // what the drag velocity is
    var ratio = self.getOpenRatio();

    if (ratio === 0) {
      // Just to be safe
      self.openPercentage(0);
      return;
    }

    var velocityThreshold = 0.3;
    var velocityX = e.gesture.velocityX;
    var direction = e.gesture.direction;

    // Going right, less than half, too slow (snap back)
    if (ratio > 0 && ratio < 0.5 && direction == 'right' && velocityX < velocityThreshold) {
      self.openPercentage(0);
    }

    // Going left, more than half, too slow (snap back)
    else if (ratio > 0.5 && direction == 'left' && velocityX < velocityThreshold) {
      self.openPercentage(100);
    }

    // Going left, less than half, too slow (snap back)
    else if (ratio < 0 && ratio > -0.5 && direction == 'left' && velocityX < velocityThreshold) {
      self.openPercentage(0);
    }

    // Going right, more than half, too slow (snap back)
    else if (ratio < 0.5 && direction == 'right' && velocityX < velocityThreshold) {
      self.openPercentage(-100);
    }

    // Going right, more than half, or quickly (snap open)
    else if (direction == 'right' && ratio >= 0 && (ratio >= 0.5 || velocityX > velocityThreshold)) {
      self.openPercentage(100);
    }

    // Going left, more than half, or quickly (span open)
    else if (direction == 'left' && ratio <= 0 && (ratio <= -0.5 || velocityX > velocityThreshold)) {
      self.openPercentage(-100);
    }

    // Snap back for safety
    else {
      self.openPercentage(0);
    }
  };

  self.enableMenuWithBackViews = function(val) {
    if (arguments.length) {
      enableMenuWithBackViews = !!val;
    }
    return enableMenuWithBackViews;
  };

  self.isAsideExposed = function() {
    return !!isAsideExposed;
  };

  self.exposeAside = function(shouldExposeAside) {
    if (!(self.left && self.left.isEnabled) && !(self.right && self.right.isEnabled)) return;
    self.close();

    isAsideExposed = shouldExposeAside;
    if ((self.left && self.left.isEnabled) && (self.right && self.right.isEnabled)) {
      self.content.setMarginLeftAndRight(isAsideExposed ? self.left.width : 0, isAsideExposed ? self.right.width : 0);
    } else if (self.left && self.left.isEnabled) {
      // set the left marget width if it should be exposed
      // otherwise set false so there's no left margin
      self.content.setMarginLeft(isAsideExposed ? self.left.width : 0);
    } else if (self.right && self.right.isEnabled) {
      self.content.setMarginRight(isAsideExposed ? self.right.width : 0);
    }
    self.$scope.$emit('$ionicExposeAside', isAsideExposed);
  };

  self.activeAsideResizing = function(isResizing) {
    $ionicBody.enableClass(isResizing, 'aside-resizing');
  };

  // End a drag with the given event
  self._endDrag = function(e) {
    if (isAsideExposed) return;

    if (isDragging) {
      self.snapToRest(e);
    }
    startX = null;
    lastX = null;
    offsetX = null;
  };

  // Handle a drag event
  self._handleDrag = function(e) {
    if (isAsideExposed || !$scope.dragContent) return;

    // If we don't have start coords, grab and store them
    if (!startX) {
      startX = e.gesture.touches[0].pageX;
      lastX = startX;
    } else {
      // Grab the current tap coords
      lastX = e.gesture.touches[0].pageX;
    }

    // Calculate difference from the tap points
    if (!isDragging && Math.abs(lastX - startX) > self.dragThresholdX) {
      // if the difference is greater than threshold, start dragging using the current
      // point as the starting point
      startX = lastX;

      isDragging = true;
      // Initialize dragging
      self.content.disableAnimation();
      offsetX = self.getOpenAmount();
    }

    if (isDragging) {
      self.openAmount(offsetX + (lastX - startX));
      //self.content.setCanScroll(false);
    }
  };

  self.canDragContent = function(canDrag) {
    if (arguments.length) {
      $scope.dragContent = !!canDrag;
    }
    return $scope.dragContent;
  };

  self.edgeThreshold = 25;
  self.edgeThresholdEnabled = false;
  self.edgeDragThreshold = function(value) {
    if (arguments.length) {
      if (isNumber(value) && value > 0) {
        self.edgeThreshold = value;
        self.edgeThresholdEnabled = true;
      } else {
        self.edgeThresholdEnabled = !!value;
      }
    }
    return self.edgeThresholdEnabled;
  };

  self.isDraggableTarget = function(e) {
    //Only restrict edge when sidemenu is closed and restriction is enabled
    var shouldOnlyAllowEdgeDrag = self.edgeThresholdEnabled && !self.isOpen();
    var startX = e.gesture.startEvent && e.gesture.startEvent.center &&
      e.gesture.startEvent.center.pageX;

    var dragIsWithinBounds = !shouldOnlyAllowEdgeDrag ||
      startX <= self.edgeThreshold ||
      startX >= self.content.element.offsetWidth - self.edgeThreshold;

    var backView = $ionicHistory.backView();
    var menuEnabled = enableMenuWithBackViews ? true : !backView;
    if (!menuEnabled) {
      var currentView = $ionicHistory.currentView() || {};
      return (dragIsWithinBounds && (backView.historyId !== currentView.historyId));
    }

    return ($scope.dragContent || self.isOpen()) &&
      dragIsWithinBounds &&
      !e.gesture.srcEvent.defaultPrevented &&
      menuEnabled &&
      !e.target.tagName.match(/input|textarea|select|object|embed/i) &&
      !e.target.isContentEditable &&
      !(e.target.dataset ? e.target.dataset.preventScroll : e.target.getAttribute('data-prevent-scroll') == 'true');
  };

  $scope.sideMenuContentTranslateX = 0;

  var deregisterBackButtonAction = noop;
  var closeSideMenu = angular.bind(self, self.close);

  $scope.$watch(function() {
    return self.getOpenAmount() !== 0;
  }, function(isOpen) {
    deregisterBackButtonAction();
    if (isOpen) {
      deregisterBackButtonAction = $ionicPlatform.registerBackButtonAction(
        closeSideMenu,
        IONIC_BACK_PRIORITY.sideMenu
      );
    }
  });

  var deregisterInstance = $ionicSideMenuDelegate._registerInstance(
    self, $attrs.delegateHandle, function() {
      return $ionicHistory.isActiveScope($scope);
    }
  );

  $scope.$on('$destroy', function() {
    deregisterInstance();
    deregisterBackButtonAction();
    self.$scope = null;
    if (self.content) {
      self.content.setCanScroll(true);
      self.content.element = null;
      self.content = null;
    }
  });

  self.initialize({
    left: {
      width: 275
    },
    right: {
      width: 275
    }
  });

}]);

(function(ionic) {

  var TRANSLATE32 = 'translate(32,32)';
  var STROKE_OPACITY = 'stroke-opacity';
  var ROUND = 'round';
  var INDEFINITE = 'indefinite';
  var DURATION = '750ms';
  var NONE = 'none';
  var SHORTCUTS = {
    a: 'animate',
    an: 'attributeName',
    at: 'animateTransform',
    c: 'circle',
    da: 'stroke-dasharray',
    os: 'stroke-dashoffset',
    f: 'fill',
    lc: 'stroke-linecap',
    rc: 'repeatCount',
    sw: 'stroke-width',
    t: 'transform',
    v: 'values'
  };

  var SPIN_ANIMATION = {
    v: '0,32,32;360,32,32',
    an: 'transform',
    type: 'rotate',
    rc: INDEFINITE,
    dur: DURATION
  };

  function createSvgElement(tagName, data, parent, spinnerName) {
    var ele = document.createElement(SHORTCUTS[tagName] || tagName);
    var k, x, y;

    for (k in data) {

      if (angular.isArray(data[k])) {
        for (x = 0; x < data[k].length; x++) {
          if (data[k][x].fn) {
            for (y = 0; y < data[k][x].t; y++) {
              createSvgElement(k, data[k][x].fn(y, spinnerName), ele, spinnerName);
            }
          } else {
            createSvgElement(k, data[k][x], ele, spinnerName);
          }
        }

      } else {
        setSvgAttribute(ele, k, data[k]);
      }
    }

    parent.appendChild(ele);
  }

  function setSvgAttribute(ele, k, v) {
    ele.setAttribute(SHORTCUTS[k] || k, v);
  }

  function animationValues(strValues, i) {
    var values = strValues.split(';');
    var back = values.slice(i);
    var front = values.slice(0, values.length - back.length);
    values = back.concat(front).reverse();
    return values.join(';') + ';' + values[0];
  }

  var IOS_SPINNER = {
    sw: 4,
    lc: ROUND,
    line: [{
      fn: function(i, spinnerName) {
        return {
          y1: spinnerName == 'ios' ? 17 : 12,
          y2: spinnerName == 'ios' ? 29 : 20,
          t: TRANSLATE32 + ' rotate(' + (30 * i + (i < 6 ? 180 : -180)) + ')',
          a: [{
            fn: function() {
              return {
                an: STROKE_OPACITY,
                dur: DURATION,
                v: animationValues('0;.1;.15;.25;.35;.45;.55;.65;.7;.85;1', i),
                rc: INDEFINITE
              };
            },
            t: 1
          }]
        };
      },
      t: 12
    }]
  };

  var spinners = {

    android: {
      c: [{
        sw: 6,
        da: 128,
        os: 82,
        r: 26,
        cx: 32,
        cy: 32,
        f: NONE
      }]
    },

    ios: IOS_SPINNER,

    'ios-small': IOS_SPINNER,

    bubbles: {
      sw: 0,
      c: [{
        fn: function(i) {
          return {
            cx: 24 * Math.cos(2 * Math.PI * i / 8),
            cy: 24 * Math.sin(2 * Math.PI * i / 8),
            t: TRANSLATE32,
            a: [{
              fn: function() {
                return {
                  an: 'r',
                  dur: DURATION,
                  v: animationValues('1;2;3;4;5;6;7;8', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }]
          };
        },
        t: 8
      }]
    },

    circles: {

      c: [{
        fn: function(i) {
          return {
            r: 5,
            cx: 24 * Math.cos(2 * Math.PI * i / 8),
            cy: 24 * Math.sin(2 * Math.PI * i / 8),
            t: TRANSLATE32,
            sw: 0,
            a: [{
              fn: function() {
                return {
                  an: 'fill-opacity',
                  dur: DURATION,
                  v: animationValues('.3;.3;.3;.4;.7;.85;.9;1', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }]
          };
        },
        t: 8
      }]
    },

    crescent: {
      c: [{
        sw: 4,
        da: 128,
        os: 82,
        r: 26,
        cx: 32,
        cy: 32,
        f: NONE,
        at: [SPIN_ANIMATION]
      }]
    },

    dots: {

      c: [{
        fn: function(i) {
          return {
            cx: 16 + (16 * i),
            cy: 32,
            sw: 0,
            a: [{
              fn: function() {
                return {
                  an: 'fill-opacity',
                  dur: DURATION,
                  v: animationValues('.5;.6;.8;1;.8;.6;.5', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }, {
              fn: function() {
                return {
                  an: 'r',
                  dur: DURATION,
                  v: animationValues('4;5;6;5;4;3;3', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }]
          };
        },
        t: 3
      }]
    },

    lines: {
      sw: 7,
      lc: ROUND,
      line: [{
        fn: function(i) {
          return {
            x1: 10 + (i * 14),
            x2: 10 + (i * 14),
            a: [{
              fn: function() {
                return {
                  an: 'y1',
                  dur: DURATION,
                  v: animationValues('16;18;28;18;16', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }, {
              fn: function() {
                return {
                  an: 'y2',
                  dur: DURATION,
                  v: animationValues('48;44;36;46;48', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }, {
              fn: function() {
                return {
                  an: STROKE_OPACITY,
                  dur: DURATION,
                  v: animationValues('1;.8;.5;.4;1', i),
                  rc: INDEFINITE
                };
              },
              t: 1
            }]
          };
        },
        t: 4
      }]
    },

    ripple: {
      f: NONE,
      'fill-rule': 'evenodd',
      sw: 3,
      circle: [{
        fn: function(i) {
          return {
            cx: 32,
            cy: 32,
            a: [{
              fn: function() {
                return {
                  an: 'r',
                  begin: (i * -1) + 's',
                  dur: '2s',
                  v: '0;24',
                  keyTimes: '0;1',
                  keySplines: '0.1,0.2,0.3,1',
                  calcMode: 'spline',
                  rc: INDEFINITE
                };
              },
              t: 1
            }, {
              fn: function() {
                return {
                  an: STROKE_OPACITY,
                  begin: (i * -1) + 's',
                  dur: '2s',
                  v: '.2;1;.2;0',
                  rc: INDEFINITE
                };
              },
              t: 1
            }]
          };
        },
        t: 2
      }]
    },

    spiral: {
      defs: [{
        linearGradient: [{
          id: 'sGD',
          gradientUnits: 'userSpaceOnUse',
          x1: 55, y1: 46, x2: 2, y2: 46,
          stop: [{
            offset: 0.1,
            class: 'stop1'
          }, {
            offset: 1,
            class: 'stop2'
          }]
        }]
      }],
      g: [{
        sw: 4,
        lc: ROUND,
        f: NONE,
        path: [{
          stroke: 'url(#sGD)',
          d: 'M4,32 c0,15,12,28,28,28c8,0,16-4,21-9'
        }, {
          d: 'M60,32 C60,16,47.464,4,32,4S4,16,4,32'
        }],
        at: [SPIN_ANIMATION]
      }]
    }

  };

  var animations = {

    android: function(ele) {
      // Note that this is called as a function, not a constructor.
      var self = {};

      this.stop = false;

      var rIndex = 0;
      var rotateCircle = 0;
      var startTime;
      var svgEle = ele.querySelector('g');
      var circleEle = ele.querySelector('circle');

      function run() {
        if (self.stop) return;

        var v = easeInOutCubic(Date.now() - startTime, 650);
        var scaleX = 1;
        var translateX = 0;
        var dasharray = (188 - (58 * v));
        var dashoffset = (182 - (182 * v));

        if (rIndex % 2) {
          scaleX = -1;
          translateX = -64;
          dasharray = (128 - (-58 * v));
          dashoffset = (182 * v);
        }

        var rotateLine = [0, -101, -90, -11, -180, 79, -270, -191][rIndex];

        setSvgAttribute(circleEle, 'da', Math.max(Math.min(dasharray, 188), 128));
        setSvgAttribute(circleEle, 'os', Math.max(Math.min(dashoffset, 182), 0));
        setSvgAttribute(circleEle, 't', 'scale(' + scaleX + ',1) translate(' + translateX + ',0) rotate(' + rotateLine + ',32,32)');

        rotateCircle += 4.1;
        if (rotateCircle > 359) rotateCircle = 0;
        setSvgAttribute(svgEle, 't', 'rotate(' + rotateCircle + ',32,32)');

        if (v >= 1) {
          rIndex++;
          if (rIndex > 7) rIndex = 0;
          startTime = Date.now();
        }

        ionic.requestAnimationFrame(run);
      }

      return function() {
        startTime = Date.now();
        run();
        return self;
      };

    }

  };

  function easeInOutCubic(t, c) {
    t /= c / 2;
    if (t < 1) return 1 / 2 * t * t * t;
    t -= 2;
    return 1 / 2 * (t * t * t + 2);
  }


  IonicModule
  .controller('$ionicSpinner', [
    '$element',
    '$attrs',
    '$ionicConfig',
  function($element, $attrs, $ionicConfig) {
    var spinnerName, anim;

    this.init = function() {
      spinnerName = $attrs.icon || $ionicConfig.spinner.icon();

      var container = document.createElement('div');
      createSvgElement('svg', {
        viewBox: '0 0 64 64',
        g: [spinners[spinnerName]]
      }, container, spinnerName);

      // Specifically for animations to work,
      // Android 4.3 and below requires the element to be
      // added as an html string, rather than dynmically
      // building up the svg element and appending it.
      $element.html(container.innerHTML);

      this.start();

      return spinnerName;
    };

    this.start = function() {
      animations[spinnerName] && (anim = animations[spinnerName]($element[0])());
    };

    this.stop = function() {
      animations[spinnerName] && (anim.stop = true);
    };

  }]);

})(ionic);

IonicModule
.controller('$ionicTab', [
  '$scope',
  '$ionicHistory',
  '$attrs',
  '$location',
  '$state',
function($scope, $ionicHistory, $attrs, $location, $state) {
  this.$scope = $scope;

  //All of these exposed for testing
  this.hrefMatchesState = function() {
    return $attrs.href && $location.path().indexOf(
      $attrs.href.replace(/^#/, '').replace(/\/$/, '')
    ) === 0;
  };
  this.srefMatchesState = function() {
    return $attrs.uiSref && $state.includes($attrs.uiSref.split('(')[0]);
  };
  this.navNameMatchesState = function() {
    return this.navViewName && $ionicHistory.isCurrentStateNavView(this.navViewName);
  };

  this.tabMatchesState = function() {
    return this.hrefMatchesState() || this.srefMatchesState() || this.navNameMatchesState();
  };
}]);

IonicModule
.controller('$ionicTabs', [
  '$scope',
  '$element',
  '$ionicHistory',
function($scope, $element, $ionicHistory) {
  var self = this;
  var selectedTab = null;
  var previousSelectedTab = null;
  var selectedTabIndex;
  var isVisible = true;
  self.tabs = [];

  self.selectedIndex = function() {
    return self.tabs.indexOf(selectedTab);
  };
  self.selectedTab = function() {
    return selectedTab;
  };
  self.previousSelectedTab = function() {
    return previousSelectedTab;
  };

  self.add = function(tab) {
    $ionicHistory.registerHistory(tab);
    self.tabs.push(tab);
  };

  self.remove = function(tab) {
    var tabIndex = self.tabs.indexOf(tab);
    if (tabIndex === -1) {
      return;
    }
    //Use a field like '$tabSelected' so developers won't accidentally set it in controllers etc
    if (tab.$tabSelected) {
      self.deselect(tab);
      //Try to select a new tab if we're removing a tab
      if (self.tabs.length === 1) {
        //Do nothing if there are no other tabs to select
      } else {
        //Select previous tab if it's the last tab, else select next tab
        var newTabIndex = tabIndex === self.tabs.length - 1 ? tabIndex - 1 : tabIndex + 1;
        self.select(self.tabs[newTabIndex]);
      }
    }
    self.tabs.splice(tabIndex, 1);
  };

  self.deselect = function(tab) {
    if (tab.$tabSelected) {
      previousSelectedTab = selectedTab;
      selectedTab = selectedTabIndex = null;
      tab.$tabSelected = false;
      (tab.onDeselect || noop)();
      tab.$broadcast && tab.$broadcast('$ionicHistory.deselect');
    }
  };

  self.select = function(tab, shouldEmitEvent) {
    var tabIndex;
    if (isNumber(tab)) {
      tabIndex = tab;
      if (tabIndex >= self.tabs.length) return;
      tab = self.tabs[tabIndex];
    } else {
      tabIndex = self.tabs.indexOf(tab);
    }

    if (arguments.length === 1) {
      shouldEmitEvent = !!(tab.navViewName || tab.uiSref);
    }

    if (selectedTab && selectedTab.$historyId == tab.$historyId) {
      if (shouldEmitEvent) {
        $ionicHistory.goToHistoryRoot(tab.$historyId);
      }

    } else if (selectedTabIndex !== tabIndex) {
      forEach(self.tabs, function(tab) {
        self.deselect(tab);
      });

      selectedTab = tab;
      selectedTabIndex = tabIndex;

      if (self.$scope && self.$scope.$parent) {
        self.$scope.$parent.$activeHistoryId = tab.$historyId;
      }

      //Use a funny name like $tabSelected so the developer doesn't overwrite the var in a child scope
      tab.$tabSelected = true;
      (tab.onSelect || noop)();

      if (shouldEmitEvent) {
        $scope.$emit('$ionicHistory.change', {
          type: 'tab',
          tabIndex: tabIndex,
          historyId: tab.$historyId,
          navViewName: tab.navViewName,
          hasNavView: !!tab.navViewName,
          title: tab.title,
          url: tab.href,
          uiSref: tab.uiSref
        });
      }

      $scope.$broadcast("tabSelected", { selectedTab: tab, selectedTabIndex: tabIndex});
    }
  };

  self.hasActiveScope = function() {
    for (var x = 0; x < self.tabs.length; x++) {
      if ($ionicHistory.isActiveScope(self.tabs[x])) {
        return true;
      }
    }
    return false;
  };

  self.showBar = function(show) {
    if (arguments.length) {
      if (show) {
        $element.removeClass('tabs-item-hide');
      } else {
        $element.addClass('tabs-item-hide');
      }
      isVisible = !!show;
    }
    return isVisible;
  };
}]);

IonicModule
.controller('$ionicView', [
  '$scope',
  '$element',
  '$attrs',
  '$compile',
  '$rootScope',
function($scope, $element, $attrs, $compile, $rootScope) {
  var self = this;
  var navElementHtml = {};
  var navViewCtrl;
  var navBarDelegateHandle;
  var hasViewHeaderBar;
  var deregisters = [];
  var viewTitle;

  var deregIonNavBarInit = $scope.$on('ionNavBar.init', function(ev, delegateHandle) {
    // this view has its own ion-nav-bar, remember the navBarDelegateHandle for this view
    ev.stopPropagation();
    navBarDelegateHandle = delegateHandle;
  });


  self.init = function() {
    deregIonNavBarInit();

    var modalCtrl = $element.inheritedData('$ionModalController');
    navViewCtrl = $element.inheritedData('$ionNavViewController');

    // don't bother if inside a modal or there's no parent navView
    if (!navViewCtrl || modalCtrl) return;

    // add listeners for when this view changes
    $scope.$on('$ionicView.beforeEnter', self.beforeEnter);
    $scope.$on('$ionicView.afterEnter', afterEnter);
    $scope.$on('$ionicView.beforeLeave', deregisterFns);
  };

  self.beforeEnter = function(ev, transData) {
    // this event was emitted, starting at intial ion-view, then bubbles up
    // only the first ion-view should do something with it, parent ion-views should ignore
    if (transData && !transData.viewNotified) {
      transData.viewNotified = true;

      if (!$rootScope.$$phase) $scope.$digest();
      viewTitle = isDefined($attrs.viewTitle) ? $attrs.viewTitle : $attrs.title;

      var navBarItems = {};
      for (var n in navElementHtml) {
        navBarItems[n] = generateNavBarItem(navElementHtml[n]);
      }

      navViewCtrl.beforeEnter(extend(transData, {
        title: viewTitle,
        showBack: !attrTrue('hideBackButton'),
        navBarItems: navBarItems,
        navBarDelegate: navBarDelegateHandle || null,
        showNavBar: !attrTrue('hideNavBar'),
        hasHeaderBar: !!hasViewHeaderBar
      }));

      // make sure any existing observers are cleaned up
      deregisterFns();
    }
  };


  function afterEnter() {
    // only listen for title updates after it has entered
    // but also deregister the observe before it leaves
    var viewTitleAttr = isDefined($attrs.viewTitle) && 'viewTitle' || isDefined($attrs.title) && 'title';
    if (viewTitleAttr) {
      titleUpdate($attrs[viewTitleAttr]);
      deregisters.push($attrs.$observe(viewTitleAttr, titleUpdate));
    }

    if (isDefined($attrs.hideBackButton)) {
      deregisters.push($scope.$watch($attrs.hideBackButton, function(val) {
        navViewCtrl.showBackButton(!val);
      }));
    }

    if (isDefined($attrs.hideNavBar)) {
      deregisters.push($scope.$watch($attrs.hideNavBar, function(val) {
        navViewCtrl.showBar(!val);
      }));
    }
  }


  function titleUpdate(newTitle) {
    if (isDefined(newTitle) && newTitle !== viewTitle) {
      viewTitle = newTitle;
      navViewCtrl.title(viewTitle);
    }
  }


  function deregisterFns() {
    // remove all existing $attrs.$observe's
    for (var x = 0; x < deregisters.length; x++) {
      deregisters[x]();
    }
    deregisters = [];
  }


  function generateNavBarItem(html) {
    if (html) {
      // every time a view enters we need to recreate its view buttons if they exist
      return $compile(html)($scope.$new());
    }
  }


  function attrTrue(key) {
    return !!$scope.$eval($attrs[key]);
  }


  self.navElement = function(type, html) {
    navElementHtml[type] = html;
  };

}]);

/*
 * We don't document the ionActionSheet directive, we instead document
 * the $ionicActionSheet service
 */
IonicModule
.directive('ionActionSheet', ['$document', function($document) {
  return {
    restrict: 'E',
    scope: true,
    replace: true,
    link: function($scope, $element) {

      var keyUp = function(e) {
        if (e.which == 27) {
          $scope.cancel();
          $scope.$apply();
        }
      };

      var backdropClick = function(e) {
        if (e.target == $element[0]) {
          $scope.cancel();
          $scope.$apply();
        }
      };
      $scope.$on('$destroy', function() {
        $element.remove();
        $document.unbind('keyup', keyUp);
      });

      $document.bind('keyup', keyUp);
      $element.bind('click', backdropClick);
    },
    template: '<div class="action-sheet-backdrop">' +
                '<div class="action-sheet-wrapper">' +
                  '<div class="action-sheet" ng-class="{\'action-sheet-has-icons\': $actionSheetHasIcon}">' +
                    '<div class="action-sheet-group action-sheet-options">' +
                      '<div class="action-sheet-title" ng-if="titleText" ng-bind-html="titleText"></div>' +
                      '<button class="button action-sheet-option" ng-click="buttonClicked($index)" ng-class="b.className" ng-repeat="b in buttons" ng-bind-html="b.text"></button>' +
                      '<button class="button destructive action-sheet-destructive" ng-if="destructiveText" ng-click="destructiveButtonClicked()" ng-bind-html="destructiveText"></button>' +
                    '</div>' +
                    '<div class="action-sheet-group action-sheet-cancel" ng-if="cancelText">' +
                      '<button class="button" ng-click="cancel()" ng-bind-html="cancelText"></button>' +
                    '</div>' +
                  '</div>' +
                '</div>' +
              '</div>'
  };
}]);


/**
 * @ngdoc directive
 * @name ionCheckbox
 * @module ionic
 * @restrict E
 * @codepen hqcju
 * @description
 * The checkbox is no different than the HTML checkbox input, except it's styled differently.
 *
 * The checkbox behaves like any [AngularJS checkbox](http://docs.angularjs.org/api/ng/input/input[checkbox]).
 *
 * @usage
 * ```html
 * <ion-checkbox ng-model="isChecked">Checkbox Label</ion-checkbox>
 * ```
 */

IonicModule
.directive('ionCheckbox', ['$ionicConfig', function($ionicConfig) {
  return {
    restrict: 'E',
    replace: true,
    require: '?ngModel',
    transclude: true,
    template:
      '<label class="item item-checkbox">' +
        '<div class="checkbox checkbox-input-hidden disable-pointer-events">' +
          '<input type="checkbox">' +
          '<i class="checkbox-icon"></i>' +
        '</div>' +
        '<div class="item-content disable-pointer-events" ng-transclude></div>' +
      '</label>',
    compile: function(element, attr) {
      var input = element.find('input');
      forEach({
        'name': attr.name,
        'ng-value': attr.ngValue,
        'ng-model': attr.ngModel,
        'ng-checked': attr.ngChecked,
        'ng-disabled': attr.ngDisabled,
        'ng-true-value': attr.ngTrueValue,
        'ng-false-value': attr.ngFalseValue,
        'ng-change': attr.ngChange,
        'ng-required': attr.ngRequired,
        'required': attr.required
      }, function(value, name) {
        if (isDefined(value)) {
          input.attr(name, value);
        }
      });
      var checkboxWrapper = element[0].querySelector('.checkbox');
      checkboxWrapper.classList.add('checkbox-' + $ionicConfig.form.checkbox());
    }
  };
}]);


/**
 * @ngdoc directive
 * @restrict A
 * @name collectionRepeat
 * @module ionic
 * @codepen 7ec1ec58f2489ab8f359fa1a0fe89c15
 * @description
 * `collection-repeat` allows an app to show huge lists of items much more performantly than
 * `ng-repeat`.
 *
 * It renders into the DOM only as many items as are currently visible.
 *
 * This means that on a phone screen that can fit eight items, only the eight items matching
 * the current scroll position will be rendered.
 *
 * **The Basics**:
 *
 * - The data given to collection-repeat must be an array.
 * - If the `item-height` and `item-width` attributes are not supplied, it will be assumed that
 *   every item in the list has the same dimensions as the first item.
 * - Don't use angular one-time binding (`::`) with collection-repeat. The scope of each item is
 *   assigned new data and re-digested as you scroll. Bindings need to update, and one-time bindings
 *   won't.
 *
 * **Performance Tips**:
 *
 * - The iOS webview has a performance bottleneck when switching out `<img src>` attributes.
 *   To increase performance of images on iOS, cache your images in advance and,
 *   if possible, lower the number of unique images. We're working on [a solution](https://github.com/ionic-team/ionic/issues/3194).
 *
 * @usage
 * #### Basic Item List ([codepen](http://codepen.io/ionic/pen/0c2c35a34a8b18ad4d793fef0b081693))
 * ```html
 * <ion-content>
 *   <ion-item collection-repeat="item in items">
 *     {% raw %}{{item}}{% endraw %}
 *   </ion-item>
 * </ion-content>
 * ```
 *
 * #### Grid of Images ([codepen](http://codepen.io/ionic/pen/5515d4efd9d66f780e96787387f41664))
 * ```html
 * <ion-content>
 *   <img collection-repeat="photo in photos"
 *     item-width="33%"
 *     item-height="200px"
 *     ng-src="{% raw %}{{photo.url}}{% endraw %}">
 * </ion-content>
 * ```
 *
 * #### Horizontal Scroller, Dynamic Item Width ([codepen](http://codepen.io/ionic/pen/67cc56b349124a349acb57a0740e030e))
 * ```html
 * <ion-content>
 *   <h2>Available Kittens:</h2>
 *   <ion-scroll direction="x" class="available-scroller">
 *     <div class="photo" collection-repeat="photo in main.photos"
 *        item-height="250" item-width="photo.width + 30">
 *        <img ng-src="{% raw %}{{photo.src}}{% endraw %}">
 *     </div>
 *   </ion-scroll>
 * </ion-content>
 * ```
 *
 * @param {expression} collection-repeat The expression indicating how to enumerate a collection,
 *   of the format  `variable in expression` – where variable is the user defined loop variable
 *   and `expression` is a scope expression giving the collection to enumerate.
 *   For example: `album in artist.albums` or `album in artist.albums | orderBy:'name'`.
 * @param {expression=} item-width The width of the repeated element. The expression must return
 *   a number (pixels) or a percentage. Defaults to the width of the first item in the list.
 *   (previously named collection-item-width)
 * @param {expression=} item-height The height of the repeated element. The expression must return
 *   a number (pixels) or a percentage. Defaults to the height of the first item in the list.
 *   (previously named collection-item-height)
 * @param {number=} item-render-buffer The number of items to load before and after the visible
 *   items in the list. Default 3. Tip: set this higher if you have lots of images to preload, but
 *   don't set it too high or you'll see performance loss.
 * @param {boolean=} force-refresh-images Force images to refresh as you scroll. This fixes a problem
 *   where, when an element is interchanged as scrolling, its image will still have the old src
 *   while the new src loads. Setting this to true comes with a small performance loss.
 */

IonicModule
.directive('collectionRepeat', CollectionRepeatDirective)
.factory('$ionicCollectionManager', RepeatManagerFactory);

var ONE_PX_TRANSPARENT_IMG_SRC = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7';
var WIDTH_HEIGHT_REGEX = /height:.*?px;\s*width:.*?px/;
var DEFAULT_RENDER_BUFFER = 3;

CollectionRepeatDirective.$inject = ['$ionicCollectionManager', '$parse', '$window', '$$rAF', '$rootScope', '$timeout'];
function CollectionRepeatDirective($ionicCollectionManager, $parse, $window, $$rAF, $rootScope, $timeout) {
  return {
    restrict: 'A',
    priority: 1000,
    transclude: 'element',
    $$tlb: true,
    require: '^^$ionicScroll',
    link: postLink
  };

  function postLink(scope, element, attr, scrollCtrl, transclude) {
    var scrollView = scrollCtrl.scrollView;
    var node = element[0];
    var containerNode = angular.element('<div class="collection-repeat-container">')[0];
    node.parentNode.replaceChild(containerNode, node);

    if (scrollView.options.scrollingX && scrollView.options.scrollingY) {
      throw new Error("collection-repeat expected a parent x or y scrollView, not " +
                      "an xy scrollView.");
    }

    var repeatExpr = attr.collectionRepeat;
    var match = repeatExpr.match(/^\s*([\s\S]+?)\s+in\s+([\s\S]+?)(?:\s+track\s+by\s+([\s\S]+?))?\s*$/);
    if (!match) {
      throw new Error("collection-repeat expected expression in form of '_item_ in " +
                      "_collection_[ track by _id_]' but got '" + attr.collectionRepeat + "'.");
    }
    var keyExpr = match[1];
    var listExpr = match[2];
    var listGetter = $parse(listExpr);
    var heightData = {};
    var widthData = {};
    var computedStyleDimensions = {};
    var data = [];
    var repeatManager;

    // attr.collectionBufferSize is deprecated
    var renderBufferExpr = attr.itemRenderBuffer || attr.collectionBufferSize;
    var renderBuffer = angular.isDefined(renderBufferExpr) ?
      parseInt(renderBufferExpr) :
      DEFAULT_RENDER_BUFFER;

    // attr.collectionItemHeight is deprecated
    var heightExpr = attr.itemHeight || attr.collectionItemHeight;
    // attr.collectionItemWidth is deprecated
    var widthExpr = attr.itemWidth || attr.collectionItemWidth;

    var afterItemsContainer = initAfterItemsContainer();

    var changeValidator = makeChangeValidator();
    initDimensions();

    // Dimensions are refreshed on resize or data change.
    scrollCtrl.$element.on('scroll-resize', refreshDimensions);

    angular.element($window).on('resize', onResize);
    var unlistenToExposeAside = $rootScope.$on('$ionicExposeAside', ionic.animationFrameThrottle(function() {
      scrollCtrl.scrollView.resize();
      onResize();
    }));
    $timeout(refreshDimensions, 0, false);

    function onResize() {
      if (changeValidator.resizeRequiresRefresh(scrollView.__clientWidth, scrollView.__clientHeight)) {
        refreshDimensions();
      }
    }

    scope.$watchCollection(listGetter, function(newValue) {
      data = newValue || (newValue = []);
      if (!angular.isArray(newValue)) {
        throw new Error("collection-repeat expected an array for '" + listExpr + "', " +
          "but got a " + typeof value);
      }
      // Wait for this digest to end before refreshing everything.
      scope.$$postDigest(function() {
        getRepeatManager().setData(data);
        if (changeValidator.dataChangeRequiresRefresh(data)) refreshDimensions();
      });
    });

    scope.$on('$destroy', function() {
      angular.element($window).off('resize', onResize);
      unlistenToExposeAside();
      scrollCtrl.$element && scrollCtrl.$element.off('scroll-resize', refreshDimensions);

      computedStyleNode && computedStyleNode.parentNode &&
        computedStyleNode.parentNode.removeChild(computedStyleNode);
      computedStyleScope && computedStyleScope.$destroy();
      computedStyleScope = computedStyleNode = null;

      repeatManager && repeatManager.destroy();
      repeatManager = null;
    });

    function makeChangeValidator() {
      var self;
      return (self = {
        dataLength: 0,
        width: 0,
        height: 0,
        // A resize triggers a refresh only if we have data, the scrollView has size,
        // and the size has changed.
        resizeRequiresRefresh: function(newWidth, newHeight) {
          var requiresRefresh = self.dataLength && newWidth && newHeight &&
            (newWidth !== self.width || newHeight !== self.height);

          self.width = newWidth;
          self.height = newHeight;

          return !!requiresRefresh;
        },
        // A change in data only triggers a refresh if the data has length, or if the data's
        // length is less than before.
        dataChangeRequiresRefresh: function(newData) {
          var requiresRefresh = newData.length > 0 || newData.length < self.dataLength;

          self.dataLength = newData.length;

          return !!requiresRefresh;
        }
      });
    }

    function getRepeatManager() {
      return repeatManager || (repeatManager = new $ionicCollectionManager({
        afterItemsNode: afterItemsContainer[0],
        containerNode: containerNode,
        heightData: heightData,
        widthData: widthData,
        forceRefreshImages: !!(isDefined(attr.forceRefreshImages) && attr.forceRefreshImages !== 'false'),
        keyExpression: keyExpr,
        renderBuffer: renderBuffer,
        scope: scope,
        scrollView: scrollCtrl.scrollView,
        transclude: transclude
      }));
    }

    function initAfterItemsContainer() {
      var container = angular.element(
        scrollView.__content.querySelector('.collection-repeat-after-container')
      );
      // Put everything in the view after the repeater into a container.
      if (!container.length) {
        var elementIsAfterRepeater = false;
        var afterNodes = [].filter.call(scrollView.__content.childNodes, function(node) {
          if (ionic.DomUtil.contains(node, containerNode)) {
            elementIsAfterRepeater = true;
            return false;
          }
          return elementIsAfterRepeater;
        });
        container = angular.element('<span class="collection-repeat-after-container">');
        if (scrollView.options.scrollingX) {
          container.addClass('horizontal');
        }
        container.append(afterNodes);
        scrollView.__content.appendChild(container[0]);
      }
      return container;
    }

    function initDimensions() {
      //Height and width have four 'modes':
      //1) Computed Mode
      //  - Nothing is supplied, so we getComputedStyle() on one element in the list and use
      //    that width and height value for the width and height of every item. This is re-computed
      //    every resize.
      //2) Constant Mode, Static Integer
      //  - The user provides a constant number for width or height, in pixels. We parse it,
      //    store it on the `value` field, and it never changes
      //3) Constant Mode, Percent
      //  - The user provides a percent string for width or height. The getter for percent is
      //    stored on the `getValue()` field, and is re-evaluated once every resize. The result
      //    is stored on the `value` field.
      //4) Dynamic Mode
      //  - The user provides a dynamic expression for the width or height.  This is re-evaluated
      //    for every item, stored on the `.getValue()` field.
      if (heightExpr) {
        parseDimensionAttr(heightExpr, heightData);
      } else {
        heightData.computed = true;
      }
      if (widthExpr) {
        parseDimensionAttr(widthExpr, widthData);
      } else {
        widthData.computed = true;
      }
    }

    function refreshDimensions() {
      var hasData = data.length > 0;

      if (hasData && (heightData.computed || widthData.computed)) {
        computeStyleDimensions();
      }

      if (hasData && heightData.computed) {
        heightData.value = computedStyleDimensions.height;
        if (!heightData.value) {
          throw new Error('collection-repeat tried to compute the height of repeated elements "' +
            repeatExpr + '", but was unable to. Please provide the "item-height" attribute. ' +
            'http://ionicframework.com/docs/api/directive/collectionRepeat/');
        }
      } else if (!heightData.dynamic && heightData.getValue) {
        // If it's a constant with a getter (eg percent), we just refresh .value after resize
        heightData.value = heightData.getValue();
      }

      if (hasData && widthData.computed) {
        widthData.value = computedStyleDimensions.width;
        if (!widthData.value) {
          throw new Error('collection-repeat tried to compute the width of repeated elements "' +
            repeatExpr + '", but was unable to. Please provide the "item-width" attribute. ' +
            'http://ionicframework.com/docs/api/directive/collectionRepeat/');
        }
      } else if (!widthData.dynamic && widthData.getValue) {
        // If it's a constant with a getter (eg percent), we just refresh .value after resize
        widthData.value = widthData.getValue();
      }
      // Dynamic dimensions aren't updated on resize. Since they're already dynamic anyway,
      // .getValue() will be used.

      getRepeatManager().refreshLayout();
    }

    function parseDimensionAttr(attrValue, dimensionData) {
      if (!attrValue) return;

      var parsedValue;
      // Try to just parse the plain attr value
      try {
        parsedValue = $parse(attrValue);
      } catch (e) {
        // If the parse fails and the value has `px` or `%` in it, surround the attr in
        // quotes, to attempt to let the user provide a simple `attr="100%"` or `attr="100px"`
        if (attrValue.trim().match(/\d+(px|%)$/)) {
          attrValue = '"' + attrValue + '"';
        }
        parsedValue = $parse(attrValue);
      }

      var constantAttrValue = attrValue.replace(/(\'|\"|px|%)/g, '').trim();
      var isConstant = constantAttrValue.length && !/([a-zA-Z]|\$|:|\?)/.test(constantAttrValue);
      dimensionData.attrValue = attrValue;

      // If it's a constant, it's either a percent or just a constant pixel number.
      if (isConstant) {
        // For percents, store the percent getter on .getValue()
        if (attrValue.indexOf('%') > -1) {
          var decimalValue = parseFloat(parsedValue()) / 100;
          dimensionData.getValue = dimensionData === heightData ?
            function() { return Math.floor(decimalValue * scrollView.__clientHeight); } :
            function() { return Math.floor(decimalValue * scrollView.__clientWidth); };
        } else {
          // For static constants, just store the static constant.
          dimensionData.value = parseInt(parsedValue());
        }

      } else {
        dimensionData.dynamic = true;
        dimensionData.getValue = dimensionData === heightData ?
          function heightGetter(scope, locals) {
            var result = parsedValue(scope, locals);
            if (result.charAt && result.charAt(result.length - 1) === '%') {
              return Math.floor(parseFloat(result) / 100 * scrollView.__clientHeight);
            }
            return parseInt(result);
          } :
          function widthGetter(scope, locals) {
            var result = parsedValue(scope, locals);
            if (result.charAt && result.charAt(result.length - 1) === '%') {
              return Math.floor(parseFloat(result) / 100 * scrollView.__clientWidth);
            }
            return parseInt(result);
          };
      }
    }

    var computedStyleNode;
    var computedStyleScope;
    function computeStyleDimensions() {
      if (!computedStyleNode) {
        transclude(computedStyleScope = scope.$new(), function(clone) {
          clone[0].removeAttribute('collection-repeat'); // remove absolute position styling
          computedStyleNode = clone[0];
        });
      }

      computedStyleScope[keyExpr] = (listGetter(scope) || [])[0];
      if (!$rootScope.$$phase) computedStyleScope.$digest();
      containerNode.appendChild(computedStyleNode);

      var style = $window.getComputedStyle(computedStyleNode);
      computedStyleDimensions.width = parseInt(style.width);
      computedStyleDimensions.height = parseInt(style.height);

      containerNode.removeChild(computedStyleNode);
    }

  }

}

RepeatManagerFactory.$inject = ['$rootScope', '$window', '$$rAF'];
function RepeatManagerFactory($rootScope, $window, $$rAF) {
  var EMPTY_DIMENSION = { primaryPos: 0, secondaryPos: 0, primarySize: 0, secondarySize: 0, rowPrimarySize: 0 };

  return function RepeatController(options) {
    var afterItemsNode = options.afterItemsNode;
    var containerNode = options.containerNode;
    var forceRefreshImages = options.forceRefreshImages;
    var heightData = options.heightData;
    var widthData = options.widthData;
    var keyExpression = options.keyExpression;
    var renderBuffer = options.renderBuffer;
    var scope = options.scope;
    var scrollView = options.scrollView;
    var transclude = options.transclude;

    var data = [];

    var getterLocals = {};
    var heightFn = heightData.getValue || function() { return heightData.value; };
    var heightGetter = function(index, value) {
      getterLocals[keyExpression] = value;
      getterLocals.$index = index;
      return heightFn(scope, getterLocals);
    };

    var widthFn = widthData.getValue || function() { return widthData.value; };
    var widthGetter = function(index, value) {
      getterLocals[keyExpression] = value;
      getterLocals.$index = index;
      return widthFn(scope, getterLocals);
    };

    var isVertical = !!scrollView.options.scrollingY;

    // We say it's a grid view if we're either dynamic or not 100% width
    var isGridView = isVertical ?
      (widthData.dynamic || widthData.value !== scrollView.__clientWidth) :
      (heightData.dynamic || heightData.value !== scrollView.__clientHeight);

    var isStaticView = !heightData.dynamic && !widthData.dynamic;

    var PRIMARY = 'PRIMARY';
    var SECONDARY = 'SECONDARY';
    var TRANSLATE_TEMPLATE_STR = isVertical ?
      'translate3d(SECONDARYpx,PRIMARYpx,0)' :
      'translate3d(PRIMARYpx,SECONDARYpx,0)';
    var WIDTH_HEIGHT_TEMPLATE_STR = isVertical ?
      'height: PRIMARYpx; width: SECONDARYpx;' :
      'height: SECONDARYpx; width: PRIMARYpx;';

    var estimatedHeight;
    var estimatedWidth;

    var repeaterBeforeSize = 0;
    var repeaterAfterSize = 0;

    var renderStartIndex = -1;
    var renderEndIndex = -1;
    var renderAfterBoundary = -1;
    var renderBeforeBoundary = -1;

    var itemsPool = [];
    var itemsLeaving = [];
    var itemsEntering = [];
    var itemsShownMap = {};
    var nextItemId = 0;

    var scrollViewSetDimensions = isVertical ?
      function() { scrollView.setDimensions(null, null, null, view.getContentSize(), true); } :
      function() { scrollView.setDimensions(null, null, view.getContentSize(), null, true); };

    // view is a mix of list/grid methods + static/dynamic methods.
    // See bottom for implementations. Available methods:
    //
    // getEstimatedPrimaryPos(i), getEstimatedSecondaryPos(i), getEstimatedIndex(scrollTop),
    // calculateDimensions(toIndex), getDimensions(index),
    // updateRenderRange(scrollTop, scrollValueEnd), onRefreshLayout(), onRefreshData()
    var view = isVertical ? new VerticalViewType() : new HorizontalViewType();
    (isGridView ? GridViewType : ListViewType).call(view);
    (isStaticView ? StaticViewType : DynamicViewType).call(view);

    var contentSizeStr = isVertical ? 'getContentHeight' : 'getContentWidth';
    var originalGetContentSize = scrollView.options[contentSizeStr];
    scrollView.options[contentSizeStr] = angular.bind(view, view.getContentSize);

    scrollView.__$callback = scrollView.__callback;
    scrollView.__callback = function(transformLeft, transformTop, zoom, wasResize) {
      var scrollValue = view.getScrollValue();
      if (renderStartIndex === -1 ||
          scrollValue + view.scrollPrimarySize > renderAfterBoundary ||
          scrollValue < renderBeforeBoundary) {
        render();
      }
      scrollView.__$callback(transformLeft, transformTop, zoom, wasResize);
    };

    var isLayoutReady = false;
    var isDataReady = false;
    this.refreshLayout = function() {
      if (data.length) {
        estimatedHeight = heightGetter(0, data[0]);
        estimatedWidth = widthGetter(0, data[0]);
      } else {
        // If we don't have any data in our array, just guess.
        estimatedHeight = 100;
        estimatedWidth = 100;
      }

      // Get the size of every element AFTER the repeater. We have to get the margin before and
      // after the first/last element to fix a browser bug with getComputedStyle() not counting
      // the first/last child's margins into height.
      var style = getComputedStyle(afterItemsNode) || {};
      var firstStyle = afterItemsNode.firstElementChild && getComputedStyle(afterItemsNode.firstElementChild) || {};
      var lastStyle = afterItemsNode.lastElementChild && getComputedStyle(afterItemsNode.lastElementChild) || {};
      repeaterAfterSize = (parseInt(style[isVertical ? 'height' : 'width']) || 0) +
        (firstStyle && parseInt(firstStyle[isVertical ? 'marginTop' : 'marginLeft']) || 0) +
        (lastStyle && parseInt(lastStyle[isVertical ? 'marginBottom' : 'marginRight']) || 0);

      // Get the offsetTop of the repeater.
      repeaterBeforeSize = 0;
      var current = containerNode;
      do {
        repeaterBeforeSize += current[isVertical ? 'offsetTop' : 'offsetLeft'];
      } while ( ionic.DomUtil.contains(scrollView.__content, current = current.offsetParent) );

      var containerPrevNode = containerNode.previousElementSibling;
      var beforeStyle = containerPrevNode ? $window.getComputedStyle(containerPrevNode) : {};
      var beforeMargin = parseInt(beforeStyle[isVertical ? 'marginBottom' : 'marginRight'] || 0);

      // Because we position the collection container with position: relative, it doesn't take
      // into account where to position itself relative to the previous element's marginBottom.
      // To compensate, we translate the container up by the previous element's margin.
      containerNode.style[ionic.CSS.TRANSFORM] = TRANSLATE_TEMPLATE_STR
        .replace(PRIMARY, -beforeMargin)
        .replace(SECONDARY, 0);
      repeaterBeforeSize -= beforeMargin;

      if (!scrollView.__clientHeight || !scrollView.__clientWidth) {
        scrollView.__clientWidth = scrollView.__container.clientWidth;
        scrollView.__clientHeight = scrollView.__container.clientHeight;
      }

      (view.onRefreshLayout || angular.noop)();
      view.refreshDirection();
      scrollViewSetDimensions();

      // Create the pool of items for reuse, setting the size to (estimatedItemsOnScreen) * 2,
      // plus the size of the renderBuffer.
      if (!isLayoutReady) {
        var poolSize = Math.max(20, renderBuffer * 3);
        for (var i = 0; i < poolSize; i++) {
          itemsPool.push(new RepeatItem());
        }
      }

      isLayoutReady = true;
      if (isLayoutReady && isDataReady) {
        // If the resize or latest data change caused the scrollValue to
        // now be out of bounds, resize the scrollView.
        if (scrollView.__scrollLeft > scrollView.__maxScrollLeft ||
            scrollView.__scrollTop > scrollView.__maxScrollTop) {
          scrollView.resize();
        }
        forceRerender(true);
      }
    };

    this.setData = function(newData) {
      data = newData;
      (view.onRefreshData || angular.noop)();
      isDataReady = true;
    };

    this.destroy = function() {
      render.destroyed = true;

      itemsPool.forEach(function(item) {
        item.scope.$destroy();
        item.scope = item.element = item.node = item.images = null;
      });
      itemsPool.length = itemsEntering.length = itemsLeaving.length = 0;
      itemsShownMap = {};

      //Restore the scrollView's normal behavior and resize it to normal size.
      scrollView.options[contentSizeStr] = originalGetContentSize;
      scrollView.__callback = scrollView.__$callback;
      scrollView.resize();

      (view.onDestroy || angular.noop)();
    };

    function forceRerender() {
      return render(true);
    }
    function render(forceRerender) {
      if (render.destroyed) return;
      var i;
      var ii;
      var item;
      var dim;
      var scope;
      var scrollValue = view.getScrollValue();
      var scrollValueEnd = scrollValue + view.scrollPrimarySize;

      view.updateRenderRange(scrollValue, scrollValueEnd);

      renderStartIndex = Math.max(0, renderStartIndex - renderBuffer);
      renderEndIndex = Math.min(data.length - 1, renderEndIndex + renderBuffer);

      for (i in itemsShownMap) {
        if (i < renderStartIndex || i > renderEndIndex) {
          item = itemsShownMap[i];
          delete itemsShownMap[i];
          itemsLeaving.push(item);
          item.isShown = false;
        }
      }

      // Render indicies that aren't shown yet
      //
      // NOTE(ajoslin): this may sound crazy, but calling any other functions during this render
      // loop will often push the render time over the edge from less than one frame to over
      // one frame, causing visible jank.
      // DON'T call any other functions inside this loop unless it's vital.
      for (i = renderStartIndex; i <= renderEndIndex; i++) {
        // We only go forward with render if the index is in data, the item isn't already shown,
        // or forceRerender is on.
        if (i >= data.length || (itemsShownMap[i] && !forceRerender)) continue;

        item = itemsShownMap[i] || (itemsShownMap[i] = itemsLeaving.length ? itemsLeaving.pop() :
                                    itemsPool.length ? itemsPool.shift() :
                                    new RepeatItem());
        itemsEntering.push(item);
        item.isShown = true;

        scope = item.scope;
        scope.$index = i;
        scope[keyExpression] = data[i];
        scope.$first = (i === 0);
        scope.$last = (i === (data.length - 1));
        scope.$middle = !(scope.$first || scope.$last);
        scope.$odd = !(scope.$even = (i & 1) === 0);

        if (scope.$$disconnected) ionic.Utils.reconnectScope(item.scope);

        dim = view.getDimensions(i);
        if (item.secondaryPos !== dim.secondaryPos || item.primaryPos !== dim.primaryPos) {
          item.node.style[ionic.CSS.TRANSFORM] = TRANSLATE_TEMPLATE_STR
            .replace(PRIMARY, (item.primaryPos = dim.primaryPos))
            .replace(SECONDARY, (item.secondaryPos = dim.secondaryPos));
        }
        if (item.secondarySize !== dim.secondarySize || item.primarySize !== dim.primarySize) {
          item.node.style.cssText = item.node.style.cssText
            .replace(WIDTH_HEIGHT_REGEX, WIDTH_HEIGHT_TEMPLATE_STR
              //TODO fix item.primarySize + 1 hack
              .replace(PRIMARY, (item.primarySize = dim.primarySize) + 1)
              .replace(SECONDARY, (item.secondarySize = dim.secondarySize))
            );
        }

      }

      // If we reach the end of the list, render the afterItemsNode - this contains all the
      // elements the developer placed after the collection-repeat
      if (renderEndIndex === data.length - 1) {
        dim = view.getDimensions(data.length - 1) || EMPTY_DIMENSION;
        afterItemsNode.style[ionic.CSS.TRANSFORM] = TRANSLATE_TEMPLATE_STR
          .replace(PRIMARY, dim.primaryPos + dim.primarySize)
          .replace(SECONDARY, 0);
      }

      while (itemsLeaving.length) {
        item = itemsLeaving.pop();
        item.scope.$broadcast('$collectionRepeatLeave');
        ionic.Utils.disconnectScope(item.scope);
        itemsPool.push(item);
        item.node.style[ionic.CSS.TRANSFORM] = 'translate3d(-9999px,-9999px,0)';
        item.primaryPos = item.secondaryPos = null;
      }

      if (forceRefreshImages) {
        for (i = 0, ii = itemsEntering.length; i < ii && (item = itemsEntering[i]); i++) {
          if (!item.images) continue;
          for (var j = 0, jj = item.images.length, img; j < jj && (img = item.images[j]); j++) {
            var src = img.src;
            img.src = ONE_PX_TRANSPARENT_IMG_SRC;
            img.src = src;
          }
        }
      }
      if (forceRerender) {
        var rootScopePhase = $rootScope.$$phase;
        while (itemsEntering.length) {
          item = itemsEntering.pop();
          if (!rootScopePhase) item.scope.$digest();
        }
      } else {
        digestEnteringItems();
      }
    }

    function digestEnteringItems() {
      var item;
      if (digestEnteringItems.running) return;
      digestEnteringItems.running = true;

      $$rAF(function process() {
        var rootScopePhase = $rootScope.$$phase;
        while (itemsEntering.length) {
          item = itemsEntering.pop();
          if (item.isShown) {
            if (!rootScopePhase) item.scope.$digest();
          }
        }
        digestEnteringItems.running = false;
      });
    }

    function RepeatItem() {
      var self = this;
      this.scope = scope.$new();
      this.id = 'item' + (nextItemId++);
      transclude(this.scope, function(clone) {
        self.element = clone;
        self.element.data('$$collectionRepeatItem', self);
        // TODO destroy
        self.node = clone[0];
        // Batch style setting to lower repaints
        self.node.style[ionic.CSS.TRANSFORM] = 'translate3d(-9999px,-9999px,0)';
        self.node.style.cssText += ' height: 0px; width: 0px;';
        ionic.Utils.disconnectScope(self.scope);
        containerNode.appendChild(self.node);
        self.images = clone[0].getElementsByTagName('img');
      });
    }

    function VerticalViewType() {
      this.getItemPrimarySize = heightGetter;
      this.getItemSecondarySize = widthGetter;

      this.getScrollValue = function() {
        return Math.max(0, Math.min(scrollView.__scrollTop - repeaterBeforeSize,
          scrollView.__maxScrollTop - repeaterBeforeSize - repeaterAfterSize));
      };

      this.refreshDirection = function() {
        this.scrollPrimarySize = scrollView.__clientHeight;
        this.scrollSecondarySize = scrollView.__clientWidth;

        this.estimatedPrimarySize = estimatedHeight;
        this.estimatedSecondarySize = estimatedWidth;
        this.estimatedItemsAcross = isGridView &&
          Math.floor(scrollView.__clientWidth / estimatedWidth) ||
          1;
      };
    }
    function HorizontalViewType() {
      this.getItemPrimarySize = widthGetter;
      this.getItemSecondarySize = heightGetter;

      this.getScrollValue = function() {
        return Math.max(0, Math.min(scrollView.__scrollLeft - repeaterBeforeSize,
          scrollView.__maxScrollLeft - repeaterBeforeSize - repeaterAfterSize));
      };

      this.refreshDirection = function() {
        this.scrollPrimarySize = scrollView.__clientWidth;
        this.scrollSecondarySize = scrollView.__clientHeight;

        this.estimatedPrimarySize = estimatedWidth;
        this.estimatedSecondarySize = estimatedHeight;
        this.estimatedItemsAcross = isGridView &&
          Math.floor(scrollView.__clientHeight / estimatedHeight) ||
          1;
      };
    }

    function GridViewType() {
      this.getEstimatedSecondaryPos = function(index) {
        return (index % this.estimatedItemsAcross) * this.estimatedSecondarySize;
      };
      this.getEstimatedPrimaryPos = function(index) {
        return Math.floor(index / this.estimatedItemsAcross) * this.estimatedPrimarySize;
      };
      this.getEstimatedIndex = function(scrollValue) {
        return Math.floor(scrollValue / this.estimatedPrimarySize) *
          this.estimatedItemsAcross;
      };
    }

    function ListViewType() {
      this.getEstimatedSecondaryPos = function() {
        return 0;
      };
      this.getEstimatedPrimaryPos = function(index) {
        return index * this.estimatedPrimarySize;
      };
      this.getEstimatedIndex = function(scrollValue) {
        return Math.floor((scrollValue) / this.estimatedPrimarySize);
      };
    }

    function StaticViewType() {
      this.getContentSize = function() {
        return this.getEstimatedPrimaryPos(data.length - 1) + this.estimatedPrimarySize +
          repeaterBeforeSize + repeaterAfterSize;
      };
      // static view always returns the same object for getDimensions, to avoid memory allocation
      // while scrolling. This could be dangerous if this was a public function, but it's not.
      // Only we use it.
      var dim = {};
      this.getDimensions = function(index) {
        dim.primaryPos = this.getEstimatedPrimaryPos(index);
        dim.secondaryPos = this.getEstimatedSecondaryPos(index);
        dim.primarySize = this.estimatedPrimarySize;
        dim.secondarySize = this.estimatedSecondarySize;
        return dim;
      };
      this.updateRenderRange = function(scrollValue, scrollValueEnd) {
        renderStartIndex = Math.max(0, this.getEstimatedIndex(scrollValue));

        // Make sure the renderEndIndex takes into account all the items on the row
        renderEndIndex = Math.min(data.length - 1,
          this.getEstimatedIndex(scrollValueEnd) + this.estimatedItemsAcross - 1);

        renderBeforeBoundary = Math.max(0,
          this.getEstimatedPrimaryPos(renderStartIndex));
        renderAfterBoundary = this.getEstimatedPrimaryPos(renderEndIndex) +
          this.estimatedPrimarySize;
      };
    }

    function DynamicViewType() {
      var self = this;
      var debouncedScrollViewSetDimensions = ionic.debounce(scrollViewSetDimensions, 25, true);
      var calculateDimensions = isGridView ? calculateDimensionsGrid : calculateDimensionsList;
      var dimensionsIndex;
      var dimensions = [];


      // Get the dimensions at index. {width, height, left, top}.
      // We start with no dimensions calculated, then any time dimensions are asked for at an
      // index we calculate dimensions up to there.
      function calculateDimensionsList(toIndex) {
        var i, prevDimension, dim;
        for (i = Math.max(0, dimensionsIndex); i <= toIndex && (dim = dimensions[i]); i++) {
          prevDimension = dimensions[i - 1] || EMPTY_DIMENSION;
          dim.primarySize = self.getItemPrimarySize(i, data[i]);
          dim.secondarySize = self.scrollSecondarySize;
          dim.primaryPos = prevDimension.primaryPos + prevDimension.primarySize;
          dim.secondaryPos = 0;
        }
      }
      function calculateDimensionsGrid(toIndex) {
        var i, prevDimension, dim;
        for (i = Math.max(dimensionsIndex, 0); i <= toIndex && (dim = dimensions[i]); i++) {
          prevDimension = dimensions[i - 1] || EMPTY_DIMENSION;
          dim.secondarySize = Math.min(
            self.getItemSecondarySize(i, data[i]),
            self.scrollSecondarySize
          );
          dim.secondaryPos = prevDimension.secondaryPos + prevDimension.secondarySize;

          if (i === 0 || dim.secondaryPos + dim.secondarySize > self.scrollSecondarySize) {
            dim.secondaryPos = 0;
            dim.primarySize = self.getItemPrimarySize(i, data[i]);
            dim.primaryPos = prevDimension.primaryPos + prevDimension.rowPrimarySize;

            dim.rowStartIndex = i;
            dim.rowPrimarySize = dim.primarySize;
          } else {
            dim.primarySize = self.getItemPrimarySize(i, data[i]);
            dim.primaryPos = prevDimension.primaryPos;
            dim.rowStartIndex = prevDimension.rowStartIndex;

            dimensions[dim.rowStartIndex].rowPrimarySize = dim.rowPrimarySize = Math.max(
              dimensions[dim.rowStartIndex].rowPrimarySize,
              dim.primarySize
            );
            dim.rowPrimarySize = Math.max(dim.primarySize, dim.rowPrimarySize);
          }
        }
      }

      this.getContentSize = function() {
        var dim = dimensions[dimensionsIndex] || EMPTY_DIMENSION;
        return ((dim.primaryPos + dim.primarySize) || 0) +
          this.getEstimatedPrimaryPos(data.length - dimensionsIndex - 1) +
          repeaterBeforeSize + repeaterAfterSize;
      };
      this.onDestroy = function() {
        dimensions.length = 0;
      };

      this.onRefreshData = function() {
        var i;
        var ii;
        // Make sure dimensions has as many items as data.length.
        // This is to be sure we don't have to allocate objects while scrolling.
        for (i = dimensions.length, ii = data.length; i < ii; i++) {
          dimensions.push({});
        }
        dimensionsIndex = -1;
      };
      this.onRefreshLayout = function() {
        dimensionsIndex = -1;
      };
      this.getDimensions = function(index) {
        index = Math.min(index, data.length - 1);

        if (dimensionsIndex < index) {
          // Once we start asking for dimensions near the end of the list, go ahead and calculate
          // everything. This is to make sure when the user gets to the end of the list, the
          // scroll height of the list is 100% accurate (not estimated anymore).
          if (index > data.length * 0.9) {
            calculateDimensions(data.length - 1);
            dimensionsIndex = data.length - 1;
            scrollViewSetDimensions();
          } else {
            calculateDimensions(index);
            dimensionsIndex = index;
            debouncedScrollViewSetDimensions();
          }

        }
        return dimensions[index];
      };

      var oldRenderStartIndex = -1;
      var oldScrollValue = -1;
      this.updateRenderRange = function(scrollValue, scrollValueEnd) {
        var i;
        var len;
        var dim;

        // Calculate more dimensions than we estimate we'll need, to be sure.
        this.getDimensions( this.getEstimatedIndex(scrollValueEnd) * 2 );

        // -- Calculate renderStartIndex
        // base case: start at 0
        if (oldRenderStartIndex === -1 || scrollValue === 0) {
          i = 0;
        // scrolling down
        } else if (scrollValue >= oldScrollValue) {
          for (i = oldRenderStartIndex, len = data.length; i < len; i++) {
            if ((dim = this.getDimensions(i)) && dim.primaryPos + dim.rowPrimarySize >= scrollValue) {
              break;
            }
          }
        // scrolling up
        } else {
          for (i = oldRenderStartIndex; i >= 0; i--) {
            if ((dim = this.getDimensions(i)) && dim.primaryPos <= scrollValue) {
              // when grid view, make sure the render starts at the beginning of a row.
              i = isGridView ? dim.rowStartIndex : i;
              break;
            }
          }
        }

        renderStartIndex = Math.min(Math.max(0, i), data.length - 1);
        renderBeforeBoundary = renderStartIndex !== -1 ? this.getDimensions(renderStartIndex).primaryPos : -1;

        // -- Calculate renderEndIndex
        var lastRowDim;
        for (i = renderStartIndex + 1, len = data.length; i < len; i++) {
          if ((dim = this.getDimensions(i)) && dim.primaryPos + dim.rowPrimarySize > scrollValueEnd) {

            // Go all the way to the end of the row if we're in a grid
            if (isGridView) {
              lastRowDim = dim;
              while (i < len - 1 &&
                    (dim = this.getDimensions(i + 1)).primaryPos === lastRowDim.primaryPos) {
                i++;
              }
            }
            break;
          }
        }

        renderEndIndex = Math.min(i, data.length - 1);
        renderAfterBoundary = renderEndIndex !== -1 ?
          ((dim = this.getDimensions(renderEndIndex)).primaryPos + (dim.rowPrimarySize || dim.primarySize)) :
          -1;

        oldScrollValue = scrollValue;
        oldRenderStartIndex = renderStartIndex;
      };
    }


  };

}

/**
 * @ngdoc directive
 * @name ionContent
 * @module ionic
 * @delegate ionic.service:$ionicScrollDelegate
 * @restrict E
 *
 * @description
 * The ionContent directive provides an easy to use content area that can be configured
 * to use Ionic's custom Scroll View, or the built in overflow scrolling of the browser.
 *
 * While we recommend using the custom Scroll features in Ionic in most cases, sometimes
 * (for performance reasons) only the browser's native overflow scrolling will suffice,
 * and so we've made it easy to toggle between the Ionic scroll implementation and
 * overflow scrolling.
 *
 * You can implement pull-to-refresh with the {@link ionic.directive:ionRefresher}
 * directive, and infinite scrolling with the {@link ionic.directive:ionInfiniteScroll}
 * directive.
 *
 * If there is any dynamic content inside the ion-content, be sure to call `.resize()` with {@link ionic.service:$ionicScrollDelegate}
 * after the content has been added.
 *
 * Be aware that this directive gets its own child scope. If you do not understand why this
 * is important, you can read [https://docs.angularjs.org/guide/scope](https://docs.angularjs.org/guide/scope).
 *
 * @param {string=} delegate-handle The handle used to identify this scrollView
 * with {@link ionic.service:$ionicScrollDelegate}.
 * @param {string=} direction Which way to scroll. 'x' or 'y' or 'xy'. Default 'y'.
 * @param {boolean=} locking Whether to lock scrolling in one direction at a time. Useful to set to false when zoomed in or scrolling in two directions. Default true.
 * @param {boolean=} padding Whether to add padding to the content.
 * Defaults to true on iOS, false on Android.
 * @param {boolean=} scroll Whether to allow scrolling of content.  Defaults to true.
 * @param {boolean=} overflow-scroll Whether to use overflow-scrolling instead of
 * Ionic scroll. See {@link ionic.provider:$ionicConfigProvider} to set this as the global default.
 * @param {boolean=} scrollbar-x Whether to show the horizontal scrollbar. Default true.
 * @param {boolean=} scrollbar-y Whether to show the vertical scrollbar. Default true.
 * @param {string=} start-x Initial horizontal scroll position. Default 0.
 * @param {string=} start-y Initial vertical scroll position. Default 0.
 * @param {expression=} on-scroll Expression to evaluate when the content is scrolled.
 * @param {expression=} on-scroll-complete Expression to evaluate when a scroll action completes. Has access to 'scrollLeft' and 'scrollTop' locals.
 * @param {boolean=} has-bouncing Whether to allow scrolling to bounce past the edges
 * of the content.  Defaults to true on iOS, false on Android.
 * @param {number=} scroll-event-interval Number of milliseconds between each firing of the 'on-scroll' expression. Default 10.
 */
IonicModule
.directive('ionContent', [
  '$timeout',
  '$controller',
  '$ionicBind',
  '$ionicConfig',
function($timeout, $controller, $ionicBind, $ionicConfig) {
  return {
    restrict: 'E',
    require: '^?ionNavView',
    scope: true,
    priority: 800,
    compile: function(element, attr) {
      var innerElement;
      var scrollCtrl;

      element.addClass('scroll-content ionic-scroll');

      if (attr.scroll != 'false') {
        //We cannot use normal transclude here because it breaks element.data()
        //inheritance on compile
        innerElement = jqLite('<div class="scroll"></div>');
        innerElement.append(element.contents());
        element.append(innerElement);
      } else {
        element.addClass('scroll-content-false');
      }

      var nativeScrolling = attr.overflowScroll !== "false" && (attr.overflowScroll === "true" || !$ionicConfig.scrolling.jsScrolling());

      // collection-repeat requires JS scrolling
      if (nativeScrolling) {
        nativeScrolling = !element[0].querySelector('[collection-repeat]');
      }
      return { pre: prelink };
      function prelink($scope, $element, $attr) {
        var parentScope = $scope.$parent;
        $scope.$watch(function() {
          return (parentScope.$hasHeader ? ' has-header' : '') +
            (parentScope.$hasSubheader ? ' has-subheader' : '') +
            (parentScope.$hasFooter ? ' has-footer' : '') +
            (parentScope.$hasSubfooter ? ' has-subfooter' : '') +
            (parentScope.$hasTabs ? ' has-tabs' : '') +
            (parentScope.$hasTabsTop ? ' has-tabs-top' : '');
        }, function(className, oldClassName) {
          $element.removeClass(oldClassName);
          $element.addClass(className);
        });

        //Only this ionContent should use these variables from parent scopes
        $scope.$hasHeader = $scope.$hasSubheader =
          $scope.$hasFooter = $scope.$hasSubfooter =
          $scope.$hasTabs = $scope.$hasTabsTop =
          false;
        $ionicBind($scope, $attr, {
          $onScroll: '&onScroll',
          $onScrollComplete: '&onScrollComplete',
          hasBouncing: '@',
          padding: '@',
          direction: '@',
          scrollbarX: '@',
          scrollbarY: '@',
          startX: '@',
          startY: '@',
          scrollEventInterval: '@'
        });
        $scope.direction = $scope.direction || 'y';

        if (isDefined($attr.padding)) {
          $scope.$watch($attr.padding, function(newVal) {
              (innerElement || $element).toggleClass('padding', !!newVal);
          });
        }

        if ($attr.scroll === "false") {
          //do nothing
        } else {
          var scrollViewOptions = {};

          // determined in compile phase above
          if (nativeScrolling) {
            // use native scrolling
            $element.addClass('overflow-scroll');

            scrollViewOptions = {
              el: $element[0],
              delegateHandle: attr.delegateHandle,
              startX: $scope.$eval($scope.startX) || 0,
              startY: $scope.$eval($scope.startY) || 0,
              nativeScrolling: true
            };

          } else {
            // Use JS scrolling
            scrollViewOptions = {
              el: $element[0],
              delegateHandle: attr.delegateHandle,
              locking: (attr.locking || 'true') === 'true',
              bouncing: $scope.$eval($scope.hasBouncing),
              startX: $scope.$eval($scope.startX) || 0,
              startY: $scope.$eval($scope.startY) || 0,
              scrollbarX: $scope.$eval($scope.scrollbarX) !== false,
              scrollbarY: $scope.$eval($scope.scrollbarY) !== false,
              scrollingX: $scope.direction.indexOf('x') >= 0,
              scrollingY: $scope.direction.indexOf('y') >= 0,
              scrollEventInterval: parseInt($scope.scrollEventInterval, 10) || 10,
              scrollingComplete: onScrollComplete
            };
          }

          // init scroll controller with appropriate options
          scrollCtrl = $controller('$ionicScroll', {
            $scope: $scope,
            scrollViewOptions: scrollViewOptions
          });

          $scope.scrollCtrl = scrollCtrl;

          $scope.$on('$destroy', function() {
            if (scrollViewOptions) {
              scrollViewOptions.scrollingComplete = noop;
              delete scrollViewOptions.el;
            }
            innerElement = null;
            $element = null;
            attr.$$element = null;
          });
        }

        function onScrollComplete() {
          $scope.$onScrollComplete({
            scrollTop: scrollCtrl.scrollView.__scrollTop,
            scrollLeft: scrollCtrl.scrollView.__scrollLeft
          });
        }

      }
    }
  };
}]);

/**
 * @ngdoc directive
 * @name exposeAsideWhen
 * @module ionic
 * @restrict A
 * @parent ionic.directive:ionSideMenus
 *
 * @description
 * It is common for a tablet application to hide a menu when in portrait mode, but to show the
 * same menu on the left side when the tablet is in landscape mode. The `exposeAsideWhen` attribute
 * directive can be used to accomplish a similar interface.
 *
 * By default, side menus are hidden underneath its side menu content, and can be opened by either
 * swiping the content left or right, or toggling a button to show the side menu. However, by adding the
 * `exposeAsideWhen` attribute directive to an {@link ionic.directive:ionSideMenu} element directive,
 * a side menu can be given instructions on "when" the menu should be exposed (always viewable). For
 * example, the `expose-aside-when="large"` attribute will keep the side menu hidden when the viewport's
 * width is less than `768px`, but when the viewport's width is `768px` or greater, the menu will then
 * always be shown and can no longer be opened or closed like it could when it was hidden for smaller
 * viewports.
 *
 * Using `large` as the attribute's value is a shortcut value to `(min-width:768px)` since it is
 * the most common use-case. However, for added flexibility, any valid media query could be added
 * as the value, such as `(min-width:600px)` or even multiple queries such as
 * `(min-width:750px) and (max-width:1200px)`.
 * @usage
 * ```html
 * <ion-side-menus>
 *   <!-- Center content -->
 *   <ion-side-menu-content>
 *   </ion-side-menu-content>
 *
 *   <!-- Left menu -->
 *   <ion-side-menu expose-aside-when="large">
 *   </ion-side-menu>
 * </ion-side-menus>
 * ```
 * For a complete side menu example, see the
 * {@link ionic.directive:ionSideMenus} documentation.
 */

IonicModule.directive('exposeAsideWhen', ['$window', function($window) {
  return {
    restrict: 'A',
    require: '^ionSideMenus',
    link: function($scope, $element, $attr, sideMenuCtrl) {

      var prevInnerWidth = $window.innerWidth;
      var prevInnerHeight = $window.innerHeight;

      ionic.on('resize', function() {
        if (prevInnerWidth === $window.innerWidth && prevInnerHeight === $window.innerHeight) {
          return;
        }
        prevInnerWidth = $window.innerWidth;
        prevInnerHeight = $window.innerHeight;
        onResize();
      }, $window);

      function checkAsideExpose() {
        var mq = $attr.exposeAsideWhen == 'large' ? '(min-width:768px)' : $attr.exposeAsideWhen;
        sideMenuCtrl.exposeAside($window.matchMedia(mq).matches);
        sideMenuCtrl.activeAsideResizing(false);
      }

      function onResize() {
        sideMenuCtrl.activeAsideResizing(true);
        debouncedCheck();
      }

      var debouncedCheck = ionic.debounce(function() {
        $scope.$apply(checkAsideExpose);
      }, 300, false);

      $scope.$evalAsync(checkAsideExpose);
    }
  };
}]);

var GESTURE_DIRECTIVES = 'onHold onTap onDoubleTap onTouch onRelease onDragStart onDrag onDragEnd onDragUp onDragRight onDragDown onDragLeft onSwipe onSwipeUp onSwipeRight onSwipeDown onSwipeLeft'.split(' ');

GESTURE_DIRECTIVES.forEach(function(name) {
  IonicModule.directive(name, gestureDirective(name));
});


/**
 * @ngdoc directive
 * @name onHold
 * @module ionic
 * @restrict A
 *
 * @description
 * Touch stays at the same location for 500ms. Similar to long touch events available for AngularJS and jQuery.
 *
 * @usage
 * ```html
 * <button on-hold="onHold()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onTap
 * @module ionic
 * @restrict A
 *
 * @description
 * Quick touch at a location. If the duration of the touch goes
 * longer than 250ms it is no longer a tap gesture.
 *
 * @usage
 * ```html
 * <button on-tap="onTap()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onDoubleTap
 * @module ionic
 * @restrict A
 *
 * @description
 * Double tap touch at a location.
 *
 * @usage
 * ```html
 * <button on-double-tap="onDoubleTap()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onTouch
 * @module ionic
 * @restrict A
 *
 * @description
 * Called immediately when the user first begins a touch. This
 * gesture does not wait for a touchend/mouseup.
 *
 * @usage
 * ```html
 * <button on-touch="onTouch()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onRelease
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when the user ends a touch.
 *
 * @usage
 * ```html
 * <button on-release="onRelease()" class="button">Test</button>
 * ```
 */

/**
 * @ngdoc directive
 * @name onDragStart
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a drag gesture has started.
 *
 * @usage
 * ```html
 * <button on-drag-start="onDragStart()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onDrag
 * @module ionic
 * @restrict A
 *
 * @description
 * Move with one touch around on the page. Blocking the scrolling when
 * moving left and right is a good practice. When all the drag events are
 * blocking you disable scrolling on that area.
 *
 * @usage
 * ```html
 * <button on-drag="onDrag()" class="button">Test</button>
 * ```
 */

/**
 * @ngdoc directive
 * @name onDragEnd
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a drag gesture has ended.
 *
 * @usage
 * ```html
 * <button on-drag-end="onDragEnd()" class="button">Test</button>
 * ```
 */

/**
 * @ngdoc directive
 * @name onDragUp
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when the element is dragged up.
 *
 * @usage
 * ```html
 * <button on-drag-up="onDragUp()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onDragRight
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when the element is dragged to the right.
 *
 * @usage
 * ```html
 * <button on-drag-right="onDragRight()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onDragDown
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when the element is dragged down.
 *
 * @usage
 * ```html
 * <button on-drag-down="onDragDown()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onDragLeft
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when the element is dragged to the left.
 *
 * @usage
 * ```html
 * <button on-drag-left="onDragLeft()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onSwipe
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a moving touch has a high velocity in any direction.
 *
 * @usage
 * ```html
 * <button on-swipe="onSwipe()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onSwipeUp
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a moving touch has a high velocity moving up.
 *
 * @usage
 * ```html
 * <button on-swipe-up="onSwipeUp()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onSwipeRight
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a moving touch has a high velocity moving to the right.
 *
 * @usage
 * ```html
 * <button on-swipe-right="onSwipeRight()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onSwipeDown
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a moving touch has a high velocity moving down.
 *
 * @usage
 * ```html
 * <button on-swipe-down="onSwipeDown()" class="button">Test</button>
 * ```
 */


/**
 * @ngdoc directive
 * @name onSwipeLeft
 * @module ionic
 * @restrict A
 *
 * @description
 * Called when a moving touch has a high velocity moving to the left.
 *
 * @usage
 * ```html
 * <button on-swipe-left="onSwipeLeft()" class="button">Test</button>
 * ```
 */


function gestureDirective(directiveName) {
  return ['$ionicGesture', '$parse', function($ionicGesture, $parse) {
    var eventType = directiveName.substr(2).toLowerCase();

    return function(scope, element, attr) {
      var fn = $parse( attr[directiveName] );

      var listener = function(ev) {
        scope.$apply(function() {
          fn(scope, {
            $event: ev
          });
        });
      };

      var gesture = $ionicGesture.on(eventType, listener, element);

      scope.$on('$destroy', function() {
        $ionicGesture.off(gesture, eventType, listener);
      });
    };
  }];
}


IonicModule
//.directive('ionHeaderBar', tapScrollToTopDirective())

/**
 * @ngdoc directive
 * @name ionHeaderBar
 * @module ionic
 * @restrict E
 *
 * @description
 * Adds a fixed header bar above some content.
 *
 * Can also be a subheader (lower down) if the 'bar-subheader' class is applied.
 * See [the header CSS docs](/docs/components/#subheader).
 *
 * @param {string=} align-title How to align the title. By default the title
 * will be aligned the same as how the platform aligns its titles (iOS centers
 * titles, Android aligns them left).
 * Available: 'left', 'right', or 'center'.  Defaults to the same as the platform.
 * @param {boolean=} no-tap-scroll By default, the header bar will scroll the
 * content to the top when tapped.  Set no-tap-scroll to true to disable this
 * behavior.
 * Available: true or false.  Defaults to false.
 *
 * @usage
 * ```html
 * <ion-header-bar align-title="left" class="bar-positive">
 *   <div class="buttons">
 *     <button class="button" ng-click="doSomething()">Left Button</button>
 *   </div>
 *   <h1 class="title">Title!</h1>
 *   <div class="buttons">
 *     <button class="button">Right Button</button>
 *   </div>
 * </ion-header-bar>
 * <ion-content class="has-header">
 *   Some content!
 * </ion-content>
 * ```
 */
.directive('ionHeaderBar', headerFooterBarDirective(true))

/**
 * @ngdoc directive
 * @name ionFooterBar
 * @module ionic
 * @restrict E
 *
 * @description
 * Adds a fixed footer bar below some content.
 *
 * Can also be a subfooter (higher up) if the 'bar-subfooter' class is applied.
 * See [the footer CSS docs](/docs/components/#footer).
 *
 * Note: If you use ionFooterBar in combination with ng-if, the surrounding content
 * will not align correctly.  This will be fixed soon.
 *
 * @param {string=} align-title Where to align the title.
 * Available: 'left', 'right', or 'center'.  Defaults to 'center'.
 *
 * @usage
 * ```html
 * <ion-content class="has-footer">
 *   Some content!
 * </ion-content>
 * <ion-footer-bar align-title="left" class="bar-assertive">
 *   <div class="buttons">
 *     <button class="button">Left Button</button>
 *   </div>
 *   <h1 class="title">Title!</h1>
 *   <div class="buttons" ng-click="doSomething()">
 *     <button class="button">Right Button</button>
 *   </div>
 * </ion-footer-bar>
 * ```
 */
.directive('ionFooterBar', headerFooterBarDirective(false));

function tapScrollToTopDirective() { //eslint-disable-line no-unused-vars
  return ['$ionicScrollDelegate', function($ionicScrollDelegate) {
    return {
      restrict: 'E',
      link: function($scope, $element, $attr) {
        if ($attr.noTapScroll == 'true') {
          return;
        }
        ionic.on('tap', onTap, $element[0]);
        $scope.$on('$destroy', function() {
          ionic.off('tap', onTap, $element[0]);
        });

        function onTap(e) {
          var depth = 3;
          var current = e.target;
          //Don't scroll to top in certain cases
          while (depth-- && current) {
            if (current.classList.contains('button') ||
                current.tagName.match(/input|textarea|select/i) ||
                current.isContentEditable) {
              return;
            }
            current = current.parentNode;
          }
          var touch = e.gesture && e.gesture.touches[0] || e.detail.touches[0];
          var bounds = $element[0].getBoundingClientRect();
          if (ionic.DomUtil.rectContains(
            touch.pageX, touch.pageY,
            bounds.left, bounds.top - 20,
            bounds.left + bounds.width, bounds.top + bounds.height
          )) {
            $ionicScrollDelegate.scrollTop(true);
          }
        }
      }
    };
  }];
}

function headerFooterBarDirective(isHeader) {
  return ['$document', '$timeout', function($document, $timeout) {
    return {
      restrict: 'E',
      controller: '$ionicHeaderBar',
      compile: function(tElement) {
        tElement.addClass(isHeader ? 'bar bar-header' : 'bar bar-footer');
        // top style tabs? if so, remove bottom border for seamless display
        $timeout(function() {
          if (isHeader && $document[0].getElementsByClassName('tabs-top').length) tElement.addClass('has-tabs-top');
        });

        return { pre: prelink };
        function prelink($scope, $element, $attr, ctrl) {
          if (isHeader) {
            $scope.$watch(function() { return $element[0].className; }, function(value) {
              var isShown = value.indexOf('ng-hide') === -1;
              var isSubheader = value.indexOf('bar-subheader') !== -1;
              $scope.$hasHeader = isShown && !isSubheader;
              $scope.$hasSubheader = isShown && isSubheader;
              $scope.$emit('$ionicSubheader', $scope.$hasSubheader);
            });
            $scope.$on('$destroy', function() {
              delete $scope.$hasHeader;
              delete $scope.$hasSubheader;
            });
            ctrl.align();
            $scope.$on('$ionicHeader.align', function() {
              ionic.requestAnimationFrame(function() {
                ctrl.align();
              });
            });

          } else {
            $scope.$watch(function() { return $element[0].className; }, function(value) {
              var isShown = value.indexOf('ng-hide') === -1;
              var isSubfooter = value.indexOf('bar-subfooter') !== -1;
              $scope.$hasFooter = isShown && !isSubfooter;
              $scope.$hasSubfooter = isShown && isSubfooter;
            });
            $scope.$on('$destroy', function() {
              delete $scope.$hasFooter;
              delete $scope.$hasSubfooter;
            });
            $scope.$watch('$hasTabs', function(val) {
              $element.toggleClass('has-tabs', !!val);
            });
            ctrl.align();
            $scope.$on('$ionicFooter.align', function() {
              ionic.requestAnimationFrame(function() {
                ctrl.align();
              });
            });
          }
        }
      }
    };
  }];
}

/**
 * @ngdoc directive
 * @name ionInfiniteScroll
 * @module ionic
 * @parent ionic.directive:ionContent, ionic.directive:ionScroll
 * @restrict E
 *
 * @description
 * The ionInfiniteScroll directive allows you to call a function whenever
 * the user gets to the bottom of the page or near the bottom of the page.
 *
 * The expression you pass in for `on-infinite` is called when the user scrolls
 * greater than `distance` away from the bottom of the content.  Once `on-infinite`
 * is done loading new data, it should broadcast the `scroll.infiniteScrollComplete`
 * event from your controller (see below example).
 *
 * @param {expression} on-infinite What to call when the scroller reaches the
 * bottom.
 * @param {string=} distance The distance from the bottom that the scroll must
 * reach to trigger the on-infinite expression. Default: 1%.
 * @param {string=} spinner The {@link ionic.directive:ionSpinner} to show while loading. The SVG
 * {@link ionic.directive:ionSpinner} is now the default, replacing rotating font icons.
 * @param {string=} icon The icon to show while loading. Default: 'ion-load-d'.  This is depreciated
 * in favor of the SVG {@link ionic.directive:ionSpinner}.
 * @param {boolean=} immediate-check Whether to check the infinite scroll bounds immediately on load.
 *
 * @usage
 * ```html
 * <ion-content ng-controller="MyController">
 *   <ion-list>
 *   ....
 *   ....
 *   </ion-list>
 *
 *   <ion-infinite-scroll
 *     on-infinite="loadMore()"
 *     distance="1%">
 *   </ion-infinite-scroll>
 * </ion-content>
 * ```
 * ```js
 * function MyController($scope, $http) {
 *   $scope.items = [];
 *   $scope.loadMore = function() {
 *     $http.get('/more-items').success(function(items) {
 *       useItems(items);
 *       $scope.$broadcast('scroll.infiniteScrollComplete');
 *     });
 *   };
 *
 *   $scope.$on('$stateChangeSuccess', function() {
 *     $scope.loadMore();
 *   });
 * }
 * ```
 *
 * An easy to way to stop infinite scroll once there is no more data to load
 * is to use angular's `ng-if` directive:
 *
 * ```html
 * <ion-infinite-scroll
 *   ng-if="moreDataCanBeLoaded()"
 *   icon="ion-loading-c"
 *   on-infinite="loadMoreData()">
 * </ion-infinite-scroll>
 * ```
 */
IonicModule
.directive('ionInfiniteScroll', ['$timeout', function($timeout) {
  return {
    restrict: 'E',
    require: ['?^$ionicScroll', 'ionInfiniteScroll'],
    template: function($element, $attrs) {
      if ($attrs.icon) return '<i class="icon {{icon()}} icon-refreshing {{scrollingType}}"></i>';
      return '<ion-spinner icon="{{spinner()}}"></ion-spinner>';
    },
    scope: true,
    controller: '$ionInfiniteScroll',
    link: function($scope, $element, $attrs, ctrls) {
      var infiniteScrollCtrl = ctrls[1];
      var scrollCtrl = infiniteScrollCtrl.scrollCtrl = ctrls[0];
      var jsScrolling = infiniteScrollCtrl.jsScrolling = !scrollCtrl.isNative();

      // if this view is not beneath a scrollCtrl, it can't be injected, proceed w/ native scrolling
      if (jsScrolling) {
        infiniteScrollCtrl.scrollView = scrollCtrl.scrollView;
        $scope.scrollingType = 'js-scrolling';
        //bind to JS scroll events
        scrollCtrl.$element.on('scroll', infiniteScrollCtrl.checkBounds);
      } else {
        // grabbing the scrollable element, to determine dimensions, and current scroll pos
        var scrollEl = ionic.DomUtil.getParentOrSelfWithClass($element[0].parentNode, 'overflow-scroll');
        infiniteScrollCtrl.scrollEl = scrollEl;
        // if there's no scroll controller, and no overflow scroll div, infinite scroll wont work
        if (!scrollEl) {
          throw 'Infinite scroll must be used inside a scrollable div';
        }
        //bind to native scroll events
        infiniteScrollCtrl.scrollEl.addEventListener('scroll', infiniteScrollCtrl.checkBounds);
      }

      // Optionally check bounds on start after scrollView is fully rendered
      var doImmediateCheck = isDefined($attrs.immediateCheck) ? $scope.$eval($attrs.immediateCheck) : true;
      if (doImmediateCheck) {
        $timeout(function() { infiniteScrollCtrl.checkBounds(); });
      }
    }
  };
}]);

/**
* @ngdoc directive
* @name ionInput
* @parent ionic.directive:ionList
* @module ionic
* @restrict E
* Creates a text input group that can easily be focused
*
* @usage
*
* ```html
* <ion-list>
*   <ion-input>
*     <input type="text" placeholder="First Name">
*   </ion-input>
*
*   <ion-input>
*     <ion-label>Username</ion-label>
*     <input type="text">
*   </ion-input>
* </ion-list>
* ```
*/

var labelIds = -1;

IonicModule
.directive('ionInput', [function() {
  return {
    restrict: 'E',
    controller: ['$scope', '$element', function($scope, $element) {
      this.$scope = $scope;
      this.$element = $element;

      this.setInputAriaLabeledBy = function(id) {
        var inputs = $element[0].querySelectorAll('input,textarea');
        inputs.length && inputs[0].setAttribute('aria-labelledby', id);
      };

      this.focus = function() {
        var inputs = $element[0].querySelectorAll('input,textarea');
        inputs.length && inputs[0].focus();
      };
    }]
  };
}]);

/**
* @ngdoc directive
* @name ionLabel
* @parent ionic.directive:ionList
* @module ionic
* @restrict E
*
* New in Ionic 1.2. It is strongly recommended that you use `<ion-label>` in place
* of any `<label>` elements for maximum cross-browser support and performance.
*
* Creates a label for a form input.
*
* @usage
*
* ```html
* <ion-list>
*   <ion-input>
*     <ion-label>Username</ion-label>
*     <input type="text">
*   </ion-input>
* </ion-list>
* ```
*/
IonicModule
.directive('ionLabel', [function() {
  return {
    restrict: 'E',
    require: '?^ionInput',
    compile: function() {

      return function link($scope, $element, $attrs, ionInputCtrl) {
        var element = $element[0];

        $element.addClass('input-label');

        $element.attr('aria-label', $element.text());
        var id = element.id || '_label-' + ++labelIds;

        if (!element.id) {
          $element.attr('id', id);
        }

        if (ionInputCtrl) {

          ionInputCtrl.setInputAriaLabeledBy(id);

          $element.on('click', function() {
            ionInputCtrl.focus();
          });
        }
      };
    }
  };
}]);

/**
 * Input label adds accessibility to <span class="input-label">.
 */
IonicModule
.directive('inputLabel', [function() {
  return {
    restrict: 'C',
    require: '?^ionInput',
    compile: function() {

      return function link($scope, $element, $attrs, ionInputCtrl) {
        var element = $element[0];

        $element.attr('aria-label', $element.text());
        var id = element.id || '_label-' + ++labelIds;

        if (!element.id) {
          $element.attr('id', id);
        }

        if (ionInputCtrl) {
          ionInputCtrl.setInputAriaLabeledBy(id);
        }

      };
    }
  };
}]);

/**
* @ngdoc directive
* @name ionItem
* @parent ionic.directive:ionList
* @module ionic
* @restrict E
* Creates a list-item that can easily be swiped,
* deleted, reordered, edited, and more.
*
* See {@link ionic.directive:ionList} for a complete example & explanation.
*
* Can be assigned any item class name. See the
* [list CSS documentation](/docs/components/#list).
*
* @usage
*
* ```html
* <ion-list>
*   <ion-item>Hello!</ion-item>
*   <ion-item href="#/detail">
*     Link to detail page
*   </ion-item>
* </ion-list>
* ```
*/
IonicModule
.directive('ionItem', ['$$rAF', function($$rAF) {
  return {
    restrict: 'E',
    controller: ['$scope', '$element', function($scope, $element) {
      this.$scope = $scope;
      this.$element = $element;
    }],
    scope: true,
    compile: function($element, $attrs) {
      var isAnchor = isDefined($attrs.href) ||
                     isDefined($attrs.ngHref) ||
                     isDefined($attrs.uiSref);
      var isComplexItem = isAnchor ||
        //Lame way of testing, but we have to know at compile what to do with the element
        /ion-(delete|option|reorder)-button/i.test($element.html());

      if (isComplexItem) {
        var innerElement = jqLite(isAnchor ? '<a></a>' : '<div></div>');
        innerElement.addClass('item-content');

        if (isDefined($attrs.href) || isDefined($attrs.ngHref)) {
          innerElement.attr('ng-href', '{{$href()}}');
          if (isDefined($attrs.target)) {
            innerElement.attr('target', '{{$target()}}');
          }
        }

        innerElement.append($element.contents());

        $element.addClass('item item-complex')
                .append(innerElement);
      } else {
        $element.addClass('item');
      }

      return function link($scope, $element, $attrs) {
        $scope.$href = function() {
          return $attrs.href || $attrs.ngHref;
        };
        $scope.$target = function() {
          return $attrs.target;
        };

        var content = $element[0].querySelector('.item-content');
        if (content) {
          $scope.$on('$collectionRepeatLeave', function() {
            if (content && content.$$ionicOptionsOpen) {
              content.style[ionic.CSS.TRANSFORM] = '';
              content.style[ionic.CSS.TRANSITION] = 'none';
              $$rAF(function() {
                content.style[ionic.CSS.TRANSITION] = '';
              });
              content.$$ionicOptionsOpen = false;
            }
          });
        }
      };

    }
  };
}]);

var ITEM_TPL_DELETE_BUTTON =
  '<div class="item-left-edit item-delete enable-pointer-events">' +
  '</div>';
/**
* @ngdoc directive
* @name ionDeleteButton
* @parent ionic.directive:ionItem
* @module ionic
* @restrict E
* Creates a delete button inside a list item, that is visible when the
* {@link ionic.directive:ionList ionList parent's} `show-delete` evaluates to true or
* `$ionicListDelegate.showDelete(true)` is called.
*
* Takes any ionicon as a class.
*
* See {@link ionic.directive:ionList} for a complete example & explanation.
*
* @usage
*
* ```html
* <ion-list show-delete="shouldShowDelete">
*   <ion-item>
*     <ion-delete-button class="ion-minus-circled"></ion-delete-button>
*     Hello, list item!
*   </ion-item>
* </ion-list>
* <ion-toggle ng-model="shouldShowDelete">
*   Show Delete?
* </ion-toggle>
* ```
*/
IonicModule
.directive('ionDeleteButton', function() {

  function stopPropagation(ev) {
    ev.stopPropagation();
  }

  return {
    restrict: 'E',
    require: ['^^ionItem', '^?ionList'],
    //Run before anything else, so we can move it before other directives process
    //its location (eg ngIf relies on the location of the directive in the dom)
    priority: Number.MAX_VALUE,
    compile: function($element, $attr) {
      //Add the classes we need during the compile phase, so that they stay
      //even if something else like ngIf removes the element and re-addss it
      $attr.$set('class', ($attr['class'] || '') + ' button icon button-icon', true);
      return function($scope, $element, $attr, ctrls) {
        var itemCtrl = ctrls[0];
        var listCtrl = ctrls[1];
        var container = jqLite(ITEM_TPL_DELETE_BUTTON);
        container.append($element);
        itemCtrl.$element.append(container).addClass('item-left-editable');

        //Don't bubble click up to main .item
        $element.on('click', stopPropagation);

        init();
        $scope.$on('$ionic.reconnectScope', init);
        function init() {
          listCtrl = listCtrl || $element.controller('ionList');
          if (listCtrl && listCtrl.showDelete()) {
            container.addClass('visible active');
          }
        }
      };
    }
  };
});


IonicModule
.directive('itemFloatingLabel', function() {
  return {
    restrict: 'C',
    link: function(scope, element) {
      var el = element[0];
      var input = el.querySelector('input, textarea');
      var inputLabel = el.querySelector('.input-label');

      if (!input || !inputLabel) return;

      var onInput = function() {
        if (input.value) {
          inputLabel.classList.add('has-input');
        } else {
          inputLabel.classList.remove('has-input');
        }
      };

      input.addEventListener('input', onInput);

      var ngModelCtrl = jqLite(input).controller('ngModel');
      if (ngModelCtrl) {
        ngModelCtrl.$render = function() {
          input.value = ngModelCtrl.$viewValue || '';
          onInput();
        };
      }

      scope.$on('$destroy', function() {
        input.removeEventListener('input', onInput);
      });
    }
  };
});

var ITEM_TPL_OPTION_BUTTONS =
  '<div class="item-options invisible">' +
  '</div>';
/**
* @ngdoc directive
* @name ionOptionButton
* @parent ionic.directive:ionItem
* @module ionic
* @restrict E
* @description
* Creates an option button inside a list item, that is visible when the item is swiped
* to the left by the user.  Swiped open option buttons can be hidden with
* {@link ionic.service:$ionicListDelegate#closeOptionButtons $ionicListDelegate.closeOptionButtons}.
*
* Can be assigned any button class.
*
* See {@link ionic.directive:ionList} for a complete example & explanation.
*
* @usage
*
* ```html
* <ion-list>
*   <ion-item>
*     I love kittens!
*     <ion-option-button class="button-positive">Share</ion-option-button>
*     <ion-option-button class="button-assertive">Edit</ion-option-button>
*   </ion-item>
* </ion-list>
* ```
*/
IonicModule.directive('ionOptionButton', [function() {
  function stopPropagation(e) {
    e.stopPropagation();
  }
  return {
    restrict: 'E',
    require: '^ionItem',
    priority: Number.MAX_VALUE,
    compile: function($element, $attr) {
      $attr.$set('class', ($attr['class'] || '') + ' button', true);
      return function($scope, $element, $attr, itemCtrl) {
        if (!itemCtrl.optionsContainer) {
          itemCtrl.optionsContainer = jqLite(ITEM_TPL_OPTION_BUTTONS);
          itemCtrl.$element.prepend(itemCtrl.optionsContainer);
        }
        itemCtrl.optionsContainer.prepend($element);

        itemCtrl.$element.addClass('item-right-editable');

        //Don't bubble click up to main .item
        $element.on('click', stopPropagation);
      };
    }
  };
}]);

var ITEM_TPL_REORDER_BUTTON =
  '<div data-prevent-scroll="true" class="item-right-edit item-reorder enable-pointer-events">' +
  '</div>';

/**
* @ngdoc directive
* @name ionReorderButton
* @parent ionic.directive:ionItem
* @module ionic
* @restrict E
* Creates a reorder button inside a list item, that is visible when the
* {@link ionic.directive:ionList ionList parent's} `show-reorder` evaluates to true or
* `$ionicListDelegate.showReorder(true)` is called.
*
* Can be dragged to reorder items in the list. Takes any ionicon class.
*
* Note: Reordering works best when used with `ng-repeat`.  Be sure that all `ion-item` children of an `ion-list` are part of the same `ng-repeat` expression.
*
* When an item reorder is complete, the expression given in the `on-reorder` attribute is called. The `on-reorder` expression is given two locals that can be used: `$fromIndex` and `$toIndex`.  See below for an example.
*
* Look at {@link ionic.directive:ionList} for more examples.
*
* @usage
*
* ```html
* <ion-list ng-controller="MyCtrl" show-reorder="true">
*   <ion-item ng-repeat="item in items">
*     Item {{item}}
*     <ion-reorder-button class="ion-navicon"
*                         on-reorder="moveItem(item, $fromIndex, $toIndex)">
*     </ion-reorder-button>
*   </ion-item>
* </ion-list>
* ```
* ```js
* function MyCtrl($scope) {
*   $scope.items = [1, 2, 3, 4];
*   $scope.moveItem = function(item, fromIndex, toIndex) {
*     //Move the item in the array
*     $scope.items.splice(fromIndex, 1);
*     $scope.items.splice(toIndex, 0, item);
*   };
* }
* ```
*
* @param {expression=} on-reorder Expression to call when an item is reordered.
* Parameters given: $fromIndex, $toIndex.
*/
IonicModule
.directive('ionReorderButton', ['$parse', function($parse) {
  return {
    restrict: 'E',
    require: ['^ionItem', '^?ionList'],
    priority: Number.MAX_VALUE,
    compile: function($element, $attr) {
      $attr.$set('class', ($attr['class'] || '') + ' button icon button-icon', true);
      $element[0].setAttribute('data-prevent-scroll', true);
      return function($scope, $element, $attr, ctrls) {
        var itemCtrl = ctrls[0];
        var listCtrl = ctrls[1];
        var onReorderFn = $parse($attr.onReorder);

        $scope.$onReorder = function(oldIndex, newIndex) {
          onReorderFn($scope, {
            $fromIndex: oldIndex,
            $toIndex: newIndex
          });
        };

        // prevent clicks from bubbling up to the item
        if (!$attr.ngClick && !$attr.onClick && !$attr.onclick) {
          $element[0].onclick = function(e) {
            e.stopPropagation();
            return false;
          };
        }

        var container = jqLite(ITEM_TPL_REORDER_BUTTON);
        container.append($element);
        itemCtrl.$element.append(container).addClass('item-right-editable');

        if (listCtrl && listCtrl.showReorder()) {
          container.addClass('visible active');
        }
      };
    }
  };
}]);

/**
 * @ngdoc directive
 * @name keyboardAttach
 * @module ionic
 * @restrict A
 *
 * @description
 * keyboard-attach is an attribute directive which will cause an element to float above
 * the keyboard when the keyboard shows. Currently only supports the
 * [ion-footer-bar]({{ page.versionHref }}/api/directive/ionFooterBar/) directive.
 *
 * ### Notes
 * - This directive requires the
 * [Ionic Keyboard Plugin](https://github.com/ionic-team/ionic-plugins-keyboard).
 * - On Android not in fullscreen mode, i.e. you have
 *   `<preference name="Fullscreen" value="false" />` or no preference in your `config.xml` file,
 *   this directive is unnecessary since it is the default behavior.
 * - On iOS, if there is an input in your footer, you will need to set
 *   `cordova.plugins.Keyboard.disableScroll(true)`.
 *
 * @usage
 *
 * ```html
 *  <ion-footer-bar align-title="left" keyboard-attach class="bar-assertive">
 *    <h1 class="title">Title!</h1>
 *  </ion-footer-bar>
 * ```
 */

IonicModule
.directive('keyboardAttach', function() {
  return function(scope, element) {
    ionic.on('native.keyboardshow', onShow, window);
    ionic.on('native.keyboardhide', onHide, window);

    //deprecated
    ionic.on('native.showkeyboard', onShow, window);
    ionic.on('native.hidekeyboard', onHide, window);


    var scrollCtrl;

    function onShow(e) {
      if (ionic.Platform.isAndroid() && !ionic.Platform.isFullScreen) {
        return;
      }

      //for testing
      var keyboardHeight = e.keyboardHeight || (e.detail && e.detail.keyboardHeight);
      element.css('bottom', keyboardHeight + "px");
      scrollCtrl = element.controller('$ionicScroll');
      if (scrollCtrl) {
        scrollCtrl.scrollView.__container.style.bottom = keyboardHeight + keyboardAttachGetClientHeight(element[0]) + "px";
      }
    }

    function onHide() {
      if (ionic.Platform.isAndroid() && !ionic.Platform.isFullScreen) {
        return;
      }

      element.css('bottom', '');
      if (scrollCtrl) {
        scrollCtrl.scrollView.__container.style.bottom = '';
      }
    }

    scope.$on('$destroy', function() {
      ionic.off('native.keyboardshow', onShow, window);
      ionic.off('native.keyboardhide', onHide, window);

      //deprecated
      ionic.off('native.showkeyboard', onShow, window);
      ionic.off('native.hidekeyboard', onHide, window);
    });
  };
});

function keyboardAttachGetClientHeight(element) {
  return element.clientHeight;
}

/**
* @ngdoc directive
* @name ionList
* @module ionic
* @delegate ionic.service:$ionicListDelegate
* @codepen JsHjf
* @restrict E
* @description
* The List is a widely used interface element in almost any mobile app, and can include
* content ranging from basic text all the way to buttons, toggles, icons, and thumbnails.
*
* Both the list, which contains items, and the list items themselves can be any HTML
* element. The containing element requires the `list` class and each list item requires
* the `item` class.
*
* However, using the ionList and ionItem directives make it easy to support various
* interaction modes such as swipe to edit, drag to reorder, and removing items.
*
* Related: {@link ionic.directive:ionItem}, {@link ionic.directive:ionOptionButton}
* {@link ionic.directive:ionReorderButton}, {@link ionic.directive:ionDeleteButton}, [`list CSS documentation`](/docs/components/#list).
*
* @usage
*
* Basic Usage:
*
* ```html
* <ion-list>
*   <ion-item ng-repeat="item in items">
*     {% raw %}Hello, {{item}}!{% endraw %}
*   </ion-item>
* </ion-list>
* ```
*
* Advanced Usage: Thumbnails, Delete buttons, Reordering, Swiping
*
* ```html
* <ion-list ng-controller="MyCtrl"
*           show-delete="shouldShowDelete"
*           show-reorder="shouldShowReorder"
*           can-swipe="listCanSwipe">
*   <ion-item ng-repeat="item in items"
*             class="item-thumbnail-left">
*
*     {% raw %}<img ng-src="{{item.img}}">
*     <h2>{{item.title}}</h2>
*     <p>{{item.description}}</p>{% endraw %}
*     <ion-option-button class="button-positive"
*                        ng-click="share(item)">
*       Share
*     </ion-option-button>
*     <ion-option-button class="button-info"
*                        ng-click="edit(item)">
*       Edit
*     </ion-option-button>
*     <ion-delete-button class="ion-minus-circled"
*                        ng-click="items.splice($index, 1)">
*     </ion-delete-button>
*     <ion-reorder-button class="ion-navicon"
*                         on-reorder="reorderItem(item, $fromIndex, $toIndex)">
*     </ion-reorder-button>
*
*   </ion-item>
* </ion-list>
* ```
*
*```javascript
* app.controller('MyCtrl', function($scope) {
*  $scope.shouldShowDelete = false;
*  $scope.shouldShowReorder = false;
*  $scope.listCanSwipe = true
* });
*```
*
* @param {string=} delegate-handle The handle used to identify this list with
* {@link ionic.service:$ionicListDelegate}.
* @param type {string=} The type of list to use (list-inset or card)
* @param show-delete {boolean=} Whether the delete buttons for the items in the list are
* currently shown or hidden.
* @param show-reorder {boolean=} Whether the reorder buttons for the items in the list are
* currently shown or hidden.
* @param can-swipe {boolean=} Whether the items in the list are allowed to be swiped to reveal
* option buttons. Default: true.
*/
IonicModule
.directive('ionList', [
  '$timeout',
function($timeout) {
  return {
    restrict: 'E',
    require: ['ionList', '^?$ionicScroll'],
    controller: '$ionicList',
    compile: function($element, $attr) {
      var listEl = jqLite('<div class="list">')
        .append($element.contents())
        .addClass($attr.type);

      $element.append(listEl);

      return function($scope, $element, $attrs, ctrls) {
        var listCtrl = ctrls[0];
        var scrollCtrl = ctrls[1];

        // Wait for child elements to render...
        $timeout(init);

        function init() {
          var listView = listCtrl.listView = new ionic.views.ListView({
            el: $element[0],
            listEl: $element.children()[0],
            scrollEl: scrollCtrl && scrollCtrl.element,
            scrollView: scrollCtrl && scrollCtrl.scrollView,
            onReorder: function(el, oldIndex, newIndex) {
              var itemScope = jqLite(el).scope();
              if (itemScope && itemScope.$onReorder) {
                // Make sure onReorder is called in apply cycle,
                // but also make sure it has no conflicts by doing
                // $evalAsync
                $timeout(function() {
                  itemScope.$onReorder(oldIndex, newIndex);
                });
              }
            },
            canSwipe: function() {
              return listCtrl.canSwipeItems();
            }
          });

          $scope.$on('$destroy', function() {
            if (listView) {
              listView.deregister && listView.deregister();
              listView = null;
            }
          });

          if (isDefined($attr.canSwipe)) {
            $scope.$watch('!!(' + $attr.canSwipe + ')', function(value) {
              listCtrl.canSwipeItems(value);
            });
          }
          if (isDefined($attr.showDelete)) {
            $scope.$watch('!!(' + $attr.showDelete + ')', function(value) {
              listCtrl.showDelete(value);
            });
          }
          if (isDefined($attr.showReorder)) {
            $scope.$watch('!!(' + $attr.showReorder + ')', function(value) {
              listCtrl.showReorder(value);
            });
          }

          $scope.$watch(function() {
            return listCtrl.showDelete();
          }, function(isShown, wasShown) {
            //Only use isShown=false if it was already shown
            if (!isShown && !wasShown) { return; }

            if (isShown) listCtrl.closeOptionButtons();
            listCtrl.canSwipeItems(!isShown);

            $element.children().toggleClass('list-left-editing', isShown);
            $element.toggleClass('disable-pointer-events', isShown);

            var deleteButton = jqLite($element[0].getElementsByClassName('item-delete'));
            setButtonShown(deleteButton, listCtrl.showDelete);
          });

          $scope.$watch(function() {
            return listCtrl.showReorder();
          }, function(isShown, wasShown) {
            //Only use isShown=false if it was already shown
            if (!isShown && !wasShown) { return; }

            if (isShown) listCtrl.closeOptionButtons();
            listCtrl.canSwipeItems(!isShown);

            $element.children().toggleClass('list-right-editing', isShown);
            $element.toggleClass('disable-pointer-events', isShown);

            var reorderButton = jqLite($element[0].getElementsByClassName('item-reorder'));
            setButtonShown(reorderButton, listCtrl.showReorder);
          });

          function setButtonShown(el, shown) {
            shown() && el.addClass('visible') || el.removeClass('active');
            ionic.requestAnimationFrame(function() {
              shown() && el.addClass('active') || el.removeClass('visible');
            });
          }
        }

      };
    }
  };
}]);

/**
 * @ngdoc directive
 * @name menuClose
 * @module ionic
 * @restrict AC
 *
 * @description
 * `menu-close` is an attribute directive that closes a currently opened side menu.
 * Note that by default, navigation transitions will not animate between views when
 * the menu is open. Additionally, this directive will reset the entering view's
 * history stack, making the new page the root of the history stack. This is done
 * to replicate the user experience seen in most side menu implementations, which is
 * to not show the back button at the root of the stack and show only the
 * menu button. We recommend that you also use the `enable-menu-with-back-views="false"`
 * {@link ionic.directive:ionSideMenus} attribute when using the menuClose directive.
 *
 * @usage
 * Below is an example of a link within a side menu. Tapping this link would
 * automatically close the currently opened menu.
 *
 * ```html
 * <a menu-close href="#/home" class="item">Home</a>
 * ```
 *
 * Note that if your destination state uses a resolve and that resolve asynchronously
 * takes longer than a standard transition (300ms), you'll need to set the
 * `nextViewOptions` manually as your resolve completes.
 *
 * ```js
 * $ionicHistory.nextViewOptions({
 *  historyRoot: true,
 *  disableAnimate: true,
 *  expire: 300
 * });
 * ```
 */
IonicModule
.directive('menuClose', ['$ionicHistory', '$timeout', function($ionicHistory, $timeout) {
  return {
    restrict: 'AC',
    link: function($scope, $element) {
      $element.bind('click', function() {
        var sideMenuCtrl = $element.inheritedData('$ionSideMenusController');
        if (sideMenuCtrl) {
          $ionicHistory.nextViewOptions({
            historyRoot: true,
            disableAnimate: true,
            expire: 300
          });
          // if no transition in 300ms, reset nextViewOptions
          // the expire should take care of it, but will be cancelled in some
          // cases. This directive is an exception to the rules of history.js
          $timeout( function() {
            $ionicHistory.nextViewOptions({
              historyRoot: false,
              disableAnimate: false
            });
          }, 300);
          sideMenuCtrl.close();
        }
      });
    }
  };
}]);

/**
 * @ngdoc directive
 * @name menuToggle
 * @module ionic
 * @restrict AC
 *
 * @description
 * Toggle a side menu on the given side.
 *
 * @usage
 * Below is an example of a link within a nav bar. Tapping this button
 * would open the given side menu, and tapping it again would close it.
 *
 * ```html
 * <ion-nav-bar>
 *   <ion-nav-buttons side="left">
 *    <!-- Toggle left side menu -->
 *    <button menu-toggle="left" class="button button-icon icon ion-navicon"></button>
 *   </ion-nav-buttons>
 *   <ion-nav-buttons side="right">
 *    <!-- Toggle right side menu -->
 *    <button menu-toggle="right" class="button button-icon icon ion-navicon"></button>
 *   </ion-nav-buttons>
 * </ion-nav-bar>
 * ```
 *
 * ### Button Hidden On Child Views
 * By default, the menu toggle button will only appear on a root
 * level side-menu page. Navigating in to child views will hide the menu-
 * toggle button. They can be made visible on child pages by setting the
 * enable-menu-with-back-views attribute of the {@link ionic.directive:ionSideMenus}
 * directive to true.
 *
 * ```html
 * <ion-side-menus enable-menu-with-back-views="true">
 * ```
 */
IonicModule
.directive('menuToggle', function() {
  return {
    restrict: 'AC',
    link: function($scope, $element, $attr) {
      $scope.$on('$ionicView.beforeEnter', function(ev, viewData) {
        if (viewData.enableBack) {
          var sideMenuCtrl = $element.inheritedData('$ionSideMenusController');
          if (!sideMenuCtrl.enableMenuWithBackViews()) {
            $element.addClass('hide');
          }
        } else {
          $element.removeClass('hide');
        }
      });

      $element.bind('click', function() {
        var sideMenuCtrl = $element.inheritedData('$ionSideMenusController');
        sideMenuCtrl && sideMenuCtrl.toggle($attr.menuToggle);
      });
    }
  };
});

/*
 * We don't document the ionModal directive, we instead document
 * the $ionicModal service
 */
IonicModule
.directive('ionModal', [function() {
  return {
    restrict: 'E',
    transclude: true,
    replace: true,
    controller: [function() {}],
    template: '<div class="modal-backdrop">' +
                '<div class="modal-backdrop-bg"></div>' +
                '<div class="modal-wrapper" ng-transclude></div>' +
              '</div>'
  };
}]);

IonicModule
.directive('ionModalView', function() {
  return {
    restrict: 'E',
    compile: function(element) {
      element.addClass('modal');
    }
  };
});

/**
 * @ngdoc directive
 * @name ionNavBackButton
 * @module ionic
 * @restrict E
 * @parent ionNavBar
 * @description
 * Creates a back button inside an {@link ionic.directive:ionNavBar}.
 *
 * The back button will appear when the user is able to go back in the current navigation stack. By
 * default, the markup of the back button is automatically built using platform-appropriate defaults
 * (iOS back button icon on iOS and Android icon on Android).
 *
 * Additionally, the button is automatically set to `$ionicGoBack()` on click/tap. By default, the
 * app will navigate back one view when the back button is clicked.  More advanced behavior is also
 * possible, as outlined below.
 *
 * @usage
 *
 * Recommended markup for default settings:
 *
 * ```html
 * <ion-nav-bar>
 *   <ion-nav-back-button>
 *   </ion-nav-back-button>
 * </ion-nav-bar>
 * ```
 *
 * With custom inner markup, and automatically adds a default click action:
 *
 * ```html
 * <ion-nav-bar>
 *   <ion-nav-back-button class="button-clear">
 *     <i class="ion-arrow-left-c"></i> Back
 *   </ion-nav-back-button>
 * </ion-nav-bar>
 * ```
 *
 * With custom inner markup and custom click action, using {@link ionic.service:$ionicHistory}:
 *
 * ```html
 * <ion-nav-bar ng-controller="MyCtrl">
 *   <ion-nav-back-button class="button-clear"
 *     ng-click="myGoBack()">
 *     <i class="ion-arrow-left-c"></i> Back
 *   </ion-nav-back-button>
 * </ion-nav-bar>
 * ```
 * ```js
 * function MyCtrl($scope, $ionicHistory) {
 *   $scope.myGoBack = function() {
 *     $ionicHistory.goBack();
 *   };
 * }
 * ```
 */
IonicModule
.directive('ionNavBackButton', ['$ionicConfig', '$document', function($ionicConfig, $document) {
  return {
    restrict: 'E',
    require: '^ionNavBar',
    compile: function(tElement, tAttrs) {

      // clone the back button, but as a <div>
      var buttonEle = $document[0].createElement('button');
      for (var n in tAttrs.$attr) {
        buttonEle.setAttribute(tAttrs.$attr[n], tAttrs[n]);
      }

      if (!tAttrs.ngClick) {
        buttonEle.setAttribute('ng-click', '$ionicGoBack()');
      }

      buttonEle.className = 'button back-button hide buttons ' + (tElement.attr('class') || '');
      buttonEle.innerHTML = tElement.html() || '';

      var childNode;
      var hasIcon = hasIconClass(tElement[0]);
      var hasInnerText;
      var hasButtonText;
      var hasPreviousTitle;

      for (var x = 0; x < tElement[0].childNodes.length; x++) {
        childNode = tElement[0].childNodes[x];
        if (childNode.nodeType === 1) {
          if (hasIconClass(childNode)) {
            hasIcon = true;
          } else if (childNode.classList.contains('default-title')) {
            hasButtonText = true;
          } else if (childNode.classList.contains('previous-title')) {
            hasPreviousTitle = true;
          }
        } else if (!hasInnerText && childNode.nodeType === 3) {
          hasInnerText = !!childNode.nodeValue.trim();
        }
      }

      function hasIconClass(ele) {
        return /ion-|icon/.test(ele.className);
      }

      var defaultIcon = $ionicConfig.backButton.icon();
      if (!hasIcon && defaultIcon && defaultIcon !== 'none') {
        buttonEle.innerHTML = '<i class="icon ' + defaultIcon + '"></i> ' + buttonEle.innerHTML;
        buttonEle.className += ' button-clear';
      }

      if (!hasInnerText) {
        var buttonTextEle = $document[0].createElement('span');
        buttonTextEle.className = 'back-text';

        if (!hasButtonText && $ionicConfig.backButton.text()) {
          buttonTextEle.innerHTML += '<span class="default-title">' + $ionicConfig.backButton.text() + '</span>';
        }
        if (!hasPreviousTitle && $ionicConfig.backButton.previousTitleText()) {
          buttonTextEle.innerHTML += '<span class="previous-title"></span>';
        }
        buttonEle.appendChild(buttonTextEle);

      }

      tElement.attr('class', 'hide');
      tElement.empty();

      return {
        pre: function($scope, $element, $attr, navBarCtrl) {
          // only register the plain HTML, the navBarCtrl takes care of scope/compile/link
          navBarCtrl.navElement('backButton', buttonEle.outerHTML);
          buttonEle = null;
        }
      };
    }
  };
}]);


/**
 * @ngdoc directive
 * @name ionNavBar
 * @module ionic
 * @delegate ionic.service:$ionicNavBarDelegate
 * @restrict E
 *
 * @description
 * If we have an {@link ionic.directive:ionNavView} directive, we can also create an
 * `<ion-nav-bar>`, which will create a topbar that updates as the application state changes.
 *
 * We can add a back button by putting an {@link ionic.directive:ionNavBackButton} inside.
 *
 * We can add buttons depending on the currently visible view using
 * {@link ionic.directive:ionNavButtons}.
 *
 * Note that the ion-nav-bar element will only work correctly if your content has an
 * ionView around it.
 *
 * @usage
 *
 * ```html
 * <body ng-app="starter">
 *   <!-- The nav bar that will be updated as we navigate -->
 *   <ion-nav-bar class="bar-positive">
 *   </ion-nav-bar>
 *
 *   <!-- where the initial view template will be rendered -->
 *   <ion-nav-view>
 *     <ion-view>
 *       <ion-content>Hello!</ion-content>
 *     </ion-view>
 *   </ion-nav-view>
 * </body>
 * ```
 *
 * @param {string=} delegate-handle The handle used to identify this navBar
 * with {@link ionic.service:$ionicNavBarDelegate}.
 * @param align-title {string=} Where to align the title of the navbar.
 * Available: 'left', 'right', 'center'. Defaults to 'center'.
 * @param {boolean=} no-tap-scroll By default, the navbar will scroll the content
 * to the top when tapped.  Set no-tap-scroll to true to disable this behavior.
 *
 */
IonicModule
.directive('ionNavBar', function() {
  return {
    restrict: 'E',
    controller: '$ionicNavBar',
    scope: true,
    link: function($scope, $element, $attr, ctrl) {
      ctrl.init();
    }
  };
});


/**
 * @ngdoc directive
 * @name ionNavButtons
 * @module ionic
 * @restrict E
 * @parent ionNavView
 *
 * @description
 * Use nav buttons to set the buttons on your {@link ionic.directive:ionNavBar}
 * from within an {@link ionic.directive:ionView}. This gives each
 * view template the ability to specify which buttons should show in the nav bar,
 * overriding any default buttons already placed in the nav bar.
 *
 * Any buttons you declare will be positioned on the navbar's corresponding side. Primary
 * buttons generally map to the left side of the header, and secondary buttons are
 * generally on the right side. However, their exact locations are platform-specific.
 * For example, in iOS, the primary buttons are on the far left of the header, and
 * secondary buttons are on the far right, with the header title centered between them.
 * For Android, however, both groups of buttons are on the far right of the header,
 * with the header title aligned left.
 *
 * We recommend always using `primary` and `secondary`, so the buttons correctly map
 * to the side familiar to users of each platform. However, in cases where buttons should
 * always be on an exact side, both `left` and `right` sides are still available. For
 * example, a toggle button for a left side menu should be on the left side; in this case,
 * we'd recommend using `side="left"`, so it's always on the left, no matter the platform.
 *
 * ***Note*** that `ion-nav-buttons` must be immediate descendants of the `ion-view` or
 * `ion-nav-bar` element (basically, don't wrap it in another div).
 *
 * @usage
 * ```html
 * <ion-nav-bar>
 * </ion-nav-bar>
 * <ion-nav-view>
 *   <ion-view>
 *     <ion-nav-buttons side="primary">
 *       <button class="button" ng-click="doSomething()">
 *         I'm a button on the primary of the navbar!
 *       </button>
 *     </ion-nav-buttons>
 *     <ion-content>
 *       Some super content here!
 *     </ion-content>
 *   </ion-view>
 * </ion-nav-view>
 * ```
 *
 * @param {string} side The side to place the buttons in the
 * {@link ionic.directive:ionNavBar}. Available sides: `primary`, `secondary`, `left`, and `right`.
 */
IonicModule
.directive('ionNavButtons', ['$document', function($document) {
  return {
    require: '^ionNavBar',
    restrict: 'E',
    compile: function(tElement, tAttrs) {
      var side = 'left';

      if (/^primary|secondary|right$/i.test(tAttrs.side || '')) {
        side = tAttrs.side.toLowerCase();
      }

      var spanEle = $document[0].createElement('span');
      spanEle.className = side + '-buttons';
      spanEle.innerHTML = tElement.html();

      var navElementType = side + 'Buttons';

      tElement.attr('class', 'hide');
      tElement.empty();

      return {
        pre: function($scope, $element, $attrs, navBarCtrl) {
          // only register the plain HTML, the navBarCtrl takes care of scope/compile/link

          var parentViewCtrl = $element.parent().data('$ionViewController');
          if (parentViewCtrl) {
            // if the parent is an ion-view, then these are ion-nav-buttons for JUST this ion-view
            parentViewCtrl.navElement(navElementType, spanEle.outerHTML);

          } else {
            // these are buttons for all views that do not have their own ion-nav-buttons
            navBarCtrl.navElement(navElementType, spanEle.outerHTML);
          }

          spanEle = null;
        }
      };
    }
  };
}]);

/**
 * @ngdoc directive
 * @name navDirection
 * @module ionic
 * @restrict A
 *
 * @description
 * The direction which the nav view transition should animate. Available options
 * are: `forward`, `back`, `enter`, `exit`, `swap`.
 *
 * @usage
 *
 * ```html
 * <a nav-direction="forward" href="#/home">Home</a>
 * ```
 */
IonicModule
.directive('navDirection', ['$ionicViewSwitcher', function($ionicViewSwitcher) {
  return {
    restrict: 'A',
    priority: 1000,
    link: function($scope, $element, $attr) {
      $element.bind('click', function() {
        $ionicViewSwitcher.nextDirection($attr.navDirection);
      });
    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionNavTitle
 * @module ionic
 * @restrict E
 * @parent ionNavView
 *
 * @description
 *
 * The nav title directive replaces an {@link ionic.directive:ionNavBar} title text with
 * custom HTML from within an {@link ionic.directive:ionView} template. This gives each
 * view the ability to specify its own custom title element, such as an image or any HTML,
 * rather than being text-only. Alternatively, text-only titles can be updated using the
 * `view-title` {@link ionic.directive:ionView} attribute.
 *
 * Note that `ion-nav-title` must be an immediate descendant of the `ion-view` or
 * `ion-nav-bar` element (basically don't wrap it in another div).
 *
 * @usage
 * ```html
 * <ion-nav-bar>
 * </ion-nav-bar>
 * <ion-nav-view>
 *   <ion-view>
 *     <ion-nav-title>
 *       <img src="logo.svg">
 *     </ion-nav-title>
 *     <ion-content>
 *       Some super content here!
 *     </ion-content>
 *   </ion-view>
 * </ion-nav-view>
 * ```
 *
 */
IonicModule
.directive('ionNavTitle', ['$document', function($document) {
  return {
    require: '^ionNavBar',
    restrict: 'E',
    compile: function(tElement, tAttrs) {
      var navElementType = 'title';
      var spanEle = $document[0].createElement('span');
      for (var n in tAttrs.$attr) {
        spanEle.setAttribute(tAttrs.$attr[n], tAttrs[n]);
      }
      spanEle.classList.add('nav-bar-title');
      spanEle.innerHTML = tElement.html();

      tElement.attr('class', 'hide');
      tElement.empty();

      return {
        pre: function($scope, $element, $attrs, navBarCtrl) {
          // only register the plain HTML, the navBarCtrl takes care of scope/compile/link

          var parentViewCtrl = $element.parent().data('$ionViewController');
          if (parentViewCtrl) {
            // if the parent is an ion-view, then these are ion-nav-buttons for JUST this ion-view
            parentViewCtrl.navElement(navElementType, spanEle.outerHTML);

          } else {
            // these are buttons for all views that do not have their own ion-nav-buttons
            navBarCtrl.navElement(navElementType, spanEle.outerHTML);
          }

          spanEle = null;
        }
      };
    }
  };
}]);

/**
 * @ngdoc directive
 * @name navTransition
 * @module ionic
 * @restrict A
 *
 * @description
 * The transition type which the nav view transition should use when it animates.
 * Current, options are `ios`, `android`, and `none`. More options coming soon.
 *
 * @usage
 *
 * ```html
 * <a nav-transition="none" href="#/home">Home</a>
 * ```
 */
IonicModule
.directive('navTransition', ['$ionicViewSwitcher', function($ionicViewSwitcher) {
  return {
    restrict: 'A',
    priority: 1000,
    link: function($scope, $element, $attr) {
      $element.bind('click', function() {
        $ionicViewSwitcher.nextTransition($attr.navTransition);
      });
    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionNavView
 * @module ionic
 * @restrict E
 * @codepen odqCz
 *
 * @description
 * As a user navigates throughout your app, Ionic is able to keep track of their
 * navigation history. By knowing their history, transitions between views
 * correctly enter and exit using the platform's transition style. An additional
 * benefit to Ionic's navigation system is its ability to manage multiple
 * histories. For example, each tab can have it's own navigation history stack.
 *
 * Ionic uses the AngularUI Router module so app interfaces can be organized
 * into various "states". Like Angular's core $route service, URLs can be used
 * to control the views. However, the AngularUI Router provides a more powerful
 * state manager in that states are bound to named, nested, and parallel views,
 * allowing more than one template to be rendered on the same page.
 * Additionally, each state is not required to be bound to a URL, and data can
 * be pushed to each state which allows much flexibility.
 *
 * The ionNavView directive is used to render templates in your application. Each template
 * is part of a state. States are usually mapped to a url, and are defined programatically
 * using angular-ui-router (see [their docs](https://github.com/angular-ui/ui-router/wiki),
 * and remember to replace ui-view with ion-nav-view in examples).
 *
 * @usage
 * In this example, we will create a navigation view that contains our different states for the app.
 *
 * To do this, in our markup we use ionNavView top level directive. To display a header bar we use
 * the {@link ionic.directive:ionNavBar} directive that updates as we navigate through the
 * navigation stack.
 *
 * Next, we need to setup our states that will be rendered.
 *
 * ```js
 * var app = angular.module('myApp', ['ionic']);
 * app.config(function($stateProvider) {
 *   $stateProvider
 *   .state('index', {
 *     url: '/',
 *     templateUrl: 'home.html'
 *   })
 *   .state('music', {
 *     url: '/music',
 *     templateUrl: 'music.html'
 *   });
 * });
 * ```
 * Then on app start, $stateProvider will look at the url, see if it matches the index state,
 * and then try to load home.html into the `<ion-nav-view>`.
 *
 * Pages are loaded by the URLs given. One simple way to create templates in Angular is to put
 * them directly into your HTML file and use the `<script type="text/ng-template">` syntax.
 * So here is one way to put home.html into our app:
 *
 * ```html
 * <script id="home" type="text/ng-template">
 *   <!-- The title of the ion-view will be shown on the navbar -->
 *   <ion-view view-title="Home">
 *     <ion-content ng-controller="HomeCtrl">
 *       <!-- The content of the page -->
 *       <a href="#/music">Go to music page!</a>
 *     </ion-content>
 *   </ion-view>
 * </script>
 * ```
 *
 * This is good to do because the template will be cached for very fast loading, instead of
 * having to fetch them from the network.
 *
 * ## Caching
 *
 * By default, views are cached to improve performance. When a view is navigated away from, its
 * element is left in the DOM, and its scope is disconnected from the `$watch` cycle. When
 * navigating to a view that is already cached, its scope is then reconnected, and the existing
 * element that was left in the DOM becomes the active view. This also allows for the scroll
 * position of previous views to be maintained.
 *
 * Caching can be disabled and enabled in multiple ways. By default, Ionic will cache a maximum of
 * 10 views, and not only can this be configured, but apps can also explicitly state which views
 * should and should not be cached.
 *
 * Note that because we are caching these views, *we aren’t destroying scopes*. Instead, scopes
 * are being disconnected from the watch cycle. Because scopes are not being destroyed and
 * recreated, controllers are not loading again on a subsequent viewing. If the app/controller
 * needs to know when a view has entered or has left, then view events emitted from the
 * {@link ionic.directive:ionView} scope, such as `$ionicView.enter`, may be useful.
 *
 * By default, when navigating back in the history, the "forward" views are removed from the cache.
 * If you navigate forward to the same view again, it'll create a new DOM element and controller
 * instance. Basically, any forward views are reset each time. This can be configured using the
 * {@link ionic.provider:$ionicConfigProvider}:
 *
 * ```js
 * $ionicConfigProvider.views.forwardCache(true);
 * ```
 *
 * #### Disable cache globally
 *
 * The {@link ionic.provider:$ionicConfigProvider} can be used to set the maximum allowable views
 * which can be cached, but this can also be use to disable all caching by setting it to 0.
 *
 * ```js
 * $ionicConfigProvider.views.maxCache(0);
 * ```
 *
 * #### Disable cache within state provider
 *
 * ```js
 * $stateProvider.state('myState', {
 *    cache: false,
 *    url : '/myUrl',
 *    templateUrl : 'my-template.html'
 * })
 * ```
 *
 * #### Disable cache with an attribute
 *
 * ```html
 * <ion-view cache-view="false" view-title="My Title!">
 *   ...
 * </ion-view>
 * ```
 *
 *
 * ## AngularUI Router
 *
 * Please visit [AngularUI Router's docs](https://github.com/angular-ui/ui-router/wiki) for
 * more info. Below is a great video by the AngularUI Router team that may help to explain
 * how it all works:
 *
 * <iframe width="560" height="315" src="//www.youtube.com/embed/dqJRoh8MnBo"
 * frameborder="0" allowfullscreen></iframe>
 *
 * Note: We do not recommend using [resolve](https://github.com/angular-ui/ui-router/wiki#resolve)
 * of AngularUI Router. The recommended approach is to execute any logic needed before beginning the state transition.
 *
 * @param {string=} name A view name. The name should be unique amongst the other views in the
 * same state. You can have views of the same name that live in different states. For more
 * information, see ui-router's
 * [ui-view documentation](http://angular-ui.github.io/ui-router/site/#/api/ui.router.state.directive:ui-view).
 */
IonicModule
.directive('ionNavView', [
  '$state',
  '$ionicConfig',
function($state, $ionicConfig) {
  // IONIC's fork of Angular UI Router, v0.2.10
  // the navView handles registering views in the history and how to transition between them
  return {
    restrict: 'E',
    terminal: true,
    priority: 2000,
    transclude: true,
    controller: '$ionicNavView',
    compile: function(tElement, tAttrs, transclude) {

      // a nav view element is a container for numerous views
      tElement.addClass('view-container');
      ionic.DomUtil.cachedAttr(tElement, 'nav-view-transition', $ionicConfig.views.transition());

      return function($scope, $element, $attr, navViewCtrl) {
        var latestLocals;

        // Put in the compiled initial view
        transclude($scope, function(clone) {
          $element.append(clone);
        });

        var viewData = navViewCtrl.init();

        // listen for $stateChangeSuccess
        $scope.$on('$stateChangeSuccess', function() {
          updateView(false);
        });
        $scope.$on('$viewContentLoading', function() {
          updateView(false);
        });

        // initial load, ready go
        updateView(true);


        function updateView(firstTime) {
          // get the current local according to the $state
          var viewLocals = $state.$current && $state.$current.locals[viewData.name];

          // do not update THIS nav-view if its is not the container for the given state
          // if the viewLocals are the same as THIS latestLocals, then nothing to do
          if (!viewLocals || (!firstTime && viewLocals === latestLocals)) return;

          // update the latestLocals
          latestLocals = viewLocals;
          viewData.state = viewLocals.$$state;

          // register, update and transition to the new view
          navViewCtrl.register(viewLocals);
        }

      };
    }
  };
}]);

IonicModule

.config(['$provide', function($provide) {
  $provide.decorator('ngClickDirective', ['$delegate', function($delegate) {
    // drop the default ngClick directive
    $delegate.shift();
    return $delegate;
  }]);
}])

/**
 * @private
 */
.factory('$ionicNgClick', ['$parse', function($parse) {
  return function(scope, element, clickExpr) {
    var clickHandler = angular.isFunction(clickExpr) ?
      clickExpr :
      $parse(clickExpr);

    element.on('click', function(event) {
      scope.$apply(function() {
        clickHandler(scope, {$event: (event)});
      });
    });

    // Hack for iOS Safari's benefit. It goes searching for onclick handlers and is liable to click
    // something else nearby.
    element.onclick = noop;
  };
}])

.directive('ngClick', ['$ionicNgClick', function($ionicNgClick) {
  return function(scope, element, attr) {
    $ionicNgClick(scope, element, attr.ngClick);
  };
}])

.directive('ionStopEvent', function() {
  return {
    restrict: 'A',
    link: function(scope, element, attr) {
      element.bind(attr.ionStopEvent, eventStopPropagation);
    }
  };
});
function eventStopPropagation(e) {
  e.stopPropagation();
}


/**
 * @ngdoc directive
 * @name ionPane
 * @module ionic
 * @restrict E
 *
 * @description A simple container that fits content, with no side effects.  Adds the 'pane' class to the element.
 */
IonicModule
.directive('ionPane', function() {
  return {
    restrict: 'E',
    link: function(scope, element) {
      element.addClass('pane');
    }
  };
});

/*
 * We don't document the ionPopover directive, we instead document
 * the $ionicPopover service
 */
IonicModule
.directive('ionPopover', [function() {
  return {
    restrict: 'E',
    transclude: true,
    replace: true,
    controller: [function() {}],
    template: '<div class="popover-backdrop">' +
                '<div class="popover-wrapper" ng-transclude></div>' +
              '</div>'
  };
}]);

IonicModule
.directive('ionPopoverView', function() {
  return {
    restrict: 'E',
    compile: function(element) {
      element.append(jqLite('<div class="popover-arrow">'));
      element.addClass('popover');
    }
  };
});

/**
 * @ngdoc directive
 * @name ionRadio
 * @module ionic
 * @restrict E
 * @codepen saoBG
 * @description
 * The radio directive is no different than the HTML radio input, except it's styled differently.
 *
 * Radio behaves like [AngularJS radio](http://docs.angularjs.org/api/ng/input/input[radio]).
 *
 * @usage
 * ```html
 * <ion-radio ng-model="choice" ng-value="'A'">Choose A</ion-radio>
 * <ion-radio ng-model="choice" ng-value="'B'">Choose B</ion-radio>
 * <ion-radio ng-model="choice" ng-value="'C'">Choose C</ion-radio>
 * ```
 *
 * @param {string=} name The name of the radio input.
 * @param {expression=} value The value of the radio input.
 * @param {boolean=} disabled The state of the radio input.
 * @param {string=} icon The icon to use when the radio input is selected.
 * @param {expression=} ng-value Angular equivalent of the value attribute.
 * @param {expression=} ng-model The angular model for the radio input.
 * @param {boolean=} ng-disabled Angular equivalent of the disabled attribute.
 * @param {expression=} ng-change Triggers given expression when radio input's model changes
 */
IonicModule
.directive('ionRadio', function() {
  return {
    restrict: 'E',
    replace: true,
    require: '?ngModel',
    transclude: true,
    template:
      '<label class="item item-radio">' +
        '<input type="radio" name="radio-group">' +
        '<div class="radio-content">' +
          '<div class="item-content disable-pointer-events" ng-transclude></div>' +
          '<i class="radio-icon disable-pointer-events icon ion-checkmark"></i>' +
        '</div>' +
      '</label>',

    compile: function(element, attr) {
      if (attr.icon) {
        var iconElm = element.find('i');
        iconElm.removeClass('ion-checkmark').addClass(attr.icon);
      }

      var input = element.find('input');
      forEach({
          'name': attr.name,
          'value': attr.value,
          'disabled': attr.disabled,
          'ng-value': attr.ngValue,
          'ng-model': attr.ngModel,
          'ng-disabled': attr.ngDisabled,
          'ng-change': attr.ngChange,
          'ng-required': attr.ngRequired,
          'required': attr.required
      }, function(value, name) {
        if (isDefined(value)) {
            input.attr(name, value);
          }
      });

      return function(scope, element, attr) {
        scope.getValue = function() {
          return scope.ngValue || attr.value;
        };
      };
    }
  };
});


/**
 * @ngdoc directive
 * @name ionRefresher
 * @module ionic
 * @restrict E
 * @parent ionic.directive:ionContent, ionic.directive:ionScroll
 * @description
 * Allows you to add pull-to-refresh to a scrollView.
 *
 * Place it as the first child of your {@link ionic.directive:ionContent} or
 * {@link ionic.directive:ionScroll} element.
 *
 * When refreshing is complete, $broadcast the 'scroll.refreshComplete' event
 * from your controller.
 *
 * @usage
 *
 * ```html
 * <ion-content ng-controller="MyController">
 *   <ion-refresher
 *     pulling-text="Pull to refresh..."
 *     on-refresh="doRefresh()">
 *   </ion-refresher>
 *   <ion-list>
 *     <ion-item ng-repeat="item in items"></ion-item>
 *   </ion-list>
 * </ion-content>
 * ```
 * ```js
 * angular.module('testApp', ['ionic'])
 * .controller('MyController', function($scope, $http) {
 *   $scope.items = [1,2,3];
 *   $scope.doRefresh = function() {
 *     $http.get('/new-items')
 *      .success(function(newItems) {
 *        $scope.items = newItems;
 *      })
 *      .finally(function() {
 *        // Stop the ion-refresher from spinning
 *        $scope.$broadcast('scroll.refreshComplete');
 *      });
 *   };
 * });
 * ```
 *
 * @param {expression=} on-refresh Called when the user pulls down enough and lets go
 * of the refresher.
 * @param {expression=} on-pulling Called when the user starts to pull down
 * on the refresher.
 * @param {string=} pulling-text The text to display while the user is pulling down.
 * @param {string=} pulling-icon The icon to display while the user is pulling down.
 * Default: 'ion-android-arrow-down'.
 * @param {string=} spinner The {@link ionic.directive:ionSpinner} icon to display
 * after user lets go of the refresher. The SVG {@link ionic.directive:ionSpinner}
 * is now the default, replacing rotating font icons. Set to `none` to disable both the
 * spinner and the icon.
 * @param {string=} refreshing-icon The font icon to display after user lets go of the
 * refresher. This is deprecated in favor of the SVG {@link ionic.directive:ionSpinner}.
 * @param {boolean=} disable-pulling-rotation Disables the rotation animation of the pulling
 * icon when it reaches its activated threshold. To be used with a custom `pulling-icon`.
 *
 */
IonicModule
.directive('ionRefresher', [function() {
  return {
    restrict: 'E',
    replace: true,
    require: ['?^$ionicScroll', 'ionRefresher'],
    controller: '$ionicRefresher',
    template:
    '<div class="scroll-refresher invisible" collection-repeat-ignore>' +
      '<div class="ionic-refresher-content" ' +
      'ng-class="{\'ionic-refresher-with-text\': pullingText || refreshingText}">' +
        '<div class="icon-pulling" ng-class="{\'pulling-rotation-disabled\':disablePullingRotation}">' +
          '<i class="icon {{pullingIcon}}"></i>' +
        '</div>' +
        '<div class="text-pulling" ng-bind-html="pullingText"></div>' +
        '<div class="icon-refreshing">' +
          '<ion-spinner ng-if="showSpinner" icon="{{spinner}}"></ion-spinner>' +
          '<i ng-if="showIcon" class="icon {{refreshingIcon}}"></i>' +
        '</div>' +
        '<div class="text-refreshing" ng-bind-html="refreshingText"></div>' +
      '</div>' +
    '</div>',
    link: function($scope, $element, $attrs, ctrls) {

      // JS Scrolling uses the scroll controller
      var scrollCtrl = ctrls[0],
          refresherCtrl = ctrls[1];
      if (!scrollCtrl || scrollCtrl.isNative()) {
        // Kick off native scrolling
        refresherCtrl.init();
      } else {
        $element[0].classList.add('js-scrolling');
        scrollCtrl._setRefresher(
          $scope,
          $element[0],
          refresherCtrl.getRefresherDomMethods()
        );

        $scope.$on('scroll.refreshComplete', function() {
          $scope.$evalAsync(function() {
            if(scrollCtrl.scrollView){
              scrollCtrl.scrollView.finishPullToRefresh();
            }
          });
        });
      }

    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionScroll
 * @module ionic
 * @delegate ionic.service:$ionicScrollDelegate
 * @codepen mwFuh
 * @restrict E
 *
 * @description
 * Creates a scrollable container for all content inside.
 *
 * @usage
 *
 * Basic usage:
 *
 * ```html
 * <ion-scroll zooming="true" direction="xy" style="width: 500px; height: 500px">
 *   <div style="width: 5000px; height: 5000px; background: url('https://upload.wikimedia.org/wikipedia/commons/a/ad/Europe_geological_map-en.jpg') repeat"></div>
 *  </ion-scroll>
 * ```
 *
 * Note that it's important to set the height of the scroll box as well as the height of the inner
 * content to enable scrolling. This makes it possible to have full control over scrollable areas.
 *
 * If you'd just like to have a center content scrolling area, use {@link ionic.directive:ionContent} instead.
 *
 * @param {string=} delegate-handle The handle used to identify this scrollView
 * with {@link ionic.service:$ionicScrollDelegate}.
 * @param {string=} direction Which way to scroll. 'x' or 'y' or 'xy'. Default 'y'.
 * @param {boolean=} locking Whether to lock scrolling in one direction at a time. Useful to set to false when zoomed in or scrolling in two directions. Default true.
 * @param {boolean=} paging Whether to scroll with paging.
 * @param {expression=} on-refresh Called on pull-to-refresh, triggered by an {@link ionic.directive:ionRefresher}.
 * @param {expression=} on-scroll Called whenever the user scrolls.
 * @param {expression=} on-scroll-complete Called whenever the scrolling paging is completed.
 * @param {boolean=} scrollbar-x Whether to show the horizontal scrollbar. Default true.
 * @param {boolean=} scrollbar-y Whether to show the vertical scrollbar. Default true.
 * @param {boolean=} zooming Whether to support pinch-to-zoom
 * @param {integer=} min-zoom The smallest zoom amount allowed (default is 0.5)
 * @param {integer=} max-zoom The largest zoom amount allowed (default is 3)
 * @param {boolean=} has-bouncing Whether to allow scrolling to bounce past the edges
 * of the content.  Defaults to true on iOS, false on Android.
 */
IonicModule
.directive('ionScroll', [
  '$timeout',
  '$controller',
  '$ionicBind',
  '$ionicConfig',
function($timeout, $controller, $ionicBind, $ionicConfig) {
  return {
    restrict: 'E',
    scope: true,
    controller: function() {},
    compile: function(element, attr) {
      element.addClass('scroll-view ionic-scroll');

      //We cannot transclude here because it breaks element.data() inheritance on compile
      var innerElement = jqLite('<div class="scroll"></div>');
      innerElement.append(element.contents());
      element.append(innerElement);

      var nativeScrolling = attr.overflowScroll !== "false" && (attr.overflowScroll === "true" || !$ionicConfig.scrolling.jsScrolling());

      return { pre: prelink };
      function prelink($scope, $element, $attr) {
        $ionicBind($scope, $attr, {
          direction: '@',
          paging: '@',
          $onScroll: '&onScroll',
          $onScrollComplete: '&onScrollComplete',
          scroll: '@',
          scrollbarX: '@',
          scrollbarY: '@',
          zooming: '@',
          minZoom: '@',
          maxZoom: '@'
        });
        $scope.direction = $scope.direction || 'y';

        if (isDefined($attr.padding)) {
          $scope.$watch($attr.padding, function(newVal) {
            innerElement.toggleClass('padding', !!newVal);
          });
        }
        if ($scope.$eval($scope.paging) === true) {
          innerElement.addClass('scroll-paging');
        }

        if (!$scope.direction) { $scope.direction = 'y'; }
        var isPaging = $scope.$eval($scope.paging) === true;

        if (nativeScrolling) {
          $element.addClass('overflow-scroll');
        }

        $element.addClass('scroll-' + $scope.direction);

        var scrollViewOptions = {
          el: $element[0],
          delegateHandle: $attr.delegateHandle,
          locking: ($attr.locking || 'true') === 'true',
          bouncing: $scope.$eval($attr.hasBouncing),
          paging: isPaging,
          scrollbarX: $scope.$eval($scope.scrollbarX) !== false,
          scrollbarY: $scope.$eval($scope.scrollbarY) !== false,
          scrollingX: $scope.direction.indexOf('x') >= 0,
          scrollingY: $scope.direction.indexOf('y') >= 0,
          zooming: $scope.$eval($scope.zooming) === true,
          maxZoom: $scope.$eval($scope.maxZoom) || 3,
          minZoom: $scope.$eval($scope.minZoom) || 0.5,
          preventDefault: true,
          nativeScrolling: nativeScrolling,
          scrollingComplete: onScrollComplete
        };

        if (isPaging) {
          scrollViewOptions.speedMultiplier = 0.8;
          scrollViewOptions.bouncing = false;
        }

        var scrollCtrl = $controller('$ionicScroll', {
          $scope: $scope,
          scrollViewOptions: scrollViewOptions
        });

        function onScrollComplete() {
          $scope.$onScrollComplete && $scope.$onScrollComplete({
            scrollTop: scrollCtrl.scrollView.__scrollTop,
            scrollLeft: scrollCtrl.scrollView.__scrollLeft
          });
        }
      }
    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionSideMenu
 * @module ionic
 * @restrict E
 * @parent ionic.directive:ionSideMenus
 *
 * @description
 * A container for a side menu, sibling to an {@link ionic.directive:ionSideMenuContent} directive.
 *
 * @usage
 * ```html
 * <ion-side-menu
 *   side="left"
 *   width="myWidthValue + 20"
 *   is-enabled="shouldLeftSideMenuBeEnabled()">
 * </ion-side-menu>
 * ```
 * For a complete side menu example, see the
 * {@link ionic.directive:ionSideMenus} documentation.
 *
 * @param {string} side Which side the side menu is currently on.  Allowed values: 'left' or 'right'.
 * @param {boolean=} is-enabled Whether this side menu is enabled.
 * @param {number=} width How many pixels wide the side menu should be.  Defaults to 275.
 */
IonicModule
.directive('ionSideMenu', function() {
  return {
    restrict: 'E',
    require: '^ionSideMenus',
    scope: true,
    compile: function(element, attr) {
      angular.isUndefined(attr.isEnabled) && attr.$set('isEnabled', 'true');
      angular.isUndefined(attr.width) && attr.$set('width', '275');

      element.addClass('menu menu-' + attr.side);

      return function($scope, $element, $attr, sideMenuCtrl) {
        $scope.side = $attr.side || 'left';

        var sideMenu = sideMenuCtrl[$scope.side] = new ionic.views.SideMenu({
          width: attr.width,
          el: $element[0],
          isEnabled: true
        });

        $scope.$watch($attr.width, function(val) {
          var numberVal = +val;
          if (numberVal && numberVal == val) {
            sideMenu.setWidth(+val);
          }
        });
        $scope.$watch($attr.isEnabled, function(val) {
          sideMenu.setIsEnabled(!!val);
        });
      };
    }
  };
});


/**
 * @ngdoc directive
 * @name ionSideMenuContent
 * @module ionic
 * @restrict E
 * @parent ionic.directive:ionSideMenus
 *
 * @description
 * A container for the main visible content, sibling to one or more
 * {@link ionic.directive:ionSideMenu} directives.
 *
 * @usage
 * ```html
 * <ion-side-menu-content
 *   edge-drag-threshold="true"
 *   drag-content="true">
 * </ion-side-menu-content>
 * ```
 * For a complete side menu example, see the
 * {@link ionic.directive:ionSideMenus} documentation.
 *
 * @param {boolean=} drag-content Whether the content can be dragged. Default true.
 * @param {boolean|number=} edge-drag-threshold Whether the content drag can only start if it is below a certain threshold distance from the edge of the screen.  Default false. Accepts three types of values:
   *  - If a non-zero number is given, that many pixels is used as the maximum allowed distance from the edge that starts dragging the side menu.
   *  - If true is given, the default number of pixels (25) is used as the maximum allowed distance.
   *  - If false or 0 is given, the edge drag threshold is disabled, and dragging from anywhere on the content is allowed.
 *
 */
IonicModule
.directive('ionSideMenuContent', [
  '$timeout',
  '$ionicGesture',
  '$window',
function($timeout, $ionicGesture, $window) {

  return {
    restrict: 'EA', //DEPRECATED 'A'
    require: '^ionSideMenus',
    scope: true,
    compile: function(element, attr) {
      element.addClass('menu-content pane');

      return { pre: prelink };
      function prelink($scope, $element, $attr, sideMenuCtrl) {
        var startCoord = null;
        var primaryScrollAxis = null;

        if (isDefined(attr.dragContent)) {
          $scope.$watch(attr.dragContent, function(value) {
            sideMenuCtrl.canDragContent(value);
          });
        } else {
          sideMenuCtrl.canDragContent(true);
        }

        if (isDefined(attr.edgeDragThreshold)) {
          $scope.$watch(attr.edgeDragThreshold, function(value) {
            sideMenuCtrl.edgeDragThreshold(value);
          });
        }

        // Listen for taps on the content to close the menu
        function onContentTap(gestureEvt) {
          if (sideMenuCtrl.getOpenAmount() !== 0) {
            sideMenuCtrl.close();
            gestureEvt.gesture.srcEvent.preventDefault();
            startCoord = null;
            primaryScrollAxis = null;
          } else if (!startCoord) {
            startCoord = ionic.tap.pointerCoord(gestureEvt.gesture.srcEvent);
          }
        }

        function onDragX(e) {
          if (!sideMenuCtrl.isDraggableTarget(e)) return;

          if (getPrimaryScrollAxis(e) == 'x') {
            sideMenuCtrl._handleDrag(e);
            e.gesture.srcEvent.preventDefault();
          }
        }

        function onDragY(e) {
          if (getPrimaryScrollAxis(e) == 'x') {
            e.gesture.srcEvent.preventDefault();
          }
        }

        function onDragRelease(e) {
          sideMenuCtrl._endDrag(e);
          startCoord = null;
          primaryScrollAxis = null;
        }

        function getPrimaryScrollAxis(gestureEvt) {
          // gets whether the user is primarily scrolling on the X or Y
          // If a majority of the drag has been on the Y since the start of
          // the drag, but the X has moved a little bit, it's still a Y drag

          if (primaryScrollAxis) {
            // we already figured out which way they're scrolling
            return primaryScrollAxis;
          }

          if (gestureEvt && gestureEvt.gesture) {

            if (!startCoord) {
              // get the starting point
              startCoord = ionic.tap.pointerCoord(gestureEvt.gesture.srcEvent);

            } else {
              // we already have a starting point, figure out which direction they're going
              var endCoord = ionic.tap.pointerCoord(gestureEvt.gesture.srcEvent);

              var xDistance = Math.abs(endCoord.x - startCoord.x);
              var yDistance = Math.abs(endCoord.y - startCoord.y);

              var scrollAxis = (xDistance < yDistance ? 'y' : 'x');

              if (Math.max(xDistance, yDistance) > 30) {
                // ok, we pretty much know which way they're going
                // let's lock it in
                primaryScrollAxis = scrollAxis;
              }

              return scrollAxis;
            }
          }
          return 'y';
        }

        var content = {
          element: element[0],
          onDrag: function() {},
          endDrag: function() {},
          setCanScroll: function(canScroll) {
            var c = $element[0].querySelector('.scroll');

            if (!c) {
              return;
            }

            var content = angular.element(c.parentElement);
            if (!content) {
              return;
            }

            // freeze our scroll container if we have one
            var scrollScope = content.scope();
            scrollScope.scrollCtrl && scrollScope.scrollCtrl.freezeScrollShut(!canScroll);
          },
          getTranslateX: function() {
            return $scope.sideMenuContentTranslateX || 0;
          },
          setTranslateX: ionic.animationFrameThrottle(function(amount) {
            var xTransform = content.offsetX + amount;
            $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(' + xTransform + 'px,0,0)';
            $timeout(function() {
              $scope.sideMenuContentTranslateX = amount;
            });
          }),
          setMarginLeft: ionic.animationFrameThrottle(function(amount) {
            if (amount) {
              amount = parseInt(amount, 10);
              $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(' + amount + 'px,0,0)';
              $element[0].style.width = ($window.innerWidth - amount) + 'px';
              content.offsetX = amount;
            } else {
              $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(0,0,0)';
              $element[0].style.width = '';
              content.offsetX = 0;
            }
          }),
          setMarginRight: ionic.animationFrameThrottle(function(amount) {
            if (amount) {
              amount = parseInt(amount, 10);
              $element[0].style.width = ($window.innerWidth - amount) + 'px';
              content.offsetX = amount;
            } else {
              $element[0].style.width = '';
              content.offsetX = 0;
            }
            // reset incase left gets grabby
            $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(0,0,0)';
          }),
          setMarginLeftAndRight: ionic.animationFrameThrottle(function(amountLeft, amountRight) {
            amountLeft = amountLeft && parseInt(amountLeft, 10) || 0;
            amountRight = amountRight && parseInt(amountRight, 10) || 0;

            var amount = amountLeft + amountRight;

            if (amount > 0) {
              $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(' + amountLeft + 'px,0,0)';
              $element[0].style.width = ($window.innerWidth - amount) + 'px';
              content.offsetX = amountLeft;
            } else {
              $element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(0,0,0)';
              $element[0].style.width = '';
              content.offsetX = 0;
            }
            // reset incase left gets grabby
            //$element[0].style[ionic.CSS.TRANSFORM] = 'translate3d(0,0,0)';
          }),
          enableAnimation: function() {
            $scope.animationEnabled = true;
            $element[0].classList.add('menu-animated');
          },
          disableAnimation: function() {
            $scope.animationEnabled = false;
            $element[0].classList.remove('menu-animated');
          },
          offsetX: 0
        };

        sideMenuCtrl.setContent(content);

        // add gesture handlers
        var gestureOpts = { stop_browser_behavior: false };
        gestureOpts.prevent_default_directions = ['left', 'right'];
        var contentTapGesture = $ionicGesture.on('tap', onContentTap, $element, gestureOpts);
        var dragRightGesture = $ionicGesture.on('dragright', onDragX, $element, gestureOpts);
        var dragLeftGesture = $ionicGesture.on('dragleft', onDragX, $element, gestureOpts);
        var dragUpGesture = $ionicGesture.on('dragup', onDragY, $element, gestureOpts);
        var dragDownGesture = $ionicGesture.on('dragdown', onDragY, $element, gestureOpts);
        var releaseGesture = $ionicGesture.on('release', onDragRelease, $element, gestureOpts);

        // Cleanup
        $scope.$on('$destroy', function() {
          if (content) {
            content.element = null;
            content = null;
          }
          $ionicGesture.off(dragLeftGesture, 'dragleft', onDragX);
          $ionicGesture.off(dragRightGesture, 'dragright', onDragX);
          $ionicGesture.off(dragUpGesture, 'dragup', onDragY);
          $ionicGesture.off(dragDownGesture, 'dragdown', onDragY);
          $ionicGesture.off(releaseGesture, 'release', onDragRelease);
          $ionicGesture.off(contentTapGesture, 'tap', onContentTap);
        });
      }
    }
  };
}]);

IonicModule

/**
 * @ngdoc directive
 * @name ionSideMenus
 * @module ionic
 * @delegate ionic.service:$ionicSideMenuDelegate
 * @restrict E
 *
 * @description
 * A container element for side menu(s) and the main content. Allows the left and/or right side menu
 * to be toggled by dragging the main content area side to side.
 *
 * To automatically close an opened menu, you can add the {@link ionic.directive:menuClose} attribute
 * directive. The `menu-close` attribute is usually added to links and buttons within
 * `ion-side-menu-content`, so that when the element is clicked, the opened side menu will
 * automatically close.
 *
 * "Burger Icon" toggles can be added to the header with the {@link ionic.directive:menuToggle}
 * attribute directive. Clicking the toggle will open and close the side menu like the `menu-close`
 * directive. The side menu will automatically hide on child pages, but can be overridden with the
 * enable-menu-with-back-views attribute mentioned below.
 *
 * By default, side menus are hidden underneath their side menu content and can be opened by swiping
 * the content left or right or by toggling a button to show the side menu. Additionally, by adding the
 * {@link ionic.directive:exposeAsideWhen} attribute directive to an
 * {@link ionic.directive:ionSideMenu} element directive, a side menu can be given instructions about
 * "when" the menu should be exposed (always viewable).
 *
 * ![Side Menu](http://ionicframework.com.s3.amazonaws.com/docs/controllers/sidemenu.gif)
 *
 * For more information on side menus, check out:
 *
 * - {@link ionic.directive:ionSideMenuContent}
 * - {@link ionic.directive:ionSideMenu}
 * - {@link ionic.directive:menuToggle}
 * - {@link ionic.directive:menuClose}
 * - {@link ionic.directive:exposeAsideWhen}
 *
 * @usage
 * To use side menus, add an `<ion-side-menus>` parent element. This will encompass all pages that have a
 * side menu, and have at least 2 child elements: 1 `<ion-side-menu-content>` for the center content,
 * and one or more `<ion-side-menu>` directives for each side menu(left/right) that you wish to place.
 *
 * ```html
 * <ion-side-menus>
 *   <!-- Left menu -->
 *   <ion-side-menu side="left">
 *   </ion-side-menu>
 *
 *   <ion-side-menu-content>
 *   <!-- Main content, usually <ion-nav-view> -->
 *   </ion-side-menu-content>
 *
 *   <!-- Right menu -->
 *   <ion-side-menu side="right">
 *   </ion-side-menu>
 *
 * </ion-side-menus>
 * ```
 * ```js
 * function ContentController($scope, $ionicSideMenuDelegate) {
 *   $scope.toggleLeft = function() {
 *     $ionicSideMenuDelegate.toggleLeft();
 *   };
 * }
 * ```
 *
 * @param {bool=} enable-menu-with-back-views Determines whether the side menu is enabled when the
 * back button is showing. When set to `false`, any {@link ionic.directive:menuToggle} will be hidden,
 * and the user cannot swipe to open the menu. When going back to the root page of the side menu (the
 * page without a back button visible), then any menuToggle buttons will show again, and menus will be
 * enabled again.
 * @param {string=} delegate-handle The handle used to identify this side menu
 * with {@link ionic.service:$ionicSideMenuDelegate}.
 *
 */
.directive('ionSideMenus', ['$ionicBody', function($ionicBody) {
  return {
    restrict: 'ECA',
    controller: '$ionicSideMenus',
    compile: function(element, attr) {
      attr.$set('class', (attr['class'] || '') + ' view');

      return { pre: prelink };
      function prelink($scope, $element, $attrs, ctrl) {

        ctrl.enableMenuWithBackViews($scope.$eval($attrs.enableMenuWithBackViews));

        $scope.$on('$ionicExposeAside', function(evt, isAsideExposed) {
          if (!$scope.$exposeAside) $scope.$exposeAside = {};
          $scope.$exposeAside.active = isAsideExposed;
          $ionicBody.enableClass(isAsideExposed, 'aside-open');
        });

        $scope.$on('$ionicView.beforeEnter', function(ev, d) {
          if (d.historyId) {
            $scope.$activeHistoryId = d.historyId;
          }
        });

        $scope.$on('$destroy', function() {
          $ionicBody.removeClass('menu-open', 'aside-open');
        });

      }
    }
  };
}]);


/**
 * @ngdoc directive
 * @name ionSlideBox
 * @module ionic
 * @codepen AjgEB
 * @deprecated will be removed in the next Ionic release in favor of the new ion-slides component.
 * Don't depend on the internal behavior of this widget.
 * @delegate ionic.service:$ionicSlideBoxDelegate
 * @restrict E
 * @description
 * The Slide Box is a multi-page container where each page can be swiped or dragged between:
 *
 *
 * @usage
 * ```html
 * <ion-slide-box on-slide-changed="slideHasChanged($index)">
 *   <ion-slide>
 *     <div class="box blue"><h1>BLUE</h1></div>
 *   </ion-slide>
 *   <ion-slide>
 *     <div class="box yellow"><h1>YELLOW</h1></div>
 *   </ion-slide>
 *   <ion-slide>
 *     <div class="box pink"><h1>PINK</h1></div>
 *   </ion-slide>
 * </ion-slide-box>
 * ```
 *
 * @param {string=} delegate-handle The handle used to identify this slideBox
 * with {@link ionic.service:$ionicSlideBoxDelegate}.
 * @param {boolean=} does-continue Whether the slide box should loop.
 * @param {boolean=} auto-play Whether the slide box should automatically slide. Default true if does-continue is true.
 * @param {number=} slide-interval How many milliseconds to wait to change slides (if does-continue is true). Defaults to 4000.
 * @param {boolean=} show-pager Whether a pager should be shown for this slide box. Accepts expressions via `show-pager="{{shouldShow()}}"`. Defaults to true.
 * @param {expression=} pager-click Expression to call when a pager is clicked (if show-pager is true). Is passed the 'index' variable.
 * @param {expression=} on-slide-changed Expression called whenever the slide is changed.  Is passed an '$index' variable.
 * @param {expression=} active-slide Model to bind the current slide index to.
 */
IonicModule
.directive('ionSlideBox', [
  '$animate',
  '$timeout',
  '$compile',
  '$ionicSlideBoxDelegate',
  '$ionicHistory',
  '$ionicScrollDelegate',
function($animate, $timeout, $compile, $ionicSlideBoxDelegate, $ionicHistory, $ionicScrollDelegate) {
  return {
    restrict: 'E',
    replace: true,
    transclude: true,
    scope: {
      autoPlay: '=',
      doesContinue: '@',
      slideInterval: '@',
      showPager: '@',
      pagerClick: '&',
      disableScroll: '@',
      onSlideChanged: '&',
      activeSlide: '=?',
      bounce: '@'
    },
    controller: ['$scope', '$element', '$attrs', function($scope, $element, $attrs) {
      var _this = this;

      var continuous = $scope.$eval($scope.doesContinue) === true;
      var bouncing = ($scope.$eval($scope.bounce) !== false); //Default to true
      var shouldAutoPlay = isDefined($attrs.autoPlay) ? !!$scope.autoPlay : false;
      var slideInterval = shouldAutoPlay ? $scope.$eval($scope.slideInterval) || 4000 : 0;

      var slider = new ionic.views.Slider({
        el: $element[0],
        auto: slideInterval,
        continuous: continuous,
        startSlide: $scope.activeSlide,
        bouncing: bouncing,
        slidesChanged: function() {
          $scope.currentSlide = slider.currentIndex();

          // Try to trigger a digest
          $timeout(function() {});
        },
        callback: function(slideIndex) {
          $scope.currentSlide = slideIndex;
          $scope.onSlideChanged({ index: $scope.currentSlide, $index: $scope.currentSlide});
          $scope.$parent.$broadcast('slideBox.slideChanged', slideIndex);
          $scope.activeSlide = slideIndex;
          // Try to trigger a digest
          $timeout(function() {});
        },
        onDrag: function() {
          freezeAllScrolls(true);
        },
        onDragEnd: function() {
          freezeAllScrolls(false);
        }
      });

      function freezeAllScrolls(shouldFreeze) {
        if (shouldFreeze && !_this.isScrollFreeze) {
          $ionicScrollDelegate.freezeAllScrolls(shouldFreeze);

        } else if (!shouldFreeze && _this.isScrollFreeze) {
          $ionicScrollDelegate.freezeAllScrolls(false);
        }
        _this.isScrollFreeze = shouldFreeze;
      }

      slider.enableSlide($scope.$eval($attrs.disableScroll) !== true);

      $scope.$watch('activeSlide', function(nv) {
        if (isDefined(nv)) {
          slider.slide(nv);
        }
      });

      $scope.$on('slideBox.nextSlide', function() {
        slider.next();
      });

      $scope.$on('slideBox.prevSlide', function() {
        slider.prev();
      });

      $scope.$on('slideBox.setSlide', function(e, index) {
        slider.slide(index);
      });

      //Exposed for testing
      this.__slider = slider;

      var deregisterInstance = $ionicSlideBoxDelegate._registerInstance(
        slider, $attrs.delegateHandle, function() {
          return $ionicHistory.isActiveScope($scope);
        }
      );
      $scope.$on('$destroy', function() {
        deregisterInstance();
        slider.kill();
      });

      this.slidesCount = function() {
        return slider.slidesCount();
      };

      this.onPagerClick = function(index) {
        $scope.pagerClick({index: index});
      };

      $timeout(function() {
        slider.load();
      });
    }],
    template: '<div class="slider">' +
      '<div class="slider-slides" ng-transclude>' +
      '</div>' +
    '</div>',

    link: function($scope, $element, $attr) {
      // Disable ngAnimate for slidebox and its children
      $animate.enabled($element, false);

      // if showPager is undefined, show the pager
      if (!isDefined($attr.showPager)) {
        $scope.showPager = true;
        getPager().toggleClass('hide', !true);
      }

      $attr.$observe('showPager', function(show) {
        if (show === undefined) return;
        show = $scope.$eval(show);
        getPager().toggleClass('hide', !show);
      });

      var pager;
      function getPager() {
        if (!pager) {
          var childScope = $scope.$new();
          pager = jqLite('<ion-pager></ion-pager>');
          $element.append(pager);
          pager = $compile(pager)(childScope);
        }
        return pager;
      }
    }
  };
}])
.directive('ionSlide', function() {
  return {
    restrict: 'E',
    require: '?^ionSlideBox',
    compile: function(element) {
      element.addClass('slider-slide');
    }
  };
})

.directive('ionPager', function() {
  return {
    restrict: 'E',
    replace: true,
    require: '^ionSlideBox',
    template: '<div class="slider-pager"><span class="slider-pager-page" ng-repeat="slide in numSlides() track by $index" ng-class="{active: $index == currentSlide}" ng-click="pagerClick($index)"><i class="icon ion-record"></i></span></div>',
    link: function($scope, $element, $attr, slideBox) {
      var selectPage = function(index) {
        var children = $element[0].children;
        var length = children.length;
        for (var i = 0; i < length; i++) {
          if (i == index) {
            children[i].classList.add('active');
          } else {
            children[i].classList.remove('active');
          }
        }
      };

      $scope.pagerClick = function(index) {
        slideBox.onPagerClick(index);
      };

      $scope.numSlides = function() {
        return new Array(slideBox.slidesCount());
      };

      $scope.$watch('currentSlide', function(v) {
        selectPage(v);
      });
    }
  };

});


/**
 * @ngdoc directive
 * @name ionSlides
 * @module ionic
 * @restrict E
 * @description
 * The Slides component is a powerful multi-page container where each page can be swiped or dragged between.
 *
 * Note: this is a new version of the Ionic Slide Box based on the [Swiper](http://www.idangero.us/swiper/#.Vmc1J-ODFBc) widget from
 * [idangerous](http://www.idangero.us/).
 *
 * ![SlideBox](http://ionicframework.com.s3.amazonaws.com/docs/controllers/slideBox.gif)
 *
 * @usage
 * ```html
 * <ion-content scroll="false">
 *   <ion-slides  options="options" slider="data.slider">
 *     <ion-slide-page>
 *       <div class="box blue"><h1>BLUE</h1></div>
 *     </ion-slide-page>
 *     <ion-slide-page>
 *       <div class="box yellow"><h1>YELLOW</h1></div>
 *     </ion-slide-page>
 *     <ion-slide-page>
 *       <div class="box pink"><h1>PINK</h1></div>
 *     </ion-slide-page>
 *   </ion-slides>
 * </ion-content>
 * ```
 *
 * ```js
 * $scope.options = {
 *   loop: false,
 *   effect: 'fade',
 *   speed: 500,
 * }
 *
 * $scope.$on("$ionicSlides.sliderInitialized", function(event, data){
 *   // data.slider is the instance of Swiper
 *   $scope.slider = data.slider;
 * });
 *
 * $scope.$on("$ionicSlides.slideChangeStart", function(event, data){
 *   console.log('Slide change is beginning');
 * });
 *
 * $scope.$on("$ionicSlides.slideChangeEnd", function(event, data){
 *   // note: the indexes are 0-based
 *   $scope.activeIndex = data.slider.activeIndex;
 *   $scope.previousIndex = data.slider.previousIndex;
 * });
 *
 * ```
 *
 * ## Slide Events
 *
 * The slides component dispatches events when the active slide changes
 *
 * <table class="table">
 *   <tr>
 *     <td><code>$ionicSlides.slideChangeStart</code></td>
 *     <td>This event is emitted when a slide change begins</td>
 *   </tr>
 *   <tr>
 *     <td><code>$ionicSlides.slideChangeEnd</code></td>
 *     <td>This event is emitted when a slide change completes</td>
 *   </tr>
 *   <tr>
 *     <td><code>$ionicSlides.sliderInitialized</code></td>
 *     <td>This event is emitted when the slider is initialized. It provides access to an instance of the slider.</td>
 *   </tr>
 * </table>
 *
 *
 * ## Updating Slides Dynamically
 * When applying data to the slider at runtime, typically everything will work as expected.
 *
 * In the event that the slides are looped, use the `updateLoop` method on the slider to ensure the slides update correctly.
 *
 * ```
 * $scope.$on("$ionicSlides.sliderInitialized", function(event, data){
 *   // grab an instance of the slider
 *   $scope.slider = data.slider;
 * });
 *
 * function dataChangeHandler(){
 *   // call this function when data changes, such as an HTTP request, etc
 *   if ( $scope.slider ){
 *     $scope.slider.updateLoop();
 *   }
 * }
 * ```
 *
 */
IonicModule
.directive('ionSlides', [
  '$animate',
  '$timeout',
  '$compile',
function($animate, $timeout, $compile) {
  return {
    restrict: 'E',
    transclude: true,
    scope: {
      options: '=',
      slider: '='
    },
    template: '<div class="swiper-container">' +
      '<div class="swiper-wrapper" ng-transclude>' +
      '</div>' +
        '<div ng-hide="!showPager" class="swiper-pagination"></div>' +
      '</div>',
    controller: ['$scope', '$element', function($scope, $element) {
      var _this = this;

      this.update = function() {
        $timeout(function() {
          if (!_this.__slider) {
            return;
          }

          _this.__slider.update();
          if (_this._options.loop) {
            _this.__slider.createLoop();
          }

          var slidesLength = _this.__slider.slides.length;

          // Don't allow pager to show with > 10 slides
          if (slidesLength > 10) {
            $scope.showPager = false;
          }

          // When slide index is greater than total then slide to last index
          if (_this.__slider.activeIndex > slidesLength - 1) {
            _this.__slider.slideTo(slidesLength - 1);
          }
        });
      };

      this.rapidUpdate = ionic.debounce(function() {
        _this.update();
      }, 50);

      this.getSlider = function() {
        return _this.__slider;
      };

      var options = $scope.options || {};

      var newOptions = angular.extend({
        pagination: $element.children().children()[1],
        paginationClickable: true,
        lazyLoading: true,
        preloadImages: false
      }, options);

      this._options = newOptions;

      $timeout(function() {
        var slider = new ionic.views.Swiper($element.children()[0], newOptions, $scope, $compile);

        $scope.$emit("$ionicSlides.sliderInitialized", { slider: slider });

        _this.__slider = slider;
        $scope.slider = _this.__slider;

        $scope.$on('$destroy', function() {
          slider.destroy();
          _this.__slider = null;
        });
      });

      $timeout(function() {
        // if it's a loop, render the slides again just incase
        _this.rapidUpdate();
      }, 200);

    }],

    link: function($scope) {
      $scope.showPager = true;
      // Disable ngAnimate for slidebox and its children
      //$animate.enabled(false, $element);
    }
  };
}])
.directive('ionSlidePage', [function() {
  return {
    restrict: 'E',
    require: '?^ionSlides',
    transclude: true,
    replace: true,
    template: '<div class="swiper-slide" ng-transclude></div>',
    link: function($scope, $element, $attr, ionSlidesCtrl) {
      ionSlidesCtrl.rapidUpdate();

      $scope.$on('$destroy', function() {
        ionSlidesCtrl.rapidUpdate();
      });
    }
  };
}]);

/**
* @ngdoc directive
* @name ionSpinner
* @module ionic
* @restrict E
 *
 * @description
 * The `ionSpinner` directive provides a variety of animated spinners.
 * Spinners enables you to give your users feedback that the app is
 * processing/thinking/waiting/chillin' out, or whatever you'd like it to indicate.
 * By default, the {@link ionic.directive:ionRefresher} feature uses this spinner, rather
 * than rotating font icons (previously included in [ionicons](http://ionicons.com/)).
 * While font icons are great for simple or stationary graphics, they're not suited to
 * provide great animations, which is why Ionic uses SVG instead.
 *
 * Ionic offers ten spinners out of the box, and by default, it will use the appropriate spinner
 * for the platform on which it's running. Under the hood, the `ionSpinner` directive dynamically
 * builds the required SVG element, which allows Ionic to provide all ten of the animated SVGs
 * within 3KB.
 *
 * <style>
 * .spinner-table {
 *   max-width: 280px;
 * }
 * .spinner-table tbody > tr > th, .spinner-table tbody > tr > td {
 *   vertical-align: middle;
 *   width: 42px;
 *   height: 42px;
 * }
 * .spinner {
 *   stroke: #444;
 *   fill: #444; }
 *   .spinner svg {
 *     width: 28px;
 *     height: 28px; }
 *   .spinner.spinner-inverse {
 *     stroke: #fff;
 *     fill: #fff; }
 *
 * .spinner-android {
 *   stroke: #4b8bf4; }
 *
 * .spinner-ios, .spinner-ios-small {
 *   stroke: #69717d; }
 *
 * .spinner-spiral .stop1 {
 *   stop-color: #fff;
 *   stop-opacity: 0; }
 * .spinner-spiral.spinner-inverse .stop1 {
 *   stop-color: #000; }
 * .spinner-spiral.spinner-inverse .stop2 {
 *   stop-color: #fff; }
 * </style>
 *
 * <script src="http://code.ionicframework.com/nightly/js/ionic.bundle.min.js"></script>
 * <table class="table spinner-table" ng-app="ionic">
 *  <tr>
 *    <th>
 *      <code>android</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="android"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>ios</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="ios"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>ios-small</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="ios-small"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>bubbles</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="bubbles"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>circles</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="circles"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>crescent</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="crescent"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>dots</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="dots"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>lines</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="lines"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>ripple</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="ripple"></ion-spinner>
 *    </td>
 *  </tr>
 *  <tr>
 *    <th>
 *      <code>spiral</code>
 *    </th>
 *    <td>
 *      <ion-spinner icon="spiral"></ion-spinner>
 *    </td>
 *  </tr>
 * </table>
 *
 * Each spinner uses SVG with SMIL animations, however, the Android spinner also uses JavaScript
 * so it also works on Android 4.0-4.3. Additionally, each spinner can be styled with CSS,
 * and scaled to any size.
 *
 *
 * @usage
 * The following code would use the default spinner for the platform it's running from. If it's neither
 * iOS or Android, it'll default to use `ios`.
 *
 * ```html
 * <ion-spinner></ion-spinner>
 * ```
 *
 * By setting the `icon` attribute, you can specify which spinner to use, no matter what
 * the platform is.
 *
 * ```html
 * <ion-spinner icon="spiral"></ion-spinner>
 * ```
 *
 * ## Spinner Colors
 * Like with most of Ionic's other components, spinners can also be styled using
 * Ionic's standard color naming convention. For example:
 *
 * ```html
 * <ion-spinner class="spinner-energized"></ion-spinner>
 * ```
 *
 *
 * ## Styling SVG with CSS
 * One cool thing about SVG is its ability to be styled with CSS! Some of the properties
 * have different names, for example, SVG uses the term `stroke` instead of `border`, and
 * `fill` instead of `background-color`.
 *
 * ```css
 * .spinner svg {
 *   width: 28px;
 *   height: 28px;
 *   stroke: #444;
 *   fill: #444;
 * }
 * ```
 *
*/
IonicModule
.directive('ionSpinner', function() {
  return {
    restrict: 'E',
    controller: '$ionicSpinner',
    link: function($scope, $element, $attrs, ctrl) {
      var spinnerName = ctrl.init();
      $element.addClass('spinner spinner-' + spinnerName);

      $element.on('$destroy', function onDestroy() {
        ctrl.stop();
      });
    }
  };
});

/**
 * @ngdoc directive
 * @name ionTab
 * @module ionic
 * @restrict E
 * @parent ionic.directive:ionTabs
 *
 * @description
 * Contains a tab's content.  The content only exists while the given tab is selected.
 *
 * Each ionTab has its own view history.
 *
 * @usage
 * ```html
 * <ion-tab
 *   title="Tab!"
 *   icon="my-icon"
 *   href="#/tab/tab-link"
 *   on-select="onTabSelected()"
 *   on-deselect="onTabDeselected()">
 * </ion-tab>
 * ```
 * For a complete, working tab bar example, see the {@link ionic.directive:ionTabs} documentation.
 *
 * @param {string} title The title of the tab.
 * @param {string=} href The link that this tab will navigate to when tapped.
 * @param {string=} icon The icon of the tab. If given, this will become the default for icon-on and icon-off.
 * @param {string=} icon-on The icon of the tab while it is selected.
 * @param {string=} icon-off The icon of the tab while it is not selected.
 * @param {expression=} badge The badge to put on this tab (usually a number).
 * @param {expression=} badge-style The style of badge to put on this tab (eg: badge-positive).
 * @param {expression=} on-select Called when this tab is selected.
 * @param {expression=} on-deselect Called when this tab is deselected.
 * @param {expression=} ng-click By default, the tab will be selected on click. If ngClick is set, it will not.  You can explicitly switch tabs using {@link ionic.service:$ionicTabsDelegate#select $ionicTabsDelegate.select()}.
 * @param {expression=} hidden Whether the tab is to be hidden or not.
 * @param {expression=} disabled Whether the tab is to be disabled or not.
 */
IonicModule
.directive('ionTab', [
  '$compile',
  '$ionicConfig',
  '$ionicBind',
  '$ionicViewSwitcher',
function($compile, $ionicConfig, $ionicBind, $ionicViewSwitcher) {

  //Returns ' key="value"' if value exists
  function attrStr(k, v) {
    return isDefined(v) ? ' ' + k + '="' + v + '"' : '';
  }
  return {
    restrict: 'E',
    require: ['^ionTabs', 'ionTab'],
    controller: '$ionicTab',
    scope: true,
    compile: function(element, attr) {

      //We create the tabNavTemplate in the compile phase so that the
      //attributes we pass down won't be interpolated yet - we want
      //to pass down the 'raw' versions of the attributes
      var tabNavTemplate = '<ion-tab-nav' +
        attrStr('ng-click', attr.ngClick) +
        attrStr('title', attr.title) +
        attrStr('icon', attr.icon) +
        attrStr('icon-on', attr.iconOn) +
        attrStr('icon-off', attr.iconOff) +
        attrStr('badge', attr.badge) +
        attrStr('badge-style', attr.badgeStyle) +
        attrStr('hidden', attr.hidden) +
        attrStr('disabled', attr.disabled) +
        attrStr('class', attr['class']) +
        '></ion-tab-nav>';

      //Remove the contents of the element so we can compile them later, if tab is selected
      var tabContentEle = document.createElement('div');
      for (var x = 0; x < element[0].children.length; x++) {
        tabContentEle.appendChild(element[0].children[x].cloneNode(true));
      }
      var childElementCount = tabContentEle.childElementCount;
      element.empty();

      var navViewName, isNavView;
      if (childElementCount) {
        if (tabContentEle.children[0].tagName === 'ION-NAV-VIEW') {
          // get the name if it's a nav-view
          navViewName = tabContentEle.children[0].getAttribute('name');
          tabContentEle.children[0].classList.add('view-container');
          isNavView = true;
        }
        if (childElementCount === 1) {
          // make the 1 child element the primary tab content container
          tabContentEle = tabContentEle.children[0];
        }
        if (!isNavView) tabContentEle.classList.add('pane');
        tabContentEle.classList.add('tab-content');
      }

      return function link($scope, $element, $attr, ctrls) {
        var childScope;
        var childElement;
        var tabsCtrl = ctrls[0];
        var tabCtrl = ctrls[1];
        var isTabContentAttached = false;
        $scope.$tabSelected = false;

        $ionicBind($scope, $attr, {
          onSelect: '&',
          onDeselect: '&',
          title: '@',
          uiSref: '@',
          href: '@'
        });

        tabsCtrl.add($scope);
        $scope.$on('$destroy', function() {
          if (!$scope.$tabsDestroy) {
            // if the containing ionTabs directive is being destroyed
            // then don't bother going through the controllers remove
            // method, since remove will reset the active tab as each tab
            // is being destroyed, causing unnecessary view loads and transitions
            tabsCtrl.remove($scope);
          }
          tabNavElement.isolateScope().$destroy();
          tabNavElement.remove();
          tabNavElement = tabContentEle = childElement = null;
        });

        //Remove title attribute so browser-tooltip does not apear
        $element[0].removeAttribute('title');

        if (navViewName) {
          tabCtrl.navViewName = $scope.navViewName = navViewName;
        }
        $scope.$on('$stateChangeSuccess', selectIfMatchesState);
        selectIfMatchesState();
        function selectIfMatchesState() {
          if (tabCtrl.tabMatchesState()) {
            tabsCtrl.select($scope, false);
          }
        }

        var tabNavElement = jqLite(tabNavTemplate);
        tabNavElement.data('$ionTabsController', tabsCtrl);
        tabNavElement.data('$ionTabController', tabCtrl);
        tabsCtrl.$tabsElement.append($compile(tabNavElement)($scope));


        function tabSelected(isSelected) {
          if (isSelected && childElementCount) {
            // this tab is being selected

            // check if the tab is already in the DOM
            // only do this if the tab has child elements
            if (!isTabContentAttached) {
              // tab should be selected and is NOT in the DOM
              // create a new scope and append it
              childScope = $scope.$new();
              childElement = jqLite(tabContentEle);
              $ionicViewSwitcher.viewEleIsActive(childElement, true);
              tabsCtrl.$element.append(childElement);
              $compile(childElement)(childScope);
              isTabContentAttached = true;
            }

            // remove the hide class so the tabs content shows up
            $ionicViewSwitcher.viewEleIsActive(childElement, true);

          } else if (isTabContentAttached && childElement) {
            // this tab should NOT be selected, and it is already in the DOM

            if ($ionicConfig.views.maxCache() > 0) {
              // keep the tabs in the DOM, only css hide it
              $ionicViewSwitcher.viewEleIsActive(childElement, false);

            } else {
              // do not keep tabs in the DOM
              destroyTab();
            }

          }
        }

        function destroyTab() {
          childScope && childScope.$destroy();
          isTabContentAttached && childElement && childElement.remove();
          tabContentEle.innerHTML = '';
          isTabContentAttached = childScope = childElement = null;
        }

        $scope.$watch('$tabSelected', tabSelected);

        $scope.$on('$ionicView.afterEnter', function() {
          $ionicViewSwitcher.viewEleIsActive(childElement, $scope.$tabSelected);
        });

        $scope.$on('$ionicView.clearCache', function() {
          if (!$scope.$tabSelected) {
            destroyTab();
          }
        });

      };
    }
  };
}]);

IonicModule
.directive('ionTabNav', [function() {
  return {
    restrict: 'E',
    replace: true,
    require: ['^ionTabs', '^ionTab'],
    template:
    '<a ng-class="{\'has-badge\':badge, \'tab-hidden\':isHidden(), \'tab-item-active\': isTabActive()}" ' +
      ' ng-disabled="disabled()" class="tab-item">' +
      '<span class="badge {{badgeStyle}}" ng-if="badge">{{badge}}</span>' +
      '<i class="icon {{getIcon()}}" ng-if="getIcon()"></i>' +
      '<span class="tab-title" ng-bind-html="title"></span>' +
    '</a>',
    scope: {
      title: '@',
      icon: '@',
      iconOn: '@',
      iconOff: '@',
      badge: '=',
      hidden: '@',
      disabled: '&',
      badgeStyle: '@',
      'class': '@'
    },
    link: function($scope, $element, $attrs, ctrls) {
      var tabsCtrl = ctrls[0],
        tabCtrl = ctrls[1];

      //Remove title attribute so browser-tooltip does not apear
      $element[0].removeAttribute('title');

      $scope.selectTab = function(e) {
        e.preventDefault();
        tabsCtrl.select(tabCtrl.$scope, true);
      };
      if (!$attrs.ngClick) {
        $element.on('click', function(event) {
          $scope.$apply(function() {
            $scope.selectTab(event);
          });
        });
      }

      $scope.isHidden = function() {
        if ($attrs.hidden === 'true' || $attrs.hidden === true) return true;
        return false;
      };

      $scope.getIconOn = function() {
        return $scope.iconOn || $scope.icon;
      };
      $scope.getIconOff = function() {
        return $scope.iconOff || $scope.icon;
      };

      $scope.isTabActive = function() {
        return tabsCtrl.selectedTab() === tabCtrl.$scope;
      };

      $scope.getIcon = function() {
        if ( tabsCtrl.selectedTab() === tabCtrl.$scope ) {
          // active
          return $scope.iconOn || $scope.icon;
        }
        else {
          // inactive
          return $scope.iconOff || $scope.icon;
        }
      };
    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionTabs
 * @module ionic
 * @delegate ionic.service:$ionicTabsDelegate
 * @restrict E
 * @codepen odqCz
 *
 * @description
 * Powers a multi-tabbed interface with a Tab Bar and a set of "pages" that can be tabbed
 * through.
 *
 * Assign any [tabs class](/docs/components#tabs) to the element to define
 * its look and feel.
 *
 * For iOS, tabs will appear at the bottom of the screen. For Android, tabs will be at the top
 * of the screen, below the nav-bar. This follows each OS's design specification, but can be
 * configured with the {@link ionic.provider:$ionicConfigProvider}.
 *
 * See the {@link ionic.directive:ionTab} directive's documentation for more details on
 * individual tabs.
 *
 * Note: do not place ion-tabs inside of an ion-content element; it has been known to cause a
 * certain CSS bug.
 *
 * @usage
 * ```html
 * <ion-tabs class="tabs-positive tabs-icon-top">
 *
 *   <ion-tab title="Home" icon-on="ion-ios-filing" icon-off="ion-ios-filing-outline">
 *     <!-- Tab 1 content -->
 *   </ion-tab>
 *
 *   <ion-tab title="About" icon-on="ion-ios-clock" icon-off="ion-ios-clock-outline">
 *     <!-- Tab 2 content -->
 *   </ion-tab>
 *
 *   <ion-tab title="Settings" icon-on="ion-ios-gear" icon-off="ion-ios-gear-outline">
 *     <!-- Tab 3 content -->
 *   </ion-tab>
 *
 * </ion-tabs>
 * ```
 *
 * @param {string=} delegate-handle The handle used to identify these tabs
 * with {@link ionic.service:$ionicTabsDelegate}.
 */

IonicModule
.directive('ionTabs', [
  '$ionicTabsDelegate',
  '$ionicConfig',
function($ionicTabsDelegate, $ionicConfig) {
  return {
    restrict: 'E',
    scope: true,
    controller: '$ionicTabs',
    compile: function(tElement) {
      //We cannot use regular transclude here because it breaks element.data()
      //inheritance on compile
      var innerElement = jqLite('<div class="tab-nav tabs">');
      innerElement.append(tElement.contents());

      tElement.append(innerElement)
              .addClass('tabs-' + $ionicConfig.tabs.position() + ' tabs-' + $ionicConfig.tabs.style());

      return { pre: prelink, post: postLink };
      function prelink($scope, $element, $attr, tabsCtrl) {
        var deregisterInstance = $ionicTabsDelegate._registerInstance(
          tabsCtrl, $attr.delegateHandle, tabsCtrl.hasActiveScope
        );

        tabsCtrl.$scope = $scope;
        tabsCtrl.$element = $element;
        tabsCtrl.$tabsElement = jqLite($element[0].querySelector('.tabs'));

        $scope.$watch(function() { return $element[0].className; }, function(value) {
          var isTabsTop = value.indexOf('tabs-top') !== -1;
          var isHidden = value.indexOf('tabs-item-hide') !== -1;
          $scope.$hasTabs = !isTabsTop && !isHidden;
          $scope.$hasTabsTop = isTabsTop && !isHidden;
          $scope.$emit('$ionicTabs.top', $scope.$hasTabsTop);
        });

        function emitLifecycleEvent(ev, data) {
          ev.stopPropagation();
          var previousSelectedTab = tabsCtrl.previousSelectedTab();
          if (previousSelectedTab) {
            previousSelectedTab.$broadcast(ev.name.replace('NavView', 'Tabs'), data);
          }
        }

        $scope.$on('$ionicNavView.beforeLeave', emitLifecycleEvent);
        $scope.$on('$ionicNavView.afterLeave', emitLifecycleEvent);
        $scope.$on('$ionicNavView.leave', emitLifecycleEvent);

        $scope.$on('$destroy', function() {
          // variable to inform child tabs that they're all being blown away
          // used so that while destorying an individual tab, each one
          // doesn't select the next tab as the active one, which causes unnecessary
          // loading of tab views when each will eventually all go away anyway
          $scope.$tabsDestroy = true;
          deregisterInstance();
          tabsCtrl.$tabsElement = tabsCtrl.$element = tabsCtrl.$scope = innerElement = null;
          delete $scope.$hasTabs;
          delete $scope.$hasTabsTop;
        });
      }

      function postLink($scope, $element, $attr, tabsCtrl) {
        if (!tabsCtrl.selectedTab()) {
          // all the tabs have been added
          // but one hasn't been selected yet
          tabsCtrl.select(0);
        }
      }
    }
  };
}]);

/**
* @ngdoc directive
* @name ionTitle
* @module ionic
* @restrict E
*
* Used for titles in header and nav bars. New in 1.2
*
* Identical to <div class="title"> but with future compatibility for Ionic 2
*
* @usage
*
* ```html
* <ion-nav-bar>
*   <ion-title>Hello</ion-title>
* <ion-nav-bar>
* ```
*/
IonicModule
.directive('ionTitle', [function() {
  return {
    restrict: 'E',
    compile: function(element) {
      element.addClass('title');
    }
  };
}]);

/**
 * @ngdoc directive
 * @name ionToggle
 * @module ionic
 * @codepen tfAzj
 * @restrict E
 *
 * @description
 * A toggle is an animated switch which binds a given model to a boolean.
 *
 * Allows dragging of the switch's nub.
 *
 * The toggle behaves like any [AngularJS checkbox](http://docs.angularjs.org/api/ng/input/input[checkbox]) otherwise.
 *
 * @param toggle-class {string=} Sets the CSS class on the inner `label.toggle` element created by the directive.
 *
 * @usage
 * Below is an example of a toggle directive which is wired up to the `airplaneMode` model
 * and has the `toggle-calm` CSS class assigned to the inner element.
 *
 * ```html
 * <ion-toggle ng-model="airplaneMode" toggle-class="toggle-calm">Airplane Mode</ion-toggle>
 * ```
 */
IonicModule
.directive('ionToggle', [
  '$timeout',
  '$ionicConfig',
function($timeout, $ionicConfig) {

  return {
    restrict: 'E',
    replace: true,
    require: '?ngModel',
    transclude: true,
    template:
      '<div class="item item-toggle">' +
        '<div ng-transclude></div>' +
        '<label class="toggle">' +
          '<input type="checkbox">' +
          '<div class="track">' +
            '<div class="handle"></div>' +
          '</div>' +
        '</label>' +
      '</div>',

    compile: function(element, attr) {
      var input = element.find('input');
      forEach({
        'name': attr.name,
        'ng-value': attr.ngValue,
        'ng-model': attr.ngModel,
        'ng-checked': attr.ngChecked,
        'ng-disabled': attr.ngDisabled,
        'ng-true-value': attr.ngTrueValue,
        'ng-false-value': attr.ngFalseValue,
        'ng-change': attr.ngChange,
        'ng-required': attr.ngRequired,
        'required': attr.required
      }, function(value, name) {
        if (isDefined(value)) {
          input.attr(name, value);
        }
      });

      if (attr.toggleClass) {
        element[0].getElementsByTagName('label')[0].classList.add(attr.toggleClass);
      }

      element.addClass('toggle-' + $ionicConfig.form.toggle());

      return function($scope, $element) {
        var el = $element[0].getElementsByTagName('label')[0];
        var checkbox = el.children[0];
        var track = el.children[1];
        var handle = track.children[0];

        var ngModelController = jqLite(checkbox).controller('ngModel');

        $scope.toggle = new ionic.views.Toggle({
          el: el,
          track: track,
          checkbox: checkbox,
          handle: handle,
          onChange: function() {
            if (ngModelController) {
              ngModelController.$setViewValue(checkbox.checked);
              $scope.$apply();
            }
          }
        });

        $scope.$on('$destroy', function() {
          $scope.toggle.destroy();
        });
      };
    }

  };
}]);

/**
 * @ngdoc directive
 * @name ionView
 * @module ionic
 * @restrict E
 * @parent ionNavView
 *
 * @description
 * A container for view content and any navigational and header bar information. When a view
 * enters and exits its parent {@link ionic.directive:ionNavView}, the view also emits view
 * information, such as its title, whether the back button should be displayed or not, whether the
 * corresponding {@link ionic.directive:ionNavBar} should be displayed or not, which transition the view
 * should use to animate, and which direction to animate.
 *
 * *Views are cached to improve performance.* When a view is navigated away from, its element is
 * left in the DOM, and its scope is disconnected from the `$watch` cycle. When navigating to a
 * view that is already cached, its scope is reconnected, and the existing element, which was
 * left in the DOM, becomes active again. This can be disabled, or the maximum number of cached
 * views changed in {@link ionic.provider:$ionicConfigProvider}, in the view's `$state` configuration, or
 * as an attribute on the view itself (see below).
 *
 * @usage
 * Below is an example where our page will load with a {@link ionic.directive:ionNavBar} containing
 * "My Page" as the title.
 *
 * ```html
 * <ion-nav-bar></ion-nav-bar>
 * <ion-nav-view>
 *   <ion-view view-title="My Page">
 *     <ion-content>
 *       Hello!
 *     </ion-content>
 *   </ion-view>
 * </ion-nav-view>
 * ```
 *
 * ## View LifeCycle and Events
 *
 * Views can be cached, which means ***controllers normally only load once***, which may
 * affect your controller logic. To know when a view has entered or left, events
 * have been added that are emitted from the view's scope. These events also
 * contain data about the view, such as the title and whether the back button should
 * show. Also contained is transition data, such as the transition type and
 * direction that will be or was used.
 *
 * Life cycle events are emitted upwards from the transitioning view's scope. In some cases, it is
 * desirable for a child/nested view to be notified of the event.
 * For this use case, `$ionicParentView` life cycle events are broadcast downwards.
 *
 * <table class="table">
 *  <tr>
 *   <td><code>$ionicView.loaded</code></td>
 *   <td>The view has loaded. This event only happens once per
 * view being created and added to the DOM. If a view leaves but is cached,
 * then this event will not fire again on a subsequent viewing. The loaded event
 * is good place to put your setup code for the view; however, it is not the
 * recommended event to listen to when a view becomes active.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.enter</code></td>
 *   <td>The view has fully entered and is now the active view.
 * This event will fire, whether it was the first load or a cached view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.leave</code></td>
 *   <td>The view has finished leaving and is no longer the
 * active view. This event will fire, whether it is cached or destroyed.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.beforeEnter</code></td>
 *   <td>The view is about to enter and become the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.beforeLeave</code></td>
 *   <td>The view is about to leave and no longer be the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.afterEnter</code></td>
 *   <td>The view has fully entered and is now the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.afterLeave</code></td>
 *   <td>The view has finished leaving and is no longer the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicView.unloaded</code></td>
 *   <td>The view's controller has been destroyed and its element has been
 * removed from the DOM.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.enter</code></td>
 *   <td>The parent view has fully entered and is now the active view.
 * This event will fire, whether it was the first load or a cached view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.leave</code></td>
 *   <td>The parent view has finished leaving and is no longer the
 * active view. This event will fire, whether it is cached or destroyed.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.beforeEnter</code></td>
 *   <td>The parent view is about to enter and become the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.beforeLeave</code></td>
 *   <td>The parent view is about to leave and no longer be the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.afterEnter</code></td>
 *   <td>The parent view has fully entered and is now the active view.</td>
 *  </tr>
 *  <tr>
 *   <td><code>$ionicParentView.afterLeave</code></td>
 *   <td>The parent view has finished leaving and is no longer the active view.</td>
 *  </tr>
 * </table>
 *
 * ## LifeCycle Event Usage
 *
 * Below is an example of how to listen to life cycle events and
 * access state parameter data
 *
 * ```js
 * $scope.$on("$ionicView.beforeEnter", function(event, data){
 *    // handle event
 *    console.log("State Params: ", data.stateParams);
 * });
 *
 * $scope.$on("$ionicView.enter", function(event, data){
 *    // handle event
 *    console.log("State Params: ", data.stateParams);
 * });
 *
 * $scope.$on("$ionicView.afterEnter", function(event, data){
 *    // handle event
 *    console.log("State Params: ", data.stateParams);
 * });
 * ```
 *
 * ## Caching
 *
 * Caching can be disabled and enabled in multiple ways. By default, Ionic will
 * cache a maximum of 10 views. You can optionally choose to disable caching at
 * either an individual view basis, or by global configuration. Please see the
 * _Caching_ section in {@link ionic.directive:ionNavView} for more info.
 *
 * @param {string=} view-title A text-only title to display on the parent {@link ionic.directive:ionNavBar}.
 * For an HTML title, such as an image, see {@link ionic.directive:ionNavTitle} instead.
 * @param {boolean=} cache-view If this view should be allowed to be cached or not.
 * Please see the _Caching_ section in {@link ionic.directive:ionNavView} for
 * more info. Default `true`
 * @param {boolean=} can-swipe-back If this view should be allowed to use the swipe to go back gesture or not.
 * This does not enable the swipe to go back feature if it is not available for the platform it's running
 * from, or there isn't a previous view. Default `true`
 * @param {boolean=} hide-back-button Whether to hide the back button on the parent
 * {@link ionic.directive:ionNavBar} by default.
 * @param {boolean=} hide-nav-bar Whether to hide the parent
 * {@link ionic.directive:ionNavBar} by default.
 */
IonicModule
.directive('ionView', function() {
  return {
    restrict: 'EA',
    priority: 1000,
    controller: '$ionicView',
    compile: function(tElement) {
      tElement.addClass('pane');
      tElement[0].removeAttribute('title');
      return function link($scope, $element, $attrs, viewCtrl) {
        viewCtrl.init();
      };
    }
  };
});

})();
/*! jQuery v3.5.1 | (c) JS Foundation and other contributors | jquery.org/license */
!function(e,t){"use strict";"object"==typeof module&&"object"==typeof module.exports?module.exports=e.document?t(e,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return t(e)}:t(e)}("undefined"!=typeof window?window:this,function(C,e){"use strict";var t=[],r=Object.getPrototypeOf,s=t.slice,g=t.flat?function(e){return t.flat.call(e)}:function(e){return t.concat.apply([],e)},u=t.push,i=t.indexOf,n={},o=n.toString,v=n.hasOwnProperty,a=v.toString,l=a.call(Object),y={},m=function(e){return"function"==typeof e&&"number"!=typeof e.nodeType},x=function(e){return null!=e&&e===e.window},E=C.document,c={type:!0,src:!0,nonce:!0,noModule:!0};function b(e,t,n){var r,i,o=(n=n||E).createElement("script");if(o.text=e,t)for(r in c)(i=t[r]||t.getAttribute&&t.getAttribute(r))&&o.setAttribute(r,i);n.head.appendChild(o).parentNode.removeChild(o)}function w(e){return null==e?e+"":"object"==typeof e||"function"==typeof e?n[o.call(e)]||"object":typeof e}var f="3.5.1",S=function(e,t){return new S.fn.init(e,t)};function p(e){var t=!!e&&"length"in e&&e.length,n=w(e);return!m(e)&&!x(e)&&("array"===n||0===t||"number"==typeof t&&0<t&&t-1 in e)}S.fn=S.prototype={jquery:f,constructor:S,length:0,toArray:function(){return s.call(this)},get:function(e){return null==e?s.call(this):e<0?this[e+this.length]:this[e]},pushStack:function(e){var t=S.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return S.each(this,e)},map:function(n){return this.pushStack(S.map(this,function(e,t){return n.call(e,t,e)}))},slice:function(){return this.pushStack(s.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},even:function(){return this.pushStack(S.grep(this,function(e,t){return(t+1)%2}))},odd:function(){return this.pushStack(S.grep(this,function(e,t){return t%2}))},eq:function(e){var t=this.length,n=+e+(e<0?t:0);return this.pushStack(0<=n&&n<t?[this[n]]:[])},end:function(){return this.prevObject||this.constructor()},push:u,sort:t.sort,splice:t.splice},S.extend=S.fn.extend=function(){var e,t,n,r,i,o,a=arguments[0]||{},s=1,u=arguments.length,l=!1;for("boolean"==typeof a&&(l=a,a=arguments[s]||{},s++),"object"==typeof a||m(a)||(a={}),s===u&&(a=this,s--);s<u;s++)if(null!=(e=arguments[s]))for(t in e)r=e[t],"__proto__"!==t&&a!==r&&(l&&r&&(S.isPlainObject(r)||(i=Array.isArray(r)))?(n=a[t],o=i&&!Array.isArray(n)?[]:i||S.isPlainObject(n)?n:{},i=!1,a[t]=S.extend(l,o,r)):void 0!==r&&(a[t]=r));return a},S.extend({expando:"jQuery"+(f+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isPlainObject:function(e){var t,n;return!(!e||"[object Object]"!==o.call(e))&&(!(t=r(e))||"function"==typeof(n=v.call(t,"constructor")&&t.constructor)&&a.call(n)===l)},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},globalEval:function(e,t,n){b(e,{nonce:t&&t.nonce},n)},each:function(e,t){var n,r=0;if(p(e)){for(n=e.length;r<n;r++)if(!1===t.call(e[r],r,e[r]))break}else for(r in e)if(!1===t.call(e[r],r,e[r]))break;return e},makeArray:function(e,t){var n=t||[];return null!=e&&(p(Object(e))?S.merge(n,"string"==typeof e?[e]:e):u.call(n,e)),n},inArray:function(e,t,n){return null==t?-1:i.call(t,e,n)},merge:function(e,t){for(var n=+t.length,r=0,i=e.length;r<n;r++)e[i++]=t[r];return e.length=i,e},grep:function(e,t,n){for(var r=[],i=0,o=e.length,a=!n;i<o;i++)!t(e[i],i)!==a&&r.push(e[i]);return r},map:function(e,t,n){var r,i,o=0,a=[];if(p(e))for(r=e.length;o<r;o++)null!=(i=t(e[o],o,n))&&a.push(i);else for(o in e)null!=(i=t(e[o],o,n))&&a.push(i);return g(a)},guid:1,support:y}),"function"==typeof Symbol&&(S.fn[Symbol.iterator]=t[Symbol.iterator]),S.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(e,t){n["[object "+t+"]"]=t.toLowerCase()});var d=function(n){var e,d,b,o,i,h,f,g,w,u,l,T,C,a,E,v,s,c,y,S="sizzle"+1*new Date,p=n.document,k=0,r=0,m=ue(),x=ue(),A=ue(),N=ue(),D=function(e,t){return e===t&&(l=!0),0},j={}.hasOwnProperty,t=[],q=t.pop,L=t.push,H=t.push,O=t.slice,P=function(e,t){for(var n=0,r=e.length;n<r;n++)if(e[n]===t)return n;return-1},R="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",M="[\\x20\\t\\r\\n\\f]",I="(?:\\\\[\\da-fA-F]{1,6}"+M+"?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+",W="\\["+M+"*("+I+")(?:"+M+"*([*^$|!~]?=)"+M+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+I+"))|)"+M+"*\\]",F=":("+I+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+W+")*)|.*)\\)|)",B=new RegExp(M+"+","g"),$=new RegExp("^"+M+"+|((?:^|[^\\\\])(?:\\\\.)*)"+M+"+$","g"),_=new RegExp("^"+M+"*,"+M+"*"),z=new RegExp("^"+M+"*([>+~]|"+M+")"+M+"*"),U=new RegExp(M+"|>"),X=new RegExp(F),V=new RegExp("^"+I+"$"),G={ID:new RegExp("^#("+I+")"),CLASS:new RegExp("^\\.("+I+")"),TAG:new RegExp("^("+I+"|[*])"),ATTR:new RegExp("^"+W),PSEUDO:new RegExp("^"+F),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+M+"*(even|odd|(([+-]|)(\\d*)n|)"+M+"*(?:([+-]|)"+M+"*(\\d+)|))"+M+"*\\)|)","i"),bool:new RegExp("^(?:"+R+")$","i"),needsContext:new RegExp("^"+M+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+M+"*((?:-\\d)?\\d*)"+M+"*\\)|)(?=[^-]|$)","i")},Y=/HTML$/i,Q=/^(?:input|select|textarea|button)$/i,J=/^h\d$/i,K=/^[^{]+\{\s*\[native \w/,Z=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,ee=/[+~]/,te=new RegExp("\\\\[\\da-fA-F]{1,6}"+M+"?|\\\\([^\\r\\n\\f])","g"),ne=function(e,t){var n="0x"+e.slice(1)-65536;return t||(n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,1023&n|56320))},re=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,ie=function(e,t){return t?"\0"===e?"\ufffd":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},oe=function(){T()},ae=be(function(e){return!0===e.disabled&&"fieldset"===e.nodeName.toLowerCase()},{dir:"parentNode",next:"legend"});try{H.apply(t=O.call(p.childNodes),p.childNodes),t[p.childNodes.length].nodeType}catch(e){H={apply:t.length?function(e,t){L.apply(e,O.call(t))}:function(e,t){var n=e.length,r=0;while(e[n++]=t[r++]);e.length=n-1}}}function se(t,e,n,r){var i,o,a,s,u,l,c,f=e&&e.ownerDocument,p=e?e.nodeType:9;if(n=n||[],"string"!=typeof t||!t||1!==p&&9!==p&&11!==p)return n;if(!r&&(T(e),e=e||C,E)){if(11!==p&&(u=Z.exec(t)))if(i=u[1]){if(9===p){if(!(a=e.getElementById(i)))return n;if(a.id===i)return n.push(a),n}else if(f&&(a=f.getElementById(i))&&y(e,a)&&a.id===i)return n.push(a),n}else{if(u[2])return H.apply(n,e.getElementsByTagName(t)),n;if((i=u[3])&&d.getElementsByClassName&&e.getElementsByClassName)return H.apply(n,e.getElementsByClassName(i)),n}if(d.qsa&&!N[t+" "]&&(!v||!v.test(t))&&(1!==p||"object"!==e.nodeName.toLowerCase())){if(c=t,f=e,1===p&&(U.test(t)||z.test(t))){(f=ee.test(t)&&ye(e.parentNode)||e)===e&&d.scope||((s=e.getAttribute("id"))?s=s.replace(re,ie):e.setAttribute("id",s=S)),o=(l=h(t)).length;while(o--)l[o]=(s?"#"+s:":scope")+" "+xe(l[o]);c=l.join(",")}try{return H.apply(n,f.querySelectorAll(c)),n}catch(e){N(t,!0)}finally{s===S&&e.removeAttribute("id")}}}return g(t.replace($,"$1"),e,n,r)}function ue(){var r=[];return function e(t,n){return r.push(t+" ")>b.cacheLength&&delete e[r.shift()],e[t+" "]=n}}function le(e){return e[S]=!0,e}function ce(e){var t=C.createElement("fieldset");try{return!!e(t)}catch(e){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function fe(e,t){var n=e.split("|"),r=n.length;while(r--)b.attrHandle[n[r]]=t}function pe(e,t){var n=t&&e,r=n&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(r)return r;if(n)while(n=n.nextSibling)if(n===t)return-1;return e?1:-1}function de(t){return function(e){return"input"===e.nodeName.toLowerCase()&&e.type===t}}function he(n){return function(e){var t=e.nodeName.toLowerCase();return("input"===t||"button"===t)&&e.type===n}}function ge(t){return function(e){return"form"in e?e.parentNode&&!1===e.disabled?"label"in e?"label"in e.parentNode?e.parentNode.disabled===t:e.disabled===t:e.isDisabled===t||e.isDisabled!==!t&&ae(e)===t:e.disabled===t:"label"in e&&e.disabled===t}}function ve(a){return le(function(o){return o=+o,le(function(e,t){var n,r=a([],e.length,o),i=r.length;while(i--)e[n=r[i]]&&(e[n]=!(t[n]=e[n]))})})}function ye(e){return e&&"undefined"!=typeof e.getElementsByTagName&&e}for(e in d=se.support={},i=se.isXML=function(e){var t=e.namespaceURI,n=(e.ownerDocument||e).documentElement;return!Y.test(t||n&&n.nodeName||"HTML")},T=se.setDocument=function(e){var t,n,r=e?e.ownerDocument||e:p;return r!=C&&9===r.nodeType&&r.documentElement&&(a=(C=r).documentElement,E=!i(C),p!=C&&(n=C.defaultView)&&n.top!==n&&(n.addEventListener?n.addEventListener("unload",oe,!1):n.attachEvent&&n.attachEvent("onunload",oe)),d.scope=ce(function(e){return a.appendChild(e).appendChild(C.createElement("div")),"undefined"!=typeof e.querySelectorAll&&!e.querySelectorAll(":scope fieldset div").length}),d.attributes=ce(function(e){return e.className="i",!e.getAttribute("className")}),d.getElementsByTagName=ce(function(e){return e.appendChild(C.createComment("")),!e.getElementsByTagName("*").length}),d.getElementsByClassName=K.test(C.getElementsByClassName),d.getById=ce(function(e){return a.appendChild(e).id=S,!C.getElementsByName||!C.getElementsByName(S).length}),d.getById?(b.filter.ID=function(e){var t=e.replace(te,ne);return function(e){return e.getAttribute("id")===t}},b.find.ID=function(e,t){if("undefined"!=typeof t.getElementById&&E){var n=t.getElementById(e);return n?[n]:[]}}):(b.filter.ID=function(e){var n=e.replace(te,ne);return function(e){var t="undefined"!=typeof e.getAttributeNode&&e.getAttributeNode("id");return t&&t.value===n}},b.find.ID=function(e,t){if("undefined"!=typeof t.getElementById&&E){var n,r,i,o=t.getElementById(e);if(o){if((n=o.getAttributeNode("id"))&&n.value===e)return[o];i=t.getElementsByName(e),r=0;while(o=i[r++])if((n=o.getAttributeNode("id"))&&n.value===e)return[o]}return[]}}),b.find.TAG=d.getElementsByTagName?function(e,t){return"undefined"!=typeof t.getElementsByTagName?t.getElementsByTagName(e):d.qsa?t.querySelectorAll(e):void 0}:function(e,t){var n,r=[],i=0,o=t.getElementsByTagName(e);if("*"===e){while(n=o[i++])1===n.nodeType&&r.push(n);return r}return o},b.find.CLASS=d.getElementsByClassName&&function(e,t){if("undefined"!=typeof t.getElementsByClassName&&E)return t.getElementsByClassName(e)},s=[],v=[],(d.qsa=K.test(C.querySelectorAll))&&(ce(function(e){var t;a.appendChild(e).innerHTML="<a id='"+S+"'></a><select id='"+S+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&v.push("[*^$]="+M+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||v.push("\\["+M+"*(?:value|"+R+")"),e.querySelectorAll("[id~="+S+"-]").length||v.push("~="),(t=C.createElement("input")).setAttribute("name",""),e.appendChild(t),e.querySelectorAll("[name='']").length||v.push("\\["+M+"*name"+M+"*="+M+"*(?:''|\"\")"),e.querySelectorAll(":checked").length||v.push(":checked"),e.querySelectorAll("a#"+S+"+*").length||v.push(".#.+[+~]"),e.querySelectorAll("\\\f"),v.push("[\\r\\n\\f]")}),ce(function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=C.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&v.push("name"+M+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&v.push(":enabled",":disabled"),a.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&v.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),v.push(",.*:")})),(d.matchesSelector=K.test(c=a.matches||a.webkitMatchesSelector||a.mozMatchesSelector||a.oMatchesSelector||a.msMatchesSelector))&&ce(function(e){d.disconnectedMatch=c.call(e,"*"),c.call(e,"[s!='']:x"),s.push("!=",F)}),v=v.length&&new RegExp(v.join("|")),s=s.length&&new RegExp(s.join("|")),t=K.test(a.compareDocumentPosition),y=t||K.test(a.contains)?function(e,t){var n=9===e.nodeType?e.documentElement:e,r=t&&t.parentNode;return e===r||!(!r||1!==r.nodeType||!(n.contains?n.contains(r):e.compareDocumentPosition&&16&e.compareDocumentPosition(r)))}:function(e,t){if(t)while(t=t.parentNode)if(t===e)return!0;return!1},D=t?function(e,t){if(e===t)return l=!0,0;var n=!e.compareDocumentPosition-!t.compareDocumentPosition;return n||(1&(n=(e.ownerDocument||e)==(t.ownerDocument||t)?e.compareDocumentPosition(t):1)||!d.sortDetached&&t.compareDocumentPosition(e)===n?e==C||e.ownerDocument==p&&y(p,e)?-1:t==C||t.ownerDocument==p&&y(p,t)?1:u?P(u,e)-P(u,t):0:4&n?-1:1)}:function(e,t){if(e===t)return l=!0,0;var n,r=0,i=e.parentNode,o=t.parentNode,a=[e],s=[t];if(!i||!o)return e==C?-1:t==C?1:i?-1:o?1:u?P(u,e)-P(u,t):0;if(i===o)return pe(e,t);n=e;while(n=n.parentNode)a.unshift(n);n=t;while(n=n.parentNode)s.unshift(n);while(a[r]===s[r])r++;return r?pe(a[r],s[r]):a[r]==p?-1:s[r]==p?1:0}),C},se.matches=function(e,t){return se(e,null,null,t)},se.matchesSelector=function(e,t){if(T(e),d.matchesSelector&&E&&!N[t+" "]&&(!s||!s.test(t))&&(!v||!v.test(t)))try{var n=c.call(e,t);if(n||d.disconnectedMatch||e.document&&11!==e.document.nodeType)return n}catch(e){N(t,!0)}return 0<se(t,C,null,[e]).length},se.contains=function(e,t){return(e.ownerDocument||e)!=C&&T(e),y(e,t)},se.attr=function(e,t){(e.ownerDocument||e)!=C&&T(e);var n=b.attrHandle[t.toLowerCase()],r=n&&j.call(b.attrHandle,t.toLowerCase())?n(e,t,!E):void 0;return void 0!==r?r:d.attributes||!E?e.getAttribute(t):(r=e.getAttributeNode(t))&&r.specified?r.value:null},se.escape=function(e){return(e+"").replace(re,ie)},se.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},se.uniqueSort=function(e){var t,n=[],r=0,i=0;if(l=!d.detectDuplicates,u=!d.sortStable&&e.slice(0),e.sort(D),l){while(t=e[i++])t===e[i]&&(r=n.push(i));while(r--)e.splice(n[r],1)}return u=null,e},o=se.getText=function(e){var t,n="",r=0,i=e.nodeType;if(i){if(1===i||9===i||11===i){if("string"==typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=o(e)}else if(3===i||4===i)return e.nodeValue}else while(t=e[r++])n+=o(t);return n},(b=se.selectors={cacheLength:50,createPseudo:le,match:G,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(te,ne),e[3]=(e[3]||e[4]||e[5]||"").replace(te,ne),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||se.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&se.error(e[0]),e},PSEUDO:function(e){var t,n=!e[6]&&e[2];return G.CHILD.test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":n&&X.test(n)&&(t=h(n,!0))&&(t=n.indexOf(")",n.length-t)-n.length)&&(e[0]=e[0].slice(0,t),e[2]=n.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(te,ne).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=m[e+" "];return t||(t=new RegExp("(^|"+M+")"+e+"("+M+"|$)"))&&m(e,function(e){return t.test("string"==typeof e.className&&e.className||"undefined"!=typeof e.getAttribute&&e.getAttribute("class")||"")})},ATTR:function(n,r,i){return function(e){var t=se.attr(e,n);return null==t?"!="===r:!r||(t+="","="===r?t===i:"!="===r?t!==i:"^="===r?i&&0===t.indexOf(i):"*="===r?i&&-1<t.indexOf(i):"$="===r?i&&t.slice(-i.length)===i:"~="===r?-1<(" "+t.replace(B," ")+" ").indexOf(i):"|="===r&&(t===i||t.slice(0,i.length+1)===i+"-"))}},CHILD:function(h,e,t,g,v){var y="nth"!==h.slice(0,3),m="last"!==h.slice(-4),x="of-type"===e;return 1===g&&0===v?function(e){return!!e.parentNode}:function(e,t,n){var r,i,o,a,s,u,l=y!==m?"nextSibling":"previousSibling",c=e.parentNode,f=x&&e.nodeName.toLowerCase(),p=!n&&!x,d=!1;if(c){if(y){while(l){a=e;while(a=a[l])if(x?a.nodeName.toLowerCase()===f:1===a.nodeType)return!1;u=l="only"===h&&!u&&"nextSibling"}return!0}if(u=[m?c.firstChild:c.lastChild],m&&p){d=(s=(r=(i=(o=(a=c)[S]||(a[S]={}))[a.uniqueID]||(o[a.uniqueID]={}))[h]||[])[0]===k&&r[1])&&r[2],a=s&&c.childNodes[s];while(a=++s&&a&&a[l]||(d=s=0)||u.pop())if(1===a.nodeType&&++d&&a===e){i[h]=[k,s,d];break}}else if(p&&(d=s=(r=(i=(o=(a=e)[S]||(a[S]={}))[a.uniqueID]||(o[a.uniqueID]={}))[h]||[])[0]===k&&r[1]),!1===d)while(a=++s&&a&&a[l]||(d=s=0)||u.pop())if((x?a.nodeName.toLowerCase()===f:1===a.nodeType)&&++d&&(p&&((i=(o=a[S]||(a[S]={}))[a.uniqueID]||(o[a.uniqueID]={}))[h]=[k,d]),a===e))break;return(d-=v)===g||d%g==0&&0<=d/g}}},PSEUDO:function(e,o){var t,a=b.pseudos[e]||b.setFilters[e.toLowerCase()]||se.error("unsupported pseudo: "+e);return a[S]?a(o):1<a.length?(t=[e,e,"",o],b.setFilters.hasOwnProperty(e.toLowerCase())?le(function(e,t){var n,r=a(e,o),i=r.length;while(i--)e[n=P(e,r[i])]=!(t[n]=r[i])}):function(e){return a(e,0,t)}):a}},pseudos:{not:le(function(e){var r=[],i=[],s=f(e.replace($,"$1"));return s[S]?le(function(e,t,n,r){var i,o=s(e,null,r,[]),a=e.length;while(a--)(i=o[a])&&(e[a]=!(t[a]=i))}):function(e,t,n){return r[0]=e,s(r,null,n,i),r[0]=null,!i.pop()}}),has:le(function(t){return function(e){return 0<se(t,e).length}}),contains:le(function(t){return t=t.replace(te,ne),function(e){return-1<(e.textContent||o(e)).indexOf(t)}}),lang:le(function(n){return V.test(n||"")||se.error("unsupported lang: "+n),n=n.replace(te,ne).toLowerCase(),function(e){var t;do{if(t=E?e.lang:e.getAttribute("xml:lang")||e.getAttribute("lang"))return(t=t.toLowerCase())===n||0===t.indexOf(n+"-")}while((e=e.parentNode)&&1===e.nodeType);return!1}}),target:function(e){var t=n.location&&n.location.hash;return t&&t.slice(1)===e.id},root:function(e){return e===a},focus:function(e){return e===C.activeElement&&(!C.hasFocus||C.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:ge(!1),disabled:ge(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!b.pseudos.empty(e)},header:function(e){return J.test(e.nodeName)},input:function(e){return Q.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:ve(function(){return[0]}),last:ve(function(e,t){return[t-1]}),eq:ve(function(e,t,n){return[n<0?n+t:n]}),even:ve(function(e,t){for(var n=0;n<t;n+=2)e.push(n);return e}),odd:ve(function(e,t){for(var n=1;n<t;n+=2)e.push(n);return e}),lt:ve(function(e,t,n){for(var r=n<0?n+t:t<n?t:n;0<=--r;)e.push(r);return e}),gt:ve(function(e,t,n){for(var r=n<0?n+t:n;++r<t;)e.push(r);return e})}}).pseudos.nth=b.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})b.pseudos[e]=de(e);for(e in{submit:!0,reset:!0})b.pseudos[e]=he(e);function me(){}function xe(e){for(var t=0,n=e.length,r="";t<n;t++)r+=e[t].value;return r}function be(s,e,t){var u=e.dir,l=e.next,c=l||u,f=t&&"parentNode"===c,p=r++;return e.first?function(e,t,n){while(e=e[u])if(1===e.nodeType||f)return s(e,t,n);return!1}:function(e,t,n){var r,i,o,a=[k,p];if(n){while(e=e[u])if((1===e.nodeType||f)&&s(e,t,n))return!0}else while(e=e[u])if(1===e.nodeType||f)if(i=(o=e[S]||(e[S]={}))[e.uniqueID]||(o[e.uniqueID]={}),l&&l===e.nodeName.toLowerCase())e=e[u]||e;else{if((r=i[c])&&r[0]===k&&r[1]===p)return a[2]=r[2];if((i[c]=a)[2]=s(e,t,n))return!0}return!1}}function we(i){return 1<i.length?function(e,t,n){var r=i.length;while(r--)if(!i[r](e,t,n))return!1;return!0}:i[0]}function Te(e,t,n,r,i){for(var o,a=[],s=0,u=e.length,l=null!=t;s<u;s++)(o=e[s])&&(n&&!n(o,r,i)||(a.push(o),l&&t.push(s)));return a}function Ce(d,h,g,v,y,e){return v&&!v[S]&&(v=Ce(v)),y&&!y[S]&&(y=Ce(y,e)),le(function(e,t,n,r){var i,o,a,s=[],u=[],l=t.length,c=e||function(e,t,n){for(var r=0,i=t.length;r<i;r++)se(e,t[r],n);return n}(h||"*",n.nodeType?[n]:n,[]),f=!d||!e&&h?c:Te(c,s,d,n,r),p=g?y||(e?d:l||v)?[]:t:f;if(g&&g(f,p,n,r),v){i=Te(p,u),v(i,[],n,r),o=i.length;while(o--)(a=i[o])&&(p[u[o]]=!(f[u[o]]=a))}if(e){if(y||d){if(y){i=[],o=p.length;while(o--)(a=p[o])&&i.push(f[o]=a);y(null,p=[],i,r)}o=p.length;while(o--)(a=p[o])&&-1<(i=y?P(e,a):s[o])&&(e[i]=!(t[i]=a))}}else p=Te(p===t?p.splice(l,p.length):p),y?y(null,t,p,r):H.apply(t,p)})}function Ee(e){for(var i,t,n,r=e.length,o=b.relative[e[0].type],a=o||b.relative[" "],s=o?1:0,u=be(function(e){return e===i},a,!0),l=be(function(e){return-1<P(i,e)},a,!0),c=[function(e,t,n){var r=!o&&(n||t!==w)||((i=t).nodeType?u(e,t,n):l(e,t,n));return i=null,r}];s<r;s++)if(t=b.relative[e[s].type])c=[be(we(c),t)];else{if((t=b.filter[e[s].type].apply(null,e[s].matches))[S]){for(n=++s;n<r;n++)if(b.relative[e[n].type])break;return Ce(1<s&&we(c),1<s&&xe(e.slice(0,s-1).concat({value:" "===e[s-2].type?"*":""})).replace($,"$1"),t,s<n&&Ee(e.slice(s,n)),n<r&&Ee(e=e.slice(n)),n<r&&xe(e))}c.push(t)}return we(c)}return me.prototype=b.filters=b.pseudos,b.setFilters=new me,h=se.tokenize=function(e,t){var n,r,i,o,a,s,u,l=x[e+" "];if(l)return t?0:l.slice(0);a=e,s=[],u=b.preFilter;while(a){for(o in n&&!(r=_.exec(a))||(r&&(a=a.slice(r[0].length)||a),s.push(i=[])),n=!1,(r=z.exec(a))&&(n=r.shift(),i.push({value:n,type:r[0].replace($," ")}),a=a.slice(n.length)),b.filter)!(r=G[o].exec(a))||u[o]&&!(r=u[o](r))||(n=r.shift(),i.push({value:n,type:o,matches:r}),a=a.slice(n.length));if(!n)break}return t?a.length:a?se.error(e):x(e,s).slice(0)},f=se.compile=function(e,t){var n,v,y,m,x,r,i=[],o=[],a=A[e+" "];if(!a){t||(t=h(e)),n=t.length;while(n--)(a=Ee(t[n]))[S]?i.push(a):o.push(a);(a=A(e,(v=o,m=0<(y=i).length,x=0<v.length,r=function(e,t,n,r,i){var o,a,s,u=0,l="0",c=e&&[],f=[],p=w,d=e||x&&b.find.TAG("*",i),h=k+=null==p?1:Math.random()||.1,g=d.length;for(i&&(w=t==C||t||i);l!==g&&null!=(o=d[l]);l++){if(x&&o){a=0,t||o.ownerDocument==C||(T(o),n=!E);while(s=v[a++])if(s(o,t||C,n)){r.push(o);break}i&&(k=h)}m&&((o=!s&&o)&&u--,e&&c.push(o))}if(u+=l,m&&l!==u){a=0;while(s=y[a++])s(c,f,t,n);if(e){if(0<u)while(l--)c[l]||f[l]||(f[l]=q.call(r));f=Te(f)}H.apply(r,f),i&&!e&&0<f.length&&1<u+y.length&&se.uniqueSort(r)}return i&&(k=h,w=p),c},m?le(r):r))).selector=e}return a},g=se.select=function(e,t,n,r){var i,o,a,s,u,l="function"==typeof e&&e,c=!r&&h(e=l.selector||e);if(n=n||[],1===c.length){if(2<(o=c[0]=c[0].slice(0)).length&&"ID"===(a=o[0]).type&&9===t.nodeType&&E&&b.relative[o[1].type]){if(!(t=(b.find.ID(a.matches[0].replace(te,ne),t)||[])[0]))return n;l&&(t=t.parentNode),e=e.slice(o.shift().value.length)}i=G.needsContext.test(e)?0:o.length;while(i--){if(a=o[i],b.relative[s=a.type])break;if((u=b.find[s])&&(r=u(a.matches[0].replace(te,ne),ee.test(o[0].type)&&ye(t.parentNode)||t))){if(o.splice(i,1),!(e=r.length&&xe(o)))return H.apply(n,r),n;break}}}return(l||f(e,c))(r,t,!E,n,!t||ee.test(e)&&ye(t.parentNode)||t),n},d.sortStable=S.split("").sort(D).join("")===S,d.detectDuplicates=!!l,T(),d.sortDetached=ce(function(e){return 1&e.compareDocumentPosition(C.createElement("fieldset"))}),ce(function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")})||fe("type|href|height|width",function(e,t,n){if(!n)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)}),d.attributes&&ce(function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")})||fe("value",function(e,t,n){if(!n&&"input"===e.nodeName.toLowerCase())return e.defaultValue}),ce(function(e){return null==e.getAttribute("disabled")})||fe(R,function(e,t,n){var r;if(!n)return!0===e[t]?t.toLowerCase():(r=e.getAttributeNode(t))&&r.specified?r.value:null}),se}(C);S.find=d,S.expr=d.selectors,S.expr[":"]=S.expr.pseudos,S.uniqueSort=S.unique=d.uniqueSort,S.text=d.getText,S.isXMLDoc=d.isXML,S.contains=d.contains,S.escapeSelector=d.escape;var h=function(e,t,n){var r=[],i=void 0!==n;while((e=e[t])&&9!==e.nodeType)if(1===e.nodeType){if(i&&S(e).is(n))break;r.push(e)}return r},T=function(e,t){for(var n=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&n.push(e);return n},k=S.expr.match.needsContext;function A(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()}var N=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function D(e,n,r){return m(n)?S.grep(e,function(e,t){return!!n.call(e,t,e)!==r}):n.nodeType?S.grep(e,function(e){return e===n!==r}):"string"!=typeof n?S.grep(e,function(e){return-1<i.call(n,e)!==r}):S.filter(n,e,r)}S.filter=function(e,t,n){var r=t[0];return n&&(e=":not("+e+")"),1===t.length&&1===r.nodeType?S.find.matchesSelector(r,e)?[r]:[]:S.find.matches(e,S.grep(t,function(e){return 1===e.nodeType}))},S.fn.extend({find:function(e){var t,n,r=this.length,i=this;if("string"!=typeof e)return this.pushStack(S(e).filter(function(){for(t=0;t<r;t++)if(S.contains(i[t],this))return!0}));for(n=this.pushStack([]),t=0;t<r;t++)S.find(e,i[t],n);return 1<r?S.uniqueSort(n):n},filter:function(e){return this.pushStack(D(this,e||[],!1))},not:function(e){return this.pushStack(D(this,e||[],!0))},is:function(e){return!!D(this,"string"==typeof e&&k.test(e)?S(e):e||[],!1).length}});var j,q=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;(S.fn.init=function(e,t,n){var r,i;if(!e)return this;if(n=n||j,"string"==typeof e){if(!(r="<"===e[0]&&">"===e[e.length-1]&&3<=e.length?[null,e,null]:q.exec(e))||!r[1]&&t)return!t||t.jquery?(t||n).find(e):this.constructor(t).find(e);if(r[1]){if(t=t instanceof S?t[0]:t,S.merge(this,S.parseHTML(r[1],t&&t.nodeType?t.ownerDocument||t:E,!0)),N.test(r[1])&&S.isPlainObject(t))for(r in t)m(this[r])?this[r](t[r]):this.attr(r,t[r]);return this}return(i=E.getElementById(r[2]))&&(this[0]=i,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):m(e)?void 0!==n.ready?n.ready(e):e(S):S.makeArray(e,this)}).prototype=S.fn,j=S(E);var L=/^(?:parents|prev(?:Until|All))/,H={children:!0,contents:!0,next:!0,prev:!0};function O(e,t){while((e=e[t])&&1!==e.nodeType);return e}S.fn.extend({has:function(e){var t=S(e,this),n=t.length;return this.filter(function(){for(var e=0;e<n;e++)if(S.contains(this,t[e]))return!0})},closest:function(e,t){var n,r=0,i=this.length,o=[],a="string"!=typeof e&&S(e);if(!k.test(e))for(;r<i;r++)for(n=this[r];n&&n!==t;n=n.parentNode)if(n.nodeType<11&&(a?-1<a.index(n):1===n.nodeType&&S.find.matchesSelector(n,e))){o.push(n);break}return this.pushStack(1<o.length?S.uniqueSort(o):o)},index:function(e){return e?"string"==typeof e?i.call(S(e),this[0]):i.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(S.uniqueSort(S.merge(this.get(),S(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),S.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return h(e,"parentNode")},parentsUntil:function(e,t,n){return h(e,"parentNode",n)},next:function(e){return O(e,"nextSibling")},prev:function(e){return O(e,"previousSibling")},nextAll:function(e){return h(e,"nextSibling")},prevAll:function(e){return h(e,"previousSibling")},nextUntil:function(e,t,n){return h(e,"nextSibling",n)},prevUntil:function(e,t,n){return h(e,"previousSibling",n)},siblings:function(e){return T((e.parentNode||{}).firstChild,e)},children:function(e){return T(e.firstChild)},contents:function(e){return null!=e.contentDocument&&r(e.contentDocument)?e.contentDocument:(A(e,"template")&&(e=e.content||e),S.merge([],e.childNodes))}},function(r,i){S.fn[r]=function(e,t){var n=S.map(this,i,e);return"Until"!==r.slice(-5)&&(t=e),t&&"string"==typeof t&&(n=S.filter(t,n)),1<this.length&&(H[r]||S.uniqueSort(n),L.test(r)&&n.reverse()),this.pushStack(n)}});var P=/[^\x20\t\r\n\f]+/g;function R(e){return e}function M(e){throw e}function I(e,t,n,r){var i;try{e&&m(i=e.promise)?i.call(e).done(t).fail(n):e&&m(i=e.then)?i.call(e,t,n):t.apply(void 0,[e].slice(r))}catch(e){n.apply(void 0,[e])}}S.Callbacks=function(r){var e,n;r="string"==typeof r?(e=r,n={},S.each(e.match(P)||[],function(e,t){n[t]=!0}),n):S.extend({},r);var i,t,o,a,s=[],u=[],l=-1,c=function(){for(a=a||r.once,o=i=!0;u.length;l=-1){t=u.shift();while(++l<s.length)!1===s[l].apply(t[0],t[1])&&r.stopOnFalse&&(l=s.length,t=!1)}r.memory||(t=!1),i=!1,a&&(s=t?[]:"")},f={add:function(){return s&&(t&&!i&&(l=s.length-1,u.push(t)),function n(e){S.each(e,function(e,t){m(t)?r.unique&&f.has(t)||s.push(t):t&&t.length&&"string"!==w(t)&&n(t)})}(arguments),t&&!i&&c()),this},remove:function(){return S.each(arguments,function(e,t){var n;while(-1<(n=S.inArray(t,s,n)))s.splice(n,1),n<=l&&l--}),this},has:function(e){return e?-1<S.inArray(e,s):0<s.length},empty:function(){return s&&(s=[]),this},disable:function(){return a=u=[],s=t="",this},disabled:function(){return!s},lock:function(){return a=u=[],t||i||(s=t=""),this},locked:function(){return!!a},fireWith:function(e,t){return a||(t=[e,(t=t||[]).slice?t.slice():t],u.push(t),i||c()),this},fire:function(){return f.fireWith(this,arguments),this},fired:function(){return!!o}};return f},S.extend({Deferred:function(e){var o=[["notify","progress",S.Callbacks("memory"),S.Callbacks("memory"),2],["resolve","done",S.Callbacks("once memory"),S.Callbacks("once memory"),0,"resolved"],["reject","fail",S.Callbacks("once memory"),S.Callbacks("once memory"),1,"rejected"]],i="pending",a={state:function(){return i},always:function(){return s.done(arguments).fail(arguments),this},"catch":function(e){return a.then(null,e)},pipe:function(){var i=arguments;return S.Deferred(function(r){S.each(o,function(e,t){var n=m(i[t[4]])&&i[t[4]];s[t[1]](function(){var e=n&&n.apply(this,arguments);e&&m(e.promise)?e.promise().progress(r.notify).done(r.resolve).fail(r.reject):r[t[0]+"With"](this,n?[e]:arguments)})}),i=null}).promise()},then:function(t,n,r){var u=0;function l(i,o,a,s){return function(){var n=this,r=arguments,e=function(){var e,t;if(!(i<u)){if((e=a.apply(n,r))===o.promise())throw new TypeError("Thenable self-resolution");t=e&&("object"==typeof e||"function"==typeof e)&&e.then,m(t)?s?t.call(e,l(u,o,R,s),l(u,o,M,s)):(u++,t.call(e,l(u,o,R,s),l(u,o,M,s),l(u,o,R,o.notifyWith))):(a!==R&&(n=void 0,r=[e]),(s||o.resolveWith)(n,r))}},t=s?e:function(){try{e()}catch(e){S.Deferred.exceptionHook&&S.Deferred.exceptionHook(e,t.stackTrace),u<=i+1&&(a!==M&&(n=void 0,r=[e]),o.rejectWith(n,r))}};i?t():(S.Deferred.getStackHook&&(t.stackTrace=S.Deferred.getStackHook()),C.setTimeout(t))}}return S.Deferred(function(e){o[0][3].add(l(0,e,m(r)?r:R,e.notifyWith)),o[1][3].add(l(0,e,m(t)?t:R)),o[2][3].add(l(0,e,m(n)?n:M))}).promise()},promise:function(e){return null!=e?S.extend(e,a):a}},s={};return S.each(o,function(e,t){var n=t[2],r=t[5];a[t[1]]=n.add,r&&n.add(function(){i=r},o[3-e][2].disable,o[3-e][3].disable,o[0][2].lock,o[0][3].lock),n.add(t[3].fire),s[t[0]]=function(){return s[t[0]+"With"](this===s?void 0:this,arguments),this},s[t[0]+"With"]=n.fireWith}),a.promise(s),e&&e.call(s,s),s},when:function(e){var n=arguments.length,t=n,r=Array(t),i=s.call(arguments),o=S.Deferred(),a=function(t){return function(e){r[t]=this,i[t]=1<arguments.length?s.call(arguments):e,--n||o.resolveWith(r,i)}};if(n<=1&&(I(e,o.done(a(t)).resolve,o.reject,!n),"pending"===o.state()||m(i[t]&&i[t].then)))return o.then();while(t--)I(i[t],a(t),o.reject);return o.promise()}});var W=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;S.Deferred.exceptionHook=function(e,t){C.console&&C.console.warn&&e&&W.test(e.name)&&C.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},S.readyException=function(e){C.setTimeout(function(){throw e})};var F=S.Deferred();function B(){E.removeEventListener("DOMContentLoaded",B),C.removeEventListener("load",B),S.ready()}S.fn.ready=function(e){return F.then(e)["catch"](function(e){S.readyException(e)}),this},S.extend({isReady:!1,readyWait:1,ready:function(e){(!0===e?--S.readyWait:S.isReady)||(S.isReady=!0)!==e&&0<--S.readyWait||F.resolveWith(E,[S])}}),S.ready.then=F.then,"complete"===E.readyState||"loading"!==E.readyState&&!E.documentElement.doScroll?C.setTimeout(S.ready):(E.addEventListener("DOMContentLoaded",B),C.addEventListener("load",B));var $=function(e,t,n,r,i,o,a){var s=0,u=e.length,l=null==n;if("object"===w(n))for(s in i=!0,n)$(e,t,s,n[s],!0,o,a);else if(void 0!==r&&(i=!0,m(r)||(a=!0),l&&(a?(t.call(e,r),t=null):(l=t,t=function(e,t,n){return l.call(S(e),n)})),t))for(;s<u;s++)t(e[s],n,a?r:r.call(e[s],s,t(e[s],n)));return i?e:l?t.call(e):u?t(e[0],n):o},_=/^-ms-/,z=/-([a-z])/g;function U(e,t){return t.toUpperCase()}function X(e){return e.replace(_,"ms-").replace(z,U)}var V=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function G(){this.expando=S.expando+G.uid++}G.uid=1,G.prototype={cache:function(e){var t=e[this.expando];return t||(t={},V(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,n){var r,i=this.cache(e);if("string"==typeof t)i[X(t)]=n;else for(r in t)i[X(r)]=t[r];return i},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][X(t)]},access:function(e,t,n){return void 0===t||t&&"string"==typeof t&&void 0===n?this.get(e,t):(this.set(e,t,n),void 0!==n?n:t)},remove:function(e,t){var n,r=e[this.expando];if(void 0!==r){if(void 0!==t){n=(t=Array.isArray(t)?t.map(X):(t=X(t))in r?[t]:t.match(P)||[]).length;while(n--)delete r[t[n]]}(void 0===t||S.isEmptyObject(r))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!S.isEmptyObject(t)}};var Y=new G,Q=new G,J=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,K=/[A-Z]/g;function Z(e,t,n){var r,i;if(void 0===n&&1===e.nodeType)if(r="data-"+t.replace(K,"-$&").toLowerCase(),"string"==typeof(n=e.getAttribute(r))){try{n="true"===(i=n)||"false"!==i&&("null"===i?null:i===+i+""?+i:J.test(i)?JSON.parse(i):i)}catch(e){}Q.set(e,t,n)}else n=void 0;return n}S.extend({hasData:function(e){return Q.hasData(e)||Y.hasData(e)},data:function(e,t,n){return Q.access(e,t,n)},removeData:function(e,t){Q.remove(e,t)},_data:function(e,t,n){return Y.access(e,t,n)},_removeData:function(e,t){Y.remove(e,t)}}),S.fn.extend({data:function(n,e){var t,r,i,o=this[0],a=o&&o.attributes;if(void 0===n){if(this.length&&(i=Q.get(o),1===o.nodeType&&!Y.get(o,"hasDataAttrs"))){t=a.length;while(t--)a[t]&&0===(r=a[t].name).indexOf("data-")&&(r=X(r.slice(5)),Z(o,r,i[r]));Y.set(o,"hasDataAttrs",!0)}return i}return"object"==typeof n?this.each(function(){Q.set(this,n)}):$(this,function(e){var t;if(o&&void 0===e)return void 0!==(t=Q.get(o,n))?t:void 0!==(t=Z(o,n))?t:void 0;this.each(function(){Q.set(this,n,e)})},null,e,1<arguments.length,null,!0)},removeData:function(e){return this.each(function(){Q.remove(this,e)})}}),S.extend({queue:function(e,t,n){var r;if(e)return t=(t||"fx")+"queue",r=Y.get(e,t),n&&(!r||Array.isArray(n)?r=Y.access(e,t,S.makeArray(n)):r.push(n)),r||[]},dequeue:function(e,t){t=t||"fx";var n=S.queue(e,t),r=n.length,i=n.shift(),o=S._queueHooks(e,t);"inprogress"===i&&(i=n.shift(),r--),i&&("fx"===t&&n.unshift("inprogress"),delete o.stop,i.call(e,function(){S.dequeue(e,t)},o)),!r&&o&&o.empty.fire()},_queueHooks:function(e,t){var n=t+"queueHooks";return Y.get(e,n)||Y.access(e,n,{empty:S.Callbacks("once memory").add(function(){Y.remove(e,[t+"queue",n])})})}}),S.fn.extend({queue:function(t,n){var e=2;return"string"!=typeof t&&(n=t,t="fx",e--),arguments.length<e?S.queue(this[0],t):void 0===n?this:this.each(function(){var e=S.queue(this,t,n);S._queueHooks(this,t),"fx"===t&&"inprogress"!==e[0]&&S.dequeue(this,t)})},dequeue:function(e){return this.each(function(){S.dequeue(this,e)})},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var n,r=1,i=S.Deferred(),o=this,a=this.length,s=function(){--r||i.resolveWith(o,[o])};"string"!=typeof e&&(t=e,e=void 0),e=e||"fx";while(a--)(n=Y.get(o[a],e+"queueHooks"))&&n.empty&&(r++,n.empty.add(s));return s(),i.promise(t)}});var ee=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,te=new RegExp("^(?:([+-])=|)("+ee+")([a-z%]*)$","i"),ne=["Top","Right","Bottom","Left"],re=E.documentElement,ie=function(e){return S.contains(e.ownerDocument,e)},oe={composed:!0};re.getRootNode&&(ie=function(e){return S.contains(e.ownerDocument,e)||e.getRootNode(oe)===e.ownerDocument});var ae=function(e,t){return"none"===(e=t||e).style.display||""===e.style.display&&ie(e)&&"none"===S.css(e,"display")};function se(e,t,n,r){var i,o,a=20,s=r?function(){return r.cur()}:function(){return S.css(e,t,"")},u=s(),l=n&&n[3]||(S.cssNumber[t]?"":"px"),c=e.nodeType&&(S.cssNumber[t]||"px"!==l&&+u)&&te.exec(S.css(e,t));if(c&&c[3]!==l){u/=2,l=l||c[3],c=+u||1;while(a--)S.style(e,t,c+l),(1-o)*(1-(o=s()/u||.5))<=0&&(a=0),c/=o;c*=2,S.style(e,t,c+l),n=n||[]}return n&&(c=+c||+u||0,i=n[1]?c+(n[1]+1)*n[2]:+n[2],r&&(r.unit=l,r.start=c,r.end=i)),i}var ue={};function le(e,t){for(var n,r,i,o,a,s,u,l=[],c=0,f=e.length;c<f;c++)(r=e[c]).style&&(n=r.style.display,t?("none"===n&&(l[c]=Y.get(r,"display")||null,l[c]||(r.style.display="")),""===r.style.display&&ae(r)&&(l[c]=(u=a=o=void 0,a=(i=r).ownerDocument,s=i.nodeName,(u=ue[s])||(o=a.body.appendChild(a.createElement(s)),u=S.css(o,"display"),o.parentNode.removeChild(o),"none"===u&&(u="block"),ue[s]=u)))):"none"!==n&&(l[c]="none",Y.set(r,"display",n)));for(c=0;c<f;c++)null!=l[c]&&(e[c].style.display=l[c]);return e}S.fn.extend({show:function(){return le(this,!0)},hide:function(){return le(this)},toggle:function(e){return"boolean"==typeof e?e?this.show():this.hide():this.each(function(){ae(this)?S(this).show():S(this).hide()})}});var ce,fe,pe=/^(?:checkbox|radio)$/i,de=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,he=/^$|^module$|\/(?:java|ecma)script/i;ce=E.createDocumentFragment().appendChild(E.createElement("div")),(fe=E.createElement("input")).setAttribute("type","radio"),fe.setAttribute("checked","checked"),fe.setAttribute("name","t"),ce.appendChild(fe),y.checkClone=ce.cloneNode(!0).cloneNode(!0).lastChild.checked,ce.innerHTML="<textarea>x</textarea>",y.noCloneChecked=!!ce.cloneNode(!0).lastChild.defaultValue,ce.innerHTML="<option></option>",y.option=!!ce.lastChild;var ge={thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function ve(e,t){var n;return n="undefined"!=typeof e.getElementsByTagName?e.getElementsByTagName(t||"*"):"undefined"!=typeof e.querySelectorAll?e.querySelectorAll(t||"*"):[],void 0===t||t&&A(e,t)?S.merge([e],n):n}function ye(e,t){for(var n=0,r=e.length;n<r;n++)Y.set(e[n],"globalEval",!t||Y.get(t[n],"globalEval"))}ge.tbody=ge.tfoot=ge.colgroup=ge.caption=ge.thead,ge.th=ge.td,y.option||(ge.optgroup=ge.option=[1,"<select multiple='multiple'>","</select>"]);var me=/<|&#?\w+;/;function xe(e,t,n,r,i){for(var o,a,s,u,l,c,f=t.createDocumentFragment(),p=[],d=0,h=e.length;d<h;d++)if((o=e[d])||0===o)if("object"===w(o))S.merge(p,o.nodeType?[o]:o);else if(me.test(o)){a=a||f.appendChild(t.createElement("div")),s=(de.exec(o)||["",""])[1].toLowerCase(),u=ge[s]||ge._default,a.innerHTML=u[1]+S.htmlPrefilter(o)+u[2],c=u[0];while(c--)a=a.lastChild;S.merge(p,a.childNodes),(a=f.firstChild).textContent=""}else p.push(t.createTextNode(o));f.textContent="",d=0;while(o=p[d++])if(r&&-1<S.inArray(o,r))i&&i.push(o);else if(l=ie(o),a=ve(f.appendChild(o),"script"),l&&ye(a),n){c=0;while(o=a[c++])he.test(o.type||"")&&n.push(o)}return f}var be=/^key/,we=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,Te=/^([^.]*)(?:\.(.+)|)/;function Ce(){return!0}function Ee(){return!1}function Se(e,t){return e===function(){try{return E.activeElement}catch(e){}}()==("focus"===t)}function ke(e,t,n,r,i,o){var a,s;if("object"==typeof t){for(s in"string"!=typeof n&&(r=r||n,n=void 0),t)ke(e,s,n,r,t[s],o);return e}if(null==r&&null==i?(i=n,r=n=void 0):null==i&&("string"==typeof n?(i=r,r=void 0):(i=r,r=n,n=void 0)),!1===i)i=Ee;else if(!i)return e;return 1===o&&(a=i,(i=function(e){return S().off(e),a.apply(this,arguments)}).guid=a.guid||(a.guid=S.guid++)),e.each(function(){S.event.add(this,t,i,r,n)})}function Ae(e,i,o){o?(Y.set(e,i,!1),S.event.add(e,i,{namespace:!1,handler:function(e){var t,n,r=Y.get(this,i);if(1&e.isTrigger&&this[i]){if(r.length)(S.event.special[i]||{}).delegateType&&e.stopPropagation();else if(r=s.call(arguments),Y.set(this,i,r),t=o(this,i),this[i](),r!==(n=Y.get(this,i))||t?Y.set(this,i,!1):n={},r!==n)return e.stopImmediatePropagation(),e.preventDefault(),n.value}else r.length&&(Y.set(this,i,{value:S.event.trigger(S.extend(r[0],S.Event.prototype),r.slice(1),this)}),e.stopImmediatePropagation())}})):void 0===Y.get(e,i)&&S.event.add(e,i,Ce)}S.event={global:{},add:function(t,e,n,r,i){var o,a,s,u,l,c,f,p,d,h,g,v=Y.get(t);if(V(t)){n.handler&&(n=(o=n).handler,i=o.selector),i&&S.find.matchesSelector(re,i),n.guid||(n.guid=S.guid++),(u=v.events)||(u=v.events=Object.create(null)),(a=v.handle)||(a=v.handle=function(e){return"undefined"!=typeof S&&S.event.triggered!==e.type?S.event.dispatch.apply(t,arguments):void 0}),l=(e=(e||"").match(P)||[""]).length;while(l--)d=g=(s=Te.exec(e[l])||[])[1],h=(s[2]||"").split(".").sort(),d&&(f=S.event.special[d]||{},d=(i?f.delegateType:f.bindType)||d,f=S.event.special[d]||{},c=S.extend({type:d,origType:g,data:r,handler:n,guid:n.guid,selector:i,needsContext:i&&S.expr.match.needsContext.test(i),namespace:h.join(".")},o),(p=u[d])||((p=u[d]=[]).delegateCount=0,f.setup&&!1!==f.setup.call(t,r,h,a)||t.addEventListener&&t.addEventListener(d,a)),f.add&&(f.add.call(t,c),c.handler.guid||(c.handler.guid=n.guid)),i?p.splice(p.delegateCount++,0,c):p.push(c),S.event.global[d]=!0)}},remove:function(e,t,n,r,i){var o,a,s,u,l,c,f,p,d,h,g,v=Y.hasData(e)&&Y.get(e);if(v&&(u=v.events)){l=(t=(t||"").match(P)||[""]).length;while(l--)if(d=g=(s=Te.exec(t[l])||[])[1],h=(s[2]||"").split(".").sort(),d){f=S.event.special[d]||{},p=u[d=(r?f.delegateType:f.bindType)||d]||[],s=s[2]&&new RegExp("(^|\\.)"+h.join("\\.(?:.*\\.|)")+"(\\.|$)"),a=o=p.length;while(o--)c=p[o],!i&&g!==c.origType||n&&n.guid!==c.guid||s&&!s.test(c.namespace)||r&&r!==c.selector&&("**"!==r||!c.selector)||(p.splice(o,1),c.selector&&p.delegateCount--,f.remove&&f.remove.call(e,c));a&&!p.length&&(f.teardown&&!1!==f.teardown.call(e,h,v.handle)||S.removeEvent(e,d,v.handle),delete u[d])}else for(d in u)S.event.remove(e,d+t[l],n,r,!0);S.isEmptyObject(u)&&Y.remove(e,"handle events")}},dispatch:function(e){var t,n,r,i,o,a,s=new Array(arguments.length),u=S.event.fix(e),l=(Y.get(this,"events")||Object.create(null))[u.type]||[],c=S.event.special[u.type]||{};for(s[0]=u,t=1;t<arguments.length;t++)s[t]=arguments[t];if(u.delegateTarget=this,!c.preDispatch||!1!==c.preDispatch.call(this,u)){a=S.event.handlers.call(this,u,l),t=0;while((i=a[t++])&&!u.isPropagationStopped()){u.currentTarget=i.elem,n=0;while((o=i.handlers[n++])&&!u.isImmediatePropagationStopped())u.rnamespace&&!1!==o.namespace&&!u.rnamespace.test(o.namespace)||(u.handleObj=o,u.data=o.data,void 0!==(r=((S.event.special[o.origType]||{}).handle||o.handler).apply(i.elem,s))&&!1===(u.result=r)&&(u.preventDefault(),u.stopPropagation()))}return c.postDispatch&&c.postDispatch.call(this,u),u.result}},handlers:function(e,t){var n,r,i,o,a,s=[],u=t.delegateCount,l=e.target;if(u&&l.nodeType&&!("click"===e.type&&1<=e.button))for(;l!==this;l=l.parentNode||this)if(1===l.nodeType&&("click"!==e.type||!0!==l.disabled)){for(o=[],a={},n=0;n<u;n++)void 0===a[i=(r=t[n]).selector+" "]&&(a[i]=r.needsContext?-1<S(i,this).index(l):S.find(i,this,null,[l]).length),a[i]&&o.push(r);o.length&&s.push({elem:l,handlers:o})}return l=this,u<t.length&&s.push({elem:l,handlers:t.slice(u)}),s},addProp:function(t,e){Object.defineProperty(S.Event.prototype,t,{enumerable:!0,configurable:!0,get:m(e)?function(){if(this.originalEvent)return e(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[t]},set:function(e){Object.defineProperty(this,t,{enumerable:!0,configurable:!0,writable:!0,value:e})}})},fix:function(e){return e[S.expando]?e:new S.Event(e)},special:{load:{noBubble:!0},click:{setup:function(e){var t=this||e;return pe.test(t.type)&&t.click&&A(t,"input")&&Ae(t,"click",Ce),!1},trigger:function(e){var t=this||e;return pe.test(t.type)&&t.click&&A(t,"input")&&Ae(t,"click"),!0},_default:function(e){var t=e.target;return pe.test(t.type)&&t.click&&A(t,"input")&&Y.get(t,"click")||A(t,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},S.removeEvent=function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n)},S.Event=function(e,t){if(!(this instanceof S.Event))return new S.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?Ce:Ee,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&S.extend(this,t),this.timeStamp=e&&e.timeStamp||Date.now(),this[S.expando]=!0},S.Event.prototype={constructor:S.Event,isDefaultPrevented:Ee,isPropagationStopped:Ee,isImmediatePropagationStopped:Ee,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=Ce,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=Ce,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=Ce,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},S.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,"char":!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&be.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&we.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},S.event.addProp),S.each({focus:"focusin",blur:"focusout"},function(e,t){S.event.special[e]={setup:function(){return Ae(this,e,Se),!1},trigger:function(){return Ae(this,e),!0},delegateType:t}}),S.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(e,i){S.event.special[e]={delegateType:i,bindType:i,handle:function(e){var t,n=e.relatedTarget,r=e.handleObj;return n&&(n===this||S.contains(this,n))||(e.type=r.origType,t=r.handler.apply(this,arguments),e.type=i),t}}}),S.fn.extend({on:function(e,t,n,r){return ke(this,e,t,n,r)},one:function(e,t,n,r){return ke(this,e,t,n,r,1)},off:function(e,t,n){var r,i;if(e&&e.preventDefault&&e.handleObj)return r=e.handleObj,S(e.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if("object"==typeof e){for(i in e)this.off(i,t,e[i]);return this}return!1!==t&&"function"!=typeof t||(n=t,t=void 0),!1===n&&(n=Ee),this.each(function(){S.event.remove(this,e,n,t)})}});var Ne=/<script|<style|<link/i,De=/checked\s*(?:[^=]|=\s*.checked.)/i,je=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function qe(e,t){return A(e,"table")&&A(11!==t.nodeType?t:t.firstChild,"tr")&&S(e).children("tbody")[0]||e}function Le(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function He(e){return"true/"===(e.type||"").slice(0,5)?e.type=e.type.slice(5):e.removeAttribute("type"),e}function Oe(e,t){var n,r,i,o,a,s;if(1===t.nodeType){if(Y.hasData(e)&&(s=Y.get(e).events))for(i in Y.remove(t,"handle events"),s)for(n=0,r=s[i].length;n<r;n++)S.event.add(t,i,s[i][n]);Q.hasData(e)&&(o=Q.access(e),a=S.extend({},o),Q.set(t,a))}}function Pe(n,r,i,o){r=g(r);var e,t,a,s,u,l,c=0,f=n.length,p=f-1,d=r[0],h=m(d);if(h||1<f&&"string"==typeof d&&!y.checkClone&&De.test(d))return n.each(function(e){var t=n.eq(e);h&&(r[0]=d.call(this,e,t.html())),Pe(t,r,i,o)});if(f&&(t=(e=xe(r,n[0].ownerDocument,!1,n,o)).firstChild,1===e.childNodes.length&&(e=t),t||o)){for(s=(a=S.map(ve(e,"script"),Le)).length;c<f;c++)u=e,c!==p&&(u=S.clone(u,!0,!0),s&&S.merge(a,ve(u,"script"))),i.call(n[c],u,c);if(s)for(l=a[a.length-1].ownerDocument,S.map(a,He),c=0;c<s;c++)u=a[c],he.test(u.type||"")&&!Y.access(u,"globalEval")&&S.contains(l,u)&&(u.src&&"module"!==(u.type||"").toLowerCase()?S._evalUrl&&!u.noModule&&S._evalUrl(u.src,{nonce:u.nonce||u.getAttribute("nonce")},l):b(u.textContent.replace(je,""),u,l))}return n}function Re(e,t,n){for(var r,i=t?S.filter(t,e):e,o=0;null!=(r=i[o]);o++)n||1!==r.nodeType||S.cleanData(ve(r)),r.parentNode&&(n&&ie(r)&&ye(ve(r,"script")),r.parentNode.removeChild(r));return e}S.extend({htmlPrefilter:function(e){return e},clone:function(e,t,n){var r,i,o,a,s,u,l,c=e.cloneNode(!0),f=ie(e);if(!(y.noCloneChecked||1!==e.nodeType&&11!==e.nodeType||S.isXMLDoc(e)))for(a=ve(c),r=0,i=(o=ve(e)).length;r<i;r++)s=o[r],u=a[r],void 0,"input"===(l=u.nodeName.toLowerCase())&&pe.test(s.type)?u.checked=s.checked:"input"!==l&&"textarea"!==l||(u.defaultValue=s.defaultValue);if(t)if(n)for(o=o||ve(e),a=a||ve(c),r=0,i=o.length;r<i;r++)Oe(o[r],a[r]);else Oe(e,c);return 0<(a=ve(c,"script")).length&&ye(a,!f&&ve(e,"script")),c},cleanData:function(e){for(var t,n,r,i=S.event.special,o=0;void 0!==(n=e[o]);o++)if(V(n)){if(t=n[Y.expando]){if(t.events)for(r in t.events)i[r]?S.event.remove(n,r):S.removeEvent(n,r,t.handle);n[Y.expando]=void 0}n[Q.expando]&&(n[Q.expando]=void 0)}}}),S.fn.extend({detach:function(e){return Re(this,e,!0)},remove:function(e){return Re(this,e)},text:function(e){return $(this,function(e){return void 0===e?S.text(this):this.empty().each(function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)})},null,e,arguments.length)},append:function(){return Pe(this,arguments,function(e){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||qe(this,e).appendChild(e)})},prepend:function(){return Pe(this,arguments,function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=qe(this,e);t.insertBefore(e,t.firstChild)}})},before:function(){return Pe(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this)})},after:function(){return Pe(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)})},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(S.cleanData(ve(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map(function(){return S.clone(this,e,t)})},html:function(e){return $(this,function(e){var t=this[0]||{},n=0,r=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"==typeof e&&!Ne.test(e)&&!ge[(de.exec(e)||["",""])[1].toLowerCase()]){e=S.htmlPrefilter(e);try{for(;n<r;n++)1===(t=this[n]||{}).nodeType&&(S.cleanData(ve(t,!1)),t.innerHTML=e);t=0}catch(e){}}t&&this.empty().append(e)},null,e,arguments.length)},replaceWith:function(){var n=[];return Pe(this,arguments,function(e){var t=this.parentNode;S.inArray(this,n)<0&&(S.cleanData(ve(this)),t&&t.replaceChild(e,this))},n)}}),S.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(e,a){S.fn[e]=function(e){for(var t,n=[],r=S(e),i=r.length-1,o=0;o<=i;o++)t=o===i?this:this.clone(!0),S(r[o])[a](t),u.apply(n,t.get());return this.pushStack(n)}});var Me=new RegExp("^("+ee+")(?!px)[a-z%]+$","i"),Ie=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=C),t.getComputedStyle(e)},We=function(e,t,n){var r,i,o={};for(i in t)o[i]=e.style[i],e.style[i]=t[i];for(i in r=n.call(e),t)e.style[i]=o[i];return r},Fe=new RegExp(ne.join("|"),"i");function Be(e,t,n){var r,i,o,a,s=e.style;return(n=n||Ie(e))&&(""!==(a=n.getPropertyValue(t)||n[t])||ie(e)||(a=S.style(e,t)),!y.pixelBoxStyles()&&Me.test(a)&&Fe.test(t)&&(r=s.width,i=s.minWidth,o=s.maxWidth,s.minWidth=s.maxWidth=s.width=a,a=n.width,s.width=r,s.minWidth=i,s.maxWidth=o)),void 0!==a?a+"":a}function $e(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}!function(){function e(){if(l){u.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",l.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",re.appendChild(u).appendChild(l);var e=C.getComputedStyle(l);n="1%"!==e.top,s=12===t(e.marginLeft),l.style.right="60%",o=36===t(e.right),r=36===t(e.width),l.style.position="absolute",i=12===t(l.offsetWidth/3),re.removeChild(u),l=null}}function t(e){return Math.round(parseFloat(e))}var n,r,i,o,a,s,u=E.createElement("div"),l=E.createElement("div");l.style&&(l.style.backgroundClip="content-box",l.cloneNode(!0).style.backgroundClip="",y.clearCloneStyle="content-box"===l.style.backgroundClip,S.extend(y,{boxSizingReliable:function(){return e(),r},pixelBoxStyles:function(){return e(),o},pixelPosition:function(){return e(),n},reliableMarginLeft:function(){return e(),s},scrollboxSize:function(){return e(),i},reliableTrDimensions:function(){var e,t,n,r;return null==a&&(e=E.createElement("table"),t=E.createElement("tr"),n=E.createElement("div"),e.style.cssText="position:absolute;left:-11111px",t.style.height="1px",n.style.height="9px",re.appendChild(e).appendChild(t).appendChild(n),r=C.getComputedStyle(t),a=3<parseInt(r.height),re.removeChild(e)),a}}))}();var _e=["Webkit","Moz","ms"],ze=E.createElement("div").style,Ue={};function Xe(e){var t=S.cssProps[e]||Ue[e];return t||(e in ze?e:Ue[e]=function(e){var t=e[0].toUpperCase()+e.slice(1),n=_e.length;while(n--)if((e=_e[n]+t)in ze)return e}(e)||e)}var Ve=/^(none|table(?!-c[ea]).+)/,Ge=/^--/,Ye={position:"absolute",visibility:"hidden",display:"block"},Qe={letterSpacing:"0",fontWeight:"400"};function Je(e,t,n){var r=te.exec(t);return r?Math.max(0,r[2]-(n||0))+(r[3]||"px"):t}function Ke(e,t,n,r,i,o){var a="width"===t?1:0,s=0,u=0;if(n===(r?"border":"content"))return 0;for(;a<4;a+=2)"margin"===n&&(u+=S.css(e,n+ne[a],!0,i)),r?("content"===n&&(u-=S.css(e,"padding"+ne[a],!0,i)),"margin"!==n&&(u-=S.css(e,"border"+ne[a]+"Width",!0,i))):(u+=S.css(e,"padding"+ne[a],!0,i),"padding"!==n?u+=S.css(e,"border"+ne[a]+"Width",!0,i):s+=S.css(e,"border"+ne[a]+"Width",!0,i));return!r&&0<=o&&(u+=Math.max(0,Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-o-u-s-.5))||0),u}function Ze(e,t,n){var r=Ie(e),i=(!y.boxSizingReliable()||n)&&"border-box"===S.css(e,"boxSizing",!1,r),o=i,a=Be(e,t,r),s="offset"+t[0].toUpperCase()+t.slice(1);if(Me.test(a)){if(!n)return a;a="auto"}return(!y.boxSizingReliable()&&i||!y.reliableTrDimensions()&&A(e,"tr")||"auto"===a||!parseFloat(a)&&"inline"===S.css(e,"display",!1,r))&&e.getClientRects().length&&(i="border-box"===S.css(e,"boxSizing",!1,r),(o=s in e)&&(a=e[s])),(a=parseFloat(a)||0)+Ke(e,t,n||(i?"border":"content"),o,r,a)+"px"}function et(e,t,n,r,i){return new et.prototype.init(e,t,n,r,i)}S.extend({cssHooks:{opacity:{get:function(e,t){if(t){var n=Be(e,"opacity");return""===n?"1":n}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(e,t,n,r){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var i,o,a,s=X(t),u=Ge.test(t),l=e.style;if(u||(t=Xe(s)),a=S.cssHooks[t]||S.cssHooks[s],void 0===n)return a&&"get"in a&&void 0!==(i=a.get(e,!1,r))?i:l[t];"string"===(o=typeof n)&&(i=te.exec(n))&&i[1]&&(n=se(e,t,i),o="number"),null!=n&&n==n&&("number"!==o||u||(n+=i&&i[3]||(S.cssNumber[s]?"":"px")),y.clearCloneStyle||""!==n||0!==t.indexOf("background")||(l[t]="inherit"),a&&"set"in a&&void 0===(n=a.set(e,n,r))||(u?l.setProperty(t,n):l[t]=n))}},css:function(e,t,n,r){var i,o,a,s=X(t);return Ge.test(t)||(t=Xe(s)),(a=S.cssHooks[t]||S.cssHooks[s])&&"get"in a&&(i=a.get(e,!0,n)),void 0===i&&(i=Be(e,t,r)),"normal"===i&&t in Qe&&(i=Qe[t]),""===n||n?(o=parseFloat(i),!0===n||isFinite(o)?o||0:i):i}}),S.each(["height","width"],function(e,u){S.cssHooks[u]={get:function(e,t,n){if(t)return!Ve.test(S.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?Ze(e,u,n):We(e,Ye,function(){return Ze(e,u,n)})},set:function(e,t,n){var r,i=Ie(e),o=!y.scrollboxSize()&&"absolute"===i.position,a=(o||n)&&"border-box"===S.css(e,"boxSizing",!1,i),s=n?Ke(e,u,n,a,i):0;return a&&o&&(s-=Math.ceil(e["offset"+u[0].toUpperCase()+u.slice(1)]-parseFloat(i[u])-Ke(e,u,"border",!1,i)-.5)),s&&(r=te.exec(t))&&"px"!==(r[3]||"px")&&(e.style[u]=t,t=S.css(e,u)),Je(0,t,s)}}}),S.cssHooks.marginLeft=$e(y.reliableMarginLeft,function(e,t){if(t)return(parseFloat(Be(e,"marginLeft"))||e.getBoundingClientRect().left-We(e,{marginLeft:0},function(){return e.getBoundingClientRect().left}))+"px"}),S.each({margin:"",padding:"",border:"Width"},function(i,o){S.cssHooks[i+o]={expand:function(e){for(var t=0,n={},r="string"==typeof e?e.split(" "):[e];t<4;t++)n[i+ne[t]+o]=r[t]||r[t-2]||r[0];return n}},"margin"!==i&&(S.cssHooks[i+o].set=Je)}),S.fn.extend({css:function(e,t){return $(this,function(e,t,n){var r,i,o={},a=0;if(Array.isArray(t)){for(r=Ie(e),i=t.length;a<i;a++)o[t[a]]=S.css(e,t[a],!1,r);return o}return void 0!==n?S.style(e,t,n):S.css(e,t)},e,t,1<arguments.length)}}),((S.Tween=et).prototype={constructor:et,init:function(e,t,n,r,i,o){this.elem=e,this.prop=n,this.easing=i||S.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=r,this.unit=o||(S.cssNumber[n]?"":"px")},cur:function(){var e=et.propHooks[this.prop];return e&&e.get?e.get(this):et.propHooks._default.get(this)},run:function(e){var t,n=et.propHooks[this.prop];return this.options.duration?this.pos=t=S.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):et.propHooks._default.set(this),this}}).init.prototype=et.prototype,(et.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=S.css(e.elem,e.prop,""))&&"auto"!==t?t:0},set:function(e){S.fx.step[e.prop]?S.fx.step[e.prop](e):1!==e.elem.nodeType||!S.cssHooks[e.prop]&&null==e.elem.style[Xe(e.prop)]?e.elem[e.prop]=e.now:S.style(e.elem,e.prop,e.now+e.unit)}}}).scrollTop=et.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},S.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},S.fx=et.prototype.init,S.fx.step={};var tt,nt,rt,it,ot=/^(?:toggle|show|hide)$/,at=/queueHooks$/;function st(){nt&&(!1===E.hidden&&C.requestAnimationFrame?C.requestAnimationFrame(st):C.setTimeout(st,S.fx.interval),S.fx.tick())}function ut(){return C.setTimeout(function(){tt=void 0}),tt=Date.now()}function lt(e,t){var n,r=0,i={height:e};for(t=t?1:0;r<4;r+=2-t)i["margin"+(n=ne[r])]=i["padding"+n]=e;return t&&(i.opacity=i.width=e),i}function ct(e,t,n){for(var r,i=(ft.tweeners[t]||[]).concat(ft.tweeners["*"]),o=0,a=i.length;o<a;o++)if(r=i[o].call(n,t,e))return r}function ft(o,e,t){var n,a,r=0,i=ft.prefilters.length,s=S.Deferred().always(function(){delete u.elem}),u=function(){if(a)return!1;for(var e=tt||ut(),t=Math.max(0,l.startTime+l.duration-e),n=1-(t/l.duration||0),r=0,i=l.tweens.length;r<i;r++)l.tweens[r].run(n);return s.notifyWith(o,[l,n,t]),n<1&&i?t:(i||s.notifyWith(o,[l,1,0]),s.resolveWith(o,[l]),!1)},l=s.promise({elem:o,props:S.extend({},e),opts:S.extend(!0,{specialEasing:{},easing:S.easing._default},t),originalProperties:e,originalOptions:t,startTime:tt||ut(),duration:t.duration,tweens:[],createTween:function(e,t){var n=S.Tween(o,l.opts,e,t,l.opts.specialEasing[e]||l.opts.easing);return l.tweens.push(n),n},stop:function(e){var t=0,n=e?l.tweens.length:0;if(a)return this;for(a=!0;t<n;t++)l.tweens[t].run(1);return e?(s.notifyWith(o,[l,1,0]),s.resolveWith(o,[l,e])):s.rejectWith(o,[l,e]),this}}),c=l.props;for(!function(e,t){var n,r,i,o,a;for(n in e)if(i=t[r=X(n)],o=e[n],Array.isArray(o)&&(i=o[1],o=e[n]=o[0]),n!==r&&(e[r]=o,delete e[n]),(a=S.cssHooks[r])&&"expand"in a)for(n in o=a.expand(o),delete e[r],o)n in e||(e[n]=o[n],t[n]=i);else t[r]=i}(c,l.opts.specialEasing);r<i;r++)if(n=ft.prefilters[r].call(l,o,c,l.opts))return m(n.stop)&&(S._queueHooks(l.elem,l.opts.queue).stop=n.stop.bind(n)),n;return S.map(c,ct,l),m(l.opts.start)&&l.opts.start.call(o,l),l.progress(l.opts.progress).done(l.opts.done,l.opts.complete).fail(l.opts.fail).always(l.opts.always),S.fx.timer(S.extend(u,{elem:o,anim:l,queue:l.opts.queue})),l}S.Animation=S.extend(ft,{tweeners:{"*":[function(e,t){var n=this.createTween(e,t);return se(n.elem,e,te.exec(t),n),n}]},tweener:function(e,t){m(e)?(t=e,e=["*"]):e=e.match(P);for(var n,r=0,i=e.length;r<i;r++)n=e[r],ft.tweeners[n]=ft.tweeners[n]||[],ft.tweeners[n].unshift(t)},prefilters:[function(e,t,n){var r,i,o,a,s,u,l,c,f="width"in t||"height"in t,p=this,d={},h=e.style,g=e.nodeType&&ae(e),v=Y.get(e,"fxshow");for(r in n.queue||(null==(a=S._queueHooks(e,"fx")).unqueued&&(a.unqueued=0,s=a.empty.fire,a.empty.fire=function(){a.unqueued||s()}),a.unqueued++,p.always(function(){p.always(function(){a.unqueued--,S.queue(e,"fx").length||a.empty.fire()})})),t)if(i=t[r],ot.test(i)){if(delete t[r],o=o||"toggle"===i,i===(g?"hide":"show")){if("show"!==i||!v||void 0===v[r])continue;g=!0}d[r]=v&&v[r]||S.style(e,r)}if((u=!S.isEmptyObject(t))||!S.isEmptyObject(d))for(r in f&&1===e.nodeType&&(n.overflow=[h.overflow,h.overflowX,h.overflowY],null==(l=v&&v.display)&&(l=Y.get(e,"display")),"none"===(c=S.css(e,"display"))&&(l?c=l:(le([e],!0),l=e.style.display||l,c=S.css(e,"display"),le([e]))),("inline"===c||"inline-block"===c&&null!=l)&&"none"===S.css(e,"float")&&(u||(p.done(function(){h.display=l}),null==l&&(c=h.display,l="none"===c?"":c)),h.display="inline-block")),n.overflow&&(h.overflow="hidden",p.always(function(){h.overflow=n.overflow[0],h.overflowX=n.overflow[1],h.overflowY=n.overflow[2]})),u=!1,d)u||(v?"hidden"in v&&(g=v.hidden):v=Y.access(e,"fxshow",{display:l}),o&&(v.hidden=!g),g&&le([e],!0),p.done(function(){for(r in g||le([e]),Y.remove(e,"fxshow"),d)S.style(e,r,d[r])})),u=ct(g?v[r]:0,r,p),r in v||(v[r]=u.start,g&&(u.end=u.start,u.start=0))}],prefilter:function(e,t){t?ft.prefilters.unshift(e):ft.prefilters.push(e)}}),S.speed=function(e,t,n){var r=e&&"object"==typeof e?S.extend({},e):{complete:n||!n&&t||m(e)&&e,duration:e,easing:n&&t||t&&!m(t)&&t};return S.fx.off?r.duration=0:"number"!=typeof r.duration&&(r.duration in S.fx.speeds?r.duration=S.fx.speeds[r.duration]:r.duration=S.fx.speeds._default),null!=r.queue&&!0!==r.queue||(r.queue="fx"),r.old=r.complete,r.complete=function(){m(r.old)&&r.old.call(this),r.queue&&S.dequeue(this,r.queue)},r},S.fn.extend({fadeTo:function(e,t,n,r){return this.filter(ae).css("opacity",0).show().end().animate({opacity:t},e,n,r)},animate:function(t,e,n,r){var i=S.isEmptyObject(t),o=S.speed(e,n,r),a=function(){var e=ft(this,S.extend({},t),o);(i||Y.get(this,"finish"))&&e.stop(!0)};return a.finish=a,i||!1===o.queue?this.each(a):this.queue(o.queue,a)},stop:function(i,e,o){var a=function(e){var t=e.stop;delete e.stop,t(o)};return"string"!=typeof i&&(o=e,e=i,i=void 0),e&&this.queue(i||"fx",[]),this.each(function(){var e=!0,t=null!=i&&i+"queueHooks",n=S.timers,r=Y.get(this);if(t)r[t]&&r[t].stop&&a(r[t]);else for(t in r)r[t]&&r[t].stop&&at.test(t)&&a(r[t]);for(t=n.length;t--;)n[t].elem!==this||null!=i&&n[t].queue!==i||(n[t].anim.stop(o),e=!1,n.splice(t,1));!e&&o||S.dequeue(this,i)})},finish:function(a){return!1!==a&&(a=a||"fx"),this.each(function(){var e,t=Y.get(this),n=t[a+"queue"],r=t[a+"queueHooks"],i=S.timers,o=n?n.length:0;for(t.finish=!0,S.queue(this,a,[]),r&&r.stop&&r.stop.call(this,!0),e=i.length;e--;)i[e].elem===this&&i[e].queue===a&&(i[e].anim.stop(!0),i.splice(e,1));for(e=0;e<o;e++)n[e]&&n[e].finish&&n[e].finish.call(this);delete t.finish})}}),S.each(["toggle","show","hide"],function(e,r){var i=S.fn[r];S.fn[r]=function(e,t,n){return null==e||"boolean"==typeof e?i.apply(this,arguments):this.animate(lt(r,!0),e,t,n)}}),S.each({slideDown:lt("show"),slideUp:lt("hide"),slideToggle:lt("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(e,r){S.fn[e]=function(e,t,n){return this.animate(r,e,t,n)}}),S.timers=[],S.fx.tick=function(){var e,t=0,n=S.timers;for(tt=Date.now();t<n.length;t++)(e=n[t])()||n[t]!==e||n.splice(t--,1);n.length||S.fx.stop(),tt=void 0},S.fx.timer=function(e){S.timers.push(e),S.fx.start()},S.fx.interval=13,S.fx.start=function(){nt||(nt=!0,st())},S.fx.stop=function(){nt=null},S.fx.speeds={slow:600,fast:200,_default:400},S.fn.delay=function(r,e){return r=S.fx&&S.fx.speeds[r]||r,e=e||"fx",this.queue(e,function(e,t){var n=C.setTimeout(e,r);t.stop=function(){C.clearTimeout(n)}})},rt=E.createElement("input"),it=E.createElement("select").appendChild(E.createElement("option")),rt.type="checkbox",y.checkOn=""!==rt.value,y.optSelected=it.selected,(rt=E.createElement("input")).value="t",rt.type="radio",y.radioValue="t"===rt.value;var pt,dt=S.expr.attrHandle;S.fn.extend({attr:function(e,t){return $(this,S.attr,e,t,1<arguments.length)},removeAttr:function(e){return this.each(function(){S.removeAttr(this,e)})}}),S.extend({attr:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return"undefined"==typeof e.getAttribute?S.prop(e,t,n):(1===o&&S.isXMLDoc(e)||(i=S.attrHooks[t.toLowerCase()]||(S.expr.match.bool.test(t)?pt:void 0)),void 0!==n?null===n?void S.removeAttr(e,t):i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:(e.setAttribute(t,n+""),n):i&&"get"in i&&null!==(r=i.get(e,t))?r:null==(r=S.find.attr(e,t))?void 0:r)},attrHooks:{type:{set:function(e,t){if(!y.radioValue&&"radio"===t&&A(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},removeAttr:function(e,t){var n,r=0,i=t&&t.match(P);if(i&&1===e.nodeType)while(n=i[r++])e.removeAttribute(n)}}),pt={set:function(e,t,n){return!1===t?S.removeAttr(e,n):e.setAttribute(n,n),n}},S.each(S.expr.match.bool.source.match(/\w+/g),function(e,t){var a=dt[t]||S.find.attr;dt[t]=function(e,t,n){var r,i,o=t.toLowerCase();return n||(i=dt[o],dt[o]=r,r=null!=a(e,t,n)?o:null,dt[o]=i),r}});var ht=/^(?:input|select|textarea|button)$/i,gt=/^(?:a|area)$/i;function vt(e){return(e.match(P)||[]).join(" ")}function yt(e){return e.getAttribute&&e.getAttribute("class")||""}function mt(e){return Array.isArray(e)?e:"string"==typeof e&&e.match(P)||[]}S.fn.extend({prop:function(e,t){return $(this,S.prop,e,t,1<arguments.length)},removeProp:function(e){return this.each(function(){delete this[S.propFix[e]||e]})}}),S.extend({prop:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return 1===o&&S.isXMLDoc(e)||(t=S.propFix[t]||t,i=S.propHooks[t]),void 0!==n?i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:e[t]=n:i&&"get"in i&&null!==(r=i.get(e,t))?r:e[t]},propHooks:{tabIndex:{get:function(e){var t=S.find.attr(e,"tabindex");return t?parseInt(t,10):ht.test(e.nodeName)||gt.test(e.nodeName)&&e.href?0:-1}}},propFix:{"for":"htmlFor","class":"className"}}),y.optSelected||(S.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),S.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){S.propFix[this.toLowerCase()]=this}),S.fn.extend({addClass:function(t){var e,n,r,i,o,a,s,u=0;if(m(t))return this.each(function(e){S(this).addClass(t.call(this,e,yt(this)))});if((e=mt(t)).length)while(n=this[u++])if(i=yt(n),r=1===n.nodeType&&" "+vt(i)+" "){a=0;while(o=e[a++])r.indexOf(" "+o+" ")<0&&(r+=o+" ");i!==(s=vt(r))&&n.setAttribute("class",s)}return this},removeClass:function(t){var e,n,r,i,o,a,s,u=0;if(m(t))return this.each(function(e){S(this).removeClass(t.call(this,e,yt(this)))});if(!arguments.length)return this.attr("class","");if((e=mt(t)).length)while(n=this[u++])if(i=yt(n),r=1===n.nodeType&&" "+vt(i)+" "){a=0;while(o=e[a++])while(-1<r.indexOf(" "+o+" "))r=r.replace(" "+o+" "," ");i!==(s=vt(r))&&n.setAttribute("class",s)}return this},toggleClass:function(i,t){var o=typeof i,a="string"===o||Array.isArray(i);return"boolean"==typeof t&&a?t?this.addClass(i):this.removeClass(i):m(i)?this.each(function(e){S(this).toggleClass(i.call(this,e,yt(this),t),t)}):this.each(function(){var e,t,n,r;if(a){t=0,n=S(this),r=mt(i);while(e=r[t++])n.hasClass(e)?n.removeClass(e):n.addClass(e)}else void 0!==i&&"boolean"!==o||((e=yt(this))&&Y.set(this,"__className__",e),this.setAttribute&&this.setAttribute("class",e||!1===i?"":Y.get(this,"__className__")||""))})},hasClass:function(e){var t,n,r=0;t=" "+e+" ";while(n=this[r++])if(1===n.nodeType&&-1<(" "+vt(yt(n))+" ").indexOf(t))return!0;return!1}});var xt=/\r/g;S.fn.extend({val:function(n){var r,e,i,t=this[0];return arguments.length?(i=m(n),this.each(function(e){var t;1===this.nodeType&&(null==(t=i?n.call(this,e,S(this).val()):n)?t="":"number"==typeof t?t+="":Array.isArray(t)&&(t=S.map(t,function(e){return null==e?"":e+""})),(r=S.valHooks[this.type]||S.valHooks[this.nodeName.toLowerCase()])&&"set"in r&&void 0!==r.set(this,t,"value")||(this.value=t))})):t?(r=S.valHooks[t.type]||S.valHooks[t.nodeName.toLowerCase()])&&"get"in r&&void 0!==(e=r.get(t,"value"))?e:"string"==typeof(e=t.value)?e.replace(xt,""):null==e?"":e:void 0}}),S.extend({valHooks:{option:{get:function(e){var t=S.find.attr(e,"value");return null!=t?t:vt(S.text(e))}},select:{get:function(e){var t,n,r,i=e.options,o=e.selectedIndex,a="select-one"===e.type,s=a?null:[],u=a?o+1:i.length;for(r=o<0?u:a?o:0;r<u;r++)if(((n=i[r]).selected||r===o)&&!n.disabled&&(!n.parentNode.disabled||!A(n.parentNode,"optgroup"))){if(t=S(n).val(),a)return t;s.push(t)}return s},set:function(e,t){var n,r,i=e.options,o=S.makeArray(t),a=i.length;while(a--)((r=i[a]).selected=-1<S.inArray(S.valHooks.option.get(r),o))&&(n=!0);return n||(e.selectedIndex=-1),o}}}}),S.each(["radio","checkbox"],function(){S.valHooks[this]={set:function(e,t){if(Array.isArray(t))return e.checked=-1<S.inArray(S(e).val(),t)}},y.checkOn||(S.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})}),y.focusin="onfocusin"in C;var bt=/^(?:focusinfocus|focusoutblur)$/,wt=function(e){e.stopPropagation()};S.extend(S.event,{trigger:function(e,t,n,r){var i,o,a,s,u,l,c,f,p=[n||E],d=v.call(e,"type")?e.type:e,h=v.call(e,"namespace")?e.namespace.split("."):[];if(o=f=a=n=n||E,3!==n.nodeType&&8!==n.nodeType&&!bt.test(d+S.event.triggered)&&(-1<d.indexOf(".")&&(d=(h=d.split(".")).shift(),h.sort()),u=d.indexOf(":")<0&&"on"+d,(e=e[S.expando]?e:new S.Event(d,"object"==typeof e&&e)).isTrigger=r?2:3,e.namespace=h.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+h.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=n),t=null==t?[e]:S.makeArray(t,[e]),c=S.event.special[d]||{},r||!c.trigger||!1!==c.trigger.apply(n,t))){if(!r&&!c.noBubble&&!x(n)){for(s=c.delegateType||d,bt.test(s+d)||(o=o.parentNode);o;o=o.parentNode)p.push(o),a=o;a===(n.ownerDocument||E)&&p.push(a.defaultView||a.parentWindow||C)}i=0;while((o=p[i++])&&!e.isPropagationStopped())f=o,e.type=1<i?s:c.bindType||d,(l=(Y.get(o,"events")||Object.create(null))[e.type]&&Y.get(o,"handle"))&&l.apply(o,t),(l=u&&o[u])&&l.apply&&V(o)&&(e.result=l.apply(o,t),!1===e.result&&e.preventDefault());return e.type=d,r||e.isDefaultPrevented()||c._default&&!1!==c._default.apply(p.pop(),t)||!V(n)||u&&m(n[d])&&!x(n)&&((a=n[u])&&(n[u]=null),S.event.triggered=d,e.isPropagationStopped()&&f.addEventListener(d,wt),n[d](),e.isPropagationStopped()&&f.removeEventListener(d,wt),S.event.triggered=void 0,a&&(n[u]=a)),e.result}},simulate:function(e,t,n){var r=S.extend(new S.Event,n,{type:e,isSimulated:!0});S.event.trigger(r,null,t)}}),S.fn.extend({trigger:function(e,t){return this.each(function(){S.event.trigger(e,t,this)})},triggerHandler:function(e,t){var n=this[0];if(n)return S.event.trigger(e,t,n,!0)}}),y.focusin||S.each({focus:"focusin",blur:"focusout"},function(n,r){var i=function(e){S.event.simulate(r,e.target,S.event.fix(e))};S.event.special[r]={setup:function(){var e=this.ownerDocument||this.document||this,t=Y.access(e,r);t||e.addEventListener(n,i,!0),Y.access(e,r,(t||0)+1)},teardown:function(){var e=this.ownerDocument||this.document||this,t=Y.access(e,r)-1;t?Y.access(e,r,t):(e.removeEventListener(n,i,!0),Y.remove(e,r))}}});var Tt=C.location,Ct={guid:Date.now()},Et=/\?/;S.parseXML=function(e){var t;if(!e||"string"!=typeof e)return null;try{t=(new C.DOMParser).parseFromString(e,"text/xml")}catch(e){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||S.error("Invalid XML: "+e),t};var St=/\[\]$/,kt=/\r?\n/g,At=/^(?:submit|button|image|reset|file)$/i,Nt=/^(?:input|select|textarea|keygen)/i;function Dt(n,e,r,i){var t;if(Array.isArray(e))S.each(e,function(e,t){r||St.test(n)?i(n,t):Dt(n+"["+("object"==typeof t&&null!=t?e:"")+"]",t,r,i)});else if(r||"object"!==w(e))i(n,e);else for(t in e)Dt(n+"["+t+"]",e[t],r,i)}S.param=function(e,t){var n,r=[],i=function(e,t){var n=m(t)?t():t;r[r.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==n?"":n)};if(null==e)return"";if(Array.isArray(e)||e.jquery&&!S.isPlainObject(e))S.each(e,function(){i(this.name,this.value)});else for(n in e)Dt(n,e[n],t,i);return r.join("&")},S.fn.extend({serialize:function(){return S.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var e=S.prop(this,"elements");return e?S.makeArray(e):this}).filter(function(){var e=this.type;return this.name&&!S(this).is(":disabled")&&Nt.test(this.nodeName)&&!At.test(e)&&(this.checked||!pe.test(e))}).map(function(e,t){var n=S(this).val();return null==n?null:Array.isArray(n)?S.map(n,function(e){return{name:t.name,value:e.replace(kt,"\r\n")}}):{name:t.name,value:n.replace(kt,"\r\n")}}).get()}});var jt=/%20/g,qt=/#.*$/,Lt=/([?&])_=[^&]*/,Ht=/^(.*?):[ \t]*([^\r\n]*)$/gm,Ot=/^(?:GET|HEAD)$/,Pt=/^\/\//,Rt={},Mt={},It="*/".concat("*"),Wt=E.createElement("a");function Ft(o){return function(e,t){"string"!=typeof e&&(t=e,e="*");var n,r=0,i=e.toLowerCase().match(P)||[];if(m(t))while(n=i[r++])"+"===n[0]?(n=n.slice(1)||"*",(o[n]=o[n]||[]).unshift(t)):(o[n]=o[n]||[]).push(t)}}function Bt(t,i,o,a){var s={},u=t===Mt;function l(e){var r;return s[e]=!0,S.each(t[e]||[],function(e,t){var n=t(i,o,a);return"string"!=typeof n||u||s[n]?u?!(r=n):void 0:(i.dataTypes.unshift(n),l(n),!1)}),r}return l(i.dataTypes[0])||!s["*"]&&l("*")}function $t(e,t){var n,r,i=S.ajaxSettings.flatOptions||{};for(n in t)void 0!==t[n]&&((i[n]?e:r||(r={}))[n]=t[n]);return r&&S.extend(!0,e,r),e}Wt.href=Tt.href,S.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:Tt.href,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(Tt.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":It,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":S.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?$t($t(e,S.ajaxSettings),t):$t(S.ajaxSettings,e)},ajaxPrefilter:Ft(Rt),ajaxTransport:Ft(Mt),ajax:function(e,t){"object"==typeof e&&(t=e,e=void 0),t=t||{};var c,f,p,n,d,r,h,g,i,o,v=S.ajaxSetup({},t),y=v.context||v,m=v.context&&(y.nodeType||y.jquery)?S(y):S.event,x=S.Deferred(),b=S.Callbacks("once memory"),w=v.statusCode||{},a={},s={},u="canceled",T={readyState:0,getResponseHeader:function(e){var t;if(h){if(!n){n={};while(t=Ht.exec(p))n[t[1].toLowerCase()+" "]=(n[t[1].toLowerCase()+" "]||[]).concat(t[2])}t=n[e.toLowerCase()+" "]}return null==t?null:t.join(", ")},getAllResponseHeaders:function(){return h?p:null},setRequestHeader:function(e,t){return null==h&&(e=s[e.toLowerCase()]=s[e.toLowerCase()]||e,a[e]=t),this},overrideMimeType:function(e){return null==h&&(v.mimeType=e),this},statusCode:function(e){var t;if(e)if(h)T.always(e[T.status]);else for(t in e)w[t]=[w[t],e[t]];return this},abort:function(e){var t=e||u;return c&&c.abort(t),l(0,t),this}};if(x.promise(T),v.url=((e||v.url||Tt.href)+"").replace(Pt,Tt.protocol+"//"),v.type=t.method||t.type||v.method||v.type,v.dataTypes=(v.dataType||"*").toLowerCase().match(P)||[""],null==v.crossDomain){r=E.createElement("a");try{r.href=v.url,r.href=r.href,v.crossDomain=Wt.protocol+"//"+Wt.host!=r.protocol+"//"+r.host}catch(e){v.crossDomain=!0}}if(v.data&&v.processData&&"string"!=typeof v.data&&(v.data=S.param(v.data,v.traditional)),Bt(Rt,v,t,T),h)return T;for(i in(g=S.event&&v.global)&&0==S.active++&&S.event.trigger("ajaxStart"),v.type=v.type.toUpperCase(),v.hasContent=!Ot.test(v.type),f=v.url.replace(qt,""),v.hasContent?v.data&&v.processData&&0===(v.contentType||"").indexOf("application/x-www-form-urlencoded")&&(v.data=v.data.replace(jt,"+")):(o=v.url.slice(f.length),v.data&&(v.processData||"string"==typeof v.data)&&(f+=(Et.test(f)?"&":"?")+v.data,delete v.data),!1===v.cache&&(f=f.replace(Lt,"$1"),o=(Et.test(f)?"&":"?")+"_="+Ct.guid+++o),v.url=f+o),v.ifModified&&(S.lastModified[f]&&T.setRequestHeader("If-Modified-Since",S.lastModified[f]),S.etag[f]&&T.setRequestHeader("If-None-Match",S.etag[f])),(v.data&&v.hasContent&&!1!==v.contentType||t.contentType)&&T.setRequestHeader("Content-Type",v.contentType),T.setRequestHeader("Accept",v.dataTypes[0]&&v.accepts[v.dataTypes[0]]?v.accepts[v.dataTypes[0]]+("*"!==v.dataTypes[0]?", "+It+"; q=0.01":""):v.accepts["*"]),v.headers)T.setRequestHeader(i,v.headers[i]);if(v.beforeSend&&(!1===v.beforeSend.call(y,T,v)||h))return T.abort();if(u="abort",b.add(v.complete),T.done(v.success),T.fail(v.error),c=Bt(Mt,v,t,T)){if(T.readyState=1,g&&m.trigger("ajaxSend",[T,v]),h)return T;v.async&&0<v.timeout&&(d=C.setTimeout(function(){T.abort("timeout")},v.timeout));try{h=!1,c.send(a,l)}catch(e){if(h)throw e;l(-1,e)}}else l(-1,"No Transport");function l(e,t,n,r){var i,o,a,s,u,l=t;h||(h=!0,d&&C.clearTimeout(d),c=void 0,p=r||"",T.readyState=0<e?4:0,i=200<=e&&e<300||304===e,n&&(s=function(e,t,n){var r,i,o,a,s=e.contents,u=e.dataTypes;while("*"===u[0])u.shift(),void 0===r&&(r=e.mimeType||t.getResponseHeader("Content-Type"));if(r)for(i in s)if(s[i]&&s[i].test(r)){u.unshift(i);break}if(u[0]in n)o=u[0];else{for(i in n){if(!u[0]||e.converters[i+" "+u[0]]){o=i;break}a||(a=i)}o=o||a}if(o)return o!==u[0]&&u.unshift(o),n[o]}(v,T,n)),!i&&-1<S.inArray("script",v.dataTypes)&&(v.converters["text script"]=function(){}),s=function(e,t,n,r){var i,o,a,s,u,l={},c=e.dataTypes.slice();if(c[1])for(a in e.converters)l[a.toLowerCase()]=e.converters[a];o=c.shift();while(o)if(e.responseFields[o]&&(n[e.responseFields[o]]=t),!u&&r&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),u=o,o=c.shift())if("*"===o)o=u;else if("*"!==u&&u!==o){if(!(a=l[u+" "+o]||l["* "+o]))for(i in l)if((s=i.split(" "))[1]===o&&(a=l[u+" "+s[0]]||l["* "+s[0]])){!0===a?a=l[i]:!0!==l[i]&&(o=s[0],c.unshift(s[1]));break}if(!0!==a)if(a&&e["throws"])t=a(t);else try{t=a(t)}catch(e){return{state:"parsererror",error:a?e:"No conversion from "+u+" to "+o}}}return{state:"success",data:t}}(v,s,T,i),i?(v.ifModified&&((u=T.getResponseHeader("Last-Modified"))&&(S.lastModified[f]=u),(u=T.getResponseHeader("etag"))&&(S.etag[f]=u)),204===e||"HEAD"===v.type?l="nocontent":304===e?l="notmodified":(l=s.state,o=s.data,i=!(a=s.error))):(a=l,!e&&l||(l="error",e<0&&(e=0))),T.status=e,T.statusText=(t||l)+"",i?x.resolveWith(y,[o,l,T]):x.rejectWith(y,[T,l,a]),T.statusCode(w),w=void 0,g&&m.trigger(i?"ajaxSuccess":"ajaxError",[T,v,i?o:a]),b.fireWith(y,[T,l]),g&&(m.trigger("ajaxComplete",[T,v]),--S.active||S.event.trigger("ajaxStop")))}return T},getJSON:function(e,t,n){return S.get(e,t,n,"json")},getScript:function(e,t){return S.get(e,void 0,t,"script")}}),S.each(["get","post"],function(e,i){S[i]=function(e,t,n,r){return m(t)&&(r=r||n,n=t,t=void 0),S.ajax(S.extend({url:e,type:i,dataType:r,data:t,success:n},S.isPlainObject(e)&&e))}}),S.ajaxPrefilter(function(e){var t;for(t in e.headers)"content-type"===t.toLowerCase()&&(e.contentType=e.headers[t]||"")}),S._evalUrl=function(e,t,n){return S.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(e){S.globalEval(e,t,n)}})},S.fn.extend({wrapAll:function(e){var t;return this[0]&&(m(e)&&(e=e.call(this[0])),t=S(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map(function(){var e=this;while(e.firstElementChild)e=e.firstElementChild;return e}).append(this)),this},wrapInner:function(n){return m(n)?this.each(function(e){S(this).wrapInner(n.call(this,e))}):this.each(function(){var e=S(this),t=e.contents();t.length?t.wrapAll(n):e.append(n)})},wrap:function(t){var n=m(t);return this.each(function(e){S(this).wrapAll(n?t.call(this,e):t)})},unwrap:function(e){return this.parent(e).not("body").each(function(){S(this).replaceWith(this.childNodes)}),this}}),S.expr.pseudos.hidden=function(e){return!S.expr.pseudos.visible(e)},S.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},S.ajaxSettings.xhr=function(){try{return new C.XMLHttpRequest}catch(e){}};var _t={0:200,1223:204},zt=S.ajaxSettings.xhr();y.cors=!!zt&&"withCredentials"in zt,y.ajax=zt=!!zt,S.ajaxTransport(function(i){var o,a;if(y.cors||zt&&!i.crossDomain)return{send:function(e,t){var n,r=i.xhr();if(r.open(i.type,i.url,i.async,i.username,i.password),i.xhrFields)for(n in i.xhrFields)r[n]=i.xhrFields[n];for(n in i.mimeType&&r.overrideMimeType&&r.overrideMimeType(i.mimeType),i.crossDomain||e["X-Requested-With"]||(e["X-Requested-With"]="XMLHttpRequest"),e)r.setRequestHeader(n,e[n]);o=function(e){return function(){o&&(o=a=r.onload=r.onerror=r.onabort=r.ontimeout=r.onreadystatechange=null,"abort"===e?r.abort():"error"===e?"number"!=typeof r.status?t(0,"error"):t(r.status,r.statusText):t(_t[r.status]||r.status,r.statusText,"text"!==(r.responseType||"text")||"string"!=typeof r.responseText?{binary:r.response}:{text:r.responseText},r.getAllResponseHeaders()))}},r.onload=o(),a=r.onerror=r.ontimeout=o("error"),void 0!==r.onabort?r.onabort=a:r.onreadystatechange=function(){4===r.readyState&&C.setTimeout(function(){o&&a()})},o=o("abort");try{r.send(i.hasContent&&i.data||null)}catch(e){if(o)throw e}},abort:function(){o&&o()}}}),S.ajaxPrefilter(function(e){e.crossDomain&&(e.contents.script=!1)}),S.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return S.globalEval(e),e}}}),S.ajaxPrefilter("script",function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")}),S.ajaxTransport("script",function(n){var r,i;if(n.crossDomain||n.scriptAttrs)return{send:function(e,t){r=S("<script>").attr(n.scriptAttrs||{}).prop({charset:n.scriptCharset,src:n.url}).on("load error",i=function(e){r.remove(),i=null,e&&t("error"===e.type?404:200,e.type)}),E.head.appendChild(r[0])},abort:function(){i&&i()}}});var Ut,Xt=[],Vt=/(=)\?(?=&|$)|\?\?/;S.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=Xt.pop()||S.expando+"_"+Ct.guid++;return this[e]=!0,e}}),S.ajaxPrefilter("json jsonp",function(e,t,n){var r,i,o,a=!1!==e.jsonp&&(Vt.test(e.url)?"url":"string"==typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Vt.test(e.data)&&"data");if(a||"jsonp"===e.dataTypes[0])return r=e.jsonpCallback=m(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,a?e[a]=e[a].replace(Vt,"$1"+r):!1!==e.jsonp&&(e.url+=(Et.test(e.url)?"&":"?")+e.jsonp+"="+r),e.converters["script json"]=function(){return o||S.error(r+" was not called"),o[0]},e.dataTypes[0]="json",i=C[r],C[r]=function(){o=arguments},n.always(function(){void 0===i?S(C).removeProp(r):C[r]=i,e[r]&&(e.jsonpCallback=t.jsonpCallback,Xt.push(r)),o&&m(i)&&i(o[0]),o=i=void 0}),"script"}),y.createHTMLDocument=((Ut=E.implementation.createHTMLDocument("").body).innerHTML="<form></form><form></form>",2===Ut.childNodes.length),S.parseHTML=function(e,t,n){return"string"!=typeof e?[]:("boolean"==typeof t&&(n=t,t=!1),t||(y.createHTMLDocument?((r=(t=E.implementation.createHTMLDocument("")).createElement("base")).href=E.location.href,t.head.appendChild(r)):t=E),o=!n&&[],(i=N.exec(e))?[t.createElement(i[1])]:(i=xe([e],t,o),o&&o.length&&S(o).remove(),S.merge([],i.childNodes)));var r,i,o},S.fn.load=function(e,t,n){var r,i,o,a=this,s=e.indexOf(" ");return-1<s&&(r=vt(e.slice(s)),e=e.slice(0,s)),m(t)?(n=t,t=void 0):t&&"object"==typeof t&&(i="POST"),0<a.length&&S.ajax({url:e,type:i||"GET",dataType:"html",data:t}).done(function(e){o=arguments,a.html(r?S("<div>").append(S.parseHTML(e)).find(r):e)}).always(n&&function(e,t){a.each(function(){n.apply(this,o||[e.responseText,t,e])})}),this},S.expr.pseudos.animated=function(t){return S.grep(S.timers,function(e){return t===e.elem}).length},S.offset={setOffset:function(e,t,n){var r,i,o,a,s,u,l=S.css(e,"position"),c=S(e),f={};"static"===l&&(e.style.position="relative"),s=c.offset(),o=S.css(e,"top"),u=S.css(e,"left"),("absolute"===l||"fixed"===l)&&-1<(o+u).indexOf("auto")?(a=(r=c.position()).top,i=r.left):(a=parseFloat(o)||0,i=parseFloat(u)||0),m(t)&&(t=t.call(e,n,S.extend({},s))),null!=t.top&&(f.top=t.top-s.top+a),null!=t.left&&(f.left=t.left-s.left+i),"using"in t?t.using.call(e,f):("number"==typeof f.top&&(f.top+="px"),"number"==typeof f.left&&(f.left+="px"),c.css(f))}},S.fn.extend({offset:function(t){if(arguments.length)return void 0===t?this:this.each(function(e){S.offset.setOffset(this,t,e)});var e,n,r=this[0];return r?r.getClientRects().length?(e=r.getBoundingClientRect(),n=r.ownerDocument.defaultView,{top:e.top+n.pageYOffset,left:e.left+n.pageXOffset}):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,n,r=this[0],i={top:0,left:0};if("fixed"===S.css(r,"position"))t=r.getBoundingClientRect();else{t=this.offset(),n=r.ownerDocument,e=r.offsetParent||n.documentElement;while(e&&(e===n.body||e===n.documentElement)&&"static"===S.css(e,"position"))e=e.parentNode;e&&e!==r&&1===e.nodeType&&((i=S(e).offset()).top+=S.css(e,"borderTopWidth",!0),i.left+=S.css(e,"borderLeftWidth",!0))}return{top:t.top-i.top-S.css(r,"marginTop",!0),left:t.left-i.left-S.css(r,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){var e=this.offsetParent;while(e&&"static"===S.css(e,"position"))e=e.offsetParent;return e||re})}}),S.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(t,i){var o="pageYOffset"===i;S.fn[t]=function(e){return $(this,function(e,t,n){var r;if(x(e)?r=e:9===e.nodeType&&(r=e.defaultView),void 0===n)return r?r[i]:e[t];r?r.scrollTo(o?r.pageXOffset:n,o?n:r.pageYOffset):e[t]=n},t,e,arguments.length)}}),S.each(["top","left"],function(e,n){S.cssHooks[n]=$e(y.pixelPosition,function(e,t){if(t)return t=Be(e,n),Me.test(t)?S(e).position()[n]+"px":t})}),S.each({Height:"height",Width:"width"},function(a,s){S.each({padding:"inner"+a,content:s,"":"outer"+a},function(r,o){S.fn[o]=function(e,t){var n=arguments.length&&(r||"boolean"!=typeof e),i=r||(!0===e||!0===t?"margin":"border");return $(this,function(e,t,n){var r;return x(e)?0===o.indexOf("outer")?e["inner"+a]:e.document.documentElement["client"+a]:9===e.nodeType?(r=e.documentElement,Math.max(e.body["scroll"+a],r["scroll"+a],e.body["offset"+a],r["offset"+a],r["client"+a])):void 0===n?S.css(e,t,i):S.style(e,t,n,i)},s,n?e:void 0,n)}})}),S.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(e,t){S.fn[t]=function(e){return this.on(t,e)}}),S.fn.extend({bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,r){return this.on(t,e,n,r)},undelegate:function(e,t,n){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",n)},hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),S.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),function(e,n){S.fn[n]=function(e,t){return 0<arguments.length?this.on(n,null,e,t):this.trigger(n)}});var Gt=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;S.proxy=function(e,t){var n,r,i;if("string"==typeof t&&(n=e[t],t=e,e=n),m(e))return r=s.call(arguments,2),(i=function(){return e.apply(t||this,r.concat(s.call(arguments)))}).guid=e.guid=e.guid||S.guid++,i},S.holdReady=function(e){e?S.readyWait++:S.ready(!0)},S.isArray=Array.isArray,S.parseJSON=JSON.parse,S.nodeName=A,S.isFunction=m,S.isWindow=x,S.camelCase=X,S.type=w,S.now=Date.now,S.isNumeric=function(e){var t=S.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},S.trim=function(e){return null==e?"":(e+"").replace(Gt,"")},"function"==typeof define&&define.amd&&define("jquery",[],function(){return S});var Yt=C.jQuery,Qt=C.$;return S.noConflict=function(e){return C.$===S&&(C.$=Qt),e&&C.jQuery===S&&(C.jQuery=Yt),S},"undefined"==typeof e&&(C.jQuery=C.$=S),S});

//     Underscore.js 1.7.0
//     http://underscorejs.org
//     (c) 2009-2014 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.
(function(){var n=this,t=n._,r=Array.prototype,e=Object.prototype,u=Function.prototype,i=r.push,a=r.slice,o=r.concat,l=e.toString,c=e.hasOwnProperty,f=Array.isArray,s=Object.keys,p=u.bind,h=function(n){return n instanceof h?n:this instanceof h?void(this._wrapped=n):new h(n)};"undefined"!=typeof exports?("undefined"!=typeof module&&module.exports&&(exports=module.exports=h),exports._=h):n._=h,h.VERSION="1.7.0";var g=function(n,t,r){if(t===void 0)return n;switch(null==r?3:r){case 1:return function(r){return n.call(t,r)};case 2:return function(r,e){return n.call(t,r,e)};case 3:return function(r,e,u){return n.call(t,r,e,u)};case 4:return function(r,e,u,i){return n.call(t,r,e,u,i)}}return function(){return n.apply(t,arguments)}};h.iteratee=function(n,t,r){return null==n?h.identity:h.isFunction(n)?g(n,t,r):h.isObject(n)?h.matches(n):h.property(n)},h.each=h.forEach=function(n,t,r){if(null==n)return n;t=g(t,r);var e,u=n.length;if(u===+u)for(e=0;u>e;e++)t(n[e],e,n);else{var i=h.keys(n);for(e=0,u=i.length;u>e;e++)t(n[i[e]],i[e],n)}return n},h.map=h.collect=function(n,t,r){if(null==n)return[];t=h.iteratee(t,r);for(var e,u=n.length!==+n.length&&h.keys(n),i=(u||n).length,a=Array(i),o=0;i>o;o++)e=u?u[o]:o,a[o]=t(n[e],e,n);return a};var v="Reduce of empty array with no initial value";h.reduce=h.foldl=h.inject=function(n,t,r,e){null==n&&(n=[]),t=g(t,e,4);var u,i=n.length!==+n.length&&h.keys(n),a=(i||n).length,o=0;if(arguments.length<3){if(!a)throw new TypeError(v);r=n[i?i[o++]:o++]}for(;a>o;o++)u=i?i[o]:o,r=t(r,n[u],u,n);return r},h.reduceRight=h.foldr=function(n,t,r,e){null==n&&(n=[]),t=g(t,e,4);var u,i=n.length!==+n.length&&h.keys(n),a=(i||n).length;if(arguments.length<3){if(!a)throw new TypeError(v);r=n[i?i[--a]:--a]}for(;a--;)u=i?i[a]:a,r=t(r,n[u],u,n);return r},h.find=h.detect=function(n,t,r){var e;return t=h.iteratee(t,r),h.some(n,function(n,r,u){return t(n,r,u)?(e=n,!0):void 0}),e},h.filter=h.select=function(n,t,r){var e=[];return null==n?e:(t=h.iteratee(t,r),h.each(n,function(n,r,u){t(n,r,u)&&e.push(n)}),e)},h.reject=function(n,t,r){return h.filter(n,h.negate(h.iteratee(t)),r)},h.every=h.all=function(n,t,r){if(null==n)return!0;t=h.iteratee(t,r);var e,u,i=n.length!==+n.length&&h.keys(n),a=(i||n).length;for(e=0;a>e;e++)if(u=i?i[e]:e,!t(n[u],u,n))return!1;return!0},h.some=h.any=function(n,t,r){if(null==n)return!1;t=h.iteratee(t,r);var e,u,i=n.length!==+n.length&&h.keys(n),a=(i||n).length;for(e=0;a>e;e++)if(u=i?i[e]:e,t(n[u],u,n))return!0;return!1},h.contains=h.include=function(n,t){return null==n?!1:(n.length!==+n.length&&(n=h.values(n)),h.indexOf(n,t)>=0)},h.invoke=function(n,t){var r=a.call(arguments,2),e=h.isFunction(t);return h.map(n,function(n){return(e?t:n[t]).apply(n,r)})},h.pluck=function(n,t){return h.map(n,h.property(t))},h.where=function(n,t){return h.filter(n,h.matches(t))},h.findWhere=function(n,t){return h.find(n,h.matches(t))},h.max=function(n,t,r){var e,u,i=-1/0,a=-1/0;if(null==t&&null!=n){n=n.length===+n.length?n:h.values(n);for(var o=0,l=n.length;l>o;o++)e=n[o],e>i&&(i=e)}else t=h.iteratee(t,r),h.each(n,function(n,r,e){u=t(n,r,e),(u>a||u===-1/0&&i===-1/0)&&(i=n,a=u)});return i},h.min=function(n,t,r){var e,u,i=1/0,a=1/0;if(null==t&&null!=n){n=n.length===+n.length?n:h.values(n);for(var o=0,l=n.length;l>o;o++)e=n[o],i>e&&(i=e)}else t=h.iteratee(t,r),h.each(n,function(n,r,e){u=t(n,r,e),(a>u||1/0===u&&1/0===i)&&(i=n,a=u)});return i},h.shuffle=function(n){for(var t,r=n&&n.length===+n.length?n:h.values(n),e=r.length,u=Array(e),i=0;e>i;i++)t=h.random(0,i),t!==i&&(u[i]=u[t]),u[t]=r[i];return u},h.sample=function(n,t,r){return null==t||r?(n.length!==+n.length&&(n=h.values(n)),n[h.random(n.length-1)]):h.shuffle(n).slice(0,Math.max(0,t))},h.sortBy=function(n,t,r){return t=h.iteratee(t,r),h.pluck(h.map(n,function(n,r,e){return{value:n,index:r,criteria:t(n,r,e)}}).sort(function(n,t){var r=n.criteria,e=t.criteria;if(r!==e){if(r>e||r===void 0)return 1;if(e>r||e===void 0)return-1}return n.index-t.index}),"value")};var m=function(n){return function(t,r,e){var u={};return r=h.iteratee(r,e),h.each(t,function(e,i){var a=r(e,i,t);n(u,e,a)}),u}};h.groupBy=m(function(n,t,r){h.has(n,r)?n[r].push(t):n[r]=[t]}),h.indexBy=m(function(n,t,r){n[r]=t}),h.countBy=m(function(n,t,r){h.has(n,r)?n[r]++:n[r]=1}),h.sortedIndex=function(n,t,r,e){r=h.iteratee(r,e,1);for(var u=r(t),i=0,a=n.length;a>i;){var o=i+a>>>1;r(n[o])<u?i=o+1:a=o}return i},h.toArray=function(n){return n?h.isArray(n)?a.call(n):n.length===+n.length?h.map(n,h.identity):h.values(n):[]},h.size=function(n){return null==n?0:n.length===+n.length?n.length:h.keys(n).length},h.partition=function(n,t,r){t=h.iteratee(t,r);var e=[],u=[];return h.each(n,function(n,r,i){(t(n,r,i)?e:u).push(n)}),[e,u]},h.first=h.head=h.take=function(n,t,r){return null==n?void 0:null==t||r?n[0]:0>t?[]:a.call(n,0,t)},h.initial=function(n,t,r){return a.call(n,0,Math.max(0,n.length-(null==t||r?1:t)))},h.last=function(n,t,r){return null==n?void 0:null==t||r?n[n.length-1]:a.call(n,Math.max(n.length-t,0))},h.rest=h.tail=h.drop=function(n,t,r){return a.call(n,null==t||r?1:t)},h.compact=function(n){return h.filter(n,h.identity)};var y=function(n,t,r,e){if(t&&h.every(n,h.isArray))return o.apply(e,n);for(var u=0,a=n.length;a>u;u++){var l=n[u];h.isArray(l)||h.isArguments(l)?t?i.apply(e,l):y(l,t,r,e):r||e.push(l)}return e};h.flatten=function(n,t){return y(n,t,!1,[])},h.without=function(n){return h.difference(n,a.call(arguments,1))},h.uniq=h.unique=function(n,t,r,e){if(null==n)return[];h.isBoolean(t)||(e=r,r=t,t=!1),null!=r&&(r=h.iteratee(r,e));for(var u=[],i=[],a=0,o=n.length;o>a;a++){var l=n[a];if(t)a&&i===l||u.push(l),i=l;else if(r){var c=r(l,a,n);h.indexOf(i,c)<0&&(i.push(c),u.push(l))}else h.indexOf(u,l)<0&&u.push(l)}return u},h.union=function(){return h.uniq(y(arguments,!0,!0,[]))},h.intersection=function(n){if(null==n)return[];for(var t=[],r=arguments.length,e=0,u=n.length;u>e;e++){var i=n[e];if(!h.contains(t,i)){for(var a=1;r>a&&h.contains(arguments[a],i);a++);a===r&&t.push(i)}}return t},h.difference=function(n){var t=y(a.call(arguments,1),!0,!0,[]);return h.filter(n,function(n){return!h.contains(t,n)})},h.zip=function(n){if(null==n)return[];for(var t=h.max(arguments,"length").length,r=Array(t),e=0;t>e;e++)r[e]=h.pluck(arguments,e);return r},h.object=function(n,t){if(null==n)return{};for(var r={},e=0,u=n.length;u>e;e++)t?r[n[e]]=t[e]:r[n[e][0]]=n[e][1];return r},h.indexOf=function(n,t,r){if(null==n)return-1;var e=0,u=n.length;if(r){if("number"!=typeof r)return e=h.sortedIndex(n,t),n[e]===t?e:-1;e=0>r?Math.max(0,u+r):r}for(;u>e;e++)if(n[e]===t)return e;return-1},h.lastIndexOf=function(n,t,r){if(null==n)return-1;var e=n.length;for("number"==typeof r&&(e=0>r?e+r+1:Math.min(e,r+1));--e>=0;)if(n[e]===t)return e;return-1},h.range=function(n,t,r){arguments.length<=1&&(t=n||0,n=0),r=r||1;for(var e=Math.max(Math.ceil((t-n)/r),0),u=Array(e),i=0;e>i;i++,n+=r)u[i]=n;return u};var d=function(){};h.bind=function(n,t){var r,e;if(p&&n.bind===p)return p.apply(n,a.call(arguments,1));if(!h.isFunction(n))throw new TypeError("Bind must be called on a function");return r=a.call(arguments,2),e=function(){if(!(this instanceof e))return n.apply(t,r.concat(a.call(arguments)));d.prototype=n.prototype;var u=new d;d.prototype=null;var i=n.apply(u,r.concat(a.call(arguments)));return h.isObject(i)?i:u}},h.partial=function(n){var t=a.call(arguments,1);return function(){for(var r=0,e=t.slice(),u=0,i=e.length;i>u;u++)e[u]===h&&(e[u]=arguments[r++]);for(;r<arguments.length;)e.push(arguments[r++]);return n.apply(this,e)}},h.bindAll=function(n){var t,r,e=arguments.length;if(1>=e)throw new Error("bindAll must be passed function names");for(t=1;e>t;t++)r=arguments[t],n[r]=h.bind(n[r],n);return n},h.memoize=function(n,t){var r=function(e){var u=r.cache,i=t?t.apply(this,arguments):e;return h.has(u,i)||(u[i]=n.apply(this,arguments)),u[i]};return r.cache={},r},h.delay=function(n,t){var r=a.call(arguments,2);return setTimeout(function(){return n.apply(null,r)},t)},h.defer=function(n){return h.delay.apply(h,[n,1].concat(a.call(arguments,1)))},h.throttle=function(n,t,r){var e,u,i,a=null,o=0;r||(r={});var l=function(){o=r.leading===!1?0:h.now(),a=null,i=n.apply(e,u),a||(e=u=null)};return function(){var c=h.now();o||r.leading!==!1||(o=c);var f=t-(c-o);return e=this,u=arguments,0>=f||f>t?(clearTimeout(a),a=null,o=c,i=n.apply(e,u),a||(e=u=null)):a||r.trailing===!1||(a=setTimeout(l,f)),i}},h.debounce=function(n,t,r){var e,u,i,a,o,l=function(){var c=h.now()-a;t>c&&c>0?e=setTimeout(l,t-c):(e=null,r||(o=n.apply(i,u),e||(i=u=null)))};return function(){i=this,u=arguments,a=h.now();var c=r&&!e;return e||(e=setTimeout(l,t)),c&&(o=n.apply(i,u),i=u=null),o}},h.wrap=function(n,t){return h.partial(t,n)},h.negate=function(n){return function(){return!n.apply(this,arguments)}},h.compose=function(){var n=arguments,t=n.length-1;return function(){for(var r=t,e=n[t].apply(this,arguments);r--;)e=n[r].call(this,e);return e}},h.after=function(n,t){return function(){return--n<1?t.apply(this,arguments):void 0}},h.before=function(n,t){var r;return function(){return--n>0?r=t.apply(this,arguments):t=null,r}},h.once=h.partial(h.before,2),h.keys=function(n){if(!h.isObject(n))return[];if(s)return s(n);var t=[];for(var r in n)h.has(n,r)&&t.push(r);return t},h.values=function(n){for(var t=h.keys(n),r=t.length,e=Array(r),u=0;r>u;u++)e[u]=n[t[u]];return e},h.pairs=function(n){for(var t=h.keys(n),r=t.length,e=Array(r),u=0;r>u;u++)e[u]=[t[u],n[t[u]]];return e},h.invert=function(n){for(var t={},r=h.keys(n),e=0,u=r.length;u>e;e++)t[n[r[e]]]=r[e];return t},h.functions=h.methods=function(n){var t=[];for(var r in n)h.isFunction(n[r])&&t.push(r);return t.sort()},h.extend=function(n){if(!h.isObject(n))return n;for(var t,r,e=1,u=arguments.length;u>e;e++){t=arguments[e];for(r in t)c.call(t,r)&&(n[r]=t[r])}return n},h.pick=function(n,t,r){var e,u={};if(null==n)return u;if(h.isFunction(t)){t=g(t,r);for(e in n){var i=n[e];t(i,e,n)&&(u[e]=i)}}else{var l=o.apply([],a.call(arguments,1));n=new Object(n);for(var c=0,f=l.length;f>c;c++)e=l[c],e in n&&(u[e]=n[e])}return u},h.omit=function(n,t,r){if(h.isFunction(t))t=h.negate(t);else{var e=h.map(o.apply([],a.call(arguments,1)),String);t=function(n,t){return!h.contains(e,t)}}return h.pick(n,t,r)},h.defaults=function(n){if(!h.isObject(n))return n;for(var t=1,r=arguments.length;r>t;t++){var e=arguments[t];for(var u in e)n[u]===void 0&&(n[u]=e[u])}return n},h.clone=function(n){return h.isObject(n)?h.isArray(n)?n.slice():h.extend({},n):n},h.tap=function(n,t){return t(n),n};var b=function(n,t,r,e){if(n===t)return 0!==n||1/n===1/t;if(null==n||null==t)return n===t;n instanceof h&&(n=n._wrapped),t instanceof h&&(t=t._wrapped);var u=l.call(n);if(u!==l.call(t))return!1;switch(u){case"[object RegExp]":case"[object String]":return""+n==""+t;case"[object Number]":return+n!==+n?+t!==+t:0===+n?1/+n===1/t:+n===+t;case"[object Date]":case"[object Boolean]":return+n===+t}if("object"!=typeof n||"object"!=typeof t)return!1;for(var i=r.length;i--;)if(r[i]===n)return e[i]===t;var a=n.constructor,o=t.constructor;if(a!==o&&"constructor"in n&&"constructor"in t&&!(h.isFunction(a)&&a instanceof a&&h.isFunction(o)&&o instanceof o))return!1;r.push(n),e.push(t);var c,f;if("[object Array]"===u){if(c=n.length,f=c===t.length)for(;c--&&(f=b(n[c],t[c],r,e)););}else{var s,p=h.keys(n);if(c=p.length,f=h.keys(t).length===c)for(;c--&&(s=p[c],f=h.has(t,s)&&b(n[s],t[s],r,e)););}return r.pop(),e.pop(),f};h.isEqual=function(n,t){return b(n,t,[],[])},h.isEmpty=function(n){if(null==n)return!0;if(h.isArray(n)||h.isString(n)||h.isArguments(n))return 0===n.length;for(var t in n)if(h.has(n,t))return!1;return!0},h.isElement=function(n){return!(!n||1!==n.nodeType)},h.isArray=f||function(n){return"[object Array]"===l.call(n)},h.isObject=function(n){var t=typeof n;return"function"===t||"object"===t&&!!n},h.each(["Arguments","Function","String","Number","Date","RegExp"],function(n){h["is"+n]=function(t){return l.call(t)==="[object "+n+"]"}}),h.isArguments(arguments)||(h.isArguments=function(n){return h.has(n,"callee")}),"function"!=typeof/./&&(h.isFunction=function(n){return"function"==typeof n||!1}),h.isFinite=function(n){return isFinite(n)&&!isNaN(parseFloat(n))},h.isNaN=function(n){return h.isNumber(n)&&n!==+n},h.isBoolean=function(n){return n===!0||n===!1||"[object Boolean]"===l.call(n)},h.isNull=function(n){return null===n},h.isUndefined=function(n){return n===void 0},h.has=function(n,t){return null!=n&&c.call(n,t)},h.noConflict=function(){return n._=t,this},h.identity=function(n){return n},h.constant=function(n){return function(){return n}},h.noop=function(){},h.property=function(n){return function(t){return t[n]}},h.matches=function(n){var t=h.pairs(n),r=t.length;return function(n){if(null==n)return!r;n=new Object(n);for(var e=0;r>e;e++){var u=t[e],i=u[0];if(u[1]!==n[i]||!(i in n))return!1}return!0}},h.times=function(n,t,r){var e=Array(Math.max(0,n));t=g(t,r,1);for(var u=0;n>u;u++)e[u]=t(u);return e},h.random=function(n,t){return null==t&&(t=n,n=0),n+Math.floor(Math.random()*(t-n+1))},h.now=Date.now||function(){return(new Date).getTime()};var _={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","`":"&#x60;"},w=h.invert(_),j=function(n){var t=function(t){return n[t]},r="(?:"+h.keys(n).join("|")+")",e=RegExp(r),u=RegExp(r,"g");return function(n){return n=null==n?"":""+n,e.test(n)?n.replace(u,t):n}};h.escape=j(_),h.unescape=j(w),h.result=function(n,t){if(null==n)return void 0;var r=n[t];return h.isFunction(r)?n[t]():r};var x=0;h.uniqueId=function(n){var t=++x+"";return n?n+t:t},h.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g};var A=/(.)^/,k={"'":"'","\\":"\\","\r":"r","\n":"n","\u2028":"u2028","\u2029":"u2029"},O=/\\|'|\r|\n|\u2028|\u2029/g,F=function(n){return"\\"+k[n]};h.template=function(n,t,r){!t&&r&&(t=r),t=h.defaults({},t,h.templateSettings);var e=RegExp([(t.escape||A).source,(t.interpolate||A).source,(t.evaluate||A).source].join("|")+"|$","g"),u=0,i="__p+='";n.replace(e,function(t,r,e,a,o){return i+=n.slice(u,o).replace(O,F),u=o+t.length,r?i+="'+\n((__t=("+r+"))==null?'':_.escape(__t))+\n'":e?i+="'+\n((__t=("+e+"))==null?'':__t)+\n'":a&&(i+="';\n"+a+"\n__p+='"),t}),i+="';\n",t.variable||(i="with(obj||{}){\n"+i+"}\n"),i="var __t,__p='',__j=Array.prototype.join,"+"print=function(){__p+=__j.call(arguments,'');};\n"+i+"return __p;\n";try{var a=new Function(t.variable||"obj","_",i)}catch(o){throw o.source=i,o}var l=function(n){return a.call(this,n,h)},c=t.variable||"obj";return l.source="function("+c+"){\n"+i+"}",l},h.chain=function(n){var t=h(n);return t._chain=!0,t};var E=function(n){return this._chain?h(n).chain():n};h.mixin=function(n){h.each(h.functions(n),function(t){var r=h[t]=n[t];h.prototype[t]=function(){var n=[this._wrapped];return i.apply(n,arguments),E.call(this,r.apply(h,n))}})},h.mixin(h),h.each(["pop","push","reverse","shift","sort","splice","unshift"],function(n){var t=r[n];h.prototype[n]=function(){var r=this._wrapped;return t.apply(r,arguments),"shift"!==n&&"splice"!==n||0!==r.length||delete r[0],E.call(this,r)}}),h.each(["concat","join","slice"],function(n){var t=r[n];h.prototype[n]=function(){return E.call(this,t.apply(this._wrapped,arguments))}}),h.prototype.value=function(){return this._wrapped},"function"==typeof define&&define.amd&&define("underscore",[],function(){return h})}).call(this);
//# sourceMappingURL=underscore-min.map
//! moment.js
//! version : 2.15.1
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
!function(a,b){"object"==typeof exports&&"undefined"!=typeof module?module.exports=b():"function"==typeof define&&define.amd?define(b):a.moment=b()}(this,function(){"use strict";function a(){return je.apply(null,arguments)}
// This is done to register the method called with moment()
// without creating circular dependencies.
function b(a){je=a}function c(a){return a instanceof Array||"[object Array]"===Object.prototype.toString.call(a)}function d(a){
// IE8 will treat undefined and null as object if it wasn't for
// input != null
return null!=a&&"[object Object]"===Object.prototype.toString.call(a)}function e(a){var b;for(b in a)
// even if its not own property I'd still call it non-empty
return!1;return!0}function f(a){return a instanceof Date||"[object Date]"===Object.prototype.toString.call(a)}function g(a,b){var c,d=[];for(c=0;c<a.length;++c)d.push(b(a[c],c));return d}function h(a,b){return Object.prototype.hasOwnProperty.call(a,b)}function i(a,b){for(var c in b)h(b,c)&&(a[c]=b[c]);return h(b,"toString")&&(a.toString=b.toString),h(b,"valueOf")&&(a.valueOf=b.valueOf),a}function j(a,b,c,d){return qb(a,b,c,d,!0).utc()}function k(){
// We need to deep clone this object.
return{empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],meridiem:null}}function l(a){return null==a._pf&&(a._pf=k()),a._pf}function m(a){if(null==a._isValid){var b=l(a),c=ke.call(b.parsedDateParts,function(a){return null!=a}),d=!isNaN(a._d.getTime())&&b.overflow<0&&!b.empty&&!b.invalidMonth&&!b.invalidWeekday&&!b.nullInput&&!b.invalidFormat&&!b.userInvalidated&&(!b.meridiem||b.meridiem&&c);if(a._strict&&(d=d&&0===b.charsLeftOver&&0===b.unusedTokens.length&&void 0===b.bigHour),null!=Object.isFrozen&&Object.isFrozen(a))return d;a._isValid=d}return a._isValid}function n(a){var b=j(NaN);return null!=a?i(l(b),a):l(b).userInvalidated=!0,b}function o(a){return void 0===a}function p(a,b){var c,d,e;if(o(b._isAMomentObject)||(a._isAMomentObject=b._isAMomentObject),o(b._i)||(a._i=b._i),o(b._f)||(a._f=b._f),o(b._l)||(a._l=b._l),o(b._strict)||(a._strict=b._strict),o(b._tzm)||(a._tzm=b._tzm),o(b._isUTC)||(a._isUTC=b._isUTC),o(b._offset)||(a._offset=b._offset),o(b._pf)||(a._pf=l(b)),o(b._locale)||(a._locale=b._locale),le.length>0)for(c in le)d=le[c],e=b[d],o(e)||(a[d]=e);return a}
// Moment prototype object
function q(b){p(this,b),this._d=new Date(null!=b._d?b._d.getTime():NaN),me===!1&&(me=!0,a.updateOffset(this),me=!1)}function r(a){return a instanceof q||null!=a&&null!=a._isAMomentObject}function s(a){return 0>a?Math.ceil(a)||0:Math.floor(a)}function t(a){var b=+a,c=0;return 0!==b&&isFinite(b)&&(c=s(b)),c}
// compare two arrays, return the number of differences
function u(a,b,c){var d,e=Math.min(a.length,b.length),f=Math.abs(a.length-b.length),g=0;for(d=0;e>d;d++)(c&&a[d]!==b[d]||!c&&t(a[d])!==t(b[d]))&&g++;return g+f}function v(b){a.suppressDeprecationWarnings===!1&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+b)}function w(b,c){var d=!0;return i(function(){if(null!=a.deprecationHandler&&a.deprecationHandler(null,b),d){for(var e,f=[],g=0;g<arguments.length;g++){if(e="","object"==typeof arguments[g]){e+="\n["+g+"] ";for(var h in arguments[0])e+=h+": "+arguments[0][h]+", ";e=e.slice(0,-2)}else e=arguments[g];f.push(e)}v(b+"\nArguments: "+Array.prototype.slice.call(f).join("")+"\n"+(new Error).stack),d=!1}return c.apply(this,arguments)},c)}function x(b,c){null!=a.deprecationHandler&&a.deprecationHandler(b,c),ne[b]||(v(c),ne[b]=!0)}function y(a){return a instanceof Function||"[object Function]"===Object.prototype.toString.call(a)}function z(a){var b,c;for(c in a)b=a[c],y(b)?this[c]=b:this["_"+c]=b;this._config=a,
// Lenient ordinal parsing accepts just a number in addition to
// number + (possibly) stuff coming from _ordinalParseLenient.
this._ordinalParseLenient=new RegExp(this._ordinalParse.source+"|"+/\d{1,2}/.source)}function A(a,b){var c,e=i({},a);for(c in b)h(b,c)&&(d(a[c])&&d(b[c])?(e[c]={},i(e[c],a[c]),i(e[c],b[c])):null!=b[c]?e[c]=b[c]:delete e[c]);for(c in a)h(a,c)&&!h(b,c)&&d(a[c])&&(
// make sure changes to properties don't modify parent config
e[c]=i({},e[c]));return e}function B(a){null!=a&&this.set(a)}function C(a,b,c){var d=this._calendar[a]||this._calendar.sameElse;return y(d)?d.call(b,c):d}function D(a){var b=this._longDateFormat[a],c=this._longDateFormat[a.toUpperCase()];return b||!c?b:(this._longDateFormat[a]=c.replace(/MMMM|MM|DD|dddd/g,function(a){return a.slice(1)}),this._longDateFormat[a])}function E(){return this._invalidDate}function F(a){return this._ordinal.replace("%d",a)}function G(a,b,c,d){var e=this._relativeTime[c];return y(e)?e(a,b,c,d):e.replace(/%d/i,a)}function H(a,b){var c=this._relativeTime[a>0?"future":"past"];return y(c)?c(b):c.replace(/%s/i,b)}function I(a,b){var c=a.toLowerCase();we[c]=we[c+"s"]=we[b]=a}function J(a){return"string"==typeof a?we[a]||we[a.toLowerCase()]:void 0}function K(a){var b,c,d={};for(c in a)h(a,c)&&(b=J(c),b&&(d[b]=a[c]));return d}function L(a,b){xe[a]=b}function M(a){var b=[];for(var c in a)b.push({unit:c,priority:xe[c]});return b.sort(function(a,b){return a.priority-b.priority}),b}function N(b,c){return function(d){return null!=d?(P(this,b,d),a.updateOffset(this,c),this):O(this,b)}}function O(a,b){return a.isValid()?a._d["get"+(a._isUTC?"UTC":"")+b]():NaN}function P(a,b,c){a.isValid()&&a._d["set"+(a._isUTC?"UTC":"")+b](c)}
// MOMENTS
function Q(a){return a=J(a),y(this[a])?this[a]():this}function R(a,b){if("object"==typeof a){a=K(a);for(var c=M(a),d=0;d<c.length;d++)this[c[d].unit](a[c[d].unit])}else if(a=J(a),y(this[a]))return this[a](b);return this}function S(a,b,c){var d=""+Math.abs(a),e=b-d.length,f=a>=0;return(f?c?"+":"":"-")+Math.pow(10,Math.max(0,e)).toString().substr(1)+d}
// token:    'M'
// padded:   ['MM', 2]
// ordinal:  'Mo'
// callback: function () { this.month() + 1 }
function T(a,b,c,d){var e=d;"string"==typeof d&&(e=function(){return this[d]()}),a&&(Be[a]=e),b&&(Be[b[0]]=function(){return S(e.apply(this,arguments),b[1],b[2])}),c&&(Be[c]=function(){return this.localeData().ordinal(e.apply(this,arguments),a)})}function U(a){return a.match(/\[[\s\S]/)?a.replace(/^\[|\]$/g,""):a.replace(/\\/g,"")}function V(a){var b,c,d=a.match(ye);for(b=0,c=d.length;c>b;b++)Be[d[b]]?d[b]=Be[d[b]]:d[b]=U(d[b]);return function(b){var e,f="";for(e=0;c>e;e++)f+=d[e]instanceof Function?d[e].call(b,a):d[e];return f}}
// format date using native date object
function W(a,b){return a.isValid()?(b=X(b,a.localeData()),Ae[b]=Ae[b]||V(b),Ae[b](a)):a.localeData().invalidDate()}function X(a,b){function c(a){return b.longDateFormat(a)||a}var d=5;for(ze.lastIndex=0;d>=0&&ze.test(a);)a=a.replace(ze,c),ze.lastIndex=0,d-=1;return a}function Y(a,b,c){Te[a]=y(b)?b:function(a,d){return a&&c?c:b}}function Z(a,b){return h(Te,a)?Te[a](b._strict,b._locale):new RegExp($(a))}
// Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
function $(a){return _(a.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(a,b,c,d,e){return b||c||d||e}))}function _(a){return a.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}function aa(a,b){var c,d=b;for("string"==typeof a&&(a=[a]),"number"==typeof b&&(d=function(a,c){c[b]=t(a)}),c=0;c<a.length;c++)Ue[a[c]]=d}function ba(a,b){aa(a,function(a,c,d,e){d._w=d._w||{},b(a,d._w,d,e)})}function ca(a,b,c){null!=b&&h(Ue,a)&&Ue[a](b,c._a,c,a)}function da(a,b){return new Date(Date.UTC(a,b+1,0)).getUTCDate()}function ea(a,b){return a?c(this._months)?this._months[a.month()]:this._months[(this._months.isFormat||cf).test(b)?"format":"standalone"][a.month()]:this._months}function fa(a,b){return a?c(this._monthsShort)?this._monthsShort[a.month()]:this._monthsShort[cf.test(b)?"format":"standalone"][a.month()]:this._monthsShort}function ga(a,b,c){var d,e,f,g=a.toLocaleLowerCase();if(!this._monthsParse)for(
// this is not used
this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],d=0;12>d;++d)f=j([2e3,d]),this._shortMonthsParse[d]=this.monthsShort(f,"").toLocaleLowerCase(),this._longMonthsParse[d]=this.months(f,"").toLocaleLowerCase();return c?"MMM"===b?(e=pe.call(this._shortMonthsParse,g),-1!==e?e:null):(e=pe.call(this._longMonthsParse,g),-1!==e?e:null):"MMM"===b?(e=pe.call(this._shortMonthsParse,g),-1!==e?e:(e=pe.call(this._longMonthsParse,g),-1!==e?e:null)):(e=pe.call(this._longMonthsParse,g),-1!==e?e:(e=pe.call(this._shortMonthsParse,g),-1!==e?e:null))}function ha(a,b,c){var d,e,f;if(this._monthsParseExact)return ga.call(this,a,b,c);
// TODO: add sorting
// Sorting makes sure if one month (or abbr) is a prefix of another
// see sorting in computeMonthsParse
for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),d=0;12>d;d++){
// test the regex
if(e=j([2e3,d]),c&&!this._longMonthsParse[d]&&(this._longMonthsParse[d]=new RegExp("^"+this.months(e,"").replace(".","")+"$","i"),this._shortMonthsParse[d]=new RegExp("^"+this.monthsShort(e,"").replace(".","")+"$","i")),c||this._monthsParse[d]||(f="^"+this.months(e,"")+"|^"+this.monthsShort(e,""),this._monthsParse[d]=new RegExp(f.replace(".",""),"i")),c&&"MMMM"===b&&this._longMonthsParse[d].test(a))return d;if(c&&"MMM"===b&&this._shortMonthsParse[d].test(a))return d;if(!c&&this._monthsParse[d].test(a))return d}}
// MOMENTS
function ia(a,b){var c;if(!a.isValid())
// No op
return a;if("string"==typeof b)if(/^\d+$/.test(b))b=t(b);else
// TODO: Another silent failure?
if(b=a.localeData().monthsParse(b),"number"!=typeof b)return a;return c=Math.min(a.date(),da(a.year(),b)),a._d["set"+(a._isUTC?"UTC":"")+"Month"](b,c),a}function ja(b){return null!=b?(ia(this,b),a.updateOffset(this,!0),this):O(this,"Month")}function ka(){return da(this.year(),this.month())}function la(a){return this._monthsParseExact?(h(this,"_monthsRegex")||na.call(this),a?this._monthsShortStrictRegex:this._monthsShortRegex):(h(this,"_monthsShortRegex")||(this._monthsShortRegex=ff),this._monthsShortStrictRegex&&a?this._monthsShortStrictRegex:this._monthsShortRegex)}function ma(a){return this._monthsParseExact?(h(this,"_monthsRegex")||na.call(this),a?this._monthsStrictRegex:this._monthsRegex):(h(this,"_monthsRegex")||(this._monthsRegex=gf),this._monthsStrictRegex&&a?this._monthsStrictRegex:this._monthsRegex)}function na(){function a(a,b){return b.length-a.length}var b,c,d=[],e=[],f=[];for(b=0;12>b;b++)c=j([2e3,b]),d.push(this.monthsShort(c,"")),e.push(this.months(c,"")),f.push(this.months(c,"")),f.push(this.monthsShort(c,""));for(
// Sorting makes sure if one month (or abbr) is a prefix of another it
// will match the longer piece.
d.sort(a),e.sort(a),f.sort(a),b=0;12>b;b++)d[b]=_(d[b]),e[b]=_(e[b]);for(b=0;24>b;b++)f[b]=_(f[b]);this._monthsRegex=new RegExp("^("+f.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+e.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+d.join("|")+")","i")}
// HELPERS
function oa(a){return pa(a)?366:365}function pa(a){return a%4===0&&a%100!==0||a%400===0}function qa(){return pa(this.year())}function ra(a,b,c,d,e,f,g){
//can't just apply() to create a date:
//http://stackoverflow.com/questions/181348/instantiating-a-javascript-object-by-calling-prototype-constructor-apply
var h=new Date(a,b,c,d,e,f,g);
//the date constructor remaps years 0-99 to 1900-1999
return 100>a&&a>=0&&isFinite(h.getFullYear())&&h.setFullYear(a),h}function sa(a){var b=new Date(Date.UTC.apply(null,arguments));
//the Date.UTC function remaps years 0-99 to 1900-1999
return 100>a&&a>=0&&isFinite(b.getUTCFullYear())&&b.setUTCFullYear(a),b}
// start-of-first-week - start-of-year
function ta(a,b,c){var// first-week day -- which january is always in the first week (4 for iso, 1 for other)
d=7+b-c,
// first-week day local weekday -- which local weekday is fwd
e=(7+sa(a,0,d).getUTCDay()-b)%7;return-e+d-1}
//http://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
function ua(a,b,c,d,e){var f,g,h=(7+c-d)%7,i=ta(a,d,e),j=1+7*(b-1)+h+i;return 0>=j?(f=a-1,g=oa(f)+j):j>oa(a)?(f=a+1,g=j-oa(a)):(f=a,g=j),{year:f,dayOfYear:g}}function va(a,b,c){var d,e,f=ta(a.year(),b,c),g=Math.floor((a.dayOfYear()-f-1)/7)+1;return 1>g?(e=a.year()-1,d=g+wa(e,b,c)):g>wa(a.year(),b,c)?(d=g-wa(a.year(),b,c),e=a.year()+1):(e=a.year(),d=g),{week:d,year:e}}function wa(a,b,c){var d=ta(a,b,c),e=ta(a+1,b,c);return(oa(a)-d+e)/7}
// HELPERS
// LOCALES
function xa(a){return va(a,this._week.dow,this._week.doy).week}function ya(){return this._week.dow}function za(){return this._week.doy}
// MOMENTS
function Aa(a){var b=this.localeData().week(this);return null==a?b:this.add(7*(a-b),"d")}function Ba(a){var b=va(this,1,4).week;return null==a?b:this.add(7*(a-b),"d")}
// HELPERS
function Ca(a,b){return"string"!=typeof a?a:isNaN(a)?(a=b.weekdaysParse(a),"number"==typeof a?a:null):parseInt(a,10)}function Da(a,b){return"string"==typeof a?b.weekdaysParse(a)%7||7:isNaN(a)?null:a}function Ea(a,b){return a?c(this._weekdays)?this._weekdays[a.day()]:this._weekdays[this._weekdays.isFormat.test(b)?"format":"standalone"][a.day()]:this._weekdays}function Fa(a){return a?this._weekdaysShort[a.day()]:this._weekdaysShort}function Ga(a){return a?this._weekdaysMin[a.day()]:this._weekdaysMin}function Ha(a,b,c){var d,e,f,g=a.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],d=0;7>d;++d)f=j([2e3,1]).day(d),this._minWeekdaysParse[d]=this.weekdaysMin(f,"").toLocaleLowerCase(),this._shortWeekdaysParse[d]=this.weekdaysShort(f,"").toLocaleLowerCase(),this._weekdaysParse[d]=this.weekdays(f,"").toLocaleLowerCase();return c?"dddd"===b?(e=pe.call(this._weekdaysParse,g),-1!==e?e:null):"ddd"===b?(e=pe.call(this._shortWeekdaysParse,g),-1!==e?e:null):(e=pe.call(this._minWeekdaysParse,g),-1!==e?e:null):"dddd"===b?(e=pe.call(this._weekdaysParse,g),-1!==e?e:(e=pe.call(this._shortWeekdaysParse,g),-1!==e?e:(e=pe.call(this._minWeekdaysParse,g),-1!==e?e:null))):"ddd"===b?(e=pe.call(this._shortWeekdaysParse,g),-1!==e?e:(e=pe.call(this._weekdaysParse,g),-1!==e?e:(e=pe.call(this._minWeekdaysParse,g),-1!==e?e:null))):(e=pe.call(this._minWeekdaysParse,g),-1!==e?e:(e=pe.call(this._weekdaysParse,g),-1!==e?e:(e=pe.call(this._shortWeekdaysParse,g),-1!==e?e:null)))}function Ia(a,b,c){var d,e,f;if(this._weekdaysParseExact)return Ha.call(this,a,b,c);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),d=0;7>d;d++){
// test the regex
if(e=j([2e3,1]).day(d),c&&!this._fullWeekdaysParse[d]&&(this._fullWeekdaysParse[d]=new RegExp("^"+this.weekdays(e,"").replace(".",".?")+"$","i"),this._shortWeekdaysParse[d]=new RegExp("^"+this.weekdaysShort(e,"").replace(".",".?")+"$","i"),this._minWeekdaysParse[d]=new RegExp("^"+this.weekdaysMin(e,"").replace(".",".?")+"$","i")),this._weekdaysParse[d]||(f="^"+this.weekdays(e,"")+"|^"+this.weekdaysShort(e,"")+"|^"+this.weekdaysMin(e,""),this._weekdaysParse[d]=new RegExp(f.replace(".",""),"i")),c&&"dddd"===b&&this._fullWeekdaysParse[d].test(a))return d;if(c&&"ddd"===b&&this._shortWeekdaysParse[d].test(a))return d;if(c&&"dd"===b&&this._minWeekdaysParse[d].test(a))return d;if(!c&&this._weekdaysParse[d].test(a))return d}}
// MOMENTS
function Ja(a){if(!this.isValid())return null!=a?this:NaN;var b=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=a?(a=Ca(a,this.localeData()),this.add(a-b,"d")):b}function Ka(a){if(!this.isValid())return null!=a?this:NaN;var b=(this.day()+7-this.localeData()._week.dow)%7;return null==a?b:this.add(a-b,"d")}function La(a){if(!this.isValid())return null!=a?this:NaN;
// behaves the same as moment#day except
// as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
// as a setter, sunday should belong to the previous week.
if(null!=a){var b=Da(a,this.localeData());return this.day(this.day()%7?b:b-7)}return this.day()||7}function Ma(a){return this._weekdaysParseExact?(h(this,"_weekdaysRegex")||Pa.call(this),a?this._weekdaysStrictRegex:this._weekdaysRegex):(h(this,"_weekdaysRegex")||(this._weekdaysRegex=nf),this._weekdaysStrictRegex&&a?this._weekdaysStrictRegex:this._weekdaysRegex)}function Na(a){return this._weekdaysParseExact?(h(this,"_weekdaysRegex")||Pa.call(this),a?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(h(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=of),this._weekdaysShortStrictRegex&&a?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)}function Oa(a){return this._weekdaysParseExact?(h(this,"_weekdaysRegex")||Pa.call(this),a?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(h(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=pf),this._weekdaysMinStrictRegex&&a?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)}function Pa(){function a(a,b){return b.length-a.length}var b,c,d,e,f,g=[],h=[],i=[],k=[];for(b=0;7>b;b++)c=j([2e3,1]).day(b),d=this.weekdaysMin(c,""),e=this.weekdaysShort(c,""),f=this.weekdays(c,""),g.push(d),h.push(e),i.push(f),k.push(d),k.push(e),k.push(f);for(
// Sorting makes sure if one weekday (or abbr) is a prefix of another it
// will match the longer piece.
g.sort(a),h.sort(a),i.sort(a),k.sort(a),b=0;7>b;b++)h[b]=_(h[b]),i[b]=_(i[b]),k[b]=_(k[b]);this._weekdaysRegex=new RegExp("^("+k.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+h.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+g.join("|")+")","i")}
// FORMATTING
function Qa(){return this.hours()%12||12}function Ra(){return this.hours()||24}function Sa(a,b){T(a,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),b)})}
// PARSING
function Ta(a,b){return b._meridiemParse}
// LOCALES
function Ua(a){
// IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
// Using charAt should be more compatible.
return"p"===(a+"").toLowerCase().charAt(0)}function Va(a,b,c){return a>11?c?"pm":"PM":c?"am":"AM"}function Wa(a){return a?a.toLowerCase().replace("_","-"):a}
// pick the locale from the array
// try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
// substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
function Xa(a){for(var b,c,d,e,f=0;f<a.length;){for(e=Wa(a[f]).split("-"),b=e.length,c=Wa(a[f+1]),c=c?c.split("-"):null;b>0;){if(d=Ya(e.slice(0,b).join("-")))return d;if(c&&c.length>=b&&u(e,c,!0)>=b-1)
//the next array item is better than a shallower substring of this one
break;b--}f++}return null}function Ya(a){var b=null;
// TODO: Find a better way to register and load all the locales in Node
if(!uf[a]&&"undefined"!=typeof module&&module&&module.exports)try{b=qf._abbr,require("./locale/"+a),
// because defineLocale currently also sets the global locale, we
// want to undo that for lazy loaded locales
Za(b)}catch(c){}return uf[a]}
// This function will load locale and then set the global locale.  If
// no arguments are passed in, it will simply return the current global
// locale key.
function Za(a,b){var c;
// moment.duration._locale = moment._locale = data;
return a&&(c=o(b)?ab(a):$a(a,b),c&&(qf=c)),qf._abbr}function $a(a,b){if(null!==b){var c=tf;
// treat as if there is no base config
// backwards compat for now: also set the locale
return b.abbr=a,null!=uf[a]?(x("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),c=uf[a]._config):null!=b.parentLocale&&(null!=uf[b.parentLocale]?c=uf[b.parentLocale]._config:x("parentLocaleUndefined","specified parentLocale is not defined yet. See http://momentjs.com/guides/#/warnings/parent-locale/")),uf[a]=new B(A(c,b)),Za(a),uf[a]}
// useful for testing
return delete uf[a],null}function _a(a,b){if(null!=b){var c,d=tf;
// MERGE
null!=uf[a]&&(d=uf[a]._config),b=A(d,b),c=new B(b),c.parentLocale=uf[a],uf[a]=c,
// backwards compat for now: also set the locale
Za(a)}else
// pass null for config to unupdate, useful for tests
null!=uf[a]&&(null!=uf[a].parentLocale?uf[a]=uf[a].parentLocale:null!=uf[a]&&delete uf[a]);return uf[a]}
// returns locale data
function ab(a){var b;if(a&&a._locale&&a._locale._abbr&&(a=a._locale._abbr),!a)return qf;if(!c(a)){if(b=Ya(a))return b;a=[a]}return Xa(a)}function bb(){return oe(uf)}function cb(a){var b,c=a._a;return c&&-2===l(a).overflow&&(b=c[We]<0||c[We]>11?We:c[Xe]<1||c[Xe]>da(c[Ve],c[We])?Xe:c[Ye]<0||c[Ye]>24||24===c[Ye]&&(0!==c[Ze]||0!==c[$e]||0!==c[_e])?Ye:c[Ze]<0||c[Ze]>59?Ze:c[$e]<0||c[$e]>59?$e:c[_e]<0||c[_e]>999?_e:-1,l(a)._overflowDayOfYear&&(Ve>b||b>Xe)&&(b=Xe),l(a)._overflowWeeks&&-1===b&&(b=af),l(a)._overflowWeekday&&-1===b&&(b=bf),l(a).overflow=b),a}
// date from iso format
function db(a){var b,c,d,e,f,g,h=a._i,i=vf.exec(h)||wf.exec(h);if(i){for(l(a).iso=!0,b=0,c=yf.length;c>b;b++)if(yf[b][1].exec(i[1])){e=yf[b][0],d=yf[b][2]!==!1;break}if(null==e)return void(a._isValid=!1);if(i[3]){for(b=0,c=zf.length;c>b;b++)if(zf[b][1].exec(i[3])){
// match[2] should be 'T' or space
f=(i[2]||" ")+zf[b][0];break}if(null==f)return void(a._isValid=!1)}if(!d&&null!=f)return void(a._isValid=!1);if(i[4]){if(!xf.exec(i[4]))return void(a._isValid=!1);g="Z"}a._f=e+(f||"")+(g||""),jb(a)}else a._isValid=!1}
// date from iso format or fallback
function eb(b){var c=Af.exec(b._i);return null!==c?void(b._d=new Date(+c[1])):(db(b),void(b._isValid===!1&&(delete b._isValid,a.createFromInputFallback(b))))}
// Pick the first defined of two or three arguments.
function fb(a,b,c){return null!=a?a:null!=b?b:c}function gb(b){
// hooks is actually the exported moment object
var c=new Date(a.now());return b._useUTC?[c.getUTCFullYear(),c.getUTCMonth(),c.getUTCDate()]:[c.getFullYear(),c.getMonth(),c.getDate()]}
// convert an array to a date.
// the array should mirror the parameters below
// note: all values past the year are optional and will default to the lowest possible value.
// [year, month, day , hour, minute, second, millisecond]
function hb(a){var b,c,d,e,f=[];if(!a._d){
// Default to current date.
// * if no year, month, day of month are given, default to today
// * if day of month is given, default month and year
// * if month is given, default only year
// * if year is given, don't default anything
for(d=gb(a),a._w&&null==a._a[Xe]&&null==a._a[We]&&ib(a),a._dayOfYear&&(e=fb(a._a[Ve],d[Ve]),a._dayOfYear>oa(e)&&(l(a)._overflowDayOfYear=!0),c=sa(e,0,a._dayOfYear),a._a[We]=c.getUTCMonth(),a._a[Xe]=c.getUTCDate()),b=0;3>b&&null==a._a[b];++b)a._a[b]=f[b]=d[b];
// Zero out whatever was not defaulted, including time
for(;7>b;b++)a._a[b]=f[b]=null==a._a[b]?2===b?1:0:a._a[b];
// Check for 24:00:00.000
24===a._a[Ye]&&0===a._a[Ze]&&0===a._a[$e]&&0===a._a[_e]&&(a._nextDay=!0,a._a[Ye]=0),a._d=(a._useUTC?sa:ra).apply(null,f),
// Apply timezone offset from input. The actual utcOffset can be changed
// with parseZone.
null!=a._tzm&&a._d.setUTCMinutes(a._d.getUTCMinutes()-a._tzm),a._nextDay&&(a._a[Ye]=24)}}function ib(a){var b,c,d,e,f,g,h,i;b=a._w,null!=b.GG||null!=b.W||null!=b.E?(f=1,g=4,c=fb(b.GG,a._a[Ve],va(rb(),1,4).year),d=fb(b.W,1),e=fb(b.E,1),(1>e||e>7)&&(i=!0)):(f=a._locale._week.dow,g=a._locale._week.doy,c=fb(b.gg,a._a[Ve],va(rb(),f,g).year),d=fb(b.w,1),null!=b.d?(e=b.d,(0>e||e>6)&&(i=!0)):null!=b.e?(e=b.e+f,(b.e<0||b.e>6)&&(i=!0)):e=f),1>d||d>wa(c,f,g)?l(a)._overflowWeeks=!0:null!=i?l(a)._overflowWeekday=!0:(h=ua(c,d,e,f,g),a._a[Ve]=h.year,a._dayOfYear=h.dayOfYear)}
// date from string and format string
function jb(b){
// TODO: Move this to another part of the creation flow to prevent circular deps
if(b._f===a.ISO_8601)return void db(b);b._a=[],l(b).empty=!0;
// This array is used to make a Date, either with `new Date` or `Date.UTC`
var c,d,e,f,g,h=""+b._i,i=h.length,j=0;for(e=X(b._f,b._locale).match(ye)||[],c=0;c<e.length;c++)f=e[c],d=(h.match(Z(f,b))||[])[0],d&&(g=h.substr(0,h.indexOf(d)),g.length>0&&l(b).unusedInput.push(g),h=h.slice(h.indexOf(d)+d.length),j+=d.length),Be[f]?(d?l(b).empty=!1:l(b).unusedTokens.push(f),ca(f,d,b)):b._strict&&!d&&l(b).unusedTokens.push(f);
// add remaining unparsed input length to the string
l(b).charsLeftOver=i-j,h.length>0&&l(b).unusedInput.push(h),
// clear _12h flag if hour is <= 12
b._a[Ye]<=12&&l(b).bigHour===!0&&b._a[Ye]>0&&(l(b).bigHour=void 0),l(b).parsedDateParts=b._a.slice(0),l(b).meridiem=b._meridiem,
// handle meridiem
b._a[Ye]=kb(b._locale,b._a[Ye],b._meridiem),hb(b),cb(b)}function kb(a,b,c){var d;
// Fallback
return null==c?b:null!=a.meridiemHour?a.meridiemHour(b,c):null!=a.isPM?(d=a.isPM(c),d&&12>b&&(b+=12),d||12!==b||(b=0),b):b}
// date from string and array of format strings
function lb(a){var b,c,d,e,f;if(0===a._f.length)return l(a).invalidFormat=!0,void(a._d=new Date(NaN));for(e=0;e<a._f.length;e++)f=0,b=p({},a),null!=a._useUTC&&(b._useUTC=a._useUTC),b._f=a._f[e],jb(b),m(b)&&(f+=l(b).charsLeftOver,f+=10*l(b).unusedTokens.length,l(b).score=f,(null==d||d>f)&&(d=f,c=b));i(a,c||b)}function mb(a){if(!a._d){var b=K(a._i);a._a=g([b.year,b.month,b.day||b.date,b.hour,b.minute,b.second,b.millisecond],function(a){return a&&parseInt(a,10)}),hb(a)}}function nb(a){var b=new q(cb(ob(a)));
// Adding is smart enough around DST
return b._nextDay&&(b.add(1,"d"),b._nextDay=void 0),b}function ob(a){var b=a._i,d=a._f;return a._locale=a._locale||ab(a._l),null===b||void 0===d&&""===b?n({nullInput:!0}):("string"==typeof b&&(a._i=b=a._locale.preparse(b)),r(b)?new q(cb(b)):(c(d)?lb(a):f(b)?a._d=b:d?jb(a):pb(a),m(a)||(a._d=null),a))}function pb(b){var d=b._i;void 0===d?b._d=new Date(a.now()):f(d)?b._d=new Date(d.valueOf()):"string"==typeof d?eb(b):c(d)?(b._a=g(d.slice(0),function(a){return parseInt(a,10)}),hb(b)):"object"==typeof d?mb(b):"number"==typeof d?
// from milliseconds
b._d=new Date(d):a.createFromInputFallback(b)}function qb(a,b,f,g,h){var i={};
// object construction must be done this way.
// https://github.com/moment/moment/issues/1423
return"boolean"==typeof f&&(g=f,f=void 0),(d(a)&&e(a)||c(a)&&0===a.length)&&(a=void 0),i._isAMomentObject=!0,i._useUTC=i._isUTC=h,i._l=f,i._i=a,i._f=b,i._strict=g,nb(i)}function rb(a,b,c,d){return qb(a,b,c,d,!1)}
// Pick a moment m from moments so that m[fn](other) is true for all
// other. This relies on the function fn to be transitive.
//
// moments should either be an array of moment objects or an array, whose
// first element is an array of moment objects.
function sb(a,b){var d,e;if(1===b.length&&c(b[0])&&(b=b[0]),!b.length)return rb();for(d=b[0],e=1;e<b.length;++e)b[e].isValid()&&!b[e][a](d)||(d=b[e]);return d}
// TODO: Use [].sort instead?
function tb(){var a=[].slice.call(arguments,0);return sb("isBefore",a)}function ub(){var a=[].slice.call(arguments,0);return sb("isAfter",a)}function vb(a){var b=K(a),c=b.year||0,d=b.quarter||0,e=b.month||0,f=b.week||0,g=b.day||0,h=b.hour||0,i=b.minute||0,j=b.second||0,k=b.millisecond||0;
// representation for dateAddRemove
this._milliseconds=+k+1e3*j+// 1000
6e4*i+// 1000 * 60
1e3*h*60*60,//using 1000 * 60 * 60 instead of 36e5 to avoid floating point rounding errors https://github.com/moment/moment/issues/2978
// Because of dateAddRemove treats 24 hours as different from a
// day when working around DST, we need to store them separately
this._days=+g+7*f,
// It is impossible translate months into days without knowing
// which months you are are talking about, so we have to store
// it separately.
this._months=+e+3*d+12*c,this._data={},this._locale=ab(),this._bubble()}function wb(a){return a instanceof vb}function xb(a){return 0>a?-1*Math.round(-1*a):Math.round(a)}
// FORMATTING
function yb(a,b){T(a,0,0,function(){var a=this.utcOffset(),c="+";return 0>a&&(a=-a,c="-"),c+S(~~(a/60),2)+b+S(~~a%60,2)})}function zb(a,b){var c=(b||"").match(a)||[],d=c[c.length-1]||[],e=(d+"").match(Ef)||["-",0,0],f=+(60*e[1])+t(e[2]);return"+"===e[0]?f:-f}
// Return a moment from input, that is local/utc/zone equivalent to model.
function Ab(b,c){var d,e;
// Use low-level api, because this fn is low-level api.
return c._isUTC?(d=c.clone(),e=(r(b)||f(b)?b.valueOf():rb(b).valueOf())-d.valueOf(),d._d.setTime(d._d.valueOf()+e),a.updateOffset(d,!1),d):rb(b).local()}function Bb(a){
// On Firefox.24 Date#getTimezoneOffset returns a floating point.
// https://github.com/moment/moment/pull/1871
return 15*-Math.round(a._d.getTimezoneOffset()/15)}
// MOMENTS
// keepLocalTime = true means only change the timezone, without
// affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
// 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
// +0200, so we adjust the time as needed, to be valid.
//
// Keeping the time actually adds/subtracts (one hour)
// from the actual represented time. That is why we call updateOffset
// a second time. In case it wants us to change the offset again
// _changeInProgress == true case, then we have to adjust, because
// there is no such time in the given timezone.
function Cb(b,c){var d,e=this._offset||0;return this.isValid()?null!=b?("string"==typeof b?b=zb(Qe,b):Math.abs(b)<16&&(b=60*b),!this._isUTC&&c&&(d=Bb(this)),this._offset=b,this._isUTC=!0,null!=d&&this.add(d,"m"),e!==b&&(!c||this._changeInProgress?Sb(this,Nb(b-e,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,a.updateOffset(this,!0),this._changeInProgress=null)),this):this._isUTC?e:Bb(this):null!=b?this:NaN}function Db(a,b){return null!=a?("string"!=typeof a&&(a=-a),this.utcOffset(a,b),this):-this.utcOffset()}function Eb(a){return this.utcOffset(0,a)}function Fb(a){return this._isUTC&&(this.utcOffset(0,a),this._isUTC=!1,a&&this.subtract(Bb(this),"m")),this}function Gb(){if(this._tzm)this.utcOffset(this._tzm);else if("string"==typeof this._i){var a=zb(Pe,this._i);0===a?this.utcOffset(0,!0):this.utcOffset(zb(Pe,this._i))}return this}function Hb(a){return this.isValid()?(a=a?rb(a).utcOffset():0,(this.utcOffset()-a)%60===0):!1}function Ib(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()}function Jb(){if(!o(this._isDSTShifted))return this._isDSTShifted;var a={};if(p(a,this),a=ob(a),a._a){var b=a._isUTC?j(a._a):rb(a._a);this._isDSTShifted=this.isValid()&&u(a._a,b.toArray())>0}else this._isDSTShifted=!1;return this._isDSTShifted}function Kb(){return this.isValid()?!this._isUTC:!1}function Lb(){return this.isValid()?this._isUTC:!1}function Mb(){return this.isValid()?this._isUTC&&0===this._offset:!1}function Nb(a,b){var c,d,e,f=a,
// matching against regexp is expensive, do it on demand
g=null;// checks for null or undefined
return wb(a)?f={ms:a._milliseconds,d:a._days,M:a._months}:"number"==typeof a?(f={},b?f[b]=a:f.milliseconds=a):(g=Ff.exec(a))?(c="-"===g[1]?-1:1,f={y:0,d:t(g[Xe])*c,h:t(g[Ye])*c,m:t(g[Ze])*c,s:t(g[$e])*c,ms:t(xb(1e3*g[_e]))*c}):(g=Gf.exec(a))?(c="-"===g[1]?-1:1,f={y:Ob(g[2],c),M:Ob(g[3],c),w:Ob(g[4],c),d:Ob(g[5],c),h:Ob(g[6],c),m:Ob(g[7],c),s:Ob(g[8],c)}):null==f?f={}:"object"==typeof f&&("from"in f||"to"in f)&&(e=Qb(rb(f.from),rb(f.to)),f={},f.ms=e.milliseconds,f.M=e.months),d=new vb(f),wb(a)&&h(a,"_locale")&&(d._locale=a._locale),d}function Ob(a,b){
// We'd normally use ~~inp for this, but unfortunately it also
// converts floats to ints.
// inp may be undefined, so careful calling replace on it.
var c=a&&parseFloat(a.replace(",","."));
// apply sign while we're at it
return(isNaN(c)?0:c)*b}function Pb(a,b){var c={milliseconds:0,months:0};return c.months=b.month()-a.month()+12*(b.year()-a.year()),a.clone().add(c.months,"M").isAfter(b)&&--c.months,c.milliseconds=+b-+a.clone().add(c.months,"M"),c}function Qb(a,b){var c;return a.isValid()&&b.isValid()?(b=Ab(b,a),a.isBefore(b)?c=Pb(a,b):(c=Pb(b,a),c.milliseconds=-c.milliseconds,c.months=-c.months),c):{milliseconds:0,months:0}}
// TODO: remove 'name' arg after deprecation is removed
function Rb(a,b){return function(c,d){var e,f;
//invert the arguments, but complain about it
return null===d||isNaN(+d)||(x(b,"moment()."+b+"(period, number) is deprecated. Please use moment()."+b+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),f=c,c=d,d=f),c="string"==typeof c?+c:c,e=Nb(c,d),Sb(this,e,a),this}}function Sb(b,c,d,e){var f=c._milliseconds,g=xb(c._days),h=xb(c._months);b.isValid()&&(e=null==e?!0:e,f&&b._d.setTime(b._d.valueOf()+f*d),g&&P(b,"Date",O(b,"Date")+g*d),h&&ia(b,O(b,"Month")+h*d),e&&a.updateOffset(b,g||h))}function Tb(a,b){var c=a.diff(b,"days",!0);return-6>c?"sameElse":-1>c?"lastWeek":0>c?"lastDay":1>c?"sameDay":2>c?"nextDay":7>c?"nextWeek":"sameElse"}function Ub(b,c){
// We want to compare the start of today, vs this.
// Getting start-of-today depends on whether we're local/utc/offset or not.
var d=b||rb(),e=Ab(d,this).startOf("day"),f=a.calendarFormat(this,e)||"sameElse",g=c&&(y(c[f])?c[f].call(this,d):c[f]);return this.format(g||this.localeData().calendar(f,this,rb(d)))}function Vb(){return new q(this)}function Wb(a,b){var c=r(a)?a:rb(a);return this.isValid()&&c.isValid()?(b=J(o(b)?"millisecond":b),"millisecond"===b?this.valueOf()>c.valueOf():c.valueOf()<this.clone().startOf(b).valueOf()):!1}function Xb(a,b){var c=r(a)?a:rb(a);return this.isValid()&&c.isValid()?(b=J(o(b)?"millisecond":b),"millisecond"===b?this.valueOf()<c.valueOf():this.clone().endOf(b).valueOf()<c.valueOf()):!1}function Yb(a,b,c,d){return d=d||"()",("("===d[0]?this.isAfter(a,c):!this.isBefore(a,c))&&(")"===d[1]?this.isBefore(b,c):!this.isAfter(b,c))}function Zb(a,b){var c,d=r(a)?a:rb(a);return this.isValid()&&d.isValid()?(b=J(b||"millisecond"),"millisecond"===b?this.valueOf()===d.valueOf():(c=d.valueOf(),this.clone().startOf(b).valueOf()<=c&&c<=this.clone().endOf(b).valueOf())):!1}function $b(a,b){return this.isSame(a,b)||this.isAfter(a,b)}function _b(a,b){return this.isSame(a,b)||this.isBefore(a,b)}function ac(a,b,c){var d,e,f,g;// 1000
// 1000 * 60
// 1000 * 60 * 60
// 1000 * 60 * 60 * 24, negate dst
// 1000 * 60 * 60 * 24 * 7, negate dst
return this.isValid()?(d=Ab(a,this),d.isValid()?(e=6e4*(d.utcOffset()-this.utcOffset()),b=J(b),"year"===b||"month"===b||"quarter"===b?(g=bc(this,d),"quarter"===b?g/=3:"year"===b&&(g/=12)):(f=this-d,g="second"===b?f/1e3:"minute"===b?f/6e4:"hour"===b?f/36e5:"day"===b?(f-e)/864e5:"week"===b?(f-e)/6048e5:f),c?g:s(g)):NaN):NaN}function bc(a,b){
// difference in months
var c,d,e=12*(b.year()-a.year())+(b.month()-a.month()),
// b is in (anchor - 1 month, anchor + 1 month)
f=a.clone().add(e,"months");
//check for negative zero, return zero if negative zero
// linear across the month
// linear across the month
return 0>b-f?(c=a.clone().add(e-1,"months"),d=(b-f)/(f-c)):(c=a.clone().add(e+1,"months"),d=(b-f)/(c-f)),-(e+d)||0}function cc(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")}function dc(){var a=this.clone().utc();return 0<a.year()&&a.year()<=9999?y(Date.prototype.toISOString)?this.toDate().toISOString():W(a,"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]"):W(a,"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]")}function ec(b){b||(b=this.isUtc()?a.defaultFormatUtc:a.defaultFormat);var c=W(this,b);return this.localeData().postformat(c)}function fc(a,b){return this.isValid()&&(r(a)&&a.isValid()||rb(a).isValid())?Nb({to:this,from:a}).locale(this.locale()).humanize(!b):this.localeData().invalidDate()}function gc(a){return this.from(rb(),a)}function hc(a,b){return this.isValid()&&(r(a)&&a.isValid()||rb(a).isValid())?Nb({from:this,to:a}).locale(this.locale()).humanize(!b):this.localeData().invalidDate()}function ic(a){return this.to(rb(),a)}
// If passed a locale key, it will set the locale for this
// instance.  Otherwise, it will return the locale configuration
// variables for this instance.
function jc(a){var b;return void 0===a?this._locale._abbr:(b=ab(a),null!=b&&(this._locale=b),this)}function kc(){return this._locale}function lc(a){
// the following switch intentionally omits break keywords
// to utilize falling through the cases.
switch(a=J(a)){case"year":this.month(0);/* falls through */
case"quarter":case"month":this.date(1);/* falls through */
case"week":case"isoWeek":case"day":case"date":this.hours(0);/* falls through */
case"hour":this.minutes(0);/* falls through */
case"minute":this.seconds(0);/* falls through */
case"second":this.milliseconds(0)}
// weeks are a special case
// quarters are also special
return"week"===a&&this.weekday(0),"isoWeek"===a&&this.isoWeekday(1),"quarter"===a&&this.month(3*Math.floor(this.month()/3)),this}function mc(a){
// 'date' is an alias for 'day', so it should be considered as such.
return a=J(a),void 0===a||"millisecond"===a?this:("date"===a&&(a="day"),this.startOf(a).add(1,"isoWeek"===a?"week":a).subtract(1,"ms"))}function nc(){return this._d.valueOf()-6e4*(this._offset||0)}function oc(){return Math.floor(this.valueOf()/1e3)}function pc(){return new Date(this.valueOf())}function qc(){var a=this;return[a.year(),a.month(),a.date(),a.hour(),a.minute(),a.second(),a.millisecond()]}function rc(){var a=this;return{years:a.year(),months:a.month(),date:a.date(),hours:a.hours(),minutes:a.minutes(),seconds:a.seconds(),milliseconds:a.milliseconds()}}function sc(){
// new Date(NaN).toJSON() === null
return this.isValid()?this.toISOString():null}function tc(){return m(this)}function uc(){return i({},l(this))}function vc(){return l(this).overflow}function wc(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}}function xc(a,b){T(0,[a,a.length],0,b)}
// MOMENTS
function yc(a){return Cc.call(this,a,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)}function zc(a){return Cc.call(this,a,this.isoWeek(),this.isoWeekday(),1,4)}function Ac(){return wa(this.year(),1,4)}function Bc(){var a=this.localeData()._week;return wa(this.year(),a.dow,a.doy)}function Cc(a,b,c,d,e){var f;return null==a?va(this,d,e).year:(f=wa(a,d,e),b>f&&(b=f),Dc.call(this,a,b,c,d,e))}function Dc(a,b,c,d,e){var f=ua(a,b,c,d,e),g=sa(f.year,0,f.dayOfYear);return this.year(g.getUTCFullYear()),this.month(g.getUTCMonth()),this.date(g.getUTCDate()),this}
// MOMENTS
function Ec(a){return null==a?Math.ceil((this.month()+1)/3):this.month(3*(a-1)+this.month()%3)}
// HELPERS
// MOMENTS
function Fc(a){var b=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==a?b:this.add(a-b,"d")}function Gc(a,b){b[_e]=t(1e3*("0."+a))}
// MOMENTS
function Hc(){return this._isUTC?"UTC":""}function Ic(){return this._isUTC?"Coordinated Universal Time":""}function Jc(a){return rb(1e3*a)}function Kc(){return rb.apply(null,arguments).parseZone()}function Lc(a){return a}function Mc(a,b,c,d){var e=ab(),f=j().set(d,b);return e[c](f,a)}function Nc(a,b,c){if("number"==typeof a&&(b=a,a=void 0),a=a||"",null!=b)return Mc(a,b,c,"month");var d,e=[];for(d=0;12>d;d++)e[d]=Mc(a,d,c,"month");return e}
// ()
// (5)
// (fmt, 5)
// (fmt)
// (true)
// (true, 5)
// (true, fmt, 5)
// (true, fmt)
function Oc(a,b,c,d){"boolean"==typeof a?("number"==typeof b&&(c=b,b=void 0),b=b||""):(b=a,c=b,a=!1,"number"==typeof b&&(c=b,b=void 0),b=b||"");var e=ab(),f=a?e._week.dow:0;if(null!=c)return Mc(b,(c+f)%7,d,"day");var g,h=[];for(g=0;7>g;g++)h[g]=Mc(b,(g+f)%7,d,"day");return h}function Pc(a,b){return Nc(a,b,"months")}function Qc(a,b){return Nc(a,b,"monthsShort")}function Rc(a,b,c){return Oc(a,b,c,"weekdays")}function Sc(a,b,c){return Oc(a,b,c,"weekdaysShort")}function Tc(a,b,c){return Oc(a,b,c,"weekdaysMin")}function Uc(){var a=this._data;return this._milliseconds=Sf(this._milliseconds),this._days=Sf(this._days),this._months=Sf(this._months),a.milliseconds=Sf(a.milliseconds),a.seconds=Sf(a.seconds),a.minutes=Sf(a.minutes),a.hours=Sf(a.hours),a.months=Sf(a.months),a.years=Sf(a.years),this}function Vc(a,b,c,d){var e=Nb(b,c);return a._milliseconds+=d*e._milliseconds,a._days+=d*e._days,a._months+=d*e._months,a._bubble()}
// supports only 2.0-style add(1, 's') or add(duration)
function Wc(a,b){return Vc(this,a,b,1)}
// supports only 2.0-style subtract(1, 's') or subtract(duration)
function Xc(a,b){return Vc(this,a,b,-1)}function Yc(a){return 0>a?Math.floor(a):Math.ceil(a)}function Zc(){var a,b,c,d,e,f=this._milliseconds,g=this._days,h=this._months,i=this._data;
// if we have a mix of positive and negative values, bubble down first
// check: https://github.com/moment/moment/issues/2166
// The following code bubbles up values, see the tests for
// examples of what that means.
// convert days to months
// 12 months -> 1 year
return f>=0&&g>=0&&h>=0||0>=f&&0>=g&&0>=h||(f+=864e5*Yc(_c(h)+g),g=0,h=0),i.milliseconds=f%1e3,a=s(f/1e3),i.seconds=a%60,b=s(a/60),i.minutes=b%60,c=s(b/60),i.hours=c%24,g+=s(c/24),e=s($c(g)),h+=e,g-=Yc(_c(e)),d=s(h/12),h%=12,i.days=g,i.months=h,i.years=d,this}function $c(a){
// 400 years have 146097 days (taking into account leap year rules)
// 400 years have 12 months === 4800
return 4800*a/146097}function _c(a){
// the reverse of daysToMonths
return 146097*a/4800}function ad(a){var b,c,d=this._milliseconds;if(a=J(a),"month"===a||"year"===a)return b=this._days+d/864e5,c=this._months+$c(b),"month"===a?c:c/12;switch(b=this._days+Math.round(_c(this._months)),a){case"week":return b/7+d/6048e5;case"day":return b+d/864e5;case"hour":return 24*b+d/36e5;case"minute":return 1440*b+d/6e4;case"second":return 86400*b+d/1e3;
// Math.floor prevents floating point math errors here
case"millisecond":return Math.floor(864e5*b)+d;default:throw new Error("Unknown unit "+a)}}
// TODO: Use this.as('ms')?
function bd(){return this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*t(this._months/12)}function cd(a){return function(){return this.as(a)}}function dd(a){return a=J(a),this[a+"s"]()}function ed(a){return function(){return this._data[a]}}function fd(){return s(this.days()/7)}
// helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
function gd(a,b,c,d,e){return e.relativeTime(b||1,!!c,a,d)}function hd(a,b,c){var d=Nb(a).abs(),e=gg(d.as("s")),f=gg(d.as("m")),g=gg(d.as("h")),h=gg(d.as("d")),i=gg(d.as("M")),j=gg(d.as("y")),k=e<hg.s&&["s",e]||1>=f&&["m"]||f<hg.m&&["mm",f]||1>=g&&["h"]||g<hg.h&&["hh",g]||1>=h&&["d"]||h<hg.d&&["dd",h]||1>=i&&["M"]||i<hg.M&&["MM",i]||1>=j&&["y"]||["yy",j];return k[2]=b,k[3]=+a>0,k[4]=c,gd.apply(null,k)}
// This function allows you to set the rounding function for relative time strings
function id(a){return void 0===a?gg:"function"==typeof a?(gg=a,!0):!1}
// This function allows you to set a threshold for relative time strings
function jd(a,b){return void 0===hg[a]?!1:void 0===b?hg[a]:(hg[a]=b,!0)}function kd(a){var b=this.localeData(),c=hd(this,!a,b);return a&&(c=b.pastFuture(+this,c)),b.postformat(c)}function ld(){
// for ISO strings we do not use the normal bubbling rules:
//  * milliseconds bubble up until they become hours
//  * days do not bubble at all
//  * months bubble up until they become years
// This is because there is no context-free conversion between hours and days
// (think of clock changes)
// and also not between days and months (28-31 days per month)
var a,b,c,d=ig(this._milliseconds)/1e3,e=ig(this._days),f=ig(this._months);a=s(d/60),b=s(a/60),d%=60,a%=60,c=s(f/12),f%=12;
// inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
var g=c,h=f,i=e,j=b,k=a,l=d,m=this.asSeconds();return m?(0>m?"-":"")+"P"+(g?g+"Y":"")+(h?h+"M":"")+(i?i+"D":"")+(j||k||l?"T":"")+(j?j+"H":"")+(k?k+"M":"")+(l?l+"S":""):"P0D"}function md(a,b){var c=a.split("_");return b%10===1&&b%100!==11?c[0]:b%10>=2&&4>=b%10&&(10>b%100||b%100>=20)?c[1]:c[2]}function nd(a,b,c){var d={mm:b?"хвіліна_хвіліны_хвілін":"хвіліну_хвіліны_хвілін",hh:b?"гадзіна_гадзіны_гадзін":"гадзіну_гадзіны_гадзін",dd:"дзень_дні_дзён",MM:"месяц_месяцы_месяцаў",yy:"год_гады_гадоў"};return"m"===c?b?"хвіліна":"хвіліну":"h"===c?b?"гадзіна":"гадзіну":a+" "+md(d[c],+a)}function od(a,b,c){var d={mm:"munutenn",MM:"miz",dd:"devezh"};return a+" "+rd(d[c],a)}function pd(a){switch(qd(a)){case 1:case 3:case 4:case 5:case 9:return a+" bloaz";default:return a+" vloaz"}}function qd(a){return a>9?qd(a%10):a}function rd(a,b){return 2===b?sd(a):a}function sd(a){var b={m:"v",b:"v",d:"z"};return void 0===b[a.charAt(0)]?a:b[a.charAt(0)]+a.substring(1)}function td(a,b,c){var d=a+" ";switch(c){case"m":return b?"jedna minuta":"jedne minute";case"mm":return d+=1===a?"minuta":2===a||3===a||4===a?"minute":"minuta";case"h":return b?"jedan sat":"jednog sata";case"hh":return d+=1===a?"sat":2===a||3===a||4===a?"sata":"sati";case"dd":return d+=1===a?"dan":"dana";case"MM":return d+=1===a?"mjesec":2===a||3===a||4===a?"mjeseca":"mjeseci";case"yy":return d+=1===a?"godina":2===a||3===a||4===a?"godine":"godina"}}function ud(a){return a>1&&5>a&&1!==~~(a/10)}function vd(a,b,c,d){var e=a+" ";switch(c){case"s":// a few seconds / in a few seconds / a few seconds ago
return b||d?"pár sekund":"pár sekundami";case"m":// a minute / in a minute / a minute ago
return b?"minuta":d?"minutu":"minutou";case"mm":// 9 minutes / in 9 minutes / 9 minutes ago
// 9 minutes / in 9 minutes / 9 minutes ago
return b||d?e+(ud(a)?"minuty":"minut"):e+"minutami";break;case"h":// an hour / in an hour / an hour ago
return b?"hodina":d?"hodinu":"hodinou";case"hh":// 9 hours / in 9 hours / 9 hours ago
// 9 hours / in 9 hours / 9 hours ago
return b||d?e+(ud(a)?"hodiny":"hodin"):e+"hodinami";break;case"d":// a day / in a day / a day ago
return b||d?"den":"dnem";case"dd":// 9 days / in 9 days / 9 days ago
// 9 days / in 9 days / 9 days ago
return b||d?e+(ud(a)?"dny":"dní"):e+"dny";break;case"M":// a month / in a month / a month ago
return b||d?"měsíc":"měsícem";case"MM":// 9 months / in 9 months / 9 months ago
// 9 months / in 9 months / 9 months ago
return b||d?e+(ud(a)?"měsíce":"měsíců"):e+"měsíci";break;case"y":// a year / in a year / a year ago
return b||d?"rok":"rokem";case"yy":// 9 years / in 9 years / 9 years ago
// 9 years / in 9 years / 9 years ago
return b||d?e+(ud(a)?"roky":"let"):e+"lety"}}function wd(a,b,c,d){var e={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[a+" Tage",a+" Tagen"],M:["ein Monat","einem Monat"],MM:[a+" Monate",a+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[a+" Jahre",a+" Jahren"]};return b?e[c][0]:e[c][1]}function xd(a,b,c,d){var e={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[a+" Tage",a+" Tagen"],M:["ein Monat","einem Monat"],MM:[a+" Monate",a+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[a+" Jahre",a+" Jahren"]};return b?e[c][0]:e[c][1]}function yd(a,b,c,d){var e={s:["mõne sekundi","mõni sekund","paar sekundit"],m:["ühe minuti","üks minut"],mm:[a+" minuti",a+" minutit"],h:["ühe tunni","tund aega","üks tund"],hh:[a+" tunni",a+" tundi"],d:["ühe päeva","üks päev"],M:["kuu aja","kuu aega","üks kuu"],MM:[a+" kuu",a+" kuud"],y:["ühe aasta","aasta","üks aasta"],yy:[a+" aasta",a+" aastat"]};return b?e[c][2]?e[c][2]:e[c][1]:d?e[c][0]:e[c][1]}function zd(a,b,c,d){var e="";switch(c){case"s":return d?"muutaman sekunnin":"muutama sekunti";case"m":return d?"minuutin":"minuutti";case"mm":e=d?"minuutin":"minuuttia";break;case"h":return d?"tunnin":"tunti";case"hh":e=d?"tunnin":"tuntia";break;case"d":return d?"päivän":"päivä";case"dd":e=d?"päivän":"päivää";break;case"M":return d?"kuukauden":"kuukausi";case"MM":e=d?"kuukauden":"kuukautta";break;case"y":return d?"vuoden":"vuosi";case"yy":e=d?"vuoden":"vuotta"}return e=Ad(a,d)+" "+e}function Ad(a,b){return 10>a?b?Og[a]:Ng[a]:a}function Bd(a,b,c){var d=a+" ";switch(c){case"m":return b?"jedna minuta":"jedne minute";case"mm":return d+=1===a?"minuta":2===a||3===a||4===a?"minute":"minuta";case"h":return b?"jedan sat":"jednog sata";case"hh":return d+=1===a?"sat":2===a||3===a||4===a?"sata":"sati";case"dd":return d+=1===a?"dan":"dana";case"MM":return d+=1===a?"mjesec":2===a||3===a||4===a?"mjeseca":"mjeseci";case"yy":return d+=1===a?"godina":2===a||3===a||4===a?"godine":"godina"}}function Cd(a,b,c,d){var e=a;switch(c){case"s":return d||b?"néhány másodperc":"néhány másodperce";case"m":return"egy"+(d||b?" perc":" perce");case"mm":return e+(d||b?" perc":" perce");case"h":return"egy"+(d||b?" óra":" órája");case"hh":return e+(d||b?" óra":" órája");case"d":return"egy"+(d||b?" nap":" napja");case"dd":return e+(d||b?" nap":" napja");case"M":return"egy"+(d||b?" hónap":" hónapja");case"MM":return e+(d||b?" hónap":" hónapja");case"y":return"egy"+(d||b?" év":" éve");case"yy":return e+(d||b?" év":" éve")}return""}function Dd(a){return(a?"":"[múlt] ")+"["+Yg[this.day()]+"] LT[-kor]"}function Ed(a){return a%100===11?!0:a%10!==1}function Fd(a,b,c,d){var e=a+" ";switch(c){case"s":return b||d?"nokkrar sekúndur":"nokkrum sekúndum";case"m":return b?"mínúta":"mínútu";case"mm":return Ed(a)?e+(b||d?"mínútur":"mínútum"):b?e+"mínúta":e+"mínútu";case"hh":return Ed(a)?e+(b||d?"klukkustundir":"klukkustundum"):e+"klukkustund";case"d":return b?"dagur":d?"dag":"degi";case"dd":return Ed(a)?b?e+"dagar":e+(d?"daga":"dögum"):b?e+"dagur":e+(d?"dag":"degi");case"M":return b?"mánuður":d?"mánuð":"mánuði";case"MM":return Ed(a)?b?e+"mánuðir":e+(d?"mánuði":"mánuðum"):b?e+"mánuður":e+(d?"mánuð":"mánuði");case"y":return b||d?"ár":"ári";case"yy":return Ed(a)?e+(b||d?"ár":"árum"):e+(b||d?"ár":"ári")}}function Gd(a,b,c,d){var e={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return b?e[c][0]:e[c][1]}function Hd(a){var b=a.substr(0,a.indexOf(" "));return Jd(b)?"a "+a:"an "+a}function Id(a){var b=a.substr(0,a.indexOf(" "));return Jd(b)?"viru "+a:"virun "+a}/**
     * Returns true if the word before the given number loses the '-n' ending.
     * e.g. 'an 10 Deeg' but 'a 5 Deeg'
     *
     * @param number {integer}
     * @returns {boolean}
     */
function Jd(a){if(a=parseInt(a,10),isNaN(a))return!1;if(0>a)
// Negative Number --> always true
return!0;if(10>a)
// Only 1 digit
return a>=4&&7>=a;if(100>a){
// 2 digits
var b=a%10,c=a/10;return Jd(0===b?c:b)}if(1e4>a){
// 3 or 4 digits --> recursively check first digit
for(;a>=10;)a/=10;return Jd(a)}
// Anything larger than 4 digits: recursively check first n-3 digits
return a/=1e3,Jd(a)}function Kd(a,b,c,d){return b?"kelios sekundės":d?"kelių sekundžių":"kelias sekundes"}function Ld(a,b,c,d){return b?Nd(c)[0]:d?Nd(c)[1]:Nd(c)[2]}function Md(a){return a%10===0||a>10&&20>a}function Nd(a){return _g[a].split("_")}function Od(a,b,c,d){var e=a+" ";return 1===a?e+Ld(a,b,c[0],d):b?e+(Md(a)?Nd(c)[1]:Nd(c)[0]):d?e+Nd(c)[1]:e+(Md(a)?Nd(c)[1]:Nd(c)[2])}/**
     * @param withoutSuffix boolean true = a length of time; false = before/after a period of time.
     */
function Pd(a,b,c){return c?b%10===1&&b%100!==11?a[2]:a[3]:b%10===1&&b%100!==11?a[0]:a[1]}function Qd(a,b,c){return a+" "+Pd(ah[c],a,b)}function Rd(a,b,c){return Pd(ah[c],a,b)}function Sd(a,b){return b?"dažas sekundes":"dažām sekundēm"}function Td(a,b,c,d){var e="";if(b)switch(c){case"s":e="काही सेकंद";break;case"m":e="एक मिनिट";break;case"mm":e="%d मिनिटे";break;case"h":e="एक तास";break;case"hh":e="%d तास";break;case"d":e="एक दिवस";break;case"dd":e="%d दिवस";break;case"M":e="एक महिना";break;case"MM":e="%d महिने";break;case"y":e="एक वर्ष";break;case"yy":e="%d वर्षे"}else switch(c){case"s":e="काही सेकंदां";break;case"m":e="एका मिनिटा";break;case"mm":e="%d मिनिटां";break;case"h":e="एका तासा";break;case"hh":e="%d तासां";break;case"d":e="एका दिवसा";break;case"dd":e="%d दिवसां";break;case"M":e="एका महिन्या";break;case"MM":e="%d महिन्यां";break;case"y":e="एका वर्षा";break;case"yy":e="%d वर्षां"}return e.replace(/%d/i,a)}function Ud(a){return 5>a%10&&a%10>1&&~~(a/10)%10!==1}function Vd(a,b,c){var d=a+" ";switch(c){case"m":return b?"minuta":"minutę";case"mm":return d+(Ud(a)?"minuty":"minut");case"h":return b?"godzina":"godzinę";case"hh":return d+(Ud(a)?"godziny":"godzin");case"MM":return d+(Ud(a)?"miesiące":"miesięcy");case"yy":return d+(Ud(a)?"lata":"lat")}}function Wd(a,b,c){var d={mm:"minute",hh:"ore",dd:"zile",MM:"luni",yy:"ani"},e=" ";return(a%100>=20||a>=100&&a%100===0)&&(e=" de "),a+e+d[c]}function Xd(a,b){var c=a.split("_");return b%10===1&&b%100!==11?c[0]:b%10>=2&&4>=b%10&&(10>b%100||b%100>=20)?c[1]:c[2]}function Yd(a,b,c){var d={mm:b?"минута_минуты_минут":"минуту_минуты_минут",hh:"час_часа_часов",dd:"день_дня_дней",MM:"месяц_месяца_месяцев",yy:"год_года_лет"};return"m"===c?b?"минута":"минуту":a+" "+Xd(d[c],+a)}function Zd(a){return a>1&&5>a}function $d(a,b,c,d){var e=a+" ";switch(c){case"s":// a few seconds / in a few seconds / a few seconds ago
return b||d?"pár sekúnd":"pár sekundami";case"m":// a minute / in a minute / a minute ago
return b?"minúta":d?"minútu":"minútou";case"mm":// 9 minutes / in 9 minutes / 9 minutes ago
// 9 minutes / in 9 minutes / 9 minutes ago
return b||d?e+(Zd(a)?"minúty":"minút"):e+"minútami";break;case"h":// an hour / in an hour / an hour ago
return b?"hodina":d?"hodinu":"hodinou";case"hh":// 9 hours / in 9 hours / 9 hours ago
// 9 hours / in 9 hours / 9 hours ago
return b||d?e+(Zd(a)?"hodiny":"hodín"):e+"hodinami";break;case"d":// a day / in a day / a day ago
return b||d?"deň":"dňom";case"dd":// 9 days / in 9 days / 9 days ago
// 9 days / in 9 days / 9 days ago
return b||d?e+(Zd(a)?"dni":"dní"):e+"dňami";break;case"M":// a month / in a month / a month ago
return b||d?"mesiac":"mesiacom";case"MM":// 9 months / in 9 months / 9 months ago
// 9 months / in 9 months / 9 months ago
return b||d?e+(Zd(a)?"mesiace":"mesiacov"):e+"mesiacmi";break;case"y":// a year / in a year / a year ago
return b||d?"rok":"rokom";case"yy":// 9 years / in 9 years / 9 years ago
// 9 years / in 9 years / 9 years ago
return b||d?e+(Zd(a)?"roky":"rokov"):e+"rokmi"}}function _d(a,b,c,d){var e=a+" ";switch(c){case"s":return b||d?"nekaj sekund":"nekaj sekundami";case"m":return b?"ena minuta":"eno minuto";case"mm":return e+=1===a?b?"minuta":"minuto":2===a?b||d?"minuti":"minutama":5>a?b||d?"minute":"minutami":b||d?"minut":"minutami";case"h":return b?"ena ura":"eno uro";case"hh":return e+=1===a?b?"ura":"uro":2===a?b||d?"uri":"urama":5>a?b||d?"ure":"urami":b||d?"ur":"urami";case"d":return b||d?"en dan":"enim dnem";case"dd":return e+=1===a?b||d?"dan":"dnem":2===a?b||d?"dni":"dnevoma":b||d?"dni":"dnevi";case"M":return b||d?"en mesec":"enim mesecem";case"MM":return e+=1===a?b||d?"mesec":"mesecem":2===a?b||d?"meseca":"mesecema":5>a?b||d?"mesece":"meseci":b||d?"mesecev":"meseci";case"y":return b||d?"eno leto":"enim letom";case"yy":return e+=1===a?b||d?"leto":"letom":2===a?b||d?"leti":"letoma":5>a?b||d?"leta":"leti":b||d?"let":"leti"}}function ae(a){var b=a;return b=-1!==a.indexOf("jaj")?b.slice(0,-3)+"leS":-1!==a.indexOf("jar")?b.slice(0,-3)+"waQ":-1!==a.indexOf("DIS")?b.slice(0,-3)+"nem":b+" pIq"}function be(a){var b=a;return b=-1!==a.indexOf("jaj")?b.slice(0,-3)+"Hu’":-1!==a.indexOf("jar")?b.slice(0,-3)+"wen":-1!==a.indexOf("DIS")?b.slice(0,-3)+"ben":b+" ret"}function ce(a,b,c,d){var e=de(a);switch(c){case"mm":return e+" tup";case"hh":return e+" rep";case"dd":return e+" jaj";case"MM":return e+" jar";case"yy":return e+" DIS"}}function de(a){var b=Math.floor(a%1e3/100),c=Math.floor(a%100/10),d=a%10,e="";return b>0&&(e+=xh[b]+"vatlh"),c>0&&(e+=(""!==e?" ":"")+xh[c]+"maH"),d>0&&(e+=(""!==e?" ":"")+xh[d]),""===e?"pagh":e}function ee(a,b,c,d){var e={s:["viensas secunds","'iensas secunds"],m:["'n míut","'iens míut"],mm:[a+" míuts",""+a+" míuts"],h:["'n þora","'iensa þora"],hh:[a+" þoras",""+a+" þoras"],d:["'n ziua","'iensa ziua"],dd:[a+" ziuas",""+a+" ziuas"],M:["'n mes","'iens mes"],MM:[a+" mesen",""+a+" mesen"],y:["'n ar","'iens ar"],yy:[a+" ars",""+a+" ars"]};return d?e[c][0]:b?e[c][0]:e[c][1]}function fe(a,b){var c=a.split("_");return b%10===1&&b%100!==11?c[0]:b%10>=2&&4>=b%10&&(10>b%100||b%100>=20)?c[1]:c[2]}function ge(a,b,c){var d={mm:b?"хвилина_хвилини_хвилин":"хвилину_хвилини_хвилин",hh:b?"година_години_годин":"годину_години_годин",dd:"день_дні_днів",MM:"місяць_місяці_місяців",yy:"рік_роки_років"};return"m"===c?b?"хвилина":"хвилину":"h"===c?b?"година":"годину":a+" "+fe(d[c],+a)}function he(a,b){var c={nominative:"неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),accusative:"неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_"),genitive:"неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи".split("_")},d=/(\[[ВвУу]\]) ?dddd/.test(b)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(b)?"genitive":"nominative";return c[d][a.day()]}function ie(a){return function(){return a+"о"+(11===this.hours()?"б":"")+"] LT"}}var je,ke;ke=Array.prototype.some?Array.prototype.some:function(a){for(var b=Object(this),c=b.length>>>0,d=0;c>d;d++)if(d in b&&a.call(this,b[d],d,b))return!0;return!1};
// Plugins that add properties should also add the key here (null value),
// so we can properly clone ourselves.
var le=a.momentProperties=[],me=!1,ne={};a.suppressDeprecationWarnings=!1,a.deprecationHandler=null;var oe;oe=Object.keys?Object.keys:function(a){var b,c=[];for(b in a)h(a,b)&&c.push(b);return c};var pe,qe={sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},re={LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},se="Invalid date",te="%d",ue=/\d{1,2}/,ve={future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},we={},xe={},ye=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,ze=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,Ae={},Be={},Ce=/\d/,De=/\d\d/,Ee=/\d{3}/,Fe=/\d{4}/,Ge=/[+-]?\d{6}/,He=/\d\d?/,Ie=/\d\d\d\d?/,Je=/\d\d\d\d\d\d?/,Ke=/\d{1,3}/,Le=/\d{1,4}/,Me=/[+-]?\d{1,6}/,Ne=/\d+/,Oe=/[+-]?\d+/,Pe=/Z|[+-]\d\d:?\d\d/gi,Qe=/Z|[+-]\d\d(?::?\d\d)?/gi,Re=/[+-]?\d+(\.\d{1,3})?/,Se=/[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i,Te={},Ue={},Ve=0,We=1,Xe=2,Ye=3,Ze=4,$e=5,_e=6,af=7,bf=8;pe=Array.prototype.indexOf?Array.prototype.indexOf:function(a){
// I know
var b;for(b=0;b<this.length;++b)if(this[b]===a)return b;return-1},T("M",["MM",2],"Mo",function(){return this.month()+1}),T("MMM",0,0,function(a){return this.localeData().monthsShort(this,a)}),T("MMMM",0,0,function(a){return this.localeData().months(this,a)}),I("month","M"),L("month",8),Y("M",He),Y("MM",He,De),Y("MMM",function(a,b){return b.monthsShortRegex(a)}),Y("MMMM",function(a,b){return b.monthsRegex(a)}),aa(["M","MM"],function(a,b){b[We]=t(a)-1}),aa(["MMM","MMMM"],function(a,b,c,d){var e=c._locale.monthsParse(a,d,c._strict);null!=e?b[We]=e:l(c).invalidMonth=a});
// LOCALES
var cf=/D[oD]?(\[[^\[\]]*\]|\s+)+MMMM?/,df="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),ef="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),ff=Se,gf=Se;
// FORMATTING
T("Y",0,0,function(){var a=this.year();return 9999>=a?""+a:"+"+a}),T(0,["YY",2],0,function(){return this.year()%100}),T(0,["YYYY",4],0,"year"),T(0,["YYYYY",5],0,"year"),T(0,["YYYYYY",6,!0],0,"year"),
// ALIASES
I("year","y"),
// PRIORITIES
L("year",1),
// PARSING
Y("Y",Oe),Y("YY",He,De),Y("YYYY",Le,Fe),Y("YYYYY",Me,Ge),Y("YYYYYY",Me,Ge),aa(["YYYYY","YYYYYY"],Ve),aa("YYYY",function(b,c){c[Ve]=2===b.length?a.parseTwoDigitYear(b):t(b)}),aa("YY",function(b,c){c[Ve]=a.parseTwoDigitYear(b)}),aa("Y",function(a,b){b[Ve]=parseInt(a,10)}),
// HOOKS
a.parseTwoDigitYear=function(a){return t(a)+(t(a)>68?1900:2e3)};
// MOMENTS
var hf=N("FullYear",!0);
// FORMATTING
T("w",["ww",2],"wo","week"),T("W",["WW",2],"Wo","isoWeek"),
// ALIASES
I("week","w"),I("isoWeek","W"),
// PRIORITIES
L("week",5),L("isoWeek",5),
// PARSING
Y("w",He),Y("ww",He,De),Y("W",He),Y("WW",He,De),ba(["w","ww","W","WW"],function(a,b,c,d){b[d.substr(0,1)]=t(a)});var jf={dow:0,// Sunday is the first day of the week.
doy:6};
// FORMATTING
T("d",0,"do","day"),T("dd",0,0,function(a){return this.localeData().weekdaysMin(this,a)}),T("ddd",0,0,function(a){return this.localeData().weekdaysShort(this,a)}),T("dddd",0,0,function(a){return this.localeData().weekdays(this,a)}),T("e",0,0,"weekday"),T("E",0,0,"isoWeekday"),
// ALIASES
I("day","d"),I("weekday","e"),I("isoWeekday","E"),
// PRIORITY
L("day",11),L("weekday",11),L("isoWeekday",11),
// PARSING
Y("d",He),Y("e",He),Y("E",He),Y("dd",function(a,b){return b.weekdaysMinRegex(a)}),Y("ddd",function(a,b){return b.weekdaysShortRegex(a)}),Y("dddd",function(a,b){return b.weekdaysRegex(a)}),ba(["dd","ddd","dddd"],function(a,b,c,d){var e=c._locale.weekdaysParse(a,d,c._strict);
// if we didn't get a weekday name, mark the date as invalid
null!=e?b.d=e:l(c).invalidWeekday=a}),ba(["d","e","E"],function(a,b,c,d){b[d]=t(a)});
// LOCALES
var kf="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),lf="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),mf="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),nf=Se,of=Se,pf=Se;T("H",["HH",2],0,"hour"),T("h",["hh",2],0,Qa),T("k",["kk",2],0,Ra),T("hmm",0,0,function(){return""+Qa.apply(this)+S(this.minutes(),2)}),T("hmmss",0,0,function(){return""+Qa.apply(this)+S(this.minutes(),2)+S(this.seconds(),2)}),T("Hmm",0,0,function(){return""+this.hours()+S(this.minutes(),2)}),T("Hmmss",0,0,function(){return""+this.hours()+S(this.minutes(),2)+S(this.seconds(),2)}),Sa("a",!0),Sa("A",!1),
// ALIASES
I("hour","h"),
// PRIORITY
L("hour",13),Y("a",Ta),Y("A",Ta),Y("H",He),Y("h",He),Y("HH",He,De),Y("hh",He,De),Y("hmm",Ie),Y("hmmss",Je),Y("Hmm",Ie),Y("Hmmss",Je),aa(["H","HH"],Ye),aa(["a","A"],function(a,b,c){c._isPm=c._locale.isPM(a),c._meridiem=a}),aa(["h","hh"],function(a,b,c){b[Ye]=t(a),l(c).bigHour=!0}),aa("hmm",function(a,b,c){var d=a.length-2;b[Ye]=t(a.substr(0,d)),b[Ze]=t(a.substr(d)),l(c).bigHour=!0}),aa("hmmss",function(a,b,c){var d=a.length-4,e=a.length-2;b[Ye]=t(a.substr(0,d)),b[Ze]=t(a.substr(d,2)),b[$e]=t(a.substr(e)),l(c).bigHour=!0}),aa("Hmm",function(a,b,c){var d=a.length-2;b[Ye]=t(a.substr(0,d)),b[Ze]=t(a.substr(d))}),aa("Hmmss",function(a,b,c){var d=a.length-4,e=a.length-2;b[Ye]=t(a.substr(0,d)),b[Ze]=t(a.substr(d,2)),b[$e]=t(a.substr(e))});var qf,rf=/[ap]\.?m?\.?/i,sf=N("Hours",!0),tf={calendar:qe,longDateFormat:re,invalidDate:se,ordinal:te,ordinalParse:ue,relativeTime:ve,months:df,monthsShort:ef,week:jf,weekdays:kf,weekdaysMin:mf,weekdaysShort:lf,meridiemParse:rf},uf={},vf=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?/,wf=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?/,xf=/Z|[+-]\d\d(?::?\d\d)?/,yf=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],
// YYYYMM is NOT allowed by the standard
["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/]],zf=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],Af=/^\/?Date\((\-?\d+)/i;a.createFromInputFallback=w("value provided is not in a recognized ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(a){a._d=new Date(a._i+(a._useUTC?" UTC":""))}),
// constant that refers to the ISO standard
a.ISO_8601=function(){};var Bf=w("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var a=rb.apply(null,arguments);return this.isValid()&&a.isValid()?this>a?this:a:n()}),Cf=w("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var a=rb.apply(null,arguments);return this.isValid()&&a.isValid()?a>this?this:a:n()}),Df=function(){return Date.now?Date.now():+new Date};yb("Z",":"),yb("ZZ",""),
// PARSING
Y("Z",Qe),Y("ZZ",Qe),aa(["Z","ZZ"],function(a,b,c){c._useUTC=!0,c._tzm=zb(Qe,a)});
// HELPERS
// timezone chunker
// '+10:00' > ['10',  '00']
// '-1530'  > ['-15', '30']
var Ef=/([\+\-]|\d\d)/gi;
// HOOKS
// This function will be called whenever a moment is mutated.
// It is intended to keep the offset in sync with the timezone.
a.updateOffset=function(){};
// ASP.NET json date format regex
var Ff=/^(\-)?(?:(\d*)[. ])?(\d+)\:(\d+)(?:\:(\d+)(\.\d*)?)?$/,Gf=/^(-)?P(?:(-?[0-9,.]*)Y)?(?:(-?[0-9,.]*)M)?(?:(-?[0-9,.]*)W)?(?:(-?[0-9,.]*)D)?(?:T(?:(-?[0-9,.]*)H)?(?:(-?[0-9,.]*)M)?(?:(-?[0-9,.]*)S)?)?$/;Nb.fn=vb.prototype;var Hf=Rb(1,"add"),If=Rb(-1,"subtract");a.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",a.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var Jf=w("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(a){return void 0===a?this.localeData():this.locale(a)});
// FORMATTING
T(0,["gg",2],0,function(){return this.weekYear()%100}),T(0,["GG",2],0,function(){return this.isoWeekYear()%100}),xc("gggg","weekYear"),xc("ggggg","weekYear"),xc("GGGG","isoWeekYear"),xc("GGGGG","isoWeekYear"),
// ALIASES
I("weekYear","gg"),I("isoWeekYear","GG"),
// PRIORITY
L("weekYear",1),L("isoWeekYear",1),
// PARSING
Y("G",Oe),Y("g",Oe),Y("GG",He,De),Y("gg",He,De),Y("GGGG",Le,Fe),Y("gggg",Le,Fe),Y("GGGGG",Me,Ge),Y("ggggg",Me,Ge),ba(["gggg","ggggg","GGGG","GGGGG"],function(a,b,c,d){b[d.substr(0,2)]=t(a)}),ba(["gg","GG"],function(b,c,d,e){c[e]=a.parseTwoDigitYear(b)}),
// FORMATTING
T("Q",0,"Qo","quarter"),
// ALIASES
I("quarter","Q"),
// PRIORITY
L("quarter",7),
// PARSING
Y("Q",Ce),aa("Q",function(a,b){b[We]=3*(t(a)-1)}),
// FORMATTING
T("D",["DD",2],"Do","date"),
// ALIASES
I("date","D"),
// PRIOROITY
L("date",9),
// PARSING
Y("D",He),Y("DD",He,De),Y("Do",function(a,b){return a?b._ordinalParse:b._ordinalParseLenient}),aa(["D","DD"],Xe),aa("Do",function(a,b){b[Xe]=t(a.match(He)[0],10)});
// MOMENTS
var Kf=N("Date",!0);
// FORMATTING
T("DDD",["DDDD",3],"DDDo","dayOfYear"),
// ALIASES
I("dayOfYear","DDD"),
// PRIORITY
L("dayOfYear",4),
// PARSING
Y("DDD",Ke),Y("DDDD",Ee),aa(["DDD","DDDD"],function(a,b,c){c._dayOfYear=t(a)}),
// FORMATTING
T("m",["mm",2],0,"minute"),
// ALIASES
I("minute","m"),
// PRIORITY
L("minute",14),
// PARSING
Y("m",He),Y("mm",He,De),aa(["m","mm"],Ze);
// MOMENTS
var Lf=N("Minutes",!1);
// FORMATTING
T("s",["ss",2],0,"second"),
// ALIASES
I("second","s"),
// PRIORITY
L("second",15),
// PARSING
Y("s",He),Y("ss",He,De),aa(["s","ss"],$e);
// MOMENTS
var Mf=N("Seconds",!1);
// FORMATTING
T("S",0,0,function(){return~~(this.millisecond()/100)}),T(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),T(0,["SSS",3],0,"millisecond"),T(0,["SSSS",4],0,function(){return 10*this.millisecond()}),T(0,["SSSSS",5],0,function(){return 100*this.millisecond()}),T(0,["SSSSSS",6],0,function(){return 1e3*this.millisecond()}),T(0,["SSSSSSS",7],0,function(){return 1e4*this.millisecond()}),T(0,["SSSSSSSS",8],0,function(){return 1e5*this.millisecond()}),T(0,["SSSSSSSSS",9],0,function(){return 1e6*this.millisecond()}),
// ALIASES
I("millisecond","ms"),
// PRIORITY
L("millisecond",16),
// PARSING
Y("S",Ke,Ce),Y("SS",Ke,De),Y("SSS",Ke,Ee);var Nf;for(Nf="SSSS";Nf.length<=9;Nf+="S")Y(Nf,Ne);for(Nf="S";Nf.length<=9;Nf+="S")aa(Nf,Gc);
// MOMENTS
var Of=N("Milliseconds",!1);
// FORMATTING
T("z",0,0,"zoneAbbr"),T("zz",0,0,"zoneName");var Pf=q.prototype;Pf.add=Hf,Pf.calendar=Ub,Pf.clone=Vb,Pf.diff=ac,Pf.endOf=mc,Pf.format=ec,Pf.from=fc,Pf.fromNow=gc,Pf.to=hc,Pf.toNow=ic,Pf.get=Q,Pf.invalidAt=vc,Pf.isAfter=Wb,Pf.isBefore=Xb,Pf.isBetween=Yb,Pf.isSame=Zb,Pf.isSameOrAfter=$b,Pf.isSameOrBefore=_b,Pf.isValid=tc,Pf.lang=Jf,Pf.locale=jc,Pf.localeData=kc,Pf.max=Cf,Pf.min=Bf,Pf.parsingFlags=uc,Pf.set=R,Pf.startOf=lc,Pf.subtract=If,Pf.toArray=qc,Pf.toObject=rc,Pf.toDate=pc,Pf.toISOString=dc,Pf.toJSON=sc,Pf.toString=cc,Pf.unix=oc,Pf.valueOf=nc,Pf.creationData=wc,
// Year
Pf.year=hf,Pf.isLeapYear=qa,
// Week Year
Pf.weekYear=yc,Pf.isoWeekYear=zc,
// Quarter
Pf.quarter=Pf.quarters=Ec,
// Month
Pf.month=ja,Pf.daysInMonth=ka,
// Week
Pf.week=Pf.weeks=Aa,Pf.isoWeek=Pf.isoWeeks=Ba,Pf.weeksInYear=Bc,Pf.isoWeeksInYear=Ac,
// Day
Pf.date=Kf,Pf.day=Pf.days=Ja,Pf.weekday=Ka,Pf.isoWeekday=La,Pf.dayOfYear=Fc,
// Hour
Pf.hour=Pf.hours=sf,
// Minute
Pf.minute=Pf.minutes=Lf,
// Second
Pf.second=Pf.seconds=Mf,
// Millisecond
Pf.millisecond=Pf.milliseconds=Of,
// Offset
Pf.utcOffset=Cb,Pf.utc=Eb,Pf.local=Fb,Pf.parseZone=Gb,Pf.hasAlignedHourOffset=Hb,Pf.isDST=Ib,Pf.isLocal=Kb,Pf.isUtcOffset=Lb,Pf.isUtc=Mb,Pf.isUTC=Mb,
// Timezone
Pf.zoneAbbr=Hc,Pf.zoneName=Ic,
// Deprecations
Pf.dates=w("dates accessor is deprecated. Use date instead.",Kf),Pf.months=w("months accessor is deprecated. Use month instead",ja),Pf.years=w("years accessor is deprecated. Use year instead",hf),Pf.zone=w("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",Db),Pf.isDSTShifted=w("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",Jb);var Qf=Pf,Rf=B.prototype;Rf.calendar=C,Rf.longDateFormat=D,Rf.invalidDate=E,Rf.ordinal=F,Rf.preparse=Lc,Rf.postformat=Lc,Rf.relativeTime=G,Rf.pastFuture=H,Rf.set=z,
// Month
Rf.months=ea,Rf.monthsShort=fa,Rf.monthsParse=ha,Rf.monthsRegex=ma,Rf.monthsShortRegex=la,
// Week
Rf.week=xa,Rf.firstDayOfYear=za,Rf.firstDayOfWeek=ya,
// Day of Week
Rf.weekdays=Ea,Rf.weekdaysMin=Ga,Rf.weekdaysShort=Fa,Rf.weekdaysParse=Ia,Rf.weekdaysRegex=Ma,Rf.weekdaysShortRegex=Na,Rf.weekdaysMinRegex=Oa,
// Hours
Rf.isPM=Ua,Rf.meridiem=Va,Za("en",{ordinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(a){var b=a%10,c=1===t(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c}}),
// Side effect imports
a.lang=w("moment.lang is deprecated. Use moment.locale instead.",Za),a.langData=w("moment.langData is deprecated. Use moment.localeData instead.",ab);var Sf=Math.abs,Tf=cd("ms"),Uf=cd("s"),Vf=cd("m"),Wf=cd("h"),Xf=cd("d"),Yf=cd("w"),Zf=cd("M"),$f=cd("y"),_f=ed("milliseconds"),ag=ed("seconds"),bg=ed("minutes"),cg=ed("hours"),dg=ed("days"),eg=ed("months"),fg=ed("years"),gg=Math.round,hg={s:45,// seconds to minute
m:45,// minutes to hour
h:22,// hours to day
d:26,// days to month
M:11},ig=Math.abs,jg=vb.prototype;jg.abs=Uc,jg.add=Wc,jg.subtract=Xc,jg.as=ad,jg.asMilliseconds=Tf,jg.asSeconds=Uf,jg.asMinutes=Vf,jg.asHours=Wf,jg.asDays=Xf,jg.asWeeks=Yf,jg.asMonths=Zf,jg.asYears=$f,jg.valueOf=bd,jg._bubble=Zc,jg.get=dd,jg.milliseconds=_f,jg.seconds=ag,jg.minutes=bg,jg.hours=cg,jg.days=dg,jg.weeks=fd,jg.months=eg,jg.years=fg,jg.humanize=kd,jg.toISOString=ld,jg.toString=ld,jg.toJSON=ld,jg.locale=jc,jg.localeData=kc,
// Deprecations
jg.toIsoString=w("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",ld),jg.lang=Jf,
// Side effect imports
// FORMATTING
T("X",0,0,"unix"),T("x",0,0,"valueOf"),
// PARSING
Y("x",Oe),Y("X",Re),aa("X",function(a,b,c){c._d=new Date(1e3*parseFloat(a,10))}),aa("x",function(a,b,c){c._d=new Date(t(a))}),a.version="2.15.1",b(rb),a.fn=Qf,a.min=tb,a.max=ub,a.now=Df,a.utc=j,a.unix=Jc,a.months=Pc,a.isDate=f,a.locale=Za,a.invalid=n,a.duration=Nb,a.isMoment=r,a.weekdays=Rc,a.parseZone=Kc,a.localeData=ab,a.isDuration=wb,a.monthsShort=Qc,a.weekdaysMin=Tc,a.defineLocale=$a,a.updateLocale=_a,a.locales=bb,a.weekdaysShort=Sc,a.normalizeUnits=J,a.relativeTimeRounding=id,a.relativeTimeThreshold=jd,a.calendarFormat=Tb,a.prototype=Qf;var kg=a,lg=(kg.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(a){return/^nm$/i.test(a)},meridiem:function(a,b,c){return 12>a?c?"vm":"VM":c?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[Môre om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},ordinalParse:/\d{1,2}(ste|de)/,ordinal:function(a){return a+(1===a||8===a||a>=20?"ste":"de")},week:{dow:1,// Maandag is die eerste dag van die week.
doy:4}}),{1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"}),mg=function(a){return 0===a?0:1===a?1:2===a?2:a%100>=3&&10>=a%100?3:a%100>=11?4:5},ng={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},og=function(a){return function(b,c,d,e){var f=mg(b),g=ng[a][mg(b)];return 2===f&&(g=g[c?0:1]),g.replace(/%d/i,b)}},pg=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],qg=(kg.defineLocale("ar-ly",{months:pg,monthsShort:pg,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(a){return"م"===a},meridiem:function(a,b,c){return 12>a?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:og("s"),m:og("m"),mm:og("m"),h:og("h"),hh:og("h"),d:og("d"),dd:og("d"),M:og("M"),MM:og("M"),y:og("y"),yy:og("y")},preparse:function(a){return a.replace(/\u200f/g,"").replace(/،/g,",")},postformat:function(a){return a.replace(/\d/g,function(a){return lg[a]}).replace(/,/g,"،")},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),kg.defineLocale("ar-ma",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),{1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"}),rg={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},sg=(kg.defineLocale("ar-sa",{months:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(a){return"م"===a},meridiem:function(a,b,c){return 12>a?"ص":"م"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},preparse:function(a){return a.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(a){return rg[a]}).replace(/،/g,",")},postformat:function(a){return a.replace(/\d/g,function(a){return qg[a]}).replace(/,/g,"،")},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),kg.defineLocale("ar-tn",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,// Monday is the first day of the week.
doy:4}}),{1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"}),tg={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},ug=function(a){return 0===a?0:1===a?1:2===a?2:a%100>=3&&10>=a%100?3:a%100>=11?4:5},vg={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},wg=function(a){return function(b,c,d,e){var f=ug(b),g=vg[a][ug(b)];return 2===f&&(g=g[c?0:1]),g.replace(/%d/i,b)}},xg=["كانون الثاني يناير","شباط فبراير","آذار مارس","نيسان أبريل","أيار مايو","حزيران يونيو","تموز يوليو","آب أغسطس","أيلول سبتمبر","تشرين الأول أكتوبر","تشرين الثاني نوفمبر","كانون الأول ديسمبر"],yg=(kg.defineLocale("ar",{months:xg,monthsShort:xg,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(a){return"م"===a},meridiem:function(a,b,c){return 12>a?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:wg("s"),m:wg("m"),mm:wg("m"),h:wg("h"),hh:wg("h"),d:wg("d"),dd:wg("d"),M:wg("M"),MM:wg("M"),y:wg("y"),yy:wg("y")},preparse:function(a){return a.replace(/\u200f/g,"").replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(a){return tg[a]}).replace(/،/g,",")},postformat:function(a){return a.replace(/\d/g,function(a){return sg[a]}).replace(/,/g,"،")},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),{1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-üncü",4:"-üncü",100:"-üncü",6:"-ncı",9:"-uncu",10:"-uncu",30:"-uncu",60:"-ıncı",90:"-ıncı"}),zg=(kg.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə".split("_"),weekdaysShort:"Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən".split("_"),weekdaysMin:"Bz_BE_ÇA_Çə_CA_Cü_Şə".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[gələn həftə] dddd [saat] LT",lastDay:"[dünən] LT",lastWeek:"[keçən həftə] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s əvvəl",s:"birneçə saniyyə",m:"bir dəqiqə",mm:"%d dəqiqə",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(a){return/^(gündüz|axşam)$/.test(a)},meridiem:function(a,b,c){return 4>a?"gecə":12>a?"səhər":17>a?"gündüz":"axşam"},ordinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(a){if(0===a)// special case for zero
return a+"-ıncı";var b=a%10,c=a%100-b,d=a>=100?100:null;return a+(yg[b]||yg[c]||yg[d])},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("be",{months:{format:"студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня".split("_"),standalone:"студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань".split("_")},monthsShort:"студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж".split("_"),weekdays:{format:"нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу".split("_"),standalone:"нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота".split("_"),isFormat:/\[ ?[Вв] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"нд_пн_ат_ср_чц_пт_сб".split("_"),weekdaysMin:"нд_пн_ат_ср_чц_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сёння ў] LT",nextDay:"[Заўтра ў] LT",lastDay:"[Учора ў] LT",nextWeek:function(){return"[У] dddd [ў] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[У мінулую] dddd [ў] LT";case 1:case 2:case 4:return"[У мінулы] dddd [ў] LT"}},sameElse:"L"},relativeTime:{future:"праз %s",past:"%s таму",s:"некалькі секунд",m:nd,mm:nd,h:nd,hh:nd,d:"дзень",dd:nd,M:"месяц",MM:nd,y:"год",yy:nd},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(a){return/^(дня|вечара)$/.test(a)},meridiem:function(a,b,c){return 4>a?"ночы":12>a?"раніцы":17>a?"дня":"вечара"},ordinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(a,b){switch(b){case"M":case"d":case"DDD":case"w":case"W":return a%10!==2&&a%10!==3||a%100===12||a%100===13?a+"-ы":a+"-і";case"D":return a+"-га";default:return a}},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("bg",{months:"януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември".split("_"),monthsShort:"янр_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек".split("_"),weekdays:"неделя_понеделник_вторник_сряда_четвъртък_петък_събота".split("_"),weekdaysShort:"нед_пон_вто_сря_чет_пет_съб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Днес в] LT",nextDay:"[Утре в] LT",nextWeek:"dddd [в] LT",lastDay:"[Вчера в] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[В изминалата] dddd [в] LT";case 1:case 2:case 4:case 5:return"[В изминалия] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"след %s",past:"преди %s",s:"няколко секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дни",M:"месец",MM:"%d месеца",y:"година",yy:"%d години"},ordinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(a){var b=a%10,c=a%100;return 0===a?a+"-ев":0===c?a+"-ен":c>10&&20>c?a+"-ти":1===b?a+"-ви":2===b?a+"-ри":7===b||8===b?a+"-ми":a+"-ти"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),{1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"}),Ag={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"},Bg=(kg.defineLocale("bn",{months:"জানুয়ারী_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব_মার্চ_এপ্র_মে_জুন_জুল_আগ_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গ_বুধ_বৃহঃ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(a){return a.replace(/[১২৩৪৫৬৭৮৯০]/g,function(a){return Ag[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return zg[a]})},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(a,b){return 12===a&&(a=0),"রাত"===b&&a>=4||"দুপুর"===b&&5>a||"বিকাল"===b?a+12:a},meridiem:function(a,b,c){return 4>a?"রাত":10>a?"সকাল":17>a?"দুপুর":20>a?"বিকাল":"রাত"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),{1:"༡",2:"༢",3:"༣",4:"༤",5:"༥",6:"༦",7:"༧",8:"༨",9:"༩",0:"༠"}),Cg={"༡":"1","༢":"2","༣":"3","༤":"4","༥":"5","༦":"6","༧":"7","༨":"8","༩":"9","༠":"0"},Dg=(kg.defineLocale("bo",{months:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),monthsShort:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),weekdays:"གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་".split("_"),weekdaysShort:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),weekdaysMin:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[དི་རིང] LT",nextDay:"[སང་ཉིན] LT",nextWeek:"[བདུན་ཕྲག་རྗེས་མ], LT",lastDay:"[ཁ་སང] LT",lastWeek:"[བདུན་ཕྲག་མཐའ་མ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ལ་",past:"%s སྔན་ལ",s:"ལམ་སང",m:"སྐར་མ་གཅིག",mm:"%d སྐར་མ",h:"ཆུ་ཚོད་གཅིག",hh:"%d ཆུ་ཚོད",d:"ཉིན་གཅིག",dd:"%d ཉིན་",M:"ཟླ་བ་གཅིག",MM:"%d ཟླ་བ",y:"ལོ་གཅིག",yy:"%d ལོ"},preparse:function(a){return a.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,function(a){return Cg[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return Bg[a]})},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(a,b){return 12===a&&(a=0),"མཚན་མོ"===b&&a>=4||"ཉིན་གུང"===b&&5>a||"དགོང་དག"===b?a+12:a},meridiem:function(a,b,c){return 4>a?"མཚན་མོ":10>a?"ཞོགས་ཀས":17>a?"ཉིན་གུང":20>a?"དགོང་དག":"མཚན་མོ"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),kg.defineLocale("br",{months:"Genver_C'hwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_C'hwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Merc'her_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h[e]mm A",LTS:"h[e]mm:ss A",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY h[e]mm A",LLLL:"dddd, D [a viz] MMMM YYYY h[e]mm A"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warc'hoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Dec'h da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s 'zo",s:"un nebeud segondennoù",m:"ur vunutenn",mm:od,h:"un eur",hh:"%d eur",d:"un devezh",dd:od,M:"ur miz",MM:od,y:"ur bloaz",yy:pd},ordinalParse:/\d{1,2}(añ|vet)/,ordinal:function(a){var b=1===a?"añ":"vet";return a+b},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",m:td,mm:td,h:td,hh:td,d:"dan",dd:td,M:"mjesec",MM:td,y:"godinu",yy:td},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("ca",{months:"gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),monthsShort:"gen._febr._mar._abr._mai._jun._jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"Dg_Dl_Dt_Dc_Dj_Dv_Ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd D MMMM YYYY H:mm"},calendar:{sameDay:function(){return"[avui a "+(1!==this.hours()?"les":"la")+"] LT"},nextDay:function(){return"[demà a "+(1!==this.hours()?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(1!==this.hours()?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(1!==this.hours()?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(1!==this.hours()?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"fa %s",s:"uns segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},ordinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(a,b){var c=1===a?"r":2===a?"n":3===a?"r":4===a?"t":"è";return"w"!==b&&"W"!==b||(c="a"),a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),"leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec".split("_")),Eg="led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro".split("_"),Fg=(kg.defineLocale("cs",{months:Dg,monthsShort:Eg,monthsParse:function(a,b){var c,d=[];for(c=0;12>c;c++)
// use custom parser to solve problem with July (červenec)
d[c]=new RegExp("^"+a[c]+"$|^"+b[c]+"$","i");return d}(Dg,Eg),shortMonthsParse:function(a){var b,c=[];for(b=0;12>b;b++)c[b]=new RegExp("^"+a[b]+"$","i");return c}(Eg),longMonthsParse:function(a){var b,c=[];for(b=0;12>b;b++)c[b]=new RegExp("^"+a[b]+"$","i");return c}(Dg),weekdays:"neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota".split("_"),weekdaysShort:"ne_po_út_st_čt_pá_so".split("_"),weekdaysMin:"ne_po_út_st_čt_pá_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[zítra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v neděli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve středu v] LT";case 4:return"[ve čtvrtek v] LT";case 5:return"[v pátek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[včera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou neděli v] LT";case 1:case 2:return"[minulé] dddd [v] LT";case 3:return"[minulou středu v] LT";case 4:case 5:return"[minulý] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"před %s",s:vd,m:vd,mm:vd,h:vd,hh:vd,d:vd,dd:vd,M:vd,MM:vd,y:vd,yy:vd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("cv",{months:"кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав".split("_"),monthsShort:"кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш".split("_"),weekdays:"вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун".split("_"),weekdaysShort:"выр_тун_ытл_юн_кӗҫ_эрн_шӑм".split("_"),weekdaysMin:"вр_тн_ыт_юн_кҫ_эр_шм".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]",LLL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm",LLLL:"dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm"},calendar:{sameDay:"[Паян] LT [сехетре]",nextDay:"[Ыран] LT [сехетре]",lastDay:"[Ӗнер] LT [сехетре]",nextWeek:"[Ҫитес] dddd LT [сехетре]",lastWeek:"[Иртнӗ] dddd LT [сехетре]",sameElse:"L"},relativeTime:{future:function(a){var b=/сехет$/i.exec(a)?"рен":/ҫул$/i.exec(a)?"тан":"ран";return a+b},past:"%s каялла",s:"пӗр-ик ҫеккунт",m:"пӗр минут",mm:"%d минут",h:"пӗр сехет",hh:"%d сехет",d:"пӗр кун",dd:"%d кун",M:"пӗр уйӑх",MM:"%d уйӑх",y:"пӗр ҫул",yy:"%d ҫул"},ordinalParse:/\d{1,2}-мӗш/,ordinal:"%d-мӗш",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,
// time formats are the same as en-gb
longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn ôl",s:"ychydig eiliadau",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},ordinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,
// traditional ordinal numbers above 31 are not commonly used in colloquial Welsh
ordinal:function(a){var b=a,c="",d=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed",// 1af to 10fed
"eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"];return b>20?c=40===b||50===b||60===b||80===b||100===b?"fed":"ain":b>0&&(c=d[b]),a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"søn_man_tir_ons_tor_fre_lør".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY HH:mm"},calendar:{sameDay:"[I dag kl.] LT",nextDay:"[I morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[I går kl.] LT",lastWeek:"[sidste] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"få sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en måned",MM:"%d måneder",y:"et år",yy:"%d år"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("de-at",{months:"Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jän._Febr._Mrz._Apr._Mai_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",m:wd,mm:"%d Minuten",h:wd,hh:"%d Stunden",d:wd,dd:wd,M:wd,MM:wd,y:wd,yy:wd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("de",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Apr._Mai_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",m:xd,mm:"%d Minuten",h:xd,hh:"%d Stunden",d:xd,dd:xd,M:xd,MM:xd,y:xd,yy:xd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),["ޖެނުއަރީ","ފެބްރުއަރީ","މާރިޗު","އޭޕްރީލު","މޭ","ޖޫން","ޖުލައި","އޯގަސްޓު","ސެޕްޓެމްބަރު","އޮކްޓޯބަރު","ނޮވެމްބަރު","ޑިސެމްބަރު"]),Gg=["އާދިއްތަ","ހޯމަ","އަންގާރަ","ބުދަ","ބުރާސްފަތި","ހުކުރު","ހޮނިހިރު"],Hg=(kg.defineLocale("dv",{months:Fg,monthsShort:Fg,weekdays:Gg,weekdaysShort:Gg,weekdaysMin:"އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(a){return"މފ"===a},meridiem:function(a,b,c){return 12>a?"މކ":"މފ"},calendar:{sameDay:"[މިއަދު] LT",nextDay:"[މާދަމާ] LT",nextWeek:"dddd LT",lastDay:"[އިއްޔެ] LT",lastWeek:"[ފާއިތުވި] dddd LT",sameElse:"L"},relativeTime:{future:"ތެރޭގައި %s",past:"ކުރިން %s",s:"ސިކުންތުކޮޅެއް",m:"މިނިޓެއް",mm:"މިނިޓު %d",h:"ގަޑިއިރެއް",hh:"ގަޑިއިރު %d",d:"ދުވަހެއް",dd:"ދުވަސް %d",M:"މަހެއް",MM:"މަސް %d",y:"އަހަރެއް",yy:"އަހަރު %d"},preparse:function(a){return a.replace(/،/g,",")},postformat:function(a){return a.replace(/,/g,"،")},week:{dow:7,// Sunday is the first day of the week.
doy:12}}),kg.defineLocale("el",{monthsNominativeEl:"Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος".split("_"),monthsGenitiveEl:"Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου".split("_"),months:function(a,b){return/D/.test(b.substring(0,b.indexOf("MMMM")))?this._monthsGenitiveEl[a.month()]:this._monthsNominativeEl[a.month()]},monthsShort:"Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ".split("_"),weekdays:"Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο".split("_"),weekdaysShort:"Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ".split("_"),weekdaysMin:"Κυ_Δε_Τρ_Τε_Πε_Πα_Σα".split("_"),meridiem:function(a,b,c){return a>11?c?"μμ":"ΜΜ":c?"πμ":"ΠΜ"},isPM:function(a){return"μ"===(a+"").toLowerCase()[0]},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[Σήμερα {}] LT",nextDay:"[Αύριο {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[Χθες {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[το προηγούμενο] dddd [{}] LT";default:return"[την προηγούμενη] dddd [{}] LT"}},sameElse:"L"},calendar:function(a,b){var c=this._calendarEl[a],d=b&&b.hours();return y(c)&&(c=c.apply(b)),c.replace("{}",d%12===1?"στη":"στις")},relativeTime:{future:"σε %s",past:"%s πριν",s:"λίγα δευτερόλεπτα",m:"ένα λεπτό",mm:"%d λεπτά",h:"μία ώρα",hh:"%d ώρες",d:"μία μέρα",dd:"%d μέρες",M:"ένας μήνας",MM:"%d μήνες",y:"ένας χρόνος",yy:"%d χρόνια"},ordinalParse:/\d{1,2}η/,ordinal:"%dη",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},ordinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},ordinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c}}),kg.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},ordinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},ordinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},ordinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aŭg_sep_okt_nov_dec".split("_"),weekdays:"Dimanĉo_Lundo_Mardo_Merkredo_Ĵaŭdo_Vendredo_Sabato".split("_"),weekdaysShort:"Dim_Lun_Mard_Merk_Ĵaŭ_Ven_Sab".split("_"),weekdaysMin:"Di_Lu_Ma_Me_Ĵa_Ve_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D[-an de] MMMM, YYYY",LLL:"D[-an de] MMMM, YYYY HH:mm",LLLL:"dddd, [la] D[-an de] MMMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(a){return"p"===a.charAt(0).toLowerCase()},meridiem:function(a,b,c){return a>11?c?"p.t.m.":"P.T.M.":c?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodiaŭ je] LT",nextDay:"[Morgaŭ je] LT",nextWeek:"dddd [je] LT",lastDay:"[Hieraŭ je] LT",lastWeek:"[pasinta] dddd [je] LT",sameElse:"L"},relativeTime:{future:"je %s",past:"antaŭ %s",s:"sekundoj",m:"minuto",mm:"%d minutoj",h:"horo",hh:"%d horoj",d:"tago",//ne 'diurno', ĉar estas uzita por proksimumo
dd:"%d tagoj",M:"monato",MM:"%d monatoj",y:"jaro",yy:"%d jaroj"},ordinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,// Monday is the first day of the week.
doy:7}}),"ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_")),Ig="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),Jg=(kg.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(a,b){return/-MMM-/.test(b)?Ig[a.month()]:Hg[a.month()]},monthsParseExact:!0,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),"ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_")),Kg="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),Lg=(kg.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(a,b){return/-MMM-/.test(b)?Kg[a.month()]:Jg[a.month()]},monthsParseExact:!0,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("et",{months:"jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[Täna,] LT",nextDay:"[Homme,] LT",nextWeek:"[Järgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s pärast",past:"%s tagasi",s:yd,m:yd,mm:yd,h:yd,hh:yd,d:yd,dd:"%d päeva",M:yd,MM:yd,y:yd,yy:yd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),{1:"۱",2:"۲",3:"۳",4:"۴",5:"۵",6:"۶",7:"۷",8:"۸",9:"۹",0:"۰"}),Mg={"۱":"1","۲":"2","۳":"3","۴":"4","۵":"5","۶":"6","۷":"7","۸":"8","۹":"9","۰":"0"},Ng=(kg.defineLocale("fa",{months:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),monthsShort:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),weekdays:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysShort:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysMin:"ی_د_س_چ_پ_ج_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(a){return/بعد از ظهر/.test(a)},meridiem:function(a,b,c){return 12>a?"قبل از ظهر":"بعد از ظهر"},calendar:{sameDay:"[امروز ساعت] LT",nextDay:"[فردا ساعت] LT",nextWeek:"dddd [ساعت] LT",lastDay:"[دیروز ساعت] LT",lastWeek:"dddd [پیش] [ساعت] LT",sameElse:"L"},relativeTime:{future:"در %s",past:"%s پیش",s:"چندین ثانیه",m:"یک دقیقه",mm:"%d دقیقه",h:"یک ساعت",hh:"%d ساعت",d:"یک روز",dd:"%d روز",M:"یک ماه",MM:"%d ماه",y:"یک سال",yy:"%d سال"},preparse:function(a){return a.replace(/[۰-۹]/g,function(a){return Mg[a]}).replace(/،/g,",")},postformat:function(a){return a.replace(/\d/g,function(a){return Lg[a]}).replace(/,/g,"،")},ordinalParse:/\d{1,2}م/,ordinal:"%dم",week:{dow:6,// Saturday is the first day of the week.
doy:12}}),"nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän".split(" ")),Og=["nolla","yhden","kahden","kolmen","neljän","viiden","kuuden",Ng[7],Ng[8],Ng[9]],Pg=(kg.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[tänään] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s päästä",past:"%s sitten",s:zd,m:zd,mm:zd,h:zd,hh:zd,d:zd,dd:zd,M:zd,MM:zd,y:zd,yy:zd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("fo",{months:"januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur".split("_"),weekdaysShort:"sun_mán_týs_mik_hós_frí_ley".split("_"),weekdaysMin:"su_má_tý_mi_hó_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[Í dag kl.] LT",nextDay:"[Í morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[Í gjár kl.] LT",lastWeek:"[síðstu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s síðani",s:"fá sekund",m:"ein minutt",mm:"%d minuttir",h:"ein tími",hh:"%d tímar",d:"ein dagur",dd:"%d dagar",M:"ein mánaði",MM:"%d mánaðir",y:"eitt ár",yy:"%d ár"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("fr-ca",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"Di_Lu_Ma_Me_Je_Ve_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd'hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},ordinalParse:/\d{1,2}(er|e)/,ordinal:function(a){return a+(1===a?"er":"e")}}),kg.defineLocale("fr-ch",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"Di_Lu_Ma_Me_Je_Ve_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd'hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},ordinalParse:/\d{1,2}(er|e)/,ordinal:function(a){return a+(1===a?"er":"e")},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"Di_Lu_Ma_Me_Je_Ve_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd'hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},ordinalParse:/\d{1,2}(er|)/,ordinal:function(a){return a+(1===a?"er":"")},week:{dow:1,// Monday is the first day of the week.
doy:4}}),"jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_")),Qg="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),Rg=(kg.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(a,b){return/-MMM-/.test(b)?Qg[a.month()]:Pg[a.month()]},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[ôfrûne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",m:"ien minút",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},ordinalParse:/\d{1,2}(ste|de)/,ordinal:function(a){return a+(1===a||8===a||a>=20?"ste":"de")},week:{dow:1,// Monday is the first day of the week.
doy:4}}),["Am Faoilleach","An Gearran","Am Màrt","An Giblean","An Cèitean","An t-Ògmhios","An t-Iuchar","An Lùnastal","An t-Sultain","An Dàmhair","An t-Samhain","An Dùbhlachd"]),Sg=["Faoi","Gear","Màrt","Gibl","Cèit","Ògmh","Iuch","Lùn","Sult","Dàmh","Samh","Dùbh"],Tg=["Didòmhnaich","Diluain","Dimàirt","Diciadain","Diardaoin","Dihaoine","Disathairne"],Ug=["Did","Dil","Dim","Dic","Dia","Dih","Dis"],Vg=["Dò","Lu","Mà","Ci","Ar","Ha","Sa"],Wg=(kg.defineLocale("gd",{months:Rg,monthsShort:Sg,monthsParseExact:!0,weekdays:Tg,weekdaysShort:Ug,weekdaysMin:Vg,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-màireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-dè aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"mìos",MM:"%d mìosan",y:"bliadhna",yy:"%d bliadhna"},ordinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(a){var b=1===a?"d":a%10===2?"na":"mh";return a+b},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_mércores_xoves_venres_sábado".split("_"),weekdaysShort:"dom._lun._mar._mér._xov._ven._sáb.".split("_"),weekdaysMin:"do_lu_ma_mé_xo_ve_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(1!==this.hours()?"ás":"á")+"] LT"},nextDay:function(){return"[mañá "+(1!==this.hours()?"ás":"á")+"] LT"},nextWeek:function(){return"dddd ["+(1!==this.hours()?"ás":"a")+"] LT"},lastDay:function(){return"[onte "+(1!==this.hours()?"á":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(1!==this.hours()?"ás":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(a){return 0===a.indexOf("un")?"n"+a:"en "+a},past:"hai %s",s:"uns segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("he",{months:"ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר".split("_"),monthsShort:"ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳".split("_"),weekdays:"ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת".split("_"),weekdaysShort:"א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳".split("_"),weekdaysMin:"א_ב_ג_ד_ה_ו_ש".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [ב]MMMM YYYY",LLL:"D [ב]MMMM YYYY HH:mm",LLLL:"dddd, D [ב]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[היום ב־]LT",nextDay:"[מחר ב־]LT",nextWeek:"dddd [בשעה] LT",lastDay:"[אתמול ב־]LT",lastWeek:"[ביום] dddd [האחרון בשעה] LT",sameElse:"L"},relativeTime:{future:"בעוד %s",past:"לפני %s",s:"מספר שניות",m:"דקה",mm:"%d דקות",h:"שעה",hh:function(a){return 2===a?"שעתיים":a+" שעות"},d:"יום",dd:function(a){return 2===a?"יומיים":a+" ימים"},M:"חודש",MM:function(a){return 2===a?"חודשיים":a+" חודשים"},y:"שנה",yy:function(a){return 2===a?"שנתיים":a%10===0&&10!==a?a+" שנה":a+" שנים"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(a){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(a)},meridiem:function(a,b,c){return 5>a?"לפנות בוקר":10>a?"בבוקר":12>a?c?'לפנה"צ':"לפני הצהריים":18>a?c?'אחה"צ':"אחרי הצהריים":"בערב"}}),{1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"}),Xg={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},Yg=(kg.defineLocale("hi",{months:"जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर".split("_"),monthsShort:"जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm बजे",LTS:"A h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm बजे",LLLL:"dddd, D MMMM YYYY, A h:mm बजे"},calendar:{sameDay:"[आज] LT",nextDay:"[कल] LT",nextWeek:"dddd, LT",lastDay:"[कल] LT",lastWeek:"[पिछले] dddd, LT",sameElse:"L"},relativeTime:{future:"%s में",past:"%s पहले",s:"कुछ ही क्षण",m:"एक मिनट",mm:"%d मिनट",h:"एक घंटा",hh:"%d घंटे",d:"एक दिन",dd:"%d दिन",M:"एक महीने",MM:"%d महीने",y:"एक वर्ष",yy:"%d वर्ष"},preparse:function(a){return a.replace(/[१२३४५६७८९०]/g,function(a){return Xg[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return Wg[a]})},
// Hindi notation for meridiems are quite fuzzy in practice. While there exists
// a rigid notion of a 'Pahar' it is not used as rigidly in modern Hindi.
meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(a,b){return 12===a&&(a=0),"रात"===b?4>a?a:a+12:"सुबह"===b?a:"दोपहर"===b?a>=10?a:a+12:"शाम"===b?a+12:void 0},meridiem:function(a,b,c){return 4>a?"रात":10>a?"सुबह":17>a?"दोपहर":20>a?"शाम":"रात"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),kg.defineLocale("hr",{months:{format:"siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",m:Bd,mm:Bd,h:Bd,hh:Bd,d:"dan",dd:Bd,M:"mjesec",MM:Bd,y:"godinu",yy:Bd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),"vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton".split(" ")),Zg=(kg.defineLocale("hu",{months:"január_február_március_április_május_június_július_augusztus_szeptember_október_november_december".split("_"),monthsShort:"jan_feb_márc_ápr_máj_jún_júl_aug_szept_okt_nov_dec".split("_"),weekdays:"vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat".split("_"),weekdaysShort:"vas_hét_kedd_sze_csüt_pén_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(a){return"u"===a.charAt(1).toLowerCase()},meridiem:function(a,b,c){return 12>a?c===!0?"de":"DE":c===!0?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return Dd.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return Dd.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s múlva",past:"%s",s:Cd,m:Cd,mm:Cd,h:Cd,hh:Cd,d:Cd,dd:Cd,M:Cd,MM:Cd,y:Cd,yy:Cd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("hy-am",{months:{format:"հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի".split("_"),standalone:"հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր".split("_")},monthsShort:"հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ".split("_"),weekdays:"կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ".split("_"),weekdaysShort:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),weekdaysMin:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY թ.",LLL:"D MMMM YYYY թ., HH:mm",LLLL:"dddd, D MMMM YYYY թ., HH:mm"},calendar:{sameDay:"[այսօր] LT",nextDay:"[վաղը] LT",lastDay:"[երեկ] LT",nextWeek:function(){return"dddd [օրը ժամը] LT"},lastWeek:function(){return"[անցած] dddd [օրը ժամը] LT"},sameElse:"L"},relativeTime:{future:"%s հետո",past:"%s առաջ",s:"մի քանի վայրկյան",m:"րոպե",mm:"%d րոպե",h:"ժամ",hh:"%d ժամ",d:"օր",dd:"%d օր",M:"ամիս",MM:"%d ամիս",y:"տարի",yy:"%d տարի"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(a){return/^(ցերեկվա|երեկոյան)$/.test(a)},meridiem:function(a){return 4>a?"գիշերվա":12>a?"առավոտվա":17>a?"ցերեկվա":"երեկոյան"},ordinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(a,b){switch(b){case"DDD":case"w":case"W":case"DDDo":return 1===a?a+"-ին":a+"-րդ";default:return a}},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(a,b){return 12===a&&(a=0),"pagi"===b?a:"siang"===b?a>=11?a:a+12:"sore"===b||"malam"===b?a+12:void 0},meridiem:function(a,b,c){return 11>a?"pagi":15>a?"siang":19>a?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("is",{months:"janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember".split("_"),monthsShort:"jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des".split("_"),weekdays:"sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur".split("_"),weekdaysShort:"sun_mán_þri_mið_fim_fös_lau".split("_"),weekdaysMin:"Su_Má_Þr_Mi_Fi_Fö_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[í dag kl.] LT",nextDay:"[á morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[í gær kl.] LT",lastWeek:"[síðasta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s síðan",s:Fd,m:Fd,mm:Fd,h:"klukkustund",hh:Fd,d:Fd,dd:Fd,M:Fd,MM:Fd,y:Fd,yy:Fd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"Domenica_Lunedì_Martedì_Mercoledì_Giovedì_Venerdì_Sabato".split("_"),weekdaysShort:"Dom_Lun_Mar_Mer_Gio_Ven_Sab".split("_"),weekdaysMin:"Do_Lu_Ma_Me_Gi_Ve_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(a){return(/^[0-9].+$/.test(a)?"tra":"in")+" "+a},past:"%s fa",s:"alcuni secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("ja",{months:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日".split("_"),weekdaysShort:"日_月_火_水_木_金_土".split("_"),weekdaysMin:"日_月_火_水_木_金_土".split("_"),longDateFormat:{LT:"Ah時m分",LTS:"Ah時m分s秒",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日Ah時m分",LLLL:"YYYY年M月D日Ah時m分 dddd"},meridiemParse:/午前|午後/i,isPM:function(a){return"午後"===a},meridiem:function(a,b,c){return 12>a?"午前":"午後"},calendar:{sameDay:"[今日] LT",nextDay:"[明日] LT",nextWeek:"[来週]dddd LT",lastDay:"[昨日] LT",lastWeek:"[前週]dddd LT",sameElse:"L"},ordinalParse:/\d{1,2}日/,ordinal:function(a,b){switch(b){case"d":case"D":case"DDD":return a+"日";default:return a}},relativeTime:{future:"%s後",past:"%s前",s:"数秒",m:"1分",mm:"%d分",h:"1時間",hh:"%d時間",d:"1日",dd:"%d日",M:"1ヶ月",MM:"%dヶ月",y:"1年",yy:"%d年"}}),kg.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(a,b){return 12===a&&(a=0),"enjing"===b?a:"siyang"===b?a>=11?a:a+12:"sonten"===b||"ndalu"===b?a+12:void 0},meridiem:function(a,b,c){return 11>a?"enjing":15>a?"siyang":19>a?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("ka",{months:{standalone:"იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი".split("_"),format:"იანვარს_თებერვალს_მარტს_აპრილის_მაისს_ივნისს_ივლისს_აგვისტს_სექტემბერს_ოქტომბერს_ნოემბერს_დეკემბერს".split("_")},monthsShort:"იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ".split("_"),weekdays:{standalone:"კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი".split("_"),format:"კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ".split("_"),weekdaysMin:"კვ_ორ_სა_ოთ_ხუ_პა_შა".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[დღეს] LT[-ზე]",nextDay:"[ხვალ] LT[-ზე]",lastDay:"[გუშინ] LT[-ზე]",nextWeek:"[შემდეგ] dddd LT[-ზე]",lastWeek:"[წინა] dddd LT-ზე",sameElse:"L"},relativeTime:{future:function(a){return/(წამი|წუთი|საათი|წელი)/.test(a)?a.replace(/ი$/,"ში"):a+"ში"},past:function(a){return/(წამი|წუთი|საათი|დღე|თვე)/.test(a)?a.replace(/(ი|ე)$/,"ის წინ"):/წელი/.test(a)?a.replace(/წელი$/,"წლის წინ"):void 0},s:"რამდენიმე წამი",m:"წუთი",mm:"%d წუთი",h:"საათი",hh:"%d საათი",d:"დღე",dd:"%d დღე",M:"თვე",MM:"%d თვე",y:"წელი",yy:"%d წელი"},ordinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(a){return 0===a?a:1===a?a+"-ლი":20>a||100>=a&&a%20===0||a%100===0?"მე-"+a:a+"-ე"},week:{dow:1,doy:7}}),{0:"-ші",1:"-ші",2:"-ші",3:"-ші",4:"-ші",5:"-ші",6:"-шы",7:"-ші",8:"-ші",9:"-шы",10:"-шы",20:"-шы",30:"-шы",40:"-шы",50:"-ші",60:"-шы",70:"-ші",80:"-ші",90:"-шы",100:"-ші"}),$g=(kg.defineLocale("kk",{months:"қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан".split("_"),monthsShort:"қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел".split("_"),weekdays:"жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі".split("_"),weekdaysShort:"жек_дүй_сей_сәр_бей_жұм_сен".split("_"),weekdaysMin:"жк_дй_сй_ср_бй_жм_сн".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгін сағат] LT",nextDay:"[Ертең сағат] LT",nextWeek:"dddd [сағат] LT",lastDay:"[Кеше сағат] LT",lastWeek:"[Өткен аптаның] dddd [сағат] LT",sameElse:"L"},relativeTime:{future:"%s ішінде",past:"%s бұрын",s:"бірнеше секунд",m:"бір минут",mm:"%d минут",h:"бір сағат",hh:"%d сағат",d:"бір күн",dd:"%d күн",M:"бір ай",MM:"%d ай",y:"бір жыл",yy:"%d жыл"},ordinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(a){var b=a%10,c=a>=100?100:null;return a+(Zg[a]||Zg[b]||Zg[c])},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("km",{months:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),monthsShort:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),weekdays:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysShort:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysMin:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[ថ្ងៃនេះ ម៉ោង] LT",nextDay:"[ស្អែក ម៉ោង] LT",nextWeek:"dddd [ម៉ោង] LT",lastDay:"[ម្សិលមិញ ម៉ោង] LT",lastWeek:"dddd [សប្តាហ៍មុន] [ម៉ោង] LT",sameElse:"L"},relativeTime:{future:"%sទៀត",past:"%sមុន",s:"ប៉ុន្មានវិនាទី",m:"មួយនាទី",mm:"%d នាទី",h:"មួយម៉ោង",hh:"%d ម៉ោង",d:"មួយថ្ងៃ",dd:"%d ថ្ងៃ",M:"មួយខែ",MM:"%d ខែ",y:"មួយឆ្នាំ",yy:"%d ឆ្នាំ"},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("ko",{months:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),monthsShort:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),weekdays:"일요일_월요일_화요일_수요일_목요일_금요일_토요일".split("_"),weekdaysShort:"일_월_화_수_목_금_토".split("_"),weekdaysMin:"일_월_화_수_목_금_토".split("_"),longDateFormat:{LT:"A h시 m분",LTS:"A h시 m분 s초",L:"YYYY.MM.DD",LL:"YYYY년 MMMM D일",LLL:"YYYY년 MMMM D일 A h시 m분",LLLL:"YYYY년 MMMM D일 dddd A h시 m분"},calendar:{sameDay:"오늘 LT",nextDay:"내일 LT",nextWeek:"dddd LT",lastDay:"어제 LT",lastWeek:"지난주 dddd LT",sameElse:"L"},relativeTime:{future:"%s 후",past:"%s 전",s:"몇 초",ss:"%d초",m:"일분",mm:"%d분",h:"한 시간",hh:"%d시간",d:"하루",dd:"%d일",M:"한 달",MM:"%d달",y:"일 년",yy:"%d년"},ordinalParse:/\d{1,2}일/,ordinal:"%d일",meridiemParse:/오전|오후/,isPM:function(a){return"오후"===a},meridiem:function(a,b,c){return 12>a?"오전":"오후"}}),{0:"-чү",1:"-чи",2:"-чи",3:"-чү",4:"-чү",5:"-чи",6:"-чы",7:"-чи",8:"-чи",9:"-чу",10:"-чу",20:"-чы",30:"-чу",40:"-чы",50:"-чү",60:"-чы",70:"-чи",80:"-чи",90:"-чу",100:"-чү"}),_g=(kg.defineLocale("ky",{months:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_"),monthsShort:"янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек".split("_"),weekdays:"Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби".split("_"),weekdaysShort:"Жек_Дүй_Шей_Шар_Бей_Жум_Ише".split("_"),weekdaysMin:"Жк_Дй_Шй_Шр_Бй_Жм_Иш".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгүн саат] LT",nextDay:"[Эртең саат] LT",nextWeek:"dddd [саат] LT",lastDay:"[Кече саат] LT",lastWeek:"[Өткен аптанын] dddd [күнү] [саат] LT",sameElse:"L"},relativeTime:{future:"%s ичинде",past:"%s мурун",s:"бирнече секунд",m:"бир мүнөт",mm:"%d мүнөт",h:"бир саат",hh:"%d саат",d:"бир күн",dd:"%d күн",M:"бир ай",MM:"%d ай",y:"бир жыл",yy:"%d жыл"},ordinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(a){var b=a%10,c=a>=100?100:null;return a+($g[a]||$g[b]||$g[c])},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("lb",{months:"Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._Mé._Dë._Më._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mé_Dë_Më_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[Gëschter um] LT",lastWeek:function(){
// Different date string for 'Dënschdeg' (Tuesday) and 'Donneschdeg' (Thursday) due to phonological rule
switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:Hd,past:Id,s:"e puer Sekonnen",m:Gd,mm:"%d Minutten",h:Gd,hh:"%d Stonnen",d:Gd,dd:"%d Deeg",M:Gd,MM:"%d Méint",y:Gd,yy:"%d Joer"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("lo",{months:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),monthsShort:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),weekdays:"ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysShort:"ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysMin:"ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"ວັນdddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(a){return"ຕອນແລງ"===a},meridiem:function(a,b,c){return 12>a?"ຕອນເຊົ້າ":"ຕອນແລງ"},calendar:{sameDay:"[ມື້ນີ້ເວລາ] LT",nextDay:"[ມື້ອື່ນເວລາ] LT",nextWeek:"[ວັນ]dddd[ໜ້າເວລາ] LT",lastDay:"[ມື້ວານນີ້ເວລາ] LT",lastWeek:"[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT",sameElse:"L"},relativeTime:{future:"ອີກ %s",past:"%sຜ່ານມາ",s:"ບໍ່ເທົ່າໃດວິນາທີ",m:"1 ນາທີ",mm:"%d ນາທີ",h:"1 ຊົ່ວໂມງ",hh:"%d ຊົ່ວໂມງ",d:"1 ມື້",dd:"%d ມື້",M:"1 ເດືອນ",MM:"%d ເດືອນ",y:"1 ປີ",yy:"%d ປີ"},ordinalParse:/(ທີ່)\d{1,2}/,ordinal:function(a){return"ທີ່"+a}}),{m:"minutė_minutės_minutę",mm:"minutės_minučių_minutes",h:"valanda_valandos_valandą",hh:"valandos_valandų_valandas",d:"diena_dienos_dieną",dd:"dienos_dienų_dienas",M:"mėnuo_mėnesio_mėnesį",MM:"mėnesiai_mėnesių_mėnesius",y:"metai_metų_metus",yy:"metai_metų_metus"}),ah=(kg.defineLocale("lt",{months:{format:"sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s+)+MMMM?|MMMM?(\[[^\[\]]*\]|\s+)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_Šeš".split("_"),weekdaysMin:"S_P_A_T_K_Pn_Š".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[Šiandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Praėjusį] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prieš %s",s:Kd,m:Ld,mm:Od,h:Ld,hh:Od,d:Ld,dd:Od,M:Ld,MM:Od,y:Ld,yy:Od},ordinalParse:/\d{1,2}-oji/,ordinal:function(a){return a+"-oji"},week:{dow:1,// Monday is the first day of the week.
doy:4}}),{m:"minūtes_minūtēm_minūte_minūtes".split("_"),mm:"minūtes_minūtēm_minūte_minūtes".split("_"),h:"stundas_stundām_stunda_stundas".split("_"),hh:"stundas_stundām_stunda_stundas".split("_"),d:"dienas_dienām_diena_dienas".split("_"),dd:"dienas_dienām_diena_dienas".split("_"),M:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),MM:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")}),bh=(kg.defineLocale("lv",{months:"janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec".split("_"),weekdays:"svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[Šodien pulksten] LT",nextDay:"[Rīt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pagājušā] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"pēc %s",past:"pirms %s",s:Sd,m:Rd,mm:Qd,h:Rd,hh:Qd,d:Rd,dd:Qd,M:Rd,MM:Qd,y:Rd,yy:Qd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),{words:{//Different grammatical cases
m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(a,b){return 1===a?b[0]:a>=2&&4>=a?b[1]:b[2]},translate:function(a,b,c){var d=bh.words[c];return 1===c.length?b?d[0]:d[1]:a+" "+bh.correctGrammaticalCase(a,d)}}),ch=(kg.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var a=["[prošle] [nedjelje] [u] LT","[prošlog] [ponedjeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srijede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return a[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",m:bh.translate,mm:bh.translate,h:bh.translate,hh:bh.translate,d:"dan",dd:bh.translate,M:"mjesec",MM:bh.translate,y:"godinu",yy:bh.translate},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("mi",{months:"Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei".split("_"),weekdaysShort:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),weekdaysMin:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te hēkona ruarua",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("mk",{months:"јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември".split("_"),monthsShort:"јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек".split("_"),weekdays:"недела_понеделник_вторник_среда_четврток_петок_сабота".split("_"),weekdaysShort:"нед_пон_вто_сре_чет_пет_саб".split("_"),weekdaysMin:"нe_пo_вт_ср_че_пе_сa".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Денес во] LT",nextDay:"[Утре во] LT",nextWeek:"[Во] dddd [во] LT",lastDay:"[Вчера во] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Изминатата] dddd [во] LT";case 1:case 2:case 4:case 5:return"[Изминатиот] dddd [во] LT"}},sameElse:"L"},relativeTime:{future:"после %s",past:"пред %s",s:"неколку секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дена",M:"месец",MM:"%d месеци",y:"година",yy:"%d години"},ordinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(a){var b=a%10,c=a%100;return 0===a?a+"-ев":0===c?a+"-ен":c>10&&20>c?a+"-ти":1===b?a+"-ви":2===b?a+"-ри":7===b||8===b?a+"-ми":a+"-ти"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("ml",{months:"ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ".split("_"),monthsShort:"ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.".split("_"),monthsParseExact:!0,weekdays:"ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച".split("_"),weekdaysShort:"ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി".split("_"),weekdaysMin:"ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ".split("_"),longDateFormat:{LT:"A h:mm -നു",LTS:"A h:mm:ss -നു",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -നു",LLLL:"dddd, D MMMM YYYY, A h:mm -നു"},calendar:{sameDay:"[ഇന്ന്] LT",nextDay:"[നാളെ] LT",nextWeek:"dddd, LT",lastDay:"[ഇന്നലെ] LT",lastWeek:"[കഴിഞ്ഞ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s കഴിഞ്ഞ്",past:"%s മുൻപ്",s:"അൽപ നിമിഷങ്ങൾ",m:"ഒരു മിനിറ്റ്",mm:"%d മിനിറ്റ്",h:"ഒരു മണിക്കൂർ",hh:"%d മണിക്കൂർ",d:"ഒരു ദിവസം",dd:"%d ദിവസം",M:"ഒരു മാസം",MM:"%d മാസം",y:"ഒരു വർഷം",yy:"%d വർഷം"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(a,b){return 12===a&&(a=0),"രാത്രി"===b&&a>=4||"ഉച്ച കഴിഞ്ഞ്"===b||"വൈകുന്നേരം"===b?a+12:a},meridiem:function(a,b,c){return 4>a?"രാത്രി":12>a?"രാവിലെ":17>a?"ഉച്ച കഴിഞ്ഞ്":20>a?"വൈകുന്നേരം":"രാത്രി"}}),{1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"}),dh={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},eh=(kg.defineLocale("mr",{months:"जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),monthsShort:"जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm वाजता",LTS:"A h:mm:ss वाजता",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm वाजता",LLLL:"dddd, D MMMM YYYY, A h:mm वाजता"},calendar:{sameDay:"[आज] LT",nextDay:"[उद्या] LT",nextWeek:"dddd, LT",lastDay:"[काल] LT",lastWeek:"[मागील] dddd, LT",sameElse:"L"},relativeTime:{future:"%sमध्ये",past:"%sपूर्वी",s:Td,m:Td,mm:Td,h:Td,hh:Td,d:Td,dd:Td,M:Td,MM:Td,y:Td,yy:Td},preparse:function(a){return a.replace(/[१२३४५६७८९०]/g,function(a){return dh[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return ch[a]})},meridiemParse:/रात्री|सकाळी|दुपारी|सायंकाळी/,meridiemHour:function(a,b){return 12===a&&(a=0),"रात्री"===b?4>a?a:a+12:"सकाळी"===b?a:"दुपारी"===b?a>=10?a:a+12:"सायंकाळी"===b?a+12:void 0},meridiem:function(a,b,c){return 4>a?"रात्री":10>a?"सकाळी":17>a?"दुपारी":20>a?"सायंकाळी":"रात्री"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),kg.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(a,b){return 12===a&&(a=0),"pagi"===b?a:"tengahari"===b?a>=11?a:a+12:"petang"===b||"malam"===b?a+12:void 0},meridiem:function(a,b,c){return 11>a?"pagi":15>a?"tengahari":19>a?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(a,b){return 12===a&&(a=0),"pagi"===b?a:"tengahari"===b?a>=11?a:a+12:"petang"===b||"malam"===b?a+12:void 0},meridiem:function(a,b,c){return 11>a?"pagi":15>a?"tengahari":19>a?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),{1:"၁",2:"၂",3:"၃",4:"၄",5:"၅",6:"၆",7:"၇",8:"၈",9:"၉",0:"၀"}),fh={"၁":"1","၂":"2","၃":"3","၄":"4","၅":"5","၆":"6","၇":"7","၈":"8","၉":"9","၀":"0"},gh=(kg.defineLocale("my",{months:"ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ".split("_"),monthsShort:"ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ".split("_"),weekdays:"တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ".split("_"),weekdaysShort:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),weekdaysMin:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ယနေ.] LT [မှာ]",nextDay:"[မနက်ဖြန်] LT [မှာ]",nextWeek:"dddd LT [မှာ]",lastDay:"[မနေ.က] LT [မှာ]",lastWeek:"[ပြီးခဲ့သော] dddd LT [မှာ]",sameElse:"L"},relativeTime:{future:"လာမည့် %s မှာ",past:"လွန်ခဲ့သော %s က",s:"စက္ကန်.အနည်းငယ်",m:"တစ်မိနစ်",mm:"%d မိနစ်",h:"တစ်နာရီ",hh:"%d နာရီ",d:"တစ်ရက်",dd:"%d ရက်",M:"တစ်လ",MM:"%d လ",y:"တစ်နှစ်",yy:"%d နှစ်"},preparse:function(a){return a.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,function(a){return fh[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return eh[a]})},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_april_mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"sø._ma._ti._on._to._fr._lø.".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",m:"ett minutt",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dager",M:"en måned",MM:"%d måneder",y:"ett år",yy:"%d år"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),{1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"}),hh={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},ih=(kg.defineLocale("ne",{months:"जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर".split("_"),monthsShort:"जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.".split("_"),monthsParseExact:!0,weekdays:"आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार".split("_"),weekdaysShort:"आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.".split("_"),weekdaysMin:"आ._सो._मं._बु._बि._शु._श.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"Aको h:mm बजे",LTS:"Aको h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, Aको h:mm बजे",LLLL:"dddd, D MMMM YYYY, Aको h:mm बजे"},preparse:function(a){return a.replace(/[१२३४५६७८९०]/g,function(a){return hh[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return gh[a]})},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(a,b){return 12===a&&(a=0),"राति"===b?4>a?a:a+12:"बिहान"===b?a:"दिउँसो"===b?a>=10?a:a+12:"साँझ"===b?a+12:void 0},meridiem:function(a,b,c){return 3>a?"राति":12>a?"बिहान":16>a?"दिउँसो":20>a?"साँझ":"राति"},calendar:{sameDay:"[आज] LT",nextDay:"[भोलि] LT",nextWeek:"[आउँदो] dddd[,] LT",lastDay:"[हिजो] LT",lastWeek:"[गएको] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%sमा",past:"%s अगाडि",s:"केही क्षण",m:"एक मिनेट",mm:"%d मिनेट",h:"एक घण्टा",hh:"%d घण्टा",d:"एक दिन",dd:"%d दिन",M:"एक महिना",MM:"%d महिना",y:"एक बर्ष",yy:"%d बर्ष"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),"jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_")),jh="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),kh=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],lh=/^(januari|februari|maart|april|mei|april|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,mh=(kg.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(a,b){return/-MMM-/.test(b)?jh[a.month()]:ih[a.month()]},monthsRegex:lh,monthsShortRegex:lh,monthsStrictRegex:/^(januari|februari|maart|mei|ju[nl]i|april|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:kh,longMonthsParse:kh,shortMonthsParse:kh,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"Zo_Ma_Di_Wo_Do_Vr_Za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},ordinalParse:/\d{1,2}(ste|de)/,ordinal:function(a){return a+(1===a||8===a||a>=20?"ste":"de")},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"sun_mån_tys_ons_tor_fre_lau".split("_"),weekdaysMin:"su_må_ty_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I går klokka] LT",lastWeek:"[Føregåande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",M:"ein månad",MM:"%d månader",y:"eit år",yy:"%d år"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),{1:"੧",2:"੨",3:"੩",4:"੪",5:"੫",6:"੬",7:"੭",8:"੮",9:"੯",0:"੦"}),nh={"੧":"1","੨":"2","੩":"3","੪":"4","੫":"5","੬":"6","੭":"7","੮":"8","੯":"9","੦":"0"},oh=(kg.defineLocale("pa-in",{
// There are months name as per Nanakshahi Calender but they are not used as rigidly in modern Punjabi.
months:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),monthsShort:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),weekdays:"ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ".split("_"),weekdaysShort:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),weekdaysMin:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),longDateFormat:{LT:"A h:mm ਵਜੇ",LTS:"A h:mm:ss ਵਜੇ",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm ਵਜੇ",LLLL:"dddd, D MMMM YYYY, A h:mm ਵਜੇ"},calendar:{sameDay:"[ਅਜ] LT",nextDay:"[ਕਲ] LT",nextWeek:"dddd, LT",lastDay:"[ਕਲ] LT",lastWeek:"[ਪਿਛਲੇ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ਵਿੱਚ",past:"%s ਪਿਛਲੇ",s:"ਕੁਝ ਸਕਿੰਟ",m:"ਇਕ ਮਿੰਟ",mm:"%d ਮਿੰਟ",h:"ਇੱਕ ਘੰਟਾ",hh:"%d ਘੰਟੇ",d:"ਇੱਕ ਦਿਨ",dd:"%d ਦਿਨ",M:"ਇੱਕ ਮਹੀਨਾ",MM:"%d ਮਹੀਨੇ",y:"ਇੱਕ ਸਾਲ",yy:"%d ਸਾਲ"},preparse:function(a){return a.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,function(a){return nh[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return mh[a]})},
// Punjabi notation for meridiems are quite fuzzy in practice. While there exists
// a rigid notion of a 'Pahar' it is not used as rigidly in modern Punjabi.
meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(a,b){return 12===a&&(a=0),"ਰਾਤ"===b?4>a?a:a+12:"ਸਵੇਰ"===b?a:"ਦੁਪਹਿਰ"===b?a>=10?a:a+12:"ਸ਼ਾਮ"===b?a+12:void 0},meridiem:function(a,b,c){return 4>a?"ਰਾਤ":10>a?"ਸਵੇਰ":17>a?"ਦੁਪਹਿਰ":20>a?"ਸ਼ਾਮ":"ਰਾਤ"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),"styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień".split("_")),ph="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia".split("_"),qh=(kg.defineLocale("pl",{months:function(a,b){return""===b?"("+ph[a.month()]+"|"+oh[a.month()]+")":/D MMMM/.test(b)?ph[a.month()]:oh[a.month()]},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru".split("_"),weekdays:"niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota".split("_"),weekdaysShort:"nie_pon_wt_śr_czw_pt_sb".split("_"),weekdaysMin:"Nd_Pn_Wt_Śr_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dziś o] LT",nextDay:"[Jutro o] LT",nextWeek:"[W] dddd [o] LT",lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zeszłą niedzielę o] LT";case 3:return"[W zeszłą środę o] LT";case 6:return"[W zeszłą sobotę o] LT";default:return"[W zeszły] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",m:Vd,mm:Vd,h:Vd,hh:Vd,d:"1 dzień",dd:"%d dni",M:"miesiąc",MM:Vd,y:"rok",yy:Vd},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("pt-br",{months:"Janeiro_Fevereiro_Março_Abril_Maio_Junho_Julho_Agosto_Setembro_Outubro_Novembro_Dezembro".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Dom_2ª_3ª_4ª_5ª_6ª_Sáb".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [às] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [às] HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){// Saturday + Sunday
return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"%s atrás",s:"poucos segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},ordinalParse:/\d{1,2}º/,ordinal:"%dº"}),kg.defineLocale("pt",{months:"Janeiro_Fevereiro_Março_Abril_Maio_Junho_Julho_Agosto_Setembro_Outubro_Novembro_Dezembro".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingo_Segunda-Feira_Terça-Feira_Quarta-Feira_Quinta-Feira_Sexta-Feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Dom_2ª_3ª_4ª_5ª_6ª_Sáb".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){// Saturday + Sunday
return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},ordinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._febr._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminică_luni_marți_miercuri_joi_vineri_sâmbătă".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_Sâm".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_Sâ".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[mâine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s în urmă",s:"câteva secunde",m:"un minut",mm:Wd,h:"o oră",hh:Wd,d:"o zi",dd:Wd,M:"o lună",MM:Wd,y:"un an",yy:Wd},week:{dow:1,// Monday is the first day of the week.
doy:7}}),[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i]),rh=(kg.defineLocale("ru",{months:{format:"января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря".split("_"),standalone:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_")},monthsShort:{
// по CLDR именно "июл." и "июн.", но какой смысл менять букву на точку ?
format:"янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.".split("_"),standalone:"янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.".split("_")},weekdays:{standalone:"воскресенье_понедельник_вторник_среда_четверг_пятница_суббота".split("_"),format:"воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?\] ?dddd/},weekdaysShort:"вс_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"вс_пн_вт_ср_чт_пт_сб".split("_"),monthsParse:qh,longMonthsParse:qh,shortMonthsParse:qh,
// полные названия с падежами, по три буквы, для некоторых, по 4 буквы, сокращения с точкой и без точки
monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
// копия предыдущего
monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
// полные названия с падежами
monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,
// Выражение, которое соотвествует только сокращённым формам
monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сегодня в] LT",nextDay:"[Завтра в] LT",lastDay:"[Вчера в] LT",nextWeek:function(a){if(a.week()===this.week())return 2===this.day()?"[Во] dddd [в] LT":"[В] dddd [в] LT";switch(this.day()){case 0:return"[В следующее] dddd [в] LT";case 1:case 2:case 4:return"[В следующий] dddd [в] LT";case 3:case 5:case 6:return"[В следующую] dddd [в] LT"}},lastWeek:function(a){if(a.week()===this.week())return 2===this.day()?"[Во] dddd [в] LT":"[В] dddd [в] LT";switch(this.day()){case 0:return"[В прошлое] dddd [в] LT";case 1:case 2:case 4:return"[В прошлый] dddd [в] LT";case 3:case 5:case 6:return"[В прошлую] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"через %s",past:"%s назад",s:"несколько секунд",m:Yd,mm:Yd,h:"час",hh:Yd,d:"день",dd:Yd,M:"месяц",MM:Yd,y:"год",yy:Yd},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(a){return/^(дня|вечера)$/.test(a)},meridiem:function(a,b,c){return 4>a?"ночи":12>a?"утра":17>a?"дня":"вечера"},ordinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(a,b){switch(b){case"M":case"d":case"DDD":return a+"-й";case"D":return a+"-го";case"w":case"W":return a+"-я";default:return a}},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("se",{months:"ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu".split("_"),monthsShort:"ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov".split("_"),weekdays:"sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat".split("_"),weekdaysShort:"sotn_vuos_maŋ_gask_duor_bear_láv".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s geažes",past:"maŋit %s",s:"moadde sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta mánnu",MM:"%d mánut",y:"okta jahki",yy:"%d jagit"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("si",{months:"ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්".split("_"),monthsShort:"ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ".split("_"),weekdays:"ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා".split("_"),weekdaysShort:"ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන".split("_"),weekdaysMin:"ඉ_ස_අ_බ_බ්‍ර_සි_සෙ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [වැනි] dddd, a h:mm:ss"},calendar:{sameDay:"[අද] LT[ට]",nextDay:"[හෙට] LT[ට]",nextWeek:"dddd LT[ට]",lastDay:"[ඊයේ] LT[ට]",lastWeek:"[පසුගිය] dddd LT[ට]",sameElse:"L"},relativeTime:{future:"%sකින්",past:"%sකට පෙර",s:"තත්පර කිහිපය",m:"මිනිත්තුව",mm:"මිනිත්තු %d",h:"පැය",hh:"පැය %d",d:"දිනය",dd:"දින %d",M:"මාසය",MM:"මාස %d",y:"වසර",yy:"වසර %d"},ordinalParse:/\d{1,2} වැනි/,ordinal:function(a){return a+" වැනි"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(a){return"ප.ව."===a||"පස් වරු"===a},meridiem:function(a,b,c){return a>11?c?"ප.ව.":"පස් වරු":c?"පෙ.ව.":"පෙර වරු"}}),"január_február_marec_apríl_máj_jún_júl_august_september_október_november_december".split("_")),sh="jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec".split("_"),th=(kg.defineLocale("sk",{months:rh,monthsShort:sh,weekdays:"nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_št_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_št_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nedeľu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo štvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[včera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulú nedeľu o] LT";case 1:case 2:return"[minulý] dddd [o] LT";case 3:return"[minulú stredu o] LT";case 4:case 5:return"[minulý] dddd [o] LT";case 6:return"[minulú sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:$d,m:$d,mm:$d,h:$d,hh:$d,d:$d,dd:$d,M:$d,MM:$d,y:$d,yy:$d},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._čet._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_če_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[včeraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prejšnjo] [nedeljo] [ob] LT";case 3:return"[prejšnjo] [sredo] [ob] LT";case 6:return"[prejšnjo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prejšnji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"čez %s",past:"pred %s",s:_d,m:_d,mm:_d,h:_d,hh:_d,d:_d,dd:_d,M:_d,MM:_d,y:_d,yy:_d},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj".split("_"),weekdays:"E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë".split("_"),weekdaysShort:"Die_Hën_Mar_Mër_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_Më_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(a){return"M"===a.charAt(0)},meridiem:function(a,b,c){return 12>a?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot në] LT",nextDay:"[Nesër në] LT",nextWeek:"dddd [në] LT",lastDay:"[Dje në] LT",lastWeek:"dddd [e kaluar në] LT",sameElse:"L"},relativeTime:{future:"në %s",past:"%s më parë",s:"disa sekonda",m:"një minutë",mm:"%d minuta",h:"një orë",hh:"%d orë",d:"një ditë",dd:"%d ditë",M:"një muaj",MM:"%d muaj",y:"një vit",yy:"%d vite"},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),{words:{//Different grammatical cases
m:["један минут","једне минуте"],mm:["минут","минуте","минута"],h:["један сат","једног сата"],hh:["сат","сата","сати"],dd:["дан","дана","дана"],MM:["месец","месеца","месеци"],yy:["година","године","година"]},correctGrammaticalCase:function(a,b){return 1===a?b[0]:a>=2&&4>=a?b[1]:b[2]},translate:function(a,b,c){var d=th.words[c];return 1===c.length?b?d[0]:d[1]:a+" "+th.correctGrammaticalCase(a,d)}}),uh=(kg.defineLocale("sr-cyrl",{months:"јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар".split("_"),monthsShort:"јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.".split("_"),monthsParseExact:!0,weekdays:"недеља_понедељак_уторак_среда_четвртак_петак_субота".split("_"),weekdaysShort:"нед._пон._уто._сре._чет._пет._суб.".split("_"),weekdaysMin:"не_по_ут_ср_че_пе_су".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[данас у] LT",nextDay:"[сутра у] LT",nextWeek:function(){switch(this.day()){case 0:return"[у] [недељу] [у] LT";case 3:return"[у] [среду] [у] LT";case 6:return"[у] [суботу] [у] LT";case 1:case 2:case 4:case 5:return"[у] dddd [у] LT"}},lastDay:"[јуче у] LT",lastWeek:function(){var a=["[прошле] [недеље] [у] LT","[прошлог] [понедељка] [у] LT","[прошлог] [уторка] [у] LT","[прошле] [среде] [у] LT","[прошлог] [четвртка] [у] LT","[прошлог] [петка] [у] LT","[прошле] [суботе] [у] LT"];return a[this.day()]},sameElse:"L"},relativeTime:{future:"за %s",past:"пре %s",s:"неколико секунди",m:th.translate,mm:th.translate,h:th.translate,hh:th.translate,d:"дан",dd:th.translate,M:"месец",MM:th.translate,y:"годину",yy:th.translate},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),{words:{//Different grammatical cases
m:["jedan minut","jedne minute"],mm:["minut","minute","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mesec","meseca","meseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(a,b){return 1===a?b[0]:a>=2&&4>=a?b[1]:b[2]},translate:function(a,b,c){var d=uh.words[c];return 1===c.length?b?d[0]:d[1]:a+" "+uh.correctGrammaticalCase(a,d)}}),vh=(kg.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var a=["[prošle] [nedelje] [u] LT","[prošlog] [ponedeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return a[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",m:uh.translate,mm:uh.translate,h:uh.translate,hh:uh.translate,d:"dan",dd:uh.translate,M:"mesec",MM:uh.translate,y:"godinu",yy:uh.translate},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(a,b,c){return 11>a?"ekuseni":15>a?"emini":19>a?"entsambama":"ebusuku"},meridiemHour:function(a,b){return 12===a&&(a=0),"ekuseni"===b?a:"emini"===b?a>=11?a:a+12:"entsambama"===b||"ebusuku"===b?0===a?0:a+12:void 0},ordinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag".split("_"),weekdaysShort:"sön_mån_tis_ons_tor_fre_lör".split("_"),weekdaysMin:"sö_må_ti_on_to_fr_lö".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Igår] LT",nextWeek:"[På] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"för %s sedan",s:"några sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en månad",MM:"%d månader",y:"ett år",yy:"%d år"},ordinalParse:/\d{1,2}(e|a)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"e":1===b?"a":2===b?"a":"e";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"masiku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),{1:"௧",2:"௨",3:"௩",4:"௪",5:"௫",6:"௬",7:"௭",8:"௮",9:"௯",0:"௦"}),wh={"௧":"1","௨":"2","௩":"3","௪":"4","௫":"5","௬":"6","௭":"7","௮":"8","௯":"9","௦":"0"},xh=(kg.defineLocale("ta",{months:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),monthsShort:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),weekdays:"ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை".split("_"),weekdaysShort:"ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி".split("_"),weekdaysMin:"ஞா_தி_செ_பு_வி_வெ_ச".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[இன்று] LT",nextDay:"[நாளை] LT",nextWeek:"dddd, LT",lastDay:"[நேற்று] LT",lastWeek:"[கடந்த வாரம்] dddd, LT",sameElse:"L"},relativeTime:{future:"%s இல்",past:"%s முன்",s:"ஒரு சில விநாடிகள்",m:"ஒரு நிமிடம்",mm:"%d நிமிடங்கள்",h:"ஒரு மணி நேரம்",hh:"%d மணி நேரம்",d:"ஒரு நாள்",dd:"%d நாட்கள்",M:"ஒரு மாதம்",MM:"%d மாதங்கள்",y:"ஒரு வருடம்",yy:"%d ஆண்டுகள்"},ordinalParse:/\d{1,2}வது/,ordinal:function(a){return a+"வது"},preparse:function(a){return a.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,function(a){return wh[a]})},postformat:function(a){return a.replace(/\d/g,function(a){return vh[a]})},
// refer http://ta.wikipedia.org/s/1er1
meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(a,b,c){return 2>a?" யாமம்":6>a?" வைகறை":10>a?" காலை":14>a?" நண்பகல்":18>a?" எற்பாடு":22>a?" மாலை":" யாமம்"},meridiemHour:function(a,b){return 12===a&&(a=0),"யாமம்"===b?2>a?a:a+12:"வைகறை"===b||"காலை"===b?a:"நண்பகல்"===b&&a>=10?a:a+12},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),kg.defineLocale("te",{months:"జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జూలై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్".split("_"),monthsShort:"జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జూలై_ఆగ._సెప్._అక్టో._నవ._డిసె.".split("_"),monthsParseExact:!0,weekdays:"ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం".split("_"),weekdaysShort:"ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని".split("_"),weekdaysMin:"ఆ_సో_మం_బు_గు_శు_శ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[నేడు] LT",nextDay:"[రేపు] LT",nextWeek:"dddd, LT",lastDay:"[నిన్న] LT",lastWeek:"[గత] dddd, LT",sameElse:"L"},relativeTime:{future:"%s లో",past:"%s క్రితం",s:"కొన్ని క్షణాలు",m:"ఒక నిమిషం",mm:"%d నిమిషాలు",h:"ఒక గంట",hh:"%d గంటలు",d:"ఒక రోజు",dd:"%d రోజులు",M:"ఒక నెల",MM:"%d నెలలు",y:"ఒక సంవత్సరం",yy:"%d సంవత్సరాలు"},ordinalParse:/\d{1,2}వ/,ordinal:"%dవ",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(a,b){return 12===a&&(a=0),"రాత్రి"===b?4>a?a:a+12:"ఉదయం"===b?a:"మధ్యాహ్నం"===b?a>=10?a:a+12:"సాయంత్రం"===b?a+12:void 0},meridiem:function(a,b,c){return 4>a?"రాత్రి":10>a?"ఉదయం":17>a?"మధ్యాహ్నం":20>a?"సాయంత్రం":"రాత్రి"},week:{dow:0,// Sunday is the first day of the week.
doy:6}}),kg.defineLocale("th",{months:"มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม".split("_"),monthsShort:"ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.".split("_"),monthsParseExact:!0,weekdays:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์".split("_"),weekdaysShort:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์".split("_"),// yes, three characters difference
weekdaysMin:"อา._จ._อ._พ._พฤ._ศ._ส.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY/MM/DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY เวลา H:mm",LLLL:"วันddddที่ D MMMM YYYY เวลา H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(a){return"หลังเที่ยง"===a},meridiem:function(a,b,c){return 12>a?"ก่อนเที่ยง":"หลังเที่ยง"},calendar:{sameDay:"[วันนี้ เวลา] LT",nextDay:"[พรุ่งนี้ เวลา] LT",nextWeek:"dddd[หน้า เวลา] LT",lastDay:"[เมื่อวานนี้ เวลา] LT",lastWeek:"[วัน]dddd[ที่แล้ว เวลา] LT",sameElse:"L"},relativeTime:{future:"อีก %s",past:"%sที่แล้ว",s:"ไม่กี่วินาที",m:"1 นาที",mm:"%d นาที",h:"1 ชั่วโมง",hh:"%d ชั่วโมง",d:"1 วัน",dd:"%d วัน",M:"1 เดือน",MM:"%d เดือน",y:"1 ปี",yy:"%d ปี"}}),kg.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"[Ngayon sa] LT",nextDay:"[Bukas sa] LT",nextWeek:"dddd [sa] LT",lastDay:"[Kahapon sa] LT",lastWeek:"dddd [huling linggo] LT",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},ordinalParse:/\d{1,2}/,ordinal:function(a){return a},week:{dow:1,// Monday is the first day of the week.
doy:4}}),"pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_")),yh=(kg.defineLocale("tlh",{months:"tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’".split("_"),monthsShort:"jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa’leS] LT",nextWeek:"LLL",lastDay:"[wa’Hu’] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:ae,past:be,s:"puS lup",m:"wa’ tup",mm:ce,h:"wa’ rep",hh:ce,d:"wa’ jaj",dd:ce,M:"wa’ jar",MM:ce,y:"wa’ DIS",yy:ce},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),{1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'üncü",4:"'üncü",100:"'üncü",6:"'ncı",9:"'uncu",10:"'uncu",30:"'uncu",60:"'ıncı",90:"'ıncı"}),zh=(kg.defineLocale("tr",{months:"Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık".split("_"),monthsShort:"Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pts_Sal_Çar_Per_Cum_Cts".split("_"),weekdaysMin:"Pz_Pt_Sa_Ça_Pe_Cu_Ct".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[yarın saat] LT",nextWeek:"[haftaya] dddd [saat] LT",lastDay:"[dün] LT",lastWeek:"[geçen hafta] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s önce",s:"birkaç saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir yıl",yy:"%d yıl"},ordinalParse:/\d{1,2}'(inci|nci|üncü|ncı|uncu|ıncı)/,ordinal:function(a){if(0===a)// special case for zero
return a+"'ıncı";var b=a%10,c=a%100-b,d=a>=100?100:null;return a+(yh[b]||yh[c]||yh[d])},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("tzl",{months:"Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi".split("_"),weekdaysShort:"Súl_Lún_Mai_Már_Xhú_Vié_Sát".split("_"),weekdaysMin:"Sú_Lú_Ma_Má_Xh_Vi_Sá".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(a){return"d'o"===a.toLowerCase()},meridiem:function(a,b,c){return a>11?c?"d'o":"D'O":c?"d'a":"D'A"},calendar:{sameDay:"[oxhi à] LT",nextDay:"[demà à] LT",nextWeek:"dddd [à] LT",lastDay:"[ieiri à] LT",lastWeek:"[sür el] dddd [lasteu à] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:ee,m:ee,mm:ee,h:ee,hh:ee,d:ee,dd:ee,M:ee,MM:ee,y:ee,yy:ee},ordinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("tzm-latn",{months:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",m:"minuḍ",mm:"%d minuḍ",h:"saɛa",hh:"%d tassaɛin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),kg.defineLocale("tzm",{months:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),monthsShort:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),weekdays:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysShort:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysMin:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ⴰⵙⴷⵅ ⴴ] LT",nextDay:"[ⴰⵙⴽⴰ ⴴ] LT",nextWeek:"dddd [ⴴ] LT",lastDay:"[ⴰⵚⴰⵏⵜ ⴴ] LT",lastWeek:"dddd [ⴴ] LT",sameElse:"L"},relativeTime:{future:"ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s",past:"ⵢⴰⵏ %s",s:"ⵉⵎⵉⴽ",m:"ⵎⵉⵏⵓⴺ",mm:"%d ⵎⵉⵏⵓⴺ",h:"ⵙⴰⵄⴰ",hh:"%d ⵜⴰⵙⵙⴰⵄⵉⵏ",d:"ⴰⵙⵙ",dd:"%d oⵙⵙⴰⵏ",M:"ⴰⵢoⵓⵔ",MM:"%d ⵉⵢⵢⵉⵔⵏ",y:"ⴰⵙⴳⴰⵙ",yy:"%d ⵉⵙⴳⴰⵙⵏ"},week:{dow:6,// Saturday is the first day of the week.
doy:12}}),kg.defineLocale("uk",{months:{format:"січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_"),standalone:"січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_")},monthsShort:"січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд".split("_"),weekdays:he,weekdaysShort:"нд_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY р.",LLL:"D MMMM YYYY р., HH:mm",LLLL:"dddd, D MMMM YYYY р., HH:mm"},calendar:{sameDay:ie("[Сьогодні "),nextDay:ie("[Завтра "),lastDay:ie("[Вчора "),nextWeek:ie("[У] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return ie("[Минулої] dddd [").call(this);case 1:case 2:case 4:return ie("[Минулого] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"за %s",past:"%s тому",s:"декілька секунд",m:ge,mm:ge,h:"годину",hh:ge,d:"день",dd:ge,M:"місяць",MM:ge,y:"рік",yy:ge},
// M. E.: those two are virtually unused but a user might want to implement them for his/her website for some reason
meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(a){return/^(дня|вечора)$/.test(a)},meridiem:function(a,b,c){return 4>a?"ночі":12>a?"ранку":17>a?"дня":"вечора"},ordinalParse:/\d{1,2}-(й|го)/,ordinal:function(a,b){switch(b){case"M":case"d":case"DDD":case"w":case"W":return a+"-й";case"D":return a+"-го";default:return a}},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("uz",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба".split("_"),weekdaysShort:"Якш_Душ_Сеш_Чор_Пай_Жум_Шан".split("_"),weekdaysMin:"Як_Ду_Се_Чо_Па_Жу_Ша".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Бугун соат] LT [да]",nextDay:"[Эртага] LT [да]",nextWeek:"dddd [куни соат] LT [да]",lastDay:"[Кеча соат] LT [да]",lastWeek:"[Утган] dddd [куни соат] LT [да]",sameElse:"L"},relativeTime:{future:"Якин %s ичида",past:"Бир неча %s олдин",s:"фурсат",m:"бир дакика",mm:"%d дакика",h:"бир соат",hh:"%d соат",d:"бир кун",dd:"%d кун",M:"бир ой",MM:"%d ой",y:"бир йил",yy:"%d йил"},week:{dow:1,// Monday is the first day of the week.
doy:7}}),kg.defineLocale("vi",{months:"tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12".split("_"),monthsShort:"Th01_Th02_Th03_Th04_Th05_Th06_Th07_Th08_Th09_Th10_Th11_Th12".split("_"),monthsParseExact:!0,weekdays:"chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(a){return/^ch$/i.test(a)},meridiem:function(a,b,c){return 12>a?c?"sa":"SA":c?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [năm] YYYY",LLL:"D MMMM [năm] YYYY HH:mm",LLLL:"dddd, D MMMM [năm] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[Hôm nay lúc] LT",nextDay:"[Ngày mai lúc] LT",nextWeek:"dddd [tuần tới lúc] LT",lastDay:"[Hôm qua lúc] LT",lastWeek:"dddd [tuần rồi lúc] LT",sameElse:"L"},relativeTime:{future:"%s tới",past:"%s trước",s:"vài giây",m:"một phút",mm:"%d phút",h:"một giờ",hh:"%d giờ",d:"một ngày",dd:"%d ngày",M:"một tháng",MM:"%d tháng",y:"một năm",yy:"%d năm"},ordinalParse:/\d{1,2}/,ordinal:function(a){return a},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("x-pseudo",{months:"J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér".split("_"),monthsShort:"J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc".split("_"),monthsParseExact:!0,weekdays:"S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý".split("_"),weekdaysShort:"S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát".split("_"),weekdaysMin:"S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~ódá~ý át] LT",nextDay:"[T~ómó~rró~w át] LT",nextWeek:"dddd [át] LT",lastDay:"[Ý~ést~érdá~ý át] LT",lastWeek:"[L~ást] dddd [át] LT",sameElse:"L"},relativeTime:{future:"í~ñ %s",past:"%s á~gó",s:"á ~féw ~sécó~ñds",m:"á ~míñ~úté",mm:"%d m~íñú~tés",h:"á~ñ hó~úr",hh:"%d h~óúrs",d:"á ~dáý",dd:"%d d~áýs",M:"á ~móñ~th",MM:"%d m~óñt~hs",y:"á ~ýéár",yy:"%d ý~éárs"},ordinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(a){var b=a%10,c=1===~~(a%100/10)?"th":1===b?"st":2===b?"nd":3===b?"rd":"th";return a+c},week:{dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("zh-cn",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"周日_周一_周二_周三_周四_周五_周六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"Ah点mm分",LTS:"Ah点m分s秒",L:"YYYY-MM-DD",LL:"YYYY年MMMD日",LLL:"YYYY年MMMD日Ah点mm分",LLLL:"YYYY年MMMD日ddddAh点mm分",l:"YYYY-MM-DD",ll:"YYYY年MMMD日",lll:"YYYY年MMMD日Ah点mm分",llll:"YYYY年MMMD日ddddAh点mm分"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(a,b){return 12===a&&(a=0),"凌晨"===b||"早上"===b||"上午"===b?a:"下午"===b||"晚上"===b?a+12:a>=11?a:a+12},meridiem:function(a,b,c){var d=100*a+b;return 600>d?"凌晨":900>d?"早上":1130>d?"上午":1230>d?"中午":1800>d?"下午":"晚上"},calendar:{sameDay:function(){return 0===this.minutes()?"[今天]Ah[点整]":"[今天]LT"},nextDay:function(){return 0===this.minutes()?"[明天]Ah[点整]":"[明天]LT"},lastDay:function(){return 0===this.minutes()?"[昨天]Ah[点整]":"[昨天]LT"},nextWeek:function(){var a,b;return a=kg().startOf("week"),b=this.diff(a,"days")>=7?"[下]":"[本]",0===this.minutes()?b+"dddAh点整":b+"dddAh点mm"},lastWeek:function(){var a,b;return a=kg().startOf("week"),b=this.unix()<a.unix()?"[上]":"[本]",0===this.minutes()?b+"dddAh点整":b+"dddAh点mm"},sameElse:"LL"},ordinalParse:/\d{1,2}(日|月|周)/,ordinal:function(a,b){switch(b){case"d":case"D":case"DDD":return a+"日";case"M":return a+"月";case"w":case"W":return a+"周";default:return a}},relativeTime:{future:"%s内",past:"%s前",s:"几秒",m:"1 分钟",mm:"%d 分钟",h:"1 小时",hh:"%d 小时",d:"1 天",dd:"%d 天",M:"1 个月",MM:"%d 个月",y:"1 年",yy:"%d 年"},week:{
// GB/T 7408-1994《数据元和交换格式·信息交换·日期和时间表示法》与ISO 8601:1988等效
dow:1,// Monday is the first day of the week.
doy:4}}),kg.defineLocale("zh-hk",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"Ah點mm分",LTS:"Ah點m分s秒",L:"YYYY年MMMD日",LL:"YYYY年MMMD日",LLL:"YYYY年MMMD日Ah點mm分",LLLL:"YYYY年MMMD日ddddAh點mm分",l:"YYYY年MMMD日",ll:"YYYY年MMMD日",lll:"YYYY年MMMD日Ah點mm分",llll:"YYYY年MMMD日ddddAh點mm分"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(a,b){return 12===a&&(a=0),"凌晨"===b||"早上"===b||"上午"===b?a:"中午"===b?a>=11?a:a+12:"下午"===b||"晚上"===b?a+12:void 0},meridiem:function(a,b,c){var d=100*a+b;return 600>d?"凌晨":900>d?"早上":1130>d?"上午":1230>d?"中午":1800>d?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},ordinalParse:/\d{1,2}(日|月|週)/,ordinal:function(a,b){switch(b){case"d":case"D":case"DDD":return a+"日";case"M":return a+"月";case"w":case"W":return a+"週";default:return a}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}}),kg.defineLocale("zh-tw",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"Ah點mm分",LTS:"Ah點m分s秒",L:"YYYY年MMMD日",LL:"YYYY年MMMD日",LLL:"YYYY年MMMD日Ah點mm分",LLLL:"YYYY年MMMD日ddddAh點mm分",l:"YYYY年MMMD日",ll:"YYYY年MMMD日",lll:"YYYY年MMMD日Ah點mm分",llll:"YYYY年MMMD日ddddAh點mm分"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(a,b){return 12===a&&(a=0),"凌晨"===b||"早上"===b||"上午"===b?a:"中午"===b?a>=11?a:a+12:"下午"===b||"晚上"===b?a+12:void 0},meridiem:function(a,b,c){var d=100*a+b;return 600>d?"凌晨":900>d?"早上":1130>d?"上午":1230>d?"中午":1800>d?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},ordinalParse:/\d{1,2}(日|月|週)/,ordinal:function(a,b){switch(b){case"d":case"D":case"DDD":return a+"日";case"M":return a+"月";case"w":case"W":return a+"週";default:return a}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}}),kg);return zh.locale("en"),zh});
/**
 * An Angular module that gives you access to the browsers local storage
 * @version v0.2.2 - 2015-05-29
 * @link https://github.com/grevory/angular-local-storage
 * @author grevory <greg@gregpike.ca>
 * @license MIT License, http://www.opensource.org/licenses/MIT
 */!function(a,b,c){"use strict";var d=b.isDefined,e=b.isUndefined,f=b.isNumber,g=b.isObject,h=b.isArray,i=b.extend,j=b.toJson,k=b.module("LocalStorageModule",[]);k.provider("localStorageService",function(){this.prefix="ls",this.storageType="localStorage",this.cookie={expiry:30,path:"/"},this.notify={setItem:!0,removeItem:!1},this.setPrefix=function(a){return this.prefix=a,this},this.setStorageType=function(a){return this.storageType=a,this},this.setStorageCookie=function(a,b){return this.cookie.expiry=a,this.cookie.path=b,this},this.setStorageCookieDomain=function(a){return this.cookie.domain=a,this},this.setNotify=function(a,b){return this.notify={setItem:a,removeItem:b},this},this.$get=["$rootScope","$window","$document","$parse",function(a,b,c,k){var l,m=this,n=m.prefix,o=m.cookie,p=m.notify,q=m.storageType;c?c[0]&&(c=c[0]):c=document,"."!==n.substr(-1)&&(n=n?n+".":"");var r=function(a){return n+a},s=function(){try{var c=q in b&&null!==b[q],d=r("__"+Math.round(1e7*Math.random()));return c&&(l=b[q],l.setItem(d,""),l.removeItem(d)),c}catch(e){return q="cookie",a.$broadcast("LocalStorageModule.notification.error",e.message),!1}}(),t=function(b,c){if(c=e(c)?null:j(c),!s||"cookie"===m.storageType)return s||a.$broadcast("LocalStorageModule.notification.warning","LOCAL_STORAGE_NOT_SUPPORTED"),p.setItem&&a.$broadcast("LocalStorageModule.notification.setitem",{key:b,newvalue:c,storageType:"cookie"}),z(b,c);try{l&&l.setItem(r(b),c),p.setItem&&a.$broadcast("LocalStorageModule.notification.setitem",{key:b,newvalue:c,storageType:m.storageType})}catch(d){return a.$broadcast("LocalStorageModule.notification.error",d.message),z(b,c)}return!0},u=function(b){if(!s||"cookie"===m.storageType)return s||a.$broadcast("LocalStorageModule.notification.warning","LOCAL_STORAGE_NOT_SUPPORTED"),A(b);var c=l?l.getItem(r(b)):null;if(!c||"null"===c)return null;try{return JSON.parse(c)}catch(d){return c}},v=function(){var b,c;for(b=0;b<arguments.length;b++)if(c=arguments[b],s&&"cookie"!==m.storageType)try{l.removeItem(r(c)),p.removeItem&&a.$broadcast("LocalStorageModule.notification.removeitem",{key:c,storageType:m.storageType})}catch(d){a.$broadcast("LocalStorageModule.notification.error",d.message),B(c)}else s||a.$broadcast("LocalStorageModule.notification.warning","LOCAL_STORAGE_NOT_SUPPORTED"),p.removeItem&&a.$broadcast("LocalStorageModule.notification.removeitem",{key:c,storageType:"cookie"}),B(c)},w=function(){if(!s)return a.$broadcast("LocalStorageModule.notification.warning","LOCAL_STORAGE_NOT_SUPPORTED"),!1;var b=n.length,c=[];for(var d in l)if(d.substr(0,b)===n)try{c.push(d.substr(b))}catch(e){return a.$broadcast("LocalStorageModule.notification.error",e.Description),[]}return c},x=function(b){var c=n?new RegExp("^"+n):new RegExp,d=b?new RegExp(b):new RegExp;if(!s||"cookie"===m.storageType)return s||a.$broadcast("LocalStorageModule.notification.warning","LOCAL_STORAGE_NOT_SUPPORTED"),C();var e=n.length;for(var f in l)if(c.test(f)&&d.test(f.substr(e)))try{v(f.substr(e))}catch(g){return a.$broadcast("LocalStorageModule.notification.error",g.message),C()}return!0},y=function(){try{return b.navigator.cookieEnabled||"cookie"in c&&(c.cookie.length>0||(c.cookie="test").indexOf.call(c.cookie,"test")>-1)}catch(d){return a.$broadcast("LocalStorageModule.notification.error",d.message),!1}}(),z=function(b,d,i){if(e(d))return!1;if((h(d)||g(d))&&(d=j(d)),!y)return a.$broadcast("LocalStorageModule.notification.error","COOKIES_NOT_SUPPORTED"),!1;try{var k="",l=new Date,m="";if(null===d?(l.setTime(l.getTime()+-864e5),k="; expires="+l.toGMTString(),d=""):f(i)&&0!==i?(l.setTime(l.getTime()+24*i*60*60*1e3),k="; expires="+l.toGMTString()):0!==o.expiry&&(l.setTime(l.getTime()+24*o.expiry*60*60*1e3),k="; expires="+l.toGMTString()),b){var n="; path="+o.path;o.domain&&(m="; domain="+o.domain),c.cookie=r(b)+"="+encodeURIComponent(d)+k+n+m}}catch(p){return a.$broadcast("LocalStorageModule.notification.error",p.message),!1}return!0},A=function(b){if(!y)return a.$broadcast("LocalStorageModule.notification.error","COOKIES_NOT_SUPPORTED"),!1;for(var d=c.cookie&&c.cookie.split(";")||[],e=0;e<d.length;e++){for(var f=d[e];" "===f.charAt(0);)f=f.substring(1,f.length);if(0===f.indexOf(r(b)+"=")){var g=decodeURIComponent(f.substring(n.length+b.length+1,f.length));try{return JSON.parse(g)}catch(h){return g}}}return null},B=function(a){z(a,null)},C=function(){for(var a=null,b=n.length,d=c.cookie.split(";"),e=0;e<d.length;e++){for(a=d[e];" "===a.charAt(0);)a=a.substring(1,a.length);var f=a.substring(b,a.indexOf("="));B(f)}},D=function(){return q},E=function(a,b,c,e){e=e||b;var f=u(e);return null===f&&d(c)?f=c:g(f)&&g(c)&&(f=i(c,f)),k(b).assign(a,f),a.$watch(b,function(a){t(e,a)},g(a[b]))},F=function(){for(var a=0,c=b[q],d=0;d<c.length;d++)0===c.key(d).indexOf(n)&&a++;return a};return{isSupported:s,getStorageType:D,set:t,add:t,get:u,keys:w,remove:v,clearAll:x,bind:E,deriveKey:r,length:F,cookie:{isSupported:y,set:z,add:z,get:A,remove:B,clearAll:C}}}]})}(window,window.angular);
/**
 * angular-ui-utils - Swiss-Army-Knife of AngularJS tools (with no external dependencies!)
 * @version v0.1.1 - 2014-02-05
 * @link http://angular-ui.github.com
 * @license MIT License, http://www.opensource.org/licenses/MIT
 */
"use strict";angular.module("ui.alias",[]).config(["$compileProvider","uiAliasConfig",function(a,b){b=b||{},angular.forEach(b,function(b,c){angular.isString(b)&&(b={replace:!0,template:b}),a.directive(c,function(){return b})})}]),angular.module("ui.event",[]).directive("uiEvent",["$parse",function(a){return function(b,c,d){var e=b.$eval(d.uiEvent);angular.forEach(e,function(d,e){var f=a(d);c.bind(e,function(a){var c=Array.prototype.slice.call(arguments);c=c.splice(1),f(b,{$event:a,$params:c}),b.$$phase||b.$apply()})})}}]),angular.module("ui.format",[]).filter("format",function(){return function(a,b){var c=a;if(angular.isString(c)&&void 0!==b)if(angular.isArray(b)||angular.isObject(b)||(b=[b]),angular.isArray(b)){var d=b.length,e=function(a,c){return c=parseInt(c,10),c>=0&&d>c?b[c]:a};c=c.replace(/\$([0-9]+)/g,e)}else angular.forEach(b,function(a,b){c=c.split(":"+b).join(a)});return c}}),angular.module("ui.highlight",[]).filter("highlight",function(){return function(a,b,c){return b||angular.isNumber(b)?(a=a.toString(),b=b.toString(),c?a.split(b).join('<span class="ui-match">'+b+"</span>"):a.replace(new RegExp(b,"gi"),'<span class="ui-match">$&</span>')):a}}),angular.module("ui.include",[]).directive("uiInclude",["$http","$templateCache","$anchorScroll","$compile",function(a,b,c,d){return{restrict:"ECA",terminal:!0,compile:function(e,f){var g=f.uiInclude||f.src,h=f.fragment||"",i=f.onload||"",j=f.autoscroll;return function(e,f){function k(){var k=++m,o=e.$eval(g),p=e.$eval(h);o?a.get(o,{cache:b}).success(function(a){if(k===m){l&&l.$destroy(),l=e.$new();var b;b=p?angular.element("<div/>").html(a).find(p):angular.element("<div/>").html(a).contents(),f.html(b),d(b)(l),!angular.isDefined(j)||j&&!e.$eval(j)||c(),l.$emit("$includeContentLoaded"),e.$eval(i)}}).error(function(){k===m&&n()}):n()}var l,m=0,n=function(){l&&(l.$destroy(),l=null),f.html("")};e.$watch(h,k),e.$watch(g,k)}}}}]),angular.module("ui.indeterminate",[]).directive("uiIndeterminate",[function(){return{compile:function(a,b){return b.type&&"checkbox"===b.type.toLowerCase()?function(a,b,c){a.$watch(c.uiIndeterminate,function(a){b[0].indeterminate=!!a})}:angular.noop}}}]),angular.module("ui.inflector",[]).filter("inflector",function(){function a(a){return a.replace(/^([a-z])|\s+([a-z])/g,function(a){return a.toUpperCase()})}function b(a,b){return a.replace(/[A-Z]/g,function(a){return b+a})}var c={humanize:function(c){return a(b(c," ").split("_").join(" "))},underscore:function(a){return a.substr(0,1).toLowerCase()+b(a.substr(1),"_").toLowerCase().split(" ").join("_")},variable:function(b){return b=b.substr(0,1).toLowerCase()+a(b.split("_").join(" ")).substr(1).split(" ").join("")}};return function(a,b){return b!==!1&&angular.isString(a)?(b=b||"humanize",c[b](a)):a}}),angular.module("ui.jq",[]).value("uiJqConfig",{}).directive("uiJq",["uiJqConfig","$timeout",function(a,b){return{restrict:"A",compile:function(c,d){if(!angular.isFunction(c[d.uiJq]))throw new Error('ui-jq: The "'+d.uiJq+'" function does not exist');var e=a&&a[d.uiJq];return function(a,c,d){function f(){b(function(){c[d.uiJq].apply(c,g)},0,!1)}var g=[];d.uiOptions?(g=a.$eval("["+d.uiOptions+"]"),angular.isObject(e)&&angular.isObject(g[0])&&(g[0]=angular.extend({},e,g[0]))):e&&(g=[e]),d.ngModel&&c.is("select,input,textarea")&&c.bind("change",function(){c.trigger("input")}),d.uiRefresh&&a.$watch(d.uiRefresh,function(){f()}),f()}}}}]),angular.module("ui.keypress",[]).factory("keypressHelper",["$parse",function(a){var b={8:"backspace",9:"tab",13:"enter",27:"esc",32:"space",33:"pageup",34:"pagedown",35:"end",36:"home",37:"left",38:"up",39:"right",40:"down",45:"insert",46:"delete"},c=function(a){return a.charAt(0).toUpperCase()+a.slice(1)};return function(d,e,f,g){var h,i=[];h=e.$eval(g["ui"+c(d)]),angular.forEach(h,function(b,c){var d,e;e=a(b),angular.forEach(c.split(" "),function(a){d={expression:e,keys:{}},angular.forEach(a.split("-"),function(a){d.keys[a]=!0}),i.push(d)})}),f.bind(d,function(a){var c=!(!a.metaKey||a.ctrlKey),f=!!a.altKey,g=!!a.ctrlKey,h=!!a.shiftKey,j=a.keyCode;"keypress"===d&&!h&&j>=97&&122>=j&&(j-=32),angular.forEach(i,function(d){var i=d.keys[b[j]]||d.keys[j.toString()],k=!!d.keys.meta,l=!!d.keys.alt,m=!!d.keys.ctrl,n=!!d.keys.shift;i&&k===c&&l===f&&m===g&&n===h&&e.$apply(function(){d.expression(e,{$event:a})})})})}}]),angular.module("ui.keypress").directive("uiKeydown",["keypressHelper",function(a){return{link:function(b,c,d){a("keydown",b,c,d)}}}]),angular.module("ui.keypress").directive("uiKeypress",["keypressHelper",function(a){return{link:function(b,c,d){a("keypress",b,c,d)}}}]),angular.module("ui.keypress").directive("uiKeyup",["keypressHelper",function(a){return{link:function(b,c,d){a("keyup",b,c,d)}}}]),angular.module("ui.mask",[]).value("uiMaskConfig",{maskDefinitions:{9:/\d/,A:/[a-zA-Z]/,"*":/[a-zA-Z0-9]/}}).directive("uiMask",["uiMaskConfig",function(a){return{priority:100,require:"ngModel",restrict:"A",compile:function(){var b=a;return function(a,c,d,e){function f(a){return angular.isDefined(a)?(s(a),N?(k(),l(),!0):j()):j()}function g(a){angular.isDefined(a)&&(D=a,N&&w())}function h(a){return N?(G=o(a||""),I=n(G),e.$setValidity("mask",I),I&&G.length?p(G):void 0):a}function i(a){return N?(G=o(a||""),I=n(G),e.$viewValue=G.length?p(G):"",e.$setValidity("mask",I),""===G&&void 0!==e.$error.required&&e.$setValidity("required",!1),I?G:void 0):a}function j(){return N=!1,m(),angular.isDefined(P)?c.attr("placeholder",P):c.removeAttr("placeholder"),angular.isDefined(Q)?c.attr("maxlength",Q):c.removeAttr("maxlength"),c.val(e.$modelValue),e.$viewValue=e.$modelValue,!1}function k(){G=K=o(e.$modelValue||""),H=J=p(G),I=n(G);var a=I&&G.length?H:"";d.maxlength&&c.attr("maxlength",2*B[B.length-1]),c.attr("placeholder",D),c.val(a),e.$viewValue=a}function l(){O||(c.bind("blur",t),c.bind("mousedown mouseup",u),c.bind("input keyup click focus",w),O=!0)}function m(){O&&(c.unbind("blur",t),c.unbind("mousedown",u),c.unbind("mouseup",u),c.unbind("input",w),c.unbind("keyup",w),c.unbind("click",w),c.unbind("focus",w),O=!1)}function n(a){return a.length?a.length>=F:!0}function o(a){var b="",c=C.slice();return a=a.toString(),angular.forEach(E,function(b){a=a.replace(b,"")}),angular.forEach(a.split(""),function(a){c.length&&c[0].test(a)&&(b+=a,c.shift())}),b}function p(a){var b="",c=B.slice();return angular.forEach(D.split(""),function(d,e){a.length&&e===c[0]?(b+=a.charAt(0)||"_",a=a.substr(1),c.shift()):b+=d}),b}function q(a){var b=d.placeholder;return"undefined"!=typeof b&&b[a]?b[a]:"_"}function r(){return D.replace(/[_]+/g,"_").replace(/([^_]+)([a-zA-Z0-9])([^_])/g,"$1$2_$3").split("_")}function s(a){var b=0;if(B=[],C=[],D="","string"==typeof a){F=0;var c=!1,d=a.split("");angular.forEach(d,function(a,d){R.maskDefinitions[a]?(B.push(b),D+=q(d),C.push(R.maskDefinitions[a]),b++,c||F++):"?"===a?c=!0:(D+=a,b++)})}B.push(B.slice().pop()+1),E=r(),N=B.length>1?!0:!1}function t(){L=0,M=0,I&&0!==G.length||(H="",c.val(""),a.$apply(function(){e.$setViewValue("")}))}function u(a){"mousedown"===a.type?c.bind("mouseout",v):c.unbind("mouseout",v)}function v(){M=A(this),c.unbind("mouseout",v)}function w(b){b=b||{};var d=b.which,f=b.type;if(16!==d&&91!==d){var g,h=c.val(),i=J,j=o(h),k=K,l=!1,m=y(this)||0,n=L||0,q=m-n,r=B[0],s=B[j.length]||B.slice().shift(),t=M||0,u=A(this)>0,v=t>0,w=h.length>i.length||t&&h.length>i.length-t,C=h.length<i.length||t&&h.length===i.length-t,D=d>=37&&40>=d&&b.shiftKey,E=37===d,F=8===d||"keyup"!==f&&C&&-1===q,G=46===d||"keyup"!==f&&C&&0===q&&!v,H=(E||F||"click"===f)&&m>r;if(M=A(this),!D&&(!u||"click"!==f&&"keyup"!==f)){if("input"===f&&C&&!v&&j===k){for(;F&&m>r&&!x(m);)m--;for(;G&&s>m&&-1===B.indexOf(m);)m++;var I=B.indexOf(m);j=j.substring(0,I)+j.substring(I+1),l=!0}for(g=p(j),J=g,K=j,c.val(g),l&&a.$apply(function(){e.$setViewValue(j)}),w&&r>=m&&(m=r+1),H&&m--,m=m>s?s:r>m?r:m;!x(m)&&m>r&&s>m;)m+=H?-1:1;(H&&s>m||w&&!x(n))&&m++,L=m,z(this,m)}}}function x(a){return B.indexOf(a)>-1}function y(a){if(!a)return 0;if(void 0!==a.selectionStart)return a.selectionStart;if(document.selection){a.focus();var b=document.selection.createRange();return b.moveStart("character",-a.value.length),b.text.length}return 0}function z(a,b){if(!a)return 0;if(0!==a.offsetWidth&&0!==a.offsetHeight)if(a.setSelectionRange)a.focus(),a.setSelectionRange(b,b);else if(a.createTextRange){var c=a.createTextRange();c.collapse(!0),c.moveEnd("character",b),c.moveStart("character",b),c.select()}}function A(a){return a?void 0!==a.selectionStart?a.selectionEnd-a.selectionStart:document.selection?document.selection.createRange().text.length:0:0}var B,C,D,E,F,G,H,I,J,K,L,M,N=!1,O=!1,P=d.placeholder,Q=d.maxlength,R={};d.uiOptions?(R=a.$eval("["+d.uiOptions+"]"),angular.isObject(R[0])&&(R=function(a,b){for(var c in a)Object.prototype.hasOwnProperty.call(a,c)&&(b[c]?angular.extend(b[c],a[c]):b[c]=angular.copy(a[c]));return b}(b,R[0]))):R=b,d.$observe("uiMask",f),d.$observe("placeholder",g),e.$formatters.push(h),e.$parsers.push(i),c.bind("mousedown mouseup",u),Array.prototype.indexOf||(Array.prototype.indexOf=function(a){if(null===this)throw new TypeError;var b=Object(this),c=b.length>>>0;if(0===c)return-1;var d=0;if(arguments.length>1&&(d=Number(arguments[1]),d!==d?d=0:0!==d&&1/0!==d&&d!==-1/0&&(d=(d>0||-1)*Math.floor(Math.abs(d)))),d>=c)return-1;for(var e=d>=0?d:Math.max(c-Math.abs(d),0);c>e;e++)if(e in b&&b[e]===a)return e;return-1})}}}}]),angular.module("ui.reset",[]).value("uiResetConfig",null).directive("uiReset",["uiResetConfig",function(a){var b=null;return void 0!==a&&(b=a),{require:"ngModel",link:function(a,c,d,e){var f;f=angular.element('<a class="ui-reset" />'),c.wrap('<span class="ui-resetwrap" />').after(f),f.bind("click",function(c){c.preventDefault(),a.$apply(function(){e.$setViewValue(d.uiReset?a.$eval(d.uiReset):b),e.$render()})})}}}]),angular.module("ui.route",[]).directive("uiRoute",["$location","$parse",function(a,b){return{restrict:"AC",scope:!0,compile:function(c,d){var e;if(d.uiRoute)e="uiRoute";else if(d.ngHref)e="ngHref";else{if(!d.href)throw new Error("uiRoute missing a route or href property on "+c[0]);e="href"}return function(c,d,f){function g(b){var d=b.indexOf("#");d>-1&&(b=b.substr(d+1)),(j=function(){i(c,a.path().indexOf(b)>-1)})()}function h(b){var d=b.indexOf("#");d>-1&&(b=b.substr(d+1)),(j=function(){var d=new RegExp("^"+b+"$",["i"]);i(c,d.test(a.path()))})()}var i=b(f.ngModel||f.routeModel||"$uiRoute").assign,j=angular.noop;switch(e){case"uiRoute":f.uiRoute?h(f.uiRoute):f.$observe("uiRoute",h);break;case"ngHref":f.ngHref?g(f.ngHref):f.$observe("ngHref",g);break;case"href":g(f.href)}c.$on("$routeChangeSuccess",function(){j()}),c.$on("$stateChangeSuccess",function(){j()})}}}}]),angular.module("ui.scroll.jqlite",["ui.scroll"]).service("jqLiteExtras",["$log","$window",function(a,b){return{registerFor:function(a){var c,d,e,f,g,h,i;return d=angular.element.prototype.css,a.prototype.css=function(a,b){var c,e;return e=this,c=e[0],c&&3!==c.nodeType&&8!==c.nodeType&&c.style?d.call(e,a,b):void 0},h=function(a){return a&&a.document&&a.location&&a.alert&&a.setInterval},i=function(a,b,c){var d,e,f,g,i;return d=a[0],i={top:["scrollTop","pageYOffset","scrollLeft"],left:["scrollLeft","pageXOffset","scrollTop"]}[b],e=i[0],g=i[1],f=i[2],h(d)?angular.isDefined(c)?d.scrollTo(a[f].call(a),c):g in d?d[g]:d.document.documentElement[e]:angular.isDefined(c)?d[e]=c:d[e]},b.getComputedStyle?(f=function(a){return b.getComputedStyle(a,null)},c=function(a,b){return parseFloat(b)}):(f=function(a){return a.currentStyle},c=function(a,b){var c,d,e,f,g,h,i;return c=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,f=new RegExp("^("+c+")(?!px)[a-z%]+$","i"),f.test(b)?(i=a.style,d=i.left,g=a.runtimeStyle,h=g&&g.left,g&&(g.left=i.left),i.left=b,e=i.pixelLeft,i.left=d,h&&(g.left=h),e):parseFloat(b)}),e=function(a,b){var d,e,g,i,j,k,l,m,n,o,p,q,r;return h(a)?(d=document.documentElement[{height:"clientHeight",width:"clientWidth"}[b]],{base:d,padding:0,border:0,margin:0}):(r={width:[a.offsetWidth,"Left","Right"],height:[a.offsetHeight,"Top","Bottom"]}[b],d=r[0],l=r[1],m=r[2],k=f(a),p=c(a,k["padding"+l])||0,q=c(a,k["padding"+m])||0,e=c(a,k["border"+l+"Width"])||0,g=c(a,k["border"+m+"Width"])||0,i=k["margin"+l],j=k["margin"+m],n=c(a,i)||0,o=c(a,j)||0,{base:d,padding:p+q,border:e+g,margin:n+o})},g=function(a,b,c){var d,g,h;return g=e(a,b),g.base>0?{base:g.base-g.padding-g.border,outer:g.base,outerfull:g.base+g.margin}[c]:(d=f(a),h=d[b],(0>h||null===h)&&(h=a.style[b]||0),h=parseFloat(h)||0,{base:h-g.padding-g.border,outer:h,outerfull:h+g.padding+g.border+g.margin}[c])},angular.forEach({before:function(a){var b,c,d,e,f,g,h;if(f=this,c=f[0],e=f.parent(),b=e.contents(),b[0]===c)return e.prepend(a);for(d=g=1,h=b.length-1;h>=1?h>=g:g>=h;d=h>=1?++g:--g)if(b[d]===c)return void angular.element(b[d-1]).after(a);throw new Error("invalid DOM structure "+c.outerHTML)},height:function(a){var b;return b=this,angular.isDefined(a)?(angular.isNumber(a)&&(a+="px"),d.call(b,"height",a)):g(this[0],"height","base")},outerHeight:function(a){return g(this[0],"height",a?"outerfull":"outer")},offset:function(a){var b,c,d,e,f,g;return f=this,arguments.length?void 0===a?f:a:(b={top:0,left:0},e=f[0],(c=e&&e.ownerDocument)?(d=c.documentElement,e.getBoundingClientRect&&(b=e.getBoundingClientRect()),g=c.defaultView||c.parentWindow,{top:b.top+(g.pageYOffset||d.scrollTop)-(d.clientTop||0),left:b.left+(g.pageXOffset||d.scrollLeft)-(d.clientLeft||0)}):void 0)},scrollTop:function(a){return i(this,"top",a)},scrollLeft:function(a){return i(this,"left",a)}},function(b,c){return a.prototype[c]?void 0:a.prototype[c]=b})}}}]).run(["$log","$window","jqLiteExtras",function(a,b,c){return b.jQuery?void 0:c.registerFor(angular.element)}]),angular.module("ui.scroll",[]).directive("ngScrollViewport",["$log",function(){return{controller:["$scope","$element",function(a,b){return b}]}}]).directive("ngScroll",["$log","$injector","$rootScope","$timeout",function(a,b,c,d){return{require:["?^ngScrollViewport"],transclude:"element",priority:1e3,terminal:!0,compile:function(e,f,g){return function(f,h,i,j){var k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T;if(H=i.ngScroll.match(/^\s*(\w+)\s+in\s+(\w+)\s*$/),!H)throw new Error('Expected ngScroll in form of "item_ in _datasource_" but got "'+i.ngScroll+'"');if(F=H[1],v=H[2],D=function(a){return angular.isObject(a)&&a.get&&angular.isFunction(a.get)},u=f[v],!D(u)&&(u=b.get(v),!D(u)))throw new Error(v+" is not a valid datasource");return r=Math.max(3,+i.bufferSize||10),q=function(){return T.height()*Math.max(.1,+i.padding||.1)},O=function(a){return a[0].scrollHeight||a[0].document.documentElement.scrollHeight},k=null,g(R=f.$new(),function(a){var b,c,d,f,g,h;if(f=a[0].localName,"dl"===f)throw new Error("ng-scroll directive does not support <"+a[0].localName+"> as a repeating tag: "+a[0].outerHTML);return"li"!==f&&"tr"!==f&&(f="div"),h=j[0]||angular.element(window),h.css({"overflow-y":"auto",display:"block"}),d=function(a){var b,c,d;switch(a){case"tr":return d=angular.element("<table><tr><td><div></div></td></tr></table>"),b=d.find("div"),c=d.find("tr"),c.paddingHeight=function(){return b.height.apply(b,arguments)},c;default:return c=angular.element("<"+a+"></"+a+">"),c.paddingHeight=c.height,c}},c=function(a,b,c){return b[{top:"before",bottom:"after"}[c]](a),{paddingHeight:function(){return a.paddingHeight.apply(a,arguments)},insert:function(b){return a[{top:"after",bottom:"before"}[c]](b)}}},g=c(d(f),e,"top"),b=c(d(f),e,"bottom"),R.$destroy(),k={viewport:h,topPadding:g.paddingHeight,bottomPadding:b.paddingHeight,append:b.insert,prepend:g.insert,bottomDataPos:function(){return O(h)-b.paddingHeight()},topDataPos:function(){return g.paddingHeight()}}}),T=k.viewport,B=1,I=1,p=[],J=[],x=!1,n=!1,G=u.loading||function(){},E=!1,L=function(a,b){var c,d;for(c=d=a;b>=a?b>d:d>b;c=b>=a?++d:--d)p[c].scope.$destroy(),p[c].element.remove();return p.splice(a,b-a)},K=function(){return B=1,I=1,L(0,p.length),k.topPadding(0),k.bottomPadding(0),J=[],x=!1,n=!1,l(!1)},o=function(){return T.scrollTop()+T.height()},S=function(){return T.scrollTop()},P=function(){return!x&&k.bottomDataPos()<o()+q()},s=function(){var b,c,d,e,f,g;for(b=0,e=0,c=f=g=p.length-1;(0>=g?0>=f:f>=0)&&(d=p[c].element.outerHeight(!0),k.bottomDataPos()-b-d>o()+q());c=0>=g?++f:--f)b+=d,e++,x=!1;return e>0?(k.bottomPadding(k.bottomPadding()+b),L(p.length-e,p.length),I-=e,a.log("clipped off bottom "+e+" bottom padding "+k.bottomPadding())):void 0},Q=function(){return!n&&k.topDataPos()>S()-q()},t=function(){var b,c,d,e,f,g;for(e=0,d=0,f=0,g=p.length;g>f&&(b=p[f],c=b.element.outerHeight(!0),k.topDataPos()+e+c<S()-q());f++)e+=c,d++,n=!1;return d>0?(k.topPadding(k.topPadding()+e),L(0,d),B+=d,a.log("clipped off top "+d+" top padding "+k.topPadding())):void 0},w=function(a,b){return E||(E=!0,G(!0)),1===J.push(a)?z(b):void 0},C=function(a,b){var c,d,e;return c=f.$new(),c[F]=b,d=a>B,c.$index=a,d&&c.$index--,e={scope:c},g(c,function(b){return e.element=b,d?a===I?(k.append(b),p.push(e)):(p[a-B].element.after(b),p.splice(a-B+1,0,e)):(k.prepend(b),p.unshift(e))}),{appended:d,wrapper:e}},m=function(a,b){var c;return a?k.bottomPadding(Math.max(0,k.bottomPadding()-b.element.outerHeight(!0))):(c=k.topPadding()-b.element.outerHeight(!0),c>=0?k.topPadding(c):T.scrollTop(T.scrollTop()+b.element.outerHeight(!0)))},l=function(b,c,e){var f;return f=function(){return a.log("top {actual="+k.topDataPos()+" visible from="+S()+" bottom {visible through="+o()+" actual="+k.bottomDataPos()+"}"),P()?w(!0,b):Q()&&w(!1,b),e?e():void 0},c?d(function(){var a,b,d;for(b=0,d=c.length;d>b;b++)a=c[b],m(a.appended,a.wrapper);return f()}):f()},A=function(a,b){return l(a,b,function(){return J.shift(),0===J.length?(E=!1,G(!1)):z(a)})},z=function(b){var c;return c=J[0],c?p.length&&!P()?A(b):u.get(I,r,function(c){var d,e,f,g;if(e=[],0===c.length)x=!0,k.bottomPadding(0),a.log("appended: requested "+r+" records starting from "+I+" recieved: eof");else{for(t(),f=0,g=c.length;g>f;f++)d=c[f],e.push(C(++I,d));a.log("appended: requested "+r+" received "+c.length+" buffer size "+p.length+" first "+B+" next "+I)}return A(b,e)}):p.length&&!Q()?A(b):u.get(B-r,r,function(c){var d,e,f,g;if(e=[],0===c.length)n=!0,k.topPadding(0),a.log("prepended: requested "+r+" records starting from "+(B-r)+" recieved: bof");else{for(s(),d=f=g=c.length-1;0>=g?0>=f:f>=0;d=0>=g?++f:--f)e.unshift(C(--B,c[d]));a.log("prepended: requested "+r+" received "+c.length+" buffer size "+p.length+" first "+B+" next "+I)}return A(b,e)})},M=function(){return c.$$phase||E?void 0:(l(!1),f.$apply())},T.bind("resize",M),N=function(){return c.$$phase||E?void 0:(l(!0),f.$apply())},T.bind("scroll",N),f.$watch(u.revision,function(){return K()}),y=u.scope?u.scope.$new():f.$new(),f.$on("$destroy",function(){return y.$destroy(),T.unbind("resize",M),T.unbind("scroll",N)}),y.$on("update.items",function(a,b,c){var d,e,f,g,h;if(angular.isFunction(b))for(e=function(a){return b(a.scope)},f=0,g=p.length;g>f;f++)d=p[f],e(d);else 0<=(h=b-B-1)&&h<p.length&&(p[b-B-1].scope[F]=c);return null}),y.$on("delete.items",function(a,b){var c,d,e,f,g,h,i,j,k,m,n,o;if(angular.isFunction(b)){for(e=[],h=0,k=p.length;k>h;h++)d=p[h],e.unshift(d);for(g=function(a){return b(a.scope)?(L(e.length-1-c,e.length-c),I--):void 0},c=i=0,m=e.length;m>i;c=++i)f=e[c],g(f)}else 0<=(o=b-B-1)&&o<p.length&&(L(b-B-1,b-B),I--);for(c=j=0,n=p.length;n>j;c=++j)d=p[c],d.scope.$index=B+c;return l(!1)}),y.$on("insert.item",function(a,b,c){var d,e,f,g,h,i,j,k,m,n,o,q;if(e=[],angular.isFunction(b)){for(f=[],i=0,m=p.length;m>i;i++)c=p[i],f.unshift(c);for(h=function(a){var f,g,h,i,j;if(g=b(a.scope)){if(C=function(a,b){return C(a,b),I++},angular.isArray(g)){for(j=[],f=h=0,i=g.length;i>h;f=++h)c=g[f],j.push(e.push(C(d+f,c)));return j}return e.push(C(d,g))}},d=j=0,n=f.length;n>j;d=++j)g=f[d],h(g)}else 0<=(q=b-B-1)&&q<p.length&&(e.push(C(b,c)),I++);for(d=k=0,o=p.length;o>k;d=++k)c=p[d],c.scope.$index=B+d;return l(!1,e)})}}}}]),angular.module("ui.scrollfix",[]).directive("uiScrollfix",["$window",function(a){return{require:"^?uiScrollfixTarget",link:function(b,c,d,e){function f(){var b;if(angular.isDefined(a.pageYOffset))b=a.pageYOffset;else{var e=document.compatMode&&"BackCompat"!==document.compatMode?document.documentElement:document.body;b=e.scrollTop}!c.hasClass("ui-scrollfix")&&b>d.uiScrollfix?c.addClass("ui-scrollfix"):c.hasClass("ui-scrollfix")&&b<d.uiScrollfix&&c.removeClass("ui-scrollfix")}var g=c[0].offsetTop,h=e&&e.$element||angular.element(a);d.uiScrollfix?"string"==typeof d.uiScrollfix&&("-"===d.uiScrollfix.charAt(0)?d.uiScrollfix=g-parseFloat(d.uiScrollfix.substr(1)):"+"===d.uiScrollfix.charAt(0)&&(d.uiScrollfix=g+parseFloat(d.uiScrollfix.substr(1)))):d.uiScrollfix=g,h.on("scroll",f),b.$on("$destroy",function(){h.off("scroll",f)})}}}]).directive("uiScrollfixTarget",[function(){return{controller:["$element",function(a){this.$element=a}]}}]),angular.module("ui.showhide",[]).directive("uiShow",[function(){return function(a,b,c){a.$watch(c.uiShow,function(a){a?b.addClass("ui-show"):b.removeClass("ui-show")})}}]).directive("uiHide",[function(){return function(a,b,c){a.$watch(c.uiHide,function(a){a?b.addClass("ui-hide"):b.removeClass("ui-hide")})}}]).directive("uiToggle",[function(){return function(a,b,c){a.$watch(c.uiToggle,function(a){a?b.removeClass("ui-hide").addClass("ui-show"):b.removeClass("ui-show").addClass("ui-hide")})}}]),angular.module("ui.unique",[]).filter("unique",["$parse",function(a){return function(b,c){if(c===!1)return b;if((c||angular.isUndefined(c))&&angular.isArray(b)){var d=[],e=angular.isString(c)?a(c):function(a){return a},f=function(a){return angular.isObject(a)?e(a):a};angular.forEach(b,function(a){for(var b=!1,c=0;c<d.length;c++)if(angular.equals(f(d[c]),f(a))){b=!0;break}b||d.push(a)}),b=d}return b}}]),angular.module("ui.validate",[]).directive("uiValidate",function(){return{restrict:"A",require:"ngModel",link:function(a,b,c,d){function e(b){return angular.isString(b)?void a.$watch(b,function(){angular.forEach(g,function(a){a(d.$modelValue)})}):angular.isArray(b)?void angular.forEach(b,function(b){a.$watch(b,function(){angular.forEach(g,function(a){a(d.$modelValue)})})}):void(angular.isObject(b)&&angular.forEach(b,function(b,c){angular.isString(b)&&a.$watch(b,function(){g[c](d.$modelValue)}),angular.isArray(b)&&angular.forEach(b,function(b){a.$watch(b,function(){g[c](d.$modelValue)})})}))}var f,g={},h=a.$eval(c.uiValidate);h&&(angular.isString(h)&&(h={validator:h}),angular.forEach(h,function(b,c){f=function(e){var f=a.$eval(b,{$value:e});return angular.isObject(f)&&angular.isFunction(f.then)?(f.then(function(){d.$setValidity(c,!0)},function(){d.$setValidity(c,!1)}),e):f?(d.$setValidity(c,!0),e):(d.$setValidity(c,!1),e)},g[c]=f,d.$formatters.push(f),d.$parsers.push(f)}),c.uiValidateWatch&&e(a.$eval(c.uiValidateWatch)))}}}),angular.module("ui.utils",["ui.event","ui.format","ui.highlight","ui.include","ui.indeterminate","ui.inflector","ui.jq","ui.keypress","ui.mask","ui.reset","ui.route","ui.scrollfix","ui.scroll","ui.scroll.jqlite","ui.showhide","ui.unique","ui.validate"]);
/*! 
 * angular-loading-bar v0.6.0
 * https://chieffancypants.github.io/angular-loading-bar
 * Copyright (c) 2014 Wes Cruver
 * License: MIT
 */
!function(){"use strict";angular.module("angular-loading-bar",["cfp.loadingBarInterceptor"]),angular.module("chieffancypants.loadingBar",["cfp.loadingBarInterceptor"]),angular.module("cfp.loadingBarInterceptor",["cfp.loadingBar"]).config(["$httpProvider",function(a){var b=["$q","$cacheFactory","$timeout","$rootScope","cfpLoadingBar",function(b,c,d,e,f){function g(){d.cancel(i),f.complete(),k=0,j=0}function h(b){var d,e=c.get("$http"),f=a.defaults;!b.cache&&!f.cache||b.cache===!1||"GET"!==b.method&&"JSONP"!==b.method||(d=angular.isObject(b.cache)?b.cache:angular.isObject(f.cache)?f.cache:e);var g=void 0!==d?void 0!==d.get(b.url):!1;return void 0!==b.cached&&g!==b.cached?b.cached:(b.cached=g,g)}var i,j=0,k=0,l=f.latencyThreshold;return{request:function(a){return a.ignoreLoadingBar||h(a)||(e.$broadcast("cfpLoadingBar:loading",{url:a.url}),0===j&&(i=d(function(){f.start()},l)),j++,f.set(k/j)),a},response:function(a){return a.config.ignoreLoadingBar||h(a.config)||(k++,e.$broadcast("cfpLoadingBar:loaded",{url:a.config.url}),k>=j?g():f.set(k/j)),a},responseError:function(a){return a.config.ignoreLoadingBar||h(a.config)||(k++,e.$broadcast("cfpLoadingBar:loaded",{url:a.config.url}),k>=j?g():f.set(k/j)),b.reject(a)}}}];a.interceptors.push(b)}]),angular.module("cfp.loadingBar",[]).provider("cfpLoadingBar",function(){this.includeSpinner=!0,this.includeBar=!0,this.latencyThreshold=100,this.startSize=.02,this.parentSelector="body",this.spinnerTemplate='<div id="loading-bar-spinner"><div class="spinner-icon"></div></div>',this.loadingBarTemplate='<div id="loading-bar"><div class="bar"><div class="peg"></div></div></div>',this.$get=["$injector","$document","$timeout","$rootScope",function(a,b,c,d){function e(){k||(k=a.get("$animate"));var e=b.find(n).eq(0);c.cancel(m),r||(d.$broadcast("cfpLoadingBar:started"),r=!0,u&&k.enter(o,e),t&&k.enter(q,e),f(v))}function f(a){if(r){var b=100*a+"%";p.css("width",b),s=a,c.cancel(l),l=c(function(){g()},250)}}function g(){if(!(h()>=1)){var a=0,b=h();a=b>=0&&.25>b?(3*Math.random()+3)/100:b>=.25&&.65>b?3*Math.random()/100:b>=.65&&.9>b?2*Math.random()/100:b>=.9&&.99>b?.005:0;var c=h()+a;f(c)}}function h(){return s}function i(){s=0,r=!1}function j(){k||(k=a.get("$animate")),d.$broadcast("cfpLoadingBar:completed"),f(1),c.cancel(m),m=c(function(){var a=k.leave(o,i);a&&a.then&&a.then(i),k.leave(q)},500)}var k,l,m,n=this.parentSelector,o=angular.element(this.loadingBarTemplate),p=o.find("div").eq(0),q=angular.element(this.spinnerTemplate),r=!1,s=0,t=this.includeSpinner,u=this.includeBar,v=this.startSize;return{start:e,set:f,status:h,inc:g,complete:j,includeSpinner:this.includeSpinner,latencyThreshold:this.latencyThreshold,parentSelector:this.parentSelector,startSize:this.startSize}}]})}();
/*!
 * angular-translate - v2.18.3 - 2020-07-08
 * 
 * Copyright (c) 2020 The angular-translate team, Pascal Precht; Licensed MIT
 */
!function(t,e){"function"==typeof define&&define.amd?define([],function(){return e()}):"object"==typeof module&&module.exports?module.exports=e():e()}(0,function(){function t(e){"use strict";var n=e.storageKey(),a=e.storage(),t=function(){var t=e.preferredLanguage();angular.isString(t)?e.use(t):a.put(n,e.use())};t.displayName="fallbackFromIncorrectStorageValue",a?a.get(n)?e.use(a.get(n)).catch(t):t():angular.isString(e.preferredLanguage())&&e.use(e.preferredLanguage())}function e(t,r,e,i){"use strict";var T,c,z,x,F,I,_,n,V,R,D,K,U,M,H,G,q={},Y=[],B=t,J=[],Q="translate-cloak",W=!1,X=!1,Z=".",tt=!1,et=!1,nt=0,at=!0,a="default",s={default:function(t){return(t||"").split("-").join("_")},java:function(t){var e=(t||"").split("-").join("_"),n=e.split("_");return 1<n.length?n[0].toLowerCase()+"_"+n[1].toUpperCase():e},bcp47:function(t){var e=(t||"").split("_").join("-"),n=e.split("-");switch(n.length){case 1:n[0]=n[0].toLowerCase();break;case 2:n[0]=n[0].toLowerCase(),4===n[1].length?n[1]=n[1].charAt(0).toUpperCase()+n[1].slice(1).toLowerCase():n[1]=n[1].toUpperCase();break;case 3:n[0]=n[0].toLowerCase(),n[1]=n[1].charAt(0).toUpperCase()+n[1].slice(1).toLowerCase(),n[2]=n[2].toUpperCase();break;default:return e}return n.join("-")},"iso639-1":function(t){return(t||"").split("_").join("-").split("-")[0].toLowerCase()}},o=function(){if(angular.isFunction(i.getLocale))return i.getLocale();var t,e,n=r.$get().navigator,a=["language","browserLanguage","systemLanguage","userLanguage"];if(angular.isArray(n.languages))for(t=0;t<n.languages.length;t++)if((e=n.languages[t])&&e.length)return e;for(t=0;t<a.length;t++)if((e=n[a[t]])&&e.length)return e;return null};o.displayName="angular-translate/service: getFirstBrowserLanguage";var rt=function(){var t=o()||"";return s[a]&&(t=s[a](t)),t};rt.displayName="angular-translate/service: getLocale";var it=function(t,e){for(var n=0,a=t.length;n<a;n++)if(t[n]===e)return n;return-1},st=function(){return this.toString().replace(/^\s+|\s+$/g,"")},f=function(t){return angular.isString(t)?t.toLowerCase():t},ot=function(t){if(t){for(var e,n=[],a=f(t),r=0,i=Y.length;r<i;r++)n.push(f(Y[r]));if(-1<(r=it(n,a)))return Y[r];if(c)for(var s in c)if(c.hasOwnProperty(s)){var o=!1,l=Object.prototype.hasOwnProperty.call(c,s)&&f(s)===f(t);if("*"===s.slice(-1)&&(o=f(s.slice(0,-1))===f(t.slice(0,s.length-1))),(l||o)&&(e=c[s],-1<it(n,f(e))))return e}var u=t.split("_");return 1<u.length&&-1<it(n,f(u[0]))?u[0]:void 0}},lt=function(t,e){if(!t&&!e)return q;if(t&&!e){if(angular.isString(t))return q[t]}else angular.isObject(q[t])||(q[t]={}),angular.extend(q[t],ut(e));return this};this.translations=lt,this.cloakClassName=function(t){return t?(Q=t,this):Q},this.nestedObjectDelimeter=function(t){return t?(Z=t,this):Z};var ut=function(t,e,n,a){var r,i,s;for(r in e||(e=[]),n||(n={}),t)Object.prototype.hasOwnProperty.call(t,r)&&(s=t[r],angular.isObject(s)?ut(s,e.concat(r),n,r):(i=e.length?""+e.join(Z)+Z+r:r,e.length&&r===a&&(n[""+e.join(Z)]="@:"+i),n[i]=s));return n};ut.displayName="flatObject",this.addInterpolation=function(t){return J.push(t),this},this.useMessageFormatInterpolation=function(){return this.useInterpolation("$translateMessageFormatInterpolation")},this.useInterpolation=function(t){return R=t,this},this.useSanitizeValueStrategy=function(t){return e.useStrategy(t),this},this.preferredLanguage=function(t){return t?(ct(t),this):T};var ct=function(t){return t&&(T=t),T};this.translationNotFoundIndicator=function(t){return this.translationNotFoundIndicatorLeft(t),this.translationNotFoundIndicatorRight(t),this},this.translationNotFoundIndicatorLeft=function(t){return t?(U=t,this):U},this.translationNotFoundIndicatorRight=function(t){return t?(M=t,this):M},this.fallbackLanguage=function(t){return ft(t),this};var ft=function(t){return t?(angular.isString(t)?(x=!0,z=[t]):angular.isArray(t)&&(x=!1,z=t),angular.isString(T)&&it(z,T)<0&&z.push(T),this):x?z[0]:z};this.use=function(t){if(t){if(!q[t]&&!D)throw new Error("$translateProvider couldn't find translationTable for langKey: '"+t+"'");return F=t,this}return F},this.resolveClientLocale=function(){return rt()};var gt=function(t){return t?(B=t,this):n?n+B:B};this.storageKey=gt,this.useUrlLoader=function(t,e){return this.useLoader("$translateUrlLoader",angular.extend({url:t},e))},this.useStaticFilesLoader=function(t){return this.useLoader("$translateStaticFilesLoader",t)},this.useLoader=function(t,e){return D=t,K=e||{},this},this.useLocalStorage=function(){return this.useStorage("$translateLocalStorage")},this.useCookieStorage=function(){return this.useStorage("$translateCookieStorage")},this.useStorage=function(t){return _=t,this},this.storagePrefix=function(t){return t?(n=t,this):t},this.useMissingTranslationHandlerLog=function(){return this.useMissingTranslationHandler("$translateMissingTranslationHandlerLog")},this.useMissingTranslationHandler=function(t){return V=t,this},this.usePostCompiling=function(t){return W=!!t,this},this.forceAsyncReload=function(t){return X=!!t,this},this.uniformLanguageTag=function(t){return t?angular.isString(t)&&(t={standard:t}):t={},a=t.standard,this},this.determinePreferredLanguage=function(t){var e=t&&angular.isFunction(t)?t():rt();return T=Y.length&&ot(e)||e,this},this.registerAvailableLanguageKeys=function(t,e){return t?(Y=t,e&&(c=e),this):Y},this.useLoaderCache=function(t){return!1===t?H=void 0:!0===t?H=!0:void 0===t?H="$translationCache":t&&(H=t),this},this.directivePriority=function(t){return void 0===t?nt:(nt=t,this)},this.statefulFilter=function(t){return void 0===t?at:(at=t,this)},this.postProcess=function(t){return G=t||void 0,this},this.keepContent=function(t){return et=!!t,this},this.$get=["$log","$injector","$rootScope","$q",function(t,o,s,m){var i,$,y,b=o.get(R||"$translateDefaultInterpolation"),S=!1,L={},f={},w=function(t,s,o,l,u,c){!F&&T&&(F=T);var a=u&&u!==F?ot(u)||u:F;if(u&&v(u),angular.isArray(t)){return function(t){for(var a={},e=[],n=function(e){var n=m.defer(),t=function(t){a[e]=t,n.resolve([e,t])};return w(e,s,o,l,u,c).then(t,t),n.promise},r=0,i=t.length;r<i;r++)e.push(n(t[r]));return m.all(e).then(function(){return a})}(t)}var e=m.defer();if(!t)throw new TypeError("translationId must be a not empty string");t=st.apply(t);var n=function(){var t=f[a]||f[T];if($=0,_&&!t){var e=i.get(B);if(t=f[e],z&&z.length){var n=it(z,e);$=0===n?1:0,it(z,T)<0&&z.push(T)}}return t}();if(n){var r=function(){u||(a=F),h(t,s,o,l,a,c).then(e.resolve,e.reject)};r.displayName="promiseResolved",n.finally(r).catch(angular.noop)}else h(t,s,o,l,a,c).then(e.resolve,e.reject);return e.promise},j=function(t){return U&&(t=[U,t].join(" ")),M&&(t=[t,M].join(" ")),t},l=function(t){F=t,_&&i.put(w.storageKey(),F),s.$emit("$translateChangeSuccess",{language:t}),b.setLocale(F);var e=function(t,e){L[e].setLocale(F)};e.displayName="eachInterpolatorLocaleSetter",angular.forEach(L,e),s.$emit("$translateChangeEnd",{language:t})},u=function(n){if(!n)throw"No language key specified for loading.";var a=m.defer();s.$emit("$translateLoadingStart",{language:n}),S=!0;var t=H;"string"==typeof t&&(t=o.get(t));var e=angular.extend({},K,{key:n,$http:angular.extend({},{cache:t},K.$http)}),r=function(t){var e={};s.$emit("$translateLoadingSuccess",{language:n}),angular.isArray(t)?angular.forEach(t,function(t){angular.extend(e,ut(t))}):angular.extend(e,ut(t)),S=!1,a.resolve({key:n,table:e}),s.$emit("$translateLoadingEnd",{language:n})};r.displayName="onLoaderSuccess";var i=function(t){s.$emit("$translateLoadingError",{language:t}),a.reject(t),s.$emit("$translateLoadingEnd",{language:t})};return i.displayName="onLoaderError",o.get(D)(e).then(r,i),a.promise};if(_&&(!(i=o.get(_)).get||!i.put))throw new Error("Couldn't use storage '"+_+"', missing get() or put() method!");if(J.length){var e=function(t){var e=o.get(t);e.setLocale(T||F),L[e.getInterpolationIdentifier()]=e};e.displayName="interpolationFactoryAdder",angular.forEach(J,e)}var c=function(a,r,i,s,o){var l=m.defer(),t=function(t){if(Object.prototype.hasOwnProperty.call(t,r)&&null!==t[r]){s.setLocale(a);var e=t[r];if("@:"===e.substr(0,2))c(a,e.substr(2),i,s,o).then(l.resolve,l.reject);else{var n=s.interpolate(t[r],i,"service",o,r);n=O(r,t[r],n,i,a),l.resolve(n)}s.setLocale(F)}else l.reject()};return t.displayName="fallbackTranslationResolver",function(t){var e=m.defer();if(Object.prototype.hasOwnProperty.call(q,t))e.resolve(q[t]);else if(f[t]){var n=function(t){lt(t.key,t.table),e.resolve(t.table)};n.displayName="translationTableResolver",f[t].then(n,e.reject)}else e.reject();return e.promise}(a).then(t,l.reject),l.promise},g=function(t,e,n,a,r){var i,s=q[t];if(s&&Object.prototype.hasOwnProperty.call(s,e)&&null!==s[e]){if(a.setLocale(t),i=a.interpolate(s[e],n,"filter",r,e),i=O(e,s[e],i,n,t,r),!angular.isString(i)&&angular.isFunction(i.$$unwrapTrustedValue)){var o=i.$$unwrapTrustedValue();if("@:"===o.substr(0,2))return g(t,o.substr(2),n,a,r)}else if("@:"===i.substr(0,2))return g(t,i.substr(2),n,a,r);a.setLocale(F)}return i},C=function(t,e,n,a){return V?o.get(V)(t,F,e,n,a):t},N=function(t,e,n,a,r,i){var s=m.defer();if(t<z.length){var o=z[t];c(o,e,n,a,i).then(function(t){s.resolve(t)},function(){return N(t+1,e,n,a,r,i).then(s.resolve,s.reject)})}else if(r)s.resolve(r);else{var l=C(e,n,r);V&&l?s.resolve(l):s.reject(j(e))}return s.promise},p=function(t,e,n,a,r){var i;if(t<z.length){var s=z[t];(i=g(s,e,n,a,r))||""===i||(i=p(t+1,e,n,a))}return i},h=function(t,e,n,a,r,i){var s,o,l,u,c,f=m.defer(),g=r?q[r]:q,p=n?L[n]:b;if(g&&Object.prototype.hasOwnProperty.call(g,t)&&null!==g[t]){var h=g[t];if("@:"===h.substr(0,2))w(h.substr(2),e,n,a,r,i).then(f.resolve,f.reject);else{var d=p.interpolate(h,e,"service",i,t);d=O(t,h,d,e,r),f.resolve(d)}}else{var v;V&&!S&&(v=C(t,e,a)),r&&z&&z.length?(s=t,o=e,l=p,u=a,c=i,N(0<y?y:$,s,o,l,u,c)).then(function(t){f.resolve(t)},function(t){f.reject(j(t))}):V&&!S&&v?a?f.resolve(a):f.resolve(v):a?f.resolve(a):f.reject(j(t))}return f.promise},d=function(t,e,n,a,r){var i,s=a?q[a]:q,o=b;if(L&&Object.prototype.hasOwnProperty.call(L,n)&&(o=L[n]),s&&Object.prototype.hasOwnProperty.call(s,t)&&null!==s[t]){var l=s[t];i="@:"===l.substr(0,2)?d(l.substr(2),e,n,a,r):(i=o.interpolate(l,e,"filter",r,t),O(t,l,i,e,a,r))}else{var u;V&&!S&&(u=C(t,e,r)),i=a&&z&&z.length?p(($=0)<y?y:$,t,e,o,r):V&&!S&&u?u:j(t)}return i},O=function(t,e,n,a,r,i){var s=G;return s&&("string"==typeof s&&(s=o.get(s)),s)?s(t,e,n,a,r,i):n},v=function(t){q[t]||!D||f[t]||(f[t]=u(t).then(function(t){return lt(t.key,t.table),t}))};w.preferredLanguage=function(t){return t&&ct(t),T},w.cloakClassName=function(){return Q},w.nestedObjectDelimeter=function(){return Z},w.fallbackLanguage=function(t){if(null!=t){if(ft(t),D&&z&&z.length)for(var e=0,n=z.length;e<n;e++)f[z[e]]||(f[z[e]]=u(z[e]));w.use(w.use())}return x?z[0]:z},w.useFallbackLanguage=function(t){if(null!=t)if(t){var e=it(z,t);-1<e&&(y=e)}else y=0},w.proposedLanguage=function(){return I},w.storage=function(){return i},w.negotiateLocale=ot,w.use=function(e){if(!e)return F;var n=m.defer();n.promise.then(null,angular.noop),s.$emit("$translateChangeStart",{language:e});var t=ot(e);return 0<Y.length&&!t?m.reject(e):(t&&(e=t),I=e,!X&&q[e]||!D||f[e]?f[e]?f[e].then(function(t){return I===t.key&&l(t.key),n.resolve(t.key),t},function(t){return!F&&z&&0<z.length&&z[0]!==t?w.use(z[0]).then(n.resolve,n.reject):n.reject(t)}):(n.resolve(e),l(e)):(f[e]=u(e).then(function(t){return lt(t.key,t.table),n.resolve(t.key),I===e&&l(t.key),t},function(t){return s.$emit("$translateChangeError",{language:t}),n.reject(t),s.$emit("$translateChangeEnd",{language:t}),m.reject(t)}),f[e].finally(function(){var t;I===(t=e)&&(I=void 0),f[t]=void 0}).catch(angular.noop)),n.promise)},w.resolveClientLocale=function(){return rt()},w.storageKey=function(){return gt()},w.isPostCompilingEnabled=function(){return W},w.isForceAsyncReloadEnabled=function(){return X},w.isKeepContent=function(){return et},w.refresh=function(t){if(!D)throw new Error("Couldn't refresh translation table, no loader registered!");s.$emit("$translateRefreshStart",{language:t});var e=m.defer(),n={};function a(e){var t=u(e);return(f[e]=t).then(function(t){q[e]={},lt(e,t.table),n[e]=!0},angular.noop),t}if(e.promise.then(function(){for(var t in q)q.hasOwnProperty(t)&&(t in n||delete q[t]);F&&l(F)},angular.noop).finally(function(){s.$emit("$translateRefreshEnd",{language:t})}),t)q[t]?a(t).then(e.resolve,e.reject):e.reject();else{var r=z&&z.slice()||[];F&&-1===r.indexOf(F)&&r.push(F),m.all(r.map(a)).then(e.resolve,e.reject)}return e.promise},w.instant=function(t,e,n,a,r){var i=a&&a!==F?ot(a)||a:F;if(null===t||angular.isUndefined(t))return t;if(a&&v(a),angular.isArray(t)){for(var s={},o=0,l=t.length;o<l;o++)s[t[o]]=w.instant(t[o],e,n,a,r);return s}if(angular.isString(t)&&t.length<1)return t;t&&(t=st.apply(t));var u,c,f=[];T&&f.push(T),i&&f.push(i),z&&z.length&&(f=f.concat(z));for(var g=0,p=f.length;g<p;g++){var h=f[g];if(q[h]&&void 0!==q[h][t]&&(u=d(t,e,n,i,r)),void 0!==u)break}u||""===u||(U||M?u=j(t):(u=b.interpolate(t,e,"filter",r),V&&!S&&(c=C(t,e,r)),V&&!S&&c&&(u=c)));return u},w.versionInfo=function(){return"2.18.3"},w.loaderCache=function(){return H},w.directivePriority=function(){return nt},w.statefulFilter=function(){return at},w.isReady=function(){return tt};var n=m.defer();n.promise.then(function(){tt=!0}),w.onReady=function(t){var e=m.defer();return angular.isFunction(t)&&e.promise.then(t),tt?e.resolve():n.promise.then(e.resolve),e.promise},w.getAvailableLanguageKeys=function(){return 0<Y.length?Y:null},w.getTranslationTable=function(t){return(t=t||w.use())&&q[t]?angular.copy(q[t]):null};var a=s.$on("$translateReady",function(){n.resolve(),a(),a=null}),r=s.$on("$translateChangeEnd",function(){n.resolve(),r(),r=null});if(D){if(angular.equals(q,{})&&w.use()&&w.use(w.use()),z&&z.length)for(var E=function(t){return lt(t.key,t.table),s.$emit("$translateChangeEnd",{language:t.key}),t},k=0,P=z.length;k<P;k++){var A=z[k];!X&&q[A]||(f[A]=u(A).then(E))}}else s.$emit("$translateReady",{language:w.use()});return w}]}function n(s,o){"use strict";var t={};return t.setLocale=function(t){t},t.getInterpolationIdentifier=function(){return"default"},t.useSanitizeValueStrategy=function(t){return o.useStrategy(t),this},t.interpolate=function(t,e,n,a,r){var i;return e=e||{},e=o.sanitize(e,"params",a,n),i=angular.isNumber(t)?""+t:angular.isString(t)?(i=s(t)(e),o.sanitize(i,"text",a,n)):""},t}function a(S,L,w,j,C){"use strict";var N=function(t){return angular.isString(t)?t.toLowerCase():t};return{restrict:"AE",scope:!0,priority:S.directivePriority(),compile:function(t,h){var d=h.translateValues?h.translateValues:void 0,v=h.translateInterpolation?h.translateInterpolation:void 0,m=h.translateSanitizeStrategy?h.translateSanitizeStrategy:void 0,$=t[0].outerHTML.match(/translate-value-+/i),y="^(.*)("+L.startSymbol()+".*"+L.endSymbol()+")(.*)",b="^(.*)"+L.startSymbol()+"(.*)"+L.endSymbol()+"(.*)";return function(r,l,u){r.interpolateParams={},r.preText="",r.postText="",r.translateNamespace=function t(e){if(e.translateNamespace)return e.translateNamespace;if(e.$parent)return t(e.$parent)}(r);var i={},s=function(t){if(angular.isFunction(s._unwatchOld)&&(s._unwatchOld(),s._unwatchOld=void 0),angular.equals(t,"")||!angular.isDefined(t)){var e=function(){return this.toString().replace(/^\s+|\s+$/g,"")}.apply(l.text()).replace(/\n/g," "),n=e.match(y);if(angular.isArray(n)){r.preText=n[1],r.postText=n[3],i.translate=L(n[2])(r.$parent);var a=e.match(b);angular.isArray(a)&&a[2]&&a[2].length&&(s._unwatchOld=r.$watch(a[2],function(t){i.translate=t,c()}))}else i.translate=e||void 0}else i.translate=t;c()},t=function(e){u.$observe(e,function(t){i[e]=t,c()})};!function(t,e,n){if(e.translateValues&&angular.extend(t,j(e.translateValues)(r.$parent)),$)for(var a in n)Object.prototype.hasOwnProperty.call(e,a)&&"translateValue"===a.substr(0,14)&&"translateValues"!==a&&(t[N(a.substr(14,1))+a.substr(15)]=n[a])}(r.interpolateParams,u,h);var e=!0;for(var n in u.$observe("translate",function(t){void 0===t?s(""):""===t&&e||(i.translate=t,c()),e=!1}),u)u.hasOwnProperty(n)&&"translateAttr"===n.substr(0,13)&&13<n.length&&t(n);if(u.$observe("translateDefault",function(t){r.defaultText=t,c()}),m&&u.$observe("translateSanitizeStrategy",function(t){r.sanitizeStrategy=j(t)(r.$parent),c()}),d&&u.$observe("translateValues",function(t){t&&r.$parent.$watch(function(){angular.extend(r.interpolateParams,j(t)(r.$parent))})}),$){var a=function(n){u.$observe(n,function(t){var e=N(n.substr(14,1))+n.substr(15);r.interpolateParams[e]=t})};for(var o in u)Object.prototype.hasOwnProperty.call(u,o)&&"translateValue"===o.substr(0,14)&&"translateValues"!==o&&a(o)}var c=function(){for(var t in i)i.hasOwnProperty(t)&&void 0!==i[t]&&f(t,i[t],r,r.interpolateParams,r.defaultText,r.translateNamespace)},f=function(e,t,n,a,r,i){t?(i&&"."===t.charAt(0)&&(t=i+t),S(t,a,v,r,n.translateLanguage,n.sanitizeStrategy).then(function(t){g(t,n,!0,e)},function(t){g(t,n,!1,e)})):g(t,n,!1,e)},g=function(t,e,n,a){if(n||void 0!==e.defaultText&&(t=e.defaultText),"translate"===a){(n||!n&&!S.isKeepContent()&&void 0===u.translateKeepContent)&&l.empty().append(e.preText+t+e.postText);var r=S.isPostCompilingEnabled(),i=void 0!==h.translateCompile,s=i&&"false"!==h.translateCompile;(r&&!i||s)&&w(l.contents())(e)}else{var o=u.$attr[a];"data-"===o.substr(0,5)&&(o=o.substr(5)),o=o.substr(15),l.attr(o,t)}};(d||$||u.translateDefault)&&r.$watch("interpolateParams",c,!0),r.$on("translateLanguageChanged",c);var p=C.$on("$translateChangeSuccess",c);l.text().length?u.translate?s(u.translate):s(""):u.translate&&s(u.translate),c(),r.$on("$destroy",p)}}}}function r(u,c){"use strict";return{restrict:"A",priority:u.directivePriority(),link:function(n,a,r){var i,s,o,l={},t=function(){angular.forEach(i,function(t,e){t&&(l[e]=!0,n.translateNamespace&&"."===t.charAt(0)&&(t=n.translateNamespace+t),u(t,s,r.translateInterpolation,void 0,n.translateLanguage,o).then(function(t){a.attr(e,t)},function(t){a.attr(e,t)}))}),angular.forEach(l,function(t,e){i[e]||(a.removeAttr(e),delete l[e])})};f(n,r.translateAttr,function(t){i=t},t),f(n,r.translateValues,function(t){s=t},t),f(n,r.translateSanitizeStrategy,function(t){o=t},t),r.translateValues&&n.$watch(r.translateValues,t,!0),n.$on("translateLanguageChanged",t);var e=c.$on("$translateChangeSuccess",t);t(),n.$on("$destroy",e)}}}function f(t,e,n,a){"use strict";e&&("::"===e.substr(0,2)?e=e.substr(2):t.$watch(e,function(t){n(t),a()},!0),n(t.$eval(e)))}function i(s,o){"use strict";return{compile:function(t){var i=function(t){t.addClass(s.cloakClassName())};return i(t),function(t,e,n){var a=function(t){t.removeClass(s.cloakClassName())}.bind(this,e),r=i.bind(this,e);n.translateCloak&&n.translateCloak.length?(n.$observe("translateCloak",function(t){s(t).then(a,r)}),o.$on("$translateChangeSuccess",function(){s(n.translateCloak).then(a,r)})):s.onReady(a)}}}}function s(){"use strict";return{restrict:"A",scope:!0,compile:function(){return{pre:function(t,e,n){t.translateNamespace=function t(e){if(e.translateNamespace)return e.translateNamespace;if(e.$parent)return t(e.$parent)}(t),t.translateNamespace&&"."===n.translateNamespace.charAt(0)?t.translateNamespace+=n.translateNamespace:t.translateNamespace=n.translateNamespace}}}}}function o(){"use strict";return{restrict:"A",scope:!0,compile:function(){return function(e,t,n){n.$observe("translateLanguage",function(t){e.translateLanguage=t}),e.$watch("translateLanguage",function(){e.$broadcast("translateLanguageChanged")})}}}}function l(i,s){"use strict";var t=function(t,e,n,a){if(!angular.isObject(e)){var r=this||{__SCOPE_IS_NOT_AVAILABLE:"More info at https://github.com/angular/angular.js/commit/8863b9d04c722b278fa93c5d66ad1e578ad6eb1f"};e=i(e)(r)}return s.instant(t,e,n,a)};return s.statefulFilter()&&(t.$stateful=!0),t}function u(t){"use strict";return t("translations")}return t.$inject=["$translate"],e.$inject=["$STORAGE_KEY","$windowProvider","$translateSanitizationProvider","pascalprechtTranslateOverrider"],n.$inject=["$interpolate","$translateSanitization"],a.$inject=["$translate","$interpolate","$compile","$parse","$rootScope"],r.$inject=["$translate","$rootScope"],i.$inject=["$translate","$rootScope"],l.$inject=["$parse","$translate"],u.$inject=["$cacheFactory"],angular.module("pascalprecht.translate",["ng"]).run(t),t.displayName="runTranslate",angular.module("pascalprecht.translate").provider("$translateSanitization",function(){"use strict";var n,a,g,p=null,h=!1,d=!1;(g={sanitize:function(t,e){return"text"===e&&(t=i(t)),t},escape:function(t,e){return"text"===e&&(t=r(t)),t},sanitizeParameters:function(t,e){return"params"===e&&(t=o(t,i)),t},escapeParameters:function(t,e){return"params"===e&&(t=o(t,r)),t},sce:function(t,e,n){return"text"===e?t=s(t):"params"===e&&"filter"!==n&&(t=o(t,r)),t},sceParameters:function(t,e){return"params"===e&&(t=o(t,s)),t}}).escaped=g.escapeParameters,this.addStrategy=function(t,e){return g[t]=e,this},this.removeStrategy=function(t){return delete g[t],this},this.useStrategy=function(t){return h=!0,p=t,this},this.$get=["$injector","$log",function(u,c){var e,f={};return u.has("$sanitize")&&(n=u.get("$sanitize")),u.has("$sce")&&(a=u.get("$sce")),{useStrategy:(e=this,function(t){e.useStrategy(t)}),sanitize:function(t,e,n,a){if(p||h||d||(c.warn("pascalprecht.translate.$translateSanitization: No sanitization strategy has been configured. This can have serious security implications. See http://angular-translate.github.io/docs/#/guide/19_security for details."),d=!0),n||null===n||(n=p),!n)return t;a||(a="service");var r,i,s,o,l=angular.isArray(n)?n:[n];return r=t,i=e,s=a,o=l,angular.forEach(o,function(e){if(angular.isFunction(e))r=e(r,i,s);else if(angular.isFunction(g[e]))r=g[e](r,i,s);else{if(!angular.isString(g[e]))throw new Error("pascalprecht.translate.$translateSanitization: Unknown sanitization strategy: '"+e+"'");if(!f[g[e]])try{f[g[e]]=u.get(g[e])}catch(t){throw f[g[e]]=function(){},new Error("pascalprecht.translate.$translateSanitization: Unknown sanitization strategy: '"+e+"'")}r=f[g[e]](r,i,s)}}),r}}}];var r=function(t){var e=angular.element("<div></div>");return e.text(t),e.html()},i=function(t){if(!n)throw new Error("pascalprecht.translate.$translateSanitization: Error cannot find $sanitize service. Either include the ngSanitize module (https://docs.angularjs.org/api/ngSanitize) or use a sanitization strategy which does not depend on $sanitize, such as 'escape'.");return n(t)},s=function(t){if(!a)throw new Error("pascalprecht.translate.$translateSanitization: Error cannot find $sce service.");return a.trustAsHtml(t)},o=function(t,n,a){if(angular.isDate(t))return t;if(angular.isObject(t)){var r=angular.isArray(t)?[]:{};if(a){if(-1<a.indexOf(t))throw new Error("pascalprecht.translate.$translateSanitization: Error cannot interpolate parameter due recursive object")}else a=[];return a.push(t),angular.forEach(t,function(t,e){angular.isFunction(t)||(r[e]=o(t,n,a))}),a.splice(-1,1),r}return angular.isNumber(t)?t:!0===t||!1===t?t:angular.isUndefined(t)||null===t?t:n(t)}}),angular.module("pascalprecht.translate").constant("pascalprechtTranslateOverrider",{}).provider("$translate",e),e.displayName="displayName",angular.module("pascalprecht.translate").factory("$translateDefaultInterpolation",n),n.displayName="$translateDefaultInterpolation",angular.module("pascalprecht.translate").constant("$STORAGE_KEY","NG_TRANSLATE_LANG_KEY"),angular.module("pascalprecht.translate").directive("translate",a),a.displayName="translateDirective",angular.module("pascalprecht.translate").directive("translateAttr",r),r.displayName="translateAttrDirective",angular.module("pascalprecht.translate").directive("translateCloak",i),i.displayName="translateCloakDirective",angular.module("pascalprecht.translate").directive("translateNamespace",s),s.displayName="translateNamespaceDirective",angular.module("pascalprecht.translate").directive("translateLanguage",o),o.displayName="translateLanguageDirective",angular.module("pascalprecht.translate").filter("translate",l),l.displayName="translateFilterFactory",angular.module("pascalprecht.translate").factory("$translationCache",u),u.displayName="$translationCache","pascalprecht.translate"});
/*!
 * angular-translate - v2.18.4 - 2021-01-14
 * 
 * Copyright (c) 2021 The angular-translate team, Pascal Precht; Licensed MIT
 */
!function(t,e){"function"==typeof define&&define.amd?define([],function(){return e()}):"object"==typeof module&&module.exports?module.exports=e():e()}(0,function(){function t(){"use strict";function a(t,e,r){this.name=t,this.isActive=!0,this.tables={},this.priority=e||0,this.langPromises={},this.urlTemplate=r}a.prototype.parseUrl=function(t,e){return angular.isFunction(t)?t(this.name,e):t.replace(/\{part\}/g,this.name).replace(/\{lang\}/g,e)},a.prototype.getTable=function(e,t,r,a,n,i){var o=this,s=this.langPromises[e],l=t.defer(),u=function(t){o.tables[e]=t,l.resolve(t)},c=function(){l.reject(o.name)},p=function(){r(angular.extend({method:"GET",url:o.parseUrl(o.urlTemplate||n,e)},a)).then(function(t){u(t.data)},function(t){i?i(o.name,e,t).then(u,c):c()})};return this.tables[e]?l.resolve(this.tables[e]):(s?s.then(l.resolve,p):p(),this.langPromises[e]=l.promise),l.promise};var n={};function i(t){return Object.prototype.hasOwnProperty.call(n,t)}function f(t){return angular.isString(t)&&""!==t}function t(t){if(!f(t))throw new TypeError("Invalid type of a first argument, a non-empty string expected.");return i(t)&&n[t].isActive}function d(){var t=[];for(var e in n)n[e].isActive&&t.push(n[e]);return t.sort(function(t,e){return t.priority-e.priority}),t}this.addPart=function(t,e,r){if(!f(t))throw new TypeError("Couldn't add part, part name has to be a string!");return i(t)||(n[t]=new a(t,e,r)),n[t].isActive=!0,this},this.setPart=function(t,e,r){if(!f(t))throw new TypeError("Couldn't set part.`lang` parameter has to be a string!");if(!f(e))throw new TypeError("Couldn't set part.`part` parameter has to be a string!");if("object"!=typeof r||null===r)throw new TypeError("Couldn't set part. `table` parameter has to be an object!");return i(e)||(n[e]=new a(e),n[e].isActive=!1),n[e].tables[t]=r,this},this.deletePart=function(t){if(!f(t))throw new TypeError("Couldn't delete part, first arg has to be string.");return i(t)&&(n[t].isActive=!1),this},this.isPartAvailable=t,this.$get=["$rootScope","$injector","$q","$http","$log",function(o,s,l,u,c){var p=function(r){if(!f(r.key))throw new TypeError("Unable to load data, a key is not a non-empty string.");if(!f(r.urlTemplate)&&!angular.isFunction(r.urlTemplate))throw new TypeError("Unable to load data, a urlTemplate is not a non-empty string or not a function.");var e=r.loadFailureHandler;if(void 0!==e){if(!angular.isString(e))throw new Error("Unable to load data, a loadFailureHandler is not a string.");e=s.get(e)}var a=[],t=d();angular.forEach(t,function(t){a.push(t.getTable(r.key,l,u,r.$http,r.urlTemplate,e)),t.urlTemplate=t.urlTemplate||r.urlTemplate});var n=!1,i=o.$on("$translatePartialLoaderStructureChanged",function(){n=!0});return l.all(a).then(function(){if(i(),n){if(!r.__retries)return r.__retries=(r.__retries||0)+1,p(r);c.warn("The partial loader has detected a multiple structure change (with addPort/removePart) while loading translations. You should consider using promises of $translate.use(lang) and $translate.refresh(). Also parts should be added/removed right before an explicit refresh if possible.")}var e={};return t=d(),angular.forEach(t,function(t){!function t(e,r){for(var a in r)r[a]&&r[a].constructor&&r[a].constructor===Object?(e[a]=e[a]||{},t(e[a],r[a])):e[a]=r[a];return e}(e,t.tables[r.key])}),e},function(){return i(),l.reject(r.key)})};return p.addPart=function(t,e,r){if(!f(t))throw new TypeError("Couldn't add part, first arg has to be a string");return i(t)?n[t].isActive||(n[t].isActive=!0,o.$emit("$translatePartialLoaderStructureChanged",t)):(n[t]=new a(t,e,r),o.$emit("$translatePartialLoaderStructureChanged",t)),p},p.deletePart=function(r,t){if(!f(r))throw new TypeError("Couldn't delete part, first arg has to be string");if(void 0===t)t=!1;else if("boolean"!=typeof t)throw new TypeError("Invalid type of a second argument, a boolean expected.");if(i(r)){var e=n[r].isActive;if(t){var a=s.get("$translate").loaderCache();"string"==typeof a&&(a=s.get(a)),"object"==typeof a&&angular.forEach(n[r].tables,function(t,e){a.remove(n[r].parseUrl(n[r].urlTemplate,e))}),delete n[r]}else n[r].isActive=!1;e&&o.$emit("$translatePartialLoaderStructureChanged",r)}return p},p.isPartLoaded=function(t,e){return angular.isDefined(n[t])&&angular.isDefined(n[t].tables[e])},p.getRegisteredParts=function(){var e=[];return angular.forEach(n,function(t){t.isActive&&e.push(t.name)}),e},p.isPartAvailable=t,p}]}return angular.module("pascalprecht.translate").provider("$translatePartialLoader",t),t.displayName="$translatePartialLoader","pascalprecht.translate"});
/*
 AngularJS v1.5.0
 (c) 2010-2016 Google, Inc. http://angularjs.org
 License: MIT
*/
(function(s,q,t){'use strict';var f="BUTTON A INPUT TEXTAREA SELECT DETAILS SUMMARY".split(" "),l=function(a,c){if(-1!==c.indexOf(a[0].nodeName))return!0};q.module("ngAria",["ng"]).provider("$aria",function(){function a(a,h,p,n){return function(d,e,b){var g=b.$normalize(h);!c[g]||l(e,p)||b[g]||d.$watch(b[a],function(b){b=n?!b:!!b;e.attr(h,b)})}}var c={ariaHidden:!0,ariaChecked:!0,ariaDisabled:!0,ariaRequired:!0,ariaInvalid:!0,ariaValue:!0,tabindex:!0,bindKeypress:!0,bindRoleForClick:!0};this.config=
function(a){c=q.extend(c,a)};this.$get=function(){return{config:function(a){return c[a]},$$watchExpr:a}}}).directive("ngShow",["$aria",function(a){return a.$$watchExpr("ngShow","aria-hidden",[],!0)}]).directive("ngHide",["$aria",function(a){return a.$$watchExpr("ngHide","aria-hidden",[],!1)}]).directive("ngValue",["$aria",function(a){return a.$$watchExpr("ngValue","aria-checked",f,!1)}]).directive("ngChecked",["$aria",function(a){return a.$$watchExpr("ngChecked","aria-checked",f,!1)}]).directive("ngRequired",
["$aria",function(a){return a.$$watchExpr("ngRequired","aria-required",f,!1)}]).directive("ngModel",["$aria",function(a){function c(c,n,d,e){return a.config(n)&&!d.attr(c)&&(e||!l(d,f))}function m(a,c){return!c.attr("role")&&c.attr("type")===a&&"INPUT"!==c[0].nodeName}function h(a,c){var d=a.type,e=a.role;return"checkbox"===(d||e)||"menuitemcheckbox"===e?"checkbox":"radio"===(d||e)||"menuitemradio"===e?"radio":"range"===d||"progressbar"===e||"slider"===e?"range":""}return{restrict:"A",require:"ngModel",
priority:200,compile:function(f,n){var d=h(n,f);return{pre:function(a,b,c,k){"checkbox"===d&&(k.$isEmpty=function(a){return!1===a})},post:function(e,b,g,k){function f(){return k.$modelValue}function h(a){b.attr("aria-checked",g.value==k.$viewValue)}function n(){b.attr("aria-checked",!k.$isEmpty(k.$viewValue))}var l=c("tabindex","tabindex",b,!1);switch(d){case "radio":case "checkbox":m(d,b)&&b.attr("role",d);c("aria-checked","ariaChecked",b,!1)&&e.$watch(f,"radio"===d?h:n);l&&b.attr("tabindex",0);
break;case "range":m(d,b)&&b.attr("role","slider");if(a.config("ariaValue")){var p=!b.attr("aria-valuemin")&&(g.hasOwnProperty("min")||g.hasOwnProperty("ngMin")),q=!b.attr("aria-valuemax")&&(g.hasOwnProperty("max")||g.hasOwnProperty("ngMax")),r=!b.attr("aria-valuenow");p&&g.$observe("min",function(a){b.attr("aria-valuemin",a)});q&&g.$observe("max",function(a){b.attr("aria-valuemax",a)});r&&e.$watch(f,function(a){b.attr("aria-valuenow",a)})}l&&b.attr("tabindex",0)}!g.hasOwnProperty("ngRequired")&&
k.$validators.required&&c("aria-required","ariaRequired",b,!1)&&g.$observe("required",function(){b.attr("aria-required",!!g.required)});c("aria-invalid","ariaInvalid",b,!0)&&e.$watch(function(){return k.$invalid},function(a){b.attr("aria-invalid",!!a)})}}}}}]).directive("ngDisabled",["$aria",function(a){return a.$$watchExpr("ngDisabled","aria-disabled",f,!1)}]).directive("ngMessages",function(){return{restrict:"A",require:"?ngMessages",link:function(a,c,f,h){c.attr("aria-live")||c.attr("aria-live",
"assertive")}}}).directive("ngClick",["$aria","$parse",function(a,c){return{restrict:"A",compile:function(m,h){var p=c(h.ngClick,null,!0);return function(c,d,e){if(!l(d,f)&&(a.config("bindRoleForClick")&&!d.attr("role")&&d.attr("role","button"),a.config("tabindex")&&!d.attr("tabindex")&&d.attr("tabindex",0),a.config("bindKeypress")&&!e.ngKeypress))d.on("keypress",function(a){function d(){p(c,{$event:a})}var e=a.which||a.keyCode;32!==e&&13!==e||c.$apply(d)})}}}}]).directive("ngDblclick",["$aria",function(a){return function(c,
m,h){!a.config("tabindex")||m.attr("tabindex")||l(m,f)||m.attr("tabindex",0)}}])})(window,window.angular);
//# sourceMappingURL=angular-aria.min.js.map

/*!
 * ngCordova
 * v0.1.27-alpha
 * Copyright 2015 Drifty Co. http://drifty.com/
 * See LICENSE in this repository for license information
 */
!function(){angular.module("ngCordova",["ngCordova.plugins"]),angular.module("ngCordova.plugins.3dtouch",[]).factory("$cordova3DTouch",["$q",function(e){var n=[],r={},o=function(e){return function(n){for(var r in e)n.type===r&&e[r]()}};return{isAvailable:function(){var n=e.defer();return window.cordova?window.ThreeDeeTouch?window.ThreeDeeTouch.isAvailable(function(e){n.resolve(e)},function(e){n.reject(e)}):n.reject("Could not find 3D touch plugin"):n.reject("Not supported in browser"),n.promise},addQuickAction:function(t,i,a,c,u,s){var l=e.defer(),f={type:t,title:i,subtitle:u};return a&&(f.iconType=a),c&&(f.iconTemplate=c),this.isAvailable().then(function(){n.push(f),r[t]=s,window.ThreeDeeTouch.configureQuickActions(n),window.ThreeDeeTouch.onHomeIconPressed=o(r),l.resolve(n)},function(e){l.reject(e)}),l.promise},addQuickActionHandler:function(n,t){var i=e.defer();return this.isAvailable().then(function(){r[n]=t,window.ThreeDeeTouch.onHomeIconPressed=o(r),i.resolve(!0)},function(e){i.reject(e)}),i.promise},enableLinkPreview:function(){var n=e.defer();return this.isAvailable().then(function(){window.ThreeDeeTouch.enableLinkPreview(),n.resolve(!0)},function(e){n.reject(e)}),n.promise},addForceTouchHandler:function(n){var r=e.defer();return this.isAvailable().then(function(){window.ThreeDeeTouch.watchForceTouches(n),r.resolve(!0)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.actionSheet",[]).factory("$cordovaActionSheet",["$q","$window",function(e,n){return{show:function(r){var o=e.defer();return n.plugins.actionsheet.show(r,function(e){o.resolve(e)}),o.promise},hide:function(){return n.plugins.actionsheet.hide()}}}]),angular.module("ngCordova.plugins.adMob",[]).factory("$cordovaAdMob",["$q","$window",function(e,n){return{createBannerView:function(r){var o=e.defer();return n.plugins.AdMob.createBannerView(r,function(){o.resolve()},function(){o.reject()}),o.promise},createInterstitialView:function(r){var o=e.defer();return n.plugins.AdMob.createInterstitialView(r,function(){o.resolve()},function(){o.reject()}),o.promise},requestAd:function(r){var o=e.defer();return n.plugins.AdMob.requestAd(r,function(){o.resolve()},function(){o.reject()}),o.promise},showAd:function(r){var o=e.defer();return n.plugins.AdMob.showAd(r,function(){o.resolve()},function(){o.reject()}),o.promise},requestInterstitialAd:function(r){var o=e.defer();return n.plugins.AdMob.requestInterstitialAd(r,function(){o.resolve()},function(){o.reject()}),o.promise}}}]),angular.module("ngCordova.plugins.appAvailability",[]).factory("$cordovaAppAvailability",["$q",function(e){return{check:function(n){var r=e.defer();return appAvailability.check(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.appRate",[]).provider("$cordovaAppRate",[function(){this.setPreferences=function(e){e&&angular.isObject(e)&&(AppRate.preferences.useLanguage=e.language||null,AppRate.preferences.displayAppName=e.appName||"",AppRate.preferences.promptAgainForEachNewVersion=e.promptForNewVersion||!0,AppRate.preferences.openStoreInApp=e.openStoreInApp||!1,AppRate.preferences.usesUntilPrompt=e.usesUntilPrompt||3,AppRate.preferences.useCustomRateDialog=e.useCustomRateDialog||!1,AppRate.preferences.storeAppURL.ios=e.iosURL||null,AppRate.preferences.storeAppURL.android=e.androidURL||null,AppRate.preferences.storeAppURL.blackberry=e.blackberryURL||null,AppRate.preferences.storeAppURL.windows8=e.windowsURL||null)},this.setCustomLocale=function(e){var n={title:"Rate %@",message:"If you enjoy using %@, would you mind taking a moment to rate it? It won’t take more than a minute. Thanks for your support!",cancelButtonLabel:"No, Thanks",laterButtonLabel:"Remind Me Later",rateButtonLabel:"Rate It Now"};n=angular.extend(n,e),AppRate.preferences.customLocale=n},this.$get=["$q",function(e){return{promptForRating:function(n){var r=e.defer(),o=AppRate.promptForRating(n);return r.resolve(o),r.promise},navigateToAppStore:function(){var n=e.defer(),r=AppRate.navigateToAppStore();return n.resolve(r),n.promise},onButtonClicked:function(e){AppRate.preferences.callbacks.onButtonClicked=e.bind(this)},onRateDialogShow:function(e){AppRate.preferences.callbacks.onRateDialogShow=e.bind(this)}}}]}]),angular.module("ngCordova.plugins.appVersion",[]).factory("$cordovaAppVersion",["$q",function(e){return{getAppName:function(){var n=e.defer();return cordova.getAppVersion.getAppName(function(e){n.resolve(e)}),n.promise},getPackageName:function(){var n=e.defer();return cordova.getAppVersion.getPackageName(function(e){n.resolve(e)}),n.promise},getVersionNumber:function(){var n=e.defer();return cordova.getAppVersion.getVersionNumber(function(e){n.resolve(e)}),n.promise},getVersionCode:function(){var n=e.defer();return cordova.getAppVersion.getVersionCode(function(e){n.resolve(e)}),n.promise}}}]),angular.module("ngCordova.plugins.backgroundGeolocation",[]).factory("$cordovaBackgroundGeolocation",["$q","$window",function(e,n){return{init:function(){n.navigator.geolocation.getCurrentPosition(function(e){return e})},configure:function(r){this.init();var o=e.defer();return n.plugins.backgroundGeoLocation.configure(function(e){o.notify(e),n.plugins.backgroundGeoLocation.finish()},function(e){o.reject(e)},r),this.start(),o.promise},start:function(){var r=e.defer();return n.plugins.backgroundGeoLocation.start(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},stop:function(){var r=e.defer();return n.plugins.backgroundGeoLocation.stop(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.badge",[]).factory("$cordovaBadge",["$q",function(e){return{hasPermission:function(){var n=e.defer();return cordova.plugins.notification.badge.hasPermission(function(e){e?n.resolve(!0):n.reject("You do not have permission")}),n.promise},promptForPermission:function(){return cordova.plugins.notification.badge.promptForPermission()},set:function(n,r,o){var t=e.defer();return cordova.plugins.notification.badge.hasPermission(function(e){e?t.resolve(cordova.plugins.notification.badge.set(n,r,o)):t.reject("You do not have permission to set Badge")}),t.promise},get:function(){var n=e.defer();return cordova.plugins.notification.badge.hasPermission(function(e){e?cordova.plugins.notification.badge.get(function(e){n.resolve(e)}):n.reject("You do not have permission to get Badge")}),n.promise},clear:function(n,r){var o=e.defer();return cordova.plugins.notification.badge.hasPermission(function(e){e?o.resolve(cordova.plugins.notification.badge.clear(n,r)):o.reject("You do not have permission to clear Badge")}),o.promise},increase:function(n,r,o){var t=e.defer();return this.hasPermission().then(function(){t.resolve(cordova.plugins.notification.badge.increase(n,r,o))},function(){t.reject("You do not have permission to increase Badge")}),t.promise},decrease:function(n,r,o){var t=e.defer();return this.hasPermission().then(function(){t.resolve(cordova.plugins.notification.badge.decrease(n,r,o))},function(){t.reject("You do not have permission to decrease Badge")}),t.promise},configure:function(e){return cordova.plugins.notification.badge.configure(e)}}}]),angular.module("ngCordova.plugins.barcodeScanner",[]).factory("$cordovaBarcodeScanner",["$q",function(e){return{scan:function(n){var r=e.defer();return cordova.plugins.barcodeScanner.scan(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},encode:function(n,r){var o=e.defer();return n=n||"TEXT_TYPE",cordova.plugins.barcodeScanner.encode(n,r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise}}}]),angular.module("ngCordova.plugins.batteryStatus",[]).factory("$cordovaBatteryStatus",["$rootScope","$window","$timeout",function(e,n,r){var o=function(n){r(function(){e.$broadcast("$cordovaBatteryStatus:status",n)})},t=function(n){r(function(){e.$broadcast("$cordovaBatteryStatus:critical",n)})},i=function(n){r(function(){e.$broadcast("$cordovaBatteryStatus:low",n)})};return document.addEventListener("deviceready",function(){navigator.battery&&(n.addEventListener("batterystatus",o,!1),n.addEventListener("batterycritical",t,!1),n.addEventListener("batterylow",i,!1))},!1),!0}]).run(["$injector",function(e){e.get("$cordovaBatteryStatus")}]),angular.module("ngCordova.plugins.beacon",[]).factory("$cordovaBeacon",["$window","$rootScope","$timeout","$q",function(e,n,r,o){var t=null,i=null,a=null,c=null,u=null,s=null,l=null,f=null;return document.addEventListener("deviceready",function(){if(e.cordova&&e.cordova.plugins&&e.cordova.plugins.locationManager){var o=new e.cordova.plugins.locationManager.Delegate;o.didDetermineStateForRegion=function(e){r(function(){n.$broadcast("$cordovaBeacon:didDetermineStateForRegion",e)}),t&&t(e)},o.didStartMonitoringForRegion=function(e){r(function(){n.$broadcast("$cordovaBeacon:didStartMonitoringForRegion",e)}),i&&i(e)},o.didExitRegion=function(e){r(function(){n.$broadcast("$cordovaBeacon:didExitRegion",e)}),a&&a(e)},o.didEnterRegion=function(e){r(function(){n.$broadcast("$cordovaBeacon:didEnterRegion",e)}),c&&c(e)},o.didRangeBeaconsInRegion=function(e){r(function(){n.$broadcast("$cordovaBeacon:didRangeBeaconsInRegion",e)}),u&&u(e)},o.peripheralManagerDidStartAdvertising=function(e){r(function(){n.$broadcast("$cordovaBeacon:peripheralManagerDidStartAdvertising",e)}),s&&s(e)},o.peripheralManagerDidUpdateState=function(e){r(function(){n.$broadcast("$cordovaBeacon:peripheralManagerDidUpdateState",e)}),l&&l(e)},o.didChangeAuthorizationStatus=function(e){r(function(){n.$broadcast("$cordovaBeacon:didChangeAuthorizationStatus",e)}),f&&f(e)},e.cordova.plugins.locationManager.setDelegate(o)}},!1),{setCallbackDidDetermineStateForRegion:function(e){t=e},setCallbackDidStartMonitoringForRegion:function(e){i=e},setCallbackDidExitRegion:function(e){a=e},setCallbackDidEnterRegion:function(e){c=e},setCallbackDidRangeBeaconsInRegion:function(e){u=e},setCallbackPeripheralManagerDidStartAdvertising:function(e){s=e},setCallbackPeripheralManagerDidUpdateState:function(e){l=e},setCallbackDidChangeAuthorizationStatus:function(e){f=e},createBeaconRegion:function(n,r,o,t,i){return o=o||void 0,t=t||void 0,new e.cordova.plugins.locationManager.BeaconRegion(n,r,o,t,i)},isBluetoothEnabled:function(){return o.when(e.cordova.plugins.locationManager.isBluetoothEnabled())},enableBluetooth:function(){return o.when(e.cordova.plugins.locationManager.enableBluetooth())},disableBluetooth:function(){return o.when(e.cordova.plugins.locationManager.disableBluetooth())},startMonitoringForRegion:function(n){return o.when(e.cordova.plugins.locationManager.startMonitoringForRegion(n))},stopMonitoringForRegion:function(n){return o.when(e.cordova.plugins.locationManager.stopMonitoringForRegion(n))},requestStateForRegion:function(n){return o.when(e.cordova.plugins.locationManager.requestStateForRegion(n))},startRangingBeaconsInRegion:function(n){return o.when(e.cordova.plugins.locationManager.startRangingBeaconsInRegion(n))},stopRangingBeaconsInRegion:function(n){return o.when(e.cordova.plugins.locationManager.stopRangingBeaconsInRegion(n))},getAuthorizationStatus:function(){return o.when(e.cordova.plugins.locationManager.getAuthorizationStatus())},requestWhenInUseAuthorization:function(){return o.when(e.cordova.plugins.locationManager.requestWhenInUseAuthorization())},requestAlwaysAuthorization:function(){return o.when(e.cordova.plugins.locationManager.requestAlwaysAuthorization())},getMonitoredRegions:function(){return o.when(e.cordova.plugins.locationManager.getMonitoredRegions())},getRangedRegions:function(){return o.when(e.cordova.plugins.locationManager.getRangedRegions())},isRangingAvailable:function(){return o.when(e.cordova.plugins.locationManager.isRangingAvailable())},isMonitoringAvailableForClass:function(n){return o.when(e.cordova.plugins.locationManager.isMonitoringAvailableForClass(n))},startAdvertising:function(n,r){return o.when(e.cordova.plugins.locationManager.startAdvertising(n,r))},stopAdvertising:function(){return o.when(e.cordova.plugins.locationManager.stopAdvertising())},isAdvertisingAvailable:function(){return o.when(e.cordova.plugins.locationManager.isAdvertisingAvailable())},isAdvertising:function(){return o.when(e.cordova.plugins.locationManager.isAdvertising())},disableDebugLogs:function(){return o.when(e.cordova.plugins.locationManager.disableDebugLogs())},enableDebugNotifications:function(){return o.when(e.cordova.plugins.locationManager.enableDebugNotifications())},disableDebugNotifications:function(){return o.when(e.cordova.plugins.locationManager.disableDebugNotifications())},enableDebugLogs:function(){return o.when(e.cordova.plugins.locationManager.enableDebugLogs())},appendToDeviceLog:function(n){return o.when(e.cordova.plugins.locationManager.appendToDeviceLog(n))}}}]),angular.module("ngCordova.plugins.ble",[]).factory("$cordovaBLE",["$q","$timeout","$log",function(e,n,r){return{scan:function(r,o){var t=e.defer();return ble.startScan(r,function(e){t.notify(e)},function(e){t.reject(e)}),n(function(){ble.stopScan(function(){t.resolve()},function(e){t.reject(e)})},1e3*o),t.promise},startScan:function(e,n,r){return ble.startScan(e,n,r)},stopScan:function(){var n=e.defer();return ble.stopScan(function(){n.resolve()},function(e){n.reject(e)}),n.promise},connect:function(n){var r=e.defer();return ble.connect(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},disconnect:function(n){var r=e.defer();return ble.disconnect(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},read:function(n,r,o){var t=e.defer();return ble.read(n,r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},write:function(n,r,o,t){var i=e.defer();return ble.write(n,r,o,t,function(e){i.resolve(e)},function(e){i.reject(e)}),i.promise},writeWithoutResponse:function(n,r,o,t){var i=e.defer();return ble.writeWithoutResponse(n,r,o,t,function(e){i.resolve(e)},function(e){i.reject(e)}),i.promise},writeCommand:function(e,n,o,t){return r.warning("writeCommand is deprecated, use writeWithoutResponse"),this.writeWithoutResponse(e,n,o,t)},startNotification:function(e,n,r,o,t){return ble.startNotification(e,n,r,o,t)},stopNotification:function(n,r,o){var t=e.defer();return ble.stopNotification(n,r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},isConnected:function(n){var r=e.defer();return ble.isConnected(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},enable:function(){var n=e.defer();return ble.enable(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},isEnabled:function(){var n=e.defer();return ble.isEnabled(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise}}}]),angular.module("ngCordova.plugins.bluetoothSerial",[]).factory("$cordovaBluetoothSerial",["$q","$window",function(e,n){return{connect:function(r){var o=e.defer(),t=e.defer(),i=!1;return n.bluetoothSerial.connect(r,function(){i=!0,o.resolve(t)},function(e){i===!1&&t.reject(e),o.reject(e)}),o.promise},connectInsecure:function(r){var o=e.defer();return n.bluetoothSerial.connectInsecure(r,function(){o.resolve()},function(e){o.reject(e)}),o.promise},disconnect:function(){var r=e.defer();return n.bluetoothSerial.disconnect(function(){r.resolve()},function(e){r.reject(e)}),r.promise},list:function(){var r=e.defer();return n.bluetoothSerial.list(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},discoverUnpaired:function(){var r=e.defer();return n.bluetoothSerial.discoverUnpaired(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},setDeviceDiscoveredListener:function(){var r=e.defer();return n.bluetoothSerial.setDeviceDiscoveredListener(function(e){r.notify(e)}),r.promise},clearDeviceDiscoveredListener:function(){n.bluetoothSerial.clearDeviceDiscoveredListener()},showBluetoothSettings:function(){var r=e.defer();return n.bluetoothSerial.showBluetoothSettings(function(){r.resolve()},function(e){r.reject(e)}),r.promise},isEnabled:function(){var r=e.defer();return n.bluetoothSerial.isEnabled(function(){r.resolve()},function(){r.reject()}),r.promise},enable:function(){var r=e.defer();return n.bluetoothSerial.enable(function(){r.resolve()},function(){r.reject()}),r.promise},isConnected:function(){var r=e.defer();return n.bluetoothSerial.isConnected(function(){r.resolve()},function(){r.reject()}),r.promise},available:function(){var r=e.defer();return n.bluetoothSerial.available(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},read:function(){var r=e.defer();return n.bluetoothSerial.read(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},readUntil:function(r){var o=e.defer();return n.bluetoothSerial.readUntil(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},write:function(r){var o=e.defer();return n.bluetoothSerial.write(r,function(){o.resolve()},function(e){o.reject(e)}),o.promise},subscribe:function(r){var o=e.defer();return n.bluetoothSerial.subscribe(r,function(e){o.notify(e)},function(e){o.reject(e)}),o.promise},subscribeRawData:function(){var r=e.defer();return n.bluetoothSerial.subscribeRawData(function(e){r.notify(e)},function(e){r.reject(e)}),r.promise},unsubscribe:function(){var r=e.defer();return n.bluetoothSerial.unsubscribe(function(){r.resolve()},function(e){r.reject(e)}),r.promise},unsubscribeRawData:function(){var r=e.defer();return n.bluetoothSerial.unsubscribeRawData(function(){r.resolve()},function(e){r.reject(e)}),r.promise},clear:function(){var r=e.defer();return n.bluetoothSerial.clear(function(){r.resolve()},function(e){r.reject(e)}),r.promise},readRSSI:function(){var r=e.defer();return n.bluetoothSerial.readRSSI(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.brightness",[]).factory("$cordovaBrightness",["$q","$window",function(e,n){return{get:function(){var r=e.defer();return n.cordova?n.cordova.plugins.brightness.getBrightness(function(e){r.resolve(e)},function(e){r.reject(e)}):r.reject("Not supported without cordova.js"),r.promise},set:function(r){var o=e.defer();return n.cordova?n.cordova.plugins.brightness.setBrightness(r,function(e){o.resolve(e)},function(e){o.reject(e)}):o.reject("Not supported without cordova.js"),o.promise},setKeepScreenOn:function(r){var o=e.defer();return n.cordova?n.cordova.plugins.brightness.setKeepScreenOn(r,function(e){o.resolve(e)},function(e){o.reject(e)}):o.reject("Not supported without cordova.js"),o.promise}}}]),angular.module("ngCordova.plugins.calendar",[]).factory("$cordovaCalendar",["$q","$window",function(e,n){return{createCalendar:function(r){var o=e.defer(),t=n.plugins.calendar.getCreateCalendarOptions();return"string"==typeof r?t.calendarName=r:t=angular.extend(t,r),n.plugins.calendar.createCalendar(t,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},deleteCalendar:function(r){var o=e.defer();return n.plugins.calendar.deleteCalendar(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},createEvent:function(r){var o=e.defer(),t={title:null,location:null,notes:null,startDate:null,endDate:null};return t=angular.extend(t,r),n.plugins.calendar.createEvent(t.title,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},createEventWithOptions:function(r){var o=e.defer(),t=[],i=window.plugins.calendar.getCalendarOptions(),a={title:null,location:null,notes:null,startDate:null,endDate:null};t=Object.keys(a);for(var c in r)-1===t.indexOf(c)?i[c]=r[c]:a[c]=r[c];return n.plugins.calendar.createEventWithOptions(a.title,a.location,a.notes,new Date(a.startDate),new Date(a.endDate),i,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},createEventInteractively:function(r){var o=e.defer(),t={title:null,location:null,notes:null,startDate:null,endDate:null};return t=angular.extend(t,r),n.plugins.calendar.createEventInteractively(t.title,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},createEventInNamedCalendar:function(r){var o=e.defer(),t={title:null,location:null,notes:null,startDate:null,endDate:null,calendarName:null};return t=angular.extend(t,r),n.plugins.calendar.createEventInNamedCalendar(t.title,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),t.calendarName,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},findEvent:function(r){var o=e.defer(),t={title:null,location:null,notes:null,startDate:null,endDate:null};return t=angular.extend(t,r),n.plugins.calendar.findEvent(t.title,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},listEventsInRange:function(r,o){var t=e.defer();return n.plugins.calendar.listEventsInRange(r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},listCalendars:function(){var r=e.defer();return n.plugins.calendar.listCalendars(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},findAllEventsInNamedCalendar:function(r){var o=e.defer();return n.plugins.calendar.findAllEventsInNamedCalendar(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},modifyEvent:function(r){var o=e.defer(),t={title:null,location:null,notes:null,startDate:null,endDate:null,newTitle:null,newLocation:null,newNotes:null,newStartDate:null,newEndDate:null};return t=angular.extend(t,r),n.plugins.calendar.modifyEvent(t.title,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),t.newTitle,t.newLocation,t.newNotes,new Date(t.newStartDate),new Date(t.newEndDate),function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},deleteEvent:function(r){var o=e.defer(),t={newTitle:null,location:null,notes:null,startDate:null,endDate:null};return t=angular.extend(t,r),n.plugins.calendar.deleteEvent(t.newTitle,t.location,t.notes,new Date(t.startDate),new Date(t.endDate),function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise}}}]),angular.module("ngCordova.plugins.camera",[]).factory("$cordovaCamera",["$q",function(e){return{getPicture:function(n){var r=e.defer();return navigator.camera?(navigator.camera.getPicture(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise):(r.resolve(null),r.promise)},cleanup:function(){var n=e.defer();return navigator.camera.cleanup(function(){n.resolve()},function(e){n.reject(e)}),n.promise}}}]),angular.module("ngCordova.plugins.capture",[]).factory("$cordovaCapture",["$q",function(e){return{captureAudio:function(n){var r=e.defer();return navigator.device.capture?(navigator.device.capture.captureAudio(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise):(r.resolve(null),r.promise)},captureImage:function(n){var r=e.defer();return navigator.device.capture?(navigator.device.capture.captureImage(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise):(r.resolve(null),r.promise)},captureVideo:function(n){var r=e.defer();return navigator.device.capture?(navigator.device.capture.captureVideo(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise):(r.resolve(null),r.promise)}}}]),angular.module("ngCordova.plugins.cardIO",[]).provider("$cordovaNgCardIO",[function(){var e=["card_type","redacted_card_number","card_number","expiry_month","expiry_year","short_expiry_year","cvv","zip"],n={expiry:!0,cvv:!0,zip:!1,suppressManual:!1,suppressConfirm:!1,hideLogo:!0};this.setCardIOResponseFields=function(n){n&&angular.isArray(n)&&(e=n)},this.setScanerConfig=function(e){e&&angular.isObject(e)&&(n.expiry=e.expiry||!0,n.cvv=e.cvv||!0,n.zip=e.zip||!1,n.suppressManual=e.suppressManual||!1,n.suppressConfirm=e.suppressConfirm||!1,n.hideLogo=e.hideLogo||!0)},this.$get=["$q",function(r){return{scanCard:function(){var o=r.defer();return CardIO.scan(n,function(n){if(null===n)o.reject(null);else{for(var r={},t=0,i=e.length;i>t;t++){var a=e[t];"short_expiry_year"===a?r[a]=String(n.expiry_year).substr(2,2)||"":r[a]=n[a]||""}o.resolve(r)}},function(){o.reject(null)}),o.promise}}}]}]),angular.module("ngCordova.plugins.clipboard",[]).factory("$cordovaClipboard",["$q","$window",function(e,n){return{copy:function(r){var o=e.defer();return n.cordova.plugins.clipboard.copy(r,function(){o.resolve()},function(){o.reject()}),o.promise},paste:function(){var r=e.defer();return n.cordova.plugins.clipboard.paste(function(e){r.resolve(e)},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.contacts",[]).factory("$cordovaContacts",["$q",function(e){return{save:function(n){var r=e.defer(),o=navigator.contacts.create(n);return o.save(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},remove:function(n){var r=e.defer(),o=navigator.contacts.create(n);return o.remove(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},clone:function(e){var n=navigator.contacts.create(e);return n.clone(e)},find:function(n){var r=e.defer(),o=n.fields||["id","displayName"];return delete n.fields,0===Object.keys(n).length?navigator.contacts.find(o,function(e){r.resolve(e)},function(e){r.reject(e)}):navigator.contacts.find(o,function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},pickContact:function(){var n=e.defer();return navigator.contacts.pickContact(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise}}}]),angular.module("ngCordova.plugins.datePicker",[]).factory("$cordovaDatePicker",["$window","$q",function(e,n){return{show:function(r){var o=n.defer();return r=r||{date:new Date,mode:"date"},e.datePicker.show(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise}}}]),angular.module("ngCordova.plugins.device",[]).factory("$cordovaDevice",[function(){return{getDevice:function(){return device},getCordova:function(){return device.cordova},getModel:function(){return device.model},getName:function(){return device.name},getPlatform:function(){return device.platform},getUUID:function(){return device.uuid},getVersion:function(){return device.version},getManufacturer:function(){return device.manufacturer}}}]),angular.module("ngCordova.plugins.deviceMotion",[]).factory("$cordovaDeviceMotion",["$q",function(e){return{getCurrentAcceleration:function(){var n=e.defer();return angular.isUndefined(navigator.accelerometer)||!angular.isFunction(navigator.accelerometer.getCurrentAcceleration)?(n.reject("Device do not support watchAcceleration"),n.promise):(navigator.accelerometer.getCurrentAcceleration(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise)},watchAcceleration:function(n){var r=e.defer();if(angular.isUndefined(navigator.accelerometer)||!angular.isFunction(navigator.accelerometer.watchAcceleration))return r.reject("Device do not support watchAcceleration"),r.promise;var o=navigator.accelerometer.watchAcceleration(function(e){r.notify(e)},function(e){r.reject(e)},n);return r.promise.cancel=function(){navigator.accelerometer.clearWatch(o)},r.promise.clearWatch=function(e){navigator.accelerometer.clearWatch(e||o)},r.promise.watchID=o,r.promise},clearWatch:function(e){return navigator.accelerometer.clearWatch(e)}}}]),angular.module("ngCordova.plugins.deviceOrientation",[]).factory("$cordovaDeviceOrientation",["$q",function(e){var n={frequency:3e3};return{getCurrentHeading:function(){var n=e.defer();return navigator.compass?(navigator.compass.getCurrentHeading(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise):(n.reject("No compass on Device"),n.promise)},watchHeading:function(r){var o=e.defer();if(!navigator.compass)return o.reject("No compass on Device"),o.promise;var t=angular.extend(n,r),i=navigator.compass.watchHeading(function(e){o.notify(e)},function(e){o.reject(e)},t);return o.promise.cancel=function(){navigator.compass.clearWatch(i)},o.promise.clearWatch=function(e){navigator.compass.clearWatch(e||i)},o.promise.watchID=i,o.promise},clearWatch:function(e){return navigator.compass.clearWatch(e)}}}]),angular.module("ngCordova.plugins.dialogs",[]).factory("$cordovaDialogs",["$q","$window",function(e,n){return{alert:function(r,o,t){var i=e.defer();return n.navigator.notification?navigator.notification.alert(r,function(){i.resolve()},o,t):(n.alert(r),i.resolve()),i.promise},confirm:function(r,o,t){var i=e.defer();return n.navigator.notification?navigator.notification.confirm(r,function(e){i.resolve(e)},o,t):n.confirm(r)?i.resolve(1):i.resolve(2),i.promise},prompt:function(r,o,t,i){var a=e.defer();if(n.navigator.notification)navigator.notification.prompt(r,function(e){a.resolve(e)},o,t,i);else{var c=n.prompt(r,i);null!==c?a.resolve({input1:c,buttonIndex:1}):a.resolve({input1:c,buttonIndex:2})}return a.promise},beep:function(e){return navigator.notification.beep(e)},activityStart:function(n,r){var o=e.defer();return"android"===cordova.platformId?(navigator.notification.activityStart(r,n),o.resolve()):o.reject(n,r),o.promise},activityStop:function(){var n=e.defer();return"android"===cordova.platformId?(navigator.notification.activityStop(),n.resolve()):n.reject(),n.promise},progressStart:function(n,r){var o=e.defer();return"android"===cordova.platformId?(navigator.notification.progressStart(r,n),o.resolve()):o.reject(n,r),o.promise},progressStop:function(){var n=e.defer();return"android"===cordova.platformId?(navigator.notification.progressStop(),n.resolve()):n.reject(),n.promise},progressValue:function(n){var r=e.defer();return"android"===cordova.platformId?(navigator.notification.progressValue(n),r.resolve()):r.reject(n),r.promise}}}]),angular.module("ngCordova.plugins.emailComposer",[]).factory("$cordovaEmailComposer",["$q",function(e){return{isAvailable:function(){var n=e.defer();return cordova.plugins.email.isAvailable(function(e){e?n.resolve():n.reject()}),n.promise},open:function(n){var r=e.defer();return cordova.plugins.email.open(n,function(){r.reject()}),r.promise},addAlias:function(e,n){cordova.plugins.email.addAlias(e,n)}}}]),angular.module("ngCordova.plugins.facebook",[]).provider("$cordovaFacebook",[function(){this.browserInit=function(e,n){this.appID=e,this.appVersion=n||"v2.0",facebookConnectPlugin.browserInit(this.appID,this.appVersion)},this.$get=["$q",function(e){return{login:function(n){var r=e.defer();return facebookConnectPlugin.login(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},showDialog:function(n){var r=e.defer();return facebookConnectPlugin.showDialog(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},api:function(n,r){var o=e.defer();return facebookConnectPlugin.api(n,r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},getAccessToken:function(){var n=e.defer();return facebookConnectPlugin.getAccessToken(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},getLoginStatus:function(){var n=e.defer();return facebookConnectPlugin.getLoginStatus(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},logout:function(){var n=e.defer();return facebookConnectPlugin.logout(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise}}}]}]),angular.module("ngCordova.plugins.facebookAds",[]).factory("$cordovaFacebookAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.FacebookAds.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.FacebookAds.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.FacebookAds.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.FacebookAds.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.FacebookAds.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.FacebookAds.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.FacebookAds.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.FacebookAds.showInterstitial(function(){r.resolve()},function(){
r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.file",[]).constant("$cordovaFileError",{1:"NOT_FOUND_ERR",2:"SECURITY_ERR",3:"ABORT_ERR",4:"NOT_READABLE_ERR",5:"ENCODING_ERR",6:"NO_MODIFICATION_ALLOWED_ERR",7:"INVALID_STATE_ERR",8:"SYNTAX_ERR",9:"INVALID_MODIFICATION_ERR",10:"QUOTA_EXCEEDED_ERR",11:"TYPE_MISMATCH_ERR",12:"PATH_EXISTS_ERR"}).provider("$cordovaFile",[function(){this.$get=["$q","$window","$cordovaFileError",function(e,n,r){return{getFreeDiskSpace:function(){var n=e.defer();return cordova.exec(function(e){n.resolve(e)},function(e){n.reject(e)},"File","getFreeDiskSpace",[]),n.promise},checkDir:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("directory cannot start with /");try{var a=o+t;n.resolveLocalFileSystemURL(a,function(e){e.isDirectory===!0?i.resolve(e):i.reject({code:13,message:"input is not a directory"})},function(e){e.message=r[e.code],i.reject(e)})}catch(c){c.message=r[c.code],i.reject(c)}return i.promise},checkFile:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("directory cannot start with /");try{var a=o+t;n.resolveLocalFileSystemURL(a,function(e){e.isFile===!0?i.resolve(e):i.reject({code:13,message:"input is not a file"})},function(e){e.message=r[e.code],i.reject(e)})}catch(c){c.message=r[c.code],i.reject(c)}return i.promise},createDir:function(o,t,i){var a=e.defer();/^\//.test(t)&&a.reject("directory cannot start with /"),i=i?!1:!0;var c={create:!0,exclusive:i};try{n.resolveLocalFileSystemURL(o,function(e){e.getDirectory(t,c,function(e){a.resolve(e)},function(e){e.message=r[e.code],a.reject(e)})},function(e){e.message=r[e.code],a.reject(e)})}catch(u){u.message=r[u.code],a.reject(u)}return a.promise},createFile:function(o,t,i){var a=e.defer();/^\//.test(t)&&a.reject("file-name cannot start with /"),i=i?!1:!0;var c={create:!0,exclusive:i};try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,c,function(e){a.resolve(e)},function(e){e.message=r[e.code],a.reject(e)})},function(e){e.message=r[e.code],a.reject(e)})}catch(u){u.message=r[u.code],a.reject(u)}return a.promise},removeDir:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getDirectory(t,{create:!1},function(e){e.remove(function(){i.resolve({success:!0,fileRemoved:e})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},removeFile:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.remove(function(){i.resolve({success:!0,fileRemoved:e})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},removeRecursively:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getDirectory(t,{create:!1},function(e){e.removeRecursively(function(){i.resolve({success:!0,fileRemoved:e})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},writeFile:function(o,t,i,a){var c=e.defer();/^\//.test(t)&&c.reject("file-name cannot start with /"),a=a?!1:!0;var u={create:!0,exclusive:a};try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,u,function(e){e.createWriter(function(e){u.append===!0&&e.seek(e.length),u.truncate&&e.truncate(u.truncate),e.onwriteend=function(e){this.error?c.reject(this.error):c.resolve(e)},e.write(i),c.promise.abort=function(){e.abort()}})},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})}catch(s){s.message=r[s.code],c.reject(s)}return c.promise},writeExistingFile:function(o,t,i){var a=e.defer();/^\//.test(t)&&a.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.createWriter(function(e){e.seek(e.length),e.onwriteend=function(e){this.error?a.reject(this.error):a.resolve(e)},e.write(i),a.promise.abort=function(){e.abort()}})},function(e){e.message=r[e.code],a.reject(e)})},function(e){e.message=r[e.code],a.reject(e)})}catch(c){c.message=r[c.code],a.reject(c)}return a.promise},readAsText:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.file(function(e){var n=new FileReader;n.onloadend=function(e){void 0!==e.target.result||null!==e.target.result?i.resolve(e.target.result):void 0!==e.target.error||null!==e.target.error?i.reject(e.target.error):i.reject({code:null,message:"READER_ONLOADEND_ERR"})},n.readAsText(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},readAsDataURL:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.file(function(e){var n=new FileReader;n.onloadend=function(e){void 0!==e.target.result||null!==e.target.result?i.resolve(e.target.result):void 0!==e.target.error||null!==e.target.error?i.reject(e.target.error):i.reject({code:null,message:"READER_ONLOADEND_ERR"})},n.readAsDataURL(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},readAsBinaryString:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.file(function(e){var n=new FileReader;n.onloadend=function(e){void 0!==e.target.result||null!==e.target.result?i.resolve(e.target.result):void 0!==e.target.error||null!==e.target.error?i.reject(e.target.error):i.reject({code:null,message:"READER_ONLOADEND_ERR"})},n.readAsBinaryString(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},readAsArrayBuffer:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1},function(e){e.file(function(e){var n=new FileReader;n.onloadend=function(e){void 0!==e.target.result||null!==e.target.result?i.resolve(e.target.result):void 0!==e.target.error||null!==e.target.error?i.reject(e.target.error):i.reject({code:null,message:"READER_ONLOADEND_ERR"})},n.readAsArrayBuffer(e)})},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(a){a.message=r[a.code],i.reject(a)}return i.promise},moveFile:function(r,o,t,i){var a=e.defer();i=i||o,(/^\//.test(o)||/^\//.test(i))&&a.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(r,function(e){e.getFile(o,{create:!1},function(e){n.resolveLocalFileSystemURL(t,function(n){e.moveTo(n,i,function(e){a.resolve(e)},function(e){a.reject(e)})},function(e){a.reject(e)})},function(e){a.reject(e)})},function(e){a.reject(e)})}catch(c){a.reject(c)}return a.promise},moveDir:function(r,o,t,i){var a=e.defer();i=i||o,(/^\//.test(o)||/^\//.test(i))&&a.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(r,function(e){e.getDirectory(o,{create:!1},function(e){n.resolveLocalFileSystemURL(t,function(n){e.moveTo(n,i,function(e){a.resolve(e)},function(e){a.reject(e)})},function(e){a.reject(e)})},function(e){a.reject(e)})},function(e){a.reject(e)})}catch(c){a.reject(c)}return a.promise},copyDir:function(o,t,i,a){var c=e.defer();a=a||t,(/^\//.test(t)||/^\//.test(a))&&c.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getDirectory(t,{create:!1,exclusive:!1},function(e){n.resolveLocalFileSystemURL(i,function(n){e.copyTo(n,a,function(e){c.resolve(e)},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})}catch(u){u.message=r[u.code],c.reject(u)}return c.promise},copyFile:function(o,t,i,a){var c=e.defer();a=a||t,/^\//.test(t)&&c.reject("file-name cannot start with /");try{n.resolveLocalFileSystemURL(o,function(e){e.getFile(t,{create:!1,exclusive:!1},function(e){n.resolveLocalFileSystemURL(i,function(n){e.copyTo(n,a,function(e){c.resolve(e)},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})},function(e){e.message=r[e.code],c.reject(e)})}catch(u){u.message=r[u.code],c.reject(u)}return c.promise},readFileMetadata:function(o,t){var i=e.defer();/^\//.test(t)&&i.reject("directory cannot start with /");try{var a=o+t;n.resolveLocalFileSystemURL(a,function(e){e.file(function(e){i.resolve(e)},function(e){e.message=r[e.code],i.reject(e)})},function(e){e.message=r[e.code],i.reject(e)})}catch(c){c.message=r[c.code],i.reject(c)}return i.promise}}}]}]),angular.module("ngCordova.plugins.fileOpener2",[]).factory("$cordovaFileOpener2",["$q",function(e){return{open:function(n,r){var o=e.defer();return cordova.plugins.fileOpener2.open(n,r,{error:function(e){o.reject(e)},success:function(){o.resolve()}}),o.promise},uninstall:function(n){var r=e.defer();return cordova.plugins.fileOpener2.uninstall(n,{error:function(e){r.reject(e)},success:function(){r.resolve()}}),r.promise},appIsInstalled:function(n){var r=e.defer();return cordova.plugins.fileOpener2.appIsInstalled(n,{success:function(e){r.resolve(e)}}),r.promise}}}]),angular.module("ngCordova.plugins.fileTransfer",[]).factory("$cordovaFileTransfer",["$q","$timeout",function(e,n){return{download:function(r,o,t,i){var a=e.defer(),c=new FileTransfer,u=t&&t.encodeURI===!1?r:encodeURI(r);return t&&void 0!==t.timeout&&null!==t.timeout&&(n(function(){c.abort()},t.timeout),t.timeout=null),c.onprogress=function(e){a.notify(e)},a.promise.abort=function(){c.abort()},c.download(u,o,a.resolve,a.reject,i,t),a.promise},upload:function(r,o,t,i){var a=e.defer(),c=new FileTransfer,u=t&&t.encodeURI===!1?r:encodeURI(r);return t&&void 0!==t.timeout&&null!==t.timeout&&(n(function(){c.abort()},t.timeout),t.timeout=null),c.onprogress=function(e){a.notify(e)},a.promise.abort=function(){c.abort()},c.upload(o,u,a.resolve,a.reject,t,i),a.promise}}}]),angular.module("ngCordova.plugins.flashlight",[]).factory("$cordovaFlashlight",["$q","$window",function(e,n){return{available:function(){var r=e.defer();return n.plugins.flashlight.available(function(e){r.resolve(e)}),r.promise},switchOn:function(){var r=e.defer();return n.plugins.flashlight.switchOn(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},switchOff:function(){var r=e.defer();return n.plugins.flashlight.switchOff(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},toggle:function(){var r=e.defer();return n.plugins.flashlight.toggle(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.flurryAds",[]).factory("$cordovaFlurryAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.FlurryAds.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.FlurryAds.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.FlurryAds.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.FlurryAds.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.FlurryAds.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.FlurryAds.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.FlurryAds.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.FlurryAds.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.ga",[]).factory("$cordovaGA",["$q","$window",function(e,n){return{init:function(r,o){var t=e.defer();return o=o>=0?o:10,n.plugins.gaPlugin.init(function(e){t.resolve(e)},function(e){t.reject(e)},r,o),t.promise},trackEvent:function(r,o,t,i,a,c){var u=e.defer();return n.plugins.gaPlugin.trackEvent(function(e){u.resolve(e)},function(e){u.reject(e)},t,i,a,c),u.promise},trackPage:function(r,o,t){var i=e.defer();return n.plugins.gaPlugin.trackPage(function(e){i.resolve(e)},function(e){i.reject(e)},t),i.promise},setVariable:function(r,o,t,i){var a=e.defer();return n.plugins.gaPlugin.setVariable(function(e){a.resolve(e)},function(e){a.reject(e)},t,i),a.promise},exit:function(){var r=e.defer();return n.plugins.gaPlugin.exit(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.geolocation",[]).factory("$cordovaGeolocation",["$q",function(e){return{getCurrentPosition:function(n){var r=e.defer();return navigator.geolocation.getCurrentPosition(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},watchPosition:function(n){var r=e.defer(),o=navigator.geolocation.watchPosition(function(e){r.notify(e)},function(e){r.reject(e)},n);return r.promise.cancel=function(){navigator.geolocation.clearWatch(o)},r.promise.clearWatch=function(e){navigator.geolocation.clearWatch(e||o)},r.promise.watchID=o,r.promise},clearWatch:function(e){return navigator.geolocation.clearWatch(e)}}}]),angular.module("ngCordova.plugins.globalization",[]).factory("$cordovaGlobalization",["$q",function(e){return{getPreferredLanguage:function(){var n=e.defer();return navigator.globalization.getPreferredLanguage(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},getLocaleName:function(){var n=e.defer();return navigator.globalization.getLocaleName(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},getFirstDayOfWeek:function(){var n=e.defer();return navigator.globalization.getFirstDayOfWeek(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},dateToString:function(n,r){var o=e.defer();return navigator.globalization.dateToString(n,function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise},stringToDate:function(n,r){var o=e.defer();return navigator.globalization.stringToDate(n,function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise},getDatePattern:function(n){var r=e.defer();return navigator.globalization.getDatePattern(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},getDateNames:function(n){var r=e.defer();return navigator.globalization.getDateNames(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},isDayLightSavingsTime:function(n){var r=e.defer();return navigator.globalization.isDayLightSavingsTime(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},numberToString:function(n,r){var o=e.defer();return navigator.globalization.numberToString(n,function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise},stringToNumber:function(n,r){var o=e.defer();return navigator.globalization.stringToNumber(n,function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise},getNumberPattern:function(n){var r=e.defer();return navigator.globalization.getNumberPattern(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},getCurrencyPattern:function(n){var r=e.defer();return navigator.globalization.getCurrencyPattern(n,function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.googleAds",[]).factory("$cordovaGoogleAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.AdMob.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.AdMob.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.AdMob.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.AdMob.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.AdMob.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.AdMob.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.AdMob.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.AdMob.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.googleAnalytics",[]).factory("$cordovaGoogleAnalytics",["$q","$window",function(e,n){return{startTrackerWithId:function(r){var o=e.defer();return n.analytics.startTrackerWithId(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},setUserId:function(r){var o=e.defer();return n.analytics.setUserId(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},debugMode:function(){var r=e.defer();return n.analytics.debugMode(function(e){r.resolve(e)},function(){r.reject()}),r.promise},trackView:function(r){var o=e.defer();return n.analytics.trackView(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},addCustomDimension:function(r,o){var t=e.defer(),i=parseInt(r,10);return isNaN(i)&&t.reject('Parameter "key" must be an integer.'),n.analytics.addCustomDimension(i,o,function(){t.resolve()},function(e){t.reject(e)}),t.promise},trackEvent:function(r,o,t,i){var a=e.defer();return n.analytics.trackEvent(r,o,t,i,function(e){a.resolve(e)},function(e){a.reject(e)}),a.promise},trackException:function(r,o){var t=e.defer();return n.analytics.trackException(r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},trackTiming:function(r,o,t,i){var a=e.defer();return n.analytics.trackTiming(r,o,t,i,function(e){a.resolve(e)},function(e){a.reject(e)}),a.promise},addTransaction:function(r,o,t,i,a,c){var u=e.defer();return n.analytics.addTransaction(r,o,t,i,a,c,function(e){u.resolve(e)},function(e){u.reject(e)}),u.promise},addTransactionItem:function(r,o,t,i,a,c,u){var s=e.defer();return n.analytics.addTransactionItem(r,o,t,i,a,c,u,function(e){s.resolve(e)},function(e){s.reject(e)}),s.promise}}}]),angular.module("ngCordova.plugins.googleMap",[]).factory("$cordovaGoogleMap",["$q","$window",function(e,n){var r=null;return{getMap:function(o){var t=e.defer();if(n.plugin.google.maps){var i=document.getElementById("map_canvas");r=n.plugin.google.maps.Map.getMap(o),r.setDiv(i),t.resolve(r)}else t.reject(null);return t.promise},isMapLoaded:function(){return!!r},addMarker:function(n){var o=e.defer();return r.addMarker(n,function(e){o.resolve(e)}),o.promise},getMapTypeIds:function(){return n.plugin.google.maps.mapTypeId},setVisible:function(n){var o=e.defer();return r.setVisible(n),o.promise},cleanup:function(){r=null}}}]),angular.module("ngCordova.plugins.googlePlayGame",[]).factory("$cordovaGooglePlayGame",["$q",function(e){return{auth:function(){var n=e.defer();return googleplaygame.auth(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise},signout:function(){var n=e.defer();return googleplaygame.signout(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise},isSignedIn:function(){var n=e.defer();return googleplaygame.isSignedIn(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise},showPlayer:function(){var n=e.defer();return googleplaygame.showPlayer(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise},submitScore:function(n){var r=e.defer();return googleplaygame.submitScore(n,function(e){return r.resolve(e)},function(e){return r.reject(e)}),r.promise},showAllLeaderboards:function(){var n=e.defer();return googleplaygame.showAllLeaderboards(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise},showLeaderboard:function(n){var r=e.defer();return googleplaygame.showLeaderboard(n,function(e){return r.resolve(e)},function(e){return r.reject(e)}),r.promise},unlockAchievement:function(n){var r=e.defer();return googleplaygame.unlockAchievement(n,function(e){return r.resolve(e)},function(e){return r.reject(e)}),r.promise},incrementAchievement:function(n){var r=e.defer();return googleplaygame.incrementAchievement(n,function(e){return r.resolve(e)},function(e){return r.reject(e)}),r.promise},showAchievements:function(){var n=e.defer();return googleplaygame.showAchievements(function(e){return n.resolve(e)},function(e){return n.reject(e)}),n.promise}}}]),angular.module("ngCordova.plugins.googlePlus",[]).factory("$cordovaGooglePlus",["$q","$window",function(e,n){return{login:function(r){var o=e.defer();return void 0===r&&(r={}),n.plugins.googleplus.login({iOSApiKey:r},function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},silentLogin:function(r){var o=e.defer();return void 0===r&&(r={}),n.plugins.googleplus.trySilentLogin({iOSApiKey:r},function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},logout:function(){var r=e.defer();n.plugins.googleplus.logout(function(e){r.resolve(e)})},disconnect:function(){var r=e.defer();n.plugins.googleplus.disconnect(function(e){r.resolve(e)})},isAvailable:function(){var r=e.defer();return n.plugins.googleplus.isAvailable(function(e){e?r.resolve(e):r.reject(e)}),r.promise}}}]),angular.module("ngCordova.plugins.healthKit",[]).factory("$cordovaHealthKit",["$q","$window",function(e,n){return{isAvailable:function(){var r=e.defer();return n.plugins.healthkit.available(function(e){r.resolve(e)},function(e){r.reject(e)}),r.promise},checkAuthStatus:function(r){var o=e.defer();return r=r||"HKQuantityTypeIdentifierHeight",n.plugins.healthkit.checkAuthStatus({type:r},function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},requestAuthorization:function(r,o){var t=e.defer();return r=r||["HKCharacteristicTypeIdentifierDateOfBirth","HKQuantityTypeIdentifierActiveEnergyBurned","HKQuantityTypeIdentifierHeight"],o=o||["HKQuantityTypeIdentifierActiveEnergyBurned","HKQuantityTypeIdentifierHeight","HKQuantityTypeIdentifierDistanceCycling"],n.plugins.healthkit.requestAuthorization({readTypes:r,writeTypes:o},function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},readDateOfBirth:function(){var r=e.defer();return n.plugins.healthkit.readDateOfBirth(function(e){r.resolve(e)},function(e){r.resolve(e)}),r.promise},readGender:function(){var r=e.defer();return n.plugins.healthkit.readGender(function(e){r.resolve(e)},function(e){r.resolve(e)}),r.promise},saveWeight:function(r,o,t){var i=e.defer();return n.plugins.healthkit.saveWeight({unit:o||"lb",amount:r,date:t||new Date},function(e){i.resolve(e)},function(e){i.resolve(e)}),i.promise},readWeight:function(r){var o=e.defer();return n.plugins.healthkit.readWeight({unit:r||"lb"},function(e){o.resolve(e)},function(e){o.resolve(e)}),o.promise},saveHeight:function(r,o,t){var i=e.defer();return n.plugins.healthkit.saveHeight({unit:o||"in",amount:r,date:t||new Date},function(e){i.resolve(e)},function(e){i.resolve(e)}),i.promise},readHeight:function(r){var o=e.defer();return n.plugins.healthkit.readHeight({unit:r||"in"},function(e){o.resolve(e)},function(e){o.resolve(e)}),o.promise},findWorkouts:function(){var r=e.defer();return n.plugins.healthkit.findWorkouts({},function(e){r.resolve(e)},function(e){r.resolve(e)}),r.promise},saveWorkout:function(r){var o=e.defer();return n.plugins.healthkit.saveWorkout(r,function(e){o.resolve(e)},function(e){o.resolve(e)}),o.promise},querySampleType:function(r){var o=e.defer();return n.plugins.healthkit.querySampleType(r,function(e){o.resolve(e)},function(e){o.resolve(e)}),o.promise}}}]),angular.module("ngCordova.plugins.httpd",[]).factory("$cordovaHttpd",["$q",function(e){return{startServer:function(n){var r=e.defer();return cordova.plugins.CorHttpd.startServer(n,function(){r.resolve()},function(){r.reject()}),r.promise},stopServer:function(){var n=e.defer();return cordova.plugins.CorHttpd.stopServer(function(){n.resolve()},function(){n.reject()}),n.promise},getURL:function(){var n=e.defer();return cordova.plugins.CorHttpd.getURL(function(e){n.resolve(e)},function(){n.reject()}),n.promise},getLocalPath:function(){var n=e.defer();return cordova.plugins.CorHttpd.getLocalPath(function(e){n.resolve(e)},function(){n.reject()}),n.promise}}}]),angular.module("ngCordova.plugins.iAd",[]).factory("$cordovaiAd",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.iAd.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.iAd.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.iAd.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.iAd.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.iAd.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.iAd.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.iAd.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.iAd.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.imagePicker",[]).factory("$cordovaImagePicker",["$q","$window",function(e,n){return{getPictures:function(r){var o=e.defer();return n.imagePicker.getPictures(function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise}}}]),angular.module("ngCordova.plugins.inAppBrowser",[]).provider("$cordovaInAppBrowser",[function(){var e,n=this.defaultOptions={};this.setDefaultOptions=function(e){n=angular.extend(n,e)},this.$get=["$rootScope","$q","$window","$timeout",function(r,o,t,i){return{open:function(a,c,u){var s=o.defer();if(u&&!angular.isObject(u))return s.reject("options must be an object"),s.promise;var l=angular.extend({},n,u),f=[];angular.forEach(l,function(e,n){f.push(n+"="+e)});var d=f.join();return e=t.open(a,c,d),e.addEventListener("loadstart",function(e){i(function(){r.$broadcast("$cordovaInAppBrowser:loadstart",e)})},!1),e.addEventListener("loadstop",function(e){s.resolve(e),i(function(){r.$broadcast("$cordovaInAppBrowser:loadstop",e)})},!1),e.addEventListener("loaderror",function(e){s.reject(e),i(function(){r.$broadcast("$cordovaInAppBrowser:loaderror",e)})},!1),e.addEventListener("exit",function(e){i(function(){r.$broadcast("$cordovaInAppBrowser:exit",e)})},!1),s.promise},close:function(){e.close(),e=null},show:function(){e.show()},executeScript:function(n){var r=o.defer();return e.executeScript(n,function(e){r.resolve(e)}),r.promise},insertCSS:function(n){var r=o.defer();return e.insertCSS(n,function(e){r.resolve(e)}),r.promise}}}]}]),angular.module("ngCordova.plugins.insomnia",[]).factory("$cordovaInsomnia",["$window",function(e){return{keepAwake:function(){return e.plugins.insomnia.keepAwake()},allowSleepAgain:function(){return e.plugins.insomnia.allowSleepAgain()}}}]),angular.module("ngCordova.plugins.instagram",[]).factory("$cordovaInstagram",["$q",function(e){return{share:function(n){var r=e.defer();return window.Instagram?(Instagram.share(n.image,n.caption,function(e){e?r.reject(e):r.resolve(!0)}),r.promise):(console.error("Tried to call Instagram.share but the Instagram plugin isn't installed!"),r.resolve(null),r.promise)},isInstalled:function(){var n=e.defer();return window.Instagram?(Instagram.isInstalled(function(e,r){e?n.reject(e):n.resolve(r)}),n.promise):(console.error("Tried to call Instagram.isInstalled but the Instagram plugin isn't installed!"),n.resolve(null),n.promise)}}}]),angular.module("ngCordova.plugins.keyboard",[]).factory("$cordovaKeyboard",["$rootScope",function(e){var n=function(){e.$evalAsync(function(){e.$broadcast("$cordovaKeyboard:show")})},r=function(){e.$evalAsync(function(){e.$broadcast("$cordovaKeyboard:hide")})};return document.addEventListener("deviceready",function(){cordova.plugins.Keyboard&&(window.addEventListener("native.keyboardshow",n,!1),window.addEventListener("native.keyboardhide",r,!1))}),{hideAccessoryBar:function(e){return cordova.plugins.Keyboard.hideKeyboardAccessoryBar(e)},close:function(){return cordova.plugins.Keyboard.close()},show:function(){return cordova.plugins.Keyboard.show()},disableScroll:function(e){return cordova.plugins.Keyboard.disableScroll(e)},isVisible:function(){return cordova.plugins.Keyboard.isVisible},clearShowWatch:function(){document.removeEventListener("native.keyboardshow",n),e.$$listeners["$cordovaKeyboard:show"]=[]},clearHideWatch:function(){document.removeEventListener("native.keyboardhide",r),e.$$listeners["$cordovaKeyboard:hide"]=[]}}}]),angular.module("ngCordova.plugins.keychain",[]).factory("$cordovaKeychain",["$q",function(e){return{getForKey:function(n,r){var o=e.defer(),t=new Keychain;return t.getForKey(o.resolve,o.reject,n,r),o.promise},setForKey:function(n,r,o){var t=e.defer(),i=new Keychain;return i.setForKey(t.resolve,t.reject,n,r,o),t.promise},removeForKey:function(n,r){var o=e.defer(),t=new Keychain;return t.removeForKey(o.resolve,o.reject,n,r),o.promise}}}]),angular.module("ngCordova.plugins.launchNavigator",[]).factory("$cordovaLaunchNavigator",["$q",function(e){return{navigate:function(n,r,o){var t=e.defer();return launchnavigator.navigate(n,r,function(){t.resolve()},function(e){t.reject(e)},o),t.promise}}}]),angular.module("ngCordova.plugins.localNotification",[]).factory("$cordovaLocalNotification",["$q","$window","$rootScope","$timeout",function(e,n,r,o){return document.addEventListener("deviceready",function(){n.cordova&&n.cordova.plugins&&n.cordova.plugins.notification&&n.cordova.plugins.notification.local&&(n.cordova.plugins.notification.local.on("schedule",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:schedule",e,n)})}),n.cordova.plugins.notification.local.on("trigger",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:trigger",e,n)})}),n.cordova.plugins.notification.local.on("update",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:update",e,n)})}),n.cordova.plugins.notification.local.on("clear",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:clear",e,n)})}),n.cordova.plugins.notification.local.on("clearall",function(e){o(function(){r.$broadcast("$cordovaLocalNotification:clearall",e)})}),n.cordova.plugins.notification.local.on("cancel",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:cancel",e,n)})}),n.cordova.plugins.notification.local.on("cancelall",function(e){o(function(){r.$broadcast("$cordovaLocalNotification:cancelall",e)})}),n.cordova.plugins.notification.local.on("click",function(e,n){o(function(){r.$broadcast("$cordovaLocalNotification:click",e,n)})}))},!1),{schedule:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.schedule(r,function(e){t.resolve(e)},o),t.promise},add:function(r,o){console.warn('Deprecated: use "schedule" instead.');var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.schedule(r,function(e){t.resolve(e)},o),t.promise},update:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.update(r,function(e){t.resolve(e)},o),t.promise},clear:function(r,o){var t=e.defer();
return o=o||null,n.cordova.plugins.notification.local.clear(r,function(e){t.resolve(e)},o),t.promise},clearAll:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.clearAll(function(e){o.resolve(e)},r),o.promise},cancel:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.cancel(r,function(e){t.resolve(e)},o),t.promise},cancelAll:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.cancelAll(function(e){o.resolve(e)},r),o.promise},isPresent:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.isPresent(r,function(e){t.resolve(e)},o),t.promise},isScheduled:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.isScheduled(r,function(e){t.resolve(e)},o),t.promise},isTriggered:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.isTriggered(r,function(e){t.resolve(e)},o),t.promise},hasPermission:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.hasPermission(function(e){e?o.resolve(e):o.reject(e)},r),o.promise},registerPermission:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.registerPermission(function(e){e?o.resolve(e):o.reject(e)},r),o.promise},promptForPermission:function(r){console.warn('Deprecated: use "registerPermission" instead.');var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.registerPermission(function(e){e?o.resolve(e):o.reject(e)},r),o.promise},getAllIds:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getAllIds(function(e){o.resolve(e)},r),o.promise},getIds:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getIds(function(e){o.resolve(e)},r),o.promise},getScheduledIds:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getScheduledIds(function(e){o.resolve(e)},r),o.promise},getTriggeredIds:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getTriggeredIds(function(e){o.resolve(e)},r),o.promise},get:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.get(r,function(e){t.resolve(e)},o),t.promise},getAll:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getAll(function(e){o.resolve(e)},r),o.promise},getScheduled:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.getScheduled(r,function(e){t.resolve(e)},o),t.promise},getAllScheduled:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getAllScheduled(function(e){o.resolve(e)},r),o.promise},getTriggered:function(r,o){var t=e.defer();return o=o||null,n.cordova.plugins.notification.local.getTriggered(r,function(e){t.resolve(e)},o),t.promise},getAllTriggered:function(r){var o=e.defer();return r=r||null,n.cordova.plugins.notification.local.getAllTriggered(function(e){o.resolve(e)},r),o.promise},getDefaults:function(){return n.cordova.plugins.notification.local.getDefaults()},setDefaults:function(e){n.cordova.plugins.notification.local.setDefaults(e)}}}]),angular.module("ngCordova.plugins.mMediaAds",[]).factory("$cordovaMMediaAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.mMedia.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.mMedia.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.mMedia.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.mMedia.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.mMedia.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.mMedia.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.mMedia.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.mMedia.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.media",[]).service("NewMedia",["$q","$interval",function(e,n){function r(e){angular.isDefined(s)||(s=n(function(){0>d&&(d=e.getDuration(),a&&d>0&&a.notify({duration:d})),e.getCurrentPosition(function(e){e>-1&&(f=e)},function(e){console.log("Error getting pos="+e)}),a&&a.notify({position:f})},1e3))}function o(){angular.isDefined(s)&&(n.cancel(s),s=void 0)}function t(){f=-1,d=-1}function i(e){this.media=new Media(e,function(e){o(),t(),a.resolve(e)},function(e){o(),t(),a.reject(e)},function(e){l=e,a.notify({status:l})})}var a,c,u,s,l=null,f=-1,d=-1;return i.prototype.play=function(n){return a=e.defer(),"object"!=typeof n&&(n={}),this.media.play(n),r(this.media),a.promise},i.prototype.pause=function(){o(),this.media.pause()},i.prototype.stop=function(){this.media.stop()},i.prototype.release=function(){this.media.release(),this.media=void 0},i.prototype.seekTo=function(e){this.media.seekTo(e)},i.prototype.setVolume=function(e){this.media.setVolume(e)},i.prototype.startRecord=function(){this.media.startRecord()},i.prototype.stopRecord=function(){this.media.stopRecord()},i.prototype.currentTime=function(){return c=e.defer(),this.media.getCurrentPosition(function(e){c.resolve(e)}),c.promise},i.prototype.getDuration=function(){return u=e.defer(),this.media.getDuration(function(e){u.resolve(e)}),u.promise},i}]).factory("$cordovaMedia",["NewMedia",function(e){return{newMedia:function(n){return new e(n)}}}]),angular.module("ngCordova.plugins.mobfoxAds",[]).factory("$cordovaMobFoxAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.MobFox.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.MobFox.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.MobFox.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.MobFox.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.MobFox.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.MobFox.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.MobFox.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.MobFox.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins",["ngCordova.plugins.3dtouch","ngCordova.plugins.actionSheet","ngCordova.plugins.adMob","ngCordova.plugins.appAvailability","ngCordova.plugins.appRate","ngCordova.plugins.appVersion","ngCordova.plugins.backgroundGeolocation","ngCordova.plugins.badge","ngCordova.plugins.barcodeScanner","ngCordova.plugins.batteryStatus","ngCordova.plugins.beacon","ngCordova.plugins.ble","ngCordova.plugins.bluetoothSerial","ngCordova.plugins.brightness","ngCordova.plugins.calendar","ngCordova.plugins.camera","ngCordova.plugins.capture","ngCordova.plugins.clipboard","ngCordova.plugins.contacts","ngCordova.plugins.datePicker","ngCordova.plugins.device","ngCordova.plugins.deviceMotion","ngCordova.plugins.deviceOrientation","ngCordova.plugins.dialogs","ngCordova.plugins.emailComposer","ngCordova.plugins.facebook","ngCordova.plugins.facebookAds","ngCordova.plugins.file","ngCordova.plugins.fileTransfer","ngCordova.plugins.fileOpener2","ngCordova.plugins.flashlight","ngCordova.plugins.flurryAds","ngCordova.plugins.ga","ngCordova.plugins.geolocation","ngCordova.plugins.globalization","ngCordova.plugins.googleAds","ngCordova.plugins.googleAnalytics","ngCordova.plugins.googleMap","ngCordova.plugins.googlePlayGame","ngCordova.plugins.googlePlus","ngCordova.plugins.healthKit","ngCordova.plugins.httpd","ngCordova.plugins.iAd","ngCordova.plugins.imagePicker","ngCordova.plugins.inAppBrowser","ngCordova.plugins.instagram","ngCordova.plugins.keyboard","ngCordova.plugins.keychain","ngCordova.plugins.launchNavigator","ngCordova.plugins.localNotification","ngCordova.plugins.media","ngCordova.plugins.mMediaAds","ngCordova.plugins.mobfoxAds","ngCordova.plugins.mopubAds","ngCordova.plugins.nativeAudio","ngCordova.plugins.network","ngCordova.plugins.pinDialog","ngCordova.plugins.preferences","ngCordova.plugins.printer","ngCordova.plugins.progressIndicator","ngCordova.plugins.push","ngCordova.plugins.push_v5","ngCordova.plugins.sms","ngCordova.plugins.socialSharing","ngCordova.plugins.spinnerDialog","ngCordova.plugins.splashscreen","ngCordova.plugins.sqlite","ngCordova.plugins.statusbar","ngCordova.plugins.toast","ngCordova.plugins.touchid","ngCordova.plugins.vibration","ngCordova.plugins.videoCapturePlus","ngCordova.plugins.zip","ngCordova.plugins.insomnia"]),angular.module("ngCordova.plugins.mopubAds",[]).factory("$cordovaMoPubAds",["$q","$window",function(e,n){return{setOptions:function(r){var o=e.defer();return n.MoPub.setOptions(r,function(){o.resolve()},function(){o.reject()}),o.promise},createBanner:function(r){var o=e.defer();return n.MoPub.createBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},removeBanner:function(){var r=e.defer();return n.MoPub.removeBanner(function(){r.resolve()},function(){r.reject()}),r.promise},showBanner:function(r){var o=e.defer();return n.MoPub.showBanner(r,function(){o.resolve()},function(){o.reject()}),o.promise},showBannerAtXY:function(r,o){var t=e.defer();return n.MoPub.showBannerAtXY(r,o,function(){t.resolve()},function(){t.reject()}),t.promise},hideBanner:function(){var r=e.defer();return n.MoPub.hideBanner(function(){r.resolve()},function(){r.reject()}),r.promise},prepareInterstitial:function(r){var o=e.defer();return n.MoPub.prepareInterstitial(r,function(){o.resolve()},function(){o.reject()}),o.promise},showInterstitial:function(){var r=e.defer();return n.MoPub.showInterstitial(function(){r.resolve()},function(){r.reject()}),r.promise}}}]),angular.module("ngCordova.plugins.nativeAudio",[]).factory("$cordovaNativeAudio",["$q","$window",function(e,n){return{preloadSimple:function(r,o){var t=e.defer();return n.plugins.NativeAudio.preloadSimple(r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise},preloadComplex:function(r,o,t,i,a){var c=e.defer();return n.plugins.NativeAudio.preloadComplex(r,o,t,i,a,function(e){c.resolve(e)},function(e){c.reject(e)}),c.promise},play:function(r,o){var t=e.defer();return n.plugins.NativeAudio.play(r,function(e){t.resolve(e)},function(e){t.reject(e)},o),t.promise},stop:function(r){var o=e.defer();return n.plugins.NativeAudio.stop(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},loop:function(r){var o=e.defer();return n.plugins.NativeAudio.loop(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},unload:function(r){var o=e.defer();return n.plugins.NativeAudio.unload(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},setVolumeForComplexAsset:function(r,o){var t=e.defer();return n.plugins.NativeAudio.setVolumeForComplexAsset(r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise}}}]),angular.module("ngCordova.plugins.network",[]).factory("$cordovaNetwork",["$rootScope","$timeout",function(e,n){var r=function(){var r=navigator.connection.type;n(function(){e.$broadcast("$cordovaNetwork:offline",r)})},o=function(){var r=navigator.connection.type;n(function(){e.$broadcast("$cordovaNetwork:online",r)})};return document.addEventListener("deviceready",function(){navigator.connection&&(document.addEventListener("offline",r,!1),document.addEventListener("online",o,!1))}),{getNetwork:function(){return navigator.connection.type},isOnline:function(){var e=navigator.connection.type;return e!==Connection.UNKNOWN&&e!==Connection.NONE},isOffline:function(){var e=navigator.connection.type;return e===Connection.UNKNOWN||e===Connection.NONE},clearOfflineWatch:function(){document.removeEventListener("offline",r),e.$$listeners["$cordovaNetwork:offline"]=[]},clearOnlineWatch:function(){document.removeEventListener("online",o),e.$$listeners["$cordovaNetwork:online"]=[]}}}]).run(["$injector",function(e){e.get("$cordovaNetwork")}]),angular.module("ngCordova.plugins.pinDialog",[]).factory("$cordovaPinDialog",["$q","$window",function(e,n){return{prompt:function(r,o,t){var i=e.defer();return n.plugins.pinDialog.prompt(r,function(e){i.resolve(e)},o,t),i.promise}}}]),angular.module("ngCordova.plugins.preferences",[]).factory("$cordovaPreferences",["$window","$q",function(e,n){return{pluginNotEnabledMessage:"Plugin not enabled",decoratePromise:function(e){e.success=function(n){return e.then(n),e},e.error=function(n){return e.then(null,n),e}},store:function(r,o,t){function i(e){c.resolve(e)}function a(e){c.reject(new Error(e))}var c=n.defer(),u=c.promise;if(e.plugins){var s;s=3===arguments.length?e.plugins.appPreferences.store(t,r,o):e.plugins.appPreferences.store(r,o),s.then(i,a)}else c.reject(new Error(this.pluginNotEnabledMessage));return this.decoratePromise(u),u},fetch:function(r,o){function t(e){a.resolve(e)}function i(e){a.reject(new Error(e))}var a=n.defer(),c=a.promise;if(e.plugins){var u;u=2===arguments.length?e.plugins.appPreferences.fetch(o,r):e.plugins.appPreferences.fetch(r),u.then(t,i)}else a.reject(new Error(this.pluginNotEnabledMessage));return this.decoratePromise(c),c},remove:function(r,o){function t(e){a.resolve(e)}function i(e){a.reject(new Error(e))}var a=n.defer(),c=a.promise;if(e.plugins){var u;u=2===arguments.length?e.plugins.appPreferences.remove(o,r):e.plugins.appPreferences.remove(r),u.then(t,i)}else a.reject(new Error(this.pluginNotEnabledMessage));return this.decoratePromise(c),c},show:function(){function r(e){t.resolve(e)}function o(e){t.reject(new Error(e))}var t=n.defer(),i=t.promise;return e.plugins?e.plugins.appPreferences.show().then(r,o):t.reject(new Error(this.pluginNotEnabledMessage)),this.decoratePromise(i),i}}}]),angular.module("ngCordova.plugins.printer",[]).factory("$cordovaPrinter",["$q","$window",function(e,n){return{isAvailable:function(){var r=e.defer();return n.plugin.printer.isAvailable(function(e){r.resolve(e)}),r.promise},print:function(r,o){var t=e.defer();return n.plugin.printer.print(r,o,function(){t.resolve()}),t.promise}}}]),angular.module("ngCordova.plugins.progressIndicator",[]).factory("$cordovaProgress",[function(){return{show:function(e){var n=e||"Please wait...";return ProgressIndicator.show(n)},showSimple:function(e){var n=e||!1;return ProgressIndicator.showSimple(n)},showSimpleWithLabel:function(e,n){var r=e||!1,o=n||"Loading...";return ProgressIndicator.showSimpleWithLabel(r,o)},showSimpleWithLabelDetail:function(e,n,r){var o=e||!1,t=n||"Loading...",i=r||"Please wait";return ProgressIndicator.showSimpleWithLabelDetail(o,t,i)},showDeterminate:function(e,n){var r=e||!1,o=n||5e4;return ProgressIndicator.showDeterminate(r,o)},showDeterminateWithLabel:function(e,n,r){var o=e||!1,t=n||5e4,i=r||"Loading...";return ProgressIndicator.showDeterminateWithLabel(o,t,i)},showAnnular:function(e,n){var r=e||!1,o=n||5e4;return ProgressIndicator.showAnnular(r,o)},showAnnularWithLabel:function(e,n,r){var o=e||!1,t=n||5e4,i=r||"Loading...";return ProgressIndicator.showAnnularWithLabel(o,t,i)},showBar:function(e,n){var r=e||!1,o=n||5e4;return ProgressIndicator.showBar(r,o)},showBarWithLabel:function(e,n,r){var o=e||!1,t=n||5e4,i=r||"Loading...";return ProgressIndicator.showBarWithLabel(o,t,i)},showSuccess:function(e,n){var r=e||!1,o=n||"Success";return ProgressIndicator.showSuccess(r,o)},showText:function(e,n,r){var o=e||!1,t=n||"Warning",i=r||"center";return ProgressIndicator.showText(o,t,i)},hide:function(){return ProgressIndicator.hide()}}}]),angular.module("ngCordova.plugins.push",[]).factory("$cordovaPush",["$q","$window","$rootScope","$timeout",function(e,n,r,o){return{onNotification:function(e){o(function(){r.$broadcast("$cordovaPush:notificationReceived",e)})},register:function(r){var o,t=e.defer();return void 0!==r&&void 0===r.ecb&&(o=null===document.querySelector("[ng-app]")?"document.body":"document.querySelector('[ng-app]')",r.ecb="angular.element("+o+").injector().get('$cordovaPush').onNotification"),n.plugins.pushNotification.register(function(e){t.resolve(e)},function(e){t.reject(e)},r),t.promise},unregister:function(r){var o=e.defer();return n.plugins.pushNotification.unregister(function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise},setBadgeNumber:function(r){var o=e.defer();return n.plugins.pushNotification.setApplicationIconBadgeNumber(function(e){o.resolve(e)},function(e){o.reject(e)},r),o.promise}}}]),angular.module("ngCordova.plugins.push_v5",[]).factory("$cordovaPushV5",["$q","$rootScope","$timeout",function(e,n,r){var o;return{initialize:function(n){var r=e.defer();return o=PushNotification.init(n),r.resolve(o),r.promise},onNotification:function(){r(function(){o.on("notification",function(e){n.$emit("$cordovaPushV5:notificationReceived",e)})})},onError:function(){r(function(){o.on("error",function(e){n.$emit("$cordovaPushV5:errorOccurred",e)})})},register:function(){var n=e.defer();return void 0===o?n.reject(new Error("init must be called before any other operation")):o.on("registration",function(e){n.resolve(e.registrationId)}),n.promise},unregister:function(){var n=e.defer();return void 0===o?n.reject(new Error("init must be called before any other operation")):o.unregister(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},getBadgeNumber:function(){var n=e.defer();return void 0===o?n.reject(new Error("init must be called before any other operation")):o.getApplicationIconBadgeNumber(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise},setBadgeNumber:function(n){var r=e.defer();return void 0===o?r.reject(new Error("init must be called before any other operation")):o.setApplicationIconBadgeNumber(function(e){r.resolve(e)},function(e){r.reject(e)},n),r.promise},finish:function(){var n=e.defer();return void 0===o?n.reject(new Error("init must be called before any other operation")):o.finish(function(e){n.resolve(e)},function(e){n.reject(e)}),n.promise}}}]),angular.module("ngCordova.plugins.recentsControl",[]).factory("$cordovaRecents",function(){return{setColor:function(e){return RecentsControl.setColor(e)},setDescription:function(e){return RecentsControl.setDescription(e)},setOptions:function(e,n){return RecentsControl.setOptions(e,n)}}}),angular.module("ngCordova.plugins.screenshot",[]).factory("$cordovaScreenshot",["$q",function(e){return{captureToFile:function(n){var r=n||{},o=r.extension||"jpg",t=r.quality||"100",i=e.defer();return navigator.screenshot?(navigator.screenshot.save(function(e,n){e?i.reject(e):i.resolve(n.filePath)},o,t,r.filename),i.promise):(i.resolve(null),i.promise)},captureToUri:function(n){var r=n||{},o=r.extension||"jpg",t=r.quality||"100",i=e.defer();return navigator.screenshot?(navigator.screenshot.URI(function(e,n){e?i.reject(e):i.resolve(n.URI)},o,t,r.filename),i.promise):(i.resolve(null),i.promise)}}}]),angular.module("ngCordova.plugins.serial",[]).factory("$cordovaSerial",["$q",function(e){var n={};return n.requestPermission=function(n){var r=e.defer();return serial.requestPermission(n,function(){r.resolve()},function(e){r.reject(e)}),r.promise},n.open=function(n){var r=e.defer();return serial.open(n,function(){r.resolve()},function(e){r.reject(e)}),r.promise},n.write=function(n){var r=e.defer();return serial.write(n,function(){r.resolve()},function(e){r.reject(e)}),r.promise},n.writeHex=function(n){var r=e.defer();return serial.writeHex(n,function(){r.resolve()},function(e){r.reject(e)}),r.promise},n.read=function(){var n=e.defer();return serial.read(function(e){var r=new Uint8Array(e);n.resolve(r)},function(e){n.reject(e)}),n.promise},n.registerReadCallback=function(e,n){serial.registerReadCallback(function(n){var r=new Uint8Array(n);e(r)},n)},n.close=function(){var n=e.defer();return serial.close(function(){n.resolve()},function(e){n.reject(e)}),n.promise},n}]),angular.module("ngCordova.plugins.sms",[]).factory("$cordovaSms",["$q",function(e){return{send:function(n,r,o){var t=e.defer();return sms.send(n,r,o,function(e){t.resolve(e)},function(e){t.reject(e)}),t.promise}}}]),angular.module("ngCordova.plugins.socialSharing",[]).factory("$cordovaSocialSharing",["$q","$window",function(e,n){return{share:function(r,o,t,i){var a=e.defer();return o=o||null,t=t||null,i=i||null,n.plugins.socialsharing.share(r,o,t,i,function(){a.resolve(!0)},function(){a.reject(!1)}),a.promise},shareWithOptions:function(r){var o=e.defer();return n.plugins.socialsharing.shareWithOptions(r,function(){o.resolve(!0)},function(){o.reject(!1)}),o.promise},shareViaTwitter:function(r,o,t){var i=e.defer();return o=o||null,t=t||null,n.plugins.socialsharing.shareViaTwitter(r,o,t,function(){i.resolve(!0)},function(){i.reject(!1)}),i.promise},shareViaWhatsApp:function(r,o,t){var i=e.defer();return o=o||null,t=t||null,n.plugins.socialsharing.shareViaWhatsApp(r,o,t,function(){i.resolve(!0)},function(){i.reject(!1)}),i.promise},shareViaFacebook:function(r,o,t){var i=e.defer();return r=r||null,o=o||null,t=t||null,n.plugins.socialsharing.shareViaFacebook(r,o,t,function(){i.resolve(!0)},function(){i.reject(!1)}),i.promise},shareViaFacebookWithPasteMessageHint:function(r,o,t,i){var a=e.defer();return o=o||null,t=t||null,n.plugins.socialsharing.shareViaFacebookWithPasteMessageHint(r,o,t,i,function(){a.resolve(!0)},function(){a.reject(!1)}),a.promise},shareViaSMS:function(r,o){var t=e.defer();return n.plugins.socialsharing.shareViaSMS(r,o,function(){t.resolve(!0)},function(){t.reject(!1)}),t.promise},shareViaEmail:function(r,o,t,i,a,c){var u=e.defer();return t=t||null,i=i||null,a=a||null,c=c||null,n.plugins.socialsharing.shareViaEmail(r,o,t,i,a,c,function(){u.resolve(!0)},function(){u.reject(!1)}),u.promise},shareVia:function(r,o,t,i,a){var c=e.defer();return o=o||null,t=t||null,i=i||null,a=a||null,n.plugins.socialsharing.shareVia(r,o,t,i,a,function(){c.resolve(!0)},function(){c.reject(!1)}),c.promise},canShareViaEmail:function(){var r=e.defer();return n.plugins.socialsharing.canShareViaEmail(function(){r.resolve(!0)},function(){r.reject(!1)}),r.promise},canShareVia:function(r,o,t,i,a){var c=e.defer();return n.plugins.socialsharing.canShareVia(r,o,t,i,a,function(e){c.resolve(e)},function(e){c.reject(e)}),c.promise},available:function(){var n=e.defer();return window.plugins.socialsharing.available(function(e){e?n.resolve():n.reject()}),n.promise}}}]),angular.module("ngCordova.plugins.spinnerDialog",[]).factory("$cordovaSpinnerDialog",["$window",function(e){return{show:function(n,r,o,t){return o=o||!1,e.plugins.spinnerDialog.show(n,r,o,t)},hide:function(){return e.plugins.spinnerDialog.hide()}}}]),angular.module("ngCordova.plugins.splashscreen",[]).factory("$cordovaSplashscreen",[function(){return{hide:function(){return navigator.splashscreen.hide()},show:function(){return navigator.splashscreen.show()}}}]),angular.module("ngCordova.plugins.sqlite",[]).factory("$cordovaSQLite",["$q","$window",function(e,n){return{openDB:function(e,r){return angular.isObject(e)&&!angular.isString(e)?("undefined"!=typeof r&&(e.bgType=r),n.sqlitePlugin.openDatabase(e)):n.sqlitePlugin.openDatabase({name:e,bgType:r})},execute:function(n,r,o){var t=e.defer();return n.transaction(function(e){e.executeSql(r,o,function(e,n){t.resolve(n)},function(e,n){t.reject(n)})}),t.promise},insertCollection:function(n,r,o){var t=e.defer(),i=o.slice(0);return n.transaction(function(e){!function n(){var o=i.splice(0,1)[0];try{e.executeSql(r,o,function(e,r){0===i.length?t.resolve(r):n()},function(e,n){t.reject(n)})}catch(a){t.reject(a)}}()}),t.promise},nestedExecute:function(n,r,o,t,i){var a=e.defer();return n.transaction(function(e){e.executeSql(r,t,function(e,n){a.resolve(n),e.executeSql(o,i,function(e,n){a.resolve(n)})})},function(e,n){a.reject(n)}),a.promise},deleteDB:function(r){var o=e.defer();return n.sqlitePlugin.deleteDatabase(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise}}}]),angular.module("ngCordova.plugins.statusbar",[]).factory("$cordovaStatusbar",[function(){return{overlaysWebView:function(e){return StatusBar.overlaysWebView(!!e)},STYLES:{DEFAULT:0,LIGHT_CONTENT:1,BLACK_TRANSLUCENT:2,BLACK_OPAQUE:3},style:function(e){switch(e){case 0:return StatusBar.styleDefault();case 1:return StatusBar.styleLightContent();case 2:return StatusBar.styleBlackTranslucent();case 3:return StatusBar.styleBlackOpaque();default:return StatusBar.styleDefault()}},styleColor:function(e){return StatusBar.backgroundColorByName(e)},styleHex:function(e){return StatusBar.backgroundColorByHexString(e)},hide:function(){return StatusBar.hide()},show:function(){return StatusBar.show()},isVisible:function(){return StatusBar.isVisible}}}]),angular.module("ngCordova.plugins.toast",[]).factory("$cordovaToast",["$q","$window",function(e,n){return{showShortTop:function(r){var o=e.defer();return n.plugins.toast.showShortTop(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showShortCenter:function(r){var o=e.defer();return n.plugins.toast.showShortCenter(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showShortBottom:function(r){var o=e.defer();return n.plugins.toast.showShortBottom(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showLongTop:function(r){var o=e.defer();return n.plugins.toast.showLongTop(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showLongCenter:function(r){var o=e.defer();return n.plugins.toast.showLongCenter(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showLongBottom:function(r){var o=e.defer();return n.plugins.toast.showLongBottom(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},showWithOptions:function(r){var o=e.defer();return n.plugins.toast.showWithOptions(r,function(e){o.resolve(e)},function(e){o.reject(e)}),o.promise},show:function(r,o,t){var i=e.defer();return n.plugins.toast.show(r,o,t,function(e){i.resolve(e)},function(e){i.reject(e)}),i.promise},hide:function(){var r=e.defer();try{n.plugins.toast.hide(),r.resolve()}catch(o){r.reject(o&&o.message)}return r.promise}}}]),angular.module("ngCordova.plugins.touchid",[]).factory("$cordovaTouchID",["$q",function(e){return{checkSupport:function(){var n=e.defer();return window.cordova?touchid.checkSupport(function(e){n.resolve(e)},function(e){n.reject(e)}):n.reject("Not supported without cordova.js"),n.promise},authenticate:function(n){var r=e.defer();return window.cordova?touchid.authenticate(function(e){r.resolve(e)},function(e){r.reject(e)},n):r.reject("Not supported without cordova.js"),r.promise}}}]),angular.module("ngCordova.plugins.tts",[]).factory("$cordovaTTS",function(){return{speak:function(e,n,r){return TTS.speak(e,n,r)}}}),angular.module("ngCordova.plugins.upsPush",[]).factory("$cordovaUpsPush",["$q","$window","$rootScope","$timeout",function(e,n,r,o){return{register:function(t){var i=e.defer();return n.push.register(function(e){o(function(){r.$broadcast("$cordovaUpsPush:notificationReceived",e)})},function(){i.resolve()},function(e){i.reject(e)},t),i.promise},unregister:function(r){var o=e.defer();return n.push.unregister(function(){o.resolve()},function(e){o.reject(e)},r),o.promise},setBadgeNumber:function(r){var o=e.defer();return n.push.setApplicationIconBadgeNumber(function(){o.resolve()},r),o.promise}}}]),angular.module("ngCordova.plugins.vibration",[]).factory("$cordovaVibration",[function(){return{vibrate:function(e){return navigator.notification.vibrate(e)},vibrateWithPattern:function(e,n){return navigator.notification.vibrateWithPattern(e,n)},cancelVibration:function(){return navigator.notification.cancelVibration()}}}]),angular.module("ngCordova.plugins.videoCapturePlus",[]).provider("$cordovaVideoCapturePlus",[function(){var e={};this.setLimit=function(n){e.limit=n},this.setMaxDuration=function(n){e.duration=n},this.setHighQuality=function(n){e.highquality=n},this.useFrontCamera=function(n){e.frontcamera=n},this.setPortraitOverlay=function(n){e.portraitOverlay=n},this.setLandscapeOverlay=function(n){e.landscapeOverlay=n},this.setOverlayText=function(n){e.overlayText=n},this.$get=["$q","$window",function(n,r){return{captureVideo:function(o){var t=n.defer();return r.plugins.videocaptureplus?(r.plugins.videocaptureplus.captureVideo(t.resolve,t.reject,angular.extend({},e,o)),t.promise):(t.resolve(null),t.promise)}}}]}]),angular.module("ngCordova.plugins.zip",[]).factory("$cordovaZip",["$q","$window",function(e,n){return{unzip:function(r,o){var t=e.defer();return n.zip.unzip(r,o,function(e){0===e?t.resolve():t.reject()},function(e){t.notify(e)}),t.promise}}}])}();
/*
 AngularJS v1.3.15
 (c) 2010-2014 Google, Inc. http://angularjs.org
 License: MIT
*/
(function(r,f,s){'use strict';f.module("ngMessages",[]).directive("ngMessages",["$compile","$animate","$templateRequest",function(q,k,l){return{restrict:"AE",controller:function(){this.$renderNgMessageClasses=f.noop;var b=[];this.registerMessage=function(d,a){for(var c=0;c<b.length;c++)if(b[c].type==a.type){if(d!=c){var g=b[d];b[d]=b[c];d<b.length?b[c]=g:b.splice(0,c)}return}b.splice(d,0,a)};this.renderMessages=function(d,a){d=d||{};var c;f.forEach(b,function(b){var e;if(e=!c||a)e=d[b.type],e=null!==
e&&!1!==e&&e;e?(b.attach(),c=!0):b.detach()});this.renderElementClasses(c)}},require:"ngMessages",link:function(b,d,a,c){c.renderElementClasses=function(b){b?k.setClass(d,"ng-active","ng-inactive"):k.setClass(d,"ng-inactive","ng-active")};var g=f.isString(a.ngMessagesMultiple)||f.isString(a.multiple),e;b.$watchCollection(a.ngMessages||a["for"],function(b){e=b;c.renderMessages(b,g)});(a=a.ngMessagesInclude||a.include)&&l(a).then(function(a){var h;a=f.element("<div/>").html(a);f.forEach(a.children(),
function(a){a=f.element(a);h?h.after(a):d.prepend(a);h=a;q(a)(b)});c.renderMessages(e,g)})}}}]).directive("ngMessage",["$animate",function(f){return{require:"^ngMessages",transclude:"element",terminal:!0,restrict:"AE",link:function(k,l,b,d,a){for(var c,g,e=l[0],n=e.parentNode,h=0,p=0;h<n.childNodes.length;h++){var m=n.childNodes[h];if(8==m.nodeType&&0<=m.nodeValue.indexOf("ngMessage")){if(m===e){c=p;break}p++}}d.registerMessage(c,{type:b.ngMessage||b.when,attach:function(){g||a(k,function(a){f.enter(a,
null,l);g=a})},detach:function(a){g&&(f.leave(g),g=null)}})}}}])})(window,window.angular);
//# sourceMappingURL=angular-messages.min.js.map

/*
 AngularJS v1.4.8
 (c) 2010-2015 Google, Inc. http://angularjs.org
 License: MIT
*/
(function(p,c,n){'use strict';function l(b,a,g){var d=g.baseHref(),k=b[0];return function(b,e,f){var g,h;f=f||{};h=f.expires;g=c.isDefined(f.path)?f.path:d;c.isUndefined(e)&&(h="Thu, 01 Jan 1970 00:00:00 GMT",e="");c.isString(h)&&(h=new Date(h));e=encodeURIComponent(b)+"="+encodeURIComponent(e);e=e+(g?";path="+g:"")+(f.domain?";domain="+f.domain:"");e+=h?";expires="+h.toUTCString():"";e+=f.secure?";secure":"";f=e.length+1;4096<f&&a.warn("Cookie '"+b+"' possibly not set or overflowed because it was too large ("+
f+" > 4096 bytes)!");k.cookie=e}}c.module("ngCookies",["ng"]).provider("$cookies",[function(){var b=this.defaults={};this.$get=["$$cookieReader","$$cookieWriter",function(a,g){return{get:function(d){return a()[d]},getObject:function(d){return(d=this.get(d))?c.fromJson(d):d},getAll:function(){return a()},put:function(d,a,m){g(d,a,m?c.extend({},b,m):b)},putObject:function(d,b,a){this.put(d,c.toJson(b),a)},remove:function(a,k){g(a,n,k?c.extend({},b,k):b)}}}]}]);c.module("ngCookies").factory("$cookieStore",
["$cookies",function(b){return{get:function(a){return b.getObject(a)},put:function(a,c){b.putObject(a,c)},remove:function(a){b.remove(a)}}}]);l.$inject=["$document","$log","$browser"];c.module("ngCookies").provider("$$cookieWriter",function(){this.$get=l})})(window,window.angular);
//# sourceMappingURL=angular-cookies.min.js.map

/*!
 * Chart.js
 * http://chartjs.org/
 * Version: 2.2.1
 *
 * Copyright 2016 Nick Downie
 * Released under the MIT license
 * https://github.com/chartjs/Chart.js/blob/master/LICENSE.md
 */
!function(t){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=t();else if("function"==typeof define&&define.amd)define([],t);else{var e;e="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,e.Chart=t()}}(function(){return function t(e,a,i){function n(r,l){if(!a[r]){if(!e[r]){var s="function"==typeof require&&require;if(!l&&s)return s(r,!0);if(o)return o(r,!0);var d=new Error("Cannot find module '"+r+"'");throw d.code="MODULE_NOT_FOUND",d}var u=a[r]={exports:{}};e[r][0].call(u.exports,function(t){var a=e[r][1][t];return n(a?a:t)},u,u.exports,t,e,a,i)}return a[r].exports}for(var o="function"==typeof require&&require,r=0;r<i.length;r++)n(i[r]);return n}({1:[function(t,e,a){},{}],2:[function(t,e,a){function i(t){if(t){var e=/^#([a-fA-F0-9]{3})$/,a=/^#([a-fA-F0-9]{6})$/,i=/^rgba?\(\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/,n=/^rgba?\(\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/,o=/(\w+)/,r=[0,0,0],l=1,s=t.match(e);if(s){s=s[1];for(var d=0;d<r.length;d++)r[d]=parseInt(s[d]+s[d],16)}else if(s=t.match(a)){s=s[1];for(var d=0;d<r.length;d++)r[d]=parseInt(s.slice(2*d,2*d+2),16)}else if(s=t.match(i)){for(var d=0;d<r.length;d++)r[d]=parseInt(s[d+1]);l=parseFloat(s[4])}else if(s=t.match(n)){for(var d=0;d<r.length;d++)r[d]=Math.round(2.55*parseFloat(s[d+1]));l=parseFloat(s[4])}else if(s=t.match(o)){if("transparent"==s[1])return[0,0,0,0];if(r=y[s[1]],!r)return}for(var d=0;d<r.length;d++)r[d]=v(r[d],0,255);return l=l||0==l?v(l,0,1):1,r[3]=l,r}}function n(t){if(t){var e=/^hsla?\(\s*([+-]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)/,a=t.match(e);if(a){var i=parseFloat(a[4]),n=v(parseInt(a[1]),0,360),o=v(parseFloat(a[2]),0,100),r=v(parseFloat(a[3]),0,100),l=v(isNaN(i)?1:i,0,1);return[n,o,r,l]}}}function o(t){if(t){var e=/^hwb\(\s*([+-]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)/,a=t.match(e);if(a){var i=parseFloat(a[4]),n=v(parseInt(a[1]),0,360),o=v(parseFloat(a[2]),0,100),r=v(parseFloat(a[3]),0,100),l=v(isNaN(i)?1:i,0,1);return[n,o,r,l]}}}function r(t){var e=i(t);return e&&e.slice(0,3)}function l(t){var e=n(t);return e&&e.slice(0,3)}function s(t){var e=i(t);return e?e[3]:(e=n(t))?e[3]:(e=o(t))?e[3]:void 0}function d(t){return"#"+x(t[0])+x(t[1])+x(t[2])}function u(t,e){return 1>e||t[3]&&t[3]<1?c(t,e):"rgb("+t[0]+", "+t[1]+", "+t[2]+")"}function c(t,e){return void 0===e&&(e=void 0!==t[3]?t[3]:1),"rgba("+t[0]+", "+t[1]+", "+t[2]+", "+e+")"}function h(t,e){if(1>e||t[3]&&t[3]<1)return f(t,e);var a=Math.round(t[0]/255*100),i=Math.round(t[1]/255*100),n=Math.round(t[2]/255*100);return"rgb("+a+"%, "+i+"%, "+n+"%)"}function f(t,e){var a=Math.round(t[0]/255*100),i=Math.round(t[1]/255*100),n=Math.round(t[2]/255*100);return"rgba("+a+"%, "+i+"%, "+n+"%, "+(e||t[3]||1)+")"}function g(t,e){return 1>e||t[3]&&t[3]<1?p(t,e):"hsl("+t[0]+", "+t[1]+"%, "+t[2]+"%)"}function p(t,e){return void 0===e&&(e=void 0!==t[3]?t[3]:1),"hsla("+t[0]+", "+t[1]+"%, "+t[2]+"%, "+e+")"}function m(t,e){return void 0===e&&(e=void 0!==t[3]?t[3]:1),"hwb("+t[0]+", "+t[1]+"%, "+t[2]+"%"+(void 0!==e&&1!==e?", "+e:"")+")"}function b(t){return k[t.slice(0,3)]}function v(t,e,a){return Math.min(Math.max(e,t),a)}function x(t){var e=t.toString(16).toUpperCase();return e.length<2?"0"+e:e}var y=t(6);e.exports={getRgba:i,getHsla:n,getRgb:r,getHsl:l,getHwb:o,getAlpha:s,hexString:d,rgbString:u,rgbaString:c,percentString:h,percentaString:f,hslString:g,hslaString:p,hwbString:m,keyword:b};var k={};for(var S in y)k[y[S]]=S},{6:6}],3:[function(t,e,a){var i=t(5),n=t(2),o=function(t){if(t instanceof o)return t;if(!(this instanceof o))return new o(t);this.values={rgb:[0,0,0],hsl:[0,0,0],hsv:[0,0,0],hwb:[0,0,0],cmyk:[0,0,0,0],alpha:1};var e;if("string"==typeof t)if(e=n.getRgba(t))this.setValues("rgb",e);else if(e=n.getHsla(t))this.setValues("hsl",e);else{if(!(e=n.getHwb(t)))throw new Error('Unable to parse color from string "'+t+'"');this.setValues("hwb",e)}else if("object"==typeof t)if(e=t,void 0!==e.r||void 0!==e.red)this.setValues("rgb",e);else if(void 0!==e.l||void 0!==e.lightness)this.setValues("hsl",e);else if(void 0!==e.v||void 0!==e.value)this.setValues("hsv",e);else if(void 0!==e.w||void 0!==e.whiteness)this.setValues("hwb",e);else{if(void 0===e.c&&void 0===e.cyan)throw new Error("Unable to parse color from object "+JSON.stringify(t));this.setValues("cmyk",e)}};o.prototype={rgb:function(){return this.setSpace("rgb",arguments)},hsl:function(){return this.setSpace("hsl",arguments)},hsv:function(){return this.setSpace("hsv",arguments)},hwb:function(){return this.setSpace("hwb",arguments)},cmyk:function(){return this.setSpace("cmyk",arguments)},rgbArray:function(){return this.values.rgb},hslArray:function(){return this.values.hsl},hsvArray:function(){return this.values.hsv},hwbArray:function(){var t=this.values;return 1!==t.alpha?t.hwb.concat([t.alpha]):t.hwb},cmykArray:function(){return this.values.cmyk},rgbaArray:function(){var t=this.values;return t.rgb.concat([t.alpha])},hslaArray:function(){var t=this.values;return t.hsl.concat([t.alpha])},alpha:function(t){return void 0===t?this.values.alpha:(this.setValues("alpha",t),this)},red:function(t){return this.setChannel("rgb",0,t)},green:function(t){return this.setChannel("rgb",1,t)},blue:function(t){return this.setChannel("rgb",2,t)},hue:function(t){return t&&(t%=360,t=0>t?360+t:t),this.setChannel("hsl",0,t)},saturation:function(t){return this.setChannel("hsl",1,t)},lightness:function(t){return this.setChannel("hsl",2,t)},saturationv:function(t){return this.setChannel("hsv",1,t)},whiteness:function(t){return this.setChannel("hwb",1,t)},blackness:function(t){return this.setChannel("hwb",2,t)},value:function(t){return this.setChannel("hsv",2,t)},cyan:function(t){return this.setChannel("cmyk",0,t)},magenta:function(t){return this.setChannel("cmyk",1,t)},yellow:function(t){return this.setChannel("cmyk",2,t)},black:function(t){return this.setChannel("cmyk",3,t)},hexString:function(){return n.hexString(this.values.rgb)},rgbString:function(){return n.rgbString(this.values.rgb,this.values.alpha)},rgbaString:function(){return n.rgbaString(this.values.rgb,this.values.alpha)},percentString:function(){return n.percentString(this.values.rgb,this.values.alpha)},hslString:function(){return n.hslString(this.values.hsl,this.values.alpha)},hslaString:function(){return n.hslaString(this.values.hsl,this.values.alpha)},hwbString:function(){return n.hwbString(this.values.hwb,this.values.alpha)},keyword:function(){return n.keyword(this.values.rgb,this.values.alpha)},rgbNumber:function(){var t=this.values.rgb;return t[0]<<16|t[1]<<8|t[2]},luminosity:function(){for(var t=this.values.rgb,e=[],a=0;a<t.length;a++){var i=t[a]/255;e[a]=.03928>=i?i/12.92:Math.pow((i+.055)/1.055,2.4)}return.2126*e[0]+.7152*e[1]+.0722*e[2]},contrast:function(t){var e=this.luminosity(),a=t.luminosity();return e>a?(e+.05)/(a+.05):(a+.05)/(e+.05)},level:function(t){var e=this.contrast(t);return e>=7.1?"AAA":e>=4.5?"AA":""},dark:function(){var t=this.values.rgb,e=(299*t[0]+587*t[1]+114*t[2])/1e3;return 128>e},light:function(){return!this.dark()},negate:function(){for(var t=[],e=0;3>e;e++)t[e]=255-this.values.rgb[e];return this.setValues("rgb",t),this},lighten:function(t){var e=this.values.hsl;return e[2]+=e[2]*t,this.setValues("hsl",e),this},darken:function(t){var e=this.values.hsl;return e[2]-=e[2]*t,this.setValues("hsl",e),this},saturate:function(t){var e=this.values.hsl;return e[1]+=e[1]*t,this.setValues("hsl",e),this},desaturate:function(t){var e=this.values.hsl;return e[1]-=e[1]*t,this.setValues("hsl",e),this},whiten:function(t){var e=this.values.hwb;return e[1]+=e[1]*t,this.setValues("hwb",e),this},blacken:function(t){var e=this.values.hwb;return e[2]+=e[2]*t,this.setValues("hwb",e),this},greyscale:function(){var t=this.values.rgb,e=.3*t[0]+.59*t[1]+.11*t[2];return this.setValues("rgb",[e,e,e]),this},clearer:function(t){var e=this.values.alpha;return this.setValues("alpha",e-e*t),this},opaquer:function(t){var e=this.values.alpha;return this.setValues("alpha",e+e*t),this},rotate:function(t){var e=this.values.hsl,a=(e[0]+t)%360;return e[0]=0>a?360+a:a,this.setValues("hsl",e),this},mix:function(t,e){var a=this,i=t,n=void 0===e?.5:e,o=2*n-1,r=a.alpha()-i.alpha(),l=((o*r===-1?o:(o+r)/(1+o*r))+1)/2,s=1-l;return this.rgb(l*a.red()+s*i.red(),l*a.green()+s*i.green(),l*a.blue()+s*i.blue()).alpha(a.alpha()*n+i.alpha()*(1-n))},toJSON:function(){return this.rgb()},clone:function(){var t,e,a=new o,i=this.values,n=a.values;for(var r in i)i.hasOwnProperty(r)&&(t=i[r],e={}.toString.call(t),"[object Array]"===e?n[r]=t.slice(0):"[object Number]"===e?n[r]=t:console.error("unexpected color value:",t));return a}},o.prototype.spaces={rgb:["red","green","blue"],hsl:["hue","saturation","lightness"],hsv:["hue","saturation","value"],hwb:["hue","whiteness","blackness"],cmyk:["cyan","magenta","yellow","black"]},o.prototype.maxes={rgb:[255,255,255],hsl:[360,100,100],hsv:[360,100,100],hwb:[360,100,100],cmyk:[100,100,100,100]},o.prototype.getValues=function(t){for(var e=this.values,a={},i=0;i<t.length;i++)a[t.charAt(i)]=e[t][i];return 1!==e.alpha&&(a.a=e.alpha),a},o.prototype.setValues=function(t,e){var a,n=this.values,o=this.spaces,r=this.maxes,l=1;if("alpha"===t)l=e;else if(e.length)n[t]=e.slice(0,t.length),l=e[t.length];else if(void 0!==e[t.charAt(0)]){for(a=0;a<t.length;a++)n[t][a]=e[t.charAt(a)];l=e.a}else if(void 0!==e[o[t][0]]){var s=o[t];for(a=0;a<t.length;a++)n[t][a]=e[s[a]];l=e.alpha}if(n.alpha=Math.max(0,Math.min(1,void 0===l?n.alpha:l)),"alpha"===t)return!1;var d;for(a=0;a<t.length;a++)d=Math.max(0,Math.min(r[t][a],n[t][a])),n[t][a]=Math.round(d);for(var u in o)u!==t&&(n[u]=i[t][u](n[t]));return!0},o.prototype.setSpace=function(t,e){var a=e[0];return void 0===a?this.getValues(t):("number"==typeof a&&(a=Array.prototype.slice.call(e)),this.setValues(t,a),this)},o.prototype.setChannel=function(t,e,a){var i=this.values[t];return void 0===a?i[e]:a===i[e]?this:(i[e]=a,this.setValues(t,i),this)},"undefined"!=typeof window&&(window.Color=o),e.exports=o},{2:2,5:5}],4:[function(t,e,a){function i(t){var e,a,i,n=t[0]/255,o=t[1]/255,r=t[2]/255,l=Math.min(n,o,r),s=Math.max(n,o,r),d=s-l;return s==l?e=0:n==s?e=(o-r)/d:o==s?e=2+(r-n)/d:r==s&&(e=4+(n-o)/d),e=Math.min(60*e,360),0>e&&(e+=360),i=(l+s)/2,a=s==l?0:.5>=i?d/(s+l):d/(2-s-l),[e,100*a,100*i]}function n(t){var e,a,i,n=t[0],o=t[1],r=t[2],l=Math.min(n,o,r),s=Math.max(n,o,r),d=s-l;return a=0==s?0:d/s*1e3/10,s==l?e=0:n==s?e=(o-r)/d:o==s?e=2+(r-n)/d:r==s&&(e=4+(n-o)/d),e=Math.min(60*e,360),0>e&&(e+=360),i=s/255*1e3/10,[e,a,i]}function o(t){var e=t[0],a=t[1],n=t[2],o=i(t)[0],r=1/255*Math.min(e,Math.min(a,n)),n=1-1/255*Math.max(e,Math.max(a,n));return[o,100*r,100*n]}function l(t){var e,a,i,n,o=t[0]/255,r=t[1]/255,l=t[2]/255;return n=Math.min(1-o,1-r,1-l),e=(1-o-n)/(1-n)||0,a=(1-r-n)/(1-n)||0,i=(1-l-n)/(1-n)||0,[100*e,100*a,100*i,100*n]}function s(t){return Z[JSON.stringify(t)]}function d(t){var e=t[0]/255,a=t[1]/255,i=t[2]/255;e=e>.04045?Math.pow((e+.055)/1.055,2.4):e/12.92,a=a>.04045?Math.pow((a+.055)/1.055,2.4):a/12.92,i=i>.04045?Math.pow((i+.055)/1.055,2.4):i/12.92;var n=.4124*e+.3576*a+.1805*i,o=.2126*e+.7152*a+.0722*i,r=.0193*e+.1192*a+.9505*i;return[100*n,100*o,100*r]}function u(t){var e,a,i,n=d(t),o=n[0],r=n[1],l=n[2];return o/=95.047,r/=100,l/=108.883,o=o>.008856?Math.pow(o,1/3):7.787*o+16/116,r=r>.008856?Math.pow(r,1/3):7.787*r+16/116,l=l>.008856?Math.pow(l,1/3):7.787*l+16/116,e=116*r-16,a=500*(o-r),i=200*(r-l),[e,a,i]}function c(t){return W(u(t))}function h(t){var e,a,i,n,o,r=t[0]/360,l=t[1]/100,s=t[2]/100;if(0==l)return o=255*s,[o,o,o];a=.5>s?s*(1+l):s+l-s*l,e=2*s-a,n=[0,0,0];for(var d=0;3>d;d++)i=r+1/3*-(d-1),0>i&&i++,i>1&&i--,o=1>6*i?e+6*(a-e)*i:1>2*i?a:2>3*i?e+(a-e)*(2/3-i)*6:e,n[d]=255*o;return n}function f(t){var e,a,i=t[0],n=t[1]/100,o=t[2]/100;return 0===o?[0,0,0]:(o*=2,n*=1>=o?o:2-o,a=(o+n)/2,e=2*n/(o+n),[i,100*e,100*a])}function p(t){return o(h(t))}function m(t){return l(h(t))}function v(t){return s(h(t))}function x(t){var e=t[0]/60,a=t[1]/100,i=t[2]/100,n=Math.floor(e)%6,o=e-Math.floor(e),r=255*i*(1-a),l=255*i*(1-a*o),s=255*i*(1-a*(1-o)),i=255*i;switch(n){case 0:return[i,s,r];case 1:return[l,i,r];case 2:return[r,i,s];case 3:return[r,l,i];case 4:return[s,r,i];case 5:return[i,r,l]}}function y(t){var e,a,i=t[0],n=t[1]/100,o=t[2]/100;return a=(2-n)*o,e=n*o,e/=1>=a?a:2-a,e=e||0,a/=2,[i,100*e,100*a]}function k(t){return o(x(t))}function S(t){return l(x(t))}function w(t){return s(x(t))}function C(t){var e,a,i,n,o=t[0]/360,l=t[1]/100,s=t[2]/100,d=l+s;switch(d>1&&(l/=d,s/=d),e=Math.floor(6*o),a=1-s,i=6*o-e,0!=(1&e)&&(i=1-i),n=l+i*(a-l),e){default:case 6:case 0:r=a,g=n,b=l;break;case 1:r=n,g=a,b=l;break;case 2:r=l,g=a,b=n;break;case 3:r=l,g=n,b=a;break;case 4:r=n,g=l,b=a;break;case 5:r=a,g=l,b=n}return[255*r,255*g,255*b]}function M(t){return i(C(t))}function D(t){return n(C(t))}function A(t){return l(C(t))}function I(t){return s(C(t))}function T(t){var e,a,i,n=t[0]/100,o=t[1]/100,r=t[2]/100,l=t[3]/100;return e=1-Math.min(1,n*(1-l)+l),a=1-Math.min(1,o*(1-l)+l),i=1-Math.min(1,r*(1-l)+l),[255*e,255*a,255*i]}function F(t){return i(T(t))}function P(t){return n(T(t))}function R(t){return o(T(t))}function _(t){return s(T(t))}function V(t){var e,a,i,n=t[0]/100,o=t[1]/100,r=t[2]/100;return e=3.2406*n+-1.5372*o+r*-.4986,a=n*-.9689+1.8758*o+.0415*r,i=.0557*n+o*-.204+1.057*r,e=e>.0031308?1.055*Math.pow(e,1/2.4)-.055:e=12.92*e,a=a>.0031308?1.055*Math.pow(a,1/2.4)-.055:a=12.92*a,i=i>.0031308?1.055*Math.pow(i,1/2.4)-.055:i=12.92*i,e=Math.min(Math.max(0,e),1),a=Math.min(Math.max(0,a),1),i=Math.min(Math.max(0,i),1),[255*e,255*a,255*i]}function L(t){var e,a,i,n=t[0],o=t[1],r=t[2];return n/=95.047,o/=100,r/=108.883,n=n>.008856?Math.pow(n,1/3):7.787*n+16/116,o=o>.008856?Math.pow(o,1/3):7.787*o+16/116,r=r>.008856?Math.pow(r,1/3):7.787*r+16/116,e=116*o-16,a=500*(n-o),i=200*(o-r),[e,a,i]}function O(t){return W(L(t))}function B(t){var e,a,i,n,o=t[0],r=t[1],l=t[2];return 8>=o?(a=100*o/903.3,n=7.787*(a/100)+16/116):(a=100*Math.pow((o+16)/116,3),n=Math.pow(a/100,1/3)),e=.008856>=e/95.047?e=95.047*(r/500+n-16/116)/7.787:95.047*Math.pow(r/500+n,3),i=.008859>=i/108.883?i=108.883*(n-l/200-16/116)/7.787:108.883*Math.pow(n-l/200,3),[e,a,i]}function W(t){var e,a,i,n=t[0],o=t[1],r=t[2];return e=Math.atan2(r,o),a=360*e/2/Math.PI,0>a&&(a+=360),i=Math.sqrt(o*o+r*r),[n,i,a]}function z(t){return V(B(t))}function N(t){var e,a,i,n=t[0],o=t[1],r=t[2];return i=r/360*2*Math.PI,e=o*Math.cos(i),a=o*Math.sin(i),[n,e,a]}function H(t){return B(N(t))}function E(t){return z(N(t))}function U(t){return Q[t]}function q(t){return i(U(t))}function j(t){return n(U(t))}function Y(t){return o(U(t))}function J(t){return l(U(t))}function X(t){return u(U(t))}function G(t){return d(U(t))}e.exports={rgb2hsl:i,rgb2hsv:n,rgb2hwb:o,rgb2cmyk:l,rgb2keyword:s,rgb2xyz:d,rgb2lab:u,rgb2lch:c,hsl2rgb:h,hsl2hsv:f,hsl2hwb:p,hsl2cmyk:m,hsl2keyword:v,hsv2rgb:x,hsv2hsl:y,hsv2hwb:k,hsv2cmyk:S,hsv2keyword:w,hwb2rgb:C,hwb2hsl:M,hwb2hsv:D,hwb2cmyk:A,hwb2keyword:I,cmyk2rgb:T,cmyk2hsl:F,cmyk2hsv:P,cmyk2hwb:R,cmyk2keyword:_,keyword2rgb:U,keyword2hsl:q,keyword2hsv:j,keyword2hwb:Y,keyword2cmyk:J,keyword2lab:X,keyword2xyz:G,xyz2rgb:V,xyz2lab:L,xyz2lch:O,lab2xyz:B,lab2rgb:z,lab2lch:W,lch2lab:N,lch2xyz:H,lch2rgb:E};var Q={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]},Z={};for(var $ in Q)Z[JSON.stringify(Q[$])]=$},{}],5:[function(t,e,a){var i=t(4),n=function(){return new d};for(var o in i){n[o+"Raw"]=function(t){return function(e){return"number"==typeof e&&(e=Array.prototype.slice.call(arguments)),i[t](e)}}(o);var r=/(\w+)2(\w+)/.exec(o),l=r[1],s=r[2];n[l]=n[l]||{},n[l][s]=n[o]=function(t){return function(e){"number"==typeof e&&(e=Array.prototype.slice.call(arguments));var a=i[t](e);if("string"==typeof a||void 0===a)return a;for(var n=0;n<a.length;n++)a[n]=Math.round(a[n]);return a}}(o)}var d=function(){this.convs={}};d.prototype.routeSpace=function(t,e){var a=e[0];return void 0===a?this.getValues(t):("number"==typeof a&&(a=Array.prototype.slice.call(e)),this.setValues(t,a))},d.prototype.setValues=function(t,e){return this.space=t,this.convs={},this.convs[t]=e,this},d.prototype.getValues=function(t){var e=this.convs[t];if(!e){var a=this.space,i=this.convs[a];e=n[a][t](i),this.convs[t]=e}return e},["rgb","hsl","hsv","cmyk","keyword"].forEach(function(t){d.prototype[t]=function(e){return this.routeSpace(t,arguments)}}),e.exports=n},{4:4}],6:[function(t,e,a){e.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},{}],7:[function(t,e,a){var i=t(27)();t(26)(i),t(22)(i),t(25)(i),t(21)(i),t(23)(i),t(24)(i),t(28)(i),t(32)(i),t(30)(i),t(31)(i),t(33)(i),t(29)(i),t(34)(i),t(35)(i),t(36)(i),t(37)(i),t(38)(i),t(41)(i),t(39)(i),t(40)(i),t(42)(i),t(43)(i),t(44)(i),t(15)(i),t(16)(i),t(17)(i),t(18)(i),t(19)(i),t(20)(i),t(8)(i),t(9)(i),t(10)(i),t(11)(i),t(12)(i),t(13)(i),t(14)(i),window.Chart=e.exports=i},{10:10,11:11,12:12,13:13,14:14,15:15,16:16,17:17,18:18,19:19,20:20,21:21,22:22,23:23,24:24,25:25,26:26,27:27,28:28,29:29,30:30,31:31,32:32,33:33,34:34,35:35,36:36,37:37,38:38,39:39,40:40,41:41,42:42,43:43,44:44,8:8,9:9}],8:[function(t,e,a){"use strict";e.exports=function(t){t.Bar=function(e,a){return a.type="bar",new t(e,a)}}},{}],9:[function(t,e,a){"use strict";e.exports=function(t){t.Bubble=function(e,a){return a.type="bubble",new t(e,a)}}},{}],10:[function(t,e,a){"use strict";e.exports=function(t){t.Doughnut=function(e,a){return a.type="doughnut",new t(e,a)}}},{}],11:[function(t,e,a){"use strict";e.exports=function(t){t.Line=function(e,a){return a.type="line",new t(e,a)}}},{}],12:[function(t,e,a){"use strict";e.exports=function(t){t.PolarArea=function(e,a){return a.type="polarArea",new t(e,a)}}},{}],13:[function(t,e,a){"use strict";e.exports=function(t){t.Radar=function(e,a){return a.options=t.helpers.configMerge({aspectRatio:1},a.options),a.type="radar",new t(e,a)}}},{}],14:[function(t,e,a){"use strict";e.exports=function(t){var e={hover:{mode:"single"},scales:{xAxes:[{type:"linear",position:"bottom",id:"x-axis-1"}],yAxes:[{type:"linear",position:"left",id:"y-axis-1"}]},tooltips:{callbacks:{title:function(){return""},label:function(t){return"("+t.xLabel+", "+t.yLabel+")"}}}};t.defaults.scatter=e,t.controllers.scatter=t.controllers.line,t.Scatter=function(e,a){return a.type="scatter",new t(e,a)}}},{}],15:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.bar={hover:{mode:"label"},scales:{xAxes:[{type:"category",categoryPercentage:.8,barPercentage:.9,gridLines:{offsetGridLines:!0}}],yAxes:[{type:"linear"}]}},t.controllers.bar=t.DatasetController.extend({dataElementType:t.elements.Rectangle,initialize:function(e,a){t.DatasetController.prototype.initialize.call(this,e,a),this.getMeta().bar=!0},getBarCount:function(){var t=this,a=0;return e.each(t.chart.data.datasets,function(e,i){var n=t.chart.getDatasetMeta(i);n.bar&&t.chart.isDatasetVisible(i)&&++a},t),a},update:function(t){var a=this;e.each(a.getMeta().data,function(e,i){a.updateElement(e,i,t)},a)},updateElement:function(t,a,i){var n=this,o=n.getMeta(),r=n.getScaleForId(o.xAxisID),l=n.getScaleForId(o.yAxisID),s=l.getBasePixel(),d=n.chart.options.elements.rectangle,u=t.custom||{},c=n.getDataset();e.extend(t,{_xScale:r,_yScale:l,_datasetIndex:n.index,_index:a,_model:{x:n.calculateBarX(a,n.index),y:i?s:n.calculateBarY(a,n.index),label:n.chart.data.labels[a],datasetLabel:c.label,base:i?s:n.calculateBarBase(n.index,a),width:n.calculateBarWidth(a),backgroundColor:u.backgroundColor?u.backgroundColor:e.getValueAtIndexOrDefault(c.backgroundColor,a,d.backgroundColor),borderSkipped:u.borderSkipped?u.borderSkipped:d.borderSkipped,borderColor:u.borderColor?u.borderColor:e.getValueAtIndexOrDefault(c.borderColor,a,d.borderColor),borderWidth:u.borderWidth?u.borderWidth:e.getValueAtIndexOrDefault(c.borderWidth,a,d.borderWidth)}}),t.pivot()},calculateBarBase:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.yAxisID),o=0;if(n.options.stacked){for(var r=a.chart,l=r.data.datasets,s=Number(l[t].data[e]),d=0;t>d;d++){var u=l[d],c=r.getDatasetMeta(d);if(c.bar&&c.yAxisID===n.id&&r.isDatasetVisible(d)){var h=Number(u.data[e]);o+=0>s?Math.min(h,0):Math.max(h,0)}}return n.getPixelForValue(o)}return n.getBasePixel()},getRuler:function(t){var e,a=this,i=a.getMeta(),n=a.getScaleForId(i.xAxisID),o=a.getBarCount();e="category"===n.options.type?n.getPixelForTick(t+1)-n.getPixelForTick(t):n.width/n.ticks.length;var r=e*n.options.categoryPercentage,l=(e-e*n.options.categoryPercentage)/2,s=r/o;if(n.ticks.length!==a.chart.data.labels.length){var d=n.ticks.length/a.chart.data.labels.length;s*=d}var u=s*n.options.barPercentage,c=s-s*n.options.barPercentage;return{datasetCount:o,tickWidth:e,categoryWidth:r,categorySpacing:l,fullBarWidth:s,barWidth:u,barSpacing:c}},calculateBarWidth:function(t){var e=this.getScaleForId(this.getMeta().xAxisID);if(e.options.barThickness)return e.options.barThickness;var a=this.getRuler(t);return e.options.stacked?a.categoryWidth:a.barWidth},getBarIndex:function(t){var e,a,i=0;for(a=0;t>a;++a)e=this.chart.getDatasetMeta(a),e.bar&&this.chart.isDatasetVisible(a)&&++i;return i},calculateBarX:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.xAxisID),o=a.getBarIndex(e),r=a.getRuler(t),l=n.getPixelForValue(null,t,e,a.chart.isCombo);return l-=a.chart.isCombo?r.tickWidth/2:0,n.options.stacked?l+r.categoryWidth/2+r.categorySpacing:l+r.barWidth/2+r.categorySpacing+r.barWidth*o+r.barSpacing/2+r.barSpacing*o},calculateBarY:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.yAxisID),o=Number(a.getDataset().data[t]);if(n.options.stacked){for(var r=0,l=0,s=0;e>s;s++){var d=a.chart.data.datasets[s],u=a.chart.getDatasetMeta(s);if(u.bar&&u.yAxisID===n.id&&a.chart.isDatasetVisible(s)){var c=Number(d.data[t]);0>c?l+=c||0:r+=c||0}}return 0>o?n.getPixelForValue(l+o):n.getPixelForValue(r+o)}return n.getPixelForValue(o)},draw:function(t){var a=this,i=t||1;e.each(a.getMeta().data,function(t,e){var n=a.getDataset().data[e];null===n||void 0===n||isNaN(n)||t.transition(i).draw()},a)},setHoverStyle:function(t){var a=this.chart.data.datasets[t._datasetIndex],i=t._index,n=t.custom||{},o=t._model;o.backgroundColor=n.hoverBackgroundColor?n.hoverBackgroundColor:e.getValueAtIndexOrDefault(a.hoverBackgroundColor,i,e.getHoverColor(o.backgroundColor)),o.borderColor=n.hoverBorderColor?n.hoverBorderColor:e.getValueAtIndexOrDefault(a.hoverBorderColor,i,e.getHoverColor(o.borderColor)),o.borderWidth=n.hoverBorderWidth?n.hoverBorderWidth:e.getValueAtIndexOrDefault(a.hoverBorderWidth,i,o.borderWidth)},removeHoverStyle:function(t){var a=this.chart.data.datasets[t._datasetIndex],i=t._index,n=t.custom||{},o=t._model,r=this.chart.options.elements.rectangle;o.backgroundColor=n.backgroundColor?n.backgroundColor:e.getValueAtIndexOrDefault(a.backgroundColor,i,r.backgroundColor),o.borderColor=n.borderColor?n.borderColor:e.getValueAtIndexOrDefault(a.borderColor,i,r.borderColor),o.borderWidth=n.borderWidth?n.borderWidth:e.getValueAtIndexOrDefault(a.borderWidth,i,r.borderWidth)}}),t.defaults.horizontalBar={hover:{mode:"label"},scales:{xAxes:[{type:"linear",position:"bottom"}],yAxes:[{position:"left",type:"category",categoryPercentage:.8,barPercentage:.9,gridLines:{offsetGridLines:!0}}]},elements:{rectangle:{borderSkipped:"left"}},tooltips:{callbacks:{title:function(t,e){var a="";return t.length>0&&(t[0].yLabel?a=t[0].yLabel:e.labels.length>0&&t[0].index<e.labels.length&&(a=e.labels[t[0].index])),a},label:function(t,e){var a=e.datasets[t.datasetIndex].label||"";return a+": "+t.xLabel}}}},t.controllers.horizontalBar=t.controllers.bar.extend({updateElement:function(t,a,i){var n=this,o=n.getMeta(),r=n.getScaleForId(o.xAxisID),l=n.getScaleForId(o.yAxisID),s=r.getBasePixel(),d=t.custom||{},u=n.getDataset(),c=n.chart.options.elements.rectangle;e.extend(t,{_xScale:r,_yScale:l,_datasetIndex:n.index,_index:a,_model:{x:i?s:n.calculateBarX(a,n.index),y:n.calculateBarY(a,n.index),label:n.chart.data.labels[a],datasetLabel:u.label,base:i?s:n.calculateBarBase(n.index,a),height:n.calculateBarHeight(a),backgroundColor:d.backgroundColor?d.backgroundColor:e.getValueAtIndexOrDefault(u.backgroundColor,a,c.backgroundColor),borderSkipped:d.borderSkipped?d.borderSkipped:c.borderSkipped,borderColor:d.borderColor?d.borderColor:e.getValueAtIndexOrDefault(u.borderColor,a,c.borderColor),borderWidth:d.borderWidth?d.borderWidth:e.getValueAtIndexOrDefault(u.borderWidth,a,c.borderWidth)},draw:function(){function t(t){return s[(u+t)%4]}var e=this._chart.ctx,a=this._view,i=a.height/2,n=a.y-i,o=a.y+i,r=a.base-(a.base-a.x),l=a.borderWidth/2;a.borderWidth&&(n+=l,o-=l,r+=l),e.beginPath(),e.fillStyle=a.backgroundColor,e.strokeStyle=a.borderColor,e.lineWidth=a.borderWidth;var s=[[a.base,o],[a.base,n],[r,n],[r,o]],d=["bottom","left","top","right"],u=d.indexOf(a.borderSkipped,0);-1===u&&(u=0),e.moveTo.apply(e,t(0));for(var c=1;4>c;c++)e.lineTo.apply(e,t(c));e.fill(),a.borderWidth&&e.stroke()},inRange:function(t,e){var a=this._view,i=!1;return a&&(i=a.x<a.base?e>=a.y-a.height/2&&e<=a.y+a.height/2&&t>=a.x&&t<=a.base:e>=a.y-a.height/2&&e<=a.y+a.height/2&&t>=a.base&&t<=a.x),i}}),t.pivot()},calculateBarBase:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.xAxisID),o=0;
if(n.options.stacked){for(var r=a.chart,l=r.data.datasets,s=Number(l[t].data[e]),d=0;t>d;d++){var u=l[d],c=r.getDatasetMeta(d);if(c.bar&&c.xAxisID===n.id&&r.isDatasetVisible(d)){var h=Number(u.data[e]);o+=0>s?Math.min(h,0):Math.max(h,0)}}return n.getPixelForValue(o)}return n.getBasePixel()},getRuler:function(t){var e,a=this,i=a.getMeta(),n=a.getScaleForId(i.yAxisID),o=a.getBarCount();e="category"===n.options.type?n.getPixelForTick(t+1)-n.getPixelForTick(t):n.width/n.ticks.length;var r=e*n.options.categoryPercentage,l=(e-e*n.options.categoryPercentage)/2,s=r/o;if(n.ticks.length!==a.chart.data.labels.length){var d=n.ticks.length/a.chart.data.labels.length;s*=d}var u=s*n.options.barPercentage,c=s-s*n.options.barPercentage;return{datasetCount:o,tickHeight:e,categoryHeight:r,categorySpacing:l,fullBarHeight:s,barHeight:u,barSpacing:c}},calculateBarHeight:function(t){var e=this,a=e.getScaleForId(e.getMeta().yAxisID);if(a.options.barThickness)return a.options.barThickness;var i=e.getRuler(t);return a.options.stacked?i.categoryHeight:i.barHeight},calculateBarX:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.xAxisID),o=Number(a.getDataset().data[t]);if(n.options.stacked){for(var r=0,l=0,s=0;e>s;s++){var d=a.chart.data.datasets[s],u=a.chart.getDatasetMeta(s);if(u.bar&&u.xAxisID===n.id&&a.chart.isDatasetVisible(s)){var c=Number(d.data[t]);0>c?l+=c||0:r+=c||0}}return 0>o?n.getPixelForValue(l+o):n.getPixelForValue(r+o)}return n.getPixelForValue(o)},calculateBarY:function(t,e){var a=this,i=a.getMeta(),n=a.getScaleForId(i.yAxisID),o=a.getBarIndex(e),r=a.getRuler(t),l=n.getPixelForValue(null,t,e,a.chart.isCombo);return l-=a.chart.isCombo?r.tickHeight/2:0,n.options.stacked?l+r.categoryHeight/2+r.categorySpacing:l+r.barHeight/2+r.categorySpacing+r.barHeight*o+r.barSpacing/2+r.barSpacing*o}})}},{}],16:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.bubble={hover:{mode:"single"},scales:{xAxes:[{type:"linear",position:"bottom",id:"x-axis-0"}],yAxes:[{type:"linear",position:"left",id:"y-axis-0"}]},tooltips:{callbacks:{title:function(){return""},label:function(t,e){var a=e.datasets[t.datasetIndex].label||"",i=e.datasets[t.datasetIndex].data[t.index];return a+": ("+i.x+", "+i.y+", "+i.r+")"}}}},t.controllers.bubble=t.DatasetController.extend({dataElementType:t.elements.Point,update:function(t){var a=this,i=a.getMeta(),n=i.data;e.each(n,function(e,i){a.updateElement(e,i,t)})},updateElement:function(a,i,n){var o=this,r=o.getMeta(),l=o.getScaleForId(r.xAxisID),s=o.getScaleForId(r.yAxisID),d=a.custom||{},u=o.getDataset(),c=u.data[i],h=o.chart.options.elements.point,f=o.index;e.extend(a,{_xScale:l,_yScale:s,_datasetIndex:f,_index:i,_model:{x:n?l.getPixelForDecimal(.5):l.getPixelForValue("object"==typeof c?c:NaN,i,f,o.chart.isCombo),y:n?s.getBasePixel():s.getPixelForValue(c,i,f),radius:n?0:d.radius?d.radius:o.getRadius(c),hitRadius:d.hitRadius?d.hitRadius:e.getValueAtIndexOrDefault(u.hitRadius,i,h.hitRadius)}}),t.DatasetController.prototype.removeHoverStyle.call(o,a,h);var g=a._model;g.skip=d.skip?d.skip:isNaN(g.x)||isNaN(g.y),a.pivot()},getRadius:function(t){return t.r||this.chart.options.elements.point.radius},setHoverStyle:function(a){var i=this;t.DatasetController.prototype.setHoverStyle.call(i,a);var n=i.chart.data.datasets[a._datasetIndex],o=a._index,r=a.custom||{},l=a._model;l.radius=r.hoverRadius?r.hoverRadius:e.getValueAtIndexOrDefault(n.hoverRadius,o,i.chart.options.elements.point.hoverRadius)+i.getRadius(n.data[o])},removeHoverStyle:function(e){var a=this;t.DatasetController.prototype.removeHoverStyle.call(a,e,a.chart.options.elements.point);var i=a.chart.data.datasets[e._datasetIndex].data[e._index],n=e.custom||{},o=e._model;o.radius=n.radius?n.radius:a.getRadius(i)}})}},{}],17:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=t.defaults;a.doughnut={animation:{animateRotate:!0,animateScale:!1},aspectRatio:1,hover:{mode:"single"},legendCallback:function(t){var e=[];e.push('<ul class="'+t.id+'-legend">');var a=t.data,i=a.datasets,n=a.labels;if(i.length)for(var o=0;o<i[0].data.length;++o)e.push('<li><span style="background-color:'+i[0].backgroundColor[o]+'"></span>'),n[o]&&e.push(n[o]),e.push("</li>");return e.push("</ul>"),e.join("")},legend:{labels:{generateLabels:function(t){var a=t.data;return a.labels.length&&a.datasets.length?a.labels.map(function(i,n){var o=t.getDatasetMeta(0),r=a.datasets[0],l=o.data[n],s=l&&l.custom||{},d=e.getValueAtIndexOrDefault,u=t.options.elements.arc,c=s.backgroundColor?s.backgroundColor:d(r.backgroundColor,n,u.backgroundColor),h=s.borderColor?s.borderColor:d(r.borderColor,n,u.borderColor),f=s.borderWidth?s.borderWidth:d(r.borderWidth,n,u.borderWidth);return{text:i,fillStyle:c,strokeStyle:h,lineWidth:f,hidden:isNaN(r.data[n])||o.data[n].hidden,index:n}}):[]}},onClick:function(t,e){var a,i,n,o=e.index,r=this.chart;for(a=0,i=(r.data.datasets||[]).length;i>a;++a)n=r.getDatasetMeta(a),n.data[o].hidden=!n.data[o].hidden;r.update()}},cutoutPercentage:50,rotation:Math.PI*-.5,circumference:2*Math.PI,tooltips:{callbacks:{title:function(){return""},label:function(t,e){return e.labels[t.index]+": "+e.datasets[t.datasetIndex].data[t.index]}}}},a.pie=e.clone(a.doughnut),e.extend(a.pie,{cutoutPercentage:0}),t.controllers.doughnut=t.controllers.pie=t.DatasetController.extend({dataElementType:t.elements.Arc,linkScales:e.noop,getRingIndex:function(t){for(var e=0,a=0;t>a;++a)this.chart.isDatasetVisible(a)&&++e;return e},update:function(t){var a=this,i=a.chart,n=i.chartArea,o=i.options,r=o.elements.arc,l=n.right-n.left-r.borderWidth,s=n.bottom-n.top-r.borderWidth,d=Math.min(l,s),u={x:0,y:0},c=a.getMeta(),h=o.cutoutPercentage,f=o.circumference;if(f<2*Math.PI){var g=o.rotation%(2*Math.PI);g+=2*Math.PI*(g>=Math.PI?-1:g<-Math.PI?1:0);var p=g+f,m={x:Math.cos(g),y:Math.sin(g)},b={x:Math.cos(p),y:Math.sin(p)},v=0>=g&&p>=0||g<=2*Math.PI&&2*Math.PI<=p,x=g<=.5*Math.PI&&.5*Math.PI<=p||g<=2.5*Math.PI&&2.5*Math.PI<=p,y=g<=-Math.PI&&-Math.PI<=p||g<=Math.PI&&Math.PI<=p,k=g<=.5*-Math.PI&&.5*-Math.PI<=p||g<=1.5*Math.PI&&1.5*Math.PI<=p,S=h/100,w={x:y?-1:Math.min(m.x*(m.x<0?1:S),b.x*(b.x<0?1:S)),y:k?-1:Math.min(m.y*(m.y<0?1:S),b.y*(b.y<0?1:S))},C={x:v?1:Math.max(m.x*(m.x>0?1:S),b.x*(b.x>0?1:S)),y:x?1:Math.max(m.y*(m.y>0?1:S),b.y*(b.y>0?1:S))},M={width:.5*(C.x-w.x),height:.5*(C.y-w.y)};d=Math.min(l/M.width,s/M.height),u={x:(C.x+w.x)*-.5,y:(C.y+w.y)*-.5}}i.borderWidth=a.getMaxBorderWidth(c.data),i.outerRadius=Math.max((d-i.borderWidth)/2,0),i.innerRadius=Math.max(h?i.outerRadius/100*h:1,0),i.radiusLength=(i.outerRadius-i.innerRadius)/i.getVisibleDatasetCount(),i.offsetX=u.x*i.outerRadius,i.offsetY=u.y*i.outerRadius,c.total=a.calculateTotal(),a.outerRadius=i.outerRadius-i.radiusLength*a.getRingIndex(a.index),a.innerRadius=a.outerRadius-i.radiusLength,e.each(c.data,function(e,i){a.updateElement(e,i,t)})},updateElement:function(t,a,i){var n=this,o=n.chart,r=o.chartArea,l=o.options,s=l.animation,d=(r.left+r.right)/2,u=(r.top+r.bottom)/2,c=l.rotation,h=l.rotation,f=n.getDataset(),g=i&&s.animateRotate?0:t.hidden?0:n.calculateCircumference(f.data[a])*(l.circumference/(2*Math.PI)),p=i&&s.animateScale?0:n.innerRadius,m=i&&s.animateScale?0:n.outerRadius,b=e.getValueAtIndexOrDefault;e.extend(t,{_datasetIndex:n.index,_index:a,_model:{x:d+o.offsetX,y:u+o.offsetY,startAngle:c,endAngle:h,circumference:g,outerRadius:m,innerRadius:p,label:b(f.label,a,o.data.labels[a])}});var v=t._model;this.removeHoverStyle(t),i&&s.animateRotate||(0===a?v.startAngle=l.rotation:v.startAngle=n.getMeta().data[a-1]._model.endAngle,v.endAngle=v.startAngle+v.circumference),t.pivot()},removeHoverStyle:function(e){t.DatasetController.prototype.removeHoverStyle.call(this,e,this.chart.options.elements.arc)},calculateTotal:function(){var t,a=this.getDataset(),i=this.getMeta(),n=0;return e.each(i.data,function(e,i){t=a.data[i],isNaN(t)||e.hidden||(n+=Math.abs(t))}),n},calculateCircumference:function(t){var e=this.getMeta().total;return e>0&&!isNaN(t)?2*Math.PI*(t/e):0},getMaxBorderWidth:function(t){for(var e,a,i=0,n=this.index,o=t.length,r=0;o>r;r++)e=t[r]._model?t[r]._model.borderWidth:0,a=t[r]._chart?t[r]._chart.config.data.datasets[n].hoverBorderWidth:0,i=e>i?e:i,i=a>i?a:i;return i}})}},{}],18:[function(t,e,a){"use strict";e.exports=function(t){function e(t,e){return a.getValueOrDefault(t.showLine,e.showLines)}var a=t.helpers;t.defaults.line={showLines:!0,spanGaps:!1,hover:{mode:"label"},scales:{xAxes:[{type:"category",id:"x-axis-0"}],yAxes:[{type:"linear",id:"y-axis-0"}]}},t.controllers.line=t.DatasetController.extend({datasetElementType:t.elements.Line,dataElementType:t.elements.Point,addElementAndReset:function(a){var i=this,n=i.chart.options,o=i.getMeta();t.DatasetController.prototype.addElementAndReset.call(i,a),e(i.getDataset(),n)&&0!==o.dataset._model.tension&&i.updateBezierControlPoints()},update:function(t){var i,n,o,r=this,l=r.getMeta(),s=l.dataset,d=l.data||[],u=r.chart.options,c=u.elements.line,h=r.getScaleForId(l.yAxisID),f=r.getDataset(),g=e(f,u);for(g&&(o=s.custom||{},void 0!==f.tension&&void 0===f.lineTension&&(f.lineTension=f.tension),s._scale=h,s._datasetIndex=r.index,s._children=d,s._model={spanGaps:f.spanGaps?f.spanGaps:u.spanGaps,tension:o.tension?o.tension:a.getValueOrDefault(f.lineTension,c.tension),backgroundColor:o.backgroundColor?o.backgroundColor:f.backgroundColor||c.backgroundColor,borderWidth:o.borderWidth?o.borderWidth:f.borderWidth||c.borderWidth,borderColor:o.borderColor?o.borderColor:f.borderColor||c.borderColor,borderCapStyle:o.borderCapStyle?o.borderCapStyle:f.borderCapStyle||c.borderCapStyle,borderDash:o.borderDash?o.borderDash:f.borderDash||c.borderDash,borderDashOffset:o.borderDashOffset?o.borderDashOffset:f.borderDashOffset||c.borderDashOffset,borderJoinStyle:o.borderJoinStyle?o.borderJoinStyle:f.borderJoinStyle||c.borderJoinStyle,fill:o.fill?o.fill:void 0!==f.fill?f.fill:c.fill,steppedLine:o.steppedLine?o.steppedLine:a.getValueOrDefault(f.steppedLine,c.stepped),scaleTop:h.top,scaleBottom:h.bottom,scaleZero:h.getBasePixel()},s.pivot()),i=0,n=d.length;n>i;++i)r.updateElement(d[i],i,t);for(g&&0!==s._model.tension&&r.updateBezierControlPoints(),i=0,n=d.length;n>i;++i)d[i].pivot()},getPointBackgroundColor:function(t,e){var i=this.chart.options.elements.point.backgroundColor,n=this.getDataset(),o=t.custom||{};return o.backgroundColor?i=o.backgroundColor:n.pointBackgroundColor?i=a.getValueAtIndexOrDefault(n.pointBackgroundColor,e,i):n.backgroundColor&&(i=n.backgroundColor),i},getPointBorderColor:function(t,e){var i=this.chart.options.elements.point.borderColor,n=this.getDataset(),o=t.custom||{};return o.borderColor?i=o.borderColor:n.pointBorderColor?i=a.getValueAtIndexOrDefault(n.pointBorderColor,e,i):n.borderColor&&(i=n.borderColor),i},getPointBorderWidth:function(t,e){var i=this.chart.options.elements.point.borderWidth,n=this.getDataset(),o=t.custom||{};return o.borderWidth?i=o.borderWidth:n.pointBorderWidth?i=a.getValueAtIndexOrDefault(n.pointBorderWidth,e,i):n.borderWidth&&(i=n.borderWidth),i},updateElement:function(t,e,i){var n,o,r=this,l=r.getMeta(),s=t.custom||{},d=r.getDataset(),u=r.index,c=d.data[e],h=r.getScaleForId(l.yAxisID),f=r.getScaleForId(l.xAxisID),g=r.chart.options.elements.point;void 0!==d.radius&&void 0===d.pointRadius&&(d.pointRadius=d.radius),void 0!==d.hitRadius&&void 0===d.pointHitRadius&&(d.pointHitRadius=d.hitRadius),n=f.getPixelForValue("object"==typeof c?c:NaN,e,u,r.chart.isCombo),o=i?h.getBasePixel():r.calculatePointY(c,e,u),t._xScale=f,t._yScale=h,t._datasetIndex=u,t._index=e,t._model={x:n,y:o,skip:s.skip||isNaN(n)||isNaN(o),radius:s.radius||a.getValueAtIndexOrDefault(d.pointRadius,e,g.radius),pointStyle:s.pointStyle||a.getValueAtIndexOrDefault(d.pointStyle,e,g.pointStyle),backgroundColor:r.getPointBackgroundColor(t,e),borderColor:r.getPointBorderColor(t,e),borderWidth:r.getPointBorderWidth(t,e),tension:l.dataset._model?l.dataset._model.tension:0,steppedLine:l.dataset._model?l.dataset._model.steppedLine:!1,hitRadius:s.hitRadius||a.getValueAtIndexOrDefault(d.pointHitRadius,e,g.hitRadius)}},calculatePointY:function(t,e,a){var i,n,o,r=this,l=r.chart,s=r.getMeta(),d=r.getScaleForId(s.yAxisID),u=0,c=0;if(d.options.stacked){for(i=0;a>i;i++)if(n=l.data.datasets[i],o=l.getDatasetMeta(i),"line"===o.type&&o.yAxisID===d.id&&l.isDatasetVisible(i)){var h=Number(d.getRightValue(n.data[e]));0>h?c+=h||0:u+=h||0}var f=Number(d.getRightValue(t));return 0>f?d.getPixelForValue(c+f):d.getPixelForValue(u+f)}return d.getPixelForValue(t)},updateBezierControlPoints:function(){function t(t,e,a){return c?Math.max(Math.min(t,a),e):t}var e,i,n,o,r,l=this,s=l.getMeta(),d=l.chart.chartArea,u=(s.data||[]).filter(function(t){return!t._model.skip}),c=l.chart.options.elements.line.capBezierPoints;for(e=0,i=u.length;i>e;++e)n=u[e],o=n._model,r=a.splineCurve(a.previousItem(u,e)._model,o,a.nextItem(u,e)._model,s.dataset._model.tension),o.controlPointPreviousX=t(r.previous.x,d.left,d.right),o.controlPointPreviousY=t(r.previous.y,d.top,d.bottom),o.controlPointNextX=t(r.next.x,d.left,d.right),o.controlPointNextY=t(r.next.y,d.top,d.bottom)},draw:function(t){var a,i,n=this,o=n.getMeta(),r=o.data||[],l=t||1;for(a=0,i=r.length;i>a;++a)r[a].transition(l);for(e(n.getDataset(),n.chart.options)&&o.dataset.transition(l).draw(),a=0,i=r.length;i>a;++a)r[a].draw()},setHoverStyle:function(t){var e=this.chart.data.datasets[t._datasetIndex],i=t._index,n=t.custom||{},o=t._model;o.radius=n.hoverRadius||a.getValueAtIndexOrDefault(e.pointHoverRadius,i,this.chart.options.elements.point.hoverRadius),o.backgroundColor=n.hoverBackgroundColor||a.getValueAtIndexOrDefault(e.pointHoverBackgroundColor,i,a.getHoverColor(o.backgroundColor)),o.borderColor=n.hoverBorderColor||a.getValueAtIndexOrDefault(e.pointHoverBorderColor,i,a.getHoverColor(o.borderColor)),o.borderWidth=n.hoverBorderWidth||a.getValueAtIndexOrDefault(e.pointHoverBorderWidth,i,o.borderWidth)},removeHoverStyle:function(t){var e=this,i=e.chart.data.datasets[t._datasetIndex],n=t._index,o=t.custom||{},r=t._model;void 0!==i.radius&&void 0===i.pointRadius&&(i.pointRadius=i.radius),r.radius=o.radius||a.getValueAtIndexOrDefault(i.pointRadius,n,e.chart.options.elements.point.radius),r.backgroundColor=e.getPointBackgroundColor(t,n),r.borderColor=e.getPointBorderColor(t,n),r.borderWidth=e.getPointBorderWidth(t,n)}})}},{}],19:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.polarArea={scale:{type:"radialLinear",lineArc:!0,ticks:{beginAtZero:!0}},animation:{animateRotate:!0,animateScale:!0},startAngle:-.5*Math.PI,aspectRatio:1,legendCallback:function(t){var e=[];e.push('<ul class="'+t.id+'-legend">');var a=t.data,i=a.datasets,n=a.labels;if(i.length)for(var o=0;o<i[0].data.length;++o)e.push('<li><span style="background-color:'+i[0].backgroundColor[o]+'">'),n[o]&&e.push(n[o]),e.push("</span></li>");return e.push("</ul>"),e.join("")},legend:{labels:{generateLabels:function(t){var a=t.data;return a.labels.length&&a.datasets.length?a.labels.map(function(i,n){var o=t.getDatasetMeta(0),r=a.datasets[0],l=o.data[n],s=l.custom||{},d=e.getValueAtIndexOrDefault,u=t.options.elements.arc,c=s.backgroundColor?s.backgroundColor:d(r.backgroundColor,n,u.backgroundColor),h=s.borderColor?s.borderColor:d(r.borderColor,n,u.borderColor),f=s.borderWidth?s.borderWidth:d(r.borderWidth,n,u.borderWidth);return{text:i,fillStyle:c,strokeStyle:h,lineWidth:f,hidden:isNaN(r.data[n])||o.data[n].hidden,index:n}}):[]}},onClick:function(t,e){var a,i,n,o=e.index,r=this.chart;for(a=0,i=(r.data.datasets||[]).length;i>a;++a)n=r.getDatasetMeta(a),n.data[o].hidden=!n.data[o].hidden;r.update()}},tooltips:{callbacks:{title:function(){return""},label:function(t,e){return e.labels[t.index]+": "+t.yLabel}}}},t.controllers.polarArea=t.DatasetController.extend({dataElementType:t.elements.Arc,linkScales:e.noop,update:function(t){var a=this,i=a.chart,n=i.chartArea,o=a.getMeta(),r=i.options,l=r.elements.arc,s=Math.min(n.right-n.left,n.bottom-n.top);i.outerRadius=Math.max((s-l.borderWidth/2)/2,0),i.innerRadius=Math.max(r.cutoutPercentage?i.outerRadius/100*r.cutoutPercentage:1,0),i.radiusLength=(i.outerRadius-i.innerRadius)/i.getVisibleDatasetCount(),a.outerRadius=i.outerRadius-i.radiusLength*a.index,a.innerRadius=a.outerRadius-i.radiusLength,o.count=a.countVisibleElements(),e.each(o.data,function(e,i){a.updateElement(e,i,t)})},updateElement:function(t,a,i){for(var n=this,o=n.chart,r=n.getDataset(),l=o.options,s=l.animation,d=o.scale,u=e.getValueAtIndexOrDefault,c=o.data.labels,h=n.calculateCircumference(r.data[a]),f=d.xCenter,g=d.yCenter,p=0,m=n.getMeta(),b=0;a>b;++b)isNaN(r.data[b])||m.data[b].hidden||++p;var v=l.startAngle,x=t.hidden?0:d.getDistanceFromCenterForValue(r.data[a]),y=v+h*p,k=y+(t.hidden?0:h),S=s.animateScale?0:d.getDistanceFromCenterForValue(r.data[a]);e.extend(t,{_datasetIndex:n.index,_index:a,_scale:d,_model:{x:f,y:g,innerRadius:0,outerRadius:i?S:x,startAngle:i&&s.animateRotate?v:y,endAngle:i&&s.animateRotate?v:k,label:u(c,a,c[a])}}),n.removeHoverStyle(t),t.pivot()},removeHoverStyle:function(e){t.DatasetController.prototype.removeHoverStyle.call(this,e,this.chart.options.elements.arc)},countVisibleElements:function(){var t=this.getDataset(),a=this.getMeta(),i=0;return e.each(a.data,function(e,a){isNaN(t.data[a])||e.hidden||i++}),i},calculateCircumference:function(t){var e=this.getMeta().count;return e>0&&!isNaN(t)?2*Math.PI/e:0}})}},{}],20:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.radar={scale:{type:"radialLinear"},elements:{line:{tension:0}}},t.controllers.radar=t.DatasetController.extend({datasetElementType:t.elements.Line,dataElementType:t.elements.Point,linkScales:e.noop,addElementAndReset:function(e){t.DatasetController.prototype.addElementAndReset.call(this,e),this.updateBezierControlPoints()},update:function(t){var a=this,i=a.getMeta(),n=i.dataset,o=i.data,r=n.custom||{},l=a.getDataset(),s=a.chart.options.elements.line,d=a.chart.scale;void 0!==l.tension&&void 0===l.lineTension&&(l.lineTension=l.tension),e.extend(i.dataset,{_datasetIndex:a.index,_children:o,_loop:!0,_model:{tension:r.tension?r.tension:e.getValueOrDefault(l.lineTension,s.tension),backgroundColor:r.backgroundColor?r.backgroundColor:l.backgroundColor||s.backgroundColor,borderWidth:r.borderWidth?r.borderWidth:l.borderWidth||s.borderWidth,borderColor:r.borderColor?r.borderColor:l.borderColor||s.borderColor,fill:r.fill?r.fill:void 0!==l.fill?l.fill:s.fill,borderCapStyle:r.borderCapStyle?r.borderCapStyle:l.borderCapStyle||s.borderCapStyle,borderDash:r.borderDash?r.borderDash:l.borderDash||s.borderDash,borderDashOffset:r.borderDashOffset?r.borderDashOffset:l.borderDashOffset||s.borderDashOffset,borderJoinStyle:r.borderJoinStyle?r.borderJoinStyle:l.borderJoinStyle||s.borderJoinStyle,scaleTop:d.top,scaleBottom:d.bottom,scaleZero:d.getBasePosition()}}),i.dataset.pivot(),e.each(o,function(e,i){a.updateElement(e,i,t)},a),a.updateBezierControlPoints()},updateElement:function(t,a,i){var n=this,o=t.custom||{},r=n.getDataset(),l=n.chart.scale,s=n.chart.options.elements.point,d=l.getPointPositionForValue(a,r.data[a]);e.extend(t,{_datasetIndex:n.index,_index:a,_scale:l,_model:{x:i?l.xCenter:d.x,y:i?l.yCenter:d.y,tension:o.tension?o.tension:e.getValueOrDefault(r.tension,n.chart.options.elements.line.tension),radius:o.radius?o.radius:e.getValueAtIndexOrDefault(r.pointRadius,a,s.radius),backgroundColor:o.backgroundColor?o.backgroundColor:e.getValueAtIndexOrDefault(r.pointBackgroundColor,a,s.backgroundColor),borderColor:o.borderColor?o.borderColor:e.getValueAtIndexOrDefault(r.pointBorderColor,a,s.borderColor),borderWidth:o.borderWidth?o.borderWidth:e.getValueAtIndexOrDefault(r.pointBorderWidth,a,s.borderWidth),pointStyle:o.pointStyle?o.pointStyle:e.getValueAtIndexOrDefault(r.pointStyle,a,s.pointStyle),hitRadius:o.hitRadius?o.hitRadius:e.getValueAtIndexOrDefault(r.hitRadius,a,s.hitRadius)}}),t._model.skip=o.skip?o.skip:isNaN(t._model.x)||isNaN(t._model.y)},updateBezierControlPoints:function(){var t=this.chart.chartArea,a=this.getMeta();e.each(a.data,function(i,n){var o=i._model,r=e.splineCurve(e.previousItem(a.data,n,!0)._model,o,e.nextItem(a.data,n,!0)._model,o.tension);o.controlPointPreviousX=Math.max(Math.min(r.previous.x,t.right),t.left),o.controlPointPreviousY=Math.max(Math.min(r.previous.y,t.bottom),t.top),o.controlPointNextX=Math.max(Math.min(r.next.x,t.right),t.left),o.controlPointNextY=Math.max(Math.min(r.next.y,t.bottom),t.top),i.pivot()})},draw:function(t){var a=this.getMeta(),i=t||1;e.each(a.data,function(t){t.transition(i)}),a.dataset.transition(i).draw(),e.each(a.data,function(t){t.draw()})},setHoverStyle:function(t){var a=this.chart.data.datasets[t._datasetIndex],i=t.custom||{},n=t._index,o=t._model;o.radius=i.hoverRadius?i.hoverRadius:e.getValueAtIndexOrDefault(a.pointHoverRadius,n,this.chart.options.elements.point.hoverRadius),o.backgroundColor=i.hoverBackgroundColor?i.hoverBackgroundColor:e.getValueAtIndexOrDefault(a.pointHoverBackgroundColor,n,e.getHoverColor(o.backgroundColor)),o.borderColor=i.hoverBorderColor?i.hoverBorderColor:e.getValueAtIndexOrDefault(a.pointHoverBorderColor,n,e.getHoverColor(o.borderColor)),o.borderWidth=i.hoverBorderWidth?i.hoverBorderWidth:e.getValueAtIndexOrDefault(a.pointHoverBorderWidth,n,o.borderWidth)},removeHoverStyle:function(t){var a=this.chart.data.datasets[t._datasetIndex],i=t.custom||{},n=t._index,o=t._model,r=this.chart.options.elements.point;o.radius=i.radius?i.radius:e.getValueAtIndexOrDefault(a.radius,n,r.radius),o.backgroundColor=i.backgroundColor?i.backgroundColor:e.getValueAtIndexOrDefault(a.pointBackgroundColor,n,r.backgroundColor),o.borderColor=i.borderColor?i.borderColor:e.getValueAtIndexOrDefault(a.pointBorderColor,n,r.borderColor),o.borderWidth=i.borderWidth?i.borderWidth:e.getValueAtIndexOrDefault(a.pointBorderWidth,n,r.borderWidth)}})}},{}],21:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.global.animation={duration:1e3,easing:"easeOutQuart",onProgress:e.noop,onComplete:e.noop},t.Animation=t.Element.extend({currentStep:null,numSteps:60,easing:"",render:null,onAnimationProgress:null,onAnimationComplete:null}),t.animationService={frameDuration:17,animations:[],dropFrames:0,request:null,addAnimation:function(t,e,a,i){var n=this;i||(t.animating=!0);for(var o=0;o<n.animations.length;++o)if(n.animations[o].chartInstance===t)return void(n.animations[o].animationObject=e);n.animations.push({chartInstance:t,animationObject:e}),1===n.animations.length&&n.requestAnimationFrame()},cancelAnimation:function(t){var a=e.findIndex(this.animations,function(e){return e.chartInstance===t});-1!==a&&(this.animations.splice(a,1),t.animating=!1)},requestAnimationFrame:function(){var t=this;null===t.request&&(t.request=e.requestAnimFrame.call(window,function(){t.request=null,t.startDigest()}))},startDigest:function(){var t=this,e=Date.now(),a=0;t.dropFrames>1&&(a=Math.floor(t.dropFrames),t.dropFrames=t.dropFrames%1);for(var i=0;i<t.animations.length;)null===t.animations[i].animationObject.currentStep&&(t.animations[i].animationObject.currentStep=0),t.animations[i].animationObject.currentStep+=1+a,t.animations[i].animationObject.currentStep>t.animations[i].animationObject.numSteps&&(t.animations[i].animationObject.currentStep=t.animations[i].animationObject.numSteps),t.animations[i].animationObject.render(t.animations[i].chartInstance,t.animations[i].animationObject),t.animations[i].animationObject.onAnimationProgress&&t.animations[i].animationObject.onAnimationProgress.call&&t.animations[i].animationObject.onAnimationProgress.call(t.animations[i].chartInstance,t.animations[i]),t.animations[i].animationObject.currentStep===t.animations[i].animationObject.numSteps?(t.animations[i].animationObject.onAnimationComplete&&t.animations[i].animationObject.onAnimationComplete.call&&t.animations[i].animationObject.onAnimationComplete.call(t.animations[i].chartInstance,t.animations[i]),t.animations[i].chartInstance.animating=!1,t.animations.splice(i,1)):++i;var n=Date.now(),o=(n-e)/t.frameDuration;t.dropFrames+=o,t.animations.length>0&&t.requestAnimationFrame()}}}},{}],22:[function(t,e,a){"use strict";e.exports=function(t){var e=t.canvasHelpers={};e.drawPoint=function(t,e,a,i,n){var o,r,l,s,d,u;if("object"==typeof e&&(o=e.toString(),"[object HTMLImageElement]"===o||"[object HTMLCanvasElement]"===o))return void t.drawImage(e,i-e.width/2,n-e.height/2);if(!(isNaN(a)||0>=a)){switch(e){default:t.beginPath(),t.arc(i,n,a,0,2*Math.PI),t.closePath(),t.fill();break;case"triangle":t.beginPath(),r=3*a/Math.sqrt(3),d=r*Math.sqrt(3)/2,t.moveTo(i-r/2,n+d/3),t.lineTo(i+r/2,n+d/3),t.lineTo(i,n-2*d/3),t.closePath(),t.fill();break;case"rect":u=1/Math.SQRT2*a,t.beginPath(),t.fillRect(i-u,n-u,2*u,2*u),t.strokeRect(i-u,n-u,2*u,2*u);break;case"rectRot":u=1/Math.SQRT2*a,t.beginPath(),t.moveTo(i-u,n),t.lineTo(i,n+u),t.lineTo(i+u,n),t.lineTo(i,n-u),t.closePath(),t.fill();break;case"cross":t.beginPath(),t.moveTo(i,n+a),t.lineTo(i,n-a),t.moveTo(i-a,n),t.lineTo(i+a,n),t.closePath();break;case"crossRot":t.beginPath(),l=Math.cos(Math.PI/4)*a,s=Math.sin(Math.PI/4)*a,t.moveTo(i-l,n-s),t.lineTo(i+l,n+s),t.moveTo(i-l,n+s),t.lineTo(i+l,n-s),t.closePath();break;case"star":t.beginPath(),t.moveTo(i,n+a),t.lineTo(i,n-a),t.moveTo(i-a,n),t.lineTo(i+a,n),l=Math.cos(Math.PI/4)*a,s=Math.sin(Math.PI/4)*a,t.moveTo(i-l,n-s),t.lineTo(i+l,n+s),t.moveTo(i-l,n+s),t.lineTo(i+l,n-s),t.closePath();break;case"line":t.beginPath(),t.moveTo(i-a,n),t.lineTo(i+a,n),t.closePath();break;case"dash":t.beginPath(),t.moveTo(i,n),t.lineTo(i+a,n),t.closePath()}t.stroke()}}}},{}],23:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.types={},t.instances={},t.controllers={},t.Controller=function(a){return this.chart=a,this.config=a.config,this.options=this.config.options=e.configMerge(t.defaults.global,t.defaults[this.config.type],this.config.options||{}),this.id=e.uid(),Object.defineProperty(this,"data",{get:function(){return this.config.data}}),t.instances[this.id]=this,this.options.responsive&&this.resize(!0),this.initialize(),this},e.extend(t.Controller.prototype,{initialize:function(){var e=this;return t.plugins.notify("beforeInit",[e]),e.bindEvents(),e.ensureScalesHaveIDs(),e.buildOrUpdateControllers(),e.buildScales(),e.updateLayout(),e.resetElements(),e.initToolTip(),e.update(),t.plugins.notify("afterInit",[e]),e},clear:function(){return e.clear(this.chart),this},stop:function(){return t.animationService.cancelAnimation(this),this},resize:function(a){var i=this,n=i.chart,o=n.canvas,r=e.getMaximumWidth(o),l=n.aspectRatio,s=i.options.maintainAspectRatio&&isNaN(l)===!1&&isFinite(l)&&0!==l?r/l:e.getMaximumHeight(o),d=n.width!==r||n.height!==s;if(!d)return i;o.width=n.width=r,o.height=n.height=s,e.retinaScale(n);var u={width:r,height:s};return t.plugins.notify("resize",[i,u]),i.options.onResize&&i.options.onResize(i,u),a||(i.stop(),i.update(i.options.responsiveAnimationDuration)),i},ensureScalesHaveIDs:function(){var t=this.options,a=t.scales||{},i=t.scale;e.each(a.xAxes,function(t,e){t.id=t.id||"x-axis-"+e}),e.each(a.yAxes,function(t,e){t.id=t.id||"y-axis-"+e}),i&&(i.id=i.id||"scale")},buildScales:function(){var a=this,i=a.options,n=a.scales={},o=[];i.scales&&(o=o.concat((i.scales.xAxes||[]).map(function(t){return{options:t,dtype:"category"}}),(i.scales.yAxes||[]).map(function(t){return{options:t,dtype:"linear"}}))),i.scale&&o.push({options:i.scale,dtype:"radialLinear",isDefault:!0}),e.each(o,function(i){var o=i.options,r=e.getValueOrDefault(o.type,i.dtype),l=t.scaleService.getScaleConstructor(r);if(l){var s=new l({id:o.id,options:o,ctx:a.chart.ctx,chart:a});n[s.id]=s,i.isDefault&&(a.scale=s)}}),t.scaleService.addScalesToLayout(this)},updateLayout:function(){t.layoutService.update(this,this.chart.width,this.chart.height)},buildOrUpdateControllers:function(){var a=this,i=[],n=[];if(e.each(a.data.datasets,function(e,o){var r=a.getDatasetMeta(o);r.type||(r.type=e.type||a.config.type),i.push(r.type),r.controller?r.controller.updateIndex(o):(r.controller=new t.controllers[r.type](a,o),n.push(r.controller))},a),i.length>1)for(var o=1;o<i.length;o++)if(i[o]!==i[o-1]){a.isCombo=!0;break}return n},resetElements:function(){var t=this;e.each(t.data.datasets,function(e,a){t.getDatasetMeta(a).controller.reset()},t)},update:function(a,i){var n=this;t.plugins.notify("beforeUpdate",[n]),n.tooltip._data=n.data;var o=n.buildOrUpdateControllers();e.each(n.data.datasets,function(t,e){n.getDatasetMeta(e).controller.buildOrUpdateElements()},n),t.layoutService.update(n,n.chart.width,n.chart.height),t.plugins.notify("afterScaleUpdate",[n]),e.each(o,function(t){t.reset()}),n.updateDatasets(),t.plugins.notify("afterUpdate",[n]),n.render(a,i)},updateDatasets:function(){var e,a,i=this;if(t.plugins.notify("beforeDatasetsUpdate",[i])){for(e=0,a=i.data.datasets.length;a>e;++e)i.getDatasetMeta(e).controller.update();t.plugins.notify("afterDatasetsUpdate",[i])}},render:function(a,i){var n=this;t.plugins.notify("beforeRender",[n]);var o=n.options.animation;if(o&&("undefined"!=typeof a&&0!==a||"undefined"==typeof a&&0!==o.duration)){var r=new t.Animation;r.numSteps=(a||o.duration)/16.66,r.easing=o.easing,r.render=function(t,a){var i=e.easingEffects[a.easing],n=a.currentStep/a.numSteps,o=i(n);t.draw(o,n,a.currentStep)},r.onAnimationProgress=o.onProgress,r.onAnimationComplete=o.onComplete,t.animationService.addAnimation(n,r,a,i)}else n.draw(),o&&o.onComplete&&o.onComplete.call&&o.onComplete.call(n);return n},draw:function(a){var i=this,n=a||1;i.clear(),t.plugins.notify("beforeDraw",[i,n]),e.each(i.boxes,function(t){t.draw(i.chartArea)},i),i.scale&&i.scale.draw(),t.plugins.notify("beforeDatasetsDraw",[i,n]),e.each(i.data.datasets,function(t,e){i.isDatasetVisible(e)&&i.getDatasetMeta(e).controller.draw(a)},i,!0),t.plugins.notify("afterDatasetsDraw",[i,n]),i.tooltip.transition(n).draw(),t.plugins.notify("afterDraw",[i,n])},getElementAtEvent:function(t){var a=this,i=e.getRelativePosition(t,a.chart),n=[];return e.each(a.data.datasets,function(t,o){if(a.isDatasetVisible(o)){var r=a.getDatasetMeta(o);e.each(r.data,function(t){return t.inRange(i.x,i.y)?(n.push(t),n):void 0})}}),n.slice(0,1)},getElementsAtEvent:function(t){var a=this,i=e.getRelativePosition(t,a.chart),n=[],o=function(){if(a.data.datasets)for(var t=0;t<a.data.datasets.length;t++){var e=a.getDatasetMeta(t);if(a.isDatasetVisible(t))for(var n=0;n<e.data.length;n++)if(e.data[n].inRange(i.x,i.y))return e.data[n]}}.call(a);return o?(e.each(a.data.datasets,function(t,e){if(a.isDatasetVisible(e)){var i=a.getDatasetMeta(e),r=i.data[o._index];r&&!r._view.skip&&n.push(r)}},a),n):n},getElementsAtXAxis:function(t){var a=this,i=e.getRelativePosition(t,a.chart),n=[],o=function(){if(a.data.datasets)for(var t=0;t<a.data.datasets.length;t++){var e=a.getDatasetMeta(t);if(a.isDatasetVisible(t))for(var n=0;n<e.data.length;n++)if(e.data[n].inLabelRange(i.x,i.y))return e.data[n]}}.call(a);return o?(e.each(a.data.datasets,function(t,i){if(a.isDatasetVisible(i)){var r=a.getDatasetMeta(i),l=e.findIndex(r.data,function(t){return o._model.x===t._model.x});-1===l||r.data[l]._view.skip||n.push(r.data[l])}},a),n):n},getElementsAtEventForMode:function(t,e){var a=this;switch(e){case"single":return a.getElementAtEvent(t);case"label":return a.getElementsAtEvent(t);case"dataset":return a.getDatasetAtEvent(t);case"x-axis":return a.getElementsAtXAxis(t);default:return t}},getDatasetAtEvent:function(t){var e=this.getElementAtEvent(t);return e.length>0&&(e=this.getDatasetMeta(e[0]._datasetIndex).data),e},getDatasetMeta:function(t){var e=this,a=e.data.datasets[t];a._meta||(a._meta={});var i=a._meta[e.id];return i||(i=a._meta[e.id]={type:null,data:[],dataset:null,controller:null,hidden:null,xAxisID:null,yAxisID:null}),i},getVisibleDatasetCount:function(){for(var t=0,e=0,a=this.data.datasets.length;a>e;++e)this.isDatasetVisible(e)&&t++;return t},isDatasetVisible:function(t){var e=this.getDatasetMeta(t);return"boolean"==typeof e.hidden?!e.hidden:!this.data.datasets[t].hidden;
},generateLegend:function(){return this.options.legendCallback(this)},destroy:function(){var a=this;a.stop(),a.clear(),e.unbindEvents(a,a.events),e.removeResizeListener(a.chart.canvas.parentNode);var i=a.chart.canvas;i.width=a.chart.width,i.height=a.chart.height,void 0!==a.chart.originalDevicePixelRatio&&a.chart.ctx.scale(1/a.chart.originalDevicePixelRatio,1/a.chart.originalDevicePixelRatio),i.style.width=a.chart.originalCanvasStyleWidth,i.style.height=a.chart.originalCanvasStyleHeight,t.plugins.notify("destroy",[a]),delete t.instances[a.id]},toBase64Image:function(){return this.chart.canvas.toDataURL.apply(this.chart.canvas,arguments)},initToolTip:function(){var e=this;e.tooltip=new t.Tooltip({_chart:e.chart,_chartInstance:e,_data:e.data,_options:e.options.tooltips},e)},bindEvents:function(){var t=this;e.bindEvents(t,t.options.events,function(e){t.eventHandler(e)})},updateHoverStyle:function(t,e,a){var i,n,o,r=a?"setHoverStyle":"removeHoverStyle";switch(e){case"single":t=[t[0]];break;case"label":case"dataset":case"x-axis":break;default:return}for(n=0,o=t.length;o>n;++n)i=t[n],i&&this.getDatasetMeta(i._datasetIndex).controller[r](i)},eventHandler:function(t){var a=this,i=a.tooltip,n=a.options||{},o=n.hover,r=n.tooltips;return a.lastActive=a.lastActive||[],a.lastTooltipActive=a.lastTooltipActive||[],"mouseout"===t.type?(a.active=[],a.tooltipActive=[]):(a.active=a.getElementsAtEventForMode(t,o.mode),a.tooltipActive=a.getElementsAtEventForMode(t,r.mode)),o.onHover&&o.onHover.call(a,a.active),("mouseup"===t.type||"click"===t.type)&&(n.onClick&&n.onClick.call(a,t,a.active),a.legend&&a.legend.handleEvent&&a.legend.handleEvent(t)),a.lastActive.length&&a.updateHoverStyle(a.lastActive,o.mode,!1),a.active.length&&o.mode&&a.updateHoverStyle(a.active,o.mode,!0),(r.enabled||r.custom)&&(i.initialize(),i._active=a.tooltipActive,i.update(!0)),i.pivot(),a.animating||e.arrayEquals(a.active,a.lastActive)&&e.arrayEquals(a.tooltipActive,a.lastTooltipActive)||(a.stop(),(r.enabled||r.custom)&&i.update(!0),a.render(o.animationDuration,!0)),a.lastActive=a.active,a.lastTooltipActive=a.tooltipActive,a}})}},{}],24:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=e.noop;t.DatasetController=function(t,e){this.initialize.call(this,t,e)},e.extend(t.DatasetController.prototype,{datasetElementType:null,dataElementType:null,initialize:function(t,e){var a=this;a.chart=t,a.index=e,a.linkScales(),a.addElements()},updateIndex:function(t){this.index=t},linkScales:function(){var t=this,e=t.getMeta(),a=t.getDataset();null===e.xAxisID&&(e.xAxisID=a.xAxisID||t.chart.options.scales.xAxes[0].id),null===e.yAxisID&&(e.yAxisID=a.yAxisID||t.chart.options.scales.yAxes[0].id)},getDataset:function(){return this.chart.data.datasets[this.index]},getMeta:function(){return this.chart.getDatasetMeta(this.index)},getScaleForId:function(t){return this.chart.scales[t]},reset:function(){this.update(!0)},createMetaDataset:function(){var t=this,e=t.datasetElementType;return e&&new e({_chart:t.chart.chart,_datasetIndex:t.index})},createMetaData:function(t){var e=this,a=e.dataElementType;return a&&new a({_chart:e.chart.chart,_datasetIndex:e.index,_index:t})},addElements:function(){var t,e,a=this,i=a.getMeta(),n=a.getDataset().data||[],o=i.data;for(t=0,e=n.length;e>t;++t)o[t]=o[t]||a.createMetaData(i,t);i.dataset=i.dataset||a.createMetaDataset()},addElementAndReset:function(t){var e=this,a=e.createMetaData(t);e.getMeta().data.splice(t,0,a),e.updateElement(a,t,!0)},buildOrUpdateElements:function(){var t=this.getMeta(),e=t.data,a=this.getDataset().data.length,i=e.length;if(i>a)e.splice(a,i-a);else if(a>i)for(var n=i;a>n;++n)this.addElementAndReset(n)},update:a,draw:function(t){var a=t||1;e.each(this.getMeta().data,function(t){t.transition(a).draw()})},removeHoverStyle:function(t,a){var i=this.chart.data.datasets[t._datasetIndex],n=t._index,o=t.custom||{},r=e.getValueAtIndexOrDefault,l=t._model;l.backgroundColor=o.backgroundColor?o.backgroundColor:r(i.backgroundColor,n,a.backgroundColor),l.borderColor=o.borderColor?o.borderColor:r(i.borderColor,n,a.borderColor),l.borderWidth=o.borderWidth?o.borderWidth:r(i.borderWidth,n,a.borderWidth)},setHoverStyle:function(t){var a=this.chart.data.datasets[t._datasetIndex],i=t._index,n=t.custom||{},o=e.getValueAtIndexOrDefault,r=e.getHoverColor,l=t._model;l.backgroundColor=n.hoverBackgroundColor?n.hoverBackgroundColor:o(a.hoverBackgroundColor,i,r(l.backgroundColor)),l.borderColor=n.hoverBorderColor?n.hoverBorderColor:o(a.hoverBorderColor,i,r(l.borderColor)),l.borderWidth=n.hoverBorderWidth?n.hoverBorderWidth:o(a.hoverBorderWidth,i,l.borderWidth)}}),t.DatasetController.extend=e.inherits}},{}],25:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.elements={},t.Element=function(t){e.extend(this,t),this.initialize.apply(this,arguments)},e.extend(t.Element.prototype,{initialize:function(){this.hidden=!1},pivot:function(){var t=this;return t._view||(t._view=e.clone(t._model)),t._start=e.clone(t._view),t},transition:function(t){var a=this;return a._view||(a._view=e.clone(a._model)),1===t?(a._view=a._model,a._start=null,a):(a._start||a.pivot(),e.each(a._model,function(i,n){if("_"===n[0]);else if(a._view.hasOwnProperty(n))if(i===a._view[n]);else if("string"==typeof i)try{var o=e.color(a._model[n]).mix(e.color(a._start[n]),t);a._view[n]=o.rgbString()}catch(r){a._view[n]=i}else if("number"==typeof i){var l=void 0!==a._start[n]&&isNaN(a._start[n])===!1?a._start[n]:0;a._view[n]=(a._model[n]-l)*t+l}else a._view[n]=i;else"number"!=typeof i||isNaN(a._view[n])?a._view[n]=i:a._view[n]=i*t},a),a)},tooltipPosition:function(){return{x:this._model.x,y:this._model.y}},hasValue:function(){return e.isNumber(this._model.x)&&e.isNumber(this._model.y)}}),t.Element.extend=e.inherits}},{}],26:[function(t,e,a){"use strict";var i=t(3);e.exports=function(t){function e(t,e,a){var i;return"string"==typeof t?(i=parseInt(t,10),-1!=t.indexOf("%")&&(i=i/100*e.parentNode[a])):i=t,i}function a(t){return void 0!==t&&null!==t&&"none"!==t}function n(t,i,n){var o=document.defaultView,r=t.parentNode,l=o.getComputedStyle(t)[i],s=o.getComputedStyle(r)[i],d=a(l),u=a(s),c=Number.POSITIVE_INFINITY;return d||u?Math.min(d?e(l,t,n):c,u?e(s,r,n):c):"none"}var o=t.helpers={};o.each=function(t,e,a,i){var n,r;if(o.isArray(t))if(r=t.length,i)for(n=r-1;n>=0;n--)e.call(a,t[n],n);else for(n=0;r>n;n++)e.call(a,t[n],n);else if("object"==typeof t){var l=Object.keys(t);for(r=l.length,n=0;r>n;n++)e.call(a,t[l[n]],l[n])}},o.clone=function(t){var e={};return o.each(t,function(t,a){o.isArray(t)?e[a]=t.slice(0):"object"==typeof t&&null!==t?e[a]=o.clone(t):e[a]=t}),e},o.extend=function(t){for(var e=function(e,a){t[a]=e},a=1,i=arguments.length;i>a;a++)o.each(arguments[a],e);return t},o.configMerge=function(e){var a=o.clone(e);return o.each(Array.prototype.slice.call(arguments,1),function(e){o.each(e,function(e,i){if("scales"===i)a[i]=o.scaleMerge(a.hasOwnProperty(i)?a[i]:{},e);else if("scale"===i)a[i]=o.configMerge(a.hasOwnProperty(i)?a[i]:{},t.scaleService.getScaleDefaults(e.type),e);else if(a.hasOwnProperty(i)&&o.isArray(a[i])&&o.isArray(e)){var n=a[i];o.each(e,function(t,e){e<n.length?"object"==typeof n[e]&&null!==n[e]&&"object"==typeof t&&null!==t?n[e]=o.configMerge(n[e],t):n[e]=t:n.push(t)})}else a.hasOwnProperty(i)&&"object"==typeof a[i]&&null!==a[i]&&"object"==typeof e?a[i]=o.configMerge(a[i],e):a[i]=e})}),a},o.scaleMerge=function(e,a){var i=o.clone(e);return o.each(a,function(e,a){"xAxes"===a||"yAxes"===a?i.hasOwnProperty(a)?o.each(e,function(e,n){var r=o.getValueOrDefault(e.type,"xAxes"===a?"category":"linear"),l=t.scaleService.getScaleDefaults(r);n>=i[a].length||!i[a][n].type?i[a].push(o.configMerge(l,e)):e.type&&e.type!==i[a][n].type?i[a][n]=o.configMerge(i[a][n],l,e):i[a][n]=o.configMerge(i[a][n],e)}):(i[a]=[],o.each(e,function(e){var n=o.getValueOrDefault(e.type,"xAxes"===a?"category":"linear");i[a].push(o.configMerge(t.scaleService.getScaleDefaults(n),e))})):i.hasOwnProperty(a)&&"object"==typeof i[a]&&null!==i[a]&&"object"==typeof e?i[a]=o.configMerge(i[a],e):i[a]=e}),i},o.getValueAtIndexOrDefault=function(t,e,a){return void 0===t||null===t?a:o.isArray(t)?e<t.length?t[e]:a:t},o.getValueOrDefault=function(t,e){return void 0===t?e:t},o.indexOf=Array.prototype.indexOf?function(t,e){return t.indexOf(e)}:function(t,e){for(var a=0,i=t.length;i>a;++a)if(t[a]===e)return a;return-1},o.where=function(t,e){if(o.isArray(t)&&Array.prototype.filter)return t.filter(e);var a=[];return o.each(t,function(t){e(t)&&a.push(t)}),a},o.findIndex=Array.prototype.findIndex?function(t,e,a){return t.findIndex(e,a)}:function(t,e,a){a=void 0===a?t:a;for(var i=0,n=t.length;n>i;++i)if(e.call(a,t[i],i,t))return i;return-1},o.findNextWhere=function(t,e,a){(void 0===a||null===a)&&(a=-1);for(var i=a+1;i<t.length;i++){var n=t[i];if(e(n))return n}},o.findPreviousWhere=function(t,e,a){(void 0===a||null===a)&&(a=t.length);for(var i=a-1;i>=0;i--){var n=t[i];if(e(n))return n}},o.inherits=function(t){var e=this,a=t&&t.hasOwnProperty("constructor")?t.constructor:function(){return e.apply(this,arguments)},i=function(){this.constructor=a};return i.prototype=e.prototype,a.prototype=new i,a.extend=o.inherits,t&&o.extend(a.prototype,t),a.__super__=e.prototype,a},o.noop=function(){},o.uid=function(){var t=0;return function(){return t++}}(),o.isNumber=function(t){return!isNaN(parseFloat(t))&&isFinite(t)},o.almostEquals=function(t,e,a){return Math.abs(t-e)<a},o.max=function(t){return t.reduce(function(t,e){return isNaN(e)?t:Math.max(t,e)},Number.NEGATIVE_INFINITY)},o.min=function(t){return t.reduce(function(t,e){return isNaN(e)?t:Math.min(t,e)},Number.POSITIVE_INFINITY)},o.sign=Math.sign?function(t){return Math.sign(t)}:function(t){return t=+t,0===t||isNaN(t)?t:t>0?1:-1},o.log10=Math.log10?function(t){return Math.log10(t)}:function(t){return Math.log(t)/Math.LN10},o.toRadians=function(t){return t*(Math.PI/180)},o.toDegrees=function(t){return t*(180/Math.PI)},o.getAngleFromPoint=function(t,e){var a=e.x-t.x,i=e.y-t.y,n=Math.sqrt(a*a+i*i),o=Math.atan2(i,a);return o<-.5*Math.PI&&(o+=2*Math.PI),{angle:o,distance:n}},o.aliasPixel=function(t){return t%2===0?0:.5},o.splineCurve=function(t,e,a,i){var n=t.skip?e:t,o=e,r=a.skip?e:a,l=Math.sqrt(Math.pow(o.x-n.x,2)+Math.pow(o.y-n.y,2)),s=Math.sqrt(Math.pow(r.x-o.x,2)+Math.pow(r.y-o.y,2)),d=l/(l+s),u=s/(l+s);d=isNaN(d)?0:d,u=isNaN(u)?0:u;var c=i*d,h=i*u;return{previous:{x:o.x-c*(r.x-n.x),y:o.y-c*(r.y-n.y)},next:{x:o.x+h*(r.x-n.x),y:o.y+h*(r.y-n.y)}}},o.nextItem=function(t,e,a){return a?e>=t.length-1?t[0]:t[e+1]:e>=t.length-1?t[t.length-1]:t[e+1]},o.previousItem=function(t,e,a){return a?0>=e?t[t.length-1]:t[e-1]:0>=e?t[0]:t[e-1]},o.niceNum=function(t,e){var a,i=Math.floor(o.log10(t)),n=t/Math.pow(10,i);return a=e?1.5>n?1:3>n?2:7>n?5:10:1>=n?1:2>=n?2:5>=n?5:10,a*Math.pow(10,i)};var r=o.easingEffects={linear:function(t){return t},easeInQuad:function(t){return t*t},easeOutQuad:function(t){return-1*t*(t-2)},easeInOutQuad:function(t){return(t/=.5)<1?.5*t*t:-0.5*(--t*(t-2)-1)},easeInCubic:function(t){return t*t*t},easeOutCubic:function(t){return 1*((t=t/1-1)*t*t+1)},easeInOutCubic:function(t){return(t/=.5)<1?.5*t*t*t:.5*((t-=2)*t*t+2)},easeInQuart:function(t){return t*t*t*t},easeOutQuart:function(t){return-1*((t=t/1-1)*t*t*t-1)},easeInOutQuart:function(t){return(t/=.5)<1?.5*t*t*t*t:-0.5*((t-=2)*t*t*t-2)},easeInQuint:function(t){return 1*(t/=1)*t*t*t*t},easeOutQuint:function(t){return 1*((t=t/1-1)*t*t*t*t+1)},easeInOutQuint:function(t){return(t/=.5)<1?.5*t*t*t*t*t:.5*((t-=2)*t*t*t*t+2)},easeInSine:function(t){return-1*Math.cos(t/1*(Math.PI/2))+1},easeOutSine:function(t){return 1*Math.sin(t/1*(Math.PI/2))},easeInOutSine:function(t){return-0.5*(Math.cos(Math.PI*t/1)-1)},easeInExpo:function(t){return 0===t?1:1*Math.pow(2,10*(t/1-1))},easeOutExpo:function(t){return 1===t?1:1*(-Math.pow(2,-10*t/1)+1)},easeInOutExpo:function(t){return 0===t?0:1===t?1:(t/=.5)<1?.5*Math.pow(2,10*(t-1)):.5*(-Math.pow(2,-10*--t)+2)},easeInCirc:function(t){return t>=1?t:-1*(Math.sqrt(1-(t/=1)*t)-1)},easeOutCirc:function(t){return 1*Math.sqrt(1-(t=t/1-1)*t)},easeInOutCirc:function(t){return(t/=.5)<1?-0.5*(Math.sqrt(1-t*t)-1):.5*(Math.sqrt(1-(t-=2)*t)+1)},easeInElastic:function(t){var e=1.70158,a=0,i=1;return 0===t?0:1===(t/=1)?1:(a||(a=.3),i<Math.abs(1)?(i=1,e=a/4):e=a/(2*Math.PI)*Math.asin(1/i),-(i*Math.pow(2,10*(t-=1))*Math.sin((1*t-e)*(2*Math.PI)/a)))},easeOutElastic:function(t){var e=1.70158,a=0,i=1;return 0===t?0:1===(t/=1)?1:(a||(a=.3),i<Math.abs(1)?(i=1,e=a/4):e=a/(2*Math.PI)*Math.asin(1/i),i*Math.pow(2,-10*t)*Math.sin((1*t-e)*(2*Math.PI)/a)+1)},easeInOutElastic:function(t){var e=1.70158,a=0,i=1;return 0===t?0:2===(t/=.5)?1:(a||(a=1*(.3*1.5)),i<Math.abs(1)?(i=1,e=a/4):e=a/(2*Math.PI)*Math.asin(1/i),1>t?-.5*(i*Math.pow(2,10*(t-=1))*Math.sin((1*t-e)*(2*Math.PI)/a)):i*Math.pow(2,-10*(t-=1))*Math.sin((1*t-e)*(2*Math.PI)/a)*.5+1)},easeInBack:function(t){var e=1.70158;return 1*(t/=1)*t*((e+1)*t-e)},easeOutBack:function(t){var e=1.70158;return 1*((t=t/1-1)*t*((e+1)*t+e)+1)},easeInOutBack:function(t){var e=1.70158;return(t/=.5)<1?.5*(t*t*(((e*=1.525)+1)*t-e)):.5*((t-=2)*t*(((e*=1.525)+1)*t+e)+2)},easeInBounce:function(t){return 1-r.easeOutBounce(1-t)},easeOutBounce:function(t){return(t/=1)<1/2.75?1*(7.5625*t*t):2/2.75>t?1*(7.5625*(t-=1.5/2.75)*t+.75):2.5/2.75>t?1*(7.5625*(t-=2.25/2.75)*t+.9375):1*(7.5625*(t-=2.625/2.75)*t+.984375)},easeInOutBounce:function(t){return.5>t?.5*r.easeInBounce(2*t):.5*r.easeOutBounce(2*t-1)+.5}};o.requestAnimFrame=function(){return window.requestAnimationFrame||window.webkitRequestAnimationFrame||window.mozRequestAnimationFrame||window.oRequestAnimationFrame||window.msRequestAnimationFrame||function(t){return window.setTimeout(t,1e3/60)}}(),o.cancelAnimFrame=function(){return window.cancelAnimationFrame||window.webkitCancelAnimationFrame||window.mozCancelAnimationFrame||window.oCancelAnimationFrame||window.msCancelAnimationFrame||function(t){return window.clearTimeout(t,1e3/60)}}(),o.getRelativePosition=function(t,e){var a,i,n=t.originalEvent||t,r=t.currentTarget||t.srcElement,l=r.getBoundingClientRect(),s=n.touches;s&&s.length>0?(a=s[0].clientX,i=s[0].clientY):(a=n.clientX,i=n.clientY);var d=parseFloat(o.getStyle(r,"padding-left")),u=parseFloat(o.getStyle(r,"padding-top")),c=parseFloat(o.getStyle(r,"padding-right")),h=parseFloat(o.getStyle(r,"padding-bottom")),f=l.right-l.left-d-c,g=l.bottom-l.top-u-h;return a=Math.round((a-l.left-d)/f*r.width/e.currentDevicePixelRatio),i=Math.round((i-l.top-u)/g*r.height/e.currentDevicePixelRatio),{x:a,y:i}},o.addEvent=function(t,e,a){t.addEventListener?t.addEventListener(e,a):t.attachEvent?t.attachEvent("on"+e,a):t["on"+e]=a},o.removeEvent=function(t,e,a){t.removeEventListener?t.removeEventListener(e,a,!1):t.detachEvent?t.detachEvent("on"+e,a):t["on"+e]=o.noop},o.bindEvents=function(t,e,a){var i=t.events=t.events||{};o.each(e,function(e){i[e]=function(){a.apply(t,arguments)},o.addEvent(t.chart.canvas,e,i[e])})},o.unbindEvents=function(t,e){var a=t.chart.canvas;o.each(e,function(t,e){o.removeEvent(a,e,t)})},o.getConstraintWidth=function(t){return n(t,"max-width","clientWidth")},o.getConstraintHeight=function(t){return n(t,"max-height","clientHeight")},o.getMaximumWidth=function(t){var e=t.parentNode,a=parseInt(o.getStyle(e,"padding-left"))+parseInt(o.getStyle(e,"padding-right")),i=e.clientWidth-a,n=o.getConstraintWidth(t);return isNaN(n)?i:Math.min(i,n)},o.getMaximumHeight=function(t){var e=t.parentNode,a=parseInt(o.getStyle(e,"padding-top"))+parseInt(o.getStyle(e,"padding-bottom")),i=e.clientHeight-a,n=o.getConstraintHeight(t);return isNaN(n)?i:Math.min(i,n)},o.getStyle=function(t,e){return t.currentStyle?t.currentStyle[e]:document.defaultView.getComputedStyle(t,null).getPropertyValue(e)},o.retinaScale=function(t){var e=t.ctx,a=t.canvas,i=a.width,n=a.height,o=t.currentDevicePixelRatio=window.devicePixelRatio||1;1!==o&&(a.height=n*o,a.width=i*o,e.scale(o,o),t.originalDevicePixelRatio=t.originalDevicePixelRatio||o),a.style.width=i+"px",a.style.height=n+"px"},o.clear=function(t){t.ctx.clearRect(0,0,t.width,t.height)},o.fontString=function(t,e,a){return e+" "+t+"px "+a},o.longestText=function(t,e,a,i){i=i||{};var n=i.data=i.data||{},r=i.garbageCollect=i.garbageCollect||[];i.font!==e&&(n=i.data={},r=i.garbageCollect=[],i.font=e),t.font=e;var l=0;o.each(a,function(e){void 0!==e&&null!==e&&o.isArray(e)!==!0?l=o.measureText(t,n,r,l,e):o.isArray(e)&&o.each(e,function(e){void 0===e||null===e||o.isArray(e)||(l=o.measureText(t,n,r,l,e))})});var s=r.length/2;if(s>a.length){for(var d=0;s>d;d++)delete n[r[d]];r.splice(0,s)}return l},o.measureText=function(t,e,a,i,n){var o=e[n];return o||(o=e[n]=t.measureText(n).width,a.push(n)),o>i&&(i=o),i},o.numberOfLabelLines=function(t){var e=1;return o.each(t,function(t){o.isArray(t)&&t.length>e&&(e=t.length)}),e},o.drawRoundedRectangle=function(t,e,a,i,n,o){t.beginPath(),t.moveTo(e+o,a),t.lineTo(e+i-o,a),t.quadraticCurveTo(e+i,a,e+i,a+o),t.lineTo(e+i,a+n-o),t.quadraticCurveTo(e+i,a+n,e+i-o,a+n),t.lineTo(e+o,a+n),t.quadraticCurveTo(e,a+n,e,a+n-o),t.lineTo(e,a+o),t.quadraticCurveTo(e,a,e+o,a),t.closePath()},o.color=function(e){return i?i(e instanceof CanvasGradient?t.defaults.global.defaultColor:e):(console.log("Color.js not found!"),e)},o.addResizeListener=function(t,e){var a=document.createElement("iframe"),i="chartjs-hidden-iframe";a.classlist?a.classlist.add(i):a.setAttribute("class",i);var n=a.style;n.width="100%",n.display="block",n.border=0,n.height=0,n.margin=0,n.position="absolute",n.left=0,n.right=0,n.top=0,n.bottom=0,t.insertBefore(a,t.firstChild),(a.contentWindow||a).onresize=function(){e&&e()}},o.removeResizeListener=function(t){var e=t.querySelector(".chartjs-hidden-iframe");e&&e.parentNode.removeChild(e)},o.isArray=Array.isArray?function(t){return Array.isArray(t)}:function(t){return"[object Array]"===Object.prototype.toString.call(t)},o.arrayEquals=function(t,e){var a,i,n,r;if(!t||!e||t.length!=e.length)return!1;for(a=0,i=t.length;i>a;++a)if(n=t[a],r=e[a],n instanceof Array&&r instanceof Array){if(!o.arrayEquals(n,r))return!1}else if(n!=r)return!1;return!0},o.callCallback=function(t,e,a){t&&"function"==typeof t.call&&t.apply(a,e)},o.getHoverColor=function(t){return t instanceof CanvasPattern?t:o.color(t).saturate(.5).darken(.1).rgbString()}}},{3:3}],27:[function(t,e,a){"use strict";e.exports=function(){var t=function(e,a){var i=this,n=t.helpers;return i.config=a||{data:{datasets:[]}},e.length&&e[0].getContext&&(e=e[0]),e.getContext&&(e=e.getContext("2d")),i.ctx=e,i.canvas=e.canvas,e.canvas.style.display=e.canvas.style.display||"block",i.width=e.canvas.width||parseInt(n.getStyle(e.canvas,"width"),10)||n.getMaximumWidth(e.canvas),i.height=e.canvas.height||parseInt(n.getStyle(e.canvas,"height"),10)||n.getMaximumHeight(e.canvas),i.aspectRatio=i.width/i.height,(isNaN(i.aspectRatio)||isFinite(i.aspectRatio)===!1)&&(i.aspectRatio=void 0!==a.aspectRatio?a.aspectRatio:2),i.originalCanvasStyleWidth=e.canvas.style.width,i.originalCanvasStyleHeight=e.canvas.style.height,n.retinaScale(i),i.controller=new t.Controller(i),n.addResizeListener(e.canvas.parentNode,function(){i.controller&&i.controller.config.options.responsive&&i.controller.resize()}),i.controller?i.controller:i};return t.defaults={global:{responsive:!0,responsiveAnimationDuration:0,maintainAspectRatio:!0,events:["mousemove","mouseout","click","touchstart","touchmove"],hover:{onHover:null,mode:"single",animationDuration:400},onClick:null,defaultColor:"rgba(0,0,0,0.1)",defaultFontColor:"#666",defaultFontFamily:"'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",defaultFontSize:12,defaultFontStyle:"normal",showLines:!0,elements:{},legendCallback:function(t){var e=[];e.push('<ul class="'+t.id+'-legend">');for(var a=0;a<t.data.datasets.length;a++)e.push('<li><span style="background-color:'+t.data.datasets[a].backgroundColor+'"></span>'),t.data.datasets[a].label&&e.push(t.data.datasets[a].label),e.push("</li>");return e.push("</ul>"),e.join("")}}},t.Chart=t,t}},{}],28:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.layoutService={defaults:{},addBox:function(t,e){t.boxes||(t.boxes=[]),t.boxes.push(e)},removeBox:function(t,e){t.boxes&&t.boxes.splice(t.boxes.indexOf(e),1)},update:function(t,a,i){function n(t){var e,a=t.isHorizontal();a?(e=t.update(t.options.fullWidth?p:k,y),S-=e.height):(e=t.update(x,v),k-=e.width),w.push({horizontal:a,minSize:e,box:t})}function o(t){var a=e.findNextWhere(w,function(e){return e.box===t});if(a)if(t.isHorizontal()){var i={left:C,right:M,top:0,bottom:0};t.update(t.options.fullWidth?p:k,m/2,i)}else t.update(a.minSize.width,S)}function r(t){var a=e.findNextWhere(w,function(e){return e.box===t}),i={left:0,right:0,top:D,bottom:A};a&&t.update(a.minSize.width,S,i)}function l(t){t.isHorizontal()?(t.left=t.options.fullWidth?s:C,t.right=t.options.fullWidth?a-s:C+k,t.top=P,t.bottom=P+t.height,P=t.bottom):(t.left=F,t.right=F+t.width,t.top=D,t.bottom=D+S,F=t.right)}if(t){var s=0,d=0,u=e.where(t.boxes,function(t){return"left"===t.options.position}),c=e.where(t.boxes,function(t){return"right"===t.options.position}),h=e.where(t.boxes,function(t){return"top"===t.options.position}),f=e.where(t.boxes,function(t){return"bottom"===t.options.position}),g=e.where(t.boxes,function(t){return"chartArea"===t.options.position});h.sort(function(t,e){return(e.options.fullWidth?1:0)-(t.options.fullWidth?1:0)}),f.sort(function(t,e){return(t.options.fullWidth?1:0)-(e.options.fullWidth?1:0)});var p=a-2*s,m=i-2*d,b=p/2,v=m/2,x=(a-b)/(u.length+c.length),y=(i-v)/(h.length+f.length),k=p,S=m,w=[];e.each(u.concat(c,h,f),n);var C=s,M=s,D=d,A=d;e.each(u.concat(c),o),e.each(u,function(t){C+=t.width}),e.each(c,function(t){M+=t.width}),e.each(h.concat(f),o),e.each(h,function(t){D+=t.height}),e.each(f,function(t){A+=t.height}),e.each(u.concat(c),r),C=s,M=s,D=d,A=d,e.each(u,function(t){C+=t.width}),e.each(c,function(t){M+=t.width}),e.each(h,function(t){D+=t.height}),e.each(f,function(t){A+=t.height});var I=i-D-A,T=a-C-M;(T!==k||I!==S)&&(e.each(u,function(t){t.height=I}),e.each(c,function(t){t.height=I}),e.each(h,function(t){t.options.fullWidth||(t.width=T)}),e.each(f,function(t){t.options.fullWidth||(t.width=T)}),S=I,k=T);var F=s,P=d;e.each(u.concat(h),l),F+=k,P+=S,e.each(c,l),e.each(f,l),t.chartArea={left:C,top:D,right:C+k,bottom:D+S},e.each(g,function(e){e.left=t.chartArea.left,e.top=t.chartArea.top,e.right=t.chartArea.right,e.bottom=t.chartArea.bottom,e.update(k,S)})}}}}},{}],29:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=e.noop;t.defaults.global.legend={display:!0,position:"top",fullWidth:!0,reverse:!1,onClick:function(t,e){var a=e.datasetIndex,i=this.chart,n=i.getDatasetMeta(a);n.hidden=null===n.hidden?!i.data.datasets[a].hidden:null,i.update()},labels:{boxWidth:40,padding:10,generateLabels:function(t){var a=t.data;return e.isArray(a.datasets)?a.datasets.map(function(a,i){return{text:a.label,fillStyle:e.isArray(a.backgroundColor)?a.backgroundColor[0]:a.backgroundColor,hidden:!t.isDatasetVisible(i),lineCap:a.borderCapStyle,lineDash:a.borderDash,lineDashOffset:a.borderDashOffset,lineJoin:a.borderJoinStyle,lineWidth:a.borderWidth,strokeStyle:a.borderColor,pointStyle:a.pointStyle,datasetIndex:i}},this):[]}}},t.Legend=t.Element.extend({initialize:function(t){e.extend(this,t),this.legendHitBoxes=[],this.doughnutMode=!1},beforeUpdate:a,update:function(t,e,a){var i=this;return i.beforeUpdate(),i.maxWidth=t,i.maxHeight=e,i.margins=a,i.beforeSetDimensions(),i.setDimensions(),i.afterSetDimensions(),i.beforeBuildLabels(),i.buildLabels(),i.afterBuildLabels(),i.beforeFit(),i.fit(),i.afterFit(),i.afterUpdate(),i.minSize},afterUpdate:a,beforeSetDimensions:a,setDimensions:function(){var t=this;t.isHorizontal()?(t.width=t.maxWidth,t.left=0,t.right=t.width):(t.height=t.maxHeight,t.top=0,t.bottom=t.height),t.paddingLeft=0,t.paddingTop=0,t.paddingRight=0,t.paddingBottom=0,t.minSize={width:0,height:0}},afterSetDimensions:a,beforeBuildLabels:a,buildLabels:function(){var t=this;t.legendItems=t.options.labels.generateLabels.call(t,t.chart),t.options.reverse&&t.legendItems.reverse()},afterBuildLabels:a,beforeFit:a,fit:function(){var a=this,i=a.options,n=i.labels,o=i.display,r=a.ctx,l=t.defaults.global,s=e.getValueOrDefault,d=s(n.fontSize,l.defaultFontSize),u=s(n.fontStyle,l.defaultFontStyle),c=s(n.fontFamily,l.defaultFontFamily),h=e.fontString(d,u,c),f=a.legendHitBoxes=[],g=a.minSize,p=a.isHorizontal();if(p?(g.width=a.maxWidth,g.height=o?10:0):(g.width=o?10:0,g.height=a.maxHeight),o)if(r.font=h,p){var m=a.lineWidths=[0],b=a.legendItems.length?d+n.padding:0;r.textAlign="left",r.textBaseline="top",e.each(a.legendItems,function(t,e){var i=n.usePointStyle?d*Math.sqrt(2):n.boxWidth,o=i+d/2+r.measureText(t.text).width;m[m.length-1]+o+n.padding>=a.width&&(b+=d+n.padding,m[m.length]=a.left),f[e]={left:0,top:0,width:o,height:d},m[m.length-1]+=o+n.padding}),g.height+=b}else{var v=n.padding,x=a.columnWidths=[],y=n.padding,k=0,S=0,w=d+v;e.each(a.legendItems,function(t,e){var a=n.usePointStyle?2*n.boxWidth:n.boxWidth,i=a+d/2+r.measureText(t.text).width;S+w>g.height&&(y+=k+n.padding,x.push(k),k=0,S=0),k=Math.max(k,i),S+=w,f[e]={left:0,top:0,width:i,height:d}}),y+=k,x.push(k),g.width+=y}a.width=g.width,a.height=g.height},afterFit:a,isHorizontal:function(){return"top"===this.options.position||"bottom"===this.options.position},draw:function(){var a=this,i=a.options,n=i.labels,o=t.defaults.global,r=o.elements.line,l=a.width,s=a.lineWidths;if(i.display){var d,u=a.ctx,c=e.getValueOrDefault,h=c(n.fontColor,o.defaultFontColor),f=c(n.fontSize,o.defaultFontSize),g=c(n.fontStyle,o.defaultFontStyle),p=c(n.fontFamily,o.defaultFontFamily),m=e.fontString(f,g,p);u.textAlign="left",u.textBaseline="top",u.lineWidth=.5,u.strokeStyle=h,u.fillStyle=h,u.font=m;var b=n.boxWidth,v=a.legendHitBoxes,x=function(e,a,n){if(!(isNaN(b)||0>=b)){if(u.save(),u.fillStyle=c(n.fillStyle,o.defaultColor),u.lineCap=c(n.lineCap,r.borderCapStyle),u.lineDashOffset=c(n.lineDashOffset,r.borderDashOffset),u.lineJoin=c(n.lineJoin,r.borderJoinStyle),u.lineWidth=c(n.lineWidth,r.borderWidth),u.strokeStyle=c(n.strokeStyle,o.defaultColor),u.setLineDash&&u.setLineDash(c(n.lineDash,r.borderDash)),i.labels&&i.labels.usePointStyle){var l=f*Math.SQRT2/2,s=l/Math.SQRT2,d=e+s,h=a+s;t.canvasHelpers.drawPoint(u,n.pointStyle,l,d,h)}else u.strokeRect(e,a,b,f),u.fillRect(e,a,b,f);u.restore()}},y=function(t,e,a,i){u.fillText(a.text,b+f/2+t,e),a.hidden&&(u.beginPath(),u.lineWidth=2,u.moveTo(b+f/2+t,e+f/2),u.lineTo(b+f/2+t+i,e+f/2),u.stroke())},k=a.isHorizontal();d=k?{x:a.left+(l-s[0])/2,y:a.top+n.padding,line:0}:{x:a.left+n.padding,y:a.top+n.padding,line:0};var S=f+n.padding;e.each(a.legendItems,function(t,e){var i=u.measureText(t.text).width,o=n.usePointStyle?f+f/2+i:b+f/2+i,r=d.x,c=d.y;k?r+o>=l&&(c=d.y+=S,d.line++,r=d.x=a.left+(l-s[d.line])/2):c+S>a.bottom&&(r=d.x=r+a.columnWidths[d.line]+n.padding,c=d.y=a.top,d.line++),x(r,c,t),v[e].left=r,v[e].top=c,y(r,c,t,i),k?d.x+=o+n.padding:d.y+=S})}},handleEvent:function(t){var a=this,i=e.getRelativePosition(t,a.chart.chart),n=i.x,o=i.y,r=a.options;if(n>=a.left&&n<=a.right&&o>=a.top&&o<=a.bottom)for(var l=a.legendHitBoxes,s=0;s<l.length;++s){var d=l[s];if(n>=d.left&&n<=d.left+d.width&&o>=d.top&&o<=d.top+d.height){r.onClick&&r.onClick.call(a,t,a.legendItems[s]);break}}}}),t.plugins.register({beforeInit:function(e){var a=e.options,i=a.legend;i&&(e.legend=new t.Legend({ctx:e.chart.ctx,options:i,chart:e}),t.layoutService.addBox(e,e.legend))}})}},{}],30:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers.noop;t.plugins={_plugins:[],register:function(t){var e=this._plugins;[].concat(t).forEach(function(t){-1===e.indexOf(t)&&e.push(t)})},unregister:function(t){var e=this._plugins;[].concat(t).forEach(function(t){var a=e.indexOf(t);-1!==a&&e.splice(a,1)})},clear:function(){this._plugins=[]},count:function(){return this._plugins.length},getAll:function(){return this._plugins},notify:function(t,e){var a,i,n=this._plugins,o=n.length;for(a=0;o>a;++a)if(i=n[a],"function"==typeof i[t]&&i[t].apply(i,e||[])===!1)return!1;return!0}},t.PluginBase=t.Element.extend({beforeInit:e,afterInit:e,beforeUpdate:e,afterUpdate:e,beforeDraw:e,afterDraw:e,destroy:e}),t.pluginService=t.plugins}},{}],31:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.scale={display:!0,position:"left",gridLines:{display:!0,color:"rgba(0, 0, 0, 0.1)",lineWidth:1,drawBorder:!0,drawOnChartArea:!0,drawTicks:!0,tickMarkLength:10,zeroLineWidth:1,zeroLineColor:"rgba(0,0,0,0.25)",offsetGridLines:!1},scaleLabel:{labelString:"",display:!1},ticks:{beginAtZero:!1,minRotation:0,maxRotation:50,mirror:!1,padding:10,reverse:!1,display:!0,autoSkip:!0,autoSkipPadding:0,labelOffset:0,callback:function(t){return e.isArray(t)?t:""+t}}},t.Scale=t.Element.extend({beforeUpdate:function(){e.callCallback(this.options.beforeUpdate,[this])},update:function(t,a,i){var n=this;return n.beforeUpdate(),n.maxWidth=t,n.maxHeight=a,n.margins=e.extend({left:0,right:0,top:0,bottom:0},i),n.beforeSetDimensions(),n.setDimensions(),n.afterSetDimensions(),n.beforeDataLimits(),n.determineDataLimits(),n.afterDataLimits(),n.beforeBuildTicks(),n.buildTicks(),n.afterBuildTicks(),n.beforeTickToLabelConversion(),n.convertTicksToLabels(),n.afterTickToLabelConversion(),n.beforeCalculateTickRotation(),n.calculateTickRotation(),n.afterCalculateTickRotation(),n.beforeFit(),n.fit(),n.afterFit(),n.afterUpdate(),n.minSize},afterUpdate:function(){e.callCallback(this.options.afterUpdate,[this])},beforeSetDimensions:function(){e.callCallback(this.options.beforeSetDimensions,[this])},setDimensions:function(){var t=this;t.isHorizontal()?(t.width=t.maxWidth,t.left=0,t.right=t.width):(t.height=t.maxHeight,t.top=0,t.bottom=t.height),t.paddingLeft=0,t.paddingTop=0,t.paddingRight=0,t.paddingBottom=0},afterSetDimensions:function(){e.callCallback(this.options.afterSetDimensions,[this])},beforeDataLimits:function(){e.callCallback(this.options.beforeDataLimits,[this])},determineDataLimits:e.noop,afterDataLimits:function(){e.callCallback(this.options.afterDataLimits,[this])},beforeBuildTicks:function(){e.callCallback(this.options.beforeBuildTicks,[this])},buildTicks:e.noop,afterBuildTicks:function(){e.callCallback(this.options.afterBuildTicks,[this])},beforeTickToLabelConversion:function(){e.callCallback(this.options.beforeTickToLabelConversion,[this])},convertTicksToLabels:function(){var t=this;t.ticks=t.ticks.map(function(e,a,i){return t.options.ticks.userCallback?t.options.ticks.userCallback(e,a,i):t.options.ticks.callback(e,a,i)},t)},afterTickToLabelConversion:function(){e.callCallback(this.options.afterTickToLabelConversion,[this])},beforeCalculateTickRotation:function(){e.callCallback(this.options.beforeCalculateTickRotation,[this])},calculateTickRotation:function(){var a=this,i=a.ctx,n=t.defaults.global,o=a.options.ticks,r=e.getValueOrDefault(o.fontSize,n.defaultFontSize),l=e.getValueOrDefault(o.fontStyle,n.defaultFontStyle),s=e.getValueOrDefault(o.fontFamily,n.defaultFontFamily),d=e.fontString(r,l,s);i.font=d;var u,c=i.measureText(a.ticks[0]).width,h=i.measureText(a.ticks[a.ticks.length-1]).width;if(a.labelRotation=o.minRotation||0,a.paddingRight=0,a.paddingLeft=0,a.options.display&&a.isHorizontal()){a.paddingRight=h/2+3,a.paddingLeft=c/2+3,a.longestTextCache||(a.longestTextCache={});for(var f,g,p=e.longestText(i,d,a.ticks,a.longestTextCache),m=p,b=a.getPixelForTick(1)-a.getPixelForTick(0)-6;m>b&&a.labelRotation<o.maxRotation;){if(f=Math.cos(e.toRadians(a.labelRotation)),g=Math.sin(e.toRadians(a.labelRotation)),u=f*c,u+r/2>a.yLabelWidth&&(a.paddingLeft=u+r/2),a.paddingRight=r/2,g*p>a.maxHeight){a.labelRotation--;break}a.labelRotation++,m=f*p}}a.margins&&(a.paddingLeft=Math.max(a.paddingLeft-a.margins.left,0),a.paddingRight=Math.max(a.paddingRight-a.margins.right,0))},afterCalculateTickRotation:function(){e.callCallback(this.options.afterCalculateTickRotation,[this])},beforeFit:function(){e.callCallback(this.options.beforeFit,[this]);
},fit:function(){var a=this,i=a.minSize={width:0,height:0},n=a.options,o=t.defaults.global,r=n.ticks,l=n.scaleLabel,s=n.display,d=a.isHorizontal(),u=e.getValueOrDefault(r.fontSize,o.defaultFontSize),c=e.getValueOrDefault(r.fontStyle,o.defaultFontStyle),h=e.getValueOrDefault(r.fontFamily,o.defaultFontFamily),f=e.fontString(u,c,h),g=e.getValueOrDefault(l.fontSize,o.defaultFontSize),p=n.gridLines.tickMarkLength;if(d?i.width=a.isFullWidth()?a.maxWidth-a.margins.left-a.margins.right:a.maxWidth:i.width=s?p:0,d?i.height=s?p:0:i.height=a.maxHeight,l.display&&s&&(d?i.height+=1.5*g:i.width+=1.5*g),r.display&&s){a.longestTextCache||(a.longestTextCache={});var m=e.longestText(a.ctx,f,a.ticks,a.longestTextCache),b=e.numberOfLabelLines(a.ticks),v=.5*u;if(d){a.longestLabelWidth=m;var x=Math.sin(e.toRadians(a.labelRotation))*a.longestLabelWidth+u*b+v*b;i.height=Math.min(a.maxHeight,i.height+x),a.ctx.font=f;var y=a.ctx.measureText(a.ticks[0]).width,k=a.ctx.measureText(a.ticks[a.ticks.length-1]).width,S=Math.cos(e.toRadians(a.labelRotation)),w=Math.sin(e.toRadians(a.labelRotation));a.paddingLeft=0!==a.labelRotation?S*y+3:y/2+3,a.paddingRight=0!==a.labelRotation?w*(u/2)+3:k/2+3}else{var C=a.maxWidth-i.width,M=r.mirror;M?m=0:m+=a.options.ticks.padding,C>m?i.width+=m:i.width=a.maxWidth,a.paddingTop=u/2,a.paddingBottom=u/2}}a.margins&&(a.paddingLeft=Math.max(a.paddingLeft-a.margins.left,0),a.paddingTop=Math.max(a.paddingTop-a.margins.top,0),a.paddingRight=Math.max(a.paddingRight-a.margins.right,0),a.paddingBottom=Math.max(a.paddingBottom-a.margins.bottom,0)),a.width=i.width,a.height=i.height},afterFit:function(){e.callCallback(this.options.afterFit,[this])},isHorizontal:function(){return"top"===this.options.position||"bottom"===this.options.position},isFullWidth:function(){return this.options.fullWidth},getRightValue:function(t){return null===t||"undefined"==typeof t?NaN:"number"==typeof t&&isNaN(t)?NaN:"object"==typeof t?t instanceof Date||t.isValid?t:this.getRightValue(this.isHorizontal()?t.x:t.y):t},getLabelForIndex:e.noop,getPixelForValue:e.noop,getValueForPixel:e.noop,getPixelForTick:function(t,e){var a=this;if(a.isHorizontal()){var i=a.width-(a.paddingLeft+a.paddingRight),n=i/Math.max(a.ticks.length-(a.options.gridLines.offsetGridLines?0:1),1),o=n*t+a.paddingLeft;e&&(o+=n/2);var r=a.left+Math.round(o);return r+=a.isFullWidth()?a.margins.left:0}var l=a.height-(a.paddingTop+a.paddingBottom);return a.top+t*(l/(a.ticks.length-1))},getPixelForDecimal:function(t){var e=this;if(e.isHorizontal()){var a=e.width-(e.paddingLeft+e.paddingRight),i=a*t+e.paddingLeft,n=e.left+Math.round(i);return n+=e.isFullWidth()?e.margins.left:0}return e.top+t*e.height},getBasePixel:function(){var t=this,e=t.min,a=t.max;return t.getPixelForValue(t.beginAtZero?0:0>e&&0>a?a:e>0&&a>0?e:0)},draw:function(a){var i=this,n=i.options;if(n.display){var o,r,l=i.ctx,s=t.defaults.global,d=n.ticks,u=n.gridLines,c=n.scaleLabel,h=0!==i.labelRotation,f=d.autoSkip,g=i.isHorizontal();d.maxTicksLimit&&(r=d.maxTicksLimit);var p=e.getValueOrDefault(d.fontColor,s.defaultFontColor),m=e.getValueOrDefault(d.fontSize,s.defaultFontSize),b=e.getValueOrDefault(d.fontStyle,s.defaultFontStyle),v=e.getValueOrDefault(d.fontFamily,s.defaultFontFamily),x=e.fontString(m,b,v),y=u.tickMarkLength,k=e.getValueOrDefault(c.fontColor,s.defaultFontColor),S=e.getValueOrDefault(c.fontSize,s.defaultFontSize),w=e.getValueOrDefault(c.fontStyle,s.defaultFontStyle),C=e.getValueOrDefault(c.fontFamily,s.defaultFontFamily),M=e.fontString(S,w,C),D=e.toRadians(i.labelRotation),A=Math.cos(D),I=i.longestLabelWidth*A;l.fillStyle=p;var T=[];if(g){if(o=!1,h&&(I/=2),(I+d.autoSkipPadding)*i.ticks.length>i.width-(i.paddingLeft+i.paddingRight)&&(o=1+Math.floor((I+d.autoSkipPadding)*i.ticks.length/(i.width-(i.paddingLeft+i.paddingRight)))),r&&i.ticks.length>r)for(;!o||i.ticks.length/(o||1)>r;)o||(o=1),o+=1;f||(o=!1)}var F="right"===n.position?i.left:i.right-y,P="right"===n.position?i.left+y:i.right,R="bottom"===n.position?i.top:i.bottom-y,_="bottom"===n.position?i.top+y:i.bottom;if(e.each(i.ticks,function(t,r){if(void 0!==t&&null!==t){var l=i.ticks.length===r+1,s=o>1&&r%o>0||r%o===0&&r+o>=i.ticks.length;if((!s||l)&&void 0!==t&&null!==t){var c,f;r===("undefined"!=typeof i.zeroLineIndex?i.zeroLineIndex:0)?(c=u.zeroLineWidth,f=u.zeroLineColor):(c=e.getValueAtIndexOrDefault(u.lineWidth,r),f=e.getValueAtIndexOrDefault(u.color,r));var p,m,b,v,x,k,S,w,C,M,A,I="middle";if(g){h||(I="top"===n.position?"bottom":"top"),A=h?"right":"center";var V=i.getPixelForTick(r)+e.aliasPixel(c);C=i.getPixelForTick(r,u.offsetGridLines)+d.labelOffset,M=h?i.top+12:"top"===n.position?i.bottom-y:i.top+y,p=b=x=S=V,m=R,v=_,k=a.top,w=a.bottom}else{"left"===n.position?d.mirror?(C=i.right+d.padding,A="left"):(C=i.right-d.padding,A="right"):d.mirror?(C=i.left-d.padding,A="right"):(C=i.left+d.padding,A="left");var L=i.getPixelForTick(r);L+=e.aliasPixel(c),M=i.getPixelForTick(r,u.offsetGridLines),p=F,b=P,x=a.left,S=a.right,m=v=k=w=L}T.push({tx1:p,ty1:m,tx2:b,ty2:v,x1:x,y1:k,x2:S,y2:w,labelX:C,labelY:M,glWidth:c,glColor:f,rotation:-1*D,label:t,textBaseline:I,textAlign:A})}}}),e.each(T,function(t){if(u.display&&(l.lineWidth=t.glWidth,l.strokeStyle=t.glColor,l.beginPath(),u.drawTicks&&(l.moveTo(t.tx1,t.ty1),l.lineTo(t.tx2,t.ty2)),u.drawOnChartArea&&(l.moveTo(t.x1,t.y1),l.lineTo(t.x2,t.y2)),l.stroke()),d.display){l.save(),l.translate(t.labelX,t.labelY),l.rotate(t.rotation),l.font=x,l.textBaseline=t.textBaseline,l.textAlign=t.textAlign;var a=t.label;if(e.isArray(a))for(var i=0,n=0;i<a.length;++i)l.fillText(""+a[i],0,n),n+=1.5*m;else l.fillText(a,0,0);l.restore()}}),c.display){var V,L,O=0;if(g)V=i.left+(i.right-i.left)/2,L="bottom"===n.position?i.bottom-S/2:i.top+S/2;else{var B="left"===n.position;V=B?i.left+S/2:i.right-S/2,L=i.top+(i.bottom-i.top)/2,O=B?-.5*Math.PI:.5*Math.PI}l.save(),l.translate(V,L),l.rotate(O),l.textAlign="center",l.textBaseline="middle",l.fillStyle=k,l.font=M,l.fillText(c.labelString,0,0),l.restore()}if(u.drawBorder){l.lineWidth=e.getValueAtIndexOrDefault(u.lineWidth,0),l.strokeStyle=e.getValueAtIndexOrDefault(u.color,0);var W=i.left,z=i.right,N=i.top,H=i.bottom,E=e.aliasPixel(l.lineWidth);g?(N=H="top"===n.position?i.bottom:i.top,N+=E,H+=E):(W=z="left"===n.position?i.right:i.left,W+=E,z+=E),l.beginPath(),l.moveTo(W,N),l.lineTo(z,H),l.stroke()}}}})}},{}],32:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.scaleService={constructors:{},defaults:{},registerScaleType:function(t,a,i){this.constructors[t]=a,this.defaults[t]=e.clone(i)},getScaleConstructor:function(t){return this.constructors.hasOwnProperty(t)?this.constructors[t]:void 0},getScaleDefaults:function(a){return this.defaults.hasOwnProperty(a)?e.scaleMerge(t.defaults.scale,this.defaults[a]):{}},updateScaleDefaults:function(t,a){var i=this.defaults;i.hasOwnProperty(t)&&(i[t]=e.extend(i[t],a))},addScalesToLayout:function(a){e.each(a.scales,function(e){t.layoutService.addBox(a,e)})}}}},{}],33:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers;t.defaults.global.title={display:!1,position:"top",fullWidth:!0,fontStyle:"bold",padding:10,text:""};var a=e.noop;t.Title=t.Element.extend({initialize:function(a){var i=this;e.extend(i,a),i.options=e.configMerge(t.defaults.global.title,a.options),i.legendHitBoxes=[]},beforeUpdate:function(){var a=this.chart.options;a&&a.title&&(this.options=e.configMerge(t.defaults.global.title,a.title))},update:function(t,e,a){var i=this;return i.beforeUpdate(),i.maxWidth=t,i.maxHeight=e,i.margins=a,i.beforeSetDimensions(),i.setDimensions(),i.afterSetDimensions(),i.beforeBuildLabels(),i.buildLabels(),i.afterBuildLabels(),i.beforeFit(),i.fit(),i.afterFit(),i.afterUpdate(),i.minSize},afterUpdate:a,beforeSetDimensions:a,setDimensions:function(){var t=this;t.isHorizontal()?(t.width=t.maxWidth,t.left=0,t.right=t.width):(t.height=t.maxHeight,t.top=0,t.bottom=t.height),t.paddingLeft=0,t.paddingTop=0,t.paddingRight=0,t.paddingBottom=0,t.minSize={width:0,height:0}},afterSetDimensions:a,beforeBuildLabels:a,buildLabels:a,afterBuildLabels:a,beforeFit:a,fit:function(){var a=this,i=e.getValueOrDefault,n=a.options,o=t.defaults.global,r=n.display,l=i(n.fontSize,o.defaultFontSize),s=a.minSize;a.isHorizontal()?(s.width=a.maxWidth,s.height=r?l+2*n.padding:0):(s.width=r?l+2*n.padding:0,s.height=a.maxHeight),a.width=s.width,a.height=s.height},afterFit:a,isHorizontal:function(){var t=this.options.position;return"top"===t||"bottom"===t},draw:function(){var a=this,i=a.ctx,n=e.getValueOrDefault,o=a.options,r=t.defaults.global;if(o.display){var l,s,d=n(o.fontSize,r.defaultFontSize),u=n(o.fontStyle,r.defaultFontStyle),c=n(o.fontFamily,r.defaultFontFamily),h=e.fontString(d,u,c),f=0,g=a.top,p=a.left,m=a.bottom,b=a.right;i.fillStyle=n(o.fontColor,r.defaultFontColor),i.font=h,a.isHorizontal()?(l=p+(b-p)/2,s=g+(m-g)/2):(l="left"===o.position?p+d/2:b-d/2,s=g+(m-g)/2,f=Math.PI*("left"===o.position?-.5:.5)),i.save(),i.translate(l,s),i.rotate(f),i.textAlign="center",i.textBaseline="middle",i.fillText(o.text,0,0),i.restore()}}}),t.plugins.register({beforeInit:function(e){var a=e.options,i=a.title;i&&(e.titleBlock=new t.Title({ctx:e.chart.ctx,options:i,chart:e}),t.layoutService.addBox(e,e.titleBlock))}})}},{}],34:[function(t,e,a){"use strict";e.exports=function(t){function e(t,e){return e&&(n.isArray(e)?Array.prototype.push.apply(t,e):t.push(e)),t}function a(t){if(!t.length)return!1;var e,a,i=[],n=[];for(e=0,a=t.length;a>e;++e){var o=t[e];if(o&&o.hasValue()){var r=o.tooltipPosition();i.push(r.x),n.push(r.y)}}var l=0,s=0;for(e=0;e<i.length;++e)i[e]&&(l+=i[e],s+=n[e]);return{x:Math.round(l/i.length),y:Math.round(s/i.length)}}function i(t){var e=t._xScale,a=t._yScale||t._scale,i=t._index,n=t._datasetIndex;return{xLabel:e?e.getLabelForIndex(i,n):"",yLabel:a?a.getLabelForIndex(i,n):"",index:i,datasetIndex:n}}var n=t.helpers;t.defaults.global.tooltips={enabled:!0,custom:null,mode:"single",backgroundColor:"rgba(0,0,0,0.8)",titleFontStyle:"bold",titleSpacing:2,titleMarginBottom:6,titleFontColor:"#fff",titleAlign:"left",bodySpacing:2,bodyFontColor:"#fff",bodyAlign:"left",footerFontStyle:"bold",footerSpacing:2,footerMarginTop:6,footerFontColor:"#fff",footerAlign:"left",yPadding:6,xPadding:6,yAlign:"center",xAlign:"center",caretSize:5,cornerRadius:6,multiKeyBackground:"#fff",callbacks:{beforeTitle:n.noop,title:function(t,e){var a="",i=e.labels,n=i?i.length:0;if(t.length>0){var o=t[0];o.xLabel?a=o.xLabel:n>0&&o.index<n&&(a=i[o.index])}return a},afterTitle:n.noop,beforeBody:n.noop,beforeLabel:n.noop,label:function(t,e){var a=e.datasets[t.datasetIndex].label||"";return a+": "+t.yLabel},labelColor:function(t,e){var a=e.getDatasetMeta(t.datasetIndex),i=a.data[t.index],n=i._view;return{borderColor:n.borderColor,backgroundColor:n.backgroundColor}},afterLabel:n.noop,afterBody:n.noop,beforeFooter:n.noop,footer:n.noop,afterFooter:n.noop}},t.Tooltip=t.Element.extend({initialize:function(){var e=this,a=t.defaults.global,i=e._options,o=n.getValueOrDefault;n.extend(e,{_model:{xPadding:i.xPadding,yPadding:i.yPadding,xAlign:i.xAlign,yAlign:i.yAlign,bodyFontColor:i.bodyFontColor,_bodyFontFamily:o(i.bodyFontFamily,a.defaultFontFamily),_bodyFontStyle:o(i.bodyFontStyle,a.defaultFontStyle),_bodyAlign:i.bodyAlign,bodyFontSize:o(i.bodyFontSize,a.defaultFontSize),bodySpacing:i.bodySpacing,titleFontColor:i.titleFontColor,_titleFontFamily:o(i.titleFontFamily,a.defaultFontFamily),_titleFontStyle:o(i.titleFontStyle,a.defaultFontStyle),titleFontSize:o(i.titleFontSize,a.defaultFontSize),_titleAlign:i.titleAlign,titleSpacing:i.titleSpacing,titleMarginBottom:i.titleMarginBottom,footerFontColor:i.footerFontColor,_footerFontFamily:o(i.footerFontFamily,a.defaultFontFamily),_footerFontStyle:o(i.footerFontStyle,a.defaultFontStyle),footerFontSize:o(i.footerFontSize,a.defaultFontSize),_footerAlign:i.footerAlign,footerSpacing:i.footerSpacing,footerMarginTop:i.footerMarginTop,caretSize:i.caretSize,cornerRadius:i.cornerRadius,backgroundColor:i.backgroundColor,opacity:0,legendColorBackground:i.multiKeyBackground}})},getTitle:function(){var t=this,a=t._options,i=a.callbacks,n=i.beforeTitle.apply(t,arguments),o=i.title.apply(t,arguments),r=i.afterTitle.apply(t,arguments),l=[];return l=e(l,n),l=e(l,o),l=e(l,r)},getBeforeBody:function(){var t=this._options.callbacks.beforeBody.apply(this,arguments);return n.isArray(t)?t:void 0!==t?[t]:[]},getBody:function(t,a){var i=this,o=i._options.callbacks,r=[];return n.each(t,function(t){var n={before:[],lines:[],after:[]};e(n.before,o.beforeLabel.call(i,t,a)),e(n.lines,o.label.call(i,t,a)),e(n.after,o.afterLabel.call(i,t,a)),r.push(n)}),r},getAfterBody:function(){var t=this._options.callbacks.afterBody.apply(this,arguments);return n.isArray(t)?t:void 0!==t?[t]:[]},getFooter:function(){var t=this,a=t._options.callbacks,i=a.beforeFooter.apply(t,arguments),n=a.footer.apply(t,arguments),o=a.afterFooter.apply(t,arguments),r=[];return r=e(r,i),r=e(r,n),r=e(r,o)},update:function(t){var e,o,r=this,l=r._options,s=r._model,d=r._active,u=r._data,c=r._chartInstance;if(d.length){s.opacity=1;var h=[],f=a(d),g=[];for(e=0,o=d.length;o>e;++e)g.push(i(d[e]));l.itemSort&&(g=g.sort(l.itemSort)),d.length>1&&n.each(g,function(t){h.push(l.callbacks.labelColor.call(r,t,c))}),n.extend(s,{title:r.getTitle(g,u),beforeBody:r.getBeforeBody(g,u),body:r.getBody(g,u),afterBody:r.getAfterBody(g,u),footer:r.getFooter(g,u),x:Math.round(f.x),y:Math.round(f.y),caretPadding:n.getValueOrDefault(f.padding,2),labelColors:h});var p=r.getTooltipSize(s);r.determineAlignment(p),n.extend(s,r.getBackgroundPoint(s,p))}else r._model.opacity=0;return t&&l.custom&&l.custom.call(r,s),r},getTooltipSize:function(t){var e=this._chart.ctx,a={height:2*t.yPadding,width:0},i=t.body,o=i.reduce(function(t,e){return t+e.before.length+e.lines.length+e.after.length},0);o+=t.beforeBody.length+t.afterBody.length;var r=t.title.length,l=t.footer.length,s=t.titleFontSize,d=t.bodyFontSize,u=t.footerFontSize;a.height+=r*s,a.height+=(r-1)*t.titleSpacing,a.height+=r?t.titleMarginBottom:0,a.height+=o*d,a.height+=o?(o-1)*t.bodySpacing:0,a.height+=l?t.footerMarginTop:0,a.height+=l*u,a.height+=l?(l-1)*t.footerSpacing:0;var c=0,h=function(t){a.width=Math.max(a.width,e.measureText(t).width+c)};return e.font=n.fontString(s,t._titleFontStyle,t._titleFontFamily),n.each(t.title,h),e.font=n.fontString(d,t._bodyFontStyle,t._bodyFontFamily),n.each(t.beforeBody.concat(t.afterBody),h),c=i.length>1?d+2:0,n.each(i,function(t){n.each(t.before,h),n.each(t.lines,h),n.each(t.after,h)}),c=0,e.font=n.fontString(u,t._footerFontStyle,t._footerFontFamily),n.each(t.footer,h),a.width+=2*t.xPadding,a},determineAlignment:function(t){var e=this,a=e._model,i=e._chart,n=e._chartInstance.chartArea;a.y<t.height?a.yAlign="top":a.y>i.height-t.height&&(a.yAlign="bottom");var o,r,l,s,d,u=(n.left+n.right)/2,c=(n.top+n.bottom)/2;"center"===a.yAlign?(o=function(t){return u>=t},r=function(t){return t>u}):(o=function(e){return e<=t.width/2},r=function(e){return e>=i.width-t.width/2}),l=function(e){return e+t.width>i.width},s=function(e){return e-t.width<0},d=function(t){return c>=t?"top":"bottom"},o(a.x)?(a.xAlign="left",l(a.x)&&(a.xAlign="center",a.yAlign=d(a.y))):r(a.x)&&(a.xAlign="right",s(a.x)&&(a.xAlign="center",a.yAlign=d(a.y)))},getBackgroundPoint:function(t,e){var a={x:t.x,y:t.y},i=t.caretSize,n=t.caretPadding,o=t.cornerRadius,r=t.xAlign,l=t.yAlign,s=i+n,d=o+n;return"right"===r?a.x-=e.width:"center"===r&&(a.x-=e.width/2),"top"===l?a.y+=s:"bottom"===l?a.y-=e.height+s:a.y-=e.height/2,"center"===l?"left"===r?a.x+=s:"right"===r&&(a.x-=s):"left"===r?a.x-=d:"right"===r&&(a.x+=d),a},drawCaret:function(t,e,a){var i,o,r,l,s,d,u=this._view,c=this._chart.ctx,h=u.caretSize,f=u.cornerRadius,g=u.xAlign,p=u.yAlign,m=t.x,b=t.y,v=e.width,x=e.height;"center"===p?("left"===g?(i=m,o=i-h,r=i):(i=m+v,o=i+h,r=i),s=b+x/2,l=s-h,d=s+h):("left"===g?(i=m+f,o=i+h,r=o+h):"right"===g?(i=m+v-f,o=i-h,r=o-h):(o=m+v/2,i=o-h,r=o+h),"top"===p?(l=b,s=l-h,d=l):(l=b+x,s=l+h,d=l));var y=n.color(u.backgroundColor);c.fillStyle=y.alpha(a*y.alpha()).rgbString(),c.beginPath(),c.moveTo(i,l),c.lineTo(o,s),c.lineTo(r,d),c.closePath(),c.fill()},drawTitle:function(t,e,a,i){var o=e.title;if(o.length){a.textAlign=e._titleAlign,a.textBaseline="top";var r=e.titleFontSize,l=e.titleSpacing,s=n.color(e.titleFontColor);a.fillStyle=s.alpha(i*s.alpha()).rgbString(),a.font=n.fontString(r,e._titleFontStyle,e._titleFontFamily);var d,u;for(d=0,u=o.length;u>d;++d)a.fillText(o[d],t.x,t.y),t.y+=r+l,d+1===o.length&&(t.y+=e.titleMarginBottom-l)}},drawBody:function(t,e,a,i){var o=e.bodyFontSize,r=e.bodySpacing,l=e.body;a.textAlign=e._bodyAlign,a.textBaseline="top";var s=n.color(e.bodyFontColor),d=s.alpha(i*s.alpha()).rgbString();a.fillStyle=d,a.font=n.fontString(o,e._bodyFontStyle,e._bodyFontFamily);var u=0,c=function(e){a.fillText(e,t.x+u,t.y),t.y+=o+r};n.each(e.beforeBody,c);var h=l.length>1;u=h?o+2:0,n.each(l,function(r,l){n.each(r.before,c),n.each(r.lines,function(r){h&&(a.fillStyle=n.color(e.legendColorBackground).alpha(i).rgbaString(),a.fillRect(t.x,t.y,o,o),a.strokeStyle=n.color(e.labelColors[l].borderColor).alpha(i).rgbaString(),a.strokeRect(t.x,t.y,o,o),a.fillStyle=n.color(e.labelColors[l].backgroundColor).alpha(i).rgbaString(),a.fillRect(t.x+1,t.y+1,o-2,o-2),a.fillStyle=d),c(r)}),n.each(r.after,c)}),u=0,n.each(e.afterBody,c),t.y-=r},drawFooter:function(t,e,a,i){var o=e.footer;if(o.length){t.y+=e.footerMarginTop,a.textAlign=e._footerAlign,a.textBaseline="top";var r=n.color(e.footerFontColor);a.fillStyle=r.alpha(i*r.alpha()).rgbString(),a.font=n.fontString(e.footerFontSize,e._footerFontStyle,e._footerFontFamily),n.each(o,function(i){a.fillText(i,t.x,t.y),t.y+=e.footerFontSize+e.footerSpacing})}},draw:function(){var t=this._chart.ctx,e=this._view;if(0!==e.opacity){var a=this.getTooltipSize(e),i={x:e.x,y:e.y},o=Math.abs(e.opacity<.001)?0:e.opacity;if(this._options.enabled){var r=n.color(e.backgroundColor);t.fillStyle=r.alpha(o*r.alpha()).rgbString(),n.drawRoundedRectangle(t,i.x,i.y,a.width,a.height,e.cornerRadius),t.fill(),this.drawCaret(i,a,o),i.x+=e.xPadding,i.y+=e.yPadding,this.drawTitle(i,e,t,o),this.drawBody(i,e,t,o),this.drawFooter(i,e,t,o)}}}})}},{}],35:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=t.defaults.global;a.elements.arc={backgroundColor:a.defaultColor,borderColor:"#fff",borderWidth:2},t.elements.Arc=t.Element.extend({inLabelRange:function(t){var e=this._view;return e?Math.pow(t-e.x,2)<Math.pow(e.radius+e.hoverRadius,2):!1},inRange:function(t,a){var i=this._view;if(i){for(var n=e.getAngleFromPoint(i,{x:t,y:a}),o=n.angle,r=n.distance,l=i.startAngle,s=i.endAngle;l>s;)s+=2*Math.PI;for(;o>s;)o-=2*Math.PI;for(;l>o;)o+=2*Math.PI;var d=o>=l&&s>=o,u=r>=i.innerRadius&&r<=i.outerRadius;return d&&u}return!1},tooltipPosition:function(){var t=this._view,e=t.startAngle+(t.endAngle-t.startAngle)/2,a=(t.outerRadius-t.innerRadius)/2+t.innerRadius;return{x:t.x+Math.cos(e)*a,y:t.y+Math.sin(e)*a}},draw:function(){var t=this._chart.ctx,e=this._view,a=e.startAngle,i=e.endAngle;t.beginPath(),t.arc(e.x,e.y,e.outerRadius,a,i),t.arc(e.x,e.y,e.innerRadius,i,a,!0),t.closePath(),t.strokeStyle=e.borderColor,t.lineWidth=e.borderWidth,t.fillStyle=e.backgroundColor,t.fill(),t.lineJoin="bevel",e.borderWidth&&t.stroke()}})}},{}],36:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=t.defaults.global;t.defaults.global.elements.line={tension:.4,backgroundColor:a.defaultColor,borderWidth:3,borderColor:a.defaultColor,borderCapStyle:"butt",borderDash:[],borderDashOffset:0,borderJoinStyle:"miter",capBezierPoints:!0,fill:!0},t.elements.Line=t.Element.extend({draw:function(){function t(t,e){var a=e._view;e._view.steppedLine===!0?(s.lineTo(e._view.x,t._view.y),s.lineTo(e._view.x,e._view.y)):0===e._view.tension?s.lineTo(a.x,a.y):s.bezierCurveTo(t._view.controlPointNextX,t._view.controlPointNextY,a.controlPointPreviousX,a.controlPointPreviousY,a.x,a.y)}var i=this,n=i._view,o=n.spanGaps,r=n.scaleZero,l=i._loop,s=i._chart.ctx;s.save();var d=i._children.slice(),u=-1;l&&d.length&&d.push(d[0]);var c,h,f,g;if(d.length&&n.fill){for(s.beginPath(),c=0;c<d.length;++c)h=d[c],f=e.previousItem(d,c),g=h._view,0===c?(l?s.moveTo(r.x,r.y):s.moveTo(g.x,r),g.skip||(u=c,s.lineTo(g.x,g.y))):(f=-1===u?f:d[u],g.skip?o||u!==c-1||(l?s.lineTo(r.x,r.y):s.lineTo(f._view.x,r)):(u!==c-1?o&&-1!==u?t(f,h):l?s.lineTo(g.x,g.y):(s.lineTo(g.x,r),s.lineTo(g.x,g.y)):t(f,h),u=c));l||s.lineTo(d[u]._view.x,r),s.fillStyle=n.backgroundColor||a.defaultColor,s.closePath(),s.fill()}var p=a.elements.line;for(s.lineCap=n.borderCapStyle||p.borderCapStyle,s.setLineDash&&s.setLineDash(n.borderDash||p.borderDash),s.lineDashOffset=n.borderDashOffset||p.borderDashOffset,s.lineJoin=n.borderJoinStyle||p.borderJoinStyle,s.lineWidth=n.borderWidth||p.borderWidth,s.strokeStyle=n.borderColor||a.defaultColor,s.beginPath(),u=-1,c=0;c<d.length;++c)h=d[c],f=e.previousItem(d,c),g=h._view,0===c?g.skip||(s.moveTo(g.x,g.y),u=c):(f=-1===u?f:d[u],g.skip||(u!==c-1&&!o||-1===u?s.moveTo(g.x,g.y):t(f,h),u=c));s.stroke(),s.restore()}})}},{}],37:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=t.defaults.global,i=a.defaultColor;a.elements.point={radius:3,pointStyle:"circle",backgroundColor:i,borderWidth:1,borderColor:i,hitRadius:1,hoverRadius:4,hoverBorderWidth:1},t.elements.Point=t.Element.extend({inRange:function(t,e){var a=this._view;return a?Math.pow(t-a.x,2)+Math.pow(e-a.y,2)<Math.pow(a.hitRadius+a.radius,2):!1},inLabelRange:function(t){var e=this._view;return e?Math.pow(t-e.x,2)<Math.pow(e.radius+e.hitRadius,2):!1},tooltipPosition:function(){var t=this._view;return{x:t.x,y:t.y,padding:t.radius+t.borderWidth}},draw:function(){var n=this._view,o=this._chart.ctx,r=n.pointStyle,l=n.radius,s=n.x,d=n.y;n.skip||(o.strokeStyle=n.borderColor||i,o.lineWidth=e.getValueOrDefault(n.borderWidth,a.elements.point.borderWidth),o.fillStyle=n.backgroundColor||i,t.canvasHelpers.drawPoint(o,r,l,s,d))}})}},{}],38:[function(t,e,a){"use strict";e.exports=function(t){var e=t.defaults.global;e.elements.rectangle={backgroundColor:e.defaultColor,borderWidth:0,borderColor:e.defaultColor,borderSkipped:"bottom"},t.elements.Rectangle=t.Element.extend({draw:function(){function t(t){return s[(u+t)%4]}var e=this._chart.ctx,a=this._view,i=a.width/2,n=a.x-i,o=a.x+i,r=a.base-(a.base-a.y),l=a.borderWidth/2;a.borderWidth&&(n+=l,o-=l,r+=l),e.beginPath(),e.fillStyle=a.backgroundColor,e.strokeStyle=a.borderColor,e.lineWidth=a.borderWidth;var s=[[n,a.base],[n,r],[o,r],[o,a.base]],d=["bottom","left","top","right"],u=d.indexOf(a.borderSkipped,0);-1===u&&(u=0),e.moveTo.apply(e,t(0));for(var c=1;4>c;c++)e.lineTo.apply(e,t(c));e.fill(),a.borderWidth&&e.stroke()},height:function(){var t=this._view;return t.base-t.y},inRange:function(t,e){var a=this._view;return a?a.y<a.base?t>=a.x-a.width/2&&t<=a.x+a.width/2&&e>=a.y&&e<=a.base:t>=a.x-a.width/2&&t<=a.x+a.width/2&&e>=a.base&&e<=a.y:!1},inLabelRange:function(t){var e=this._view;return e?t>=e.x-e.width/2&&t<=e.x+e.width/2:!1},tooltipPosition:function(){var t=this._view;return{x:t.x,y:t.y}}})}},{}],39:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a={position:"bottom"},i=t.Scale.extend({getLabels:function(){var t=this.chart.data;return(this.isHorizontal()?t.xLabels:t.yLabels)||t.labels},determineDataLimits:function(){var t=this,a=t.getLabels();t.minIndex=0,t.maxIndex=a.length-1;var i;void 0!==t.options.ticks.min&&(i=e.indexOf(a,t.options.ticks.min),t.minIndex=-1!==i?i:t.minIndex),void 0!==t.options.ticks.max&&(i=e.indexOf(a,t.options.ticks.max),t.maxIndex=-1!==i?i:t.maxIndex),t.min=a[t.minIndex],t.max=a[t.maxIndex]},buildTicks:function(){var t=this,e=t.getLabels();t.ticks=0===t.minIndex&&t.maxIndex===e.length-1?e:e.slice(t.minIndex,t.maxIndex+1)},getLabelForIndex:function(t){return this.ticks[t]},getPixelForValue:function(t,e,a,i){var n=this,o=Math.max(n.maxIndex+1-n.minIndex-(n.options.gridLines.offsetGridLines?0:1),1);if(void 0!==t){var r=n.getLabels(),l=r.indexOf(t);e=-1!==l?l:e}if(n.isHorizontal()){var s=n.width-(n.paddingLeft+n.paddingRight),d=s/o,u=d*(e-n.minIndex)+n.paddingLeft;return n.options.gridLines.offsetGridLines&&i&&(u+=d/2),n.left+Math.round(u)}var c=n.height-(n.paddingTop+n.paddingBottom),h=c/o,f=h*(e-n.minIndex)+n.paddingTop;return n.options.gridLines.offsetGridLines&&i&&(f+=h/2),n.top+Math.round(f)},getPixelForTick:function(t,e){return this.getPixelForValue(this.ticks[t],t+this.minIndex,null,e)},getValueForPixel:function(t){var e,a=this,i=Math.max(a.ticks.length-(a.options.gridLines.offsetGridLines?0:1),1),n=a.isHorizontal(),o=n?a.width-(a.paddingLeft+a.paddingRight):a.height-(a.paddingTop+a.paddingBottom),r=o/i;return t-=n?a.left:a.top,a.options.gridLines.offsetGridLines&&(t-=r/2),t-=n?a.paddingLeft:a.paddingTop,e=0>=t?0:Math.round(t/r)},getBasePixel:function(){return this.bottom}});t.scaleService.registerScaleType("category",i,a)}},{}],40:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a={position:"left",ticks:{callback:function(t,a,i){var n=i.length>3?i[2]-i[1]:i[1]-i[0];Math.abs(n)>1&&t!==Math.floor(t)&&(n=t-Math.floor(t));var o=e.log10(Math.abs(n)),r="";if(0!==t){var l=-1*Math.floor(o);l=Math.max(Math.min(l,20),0),r=t.toFixed(l)}else r="0";return r}}},i=t.LinearScaleBase.extend({determineDataLimits:function(){function t(t){return l?t.xAxisID===a.id:t.yAxisID===a.id}var a=this,i=a.options,n=a.chart,o=n.data,r=o.datasets,l=a.isHorizontal();if(a.min=null,a.max=null,i.stacked){var s={},d=!1,u=!1;e.each(r,function(o,r){var l=n.getDatasetMeta(r);void 0===s[l.type]&&(s[l.type]={positiveValues:[],negativeValues:[]});var c=s[l.type].positiveValues,h=s[l.type].negativeValues;n.isDatasetVisible(r)&&t(l)&&e.each(o.data,function(t,e){var n=+a.getRightValue(t);isNaN(n)||l.data[e].hidden||(c[e]=c[e]||0,h[e]=h[e]||0,i.relativePoints?c[e]=100:0>n?(u=!0,h[e]+=n):(d=!0,c[e]+=n))})}),e.each(s,function(t){var i=t.positiveValues.concat(t.negativeValues),n=e.min(i),o=e.max(i);a.min=null===a.min?n:Math.min(a.min,n),a.max=null===a.max?o:Math.max(a.max,o)})}else e.each(r,function(i,o){var r=n.getDatasetMeta(o);n.isDatasetVisible(o)&&t(r)&&e.each(i.data,function(t,e){var i=+a.getRightValue(t);isNaN(i)||r.data[e].hidden||(null===a.min?a.min=i:i<a.min&&(a.min=i),null===a.max?a.max=i:i>a.max&&(a.max=i))})});this.handleTickRangeOptions()},getTickLimit:function(){var a,i=this,n=i.options.ticks;if(i.isHorizontal())a=Math.min(n.maxTicksLimit?n.maxTicksLimit:11,Math.ceil(i.width/50));else{var o=e.getValueOrDefault(n.fontSize,t.defaults.global.defaultFontSize);a=Math.min(n.maxTicksLimit?n.maxTicksLimit:11,Math.ceil(i.height/(2*o)))}return a},handleDirectionalChanges:function(){this.isHorizontal()||this.ticks.reverse()},getLabelForIndex:function(t,e){return+this.getRightValue(this.chart.data.datasets[e].data[t])},getPixelForValue:function(t){var e,a,i=this,n=i.paddingLeft,o=i.paddingBottom,r=i.start,l=+i.getRightValue(t),s=i.end-r;return i.isHorizontal()?(a=i.width-(n+i.paddingRight),e=i.left+a/s*(l-r),Math.round(e+n)):(a=i.height-(i.paddingTop+o),e=i.bottom-o-a/s*(l-r),Math.round(e))},getValueForPixel:function(t){var e=this,a=e.isHorizontal(),i=e.paddingLeft,n=e.paddingBottom,o=a?e.width-(i+e.paddingRight):e.height-(e.paddingTop+n),r=(a?t-e.left-i:e.bottom-n-t)/o;return e.start+(e.end-e.start)*r},getPixelForTick:function(t){return this.getPixelForValue(this.ticksAsNumbers[t])}});t.scaleService.registerScaleType("linear",i,a)}},{}],41:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=e.noop;t.LinearScaleBase=t.Scale.extend({handleTickRangeOptions:function(){var t=this,a=t.options,i=a.ticks;if(i.beginAtZero){var n=e.sign(t.min),o=e.sign(t.max);0>n&&0>o?t.max=0:n>0&&o>0&&(t.min=0)}void 0!==i.min?t.min=i.min:void 0!==i.suggestedMin&&(t.min=Math.min(t.min,i.suggestedMin)),void 0!==i.max?t.max=i.max:void 0!==i.suggestedMax&&(t.max=Math.max(t.max,i.suggestedMax)),t.min===t.max&&(t.max++,i.beginAtZero||t.min--)},getTickLimit:a,handleDirectionalChanges:a,buildTicks:function(){var t=this,a=t.options,i=t.ticks=[],n=a.ticks,o=e.getValueOrDefault,r=t.getTickLimit();r=Math.max(2,r);var l,s=n.fixedStepSize&&n.fixedStepSize>0||n.stepSize&&n.stepSize>0;if(s)l=o(n.fixedStepSize,n.stepSize);else{var d=e.niceNum(t.max-t.min,!1);l=e.niceNum(d/(r-1),!0)}var u=Math.floor(t.min/l)*l,c=Math.ceil(t.max/l)*l,h=(c-u)/l;h=e.almostEquals(h,Math.round(h),l/1e3)?Math.round(h):Math.ceil(h),i.push(void 0!==n.min?n.min:u);for(var f=1;h>f;++f)i.push(u+f*l);i.push(void 0!==n.max?n.max:c),t.handleDirectionalChanges(),t.max=e.max(i),t.min=e.min(i),n.reverse?(i.reverse(),t.start=t.max,t.end=t.min):(t.start=t.min,t.end=t.max)},convertTicksToLabels:function(){var e=this;e.ticksAsNumbers=e.ticks.slice(),e.zeroLineIndex=e.ticks.indexOf(0),t.Scale.prototype.convertTicksToLabels.call(e)}})}},{}],42:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a={position:"left",ticks:{callback:function(t,a,i){var n=t/Math.pow(10,Math.floor(e.log10(t)));return 1===n||2===n||5===n||0===a||a===i.length-1?t.toExponential():""}}},i=t.Scale.extend({determineDataLimits:function(){function t(t){return d?t.xAxisID===a.id:t.yAxisID===a.id}var a=this,i=a.options,n=i.ticks,o=a.chart,r=o.data,l=r.datasets,s=e.getValueOrDefault,d=a.isHorizontal();if(a.min=null,a.max=null,i.stacked){var u={};e.each(l,function(n,r){var l=o.getDatasetMeta(r);o.isDatasetVisible(r)&&t(l)&&(void 0===u[l.type]&&(u[l.type]=[]),e.each(n.data,function(t,e){var n=u[l.type],o=+a.getRightValue(t);isNaN(o)||l.data[e].hidden||(n[e]=n[e]||0,i.relativePoints?n[e]=100:n[e]+=o)}))}),e.each(u,function(t){var i=e.min(t),n=e.max(t);a.min=null===a.min?i:Math.min(a.min,i),a.max=null===a.max?n:Math.max(a.max,n)})}else e.each(l,function(i,n){var r=o.getDatasetMeta(n);o.isDatasetVisible(n)&&t(r)&&e.each(i.data,function(t,e){var i=+a.getRightValue(t);isNaN(i)||r.data[e].hidden||(null===a.min?a.min=i:i<a.min&&(a.min=i),null===a.max?a.max=i:i>a.max&&(a.max=i))})});a.min=s(n.min,a.min),a.max=s(n.max,a.max),a.min===a.max&&(0!==a.min&&null!==a.min?(a.min=Math.pow(10,Math.floor(e.log10(a.min))-1),a.max=Math.pow(10,Math.floor(e.log10(a.max))+1)):(a.min=1,a.max=10))},buildTicks:function(){for(var t=this,a=t.options,i=a.ticks,n=e.getValueOrDefault,o=t.ticks=[],r=n(i.min,Math.pow(10,Math.floor(e.log10(t.min))));r<t.max;){o.push(r);var l=Math.floor(e.log10(r)),s=Math.floor(r/Math.pow(10,l))+1;10===s&&(s=1,++l),r=s*Math.pow(10,l)}var d=n(i.max,r);o.push(d),t.isHorizontal()||o.reverse(),t.max=e.max(o),t.min=e.min(o),i.reverse?(o.reverse(),t.start=t.max,t.end=t.min):(t.start=t.min,t.end=t.max)},convertTicksToLabels:function(){this.tickValues=this.ticks.slice(),t.Scale.prototype.convertTicksToLabels.call(this)},getLabelForIndex:function(t,e){return+this.getRightValue(this.chart.data.datasets[e].data[t])},getPixelForTick:function(t){return this.getPixelForValue(this.tickValues[t])},getPixelForValue:function(t){var a,i,n=this,o=n.start,r=+n.getRightValue(t),l=e.log10(n.end)-e.log10(o),s=n.paddingTop,d=n.paddingBottom,u=n.paddingLeft;return n.isHorizontal()?0===r?i=n.left+u:(a=n.width-(u+n.paddingRight),i=n.left+a/l*(e.log10(r)-e.log10(o)),i+=u):0===r?i=n.top+s:(a=n.height-(s+d),i=n.bottom-d-a/l*(e.log10(r)-e.log10(o))),i},getValueForPixel:function(t){var a,i,n=this,o=e.log10(n.end)-e.log10(n.start);return n.isHorizontal()?(i=n.width-(n.paddingLeft+n.paddingRight),a=n.start*Math.pow(10,(t-n.left-n.paddingLeft)*o/i)):(i=n.height-(n.paddingTop+n.paddingBottom),a=Math.pow(10,(n.bottom-n.paddingBottom-t)*o/i)/n.start),a}});t.scaleService.registerScaleType("logarithmic",i,a)}},{}],43:[function(t,e,a){"use strict";e.exports=function(t){var e=t.helpers,a=t.defaults.global,i={display:!0,animate:!0,lineArc:!1,position:"chartArea",angleLines:{display:!0,color:"rgba(0, 0, 0, 0.1)",lineWidth:1},ticks:{showLabelBackdrop:!0,backdropColor:"rgba(255,255,255,0.75)",backdropPaddingY:2,backdropPaddingX:2},pointLabels:{fontSize:10,callback:function(t){return t}}},n=t.LinearScaleBase.extend({getValueCount:function(){return this.chart.data.labels.length;
},setDimensions:function(){var t=this,i=t.options,n=i.ticks;t.width=t.maxWidth,t.height=t.maxHeight,t.xCenter=Math.round(t.width/2),t.yCenter=Math.round(t.height/2);var o=e.min([t.height,t.width]),r=e.getValueOrDefault(n.fontSize,a.defaultFontSize);t.drawingArea=i.display?o/2-(r/2+n.backdropPaddingY):o/2},determineDataLimits:function(){var t=this,a=t.chart;t.min=null,t.max=null,e.each(a.data.datasets,function(i,n){if(a.isDatasetVisible(n)){var o=a.getDatasetMeta(n);e.each(i.data,function(e,a){var i=+t.getRightValue(e);isNaN(i)||o.data[a].hidden||(null===t.min?t.min=i:i<t.min&&(t.min=i),null===t.max?t.max=i:i>t.max&&(t.max=i))})}}),t.handleTickRangeOptions()},getTickLimit:function(){var t=this.options.ticks,i=e.getValueOrDefault(t.fontSize,a.defaultFontSize);return Math.min(t.maxTicksLimit?t.maxTicksLimit:11,Math.ceil(this.drawingArea/(1.5*i)))},convertTicksToLabels:function(){var e=this;t.LinearScaleBase.prototype.convertTicksToLabels.call(e),e.pointLabels=e.chart.data.labels.map(e.options.pointLabels.callback,e)},getLabelForIndex:function(t,e){return+this.getRightValue(this.chart.data.datasets[e].data[t])},fit:function(){var t,i,n,o,r,l,s,d,u,c,h,f,g=this.options.pointLabels,p=e.getValueOrDefault(g.fontSize,a.defaultFontSize),m=e.getValueOrDefault(g.fontStyle,a.defaultFontStyle),b=e.getValueOrDefault(g.fontFamily,a.defaultFontFamily),v=e.fontString(p,m,b),x=e.min([this.height/2-p-5,this.width/2]),y=this.width,k=0;for(this.ctx.font=v,i=0;i<this.getValueCount();i++){t=this.getPointPosition(i,x),n=this.ctx.measureText(this.pointLabels[i]?this.pointLabels[i]:"").width+5;var S=this.getIndexAngle(i)+Math.PI/2,w=360*S/(2*Math.PI)%360;0===w||180===w?(o=n/2,t.x+o>y&&(y=t.x+o,r=i),t.x-o<k&&(k=t.x-o,s=i)):180>w?t.x+n>y&&(y=t.x+n,r=i):t.x-n<k&&(k=t.x-n,s=i)}u=k,c=Math.ceil(y-this.width),l=this.getIndexAngle(r),d=this.getIndexAngle(s),h=c/Math.sin(l+Math.PI/2),f=u/Math.sin(d+Math.PI/2),h=e.isNumber(h)?h:0,f=e.isNumber(f)?f:0,this.drawingArea=Math.round(x-(f+h)/2),this.setCenterPoint(f,h)},setCenterPoint:function(t,e){var a=this,i=a.width-e-a.drawingArea,n=t+a.drawingArea;a.xCenter=Math.round((n+i)/2+a.left),a.yCenter=Math.round(a.height/2+a.top)},getIndexAngle:function(t){var e=2*Math.PI/this.getValueCount(),a=this.chart.options&&this.chart.options.startAngle?this.chart.options.startAngle:0,i=a*Math.PI*2/360;return t*e-Math.PI/2+i},getDistanceFromCenterForValue:function(t){var e=this;if(null===t)return 0;var a=e.drawingArea/(e.max-e.min);return e.options.reverse?(e.max-t)*a:(t-e.min)*a},getPointPosition:function(t,e){var a=this,i=a.getIndexAngle(t);return{x:Math.round(Math.cos(i)*e)+a.xCenter,y:Math.round(Math.sin(i)*e)+a.yCenter}},getPointPositionForValue:function(t,e){return this.getPointPosition(t,this.getDistanceFromCenterForValue(e))},getBasePosition:function(){var t=this,e=t.min,a=t.max;return t.getPointPositionForValue(0,t.beginAtZero?0:0>e&&0>a?a:e>0&&a>0?e:0)},draw:function(){var t=this,i=t.options,n=i.gridLines,o=i.ticks,r=i.angleLines,l=i.pointLabels,s=e.getValueOrDefault;if(i.display){var d=t.ctx,u=s(o.fontSize,a.defaultFontSize),c=s(o.fontStyle,a.defaultFontStyle),h=s(o.fontFamily,a.defaultFontFamily),f=e.fontString(u,c,h);if(e.each(t.ticks,function(r,l){if(l>0||i.reverse){var c=t.getDistanceFromCenterForValue(t.ticksAsNumbers[l]),h=t.yCenter-c;if(n.display&&0!==l)if(d.strokeStyle=e.getValueAtIndexOrDefault(n.color,l-1),d.lineWidth=e.getValueAtIndexOrDefault(n.lineWidth,l-1),i.lineArc)d.beginPath(),d.arc(t.xCenter,t.yCenter,c,0,2*Math.PI),d.closePath(),d.stroke();else{d.beginPath();for(var g=0;g<t.getValueCount();g++){var p=t.getPointPosition(g,c);0===g?d.moveTo(p.x,p.y):d.lineTo(p.x,p.y)}d.closePath(),d.stroke()}if(o.display){var m=s(o.fontColor,a.defaultFontColor);if(d.font=f,o.showLabelBackdrop){var b=d.measureText(r).width;d.fillStyle=o.backdropColor,d.fillRect(t.xCenter-b/2-o.backdropPaddingX,h-u/2-o.backdropPaddingY,b+2*o.backdropPaddingX,u+2*o.backdropPaddingY)}d.textAlign="center",d.textBaseline="middle",d.fillStyle=m,d.fillText(r,t.xCenter,h)}}}),!i.lineArc){d.lineWidth=r.lineWidth,d.strokeStyle=r.color;for(var g=t.getDistanceFromCenterForValue(i.reverse?t.min:t.max),p=s(l.fontSize,a.defaultFontSize),m=s(l.fontStyle,a.defaultFontStyle),b=s(l.fontFamily,a.defaultFontFamily),v=e.fontString(p,m,b),x=t.getValueCount()-1;x>=0;x--){if(r.display){var y=t.getPointPosition(x,g);d.beginPath(),d.moveTo(t.xCenter,t.yCenter),d.lineTo(y.x,y.y),d.stroke(),d.closePath()}var k=t.getPointPosition(x,g+5),S=s(l.fontColor,a.defaultFontColor);d.font=v,d.fillStyle=S;var w=t.pointLabels,C=this.getIndexAngle(x)+Math.PI/2,M=360*C/(2*Math.PI)%360;0===M||180===M?d.textAlign="center":180>M?d.textAlign="left":d.textAlign="right",90===M||270===M?d.textBaseline="middle":M>270||90>M?d.textBaseline="bottom":d.textBaseline="top",d.fillText(w[x]?w[x]:"",k.x,k.y)}}}}});t.scaleService.registerScaleType("radialLinear",n,i)}},{}],44:[function(t,e,a){"use strict";var i=t(1);i="function"==typeof i?i:window.moment,e.exports=function(t){var e=t.helpers,a={units:[{name:"millisecond",steps:[1,2,5,10,20,50,100,250,500]},{name:"second",steps:[1,2,5,10,30]},{name:"minute",steps:[1,2,5,10,30]},{name:"hour",steps:[1,2,3,6,12]},{name:"day",steps:[1,2,5]},{name:"week",maxStep:4},{name:"month",maxStep:3},{name:"quarter",maxStep:4},{name:"year",maxStep:!1}]},n={position:"bottom",time:{parser:!1,format:!1,unit:!1,round:!1,displayFormat:!1,isoWeekday:!1,displayFormats:{millisecond:"h:mm:ss.SSS a",second:"h:mm:ss a",minute:"h:mm:ss a",hour:"MMM D, hA",day:"ll",week:"ll",month:"MMM YYYY",quarter:"[Q]Q - YYYY",year:"YYYY"}},ticks:{autoSkip:!1}},o=t.Scale.extend({initialize:function(){if(!i)throw new Error("Chart.js - Moment.js could not be found! You must include it before Chart.js to use the time scale. Download at https://momentjs.com");t.Scale.prototype.initialize.call(this)},getLabelMoment:function(t,e){return"undefined"!=typeof this.labelMoments[t]?this.labelMoments[t][e]:null},getMomentStartOf:function(t){var e=this;return"week"===e.options.time.unit&&e.options.time.isoWeekday!==!1?t.clone().startOf("isoWeek").isoWeekday(e.options.time.isoWeekday):t.clone().startOf(e.tickUnit)},determineDataLimits:function(){var t=this;t.labelMoments=[];var a=[];t.chart.data.labels&&t.chart.data.labels.length>0?(e.each(t.chart.data.labels,function(e){var i=t.parseTime(e);i.isValid()&&(t.options.time.round&&i.startOf(t.options.time.round),a.push(i))},t),t.firstTick=i.min.call(t,a),t.lastTick=i.max.call(t,a)):(t.firstTick=null,t.lastTick=null),e.each(t.chart.data.datasets,function(n,o){var r=[],l=t.chart.isDatasetVisible(o);"object"==typeof n.data[0]&&null!==n.data[0]?e.each(n.data,function(e){var a=t.parseTime(t.getRightValue(e));a.isValid()&&(t.options.time.round&&a.startOf(t.options.time.round),r.push(a),l&&(t.firstTick=null!==t.firstTick?i.min(t.firstTick,a):a,t.lastTick=null!==t.lastTick?i.max(t.lastTick,a):a))},t):r=a,t.labelMoments.push(r)},t),t.options.time.min&&(t.firstTick=t.parseTime(t.options.time.min)),t.options.time.max&&(t.lastTick=t.parseTime(t.options.time.max)),t.firstTick=(t.firstTick||i()).clone(),t.lastTick=(t.lastTick||i()).clone()},buildTicks:function(){var i=this;i.ctx.save();var n=e.getValueOrDefault(i.options.ticks.fontSize,t.defaults.global.defaultFontSize),o=e.getValueOrDefault(i.options.ticks.fontStyle,t.defaults.global.defaultFontStyle),r=e.getValueOrDefault(i.options.ticks.fontFamily,t.defaults.global.defaultFontFamily),l=e.fontString(n,o,r);if(i.ctx.font=l,i.ticks=[],i.unitScale=1,i.scaleSizeInUnits=0,i.options.time.unit)i.tickUnit=i.options.time.unit||"day",i.displayFormat=i.options.time.displayFormats[i.tickUnit],i.scaleSizeInUnits=i.lastTick.diff(i.firstTick,i.tickUnit,!0),i.unitScale=e.getValueOrDefault(i.options.time.unitStepSize,1);else{var s=i.isHorizontal()?i.width-(i.paddingLeft+i.paddingRight):i.height-(i.paddingTop+i.paddingBottom),d=i.tickFormatFunction(i.firstTick,0,[]),u=i.ctx.measureText(d).width,c=Math.cos(e.toRadians(i.options.ticks.maxRotation)),h=Math.sin(e.toRadians(i.options.ticks.maxRotation));u=u*c+n*h;var f=s/u;i.tickUnit="millisecond",i.scaleSizeInUnits=i.lastTick.diff(i.firstTick,i.tickUnit,!0),i.displayFormat=i.options.time.displayFormats[i.tickUnit];for(var g=0,p=a.units[g];g<a.units.length;){if(i.unitScale=1,e.isArray(p.steps)&&Math.ceil(i.scaleSizeInUnits/f)<e.max(p.steps)){for(var m=0;m<p.steps.length;++m)if(p.steps[m]>=Math.ceil(i.scaleSizeInUnits/f)){i.unitScale=e.getValueOrDefault(i.options.time.unitStepSize,p.steps[m]);break}break}if(p.maxStep===!1||Math.ceil(i.scaleSizeInUnits/f)<p.maxStep){i.unitScale=e.getValueOrDefault(i.options.time.unitStepSize,Math.ceil(i.scaleSizeInUnits/f));break}++g,p=a.units[g],i.tickUnit=p.name;var b=i.firstTick.diff(i.getMomentStartOf(i.firstTick),i.tickUnit,!0),v=i.getMomentStartOf(i.lastTick.clone().add(1,i.tickUnit)).diff(i.lastTick,i.tickUnit,!0);i.scaleSizeInUnits=i.lastTick.diff(i.firstTick,i.tickUnit,!0)+b+v,i.displayFormat=i.options.time.displayFormats[p.name]}}var x;if(i.options.time.min?x=i.getMomentStartOf(i.firstTick):(i.firstTick=i.getMomentStartOf(i.firstTick),x=i.firstTick),!i.options.time.max){var y=i.getMomentStartOf(i.lastTick),k=y.diff(i.lastTick,i.tickUnit,!0);0>k?i.lastTick=i.getMomentStartOf(i.lastTick.add(1,i.tickUnit)):k>=0&&(i.lastTick=y),i.scaleSizeInUnits=i.lastTick.diff(i.firstTick,i.tickUnit,!0)}i.smallestLabelSeparation=i.width,e.each(i.chart.data.datasets,function(t,e){for(var a=1;a<i.labelMoments[e].length;a++)i.smallestLabelSeparation=Math.min(i.smallestLabelSeparation,i.labelMoments[e][a].diff(i.labelMoments[e][a-1],i.tickUnit,!0))},i),i.options.time.displayFormat&&(i.displayFormat=i.options.time.displayFormat),i.ticks.push(i.firstTick.clone());for(var S=1;S<=i.scaleSizeInUnits;++S){var w=x.clone().add(S,i.tickUnit);if(i.options.time.max&&w.diff(i.lastTick,i.tickUnit,!0)>=0)break;S%i.unitScale===0&&i.ticks.push(w)}var C=i.ticks[i.ticks.length-1].diff(i.lastTick,i.tickUnit);(0!==C||0===i.scaleSizeInUnits)&&(i.options.time.max?(i.ticks.push(i.lastTick.clone()),i.scaleSizeInUnits=i.lastTick.diff(i.ticks[0],i.tickUnit,!0)):(i.ticks.push(i.lastTick.clone()),i.scaleSizeInUnits=i.lastTick.diff(i.firstTick,i.tickUnit,!0))),i.ctx.restore()},getLabelForIndex:function(t,e){var a=this,i=a.chart.data.labels&&t<a.chart.data.labels.length?a.chart.data.labels[t]:"";return"object"==typeof a.chart.data.datasets[e].data[0]&&(i=a.getRightValue(a.chart.data.datasets[e].data[t])),a.options.time.tooltipFormat&&(i=a.parseTime(i).format(a.options.time.tooltipFormat)),i},tickFormatFunction:function(t,a,i){var n=t.format(this.displayFormat),o=this.options.ticks,r=e.getValueOrDefault(o.callback,o.userCallback);return r?r(n,a,i):n},convertTicksToLabels:function(){var t=this;t.tickMoments=t.ticks,t.ticks=t.ticks.map(t.tickFormatFunction,t)},getPixelForValue:function(t,e,a){var n=this;t&&t.isValid||(t=i(n.getRightValue(t)));var o=t&&t.isValid&&t.isValid()?t:n.getLabelMoment(a,e);if(o){var r=o.diff(n.firstTick,n.tickUnit,!0),l=0!==r?r/n.scaleSizeInUnits:r;if(n.isHorizontal()){var s=n.width-(n.paddingLeft+n.paddingRight),d=s*l+n.paddingLeft;return n.left+Math.round(d)}var u=n.height-(n.paddingTop+n.paddingBottom),c=u*l+n.paddingTop;return n.top+Math.round(c)}},getPixelForTick:function(t){return this.getPixelForValue(this.tickMoments[t],null,null)},getValueForPixel:function(t){var e=this,a=e.isHorizontal()?e.width-(e.paddingLeft+e.paddingRight):e.height-(e.paddingTop+e.paddingBottom),n=(t-(e.isHorizontal()?e.left+e.paddingLeft:e.top+e.paddingTop))/a;return n*=e.scaleSizeInUnits,e.firstTick.clone().add(i.duration(n,e.tickUnit).asSeconds(),"seconds")},parseTime:function(t){var e=this;return"string"==typeof e.options.time.parser?i(t,e.options.time.parser):"function"==typeof e.options.time.parser?e.options.time.parser(t):"function"==typeof t.getMonth||"number"==typeof t?i(t):t.isValid&&t.isValid()?t:"string"!=typeof e.options.time.format&&e.options.time.format.call?(console.warn("options.time.format is deprecated and replaced by options.time.parser. See http://nnnick.github.io/Chart.js/docs-v2/#scales-time-scale"),e.options.time.format(t)):i(t,e.options.time.format)}});t.scaleService.registerScaleType("time",o,n)}},{1:1}]},{},[7])(7)});
/*!
 * angular-chart.js - An angular.js wrapper for Chart.js
 * http://jtblin.github.io/angular-chart.js/
 * Version: 1.0.0
 *
 * Copyright 2016 Jerome Touffe-Blin
 * Released under the BSD-2-Clause license
 * https://github.com/jtblin/angular-chart.js/blob/master/LICENSE
 */
!function(t){"use strict";if("object"==typeof exports)module.exports=t("undefined"!=typeof angular?angular:require("angular"),"undefined"!=typeof Chart?Chart:require("chart.js"));else if("function"==typeof define&&define.amd)define(["angular","chart"],t);else{if("undefined"==typeof angular||"undefined"==typeof Chart)throw new Error("Chart.js library needs to included, see http://jtblin.github.io/angular-chart.js/");t(angular,Chart)}}(function(t,r){"use strict";function a(){var a={responsive:!0},e={Chart:r,getOptions:function(r){var e=r&&a[r]||{};return t.extend({},a,e)}};this.setOptions=function(r,e){return e?void(a[r]=t.extend(a[r]||{},e)):(e=r,void(a=t.extend(a,e)))},this.$get=function(){return e}}function e(a,e){function o(t,r,e){var n=w(t,r);if(p(r)&&$(t,r,e,n)){var o=e[0],c=o.getContext("2d");r.chartGetColor=C(r);var i=y(t,r);F(r),r.chart=new a.Chart(c,{type:t,data:i,options:n}),r.$emit("chart-create",r.chart),D(o,r)}}function c(t,r){return t&&r&&t.length&&r.length?Array.isArray(t[0])?t.length===r.length&&t.every(function(t,a){return t.length===r[a].length}):r.reduce(i,0)>0?t.length===r.length:!1:!1}function i(t,r){return t+r}function u(r,a,e){var n=null;return function(o){var c=r.chart.getElementsAtEvent||r.chart.getPointsAtEvent;if(c){var i=c.call(r.chart,o);e!==!1&&t.equals(n,i)!==!1||(n=i,r[a](i,o))}}}function h(e,n){for(var o=t.copy(n.chartColors||a.getOptions(e).chartColors||r.defaults.global.colors),c=o.length<n.chartData.length;o.length<n.chartData.length;)o.push(n.chartGetColor());return c&&(n.chartColors=o),o.map(l)}function l(t){return"object"==typeof t&&null!==t?t:"string"==typeof t&&"#"===t[0]?f(v(t.substr(1))):s()}function s(){var t=[d(0,255),d(0,255),d(0,255)];return f(t)}function f(t){return{backgroundColor:g(t,.2),pointBackgroundColor:g(t,1),pointHoverBackgroundColor:g(t,.8),borderColor:g(t,1),pointBorderColor:"#fff",pointHoverBorderColor:g(t,1)}}function d(t,r){return Math.floor(Math.random()*(r-t+1))+t}function g(t,r){return n?"rgb("+t.join(",")+")":"rgba("+t.concat(r).join(",")+")"}function v(t){var r=parseInt(t,16),a=r>>16&255,e=r>>8&255,n=255&r;return[a,e,n]}function p(t){return t.chartData&&t.chartData.length}function C(t){return"function"==typeof t.chartGetColor?t.chartGetColor:s}function y(t,r){var a=h(t,r);return Array.isArray(r.chartData[0])?b(r.chartLabels,r.chartData,r.chartSeries||[],a,r.chartDatasetOverride):m(r.chartLabels,r.chartData,a,r.chartDatasetOverride)}function b(r,a,e,n,o){return{labels:r,datasets:a.map(function(r,a){var c=t.extend({},n[a],{label:e[a],data:r});return o&&o.length>=a&&t.merge(c,o[a]),c})}}function m(r,a,e,n){var o={labels:r,datasets:[{data:a,backgroundColor:e.map(function(t){return t.pointBackgroundColor}),hoverBackgroundColor:e.map(function(t){return t.backgroundColor})}]};return n&&t.merge(o.datasets[0],n),o}function w(r,e){return t.extend({},a.getOptions(r),e.chartOptions)}function D(r,a){r.onclick=a.chartClick?u(a,"chartClick",!1):t.noop,r.onmousemove=a.chartHover?u(a,"chartHover",!0):t.noop}function B(t,r){Array.isArray(r.chartData[0])?r.chart.data.datasets.forEach(function(r,a){r.data=t[a]}):r.chart.data.datasets[0].data=t,r.chart.update(),r.$emit("chart-update",r.chart)}function A(t){return!t||Array.isArray(t)&&!t.length||"object"==typeof t&&!Object.keys(t).length}function $(t,r,a,n){return n.responsive&&0===a[0].clientHeight?(e(function(){o(t,r,a)},50,!1),!1):!0}function F(t){t.chart&&(t.chart.destroy(),t.$emit("chart-destroy",t.chart))}return function(r){return{restrict:"CA",scope:{chartGetColor:"=?",chartType:"=",chartData:"=?",chartLabels:"=?",chartOptions:"=?",chartSeries:"=?",chartColors:"=?",chartClick:"=?",chartHover:"=?",chartDatasetOverride:"=?"},link:function(a,e){function i(t,n){if(!t||!t.length||Array.isArray(t[0])&&!t[0].length)return void F(a);var i=r||a.chartType;if(i)return a.chart&&c(t,n)?B(t,a):void o(i,a,e)}function u(n,c){if(!A(n)&&!t.equals(n,c)){var i=r||a.chartType;i&&o(i,a,e)}}function h(r,n){A(r)||t.equals(r,n)||o(r,a,e)}n&&window.G_vmlCanvasManager.initElement(e[0]),a.$watch("chartData",i,!0),a.$watch("chartSeries",u,!0),a.$watch("chartLabels",u,!0),a.$watch("chartOptions",u,!0),a.$watch("chartColors",u,!0),a.$watch("chartDatasetOverride",u,!0),a.$watch("chartType",h,!1),a.$on("$destroy",function(){F(a)}),a.$on("$resize",function(){a.chart&&a.chart.resize()})}}}}r.defaults.global.multiTooltipTemplate="<%if (datasetLabel){%><%=datasetLabel%>: <%}%><%= value %>",r.defaults.global.tooltips.mode="label",r.defaults.global.elements.line.borderWidth=2,r.defaults.global.elements.rectangle.borderWidth=2,r.defaults.global.legend.display=!1,r.defaults.global.colors=["#97BBCD","#DCDCDC","#F7464A","#46BFBD","#FDB45C","#949FB1","#4D5360"];var n="object"==typeof window.G_vmlCanvasManager&&null!==window.G_vmlCanvasManager&&"function"==typeof window.G_vmlCanvasManager.initElement;return n&&(r.defaults.global.animation=!1),t.module("chart.js",[]).provider("ChartJs",a).factory("ChartJsFactory",["ChartJs","$timeout",e]).directive("chartBase",["ChartJsFactory",function(t){return new t}]).directive("chartLine",["ChartJsFactory",function(t){return new t("line")}]).directive("chartBar",["ChartJsFactory",function(t){return new t("bar")}]).directive("chartHorizontalBar",["ChartJsFactory",function(t){return new t("horizontalBar")}]).directive("chartRadar",["ChartJsFactory",function(t){return new t("radar")}]).directive("chartDoughnut",["ChartJsFactory",function(t){return new t("doughnut")}]).directive("chartPie",["ChartJsFactory",function(t){return new t("pie")}]).directive("chartPolarArea",["ChartJsFactory",function(t){return new t("polarArea")}]).directive("chartBubble",["ChartJsFactory",function(t){return new t("bubble")}]).name});
//# sourceMappingURL=angular-chart.min.js.map

/*! globalize - v0.1.0pre - 2012-10-18
* http://wiki.jqueryui.com/Globalize
* Copyright 2012 ; Licensed MIT */
(function(e,t){var n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x,T,N;n=function(e){return new n.prototype.init(e)},typeof require!="undefined"&&typeof exports!="undefined"&&typeof module!="undefined"?module.exports=n:e.Globalize=n,n.cultures={},n.prototype={constructor:n,init:function(e){return this.cultures=n.cultures,this.cultureSelector=e,this}},n.prototype.init.prototype=n.prototype,n.cultures["default"]={name:"en",englishName:"English",nativeName:"English",isRTL:!1,language:"en",numberFormat:{pattern:["-n"],decimals:2,",":",",".":".",groupSizes:[3],"+":"+","-":"-",NaN:"NaN",negativeInfinity:"-Infinity",positiveInfinity:"Infinity",percent:{pattern:["-n %","n %"],decimals:2,groupSizes:[3],",":",",".":".",symbol:"%"},currency:{pattern:["($n)","$n"],decimals:2,groupSizes:[3],",":",",".":".",symbol:"$"}},calendars:{standard:{name:"Gregorian_USEnglish","/":"/",":":":",firstDay:0,days:{names:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],namesAbbr:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],namesShort:["Su","Mo","Tu","We","Th","Fr","Sa"]},months:{names:["January","February","March","April","May","June","July","August","September","October","November","December",""],namesAbbr:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec",""]},AM:["AM","am","AM"],PM:["PM","pm","PM"],eras:[{name:"A.D.",start:null,offset:0}],twoDigitYearMax:2029,patterns:{d:"M/d/yyyy",D:"dddd, MMMM dd, yyyy",t:"h:mm tt",T:"h:mm:ss tt",f:"dddd, MMMM dd, yyyy h:mm tt",F:"dddd, MMMM dd, yyyy h:mm:ss tt",M:"MMMM dd",Y:"yyyy MMMM",S:"yyyy'-'MM'-'dd'T'HH':'mm':'ss"}}},messages:{}},n.cultures["default"].calendar=n.cultures["default"].calendars.standard,n.cultures.en=n.cultures["default"],n.cultureSelector="en",r=/^0x[a-f0-9]+$/i,i=/^[+\-]?infinity$/i,s=/^[+\-]?\d*\.?\d*(e[+\-]?\d+)?$/,o=/^\s+|\s+$/g,u=function(e,t){if(e.indexOf)return e.indexOf(t);for(var n=0,r=e.length;n<r;n++)if(e[n]===t)return n;return-1},a=function(e,t){return e.substr(e.length-t.length)===t},f=function(){var e,n,r,i,s,o,u=arguments[0]||{},a=1,p=arguments.length,d=!1;typeof u=="boolean"&&(d=u,u=arguments[1]||{},a=2),typeof u!="object"&&!c(u)&&(u={});for(;a<p;a++)if((e=arguments[a])!=null)for(n in e){r=u[n],i=e[n];if(u===i)continue;d&&i&&(h(i)||(s=l(i)))?(s?(s=!1,o=r&&l(r)?r:[]):o=r&&h(r)?r:{},u[n]=f(d,o,i)):i!==t&&(u[n]=i)}return u},l=Array.isArray||function(e){return Object.prototype.toString.call(e)==="[object Array]"},c=function(e){return Object.prototype.toString.call(e)==="[object Function]"},h=function(e){return Object.prototype.toString.call(e)==="[object Object]"},p=function(e,t){return e.indexOf(t)===0},d=function(e){return(e+"").replace(o,"")},v=function(e){return isNaN(e)?NaN:Math[e<0?"ceil":"floor"](e)},m=function(e,t,n){var r;for(r=e.length;r<t;r+=1)e=n?"0"+e:e+"0";return e},g=function(e,t){var n=0,r=!1;for(var i=0,s=e.length;i<s;i++){var o=e.charAt(i);switch(o){case"'":r?t.push("'"):n++,r=!1;break;case"\\":r&&t.push("\\"),r=!r;break;default:t.push(o),r=!1}}return n},y=function(e,t){t=t||"F";var n,r=e.patterns,i=t.length;if(i===1){n=r[t];if(!n)throw"Invalid date format string '"+t+"'.";t=n}else i===2&&t.charAt(0)==="%"&&(t=t.charAt(1));return t},b=function(e,t,n){function T(e,t){var n,r=e+"";return t>1&&r.length<t?(n=c[t-2]+r,n.substr(n.length-t,t)):(n=r,n)}function N(){return h||p?h:(h=d.test(t),p=!0,h)}function C(e,t){if(w)return w[t];switch(t){case 0:return e.getFullYear();case 1:return e.getMonth();case 2:return e.getDate();default:throw"Invalid part value "+t}}var r=n.calendar,i=r.convert,s;if(!t||!t.length||t==="i"){if(n&&n.name.length)if(i)s=b(e,r.patterns.F,n);else{var o=new Date(e.getTime()),u=S(e,r.eras);o.setFullYear(x(e,r,u)),s=o.toLocaleString()}else s=e.toString();return s}var a=r.eras,f=t==="s";t=y(r,t),s=[];var l,c=["0","00","000"],h,p,d=/([^d]|^)(d|dd)([^d]|$)/g,v=0,m=E(),w;!f&&i&&(w=i.fromGregorian(e));for(;;){var k=m.lastIndex,L=m.exec(t),A=t.slice(k,L?L.index:t.length);v+=g(A,s);if(!L)break;if(v%2){s.push(L[0]);continue}var O=L[0],M=O.length;switch(O){case"ddd":case"dddd":var _=M===3?r.days.namesAbbr:r.days.names;s.push(_[e.getDay()]);break;case"d":case"dd":h=!0,s.push(T(C(e,2),M));break;case"MMM":case"MMMM":var D=C(e,1);s.push(r.monthsGenitive&&N()?r.monthsGenitive[M===3?"namesAbbr":"names"][D]:r.months[M===3?"namesAbbr":"names"][D]);break;case"M":case"MM":s.push(T(C(e,1)+1,M));break;case"y":case"yy":case"yyyy":D=w?w[0]:x(e,r,S(e,a),f),M<4&&(D%=100),s.push(T(D,M));break;case"h":case"hh":l=e.getHours()%12,l===0&&(l=12),s.push(T(l,M));break;case"H":case"HH":s.push(T(e.getHours(),M));break;case"m":case"mm":s.push(T(e.getMinutes(),M));break;case"s":case"ss":s.push(T(e.getSeconds(),M));break;case"t":case"tt":D=e.getHours()<12?r.AM?r.AM[0]:" ":r.PM?r.PM[0]:" ",s.push(M===1?D.charAt(0):D);break;case"f":case"ff":case"fff":s.push(T(e.getMilliseconds(),3).substr(0,M));break;case"z":case"zz":l=e.getTimezoneOffset()/60,s.push((l<=0?"+":"-")+T(Math.floor(Math.abs(l)),M));break;case"zzz":l=e.getTimezoneOffset()/60,s.push((l<=0?"+":"-")+T(Math.floor(Math.abs(l)),2)+":"+T(Math.abs(e.getTimezoneOffset()%60),2));break;case"g":case"gg":r.eras&&s.push(r.eras[S(e,a)].name);break;case"/":s.push(r["/"]);break;default:throw"Invalid date format pattern '"+O+"'."}}return s.join("")},function(){var e;e=function(e,t,n){var r=n.groupSizes,i=r[0],s=1,o=Math.pow(10,t),u=Math.round(e*o)/o;isFinite(u)||(u=e),e=u;var a=e+"",f="",l=a.split(/e/i),c=l.length>1?parseInt(l[1],10):0;a=l[0],l=a.split("."),a=l[0],f=l.length>1?l[1]:"";var h;c>0?(f=m(f,c,!1),a+=f.slice(0,c),f=f.substr(c)):c<0&&(c=-c,a=m(a,c+1,!0),f=a.slice(-c,a.length)+f,a=a.slice(0,-c)),t>0?f=n["."]+(f.length>t?f.slice(0,t):m(f,t)):f="";var p=a.length-1,d=n[","],v="";while(p>=0){if(i===0||i>p)return a.slice(0,p+1)+(v.length?d+v+f:f);v=a.slice(p-i+1,p+1)+(v.length?d+v:""),p-=i,s<r.length&&(i=r[s],s++)}return a.slice(0,p+1)+d+v+f},w=function(t,n,r){if(!isFinite(t))return t===Infinity?r.numberFormat.positiveInfinity:t===-Infinity?r.numberFormat.negativeInfinity:r.numberFormat.NaN;if(!n||n==="i")return r.name.length?t.toLocaleString():t.toString();n=n||"D";var i=r.numberFormat,s=Math.abs(t),o=-1,u;n.length>1&&(o=parseInt(n.slice(1),10));var a=n.charAt(0).toUpperCase(),f;switch(a){case"D":u="n",s=v(s),o!==-1&&(s=m(""+s,o,!0)),t<0&&(s="-"+s);break;case"N":f=i;case"C":f=f||i.currency;case"P":f=f||i.percent,u=t<0?f.pattern[0]:f.pattern[1]||"n",o===-1&&(o=f.decimals),s=e(s*(a==="P"?100:1),o,f);break;default:throw"Bad number format specifier: "+a}var l=/n|\$|-|%/g,c="";for(;;){var h=l.lastIndex,p=l.exec(u);c+=u.slice(h,p?p.index:u.length);if(!p)break;switch(p[0]){case"n":c+=s;break;case"$":c+=i.currency.symbol;break;case"-":/[1-9]/.test(s)&&(c+=i["-"]);break;case"%":c+=i.percent.symbol}}return c}}(),E=function(){return/\/|dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|y|hh|h|HH|H|mm|m|ss|s|tt|t|fff|ff|f|zzz|zz|z|gg|g/g},S=function(e,t){if(!t)return 0;var n,r=e.getTime();for(var i=0,s=t.length;i<s;i++){n=t[i].start;if(n===null||r>=n)return i}return 0},x=function(e,t,n,r){var i=e.getFullYear();return!r&&t.eras&&(i-=t.eras[n].offset),i},function(){var e,t,n,r,i,s,o;e=function(e,t){if(t<100){var n=new Date,r=S(n),i=x(n,e,r),s=e.twoDigitYearMax;s=typeof s=="string"?(new Date).getFullYear()%100+parseInt(s,10):s,t+=i-i%100,t>s&&(t-=100)}return t},t=function(e,t,n){var r,i=e.days,a=e._upperDays;return a||(e._upperDays=a=[o(i.names),o(i.namesAbbr),o(i.namesShort)]),t=s(t),n?(r=u(a[1],t),r===-1&&(r=u(a[2],t))):r=u(a[0],t),r},n=function(e,t,n){var r=e.months,i=e.monthsGenitive||e.months,a=e._upperMonths,f=e._upperMonthsGen;a||(e._upperMonths=a=[o(r.names),o(r.namesAbbr)],e._upperMonthsGen=f=[o(i.names),o(i.namesAbbr)]),t=s(t);var l=u(n?a[1]:a[0],t);return l<0&&(l=u(n?f[1]:f[0],t)),l},r=function(e,t){var n=e._parseRegExp;if(!n)e._parseRegExp=n={};else{var r=n[t];if(r)return r}var i=y(e,t).replace(/([\^\$\.\*\+\?\|\[\]\(\)\{\}])/g,"\\\\$1"),s=["^"],o=[],u=0,a=0,f=E(),l;while((l=f.exec(i))!==null){var c=i.slice(u,l.index);u=f.lastIndex,a+=g(c,s);if(a%2){s.push(l[0]);continue}var h=l[0],p=h.length,d;switch(h){case"dddd":case"ddd":case"MMMM":case"MMM":case"gg":case"g":d="(\\D+)";break;case"tt":case"t":d="(\\D*)";break;case"yyyy":case"fff":case"ff":case"f":d="(\\d{"+p+"})";break;case"dd":case"d":case"MM":case"M":case"yy":case"y":case"HH":case"H":case"hh":case"h":case"mm":case"m":case"ss":case"s":d="(\\d\\d?)";break;case"zzz":d="([+-]?\\d\\d?:\\d{2})";break;case"zz":case"z":d="([+-]?\\d\\d?)";break;case"/":d="(\\/)";break;default:throw"Invalid date format pattern '"+h+"'."}d&&s.push(d),o.push(l[0])}g(i.slice(u),s),s.push("$");var v=s.join("").replace(/\s+/g,"\\s+"),m={regExp:v,groups:o};return n[t]=m},i=function(e,t,n){return e<t||e>n},s=function(e){return e.split("� ").join(" ").toUpperCase()},o=function(e){var t=[];for(var n=0,r=e.length;n<r;n++)t[n]=s(e[n]);return t},T=function(s,o,u){s=d(s);var a=u.calendar,f=r(a,o),l=(new RegExp(f.regExp)).exec(s);if(l===null)return null;var c=f.groups,h=null,v=null,m=null,g=null,y=null,b=0,w,E=0,S=0,x=0,T=null,N=!1;for(var C=0,k=c.length;C<k;C++){var L=l[C+1];if(L){var A=c[C],O=A.length,M=parseInt(L,10);switch(A){case"dd":case"d":g=M;if(i(g,1,31))return null;break;case"MMM":case"MMMM":m=n(a,L,O===3);if(i(m,0,11))return null;break;case"M":case"MM":m=M-1;if(i(m,0,11))return null;break;case"y":case"yy":case"yyyy":v=O<4?e(a,M):M;if(i(v,0,9999))return null;break;case"h":case"hh":b=M,b===12&&(b=0);if(i(b,0,11))return null;break;case"H":case"HH":b=M;if(i(b,0,23))return null;break;case"m":case"mm":E=M;if(i(E,0,59))return null;break;case"s":case"ss":S=M;if(i(S,0,59))return null;break;case"tt":case"t":N=a.PM&&(L===a.PM[0]||L===a.PM[1]||L===a.PM[2]);if(!N&&(!a.AM||L!==a.AM[0]&&L!==a.AM[1]&&L!==a.AM[2]))return null;break;case"f":case"ff":case"fff":x=M*Math.pow(10,3-O);if(i(x,0,999))return null;break;case"ddd":case"dddd":y=t(a,L,O===3);if(i(y,0,6))return null;break;case"zzz":var _=L.split(/:/);if(_.length!==2)return null;w=parseInt(_[0],10);if(i(w,-12,13))return null;var D=parseInt(_[1],10);if(i(D,0,59))return null;T=w*60+(p(L,"-")?-D:D);break;case"z":case"zz":w=M;if(i(w,-12,13))return null;T=w*60;break;case"g":case"gg":var P=L;if(!P||!a.eras)return null;P=d(P.toLowerCase());for(var H=0,B=a.eras.length;H<B;H++)if(P===a.eras[H].name.toLowerCase()){h=H;break}if(h===null)return null}}}var j=new Date,F,I=a.convert;F=I?I.fromGregorian(j)[0]:j.getFullYear(),v===null?v=F:a.eras&&(v+=a.eras[h||0].offset),m===null&&(m=0),g===null&&(g=1);if(I){j=I.toGregorian(v,m,g);if(j===null)return null}else{j.setFullYear(v,m,g);if(j.getDate()!==g)return null;if(y!==null&&j.getDay()!==y)return null}N&&b<12&&(b+=12),j.setHours(b,E,S,x);if(T!==null){var q=j.getMinutes()-(T+j.getTimezoneOffset());j.setHours(j.getHours()+parseInt(q/60,10),q%60)}return j}}(),N=function(e,t,n){var r=t["-"],i=t["+"],s;switch(n){case"n -":r=" "+r,i=" "+i;case"n-":a(e,r)?s=["-",e.substr(0,e.length-r.length)]:a(e,i)&&(s=["+",e.substr(0,e.length-i.length)]);break;case"- n":r+=" ",i+=" ";case"-n":p(e,r)?s=["-",e.substr(r.length)]:p(e,i)&&(s=["+",e.substr(i.length)]);break;case"(n)":p(e,"(")&&a(e,")")&&(s=["-",e.substr(1,e.length-2)])}return s||["",e]},n.prototype.findClosestCulture=function(e){return n.findClosestCulture.call(this,e)},n.prototype.format=function(e,t,r){return n.format.call(this,e,t,r)},n.prototype.localize=function(e,t){return n.localize.call(this,e,t)},n.prototype.parseInt=function(e,t,r){return n.parseInt.call(this,e,t,r)},n.prototype.parseFloat=function(e,t,r){return n.parseFloat.call(this,e,t,r)},n.prototype.culture=function(e){return n.culture.call(this,e)},n.addCultureInfo=function(e,t,n){var r={},i=!1;typeof e!="string"?(n=e,e=this.culture().name,r=this.cultures[e]):typeof t!="string"?(n=t,i=this.cultures[e]==null,r=this.cultures[e]||this.cultures["default"]):(i=!0,r=this.cultures[t]),this.cultures[e]=f(!0,{},r,n),i&&(this.cultures[e].calendar=this.cultures[e].calendars.standard)},n.findClosestCulture=function(e){var t;if(!e)return this.findClosestCulture(this.cultureSelector)||this.cultures["default"];typeof e=="string"&&(e=e.split(","));if(l(e)){var n,r=this.cultures,i=e,s,o=i.length,u=[];for(s=0;s<o;s++){e=d(i[s]);var a,f=e.split(";");n=d(f[0]),f.length===1?a=1:(e=d(f[1]),e.indexOf("q=")===0?(e=e.substr(2),a=parseFloat(e),a=isNaN(a)?0:a):a=1),u.push({lang:n,pri:a})}u.sort(function(e,t){return e.pri<t.pri?1:e.pri>t.pri?-1:0});for(s=0;s<o;s++){n=u[s].lang,t=r[n];if(t)return t}for(s=0;s<o;s++){n=u[s].lang;do{var c=n.lastIndexOf("-");if(c===-1)break;n=n.substr(0,c),t=r[n];if(t)return t}while(1)}for(s=0;s<o;s++){n=u[s].lang;for(var h in r){var p=r[h];if(p.language==n)return p}}}else if(typeof e=="object")return e;return t||null},n.format=function(e,t,n){var r=this.findClosestCulture(n);return e instanceof Date?e=b(e,t,r):typeof e=="number"&&(e=w(e,t,r)),e},n.localize=function(e,t){return this.findClosestCulture(t).messages[e]||this.cultures["default"].messages[e]},n.parseDate=function(e,t,n){n=this.findClosestCulture(n);var r,i,s;if(t){typeof t=="string"&&(t=[t]);if(t.length)for(var o=0,u=t.length;o<u;o++){var a=t[o];if(a){r=T(e,a,n);if(r)break}}}else{s=n.calendar.patterns;for(i in s){r=T(e,s[i],n);if(r)break}}return r||null},n.parseInt=function(e,t,r){return v(n.parseFloat(e,t,r))},n.parseFloat=function(e,t,n){typeof t!="number"&&(n=t,t=10);var o=this.findClosestCulture(n),u=NaN,a=o.numberFormat;e.indexOf(o.numberFormat.currency.symbol)>-1&&(e=e.replace(o.numberFormat.currency.symbol,""),e=e.replace(o.numberFormat.currency["."],o.numberFormat["."])),e.indexOf(o.numberFormat.percent.symbol)>-1&&(e=e.replace(o.numberFormat.percent.symbol,"")),e=e.replace(/ /g,"");if(i.test(e))u=parseFloat(e);else if(!t&&r.test(e))u=parseInt(e,16);else{var f=N(e,a,a.pattern[0]),l=f[0],c=f[1];l===""&&a.pattern[0]!=="(n)"&&(f=N(e,a,"(n)"),l=f[0],c=f[1]),l===""&&a.pattern[0]!=="-n"&&(f=N(e,a,"-n"),l=f[0],c=f[1]),l=l||"+";var h,p,d=c.indexOf("e");d<0&&(d=c.indexOf("E")),d<0?(p=c,h=null):(p=c.substr(0,d),h=c.substr(d+1));var v,m,g=a["."],y=p.indexOf(g);y<0?(v=p,m=null):(v=p.substr(0,y),m=p.substr(y+g.length));var b=a[","];v=v.split(b).join("");var w=b.replace(/\u00A0/g," ");b!==w&&(v=v.split(w).join(""));var E=l+v;m!==null&&(E+="."+m);if(h!==null){var S=N(h,a,"-n");E+="e"+(S[0]||"+")+S[1]}s.test(E)&&(u=parseFloat(E))}return u},n.culture=function(e){return typeof e!="undefined"&&(this.cultureSelector=e),this.findClosestCulture(e)||this.cultures["default"]}})(this);
/*!
* DevExtreme (dx.all.js)
* Version: 20.2.5
* Build date: Fri Jan 15 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
"use strict";!function(e){function t(i){if(n[i])return n[i].exports;var o=n[i]={i:i,l:!1,exports:{}};return e[i].call(o.exports,o,o.exports,t),o.l=!0,o.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,i){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:i})},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=1005)}([function(e,t,n){t.extend=t.extendFromObject=void 0;var i=n(1),o=function(e,t,n){e=e||{};for(var i in t)if(Object.prototype.hasOwnProperty.call(t,i)){var o=t[i];i in e&&!n||(e[i]=o)}return e};t.extendFromObject=o;var a=function e(t){t=t||{};var n=1,o=!1;for("boolean"==typeof t&&(o=t,t=arguments[1]||{},n++);n<arguments.length;n++){var a=arguments[n];if(null!=a)for(var r in a){var s=t[r],l=a[r],u=!1,c=void 0;"__proto__"!==r&&t!==l&&(o&&l&&((0,i.isPlainObject)(l)||(u=Array.isArray(l)))?(c=u?s&&Array.isArray(s)?s:[]:s&&(0,i.isPlainObject)(s)?s:{},t[r]=e(o,c,l)):void 0!==l&&(t[r]=l))}}return t};t.extend=a},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.isEvent=t.type=t.isDeferred=t.isPromise=t.isRenderer=t.isWindow=t.isPrimitive=t.isPlainObject=t.isEmptyObject=t.isObject=t.isNumeric=t.isString=t.isFunction=t.isDefined=t.isDate=t.isExponential=t.isBoolean=void 0;var o={"[object Array]":"array","[object Date]":"date","[object Object]":"object","[object String]":"string","[object Null]":"null"},a=function(e){var t=Object.prototype.toString.call(e);return"object"===i(e)?o[t]||"object":i(e)};t.type=a;var r=function(e){return"boolean"==typeof e};t.isBoolean=r;var s=function(e){return h(e)&&e.toString().indexOf("e")!==-1};t.isExponential=s;var l=function(e){return"date"===a(e)};t.isDate=l;var u=function(e){return null!==e&&void 0!==e};t.isDefined=u;var c=function(e){return"function"==typeof e};t.isFunction=c;var d=function(e){return"string"==typeof e};t.isString=d;var h=function(e){return"number"==typeof e&&isFinite(e)||!isNaN(e-parseFloat(e))};t.isNumeric=h;var f=function(e){return"object"===a(e)};t.isObject=f;var p=function(e){var t;for(t in e)return!1;return!0};t.isEmptyObject=p;var g=function(e){if(!e||"[object Object]"!==Object.prototype.toString.call(e))return!1;var t=Object.getPrototypeOf(e),n=Object.hasOwnProperty.call(t,"constructor")&&t.constructor;return"function"==typeof n&&Object.toString.call(n)===Object.toString.call(Object)};t.isPlainObject=g;var _=function(e){return["object","array","function"].indexOf(a(e))===-1};t.isPrimitive=_;var m=function(e){return null!=e&&e===e.window};t.isWindow=m;var v=function(e){return!(!e.jquery&&!e.dxRenderer)};t.isRenderer=v;var y=function(e){return e&&c(e.then)};t.isPromise=y;var b=function(e){return e&&c(e.done)&&c(e.fail)};t.isDeferred=b;var x=function(e){return!(!e||!e.preventDefault)};t.isEvent=x},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(236)),a=o.default.get();t.default=a,e.exports=t.default},function(e,t,n){t.reverseEach=t.each=t.map=void 0;var i=function(e,t){if(Array.isArray(e))return e.map(t);var n=[];for(var i in e)n.push(t(e[i],i));return n};t.map=i;var o=function(e,t){if(e){if("length"in e)for(var n=0;n<e.length&&t.call(e[n],n,e[n])!==!1;n++);else for(var i in e)if(t.call(e[i],i,e[i])===!1)break;return e}};t.each=o;var a=function(e,t){if(e&&"length"in e&&0!==e.length)for(var n=e.length-1;n>=0&&t.call(e[n],n,e[n])!==!1;n--);};t.reverseEach=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.equalByValue=t.grep=t.asyncNoop=t.noop=t.applyServerDecimalSeparator=t.escapeRegExp=t.getKeyHash=t.pairToObject=t.denormalizeKey=t.normalizeKey=t.splitPair=t.findBestMatches=t.deferUpdater=t.deferRenderer=t.deferUpdate=t.deferRender=t.executeAsync=t.ensureDefined=void 0;var o=i(n(28)),a=i(n(35)),r=n(6),s=n(19),l=n(3),u=n(1),c=function(e,t){return(0,u.isDefined)(e)?e:t};t.ensureDefined=c;var d=function(e,t){var n=new r.Deferred,i=t||this,o={promise:n.promise(),abort:function(){clearTimeout(s),n.rejectWith(i)}},a=function(){var t=e.call(i);t&&t.done&&(0,u.isFunction)(t.done)?t.done(function(){n.resolveWith(i)}):n.resolveWith(i)},s=(arguments[2]||setTimeout)(a,"number"==typeof t?t:0);return o};t.executeAsync=d;var h,f=[],p=[],g=[],_=function(e,t,n){if(h&&h!==e)return f.push(t),p.push(e),n=n||new r.Deferred,g.push(n),n;var i=h,o=g.length;h=e;var a=t();return a||(g.length>o?a=r.when.apply(this,g.slice(o)):n&&n.resolve()),h=i,n&&a&&a.done&&a.done(n.resolve).fail(n.reject),!h&&f.length&&("render"===p.shift()?m:v)(f.shift(),g.shift()),a||(0,r.when)()},m=function(e,t){return _("render",e,t)};t.deferRender=m;var v=function(e,t){return _("update",e,t)};t.deferUpdate=v;var y=function(e){return function(){var t=this;return _("render",function(){return e.call(t)})}};t.deferRenderer=y;var b=function(e){return function(){var t=this;return _("update",function(){return e.call(t)})}};t.deferUpdater=b;var x=function(e,t,n){var i=[],o=0;return(0,l.each)(t,function(t,a){var r=0,s=n?n(a):a;(0,l.each)(e,function(e,t){var n=s[e];if(void 0!==n)return w(n,t)?void r++:(r=-1,!1)}),r<o||(r>o&&(i.length=0,o=r),i.push(a))}),i};t.findBestMatches=x;var w=function(e,t){if(Array.isArray(e)&&Array.isArray(t)){var n=!1;return(0,l.each)(e,function(e,i){if(i!==t[e])return n=!0,!1}),!n}return e===t},C=function(e){var t,n;switch((0,u.type)(e)){case"string":return e.split(/\s+/,2);case"object":return[null!==(t=e.x)&&void 0!==t?t:e.h,null!==(n=e.y)&&void 0!==n?n:e.v];case"number":return[e];case"array":return e;default:return null}};t.splitPair=C;var k=function(e){var t=(0,u.isString)(e)?e:e.toString(),n=t.match(/[^a-zA-Z0-9_]/g);return n&&(0,l.each)(n,function(e,n){t=t.replace(n,"__"+n.charCodeAt()+"__")}),t};t.normalizeKey=k;var S=function(e){var t=e.match(/__\d+__/g);return t&&t.forEach(function(t){var n=parseInt(t.replace("__",""));e=e.replace(t,String.fromCharCode(n))}),e};t.denormalizeKey=S;var D=function(e,t){var n=C(e),i=t?parseFloat(n&&n[0]):parseInt(n&&n[0],10),o=t?parseFloat(n&&n[1]):parseInt(n&&n[1],10);return isFinite(i)||(i=0),isFinite(o)||(o=i),{h:i,v:o}};t.pairToObject=D;var I=function(e){if(e instanceof a.default)return e.toString();if((0,u.isObject)(e)||Array.isArray(e))try{var t=JSON.stringify(e);return"{}"===t?e:t}catch(t){return e}return e};t.getKeyHash=I;var T=function(e){return e.replace(/[[\]{}\-()*+?.\\^$|\s]/g,"\\$&")};t.escapeRegExp=T;var O=function(e){var t=(0,o.default)().serverDecimalSeparator;return(0,u.isDefined)(e)&&(e=e.toString().replace(".",t)),e};t.applyServerDecimalSeparator=O;var E=function(){};t.noop=E;var A=function(){return(new r.Deferred).resolve().promise()};t.asyncNoop=A;var P=function(e,t,n){for(var i,o=[],a=!n,r=0;r<e.length;r++)i=!!t(e[r],r),i===a&&o.push(e[r]);return o};t.grep=P;var R=function(e,t,n){if(e.length!==t.length)return!1;for(var i=0;i<e.length;i++)if(!V(e[i],t[i],n+1))return!1;return!0},M=function(e,t,n,i){for(var o in e)if(Object.prototype.hasOwnProperty.call(e,o)&&!V(e[o],t[o],n+1,i))return!1;for(var a in t)if(!(a in e))return!1;return!0},B=3,V=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3];e=(0,s.toComparable)(e,!0),t=(0,s.toComparable)(t,!0);var o=i?e===t:e==t;return!!(o||n>=B)||((0,u.isObject)(e)&&(0,u.isObject)(t)?M(e,t,n,i):!(!Array.isArray(e)||!Array.isArray(t))&&R(e,t,n))};t.equalByValue=V},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){return!(0,I.isWindow)(e)&&"#document"!==e.nodeName&&k.default.elementMatches(e,t)}function r(e,t){var n=e[0];if(n)if(k.default.isNode(n)||(0,I.isWindow)(n))t.apply(j,e);else{if((0,I.isString)(n)||!("length"in n))throw O.default.Error("E0025");var i=Array.prototype.slice.call(e,0);Array.prototype.forEach.call(n,function(e){i[0]=e,r(i,t)})}}function s(e){return function(){r(arguments,e)}}function l(){var e=!1;try{var t=Object.defineProperty({},"passive",{get:function(){return e=!0,!0}});R.addEventListener("test",null,t)}catch(e){}return e}function u(e,t){var n=H.get(e);t=t||"";var i=t.split("."),o=i.slice(1),r=!!i[0];t=i[0]||M,n||(n={},H.set(e,n)),n[t]||(n[t]={handleObjects:[],nativeHandler:null});var s=n[t];return{addHandler:function(n,i,l){var u=function(i,o){var a,r,s=[i],u=i.currentTarget,c=i.relatedTarget;t in B&&(a=c&&u&&(c===u||G(u,c))),void 0!==o&&s.push(o),N.callMethod(t,"handle",e,[i,l]),a||(r=n.apply(u,s)),r===!1&&(i.preventDefault(),i.stopPropagation())},d=function(t,n){if(!x||t.type!==x)if(t.data=l,t.delegateTarget=e,i)for(var o=t.target;o&&o!==e;)a(o,i)&&(t.currentTarget=o,u(t,n)),o=o.parentNode;else t.currentTarget=t.delegateTarget||t.target,u(t,n)},h={handler:n,wrappedHandler:d,selector:i,type:t,data:l,namespace:o.join("."),namespaces:o,guid:++z};s.handleObjects.push(h);var f,p=1===s.handleObjects.length,g=p&&r;g&&(g=!N.callMethod(t,"setup",e,[l,o,n])),g&&(s.nativeHandler=c(t),W()&&L.indexOf(t)>-1&&(f={passive:!1}),s.removeListener=k.default.listen(e,B[t]||t,s.nativeHandler,f)),N.callMethod(t,"add",e,[h])},removeHandler:function(i,a){var s=function(t){var r=n[t];if(!r.handleObjects.length)return void delete n[t];var s;r.handleObjects=r.handleObjects.filter(function(n){var r=o.length&&!d(n.namespaces,o)||i&&n.handler!==i||a&&n.selector!==a;return r||(s=n.handler,N.callMethod(t,"remove",e,[n])),r});var l=!r.handleObjects.length,u=l&&t!==M;u&&(N.callMethod(t,"teardown",e,[o,s]),r.nativeHandler&&r.removeListener(),delete n[t])};if(r)s(t);else for(var l in n)s(l);var u=0===Object.keys(n).length;u&&H.delete(e)},callHandlers:function(e,t){var i=!1,a=function(n){i||o.length&&!d(n.namespaces,o)||(n.wrappedHandler(e,t),i=e.isImmediatePropagationStopped())};s.handleObjects.forEach(a),o.length&&n[M]&&n[M].handleObjects.forEach(a)}}}function c(e){return function(t,n){var i=u(this,e);t=j.Event(t),i.callHandlers(t,n)}}function d(e,t){for(var n=0;n<t.length;n++)if(e.indexOf(t[n])<0)return!1;return!0}function h(e){return function(t,n,i,o,a){a||(a=o,o=void 0),"string"!=typeof i&&(o=i,i=void 0),a||"string"!=typeof n||(a=o||i,i=void 0,o=void 0),e(t,n,i,o,a)}}function f(e){return function(t,n,i,o){"function"==typeof i&&(o=i,i=void 0),e(t,n,i,o)}}function p(e){return function(t,n,i){"string"==typeof n&&(n={type:n}),n.target||(n.target=t),n.currentTarget=t,n.delegateTarget||(n.delegateTarget=t),!n.type&&n.originalEvent&&(n.type=n.originalEvent.type),e(t,n instanceof j.Event?n:j.Event(n),i)}}function g(e){return function(t,n){return this instanceof j.Event?(t||(t={}),"string"==typeof t&&(t={type:t}),n||(n={}),void e.call(this,t,n)):new j.Event(t,n)}}function _(e){var t=function(t,n){if(n&&n.indexOf(" ")>-1){var i=Array.prototype.slice.call(arguments,0);n.split(" ").forEach(function(t){i[1]=t,e.apply(this,i)})}else e.apply(this,arguments)};return function(e,n){if("object"===o(n)){var i=Array.prototype.slice.call(arguments,0);for(var a in n)i[1]=a,i[i.length-1]=n[a],t.apply(this,i)}else t.apply(this,arguments)}}function m(e,t){var n=V[e]||e,i=function(e,t){return"click"===e&&"a"===t.localName};i(e,t)||(0,I.isFunction)(t[n])&&(x=e,t[n](),x=void 0)}function v(e){var t=function(e){var t=/^(?:mouse|pointer|contextmenu|drag|drop)|click/;return!e.which&&void 0!==e.button&&t.test(e.type)},n=function(e){return null==e.which&&0===e.type.indexOf("key")};if(n(e))return null!=e.charCode?e.charCode:e.keyCode;if(t(e)){var i={1:1,2:3,3:1,4:2};return i[e.button]}return e.which}function y(e){e&&(j.Event=e,j.Event.prototype=e.prototype)}function b(e,t,n){Object.defineProperty(n||j.Event.prototype,e,{enumerable:!0,configurable:!0,get:function(){return this.originalEvent&&t(this.originalEvent)},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})}t.default=void 0;var x,w=i(n(116)),C=n(0),k=i(n(11)),S=n(7),D=i(n(54)),I=n(1),T=i(n(27)),O=i(n(22)),E=i(n(182)),A=i(n(237)),P=i(n(64)),R=(0,S.getWindow)(),M="dxEmptyEventType",B={mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},V={focusin:"focus",focusout:"blur"},F=["blur","focus","load"],L=["touchmove","wheel","mousewheel","touchstart"],H=new E.default,z=0,N=function(){var e={};return w.default.add(function(t,n){e[t]=n}),{getField:function(t,n){return e[t]&&e[t][n]},callMethod:function(t,n,i,o){return e[t]&&e[t][n]&&e[t][n].apply(i,o)}}}(),j=(0,D.default)({on:s(h(_(function(e,t,n,i,o){var a=u(e,t);a.addHandler(o,n,i)}))),one:s(h(function(e,t,n,i,o){var a=function i(){j.off(e,t,n,i),o.apply(this,arguments)};j.on(e,t,n,i,a)})),off:s(f(_(function(e,t,n,i){var o=u(e,t);o.removeHandler(i,n)}))),trigger:s(p(function(e,t,n){var i=t.type,o=u(e,t.type);N.callMethod(i,"trigger",e,[t,n]),o.callHandlers(t,n);var a=N.getField(i,"noBubble")||t.isPropagationStopped()||F.indexOf(i)!==-1;if(!a){var r=[],s=function e(t){var n=t.parentNode;n&&(r.push(n),e(n))};s(e),r.push(R);for(var l=0;r[l]&&!t.isPropagationStopped();){var c=u(r[l],t.type);c.callHandlers((0,C.extend)(t,{currentTarget:r[l]}),n),l++}}(e.nodeType||(0,I.isWindow)(e))&&(N.callMethod(i,"_default",e,[t,n]),m(i,e))})),triggerHandler:s(p(function(e,t,n){var i=u(e,t.type);i.callHandlers(t,n)}))}),W=(0,P.default)(l),G=function e(t,n){return(0,I.isWindow)(t)?e(t.document,n):t.contains?t.contains(n):!!(n.compareDocumentPosition(t)&n.DOCUMENT_POSITION_CONTAINS)};y(g(function(e,t){var n=this,i=!1,o=!1,a=!1;(0,C.extend)(n,e),(e instanceof j.Event||(0,S.hasWindow)()&&e instanceof R.Event)&&(n.originalEvent=e,n.currentTarget=void 0),e instanceof j.Event||(0,C.extend)(n,{isPropagationStopped:function(){return!!(i||n.originalEvent&&n.originalEvent.propagationStopped)},stopPropagation:function(){i=!0,n.originalEvent&&n.originalEvent.stopPropagation()},isImmediatePropagationStopped:function(){return o},stopImmediatePropagation:function(){this.stopPropagation(),o=!0,n.originalEvent&&n.originalEvent.stopImmediatePropagation()},isDefaultPrevented:function(){return!!(a||n.originalEvent&&n.originalEvent.defaultPrevented)},preventDefault:function(){a=!0,n.originalEvent&&n.originalEvent.preventDefault()}}),b("which",v,n),0===e.type.indexOf("touch")&&(delete t.pageX,delete t.pageY),(0,C.extend)(n,t),n.guid=++z})),(0,A.default)(b);var q=(0,T.default)(),$=(0,T.default)();j.set=function(e){q.fire(),j.inject(e),y(e.Event),$.fire()},j.subscribeGlobal=function(){r(arguments,h(function(){var e=arguments;j.on.apply(this,e),q.add(function(){var t=Array.prototype.slice.call(e,0);t.splice(3,1),j.off.apply(this,t)}),$.add(function(){j.on.apply(this,e)})}))},j.forcePassiveFalseEventNames=L,j.passiveEventHandlersSupported=W;var K=j;t.default=K,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if((0,l.isDeferred)(e))return e;if((0,l.isPromise)(e)){var n=new h;return e.then(function(){n.resolveWith.apply(n,[t].concat([[].slice.call(arguments)]))},function(){n.rejectWith.apply(n,[t].concat([[].slice.call(arguments)]))}),n}return(new h).resolveWith(t,[e])}function a(e){h=e.Deferred,f=e.when}function r(){return new h}function s(){return f.apply(this,arguments)}t.fromPromise=o,t.setStrategy=a,t.Deferred=r,t.when=s;var l=n(1),u=n(0),c=i(n(27)),d=[{method:"resolve",handler:"done",state:"resolved"},{method:"reject",handler:"fail",state:"rejected"},{method:"notify",handler:"progress"}],h=function(){var e=this;this._state="pending",this._promise={},d.forEach(function(t){var n=t.method;this[n+"Callbacks"]=new c.default,this[n]=function(){return this[n+"With"](this._promise,arguments)}.bind(this),this._promise[t.handler]=function(t){if(!t)return this;var i=e[n+"Callbacks"];return i.fired()?t.apply(e[n+"Context"],e[n+"Args"]):i.add(function(e,n){t.apply(e,n)}.bind(this)),this}}.bind(this)),this._promise.always=function(e){return this.done(e).fail(e)},this._promise.catch=function(e){return this.then(null,e)},this._promise.then=function(e,t){var n=new h;return["done","fail"].forEach(function(i){var o="done"===i?e:t;this[i](function(){if(!o)return void n["done"===i?"resolve":"reject"].apply(this,arguments);var e=o&&o.apply(this,arguments);(0,l.isDeferred)(e)?e.done(n.resolve).fail(n.reject):(0,l.isPromise)(e)?e.then(n.resolve,n.reject):n.resolve.apply(this,(0,l.isDefined)(e)?[e]:arguments)})}.bind(this)),n.promise()},this._promise.state=function(){return e._state},this._promise.promise=function(t){return t?(0,u.extend)(t,e._promise):e._promise},this._promise.promise(this)};d.forEach(function(e){var t=e.method,n=e.state;h.prototype[t+"With"]=function(e,i){var o=this[t+"Callbacks"];return"pending"===this.state()&&(this[t+"Args"]=i,this[t+"Context"]=e,n&&(this._state=n),o.fire(e,i)),this}});var f=function(){if(1===arguments.length)return o(arguments[0]);for(var e=[].slice.call(arguments),t=[],n=0,i=new h,a=function(o){return function(a){t[o]=this,e[o]=arguments.length>1?[].slice.call(arguments):a,n++,n===e.length&&i.resolveWith(t,e)}},r=0;r<e.length;r++)(0,l.isDeferred)(e[r])?e[r].promise().done(a(r)).fail(i.reject):n++;return n===e.length&&i.resolveWith(t,e),i.promise()}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getNavigator=t.getCurrentScreenFactor=t.defaultScreenFactorFunc=t.hasProperty=t.getWindow=t.hasWindow=void 0;var o=i(n(11)),a=function(){return"undefined"!=typeof window};t.hasWindow=a;var r=a()&&window;r||(r={},r.window=r);var s=function(){return r};t.getWindow=s;var l=function(e){return a()&&e in r};t.hasProperty=l;var u=function(e){return e<768?"xs":e<992?"sm":e<1200?"md":"lg"};t.defaultScreenFactorFunc=u;var c=function(e){var t=e||u,n=o.default.getDocumentElement().clientWidth;return t(n)};t.getCurrentScreenFactor=c;var d=function(){return a()?r.navigator:{userAgent:""}};t.getNavigator=d},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.isCommandKeyPressed=t.addNamespace=t.getChar=t.normalizeKeyName=t.fireEvent=t.createEvent=t.setEventFixMethod=t.needSkipEvent=t.stopEventsSkipping=t.forceSkipEvents=t.hasTouches=t.eventDelta=t.eventData=t.isFakeClickEvent=t.isKeyboardEvent=t.isTouchEvent=t.isDxMouseWheelEvent=t.isMouseEvent=t.isPointerEvent=t.eventSource=void 0;var o=i(n(2)),a=i(n(364)),r=i(n(5)),s=n(3),l=n(0),u=n(75),c={backspace:"backspace",tab:"tab",enter:"enter",escape:"escape",pageup:"pageUp",pagedown:"pageDown",end:"end",home:"home",arrowleft:"leftArrow",arrowup:"upArrow",arrowright:"rightArrow",arrowdown:"downArrow","delete":"del"," ":"space",f:"F",a:"A","*":"asterisk","-":"minus",alt:"alt",control:"control",shift:"shift",left:"leftArrow",up:"upArrow",right:"rightArrow",down:"downArrow",multiply:"asterisk",spacebar:"space",del:"del",subtract:"minus",esc:"escape"},d={8:"backspace",9:"tab",13:"enter",27:"escape",33:"pageUp",34:"pageDown",35:"end",36:"home",37:"leftArrow",38:"upArrow",39:"rightArrow",40:"downArrow",46:"del",32:"space",70:"F",65:"A",106:"asterisk",109:"minus",189:"minus",173:"minus",16:"shift",17:"control",18:"alt"},h={dx:/^dx/i,mouse:/(mouse|wheel)/i,touch:/^touch/i,keyboard:/^key/i,pointer:/^(ms)?pointer/i},f=function(e){return e},p=function(e){return f(r.default.Event(e,e),e)},g=function(e){return"dx"===v(e)},_=function(e){return"mouse"===v(e)},m=function(e){return"touch"===v(e)},v=function(e){var t=e.type,n="other";return(0,s.each)(h,function(e){if(this.test(t))return n=e,!1}),n};t.eventSource=v;var y=function(e){return"pointer"===v(e)};t.isPointerEvent=y;var b=function(e){return _(e)||(y(e)||g(e))&&"mouse"===e.pointerType};t.isMouseEvent=b;var x=function(e){return e&&"dxmousewheel"===e.type};t.isDxMouseWheelEvent=x;var w=function(e){return m(e)||(y(e)||g(e))&&"touch"===e.pointerType};t.isTouchEvent=w;var C=function(e){return"keyboard"===v(e)};t.isKeyboardEvent=C;var k=function(e){var t=e.screenX,n=e.offsetX,i=e.pageX;return 0===t&&!n&&0===i};t.isFakeClickEvent=k;var S=function(e){var t=e.pageX,n=e.pageY,i=e.timeStamp;return{x:t,y:n,time:i}};t.eventData=S;var D=function(e,t){return{x:t.x-e.x,y:t.y-e.y,time:t.time-e.time||1}};t.eventDelta=D;var I=function(e){var t=e.originalEvent,n=e.pointers;return m(e)?(t.touches||[]).length:g(e)?(n||[]).length:0};t.hasTouches=I;var T=!1,O=function(){return T=!0};t.forceSkipEvents=O;var E=function(){return T=!1};t.stopEventsSkipping=E;var A=function(e){if(T)return!0;var t=e.target,n=(0,o.default)(t),i=n.is("input, textarea, select");if(n.is(".dx-skip-gesture-event *, .dx-skip-gesture-event"))return!0;if(x(e)){var a=n.is("textarea")&&n.hasClass("dx-texteditor-input");if(a)return!1;var r=t.isContentEditable||t.hasAttribute("contenteditable");if(r)return!1;var s=n.is("input[type='number'], textarea, select")&&n.is(":focus");return s}return b(e)?i||e.which>1:w(e)?i&&(0,u.focused)(n):void 0};t.needSkipEvent=A;var P=function(e){return f=e};t.setEventFixMethod=P;var R=function(e,t){var n=p(e);return t&&(0,l.extend)(n,t),n};t.createEvent=R;var M=function(e){var t=e.originalEvent,n=e.delegateTarget,i=R(t,e);return r.default.trigger(n||i.target,i),i};t.fireEvent=M;var B=function(e){var t=e.key,n=e.which,i=!!t;if(t=i?t:n)return t=i?c[t.toLowerCase()]||t:d[t]||String.fromCharCode(t)};t.normalizeKeyName=B;var V=function(e){var t=e.key,n=e.which;return t||String.fromCharCode(n)};t.getChar=V;var F=a.default;t.addNamespace=F;var L=function(e){var t=e.ctrlKey,n=e.metaKey;return t||n};t.isCommandKeyPressed=L},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(155)),r=i(n(22)),s=n(136),l=function(e,t,n){n?t[e]=n:n=t,(0,s.name)(n,e),a.default.fire(e,n)},u=function(e,t){o.default.fn[e]=function(n){var i,o="string"==typeof n;if(o){var a=n,s=[].slice.call(arguments).slice(1);this.each(function(){var n=t.getInstance(this);if(!n)throw r.default.Error("E0009",e);var o=n[a],l=o.apply(n,s);void 0===i&&(i=l)})}else this.each(function(){var e=t.getInstance(this);e?e.option(n):new t(this,n)}),i=this;return i}};a.default.add(u);var c=l;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(54)),r=n(0),s=n(3),l=n(42),u=n(40),c=i(n(82)),d=i(n(352)),h=(0,r.extend)(!0,{},d.default),f=function(e,t){return e[t]||{}},p={},g=(0,a.default)({engine:function(){return"base"},_dictionary:h,load:function(e){(0,r.extend)(!0,this._dictionary,e)},_localizablePrefix:"@",setup:function(e){this._localizablePrefix=e},localizeString:function(e){var t=this,n=new RegExp("(^|[^a-zA-Z_0-9"+t._localizablePrefix+"-]+)("+t._localizablePrefix+"{1,2})([a-zA-Z_0-9-]+)","g"),i=t._localizablePrefix+t._localizablePrefix;return e.replace(n,function(e,n,o,a){var r,s=t._localizablePrefix+a;return o!==i&&(r=t.format(a)),r||(p[a]=(0,u.humanize)(a)),n+(r||s)})},localizeNode:function(e){var t=this;(0,o.default)(e).each(function(e,n){n.nodeType&&(3===n.nodeType?n.nodeValue=t.localizeString(n.nodeValue):(0,o.default)(n).is("iframe")||((0,s.each)(n.attributes||[],function(e,n){if("string"==typeof n.value){var i=t.localizeString(n.value);n.value!==i&&(n.value=i)}}),(0,o.default)(n).contents().each(function(e,n){t.localizeNode(n)})))})},getMessagesByLocales:function(){return this._dictionary},getDictionary:function(e){return e?p:(0,r.extend)({},p,this.getMessagesByLocales()[c.default.locale()])},getFormatter:function(e){return this._getFormatterBase(e)||this._getFormatterBase(e,"en")},_getFormatterBase:function(e,t){var n=this,i=c.default.getValueByClosestLocale(function(t){return f(n._dictionary,t)[e]});if(i)return function(){var e=1===arguments.length&&Array.isArray(arguments[0])?arguments[0].slice(0):Array.prototype.slice.call(arguments,0);return e.unshift(i),l.format.apply(this,e)}},format:function(e){var t=this.getFormatter(e),n=Array.prototype.slice.call(arguments,1);return t&&t.apply(this,n)||""}}),_=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(54)),r=n(4),s=1,l=3,u=9,c={querySelectorAll:function(e,t){return e.querySelectorAll(t)},elementMatches:function(e,t){var n=this,i=e.matches||e.matchesSelector||e.mozMatchesSelector||e.msMatchesSelector||e.oMatchesSelector||e.webkitMatchesSelector||function(t){var i=e.document||e.ownerDocument;if(!i)return!1;for(var o=n.querySelectorAll(i,t),a=0;a<o.length;a++)if(o[a]===e)return!0};return i.call(e,t)},createElement:function(e,t){return t=t||this._document,t.createElement(e)},createElementNS:function(e,t,n){return n=n||this._document,n.createElementNS(e,t)},createTextNode:function(e,t){return t=t||this._document,t.createTextNode(e)},isNode:function(e){return"object"===o(e)&&"nodeType"in e},isElementNode:function(e){return e&&e.nodeType===s},isTextNode:function(e){return e&&e.nodeType===l},isDocument:function(e){return e&&e.nodeType===u},removeElement:function(e){var t=e&&e.parentNode;t&&t.removeChild(e)},insertElement:function(e,t,n){e&&t&&e!==t&&(n?e.insertBefore(t,n):e.appendChild(t))},getAttribute:function(e,t){return e.getAttribute(t)},setAttribute:function(e,t,n){e.setAttribute(t,n)},removeAttribute:function(e,t){e.removeAttribute(t)},setProperty:function(e,t,n){e[t]=n},setText:function(e,t){e&&(e.textContent=t)},setClass:function(e,t,n){if(1===e.nodeType&&t)if(e.classList)n?e.classList.add(t):e.classList.remove(t);else{var i,o="string"==typeof e.className,a=o?e.className:this.getAttribute(e,"class")||"",r=a.split(" "),s=r.indexOf(t);n&&s<0&&(i=a?a+" "+t:t),!n&&s>=0&&(r.splice(s,1),i=r.join(" ")),void 0!==i&&(o?e.className=i:this.setAttribute(e,"class",i))}},setStyle:function(e,t,n){e.style[t]=n||""},_document:"undefined"==typeof document?void 0:document,getDocument:function(){return this._document},getActiveElement:function(){return this._document.activeElement},getBody:function(){return this._document.body},createDocumentFragment:function(){return this._document.createDocumentFragment()},getDocumentElement:function(){return this._document.documentElement},getLocation:function(){return this._document.location},getSelection:function(){return this._document.selection},getReadyState:function(){return this._document.readyState},getHead:function(){return this._document.head},hasDocumentProperty:function(e){return e in this._document},listen:function(e,t,n,i){return e&&"addEventListener"in e?(e.addEventListener(t,n,i),function(){e.removeEventListener(t,n)}):r.noop}},d=(0,a.default)(c);t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function c(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function d(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?c(Object(n),!0).forEach(function(t){h(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):c(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function h(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.groupBy=t.find=t.merge=t.normalizeIndexes=t.removeDuplicates=t.intersection=t.inArray=t.wrapToArray=t.isEmpty=void 0;var f=n(1),p=n(3),g=n(50),_=i(n(28)),m=function(e){return Array.isArray(e)&&!e.length};t.isEmpty=m;var v=function(e){return Array.isArray(e)?e:[e]};t.wrapToArray=v;var y=function(e,t){if(!t)return-1;var n=Array.isArray(t)?t:t.toArray();return n.indexOf(e)};t.inArray=y;var b=function(e,t){if(!Array.isArray(e)||0===e.length||!Array.isArray(t)||0===t.length)return[];var n=[];return(0,p.each)(e,function(e,i){var o=y(i,t);o!==-1&&n.push(i)}),n};t.intersection=b;var x=function(e,t){if(!Array.isArray(e)||0===e.length)return[];if(!Array.isArray(t)||0===t.length)return e.slice();var n=[];return(0,p.each)(e,function(e,i){var o=y(i,t);o===-1&&n.push(i)}),n};t.removeDuplicates=x;var w=function(e,t,n,i){var o={},a=0,r=(0,_.default)().useLegacyVisibleIndex;return(0,p.each)(e,function(e,i){e=i[t],e>=0?(o[e]=o[e]||[],i===n?o[e].unshift(i):o[e].push(i)):i[t]=void 0}),r||(0,p.each)(e,function(){if(!(0,f.isDefined)(this[t])&&(!i||i(this))){for(;o[a];)a++;o[a]=[this],a++}}),a=0,(0,g.orderEach)(o,function(e,n){(0,p.each)(n,function(){e>=0&&(this[t]=a++)})}),r&&(0,p.each)(e,function(){(0,f.isDefined)(this[t])||i&&!i(this)||(this[t]=a++)}),a};t.normalizeIndexes=w;var C=function(e,t){for(var n=0;n<t.length;n++)e[e.length]=t[n];return e};t.merge=C;var k=function(e,t){for(var n=0;n<e.length;n++)if(t(e[n]))return e[n]};t.find=k;var S=function(e,t){return e.reduce(function(e,n){return d(d({},e),{},h({},t(n),[].concat(o(e[t(n)]||[]),[n])))},{})};t.groupBy=S},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=n(7),u=n(0),c=n(1),d=n(3),h=i(n(22)),f=i(n(27)),p=i(n(49)),g=i(n(114)),_=n(92),m=n(269),v=n(78),y=i(n(28)),b=(0,l.getNavigator)(),x=(0,l.getWindow)(),w={iPhone:"iPhone",iPhone5:"iPhone",iPhone6:"iPhone",iPhone6plus:"iPhone",iPad:"iPad",iPadMini:"iPad Mini",androidPhone:"Android Mobile",androidTablet:"Android",msSurface:"Windows ARM Tablet PC",desktop:"desktop"},C={deviceType:"desktop",platform:"generic",version:[],phone:!1,tablet:!1,android:!1,ios:!1,generic:!0,grade:"A",mac:!1},k={generic:function(e){var t=/windows phone/i.test(e)||e.match(/WPDesktop/),n=!t&&/Windows(.*)arm(.*)Tablet PC/i.test(e),i=!t&&!n&&/msapphost/i.test(e),o=/((intel|ppc) mac os x)/.test(e.toLowerCase());if(t||n||i||o)return{deviceType:t?"phone":n?"tablet":"desktop",platform:"generic",version:[],grade:"A",mac:o}},ios:function(e){if(/ip(hone|od|ad)/i.test(e)){var t=/ip(hone|od)/i.test(e),n=e.match(/os (\d+)_(\d+)_?(\d+)?/i),i=n?[parseInt(n[1],10),parseInt(n[2],10),parseInt(n[3]||0,10)]:[],o=480===x.screen.height,a=o?"B":"A";return{deviceType:t?"phone":"tablet",platform:"ios",version:i,grade:a}}},android:function(e){if(/android|htc_|silk/i.test(e)){var t=/mobile/i.test(e),n=e.match(/android (\d+)\.?(\d+)?\.?(\d+)?/i),i=n?[parseInt(n[1],10),parseInt(n[2]||0,10),parseInt(n[3]||0,10)]:[],o=i.length>1&&(i[0]<4||4===i[0]&&i[1]<4),a=o?"B":"A";return{deviceType:t?"phone":"tablet",platform:"android",version:i,grade:a}}}},S=function(){function e(t){o(this,e),this._window=(null===t||void 0===t?void 0:t.window)||x,this._realDevice=this._getDevice(),
this._currentDevice=void 0,this._currentOrientation=void 0,this._eventsStrategy=new _.EventsStrategy(this),this.changed=(0,f.default)(),(0,l.hasWindow)()&&(p.default.add(this._recalculateOrientation.bind(this)),g.default.add(this._recalculateOrientation.bind(this)))}return r(e,[{key:"current",value:function(e){if(e)return this._currentDevice=this._getDevice(e),this._forced=!0,void this.changed.fire();if(!this._currentDevice){e=void 0;try{e=this._getDeviceOrNameFromWindowScope()}catch(t){e=this._getDeviceNameFromSessionStorage()}finally{e||(e=this._getDeviceNameFromSessionStorage()),e&&(this._forced=!0)}this._currentDevice=this._getDevice(e)}return this._currentDevice}},{key:"real",value:function(e){return(0,u.extend)({},this._realDevice)}},{key:"orientation",value:function(){return this._currentOrientation}},{key:"isForced",value:function(){return this._forced}},{key:"isRippleEmulator",value:function(){return!!this._window.tinyHippos}},{key:"_getCssClasses",value:function(e){var t=[],n=this._realDevice;return e=e||this.current(),e.deviceType&&(t.push("dx-device-".concat(e.deviceType)),"desktop"!==e.deviceType&&t.push("dx-device-mobile")),t.push("dx-device-".concat(n.platform)),n.version&&n.version.length&&t.push("dx-device-".concat(n.platform,"-").concat(n.version[0])),this.isSimulator()&&t.push("dx-simulator"),(0,y.default)().rtlEnabled&&t.push("dx-rtl"),t}},{key:"attachCssClasses",value:function(e,t){this._deviceClasses=this._getCssClasses(t).join(" "),(0,s.default)(e).addClass(this._deviceClasses)}},{key:"detachCssClasses",value:function(e){(0,s.default)(e).removeClass(this._deviceClasses)}},{key:"isSimulator",value:function(){try{return this._isSimulator||(0,l.hasWindow)()&&this._window.top!==this._window.self&&this._window.top["dx-force-device"]||this.isRippleEmulator()}catch(e){return!1}}},{key:"forceSimulator",value:function(){this._isSimulator=!0}},{key:"_getDevice",value:function(e){if("genericPhone"===e&&(e={deviceType:"phone",platform:"generic",generic:!0}),(0,c.isPlainObject)(e))return this._fromConfig(e);var t;if(e){if(t=w[e],!t)throw h.default.Error("E0005")}else t=b.userAgent;return this._fromUA(t)}},{key:"_getDeviceOrNameFromWindowScope",value:function(){var e;return(0,l.hasWindow)()&&(this._window.top["dx-force-device-object"]||this._window.top["dx-force-device"])&&(e=this._window.top["dx-force-device-object"]||this._window.top["dx-force-device"]),e}},{key:"_getDeviceNameFromSessionStorage",value:function(){var e=(0,m.sessionStorage)();if(e){var t=e.getItem("dx-force-device");try{return JSON.parse(t)}catch(e){return t}}}},{key:"_fromConfig",value:function(e){var t=(0,u.extend)({},C,this._currentDevice,e),n={phone:"phone"===t.deviceType,tablet:"tablet"===t.deviceType,android:"android"===t.platform,ios:"ios"===t.platform,generic:"generic"===t.platform};return(0,u.extend)(t,n)}},{key:"_fromUA",value:function(e){var t;return(0,d.each)(k,function(n,i){return t=i(e),!t}),t?this._fromConfig(t):C}},{key:"_changeOrientation",value:function(){var e=(0,s.default)(this._window),t=e.height()>e.width()?"portrait":"landscape";this._currentOrientation!==t&&(this._currentOrientation=t,this._eventsStrategy.fireEvent("orientationChanged",[{orientation:t}]))}},{key:"_recalculateOrientation",value:function(){var e=(0,s.default)(this._window).width();this._currentWidth!==e&&(this._currentWidth=e,this._changeOrientation())}},{key:"on",value:function(e,t){return this._eventsStrategy.on(e,t),this}},{key:"off",value:function(e,t){return this._eventsStrategy.off(e,t),this}}]),e}(),D=new S;v.changeCallback.add(function(e,t){D.detachCssClasses(t),D.attachCssClasses(e)});var I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){for(var n,i=0,o=e.length,a=[];i<o;)n=t(e[i],i),null!==n&&a.push(n),i++;return a}function a(e,t){return o(t,function(t){return e[t]?e[t]:null})}function r(e,t,n,i){var o=i;return(0,O.each)(t,function(t,i){e[i]&&(e[i]-=n,o-=n)}),o}function s(e){return String(e).toLowerCase()}function l(e){return e&&(e.originalTop=e.top,e.originalBottom=e.bottom,e.originalLeft=e.left,e.originalRight=e.right),e}function u(e){return-Y<e&&e<+Y?e:0}function c(e){var t=u(L(e.x)),n=u(L(e.y)),i=u(H(e.width+e.x)),o=u(H(e.height+e.y)),a={x:t,y:n,width:i-t,height:o-n};return a.isEmpty=!(a.x||a.y||a.width||a.height),a}function d(e,t,n){var i=Q(q(n*Z).toFixed(3)),o=Q($(n*Z).toFixed(3)),a=e.width/2,r=e.height/2,s=e.x+a,l=e.y+r,u=V(a*i)+V(r*o),d=V(a*o)+V(r*i),h=t[0]+(s-t[0])*i+(l-t[1])*o,f=t[1]-(s-t[0])*o+(l-t[1])*i;return c({x:h-u,y:f-d,width:2*u,height:2*d})}function h(e,t){var n=Array.prototype.slice,i=n.call(P.default.getDocument().styleSheets).reduce(function(e,t){return e.concat(n.call(t.cssRules||t.rules))},[]),o=i.filter(function(t){try{return P.default.elementMatches(e,t.selectorText)}catch(e){return!1}});return o.some(function(e){return!!e.style[t]})}function f(e,t,n,i){var o=90,a=i>0?i:0;n=(0,I.isDefined)(n)?n+t-o:0;var r=se(n);return{x:K(e.x+a*r.cos),y:K(e.y+a*r.sin)}}function p(e){e.forEach(function(e){var t=e.height,n=0,i=parseFloat(t)||void 0;((0,I.isString)(t)&&t.indexOf("px")>-1||(0,I.isNumeric)(t)&&t>1)&&(i=K(i),n=1),!n&&i&&((0,I.isString)(t)&&t.indexOf("%")>-1?i/=100:i<0&&(i=i<-1?1:V(i))),e.height=i,e.unit=n});var t=e.filter(function(e){return!e.unit}).reduce(function(e,t){return e+(t.height||0)},0),n=e.filter(function(e){return!e.unit}).length,i=e.filter(function(e){return!e.unit&&!e.height}).length;if(t<1&&i)e.filter(function(e){return!e.unit&&!e.height}).forEach(function(e){return e.height=(1-t)/i});else if(t>1||t<1&&!i||1===t&&i){if(i){var o=t/n,a=i*o;e.filter(function(e){return!e.unit&&e.height}).forEach(function(e){return e.height*=(t-a)/t}),e.filter(function(e){return!e.unit&&!e.height}).forEach(function(e){return e.height=o})}e.forEach(function(e){return!e.unit&&(e.height*=1/t)})}}function g(e,t,n){var i=0,o=ee,a=n?t.width-t.left-t.right:t.height-t.top-t.bottom,r=a-o*(e.length-1),s=n?"left":"top",u=n?"right":"bottom",c=e.reduce(function(e,t){return e+(t.unit?t.height:0)},0);r-=c,e.forEach(function(e){var n=e.unit?e.height:K(e.height*r);e.canvas=e.canvas||{},(0,T.extend)(e.canvas,t),e.canvas[s]=t[s]+i,e.canvas[u]=t[u]+(a-n-i),i=i+n+o,l(e.canvas)})}function _(e,t){return{checkInterval:e.checkInterval||t.checkInterval,size:G(e.size||0,t.size||0),percentStick:e.percentStick||t.percentStick,sizePointNormalState:G(e.sizePointNormalState||0,t.sizePointNormalState||0)}}function m(e){return Array.isArray(e)?{startValue:e[0],endValue:e[1]}:e||{}}function v(e,t){return t?e:[e.startValue,e.endValue]}function y(e,t){return"datetime"===e.dataType?function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1;return new Date(e.getTime()+n*t)}:"logarithmic"===e.axisType?function(t,n){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1,o=x(t,e.base)+i*n;return w(o,e.base)}:function(e,n){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1,o=e+i*n;return t&&o*e<=0?0:o}}function b(e,t,n,i){var o=(0,I.isDefined)(t.startValue),a=(0,I.isDefined)(t.endValue),r="discrete"!==e.axisType;i=i||n;var s=y(e,!1),l=o?t.startValue:i.min,u=a?t.endValue:i.max,c=t.length,d=i.categories;if(r&&!(0,I.isDefined)(l)&&!(0,I.isDefined)(u))return{startValue:l,endValue:u};if((0,I.isDefined)(c))if(r)"datetime"!==e.dataType||(0,I.isNumeric)(c)||(c=U(c)),a&&!o||!a&&!o?((0,I.isDefined)(n.max)&&(u=u>n.max?n.max:u),l=s(u,c,-1)):o&&!a&&((0,I.isDefined)(n.min)&&(l=l<n.min?n.min:l),u=s(l,c));else if(c=parseInt(c),!isNaN(c)&&isFinite(c))if(c--,a||o){if(o&&!a){var h=be(d,l,void 0);u=h.categories[c]}else if(!o&&a){var f=be(d,void 0,u);l=f.categories[f.categories.length-1-c]}}else u=d[d.length-1],l=d[d.length-1-c];return r&&((0,I.isDefined)(n.max)&&u>n.max&&(u=n.max),(0,I.isDefined)(n.min)&&l<n.min&&(l=n.min)),{startValue:l,endValue:u}}function x(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=arguments.length>3?arguments[3]:void 0;if(!n)return te(e,t);if(0===e)return 0;var o=te(V(e),t)-(i-1);return o<0?0:(0,E.adjust)((0,E.sign)(e)*o,Number(z(t,i-1).toFixed(V(i))))}function w(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=arguments.length>3?arguments[3]:void 0;if(!n)return ie(e,t);if(0===e)return 0;var o=ie(V(e)+(i-1),t);return o<0?0:(0,E.adjust)((0,E.sign)(e)*o,Number(z(t,i).toFixed(V(i))))}function C(e,t){return Array.isArray(t)?e.length===t.length&&e.every(function(e,n){return k(e)===k(t[n])}):k(e.startValue)===k(t.startValue)&&k(e.endValue)===k(t.endValue)}function k(e){return e&&e.valueOf()}function S(e,t,n){return t>=e.left&&t<=e.right&&n>=e.top&&n<=e.bottom}t.map=o,t.normalizeEnum=s,t.setCanvasValues=l,t.normalizeBBox=c,t.rotateBBox=d,t.checkElementHasPropertyFromStyleSheet=h,t.convertPolarToXY=f,t.normalizePanesHeight=p,t.updatePanesCanvases=g,t.mergeMarginOptions=_,t.getVizRangeObject=m,t.convertVisualRangeObject=v,t.getAddFunction=y,t.adjustVisualRange=b,t.getLogExt=x,t.raiseToExt=w,t.rangesAreEqual=C,t.valueOf=k,t.pointInCanvas=S,t.getVerticallyShiftedAngularCoords=t.unique=t.getCategoriesInfo=t.processSeriesTemplate=t.convertXYToPolar=t.patchFontOptions=t.enumParser=t.parseScalar=t.decreaseGaps=t.getPower=t.roundValue=t.getAppropriateFormat=t.getDecimalOrder=t.getDistance=t.getCosAndSin=t.degreesToRadians=t.convertAngleToRendererSpace=t.normalizeAngle=t.raiseTo=t.getAdjustedLog10=t.getLog=t.PANE_PADDING=void 0;var D=n(4),I=n(1),T=n(0),O=n(3),E=n(29),A=i(n(20)),P=i(n(11)),R=i(n(98)),M=Math.PI,B=Math.LN10,V=Math.abs,F=Math.log,L=Math.floor,H=Math.ceil,z=Math.pow,N=Math.sqrt,j=Math.atan2,W=Math.min,G=Math.max,q=Math.cos,$=Math.sin,K=Math.round,U=A.default.dateToMilliseconds,Y=1e10,Z=M/180,X=isNaN,Q=Number,J=NaN,ee=10;t.PANE_PADDING=ee;var te=function(e,t){return e?F(e)/F(t):J};t.getLog=te;var ne=function(e){return(0,E.adjust)(te(e,10))};t.getAdjustedLog10=ne;var ie=function(e,t){return z(t,e)};t.raiseTo=ie;var oe=function(e){return(e%360+360)%360};t.normalizeAngle=oe;var ae=function(e){return 90-e};t.convertAngleToRendererSpace=ae;var re=function(e){return M*e/180};t.degreesToRadians=re;var se=function(e){var t=re(e);return{cos:q(t),sin:$(t)}};t.getCosAndSin=se;var le=1e-14,ue=function(e,t,n,i){var o=n-e,a=i-t;return N(a*a+o*o)};t.getDistance=ue;var ce=function(e){var t,n=V(e);return X(n)?J:n>0?(n=F(n)/B,t=H(n),t-n<le?t:L(n)):0};t.getDecimalOrder=ce;var de=function(e,t,n){var i,o=G(ce(e),ce(t)),a=-ce(V(t-e)/n);return X(o)||X(a)?null:(V(o)<=4?(i="fixedPoint",a<0&&(a=0),a>4&&(a=4)):(i="exponential",a+=o-1,a>3&&(a=3)),{type:i,precision:a})};t.getAppropriateFormat=de;var he=function(e,t){if(t>20&&(t=20),(0,I.isNumeric)(e))return Q((0,I.isExponential)(e)?e.toExponential(t):e.toFixed(t))};t.roundValue=he;var fe=function(e){return e.toExponential().split("e")[1]};t.getPower=fe;var pe=function(e,t,n){var i;do i=a(e,t),i.push(H(n/i.length)),n=r(e,t,W.apply(null,i),n);while(n>0&&i.length>1);return n};t.decreaseGaps=pe;var ge=function(e,t){return void 0!==e?e:t};t.parseScalar=ge;var _e=function(e){var t,n,i={};for(t=0,n=e.length;t<n;++t)i[s(e[t])]=1;return function(e,t){var n=s(e);return i[n]?n:t}};t.enumParser=_e;var me=function(e){var t={};return(0,O.each)(e||{},function(n,i){if(/^(cursor)$/i.test(n));else if("opacity"===n)i=null;else if("color"===n){if(n="fill","opacity"in e){var o=new R.default(i);i="rgba(".concat(o.r,",").concat(o.g,",").concat(o.b,",").concat(e.opacity,")")}}else n="font-"+n;t[n]=i}),t};t.patchFontOptions=me;var ve=function(e,t,n){var i=ue(e.x,e.y,t,n),o=j(n-e.y,t-e.x);return{phi:K(oe(180*o/M)),r:K(i)}};t.convertXYToPolar=ve;var ye=function(e,t){var n,i,a,r=(0,I.isFunction)(e.customizeSeries)?e.customizeSeries:D.noop,s=e.nameField,l={},u=[],c=0;for(t=t||[],i=t.length;c<i;c++)a=t[c],s in a&&(n=l[a[s]],n||(n=l[a[s]]={name:a[s],nameFieldValue:a[s]},u.push(n.name)));return o(u,function(e){var t=l[e];return(0,T.extend)(t,r.call(null,t.name))})};t.processSeriesTemplate=ye;var be=function(e,t,n){if(0===e.length)return{categories:[]};t=(0,I.isDefined)(t)?t:e[0],n=(0,I.isDefined)(n)?n:e[e.length-1];var i,a=o(e,function(e){return null===e||void 0===e?void 0:e.valueOf()}),r=a.indexOf(t.valueOf()),s=a.indexOf(n.valueOf()),l=!1;r<0&&(r=0),s<0&&(s=e.length-1),s<r&&(i=s,s=r,r=i,l=!0);var u=e.slice(r,s+1),c=u.length-1;return{categories:u,start:u[l?c:0],end:u[l?0:c],inverted:l}};t.getCategoriesInfo=be;var xe=function(e){var t={};return o(e,function(e){var n=t[e]?null:e;return t[e]=!0,n})};t.unique=xe;var we=function(e,t,n){var i=e.x+e.width/2>=n.x,o=(i?e.x:e.x+e.width)-n.x,a=e.y-n.y,r=a+t,s=K(N(o*o+a*a-r*r)),l=(i?+s:-s)||o;return{x:n.x+(i?l:l-e.width),y:e.y+t}};t.getVerticallyShiftedAngularCoords=we},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(22)),a=n(1),r=function(e,t,n){return function(){var i=this.callBase;this.callBase=e[t];try{return n.apply(this,arguments)}finally{this.callBase=i}}},s=function(e){var t=function(){};return t.prototype=e.prototype,new t},l=function(e){var t,n,i,o=this;if(!e)return o;for(n in e)i=e[n],t="function"==typeof o.prototype[n]&&"function"==typeof i,o.prototype[n]=t?r(o.parent.prototype,n,i):i;return o},u=function(){var e,t,n,i=this,o=Object.prototype.hasOwnProperty.bind(i),a=!o("_includedCtors")&&!o("_includedPostCtors");for(a&&(i._includedCtors=i._includedCtors.slice(0),i._includedPostCtors=i._includedPostCtors.slice(0)),n=0;n<arguments.length;n++){e=arguments[n],e.ctor&&i._includedCtors.push(e.ctor),e.postCtor&&i._includedPostCtors.push(e.postCtor);for(t in e)"ctor"!==t&&"postCtor"!==t&&(i.prototype[t]=e[t])}return i},c=function(e){var t=Object.prototype.hasOwnProperty.bind(this)("parent"),n=!t&&this.parent;if(n){var i=Object.getPrototypeOf(this);return i===e||i.subclassOf(e)}return this.parent===e||!(!this.parent||!this.parent.subclassOf)&&this.parent.subclassOf(e)},d=function(){throw o.default.Error("E0001")},h=function(){var e=Object.prototype.hasOwnProperty;return function(t,n){for(var i in t){if(!e.call(t,i))return;n[i]=t[i]}}}(),f=function(){};f.inherit=function(e){var t=function(){if(!this||(0,a.isWindow)(this)||"function"!=typeof this.constructor)throw o.default.Error("E0003");var e,t=this,n=t.ctor,i=t.constructor._includedCtors,r=t.constructor._includedPostCtors;for(e=0;e<i.length;e++)i[e].call(t);for(n&&n.apply(t,arguments),e=0;e<r.length;e++)r[e].call(t)};return t.prototype=s(this),h(this,t),t.inherit=this.inherit,t.abstract=d,t.redefine=l,t.include=u,t.subclassOf=c,t.parent=this,t._includedCtors=this._includedCtors?this._includedCtors.slice(0):[],t._includedPostCtors=this._includedPostCtors?this._includedPostCtors.slice(0):[],t.prototype.constructor=t,t.redefine(e),t},f.abstract=d;var p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e,t,n){e="role"===e||"id"===e?e:"aria-".concat(e),t=(0,_.isDefined)(t)?t.toString():null,n.attr(e,t)}t.default=void 0;var r=i(n(2)),s=i(n(108)),l=i(n(65)),u=n(119),c=n(4),d=n(3),h=n(0),f=n(75),p=n(12),g=n(8),_=n(1);n(21),n(140),n(158);var m=l.default.inherit({_feedbackHideTimeout:400,_feedbackShowTimeout:30,_supportedKeys:function(){return{}},_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{hoveredElement:null,isActive:!1,disabled:!1,visible:!0,hint:void 0,activeStateEnabled:!1,onContentReady:null,hoverStateEnabled:!1,focusStateEnabled:!1,tabIndex:0,accessKey:null,onFocusIn:null,onFocusOut:null,onKeyboardHandled:null,ignoreParentReadOnly:!1})},_init:function(){this.callBase(),this._initContentReadyAction()},_innerWidgetOptionChanged:function(e,t){var n=m.getOptionsFromContainer(t);e&&e.option(n),this._options.cache(t.name,n)},_bindInnerWidgetOptions:function(e,t){var n=this,i=function(){return n._options.silent(t,(0,h.extend)({},e.option()))};i(),e.on("optionChanged",i)},_getAriaTarget:function(){return this._focusTarget()},_initContentReadyAction:function(){this._contentReadyAction=this._createActionByOption("onContentReady",{excludeValidators:["disabled","readOnly"]})},_initMarkup:function(){var e=this.option(),t=e.disabled,n=e.visible;this.$element().addClass("dx-widget"),this._toggleDisabledState(t),this._toggleVisibility(n),this._renderHint(),this._isFocusable()&&this._renderFocusTarget(),this.callBase()},_render:function(){this.callBase(),this._renderContent(),this._renderFocusState(),this._attachFeedbackEvents(),this._attachHoverEvents(),this._toggleIndependentState()},_renderHint:function(){var e=this.option(),t=e.hint;this.$element().attr("title",t||null)},_renderContent:function(){var e=this;(0,c.deferRender)(function(){return e._disposed?void 0:e._renderContentImpl()}).done(function(){return e._disposed?void 0:e._fireContentReadyAction()})},_renderContentImpl:c.noop,_fireContentReadyAction:(0,c.deferRenderer)(function(){return this._contentReadyAction()}),_dispose:function(){this._contentReadyAction=null,this._detachKeyboardEvents(),this.callBase()},_resetActiveState:function(){this._toggleActiveState(this._eventBindingTarget(),!1)},_clean:function(){this._cleanFocusState(),this._resetActiveState(),this.callBase(),this.$element().empty()},_toggleVisibility:function(e){this.$element().toggleClass("dx-state-invisible",!e),this.setAria("hidden",!e||void 0)},_renderFocusState:function(){this._attachKeyboardEvents(),this._isFocusable()&&(this._renderFocusTarget(),this._attachFocusEvents(),this._renderAccessKey())},_renderAccessKey:function(){var e=this,t=this._focusTarget(),n=this.option(),i=n.accessKey,o="UIFeedback";t.attr("accesskey",i),u.dxClick.off(t,{namespace:o}),i&&u.dxClick.on(t,function(t){(0,g.isFakeClickEvent)(t)&&(t.stopImmediatePropagation(),e.focus())},{namespace:o})},_isFocusable:function(){var e=this.option(),t=e.focusStateEnabled,n=e.disabled;return t&&!n},_eventBindingTarget:function(){return this.$element()},_focusTarget:function(){return this._getActiveElement()},_getActiveElement:function(){var e=this._eventBindingTarget();return this._activeStateUnit?e.find(this._activeStateUnit).not(".dx-state-disabled"):e},_renderFocusTarget:function(){var e=this.option(),t=e.tabIndex;this._focusTarget().attr("tabIndex",t)},_keyboardEventBindingTarget:function(){return this._eventBindingTarget()},_refreshFocusEvent:function(){this._detachFocusEvents(),this._attachFocusEvents()},_focusEventTarget:function(){return this._focusTarget()},_focusInHandler:function(e){var t=this;e.isDefaultPrevented()||this._createActionByOption("onFocusIn",{beforeExecute:function(){return t._updateFocusState(e,!0)},excludeValidators:["readOnly"]})({event:e})},_focusOutHandler:function(e){var t=this;e.isDefaultPrevented()||this._createActionByOption("onFocusOut",{beforeExecute:function(){return t._updateFocusState(e,!1)},excludeValidators:["readOnly","disabled"]})({event:e})},_updateFocusState:function(e,t){var n=e.target;(0,p.inArray)(n,this._focusTarget())!==-1&&this._toggleFocusClass(t,(0,r.default)(n))},_toggleFocusClass:function(e,t){var n=t&&t.length?t:this._focusTarget();n.toggleClass("dx-state-focused",e)},_hasFocusClass:function(e){var t=(0,r.default)(e||this._focusTarget());return t.hasClass("dx-state-focused")},_isFocused:function(){return this._hasFocusClass()},_getKeyboardListeners:function(){return[]},_attachKeyboardEvents:function(){var e=this;this._detachKeyboardEvents();var t=this.option(),n=t.focusStateEnabled,i=t.onKeyboardHandled,o=this._getKeyboardListeners().length,a=!!i,r=n||o||a;r&&(this._keyboardListenerId=u.keyboard.on(this._keyboardEventBindingTarget(),this._focusTarget(),function(t){return e._keyboardHandler(t)}))},_keyboardHandler:function(e,t){if(!t){var n=e.originalEvent,i=e.keyName,o=e.which,a=this._supportedKeys(n),r=a[i]||a[o];if(void 0!==r){var s=r.bind(this),l=s(n,e);if(!l)return!1}}var u=this._getKeyboardListeners(),c=this.option(),d=c.onKeyboardHandled;return u.forEach(function(t){return t&&t._keyboardHandler(e)}),d&&d(e),!0},_refreshFocusState:function(){this._cleanFocusState(),this._renderFocusState()},_cleanFocusState:function(){var e=this._focusTarget();e.removeAttr("tabIndex"),this._toggleFocusClass(!1),this._detachFocusEvents(),this._detachKeyboardEvents()},_detachKeyboardEvents:function(){u.keyboard.off(this._keyboardListenerId),this._keyboardListenerId=null},_attachHoverEvents:function(){var e=this,t=this.option(),n=t.hoverStateEnabled,i=this._activeStateUnit,o="UIFeedback",a=this._eventBindingTarget();u.hover.off(a,{selector:i,namespace:o}),n&&u.hover.on(a,new s.default(function(t){var n=t.event,i=t.element;e._hoverStartHandler(n),e.option("hoveredElement",(0,r.default)(i))},{excludeValidators:["readOnly"]}),function(t){e.option("hoveredElement",null),e._hoverEndHandler(t)},{selector:i,namespace:o})},_attachFeedbackEvents:function(){var e=this,t=this.option(),n=t.activeStateEnabled,i=this._activeStateUnit,o="UIFeedback",a=this._eventBindingTarget();u.active.off(a,{namespace:o,selector:i}),n&&u.active.on(a,new s.default(function(t){var n=t.event,i=t.element;return e._toggleActiveState((0,r.default)(i),!0,n)}),new s.default(function(t){var n=t.event,i=t.element;return e._toggleActiveState((0,r.default)(i),!1,n)},{excludeValidators:["disabled","readOnly"]}),{showTimeout:this._feedbackShowTimeout,hideTimeout:this._feedbackHideTimeout,selector:i,namespace:o})},_detachFocusEvents:function(){var e=this._focusEventTarget();u.focus.off(e,{namespace:"".concat(this.NAME,"Focus")})},_attachFocusEvents:function(){var e=this,t=this._focusEventTarget();u.focus.on(t,function(t){return e._focusInHandler(t)},function(t){return e._focusOutHandler(t)},{namespace:"".concat(this.NAME,"Focus"),isFocusable:function(e,t){return(0,r.default)(t).is(f.focusable)}})},_hoverStartHandler:c.noop,_hoverEndHandler:c.noop,_toggleActiveState:function(e,t){this.option("isActive",t),e.toggleClass("dx-state-active",t)},_updatedHover:function(){var e=this._options.silent("hoveredElement");this._hover(e,e)},_findHoverTarget:function(e){return e&&e.closest(this._activeStateUnit||this._eventBindingTarget())},_hover:function(e,t){var n=this.option(),i=n.hoverStateEnabled,o=n.disabled,a=n.isActive;if(t=this._findHoverTarget(t),t&&t.toggleClass("dx-state-hover",!1),e&&i&&!o&&!a){var r=this._findHoverTarget(e);r&&r.toggleClass("dx-state-hover",!0)}},_toggleDisabledState:function(e){this.$element().toggleClass("dx-state-disabled",Boolean(e)),this.setAria("disabled",e||void 0)},_toggleIndependentState:function(){this.$element().toggleClass("dx-state-independent",this.option("ignoreParentReadOnly"))},_setWidgetOption:function(e,t){var n=this;if(this[e]){if((0,_.isPlainObject)(t[0]))return void(0,d.each)(t[0],function(t,i){return n._setWidgetOption(e,[t,i])});var i=t[0],o=t[1];1===t.length&&(o=this.option(i));var a=this["".concat(e,"OptionMap")];this[e].option(a?a(i):i,o)}},_optionChanged:function(e){var t=e.name,n=e.value,i=e.previousValue;switch(t){case"disabled":this._toggleDisabledState(n),this._updatedHover(),this._refreshFocusState();break;case"hint":this._renderHint();break;case"ignoreParentReadOnly":this._toggleIndependentState();break;case"activeStateEnabled":this._attachFeedbackEvents();break;case"hoverStateEnabled":this._attachHoverEvents(),this._updatedHover();break;case"tabIndex":case"focusStateEnabled":this._refreshFocusState();break;case"onFocusIn":case"onFocusOut":break;case"accessKey":this._renderAccessKey();break;case"hoveredElement":this._hover(n,i);break;case"isActive":this._updatedHover();break;case"visible":this._toggleVisibility(n),this._isVisibilityChangeSupported()&&this._checkVisibilityChanged(n?"shown":"hiding");break;case"onKeyboardHandled":this._attachKeyboardEvents();break;case"onContentReady":this._initContentReadyAction();break;default:this.callBase(e)}},_isVisible:function(){var e=this.option(),t=e.visible;return this.callBase()&&t},beginUpdate:function(){this._ready(!1),this.callBase()},endUpdate:function(){this.callBase(),this._initialized&&this._ready(!0)},_ready:function(e){return 0===arguments.length?this._isReady:void(this._isReady=e)},setAria:function(){if((0,_.isPlainObject)(arguments.length<=0?void 0:arguments[0])){var e=(arguments.length<=1?void 0:arguments[1])||this._getAriaTarget();(0,d.each)(arguments.length<=0?void 0:arguments[0],function(t,n){return a(t,n,e)})}else a(arguments.length<=0?void 0:arguments[0],arguments.length<=1?void 0:arguments[1],(arguments.length<=2?void 0:arguments[2])||this._getAriaTarget())},isReady:function(){return this._ready()},repaint:function(){this._refresh()},focus:function(){u.focus.trigger(this._focusTarget())},registerKeyHandler:function(e,t){var n=this._supportedKeys();this._supportedKeys=function(){return(0,h.extend)(n,o({},e,t))}}});m.getOptionsFromContainer=function(e){var t=e.name,n=e.fullName,i=e.value,o={};if(t===n)o=i;else{var a=n.split(".").pop();o[a]=i}return o};var v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(135)),a=i(n(22)),r=(0,o.default)(a.default.ERROR_MESSAGES,{E1001:"Module '{0}'. Controller '{1}' is already registered",E1002:"Module '{0}'. Controller '{1}' does not inherit from DevExpress.ui.dxDataGrid.Controller",E1003:"Module '{0}'. View '{1}' is already registered",E1004:"Module '{0}'. View '{1}' does not inherit from DevExpress.ui.dxDataGrid.View",E1005:"Public method '{0}' is already registered",E1006:"Public method '{0}.{1}' does not exist",E1007:"State storing cannot be provided due to the restrictions of the browser",E1010:"The template does not contain the TextBox widget",E1011:'Items cannot be deleted from the List. Implement the "remove" function in the data store',E1012:"Editing type '{0}' with the name '{1}' is unsupported",E1016:"Unexpected type of data source is provided for a lookup column",E1018:"The 'collapseAll' method cannot be called if you use a remote data source",E1019:"Search mode '{0}' is unavailable",E1020:"The type cannot be changed after initialization",E1021:"{0} '{1}' you are trying to remove does not exist",E1022:'The "markers" option is given an invalid value. Assign an array instead',E1023:'The "routes" option is given an invalid value. Assign an array instead',E1025:"This layout is too complex to render",E1026:'The "calculateCustomSummary" function is missing from a field whose "summaryType" option is set to "custom"',E1030:"Unknown ScrollView refresh strategy: '{0}'",E1031:"Unknown subscription in the Scheduler widget: '{0}'",E1032:"Unknown start date in an appointment: '{0}'",E1033:"Unknown step in the date navigator: '{0}'",E1034:"The browser does not implement an API for saving files",E1035:"The editor cannot be created because of an internal error: {0}",E1037:"Invalid structure of grouped data",E1038:"The browser does not support local storages for local web pages",E1039:"A cell's position cannot be calculated",E1040:"The '{0}' key value is not unique within the data array",E1041:"The '{0}' script is referenced after the DevExtreme scripts or not referenced at all",E1042:"{0} requires the key field to be specified",E1043:"Changes cannot be processed due to the incorrectly set key",E1044:"The key field specified by the keyExpr option does not match the key field specified in the data store",E1045:"Editing requires the key field to be specified in the data store",E1046:"The '{0}' key field is not found in data objects",E1047:'The "{0}" field is not found in the fields array',E1048:'The "{0}" operation is not found in the filterOperations array',E1049:"Column '{0}': filtering is allowed but the 'dataField' or 'name' option is not specified",E1050:"The validationRules option does not apply to third-party editors defined in the editCellTemplate",E1051:'HtmlEditor\'s valueType is "{0}", but the {0} converter was not imported.',E1052:'{0} should have the "dataSource" option specified',E1053:'The "buttons" option accepts an array that contains only objects or string values',E1054:"All text editor buttons must have names",E1055:'One or several text editor buttons have invalid or non-unique "name" values',E1056:'The {0} widget does not support buttons of the "{1}" type',E1058:'The "startDayHour" must be earlier than the "endDayHour"',E1059:"The following column names are not unique: {0}",E1060:"All editable columns must have names",W1001:'The "key" option cannot be modified after initialization',W1002:"An item with the key '{0}' does not exist",W1003:"A group with the key '{0}' in which you are trying to select items does not exist",W1004:"The item '{0}' you are trying to select in the group '{1}' does not exist",W1005:"Due to column data types being unspecified, data has been loaded twice in order to apply initial filter settings. To resolve this issue, specify data types for all grid columns.",W1006:"The map service returned the following error: '{0}'",W1007:"No item with key {0} was found in the data source, but this key was used as the parent key for item {1}",W1008:"Cannot scroll to the '{0}' date because it does not exist on the current view",W1009:"Searching works only if data is specified using the dataSource option",W1010:"The capability to select all items works with source data of plain structure only",W1011:'The "keyExpr" option is not applied when dataSource is not an array',W1012:"The '{0}' key field is not found in data objects",W1013:'The "message" field in the dialog component was renamed to "messageHtml". Change your code correspondingly. In addition, if you used HTML code in the message, make sure that it is secure',W1014:"The Floating Action Button exceeds the recommended speed dial action count. If you need to display more speed dial actions, increase the maxSpeedDialActionCount option value in the global config.",W1015:'The "cellDuration" should divide the range from the "startDayHour" to the "endDayHour" into even intervals'});t.default=r,e.exports=t.default},function(e,t,n){function i(e){return a(e)}function o(e){a=e}t.getPublicElement=i,t.setPublicElementWrapper=o;var a=function(e){return e&&e.get(0)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return t.unwrapObservables?h(e):e}function a(e){for(var t={},n=0,i=e.length;n<i;n++){var o=e[n];t[o]=y(o)}return function(e,n){var i;return(0,c.each)(t,function(t){var o=this(e,n);if(void 0!==o){for(var a=i||(i={}),r=t.split("."),s=r.length-1,l=0;l<s;l++){var u=r[l];u in a||(a[u]={}),a=a[u]}a[r[s]]=o}}),i}}t.toComparable=t.compileSetter=t.compileGetter=void 0;var r=i(n(22)),s=i(n(15)),l=n(50),u=n(1),c=n(3),d=i(n(101)),h=d.default.unwrap,f=d.default.isWrapped,p=d.default.assign,g=function(e){return e.replace(/\[/g,".").replace(/\]/g,"")},_=function(e,t,n){return n=n||{},"this"===t?o(e,n):o(e[t],n)},m=function(e,t,n,i){if("this"===t)throw new r.default.Error("E4016");var o=e[t];i.unwrapObservables&&f(o)?p(o,n):e[t]=n},v=function(e){return e=e||{},e.unwrapObservables=void 0===e.unwrapObservables||e.unwrapObservables,e},y=function(e){if(arguments.length>1&&(e=[].slice.call(arguments)),!e||"this"===e)return function(e){return e};if("string"==typeof e){e=g(e);var t=e.split(".");return function(e,n){n=v(n);for(var i=n.functionsAsIs,a=("defaultValue"in n),r=o(e,n),s=0;s<t.length;s++){if(!r){if(null==r&&a)return n.defaultValue;break}var l=t[s];if(a&&(0,u.isObject)(r)&&!(l in r))return n.defaultValue;var c=o(r[l],n);!i&&(0,u.isFunction)(c)&&(c=c.call(r)),r=c}return r}}return Array.isArray(e)?a(e):(0,u.isFunction)(e)?e:void 0};t.compileGetter=y;var b=function(e,t,n,i){if((0,u.isDefined)(n))return n;var o={};return m(e,t,o,i),o},x=function(e){e=g(e||"this").split(".");var t=e.length-1;return function(n,i,a){a=v(a);var r=o(n,a);e.forEach(function(e,n){var o=_(r,e,a),s=!a.functionsAsIs&&(0,u.isFunction)(o)&&!f(o);n===t?a.merge&&(0,u.isPlainObject)(i)&&(!(0,u.isDefined)(o)||(0,u.isPlainObject)(o))?(o=b(r,e,o,a),(0,l.deepExtendArraySafe)(o,i,!1,!0)):s?r[e](i):m(r,e,i,a):(o=b(r,e,o,a),s&&(o=o.call(r)),r=o)})}};t.compileSetter=x;var w=function(e,t){return e instanceof Date?e.getTime():e&&e instanceof s.default&&e.valueOf?e.valueOf():t||"string"!=typeof e?e:e.toLowerCase();
};t.toComparable=w},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return o=a()?Reflect.construct:function(e,t,n){var i=[null];i.push.apply(i,t);var o=Function.bind.apply(e,i),a=new o;return n&&r(a,n.prototype),a},o.apply(null,arguments)}function a(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function s(e,t){return A(e)*t}function l(e){var t,n=-1;return(0,k.isString)(e)?e:(0,k.isObject)(e)?((0,D.each)(e,function(e,i){for(t=0;t<E.length;t++)i&&(e===E[t]+"s"||e===E[t])&&n<t&&(n=t)}),E[n]):""}function u(e){return V[l(e)]||""}function c(e,t,n,i){e=new Date(e.getTime());var o,a,r=new Date(e.getTime()),s=l(t);switch(s){case"second":e=new Date(1e3*Math.floor(r.getTime()/1e3));break;case"minute":e=new Date(6e4*Math.floor(r.getTime()/6e4));break;case"hour":e=new Date(36e5*Math.floor(r.getTime()/36e5));break;case"year":e.setMonth(0);case"month":e.setDate(1);case"day":e.setHours(0,0,0,0);break;case"week":e=b(e,i||0),e.setHours(0,0,0,0);break;case"quarter":o=L(e.getMonth()),a=e.getMonth(),e.setDate(1),e.setHours(0,0,0,0),a!==o&&e.setMonth(o)}return n&&"hour"!==s&&"minute"!==s&&"second"!==s&&w(r,e),e}function d(e){return c(e,"day")}function h(e,t,n){var i=new Date(e.getTime()),o=(0,k.isString)(t)?p(t.toLowerCase()):(0,k.isNumeric)(t)?M(t):t;return o.years&&i.setFullYear(i.getFullYear()+o.years*n),o.quarters&&i.setMonth(i.getMonth()+3*o.quarters*n),o.months&&i.setMonth(i.getMonth()+o.months*n),o.weeks&&i.setDate(i.getDate()+7*o.weeks*n),o.days&&i.setDate(i.getDate()+o.days*n),o.hours&&i.setTime(i.getTime()+36e5*o.hours*n),o.minutes&&i.setTime(i.getTime()+6e4*o.minutes*n),o.seconds&&i.setTime(i.getTime()+1e3*o.seconds*n),o.milliseconds&&i.setTime(i.getTime()+o.milliseconds*n),i}function f(e){var t=ce(e.getFullYear(),e.getMonth()+1,0);return t.getDate()}function p(e){var t={};switch(e){case"year":t.years=1;break;case"month":t.months=1;break;case"quarter":t.months=3;break;case"week":t.weeks=1;break;case"day":t.days=1;break;case"hour":t.hours=1;break;case"minute":t.minutes=1;break;case"second":t.seconds=1;break;case"millisecond":t.milliseconds=1}return t}function g(e,t){return _(e,t)&&e.getDate()===t.getDate()}function _(e,t){return m(e,t)&&e.getMonth()===t.getMonth()}function m(e,t){return e&&t&&e.getFullYear()===t.getFullYear()}function v(e){return e&&e.getFullYear()-e.getFullYear()%100}function y(e){return e&&e.getFullYear()-e.getFullYear()%10}function b(e,t){var n=(e.getDay()-t+7)%7,i=new Date(e);return i.setDate(e.getDate()-n),i}function x(e,t,n){var i=e;return(0,k.isDefined)(e)?((0,k.isDefined)(t)&&e<t&&(i=t),(0,k.isDefined)(n)&&e>n&&(i=n),i):e}function w(e,t){if((0,k.isDefined)(e)){var n=t.getHours()-e.getHours();if(0!==n){var i=1===n||n===-23?-1:1,o=new Date(t.getTime()+36e5*i);(i>0||o.getDate()===t.getDate())&&t.setTime(o.getTime())}}}function C(e,t){return 60*(t.getTimezoneOffset()-e.getTimezoneOffset())*1e3}t.default=void 0;var k=n(1),S=n(29),D=n(3),I=n(40),T=i(n(23)),O=T.default.msie&&parseInt(T.default.version)<=11,E=["millisecond","second","minute","hour","day","week","month","quarter","year"],A=function e(t){switch(t){case"millisecond":return 1;case"second":return 1e3*e("millisecond");case"minute":return 60*e("second");case"hour":return 60*e("minute");case"day":return 24*e("hour");case"week":return 7*e("day");case"month":return 30*e("day");case"quarter":return 3*e("month");case"year":return 365*e("day");default:return 0}},P=function(e,t,n){var i=t.getTime()-e.getTime(),o=A(n)||1;return Math.floor(i/o)},R=function(e,t){var n=l(e);switch(n){case"millisecond":return"second";case"second":return"minute";case"minute":return"hour";case"hour":return"day";case"day":return t?"week":"month";case"week":return"month";case"month":return"quarter";case"quarter":return"year";case"year":return"year";default:return 0}},M=function(e){var t,n,i,o=["millisecond","second","minute","hour","day","month","year"],a={};for(t=o.length-1;t>=0;t--)i=o[t],n=Math.floor(e/A(i)),n>0&&(a[i+"s"]=n,e-=s(i,n));return a},B=function(e){var t=0;return(0,k.isObject)(e)&&(0,D.each)(e,function(e,n){t+=s(e.substr(0,e.length-1),n)}),(0,k.isString)(e)&&(t=s(e,1)),t},V={millisecond:"millisecond",second:"longtime",minute:"shorttime",hour:"shorttime",day:"day",week:"day",month:"month",quarter:"quarter",year:"year"},F=function(e){return Math.floor(e/3)},L=function(e){return 3*F(e)},H=function(e){var t=d(e);return t.setDate(t.getDate()+1),new Date(t.getTime()-1)},z=function(e,t){var n=0,i={year:e.getFullYear()!==t.getFullYear(),month:e.getMonth()!==t.getMonth(),day:e.getDate()!==t.getDate(),hour:e.getHours()!==t.getHours(),minute:e.getMinutes()!==t.getMinutes(),second:e.getSeconds()!==t.getSeconds(),millisecond:e.getMilliseconds()!==t.getMilliseconds()};return(0,D.each)(i,function(e,t){t&&n++}),0===n&&0!==C(e,t)&&(i.hour=!0,n++),i.count=n,i},N=function(e,t,n){var i=n?-1:1;return(0,k.isDate)(e)?h(e,t,i):(0,S.adjust)(e+t*i,t)},j=function(e,t,n){var i,o=[];for(o.push((0,k.isDate)(e)?new Date(e.getTime()):e),i=e;i<t;)i=N(i,n),o.push(i);return o},W=function(e,t){return"month"===e?ce(t.getFullYear(),t.getMonth(),1):"year"===e?ce(t.getFullYear(),0,t.getDate()):"decade"===e?ce(y(t),t.getMonth(),t.getDate()):"century"===e?ce(v(t),t.getMonth(),t.getDate()):void 0},G=function(e,t){return"month"===e?ce(t.getFullYear(),t.getMonth(),f(t)):"year"===e?ce(t.getFullYear(),11,t.getDate()):"decade"===e?ce(y(t)+9,t.getMonth(),t.getDate()):"century"===e?ce(v(t)+90,t.getMonth(),t.getDate()):void 0},q=function(e,t){var n=ce(t.getFullYear(),t.getMonth(),1);return"month"===e?n:(n.setMonth(0),"year"===e?n:("decade"===e&&n.setFullYear(y(t)),"century"===e&&n.setFullYear(v(t)),n))},$=function(e,t){var n=new Date(t);return n.setDate(f(t)),"month"===e?n:(n.setMonth(11),n.setDate(f(n)),"year"===e?n:("decade"===e&&n.setFullYear(y(t)+9),"century"===e&&n.setFullYear(v(t)+99),n))},K=function(e){switch(e){case"month":return"year";case"year":return"decade";case"decade":return"century"}},U=function(e){switch(e){case"century":return"decade";case"decade":return"year";case"year":return"month"}},Y=function(e){var t=1;return"year"===e&&(t=12),"decade"===e&&(t=120),"century"===e&&(t=1200),t},Z=function(e){var t=1;return"decade"===e&&(t=12),"century"===e&&(t=120),t},X=function(e,t){if((0,k.isDefined)(e)&&(0,k.isDefined)(t)){var n=e.getFullYear()-e.getFullYear()%10,i=t.getFullYear()-t.getFullYear()%10;return e&&t&&n===i}},Q=function(e,t){if((0,k.isDefined)(e)&&(0,k.isDefined)(t)){var n=e.getFullYear()-e.getFullYear()%100,i=t.getFullYear()-t.getFullYear()%100;return e&&t&&n===i}},J=function(){return"yyyy/MM/dd"},ee=function(e){if((0,k.isDefined)(e))return ce(e.getFullYear(),e.getMonth(),1)},te=function(e){if((0,k.isDefined)(e))return ce(e.getFullYear(),e.getMonth()+1,0)},ne=function(e,t){var n=de.getDatesInterval(e,t,"day"),i=new Date(e);return n>=6&&(i=new Date(i.setDate(i.getDate()+7))),i},ie=function(e,t,n,i){return"date"===i&&(t=t&&de.correctDateWithUnitBeginning(t,"day"),n=n&&de.correctDateWithUnitBeginning(n,"day"),e=e&&de.correctDateWithUnitBeginning(e,"day")),x(e,t,n)===e},oe=function(e){var t=Math.floor(e),n=e%1*60;return{hours:t,minutes:n}},ae=function(e,t){var n=this.dateTimeFromDecimal(t),i=new Date(e);return(e.getHours()===n.hours&&e.getMinutes()<n.minutes||e.getHours()<n.hours)&&i.setHours(n.hours,n.minutes,0,0),i},re=function(e){return e.setHours(e.getHours()+1),e.setMinutes(0),e},se=function(e){return new Date(e)},le=function(e,t,n){for(var i=[],o=new Date(e.getTime());o<t;)i.push(new Date(o.getTime())),o=this.addInterval(o,n);return i},ue=function(e){return new Date(O&&(0,k.isDate)(e)?e.getTime():e)},ce=function(e){var t=o(Date,Array.prototype.slice.call(arguments));return t.setFullYear(e),t},de={dateUnitIntervals:E,convertMillisecondsToDateUnits:M,dateToMilliseconds:B,getNextDateUnit:R,convertDateUnitToMilliseconds:s,getDateUnitInterval:l,getDateFormatByTickInterval:u,getDatesDifferences:z,correctDateWithUnitBeginning:c,trimTime:d,setToDayEnd:H,roundDateByStartDayHour:ae,dateTimeFromDecimal:oe,addDateInterval:h,addInterval:N,getSequenceByInterval:j,getDateIntervalByString:p,sameDate:g,sameMonthAndYear:_,sameMonth:_,sameYear:m,sameDecade:X,sameCentury:Q,getDifferenceInMonth:Y,getDifferenceInMonthForCells:Z,getFirstYearInDecade:y,getFirstDecadeInCentury:v,getShortDateFormat:J,getViewFirstCellDate:W,getViewLastCellDate:G,getViewDown:U,getViewUp:K,getLastMonthDay:f,getLastMonthDate:te,getFirstMonthDate:ee,getFirstWeekDate:b,normalizeDateByWeek:ne,getQuarter:F,getFirstQuarterMonth:L,dateInRange:ie,roundToHour:re,normalizeDate:x,getViewMinBoundaryDate:q,getViewMaxBoundaryDate:$,fixTimezoneGap:w,getTimezonesDifference:C,makeDate:se,getDatesInterval:P,getDatesOfInterval:le,createDate:ue,createDateWithFullYear:ce};de.sameView=function(e,t,n){return de[(0,I.camelize)("same "+e)](t,n)};var he=de;t.default=he,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.name=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=i(n(11)),l=n(33),u=n(120),c=n(8),d=n(372),h=i(n(25)),f=i(n(121)),p=i(n(99)),g=n(60),_="dxclick";t.name=_;var m=10,v=Math.abs,y=function(e){return(0,o.default)(e).is("input, textarea, select, button ,:focus, :focus *")},b={requestAnimationFrame:u.requestAnimationFrame,cancelAnimationFrame:u.cancelAnimationFrame},x=f.default.inherit({ctor:function(e){this.callBase(e),this._makeElementClickable((0,o.default)(e))},_makeElementClickable:function(e){e.attr("onclick")||e.attr("onclick","void(0)")},start:function(e){this._blurPrevented=e.isDefaultPrevented(),this._startTarget=e.target,this._startEventData=(0,c.eventData)(e)},end:function(e){return this._eventOutOfElement(e,this.getElement().get(0))||e.type===h.default.cancel?void this._cancel(e):(y(e.target)||this._blurPrevented||(0,l.resetActiveElement)(),this._accept(e),void(this._clickAnimationFrame=b.requestAnimationFrame(function(){this._fireClickEvent(e)}.bind(this))))},_eventOutOfElement:function(e,t){var n=e.target,i=!(0,l.contains)(t,n)&&t!==n,o=(0,c.eventDelta)((0,c.eventData)(e),this._startEventData),a=v(o.x)>m||v(o.y)>m;return i||a},_fireClickEvent:function(e){this._fireEvent(_,e,{target:(0,l.closestCommonParent)(this._startTarget,e.target)})},dispose:function(){b.cancelAnimationFrame(this._clickAnimationFrame)}}),w=r.default.real(),C=w.generic||w.ios&&(0,g.compare)(w.version,[9,3])>=0||w.android&&(0,g.compare)(w.version,[5])>=0;!function(){function e(){r=null}var t="dx-native-click",n=function(e){return C||(0,o.default)(e).closest("."+t).length},i=null,r=null,s=function(t){var o=t.originalEvent,a=r===o||o&&o.DXCLICK_FIRED,s=!t.which||1===t.which;s&&!i&&n(t.target)&&!a&&(o&&(o.DXCLICK_FIRED=!0),(0,d.unsubscribeNodesDisposing)(r,e),r=o,(0,d.subscribeNodesDisposing)(r,e),(0,c.fireEvent)({type:_,originalEvent:t}))};x=x.inherit({_makeElementClickable:function(e){n(e)||this.callBase(e),a.default.on(e,"click",s)},configure:function(e){this.callBase(e),e.useNative&&this.getElement().addClass(t)},start:function(e){i=null,n(e.target)||this.callBase(e)},end:function(e){n(e.target)||this.callBase(e)},cancel:function(){i=!0},dispose:function(){this.callBase(),a.default.off(this.getElement(),"click",s)}})}(),function(){var e=r.default.real().generic;if(!e){var t=null,n=!1,i=function(e){t=e.target,n=e.isDefaultPrevented()},u=function(e){var i=(0,o.default)(e.target);n||!t||i.is(t)||(0,o.default)(t).is("label")||!y(i)||(0,l.resetActiveElement)(),t=null,n=!1},d="NATIVE_CLICK_FIXER",f=s.default.getDocument();a.default.subscribeGlobal(f,(0,c.addNamespace)(h.default.down,d),i),a.default.subscribeGlobal(f,(0,c.addNamespace)("click",d),u)}}(),(0,p.default)({emitter:x,bubble:!0,events:[_]})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(135)),a=(0,o.default)({E0001:"Method is not implemented",E0002:"Member name collision: {0}",E0003:"A class must be instantiated using the 'new' keyword",E0004:"The NAME property of the component is not specified",E0005:"Unknown device",E0006:"Unknown endpoint key is requested",E0007:"'Invalidate' method is called outside the update transaction",E0008:"Type of the option name is not appropriate to create an action",E0009:"Component '{0}' has not been initialized for an element",E0010:"Animation configuration with the '{0}' type requires '{1}' configuration as {2}",E0011:"Unknown animation type '{0}'",E0012:"jQuery version is too old. Please upgrade jQuery to 1.10.0 or later",E0013:"KnockoutJS version is too old. Please upgrade KnockoutJS to 2.3.0 or later",E0014:"The 'release' method shouldn't be called for an unlocked Lock object",E0015:"Queued task returned an unexpected result",E0017:"Event namespace is not defined",E0018:"DevExpress.ui.DevExpressPopup widget is required",E0020:"Template engine '{0}' is not supported",E0021:"Unknown theme is set: {0}",E0022:"LINK[rel=DevExpress-theme] tags must go before DevExpress included scripts",E0023:"Template name is not specified",E0024:"DevExtreme bundle already included",E0025:"Unexpected argument type",E0100:"Unknown validation type is detected",E0101:"Misconfigured range validation rule is detected",E0102:"Misconfigured comparison validation rule is detected",E0103:"validationCallback of an asynchronous rule should return a jQuery or a native promise",E0110:"Unknown validation group is detected",E0120:"Adapter for a DevExpressValidator component cannot be configured",E0121:"The 'customItem' field of the 'onCustomItemCreating' function's parameter should contain a custom item or Promise that is resolved after the item is created.",W0000:"'{0}' is deprecated in {1}. {2}",W0001:"{0} - '{1}' option is deprecated in {2}. {3}",W0002:"{0} - '{1}' method is deprecated in {2}. {3}",W0003:"{0} - '{1}' property is deprecated in {2}. {3}",W0004:"Timeout for theme loading is over: {0}",W0005:"'{0}' event is deprecated in {1}. {2}",W0006:"Invalid recurrence rule: '{0}'",W0007:"'{0}' Globalize culture is not defined",W0008:"Invalid view name: '{0}'",W0009:"Invalid time zone name: '{0}'",W0010:"{0} is deprecated in {1}. {2}",W0011:"Number parsing is invoked while the parser is not defined",W0012:"Date parsing is invoked while the parser is not defined",W0013:"'{0}' file is deprecated in {1}. {2}",W0014:"{0} - '{1}' type is deprecated in {2}. {3}",W0015:"Instead of returning a value from the '{0}' function, write it into the '{1}' field of the function's parameter.",W0016:'The "{0}" option does not accept the "{1}" value since v.{2}. {3}.'});t.default=a,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(0),o=n(7),a=(0,o.getNavigator)(),r=/(webkit)[ /]([\w.]+)/,s=/(msie) (\d{1,2}\.\d)/,l=/(trident).*rv:(\d{1,2}\.\d)/,u=/(edge)\/((\d+)?[\w.]+)/,c=/(mozilla)(?:.*? rv:([\w.]+))/,d=function(e){e=e.toLowerCase();var t={},n=s.exec(e)||l.exec(e)||u.exec(e)||e.indexOf("compatible")<0&&c.exec(e)||r.exec(e)||[],i=n[1],o=n[2];return"webkit"===i&&(t.webkit=!0,e.indexOf("chrome")>=0||e.indexOf("crios")>=0?(i="chrome",o=/(?:chrome|crios)\/(\d+\.\d+)/.exec(e),o=o&&o[1]):e.indexOf("fxios")>=0?(i="mozilla",o=/fxios\/(\d+\.\d+)/.exec(e),o=o&&o[1]):e.indexOf("safari")>=0&&/version|phantomjs/.test(e)?(i="safari",o=/(?:version|phantomjs)\/([0-9.]+)/.exec(e),o=o&&o[1]):(i="unknown",o=/applewebkit\/([0-9.]+)/.exec(e),o=o&&o[1])),"trident"!==i&&"edge"!==i||(i="msie"),i&&(t[i]=!0,t.version=o),t},h=(0,i.extend)({_fromUA:d},d(a.userAgent));t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getDefaultAlignment=t.getBoundingRect=void 0;var o=i(n(28)),a=n(1),r=function(e){var t=null!==e&&void 0!==e?e:(0,o.default)().rtlEnabled;return t?"right":"left"};t.getDefaultAlignment=r;var s=function(e){if((0,a.isWindow)(e))return{width:e.outerWidth,height:e.outerHeight};var t;try{t=e.getBoundingClientRect()}catch(e){t={width:0,height:0,bottom:0,top:0,left:0,right:0}}return t};t.getBoundingRect=s},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return a=function(){return e},e}function r(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=a();if(t&&t.has(e))return t.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=o?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}t.default=void 0;var s=r(n(46)),l=n(3),u=o(n(23)),c=o(n(13)),d=o(n(66)),h=o(n(254)),f=o(n(373)),p=o(n(256)),g=o(n(374)),_=function(e,t,n){if(e.pointerEvents&&n.msie)return f.default;var i=t.tablet,o=t.phone;return!e.touch||i||o?e.touch?h.default:p.default:g.default},m=_(s,c.default.real(),u.default);(0,l.each)(m.map,function(e,t){(0,d.default)(e,new m(e,t))});var v={down:"dxpointerdown",up:"dxpointerup",move:"dxpointermove",cancel:"dxpointercancel",enter:"dxpointerenter",leave:"dxpointerleave",over:"dxpointerover",out:"dxpointerout"},y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?o(Object(n),!0).forEach(function(t){r(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):o(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var s=i(n(37)),l=i(n(44)),u=a(a(a({},l.default),s.default),{},{modules:[]});t.default=u,e.exports=t.default},function(e,t,n){t.default=void 0;var i=function(e){this._options=e||{},this._list=[],this._queue=[],this._firing=!1,this._fired=!1,this._firingIndexes=[]};i.prototype._fireCore=function(e,t){var n=this._firingIndexes,i=this._list,o=this._options.stopOnFalse,a=n.length;for(n[a]=0;n[a]<i.length;n[a]++){var r=i[n[a]].apply(e,t);if(r===!1&&o)break}n.pop()},i.prototype.add=function(e){return"function"!=typeof e||this._options.unique&&this.has(e)||this._list.push(e),this},i.prototype.remove=function(e){var t=this._list,n=this._firingIndexes,i=t.indexOf(e);if(i>-1&&(t.splice(i,1),this._firing&&n.length))for(var o=0;o<n.length;o++)i<=n[o]&&n[o]--;return this},i.prototype.has=function(e){var t=this._list;return e?t.indexOf(e)>-1:!!t.length},i.prototype.empty=function(e){return this._list=[],this},i.prototype.fireWith=function(e,t){var n=this._queue;if(t=t||[],t=t.slice?t.slice():t,this._options.syncStrategy)this._firing=!0,this._fireCore(e,t);else{if(n.push([e,t]),this._firing)return;for(this._firing=!0;n.length;){var i=n.shift();this._fireCore(i[0],i[1])}}return this._firing=!1,this._fired=!0,this},i.prototype.fire=function(){this.fireWith(this,arguments)},i.prototype.fired=function(){return this._fired};var o=function(e){return new i(e)},a=o;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(22)),r={rtlEnabled:!1,defaultCurrency:"USD",oDataFilterToLower:!0,serverDecimalSeparator:".",decimalSeparator:".",thousandsSeparator:",",forceIsoDateParsing:!0,wrapActionsBeforeExecute:!0,useLegacyStoreResult:!1,useJQuery:void 0,editorStylingMode:void 0,useLegacyVisibleIndex:!1,floatingActionButtonConfig:{icon:"add",closeIcon:"close",label:"",position:{at:"right bottom",my:"right bottom",offset:{x:-16,y:-16}},maxSpeedDialActionCount:5,shading:!1,direction:"auto"},optionsParser:function(e){"{"!==e.trim().charAt(0)&&(e="{"+e+"}");try{return new Function("return "+e)()}catch(t){throw a.default.Error("E3018",t,e)}}},s=["decimalSeparator","thousandsSeparator"],l=function(){if(!arguments.length)return r;var e=arguments.length<=0?void 0:arguments[0];s.forEach(function(t){if(e[t]){var n="Now, the ".concat(t," is selected based on the specified locale.");a.default.log("W0003","config",t,"19.2",n)}}),(0,o.extend)(r,e)};"undefined"!=typeof DevExpress&&DevExpress.config&&l(DevExpress.config);var u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return Math.abs(parseInt(e.toExponential().split("e")[1]))}function o(){var e=3e-4,t="0.000300",n=3;return t!==e.toPrecision(n)}function a(e,t){var n,a=r(t||0)+2,s=e.toString().split("."),l=e,u=Math.abs(e),c=(0,f.isExponential)(e),d=u>1?10:0;return 1===s.length?e:(c||((0,f.isExponential)(t)&&(a=s[0].length+i(t)),e=u,e=e-Math.floor(e)+d),a=o()&&i(e)>6||a>7?15:7,c||(n=parseFloat(e.toPrecision(a)).toString().split("."),n[0]!==d.toString())?parseFloat(l.toPrecision(a)):parseFloat(s[0]+"."+n[1]))}function r(e){var t=e.toString();if(t.indexOf(".")<0)return 0;var n=t.split("."),i=n[1].indexOf("e");return i>=0?i:n[1].length}function s(e,t){if(e<0&&t%2!==1)return NaN;var n=Math.pow(Math.abs(e),1/t);return t%2===1&&e<0?-n:n}function l(e,t,n,i){var o=1e-8;if(Math.abs(e)<o){if(e=t,t=n,n=i,Math.abs(e)<o)return e=t,t=n,Math.abs(e)<o?[]:[-t/e];var a=t*t-4*e*n;return Math.abs(a)<o?[-t/(2*e)]:a>0?[(-t+Math.sqrt(a))/(2*e),(-t-Math.sqrt(a))/(2*e)]:[]}var r,l,u=(3*e*n-t*t)/(3*e*e),c=(2*t*t*t-9*e*t*n+27*e*e*i)/(27*e*e*e);if(Math.abs(u)<o)r=[s(-c,3)];else if(Math.abs(c)<o)r=[0].concat(u<0?[Math.sqrt(-u),-Math.sqrt(-u)]:[]);else{var d=c*c/4+u*u*u/27;if(Math.abs(d)<o)r=[-1.5*c/u,3*c/u];else if(d>0)l=s(-c/2-Math.sqrt(d),3),r=[l-u/(3*l)];else{l=2*Math.sqrt(-u/3);var h=Math.acos(3*c/u/l)/3,f=2*Math.PI/3;r=[l*Math.cos(h),l*Math.cos(h-f),l*Math.cos(h-2*f)]}}for(var p=0;p<r.length;p++)r[p]-=t/(3*e);return r}function u(e){return Math.trunc?Math.trunc(e):e>0?Math.floor(e):Math.ceil(e)}function c(e,t,n){if(t===parseInt(t))return e%t;var i=h(e/t,n);return(i-parseInt(i))*t}function d(e){var t,n=e.toString();return(null===(t=n.split(".")[1])||void 0===t?void 0:t.length)||parseInt(n.split("e-")[1])||0}function h(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return parseFloat(e.toFixed(t))}t.adjust=a,t.getPrecision=r,t.getExponent=i,t.getRoot=s,t.solveCubicEquation=l,t.trunc=u,t.roundFloatPart=h,t.getExponentLength=d,t.getRemainderByDivision=c,t.inRange=t.fitIntoRange=t.sign=void 0;var f=n(1),p=function(e){return 0===e?0:e/Math.abs(e)};t.sign=p;var g=function(e,t,n){var i=!t&&0!==t,o=!n&&0!==n;return i&&(t=o?e:Math.min(e,n)),o&&(n=i?e:Math.max(e,t)),Math.min(Math.max(e,t),n)};t.fitIntoRange=g;var _=function(e,t,n){return e>=t&&e<=n};t.inRange=_},function(e,t,n){function i(e){return"string"===(0,r.type)(e)&&"%"===e[e.length-1]}function o(e,t){e.length&&(0,a.data)(e.get(0),s,t)}t.parseTranslate=t.resetPosition=t.move=t.getTranslate=t.getTranslateCss=t.clearCache=t.locate=void 0;var a=n(39),r=n(1),s="dxTranslator",l=/matrix(3d)?\((.+?)\)/,u=/translate(?:3d)?\((.+?)\)/,c=function(e){var t=f(e);return{left:t.x,top:t.y}};t.locate=c;var d=function(e){e.length&&(0,a.removeData)(e.get(0),s)};t.clearCache=d;var h=function(e){e.x=e.x||0,e.y=e.y||0;var t=i(e.x)?e.x:e.x+"px",n=i(e.y)?e.y:e.y+"px";return"translate("+t+", "+n+")"};t.getTranslateCss=h;var f=function(e){var t=e.length?(0,a.data)(e.get(0),s):null;if(!t){var n=e.css("transform")||h({x:0,y:0}),i=n.match(l),r=i&&i[1];i?(i=i[2].split(","),"3d"===r?i=i.slice(12,15):(i.push(0),i=i.slice(4,7))):i=[0,0,0],t={x:parseFloat(i[0]),y:parseFloat(i[1]),z:parseFloat(i[2])},o(e,t)}return t};t.getTranslate=f;var p=function(e,t){var n,a=t.left,r=t.top;void 0===a?(n=f(e),n.y=r||0):void 0===r?(n=f(e),n.x=a||0):(n={x:a||0,y:r||0,z:0},o(e,n)),e.css({transform:h(n)}),(i(a)||i(r))&&d(e)};t.move=p;var g=function(e,t){var n,i={left:0,top:0,transform:"none"};t&&(n=e.css("transition"),i.transition="none"),e.css(i),d(e),t&&(e.get(0).offsetHeight,e.css("transition",n))};t.resetPosition=g;var _=function(e){var t=e.match(u);if(t&&t[1])return t=t[1].split(","),t={x:parseFloat(t[0]),y:parseFloat(t[1]),z:parseFloat(t[2])}};t.parseTranslate=_},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(44)),r=(0,o.extend)({},a.default,{modules:[],foreachNodes:function(e,t){for(var n=0;n<e.length;n++)t(e[n])!==!1&&e[n].hasChildren&&e[n].children.length&&this.foreachNodes(e[n].children,t)}});t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(54)),a=n(1),r=n(3),s=n(12),l=i(n(22)),u=n(242),c=n(161),d=n(208),h=i(n(243)),f=i(n(340)),p=i(n(82)),g=i(n(45)),_=i(n(341)),m=0,v="undefined"!=typeof Intl,y={shortdate:"M/d/y",shorttime:"h:mm a",longdate:"EEEE, MMMM d, y",longtime:"h:mm:ss a",monthandday:"MMMM d",monthandyear:"MMMM y",quarterandyear:"QQQ y",day:"d",year:"y",shortdateshorttime:"M/d/y, h:mm a",longdatelongtime:"EEEE, MMMM d, y, h:mm:ss a",month:"LLLL",shortyear:"yy",dayofweek:"EEEE",quarter:"QQQ",hour:"HH",minute:"mm",second:"ss",millisecond:"SSS","datetime-local":"yyyy-MM-ddTHH':'mm':'ss"},b={year:["y","yy","yyyy"],day:["d","dd"],month:["M","MM","MMM","MMMM"],hours:["H","HH","h","hh","ah"],minutes:["m","mm"],seconds:["s","ss"],milliseconds:["S","SS","SSS"]},x=(0,o.default)({engine:function(){return"base"},_getPatternByFormat:function(e){return y[e.toLowerCase()]},_expandPattern:function(e){return this._getPatternByFormat(e)||e},formatUsesMonthName:function(e){return this._expandPattern(e).indexOf("MMMM")!==-1},formatUsesDayName:function(e){return this._expandPattern(e).indexOf("EEEE")!==-1},getFormatParts:function(e){var t=this._getPatternByFormat(e)||e,n=[];return(0,r.each)(t.split(/\W+/),function(e,t){(0,r.each)(b,function(e,i){(0,s.inArray)(t,i)>-1&&n.push(e)})}),n},getMonthNames:function(e){return h.default.getMonthNames(e)},getDayNames:function(e){return h.default.getDayNames(e)},getQuarterNames:function(e){return h.default.getQuarterNames(e)},getPeriodNames:function(e){return h.default.getPeriodNames(e)},getTimeSeparator:function(){return":"},is24HourFormat:function(e){for(var t=new Date(2017,0,20,11,0,0,0),n=new Date(2017,0,20,23,0,0,0),i=this.format(t,e),o=this.format(n,e),a=0;a<i.length;a++)if(i[a]!==o[a])return!isNaN(parseInt(i[a]))},format:function(e,t){if(e){if(!t)return e;var n;if("function"==typeof t)n=t;else if(t.formatter)n=t.formatter;else if(t=t.type||t,(0,a.isString)(t))return t=y[t.toLowerCase()]||t,g.default.convertDigits((0,u.getFormatter)(t,this)(e));if(n)return n(e)}},parse:function(e,t){var n,i,o=this;if(e){if(!t)return this.parse(e,"shortdate");if(t.parser)return t.parser(e);if("string"!=typeof t||y[t.toLowerCase()]){i=function(e){var n=o.format(e,t);return g.default.convertDigits(n,!0)};try{n=(0,c.getFormat)(i)}catch(e){}}else n=t;if(n)return e=g.default.convertDigits(e,!0),(0,d.getParser)(n,this)(e);l.default.log("W0012");var a=new Date(e);if(a&&!isNaN(a.getTime()))return a}},firstDayOfWeekIndex:function(){var e=p.default.getValueByClosestLocale(function(e){return f.default[e]});return void 0===e?m:e}});v&&x.inject(_.default);var w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.createTextElementHiddenCopy=t.contains=t.clipboardText=t.normalizeTemplateElement=t.extractTemplateMarkup=t.closestCommonParent=t.clearSelection=t.resetActiveElement=void 0;var o=i(n(11)),a=i(n(2)),r=n(1),s=n(7),l=(0,s.getWindow)(),u=function(){var e=o.default.getActiveElement(),t=o.default.getBody();if(e&&e!==t&&e.blur)try{e.blur()}catch(e){t.blur()}};t.resetActiveElement=u;var c=function(){var e=l.getSelection();if(e&&"Caret"!==e.type)if(e.empty)e.empty();else if(e.removeAllRanges)try{e.removeAllRanges()}catch(e){}};t.clearSelection=c;var d=function(e,t){var n=(0,a.default)(e),i=(0,a.default)(t);if(n[0]===i[0])return n[0];for(var o=n.parents(),r=i.parents(),s=Math.min(o.length,r.length),l=-s;l<0;l++)if(o.get(l)===r.get(l))return o.get(l)};t.closestCommonParent=d;var h=function(e){e=(0,a.default)(e);var t=e.length&&e.filter(function(){var e=(0,a.default)(this);return e.is("script[type]")&&e.attr("type").indexOf("script")<0});return t.length?t.eq(0).html():(e=(0,a.default)("<div>").append(e),e.html())};t.extractTemplateMarkup=h;var f=function e(t){var n=(0,r.isDefined)(t)&&(t.nodeType||(0,r.isRenderer)(t))?(0,a.default)(t):(0,a.default)("<div>").html(t).contents();return 1===n.length&&(n.is("script")?n=e(n.html().trim()):n.is("table")&&(n=n.children("tbody").contents())),n};t.normalizeTemplateElement=f;var p=function(e,t){var n=e.originalEvent&&e.originalEvent.clipboardData||l.clipboardData;return 1===arguments.length?n&&n.getData("Text"):void(n&&n.setData("Text",t))};t.clipboardText=p;var g=function e(t,n){return!!n&&(o.default.isTextNode(n)&&(n=n.parentNode),o.default.isDocument(t)?t.documentElement.contains(n):(0,r.isWindow)(t)?e(t.document,n):t.contains?t.contains(n):!!(n.compareDocumentPosition(t)&n.DOCUMENT_POSITION_CONTAINS))};t.contains=g;var _=function(e,t,n){var i=l.getComputedStyle((0,a.default)(e).get(0)),o=n&&n.includePaddings;return(0,a.default)("<div>").text(t).css({fontStyle:i.fontStyle,fontVariant:i.fontVariant,fontWeight:i.fontWeight,fontSize:i.fontSize,fontFamily:i.fontFamily,letterSpacing:i.letterSpacing,border:i.border,paddingTop:o?i.paddingTop:"",paddingRight:o?i.paddingRight:"",paddingBottom:o?i.paddingBottom:"",paddingLeft:o?i.paddingLeft:"",visibility:"hidden",whiteSpace:"pre",position:"absolute","float":"left"})};t.createTextElementHiddenCopy=_},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(135)),a=i(n(22)),r={},s=(0,o.default)(a.default.ERROR_MESSAGES,{E4000:"[DevExpress.data]: {0}",E4001:"Unknown aggregating function is detected: '{0}'",E4002:"Unsupported OData protocol version is used",E4003:"Unknown filter operation is used: {0}",E4004:"The thenby() method is called before the sortby() method",E4005:"Store requires a key expression for this operation",E4006:"ArrayStore 'data' option must be an array",E4007:"Compound keys cannot be auto-generated",E4008:"Attempt to insert an item with a duplicated key",E4009:"Data item cannot be found",E4010:"CustomStore does not support creating queries",E4011:"Custom Store method is not implemented or is not a function: {0}",E4012:"Custom Store method returns an invalid value: {0}",E4013:"Local Store requires the 'name' configuration option is specified",E4014:"Unknown data type is specified for ODataStore: {0}",E4015:"Unknown entity name or alias is used: {0}",E4016:"The compileSetter(expr) method is called with 'self' passed as a parameter",E4017:"Keys cannot be modified",E4018:"The server has returned a non-numeric value in a response to an item count request",E4019:"Mixing of group operators inside a single group of filter expression is not allowed",E4020:"Unknown store type is detected: {0}",E4021:"The server response does not provide the totalCount value",E4022:"The server response does not provide the groupCount value",E4023:"Could not parse the following XML: {0}",E4024:"String function {0} cannot be used with the data field {1} of type {2}.",W4000:"Data returned from the server has an incorrect structure",W4001:'The {0} field is listed in both "keyType" and "fieldTypes". The value of "fieldTypes" is used.',W4002:"Data loading has failed for some cells due to the following error: {0}"}),l=null,u=function(e){r.errorHandler&&r.errorHandler(e)};r={errors:s,errorHandler:l,_errorHandler:u};var c=r;
t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=o.default.inherit({ctor:function(e){e&&(e=String(e)),this._value=this._normalize(e||this._generate())},_normalize:function(e){for(e=e.replace(/[^a-f0-9]/gi,"").toLowerCase();e.length<32;)e+="0";return[e.substr(0,8),e.substr(8,4),e.substr(12,4),e.substr(16,4),e.substr(20,12)].join("-")},_generate:function(){for(var e="",t=0;t<32;t++)e+=Math.round(15*Math.random()).toString(16);return e},toString:function(){return this._value},valueOf:function(){return this._value},toJSON:function(){return this._value}}),r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){if(!(0,N.hasWindow)())return null;var e,t=(0,B.default)("<div>",D).addClass("dx-theme-marker").appendTo(D.documentElement);try{return(e=t.css("fontFamily"))?(e=e.replace(/["']/g,""),e.substr(0,te.length)!==te?null:e.substr(te.length)):null}finally{t.remove()}}function a(e){function t(){E=null,clearInterval(i),o=!0,j.themeReadyCallback.fire(),j.themeReadyCallback.empty(),U.resolve()}var n,i,o=!0;if(E=e,r()||!ee)t();else{if(!o)return void(E&&(E=e));n=Date.now(),o=!1,i=setInterval(function(){var e=r(),i=!e&&Date.now()-n>ee;i&&W.default.log("W0004",E),(e||i)&&t()},10)}}function r(){if(!E)return!0;var e=E===J;if("resolved"===U.state()&&e)return!0;var t=o();return!(!t||!e)||t===E}function s(){var e=(0,B.default)(Y,D);e.length&&(T={},I=(0,B.default)((0,F.parseHTML)("<link rel=stylesheet>"),D),e.each(function(){var e=(0,B.default)(this,D),t=e.attr(Z),n=e.attr("href"),i="true"===e.attr(X);T[t]={url:n,isActive:i}}),e.last().after(I),e.remove())}function l(e){var t=e?e.split("."):[],n=null;if(T){if(e in T)return e;(0,L.each)(T,function(e,i){var o=e.split(".");if(!(t[0]&&o[0]!==t[0]||t[1]&&t[1]!==o[1]||t[2]&&t[2]!==o[2]))return n&&!i.isActive||(n=e),!i.isActive&&void 0})}return n}function u(e){try{e!==D&&(T=null)}catch(e){T=null}D=e}function c(e){e=e||{},u(e.context||R.default.getDocument()),D&&(s(),O=void 0,d(e))}function d(e){if(!arguments.length)return O=O||o();p($()),e=e||{},"string"==typeof e&&(e={theme:e});var t,n=e._autoInit,i=e.loadCallback;if(O=l(e.theme||O),O&&(t=T[O]),i&&j.themeReadyCallback.add(i),t)I.attr("href",T[O].url),(j.themeReadyCallback.has()||"resolved"!==U.state()||e._forceTimeout)&&a(O);else{if(!n)throw W.default.Error("E0021",O);a(J),j.themeReadyCallback.fire(),j.themeReadyCallback.empty()}U.done(function(){return f((0,z.originalViewPort)(),O)})}function h(e){e=e||d();var t=[],n=e&&e.split(".");return n&&(t.push("dx-theme-"+n[0],"dx-theme-"+n[0]+"-typography"),n.length>1&&t.push("dx-color-scheme-"+n[1]+(m(e)?"-"+n[2]:""))),t}function f(e,t){A=h(t).join(" "),(0,B.default)(e).addClass(A);var n=function(){var t=(0,N.hasWindow)()&&G.devicePixelRatio;if(t&&!(t<2)){var n=(0,B.default)("<div>");n.css("border",".5px solid transparent"),(0,B.default)("body").append(n),1===n.outerHeight()&&((0,B.default)(e).addClass(Q),A+=" "+Q),n.remove()}};n()}function p(e){(0,B.default)(e).removeClass(A)}function g(e){j.themeReadyCallback.add(e)}function _(e,t){return t||(t=O||o()),new RegExp(e).test(t)}function m(e){return _("material",e)}function v(e){return _("generic",e)}function y(e){return _("dark",e)}function b(e,t){var n="Roboto, RobotoFallback, Arial",i="Arial",o=R.default.getDocument(),a=o.createElement("span");a.style.position="absolute",a.style.top="-9999px",a.style.left="-9999px",a.style.visibility="hidden",a.style.fontFamily=i,a.style.fontSize="250px",a.style.fontWeight=t,a.innerHTML=e,o.body.appendChild(a);var r=a.offsetWidth;a.style.fontFamily=n;var s=a.offsetWidth;return a.parentNode.removeChild(a),r!==s}function x(e,t){var n=15,i=2e3;return new M.default(function(o){var a=function(){b(e,t)&&r()},r=function(){clearInterval(s),clearTimeout(l),o()},s=setInterval(a,n),l=setTimeout(r,i)})}function w(){if(c({_autoInit:!0,_forceTimeout:!0}),(0,B.default)(Y,D).length)throw W.default.Error("E0022")}function C(){I&&I.attr("href","about:blank"),O=null,E=null,U=new V.Deferred}function k(e){U.done(e)}function S(e){ee=e}t.waitForThemeLoad=a,t.init=c,t.current=d,t.attachCssClasses=f,t.detachCssClasses=p,t.isMaterial=m,t.isGeneric=v,t.isDark=y,t.isWebFontLoaded=b,t.waitWebFont=x,t.ready=g,t.resetTheme=C,t.initialized=k,t.setDefaultTimeout=S,t.default=void 0;var D,I,T,O,E,A,P=i(n(13)),R=i(n(11)),M=i(n(77)),B=i(n(2)),V=n(6),F=n(238),L=n(3),H=i(n(49)),z=n(78),N=n(7),j=n(258),W=i(n(17)),G=(0,N.getWindow)(),q=H.default.add,$=z.value,K=z.changeCallback,U=new V.Deferred,Y="link[rel=dx-theme]",Z="data-theme",X="data-active",Q="dx-hairlines",J="any",ee=15e3,te="dx.";(0,N.hasWindow)()?w():q(w),K.add(function(e,t){U.done(function(){p(t),f(e)})}),P.default.changed.add(function(){c({_autoInit:!0})});var ne={setDefaultTimeout:S,initialized:k,resetTheme:C,ready:g,waitWebFont:x,isWebFontLoaded:b,isDark:y,isGeneric:v,isMaterial:m,detachCssClasses:p,attachCssClasses:f,current:d,waitForThemeLoad:a};t.default=ne},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return"date"===e||"datetime"===e}t.default=void 0;var a=i(n(2)),r=n(1),s=n(6),l=i(n(196)),u=n(42),c=n(3),d=n(0),h=n(24),f=n(19),p=n(4),g=i(n(193)),_=i(n(41)),m=i(n(79)),v=n(7),y=i(n(5)),b="dx-selection-disabled",x="dx-datagrid-group-opened",w="dx-datagrid-group-closed",C="dx-datagrid-expand",k="nodata",S={year:function(e){return e&&e.getFullYear()},month:function(e){return e&&e.getMonth()+1},day:function(e){return e&&e.getDate()},quarter:function(e){return e&&Math.floor(e.getMonth()/3)+1},hour:function(e){return e&&e.getHours()},minute:function(e){return e&&e.getMinutes()},second:function(e){return e&&e.getSeconds()}},D=function(){var e=arguments[1],t=this.calculateCellValue(e);if(!(0,r.isDefined)(t))return null;if(o(this.dataType)){var n=arguments[0];return S[n](t)}if("number"===this.dataType){var i=arguments[0];return Math.floor(Number(t)/i)*i}},I=function(e,t){return(0,r.isFunction)(e)&&(0,r.isFunction)(t)&&e.originalCallback&&t.originalCallback?e.originalCallback===t.originalCallback&&e.columnIndex===t.columnIndex:e===t},T=function(e){e.get(0).textContent="\xa0"},O=function(e){e=e||[];for(var t=_.default.normalizeSortingInfo(e),n=0;n<e.length;n++)e&&e[n]&&void 0!==e[n].isExpanded&&(t[n].isExpanded=e[n].isExpanded),e&&e[n]&&void 0!==e[n].groupInterval&&(t[n].groupInterval=e[n].groupInterval);return t},E=function(e,t){var n=m.default.format(e,t.format)||e&&e.toString()||"",i={value:e,valueText:t.getDisplayFormat?t.getDisplayFormat(n):n,target:t.target||"row",groupInterval:t.groupInterval};return t.customizeText?t.customizeText.call(t,i):i.valueText},A=function(e,t){var n=e.displayFormat||e.columnCaption&&t[e.summaryType+"OtherColumn"]||t[e.summaryType];return E(e.value,{format:e.valueFormat,getDisplayFormat:function(t){return n?(0,u.format)(n,t,e.columnCaption):t},customizeText:e.customizeText})},P=function(e){var t=e.data&&e.data(),n=t&&t.dxComponents,i=n&&n[0];return i&&t[i]},R=function e(t,n){if(Array.isArray(t)&&Array.isArray(n)){if(t.length!==n.length)return!1;for(var i=0;i<t.length;i++)if(!e(t[i],n[i]))return!1;return!0}return(0,r.isFunction)(t)&&t.columnIndex>=0&&(0,r.isFunction)(n)&&n.columnIndex>=0?t.columnIndex===n.columnIndex&&(0,f.toComparable)(t.filterValue)===(0,f.toComparable)(n.filterValue):(0,f.toComparable)(t)==(0,f.toComparable)(n)},M={renderNoDataText:function(e){var t=this;if(e=e||this.element()){var n=t.addWidgetPrefix(k),i=e.find("."+n).last(),o=this._dataController.isEmpty(),r=this._dataController.isLoading();i.length||(i=(0,a.default)("<span>").addClass(n).appendTo(e)),o&&!r?i.removeClass("dx-hidden").text(t._getNoDataText()):i.addClass("dx-hidden")}},renderLoadPanel:function(e,t,n){var i,o=this;o._loadPanel&&o._loadPanel.$element().remove(),i=o.option("loadPanel"),i&&("auto"===i.enabled?!n:i.enabled)?(i=(0,d.extend)({shading:!1,message:i.text,position:function(){var t=(0,a.default)((0,v.getWindow)());return e.height()>t.height()?{of:t,boundary:e,collision:"fit"}:{of:e}},container:t},i),o._loadPanel=o._createComponent((0,a.default)("<div>").appendTo(t),g.default,i)):o._loadPanel=null},getIndexByKey:function(e,t,n){var i=-1;if(void 0!==e&&Array.isArray(t)){n=arguments.length<=2?"key":n;for(var o=0;o<t.length;o++){var a=(0,r.isDefined)(n)?t[o][n]:t[o];if((0,p.equalByValue)(e,a)){i=o;break}}}return i},combineFilters:function(e,t){var n=[];t=t||"and";for(var i=0;i<e.length;i++)e[i]&&(n.length&&n.push(t),n.push(e[i]));if(1===n.length&&(n=n[0]),n.length)return n},checkChanges:function(e,t){for(var n=0,i=0;i<t.length;i++)e[t[i]]&&n++;return e.length&&e.length===n},equalFilterParameters:R,proxyMethod:function(e,t,n){e[t]||(e[t]=function(){var e=this._dataSource;return e?e[t].apply(e,arguments):n})},formatValue:E,getFormatOptionsByColumn:function(e,t){return{format:e.format,getDisplayFormat:e.getDisplayFormat,customizeText:e.customizeText,target:t,trueText:e.trueText,falseText:e.falseText}},getDisplayValue:function(e,t,n,i){return e.displayValueMap&&void 0!==e.displayValueMap[t]?e.displayValueMap[t]:e.calculateDisplayValue&&n&&"group"!==i?e.calculateDisplayValue(n):!e.lookup||"group"===i&&(e.calculateGroupValue||e.calculateDisplayValue)?t:e.lookup.calculateCellValue(t)},getGroupRowSummaryText:function(e,t){for(var n="(",i=0;i<e.length;i++){var o=e[i];n+=(i>0?", ":"")+A(o,t)}return n+=")"},getSummaryText:A,normalizeSortingInfo:O,getFormatByDataType:function(e){switch(e){case"date":return"shortDate";case"datetime":return"shortDateShortTime"}},getHeaderFilterGroupParameters:function(e,t){var n=[],i=e.dataField||e.name,o=l.default.getGroupInterval(e);return o?((0,c.each)(o,function(a,r){n.push(t?{selector:i,groupInterval:r,isExpanded:a<o.length-1}:D.bind(e,r))}),n):(t?n=[{selector:i,isExpanded:!1}]:(n=function t(n){var t=e.calculateCellValue(n);return void 0!==t&&""!==t||(t=null),t},e.sortingMethod&&(n=[{selector:n,compare:e.sortingMethod.bind(e)}])),n)},equalSortParameters:function(e,t,n){if(e=O(e),t=O(t),Array.isArray(e)&&Array.isArray(t)){if(e.length!==t.length)return!1;for(var i=0;i<e.length;i++)if(!I(e[i].selector,t[i].selector)||e[i].desc!==t[i].desc||e[i].groupInterval!==t[i].groupInterval||!n&&Boolean(e[i].isExpanded)!==Boolean(t[i].isExpanded))return!1;return!0}return(!e||!e.length)==(!t||!t.length)},getPointsByColumns:function(e,t,n,i){for(var o,a,r,s=e.length,l=!1,u=i||0,c=[],d=0;d<=s;d++){d<s&&(o=e.eq(d),a=o.offset(),r="rtl"===o.css("direction"));var f={index:u,x:a?a.left+(!n&&r^d===s?(0,h.getBoundingRect)(o[0]).width:0):0,y:a?a.top+(n&&d===s?(0,h.getBoundingRect)(o[0]).height:0):0,columnIndex:u};if(!n&&d>0){var p=e.eq(d-1).offset();p.top<f.y&&(f.y=p.top)}t&&(l=t(f)),l||c.push(f),u++}return c},getExpandCellTemplate:function(){return{allowRenderToDetachedContainer:!0,render:function(e,t){var n=(0,a.default)(e);if(!(0,r.isDefined)(t.value)||t.data&&t.data.isContinuation||t.row.isNewRow)T(n);else{var i=t.component.getView("rowsView");n.addClass(C).addClass(b),(0,a.default)("<div>").addClass(t.value?x:w).appendTo(n),i.setAria("label",t.value?i.localize("dxDataGrid-ariaCollapse"):i.localize("dxDataGrid-ariaExpand"),n)}}}},setEmptyText:T,isDateType:o,getSelectionRange:function(e){try{if(e)return{selectionStart:e.selectionStart,selectionEnd:e.selectionEnd}}catch(e){}return{}},setSelectionRange:function(e,t){try{e&&e.setSelectionRange&&e.setSelectionRange(t.selectionStart,t.selectionEnd)}catch(e){}},focusAndSelectElement:function(e,t){y.default.trigger(t,"focus");var n=e.option("editing.selectTextOnEditStart"),i=e.getController("keyboardNavigation"),o=i&&i._isFastEditingStarted(),a=t.get(0);if(n&&!o&&t.is(".dx-texteditor-input")&&!t.is("[readonly]")){var r=P(t.closest(".dx-texteditor"));(0,s.when)(r&&r._loadItemDeferred).done(function(){a.select()})}},getWidgetInstance:P,getLastResizableColumnIndex:function(e,t){var n,i=e.some(function(e){return e&&!e.command&&!e.fixed&&e.allowResizing!==!1});for(n=e.length-1;e[n];n--){var o=e[n],a=t&&t[n],r=!i||o.allowResizing!==!1;if(!o.command&&!o.fixed&&"adaptiveHidden"!==a&&r)break}return n},isElementInCurrentGrid:function(e,t){if(t&&t.length){var n=t.closest("."+e.getWidgetContainerClass()).parent();return n.is(e.component.$element())}return!1}};t.default=M,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(13)),y=n(87),b=o(n(9)),x=n(36),w=o(n(108)),C=o(n(100)),k=o(n(16)),S=n(119),D=n(0),I=n(118),T=n(59),O=n(18),E="content",A=function(e){function t(){var e;a(this,t);for(var i=arguments.length,o=new Array(i),r=0;r<i;r++)o[r]=arguments[r];return e=n.call.apply(n,[this].concat(o)),e._feedbackHideTimeout=100,e}c(t,e);var n=h(t);return s(t,[{key:"_$content",value:function(){return this.$element().find(".dx-button-content")}},{key:"_$submitInput",value:function(){return this.$element().find(".dx-button-submit-input")}},{key:"_attachActiveEvents",value:function(e,t){var n=this._eventBindingTarget(),i="inkRipple",o=this._activeStateUnit;S.active.off(n,{namespace:i,selector:o}),S.active.on(n,new w.default(e),new w.default(t,{excludeValidators:["disabled","readOnly"]}),{showTimeout:this._feedbackShowTimeout,hideTimeout:this._feedbackHideTimeout,selector:o,namespace:i})}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return"desktop"===v.default.real().deviceType&&!v.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,x.isMaterial)((0,x.current)())},options:{useInkRipple:!0}}])}},{key:"_executeClickAction",value:function(e){this._clickAction({validationGroup:this._validationGroupConfig,event:e})}},{key:"_findGroup",value:function(){var e=this.$element(),t=this._modelByElement(e),n=this.option(),i=n.validationGroup;return i||C.default.findGroup(e,t)}},{key:"_getContentData",value:function(){var e=this.option(),t=e.icon,n=e.text,i=e.type,o=e._templateData;return(0,D.extend)({icon:"back"!==i||t?t:"back",text:n},o)}},{key:"_getDefaultOptions",value:function(){return(0,D.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{hoverStateEnabled:!0,onClick:null,type:"normal",text:"",icon:"",iconPosition:"left",validationGroup:void 0,activeStateEnabled:!0,template:"content",useSubmitBehavior:!1,useInkRipple:!1,_templateData:{},stylingMode:"contained"})}},{key:"_getSubmitAction",value:function(){var e=this,t=!0,n="valid";return this._createAction(function(i){var o=i.event;if(t){var a=e._validationGroupConfig;if(a){var r=a.validate(),s=r.status,l=r.complete;n=s,"pending"===s&&(t=!1,e.option("disabled",!0),l.then(function(i){var o=i.status;e.option("disabled",!1),n=o,"valid"===n&&e._submitInput().click(),t=!0}))}}"valid"!==n&&o.preventDefault(),o.stopPropagation()})}},{key:"_initMarkup",value:function(){this.$element().addClass("dx-button"),this._renderType(),this._renderStylingMode(),this._renderInkRipple(),this._renderClick(),this._updateAriaLabel(),l(_(t.prototype),"_initMarkup",this).call(this),this._updateContent(),this.setAria("role","button")}},{key:"_getAnonymousTemplateName",value:function(){return E}},{key:"_initTemplates",value:function(){var e=this;this._templateManager.addDefaultTemplates({content:new I.FunctionTemplate(function(t){var n=t.model,i=void 0===n?{}:n,o=t.container,a=i.text,r=i.icon,s=e.option(),l=s.iconPosition,u=(0,T.getImageContainer)(r),c=a&&(0,m.default)("<span>").text(a).addClass("dx-button-text"),d=(0,m.default)(o);d.append(c),"left"===l?d.prepend(u):(u.addClass("dx-icon-right"),d.append(u))})}),l(_(t.prototype),"_initTemplates",this).call(this)}},{key:"_optionChanged",value:function(e){var n=e.name,i=e.previousValue;switch(n){case"onClick":this._updateClick();break;case"icon":case"text":this._updateContent(),this._updateAriaLabel();break;case"type":this._updateType(i),this._updateContent();break;case"_templateData":break;case"template":case"iconPosition":this._updateContent();break;case"stylingMode":this._updateStylingMode();break;case"useSubmitBehavior":this._updateSubmitInput();break;case"useInkRipple":this._invalidate();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_renderClick",value:function(){var e=this,t=this.$element();S.dxClick.off(t,{namespace:this.NAME}),S.dxClick.on(t,function(t){return e._executeClickAction(t)},{namespace:this.NAME}),this._updateClick()}},{key:"_renderInkRipple",value:function(){var e=this,t=this.option(),n=t.text,i=t.icon,o=t.type,a=t.useInkRipple;if(a){var r=!n&&i||"back"===o,s=(0,y.render)(r?{waveSizeCoefficient:1,useHoldAnimation:!1,isCentered:!0}:{}),l=function(t,n){var i=e.option(),o=i.activeStateEnabled,a=i.useInkRipple;if(a&&o&&!e._disposed){var r={element:e._$content(),event:t};n?s.showWave(r):s.hideWave(r)}};this._attachActiveEvents(function(e){var t=e.event;return l(t,!0)},function(e){var t=e.event;return l(t)})}}},{key:"_renderStylingMode",value:function(){var e=this.$element(),t=this.option(),n=t.stylingMode;["contained","text","outlined"].indexOf(n)===-1&&(n=this._getDefaultOptions().stylingMode),e.addClass("dx-button-mode-".concat(n))}},{key:"_renderSubmitInput",value:function(){var e=this.option(),t=e.useSubmitBehavior;if(t){var n=this._getSubmitAction(),i=this._$content();(0,m.default)("<input>").attr("type","submit").attr("tabindex",-1).addClass("dx-button-submit-input").appendTo(i),S.click.on(this._$submitInput(),function(e){return n({event:e})})}}},{key:"_renderType",value:function(){var e=this.option(),t=e.type,n=this.$element();t&&n.addClass("dx-button-".concat(t))}},{key:"_submitInput",value:function(){return this._$submitInput().get(0)}},{key:"_supportedKeys",value:function(){var e=this,n=function(t){t.preventDefault(),e._executeClickAction(t)};return(0,D.extend)(l(_(t.prototype),"_supportedKeys",this).call(this),{space:n,enter:n})}},{key:"_updateAriaLabel",value:function(){var e=this._getAriaTarget(),t=this.option(),n=t.icon,i=t.text;i||("image"===(0,T.getImageSourceType)(n)&&(n=n.indexOf("base64")===-1?n.replace(/.+\/([^.]+)\..+$/,"$1"):"Base64"),i=n||""),e.attr("aria-label",i||null)}},{key:"_updateClick",value:function(){var e=this;this._clickAction=this._createActionByOption("onClick",{excludeValidators:["readOnly"],afterExecute:function(){var t=e.option(),n=t.useSubmitBehavior;n&&setTimeout(function(){return e._submitInput().click()})}})}},{key:"_updateContent",value:function(){var e=this.$element(),t=this._$content(),n=this._getContentData(),i=this.option(),o=i.template,a=i.iconPosition,r=n.icon,s=n.text;t.length?t.empty():t=(0,m.default)("<div>").addClass("dx-button-content").appendTo(e),e.toggleClass("dx-button-has-icon",!!r).toggleClass("dx-button-icon-right",!!r&&"left"!==a).toggleClass("dx-button-has-text",!!s);var l=(0,m.default)(this._getTemplateByOption("template").render({model:n,container:(0,O.getPublicElement)(t),transclude:this._templateManager.anonymousTemplateName===o}));l.hasClass("dx-template-wrapper")&&(l.addClass("dx-button-content"),t.replaceWith(l)),this._updateSubmitInput()}},{key:"_updateSubmitInput",value:function(){var e=this.option(),t=e.useSubmitBehavior,n=this._$submitInput();!t&&n.length?n.remove():t&&!n.length&&this._renderSubmitInput()}},{key:"_updateStylingMode",value:function(){var e=this.$element();["contained","text","outlined"].map(function(e){return"dx-button-mode-".concat(e)}).forEach(e.removeClass.bind(e)),this._renderStylingMode()}},{key:"_updateType",value:function(e){var t=this.$element();[e,"back","danger","default","normal","success"].map(function(e){return"dx-button-".concat(e)}).forEach(t.removeClass.bind(t)),this._renderType()}},{key:"_validationGroupConfig",get:function(){return C.default.getGroupConfig(this._findGroup())}}]),t}(k.default);(0,b.default)("dxButton",A);var P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return d}function a(){return d.data.apply(this,arguments)}function r(e){v=e}function s(e){y=e}function l(e){return d.cleanData.call(this,e)}function u(e,t){return d.removeData.call(this,e,t)}function c(e,t){if(f.default.isElementNode(e)){var n=e.getElementsByTagName("*");d.cleanData(n),t&&d.cleanData([e])}}t.getDataStrategy=o,t.data=a,t.beforeCleanData=r,t.afterCleanData=s,t.cleanData=l,t.removeData=u,t.cleanDataRecursive=c,t.setDataStrategy=t.strategyChanging=void 0;var d,h=i(n(182)),f=i(n(11)),p=i(n(5)),g=i(n(183)),_=new h.default,m=new g.default;t.strategyChanging=m;var v=function(){},y=function(){},b=function(e){m.fire(e),d=e;var t=d.cleanData;d.cleanData=function(e){v(e);var n=t.call(this,e);return y(e),n}};t.setDataStrategy=b,b({data:function(){var e=arguments[0],t=arguments[1],n=arguments[2];if(e){var i=_.get(e);return i||(i={},_.set(e,i)),void 0===t?i:2===arguments.length?i[t]:(i[t]=n,n)}},removeData:function(e,t){if(e)if(void 0===t)_.delete(e);else{var n=_.get(e);n&&delete n[t]}},cleanData:function(e){for(var t=0;t<e.length;t++)p.default.off(e[t]),_.delete(e[t])}})},function(e,t,n){t.captionize=t.titleize=t.humanize=t.camelize=t.underscore=t.dasherize=void 0;var i=n(3),o=function(e){return void 0===e||null===e?"":String(e)},a=function(e){return o(e).charAt(0).toUpperCase()+e.substr(1)},r=function(e){return o(e).replace(/([a-z\d])([A-Z])/g,"$1 $2").split(/[\s_-]+/)},s=function(e){return(0,i.map)(r(e),function(e){return e.toLowerCase()}).join("-")};t.dasherize=s;var l=function(e){return s(e).replace(/-/g,"_")};t.underscore=l;var u=function(e,t){return(0,i.map)(r(e),function(e,n){return e=e.toLowerCase(),(t||n>0)&&(e=a(e)),e}).join("")};t.camelize=u;var c=function(e){return a(s(e).replace(/-/g," "))};t.humanize=c;var d=function(e){return(0,i.map)(r(e),function(e){return a(e.toLowerCase())}).join(" ")};t.titleize=d;var h=["0","1","2","3","4","5","6","7","8","9"],f=function(e){var t,n,i=[],o=!1,a=!1;for(t=0;t<e.length;t++)n=e.charAt(t),a=n===n.toUpperCase()&&"-"!==n&&")"!==n&&"/"!==n||n in h,"_"===n||"."===n?(n=" ",a=!0):0===t?(n=n.toUpperCase(),a=!0):!o&&a&&i.length>0&&i.push(" "),i.push(n),o=a;return i.join("")};t.captionize=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function c(e){return/^(or|\|\||\|)$/i.test(e)}function d(e){return/^(and|&&|&)$/i.test(e)}function h(e){var t,n,i=[];for(n=0;n<e.length;n++)t=e.charCodeAt(n),t<128?i.push(t):t<2048?i.push(192+(t>>6),128+(63&t)):t<65536?i.push(224+(t>>12),128+(t>>6&63),128+(63&t)):t<2097152&&i.push(240+(t>>18),128+(t>>12&63),128+(t>>6&63),128+(63&t));return i}function f(e,t){var n,i;return function(){var o=this;return i=arguments,n||(n=setTimeout(function(){n=void 0,i&&e.call(o,i)},(0,g.isFunction)(t)?t():t)),n}}function p(e,t){var n=[],i=f(function(){e.call(this,n),n=[]},t);return function(e){if(Array.isArray(e)){var t;(t=n).push.apply(t,o(e))}return i.call(this,n)}}t.default=void 0;var g=n(1),_=i(n(11)),m=i(n(49)),v=n(7),y=n(3),b=n(6),x=n(4),w=m.default.add,C="DEVEXTREME_XHR_ERROR_UNLOAD",k=function(e){return[e[0],e.length<3?"=":String(e[1]).toLowerCase(),e.length<2||e[e.length-1]]},S=function(e){return Array.isArray(e)||(e=[e]),(0,y.map)(e,function(e){var t={selector:(0,g.isFunction)(e)||"string"==typeof e?e:e.getter||e.field||e.selector,desc:!(!e.desc&&"d"!==String(e.dir).charAt(0).toLowerCase())};return e.compare&&(t.compare=e.compare),t})},D=function(){var e,t={timeout:"Network connection timeout",error:"Unspecified network error",parsererror:"Unexpected server response"},n=function(e){var n=t[e];return n?n:e};return w(function(){var t=(0,v.getWindow)();_.default.listen(t,"beforeunload",function(){e=!0})}),function(t,i){return e?C:t.status<400?n(i):t.statusText}}(),I={count:{seed:0,step:function(e){return 1+e}},sum:{seed:0,step:function(e,t){return e+t}},min:{step:function(e,t){return t<e?t:e}},max:{step:function(e,t){return t>e?t:e}},avg:{seed:[0,0],step:function(e,t){return[e[0]+t,e[1]+1]},finalize:function(e){return e[1]?e[0]/e[1]:NaN}}},T=function(){var e,t=0,n=function(){0===t&&(e=new b.Deferred),t++},i=function(){t--,t<1&&e.resolve()},o=function(){var n=0===t?(new b.Deferred).resolve():e;return n.promise()},a=function(){t=0,e&&e.resolve()};return{obtain:n,release:i,promise:o,reset:a}}(),O=function(e,t,n){if(Array.isArray(e)){for(var i,o=(0,y.map)(t,function(e,t){return t}),a=0;a<o.length;a++)if(i=o[a],!(0,x.equalByValue)(t[i],n[i],0,!1))return!1;return!0}return(0,x.equalByValue)(t,n,0,!1)},E="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",A=function(e){function t(e){return E.charAt(e)}Array.isArray(e)||(e=h(String(e)));for(var n="",i=0;i<e.length;i+=3){var o=e[i],a=e[i+1],r=e[i+2];n+=(0,y.map)([o>>2,(3&o)<<4|a>>4,isNaN(a)?64:(15&a)<<2|r>>6,isNaN(r)?64:63&r],t).join("")}return n},P=function(e){return"!"===e[0]&&Array.isArray(e[1])},R=function(e){return"and"===e||"or"===e},M=function(e){var t=e[0],n=e[1];return!!Array.isArray(t)||!(!(0,g.isFunction)(t)||!(Array.isArray(n)||(0,g.isFunction)(n)||R(n)))},B=function(){var e=new b.Deferred;return e.resolve.apply(e,arguments).promise()},V=function(){var e=new b.Deferred;return e.reject.apply(e,arguments).promise()},F={XHR_ERROR_UNLOAD:C,normalizeBinaryCriterion:k,normalizeSortingInfo:S,errorMessageFromXhr:D,aggregators:I,keysEqual:O,throttleChanges:p,trivialPromise:B,rejectedPromise:V,isDisjunctiveOperator:c,isConjunctiveOperator:d,processRequestResultLock:T,isUnaryOperation:P,isGroupCriterion:M,base64_encode:A},L=F;t.default=L,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.isEmpty=t.replaceAll=t.format=t.quadToObject=t.encodeHtml=void 0;var o=n(1),a=function(){var e=[new RegExp("&","g"),new RegExp('"',"g"),new RegExp("'","g"),new RegExp("<","g"),new RegExp(">","g")];return function(t){return String(t).replace(e[0],"&amp;").replace(e[1],"&quot;").replace(e[2],"&#39;").replace(e[3],"&lt;").replace(e[4],"&gt;")}}();t.encodeHtml=a;var r=function(e){switch(i(e)){case"string":return e.split(/\s+/,4);case"object":return[e.x||e.h||e.left,e.y||e.v||e.top,e.x||e.h||e.right,e.y||e.v||e.bottom];case"number":return[e];default:return e}},s=function(e){var t=r(e),n=parseInt(t&&t[0],10),i=parseInt(t&&t[1],10),o=parseInt(t&&t[2],10),a=parseInt(t&&t[3],10);return isFinite(n)||(n=0),isFinite(i)||(i=n),isFinite(o)||(o=n),isFinite(a)||(a=i),{top:i,right:o,bottom:a,left:n}};t.quadToObject=s;var l=function(){var e,t,n,i=arguments[0],a=[].slice.call(arguments).slice(1);if((0,o.isFunction)(i))return i.apply(this,a);for(var r=0;r<a.length;r++)t=new RegExp("\\{"+r+"\\}","gm"),n=a[r],"string"===(0,o.type)(n)&&n.indexOf("$")>=0&&(e="$".replace("$","$$").length,n=n.replace("$",1===e?"$$$$":"$$")),i=i.replace(t,n);return i};t.format=l;var u=function(){var e=function(e){return(e+"").replace(/([+*?.[^\]$(){}><|=!:])/g,"\\$1")};return function(t,n,i){return t.replace(new RegExp("("+e(n)+")","gi"),i)}}();t.replaceAll=u;var c=function(){var e=/\s/g;return function(t){return!t||!t.replace(e,"")}}();t.isEmpty=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(){var e=this,t=e.element,n=e.config;if(h(t,n.from),h(t,n.to),e.configurator.setup(t,n),t.data(M,e),ue.off&&(n.duration=0,n.delay=0),e.strategy.initAnimation(t,n),n.start){var i=(0,v.getPublicElement)(t);n.start.apply(this,[i,n])}}function r(e,t){var n=s(e);l(e,n),n.push(t),u(e)||c(e,n)}function s(e){return e.data(B)||[]}function l(e,t){e.data(B,t)}function u(e){return!!e.data(M)}function c(e,t){if(t=s(e),t.length){var n=t.shift();0===t.length&&se(e),d(n).done(function(){u(e)||c(e)})}}function d(e){return e.setup(),ue.off||e.isSynchronous?e.start():e.startTimeout=setTimeout(function(){e.start()}),e.deferred.promise()}function h(e,t){if(t&&t.position){var n=(0,p.default)(A),i=0,o=0,a=D.default.calculate(e,t.position),r=e.offset(),s=e.position();s.top>r.top&&(o=n.scrollTop()),s.left>r.left&&(i=n.scrollLeft()),(0,y.extend)(t,{left:a.h.location-r.left+s.left-i,top:a.v.location-r.top+s.top-o}),delete t.position}}function f(e,t){(0,x.each)(t,function(t,n){try{e.css(t,(0,b.isFunction)(n)?n():n)}catch(e){}})}t.default=void 0;var p=i(n(2)),g=n(7),_=i(n(5)),m=i(n(22)),v=n(18),y=n(0),b=n(1),x=n(3),w=n(30),C=n(253),k=n(120),S=n(46),D=i(n(84)),I=i(n(138)),T=n(8),O=n(6),E=n(4),A=(0,g.getWindow)(),P=(0,T.addNamespace)(I.default,"dxFX"),R=/^([+-])=(.*)/i,M="dxAnimData",B="dxAnimQueue",V="transform",F={initAnimation:function(e,t){e.css({transitionProperty:"none"}),"string"==typeof t.from?e.addClass(t.from):f(e,t.from);var n=this,i=new O.Deferred,o=t.cleanupWhen;t.transitionAnimation={deferred:i,finish:function(){n._finishTransition(e),o?(0,O.when)(i,o).always(function(){n._cleanup(e,t)}):n._cleanup(e,t),i.resolveWith(e,[t,e])}},this._completeAnimationCallback(e,t).done(function(){t.transitionAnimation.finish()}).fail(function(){i.rejectWith(e,[t,e])}),t.duration||t.transitionAnimation.finish(),e.css("transform")},animate:function(e,t){return this._startAnimation(e,t),t.transitionAnimation.deferred.promise();
},_completeAnimationCallback:function(e,t){var n,i=this,o=Date.now()+t.delay,a=new O.Deferred,r=new O.Deferred,s=new O.Deferred,l=(0,S.transitionEndEventName)()+".dxFX";t.transitionAnimation.cleanup=function(){clearTimeout(n),clearTimeout(u),_.default.off(e,l),_.default.off(e,P)},_.default.one(e,l,function(){Date.now()-o>=t.duration&&r.reject()}),_.default.off(e,P),_.default.on(e,P,function(){i.stop(e,t),a.reject()});var u=setTimeout(function(){n=setTimeout(function(){s.reject()},t.duration+t.delay+ue._simulatedTransitionEndDelay),(0,O.when)(r,s).fail(function(){a.resolve()}.bind(this))});return a.promise()},_startAnimation:function(e,t){e.css({transitionProperty:"all",transitionDelay:t.delay+"ms",transitionDuration:t.duration+"ms",transitionTimingFunction:t.easing}),"string"==typeof t.to?e[0].className+=" "+t.to:t.to&&f(e,t.to)},_finishTransition:function(e){e.css("transition","none")},_cleanup:function(e,t){t.transitionAnimation.cleanup(),"string"==typeof t.from&&(e.removeClass(t.from),e.removeClass(t.to))},stop:function(e,t,n){t&&(n?t.transitionAnimation.finish():((0,b.isPlainObject)(t.to)&&(0,x.each)(t.to,function(t){e.css(t,e.css(t))}),this._finishTransition(e),this._cleanup(e,t)))}},L={initAnimation:function(e,t){f(e,t.from)},animate:function(e,t){var n=new O.Deferred,i=this;return t?((0,x.each)(t.to,function(n){void 0===t.from[n]&&(t.from[n]=i._normalizeValue(e.css(n)))}),t.to[V]&&(t.from[V]=i._parseTransform(t.from[V]),t.to[V]=i._parseTransform(t.to[V])),t.frameAnimation={to:t.to,from:t.from,currentValue:t.from,easing:(0,C.convertTransitionTimingFuncToEasing)(t.easing),duration:t.duration,startTime:(new Date).valueOf(),finish:function(){this.currentValue=this.to,this.draw(),(0,k.cancelAnimationFrame)(t.frameAnimation.animationFrameId),n.resolve()},draw:function(){if(t.draw)return void t.draw(this.currentValue);var n=(0,y.extend)({},this.currentValue);n[V]&&(n[V]=(0,x.map)(n[V],function(e,t){return"translate"===t?(0,w.getTranslateCss)(e):"scale"===t?"scale("+e+")":"rotate"===t.substr(0,t.length-1)?t+"("+e+"deg)":void 0}).join(" ")),e.css(n)}},t.delay?(t.frameAnimation.startTime+=t.delay,t.frameAnimation.delayTimeout=setTimeout(function(){i._startAnimation(e,t)},t.delay)):i._startAnimation(e,t),n.promise()):n.reject().promise()},_startAnimation:function(e,t){_.default.off(e,P),_.default.on(e,P,function(){t.frameAnimation&&(0,k.cancelAnimationFrame)(t.frameAnimation.animationFrameId)}),this._animationStep(e,t)},_parseTransform:function(e){var t={};return(0,x.each)(e.match(/(\w|\d)+\([^)]*\)\s*/g),function(e,n){var i=(0,w.parseTranslate)(n),o=n.match(/scale\((.+?)\)/),a=n.match(/(rotate.)\((.+)deg\)/);i&&(t.translate=i),o&&o[1]&&(t.scale=parseFloat(o[1])),a&&a[1]&&(t[a[1]]=parseFloat(a[2]))}),t},stop:function(e,t,n){var i=t&&t.frameAnimation;i&&((0,k.cancelAnimationFrame)(i.animationFrameId),clearTimeout(i.delayTimeout),n&&i.finish(),delete t.frameAnimation)},_animationStep:function(e,t){var n=t&&t.frameAnimation;if(n){var i=(new Date).valueOf();if(i>=n.startTime+n.duration)return void n.finish();n.currentValue=this._calcStepValue(n,i-n.startTime),n.draw();var o=this;n.animationFrameId=(0,k.requestAnimationFrame)(function(){o._animationStep(e,t)})}},_calcStepValue:function(e,t){var n=function n(i,a){var r=Array.isArray(a)?[]:{},s=function(n){var o=t/e.duration,r=t,s=1*i[n],l=a[n]-i[n],u=e.duration;return(0,C.getEasing)(e.easing)(o,r,s,l,u)};return(0,x.each)(a,function(e,t){return"string"==typeof t&&parseFloat(t,10)===!1||void(r[e]="object"===o(t)?n(i[e],t):s(e))}),r};return n(e.from,e.to)},_normalizeValue:function(e){var t=parseFloat(e,10);return t===!1?e:t}},H={initAnimation:function(){},animate:function(){return(new O.Deferred).resolve().promise()},stop:E.noop,isSynchronous:!0},z=function(e){e=e||{};var t={transition:(0,S.transition)()?F:L,frame:L,noAnimation:H},n=e.strategy||"transition";return"css"!==e.type||(0,S.transition)()||(n="noAnimation"),t[n]},N=function(e,t,n,i){(0,x.each)(["from","to"],function(){if(!n(e[this]))throw m.default.Error("E0010",t,this,i)})},j=function(e,t){return N(e,t,function(e){return(0,b.isPlainObject)(e)},"a plain object")},W=function(e,t){return N(e,t,function(e){return"string"==typeof e},"a string")},G={setup:function(){}},q={validateConfig:function(e){W(e,"css")},setup:function(){}},$={top:{my:"bottom center",at:"top center"},bottom:{my:"top center",at:"bottom center"},right:{my:"left center",at:"right center"},left:{my:"right center",at:"left center"}},K={validateConfig:function(e){j(e,"slide")},setup:function(e,t){var n=(0,w.locate)(e);if("slide"!==t.type){var i="slideIn"===t.type?t.from:t.to;i.position=(0,y.extend)({of:A},$[t.direction]),h(e,i)}this._setUpConfig(n,t.from),this._setUpConfig(n,t.to),(0,w.clearCache)(e)},_setUpConfig:function(e,t){t.left="left"in t?t.left:"+=0",t.top="top"in t?t.top:"+=0",this._initNewPosition(e,t)},_initNewPosition:function(e,t){var n={left:t.left,top:t.top};delete t.left,delete t.top;var i=this._getRelativeValue(n.left);void 0!==i?n.left=i+e.left:t.left=0,i=this._getRelativeValue(n.top),void 0!==i?n.top=i+e.top:t.top=0,t[V]=(0,w.getTranslateCss)({x:n.left,y:n.top})},_getRelativeValue:function(e){var t;if("string"==typeof e&&(t=R.exec(e)))return parseInt(t[1]+"1")*t[2]}},U={setup:function(e,t){var n,i=t.from,o=(0,b.isPlainObject)(i)?t.skipElementInitialStyles?0:e.css("opacity"):String(i);switch(t.type){case"fadeIn":n=1;break;case"fadeOut":n=0;break;default:n=String(t.to)}t.from={visibility:"visible",opacity:o},t.to={opacity:n}}},Y={validateConfig:function(e){j(e,"pop")},setup:function(e,t){var n=t.from,i=t.to,o="opacity"in n?n.opacity:e.css("opacity"),a="opacity"in i?i.opacity:1,r="scale"in n?n.scale:0,s="scale"in i?i.scale:1;t.from={opacity:o};var l=(0,w.getTranslate)(e);t.from[V]=this._getCssTransform(l,r),t.to={opacity:a},t.to[V]=this._getCssTransform(l,s)},_getCssTransform:function(e,t){return(0,w.getTranslateCss)(e)+"scale("+t+")"}},Z={custom:G,slide:K,slideIn:K,slideOut:K,fade:U,fadeIn:U,fadeOut:U,pop:Y,css:q},X=function(e){var t=Z[e.type];if(!t)throw m.default.Error("E0011",e.type);return t},Q={type:"custom",from:{},to:{},duration:400,start:E.noop,complete:E.noop,easing:"ease",delay:0},J={duration:400,easing:"ease",delay:0},ee=function(e){var t=e.element,n=e.config;if(t.removeData(M),n.complete){var i=(0,v.getPublicElement)(t);n.complete.apply(this,[i,n])}e.deferred.resolveWith(this,[t,n])},te=function(){var e=this,t=e.element,n=e.config;return e.isStarted=!0,e.strategy.animate(t,n).done(function(){ee(e)}).fail(function(){e.deferred.rejectWith(this,[t,n])})},ne=function(e){var t=this,n=t.element,i=t.config;clearTimeout(t.startTimeout),t.isStarted||t.start(),t.strategy.stop(n,i,e)},ie=(0,T.addNamespace)(I.default,"dxFXStartAnimation"),oe=function(e){_.default.off(e.element,ie),_.default.on(e.element,ie,function(){ue.stop(e.element)}),e.deferred.always(function(){_.default.off(e.element,ie)})},ae=function(e,t){var n="css"===t.type?J:Q,i=(0,y.extend)(!0,{},n,t),o=X(i),r=z(i),s={element:(0,p.default)(e),config:i,configurator:o,strategy:r,isSynchronous:r.isSynchronous,setup:a,start:te,stop:ne,deferred:new O.Deferred};return(0,b.isFunction)(o.validateConfig)&&o.validateConfig(i),oe(s),s},re=function(e,t){var n=(0,p.default)(e);if(!n.length)return(new O.Deferred).resolve().promise();var i=ae(n,t);return r(n,i),i.deferred.promise()},se=function(e){e.removeData(B)},le=function(e,t){var n=(0,p.default)(e),i=s(n);(0,x.each)(i,function(e,t){t.config.delay=0,t.config.duration=0,t.isSynchronous=!0}),u(n)||c(n,i);var o=n.data(M);o&&o.stop(t),n.removeData(M),se(n)},ue={off:!1,animationTypes:Z,animate:re,createAnimation:ae,isAnimating:u,stop:le,_simulatedTransitionEndDelay:100},ce=ue;t.default=ce,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(15)),s=i(n(27)),l=n(4),u=n(1),c=n(12),d=n(3),h=i(n(17)),f=i(n(10)),p=n(7),g="dxDataGrid",_=r.default.inherit({_endUpdateCore:function(){},ctor:function(e){var t=this;t._updateLockCount=0,t.component=e,t._actions={},t._actionConfigs={},(0,d.each)(this.callbackNames()||[],function(e,n){var i=t.callbackFlags(n)||{};i.unique=!0,i.syncStrategy=!0,t[this]=(0,s.default)(i)})},init:function(){},callbackNames:function(){},callbackFlags:function(){},publicMethods:function(){},beginUpdate:function(){this._updateLockCount++},endUpdate:function(){this._updateLockCount>0&&(this._updateLockCount--,this._updateLockCount||this._endUpdateCore())},option:function(e){var t=this.component,n=t._optionCache;return 1===arguments.length&&n?(e in n||(n[e]=t.option(e)),n[e]):t.option.apply(t,arguments)},_silentOption:function(e,t){var n=this.component,i=n._optionCache;return i&&(i[e]=t),n._setOptionWithoutOptionChange(e,t)},localize:function(e){var t=this.component._optionCache;return t?(e in t||(t[e]=f.default.format(e)),t[e]):f.default.format(e)},on:function(){return this.component.on.apply(this.component,arguments)},off:function(){return this.component.off.apply(this.component,arguments)},optionChanged:function(e){e.name in this._actions&&(this.createAction(e.name,this._actionConfigs[e.name]),e.handled=!0)},getAction:function(e){return this._actions[e]},setAria:function(e,t,n){var i=n.get(0),o="role"!==e&&"id"!==e?"aria-":"";i.setAttribute?i.setAttribute(o+e,t):n.attr(o+e,t)},_createComponent:function(){return this.component._createComponent.apply(this.component,arguments)},getController:function(e){return this.component._controllers[e]},createAction:function(e,t){if((0,u.isFunction)(e)){var n=this.component._createAction(e.bind(this),t);return function(e){n({event:e})}}this._actions[e]=this.component._createActionByOption(e,t),this._actionConfigs[e]=t},executeAction:function(e,t){var n=this._actions[e];return n&&n(t)},dispose:function(){var e=this;(0,d.each)(e.callbackNames()||[],function(){e[this].empty()})},addWidgetPrefix:function(e){var t=this.component.NAME;return"dx-"+t.slice(2).toLowerCase()+(e?"-"+e:"")},getWidgetContainerClass:function(){var e=this.component.NAME===g?null:"container";return this.addWidgetPrefix(e)}}),m=_,v=m.inherit({getView:function(e){return this.component._views[e]},getViews:function(){return this.component._views}}),y=_.inherit({_isReady:function(){return this.component.isReady()},_endUpdateCore:function(){this.callBase(),!this._isReady()&&this._requireReady&&(this._requireRender=!1,this.component._requireResize=!1),this._requireRender&&(this._requireRender=!1,this.render(this._$parent))},_invalidate:function(e,t){this._requireRender=!0,this.component._requireResize=(0,p.hasWindow)()&&(this.component._requireResize||e),this._requireReady=this._requireReady||t},_renderCore:function(){},_resizeCore:function(){},_afterRender:function(){},_parentElement:function(){return this._$parent},ctor:function(e){this.callBase(e),this.renderCompleted=(0,s.default)(),this.resizeCompleted=(0,s.default)()},element:function(){return this._$element},getElementHeight:function(){var e=this.element();if(!e)return 0;var t=parseFloat(e.css("marginTop"))||0,n=parseFloat(e.css("marginBottom"))||0,i=e.get(0).offsetHeight;return i+t+n},isVisible:function(){return!0},getTemplate:function(e){return this.component._getTemplate(e)},render:function(e,t){var n=this._$element,i=this.isVisible();(n||e)&&(this._requireReady=!1,n||(n=this._$element=(0,o.default)("<div>").appendTo(e),this._$parent=e),n.toggleClass("dx-hidden",!i),i&&(this.component._optionCache={},this._renderCore(t),this.component._optionCache=void 0,this._afterRender(e),this.renderCompleted.fire(t)))},resize:function(){this.isResizing=!0,this._resizeCore(),this.resizeCompleted.fire(),this.isResizing=!1},focus:function(){a.default.trigger(this.element(),"focus")}}),b=1e6,x=function(e,t){var n=t.modules,i=t.modulesOrder,o=t.controllerTypes||{},a=t.viewTypes||{};t.controllerTypes||(i&&n.sort(function(e,t){var n=(0,c.inArray)(e.name,i),o=(0,c.inArray)(t.name,i);return n<0&&(n=b),o<0&&(o=b),n-o}),(0,d.each)(n,function(){var e=this.controllers,t=this.name,n=this.views;e&&(0,d.each)(e,function(e,n){if(o[e])throw h.default.Error("E1001",t,e);if(!(n&&n.subclassOf&&n.subclassOf(m)))throw n.subclassOf(m),h.default.Error("E1002",t,e);o[e]=n}),n&&(0,d.each)(n,function(e,n){if(a[e])throw h.default.Error("E1003",t,e);if(!(n&&n.subclassOf&&n.subclassOf(y)))throw h.default.Error("E1004",t,e);a[e]=n})}),(0,d.each)(n,function(){var e=this.extenders;e&&(e.controllers&&(0,d.each)(e.controllers,function(e,t){o[e]&&(o[e]=o[e].inherit(t))}),e.views&&(0,d.each)(e.views,function(e,t){a[e]&&(a[e]=a[e].inherit(t))}))}),t.controllerTypes=o,t.viewTypes=a);var r=function(e,t,n){var i=n.publicMethods();i&&(0,d.each)(i,function(i,o){if(!n[o])throw h.default.Error("E1006",t,o);if(e[o])throw h.default.Error("E1005",o);e[o]=function(){return n[o].apply(n,arguments)}})},s=function(t){var n={};return(0,d.each)(t,function(t,i){var o=new i(e);o.name=t,r(e,t,o),n[t]=o}),n};e._controllers=s(o),e._views=s(a)},w=function(e,t,n){n=n||[],e._controllers&&(0,d.each)(e._controllers,function(){this[t]&&this[t].apply(this,n)}),e._views&&(0,d.each)(e._views,function(){this[t]&&this[t].apply(this,n)})},C={modules:[],View:y,ViewController:v,Controller:m,registerModule:function(e,t){for(var n=this.modules,i=0;i<n.length;i++)if(n[i].name===e)return;t.name=e,n.push(t),delete this.controllerTypes,delete this.viewTypes},registerModulesOrder:function(e){this.modulesOrder=e},unregisterModule:function(e){this.modules=(0,l.grep)(this.modules,function(t){return t.name!==e}),delete this.controllerTypes,delete this.viewTypes},processModules:x,callModuleItemsMethod:w};t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return u(e)||l(e,t)||r(e,t)||a()}function a(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function l(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function u(e){if(Array.isArray(e))return e}t.default=void 0;var c=i(n(54)),d=n(12),h=n(4),f=n(3),p=n(1),g=n(268),_=i(n(28)),m=i(n(22)),v=n(240),y=i(n(107)),b=i(n(336)),x="undefined"!=typeof Intl,w=4,C=10,k=["currency","fixedpoint","exponential","percent","decimal"],S={1:"K",2:"M",3:"B",4:"T"},D={largenumber:"auto",thousands:1,millions:2,billions:3,trillions:4},I=(0,c.default)({engine:function(){return"base"},numericFormats:k,defaultLargeNumberFormatPostfixes:S,_parseNumberFormatString:function(e){var t={};if(e&&"string"==typeof e){var n=e.toLowerCase().split(" ");return(0,f.each)(n,function(e,n){(0,d.inArray)(n,k)>-1?t.formatType=n:n in D&&(t.power=D[n])}),t.power&&!t.formatType&&(t.formatType="fixedpoint"),t.formatType?t:void 0}},_calculateNumberPower:function(e,t,n,i){var o=Math.abs(e),a=0;if(o>1)for(;o&&o>=t&&(void 0===i||a<i);)a++,o/=t;else if(o>0&&o<1)for(;o<1&&(void 0===n||a>n);)a--,o*=t;return a},_getNumberByPower:function(e,t,n){for(var i=e;t>0;)i/=n,t--;for(;t<0;)i*=n,t++;return i},_formatNumber:function(e,t,n){"auto"===t.power&&(t.power=this._calculateNumberPower(e,1e3,0,w)),t.power&&(e=this._getNumberByPower(e,t.power,1e3));var i=this.defaultLargeNumberFormatPostfixes[t.power]||"",o=this._formatNumberCore(e,t.formatType,n);return o=o.replace(/(\d|.$)(\D*)$/,"$1"+i+"$2")},_formatNumberExponential:function(e,t){var n=this._calculateNumberPower(e,C),i=this._getNumberByPower(e,n,C);void 0===t.precision&&(t.precision=1),i.toFixed(t.precision||0)>=C&&(n++,i/=C);var o=(n>=0?"+":"")+n.toString();return this._formatNumberCore(i,"fixedpoint",t)+"E"+o},_addZeroes:function(e,t){var n=Math.pow(10,t),i=e<0?"-":"";e=(Math.abs(e)*n>>>0)/n;for(var o=e.toString();o.length<t;)o="0"+o;return i+o},_addGroupSeparators:function(e){var t=e.toString().split(".");return t[0].replace(/\B(?=(\d{3})+(?!\d))/g,(0,_.default)().thousandsSeparator)+(t[1]?(0,_.default)().decimalSeparator+t[1]:"")},_formatNumberCore:function(e,t,n){return"exponential"===t?this._formatNumberExponential(e,n):("decimal"!==t&&null!==n.precision&&(n.precision=n.precision||0),"percent"===t&&(e=100*e),void 0!==n.precision&&(e="decimal"===t?this._addZeroes(e,n.precision):null===n.precision?e.toPrecision():(0,v.toFixed)(e,n.precision)),e="decimal"!==t?this._addGroupSeparators(e):e.toString().replace(".",(0,_.default)().decimalSeparator),"percent"===t&&(e+="%"),e)},_normalizeFormat:function(e){return e?"function"==typeof e?e:((0,p.isPlainObject)(e)||(e={type:e}),e):{}},_getSeparators:function(){return{decimalSeparator:this.getDecimalSeparator(),thousandsSeparator:this.getThousandsSeparator()}},getThousandsSeparator:function(){return this.format(1e4,"fixedPoint")[2]},getDecimalSeparator:function(){return this.format(1.2,{type:"fixedPoint",precision:1})[1]},convertDigits:function(e,t){var n=this.format(90,"decimal");if("string"!=typeof e||"0"===n[1])return e;var i=t?n[1]:"0",o=t?"0":n[1],a=t?n[0]:"9",r=new RegExp("["+i+"-"+a+"]","g");return e.replace(r,function(e){return String.fromCharCode(e.charCodeAt(0)+(o.charCodeAt(0)-i.charCodeAt(0)))})},getNegativeEtalonRegExp:function(e){var t=this._getSeparators(),n=new RegExp("[0-9"+(0,h.escapeRegExp)(t.decimalSeparator+t.thousandsSeparator)+"]+","g"),i=["\\","(",")","[","]","*","+","$","^","?","|","{","}"],o=this.format(-1,e).replace(n,"1");return i.forEach(function(e){o=o.replace(e,"\\".concat(e))}),o=o.replace(" ","\\s"),o=o.replace("1",".+"),new RegExp(o,"g")},getSign:function(e,t){if("-"===e.replace(/[^0-9-]/g,"").charAt(0))return-1;if(!t)return 1;var n=this.getNegativeEtalonRegExp(t);return e.match(n)?-1:1},format:function(e,t){if("number"!=typeof e)return e;if("number"==typeof t)return e;if(t=t&&t.formatter||t,"function"==typeof t)return t(e);t=this._normalizeFormat(t),t.type||(t.type="decimal");var n=this._parseNumberFormatString(t.type);if(!n){var i=this._getSeparators();return i.unlimitedIntegerDigits=t.unlimitedIntegerDigits,this.convertDigits((0,g.getFormatter)(t.type,i)(e))}return this._formatNumber(e,n,t)},parse:function(e,t){if(e){if(t&&t.parser)return t.parser(e);e=this.convertDigits(e,!0),t&&"string"!=typeof t&&m.default.log("W0011");var n=this.getDecimalSeparator(),i=new RegExp("[^0-9"+(0,h.escapeRegExp)(n)+"]","g"),o=e.replace(i,"").replace(n,".").replace(/\.$/g,"");if("."===o||""===o)return null;if(this._calcSignificantDigits(o)>15)return NaN;var a=+o*this.getSign(e,t);t=this._normalizeFormat(t);var r=this._parseNumberFormatString(t.type),s=null===r||void 0===r?void 0:r.power;if(s){if("auto"===s){var l=e.match(/\d(K|M|B|T)/);l&&(s=(0,d.find)(Object.keys(S),function(e){return S[e]===l[1]}))}a*=Math.pow(10,3*s)}return"percent"===(null===r||void 0===r?void 0:r.formatType)&&(a/=100),a}},_calcSignificantDigits:function(e){var t=e.split("."),n=o(t,2),i=n[0],a=n[1],r=function(e){for(var t=-1,n=0;n<e.length;n++)if("0"!==e[n]){t=n;break}return t>-1?e.length-t:0},s=0;return i&&(s+=r(i.split(""))),a&&(s+=r(a.split("").reverse())),s}});I.inject(y.default),x&&I.inject(b.default);var T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}Object.defineProperty(t,"stylePropPrefix",{enumerable:!0,get:function(){return c.stylePropPrefix}}),Object.defineProperty(t,"styleProp",{enumerable:!0,get:function(){return c.styleProp}}),t.nativeScrolling=t.animation=t.transitionEndEventName=t.transition=t.touch=t.inputType=t.supportProp=t.pointerEvents=t.touchEvents=void 0;var o=n(12),a=i(n(11)),r=n(4),s=i(n(64)),l=n(7),u=i(n(13)),c=n(97),d=(0,l.getNavigator)(),h=d.maxTouchPoints,f=d.msMaxTouchPoints,p=d.pointerEnabled,g={webkitTransition:"webkitTransitionEnd",MozTransition:"transitionend",OTransition:"oTransitionEnd",msTransition:"MsTransitionEnd",transition:"transitionend"},_=function(e){return!!(0,c.styleProp)(e)};t.supportProp=_;var m=function(){var e=u.default.real(),t=e.platform,n=e.version,i=e.mac,a=n&&n[0]<4&&"android"===t,r=!a&&(0,o.inArray)(t,["ios","android"])>-1||i;return r},v=function(e){if("text"===e)return!0;var t=a.default.createElement("input");try{return t.setAttribute("type",e),t.value="wrongValue",!t.value}catch(e){return!1}};t.inputType=v;var y=function(e,t){return(e("ontouchstart")||!!t)&&!e("callPhantom")},b=function(e,t){var n=(0,r.ensureDefined)(t,!0),i=(0,r.ensureDefined)(t,!1);return e("PointerEvent")&&n||i},x=y(l.hasProperty,h);t.touchEvents=x;var w=b(l.hasProperty,p);t.pointerEvents=w;var C=!!h||!!f,k=x||w&&C;t.touch=k;var S=(0,s.default)(function(){return _("transition")});t.transition=S;var D=(0,s.default)(function(){return g[(0,c.styleProp)("transition")]});t.transitionEndEventName=D;var I=(0,s.default)(function(){return _("animation")});t.animation=I;var T=m();t.nativeScrolling=T},function(e,t,n){t.default=void 0;var i=n(251),o=function(){var e=Array.isArray(arguments[0])?"array":"remote";return i.queryImpl[e].apply(this,arguments)},a=o;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(30),a=i(n(9)),r=i(n(13)),s=n(18),l=i(n(2)),u=n(93),c=n(12),d=i(n(23)),h=n(4),f=n(0),p=n(40),g=n(3),_=n(184),m=n(24),v=n(1),y=n(60),b=n(7),x=n(55),w=i(n(10)),C=i(n(38)),k=i(n(68)),S=n(36);n(418);var D=(0,b.getWindow)(),I="dx-popup",T="dx-popup-wrapper",O="dx-popup-fullscreen",E="dx-popup-fullscreen-width",A="dx-popup-normal",P="dx-popup-content",R="dx-popup-draggable",M="dx-popup-title",B="dx-closebutton",V="dx-popup-bottom",F="dx-template-wrapper",L="dx-popup-flex-height",H="dx-popup-inherit-height",z=["cancel","clear","done"],N="default",j="normal",W="text",G="contained",q=d.default.msie&&11===parseInt(d.default.version),$=d.default.safari&&(0,y.compare)(d.default.version,[11])<0,K={"static":"",inherit:H,flex:L},U=function(e){var t=r.default.current(),n=t.platform,i="bottom",o="before";if("ios"===n)switch(e){case"cancel":i="top";break;case"clear":i="top",o="after";break;case"done":o="after"}else if("android"===n&&t.version&&parseInt(t.version[0])>4)switch(e){case"cancel":o="after";break;case"done":o="after"}else"android"===n&&(o="center");return{toolbar:i,location:o}},Y=k.default.inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{fullScreen:!1,title:"",showTitle:!0,titleTemplate:"title",onTitleRendered:null,dragEnabled:!1,toolbarItems:[],showCloseButton:!1,bottomTemplate:"bottom",useDefaultToolbarButtons:!1,useFlatToolbarButtons:!1,autoResizeEnabled:!0})},_defaultOptionsRules:function(){var e=(0,S.current)();return this.callBase().concat([{device:{platform:"ios"},options:{animation:this._iosAnimation}},{device:{platform:"android"},options:{animation:this._androidAnimation}},{device:{platform:"generic"},options:{showCloseButton:!0}},{device:function(e){return"desktop"===r.default.real().deviceType&&"generic"===e.platform},options:{dragEnabled:!0}},{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,S.isMaterial)(e)},options:{useDefaultToolbarButtons:!0,useFlatToolbarButtons:!0}}])},_iosAnimation:{show:{type:"slide",duration:400,from:{position:{my:"top",at:"bottom"}},to:{position:{my:"center",at:"center"}}},hide:{type:"slide",duration:400,from:{opacity:1,position:{my:"center",at:"center"}},to:{opacity:1,position:{my:"top",at:"bottom"}}}},_androidAnimation:function(){var e={show:{type:"slide",duration:300,from:{top:"30%",opacity:0},to:{top:0,opacity:1}},hide:{type:"slide",duration:300,from:{top:0,opacity:1},to:{top:"30%",opacity:0}}},t={show:{type:"fade",duration:400,from:0,to:1},hide:{type:"fade",duration:400,from:1,to:0}};return this.option("fullScreen")?e:t},_init:function(){this.callBase(),this.$element().addClass(I),this._wrapper().addClass(T),this._$popupContent=this._$content.wrapInner((0,l.default)("<div>").addClass(P)).children().eq(0)},_render:function(){var e=this.option("fullScreen");this._toggleFullScreenClass(e),this.callBase()},_toggleFullScreenClass:function(e){this._$content.toggleClass(O,e).toggleClass(A,!e)},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({title:new u.EmptyTemplate,bottom:new u.EmptyTemplate})},_renderContentImpl:function(){this._renderTitle(),this.callBase(),this._renderBottom()},_renderTitle:function(){var e=this._getToolbarItems("top"),t=this.option("title"),n=this.option("showTitle");if(n&&t&&e.unshift({location:r.default.current().ios?"center":"before",text:t}),n||e.length>0){this._$title&&this._$title.remove();var i=(0,l.default)("<div>").addClass(M).insertBefore(this.$content());this._$title=this._renderTemplateByType("titleTemplate",e,i).addClass(M),this._renderDrag(),this._executeTitleRenderAction(this._$title)}else this._$title&&this._$title.detach()},_renderTemplateByType:function(e,t,n,i){var o=this._getTemplateByOption(e),a=o instanceof u.EmptyTemplate;if(a){var r=(0,f.extend)({},this.option("integrationOptions"),{skipTemplates:["content","title"]}),c=(0,f.extend)(i,{items:t,rtlEnabled:this.option("rtlEnabled"),useDefaultButtons:this.option("useDefaultToolbarButtons"),useFlatButtons:this.option("useFlatToolbarButtons"),integrationOptions:r});this._getTemplate("dx-polymorph-widget").render({container:n,model:{widget:"dxToolbarBase",options:c}});var d=n.children("div");return n.replaceWith(d),d}var h=(0,l.default)(o.render({container:(0,s.getPublicElement)(n)}));return h.hasClass(F)&&(n.replaceWith(h),n=h),n},_executeTitleRenderAction:function(e){this._getTitleRenderAction()({titleElement:(0,s.getPublicElement)(e)})},_getTitleRenderAction:function(){return this._titleRenderAction||this._createTitleRenderAction()},_createTitleRenderAction:function(){return this._titleRenderAction=this._createActionByOption("onTitleRendered",{element:this.element(),excludeValidators:["disabled","readOnly"]})},_getCloseButton:function(){return{toolbar:"top",location:"after",template:this._getCloseButtonRenderer()}},_getCloseButtonRenderer:function(){var e=this;return function(t,n,i){var o=(0,l.default)("<div>").addClass(B);e._createComponent(o,C.default,{icon:"close",onClick:e._createToolbarItemAction(void 0),integrationOptions:{}}),(0,l.default)(i).append(o)}},_getToolbarItems:function(e){var t=this,n=this.option("toolbarItems"),i=[];this._toolbarItemClasses=[];var o=r.default.current().platform,a=0;return(0,g.each)(n,function(n,r){var s=(0,v.isDefined)(r.shortcut),l=s?U(r.shortcut):r;if(s&&"ios"===o&&a<2&&(l.toolbar="top",a++),l.toolbar=r.toolbar||l.toolbar||"top",l&&l.toolbar===e){s&&(0,f.extend)(l,{location:r.location},t._getToolbarItemByAlias(r));var u="generic"===o;"done"===r.shortcut&&u||"cancel"===r.shortcut&&!u?i.unshift(l):i.push(l)}}),"top"===e&&this.option("showCloseButton")&&this.option("showTitle")&&i.push(this._getCloseButton()),i},_getLocalizationKey:function(e){return"done"===e.toLowerCase()?"OK":(0,p.camelize)(e,!0)},_getToolbarItemByAlias:function(e){var t=this,n=e.shortcut;if((0,c.inArray)(n,z)<0)return!1;var i=(0,f.extend)({text:w.default.format(this._getLocalizationKey(n)),onClick:this._createToolbarItemAction(e.onClick),integrationOptions:{},type:t.option("useDefaultToolbarButtons")?N:j,stylingMode:t.option("useFlatToolbarButtons")?W:G},e.options||{}),o=I+"-"+n;return this._toolbarItemClasses.push(o),{template:function(e,n,a){var r=(0,l.default)("<div>").addClass(o).appendTo(a);t._createComponent(r,C.default,i)}}},_createToolbarItemAction:function(e){return this._createAction(e,{afterExecute:function(e){e.component.hide()}})},_renderBottom:function(){var e=this._getToolbarItems("bottom");if(e.length){this._$bottom&&this._$bottom.remove();var t=(0,l.default)("<div>").addClass(V).insertAfter(this.$content());this._$bottom=this._renderTemplateByType("bottomTemplate",e,t,{compactMode:!0}).addClass(V),this._toggleClasses()}else this._$bottom&&this._$bottom.detach()},_toggleClasses:function(){var e=this,t=z;(0,g.each)(t,function(t,n){var i=I+"-"+n;(0,c.inArray)(i,e._toolbarItemClasses)>=0?(e._wrapper().addClass(i+"-visible"),e._$bottom.addClass(i)):(e._wrapper().removeClass(i+"-visible"),e._$bottom.removeClass(i))})},_getContainer:function(){return this.option("fullScreen")?(0,l.default)(D):this.callBase()},_getDragTarget:function(){return this.topToolbar()},_renderGeometryImpl:function(e){e||this._resetContentHeight(),this.callBase.apply(this,arguments),this._setContentHeight()},_resetContentHeight:function(){this._$popupContent.css({height:"auto",maxHeight:"none"})},_renderDrag:function(){this.callBase(),this._$content.toggleClass(R,this.option("dragEnabled"))},_renderResize:function(){this.callBase(),this._resizable.option("onResize",function(){this._setContentHeight(),this._actions.onResize(arguments)}.bind(this))},_setContentHeight:function(){(this.option("forceApplyBindings")||h.noop)();var e=this.overlayContent().get(0),t=this._chooseHeightStrategy(e);this.$content().css(this._getHeightCssStyles(t,e)),this._setHeightClasses(this.overlayContent(),t)},_heightStrategyChangeOffset:function(e,t){return e===K.flex?-t:0},_chooseHeightStrategy:function(e){var t="auto"===e.style.width||""===e.style.width,n=K.static;return this._isAutoHeight()&&this.option("autoResizeEnabled")&&(t||$?q||(n=K.inherit):n=K.flex),n},_getHeightCssStyles:function(e,t){var n={},i=this._getOptionValue("maxHeight",t),o=this._getOptionValue("minHeight",t),a=this._splitPopupHeight(),r=a.header+a.footer+a.contentVerticalOffsets+a.popupVerticalOffsets+this._heightStrategyChangeOffset(e,a.popupVerticalPaddings);if(e===K.static){if(!this._isAutoHeight()||i||o){var s=this.option("fullScreen")?Math.min((0,m.getBoundingRect)(t).height,(0,b.getWindow)().innerHeight):(0,m.getBoundingRect)(t).height,u=s-r;n={height:Math.max(0,u),minHeight:"auto",maxHeight:"auto"}}}else{var c=(0,l.default)(this._getContainer()).get(0),d=(0,_.addOffsetToMaxHeight)(i,-r,c),h=(0,_.addOffsetToMinHeight)(o,-r,c);n={height:"auto",minHeight:h,maxHeight:d}}return n},_setHeightClasses:function(e,t){var n="";for(var i in K)K[i]!==t&&(n+=" "+K[i]);e.removeClass(n).addClass(t)},_isAutoHeight:function(){return"auto"===this.overlayContent().get(0).style.height},_splitPopupHeight:function(){var e=this.topToolbar(),t=this.bottomToolbar();return{header:(0,_.getVisibleHeight)(e&&e.get(0)),footer:(0,_.getVisibleHeight)(t&&t.get(0)),contentVerticalOffsets:(0,_.getVerticalOffsets)(this.overlayContent().get(0),!0),popupVerticalOffsets:(0,_.getVerticalOffsets)(this.$content().get(0),!0),popupVerticalPaddings:(0,_.getVerticalOffsets)(this.$content().get(0),!1)}},_shouldFixBodyPosition:function(){return this.callBase()||this.option("fullScreen")},_toggleSafariFullScreen:function(e){var t=this._shouldFixBodyPosition()&&e&&!this._isShown;t?this._bodyScrollTop=e?D.pageYOffset:void 0:this._toggleSafariScrolling(!e)},_renderDimensions:function(){this.option("fullScreen")?this._$content.css({width:"100%",height:"100%",minWidth:"",maxWidth:"",minHeight:"",maxHeight:""}):this.callBase.apply(this,arguments),(0,b.hasWindow)()&&this._renderFullscreenWidthClass()},_renderFullscreenWidthClass:function(){this.overlayContent().toggleClass(E,this.overlayContent().outerWidth()===(0,l.default)(D).width())},refreshPosition:function(){this._renderPosition()},_renderPosition:function(){return this.option("fullScreen")?void(0,o.move)(this._$content,{top:0,left:0}):((this.option("forceApplyBindings")||h.noop)(),this.callBase.apply(this,arguments))},_optionChanged:function(e){switch(e.name){case"showTitle":case"title":case"titleTemplate":this._renderTitle(),
this._renderGeometry(),(0,x.triggerResizeEvent)(this._$content);break;case"bottomTemplate":this._renderBottom(),this._renderGeometry(),(0,x.triggerResizeEvent)(this._$content);break;case"onTitleRendered":this._createTitleRenderAction(e.value);break;case"toolbarItems":case"useDefaultToolbarButtons":case"useFlatToolbarButtons":var t=!e.fullName.match(/^toolbarItems((\[\d+\])(\.(options|visible).*)?)?$/);this._renderTitle(),this._renderBottom(),t&&(this._renderGeometry(),(0,x.triggerResizeEvent)(this._$content));break;case"dragEnabled":this._renderDrag();break;case"autoResizeEnabled":this._renderGeometry(),(0,x.triggerResizeEvent)(this._$content);break;case"fullScreen":this._toggleFullScreenClass(e.value),this._toggleSafariFullScreen(e.value),this._renderGeometry(),(0,x.triggerResizeEvent)(this._$content);break;case"showCloseButton":this._renderTitle();break;default:this.callBase(e)}},bottomToolbar:function(){return this._$bottom},topToolbar:function(){return this._$title},$content:function(){return this._$popupContent},content:function(){return(0,s.getPublicElement)(this._$popupContent)},overlayContent:function(){return this._$content}});(0,a.default)("dxPopup",Y);var Z=Y;t.default=Z,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(11)),a=i(n(54)),r=n(7),s=i(n(64)),l=[],u=function(){return"complete"===o.default.getReadyState()||"loading"!==o.default.getReadyState()&&!o.default.getDocumentElement().doScroll},c=(0,s.default)(function(){var e=o.default.listen(o.default.getDocument(),"DOMContentLoaded",function(){d.fire(),e()})}),d={add:function(e){var t=(0,r.hasWindow)();t&&u()?e():(l.push(e),t&&c())},fire:function(){l.forEach(function(e){return e()}),l=[]}},h=(0,a.default)(d);t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.deepExtendArraySafe=t.orderEach=t.clone=void 0;var o=n(1),a=i(n(101)),r=function(){function e(){}return function(t){return e.prototype=t,new e}}();t.clone=r;var s=function(e,t){var n,i,a=[];for(n in e)Object.prototype.hasOwnProperty.call(e,n)&&a.push(n);for(a.sort(function(e,t){var n=(0,o.isNumeric)(e),i=(0,o.isNumeric)(t);return n&&i?e-t:n&&!i?-1:!n&&i?1:e<t?-1:e>t?1:0}),i=0;i<a.length;i++)n=a[i],t(n,e[n])};t.orderEach=s;var l=function(e,t,n,i){!i&&a.default.isWrapped(e[t])?a.default.assign(e[t],n):e[t]=n},u=function e(t,n,i,a){var r,s;for(var u in n)if(r=t[u],s=n[u],"__proto__"!==u&&t!==s){if((0,o.isPlainObject)(s)){var c=i?(0,o.isObject)(r):(0,o.isPlainObject)(r);s=e(c?r:{},s,i,a)}void 0!==s&&r!==s&&l(t,u,s,a)}return t};t.deepExtendArraySafe=u},function(e,t,n){function i(e,t){for(var n in t)e[n]=t[n];return e}function o(e){var t=e.parentNode;t&&t.removeChild(e)}function a(e,t,n){var i,o,a,s=arguments,l={};for(a in t)"key"==a?i=t[a]:"ref"==a?o=t[a]:l[a]=t[a];if(arguments.length>3)for(n=[n],a=3;a<arguments.length;a++)n.push(s[a]);if(null!=n&&(l.children=n),"function"==typeof e&&null!=e.defaultProps)for(a in e.defaultProps)void 0===l[a]&&(l[a]=e.defaultProps[a]);return r(e,l,i,o,null)}function r(e,t,n,i,o){var a={type:e,props:t,key:n,ref:i,__k:null,__:null,__b:0,__e:null,__d:void 0,__c:null,__h:null,constructor:void 0,__v:null==o?++R.__v:o};return null!=R.vnode&&R.vnode(a),a}function s(){return{current:null}}function l(e){return e.children}function u(e,t){this.props=e,this.context=t}function c(e,t){if(null==t)return e.__?c(e.__,e.__.__k.indexOf(e)+1):null;for(var n;t<e.__k.length;t++)if(null!=(n=e.__k[t])&&null!=n.__e)return n.__e;return"function"==typeof e.type?c(e):null}function d(e){var t,n;if(null!=(e=e.__)&&null!=e.__c){for(e.__e=e.__c.base=null,t=0;t<e.__k.length;t++)if(null!=(n=e.__k[t])&&null!=n.__e){e.__e=e.__c.base=n.__e;break}return d(e)}}function h(e){(!e.__d&&(e.__d=!0)&&B.push(e)&&!f.__r++||F!==R.debounceRendering)&&((F=R.debounceRendering)||V)(f)}function f(){for(var e;f.__r=B.length;)e=B.sort(function(e,t){return e.__v.__b-t.__v.__b}),B=[],e.some(function(e){var t,n,o,a,r,s;e.__d&&(r=(a=(t=e).__v).__e,(s=t.__P)&&(n=[],(o=i({},a)).__v=a.__v+1,C(s,a,o,t.__n,void 0!==s.ownerSVGElement,null!=a.__h?[r]:null,n,null==r?c(a):r,a.__h),k(n,a),a.__e!=r&&d(a)))})}function p(e,t,n,i,a,s,u,d,h,f){var p,_,v,y,b,x,w,k=i&&i.__k||N,S=k.length;for(h==z&&(h=null!=u?u[0]:S?c(i,0):null),n.__k=[],p=0;p<t.length;p++)if(null!=(y=n.__k[p]=null==(y=t[p])||"boolean"==typeof y?null:"string"==typeof y||"number"==typeof y?r(null,y,null,null,y):Array.isArray(y)?r(l,{children:y},null,null,null):y.__b>0?r(y.type,y.props,y.key,null,y.__v):y)){if(y.__=n,y.__b=n.__b+1,null===(v=k[p])||v&&y.key==v.key&&y.type===v.type)k[p]=void 0;else for(_=0;_<S;_++){if((v=k[_])&&y.key==v.key&&y.type===v.type){k[_]=void 0;break}v=null}C(e,y,v=v||z,a,s,u,d,h,f),b=y.__e,(_=y.ref)&&v.ref!=_&&(w||(w=[]),v.ref&&w.push(v.ref,null,y),w.push(_,y.__c||b,y)),null!=b?(null==x&&(x=b),"function"==typeof y.type&&null!=y.__k&&y.__k===v.__k?y.__d=h=g(y,h,e):h=m(e,y,v,k,u,b,h),f||"option"!==n.type?"function"==typeof n.type&&(n.__d=h):e.value=""):h&&v.__e==h&&h.parentNode!=e&&(h=c(v))}if(n.__e=x,null!=u&&"function"!=typeof n.type)for(p=u.length;p--;)null!=u[p]&&o(u[p]);for(p=S;p--;)null!=k[p]&&("function"==typeof n.type&&null!=k[p].__e&&k[p].__e==n.__d&&(n.__d=c(i,p+1)),I(k[p],k[p]));if(w)for(p=0;p<w.length;p++)D(w[p],w[++p],w[++p])}function g(e,t,n){var i,o;for(i=0;i<e.__k.length;i++)(o=e.__k[i])&&(o.__=e,t="function"==typeof o.type?g(o,t,n):m(n,o,o,e.__k,null,o.__e,t));return t}function _(e,t){return t=t||[],null==e||"boolean"==typeof e||(Array.isArray(e)?e.some(function(e){_(e,t)}):t.push(e)),t}function m(e,t,n,i,o,a,r){var s,l,u;if(void 0!==t.__d)s=t.__d,t.__d=void 0;else if(o==n||a!=r||null==a.parentNode)e:if(null==r||r.parentNode!==e)e.appendChild(a),s=null;else{for(l=r,u=0;(l=l.nextSibling)&&u<i.length;u+=2)if(l==a)break e;e.insertBefore(a,r),s=r}return void 0!==s?s:a.nextSibling}function v(e,t,n,i,o){var a;for(a in n)"children"===a||"key"===a||a in t||b(e,a,null,n[a],i);for(a in t)o&&"function"!=typeof t[a]||"children"===a||"key"===a||"value"===a||"checked"===a||n[a]===t[a]||b(e,a,t[a],n[a],i)}function y(e,t,n){"-"===t[0]?e.setProperty(t,n):e[t]=null==n?"":"number"!=typeof n||j.test(t)?n:n+"px"}function b(e,t,n,i,o){var a,r,s;if(o&&"className"==t&&(t="class"),"style"===t)if("string"==typeof n)e.style.cssText=n;else{if("string"==typeof i&&(e.style.cssText=i=""),i)for(t in i)n&&t in n||y(e.style,t,"");if(n)for(t in n)i&&n[t]===i[t]||y(e.style,t,n[t])}else"o"===t[0]&&"n"===t[1]?(a=t!==(t=t.replace(/Capture$/,"")),(r=t.toLowerCase())in e&&(t=r),t=t.slice(2),e.l||(e.l={}),e.l[t+a]=n,s=a?w:x,n?i||e.addEventListener(t,s,a):e.removeEventListener(t,s,a)):"list"!==t&&"tagName"!==t&&"form"!==t&&"type"!==t&&"size"!==t&&"download"!==t&&"href"!==t&&"contentEditable"!==t&&!o&&t in e?e[t]=null==n?"":n:"function"!=typeof n&&"dangerouslySetInnerHTML"!==t&&(t!==(t=t.replace(/xlink:?/,""))?null==n||!1===n?e.removeAttributeNS("http://www.w3.org/1999/xlink",t.toLowerCase()):e.setAttributeNS("http://www.w3.org/1999/xlink",t.toLowerCase(),n):null==n||!1===n&&!/^ar/.test(t)?e.removeAttribute(t):e.setAttribute(t,n))}function x(e){this.l[e.type+!1](R.event?R.event(e):e)}function w(e){this.l[e.type+!0](R.event?R.event(e):e)}function C(e,t,n,o,a,r,s,c,d){var h,f,g,_,m,v,y,b,x,w,C,k=t.type;if(void 0!==t.constructor)return null;null!=n.__h&&(d=n.__h,c=t.__e=n.__e,t.__h=null,r=[c]),(h=R.__b)&&h(t);try{e:if("function"==typeof k){if(b=t.props,x=(h=k.contextType)&&o[h.__c],w=h?x?x.props.value:h.__:o,n.__c?y=(f=t.__c=n.__c).__=f.__E:("prototype"in k&&k.prototype.render?t.__c=f=new k(b,w):(t.__c=f=new u(b,w),f.constructor=k,f.render=T),x&&x.sub(f),f.props=b,f.state||(f.state={}),f.context=w,f.__n=o,g=f.__d=!0,f.__h=[]),null==f.__s&&(f.__s=f.state),null!=k.getDerivedStateFromProps&&(f.__s==f.state&&(f.__s=i({},f.__s)),i(f.__s,k.getDerivedStateFromProps(b,f.__s))),_=f.props,m=f.state,g)null==k.getDerivedStateFromProps&&null!=f.componentWillMount&&f.componentWillMount(),null!=f.componentDidMount&&f.__h.push(f.componentDidMount);else{if(null==k.getDerivedStateFromProps&&b!==_&&null!=f.componentWillReceiveProps&&f.componentWillReceiveProps(b,w),!f.__e&&null!=f.shouldComponentUpdate&&!1===f.shouldComponentUpdate(b,f.__s,w)||t.__v===n.__v){f.props=b,f.state=f.__s,t.__v!==n.__v&&(f.__d=!1),f.__v=t,t.__e=n.__e,t.__k=n.__k,f.__h.length&&s.push(f);break e}null!=f.componentWillUpdate&&f.componentWillUpdate(b,f.__s,w),null!=f.componentDidUpdate&&f.__h.push(function(){f.componentDidUpdate(_,m,v)})}f.context=w,f.props=b,f.state=f.__s,(h=R.__r)&&h(t),f.__d=!1,f.__v=t,f.__P=e,h=f.render(f.props,f.state,f.context),f.state=f.__s,null!=f.getChildContext&&(o=i(i({},o),f.getChildContext())),g||null==f.getSnapshotBeforeUpdate||(v=f.getSnapshotBeforeUpdate(_,m)),C=null!=h&&h.type===l&&null==h.key?h.props.children:h,p(e,Array.isArray(C)?C:[C],t,n,o,a,r,s,c,d),f.base=t.__e,t.__h=null,f.__h.length&&s.push(f),y&&(f.__E=f.__=null),f.__e=!1}else null==r&&t.__v===n.__v?(t.__k=n.__k,t.__e=n.__e):t.__e=S(n.__e,t,n,o,a,r,s,d);(h=R.diffed)&&h(t)}catch(e){t.__v=null,(d||null!=r)&&(t.__e=c,t.__h=!!d,r[r.indexOf(c)]=null),R.__e(e,t,n)}}function k(e,t){R.__c&&R.__c(t,e),e.some(function(t){try{e=t.__h,t.__h=[],e.some(function(e){e.call(t)})}catch(e){R.__e(e,t.__v)}})}function S(e,t,n,i,o,a,r,s){var l,u,c,d,h,f=n.props,g=t.props;if(o="svg"===t.type||o,null!=a)for(l=0;l<a.length;l++)if(null!=(u=a[l])&&((null===t.type?3===u.nodeType:u.localName===t.type)||e==u)){e=u,a[l]=null;break}if(null==e){if(null===t.type)return document.createTextNode(g);e=o?document.createElementNS("http://www.w3.org/2000/svg",t.type):document.createElement(t.type,g.is&&{is:g.is}),a=null,s=!1}if(null===t.type)f===g||s&&e.data===g||(e.data=g);else{if(null!=a&&(a=N.slice.call(e.childNodes)),c=(f=n.props||z).dangerouslySetInnerHTML,d=g.dangerouslySetInnerHTML,!s){if(null!=a)for(f={},h=0;h<e.attributes.length;h++)f[e.attributes[h].name]=e.attributes[h].value;(d||c)&&(d&&(c&&d.__html==c.__html||d.__html===e.innerHTML)||(e.innerHTML=d&&d.__html||""))}v(e,g,f,o,s),d?t.__k=[]:(l=t.props.children,p(e,Array.isArray(l)?l:[l],t,n,i,"foreignObject"!==t.type&&o,a,r,z,s)),s||("value"in g&&void 0!==(l=g.value)&&(l!==e.value||"progress"===t.type&&!l)&&b(e,"value",l,f.value,!1),"checked"in g&&void 0!==(l=g.checked)&&l!==e.checked&&b(e,"checked",l,f.checked,!1))}return e}function D(e,t,n){try{"function"==typeof e?e(t):e.current=t}catch(e){R.__e(e,n)}}function I(e,t,n){var i,a,r;if(R.unmount&&R.unmount(e),(i=e.ref)&&(i.current&&i.current!==e.__e||D(i,null,t)),n||"function"==typeof e.type||(n=null!=(a=e.__e)),e.__e=e.__d=void 0,null!=(i=e.__c)){if(i.componentWillUnmount)try{i.componentWillUnmount()}catch(e){R.__e(e,t)}i.base=i.__P=null}if(i=e.__k)for(r=0;r<i.length;r++)i[r]&&I(i[r],t,n);null!=a&&o(a)}function T(e,t,n){return this.constructor(e,n)}function O(e,t,n){var i,o,r;R.__&&R.__(e,t),o=(i=n===L)?null:n&&n.__k||t.__k,e=a(l,null,[e]),r=[],C(t,(i?t:n||t).__k=e,o||z,z,void 0!==t.ownerSVGElement,n&&!i?[n]:o?null:t.childNodes.length?N.slice.call(t.childNodes):null,r,n||z,i),k(r,e)}function E(e,t){O(e,t,L)}function A(e,t,n){var o,a,s,l=arguments,u=i({},e.props);for(s in t)"key"==s?o=t[s]:"ref"==s?a=t[s]:u[s]=t[s];if(arguments.length>3)for(n=[n],s=3;s<arguments.length;s++)n.push(l[s]);return null!=n&&(u.children=n),r(e.type,u,o||e.key,a||e.ref,null)}function P(e,t){var n={__c:t="__cC"+H++,__:e,Consumer:function(e,t){return e.children(t)},Provider:function(e){var n,i;return this.getChildContext||(n=[],(i={})[t]=this,this.getChildContext=function(){return i},this.shouldComponentUpdate=function(e){this.props.value!==e.value&&n.some(h)},this.sub=function(e){n.push(e);var t=e.componentWillUnmount;e.componentWillUnmount=function(){n.splice(n.indexOf(e),1),t&&t.call(e)}}),e.children}};return n.Provider.__=n.Consumer.contextType=n}Object.defineProperty(t,"__esModule",{value:!0}),n.d(t,"render",function(){return O}),n.d(t,"hydrate",function(){return E}),n.d(t,"createElement",function(){return a}),n.d(t,"h",function(){return a}),n.d(t,"Fragment",function(){return l}),n.d(t,"createRef",function(){return s}),n.d(t,"isValidElement",function(){return M}),n.d(t,"Component",function(){return u}),n.d(t,"cloneElement",function(){return A}),n.d(t,"createContext",function(){return P}),n.d(t,"toChildArray",function(){return _}),n.d(t,"options",function(){return R});var R,M,B,V,F,L,H,z={},N=[],j=/acit|ex(?:s|g|n|p|$)|rph|grid|ows|mnc|ntw|ine[ch]|zoo|^ord|itera/i;R={__e:function(e,t){for(var n,i,o,a=t.__h;t=t.__;)if((n=t.__c)&&!n.__)try{if((i=n.constructor)&&null!=i.getDerivedStateFromError&&(n.setState(i.getDerivedStateFromError(e)),o=n.__d),null!=n.componentDidCatch&&(n.componentDidCatch(e),o=n.__d),o)return t.__h=a,n.__E=n}catch(t){e=t}throw e},__v:0},M=function(e){return null!=e&&void 0===e.constructor},u.prototype.setState=function(e,t){var n;n=null!=this.__s&&this.__s!==this.state?this.__s:this.__s=i({},this.state),"function"==typeof e&&(e=e(i({},n),this.props)),e&&i(n,e),null!=e&&this.__v&&(t&&this.__h.push(t),h(this))},u.prototype.forceUpdate=function(e){this.__v&&(this.__e=!0,e&&this.__h.push(e),h(this))},u.prototype.render=l,B=[],V="function"==typeof Promise?Promise.prototype.then.bind(Promise.resolve()):setTimeout,f.__r=0,L=z,H=0},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(39),r=i(n(27)),s=n(7),l=n(8),u=n(0),c=i(n(16)),d=i(n(100)),h=i(n(5)),f=i(n(388)),p=i(n(35)),g="dx-invalid-message-auto",_="dx-state-readonly",m="dx-invalid",v="dx-show-invalid-badge",y="dx-validation-target",b="valid",x="invalid",w="editorReadOnly",C={validationMessageMode:"mode",validationMessageOffset:"offset",validationBoundary:"boundary"},k=c.default.inherit({ctor:function(){this.showValidationMessageTimeout=null,this.validationRequest=(0,r.default)(),this.callBase.apply(this,arguments)},_createElement:function(e){this.callBase(e);var t=this.$element();t&&(0,a.data)(t[0],y,this)},_initOptions:function(e){this.callBase.apply(this,arguments),this.option(d.default.initValidationOptions(e))},_init:function(){this.callBase(),this._options.cache("validationTooltipOptions",this.option("validationTooltipOptions"));var e=this.$element();e.addClass(v)},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{value:null,name:"",onValueChanged:null,readOnly:!1,isValid:!0,validationError:null,validationErrors:null,validationStatus:b,validationMessageMode:"auto",validationBoundary:void 0,validationMessageOffset:{h:0,v:0},validationTooltipOptions:{}})},_attachKeyboardEvents:function(){this.option("readOnly")||this.callBase()},_setOptionsByReference:function(){this.callBase(),(0,u.extend)(this._optionsByReference,{validationError:!0})},_createValueChangeAction:function(){this._valueChangeAction=this._createActionByOption("onValueChanged",{excludeValidators:["disabled","readOnly"]})},_suppressValueChangeAction:function(){this._valueChangeActionSuppressed=!0},_resumeValueChangeAction:function(){this._valueChangeActionSuppressed=!1},_initMarkup:function(){this._toggleReadOnlyState(),this._setSubmitElementName(this.option("name")),this.callBase(),this._renderValidationState()},_raiseValueChangeAction:function(e,t){this._valueChangeAction||this._createValueChangeAction(),this._valueChangeAction(this._valueChangeArgs(e,t))},_valueChangeArgs:function(e,t){return{value:e,previousValue:t,event:this._valueChangeEventInstance}},_saveValueChangeEvent:function(e){this._valueChangeEventInstance=e},_focusInHandler:function(e){var t="auto"===this.option("validationMessageMode");if(this._canValueBeChangedByClick()&&t){var n,i=null===(n=this._validationMessage)||void 0===n?void 0:n._wrapper();null===i||void 0===i?void 0:i.removeClass(g),clearTimeout(this.showValidationMessageTimeout),this.showValidationMessageTimeout=setTimeout(function(){return null===i||void 0===i?void 0:i.addClass(g)},150)}return this.callBase(e)},_canValueBeChangedByClick:function(){return!1},_getValidationErrors:function(){var e=this.option("validationErrors");return!e&&this.option("validationError")&&(e=[this.option("validationError")]),e},_disposeValidationMessage:function(){this._$validationMessage&&(this._$validationMessage.remove(),this.setAria("describedby",null),this._$validationMessage=null)},_toggleValidationClasses:function(e){this.$element().toggleClass(m,e),this.setAria(x,e||void 0)},_renderValidationState:function(){var e=this.option("isValid")&&this.option("validationStatus")!==x,t=this._getValidationErrors(),n=this.$element();this._toggleValidationClasses(!e),(0,s.hasWindow)()&&(this._disposeValidationMessage(),!e&&t&&(this._$validationMessage=(0,o.default)("<div>").appendTo(n),this.setAria("describedby","dx-"+new p.default),this._validationMessage=new f.default(this._$validationMessage,(0,u.extend)({validationErrors:t,target:this._getValidationMessageTarget(),container:n,mode:this.option("validationMessageMode"),positionRequest:"below",offset:this.option("validationMessageOffset"),boundary:this.option("validationBoundary"),rtlEnabled:this.option("rtlEnabled")},this._options.cache("validationTooltipOptions"))),this._bindInnerWidgetOptions(this._validationMessage,"validationTooltipOptions")))},_getValidationMessageTarget:function(){return this.$element()},_toggleReadOnlyState:function(){var e=this.option("readOnly");this._toggleBackspaceHandler(e),this.$element().toggleClass(_,!!e),this.setAria("readonly",e||void 0)},_toggleBackspaceHandler:function(e){var t=this._keyboardEventBindingTarget(),n=(0,l.addNamespace)("keydown",w);h.default.off(t,n),e&&h.default.on(t,n,function(e){"backspace"===(0,l.normalizeKeyName)(e)&&e.preventDefault()})},_dispose:function(){var e=this.$element()[0];(0,a.data)(e,y,null),clearTimeout(this.showValidationMessageTimeout),this.callBase()},_setSubmitElementName:function(e){var t=this._getSubmitElement();t&&(e.length>0?t.attr("name",e):t.removeAttr("name"))},_getSubmitElement:function(){return null},_setValidationMessageOption:function(e){var t,n=e.name,i=e.value,o=C[n]?C[n]:n;null===(t=this._validationMessage)||void 0===t?void 0:t.option(o,i)},_optionChanged:function(e){var t;switch(e.name){case"onValueChanged":this._createValueChangeAction();break;case"readOnly":this._toggleReadOnlyState(),this._refreshFocusState();break;case"value":e.value!=e.previousValue&&this.validationRequest.fire({value:e.value,editor:this}),this._valueChangeActionSuppressed||(this._raiseValueChangeAction(e.value,e.previousValue),this._saveValueChangeEvent(void 0));break;case"width":this.callBase(e),null===(t=this._validationMessage)||void 0===t?void 0:t.updateMaxWidth();break;case"name":this._setSubmitElementName(e.value);break;case"isValid":case"validationError":case"validationErrors":case"validationStatus":this.option(d.default.synchronizeValidationOptions(e,this.option())),this._renderValidationState();break;case"validationBoundary":case"validationMessageMode":case"validationMessageOffset":this._setValidationMessageOption(e);break;case"rtlEnabled":this._setValidationMessageOption(e),this.callBase(e);break;case"validationTooltipOptions":this._innerWidgetOptionChanged(this._validationMessage,e);break;default:this.callBase(e)}},reset:function(){var e=this._getDefaultOptions();this.option("value",e.value)}}),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e,t){C.options.__h&&C.options.__h(x,e,k||t),k=0;var n=x.__H||(x.__H={__:[],__h:[]});return e>=n.__.length&&n.__.push({}),n.__[e]}function o(e){return k=1,a(y,e)}function a(e,t,n){var o=i(b++,2);return o.t=e,o.__c||(o.__=[n?n(t):y(void 0,t),function(e){var t=o.t(o.__[0],e);o.__[0]!==t&&(o.__=[t,o.__[1]],o.__c.setState({}))}],o.__c=x),o.__}function r(e,t){var n=i(b++,3);!C.options.__s&&v(n.__H,t)&&(n.__=e,n.__H=t,x.__H.__h.push(n))}function s(e,t){var n=i(b++,4);!C.options.__s&&v(n.__H,t)&&(n.__=e,n.__H=t,x.__h.push(n))}function l(e){return k=5,c(function(){return{current:e}},[])}function u(e,t,n){k=6,s(function(){"function"==typeof e?e(t()):e&&(e.current=t())},null==n?n:n.concat(e))}function c(e,t){var n=i(b++,7);return v(n.__H,t)&&(n.__=e(),n.__H=t,n.__h=e),n.__}function d(e,t){return k=8,c(function(){return e},t)}function h(e){var t=x.context[e.__c],n=i(b++,9);return n.__c=e,t?(null==n.__&&(n.__=!0,t.sub(x)),t.props.value):e.__}function f(e,t){C.options.useDebugValue&&C.options.useDebugValue(t?t(e):e)}function p(e){var t=i(b++,10),n=o();return t.__=e,x.componentDidCatch||(x.componentDidCatch=function(e){t.__&&t.__(e),n[1](e)}),[n[0],function(){n[1](void 0)}]}function g(){S.forEach(function(e){if(e.__P)try{e.__H.__h.forEach(_),e.__H.__h.forEach(m),e.__H.__h=[]}catch(t){e.__H.__h=[],C.options.__e(t,e.__v)}}),S=[]}function _(e){var t=x;"function"==typeof e.__c&&e.__c(),x=t}function m(e){var t=x;e.__c=e.__(),x=t}function v(e,t){return!e||e.length!==t.length||t.some(function(t,n){return t!==e[n]})}function y(e,t){return"function"==typeof t?t(e):t}Object.defineProperty(t,"__esModule",{value:!0}),n.d(t,"useState",function(){return o}),n.d(t,"useReducer",function(){return a}),n.d(t,"useEffect",function(){return r}),n.d(t,"useLayoutEffect",function(){return s}),n.d(t,"useRef",function(){return l}),n.d(t,"useImperativeHandle",function(){return u}),n.d(t,"useMemo",function(){return c}),n.d(t,"useCallback",function(){return d}),n.d(t,"useContext",function(){return h}),n.d(t,"useDebugValue",function(){return f}),n.d(t,"useErrorBoundary",function(){return p});var b,x,w,C=n(51),k=0,S=[],D=C.options.__b,I=C.options.__r,T=C.options.diffed,O=C.options.__c,E=C.options.unmount;C.options.__b=function(e){x=null,D&&D(e)},C.options.__r=function(e){I&&I(e),b=0;var t=(x=e.__c).__H;t&&(t.__h.forEach(_),t.__h.forEach(m),t.__h=[])},C.options.diffed=function(e){T&&T(e);var t=e.__c;t&&t.__H&&t.__H.__h.length&&(1!==S.push(t)&&w===C.options.requestAnimationFrame||((w=C.options.requestAnimationFrame)||function(e){var t,n=function(){clearTimeout(i),A&&cancelAnimationFrame(t),setTimeout(e)},i=setTimeout(n,100);A&&(t=requestAnimationFrame(n))})(g)),x=void 0},C.options.__c=function(e,t){t.some(function(e){try{e.__h.forEach(_),e.__h=e.__h.filter(function(e){return!e.__||m(e)})}catch(n){t.some(function(e){e.__h&&(e.__h=[])}),t=[],C.options.__e(n,e.__v)}}),O&&O(e,t)},C.options.unmount=function(e){E&&E(e);var t=e.__c;if(t&&t.__H)try{t.__H.__.forEach(_)}catch(e){C.options.__e(e,t.__v)}};var A="function"==typeof requestAnimationFrame},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=l.default.inherit(e),n=t,i=new n(e),o={},u=function(t,n){(0,s.each)(t,function(t){(0,r.isFunction)(i[t])?!n&&e[t]||(e[t]=function(){return i[t].apply(e,arguments)}):(n&&(o[t]=e[t]),e[t]=i[t])})};return u(e,!0),e.inject=function(e){n=n.inherit(e),i=new n,u(e)},e.resetInjection=function(){(0,a.extend)(e,o),n=t,i=new t},e}t.default=o;var a=n(0),r=n(1),s=n(3),l=i(n(15));e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.triggerResizeEvent=t.triggerHidingEvent=t.triggerShownEvent=void 0;var o=i(n(2)),a=i(n(5)),r=function(e){var t=".dx-visibility-change-handler";return function(n){for(var i=(0,o.default)(n||"body"),r=i.filter(t).add(i.find(t)),s=0;s<r.length;s++)a.default.triggerHandler(r[s],e)}},s=r("dxshown");t.triggerShownEvent=s;var l=r("dxhiding");t.triggerHidingEvent=l;var u=r("dxresize");t.triggerResizeEvent=u},function(e,t){e.exports=window.jQuery},function(e,t,n){function i(e){var t=(0,w.normalizeEnum)(e);return k[t]||k[S[t]||r()]}function o(e,t){return S[e+"."+t]||D[e+"."+t]||S[e]}function a(e,t,n){return o(e+t,n)||o(e,n)}function r(e,t){if(!arguments.length)return O||o((0,C.current)())||y;var n=(0,w.normalizeEnum)(t);return O=(e&&e.platform?a((0,w.normalizeEnum)(e.platform),e.version,n):o((0,w.normalizeEnum)(e),n))||O,this}function s(e,t){var n=e.indexOf(t);return n>0?{name:e.substring(0,n),scheme:e.substring(n+1)}:null}function l(e,t){var n=s(e,".")||{name:e},i=n.name,o=n.scheme;o?(S[i]=S[i]||t,S[i+"."+o]=t):S[i]=t}function u(e,t){var n=(0,w.normalizeEnum)(e&&e.name);n&&(e.isDefault&&(y=n),l(n,n),k[n]=I(!0,{},i(t),f(e)))}function c(e,t){D[e]=t}function d(e,t,n,i){var o=n?n[t]:i;void 0!==o&&void 0===e[t]&&(e[t]=o)}function h(e,t,n,i){var o=n?n[t]:i;void 0!==o&&(e[t]=I(!0,{},o,e[t]))}function f(e){return e=I(!0,{loadingIndicator:{font:{}},"export":{font:{}},legend:{font:{},border:{}},title:{font:{}},tooltip:{font:{}},"chart:common":{},"chart:common:axis":{grid:{},minorGrid:{},tick:{},minorTick:{},title:{font:{}},label:{font:{}}},"chart:common:annotation":{font:{},border:{}},chart:{commonSeriesSettings:{candlestick:{}}},pie:{},polar:{},gauge:{scale:{tick:{},minorTick:{},label:{font:{}}}},barGauge:{},funnel:{},sankey:{},map:{background:{}},treeMap:{tile:{selectionStyle:{border:{}}},group:{border:{},selectionStyle:{border:{}},label:{font:{}}}},rangeSelector:{scale:{tick:{},minorTick:{},label:{font:{}}},chart:{}},sparkline:{},bullet:{}},e),d(e.loadingIndicator,"backgroundColor",e),d(e.chart.commonSeriesSettings.candlestick,"innerColor",null,e.backgroundColor),d(e.map.background,"color",null,e.backgroundColor),d(e.title.font,"color",null,e.primaryTitleColor),h(e.title,"subtitle",null,e.title),d(e.legend.font,"color",null,e.secondaryTitleColor),d(e.legend.border,"color",null,e.gridColor),p(e),T(["chart","pie","polar","gauge","barGauge","map","treeMap","funnel","rangeSelector","sparkline","bullet","sankey"],function(t,n){d(e[n],"redrawOnResize",e),d(e[n],"containerBackgroundColor",null,e.backgroundColor),h(e[n],"tooltip",e),h(e[n],"export",e)}),T(["chart","pie","polar","gauge","barGauge","map","treeMap","funnel","rangeSelector","sankey"],function(t,n){h(e[n],"loadingIndicator",e),h(e[n],"legend",e),h(e[n],"title",e)}),T(["chart","pie","polar"],function(t,n){h(e,n,null,e["chart:common"])}),T(["chart","polar"],function(t,n){e[n]=e[n]||{},h(e[n],"commonAxisSettings",null,e["chart:common:axis"])}),T(["chart","polar","map","pie"],function(t,n){e[n]=e[n]||{},h(e[n],"commonAnnotationSettings",null,e["chart:common:annotation"])}),h(e.rangeSelector.chart,"commonSeriesSettings",e.chart),h(e.rangeSelector.chart,"dataPrepareSettings",e.chart),d(e.treeMap.group.border,"color",null,e.gridColor),d(e.treeMap.tile.selectionStyle.border,"color",null,e.primaryTitleColor),d(e.treeMap.group.selectionStyle.border,"color",null,e.primaryTitleColor),d(e.map.legend,"backgroundColor",e),g(e),e}function p(e){var t=e["chart:common:axis"],n="color";T([t.grid,t.minorGrid],function(t,i){d(i,n,null,e.gridColor)}),T([t,t.tick,t.minorTick,t.label.font],function(t,i){d(i,n,null,e.axisColor)}),d(t.title.font,n,null,e.secondaryTitleColor),d(e.gauge.scale.label.font,n,null,e.axisColor),d(e.gauge.scale.tick,n,null,e.backgroundColor),d(e.gauge.scale.minorTick,n,null,e.backgroundColor),d(e.rangeSelector.scale.label.font,n,null,e.axisColor)}function g(e){var t=e.map;T(["area","line","marker"],function(e,n){h(t,"layer:"+n,null,t.layer)}),T(["dot","bubble","pie","image"],function(e,n){h(t,"layer:marker:"+n,null,t["layer:marker"])})}function _(e){var t=++E;e._cache=t,A[t]=e}function m(e){delete A[e._cache]}function v(){return T(A,function(){this.refresh()}),this}t.getTheme=i,t.currentTheme=r,t.registerTheme=u,t.registerThemeSchemeAlias=c,t.addCacheItem=_,t.removeCacheItem=m,t.refreshTheme=v;var y,b=n(0),x=n(3),w=n(14),C=n(36),k={},S={},D={},I=b.extend,T=x.each,O=null,E=0,A={}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.default=void 0;var c=i(n(2)),d=i(n(5)),h=i(n(390)),f=i(n(17)),p=n(0),g=n(3),_=n(4),m=n(1),v=i(n(207)),y=n(19),b=n(69),x=n(70),w=i(n(210)),C=n(6),k="dxItemDeleting",S=-1,D=function(e){return e!==S},I=h.default.inherit({_setOptionsByReference:function(){this.callBase(),(0,p.extend)(this._optionsByReference,{selectedItem:!0})},_getDefaultOptions:function(){return(0,p.extend)(this.callBase(),{selectionMode:"none",selectionRequired:!1,selectionByClick:!0,selectedItems:[],selectedItemKeys:[],maxFilterLengthInRequest:1500,keyExpr:null,selectedIndex:S,selectedItem:null,onSelectionChanged:null,onItemReordered:null,onItemDeleting:null,onItemDeleted:null})},ctor:function(e,t){this._userOptions=t||{},this.callBase(e,t)},_init:function(){this._initEditStrategy(),this.callBase(),this._initKeyGetter(),this._initSelectionModule()},_initKeyGetter:function(){this._keyGetter=(0,y.compileGetter)(this.option("keyExpr"))},_getKeysByItems:function(e){return this._editStrategy.getKeysByItems(e)},_getItemsByKeys:function(e,t){return this._editStrategy.getItemsByKeys(e,t)},_getKeyByIndex:function(e){return this._editStrategy.getKeyByIndex(e)},_getIndexByKey:function(e){return this._editStrategy.getIndexByKey(e)},_getIndexByItemData:function(e){return this._editStrategy.getIndexByItemData(e)},_isKeySpecified:function(){return!(!this._dataSource||!this._dataSource.key())},_getCombinedFilter:function(){return this._dataSource&&this._dataSource.filter()},key:function(){return this.option("keyExpr")?this.option("keyExpr"):this._dataSource&&this._dataSource.key()},keyOf:function(e){var t=e,n=this._dataSource&&this._dataSource.store();return this.option("keyExpr")?t=this._keyGetter(e):n&&(t=n.keyOf(e)),t},_nullValueSelectionSupported:function(){return!1},_initSelectionModule:function(){var e=this,t=e._editStrategy.itemsGetter;this._selection=new w.default({allowNullValue:this._nullValueSelectionSupported(),mode:this.option("selectionMode"),maxFilterLengthInRequest:this.option("maxFilterLengthInRequest"),equalByReference:!this._isKeySpecified(),onSelectionChanged:function(t){(t.addedItemKeys.length||t.removedItemKeys.length)&&(e.option("selectedItems",e._getItemsByKeys(t.selectedItemKeys,t.selectedItems)),e._updateSelectedItems(t))},filter:e._getCombinedFilter.bind(e),totalCount:function(){var t=e.option("items"),n=e._dataSource;return n&&n.totalCount()>=0?n.totalCount():t.length},key:e.key.bind(e),keyOf:e.keyOf.bind(e),load:function(t){if(e._dataSource){var n=e._dataSource.loadOptions();t.customQueryParams=n.customQueryParams,t.userData=e._dataSource._userData}var i=e._dataSource&&e._dataSource.store();return i?i.load(t).done(function(t){if(!e._disposed){var n=(0,x.normalizeLoadResult)(t).data;e._dataSource._applyMapFunction(n)}}):(new C.Deferred).resolve(this.plainItems())},dataFields:function(){return e._dataSource&&e._dataSource.select()},plainItems:t.bind(e._editStrategy)})},_initEditStrategy:function(){var e=v.default;this._editStrategy=new e(this)},_getSelectedItemIndices:function(e){var t=this,n=[];return e=e||this._selection.getSelectedItemKeys(),t._editStrategy.beginCache(),(0,g.each)(e,function(e,i){var o=t._getIndexByKey(i);D(o)&&n.push(o)}),t._editStrategy.endCache(),n},_initMarkup:function(){var e=this;this._rendering=!0,this._dataSource&&this._dataSource.isLoading()||this._syncSelectionOptions().done(function(){return e._normalizeSelectedItems()}),this.callBase()},_render:function(){this.callBase(),this._rendering=!1},_fireContentReadyAction:function(){this._rendering=!1,this._rendered=!0,this.callBase.apply(this,arguments)},_syncSelectionOptions:function(e){e=e||this._chooseSelectOption();var t,n,i,o;switch(e){case"selectedIndex":t=this._editStrategy.getItemDataByIndex(this.option("selectedIndex")),(0,m.isDefined)(t)?(this._setOptionWithoutOptionChange("selectedItems",[t]),this._setOptionWithoutOptionChange("selectedItem",t),this._setOptionWithoutOptionChange("selectedItemKeys",this._editStrategy.getKeysByItems([t]))):(this._setOptionWithoutOptionChange("selectedItems",[]),
this._setOptionWithoutOptionChange("selectedItemKeys",[]),this._setOptionWithoutOptionChange("selectedItem",null));break;case"selectedItems":if(o=this.option("selectedItems")||[],n=o.length?this._editStrategy.getIndexByItemData(o[0]):S,this.option("selectionRequired")&&!D(n))return this._syncSelectionOptions("selectedIndex");this._setOptionWithoutOptionChange("selectedItem",o[0]),this._setOptionWithoutOptionChange("selectedIndex",n),this._setOptionWithoutOptionChange("selectedItemKeys",this._editStrategy.getKeysByItems(o));break;case"selectedItem":if(t=this.option("selectedItem"),n=this._editStrategy.getIndexByItemData(t),this.option("selectionRequired")&&!D(n))return this._syncSelectionOptions("selectedIndex");(0,m.isDefined)(t)?(this._setOptionWithoutOptionChange("selectedItems",[t]),this._setOptionWithoutOptionChange("selectedIndex",n),this._setOptionWithoutOptionChange("selectedItemKeys",this._editStrategy.getKeysByItems([t]))):(this._setOptionWithoutOptionChange("selectedItems",[]),this._setOptionWithoutOptionChange("selectedItemKeys",[]),this._setOptionWithoutOptionChange("selectedIndex",S));break;case"selectedItemKeys":if(i=this.option("selectedItemKeys"),this.option("selectionRequired")){var a=this._getIndexByKey(i[0]);if(!D(a))return this._syncSelectionOptions("selectedIndex")}return this._selection.setSelection(i)}return(new C.Deferred).resolve().promise()},_chooseSelectOption:function(){var e="selectedIndex",t=function(e){var t=this.option(e),n=(0,m.isDefined)(t)&&t.length;return n||e in this._userOptions}.bind(this);return t("selectedItems")?e="selectedItems":t("selectedItem")?e="selectedItem":t("selectedItemKeys")&&(e="selectedItemKeys"),e},_compareKeys:function(e,t){if(e.length!==t.length)return!1;for(var n=0;n<t.length;n++)if(e[n]!==t[n])return!1;return!0},_normalizeSelectedItems:function(){if("none"===this.option("selectionMode"))this._setOptionWithoutOptionChange("selectedItems",[]),this._syncSelectionOptions("selectedItems");else if("single"===this.option("selectionMode")){var e=this.option("selectedItems");if(e.length>1||!e.length&&this.option("selectionRequired")&&this.option("items")&&this.option("items").length){var t=this._selection.getSelectedItems(),n=void 0===e[0]?t[0]:e[0];return void 0===n&&(n=this._editStrategy.itemsGetter()[0]),this.option("grouped")&&n&&n.items&&(n.items=[n.items[0]]),this._selection.setSelection(this._getKeysByItems([n])),this._setOptionWithoutOptionChange("selectedItems",[n]),this._syncSelectionOptions("selectedItems")}this._selection.setSelection(this._getKeysByItems(e))}else{var i=this._getKeysByItems(this.option("selectedItems")),o=this._selection.getSelectedItemKeys();this._compareKeys(o,i)||this._selection.setSelection(i)}return(new C.Deferred).resolve().promise()},_itemClickHandler:function(e){this._createAction(function(e){this._itemSelectHandler(e.event)}.bind(this),{validatingTargetName:"itemElement"})({itemElement:(0,c.default)(e.currentTarget),event:e}),this.callBase.apply(this,arguments)},_itemSelectHandler:function(e){if(this.option("selectionByClick")){var t=e.currentTarget;this.isItemSelected(t)?this.unselectItem(e.currentTarget):this.selectItem(e.currentTarget)}},_selectedItemElement:function(e){return this._itemElements().eq(e)},_postprocessRenderItem:function(e){if("none"!==this.option("selectionMode")){var t=(0,c.default)(e.itemElement),n=this._editStrategy.getNormalizedIndex(t),i=this._isItemSelected(n);this._processSelectableItem(t,i)}},_processSelectableItem:function(e,t){e.toggleClass(this._selectedItemClass(),t),this._setAriaSelected(e,String(t))},_updateSelectedItems:function(e){var t=this,n=e.addedItemKeys,i=e.removedItemKeys;if(t._rendered&&(n.length||i.length)){var o=t._selectionChangePromise;if(!t._rendering){var a,r=[],s=[];t._editStrategy.beginCache();for(var l=0;l<n.length;l++)a=t._getIndexByKey(n[l]),r.push(a),t._addSelection(a);for(var u=0;u<i.length;u++)a=t._getIndexByKey(i[u]),s.push(a),t._removeSelection(a);t._editStrategy.endCache(),t._updateSelection(r,s)}(0,C.when)(o).done(function(){t._fireSelectionChangeEvent(e.addedItems,e.removedItems)})}},_fireSelectionChangeEvent:function(e,t){this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})({addedItems:e,removedItems:t})},_updateSelection:_.noop,_setAriaSelected:function(e,t){this.setAria("selected",t,e)},_removeSelection:function(e){var t=this._editStrategy.getItemElement(e);D(e)&&(this._processSelectableItem(t,!1),d.default.triggerHandler(t,"stateChanged",!1))},_addSelection:function(e){var t=this._editStrategy.getItemElement(e);D(e)&&(this._processSelectableItem(t,!0),d.default.triggerHandler(t,"stateChanged",!0))},_isItemSelected:function(e){var t=this._getKeyByIndex(e);return this._selection.isItemSelected(t)},_optionChanged:function(e){var t=this;switch(e.name){case"selectionMode":this._invalidate();break;case"dataSource":(!e.value||Array.isArray(e.value)&&!e.value.length)&&this.option("selectedItemKeys",[]),this.callBase(e);break;case"selectedIndex":case"selectedItem":case"selectedItems":case"selectedItemKeys":this._syncSelectionOptions(e.name).done(function(){return t._normalizeSelectedItems()});break;case"keyExpr":this._initKeyGetter();break;case"selectionRequired":this._normalizeSelectedItems();break;case"selectionByClick":case"onSelectionChanged":case"onItemDeleting":case"onItemDeleted":case"onItemReordered":case"maxFilterLengthInRequest":break;default:this.callBase(e)}},_clearSelectedItems:function(){this._setOptionWithoutOptionChange("selectedItems",[]),this._syncSelectionOptions("selectedItems")},_waitDeletingPrepare:function(e){if(e.data(k))return(new C.Deferred).resolve().promise();e.data(k,!0);var t=new C.Deferred,n={cancel:!1},i=this._itemEventHandler(e,"onItemDeleting",n,{excludeValidators:["disabled","readOnly"]});return(0,C.when)(i).always(function(o){var a=!i,r=!a&&"resolved"===i.state(),s=!!arguments.length,l=a||r&&!s||r&&o;(0,C.when)((0,C.fromPromise)(n.cancel)).always(function(){e.data(k,!1)}).done(function(e){l&&!e?t.resolve():t.reject()}).fail(t.reject)}.bind(this)),t.promise()},_deleteItemFromDS:function(e){if(!this._dataSource)return(new C.Deferred).resolve().promise();var t=new C.Deferred,n=this.option("disabled"),i=this._dataSource.store();if(this.option("disabled",!0),!i.remove)throw f.default.Error("E1011");return i.remove(i.keyOf(this._getItemData(e))).done(function(e){void 0!==e?t.resolve():t.reject()}).fail(function(){t.reject()}),t.always(function(){this.option("disabled",n)}.bind(this)),t},_tryRefreshLastPage:function(){var e=new C.Deferred;return this._isLastPage()||this.option("grouped")?e.resolve():this._refreshLastPage().done(function(){e.resolve()}),e.promise()},_refreshLastPage:function(){return this._expectLastItemLoading(),this._dataSource.load()},_updateSelectionAfterDelete:function(e){var t=this._getKeyByIndex(e);this._selection.deselect([t])},_updateIndicesAfterIndex:function(e){for(var t=this._itemElements(),n=e+1;n<t.length;n++)(0,c.default)(t[n]).data(this._itemIndexKey(),n-1)},_simulateOptionChange:function(e){var t=this.option(e);t instanceof b.DataSource||this._optionChangedAction({name:e,fullName:e,value:t})},isItemSelected:function(e){return this._isItemSelected(this._editStrategy.getNormalizedIndex(e))},selectItem:function(e){if("none"!==this.option("selectionMode")){var t=this._editStrategy.getNormalizedIndex(e);if(D(t)){var n=this._getKeyByIndex(t);if(!this._selection.isItemSelected(n))if("single"===this.option("selectionMode"))this._selection.setSelection([n]);else{var i=this.option("selectedItemKeys")||[];this._selection.setSelection([].concat(o(i),[n]))}}}},unselectItem:function(e){var t=this._editStrategy.getNormalizedIndex(e);if(D(t)){var n=this._selection.getSelectedItemKeys();if(!(this.option("selectionRequired")&&n.length<=1)){var i=this._getKeyByIndex(t);this._selection.isItemSelected(i)&&this._selection.deselect([i])}}},_deleteItemElementByIndex:function(e){this._updateSelectionAfterDelete(e),this._updateIndicesAfterIndex(e),this._editStrategy.deleteItemAtIndex(e)},_afterItemElementDeleted:function(e,t){var n=this._dataSource?"dataSource":"items";this._simulateOptionChange(n),this._itemEventHandler(e,"onItemDeleted",t,{beforeExecute:function(){e.remove()},excludeValidators:["disabled","readOnly"]}),this._renderEmptyMessage()},deleteItem:function(e){var t=this,n=new C.Deferred,i=this._editStrategy.getItemElement(e),o=this._editStrategy.getNormalizedIndex(e),a=this._itemResponseWaitClass();return D(o)?this._waitDeletingPrepare(i).done(function(){i.addClass(a);var e=t._extendActionArgs(i);t._deleteItemFromDS(i).done(function(){t._deleteItemElementByIndex(o),t._afterItemElementDeleted(i,e),t._tryRefreshLastPage().done(function(){n.resolveWith(t)})}).fail(function(){i.removeClass(a),n.rejectWith(t)})}).fail(function(){n.rejectWith(t)}):n.rejectWith(t),n.promise()},reorderItem:function(e,t){var n=new C.Deferred,i=this,o=this._editStrategy,a=o.getItemElement(e),r=o.getItemElement(t),s=o.getNormalizedIndex(e),l=o.getNormalizedIndex(t),u=this._dataSource?"dataSource":"items",c=D(s)&&D(l)&&s!==l;return c?n.resolveWith(this):n.rejectWith(this),n.promise().done(function(){r[o.itemPlacementFunc(s,l)](a),o.moveItemAtIndexToIndex(s,l),this._updateIndicesAfterIndex(s),i.option("selectedItems",i._getItemsByKeys(i._selection.getSelectedItemKeys(),i._selection.getSelectedItems())),"items"===u&&i._simulateOptionChange(u),i._itemEventHandler(a,"onItemReordered",{fromIndex:o.getIndex(s),toIndex:o.getIndex(l)},{excludeValidators:["disabled","readOnly"]})})}}),T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getImageContainer=t.getImageSourceType=void 0;var o=i(n(2)),a="dx-icon",r="dx-svg-icon",s=function(e){return!(!e||"string"!=typeof e)&&(/^\s*<svg[^>]*>(.|\r\n|\r|\n)*?<\/svg>\s*$/i.test(e)?"svg":/data:.*base64|\.|[^<\s]\//.test(e)?"image":/^[\w-_]+$/.test(e)?"dxIcon":!!/^\s?([\w-_]\s?)+$/.test(e)&&"fontIcon")};t.getImageSourceType=s;var l=function(e){switch(s(e)){case"image":return(0,o.default)("<img>").attr("src",e).addClass(a);case"fontIcon":return(0,o.default)("<i>").addClass("".concat(a," ").concat(e));case"dxIcon":return(0,o.default)("<i>").addClass("".concat(a," ").concat(a,"-").concat(e));case"svg":return(0,o.default)("<i>").addClass("".concat(a," ").concat(r)).append(e);default:return null}};t.getImageContainer=l},function(e,t,n){function i(e,t,n){function i(e){return"string"==typeof e?e.split("."):"number"==typeof e?[e]:e}e=i(e),t=i(t);var o=Math.max(e.length,t.length);isFinite(n)&&(o=Math.min(o,n));for(var a=0;a<o;a++){var r=parseInt(e[a]||0,10),s=parseInt(t[a]||0,10);if(r<s)return-1;if(r>s)return 1}return 0}t.compare=i},function(e,t){e.exports=window.ko},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return+e||0}function a(e){var t=S(e)===f,n=!(0,c.isDate)(e)&&Date.parse(e);if((!n||b)&&t){var i=e.match(v);if(i){var a=new Date(o(i[1]),o(i[2]),o(i[3]));return a.setFullYear(o(i[1])),a.setMonth(o(i[2])-1),a.setDate(o(i[3])),a}}return(0,c.isNumeric)(n)?new Date(n):e}function r(e){var t=e.match(g);{if(t){var n=t[1],i=--t[3],a=t[5],r=0,s=0;r=o(t[14]),s=o(t[16]),"-"===t[13]&&(r=-r,s=-s);var l=o(t[6])-r,u=o(t[8])-s,c=o(t[10]),d=function(e){return e=e||"",o(e)*Math.pow(10,y-e.length)},h=d(t[11]);return t[12]?new Date(Date.UTC(n,i,a,l,u,c,h)):new Date(n,i,a,l,u,c,h)}if(t=e.match(_))return new Date(0,0,0,o(t[1]),o(t[2]),o(t[4]))}}t.default=void 0;var s=i(n(28)),l=n(242),u=i(n(243)),c=n(1),d=i(n(23)),h="number",f="yyyy/MM/dd",p="yyyy/MM/dd HH:mm:ss",g=/^(\d{4,})(-)?(\d{2})(-)?(\d{2})(?:T(\d{2})(:)?(\d{2})?(:)?(\d{2}(?:\.(\d{1,3})\d*)?)?)?(Z|([+-])(\d{2})(:)?(\d{2})?)?$/,_=/^(\d{2}):(\d{2})(:(\d{2}))?$/,m=["","yyyy","","MM","","dd","THH","","mm","","ss",".SSS"],v=/^(\d{4})\/(\d{2})\/(\d{2})$/,y=3,b=d.default.msie&&parseInt(d.default.version)<=11,x=function(e,t){var n;return(0,c.isString)(e)&&!t&&(n=r(e)),n||a(e)},w=function(e,t){var n=e.match(g),i="";if(n){for(var o=1;o<m.length;o++)n[o]&&(i+=m[o]||n[o]);return"Z"===n[12]&&(i+="'Z'"),n[14]&&(i+=n[15]?"xxx":n[16]?"xx":"x"),i}if(n=e.match(_))return n[3]?"HH:mm:ss":"HH:mm"},C=function(e){return"number"==typeof e?new Date(e):x(e,!(0,s.default)().forceIsoDateParsing)},k=function(e,t){return t?(0,c.isDate)(e)?t===h?e&&e.valueOf?e.valueOf():null:(0,l.getFormatter)(t,u.default)(e):null:e},S=function(e){if("number"==typeof e)return h;if((0,c.isString)(e)){var t;return(0,s.default)().forceIsoDateParsing&&(t=w(e)),t?t:e.indexOf(":")>=0?p:f}return e?null:void 0},D={dateParser:x,deserializeDate:C,serializeDate:k,getDateSerializationFormat:S};t.default=D,e.exports=t.default},function(e,t){e.exports=window.DevExpress.Quill},function(e,t,n){t.default=void 0;var i=function(e){var t,n=function(){return t=e.apply(this,arguments),n=function(){return t},t};return function(){return n.apply(this,arguments)}},o=i;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(28)),r=i(n(22)),s=i(n(114)),l=i(n(117)),u=n(360),c=n(136),d=n(39),h=n(3),f=n(0),p=n(18),g=n(4),_=n(12),m=n(1),v=n(7),y=n(119),b=l.default.abstract,x=l.default.inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{width:void 0,height:void 0,rtlEnabled:(0,a.default)().rtlEnabled,elementAttr:{},disabled:!1,integrationOptions:{}},this._useTemplates()?u.TemplateManager.createDefaultOptions():{})},ctor:function(e,t){this._createElement(e),(0,c.attachInstanceToElement)(this._$element,this,this._dispose),this.callBase(t)},_createElement:function(e){this._$element=(0,o.default)(e)},_getSynchronizableOptionsForCreateComponent:function(){return["rtlEnabled","disabled","templatesRenderAsynchronously"]},_visibilityChanged:b,_dimensionChanged:b,_init:function(){this.callBase(),this._attachWindowResizeCallback(),this._initTemplateManager()},_setOptionsByDevice:function(e){this.callBase([].concat(this.constructor._classCustomRules||[],e||[]))},_isInitialOptionValue:function(e){var t=this.constructor._classCustomRules&&Object.prototype.hasOwnProperty.call(this._convertRulesToOptions(this.constructor._classCustomRules),e);return!t&&this.callBase(e)},_attachWindowResizeCallback:function(){if(this._isDimensionChangeSupported()){var e=this._windowResizeCallBack=this._dimensionChanged.bind(this);s.default.add(e)}},_isDimensionChangeSupported:function(){return this._dimensionChanged!==b},_renderComponent:function(){this._initMarkup(),(0,v.hasWindow)()&&this._render()},_initMarkup:function(){var e=this.option()||{},t=e.rtlEnabled;this._renderElementAttributes(),this._toggleRTLDirection(t),this._renderVisibilityChange(),this._renderDimensions()},_render:function(){this._attachVisibilityChangeHandlers()},_renderElementAttributes:function(){var e=this.option()||{},t=e.elementAttr,n=(0,f.extend)({},t),i=n.class;delete n.class,this.$element().attr(n).addClass(i)},_renderVisibilityChange:function(){if(this._isDimensionChangeSupported()&&this._attachDimensionChangeHandlers(),this._isVisibilityChangeSupported()){var e=this.$element();e.addClass("dx-visibility-change-handler")}},_renderDimensions:function(){var e=this.$element(),t=e.get(0),n=this._getOptionValue("width",t),i=this._getOptionValue("height",t);this._isCssUpdateRequired(t,i,n)&&e.css({width:null===n?"":n,height:null===i?"":i})},_isCssUpdateRequired:function(e,t,n){return!!((0,m.isDefined)(n)||(0,m.isDefined)(t)||e.style.width||e.style.height)},_attachDimensionChangeHandlers:function(){var e=this,t=this.$element(),n="".concat(this.NAME,"VisibilityChange");y.resize.off(t,{namespace:n}),y.resize.on(t,function(){return e._dimensionChanged()},{namespace:n})},_attachVisibilityChangeHandlers:function(){var e=this;if(this._isVisibilityChangeSupported()){var t=this.$element(),n="".concat(this.NAME,"VisibilityChange");this._isHidden=!this._isVisible(),y.visibility.off(t,{namespace:n}),y.visibility.on(t,function(){return e._checkVisibilityChanged("shown")},function(){return e._checkVisibilityChanged("hiding")},{namespace:n})}},_isVisible:function(){var e=this.$element();return e.is(":visible")},_checkVisibilityChanged:function(e){var t=this._isVisible();t&&("hiding"!==e||this._isHidden?"shown"===e&&this._isHidden&&(this._isHidden=!1,this._visibilityChanged(!0)):(this._visibilityChanged(!1),this._isHidden=!0))},_isVisibilityChangeSupported:function(){return this._visibilityChanged!==b&&(0,v.hasWindow)()},_clean:g.noop,_modelByElement:function(){var e=this.option(),t=e.modelByElement,n=this.$element();return t?t(n):void 0},_invalidate:function(){if(this._isUpdateAllowed())throw r.default.Error("E0007");this._requireRefresh=!0},_refresh:function(){this._clean(),this._renderComponent()},_dispose:function(){this._templateManager&&this._templateManager.dispose(),this.callBase(),this._clean(),this._detachWindowResizeCallback()},_detachWindowResizeCallback:function(){this._isDimensionChangeSupported()&&s.default.remove(this._windowResizeCallBack)},_toggleRTLDirection:function(e){var t=this.$element();t.toggleClass("dx-rtl",e)},_createComponent:function(e,t){var n=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},a=(0,g.grep)(this._getSynchronizableOptionsForCreateComponent(),function(e){return!(e in i)}),r=this.option(),s=r.integrationOptions,l=this.option(),u=l.nestedComponentOptions;u=u||g.noop;var c=(0,f.extend)({integrationOptions:s},u(this));a.forEach(function(e){return c[e]=n.option(e)}),this._extendConfig(i,c);var d=void 0;if((0,m.isString)(t)){var h=(0,o.default)(e)[t](i);d=h[t]("instance")}else e&&(d=t.getInstance(e),d?d.option(i):d=new t(e,i));if(d){var p=function(e){var t=e.name,n=e.value;(0,_.inArray)(t,a)>=0&&d.option(t,n)};this.on("optionChanged",p),d.on("disposing",function(){return n.off("optionChanged",p)})}return d},_extendConfig:function(e,t){(0,h.each)(t,function(t,n){!Object.prototype.hasOwnProperty.call(e,t)&&(e[t]=n)})},_defaultActionConfig:function(){var e=this.$element(),t=this._modelByElement(e);return(0,f.extend)(this.callBase(),{context:t})},_defaultActionArgs:function(){var e=this.$element(),t=this._modelByElement(e),n=this.element();return(0,f.extend)(this.callBase(),{element:n,model:t})},_optionChanged:function(e){switch(e.name){case"width":case"height":this._renderDimensions();break;case"rtlEnabled":this._invalidate();break;case"elementAttr":this._renderElementAttributes();break;case"disabled":case"integrationOptions":break;default:this.callBase(e)}},_removeAttributes:function(e){for(var t=e.attributes,n=t.length-1;n>=0;n--){var i=t[n];if(i){var o=i.name;o.indexOf("aria-")&&o.indexOf("dx-")===-1&&"role"!==o&&"style"!==o&&"tabindex"!==o||e.removeAttribute(o)}}},_removeClasses:function(e){e.className=e.className.split(" ").filter(function(e){return 0!==e.lastIndexOf("dx-",0)}).join(" ")},_updateDOMComponent:function(e){e?this._renderComponent():this._requireRefresh&&(this._requireRefresh=!1,this._refresh())},endUpdate:function(){var e=this._isInitializingRequired();this.callBase(),this._isUpdateAllowed()&&this._updateDOMComponent(e)},$element:function(){return this._$element},element:function(){var e=this.$element();return(0,p.getPublicElement)(e)},dispose:function(){var e=this.$element().get(0);(0,d.cleanDataRecursive)(e,!0),e.textContent="",this._removeAttributes(e),this._removeClasses(e)},resetOption:function(e){if(this.callBase(e),"width"===e||"height"===e){var t=this.initialOption(e);!(0,m.isDefined)(t)&&this.$element().css(e,"")}},_getAnonymousTemplateName:function(){},_initTemplateManager:function(){if(!this._templateManager&&this._useTemplates()){var e=this.option(),t=e.integrationOptions,n=void 0===t?{}:t,i=n.createTemplate;this._templateManager=new u.TemplateManager(i,this._getAnonymousTemplateName()),this._initTemplates()}},_initTemplates:function(){var e=this,t=this._templateManager.extractTemplates(this.$element()),n=t.templates,i=t.anonymousTemplateMeta,o=this.option("integrationOptions.templates.".concat(i.name));n.forEach(function(t){var n=t.name,i=t.template;e._options.silent("integrationOptions.templates.".concat(n),i)}),i.name&&!o&&(this._options.silent("integrationOptions.templates.".concat(i.name),i.template),this._options.silent("_hasAnonymousTemplateContent",!0))},_getTemplateByOption:function(e){return this._getTemplate(this.option(e))},_getTemplate:function(e){var t=this.option("integrationOptions.templates"),n=this.option("templatesRenderAsynchronously"),i=this.option("integrationOptions.skipTemplates");return this._templateManager.getTemplate(e,t,{isAsyncTemplate:n,skipTemplates:i},this)},_saveTemplate:function(e,t){this._setOptionWithoutOptionChange("integrationOptions.templates."+e,this._templateManager._createTemplate(t))},_useTemplates:function(){return!0}});x.getInstance=function(e){return(0,c.getInstanceByElement)((0,o.default)(e),this)},x.defaultOptions=function(e){this._classCustomRules=this._classCustomRules||[],this._classCustomRules.push(e)};var w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=i(n(116)),r=function(e,t){var n={};"noBubble"in t&&(n.noBubble=t.noBubble),"bindType"in t&&(n.bindType=t.bindType),"delegateType"in t&&(n.delegateType=t.delegateType),(0,o.each)(["setup","teardown","add","remove","trigger","handle","_default","dispose"],function(e,i){t[i]&&(n[i]=function(){var e=[].slice.call(arguments);return e.unshift(this),t[i].apply(t,e)})}),a.default.fire(e,n)};r.callbacks=a.default;var s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.TemplateBase=t.renderedCallbacks=void 0;var s=i(n(2)),l=i(n(11)),u=i(n(27)),c=n(33),d=n(55),h=i(n(22)),f=(0,u.default)({syncStrategy:!0});t.renderedCallbacks=f;var p=function(){function e(){o(this,e)}return r(e,[{key:"render",value:function(e){e=e||{};var t=e.onRendered;delete e.onRendered;var n=this._renderCore(e);return this._ensureResultInContainer(n,e.container),f.fire(n,e.container),t&&t(),n}},{key:"_ensureResultInContainer",value:function(e,t){if(t){var n=(0,s.default)(t),i=(0,c.contains)(n.get(0),e.get(0));if(n.append(e),!i){var o=l.default.getBody().contains(n.get(0));o&&(0,d.triggerShownEvent)(e)}}}},{key:"_renderCore",value:function(){throw h.default.Error("E0001")}}]),e}();t.TemplateBase=p},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(272)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.DataSource=void 0;var o=i(n(15)),a=n(0),r=n(4),s=n(3),l=n(1),u=i(n(41)),c=n(90),d=i(n(123)),h=n(92),f=i(n(34)),p=n(12),g=n(252),_=n(6),m=i(n(391)),v=n(70),y=o.default.inherit({ctor:function(e){var t,n=this;e=(0,v.normalizeDataSourceOptions)(e),this._eventsStrategy=new h.EventsStrategy(this);var i=0!==e.pushAggregationTimeout?u.default.throttleChanges(this._onPush,function(){return void 0===e.pushAggregationTimeout?5*n._changedTime:e.pushAggregationTimeout}):this._onPush;this._changedTime=0,this._onPushHandler=function(e){n._aggregationTimeoutId=i.call(n,e)},this._store=e.store,this._store.on("push",this._onPushHandler),this._storeLoadOptions=this._extractLoadOptions(e),this._mapFunc=e.map,this._postProcessFunc=e.postProcess,this._pageIndex=void 0!==e.pageIndex?e.pageIndex:0,this._pageSize=void 0!==e.pageSize?e.pageSize:20,this._loadingCount=0,this._loadQueue=this._createLoadQueue(),this._searchValue="searchValue"in e?e.searchValue:null,this._searchOperation=e.searchOperation||"contains",this._searchExpr=e.searchExpr,this._paginate=e.paginate,this._reshapeOnPush=null!==(t=e.reshapeOnPush)&&void 0!==t&&t,(0,s.each)(["onChanged","onLoadError","onLoadingChanged","onCustomizeLoadResult","onCustomizeStoreLoadOptions"],function(t,i){i in e&&n.on(i.substr(2,1).toLowerCase()+i.substr(3),e[i])}),this._operationManager=new m.default,this._init()},_init:function(){this._items=[],this._userData={},this._totalCount=-1,this._isLoaded=!1,(0,l.isDefined)(this._paginate)||(this._paginate=!this.group()),this._isLastPage=!this._paginate},dispose:function(){var e;this._store.off("push",this._onPushHandler),this._eventsStrategy.dispose(),clearTimeout(this._aggregationTimeoutId),delete this._store,null===(e=this._delayedLoadTask)||void 0===e?void 0:e.abort(),this._operationManager.cancelAll(),this._disposed=!0},_extractLoadOptions:function(e){var t={},n=["sort","filter","select","group","requireTotalCount"],i=this._store._customLoadOptions();return i&&(n=n.concat(i)),(0,s.each)(n,function(){t[this]=e[this]}),t},loadOptions:function(){return this._storeLoadOptions},items:function(){return this._items},pageIndex:function(e){return(0,l.isNumeric)(e)?(this._pageIndex=e,void(this._isLastPage=!this._paginate)):this._pageIndex},paginate:function(e){return(0,l.isBoolean)(e)?void(this._paginate!==e&&(this._paginate=e,this.pageIndex(0))):this._paginate},pageSize:function(e){return(0,l.isNumeric)(e)?void(this._pageSize=e):this._pageSize},isLastPage:function(){return this._isLastPage},generateStoreLoadOptionAccessor:function(e){var t=this;return function(n){var i=(0,v.normalizeStoreLoadOptionAccessorArguments)(n);return void 0===i?t._storeLoadOptions[e]:void(t._storeLoadOptions[e]=i)}},sort:function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return this.generateStoreLoadOptionAccessor("sort")(t)},filter:function(){var e=(0,v.normalizeStoreLoadOptionAccessorArguments)(arguments);return void 0===e?this._storeLoadOptions.filter:(this._storeLoadOptions.filter=e,void this.pageIndex(0))},group:function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return this.generateStoreLoadOptionAccessor("group")(t)},select:function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return this.generateStoreLoadOptionAccessor("select")(t)},requireTotalCount:function(e){return(0,l.isBoolean)(e)?void(this._storeLoadOptions.requireTotalCount=e):this._storeLoadOptions.requireTotalCount},searchValue:function(e){return arguments.length<1?this._searchValue:(this._searchValue=e,void this.pageIndex(0))},searchOperation:function(e){return(0,l.isString)(e)?(this._searchOperation=e,void this.pageIndex(0)):this._searchOperation},searchExpr:function(e){var t=arguments.length;return 0===t?this._searchExpr:(t>1&&(e=[].slice.call(arguments)),this._searchExpr=e,void this.pageIndex(0))},store:function(){return this._store},key:function(){var e;return null===(e=this._store)||void 0===e?void 0:e.key()},totalCount:function(){return this._totalCount},isLoaded:function(){return this._isLoaded},isLoading:function(){return this._loadingCount>0},beginLoading:function(){this._changeLoadingCount(1)},endLoading:function(){this._changeLoadingCount(-1)},_createLoadQueue:function(){return(0,g.create)()},_changeLoadingCount:function(e){var t=this.isLoading();this._loadingCount+=e;var n=this.isLoading();t^n&&this._eventsStrategy.fireEvent("loadingChanged",[n])},_scheduleLoadCallbacks:function(e){var t=this;this.beginLoading(),e.always(function(){t.endLoading()})},_scheduleFailCallbacks:function(e){var t=this;e.fail(function(){for(var e=arguments.length,n=new Array(e),i=0;i<e;i++)n[i]=arguments[i];n[0]!==v.CANCELED_TOKEN&&t._eventsStrategy.fireEvent("loadError",n)})},_fireChanged:function(e){var t=new Date;this._eventsStrategy.fireEvent("changed",e),this._changedTime=new Date-t},_scheduleChangedCallbacks:function(e){var t=this;e.done(function(){return t._fireChanged()})},loadSingle:function(e,t){var n=this,i=new _.Deferred,o=this.key(),a=this._store,r=this._createStoreLoadOptions(),s=function(e){!(0,l.isDefined)(e)||(0,p.isEmpty)(e)?i.reject(new f.default.errors.Error("E4009")):(Array.isArray(e)||(e=[e]),i.resolve(n._applyMapFunction(e)[0]))};this._scheduleFailCallbacks(i),arguments.length<2&&(t=e,e=o),delete r.skip,delete r.group,delete r.refresh,delete r.pageIndex,delete r.searchString;var u=function(){return a instanceof d.default&&!a._byKeyViaLoad()};return function(){return e===o||u()?a.byKey(t,r):(r.take=1,r.filter=r.filter?[r.filter,[e,t]]:[e,t],a.load(r))}().fail(i.reject).done(s),i.promise()},load:function(){var e=this,t=new _.Deferred,n=function(){if(!e._disposed&&(0,v.isPending)(t))return e._loadFromStore(i,t)};this._scheduleLoadCallbacks(t),this._scheduleFailCallbacks(t),this._scheduleChangedCallbacks(t);var i=this._createLoadOperation(t);return this._eventsStrategy.fireEvent("customizeStoreLoadOptions",[i]),this._loadQueue.add(function(){return"number"==typeof i.delay?e._delayedLoadTask=(0,r.executeAsync)(n,i.delay):n(),t.promise()}),t.promise({operationId:i.operationId})},_onPush:function(e){var t=this;if(this._reshapeOnPush)this.load();else{this._eventsStrategy.fireEvent("changing",[{changes:e}]);var n=this.group(),i=this.items(),o=0,a=this.paginate()||n?e.filter(function(e){return"update"===e.type}):e;n&&(o=Array.isArray(n)?n.length:1),this._mapFunc&&a.forEach(function(e){"insert"===e.type&&(e.data=t._mapFunc(e.data))}),(0,c.applyBatch)({keyInfo:this.store(),data:i,changes:a,groupCount:o,useInsertIndex:!0}),this._fireChanged([{changes:e}])}},_createLoadOperation:function(e){var t=this,n=this._operationManager.add(e),i=this._createStoreLoadOptions();return e.always(function(){return t._operationManager.remove(n)}),{operationId:n,storeLoadOptions:i}},reload:function(){var e=this.store();return e instanceof d.default&&e.clearRawDataCache(),this._init(),this.load()},cancel:function(e){return this._operationManager.cancel(e)},cancelAll:function(){return this._operationManager.cancelAll()},_addSearchOptions:function(e){this._disposed||(this.store()._useDefaultSearch?this._addSearchFilter(e):(e.searchOperation=this._searchOperation,e.searchValue=this._searchValue,e.searchExpr=this._searchExpr))},_createStoreLoadOptions:function(){var e=(0,a.extend)({},this._storeLoadOptions);return this._addSearchOptions(e),this._paginate&&this._pageSize&&(e.skip=this._pageIndex*this._pageSize,e.take=this._pageSize),e.userData=this._userData,e},_addSearchFilter:function(e){var t=this._searchValue,n=this._searchOperation,i=this._searchExpr,o=[];t&&(i||(i="this"),Array.isArray(i)||(i=[i]),(0,s.each)(i,function(e,i){o.length&&o.push("or"),o.push([i,n,t])}),e.filter?e.filter=[o,e.filter]:e.filter=o)},_loadFromStore:function(e,t){var n=this,i=function(i,o){if(!n._disposed&&(0,v.isPending)(t)){var r=(0,a.extend)((0,v.normalizeLoadResult)(i,o),e);n._eventsStrategy.fireEvent("customizeLoadResult",[r]),(0,_.when)(r.data).done(function(e){r.data=e,n._processStoreLoadResult(r,t)}).fail(t.reject)}};return e.data?(new _.Deferred).resolve(e.data).done(i):this.store().load(e.storeLoadOptions).done(i).fail(t.reject)},_processStoreLoadResult:function(e,t){var n=this,i=e.data,o=e.extra,a=e.storeLoadOptions,r=function(){return n._isLoaded=!0,n._totalCount=isFinite(o.totalCount)?o.totalCount:-1,t.resolve(i,o)},s=function(){n.store().totalCount(a).done(function(e){o.totalCount=e,r()}).fail(t.reject)};this._disposed||(i=this._applyPostProcessFunction(this._applyMapFunction(i)),(0,l.isPlainObject)(o)||(o={}),this._items=i,(!i.length||!this._paginate||this._pageSize&&i.length<this._pageSize)&&(this._isLastPage=!0),a.requireTotalCount&&!isFinite(o.totalCount)?s():r())},_applyMapFunction:function(e){return this._mapFunc?(0,v.mapDataRespectingGrouping)(e,this._mapFunc,this.group()):e},_applyPostProcessFunction:function(e){return this._postProcessFunc?this._postProcessFunc(e):e},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this}});t.DataSource=y},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){
var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?o(Object(n),!0).forEach(function(t){r(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):o(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function s(e,t){if(null==e)return{};var n,i,o=l(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function l(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}t.normalizeDataSourceOptions=t.normalizeLoadResult=t.mapDataRespectingGrouping=t.normalizeStoreLoadOptionAccessorArguments=t.isPending=t.CANCELED_TOKEN=void 0;var u=i(n(76)),c=i(n(102)),d=i(n(81)),h=n(3),f=i(n(123)),p=n(0),g=n(1),_=i(n(41)),m="canceled";t.CANCELED_TOKEN=m;var v=function(e){return"pending"===e.state()};t.isPending=v;var y=function(e){switch(e.length){case 0:return;case 1:return e[0]}return[].slice.call(e)};t.normalizeStoreLoadOptionAccessorArguments=y;var b=function(e,t,n){return(0,h.map)(e,function(e){var i=(e.items,s(e,["items"]));return a(a({},i),{},{items:x(e.items,t-1,n)})})},x=function(e,t,n){return Array.isArray(e)?t?b(e,t,n):(0,h.map)(e,n):e},w=function(e,t,n){var i=n?_.default.normalizeSortingInfo(n).length:0;return x(e,i,t)};t.mapDataRespectingGrouping=w;var C=function(e,t){var n;return null!==(n=e)&&void 0!==n&&n.data&&(t=e,e=e.data),Array.isArray(e)||(e=[e]),{data:e,extra:t}};t.normalizeLoadResult=C;var k=function(e){var t={};return(0,h.each)(["useDefaultSearch","key","load","loadMode","cacheRawData","byKey","lookup","totalCount","insert","update","remove"],function(){t[this]=e[this],delete e[this]}),new f.default(t)},S=function(e){var t=e.type;return delete e.type,c.default.create(t,e)},D=function(e,t){return new f.default({load:function(){return u.default.sendRequest({url:e,dataType:"json"})},loadMode:null===t||void 0===t?void 0:t.fromUrlLoadMode})},I=function(e,t){var n;return"string"==typeof e&&(e={paginate:!1,store:D(e,t)}),void 0===e&&(e=[]),e=Array.isArray(e)||e instanceof c.default?{store:e}:(0,p.extend)({},e),void 0===e.store&&(e.store=[]),n=e.store,"load"in e?n=k(e):Array.isArray(n)?n=new d.default(n):(0,g.isPlainObject)(n)&&(n=S((0,p.extend)({},n))),e.store=n,e};t.normalizeDataSourceOptions=I},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.BindableTemplate=void 0;var g=o(n(2)),_=n(67),m=o(n(5)),v=o(n(138)),y=n(1),b=function(){var e=function(e,t,n){return t(function(){return e},n)},t=function(e,t,n,i,o){var a={},r=n.slice(),s=n.map(function(n){var s=i[n];return t(s?function(){return s(e)}:function(){return e[n]},function(e){if(a[n]=e,r.length){var t=r.indexOf(n);t>=0&&r.splice(t,1)}r.length||o(a)})});return function(){s.forEach(function(e){return e()})}};return function(n,i,o,a,r){var s,l=e(n,i,function(e){return s&&s(),(0,y.isPrimitive)(e)?void r(e):void(s=t(e,i,o,a,r))});return function(){s&&s(),l&&l()}}}(),x=function(e){function t(e,i,o,r){var s;return a(this,t),s=n.call(this),s._render=e,s._fields=i,s._fieldsMap=r||{},s._watchMethod=o,s}l(t,e);var n=c(t);return s(t,[{key:"_renderCore",value:function(e){var t=this,n=(0,g.default)(e.container),i=b(e.model,this._watchMethod,this._fields,this._fieldsMap,function(i){n.empty(),t._render(n,i,e.model)});return m.default.on(n,v.default,i),n.contents()}}]),t}(_.TemplateBase);t.BindableTemplate=x},function(e,t){e.exports=window.Globalize},function(e,t,n){n.d(t,"a",function(){return i});var i,o=n(74);!function(e){e.MONTH_DAYS=[31,28,31,30,31,30,31,31,30,31,30,31],e.ONE_DAY=864e5,e.MAXYEAR=9999,e.ORDINAL_BASE=new Date(Date.UTC(1970,0,1)),e.PY_WEEKDAYS=[6,0,1,2,3,4,5],e.getYearDay=function(t){var n=new Date(t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate());return Math.ceil((n.valueOf()-new Date(t.getUTCFullYear(),0,1).valueOf())/e.ONE_DAY)+1},e.isLeapYear=function(e){return e%4===0&&e%100!==0||e%400===0},e.isDate=function(e){return e instanceof Date},e.isValidDate=function(t){return e.isDate(t)&&!isNaN(t.getTime())},e.tzOffset=function(e){return 60*e.getTimezoneOffset()*1e3},e.daysBetween=function(t,n){var i=t.getTime()-e.tzOffset(t),o=n.getTime()-e.tzOffset(n),a=i-o;return Math.round(a/e.ONE_DAY)},e.toOrdinal=function(t){return e.daysBetween(t,e.ORDINAL_BASE)},e.fromOrdinal=function(t){return new Date(e.ORDINAL_BASE.getTime()+t*e.ONE_DAY)},e.getMonthDays=function(t){var n=t.getUTCMonth();return 1===n&&e.isLeapYear(t.getUTCFullYear())?29:e.MONTH_DAYS[n]},e.getWeekday=function(t){return e.PY_WEEKDAYS[t.getUTCDay()]},e.monthRange=function(t,n){var i=new Date(Date.UTC(t,n,1));return[e.getWeekday(i),e.getMonthDays(i)]},e.combine=function(e,t){return t=t||e,new Date(Date.UTC(e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate(),t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds()))},e.clone=function(e){var t=new Date(e.getTime());return t},e.cloneDates=function(t){for(var n=[],i=0;i<t.length;i++)n.push(e.clone(t[i]));return n},e.sort=function(e){e.sort(function(e,t){return e.getTime()-t.getTime()})},e.timeToUntilString=function(e,t){void 0===t&&(t=!0);var n=new Date(e);return[Object(o.i)(n.getUTCFullYear().toString(),4,"0"),Object(o.i)(n.getUTCMonth()+1,2,"0"),Object(o.i)(n.getUTCDate(),2,"0"),"T",Object(o.i)(n.getUTCHours(),2,"0"),Object(o.i)(n.getUTCMinutes(),2,"0"),Object(o.i)(n.getUTCSeconds(),2,"0"),t?"Z":""].join("")},e.untilStringToDate=function(e){var t=/^(\d{4})(\d{2})(\d{2})(T(\d{2})(\d{2})(\d{2})Z?)?$/,n=t.exec(e);if(!n)throw new Error("Invalid UNTIL value: "+e);return new Date(Date.UTC(parseInt(n[1],10),parseInt(n[2],10)-1,parseInt(n[3],10),parseInt(n[5],10)||0,parseInt(n[6],10)||0,parseInt(n[7],10)||0))}}(i||(i={})),t.b=i},function(e,t,n){function i(e,t,n){void 0===n&&(n=" ");var i=String(e);return t>>=0,i.length>t?String(i):(t-=i.length,t>n.length&&(n+=c(n,t/n.length)),n.slice(0,t)+String(i))}n.d(t,"f",function(){return a}),n.d(t,"e",function(){return r}),n.d(t,"g",function(){return s}),n.d(t,"d",function(){return l}),n.d(t,"k",function(){return u}),n.d(t,"l",function(){return c}),n.d(t,"n",function(){return d}),t.i=i,n.d(t,"m",function(){return h}),n.d(t,"j",function(){return f}),n.d(t,"a",function(){return p}),n.d(t,"b",function(){return g}),n.d(t,"h",function(){return _}),n.d(t,"c",function(){return m});var o=n(168),a=function(e){return null!==e&&void 0!==e},r=function(e){return"number"==typeof e},s=function(e){return o.a.indexOf(e)>=0},l=Array.isArray,u=function(e,t){void 0===t&&(t=e),1===arguments.length&&(t=e,e=0);for(var n=[],i=e;i<t;i++)n.push(i);return n},c=function(e,t){var n=0,i=[];if(l(e))for(;n<t;n++)i[n]=[].concat(e);else for(;n<t;n++)i[n]=e;return i},d=function(e){return l(e)?e:[e]},h=function(e,t,n){var i=e.split(t);return n?i.slice(0,n).concat([i.slice(n).join(t)]):i},f=function(e,t){var n=e%t;return n*t<0?n+t:n},p=function(e,t){return{div:Math.floor(e/t),mod:f(e,t)}},g=function(e){return!a(e)||0===e.length},_=function(e){return!g(e)},m=function(e,t){return _(e)&&e.indexOf(t)!==-1}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=(0,a.default)(e);return t.is(":visible")&&"hidden"!==t.css("visibility")&&"hidden"!==t.parents().css("visibility")}t.focused=t.tabbable=t.focusable=void 0;var a=i(n(2)),r=i(n(11)),s=function(e,t){if(!o(e))return!1;var n=e.nodeName.toLowerCase(),i=!isNaN(t),a=e.disabled,r=/^(input|select|textarea|button|object|iframe)$/.test(n),s="a"===n,l=!0,u=e.isContentEditable;return l=r||u?!a:s?e.href||i:i},l=function(e,t){return s(t,(0,a.default)(t).attr("tabIndex"))};t.focusable=l;var u=function(e,t){var n=(0,a.default)(t).attr("tabIndex");return(isNaN(n)||n>=0)&&s(t,n)};t.tabbable=u;var c=function(e){var t=(0,a.default)(e).get(0);return r.default.getActiveElement()===t};t.focused=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return(e.method||"GET").toUpperCase()}t.default=void 0;var a=n(6),r=i(n(11)),s=i(n(348)),l=n(7),u=n(0),c=n(1),d=i(n(77)),h=i(n(54)),f=(0,l.getWindow)(),p="success",g="error",_="timeout",m="nocontent",v="parsererror",y=function(e){return 200<=e&&e<300},b=function(e){return 204!==e},x=function(e){var t=[];for(var n in e){var i=e[n];void 0!==i&&(null===i&&(i=""),"function"==typeof i&&(i=i()),t.push(encodeURIComponent(n)+"="+encodeURIComponent(i)))}return t.join("&")},w=function(e){var t=r.default.createElement("script");for(var n in e)t[n]=e[n];return t},C=function(e){e.parentNode.removeChild(e)},k=function(e){return r.default.getHead().appendChild(e)},S=function(e){var t=w({text:e});k(t),C(t)},D=function(e){var t=w({src:e});return new d.default(function(e,n){var i={load:e,error:n},o=function(e){i[e.type](),C(t)};for(var a in i)r.default.listen(t,a,o);k(t)})},I=function(e){var t=e.dataType||"*",n="text/javascript, application/javascript, application/ecmascript, application/x-ecmascript",i={"*":"*/*",text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript",jsonp:n,script:n};return(0,u.extendFromObject)(i,e.accepts,!0),i[t]?i[t]+("*"!==t?", */*; q=0.01":""):i["*"]},T=function(e){var t;return e.data&&!e.upload&&"GET"!==o(e)&&(t="application/x-www-form-urlencoded;charset=utf-8"),e.contentType||t},O=function(e){return e.responseType&&"text"!==e.responseType||"string"!=typeof e.responseText?e.response:e.responseText},E=function(e,t,n){var i=O(t);switch(n){case"jsonp":S(i);break;case"script":S(i),e.resolve(i,p,t);break;case"json":try{e.resolve(JSON.parse(i),p,t)}catch(n){e.reject(t,v,n)}break;default:e.resolve(i,p,t)}},A=function(e){if(!(0,l.hasWindow)())return!0;var t=!1,n=r.default.createElement("a"),i=r.default.createElement("a");n.href=f.location.href;try{i.href=e,i.href=i.href,t=n.protocol+"//"+n.host!=i.protocol+"//"+i.host}catch(e){t=!0}return t},P=function(e,t){return e&&setTimeout(function(){t.customStatus=_,t.abort()},e)},R=function(e){if("jsonp"===e.dataType){var t=Math.random().toString().replace(/\D/g,""),n=e.jsonpCallback||"dxCallback"+Date.now()+"_"+t,i=e.jsonp||"callback";return e.data=e.data||{},e.data[i]=n,n}},M=function(e,t){var n=e.data,i="string"==typeof n,a=e.url||f.location.href;return i||e.cache||(n=n||{},n._=Date.now()),n&&!e.upload&&(i||(n=x(n)),"GET"===o(e)?(""!==n&&(a+=(a.indexOf("?")>-1?"&":"?")+n),n=null):t["Content-Type"]&&t["Content-Type"].indexOf("application/x-www-form-urlencoded")>-1&&(n=n.replace(/%20/g,"+"))),{url:a,parameters:n}},B=function(e){var t=e.headers||{};return t["Content-Type"]=t["Content-Type"]||T(e),t.Accept=t.Accept||I(e),e.crossDomain||t["X-Requested-With"]||(t["X-Requested-With"]="XMLHttpRequest"),t},V=function(e){var t,n=s.default.getXhr(),i=new a.Deferred,r=i.promise(),l=!(0,c.isDefined)(e.async)||e.async,u=e.dataType,d=e.timeout||0;e.crossDomain=A(e.url);var h="jsonp"===u||"script"===u;void 0===e.cache&&(e.cache=!h);var _=R(e),v=B(e),x=M(e,v),w=x.url,C=x.parameters;if(_&&(f[_]=function(e){i.resolve(e,p,n)}),e.crossDomain&&h){var k=function(){i.reject(n,g)},S=function(){"jsonp"!==u&&i.resolve(null,p,n)};return D(w).then(S,k),r}if(e.crossDomain&&!("withCredentials"in n))return i.reject(n,g),r;if(n.open(o(e),w,l,e.username,e.password),l&&(n.timeout=d,t=P(d,n,i)),n.onreadystatechange=function(e){4===n.readyState&&(clearTimeout(t),y(n.status)?b(n.status)?E(i,n,u):i.resolve(null,m,n):i.reject(n,n.customStatus||g))},e.upload&&(n.upload.onprogress=e.upload.onprogress,n.upload.onloadstart=e.upload.onloadstart,n.upload.onabort=e.upload.onabort),e.xhrFields)for(var I in e.xhrFields)n[I]=e.xhrFields[I];"arraybuffer"===e.responseType&&(n.responseType=e.responseType);for(var T in v)Object.prototype.hasOwnProperty.call(v,T)&&(0,c.isDefined)(v[T])&&n.setRequestHeader(T,v[T]);return e.beforeSend&&e.beforeSend(n),n.send(C),r.abort=function(){n.abort()},r},F=(0,h.default)({sendRequest:V});t.default=F,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(6),o=n(7),a=(0,o.hasWindow)()?(0,o.getWindow)().Promise:Promise;a||(a=function(e){var t=new i.Deferred;return e(t.resolve.bind(this),t.reject.bind(this)),t.promise()},a.resolve=function(e){return(new i.Deferred).resolve(e).promise()},a.reject=function(e){return(new i.Deferred).reject(e).promise()},a.all=function(e){return i.when.apply(this,e).then(function(){return[].slice.call(arguments)})});var r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return c}t.originalViewPort=o,t.changeCallback=t.value=void 0;var a=i(n(2)),r=i(n(49)),s=i(n(27)),l=r.default.add,u=(0,s.default)();t.changeCallback=u;var c=(0,a.default)(),d=function(){var e;return function(t){if(!arguments.length)return e;var n=(0,a.default)(t);c=n;var i=!!n.length,o=d();e=i?n:(0,a.default)("body"),u.fire(i?d():(0,a.default)(),o)}}();t.value=d,l(function(){d(".dx-viewport")})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(20)),r=i(n(45)),s=i(n(32)),l=i(n(54));n(107);var u=(0,l.default)({format:function(e,t){var n=(0,o.isString)(t)&&""!==t||(0,o.isPlainObject)(t)||(0,o.isFunction)(t),i=(0,o.isNumeric)(e)||(0,o.isDate)(e);return n&&i?(0,o.isFunction)(t)?t(e):((0,o.isString)(t)&&(t={type:t}),(0,o.isNumeric)(e)?r.default.format(e,t):(0,o.isDate)(e)?s.default.format(e,t):void 0):(0,o.isDefined)(e)?e.toString():""},getTimeFormat:function(e){return e?"longtime":"shorttime"},_normalizeFormat:function(e){return Array.isArray(e)?1===e.length?e[0]:function(t){return e.map(function(e){return s.default.format(t,e)}).join(" ")}:e},getDateFormatByDifferences:function(e,t){var n=[],i=t&&e.millisecond&&!(e.year||e.month||e.day);if(i){var o=function(e){return e.getSeconds()+e.getMilliseconds()/1e3+"s"};n.push(o)}else e.millisecond&&n.push("millisecond");if((e.hour||e.minute||!i&&e.second)&&n.unshift(this.getTimeFormat(e.second)),e.year&&e.month&&e.day)return t&&"month"===t?"monthandyear":(n.unshift("shortdate"),this._normalizeFormat(n));if(e.year&&e.month)return"monthandyear";if(e.year&&e.quarter)return"quarterandyear";if(e.year)return"year";if(e.quarter)return"quarter";if(e.month&&e.day){if(t){var a=function(e){return s.default.getMonthNames("abbreviated")[e.getMonth()]+" "+s.default.format(e,"day")};n.unshift(a)}else n.unshift("monthandday");return this._normalizeFormat(n)}if(e.month)return"month";if(e.day){if(t)n.unshift("day");else{var r=function(e){return s.default.format(e,"dayofweek")+", "+s.default.format(e,"day")};n.unshift(r)}return this._normalizeFormat(n)}return this._normalizeFormat(n)},getDateFormatByTicks:function(e){var t,n,i;if(e.length>1)for(t=a.default.getDatesDifferences(e[0],e[1]),i=1;i<e.length-1;i++)n=a.default.getDatesDifferences(e[i],e[i+1]),t.count<n.count&&(t=n);else t={year:!0,month:!0,day:!0,hour:e[0].getHours()>0,minute:e[0].getMinutes()>0,second:e[0].getSeconds()>0,millisecond:e[0].getMilliseconds()>0};var o=this.getDateFormatByDifferences(t);return o},getDateFormatByTickInterval:function(e,t,n){var i,r={week:"day"},s=function(e,t,n){switch(t){case"year":case"quarter":e.month=n;case"month":e.day=n;case"week":case"day":e.hour=n;case"hour":e.minute=n;case"minute":e.second=n;case"second":e.millisecond=n}},l=function(e,t,n){!n.getMilliseconds()&&n.getSeconds()?n.getSeconds()-t.getSeconds()===1&&(e.millisecond=!0,e.second=!1):!n.getSeconds()&&n.getMinutes()?n.getMinutes()-t.getMinutes()===1&&(e.second=!0,e.minute=!1):!n.getMinutes()&&n.getHours()?n.getHours()-t.getHours()===1&&(e.minute=!0,e.hour=!1):!n.getHours()&&n.getDate()>1?n.getDate()-t.getDate()===1&&(e.hour=!0,e.day=!1):1===n.getDate()&&n.getMonth()?n.getMonth()-t.getMonth()===1&&(e.day=!0,e.month=!1):!n.getMonth()&&n.getFullYear()&&n.getFullYear()-t.getFullYear()===1&&(e.month=!0,e.year=!1)};n=(0,o.isString)(n)?n.toLowerCase():n;var u=a.default.getDatesDifferences(e,t);e!==t&&l(u,e>t?t:e,e>t?e:t),i=a.default.getDateUnitInterval(u),s(u,i,!0),i=a.default.getDateUnitInterval(n||"second"),s(u,i,!1),u[r[i]||i]=!0;var c=this.getDateFormatByDifferences(u);return c}});t.default=u,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}t.drop=t.leave=t.enter=t.end=t.start=t.move=void 0;var s=r(n(2)),l=n(39),u=n(12),c=a(n(3)),d=n(33),h=r(n(66)),f=n(8),p=r(n(163)),g=r(n(99)),_="dxdragstart";t.start=_;var m="dxdrag";t.move=m;var v="dxdragend";t.end=v;var y="dxdragenter";t.enter=y;var b="dxdragleave";t.leave=b;var x="dxdrop";t.drop=x;var w="dxDragEventsCount",C=[],k=[],S=[],D={setup:function(e,t){var n=(0,u.inArray)(e,C)!==-1;n||(C.push(e),k.push([]),S.push(t||{}))},add:function(e,t){var n=(0,u.inArray)(e,C);this.updateEventsCounter(e,t.type,1);var i=t.selector;(0,u.inArray)(i,k[n])===-1&&k[n].push(i)},updateEventsCounter:function(e,t,n){if([y,b,x].indexOf(t)>-1){var i=(0,l.data)(e,w)||0;(0,l.data)(e,w,Math.max(0,i+n))}},remove:function(e,t){this.updateEventsCounter(e,t.type,-1)},teardown:function(e){var t=(0,l.data)(e,w);if(!t){var n=(0,u.inArray)(e,C);C.splice(n,1),k.splice(n,1),S.splice(n,1),(0,l.removeData)(e,w)}}};(0,h.default)(y,D),(0,h.default)(b,D),(0,h.default)(x,D);var I=function(e){var t=(0,u.inArray)(e.get(0),C),n=k[t].filter(function(e){return e}),i=e.find(n.join(", "));return(0,u.inArray)(void 0,k[t])!==-1&&(i=i.add(e)),i},T=function(e){var t=(0,u.inArray)(e.get(0),C);return S[t]},O=function(e,t){return e.itemPositionFunc?e.itemPositionFunc(t):t.offset()},E=function(e,t){return e.itemSizeFunc?e.itemSizeFunc(t):{width:t.get(0).getBoundingClientRect().width,height:t.get(0).getBoundingClientRect().height}},A=p.default.inherit({ctor:function(e){this.callBase(e),this.direction="both"},_init:function(e){this._initEvent=e},_start:function(e){e=this._fireEvent(_,this._initEvent),this._maxLeftOffset=e.maxLeftOffset,this._maxRightOffset=e.maxRightOffset,this._maxTopOffset=e.maxTopOffset,this._maxBottomOffset=e.maxBottomOffset;var t=(0,u.wrapToArray)(e.targetElements||(null===e.targetElements?[]:C));this._dropTargets=c.map(t,function(e){return(0,s.default)(e).get(0)})},_move:function(e){var t=(0,f.eventData)(e),n=this._calculateOffset(t);e=this._fireEvent(m,e,{offset:n}),this._processDropTargets(e),e._cancelPreventDefault||e.preventDefault()},_calculateOffset:function(e){return{x:this._calculateXOffset(e),y:this._calculateYOffset(e)}},_calculateXOffset:function(e){if("vertical"!==this.direction){var t=e.x-this._startEventData.x;return this._fitOffset(t,this._maxLeftOffset,this._maxRightOffset)}return 0},_calculateYOffset:function(e){if("horizontal"!==this.direction){var t=e.y-this._startEventData.y;return this._fitOffset(t,this._maxTopOffset,this._maxBottomOffset)}return 0},_fitOffset:function(e,t,n){return null!=t&&(e=Math.max(e,-t)),null!=n&&(e=Math.min(e,n)),e},_processDropTargets:function(e){var t=this._findDropTarget(e),n=t===this._currentDropTarget;n||(this._fireDropTargetEvent(e,b),this._currentDropTarget=t,this._fireDropTargetEvent(e,y))},_fireDropTargetEvent:function(e,t){if(this._currentDropTarget){var n={type:t,originalEvent:e,draggingElement:this._$element.get(0),target:this._currentDropTarget};(0,f.fireEvent)(n)}},_findDropTarget:function(e){var t,n=this;return c.each(C,function(i,o){if(n._checkDropTargetActive(o)){var a=(0,s.default)(o);c.each(I(a),function(i,o){var r=(0,s.default)(o);n._checkDropTarget(T(a),r,(0,s.default)(t),e)&&(t=o)})}}),t},_checkDropTargetActive:function(e){var t=!1;return c.each(this._dropTargets,function(n,i){return t=t||i===e||(0,d.contains)(i,e),!t}),t},_checkDropTarget:function(e,t,n,i){var o=t.get(0)===(0,s.default)(i.target).get(0);if(o)return!1;var a=O(e,t);if(i.pageX<a.left)return!1;if(i.pageY<a.top)return!1;var r=E(e,t);return!(i.pageX>a.left+r.width)&&(!(i.pageY>a.top+r.height)&&((!n.length||!n.closest(t).length)&&(!(e.checkDropTarget&&!e.checkDropTarget(t,i))&&t)))},_end:function(e){var t=(0,f.eventData)(e);this._fireEvent(v,e,{offset:this._calculateOffset(t)}),this._fireDropTargetEvent(e,x),delete this._currentDropTarget}});(0,g.default)({emitter:A,events:[_,m,v]})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(41)),a=i(n(47)),r=i(n(34)),s=i(n(102)),l=n(90),u=s.default.inherit({ctor:function(e){e=Array.isArray(e)?{data:e}:e||{},this.callBase(e);var t=e.data;if(t&&!Array.isArray(t))throw r.default.errors.Error("E4006");this._array=t||[]},createQuery:function(){return(0,a.default)(this._array,{errorHandler:this._errorHandler})},_byKeyImpl:function(e){var t=(0,l.indexByKey)(this,this._array,e);return t===-1?o.default.rejectedPromise(r.default.errors.Error("E4009")):o.default.trivialPromise(this._array[t])},_insertImpl:function(e){return(0,l.insert)(this,this._array,e)},_pushImpl:function(e){(0,l.applyBatch)({keyInfo:this,data:this._array,changes:e})},_updateImpl:function(e,t){return(0,l.update)(this,this._array,e,t)},_removeImpl:function(e){return(0,l.remove)(this,this._array,e)},clear:function(){this._eventsStrategy.fireEvent("modifying"),this._array=[],this._eventsStrategy.fireEvent("modified")}},"array"),c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(54)),a=i(n(337)),r=i(n(338)),s="en",l=(0,o.default)({locale:function(){var e=s;return function(t){return t?void(e=t):e}}(),getValueByClosestLocale:function(e){for(var t,n=this.locale(),i=e(n);!i&&!t;)n=(0,r.default)(a.default,n),n?i=e(n):t=!0;return void 0===i&&n!==s?e(s):i}});t.default=l,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(1),o={toXml:function(e,t,n){var o=["<",e];for(var a in t){var r=t[a];(0,i.isDefined)(r)&&o.push(" ",a,'="',r,'"')}return(0,i.isDefined)(n)&&""!==n?o.push(">",n,"</",e,">"):o.push(" />"),o.join("")}},a=o;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=i(n(2)),r=n(4),s=n(3),l=n(7),u=i(n(11)),c=n(1),d=n(0),h=n(24),f=i(n(23)),p=n(30),g=n(46),_=i(n(13)),m=(0,l.getWindow)(),v=/left|right/,y=/top|bottom/,b=/fit|flip|none/,x=/scale(.+)/,w=f.default.safari,C=function(e){var t={h:"center",v:"center"},n=(0,r.splitPair)(e);return n&&(0,s.each)(n,function(){var e=String(this).toLowerCase();v.test(e)?t.h=e:y.test(e)&&(t.v=e)}),t},k=function(e){return(0,r.pairToObject)(e)},S=function(e){var t=(0,r.splitPair)(e),n=String(t&&t[0]).toLowerCase(),i=String(t&&t[1]).toLowerCase();return b.test(n)||(n="none"),b.test(i)||(i=n),{h:n,v:i}},D=function(e){switch(e){case"center":return.5;case"right":case"bottom":return 1;default:return 0}},I=function(e){switch(e){case"left":return"right";case"right":return"left";case"top":return"bottom";case"bottom":return"top";default:return e}},T=function(e,t){var n=0;return e.myLocation<t.min&&(n+=t.min-e.myLocation),e.myLocation>t.max&&(n+=e.myLocation-t.max),n},O=function(e,t,n){return t.myLocation<n.min?"h"===e?"left":"top":t.myLocation>n.max?"h"===e?"right":"bottom":"none"},E=function(e){e.myLocation=e.atLocation+D(e.atAlign)*e.atSize-D(e.myAlign)*e.mySize+e.offset},A={fit:function(e,t){var n=!1;e.myLocation>t.max&&(e.myLocation=t.max,n=!0),e.myLocation<t.min&&(e.myLocation=t.min,n=!0),e.fit=n},flip:function(e,t){if(e.flip=!1,("center"!==e.myAlign||"center"!==e.atAlign)&&(e.myLocation<t.min||e.myLocation>t.max)){var n=(0,d.extend)({},e,{myAlign:I(e.myAlign),atAlign:I(e.atAlign),offset:-e.offset});E(n),n.oversize=T(n,t),(n.myLocation>=t.min&&n.myLocation<=t.max||e.oversize>n.oversize)&&(e.myLocation=n.myLocation,e.oversize=n.oversize,e.flip=!0)}},flipfit:function(e,t){this.flip(e,t),this.fit(e,t)},none:function(e){e.oversize=0}},P=function(){var e=(0,a.default)("<div>").css({width:100,height:100,overflow:"scroll",position:"absolute",top:-9999}).appendTo((0,a.default)("body")),t=e.get(0).offsetWidth-e.get(0).clientWidth;e.remove(),o=t},R={h:{location:0,flip:!1,fit:!1,oversize:0},v:{location:0,flip:!1,fit:!1,oversize:0}},M=function(e,t){var n=(0,a.default)(e),i=n.offset(),r=(0,d.extend)(!0,{},R,{h:{location:i.left},v:{location:i.top}});if(!t)return r;var s=C(t.my),l=C(t.at),f=(0,a.default)(t.of).length&&t.of||m,p=k(t.offset),v=S(t.collision),y=t.boundary,b=k(t.boundaryOffset),x={mySize:n.outerWidth(),myAlign:s.h,atAlign:l.h,offset:p.h,collision:v.h,boundaryOffset:b.h},D={mySize:n.outerHeight(),myAlign:s.v,atAlign:l.v,offset:p.v,collision:v.v,boundaryOffset:b.v};if(f.preventDefault)x.atLocation=f.pageX,D.atLocation=f.pageY,x.atSize=0,D.atSize=0;else if(f=(0,a.default)(f),(0,c.isWindow)(f[0]))x.atLocation=f.scrollLeft(),D.atLocation=f.scrollTop(),"phone"===_.default.real().deviceType&&f[0].visualViewport?(x.atLocation=Math.max(x.atLocation,f[0].visualViewport.offsetLeft),D.atLocation=Math.max(D.atLocation,f[0].visualViewport.offsetTop),x.atSize=f[0].visualViewport.width,D.atSize=f[0].visualViewport.height):(x.atSize=f[0].innerWidth>f[0].outerWidth?f[0].innerWidth:f.width(),D.atSize=f[0].innerHeight>f[0].outerHeight||w?f[0].innerHeight:f.height());else if(9===f[0].nodeType)x.atLocation=0,D.atLocation=0,x.atSize=f.width(),D.atSize=f.height();else{var I=(0,h.getBoundingRect)(f.get(0)),M=B(f);x.atLocation=M.left,D.atLocation=M.top,x.atSize=Math.max(I.width,f.outerWidth()),D.atSize=Math.max(I.height,f.outerHeight())}E(x),E(D);var V=function(){var e=(0,a.default)(m),t=e.width(),n=e.height(),i=e.scrollLeft(),r=e.scrollTop(),s=u.default.getDocumentElement(),l=g.touch?s.clientWidth/t:1,c=g.touch?s.clientHeight/n:1;void 0===o&&P();var d=t,h=n;if(y){var f=(0,a.default)(y),p=f.offset();i=p.left,r=p.top,d=f.width(),h=f.height()}return{h:{min:i+x.boundaryOffset,max:i+d/l-x.mySize-x.boundaryOffset},v:{min:r+D.boundaryOffset,max:r+h/c-D.mySize-D.boundaryOffset}}}();x.oversize=T(x,V.h),D.oversize=T(D,V.v),x.collisionSide=O("h",x,V.h),D.collisionSide=O("v",D,V.v),A[x.collision]&&A[x.collision](x,V.h),A[D.collision]&&A[D.collision](D,V.v);var F=function(e){return t.precise?e:Math.round(e)};return(0,d.extend)(!0,r,{h:{location:F(x.myLocation),oversize:F(x.oversize),fit:x.fit,flip:x.flip,collisionSide:x.collisionSide},v:{location:F(D.myLocation),oversize:F(D.oversize),fit:D.fit,flip:D.flip,collisionSide:D.collisionSide},precise:t.precise}),r},B=function e(t){var n,i,o=arguments.length>1&&void 0!==arguments[1]?arguments[1]:t,a=o.get(0);if(!a)return t.offset();var r,s=(null===(n=a.getAttribute)||void 0===n?void 0:n.call(a,"style"))||"",l=null===(i=s.match(x))||void 0===i?void 0:i[0];return l?(a.setAttribute("style",s.replace(l,"")),r=e(t,o.parent()),a.setAttribute("style",s)):r=e(t,o.parent()),r},V=function(e,t){var n=(0,a.default)(e);if(!t)return n.offset();(0,p.resetPosition)(n,!0);var i=B(n),o=t.h&&t.v?t:M(n,t),r=function(e){return t.precise?e:Math.round(e)};return(0,p.move)(n,{left:o.h.location-r(i.left),top:o.v.location-r(i.top)}),o},F=function(e){return e=(0,a.default)(e).get(0),(0,c.isWindow)(e)?null:e&&"pageY"in e&&"pageX"in e?{top:e.pageY,left:e.pageX}:(0,a.default)(e).offset()};V.inverseAlign||(V.inverseAlign=I),V.normalizeAlign||(V.normalizeAlign=C);var L={calculateScrollbarWidth:P,calculate:M,setup:V,offset:F};t.default=L,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return a.default&&(0,r.default)().useJQuery}t.default=o;var a=i(n(56)),r=i(n(28)),s=(0,r.default)().useJQuery;a.default&&s!==!1&&(0,r.default)({useJQuery:!0}),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return u(e)||l(e,t)||r(e,t)||a()}function a(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function l(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function u(e){if(Array.isArray(e))return e}t.default=void 0;var c=i(n(20)),d=i(n(270)),h=i(n(369)),f=c.default.dateToMilliseconds,p=60,g=function(e){return e?new Date(Date.UTC(e.getFullYear(),e.getMonth(),e.getDate(),e.getHours(),e.getMinutes(),e.getSeconds())):null},_=function(e){var t=(0,h.default)(e),n=t.getTimezoneOffset();return t.addTime(t.getTimezoneOffset("minute")),t.subtractMinutes(n-t.getTimezoneOffset()),t.source},m=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:new Date,t=v(e);return d.default.getDisplayedTimeZones(t.getTime())},v=function(e){return new Date(Date.UTC(e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate(),e.getUTCHours(),e.getUTCMinutes()))},y=function(e,t,n,i){return x(n,i)-x(e,t)},b=function(e,t,n,i){return y(e,t,n,i)*f("minute")},x=function(e,t){return new Date(e).getTimezoneOffset()-new Date(t).getTimezoneOffset()},w=function(e,t){return x(e,t)*f("minute")},C=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:new Date;
if("string"==typeof e){var n=v(t);return d.default.getTimeZoneOffsetById(e,n.getTime())}return e},k=function(e,t,n){return C(n,e)-C(n,t)},S=function(e,t,n,i,o){var a=k(e,t,i),r=k(e,t,o),s=a-r;return new Date(n.getTime()-s*f("hour"))},D=function(e,t,n,i){var o=arguments.length>4&&void 0!==arguments[4]&&arguments[4],a=(e.getTimezoneOffset()-t.getTimezoneOffset())/p;return i?a=k(t,e,i):n&&(a=k(t,e,n)),new Date(e.getTime()+(o?-1:1)*a*f("hour"))},I=function(e){var t=new Date(new Date(e).setHours(0,0,0,0)),n=new Date(new Date(e).setHours(23,59,59,0));return t.getTimezoneOffset()-n.getTimezoneOffset()!==0},T=function(e){var t=new Date(e);if(I(t)){var n=new Date(t);return new Date(n.setDate(n.getDate()+1))}return t},O=function(e,t){return t=new Date(t.getTime()-1),c.default.sameDate(e,t)},E=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:new Date;return 6e4*e.getTimezoneOffset()},A=function(e){if(Intl){var t=Intl.DateTimeFormat().resolvedOptions().timeZone;if(t)return t===e}return R(e)},P=function(){var e=M(),t=o(e,2),n=t[0],i=t[1];return n.getTimezoneOffset()!==i.getTimezoneOffset()},R=function(e){var t=M(),n=o(t,2),i=n[0],a=n[1],r=-i.getTimezoneOffset()/60,s=-a.getTimezoneOffset()/60,l=C(e,i),u=C(e,a);return r===l&&s===u},M=function(){var e=new Date(Date.now()),t=new Date,n=new Date;return t.setFullYear(e.getFullYear(),0,1),n.setFullYear(e.getFullYear(),6,1),[t,n]},B={getDaylightOffset:x,getDaylightOffsetInMs:w,getTimezoneOffsetChangeInMinutes:y,getTimezoneOffsetChangeInMs:b,calculateTimezoneByValue:C,getCorrectedDateByDaylightOffsets:S,isSameAppointmentDates:O,correctRecurrenceExceptionByTimezone:D,getClientTimezoneOffset:E,createUTCDateWithLocalOffset:g,createDateFromUTCWithLocalOffset:_,createUTCDate:v,isTimezoneChangeInDate:I,getDateWithoutTimezoneChange:T,hasDSTInLocalTimeZone:P,isEqualLocalTimeZone:A,getTimeZones:m},V=B;t.default=V,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=w(t.element,e.wavesNumber).eq(t.wave||v);e.hidingTimeout&&clearTimeout(e.hidingTimeout),s(n),n.css(C(e,t)),e.showingTimeout=setTimeout(a.bind(this,e,n),0)}function a(e,t){var n=e.durations.showingScale+"ms";t.addClass(h).css("transitionDuration",n)}function r(e){return{showingScale:e?m:_,hidingScale:_,hidingOpacity:_}}function s(e){e.removeClass(f).css("transitionDuration","")}function l(e,t){e.showingTimeout&&clearTimeout(e.showingTimeout);var n=w(t.element,t.wavesNumber).eq(t.wave||v),i=e.durations,o=i.hidingScale+"ms, "+i.hidingOpacity+"ms";n.addClass(f).removeClass(h).css("transitionDuration",o);var a=Math.max(i.hidingScale,i.hidingOpacity);e.hidingTimeout=setTimeout(s.bind(this,n),a)}t.showWave=o,t.hideWave=l,t.render=t.initConfig=void 0;var u=i(n(2)),c="dx-inkripple",d="dx-inkripple-wave",h="dx-inkripple-showing",f="dx-inkripple-hiding",p=2,g=4e3,_=300,m=1e3,v=0,y=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.useHoldAnimation,n=e.waveSizeCoefficient,i=e.isCentered,o=e.wavesNumber;return{waveSizeCoefficient:n||p,isCentered:i||!1,wavesNumber:o||1,durations:r(null===t||void 0===t||t)}};t.initConfig=y;var b=function(e){var t=y(e);return{showWave:o.bind(this,t),hideWave:l.bind(this,t)}};t.render=b;var x=function(e){var t=e.children("."+c);return 0===t.length&&(t=(0,u.default)("<div>").addClass(c).appendTo(e)),t},w=function(e,t){for(var n=x((0,u.default)(e)),i=n.children("."+d).toArray(),o=i.length;o<t;o++){var a=(0,u.default)("<div>").appendTo(n).addClass(d);i.push(a[0])}return(0,u.default)(i)},C=function(e,t){var n,i,o=(0,u.default)(t.element),a=o.outerWidth(),r=o.outerHeight(),s=parseInt(Math.sqrt(a*a+r*r)),l=Math.min(g,parseInt(s*e.waveSizeCoefficient));if(e.isCentered)n=(a-l)/2,i=(r-l)/2;else{var c=t.event,d=o.offset(),h=c.pageX-d.left,f=c.pageY-d.top;n=h-l/2,i=f-l/2}return{left:n,top:i,height:l,width:l}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=n(46),s=n(36),l=n(0),u=i(n(13)),c=i(n(9)),d=i(n(16)),h=(0,a.getNavigator)(),f="dx-loadindicator",p="dx-loadindicator-wrapper",g="dx-loadindicator-content",_="dx-loadindicator-icon",m="dx-loadindicator-segment",v="dx-loadindicator-segment-inner",y="dx-loadindicator-image",b=d.default.inherit({_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{indicatorSrc:"",activeStateEnabled:!1,hoverStateEnabled:!1,_animatingSegmentCount:1,_animatingSegmentInner:!1})},_defaultOptionsRules:function(){var e=(0,s.current)();return this.callBase().concat([{device:function(){var e=u.default.real(),t="android"===e.platform&&!/chrome/i.test(h.userAgent);return t},options:{viaImage:!0}},{device:function(){return(0,s.isMaterial)(e)},options:{_animatingSegmentCount:2,_animatingSegmentInner:!0}},{device:function(){return(0,s.isGeneric)(e)},options:{_animatingSegmentCount:7}}])},_init:function(){this.callBase(),this.$element().addClass(f)},_initMarkup:function(){this.callBase(),this._renderWrapper(),this._renderIndicatorContent(),this._renderMarkup()},_renderWrapper:function(){this._$wrapper=(0,o.default)("<div>").addClass(p),this.$element().append(this._$wrapper)},_renderIndicatorContent:function(){this._$content=(0,o.default)("<div>").addClass(g),this._$wrapper.append(this._$content)},_renderMarkup:function(){!(0,r.animation)()||this.option("viaImage")||this.option("indicatorSrc")?this._renderMarkupForImage():this._renderMarkupForAnimation()},_renderMarkupForAnimation:function(){var e=this.option("_animatingSegmentInner");this._$indicator=(0,o.default)("<div>").addClass(_),this._$content.append(this._$indicator);for(var t=this.option("_animatingSegmentCount");t>=0;--t){var n=(0,o.default)("<div>").addClass(m).addClass(m+t);e&&n.append((0,o.default)("<div>").addClass(v)),this._$indicator.append(n)}},_renderMarkupForImage:function(){var e=this.option("indicatorSrc");this._$wrapper.addClass(y),e&&this._$wrapper.css("backgroundImage","url("+e+")")},_renderDimensions:function(){this.callBase(),this._updateContentSizeForAnimation()},_updateContentSizeForAnimation:function(){if(this._$indicator){var e=this.option("width"),t=this.option("height");if(e||t){e=this.$element().width(),t=this.$element().height();var n=Math.min(t,e);this._$wrapper.css({height:n,width:n,fontSize:n})}}},_clean:function(){this.callBase(),this._removeMarkupForAnimation(),this._removeMarkupForImage()},_removeMarkupForAnimation:function(){this._$indicator&&(this._$indicator.remove(),delete this._$indicator)},_removeMarkupForImage:function(){this._$wrapper.css("backgroundImage","none")},_optionChanged:function(e){switch(e.name){case"_animatingSegmentCount":case"_animatingSegmentInner":case"indicatorSrc":this._invalidate();break;default:this.callBase(e)}}});(0,c.default)("dxLoadIndicator",b);var x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return x.default.sendRequest(e)}function a(e){var t=function t(n,i,o,a,r,s){function l(e,r,s){(0,O.when)(t(n,i,o,a,r,s)).done(e.resolve)}a=a||[],n=n||[];var u,c;for(r=r||0,u=r;u<n.length;u++){if(e&&u>r&&u%1e4===0&&new Date-A>=300)return A=new Date,c=new O.Deferred,setTimeout(l(c,u,!1),0),c;var d=n[u];if(!s){if(a.unshift(d),o&&i(a,u)===!1)return;if(d.children){var h=t(d.children,i,o,a);if(e&&h)return c=new O.Deferred,h.done(l(c,u,!0)),c}}if(s=!1,!o&&i(a,u)===!1)return;a.shift(),n[u]!==d&&u--}};return t}function r(e,t){if(e&&(0,b.isDefined)(t))for(var n=0;n<e.length;n++){var i=e[n];if(i.name===t||i.caption===t||i.dataField===t||i.index===t)return n}return-1}function s(e,t){var n=e===e&&D.default.format(e,t.format),i={value:e,valueText:n||""};return t.customizeText?t.customizeText.call(t,i):i.valueText}function l(e){return function(t,n){var i=0,o=e(t),a=e(n),r=(0,b.isDefined)(o),s=(0,b.isDefined)(a);return r&&s&&(o>a?i=1:o<a&&(i=-1)),r&&!s&&(i=1),!r&&s&&(i=-1),i}}function u(e){for(var t=[],n=e.length-1;n>=0;n--)t.push(e[n].key||e[n].value);return t}function c(e,t,n,i){n=n||0,i=i||"children",e.length&&t(e,n);for(var o=0;o<e.length;o++){var a=e[o];a[i]&&a[i].length&&c(a[i],t,n+1,i)}}function d(e,t){for(var n=[],i=0;i<e.length;i++)n.push(Math.max(e[i]||0,t[i]||0));return n}function h(e,t){var n=e[t],i=0,o=("columns"===t?e.columnExpandedPaths:e.rowExpandedPaths)||[];for(e.headerName===t?i=e.path.length:e.headerName&&e.headerName!==t&&e.oppositePath?i=e.oppositePath.length:(0,C.each)(o,function(e,t){i=Math.max(i,t.length)});n[i+1]&&n[i].expanded;)i++;return i}function f(e){return(0,C.map)(["year","quarter","month"],function(t,n){return(0,k.extend)({},e,{groupInterval:t,groupIndex:n})})}function p(e,t,n,i){var o=[];return Object.keys(t||[]).forEach(function(a){if(!a||0!==a.indexOf("__")){for(var r,s=1,l=n.length?n+"."+a:a,u=i[l],c=(0,w.compileGetter)(l),d=t[a];!(0,b.isDefined)(d)&&e[s];)d=c(e[s]),s++;!u&&(0,b.isDefined)(d)&&(u=(0,b.type)(d)),r=[{dataField:l,dataType:u,groupName:"date"===u?a:void 0,groupInterval:void 0,displayFolder:n}],"date"===u?r=r.concat(f(r[0])):"object"===u&&(r=p(e,d,l,i)),o.push.apply(o,r)}}),o}function g(e,t){var n=_(t);return p(e,e[0],"",n)}function _(e){var t={};return(0,C.each)(e,function(e,n){t[n.dataField]=t[n.dataField]||n.dataType}),t}function m(e){if("date"===e.dataType)e.format||E(e,"format",M[e.groupInterval]);else if("number"===e.dataType){var t=(0,b.isNumeric)(e.groupInterval)&&e.groupInterval>0&&e.groupInterval;t&&!e.customizeText&&E(e,"customizeText",function(n){var i=n.value+t,o=D.default.format(i,e.format);return n.valueText&&o?n.valueText+" - "+o:""})}}function v(e,t){var n=[];t=t||[];for(var i=0;i<t.length;i++)n.push((0,k.extend)({},e[i],{groupIndex:null,groupName:null,filterType:"include",filterValues:[t[i]]}));return n}function y(e){return e.charAt(0).toUpperCase()+e.slice(1)}t.sendRequest=o,t.findField=r,t.formatValue=s,t.getCompareFunction=l,t.createPath=u,t.foreachDataLevel=c,t.mergeArraysByMaxValue=d,t.getExpandedLevel=h,t.discoverObjectFields=g,t.getFieldsDataType=_,t.setDefaultFieldValueFormatting=m,t.getFiltersByPath=v,t.capitalizeFirstLetter=y,t.storeDrillDownMixin=t.foreachTreeAsync=t.foreachTree=t.setFieldProperty=void 0;var b=n(1),x=i(n(76)),w=n(19),C=n(3),k=n(0),S=i(n(32)),D=i(n(79)),I=n(69),T=i(n(81)),O=n(6),E=function(e,t,n,i){var o=e._initProperties=e._initProperties||{},a=i?n:e[t];Object.prototype.hasOwnProperty.call(o,t)&&!i||(o[t]=a),e[t]=n};t.setFieldProperty=E;var A=new Date,P=a(!1);t.foreachTree=P;var R=a(!0);t.foreachTreeAsync=R;var M={month:function(e){return S.default.getMonthNames()[e-1]},quarter:function(e){return S.default.format(new Date(2e3,3*e-1),"quarter")},dayOfWeek:function(e){return S.default.getDayNames()[e]}},B={createDrillDownDataSource:function(e,t){function n(e){return function(t){var n;return i?n=i[e](t):(n=new O.Deferred,(0,O.when)(o).done(function(i){var o=new T.default(i);o[e](t).done(n.resolve).fail(n.reject)}).fail(n.reject)),n}}var i,o=this.getDrillDownItems(e,t),a=new I.DataSource({load:n("load"),totalCount:n("totalCount"),key:this.key()});return a}};t.storeDrillDownMixin=B},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){for(var n,i="string"==typeof t?t.split():t.slice();i.length;)if(n=i.shift(),n in e)return!0;return!1}function h(e,t,n,i){var o,a;if(i){for(var r=0;r<t.length;r++)if(o=t[r].items||t[r].collapsedItems||[],a=h(e,o||[],n,i-1))return a}else if(S(e,t,n)>=0)return t}function f(e,t,n,i){return i?h(e,t,n,i)||[]:t}function p(e,t){if(e.key()&&(!t._dataByKeyMap||t._dataByKeyMapLength!==t.length)){for(var n={},i=t.length,o=0;o<i;o++)n[JSON.stringify(e.keyOf(t[o]))]=t[o];t._dataByKeyMap=n,t._dataByKeyMapLength=i}}function g(e,t){if(e._dataByKeyMap)return e._dataByKeyMap[JSON.stringify(t)]}function _(e,t){return!e._dataByKeyMap||e._dataByKeyMap[JSON.stringify(t)]}function m(e,t,n){e._dataByKeyMap&&(e._dataByKeyMap[JSON.stringify(t)]=n)}function v(e,t){var n=e?Object.create(Object.getPrototypeOf(e)):{},i=(0,O.extendFromObject)({},e);return(0,A.deepExtendArraySafe)(n,i,!0,!0),(0,A.deepExtendArraySafe)(n,t,!0,!0)}function y(e){var t=e.keyInfo,n=e.data,i=e.changes,o=e.groupCount,r=e.useInsertIndex,s=e.immutable,l=e.disableCache,u=e.logError,c=s===!0?a(n):n;return i.forEach(function(e){var n="insert"===e.type?c:f(t,c,e.key,o);switch(!l&&p(t,n),e.type){case"update":w(t,n,e.key,e.data,!0,s,u);break;case"insert":C(t,n,e.data,r&&(0,D.isDefined)(e.index)?e.index:-1,!0,u);break;case"remove":k(t,n,e.key,!0,u)}}),c}function b(e,t,n){return e?t&&E.default.errors.log(n):R.default.rejectedPromise(E.default.errors.Error(n))}function x(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=n.keyExpr,o=void 0===i?"id":i,a=n.immutable,r=void 0===a||a,s=(0,P.compileGetter)(o),l={key:function(){return o},keyOf:function(e){return s(e)}};return y({keyInfo:l,data:e,changes:t,immutable:r,disableCache:!0,logError:!0})}function w(e,t,n,i,o,a,r){var s,l=!0,u=e.key();if(u){if(d(i,u)&&!R.default.keysEqual(u,n,e.keyOf(i)))return b(o,r,"E4017");if(s=g(t,n),!s){var c=S(e,t,n);if(c<0)return b(o,r,"E4009");if(s=t[c],a===!0&&(0,D.isDefined)(s)){var h=v(s,i);return t[c]=h,!o&&R.default.trivialPromise(h,n)}}}else s=n;if((0,A.deepExtendArraySafe)(s,i,l),!o)return(0,I.default)().useLegacyStoreResult?R.default.trivialPromise(n,i):R.default.trivialPromise(s,n)}function C(e,t,n,i,a,r){var s,l=e.key(),u=(0,D.isPlainObject)(n)?(0,O.extend)({},n):n;if(l){if(s=e.keyOf(u),void 0===s||"object"===o(s)&&(0,D.isEmptyObject)(s)){if(Array.isArray(l))throw E.default.errors.Error("E4007");s=u[l]=String(new T.default)}else if(void 0!==t[S(e,t,s)])return b(a,r,"E4008")}else s=u;if(i>=0?t.splice(i,0,u):t.push(u),m(t,s,u),!a)return R.default.trivialPromise((0,I.default)().useLegacyStoreResult?n:u,s)}function k(e,t,n,i,o){var a=S(e,t,n);return a>-1&&t.splice(a,1),i?a<0?b(i,o,"E4009"):void 0:R.default.trivialPromise(n)}function S(e,t,n){var i=e.key();if(!_(t,n))return-1;for(var o=0,a=t.length;o<a;o++)if(R.default.keysEqual(i,e.keyOf(t[o]),n))return o;return-1}t.applyBatch=y,t.createObjectWithChanges=v,t.update=w,t.insert=C,t.remove=k,t.indexByKey=S,t.applyChanges=x;var D=n(1),I=i(n(28)),T=i(n(35)),O=n(0),E=i(n(34)),A=n(50),P=n(19),R=i(n(41))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return e=e.groupBy(t[0].selector),t.length>1&&(e=e.select(function(e){return(0,l.extend)({},e,{items:o((0,c.default)(e.items),t.slice(1)).toArray()})})),e}function a(e,t){var n=[];return(0,u.each)(e,function(e,i){var o=(0,s.grep)(t,function(e){return i.selector===e.selector});o.length<1&&n.push(i)}),n.concat(t)}function r(e,t,n){t=t||{};var i=t.filter;if(i&&(e=e.filter(i)),n)return e;var r=t.sort,s=t.select,l=t.group,c=t.skip,h=t.take;return l&&(l=d.default.normalizeSortingInfo(l),l.keepInitialKeyOrder=!!t.group.keepInitialKeyOrder),(r||l)&&(r=d.default.normalizeSortingInfo(r||[]),l&&!l.keepInitialKeyOrder&&(r=a(l,r)),(0,u.each)(r,function(t){e=e[t?"thenBy":"sortBy"](this.selector,this.desc,this.compare)})),s&&(e=e.select(s)),l&&(e=o(e,l)),(h||c)&&(e=e.slice(c||0,h)),e}t.default=void 0;var s=n(4),l=n(0),u=n(3),c=i(n(156)),d=i(n(41)),h={multiLevelGroup:o,arrangeSortingInfo:a,queryByOptions:r};t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.EventsStrategy=void 0;var s=i(n(27)),l=n(3),u=n(1),c=function(){function e(t){o(this,e),this._events={},this._owner=t}return r(e,[{key:"hasEvent",value:function(e){var t=this._events[e];return!!t&&t.has()}},{key:"fireEvent",value:function(e,t){var n=this._events[e];return n&&n.fireWith(this._owner,t),this._owner}},{key:"on",value:function(e,t){var n=this;if((0,u.isPlainObject)(e))(0,l.each)(e,function(e,t){n.on(e,t)});else{var i=this._events[e];i||(i=(0,s.default)(),this._events[e]=i);var o=i.originalAdd||i.add;o.call(i,t)}}},{key:"off",value:function(e,t){var n=this._events[e];n&&((0,u.isFunction)(t)?n.remove(t):n.empty())}},{key:"dispose",value:function(){(0,l.each)(this._events,function(e,t){t.empty()})}}],[{key:"create",value:function(t,n){return n?(0,u.isFunction)(n)?n(t):n:new e(t)}}]),e}();t.EventsStrategy=c},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.EmptyTemplate=void 0;var g=o(n(2)),_=n(67),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_renderCore",value:function(){return(0,g.default)()}}]),t}(_.TemplateBase);t.EmptyTemplate=m},function(e,t,n){t.default=void 0;var i=n(69),o=n(0),a=n(70),r="_dataSourceOptions",s="_dataSourceChangedHandler",l="_dataSourceLoadErrorHandler",u="_dataSourceLoadingChangedHandler",c="_dataSourceFromUrlLoadMode",d="_getSpecificDataSourceOption",h="_normalizeDataSource",f={postCtor:function(){this.on("disposing",function(){this._disposeDataSource()}.bind(this))},_refreshDataSource:function(){this._initDataSource(),this._loadDataSource()},_initDataSource:function(){var e,t,n=d in this?this[d]():this.option("dataSource");this._disposeDataSource(),n&&(n instanceof i.DataSource?(this._isSharedDataSource=!0,this._dataSource=n):(e=r in this?this[r]():{},t=this._dataSourceType?this._dataSourceType():i.DataSource,n=(0,a.normalizeDataSourceOptions)(n,{fromUrlLoadMode:c in this&&this[c]()}),this._dataSource=new t((0,o.extend)(!0,{},e,n))),h in this&&(this._dataSource=this[h](this._dataSource)),this._addDataSourceHandlers())},_addDataSourceHandlers:function(){s in this&&this._addDataSourceChangeHandler(),l in this&&this._addDataSourceLoadErrorHandler(),u in this&&this._addDataSourceLoadingChangedHandler(),this._addReadyWatcher()},_addReadyWatcher:function(){this._dataSource.on("loadingChanged",function(e){this._ready&&this._ready(!e)}.bind(this))},_addDataSourceChangeHandler:function(){var e=this._dataSource;this._proxiedDataSourceChangedHandler=function(t){this[s](e.items(),t)}.bind(this),e.on("changed",this._proxiedDataSourceChangedHandler)},_addDataSourceLoadErrorHandler:function(){this._proxiedDataSourceLoadErrorHandler=this[l].bind(this),this._dataSource.on("loadError",this._proxiedDataSourceLoadErrorHandler)},_addDataSourceLoadingChangedHandler:function(){this._proxiedDataSourceLoadingChangedHandler=this[u].bind(this),this._dataSource.on("loadingChanged",this._proxiedDataSourceLoadingChangedHandler)},_loadDataSource:function(){if(this._dataSource){var e=this._dataSource;e.isLoaded()?this._proxiedDataSourceChangedHandler&&this._proxiedDataSourceChangedHandler():e.load()}},_loadSingle:function(e,t){return e="this"===e?this._dataSource.key()||"this":e,this._dataSource.loadSingle(e,t)},_isLastPage:function(){return!this._dataSource||this._dataSource.isLastPage()||!this._dataSource._pageSize},_isDataSourceLoading:function(){return this._dataSource&&this._dataSource.isLoading()},_disposeDataSource:function(){this._dataSource&&(this._isSharedDataSource?(delete this._isSharedDataSource,this._proxiedDataSourceChangedHandler&&this._dataSource.off("changed",this._proxiedDataSourceChangedHandler),this._proxiedDataSourceLoadErrorHandler&&this._dataSource.off("loadError",this._proxiedDataSourceLoadErrorHandler),this._proxiedDataSourceLoadingChangedHandler&&this._dataSource.off("loadingChanged",this._proxiedDataSourceLoadingChangedHandler)):this._dataSource.dispose(),delete this._dataSource,delete this._proxiedDataSourceChangedHandler,delete this._proxiedDataSourceLoadErrorHandler,delete this._proxiedDataSourceLoadingChangedHandler)},getDataSource:function(){return this._dataSource||null}},p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.escapeServiceOperationParams=t.formatFunctionInvocationUrl=t.generateExpand=t.generateSelect=t.convertPrimitiveValue=t.keyConverters=t.serializeKey=t.serializeValue=t.serializePropName=t.EdmLiteral=t.sendRequest=void 0;var a=i(n(15)),r=n(0),s=n(1),l=n(3),u=i(n(76)),c=i(n(35)),d=n(4),h=n(6),f=i(n(34)),p=i(n(41)),g=n(42),_=/^(\{{0,1}([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}\}{0,1})$/,m=/^\/Date\((-?\d+)((\+|-)?(\d+)?)\)\/$/,v=/^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(\.\d+)?(Z|[-+]{1}\d{2}(:?)(\d{2})?)?$/,y="application/json;odata=verbose",b=function(e){return"string"===(0,s.type)(e)?e.split():e},x=function(e){return/\./.test(e)},w=function(e,t,n){for(e=String(e);e.length<t;)e=n?"".concat(e,"0"):"0".concat(e);return e},C=function(e,t,n){var i=[],o=function(){return e.getHours()+e.getMinutes()+e.getSeconds()+e.getMilliseconds()<1},a=function(e){return w(e,2)};return i.push(e.getFullYear()),i.push("-"),i.push(a(e.getMonth()+1)),i.push("-"),i.push(a(e.getDate())),t&&o()||(i.push("T"),i.push(a(e.getHours())),i.push(":"),i.push(a(e.getMinutes())),i.push(":"),i.push(a(e.getSeconds())),e.getMilliseconds()&&(i.push("."),i.push(w(e.getMilliseconds(),3))),n||i.push("Z")),i.join("")},k=function(e){var t=new Date(60*new Date(0).getTimezoneOffset()*1e3),n=e.replace("Z","").split("T"),i=/(\d{4})-(\d{2})-(\d{2})/.exec(n[0]),o=/(\d{2}):(\d{2}):(\d{2})\.?(\d{0,7})?/.exec(n[1]);if(t.setFullYear(Number(i[1])),t.setMonth(Number(i[2])-1),t.setDate(Number(i[3])),Array.isArray(o)&&o.length){t.setHours(Number(o[1])),t.setMinutes(Number(o[2])),t.setSeconds(Number(o[3]));var a=(o[4]||"").slice(0,3);a=w(a,3,!0),t.setMilliseconds(Number(a))}return t},S=function(e){return/^(?:[a-z]+:)?\/\//i.test(e)},D=function(e){var t=e.indexOf("?");return t>-1?e.substr(0,t):e},I=function(e,t){var n,i=D(e).split("/"),o=t.split("/");for(i.pop();o.length;)n=o.shift(),".."===n?i.pop():i.push(n);return i.join("/")},T=function(e){var t=[];for(var n in e)t.push(n+"="+e[n]);return t.join("&")},O=function(e,t){var n,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},o=function(t){return JSON.stringify(t,function(t,n){if(!(this[t]instanceof Date))return n;switch(n=C(this[t]),e){case 2:return n.substr(0,n.length-1);case 3:case 4:return n;default:throw f.default.errors.Error("E4002")}})};t=(0,r.extend)({async:!0,method:"get",url:"",params:{},payload:null,headers:{},timeout:3e4},t),null===(n=i.beforeSend)||void 0===n?void 0:n.call(i,t);var a=t,s=a.async,l=a.timeout,u=a.headers,c=t,d=c.url,h=c.method,p=i.jsonp,g=i.withCredentials;h=(h||"get").toLowerCase();var _="get"===h,m=_&&p,v=(0,r.extend)({},t.params),b=_?v:o(t.payload),x=!_&&T(v),w=!_&&y;return x&&(d+=(d.indexOf("?")>-1?"&":"?")+x),m&&(b.$format="json"),{url:d,data:b,dataType:m?"jsonp":"json",jsonp:m&&"$callback",method:h,async:s,timeout:l,headers:u,contentType:w,accepts:{json:[y,"text/plain"].join()},xhrFields:{withCredentials:g}}},E=function e(t,n,i){var o=i.deserializeDates,a=i.fieldTypes,r=i.countOnly,s=i.isPaged,l=new h.Deferred,c=O(t,n,i);return u.default.sendRequest(c).always(function(n,u){var d={deserializeDates:o,fieldTypes:a},h=R(n,u,d,c),g=h.error,_=h.data,m=h.count,v=h.nextUrl;if(g)g.message!==p.default.XHR_ERROR_UNLOAD&&l.reject(g);else if(r)isFinite(m)?l.resolve(m):l.reject(new f.default.errors.Error("E4018"));else if(v&&!s)S(v)||(v=I(c.url,v)),e(t,{url:v},i).fail(l.reject).done(function(e){return l.resolve(_.concat(e))});else{var y=isFinite(m)?{totalCount:m}:void 0;l.resolve(_,y)}}),l.promise()};t.sendRequest=E;var A=function(e){var t,n=e;if("message"in e){var i;t=(null===(i=e.message)||void 0===i?void 0:i.value)||e.message}for(;(n=n.innererror||n.internalexception)&&(t=n.message,!n.internalexception||t.indexOf("inner exception")!==-1););return t},P=function(e,t,n){var i,o,a,s;if("nocontent"===t)return null;var l="Unknown error",u=e,c=200,d={requestOptions:n};if("success"!==t){var h=e.status,f=e.responseText;c=h,l=p.default.errorMessageFromXhr(e,t);try{u=JSON.parse(f)}catch(e){}}var g=(null===(i=u)||void 0===i?void 0:i.then)||(null===(o=u)||void 0===o?void 0:o.error)||(null===(a=u)||void 0===a?void 0:a["odata.error"])||(null===(s=u)||void 0===s?void 0:s["@odata.error"]);if(g){l=A(g)||l,d.errorDetails=g,200===c&&(c=500);var _=Number(g.code);isFinite(_)&&_>=400&&(c=_)}return c>=400||0===c?(d.httpStatus=c,(0,r.extend)(Error(l),d)):null},R=function(e,t,n,i){var o=P(e,t,i);if(o)return{error:o};if(!(0,s.isPlainObject)(e))return{data:e};var a="d"in e&&(Array.isArray(e.d)||(0,s.isObject)(e.d))?M(e,t):B(e,t);return F(a,n),a},M=function(e){var t,n=e.d;return(0,s.isDefined)(n)?{data:null!==(t=n.results)&&void 0!==t?t:n,nextUrl:n.__next,count:parseInt(n.__count,10)}:{error:Error("Malformed or unsupported JSON response received")}},B=function(e){var t;return{data:null!==(t=e.value)&&void 0!==t?t:e,nextUrl:e["@odata.nextLink"],count:parseInt(e["@odata.count"],10)}},V=a.default.inherit({ctor:function(e){this._value=e},valueOf:function(){return this._value}});t.EdmLiteral=V;var F=function e(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};(0,l.each)(t,function(i,a){if(null!==a&&"object"===o(a))"results"in a&&(t[i]=a.results),e(t[i],n);else if("string"==typeof a){var r=n.fieldTypes,s=n.deserializeDates,l=!r||"String"!==r[i];if(l&&_.test(a)&&(t[i]=new c.default(a)),s!==!1)if(a.match(m)){var u=new Date(Number(RegExp.$1)+60*RegExp.$2*1e3);t[i]=new Date(u.valueOf()+60*u.getTimezoneOffset()*1e3)}else v.test(a)&&(t[i]=new Date(k(t[i]).valueOf()))}})},L=function(e){return"datetime'".concat(C(e,!0,!0),"'")},H=function(e){return"'".concat(e.replace(/'/g,"''"),"'")},z=function(e){return e instanceof V?e.valueOf():e.replace(/\./g,"/")};t.serializePropName=z;var N=function e(t){return t instanceof Date?C(t,!1,!1):t instanceof c.default?t.valueOf():Array.isArray(t)?"[".concat(t.map(function(t){return e(t)}).join(","),"]"):j(t)},j=function(e){return e instanceof Date?L(e):e instanceof c.default?"guid'".concat(e,"'"):e instanceof V?e.valueOf():"string"==typeof e?H(e):String(e)},W=function(e,t){switch(t){case 2:case 3:return j(e);case 4:return N(e);default:throw f.default.errors.Error("E4002")}};t.serializeValue=W;var G=function(e,t){if((0,s.isPlainObject)(e)){var n=[];return(0,l.each)(e,function(e,i){return n.push("".concat(z(e),"=").concat(W(i,t)))}),n.join()}return W(e,t)};t.serializeKey=G;var q={String:function(e){return"".concat(e)},Int32:function(e){return Math.floor(e)},Int64:function(e){return e instanceof V?e:new V("".concat(e,"L"))},Guid:function(e){return e instanceof c.default?e:new c.default(e)},Boolean:function(e){return!!e},Single:function(e){return e instanceof V?e:new V(e+"f")},Decimal:function(e){return e instanceof V?e:new V(e+"m")}};t.keyConverters=q;var $=function(e,t){if(null===t)return null;var n=q[e];if(!n)throw f.default.errors.Error("E4014",e);return n(t)};t.convertPrimitiveValue=$;var K=function(e,t){if(t)return e<4?z(t.join()):(0,d.grep)(t,x,!0).join()};t.generateSelect=K;var U=function e(t){var n="",i=[],o=[];return(0,l.each)(t,function(t,n){Array.isArray(n)&&[].push.apply(i,n),(0,s.isPlainObject)(n)&&o.push("".concat(t).concat(e(n)))}),(i.length||o.length)&&(n+="(",i.length&&(n+="$select=".concat((0,l.map)(i,z).join())),o.length&&(i.length&&(n+=";"),n+="$expand=".concat((0,l.map)(o,z).join())),n+=")"),n},Y=function(e){var t=[];return(0,l.each)(e,function(e,n){return t.push("".concat(e).concat(U(n)))}),t.join()},Z=function e(t,n,i){var o=i(n,t.shift(),t);o!==!1&&e(t,o,i)},X=function(e,t,n){return(0,l.each)(e,function(e,i){return Z(i.split("."),t,n)})},Q=function(e,t){var n={};return e&&(0,l.each)(b(e),function(){n[z(this)]=1}),t&&(0,l.each)(b(t),function(){var e=this.split(".");e.length<2||(e.pop(),n[z(e.join("."))]=1)}),(0,l.map)(n,function(e,t){return t}).join()},J=function(e,t){var n={};if(e||t)return e&&X(b(e),n,function(e,t,n){return e[t]=e[t]||{},!!n.length&&e[t]}),t&&X((0,d.grep)(b(t),x),n,function(e,t,n){return n.length?e[t]=e[t]||{}:(e[t]=e[t]||[],e[t].push(t),!1)}),Y(n)},ee=function(e,t,n){return e<4?Q(t,n):J(t,n)};t.generateExpand=ee;var te=function(e,t){return(0,g.format)("{0}({1})",e,(0,l.map)(t||{},function(e,t){return(0,g.format)("{0}={1}",t,e)}).join(","))};t.formatFunctionInvocationUrl=te;var ne=function(e,t){if(!e)return e;var n={};return(0,l.each)(e,function(e,i){n[e]=W(i,t)}),n};t.escapeServiceOperationParams=ne},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(603)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.setHeight=t.setWidth=t.normalizeStyleProp=t.stylePropPrefix=t.styleProp=void 0;var o=n(40),a=i(n(64)),r=n(1),s=i(n(11)),l=["","Webkit","Moz","O","Ms"],u={"":"",Webkit:"-webkit-",Moz:"-moz-",O:"-o-",ms:"-ms-"},c=(0,a.default)(function(){return s.default.createElement("dx").style}),d=function(e,t){e=(0,o.camelize)(e,!0);for(var n,i=0,a=l.length;i<a;i++){var r=l[i],s=r+e,u=(0,o.camelize)(s);if(n=t(u,r),void 0===n&&(n=t(s,r)),void 0!==n)break}return n||""},h=function(e){if(e in c())return e;var t=e;e=e.charAt(0).toUpperCase()+e.substr(1);for(var n=1;n<l.length;n++){var i=l[n].toLowerCase()+e;if(i in c())return i}return t};t.styleProp=h;var f=function(e){return d(e,function(e,t){if(e in c())return u[t];
})};t.stylePropPrefix=f;var p=["fillOpacity","columnCount","flexGrow","flexShrink","fontWeight","lineHeight","opacity","zIndex","zoom"],g=function(e,t){return(0,r.isNumeric)(t)&&p.indexOf(e)===-1&&(t+="px"),t};t.normalizeStyleProp=g;var _=function(e,t,n){if(e){n=(0,r.isNumeric)(n)?n+="px":n;for(var i=0;i<e.length;++i)e[i].style[t]=n}},m=function(e,t){_(e,"width",t)};t.setWidth=m;var v=function(e,t){_(e,"height",t)};t.setHeight=v},function(e,t,n){function i(e){this.baseColor=e;var t;e&&(t=String(e).toLowerCase().replace(/ /g,""),t=m[t]?"#"+m[t]:t,t=o(t)),t||(this.colorIsInvalid=!0),t=t||{},this.r=a(t[0]),this.g=a(t[1]),this.b=a(t[2]),this.a=a(t[3],1,1),t[4]?this.hsv={h:t[4][0],s:t[4][1],v:t[4][2]}:this.hsv=s(this.r,this.g,this.b),t[5]?this.hsl={h:t[5][0],s:t[5][1],l:t[5][2]}:this.hsl=c(this.r,this.g,this.b)}function o(e){if("transparent"===e)return[0,0,0,0];for(var t,n=0,i=v.length;n<i;++n)if(t=v[n].re.exec(e))return v[n].process(t);return null}function a(e,t,n){return t=t||0,n=n||255,e<0||isNaN(e)?t:e>n?n:e}function r(e,t,n){return"#"+(16777216|e<<16|t<<8|n).toString(16).slice(1)}function s(e,t,n){var i,o,a=Math.max(e,t,n),r=Math.min(e,t,n),s=a-r,l=a;if(o=0===a?0:1-r/a,a===r)i=0;else switch(a){case e:i=60*((t-n)/s),t<n&&(i+=360);break;case t:i=60*((n-e)/s)+120;break;case n:i=60*((e-t)/s)+240}return o*=100,l*=100/255,{h:Math.round(i),s:Math.round(o),v:Math.round(l)}}function l(e,t,n){var i,o,a,r=Math.floor(e%360/60),s=(100-t)*n/100,l=(n-s)*(e%60/60),u=s+l,c=n-l;switch(r){case 0:i=n,o=u,a=s;break;case 1:i=c,o=n,a=s;break;case 2:i=s,o=n,a=u;break;case 3:i=s,o=c,a=n;break;case 4:i=u,o=s,a=n;break;case 5:i=n,o=s,a=c}return[Math.round(2.55*i),Math.round(2.55*o),Math.round(2.55*a)]}function u(e,t,n,i){var o=Math.max(e,t,n);switch(o){case e:return(t-n)/i+(t<n?6:0);case t:return(n-e)/i+2;case n:return(e-t)/i+4}}function c(e,t,n){e=g(e,255),t=g(t,255),n=g(n,255);var i,o,a=Math.max(e,t,n),r=Math.min(e,t,n),s=a+r,l=s/2;if(a===r)i=o=0;else{var c=a-r;o=l>.5?c/(2-s):c/s,i=u(e,t,n,c),i/=6}return{h:y(360*i),s:y(100*o),l:y(100*l)}}function d(e,t){var n=t;return"r"===e&&(n=t+1/3),"b"===e&&(n=t-1/3),n}function h(e){return e<0&&(e+=1),e>1&&(e-=1),e}function f(e,t,n){return n=h(n),n<1/6?e+6*(t-e)*n:n<.5?t:n<2/3?e+(t-e)*(2/3-n)*6:e}function p(e,t,n){var i,o,a;if(e=g(e,360),t=g(t,100),n=g(n,100),0===t)i=o=a=n;else{var r=n<.5?n*(1+t):n+t-n*t,s=2*n-r;i=f(s,r,d("r",e)),o=f(s,r,d("g",e)),a=f(s,r,d("b",e))}return[y(255*i),y(255*o),y(255*a)]}function g(e,t){return e=Math.min(t,Math.max(0,parseFloat(e))),Math.abs(e-t)<1e-6?1:e%t/parseFloat(t)}function _(e,t,n){return t=t||0,n=n||255,!(e%1!==0||e<t||e>n||"number"!=typeof e||isNaN(e))}t.default=void 0;var m={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"00ffff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000000",blanchedalmond:"ffebcd",blue:"0000ff",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"00ffff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dodgerblue:"1e90ff",feldspar:"d19275",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"ff00ff",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgrey:"d3d3d3",lightgreen:"90ee90",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslateblue:"8470ff",lightslategray:"778899",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"00ff00",limegreen:"32cd32",linen:"faf0e6",magenta:"ff00ff",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370d8",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"d87093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"ff0000",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",violetred:"d02090",wheat:"f5deb3",white:"ffffff",whitesmoke:"f5f5f5",yellow:"ffff00",yellowgreen:"9acd32"},v=[{re:/^rgb\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/,process:function(e){return[parseInt(e[1],10),parseInt(e[2],10),parseInt(e[3],10)]}},{re:/^rgba\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3}),\s*(\d*\.*\d+)\)$/,process:function(e){return[parseInt(e[1],10),parseInt(e[2],10),parseInt(e[3],10),parseFloat(e[4])]}},{re:/^#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})$/,process:function(e){return[parseInt(e[1],16),parseInt(e[2],16),parseInt(e[3],16)]}},{re:/^#([a-f0-9]{1})([a-f0-9]{1})([a-f0-9]{1})$/,process:function(e){return[parseInt(e[1]+e[1],16),parseInt(e[2]+e[2],16),parseInt(e[3]+e[3],16)]}},{re:/^hsv\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/,process:function(e){var t=parseInt(e[1],10),n=parseInt(e[2],10),i=parseInt(e[3],10),o=l(t,n,i);return[o[0],o[1],o[2],1,[t,n,i]]}},{re:/^hsl\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/,process:function(e){var t=parseInt(e[1],10),n=parseInt(e[2],10),i=parseInt(e[3],10),o=p(t,n,i);return[o[0],o[1],o[2],1,null,[t,n,i]]}}],y=Math.round;i.prototype={constructor:i,highlight:function(e){return e=e||10,this.alter(e).toHex()},darken:function(e){return e=e||10,this.alter(-e).toHex()},alter:function(e){var t=new i;return t.r=a(this.r+e),t.g=a(this.g+e),t.b=a(this.b+e),t},blend:function(e,t){var n=e instanceof i?e:new i(e),o=new i;return o.r=a(y(this.r*(1-t)+n.r*t)),o.g=a(y(this.g*(1-t)+n.g*t)),o.b=a(y(this.b*(1-t)+n.b*t)),o},toHex:function(){return r(this.r,this.g,this.b)},getPureColor:function(){var e=l(this.hsv.h,100,100);return new i("rgb("+e.join(",")+")")},isValidHex:function(e){return/(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(e)},isValidRGB:function(e,t,n){return!!(_(e)&&_(t)&&_(n))},isValidAlpha:function(e){return!(isNaN(e)||e<0||e>1||"number"!=typeof e)},colorIsInvalid:!1,fromHSL:function(e){var t=new i,n=p(e.h,e.s,e.l);return t.r=n[0],t.g=n[1],t.b=n[2],t}};var b=i;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(49)),r=i(n(11)),s=i(n(5)),l=n(39),u=i(n(15)),c=n(0),d=n(12),h=n(3),f=i(n(66)),p=n(8),g=i(n(25)),_=n(139),m="dxEventManager",v="dxEmitter",y=u.default.inherit({ctor:function(){this._attachHandlers(),this.reset(),this._proxiedCancelHandler=this._cancelHandler.bind(this),this._proxiedAcceptHandler=this._acceptHandler.bind(this)},_attachHandlers:function(){a.default.add(function(){var e=r.default.getDocument();s.default.subscribeGlobal(e,(0,p.addNamespace)(g.default.down,m),this._pointerDownHandler.bind(this)),s.default.subscribeGlobal(e,(0,p.addNamespace)(g.default.move,m),this._pointerMoveHandler.bind(this)),s.default.subscribeGlobal(e,(0,p.addNamespace)([g.default.up,g.default.cancel].join(" "),m),this._pointerUpHandler.bind(this)),s.default.subscribeGlobal(e,(0,p.addNamespace)(_.name,m),this._mouseWheelHandler.bind(this))}.bind(this))},_eachEmitter:function(e){for(var t=this._activeEmitters||[],n=0;t.length>n;){var i=t[n];if(e(i)===!1)break;t[n]===i&&n++}},_applyToEmitters:function(e,t){this._eachEmitter(function(n){n[e].call(n,t)})},reset:function(){this._eachEmitter(this._proxiedCancelHandler),this._activeEmitters=[]},resetEmitter:function(e){this._proxiedCancelHandler(e)},_pointerDownHandler:function(e){(0,p.isMouseEvent)(e)&&e.which>1||this._updateEmitters(e)},_updateEmitters:function(e){this._isSetChanged(e)&&(this._cleanEmitters(e),this._fetchEmitters(e))},_isSetChanged:function(e){var t=this._closestEmitter(e),n=this._emittersSet||[],i=t.length!==n.length;return(0,h.each)(t,function(e,t){return i=i||n[e]!==t,!i}),this._emittersSet=t,i},_closestEmitter:function(e){function t(t,o){o&&o.validatePointers(e)&&o.validate(e)&&(o.addCancelCallback(n._proxiedCancelHandler),o.addAcceptCallback(n._proxiedAcceptHandler),i.push(o))}for(var n=this,i=[],a=(0,o.default)(e.target);a.length;){var r=(0,l.data)(a.get(0),v)||[];(0,h.each)(r,t),a=a.parent()}return i},_acceptHandler:function(e,t){var n=this;this._eachEmitter(function(i){i!==e&&n._cancelEmitter(i,t)})},_cancelHandler:function(e,t){this._cancelEmitter(e,t)},_cancelEmitter:function(e,t){var n=this._activeEmitters;t?e.cancel(t):e.reset(),e.removeCancelCallback(),e.removeAcceptCallback();var i=(0,d.inArray)(e,n);i>-1&&n.splice(i,1)},_cleanEmitters:function(e){this._applyToEmitters("end",e),this.reset(e)},_fetchEmitters:function(e){this._activeEmitters=this._emittersSet.slice(),this._applyToEmitters("start",e)},_pointerMoveHandler:function(e){this._applyToEmitters("move",e)},_pointerUpHandler:function(e){this._updateEmitters(e)},_mouseWheelHandler:function(e){this._allowInterruptionByMouseWheel()&&(e.pointers=[null],this._pointerDownHandler(e),this._adjustWheelEvent(e),this._pointerMoveHandler(e),e.pointers=[],this._pointerUpHandler(e))},_allowInterruptionByMouseWheel:function(){var e=!0;return this._eachEmitter(function(t){return e=t.allowInterruptionByMouseWheel()&&e}),e},_adjustWheelEvent:function(e){var t=null;if(this._eachEmitter(function(n){if(n.gesture){var i=n.getDirection(e);return"horizontal"!==i&&!e.shiftKey||"vertical"!==i&&e.shiftKey?(t=n,!1):void 0}}),t){var n=t.getDirection(e),i="both"===n&&!e.shiftKey||"vertical"===n,o=i?"pageY":"pageX";e[o]+=e.delta}},isActive:function(e){var t=!1;return this._eachEmitter(function(n){t=t||n.getElement().is(e)}),t}}),b=new y,x="dxEmitterSubscription",w=function(e){var t=e.emitter,n=e.events[0],i=e.events;(0,h.each)(i,function(o,a){(0,f.default)(a,{noBubble:!e.bubble,setup:function(e){var i=(0,l.data)(e,x)||{},o=(0,l.data)(e,v)||{},r=o[n]||new t(e);i[a]=!0,o[n]=r,(0,l.data)(e,v,o),(0,l.data)(e,x,i)},add:function(e,t){var i=(0,l.data)(e,v),o=i[n];o.configure((0,c.extend)({delegateSelector:t.selector},t.data),t.type)},teardown:function(e){var t=(0,l.data)(e,x),o=(0,l.data)(e,v),r=o[n];delete t[a];var s=!0;(0,h.each)(i,function(e,n){return s=s&&!t[n]}),s&&(b.isActive(e)&&b.resetEmitter(r),r&&r.dispose(),delete o[n])}})})},C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&r(e,t)}function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function s(e){var t=c();return function(){var n,i=d(e);if(t){var o=d(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return l(this,n)}}function l(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?u(e):t}function u(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function c(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function h(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function f(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function p(e,t,n){return t&&f(e.prototype,t),n&&f(e,n),e}t.default=void 0;var g=o(n(15)),_=n(0),m=n(12),v=n(3),y=n(92),b=o(n(22)),x=n(4),w=n(1),C=o(n(45)),k=o(n(10)),S=o(n(77)),D=n(6),I={valid:"valid",invalid:"invalid",pending:"pending"},T=function(){function e(){h(this,e),this.NAME="base"}return p(e,[{key:"defaultMessage",value:function(e){return k.default.getFormatter("validation-".concat(this.NAME))(e)}},{key:"defaultFormattedMessage",value:function(e){return k.default.getFormatter("validation-".concat(this.NAME,"-formatted"))(e)}},{key:"_isValueEmpty",value:function(e){return!L.required.validate(e,{})}},{key:"validate",value:function(e,t){var n=this,i=Array.isArray(e)?e:[e],o=!0;return i.length?i.every(function(e){return o=n._validate(e,t)}):o=this._validate(null,t),o}}]),e}(),O=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="required",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){return!!(0,w.isDefined)(e)&&(e!==!1&&(e=String(e),!t.trim&&(0,w.isDefined)(t.trim)||(e=e.trim()),""!==e))}}]),t}(T),E=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="numeric",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){return!(t.ignoreEmptyValue===!1||!this._isValueEmpty(e))||(t.useCultureSettings&&(0,w.isString)(e)?!isNaN(C.default.parse(e)):(0,w.isNumeric)(e))}}]),t}(T),A=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="range",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){if(t.ignoreEmptyValue!==!1&&this._isValueEmpty(e))return!0;var n=L.numeric.validate(e,t),i=(0,w.isDefined)(e)&&""!==e,o=n?parseFloat(e):i&&e.valueOf(),a=t.min,r=t.max;if(!n&&!(0,w.isDate)(e)&&!i)return!1;if((0,w.isDefined)(a))return(0,w.isDefined)(r)?o>=a&&o<=r:o>=a;if((0,w.isDefined)(r))return o<=r;throw b.default.Error("E0101")}}]),t}(T),P=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="stringLength",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){return e=(0,w.isDefined)(e)?String(e):"",!t.trim&&(0,w.isDefined)(t.trim)||(e=e.trim()),!(!t.ignoreEmptyValue||!this._isValueEmpty(e))||L.range.validate(e.length,(0,_.extend)({},t))}}]),t}(T),R=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="custom",e}a(t,e);var n=s(t);return p(t,[{key:"validate",value:function(e,t){if(t.ignoreEmptyValue&&this._isValueEmpty(e))return!0;var n=t.validator,i=n&&(0,w.isFunction)(n.option)&&n.option("dataGetter"),o=(0,w.isFunction)(i)&&i(),a={value:e,validator:n,rule:t};return o&&(0,_.extend)(a,o),t.validationCallback(a)}}]),t}(T),M=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="async",e}a(t,e);var n=s(t);return p(t,[{key:"validate",value:function(e,t){if((0,w.isDefined)(t.reevaluate)||(0,_.extend)(t,{reevaluate:!0}),t.ignoreEmptyValue&&this._isValueEmpty(e))return!0;var n=t.validator,i=n&&(0,w.isFunction)(n.option)&&n.option("dataGetter"),o=(0,w.isFunction)(i)&&i(),a={value:e,validator:n,rule:t};o&&(0,_.extend)(a,o);var r=t.validationCallback(a);if(!(0,w.isPromise)(r))throw b.default.Error("E0103");return this._getWrappedPromise((0,D.fromPromise)(r).promise())}},{key:"_getWrappedPromise",value:function(e){var t=new D.Deferred;return e.then(function(e){t.resolve(e)},function(e){var n={isValid:!1};(0,w.isDefined)(e)&&((0,w.isString)(e)?n.message=e:(0,w.isObject)(e)&&(0,w.isDefined)(e.message)&&(0,w.isString)(e.message)&&(n.message=e.message)),t.resolve(n)}),t.promise()}}]),t}(R),B=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="compare",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){if(!t.comparisonTarget)throw b.default.Error("E0102");if(t.ignoreEmptyValue&&this._isValueEmpty(e))return!0;(0,_.extend)(t,{reevaluate:!0});var n=t.comparisonTarget(),i=t.comparisonType||"==";switch(i){case"==":return e==n;case"!=":return e!=n;case"===":return e===n;case"!==":return e!==n;case">":return e>n;case">=":return e>=n;case"<":return e<n;case"<=":return e<=n}}}]),t}(T),V=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="pattern",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){if(t.ignoreEmptyValue!==!1&&this._isValueEmpty(e))return!0;var n=t.pattern;return(0,w.isString)(n)&&(n=new RegExp(n)),n.test(e)}}]),t}(T),F=function(e){function t(){var e;return h(this,t),e=n.call(this),e.NAME="email",e}a(t,e);var n=s(t);return p(t,[{key:"_validate",value:function(e,t){return!(t.ignoreEmptyValue===!1||!this._isValueEmpty(e))||L.pattern.validate(e,(0,_.extend)({},t,{pattern:/^[\d\w._-]+@([\d\w._-]+\.)+[\w]+$/i}))}}]),t}(T),L={required:new O,numeric:new E,range:new A,stringLength:new P,custom:new R,async:new M,compare:new B,pattern:new V,email:new F},H=g.default.inherit({ctor:function(e){this.group=e,this.validators=[],this._pendingValidators=[],this._onValidatorStatusChanged=this._onValidatorStatusChanged.bind(this),this._resetValidationInfo(),this._eventsStrategy=new y.EventsStrategy(this)},validate:function(){var e=this,t={isValid:!0,brokenRules:[],validators:[],status:I.valid,complete:null};return this._unsubscribeFromAllChangeEvents(),this._pendingValidators=[],this._resetValidationInfo(),(0,v.each)(this.validators,function(n,i){var o=i.validate();t.isValid=t.isValid&&o.isValid,o.brokenRules&&(t.brokenRules=t.brokenRules.concat(o.brokenRules)),t.validators.push(i),o.status===I.pending&&e._addPendingValidator(i),e._subscribeToChangeEvents(i)}),this._pendingValidators.length?t.status=I.pending:(t.status=t.isValid?I.valid:I.invalid,this._unsubscribeFromAllChangeEvents(),this._raiseValidatedEvent(t)),this._updateValidationInfo(t),(0,_.extend)({},this._validationInfo.result)},_subscribeToChangeEvents:function(e){e.on("validating",this._onValidatorStatusChanged),e.on("validated",this._onValidatorStatusChanged)},_unsubscribeFromChangeEvents:function(e){e.off("validating",this._onValidatorStatusChanged),e.off("validated",this._onValidatorStatusChanged)},_unsubscribeFromAllChangeEvents:function(){var e=this;(0,v.each)(this.validators,function(t,n){e._unsubscribeFromChangeEvents(n)})},_updateValidationInfo:function(e){this._validationInfo.result=e,e.status===I.pending&&(this._validationInfo.deferred||(this._validationInfo.deferred=new D.Deferred,this._validationInfo.result.complete=this._validationInfo.deferred.promise()))},_addPendingValidator:function(e){var t=(0,x.grep)(this._pendingValidators,function(t){return t===e})[0];t||this._pendingValidators.push(e)},_removePendingValidator:function(e){var t=(0,m.inArray)(e,this._pendingValidators);t>=0&&this._pendingValidators.splice(t,1)},_orderBrokenRules:function(e){var t=[];return(0,v.each)(this.validators,function(n,i){var o=(0,x.grep)(e,function(e){return e.validator===i});o.length&&(t=t.concat(o))}),t},_updateBrokenRules:function(e){if(this._validationInfo.result){var t=this._validationInfo.result.brokenRules,n=(0,x.grep)(t,function(t){return t.validator!==e.validator});e.brokenRules&&(t=n.concat(e.brokenRules)),this._validationInfo.result.brokenRules=this._orderBrokenRules(t)}},_onValidatorStatusChanged:function(e){return e.status===I.pending?void this._addPendingValidator(e.validator):void this._resolveIfComplete(e)},_resolveIfComplete:function(e){if(this._removePendingValidator(e.validator),this._updateBrokenRules(e),!this._pendingValidators.length){if(this._unsubscribeFromAllChangeEvents(),!this._validationInfo.result)return;this._validationInfo.result.status=0===this._validationInfo.result.brokenRules.length?I.valid:I.invalid,this._validationInfo.result.isValid=this._validationInfo.result.status===I.valid;var t=(0,_.extend)({},this._validationInfo.result,{complete:null}),n=this._validationInfo.deferred;this._resetValidationInfo(),this._raiseValidatedEvent(t),n&&setTimeout(function(){n.resolve(t)})}},_raiseValidatedEvent:function(e){this._eventsStrategy.fireEvent("validated",[e])},_resetValidationInfo:function(){this._validationInfo={result:null,deferred:null}},_synchronizeValidationInfo:function(){this._validationInfo.result&&(this._validationInfo.result.validators=this.validators)},removeRegisteredValidator:function(e){var t=(0,m.inArray)(e,this.validators);t>-1&&(this.validators.splice(t,1),this._synchronizeValidationInfo(),this._resolveIfComplete({validator:e}))},registerValidator:function(e){(0,m.inArray)(e,this.validators)<0&&(this.validators.push(e),this._synchronizeValidationInfo())},reset:function(){(0,v.each)(this.validators,function(e,t){t.reset()}),this._pendingValidators=[],this._resetValidationInfo()},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this}}),z={groups:[],getGroupConfig:function(e){var t=(0,x.grep)(this.groups,function(t){return t.group===e});if(t.length)return t[0]},findGroup:function(e,t){var n=e.parents(".dx-validationgroup").first();return n.length?n.dxValidationGroup("instance"):t},initGroups:function(){this.groups=[],this.addGroup()},addGroup:function(e){var t=this.getGroupConfig(e);return t||(t=new H(e),this.groups.push(t)),t},removeGroup:function(e){var t=this.getGroupConfig(e),n=(0,m.inArray)(t,this.groups);return n>-1&&this.groups.splice(n,1),t},_setDefaultMessage:function(e){var t=e.rule,n=e.validator,i=e.name;(0,w.isDefined)(t.message)||(n.defaultFormattedMessage&&(0,w.isDefined)(i)?t.message=n.defaultFormattedMessage(i):t.message=n.defaultMessage())},_addBrokenRule:function(e){var t=e.result,n=e.rule;t.brokenRule||(t.brokenRule=n),t.brokenRules||(t.brokenRules=[]),t.brokenRules.push(n)},validate:function(e,t,n){var i=this,o={name:n,value:e,brokenRule:null,brokenRules:null,isValid:!0,validationRules:t,pendingRules:null,status:I.valid,complete:null},a=[];return(0,v.each)(t||[],function(t,r){var s,l=L[r.type];if(!l)throw b.default.Error("E0100");return(0,w.isDefined)(r.isValid)&&r.value===e&&!r.reevaluate?!!r.isValid||(o.isValid=!1,i._addBrokenRule({result:o,rule:r}),!1):(r.value=e,"async"===r.type?(a.push({rule:r,ruleValidator:l}),!0):(s=l.validate(e,r),r.isValid=s,s||(o.isValid=!1,i._setDefaultMessage({rule:r,validator:l,name:n}),i._addBrokenRule({result:o,rule:r})),!!r.isValid&&void 0))}),o.isValid&&!o.brokenRules&&a.length&&(o=this._validateAsyncRules({value:e,items:a,result:o,name:n})),o.status=o.pendingRules?I.pending:o.isValid?I.valid:I.invalid,o},_validateAsyncRules:function(e){var t=this,n=e.result,i=e.value,o=e.items,a=e.name,r=[];return(0,v.each)(o,function(e,o){var s=o.ruleValidator.validate(i,o.rule);if((0,w.isPromise)(s)){n.pendingRules||(n.pendingRules=[]),n.pendingRules.push(o.rule);var l=s.then(function(e){var n=t._getPatchedRuleResult(e);return t._updateRuleConfig({rule:o.rule,ruleResult:n,validator:o.ruleValidator,name:a}),n});r.push(l)}else t._updateRuleConfig({rule:o.rule,ruleResult:t._getPatchedRuleResult(s),validator:o.ruleValidator,name:a})}),r.length&&(n.complete=S.default.all(r).then(function(e){return t._getAsyncRulesResult({result:n,values:e})})),n},_updateRuleConfig:function(e){var t=e.rule,n=e.ruleResult,i=e.validator,o=e.name;t.isValid=n.isValid,n.isValid||((0,w.isDefined)(n.message)&&(0,w.isString)(n.message)&&n.message.length?t.message=n.message:this._setDefaultMessage({rule:t,validator:i,name:o}))},_getPatchedRuleResult:function(e){var t,n=!0;return(0,w.isObject)(e)?(t=(0,_.extend)({},e),(0,w.isDefined)(t.isValid)||(t.isValid=n)):t={isValid:(0,w.isBoolean)(e)?e:n},t},_getAsyncRulesResult:function(e){var t=this,n=e.values,i=e.result;return(0,v.each)(n,function(e,n){if(n.isValid===!1){i.isValid=n.isValid;var o=i.pendingRules[e];t._addBrokenRule({result:i,rule:o})}}),i.pendingRules=null,i.complete=null,i.status=i.isValid?I.valid:I.invalid,i},registerValidatorInGroup:function(e,t){var n=z.addGroup(e);n.registerValidator.call(n,t)},_shouldRemoveGroup:function(e,t){var n=void 0===e,i=e&&"dxValidationGroup"===e.NAME;return!n&&!i&&!t.length},removeRegisteredValidator:function(e,t){var n=z.getGroupConfig(e);if(n){n.removeRegisteredValidator.call(n,t);var i=n.validators;this._shouldRemoveGroup(e,i)&&this.removeGroup(e)}},initValidationOptions:function(e){var t=this,n={};if(e){var i=["isValid","validationStatus","validationError","validationErrors"];i.forEach(function(i){i in e&&(0,_.extend)(n,t.synchronizeValidationOptions({name:i,value:e[i]},e))})}return n},synchronizeValidationOptions:function(e,t){var n=e.name,i=e.value;switch(n){case"validationStatus":var o=i===I.valid||i===I.pending;return t.isValid!==o?{isValid:o}:{};case"isValid":var a=t.validationStatus,r=a;return i&&a===I.invalid?r=I.valid:i||a===I.invalid||(r=I.invalid),r!==a?{validationStatus:r}:{};case"validationErrors":var s=i&&i.length?i[0]:null;return t.validationError!==s?{validationError:s}:{};case"validationError":var l=t.validationErrors;if(!i&&l)return{validationErrors:null};if(i&&!l)return{validationErrors:[i]};if(i&&l&&i!==l[0])return l[0]=i,{validationErrors:l.slice()}}return{}},validateGroup:function(e){var t=z.getGroupConfig(e);if(!t)throw b.default.Error("E0110");return t.validate()},resetGroup:function(e){var t=z.getGroupConfig(e);if(!t)throw b.default.Error("E0110");return t.reset()}};z.initGroups();var N=z;t.default=N,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(106),a=i(n(54)),r=(0,a.default)({isWrapped:function(){return!1},isWritableWrapped:function(){return!1},wrap:function(e){return e},unwrap:function(e){return e},assign:function(){o.logger.error("Method 'assign' should not be used for not wrapped variables. Use 'isWrapped' method for ensuring.")}});t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=n(92),r=n(3),s=i(n(34)),l=i(n(41)),u=n(19),c=i(n(91)),d=n(6),h=n(4),f=o.default.abstract,p=c.default.queryByOptions,g={},_=o.default.inherit({ctor:function(e){var t=this;e=e||{},this._eventsStrategy=new a.EventsStrategy(this),(0,r.each)(["onLoaded","onLoading","onInserted","onInserting","onUpdated","onUpdating","onPush","onRemoved","onRemoving","onModified","onModifying"],function(n,i){i in e&&t.on(i.slice(2).toLowerCase(),e[i])}),this._key=e.key,this._errorHandler=e.errorHandler,this._useDefaultSearch=!0},_customLoadOptions:function(){return null},key:function(){return this._key},keyOf:function(e){return this._keyGetter||(this._keyGetter=(0,u.compileGetter)(this.key())),this._keyGetter(e)},_requireKey:function(){if(!this.key())throw s.default.errors.Error("E4005")},load:function(e){var t=this;return e=e||{},this._eventsStrategy.fireEvent("loading",[e]),this._withLock(this._loadImpl(e)).done(function(n){t._eventsStrategy.fireEvent("loaded",[n,e])})},_loadImpl:function(e){return p(this.createQuery(e),e).enumerate()},_withLock:function(e){var t=new d.Deferred;return e.done(function(){var e=this,n=arguments;l.default.processRequestResultLock.promise().done(function(){t.resolveWith(e,n)})}).fail(function(){t.rejectWith(this,arguments)}),t},createQuery:f,totalCount:function(e){return this._totalCountImpl(e)},_totalCountImpl:function(e){return p(this.createQuery(e),e,!0).count()},byKey:function(e,t){return this._addFailHandlers(this._withLock(this._byKeyImpl(e,t)))},_byKeyImpl:f,insert:function(e){var t=this;return t._eventsStrategy.fireEvent("modifying"),t._eventsStrategy.fireEvent("inserting",[e]),t._addFailHandlers(t._insertImpl(e).done(function(e,n){t._eventsStrategy.fireEvent("inserted",[e,n]),t._eventsStrategy.fireEvent("modified")}))},_insertImpl:f,update:function(e,t){var n=this;return n._eventsStrategy.fireEvent("modifying"),n._eventsStrategy.fireEvent("updating",[e,t]),n._addFailHandlers(n._updateImpl(e,t).done(function(){n._eventsStrategy.fireEvent("updated",[e,t]),n._eventsStrategy.fireEvent("modified")}))},_updateImpl:f,push:function(e){this._pushImpl(e),this._eventsStrategy.fireEvent("push",[e])},_pushImpl:h.noop,remove:function(e){var t=this;return t._eventsStrategy.fireEvent("modifying"),t._eventsStrategy.fireEvent("removing",[e]),t._addFailHandlers(t._removeImpl(e).done(function(e){t._eventsStrategy.fireEvent("removed",[e]),t._eventsStrategy.fireEvent("modified")}))},_removeImpl:f,_addFailHandlers:function(e){return e.fail(this._errorHandler).fail(s.default._errorHandler)},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this}});_.create=function(e,t){if(!(e in g))throw s.default.errors.Error("E4020",e);return new g[e](t)},_.registerClass=function(e,t){return t&&(g[t]=e),e},_.inherit=function(e){return function(t,n){var i=e.apply(this,[t]);return _.registerClass(i,n),i}}(_.inherit);var m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(8),a=i(n(121)),r=i(n(99)),s=Math.abs,l="dxhold",u=750,c=5,d=a.default.inherit({start:function(e){this._startEventData=(0,o.eventData)(e),this._startTimer(e)},_startTimer:function(e){var t="timeout"in this?this.timeout:u;this._holdTimer=setTimeout(function(){this._requestAccept(e),this._fireEvent(l,e,{target:e.target}),this._forgetAccept()}.bind(this),t)},move:function(e){this._touchWasMoved(e)&&this._cancel(e)},_touchWasMoved:function(e){var t=(0,o.eventDelta)(this._startEventData,(0,o.eventData)(e));return s(t.x)>c||s(t.y)>c},end:function(){this._stopTimer()},_stopTimer:function(){clearTimeout(this._holdTimer)},cancel:function(){this._stopTimer()},dispose:function(){this._stopTimer()}});(0,r.default)({emitter:d,bubble:!0,events:[l]});var h={name:l};t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(584)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(){if(!u.default)throw l.default.Error("E1041","devexpress-diagram");return u}t.getDiagram=s;var l=r(n(17)),u=a(n(878))},function(e,t,n){t.debug=t.logger=void 0;var i=n(1),o=function(){},a=function(e){return"undefined"!=typeof console&&(0,i.isFunction)(console[e])?console[e].bind(console):o},r={info:a("info"),warn:a("warn"),error:a("error")};t.logger=r;var s=function(){function e(e,t){if(!e)throw new Error(t)}function t(t,n){e(null!==t&&void 0!==t,n)}return{assert:e,assertParam:t}}();t.debug=s},function(e,t,n){t.default=void 0;var i=n(0),o={_formatNumberCore:function(e,t,n){
if("currency"===t){n.precision=n.precision||0;var o=this.format(e,(0,i.extend)({},n,{type:"fixedpoint"})),a=this.getCurrencySymbol().symbol.replace("$","$$$$");return o=o.replace(/^(\D*)(\d.*)/,"$1"+a+"$2")}return this.callBase.apply(this,arguments)},getCurrencySymbol:function(){return{symbol:"$"}},getOpenXmlCurrencyFormat:function(){return"$#,##0{0}_);\\($#,##0{0}\\)"}};t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=n(7),u=n(1),c=n(3),d=function(){function e(t,n){o(this,e),n=n||{},this._action=t,this._context=n.context||(0,l.getWindow)(),this._beforeExecute=n.beforeExecute,this._afterExecute=n.afterExecute,this._component=n.component,this._validatingTargetName=n.validatingTargetName;var i=this._excludeValidators={};if(n.excludeValidators)for(var a=0;a<n.excludeValidators.length;a++)i[n.excludeValidators[a]]=!0}return r(e,[{key:"execute",value:function(){var e={action:this._action,args:Array.prototype.slice.call(arguments),context:this._context,component:this._component,validatingTargetName:this._validatingTargetName,cancel:!1,handled:!1},t=this._beforeExecute,n=this._afterExecute,i=e.args[0]||{};if(this._validateAction(e)&&(null===t||void 0===t?void 0:t.call(this._context,e),!e.cancel)){var o=this._executeAction(e);if(!i.cancel)return null===n||void 0===n?void 0:n.call(this._context,e),o}}},{key:"_validateAction",value:function(t){var n=this._excludeValidators,i=e.executors;for(var o in i)if(!n[o]){var a,r=i[o];if(null===(a=r.validate)||void 0===a?void 0:a.call(r,t),t.cancel)return!1}return!0}},{key:"_executeAction",value:function(t){var n,i=e.executors;for(var o in i){var a,r=i[o];if(null===(a=r.execute)||void 0===a?void 0:a.call(r,t),t.handled){n=t.result;break}}return n}}],[{key:"registerExecutor",value:function(t,n){return(0,u.isPlainObject)(t)?void(0,c.each)(t,e.registerExecutor):void(e.executors[t]=n)}},{key:"unregisterExecutor",value:function(){for(var t=arguments.length,n=new Array(t),i=0;i<t;i++)n[i]=arguments[i];(0,c.each)(n,function(){delete e.executors[this]})}}]),e}();t.default=d,d.executors={};var h=function(e){return function(t){if(t.args.length){var n=t.args[0],i=n[t.validatingTargetName]||n.element;i&&e((0,s.default)(i))&&(t.cancel=!0)}}};d.registerExecutor({disabled:{validate:h(function(e){return e.is(".dx-state-disabled, .dx-state-disabled *")})},readOnly:{validate:h(function(e){return e.is(".dx-state-readonly, .dx-state-readonly *:not(.dx-state-independent)")})},undefined:{execute:function(e){e.action||(e.result=void 0,e.handled=!0)}},func:{execute:function(e){(0,u.isFunction)(e.action)&&(e.result=e.action.call(e.context,e.args[0]),e.handled=!0)}}}),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(46),s=i(n(23)),l=n(4),u=n(1),c=n(0),d=n(18),h=n(7),f=i(n(11)),p=i(n(13)),g=i(n(9)),_=i(n(65)),m=n(75),v=n(8),y=i(n(285)),b=n(429),x=i(n(284)),w=n(431),C=n(6),k="dxScrollable",S="dxScrollableStrategy",D="dx-scrollable",I="dx-scrollable-disabled",T="dx-scrollable-container",O="dx-scrollable-wrapper",E="dx-scrollable-content",A="vertical",P="horizontal",R="both",M=_.default.inherit({_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{disabled:!1,onScroll:null,direction:A,showScrollbar:"onScroll",useNative:!0,bounceEnabled:!0,scrollByContent:!0,scrollByThumb:!1,onUpdated:null,onStart:null,onEnd:null,onBounce:null,onStop:null,useSimulatedScrollbar:!1,useKeyboard:!0,inertiaEnabled:!0,pushBackValue:0,updateManually:!1})},_defaultOptionsRules:function(){return this.callBase().concat((0,w.deviceDependentOptions)(),[{device:function(){return r.nativeScrolling&&"android"===p.default.real().platform&&!s.default.mozilla},options:{useSimulatedScrollbar:!0}},{device:function(){return"ios"===p.default.real().platform},options:{pushBackValue:1}}])},_initOptions:function(e){this.callBase(e),"useSimulatedScrollbar"in e||this._setUseSimulatedScrollbar()},_setUseSimulatedScrollbar:function(){this.initialOption("useSimulatedScrollbar")||this.option("useSimulatedScrollbar",!this.option("useNative"))},_init:function(){this.callBase(),this._initScrollableMarkup(),this._locked=!1},_getWindowDevicePixelRatio:function(){return(0,h.hasWindow)()?(0,h.getWindow)().devicePixelRatio:1},_visibilityChanged:function(e){e?(this.update(),this._updateRtlPosition(),this._savedScrollOffset&&this.scrollTo(this._savedScrollOffset),delete this._savedScrollOffset):this._savedScrollOffset=this.scrollOffset()},_initScrollableMarkup:function(){var e=this.$element().addClass(D),t=this._$container=(0,o.default)("<div>").addClass(T),n=this._$wrapper=(0,o.default)("<div>").addClass(O),i=this._$content=(0,o.default)("<div>").addClass(E);f.default.hasDocumentProperty("onbeforeactivate")&&s.default.msie&&s.default.version<12&&a.default.on(e,(0,v.addNamespace)("beforeactivate",k),function(e){(0,o.default)(e.target).is(m.focusable)||e.preventDefault()}),i.append(e.contents()).appendTo(t),t.appendTo(n),n.appendTo(e)},_dimensionChanged:function(){this.update(),this._updateRtlPosition()},_initMarkup:function(){this.callBase(),this._renderDirection()},_render:function(){this._renderStrategy(),this._attachEventHandlers(),this._renderDisabledState(),this._createActions(),this.update(),this.callBase(),this._rtlConfig={scrollRight:0,clientWidth:this._container().get(0).clientWidth,windowPixelRatio:this._getWindowDevicePixelRatio()},this._updateRtlPosition()},_isHorizontalAndRtlEnabled:function(){return this.option("rtlEnabled")&&this.option("direction")!==A},_updateRtlPosition:function(){var e=this;this._updateBounds(),this._isHorizontalAndRtlEnabled()&&(0,l.deferUpdate)(function(){var t=e._getMaxOffset().left-e._rtlConfig.scrollRight;t<=0&&(t=0,e._rtlConfig.scrollRight=e._getMaxOffset().left),(0,l.deferRender)(function(){e.scrollLeft()!==t&&(e._rtlConfig.skipUpdating=!0,e.scrollTo({left:t}),e._rtlConfig.skipUpdating=!1)})})},_getMaxOffset:function(){var e=this._container().get(0),t=e.scrollWidth,n=e.clientWidth,i=e.scrollHeight,o=e.clientHeight;return{left:t-n,top:i-o}},_updateBounds:function(){this._strategy.updateBounds()},_attachEventHandlers:function(){var e=this._strategy,t={getDirection:e.getDirection.bind(e),validate:this._validate.bind(this),isNative:this.option("useNative"),scrollTarget:this._$container};a.default.off(this._$wrapper,"."+k),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.init,k),t,this._initHandler.bind(this)),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.start,k),e.handleStart.bind(e)),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.move,k),e.handleMove.bind(e)),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.end,k),e.handleEnd.bind(e)),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.cancel,k),e.handleCancel.bind(e)),a.default.on(this._$wrapper,(0,v.addNamespace)(y.default.stop,k),e.handleStop.bind(e)),a.default.off(this._$container,"."+k),a.default.on(this._$container,(0,v.addNamespace)("scroll",k),e.handleScroll.bind(e))},_updateRtlConfig:function(){if(this._isHorizontalAndRtlEnabled()&&!this._rtlConfig.skipUpdating){var e=this._container().get(0),t=e.clientWidth,n=e.scrollLeft,i=this._getWindowDevicePixelRatio();this._rtlConfig.windowPixelRatio===i&&this._rtlConfig.clientWidth===t&&(this._rtlConfig.scrollRight=this._getMaxOffset().left-n),this._rtlConfig.clientWidth=t,this._rtlConfig.windowPixelRatio=i}},_validate:function(e){return!this._isLocked()&&(this._updateIfNeed(),this._strategy.validate(e))},_initHandler:function(){var e=this._strategy;e.handleInit.apply(e,arguments)},_renderDisabledState:function(){this.$element().toggleClass(I,this.option("disabled")),this.option("disabled")?this._lock():this._unlock()},_renderDirection:function(){this.$element().removeClass("dx-scrollable-"+P).removeClass("dx-scrollable-"+A).removeClass("dx-scrollable-"+R).addClass("dx-scrollable-"+this.option("direction"))},_renderStrategy:function(){this._createStrategy(),this._strategy.render(),this.$element().data(S,this._strategy)},_createStrategy:function(){this._strategy=this.option("useNative")?new x.default(this):new b.SimulatedStrategy(this)},_createActions:function(){this._strategy&&this._strategy.createActions()},_clean:function(){this._strategy&&this._strategy.dispose()},_optionChanged:function(e){switch(e.name){case"onStart":case"onEnd":case"onStop":case"onUpdated":case"onScroll":case"onBounce":this._createActions();break;case"direction":this._resetInactiveDirection(),this._invalidate();break;case"useNative":this._setUseSimulatedScrollbar(),this._invalidate();break;case"inertiaEnabled":case"scrollByContent":case"scrollByThumb":case"bounceEnabled":case"useKeyboard":case"showScrollbar":case"useSimulatedScrollbar":case"pushBackValue":this._invalidate();break;case"disabled":this._renderDisabledState(),this._strategy&&this._strategy.disabledChanged();break;case"updateManually":break;case"width":this.callBase(e),this._updateRtlPosition();break;default:this.callBase(e)}},_resetInactiveDirection:function(){var e=this._getInactiveProp();if(e&&(0,h.hasWindow)()){var t=this.scrollOffset();t[e]=0,this.scrollTo(t)}},_getInactiveProp:function(){var e=this.option("direction");return e===A?"left":e===P?"top":void 0},_location:function(){return this._strategy.location()},_normalizeLocation:function(e){if((0,u.isPlainObject)(e)){var t=(0,l.ensureDefined)(e.left,e.x),n=(0,l.ensureDefined)(e.top,e.y);return{left:(0,u.isDefined)(t)?-t:void 0,top:(0,u.isDefined)(n)?-n:void 0}}var i=this.option("direction");return{left:i!==A?-e:void 0,top:i!==P?-e:void 0}},_isLocked:function(){return this._locked},_lock:function(){this._locked=!0},_unlock:function(){this.option("disabled")||(this._locked=!1)},_isDirection:function(e){var t=this.option("direction");return e===A?t!==P:e===P?t!==A:t===e},_updateAllowedDirection:function(){var e=this._strategy._allowedDirections();this._isDirection(R)&&e.vertical&&e.horizontal?this._allowedDirectionValue=R:this._isDirection(P)&&e.horizontal?this._allowedDirectionValue=P:this._isDirection(A)&&e.vertical?this._allowedDirectionValue=A:this._allowedDirectionValue=null},_allowedDirection:function(){return this._allowedDirectionValue},_container:function(){return this._$container},$content:function(){return this._$content},content:function(){return(0,d.getPublicElement)(this._$content)},scrollOffset:function(){return this._getScrollOffset()},_getScrollOffset:function(){return{top:-this._location().top,left:-this._location().left}},scrollTop:function(){return this.scrollOffset().top},scrollLeft:function(){return this.scrollOffset().left},clientHeight:function(){return this._$container.height()},scrollHeight:function(){return this.$content().outerHeight()-2*this._strategy.verticalOffset()},clientWidth:function(){return this._$container.width()},scrollWidth:function(){return this.$content().outerWidth()},update:function(){if(this._strategy)return(0,C.when)(this._strategy.update()).done(function(){this._updateAllowedDirection()}.bind(this))},scrollBy:function(e){e=this._normalizeLocation(e),(e.top||e.left)&&(this._updateIfNeed(),this._strategy.scrollBy(e),this._updateRtlConfig())},scrollTo:function(e){e=this._normalizeLocation(e),this._updateIfNeed();var t=this._location();this.option("useNative")||(e=this._strategy._applyScaleRatio(e),t=this._strategy._applyScaleRatio(t));var n=this._normalizeLocation({left:t.left-(0,l.ensureDefined)(e.left,t.left),top:t.top-(0,l.ensureDefined)(e.top,t.top)});(n.top||n.left)&&(this._strategy.scrollBy(n),this._updateRtlConfig())},scrollToElement:function(e,t){var n=(0,o.default)(e),i=this.$content().find(e).length,a=n.parents("."+D).length-n.parents("."+E).length===0;if(i&&a){var r={top:0,left:0},s=this.option("direction");s!==A&&(r.left=this.getScrollElementPosition(n,P,t)),s!==P&&(r.top=this.getScrollElementPosition(n,A,t)),this.scrollTo(r)}},scrollToElementTopLeft:function(e){var t=(0,o.default)(e),n=this.$content().find(e).length,i=t.parents("."+D).length-t.parents("."+E).length===0;if(n&&i){var a={top:0,left:0},r=this.option("direction");if(r!==A){var s=this._elementPositionRelativeToContent(t,"left");a.left=this.option("rtlEnabled")===!0?s+t.width()-this.clientWidth():s}r!==P&&(a.top=this._elementPositionRelativeToContent(t,"top")),this.scrollTo(a)}},getScrollElementPosition:function(e,t,n){n=n||{};var i=t===A,o=(i?n.top:n.left)||0,a=(i?n.bottom:n.right)||0,r=i?this._strategy.verticalOffset():0,s=this._elementPositionRelativeToContent(e,i?"top":"left"),l=s-r,u=e[i?"outerHeight":"outerWidth"](),c=i?this.scrollTop():this.scrollLeft(),d=this._container().get(0)[i?"clientHeight":"clientWidth"],h=c-l+o,f=c-l-u+d-a;return h<=0&&f>=0?c:c-(Math.abs(h)>Math.abs(f)?f:h)},_elementPositionRelativeToContent:function(e,t){for(var n=0;this._hasScrollContent(e);)n+=e.position()[t],e=e.offsetParent();return n},_hasScrollContent:function(e){var t=this.$content();return e.closest(t).length&&!e.is(t)},_updateIfNeed:function(){this.option("updateManually")||this.update()},_useTemplates:function(){return!1}});(0,g.default)(k,M);var B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(457)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){var e=m.image.testFormats(F);return{unsupported:e.unsupported,supported:e.supported.concat(L)}}function a(e,t,n){return n=n||o(),e=String(e).toUpperCase(),n.supported.indexOf(e)!==-1?e:void(n.unsupported.indexOf(e)!==-1&&t&&t("W2108",[e]))}function r(e){return"SVG"===e?m.svg.getData:"PDF"===e?m.pdf.getData:m.image.getData}function s(e,t){var n=(0,g.getWindow)().document,i=n.createElement("iframe");i.onload=l(e,t),i.style.position="fixed",i.style.width="0",i.style.height="0",i.style.right="0",i.style.bottom="0",n.body.appendChild(i)}function l(e,t){return function(){var t=this,n=this.contentWindow,i=n.document.createElement("img");n.document.body.appendChild(i);var o=function(){t.parentElement.removeChild(t)};i.addEventListener("load",function(){n.focus(),n.print()}),i.addEventListener("error",o),n.addEventListener("afterprint",function(){setTimeout(o,0)}),i.src=e}}function u(e,t,n){var i=S-T,o=S+I+P+n*A,a={rect:{width:T-2*P,height:A,x:i+P,y:o},text:{x:i+(e.rtl?T-E:E),y:o+A-O}};return"printing"===t&&(a.separator={stroke:e.button.default.borderColor,"stroke-width":P,cursor:"pointer",sharp:"v",d:"M "+i+" "+(o+A-P)+" L "+(i+T)+" "+(o+A-P)}),a}function c(e,t,n){var i={},o=n.type,a=n.format,r=u(t,o,n.itemIndex),s=(0,_.patchFontOptions)(t.font);s["pointer-events"]="none";var l=e.g().attr({"class":H+"-list-item"});i[z]=o,a&&(i[N]=a);var c=e.rect();c.attr(r.rect).css({cursor:"pointer","pointer-events":"all"}).data(i),c.on(x.start+".export",function(){return c.attr({fill:t.button.hover.backgroundColor})}).on(x.end+".export",function(){return c.attr({fill:null})}),c.append(l);var d=e.text(n.text).css(s).attr(r.text).append(l);return"printing"===o&&e.path(null,"line").attr(r.separator).append(l),{g:l,rect:c,resetState:function(){return c.attr({fill:null})},fixPosition:function(){var e=d.getBBox();d.move(r.text.x-e.x-(t.rtl?e.width:0))}}}function d(e,t){var n=[];return t.printingEnabled&&n.push(c(e,t,{type:"printing",text:v.default.format("vizExport-printingButtonText"),itemIndex:n.length})),n=t.formats.reduce(function(n,i){return n.push(c(e,t,{type:"exporting",text:v.default.getFormatter("vizExport-exportButtonText")(i),format:i,itemIndex:n.length})),n},n)}function h(e){var t=j.exec(e);return null===t||void 0===t?void 0:t[1]}function f(e,t,n,i){return(i||t.format)&&(i=a(i||t.format,e._incidentOccurred)),{format:i||V,fileName:n||t.fileName||"file",proxyUrl:t.proxyUrl,backgroundColor:t.backgroundColor,width:e._canvas.width,height:e._canvas.height,margin:t.margin,svgToCanvas:t.svgToCanvas,forceProxy:t.forceProxy,exportingAction:e._createActionByOption("onExporting"),exportedAction:e._createActionByOption("onExported"),fileSavingAction:e._createActionByOption("onFileSaving")}}t.plugin=t.ExportMenu=t.combineMarkups=t.exportWidgets=t.getMarkup=t.exportFromMarkup=void 0;var p=n(0),g=n(7),_=n(14),m=n(181),v=i(n(10)),y=n(1),b=n(57),x=n(158),w=i(n(25)),C=n(106),k=[w.default.down,w.default.move].join(" "),S=35,D=[[9,12,26,12,26,14,9,14],[9,17,26,17,26,19,9,19],[9,22,26,22,26,24,9,24]],I=4,T=120,O=8,E=15,A=30,P=1,R=10,M=2,B=3,V="PNG",F=[V,"JPEG","GIF"],L=["PDF","SVG"],H="dx-export-menu",z="export-element-type",N="export-element-format",j=/data-backgroundcolor="([^"]*)"/,W=function(e,t){t.format=a(t.format)||V,t.fileName=t.fileName||"file",t.exportingAction=t.onExporting,t.exportedAction=t.onExported,t.fileSavingAction=t.onFileSaving,t.margin=(0,y.isDefined)(t.margin)?t.margin:R,t.backgroundColor=(0,y.isDefined)(t.backgroundColor)?t.backgroundColor:h(e)||(0,b.getTheme)().backgroundColor,(0,m.export)(e,t,r(t.format))};t.exportFromMarkup=W;var G=function(e){return $(e).markup};t.getMarkup=G;var q=function(e,t){t=t||{};var n=$(e,{gridLayout:t.gridLayout,verticalAlignment:t.verticalAlignment,horizontalAlignment:t.horizontalAlignment});t.width=n.width,t.height=n.height,W(n.markup,t)};t.exportWidgets=q;var $=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};Array.isArray(e)?Array.isArray(e[0])||(e=e.map(function(e){return[e]})):e=[[e]];var n=!t.gridLayout,i=e.reduce(function(e,t,i){var o=t.reduce(function(e,t,n){var o=t.getSize(),a=t.option("backgroundColor")||(0,b.getTheme)(t.option("theme")).backgroundColor;return a&&e.backgroundColors.indexOf(a)===-1&&e.backgroundColors.push(a),e.hOffset=e.width,e.width+=o.width,e.height=Math.max(e.height,o.height),e.itemWidth=Math.max(e.itemWidth,o.width),e.items.push({markup:t.svg(),width:o.width,height:o.height,c:n,r:i,hOffset:e.hOffset}),e},{items:[],height:0,itemWidth:0,hOffset:0,width:0,backgroundColors:e.backgroundColors});return e.rowOffsets.push(e.totalHeight),e.rowHeights.push(o.height),e.totalHeight+=o.height,e.items=e.items.concat(o.items),e.itemWidth=Math.max(e.itemWidth,o.itemWidth),e.maxItemLen=Math.max(e.maxItemLen,o.items.length),e.totalWidth=n?Math.max(e.totalWidth,o.width):e.maxItemLen*e.itemWidth,e},{items:[],rowOffsets:[],rowHeights:[],itemWidth:0,totalHeight:0,maxItemLen:0,totalWidth:0,backgroundColors:[]}),o='data-backgroundcolor="'.concat(1===i.backgroundColors.length?i.backgroundColors[0]:"",'" '),a=function(e){var n=t.verticalAlignment,o=i.rowHeights[e.r]-e.height;return i.rowOffsets[e.r]+("bottom"===n?o:"center"===n?o/2:0)},r=function(e){if(n)return e.hOffset;var o=t.horizontalAlignment,a=i.itemWidth,r=a-e.width;return e.c*a+("right"===o?r:"center"===o?r/2:0)},s=i.totalHeight,l=i.totalWidth;return{markup:"<svg "+o+'height="'+s+'" width="'+l+'" version="1.1" xmlns="http://www.w3.org/2000/svg">'+i.items.map(function(e){return'<g transform="translate('.concat(r(e),",").concat(a(e),')">').concat(e.markup,"</g>")}).join("")+"</svg>",width:l,height:s}};t.combineMarkups=$;var K=function(e){var t=this._renderer=e.renderer;this._incidentOccurred=e.incidentOccurred,this._exportTo=e.exportTo,this._print=e.print,this._shadow=t.shadowFilter("-50%","-50%","200%","200%",M,6,B),this._shadow.attr({opacity:.8}),this._group=t.g().attr({"class":H,"hidden-for-export":!0}).linkOn(t.root,{name:"export-menu",after:"peripheral"}),this._buttonGroup=t.g().attr({"class":H+"-button"}).append(this._group),this._listGroup=t.g().attr({"class":H+"-list"}).append(this._group),this._overlay=t.rect(-T+S,S+I,T,0),this._overlay.attr({"stroke-width":P,cursor:"pointer",rx:4,ry:4,filter:this._shadow.id}),this._overlay.data({"export-element-type":"list"}),this.validFormats=o(),this._subscribeEvents()};t.ExportMenu=K,(0,p.extend)(K.prototype,{getLayoutOptions:function(){if(this._hiddenDueToLayout)return{width:0,height:0,cutSide:"vertical",cutLayoutSide:"top"};var e=this._buttonGroup.getBBox();return e.cutSide="vertical",e.cutLayoutSide="top",e.height+=R,e.position={vertical:"top",horizontal:"right"},e.verticalAlignment="top",e.horizontalAlignment="right",e},shift:function(e,t){this._group.attr({translateY:this._group.attr("translateY")+t})},draw:function(e,t,n){this._group.move(e-S-M-B+n.left,Math.floor(t/2-S/2));var i=this.getLayoutOptions();return(i.width>e||i.height>t)&&this.freeSpace(),this},show:function(){this._group.linkAppend()},hide:function(){this._group.linkRemove()},setOptions:function(e){var t=this;this._options=e,e.formats?e.formats=e.formats.reduce(function(e,n){return n=a(n,t._incidentOccurred,t.validFormats),n&&e.push(n),e},[]):e.formats=this.validFormats.supported.slice(),e.printingEnabled=void 0===e.printingEnabled||e.printingEnabled,e.enabled&&(e.formats.length||e.printingEnabled)?(this.show(),this._updateButton(),this._updateList(),this._hideList()):this.hide()},dispose:function(){this._unsubscribeEvents(),this._group.linkRemove().linkOff(),this._group.dispose(),this._shadow.dispose()},layoutOptions:function(){return this._options.enabled&&{horizontalAlignment:"right",verticalAlignment:"top",weak:!0}},measure:function(){this._fillSpace();var e=this._options.button.margin;return[S+e.left+e.right,S+e.top+e.bottom]},move:function(e){var t=this._options.button.margin;this._group.attr({translateX:Math.round(e[0])+t.left,translateY:Math.round(e[1])+t.top})},_fillSpace:function(){this._hiddenDueToLayout=!1,this.show()},freeSpace:function(){this._incidentOccurred("W2107"),this._hiddenDueToLayout=!0,this.hide()},_hideList:function(){this._listGroup.remove(),this._listShown=!1,this._setButtonState("default"),this._menuItems.forEach(function(e){return e.resetState()})},_showList:function(){this._listGroup.append(this._group),this._listShown=!0,this._menuItems.forEach(function(e){return e.fixPosition()})},_setButtonState:function(e){var t=this._options.button[e];this._button.attr({stroke:t.borderColor,fill:t.backgroundColor}),this._icon.attr({fill:t.color})},_subscribeEvents:function(){var e=this;this._renderer.root.on(w.default.up+".export",function(t){var n=t.target[z];return n?void("button"===n?e._listShown?(e._setButtonState("default"),e._hideList()):(e._setButtonState("focus"),e._showList()):"printing"===n?(e._print(),e._hideList()):"exporting"===n&&(e._exportTo(t.target[N]),e._hideList())):void(e._button&&e._hideList())}),this._listGroup.on(k,function(e){return e.stopPropagation()}),this._buttonGroup.on(w.default.enter,function(){return e._setButtonState("hover")}),this._buttonGroup.on(w.default.leave,function(){return e._setButtonState(e._listShown?"focus":"default")}),this._buttonGroup.on(w.default.down+".export",function(){return e._setButtonState("active")})},_unsubscribeEvents:function(){this._renderer.root.off(".export"),this._listGroup.off(),this._buttonGroup.off()},_updateButton:function(){var e=this._renderer,t=this._options,n={"export-element-type":"button"};this._button||(this._button=e.rect(0,0,S,S).append(this._buttonGroup),this._button.attr({rx:4,ry:4,fill:t.button.default.backgroundColor,stroke:t.button.default.borderColor,"stroke-width":1,cursor:"pointer"}),this._button.data(n),this._icon=e.path(D).append(this._buttonGroup),this._icon.attr({fill:t.button.default.color,cursor:"pointer"}),this._icon.data(n),this._buttonGroup.setTitle(v.default.format("vizExport-titleMenuText")))},_updateList:function(){var e=this._options,t=e.button.default,n=this._listGroup,i=d(this._renderer,e);this._shadow.attr({color:e.shadowColor}),this._overlay.attr({height:i.length*A+2*P,fill:t.backgroundColor,stroke:t.borderColor}),n.clear(),this._overlay.append(n),i.forEach(function(e){return e.g.append(n)}),this._menuItems=i}});var U={name:"export",init:function(){var e=this;this._exportMenu=new K({renderer:this._renderer,incidentOccurred:this._incidentOccurred,print:function(){return e.print()},exportTo:function(t){return e.exportTo(void 0,t)}}),this._layout.add(this._exportMenu)},dispose:function(){this._exportMenu.dispose()},members:{_getExportMenuOptions:function(){return(0,p.extend)({},this._getOption("export"),{rtl:this._getOption("rtlEnabled",!0)})},_disablePointerEvents:function(){var e=this._renderer.root.attr("pointer-events");return this._renderer.root.attr({"pointer-events":"none"}),e},exportTo:function(e,t){var n=this,i=this._exportMenu,o=f(this,this._getOption("export")||{},e,t);i&&i.hide();var a=this._disablePointerEvents(),s=(0,m.export)(this._renderer.root.element,o,r(o.format)).fail(C.logger.error).always(function(){n._renderer.root.attr({"pointer-events":a})});return i&&i.show(),s},print:function(){var e=this,t=this._exportMenu,n=f(this,this._getOption("export")||{});n.exportingAction=null,n.exportedAction=null,n.margin=0,n.format="PNG",n.forceProxy=!0,n.fileSavingAction=function(e){s("data:image/png;base64,".concat(e.data),{__test:n.__test}),e.cancel=!0};var i=this._disablePointerEvents();t&&t.hide();var o=(0,m.export)(this._renderer.root.element,n,r(n.format)).fail(C.logger.error).always(function(){e._renderer.root.attr({"pointer-events":i})});return t&&t.show(),o}},customize:function(e){var t=e.prototype;e.addChange({code:"EXPORT",handler:function(){this._exportMenu.setOptions(this._getExportMenuOptions()),this._change(["LAYOUT"])},isThemeDependent:!0,isOptionChange:!0,option:"export"}),t._optionChangesMap.onExporting="EXPORT",t._optionChangesMap.onExported="EXPORT",t._optionChangesMap.onFileSaving="EXPORT"},fontFields:["export.font"]};t.plugin=U},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return!0}function a(){return!1}function r(e,t){return!(e.width===t.width&&e.height===t.height&&e.left===t.left&&e.top===t.top&&e.right===t.right&&e.bottom===t.bottom)}function s(e){e.component._eventsStrategy.hasEvent("incidentOccurred")||A.apply(null,[e.target.id].concat(e.target.args||[]))}function l(e){return e.reduce(function(e,t){return t>0&&!e?t:e},0)}function u(e){e.forEach(function(e){return e()})}function c(e){return(0,g.isDefined)(e)&&e>0}t.default=void 0;var d=i(n(2)),h=n(4),f=n(7),p=i(n(11)),g=n(1),_=n(3),m=i(n(114)),v=n(0),y=n(322),b=i(n(65)),x=n(172),w=n(14),C=i(n(543)),k=n(171),S=i(n(917)),D=i(n(13)),I=i(n(5)),T=n(6),O=n(918),E=Math.floor,A=C.default.log,P="rtlEnabled",R="dx-sized-element",M=b.default.prototype.option,B=function(){var e={_initTemplates:function(){},ctor:function(e,t){this.callBase(e,t);var n=p.default.createElement("div"),i=t&&(0,g.isNumeric)(t.width)?t.width+"px":"100%",o=t&&(0,g.isNumeric)(t.height)?t.height+"px":this._getDefaultSize().height+"px";p.default.setStyle(n,"width",i),p.default.setStyle(n,"height",o),p.default.setClass(n,R),p.default.insertElement(e,n)}},t=b.default.inherit(e),n=t.inherit;return t.inherit=function(e){for(var t in e)((0,g.isFunction)(e[t])&&"_"!==t.substr(0,1)&&"option"!==t||"_dispose"===t||"_optionChanged"===t)&&(e[t]=h.noop);return n.call(this,e)},t},V=!(0,f.hasWindow)(),F=V?B():b.default.inherit({_eventsMap:{onIncidentOccurred:{name:"incidentOccurred"},onDrawn:{name:"drawn"}},_getDefaultOptions:function(){return(0,v.extend)(this.callBase(),{onIncidentOccurred:s})},_useLinks:!0,_init:function(){var e=this;e._$element.children("."+R).remove(),e.callBase.apply(e,arguments),e._changesLocker=0,e._optionChangedLocker=0,e._asyncFirstDrawing=!0,e._changes=(0,x.changes)(),e._suspendChanges(),e._themeManager=e._createThemeManager(),e._themeManager.setCallback(function(){e._requestChange(e._themeDependentChanges)}),e._renderElementAttributes(),e._initRenderer();var t=e._useLinks&&e._renderer.root;t&&t.enableLinks().virtualLink("core").virtualLink("peripheral"),e._renderVisibilityChange(),e._attachVisibilityChangeHandlers(),e._toggleParentsScrollSubscription(this._isVisible()),e._initEventTrigger(),e._incidentOccurred=(0,O.createIncidentOccurred)(e.NAME,e._eventTrigger),e._layout=new S.default,t&&t.linkAfter("core"),e._initPlugins(),e._initCore(),t&&t.linkAfter(),e._change(e._initialChanges)},_createThemeManager:function(){return new y.BaseThemeManager(this._getThemeManagerOptions())},_getThemeManagerOptions:function(){return{themeSection:this._themeSection,fontFields:this._fontFields}},_initialChanges:["LAYOUT","RESIZE_HANDLER","THEME","DISABLED"],_initPlugins:function(){var e=this;(0,_.each)(e._plugins,function(t,n){n.init.call(e)})},_disposePlugins:function(){var e=this;(0,_.each)(e._plugins.slice().reverse(),function(t,n){n.dispose.call(e)})},_change:function(e){this._changes.add(e)},_suspendChanges:function(){++this._changesLocker},_resumeChanges:function(){var e=this;0===--e._changesLocker&&e._changes.count()>0&&!e._applyingChanges&&(e._renderer.lock(),e._applyingChanges=!0,e._applyChanges(),e._changes.reset(),e._applyingChanges=!1,e._changesApplied(),e._renderer.unlock(),e._optionsQueue&&e._applyQueuedOptions(),e.resolveItemsDeferred(e._legend?[e._legend]:[]),e._optionChangedLocker++,e._notify(),e._optionChangedLocker--)},resolveItemsDeferred:function(e){this._resolveDeferred(this._getTemplatesItems(e))},_collectTemplatesFromItems:function(e){return e.reduce(function(e,t){return{items:e.items.concat(t.getTemplatesDef()),groups:e.groups.concat(t.getTemplatesGroups())}},{items:[],groups:[]})},_getTemplatesItems:function(e){var t=this._collectTemplatesFromItems(e),n=this._getExtraTemplatesItems();return{items:n.items.concat(t.items),groups:n.groups.concat(t.groups),launchRequest:[n.launchRequest],doneRequest:[n.doneRequest]}},_getExtraTemplatesItems:function(){return{items:[],groups:[],launchRequest:function(){},doneRequest:function(){}}},_resolveDeferred:function(e){var t=e.items,n=e.launchRequest,i=e.doneRequest,o=e.groups,a=this;if(a._setGroupsVisibility(o,"hidden"),a._changesApplying)return a._changesApplying=!1,void u(i);var r=!0;T.when.apply(a,t).done(function(){if(r)return void a._setGroupsVisibility(o,"visible");u(n),a._changesApplying=!0;var e=["LAYOUT","FULL_RENDER"];a._asyncFirstDrawing?(e.push("FORCE_FIRST_DRAWING"),a._asyncFirstDrawing=!1):e.push("FORCE_DRAWING"),a._requestChange(e),a._setGroupsVisibility(o,"visible")}),r=!1},_setGroupsVisibility:function(e,t){e.forEach(function(e){return e.attr({visibility:t})})},_applyQueuedOptions:function(){var e=this,t=e._optionsQueue;e._optionsQueue=null,e.beginUpdate(),(0,_.each)(t,function(e,t){t()}),e.endUpdate()},_requestChange:function(e){this._suspendChanges(),this._change(e),this._resumeChanges()},_applyChanges:function(){var e,t=this,n=t._changes,i=t._totalChangesOrder,o=i.length;for(e=0;e<o;++e)n.has(i[e])&&t["_change_"+i[e]]()},_optionChangesOrder:["EVENTS","THEME","RENDERER","RESIZE_HANDLER"],_layoutChangesOrder:["ELEMENT_ATTR","CONTAINER_SIZE","LAYOUT"],_customChangesOrder:["DISABLED"],_change_EVENTS:function(){this._eventTrigger.applyChanges()},_change_THEME:function(){this._setThemeAndRtl()},_change_RENDERER:function(){this._setRendererOptions()},_change_RESIZE_HANDLER:function(){this._setupResizeHandler()},_change_ELEMENT_ATTR:function(){this._renderElementAttributes(),this._change(["CONTAINER_SIZE"])},_change_CONTAINER_SIZE:function(){this._updateSize()},_change_LAYOUT:function(){this._setContentSize()},_change_DISABLED:function(){var e=this._renderer,t=e.root;this.option("disabled")?(this._initDisabledState=t.attr("pointer-events"),t.attr({"pointer-events":"none",filter:e.getGrayScaleFilter().id})):"none"===t.attr("pointer-events")&&t.attr({"pointer-events":(0,g.isDefined)(this._initDisabledState)?this._initDisabledState:null,filter:null})},_themeDependentChanges:["RENDERER"],_initRenderer:function(){var e=this;e._canvas=e._calculateCanvas(),e._renderer=new k.Renderer({cssClass:e._rootClassPrefix+" "+e._rootClass,pathModified:e.option("pathModified"),container:e._$element[0]}),e._renderer.resize(e._canvas.width,e._canvas.height)},_disposeRenderer:function(){this._renderer.dispose();
},_getAnimationOptions:h.noop,render:function(){this._requestChange(["CONTAINER_SIZE"]);var e=this._isVisible();this._toggleParentsScrollSubscription(e),!e&&this._stopCurrentHandling()},_toggleParentsScrollSubscription:function(e){var t=(0,d.default)(this._renderer.root.element).parents(),n="scroll.viz_widgets";"generic"===D.default.real().platform&&(t=t.add((0,f.getWindow)())),this._proxiedTargetParentsScrollHandler=this._proxiedTargetParentsScrollHandler||function(){this._stopCurrentHandling()}.bind(this),I.default.off((0,d.default)().add(this._$prevRootParents),n,this._proxiedTargetParentsScrollHandler),e&&(I.default.on(t,n,this._proxiedTargetParentsScrollHandler),this._$prevRootParents=t)},_stopCurrentHandling:h.noop,_dispose:function(){var e=this;e.callBase.apply(e,arguments),e._toggleParentsScrollSubscription(!1),e._removeResizeHandler(),e._layout.dispose(),e._eventTrigger.dispose(),e._disposeCore(),e._disposePlugins(),e._disposeRenderer(),e._themeManager.dispose(),e._themeManager=e._renderer=e._eventTrigger=null},_initEventTrigger:function(){var e=this;e._eventTrigger=(0,O.createEventTrigger)(e._eventsMap,function(t){return e._createActionByOption(t)})},_calculateCanvas:function(){var e=this,t=e.option("size")||{},n=e.option("margin")||{},i=e._getDefaultSize()||{},o=function(t,n){if(c(t[n])||!(0,f.hasWindow)())return 0;var i=e._$element[n]();return i<=1?0:i},a=o(t,"width"),r=o(t,"height"),s={width:t.width<=0?0:E(l([t.width,a,i.width])),height:t.height<=0?0:E(l([t.height,r,i.height])),left:l([n.left,i.left]),top:l([n.top,i.top]),right:l([n.right,i.right]),bottom:l([n.bottom,i.bottom])};return(s.width-s.left-s.right<=0||s.height-s.top-s.bottom<=0)&&(s={width:0,height:0}),s},_updateSize:function(){var e=this,t=e._calculateCanvas();e._renderer.fixPlacement(),(r(e._canvas,t)||e.__forceRender)&&(e._canvas=t,e._recreateSizeDependentObjects(!0),e._renderer.resize(t.width,t.height),e._change(["LAYOUT"]))},_recreateSizeDependentObjects:h.noop,_getMinSize:function(){return[0,0]},_getAlignmentRect:h.noop,_setContentSize:function(){var e=this._canvas,t=this._layout,n=e.width>0&&e.height>0?[e.left,e.top,e.width-e.right,e.height-e.bottom]:[0,0,0,0];n=t.forward(n,this._getMinSize());var i=this._applySize(n)||n;t.backward(i,this._getAlignmentRect()||i)},_getOption:function(e,t){var n=this._themeManager.theme(e),i=this.option(e);return t?void 0!==i?i:n:(0,v.extend)(!0,{},n,i)},_setupResizeHandler:function(){var e=this,t=(0,w.parseScalar)(this._getOption("redrawOnResize",!0),!0);e._resizeHandler&&e._removeResizeHandler(),e._resizeHandler=(0,O.createResizeHandler)(function(){t?e._requestChange(["CONTAINER_SIZE"]):e._renderer.fixPlacement()}),m.default.add(e._resizeHandler)},_removeResizeHandler:function(){this._resizeHandler&&(m.default.remove(this._resizeHandler),this._resizeHandler.dispose(),this._resizeHandler=null)},_onBeginUpdate:h.noop,beginUpdate:function(){var e=this;return e._initialized&&e._isUpdateAllowed()&&(e._onBeginUpdate(),e._suspendChanges()),e.callBase.apply(e,arguments),e},endUpdate:function(){return this.callBase(),this._isUpdateAllowed()&&this._resumeChanges(),this},option:function(e){var t=this;return t._initialized&&t._applyingChanges&&(arguments.length>1||(0,g.isObject)(e))?(t._optionsQueue=t._optionsQueue||[],void t._optionsQueue.push(t._getActionForUpdating(arguments))):M.apply(t,arguments)},_getActionForUpdating:function(e){var t=this;return function(){M.apply(t,e)}},_clean:h.noop,_render:h.noop,_optionChanged:function(e){var t=this;if(!t._optionChangedLocker){var n=t.getPartialChangeOptionsName(e),i=[];n.length>0?n.forEach(function(e){return i.push(t._partialOptionChangesMap[e])}):i.push(t._optionChangesMap[e.name]),i=i.filter(function(e){return!!e}),t._eventTrigger.change(e.name)?t._change(["EVENTS"]):i.length>0?t._change(i):t.callBase.apply(t,arguments)}},_notify:h.noop,_changesApplied:h.noop,_optionChangesMap:{size:"CONTAINER_SIZE",margin:"CONTAINER_SIZE",redrawOnResize:"RESIZE_HANDLER",theme:"THEME",rtlEnabled:"THEME",encodeHtml:"THEME",elementAttr:"ELEMENT_ATTR",disabled:"DISABLED"},_partialOptionChangesMap:{},_partialOptionChangesPath:{},getPartialChangeOptionsName:function(e){var t=this,n=e.fullName,i=n.split(/[.]/),o=e.name,a=e.value,r=this._partialOptionChangesPath[o],s=[];return r&&(r===!0?s.push(o):(r.forEach(function(e){n.indexOf(e)>=0&&s.push(e)}),1===i.length&&("object"===(0,g.type)(a)?t._addOptionsNameForPartialUpdate(a,r,s):"array"===(0,g.type)(a)&&a.length>0&&a.every(function(e){return t._checkOptionsForPartialUpdate(e,r)})&&a.forEach(function(e){return t._addOptionsNameForPartialUpdate(e,r,s)})))),s.filter(function(e,t,n){return n.indexOf(e)===t})},_checkOptionsForPartialUpdate:function(e,t){return!Object.keys(e).some(function(e){return t.indexOf(e)===-1})},_addOptionsNameForPartialUpdate:function(e,t,n){var i=Object.keys(e);this._checkOptionsForPartialUpdate(e,t)&&i.forEach(function(e){return t.indexOf(e)>-1&&n.push(e)})},_visibilityChanged:function(){this.render()},_setThemeAndRtl:function(){this._themeManager.setTheme(this.option("theme"),this.option(P))},_getRendererOptions:function(){return{rtl:this.option(P),encodeHtml:this.option("encodeHtml"),animation:this._getAnimationOptions()}},_setRendererOptions:function(){this._renderer.setOptions(this._getRendererOptions())},svg:function(){return this._renderer.svg()},getSize:function(){var e=this._canvas||{};return{width:e.width,height:e.height}},isReady:a,_dataIsReady:o,_resetIsReady:function(){this.isReady=a},_drawn:function(){var e=this;e.isReady=a,e._dataIsReady()&&e._renderer.onEndAnimation(function(){e.isReady=o}),e._eventTrigger("drawn",{})}}),L=F;t.default=L,(0,x.replaceInherit)(F),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e>0?Math.round(e):0}function a(e,t,n,i){var o,r,s,l,u=[],c=i.nodes,d=t.length,h=0;for(r=0;r<d;++r){var f;s=t[r],o=new b.default,o._id=c.length,o.ctx=i.ctx,o.parent=e,o.level=n,o.index=u.length,o.data=s,i.buildNode(o),c.push(o),u.push(o),l=s[i.itemsField],null!==(f=l)&&void 0!==f&&f.length&&a(o,l,n+1,i),s[i.valueField]>0&&(o.value=Number(s[i.valueField])),h+=o.value}e.nodes=u,e.value=h}function r(e,t,n){var i,o,a=t.nodes,s=a.length;for(o=0;o<s;++o)i=a[o],n(e,i),i.isNode()&&r(e,i,n)}function s(e,t){t.color=t.data[e.colorField]||e.getColor(t)||t.parent.color,t.updateStyles(),t.tile=!t.ctx.forceReset&&t.tile||A[Number(t.isNode())](e,t),t.applyState()}function l(e,t){var n=e.renderer.simpleRect().append(e.group);return e.setTrackerData(t,n),n}function u(e,t){var n=e.renderer.simpleRect().append(e.group),i=e.renderer.simpleRect().append(e.group);return e.setTrackerData(t,i),{outer:n,inner:i}}function c(e,t){t.updateLabelStyle(),t.labelState.visible&&d(e,t,t.labelState,t.labelParams)}function d(e,t,n,i){var o=t.data[e.labelField];t.label=o?String(o):null,o=t.customLabel||t.label,o&&(t.text=e.renderer.text(o).attr(n.attr).css(n.css).append(e.group),e.setTrackerData(t,t.text))}function h(e,t){var n,i=t.nodes,o=[],a=[],r=0,s=o.length=a.length=i.length;for(n=0;n<s;++n)r+=i[n].value,o[n]={value:i[n].value,i:n};for(r>0&&e.algorithm({items:o.slice(),sum:r,rect:t.innerRect.slice(),isRotated:1&i[0].level,directions:e.directions}),n=0;n<s;++n)a[n]=o[n].rect||P;t.rects=a}function f(e,t){var n,i=t.parent.rects[t.index],o=e.rectOffsets;t.isNode()?(_(t.tile.outer,g(i,t.parent.innerRect,o.headerEdge,o.headerInner)),i=p(i,e.groupPadding),n=Math.min(e.headerHeight,i[3]-i[1]),t.rect=[i[0],i[1],i[2],i[1]+n],_(t.tile.inner,p(t.rect,o.headerEdge)),i[1]+=n,t.innerRect=i,h(e,t)):(t.rect=i,_(t.tile,g(i,t.parent.innerRect,o.tileEdge,o.tileInner)))}function p(e,t){return[e[0]+t,e[1]+t,e[2]-t,e[3]-t]}function g(e,t,n,i){return[e[0]+(e[0]===t[0]?n:+i),e[1]+(e[1]===t[1]?n:+i),e[2]-(e[2]===t[2]?n:-i),e[3]-(e[3]===t[3]?n:-i)]}function _(e,t){e.attr({x:t[0],y:t[1],width:T(t[2]-t[0],0),height:T(t[3]-t[1],0)})}function m(e,t){t.text&&t.labelState.visible&&v(t,t.labelParams)}function v(e,t){var n=e.rect,i=e.text,o=i.getBBox(),a=t.paddingLeftRight,r=t.paddingTopBottom,s=n[2]-n[0]-2*a;i.setMaxSize(s,n[3]-n[1]-r,e.isNode()?{textOverflow:t.groupLabelOverflow,wordWrap:"none"}:{textOverflow:t.tileLabelOverflow,wordWrap:t.tileLabelWordWrap,hideOverflowEllipsis:!0}),i.move(t.rtlEnabled?n[2]-a-o.x-o.width:n[0]+a-o.x,n[1]+r-o.y)}t.default=void 0;var y=n(331),b=i(n(204)),x=n(175),w=n(235),C=n(14),k=n(4),S=i(n(112));n(565),n(567);var D=i(n(9)),I=n(174),T=Math.max,O={lefttoprightbottom:[1,1],leftbottomrighttop:[1,-1],righttopleftbottom:[-1,1],rightbottomlefttop:[-1,-1]};(0,x.setDefaultAlgorithm)("squarified"),(0,w.setDefaultColorizer)("discrete");var E=S.default.inherit({_handlers:{beginBuildNodes:k.noop,buildNode:k.noop,endBuildNodes:k.noop,setTrackerData:k.noop,calculateState:function(e){return(0,y.buildRectAppearance)(e)}},_rootClass:"dxtm-tree-map",_rootClassPrefix:"dxtm",_getDefaultSize:function(){return{width:400,height:400}},_themeSection:"treeMap",_fontFields:["tile.label.font","group.label.font"],_init:function(){var e=this;e._rectOffsets={},e._handlers=Object.create(e._handlers),e._context={suspend:function(){e._applyingChanges||e._suspendChanges()},resume:function(){e._applyingChanges||e._resumeChanges()},change:function(t){e._change(t)},settings:[{},{}],calculateState:e._handlers.calculateState,calculateLabelState:y.buildTextAppearance},e._root=e._topNode={nodes:[]},e.callBase.apply(e,arguments)},_initialChanges:["DATA_SOURCE"],_initCore:function(){var e=this,t=e._renderer;e._createProxyType(),e._tilesGroup=t.g().linkOn(t.root,"tiles").linkAppend(),e._labelsGroup=t.g().linkOn(t.root,"labels").linkAppend()},_createProxyType:k.noop,_disposeCore:function(){var e=this;e._filter&&e._filter.dispose(),e._labelsGroup.linkOff(),e._tilesGroup.linkOff()},_applySize:function(e){this._tilingRect=e.slice(),this._change(["TILING"])},_optionChangesMap:{dataSource:"DATA_SOURCE",valueField:"NODES_CREATE",childrenField:"NODES_CREATE",colorField:"TILES",colorizer:"TILES",labelField:"LABELS",tile:"TILE_SETTINGS",group:"GROUP_SETTINGS",maxDepth:"MAX_DEPTH",layoutAlgorithm:"TILING",layoutDirection:"TILING"},_themeDependentChanges:["TILE_SETTINGS","GROUP_SETTINGS","MAX_DEPTH"],_changeDataSource:function(){var e=this;e._isDataExpected=e._isSyncData=!0,e._updateDataSource(),e._isSyncData=!1,e._isDataExpected&&e._suspendChanges()},_dataSourceChangedHandler:function(){var e=this;e._isDataExpected?(e._isDataExpected=!1,e._change(["NODES_CREATE"]),e._isSyncData||e._resumeChanges()):e._requestChange(["NODES_CREATE"])},_optionChangesOrder:["DATA_SOURCE","TILE_SETTINGS","GROUP_SETTINGS","MAX_DEPTH"],_change_DATA_SOURCE:function(){this._changeDataSource()},_change_TILE_SETTINGS:function(){this._changeTileSettings()},_change_GROUP_SETTINGS:function(){this._changeGroupSettings()},_change_MAX_DEPTH:function(){this._changeMaxDepth()},_customChangesOrder:["NODES_CREATE","NODES_RESET","TILES","LABELS","TILING","LABELS_LAYOUT"],_change_NODES_CREATE:function(){this._buildNodes()},_change_NODES_RESET:function(){this._resetNodes()},_change_TILES:function(){this._applyTilesAppearance()},_change_LABELS:function(){this._applyLabelsAppearance()},_change_TILING:function(){this._performTiling()},_change_LABELS_LAYOUT:function(){this._performLabelsLayout()},_applyChanges:function(){var e=this;e.callBase.apply(e,arguments),e._isDataExpected||e._drawn(),e._context.forceReset=!1},_buildNodes:function(){var e=this,t=e._root=e._topNode=new b.default;t._id=0,t.parent={},t.data={},t.level=t.index=-1,t.ctx=e._context,t.label=null,e._nodes=[t],e._handlers.beginBuildNodes();var n=e._processDataSourceItems(e._dataSourceItems()||[]);a(t,n.items,0,{itemsField:!n.isPlain&&e._getOption("childrenField",!0)||"items",valueField:e._getOption("valueField",!0)||"value",buildNode:e._handlers.buildNode,ctx:e._context,nodes:e._nodes}),e._onNodesCreated(),e._handlers.endBuildNodes(),e._change(["NODES_RESET"])},_onNodesCreated:k.noop,_processDataSourceItems:function(e){return{items:e,isPlain:!1}},_changeTileSettings:function(){var e=this,t=e._getOption("tile"),n=e._rectOffsets,i=o(t.border.width),a=i/2,r=1&i?.5:0,s=t.label,l=e._context.settings[0];e._change(["TILES","LABELS"]),l.state=e._handlers.calculateState(t),e._filter=e._filter||e._renderer.shadowFilter("-50%","-50%","200%","200%"),e._filter.attr(s.shadow),e._calculateLabelSettings(l,s,e._filter.id),n.tileEdge===a&&n.tileInner===r||(n.tileEdge=a,n.tileInner=r,e._change(["TILING"]))},_changeGroupSettings:function(){var e=this,t=e._getOption("group"),n=t.label,i=e._rectOffsets,a=o(t.border.width),r=a/2,s=1&a?.5:0,l=0,u=o(t.padding),c=e._context.settings[1];e._change(["TILES","LABELS"]),c.state=e._handlers.calculateState(t),e._calculateLabelSettings(c,n),l=t.headerHeight>=0?o(t.headerHeight):c.labelParams.height+2*o(n.paddingTopBottom),e._headerHeight!==l&&(e._headerHeight=l,e._change(["TILING"])),e._groupPadding!==u&&(e._groupPadding=u,e._change(["TILING"])),i.headerEdge===r&&i.headerInner===s||(i.headerEdge=r,i.headerInner=s,e._change(["TILING"]))},_calculateLabelSettings:function(e,t,n){var i=this._getTextBBox(t.font),a=o(t.paddingLeftRight),r=o(t.paddingTopBottom),s=this._getOption("tile.label"),l=this._getOption("group.label");e.labelState=(0,y.buildTextAppearance)(t,n),e.labelState.visible=!("visible"in t&&!t.visible),e.labelParams={height:i.height,rtlEnabled:this._getOption("rtlEnabled",!0),paddingTopBottom:r,paddingLeftRight:a,tileLabelWordWrap:s.wordWrap,tileLabelOverflow:s.textOverflow,groupLabelOverflow:l.textOverflow}},_changeMaxDepth:function(){var e=this._getOption("maxDepth",!0);e=e>=1?Math.round(e):1/0,this._maxDepth!==e&&(this._maxDepth=e,this._change(["NODES_RESET"]))},_resetNodes:function(){var e=this;e._tilesGroup.clear(),e._renderer.initHatching(),e._context.forceReset=!0,e._context.minLevel=e._topNode.level+1,e._context.maxLevel=e._context.minLevel+e._maxDepth-1,e._change(["TILES","LABELS","TILING"])},_processNodes:function(e,t){r(e,this._topNode,t)},_applyTilesAppearance:function(){var e=this,t=(0,w.getColorizer)(e._getOption("colorizer"),e._themeManager,e._topNode);e._processNodes({renderer:e._renderer,group:e._tilesGroup,setTrackerData:e._handlers.setTrackerData,colorField:e._getOption("colorField",!0)||"color",getColor:t},s)},_applyLabelsAppearance:function(){var e=this;e._labelsGroup.clear(),e._processNodes({renderer:e._renderer,group:e._labelsGroup,setTrackerData:e._handlers.setTrackerData,labelField:e._getOption("labelField",!0)||"name"},c),e._change(["LABELS_LAYOUT"])},_performTiling:function(){var e=this,t={algorithm:(0,x.getAlgorithm)(e._getOption("layoutAlgorithm",!0)),directions:O[String(e._getOption("layoutDirection",!0)).toLowerCase()]||O.lefttoprightbottom,headerHeight:e._headerHeight,groupPadding:e._groupPadding,rectOffsets:e._rectOffsets};e._topNode.innerRect=e._tilingRect,h(t,e._topNode),e._processNodes(t,f),e._change(["LABELS_LAYOUT"]),e._onTilingPerformed()},_onTilingPerformed:k.noop,_performLabelsLayout:function(){this._processNodes(null,m)},_getTextBBox:function(e){var t=this._renderer,n=this._textForCalculations||t.text("0",0,0);this._textForCalculations=n,n.css((0,C.patchFontOptions)(e)).append(t.root);var i=n.getBBox();return n.remove(),i}}),A=[l,u],P=[0,0,0,0];(0,D.default)("dxTreeMap",E);var R=E;t.default=R,E.addPlugin(I.plugin),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(7),a=i(n(11)),r=i(n(27)),s=i(n(49)),l=i(n(64)),u=function(){var e,t=(0,r.default)(),n=t.add,i=t.remove;if(!(0,o.hasWindow)())return t;var u,c=function(){var e=(0,o.getWindow)();return{width:e.innerWidth,height:e.innerHeight}},d=function(){var n=c();if(n.width!==e.width||n.height!==e.height){var i;n.width===e.width&&(i="height"),n.height===e.height&&(i="width"),e=n,t.fire(i)}},h=(0,l.default)(function(){e=c()});return t.add=function(){var e=n.apply(t,arguments);return h(),s.default.add(function(){!u&&t.has()&&(u=a.default.listen((0,o.getWindow)(),"resize",d))}),e},t.remove=function(){var e=i.apply(t,arguments);return!t.has()&&u&&(u(),u=void 0),e},t}(),c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(594)),a=i(n(9));(0,a.default)("dxList",o.default);var r=o.default;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(183)),a=new o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(28)),a=n(0),r=n(333),s=n(185),l=i(n(15)),u=i(n(108)),c=i(n(22)),d=i(n(27)),h=n(92),f=n(136),p=n(359),g=n(1),_=n(4),m=function(e){return e.charAt(2).toLowerCase()+e.substr(3)},v=function(e){return 0===e.indexOf("_",0)},y=l.default.inherit({_setDeprecatedOptions:function(){this._deprecatedOptions={}},_getDeprecatedOptions:function(){return this._deprecatedOptions},_getDefaultOptions:function(){return{onInitialized:null,onOptionChanged:null,onDisposing:null,defaultOptionsRules:null}},_defaultOptionsRules:function(){return[]},_setOptionsByDevice:function(e){this._options.applyRules(e)},_convertRulesToOptions:function(e){return(0,s.convertRulesToOptions)(e)},_isInitialOptionValue:function(e){return this._options.isInitial(e)},_setOptionsByReference:function(){this._optionsByReference={}},_getOptionsByReference:function(){return this._optionsByReference},ctor:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e._optionChangedCallbacks,n=e._disposingCallbacks;this.NAME=(0,f.name)(this.constructor),this._eventsStrategy=h.EventsStrategy.create(this,e.eventsStrategy),this._updateLockCount=0,this._optionChangedCallbacks=t||(0,d.default)(),this._disposingCallbacks=n||(0,d.default)(),this.postponedOperations=new p.PostponedOperations,this._createOptions(e)},_createOptions:function(e){var t=this;this.beginUpdate();try{this._setOptionsByReference(),this._setDeprecatedOptions(),this._options=new r.Options(this._getDefaultOptions(),this._getDefaultOptions(),this._getOptionsByReference(),this._getDeprecatedOptions()),this._options.onChanging(function(e,n,i){return t._initialized&&t._optionChanging(e,n,i)}),this._options.onDeprecated(function(e,n){return t._logDeprecatedOptionWarning(e,n)}),this._options.onChanged(function(e,n,i){return t._notifyOptionChanged(e,n,i)}),this._options.onStartChange(function(){return t.beginUpdate()}),this._options.onEndChange(function(){return t.endUpdate()}),this._options.addRules(this._defaultOptionsRules()),e&&e.onInitializing&&e.onInitializing.apply(this,[e]),this._setOptionsByDevice(e.defaultOptionsRules),this._initOptions(e)}finally{this.endUpdate()}},_initOptions:function(e){this.option(e)},_init:function(){var e=this;this._createOptionChangedAction(),this.on("disposing",function(t){e._disposingCallbacks.fireWith(e,[t])})},_logDeprecatedOptionWarning:function(e,t){var n=t.message||"Use the '".concat(t.alias,"' option instead");c.default.log("W0001",this.NAME,e,t.since,n)},_logDeprecatedComponentWarning:function(e,t){c.default.log("W0000",this.NAME,e,"Use the '".concat(t,"' widget instead"))},_createOptionChangedAction:function(){this._optionChangedAction=this._createActionByOption("onOptionChanged",{excludeValidators:["disabled","readOnly"]})},_createDisposingAction:function(){this._disposingAction=this._createActionByOption("onDisposing",{excludeValidators:["disabled","readOnly"]})},_optionChanged:function(e){switch(e.name){case"onDisposing":case"onInitialized":break;case"onOptionChanged":this._createOptionChangedAction();break;case"defaultOptionsRules":}},_dispose:function(){this._optionChangedCallbacks.empty(),this._createDisposingAction(),this._disposingAction(),this._eventsStrategy.dispose(),this._options.dispose(),this._disposed=!0},_lockUpdate:function(){this._updateLockCount++},_unlockUpdate:function(){this._updateLockCount=Math.max(this._updateLockCount-1,0)},_isUpdateAllowed:function(){return 0===this._updateLockCount},_isInitializingRequired:function(){return!this._initializing&&!this._initialized},_commitUpdate:function(){this.postponedOperations.callPostponedOperations(),this._isInitializingRequired()&&this._initializeComponent()},_initializeComponent:function(){this._initializing=!0;try{this._init()}finally{this._initializing=!1,this._lockUpdate(),this._createActionByOption("onInitialized",{excludeValidators:["disabled","readOnly"]})(),this._unlockUpdate(),this._initialized=!0}},instance:function(){return this},beginUpdate:function(){this._lockUpdate()},endUpdate:function(){this._unlockUpdate(),this._isUpdateAllowed()&&this._commitUpdate()},_optionChanging:_.noop,_notifyOptionChanged:function(e,t,n){if(this._initialized)for(var i=[e].concat(this._options.getAliasesByName(e)),o=0;o<i.length;o++){var r=i[o],s={name:r.split(/[.[]/)[0],fullName:r,value:t,previousValue:n};v(r)||(this._optionChangedCallbacks.fireWith(this,[(0,a.extend)(this._defaultActionArgs(),s)]),this._optionChangedAction((0,a.extend)({},s))),this._disposed||this._cancelOptionChange===r||this._optionChanged(s)}},initialOption:function(e){return this._options.initial(e)},_defaultActionConfig:function(){return{context:this,component:this}},_defaultActionArgs:function(){return{component:this}},_createAction:function(e,t){var n,i=this;return function(o){return(0,g.isDefined)(o)||(o={}),(0,g.isPlainObject)(o)||(o={actionValue:o}),n=n||new u.default(e,(0,a.extend)(t,i._defaultActionConfig())),n.execute.call(n,(0,a.extend)(o,i._defaultActionArgs()))}},_createActionByOption:function(e,t){var n,i,a,r=this,s=function(){if(!i){if(t=t||{},"string"!=typeof e)throw c.default.Error("E0008");0===e.indexOf("on")&&(i=m(e)),a=r.option(e)}if(n||a||t.beforeExecute||t.afterExecute||r._eventsStrategy.hasEvent(i)){if(!n){var s=t.beforeExecute;t.beforeExecute=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];s&&s.apply(r,t),r._eventsStrategy.fireEvent(i,t[0].args)},n=r._createAction(a,t)}for(var l=arguments.length,u=new Array(l),d=0;d<l;d++)u[d]=arguments[d];if((0,o.default)().wrapActionsBeforeExecute){var h=r.option("beforeActionExecute")||_.noop,f=h(r,n,t)||n;return f.apply(r,u)}return n.apply(r,u)}};if((0,o.default)().wrapActionsBeforeExecute)return s;var l=this.option("onActionCreated")||_.noop;return l(this,s,t)||s},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this},hasActionSubscription:function(e){return!!this._options.silent(e)||this._eventsStrategy.hasEvent(m(e))},isOptionDeprecated:function(e){return this._options.isDeprecated(e)},_setOptionWithoutOptionChange:function(e,t){this._cancelOptionChange=e,this.option(e,t),this._cancelOptionChange=!1},_getOptionValue:function(e,t){var n=this.option(e);return(0,g.isFunction)(n)?n.bind(t)():n},option:function(){var e;return(e=this._options).option.apply(e,arguments)},resetOption:function(e){this.beginUpdate(),this._options.reset(e),this.endUpdate()}}),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}function s(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&l(e,t)}function l(e,t){return(l=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function u(e){var t=h();return function(){var n,i=f(e);if(t){var o=f(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return c(this,n)}}function c(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?d(e):t}function d(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function h(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function f(e){return(f=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.FunctionTemplate=void 0;var p=n(67),g=n(33),_=function(e){function t(e){var i;return o(this,t),i=n.call(this),i._render=e,i}s(t,e);var n=u(t);return r(t,[{key:"_renderCore",value:function(e){return(0,g.normalizeTemplateElement)(this._render(e))}}]),t}(p.TemplateBase);t.FunctionTemplate=_},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return t?(0,u.addNamespace)(e,t):e}function a(e,t){return"function"==typeof e?e(t):e.execute(t)}t.keyboard=t.click=t.dxClick=t.focus=t.visibility=t.hover=t.resize=t.active=void 0;var r=i(n(11)),s=i(n(5)),l=i(n(363)),u=n(8),c={on:function(e,t,n,i){var r=i.selector,l=i.showTimeout,u=i.hideTimeout,c=i.namespace;s.default.on(e,o("dxactive",c),r,{timeout:l},function(e){return a(t,{event:e,element:e.currentTarget})}),s.default.on(e,o("dxinactive",c),r,{timeout:u},function(e){return a(n,{event:e,element:e.currentTarget})})},off:function(e,t){var n=t.namespace,i=t.selector;s.default.off(e,o("dxactive",n),i),s.default.off(e,o("dxinactive",n),i)}};t.active=c;var d={on:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=n.namespace;s.default.on(e,o("dxresize",i),t)},off:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.namespace;s.default.off(e,o("dxresize",n))}};t.resize=d;var h={on:function(e,t,n,i){var r=i.selector,l=i.namespace;s.default.on(e,o("dxhoverend",l),r,function(e){return n(e)}),s.default.on(e,o("dxhoverstart",l),r,function(e){return a(t,{element:e.target,event:e})})},off:function(e,t){var n=t.selector,i=t.namespace;s.default.off(e,o("dxhoverstart",i),n),s.default.off(e,o("dxhoverend",i),n)}};t.hover=h;var f={on:function(e,t,n,i){var a=i.namespace;s.default.on(e,o("dxhiding",a),n),s.default.on(e,o("dxshown",a),t)},off:function(e,t){var n=t.namespace;s.default.off(e,o("dxhiding",n)),s.default.off(e,o("dxshown",n))}};t.visibility=f;var p={on:function(e,t,n,i){var a=i.namespace,l=i.isFocusable;s.default.on(e,o("focusin",a),t),s.default.on(e,o("focusout",a),n),r.default.hasDocumentProperty("onbeforeactivate")&&s.default.on(e,o("beforeactivate",a),function(e){return l(null,e.target)||e.preventDefault()})},off:function(e,t){var n=t.namespace;s.default.off(e,o("focusin",n)),s.default.off(e,o("focusout",n)),r.default.hasDocumentProperty("onbeforeactivate")&&s.default.off(e,o("beforeactivate",n))},trigger:function(e){return s.default.trigger(e,"focus")}};t.focus=p;var g={on:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=n.namespace;s.default.on(e,o("dxclick",i),t)},off:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.namespace;s.default.off(e,o("dxclick",n))}};t.dxClick=g;var _={on:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=n.namespace;s.default.on(e,o("click",i),t)},off:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.namespace;s.default.off(e,o("click",n))}};t.click=_;var m=0,v={},y=function(){return"keyboardProcessorId".concat(m++)},b={on:function(e,t,n){var i=y();return v[i]=new l.default({element:e,focusTarget:t,handler:n}),i},off:function(e){e&&v[e]&&(v[e].dispose(),delete v[e])},_getProcessor:function(e){return v[e]}};t.keyboard=b},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return h(),c.apply(l,arguments)}function a(){h(),d.apply(l,arguments)}t.requestAnimationFrame=o,t.cancelAnimationFrame=a;var r=n(7),s=i(n(64)),l=(0,r.hasWindow)()?(0,r.getWindow)():{},u=1e3/60,c=function(e){return setTimeout(e,u)},d=function(e){clearTimeout(e)},h=(0,s.default)(function(){var e=l.requestAnimationFrame||l.webkitRequestAnimationFrame||l.mozRequestAnimationFrame||l.oRequestAnimationFrame||l.msRequestAnimationFrame,t=l.cancelAnimationFrame||l.webkitCancelAnimationFrame||l.mozCancelAnimationFrame||l.oCancelAnimationFrame||l.msCancelAnimationFrame;if(e&&t&&(c=e,d=t),e&&!t){var n={};c=function(t){var i=e.call(l,function(){try{if(i in n)return;t.apply(this,arguments)}finally{delete n[i]}});return i},d=function(e){n[e]=!0}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(15)),s=i(n(27)),l=n(0),u=n(8),c=r.default.inherit({ctor:function(e){this._$element=(0,o.default)(e),this._cancelCallback=(0,s.default)(),this._acceptCallback=(0,s.default)()},getElement:function(){return this._$element},validate:function(e){return!(0,u.isDxMouseWheelEvent)(e)},validatePointers:function(e){return 1===(0,u.hasTouches)(e)},allowInterruptionByMouseWheel:function(){return!0},configure:function(e){(0,l.extend)(this,e)},addCancelCallback:function(e){this._cancelCallback.add(e)},removeCancelCallback:function(){this._cancelCallback.empty()},_cancel:function(e){this._cancelCallback.fire(this,e)},addAcceptCallback:function(e){this._acceptCallback.add(e)},removeAcceptCallback:function(){this._acceptCallback.empty()},_accept:function(e){this._acceptCallback.fire(this,e)},_requestAccept:function(e){this._acceptRequestEvent=e},_forgetAccept:function(){this._accept(this._acceptRequestEvent),this._acceptRequestEvent=null},start:a.noop,move:a.noop,end:a.noop,cancel:a.noop,reset:function(){this._acceptRequestEvent&&this._accept(this._acceptRequestEvent)},_fireEvent:function(e,t,n){var i=(0,l.extend)({type:e,originalEvent:t,target:this._getEmitterTarget(t),delegateTarget:this.getElement().get(0)},n);return t=(0,u.fireEvent)(i),t.cancel&&this._cancel(t),t},_getEmitterTarget:function(e){return(this.delegateSelector?(0,o.default)(e.target).closest(this.delegateSelector):this.getElement()).get(0)},dispose:a.noop}),d=c;t.default=d,e.exports=t.default},function(e,t){e.exports=window.angular},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e&&(0,v.isFunction)(e.then)}function a(e){return(new k.Deferred).resolve(e).promise()}function r(e,t){if(!(0,v.isFunction)(t))throw b.default.errors.Error("E4011",e)}function s(e){throw b.default.errors.Error("E4012",e)}function l(e){function t(e){var t=e[0],n=e[1];return t&&t.getResponseHeader?_.default.errorMessageFromXhr(t,n):null}return function(n){var i;i=n instanceof Error?n:new Error(t(arguments)||n&&String(n)||"Unknown error"),i.message!==_.default.XHR_ERROR_UNLOAD&&e.reject(i)}}function u(e,t){var n,i=e._loadFunc;return r(D,i),n=i.apply(e,[t]),Array.isArray(n)?n=a(n):null===n||void 0===n?n=a([]):o(n)||s(D),(0,k.fromPromise)(n)}function c(e,t){var n,i=e._totalCountFunc;if(!(0,v.isFunction)(i))throw b.default.errors.Error("E4021");return n=i.apply(e,[t]),o(n)||(n=Number(n),isFinite(n)||s(S),n=a(n)),(0,k.fromPromise)(n)}function d(e,t,n){var i,s=e._byKeyFunc;return r(I,s),i=s.apply(e,[t,n]),o(i)||(i=a(i)),(0,k.fromPromise)(i)}function h(e,t,n,i){if(t.__rawData)i(t.__rawData);else{var o=t.__rawDataPromise||u(t,n);t._cacheRawData&&(t.__rawDataPromise=o),o.always(function(){delete t.__rawDataPromise}).done(function(e){t._cacheRawData&&(t.__rawData=e),i(e)}).fail(l(e))}}function f(e,t,n,i){n=n||{};var o={};"userData"in n&&(o.userData=n.userData),h(e,t,o,function(o){var a,r,s,l,u=(0,w.default)(o,{errorHandler:t._errorHandler}),c=[];i||(a=C.default.queryByOptions(u,n),a===u?s=o.slice(0):c.push(a.enumerate().done(function(e){s=e}))),(n.requireTotalCount||i)&&(r=C.default.queryByOptions(u,n,!0),r===u?l=o.length:c.push(r.count().done(function(e){l=e;
}))),k.when.apply(g.default,c).done(function(){i?e.resolve(l):n.requireTotalCount?e.resolve(s,{totalCount:l}):e.resolve(s)}).fail(function(t){e.reject(t)})})}function p(e,t,n){h(e,t,{},function(i){for(var o,a=t.key(),r=0,s=i.length;r<s;r++)if(o=i[r],_.default.keysEqual(a,t.keyOf(i[r]),n))return void e.resolve(o);e.reject(b.default.errors.Error("E4009"))})}t.default=void 0;var g=i(n(2)),_=i(n(41)),m=n(90),v=n(1),y=i(n(28)),b=i(n(34)),x=i(n(102)),w=i(n(156)),C=i(n(91)),k=n(6),S="totalCount",D="load",I="byKey",T="insert",O="update",E="remove",A=x.default.inherit({ctor:function(e){e=e||{},this.callBase(e),this._useDefaultSearch=!!e.useDefaultSearch||"raw"===e.loadMode,this._loadMode=e.loadMode,this._cacheRawData=e.cacheRawData!==!1,this._loadFunc=e[D],this._totalCountFunc=e[S],this._byKeyFunc=e[I],this._insertFunc=e[T],this._updateFunc=e[O],this._removeFunc=e[E]},createQuery:function(){throw b.default.errors.Error("E4010")},clearRawDataCache:function(){delete this.__rawData},_totalCountImpl:function(e){var t=new k.Deferred;return"raw"!==this._loadMode||this._totalCountFunc?(c(this,e).done(function(e){t.resolve(Number(e))}).fail(l(t)),t=this._addFailHandlers(t)):f(t,this,e,!0),t.promise()},_pushImpl:function(e){this.__rawData&&(0,m.applyBatch)({keyInfo:this,data:this.__rawData,changes:e})},_loadImpl:function(e){var t=new k.Deferred;return"raw"===this._loadMode?f(t,this,e,!1):(u(this,e).done(function(e,n){t.resolve(e,n)}).fail(l(t)),t=this._addFailHandlers(t)),t.promise()},_byKeyImpl:function(e,t){var n=new k.Deferred;return this._byKeyViaLoad()?(this._requireKey(),p(n,this,e)):d(this,e,t).done(function(e){n.resolve(e)}).fail(l(n)),n.promise()},_byKeyViaLoad:function(){return"raw"===this._loadMode&&!this._byKeyFunc},_insertImpl:function(e){var t,n=this,i=n._insertFunc,s=new k.Deferred;return r(T,i),t=i.apply(n,[e]),o(t)||(t=a(t)),(0,k.fromPromise)(t).done(function(t){(0,y.default)().useLegacyStoreResult?s.resolve(e,t):s.resolve(t||e,n.keyOf(t))}).fail(l(s)),s.promise()},_updateImpl:function(e,t){var n,i=this._updateFunc,s=new k.Deferred;return r(O,i),n=i.apply(this,[e,t]),o(n)||(n=a(n)),(0,k.fromPromise)(n).done(function(n){(0,y.default)().useLegacyStoreResult?s.resolve(e,t):s.resolve(n||t,e)}).fail(l(s)),s.promise()},_removeImpl:function(e){var t,n=this._removeFunc,i=new k.Deferred;return r(E,n),t=n.apply(this,[e]),o(t)||(t=a()),(0,k.fromPromise)(t).done(function(){i.resolve(e)}).fail(l(i)),i.promise()}}),P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(72)),a=i(n(82));if(o.default&&o.default.load){var r={supplemental:{version:{_cldrVersion:"28",_unicodeVersion:"8.0.0",_number:"$Revision: 11965 $"},likelySubtags:{en:"en-Latn-US",de:"de-Latn-DE",ru:"ru-Cyrl-RU",ja:"ja-Jpan-JP"}}};o.default.locale()||(o.default.load(r),o.default.locale("en")),a.default.inject({locale:function(e){return e?void o.default.locale(e):o.default.locale().locale}})}},function(e,t,n){function i(e,t,n){var i=a,r={};r[e]=i[e]?i[e]:{},r[e][t]=n,i=(0,o.extend)(i,r)}t.register=i,t.registry=void 0;var o=n(0),a={};t.registry=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(615)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=n(1),s=n(0),l=n(12),u=n(3),c=n(6),d=n(18),h=i(n(22)),f=i(n(11)),p=n(87),g=i(n(10)),_=i(n(9)),m=i(n(282)),v=n(8),y=".dx-state-disabled",b="dx-selectbox",x="dx-selectbox-popup",w="dx-selectbox-container",C="dx-selectbox-popup-wrapper",k=m.default.inherit({_supportedKeys:function(){var e=this,t=this.callBase(),n=function(e){var t=this._isEditable();t?this._valueSubstituted()&&(this._preventFiltering=!0):this.option("showClearButton")&&(e.preventDefault(),this.reset()),this._savedTextRemoveEvent=e,this._preventSubstitution=!0},i=function(){e.option("searchEnabled")&&e._valueSubstituted()&&e._searchHandler()};return(0,s.extend)({},t,{tab:function(){this.option("opened")&&"instantly"===this.option("applyValueMode")&&this._cleanInputSelection(),this._wasSearch()&&this._clearFilter(),this._wasTabPressed=!0,t.tab&&t.tab.apply(this,arguments)},upArrow:function(e){if(t.upArrow&&t.upArrow.apply(this,arguments))return this.option("opened")||this._setNextValue(e),!0},downArrow:function(e){if(t.downArrow&&t.downArrow.apply(this,arguments))return this.option("opened")||this._setNextValue(e),!0},leftArrow:function(){i(),t.leftArrow&&t.leftArrow.apply(this,arguments)},rightArrow:function(){i(),t.rightArrow&&t.rightArrow.apply(this,arguments)},home:function(){i(),t.home&&t.home.apply(this,arguments)},end:function(){i(),t.end&&t.end.apply(this,arguments)},escape:function(){var e=t.escape&&t.escape.apply(this,arguments);return this._cancelEditing(),!(0,r.isDefined)(e)||e},enter:function(e){var n=this.option("opened"),i=this._input().val().trim(),o=i&&this._list&&!this._list.option("focusedElement");if(!i&&(0,r.isDefined)(this.option("value"))&&this.option("allowClearing"))this.option({selectedItem:null,value:null}),this.close();else{if(this.option("acceptCustomValue"))return e.preventDefault(),o&&(this._valueChangeEventHandler(e),n&&this._toggleOpenState()),n;if(t.enter&&t.enter.apply(this,arguments))return n}},space:function(e){var t=this.option("opened"),n=this.option("searchEnabled"),i=this.option("acceptCustomValue");if(t&&!n&&!i)return e.preventDefault(),this._valueChangeEventHandler(e),!0},backspace:n,del:n})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{placeholder:g.default.format("Select"),fieldTemplate:null,valueChangeEvent:"change",acceptCustomValue:!1,onCustomItemCreating:function(e){(0,r.isDefined)(e.customItem)||(e.customItem=e.text)},showSelectionControls:!1,allowClearing:!0,tooltipEnabled:!1,openOnFieldClick:!0,showDropDownButton:!0,displayCustomValue:!1,useInkRipple:!1,useHiddenSubmitElement:!0})},_init:function(){this.callBase(),this._initCustomItemCreatingAction()},_initMarkup:function(){this.$element().addClass(b),this._renderTooltip(),this.option("useInkRipple")&&this._renderInkRipple(),this.callBase(),this._$container.addClass(w)},_renderInkRipple:function(){this._inkRipple=(0,p.render)()},_toggleActiveState:function(e,t,n){if(this.callBase.apply(this,arguments),this._inkRipple&&!this._isEditable()){var i={element:this._inputWrapper(),event:n};t?this._inkRipple.showWave(i):this._inkRipple.hideWave(i)}},_createPopup:function(){this.callBase(),this._popup.$element().addClass(x),this._popup.overlayContent().attr("tabindex",-1)},_popupWrapperClass:function(){return this.callBase()+" "+C},_cancelEditing:function(){!this.option("searchEnabled")&&this._list&&(this._focusListElement(null),this._updateField(this.option("selectedItem")))},_renderOpenedState:function(){this.callBase(),this.option("opened")&&(this._scrollToSelectedItem(),this._focusSelectedElement())},_focusSelectedElement:function(){var e=this._searchValue();if(!e)return void this._focusListElement(null);var t=this._list._itemElements(),n=(0,l.inArray)(this.option("selectedItem"),this.option("items")),i=n>=0&&!this._isCustomItemSelected()?t.eq(n):null;this._focusListElement(i)},_renderFocusedElement:function(){if(this._list){var e=this._searchValue();if(!e||this.option("acceptCustomValue"))return void this._focusListElement(null);var t=this._list._itemElements(),n=t.not(y).eq(0);this._focusListElement(n)}},_focusListElement:function(e){this._preventInputValueRender=!0,this._list.option("focusedElement",(0,d.getPublicElement)(e)),delete this._preventInputValueRender},_scrollToSelectedItem:function(){this._list&&this._list.scrollToItem(this._list.option("selectedItem"))},_listContentReadyHandler:function(){this.callBase();var e=this._dataSource&&this._dataSource.paginate();e&&this._needPopupRepaint()||this._scrollToSelectedItem()},_renderValue:function(){return this._renderInputValue(),this._setSubmitValue(),(new c.Deferred).resolve()},_renderInputValue:function(){return this.callBase().always(function(){this._renderInputValueAsync()}.bind(this))},_renderInputValueAsync:function(){this._renderTooltip(),this._renderInputValueImpl().always(function(){this._refreshSelected()}.bind(this))},_renderInputValueImpl:function(){return this._renderField(),(new c.Deferred).resolve()},_setNextItem:function(e){var t=this._calcNextItem(e),n=this._valueGetter(t);this._setValue(n)},_setNextValue:function(e){var t=this._dataSource.isLoaded()?(new c.Deferred).resolve():this._dataSource.load();t.done(function(){var t=this._getSelectedIndex(),n=this._dataSource.pageSize(),i=this._dataSource.isLastPage(),o=t===this._items().length-1;this._saveValueChangeEvent(e);var a="downArrow"===(0,v.normalizeKeyName)(e)?1:-1;n&&!i&&o&&a>0?(this._popup||this._createPopup(),this._dataSource.isLoading()||this._list._loadNextPage().done(this._setNextItem.bind(this,a))):this._setNextItem(a)}.bind(this))},_setSelectedItem:function(e){var t=!this._isCustomValueAllowed()&&void 0===e;this.callBase(t?null:e),t||this._isEditable()&&!this._isCustomItemSelected()||this._setListOption("selectedItem",this.option("selectedItem"))},_isCustomValueAllowed:function(){return this.option("acceptCustomValue")||this.callBase()},_displayValue:function(e){return e=!(0,r.isDefined)(e)&&this._isCustomValueAllowed()?this.option("value"):e,this.callBase(e)},_listConfig:function(){var e=(0,s.extend)(this.callBase(),{pageLoadMode:"scrollBottom",onSelectionChanged:this._getSelectionChangeHandler(),selectedItem:this.option("selectedItem"),onFocusedItemChanged:this._listFocusedItemChangeHandler.bind(this)});return this.option("showSelectionControls")&&(0,s.extend)(e,{showSelectionControls:!0,selectionByClick:!0}),e},_listFocusedItemChangeHandler:function(e){if(!this._preventInputValueRender){var t=e.component,n=(0,o.default)(t.option("focusedElement")),i=t._getItemData(n);this._updateField(i)}},_updateField:function(e){var t=this._getTemplateByOption("fieldTemplate");if(!t||!this.option("fieldTemplate")){var n=this._displayGetter(e);return this.option("text",n),void this._renderDisplayText(n)}this._renderField()},_getSelectionChangeHandler:function(){return this.option("showSelectionControls")?this._selectionChangeHandler.bind(this):a.noop},_selectionChangeHandler:function(e){(0,u.each)(e.addedItems||[],function(e,t){this._setValue(this._valueGetter(t))}.bind(this))},_getActualSearchValue:function(){return this._dataSource.searchValue()},_toggleOpenState:function(e){if(!this.option("disabled")){if(e=arguments.length?e:!this.option("opened"),e||this._restoreInputText(!0),this._wasSearch()&&e){this._wasSearch(!1);var t=this.option("showDataBeforeSearch")||0===this.option("minSearchLength");if(t&&this._dataSource){if(this._searchTimer)return;var n=this._getActualSearchValue();n&&this._wasSearch(!0),this._filterDataSource(n||null)}else this._setListOption("items",[])}e&&this._scrollToSelectedItem(),this.callBase(e)}},_renderTooltip:function(){this.option("tooltipEnabled")&&this.$element().attr("title",this.option("displayValue"))},_renderDimensions:function(){this.callBase(),this._dimensionChanged()},_isValueEqualInputText:function(){var e=this.option("selectedItem"),t=this._displayGetter(e),n=t?String(t):"",i=this._searchValue();return n===i},_popupHidingHandler:function(){this._isValueEqualInputText()&&this._cancelEditing(),this.callBase()},_restoreInputText:function(e){this.option("readOnly")||this._loadItemDeferred&&this._loadItemDeferred.always(function(){var t=this.option("selectedItem");return this.option("acceptCustomValue")?void(e||(this._updateField(t),this._clearFilter())):this.option("searchEnabled")&&!this._searchValue()&&this.option("allowClearing")?void this._clearTextValue():void(this._isValueEqualInputText()||this._renderInputValue().always(function(e){var n=(0,a.ensureDefined)(e,t);this._setSelectedItem(n),this._updateField(n),this._clearFilter()}.bind(this)))}.bind(this))},_focusOutHandler:function(e){if(!this._preventNestedFocusEvent(e)){var t=this._isOverlayNestedTarget(e.relatedTarget);t||(this._restoreInputText(),this._clearSearchTimer());var n=this._wasSearch()&&!this.option("acceptCustomValue")&&this.option("searchEnabled")&&(this.option("opened")||this._wasTabPressed)&&!t;this._wasTabPressed=void 0,n&&this._searchCanceled()}e.target=this._input().get(0),this.callBase(e)},_isOverlayNestedTarget:function(e){return!!(0,o.default)(e).closest(".".concat(C)).length},_clearTextValue:function(){this.option("selectedItem")&&(this._savedTextRemoveEvent&&this._saveValueChangeEvent(this._savedTextRemoveEvent),this.option("value",null)),delete this._savedTextRemoveEvent},_shouldOpenPopup:function(){return this._needPassDataSourceToList()},_isFocused:function(){var e=f.default.getActiveElement();return this.callBase()&&(0,o.default)(e).closest(this._input()).length>0},_renderValueChangeEvent:function(){this._isEditable()&&this.callBase()},_isEditable:function(){return this.option("acceptCustomValue")||this.option("searchEnabled")},_fieldRenderData:function(){var e=this._list&&this.option("opened")&&(0,o.default)(this._list.option("focusedElement"));return e&&e.length?this._list._getItemData(e):this.option("selectedItem")},_readOnlyPropValue:function(){return!this._isEditable()||this.option("readOnly")},_isSelectedValue:function(e){return this._isValueEquals(e,this.option("value"))},_shouldCloseOnItemClick:function(){return!(this.option("showSelectionControls")&&"single"!==this.option("selectionMode"))},_listItemClickHandler:function(e){var t=this._getCurrentValue();this._focusListElement((0,o.default)(e.itemElement)),this._saveValueChangeEvent(e.event),this._shouldClearFilter()&&this._clearFilter(),this._completeSelection(this._valueGetter(e.itemData)),this._shouldCloseOnItemClick()&&this.option("opened",!1),this.option("searchEnabled")&&t===this._valueGetter(e.itemData)&&this._updateField(e.itemData)},_shouldClearFilter:function(){return this._wasSearch()},_completeSelection:function(e){this._setValue(e)},_loadItem:function(e,t){var n=this,i=new c.Deferred;return this.callBase(e,t).done(function(e){i.resolve(e)}.bind(this)).fail(function(){var t=n.option("selectedItem");n.option("acceptCustomValue")&&e===n._valueGetter(t)?i.resolve(t):i.reject()}.bind(this)),i.promise()},_loadInputValue:function(e,t){return this._loadItemDeferred=this._loadItem(e).always(t),this._loadItemDeferred},_isCustomItemSelected:function(){var e=this.option("selectedItem"),t=this._searchValue(),n=this._displayGetter(e);return!n||t!==n.toString()},_valueChangeEventHandler:function(e){this.option("acceptCustomValue")&&this._isCustomItemSelected()&&!this._isValueChanging&&(this._isValueChanging=!0,this._customItemAddedHandler(e),this._isValueChanging=!1)},_initCustomItemCreatingAction:function(){this._customItemCreatingAction=this._createActionByOption("onCustomItemCreating")},_createCustomItem:function(e){var t={text:e},n=this._customItemCreatingAction(t),i=(0,a.ensureDefined)(n,t.customItem);return(0,r.isDefined)(n)&&h.default.log("W0015","onCustomItemCreating","customItem"),i},_customItemAddedHandler:function(e){var t=this._searchValue(),n=this._createCustomItem(t);if(this._saveValueChangeEvent(e),void 0===n)throw this._renderValue(),h.default.Error("E0121");(0,r.isPromise)(n)?(0,c.fromPromise)(n).done(this._setCustomItem.bind(this)).fail(this._setCustomItem.bind(this,null)):this._setCustomItem(n)},_setCustomItem:function(e){this._disposed||(e=e||null,this.option("selectedItem",e),this._shouldClearFilter()&&this._filterDataSource(null),this._setValue(this._valueGetter(e)),this._renderDisplayText(this._displayGetter(e)))},_clearValueHandler:function(e){return this.callBase(e),!1},_wasSearch:function(e){return arguments.length?void(this._wasSearchValue=e):this._wasSearchValue},_searchHandler:function(e){return this._preventFiltering?void delete this._preventFiltering:(this._needPassDataSourceToList()&&this._wasSearch(!0),void this.callBase(e))},_dataSourceFiltered:function(e){this.callBase(),null!==e&&(this._renderInputSubstitution(),this._renderFocusedElement())},_valueSubstituted:function(){var e=this._input().get(0),t=0===e.selectionStart&&e.selectionEnd===this._searchValue().length,n=e.selectionStart!==e.selectionEnd;return this._wasSearch()&&n&&!t},_shouldSubstitutionBeRendered:function(){return!this._preventSubstitution&&this.option("searchEnabled")&&!this.option("acceptCustomValue")&&"startswith"===this.option("searchMode")},_renderInputSubstitution:function(){if(!this._shouldSubstitutionBeRendered())return void delete this._preventSubstitution;var e=this._list&&this._getPlainItems(this._list.option("items"))[0];if(e){var t=this._input(),n=t.val().length;if(0!==n){var i=t.get(0),o=this._displayGetter(e).toString();i.value=o,this._caret({start:n,end:o.length})}}},_cleanInputSelection:function(){var e=this._input().get(0),t=e.value.length;e.selectionStart=t,e.selectionEnd=t},_dispose:function(){this._renderInputValueAsync=a.noop,delete this._loadItemDeferred,this.callBase()},_optionChanged:function(e){switch(e.name){case"onCustomItemCreating":this._initCustomItemCreatingAction();break;case"tooltipEnabled":this._renderTooltip();break;case"displayCustomValue":case"acceptCustomValue":case"showSelectionControls":case"useInkRipple":this._invalidate();break;case"allowClearing":break;default:this.callBase(e)}},_clean:function(){delete this._inkRipple,this.callBase()}});(0,_.default)("dxSelectBox",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i,o,r){var s=l(t,n.originalEvent,r);if(!s){var u=(0,y.normalizeKeyName)(n);"enter"===u||"space"===u?(a(n.target,t),i&&i({event:n})):"tab"===u?o.addClass(C):g(e,t,n)}}function a(e,t){var n=(0,m.default)(e),i=n.attr("aria-label"),o=r(i,t.element()),a=o.index(n);R=(0,b.extend)({},{ariaLabel:i,index:a},{viewInstance:t})}function r(e,t){var n,i=(0,m.default)(t);return n=e?i.find('[aria-label="'.concat(e,'"][tabindex]')):i.find("[tabindex]")}function s(e){for(var t in e){var n=e[t],i=(0,m.default)(n).first();if(i.length)return i}}function l(e,t,n){var i={event:t,handled:!1};return n?n(i):e._createActionByOption("onKeyDown")(i),i.handled}function u(){P="visible"===x.default.getDocument().visibilityState}function c(){v.default.on(x.default.getDocument(),"visibilitychange",u)}function d(){v.default.off(x.default.getDocument(),"visibilitychange",u)}function h(e){P=!0,e.focus(),P=!1}function f(e,t,n,i,a,r){if(t.option("useLegacyKeyboardNavigation"))return w.noop;var s=(0,m.default)(t.element()),l=function(n){return o(e,t,n,a,s,r)},u=function(){A=!0,s.removeClass(C)},c=function(){var e=!A&&!P;e&&s.addClass(C),A=!1};return v.default.on(n,"keydown",i,l),v.default.on(n,"mousedown",i,u),v.default.on(n,"focusin",i,c),function(){v.default.off(n,"keydown",i,l),v.default.off(n,"mousedown",i,u),v.default.off(n,"focusin",i,c)}}function p(e){if(!e.option("useLegacyKeyboardNavigation")&&R){var t=R.viewInstance;if(t){var n=r(R.ariaLabel,t.element()),i=n.eq(R.index);R=null,v.default.trigger(i,"focus")}}}function g(e,t,n){var i=(0,y.normalizeKeyName)(n);if(n.ctrlKey&&("upArrow"===i||"downArrow"===i))for(var o=Object.keys(E),a=o.indexOf(e);a>=0&&a<o.length;){a="upArrow"===i?--a:++a;var r=o[a],l=E[r],u=s(l);if(u&&u.length){u.attr("tabindex",t.option("tabindex")||0),v.default.trigger(u,"focus"),u.removeClass(k);break}}}function _(e,t){e.option("useLegacyKeyboardnavigation")||t.attr("tabindex",e.option("tabindex")||0)}t.subscribeVisibilityChange=c,t.unsubscribeVisibilityChange=d,t.hiddenFocus=h,t.registerKeyboardAction=f,t.restoreFocus=p,t.selectView=g,t.setTabIndex=_;var m=i(n(2)),v=i(n(5)),y=n(8),b=n(0),x=i(n(11)),w=n(4),C="dx-state-focused",k="dx-cell-focus-disabled",S=".dx-row-focused",D=".dx-datagrid-rowsview .dx-row",I="".concat(D," > td"),T=".dx-treelist-rowsview .dx-row",O="".concat(T," > td"),E={groupPanel:[".dx-datagrid-group-panel .dx-group-panel-item[tabindex]"],columnHeaders:[".dx-datagrid-headers .dx-header-row > td.dx-datagrid-action",".dx-treelist-headers .dx-header-row > td.dx-treelist-action"],filterRow:[".dx-datagrid-headers .dx-datagrid-filter-row .dx-editor-cell .dx-texteditor-input",".dx-treelist-headers .dx-treelist-filter-row .dx-editor-cell .dx-texteditor-input"],rowsView:["".concat(S),"".concat(D,"[tabindex]"),"".concat(I,"[tabindex]"),"".concat(I),"".concat(T,"[tabindex]"),"".concat(O,"[tabindex]"),"".concat(O)],footer:[".dx-datagrid-total-footer .dx-datagrid-summary-item",".dx-treelist-total-footer .dx-treelist-summary-item"],filterPanel:[".dx-datagrid-filter-panel .dx-icon-filter",".dx-treelist-filter-panel .dx-icon-filter"],pager:[".dx-datagrid-pager [tabindex]",".dx-treelist-pager [tabindex]"]},A=!1,P=!1,R=null},function(e,t,n){n.d(t,"b",function(){return p}),n.d(t,"a",function(){return g}),n.d(t,"d",function(){return _});var i=n(73),o=n(307),a=n(790),r=n(148),s=n(510),l=n(511),u=n(512),c=n(792),d=n(168),h=n(513),f=function(){return f._nlp||(f._nlp=n(799)),f._nlp},p={MO:new d.b(0),TU:new d.b(1),WE:new d.b(2),TH:new d.b(3),FR:new d.b(4),SA:new d.b(5),SU:new d.b(6)},g={freq:r.a.YEARLY,dtstart:null,interval:1,wkst:p.MO,count:null,until:null,tzid:null,bysetpos:null,bymonth:null,bymonthday:null,bynmonthday:null,byyearday:null,byweekno:null,byweekday:null,bynweekday:null,byhour:null,byminute:null,bysecond:null,byeaster:null},_=Object.keys(g),m=function(){function e(e,t){void 0===e&&(e={}),void 0===t&&(t=!1),this._cache=t?null:new c.a,this.origOptions=Object(s.b)(e);var n=Object(s.c)(e).parsedOptions;this.options=n}return e.parseText=function(e,t){return f().parseText(e,t)},e.fromText=function(e,t){return f().fromText(e,t)},e.fromString=function(t){return new e(e.parseString(t)||void 0)},e.prototype._iter=function(e){return Object(h.a)(e,this.options)},e.prototype._cacheGet=function(e,t){return!!this._cache&&this._cache._cacheGet(e,t)},e.prototype._cacheAdd=function(e,t,n){if(this._cache)return this._cache._cacheAdd(e,t,n)},e.prototype.all=function(e){if(e)return this._iter(new a.a("all",{},e));var t=this._cacheGet("all");return t===!1&&(t=this._iter(new o.a("all",{})),this._cacheAdd("all",t)),t},e.prototype.between=function(e,t,n,r){if(void 0===n&&(n=!1),!i.b.isValidDate(e)||!i.b.isValidDate(t))throw new Error("Invalid date passed in to RRule.between");var s={before:t,after:e,inc:n};if(r)return this._iter(new a.a("between",s,r));var l=this._cacheGet("between",s);return l===!1&&(l=this._iter(new o.a("between",s)),this._cacheAdd("between",l,s)),l},e.prototype.before=function(e,t){if(void 0===t&&(t=!1),!i.b.isValidDate(e))throw new Error("Invalid date passed in to RRule.before");var n={dt:e,inc:t},a=this._cacheGet("before",n);return a===!1&&(a=this._iter(new o.a("before",n)),this._cacheAdd("before",a,n)),a},e.prototype.after=function(e,t){if(void 0===t&&(t=!1),!i.b.isValidDate(e))throw new Error("Invalid date passed in to RRule.after");var n={dt:e,inc:t},a=this._cacheGet("after",n);return a===!1&&(a=this._iter(new o.a("after",n)),this._cacheAdd("after",a,n)),a},e.prototype.count=function(){return this.all().length},e.prototype.toString=function(){return Object(u.a)(this.origOptions)},e.prototype.toText=function(e,t,n){return f().toText(this,e,t,n)},e.prototype.isFullyConvertibleToText=function(){return f().isFullyConvertible(this)},e.prototype.clone=function(){return new e(this.origOptions)},e.FREQUENCIES=["YEARLY","MONTHLY","WEEKLY","DAILY","HOURLY","MINUTELY","SECONDLY"],e.YEARLY=r.a.YEARLY,e.MONTHLY=r.a.MONTHLY,e.WEEKLY=r.a.WEEKLY,e.DAILY=r.a.DAILY,e.HOURLY=r.a.HOURLY,e.MINUTELY=r.a.MINUTELY,e.SECONDLY=r.a.SECONDLY,e.MO=p.MO,e.TU=p.TU,e.WE=p.WE,e.TH=p.TH,e.FR=p.FR,e.SA=p.SA,e.SU=p.SU,e.parseString=l.b,e.optionsToString=u.a,e}();t.c=m},function(e,t,n){function i(e,t){function n(){this.constructor=e}a(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}function o(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;for(var i=Array(e),o=0,t=0;t<n;t++)for(var a=arguments[t],r=0,s=a.length;r<s;r++,o++)i[o]=a[r];return i}t.b=i,n.d(t,"a",function(){return r}),t.c=o;/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var a=function(e,t){return(a=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},r=function(){return r=Object.assign||function(e){for(var t,n=1,i=arguments.length;n<i;n++){t=arguments[n];for(var o in t)Object.prototype.hasOwnProperty.call(t,o)&&(e[o]=t[o])}return e},r.apply(this,arguments)}},function(e,t,n){function i(e){return!!(e&&String(e).length>0)}function o(e,t,n,i,o){e.attr({text:t}).setMaxSize(n,o,i).textChanged&&e.setTitle(t)}function a(e){return e>=0?f(e):_}function r(e){var t;return e>=0?t={left:f(e),top:f(e),right:f(e),bottom:f(e)}:(e=e||{},t={left:a(e.left),top:a(e.top),right:a(e.right),bottom:a(e.bottom)}),t}function s(e,t){return e[2]-e[0]<t.width||e[3]-e[1]<t.height}function l(e){var t=(0,u.isString)(e)?{text:e}:e||{};return t.subtitle=(0,u.isString)(t.subtitle)?{text:t.subtitle}:t.subtitle||{},t}t.plugin=t.Title=void 0;var u=n(1),c=n(0),d=n(14),h=n(323),f=Number,p=(0,d.enumParser)(["left","center","right"]),g=(0,d.enumParser)(["top","bottom"]),_=10,m=function(e){this._params=e,this._group=e.renderer.g().attr({"class":e.cssClass}).linkOn(e.root||e.renderer.root,"title"),this._hasText=!1};t.Title=m,(0,c.extend)(m.prototype,h.LayoutElement.prototype,{dispose:function(){var e=this;e._group.linkRemove(),e._group.linkOff(),e._titleElement&&(e._clipRect.dispose(),e._titleElement=e._subtitleElement=e._clipRect=null),e._params=e._group=e._options=null},_updateOptions:function(e){this._options=e,this._options.horizontalAlignment=p(e.horizontalAlignment,"center"),this._options.verticalAlignment=g(e.verticalAlignment,"top"),this._options.margin=r(e.margin)},_updateStructure:function(){var e=this,t=e._params.renderer,n=e._group,o=e._options,a=o.horizontalAlignment;e._titleElement||(e._titleElement=t.text().append(n),e._subtitleElement=t.text(),e._clipRect=t.clipRect(),n.attr({"clip-path":e._clipRect.id})),e._titleElement.attr({align:a,"class":o.cssClass}),e._subtitleElement.attr({align:a,"class":o.subtitle.cssClass}),n.linkAppend(),i(o.subtitle.text)?e._subtitleElement.append(n):e._subtitleElement.remove()},_updateTexts:function(){var e,t=this,n=t._options,o=n.subtitle,a=t._titleElement,r=t._subtitleElement,s="A";a.attr({text:s,y:0}).css((0,d.patchFontOptions)(n.font)),e=a.getBBox(),t._baseLineCorrection=e.height+e.y,a.attr({text:n.text}),e=a.getBBox();var l=-e.y;a.attr({y:l}),i(o.text)&&r.attr({text:o.text,y:0}).css((0,d.patchFontOptions)(o.font))},_shiftSubtitle:function(){var e=this,t=e._titleElement.getBBox(),n=e._subtitleElement,i=e._options.subtitle.offset;n.move(0,t.y+t.height-n.getBBox().y-i)},_updateBoundingRectAlignment:function(){var e=this._boundingRect,t=this._options;e.verticalAlignment=t.verticalAlignment,e.horizontalAlignment=t.horizontalAlignment,e.cutLayoutSide=t.verticalAlignment,e.cutSide="vertical",e.position={horizontal:t.horizontalAlignment,vertical:t.verticalAlignment}},hasText:function(){return this._hasText},update:function(e,t){var n=this,o=(0,c.extend)(!0,{},e,l(t)),a=i(o.text),r=a||a!==n._hasText;return n._baseLineCorrection=0,n._updateOptions(o),n._boundingRect={},a?(n._updateStructure(),n._updateTexts()):n._group.linkRemove(),n._updateBoundingRect(),n._updateBoundingRectAlignment(),n._hasText=a,r},draw:function(e,t){var n=this;return n._hasText&&(n._group.linkAppend(),n._correctTitleLength(e),n._group.getBBox().height>t&&this.freeSpace()),n},_correctTitleLength:function(e){var t=this,n=t._options,i=n.margin,a=e-i.left-i.right,r=n.placeholderSize;o(t._titleElement,n.text,a,n,r),t._subtitleElement&&(f(r)>0&&(r-=t._titleElement.getBBox().height),o(t._subtitleElement,n.subtitle.text,a,n.subtitle,r),t._shiftSubtitle()),t._updateBoundingRect();var s=this.getCorrectedLayoutOptions(),l=s.x,u=s.y,c=s.height;this._clipRect.attr({x:l,y:u,width:e,height:c})},getLayoutOptions:function(){return this._boundingRect||null},shift:function(e,t){var n=this,i=n.getLayoutOptions();return n._group.move(e-i.x,t-i.y),n},_updateBoundingRect:function(){var e=this,t=e._options,n=t.margin,i=e._boundingRect,o=e._hasText?e._group.getBBox():{width:0,height:0,x:0,y:0,isEmpty:!0};o.isEmpty||(o.height+=n.top+n.bottom-e._baseLineCorrection,o.width+=n.left+n.right,o.x-=n.left,o.y+=e._baseLineCorrection-n.top),t.placeholderSize>0&&(o.height=t.placeholderSize),i.height=o.height,i.width=o.width,i.x=o.x,i.y=o.y},getCorrectedLayoutOptions:function(){var e=this.getLayoutOptions(),t=this._baseLineCorrection;return(0,c.extend)({},e,{y:e.y-t,height:e.height+t})},layoutOptions:function(){return this._hasText?{horizontalAlignment:this._boundingRect.horizontalAlignment,verticalAlignment:this._boundingRect.verticalAlignment,priority:0}:null},measure:function(e){return this.draw(e[0],e[1]),[this._boundingRect.width,this._boundingRect.height]},move:function(e,t){var n=this._boundingRect;s(e,n)?this.shift(t[0],t[1]):this.shift(Math.round(e[0]),Math.round(e[1]))},freeSpace:function(){var e=this;e._params.incidentOccurred("W2103"),e._group.linkRemove(),e._boundingRect.width=e._boundingRect.height=0},getOptions:function(){return this._options},changeLink:function(e){this._group.linkRemove(),this._group.linkOn(e,"title")}});var v={name:"title",init:function(){var e=this;e._title=new m({renderer:e._renderer,cssClass:e._rootClassPrefix+"-title",incidentOccurred:e._incidentOccurred}),e._layout.add(e._title)},dispose:function(){this._title.dispose(),this._title=null},customize:function(e){e.addChange({code:"TITLE",handler:function(){this._title.update(this._themeManager.theme("title"),this.option("title"))&&this._change(["LAYOUT"])},isThemeDependent:!0,option:"title",isOptionChange:!0})},fontFields:["title.font","title.subtitle.font"]};t.plugin=v},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e){var t=0;return(0,p.each)(e,function(e,n){t+=n}),t}function r(e){return(0,f.inArray)(e,[A,E,I,T,O])!==-1}function s(e,t){return a((0,m.map)(e,function(e){return(e-t)*(e-t)}))/e.length}function l(e,t,n){var i=n.getOptions().valueErrorBar,o=n.getValueFields()[0],a=i.lowValueField||D,r=i.highValueField||S;if(n.areErrorBarsVisible()&&void 0===i.type){var s=t.reduce(function(e,t){return(0,_.isDefined)(t[a])&&(e[0]+=t[o]-t[a],e[1]++),(0,_.isDefined)(t[r])&&(e[2]+=t[r]-t[o],e[3]++),e},[0,0,0,0]);s[1]&&(e[a]=e[o]-s[0]/s[1]),s[2]&&(e[r]=e[o]+s[2]/s[3])}return e}function u(e,t,n){var i=n.getOptions().valueErrorBar,o=i.lowValueField||D,a=i.highValueField||S;return n.areErrorBarsVisible()&&void 0===i.type&&(e[o]=0,e[a]=0,e=t.reduce(function(e,t){return e[o]+=t[o],e[a]+=t[a],e},e)),e}function c(e){return function(t,n){var i=t.intervalStart,a=t.data,r=n.getValueFields()[0],s=a[0];return s=a.reduce(function(t,n){var i=n[r];return null===t[r]&&(t=n),null!==i&&e(i,t[r])?n:t},s),(0,h.extend)({},s,o({},n.getArgumentField(),i))}}function d(e,t,n){var i=!0;for(var o in t){var a="argument"===o;if(a||"size"===o?!(0,_.isDefined)(e[o]):void 0===e[o]){var r=t[o];a||(n[r]=(n[r]||0)+1),i=!1}}return i}t.polar=t.chart=void 0;var h=n(0),f=n(12),p=n(3),g=i(n(230)),_=n(1),m=n(14),v=n(4),y=Math,b=y.abs,x=y.sqrt,w=y.max,C=12,k=400,S="highError",D="lowError",I="variance",T="stddeviation",O="stderror",E="percent",A="fixed",P="undefined",R="discrete",M="logarithmic",B="datetime",V={};t.chart=V;var F={};t.polar=F;var L={_defaultDuration:k,_defaultTrackerWidth:C,_applyStyle:v.noop,_updateOptions:v.noop,_parseStyle:v.noop,_prepareSegment:v.noop,_drawSegment:v.noop,_appendInGroup:function(){this._group.append(this._extGroups.seriesGroup)},_createLegendState:function(e,t){return{fill:e.color||t,hatching:e.hatching?(0,h.extend)({},e.hatching,{direction:"right"}):void 0}},_applyElementsClipRect:function(e){e["clip-path"]=this._paneClipRectID},_applyMarkerClipRect:function(e){e["clip-path"]=this._forceClipping?this._paneClipRectID:null},_createGroup:function(e,t,n,i){var o=t[e]=t[e]||this._renderer.g();n&&o.append(n),i&&o.attr(i)},_applyClearingSettings:function(e){e.opacity=null,e.scale=null,this._options.rotated?e.translateX=null:e.translateY=null},_createGroups:function(){var e=this;e._createGroup("_markersGroup",e,e._group),e._createGroup("_labelsGroup",e)},_setMarkerGroupSettings:function(){var e=this,t=e._createPointStyles(e._getMarkerGroupOptions()).normal;t.class="dxc-markers",t.opacity=1,e._applyMarkerClipRect(t),e._markersGroup.attr(t)},getVisibleArea:function(){return this._visibleArea},areErrorBarsVisible:function(){var e=this._options.valueErrorBar;return e&&this._errorBarsEnabled()&&"none"!==e.displayMode&&(r((0,m.normalizeEnum)(e.type))||(0,_.isDefined)(e.lowValueField)||(0,_.isDefined)(e.highValueField))},groupPointsByCoords:function(e){var t=[];return(0,p.each)(this.getVisiblePoints(),function(n,i){var o=parseInt(e?i.vy:i.vx);t[o]?Array.isArray(t[o])?t[o].push(i):t[o]=[t[o],i]:t[o]=i}),t},_createErrorBarGroup:function(e){var t,n=this,i=n._options.valueErrorBar;n.areErrorBarsVisible()&&(t={"class":"dxc-error-bars",stroke:i.color,"stroke-width":i.lineWidth,opacity:e?.001:i.opacity||1,"stroke-linecap":"square",sharp:!0,"clip-path":n._forceClipping?n._paneClipRectID:n._widePaneClipRectID},n._createGroup("_errorBarGroup",n,n._group,t))},_setGroupsSettings:function(e){var t=this;t._setMarkerGroupSettings(),t._setLabelGroupSettings(e),t._createErrorBarGroup(e)},_getCreatingPointOptions:function(){var e,t,n=this,i=n._predefinedPointOptions;return i||(e=n._getPointOptions(),n._predefinedPointOptions=i=(0,h.extend)(!0,{styles:{}},e),t=e.styles&&e.styles.normal||{},i.styles=i.styles||{},i.styles.normal={"stroke-width":t["stroke-width"],r:t.r,opacity:t.opacity}),i},_getPointOptions:function(){return this._parsePointOptions(this._preparePointOptions(),this._options.label)},_getOptionsForPoint:function(){return this._options.point},_parsePointStyle:function(e,t,n,i){var o=e.border||{},a=void 0!==e.size?e.size:i;return{fill:e.color||t,stroke:o.color||n,"stroke-width":o.visible?o.width:0,r:a/2+(o.visible&&0!==a?~~(o.width/2)||0:0)}},_createPointStyles:function(e){var t=this,n=e.color||t._options.mainSeriesColor,i=t._options.containerBackgroundColor,o=t._parsePointStyle(e,n,n);return o.visibility=e.visible?"visible":"hidden",{normal:o,hover:t._parsePointStyle(e.hoverStyle,i,n,e.size),selection:t._parsePointStyle(e.selectionStyle,i,n,e.size)}},_checkData:function(e,t,n){return n=n||{value:this.getValueFields()[0]},n.argument=this.getArgumentField(),d(e,n,t||{})&&e.value===e.value},getValueRangeInitialValue:function(){},_getRangeData:function(){return g.default.getRangeData(this)},_getPointDataSelector:function(){var e,t,n=this,i=this.getValueFields()[0],o=this.getArgumentField(),a=this.getTagField(),r=this.areErrorBarsVisible();if(r){var s=this._options.valueErrorBar;e=s.lowValueField||D,t=s.highValueField||S}return function(s){var l={value:n._processEmptyValue(s[i]),argument:s[o],tag:s[a],data:s};return r&&(l.lowError=s[e],l.highError=s[t]),l}},_errorBarsEnabled:function(){return this.valueAxisType!==R&&this.valueAxisType!==M&&this.valueType!==B},_drawPoint:function(e){var t=e.point;t.isInVisibleArea()?(t.clearVisibility(),t.draw(this._renderer,e.groups,e.hasAnimation,e.firstDrawing),this._drawnPoints.push(t)):t.setInvisibility()},_animateComplete:function(){var e=this,t={duration:e._defaultDuration};e._labelsGroup&&e._labelsGroup.animate({opacity:1},t),e._errorBarGroup&&e._errorBarGroup.animate({opacity:e._options.valueErrorBar.opacity||1},t)},_animate:function(){var e=this,t=e._drawnPoints.length-1;(0,p.each)(e._drawnPoints||[],function(n,i){i.animate(n===t?function(){e._animateComplete()}:void 0,{translateX:i.x,translateY:i.y})})},_defaultAggregator:"avg",_aggregators:{avg:function(e,t){var n,i=e.data,a=e.intervalStart;if(i.length){var r=t.getValueFields()[0],s=i.reduce(function(e,t){var n=t[r];return(0,_.isDefined)(n)?(e[0]+=n,e[1]++):null===n&&e[2]++,e},[0,0,0]);return l((n={},o(n,r,s[2]===i.length?null:s[0]/s[1]),o(n,t.getArgumentField(),a),n),i,t)}},sum:function(e,t){var n,i=e.intervalStart,a=e.data;if(a.length){var r=t.getValueFields()[0],s=a.reduce(function(e,t){var n=t[r];return void 0!==n&&(e[0]+=n),null===n?e[1]++:void 0===n&&e[2]++,e},[0,0,0]),l=s[0];if(s[1]===a.length&&(l=null),s[2]!==a.length)return u((n={},o(n,r,l),o(n,t.getArgumentField(),i),n),a,t)}},count:function(e,t){var n,i=e.data,a=e.intervalStart,r=t.getValueFields()[0];return n={},o(n,t.getArgumentField(),a),o(n,r,i.filter(function(e){return void 0!==e[r]}).length),n},min:c(function(e,t){return e<t}),max:c(function(e,t){return e>t})},_endUpdateData:function(){delete this._predefinedPointOptions},getArgumentField:function(){return this._options.argumentField||"arg"},getValueFields:function(){var e,t,n=this._options,i=n.valueErrorBar,o=[n.valueField||"val"];return i&&(e=i.lowValueField,t=i.highValueField,(0,_.isString)(e)&&o.push(e),(0,_.isString)(t)&&o.push(t)),o},_calculateErrorBars:function(e){if(this.areErrorBarsVisible()){var t,n,i,o,r,l=this,u=l._options,c=u.valueErrorBar,d=(0,m.normalizeEnum)(c.type),h=parseFloat(c.value),f=l.getValueFields()[0],g=c.lowValueField||D,v=c.highValueField||S,y=function(e,n){t=n.value,n.lowError=t-h,n.highError=t+h};switch(d){case A:r=y;break;case E:r=function(e,n){t=n.value;var i=t*h/100;n.lowError=t-i,n.highError=t+i};break;case P:r=function(e,t){t.lowError=t.data[g],t.highError=t.data[v]};break;default:switch(n=(0,m.map)(e,function(e){return(0,_.isDefined)(e.data[f])?e.data[f]:null}),i=n.length,h=h||1,d){case I:h=s(n,a(n)/i)*h,r=y;break;case T:o=a(n)/i,h=x(s(n,o))*h,r=function(e,t){t.lowError=o-h,t.highError=o+h};break;case O:h=x(s(n,a(n)/i)/i)*h,r=y}}r&&(0,p.each)(e,r)}},_patchMarginOptions:function(e){var t=this._getCreatingPointOptions(),n=t.styles,i=[n.normal,n.hover,n.selection].reduce(function(e,t){return w(e,2*t.r+t["stroke-width"])},0);return e.size=t.visible?i:0,e.sizePointNormalState=t.visible?2*n.normal.r+n.normal["stroke-width"]:2,e},usePointsToDefineAutoHiding:function(){return!0}};t.chart=V=(0,h.extend)({},L,{drawTrackers:function(){var e,t,n=this,i=n._segments||[],o=n._options.rotated;n.isVisible()&&(i.length&&(e=n._trackers=n._trackers||[],t=n._trackersGroup=(n._trackersGroup||n._renderer.g().attr({fill:"gray",opacity:.001,stroke:"gray","class":"dxc-trackers"})).attr({"clip-path":this._paneClipRectID||null}).append(n._group),(0,p.each)(i,function(i,o){e[i]?n._updateTrackerElement(o,e[i]):e[i]=n._drawTrackerElement(o).data({"chart-data-series":n}).append(t)})),n._trackersTranslator=n.groupPointsByCoords(o))},_checkAxisVisibleAreaCoord:function(e,t){var n=e?this.getArgumentAxis():this.getValueAxis(),i=n.getVisibleArea();return(0,_.isDefined)(t)&&i[0]<=t&&i[1]>=t},checkSeriesViewportCoord:function(e,t){return!0},getSeriesPairCoord:function(e,t){for(var n=null,i=!t&&!this._options.rotated||t&&this._options.rotated,o=i?"vy":"vx",a=i?"vx":"vy",r=this.getVisiblePoints(),s=0;s<r.length;s++){var l=r[s],u=l[o]===e?l[a]:void 0;if(this._checkAxisVisibleAreaCoord(!t,u)){n=u;break}}return n},_getNearestPoints:function(e,t){return[e,t]},_getBezierPoints:function(){return[]},_getNearestPointsByCoord:function(e,t){var n=this,i=n.getOptions().rotated,o=!t&&!i||t&&i,a=o?"vy":"vx",r=n.getPoints(),s=n._getBezierPoints(),l=[];return r.length>1?r.forEach(function(t,i){var o=r[i+1];o&&(t[a]<=e&&o[a]>=e||t[a]>=e&&o[a]<=e)&&l.push(n._getNearestPoints(t,o,s))}):l.push([r[0],r[0]]),l},getNeighborPoint:function(e,t){var n,i=this._options.rotated?t:e,o=i,a=this._trackersTranslator,r=null,s=this._options.rotated?e:t,l=this._options.rotated?"vx":"vy";if(this.isVisible()&&a){r=a[i];do r=a[o]||a[i],i--,o++;while((i>=0||o<a.length)&&!r);Array.isArray(r)&&(n=b(r[0][l]-s),(0,p.each)(r,function(e,t){var i=b(t[l]-s);n>=i&&(n=i,r=t)}))}return r},_applyVisibleArea:function(){var e=this,t=e._options.rotated,n=(t?e.getValueAxis():e.getArgumentAxis()).getVisibleArea(),i=(t?e.getArgumentAxis():e.getValueAxis()).getVisibleArea();e._visibleArea={minX:n[0],maxX:n[1],minY:i[0],maxY:i[1]}},getPointCenterByArg:function(e){var t=this.getPointsByArg(e)[0];return t?t.getCenterCoord():void 0}}),t.polar=F=(0,h.extend)({},L,{drawTrackers:function(){V.drawTrackers.call(this);var e,t=this._trackersTranslator;this.isVisible()&&((0,p.each)(t,function(t,n){if(n)return e=t,!1}),t[e+360]=t[e])},getNeighborPoint:function(e,t){var n=(0,m.convertXYToPolar)(this.getValueAxis().getCenter(),e,t);return V.getNeighborPoint.call(this,n.phi,n.r)},_applyVisibleArea:function(){var e=this,t=e.getValueAxis().getCanvas();e._visibleArea={minX:t.left,maxX:t.width-t.right,minY:t.top,maxY:t.height-t.bottom}},getSeriesPairCoord:function(e,t){for(var n=null,i=t?"argument":"radius",o=this.getVisiblePoints(),a=0;a<o.length;a++){var r=o[a],s=(0,_.isDefined)(r[i])&&(0,_.isDefined)(e[i])&&r[i].valueOf()===e[i].valueOf()?{x:r.x,y:r.y}:void 0;if((0,_.isDefined)(s)){n=s;break}}return n}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){e.css({left:"-9999px"}).detach()}function a(e,t){var n=e;switch(t){case"argument":n={format:e.argumentFormat};break;case"percent":n={format:{type:"percent",precision:e.format&&e.format.percentPrecision}}}return n}t.plugin=t.Tooltip=void 0;var r=i(n(11)),s=n(7),l=n(40),u=i(n(2)),c=n(171),d=n(1),h=n(0),f=n(14),p=i(n(79)),g=n(549),_=p.default.format,m=Math.ceil,v=Math.max,y=Math.min,b=(0,s.getWindow)(),x=3e3,w=function(e){var t,n=this;n._eventTrigger=e.eventTrigger,n._widgetRoot=e.widgetRoot,n._widget=e.widget,n._wrapper=(0,u.default)("<div>").css({position:"absolute",overflow:"hidden",pointerEvents:"none"}).addClass(e.cssClass),n._renderer=t=new c.Renderer({pathModified:e.pathModified,container:n._wrapper[0]});var i=t.root;i.attr({"pointer-events":"none"}),n._text=t.text(void 0,0,0),n._textGroupHtml=(0,u.default)("<div>").css({position:"absolute",padding:0,margin:0,border:"0px solid transparent"}).appendTo(n._wrapper),n._textHtml=(0,u.default)("<div>").css({position:"relative",display:"inline-block",padding:0,margin:0,border:"0px solid transparent"}).appendTo(n._textGroupHtml)};t.Tooltip=w,w.prototype={constructor:w,dispose:function(){this._wrapper.remove(),this._renderer.dispose(),this._options=this._widgetRoot=null},_getContainer:function(){var e=this._options,t=(0,u.default)(this._widgetRoot).closest(e.container);return 0===t.length&&(t=(0,u.default)(e.container)),(t.length?t:(0,u.default)("body")).get(0)},setTemplate:function(e){var t=this;t._template=e?t._widget._getTemplate(e):null},setOptions:function(e){var t=this;e=e||{};var n=this;n._options=e,n._textFontStyles=(0,f.patchFontOptions)(e.font),n._textFontStyles.color=n._textFontStyles.fill,n._wrapper.css({zIndex:e.zIndex}),n._customizeTooltip=e.customizeTooltip;var i=n._textGroupHtml,o=n._textHtml;this.plaque&&this.plaque.clear(),this.setTemplate(e.contentTemplate);var a=e.interactive?"auto":"none";e.interactive&&this._renderer.root.css({"-ms-user-select":"auto","-moz-user-select":"auto","-webkit-user-select":"auto"});var r=function(r){var s=r.group,l=r.onRender,u=r.eventData,c=r.isMoving,d=r.templateCallback,h=void 0===d?function(){}:d,f=n._state;if(!c){var p=n._template,g=p&&!f.formatObject.skipTemplate;if(f.html||g){if(i.css({color:f.textColor,width:x,pointerEvents:a}),g)return void p.render({model:f.formatObject,container:o,onRendered:function(){return f.html=o.html(),0===o.width()&&0===o.height()?(t.plaque.clear(),void h(!1)):(l(),n._riseEvents(u),n._moveWrapper(),n.plaque.customizeCloud({fill:f.color,stroke:f.borderColor,"pointer-events":a}),void h(!0))}});n._text.attr({text:""}),o.html(f.html)}else n._text.css({fill:f.textColor}).attr({text:f.text,"class":e.cssClass,"pointer-events":a}).append(s.attr({align:e.textAlignment}));n._riseEvents(u),n.plaque.customizeCloud({fill:f.color,stroke:f.borderColor,"pointer-events":a})}return l(),n._moveWrapper(),!0};return this.plaque=new g.Plaque({opacity:n._options.opacity,color:n._options.color,border:n._options.border,paddingLeftRight:n._options.paddingLeftRight,paddingTopBottom:n._options.paddingTopBottom,arrowLength:n._options.arrowLength,arrowWidth:20,shadow:n._options.shadow,cornerRadius:n._options.cornerRadius},n,n._renderer.root,r,(!0),function(e,t){var n=e._state;if(n.html){var i,a=b.getComputedStyle;return a?(i=a(o.get(0)),i={x:0,y:0,width:m(parseFloat(i.width)),height:m(parseFloat(i.height))}):(i=o.get(0).getBoundingClientRect(),i={x:0,y:0,width:m(i.width?i.width:i.right-i.left),height:m(i.height?i.height:i.bottom-i.top)}),i}return t.getBBox()},function(e,t,i,o){var a=e._state;a.html?n._textGroupHtml.css({left:i,top:o}):t.move(i,o)}),n},_riseEvents:function(e){var t=this;t._eventData&&t._eventTrigger("tooltipHidden",t._eventData),t._eventData=e,t._eventTrigger("tooltipShown",t._eventData)},setRendererOptions:function(e){return this._renderer.setOptions(e),this._textGroupHtml.css({direction:e.rtl?"rtl":"ltr"}),this},update:function(e){var t=this;t.setOptions(e),o(t._wrapper);var n={};for(var i in t._textFontStyles)n[(0,l.camelize)(i)]=t._textFontStyles[i];return t._textGroupHtml.css(n),t._text.css(t._textFontStyles),t._eventData=null,t},_prepare:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this._customizeTooltip,i=this._options,o={};return(0,d.isFunction)(n)&&(o=n.call(e,e),o=(0,d.isPlainObject)(o)?o:{},"text"in o&&(t.text=(0,d.isDefined)(o.text)?String(o.text):""),"html"in o&&(t.html=(0,d.isDefined)(o.html)?String(o.html):"")),"text"in t||"html"in t||(t.text=e.valueText||e.description||""),t.color=o.color||i.color,t.borderColor=o.borderColor||(i.border||{}).color,t.textColor=o.fontColor||(this._textFontStyles||{}).color,!!t.text||!!t.html||!!this._template},show:function(e,t,n,i,o){var a=this;if(a._options.forceEvents)return n.x=t.x,n.y=t.y-t.offset,a._riseEvents(n),!0;var r={formatObject:e,eventData:n,templateCallback:o};if(!a._prepare(e,r,i))return!1;a._state=r,a._wrapper.appendTo(a._getContainer()),a._clear();var s=(0,h.extend)({},a._options,{canvas:a._getCanvas()},r,{x:t.x,y:t.y,offset:t.offset});return this.plaque.clear().draw(s)},isCursorOnTooltip:function(e,t){if(this._options.interactive&&this.isEnabled()){var n=this.plaque.getBBox();return e>n.x&&e<n.x+n.width&&t>n.y&&t<n.y+n.height}return!1},hide:function(){var e=this;o(e._wrapper),e._eventData&&(e._eventTrigger("tooltipHidden",e._eventData),e._clear(),e._eventData=null)},_clear:function(){this._textHtml.empty()},move:function(e,t,n){this.plaque.draw({x:e,y:t,offset:n,canvas:this._getCanvas(),isMoving:!0})},_moveWrapper:function(){var e=this,t=this.plaque.getBBox();e._renderer.resize(t.width,t.height);var n=e._wrapper.css({left:0,top:0}).offset(),i=t.x,o=t.y;e._wrapper.css({left:i-n.left,top:o-n.top}),this.plaque.moveRoot(-i,-o),this._state.html&&(e._textHtml.css({left:-i,top:-o}),e._textGroupHtml.css({width:e._textHtml.width()}))},formatValue:function(e,t){var n=t?a(this._options,t):this._options;return _(e,n.format)},getLocation:function(){return(0,f.normalizeEnum)(this._options.location)},isEnabled:function(){return!!this._options.enabled||!!this._options.forceEvents},isShared:function(){return!!this._options.shared},_getCanvas:function(){var e=this._getContainer(),t=e.getBoundingClientRect(),n=r.default.getDocumentElement(),i=r.default.getDocument(),o=b.pageXOffset||n.scrollLeft||0,a=b.pageYOffset||n.scrollTop||0,s={left:o,top:a,width:v(n.clientWidth,i.body.clientWidth)+o,height:v(i.body.scrollHeight,n.scrollHeight,i.body.offsetHeight,n.offsetHeight,i.body.clientHeight,n.clientHeight),right:0,bottom:0};return e!==r.default.getBody()&&(o=v(s.left,s.left+t.left),a=v(s.top,s.top+t.top),s.width=y(t.width,s.width)+o+s.left,s.height=y(t.height,s.height)+a+s.top,s.left=o,s.top=a),s}};var C={name:"tooltip",init:function(){this._initTooltip()},dispose:function(){this._disposeTooltip()},members:{_initTooltip:function(){this._tooltip=new w({cssClass:this._rootClassPrefix+"-tooltip",eventTrigger:this._eventTrigger,pathModified:this.option("pathModified"),widgetRoot:this.element(),widget:this})},_disposeTooltip:function(){this._tooltip.dispose(),this._tooltip=null},_setTooltipRendererOptions:function(){this._tooltip.setRendererOptions(this._getRendererOptions())},_setTooltipOptions:function(){this._tooltip.update(this._getOption("tooltip"))}},extenders:{_stopCurrentHandling:function(){this._tooltip&&this._tooltip.hide()}},customize:function(e){var t=e.prototype;t._eventsMap.onTooltipShown={name:"tooltipShown"},t._eventsMap.onTooltipHidden={name:"tooltipHidden"},e.addChange({code:"TOOLTIP_RENDERER",handler:function(){this._setTooltipRendererOptions()},isThemeDependent:!0,isOptionChange:!0}),e.addChange({code:"TOOLTIP",handler:function(){this._setTooltipOptions()},isThemeDependent:!0,isOptionChange:!0,option:"tooltip"})},fontFields:["tooltip.font"]};t.plugin=C},function(e,t,n){function i(e,t){return t<e}function o(e,t){return t>e}function a(e,t,n,i){var o=u(t);u(e)?o&&i(e,t)&&n(t):o&&n(t)}t.Range=void 0;var r=n(1),s=n(0),l=n(14),u=r.isDefined,c=r.isDate,d=r.isFunction,h="min",f="max",p="minVisible",g="maxVisible",_="base",m="axisType",v=function(e){e&&(0,s.extend)(this,e)};t.Range=v;var y=v;y.prototype={constructor:y,addRange:function(e){var t=this,n=t.categories,r=e.categories,s="discrete"===t[m],c=function(n,i){a(t[n],e[n],function(e){t[n]=e},i)},d=function(e,n,i){a(t[e],t[n],function(n){u(t[e])&&(t[e]=n)},i)},v=function(n){t[n]=t[n]||e[n]};return v("invert"),v(m),v("dataType"),v("isSpacedMargin"),"logarithmic"===t[m]?v(_):t[_]=void 0,c(h,i),c(f,o),s?(v(p),v(g)):(c(p,i),c(g,o)),c("interval",i),s||(d(h,p,i),d(h,g,i),d(f,g,o),d(f,p,o)),void 0===n?t.categories=r:t.categories=r?(0,l.unique)(n.concat(r)):n,"logarithmic"===t[m]&&(v("allowNegatives"),c("linearThreshold",i)),t},isEmpty:function(){return!(u(this[h])&&u(this[f])||this.categories&&0!==this.categories.length)},correctValueZeroLevel:function(){function e(e,n){t[e]<0&&t[n]<0&&(t[n]=0),t[e]>0&&t[n]>0&&(t[e]=0)}var t=this;return"logarithmic"===t[m]||c(t[f])||c(t[h])?t:(e(h,f),e(p,g),t)},sortCategories:function(e){if(e!==!1&&this.categories)if(Array.isArray(e)){var t=e.map(function(e){return e.valueOf()}),n=this.categories.filter(function(e){return t.indexOf(e.valueOf())===-1});this.categories=e.concat(n)}else{var i=!d(e);i&&"string"!==this.dataType?e=function(e,t){return e.valueOf()-t.valueOf()}:i&&(e=!1),e&&this.categories.sort(e)}}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){function n(e){var t=e[0];return e=e.slice(1),o(t,i(t,e))}function i(e,t){return t=[d.ERROR_MESSAGES[e]].concat(t),s.format.apply(this,t).replace(/\.*\s*?$/,"")}function o(e,t){return s.format.apply(this,["{0} - {1}. See:\n{2}",e,t,c(e)])}function l(e){var t=e[0];e=e.slice(1);var n=i(t,e),r=c(t),s=o(t,n);return(0,a.extend)(new Error(s),{__id:t,__details:n,url:r})}function c(e){return u+e}var d={ERROR_MESSAGES:(0,a.extend)(t,e),Error:function(){return l([].slice.call(arguments))},log:function(e){var t="log";/^E\d+$/.test(e)?t="error":/^W\d+$/.test(e)&&(t="warn"),r.logger[t]("log"===t?e:n([].slice.call(arguments)))}};return d}t.default=o;var a=n(0),r=n(106),s=n(42),l=i(n(160)),u="http://js.devexpress.com/error/"+l.default.split(".").slice(0,2).join("_")+"/";e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=(0,r.data)(e.get(0)),o=g(t.constructor);i[o]=t,n&&s.default.one(e,c.default,function(){n.call(t)}),i[d]||(i[d]=[]),i[d].push(o)}function a(e,t){var n=g(t);return(0,r.data)(e.get(0),n)}t.attachInstanceToElement=o,t.getInstanceByElement=a,t.name=void 0;var r=n(39),s=i(n(5)),l=i(n(182)),u=n(1),c=i(n(138)),d="dxComponents",h="dxPrivateComponent",f=new l.default,p=0,g=function(e,t){if((0,u.isDefined)(t))return void f.set(e,t);if(!f.has(e)){var n=h+p++;return f.set(e,n),n}return f.get(e)};t.name=g},function(e,t,n){var i=n(7),o=i.getWindow(),a=o.DevExpress=o.DevExpress||{},r=a.errors=n(22);if(a._DEVEXTREME_BUNDLE_INITIALIZED)throw r.Error("E0024");a._DEVEXTREME_BUNDLE_INITIALIZED=!0,a.clientExporter=n(181),a.excelExporter=n(350),a.pdfExporter=n(355),a.VERSION=n(160),a.Class=n(15),a.DOMComponent=n(65),a.Component=n(117),a.registerComponent=n(9),a.devices=n(13),a.Color=n(98);var s=n(120);a.utils={},a.utils.requestAnimationFrame=s.requestAnimationFrame,a.utils.cancelAnimationFrame=s.cancelAnimationFrame,a.utils.initMobileViewport=n(365).initMobileViewport,a.utils.getTimeZones=n(366).getTimeZones,a.utils.extendFromObject=n(0).extendFromObject,a.utils.triggerShownEvent=n(55).triggerShownEvent,a.utils.triggerHidingEvent=n(55).triggerHidingEvent,a.utils.resetActiveElement=n(33).resetActiveElement,a.utils.findBestMatches=n(4).findBestMatches,a.createQueue=n(252).create,a.utils.dom=n(33),a.utils.common=n(4),a.utils.date=n(20),a.utils.browser=n(23),a.utils.inflector=n(40),a.utils.iterator=n(3),a.utils.readyCallbacks=n(49),a.utils.resizeCallbacks=n(114),a.utils.console=n(106),a.utils.string=n(42),a.utils.support=n(46),a.utils.ajax=n(76),a.viewPort=n(78).value,a.hideTopOverlay=n(370),a.formatHelper=n(79),a.config=n(28),a.animationPresets=n(187).presets,a.fx=n(43),a.TransitionExecutor=n(271).TransitionExecutor,a.AnimationPresetCollection=n(187).PresetCollection,a.events=n(371),a.events.click=n(21),a.events.utils=n(8),a.events.GestureEmitter=n(163),a.localization=n(375),a.templateRendered=n(67).renderedCallbacks,a.setTemplateEngine=n(162).setTemplateEngine,e.exports=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(39),r=i(n(5)),s=i(n(66)),l="dxremove",u="dxRemoveEvent";(0,a.beforeCleanData)(function(e){e=[].slice.call(e);for(var t=0;t<e.length;t++){var n=(0,o.default)(e[t]);n.prop(u)&&(n[0][u]=null,r.default.triggerHandler(n,l))}}),(0,s.default)(l,{noBubble:!0,setup:function(e){(0,o.default)(e).prop(u,!0)}});var c=l;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.name=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(66)),s=n(8),l="dxmousewheel";t.name=l;var u="dxWheel",c="wheel",d=0,h=30,f={setup:function(e){var t=(0,o.default)(e);a.default.on(t,(0,s.addNamespace)(c,u),f._wheelHandler.bind(f))},teardown:function(e){a.default.off(e,".".concat(u))},_wheelHandler:function(e){var t=e.originalEvent,n=t.deltaMode,i=t.deltaY,o=t.deltaX,a=t.deltaZ;(0,s.fireEvent)({type:l,originalEvent:e,delta:this._normalizeDelta(i,n),deltaX:o,deltaY:i,deltaZ:a,deltaMode:n,pointerType:"mouse"}),e.stopPropagation()},_normalizeDelta:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:d;return t===d?-e:-h*e}};(0,r.default)(l,f)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.inactive=t.active=t.lock=void 0;var o=i(n(15)),a=n(4),r=n(33),s=i(n(13)),l=n(8),u=i(n(25)),c=i(n(121)),d=i(n(99)),h="dxactive";t.active=h;var f="dxinactive";t.inactive=f;var p,g=30,_=400,m=o.default.inherit({ctor:function(e,t){this._timeout=e,this._fire=t},start:function(){var e=this;this._schedule(function(){e.force()})},_schedule:function(e){this.stop(),this._timer=setTimeout(e,this._timeout)},stop:function(){clearTimeout(this._timer)},force:function(){this._fired||(this.stop(),this._fire(),this._fired=!0)},fired:function(){return this._fired}}),v=c.default.inherit({ctor:function(){this.callBase.apply(this,arguments),this._active=new m(0,a.noop),this._inactive=new m(0,a.noop)},configure:function(e,t){switch(t){case h:e.activeTimeout=e.timeout;break;case f:e.inactiveTimeout=e.timeout}this.callBase(e)},start:function(e){if(p){var t=(0,r.contains)(this.getElement().get(0),p.getElement().get(0)),n=!p._active.fired();if(t&&n)return void this._cancel();p._inactive.force()}p=this,this._initEvents(e),this._active.start()},_initEvents:function(e){var t=this,n=this._getEmitterTarget(e),i=(0,l.isMouseEvent)(e),o=s.default.isSimulator(),r=o||!i,u=(0,a.ensureDefined)(this.activeTimeout,g),c=(0,a.ensureDefined)(this.inactiveTimeout,_);this._active=new m(r?u:0,function(){t._fireEvent(h,e,{target:n})}),this._inactive=new m(r?c:0,function(){
t._fireEvent(f,e,{target:n}),p=null})},cancel:function(e){this.end(e)},end:function(e){var t=e.type!==u.default.up;t?this._active.stop():this._active.force(),this._inactive.start(),t&&this._inactive.force()},dispose:function(){this._active.stop(),this._inactive.stop(),p===this&&(p=null),this.callBase()},lockInactive:function(){return this._active.force(),this._inactive.stop(),p=null,this._cancel(),this._inactive.force.bind(this._inactive)}});v.lock=function(e){var t=p?p.lockInactive():a.noop;e.done(t)},(0,d.default)({emitter:v,events:[h,f]});var y=v.lock;t.lock=y},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(15)),r=n(3),s=n(136),l="dx-state-invisible",u="dx-state-disabled",c="dx-item-content-placeholder",d=function(e,t,n){var i=function(){var e;return function(t){e!==t&&(n(t,e),e=t)}}();return{dispose:e(t,i),force:function(){i(t())}}},h=a.default.inherit({ctor:function(e,t,n){this._$element=e,this._options=t,this._rawData=n,(0,s.attachInstanceToElement)(e,this,this._dispose),this._render()},_render:function(){var e=(0,o.default)("<div>").addClass(c);this._$element.append(e),this._watchers=[],this._renderWatchers()},_renderWatchers:function(){this._startWatcher("disabled",this._renderDisabled.bind(this)),this._startWatcher("visible",this._renderVisible.bind(this))},_startWatcher:function(e,t){var n=this._rawData,i=this._options.fieldGetter(e),o=d(this._options.watchMethod(),function(){return i(n)},function(e,n){this._dirty=!0,t(e,n)}.bind(this));this._watchers.push(o)},setDataField:function(){if(this._dirty=!1,(0,r.each)(this._watchers,function(e,t){t.force()}),this._dirty)return!0},_renderDisabled:function(e,t){this._$element.toggleClass(u,!!e),this._updateOwnerFocus(e)},_updateOwnerFocus:function(e){var t=this._options.owner;t&&e&&t._resetItemFocus(this._$element)},_renderVisible:function(e,t){this._$element.toggleClass(l,void 0!==e&&!e)},_dispose:function(){(0,r.each)(this._watchers,function(e,t){t.dispose()})}});h.getInstance=function(e){return(0,s.getInstanceByElement)(e,this)};var f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.name=void 0;var o=i(n(5)),a=n(33),r=i(n(11)),s=i(n(15)),l=i(n(66)),u=n(21),c=n(8),d="dxdblclick";t.name=d;var h="dxDblClick",f=(0,c.addNamespace)(u.name,h),p=300,g=s.default.inherit({ctor:function(){this._handlerCount=0,this._forgetLastClick()},_forgetLastClick:function(){this._firstClickTarget=null,this._lastClickTimeStamp=-p},add:function(){this._handlerCount<=0&&o.default.on(r.default.getDocument(),f,this._clickHandler.bind(this)),this._handlerCount++},_clickHandler:function(e){var t=e.timeStamp||Date.now(),n=t-this._lastClickTimeStamp,i=n<0,o=!i&&n<p;o?((0,c.fireEvent)({type:d,target:(0,a.closestCommonParent)(this._firstClickTarget,e.target),originalEvent:e}),this._forgetLastClick()):(this._firstClickTarget=e.target,this._lastClickTimeStamp=t)},remove:function(){this._handlerCount--,this._handlerCount<=0&&(this._forgetLastClick(),o.default.off(r.default.getDocument(),f))}});(0,l.default)(d,new g)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(2)),r=n(7),s=n(18),l=i(n(11)),u=i(n(5)),c=i(n(9)),d=n(4),h=n(0),f=n(30),p=i(n(84)),g=n(1),_=n(29),m=n(8),v=i(n(48)),y=n(24),b=(0,r.getWindow)(),x="dx-popover",w="dx-popover-wrapper",C="dx-popover-arrow",k="dx-popover-without-title",S={left:"right",top:"bottom",right:"left",bottom:"top",center:"center"},D={left:-1,top:-1,center:0,right:1,bottom:1},I={top:{my:"bottom center",at:"top center",collision:"fit flip"},bottom:{my:"top center",at:"bottom center",collision:"fit flip"},right:{my:"left center",at:"right center",collision:"flip fit"},left:{my:"right center",at:"left center",collision:"flip fit"}},T={left:"borderLeftWidth",top:"borderTopWidth",right:"borderRightWidth",bottom:"borderBottomWidth"},O=function(e){return(0,g.isObject)(e)?e.name:e},E=function(e,t){var n=e.option(t);return O(n)},A=function(e,t){var n=e.option(t);return(0,g.isObject)(n)&&n.delay},P=function(e,t){var n=e.option("target"),i=(0,g.isString)(n),o=E(e,t+"Event");if(o&&!e.option("disabled")){var r=(0,m.addNamespace)(o,e.NAME),c=e._createAction(function(){var n=A(e,t+"Event");this._clearEventsTimeouts(),n?this._timeouts[t]=setTimeout(function(){e[t]()},n):e[t]()}.bind(e),{validatingTargetName:"target"}),d=function(e){c({event:e,target:(0,a.default)(e.currentTarget)})},h="_"+t+"EventHandler";if(i)e[h]=d,u.default.on(l.default.getDocument(),r,n,d);else{var f=(0,s.getPublicElement)((0,a.default)(n));e[h]=void 0,u.default.on(f,r,d)}}},R=function(e,t,n,i){var o=i||E(e,n+"Event");if(o){o=(0,m.addNamespace)(o,e.NAME);var r="_"+n+"EventHandler";e[r]?u.default.off(l.default.getDocument(),o,t,e[r]):u.default.off((0,s.getPublicElement)((0,a.default)(t)),o)}},M=v.default.inherit({_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{target:b,shading:!1,position:"bottom",closeOnOutsideClick:!0,animation:{show:{type:"fade",from:0,to:1},hide:{type:"fade",to:0}},showTitle:!1,width:"auto",height:"auto",dragEnabled:!1,resizeEnabled:!1,fullScreen:!1,closeOnTargetScroll:!0,arrowPosition:"",arrowOffset:0,boundaryOffset:{h:10,v:10},_fixedPosition:!0})},_defaultOptionsRules:function(){return[{device:{platform:"ios"},options:{arrowPosition:{boundaryOffset:{h:20,v:-10},collision:"fit"}}},{device:function(){return!(0,r.hasWindow)()},options:{animation:null}}]},_init:function(){this.callBase(),this._renderArrow(),this._timeouts={},this.$element().addClass(x),this._wrapper().addClass(w)},_render:function(){this.callBase.apply(this,arguments),this._detachEvents(this.option("target")),this._attachEvents()},_detachEvents:function(e){R(this,e,"show"),R(this,e,"hide")},_attachEvents:function(){P(this,"show"),P(this,"hide")},_renderArrow:function(){this._$arrow=(0,a.default)("<div>").addClass(C).prependTo(this.overlayContent())},_documentDownHandler:function(e){return!this._isOutsideClick(e)||this.callBase(e)},_isOutsideClick:function(e){return!(0,a.default)(e.target).closest(this.option("target")).length},_animate:function(e){e&&e.to&&"object"===o(e.to)&&(0,h.extend)(e.to,{position:this._getContainerPosition()}),this.callBase.apply(this,arguments)},_stopAnimation:function(){this.callBase.apply(this,arguments)},_renderTitle:function(){this._wrapper().toggleClass(k,!this.option("showTitle")),this.callBase()},_renderPosition:function(){this.callBase(),this._renderOverlayPosition()},_renderOverlayBoundaryOffset:d.noop,_renderOverlayPosition:function(){this._resetOverlayPosition(),this._updateContentSize();var e=this._getContainerPosition(),t=p.default.setup(this._$content,e),n=this._getSideByLocation(t);this._togglePositionClass("dx-position-"+n),this._toggleFlippedClass(t.h.flip,t.v.flip);var i=this._isHorizontalSide()||this._isVerticalSide();i&&this._renderArrowPosition(n)},_resetOverlayPosition:function(){this._setContentHeight(!0),this._togglePositionClass("dx-position-"+this._positionSide),(0,f.move)(this._$content,{left:0,top:0}),this._$arrow.css({top:"auto",right:"auto",bottom:"auto",left:"auto"})},_updateContentSize:function(){if(this._$popupContent){var e=p.default.calculate(this._$content,this._getContainerPosition());if(e.h.oversize>0&&this._isHorizontalSide()&&!e.h.fit){var t=this._$content.width()-e.h.oversize;this._$content.width(t)}if(e.v.oversize>0&&this._isVerticalSide()&&!e.v.fit){var n=this._$content.height()-e.v.oversize,i=this._$popupContent.height()-e.v.oversize;this._$content.height(n),this._$popupContent.height(i)}}},_getContainerPosition:function(){var e=(0,d.pairToObject)(this._position.offset||""),t=e.h,n=e.v,i=this._isVerticalSide(),o=this._isHorizontalSide();if(i||o){var a=this._isPopoverInside(),r=(a?-1:1)*D[this._positionSide],s=i?this._$arrow.height():this._$arrow.width(),l=this._getContentBorderWidth(this._positionSide),u=r*(s-l);i?n+=u:t+=u}return(0,h.extend)({},this._position,{offset:t+" "+n})},_getContentBorderWidth:function(e){var t=this._$content.css(T[e]);return parseInt(t)||0},_getSideByLocation:function(e){var t=e.v.flip,n=e.h.flip;return this._isVerticalSide()&&t||this._isHorizontalSide()&&n||this._isPopoverInside()?S[this._positionSide]:this._positionSide},_togglePositionClass:function(e){this._$wrapper.removeClass("dx-position-left dx-position-right dx-position-top dx-position-bottom").addClass(e)},_toggleFlippedClass:function(e,t){this._$wrapper.toggleClass("dx-popover-flipped-horizontal",e).toggleClass("dx-popover-flipped-vertical",t)},_renderArrowPosition:function(e){var t=(0,y.getBoundingRect)(this._$arrow.get(0)),n=-(this._isVerticalSide(e)?t.height:t.width);this._$arrow.css(S[e],n);var i,o=this._isVerticalSide(e)?"left":"top",r=this._isVerticalSide(e)?"width":"height",s=(0,a.default)(this._position.of),l=p.default.offset(s)||{top:0,left:0},u=p.default.offset(this._$content),c=t[r],d=u[o],h=(0,y.getBoundingRect)(this._$content.get(0))[r],f=l[o],g=s.get(0).preventDefault?0:(0,y.getBoundingRect)(s.get(0))[r],m=Math.max(d,f),v=Math.min(d+h,f+g);i="start"===this.option("arrowPosition")?m-d:"end"===this.option("arrowPosition")?v-d-c:(m+v)/2-d-c/2;var b=this._getContentBorderWidth(e),x=(0,_.fitIntoRange)(i-b+this.option("arrowOffset"),b,h-c-2*b);this._$arrow.css(o,x)},_isPopoverInside:function(){var e=this._transformStringPosition(this.option("position"),I),t=p.default.setup.normalizeAlign(e.my),n=p.default.setup.normalizeAlign(e.at);return t.h===n.h&&t.v===n.v},_setContentHeight:function(e){e&&this.callBase()},_renderWrapperPosition:function(){this.option("shading")&&this._$wrapper.css({top:0,left:0})},_renderWrapperDimensions:function(){this.option("shading")&&this._$wrapper.css({width:"100%",height:"100%"})},_normalizePosition:function(){var e=(0,h.extend)({},this._transformStringPosition(this.option("position"),I));e.of||(e.of=this.option("target")),e.collision||(e.collision="flip"),e.boundaryOffset||(e.boundaryOffset=this.option("boundaryOffset")),this._positionSide=this._getDisplaySide(e),this._position=e},_getDisplaySide:function(e){var t=p.default.setup.normalizeAlign(e.my),n=p.default.setup.normalizeAlign(e.at),i=D[t.h]===D[n.h]&&D[t.v]===D[n.v]?-1:1,o=Math.abs(D[t.h]-i*D[n.h]),a=Math.abs(D[t.v]-i*D[n.v]);return o>a?n.h:n.v},_isVerticalSide:function(e){return e=e||this._positionSide,"top"===e||"bottom"===e},_isHorizontalSide:function(e){return e=e||this._positionSide,"left"===e||"right"===e},_clearEventTimeout:function(e){clearTimeout(this._timeouts[e])},_clearEventsTimeouts:function(){this._clearEventTimeout("show"),this._clearEventTimeout("hide")},_clean:function(){this._detachEvents(this.option("target")),this.callBase.apply(this,arguments)},_optionChanged:function(e){switch(e.name){case"boundaryOffset":case"arrowPosition":case"arrowOffset":this._renderGeometry();break;case"fullScreen":e.value&&this.option("fullScreen",!1);break;case"target":e.previousValue&&this._detachEvents(e.previousValue),this.callBase(e);break;case"showEvent":case"hideEvent":var t=e.name.substring(0,4),n=O(e.previousValue);this.hide(),R(this,this.option("target"),t,n),P(this,t);break;case"visible":this._clearEventTimeout(e.value?"show":"hide"),this.callBase(e);break;default:this.callBase(e)}},show:function(e){return e&&this.option("target",e),this.callBase()}});(0,c.default)("dxPopover",M);var B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=n(0),l=n(87),u=i(n(52)),c=i(n(9)),d=n(8),h=n(21),f="dx-checkbox",p="dx-checkbox-icon",g="dx-checkbox-checked",_="dx-checkbox-container",m="dx-checkbox-text",v="dx-checkbox-has-text",y="dx-checkbox-indeterminate",b=100,x=u.default.inherit({_supportedKeys:function(){var e=function(e){e.preventDefault(),this._clickAction({event:e})};return(0,s.extend)(this.callBase(),{space:e})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{hoverStateEnabled:!0,activeStateEnabled:!0,value:!1,text:"",useInkRipple:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}}])},_canValueBeChangedByClick:function(){return!0},_feedbackHideTimeout:b,_initMarkup:function(){this._renderSubmitElement(),this._$container=(0,o.default)("<div>").addClass(_),this.setAria("role","checkbox"),this.$element().addClass(f),this._renderValue(),this._renderIcon(),this._renderText(),this.option("useInkRipple")&&this._renderInkRipple(),this.$element().append(this._$container),this.callBase()},_render:function(){this._renderClick(),this.callBase()},_renderSubmitElement:function(){this._$submitElement=(0,o.default)("<input>").attr("type","hidden").appendTo(this.$element())},_getSubmitElement:function(){return this._$submitElement},_renderInkRipple:function(){this._inkRipple=(0,l.render)({waveSizeCoefficient:2.5,useHoldAnimation:!1,wavesNumber:2,isCentered:!0})},_renderInkWave:function(e,t,n,i){if(this._inkRipple){var o={element:e,event:t,wave:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}},_updateFocusState:function(e,t){this.callBase.apply(this,arguments),this._renderInkWave(this._$icon,e,t,0)},_toggleActiveState:function(e,t,n){this.callBase.apply(this,arguments),this._renderInkWave(this._$icon,n,t,1)},_renderIcon:function(){this._$icon=(0,o.default)("<span>").addClass(p).prependTo(this._$container)},_renderText:function(){var e=this.option("text");return e?(this._$text||(this._$text=(0,o.default)("<span>").addClass(m)),this._$text.text(e),this._$container.append(this._$text),void this.$element().addClass(v)):void(this._$text&&(this._$text.remove(),this.$element().removeClass(v)))},_renderClick:function(){var e=this,t=(0,d.addNamespace)(h.name,e.NAME);e._clickAction=e._createAction(e._clickHandler),a.default.off(e.$element(),t),a.default.on(e.$element(),t,function(t){e._clickAction({event:t})})},_clickHandler:function(e){var t=e.component;t._saveValueChangeEvent(e.event),t.option("value",!t.option("value"))},_renderValue:function(){var e=this.$element(),t=this.option("value"),n=void 0===t;e.toggleClass(g,Boolean(t)),e.toggleClass(y,n),this._getSubmitElement().val(t),this.setAria("checked",n?"mixed":t||"false")},_optionChanged:function(e){switch(e.name){case"useInkRipple":this._invalidate();break;case"value":this._renderValue(),this.callBase(e);break;case"text":this._renderText(),this._renderDimensions();break;default:this.callBase(e)}},_clean:function(){delete this._inkRipple,this.callBase()}});(0,c.default)("dxCheckBox",x);var w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(2)),r=i(n(62)),s=n(1),l=n(3),u=i(n(32)),c=function(){return["year","day","month","day"]},d=6e4,h=60*d*24,f=365*h,p=function(e){var t=o(e);return"string"===t?"format":"object"===t&&void 0!==e.type?e.type:null},g={SUPPORTED_FORMATS:["date","time","datetime"],DATE_COMPONENT_TEXT_FORMATTER:function(e,t){var n=(0,a.default)("<div>").addClass("dx-dateview-formatter-container");return(0,a.default)("<span>").text(e).addClass("dx-dateview-value-formatter").appendTo(n),(0,a.default)("<span>").text(t).addClass("dx-dateview-name-formatter").appendTo(n),n},ONE_MINUTE:d,ONE_DAY:h,ONE_YEAR:f,MIN_DATEVIEW_DEFAULT_DATE:new Date(1900,0,1),MAX_DATEVIEW_DEFAULT_DATE:function(){var e=new Date;return new Date(e.getFullYear()+50,e.getMonth(),e.getDate(),23,59,59)}(),FORMATS_INFO:{date:{getStandardPattern:function(){return"yyyy-MM-dd"},components:c()},time:{getStandardPattern:function(){return"HH:mm"},components:["hours","minutes","seconds","milliseconds"]},datetime:{getStandardPattern:function(){var e;return function(){var t="yyyy-MM-ddTHH:mmZ",n="2000-01-01T01:01Z",i=(0,a.default)("<input>").attr("type","datetime");i.val(n),i.val()&&(e=t)}(),e||(e="yyyy-MM-ddTHH:mm:ssZ"),g.FORMATS_INFO.datetime.getStandardPattern=function(){return e},e},components:c().concat(["hours","minutes","seconds","milliseconds"])},"datetime-local":{getStandardPattern:function(){return"yyyy-MM-ddTHH:mm:ss"},components:c().concat(["hours","minutes","seconds"])}},FORMATS_MAP:{date:"shortdate",time:"shorttime",datetime:"shortdateshorttime"},SUBMIT_FORMATS_MAP:{date:"date",time:"time",datetime:"datetime-local"},toStandardDateFormat:function(e,t){var n=g.FORMATS_INFO[t].getStandardPattern();return r.default.serializeDate(e,n)},fromStandardDateFormat:function(e){var t=r.default.dateParser(e);return(0,s.isDate)(t)?t:void 0},getMaxMonthDay:function(e,t){return new Date(e,t+1,0).getDate()},mergeDates:function(e,t,n){if(!t)return t||null;if(!e||isNaN(e.getTime())){var i=new Date(null);e=new Date(i.getFullYear(),i.getMonth(),i.getDate())}var o=new Date(e.valueOf()),a=g.FORMATS_INFO[n];return(0,l.each)(a.components,function(){var e=g.DATE_COMPONENTS_INFO[this];o[e.setter](t[e.getter]())}),o},getLongestCaptionIndex:function(e){var t,n=0,i=0;for(t=0;t<e.length;++t)e[t].length>i&&(n=t,i=e[t].length);return n},formatUsesMonthName:function(e){return u.default.formatUsesMonthName(e)},formatUsesDayName:function(e){return u.default.formatUsesDayName(e)},getLongestDate:function(e,t,n){var i=p(e),o=9;i&&!g.formatUsesMonthName(i)||(o=g.getLongestCaptionIndex(t));var a=new Date(1888,o,21,23,59,59,999);if(!i||g.formatUsesDayName(i)){var r=a.getDate()-a.getDay()+g.getLongestCaptionIndex(n);a.setDate(r)}return a},normalizeTime:function(e){e.setSeconds(0),e.setMilliseconds(0)}};g.DATE_COMPONENTS_INFO={year:{getter:"getFullYear",setter:"setFullYear",formatter:function(e,t){var n=new Date(t.getTime());return n.setFullYear(e),u.default.format(n,"yyyy")},startValue:void 0,endValue:void 0},day:{getter:"getDate",setter:"setDate",formatter:function(e,t){var n=new Date(t.getTime());return n.setDate(e),u.default.format(n,"d")},startValue:1,endValue:void 0},month:{getter:"getMonth",setter:"setMonth",formatter:function(e){return u.default.getMonthNames()[e]},startValue:0,endValue:11},hours:{getter:"getHours",setter:"setHours",formatter:function(e){return u.default.format(new Date(0,0,0,e),"hour")},startValue:0,endValue:23},minutes:{getter:"getMinutes",setter:"setMinutes",formatter:function(e){return u.default.format(new Date(0,0,0,0,e),"minute")},startValue:0,endValue:59},seconds:{getter:"getSeconds",setter:"setSeconds",formatter:function(e){return u.default.format(new Date(0,0,0,0,0,e),"second")},startValue:0,endValue:59},milliseconds:{getter:"getMilliseconds",setter:"setMilliseconds",formatter:function(e){return u.default.format(new Date(0,0,0,0,0,0,e),"millisecond")},startValue:0,endValue:999}};var _=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(638)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return u(e)||l(e,t)||r(e,t)||a()}function a(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function l(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function u(e){if(Array.isArray(e))return e}function c(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function d(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function h(e,t,n){return t&&d(e.prototype,t),n&&d(e,n),e}function f(){return S||(S=new D),S}t.getRecurrenceProcessor=f;var p=i(n(22)),g=n(3),_=n(12),m=n(224),v=i(n(20)),y=i(n(86)),b=v.default.dateToMilliseconds,x=["freq","interval","byday","byweekno","byyearday","bymonth","bymonthday","count","until","byhour","byminute","bysecond","bysetpos","wkst"],w=["DAILY","WEEKLY","MONTHLY","YEARLY","SECONDLY","MINUTELY","HOURLY"],C={SU:0,MO:1,TU:2,WE:3,TH:4,FR:5,SA:6},k=[],S=null,D=function(){function e(){c(this,e),this.rRule=null,this.rRuleSet=null,this.validator=new I}return h(e,[{key:"generateDates",value:function(e){var t=[],n=this.evalRecurrenceRule(e.rule),i=n.rule;if(!n.isValid||!i.freq)return t;var o=y.default.createUTCDateWithLocalOffset(e.start),a=y.default.createUTCDateWithLocalOffset(e.end),r=y.default.createUTCDateWithLocalOffset(e.min),s=y.default.createUTCDateWithLocalOffset(e.max),l=a?a.getTime()-o.getTime():0;this._initializeRRule(e,o);var u=r.getTime(),c=this._getLeftBorder(e,r,l);return this.rRuleSet.between(c,s,!0).forEach(function(e){var n=e.getTime()+l;if(n>=u){var i=y.default.createDateFromUTCWithLocalOffset(e);t.push(i)}}),t}},{key:"hasRecurrence",value:function(e){return!!this.generateDates(e).length}},{key:"evalRecurrenceRule",value:function(e){var t={rule:{},isValid:!1};return e&&(t.rule=this._parseRecurrenceRule(e),t.isValid=this.validator.validateRRule(t.rule,e)),t}},{key:"isValidRecurrenceRule",value:function(e){return this.evalRecurrenceRule(e).isValid}},{key:"daysFromByDayRule",value:function(e){var t=[];return e.byday&&(t=Array.isArray(e.byday)?e.byday:e.byday.split(",")),t}},{key:"getAsciiStringByDate",value:function(e){var t=this._getTimeZoneOffset()*b("minute"),n=new Date(e.getTime()+t);return n.getFullYear()+("0"+(n.getMonth()+1)).slice(-2)+("0"+n.getDate()).slice(-2)+"T"+("0"+n.getHours()).slice(-2)+("0"+n.getMinutes()).slice(-2)+("0"+n.getSeconds()).slice(-2)+"Z"}},{key:"getRecurrenceString",value:function(e){if(e&&e.freq){var t="";for(var n in e){var i=e[n];"interval"===n&&i<2||("until"===n&&(i=this.getAsciiStringByDate(i)),t+=n+"="+i+";")}return t=t.substring(0,t.length-1),t.toUpperCase()}}},{key:"_parseExceptionToRawArray",value:function(e){return e.match(/(\d{4})(\d{2})(\d{2})(T(\d{2})(\d{2})(\d{2}))?(Z)?/)}},{key:"getDateByAsciiString",value:function(e){if("string"!=typeof e)return e;var t=this._parseExceptionToRawArray(e);if(!t)return null;var n=this._createDateTuple(t),i=o(n,7),a=i[0],r=i[1],s=i[2],l=i[3],u=i[4],c=i[5],d=i[6];return d?new Date(Date.UTC(a,r,s,l,u,c)):new Date(a,r,s,l,u,c)}},{key:"_dispose",value:function(){this.rRuleSet&&(delete this.rRuleSet,this.rRuleSet=null),this.rRule&&(delete this.rRule,this.rRule=null)}},{key:"_getTimeZoneOffset",value:function(){return(new Date).getTimezoneOffset()}},{key:"_initializeRRule",value:function(e,t){var n=this,i=m.RRule.parseString(e.rule),o=e.firstDayOfWeek;if(i.dtstart=t,!i.wkst&&o){var a=[6,0,1,2,3,4,5];i.wkst=a[o]}if(this._createRRule(i),e.exception){var r=e.exception,s=r.split(",").map(function(e){return n.getDateByAsciiString(e)});s.forEach(function(t){e.getPostProcessedException&&(t=e.getPostProcessedException(t));var i=y.default.createUTCDateWithLocalOffset(t);n.rRuleSet.exdate(i)})}}},{key:"_createRRule",value:function(e){this._dispose();var t=new m.RRuleSet;this.rRuleSet=t,this.rRule=new m.RRule(e),this.rRuleSet.rrule(this.rRule)}},{key:"_getLeftBorder",value:function(e,t,n){return e.end&&!y.default.isSameAppointmentDates(e.start,e.end)?new Date(t.getTime()-n):t}},{key:"_parseRecurrenceRule",value:function(e){for(var t={},n=e.split(";"),i=0,o=n.length;i<o;i++){var a=n[i].split("="),r=a[0].toLowerCase(),s=a[1];t[r]=s}var l=parseInt(t.count);if(isNaN(l)||(t.count=l),t.interval){var u=parseInt(t.interval);isNaN(u)||(t.interval=u)}else t.interval=1;return t.freq&&t.until&&(t.until=this.getDateByAsciiString(t.until)),t}},{key:"_createDateTuple",value:function(e){var t=void 0!==e[8];return e.shift(),void 0===e[3]?e.splice(3):(e.splice(3,1),e.splice(6)),e[1]--,e.unshift(null),[parseInt(e[1]),parseInt(e[2]),parseInt(e[3]),parseInt(e[4])||0,parseInt(e[5])||0,parseInt(e[6])||0,t]}}]),e}(),I=function(){function e(){c(this,e)}return h(e,[{key:"validateRRule",value:function(e,t){return!(this._brokenRuleNameExists(e)||(0,_.inArray)(e.freq,w)===-1||this._wrongCountRule(e)||this._wrongIntervalRule(e)||this._wrongDayOfWeek(e)||this._wrongByMonthDayRule(e)||this._wrongByMonth(e)||this._wrongUntilRule(e))||(this._logBrokenRule(t),!1)}},{key:"_wrongUntilRule",value:function(e){var t=!1,n=e.until;return void 0===n||n instanceof Date||(t=!0),t}},{key:"_wrongCountRule",value:function(e){var t=!1,n=e.count;return n&&"string"==typeof n&&(t=!0),t}},{key:"_wrongByMonthDayRule",value:function(e){var t=!1,n=e.bymonthday;return n&&isNaN(parseInt(n))&&(t=!0),t}},{key:"_wrongByMonth",value:function(e){var t=!1,n=e.bymonth;return n&&isNaN(parseInt(n))&&(t=!0),t}},{key:"_wrongIntervalRule",value:function(e){var t=!1,n=e.interval;return n&&"string"==typeof n&&(t=!0),t}},{key:"_wrongDayOfWeek",value:function(e){var t=e.byday,n=f().daysFromByDayRule(e),i=!1;return""===t&&(i=!0),(0,g.each)(n,function(e,t){if(!Object.prototype.hasOwnProperty.call(C,t))return i=!0,!1}),i}},{key:"_brokenRuleNameExists",value:function(e){var t=!1;return(0,g.each)(e,function(e){if((0,_.inArray)(e,x)===-1)return t=!0,!1}),t}},{key:"_logBrokenRule",value:function(e){(0,_.inArray)(e,k)===-1&&(p.default.log("W0006",e),k.push(e))}}]),e}()},function(e,t,n){function i(e){return e<o.HOURLY}n.d(t,"a",function(){return o}),t.b=i;var o;!function(e){e[e.YEARLY=0]="YEARLY",e[e.MONTHLY=1]="MONTHLY",e[e.WEEKLY=2]="WEEKLY",e[e.DAILY=3]="DAILY",e[e.HOURLY=4]="HOURLY",e[e.MINUTELY=5]="MINUTELY",e[e.SECONDLY=6]="SECONDLY"}(o||(o={}))},function(e,t,n){function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.getMapFromObject=t.findItemsByKeys=t.extendAttributes=t.getDisplayFileSize=t.whenSome=void 0;var o=n(6),a=n(0),r=n(4),s=n(1),l=function(e,t,n){t=t||r.noop,n=n||r.noop,Array.isArray(e)||(e=[e]);var i=e.map(function(e,i){return(0,o.when)(e).then(function(n){return(0,s.isFunction)(t)&&t({item:e,index:i,result:n}),n},function(e){return e||(e={}),e.index=i,(0,s.isFunction)(n)&&n(e),(new o.Deferred).resolve().promise()})});return o.when.apply(null,i)};t.whenSome=l;var u=function(e){for(var t=["B","KB","MB","GB","TB"],n=0,i=e;i>=1024&&n<=t.length-1;)i/=1024,n++;return i=Math.round(10*i)/10,"".concat(i," ").concat(t[n])};t.getDisplayFileSize=u;var c=function(e,t,n){return n.forEach(function(n){(0,a.extend)(!0,e,(0,s.isDefined)(t[n])?i({},n,t[n]):{})}),e};t.extendAttributes=c;var d=function(e,t){var n={};t.forEach(function(e){n[e]=null}),e.forEach(function(e){var t=e.fileItem.key;Object.prototype.hasOwnProperty.call(n,t)&&(n[t]=e)});var i=[];return t.forEach(function(e){var t=n[e];t&&i.push(t)}),i};t.findItemsByKeys=d;var h=function(e){var t=Object.keys(e),n=[];return t.forEach(function(t){return n.push(e[t])}),{keys:t,values:n}};t.getMapFromObject=h},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(105),a=n(177),r=n(1),s=n(7),l=n(0),u=i(n(10)),c="separator",d={widget:c},h={SMALL_EDITOR_ITEM:"dx-diagram-sm-edit-item",MEDIUM_EDITOR_ITEM:"dx-diagram-md-edit-item",LARGE_EDITOR_ITEM:"dx-diagram-lg-edit-item",IMAGE_DROPDOWN_ITEM:"dx-diagram-image-dropdown-item",COLOR_EDITOR_ITEM:"dx-diagram-color-edit-item",LARGE_ICON_ITEM:"dx-diagram-lg-icon-item"},f={SHOW_TOOLBOX_COMMAND_NAME:"toolbox",SHOW_PROPERTIES_PANEL_COMMAND_NAME:"propertiesPanel",getAllCommands:function(){var e=this,t=(0,o.getDiagram)(),n=t.DiagramCommand;return this._allCommands||(this._allCommands={separator:d,exportSvg:{command:n.ExportSvg,text:u.default.format("dxDiagram-commandExportToSvg"),getParameter:function(t){return function(n){return e._exportTo(t,n,"SVG","image/svg+xml")}}},exportPng:{command:n.ExportPng,text:u.default.format("dxDiagram-commandExportToPng"),getParameter:function(t){return function(n){return e._exportTo(t,n,"PNG","image/png")}}},exportJpg:{command:n.ExportJpg,text:u.default.format("dxDiagram-commandExportToJpg"),getParameter:function(t){return function(n){return e._exportTo(t,n,"JPEG","image/jpeg")}}},undo:{command:n.Undo,hint:u.default.format("dxDiagram-commandUndo"),text:u.default.format("dxDiagram-commandUndo"),icon:"undo",menuIcon:"undo"},redo:{command:n.Redo,hint:u.default.format("dxDiagram-commandRedo"),text:u.default.format("dxDiagram-commandRedo"),icon:"redo",menuIcon:"redo"},cut:{command:n.Cut,hint:u.default.format("dxDiagram-commandCut"),text:u.default.format("dxDiagram-commandCut"),icon:"cut",menuIcon:"cut"},copy:{command:n.Copy,hint:u.default.format("dxDiagram-commandCopy"),text:u.default.format("dxDiagram-commandCopy"),icon:"copy",menuIcon:"copy"},paste:{command:n.PasteInPosition,hint:u.default.format("dxDiagram-commandPaste"),text:u.default.format("dxDiagram-commandPaste"),icon:"paste",menuIcon:"paste"},selectAll:{command:n.SelectAll,hint:u.default.format("dxDiagram-commandSelectAll"),text:u.default.format("dxDiagram-commandSelectAll"),icon:"dx-diagram-i-button-select-all dx-diagram-i",menuIcon:"dx-diagram-i-menu-select-all dx-diagram-i"},"delete":{command:n.Delete,hint:u.default.format("dxDiagram-commandDelete"),text:u.default.format("dxDiagram-commandDelete"),icon:"remove",menuIcon:"remove"},fontName:{command:n.FontName,hint:u.default.format("dxDiagram-commandFontName"),text:u.default.format("dxDiagram-commandFontName"),widget:"dxSelectBox",items:["Arial","Arial Black","Helvetica","Times New Roman","Courier New","Courier","Verdana","Georgia","Comic Sans MS","Trebuchet MS"].map(function(e){return{text:e,value:e}}),cssClass:h.MEDIUM_EDITOR_ITEM},fontSize:{command:n.FontSize,hint:u.default.format("dxDiagram-commandFontSize"),text:u.default.format("dxDiagram-commandFontSize"),widget:"dxSelectBox",items:[8,9,10,11,12,14,16,18,20,22,24,26,28,36,48,72].map(function(e){return{text:e+"pt",value:e+"pt"}}),cssClass:h.SMALL_EDITOR_ITEM},bold:{command:n.Bold,hint:u.default.format("dxDiagram-commandBold"),text:u.default.format("dxDiagram-commandBold"),icon:"bold",menuIcon:"bold"},italic:{command:n.Italic,hint:u.default.format("dxDiagram-commandItalic"),text:u.default.format("dxDiagram-commandItalic"),icon:"italic",menuIcon:"italic"},underline:{command:n.Underline,hint:u.default.format("dxDiagram-commandUnderline"),text:u.default.format("dxDiagram-commandUnderline"),icon:"underline",menuIcon:"underline"},fontColor:{command:n.FontColor,text:u.default.format("dxDiagram-commandTextColor"),hint:u.default.format("dxDiagram-commandTextColor"),widget:"dxColorBox",icon:"dx-icon dx-icon-color",menuIcon:"dx-icon dx-icon-color",cssClass:h.COLOR_EDITOR_ITEM},lineColor:{command:n.StrokeColor,text:u.default.format("dxDiagram-commandLineColor"),hint:u.default.format("dxDiagram-commandLineColor"),widget:"dxColorBox",icon:"dx-icon dx-icon-background",menuIcon:"dx-icon dx-icon-background",cssClass:h.COLOR_EDITOR_ITEM},lineWidth:{command:n.StrokeWidth,text:u.default.format("dxDiagram-commandLineWidth"),hint:u.default.format("dxDiagram-commandLineWidth"),widget:"dxSelectBox",items:[1,2,3,4,5,6,7,8].map(function(e){return{text:e+"px",value:e.toString()}}),cssClass:h.SMALL_EDITOR_ITEM},lineStyle:{command:n.StrokeStyle,text:u.default.format("dxDiagram-commandLineStyle"),hint:u.default.format("dxDiagram-commandLineStyle"),
widget:"dxSelectBox",items:[{value:"",menuIcon:"dx-diagram-i-line-solid dx-diagram-i",hint:u.default.format("dxDiagram-commandLineStyleSolid")},{value:"2,2",menuIcon:"dx-diagram-i-line-dotted dx-diagram-i",hint:u.default.format("dxDiagram-commandLineStyleDotted")},{value:"6,2",menuIcon:"dx-diagram-i-line-dashed dx-diagram-i",hint:u.default.format("dxDiagram-commandLineStyleDashed")}],cssClass:h.IMAGE_DROPDOWN_ITEM},fillColor:{command:n.FillColor,text:u.default.format("dxDiagram-commandFillColor"),hint:u.default.format("dxDiagram-commandFillColor"),widget:"dxColorBox",icon:"dx-diagram-i dx-diagram-i-button-fill",menuIcon:"dx-diagram-i dx-diagram-i-menu-fill",cssClass:h.COLOR_EDITOR_ITEM},textAlignLeft:{command:n.TextLeftAlign,hint:u.default.format("dxDiagram-commandAlignLeft"),text:u.default.format("dxDiagram-commandAlignLeft"),icon:"alignleft",menuIcon:"alignleft"},textAlignCenter:{command:n.TextCenterAlign,hint:u.default.format("dxDiagram-commandAlignCenter"),text:u.default.format("dxDiagram-commandAlignCenter"),icon:"aligncenter",menuIcon:"aligncenter"},textAlignRight:{command:n.TextRightAlign,hint:u.default.format("dxDiagram-commandAlignRight"),text:u.default.format("dxDiagram-commandAlignRight"),icon:"alignright",menu:"alignright"},lock:{command:n.Lock,hint:u.default.format("dxDiagram-commandLock"),text:u.default.format("dxDiagram-commandLock"),icon:"dx-diagram-i-button-lock dx-diagram-i",menuIcon:"dx-diagram-i-menu-lock dx-diagram-i"},unlock:{command:n.Unlock,hint:u.default.format("dxDiagram-commandUnlock"),text:u.default.format("dxDiagram-commandUnlock"),icon:"dx-diagram-i-button-unlock dx-diagram-i",menuIcon:"dx-diagram-i-menu-unlock dx-diagram-i"},bringToFront:{command:n.BringToFront,hint:u.default.format("dxDiagram-commandBringToFront"),text:u.default.format("dxDiagram-commandBringToFront"),icon:"dx-diagram-i-button-bring-to-front dx-diagram-i",menuIcon:"dx-diagram-i-menu-bring-to-front dx-diagram-i"},sendToBack:{command:n.SendToBack,hint:u.default.format("dxDiagram-commandSendToBack"),text:u.default.format("dxDiagram-commandSendToBack"),icon:"dx-diagram-i-button-send-to-back dx-diagram-i",menuIcon:"dx-diagram-i-menu-send-to-back dx-diagram-i"},insertShapeImage:{command:n.InsertShapeImage,text:u.default.format("dxDiagram-commandInsertShapeImage"),icon:"dx-diagram-i-button-image-insert dx-diagram-i",menuIcon:"dx-diagram-i-menu-image-insert dx-diagram-i"},editShapeImage:{command:n.EditShapeImage,text:u.default.format("dxDiagram-commandEditShapeImage"),icon:"dx-diagram-i-button-image-edit dx-diagram-i",menuIcon:"dx-diagram-i-menu-image-edit dx-diagram-i"},deleteShapeImage:{command:n.DeleteShapeImage,text:u.default.format("dxDiagram-commandDeleteShapeImage"),icon:"dx-diagram-i-button-image-delete dx-diagram-i",menuIcon:"dx-diagram-i-menu-image-delete dx-diagram-i"},connectorLineType:{command:n.ConnectorLineOption,widget:"dxSelectBox",hint:u.default.format("dxDiagram-commandConnectorLineType"),items:[{value:0,menuIcon:"dx-diagram-i-connector-straight dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineStraight")},{value:1,menuIcon:"dx-diagram-i-connector-orthogonal dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineOrthogonal")}],cssClass:h.IMAGE_DROPDOWN_ITEM},connectorLineStart:{command:n.ConnectorStartLineEnding,widget:"dxSelectBox",items:[{value:0,menuIcon:"dx-diagram-i-connector-begin-none dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineNone")},{value:1,menuIcon:"dx-diagram-i-connector-begin-arrow dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")},{value:2,menuIcon:"dx-diagram-i-connector-begin-outlined-triangle dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")},{value:3,menuIcon:"dx-diagram-i-connector-begin-filled-triangle dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")}],hint:u.default.format("dxDiagram-commandConnectorLineStart"),cssClass:h.IMAGE_DROPDOWN_ITEM},connectorLineEnd:{command:n.ConnectorEndLineEnding,widget:"dxSelectBox",items:[{value:0,menuIcon:"dx-diagram-i-connector-end-none dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineNone")},{value:1,menuIcon:"dx-diagram-i-connector-end-arrow dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")},{value:2,menuIcon:"dx-diagram-i-connector-end-outlined-triangle dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")},{value:3,menuIcon:"dx-diagram-i-connector-end-filled-triangle dx-diagram-i",hint:u.default.format("dxDiagram-commandConnectorLineArrow")}],hint:u.default.format("dxDiagram-commandConnectorLineEnd"),cssClass:h.IMAGE_DROPDOWN_ITEM},layoutTreeTopToBottom:{command:n.AutoLayoutTreeVertical,text:u.default.format("dxDiagram-commandLayoutTopToBottom"),hint:u.default.format("dxDiagram-commandLayoutTopToBottom"),icon:"dx-diagram-i-button-layout-tree-tb dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutTreeBottomToTop:{command:n.AutoLayoutTreeVerticalBottomToTop,text:u.default.format("dxDiagram-commandLayoutBottomToTop"),hint:u.default.format("dxDiagram-commandLayoutBottomToTop"),icon:"dx-diagram-i-button-layout-tree-bt dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutTreeLeftToRight:{command:n.AutoLayoutTreeHorizontal,text:u.default.format("dxDiagram-commandLayoutLeftToRight"),hint:u.default.format("dxDiagram-commandLayoutLeftToRight"),icon:"dx-diagram-i-button-layout-tree-lr dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutTreeRightToLeft:{command:n.AutoLayoutTreeHorizontalRightToLeft,text:u.default.format("dxDiagram-commandLayoutRightToLeft"),hint:u.default.format("dxDiagram-commandLayoutRightToLeft"),icon:"dx-diagram-i-button-layout-tree-rl dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutLayeredTopToBottom:{command:n.AutoLayoutLayeredVertical,text:u.default.format("dxDiagram-commandLayoutTopToBottom"),hint:u.default.format("dxDiagram-commandLayoutTopToBottom"),icon:"dx-diagram-i-button-layout-layered-tb dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutLayeredBottomToTop:{command:n.AutoLayoutLayeredVerticalBottomToTop,text:u.default.format("dxDiagram-commandLayoutBottomToTop"),hint:u.default.format("dxDiagram-commandLayoutBottomToTop"),icon:"dx-diagram-i-button-layout-layered-bt dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutLayeredLeftToRight:{command:n.AutoLayoutLayeredHorizontal,text:u.default.format("dxDiagram-commandLayoutLeftToRight"),hint:u.default.format("dxDiagram-commandLayoutLeftToRight"),icon:"dx-diagram-i-button-layout-layered-lr dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},layoutLayeredRightToLeft:{command:n.AutoLayoutLayeredHorizontalRightToLeft,text:u.default.format("dxDiagram-commandLayoutRightToLeft"),hint:u.default.format("dxDiagram-commandLayoutRightToLeft"),icon:"dx-diagram-i-button-layout-layered-rl dx-diagram-i",cssClass:h.LARGE_ICON_ITEM},fullScreen:{command:n.Fullscreen,hint:u.default.format("dxDiagram-commandFullscreen"),text:u.default.format("dxDiagram-commandFullscreen"),icon:"dx-diagram-i dx-diagram-i-button-fullscreen",menuIcon:"dx-diagram-i dx-diagram-i-menu-fullscreen",cssClass:h.COLOR_EDITOR_ITEM},units:{command:n.ViewUnits,hint:u.default.format("dxDiagram-commandUnits"),text:u.default.format("dxDiagram-commandUnits"),widget:"dxSelectBox"},simpleView:{command:n.ToggleSimpleView,hint:u.default.format("dxDiagram-commandSimpleView"),text:u.default.format("dxDiagram-commandSimpleView"),widget:"dxCheckBox"},showGrid:{command:n.ShowGrid,hint:u.default.format("dxDiagram-commandShowGrid"),text:u.default.format("dxDiagram-commandShowGrid"),widget:"dxCheckBox"},snapToGrid:{command:n.SnapToGrid,hint:u.default.format("dxDiagram-commandSnapToGrid"),text:u.default.format("dxDiagram-commandSnapToGrid"),widget:"dxCheckBox"},gridSize:{command:n.GridSize,hint:u.default.format("dxDiagram-commandGridSize"),text:u.default.format("dxDiagram-commandGridSize"),widget:"dxSelectBox"},pageSize:{command:n.PageSize,hint:u.default.format("dxDiagram-commandPageSize"),text:u.default.format("dxDiagram-commandPageSize"),widget:"dxSelectBox",cssClass:h.LARGE_EDITOR_ITEM,getCommandValue:function(e){return JSON.parse(e)},getEditorValue:function(e){return JSON.stringify(e)}},pageOrientation:{command:n.PageLandscape,hint:u.default.format("dxDiagram-commandPageOrientation"),text:u.default.format("dxDiagram-commandPageOrientation"),widget:"dxSelectBox",items:[{value:!0,text:u.default.format("dxDiagram-commandPageOrientationLandscape")},{value:!1,text:u.default.format("dxDiagram-commandPageOrientationPortrait")}],cssClass:h.MEDIUM_EDITOR_ITEM},pageColor:{command:n.PageColor,hint:u.default.format("dxDiagram-commandPageColor"),text:u.default.format("dxDiagram-commandPageColor"),widget:"dxColorBox",icon:"dx-diagram-i dx-diagram-i-button-fill",menuIcon:"dx-diagram-i dx-diagram-i-menu-fill",cssClass:h.COLOR_EDITOR_ITEM},zoomLevel:{command:n.ZoomLevel,hint:u.default.format("dxDiagram-commandZoomLevel"),text:u.default.format("dxDiagram-commandZoomLevel"),widget:"dxTextBox",items:[d,{command:n.FitToScreen,hint:u.default.format("dxDiagram-commandFitToContent"),text:u.default.format("dxDiagram-commandFitToContent")},{command:n.FitToWidth,hint:u.default.format("dxDiagram-commandFitToWidth"),text:u.default.format("dxDiagram-commandFitToWidth")},d,{command:n.AutoZoomToContent,hint:u.default.format("dxDiagram-commandAutoZoomByContent"),text:u.default.format("dxDiagram-commandAutoZoomByContent")},{command:n.AutoZoomToWidth,hint:u.default.format("dxDiagram-commandAutoZoomByWidth"),text:u.default.format("dxDiagram-commandAutoZoomByWidth")}],getEditorDisplayValue:function(e){return Math.round(100*e)+"%"},cssClass:h.SMALL_EDITOR_ITEM},toolbox:{command:this.SHOW_TOOLBOX_COMMAND_NAME,iconChecked:"dx-diagram-i dx-diagram-i-button-toolbox-close",iconUnchecked:"dx-diagram-i dx-diagram-i-button-toolbox-open",hint:u.default.format("dxDiagram-uiShowToolbox"),text:u.default.format("dxDiagram-uiShowToolbox")},propertiesPanel:{command:this.SHOW_PROPERTIES_PANEL_COMMAND_NAME,iconChecked:"close",iconUnchecked:"dx-diagram-i dx-diagram-i-button-properties-panel-open",hint:u.default.format("dxDiagram-uiProperties"),text:u.default.format("dxDiagram-uiProperties")}})},getMainToolbarCommands:function(e,t){var n=this.getAllCommands(),i=e?this._getPreparedCommands(n,e):this._getDefaultMainToolbarCommands(n);return this._prepareToolbarCommands(i,t)},_getDefaultMainToolbarCommands:function(e){return this._defaultMainToolbarCommands||(this._defaultMainToolbarCommands=[e.undo,e.redo,e.separator,e.fontName,e.fontSize,e.bold,e.italic,e.underline,e.separator,e.lineWidth,e.lineStyle,e.separator,e.fontColor,e.lineColor,e.fillColor,e.separator,e.textAlignLeft,e.textAlignCenter,e.textAlignRight,e.separator,e.connectorLineType,e.connectorLineStart,e.connectorLineEnd,e.separator,{text:u.default.format("dxDiagram-uiLayout"),showText:"always",items:[{text:u.default.format("dxDiagram-uiLayoutTree"),items:[e.layoutTreeTopToBottom,e.layoutTreeBottomToTop,e.layoutTreeLeftToRight,e.layoutTreeRightToLeft]},{text:u.default.format("dxDiagram-uiLayoutLayered"),items:[e.layoutLayeredTopToBottom,e.layoutLayeredBottomToTop,e.layoutLayeredLeftToRight,e.layoutLayeredRightToLeft]}]}])},getHistoryToolbarCommands:function(e,t){var n=this.getAllCommands(),i=e?this._getPreparedCommands(n,e):this._getDefaultHistoryToolbarCommands(n);return this._prepareToolbarCommands(i,t)},_getDefaultHistoryToolbarCommands:function(e){return this._defaultHistoryToolbarCommands||(this._defaultHistoryToolbarCommands=[e.undo,e.redo,e.separator,e.toolbox])},getViewToolbarCommands:function(e,t){var n=this.getAllCommands(),i=e?this._getPreparedCommands(n,e):this._getDefaultViewToolbarCommands(n);return this._prepareToolbarCommands(i,t)},_getDefaultViewToolbarCommands:function(e){return this._defaultViewToolbarCommands||(this._defaultViewToolbarCommands=[e.zoomLevel,e.separator,e.fullScreen,e.separator,{widget:"dxButton",icon:"export",text:u.default.format("dxDiagram-uiExport"),hint:u.default.format("dxDiagram-uiExport"),items:[e.exportSvg,e.exportPng,e.exportJpg]},{icon:"preferences",hint:u.default.format("dxDiagram-uiSettings"),text:u.default.format("dxDiagram-uiSettings"),items:[e.units,e.separator,e.showGrid,e.snapToGrid,e.gridSize,e.separator,e.simpleView,e.toolbox]}])},getPropertiesToolbarCommands:function(e,t){var n=this.getAllCommands(),i=e?this._getPreparedCommands(n,e):this._getDefaultPropertiesToolbarCommands(n);return this._prepareToolbarCommands(i,t)},_getDefaultPropertiesToolbarCommands:function(e){return this._defaultPropertiesToolbarCommands||(this._defaultPropertiesToolbarCommands=[e.propertiesPanel])},_getDefaultPropertyPanelCommandGroups:function(){return this._defaultPropertyPanelCommandGroups||(this._defaultPropertyPanelCommandGroups=[{title:u.default.format("dxDiagram-uiStyle"),groups:[{title:u.default.format("dxDiagram-uiText"),commands:["fontName","fontSize","bold","italic","underline","textAlignLeft","textAlignCenter","textAlignRight","fontColor"]},{title:u.default.format("dxDiagram-uiObject"),commands:["lineStyle","lineWidth","lineColor","fillColor"]},{title:u.default.format("dxDiagram-uiConnector"),commands:["connectorLineType","connectorLineStart","connectorLineEnd"]}]},{title:u.default.format("dxDiagram-uiLayout"),groups:[{title:u.default.format("dxDiagram-uiLayoutLayered"),commands:["layoutLayeredTopToBottom","layoutLayeredBottomToTop","layoutLayeredLeftToRight","layoutLayeredRightToLeft"]},{title:u.default.format("dxDiagram-uiLayoutTree"),commands:["layoutTreeTopToBottom","layoutTreeBottomToTop","layoutTreeLeftToRight","layoutTreeRightToLeft"]}]},{title:u.default.format("dxDiagram-uiDiagram"),groups:[{title:u.default.format("dxDiagram-uiPage"),commands:["pageSize","pageOrientation","pageColor"]}]}])},_preparePropertyPanelGroups:function(e){var t=this,n=this.getAllCommands(),i=[];return e.forEach(function(e){var o=e.commands;o&&(o=t._getPreparedCommands(n,o),o=t._prepareToolbarCommands(o));var a;e.groups&&(a=[],e.groups.forEach(function(e){var i=e.commands;i&&(i=t._getPreparedCommands(n,i),i=t._prepareToolbarCommands(i)),a.push({title:e.title,commands:i})})),i.push({title:e.title,commands:o,groups:a})}),i},getPropertyPanelCommandTabs:function(e){return e=e||this._getDefaultPropertyPanelCommandGroups(),this._preparePropertyPanelGroups(e)},getContextMenuCommands:function(e){var t=this.getAllCommands(),n=e?this._getPreparedCommands(t,e):this._getDefaultContextMenuCommands(t);return this._prepareContextMenuCommands(n)},_getDefaultContextMenuCommands:function(e){return this._defaultContextMenuCommands||(this._defaultContextMenuCommands=[e.cut,e.copy,e.paste,e.delete,e.separator,e.selectAll,e.separator,e.bringToFront,e.sendToBack,e.separator,e.lock,e.unlock,e.separator,e.insertShapeImage,e.editShapeImage,e.deleteShapeImage])},_getPreparedCommands:function(e,t){var n=this;return t.map(function(t){if(e[t])return e[t];if(t.text||t.icon||t.name){var i=t.name&&e[t.name],o={command:i&&i.command,name:t.name,text:t.text||i&&i.text,hint:t.text||i&&i.hint,icon:t.icon||i&&i.icon,menuIcon:t.icon||i&&i.menuIcon,widget:i&&i.widget,cssClass:i&&i.cssClass,getParameter:i&&i.getParameter,getCommandValue:i&&i.getCommandValue,getEditorValue:i&&i.getEditorValue,getEditorDisplayValue:i&&i.getEditorDisplayValue,iconChecked:i&&i.iconChecked,iconUnchecked:i&&i.iconUnchecked};return Array.isArray(t.items)?o.items=n._getPreparedCommands(e,t.items):o.items=i&&i.items,o}}).filter(function(e){return e})},_prepareContextMenuCommands:function(e,t,n){var i=this,o=!1;return e.map(function(e){if(i._isValidCommand(e,t)){if(e!==d){var a=i._cloneCommand(e,t);return a.icon=a.menuIcon,a.beginGroup=o,a.rootCommand=a.command?void 0:n&&n.command,o=!1,a}o=!0}}).filter(function(e){return e})},_prepareToolbarCommands:function(e,t){var n=this;return e.map(function(e){if(n._isValidCommand(e,t))return n._cloneCommand(e,t)}).filter(function(e){return e}).filter(function(e,t,n){return(e.widget!==c||t!==n.length-1)&&e})},_cloneCommand:function(e,t){var n=(0,l.extend)({},e);return Array.isArray(e.items)&&(n.items=this._prepareContextMenuCommands(e.items,t,n)),n},_isValidCommand:function(e,t){return t=t||[],t.indexOf(e.command)===-1},_exportTo:function(e,t,n,i){var o=(0,s.getWindow)();if(o&&o.atob&&(0,r.isFunction)(o.Blob)){var l=this._getBlobByDataURI(o,t,i),u=e.option("export");a.fileSaver.saveAs(u.fileName||"foo",n,l,u.proxyURL)}},_getBlobByDataURI:function(e,t,n){for(var i=e.atob(t.split(",")[1]),o=new Uint8Array(i.length),a=0;a<i.length;a++)o[a]=i.charCodeAt(a);return new e.Blob([o.buffer],{type:n})}},p=f;t.default=p,e.exports=t.default},function(e,t,n){t.default=void 0;var i={events:{mouseover:"mouseover",mouseout:"mouseout",mousemove:"mousemove",touchstart:"touchstart",touchmove:"touchmove",touchend:"touchend",mousedown:"mousedown",mouseup:"mouseup",click:"click",selectSeries:"selectseries",deselectSeries:"deselectseries",selectPoint:"selectpoint",deselectPoint:"deselectpoint",showPointTooltip:"showpointtooltip",hidePointTooltip:"hidepointtooltip"},states:{hover:"hover",normal:"normal",selection:"selection",normalMark:0,hoverMark:1,selectedMark:2,applyHover:"applyHover",applySelected:"applySelected",resetItem:"resetItem"},radialLabelIndent:30,pieLabelSpacing:10,pieSeriesSpacing:4};t.default=i,e.exports=t.default},function(e,t,n){function i(e){return[-e,-e,e,-e,e,e,-e,e,-e,-e]}function o(e){var t=b(e);return[-t,0,0,-t,t,0,0,t,-t,0]}function a(e){var t=b(e),n=y(t/2),i=b(t/2);return[-t,-n,-n,-t,0,-i,n,-t,t,-n,i,0,t,n,n,t,0,i,-n,t,-t,n,-i,0]}function r(e){return[-e,-e,e,-e,0,e,-e,-e]}function s(e){return[-e,e,e,e,0,-e,-e,e]}t.default=void 0;var l=n(0),u=n(3),c=n(4),d=n(7),h=n(327),f=n(1),p=n(14),g=(0,d.getWindow)(),_=l.extend,m=Math,v=m.round,y=m.floor,b=m.ceil,x=20,w=20,C=10,k="canvas_position_default",S={deleteLabel:function(){this._label.dispose(),this._label=null},_hasGraphic:function(){return this.graphic},clearVisibility:function(){var e=this,t=e.graphic;t&&t.attr("visibility")&&t.attr({visibility:null})},isVisible:function(){return this.inVisibleArea&&this.series.isVisible()},setInvisibility:function(){var e=this,t=e.graphic;t&&"hidden"!==t.attr("visibility")&&t.attr({visibility:"hidden"}),e._errorBar&&e._errorBar.attr({visibility:"hidden"}),e._label.draw(!1)},clearMarker:function(){var e=this.graphic;e&&e.attr(this._emptySettings)},_createLabel:function(){this._label=new h.Label({renderer:this.series._renderer,labelsGroup:this.series._labelsGroup,point:this})},_updateLabelData:function(){this._label.setData(this._getLabelFormatObject())},_updateLabelOptions:function(){!this._label&&this._createLabel(),this._label.setOptions(this._options.label)},_checkImage:function(e){return(0,f.isDefined)(e)&&("string"==typeof e||(0,f.isDefined)(e.url))},_fillStyle:function(){this._styles=this._options.styles},_checkSymbol:function(e,t){var n=e.symbol,i=t.symbol,o="circle"===n&&"circle"!==i||"circle"!==n&&"circle"===i,a=this._checkImage(e.image)!==this._checkImage(t.image);return!(!o&&!a)},_populatePointShape:function(e,t){switch(e){case"square":return i(t);case"polygon":return o(t);case"triangle":case"triangleDown":return r(t);case"triangleUp":return s(t);case"cross":return a(t)}},hasCoords:function(){return null!==this.x&&null!==this.y},correctValue:function(e){var t=this,n=t.series.getValueAxis();t.hasValue()&&(t.value=t.properValue=n.validateUnit(t.initialValue.valueOf()+e.valueOf()),t.minValue=n.validateUnit(e))},resetCorrection:function(){this.value=this.properValue=this.initialValue,this.minValue=k},resetValue:function(){var e=this;e.hasValue()&&(e.value=e.properValue=e.initialValue=0,e.minValue=0,e._label.setDataField("value",e.value))},_getTranslates:function(e){var t=this.x,n=this.y;return e&&(this._options.rotated?t=this.defaultX:n=this.defaultY),{x:t,y:n}},_createImageMarker:function(e,t,n){var i=n.width||x,o=n.height||w;return e.image(-v(.5*i),-v(.5*o),i,o,n.url?n.url.toString():n.toString(),"center").attr({translateX:t.translateX,translateY:t.translateY,visibility:t.visibility})},_createSymbolMarker:function(e,t){var n,i=this._options.symbol;return"circle"===i?(delete t.points,n=e.circle().attr(t)):"square"!==i&&"polygon"!==i&&"triangle"!==i&&"triangleDown"!==i&&"triangleUp"!==i&&"cross"!==i||(n=e.path([],"area").attr(t).sharp()),n},_createMarker:function(e,t,n,i){var o=this,a=o._checkImage(n)?o._createImageMarker(e,i,n):o._createSymbolMarker(e,i);return a&&a.data({"chart-data-point":o}).append(t),a},_getSymbolBBox:function(e,t,n){return{x:e-n,y:t-n,width:2*n,height:2*n}},_getImageBBox:function(e,t){var n=this._options.image,i=n.width||x,o=n.height||w;return{x:e-v(i/2),y:t-v(o/2),width:i,height:o}},_getGraphicBBox:function(){var e,t=this,n=t._options,i=t.x,o=t.y;return e=n.visible?t._checkImage(n.image)?t._getImageBBox(i,o):t._getSymbolBBox(i,o,n.styles.normal.r):{x:i,y:o,width:0,height:0}},hideInsideLabel:c.noop,_getShiftLabelCoords:function(e){var t=this._addLabelAlignmentAndOffset(e,this._getLabelCoords(e));return this._checkLabelPosition(e,t)},_drawLabel:function(){var e=this,t=e._getCustomLabelVisibility(),n=e._label,i=e._showForZeroValues()&&e.hasValue()&&t!==!1&&(e.series.getLabelVisibility()||t);n.draw(!!i)},correctLabelPosition:function(e){var t=this,n=t._getShiftLabelCoords(e);t.hideInsideLabel(e,n)||(e.setFigureToDrawConnector(t._getLabelConnector(e.pointPosition)),e.shift(v(n.x),v(n.y)))},_showForZeroValues:function(){return!0},_getLabelConnector:function(e){var t=this._getGraphicBBox(e),n=t.width/2,i=t.height/2;return{x:t.x+n,y:t.y+i,r:this._options.visible?Math.max(n,i):0}},_getPositionFromLocation:function(){return{x:this.x,y:this.y}},_isPointInVisibleArea:function(e,t){return e.minX<=t.x+t.width&&e.maxX>=t.x&&e.minY<=t.y+t.height&&e.maxY>=t.y},_checkLabelPosition:function(e,t){var n=this,i=n._getVisibleArea(),o=e.getBoundingRect(),a=n._getGraphicBBox(e.pointPosition),r=n._getGraphicBBox(),s="inside"===e.getLayoutOptions().position,l=C;return n._isPointInVisibleArea(i,r)&&(n._options.rotated?(i.minX>t.x&&(t.x=s?i.minX:a.x+a.width+l),i.maxX<t.x+o.width&&(t.x=s?i.maxX-o.width:a.x-l-o.width),i.minY>t.y&&(t.y=i.minY),i.maxY<t.y+o.height&&(t.y=i.maxY-o.height)):(i.minX>t.x&&(t.x=i.minX),i.maxX<t.x+o.width&&(t.x=i.maxX-o.width),i.minY>t.y&&(t.y=s?i.minY:a.y+a.height+l),i.maxY<t.y+o.height&&(t.y=s?i.maxY-o.height:a.y-o.height-l))),t},_addLabelAlignmentAndOffset:function(e,t){var n=e.getBoundingRect(),i=e.getLayoutOptions();return this._options.rotated||("left"===i.alignment?t.x+=n.width/2:"right"===i.alignment&&(t.x-=n.width/2)),t.x+=i.horizontalOffset,t.y+=i.verticalOffset,t},_getLabelCoords:function(e){return this._getLabelCoordOfPosition(e,this._getLabelPosition(e.pointPosition))},_getLabelCoordOfPosition:function(e,t){var n=this,i=e.getBoundingRect(),o=n._getGraphicBBox(e.pointPosition),a=C,r=o.height/2-i.height/2,s=o.width/2-i.width/2,l=o.x,u=o.y;switch(t){case"left":l-=i.width+a,u+=r;break;case"right":l+=o.width+a,u+=r;break;case"top":l+=s,u-=i.height+a;break;case"bottom":l+=s,u+=o.height+a;break;case"inside":l+=s,u+=r}return{x:l,y:u}},_drawMarker:function(e,t,n){var i=this,o=i._options,a=i._getTranslates(n),r=i._getStyle();i.graphic=i._createMarker(e,t,o.image,_({translateX:a.x,translateY:a.y,points:i._populatePointShape(o.symbol,r.r)},r))},_getErrorBarSettings:function(){return{visibility:"visible"}},_getErrorBarBaseEdgeLength:function(){return 2*this.getPointRadius()},_drawErrorBar:function(e,t){if(this._options.errorBars){var n,i=this,o=i._options,a=o.errorBars,r=[],s=i._errorBarPos,l=i._highErrorCoord,c=i._lowErrorCoord,d=(0,p.normalizeEnum)(a.displayMode),h="high"===d,g="low"===d,_=(h||!(0,f.isDefined)(c))&&(0,f.isDefined)(l)&&!g,m=(g||!(0,f.isDefined)(l))&&(0,f.isDefined)(c)&&!h,v=a.edgeLength;v<=1&&v>0&&(v=this._getErrorBarBaseEdgeLength()*a.edgeLength),v=y(parseInt(v)/2),_&&(c=i._baseErrorBarPos),m&&(l=i._baseErrorBarPos),"none"!==d&&(0,f.isDefined)(l)&&(0,f.isDefined)(c)&&(0,f.isDefined)(s)?(!m&&r.push([s-v,l,s+v,l]),r.push([s,l,s,c]),!_&&r.push([s+v,c,s-v,c]),o.rotated&&(0,u.each)(r,function(e,t){t.reverse()}),n=i._getErrorBarSettings(a),i._errorBar?(n.points=r,i._errorBar.attr(n)):i._errorBar=e.path(r,"line").attr(n).append(t)):i._errorBar&&i._errorBar.attr({visibility:"hidden"})}},getTooltipParams:function(){var e=this,t=e.graphic;return{x:e.x,y:e.y,offset:t?t.getBBox().height/2:0}},setPercentValue:function(e,t,n,i){var o=this,a=o.value/e||0,r=o.minValue/e||0,s=a-r;o._label.setDataField("percent",s),o._label.setDataField("total",t),o.series.isFullStackedSeries()&&o.hasValue()&&(o.leftHole&&(o.leftHole/=e-n,o.minLeftHole/=e-n),o.rightHole&&(o.rightHole/=e-i,o.minRightHole/=e-i),o.value=o.properValue=a,o.minValue=r?r:o.minValue)},_storeTrackerR:function(){var e=this,t=g.navigator,n=e._options.styles.normal.r,i=(0,d.hasProperty)("ontouchstart")||t.msPointerEnabled&&t.msMaxTouchPoints||t.pointerEnabled&&t.maxTouchPoints?20:6;return e._options.trackerR=n<i?i:n,e._options.trackerR},_translateErrorBars:function(){var e=this,t=e._options,n=t.rotated,i=t.errorBars,o=e._getValTranslator();i&&((0,f.isDefined)(e.lowError)&&(e._lowErrorCoord=o.translate(e.lowError)),(0,f.isDefined)(e.highError)&&(e._highErrorCoord=o.translate(e.highError)),e._errorBarPos=y(n?e.vy:e.vx),e._baseErrorBarPos="stdDeviation"===i.type?e._lowErrorCoord+(e._highErrorCoord-e._lowErrorCoord)/2:n?e.vx:e.vy)},_translate:function(){var e=this,t=e._getValTranslator(),n=e._getArgTranslator();e._options.rotated?(e.vx=e.x=t.translate(e.value),e.vy=e.y=n.translate(e.argument),e.minX=t.translate(e.minValue),e.defaultX=t.translate(k)):(e.vy=e.y=t.translate(e.value),e.vx=e.x=n.translate(e.argument),e.minY=t.translate(e.minValue),e.defaultY=t.translate(k)),e._translateErrorBars(),e._calculateVisibility(e.x,e.y)},_updateData:function(e){var t=this;t.value=t.properValue=t.initialValue=t.originalValue=e.value,t.minValue=t.initialMinValue=t.originalMinValue=(0,f.isDefined)(e.minValue)?e.minValue:k},_getImageSettings:function(e){return{href:e.url||e.toString(),width:e.width||x,height:e.height||w}},getCrosshairData:function(){var e=this,t=e._options.rotated,n=e.properValue,i=e.argument;return{x:e.vx,y:e.vy,xValue:t?n:i,yValue:t?i:n,axis:e.series.axis}},getPointRadius:function(){var e,t=this._getStyle(),n=this._options,i=t.r,o=n.symbol,a="square"===o,r="triangle"===o||"triangleDown"===o||"triangleUp"===o;return n.visible&&!n.image&&i?(e=t["stroke-width"]/2,(a||r?1.4*i:i)+e):0},_updateMarker:function(e,t){var n,i=this,o=i._options,a=o.image,r=i.isVisible()?{}:{visibility:"hidden"};n=i._checkImage(a)?_({},{visibility:t.visibility},r,i._getImageSettings(a)):_({},t,r,{points:i._populatePointShape(o.symbol,t.r)}),e||(n.translateX=i.x,n.translateY=i.y),i.graphic.attr(n).sharp()},_getLabelFormatObject:function(){var e=this;return{argument:e.initialArgument,value:e.initialValue,originalArgument:e.originalArgument,originalValue:e.originalValue,seriesName:e.series.name,lowErrorValue:e.lowError,highErrorValue:e.highError,point:e}},_getLabelPosition:function(){var e=this._options.rotated;return this.initialValue>0?e?"right":"top":e?"left":"bottom"},_getFormatObject:function(e){var t=this,n=t._label.getData();return _({},n,{argumentText:e.formatValue(t.initialArgument,"argument"),valueText:e.formatValue(t.initialValue)},(0,f.isDefined)(n.percent)?{percentText:e.formatValue(n.percent,"percent")}:{},(0,f.isDefined)(n.total)?{totalText:e.formatValue(n.total)}:{})},getMarkerVisibility:function(){return this._options.visible},coordsIn:function(e,t){var n=this._storeTrackerR();return e>=this.x-n&&e<=this.x+n&&t>=this.y-n&&t<=this.y+n},getMinValue:function(e){var t=this._options.errorBars;if(t&&!e){var n=t.displayMode,i="high"!==n&&(0,f.isDefined)(this.lowError)?this.lowError:this.value,o="low"!==n&&(0,f.isDefined)(this.highError)?this.highError:this.value;return i<o?i:o}return this.value},getMaxValue:function(e){var t=this._options.errorBars;if(t&&!e){var n=t.displayMode,i="high"!==n&&(0,f.isDefined)(this.lowError)?this.lowError:this.value,o="low"!==n&&(0,f.isDefined)(this.highError)?this.highError:this.value;return i>o?i:o}return this.value}};t.default=S,e.exports=t.default},function(e,t,n){t.plugin=t.LoadingIndicator=void 0;var i=n(14),o=0,a=1,r="linear",s=400,l="loadingIndicatorReady",u=function(e){var t=this,n=e.renderer;t._group=n.g().attr({"class":"dx-loading-indicator"}).linkOn(n.root,{name:"loading-indicator",after:"peripheral"}),t._rect=n.rect().attr({opacity:0}).append(t._group),t._text=n.text().attr({align:"center"}).append(t._group),t._createStates(e.eventTrigger,t._group,n.root,e.notify)};t.LoadingIndicator=u,u.prototype={constructor:u,_createStates:function(e,t,n,i){var a=this;a._states=[{opacity:0,start:function(){i(!1)},complete:function(){t.linkRemove(),n.css({"pointer-events":""}),e(l)}},{opacity:.85,start:function(){t.linkAppend(),n.css({"pointer-events":"none"}),i(!0)},complete:function(){e(l)}}],a._state=o},setSize:function(e){var t=e.width,n=e.height;this._rect.attr({width:t,height:n}),this._text.attr({x:t/2,y:n/2})},setOptions:function(e){this._rect.attr({fill:e.backgroundColor}),this._text.css((0,i.patchFontOptions)(e.font)).attr({text:e.text,"class":e.cssClass}),this[e.show?"show":"hide"]()},dispose:function(){var e=this;e._group.linkRemove().linkOff(),e._group=e._rect=e._text=e._states=null},_transit:function(e){var t,n=this;n._state!==e&&(n._state=e,n._isHiding=!1,t=n._states[e],n._rect.stopAnimation().animate({opacity:t.opacity},{complete:t.complete,easing:r,duration:s,unstoppable:!0}),n._noHiding=!0,t.start(),n._noHiding=!1)},show:function(){this._transit(a)},hide:function(){this._transit(o)},scheduleHiding:function(){this._noHiding||(this._isHiding=!0)},fulfillHiding:function(){this._isHiding&&this.hide()}};var c={name:"loading_indicator",init:function(){function e(e){t._skipLoadingIndicatorOptions=!0,t.option("loadingIndicator",{show:e}),t._skipLoadingIndicatorOptions=!1,e&&t._stopCurrentHandling()}var t=this;t._loadingIndicator=new u({eventTrigger:t._eventTrigger,renderer:t._renderer,notify:e}),t._scheduleLoadingIndicatorHiding()},dispose:function(){this._loadingIndicator.dispose(),this._loadingIndicator=null},members:{_scheduleLoadingIndicatorHiding:function(){this._loadingIndicator.scheduleHiding()},_fulfillLoadingIndicatorHiding:function(){this._loadingIndicator.fulfillHiding()},showLoadingIndicator:function(){this._loadingIndicator.show()},hideLoadingIndicator:function(){this._loadingIndicator.hide()},_onBeginUpdate:function(){this._optionChangedLocker||this._scheduleLoadingIndicatorHiding()}},extenders:{_dataSourceLoadingChangedHandler:function(e){e&&(this._options.silent("loadingIndicator")||{}).enabled&&this._loadingIndicator.show()},_setContentSize:function(){this._loadingIndicator.setSize(this._canvas)},endUpdate:function(){this._initialized&&this._dataIsReady()&&this._fulfillLoadingIndicatorHiding()}},customize:function(e){var t=e.prototype;if(t._dataSourceChangedHandler){var n=t._dataSourceChangedHandler;t._dataSourceChangedHandler=function(){this._scheduleLoadingIndicatorHiding(),n.apply(this,arguments)}}e.addChange({code:"LOADING_INDICATOR",handler:function(){this._skipLoadingIndicatorOptions||this._loadingIndicator.setOptions(this._getOption("loadingIndicator")),this._scheduleLoadingIndicatorHiding()},isThemeDependent:!0,option:"loadingIndicator",isOptionChange:!0}),t._eventsMap.onLoadingIndicatorReady={name:"loadingIndicatorReady"};var i=t._drawn;t._drawn=function(){i.apply(this,arguments),this._dataIsReady()&&this._fulfillLoadingIndicatorHiding()}},fontFields:["loadingIndicator.font"]};t.plugin=c},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");
e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&r(e,t)}function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function s(e){var t=c();return function(){var n,i=d(e);if(t){var o=d(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return l(this,n)}}function l(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?u(e):t}function u(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function c(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.ChildDefaultTemplate=void 0;var h=n(67),f=function(e){function t(e){var i;return o(this,t),i=n.call(this),i.name=e,i}a(t,e);var n=s(t);return t}(h.TemplateBase);t.ChildDefaultTemplate=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(183)),a=new o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=n(1),r=n(3),s=n(19),l=n(6),u=i(n(34)),c=i(n(41)),d=o.default.inherit({toArray:function(){var e=[];for(this.reset();this.next();)e.push(this.current());return e},countable:function(){return!1}}),h=d.inherit({ctor:function(e){this.array=e,this.index=-1},next:function(){return this.index+1<this.array.length&&(this.index++,!0)},current:function(){return this.array[this.index]},reset:function(){this.index=-1},toArray:function(){return this.array.slice(0)},countable:function(){return!0},count:function(){return this.array.length}}),f=d.inherit({ctor:function(e){this.iter=e},next:function(){return this.iter.next()},current:function(){return this.iter.current()},reset:function(){return this.iter.reset()}}),p=f.inherit({ctor:function(e,t){this.callBase(e),this.index=-1,this.mapper=t},current:function(){return this.mapper(this.callBase(),this.index)},next:function(){var e=this.callBase();return e&&this.index++,e}}),g=function(e,t){return e=(0,s.toComparable)(e),t=(0,s.toComparable)(t),null===e&&null!==t?-1:null!==e&&null===t?1:void 0===e&&void 0!==t?1:void 0!==e&&void 0===t?-1:e<t?-1:e>t?1:0},_=d.inherit({ctor:function(e,t,n,i){e instanceof p||(e=new p(e,this._wrap)),this.iter=e,this.rules=[{getter:t,desc:n,compare:i}]},thenBy:function(e,t,n){var i=new _(this.sortedIter||this.iter,e,t,n);return this.sortedIter||(i.rules=this.rules.concat(i.rules)),i},next:function(){return this._ensureSorted(),this.sortedIter.next()},current:function(){return this._ensureSorted(),this.sortedIter.current()},reset:function(){delete this.sortedIter},countable:function(){return this.sortedIter||this.iter.countable()},count:function(){return this.sortedIter?this.sortedIter.count():this.iter.count()},_ensureSorted:function(){var e=this;e.sortedIter||((0,r.each)(e.rules,function(){this.getter=(0,s.compileGetter)(this.getter)}),e.sortedIter=new p(new h(this.iter.toArray().sort(function(t,n){return e._compare(t,n)})),e._unwrap))},_wrap:function(e,t){return{index:t,value:e}},_unwrap:function(e){return e.value},_compare:function(e,t){var n=e.index,i=t.index;if(e=e.value,t=t.value,e===t)return n-i;for(var o=0,a=this.rules.length;o<a;o++){var r=this.rules[o],s=r.getter(e),l=r.getter(t),u=r.compare||g,c=u(s,l);if(c)return r.desc?-c:c}return n-i}}),m=function(){function e(e,n,i){return function(o){o=(0,s.toComparable)(e(o));var a=t(n)?o===n:o==n;return i&&(a=!a),a}}function t(e){return""===e||0===e||e===!1}function n(e){var t=e[0],n=m(e[1]);if("!"===t)return function(e){return!n(e)};throw u.default.errors.Error("E4003",t)}var i=function(e){var t=[],n=!1,i=!1;return(0,r.each)(e,function(){if(Array.isArray(this)||(0,a.isFunction)(this)){if(t.length>1&&n!==i)throw new u.default.errors.Error("E4019");t.push(m(this)),n=i,i=!0}else i=c.default.isConjunctiveOperator(this)}),function(e){for(var i=n,o=0;o<t.length;o++)if(t[o](e)!==n){i=!n;break}return i}},o=function(e){return(0,a.isDefined)(e)?e.toString():""},l=function(t){t=c.default.normalizeBinaryCriterion(t);var n=(0,s.compileGetter)(t[0]),i=t[1],a=t[2];switch(a=(0,s.toComparable)(a),i.toLowerCase()){case"=":return e(n,a);case"<>":return e(n,a,!0);case">":return function(e){return(0,s.toComparable)(n(e))>a};case"<":return function(e){return(0,s.toComparable)(n(e))<a};case">=":return function(e){return(0,s.toComparable)(n(e))>=a};case"<=":return function(e){return(0,s.toComparable)(n(e))<=a};case"startswith":return function(e){return 0===(0,s.toComparable)(o(n(e))).indexOf(a)};case"endswith":return function(e){var t=(0,s.toComparable)(o(n(e))),i=o(a);return!(t.length<i.length)&&t.lastIndexOf(a)===t.length-a.length};case"contains":return function(e){return(0,s.toComparable)(o(n(e))).indexOf(a)>-1};case"notcontains":return function(e){return(0,s.toComparable)(o(n(e))).indexOf(a)===-1}}throw u.default.errors.Error("E4003",i)};return function(e){return(0,a.isFunction)(e)?e:c.default.isGroupCriterion(e)?i(e):c.default.isUnaryOperation(e)?n(e):l(e)}}(),v=f.inherit({ctor:function(e,t){this.callBase(e),this.criteria=m(t)},next:function(){for(;this.iter.next();)if(this.criteria(this.current()))return!0;return!1}}),y=d.inherit({ctor:function(e,t){this.iter=e,this.getter=t},next:function(){return this._ensureGrouped(),this.groupedIter.next()},current:function(){return this._ensureGrouped(),this.groupedIter.current()},reset:function(){delete this.groupedIter},countable:function(){return!!this.groupedIter},count:function(){return this.groupedIter.count()},_ensureGrouped:function(){if(!this.groupedIter){var e={},t=[],n=this.iter,i=(0,s.compileGetter)(this.getter);for(n.reset();n.next();){var o=n.current(),a=i(o);a in e?e[a].push(o):(e[a]=[o],t.push(a))}this.groupedIter=new h((0,r.map)(t,function(t){return{key:t,items:e[t]}}))}}}),b=f.inherit({ctor:function(e,t){this.callBase(e),this.getter=(0,s.compileGetter)(t)},current:function(){return this.getter(this.callBase())},countable:function(){return this.iter.countable()},count:function(){return this.iter.count()}}),x=f.inherit({ctor:function(e,t,n){this.callBase(e),this.skip=Math.max(0,t),this.take=Math.max(0,n),this.pos=0},next:function(){if(this.pos>=this.skip+this.take)return!1;for(;this.pos<this.skip&&this.iter.next();)this.pos++;return this.pos++,this.iter.next()},reset:function(){this.callBase(),this.pos=0},countable:function(){return this.iter.countable()},count:function(){return Math.min(this.iter.count()-this.skip,this.take)}}),w=function e(t,n){function i(t){return e(t,n)}n=n||{},t instanceof d||(t=new h(t));var o=function(e){var t=n.errorHandler;t&&t(e),u.default._errorHandler(e)},r=function(e){var n,i=(new l.Deferred).fail(o),a=e.step,r=e.finalize;try{t.reset(),n="seed"in e?e.seed:t.next()?t.current():NaN;for(var s=n;t.next();)s=a(s,t.current());i.resolve(r?r(s):s)}catch(e){i.reject(e)}return i.promise()},f=function(e,t,n){return r(arguments.length<2?{step:arguments[0]}:{seed:e,step:t,finalize:n})},p=function(e){return r(c.default.aggregators[e])},g=function(e){return(0,a.isFunction)(e)||Array.isArray(e)||(e=[].slice.call(arguments)),i(new b(t,e))},m=function(e){return g((0,s.compileGetter)(e))};return{toArray:function(){return t.toArray()},enumerate:function(){var e=(new l.Deferred).fail(o);try{e.resolve(t.toArray())}catch(t){e.reject(t)}return e.promise()},sortBy:function(e,n,o){return i(new _(t,e,n,o))},thenBy:function(e,n,o){if(t instanceof _)return i(t.thenBy(e,n,o));throw u.default.errors.Error("E4004")},filter:function(e){return Array.isArray(e)||(e=[].slice.call(arguments)),i(new v(t,e))},slice:function(e,n){return void 0===n&&(n=Number.MAX_VALUE),i(new x(t,e,n))},select:g,groupBy:function(e){return i(new y(t,e))},aggregate:f,count:function(){if(t.countable()){var e=(new l.Deferred).fail(o);try{e.resolve(t.count())}catch(t){e.reject(t)}return e.promise()}return p("count")},sum:function(e){return e?m(e).sum():p("sum")},min:function(e){return e?m(e).min():p("min")},max:function(e){return e?m(e).max():p("max")},avg:function(e){return e?m(e).avg():p("avg")}}},C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(5)),a=i(n(23)),r=i(n(11)),s=i(n(15)),l=n(8),u="dxPointerEvents",c=s.default.inherit({ctor:function(e,t){this._eventName=e,this._originalEvents=(0,l.addNamespace)(t,u),this._handlerCount=0,this.noBubble=this._isNoBubble()},_isNoBubble:function(){var e=this._eventName;return"dxpointerenter"===e||"dxpointerleave"===e},_handler:function(e){var t=this._getDelegateTarget(e);return this._fireEvent({type:this._eventName,pointerType:e.pointerType||(0,l.eventSource)(e),originalEvent:e,delegateTarget:t,timeStamp:a.default.mozilla?(new Date).getTime():e.timeStamp})},_getDelegateTarget:function(e){var t;return this.noBubble&&(t=e.delegateTarget),t},_fireEvent:function(e){return(0,l.fireEvent)(e)},_setSelector:function(e){this._selector=this.noBubble&&e?e.selector:null},_getSelector:function(){return this._selector},setup:function(){return!0},add:function(e,t){if(this._handlerCount<=0||this.noBubble){e=this.noBubble?e:r.default.getDocument(),this._setSelector(t);var n=this;o.default.on(e,this._originalEvents,this._getSelector(),function(e){n._handler(e)})}this.noBubble||this._handlerCount++},remove:function(e){this._setSelector(e),this.noBubble||this._handlerCount--},teardown:function(e){this._handlerCount&&!this.noBubble||(e=this.noBubble?e:r.default.getDocument(),this._originalEvents!=="."+u&&o.default.off(e,this._originalEvents,this._getSelector()))},dispose:function(e){e=this.noBubble?e:r.default.getDocument(),o.default.off(e,this._originalEvents)}}),d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.end=t.start=void 0;var o=i(n(5)),a=n(39),r=i(n(15)),s=i(n(13)),l=i(n(66)),u=n(8),c=i(n(25)),d="dxHoverStart",h="dxhoverstart";t.start=h;var f=(0,u.addNamespace)(c.default.enter,d),p="dxHoverEnd",g="dxhoverend";t.end=g;var _=(0,u.addNamespace)(c.default.leave,p),m=r.default.inherit({noBubble:!0,ctor:function(){this._handlerArrayKeyPath=this._eventNamespace+"_HandlerStore"},setup:function(e){(0,a.data)(e,this._handlerArrayKeyPath,{})},add:function(e,t){var n=this,i=function(e){n._handler(e)};o.default.on(e,this._originalEventName,t.selector,i),(0,a.data)(e,this._handlerArrayKeyPath)[t.guid]=i},_handler:function(e){(0,u.isTouchEvent)(e)||s.default.isSimulator()||(0,u.fireEvent)({type:this._eventName,originalEvent:e,delegateTarget:e.delegateTarget})},remove:function(e,t){var n=(0,a.data)(e,this._handlerArrayKeyPath)[t.guid];o.default.off(e,this._originalEventName,t.selector,n)},teardown:function(e){(0,a.removeData)(e,this._handlerArrayKeyPath)}}),v=m.inherit({ctor:function(){this._eventNamespace=d,this._eventName=h,this._originalEventName=f,this.callBase()},_handler:function(e){var t=e.pointers||[];t.length||this.callBase(e)}}),y=m.inherit({ctor:function(){this._eventNamespace=p,this._eventName=g,this._originalEventName=_,this.callBase()}});(0,l.default)(h,new v),(0,l.default)(g,new y)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.odata=void 0;var o=n(1),a=n(3),r=i(n(28)),s=n(0),l=i(n(186)),u=n(95),c=i(n(34)),d=i(n(41)),h=2,f=["contains","notcontains","startswith","endswith"],p=function(){var e,t,n,i=function(e){return function(t,n){return"".concat(t," ").concat(e," ").concat(n)}},o=function(e,n){return function(i,o){var a=[e,"("];return t&&(i=i.indexOf("tolower(")===-1?"tolower(".concat(i,")"):i,o=o.toLowerCase()),n?a.push(o,",",i):a.push(i,",",o),a.push(")"),a.join("")}},l=function(e){return f.some(function(t){return t===e})},h={"=":i("eq"),"<>":i("ne"),">":i("gt"),">=":i("ge"),"<":i("lt"),"<=":i("le"),startswith:o("startswith"),endswith:o("endswith")},p=(0,s.extend)({},h,{contains:o("substringof",!0),notcontains:o("not substringof",!0)}),g=(0,s.extend)({},h,{contains:o("contains"),notcontains:o("not contains")}),_=function(t){var i;t=d.default.normalizeBinaryCriterion(t);var o=t[1],a=t[0],r=n&&n[a];if(r&&l(o)&&"String"!==r)throw new c.default.errors.Error("E4024",o,a,r);var s=4===e?g:p,h=s[o.toLowerCase()];if(!h)throw c.default.errors.Error("E4003",o);var f=t[2];return null!==(i=n)&&void 0!==i&&i[a]&&(f=(0,u.convertPrimitiveValue)(n[a],f)),h((0,u.serializePropName)(a),(0,u.serializeValue)(f,e))},m=function(e){var t=e[0],n=y(e[1]);if("!"===t)return"not (".concat(n,")");throw c.default.errors.Error("E4003",t)},v=function(e){var t,n,i=[];return(0,a.each)(e,function(e,o){if(Array.isArray(o)){if(i.length>1&&t!==n)throw new c.default.errors.Error("E4019");i.push("(".concat(y(o),")")),t=n,n="and"}else n=d.default.isConjunctiveOperator(this)?"and":"or"}),i.join(" ".concat(t," "))},y=function(e){return Array.isArray(e[0])?v(e):d.default.isUnaryOperation(e)?m(e):_(e)};return function(i,o,a,s){return n=a,t=null!==s&&void 0!==s?s:(0,r.default)().oDataFilterToLower,e=o,y(i)}}(),g=function(e){var t,n,i,a,r=[],l=[],c=e.expand,d=e.version||h,f=function(){return n||void 0!==i},g=function e(t){for(var n=0;n<t.length;n++){if((0,o.isFunction)(t[n]))return!0;if(Array.isArray(t[n])&&e(t[n]))return!0}return!1},_=function(){var o={};if(a||(r.length&&(o.$orderby=r.join(",")),n&&(o.$skip=n),void 0!==i&&(o.$top=i),o.$select=(0,u.generateSelect)(d,t)||void 0,o.$expand=(0,u.generateExpand)(d,c,t)||void 0),l.length){var s=l.length<2?l[0]:l,h=null===e||void 0===e?void 0:e.fieldTypes,f=null===e||void 0===e?void 0:e.filterToLower;o.$filter=p(s,d,h,f)}return a&&(o.$top=0),(e.requireTotalCount||a)&&(4!==d?o.$inlinecount="allpages":o.$count="true"),o},m=function(e){for(var t=-1,n=0;n<e.length;n++)if("select"===e[n].name){t=n;break}if(!(t<0)&&(0,o.isFunction)(e[t].args[0])){var i=e[1+t];i&&"slice"===i.name&&(e[1+t]=e[t],e[t]=i)}};return{optimize:m,exec:function(t){return(0,u.sendRequest)(d,{url:t,params:(0,s.extend)(_(),null===e||void 0===e?void 0:e.params)},{beforeSend:e.beforeSend,jsonp:e.jsonp,withCredentials:e.withCredentials,countOnly:a,deserializeDates:e.deserializeDates,fieldTypes:e.fieldTypes,isPaged:isFinite(i)})},multiSort:function(e){var t;if(f())return!1;for(var n=0;n<e.length;n++){var i=e[n][0],o=!!e[n][1],a=void 0;if("string"!=typeof i)return!1;a=(0,u.serializePropName)(i),o&&(a+=" desc"),t=t||[],t.push(a)}r=t},slice:function(e,t){return!f()&&(n=e,void(i=t))},filter:function(e){return!f()&&(Array.isArray(e)||(e=[].slice.call(arguments)),!g(e)&&(l.length&&l.push("and"),void l.push(e)))},select:function(e){return!t&&!(0,o.isFunction)(e)&&(Array.isArray(e)||(e=[].slice.call(arguments)),void(t=e))},count:function(){return a=!0}}};l.default.odata=g;var _=g;t.odata=_},function(e,t,n){t.default=void 0;var i="20.2.5";t.default=i,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getFormat=void 0;var o=i(n(45)),a="\u060c",r=" .,:;/\\<>()-[]"+a,s=". m.",l=function(e){var t=e&&o.default.convertDigits(e,!1).charCodeAt(0),n=o.default.convertDigits("0",!1).charCodeAt(0);return n<=t&&t<n+10},u=function(e,t,n){var i=e[t],o=e[t+1];if(!n){if("."===i||" "===i&&e.slice(t-1,t+3)===s)return!0;if("-"===i&&!l(o))return!0}return r.indexOf(i)<0&&n===l(i)},c=function(e,t){if(!l(e[t]))for(;t>0&&!l(e[t-1])&&("."===e[t-1]||r.indexOf(e[t-1])<0);)t--;return t},d=function(e,t,n,i){var o=0,a=[],r=function(t){return e[o]!==t[o]&&(void 0===i||l(e[o])===i)};for(Array.isArray(t)||(t=[t]),o=0;o<e.length;o++)if(n.indexOf(o)<0&&t.filter(r).length){o=c(e,o);do{if(i=l(e[o]),!a.length&&!i&&l(t[0][o]))break;a.push(o),n.unshift(o),o++}while(e[o]&&u(e,o,i));break}return 1!==a.length||"0"!==e[n[0]-1]&&"\u0660"!==e[n[0]-1]||n.unshift(n[0]-1),a},h=function(e,t,n,i){var o=t[0],a=o<i.length?i[o]:o;return t.forEach(function(t,i){e=e.substr(0,a+i)+(n.length>1?n[i]:n)+e.substr(a+i+1)}),1===t.length&&(e=e.replace("0"+n,n+n),e=e.replace("\u0660"+n,n+n)),e},f=function(e,t,n,i){var o,a,r;if(!l(e[t[0]]||"0")){for(var s=Math.max(t.length<=3?3:4,n.length);t.length>s;){for(a=t.pop(),r=i[a],i[a]=-1,o=a+1;o<i.length;o++)i[o]--;e=e.substr(0,r)+e.substr(r+1)}for(a=t[t.length-1]+1,r=a<i.length?i[a]:a;t.length<s;){for(t.push(t[t.length-1]+1),o=a;o<i.length;o++)i[o]++;e=e.substr(0,r)+" "+e.substr(r)}}return e=h(e,t,n,i)},p=function(e,t){return Array.isArray(e)?e.map(function(e){return(t(e)||"").toString()}):(t(e)||"").toString()},g=/[a-zA-Z]/g,_=function(e,t,n,i){var o=t.split("").map(function(e,t){return n.indexOf(t)<0&&(e.match(g)||"'"===e)?i[t]:-1});return e=e.split("").map(function(e,t){var n=e,i=o.indexOf(t)>=0,a=t>0&&o.indexOf(t-1)>=0,r=o.indexOf(t+1)>=0;return i&&(a||(n="'"+n),r||(n+="'")),n}).join("")},m=function(e){var t=[],n=p(new Date(2009,8,8,6,5,4),e),i=n.split("").map(function(e,t){return t}),o=n,a={},r=[{date:new Date(2009,8,8,6,5,4,100),pattern:"S"},{date:new Date(2009,8,8,6,5,2),pattern:"s"},{date:new Date(2009,8,8,6,2,4),pattern:"m"},{date:new Date(2009,8,8,18,5,4),pattern:"H",isDigit:!0},{date:new Date(2009,8,8,2,5,4),pattern:"h",isDigit:!0},{date:new Date(2009,8,8,18,5,4),pattern:"a",isDigit:!1},{date:new Date(2009,8,1,6,5,4),pattern:"d"},{date:[new Date(2009,8,2,6,5,4),new Date(2009,8,3,6,5,4),new Date(2009,8,4,6,5,4)],pattern:"E"},{date:new Date(2009,9,6,6,5,4),pattern:"M"},{date:new Date(1998,8,8,6,5,4),pattern:"y"}];if(o)return r.forEach(function(r){var s=d(n,p(r.date,e),t,r.isDigit),l="M"!==r.pattern||a.d?r.pattern:"L";o=f(o,s,l,i),a[l]=s.length}),o=_(o,n,t,i),t.length?o:void 0};t.getFormat=m},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){c[e]=t}function a(e){if((0,l.isString)(e)){if(s=c[e],!s)throw u.default.Error("E0020",e)}else s=e}function r(){return s}t.registerTemplateEngine=o,t.setTemplateEngine=a,t.getCurrentTemplateEngine=r;var s,l=n(1),u=i(n(22)),c={}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=n(97),l=i(n(64)),u=n(33),c=i(n(49)),d=n(29),h=n(4),f=n(1),p=n(8),g=i(n(121)),_=c.default.add,m=Math.abs,v=0,y=1,b=2,x=10,w=0,C=180,k=function(){return(0,s.styleProp)("pointer-events")},S=(0,l.default)(function(){var e="dx-gesture-cover",t="desktop"===r.default.real().deviceType;if(!k()||!t)return h.noop;var n=(0,o.default)("<div>").addClass(e).css("pointerEvents","none");return a.default.subscribeGlobal(n,"dxmousewheel",function(e){e.preventDefault()}),_(function(){n.appendTo("body")}),function(e,t){n.css("pointerEvents",e?"all":"none"),e&&n.css("cursor",t)}}),D=function(e,t){var n=S();n(e,t)},I=g.default.inherit({gesture:!0,configure:function(e){this.getElement().css("msTouchAction",e.immediate?"pinch-zoom":""),this.callBase(e)},allowInterruptionByMouseWheel:function(){return this._stage!==b},getDirection:function(){return this.direction},_cancel:function(){this.callBase.apply(this,arguments),this._toggleGestureCover(!1),this._stage=v},start:function(e){return e._needSkipEvent||(0,p.needSkipEvent)(e)?void this._cancel(e):(this._startEvent=(0,p.createEvent)(e),this._startEventData=(0,p.eventData)(e),this._stage=y,this._init(e),void this._setupImmediateTimer())},_setupImmediateTimer:function(){clearTimeout(this._immediateTimer),this._immediateAccepted=!1,this.immediate&&(this._immediateTimer=setTimeout(function(){this._immediateAccepted=!0}.bind(this),C))},move:function(e){if(this._stage===y&&this._directionConfirmed(e)){if(this._stage=b,this._resetActiveElement(),this._toggleGestureCover(!0),this._clearSelection(e),this._adjustStartEvent(e),this._start(this._startEvent),this._stage===v)return;this._requestAccept(e),this._move(e),this._forgetAccept()}else this._stage===b&&(this._clearSelection(e),this._move(e))},_directionConfirmed:function(e){var t=this._getTouchBoundary(e),n=(0,p.eventDelta)(this._startEventData,(0,p.eventData)(e)),i=m(n.x),o=m(n.y),a=this._validateMove(t,i,o),r=this._validateMove(t,o,i),s=this.getDirection(e),l="both"===s&&(a||r),u="horizontal"===s&&a,c="vertical"===s&&r;return l||u||c||this._immediateAccepted},_validateMove:function(e,t,n){return t&&t>=e&&(!this.immediate||t>=n)},_getTouchBoundary:function(e){return this.immediate||(0,p.isDxMouseWheelEvent)(e)?w:x},_adjustStartEvent:function(e){var t=this._getTouchBoundary(e),n=(0,p.eventDelta)(this._startEventData,(0,p.eventData)(e));this._startEvent.pageX+=(0,d.sign)(n.x)*t,this._startEvent.pageY+=(0,d.sign)(n.y)*t},_resetActiveElement:function(){"ios"===r.default.real().platform&&this.getElement().find(":focus").length&&(0,u.resetActiveElement)()},_toggleGestureCover:function(e){this._toggleGestureCoverImpl(e)},_toggleGestureCoverImpl:function(e){var t=this._stage===b;t&&D(e,this.getElement().css("cursor"))},_clearSelection:function(e){(0,p.isDxMouseWheelEvent)(e)||(0,p.isTouchEvent)(e)||(0,u.clearSelection)()},end:function(e){this._toggleGestureCover(!1),this._stage===b?this._end(e):this._stage===y&&this._stop(e),this._stage=v},dispose:function(){clearTimeout(this._immediateTimer),this.callBase.apply(this,arguments),this._toggleGestureCover(!1)},_init:h.noop,_start:h.noop,_move:h.noop,_stop:h.noop,_end:h.noop});I.initialTouchBoundary=x,I.touchBoundary=function(e){return(0,f.isDefined)(e)?void(x=e):x};var T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(78),r="dx-swatch-",s=function(e){var t=(0,o.default)(e),n=t.closest('[class^="'.concat(r,'"], [class*=" ').concat(r,'"]')),i=(0,a.value)();if(!n.length)return i;var s=new RegExp("(\\s|^)(".concat(r,".*?)(\\s|$)")),l=n[0].className.match(s)[2],u=i.children("."+l);return u.length||(u=(0,o.default)("<div>").addClass(l).appendTo(i)),u},l={getSwatchContainer:s};t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=i(n(15)),l=n(191),u=n(8),c="dxListEditDecorator",d=(0,u.addNamespace)(l.start,c),h=(0,u.addNamespace)(l.swipe,c),f=(0,u.addNamespace)(l.end,c),p=s.default.inherit({ctor:function(e){this._list=e,this._init()},_init:r.noop,_shouldHandleSwipe:!1,_attachSwipeEvent:function(e){var t={itemSizeFunc:function(){return this._clearSwipeCache&&(this._itemWidthCache=this._list.$element().width(),this._clearSwipeCache=!1),this._itemWidthCache}.bind(this)};a.default.on(e.$itemElement,d,t,this._itemSwipeStartHandler.bind(this)),a.default.on(e.$itemElement,h,this._itemSwipeUpdateHandler.bind(this)),a.default.on(e.$itemElement,f,this._itemSwipeEndHandler.bind(this))},_itemSwipeStartHandler:function(e){var t=(0,o.default)(e.currentTarget);return t.is(".dx-state-disabled, .dx-state-disabled *")?void(e.cancel=!0):(clearTimeout(this._list._inkRippleTimer),void this._swipeStartHandler(t,e))},_itemSwipeUpdateHandler:function(e){var t=(0,o.default)(e.currentTarget);this._swipeUpdateHandler(t,e)},_itemSwipeEndHandler:function(e){var t=(0,o.default)(e.currentTarget);this._swipeEndHandler(t,e),this._clearSwipeCache=!0},beforeBag:r.noop,afterBag:r.noop,_commonOptions:function(){return{activeStateEnabled:this._list.option("activeStateEnabled"),hoverStateEnabled:this._list.option("hoverStateEnabled"),focusStateEnabled:this._list.option("focusStateEnabled")}},modifyElement:function(e){this._shouldHandleSwipe&&(this._attachSwipeEvent(e),this._clearSwipeCache=!0)},afterRender:r.noop,handleClick:r.noop,handleKeyboardEvents:r.noop,handleEnterPressing:r.noop,handleContextMenu:r.noop,_swipeStartHandler:r.noop,_swipeUpdateHandler:r.noop,_swipeEndHandler:r.noop,visibilityChange:r.noop,getExcludedSelectors:r.noop,dispose:r.noop}),g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(9)),r=n(4),s=n(0),l=n(12),u=n(3),c=i(n(656)),d=i(n(657)),h=i(n(418)),f=n(154),p={actionSheet:c.default,dropDownMenu:d.default},g="dx-toolbar-item-auto-hide",_="dx-toolbar-text-auto-hide",m="dx-toolbar-item-invisible",v=h.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{menuItemTemplate:"menuItem",submenuType:"dropDownMenu",menuContainer:void 0})},_dimensionChanged:function(e){"height"!==e&&(this._menuStrategy.toggleMenuVisibility(!1,!0),this.callBase(),this._menuStrategy.renderMenuItems())},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({actionSheetItem:new f.ChildDefaultTemplate("item")})},_initMarkup:function(){this.callBase(),this._renderMenu()},_postProcessRenderItems:function(){var e=this;this._hideOverflowItems(),this._menuStrategy._updateMenuVisibility(),this.callBase(),(0,r.deferRender)(function(){e._menuStrategy.renderMenuItems()})},_renderItem:function(e,t,n,i){var o=this.callBase(e,t,n,i);return"auto"===t.locateInMenu&&o.addClass(g),"dxButton"===t.widget&&"inMenu"===t.showText&&o.toggleClass(_),o},_getItemsWidth:function(){return this._getSummaryItemsWidth([this._$beforeSection,this._$centerSection,this._$afterSection])},_hideOverflowItems:function(e){var t=this.$element().find("."+g);if(t.length){e=e||this.$element().width(),(0,o.default)(t).removeClass(m);for(var n=this._getItemsWidth();t.length&&e<n;){var i=t.eq(-1);n-=i.outerWidth(),i.addClass(m),t.splice(-1,1)}}},_getMenuItems:function(){var e=this,t=(0,r.grep)(this.option("items")||[],function(t){return e._isMenuItem(t)}),n=this._itemContainer().children("."+g+"."+m).not(".dx-state-invisible");this._restoreItems=this._restoreItems||[];var i=[].slice.call(n).map(function(t){var n=e._getItemData(t),i=(0,o.default)(t).children(),a=i.children();return(0,s.extend)({menuItemTemplate:function(){e._restoreItems.push({container:i,item:a});var t=(0,o.default)("<div>").addClass(g);return t.append(a)}},n)});return(0,l.merge)(i,t)},_getToolbarItems:function(){var e=this;return(0,r.grep)(this.option("items")||[],function(t){return!e._isMenuItem(t)})},_renderMenu:function(){var e=this;this._renderMenuStrategy(),(0,r.deferRender)(function(){e._menuStrategy.render()})},_renderMenuStrategy:function(){var e=this.option("submenuType");this._requireDropDownStrategy()&&(e="dropDownMenu");var t=p[e];this._menuStrategy&&this._menuStrategy.NAME===e||(this._menuStrategy=new t(this))},_requireDropDownStrategy:function(){var e=this.option("items")||[],t=!1;return(0,u.each)(e,function(e,n){"auto"===n.locateInMenu?t=!0:"always"===n.locateInMenu&&n.widget&&(t=!0)}),t},_arrangeItems:function(){if(!this.$element().is(":hidden")){this._$centerSection.css({margin:"0 auto","float":"none"}),(0,u.each)(this._restoreItems||[],function(e,t){(0,o.default)(t.container).append(t.item)}),this._restoreItems=[];var e=this.$element().width();this._hideOverflowItems(e),this.callBase(e)}},_itemOptionChanged:function(e,t,n){this._isMenuItem(e)?this._menuStrategy.renderMenuItems():this._isToolbarItem(e)?this.callBase(e,t,n):(this.callBase(e,t,n),this._menuStrategy.renderMenuItems()),"location"===t&&this.repaint()},_isMenuItem:function(e){return"menu"===e.location||"always"===e.locateInMenu},_isToolbarItem:function(e){return void 0===e.location||"never"===e.locateInMenu},_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"submenuType":this._invalidate();break;case"visible":this.callBase.apply(this,arguments),this._menuStrategy.handleToolbarVisibilityChange(n);break;case"menuItemTemplate":this._changeMenuOption("itemTemplate",this._getTemplate(n));break;case"onItemClick":this._changeMenuOption(t,n),this.callBase.apply(this,arguments);break;case"menuContainer":this._changeMenuOption("container",n);break;default:this.callBase.apply(this,arguments)}},_changeMenuOption:function(e,t){this._menuStrategy.widgetOption(e,t)}});(0,a.default)("dxToolbar",v);var y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(477)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){n.d(t,"a",function(){return i}),n.d(t,"b",function(){return o});var i=["MO","TU","WE","TH","FR","SA","SU"],o=function(){function e(e,t){if(0===t)throw new Error("Can't create weekday with n == 0");this.weekday=e,this.n=t}return e.fromStr=function(t){return new e(i.indexOf(t))},e.prototype.nth=function(t){return this.n===t?this:new e(this.weekday,t)},e.prototype.equals=function(e){return this.weekday===e.weekday&&this.n===e.n},e.prototype.toString=function(){var e=i[this.weekday];return this.n&&(e=(this.n>0?"+":"")+String(this.n)+e),e},e.prototype.getJsWeekday=function(){return 6===this.weekday?0:this.weekday+1},e}()},function(e,t,n){t.AGENDA_LAST_IN_DATE_APPOINTMENT_CLASS=t.APPOINTMENT_DRAG_SOURCE_CLASS=t.DIRECTION_APPOINTMENT_CLASSES=t.REDUCED_APPOINTMENT_PARTS_CLASSES=t.ALL_DAY_APPOINTMENT_CLASS=t.EMPTY_APPOINTMENT_CLASS=t.RECURRENCE_APPOINTMENT_CLASS=t.REDUCED_APPOINTMENT_ICON=t.REDUCED_APPOINTMENT_CLASS=t.APPOINTMENT_SETTINGS_KEY=t.LIST_ITEM_CLASS=t.FIXED_CONTAINER_CLASS=t.LIST_ITEM_DATA_KEY=void 0;var i="dxListItemData";t.LIST_ITEM_DATA_KEY=i;var o="dx-scheduler-fixed-appointments";t.FIXED_CONTAINER_CLASS=o;var a="dx-list-item";t.LIST_ITEM_CLASS=a;var r="dxAppointmentSettings";t.APPOINTMENT_SETTINGS_KEY=r;var s="dx-scheduler-appointment-reduced";t.REDUCED_APPOINTMENT_CLASS=s;var l="dx-scheduler-appointment-reduced-icon";t.REDUCED_APPOINTMENT_ICON=l;var u="dx-scheduler-appointment-recurrence";t.RECURRENCE_APPOINTMENT_CLASS=u;var c="dx-scheduler-appointment-empty";t.EMPTY_APPOINTMENT_CLASS=c;var d="dx-scheduler-all-day-appointment";t.ALL_DAY_APPOINTMENT_CLASS=d;var h={head:"dx-scheduler-appointment-head",body:"dx-scheduler-appointment-body",tail:"dx-scheduler-appointment-tail"};t.REDUCED_APPOINTMENT_PARTS_CLASSES=h;var f={horizontal:"dx-scheduler-appointment-horizontal",vertical:"dx-scheduler-appointment-vertical"};t.DIRECTION_APPOINTMENT_CLASSES=f;var p="dx-scheduler-appointment-drag-source";t.APPOINTMENT_DRAG_SOURCE_CLASS=p;var g="dx-scheduler-last-in-date-agenda-appointment";t.AGENDA_LAST_IN_DATE_APPOINTMENT_CLASS=g},function(e,t,n){t.LayoutProps=void 0;var i={viewData:{groupedData:[],cellCountInGroupRow:0}};t.LayoutProps=i},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return u(e)||l(e,t)||r(e,t)||a()}function a(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function l(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function u(e){if(Array.isArray(e))return e}function c(e){0===ct().backupCounter&&Te.default.getBody().appendChild(ct().backupContainer),++ct().backupCounter,e.append({element:ct().backupContainer})}function d(e,t){e.append({element:t}),--ct().backupCounter,0===ct().backupCounter&&Te.default.getBody().removeChild(ct().backupContainer)}function h(e){return e&&"string"!=typeof e}function f(e){return Te.default.createElementNS("http://www.w3.org/2000/svg",e)}function p(e,t){return null!==e?"url("+(t?Fe.location.href.split("#")[0]:"")+"#"+e+")":e}function g(e,t){var n;for(n in t)e[n]=t[n];return e}function _(e,t){return e=e.toString().split("e"),e=Ne(+(e[0]+"e"+(e[1]?+e[1]+t:t))),e=e.toString().split("e"),+(e[0]+"e"+(e[1]?+e[1]-t:-t))}function m(e){var t;try{t=e.getBoundingClientRect()}catch(e){}return t||{left:0,top:0}}function v(e,t){return t.hatching&&"none"!==(0,
Be.normalizeEnum)(t.hatching.direction)?(t=g({},t),t.fill=e._hatching=e.renderer.lockHatching(t.fill,t.hatching,e._hatching),delete t.hatching):e._hatching&&(e.renderer.releaseHatching(e._hatching),e._hatching=null),t}function y(e,t,n,i,o,a){var r,s=!0,l=_(a,3)-_(o,3);return l&&(Ge(l)%360===0&&(o=0,a=360,r=!0,a-=.01),o>360&&(o%=360),a>360&&(a%=360),o>a&&(o-=360),s=!1),o*=$e,a*=$e,[e,t,He(i,n),Le(i,n),We(o),je(o),We(a),je(a),r,ze(Ge(a-o)/qe)%2?"1":"0",s]}function b(e,t){var n=[["M",0,0]];switch(t){case"line":n=x(e);break;case"area":n=x(e,!0);break;case"bezier":n=w(e);break;case"bezierarea":n=w(e,!0)}return n}function x(e,t){return C(e,k,t)}function w(e,t){return C(e,S,t)}function C(e,t,n){var i,o,a,r=[];if(null!==(i=e[0])&&void 0!==i&&i.length)for(o=0,a=e.length;o<a;++o)t(e[o],n,r);else t(e,n,r);return r}function k(e,t,n){var i=0,o=n.length,a=o,r=(e||[]).length;if(r){if(void 0!==e[0].x)for(;i<r;)n[a++]=["L",e[i].x,e[i++].y];else for(;i<r;)n[a++]=["L",e[i++],e[i++]];n[o][0]="M"}else n[a]=["M",0,0];return t&&n.push(["Z"]),n}function S(e,t,n){var i,o=n.length,a=(e||[]).length;if(a)if(void 0!==e[0].x)for(n[o++]=["M",e[0].x,e[0].y],i=1;i<a;)n[o++]=["C",e[i].x,e[i++].y,e[i].x,e[i++].y,e[i].x,e[i++].y];else for(n[o++]=["M",e[0],e[1]],i=2;i<a;)n[o++]=["C",e[i++],e[i++],e[i++],e[i++],e[i++],e[i++]];else n[o]=["M",0,0];return t&&n.push(["Z"]),n}function D(e){var t,n,i,o,a=[],r=0,s=e.length;for(t=0;t<s;++t)for(n=e[t],i=0,o=n.length;i<o;++i)a[r++]=n[i];return a.join(" ")}function I(e,t,n){var i,o,a=e.length,r=t.length,s=n.indexOf("area")!==-1?E:O;if(0===a)for(i=0;i<r;i++)e.push(t[i].slice(0));else a<r?s(e,t,n):a>r&&(o=t.slice(0),s(t,e,n));return o}function T(e,t){var n=e[e.length-2],i=e[e.length-1];switch(t){case"line":case"area":e[0]="L";break;case"bezier":case"bezierarea":e[0]="C",e[1]=e[3]=e[5]=n,e[2]=e[4]=e[6]=i}}function O(e,t,n){var i=e[e.length-1].slice(),o=e.length;for(T(i,n);o<t.length;o++)e[o]=i.slice(0)}function E(e,t,n){var i,o,a,r,s=e.length,l=t.length;if((s-1)%2===0&&(l-1)%2===0){i=(s-1)/2-1,o=e.slice(0,i+1),a=o[o.length-1].slice(0),r=e.slice(i+1)[0].slice(0),T(a,n),T(r,n);for(var u=i;u<(l-1)/2-1;u++)e.splice(u+1,0,a),e.splice(u+3,0,r)}}function A(e,t){var n,i,o=e._styles,a="";t=t||{};for(n in t)i=t[n],(0,Ve.isDefined)(i)&&(i+="number"!=typeof i||Ye[n]?"":"px",o[n]=""!==i?i:null);for(n in o)i=o[n],i&&(a+=n+":"+i+";");return a&&e.element.setAttribute("style",a),e}function P(e,t){var n=e.element,i=e.attr(t);i&&i.indexOf("DevExpress")!==-1&&(n.removeAttribute(t),n.setAttribute(t,p(i,e.renderer.pathModified)))}function R(e,t){t=t||{};var n,i,o,a,r,s,l=e._settings,u={},c=e.element,d=e.renderer,f=d.rtl;if(!h(t))return t in l?l[t]:t in ut?ut[t]:0;g(u,t);for(n in u)if(i=u[n],void 0!==i){if(l[n]=i,"align"===n)n="text-anchor",i={left:f?"end":"start",center:"middle",right:f?"start":"end"}[i]||null;else{if("dashStyle"===n){a=!0;continue}if(n===Qe)a=!0;else if(!i||"fill"!==n&&"clip-path"!==n&&"filter"!==n||0!==i.indexOf("DevExpress")){if(/^(translate(X|Y)|rotate[XY]?|scale(X|Y)|sharp|sharpDirection)$/i.test(n)){o=!0;continue}/^(x|y|d)$/i.test(n)&&(o=!0)}else e._addFixIRICallback(),i=p(i,d.pathModified)}null===i?c.removeAttribute(n):c.setAttribute(n,i)}if(a&&"dashStyle"in l)if(i=l.dashStyle,r=("_originalSW"in e?e._originalSW:l[Qe])||1,n="stroke-dasharray",i=null===i?"":(0,Be.normalizeEnum)(i),""===i||"solid"===i||i===at)e.element.removeAttribute(n);else{for(i=i.replace(/longdash/g,"8,3,").replace(/dash/g,"4,3,").replace(/dot/g,"1,3,").replace(/,$/,"").split(","),s=i.length;s--;)i[s]=parseInt(i[s])*r;e.element.setAttribute(n,i.join(","))}return o&&e._applyTransformation(),e}function M(e){var t,n=this;return h(e)&&(e=g({},e),t=e.segments,"points"in e&&(t=b(e.points,n.type),delete e.points),t&&(e.d=D(t),n.segments=t,delete e.segments)),R(n,e)}function B(e){var t,n,i,o,a,r,s=this._settings;return h(e)&&(e=g({},e),("x"in e||"y"in e||"innerRadius"in e||"outerRadius"in e||"startAngle"in e||"endAngle"in e)&&(s.x=t="x"in e?e.x:s.x,delete e.x,s.y=n="y"in e?e.y:s.y,delete e.y,s.innerRadius=i="innerRadius"in e?e.innerRadius:s.innerRadius,delete e.innerRadius,s.outerRadius=o="outerRadius"in e?e.outerRadius:s.outerRadius,delete e.outerRadius,s.startAngle=a="startAngle"in e?e.startAngle:s.startAngle,delete e.startAngle,s.endAngle=r="endAngle"in e?e.endAngle:s.endAngle,delete e.endAngle,e.d=ft.apply(null,y(t,n,i,o,a,r)))),R(this,e)}function V(e){var t,n,i,o,a,r,s,l=this;return h(e)&&(e=g({},e),void 0===e.x&&void 0===e.y&&void 0===e.width&&void 0===e.height&&void 0===e[Qe]||(t=void 0!==e.x?l._originalX=e.x:l._originalX||0,n=void 0!==e.y?l._originalY=e.y:l._originalY||0,i=void 0!==e.width?l._originalWidth=e.width:l._originalWidth||0,o=void 0!==e.height?l._originalHeight=e.height:l._originalHeight||0,a=void 0!==e[Qe]?l._originalSW=e[Qe]:l._originalSW,r=~~((i<o?i:o)/2),s=(a||0)<r?a||0:r,e.x=t+s/2,e.y=n+s/2,e.width=i-s,e.height=o-s,((a||0)!==s||!(0===s&&void 0===a))&&(e[Qe]=s)),"sharp"in e&&delete e.sharp),R(l,e)}function F(e){var t,n=this;if(!h(e))return R(n,e);e=g({},e);var i=n._settings,o=(0,Ve.isDefined)(i[Xe])&&(0,Ve.isDefined)(i[Qe]);void 0!==e[Ze]&&(i[Ze]=e[Ze],delete e[Ze],t=!0),void 0!==e[Xe]&&(i[Xe]=e[Xe],delete e[Xe]),void 0!==e[Qe]&&(i[Qe]=e[Qe],delete e[Qe]),void 0!==e[Je]&&(i[Je]=e[Je],delete e[Je]),void 0!==e[ot]&&(pe(n,e[ot]),delete e[ot]);var a=(0,Ve.isDefined)(i[Xe])&&(0,Ve.isDefined)(i[Qe]);return R(n,e),t=t||a!==o&&i[Ze],t&&(ce(n,i.text,a),n._hasEllipsis=!1),(t||void 0!==e.x||void 0!==e.y)&&fe(n),a&&_e(n),n}function L(e){return e=e||{},A(this,e),et in e&&fe(this),this}function H(e,t,n,i,o){var a,r,s,l,u;if(void 0!==n.wholeText)e.push({value:n.wholeText,style:i,className:o,line:t,height:i[et]||0});else if("BR"===n.tagName)++t;else if(Te.default.isElementNode(n)){switch(g(a={},i),n.tagName){case"B":case"STRONG":a[nt]="bold";break;case"I":case"EM":a[tt]="italic";break;case"U":a[it]="underline"}for(r=n.style,r.color&&(a.fill=r.color),r.fontSize&&(a[et]=r.fontSize),r.fontStyle&&(a[tt]=r.fontStyle),r.fontWeight&&(a[nt]=r.fontWeight),r.textDecoration&&(a[it]=r.textDecoration),s=0,u=n.childNodes,l=u.length;s<l;++s)t=H(e,t,u[s],a,n.className||o)}return t}function z(e){var t,n,i,o=e[0];for(t=1,n=e.length;t<n;++t)i=e[t],i.line===o.line?(o.height=ge(o.height,i.height),o.inherits=o.inherits||0===parseFloat(i.height),i.height=NaN):o=i}function N(e){var t=/(?:(<[a-z0-9]+\s*))([\s\S]*?)(>|\/>)/gi,n=/(style|class)\s*=\s*(["'])(?:(?!\2).)*\2\s?/gi;return e.replace(t,function(e,t,i,o){return i=(i&&i.match(n)||[]).map(function(e){return e}).join(" "),t+i+o})}function j(e){var t=[],n=Te.default.createElement("div");return n.innerHTML=e.replace(/\r/g,"").replace(/\n/g,"<br/>"),H(t,0,n,{},""),z(t),t}function W(e){for(var t=e.replace(/\r/g,"").split(/\n/g),n=0,i=[];n<t.length;n++)i.push({value:t[n].trim(),height:0,line:n});return i}function G(e,t,n){var i,o,a;for(i=0,o=e.length;i<o;++i)a=e[i],a[n]=f("tspan"),a[n].appendChild(Te.default.createTextNode(a.value)),a.style&&A({element:a[n],_styles:{}},a.style),a.className&&a[n].setAttribute("class",a.className),t.appendChild(a[n])}function q(){this._hasEllipsis&&this.attr({text:this._settings.text})}function $(e){var t,n,i,o,a,r,s,l=this,u=!1;q.call(l);var c=l.renderer.text(st).attr(l._styles).append(l.renderer.root),d=c.getBBox().width;if(l._getElementBBox().width>e)for(e-d<0?e=0:e-=d,t=J(l.element,l._texts,e),n=0,i=t.length;n<i;++n)if(o=t[n].parts,1!==t[n].commonLength)for(a=0,r=o.length;a<r;++a)s=o[a],(0,Ve.isDefined)(s.endIndex)?(le(s,s.endIndex),u=!0):s.startBox>e&&ue(s);return c.remove(),l._hasEllipsis=u,u}function K(e){var t;return e&&(t=e.cloneNode(),t.removeAttribute("y"),t.removeAttribute("x")),t||e}function U(e){for(var t=Te.default.querySelectorAll(e,"title"),n=0;n<t.length;n++)e.removeChild(t[n]);return t}function Y(e){var t=U(e);return function(){for(var n=0;n<t.length;n++)e.appendChild(t[n])}}function Z(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this,o=[],a=!1,r=!1,s=e;q.call(i);var l=Y(this.element),u=i.renderer.text(st).attr(i._styles).append(i.renderer.root),c=u.getBBox().width,d=i._getElementBBox(),h=d.width,f=d.height;return(h||f)&&(h>e||t&&f>t)&&(e-c<0?s=0:s-=c,o=se(i.element,i._texts,e,s,n,t),o=re(o,s,n,t,parseFloat(this._getLineHeight())),this._texts=o.reduce(function(e,t){return e.concat(t.parts)},[]).filter(function(e){return""!==e.value}).map(function(e){return e.stroke&&e.tspan.parentNode.appendChild(e.stroke),e}).map(function(e){return e.tspan.parentNode.appendChild(e.tspan),e}),!this._texts.length&&(this._texts=null),a=!0,this._texts?fe(this):(this.element.textContent="",r=!0)),u.remove(),i._hasEllipsis=a,l(),{rowCount:o.length,textChanged:a,textIsEmpty:r}}function X(e,t,n,i){var o,a;if(n<=t&&i>t)for(o=1,a=e.value.length;o<=a;++o)if(n+e.tspan.getSubStringLength(0,o)>t)return o-1}function Q(e){return e.value.length?e.tspan.getSubStringLength(0,e.value.length):0}function J(e,t,n){var i,o,a,r,s,l=[];if(t)for(i=0,o=t.length;i<o;++i)a=t[i],l[a.line]?(a.startBox=r,l[a.line].parts.push(a),l[a.line].commonLength+=a.value.length):(a.startBox=r=0,l.push({commonLength:a.value.length,parts:[a]})),s=r+a.tspan.getSubStringLength(0,a.value.length),a.endIndex=X(a,n,r,s),r=s;else a={value:e.textContent,tspan:e},a.startBox=r=0,s=r+Q(a),a.endIndex=X(a,n,r,s),l=[{commonLength:e.textContent.length,parts:[a]}];return l}function ee(e,t){for(var n=e.startBox>0?[0]:[],i=e.value.split("").reduce(function(e,t,n){return" "===t&&e.push(n),e},n),o=0;void 0!==i[o+1]&&e.startBox+e.tspan.getSubStringLength(0,i[o+1])<t;)o++;return i[o]}function te(e,t){for(var n=0;n<e.value.length-1;n++)if(e.startBox+e.tspan.getSubStringLength(0,n+1)>t)return n}function ne(e,t){var n=t.hideOverflowEllipsis;return n&&0===e?"":st}function ie(e,t,n){var i=ne(t,n);if(e.value.length&&e.tspan.parentNode)for(var o=e.value.length-1;o>=1;o--){if(e.startBox+e.tspan.getSubStringLength(0,o)<t){le(e,o,i);break}1===o&&le(e,0,i)}}function oe(e,t,n,i){var o,a=e.value;"none"!==i.wordWrap&&(o="normal"===i.wordWrap?ee(e,t):te(e,t));var r,s=[];if(isFinite(o)){le(e,o,"");var l=" "===a[o]?1:0,u=a.slice(o+l);if(u.length){var c=K(e.tspan);if(c.textContent=u,e.tspan.parentNode.appendChild(c),r=g(g({},e),{value:u,startBox:0,height:0,tspan:c,stroke:K(e.stroke),endBox:c.getSubStringLength(0,u.length)}),r.stroke&&(r.stroke.textContent=u),r.endBox>t&&(s=oe(r,t,n,i),!s.length))return[]}}if(e.value.length){if("ellipsis"===i.textOverflow&&e.tspan.getSubStringLength(0,e.value.length)>t&&ie(e,n,i),"hide"===i.textOverflow&&e.tspan.getSubStringLength(0,e.value.length)>t)return[]}else e.tspan.parentNode.removeChild(e.tspan);var d=[];return r&&d.push(r),[{commonLength:a.length,parts:d}].concat(s)}function ae(e,t){return e.parts.reduce(function(e,n){return Le(e,he(n,t))},0)}function re(e,t,n,i,a){var r=n.textOverflow;if(!isFinite(i)||0===Number(i)||"none"===r)return e;var s=e.reduce(function(e,s,l,u){var c=o(e,2),d=c[0],h=c[1],f=ae(s,a);if(h+=f,h<i)d.push(s);else if(s.parts.forEach(function(e){ue(e)}),"ellipsis"===r){var p=u[l-1];if(p){var g=p.parts[p.parts.length-1];g.hasEllipsis||(0===t||g.endBox<t?le(g,g.value.length,ne(t,n)):ie(g,t,n))}}return[d,h]},[[],0]);return"hide"===r&&s[1]>i?(s[0].forEach(function(e){e.parts.forEach(function(e){ue(e)})}),[]):s[0]}function se(e,t,n,i,a){if(!t){var r=e.textContent,s={value:r,height:0,line:0};e.textContent="",G([s],e,"tspan"),t=[s]}return t.reduce(function(e,t){var r=o(e,5),s=r[0],l=r[1],u=r[2],c=r[3],d=r[4],h=s[s.length-1];if(c)return[s,l,u,c];if(h&&t.line===d){if(t.startBox=l,l>i&&"none"===a.wordWrap&&"ellipsis"===a.textOverflow)return ue(t),[s,l,u,c,d];h.parts.push(t),h.commonLength+=t.value.length}else t.startBox=l=0,s.push({commonLength:t.value.length,parts:[t]});if(t.endBox=u=l+Q(t),l=u,(0,Ve.isDefined)(n)&&u>n){var f=oe(t,n,i,a);f.length?s=s.concat(f.filter(function(e){return e.parts.length>0})):(s=[],c=!0)}return[s,l,u,c,t.line]},[[],0,0,!1,0])[0]}function le(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:st,i=e.value.substr(0,t)+n;e.value=e.tspan.textContent=i,e.stroke&&(e.stroke.textContent=i),n===st&&(e.hasEllipsis=!0)}function ue(e){e.tspan.parentNode&&e.tspan.parentNode.removeChild(e.tspan),e.stroke&&e.stroke.parentNode&&e.stroke.parentNode.removeChild(e.stroke)}function ce(e,t,n){var i,o;e._texts=null,e.clear(),null!==t&&(t=""+t,e.renderer.encodeHtml||!/<[a-z][\s\S]*>/i.test(t)&&t.indexOf("&")===-1?/\n/g.test(t)?i=W(t):n&&(i=[{value:t.trim(),height:0}]):(o=N(t),i=j(o)),i?i.length&&(e._texts=i,n&&G(i,e.element,Xe),G(i,e.element,"tspan")):e.element.appendChild(Te.default.createTextNode(t)))}function de(e,t,n){e.tspan.setAttribute(t,n),e.stroke&&e.stroke.setAttribute(t,n)}function he(e,t){return e.inherits?ge(e.height,t):e.height||t}function fe(e){if(e._texts){var t,n,i=e._texts,o=e._settings.x,a=e._getLineHeight(),r=i[0];for(de(r,"x",o),de(r,"y",e._settings.y),t=1,n=i.length;t<n;++t)if(r=i[t],parseFloat(r.height)>=0){de(r,"x",o);var s=he(r,a);de(r,"dy",s)}}}function pe(e,t){if(e._texts&&"center"!==t)for(var n=e._texts,i="left"===t?-1:1,o=Math.max.apply(Math,n.map(function(e){return Q(e)})),a=0;a<n.length;a++){var r=n[a],s=Q(r);0!==o&&o!==s&&de(r,"dx",i*Ne((o-s)/2*10)/10)}}function ge(e,t){var n=parseFloat(e),i=parseFloat(t),o=n||rt,a=i||rt;return o>a?isNaN(n)?o:e:isNaN(i)?a:t}function _e(e){if(e._texts){var t,n,i,o=e._texts,a=e._settings[Xe],r=e._settings[Qe],s=e._settings[Je]||1;for(n=0,i=o.length;n<i;++n)t=o[n].stroke,t.setAttribute(Xe,a),t.setAttribute(Qe,r),t.setAttribute(Je,s),t.setAttribute("stroke-linejoin","round")}}function me(e,t,n,i){n=n||{};var o,a,r=e.renderer,s=e._settings,l={},u={translateX:0,translateY:0,scaleX:1,scaleY:1,rotate:0,rotateX:0,rotateY:0};if(i&&(n.complete=i),r.animationEnabled()){for(o in t)a=t[o],/^(translate(X|Y)|rotate[XY]?|scale(X|Y))$/i.test(o)?(l.transform=l.transform||{from:{},to:{}},l.transform.from[o]=o in s?Number(s[o].toFixed(3)):u[o],l.transform.to[o]=a):"arc"===o||"segments"===o?l[o]=a:l[o]={from:o in s?s[o]:parseFloat(e.element.getAttribute(o)||0),to:a};r.animateElement(e,l,g(g({},r._animation),n))}else n.step&&n.step.call(e,1,1),n.complete&&n.complete.call(e),e.attr(t);return e}function ve(e,t,n){var i,o,a=this,r=a.segments||[];return a.renderer.animationEnabled()&&"points"in e&&(i=b(e.points,a.type),o=I(r,i,a.type),e.segments={from:r,to:i,end:o},delete e.points),me(a,e,t,n)}function ye(e,t,n){var i=this,o=i._settings,a={from:{},to:{}};return i.renderer.animationEnabled()&&("x"in e||"y"in e||"innerRadius"in e||"outerRadius"in e||"startAngle"in e||"endAngle"in e)&&(a.from.x=o.x||0,a.from.y=o.y||0,a.from.innerRadius=o.innerRadius||0,a.from.outerRadius=o.outerRadius||0,a.from.startAngle=o.startAngle||0,a.from.endAngle=o.endAngle||0,a.to.x="x"in e?e.x:o.x,delete e.x,a.to.y="y"in e?e.y:o.y,delete e.y,a.to.innerRadius="innerRadius"in e?e.innerRadius:o.innerRadius,delete e.innerRadius,a.to.outerRadius="outerRadius"in e?e.outerRadius:o.outerRadius,delete e.outerRadius,a.to.startAngle="startAngle"in e?e.startAngle:o.startAngle,delete e.startAngle,a.to.endAngle="endAngle"in e?e.endAngle:o.endAngle,delete e.endAngle,e.arc=a),me(i,e,t,n)}function be(e,t){var n={is:!1,name:t.name||t,after:t.after};return e?n.to=e:n.virtual=!0,n}function xe(e){yt.remove(e)}function we(e,t){var n,i;for(n=t;i=e[n];++n)i._link.i=n}function Ce(e,t){var n,i,o=t._links,a=e._link.after=e._link.after||t._linkAfter;if(a){for(n=0;(i=o[n])&&i._link.name!==a;++n);if(i)for(++n;(i=o[n])&&i._link.after===a;++n);}else n=o.length;o.splice(n,0,e),we(o,n)}function ke(e){var t,n=e._link.to._links;for(t=0;n[t]!==e;++t);n.splice(t,1),we(n,t)}function Se(e){var t=this;t.root=t._createElement("svg",{xmlns:"http://www.w3.org/2000/svg",version:"1.1",fill:at,stroke:at,"stroke-width":0}).attr({"class":e.cssClass}).css({"line-height":"normal","-ms-user-select":at,"-moz-user-select":at,"-webkit-user-select":at,"-webkit-tap-highlight-color":"rgba(0, 0, 0, 0)",display:"block",overflow:"hidden"}),t._init(),t.pathModified=!!e.pathModified,t._$container=(0,Ie.default)(e.container),t.root.append({element:e.container}),t.fixPlacement(),t._locker=0,t._backed=!1}function De(e,t){return"@"+e+"::"+t.step+":"+t.width+":"+t.opacity+":"+t.direction}t.getFuncIri=p,t.processHatchingAttrs=v,t.Renderer=Se,t.refreshPaths=t.TextSvgElement=t.RectSvgElement=t.ArcSvgElement=t.PathSvgElement=t.SvgElement=void 0;var Ie=i(n(2)),Te=i(n(11)),Oe=n(7),Ee=i(n(64)),Ae=i(n(5)),Pe=i(n(23)),Re=n(206),Me=n(907),Be=n(14),Ve=n(1),Fe=(0,Oe.getWindow)(),Le=Math.max,He=Math.min,ze=Math.floor,Ne=Math.round,je=Math.sin,We=Math.cos,Ge=Math.abs,qe=Math.PI,$e=qe/180,Ke=.5,Ue=5,Ye={"column-count":!0,"fill-opacity":!0,"flex-grow":!0,"flex-shrink":!0,"font-weight":!0,"line-height":!0,opacity:!0,order:!0,orphans:!0,widows:!0,"z-index":!0,zoom:!0},Ze="text",Xe="stroke",Qe="stroke-width",Je="stroke-opacity",et="font-size",tt="font-style",nt="font-weight",it="text-decoration",ot="textsAlignment",at="none",rt=12,st="...",lt=function(){return Object.create?function(e){return Object.create(e)}:function(e){var t=function(){};return t.prototype=e,new t}}(),ut={scaleX:1,scaleY:1,"pointer-events":null},ct=(0,Ee.default)(function(){var e=Te.default.createElement("div"),t=0;return e.style.left="-9999px",e.style.position="absolute",{backupContainer:e,backupCounter:t}}),dt=function(){var e=1;return function(){return"DevExpress_"+e++}}(),ht={full:at,lefttop:"xMinYMin",leftcenter:"xMinYMid",leftbottom:"xMinYMax",centertop:"xMidYMin",center:"xMidYMid",centerbottom:"xMidYMax",righttop:"xMaxYMin",rightcenter:"xMaxYMid",rightbottom:"xMaxYMax"},ft=function(e,t,n,i,o,a,r,s,l,u){return["M",(e+i*o).toFixed(Ue),(t-i*a).toFixed(Ue),"A",i.toFixed(Ue),i.toFixed(Ue),0,u,0,(e+i*r).toFixed(Ue),(t-i*s).toFixed(Ue),l?"M":"L",(e+n*r).toFixed(5),(t-n*s).toFixed(Ue),"A",n.toFixed(Ue),n.toFixed(Ue),0,u,1,(e+n*o).toFixed(Ue),(t-n*a).toFixed(Ue),"Z"].join(" ")},pt=function(e,t,n){var i=this;i.renderer=e,i.element=f(t),i._settings={},i._styles={},"path"===t&&(i.type=n||"line")};t.SvgElement=pt,pt.prototype={constructor:pt,_getJQElement:function(){return this._$element||(this._$element=(0,Ie.default)(this.element))},_addFixIRICallback:function(){var e=this,t=function(){P(e,"fill"),P(e,"clip-path"),P(e,"filter")};e.element._fixFuncIri=t,t.renderer=e.renderer,yt.add(t),e._addFixIRICallback=function(){}},_clearChildrenFuncIri:function(){var e=function e(t){var n;for(n=0;n<t.childNodes.length;n++)xe(t.childNodes[n]._fixFuncIri),e(t.childNodes[n])};e(this.element)},dispose:function(){return xe(this.element._fixFuncIri),this._clearChildrenFuncIri(),this._getJQElement().remove(),this},append:function(e){return(e||this.renderer.root).element.appendChild(this.element),this},remove:function(){var e=this.element;return e.parentNode&&e.parentNode.removeChild(e),this},enableLinks:function(){return this._links=[],this},virtualLink:function(e){return Ce({_link:be(null,e)},this),this},linkAfter:function(e){return this._linkAfter=e,this},linkOn:function(e,t){return this._link=be(e,t),Ce(this,e),this},linkOff:function(){return ke(this),this._link=null,this},linkAppend:function(){var e,t,n=this._link,i=n.to._links;for(e=n.i+1;(t=i[e])&&!t._link.is;++e);return this._insert(n.to,t),n.is=!0,this},_insert:function(e,t){e.element.insertBefore(this.element,t?t.element:null)},linkRemove:function(){return this.remove(),this._link.is=!1,this},clear:function(){return this._clearChildrenFuncIri(),this._getJQElement().empty(),this},toBackground:function(){var e=this.element,t=e.parentNode;return t&&t.insertBefore(e,t.firstChild),this},toForeground:function(){var e=this.element,t=e.parentNode;return t&&t.appendChild(e),this},attr:function(e){return R(this,e)},smartAttr:function(e){return this.attr(v(this,e))},css:function(e){return A(this,e)},animate:function(e,t,n){return me(this,e,t,n)},sharp:function(e,t){return this.attr({sharp:e||!0,sharpDirection:t})},_applyTransformation:function(){var e,t,n=this._settings,i=[],o=n.sharp,a=n.sharpDirection||1,r=n[Qe]%2,s=!r||"h"!==o&&o!==!0?0:Ke*a,l=!r||"v"!==o&&o!==!0?0:Ke*a;i.push("translate("+((n.translateX||0)+s)+","+((n.translateY||0)+l)+")"),n.rotate&&(e="rotateX"in n?n.rotateX:n.x,t="rotateY"in n?n.rotateY:n.y,i.push("rotate("+n.rotate+","+(e||0)+","+(t||0)+")"));var u=(0,Ve.isDefined)(n.scaleX),c=(0,Ve.isDefined)(n.scaleY);(u||c)&&i.push("scale("+(u?n.scaleX:1)+","+(c?n.scaleY:1)+")"),i.length&&this.element.setAttribute("transform",i.join(" "))},move:function(e,t,n,i){var o={};return(0,Ve.isDefined)(e)&&(o.translateX=e),(0,Ve.isDefined)(t)&&(o.translateY=t),n?this.animate(o,i):this.attr(o),this},rotate:function(e,t,n,i,o){var a={rotate:e||0};return(0,Ve.isDefined)(t)&&(a.rotateX=t),(0,Ve.isDefined)(n)&&(a.rotateY=n),i?this.animate(a,o):this.attr(a),this},_getElementBBox:function(){var e,t=this.element;try{e=t.getBBox&&t.getBBox()}catch(e){}return e||{x:0,y:0,width:t.offsetWidth||0,height:t.offsetHeight||0}},getBBox:function(){var e=this._settings,t=this._getElementBBox();return t=e.rotate?(0,Be.rotateBBox)(t,[("rotateX"in e?e.rotateX:e.x)||0,("rotateY"in e?e.rotateY:e.y)||0],-e.rotate):(0,Be.normalizeBBox)(t)},markup:function(){return(0,Re.getSvgMarkup)(this.element)},getOffset:function(){return this._getJQElement().offset()},stopAnimation:function(e){var t=this.animation;return t&&t.stop(e),this},setTitle:function(e){var t=f("title");t.textContent=e||"",this.element.appendChild(t)},removeTitle:function(){U(this.element)},data:function(e,t){var n,i=this.element;if(void 0!==t)i[e]=t;else for(n in e)i[n]=e[n];return this},on:function(){var e=[this._getJQElement()];return e.push.apply(e,arguments),Ae.default.on.apply(Ae.default,e),this},off:function(){var e=[this._getJQElement()];return e.push.apply(e,arguments),Ae.default.off.apply(Ae.default,e),this},trigger:function(){var e=[this._getJQElement()];return e.push.apply(e,arguments),Ae.default.trigger.apply(Ae.default,e),this}};var gt=function(e,t){pt.call(this,e,"path",t)};t.PathSvgElement=gt,gt.prototype=lt(pt.prototype),g(gt.prototype,{constructor:gt,attr:M,animate:ve});var _t=function(e){pt.call(this,e,"path","arc")};t.ArcSvgElement=_t,_t.prototype=lt(pt.prototype),g(_t.prototype,{constructor:_t,attr:B,animate:ye});var mt=function(e){pt.call(this,e,"rect")};t.RectSvgElement=mt,mt.prototype=lt(pt.prototype),g(mt.prototype,{constructor:mt,attr:V});var vt=function(e){pt.call(this,e,"text"),this.css({"white-space":"pre"})};t.TextSvgElement=vt,vt.prototype=lt(pt.prototype),g(vt.prototype,{constructor:vt,attr:F,css:L,applyEllipsis:$,setMaxSize:Z,restoreText:q,_getLineHeight:function(){return isNaN(parseFloat(this._styles[et]))?rt:this._styles[et]}}),Se.prototype={constructor:Se,_init:function(){var e=this;e._defs=e._createElement("defs").append(e.root),e._animationController=new Me.AnimationController(e.root.element),e._animation={enabled:!0,duration:1e3,easing:"easeOutCubic"}},fixPlacement:function(){if(Pe.default.mozilla||Pe.default.msie){var e=m(this._$container.get(0)),t=_(e.left%1,2),n=_(e.top%1,2);Pe.default.msie?this.root.css({transform:"translate("+-t+"px,"+-n+"px)"}):Pe.default.mozilla&&this.root.move(-t,-n)}},removePlacementFix:function(){(Pe.default.mozilla||Pe.default.msie)&&(Pe.default.msie?this.root.css({transform:""}):Pe.default.mozilla&&this.root.attr({transform:null}))},setOptions:function(e){var t=this;return t.rtl=!!e.rtl,t.encodeHtml=!!e.encodeHtml,t.updateAnimationOptions(e.animation||{}),t.root.attr({direction:t.rtl?"rtl":"ltr"}),t},_createElement:function(e,t,n){var i=new pt(this,e,n);return t&&i.attr(t),i},lock:function(){var e=this;return 0===e._locker&&(e._backed=!e._$container.is(":visible"),e._backed&&c(e.root)),++e._locker,e},unlock:function(){var e=this;return--e._locker,0===e._locker&&(e._backed&&(d(e.root,e._$container[0]),e.fixPlacement()),e._backed=!1),e},resize:function(e,t){return e>=0&&t>=0&&this.root.attr({width:e,height:t}),this},dispose:function(){var e,t=this;t.root.dispose(),t._defs.dispose(),t._animationController.dispose(),yt.removeByRenderer(t);for(e in t)t[e]=null;return t},animationEnabled:function(){return!!this._animation.enabled},updateAnimationOptions:function(e){return g(this._animation,e),this},stopAllAnimations:function(e){return this._animationController[e?"lock":"stop"](),this},animateElement:function(e,t,n){return this._animationController.animateElement(e,t,n),this},svg:function(){this.removePlacementFix();var e=this.root.markup();return this.fixPlacement(),e},getRootOffset:function(){return this.root.getOffset()},onEndAnimation:function(e){this._animationController.onEndAnimation(e)},rect:function(e,t,n,i){var o=new mt(this);return o.attr({x:e||0,y:t||0,width:n||0,height:i||0})},simpleRect:function(){return this._createElement("rect")},circle:function(e,t,n){return this._createElement("circle",{cx:e||0,cy:t||0,r:n||0})},g:function(){return this._createElement("g")},image:function e(t,n,i,o,a,r){var e=this._createElement("image",{x:t||0,y:n||0,width:i||0,height:o||0,preserveAspectRatio:ht[(0,Be.normalizeEnum)(r)]||at});return e.element.setAttributeNS("http://www.w3.org/1999/xlink","href",a||""),e},path:function(e,t){var n=new gt(this,t);return n.attr({points:e||[]})},arc:function(e,t,n,i,o,a){var r=new _t(this);return r.attr({x:e||0,y:t||0,innerRadius:n||0,outerRadius:i||0,startAngle:o||0,endAngle:a||0})},text:function(e,t,n){var i=new vt(this);return i.attr({text:e,x:t||0,y:n||0})},linearGradient:function(e){var t=dt(),n=this,i=n._createElement("linearGradient",{id:t}).append(n._defs);return i.id=t,e.forEach(function(e){n._createElement("stop",{offset:e.offset,"stop-color":e["stop-color"]}).append(i)}),i},pattern:function e(t,n,i){n=n||{};var o=this,a=n.step||6,r=a/2,s=1.5*a,l=i||dt(),u="right"===(0,Be.normalizeEnum)(n.direction)?"M "+r+" "+-r+" L "+-r+" "+r+" M 0 "+a+" L "+a+" 0 M "+s+" "+r+" L "+r+" "+s:"M 0 0 L "+a+" "+a+" M "+-r+" "+r+" L "+r+" "+s+" M "+r+" "+-r+" L "+s+" "+r,e=o._createElement("pattern",{id:l,width:a,height:a,patternUnits:"userSpaceOnUse"}).append(o._defs);e.id=l;o.rect(0,0,a,a).attr({fill:t,opacity:n.opacity}).append(e),new gt(this).attr({d:u,"stroke-width":n.width||1,stroke:t}).append(e);return e},_getPointsWithYOffset:function(e,t){return e.map(function(e,n){return n%2!==0?e+t:e})},clipShape:function(e,t){var n=this,i=dt(),o=n._createElement("clipPath",{id:i}).append(n._defs),a=e.apply(n,t).append(o);return a.id=i,a.remove=function(){throw"Not implemented"},a.dispose=function(){return o.dispose(),o=null,this},a},clipRect:function(e,t,n,i){return this.clipShape(this.rect,arguments)},clipCircle:function(e,t,n){return this.clipShape(this.circle,arguments)},shadowFilter:function(e,t,n,i,o,a,r,s,l){var u=this,c=dt(),d=u._createElement("filter",{id:c,x:e||0,y:t||0,width:n||0,height:i||0}).append(u._defs),h=u._createElement("feGaussianBlur",{"in":"SourceGraphic",result:"gaussianBlurResult",stdDeviation:r||0}).append(d),f=u._createElement("feOffset",{"in":"gaussianBlurResult",result:"offsetResult",dx:o||0,dy:a||0}).append(d),p=u._createElement("feFlood",{result:"floodResult","flood-color":s||"","flood-opacity":l}).append(d),g=u._createElement("feComposite",{"in":"floodResult",in2:"offsetResult",operator:"in",result:"compositeResult"}).append(d),_=u._createElement("feComposite",{"in":"SourceGraphic",in2:"compositeResult",operator:"over"}).append(d);return d.id=c,d.gaussianBlur=h,d.offset=f,d.flood=p,d.composite=g,d.finalComposite=_,d.attr=function(e){var t=this,n={},i={},o={};return"x"in e&&(n.x=e.x),"y"in e&&(n.y=e.y),"width"in e&&(n.width=e.width),"height"in e&&(n.height=e.height),R(t,n),"blur"in e&&t.gaussianBlur.attr({stdDeviation:e.blur}),"offsetX"in e&&(i.dx=e.offsetX),"offsetY"in e&&(i.dy=e.offsetY),t.offset.attr(i),"color"in e&&(o["flood-color"]=e.color),"opacity"in e&&(o["flood-opacity"]=e.opacity),t.flood.attr(o),t},d},brightFilter:function(e,t){var n=this,i=dt(),o=n._createElement("filter",{id:i}).append(n._defs),a=n._createElement("feComponentTransfer").append(o),r={type:e,slope:t};return o.id=i,n._createElement("feFuncR",r).append(a),n._createElement("feFuncG",r).append(a),n._createElement("feFuncB",r).append(a),o},getGrayScaleFilter:function(){if(this._grayScaleFilter)return this._grayScaleFilter;var e=this,t=dt(),n=e._createElement("filter",{id:t}).append(e._defs);return e._createElement("feColorMatrix").attr({type:"matrix",values:"0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0 0 0 0.6 0"}).append(n),n.id=t,e._grayScaleFilter=n,n},initHatching:function(){var e,t=this._hatchingStorage=this._hatchingStorage||{byHash:{},baseId:dt()},n=t.byHash;for(e in n)n[e].pattern.dispose();t.byHash={},t.refToHash={},t.nextId=0},lockHatching:function(e,t,n){var i,o,a=this._hatchingStorage,r=De(e,t);return a.refToHash[n]!==r&&(n&&this.releaseHatching(n),i=a.byHash[r],i||(o=this.pattern(e,t,a.baseId+"-hatching-"+a.nextId++),i=a.byHash[r]={pattern:o,count:0},a.refToHash[o.id]=r),++i.count,n=i.pattern.id),n},releaseHatching:function(e){var t=this._hatchingStorage,n=t.refToHash[e],i=t.byHash[n];i&&0===--i.count&&(i.pattern.dispose(),delete t.byHash[n],delete t.refToHash[e])}};var yt=function(){var e=[];return{add:function(t){e.push(t)},remove:function(t){e=e.filter(function(e){return e!==t})},removeByRenderer:function(t){e=e.filter(function(e){return e.renderer!==t})},fire:function(){e.forEach(function(e){e()})}}}(),bt=function(){yt.fire()};t.refreshPaths=bt},function(e,t,n){function i(){this.reset()}function o(e,t){return e!==t?(0,h.extend)({},e,t):(0,h.extend)({},e)}function a(e,t){return e!==t?e.concat(t):e.slice()}function r(e){e._totalChangesOrder=e._optionChangesOrder.concat(e._layoutChangesOrder,e._customChangesOrder)}function s(e){var t=this.prototype,n=e.code;t["_change_"+n]=e.handler,e.isThemeDependent&&t._themeDependentChanges.push(n),e.option&&(t._optionChangesMap[e.option]=n),(e.isOptionChange?t._optionChangesOrder:t._customChangesOrder).push(n),r(t)}function l(){var e=function e(){var t,n,i=e._chain.length;for(t=0;t<i;++t)n=e._chain[t].apply(this,arguments);return n};return e._chain=[],e.add=function(t){e._chain.push(t)},e.copy=function(t){e._chain=t._chain.slice()},e}function u(e,t,n){var i=e[t];i?i.add?(Object.prototype.hasOwnProperty.call(e,t)===!1&&(i=l(),i.copy(e[t])),i.add(n)):(i=l(),i.add(e[t]),i.add(n)):i=n,e[t]=i}function c(e){var t=this.prototype;t._plugins.push(e),e.fontFields&&t._fontFields.push.apply(t._fontFields,e.fontFields),e.members&&(0,h.extend)(this.prototype,e.members),e.customize&&e.customize(this),e.extenders&&Object.keys(e.extenders).forEach(function(n){var i=e.extenders[n];u(t,n,i)},this)}function d(){return new i}t.expand=u,t.changes=d,t.replaceInherit=void 0;var h=n(0),f=n(7),p=n(4),g=!(0,f.hasWindow)();i.prototype={constructor:i,add:function(e){var t,n=e.length,i=this._flags;for(t=0;t<n;++t)i[e[t]]=1},has:function(e){return this._flags[e]>0},count:function(){return Object.keys(this._flags).length},reset:function(){this._flags={}}};var _=g?function(e){var t=e.inherit;e.inherit=function(){var e=t.apply(this,arguments),n=e.prototype;return["_plugins","_eventsMap","_initialChanges","_themeDependentChanges","_optionChangesMap","_optionChangesOrder","_layoutChangesOrder","_customChangesOrder","_totalChangesOrder"].forEach(function(e){n[e]={}}),e.addPlugin=p.noop,e},e.addChange=p.noop,e.addPlugin=p.noop}:function(e){var t=e.inherit;e.inherit=function(){var e=this.prototype,n=e._plugins,i=e._fontFields,s=e._eventsMap,l=e._initialChanges,u=e._themeDependentChanges,d=e._optionChangesMap,h=e._partialOptionChangesMap,f=e._partialOptionChangesPath,p=e._optionChangesOrder,g=e._layoutChangesOrder,_=e._customChangesOrder,m=t.apply(this,arguments);return e=m.prototype,e._plugins=a(n,e._plugins),e._fontFields=a(i,e._fontFields),e._eventsMap=o(s,e._eventsMap),e._initialChanges=a(l,e._initialChanges),e._themeDependentChanges=a(u,e._themeDependentChanges),e._optionChangesMap=o(d,e._optionChangesMap),e._partialOptionChangesMap=o(h,e._partialOptionChangesMap),e._partialOptionChangesPath=o(f,e._partialOptionChangesPath),e._optionChangesOrder=a(p,e._optionChangesOrder),
e._layoutChangesOrder=a(g,e._layoutChangesOrder),e._customChangesOrder=a(_,e._customChangesOrder),r(e),m.addPlugin=c,m},e.prototype._plugins=[],e.prototype._fontFields=[],e.addChange=s,e.addPlugin=c};t.replaceInherit=_},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}t.polar=t.chart=void 0;var r=n(0),s=n(3),l=a(n(132)),u=n(202),c=n(14),d=n(1),h=u.chart.area,f=l.chart,p=l.polar,g=r.extend,_=s.each,m={};t.chart=m;var v={};t.polar=v;var y={_createLegendState:function(e,t){return{fill:e.color||t,hatching:e.hatching}},_parsePointStyle:function(e,t,n){var i=e.color||t,o=f._parsePointStyle.call(this,e,i,n);return o.fill=i,o.hatching=e.hatching,o.dashStyle=e.border&&e.border.dashStyle||"solid",delete o.r,o},_applyMarkerClipRect:function(e){e["clip-path"]=null},_setGroupsSettings:function(e,t){var n=this,i={};f._setGroupsSettings.apply(n,arguments),e&&t?i=this._getAffineCoordOptions():e||(i={scaleX:1,scaleY:1,translateX:0,translateY:0}),n._markersGroup.attr(i)},_drawPoint:function(e){e.hasAnimation=e.hasAnimation&&!e.firstDrawing,e.firstDrawing=!1,f._drawPoint.call(this,e)},_getMainColor:function(){return this._options.mainSeriesColor},_createPointStyles:function(e){var t=this,n=e.color||t._getMainColor();return{normal:t._parsePointStyle(e,n,n),hover:t._parsePointStyle(e.hoverStyle||{},n,n),selection:t._parsePointStyle(e.selectionStyle||{},n,n)}},_updatePointsVisibility:function(){var e=this._options.visible;(0,s.each)(this._points,function(t,n){n._options.visible=e})},_getOptionsForPoint:function(){return this._options},_animate:function(e){var t=this,n=function(){t._animateComplete()},i=function(e,t){var n=e.length-1;_(e||[],function(e,i){i.animate(e===n?t:void 0,i.getMarkerCoords())})};t._animatePoints(e,n,i)},getValueRangeInitialValue:h.getValueRangeInitialValue,_patchMarginOptions:function(e){return e.checkInterval=!0,e},_defaultAggregator:"sum",_defineDrawingState:function(){},usePointsToDefineAutoHiding:function(){return!1}};m.bar=g({},f,y,{_getAffineCoordOptions:function(){var e=this._options.rotated,t=e?"X":"Y",n={scaleX:e?.001:1,scaleY:e?1:.001};return n["translate"+t]=this.getValueAxis().getTranslator().translate("canvas_position_default"),n},_animatePoints:function(e,t,n){var i=this;i._markersGroup.animate({scaleX:1,scaleY:1,translateY:0,translateX:0},void 0,t),e||n(i._drawnPoints,t)},checkSeriesViewportCoord:function(e,t){if(0===this._points.length)return!1;if(e.isArgumentAxis)return!0;var n=e.getTranslator(),i=this.getViewport(),o=n.translate(i.categories?i.categories[0]:i.min),a=n.translate(i.categories?i.categories[i.categories.length-1]:i.max),r=this.getOptions().rotated,s=e.getOptions().inverted;return r&&!s||!r&&s?t>=o&&t<=a:t>=a&&t<=o},getSeriesPairCoord:function(e,t){for(var n=null,i=this._options.rotated,o=!t&&!i||t&&i,a=o?"vy":"vx",r=o?"vx":"vy",s=this.getPoints(),l=0;l<s.length;l++){var u=s[l],c=void 0;if(c=t?u.getCenterCoord()[a[1]]===e?u[r]:void 0:u[a]===e?u[r]:void 0,this._checkAxisVisibleAreaCoord(!t,c)){n=c;break}}return n}}),v.bar=g({},p,y,{_animatePoints:function(e,t,n){n(this._drawnPoints,t)},_setGroupsSettings:f._setGroupsSettings,_drawPoint:function(e,t,n){f._drawPoint.call(this,e,t,n)},_parsePointStyle:function(e){var t=y._parsePointStyle.apply(this,arguments);return t.opacity=e.opacity,t},_createGroups:f._createGroups,_setMarkerGroupSettings:function(){var e=this,t=e._createPointStyles(e._getMarkerGroupOptions()).normal;t.class="dxc-markers",e._applyMarkerClipRect(t);var n=g({},t);delete n.opacity,e._markersGroup.attr(n)},getSeriesPairCoord:function(e,t){for(var n=null,i=t?"argument":"radius",o=this.getVisiblePoints(),a=this.getArgumentAxis(),r=a.getAngles()[0],s=0;s<o.length;s++){var l=o[s],u=(0,d.isDefined)(l[i])&&(0,d.isDefined)(e[i])&&l[i].valueOf()===e[i].valueOf()?(0,c.convertPolarToXY)(a.getCenter(),r,-a.getTranslatedAngle(l.angle),l.radius):void 0;if((0,d.isDefined)(u)){n=u;break}}return n},_createLegendState:h._createLegendState})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.plugin=void 0;var o,a=n(4),r=i(n(94)),s=r.default.postCtor,l={_dataSourceLoadErrorHandler:function(){this._dataSourceChangedHandler()},_dataSourceOptions:function(){return{paginate:!1}},_updateDataSource:function(){this._refreshDataSource(),this.option("dataSource")||this._dataSourceChangedHandler()},_dataIsLoaded:function(){return!this._dataSource||this._dataSource.isLoaded()},_dataSourceItems:function(){return this._dataSource&&this._dataSource.items()}};for(o in r.default)"postCtor"!==o&&(l[o]=r.default[o]);var u={name:"data_source",init:function(){s.call(this)},dispose:a.noop,members:l};t.plugin=u},function(e,t,n){function i(e){return h[(0,c.normalizeEnum)(e)]||(0,u.isFunction)(e)&&e||l}function o(e,t){h[e]=t}function a(e){l=h[e]}function r(e,t,n){var i=void 0!==n?n:p(e),o=1-i,a=t[i],r=t[o],s=f[a],l=f[r];return{staticSide:e[2+i]-e[i],variedSide:e[2+o]-e[o],static1:i+s[0],static2:i+s[1],varied1:o+l[0],varied2:o+l[1],staticDir:a,variedDir:r}}function s(e,t,n,i,o){var a,r,s,l,u=[0,0,0,0],c=i.static1,h=i.static2,f=n[c],p=i.staticDir,g=i.staticSide,_=o.sum;for(u[i.varied1]=n[i.varied1],u[i.varied2]=n[i.varied1]+i.variedDir*o.side,a=t,r=t+o.count;a<r;++a)s=u.slice(),s[c]=f,l=d(g*e[a].value/_)||0,_-=e[a].value,g-=l,f+=p*l,s[h]=f,e[a].rect=s;n[i.varied1]=u[i.varied2]}t.getAlgorithm=i,t.addAlgorithm=o,t.setDefaultAlgorithm=a,t.buildSidesData=r,t.calculateRectangles=s,t.getStaticSideIndex=void 0;var l,u=n(1),c=n(14),d=Math.round,h={},f={};f[-1]=[2,0],f[1]=[0,2];var p=function(e){return e[2]-e[0]<e[3]-e[1]?0:1};t.getStaticSideIndex=p},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=e.ctx;n.suspend(),n.change(t),n.resume()}function a(e,t){var n,i=e.nodes,o=i&&i.length;for(n=0;n<o;++n)t.push(i[n].proxy),a(i[n],t)}function r(e,t){for(var n=t-e.value;e;)e.value+=n,e=e.parent}function s(e,t){var n=e.ctx.calculateLabelState(t);return"visible"in t&&(n.visible=!!t.visible),n}var l=i(n(113)),u=i(n(204)),c=n(0),d=n(4),h=l.default.prototype,f=u.default.prototype;h._eventsMap.onNodesInitialized={name:"nodesInitialized"},h._eventsMap.onNodesRendering={name:"nodesRendering"},h._createProxyType=function(){function e(e){var t=this;e.proxy=t,t._id=e._id,t.level=e.level,t.index=e.index,t.data=e.data}var t,n=this;e.prototype={constructor:e,getParent:function(){return t[this._id].parent.proxy||null},getChild:function(e){var n=t[this._id].nodes;return n?n[e].proxy:null},getChildrenCount:function(){var e=t[this._id].nodes;return e?e.length:0},getAllChildren:function(){var e,n=t[this._id].nodes,i=n&&n.length,o=[];for(e=0;e<i;++e)o.push(n[e].proxy);return o},getAllNodes:function(){var e=[];return a(t[this._id],e),e},isLeaf:function(){return!t[this._id].isNode()},isActive:function(){return t[this._id].isActive()},value:function(e){var n,i=t[this._id];return void 0!==e?(r(i,e>0?Number(e):0),o(i,["TILING"]),n=this):n=i.value,n},label:function(e){var n,i=t[this._id];return void 0!==e?(i.customLabel=e?String(e):null,o(i,["LABELS"]),n=this):n=i.customLabel||i.label,n},customize:function(e){var n=t[this._id];return e&&(n._custom=n._custom||{},(0,c.extend)(!0,n._custom,e),n._partialState=n._partialLabelState=null),o(n,["TILES","LABELS"]),this},resetCustomization:function(){var e=t[this._id];return e._custom=e._partialState=e._partialLabelState=null,o(e,["TILES","LABELS"]),this}},n._extendProxyType(e.prototype),n._handlers.beginBuildNodes=function(){t=n._nodes,new e(n._root)},n._handlers.buildNode=function(t){new e(t)},n._handlers.endBuildNodes=function(){n._eventTrigger("nodesInitialized",{root:n._root.proxy})}},h._extendProxyType=d.noop;var p=h._resetNodes;h._resetNodes=function(){p.call(this),this._eventTrigger("nodesRendering",{node:this._topNode.proxy})};var g=f.updateStyles;f.updateStyles=function(){var e=this;g.call(e),e._custom&&(e._partialState=!e.ctx.forceReset&&e._partialState||e.ctx.calculateState(e._custom),(0,c.extend)(!0,e.state,e._partialState))};var _=f.updateLabelStyle;f.updateLabelStyle=function(){var e=this,t=e._custom;_.call(e),t&&t.label&&(e._partialLabelState=!e.ctx.forceReset&&e._partialLabelState||s(e,t.label),e.labelState=(0,c.extend)(!0,{},e.labelState,e._partialLabelState))},h.getRootNode=function(){return this._root.proxy},h.resetNodes=function(){var e=this._context;return e.suspend(),e.change(["NODES_CREATE"]),e.resume(),this}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.fileSaver=t.MIME_TYPES=void 0;var o=i(n(2)),a=i(n(11)),r=n(7),s=i(n(5)),l=i(n(17)),u=n(1),c=n(106),d=(0,r.getWindow)(),h=(0,r.getNavigator)(),f={EXCEL:"xlsx",CSS:"css",PNG:"png",JPEG:"jpeg",GIF:"gif",SVG:"svg",PDF:"pdf"},p={CSS:"text/css",EXCEL:"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",PNG:"image/png",JPEG:"image/jpeg",GIF:"image/gif",SVG:"image/svg+xml",PDF:"application/pdf"};t.MIME_TYPES=p;var g={_revokeObjectURLTimeout:3e4,_getDataUri:function(e,t){var n=this._getMimeType(e);return"data:".concat(n,";base64,").concat(t)},_getMimeType:function(e){return p[e]||"application/octet-stream"},_linkDownloader:function(e,t){var n=a.default.createElement("a");return n.download=e,n.href=t,n.target="_blank",n},_formDownloader:function(e,t,n,i){function r(e,t){for(var n in t)e.setAttribute(n,t[n]);return e}var l={method:"post",action:e,enctype:"multipart/form-data"},u=(0,o.default)("<form>").css({display:"none"}).attr(l);u.append(r(a.default.createElement("input"),{type:"hidden",name:"fileName",value:t})),u.append(r(a.default.createElement("input"),{type:"hidden",name:"contentType",value:n})),u.append(r(a.default.createElement("input"),{type:"hidden",name:"data",value:i})),u.appendTo("body"),s.default.trigger(u,"submit"),s.default.trigger(u,"submit")&&u.remove()},_saveByProxy:function(e,t,n,i){var o=this._getMimeType(n);return this._formDownloader(e,t,o,i)},_winJSBlobSave:function(e,t,n){var i=new Windows.Storage.Pickers.FileSavePicker;i.suggestedStartLocation=Windows.Storage.Pickers.PickerLocationId.documentsLibrary;var o=f[n];if(o){var a=this._getMimeType(n);i.fileTypeChoices.insert(a,["."+o])}i.suggestedFileName=t,i.pickSaveFileAsync().then(function(t){t&&t.openAsync(Windows.Storage.FileAccessMode.readWrite).then(function(t){var n=e.msDetachStream();Windows.Storage.Streams.RandomAccessStream.copyAsync(n,t).then(function(){t.flushAsync().done(function(){n.close(),t.close()})})})})},_click:function(e){try{e.dispatchEvent(new MouseEvent("click",{cancelable:!0}))}catch(n){var t=a.default.getDocument().createEvent("MouseEvents");t.initMouseEvent("click",!0,!0,d,0,0,0,80,20,!1,!1,!1,!1,0,null),e.dispatchEvent(t)}},_saveBlobAs:function(e,t,n){var i=this;if(this._blobSaved=!1,(0,u.isDefined)(h.msSaveOrOpenBlob))h.msSaveOrOpenBlob(n,e),this._blobSaved=!0;else if((0,u.isDefined)(d.WinJS))this._winJSBlobSave(n,e,t),this._blobSaved=!0;else{var o=d.URL||d.webkitURL||d.mozURL||d.msURL||d.oURL;if((0,u.isDefined)(o)){var a=o.createObjectURL(n),r=this._linkDownloader(e,a);setTimeout(function(){o.revokeObjectURL(a),i._objectUrlRevoked=!0},this._revokeObjectURLTimeout),this._click(r)}else c.logger.warn("window.URL || window.webkitURL || window.mozURL || window.msURL || window.oURL is not defined")}},saveAs:function(e,t,n,i,o){var a=f[t];if(a&&(e+="."+a),(0,u.isDefined)(i)&&l.default.log("W0001","Export","proxyURL","19.2","This option is no longer required"),o)this._saveByProxy(i,e,t,n);else if((0,u.isFunction)(d.Blob))this._saveBlobAs(e,t,n);else if((0,u.isDefined)(i)&&!(0,u.isDefined)(h.userAgent.match(/iPad/i)))this._saveByProxy(i,e,t,n);else{(0,u.isDefined)(h.userAgent.match(/iPad/i))||l.default.log("E1034");var r=this._linkDownloader(e,this._getDataUri(t,n));this._click(r)}}};t.fileSaver=g},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.name=void 0;var o=i(n(2)),a=i(n(5)),r=n(46),s=i(n(13)),l=i(n(15)),u=i(n(66)),c=n(8),d=i(n(103)),h="dxContexMenu",f=(0,c.addNamespace)("contextmenu",h),p=(0,c.addNamespace)(d.default.name,h),g="dxcontextmenu",_=l.default.inherit({setup:function(e){var t=(0,o.default)(e);a.default.on(t,f,this._contextMenuHandler.bind(this)),(r.touch||s.default.isSimulator())&&a.default.on(t,p,this._holdHandler.bind(this))},_holdHandler:function(e){(0,c.isMouseEvent)(e)&&!s.default.isSimulator()||this._fireContextMenu(e)},_contextMenuHandler:function(e){this._fireContextMenu(e)},_fireContextMenu:function(e){return(0,c.fireEvent)({type:g,originalEvent:e})},teardown:function(e){a.default.off(e,"."+h)}});(0,u.default)(g,new _);var m=g;t.name=m},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(191),a=i(n(5)),r=i(n(65)),s=n(3),l=n(8),u=n(0),c=n(136),d="dxSwipeable",h="dx-swipeable",f={onStart:o.start,onUpdated:o.swipe,onEnd:o.end,onCancel:"dxswipecancel"},p=r.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{elastic:!0,immediate:!1,direction:"horizontal",itemSizeFunc:null,onStart:null,onUpdated:null,onEnd:null,onCancel:null})},_render:function(){this.callBase(),this.$element().addClass(h),this._attachEventHandlers()},_attachEventHandlers:function(){if(this._detachEventHandlers(),!this.option("disabled")){var e=this.NAME;this._createEventData(),(0,s.each)(f,function(t,n){var i=this._createActionByOption(t,{context:this});n=(0,l.addNamespace)(n,e),a.default.on(this.$element(),n,this._eventData,function(e){return i({event:e})})}.bind(this))}},_createEventData:function(){this._eventData={elastic:this.option("elastic"),itemSizeFunc:this.option("itemSizeFunc"),direction:this.option("direction"),immediate:this.option("immediate")}},_detachEventHandlers:function(){a.default.off(this.$element(),"."+d)},_optionChanged:function(e){switch(e.name){case"disabled":case"onStart":case"onUpdated":case"onEnd":case"onCancel":case"elastic":case"immediate":case"itemSizeFunc":case"direction":this._detachEventHandlers(),this._attachEventHandlers();break;case"rtlEnabled":break;default:this.callBase(e)}},_useTemplates:function(){return!1}});(0,c.name)(p,d);var g=p;t.default=g,e.exports=t.default},function(e,t,n){var i=n(137);n(257),n(386),n(398),n(124),n(407),n(190),n(408),n(409),n(21),n(178),n(142),n(80),n(103),n(158),n(25),n(191),n(275),e.exports=i},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){if(!e)return(new c.Deferred).resolve();var i=t.exportingAction,o=t.exportedAction,r=t.fileSavingAction,s={fileName:t.fileName,format:t.format,cancel:!1};return(0,u.isFunction)(i)&&i(s),s.cancel?(new c.Deferred).resolve():n(e,t).then(function(e){(0,u.isFunction)(o)&&o(),(0,u.isFunction)(r)&&(s.data=e,r(s)),s.cancel||a.fileSaver.saveAs(s.fileName,t.format,e,t.proxyUrl,t.forceProxy)})}t.export=o,Object.defineProperty(t,"fileSaver",{enumerable:!0,get:function(){return a.fileSaver}}),t.svg=t.pdf=t.image=t.excel=void 0;var a=n(177),r=n(335),s=n(247),l=n(347),u=n(1),c=n(6),d=i(n(239)),h=n(349),f={creator:r.ExcelCreator,getData:r.getData,formatConverter:d.default};t.excel=f;var p={creator:s.imageCreator,getData:s.getData,testFormats:s.testFormats};t.image=p;var g={getData:h.getData};t.pdf=g;var _={creator:l.svgCreator,getData:l.getData};t.svg=_},function(e,t,n){t.default=void 0;var i=n(12),o=n(7),a=(0,o.hasWindow)()?(0,o.getWindow)().WeakMap:WeakMap;a||(a=function(){var e=[],t=[];this.set=function(n,o){var a=(0,i.inArray)(n,e);a===-1?(e.push(n),t.push(o)):t[a]=o},this.get=function(n){var o=(0,i.inArray)(n,e);if(o!==-1)return t[o]},this.has=function(t){var n=(0,i.inArray)(t,e);return n!==-1},this.delete=function(n){var o=(0,i.inArray)(n,e);o!==-1&&(e.splice(o,1),t.splice(o,1))}});var r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(3),l=i(n(27)),u=function(){function e(){o(this,e),this.memory=[],this.callbacks=(0,l.default)()}return r(e,[{key:"add",value:function(e){(0,s.each)(this.memory,function(t,n){return e.apply(e,n)}),this.callbacks.add(e)}},{key:"remove",value:function(e){this.callbacks.remove(e)}},{key:"fire",value:function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];this.memory.push(t),this.callbacks.fire.apply(this.callbacks,t)}}]),e}();t.default=u,e.exports=t.default},function(e,t,n){t.parseHeight=t.getVisibleHeight=t.getVerticalOffsets=t.addOffsetToMinHeight=t.addOffsetToMaxHeight=t.getElementBoxParams=t.getSize=void 0;var i=n(7),o=n(1),a=(0,i.getWindow)(),r=["auto","none","inherit","initial"],s=function(e,t){var n=0;return t.forEach(function(t){n+=parseFloat(e[t])||0}),n},l=function(e,t){var n="width"===e?"Left":"Top",i="width"===e?"Right":"Bottom";return{padding:s(t,["padding"+n,"padding"+i]),border:s(t,["border"+n+"Width","border"+i+"Width"]),margin:s(t,["margin"+n,"margin"+i])}};t.getElementBoxParams=l;var u=function(e,t,n){var i=t[e];return"border-box"===t.boxSizing&&i.length&&"%"!==i[i.length-1]?n.border+n.padding:0},c=function(e,t,n){var i=a.getComputedStyle(e),o=l(t,i),r=e.getClientRects().length,s=e.getBoundingClientRect()[t],c=r?s:0;return c<=0?(c=parseFloat(i[t]||e.style[t])||0,c-=u(t,i,o)):c-=o.padding+o.border,n.paddings&&(c+=o.padding),n.borders&&(c+=o.border),n.margins&&(c+=o.margin),c};t.getSize=c;var d=function(e){return(0,o.isWindow)(e)?e.innerHeight:e.offsetHeight},h=function(e,t){return e.indexOf("px")>0?e=parseInt(e.replace("px","")):e.indexOf("%")>0?e=parseInt(e.replace("%",""))*d(t)/100:isNaN(e)||(e=parseInt(e)),e};t.parseHeight=h;var f=function(e,t,n){if(!e)return null;if(r.indexOf(e)>-1)return t?null:e;if((0,o.isString)(e)&&(e=h(e,n)),(0,o.isNumeric)(e))return Math.max(0,e+t);var i=t<0?" - ":" ";return"calc("+e+i+Math.abs(t)+"px)"},p=function(e,t,n){var i=f(e,t,n);return null!==i?i:"none"};t.addOffsetToMaxHeight=p;var g=function(e,t,n){var i=f(e,t,n);return null!==i?i:0};t.addOffsetToMinHeight=g;var _=function(e,t){if(!e)return 0;var n=l("height",a.getComputedStyle(e));return n.padding+n.border+(t?n.margin:0)};t.getVerticalOffsets=_;var m=function(e){if(e){var t=e.getBoundingClientRect();if(t.height)return t.height}return 0};t.getVisibleHeight=m},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.createDefaultOptionRules=t.getNestedOptionValue=t.getParentName=t.getFieldName=t.deviceMatch=t.normalizeOptions=t.convertRulesToOptions=void 0;var a=i(n(13)),r=n(1),s=n(4),l=n(0),u=n(19),c={},d=function(e){var t=a.default.current();return e.reduce(function(e,n){var i=n.device,o=n.options,a=i||{},s=(0,r.isFunction)(a)?a(t):f(t,a);return s&&(0,l.extend)(!0,e,o),e},{})};t.convertRulesToOptions=d;var h=function(e,t){return"string"!=typeof e?e:o({},e,t)};t.normalizeOptions=h;var f=function(e,t){return(0,r.isEmptyObject)(t)||(0,s.findBestMatches)(e,[t]).length>0};t.deviceMatch=f;var p=function(e){return e.substr(e.lastIndexOf(".")+1)};t.getFieldName=p;var g=function(e){return e.substr(0,e.lastIndexOf("."))};t.getParentName=g;var _=function(e,t){return c[t]=c[t]||(0,u.compileGetter)(t),c[t](e,{functionsAsIs:!0})};t.getNestedOptionValue=_;var m=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[];return e};t.createDefaultOptionRules=m},function(e,t,n){t.default=void 0;var i={};t.default=i,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.PresetCollection=t.presets=void 0;var o=i(n(117)),a=n(3),r=n(0),s=i(n(13)),l=i(n(43)),u={forward:" dx-forward",backward:" dx-backward",none:" dx-no-direction",undefined:" dx-no-direction"},c="preset_",d=o.default.inherit({ctor:function(){this.callBase.apply(this,arguments),this._registeredPresets=[],this.resetToDefaults()},_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{defaultAnimationDuration:400,defaultAnimationDelay:0,defaultStaggerAnimationDuration:300,defaultStaggerAnimationDelay:40,defaultStaggerAnimationStartDelay:500})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(e){return e.phone},options:{defaultStaggerAnimationDuration:350,defaultStaggerAnimationDelay:50,defaultStaggerAnimationStartDelay:0}},{device:function(){return s.default.current().android||s.default.real.android},options:{defaultAnimationDelay:100}}])},_getPresetOptionName:function(e){return c+e},_createAndroidSlideAnimationConfig:function(e,t){var n=this,i=function(e){return{type:"slide",delay:void 0===e.delay?n.option("defaultAnimationDelay"):e.delay,duration:void 0===e.duration?n.option("defaultAnimationDuration"):e.duration}};return{enter:function(n,o){var a=n.parent().width()*t,r=o.direction,s=i(o);return s.to={left:0,opacity:1},"forward"===r?s.from={left:a,opacity:e}:"backward"===r?s.from={left:-a,opacity:e}:s.from={left:0,opacity:0},l.default.createAnimation(n,s)},leave:function(n,o){var a=n.parent().width()*t,r=o.direction,s=i(o);return s.from={left:0,opacity:1},"forward"===r?s.to={left:-a,opacity:e}:"backward"===r?s.to={left:a,opacity:e}:s.to={left:0,opacity:0},l.default.createAnimation(n,s)}}},_createOpenDoorConfig:function(){var e=this,t=function(t){return{type:"css",extraCssClasses:"dx-opendoor-animation",delay:void 0===t.delay?e.option("defaultAnimationDelay"):t.delay,duration:void 0===t.duration?e.option("defaultAnimationDuration"):t.duration}};return{enter:function(e,n){var i=n.direction,o=t(n);return o.delay="none"===i?o.delay:o.duration,o.from="dx-enter dx-opendoor-animation"+u[i],o.to="dx-enter-active",l.default.createAnimation(e,o)},leave:function(e,n){var i=n.direction,o=t(n);return o.from="dx-leave dx-opendoor-animation"+u[i],o.to="dx-leave-active",l.default.createAnimation(e,o)}}},_createWinPopConfig:function(){var e=this,t={type:"css",extraCssClasses:"dx-win-pop-animation",duration:e.option("defaultAnimationDuration")};return{enter:function(n,i){var o=t,a=i.direction;return o.delay="none"===a?e.option("defaultAnimationDelay"):e.option("defaultAnimationDuration")/2,o.from="dx-enter dx-win-pop-animation"+u[a],o.to="dx-enter-active",l.default.createAnimation(n,o)},leave:function(n,i){var o=t,a=i.direction;return o.delay=e.option("defaultAnimationDelay"),o.from="dx-leave dx-win-pop-animation"+u[a],o.to="dx-leave-active",l.default.createAnimation(n,o)}}},resetToDefaults:function(){this.clear(),this.registerDefaultPresets(),this.applyChanges()},clear:function(e){var t=this,n=[];(0,a.each)(this._registeredPresets,function(i,o){e&&e!==o.name?n.push(o):t.option(t._getPresetOptionName(o.name),void 0)}),this._registeredPresets=n,this.applyChanges()},registerPreset:function(e,t){this._registeredPresets.push({name:e,config:t})},applyChanges:function(){var e=this,t=[];(0,a.each)(this._registeredPresets,function(n,i){var o={device:i.config.device,options:{}};o.options[e._getPresetOptionName(i.name)]=i.config.animation,t.push(o)}),this._setOptionsByDevice(t)},getPreset:function(e){for(var t=e;"string"==typeof t;)t=this.option(this._getPresetOptionName(t));return t},registerDefaultPresets:function(){this.registerPreset("pop",{animation:{extraCssClasses:"dx-android-pop-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("openDoor",{animation:this._createOpenDoorConfig()}),this.registerPreset("win-pop",{animation:this._createWinPopConfig()}),this.registerPreset("fade",{animation:{extraCssClasses:"dx-fade-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("slide",{device:function(){return s.default.current().android||s.default.real.android},animation:this._createAndroidSlideAnimationConfig(1,1)}),this.registerPreset("slide",{device:function(){return!s.default.current().android&&!s.default.real.android},animation:{extraCssClasses:"dx-slide-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("ios7-slide",{animation:{extraCssClasses:"dx-ios7-slide-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("overflow",{animation:{extraCssClasses:"dx-overflow-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("ios7-toolbar",{device:function(){return!s.default.current().android&&!s.default.real.android},animation:{extraCssClasses:"dx-ios7-toolbar-animation",delay:this.option("defaultAnimationDelay"),duration:this.option("defaultAnimationDuration")}}),this.registerPreset("ios7-toolbar",{device:function(){return s.default.current().android||s.default.real.android},animation:this._createAndroidSlideAnimationConfig(0,.4)}),this.registerPreset("stagger-fade",{animation:{extraCssClasses:"dx-fade-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-slide",{animation:{extraCssClasses:"dx-slide-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-fade-slide",{animation:{extraCssClasses:"dx-fade-slide-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-drop",{animation:{extraCssClasses:"dx-drop-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-fade-drop",{animation:{extraCssClasses:"dx-fade-drop-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-fade-rise",{animation:{extraCssClasses:"dx-fade-rise-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-3d-drop",{animation:{extraCssClasses:"dx-3d-drop-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}}),this.registerPreset("stagger-fade-zoom",{animation:{extraCssClasses:"dx-fade-zoom-animation",staggerDelay:this.option("defaultStaggerAnimationDelay"),duration:this.option("defaultStaggerAnimationDuration"),delay:this.option("defaultStaggerAnimationStartDelay")}})}});t.PresetCollection=d;var h=new d;t.presets=h},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(30),a=i(n(9)),r=i(n(65)),s=i(n(2)),l=n(12),u=n(4),c=n(0),d=n(3),h=n(29),f=n(1),p=n(7),g=i(n(5)),_=n(80),m=n(24),v=n(8),y=n(55),b="dxResizable",x="dx-resizable",w="dx-resizable-resizing",C="dx-resizable-handle",k="dx-resizable-handle-top",S="dx-resizable-handle-bottom",D="dx-resizable-handle-left",I="dx-resizable-handle-right",T="dx-resizable-handle-corner",O=(0,v.addNamespace)(_.start,b),E=(0,v.addNamespace)(_.move,b),A=(0,v.addNamespace)(_.end,b),P={left:"borderLeftWidth",top:"borderTopWidth",right:"borderRightWidth",bottom:"borderBottomWidth"},R=r.default.inherit({_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{handles:"all",step:"1",stepPrecision:"simple",area:void 0,minWidth:30,maxWidth:1/0,minHeight:30,maxHeight:1/0,onResizeStart:null,onResize:null,onResizeEnd:null,roundStepValue:!0})},_init:function(){this.callBase(),this.$element().addClass(x)},_initMarkup:function(){this.callBase(),this._renderHandles()},_render:function(){this.callBase(),this._renderActions()},_renderActions:function(){this._resizeStartAction=this._createActionByOption("onResizeStart"),this._resizeEndAction=this._createActionByOption("onResizeEnd"),this._resizeAction=this._createActionByOption("onResize")},_renderHandles:function(){var e=this;this._handles=[];var t=this.option("handles");if("none"!==t){var n="all"===t?["top","bottom","left","right"]:t.split(" ");(0,d.each)(n,function(t,n){e._renderHandle(n)}),(0,l.inArray)("bottom",n)+1&&(0,l.inArray)("right",n)+1&&this._renderHandle("corner-bottom-right"),(0,l.inArray)("bottom",n)+1&&(0,l.inArray)("left",n)+1&&this._renderHandle("corner-bottom-left"),(0,l.inArray)("top",n)+1&&(0,l.inArray)("right",n)+1&&this._renderHandle("corner-top-right"),(0,l.inArray)("top",n)+1&&(0,l.inArray)("left",n)+1&&this._renderHandle("corner-top-left"),this._attachEventHandlers()}},_renderHandle:function(e){var t=(0,s.default)("<div>").addClass(C).addClass(C+"-"+e).appendTo(this.$element());this._handles.push(t)},_attachEventHandlers:function(){if(!this.option("disabled")){var e={};e[O]=this._dragStartHandler.bind(this),e[E]=this._dragHandler.bind(this),e[A]=this._dragEndHandler.bind(this),this._handles.forEach(function(t){g.default.on(t,e,{direction:"both",immediate:!0})})}},_detachEventHandlers:function(){this._handles.forEach(function(e){g.default.off(e)})},_toggleEventHandlers:function(e){e?this._attachEventHandlers():this._detachEventHandlers()},_dragStartHandler:function(e){var t=this.$element();if(t.is(".dx-state-disabled, .dx-state-disabled *"))return void(e.cancel=!0);this._toggleResizingClass(!0),this._movingSides=this._getMovingSides(e),this._elementLocation=(0,o.locate)(t);var n=(0,m.getBoundingRect)(t.get(0));this._elementSize={width:n.width,height:n.height},this._renderDragOffsets(e),this._resizeStartAction({event:e,width:this._elementSize.width,height:this._elementSize.height,handles:this._movingSides}),e.targetElements=null},_toggleResizingClass:function(e){this.$element().toggleClass(w,e)},_renderDragOffsets:function(e){var t=this._getArea();if(t){var n=(0,s.default)(e.target).closest("."+C),i=n.outerWidth(),o=n.outerHeight(),a=n.offset(),r=t.offset,l=this._getAreaScrollOffset();e.maxLeftOffset=a.left-r.left-l.scrollX,e.maxRightOffset=r.left+t.width-a.left-i+l.scrollX,e.maxTopOffset=a.top-r.top-l.scrollY,e.maxBottomOffset=r.top+t.height-a.top-o+l.scrollY}},_getBorderWidth:function(e,t){if((0,f.isWindow)(e.get(0)))return 0;var n=e.css(P[t]);return parseInt(n)||0},_dragHandler:function(e){var t=this.$element(),n=this._movingSides,i=this._elementLocation,a=this._elementSize,r=this._getOffset(e),s=a.width+r.x*(n.left?-1:1),l=a.height+r.y*(n.top?-1:1);(r.x||"strict"===this.option("stepPrecision"))&&this._renderWidth(s),(r.y||"strict"===this.option("stepPrecision"))&&this._renderHeight(l);var u=(0,m.getBoundingRect)(t.get(0)),c=r.y-((u.height||l)-l),d=r.x-((u.width||s)-s);(0,o.move)(t,{top:i.top+(n.top?c:0),left:i.left+(n.left?d:0)}),this._resizeAction({event:e,width:this.option("width")||s,height:this.option("height")||l,handles:this._movingSides}),(0,y.triggerResizeEvent)(t)},_getOffset:function(e){
var t=e.offset,n=(0,u.pairToObject)(this.option("step"),!this.option("roundStepValue")),i=this._getMovingSides(e),o="strict"===this.option("stepPrecision");return i.left||i.right||(t.x=0),i.top||i.bottom||(t.y=0),o?this._getStrictOffset(t,n,i):this._getSimpleOffset(t,n)},_getSimpleOffset:function(e,t){return{x:e.x-e.x%t.h,y:e.y-e.y%t.v}},_getStrictOffset:function(e,t,n){var i=this._elementLocation,o=this._elementSize,a=n.left?i.left:i.left+o.width,r=n.top?i.top:i.top+o.height,s=(a+e.x)%t.h,l=(r+e.y)%t.v,u=Math.sign||function(e){return e=+e,0===e||isNaN(e)?e:e>0?1:-1},c=function(e,t){return(1+.2*u(t))%1*e},d=function(e,t){return Math.abs(e)<.2*t},h=e.x-s,f=e.y-l;return s>c(t.h,e.x)&&(h+=t.h),l>c(t.v,e.y)&&(f+=t.v),{x:!n.left&&!n.right||d(e.x,t.h)?0:h,y:!n.top&&!n.bottom||d(e.y,t.v)?0:f}},_getMovingSides:function(e){var t=(0,s.default)(e.target),n=t.hasClass(T+"-top-left"),i=t.hasClass(T+"-top-right"),o=t.hasClass(T+"-bottom-left"),a=t.hasClass(T+"-bottom-right");return{top:t.hasClass(k)||n||i,left:t.hasClass(D)||n||o,bottom:t.hasClass(S)||o||a,right:t.hasClass(I)||i||a}},_getArea:function(){var e=this.option("area");return(0,f.isFunction)(e)&&(e=e.call(this)),(0,f.isPlainObject)(e)?this._getAreaFromObject(e):this._getAreaFromElement(e)},_getAreaScrollOffset:function(){var e=this.option("area"),t=!(0,f.isFunction)(e)&&!(0,f.isPlainObject)(e),n={scrollY:0,scrollX:0};if(t){var i=(0,s.default)(e)[0];(0,f.isWindow)(i)&&(n.scrollX=i.pageXOffset,n.scrollY=i.pageYOffset)}return n},_getAreaFromObject:function(e){var t={width:e.right-e.left,height:e.bottom-e.top,offset:{left:e.left,top:e.top}};return this._correctAreaGeometry(t),t},_getAreaFromElement:function(e){var t,n=(0,s.default)(e);return n.length&&(t={width:n.innerWidth(),height:n.innerHeight(),offset:(0,c.extend)({top:0,left:0},(0,f.isWindow)(n[0])?{}:n.offset())},this._correctAreaGeometry(t,n)),t},_correctAreaGeometry:function(e,t){var n=t?this._getBorderWidth(t,"left"):0,i=t?this._getBorderWidth(t,"top"):0;e.offset.left+=n+this._getBorderWidth(this.$element(),"left"),e.offset.top+=i+this._getBorderWidth(this.$element(),"top"),e.width-=this.$element().outerWidth()-this.$element().innerWidth(),e.height-=this.$element().outerHeight()-this.$element().innerHeight()},_dragEndHandler:function(e){var t=this.$element();this._resizeEndAction({event:e,width:t.outerWidth(),height:t.outerHeight(),handles:this._movingSides}),this._toggleResizingClass(!1)},_renderWidth:function(e){this.option("width",(0,h.fitIntoRange)(e,this.option("minWidth"),this.option("maxWidth")))},_renderHeight:function(e){this.option("height",(0,h.fitIntoRange)(e,this.option("minHeight"),this.option("maxHeight")))},_optionChanged:function(e){switch(e.name){case"disabled":this._toggleEventHandlers(!e.value),this.callBase(e);break;case"handles":this._invalidate();break;case"minWidth":case"maxWidth":(0,p.hasWindow)()&&this._renderWidth(this.$element().outerWidth());break;case"minHeight":case"maxHeight":(0,p.hasWindow)()&&this._renderHeight(this.$element().outerHeight());break;case"onResize":case"onResizeStart":case"onResizeEnd":this._renderActions();break;case"area":case"stepPrecision":case"step":case"roundStepValue":break;default:this.callBase(e)}},_clean:function(){this.$element().find("."+C).remove()},_useTemplates:function(){return!1}});(0,a.default)(b,R);var M=R;t.default=M,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=i(n(122));a.default&&(o=a.default.module("dx",[]));var r=o;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}n(124);var a=i(n(72)),r=i(n(45)),s=i(n(22));if(n(72),a.default&&a.default.formatNumber){var l={main:{en:{identity:{version:{_cldrVersion:"28",_number:"$Revision: 11972 $"},language:"en"},numbers:{defaultNumberingSystem:"latn",otherNumberingSystems:{"native":"latn"},minimumGroupingDigits:"1","symbols-numberSystem-latn":{decimal:".",group:",",list:";",percentSign:"%",plusSign:"+",minusSign:"-",exponential:"E",superscriptingExponent:"\xd7",perMille:"\u2030",infinity:"\u221e",nan:"NaN",timeSeparator:":"},"decimalFormats-numberSystem-latn":{standard:"#,##0.###","long":{decimalFormat:{"1000-count-one":"0 thousand","1000-count-other":"0 thousand","10000-count-one":"00 thousand","10000-count-other":"00 thousand","100000-count-one":"000 thousand","100000-count-other":"000 thousand","1000000-count-one":"0 million","1000000-count-other":"0 million","10000000-count-one":"00 million","10000000-count-other":"00 million","100000000-count-one":"000 million","100000000-count-other":"000 million","1000000000-count-one":"0 billion","1000000000-count-other":"0 billion","10000000000-count-one":"00 billion","10000000000-count-other":"00 billion","100000000000-count-one":"000 billion","100000000000-count-other":"000 billion","1000000000000-count-one":"0 trillion","1000000000000-count-other":"0 trillion","10000000000000-count-one":"00 trillion","10000000000000-count-other":"00 trillion","100000000000000-count-one":"000 trillion","100000000000000-count-other":"000 trillion"}},"short":{decimalFormat:{"1000-count-one":"0K","1000-count-other":"0K","10000-count-one":"00K","10000-count-other":"00K","100000-count-one":"000K","100000-count-other":"000K","1000000-count-one":"0M","1000000-count-other":"0M","10000000-count-one":"00M","10000000-count-other":"00M","100000000-count-one":"000M","100000000-count-other":"000M","1000000000-count-one":"0B","1000000000-count-other":"0B","10000000000-count-one":"00B","10000000000-count-other":"00B","100000000000-count-one":"000B","100000000000-count-other":"000B","1000000000000-count-one":"0T","1000000000000-count-other":"0T","10000000000000-count-one":"00T","10000000000000-count-other":"00T","100000000000000-count-one":"000T","100000000000000-count-other":"000T"}}},"scientificFormats-numberSystem-latn":{standard:"#E0"},"percentFormats-numberSystem-latn":{standard:"#,##0%"},"currencyFormats-numberSystem-latn":{currencySpacing:{beforeCurrency:{currencyMatch:"[:^S:]",surroundingMatch:"[:digit:]",insertBetween:"\xa0"},afterCurrency:{currencyMatch:"[:^S:]",surroundingMatch:"[:digit:]",insertBetween:"\xa0"}},standard:"\xa4#,##0.00",accounting:"\xa4#,##0.00;(\xa4#,##0.00)","short":{standard:{"1000-count-one":"\xa40K","1000-count-other":"\xa40K","10000-count-one":"\xa400K","10000-count-other":"\xa400K","100000-count-one":"\xa4000K","100000-count-other":"\xa4000K","1000000-count-one":"\xa40M","1000000-count-other":"\xa40M","10000000-count-one":"\xa400M","10000000-count-other":"\xa400M","100000000-count-one":"\xa4000M","100000000-count-other":"\xa4000M","1000000000-count-one":"\xa40B","1000000000-count-other":"\xa40B","10000000000-count-one":"\xa400B","10000000000-count-other":"\xa400B","100000000000-count-one":"\xa4000B","100000000000-count-other":"\xa4000B","1000000000000-count-one":"\xa40T","1000000000000-count-other":"\xa40T","10000000000000-count-one":"\xa400T","10000000000000-count-other":"\xa400T","100000000000000-count-one":"\xa4000T","100000000000000-count-other":"\xa4000T"}},"unitPattern-count-one":"{0} {1}","unitPattern-count-other":"{0} {1}"},"miscPatterns-numberSystem-latn":{atLeast:"{0}+",range:"{0}\u2013{1}"}}}}};"en"===a.default.locale().locale&&(a.default.load(l),a.default.locale("en"));var u={},c=function(e){var t,n;return n="object"===o(e)?a.default.locale().locale+":"+JSON.stringify(e):a.default.locale().locale+":"+e,t=u[n],t||(t=u[n]=a.default.numberFormatter(e)),t},d={engine:function(){return"globalize"},_formatNumberCore:function(e,t,n){return"exponential"===t?this.callBase.apply(this,arguments):c(this._normalizeFormatConfig(t,n,e))(e)},_normalizeFormatConfig:function(e,t,n){var i;return i="decimal"===e?{minimumIntegerDigits:t.precision||1,useGrouping:!1,minimumFractionDigits:0,maximumFractionDigits:20,round:n<0?"ceil":"floor"}:this._getPrecisionConfig(t.precision),"percent"===e&&(i.style="percent"),i},_getPrecisionConfig:function(e){var t;return t=null===e?{minimumFractionDigits:0,maximumFractionDigits:20}:{minimumFractionDigits:e||0,maximumFractionDigits:e||0}},format:function(e,t){return"number"!=typeof e?e:(t=this._normalizeFormat(t),t&&("function"==typeof t||t.type||t.formatter)?this.callBase.apply(this,arguments):c(t)(e))},parse:function(e,t){if(e){if(t&&(t.parser||"string"==typeof t))return this.callBase.apply(this,arguments);t&&s.default.log("W0011");var n=a.default.parseNumber(e);return isNaN(n)&&(n=this.callBase.apply(this,arguments)),n}}};r.default.resetInjection(),r.default.inject(d)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.end=t.start=t.swipe=void 0;var o=n(8),a=i(n(163)),r=i(n(99)),s="dxswipestart";t.start=s;var l="dxswipe";t.swipe=l;var u="dxswipeend";t.end=u;var c={defaultItemSizeFunc:function(){return this.getElement().width()},getBounds:function(){return[this._maxLeftOffset,this._maxRightOffset]},calcOffsetRatio:function(e){var t=(0,o.eventData)(e);return(t.x-(this._savedEventData&&this._savedEventData.x||0))/this._itemSizeFunc().call(this,e)},isFastSwipe:function(e){var t=(0,o.eventData)(e);return this.FAST_SWIPE_SPEED_LIMIT*Math.abs(t.x-this._tickData.x)>=t.time-this._tickData.time}},d={defaultItemSizeFunc:function(){return this.getElement().height()},getBounds:function(){return[this._maxTopOffset,this._maxBottomOffset]},calcOffsetRatio:function(e){var t=(0,o.eventData)(e);return(t.y-(this._savedEventData&&this._savedEventData.y||0))/this._itemSizeFunc().call(this,e)},isFastSwipe:function(e){var t=(0,o.eventData)(e);return this.FAST_SWIPE_SPEED_LIMIT*Math.abs(t.y-this._tickData.y)>=t.time-this._tickData.time}},h={horizontal:c,vertical:d},f=a.default.inherit({TICK_INTERVAL:300,FAST_SWIPE_SPEED_LIMIT:10,ctor:function(e){this.callBase(e),this.direction="horizontal",this.elastic=!0},_getStrategy:function(){return h[this.direction]},_defaultItemSizeFunc:function(){return this._getStrategy().defaultItemSizeFunc.call(this)},_itemSizeFunc:function(){return this.itemSizeFunc||this._defaultItemSizeFunc},_init:function(e){this._tickData=(0,o.eventData)(e)},_start:function(e){this._savedEventData=(0,o.eventData)(e),e=this._fireEvent(s,e),e.cancel||(this._maxLeftOffset=e.maxLeftOffset,this._maxRightOffset=e.maxRightOffset,this._maxTopOffset=e.maxTopOffset,this._maxBottomOffset=e.maxBottomOffset)},_move:function(e){var t=this._getStrategy(),n=(0,o.eventData)(e),i=t.calcOffsetRatio.call(this,e);i=this._fitOffset(i,this.elastic),n.time-this._tickData.time>this.TICK_INTERVAL&&(this._tickData=n),this._fireEvent(l,e,{offset:i}),e.preventDefault()},_end:function(e){var t=this._getStrategy(),n=t.calcOffsetRatio.call(this,e),i=t.isFastSwipe.call(this,e),o=n,a=this._calcTargetOffset(n,i);o=this._fitOffset(o,this.elastic),a=this._fitOffset(a,!1),this._fireEvent(u,e,{offset:o,targetOffset:a})},_fitOffset:function(e,t){var n=this._getStrategy(),i=n.getBounds.call(this);return e<-i[0]?t?(-2*i[0]+e)/3:-i[0]:e>i[1]?t?(2*i[1]+e)/3:i[1]:e},_calcTargetOffset:function(e,t){var n;return t?(n=Math.ceil(Math.abs(e)),e<0&&(n=-n)):n=Math.round(e),n}});(0,r.default)({emitter:f,events:[s,l,u]})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(35)),s=i(n(9)),l=n(4),u=n(75),c=n(3),d=n(1),h=n(0),f=n(18),p=i(n(17)),g=i(n(84)),_=n(24),m=i(n(583)),v=i(n(16)),y=i(n(10)),b=n(8),x=i(n(104)),w=n(21),C=i(n(13)),k=n(118),S=i(n(48)),D=n(7),I=n(213),T="dx-dropdowneditor",O="dx-dropdowneditor-input-wrapper",E="dx-dropdowneditor-icon",A="dx-dropdowneditor-overlay",P="dx-dropdowneditor-overlay-flipped",R="dx-dropdowneditor-active",M="dx-dropdowneditor-field-clickable",B="dx-dropdowneditor-field-template-wrapper",V="ios"===C.default.current().platform,F=x.default.inherit({_supportedKeys:function(){return(0,h.extend)({},this.callBase(),{tab:function(e){if(this.option("opened")){if("instantly"===this.option("applyValueMode"))return void this.close();var t=e.shiftKey?this._getLastPopupElement():this._getFirstPopupElement();t&&a.default.trigger(t,"focus"),e.preventDefault()}},escape:function(e){return this.option("opened")&&e.preventDefault(),this.close(),!0},upArrow:function(e){return!(!(0,b.isCommandKeyPressed)(e)&&(e.preventDefault(),e.stopPropagation(),e.altKey))||(this.close(),!1)},downArrow:function(e){return!(!(0,b.isCommandKeyPressed)(e)&&(e.preventDefault(),e.stopPropagation(),e.altKey))||(this._validatedOpening(),!1)},enter:function(e){return this.option("opened")&&(e.preventDefault(),this._valueChangeEventHandler(e)),!0}})},_getDefaultButtons:function(){return this.callBase().concat([{name:"dropDown",Ctor:m.default}])},_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{value:null,onOpened:null,onClosed:null,opened:!1,acceptCustomValue:!0,applyValueMode:"instantly",deferRendering:!0,activeStateEnabled:!0,dropDownButtonTemplate:"dropDownButton",fieldTemplate:null,openOnFieldClick:!1,showDropDownButton:!0,buttons:void 0,dropDownOptions:{showTitle:!1},popupPosition:this._getDefaultPopupPosition(),onPopupInitialized:null,applyButtonText:y.default.format("OK"),cancelButtonText:y.default.format("Cancel"),buttonsLocation:"default",useHiddenSubmitElement:!1})},_getDefaultPopupPosition:function(e){var t=(0,_.getDefaultAlignment)(e);return{offset:{h:0,v:-1},my:t+" top",at:t+" bottom",collision:"flip flip"}},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(e){var t="generic"===e.platform;return t},options:{popupPosition:{offset:{v:0}}}}])},_inputWrapper:function(){return this.$element().find("."+O)},_init:function(){this.callBase(),this._initVisibilityActions(),this._initPopupInitializedAction(),this._updatePopupPosition(this.option("rtlEnabled")),this._options.cache("dropDownOptions",this.option("dropDownOptions"))},_updatePopupPosition:function(e){var t=this._getDefaultPopupPosition(e),n=t.my,i=t.at,o=this.option("popupPosition");this.option("popupPosition",(0,h.extend)({},o,{my:n,at:i}))},_initVisibilityActions:function(){this._openAction=this._createActionByOption("onOpened",{excludeValidators:["disabled","readOnly"]}),this._closeAction=this._createActionByOption("onClosed",{excludeValidators:["disabled","readOnly"]})},_initPopupInitializedAction:function(){this._popupInitializedAction=this._createActionByOption("onPopupInitialized",{excludeValidators:["disabled","readOnly"]})},_initMarkup:function(){this._renderSubmitElement(),this.callBase(),this.$element().addClass(T),this.setAria("role","combobox")},_render:function(){this.callBase(),this._renderOpenHandler(),this._attachFocusOutHandler(),this._renderOpenedState()},_renderContentImpl:function(){this.option("deferRendering")||this._createPopup()},_renderInput:function(){this.callBase(),this.$element().wrapInner((0,o.default)("<div>").addClass(O)),this._$container=this.$element().children().eq(0),this._setDefaultAria()},_setDefaultAria:function(){this.setAria({haspopup:"true",autocomplete:"list"})},_readOnlyPropValue:function(){return!this.option("acceptCustomValue")||this.callBase()},_cleanFocusState:function(){this.callBase(),this.option("fieldTemplate")&&this._detachFocusEvents()},_getFieldTemplate:function(){return this.option("fieldTemplate")&&this._getTemplateByOption("fieldTemplate")},_renderMask:function(){this.option("fieldTemplate")||this.callBase()},_renderField:function(){var e=this._getFieldTemplate();e&&this._renderTemplatedField(e,this._fieldRenderData())},_renderPlaceholder:function(){var e=!!this._getFieldTemplate();e||this.callBase()},_renderValue:function(){this.option("useHiddenSubmitElement")&&this._setSubmitValue();var e=this.callBase();e.always(this._renderField.bind(this))},_renderTemplatedField:function(e,t){var n=this,i=(0,u.focused)(this._input()),r=this._$container;this._detachKeyboardEvents(),this._refreshButtonsContainer();var s=this._$beforeButtonsContainer&&this._$beforeButtonsContainer[0].parentNode,l=this._$afterButtonsContainer&&this._$afterButtonsContainer[0].parentNode;s&&s.removeChild(this._$beforeButtonsContainer[0]),l&&l.removeChild(this._$afterButtonsContainer[0]),this._detachFocusEvents(),r.empty();var c=(0,o.default)("<div>").addClass(B).appendTo(r);e.render({model:t,container:(0,f.getPublicElement)(c),onRendered:function(){var e=n._input();if(!e.length)throw p.default.Error("E1010");n._integrateInput(),i&&a.default.trigger(e,"focus")}}),r.prepend(this._$beforeButtonsContainer),r.append(this._$afterButtonsContainer)},_refreshButtonsContainer:function(){this._$buttonsContainer=this.$element().children().eq(0)},_integrateInput:function(){this._refreshEvents(),this._refreshValueChangeEvent(),this._renderFocusState(),this._refreshEmptinessEvent()},_refreshEmptinessEvent:function(){a.default.off(this._input(),"input blur",this._toggleEmptinessEventHandler),this._renderEmptinessEvent()},_fieldRenderData:function(){return this.option("value")},_initTemplates:function(){this._templateManager.addDefaultTemplates({dropDownButton:new k.FunctionTemplate(function(e){var t=(0,o.default)("<div>").addClass(E);(0,o.default)(e.container).append(t)})}),this.callBase()},_renderOpenHandler:function(){var e=this._inputWrapper(),t=(0,b.addNamespace)(w.name,this.NAME),n=this.option("openOnFieldClick");a.default.off(e,t),a.default.on(e,t,this._getInputClickHandler(n)),this.$element().toggleClass(M,n),n&&(this._openOnFieldClickAction=this._createAction(this._openHandler.bind(this)))},_attachFocusOutHandler:function(){var e=this;V&&(this._detachFocusOutEvents(),a.default.on(this._inputWrapper(),(0,b.addNamespace)("focusout",this.NAME),function(t){var n=t.relatedTarget,i=e.content?(0,o.default)(e.content()).closest("."+A):e._$popup;if(n&&e.option("opened")){var a=0===(0,o.default)(n).closest("."+A,i).length;a&&e.close()}}))},_detachFocusOutEvents:function(){V&&a.default.off(this._inputWrapper(),(0,b.addNamespace)("focusout",this.NAME))},_getInputClickHandler:function(e){var t=this;return e?function(e){t._executeOpenAction(e)}:function(e){t._focusInput()}},_openHandler:function(){this._toggleOpenState()},_executeOpenAction:function(e){this._openOnFieldClickAction({event:e})},_keyboardEventBindingTarget:function(){return this._input()},_focusInput:function(){return!this.option("disabled")&&(this.option("focusStateEnabled")&&!(0,u.focused)(this._input())&&a.default.trigger(this._input(),"focus"),!0)},_toggleOpenState:function(e){this._focusInput()&&(this.option("readOnly")||(e=arguments.length?e:!this.option("opened"),this.option("opened",e)))},_renderOpenedState:function(){var e=this.option("opened");e&&this._createPopup(),this.$element().toggleClass(R,e),this._setPopupOption("visible",e),this.setAria({expanded:e}),this.setAria("owns",(e||void 0)&&this._popupContentId,this.$element())},_createPopup:function(){this._$popup||(this._$popup=(0,o.default)("<div>").addClass(A).addClass(this.option("customOverlayCssClass")).appendTo(this.$element()),this._renderPopup(),this._renderPopupContent())},_renderPopupContent:l.noop,_renderPopup:function(){var e=(0,h.extend)(this._popupConfig(),this._options.cache("dropDownOptions"));this._popup=this._createComponent(this._$popup,S.default,e),this._popup.on({showing:this._popupShowingHandler.bind(this),shown:this._popupShownHandler.bind(this),hiding:this._popupHidingHandler.bind(this),hidden:this._popupHiddenHandler.bind(this),contentReady:this._contentReadyHandler.bind(this)}),this._contentReadyHandler(),this._setPopupContentId(this._popup.$content()),this._bindInnerWidgetOptions(this._popup,"dropDownOptions")},_setPopupContentId:function(e){this._popupContentId="dx-"+new r.default,this.setAria("id",this._popupContentId,e)},_contentReadyHandler:l.noop,_popupConfig:function(){var e=this;return{onInitialized:this._popupInitializedHandler(),position:(0,h.extend)(this.option("popupPosition"),{of:this.$element()}),showTitle:this.option("dropDownOptions.showTitle"),width:function(){return(0,I.getElementWidth)(e.$element())},height:"auto",shading:!1,closeOnTargetScroll:!0,closeOnOutsideClick:this._closeOutsideDropDownHandler.bind(this),animation:{show:{type:"fade",duration:0,from:0,to:1},hide:{type:"fade",duration:400,from:1,to:0}},deferRendering:!1,focusStateEnabled:!1,showCloseButton:!1,dragEnabled:!1,toolbarItems:this._getPopupToolbarItems(),onPositioned:this._popupPositionedHandler.bind(this),fullScreen:!1,contentTemplate:null}},_popupInitializedHandler:function(){var e=this;if(this.option("onPopupInitialized"))return function(t){e._popupInitializedAction({popup:t.component})}},_dimensionChanged:function(){var e=this,t=(0,I.getSizeValue)(this.option("dropDownOptions.width"));void 0===t&&this._setPopupOption("width",function(){return(0,I.getElementWidth)(e.$element())})},_popupPositionedHandler:function(e){e.position&&this._popup.overlayContent().toggleClass(P,e.position.v.flip)},_popupShowingHandler:l.noop,_popupHidingHandler:function(){this.option("opened",!1)},_popupShownHandler:function(){var e;this._openAction(),null===(e=this._validationMessage)||void 0===e?void 0:e.option("positionRequest",this._getValidationMessagePositionRequest())},_popupHiddenHandler:function(){var e;this._closeAction(),null===(e=this._validationMessage)||void 0===e?void 0:e.option("positionRequest",this._getValidationMessagePositionRequest())},_getValidationMessagePositionRequest:function(){var e="below";if(this._popup&&this._popup.option("visible")){var t=g.default.setup(this.$element()),n=t.top,i=g.default.setup(this._popup.$content()),o=i.top;e=n+this.option("popupPosition").offset.v>o?"below":"above"}return e},_closeOutsideDropDownHandler:function(e){var t=e.target,n=(0,o.default)(t),i=this.getButton("dropDown"),a=i&&i.$element(),r=!!n.closest(this.$element()).length,s=!!n.closest(a).length,l=!r&&!s;return l},_clean:function(){delete this._openOnFieldClickAction,this._$popup&&(this._$popup.remove(),delete this._$popup,delete this._popup),this.callBase()},_setPopupOption:function(e,t){this._setWidgetOption("_popup",arguments)},_validatedOpening:function(){this.option("readOnly")||this._toggleOpenState(!0)},_getPopupToolbarItems:function(){return"useButtons"===this.option("applyValueMode")?this._popupToolbarItemsConfig():[]},_getFirstPopupElement:function(){return this._popup._wrapper().find(".dx-popup-done.dx-button")},_getLastPopupElement:function(){return this._popup._wrapper().find(".dx-popup-cancel.dx-button")},_popupElementTabHandler:function(e){var t=(0,o.default)(e.currentTarget);(e.shiftKey&&t.is(this._getFirstPopupElement())||!e.shiftKey&&t.is(this._getLastPopupElement()))&&(a.default.trigger(this._input(),"focus"),e.preventDefault())},_popupElementEscHandler:function(){a.default.trigger(this._input(),"focus"),this.close()},_popupButtonInitializedHandler:function(e){e.component.registerKeyHandler("tab",this._popupElementTabHandler.bind(this)),e.component.registerKeyHandler("escape",this._popupElementEscHandler.bind(this))},_popupToolbarItemsConfig:function(){var e=[{shortcut:"done",options:{onClick:this._applyButtonHandler.bind(this),text:this.option("applyButtonText"),onInitialized:this._popupButtonInitializedHandler.bind(this)}},{shortcut:"cancel",options:{onClick:this._cancelButtonHandler.bind(this),text:this.option("cancelButtonText"),onInitialized:this._popupButtonInitializedHandler.bind(this)}}];return this._applyButtonsLocation(e)},_applyButtonsLocation:function(e){var t=this.option("buttonsLocation"),n=e;if("default"!==t){var i=(0,l.splitPair)(t);(0,c.each)(n,function(e,t){(0,h.extend)(t,{toolbar:i[0],location:i[1]})})}return n},_applyButtonHandler:function(){this.close(),this.option("focusStateEnabled")&&this.focus()},_cancelButtonHandler:function(){this.close(),this.option("focusStateEnabled")&&this.focus()},_popupOptionChanged:function(e){var t=v.default.getOptionsFromContainer(e);this._setPopupOption(t);var n=Object.keys(t);n.indexOf("width")===-1&&n.indexOf("height")===-1||this._dimensionChanged()},_renderSubmitElement:function(){this.option("useHiddenSubmitElement")&&(this._$submitElement=(0,o.default)("<input>").attr("type","hidden").appendTo(this.$element()))},_setSubmitValue:function(){this._getSubmitElement().val(this.option("value"))},_getSubmitElement:function(){return this.option("useHiddenSubmitElement")?this._$submitElement:this.callBase()},_dispose:function(){this._detachFocusOutEvents(),this.callBase()},_setDeprecatedOptions:function(){this.callBase(),(0,h.extend)(this._deprecatedOptions,{showPopupTitle:{since:"20.1",alias:"dropDownOptions.showTitle"}})},_optionChanged:function(e){var t;switch(e.name){case"width":case"height":this.callBase(e),null===(t=this._popup)||void 0===t?void 0:t.repaint();break;case"opened":this._renderOpenedState();break;case"onOpened":case"onClosed":this._initVisibilityActions();break;case"onPopupInitialized":this._initPopupInitializedAction();break;case"fieldTemplate":(0,d.isDefined)(e.value)?this._renderField():this._invalidate();break;case"acceptCustomValue":case"openOnFieldClick":this._invalidate();break;case"dropDownButtonTemplate":case"showDropDownButton":this._updateButtons(["dropDown"]);break;case"dropDownOptions":this._popupOptionChanged(e),this._options.cache("dropDownOptions",this.option("dropDownOptions"));break;case"popupPosition":break;case"deferRendering":(0,D.hasWindow)()&&this._createPopup();break;case"applyValueMode":case"applyButtonText":case"cancelButtonText":case"buttonsLocation":this._setPopupOption("toolbarItems",this._getPopupToolbarItems());break;case"showPopupTitle":this._setPopupOption("showTitle",e.value);break;case"useHiddenSubmitElement":this._$submitElement&&(this._$submitElement.remove(),this._$submitElement=void 0),this._renderSubmitElement();break;case"rtlEnabled":this._updatePopupPosition(e.value),this.callBase(e);break;default:this.callBase(e)}},open:function(){this.option("opened",!0)},close:function(){this.option("opened",!1)},field:function(){return(0,f.getPublicElement)(this._input())},content:function(){return this._popup?this._popup.content():null}});(0,s.default)("dxDropDownEditor",F);var L=F;t.default=L,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(10)),s=i(n(9)),l=n(0),u=i(n(88)),c=i(n(68)),d=n(6),h=n(36),f="dx-loadpanel",p="dx-loadpanel-wrapper",g="dx-loadpanel-indicator",_="dx-loadpanel-message",m="dx-loadpanel-content",v="dx-loadpanel-content-wrapper",y="dx-loadpanel-pane-hidden",b=c.default.inherit({_supportedKeys:function(){return(0,l.extend)(this.callBase(),{escape:a.noop})},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{message:r.default.format("Loading"),width:222,height:90,animation:null,showIndicator:!0,indicatorSrc:"",showPane:!0,delay:0,templatesRenderAsynchronously:!1,hideTopOverlayHandler:null,resizeEnabled:!1,focusStateEnabled:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"generic"},options:{shadingColor:"transparent"}},{device:function(){return(0,h.isMaterial)()},options:{message:"",width:60,height:60,maxHeight:60,maxWidth:60}}])},_init:function(){this.callBase.apply(this,arguments)},_render:function(){this.callBase(),this.$element().addClass(f),this._wrapper().addClass(p)},_renderContentImpl:function(){this.callBase(),this.$content().addClass(m),this._$contentWrapper=(0,o.default)("<div>").addClass(v),this._$contentWrapper.appendTo(this._$content),this._togglePaneVisible(),this._cleanPreviousContent(),this._renderLoadIndicator(),this._renderMessage()},_show:function(){var e=this.option("delay");if(!e)return this.callBase();var t=new d.Deferred,n=this.callBase.bind(this);return this._clearShowTimeout(),this._showTimeout=setTimeout(function(){n().done(function(){t.resolve()})},e),t.promise()},_hide:function(){return this._clearShowTimeout(),this.callBase()},_clearShowTimeout:function(){clearTimeout(this._showTimeout)},_renderMessage:function(){if(this._$contentWrapper){var e=this.option("message");if(e){var t=(0,o.default)("<div>").addClass(_).text(e);this._$contentWrapper.append(t)}}},_renderLoadIndicator:function(){this._$contentWrapper&&this.option("showIndicator")&&(this._$indicator||(this._$indicator=(0,o.default)("<div>").addClass(g).appendTo(this._$contentWrapper)),this._createComponent(this._$indicator,u.default,{indicatorSrc:this.option("indicatorSrc")}))},_cleanPreviousContent:function(){this.$content().find("."+_).remove(),this.$content().find("."+g).remove(),delete this._$indicator},_togglePaneVisible:function(){this.$content().toggleClass(y,!this.option("showPane"))},_optionChanged:function(e){switch(e.name){case"delay":break;case"message":case"showIndicator":this._cleanPreviousContent(),this._renderLoadIndicator(),this._renderMessage();break;case"showPane":this._togglePaneVisible();break;case"indicatorSrc":this._renderLoadIndicator();break;default:this.callBase(e)}},_dispose:function(){this._clearShowTimeout(),this.callBase()}});(0,s.default)("dxLoadPanel",b);var x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.animation=void 0;var o=i(n(43)),a=n(40),r={moveTo:function(e){var t,n=e.$element,i=e.position,a=e.direction||"left",r={};switch(a){case"right":r.transform="translate("+i+"px, 0px)",t="custom";break;case"left":r.left=i,t="slide";break;case"top":case"bottom":r.top=i,t="slide"}o.default.animate(n,{type:t,to:r,duration:e.duration,complete:e.complete})},margin:function e(t){var n=t.$element,e=t.margin,i=t.direction||"left",r={};r["margin"+(0,a.camelize)(i,!0)]=e,o.default.animate(n,{to:r,duration:t.duration,complete:t.complete})},fade:function(e,t,n,i){o.default.animate(e,{type:"fade",to:t.to,from:t.from,duration:n,complete:i})},size:function e(t){var n=t.$element,e=t.size,i=t.direction||"left",a=t.marginTop||0,r=t.duration,s={};"right"===i||"left"===i?s.width=e:s.height=e,"bottom"===i&&(s.marginTop=a),o.default.animate(n,{to:s,duration:r,complete:t.complete})},complete:function(e){o.default.stop(e,!0)}};t.animation=r},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(453)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(1),o=n(12),a=n(3),r=["year","month","day"],s=["year","month","day","hour","minute"],l=function(e){return"date"===e||"datetime"===e},u=function(e){var t,n=[],a=["year","month","day","hour","minute","second"],u=e.headerFilter&&e.headerFilter.groupInterval,c="quarter"===u?"month":u;return l(e.dataType)&&null!==u?(n="datetime"===e.dataType?s:r,t=(0,o.inArray)(c,a),t>=0?(n=a.slice(0,t),n.push(u),n):n):(0,i.isDefined)(u)?Array.isArray(u)?u:[u]:void 0},c=function(){var e=function(e,t){var n=e.dataField||e.selector;return"search"===t&&(n=e.displayField||e.calculateDisplayValue||n),n},t=function(e){return e.getHours()+e.getMinutes()+e.getSeconds()+e.getMilliseconds()<1},n=function(e){return(0,i.isDate)(e)?[e.getFullYear(),e.getMonth(),e.getDate(),e.getHours(),e.getMinutes(),e.getSeconds()]:(0,a.map)((""+e).split("/"),function(e,t){return 1===t?Number(e)-1:Number(e)})},o=function(n,o){var a,r,s,u=this,c=e(u,o);if(Array.isArray(n)&&(0,i.isDefined)(n[0])&&(0,i.isDefined)(n[1]))return r=[c,">=",n[0]],s=[c,"<=",n[1]],l(u.dataType)&&t(n[1])&&(a=new Date(n[1].getTime()),"date"===u.dataType&&a.setDate(n[1].getDate()+1),s=[c,"<",a]),[r,"and",s]},r=function(t,i,o){var a,r,s,l=this,c=n(t),d=e(l,o);switch("headerFilter"===o?s=u(l)[c.length-1]:"datetime"===l.dataType&&(s="minute"),s){case"year":a=new Date(c[0],0,1),r=new Date(c[0]+1,0,1);break;case"month":a=new Date(c[0],c[1],1),r=new Date(c[0],c[1]+1,1);break;case"quarter":a=new Date(c[0],3*c[1],1),r=new Date(c[0],3*c[1]+3,1);break;case"hour":a=new Date(c[0],c[1],c[2],c[3]),r=new Date(c[0],c[1],c[2],c[3]+1);break;case"minute":a=new Date(c[0],c[1],c[2],c[3],c[4]),r=new Date(c[0],c[1],c[2],c[3],c[4]+1);break;case"second":
a=new Date(c[0],c[1],c[2],c[3],c[4],c[5]),r=new Date(c[0],c[1],c[2],c[3],c[4],c[5]+1);break;default:a=new Date(c[0],c[1],c[2]),r=new Date(c[0],c[1],c[2]+1)}switch(i){case"<":return[d,"<",a];case"<=":return[d,"<",r];case">":return[d,">=",r];case">=":return[d,">=",a];case"<>":return[[d,"<",a],"or",[d,">=",r]];default:return[[d,">=",a],"and",[d,"<",r]]}},s=function(t,n,o){var a=this,r=e(a,o),s=u(a);if("headerFilter"===o&&s&&(0,i.isDefined)(t)){var l=(""+t).split("/"),c=Number(l[l.length-1]),d=s[l.length-1],h=[r,">=",c],f=[r,"<",c+d],p=[h,"and",f];return p}return[r,n||"=",t]};return{defaultCalculateFilterExpression:function(t,n,a){var u=this,c=e(u,a),d=u.calculateDisplayValue&&"search"===a,h=d&&u.lookup&&u.lookup.dataType||u.dataType,f=null;if("headerFilter"!==a&&"filterBuilder"!==a||null!==t)if("string"!==h||u.lookup&&!d){if("between"===n)return o.apply(u,[t,a]);if(l(h)&&(0,i.isDefined)(t))return r.apply(u,arguments);if("number"===h)return s.apply(u,arguments);"object"!==h&&(f=[c,n||"=",t])}else f=[c,n||"contains",t];else f=[c,n||"=",null],"string"===h&&(f=[f,"="===n?"or":"and",[c,n||"=",""]]);return f},getGroupInterval:u}}();t.default=c,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}t.ColumnsView=void 0;var s=r(n(2)),l=r(n(11)),u=n(7),c=r(n(5)),d=n(39),h=r(n(25)),f=n(21),p=n(142),g=r(n(23)),_=n(4),m=n(97),v=n(18),y=n(1),b=n(24),x=a(n(3)),w=n(0),C=r(n(44)),k=r(n(37)),S=r(n(462)),D=n(6),I="scroll-container",T="group-space",O="content",E="table",A="table-fixed",P="content-fixed",R="dx-row",M="dx-group-row",B="dx-master-detail-row",V="filter-row",F="cell-updated-animation",L="0.0001px",H="dxCellHintVisible",z="dx-field-item-content",N={render:function(e){e.container.append(e.content)}},j=function(e,t){function n(e){return setTimeout(function(){i=o=null},e)}var i,o,a;c.default.on(t,"touchstart touchend",".dx-row",function(e){clearTimeout(a),"touchstart"===e.type?(i=e.target,o=e.currentTarget,a=n(1e3)):a=n()}),c.default.on(t,[f.name,p.name,h.default.down].join(" "),".dx-row",{useNative:e._isNativeClick()},e.createAction(function(t){var n=t.event;i&&(n.target=i,n.currentTarget=o),(0,s.default)(n.target).closest("a").length||(t.rowIndex=e.getRowIndex(n.currentTarget),t.rowIndex>=0&&(t.rowElement=(0,v.getPublicElement)((0,s.default)(n.currentTarget)),t.columns=e.getColumns(),n.type===h.default.down?e._rowPointerDown(t):n.type===f.name?e._rowClick(t):e._rowDblClick(t)))}))},W=function(e){return"auto"===e?"":(0,y.isNumeric)(e)?e+"px":e},G=function(e,t,n){e.style.width=e.style.maxWidth="auto"===t.width?"":n},q=function(e,t){if(e&&t){var n,i=e.attributes,o=t.attributes;for(n=0;n<i.length;n++){var a=i[n].nodeName;t.hasAttribute(a)||e.removeAttribute(a)}for(n=0;n<o.length;n++)e.setAttribute(o[n].nodeName,o[n].nodeValue)}},$=C.default.View.inherit(S.default).inherit({_createScrollableOptions:function(){var e=this,t=e.option("scrolling"),n=e.option("scrolling.useNative"),i=(0,w.extend)({pushBackValue:0},t,{direction:"both",bounceEnabled:!1,useKeyboard:!1});return void 0===n&&(n=!0),"auto"===n?(delete i.useNative,delete i.useSimulatedScrollbar):(i.useNative=!!n,i.useSimulatedScrollbar=!n),i},_updateCell:function(e,t){t.rowType&&this._cellPrepared(e,t)},_createCell:function(e){var t=e.column,n=t.alignment||(0,b.getDefaultAlignment)(this.option("rtlEnabled")),i=l.default.createElement("td");i.style.textAlign=n;var o=(0,s.default)(i);return"data"===e.rowType&&t.headerId&&!t.type&&this.component.option("showColumnHeaders")&&this.setAria("describedby",t.headerId,o),t.cssClass&&o.addClass(t.cssClass),"expand"===t.command&&(o.addClass(t.cssClass),o.addClass(this.addWidgetPrefix(T))),t.colspan>1?o.attr("colSpan",t.colspan):t.isBand||"auto"===t.visibleWidth||this.option("legacyRendering")||!this.option("columnAutoWidth")||((t.width||t.minWidth)&&(i.style.minWidth=W(t.minWidth||t.width)),t.width&&G(i,t,W(t.width))),o},_createRow:function(e){var t=(0,s.default)("<tr>").addClass(R);return this.setAria("role","row",t),t},_isAltRow:function(e){return e&&e.dataIndex%2===1},_createTable:function(e,t){var n=this,i=(0,s.default)("<table>").addClass(n.addWidgetPrefix(E)).addClass(n.addWidgetPrefix(A));if(e&&!t?(i.append(n._createColGroup(e)),g.default.safari&&i.append((0,s.default)("<thead>").append("<tr>")),n.setAria("role","presentation",i)):n.setAria("hidden",!0,i),this.setAria("role","presentation",(0,s.default)("<tbody>").appendTo(i)),t)return i;g.default.mozilla&&c.default.on(i,"mousedown","td",function(e){e.ctrlKey&&e.preventDefault()}),n.option("cellHintEnabled")&&c.default.on(i,"mousemove",".dx-row > td",this.createAction(function(e){var t=e.event,i=(0,s.default)(t.target),o=(0,s.default)(t.currentTarget),a=o.parent(),r=a.hasClass("dx-data-row"),l=a.hasClass("dx-header-row"),u=a.hasClass(M),c=a.hasClass(B),d=a.hasClass(n.addWidgetPrefix(V)),h=n._columnsController.getVisibleColumns(),f=a.data("options"),p=o.index(),_=f&&f.cells&&f.cells[p],m=_?_.column:h[p],v=g.default.msie?1:0;if(!c&&!d&&(!r||r&&m&&!m.cellTemplate)&&(!l||l&&m&&!m.headerCellTemplate)&&(!u||u&&m&&(void 0===m.groupIndex||!m.groupCellTemplate))){i.data(H)&&(i.removeAttr("title"),i.data(H,!1));var b=i[0].scrollWidth-i[0].clientWidth-v;b>0&&!(0,y.isDefined)(i.attr("title"))&&(i.attr("title",i.text()),i.data(H,!0))}}));var o=function(e){var t=(0,s.default)(e.currentTarget),i=(0,s.default)(e.target).closest("."+z),o=t.parent().data("options"),a=o&&o.cells&&o.cells[t.index()];if(t.closest("table").is(e.delegateTarget)){var r=(0,w.extend)({},a,{cellElement:(0,v.getPublicElement)(t),event:e,eventType:e.type});if(i.length){var l=i.data("dx-form-item");l.column&&(r.column=l.column,r.columnIndex=n._columnsController.getVisibleIndex(r.column.index))}return r}};return c.default.on(i,"mouseover",".dx-row > td",function(e){var t=o(e);t&&n.executeAction("onCellHoverChanged",t)}),c.default.on(i,"mouseout",".dx-row > td",function(e){var t=o(e);t&&n.executeAction("onCellHoverChanged",t)}),c.default.on(i,f.name,".dx-row > td",function(e){var t=o(e);t&&n.executeAction("onCellClick",t)}),c.default.on(i,p.name,".dx-row > td",function(e){var t=o(e);t&&n.executeAction("onCellDblClick",t)}),j(n,i),i},_isNativeClick:_.noop,_rowPointerDown:_.noop,_rowClick:_.noop,_rowDblClick:_.noop,_createColGroup:function(e){for(var t=(0,s.default)("<colgroup>"),n=0;n<e.length;n++)for(var i=e[n].colspan||1,o=0;o<i;o++)t.append(this._createCol(e[n]));return t},_createCol:function(e){var t=e.visibleWidth||e.width;"adaptiveHidden"===t&&(t=L);var n=(0,s.default)("<col>");return(0,m.setWidth)(n,t),n},renderDelayedTemplates:function(){var e=this._delayedTemplates,t=e.filter(function(e){return!e.async}),n=e.filter(function(e){return e.async});this._delayedTemplates=[],this._renderDelayedTemplatesCore(t),this._renderDelayedTemplatesCoreAsync(n)},_renderDelayedTemplatesCoreAsync:function(e){var t=this;e.length&&(0,u.getWindow)().setTimeout(function(){t._renderDelayedTemplatesCore(e,!0)})},_renderDelayedTemplatesCore:function(e,t){for(var n=new Date;e.length;){var i=e.shift(),o=i.options,a=l.default.getDocument();if(t&&!(0,s.default)(o.container).closest(a).length||i.template.render(o),t&&new Date-n>30){this._renderDelayedTemplatesCoreAsync(e);break}}!e.length&&this._delayedTemplates.length&&this.renderDelayedTemplates()},_processTemplate:function(e){var t,n=this;if(e&&e.render&&!(0,y.isRenderer)(e))t={allowRenderToDetachedContainer:e.allowRenderToDetachedContainer,render:function(t){e.render(t.container,t.model),t.deferred&&t.deferred.resolve()}};else if((0,y.isFunction)(e))t={render:function(t){var n=e((0,v.getPublicElement)(t.container),t.model);n&&(n.nodeType||(0,y.isRenderer)(n))&&t.container.append(n),t.deferred&&t.deferred.resolve()}};else{var i=(0,y.isString)(e)?e:(0,s.default)(e).attr("id");i?(n._templatesCache[i]||(n._templatesCache[i]=n.getTemplate(e)),t=n._templatesCache[i]):t=n.getTemplate(e)}return t},renderTemplate:function(e,t,n,i){var o=this,a=o._processTemplate(t,n),r=n.column,s="data"===n.rowType,l=new D.Deferred,u={container:e,model:n,deferred:l,onRendered:function(){l.resolve()}};if(a){n.component=o.component;var c=r&&(r.renderAsync&&s||o.option("renderAsync")&&(r.renderAsync!==!1&&(r.command||r.showEditorAlways)&&s||"filter"===n.rowType));!a.allowRenderToDetachedContainer&&!i||c?o._delayedTemplates.push({template:a,options:u,async:c}):a.render(u)}else l.reject();return l.promise()},_getBodies:function(e){return(0,s.default)(e).children("tbody").not(".dx-header").not(".dx-footer")},_wrapRowIfNeed:function(e,t){var n=this.option("rowTemplate")&&this._getBodies(this._tableElement||e);if(n&&n.filter("."+R).length){var i=(0,s.default)("<tbody>").addClass(t.attr("class"));return this.setAria("role","presentation",i),i.append(t)}return t},_appendRow:function(e,t,n){n=n||N,n.render({content:t,container:e})},_resizeCore:function(){var e=this._scrollLeft;e>=0&&(this._scrollLeft=0,this.scrollTo({left:e}))},_renderCore:function(e){var t=this.element().parent();t&&!t.parent().length||this.renderDelayedTemplates(e)},_renderTable:function(e){e=e||{},e.columns=this._columnsController.getVisibleColumns();var t=e.change&&e.change.changeType,n=this._createTable(e.columns,"append"===t||"prepend"===t||"update"===t);return this._renderRows(n,e),n},_renderRows:function(e,t){for(var n=this,i=n._getRows(t.change),o=t.change&&t.change.columnIndices||[],a=t.change&&t.change.changeTypes||[],r=0;r<i.length;r++)n._renderRow(e,(0,w.extend)({row:i[r],columnIndices:o[r],changeType:a[r]},t))},_renderRow:function(e,t){t.columnIndices||(t.row.cells=[]);var n=this._createRow(t.row),i=this._wrapRowIfNeed(e,n);"remove"!==t.changeType&&this._renderCells(n,t),this._appendRow(e,i);var o=(0,w.extend)({columns:t.columns},t.row);this._addWatchMethod(o,t.row),this._rowPrepared(i,o,t.row)},_needRenderCell:function(e,t){return!t||t.indexOf(e)>=0},_renderCells:function(e,t){for(var n=this,i=0,o=t.row,a=t.columns,r=0;r<a.length;r++)this._needRenderCell(r,t.columnIndices)&&n._renderCell(e,(0,w.extend)({column:a[r],columnIndex:i,value:o.values&&o.values[i],oldValue:o.oldValues&&o.oldValues[i]},t)),a[r].colspan>1?i+=a[r].colspan:i++},_updateCells:function(e,t,n){var i=e.children(),o=t.children(),a=this.option("highlightChanges"),r=this.addWidgetPrefix(F);n.forEach(function(e,t){var n=i.eq(e),s=o.eq(t);n.replaceWith(s),a&&!s.hasClass("dx-command-expand")&&s.addClass(r)}),q(e.get(0),t.get(0))},_setCellAriaAttributes:function(e,t){if("freeSpace"!==t.rowType){this.setAria("selected",!1,e),this.setAria("role","gridcell",e);var n=this._columnsController.getColumnIndexOffset(),i=t.columnIndex+n+1;this.setAria("colindex",i,e)}},_renderCell:function(e,t){var n=this._getCellOptions(t);t.columnIndices?t.row.cells&&(t.row.cells[n.columnIndex]=n):t.row.cells.push(n);var i=this._createCell(n);return this._setCellAriaAttributes(i,n),this._renderCellContent(i,n),e.get(0).appendChild(i.get(0)),i},_renderCellContent:function(e,t){var n=this,i=this._getCellTemplate(t);(0,D.when)(!i||this.renderTemplate(e,i,t)).done(function(){n._updateCell(e,t)})},_getCellTemplate:function(){},_getRows:function(){return[]},_getCellOptions:function(e){var t={column:e.column,columnIndex:e.columnIndex,rowType:e.row.rowType,isAltRow:this._isAltRow(e.row)};return this._addWatchMethod(t),t},_addWatchMethod:function(e,t){if(this.option("repaintChangesOnly")){var n=[];return t=t||e,t.watch=t.watch||function(e,i){var o=e(t.data),a=function(n){var a=e(t.data);JSON.stringify(o)!==JSON.stringify(a)&&(n&&i(a,o),o=a)};n.push(a);var r=function(){var e=n.indexOf(a);e>=0&&n.splice(e,1)};return r},t.update=t.update||function(t){t&&(this.data=e.data=t.data,this.rowIndex=e.rowIndex=t.rowIndex,this.dataIndex=e.dataIndex=t.dataIndex,this.isExpanded=e.isExpanded=t.isExpanded,e.row&&(e.row=t)),n.forEach(function(e){e(t)})},t!==e&&(e.watch=t.watch.bind(t)),e}},_cellPrepared:function(e,t){t.cellElement=(0,v.getPublicElement)((0,s.default)(e)),this.executeAction("onCellPrepared",t)},_rowPrepared:function(e,t){(0,d.data)(e.get(0),"options",t),t.rowElement=(0,v.getPublicElement)(e),this.executeAction("onRowPrepared",t)},_columnOptionChanged:function(e){var t=e.optionNames;if(k.default.checkChanges(t,["width","visibleWidth"])){var n=this._columnsController.getVisibleColumns(),i=x.map(n,function(e){var t=e.visibleWidth||e.width;return(0,y.isDefined)(t)?t:"auto"});return void this.setColumnWidths({widths:i,optionNames:t})}this._requireReady||this.render()},getCellIndex:function(e){var t=e.length?e[0].cellIndex:-1;return t},getTableElements:function(){return this._tableElement||(0,s.default)()},_getTableElement:function(){return this._tableElement},_setTableElement:function(e){this._tableElement=e},optionChanged:function(e){switch(this.callBase(e),e.name){case"cellHintEnabled":case"onCellPrepared":case"onRowPrepared":case"onCellHoverChanged":this._invalidate(!0,!0),e.handled=!0}},init:function(){var e=this;e._scrollLeft=-1,e._columnsController=e.getController("columns"),e._dataController=e.getController("data"),e._delayedTemplates=[],e._templatesCache={},e.createAction("onCellClick"),e.createAction("onRowClick"),e.createAction("onCellDblClick"),e.createAction("onRowDblClick"),e.createAction("onCellHoverChanged",{excludeValidators:["disabled","readOnly"]}),e.createAction("onCellPrepared",{excludeValidators:["disabled","readOnly"],category:"rendering"}),e.createAction("onRowPrepared",{excludeValidators:["disabled","readOnly"],category:"rendering",afterExecute:function(t){e._afterRowPrepared(t)}}),e._columnsController.columnsChanged.add(e._columnOptionChanged.bind(e)),e._dataController&&e._dataController.changed.add(e._handleDataChanged.bind(e))},_afterRowPrepared:_.noop,_handleDataChanged:function(){},callbackNames:function(){return["scrollChanged"]},scrollTo:function(e){var t=this.element(),n=t&&t.children("."+this.addWidgetPrefix(I)).not("."+this.addWidgetPrefix(P));(0,y.isDefined)(e)&&(0,y.isDefined)(e.left)&&this._scrollLeft!==e.left&&(this._scrollLeft=e.left,n&&n.scrollLeft(e.left))},_wrapTableInScrollContainer:function(e){var t=this,n=(0,s.default)("<div>");return c.default.on(n,"scroll",function(){var e=n.scrollLeft();e!==t._scrollLeft&&t.scrollChanged.fire({left:e},t.name)}),n.addClass(this.addWidgetPrefix(O)).addClass(this.addWidgetPrefix(I)).append(e).appendTo(this.element()),this.setAria("role","presentation",n),n},_updateContent:function(e){this._setTableElement(e),this._wrapTableInScrollContainer(e)},_findContentElement:_.noop,_getWidths:function(e){var t,n=[],i=this.option("legacyRendering");return e&&x.each(e,function(e,o){if(t=o.offsetWidth,o.getBoundingClientRect){var a=(0,b.getBoundingRect)(o);a.width>t-1&&(t=i?Math.ceil(a.width):a.width)}n.push(t)}),n},getColumnWidths:function(e){var t,n,i=this,o=[];if((this.option("forceApplyBindings")||_.noop)(),e=e||i._getTableElement()){t=e.children("tbody").children();for(var a=0;a<t.length;a++){var r=t.eq(a),s="none"!==r.get(0).style.display&&!r.hasClass("dx-state-invisible");if(!r.is("."+M)&&!r.is("."+B)&&s){n=r.children("td");break}}o=i._getWidths(n)}return o},getVisibleColumnIndex:function(e,t){return e},setColumnWidths:function(e){var t,n,i,o,a=e.widths,r=e.$tableElement,s=e.columns,l=e.fixed,u=this.option("columnAutoWidth"),c=this.option("legacyRendering");if(r=r||this._getTableElement(),r&&r.length&&a){o=0,t=r.children("colgroup").children("col"),(0,m.setWidth)(t,"auto"),s=s||this.getColumns(null,r);for(var d=0;d<s.length;d++){if(!c&&u&&!l&&(n=s[d].width,n&&!s[d].command)){n=s[d].visibleWidth||n,n=W(n),i=W(s[d].minWidth||n);for(var h=h||r.children().children(".dx-row").not("."+M).not("."+B),f=0;f<h.length;f++){var p=this.getVisibleColumnIndex(d,f),g=h[f].cells[p];g&&(G(g,s[d],n),g.style.minWidth=i)}}s[d].colspan?o+=s[d].colspan:(n=a[o],"adaptiveHidden"===n&&(n=L),"number"==typeof n&&(n=n.toFixed(3)+"px"),(0,m.setWidth)(t.eq(o),(0,y.isDefined)(n)?n:"auto"),o++)}}},getCellElements:function(e){return this._getCellElementsCore(e)},_getCellElementsCore:function(e){var t=this._getRowElements().eq(e);return t.children()},_getCellElement:function(e,t){var n,i=this,o=i.getCellElements(e),a=i._getVisibleColumnIndex(o,e,t);if(o.length&&a>=0&&(n=o.eq(a)),n&&n.length)return n},_getRowElement:function(e){var t=this,n=(0,s.default)(),i=t.getTableElements();if(x.each(i,function(i,o){n=n.add(t._getRowElements((0,s.default)(o)).eq(e))}),n.length)return n},getCellElement:function(e,t){return(0,v.getPublicElement)(this._getCellElement(e,t))},getRowElement:function(e){var t=this._getRowElement(e),n=[];if(t&&!(0,v.getPublicElement)(t).get)for(var i=0;i<t.length;i++)n.push(t[i]);else n=t;return n},_getVisibleColumnIndex:function(e,t,n){if((0,y.isString)(n)){var i=this._columnsController.columnOption(n,"index");return this._columnsController.getVisibleIndex(i)}return n},getColumnElements:function(){},getColumns:function(e){return this._columnsController.getVisibleColumns(e)},getCell:function(e,t){var n,i=t||this._getRowElements();if(i.length>0&&e.rowIndex>=0&&("virtual"!==this.option("scrolling.mode")&&(e.rowIndex=e.rowIndex<i.length?e.rowIndex:i.length-1),n=this.getCellElements(e.rowIndex),n&&n.length>0))return n.eq(n.length>e.columnIndex?e.columnIndex:n.length-1)},getRowsCount:function(){var e=this._getTableElement();return e&&1===e.length?e[0].rows.length:0},_getRowElementsCore:function(e){if(e=e||this._getTableElement()){var t=this.option("rowTemplate")&&e.find("> tbody."+R);return t&&t.length?t:e.find("> tbody > ."+R+", > ."+R)}return(0,s.default)()},_getRowElements:function(e){return this._getRowElementsCore(e)},getRowIndex:function(e){return this._getRowElements().index(e)},getBoundingRect:function(){},getName:function(){},setScrollerSpacing:function(e){var t=this,n=t.element(),i=t.option("rtlEnabled");n&&n.css({paddingLeft:i?e:"",paddingRight:i?"":e})},isScrollbarVisible:function(e){var t=this.element(),n=this._tableElement;return!(!t||!n)&&(e?n.outerWidth()-t.width()>0:n.outerHeight()-t.height()>0)}});t.ColumnsView=$},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return a=function(){return e},e}function r(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=a();if(t&&t.has(e))return t.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=o?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function s(e){return c(e)||u(e)||_(e)||l()}function l(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function u(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function c(e){if(Array.isArray(e))return m(e)}function d(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function h(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?d(Object(n),!0).forEach(function(t){f(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):d(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function f(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function p(e,t){return y(e)||v(e,t)||_(e,t)||g()}function g(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function _(e,t){if(e){if("string"==typeof e)return m(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?m(e,t):void 0}}function m(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function v(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function y(e){if(Array.isArray(e))return e}function b(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function x(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function w(e,t,n){return t&&x(e.prototype,t),n&&x(e,n),e}function C(e,t,n){return(C="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=k(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function k(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=A(e),null!==e););return e}function S(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&D(e,t)}function D(e,t){return(D=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function I(e){var t=E();return function(){var n,i=A(e);if(t){var o=A(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return T(this,n)}}function T(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?O(e):t}function O(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function E(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function A(e){return(A=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var P=r(n(51)),R=n(53),M=o(n(2)),B=o(n(11)),V=o(n(65)),F=n(0),L=n(828),H=n(18),z="dx-template-wrapper",N=function(e,t){return function(n){e.hasOwnProperty(n)&&void 0===e[n]&&(e[n]=t(n))}},j=function(e){function t(){return b(this,t),n.apply(this,arguments)}S(t,e);var n=I(t);return w(t,[{key:"_getDefaultOptions",value:function(){var e=this;return(0,F.extend)(!0,C(A(t.prototype),"_getDefaultOptions",this).call(this),this._viewComponent.defaultProps,this._propsInfo.twoWay.reduce(function(t,n){var i,o=p(n,3),a=o[0],r=o[1],s=o[2];return h(h({},t),{},(i={},f(i,a,r),f(i,s,function(t){return e.option(a,t)}),i))},{}))}},{key:"_initMarkup",value:function(){var e=this.getProps();this._shouldRefresh&&(this._shouldRefresh=!1,this._renderPreact(h(h({},e),{},{width:null,height:null,style:"",className:"",children:null}))),this._renderPreact(e)}},{key:"_renderPreact",value:function(e){var t=this.$element()[0];t.parentNode||this._documentFragment.appendChild(t),P.render(P.h(this._viewComponent,e),t,this._preactReplaced?void 0:t),this._preactReplaced=!0,t.parentNode===this._documentFragment&&this._documentFragment.removeChild(t)}},{key:"_render",value:function(){}},{key:"_dispose",value:function(){P.render(null,this.$element()[0]),C(A(t.prototype),"_dispose",this).call(this)}},{key:"_patchOptionValues",value:function(e){var t=this,n=this._propsInfo,i=n.allowNull,o=n.elements,a=n.twoWay,r=this._viewComponent.defaultProps;return i.forEach(N(e,function(){return null})),Object.keys(r).forEach(N(e,function(e){return r[e]})),a.forEach(function(t){var n=p(t,2),i=n[0],o=n[1];return N(e,function(){return o})(i)}),o.forEach(function(n){n in e&&(e[n]=t._patchElementParam(e[n]))}),e}},{key:"getProps",value:function(){var e=this._patchOptionValues(h(h({},this.option()),{},{ref:this._viewRef,children:this._extractDefaultSlot()}));return h(h(h(h({},e),this.elementAttr),e.elementAttr),{},{className:[].concat(s((this.elementAttr.class||"").split(" ")),s((e.elementAttr.class||"").split(" "))).filter(function(e,t,n){return e&&n.indexOf(e)===t}).join(" ").trim(),"class":""},this._actionsMap)}},{key:"_getActionConfigs",value:function(){return{}}},{key:"_init",value:function(){var e=this;C(A(t.prototype),"_init",this).call(this),this._documentFragment=B.default.createDocumentFragment(),this._actionsMap={},Object.keys(this._getActionConfigs()).forEach(function(t){return e._addAction(t)}),this._viewRef=P.createRef(),this._supportedKeys=function(){return{}}}},{key:"_addAction",value:function(e,t){if(!t){var n=this._createActionByOption(e,this._getActionConfigs()[e]);t=function(e){return Object.keys(e).forEach(function(t){B.default.isNode(e[t])&&(e[t]=(0,H.getPublicElement)((0,M.default)(e[t])))}),n(e)}}this._actionsMap[e]=t}},{key:"_optionChanged",value:function(e){var n=e||{},i=n.name;i&&this._getActionConfigs()[i]&&this._addAction(i),C(A(t.prototype),"_optionChanged",this).call(this,e),this._invalidate()}},{key:"_extractDefaultSlot",value:function(){var e=this;if(this.option("_hasAnonymousTemplateContent")){var t=function(t){if(t){var n=t.parentNode;n.removeChild(t),e._getTemplate(e._templateManager.anonymousTemplateName).render({container:(0,H.getPublicElement)((0,M.default)(n)),transclude:!0})}};return P.h(P.Fragment,{},P.h("div",{style:{display:"none"},ref:t}))}return null}},{key:"_createTemplateComponent",value:function(e,t){if(t){var n=this._getTemplate(t);return function(t){var i=t.data,o=t.index,a=(0,R.useRef)();return(0,R.useLayoutEffect)(function(){var e=a.current.parentNode;e.removeChild(a.current);var t=(0,M.default)(e),r=t.contents();Object.keys(i).forEach(function(e){B.default.isNode(i[e])&&(i[e]=(0,H.getPublicElement)((0,M.default)(i[e])))});var s=(0,M.default)(n.render(h({container:(0,H.getPublicElement)(t),model:i},isFinite(o)?{index:o}:{})));s.hasClass(z)&&(0,L.wrapElement)(t,s);var l=t.contents();return function(){(0,L.removeDifferentElements)(r,l)}},Object.keys(e).map(function(t){return e[t]})),P.h(P.Fragment,{},P.h("div",{style:{display:"none"},ref:a}))}}}},{key:"_wrapKeyDownHandler",value:function(e){var t=this;return function(n){var i=n.keyName,o=n.originalEvent,a=n.which,r=t._supportedKeys(),s=r[i]||r[a];if(void 0!==s){var l=s.bind(t),u=l(o,n);if(!u)return o.cancel=!0,o}return null===e||void 0===e?void 0:e(o,n)}}},{key:"_toPublicElement",value:function(e){return(0,H.getPublicElement)((0,M.default)(e))}},{key:"_patchElementParam",value:function(e){var t,n,i;try{i=(0,M.default)(e)}catch(t){return e}return i=null===(t=i)||void 0===t?void 0:t.get(0),null!==(n=i)&&void 0!==n&&n.nodeType?i:e}},{key:"repaint",value:function(){this._shouldRefresh=!0,this._refresh()}},{key:"registerKeyHandler",value:function(e,t){var n=this._supportedKeys();this._supportedKeys=function(){return h(h({},n),{},f({},e,t))}}},{key:"setAria",value:function(){throw new Error('"setAria" method is deprecated, use "aria" property instead')}},{key:"viewRef",get:function(){return this._viewRef.current}},{key:"elementAttr",get:function(){var e;if(!this._elementAttr){var t=this.$element()[0].attributes;this._elementAttr=h({},Object.keys(t).reduce(function(e,n){return t[n].specified&&(e[t[n].name]=t[n].value),e},{}))}for(var n=this.$element()[0].style,i={},o=0;o<n.length;o++)i[n[o]]=n.getPropertyValue(n[o]);this._elementAttr.style=i;var a=this.$element()[0].getAttribute("class")||"";return this._storedClasses=null!==(e=this._storedClasses)&&void 0!==e?e:a.split(" ").filter(function(e){return 0===e.indexOf("dx-")}).join(" "),this._elementAttr.class=a.split(" ").filter(function(e){return 0!==e.indexOf("dx-")}).concat(this._storedClasses).join(" ").trim(),this._elementAttr}}]),t}(V.default);t.default=j,j.IS_RENOVATED_WIDGET=!1,j.IS_RENOVATED_WIDGET=!0,e.exports=t.default},function(e,t,n){function i(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function o(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?i(Object(n),!0).forEach(function(t){a(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):i(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function a(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.isVerticalGroupOrientation=t.getIsGroupedAllDayPanel=t.getGroupCellClasses=t.addHeightToStyle=t.getKeyByGroup=t.getKeyByDateAndGroup=void 0;var r=n(226),s=n(830),l=function(e,t){var n=e.getTime();return t?(n+t).toString():n.toString()};t.getKeyByDateAndGroup=l;var u=function(e){return e.toString()};t.getKeyByGroup=u;var c=function(e,t){var n=t||{};return o(o({},n),{},{height:e?"".concat(e,"px"):n.height})};t.addHeightToStyle=c;var d=function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0],t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"";return(0,r.combineClasses)(a({"dx-scheduler-first-group-cell":e,"dx-scheduler-last-group-cell":t},n,!0))};t.getGroupCellClasses=d;var h=function(e,t){var n,i=e.groupedData,o=i[t],a=!(null===o||void 0===o||null===(n=o.allDayPanel)||void 0===n||!n.length),r=!(null===o||void 0===o||!o.isGroupedAllDayPanel);return a&&r};t.getIsGroupedAllDayPanel=h;var f=function(e){return e===s.VERTICAL_GROUP_ORIENTATION};t.isVerticalGroupOrientation=f},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,
e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(166)),y=o(n(110)),b=o(n(319)),x=n(0),w=n(7),C=o(n(538)),k=o(n(539)),S=n(105);n(127),n(434),n(144);var D="dx-format-active",I="dx-diagram-toolbar",T="dx-diagram-toolbar-separator",O="dx-diagram-toolbar-menu-separator",E="dx-diagram-mobile-toolbar-color-box-opened",A=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){this._commands=[],this._itemHelpers={},this._commandContextMenus={},this._contextMenuList=[],this._valueConverters={},this.bar=new P(this),this._createOnInternalCommand(),this._createOnCustomCommand(),this._createOnSubMenuVisibilityChangingAction(),l(_(t.prototype),"_init",this).call(this)}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this);var e=!(0,w.hasWindow)();this.option("skipAdjustSize")||e||this.$element().width(""),this._commands=this._getCommands(),this._itemHelpers={},this._commandContextMenus={},this._contextMenuList=[];var n=this._createMainElement();if(this._renderToolbar(n),!this.option("skipAdjustSize")&&!e){var i=this.$element().find(".dx-toolbar-before");this.$element().width(i.width())}}},{key:"_createMainElement",value:function(){return(0,m.default)("<div>").addClass(I).appendTo(this._$element)}},{key:"_getCommands",value:function(){return this.option("commands")||[]}},{key:"_renderToolbar",value:function(e){var t=this._commands.filter(function(e){return["after","center"].indexOf(e.position)===-1}),n=this._commands.filter(function(e){return"center"===e.position}),i=this._commands.filter(function(e){return"after"===e.position}),o=[].concat(this._prepareToolbarItems(t,"before",this._executeCommand)).concat(this._prepareToolbarItems(n,"center",this._executeCommand)).concat(this._prepareToolbarItems(i,"after",this._executeCommand));this._toolbarInstance=this._createComponent(e,v.default,{dataSource:o})}},{key:"_prepareToolbarItems",value:function(e,t,n){var i=this;return e.map(function(e){return(0,x.extend)(!0,{location:t,locateInMenu:i.option("locateInMenu")},i._createItem(e,t,n),i._createItemOptions(e),i._createItemActionOptions(e,n))})}},{key:"_createItem",value:function(e,t,n){var i=this;return(e.getCommandValue||e.getEditorValue||e.getEditorDisplayValue)&&(this._valueConverters[e.command]={getCommandValue:e.getCommandValue,getEditorValue:e.getEditorValue,getEditorDisplayValue:e.getEditorDisplayValue}),"separator"===e.widget?{template:function(e,t,n){(0,m.default)(n).addClass(T)},menuItemTemplate:function(e,t,n){(0,m.default)(n).addClass(O)}}:{widget:e.widget||"dxButton",cssClass:e.cssClass,options:{stylingMode:this.option("buttonStylingMode"),type:this.option("buttonType"),text:e.text,hint:e.hint,icon:e.icon||e.iconUnchecked||e.iconChecked,iconChecked:e.iconChecked,iconUnchecked:e.iconUnchecked,onInitialized:function(t){return i._onItemInitialized(t.component,e)},onContentReady:function(t){return i._onItemContentReady(t.component,e,n)}}}}},{key:"_createItemOptions",value:function(e){var t=e.widget,n=e.command,i=e.items,o=e.valueExpr,a=e.displayExpr,r=e.showText,s=e.hint,l=e.icon;return"dxSelectBox"===t?this._createSelectBoxItemOptions(n,s,i,o,a):"dxTextBox"===t?this._createTextBoxItemOptions(n,s):"dxColorBox"===t?this._createColorBoxItemOptions(n,s,l):t&&"dxButton"!==t?void 0:{showText:r||"inMenu"}}},{key:"_createSelectBoxItemOptions",value:function(e,t,n,i,o){var a=this._createTextEditorItemOptions(t);a=(0,x.extend)(!0,a,{options:{dataSource:n,displayExpr:o||"text",valueExpr:i||"value",dropDownOptions:{container:this.option("container")}}});var r=n&&n.every(function(e){return void 0!==e.icon});return r&&(a=(0,x.extend)(!0,a,{options:{fieldTemplate:function(e,t){(0,m.default)("<i>").addClass(e&&e.icon).appendTo(t),(0,m.default)("<div>").dxTextBox({readOnly:!0,stylingMode:"outlined"}).appendTo(t)},itemTemplate:function(e,t,n){return(0,m.default)(n).attr("title",e.hint),'<i class="'.concat(e.icon,'"></i>')}}})),a}},{key:"_createTextBoxItemOptions",value:function(e,t){var n=this,i=this._createTextEditorItemOptions(t);return i=(0,x.extend)(!0,i,{options:{readOnly:!0,focusStateEnabled:!1,hoverStateEnabled:!1,buttons:[{name:"dropDown",location:"after",options:{icon:"spindown",disabled:!1,stylingMode:"text",onClick:function(t){var i=n._commandContextMenus[e];i&&n._toggleContextMenu(i)}}}]}})}},{key:"_createColorBoxItemOptions",value:function(e,t,n){var i=this,o=this._createTextEditorItemOptions(t);return n&&(o=(0,x.extend)(!0,o,{options:{openOnFieldClick:!0,fieldTemplate:function(e,t){(0,m.default)("<i>").addClass(n).css("borderBottomColor",e).appendTo(t),(0,m.default)("<div>").dxTextBox({readOnly:!0,stylingMode:"outlined"}).appendTo(t)}}})),o=(0,x.extend)(!0,o,{options:{dropDownOptions:{container:this.option("container")},onOpened:function(){i.option("isMobileView")&&(0,m.default)("body").addClass(E)},onClosed:function(){(0,m.default)("body").removeClass(E)}}})}},{key:"_createTextEditorItemOptions",value:function(e){return{options:{stylingMode:this.option("editorStylingMode"),hint:e}}}},{key:"_createItemActionOptions",value:function(e,t){var n=this;switch(e.widget){case"dxSelectBox":case"dxColorBox":case"dxCheckBox":return{options:{onValueChanged:function(i){var o=k.default.getItemCommandParameter(n,e,i.component.option("value"));t.call(n,e.command,e.name,o)}}};case"dxTextBox":return{};default:return{options:{onClick:function(i){if(e.items){var o=i.component._contextMenu;o&&n._toggleContextMenu(o)}else{var a=k.default.getItemCommandParameter(n,e);t.call(n,e.command,e.name,a)}}}}}}},{key:"_toggleContextMenu",value:function(e){this._contextMenuList.forEach(function(t){e!==t&&t.hide()}),e.toggle()}},{key:"_onItemInitialized",value:function(e,t){this._addItemHelper(t.command,new R(e))}},{key:"_onItemContentReady",value:function(e,t,n){var i=this,o=(0,S.getDiagram)(),a=o.Browser;if(("dxButton"===e.NAME||"dxTextBox"===e.NAME)&&t.items){var r=(0,m.default)("<div>").appendTo(this.$element());e._contextMenu=this._createComponent(r,y.default,{items:t.items,target:e.$element(),cssClass:k.default.getContextMenuCssClass(),showEvent:"",closeOnOutsideClick:!a.TouchUI,focusStateEnabled:!1,position:{at:"left bottom"},itemTemplate:function(e,t,n){k.default.getContextMenuItemTemplate(this,e,t,n)},onItemClick:function(e){var t=e.component,o=e.itemData;k.default.onContextMenuItemClick(i,o,n.bind(i)),o.items&&o.items.length||t.hide()},onShowing:function(e){i._showingSubMenu||(i._showingSubMenu=e.component,i._onSubMenuVisibilityChangingAction({visible:!0,component:i}),e.component.option("items",e.component.option("items")),delete i._showingSubMenu)},onInitialized:function(n){var o=n.component;return i._onContextMenuInitialized(o,t,e)},onDisposing:function(e){var n=e.component;return i._onContextMenuDisposing(n,t)}})}}},{key:"_onContextMenuInitialized",value:function(e,t,n){this._contextMenuList.push(e),t.command&&(this._commandContextMenus[t.command]=e),this._addContextMenuHelper(t,e,[],n)}},{key:"_addItemHelper",value:function(e,t){if(void 0!==e){if(this._itemHelpers[e])throw new Error("Toolbar cannot contain duplicated commands.");this._itemHelpers[e]=t}}},{key:"_addContextMenuHelper",value:function(e,t,n,i){var o=this;e.items&&e.items.forEach(function(e,a){var r=n.concat(a);o._addItemHelper(e.command,new M(t,r,e.command,i)),o._addContextMenuHelper(e,t,r,i)})}},{key:"_onContextMenuDisposing",value:function(e,t){this._contextMenuList.splice(this._contextMenuList.indexOf(e),1),delete this._commandContextMenus[t.command]}},{key:"_executeCommand",value:function(e,t,n){if(!this._updateLocked){if("number"==typeof e){var i=this._valueConverters[e];i&&i.getCommandValue&&(n=i.getCommandValue(n)),this.bar.raiseBarCommandExecuted(e,n)}else"string"==typeof e&&this._onInternalCommandAction({command:e});void 0!==t&&this._onCustomCommandAction({name:t})}}},{key:"_createOnInternalCommand",value:function(){this._onInternalCommandAction=this._createActionByOption("onInternalCommand")}},{key:"_createOnCustomCommand",value:function(){this._onCustomCommandAction=this._createActionByOption("onCustomCommand")}},{key:"_setItemEnabled",value:function(e,t){if(e in this._itemHelpers){var n=this._itemHelpers[e];n.canUpdate(this._showingSubMenu)&&n.setEnabled(t)}}},{key:"_setEnabled",value:function(e){this._toolbarInstance.option("disabled",!e),this._contextMenuList.forEach(function(t){t.option("disabled",!e)})}},{key:"_setItemValue",value:function(e,t){try{if(this._updateLocked=!0,e in this._itemHelpers){var n=this._itemHelpers[e];if(n.canUpdate(this._showingSubMenu)){var i=this._valueConverters[e];i&&i.getEditorValue&&(t=i.getEditorValue(t));var o;i&&i.getEditorDisplayValue&&(o=i.getEditorDisplayValue(t));var a=this._commandContextMenus[e];n.setValue(t,o,a,a&&e)}}}finally{this._updateLocked=!1}}},{key:"_setItemSubItems",value:function(e,t){if(this._updateLocked=!0,e in this._itemHelpers){var n=this._itemHelpers[e];if(n.canUpdate(this._showingSubMenu)){var i=this._commandContextMenus[e];n.setItems(t,i,i&&e)}}this._updateLocked=!1}},{key:"_createOnSubMenuVisibilityChangingAction",value:function(){this._onSubMenuVisibilityChangingAction=this._createActionByOption("onSubMenuVisibilityChanging")}},{key:"_optionChanged",value:function(e){switch(e.name){case"isMobileView":(0,m.default)("body").removeClass(E),this._invalidate();break;case"onSubMenuVisibilityChanging":this._createOnSubMenuVisibilityChangingAction();break;case"onInternalCommand":this._createOnInternalCommand();break;case"onCustomCommand":this._createOnCustomCommand();break;case"container":case"commands":this._invalidate();break;case"export":break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_getDefaultOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{isMobileView:!1,"export":{fileName:"Diagram",proxyUrl:void 0},locateInMenu:"auto",buttonStylingMode:"text",buttonType:"normal",editorStylingMode:"filled",skipAdjustSize:!1})}},{key:"setCommandChecked",value:function(e,t){this._setItemValue(e,t)}},{key:"setCommandEnabled",value:function(e,t){this._setItemEnabled(e,t)}}]),t}(C.default),P=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getCommandKeys",value:function(){return this._getKeys(this._owner._commands)}},{key:"setItemValue",value:function(e,t){this._owner._setItemValue(e,t)}},{key:"setItemEnabled",value:function(e,t){this._owner._setItemEnabled(e,t)}},{key:"setEnabled",value:function(e){this._owner._setEnabled(e)}},{key:"setItemSubItems",value:function(e,t){this._owner._setItemSubItems(e,t)}}]),t}(b.default),R=function(){function e(t){a(this,e),this._widget=t}return s(e,[{key:"canUpdate",value:function(e){return void 0===e}},{key:"setEnabled",value:function(e){this._widget.option("disabled",!e)}},{key:"setValue",value:function(e,t,n,i){"value"in this._widget.option()?this._updateEditorValue(e,t):void 0!==e&&this._updateButtonValue(e),n&&this._updateContextMenuItemValue(n,"",i,e)}},{key:"setItems",value:function(e,t,n){t?this._updateContextMenuItems(t,"",n,e):this._updateEditorItems(e)}},{key:"_updateContextMenuItems",value:function(e,t,n,i){k.default.updateContextMenuItems(e,t,n,i)}},{key:"_updateEditorItems",value:function(e){"items"in this._widget.option()&&this._widget.option("items",e.map(function(e){return{value:k.default.getItemValue(e),text:e.text}}))}},{key:"_updateEditorValue",value:function(e,t){this._widget.option("value",e),!this._widget.option("selectedItem")&&t&&this._widget.option("value",t)}},{key:"_updateButtonValue",value:function(e){this._widget.option("iconChecked")&&this._widget.option("iconUnchecked")?this._widget.option("icon",e?this._widget.option("iconChecked"):this._widget.option("iconUnchecked")):this._widget.$element().toggleClass(D,e)}},{key:"_updateContextMenuItemValue",value:function(e,t,n,i){k.default.updateContextMenuItemValue(e,t,n,i)}}]),e}(),M=function(e){function t(e,i,o,r){var s;return a(this,t),s=n.call(this,e),s._indexPath=i,s._rootCommandKey=o,s._rootWidget=r,s}c(t,e);var n=h(t);return s(t,[{key:"canUpdate",value:function(e){return l(_(t.prototype),"canUpdate",this).call(this,e)||e===this._widget}},{key:"setEnabled",value:function(e){this._widget.option(this._getItemOptionText()+"disabled",!e);var t=this._hasEnabledCommandItems(this._widget.option("items"));this._rootWidget.option("disabled",!t)}},{key:"_hasEnabledCommandItems",value:function(e){var t=this;return!!e&&e.some(function(e){return void 0!==e.command&&!e.disabled||t._hasEnabledCommandItems(e.items)})}},{key:"setValue",value:function(e){this._updateContextMenuItemValue(this._widget,this._getItemOptionText(),this._rootCommandKey,e)}},{key:"setItems",value:function(e){this._updateContextMenuItems(this._widget,this._getItemOptionText(),this._rootCommandKey,e)}},{key:"_getItemOptionText",value:function(){return k.default.getItemOptionText(this._widget,this._indexPath)}}]),t}(R),B=A;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e){return void 0===e?w||A:(e=(0,C.normalizeEnum)(e),void(w=e in R?e:void 0))}function r(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{keepLastColorInEnd:!1};return n.type=n.baseColorSet,n.extensionMode=n.paletteExtensionMode,p(e,n).generateColors(t)}function s(e,t){t=t||{},e=e||(void 0===w?t.themeDefault:a());var n,i=t.type;return O(e)?e.slice(0):((0,D.isString)(e)&&(n=R[(0,C.normalizeEnum)(e)]),n||(n=R[a()]),i?n[i].slice(0):n)}function l(e,t){var n,i={};O(t)?i.simpleSet=t.slice(0):t&&(i.simpleSet=O(t.simpleSet)?t.simpleSet.slice(0):void 0,i.indicatingSet=O(t.indicatingSet)?t.indicatingSet.slice(0):void 0,i.gradientSet=O(t.gradientSet)?t.gradientSet.slice(0):void 0,i.accentColor=t.accentColor),i.accentColor||(i.accentColor=i.simpleSet&&i.simpleSet[0]),(i.simpleSet||i.indicatingSet||i.gradientSet)&&(n=(0,C.normalizeEnum)(e),(0,k.extend)(R[n]=R[n]||{},i))}function u(e,t){return e=s(e,{themeDefault:t}),e.accentColor||e[0]}function c(e){var t=0;this.next=function(){var n=e[t++];return t===e.length&&this.reset(),n},this.reset=function(){t=0}}function d(e,t){function n(){var t=o.next();a=t?g(e,t):e.slice(0)}var i=t.useHighlight?E:0,o=new c([0,i,-i]),a=[];return{getColor:function(t){var i=a[t%e.length];return t%e.length===e.length-1&&n(),i},generateColors:function(e){var n=[];e=e||t.count;for(var i=0;i<e;i++)n.push(this.getColor(i));return n},reset:function(){o.reset(),n()}}}function h(e,t){function n(e,t,n){var i=new S.default(e).hsl,o=i.l/100,a=n-1/n,r=o-.5*a,s=o+.5*a,l=(n-1)/2,u=t-l;return r<Math.min(.5,.9*o)&&(r=Math.min(.5,.9*o)),s>Math.max(.8,o+.15*(1-o))&&(s=Math.max(.8,o+.15*(1-o))),u<0?o-=(r-o)*u/l:o+=(s-o)*(u/l),i.l=100*o,S.default.prototype.fromHSL(i).toHex()}return{getColor:function(t,i){var o=e.length,a=I((i-1)/o+1),r=e[t%o];return a>1?n(r,I(t/o),a):r},generateColors:function(e){var n=[];e=e||t.count;for(var i=0;i<e;i++)n.push(this.getColor(i,e));return n},reset:function(){}}}function f(e,t){function n(e,t,n,i){for(var o,a,r=Math.floor(e/t),s=e-t*r,l=n;l<n+e;)a=r,s>0&&(a+=1,s--),o=a>2?Math.floor(a/2):0,i.push(l+o),l+=a;return i.sort(function(e,t){return e-t})}function i(e,t,n){t=(n+t)%n;for(var i=0,o=t;o<2*n;o+=1){var a=(n+o)%n;if(e[a])return[e[a],i];i++}}function o(e,t){for(var n=0;n<t;n++){var o=e[n];if(!o){var a=e[n-1];if(!a)continue;var r=i(e,n,t),s=new S.default(r[0]);a=new S.default(a);for(var l=0;l<r[1];l++,n++)e[n]=a.blend(s,(l+1)/(r[1]+1)).toHex()}}return e}function a(i){if(i<=r)return e;var a=[],s=r-2,l=0,u=[];u=t.keepLastColorInEnd?n(i-2,s,1,[0,i-1]):n(i-1,r-1,1,[0]);for(var c=0;c<i;c++)u.indexOf(c)>-1&&(a[c]=e[l++]);return a=o(a,i)}var r=e.length,s=[];return{getColor:function(e,n){return n=n||t.count||r,s.length!==n&&(s=a(n)),s[e%n]},generateColors:function(e,n){if(e=e||t.count||r,n&&e>r){for(var i=a(r),o=0;o<e-r;o++)i.push(i[o]);return i}return r>0?a(e).slice(0,e):[]},reset:function(){}}}function p(e,t,n){var i={dispose:function(){this._extensionStrategy=null},getNextColor:function(e){return this._extensionStrategy.getColor(this._currentColor++,e)},generateColors:function(e,t){return this._extensionStrategy.generateColors(e,(t||{}).repeat)},reset:function(){return this._currentColor=0,this._extensionStrategy.reset(),this}};t=t||{};var o=(t.extensionMode||"").toLowerCase(),a=s(e,{type:t.type||"simpleSet",themeDefault:n});return"alternate"===o?i._extensionStrategy=d(a,t):"extrapolate"===o?i._extensionStrategy=h(a,t):i._extensionStrategy=f(a,t),i.reset(),i}function g(e,t){var n,i=[],o=e.length;for(n=0;n<o;++n)i.push(_(e[n],t));return i}function _(e,t){var n=new S.default(e).alter(t),i=m(n);return(i>200||i<55)&&(n=new S.default(e).alter(-t/2)),n.toHex()}function m(e){return.3*e.r+.59*e.g+.11*e.b}function v(e,t,n){var i=t>0?y(s(e,{type:"gradientSet",themeDefault:n}),t):[];return{getColor:function(e){return i[e]||null}}}function y(e,t){function n(e){var t=a*e,n=I(t),i=T(t);s.push(r[n].blend(r[i],t-n).toHex())}var i,o=t-1,a=e.length-1,r=[],s=[];for(i=0;i<=a;++i)r.push(new S.default(e[i]));if(o>0)for(i=0;i<=o;++i)n(i/o);else n(.5);return s}function b(e,t){var n=s(e,{type:"gradientSet",themeDefault:t}),i=new S.default(n[0]),o=new S.default(n[1]);return{getColor:function(e){return 0<=e&&e<=1?i.blend(o,e).toHex():null}}}t.currentPalette=a,t.generateColors=r,t.getPalette=s,t.registerPalette=l,t.getAccentColor=u,t.createPalette=p,t.getDiscretePalette=v,t.getGradientPalette=b;var x,w,C=n(14),k=n(0),S=i(n(98)),D=n(1),I=Math.floor,T=Math.ceil,O=Array.isArray,E=50,A="material",P={simpleSet:["#5f8b95","#ba4d51","#af8a53","#955f71","#859666","#7e688c"],indicatingSet:["#a3b97c","#e1b676","#ec7f83"],gradientSet:["#5f8b95","#ba4d51"],accentColor:"#ba4d51"},R=(x={},o(x,A,{simpleSet:["#1db2f5","#f5564a","#97c95c","#ffc720","#eb3573","#a63db8"],indicatingSet:["#97c95c","#ffc720","#f5564a"],gradientSet:["#1db2f5","#97c95c"],accentColor:"#1db2f5"}),o(x,"office",P),o(x,"harmony light",{simpleSet:["#fcb65e","#679ec5","#ad79ce","#7abd5c","#e18e92","#b6d623","#b7abea","#85dbd5"],indicatingSet:["#b6d623","#fcb65e","#e18e92"],gradientSet:["#7abd5c","#fcb65e"],accentColor:"#679ec5"}),o(x,"soft pastel",{simpleSet:["#60a69f","#78b6d9","#6682bb","#a37182","#eeba69","#90ba58","#456c68","#7565a4"],indicatingSet:["#90ba58","#eeba69","#a37182"],gradientSet:["#78b6d9","#eeba69"],accentColor:"#60a69f"}),o(x,"pastel",{simpleSet:["#bb7862","#70b3a1","#bb626a","#057d85","#ab394b","#dac599","#153459","#b1d2c6"],indicatingSet:["#70b3a1","#dac599","#bb626a"],gradientSet:["#bb7862","#70b3a1"],accentColor:"#bb7862"}),o(x,"bright",{simpleSet:["#70c92f","#f8ca00","#bd1550","#e97f02","#9d419c","#7e4452","#9ab57e","#36a3a6"],indicatingSet:["#70c92f","#f8ca00","#bd1550"],gradientSet:["#e97f02","#f8ca00"],accentColor:"#e97f02"}),o(x,"soft",{simpleSet:["#cbc87b","#9ab57e","#e55253","#7e4452","#e8c267","#565077","#6babac","#ad6082"],indicatingSet:["#9ab57e","#e8c267","#e55253"],gradientSet:["#9ab57e","#e8c267"],accentColor:"#565077"}),o(x,"ocean",{simpleSet:["#75c099","#acc371","#378a8a","#5fa26a","#064970","#38c5d2","#00a7c6","#6f84bb"],indicatingSet:["#c8e394","#7bc59d","#397c8b"],gradientSet:["#acc371","#38c5d2"],accentColor:"#378a8a"}),o(x,"vintage",{simpleSet:["#dea484","#efc59c","#cb715e","#eb9692","#a85c4c","#f2c0b5","#c96374","#dd956c"],indicatingSet:["#ffe5c6","#f4bb9d","#e57660"],gradientSet:["#efc59c","#cb715e"],accentColor:"#cb715e"}),o(x,"violet",{simpleSet:["#d1a1d1","#eeacc5","#7b5685","#7e7cad","#a13d73","#5b41ab","#e287e2","#689cc1"],indicatingSet:["#d8e2f6","#d0b2da","#d56a8a"],gradientSet:["#eeacc5","#7b5685"],accentColor:"#7b5685"}),o(x,"carmine",{simpleSet:["#fb7764","#73d47f","#fed85e","#d47683","#dde392","#757ab2"],indicatingSet:["#5cb85c","#f0ad4e","#d9534f"],gradientSet:["#fb7764","#73d47f"],accentColor:"#f05b41"}),o(x,"dark moon",{simpleSet:["#4ddac1","#f4c99a","#80dd9b","#f998b3","#4aaaa0","#a5aef1"],indicatingSet:["#59d8a4","#f0ad4e","#f9517e"],gradientSet:["#4ddac1","#f4c99a"],accentColor:"#3debd3"}),o(x,"soft blue",{simpleSet:["#7ab8eb","#97da97","#facb86","#e78683","#839bda","#4db7be"],indicatingSet:["#5cb85c","#f0ad4e","#d9534f"],gradientSet:["#7ab8eb","#97da97"],accentColor:"#7ab8eb"}),o(x,"dark violet",{simpleSet:["#9c63ff","#64c064","#eead51","#d2504b","#4b6bbf","#2da7b0"],indicatingSet:["#5cb85c","#f0ad4e","#d9534f"],gradientSet:["#9c63ff","#64c064"],accentColor:"#9c63ff"}),o(x,"green mist",{simpleSet:["#3cbab2","#8ed962","#5b9d95","#efcc7c","#f1929f","#4d8dab"],indicatingSet:["#72d63c","#ffc852","#f74a5e"],gradientSet:["#3cbab2","#8ed962"],accentColor:"#3cbab2"}),x)},function(e,t,n){function i(e){return(0,l.map)(e,function(e){return e.getCoords()}).concat((0,l.map)(e.slice().reverse(),function(e){return e.getCoords(!0)}))}t.polar=t.chart=void 0;var o=n(50),a=n(0),r=n(132),s=n(231),l=n(14),u=s.chart.line,c=s.polar.line,d=a.extend,h=s.chart.spline._calculateBezierPoints,f={};t.chart=f;var p={};t.polar=p;var g={_createBorderElement:u._createMainElement,_createLegendState:function(e,t){return{fill:e.color||t,opacity:e.opacity,hatching:e.hatching}},getValueRangeInitialValue:function(){return"logarithmic"!==this.valueAxisType&&"datetime"!==this.valueType&&this.showZero!==!1?0:r.chart.getValueRangeInitialValue.call(this)},_getDefaultSegment:function(e){var t=u._getDefaultSegment(e);return t.area=t.line.concat(t.line.slice().reverse()),t},_updateElement:function(e,t,n,i){var o={points:t.line},a={points:t.area},r=e.line;n?(r&&r.animate(o),e.area.animate(a,{},i)):(r&&r.attr(o),e.area.attr(a))},_removeElement:function(e){e.line&&e.line.remove(),e.area.remove()},_drawElement:function(e){return{line:this._bordersGroup&&this._createBorderElement(e.line,{"stroke-width":this._styles.normal.border["stroke-width"]}).append(this._bordersGroup),area:this._createMainElement(e.area).append(this._elementsGroup)}},_applyStyle:function(e){var t=this;t._elementsGroup&&t._elementsGroup.smartAttr(e.elements),t._bordersGroup&&t._bordersGroup.attr(e.border),(t._graphics||[]).forEach(function(t){t.line&&t.line.attr({"stroke-width":e.border["stroke-width"]}).sharp()})},_parseStyle:function(e,t,n){var i=e.border||{},o=u._parseLineOptions(i,n);return o.stroke=i.visible&&o["stroke-width"]?o.stroke:"none",o["stroke-width"]=o["stroke-width"]||1,{border:o,elements:{stroke:"none",fill:e.color||t,hatching:e.hatching,opacity:e.opacity}}},_areBordersVisible:function(){var e=this._options;return e.border.visible||e.hoverStyle.border.visible||e.selectionStyle.border.visible},_createMainElement:function(e,t){return this._renderer.path(e,"area").attr(t)},_getTrackerSettings:function(e){return{"stroke-width":e.singlePointSegment?this._defaultTrackerWidth:0}},_getMainPointsFromSegment:function(e){return e.area}},_=f.area=d({},u,g,{_prepareSegment:function(e,t){var n=this,o=n._processSinglePointsAreaSegment(e,t),a=i(o),r=n.getArgumentAxis();if(r.getAxisPosition){var s=r.getAxisPosition(),l=r.getOptions(),u=(t?1:-1)*Math.round(l.width/2);l.visible&&a.forEach(function(n,i){if(n){var o=1===e.length?0:i<e.length?i:a.length-1-i;t&&n.x===e[o].defaultX&&n.x===s-r.getAxisShift()&&(n.x+=u),!t&&n.y===e[o].defaultY&&n.y===s-r.getAxisShift()&&(n.y+=u)}})}return{line:o,area:a,singlePointSegment:o!==e}},_processSinglePointsAreaSegment:function(e,t){if(e&&1===e.length){var n=e[0],i=(0,o.clone)(n);return i[t?"y":"x"]+=1,i.argument=null,[n,i]}return e}});p.area=d({},c,g,{_prepareSegment:function(e,t,n){return n&&c._closeSegment.call(this,e),_._prepareSegment.call(this,e)},_processSinglePointsAreaSegment:function(e){return s.polar.line._prepareSegment.call(this,e).line}}),f.steparea=d({},_,{_prepareSegment:function(e,t){var n=s.chart.stepline;return e=_._processSinglePointsAreaSegment(e,t),_._prepareSegment.call(this,n._calculateStepLinePoints.call(this,e),t)},getSeriesPairCoord:s.chart.stepline.getSeriesPairCoord}),f.splinearea=d({},_,{_areaPointsToSplineAreaPoints:function(e){var t=e[e.length/2-1],n=e[e.length/2];e.splice(e.length/2,0,{x:t.x,y:t.y},{x:n.x,y:n.y})},_prepareSegment:function(e,t){var n=_._processSinglePointsAreaSegment(e,t),i=_._prepareSegment.call(this,h(n,t));return this._areaPointsToSplineAreaPoints(i.area),i.singlePointSegment=n!==e,i},_getDefaultSegment:function(e){var t=_._getDefaultSegment(e);return this._areaPointsToSplineAreaPoints(t.area),t},_createMainElement:function(e,t){return this._renderer.path(e,"bezierarea").attr(t)},_createBorderElement:s.chart.spline._createMainElement,getSeriesPairCoord:s.chart.spline.getSeriesPairCoord,_getNearestPoints:s.chart.spline._getNearestPoints,_getBezierPoints:s.chart.spline._getBezierPoints,obtainCubicBezierTCoef:s.chart.spline.obtainCubicBezierTCoef})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return e&&t&&e.length===t.length&&a(e,t)}function a(e,t){var n,i,o,a=e.length;for(n=0;n<a;++n)if(i=e[n]!==e[n],o=t[n]!==t[n],!(i&&o||e[n]===t[n]))return!1;return!0}t.compareArrays=o,t.getSampleText=t.formatValue=t.dxBaseGauge=void 0;var r=n(14),s=n(0),l=n(557),u=i(n(112)),c=i(n(946)),d=i(n(947)),h=i(n(79)),f=n(111),p=n(131),g=n(133),_=n(153),m=Number,v=s.extend,y=h.default.format,b=u.default.inherit({_rootClassPrefix:"dxg",_themeSection:"gauge",_createThemeManager:function(){return new c.default.ThemeManager(this._getThemeManagerOptions())},_initCore:function(){var e=this,t=e._renderer.root;e._valueChangingLocker=0,e._translator=e._factory.createTranslator(),e._tracker=e._factory.createTracker({renderer:e._renderer,container:t}),e._setTrackerCallbacks()},_beginValueChanging:function(){this._resetIsReady(),this._onBeginUpdate(),++this._valueChangingLocker},_endValueChanging:function(){0===--this._valueChangingLocker&&this._drawn()},_setTrackerCallbacks:function(){var e=this,t=e._renderer,n=e._tooltip;e._tracker.setCallbacks({"tooltip-show":function(e,i,o){var a=e.getTooltipParameters(),r=t.getRootOffset(),s=v({value:a.value,valueText:n.formatValue(a.value),color:a.color},i);return n.show(s,{x:a.x+r.left,y:a.y+r.top,offset:a.offset},{target:i},void 0,o)},"tooltip-hide":function(){return n.hide()}})},_dispose:function(){this._cleanCore(),this.callBase.apply(this,arguments)},_disposeCore:function(){var e=this;e._themeManager.dispose(),e._tracker.dispose(),e._translator=e._tracker=null},_cleanCore:function(){var e=this;e._tracker.deactivate(),e._cleanContent()},_renderCore:function(){var e=this;e._isValidDomain&&(e._renderContent(),e._tracker.setTooltipState(e._tooltip.isEnabled()),e._tracker.activate(),e._noAnimation=!1)},_applyChanges:function(){this.callBase.apply(this,arguments),this._resizing=this._noAnimation=!1},_setContentSize:function(){var e=this;e._resizing=e._noAnimation=2===e._changes.count(),e.callBase.apply(e,arguments)},_applySize:function(e){var t=this;t._innerRect={left:e[0],top:e[1],right:e[2],bottom:e[3]};var n=t._layout._cache;return t._cleanCore(),t._renderCore(),t._layout._cache=t._layout._cache||n,[e[0],t._innerRect.top,e[2],t._innerRect.bottom]},_initialChanges:["DOMAIN"],_themeDependentChanges:["DOMAIN"],_optionChangesMap:{subtitle:"MOSTLY_TOTAL",indicator:"MOSTLY_TOTAL",geometry:"MOSTLY_TOTAL",animation:"MOSTLY_TOTAL",startValue:"DOMAIN",endValue:"DOMAIN"},_optionChangesOrder:["DOMAIN","MOSTLY_TOTAL"],_change_DOMAIN:function(){this._setupDomain()},_change_MOSTLY_TOTAL:function(){this._applyMostlyTotalChange()},_setupDomain:function(){var e=this;e._setupDomainCore(),e._isValidDomain=isFinite(1/(e._translator.getDomain()[1]-e._translator.getDomain()[0])),e._isValidDomain||e._incidentOccurred("W2301"),e._change(["MOSTLY_TOTAL"])},_applyMostlyTotalChange:function(){var e=this;e._setupCodomain(),e._setupAnimationSettings(),e._setupDefaultFormat(),e._change(["LAYOUT"])},_setupAnimationSettings:function(){var e=this,t=e.option("animation");e._animationSettings=null,(void 0===t||t)&&(t=v({enabled:!0,duration:1e3,easing:"easeOutCubic"},t),t.enabled&&t.duration>0&&(e._animationSettings={duration:m(t.duration),easing:t.easing})),e._containerBackgroundColor=e.option("containerBackgroundColor")||e._themeManager.theme().containerBackgroundColor},_setupDefaultFormat:function(){var e=this._translator.getDomain();this._defaultFormatOptions=(0,r.getAppropriateFormat)(e[0],e[1],this._getApproximateScreenRange())},_setupDomainCore:null,_calculateSize:null,_cleanContent:null,_renderContent:null,_setupCodomain:null,_getApproximateScreenRange:null,_factory:{createTranslator:function(){return new l.Translator1D},createTracker:function(e){return new d.default(e)}}});t.dxBaseGauge=b;var x=function(e,t,n){t=t||{};var i,o=y(e,t.format);return"function"==typeof t.customizeText?(i=v({value:e,valueText:o},n),String(t.customizeText.call(i,i))):o};t.formatValue=x;var w=function(e,t){var n=x(e.getDomainStart(),t),i=x(e.getDomainEnd(),t);return n.length>=i.length?n:i};t.getSampleText=w,b.addPlugin(f.plugin),b.addPlugin(p.plugin),b.addPlugin(g.plugin),b.addPlugin(_.plugin);var C=b.prototype._setTooltipOptions;b.prototype._setTooltipOptions=function(){C.apply(this,arguments),this._tracker&&this._tracker.setTooltipState(this._tooltip.isEnabled())}},function(e,t,n){function i(){}function o(e,t){e.smartAttr(t)}function a(e,t){e.outer.attr({stroke:t.stroke,"stroke-width":t["stroke-width"],"stroke-opacity":t["stroke-opacity"]}),e.inner.smartAttr({fill:t.fill,opacity:t.opacity,hatching:t.hatching})}t.default=void 0;var r=n(0),s=[o,a];(0,r.extend)(i.prototype,{value:0,isNode:function(){return!!(this.nodes&&this.level<this.ctx.maxLevel)},isActive:function(){var e=this.ctx;return this.level>=e.minLevel&&this.level<=e.maxLevel},updateStyles:function(){var e=this,t=Number(e.isNode());e.state=e._buildState(e.ctx.settings[t].state,!t&&e.color&&{fill:e.color})},_buildState:function(e,t){var n=(0,r.extend)({},e);return t?(0,r.extend)(n,t):n},updateLabelStyle:function(){var e=this.ctx.settings[Number(this.isNode())];this.labelState=e.labelState,this.labelParams=e.labelParams},_getState:function(){return this.state},applyState:function(){s[Number(this.isNode())](this.tile,this._getState())}});var l=i;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return r[a.default.locale()]}t.getLanguageId=o;var a=i(n(82)),r={ar:1,bg:2,ca:3,"zh-Hans":4,cs:5,da:6,de:7,el:8,en:9,es:10,fi:11,fr:12,he:13,hu:14,is:15,it:16,ja:17,ko:18,nl:19,no:20,pl:21,pt:22,rm:23,ro:24,ru:25,hr:26,sk:27,sq:28,sv:29,th:30,tr:31,ur:32,id:33,uk:34,be:35,sl:36,et:37,lv:38,lt:39,tg:40,fa:41,vi:42,hy:43,az:44,eu:45,hsb:46,mk:47,tn:50,xh:52,zu:53,af:54,ka:55,fo:56,hi:57,mt:58,se:59,ga:60,ms:62,kk:63,ky:64,sw:65,tk:66,uz:67,tt:68,bn:69,pa:70,gu:71,or:72,ta:73,te:74,kn:75,ml:76,as:77,mr:78,sa:79,mn:80,bo:81,cy:82,km:83,lo:84,gl:86,kok:87,syr:90,si:91,iu:93,am:94,tzm:95,ne:97,fy:98,ps:99,fil:100,dv:101,ha:104,yo:106,quz:107,nso:108,ba:109,lb:110,kl:111,ig:112,ii:120,arn:122,moh:124,br:126,ug:128,mi:129,oc:130,co:131,gsw:132,sah:133,qut:134,rw:135,wo:136,prs:140,gd:145,"ar-SA":1025,"bg-BG":1026,"ca-ES":1027,"zh-TW":1028,"cs-CZ":1029,"da-DK":1030,"de-DE":1031,"el-GR":1032,"en-US":1033,"fi-FI":1035,"fr-FR":1036,"he-IL":1037,"hu-HU":1038,"is-IS":1039,"it-IT":1040,"ja-JP":1041,"ko-KR":1042,"nl-NL":1043,
"nb-NO":1044,"pl-PL":1045,"pt-BR":1046,"rm-CH":1047,"ro-RO":1048,"ru-RU":1049,"hr-HR":1050,"sk-SK":1051,"sq-AL":1052,"sv-SE":1053,"th-TH":1054,"tr-TR":1055,"ur-PK":1056,"id-ID":1057,"uk-UA":1058,"be-BY":1059,"sl-SI":1060,"et-EE":1061,"lv-LV":1062,"lt-LT":1063,"tg-Cyrl-TJ":1064,"fa-IR":1065,"vi-VN":1066,"hy-AM":1067,"az-Latn-AZ":1068,"eu-ES":1069,"hsb-DE":1070,"mk-MK":1071,"tn-ZA":1074,"xh-ZA":1076,"zu-ZA":1077,"af-ZA":1078,"ka-GE":1079,"fo-FO":1080,"hi-IN":1081,"mt-MT":1082,"se-NO":1083,"ms-MY":1086,"kk-KZ":1087,"ky-KG":1088,"sw-KE":1089,"tk-TM":1090,"uz-Latn-UZ":1091,"tt-RU":1092,"bn-IN":1093,"pa-IN":1094,"gu-IN":1095,"or-IN":1096,"ta-IN":1097,"te-IN":1098,"kn-IN":1099,"ml-IN":1100,"as-IN":1101,"mr-IN":1102,"sa-IN":1103,"mn-MN":1104,"bo-CN":1105,"cy-GB":1106,"km-KH":1107,"lo-LA":1108,"gl-ES":1110,"kok-IN":1111,"syr-SY":1114,"si-LK":1115,"iu-Cans-CA":1117,"am-ET":1118,"ne-NP":1121,"fy-NL":1122,"ps-AF":1123,"fil-PH":1124,"dv-MV":1125,"ha-Latn-NG":1128,"yo-NG":1130,"quz-BO":1131,"nso-ZA":1132,"ba-RU":1133,"lb-LU":1134,"kl-GL":1135,"ig-NG":1136,"ii-CN":1144,"arn-CL":1146,"moh-CA":1148,"br-FR":1150,"ug-CN":1152,"mi-NZ":1153,"oc-FR":1154,"co-FR":1155,"gsw-FR":1156,"sah-RU":1157,"qut-GT":1158,"rw-RW":1159,"wo-SN":1160,"prs-AF":1164,"gd-GB":1169,"ar-IQ":2049,"zh-CN":2052,"de-CH":2055,"en-GB":2057,"es-MX":2058,"fr-BE":2060,"it-CH":2064,"nl-BE":2067,"nn-NO":2068,"pt-PT":2070,"sr-Latn-CS":2074,"sv-FI":2077,"az-Cyrl-AZ":2092,"dsb-DE":2094,"se-SE":2107,"ga-IE":2108,"ms-BN":2110,"uz-Cyrl-UZ":2115,"bn-BD":2117,"mn-Mong-CN":2128,"iu-Latn-CA":2141,"tzm-Latn-DZ":2143,"quz-EC":2155,"ar-EG":3073,"zh-HK":3076,"de-AT":3079,"en-AU":3081,"es-ES":3082,"fr-CA":3084,"sr-Cyrl-CS":3098,"se-FI":3131,"quz-PE":3179,"ar-LY":4097,"zh-SG":4100,"de-LU":4103,"en-CA":4105,"es-GT":4106,"fr-CH":4108,"hr-BA":4122,"smj-NO":4155,"ar-DZ":5121,"zh-MO":5124,"de-LI":5127,"en-NZ":5129,"es-CR":5130,"fr-LU":5132,"bs-Latn-BA":5146,"smj-SE":5179,"ar-MA":6145,"en-IE":6153,"es-PA":6154,"fr-MC":6156,"sr-Latn-BA":6170,"sma-NO":6203,"ar-TN":7169,"en-ZA":7177,"es-DO":7178,"sr-Cyrl-BA":7194,"sma-SE":7227,"ar-OM":8193,"en-JM":8201,"es-VE":8202,"bs-Cyrl-BA":8218,"sms-FI":8251,"ar-YE":9217,"en-029":9225,"es-CO":9226,"sr-Latn-RS":9242,"smn-FI":9275,"ar-SY":10241,"en-BZ":10249,"es-PE":10250,"sr-Cyrl-RS":10266,"ar-JO":11265,"en-TT":11273,"es-AR":11274,"sr-Latn-ME":11290,"ar-LB":12289,"en-ZW":12297,"es-EC":12298,"sr-Cyrl-ME":12314,"ar-KW":13313,"en-PH":13321,"es-CL":13322,"ar-AE":14337,"es-UY":14346,"ar-BH":15361,"es-PY":15370,"ar-QA":16385,"en-IN":16393,"es-BO":16394,"en-MY":17417,"es-SV":17418,"en-SG":18441,"es-HN":18442,"es-NI":19466,"es-PR":20490,"es-US":21514,"bs-Cyrl":25626,"bs-Latn":26650,"sr-Cyrl":27674,"sr-Latn":28698,smn:28731,"az-Cyrl":29740,sms:29755,zh:30724,nn:30740,bs:30746,"az-Latn":30764,sma:30779,"uz-Cyrl":30787,"mn-Cyrl":30800,"iu-Cans":30813,"zh-Hant":31748,nb:31764,sr:31770,"tg-Cyrl":31784,dsb:31790,smj:31803,"uz-Latn":31811,"mn-Mong":31824,"iu-Latn":31837,"tzm-Latn":31839,"ha-Latn":31848}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=u.default.createElement("div"),i=e.cloneNode(!0);return t&&(0,d.default)(i).css("backgroundColor",t),n.appendChild(i),n.innerHTML}function a(e){var t=!0;return e.indexOf("xmlns:xlink")===-1&&(e=e.replace("<svg",'<svg xmlns:xlink="http://www.w3.org/1999/xlink"')),e=e.replace(/xmlns="[\s\S]*?"/gi,function(e){return t?(t=!1,e):""}),e.replace(/xmlns:NS1="[\s\S]*?"/gi,"").replace(/NS1:xmlns:xlink="([\s\S]*?)"/gi,'xmlns:xlink="$1"')}function r(e){return e.replace(/&quot;/gi,"&#34;").replace(/&amp;/gi,"&#38;").replace(/&apos;/gi,"&#39;").replace(/&lt;/gi,"&#60;").replace(/&gt;/gi,"&#62;").replace(/&nbsp;/gi,"&#160;").replace(/&shy;/gi,"&#173;")}function s(e,t){return a(r(o(e,t)))}function l(e){return u.default.isNode(e)?e:(new h.DOMParser).parseFromString(e,"image/svg+xml").childNodes[0]}t.getSvgMarkup=s,t.getSvgElement=l;var u=i(n(11)),c=n(7),d=i(n(2)),h=(0,c.getWindow)()},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=n(12),r=i(n(392)),s=r.default.inherit({_getPlainItems:function(){return this._collectionWidget.option("items")||[]},getIndexByItemData:function(e){var t=this._collectionWidget.keyOf.bind(this._collectionWidget);return t?this.getIndexByKey(t(e)):(0,a.inArray)(e,this._getPlainItems())},getItemDataByIndex:function(e){return this._getPlainItems()[e]},deleteItemAtIndex:function(e){this._getPlainItems().splice(e,1)},itemsGetter:function(){return this._getPlainItems()},getKeysByItems:function(e){var t=this._collectionWidget.keyOf.bind(this._collectionWidget),n=e;if(t){n=[];for(var i=0;i<e.length;i++)n.push(t(e[i]))}return n},getIndexByKey:function(e){var t=this._cache,n=t&&t.keys||this.getKeysByItems(this._getPlainItems());if(t&&!t.keys&&(t.keys=n),"object"!==o(e))return n.indexOf(e);for(var i=0,a=n.length;i<a;i++)if(this._equalKeys(e,n[i]))return i;return-1},getItemsByKeys:function(e,t){return(t||e).slice()},moveItemAtIndexToIndex:function(e,t){var n=this._getPlainItems(),i=n[e];n.splice(e,1),n.splice(t,0,i)},_isItemIndex:function(e){return"number"==typeof e&&Math.round(e)===e},_getNormalizedItemIndex:function(e){return this._collectionWidget._itemElements().index(e)},_normalizeItemIndex:function(e){return e},_denormalizeItemIndex:function(e){return e},_getItemByNormalizedIndex:function(e){return e>-1?this._collectionWidget._itemElements().eq(e):null},_itemsFromSameParent:function(){return!0}}),l=s;t.default=l,e.exports=t.default},function(e,t,n){t.getParser=t.getPatternSetters=t.getRegExpInfo=void 0;var i=n(4),o={3:"abbreviated",4:"wide",5:"narrow"},a=function(e,t){return e>2?Object.keys(o).map(function(e){return["format","standalone"].map(function(n){return t.getMonthNames(o[e],n).join("|")}).join("|")}).join("|"):"0?[1-9]|1[012]"},r={y:function(e){return"[0-9]+"},M:a,L:a,Q:function(e,t){return e>2?t.getQuarterNames(o[e],"format").join("|"):"0?[1-4]"},E:function(e,t){return"\\D*"},a:function(e,t){return t.getPeriodNames(o[e<3?3:e],"format").join("|")},d:function(e){return"0?[1-9]|[12][0-9]|3[01]"},H:function(e){return"0?[0-9]|1[0-9]|2[0-3]"},h:function(e){return"0?[1-9]|1[012]"},m:function(e){return"0?[0-9]|[1-5][0-9]"},s:function(e){return"0?[0-9]|[1-5][0-9]"},S:function(e){return"[0-9]{1,"+e+"}"},w:function(e){return"0?[0-9]|[1-5][0-9]"}},s=Number,l=function(e,t){return e.map(function(e){return e.toLowerCase()}).indexOf(t.toLowerCase())},u=function(e,t,n){return t>2?["format","standalone"].map(function(t){return Object.keys(o).map(function(i){var a=n.getMonthNames(o[i],t);return l(a,e)})}).reduce(function(e,t){return e.concat(t)}).filter(function(e){return e>=0})[0]:s(e)-1},c={y:function(e,t){var n=s(e);return 2===t?n<30?2e3+n:1900+n:n},M:u,L:u,Q:function(e,t,n){return t>2?n.getQuarterNames(o[t],"format").indexOf(e):s(e)-1},E:function(e,t,n){var i=n.getDayNames(o[t<3?3:t],"format");return l(i,e)},a:function(e,t,n){var i=n.getPeriodNames(o[t<3?3:t],"format");return l(i,e)},d:s,H:s,h:s,m:s,s:s,S:function(e,t){for(t=Math.max(t,3),e=e.slice(0,3);t<3;)e+="0",t++;return s(e)}},d=["y","M","d","h","m","s","S"],h={y:"setFullYear",M:"setMonth",L:"setMonth",a:function(e,t,n){var i=e.getHours(),o=n.h;void 0!==o&&o!==i&&i--,t||12!==i?t&&12!==i&&(i+=12):i=0,e.setHours(i)},d:"setDate",H:"setHours",h:"setHours",m:"setMinutes",s:"setSeconds",S:"setMilliseconds"},f=function(e,t){var n=e[t],i=0;do t++,i++;while(e[t]===n);return i},p=function(e,t){for(var n="",i=0;i<t;i++)n+=e;return n},g=function(e,t){for(var n,o="",a="",s=[],l=function(){a&&(s.push("'"+a+"'"),o+=(0,i.escapeRegExp)(a)+")",a="")},u=0;u<e.length;u++){var c=e[u],d="'"===c,h=r[c];if(!d||(n=!n,"'"===e[u-1]))if(h&&!n){var g=f(e,u),_=p(c,g);l(),s.push(_),o+="("+h(g,t)+")",u+=g-1}else a||(o+="("),a+=c}return l(),{patterns:s,regexp:new RegExp("^"+o+"$","i")}};t.getRegExpInfo=g;var _=function(){return h};t.getPatternSetters=_;var m=function(e,t,n,i,o){var a=t[0],r=h[a],s=c[a];if(r&&s){var l=s(n,t.length,i);o[t]=l,e[r]?e[r](l):r(e,l,o)}},v=function(e,t,n){var i=h[t],o="g"+i.substr(1),a=n[o]();e[i](a)},y=function(e){return e.map(function(e){return"'"===e[0]?"":"H"===e[0]?"h":e[0]})},b=function(e){var t=e.map(function(e){return d.indexOf(e)});return Math.max.apply(Math,t)},x=function(e){var t=e.filter(function(e){return d.indexOf(e)<0});return d.concat(t)},w=function(e,t){var n=g(e,t);return function(e){var i=n.regexp.exec(e);if(i){var o=new Date,a=new Date(o.getFullYear(),0,1),r=y(n.patterns),s=b(r),l=x(r),u={};return l.forEach(function(e,l){if(e&&!(l<d.length&&l>s)){var c=r.indexOf(e);if(c>=0){var h=n.patterns[c],f=i[c+1];m(a,h,f,t,u)}else v(a,e,o)}}),a}return null}};t.getParser=w},function(e,t,n){t.hideCallback=void 0;var i=n(12),o=function(){var e=[];return{add:function(t){var n=(0,i.inArray)(t,e);n===-1&&e.push(t)},remove:function(t){var n=(0,i.inArray)(t,e);n!==-1&&e.splice(n,1)},fire:function(){var t=e.pop(),n=!!t;return n&&t(),n},hasCallback:function(){return e.length>0}}}();t.hideCallback=o},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=i(n(393)),r=i(n(394)),s=n(0),l=n(4),u=n(1),c=n(6),d=o.default.inherit({ctor:function(e){this.options=(0,s.extend)(this._getDefaultOptions(),e,{selectedItemKeys:e.selectedKeys||[]}),this._selectionStrategy=this.options.deferred?new a.default(this.options):new r.default(this.options),this._focusedItemIndex=-1,this.options.equalByReference||this._selectionStrategy.updateSelectedItemKeyHash(this.options.selectedItemKeys)},_getDefaultOptions:function(){return{allowNullValue:!1,deferred:!1,equalByReference:!1,mode:"multiple",selectedItems:[],selectionFilter:[],maxFilterLengthInRequest:0,onSelectionChanged:l.noop,key:l.noop,keyOf:function(e){return e},load:function(){return(new c.Deferred).resolve([])},totalCount:function(){return-1},isSelectableItem:function(){return!0},isItemSelected:function(){return!1},getItemData:function(e){return e},dataFields:l.noop,filter:l.noop}},validate:function(){this._selectionStrategy.validate()},getSelectedItemKeys:function(){return this._selectionStrategy.getSelectedItemKeys()},getSelectedItems:function(){return this._selectionStrategy.getSelectedItems()},selectionFilter:function(e){if(void 0===e)return this.options.selectionFilter;var t=this.options.selectionFilter!==e&&JSON.stringify(this.options.selectionFilter)!==JSON.stringify(e);this.options.selectionFilter=e,t&&this.onSelectionChanged()},setSelection:function(e){return this.selectedItemKeys(e)},select:function(e){return this.selectedItemKeys(e,!0)},deselect:function(e){return this.selectedItemKeys(e,!0,!0)},selectedItemKeys:function(e,t,n,i){var o=this;return e=(0,u.isDefined)(e)?e:[],e=Array.isArray(e)?e:[e],o.validate(),this._selectionStrategy.selectedItemKeys(e,t,n,i)},clearSelection:function(){return this.selectedItemKeys([])},_addSelectedItem:function(e,t){this._selectionStrategy.addSelectedItem(t,e)},_removeSelectedItem:function(e){this._selectionStrategy.removeSelectedItem(e)},_setSelectedItems:function(e,t){this._selectionStrategy.setSelectedItems(e,t)},onSelectionChanged:function(){this._selectionStrategy.onSelectionChanged()},changeItemSelection:function(e,t){var n,i=this.options.plainItems(),o=i[e];if(!this.isSelectable()||!this.isDataItem(o))return!1;var a=this.options.getItemData(o),r=this.options.keyOf(a);if(t=t||{},t.shift&&"multiple"===this.options.mode&&this._focusedItemIndex>=0)n=this.changeItemSelectionWhenShiftKeyPressed(e,i);else if(t.control){this._resetItemSelectionWhenShiftKeyPressed();var s=this._selectionStrategy.isItemDataSelected(a);"single"===this.options.mode&&this.clearSelectedItems(),s?this._removeSelectedItem(r):this._addSelectedItem(a,r),n=!0}else{this._resetItemSelectionWhenShiftKeyPressed();var l=this._selectionStrategy.equalKeys(this.options.selectedItemKeys[0],r);1===this.options.selectedItemKeys.length&&l||(this._setSelectedItems([r],[a]),n=!0)}return n?(this._focusedItemIndex=e,this.onSelectionChanged(),!0):void 0},isDataItem:function(e){return this.options.isSelectableItem(e)},isSelectable:function(){return"single"===this.options.mode||"multiple"===this.options.mode},isItemDataSelected:function(e){return this._selectionStrategy.isItemDataSelected(e)},isItemSelected:function(e){return this._selectionStrategy.isItemKeySelected(e)},_resetItemSelectionWhenShiftKeyPressed:function(){delete this._shiftFocusedItemIndex},_resetFocusedItemIndex:function(){this._focusedItemIndex=-1},changeItemSelectionWhenShiftKeyPressed:function(e,t){var n,i,o=!1,a=this.options.keyOf,r=t[this._focusedItemIndex],s=this.options.getItemData(r),l=a(s),c=r&&this.isItemDataSelected(s);(0,u.isDefined)(this._shiftFocusedItemIndex)||(this._shiftFocusedItemIndex=this._focusedItemIndex);var d,h;if(this._shiftFocusedItemIndex!==this._focusedItemIndex)for(n=this._focusedItemIndex<this._shiftFocusedItemIndex?1:-1,i=this._focusedItemIndex;i!==this._shiftFocusedItemIndex;i+=n)this.isDataItem(t[i])&&(h=a(this.options.getItemData(t[i])),this._removeSelectedItem(h),o=!0);if(e!==this._shiftFocusedItemIndex)for(n=e<this._shiftFocusedItemIndex?1:-1,i=e;i!==this._shiftFocusedItemIndex;i+=n)this.isDataItem(t[i])&&(d=this.options.getItemData(t[i]),h=a(d),this._addSelectedItem(d,h),o=!0);return this.isDataItem(r)&&!c&&(this._addSelectedItem(s,l),o=!0),o},clearSelectedItems:function(){this._setSelectedItems([],[])},selectAll:function(e){return this._resetFocusedItemIndex(),e?this._onePageSelectAll(!1):this.selectedItemKeys([],!0,!1,!0)},deselectAll:function(e){return this._resetFocusedItemIndex(),e?this._onePageSelectAll(!0):this.selectedItemKeys([],!0,!0,!0)},_onePageSelectAll:function(e){for(var t=this._selectionStrategy.getSelectableItems(this.options.plainItems()),n=0;n<t.length;n++){var i=t[n];if(this.isDataItem(i)){var o=this.options.getItemData(i),a=this.options.keyOf(o),r=this.isItemSelected(a);r||e||this._addSelectedItem(o,a),r&&e&&this._removeSelectedItem(a)}}return this.onSelectionChanged(),(new c.Deferred).resolve()},getSelectAllState:function(e){return this._selectionStrategy.getSelectAllState(e)}});t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(19),l=n(4),u=i(n(62)),c=n(3),d=n(1),h=n(6),f=i(n(277)),p=2e5,g=function(){function e(t){o(this,e),t=(0,l.ensureDefined)(t,{}),this._keyGetter=(0,s.compileGetter)(this._getKeyExpr(t)),this._nameGetter=(0,s.compileGetter)(this._getNameExpr(t)),this._isDirGetter=(0,s.compileGetter)(this._getIsDirExpr(t)),this._sizeGetter=(0,s.compileGetter)(this._getSizeExpr(t)),this._dateModifiedGetter=(0,s.compileGetter)(this._getDateModifiedExpr(t)),this._thumbnailGetter=(0,s.compileGetter)(t.thumbnailExpr||"thumbnail")}return r(e,[{key:"getItems",value:function(e){return[]}},{key:"renameItem",value:function(e,t){}},{key:"createDirectory",value:function(e,t){}},{key:"deleteItems",value:function(e){}},{key:"moveItems",value:function(e,t){}},{key:"copyItems",value:function(e,t){}},{key:"uploadFileChunk",value:function(e,t,n){}},{key:"abortFileUpload",value:function(e,t,n){}},{key:"downloadItems",value:function(e){}},{key:"getItemsContent",value:function(e){}},{key:"getFileUploadChunkSize",value:function(){return p}},{key:"_getItemsByType",value:function(e,t){return this.getItems(e).filter(function(e){return e.isDirectory===t})}},{key:"_convertDataObjectsToFileItems",value:function(e,t){var n=this,i=[];return(0,c.each)(e,function(e,o){var a=n._createFileItem(o,t);i.push(a)}),i}},{key:"_createFileItem",value:function(e,t){var n=new f.default(t,this._nameGetter(e),(!!this._isDirGetter(e)));return n.size=this._sizeGetter(e),void 0===n.size&&(n.size=0),n.dateModified=u.default.deserializeDate(this._dateModifiedGetter(e)),void 0===n.dateModified&&(n.dateModified=new Date),n.isDirectory&&(n.hasSubDirectories=this._hasSubDirs(e)),n.key=this._keyGetter(e),n.key||(n.key=n.relativeName),n.thumbnail=this._thumbnailGetter(e)||"",n.dataItem=e,n}},{key:"_hasSubDirs",value:function(e){return!0}},{key:"_getKeyExpr",value:function(e){return e.keyExpr||this._defaultKeyExpr}},{key:"_defaultKeyExpr",value:function(e){return 2===arguments.length?void(e.__KEY__=arguments[1]):Object.prototype.hasOwnProperty.call(e,"__KEY__")?e.__KEY__:null}},{key:"_getNameExpr",value:function(e){return e.nameExpr||"name"}},{key:"_getIsDirExpr",value:function(e){return e.isDirectoryExpr||"isDirectory"}},{key:"_getSizeExpr",value:function(e){return e.sizeExpr||"size"}},{key:"_getDateModifiedExpr",value:function(e){return e.dateModifiedExpr||"dateModified"}},{key:"_executeActionAsDeferred",value:function(e,t){var n=new h.Deferred;try{var i=e();(0,d.isPromise)(i)?(0,h.fromPromise)(i).done(function(e){return n.resolve(t&&e||void 0)}).fail(function(e){return n.reject(e)}):n.resolve(t&&i||void 0)}catch(e){return n.reject(e)}return n.promise()}}]),e}(),_=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=function(){function e(t,n,i){o(this,e),this.instance=null,this.$container=null,this.$placeMarker=null,this.editor=n,this.name=t,this.options=i||{}}return r(e,[{key:"_addPlaceMarker",value:function(e){this.$placeMarker=(0,s.default)("<div>").appendTo(e)}},{key:"_addToContainer",value:function(e){var t=this.$placeMarker,n=this.$container;t?t.replaceWith(e):e.appendTo(n)}},{key:"_attachEvents",value:function(){throw"Not implemented"}},{key:"_create",value:function(){throw"Not implemented"}},{key:"_isRendered",value:function(){return!!this.instance}},{key:"_isVisible",value:function(){var e=this.editor,t=this.options;return t.visible||!e.option("readOnly")}},{key:"_isDisabled",value:function(){throw"Not implemented"}},{key:"_shouldRender",value:function(){return this._isVisible()&&!this._isRendered()}},{key:"dispose",value:function(){var e=this.instance,t=this.$placeMarker;e&&(e.dispose?e.dispose():e.remove(),this.instance=null),t&&t.remove()}},{key:"render",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.$container;if(this.$container=e,this._isVisible()){var t=this._create(),n=t.instance,i=t.$element;this.instance=n,this._attachEvents(n,i)}else this._addPlaceMarker(e)}},{key:"update",value:function(){return this._shouldRender()&&this.render(),!!this.instance}}]),e}();t.default=l,e.exports=t.default},function(e,t,n){t.getSizeValue=t.getElementWidth=void 0;var i=n(7),o=function(e){if((0,i.hasWindow)())return e.outerWidth()};t.getElementWidth=o;var a=function(e){return null===e&&(e=void 0),"function"==typeof e&&(e=e()),e};t.getSizeValue=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var a=i(n(2)),r=i(n(58)),s=n(0),l=n(1),u=n(90),c=i(n(41)),d=n(6),h=n(432),f=i(n(11)),p=n(4),g="__dx_key__",_=r.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{repaintChangesOnly:!1})},ctor:function(){var e=this;this.callBase.apply(this,arguments),this._customizeStoreLoadOptions=function(t){var n=e._dataSource;n&&!n.isLoaded()&&(e._correctionIndex=0),e._correctionIndex&&t.storeLoadOptions&&(t.storeLoadOptions.skip+=e._correctionIndex)},this._dataSource&&this._dataSource.on("customizeStoreLoadOptions",this._customizeStoreLoadOptions)},reload:function(){this._correctionIndex=0},_init:function(){this.callBase(),this._refreshItemsCache(),this._correctionIndex=0},_findItemElementByKey:function(e){var t=this,n=(0,a.default)(),i=this.key();return this.itemElements().each(function(o,r){var s=(0,a.default)(r),l=t._getItemData(s);if(i?c.default.keysEqual(i,t.keyOf(l),e):t._isItemEquals(l,e))return n=s,!1}),n},_dataSourceChangedHandler:function(e,t){null!==t&&void 0!==t&&t.changes?this._modifyByChanges(t.changes):(this.callBase(e,t),this._refreshItemsCache())},_isItemEquals:function(e,t){e&&e[g]&&(e=e.data);try{return JSON.stringify(e)===JSON.stringify(t)}catch(n){return e===t}},_isItemStrictEquals:function(e,t){return this._isItemEquals(e,t)},_partialRefresh:function(){var e=this;if(this.option("repaintChangesOnly")){var t=function(t){return t&&void 0!==t[g]?t[g]:e.keyOf(t)},n=(0,h.findChanges)(this._itemsCache,this._editStrategy.itemsGetter(),t,this._isItemStrictEquals.bind(this));if(n&&this._itemsCache.length)return this._modifyByChanges(n,!0),this._renderEmptyMessage(),!0;this._refreshItemsCache()}return!1},_refreshItemsCache:function(){if(this.option("repaintChangesOnly")){var e=this._editStrategy.itemsGetter();try{this._itemsCache=(0,s.extend)(!0,[],e),this.key()||(this._itemsCache=this._itemsCache.map(function(t,n){var i;return i={},o(i,g,e[n]),o(i,"data",t),i}))}catch(t){this._itemsCache=(0,s.extend)([],e)}}},_dispose:function(){this._dataSource&&this._dataSource.off("customizeStoreLoadOptions",this._customizeStoreLoadOptions),this.callBase()},_updateByChange:function(e,t,n,i){var o=this;if(i)this._renderItem(n.index,n.data,null,this._findItemElementByKey(n.key));else{var a=t[(0,u.indexByKey)(e,t,n.key)];a&&(0,u.update)(e,t,n.key,n.data).done(function(){o._renderItem(t.indexOf(a),a,null,o._findItemElementByKey(n.key))})}},_insertByChange:function(e,t,n,i){var o=this;(0,d.when)(i||(0,u.insert)(e,t,n.data,n.index)).done(function(){o._beforeItemElementInserted(n),o._renderItem((0,l.isDefined)(n.index)?n.index:t.length,n.data),o._afterItemElementInserted(),o._correctionIndex++})},_updateSelectionAfterRemoveByChange:function(e){var t=this.option("selectedIndex");t>e?this.option("selectedIndex",t-1):t===e&&1===this.option("selectedItems").length?this.option("selectedItems",[]):this._normalizeSelectedItems()},_beforeItemElementInserted:function(e){var t=this.option("selectedIndex");e.index<=t&&this.option("selectedIndex",t+1)},_afterItemElementInserted:p.noop,_removeByChange:function(e,t,n,i){var o=this,a=i?n.index:(0,u.indexByKey)(e,t,n.key),r=i?n.oldItem:t[a];if(r){var s=this._findItemElementByKey(n.key),l=this._extendActionArgs(s);this._waitDeletingPrepare(s).done(function(){i?(o._updateIndicesAfterIndex(a-1),o._afterItemElementDeleted(s,l),o._updateSelectionAfterRemoveByChange(a)):(o._deleteItemElementByIndex(a),o._afterItemElementDeleted(s,l))}),this._correctionIndex--}},_modifyByChanges:function(e,t){var n=this,i=this._editStrategy.itemsGetter(),o={key:this.key.bind(this),keyOf:this.keyOf.bind(this)},a=this._dataSource,r=a&&a.paginate(),s=a&&a.group();(r||s)&&(e=e.filter(function(e){return"insert"!==e.type||void 0!==e.index})),e.forEach(function(e){return n["_".concat(e.type,"ByChange")](o,i,e,t)}),this._renderedItemsCount=i.length,this._refreshItemsCache(),this._fireContentReadyAction()},_appendItemToContainer:function(e,t,n){var i=e.children(this._itemSelector()).get(n);f.default.insertElement(e.get(0),t.get(0),i)},_optionChanged:function(e){switch(e.name){case"items":var t=this._partialRefresh(e.value);t||this.callBase(e);break;case"dataSource":this.option("repaintChangesOnly")&&e.value||this.option("items",[]),this.callBase(e);break;case"repaintChangesOnly":break;default:this.callBase(e)}}});t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}t.default=void 0;var l,u,c=i(n(2)),d=n(7),h=i(n(5)),f=n(42),p=i(n(9)),g=n(30),_=i(n(430)),m=i(n(23)),v=n(40),y=n(0),b=i(n(65)),x=n(18),w=n(8),C=i(n(25)),k=n(80),S=i(n(84)),D=n(1),I=n(4),T=n(78),O=n(93),E=n(6),A=n(24),P=(0,d.getWindow)(),R="dxDraggable",M=(0,w.addNamespace)(k.start,R),B=(0,w.addNamespace)(k.move,R),V=(0,w.addNamespace)(k.end,R),F=(0,w.addNamespace)(k.enter,R),L=(0,w.addNamespace)(k.leave,R),H=(0,w.addNamespace)(C.default.down,R),z="clone",N="content",j=function(e){return{x:e.pageX-(0,c.default)(P).scrollLeft(),y:e.pageY-(0,c.default)(P).scrollTop()}},W=function(){function e(t,n){a(this,e),this._preventScroll=!0,this._component=n,"vertical"===t?(this._scrollValue="scrollTop",this._overFlowAttr="overflowY",this._sizeAttr="height",this._scrollSizeProp="scrollHeight",this._clientSizeProp="clientHeight",this._limitProps={start:"top",end:"bottom"}):(this._scrollValue="scrollLeft",this._overFlowAttr="overflowX",this._sizeAttr="width",this._scrollSizeProp="scrollWidth",this._clientSizeProp="clientWidth",this._limitProps={start:"left",end:"right"})}return s(e,[{key:"updateScrollable",value:function(e,t){var n=this;e.some(function(e){return n._trySetScrollable(e,t)})||(n._$scrollableAtPointer=null,n._scrollSpeed=0)}},{key:"isScrolling",value:function(){return!!this._scrollSpeed}},{key:"isScrollable",value:function(e){var t=this;return("auto"===e.css(t._overFlowAttr)||e.hasClass("dx-scrollable-container"))&&e.prop(t._scrollSizeProp)>e[t._sizeAttr]()}},{key:"_trySetScrollable",value:function(e,t){var n,i=this,o=(0,c.default)(e),a=i._component.option("scrollSensitivity"),r=i.isScrollable(o);return r&&(n=i._calculateDistanceToBorders(o,t),a>n[i._limitProps.start]?i._preventScroll||(i._scrollSpeed=-i._calculateScrollSpeed(n[i._limitProps.start]),i._$scrollableAtPointer=o):a>n[i._limitProps.end]?i._preventScroll||(i._scrollSpeed=i._calculateScrollSpeed(n[i._limitProps.end]),i._$scrollableAtPointer=o):(r=!1,i._preventScroll=!1)),r}},{key:"_calculateDistanceToBorders",value:function(e,t){var n,i=e.get(0);return i?(n=(0,A.getBoundingRect)(i),{left:t.x-n.left,top:t.y-n.top,right:n.right-t.x,bottom:n.bottom-t.y}):{}}},{key:"_calculateScrollSpeed",value:function(e){var t=this._component,n=t.option("scrollSensitivity"),i=t.option("scrollSpeed");return Math.ceil(Math.pow((n-e)/n,2)*i)}},{key:"scrollByStep",value:function(){var e=this;if(e._$scrollableAtPointer&&e._scrollSpeed){if(e._$scrollableAtPointer.hasClass("dx-scrollable-container")){var t=e._$scrollableAtPointer.closest(".dx-scrollable"),n=t.data("dxScrollable")||t.data("dxScrollView");if(n){var i=n.scrollOffset()[e._limitProps.start]+e._scrollSpeed;n.scrollTo(o({},e._limitProps.start,i))}}else{var a=e._$scrollableAtPointer[e._scrollValue]()+e._scrollSpeed;e._$scrollableAtPointer[e._scrollValue](a)}var r=e._component._dragMoveArgs;r&&e._component._dragMoveHandler(r)}}},{key:"reset",value:function(){this._$scrollableAtPointer=null,this._scrollSpeed=0,this._preventScroll=!0}},{key:"isOutsideScrollable",value:function(e,t){if(!e)return!1;var n=(0,A.getBoundingRect)(e.get(0)),i=n[this._limitProps.start],o=n[this._sizeAttr],a=j(t),r="width"===this._sizeAttr?a.x:a.y;return r<i||r>i+o}}]),e}(),G=_.default.inherit({ctor:function(e){this.callBase(),this._strategy=e},_step:function(){var e=this._strategy._horizontalScrollHelper,t=this._strategy._verticalScrollHelper;e&&e.scrollByStep(),t&&t.scrollByStep()}}),q=b.default.inherit({reset:I.noop,dragMove:I.noop,dragEnter:I.noop,dragLeave:I.noop,dragEnd:function(e){var t=this._getSourceDraggable();return t._fireRemoveEvent(e),(new E.Deferred).resolve()},_fireRemoveEvent:I.noop,_getDefaultOptions:function(){return(0,y.extend)(this.callBase(),{onDragStart:null,onDragMove:null,onDragEnd:null,onDragEnter:null,onDragLeave:null,onDrop:null,immediate:!0,dragDirection:"both",boundary:void 0,boundOffset:0,allowMoveByClick:!1,itemData:null,container:void 0,dragTemplate:void 0,contentTemplate:"content",handle:"",filter:"",clone:!1,autoScroll:!0,scrollSpeed:30,scrollSensitivity:60,group:void 0,data:void 0})},_setOptionsByReference:function(){this.callBase.apply(this,arguments),(0,y.extend)(this._optionsByReference,{component:!0,group:!0,itemData:!0,data:!0})},_init:function(){this.callBase(),this._attachEventHandlers(),this._scrollAnimator=new G(this),this._horizontalScrollHelper=new W("horizontal",this),this._verticalScrollHelper=new W("vertical",this)},_normalizeCursorOffset:function(e){return(0,D.isObject)(e)&&(e={h:e.x,v:e.y}),e=(0,I.splitPair)(e).map(function(e){return parseFloat(e)}),{left:e[0],top:1===e.length?e[0]:e[1]}},_getNormalizedCursorOffset:function(e,t){return(0,D.isFunction)(e)&&(e=e.call(this,t)),this._normalizeCursorOffset(e)},_calculateElementOffset:function(e){var t,n,i=e.event,o=(0,c.default)(e.itemElement),a=(0,c.default)(e.dragElement),r=this._dragElementIsCloned(),s=this.option("cursorOffset"),l={left:0,top:0},u=this._initialLocate=(0,g.locate)(a);return(r||e.initialOffset||s)&&(t=e.initialOffset||o.offset(),s&&(l=this._getNormalizedCursorOffset(s,e),isFinite(l.left)&&(t.left=i.pageX),isFinite(l.top)&&(t.top=i.pageY)),n=a.offset(),t.top-=n.top+(l.top||0)-u.top,t.left-=n.left+(l.left||0)-u.left),t},_initPosition:function(e){var t=(0,c.default)(e.dragElement),n=this._calculateElementOffset(e);n&&this._move(n,t),this._startPosition=(0,g.locate)(t)},_startAnimator:function(){this._scrollAnimator.inProgress()||this._scrollAnimator.start()},_stopAnimator:function(){this._scrollAnimator.stop()},_addWidgetPrefix:function(e){var t=this.NAME;return(0,v.dasherize)(t)+(e?"-"+e:"")},_getItemsSelector:function(){return this.option("filter")||""},_$content:function(){var e=this.$element(),t=e.children(".dx-template-wrapper");return t.length?t:e},_attachEventHandlers:function(){var e=this;if(!this.option("disabled")){var t=this._$content(),n=this._getItemsSelector(),i=this.option("allowMoveByClick"),o={direction:this.option("dragDirection"),immediate:this.option("immediate"),checkDropTarget:function(t,n){var i=e.option("group"),o=e._getSourceDraggable().option("group"),a=e._getScrollable(t);return!e._verticalScrollHelper.isOutsideScrollable(a,n)&&!e._horizontalScrollHelper.isOutsideScrollable(a,n)&&(o&&o===i)}};i&&(t=this._getArea(),h.default.on(t,H,o,this._pointerDownHandler.bind(this))),">"===n[0]&&(n=n.slice(1)),h.default.on(t,M,n,o,this._dragStartHandler.bind(this)),h.default.on(t,B,o,this._dragMoveHandler.bind(this)),h.default.on(t,V,o,this._dragEndHandler.bind(this)),h.default.on(t,F,o,this._dragEnterHandler.bind(this)),h.default.on(t,L,o,this._dragLeaveHandler.bind(this))}},_dragElementIsCloned:function(){return this._$dragElement&&this._$dragElement.hasClass(this._addWidgetPrefix(z))},_getDragTemplateArgs:function(e,t){return{container:(0,x.getPublicElement)(t),model:{itemData:this.option("itemData"),itemElement:(0,x.getPublicElement)(e)}}},_createDragElement:function(e){var t=e,n=this.option("clone"),i=this._getContainer(),o=this.option("dragTemplate");return o?(o=this._getTemplate(o),t=(0,c.default)("<div>").appendTo(i),o.render(this._getDragTemplateArgs(e,t))):n&&(t=(0,c.default)("<div>").appendTo(i),e.clone().css({width:e.css("width"),height:e.css("height")}).appendTo(t)),t.toggleClass(this._addWidgetPrefix(z),t.get(0)!==e.get(0)).toggleClass("dx-rtl",this.option("rtlEnabled"))},_resetDragElement:function(){this._dragElementIsCloned()?this._$dragElement.remove():this._toggleDraggingClass(!1),this._$dragElement=null},_resetSourceElement:function(){this._toggleDragSourceClass(!1),
this._$sourceElement=null},_detachEventHandlers:function(){h.default.off(this._$content(),"."+R),h.default.off(this._getArea(),"."+R)},_move:function(e,t){(0,g.move)(t||this._$dragElement,e)},_getDraggableElement:function(e){var t=this._getSourceElement();if(t)return t;var n=this.option("allowMoveByClick");if(n)return this.$element();var i=(0,c.default)(e&&e.target),o=this._getItemsSelector();if(">"===o[0]){var a=this._$content().find(o);a.is(i)||(i=i.closest(a))}return i},_getSourceElement:function(){var e=this._getSourceDraggable();return e._$sourceElement},_pointerDownHandler:function(e){if(!(0,w.needSkipEvent)(e)){var t={},n=this.$element(),i=this.option("dragDirection");"horizontal"!==i&&"both"!==i||(t.left=e.pageX-n.offset().left+(0,g.locate)(n).left-n.width()/2),"vertical"!==i&&"both"!==i||(t.top=e.pageY-n.offset().top+(0,g.locate)(n).top-n.height()/2),this._move(t,n),this._getAction("onDragMove")(this._getEventArgs(e))}},_isValidElement:function(e,t){var n=this.option("handle"),i=(0,c.default)(e.originalEvent&&e.originalEvent.target);return!(n&&!i.closest(n).length)&&(!!t.length&&!t.is(".dx-state-disabled, .dx-state-disabled *"))},_dragStartHandler:function(e){var t=this._getDraggableElement(e);if(!this._$sourceElement){if(!this._isValidElement(e,t))return void(e.cancel=!0);var n=this._getDragStartArgs(e,t);if(this._getAction("onDragStart")(n),n.cancel)return void(e.cancel=!0);this.option("itemData",n.itemData),this._setSourceDraggable(),this._$sourceElement=t;var i=t.offset(),o=this._$dragElement=this._createDragElement(t);this._toggleDraggingClass(!0),this._toggleDragSourceClass(!0);var a="fixed"===o.css("position");this._initPosition((0,y.extend)({},n,{dragElement:o.get(0),initialOffset:a&&i}));var r=this._getArea(),s=this._getAreaOffset(r),l=this._getBoundOffset(),u=r.outerWidth(),c=r.outerHeight(),d=o.width(),h=o.height(),f={left:o.offset().left-s.left,top:o.offset().top-s.top};r.length&&(e.maxLeftOffset=f.left-l.left,e.maxRightOffset=u-f.left-d-l.right,e.maxTopOffset=f.top-l.top,e.maxBottomOffset=c-f.top-h-l.bottom),this.option("autoScroll")&&this._startAnimator()}},_getAreaOffset:function(e){var t=e&&S.default.offset(e);return t?t:{left:0,top:0}},_toggleDraggingClass:function(e){this._$dragElement&&this._$dragElement.toggleClass(this._addWidgetPrefix("dragging"),e)},_toggleDragSourceClass:function(e,t){var n=t||this._$sourceElement;n&&n.toggleClass(this._addWidgetPrefix("source"),e)},_getBoundOffset:function(){var e=this.option("boundOffset");return(0,D.isFunction)(e)&&(e=e.call(this)),(0,f.quadToObject)(e)},_getArea:function(){var e=this.option("boundary");return(0,D.isFunction)(e)&&(e=e.call(this)),(0,c.default)(e)},_getContainer:function(){var e=this.option("container");return void 0===e&&(e=(0,T.value)()),(0,c.default)(e)},_dragMoveHandler:function(e,t){if(this._dragMoveArgs=e,!this._$dragElement)return void(e.cancel=!0);var n=e.offset,i=this._startPosition;this._move({left:i.left+n.x,top:i.top+n.y}),t||this._updateScrollable(e);var o=this._getEventArgs(e);if(this._getAction("onDragMove")(o),o.cancel!==!0){var a=this._getTargetDraggable();a.dragMove(e,t)}},_updateScrollable:function(e){var t=this;if(t.option("autoScroll")){var n=j(e),i=t.getElementsFromPoint(n);t._verticalScrollHelper.updateScrollable(i,n),t._horizontalScrollHelper.updateScrollable(i,n)}},getElementsFromPoint:function(e,t){var n=(t||this._$dragElement.get(0)).ownerDocument;if(m.default.msie){var i=n.msElementsFromPoint(e.x,e.y);return i?Array.prototype.slice.call(i):[]}return n.elementsFromPoint(e.x,e.y)},_getScrollable:function(e){var t,n=this;return e.parents().toArray().some(function(e){var i=(0,c.default)(e);if(n._horizontalScrollHelper.isScrollable(i)||n._verticalScrollHelper.isScrollable(i))return t=i,!0}),t},_defaultActionArgs:function(){var e=this.callBase.apply(this,arguments),t=this.option("component");return t&&(e.component=t,e.element=t.element()),e},_getEventArgs:function(e){var t=this._getSourceDraggable(),n=this._getTargetDraggable();return{event:e,itemData:t.option("itemData"),itemElement:(0,x.getPublicElement)(t._$sourceElement),fromComponent:t.option("component")||t,toComponent:n.option("component")||n,fromData:t.option("data"),toData:n.option("data")}},_getDragStartArgs:function(e,t){var n=this._getEventArgs(e);return{event:n.event,itemData:n.itemData,itemElement:t,fromData:n.fromData}},_revertItemToInitialPosition:function(){!this._dragElementIsCloned()&&this._move(this._initialLocate,this._$sourceElement)},_dragEndHandler:function(e){var t=this,n=new E.Deferred,i=this._getEventArgs(e),o=this._getEventArgs(e),a=this._getTargetDraggable(),r=!0;try{this._getAction("onDragEnd")(i)}finally{(0,E.when)((0,E.fromPromise)(i.cancel)).done(function(e){return e||(a!==t&&a._getAction("onDrop")(o),o.cancel)?void n.resolve():(r=!1,void(0,E.when)((0,E.fromPromise)(a.dragEnd(i))).always(n.resolve))}).fail(n.resolve),n.done(function(){r&&t._revertItemToInitialPosition(),t.reset(),a.reset(),t._stopAnimator(),t._horizontalScrollHelper.reset(),t._verticalScrollHelper.reset(),t._resetDragElement(),t._resetSourceElement(),t._resetTargetDraggable(),t._resetSourceDraggable()})}},_isTargetOverAnotherDraggable:function(e){var t=this,n=this._getSourceDraggable();if(this===n)return!1;var i=n._$dragElement,o=n.$element(),a=this.$element(),r=j(e),s=this.getElementsFromPoint(r,e.target),l=s.filter(function(e){var n=(0,c.default)(e);if(n.hasClass(t._addWidgetPrefix()))return!n.closest(i).length})[0],u=this._getSourceElement(),d=l===o.get(0),h=(0,c.default)(l).closest(u).length;return!l||l===a.get(0)&&!d&&!h},_dragEnterHandler:function(e){this._fireDragEnterEvent(e),this._isTargetOverAnotherDraggable(e)&&this._setTargetDraggable();var t=this._getSourceDraggable();t.dragEnter(e)},_dragLeaveHandler:function(e){this._fireDragLeaveEvent(e),this._resetTargetDraggable(),this!==this._getSourceDraggable()&&this.reset();var t=this._getSourceDraggable();t.dragLeave(e)},_getAction:function(e){return this["_"+e+"Action"]||this._createActionByOption(e)},_getAnonymousTemplateName:function(){return N},_initTemplates:function(){this.option("contentTemplate")&&(this._templateManager.addDefaultTemplates({content:new O.EmptyTemplate}),this.callBase.apply(this,arguments))},_render:function(){this.callBase(),this.$element().addClass(this._addWidgetPrefix());var e=this._templateManager.anonymousTemplateName===this.option("contentTemplate"),t=this._getTemplateByOption("contentTemplate");t&&(0,c.default)(t.render({container:this.element(),transclude:e}))},_optionChanged:function(e){var t=e.name;switch(t){case"onDragStart":case"onDragMove":case"onDragEnd":case"onDrop":case"onDragEnter":case"onDragLeave":this["_"+t+"Action"]=this._createActionByOption(t);break;case"dragTemplate":case"contentTemplate":case"container":case"clone":break;case"allowMoveByClick":case"dragDirection":case"disabled":case"boundary":case"filter":case"immediate":this._resetDragElement(),this._detachEventHandlers(),this._attachEventHandlers();break;case"autoScroll":this._verticalScrollHelper.reset(),this._horizontalScrollHelper.reset();break;case"scrollSensitivity":case"scrollSpeed":case"boundOffset":case"handle":case"group":case"data":case"itemData":break;default:this.callBase(e)}},_getTargetDraggable:function(){return l||this},_getSourceDraggable:function(){return u||this},_setTargetDraggable:function(){var e=this.option("group"),t=this._getSourceDraggable();e&&e===t.option("group")&&(l=this)},_setSourceDraggable:function(){u=this},_resetSourceDraggable:function(){u=null},_resetTargetDraggable:function(){l=null},_dispose:function(){this.callBase(),this._detachEventHandlers(),this._resetDragElement(),this._resetTargetDraggable(),this._resetSourceDraggable(),this._$sourceElement=null,this._stopAnimator()},_fireDragEnterEvent:function(e){var t=this._getEventArgs(e);this._getAction("onDragEnter")(t)},_fireDragLeaveEvent:function(e){var t=this._getEventArgs(e);this._getAction("onDragLeave")(t)}});(0,p.default)(R,q);var $=q;t.default=$,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(5)),y=o(n(9)),b=n(0),x=n(4),w=n(7),C=n(40),k=n(1),S=n(97),D=n(3),I=o(n(23)),T=o(n(141)),O=o(n(58)),E="dx-box",A=".dx-box",P="dx-box-item",R="dxBoxItemData",M={row:"minWidth",col:"minHeight"},B={row:"maxWidth",col:"maxHeight"},V=1,F={start:"flex-start",end:"flex-end",center:"center","space-between":"space-between","space-around":"space-around"},L={start:"flex-start",end:"flex-end",center:"center",stretch:"stretch"},H={row:"row",col:"column"},z=function(e,t,n){if(n=(0,S.normalizeStyleProp)(t,n),e.style[(0,S.styleProp)(t)]=n,!(0,w.hasWindow)()){if(""===n||!(0,k.isDefined)(n))return;var i=(0,C.dasherize)(t),o=i+": "+n+";";e.attributes.style?e.attributes.style.value.indexOf(o)<0&&(e.attributes.style.value+=" "+o):e.setAttribute("style",o)}},N="dxBox",j="dxupdate."+N,W="dx-box-fallback-item",G={row:"nowrap",col:"normal"},q={row:"width",col:"height"},$={row:"height",col:"width"},K={row:"marginLeft",col:"marginTop"},U={row:"marginRight",col:"marginBottom"},Y={row:"marginTop",col:"marginLeft"},Z={row:"marginBottom",col:"marginRight"},X={marginLeft:"marginRight",marginRight:"marginLeft"},Q=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_renderVisible",value:function(e,n){l(_(t.prototype),"_renderVisible",this).call(this,e),(0,k.isDefined)(n)&&this._options.fireItemStateChangedAction({name:"visible",state:e,oldState:n})}}]),t}(T.default),J=function(){function e(t,n){a(this,e),this._$element=t,this._option=n,this.initSize=x.noop,this.update=x.noop}return s(e,[{key:"renderBox",value:function(){this._$element.css({display:(0,S.stylePropPrefix)("flexDirection")+"flex"}),z(this._$element.get(0),"flexDirection",H[this._option("direction")])}},{key:"renderAlign",value:function(){this._$element.css({justifyContent:this._normalizedAlign()})}},{key:"_normalizedAlign",value:function(){var e=this._option("align");return e in F?F[e]:e}},{key:"renderCrossAlign",value:function(){this._$element.css({alignItems:this._normalizedCrossAlign()})}},{key:"_normalizedCrossAlign",value:function(){var e=this._option("crossAlign");return e in L?L[e]:e}},{key:"renderItems",value:function(e){var t=(0,S.stylePropPrefix)("flexDirection"),n=this._option("direction");(0,D.each)(e,function(){var e=(0,m.default)(this),i=e.data(R);e.css({display:t+"flex"}).css(B[n],i.maxSize||"none").css(M[n],i.minSize||"0"),z(e.get(0),"flexBasis",i.baseSize||0),z(e.get(0),"flexGrow",i.ratio),z(e.get(0),"flexShrink",(0,k.isDefined)(i.shrink)?i.shrink:V),e.children().each(function(e,t){(0,m.default)(t).css({width:"auto",height:"auto",display:(0,S.stylePropPrefix)("flexDirection")+"flex",flexBasis:0}),z(t,"flexGrow",1),z(t,"flexDirection",(0,m.default)(t)[0].style.flexDirection||"column")})})}}]),e}(),ee=function(){function e(t,n){a(this,e),this._$element=t,this._option=n}return s(e,[{key:"renderBox",value:function(){this._$element.css({fontSize:0,whiteSpace:G[this._option("direction")],verticalAlign:"top"}),v.default.off(this._$element,j),v.default.on(this._$element,j,this.update.bind(this))}},{key:"renderAlign",value:function(){var e=this._$items;if(e){var t=this._option("align"),n=this.totalItemSize,i=this._option("direction"),o=this._$element[q[i]](),a=o-n,r=0;switch(this._setItemsMargins(e,i,0),t){case"start":break;case"end":r=a,e.first().css(this._chooseMarginSide(K[i]),r);break;case"center":r=.5*a,e.first().css(this._chooseMarginSide(K[i]),r),e.last().css(this._chooseMarginSide(U[i]),r);break;case"space-between":r=.5*a/(e.length-1),this._setItemsMargins(e,i,r),e.first().css(this._chooseMarginSide(K[i]),0),e.last().css(this._chooseMarginSide(U[i]),0);break;case"space-around":r=.5*a/e.length,this._setItemsMargins(e,i,r)}}}},{key:"_setItemsMargins",value:function(e,t,n){e.css(this._chooseMarginSide(K[t]),n).css(this._chooseMarginSide(U[t]),n)}},{key:"renderCrossAlign",value:function(){var e=this._$items;if(e){var t=this._option("crossAlign"),n=this._option("direction"),i=this._$element[$[n]](),o=this;switch(t){case"start":break;case"end":(0,D.each)(e,function(){var e=(0,m.default)(this),t=e[$[n]](),a=i-t;e.css(o._chooseMarginSide(Y[n]),a)});break;case"center":(0,D.each)(e,function(){var e=(0,m.default)(this),t=e[$[n]](),a=.5*(i-t);e.css(o._chooseMarginSide(Y[n]),a).css(o._chooseMarginSide(Z[n]),a)});break;case"stretch":e.css(o._chooseMarginSide(Y[n]),0).css(o._chooseMarginSide(Z[n]),0).css($[n],"100%")}}}},{key:"_chooseMarginSide",value:function(e){return this._option("rtlEnabled")?X[e]||e:e}},{key:"renderItems",value:function(e){var t=this;this._$items=e;var n=this._option("direction"),i=0,o=0,a=0;(0,D.each)(e,function(e,r){var s=(0,m.default)(r);s.css({display:"inline-block",verticalAlign:"top"}),s[q[n]]("auto"),s.removeClass(W);var l=s.data(R),u=l.ratio||0,c=t._baseSize(s),d=(0,k.isDefined)(l.shrink)?l.shrink:V;i+=u,o+=d*c,a+=c});var r=this._boxSize()-a,s=function(e){var n=e.data(R),a=t._baseSize(e),s=r>=0?n.ratio||0:((0,k.isDefined)(n.shrink)?n.shrink:V)*a,l=r>=0?i:o,u=l?Math.round(r*s/l):0;return a+u},l=0;(0,D.each)(e,function(e,t){var i=(0,m.default)(t),o=(0,m.default)(t).data(R),a=s(i);l+=a,i.css(B[n],o.maxSize||"none").css(M[n],o.minSize||"0").css(q[n],a),i.addClass(W)}),this.totalItemSize=l}},{key:"_baseSize",value:function(e){var t=(0,m.default)(e).data(R);return null==t.baseSize?0:"auto"===t.baseSize?this._contentSize(e):this._parseSize(t.baseSize)}},{key:"_contentSize",value:function(e){return(0,m.default)(e)[q[this._option("direction")]]()}},{key:"_parseSize",value:function(e){return String(e).match(/.+%$/)?.01*parseFloat(e)*this._boxSizeValue:e}},{key:"_boxSize",value:function(e){return arguments.length?void(this._boxSizeValue=e):(this._boxSizeValue=this._boxSizeValue||this._totalBaseSize(),this._boxSizeValue)}},{key:"_totalBaseSize",value:function(){var e=this,t=0;return(0,D.each)(this._$items,function(n,i){t+=e._baseSize(i)}),t}},{key:"initSize",value:function(){this._boxSize(this._$element[q[this._option("direction")]]())}},{key:"update",value:function(){if(this._$items&&!this._$element.is(":hidden")){this._$items.detach(),this.initSize(),this._$element.append(this._$items),this.renderItems(this._$items),this.renderAlign(),this.renderCrossAlign();var e=this._$element.get(0);this._$items.find(A).each(function(){e===(0,m.default)(this).parent().closest(A).get(0)&&v.default.triggerHandler(this,j)})}}}]),e}(),te=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,b.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{direction:"row",align:"start",crossAlign:"stretch",activeStateEnabled:!1,focusStateEnabled:!1,onItemStateChanged:void 0,_layoutStrategy:"flex",_queue:void 0})}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return I.default.msie},options:{_layoutStrategy:"fallback"}}])}},{key:"_itemClass",value:function(){return P}},{key:"_itemDataKey",value:function(){return R}},{key:"_itemElements",value:function(){return this._itemContainer().children(this._itemSelector())}},{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this.$element().addClass("".concat(E,"-").concat(this.option("_layoutStrategy"))),this._initLayout(),this._initBoxQueue()}},{key:"_initLayout",value:function(){this._layout="fallback"===this.option("_layoutStrategy")?new ee(this.$element(),this.option.bind(this)):new J(this.$element(),this.option.bind(this))}},{key:"_initBoxQueue",value:function(){this._queue=this.option("_queue")||[]}},{key:"_queueIsNotEmpty",value:function(){return!this.option("_queue")&&!!this._queue.length}},{key:"_pushItemToQueue",value:function(e,t){this._queue.push({$item:e,config:t})}},{key:"_shiftItemFromQueue",value:function(){return this._queue.shift()}},{key:"_initMarkup",value:function(){this.$element().addClass(E),this._layout.renderBox(),l(_(t.prototype),"_initMarkup",this).call(this),this._renderAlign(),this._renderActions()}},{key:"_renderActions",value:function(){this._onItemStateChanged=this._createActionByOption("onItemStateChanged")}},{key:"_renderAlign",value:function(){this._layout.renderAlign(),this._layout.renderCrossAlign()}},{key:"_renderItems",value:function(e){var n=this;for(this._layout.initSize(),l(_(t.prototype),"_renderItems",this).call(this,e);this._queueIsNotEmpty();){var i=this._shiftItemFromQueue();this._createComponent(i.$item,t,(0,b.extend)({_layoutStrategy:this.option("_layoutStrategy"),itemTemplate:this.option("itemTemplate"),itemHoldTimeout:this.option("itemHoldTimeout"),onItemHold:this.option("onItemHold"),onItemClick:this.option("onItemClick"),onItemContextMenu:this.option("onItemContextMenu"),onItemRendered:this.option("onItemRendered"),_queue:this._queue},i.config))}this._layout.renderItems(this._itemElements()),clearTimeout(this._updateTimer),this._updateTimer=setTimeout(function(){n._isUpdated||n._layout.update(),n._isUpdated=!1,n._updateTimer=null})}},{key:"_renderItemContent",value:function(e){var n=e.itemData&&e.itemData.node;return n?this._renderItemContentByNode(e,n):l(_(t.prototype),"_renderItemContent",this).call(this,e)}},{key:"_postprocessRenderItem",value:function(e){var t=e.itemData.box;t&&this._pushItemToQueue(e.itemContent,t)}},{key:"_createItemByTemplate",value:function(e,n){return n.itemData.box?e.source?e.source():(0,m.default)():l(_(t.prototype),"_createItemByTemplate",this).call(this,e,n)}},{key:"_visibilityChanged",value:function(e){e&&this._dimensionChanged()}},{key:"_dimensionChanged",value:function(){this._updateTimer||(this._isUpdated=!0,this._layout.update())}},{key:"_dispose",value:function(){clearTimeout(this._updateTimer),l(_(t.prototype),"_dispose",this).apply(this,arguments)}},{key:"_itemOptionChanged",value:function(e,n,i,o){"visible"===n&&this._onItemStateChanged({name:n,state:i,oldState:o!==!1}),l(_(t.prototype),"_itemOptionChanged",this).call(this,e,n,i)}},{key:"_optionChanged",value:function(e){switch(e.name){case"_layoutStrategy":case"_queue":case"direction":this._invalidate();break;case"align":this._layout.renderAlign();break;case"crossAlign":this._layout.renderCrossAlign();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_itemOptions",value:function(){var e=this,n=l(_(t.prototype),"_itemOptions",this).call(this);return n.fireItemStateChangedAction=function(t){e._onItemStateChanged(t)},n}},{key:"repaint",value:function(){this._dimensionChanged()}}]),t}(O.default);te.ItemClass=Q,(0,y.default)("dxBox",te);var ne=te;t.default=ne,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(621)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=i(n(15)),l=i(n(32)),u=s.default.abstract,c=s.default.inherit({ctor:function(e){this.dateBox=e},widgetOption:function(){return this._widget&&this._widget.option.apply(this._widget,arguments)},_renderWidget:function(e){e=e||(0,o.default)("<div>"),this._widget=this._createWidget(e),this._widget.$element().appendTo(this._getWidgetContainer())},_createWidget:function(e){var t=this._getWidgetName(),n=this._getWidgetOptions();return this.dateBox._createComponent(e,t,n)},_getWidgetOptions:u,_getWidgetName:u,getDefaultOptions:function(){return{mode:"text"}},getDisplayFormat:u,supportedKeys:r.noop,getKeyboardListener:r.noop,customizeButtons:r.noop,getParsedText:function(e,t){var n=l.default.parse(e,t);return n?n:l.default.parse(e)},renderInputMinMax:r.noop,renderOpenedState:function(){this._updateValue()},popupConfig:u,_dimensionChanged:function(){var e;null===(e=this._getPopup())||void 0===e?void 0:e.repaint()},renderPopupContent:function(){var e=this._getPopup();this._renderWidget();var t=e.$content().parent();a.default.off(t,"mousedown"),a.default.on(t,"mousedown",this._preventFocusOnPopup.bind(this))},getFirstPopupElement:r.noop,getLastPopupElement:r.noop,_preventFocusOnPopup:function(e){e.preventDefault()},_getWidgetContainer:function(){return this._getPopup().$content()},_getPopup:function(){return this.dateBox._popup},popupShowingHandler:r.noop,popupHiddenHandler:r.noop,_updateValue:function(){this._widget&&this._widget.option("value",this.dateBoxValue())},useCurrentDateByDefault:r.noop,getDefaultDate:function(){return new Date},textChangedHandler:r.noop,renderValue:function(){this.dateBox.option("opened")&&this._updateValue()},getValue:function(){return this._widget.option("value")},isAdaptivityChanged:function(){return!1},dispose:function(){var e=this._getPopup();e&&e.$content().empty()},dateBoxValue:function(){return arguments.length?this.dateBox.dateValue.apply(this.dateBox,arguments):this.dateBox.dateOption.apply(this.dateBox,["value"])}}),d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(13)),s=i(n(9)),l=n(0),u=i(n(16)),c=i(n(38)),d=i(n(143)),h=i(n(94)),f=i(n(115)),p=n(36),g=n(154),_=(0,a.getWindow)(),m="dx-dropdownmenu",v="dx-dropdownmenu-popup",y="dx-dropdownmenu-popup-wrapper",b="dx-dropdownmenu-list",x="dx-dropdownmenu-button",w={popupWidth:"width",popupHeight:"height",popupMaxHeight:"maxHeight",popupAutoResizeEnabled:"autoResizeEnabled"},C={buttonIcon:"icon",buttonText:"text",buttonWidth:"width",buttonHeight:"height",buttonTemplate:"template"},k=u.default.inherit({_supportedKeys:function(){var e={};return this.option("opened")&&this._list.option("focusedElement")||(e=this._button._supportedKeys()),(0,l.extend)(this.callBase(),e,{tab:function(){this._popup&&this._popup.hide()}})},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{items:[],onItemClick:null,dataSource:null,itemTemplate:"item",buttonText:"",buttonIcon:"overflow",buttonWidth:void 0,buttonHeight:void 0,buttonTemplate:"content",onButtonClick:null,usePopover:!1,popupWidth:"auto",popupHeight:"auto",activeStateEnabled:!0,hoverStateEnabled:!0,opened:!1,selectionMode:"none",selectedItemKeys:[],deferRendering:!1,popupPosition:{my:"top center",at:"bottom center",collision:"fit flip",offset:{v:1}},popupAnimation:void 0,onItemRendered:null,menuWidget:f.default,popupMaxHeight:void 0,closeOnClick:!0,useInkRipple:!1,container:void 0,popupAutoResizeEnabled:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios"},options:{usePopover:!0}},{device:{platform:"generic"},options:{popupPosition:{offset:{v:4}}}},{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}},{device:{platform:"android"},options:{popupPosition:{my:"top "+(this.option("rtlEnabled")?"left":"right"),at:"top "+(this.option("rtlEnabled")?"left":"right"),collision:"flipfit"},popupAnimation:{show:{type:"pop",duration:200,from:{scale:0},to:{scale:1}},hide:{type:"pop",duration:200,from:{scale:1},to:{scale:0}}}}},{device:function(){return(0,p.isMaterial)()},options:{useInkRipple:!0}}])},_initOptions:function(e){"android"===r.default.current().platform&&(e.popupPosition||(e.popupPosition={at:(e.usePopover?"bottom ":"top ")+(e.rtlEnabled?"left":"right")})),this.callBase(e)},_dataSourceOptions:function(){return{paginate:!1}},_init:function(){this.callBase(),this.$element().addClass(m),this._initDataSource(),this._initItemClickAction(),this._initButtonClickAction()},_initItemClickAction:function(){this._itemClickAction=this._createActionByOption("onItemClick")},_initButtonClickAction:function(){this._buttonClickAction=this._createActionByOption("onButtonClick")},_initTemplates:function(){this._templateManager.addDefaultTemplates({content:new g.ChildDefaultTemplate("content")}),this.callBase()},_initMarkup:function(){this._renderButton(),this.callBase()},_render:function(){this.callBase(),this.setAria({role:"menubar",haspopup:!0,expanded:this.option("opened")})},_renderContentImpl:function(){this.option("opened")&&this._renderPopup()},_clean:function(){this._cleanFocusState(),this._popup&&(this._popup.$element().remove(),delete this._$popup)},_renderButton:function(){var e=this.$element().addClass(x),t=this._buttonOptions();this._button=this._createComponent(e,c.default,t)},_toggleActiveState:function(e,t,n){this._button._toggleActiveState(e,t,n)},_buttonOptions:function(){return{text:this.option("buttonText"),icon:this.option("buttonIcon"),width:this.option("buttonWidth"),height:this.option("buttonHeight"),useInkRipple:this.option("useInkRipple"),template:this.option("buttonTemplate"),focusStateEnabled:!1,onClick:function(e){this.option("opened",!this.option("opened")),this._buttonClickAction(e)}.bind(this)}},_toggleMenuVisibility:function(e){var t=void 0===e?!this._popup.option("visible"):e;e&&this._renderPopup(),this._popup.toggle(t),this.setAria("expanded",t)},_renderPopup:function(){if(!this._$popup){var e=this._$popup=(0,o.default)("<div>").appendTo(this.$element()),t=this._popupOptions();this._popup=this._createComponent(e,d.default,t)}},_popupOptions:function(){var e=!this.option("usePopover");return{onInitialized:function(t){t.component._wrapper().addClass(y).toggleClass(v,e)},visible:this.option("opened"),deferRendering:!1,contentTemplate:function(e){this._renderList(e)}.bind(this),position:this.option("popupPosition"),animation:this.option("popupAnimation"),onOptionChanged:function(e){"visible"===e.name&&this.option("opened",e.value)}.bind(this),target:this.$element(),height:this.option("popupHeight"),width:this.option("popupWidth"),maxHeight:this.option("popupMaxHeight"),container:this.option("container"),autoResizeEnabled:this.option("popupAutoResizeEnabled")}},_renderList:function(e){var t=(0,o.default)(e),n=this._listOptions();t.addClass(b),this._list=this._createComponent(t,this.option("menuWidget"),n),this._list._getAriaTarget=function(){return this.$element()}.bind(this),this._setListDataSource();var i=.5*(0,o.default)(_).height();t.height()>i&&t.height(i)},_listOptions:function(){return{pageLoadMode:"scrollBottom",indicateLoading:!1,noDataText:"",selectionMode:this.option("selectionMode"),selectedItemKeys:this.option("selectedItemKeys"),itemTemplate:this.option("itemTemplate"),onItemClick:function(e){this.option("closeOnClick")&&this.option("opened",!1),this._itemClickAction(e)}.bind(this),tabIndex:-1,focusStateEnabled:this.option("focusStateEnabled"),activeStateEnabled:this.option("activeStateEnabled"),onItemRendered:this.option("onItemRendered"),_itemAttributes:{role:"menuitem"}}},_setListDataSource:function(){this._list&&this._list.option("dataSource",this._dataSource||this.option("items")),delete this._deferRendering},_getKeyboardListeners:function(){return this.callBase().concat([this._list])},_toggleVisibility:function(e){this.callBase(e),this._button.option("visible",e)},_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"items":case"dataSource":this.option("deferRendering")&&!this.option("opened")?this._deferRendering=!0:(this._refreshDataSource(),this._setListDataSource());break;case"itemTemplate":this._list&&this._list.option(t,this._getTemplate(n));break;case"onItemClick":this._initItemClickAction();break;case"onButtonClick":this._buttonClickAction();break;case"buttonIcon":case"buttonText":case"buttonWidth":case"buttonHeight":case"buttonTemplate":this._button.option(C[t],n),this._renderPopup();break;case"popupWidth":case"popupHeight":case"popupMaxHeight":case"popupAutoResizeEnabled":this._popup.option(w[t],n);break;case"usePopover":case"menuWidget":case"useInkRipple":this._invalidate();break;case"focusStateEnabled":case"activeStateEnabled":this._list&&this._list.option(t,n),this.callBase(e);break;case"selectionMode":case"selectedItemKeys":case"onItemRendered":this._list&&this._list.option(t,n);break;case"opened":this._deferRendering&&(this._refreshDataSource(),this._setListDataSource()),this._toggleMenuVisibility(n);break;case"deferRendering":case"popupPosition":case"closeOnClick":break;case"container":this._popup&&this._popup.option(e.name,e.value);break;default:this.callBase(e)}},open:function(){this.option("opened",!0)},close:function(){this.option("opened",!1)}}).include(h.default);(0,s.default)("dxDropDownMenu",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=i(n(9)),l=i(n(38)),u=n(87),c=n(8),d=n(0),h=n(1),f=i(n(25)),p=n(3),g=i(n(439)),_=n(440),m=n(36),v=i(n(103)),y=i(n(109)),b=i(n(214)),x=n(59),w=n(71),C=n(6),k="dx-tabs",S="dx-tabs-wrapper",D="dx-tabs-stretched",I="dx-tabs-scrollable",T="dx-tabs-nav-buttons",O="dx-overflow-hidden",E="dx-tab",A="dx-tab-selected",P="dx-tabs-nav-button",R="dx-tabs-nav-button-left",M="dx-tabs-nav-button-right",B="dx-tab-text",V="dxTabData",F="chevronnext",L="chevronprev",H=100,z=5,N=300,j=30,W=b.default.inherit({_activeStateUnit:"."+E,_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{hoverStateEnabled:!0,showNavButtons:!0,scrollByContent:!0,scrollingEnabled:!0,selectionMode:"single",activeStateEnabled:!0,selectionRequired:!1,selectOnFocus:!0,loopItemFocus:!1,useInkRipple:!1,badgeExpr:function(e){return e?e.badge:void 0},_itemAttributes:{role:"tab"}})},_defaultOptionsRules:function(){var e=(0,m.current)();return this.callBase().concat([{device:function(){return"desktop"!==r.default.real().deviceType},options:{showNavButtons:!1}},{device:{deviceType:"desktop"},options:{scrollByContent:!1}},{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,m.isMaterial)(e)},options:{useInkRipple:!0,selectOnFocus:!1}}])},_init:function(){this.callBase(),this.setAria("role","tablist"),this.$element().addClass(k),this._renderWrapper(),this._renderMultiple(),this._feedbackHideTimeout=H},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({item:new w.BindableTemplate(function(e,t){
(0,h.isPlainObject)(t)?this._prepareDefaultItemTemplate(t,e):e.text(String(t));var n=(0,x.getImageContainer)(t.icon);n&&n.prependTo(e),e.wrapInner((0,o.default)("<span>").addClass(B))}.bind(this),["text","html","icon"],this.option("integrationOptions.watchMethod"))})},_createItemByTemplate:function(e,t){var n=this,i=t.itemData,o=t.container,a=t.index;return this._deferredTemplates[a]=new C.Deferred,e.render({model:i,container:o,index:a,onRendered:function(){return n._deferredTemplates[a].resolve()}})},_itemClass:function(){return E},_selectedItemClass:function(){return A},_itemDataKey:function(){return V},_initMarkup:function(){this._deferredTemplates=[],this.callBase(),this.option("useInkRipple")&&this._renderInkRipple(),this.$element().addClass(O)},_render:function(){this.callBase(),this._deferRenderScrolling()},_deferRenderScrolling:function(){var e=this;C.when.apply(this,this._deferredTemplates).done(function(){return e._renderScrolling()})},_renderScrolling:function(){var e=[D,_.TABS_EXPANDED_CLASS,O];this.$element().removeClass(e.join(" ")),this.option("scrollingEnabled")&&this._isItemsWidthExceeded()&&(this._scrollable||(this._renderScrollable(),this._renderNavButtons()),this._scrollable.update(),this._updateNavButtonsVisibility(),this.option("rtlEnabled")&&this._scrollable.scrollTo({left:this._scrollable.scrollWidth()-this._scrollable.clientWidth()}),this._scrollToItem(this.option("selectedItem"))),this.option("scrollingEnabled")&&this._isItemsWidthExceeded()||(this._cleanScrolling(),this._needStretchItems()&&!this._isItemsWidthExceeded()&&this.$element().addClass(D),this.$element().removeClass(T).addClass(_.TABS_EXPANDED_CLASS))},_isItemsWidthExceeded:function(){var e=this._getSummaryItemsWidth(this._getVisibleItems(),!0);return e-1>this.$element().width()},_needStretchItems:function(){var e=this._getVisibleItems(),t=this.$element().width(),n=[];(0,p.each)(e,function(e,t){n.push((0,o.default)(t).outerWidth(!0))});var i=Math.max.apply(null,n);return i>t/e.length},_cleanNavButtons:function(){this._leftButton&&this._rightButton&&(this._leftButton.$element().remove(),this._rightButton.$element().remove(),this._leftButton=null,this._rightButton=null)},_cleanScrolling:function(){this._scrollable&&(this._$wrapper.appendTo(this.$element()),this._scrollable.$element().remove(),this._scrollable=null,this._cleanNavButtons())},_renderInkRipple:function(){this._inkRipple=(0,u.render)()},_toggleActiveState:function(e,t,n){if(this.callBase.apply(this,arguments),this._inkRipple){var i={element:e,event:n};t?this._inkRipple.showWave(i):this._inkRipple.hideWave(i)}},_renderMultiple:function(){"multiple"===this.option("selectionMode")&&this.option("selectOnFocus",!1)},_renderWrapper:function(){this._$wrapper=(0,o.default)("<div>").addClass(S),this.$element().append(this._$wrapper)},_itemContainer:function(){return this._$wrapper},_renderScrollable:function(){var e=this.$element().wrapInner((0,o.default)("<div>").addClass(I)).children();this._scrollable=this._createComponent(e,y.default,{direction:"horizontal",showScrollbar:!1,useKeyboard:!1,useNative:!1,scrollByContent:this.option("scrollByContent"),onScroll:this._updateNavButtonsVisibility.bind(this)}),this.$element().append(this._scrollable.$element())},_scrollToItem:function(e){if(this._scrollable){var t=this._editStrategy.getItemElement(e);this._scrollable.scrollToElement(t)}},_renderNavButtons:function(){if(this.$element().toggleClass(T,this.option("showNavButtons")),this.option("showNavButtons")){var e=this.option("rtlEnabled");this._leftButton=this._createNavButton(-j,e?F:L);var t=this._leftButton.$element();t.addClass(R),this.$element().prepend(t),this._rightButton=this._createNavButton(j,e?L:F);var n=this._rightButton.$element();n.addClass(M),this.$element().append(n)}},_updateNavButtonsVisibility:function(){this._leftButton&&this._leftButton.option("disabled",this._scrollable.scrollLeft()<=0),this._rightButton&&this._rightButton.option("disabled",this._scrollable.scrollLeft()>=Math.round(this._scrollable.scrollWidth()-this._scrollable.clientWidth()))},_updateScrollPosition:function(e,t){this._scrollable.update(),this._scrollable.scrollBy(e/t)},_createNavButton:function(e,t){var n=this,i=n._createAction(function(){n._holdInterval=setInterval(function(){n._updateScrollPosition(e,z)},z)}),r=(0,c.addNamespace)(v.default.name,"dxNavButton"),s=(0,c.addNamespace)(f.default.up,"dxNavButton"),u=(0,c.addNamespace)(f.default.out,"dxNavButton"),d=this._createComponent((0,o.default)("<div>").addClass(P),l.default,{focusStateEnabled:!1,icon:t,onClick:function(){n._updateScrollPosition(e,1)},integrationOptions:{}}),h=d.$element();return a.default.on(h,r,{timeout:N},function(e){i({event:e})}.bind(this)),a.default.on(h,s,function(){n._clearInterval()}),a.default.on(h,u,function(){n._clearInterval()}),d},_clearInterval:function(){this._holdInterval&&clearInterval(this._holdInterval)},_updateSelection:function(e){this._scrollable&&this._scrollable.scrollToElement(this.itemElements().eq(e[0]),{left:1,right:1})},_visibilityChanged:function(e){e&&this._dimensionChanged()},_dimensionChanged:function(){this._renderScrolling()},_itemSelectHandler:function(e){"single"===this.option("selectionMode")&&this.isItemSelected(e.currentTarget)||this.callBase(e)},_clean:function(){this._deferredTemplates=[],this._cleanScrolling(),this.callBase()},_optionChanged:function(e){switch(e.name){case"useInkRipple":case"scrollingEnabled":case"showNavButtons":this._invalidate();break;case"scrollByContent":this._scrollable&&this._scrollable.option(e.name,e.value);break;case"width":this.callBase(e),this._dimensionChanged();break;case"selectionMode":this._renderMultiple(),this.callBase(e);break;case"badgeExpr":this._invalidate();break;default:this.callBase(e)}},_afterItemElementInserted:function(){this.callBase(),this._deferRenderScrolling()},_afterItemElementDeleted:function(e,t){this.callBase(e,t),this._renderScrolling()}});W.ItemClass=g.default,(0,s.default)("dxTabs",W);var G=W;t.default=G,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(39),a=i(n(27)),r=i(n(17)),s=i(n(65)),l=n(0),u=n(3),c=i(n(100)),d=i(n(644)),h=i(n(9)),f=n(6),p=i(n(35)),g="dx-validator",_="valid",m="invalid",v="pending",y=s.default.inherit({_initOptions:function(e){this.callBase.apply(this,arguments),this.option(c.default.initValidationOptions(e))},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{validationRules:[]})},_init:function(){this.callBase(),this._initGroupRegistration(),this.focused=(0,a.default)(),this._initAdapter(),this._validationInfo={result:null,deferred:null,skipValidation:!1}},_initGroupRegistration:function(){var e=this._findGroup();this._groupWasInit||this.on("disposing",function(e){c.default.removeRegisteredValidator(e.component._validationGroup,e.component)}),this._groupWasInit&&this._validationGroup===e||(c.default.removeRegisteredValidator(this._validationGroup,this),this._groupWasInit=!0,this._validationGroup=e,c.default.registerValidatorInGroup(e,this))},_setOptionsByReference:function(){this.callBase(),(0,l.extend)(this._optionsByReference,{validationGroup:!0})},_initAdapter:function(){var e=this,t=this.$element()[0],n=(0,o.data)(t,"dx-validation-target"),i=this.option("adapter");if(!i){if(n)return i=new d.default(n,this),i.validationRequestsCallbacks.push(function(t){e._validationInfo.skipValidation||e.validate(t)}),void this.option("adapter",i);throw r.default.Error("E0120")}var a=i.validationRequestsCallbacks;a&&a.push(function(t){e.validate(t)})},_toggleRTLDirection:function(e){var t,n,i,o=null!==(t=null===(n=this.option("adapter"))||void 0===n?void 0:null===(i=n.editor)||void 0===i?void 0:i.option("rtlEnabled"))&&void 0!==t?t:e;this.callBase(o)},_initMarkup:function(){this.$element().addClass(g),this.callBase()},_visibilityChanged:function(e){e&&this._initGroupRegistration()},_optionChanged:function(e){switch(e.name){case"validationGroup":return void this._initGroupRegistration();case"validationRules":return this._resetValidationRules(),void(void 0!==this.option("isValid")&&this.validate());case"adapter":this._initAdapter();break;case"isValid":case"validationStatus":this.option(c.default.synchronizeValidationOptions(e,this.option()));break;default:this.callBase(e)}},_getValidationRules:function(){var e=this;return this._validationRules||(this._validationRules=(0,u.map)(this.option("validationRules"),function(t,n){return(0,l.extend)({},t,{validator:e,index:n})})),this._validationRules},_findGroup:function(){var e=this.$element();return this.option("validationGroup")||c.default.findGroup(e,this._modelByElement(e))},_resetValidationRules:function(){delete this._validationRules},validate:function(e){var t=this,n=this.option("adapter"),i=this.option("name"),o=n.bypass&&n.bypass(),a=e&&void 0!==e.value?e.value:n.getValue(),r=n.getCurrentValidationError&&n.getCurrentValidationError(),s=this._getValidationRules(),u=this._validationInfo&&this._validationInfo.result;if(u&&u.status===v&&u.value===a)return(0,l.extend)({},u);var d;return o?d={isValid:!0,status:_}:r&&r.editorSpecific?(r.validator=this,d={isValid:!1,status:m,brokenRule:r,brokenRules:[r]}):d=c.default.validate(a,s,i),d.id=(new p.default).toString(),this._applyValidationResult(d,n),d.complete&&d.complete.then(function(e){e.id===t._validationInfo.result.id&&t._applyValidationResult(e,n)}),(0,l.extend)({},this._validationInfo.result)},reset:function(){var e=this.option("adapter"),t={id:null,isValid:!0,brokenRule:null,brokenRules:null,pendingRules:null,status:_,complete:null};this._validationInfo.skipValidation=!0,e.reset(),this._validationInfo.skipValidation=!1,this._resetValidationRules(),this._applyValidationResult(t,e)},_updateValidationResult:function(e){if(this._validationInfo.result&&this._validationInfo.result.id===e.id)for(var t in e)"id"!==t&&"complete"!==t&&(this._validationInfo.result[t]=e[t]);else{var n=this._validationInfo.deferred&&this._validationInfo.result.complete;this._validationInfo.result=(0,l.extend)({},e,{complete:n})}},_applyValidationResult:function(e,t){var n=this._createActionByOption("onValidated",{excludeValidators:["readOnly"]});return e.validator=this,this._updateValidationResult(e),t.applyValidationResults&&t.applyValidationResults(this._validationInfo.result),this.option({validationStatus:this._validationInfo.result.status}),this._validationInfo.result.status===v?(this._validationInfo.deferred||(this._validationInfo.deferred=new f.Deferred,this._validationInfo.result.complete=this._validationInfo.deferred.promise()),void this._eventsStrategy.fireEvent("validating",[this._validationInfo.result])):void(this._validationInfo.result.status!==v&&(n(e),this._validationInfo.deferred&&(this._validationInfo.deferred.resolve(e),this._validationInfo.deferred=null)))},focus:function(){var e=this.option("adapter");e&&e.focus&&e.focus()},_useTemplates:function(){return!1}});(0,h.default)("dxValidator",y);var b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(464)),a=o.default,r={extend:function(e){a=a.inherit(e)},create:function(e){return new a(e)}};t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=e.format||Se[e.dataType];return he.default.format(t,n)}function a(e){return e&&e.length>1&&"!"===e[0]&&!T(e)}function r(e){return a(e)?e[1]:e}function s(e,t){return a(e)?e[1]=t:e=t,e}function l(e,t){var n=function(e){return e.indexOf("!")!==-1},i=function(e){var t=e.slice(0);e.length=0,e.push("!",t)},o=function(e){var t=r(e);e.length=0,[].push.apply(e,t)};n(t)?a(e)||i(e):a(e)&&o(e)}function u(e,t){l(e,t);var n,i=r(e),o=function(e){return e.indexOf("!")===-1?e:e.substring(1)},a=function(e,t){for(n=0;n<e.length;n++)Array.isArray(e[n])||(e[n]=t)};return t=o(t),a(i,t),e}function c(e,t){var n=h(e);return t.filter(function(e){return e.value===n})[0]}function d(e){if(T(e))return xe;for(var t="",n=0;n<e.length;n++){var i=e[n];if(!Array.isArray(i)){if(t&&t!==i)throw new se.default.errors.Error("E4019");"!"!==i&&(t=i)}}return t}function h(e){var t=r(e),n=d(t);return n||(n=xe),t!==e&&(n="!"+n),n}function f(e){return e.lookup&&De||ke[e.dataType||ye]}function p(e){return Array.isArray(e)&&e.length}function g(e){var t=p(e.filterOperations)?e.filterOperations:f(e);return(0,fe.extend)([],t)}function _(e,t){var n=ve.default.getNameByFilterOperation(e);return t&&t[n]?t[n]:n}function m(e,t){for(var n=0;n<t.length;n++)if(t[n].value===e)return t[n];throw new ce.default.Error("E1048",e)}function v(e,t){var n=e.filter(function(e){return e.name===t});return n.length?n[0]:null}function y(e,t,n){var i=g(e),o=!!e.lookup;return n.forEach(function(t){if(!e.filterOperations&&i.indexOf(t.name)===-1){var n=t&&t.dataTypes,a=!!o&&!!t.notForLookup;!a&&n&&n.indexOf(e.dataType||ye)>=0&&i.push(t.name)}}),i.map(function(e){var i=v(n,e);return i?{icon:i.icon||be,text:i.caption||(0,pe.captionize)(i.name),value:i.name,isCustom:!0}:{icon:ve.default.getIconByFilterOperation(e)||be,text:_(e,t),value:e}})}function b(e){return e.defaultFilterOperation||g(e)[0]}function x(e,t){var n=[e.dataField,"",""],i=b(e);return Y(n,i,t),n}function w(e,t){var n=r(e),i=n.indexOf(t);return n.splice(i,1),1!==n.length&&n.splice(i,1),e}function C(e){return e.indexOf("not")!==-1?["!",[e.substring(3).toLowerCase()]]:[e]}function k(e){var t=r(e);if(T(t))return!1;var n=t.some(function(e){return T(e)});return!n}function S(e,t){var n=r(t),i=h(n);return 1===n.length?n.unshift(e):n.push(e,i),t}function D(e,t){for(var n=0;n<t.length;n++){if(t[n].name===e)return t[n];if(t[n].dataField.toLowerCase()===e.toLowerCase())return t[n]}var i=q(t,!0).filter(function(t){return t.dataField.toLowerCase()===e.toLowerCase()});if(i.length>0)return i[0];throw new ce.default.Error("E1047",e)}function I(e){return!!Array.isArray(e)&&(e.length<2||Array.isArray(e[0])||Array.isArray(e[1]))}function T(e){return!!Array.isArray(e)&&(e.length>1&&!Array.isArray(e[0])&&!Array.isArray(e[1]))}function O(e,t){for(var n=d(e).toLowerCase()||xe,i=[],o=0;o<e.length;o++)I(e[o])?(i.push(P(e[o],t)),i.push(n)):T(e[o])&&(i.push(A(e[o],t)),i.push(n));return 0===i.length&&i.push(n),i}function E(e,t){var n=v(t,e[1]);return n&&n.name===e[1]}function A(e,t){return E(e,t)?e:(e.length<3&&(e[2]=e[1],e[1]=we),e)}function P(e,t){return e?(e=(0,fe.extend)(!0,[],e),T(e)?[A(e,t),xe]:a(e)?["!",T(e[1])?[A(e[1],t),xe]:a(e[1])?[P(e[1],t),xe]:O(e[1],t)]:O(e,t)):[xe]}function R(e){return e.reduce(function(e,t){if((0,le.isDefined)(t.dataField)){var n={};for(var i in t)t[i]&&Ie.indexOf(i)>-1&&(n[i]=t[i]);n.defaultCalculateFilterExpression=de.default.defaultCalculateFilterExpression,(0,le.isDefined)(n.dataType)||(n.dataType=ye),(0,le.isDefined)(n.trueText)||(n.trueText=_e.default.format("dxDataGrid-trueText")),(0,le.isDefined)(n.falseText)||(n.falseText=_e.default.format("dxDataGrid-falseText")),e.push(n)}return e},[])}function M(e,t,n,i){var o=D(e[0],t),a=A(e,n),r=n.length&&v(n,a[1]);return r&&r.calculateFilterExpression?r.calculateFilterExpression.apply(r,[a[2],o,t]):o.createFilterExpression?o.createFilterExpression.apply(o,[a[2],a[1],i]):o.calculateFilterExpression?o.calculateFilterExpression.apply(o,[a[2],a[1],i]):o.defaultCalculateFilterExpression.apply(o,[a[2],a[1],i])}function B(e,t,n,i){if(!(0,le.isDefined)(e))return null;if(a(e)){var o=B(e[1],t,n,i);return["!",o]}var s=r(e);if(T(s))return M(s,t,n,i)||null;for(var l,u=[],c=h(s),d=0;d<s.length;d++)I(s[d])?(l=B(s[d],t,n,i),l&&(d&&u.push(c),u.push(l))):T(s[d])&&(l=M(s[d],t,n,i),l&&(u.length&&u.push(c),u.push(l)));return 1===u.length&&(u=u[0]),u.length?u:null}function V(e){var t,n=r(e);if(0===n.length)return null;var i=[];for(t=0;t<n.length;t++)if(I(n[t])){var o=V(n[t]);o?n[t]=o:i.push(n[t])}else T(n[t])&&(X(n[t])||i.push(n[t]));for(t=0;t<i.length;t++)w(n,i[t]);return 1===n.length?null:(n.splice(n.length-1,1),1===n.length&&(e=s(e,n[0])),0===e.length?null:e)}function F(e,t,n){if(""===t)return void n("");var i=e.lookup;if(i.items)n(i.calculateCellValue(t)||"");else{var o=(0,le.isFunction)(i.dataSource)?i.dataSource({}):i.dataSource,a=new me.DataSource(o);a.loadSingle(i.valueExpr,t).done(function(e){n(e?i.displayExpr?(0,ue.compileGetter)(i.displayExpr)(e):e:"")}).fail(function(){n("")})}}function L(e,t,n,i){var a;return a=t===!0?e.trueText||_e.default.format("dxDataGrid-trueText"):t===!1?e.falseText||_e.default.format("dxDataGrid-falseText"):o(e,t),e.customizeText&&(a=e.customizeText.call(e,{value:t,valueText:a,target:i})),n&&n.customizeText&&(a=n.customizeText.call(n,{value:t,valueText:a,field:e,target:i})),a}function H(e,t,n,i){return t.map(function(t){return L(e,t,n,i)})}function z(e){return""===e||null===e}function N(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"filterBuilder";if(z(t))return"";if(Array.isArray(t)){var o=new re.Deferred;return re.when.apply(this,H(e,t,n,i)).done(function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var i=t.some(function(e){return!z(e)})?t.map(function(e){return z(e)?"?":e}):"";o.resolve(i)}),o}return L(e,t,n,i)}function j(e,t){return e.some(function(e){return e.dataField===t})}function W(e,t,n){var i=n.dataField;$(i)&&(n.parentId=K(i),j(t,n.parentId)||j(e,n.parentId)||W(e,t,{id:n.parentId,dataType:"object",dataField:n.parentId,caption:G(n.parentId,!0),filterOperations:["isblank","isnotblank"]})),t.push(n)}function G(e,t){var n="";if(t)e=e.substring(e.lastIndexOf(".")+1);else if($(e))return e.split(".").forEach(function(e,t,i){n+=(0,pe.captionize)(e),t!==i.length-1&&(n+=".")}),n;return(0,pe.captionize)(e)}function q(e,t){for(var n=[],i=0;i<e.length;i++){var o=(0,fe.extend)(!0,{caption:G(e[i].dataField,t)},e[i]);o.id=o.name||o.dataField,t?W(e,n,o):n.push(o)}return n}function $(e){return e.lastIndexOf(".")!==-1}function K(e){return e.substring(0,e.lastIndexOf("."))}function U(e,t){if($(e.dataField))for(var n=K(e.dataField),i=0;i<t.length;i++)if(t[i].dataField===n)return U(t[i],t)+"."+e.caption;return e.caption}function Y(e,t,n){var i=v(n,t);return i?(i.hasValue===!1?(e[1]=t,e.length=2):(e[1]=t,e[2]=""),e):("isblank"===t?(e[1]=we,e[2]=null):"isnotblank"===t?(e[1]=Ce,e[2]=null):(i=v(n,e[1]),(i||2===e.length||null===e[2])&&(e[2]=""),e[1]=t),e)}function Z(e){var t;return t=null===e[2]?e[1]===we?"isblank":"isnotblank":e[1]}function X(e){return""!==e[2]}function Q(e,t,n){var i=(0,fe.extend)(!0,[],e),o=-1;return i.some(function(e,t){if("between"===e.name)return o=t,!0}),o!==-1?i[o]=(0,fe.extend)((0,ge.getConfig)(t,n),i[o]):i.unshift((0,ge.getConfig)(t,n)),i}function J(e,t){return e[0]===t}function ee(e,t){if(!e||0===e.length)return null;if(T(e)){var n=J(e,t);return n?null:e}return te(e,[t],!1)}function te(e,t,n){var i=[];return e.forEach(function(e){T(e)?J(e,t[0])?n?(i.push(t),n=!1):i.splice(i.length-1,1):i.push(e):(i.length||I(e))&&i.push(e)}),0===i.length?null:(n&&(i.push(xe),i.push(t)),1===i.length?i[0]:i)}function ne(e,t){if(null===e||0===e.length)return t;if(T(e))return J(e,t[0])?t:[e,xe,t];var n=h(e);return n!==xe?[t,"and",e]:te(e,t,!0)}function ie(e,t){if(null===e||0===e.length)return[];if(T(e))return J(e,t)?[e]:[];var n=h(e);if(n!==xe)return[];var i=e.filter(function(e){return T(e)&&J(e,t)});return i}function oe(e,t){return null!==e&&0!==e.length&&(T(e)?e[0]===t:e.some(function(e){return(T(e)||I(e))&&oe(e,t)}))}t.getGroupCriteria=r,t.setGroupValue=u,t.getGroupMenuItem=c,t.getGroupValue=h,t.getFilterOperations=g,t.getCaptionByOperation=_,t.getOperationFromAvailable=m,t.getCustomOperation=v,t.getAvailableOperations=y,t.getDefaultOperation=b,t.createCondition=x,t.removeItem=w,t.createEmptyGroup=C,t.isEmptyGroup=k,t.addItem=S,t.getField=D,t.isGroup=I,t.isCondition=T,t.convertToInnerStructure=P,t.getNormalizedFields=R,t.getFilterExpression=B,t.getNormalizedFilter=V,t.getCurrentLookupValueText=F,t.getCurrentValueText=N,t.getItems=q,t.getCaptionWithParents=U,t.updateConditionByOperation=Y,t.getOperationValue=Z,t.isValidCondition=X,t.getMergedOperations=Q,t.removeFieldConditionsFromFilter=ee,t.syncFilters=ne,t.getMatchedConditions=ie,t.filterHasField=oe,t.renderValueText=void 0;var ae=i(n(2)),re=n(6),se=i(n(34)),le=n(1),ue=n(19),ce=i(n(17)),de=i(n(196)),he=i(n(79)),fe=n(0),pe=n(40),ge=n(717),_e=i(n(10)),me=n(69),ve=i(n(718)),ye="string",be="icon-none",xe="and",we="=",Ce="<>",ke={number:["=","<>","<",">","<=",">=","isblank","isnotblank"],string:["contains","notcontains","startswith","endswith","=","<>","isblank","isnotblank"],date:["=","<>","<",">","<=",">=","isblank","isnotblank"],datetime:["=","<>","<",">","<=",">=","isblank","isnotblank"],"boolean":["=","<>","isblank","isnotblank"],object:["isblank","isnotblank"]},Se={date:"shortDate",datetime:"shortDateShortTime"},De=["=","<>","isblank","isnotblank"],Ie=["caption","customizeText","dataField","dataType","editorTemplate","falseText","editorOptions","filterOperations","format","lookup","trueText","calculateFilterExpression","name"],Te="dx-filterbuilder",Oe=Te+"-text",Ee=Oe+"-part",Ae=Oe+"-separator",Pe=Ae+"-empty",Re=function(e,t,n){if(Array.isArray(t)){var i=t.length-1;e.empty(),t.forEach(function(t,o){(0,ae.default)("<span>").addClass(Ee).text(t).appendTo(e),o!==i&&(0,ae.default)("<span>").addClass(Ae).text(n&&n.valueSeparator?n.valueSeparator:"|").addClass(Pe).appendTo(e)})}else t?e.text(t):e.text(_e.default.format("dxFilterBuilder-enterValueText"))};t.renderValueText=Re},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var i=n(129),o=n(514),a=n(515);n.d(t,"rrulestr",function(){return a.a});var r=n(148);n.d(t,"Frequency",function(){return r.a});var s=n(168);n.d(t,"Weekday",function(){return s.b}),n.d(t,"RRule",function(){return i.c}),n.d(t,"RRuleSet",function(){return o.a}),/*!
 * rrule.js - Library for working with recurrence rules for calendar dates.
 * https://github.com/jakubroztocil/rrule
 *
 * Copyright 2010, Jakub Roztocil and Lars Schoning
 * Licenced under the BSD licence.
 * https://github.com/jakubroztocil/rrule/blob/master/LICENCE
 *
 * Based on:
 * python-dateutil - Extensions to the standard Python datetime module.
 * Copyright (c) 2003-2011 - Gustavo Niemeyer <gustavo@niemeyer.net>
 * Copyright (c) 2012 - Tomi Pieviläinen <tomi.pievilainen@iki.fi>
 * https://github.com/jakubroztocil/rrule/blob/master/LICENCE
 *
 */
t.default=i.c},function(e,t,n){t.default=void 0;var i={notifyObserver:function(e,t){var n=this.option("observer");n&&n.fire(e,t)},invoke:function(){var e=this.option("observer");if(e)return e.fire.apply(e,arguments)}},o=i;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return Object.keys(e).filter(function(t){return e[t]}).join(" ")}t.combineClasses=i},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,p.useCallback)(function(){var t=(e.children,e.className,u(e,["children","className"]));return t},[e]);return g({props:s({},e),restAttributes:t()})}t.Row=h,t.RowProps=t.viewFunction=void 0;var f=a(n(51)),p=n(53),g=function(e){return f.h("tr",d({},e.restAttributes,{className:e.props.className}),e.props.children)};t.viewFunction=g;var _={className:""};t.RowProps=_,h.defaultProps=s({},_)},function(e,t,n){function i(e,t,n){if(e){var i=e.fill;return(0,D.extend)({},{state:n,fill:i===ie?t:i,opacity:e.opacity,hatching:G({},e.hatching,{step:K,width:$})})}}function o(e,t,n){var i=(0,P.processHatchingAttrs)(e,t);return i.fill&&0===i.fill.indexOf("DevExpress")&&(i.fill=(0,P.getFuncIri)(i.fill)),i.opacity=i.opacity>=0?i.opacity:1,(0,D.extend)({},i,{size:n})}function a(e){var t=e.margin;t>=0?(t=M(e.margin),t={top:t,bottom:t,left:t,right:t}):t={top:t.top>=0?M(t.top):q,bottom:t.bottom>=0?M(t.bottom):q,left:t.left>=0?M(t.left):q,right:t.right>=0?M(t.right):q},e.margin=t}function r(e,t,n){var i,o,a=7,r=4;switch(e.itemTextPosition){case Z:case Y:i=t.width+a+n.width,o=F(t.height,n.height);break;case X:case Q:i=F(t.width,n.width),o=t.height+r+n.height}return{width:i,height:o}}function s(e,t){var n={};return n.left=L(e.x,t.x),n.top=L(e.y,t.y),n.right=F(e.x+e.width,t.x+t.width),n.bottom=F(e.y+e.height,t.y+t.height),n}function l(e,t,n,i){var o=t&&n[t[e]];o&&o.renderMarker(o.states[i])}function u(e,t,n){return e?(a(e),e.horizontalAlignment=re(e.horizontalAlignment,Y),e.verticalAlignment=se(e.verticalAlignment,e.horizontalAlignment===U?Q:X),e.orientation=le(e.orientation,e.horizontalAlignment===U?J:ee),e.itemTextPosition=ue(e.itemTextPosition,e.orientation===J?Q:Y),e.position=n?ce(e.position,ne):ne,e.itemsAlignment=de(e.itemsAlignment,null),e.hoverMode=W(e.hoverMode),e.customizeText=N(e.customizeText)?e.customizeText:function(){return this[t]},e.customizeHint=N(e.customizeHint)?e.customizeHint:A.noop,e._incidentOccurred=e._incidentOccurred||A.noop,e):null}function c(e,t){return e.rect(0,0,t,t)}function d(e,t){return e.circle(t/2,t/2,t/2)}function h(e){return"circle"===W(e)}function f(e,t,n){return t>=e.left&&t<=e.right&&n>=e.top&&n<=e.bottom}function p(e,t,n,i){var o={x:0,y:0},a=0,r=0,s=0;if(s="y"===t.direction?i.top+i.bottom:i.left+i.right,e.forEach(function(e,n){var i=e[0],s=e.length;e.forEach(function(e,n){var i=e.offset||t.spacing;o[t.direction]+=e[t.measure]+(n!==s-1?i:0),a=F(a,o[t.direction])}),o[t.direction]=0,o[t.altDirection]+=i[t.altMeasure]+i.altOffset||t.altSpacing,r=F(r,o[t.altDirection])}),a+s>t.length)return t.countItem=g(t,n),!0}function g(e,t){return e.altCountItem++,H(t/e.altCountItem)}function _(e,t){return e.reduce(function(e,n){var i=n.offset||t.spacing;return e+n[t.measure]+i},0)}function m(e,t){return e.reduce(function(e,n){return F(e,_(n,t))},0)}function v(e,t,n){var i,o=_(e,t);switch(t.itemsAlignment){case Y:i=n-o;break;case U:i=(n-o)/2;break;default:i=0}return i}function y(e){switch(e.itemTextPosition){case Q:return{horizontal:U,vertical:X};case X:return{horizontal:U,vertical:Q};case Z:return{horizontal:Y,vertical:U};case Y:return{horizontal:Z,vertical:U}}}function b(e,t,n){var i={};return n%t.countItem===0&&(t.markerOffset?e.push([],[]):e.push([])),t.markerOffset?(i.firstLine=e[e.length-1],i.secondLine=e[e.length-2]):i.firstLine=i.secondLine=e[e.length-1],i}function x(e,t){var n=e.reduce(function(e,n){var i=n?n[t]:e;return F(e,i)},0);e.forEach(function(e){e&&(e[t]=n)})}function w(e){var t,n,i=e.length,o=e[0].length,a=[];for(t=0;t<o;t++)for(a[t]=[],n=0;n<i;n++)a[t][n]=e[n][t];return a}function C(e){switch(e){case X:case Q:return U;case Z:return Y;case Y:return Z}}function k(e){return e.horizontalAlignment===U?U:e.itemTextPosition===Y?Z:e.itemTextPosition===Z?Y:U}t.plugin=t.Legend=void 0;var S=n(14),D=n(0),I=n(323),T=n(1),O=n(131),E=n(50),A=n(4),P=n(171),R=n(6),M=Number,B=Math,V=B.round,F=B.max,L=B.min,H=B.ceil,z=T.isDefined,N=T.isFunction,j=S.enumParser,W=S.normalizeEnum,G=D.extend,q=10,$=2,K=5,U="center",Y="right",Z="left",X="top",Q="bottom",J="horizontal",ee="vertical",te="inside",ne="outside",ie="none",oe="height",ae="width",re=j([Z,U,Y]),se=j([X,Q]),le=j([ee,J]),ue=j([Z,Y,X,Q]),ce=j([ne,te]),de=j([Z,U,Y]),he=function(e){return h(e)?d:c},fe=function(e){var t=this;t._renderer=e.renderer,t._legendGroup=e.group,t._backgroundClass=e.backgroundClass,t._itemGroupClass=e.itemGroupClass,t._textField=e.textField,t._getCustomizeObject=e.getFormatObject,t._titleGroupClass=e.titleGroupClass,t._allowInsidePosition=e.allowInsidePosition,t._widget=e.widget,t._updated=!1};t.Legend=fe;var pe=fe,ge=pe.prototype=(0,E.clone)(I.LayoutElement.prototype);(0,D.extend)(ge,{constructor:pe,getOptions:function(){return this._options},update:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],t=arguments.length>1?arguments[1]:void 0,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this;t=i._options=u(t,i._textField,i._allowInsidePosition)||{};var a=t.markerSize;if(this._updated=!0,this._data=e.map(function(e){return e.size=M(e.size>0?e.size:a),e.marker=o(e,e.states.normal),Object.defineProperty(e.marker,"size",{get:function(){return e.size},set:function(t){e.size=t}}),Object.defineProperty(e.marker,"opacity",{get:function(){return e.states.normal.opacity},set:function(t){e.states.normal.opacity=e.states.hover.opacity=e.states.selection.opacity=t}}),e}),t.customizeItems&&(i._data=t.customizeItems(e.slice())||e),i._boundingRect={width:0,height:0,x:0,y:0},i.isVisible()&&!i._title&&(i._title=new O.Title({renderer:i._renderer,cssClass:i._titleGroupClass,root:i._legendGroup})),i._title){var r=t.title;n.horizontalAlignment=k(t),i._title.update(n,r)}return this.erase(),i},isVisible:function(){return this._options&&this._options.visible},draw:function(e,t){var n=this,i=n._getItemData();if(n.erase(),!(n.isVisible()&&i&&i.length))return n;if(n._insideLegendGroup=n._renderer.g().enableLinks().append(n._legendGroup),n._title.changeLink(n._insideLegendGroup),n._createBackground(),n._title.hasText()){var o=n._background?2*n._options.paddingLeftRight:0;n._title.draw(e-o,t)}return n._markersGroup=n._renderer.g().attr({"class":n._itemGroupClass}).append(n._insideLegendGroup),n._createItems(i),n._updateElementsPosition(e,t),n},_measureElements:function(){var e=this._options,t=0;this._items.forEach(function(n){var i=n.label.getBBox(),o=n.marker.getBBox();n.markerBBox=o,n.markerSize=Math.max(o.width,o.height);var a=r(e,o,i);n.labelBBox=i,n.bBox=a,t=F(t,a.height)}),e.equalRowHeight&&this._items.forEach(function(e){return e.bBox.height=t})},_updateElementsPosition:function(e,t){var n=this,i=n._options;this._size={width:e,height:t},n._measureElements(),n._locateElements(i),n._finalUpdate(i);var o=n.getLayoutOptions();(o.width>e||o.height>t)&&n.freeSpace()},_createItems:function(e){var t=this,n=t._options,a=t._renderer,r=he(n.markerShape);t._markersId={};var s=n.markerTemplate?n.markerTemplate:function(e,t){var n=e.marker;r(a,n.size).attr({fill:n.fill,opacity:n.opacity}).append({element:t})},l=t._widget._getTemplate(s),u=t._markersGroup;u.css((0,S.patchFontOptions)(n.font)),t._deferredItems=[],t._templatesGroups=[],t._items=(e||[]).map(function(e,r){var s=e.states,c=s.normal,d=c.fill;e.size=e.marker.size;var h={normal:(0,D.extend)(c,{fill:d||n.markerColor||n.defaultColor,state:"normal"}),hover:i(s.hover,d,"hovered"),selection:i(s.selection,d,"selected")};e.states=h;var f=a.g().append(u),p=a.g().attr({"class":"dxl-marker"}).append(f);t._deferredItems[r]=new R.Deferred,t._templatesGroups.push(p);var g={label:t._createLabel(e,f),marker:p,renderer:a,group:f,tracker:{id:e.id,argument:e.argument,argumentIndex:e.argumentIndex},states:h,itemTextPosition:n.itemTextPosition,markerOffset:0,bBoxes:[],renderMarker:function(n){e.marker=o(g,n,e.size),p.clear(),l.render({model:e,container:p.element,onRendered:t._deferredItems[r].resolve})}};return g.renderMarker(h.normal),t._createHint(e,f),void 0!==e.id&&(t._markersId[e.id]=r),g})},getTemplatesGroups:function(){return this._templatesGroups||[]},getTemplatesDef:function(){return this._deferredItems||[]},_getItemData:function(){var e=this._data||[],t=this._options||{};return t.inverted&&(e=e.slice().reverse()),e.filter(function(e){return e.visible})},_finalUpdate:function(e){this._adjustBackgroundSettings(e),this._setBoundingRect(e.margin)},erase:function(){var e=this,t=e._insideLegendGroup;return t&&t.dispose(),e._insideLegendGroup=e._markersGroup=e._x1=e._x2=e._y2=e._y2=null,e},_locateElements:function(e){this._moveInInitialValues(),this._locateRowsColumns(e)},_moveInInitialValues:function(){var e=this;e._title.hasText()&&e._title.move([0,0]),e._legendGroup&&e._legendGroup.move(0,0),e._background&&e._background.attr({x:0,y:0,width:0,height:0})},applySelected:function(e){return l(e,this._markersId,this._items,"selection"),this},applyHover:function(e){return l(e,this._markersId,this._items,"hover"),this},resetItem:function(e){return l(e,this._markersId,this._items,"normal"),this},_createLabel:function(e,t){var n=this._getCustomizeObject(e),i=this._options,o=C(i.itemTextPosition),a=i.customizeText.call(n,n),r=z(e.textOpacity)?{color:i.font.color,opacity:e.textOpacity}:{};return this._renderer.text(a,0,0).css((0,S.patchFontOptions)(r)).attr({align:o,"class":i.cssClass}).append(t)},_createHint:function(e,t){var n=this._getCustomizeObject(e),i=this._options.customizeHint.call(n,n);z(i)&&""!==i&&t.setTitle(i)},_createBackground:function(){var e=this,t=e._options.position===te,n=e._options.backgroundColor,i=n||(t?e._options.containerBackgroundColor:ie);(e._options.border.visible||(t||n)&&n!==ie)&&(e._background=e._renderer.rect(0,0,0,0).attr({fill:i,"class":e._backgroundClass}).append(e._insideLegendGroup))},_locateRowsColumns:function(e){var t,n=this,i=0,o=n._getItemsLayoutOptions(),a=n._items.length;do t=[],n._createLines(t,o),n._alignLines(t,o),i++;while(p(t,o,a,e.margin)&&i<a);n._applyItemPosition(t,o)},_createLines:function(e,t){this._items.forEach(function(n,i){var o,a,r=b(e,t,i),s={width:n.labelBBox.width,height:n.labelBBox.height,element:n.label,bBox:n.labelBBox,pos:y(t),itemIndex:i},l={width:n.markerBBox.width,height:n.markerBBox.height,element:n.marker,pos:{horizontal:U,vertical:U},bBox:{width:n.markerBBox.width,height:n.markerBBox.height,x:n.markerBBox.x,y:n.markerBBox.y},itemIndex:i},u=t.markerOffset?"altOffset":"offset";t.inverseLabelPosition?(o=s,a=l):(o=l,a=s),o[u]=t.labelOffset,r.secondLine.push(o),r.firstLine.push(a)})},_alignLines:function(e,t){function n(e){x(e,o)}var i,o=t.altMeasure;if(e.forEach(function(e){return x(e,o)}),o=t.measure,t.itemsAlignment){if(t.markerOffset)for(i=0;i<e.length;)w([e[i++],e[i++]]).forEach(n)}else w(e).forEach(n)},_applyItemPosition:function(e,t){var n=this,i={x:0,y:0},o=m(e,t);e.forEach(function(e){var a=e[0],r=a.altOffset||t.altSpacing;i[t.direction]=v(e,t,o),e.forEach(function(e){var o=e.offset||t.spacing,a=new I.WrapperLayoutElement(e.element,e.bBox),r={x:i.x,y:i.y,width:e.width,height:e.height},s=new I.WrapperLayoutElement(null,r),l=n._items[e.itemIndex];a.position({of:s,my:e.pos,at:e.pos}),l.bBoxes.push(s),i[t.direction]+=e[t.measure]+o}),i[t.altDirection]+=a[t.altMeasure]+r}),this._items.forEach(function(e){var t=s(e.bBoxes[0].getLayoutOptions(),e.bBoxes[1].getLayoutOptions()),i=n._options.columnItemSpacing/2,o=n._options.rowItemSpacing/2;e.tracker.left=t.left-i,e.tracker.right=t.right+i,e.tracker.top=t.top-o,e.tracker.bottom=t.bottom+o})},_getItemsLayoutOptions:function(){var e=this,t=e._options,n=t.orientation,i={itemsAlignment:t.itemsAlignment,orientation:t.orientation},o=e._size.width-(e._background?2*t.paddingLeftRight:0),a=e._size.height-(e._background?2*t.paddingTopBottom:0);return n===J?(i.length=o,i.spacing=t.columnItemSpacing,i.direction="x",i.measure=ae,i.altMeasure=oe,i.altDirection="y",i.altSpacing=t.rowItemSpacing,i.countItem=t.columnCount,i.altCountItem=t.rowCount,i.marginTextLabel=4,i.labelOffset=7,t.itemTextPosition!==Q&&t.itemTextPosition!==X||(i.labelOffset=4,i.markerOffset=!0)):(i.length=a,i.spacing=t.rowItemSpacing,i.direction="y",i.measure=oe,i.altMeasure=ae,i.altDirection="x",i.altSpacing=t.columnItemSpacing,i.countItem=t.rowCount,i.altCountItem=t.columnCount,i.marginTextLabel=7,i.labelOffset=4,t.itemTextPosition!==Y&&t.itemTextPosition!==Z||(i.labelOffset=7,i.markerOffset=!0)),i.countItem||(i.altCountItem?i.countItem=H(e._items.length/i.altCountItem):i.countItem=e._items.length),t.itemTextPosition!==X&&t.itemTextPosition!==Z||(i.inverseLabelPosition=!0),i.itemTextPosition=t.itemTextPosition,i.altCountItem=i.altCountItem||H(e._items.length/i.countItem),i},_adjustBackgroundSettings:function(e){if(this._background){var t=e.border,n=this._calculateTotalBox(),i={x:V(n.x-e.paddingLeftRight),y:V(n.y-e.paddingTopBottom),width:V(n.width)+2*e.paddingLeftRight,height:V(n.height),opacity:e.backgroundOpacity};t.visible&&t.width&&t.color&&t.color!==ie&&(i["stroke-width"]=t.width,i.stroke=t.color,i["stroke-opacity"]=t.opacity,i.dashStyle=t.dashStyle,i.rx=t.cornerRadius||0,i.ry=t.cornerRadius||0),this._background.attr(i)}},_setBoundingRect:function(e){if(this._insideLegendGroup){var t=this._calculateTotalBox();t.height+=e.top+e.bottom,t.widthWithoutMargins=t.width,t.width+=e.left+e.right,t.x-=e.left,t.y-=e.top,this._boundingRect=t}},_calculateTotalBox:function(){var e=this._markersGroup.getBBox(),t=this._title.getCorrectedLayoutOptions(),n=this._insideLegendGroup.getBBox(),i=this._background?2*this._options.paddingTopBottom:0;return n.height=e.height+t.height+i,t.width>n.width&&(n.width=t.width),n},getActionCallback:function(e){var t=this;return t._options.visible?function(n){t[n](e.index)}:A.noop},getLayoutOptions:function(){var e=this._options,t=this._insideLegendGroup?this._boundingRect:{width:0,height:0,x:0,y:0};return e?(t.verticalAlignment=e.verticalAlignment,t.horizontalAlignment=e.horizontalAlignment,e.orientation===J?(t.cutLayoutSide=e.verticalAlignment,t.cutSide="vertical"):e.horizontalAlignment===U?(t.cutLayoutSide=e.verticalAlignment,t.cutSide="vertical"):(t.cutLayoutSide=e.horizontalAlignment,t.cutSide="horizontal"),t.position={horizontal:e.horizontalAlignment,vertical:e.verticalAlignment},t):null},shift:function(e,t){var n=this,i={};return n._insideLegendGroup&&n._insideLegendGroup.attr({translateX:e-n._boundingRect.x,translateY:t-n._boundingRect.y}),n._title&&n._shiftTitle(n._boundingRect.widthWithoutMargins),n._markersGroup&&n._shiftMarkers(),n._insideLegendGroup&&(i=n._legendGroup.getBBox()),n._x1=i.x,n._y1=i.y,n._x2=i.x+i.width,n._y2=i.y+i.height,n},_shiftTitle:function(e){var t=this,n=t._title,i=n.getCorrectedLayoutOptions();if(i&&n.hasText()){var o=e-(t._background?2*t._options.paddingLeftRight:0),a=n.getOptions(),r=i.y+a.margin.top,s=0;a.verticalAlignment===Q&&t._markersGroup&&(r+=t._markersGroup.getBBox().height),a.horizontalAlignment===Y?s=o-i.width:a.horizontalAlignment===U&&(s=(o-i.width)/2),n.shift(s,r)}},_shiftMarkers:function(){var e=this._title.getLayoutOptions(),t=this._markersGroup.getBBox(),n=this._title.getOptions()||{},i=0,o=0;e.width>t.width&&this._options.horizontalAlignment===U&&(i=e.width/2-t.width/2),n.verticalAlignment===X&&(o=e.height),0===i&&0===o||(this._markersGroup.attr({translateX:i,translateY:o}),this._items.forEach(function(e){e.tracker.left+=i,e.tracker.right+=i,e.tracker.top+=o,e.tracker.bottom+=o}))},getPosition:function(){return this._options.position},coordsIn:function(e,t){return e>=this._x1&&e<=this._x2&&t>=this._y1&&t<=this._y2},getItemByCoord:function(e,t){var n=this._items,i=this._insideLegendGroup;e-=i.attr("translateX"),t-=i.attr("translateY");for(var o=0;o<n.length;o++)if(f(n[o].tracker,e,t))return n[o].tracker;return null},dispose:function(){var e=this;return e._title&&e._title.dispose(),e._legendGroup=e._insideLegendGroup=e._title=e._renderer=e._options=e._data=e._items=null,e},layoutOptions:function(){if(!this.isVisible())return null;var e=this.getLayoutOptions();return{horizontalAlignment:this._options.horizontalAlignment,verticalAlignment:this._options.verticalAlignment,side:e.cutSide,priority:1,position:this.getPosition()}},measure:function(e){this._updated||!this._insideLegendGroup?(this.draw(e[0],e[1]),this._updated=!1):(this._items.forEach(function(e){e.bBoxes=[]}),this._updateElementsPosition(e[0],e[1]));var t=this.getLayoutOptions();return[t.width,t.height]},move:function(e){this.shift(e[0],e[1])},freeSpace:function(){this._options._incidentOccurred("W2104"),this.erase()}});var _e={name:"legend",init:function(){var e=this,t=this._renderer.g().attr({"class":this._rootClassPrefix+"-legend"}).enableLinks().append(e._renderer.root);e._legend=new fe({renderer:e._renderer,group:t,widget:this,itemGroupClass:this._rootClassPrefix+"-item",titleGroupClass:this._rootClassPrefix+"-title",textField:"text",getFormatObject:function(e){return{item:e.item,text:e.text}}}),e._layout.add(e._legend)},extenders:{_applyTilesAppearance:function(){var e=this;this._items.forEach(function(t){e._applyLegendItemStyle(t.id,t.getState())})},_buildNodes:function(){this._createLegendItems()}},members:{_applyLegendItemStyle:function(e,t){var n=this._legend;switch(t){case"hover":n.applyHover(e);break;case"selection":n.applySelected(e);break;default:n.resetItem(e)}},_createLegendItems:function(){this._legend.update(this._getLegendData(),this._getOption("legend"),this._themeManager.theme("legend").title)&&this._requestChange(["LAYOUT"])}},dispose:function(){this._legend.dispose()},customize:function(e){e.prototype._proxyData.push(function(e,t){if(this._legend.coordsIn(e,t)){var n=this._legend.getItemByCoord(e,t);if(n)return{id:n.id,type:"legend"}}}),e.addChange({code:"LEGEND",handler:function(){this._createLegendItems()},isThemeDependent:!0,option:"legend",isOptionChange:!0})}};t.plugin=_e},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=e.initialValue,n=e._getValTranslator().getBusinessRange().invert,i="discrete"===e.series.valueAxisType,o=e.series.isFullStackedSeries(),a=!i&&(t>=0&&!n||t<0&&n)||i&&!n||o;return a?_:m}t.default=void 0;var a=n(0),r=i(n(152)),s=a.extend,l=Math,u=l.floor,c=l.abs,d="canvas_position_default",h=9,f=4,p="right",g="left",_="top",m="bottom",v=s({},r.default,{correctCoordinates:function(e){var t=this,n=u(e.offset-e.width/2);t._options.rotated?(t.height=e.width,t.yCorrection=n,t.xCorrection=null):(t.width=e.width,t.xCorrection=n,t.yCorrection=null)},_getGraphicBBox:function(e){var t={x:this.x,y:this.y,width:this.width,height:this.height};if(e){var n="top"===e;this._options.rotated?(t.x=n?t.x+t.width:t.x,t.width=0):(t.y=n?t.y:t.y+t.height,t.height=0)}return t},_getLabelConnector:function(e){return this._getGraphicBBox(e)},_getLabelPosition:function(){var e=o(this);return this._options.rotated&&(e=e===_?p:g),e},_getLabelCoords:function(e){var t,n=this;return t=0===n.initialValue&&n.series.isFullStackedSeries()?this._options.rotated?n._getLabelCoordOfPosition(e,p):n._getLabelCoordOfPosition(e,_):"inside"===e.getLayoutOptions().position?n._getLabelCoordOfPosition(e,"inside"):r.default._getLabelCoords.call(this,e)},_drawLabel:function(){this._label.pointPosition="inside"!==this._label.getLayoutOptions().position&&o(this),r.default._drawLabel.call(this)},hideInsideLabel:function(e,t){var n=this._getGraphicBBox(),i=e.getBoundingRect();return!(!this._options.resolveLabelsOverlapping||!(t.y<=n.y&&t.y+i.height>=n.y+n.height||t.x<=n.x&&t.x+i.width>=n.x+n.width)||t.y>n.y+n.height||t.y+i.height<n.y||t.x>n.x+n.width||t.x+i.width<n.x)&&(e.draw(!1),!0)},_showForZeroValues:function(){return this._options.label.showForZeroValues||this.initialValue},_drawMarker:function(e,t,n){var i=this,o=i._getStyle(),a=i._options.cornerRadius,r=i._options.rotated,s=i.getMarkerCoords(),l=s.x,u=s.y,c=s.width,d=s.height;n&&(r?(c=0,l=i.defaultX):(d=0,u=i.defaultY)),i.graphic=e.rect(l,u,c,d).attr({rx:a,ry:a}).smartAttr(o).data({"chart-data-point":i}).append(t)},_getSettingsForTracker:function(){var e=this,t=e.y,n=e.height,i=e.x,o=e.width;return e._options.rotated?1===o&&(o=h,i-=f):1===n&&(n=h,t-=f),{x:i,y:t,width:o,height:n}},getGraphicSettings:function(){var e=this.graphic;return{x:e.attr("x"),y:e.attr("y"),height:e.attr("height"),width:e.attr("width")}},_getEdgeTooltipParams:function(){var e,t,n=this.value>=0,i=this._getValTranslator().getBusinessRange().invert,o=this.x,a=this.y,r=this.width,s=this.height;return this._options.rotated?(t=a+s/2,e=i?n?o:o+r:n?o+r:o):(e=o+r/2,t=i?n?a+s:a:n?a:a+s),{x:e,y:t,offset:0}},getTooltipParams:function(e){if("edge"===e)return this._getEdgeTooltipParams();var t=this.getCenterCoord();return t.offset=0,t},getCenterCoord:function(){var e=this.width,t=this.height,n=this.x,i=this.y;return{x:n+e/2,y:i+t/2}},_truncateCoord:function(e,t){return null===e?e:e<t[0]?t[0]:e>t[1]?t[1]:e},_getErrorBarBaseEdgeLength:function(){return this._options.rotated?this.height:this.width},_translateErrorBars:function(e){r.default._translateErrorBars.call(this),(this._errorBarPos<e[0]||this._errorBarPos>e[1])&&(this._errorBarPos=void 0)},_translate:function(){var e,t,n,i=this,o=i._options.rotated,a=o?"x":"y",r=o?"y":"x",s=o?"width":"height",l=o?"height":"width",u=i._getArgTranslator(),h=i._getValTranslator(),f=i.series.getArgumentAxis().getVisibleArea(),p=i.series.getValueAxis().getVisibleArea();e=u.translate(i.argument),i[r]=e=null===e?e:e+(i[r+"Correction"]||0),t=h.translate(i.value,1),n=h.translate(i.minValue),i["v"+a]=t,i["v"+r]=e+i[l]/2,t=i._truncateCoord(t,p),n=i._truncateCoord(n,p),i[s]=c(t-n),t=t<n?t:n,i._calculateVisibility(o?t:e,o?e:t,i.width,i.height),i[a]=null===t?t:t+(i[a+"Correction"]||0),i["min"+a.toUpperCase()]=null===n?n:n+(i[a+"Correction"]||0),i["default"+a.toUpperCase()]=h.translate(d),i._translateErrorBars(f),i.inVisibleArea&&null!==i[r]&&(i[r]<f[0]&&(i[l]=i[l]-(f[0]-i[r]),i[r]=f[0]),i[r]+i[l]>f[1]&&(i[l]=f[1]-i[r]))},_updateMarker:function(e,t){this.graphic.smartAttr(s({},t,e?{}:this.getMarkerCoords()))},getMarkerCoords:function(){var e=this,t=e.x,n=e.y,i=e.width,o=e.height,a=e.series.getArgumentAxis(),r=e._options.rotated;if(a.getAxisPosition){var s=a.getOptions(),l=Math.round(s.width/2),u=a.getAxisPosition();if(s.visible)if(r){var c=e.minX===e.defaultX&&e.minX===u-a.getAxisShift();t+=c?l:0,i-=c?l:0,i<0&&(i=0)}else o-=e.minY===e.defaultY&&e.minY===u-a.getAxisShift()?l:0,o<0&&(o=0)}return{x:t,y:n,width:i,height:o}},coordsIn:function(e,t){var n=this;return e>=n.x&&e<=n.x+n.width&&t>=n.y&&t<=n.y+n.height}});t.default=v,e.exports=t.default},function(e,t,n){function i(e,t,n){e.min=e.min<t?e.min:t,e.max=e.max>n?e.max:n}function o(e,t){if("logarithmic"!==e)return null;var n=t.getOptions().logarithmBase;return function(e){var t=(0,f.getLog)(m(e),n),i=t<0?v:y;return i(t)}}function a(e,t,n){var o=i;return e===_?o=function(e,t,n){t!==n&&e.categories.push(n),e.categories.push(t)}:t&&(o=function(e,n){var o=t.calculateInterval(n,e.prevValue),a=e.interval;e.interval=(a<o?a:o)||a,e.prevValue=n,i(e,n,n)}),n?function(e,t,i){var a=[];o(e,t,i),0!==t&&a.push(n(t)),0!==i&&a.push(n(i));var r=b.apply(null,a);e.linearThreshold=e.linearThreshold<r?e.linearThreshold:r}:o}function r(e,t,n){var i={axisType:e,dataType:t};return e===_?i.categories=[]:(i.min=n,i.max=n),i}function s(e){return e.categories&&(e.categories=(0,f.unique)(e.categories)),e}function l(e,t,n,i){if(t&&(0,p.isDefined)(t.value)){var o=e.value,a=t.value,r=e.argument,s=t.argument,l=(n-r)*(a-o)/(s-r)+o.valueOf();return"datetime"===i.dataType?new Date(l):l}return e.value}function u(e,t,n,i,o){var a=l(n,i,o,t);e(t,a,a)}function c(e){return e.type.toLowerCase().indexOf("line")>=0||e.type.toLowerCase().indexOf("area")>=0}function d(e){var t=a(e.valueAxisType),n=e.getArgumentAxis(),i=n&&e.getArgumentAxis().visualRange()||{},o=c(e)?u:g.noop;if(n&&n.getMarginOptions().checkInterval){var r=e.getArgumentAxis().getTranslator().getBusinessRange(),s=(0,f.getAddFunction)(r,!1),l=r.interval;isFinite(l)&&(0,p.isDefined)(i.startValue)&&(0,p.isDefined)(i.endValue)&&(i.startValue=s(i.startValue,l,-1),i.endValue=s(i.endValue,l))}var d=h(i);return function(e,n,a,r){var s=n.argument;return n.hasValue()?(d(s)?(e.startCalc||(e.startCalc=!0,o(t,e,n,r[a-1],i.startValue)),t(e,n.getMinValue(),n.getMaxValue())):!i.categories&&(0,p.isDefined)(i.startValue)&&s>i.startValue&&(e.startCalc||o(t,e,n,r[a-1],i.startValue),e.endCalc=!0,o(t,e,n,r[a-1],i.endValue)),e):e}}function h(e){if(e.categories){var t=e.categories.reduce(function(e,t){return e[t.valueOf()]=!0,e},{});return function(e){return(0,p.isDefined)(e)&&t[e.valueOf()]}}return(0,p.isDefined)(e.startValue)||(0,p.isDefined)(e.endValue)?(0,p.isDefined)(e.endValue)?(0,p.isDefined)(e.startValue)?function(t){return t>=e.startValue&&t<=e.endValue}:function(t){return t<=e.endValue}:function(t){return t>=e.startValue}:function(){return!0}}t.default=void 0;var f=n(14),p=n(1),g=n(4),_="discrete",m=Math.abs,v=Math.floor,y=Math.ceil,b=Math.min,x={getViewPortFilter:h,getArgumentRange:function(e){var t=e._data||[],n={};if(t.length)if(e.argumentAxisType===_)n={categories:t.map(function(e){return e.argument})};else{var i;if(t.length>1){var o=e.getArgumentAxis().calculateInterval(t[0].argument,t[1].argument),a=e.getArgumentAxis().calculateInterval(t[t.length-1].argument,t[t.length-2].argument);i=b(o,a)}n={min:t[0].argument,max:t[t.length-1].argument,interval:i}}return s(n)},getRangeData:function(e){var t=e.getPoints(),n=e.useAggregation(),i=a(e.argumentAxisType,t.length>1&&e.getArgumentAxis(),o(e.argumentAxisType,e.getArgumentAxis())),l=a(e.valueAxisType,null,o(e.valueAxisType,e.getValueAxis())),u=d(e),c=t.reduce(function(e,t,n,o){var a=t.argument;return t.isArgumentCorrect()?(i(e.arg,a,a),t.hasValue()&&(l(e.val,t.getMinValue(),t.getMaxValue()),u(e.viewport,t,n,o)),e):e},{arg:r(e.argumentAxisType,e.argumentType),val:r(e.valueAxisType,e.valueType,t.length?e.getValueRangeInitialValue():void 0),viewport:r(e.valueAxisType,e.valueType,t.length?e.getValueRangeInitialValue():void 0)});if(n){var h=this.getArgumentRange(e);if(e.argumentAxisType===_)c.arg=h;else{var f=e.getArgumentAxis().getViewport();((0,p.isDefined)(f.startValue)||(0,p.isDefined)(f.length))&&i(c.arg,h.min,h.min),((0,p.isDefined)(f.endValue)||(0,p.isDefined)(f.length)&&(0,p.isDefined)(f.startValue))&&i(c.arg,h.max,h.max)}}return s(c.arg),s(c.val),c},getViewport:function(e){var t=e.getPoints(),n={},i=d(e);return n=r(e.valueAxisType,e.valueType,t.length?e.getValueRangeInitialValue():void 0),t.some(function(e,o){return i(n,e,o,t),n.endCalc}),n},getPointsInViewPort:function(e){var t=h(e.getArgumentAxis().visualRange()||{}),n=e.getValueAxis().visualRange()||{},i=h(n),o=e.getPoints(),a=function(e,t,o){var a=t.getMinValue(),r=t.getMaxValue(),s=i(a),l=i(r);s&&e.push(a),r!==a&&l&&e.push(r),!o||s||l||(e.length?e.push(n.endValue):e.push(n.startValue))},r=c(e)?function(e,n,i){var o=n[i],r=n[i-1],s=n[i+1];s&&t(s.argument)&&a(e[1],o,!0),r&&t(r.argument)&&a(e[1],o,!0)}:g.noop,s=function(e,n,i){return t(n.argument)?a(e[0],n):r(e,o,i),e};return o.reduce(s,[[],[]])}};t.default=x,e.exports=t.default},function(e,t,n){function i(e,t,n,i){var o=(0,s.clone)(e);return o.x=t,o.y=n,o.angle=i,o}function o(e,t,n,o,a){var r=e.angle+a,s=(0,c.getCosAndSin)(r),l=n.x+(e.radius+o*a)*s.cos,u=n.y-(e.radius+o*a)*s.sin;return i(t,l,u,r)}function a(e,t,n,i,o){var a=t-e,r=3*n-3*t,s=3*i-6*n+3*t,l=o-3*i+3*n-t;return(0,d.solveCubicEquation)(l,s,r,a)}t.polar=t.chart=void 0;var r=n(132),s=n(50),l=n(0),u=n(3),c=n(14),d=n(29),h="discrete",f=Math.round,p=Math.sqrt,g=Math.pow,_=Math.min,m=Math.max,v=Math.abs,y={};t.chart=y;var b={};t.polar=b;var x={autoHidePointMarkersEnabled:function(){return!0},_applyGroupSettings:function(e,t,n){var i=this;t=(0,l.extend)(t,e),i._applyElementsClipRect(t),n.attr(t)},_setGroupsSettings:function(e){var t=this,n=t._styles.normal;t._applyGroupSettings(n.elements,{"class":"dxc-elements"},t._elementsGroup),t._bordersGroup&&t._applyGroupSettings(n.border,{"class":"dxc-borders"},t._bordersGroup),r.chart._setGroupsSettings.call(t,e),e&&t._markersGroup&&t._markersGroup.attr({opacity:.001})},_createGroups:function(){var e=this;e._createGroup("_elementsGroup",e,e._group),e._areBordersVisible()&&e._createGroup("_bordersGroup",e,e._group),r.chart._createGroups.call(e)},_areBordersVisible:function(){return!1},_getDefaultSegment:function(e){return{line:(0,c.map)(e.line||[],function(e){return e.getDefaultCoords()})}},_prepareSegment:function(e){return{line:e}},_parseLineOptions:function(e,t){return{stroke:e.color||t,"stroke-width":e.width,dashStyle:e.dashStyle||"solid"}},_parseStyle:function(e,t){return{elements:this._parseLineOptions(e,t)}},_applyStyle:function(e){var t=this;t._elementsGroup&&t._elementsGroup.attr(e.elements),(0,u.each)(t._graphics||[],function(t,n){n.line&&n.line.attr({"stroke-width":e.elements["stroke-width"]}).sharp()})},_drawElement:function(e,t){return{line:this._createMainElement(e.line,{"stroke-width":this._styles.normal.elements["stroke-width"]}).append(t)}},_removeElement:function(e){e.line.remove()},_updateElement:function(e,t,n,i){var o={points:t.line},a=e.line;n?a.animate(o,{},i):a.attr(o)},_animateComplete:function(){var e=this;r.chart._animateComplete.call(e),e._markersGroup&&e._markersGroup.animate({opacity:1},{duration:e._defaultDuration})},_animate:function(){var e=this,t=e._graphics.length-1;(0,u.each)(e._graphics||[],function(n,i){var o;n===t&&(o=function(){e._animateComplete()}),e._updateElement(i,e._segments[n],!0,o)})},_drawPoint:function(e){r.chart._drawPoint.call(this,{point:e.point,groups:e.groups})},_createMainElement:function(e,t){return this._renderer.path(e,"line").attr(t)},_sortPoints:function(e,t){
return t?e.sort(function(e,t){return t.y-e.y}):e.sort(function(e,t){return e.x-t.x})},_drawSegment:function(e,t,n,i){var o=this,a=o._options.rotated,r=o._prepareSegment(e,a,i);o._segments.push(r),o._graphics[n]?t||o._updateElement(o._graphics[n],r):o._graphics[n]=o._drawElement(t?o._getDefaultSegment(r):r,o._elementsGroup)},_getTrackerSettings:function(){var e=this,t=e._defaultTrackerWidth,n=e._styles.normal.elements["stroke-width"];return{"stroke-width":n>t?n:t,fill:"none"}},_getMainPointsFromSegment:function(e){return e.line},_drawTrackerElement:function(e){return this._createMainElement(this._getMainPointsFromSegment(e),this._getTrackerSettings(e))},_updateTrackerElement:function(e,t){var n=this._getTrackerSettings(e);n.points=this._getMainPointsFromSegment(e),t.attr(n)},checkSeriesViewportCoord:function(e,t){if(0===this._points.length)return!1;var n=e.isArgumentAxis?this.getArgumentRange():this.getViewport(),i=e.getTranslator().translate(n.categories?n.categories[0]:n.min),o=e.getTranslator().translate(n.categories?n.categories[n.categories.length-1]:n.max),a=this.getOptions().rotated,r=e.getOptions().inverted;return e.isArgumentAxis&&(!a&&!r||a&&r)||!e.isArgumentAxis&&(a&&!r||!a&&r)?t>=i&&t<=o:t>=o&&t<=i}},w=y.line=(0,l.extend)({},r.chart,x,{getPointCenterByArg:function(e){var t=this.getArgumentAxis().getTranslator().translate(e);return{x:t,y:t}},getSeriesPairCoord:function(e,t){for(var n=this,i=null,o=this._getNearestPointsByCoord(e,t),a=t&&!n._options.rotated||!t&&n._options.rotated,r=0;r<o.length;r++){var s=o[r],l=(s[1].vy-s[0].vy)/(s[1].vx-s[0].vx),u=s[0].vy-s[0].vx*l,c=void 0;if(c=s[1].vx-s[0].vx===0?a?s[0].vy:s[0].vx:a?l*e+u:(e-u)/l,this._checkAxisVisibleAreaCoord(!t,c)){i=c;break}}return i}});y.stepline=(0,l.extend)({},w,{_calculateStepLinePoints:function(e){var t=[],n=this._options.rotated?"x":"y";return(0,u.each)(e,function(e,i){var o;if(!e)return void t.push(i);var a=t[t.length-1][n];a!==i[n]&&(o=(0,s.clone)(i),o[n]=a,t.push(o)),t.push(i)}),t},_prepareSegment:function(e){return w._prepareSegment(this._calculateStepLinePoints(e))},getSeriesPairCoord:function(e,t){for(var n,i=this._options.rotated,o=!t&&!i||t&&i,a=o?"vy":"vx",r=o?"vx":"vy",s=this._getNearestPointsByCoord(e,t),l=0;l<s.length;l++){var u=s[l],c=void 0;if(c=t?e!==u[1][a]?u[0][r]:u[1][r]:e===u[0][a]?u[0][r]:u[1][r],this._checkAxisVisibleAreaCoord(!t,c)){n=c;break}}return n}}),y.spline=(0,l.extend)({},w,{_calculateBezierPoints:function(e,t){var n=[],o=e,a=function(e,t,n){return e>t&&n>e||e<t&&n<e?e:n};return 1!==o.length?o.forEach(function(e,r){var s,l,u,c,d,h,f,p,g,_,m,v,y,b,x=o[r-1],w=o[r+1],C=.5;if(!r||r===o.length-1)return void n.push(e,e);var k=e.x,S=e.y;d=x.x,h=w.x,f=x.y,p=w.y;var D=!!(!t&&(S<=x.y&&S<=w.y||S>=x.y&&S>=w.y)||t&&(k<=x.x&&k<=w.x||k>=x.x&&k>=w.x));if(D)t?(u=s=k,c=(S+w.y)/2,l=(S+x.y)/2):(c=l=S,u=(k+w.x)/2,s=(k+x.x)/2);else{if(g=p-f,_=d-h,m=f*h-d*p,t){if(!g)return void n.push(e,e,e);y=S,v=-1*(_*y+m)/g,b=v-k,d-=b,h-=b}else{if(!_)return void n.push(e,e,e);v=k,y=-1*(g*v+m)/_,b=y-S,f-=b,p-=b}u=(k+C*h)/(1+C),c=(S+C*p)/(1+C),s=(k+C*d)/(1+C),l=(S+C*f)/(1+C)}t?(s=a(x.x,k,s),u=a(w.x,k,u)):(l=a(x.y,S,l),c=a(w.y,S,c));var I=i(e,s,l),T=i(e,u,c);n.push(I,e,T)}):n.push(o[0]),n},_prepareSegment:function(e,t){return w._prepareSegment(this._calculateBezierPoints(e,t))},_createMainElement:function(e,t){return this._renderer.path(e,"bezier").attr(t)},getSeriesPairCoord:function(e,t){for(var n=this,i=null,o=!t&&!this._options.rotated||t&&this._options.rotated,r=o?"vy":"vx",s=o?"y":"x",l=o?"vx":"vy",u=o?"x":"y",c=t?n.getValueAxis():n.getArgumentAxis(),d=c.getVisibleArea(),h=this._getNearestPointsByCoord(e,t),f=function(t){var n=h[t];if(1===n.length)d[0]<=n[0][l]&&d[1]>=n[0][l]&&(i=n[0][l]);else{var o=a(e,n[0][r],n[1][s],n[2][s],n[3][r]);o.forEach(function(e){if(e>=0&&e<=1){var t=Math.pow(1-e,3)*n[0][l]+3*Math.pow(1-e,2)*e*n[1][u]+3*(1-e)*e*e*n[2][u]+e*e*e*n[3][l];d[0]<=t&&d[1]>=t&&(i=t)}})}if(null!==i)return"break"},p=0;p<h.length;p++){var g=f(p);if("break"===g)break}return i},_getNearestPoints:function(e,t,n){var i=n.indexOf(e);return[e,n[i+1],n[i+2],t]},_getBezierPoints:function(){return this._segments.length>0?this._segments.reduce(function(e,t){return e.concat(t.line)},[]):[]}}),b.line=(0,l.extend)({},r.polar,x,{_sortPoints:function(e){return e},_prepareSegment:function(e,t,n){var i,o=[],a=this.getValueAxis().getCenter();if(n&&this._closeSegment(e),this.argumentAxisType===h||this.valueAxisType===h)return w._prepareSegment.call(this,e);for(i=1;i<e.length;i++)o=o.concat(this._getTangentPoints(e[i],e[i-1],a));return o.length||(o=e),{line:o}},_getRemainingAngle:function(e){var t=(0,c.normalizeAngle)(e);return e>=0?360-t:-t},_closeSegment:function(e){var t;t=this._segments.length?this._segments[0].line[0]:i(e[0],e[0].x,e[0].y,e[0].angle),t=this._modifyReflectedPoint(t,e[e.length-1]),t&&e.push(t)},_modifyReflectedPoint:function(e,t){if(t.angle!==e.angle){if((0,c.normalizeAngle)(f(t.angle))===(0,c.normalizeAngle)(f(e.angle)))e.angle=t.angle;else{var n=t.angle-e.angle;e.angle=t.angle+this._getRemainingAngle(n)}return e}},_getTangentPoints:function(e,t,n){var i,a=[],r=Math.round(t.angle-e.angle),s=(t.radius-e.radius)/r;if(0===r)a=[t,e];else if(r>0)for(i=r;i>=0;i--)a.push(o(e,t,n,s,i));else for(i=0;i>=r;i--)a.push(o(e,t,n,s,r-i));return a},getSeriesPairCoord:function(e,t){var n,i=this,a=i.getArgumentAxis(),r=t?"angle":"radius",s=e[r],l=a.getCenter(),u=function(e,t){return p(g(e.x-t.x,2)+g(e.y-t.y,2))},f=function(e,t,n){var i=n.x,o=n.y;return u({x:i,y:o},l)<=a.getRadius()&&_(e.x,t.x)<=i&&m(e.x,t.x)>=i&&_(e.y,t.y)<=o&&m(e.y,t.y)>=o},y=i.getNeighborPoints(s,r);if(1===y.length)n=y[0];else if(y.length>1){var b=y[0],x=y[1];if(i.argumentAxisType!==h&&i.valueAxisType!==h){var w,C;t?(w=(b.radius-x.radius)/(b.angle-x.angle),C=s-x.angle):(w=(b.radius-x.radius)/(b.angle-x.angle),C=(s-x.radius)/w),n=o(x,b,l,w,C)}else if(t){var k=(0,c.getCosAndSin)(-s),S=(x.y-b.y)/(x.x-b.x),D=b.y-b.x*S,I=k.sin/k.cos,T=l.y-I*l.x,O=(T-D)/(S-I),E=S*O+D;if(f(b,x,{x:O,y:E})){var A=v((0,d.trunc)((360+s)/90)%4);(0===A&&O>=l.x&&E<=l.y||1===A&&O<=l.x&&E<=l.y||2===A&&O<=l.x&&E>=l.y||3===A&&O>=l.x&&E>=l.y)&&(n={x:O,y:E})}}else{var P=(x.y-b.y)/(x.x-b.x),R=b.y-b.x*P,M=1+P*P,B=-2*l.x+2*P*R-2*P*l.y,V=-g(s,2)+g(R-l.y,2)+g(l.x,2),F=B*B-4*M*V;if(F>=0){var L=(-B-p(F))/(2*M),H=(-B+p(F))/(2*M),z=P*L+R,N=P*H+R;n=f(b,x,{x:L,y:z})?{x:L,y:z}:f(b,x,{x:H,y:N})?{x:H,y:N}:void 0}}}return n},getNeighborPoints:function(e,t){var n=this.getPoints(),o=[];if(this.getOptions().closed){n=(0,l.extend)(!0,[],n);var a=n[n.length-1],r=i(n[0],n[0].x,n[0].y,n[0].angle),s=i(a,a.x,a.y,a.angle),u=this._modifyReflectedPoint(r,a),c=this._modifyReflectedPoint(s,n[0]);c&&n.unshift(c),u&&n.push(u)}for(var d=1;d<n.length&&(n[d-1][t]===e?o.push(n[d-1]):n[d][t]===e?o.push(n[d]):(n[d][t]>e&&n[d-1][t]<e||n[d-1][t]>e&&n[d][t]<e)&&(o.push(n[d-1]),o.push(n[d])),!(o.length>0));d++);return o}})},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function l(e,t,n,i,o,a){var r,s=a.allowNegatives,l=a.linearThreshold;return(0,M.tickGenerator)({axisType:e.type,dataType:e.dataType,logBase:e.logarithmBase,allowNegatives:s,linearThreshold:l,axisDivisionFactor:o(e.axisDivisionFactor||ue),minorAxisDivisionFactor:o(e.minorAxisDivisionFactor||ce),numberMultipliers:e.numberMultipliers,calculateMinors:e.minorTick.visible||e.minorGrid.visible||e.calculateMinors,allowDecimals:e.allowDecimals,endOnTick:e.endOnTick,incidentOccurred:t,firstDayOfWeek:null===(r=e.workWeek)||void 0===r?void 0:r[0],skipTickGeneration:n,skipCalculationLimits:e.skipCalculationLimits,generateExtraTick:e.generateExtraTick,minTickInterval:e.minTickInterval,rangeIsEmpty:i})}function u(e,t,n){var i=e.getOptions();return(0,F.tick)(e,t,i.tick,i.grid,n,!1)}function c(e,t){var n=e.getOptions();return(0,F.tick)(e,t,n.minorTick,n.minorGrid)}function d(e,t,n){var i=e.getOptions();return(0,F.tick)(e,t,(0,E.extend)({},i.tick,{visible:i.showCustomBoundaryTicks}),i.grid,void 0,!1,n?-1:1)}function h(e,t,n,i){(e||[]).forEach(function(e){return e[t](n,i)})}function f(e){h(e,"initCoords")}function p(e,t){h(e,"drawMark",t)}function g(e,t){h(e,"drawGrid",t)}function _(e,t,n){h(e,"updateTickPosition",t,n)}function m(e,t){h(e,"updateGridPosition",t)}function v(e){var t=e.length-1;for(t;t>=0&&y(e,t);t--);for(t=0;t<e.length&&y(e,t);t++)t--}function y(e,t){return(null===e[t].coords.x||null===e[t].coords.y)&&(e.splice(t,1),!0)}function b(e){var t,n=e.label,i=e.position,o=e.isHorizontal?te:ne,a=e.isHorizontal?ee:ie,r=n.position;if(i!==o&&i!==a&&(i=o),r&&"outside"!==r){if("inside"===r){var l;r=(l={},s(l,ee,te),s(l,te,ee),s(l,ne,ie),s(l,ie,ne),l)[i]}}else r=i;if(r!==o&&r!==a&&(r=i),n.alignment!==oe&&!n.userAlignment){var u;n.alignment=(u={},s(u,ee,oe),s(u,te,oe),s(u,ne,ie),s(u,ie,ne),u)[r]}e.position=i,n.position=r,e.hoverMode=e.hoverMode?e.hoverMode.toLowerCase():"none",n.minSpacing=null!==(t=n.minSpacing)&&void 0!==t?t:Q,e.type&&(e.type=e.type.toLowerCase()),e.argumentType&&(e.argumentType=e.argumentType.toLowerCase()),e.valueType&&(e.valueType=e.valueType.toLowerCase())}function x(e,t){var n=180*K.asin((e[0].height+t.minSpacing)/(e[1].x-e[0].x))/K.PI;return n<45?-45:-90}function w(e,t,n){e.forEach(function(e,i){e.getContentContainer()&&(i%t!==0?e.removeLabel():n&&n(e,i))})}function C(e,t){return void 0===e?t:null===e?void 0:e}function k(e,t,n,i,o){var a=(0,E.extend)({},e,{endOnTick:!0,axisDivisionFactor:t,skipCalculationLimits:!0,generateExtraTick:!0,minTickInterval:o});return function(e,t,o,r,s){return l(a,z.noop,t,n.isEmpty(),function(e){return e},n)({min:o,max:r,categories:n.categories,isSpacedMargin:n.isSpacedMargin},i,e,(0,T.isDefined)(e),void 0,void 0,void 0,s)}}function S(e,t){return Math.max(t.start,t.end)!==e?1:-1}t.Axis=t.calculateCanvasMargins=t.measureLabels=void 0;var D=n(551),I=n(14),T=n(1),O=r(n(328)),E=n(0),A=n(12),P=r(n(79)),R=n(325),M=n(552),B=n(329),V=n(134),F=n(553),L=n(29),H=r(n(20)),z=n(4),N=r(n(554)),j=a(n(936)),W=r(n(937)),G=r(n(938)),q=n(6),$=O.default.convertTicksToValues,K=Math,U=K.abs,Y=K.max,Z=K.min,X=Array.isArray,Q=5,J=4,ee=O.default.top,te=O.default.bottom,ne=O.default.left,ie=O.default.right,oe=O.default.center,ae="keep",re="shift",se="reset",le="rotate",ue=50,ce=15,de=5,he=5,fe=.8,pe={day:864e5,week:6048e5},ge=function(e){e.forEach(function(e){var t=e.getContentContainer();e.labelBBox=t?t.getBBox():{x:0,y:0,width:0,height:0}})};t.measureLabels=ge;var _e=function(e,t){var n=t.left,i=t.top,o=t.width-t.right,a=t.height-t.bottom;return e.reduce(function(e,t){return!t||t.isEmpty?e:{left:Y(e.left,n-t.x),top:Y(e.top,i-t.y),right:Y(e.right,t.x+t.width-o),bottom:Y(e.bottom,t.y+t.height-a)}},{left:0,right:0,top:0,bottom:0})};t.calculateCanvasMargins=_e;var me=function(e){var t=this;t._renderer=e.renderer,t._incidentOccurred=e.incidentOccurred,t._eventTrigger=e.eventTrigger,t._stripsGroup=e.stripsGroup,t._labelAxesGroup=e.labelAxesGroup,t._constantLinesGroup=e.constantLinesGroup,t._scaleBreaksGroup=e.scaleBreaksGroup,t._axesContainerGroup=e.axesContainerGroup,t._gridContainerGroup=e.gridGroup,t._axisCssPrefix=e.widgetClass+"-"+(e.axisClass?e.axisClass+"-":""),t._setType(e.axisType,e.drawingType),t._createAxisGroups(),t._translator=t._createTranslator(),t.isArgumentAxis=e.isArgumentAxis,t._viewport={},t._firstDrawing=!0,t._initRange={},t._getTemplate=e.getTemplate};t.Axis=me,me.prototype={constructor:me,_drawAxis:function(){var e=this._options;e.visible&&(this._axisElement=this._createAxisElement(),this._updateAxisElementPosition(),this._axisElement.attr({"stroke-width":e.width,stroke:e.color,"stroke-opacity":e.opacity}).sharp(this._getSharpParam(!0),this.getAxisSharpDirection()).append(this._axisLineGroup))},_createPathElement:function(e,t,n){return this.sharp(this._renderer.path(e,"line").attr(t),n)},sharp:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1;return e.sharp(this._getSharpParam(),t)},customPositionIsAvailable:function(){return!1},getOrthogonalAxis:z.noop,getCustomPosition:z.noop,getCustomBoundaryPosition:z.noop,resolveOverlappingForCustomPositioning:z.noop,hasCustomPosition:function(){return!1},customPositionIsBoundaryOrthogonalAxis:function(){return!1},getResolvedBoundaryPosition:function(){return this.getOptions().position},getAxisSharpDirection:function(){var e=this.getResolvedBoundaryPosition();return this.hasCustomPosition()||e!==te&&e!==ie?1:-1},getSharpDirectionByCoords:function(e){var t=this._getCanvasStartEnd(),n=Math.max(t.start,t.end);return this.getRadius?0:n!==e[this._isHorizontal?"x":"y"]?1:-1},_getGridLineDrawer:function(){var e=this;return function(t,n){var i=e._getGridPoints(t.coords);return i.points?e._createPathElement(i.points,n,e.getSharpDirectionByCoords(t.coords)):null}},_getGridPoints:function(e){var t=this,n=this._isHorizontal,i=n?"x":"y",o=this._orthogonalPositions,a=o.start,r=o.end,s=t.borderOptions,l=n?ne:ee,u=n?ie:te,c=t.getCanvas(),d={left:c.left,right:c.width-c.right,top:c.top,bottom:c.height-c.bottom},h=s.visible&&s[l]?d[l]:void 0,f=s.visible&&s[u]?d[u]:void 0,p=J+h,g=f-J;return t.areCoordsOutsideAxis(e)||void 0===e[i]||e[i]<p||e[i]>g?{points:null}:{points:n?null!==e[i]?[e[i],a,e[i],r]:null:null!==e[i]?[a,e[i],r,e[i]]:null}},_getConstantLinePos:function(e,t,n){var i=this._getTranslatedCoord(e);if(!(!(0,T.isDefined)(i)||i<Z(t,n)||i>Y(t,n)))return i},_getConstantLineGraphicAttributes:function(e){var t=this._orthogonalPositions.start,n=this._orthogonalPositions.end;return{points:this._isHorizontal?[e,t,e,n]:[t,e,n,e]}},_createConstantLine:function(e,t){return this._createPathElement(this._getConstantLineGraphicAttributes(e).points,t,S(e,this._getCanvasStartEnd()))},_drawConstantLineLabelText:function(e,t,n,i,o){var a=i.font,r=i.cssClass;return this._renderer.text(e,t,n).css((0,I.patchFontOptions)((0,E.extend)({},this._options.label.font,a))).attr({align:"center","class":r}).append(o)},_drawConstantLineLabels:function(e,t,n,i){var o,a=this,r=t.text,s=a._options,l=s.label;a._checkAlignmentConstantLineLabels(t),r=null!==(o=r)&&void 0!==o?o:a.formatLabel(e,l);var u=a._getConstantLineLabelsCoords(n,t);return a._drawConstantLineLabelText(r,u.x,u.y,t,i)},_getStripPos:function(e,t,n,i,o){var a,r,s,l,u,c=!(!o.minVisible&&!o.maxVisible),d=(o.categories||[]).reduce(function(e,t){return e.push(t.valueOf()),e},[]);if(!c&&(0,T.isDefined)(e)&&(0,T.isDefined)(t)){var h=this.parser(e),f=this.parser(t);if(l=(0,A.inArray)((0,T.isDefined)(h)?h.valueOf():void 0,d),u=(0,A.inArray)((0,T.isDefined)(f)?f.valueOf():void 0,d),l===-1||u===-1)return{from:0,to:0,outOfCanvas:!0};l>u&&(s=t,t=e,e=s)}(0,T.isDefined)(e)?(e=this.validateUnit(e,"E2105","strip"),a=this._getTranslatedCoord(e,-1)):a=n,(0,T.isDefined)(t)?(t=this.validateUnit(t,"E2105","strip"),r=this._getTranslatedCoord(t,1)):r=i;var p=a<r?{from:a,to:r}:{from:r,to:a},g=this.getVisibleArea();return(p.from<=g[0]&&p.to<=g[0]||p.from>=g[1]&&p.to>=g[1])&&(p.outOfCanvas=!0),p},_getStripGraphicAttributes:function(e,t){var n,i,o,a,r=this._orthogonalPositions,s=r.start,l=r.end;return this._isHorizontal?(n=e,i=Z(s,l),o=t-e,a=U(s-l)):(n=Z(s,l),i=e,o=U(s-l),a=U(e-t)),{x:n,y:i,width:o,height:a}},_createStrip:function(e){return this._renderer.rect(e.x,e.y,e.width,e.height)},_adjustStripLabels:function(){var e=this;this._strips.forEach(function(t){t.label&&t.label.attr(e._getAdjustedStripLabelCoords(t))})},_adjustLabelsCoord:function(e,t,n){var i=this,o=this,a=function(o){return i._getLabelAdjustedCoord(o,e+(o.labelOffset||0),t,n)};o._majorTicks.forEach(function(e){e.label?(e.updateMultilineTextAlignment(),e.label.attr(a(e))):e.templateContainer&&e.templateContainer.attr(a(e))})},_adjustLabels:function(e){var t=this,n=t.getOptions(),i=n.position===n.label.position,o=t._majorTicks.reduce(function(e,t){if(!t.getContentContainer())return e;var n=t.labelRotationAngle?(0,I.rotateBBox)(t.labelBBox,[t.labelCoords.x,t.labelCoords.y],-t.labelRotationAngle):t.labelBBox;return{width:Y(e.width||0,n.width),height:Y(e.height||0,n.height),offset:Y(e.offset||0,t.labelOffset||0)}},{}),a=i?t._isHorizontal?o.height:o.width:0;return t._adjustLabelsCoord(e,o.width),e+a+(a&&t._options.label.indentFromAxis)+(i?o.offset:0)},_getLabelAdjustedCoord:function(e,t,n){t=t||0;var i,o,a=this,r=a._options,s=e.templateContainer&&e.templateContainer.getBBox(),l=s||(0,I.rotateBBox)(e.labelBBox,[e.labelCoords.x,e.labelCoords.y],-e.labelRotationAngle||0),u=e.labelAlignment||r.label.alignment,c="discrete"===a._options.type,d=e.labelRotationAngle%90===0,h=r.label.indentFromAxis,f=r.label.position,p=a._axisPosition,g=e.labelCoords,_=g.x;return a._isHorizontal?(o=f===te?p+h-l.y+t:p-h-(l.y+l.height)-t,i=u===ie?c&&d?e.coords.x-(l.x+l.width):_-l.x-l.width:u===ne?c&&d?_-l.x-(e.coords.x-_):_-l.x:_-l.x-l.width/2):(o=g.y-l.y-l.height/2,f===ne?(i=u===ne?p-h-n-l.x:u===oe?p-h-n/2-l.x-l.width/2:p-h-l.x-l.width,i-=t):(i=u===ie?p+h+n-l.x-l.width:u===oe?p+h+n/2-l.x-l.width/2:p+h-l.x,i+=t)),{translateX:i,translateY:o}},_createAxisConstantLineGroups:function(){var e=this,t=e._renderer,n=e._axisCssPrefix,i=n+"constant-lines",o=t.g().attr({"class":i}),a=t.g().attr({"class":i}),r=t.g().attr({"class":i});return{inside:o,outside1:a,left:a,top:a,outside2:r,right:r,bottom:r,remove:function(){this.inside.remove(),this.outside1.remove(),this.outside2.remove()},clear:function(){this.inside.clear(),this.outside1.clear(),this.outside2.clear()}}},_createAxisGroups:function(){var e=this,t=e._renderer,n=e._axisCssPrefix;e._axisGroup=t.g().attr({"class":n+"axis"}).enableLinks(),e._axisStripGroup=t.g().attr({"class":n+"strips"}),e._axisGridGroup=t.g().attr({"class":n+"grid"}),e._axisElementsGroup=t.g().attr({"class":n+"elements"}).linkOn(e._axisGroup,"axisElements").linkAppend(),e._axisLineGroup=t.g().attr({"class":n+"line"}).linkOn(e._axisGroup,"axisLine").linkAppend(),e._axisTitleGroup=t.g().attr({"class":n+"title"}).append(e._axisGroup),e._axisConstantLineGroups={above:e._createAxisConstantLineGroups(),under:e._createAxisConstantLineGroups()},e._axisStripLabelGroup=t.g().attr({"class":n+"axis-labels"})},_clearAxisGroups:function(){var e=this;e._axisGroup.remove(),e._axisStripGroup.remove(),e._axisStripLabelGroup.remove(),e._axisConstantLineGroups.above.remove(),e._axisConstantLineGroups.under.remove(),e._axisGridGroup.remove(),e._axisTitleGroup.clear(),(!e._options.label.template||!e.isRendered())&&e._axisElementsGroup.clear(),e._axisLineGroup&&e._axisLineGroup.clear(),e._axisStripGroup&&e._axisStripGroup.clear(),e._axisGridGroup&&e._axisGridGroup.clear(),e._axisConstantLineGroups.above.clear(),e._axisConstantLineGroups.under.clear(),e._axisStripLabelGroup&&e._axisStripLabelGroup.clear()},_getLabelFormatObject:function(e,t,n,i,o,a){n=n||this._getViewportRange();var r={value:e,valueText:(0,D.smartFormatter)(e,{labelOptions:t,ticks:a||$(this._majorTicks),tickInterval:null!==o&&void 0!==o?o:this._tickInterval,dataType:this._options.dataType,logarithmBase:this._options.logarithmBase,type:this._options.type,showTransition:!this._options.marker.visible,point:i})||"",min:n.minVisible,max:n.maxVisible};return i&&(r.point=i),r},formatLabel:function(e,t,n,i,o,a){var r=this._getLabelFormatObject(e,t,n,i,o,a);return(0,T.isFunction)(t.customizeText)?t.customizeText.call(r,r):r.valueText},formatHint:function(e,t,n){var i=this._getLabelFormatObject(e,t,n);return(0,T.isFunction)(t.customizeHint)?t.customizeHint.call(i,i):void 0},formatRange:function(e,t,n){return(0,D.formatRange)(e,t,n,this.getOptions())},_setTickOffset:function(){var e=this._options,t=e.discreteAxisDivisionMode;this._tickOffset=+("crossLabels"!==t||!t)},resetApplyingAnimation:function(e){this._resetApplyingAnimation=!0,e&&(this._firstDrawing=!0)},getMargins:function(){var e=this;if(e.hasCustomPosition())return{left:0,right:0,top:0,bottom:0};var t=e._options,n=t.position,i=t.placeholderSize,o=e.getCanvas(),a=o.left,r=o.top,s=o.width-o.right,l=o.height-o.bottom,u=Y(t.grid.visible&&t.grid.width||0,t.tick.visible&&t.tick.width||0),c=e._axisConstantLineGroups.above,d=e._axisConstantLineGroups.under,h=[e._axisElementsGroup,c.outside1,c.outside2,d.outside1,d.outside2,e._axisLineGroup].map(function(e){return e&&e.getBBox()}).concat(function(t){var n=t&&t.getBBox();return!n||n.isEmpty?n:(e._isHorizontal?(n.x=a,n.width=s-a):(n.y=r,n.height=l-r),n)}(e._axisTitleGroup)),f=_e(h,o);return f[n]+=t.crosshairMargin,i&&(f[n]=i),u&&(e._isHorizontal&&o.right<u&&f.right<u&&(f.right=u),!e._isHorizontal&&o.bottom<u&&f.bottom<u&&(f.bottom=u)),f},validateUnit:function(e,t,n){var i=this;return e=i.parser(e),void 0===e&&t&&i._incidentOccurred(t,[n]),e},_setType:function(e,t){var n,i=this;switch(e){case"xyAxes":n=N.default;break;case"polarAxes":n=j}(0,E.extend)(i,n[t])},_getSharpParam:function(){return!0},_disposeBreaksGroup:z.noop,dispose:function(){var e=this;[e._axisElementsGroup,e._axisStripGroup,e._axisGroup].forEach(function(e){e.dispose()}),e._strips=e._title=null,e._axisStripGroup=e._axisConstantLineGroups=e._axisStripLabelGroup=e._axisBreaksGroup=null,e._axisLineGroup=e._axisElementsGroup=e._axisGridGroup=null,e._axisGroup=e._axisTitleGroup=null,e._axesContainerGroup=e._stripsGroup=e._constantLinesGroup=null,e._renderer=e._options=e._textOptions=e._textFontStyles=null,e._translator=null,e._majorTicks=e._minorTicks=null,e._disposeBreaksGroup(),e._templatesRendered&&e._templatesRendered.reject()},getOptions:function(){return this._options},setPane:function(e){this.pane=e,this._options.pane=e},setTypes:function(e,t,n){this._options.type=e||this._options.type,this._options[n]=t||this._options[n],this._updateTranslator()},resetTypes:function(e){this._options.type=this._initTypes.type,this._options[e]=this._initTypes[e]},getTranslator:function(){return this._translator},updateOptions:function(e){var t=this,n=e.label;b(e),t._options=e,e.tick=e.tick||{},e.minorTick=e.minorTick||{},e.grid=e.grid||{},e.minorGrid=e.minorGrid||{},e.title=e.title||{},e.marker=e.marker||{},t._initTypes={type:e.type,argumentType:e.argumentType,valueType:e.valueType},t._setTickOffset(),t._isHorizontal=e.isHorizontal,t.pane=e.pane,t.name=e.name,t.priority=e.priority,t._hasLabelFormat=""!==n.format&&(0,T.isDefined)(n.format),t._textOptions={opacity:n.opacity,align:"center","class":n.cssClass},t._textFontStyles=(0,I.patchFontOptions)(n.font),e.type===O.default.logarithmic&&e.logarithmBaseError&&(t._incidentOccurred("E2104"),delete e.logarithmBaseError),t._updateTranslator(),t._createConstantLines(),t._strips=(e.strips||[]).map(function(e){return(0,G.default)(t,e)}),t._majorTicks=t._minorTicks=null,t._firstDrawing=!0},calculateInterval:function(e,t){var n=this._options;if(!n||n.type!==O.default.logarithmic)return U(e-t);var i=new V.Range(this.getTranslator().getBusinessRange()),o=i.allowNegatives,a=i.linearThreshold;return U((0,I.getLogExt)(e,n.logarithmBase,o,a)-(0,I.getLogExt)(t,n.logarithmBase,o,a))},getCanvasRange:function(){var e=this._translator;return{startValue:e.from(e.translate("canvas_position_start")),endValue:e.from(e.translate("canvas_position_end"))}},_processCanvas:function(e){return e},updateCanvas:function(e,t){if(t)this._orthogonalPositions=null;else{var n=this._orthogonalPositions={start:this._isHorizontal?e.top:e.left,end:this._isHorizontal?e.height-e.bottom:e.width-e.right};n.center=n.start+(n.end-n.start)/2}this._canvas=e,this._translator.updateCanvas(this._processCanvas(e)),this._initAxisPositions()},getCanvas:function(){return this._canvas},getAxisShift:function(){return this._axisShift||0},hideTitle:function(){var e=this;e._options.title.text&&(e._incidentOccurred("W2105",[e._isHorizontal?"horizontal":"vertical"]),e._axisTitleGroup.clear())},getTitle:function(){return this._title},hideOuterElements:function(){var e=this,t=e._options;!t.label.visible&&!e._outsideConstantLines.length||e._translator.getBusinessRange().isEmpty()||(e._incidentOccurred("W2106",[e._isHorizontal?"horizontal":"vertical"]),e._axisElementsGroup.clear(),h(e._outsideConstantLines,"removeLabel"))},_resolveLogarithmicOptionsForRange:function(e){var t=this._options;t.type===O.default.logarithmic&&(e.addRange({allowNegatives:void 0!==t.allowNegatives?t.allowNegatives:e.min<=0}),isNaN(t.linearThreshold)||(e.linearThreshold=t.linearThreshold))},adjustViewport:function(e){var t=this,n=t._options,i=n.type===O.default.discrete,o=t._seriesData&&t._seriesData.categories||[],a=t.adjustRange((0,I.getVizRangeObject)(n.wholeRange)),r=t.getViewport()||{},s=new V.Range(e);t._addConstantLinesToRange(s,"minVisible","maxVisible");var l=(0,T.isDefined)(r.startValue),u=(0,T.isDefined)(r.endValue);i||(l=l&&(!(0,T.isDefined)(a.endValue)||r.startValue<a.endValue),u=u&&(!(0,T.isDefined)(a.startValue)||r.endValue>a.startValue));var c=l?r.startValue:s.minVisible,d=u?r.endValue:s.maxVisible;if(i){var h=(0,I.getCategoriesInfo)(o,a.startValue,a.endValue);o=h.categories,s.categories=o}else{var f,p;s.min=null!==(f=a.startValue)&&void 0!==f?f:s.min,s.max=null!==(p=a.endValue)&&void 0!==p?p:s.max}var g=(0,I.adjustVisualRange)({axisType:n.type,dataType:n.dataType,base:n.logarithmBase},{startValue:l?r.startValue:void 0,endValue:u?r.endValue:void 0,length:r.length},{categories:o,min:a.startValue,max:a.endValue},{categories:o,min:c,max:d});return s.minVisible=g.startValue,s.maxVisible=g.endValue,!(0,T.isDefined)(s.min)&&(s.min=s.minVisible),!(0,T.isDefined)(s.max)&&(s.max=s.maxVisible),s.addRange({}),t._resolveLogarithmicOptionsForRange(s),s},adjustRange:function(e){e=e||{};var t=this._options.type===O.default.discrete,n=this._options.type===O.default.logarithmic,i=this._options.allowNegatives===!1;if(n&&(e.startValue=i&&e.startValue<=0?null:e.startValue,e.endValue=i&&e.endValue<=0?null:e.endValue),!t&&(0,T.isDefined)(e.startValue)&&(0,T.isDefined)(e.endValue)&&e.startValue>e.endValue){var o=e.endValue;e.endValue=e.startValue,e.startValue=o}return e},_getVisualRangeUpdateMode:function(e,t,n){var i=this._options.visualRangeUpdateMode,o=this._translator,a=this._seriesData;if(this.isArgumentAxis){if([re,ae,se].indexOf(i)===-1)if(a.axisType===O.default.discrete){var r=a.categories,s=t.categories,l=this.visualRange();i=r&&s&&r.length&&s.map(function(e){return e.valueOf()}).join(",").indexOf(r.map(function(e){return e.valueOf()}).join(","))!==-1&&(l.startValue.valueOf()!==r[0].valueOf()||l.endValue.valueOf()!==r[r.length-1].valueOf())?ae:se}else{var u=o.translate(a.min),c=o.translate(e.startValue),d=o.translate(a.max),h=o.translate(e.endValue);i=u===c&&d===h?se:u!==c&&d===h?re:ae}}else[ae,se].indexOf(i)===-1&&(i=n===ae?ae:se);return i},_handleBusinessRangeChanged:function(e,t,n){var i=this,o=this.visualRange();if(!t&&!i._translator.getBusinessRange().isEmpty()){var a=i._lastVisualRangeUpdateMode=i._getVisualRangeUpdateMode(o,n,e);if(!i.isArgumentAxis){var r=i.getViewport();(0,T.isDefined)(r.startValue)||(0,T.isDefined)(r.endValue)||(0,T.isDefined)(r.length)||(a=se)}i._prevDataWasEmpty&&(a=ae),a===ae&&i._setVisualRange([o.startValue,o.endValue]),a===se&&i._setVisualRange([null,null]),a===re&&i._setVisualRange({length:i.getVisualRangeLength()})}},getVisualRangeLength:function(e){var t,n=e||this._translator.getBusinessRange(),i=this._options.type;if(i===O.default.logarithmic)t=(0,L.adjust)(this.calculateInterval(n.maxVisible,n.minVisible));else if(i===O.default.discrete){var o=(0,I.getCategoriesInfo)(n.categories,n.minVisible,n.maxVisible);t=o.categories.length}else t=n.maxVisible-n.minVisible;return t},getVisualRangeCenter:function(e){var t,n=this._translator.getBusinessRange(),i=e||n,o=this._options,a=o.type,r=o.logarithmBase;if((0,T.isDefined)(i.minVisible)&&(0,T.isDefined)(i.maxVisible)){if(a===O.default.logarithmic){var s=i.allowNegatives,l=i.linearThreshold,u=i.minVisible,c=i.maxVisible;t=(0,I.raiseToExt)((0,L.adjust)((0,I.getLogExt)(c,r,s,l)+(0,I.getLogExt)(u,r,s,l))/2,r,s,l)}else if(a===O.default.discrete){var d=(0,I.getCategoriesInfo)(i.categories,i.minVisible,i.maxVisible),h=Math.ceil(d.categories.length/2)-1;t=n.categories.indexOf(d.categories[h])}else t=(i.maxVisible.valueOf()+i.minVisible.valueOf())/2;return t}},setBusinessRange:function(e,t,n,i){var o,a,r=this,s=r._options,l=s.type===O.default.discrete;r._handleBusinessRangeChanged(n,t,e),r._seriesData=new V.Range(e);var u=r._seriesData.isEmpty();if(r._prevDataWasEmpty=u,r._seriesData.addRange({categories:s.categories,dataType:s.dataType,axisType:s.type,base:s.logarithmBase,invert:s.inverted}),r._resolveLogarithmicOptionsForRange(r._seriesData),!l){if(!(0,T.isDefined)(r._seriesData.min)&&!(0,T.isDefined)(r._seriesData.max)){var c=r.getViewport();c&&r._seriesData.addRange({min:c.startValue,max:c.endValue})}var d=s.synchronizedValue;(0,T.isDefined)(d)&&r._seriesData.addRange({min:d,max:d})}r._seriesData.minVisible=null!==(o=r._seriesData.minVisible)&&void 0!==o?o:r._seriesData.min,r._seriesData.maxVisible=null!==(a=r._seriesData.maxVisible)&&void 0!==a?a:r._seriesData.max,!r.isArgumentAxis&&s.showZero&&r._seriesData.correctValueZeroLevel(),r._seriesData.sortCategories(r.getCategoriesSorter(i)),r._seriesData.userBreaks=r._getScaleBreaks(s,r._seriesData,r._series,r.isArgumentAxis),r._translator.updateBusinessRange(r._getViewportRange())},_addConstantLinesToRange:function(e,t,n){this._outsideConstantLines.concat(this._insideConstantLines||[]).forEach(function(i){if(i.options.extendAxis){var o,a=i.getParsedValue();e.addRange((o={},s(o,t,a),s(o,n,a),o))}})},setGroupSeries:function(e){this._series=e},getLabelsPosition:function(){var e=this,t=e._options,n=t.position,i=t.label.indentFromAxis+(e._axisShift||0)+e._constantLabelOffset,o=e._axisPosition;return n===ee||n===ne?o-i:o+i},getFormattedValue:function(e,t,n){var i=this._options.label;return(0,T.isDefined)(e)?this.formatLabel(e,(0,E.extend)(!0,{},i,t),void 0,n):null},_getBoundaryTicks:function(e,t){var n=this,i=e.length,o=n._options,a=o.customBoundTicks,r=t.minVisible,s=t.maxVisible,l=o.showCustomBoundaryTicks?n._boundaryTicksVisibility:{},u=[];return o.type===O.default.discrete?n._tickOffset&&0!==e.length&&(u=[e[0],e[e.length-1]]):a?(l.min&&(0,T.isDefined)(a[0])&&u.push(a[0]),l.max&&(0,T.isDefined)(a[1])&&u.push(a[1])):(l.min&&(0===i||e[0]>r)&&u.push(r),l.max&&(0===i||e[i-1]<s)&&u.push(s)),u},setPercentLabelFormat:function(){this._hasLabelFormat||(this._options.label.format="percent")},resetAutoLabelFormat:function(){this._hasLabelFormat||delete this._options.label.format},getMultipleAxesSpacing:function(){return this._options.multipleAxesSpacing||0},getTicksValues:function(){return{majorTicksValues:$(this._majorTicks),minorTicksValues:$(this._minorTicks)}},estimateTickInterval:function(e){var t=this;return t.updateCanvas(e),t._tickInterval!==t._getTicks(t._getViewportRange(),z.noop,!0).tickInterval},setTicks:function(e){var t=e.majorTicks||[];this._majorTicks=t.map(u(this,this._renderer,this._getSkippedCategory(t))),this._minorTicks=(e.minorTicks||[]).map(c(this,this._renderer)),this._isSynchronized=!0},_adjustDivisionFactor:function(e){return e},_getTicks:function(e,t,n){var i=this,o=i._options,a=o.customTicks,r=o.customMinorTicks;return l(o,t||i._incidentOccurred,n,i._translator.getBusinessRange().isEmpty(),i._adjustDivisionFactor.bind(i),e)({min:e.minVisible,max:e.maxVisible,categories:e.categories,
isSpacedMargin:e.isSpacedMargin},i._getScreenDelta(),o.tickInterval,"ignore"===o.label.overlappingBehavior||o.forceUserTickInterval,{majors:a,minors:r},o.minorTickInterval,o.minorTickCount,i._initialBreaks)},_createTicksAndLabelFormat:function(e,t){var n=this._options,i=this._getTicks(e,t,!1);return e.isEmpty()||n.type!==O.default.discrete||"datetime"!==n.dataType||this._hasLabelFormat||!i.ticks.length||(n.label.format=P.default.getDateFormatByTicks(i.ticks)),i},getAggregationInfo:function(e,t){var n,i,o,a=this,r=a._options,s=a._marginOptions,l=new V.Range(a.getTranslator().getBusinessRange()).addRange(t),u=a.getViewport(),c=null!==(n=null===u||void 0===u?void 0:u.startValue)&&void 0!==n?n:l.minVisible,d=null!==(i=null===u||void 0===u?void 0:u.endValue)&&void 0!==i?i:l.maxVisible,h=[];if(r.type===O.default.discrete&&r.aggregateByCategory)return{aggregateByCategory:!0};var f=r.aggregationInterval,p=r.aggregationGroupWidth;!p&&s&&(s.checkInterval&&(p=r.axisDivisionFactor),s.sizePointNormalState&&(p=Math.min(s.sizePointNormalState,r.axisDivisionFactor)));var g=!r.aggregationGroupWidth&&!f&&t.interval,_=k(r,p,l,a._getScreenDelta(),g),m=_(f,!0,c,d,null===(o=a._seriesData)||void 0===o?void 0:o.breaks).tickInterval;if(r.type!==O.default.discrete){var v=e?l.min:c,y=e?l.max:d;if((0,T.isDefined)(v)&&(0,T.isDefined)(y)){var b=(0,I.getAddFunction)({base:r.logarithmBase,axisType:r.type,dataType:r.dataType},!1),x=v,w=y;if(!e){var C=Math.max(a.calculateInterval(y,v),"datetime"===r.dataType?H.default.dateToMilliseconds(m):m);x=b(v,C,-1),w=b(y,C)}x=x<l.min?l.min:x,w=w>l.max?l.max:w;var S=a._getScaleBreaks(r,{minVisible:x,maxVisible:w},a._series,a.isArgumentAxis),D=a._filterBreaks(S,{minVisible:x,maxVisible:w},r.breakStyle);h=_(m,!1,x,w,D).ticks}}return a._aggregationInterval=m,{interval:m,ticks:h}},createTicks:function(e){var t=this,n=t._renderer,i=t._options;if(e){t._isSynchronized=!1,t.updateCanvas(e);var o=t._getViewportRange();t._initialBreaks=o.breaks=this._seriesData.breaks=t._filterBreaks(this._seriesData.userBreaks,o,i.breakStyle),t._estimatedTickInterval=t._getTicks(t.adjustViewport(this._seriesData),z.noop,!0).tickInterval;var a=this._calculateValueMargins();o.addRange({minVisible:a.minValue,maxVisible:a.maxValue,isSpacedMargin:a.isSpacedMargin});var r=t._createTicksAndLabelFormat(o),s=t._getBoundaryTicks(r.ticks,t._getViewportRange());i.showCustomBoundaryTicks&&s.length?(t._boundaryTicks=[s[0]].map(d(t,n,!0)),s.length>1&&(t._boundaryTicks=t._boundaryTicks.concat([s[1]].map(d(t,n,!1))))):t._boundaryTicks=[];var l=(r.minorTicks||[]).filter(function(e){return!s.some(function(t){return(0,I.valueOf)(t)===(0,I.valueOf)(e)})});t._tickInterval=r.tickInterval,t._minorTickInterval=r.minorTickInterval;var h=t._majorTicks||[],f=h.reduce(function(e,t){return e[t.value.valueOf()]=t,e},{}),p=(0,T.type)(r.ticks[0])===(0,T.type)(h[0]&&h[0].value),g=t._getSkippedCategory(r.ticks),_=r.ticks.map(function(e){var i=f[e.valueOf()];return i&&p?(delete f[e.valueOf()],i.setSkippedCategory(g),i):u(t,n,g)(e)});t._majorTicks=_;var m=t._minorTicks||[];t._minorTicks=l.map(function(e,i){var o=m[i];return o?(o.updateValue(e),o):c(t,n)(e)}),t._ticksToRemove=Object.keys(f).map(function(e){return f[e]}).concat(m.slice(t._minorTicks.length,m.length)),t._ticksToRemove.forEach(function(e){var t;return null===(t=e.label)||void 0===t?void 0:t.removeTitle()}),r.breaks&&(t._seriesData.breaks=r.breaks),t._reinitTranslator(t._getViewportRange())}},_reinitTranslator:function(e){var t=this,n=t._translator;t._isSynchronized||n.updateBusinessRange(e)},_getViewportRange:function(){return this.adjustViewport(this._seriesData)},setMarginOptions:function(e){this._marginOptions=e},getMarginOptions:function(){var e;return null!==(e=this._marginOptions)&&void 0!==e?e:{}},_calculateRangeInterval:function(e){var t="datetime"===this._options.dataType,n=[],i=function(e){(0,T.isDefined)(e)&&n.push(t?H.default.dateToMilliseconds(e):e)};return i(this._tickInterval),i(this._estimatedTickInterval),(0,T.isDefined)(e)&&n.push(e),i(this._aggregationInterval),this._calculateWorkWeekInterval(Z.apply(this,n))},_calculateWorkWeekInterval:function(e){var t=this._options;if("datetime"===t.dataType&&t.workdaysOnly&&e){var n=t.workWeek.length*pe.day,i=pe.week-n;if(n!==e&&i<e){var o=Math.ceil(e/pe.week);e=i>=e?pe.day:e-i*o}else i>=e&&e>pe.day&&(e=pe.day)}return e},_getConvertIntervalCoefficient:function(e,t){var n=this._translator.ratioOfCanvasRange();return n/(n*t/(e+t))},_calculateValueMargins:function(e){this._resetMargins();var t,n=this,i=n.getMarginOptions(),o=(i.size||0)/2,a=n._options,r=n._getViewportRange(),s=n.getViewport(),l=n._getScreenDelta(),u=(a.type||"").indexOf(O.default.discrete)!==-1,c=a.valueMarginsEnabled&&!u&&!n.customPositionIsBoundaryOrthogonalAxis(),d=n._translator,h=a.minValueMargin,f=a.maxValueMargin,p=0,g=0,_=0;if(r.stubData||!l)return{startPadding:0,endPadding:0};if(n.isArgumentAxis&&i.checkInterval){t=n._calculateRangeInterval(r.interval);var m=d.getInterval(t);isFinite(m)?_=Math.ceil(m/(2*n._getConvertIntervalCoefficient(m,l))):t=0}var v,y,b=l*fe/2;c&&((0,T.isDefined)(h)?v=isFinite(h)?h:0:!n.isArgumentAxis&&i.checkInterval&&(0,I.valueOf)(r.minVisible)>0&&(0,I.valueOf)(r.minVisible)===(0,I.valueOf)(r.min)?p=he:(p=Math.max(o,_),p=Math.min(b,p)),(0,T.isDefined)(f)?y=isFinite(f)?f:0:!n.isArgumentAxis&&i.checkInterval&&(0,I.valueOf)(r.maxVisible)<0&&(0,I.valueOf)(r.maxVisible)===(0,I.valueOf)(r.max)?g=he:(g=Math.max(o,_),g=Math.min(b,g)));var x=i.percentStick&&!this.isArgumentAxis;x&&(1===U(r.max)&&(g=0),1===U(r.min)&&(p=0));var w=n._getCanvasStartEnd(),C=1+(v||0)+(y||0),k=(l-p-g)/C||l;void 0===v&&void 0===y||(void 0!==v&&(p=k*v),void 0!==y&&(g=k*y));var S,D;if(a.type!==O.default.discrete&&e&&e.length>1&&!a.skipViewportExtending&&!s.action&&a.endOnTick!==!1){var E=e.length,A=d.translate(e[0].value),P=d.translate(e[E-1].value),R=A>P?-1:1,M=Y(R*(w.start-A),0),B=Y(R*(P-w.end),0);if(M>p||B>g){var V=B+M,F=n._getConvertIntervalCoefficient(V,l);M>=p&&(S=e[0].value),B>=g&&(D=e[E-1].value),p=Y(M,p)/F,g=Y(B,g)/F}}if(v=void 0===v?p/k:v,y=void 0===y?g/k:y,!u)if(this._translator.isInverted()){var L,H;S=null!==(L=S)&&void 0!==L?L:d.from(w.start+l*v,-1),D=null!==(H=D)&&void 0!==H?H:d.from(w.end-l*y,1)}else{var z,N;S=null!==(z=S)&&void 0!==z?z:d.from(w.start-l*v,-1),D=null!==(N=D)&&void 0!==N?N:d.from(w.end+l*y,1)}var j=n.getCorrectedValuesToZero(S,D),W=j.correctedMin,G=j.correctedMax,q=j.start,$=j.end;return p=null!==q&&void 0!==q?q:p,g=null!==$&&void 0!==$?$:g,{startPadding:d.isInverted()?g:p,endPadding:d.isInverted()?p:g,minValue:null!==W&&void 0!==W?W:S,maxValue:null!==G&&void 0!==G?G:D,interval:t,isSpacedMargin:p===g&&0!==p}},getCorrectedValuesToZero:function(e,t){var n,i,o,a,r=this,s=r._translator,l=r._getCanvasStartEnd(),u=r._getViewportRange(),c=r._getScreenDelta(),d=r._options,h=function(e,t){var o=U(l.start-e),a=U(l.end-t),s=r._getConvertIntervalCoefficient(o+a,c);n=o/s,i=a/s};return r.isArgumentAxis||"datetime"===d.dataType||(e*u.min<=0&&e*u.minVisible<=0&&(h(s.translate(0),s.translate(t)),o=0),t*u.max<=0&&t*u.maxVisible<=0&&(h(s.translate(e),s.translate(0)),a=0)),{start:isFinite(n)?n:null,end:isFinite(i)?i:null,correctedMin:o,correctedMax:a}},applyMargins:function(){if(!this._isSynchronized){var e=this._calculateValueMargins(this._majorTicks),t=(0,E.extend)({},this._canvas,{startPadding:e.startPadding,endPadding:e.endPadding});if(this._translator.updateCanvas(this._processCanvas(t)),isFinite(e.interval)){var n=this._translator.getBusinessRange();n.addRange({interval:e.interval}),this._translator.updateBusinessRange(n)}}},_resetMargins:function(){this._reinitTranslator(this._getViewportRange()),this._canvas&&this._translator.updateCanvas(this._processCanvas(this._canvas))},_createConstantLines:function(){var e=this,t=(this._options.constantLines||[]).map(function(t){return(0,W.default)(e,t)});this._outsideConstantLines=t.filter(function(e){return"outside"===e.labelPosition}),this._insideConstantLines=t.filter(function(e){return"inside"===e.labelPosition})},draw:function(e,t){var n=this,i=this._options;n.borderOptions=t||{visible:!1},n._resetMargins(),n.createTicks(e),n.applyMargins(),n._clearAxisGroups(),f(n._majorTicks),f(n._minorTicks),f(n._boundaryTicks),n._axisGroup.append(n._axesContainerGroup),n._drawAxis(),n._drawTitle(),p(n._majorTicks,i.tick),p(n._minorTicks,i.minorTick),p(n._boundaryTicks,i.tick);var o=n._getGridLineDrawer();g(n._majorTicks,o),g(n._minorTicks,o),h(n._majorTicks,"drawLabel",n._getViewportRange(),n._getTemplate()),n._templatesRendered&&n._templatesRendered.reject(),n._templatesRendered=new q.Deferred,q.when.apply(this,n._majorTicks.map(function(e){return e.getTemplateDeferred()})).done(function(){n._templatesRendered.resolve()}),n._majorTicks.forEach(function(e){e.labelRotationAngle=0,e.labelAlignment=void 0,e.labelOffset=0}),h(n._outsideConstantLines.concat(n._insideConstantLines),"draw"),h(n._strips,"draw"),n._dateMarkers=n._drawDateMarkers()||[],n._labelAxesGroup&&n._axisStripLabelGroup.append(n._labelAxesGroup),n._gridContainerGroup&&n._axisGridGroup.append(n._gridContainerGroup),n._stripsGroup&&n._axisStripGroup.append(n._stripsGroup),n._constantLinesGroup&&(n._axisConstantLineGroups.above.inside.append(n._constantLinesGroup.above),n._axisConstantLineGroups.above.outside1.append(n._constantLinesGroup.above),n._axisConstantLineGroups.above.outside2.append(n._constantLinesGroup.above),n._axisConstantLineGroups.under.inside.append(n._constantLinesGroup.under),n._axisConstantLineGroups.under.outside1.append(n._constantLinesGroup.under),n._axisConstantLineGroups.under.outside2.append(n._constantLinesGroup.under)),n._measureTitle(),ge(n._majorTicks),!i.label.template&&n._applyWordWrap(),ge(n._outsideConstantLines),ge(n._insideConstantLines),ge(n._strips),ge(n._dateMarkers),n._adjustConstantLineLabels(n._insideConstantLines),n._adjustStripLabels();var a=n._constantLabelOffset=n._adjustConstantLineLabels(n._outsideConstantLines);n._translator.getBusinessRange().isEmpty()||(n._setLabelsPlacement(),a=n._adjustLabels(a)),a=n._adjustDateMarkers(a),n._adjustTitle(a)},getTemplatesDef:function(){return this._templatesRendered},setRenderedState:function(e){this._drawn=e},isRendered:function(){return this._drawn},_applyWordWrap:function(){var e,t,n,i=this,o=this._options,a=i._tickInterval;(0,T.isDefined)(a)&&(e=i.getTranslator().getInterval("datetime"===o.dataType?H.default.dateToMilliseconds(a):a));var r=i._validateDisplayMode(o.label.displayMode),s=i._validateOverlappingMode(o.label.overlappingBehavior,r),l=o.label.wordWrap||"none",u=o.label.textOverflow||"none";if(("none"!==l||"none"!==u)&&r!==le&&s!==le&&"auto"!==s){var c=(0,T.isDefined)(o.placeholderSize)?o.placeholderSize-o.label.indentFromAxis:void 0;i._isHorizontal?(t=e,n=c):(t=c,n=e);var d=!1,h=!1;t&&i._majorTicks.some(function(e){return e.labelBBox.width>t})&&(d=!0),n&&i._majorTicks.some(function(e){return e.labelBBox.height>n})&&(h=!0),(d||h)&&(i._majorTicks.forEach(function(e){e.label&&e.label.setMaxSize(t,n,o.label)}),ge(i._majorTicks))}},_measureTitle:z.noop,animate:function(){h(this._majorTicks,"animateLabels")},updateSize:function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],i=this;i.updateCanvas(e),n&&(i._checkTitleOverflow(),i._measureTitle(),i._updateTitleCoords()),i._reinitTranslator(i._getViewportRange()),i.applyMargins();var o=!i._firstDrawing&&t,a=this._options;f(i._majorTicks),f(i._minorTicks),f(i._boundaryTicks),this._resetApplyingAnimation&&!this._firstDrawing&&i._resetStartCoordinates(),v(i._majorTicks),v(i._minorTicks),v(i._boundaryTicks),this._axisElement&&i._updateAxisElementPosition(),_(i._majorTicks,a.tick,o),_(i._minorTicks,a.minorTick,o),_(i._boundaryTicks,a.tick),h(i._majorTicks,"updateLabelPosition",o),i._outsideConstantLines.concat(i._insideConstantLines||[]).forEach(function(e){return e.updatePosition(o)}),h(i._strips,"updatePosition",o),m(i._majorTicks,o),m(i._minorTicks,o),o&&h(i._ticksToRemove||[],"fadeOutElements"),i.prepareAnimation(),i._ticksToRemove=null,i._translator.getBusinessRange().isEmpty()||(i._firstDrawing=!1),this._resetApplyingAnimation=!1},prepareAnimation:function(){var e=this,t="saveCoords";h(e._majorTicks,t),h(e._minorTicks,t),h(e._insideConstantLines,t),h(e._outsideConstantLines,t),h(e._strips,t)},_resetStartCoordinates:function(){var e=this,t="resetCoordinates";h(e._majorTicks,t),h(e._minorTicks,t),h(e._insideConstantLines,t),h(e._outsideConstantLines,t),h(e._strips,t)},applyClipRects:function(e,t){this._axisGroup.attr({"clip-path":t}),this._axisStripGroup.attr({"clip-path":e})},_validateVisualRange:function(e){var t=(0,I.getVizRangeObject)(e);return void 0!==t.startValue&&(t.startValue=this.validateUnit(t.startValue)),void 0!==t.endValue&&(t.endValue=this.validateUnit(t.endValue)),(0,I.convertVisualRangeObject)(t,!X(e))},_validateOptions:function(e){var t=this;e.wholeRange=t._validateVisualRange(e.wholeRange),e.visualRange=e._customVisualRange=t._validateVisualRange(e._customVisualRange),t._setVisualRange(e._customVisualRange)},beforeCleanGroups:function(){this._options.label.template&&this._axisElementsGroup&&this._axisElementsGroup.linkRemove()},afterCleanGroups:function(){this._options.label.template&&this._axisElementsGroup&&this._axisElementsGroup.linkAppend()},validate:function(){var e=this,t=e._options,n=e.isArgumentAxis?t.argumentType:t.valueType,i=n?(0,R.getParser)(n):function(e){return e};e.parser=i,t.dataType=n,e._validateOptions(t)},resetVisualRange:function(e){this._seriesData.minVisible=this._seriesData.min,this._seriesData.maxVisible=this._seriesData.max,this.handleZooming([null,null],{start:!!e,end:!!e})},_setVisualRange:function(e,t){var n=this.adjustRange((0,I.getVizRangeObject)(e));t?((0,T.isDefined)(n.startValue)&&(this._viewport.startValue=n.startValue),(0,T.isDefined)(n.endValue)&&(this._viewport.endValue=n.endValue)):this._viewport=n},_applyZooming:function(e,t){var n=this;n._resetVisualRangeOption(),n._setVisualRange(e,t);var i=n.getViewport();n._seriesData.userBreaks=n._getScaleBreaks(n._options,{minVisible:i.startValue,maxVisible:i.endValue},n._series,n.isArgumentAxis),n._translator.updateBusinessRange(n._getViewportRange())},getZoomStartEventArg:function(e,t){return{axis:this,range:this.visualRange(),cancel:!1,event:e,actionType:t}},_getZoomEndEventArg:function(e,t,n,i,o){var a=this.visualRange();return{axis:this,previousRange:e,range:a,cancel:!1,event:t,actionType:n,zoomFactor:i,shift:o,rangeStart:a.startValue,rangeEnd:a.endValue}},getZoomBounds:function(){var e=(0,I.getVizRangeObject)(this._options.wholeRange),t=this.getTranslator().getBusinessRange(),n={startValue:C(this._initRange.startValue,t.min),endValue:C(this._initRange.endValue,t.max)};return{startValue:C(e.startValue,n.startValue),endValue:C(e.endValue,n.endValue)}},setInitRange:function(){this._initRange={},0===Object.keys(this._options.wholeRange||{}).length&&(this._initRange=this.getZoomBounds())},_resetVisualRangeOption:function(){this._options._customVisualRange={}},getTemplatesGroups:function(){var e=this._majorTicks;return e?this._majorTicks.map(function(e){return e.templateContainer}).filter(function(e){return(0,T.isDefined)(e)}):[]},setCustomVisualRange:function(e){this._options._customVisualRange=e},visualRange:function e(){var e,t=this,n=arguments;if(0===n.length){var i=t._getAdjustedBusinessRange(),o=i.minVisible,a=i.maxVisible;if(t._options.type===O.default.discrete){var r,s;return o=null!==(r=o)&&void 0!==r?r:i.categories[0],a=null!==(s=a)&&void 0!==s?s:i.categories[i.categories.length-1],{startValue:o,endValue:a,categories:(0,I.getCategoriesInfo)(i.categories,o,a).categories}}return{startValue:o,endValue:a}}e=X(n[0])?n[0]:(0,T.isPlainObject)(n[0])?(0,E.extend)({},n[0]):[n[0],n[1]];var l=t.handleZooming(e,n[1]);l.isPrevented||t._visualRange(t,l)},handleZooming:function(e,t,n,i){var o=this;t=t||{},(0,T.isDefined)(e)&&(e=o._validateVisualRange(e),e.action=i);var a=o.getZoomStartEventArg(n,i),r=a.range;!t.start&&o._eventTrigger("zoomStart",a);var s={isPrevented:a.cancel,skipEventRising:t.skipEventRising,range:e||a.range};return a.cancel||((0,T.isDefined)(e)&&o._applyZooming(e,t.allowPartialUpdate),(0,T.isDefined)(o._storedZoomEndParams)||(o._storedZoomEndParams={startRange:r,type:this.getOptions().type}),o._storedZoomEndParams.event=n,o._storedZoomEndParams.action=i,o._storedZoomEndParams.prevent=!!t.end),s},handleZoomEnd:function(){var e=this;if((0,T.isDefined)(e._storedZoomEndParams)&&!e._storedZoomEndParams.prevent){var t=e._storedZoomEndParams.startRange,n=e._storedZoomEndParams.event,i=e._storedZoomEndParams.action,o={minVisible:t.startValue,maxVisible:t.endValue,categories:t.categories},a=e.getOptions().type===e._storedZoomEndParams.type,r=a?(0,L.adjust)(e.getVisualRangeCenter()-e.getVisualRangeCenter(o)):NaN,s=a?+(Math.round(e.getVisualRangeLength(o)/(e.getVisualRangeLength()||1)+"e+2")+"e-2"):NaN,l=e._getZoomEndEventArg(t,n,i,s,r);l.cancel=e.checkZoomingLowerLimitOvercome(1===s?"pan":"zoom",s).stopInteraction,e._eventTrigger("zoomEnd",l),l.cancel&&e._restorePreviousVisualRange(t),e._storedZoomEndParams=null}},_restorePreviousVisualRange:function(e){var t=this;t._storedZoomEndParams=null,t._applyZooming(e),t._visualRange(t,e)},checkZoomingLowerLimitOvercome:function(e,t,n){var i,o=this,a=o._options,r=o._translator,s=a.minVisualRangeLength,l=n,u="zoom"===e&&t>=1,c=r.getBusinessRange();n&&(i=o.adjustRange((0,I.getVizRangeObject)(n)),i={minVisible:i.startValue,maxVisible:i.endValue,categories:c.categories});var d=o.getVisualRangeLength(c),h=o.getVisualRangeLength(i);if((0,T.isDefined)(s)||"discrete"===a.type)s=r.convert(s),i&&s<d&&s>=h?(l=(0,I.getVizRangeObject)(r.getRangeByMinZoomValue(s,i)),u=!1):u&=s>h;else{var f=o._translator.canvasLength,p={minVisible:c.min,maxVisible:c.max,categories:c.categories};u&=o.getVisualRangeLength(p)/f>=h}return{stopInteraction:!!u,correctedRange:l}},dataVisualRangeIsReduced:function(){var e,t,n=this.getTranslator();if("discrete"===this._options.type){var i=n.getBusinessRange().categories;e=i[0],t=i[i.length-1]}else{var o=this._seriesData;e=o.min,t=o.max}if(!(0,T.isDefined)(e)||!(0,T.isDefined)(t))return!1;var a=n.translate(e),r=n.translate(t),s=[Math.min(a,r),Math.max(a,r)],l=this.visualRange(),u=n.translate(l.startValue),c=n.translate(l.endValue);return u>s[0]&&u<s[1]||c>s[0]&&c<s[1]||u===c&&s[0]!==s[1]},isExtremePosition:function(e){var t,n;"discrete"===this._options.type?(n=this._translator.getBusinessRange(),t=e?n.categories[n.categories.length-1]:n.categories[0]):(n=this.getZoomBounds(),t=e?n.endValue:n.startValue);var i=this.getTranslator(),o=i.translate(t),a=this.visualRange(),r=e?i.translate(a.endValue):i.translate(a.startValue);return U(r-o)<de},getViewport:function(){return this._viewport},getFullTicks:function(){var e=this._majorTicks||[];return this._options.type===O.default.discrete?$(e):$(e.concat(this._minorTicks,this._boundaryTicks)).sort(function(e,t){return(0,I.valueOf)(e)-(0,I.valueOf)(t)})},measureLabels:function(e,t){var n,i,o=this,a=o._options,r=a.visible?a.width:0,s=t?a.label.indentFromAxis+.5*a.tick.length:0,l=o._getViewportRange();if(l.isEmpty()||!a.label.visible||!o._axisElementsGroup)return{height:r,width:r,x:0,y:0};o._majorTicks?n=$(o._majorTicks):(o.updateCanvas(e),n=o._createTicksAndLabelFormat(l,z.noop),i=n.tickInterval,n=n.ticks);var u=n.reduce(function(e,t,r){var s=o.formatLabel(t,a.label,l,void 0,i,n);return e.length<s.length?s:e},o.formatLabel(n[0],a.label,l,void 0,i,n)),c=o._renderer.text(u,0,0).css(o._textFontStyles).attr(o._textOptions).append(o._renderer.root),d=c.getBBox();return c.remove(),{x:d.x,y:d.y,width:d.width+s,height:d.height+s}},_setLabelsPlacement:function(){if(this._options.label.visible){var e,t=this,n=t._options.label,i=t._validateDisplayMode(n.displayMode),o=t._validateOverlappingMode(n.overlappingBehavior,i),a="none"===o||"ignore"===o,r={rotationAngle:n.rotationAngle,staggeringSpacing:n.staggeringSpacing},s=t._majorTicks.map(function(e){return e.labelBBox}),l=t._getStep(s);switch(i){case le:a&&(e=!0,l=1),t._applyLabelMode(i,l,s,n,e);break;case"stagger":a&&(l=2),t._applyLabelMode(i,Y(l,2),s,n);break;default:t._applyLabelOverlapping(s,o,l,r)}}},_applyLabelOverlapping:function(e,t,n,i){var o=this,a=o._options.label,r=o._majorTicks;if("none"!==t&&"ignore"!==t){var s=function(e,t,n){return 0!==t&&O.default.areLabelsOverlap(e,n[t-1],a.minSpacing,a.alignment)};n>1&&e.some(s)&&o._applyLabelMode(t,n,e,i),o._checkBoundedLabelsOverlapping(r,e,t)}},_applyLabelMode:function(e,t,n,i,o){var a,r,s,l=this,u=l._majorTicks,c=l._options.label,d=i.rotationAngle;switch(e){case le:c.userAlignment||(r=d<0?ie:ne,d%90===0&&(r=oe)),t=o?t:l._getStep(n,d),s=function(e){var t=e.getContentContainer();t&&(t.rotate(d),e.labelRotationAngle=d,r&&(e.labelAlignment=r))},w(u,t,s);break;case"stagger":a=l._getMaxLabelHeight(n,i.staggeringSpacing),s=function(e,n){n/(t-1)%2!==0&&(e.labelOffset=a)},w(u,t-1,s);break;case"auto":case"_auto":2===t?l._applyLabelMode("stagger",t,n,i):l._applyLabelMode(le,t,n,{rotationAngle:x(n,c)});break;default:w(u,t)}},getMarkerTrackers:z.noop,_drawDateMarkers:z.noop,_adjustDateMarkers:z.noop,coordsIn:z.noop,areCoordsOutsideAxis:z.noop,_getSkippedCategory:z.noop,_initAxisPositions:z.noop,_drawTitle:z.noop,_updateTitleCoords:z.noop,_adjustConstantLineLabels:z.noop,_createTranslator:function(){return new B.Translator2D({},{},{})},_updateTranslator:function(){var e=this._translator;e.update(e.getBusinessRange(),this._canvas||{},this._getTranslatorOptions())},_getTranslatorOptions:function(){var e,t,n,i=this._options;return{isHorizontal:this._isHorizontal,shiftZeroValue:!this.isArgumentAxis,interval:i.semiDiscreteInterval,firstDayOfWeek:null===(e=i.workWeek)||void 0===e?void 0:e[0],stick:this._getStick(),breaksSize:null!==(t=null===(n=i.breakStyle)||void 0===n?void 0:n.width)&&void 0!==t?t:0}},getVisibleArea:function(){var e=this._getCanvasStartEnd();return[e.start,e.end].sort(function(e,t){return e-t})},_getCanvasStartEnd:function(){var e=this._isHorizontal,t=this._canvas||{},n=this._translator.getBusinessRange().invert,i=e?[t.left,t.width-t.right]:[t.height-t.bottom,t.top];return n&&i.reverse(),{start:i[0],end:i[1]}},_getScreenDelta:function(){var e=this,t=e._getCanvasStartEnd(),n=e._seriesData?e._seriesData.breaks||[]:[],i=n.length,o=U(t.start-t.end);return o-(i?n[i-1].cumulativeWidth:0)},_getScaleBreaks:function(){return[]},_filterBreaks:function(){return[]},_adjustTitle:z.noop,_checkTitleOverflow:z.noop,getSpiderTicks:z.noop,setSpiderTicks:z.noop,_checkBoundedLabelsOverlapping:z.noop,drawScaleBreaks:z.noop,_visualRange:z.noop,_rotateConstantLine:z.noop,applyVisualRangeSetter:function(e){this._visualRange=e},getCategoriesSorter:function(e){var t;if(this.isArgumentAxis)t=e;else{var n=this._options.categoriesSortingMethod;t=null!==n&&void 0!==n?n:this._options.categories}return t},_getAdjustedBusinessRange:function(){return this.adjustViewport(this._translator.getBusinessRange())}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return{draw:function(n,i){var o=this,a=n._renderer.g().append(i).css((0,d.patchFontOptions)(e.font));this.plaque&&this.plaque.clear(),this.plaque=new h.Plaque((0,c.extend)(!0,{},e,{cornerRadius:(e.border||{}).cornerRadius}),n,a,t,n._isAnnotationBounded(e)),this.plaque.draw(n._getAnnotationCoords(this)),e.allowDragging&&a.on(w,{immediate:!0},function(e){o._dragOffsetX=o.plaque.x-e.pageX,o._dragOffsetY=o.plaque.y-e.pageY}).on(C,function(e){o.plaque.move(e.pageX+o._dragOffsetX,e.pageY+o._dragOffsetY)}).on(k,function(e){o.offsetX=(o.offsetX||0)+e.offset.x,o.offsetY=(o.offsetY||0)+e.offset.y})},hitTest:function(e,t){return this.plaque.hitTest(e,t)},showTooltip:function(e,t){var n=t.x,i=t.y,o=this,a=o.options;if(e.annotation!==o){e.setTemplate(a.tooltipTemplate);var r=function(t){t&&(e.annotation=o)};r(e.show(a,{x:n,y:i},{target:a},a.customizeTooltip,r))}else e.move(n,i)}}}function a(e,t){var n;return"text"===e.type?n=function(e,n){var i=t._renderer.text(e.text).attr({"class":e.cssClass}).append({element:n});(e.width>0||e.height>0)&&i.setMaxSize(e.width,e.height,{wordWrap:e.wordWrap,textOverflow:e.textOverflow})}:"image"===e.type?n=function(e,n){var i=e.image||{},o=i.width,a=i.height,r=i.url,s=i.location,l=e.width,u=e.height,c=l>0?Math.min(o,l):o,d=u>0?Math.min(a,u):a;t._renderer.image(0,0,c,d,r,s||"center").append({element:n})}:"custom"===e.type&&(n=e.template),n}function r(e){return"string"==typeof e?{url:e}:e}t.plugins=t.createAnnotations=void 0;var s=i(n(11)),l=n(1),u=n(133),c=n(0),d=n(14),h=n(549),f=i(n(25)),p=n(80),g=n(8),_=i(n(5)),m=s.default.getDocument,v="annotations",y="."+v,b=(0,g.addNamespace)([f.default.down,f.default.move],v),x=(0,g.addNamespace)(f.default.up,v),w=p.start+y,C=p.move+y,k=p.end+y,S=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=arguments.length>3?arguments[3]:void 0,s=arguments.length>4?arguments[4]:void 0,u=r(n.image);return t.reduce(function(t,d){var h=r(d.image),f=(0,l.isFunction)(i)?i(d):{};f&&(f.image=r(f.image));var p=(0,c.extend)(!0,{},n,d,{image:u},{image:h},f),g=a(p,e),_=g&&(0,c.extend)(!0,s(p),o(p,e._getTemplate(g)));return _&&t.push(_),t},[])};t.createAnnotations=S;var D={name:"annotations_chart",init:function(){},dispose:function(){},members:{_getAnnotationCoords:function(e){var t,n,i,o={offsetX:e.offsetX,offsetY:e.offsetY},a=this._options.silent("rotated")?"y":"x",r=this._options.silent("rotated")?"x":"y",s=this.getArgumentAxis(),u=s.validateUnit(e.argument),c=this.getValueAxis(e.axis),d=null===(t=c)||void 0===t?void 0:t.pane;if(e.series){var h;i=this.series.filter(function(t){return t.name===e.series})[0],c=null===(h=i)||void 0===h?void 0:h.getValueAxis(),(0,l.isDefined)(c)&&(d=c.pane)}if((0,l.isDefined)(u)){if(i){var f=i.getPointCenterByArg(u);f&&(o[a]=f[a])}else o[a]=s.getTranslator().translate(u);!(0,l.isDefined)(d)&&(d=s.pane)}var p=null===(n=c)||void 0===n?void 0:n.validateUnit(e.value);if((0,l.isDefined)(p)){var g;o[r]=null===(g=c)||void 0===g?void 0:g.getTranslator().translate(p),!(0,l.isDefined)(d)&&(0,l.isDefined)(c)&&(d=c.pane)}if(o.canvas=this._getCanvasForPane(d),(0,l.isDefined)(o[a])&&!(0,l.isDefined)(p)){var _;(0,l.isDefined)(c)||(0,l.isDefined)(i)?(0,l.isDefined)(c)&&!(0,l.isDefined)(i)?o[r]=this._argumentAxes.filter(function(e){return e.pane===c.pane})[0].getAxisPosition():null!==(_=i)&&void 0!==_&&_.checkSeriesViewportCoord(s,o[a])&&(o[r]=i.getSeriesPairCoord(o[a],!0)):o[r]=s.getAxisPosition()}return!(0,l.isDefined)(u)&&(0,l.isDefined)(o[r])&&((0,l.isDefined)(c)&&!(0,l.isDefined)(i)?o[a]=c.getAxisPosition():(0,l.isDefined)(i)&&i.checkSeriesViewportCoord(c,o[r])&&(o[a]=i.getSeriesPairCoord(o[r],!1))),o},_annotationsPointerEventHandler:function(e){var t=e.originalEvent||{},n=t.touches&&t.touches[0]||{},i=this._renderer.getRootOffset(),o={x:n.pageX||t.pageX||e.pageX,y:n.pageY||t.pageY||e.pageY},a=this._annotations.items.filter(function(e){return e.hitTest(o.x-i.left,o.y-i.top)})[0];return a&&a.options.tooltipEnabled?(this._clear(),a.options.allowDragging&&e.type===f.default.down&&(this._annotations._hideToolTipForDrag=!0),void(this._annotations._hideToolTipForDrag||(a.showTooltip(this._annotations.tooltip,o),e.stopPropagation()))):void this._annotations.hideTooltip()},_isAnnotationBounded:function(e){return(0,l.isDefined)(e.value)||(0,l.isDefined)(e.argument)},_pullOptions:function(e){return{type:e.type,name:e.name,x:e.x,y:e.y,value:e.value,argument:e.argument,axis:e.axis,series:e.series,options:e,offsetX:e.offsetX,offsetY:e.offsetY}},_forceAnnotationRender:function(){this._change(["FORCE_RENDER"])},_clear:function(){this.hideTooltip(),this.clearHover()}}},I={name:"annotations_polar_chart",init:function(){},dispose:function(){},members:{_getAnnotationCoords:function(e){var t,n,i={offsetX:e.offsetX,offsetY:e.offsetY,canvas:this._calcCanvas()},o=this.getArgumentAxis(),a=o.validateUnit(e.argument),r=this.getValueAxis().validateUnit(e.value),s=e.radius,u=e.angle;return e.series&&(n=this.series.filter(function(t){return t.name===e.series})[0]),(0,c.extend)(!0,i,this.getXYFromPolar(u,s,a,r)),(0,l.isDefined)(n)&&(!(0,l.isDefined)(i.angle)||(0,l.isDefined)(r)||(0,l.isDefined)(s)?!(0,l.isDefined)(i.radius)||(0,l.isDefined)(a)||(0,l.isDefined)(u)||(t=n.getSeriesPairCoord({radius:i.radius},!1)):((0,l.isDefined)(a)||(a=o.getTranslator().from(isFinite(u)?this.getActualAngle(u):i.angle)),t=n.getSeriesPairCoord({argument:a,angle:-i.angle},!0)),(0,l.isDefined)(t)&&(i.x=t.x,i.y=t.y)),e.series&&!(0,l.isDefined)(t)&&(i.x=i.y=void 0),i},_annotationsPointerEventHandler:D.members._annotationsPointerEventHandler,_isAnnotationBounded:D.members._isAnnotationBounded,_pullOptions:function(e){var t=(0,c.extend)({},{radius:e.radius,angle:e.angle},D.members._pullOptions(e));return delete t.axis,t},_forceAnnotationRender:D.members._forceAnnotationRender,_clear:D.members._clear}},T={name:"annotations_vector_map",init:function(){},dispose:function(){this._annotations._offTracker(),this._annotations._offTracker=null},members:{_getAnnotationCoords:function(e){var t={offsetX:e.offsetX,offsetY:e.offsetY};if(t.canvas=this._projection.getCanvas(),e.coordinates){var n=this._projection.toScreenPoint(e.coordinates);t.x=n[0],t.y=n[1]}return t},_annotationsPointerEventHandler:D.members._annotationsPointerEventHandler,_isAnnotationBounded:function(e){return(0,l.isDefined)(e.coordinates)},_pullOptions:function(e){var t=(0,c.extend)({},{coordinates:e.coordinates},D.members._pullOptions(e));return delete t.axis,delete t.series,delete t.argument,delete t.value,t},_forceAnnotationRender:function(){this._change(["EXTRA_ELEMENTS"])},_getAnnotationStyles:function(){return{"text-anchor":"start"}},_clear:function(){}},extenders:{_prepareExtraElements:function(){var e=this,t=function(){e._renderExtraElements()};e._annotations._offTracker=e._tracker.on({move:t,zoom:t,end:t})}}},O={name:"annotations_pie_chart",init:function(){},dispose:function(){},members:{_getAnnotationCoords:function(e){var t,n={offsetX:e.offsetX,offsetY:e.offsetY,canvas:this._canvas};if(e.argument){t=e.series?this.getSeriesByName(e.series):this.series[0];var i=t.getPointsByArg(e.argument)[0],o=i.getAnnotationCoords(e.location),a=o.x,r=o.y;n.x=a,n.y=r}return n},_isAnnotationBounded:function(e){return e.argument},_annotationsPointerEventHandler:D.members._annotationsPointerEventHandler,_pullOptions:function(e){var t=(0,c.extend)({},{location:e.location},D.members._pullOptions(e));return delete t.axis,t},_clear:D.members._clear,_forceAnnotationRender:D.members._forceAnnotationRender}},E={name:"annotations_core",init:function(){this._annotations={items:[],_hideToolTipForDrag:!1,tooltip:new u.Tooltip({cssClass:"".concat(this._rootClassPrefix,"-annotation-tooltip"),eventTrigger:this._eventTrigger,widgetRoot:this.element(),widget:this}),hideTooltip:function(){this.tooltip.annotation=null,this.tooltip.hide()},clearItems:function(){this.items.forEach(function(e){return e.plaque.clear()}),this.items=[]}},this._annotations.tooltip.setRendererOptions(this._getRendererOptions())},dispose:function(){this._annotationsGroup.linkRemove().linkOff(),_.default.off(m(),y),this._annotationsGroup.off(y),this._annotations.tooltip&&this._annotations.tooltip.dispose()},extenders:{_createHtmlStructure:function(){var e=this;this._annotationsGroup=this._renderer.g().attr({"class":"".concat(this._rootClassPrefix,"-annotations")}).css(this._getAnnotationStyles()).linkOn(this._renderer.root,"annotations").linkAppend(),_.default.on(m(),b,function(){return e._annotations.hideTooltip()}),_.default.on(m(),x,function(t){e._annotations._hideToolTipForDrag=!1,e._annotationsPointerEventHandler(t)}),this._annotationsGroup.on(b,this._annotationsPointerEventHandler.bind(this))},_renderExtraElements:function(){var e=this;this._annotationsGroup.clear(),this._annotations.items.forEach(function(t){return t.draw(e,e._annotationsGroup)})},_stopCurrentHandling:function(){this._annotations.hideTooltip()}},members:{
_buildAnnotations:function(){this._annotations.clearItems();var e=this._getOption("annotations",!0);null!==e&&void 0!==e&&e.length&&(this._annotations.items=S(this,e,this._getOption("commonAnnotationSettings"),this._getOption("customizeAnnotation",!0),this._pullOptions))},_setAnnotationTooltipOptions:function(){var e=(0,c.extend)({},this._getOption("tooltip"));e.contentTemplate=e.customizeTooltip=void 0,this._annotations.tooltip.update(e)},_getAnnotationCoords:function(){return{}},_pullOptions:function(){return{}},_getAnnotationStyles:function(){return{}}},customize:function(e){e.addChange({code:"ANNOTATIONITEMS",handler:function(){this._requestChange(["ANNOTATIONS"])},isOptionChange:!0,option:"annotations"}),e.addChange({code:"ANNOTATIONSSETTINGS",handler:function(){this._requestChange(["ANNOTATIONS"])},isOptionChange:!0,option:"commonAnnotationSettings"}),e.addChange({code:"ANNOTATIONS",handler:function(){this._buildAnnotations(),this._setAnnotationTooltipOptions(),this._forceAnnotationRender()},isThemeDependent:!0,isOptionChange:!0})},fontFields:["commonAnnotationSettings.font"]},A={core:E,chart:D,polarChart:I,vectorMap:T,pieChart:O};t.plugins=A},function(e,t,n){t.formatValue=t.consts=t.utils=t.HEIGHT_COMPACT_MODE=void 0;var i=n(551),o=n(1),a=24;t.HEIGHT_COMPACT_MODE=a;var r=4,s=". . .",l={trackerSettings:{fill:"grey",stroke:"grey",opacity:1e-4},animationSettings:{duration:250}};t.utils=l;var u={emptySliderMarkerText:s,pointerSize:r};t.consts=u;var c=function(e,t,n,a,r,s){var l={value:e,valueText:(0,i.smartFormatter)(e,{labelOptions:t,ticks:n?n.ticks:[],tickInterval:n?n.tickInterval:void 0,dataType:a,type:r,logarithmBase:s})};return String((0,o.isFunction)(t.customizeText)?t.customizeText.call(l,l):l.valueText)};t.formatValue=c},function(e,t,n){function i(e){return function(t){return t.isNode()?void 0:e(t)}}function o(e){return function(t){var n=!t.isNode()&&t.parent;return n?n._groupColor=n._groupColor||e(n):void 0}}function a(e,t,n){var a=(0,h.normalizeEnum)(e.type||d),r=p[a]&&p[a](e,t,n);return r?(e.colorizeGroups?o:i)(r):f.noop}function r(e,t){p[e]=t}function s(e){d=e}function l(e){return e.value}function u(e){return function(t){return Number(t.data[e])}}function c(e){return e.colorCodeField?u(e.colorCodeField):l}t.getColorizer=a,t.addColorizer=r,t.setDefaultColorizer=s,t.createColorCodeGetter=c;var d,h=n(14),f=n(4),p={}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=n(39),r=i(n(11)),s=n(7),l=n(1),u=n(97),c=n(184),d=n(238),h=(0,s.getWindow)(),f=function(e,t){return e?"string"==typeof e?"body"===e?(this[0]=t?t.body:r.default.getBody(),this.length=1,this):(t=t||r.default.getDocument(),"<"===e[0]?(this[0]=r.default.createElement(e.slice(1,-1),t),this.length=1,this):([].push.apply(this,r.default.querySelectorAll(t,e)),this)):r.default.isNode(e)||(0,l.isWindow)(e)?(this[0]=e,this.length=1,this):Array.isArray(e)?([].push.apply(this,e),this):o(e.toArray?e.toArray():[e]):(this.length=0,this)};o=function(e,t){return new f(e,t)},o.fn={dxRenderer:!0},f.prototype=o.fn;var p=function(e,t){for(var n=0;n<this.length;n++){var i=o(this[n]);i[e].apply(i,t)}return this},g=function(e,t,n){void 0!==n&&null!==n?r.default.setAttribute(e,t,n):r.default.removeAttribute(e,t)};f.prototype.show=function(){return this.toggle(!0)},f.prototype.hide=function(){return this.toggle(!1)},f.prototype.toggle=function(e){return this[0]&&this.toggleClass("dx-state-invisible",!e),this},f.prototype.attr=function(e,t){if(this.length>1&&arguments.length>1)return p.call(this,"attr",arguments);if(!this[0])return(0,l.isObject)(e)||void 0!==t?this:void 0;if(!this[0].getAttribute)return this.prop(e,t);if("string"==typeof e&&1===arguments.length){var n=this[0].getAttribute(e);return null==n?void 0:n}if((0,l.isPlainObject)(e))for(var i in e)this.attr(i,e[i]);else g(this[0],e,t);return this},f.prototype.removeAttr=function(e){return this[0]&&r.default.removeAttribute(this[0],e),this},f.prototype.prop=function(e,t){if(!this[0])return this;if("string"==typeof e&&1===arguments.length)return this[0][e];if((0,l.isPlainObject)(e))for(var n in e)this.prop(n,e[n]);else r.default.setProperty(this[0],e,t);return this},f.prototype.addClass=function(e){return this.toggleClass(e,!0)},f.prototype.removeClass=function(e){return this.toggleClass(e,!1)},f.prototype.hasClass=function(e){if(!this[0]||void 0===this[0].className)return!1;for(var t=e.split(" "),n=0;n<t.length;n++)if(this[0].classList){if(this[0].classList.contains(t[n]))return!0}else{var i=(0,l.isString)(this[0].className)?this[0].className:r.default.getAttribute(this[0],"class");if((i||"").split(" ").indexOf(t[n])>=0)return!0}return!1},f.prototype.toggleClass=function(e,t){if(this.length>1)return p.call(this,"toggleClass",arguments);if(!this[0]||!e)return this;t=void 0===t?!this.hasClass(e):t;for(var n=e.split(" "),i=0;i<n.length;i++)r.default.setClass(this[0],n[i],t);return this},["width","height","outerWidth","outerHeight","innerWidth","innerHeight"].forEach(function(e){var t=e.toLowerCase().indexOf("width")>=0?"Width":"Height",n=t.toLowerCase(),i=0===e.indexOf("outer"),o=0===e.indexOf("inner");f.prototype[e]=function(a){if(this.length>1&&arguments.length>0)return p.call(this,e,arguments);var s=this[0];if(s){if((0,l.isWindow)(s))return i?s["inner"+t]:r.default.getDocumentElement()["client"+t];if(r.default.isDocument(s)){var u=r.default.getDocumentElement(),d=r.default.getBody();return Math.max(d["scroll"+t],d["offset"+t],u["scroll"+t],u["offset"+t],u["client"+t])}if(0===arguments.length||"boolean"==typeof a){var f={paddings:o||i,borders:i,margins:a};return(0,c.getSize)(s,n,f)}if(void 0===a||null===a)return this;if((0,l.isNumeric)(a)){var g=h.getComputedStyle(s),_=(0,c.getElementBoxParams)(n,g),m="border-box"===g.boxSizing;a=Number(a),i?a-=m?0:_.border+_.padding:o?a+=m?_.border:-_.padding:m&&(a+=_.border+_.padding)}return a+=(0,l.isNumeric)(a)?"px":"",r.default.setStyle(s,n,a),this}}}),f.prototype.html=function(e){return arguments.length?(this.empty(),"string"==typeof e&&!(0,d.isTablePart)(e)||"number"==typeof e?(this[0].innerHTML=e,this):this.append((0,d.parseHTML)(e))):this[0].innerHTML};var _=function(e,t){if(this[0]&&e){"string"==typeof e?e=(0,d.parseHTML)(e):e.nodeType?e=[e]:(0,l.isNumeric)(e)&&(e=[r.default.createTextNode(e)]);for(var n=0;n<e.length;n++){var i=e[n],o=this[0],a="TABLE"===o.tagName&&"TR"===i.tagName;a&&o.tBodies&&o.tBodies.length&&(o=o.tBodies[0]),r.default.insertElement(o,i.nodeType?i:i[0],t)}}},m=function(e,t){if(this[0]&&this[0].style&&!(null===t||"number"==typeof t&&isNaN(t))){e=(0,u.styleProp)(e);for(var n=0;n<this.length;n++)this[n].style[e]=(0,u.normalizeStyleProp)(e,t)}};f.prototype.css=function(e,t){if((0,l.isString)(e)){if(2!==arguments.length){if(!this[0])return;e=(0,u.styleProp)(e);var n=h.getComputedStyle(this[0])[e]||this[0].style[e];return(0,l.isNumeric)(n)?n.toString():n}m.call(this,e,t)}else if((0,l.isPlainObject)(e))for(var i in e)m.call(this,i,e[i]);return this},f.prototype.prepend=function(e){if(arguments.length>1){for(var t=0;t<arguments.length;t++)this.prepend(arguments[t]);return this}return _.apply(this,[e,this[0].firstChild]),this},f.prototype.append=function(e){if(arguments.length>1){for(var t=0;t<arguments.length;t++)this.append(arguments[t]);return this}return _.apply(this,[e]),this},f.prototype.prependTo=function(e){if(this.length>1){for(var t=this.length-1;t>=0;t--)o(this[t]).prependTo(e);return this}return e=o(e),e[0]&&r.default.insertElement(e[0],this[0],e[0].firstChild),this},f.prototype.appendTo=function(e){return this.length>1?p.call(this,"appendTo",arguments):(r.default.insertElement(o(e)[0],this[0]),this)},f.prototype.insertBefore=function(e){return e&&e[0]&&r.default.insertElement(e[0].parentNode,this[0],e[0]),this},f.prototype.insertAfter=function(e){return e&&e[0]&&r.default.insertElement(e[0].parentNode,this[0],e[0].nextSibling),this},f.prototype.before=function(e){return this[0]&&r.default.insertElement(this[0].parentNode,e[0],this[0]),this},f.prototype.after=function(e){return this[0]&&r.default.insertElement(this[0].parentNode,e[0],this[0].nextSibling),this},f.prototype.wrap=function(e){if(this[0]){var t=o(e);t.insertBefore(this),t.append(this)}return this},f.prototype.wrapInner=function(e){var t=this.contents();return t.length?t.wrap(e):this.append(e),this},f.prototype.replaceWith=function(e){if(e&&e[0])return e.is(this)?this:(e.insertBefore(this),this.remove(),e)},f.prototype.remove=function(){return this.length>1?p.call(this,"remove",arguments):((0,a.cleanDataRecursive)(this[0],!0),r.default.removeElement(this[0]),this)},f.prototype.detach=function(){return this.length>1?p.call(this,"detach",arguments):(r.default.removeElement(this[0]),this)},f.prototype.empty=function(){return this.length>1?p.call(this,"empty",arguments):((0,a.cleanDataRecursive)(this[0]),r.default.setText(this[0],""),this)},f.prototype.clone=function(){for(var e=[],t=0;t<this.length;t++)e.push(this[t].cloneNode(!0));return o(e)},f.prototype.text=function(e){if(!arguments.length){for(var t="",n=0;n<this.length;n++)t+=this[n]&&this[n].textContent||"";return t}var i=(0,l.isFunction)(e)?e():e;return(0,a.cleanDataRecursive)(this[0],!1),r.default.setText(this[0],(0,l.isDefined)(i)?i:""),this},f.prototype.val=function(e){return 1===arguments.length?this.prop("value",(0,l.isDefined)(e)?e:""):this.prop("value")},f.prototype.contents=function(){if(!this[0])return o();var e=[];return e.push.apply(e,this[0].childNodes),o(e)},f.prototype.find=function(e){var t=o();if(!e)return t;var n,i=[];if("string"==typeof e)for(e=e.trim(),n=0;n<this.length;n++){var a=this[n];if(r.default.isElementNode(a)){var s=a.getAttribute("id"),l=s||"dx-query-children";s||g(a,"id",l),l="[id='"+l+"'] ";var u=l+e.replace(/([^\\])(,)/g,"$1, "+l);i.push.apply(i,r.default.querySelectorAll(a,u)),g(a,"id",s)}else r.default.isDocument(a)&&i.push.apply(i,r.default.querySelectorAll(a,e))}else for(n=0;n<this.length;n++)e=r.default.isNode(e)?e:e[0],this[n]!==e&&this[n].contains(e)&&i.push(e);return t.add(i)};var v=function(e,t){return!t.nodeType||!!(t.offsetWidth||t.offsetHeight||t.getClientRects().length)};f.prototype.filter=function(e){if(!e)return o();if(":visible"===e)return this.filter(v);if(":hidden"===e)return this.filter(function(e,t){return!v(e,t)});for(var t=[],n=0;n<this.length;n++){var i=this[n];if(r.default.isElementNode(i)&&"string"===(0,l.type)(e))r.default.elementMatches(i,e)&&t.push(i);else if(r.default.isNode(e)||(0,l.isWindow)(e))e===i&&t.push(i);else if((0,l.isFunction)(e))e.call(i,n,i)&&t.push(i);else for(var a=0;a<e.length;a++)e[a]===i&&t.push(i)}return o(t)},f.prototype.not=function(e){for(var t=[],n=this.filter(e).toArray(),i=0;i<this.length;i++)n.indexOf(this[i])===-1&&t.push(this[i]);return o(t)},f.prototype.is=function(e){return!!this.filter(e).length},f.prototype.children=function(e){for(var t=[],n=0;n<this.length;n++)for(var i=this[n]?this[n].childNodes:[],a=0;a<i.length;a++)r.default.isElementNode(i[a])&&t.push(i[a]);return t=o(t),e?t.filter(e):t},f.prototype.siblings=function(){var e=this[0];if(!e||!e.parentNode)return o();for(var t=[],n=e.parentNode.childNodes||[],i=0;i<n.length;i++){var a=n[i];r.default.isElementNode(a)&&a!==e&&t.push(a)}return o(t)},f.prototype.each=function(e){for(var t=0;t<this.length&&e.call(this[t],t,this[t])!==!1;t++);},f.prototype.index=function(e){return e?(e=o(e),this.toArray().indexOf(e[0])):this.parent().children().index(this)},f.prototype.get=function(e){return this[e<0?this.length+e:e]},f.prototype.eq=function(e){return e=e<0?this.length+e:e,o(this[e])},f.prototype.first=function(){return this.eq(0)},f.prototype.last=function(){return this.eq(-1)},f.prototype.parent=function(e){if(!this[0])return o();var t=o(this[0].parentNode);return!e||t.is(e)?t:o()},f.prototype.parents=function(e){for(var t=[],n=this.parent();n&&n[0]&&!r.default.isDocument(n[0]);)r.default.isElementNode(n[0])&&(!e||e&&n.is(e))&&t.push(n.get(0)),n=n.parent();return o(t)},f.prototype.closest=function(e){if(this.is(e))return this;for(var t=this.parent();t&&t.length;){if(t.is(e))return t;t=t.parent()}return o()},f.prototype.next=function(e){if(!this[0])return o();var t=o(this[0].nextSibling);if(!arguments.length)return t;for(;t&&t.length;){if(t.is(e))return t;t=t.next()}return o()},f.prototype.prev=function(){return this[0]?o(this[0].previousSibling):o()},f.prototype.add=function(e){for(var t=o(e),n=this.toArray(),i=0;i<t.length;i++){var a=t[i];n.indexOf(a)===-1&&n.push(a)}return o(n)};var y=[];f.prototype.splice=function(){return o(y.splice.apply(this,arguments))},f.prototype.slice=function(){return o(y.slice.apply(this,arguments))},f.prototype.toArray=function(){return y.slice.call(this)};var b=function(e){return(0,l.isWindow)(e)?e:e.defaultView};f.prototype.offset=function(){if(this[0]){if(!this[0].getClientRects().length)return{top:0,left:0};var e=this[0].getBoundingClientRect(),t=b(this[0].ownerDocument),n=this[0].ownerDocument.documentElement;return{top:e.top+t.pageYOffset-n.clientTop,left:e.left+t.pageXOffset-n.clientLeft}}},f.prototype.offsetParent=function(){if(!this[0])return o();for(var e=o(this[0].offsetParent);e[0]&&"static"===e.css("position");)e=o(e[0].offsetParent);return e=e[0]?e:o(r.default.getDocumentElement())},f.prototype.position=function(){if(this[0]){var e,t=parseFloat(this.css("marginTop")),n=parseFloat(this.css("marginLeft"));if("fixed"===this.css("position"))return e=this[0].getBoundingClientRect(),{top:e.top-t,left:e.left-n};e=this.offset();var i=this.offsetParent(),o={top:0,left:0};return"HTML"!==i[0].nodeName&&(o=i.offset()),o={top:o.top+parseFloat(i.css("borderTopWidth")),left:o.left+parseFloat(i.css("borderLeftWidth"))},{top:e.top-o.top-t,left:e.left-o.left-n}}},[{name:"scrollLeft",offsetProp:"pageXOffset",scrollWindow:function(e,t){e.scrollTo(t,e.pageYOffset)}},{name:"scrollTop",offsetProp:"pageYOffset",scrollWindow:function(e,t){e.scrollTo(e.pageXOffset,t)}}].forEach(function(e){var t=e.name;f.prototype[t]=function(n){if(this[0]){var i=b(this[0]);return void 0===n?i?i[e.offsetProp]:this[0][t]:(i?e.scrollWindow(i,n):this[0][t]=n,this)}}}),f.prototype.data=function(e,t){if(this[0])return arguments.length<2?a.data.call(o,this[0],e):(a.data.call(o,this[0],e,t),this)},f.prototype.removeData=function(e){return this[0]&&(0,a.removeData)(this[0],e),this};var x=function(){return o.apply(this,arguments)};Object.defineProperty(x,"fn",{enumerable:!0,configurable:!0,get:function(){return o.fn},set:function(e){o.fn=e}});var w={set:function(e){o=e},get:function(){return x}};t.default=w,e.exports=t.default},function(e,t,n){function i(e){o.forEach(function(t){e(t,function(e){return a(t,e)})},this)}t.default=i;var o=["pageX","pageY","screenX","screenY","clientX","clientY"],a=function(e,t){if(t[e]&&!t.touches||!t.touches)return t[e];var n=t.touches.length?t.touches:t.changedTouches;if(n.length)return n[0][e]};e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.isTablePart=t.parseHTML=void 0;var o=n(12),a=i(n(11)),r=/<([a-z][^/\0>\x20\t\r\n\f]+)/i,s={"default":{tagsCount:0,startTags:"",endTags:""},thead:{tagsCount:1,startTags:"<table>",endTags:"</table>"},td:{tagsCount:3,startTags:"<table><tbody><tr>",endTags:"</tr></tbody></table>"},col:{tagsCount:2,startTags:"<table><colgroup>",endTags:"</colgroup></table>"},tr:{tagsCount:2,startTags:"<table><tbody>",endTags:"</tbody></table>"}};s.tbody=s.colgroup=s.caption=s.tfoot=s.thead,s.th=s.td;var l=function(e){if("string"!=typeof e)return null;var t=a.default.createDocumentFragment(),n=t.appendChild(a.default.createElement("div")),i=r.exec(e),l=i&&i[1].toLowerCase(),u=s[l]||s.default;n.innerHTML=u.startTags+e+u.endTags;for(var c=0;c<u.tagsCount;c++)n=n.lastChild;return(0,o.merge)([],n.childNodes)};t.parseHTML=l;var u=function(e){var t=r.exec(e);return t&&t[1]in s};t.isTablePart=u},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(42),a=i(n(45)),r=i(n(32)),s=n(1),l=n(161),u=n(205);n(107);var c=1632,d={thousands:"#,##0{0},&quot;K&quot;",millions:"#,##0{0},,&quot;M&quot;",billions:"#,##0{0},,,&quot;B&quot;",trillions:"#,##0{0},,,,&quot;T&quot;",percent:"0{0}%",decimal:"#{0}",fixedpoint:"#,##0{0}",exponential:"0{0}E+00",currency:" "},h=/a+/g,f=/E/g,p=/dE+/g,g=/L/g,_=/h/g,m=/\//g,v=/\[/g,y=/]/g,b=/./g,x={_applyPrecision:function(e,t){var n,i;if(t>0){for(n="decimal"!==e?".":"",i=0;i<t;i++)n+="0";return n}return""},_hasArabicDigits:function(e){for(var t,n=0;n<e.length;n++)if(t=e.charCodeAt(n),t>=c&&t<c+10)return!0;return!1},_convertDateFormatToOpenXml:function(e){return e.replace(m,"\\/").split("'").map(function(e,t){return t%2===0?e.replace(h,"AM/PM").replace(p,"d").replace(f,"d").replace(g,"M").replace(_,"H").replace(v,"\\[").replace(y,"\\]"):e?e.replace(b,"\\$&"):"'"}).join("")},_convertDateFormat:function(e){var t=(r.default.format(new Date(2009,8,8,6,5,4),e)||"").toString(),n=(0,l.getFormat)(function(t){return r.default.format(t,e)});return n&&(n=this._convertDateFormatToOpenXml(n),n=this._getLanguageInfo(t)+n),n},_getLanguageInfo:function(e){var t=(0,u.getLanguageId)(),n=t?t.toString(16):"",i="";if(this._hasArabicDigits(e)){for(;n.length<3;)n="0"+n;i="[$-2010"+n+"]"}else n&&(i="[$-"+n+"]");return i},_convertNumberFormat:function(e,t,n){var i,r;return r="currency"===e?a.default.getOpenXmlCurrencyFormat(n):d[e.toLowerCase()],r&&(i=(0,o.format)(r,this._applyPrecision(e,t))),i},convertFormat:function(e,t,n,i){if((0,s.isDefined)(e)){if("date"===n)return x._convertDateFormat(e);if((0,s.isString)(e)&&d[e.toLowerCase()])return x._convertNumberFormat(e,t,i)}}},w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){var t=(0,r.sign)(e);return t*Math.round(Math.abs(e))}function o(e,t){var n=Math.pow(s,t),o=n*s,a=e*o/s;return i(a)/n}function a(e,t){var n=t||0,i=n>0?o.apply(void 0,arguments):e;return i.toFixed(n)}t.toFixed=a;var r=n(29),s=10},function(e,t,n){t.default=void 0;var i=function(e,t){if(t){var n=e;if("string"==typeof e){n="";for(var i=0;i<e.length;i++)"$"!==e[i]&&(n+="\\"),n+=e[i]}for(var o={".00":"{0}","'":"\\'","\\(":"\\(","\\)":"\\)"," ":"\\ ",'"':"&quot;","\\\xa4":n},a=t.split(";"),r=0;r<a.length;r++)for(var s in o)Object.prototype.hasOwnProperty.call(o,s)&&(a[r]=a[r].replace(new RegExp(s,"g"),o[s]));return 2===a.length?a[0]+"_);"+a[1]:a[0]}};t.default=i,e.exports=t.default},function(e,t,n){function i(e,t){for(;e.length<t;)e="0"+e;return e}t.getFormatter=void 0;var o={3:"abbreviated",4:"wide",5:"narrow"},a={y:function(e,t,n){var o=e[n?"getUTCFullYear":"getFullYear"]();return 2===t&&(o%=100),i(o.toString(),t)},M:function(e,t,n,a){var r=e[n?"getUTCMonth":"getMonth"](),s=o[t];return s?a.getMonthNames(s,"format")[r]:i((r+1).toString(),Math.min(t,2))},L:function(e,t,n,a){var r=e[n?"getUTCMonth":"getMonth"](),s=o[t];return s?a.getMonthNames(s,"standalone")[r]:i((r+1).toString(),Math.min(t,2))},Q:function(e,t,n,a){var r=e[n?"getUTCMonth":"getMonth"](),s=Math.floor(r/3),l=o[t];return l?a.getQuarterNames(l)[s]:i((s+1).toString(),Math.min(t,2))},E:function(e,t,n,i){var a=e[n?"getUTCDay":"getDay"](),r=o[t<3?3:t];return i.getDayNames(r)[a]},a:function(e,t,n,i){var a=e[n?"getUTCHours":"getHours"](),r=a<12?0:1,s=o[t];return i.getPeriodNames(s)[r]},d:function(e,t,n){return i(e[n?"getUTCDate":"getDate"]().toString(),Math.min(t,2))},H:function(e,t,n){return i(e[n?"getUTCHours":"getHours"]().toString(),Math.min(t,2))},h:function(e,t,n){var o=e[n?"getUTCHours":"getHours"]();return i((o%12||12).toString(),Math.min(t,2))},m:function(e,t,n){return i(e[n?"getUTCMinutes":"getMinutes"]().toString(),Math.min(t,2))},s:function(e,t,n){return i(e[n?"getUTCSeconds":"getSeconds"]().toString(),Math.min(t,2))},S:function(e,t,n){return i(e[n?"getUTCMilliseconds":"getMilliseconds"]().toString(),3).substr(0,t)},x:function(e,t,n){var o=n?0:e.getTimezoneOffset(),a=o>0?"-":"+",r=Math.abs(o),s=Math.floor(r/60),l=r%60,u=i(s.toString(),2),c=i(l.toString(),2);return a+u+(t>=3?":":"")+(t>1||l?c:"")},X:function(e,t,n){return n||!e.getTimezoneOffset()?"Z":a.x(e,t,n)},Z:function(e,t,n){return a.X(e,t>=5?3:2,n)}},r=function(e,t){return function(n){var i,o,r,s,l=0,u="'",c=!1,d="";if(!n)return null;if(!e)return n;var h="Z"===e[e.length-1]||"'Z'"===e.slice(-3);for(i=0;i<e.length;i++)r=e[i],o=a[r],s=r===e[i+1],l++,s||(o&&!c&&(d+=o(n,l,h,t)),l=0),r!==u||s?!c&&o||(d+=r):c=!c,r===u&&s&&i++;return d}};t.getFormatter=r},function(e,t,n){t.default=void 0;var i=n(3),o=["January","February","March","April","May","June","July","August","September","October","November","December"],a=["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],r=["AM","PM"],s=["Q1","Q2","Q3","Q4"],l=function(e,t){var n={abbreviated:3,"short":2,narrow:1};return(0,i.map)(e,function(e){return e.substr(0,n[t])})},u={getMonthNames:function(e){return l(o,e)},getDayNames:function(e){return l(a,e)},getQuarterNames:function(e){return s},getPeriodNames:function(e){return r}};t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r=i(n(345)),s={tryCreateTag:function(e){var t=null;return(0,o.isDefined)(e)&&(t={patternFill:r.default.tryCreateTag(e.patternFill)},s.isEmpty(t)&&(t=null)),t},tryCreateFillFromSimpleFormat:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.backgroundColor,n=e.fillPatternType,i=e.fillPatternColor;return!(0,o.isDefined)(t)||(0,o.isDefined)(n)&&(0,o.isDefined)(i)?(0,o.isDefined)(n)&&(0,o.isDefined)(i)?{patternFill:{patternType:n,foregroundColor:{rgb:i},backgroundColor:{rgb:t}}}:void 0:{patternFill:{patternType:"solid",foregroundColor:{rgb:t}}}},copySimpleFormat:function(e,t){void 0!==e.backgroundColor&&(t.backgroundColor=e.backgroundColor),void 0!==e.fillPatternType&&(t.fillPatternType=e.fillPatternType),void 0!==e.fillPatternColor&&(t.fillPatternColor=e.fillPatternColor)},copy:function(e){var t=null;return(0,o.isDefined)(e)&&(t={},void 0!==e.patternFill&&(t.patternFill=r.default.copy(e.patternFill))),t},areEqual:function(e,t){return s.isEmpty(e)&&s.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&r.default.areEqual(e.patternFill,t.patternFill)},isEmpty:function(e){return!(0,o.isDefined)(e)||r.default.isEmpty(e.patternFill)},toXml:function(e){return a.default.toXml("fill",{},r.default.toXml(e.patternFill))}},l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r={_tryConvertColor:function(e){if("string"!=typeof e)return e;var t;if(e.length>0&&"#"===e[0]){var n=e.substr(1,e.length);t=6===n.length?"FF"+n:8===n.length?n[6]+n[7]+n.substr(0,6):n}else t=e;return t},tryCreateTag:function(e){var t=null;return(0,o.isDefined)(e)&&(t="string"==typeof e?{rgb:this._tryConvertColor(e)}:{rgb:this._tryConvertColor(e.rgb),theme:e.theme},r.isEmpty(t)&&(t=null)),t},copy:function(e){var t=null;return(0,o.isDefined)(e)&&("string"==typeof e?t=e:(t={},void 0!==e.rgb&&(t.rgb=e.rgb),void 0!==e.theme&&(t.theme=e.theme))),t},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.rgb)&&!(0,o.isDefined)(e.theme)},areEqual:function(e,t){return r.isEmpty(e)&&r.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.rgb===t.rgb&&e.theme===t.theme},toXml:function(e,t){return a.default.toXml(e,{rgb:t.rgb,theme:t.theme})}},s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r=i(n(245)),s={tryCreateTag:function(e){var t=null;return(0,o.isDefined)(e)&&(t={size:e.size,name:e.name,family:e.family,scheme:e.scheme,bold:e.bold,italic:e.italic,underline:e.underline,color:r.default.tryCreateTag(e.color)},s.isEmpty(t)&&(t=null)),t},copy:function(e){var t=null;return(0,o.isDefined)(e)&&(t={},void 0!==e.size&&(t.size=e.size),void 0!==e.name&&(t.name=e.name),void 0!==e.family&&(t.family=e.family),void 0!==e.scheme&&(t.scheme=e.scheme),void 0!==e.bold&&(t.bold=e.bold),void 0!==e.italic&&(t.italic=e.italic),void 0!==e.underline&&(t.underline=e.underline),void 0!==e.color&&(t.color=r.default.copy(e.color))),t},areEqual:function(e,t){return s.isEmpty(e)&&s.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.size===t.size&&e.name===t.name&&e.family===t.family&&e.scheme===t.scheme&&(e.bold===t.bold||!e.bold==!t.bold)&&(e.italic===t.italic||!e.italic==!t.italic)&&e.underline===t.underline&&r.default.areEqual(e.color,t.color)},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.size)&&!(0,o.isDefined)(e.name)&&!(0,o.isDefined)(e.family)&&!(0,o.isDefined)(e.scheme)&&(!(0,o.isDefined)(e.bold)||!e.bold)&&(!(0,o.isDefined)(e.italic)||!e.italic)&&!(0,o.isDefined)(e.underline)&&r.default.isEmpty(e.color)},toXml:function(e){var t=[(0,o.isDefined)(e.bold)&&e.bold?a.default.toXml("b",{}):"",(0,o.isDefined)(e.size)?a.default.toXml("sz",{val:e.size}):"",(0,o.isDefined)(e.color)?r.default.toXml("color",e.color):"",(0,o.isDefined)(e.name)?a.default.toXml("name",{val:e.name}):"",(0,o.isDefined)(e.family)?a.default.toXml("family",{val:e.family}):"",(0,o.isDefined)(e.scheme)?a.default.toXml("scheme",{val:e.scheme}):"",(0,o.isDefined)(e.italic)&&e.italic?a.default.toXml("i",{}):"",(0,o.isDefined)(e.underline)?a.default.toXml("u",{val:e.underline}):""].join("");return a.default.toXml("font",{},t)}},l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=(0,z.default)("<canvas>")[0];return i.width=e+2*n,i.height=t+2*n,i.hidden=!0,i}function a(e,t){var n=e.toDataURL(t,le),i=X.atob(n.substring(("data:"+t+";base64,").length));return i}function r(e,t,n,i,o,a,r,s){var l=(e+n)/2,u=(t+i)/2,c=oe(t-i,e-n),d=a?1:-1;c+=90*(J/180)*(r?1:-1);var h=ne(ie(n-e,2)+ie(i-t,2))/2,f=ne(te(ie(o,2)-ie(h,2))),p=l+d*(f*ae(c)),g=u+d*(f*re(c)),_=oe(t-g,e-p),m=oe(i-g,n-p);s.arc(p,g,o,_,m,!r)}function s(e,t){var n,i=H(e.attributes||{}),o=(0,q.extend)({},i,{text:e.textContent.replace(/\s+/g," "),textAlign:"middle"===i["text-anchor"]?"center":i["text-anchor"]}),a=i.transform;return a&&(n=a.match(/translate\(-*\d+([.]\d+)*(,*\s*-*\d+([.]\d+)*)*/),n&&(n=n[0].match(/-*\d+([.]\d+)*/g),o.translateX=se(n[0]),o.translateY=n[1]?se(n[1]):0),n=a.match(/rotate\(-*\d+([.]\d+)*(,*\s*-*\d+([.]\d+)*,*\s*-*\d+([.]\d+)*)*/),n&&(n=n[0].match(/-*\d+([.]\d+)*/g),o.rotationAngle=se(n[0]),o.rotationX=n[1]&&se(n[1]),o.rotationY=n[2]&&se(n[2])),n=a.match(/scale\(-*\d+([.]\d+)*(,*\s*-*\d+([.]\d+)*)*/),n&&(n=n[0].match(/-*\d+([.]\d+)*/g),o.scaleX=se(n[0]),n.length>1?o.scaleY=se(n[1]):o.scaleY=o.scaleX)),d(e,o,t),o}function l(e,t){var n=t.x,i=t.y,o=t.width,a=t.height,r=t.rx;r?(r=ee(r,o/2,a/2),e.save(),e.translate(n,i),e.moveTo(o/2,0),e.arcTo(o,0,o,a,r),e.arcTo(o,a,0,a,r),e.arcTo(0,a,0,0,r),e.arcTo(0,0,r,0,r),e.lineTo(o/2,0),e.restore()):e.rect(n,i,o,a)}function u(e,t,n){var i=new Z.Deferred,o=new X.Image;return o.onload=function(){e.save(),e.globalAlpha=t.globalAlpha,w(e,t),C(e,t,n),e.drawImage(o,t.x,t.y,t.width,t.height),e.restore(),i.resolve()},o.onerror=function(){i.resolve()},o.setAttribute("crossOrigin","anonymous"),o.src=t.href||t["xlink:href"],i}function c(e,t){var n,i,o,a=t.replace(/,/g," ").split(/([A-Z])/i).filter(function(e){return""!==e.trim()}),s=0;do switch(n=(a[s+1]||"").trim().split(" "),a[s]){case"M":e.moveTo(se(n[0]),se(n[1])),s+=2;break;case"L":for(var l=0;l<n.length/2;l++)e.lineTo(se(n[2*l]),se(n[2*l+1]));s+=2;break;case"C":e.bezierCurveTo(se(n[0]),se(n[1]),se(n[2]),se(n[3]),se(n[4]),se(n[5])),s+=2;break;case"a":i=a[s-1].trim().split(" "),o=i.length-1,r(se(i[o-1]),se(i[o]),se(i[o-1])+se(n[5]),se(i[o])+se(n[6]),se(n[0]),se(n[3]),se(n[4]),e),s+=2;break;case"A":i=a[s-1].trim().split(" "),o=i.length-1,r(se(i[o-1]),se(i[o]),se(n[5]),se(n[6]),se(n[0]),se(n[3]),se(n[4]),e),s+=2;break;case"Z":e.closePath(),s+=1;break;default:s++}while(s<a.length)}function d(e,t,n){var i,o=e.style||{};for(i in o)""!==o[i]&&(t[(0,Y.camelize)(i)]=o[i]);n&&$.default.isElementNode(e)&&(o=X.getComputedStyle(e),["fill","stroke","stroke-width","font-family","font-size","font-style","font-weight"].forEach(function(e){e in o&&""!==o[e]&&(t[(0,Y.camelize)(e)]=o[e])}),["opacity","fill-opacity","stroke-opacity"].forEach(function(e){e in o&&""!==o[e]&&"1"!==o[e]&&(t[e]=se(o[e]))})),t.textDecoration=t.textDecoration||t.textDecorationLine,t.globalAlpha=(0,j.isDefined)(t.opacity)?t.opacity:t.globalAlpha}function h(e){var t=e&&e.match(/url\(.*#(.*?)["']?\)/i);return t&&t[1]}function f(e,t){var n=[];t.fontSize=t.fontSize||ce,t.fontFamily||de,t.fill=t.fill||he,t.fontStyle&&n.push(t.fontStyle),t.fontWeight&&n.push(t.fontWeight),n.push(t.fontSize),n.push(t.fontFamily),e.font=n.join(" "),e.textAlign=t.textAlign,e.fillStyle=t.fill,e.globalAlpha=t.globalAlpha}function p(e,t,n){f(e,t),x(e,t,n),t.text&&e.fillText(t.text,t.x||0,t.y||0),E(e,t,!0),g(e,t,n)}function g(e,t,n){if(t.textDecoration&&"none"!==t.textDecoration){var i=t.x,o=e.measureText(t.text).width,a=parseInt(t.fontSize,10),r=a*ue<1?1:a*ue,s=t.y;switch(t.textDecoration){case"line-through":s-=a/3+r/2;break;case"overline":s-=a-r;break;case"underline":s+=r}e.rect(i,s,o,r),P(e,t,n),E(e,t)}}function _(e){e.strokeOpacity=void 0!==e["stroke-opacity"]?e["stroke-opacity"]:1,e.fillOpacity=void 0!==e["fill-opacity"]?e["fill-opacity"]:1,void 0!==e.opacity&&(e.strokeOpacity*=e.opacity,e.fillOpacity*=e.opacity)}function m(e){for(var t=e.childNodes,n=0;n<t.length;n++)if("tspan"===t[n].tagName)return!0;return!1}function v(e,t,n,i){for(var o,a=[],r=0,l=0;l<e.length;l++){var u=e[l];if(void 0===u.tagName)y(u,t,n,i);else if("tspan"===u.tagName||"text"===u.tagName){var c=s(u,i.rootAppended),d=(0,q.extend)({},n,c);if("tspan"===u.tagName&&m(u)){v(u.childNodes,t,d,i);continue}d.textAlign="start",o&&void 0===c.x||(o={elements:[],options:[],widths:[],offsets:[]},a.push(o)),void 0!==c.y&&(r=0),void 0!==c.dy&&(r+=parseFloat(c.dy)),o.elements.push(u),o.options.push(d),o.offsets.push(r),f(t,d),o.widths.push(t.measureText(d.text).width)}}a.forEach(function(e){var o=e.widths.reduce(function(e,t){return e+t},0),a=0,r=0;"center"===n.textAlign&&(a=o/2),"end"===n.textAlign&&(a=o),e.options.forEach(function(t,n){var i=e.widths[n];t.x=t.x-a+r,t.y+=e.offsets[n],r+=i}),e.elements.forEach(function(n,o){v(n.childNodes,t,e.options[o],i)})})}function y(e,t,n,i){var o=e.tagName,a="text"===o||"tspan"===o||void 0===o,r="image"===o,d=(0,q.extend)({},n,s(e,i.rootAppended));if("hidden"!==d.visibility&&!d["hidden-for-export"]){t.save(),!r&&w(t,d),C(t,d,i),_(d);var h;switch(t.beginPath(),e.tagName){case void 0:p(t,d,i);break;case"text":case"tspan":v(e.childNodes,t,d,i);break;case"image":h=u(t,d,i);break;case"path":c(t,d.d);break;case"rect":l(t,d),t.closePath();break;case"circle":t.arc(d.cx,d.cy,d.r,0,2*J,1)}return a||(x(t,d,i),P(t,d,i),E(t,d)),b(t,d,i,e),t.restore(),h}}function b(e,t,n,i){var o=n.gradients;if(0!==o.length){var a=h(t.fill);if(a&&o[a]){var r=i.getBBox(),s=e.createLinearGradient(r.x,0,r.x+r.width,0);o[a].forEach(function(e){var t=parseInt(e.offset.replace(/%/,""));s.addColorStop(t/100,e.stopColor)}),e.globalAlpha=t.opacity,e.fillStyle=s,e.fill()}}}function x(e,t,n){var i,o=h(t.filter);o&&(i=n.filters[o],i||(i={offsetX:0,offsetY:0,blur:0,color:"#000"}),e.shadowOffsetX=i.offsetX,e.shadowOffsetY=i.offsetY,e.shadowColor=i.color,e.shadowBlur=i.blur)}function w(e,t){e.translate(t.translateX||0,t.translateY||0),t.translateX=void 0,t.translateY=void 0,t.rotationAngle&&(e.translate(t.rotationX||0,t.rotationY||0),e.rotate(t.rotationAngle*J/180),e.translate(-(t.rotationX||0),-(t.rotationY||0)),t.rotationAngle=void 0,t.rotationX=void 0,t.rotationY=void 0),isFinite(t.scaleX)&&(e.scale(t.scaleX,t.scaleY),t.scaleX=void 0,t.scaleY=void 0)}function C(e,t,n){t["clip-path"]&&(y(n.clipPaths[h(t["clip-path"])],e,{},n),e.clip(),t["clip-path"]=void 0)}function k(e,t){var n=new N.default(e);return"rgba("+n.r+","+n.g+","+n.b+","+t+")"}function S(e){var t=[];return(0,G.each)(e.childNodes,function(e,n){var i=n.attributes;t.push({offset:i.offset.value,stopColor:i["stop-color"].value})}),t}function D(e){
var t,n,i={};return(0,G.each)(e.childNodes,function(e,o){var a=o.attributes;if(a.result)switch(a.result.value){case"gaussianBlurResult":i.blur=se(a.stdDeviation.value);break;case"offsetResult":i.offsetX=se(a.dx.value),i.offsetY=se(a.dy.value);break;case"floodResult":t=a["flood-color"]?a["flood-color"].value:"#000",n=a["flood-opacity"]?a["flood-opacity"].value:1,i.color=k(t,n)}}),i}function I(e,t){for(var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:new Z.Deferred,i=0;i<e.length;i++){var o=t(e[i]);if((0,j.isPromise)(o)){o.then(function(){I(Array.prototype.slice.call(e,i+1),t,n)});break}}return i===e.length&&n.resolve(),n}function T(e,t,n,i){return I(e,function(e){switch(e.tagName&&e.tagName.toLowerCase()){case"g":case"svg":var o=(0,q.extend)({},n,s(e,i.rootAppended));t.save(),w(t,o),C(t,o,i);var a=function(){t.restore()},r=T(e.childNodes,t,o,i);return(0,j.isPromise)(r)?r.then(a):a(),r;case"defs":return T(e.childNodes,t,{},i);case"clippath":i.clipPaths[e.attributes.id.textContent]=e.childNodes[0];break;case"pattern":i.patterns[e.attributes.id.textContent]=e;break;case"filter":i.filters[e.id]=D(e);break;case"lineargradient":i.gradients[e.attributes.id.textContent]=S(e);break;default:return y(e,t,n,i)}})}function O(e,t){var n=t["stroke-dasharray"]&&t["stroke-dasharray"].match(/(\d+)/g);n&&n.length&&(n=(0,G.map)(n,function(e){return se(e)}),e.setLineDash(n))}function E(e,t,n){var i=t.stroke;i&&"none"!==i&&0!==t["stroke-width"]&&(O(e,t),e.lineJoin=t["stroke-linejoin"],e.lineWidth=t["stroke-width"],e.globalAlpha=t.strokeOpacity,e.strokeStyle=i,n?e.strokeText(t.text,t.x,t.y):e.stroke(),e.globalAlpha=1)}function A(e,t,n){var i=s(t,n.rootAppended),a=o(i.width,i.height,0),r=a.getContext("2d");return T(t.childNodes,r,i,n),e.createPattern(a,"repeat")}function P(e,t,n){var i=t.fill;if(i&&"none"!==i){if(i.search(/url/)===-1)e.fillStyle=i;else{var o=n.patterns[h(i)];if(!o)return;e.fillStyle=A(e,o,n)}e.globalAlpha=t.fillOpacity,e.fill(),e.globalAlpha=1}}function R(e,t,n,i,o){e.fillStyle=i||"#ffffff",e.fillRect(-o,-o,t+2*o,n+2*o)}function M(){var e=$.default.createElement("div");return e.style.left="-9999px",e.style.position="absolute",e}function B(e,t,n){return T(e.childNodes,t.getContext("2d"),{},{clipPaths:{},patterns:{},filters:{},gradients:{},rootAppended:n})}function V(e,t,n,i,a){var r,s=arguments.length>5&&void 0!==arguments[5]?arguments[5]:B,l=o(t,n,a),u=l.getContext("2d"),c=(0,W.getSvgElement)(e),d=$.default.isElementNode(e);return u.translate(a,a),$.default.getBody().appendChild(l),d||(r=M(),r.appendChild(c),$.default.getBody().appendChild(r)),c.attributes.direction&&(l.dir=c.attributes.direction.textContent),R(u,t,n,i,a),(0,Z.fromPromise)(s(c,l,d&&(0,K.contains)($.default.getBody(),e))).then(function(){return l}).always(function(){r&&$.default.getBody().removeChild(r),$.default.getBody().removeChild(l)})}function F(e,t){return fe.getData(e,t)}function L(e){var t=o(100,100,0);return e.reduce(function(e,n){var i=("image/"+n).toLowerCase();return t.toDataURL(i).indexOf(i)!==-1?e.supported.push(n):e.unsupported.push(n),e},{supported:[],unsupported:[]})}t.getData=F,t.testFormats=L,t.imageCreator=void 0;var H,z=i(n(2)),N=i(n(98)),j=n(1),W=n(206),G=n(3),q=n(0),$=i(n(11)),K=n(33),U=n(7),Y=n(40),Z=n(6),X=(0,U.getWindow)(),Q=Math,J=Q.PI,ee=Q.min,te=Q.abs,ne=Q.sqrt,ie=Q.pow,oe=Q.atan2,ae=Q.cos,re=Q.sin,se=Number,le=1,ue=.05,ce="10px",de="sans-serif",he="#000";H=function(e){var t,n={};return(0,G.each)(e,function(e,i){t=i.textContent,isFinite(t)&&(t=se(t)),n[i.name.toLowerCase()]=t}),n};var fe={getImageData:function(e,t){var n="image/"+t.format,i=t.width,o=t.height,r=t.backgroundColor;return(0,j.isFunction)(t.__parseAttributesFn)&&(H=t.__parseAttributesFn),V(e,i,o,r,t.margin,t.svgToCanvas).then(function(e){return a(e,n)})},getData:function(e,t){var n=this;return fe.getImageData(e,t).then(function(e){var i="image/"+t.format,o=(0,j.isFunction)(X.Blob)&&!t.forceProxy?n._getBlob(e,i):n._getBase64(e);return o})},_getBlob:function(e,t){var n,i=new Uint8Array(e.length);for(n=0;n<e.length;n++)i[n]=e.charCodeAt(n);return new X.Blob([i.buffer],{type:t})},_getBase64:function(e){return X.btoa(e)}};t.imageCreator=fe},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.Export=void 0;var o=n(1),a=i(n(10)),r=n(353),s=n(0),l=n(7),u=7,c=255,d={getFullOptions:function(e){var t=(0,s.extend)({},e);if(!(0,o.isDefined)(t.worksheet)||!(0,o.isObject)(t.worksheet))throw Error('The "worksheet" field must contain an object.');if((0,o.isDefined)(t.topLeftCell)){if((0,o.isString)(t.topLeftCell)){var n=t.worksheet.getCell(t.topLeftCell),i=n.row,r=n.col;t.topLeftCell={row:i,column:r}}}else t.topLeftCell={row:1,column:1};return(0,o.isDefined)(t.keepColumnWidths)||(t.keepColumnWidths=!0),(0,o.isDefined)(t.loadPanel)||(t.loadPanel={}),(0,o.isDefined)(t.loadPanel.enabled)||(t.loadPanel.enabled=!0),(0,o.isDefined)(t.loadPanel.text)||(t.loadPanel.text=a.default.format("dxDataGrid-exporting")),t},convertDateForExcelJS:function(e){return new Date(Date.UTC(e.getFullYear(),e.getMonth(),e.getDate(),e.getHours(),e.getMinutes(),e.getSeconds(),e.getMilliseconds()))},setNumberFormat:function(e,t){e.numFmt=t},tryConvertToExcelNumberFormat:function(e,t){var n=r.ExportFormat.formatObjectConverter(e,t),i=n.currency;return e=n.format,t=n.dataType,r.ExportFormat.convertFormat(e,n.precision,t,i)},setAlignment:function(e,t,n){e.alignment=e.alignment||{},(0,o.isDefined)(t)&&(e.alignment.wrapText=t),(0,o.isDefined)(n)&&(e.alignment.horizontal=n),e.alignment.vertical="top"},setColumnsWidth:function(e,t,n){if((0,o.isDefined)(t))for(var i=0;i<t.length;i++){var a=t[i];"number"==typeof a&&isFinite(a)&&(e.getColumn(n+i).width=Math.min(c,Math.floor(a/u*100)/100))}},tryGetMergeRange:function(e,t,n,i){if(!n[e]||!n[e][t]){var o=i.getCellMerging(e,t);if(o.colspan||o.rowspan){for(var a=e;a<=e+o.rowspan;a++)for(var r=t;r<=t+o.colspan;r++)n[a]||(n[a]=[]),n[a][r]=!0;return{start:{row:e,column:t},end:{row:e+(o.rowspan||0),column:t+(o.colspan||0)}}}}},mergeCells:function(e,t,n){n.forEach(function(n){e.mergeCells(n.start.row+t.row,n.start.column+t.column,n.end.row+t.row,n.end.column+t.column)})},setLoadPanelOptions:function(e,t,n){(0,l.hasWindow)()&&(e._setOptionWithoutOptionChange("loadPanel",t),n._renderLoadPanel(e))},"export":function(e,t){var n=this,i=e.customizeCell,a=e.component,r=e.worksheet,l=e.topLeftCell,u=e.autoFilterEnabled,c=e.keepColumnWidths,d=e.selectedRowsOnly,h=e.loadPanel,f=(0,s.extend)({},a.option("loadPanel"));"animation"in a.option("loadPanel")&&(h.animation=null),this.setLoadPanelOptions(a,h,t);var p=!!a.option("wordWrapEnabled");r.properties.outlineProperties={summaryBelow:!1,summaryRight:!1};var g={from:{row:l.row,column:l.column},to:{row:l.row,column:l.column}},_=a.getDataProvider(d);return new Promise(function(e){_.ready().done(function(){var d=_.getColumns(),h=(0,o.isFunction)(_.getHeaderRowCount)?_.getHeaderRowCount():1,f=_.getRowsCount();c&&n.setColumnsWidth(r,_.getColumnsWidths(),g.from.column);for(var m=[],v=[],y=0;y<f;y++){var b=r.getRow(g.from.row+y);n.exportRow(y,d.length,b,g.from.column,_,i,h,m,v,p,t),y>=1&&g.to.row++}n.mergeCells(r,l,v),g.to.column+=d.length>0?d.length-1:0;var x=r.views[0]||{};a.option("rtlEnabled")&&(x.rightToLeft=!0),h>0&&(Object.keys(x).indexOf("state")===-1&&(0,s.extend)(x,t._getWorksheetFrozenState(_,g)),t._trySetAutoFilter(_,r,g,h,u)),Object.keys(x).length>0&&(r.views=[x]),e(g)}).always(function(){n.setLoadPanelOptions(a,f,t)})})},exportRow:function(e,t,n,i,a,r,s,l,u,c,d){var h=a.getStyles();d._trySetOutlineLevel(a,n,e,s);for(var f=0;f<t;f++){var p=a.getCellData(e,f,!0),g=p.cellSourceData,_=n.getCell(i+f);if((0,o.isDate)(p.value)?_.value=this.convertDateForExcelJS(p.value):_.value=p.value,(0,o.isDefined)(_.value)){var m=h[a.getStyleId(e,f)],v=m.bold,y=m.alignment,b=m.format,x=m.dataType,w=this.tryConvertToExcelNumberFormat(b,x);(0,o.isDefined)(w)?w=w.replace(/&quot;/g,'"'):(0,o.isString)(_.value)&&/^[@=+-]/.test(_.value)&&(w="@"),this.setNumberFormat(_,w),d._trySetFont(_,v),this.setAlignment(_,c,y)}if((0,o.isFunction)(r)&&r(d._getCustomizeCellOptions(_,g)),d._needMergeRange(e,s)){var C=this.tryGetMergeRange(e,f,l,a);(0,o.isDefined)(C)&&u.push(C)}}}};t.Export=d},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.equals=void 0;var a=i(n(11)),r=n(19),s=n(1),l=function(e,t){return 1/e===1/t},u=function(e,t){if(e=(0,r.toComparable)(e,!0),t=(0,r.toComparable)(t,!0),e&&t&&(0,s.isRenderer)(e)&&(0,s.isRenderer)(t))return t.is(e);var n=e!==e,i=t!==t;return!(!n||!i)||(0===e&&0===t?l(e,t):!(null!==e&&"object"===o(e)&&!a.default.isElementNode(e))&&e===t)};t.equals=u},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.acquireTemplate=t.acquireIntegrationTemplate=t.defaultCreateElement=t.templateKey=t.validateTemplateSource=t.getNormalizedTemplateArgs=t.addOneRenderedCall=t.suitableTemplatesByName=t.findTemplates=void 0;var o=i(n(28)),a=i(n(13)),r=i(n(22)),s=i(n(2)),l=n(154),u=n(93),c=n(361),d=n(67),h=n(12),f=n(4),p=n(33),g=n(0),_=n(1),m=function(e,t){var n="data-options",i=(0,s.default)(e).contents().filter("[".concat(n,'*="').concat(t,'"]'));return[].slice.call(i).map(function(e){var i=(0,s.default)(e).attr(n)||"";return{element:e,options:(0,o.default)().optionsParser(i)[t]}}).filter(function(e){return!!e.options})};t.findTemplates=m;var v=function(e){var t=(0,h.groupBy)(e,function(e){return e.options.name});if(t[void 0])throw r.default.Error("E0023");var n={};return Object.keys(t).forEach(function(e){var i,o=null===(i=(0,f.findBestMatches)(a.default.current(),t[e],function(e){return e.options})[0])||void 0===i?void 0:i.element;o&&(n[e]=o)}),n};t.suitableTemplatesByName=v;var y=function(e){var t=e.render.bind(e);return(0,g.extend)({},e,{render:function(e){var n=t(e);return e&&e.onRendered&&e.onRendered(),n}})};t.addOneRenderedCall=y;var b=function(e){var t=[];return"model"in e&&t.push(e.model),"index"in e&&t.push(e.index),t.push(e.container),t};t.getNormalizedTemplateArgs=b;var x=function(e){return"string"==typeof e?(0,p.normalizeTemplateElement)(e):e};t.validateTemplateSource=x;var w=function(e){return(0,_.isRenderer)(e)&&e[0]||e};t.templateKey=w;var C=function(e){return new c.Template(e)};t.defaultCreateElement=C;var k=function(e,t,n,i){var o=null;return i&&i.indexOf(e)!==-1||(o=t[e],!o||o instanceof d.TemplateBase||n||(o=y(o))),o};t.acquireIntegrationTemplate=k;var S=function(e,t,n,i,o,a){return null==e?new u.EmptyTemplate:e instanceof l.ChildDefaultTemplate?a[e.name]:e instanceof d.TemplateBase?e:(0,_.isFunction)(e.render)&&!(0,_.isRenderer)(e)?i?e:y(e):e.nodeType||(0,_.isRenderer)(e)?t((0,s.default)(e)):k(e,n,i,o)||a[e]||t(e)};t.acquireTemplate=S},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.queryImpl=void 0;var o=i(n(156)),a=i(n(367)),r={array:o.default,remote:a.default};t.queryImpl=r},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){function t(){for(;o.length;){s=!0;var e=o.shift(),n=e();if(void 0!==n){if(n.then)return void(0,r.when)(n).always(t);throw a.default.Error("E0015")}}s=!1}function n(n,i){e?(o[0]&&i&&i(o[0]),o=[n]):o.push(n),s||t()}function i(){return s}var o=[],s=!1;return{add:n,busy:i}}t.create=o,t.enqueue=void 0;var a=i(n(22)),r=n(6),s=o().add;t.enqueue=s},function(e,t,n){function i(e){u=e}function o(e){return u[e]}t.setEasing=i,t.getEasing=o,t.convertTransitionTimingFuncToEasing=void 0;var a=n(1),r=/cubic-bezier\((\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\)/,s={linear:"cubic-bezier(0, 0, 1, 1)",swing:"cubic-bezier(0.445, 0.05, 0.55, 0.95)",ease:"cubic-bezier(0.25, 0.1, 0.25, 1)","ease-in":"cubic-bezier(0.42, 0, 1, 1)","ease-out":"cubic-bezier(0, 0, 0.58, 1)","ease-in-out":"cubic-bezier(0.42, 0, 0.58, 1)"},l=function(e,t,n,i){var o=3*e,a=3*(n-e)-o,r=1-o-a,s=3*t,l=3*(i-t)-s,u=1-s-l,c=function(e){return e*(o+e*(a+e*r))},d=function(e){return e*(s+e*(l+e*u))},h=function(e){return o+e*(2*a+3*e*r)},f=function(e){for(var t,n=e,i=0;i<14&&(t=c(n)-e,!(Math.abs(t)<.001));)n-=t/h(n),i++;return n};return function(e){return d(f(e))}},u={},c=function(e){e=s[e]||e;var t,n=e.match(r);n||(t="linear",n=s[t].match(r)),n=n.slice(1,5);for(var i=0;i<n.length;i++)n[i]=parseFloat(n[i]);var o=t||"cubicbezier_"+n.join("_").replace(/\./g,"p");return(0,a.isFunction)(u[o])||(u[o]=function(e,t,i,o,a){return o*l(n[0],n[1],n[2],n[3])(t/a)+i}),o};t.convertTransitionTimingFuncToEasing=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(13)),a=n(0),r=n(3),s=i(n(157)),l={dxpointerdown:"touchstart",dxpointermove:"touchmove",dxpointerup:"touchend",dxpointercancel:"touchcancel",dxpointerover:"",dxpointerout:"",dxpointerenter:"",dxpointerleave:""},u=function(e){var t=[];return(0,r.each)(e.touches,function(e,n){t.push((0,a.extend)({pointerId:n.identifier},n))}),{pointers:t,pointerId:e.changedTouches[0].identifier}},c=function(e){return"ios"===o.default.real().platform&&("dxpointerdown"===e||"dxpointerup"===e)},d=s.default.inherit({ctor:function(){this.callBase.apply(this,arguments),this._pointerId=0},_handler:function(e){if(c(this._eventName)){var t=e.changedTouches[0];if(this._pointerId===t.identifier&&0!==this._pointerId)return;this._pointerId=t.identifier}return this.callBase.apply(this,arguments)},_fireEvent:function(e){return this.callBase((0,a.extend)(u(e.originalEvent),e))}});d.map=l,d.normalize=u;var h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=i(n(49)),r=i(n(11)),s=function(e,t){a.default.add(function(){e.split(" ").forEach(function(e){r.default.listen(r.default.getDocument(),e,t,!0)})})},l=function(e,t,n){n=n||function(){};var i=[],a=function(e){var n=-1;return(0,o.each)(i,function(i,o){return!t(e,o)||(n=i,!1)}),n},r=function(e){a(e)===-1&&(n(e),i.push(e))},l=function(e){var t=a(e);t>-1&&i.splice(t,1)},u=function(e){i[a(e)]=e};s(e.dxpointerdown,r),s(e.dxpointermove,u),s(e.dxpointerup,l),s(e.dxpointercancel,l),this.pointers=function(){return i},this.reset=function(){i=[]}},u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=n(0),r=i(n(157)),s=i(n(255)),l={dxpointerdown:"mousedown",dxpointermove:"mousemove",dxpointerup:"mouseup",dxpointercancel:"",dxpointerover:"mouseover",dxpointerout:"mouseout",dxpointerenter:"mouseenter",dxpointerleave:"mouseleave"},u=function(e){return e.pointerId=1,{pointers:o.pointers(),pointerId:1}},c=!1,d=function(){c||(o=new s.default(l,function(){return!0}),c=!0)},h=r.default.inherit({ctor:function(){this.callBase.apply(this,arguments),d()},_fireEvent:function(e){return this.callBase((0,a.extend)(u(e.originalEvent),e))}});h.map=l,h.normalize=u,h.activate=d,h.resetObserver=function(){o.reset()};var f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=n(60),r=i(n(135)),s=i(n(85));n(376),n(377),n(378),n(379),n(380),n(381),n(382),n(383),n(384),n(385);var l=(0,s.default)();if(l&&(0,a.compare)(o.default.fn.jquery,[1,10])<0)throw r.default.Error("E0012")},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.themeReadyCallback=void 0;var o=i(n(27)),a=new o.default;t.themeReadyCallback=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(22)),a=function(){var e={},t=function(t){return e[t]||0};return{obtain:function(n){e[n]=t(n)+1},release:function(n){var i=t(n);if(i<1)throw o.default.Error("E0014");1===i?delete e[n]:e[n]=i-1},locked:function(e){return t(e)>0}}},r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(47)),a=n(4),r=n(1),s=i(n(15)),l=n(6),u=s.default.inherit({ctor:function(e){this.options=e,this._setOption("disabledItemKeys",[]),this._clearItemKeys()},_clearItemKeys:function(){this._setOption("addedItemKeys",[]),this._setOption("removedItemKeys",[]),this._setOption("removedItems",[]),this._setOption("addedItems",[])},validate:a.noop,_setOption:function(e,t){this.options[e]=t},onSelectionChanged:function e(){var t=this.options.addedItemKeys,n=this.options.removedItemKeys,i=this.options.addedItems,o=this.options.removedItems,r=this.options.selectedItems,s=this.options.selectedItemKeys,e=this.options.onSelectionChanged||a.noop;this._clearItemKeys(),e({selectedItems:r,selectedItemKeys:s,addedItemKeys:t,removedItemKeys:n,addedItems:i,removedItems:o})},equalKeys:function(e,t){return this.options.equalByReference&&(0,r.isObject)(e)&&(0,r.isObject)(t)?e===t:(0,a.equalByValue)(e,t)},getSelectableItems:function(e){return e.filter(function(e){return!e.disabled})},_clearSelection:function(e,t,n,i){return e=e||[],e=Array.isArray(e)?e:[e],this.validate(),this.selectedItemKeys(e,t,n,i)},_loadFilteredData:function(e,t,n,i){var a=encodeURI(JSON.stringify(e)).length,s=this.options.maxFilterLengthInRequest&&a>this.options.maxFilterLengthInRequest,u=new l.Deferred,c={filter:s?void 0:e,select:s?this.options.dataFields():n||this.options.dataFields()};return e&&0===e.length?u.resolve([]):this.options.load(c).done(function(n){var a=(0,r.isPlainObject)(n)?n.data:n;t&&!i?a=a.filter(t):s&&(a=(0,o.default)(a).filter(e).toArray()),u.resolve(a)}).fail(u.reject.bind(u)),u},updateSelectedItemKeyHash:function(e){for(var t=0;t<e.length;t++){var n=(0,a.getKeyHash)(e[t]);if(!(0,r.isObject)(n)){this.options.keyHashIndices[n]=this.options.keyHashIndices[n]||[];var i=this.options.keyHashIndices[n];i.push(t)}}},_isAnyItemSelected:function(e){for(var t=0;t<e.length;t++)if(this.options.isItemSelected(e[t]))return;return!1},_getFullSelectAllState:function(){var e=this.options.plainItems(),t=this.options.filter(),n=this.options.selectedItems;t&&(n=(0,o.default)(n).filter(t).toArray());var i=n.length;return i?i>=this.options.totalCount()-this.options.disabledItemKeys.length||void 0:this._isAnyItemSelected(e)},_getVisibleSelectAllState:function(){for(var e=this.getSelectableItems(this.options.plainItems()),t=!1,n=!1,i=0;i<e.length;i++){var o=e[i],a=this.options.getItemData(o),r=this.options.keyOf(a);this.options.isSelectableItem(o)&&(this.isItemKeySelected(r)?t=!0:n=!0)}return!!t&&(!n||void 0)}});t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getClosestNodeWithContext=void 0;var o=i(n(61)),a=function e(t){var n=o.default.contextFor(t);return!n&&t.parentNode?e(t.parentNode):t};t.getClosestNodeWithContext=a},function(e,t,n){var i=n(180),o=i.data=n(263);o.odata=n(413),e.exports=o},function(e,t,n){var i=n(137);e.exports=i.data=i.data||{},Object.defineProperty(i.data,"errorHandler",{get:function(){return n(34).errorHandler},set:function(e){n(34).errorHandler=e}}),Object.defineProperty(i.data,"_errorHandler",{get:function(){return n(34)._errorHandler},set:function(e){n(34)._errorHandler=e}}),i.data.DataSource=n(276),i.data.query=n(47),i.data.Store=n(102),i.data.ArrayStore=n(81),i.data.CustomStore=n(123),i.data.LocalStore=n(410),i.data.base64_encode=n(41).base64_encode,i.data.applyChanges=n(411),i.data.Guid=n(35),i.data.utils={},i.data.utils.compileGetter=n(19).compileGetter,i.data.utils.compileSetter=n(19).compileSetter,i.EndpointSelector=n(412),i.data.queryImpl=n(251).queryImpl,i.data.queryAdapters=n(186);var o=n(41);i.data.utils.normalizeBinaryCriterion=o.normalizeBinaryCriterion,i.data.utils.normalizeSortingInfo=o.normalizeSortingInfo,i.data.utils.errorMessageFromXhr=o.errorMessageFromXhr,i.data.utils.aggregators=o.aggregators,i.data.utils.keysEqual=o.keysEqual,i.data.utils.isDisjunctiveOperator=o.isDisjunctiveOperator,i.data.utils.isConjunctiveOperator=o.isConjunctiveOperator,i.data.utils.processRequestResultLock=o.processRequestResultLock,i.data.utils.toComparable=n(19).toComparable,i.data.utils.multiLevelGroup=n(91).multiLevelGroup,i.data.utils.arrangeSortingInfo=n(91).arrangeSortingInfo,i.data.utils.normalizeDataSourceOptions=n(70).normalizeDataSourceOptions},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=i(n(11)),r=n(7),s=i(n(64)),l=(0,r.getWindow)(),u="dxproxy.devexpress.com:8000",c={},d=(0,s.default)(function(){var e=a.default.createElement("a"),t=["protocol","hostname","port","pathname","search","hash"],n=function(e){return"/"!==e.charAt(0)&&(e="/"+e),e};return function(i){e.href=i;var a={};return(0,o.each)(t,function(){a[this]=e[this]}),a.pathname=n(a.pathname),a}}),h=function(e){var t=d();return t(e)},f=function(){return l.location.pathname.split("/")[1]},p={parseUrl:h,isProxyUsed:function(){return l.location.host===u},formatProxyUrl:function(e){var t=h(e);if(!/^(localhost$|127\.)/i.test(t.hostname))return e;var n=u+"/"+f()+"_"+t.port;c[n]=t.hostname+":"+t.port;var i="http://"+n+t.pathname+t.search;return i},formatLocalUrl:function(e){if(e.indexOf(u)<0)return e;var t=e;for(var n in c)if(Object.prototype.hasOwnProperty.call(c,n)&&e.indexOf(n)>=0){t=e.replace(n,c[n]);break}return t}};t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var a=n(1),r=i(n(28)),s=n(95),l=i(n(264)),u=i(n(34)),c=i(n(47)),d=i(n(102)),h=i(n(266)),f=n(6);n(159);var p="5d46402c-7899-4ea9-bd81-8b73c47c7683",g=function(e,t){return o({},e,t)},_=function(e,t){var n={};for(var i in e)n[i]=e[i];for(var o in t)o in n?n[o]!==t[o]&&u.default.errors.log("W4001",o):n[o]=t[o];return n},m=d.default.inherit({ctor:function(e){this.callBase(e),this._requestDispatcher=new h.default(e);var t=this.key(),n=e.fieldTypes,i=e.keyType;if(i){var o="string"==typeof i;t||(t=o?p:Object.keys(i),this._legacyAnonymousKey=t),o&&(i=g(t,i)),n=_(n,i)}this._fieldTypes=n||{},2===this.version()?this._updateMethod="MERGE":this._updateMethod="PATCH"},_customLoadOptions:function(){return["expand","customQueryParams"]},_byKeyImpl:function(e,t){var n={};return t&&(n.$expand=(0,s.generateExpand)(this.version(),t.expand,t.select)||void 0,n.$select=(0,s.generateSelect)(this.version(),t.select)||void 0),this._requestDispatcher.sendRequest(this._byKeyUrl(e),"GET",n)},createQuery:function(e){var t,n,i={adapter:"odata",beforeSend:this._requestDispatcher.beforeSend,errorHandler:this._errorHandler,jsonp:this._requestDispatcher.jsonp,version:this._requestDispatcher.version,withCredentials:this._requestDispatcher._withCredentials,expand:null===e||void 0===e?void 0:e.expand,requireTotalCount:null===e||void 0===e?void 0:e.requireTotalCount,deserializeDates:this._requestDispatcher._deserializeDates,fieldTypes:this._fieldTypes};if(n=null!==(t=null===e||void 0===e?void 0:e.urlOverride)&&void 0!==t?t:this._requestDispatcher.url,(0,a.isDefined)(this._requestDispatcher.filterToLower)&&(i.filterToLower=this._requestDispatcher.filterToLower),null!==e&&void 0!==e&&e.customQueryParams){var o=(0,s.escapeServiceOperationParams)(null===e||void 0===e?void 0:e.customQueryParams,this.version());4===this.version()?n=(0,s.formatFunctionInvocationUrl)(n,o):i.params=o}return(0,c.default)(n,i)},_insertImpl:function(e){var t=this;this._requireKey();var n=new f.Deferred;return(0,f.when)(this._requestDispatcher.sendRequest(this._requestDispatcher.url,"POST",null,e)).done(function(i){return n.resolve(i&&!(0,r.default)().useLegacyStoreResult?i:e,t.keyOf(i))}).fail(n.reject),n.promise()},_updateImpl:function(e,t){var n=new f.Deferred;return(0,f.when)(this._requestDispatcher.sendRequest(this._byKeyUrl(e),this._updateMethod,null,t)).done(function(i){return(0,r.default)().useLegacyStoreResult?n.resolve(e,t):n.resolve(i||t,e)}).fail(n.reject),n.promise()},_removeImpl:function(e){var t=new f.Deferred;return(0,f.when)(this._requestDispatcher.sendRequest(this._byKeyUrl(e),"DELETE")).done(function(){return t.resolve(e)}).fail(t.reject),t.promise()},_convertKey:function(e){var t=e,n=this._fieldTypes,i=this.key()||this._legacyAnonymousKey;if(Array.isArray(i)){t={};for(var o=0;o<i.length;o++){var a=i[o];t[a]=(0,s.convertPrimitiveValue)(n[a],e[a])}}else n[i]&&(t=(0,s.convertPrimitiveValue)(n[i],e));return t},_byKeyUrl:function(e,t){var n=t?l.default.formatLocalUrl(this._requestDispatcher.url):this._requestDispatcher.url,i=this._convertKey(e);return"".concat(n,"(").concat(encodeURIComponent((0,s.serializeKey)(i,this.version())),")")},version:function(){return this._requestDispatcher.version}},"odata"),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=n(95);n(159);var s=2,l=function(){function e(t){i(this,e),t=t||{},this._url=String(t.url).replace(/\/+$/,""),this._beforeSend=t.beforeSend,this._jsonp=t.jsonp,this._version=t.version||s,this._withCredentials=t.withCredentials,this._deserializeDates=t.deserializeDates,this._filterToLower=t.filterToLower}return a(e,[{key:"sendRequest",value:function(e,t,n,i){return(0,r.sendRequest)(this.version,{url:e,method:t,params:n||{},payload:i},{beforeSend:this._beforeSend,jsonp:this._jsonp,withCredentials:this._withCredentials,deserializeDates:this._deserializeDates})}},{key:"version",get:function(){return this._version}},{key:"beforeSend",get:function(){return this._beforeSend}},{key:"url",get:function(){return this._url}},{key:"jsonp",get:function(){return this._jsonp}},{key:"filterToLower",get:function(){return this._filterToLower}}]),e}();t.default=l,e.exports=t.default},function(e,t){e.exports=window.JSZip},function(e,t,n){function i(e){return e.split(",").slice(1).map(function(e){return e.split("").filter(function(e){return"#"===e||"0"===e}).length})}function o(e){var t=e.split(";");return 1===t.length&&t.push("-"+t[0]),t}function a(e){return e.toString().split("").reverse().join("")}function r(e){return e.indexOf("%")!==-1&&!e.match(/'[^']*%[^']*'/g)}function s(e){return e.replace(/'.+'/g,"")}function l(e){if(!e)return 0;var t=s(e);return t.length-t.replace(/[#]/g,"").length}function u(e){if(!e)return 0;var t=s(e);return t.length-t.replace(/[0]/g,"").length}function c(e,t,n){if(!e)return"";for(e.length>n&&(e=e.substr(0,n));e.length>t&&"0"===e.slice(-1);)e=e.substr(0,e.length-1);for(;e.length<t;)e+="0";return e}function d(e,t,n){if(!t.length)return e;for(var i=[],o=0;e;){var a=t[o];if(!a)break;i.push(e.slice(0,a)),e=e.slice(a),o<t.length-1&&o++}return i.join(n)}function h(e,t){return e.split(w).map(function(e,n){var i=n%2;return!e&&i?w:i?e:e.replace(/[,#0]+/,t)}).join("")}function f(e){for(var t=!1,n=0;n<e.length;n++)if("'"===e[n]&&(t=!t),"."===e[n]&&!t)return n;return e.length}function p(e,t){return t=t||x,function(n){if("number"!=typeof n||isNaN(n))return"";var s=o(e),p=1/n===1/0,g=n>0||p,_=s[g?0:1];r(_)&&(n=100*n),g||(n=-n);var m=f(_),v=[_.substr(0,m),_.substr(m+1)],x=u(v[1]),w=x+l(v[1]),k=u(v[0]),S=l(v[0])||t.unlimitedIntegerDigits?void 0:k,D=Math.floor(n).toString().length,I=(0,y.fitIntoRange)(w,0,C-D),T=i(v[0]).reverse(),O=(0,b.toFixed)(n,I<0?0:I).split("."),E=c(a(O[0]),k,S),A=c(O[1],x,w);E=d(E,T,t.thousandsSeparator);var P=a(h(a(v[0]),E)),R=w?h(v[1],A):"",M=P+(R.match(/\d/)?t.decimalSeparator:"")+R;return M}}function g(e,t,n){var i=(t?.01:1)*parseFloat(e)||0;return n?-i:i}function _(e,t,n,i){var o,a,r,s=e;do r&&(o=a.length===r.length?"0":"1",e=i?o+e:e+o),a=r||t(g(s,n)),s=i?"1"+s:s+"1",r=t(g(s,n));while(a!==r&&(i?a.length===r.length:a.length<=r.length));if(i&&r.length>a.length){var l=t(12345).indexOf("12345")===-1;do e="1"+e;while(l&&g(e,n)<1e5)}return e}function m(e,t,n,i){var o=t(g(e,n,i)),a=e.split("."),r=a[0]+".3"+a[1].slice(1),s=g(r,n,i),l=t(s).indexOf("3")-1;return o=o.replace(/(\d)\D(\d)/g,"$1,$2"),l>=0&&(o=o.slice(0,l)+"."+o.slice(l+1)),o=o.replace(/1+/,"1").replace(/1/g,"#"),n||(o=o.replace("%","'%'")),o}function v(e){var t=".",n=e(1).indexOf("100")>=0;t=_(t,e,n,!0),t=_(t,e,n,!1);var i=m(t,e,n,!1),o=m(t,e,n,!0);return o==="-"+i?i:i+";"+o}t.getFormatter=p,t.getFormat=v;var y=n(29),b=n(240),x={thousandsSeparator:",",decimalSeparator:"."},w="'",C=15},function(e,t,n){t.sessionStorage=void 0;var i=n(7),o=(0,i.getWindow)(),a=function(){var e;try{e=o.sessionStorage}catch(e){}return e};t.sessionStorage=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(47)),a=i(n(22)),r=i(n(368)),s=n(29),l={_timeZones:r.default.zones,getDisplayedTimeZones:function(e){var t=this,n=this._timeZones.map(function(n){var i=t.getUtcOffset(n.offsets,n.offsetIndices,n.untils,e),o="(GMT ".concat(t.formatOffset(i),") ").concat(t.formatId(n.id));return{offset:i,title:o,id:n.id}});return(0,o.default)(n).sortBy("offset").toArray()},formatOffset:function(e){var t=Math.floor(e),n=e-t,i=(0,s.sign)(e)>=0?"+":"-",o="0".concat(Math.abs(t)).slice(-2),a=n>0?":".concat(60*n):":00";return i+o+a},formatId:function(e){return e.split("/").join(" - ").split("_").join(" ")},getTimezoneById:function(e){var t,n=0,i=this._timeZones;if(e)for(;!t;){if(!i[n])return void a.default.log("W0009",e);var o=i[n].id;o===e&&(t=i[n]),n++}return t},getTimeZoneOffsetById:function(e,t){var n,i,o,a,r=this.getTimezoneById(e);return r&&(n=r.offsets,o=r.untils,i=r.offsetIndices,a=this.getUtcOffset(n,i,o,t)),a},getUtcOffset:function(e,t,n,i){for(var o=0,a=t.split(""),r=e.split("|"),s=n.split("|").map(function(e){return"Infinity"===e?null:1e3*parseInt(e,36)}),l=0,u=0,c=s.length;u<c;u++){l+=s[u];{if(!(i>=l))break;o=u}}s[o+1]&&o++;var d=Number(r[Number(a[o])]);return-d/60||d}},u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.TransitionExecutor=void 0;var o=i(n(2)),a=i(n(15)),r=n(0),s=n(4),l=n(1),u=n(3),c=i(n(43)),d=n(187),h=n(6),f={forward:" dx-forward",backward:" dx-backward",none:" dx-no-direction",undefined:" dx-no-direction"},p="dx-animating",g=a.default.inherit({ctor:function(){this._accumulatedDelays={enter:0,leave:0},this._animations=[],this.reset()},_createAnimations:function(e,t,n,i){var a=this,r=[];n=n||{};var s=this._prepareElementAnimationConfig(t,n,i);return s&&e.each(function(){var e=a._createAnimation((0,o.default)(this),s,n);e&&(e.element.addClass(p),e.setup(),r.push(e))}),r},_prepareElementAnimationConfig:function(e,t,n){var i;if("string"==typeof e){var o=e;e=d.presets.getPreset(o)}if(e)if((0,l.isFunction)(e[n]))i=e[n];else{if(i=(0,r.extend)({skipElementInitialStyles:!0,cleanupWhen:this._completePromise},e,t),!i.type||"css"===i.type){var a="dx-"+n,s=(i.extraCssClasses?" "+i.extraCssClasses:"")+f[i.direction];i.type="css",i.from=(i.from||a)+s,i.to=i.to||a+"-active"}i.staggerDelay=i.staggerDelay||0,i.delay=i.delay||0,i.staggerDelay&&(i.delay+=this._accumulatedDelays[n],this._accumulatedDelays[n]+=i.staggerDelay)}else i=void 0;return i},_createAnimation:function(e,t,n){var i;return(0,l.isPlainObject)(t)?i=c.default.createAnimation(e,t):(0,l.isFunction)(t)&&(i=t(e,n)),i},_startAnimations:function(){for(var e=this._animations,t=0;t<e.length;t++)e[t].start()},_stopAnimations:function(e){for(var t=this._animations,n=0;n<t.length;n++)t[n].stop(e)},_clearAnimations:function(){for(var e=this._animations,t=0;t<e.length;t++)e[t].element.removeClass(p);this._animations.length=0},reset:function(){this._accumulatedDelays.enter=0,this._accumulatedDelays.leave=0,this._clearAnimations(),this._completeDeferred=new h.Deferred,this._completePromise=this._completeDeferred.promise()},enter:function(e,t,n){var i=this._createAnimations(e,t,n,"enter");this._animations.push.apply(this._animations,i)},leave:function(e,t,n){var i=this._createAnimations(e,t,n,"leave");
this._animations.push.apply(this._animations,i)},start:function(){var e,t=this;if(this._animations.length){var n=(0,u.map)(this._animations,function(e){var t=new h.Deferred;return e.deferred.always(function(){t.resolve()}),t.promise()});e=h.when.apply(o.default,n).always(function(){t._completeDeferred.resolve(),t.reset()}),(0,s.executeAsync)(function(){t._startAnimations()})}else t.reset(),e=(new h.Deferred).resolve().promise();return e},stop:function(e){this._stopAnimations(e)}});t.TransitionExecutor=g},function(e,t,n){function i(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return i=function(){return e},e}function o(e){if(e&&e.__esModule)return e;if(null===e||"object"!==r(e)&&"function"!=typeof e)return{"default":e};var t=i();if(t&&t.has(e))return t.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var a in e)if(Object.prototype.hasOwnProperty.call(e,a)){var s=o?Object.getOwnPropertyDescriptor(e,a):null;s&&(s.get||s.set)?Object.defineProperty(n,a,s):n[a]=e[a]}return n.default=e,t&&t.set(e,n),n}function a(e){return e&&e.__esModule?e:{"default":e}}function r(e){"@babel/helpers - typeof";return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var s=a(n(43)),l=a(n(84)),u=n(30),c=a(n(9)),d=a(n(13)),h=a(n(11)),f=n(18),p=a(n(2)),g=n(93),_=n(12),m=a(n(23)),v=n(4),y=n(6),b=n(33),x=n(0),w=n(3),C=n(29),k=a(n(49)),S=n(1),D=n(60),I=n(78),T=n(7),O=a(n(5)),E=n(80),A=a(n(25)),P=n(119),R=n(8),M=n(55),B=n(209),V=a(n(188)),F=n(75),L=a(n(164)),H=a(n(16)),z=o(n(273)),N=k.default.add,j=(0,T.getWindow)(),W=(0,T.getNavigator)(),G=I.changeCallback,q="dx-overlay",$="dx-overlay-wrapper",K="dx-overlay-content",U="dx-overlay-shader",Y="dx-overlay-modal",Z="dx-inner-overlay",X="dx-state-invisible",Q="content",J="dx-rtl",ee=["onShowing","onShown","onHiding","onHidden","onPositioning","onPositioned","onResizeStart","onResize","onResizeEnd"],te=[],ne="dx-state-disabled",ie="dx-prevent-safari-scrolling",oe="tab",ae={top:{my:"top center",at:"top center"},bottom:{my:"bottom center",at:"bottom center"},right:{my:"right center",at:"right center"},left:{my:"left center",at:"left center"},center:{my:"center",at:"center"},"right bottom":{my:"right bottom",at:"right bottom"},"right top":{my:"right top",at:"right top"},"left bottom":{my:"left bottom",at:"left bottom"},"left top":{my:"left top",at:"left top"}},re=d.default.real(),se=re.version,le=m.default.mozilla&&"desktop"===re.deviceType,ue="ios"===re.platform,ce=m.default.safari&&"desktop"!==re.deviceType,de="android"===re.platform&&0===(0,D.compare)(se,[4,0],2)&&W.userAgent.indexOf("Chrome")===-1,he=function(e){if(le&&e.width(),de){var t=e.parents(),n=t.is(".dx-scrollable-native");n||(t.css("backfaceVisibility","hidden"),t.css("backfaceVisibility"),t.css("backfaceVisibility","visible"))}},fe=function(e){return(0,S.isEvent)(e)&&(e=e.target),(0,p.default)(e)};N(function(){O.default.subscribeGlobal(h.default.getDocument(),A.default.down,function(e){for(var t=te.length-1;t>=0;t--)if(!te[t]._proxiedDocumentDownHandler(e))return})});var pe=H.default.inherit({_supportedKeys:function(){var e=5,t=function(e,t,n){if(this.option("dragEnabled")){n.preventDefault(),n.stopPropagation();var i=this._allowedOffsets(),o={top:(0,C.fitIntoRange)(e,-i.top,i.bottom),left:(0,C.fitIntoRange)(t,-i.left,i.right)};this._changePosition(o)}};return(0,x.extend)(this.callBase(),{escape:function(){this.hide()},upArrow:t.bind(this,-e,0),downArrow:t.bind(this,e,0),leftArrow:t.bind(this,0,-e),rightArrow:t.bind(this,0,e)})},_getDefaultOptions:function(){var e=this;return(0,x.extend)(this.callBase(),{activeStateEnabled:!1,visible:!1,deferRendering:!0,shading:!0,shadingColor:"",position:{my:"center",at:"center"},width:function(){return.8*(0,p.default)(j).width()},minWidth:null,maxWidth:null,height:function(){return.8*(0,p.default)(j).height()},minHeight:null,maxHeight:null,animation:{show:{type:"pop",duration:300,from:{scale:.55}},hide:{type:"pop",duration:300,to:{opacity:0,scale:.55},from:{opacity:1,scale:1}}},closeOnOutsideClick:!1,onShowing:null,onShown:null,onHiding:null,onHidden:null,contentTemplate:"content",dragEnabled:!1,resizeEnabled:!1,onResizeStart:null,onResize:null,onResizeEnd:null,innerOverlay:!1,target:void 0,container:void 0,hideTopOverlayHandler:function(){e.hide()},closeOnTargetScroll:!1,onPositioned:null,boundaryOffset:{h:0,v:0},propagateOutsideClick:!1,ignoreChildEvents:!0,_checkParentVisibility:!0,_fixedPosition:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){var e=d.default.real(),t=e.platform,n=e.version;return"android"===t&&(0,D.compare)(n,[4,2])<0},options:{animation:{show:{type:"fade",duration:400},hide:{type:"fade",duration:400,to:{opacity:0},from:{opacity:1}}}}},{device:function(){return!(0,T.hasWindow)()},options:{width:null,height:null,animation:null,_checkParentVisibility:!1}}])},_setOptionsByReference:function(){this.callBase(),(0,x.extend)(this._optionsByReference,{animation:!0})},_wrapper:function(){return this._$wrapper},_container:function(){return this._$content},_eventBindingTarget:function(){return this._$content},_init:function(){this.callBase(),this._initActions(),this._initCloseOnOutsideClickHandler(),this._initTabTerminatorHandler(),this._$wrapper=(0,p.default)("<div>").addClass($),this._$content=(0,p.default)("<div>").addClass(K),this._initInnerOverlayClass();var e=this.$element();this._$wrapper.addClass(e.attr("class")),e.addClass(q),this._$wrapper.attr("data-bind","dxControlsDescendantBindings: true"),O.default.on(this._$wrapper,"MSPointerDown",v.noop),O.default.on(this._$wrapper,"focusin",function(e){e.stopPropagation()}),this._toggleViewPortSubscription(!0),this._initHideTopOverlayHandler(this.option("hideTopOverlayHandler"))},_initOptions:function(e){this._initTarget(e.target);var t=void 0===e.container?this.option("container"):e.container;this._initContainer(t),this.callBase(e)},_initInnerOverlayClass:function(){this._$content.toggleClass(Z,this.option("innerOverlay"))},_initTarget:function(e){if((0,S.isDefined)(e)){var t=this.option();(0,w.each)(["position.of","animation.show.from.position.of","animation.show.to.position.of","animation.hide.from.position.of","animation.hide.to.position.of"],function(n,i){for(var o=i.split("."),a=t;a;){if(1===o.length){(0,S.isPlainObject)(a)&&(a[o.shift()]=e);break}a=a[o.shift()]}})}},_initContainer:function(e){e=void 0===e?(0,I.value)():e;var t=this.$element(),n=t.closest(e);n.length||(n=(0,p.default)(e).first()),this._$container=n.length?n:t.parent()},_initHideTopOverlayHandler:function(e){this._hideTopOverlayHandler=e},_initActions:function(){var e=this;this._actions={},(0,w.each)(ee,function(t,n){e._actions[n]=e._createActionByOption(n,{excludeValidators:["disabled","readOnly"]})||v.noop})},_initCloseOnOutsideClickHandler:function(){var e=this;this._proxiedDocumentDownHandler=function(){return e._documentDownHandler.apply(e,arguments)}},_documentDownHandler:function(e){this._showAnimationProcessing&&this._stopAnimation();var t=this.option("closeOnOutsideClick");(0,S.isFunction)(t)&&(t=t(e));var n=this._$content,i=(0,p.default)(j.document).is(e.target)||(0,b.contains)(j.document,e.target),o=(0,p.default)(e.target).closest("."+Z).length,a=i&&!o&&!(n.is(e.target)||(0,b.contains)(n.get(0),e.target));return a&&t&&this._outsideClickHandler(e),this.option("propagateOutsideClick")},_outsideClickHandler:function(e){this.option("shading")&&e.preventDefault(),this.hide()},_getAnonymousTemplateName:function(){return Q},_initTemplates:function(){this._templateManager.addDefaultTemplates({content:new g.EmptyTemplate}),this.callBase()},_isTopOverlay:function(){for(var e=this._overlayStack(),t=e.length-1;t>=0;t--){var n=e[t]._findTabbableBounds();if(n.first||n.last)return e[t]===this}return!1},_overlayStack:function(){return te},_zIndexInitValue:function(){return pe.baseZIndex()},_toggleViewPortSubscription:function(e){G.remove(this._viewPortChangeHandle),e&&(this._viewPortChangeHandle=this._viewPortChangeHandler.bind(this),G.add(this._viewPortChangeHandle))},_viewPortChangeHandler:function(){this._initContainer(this.option("container")),this._refresh()},_renderVisibilityAnimate:function(e){return this._stopAnimation(),e?this._show():this._hide()},_normalizePosition:function(){var e=this.option("position");this._position="function"==typeof e?e():e},_getAnimationConfig:function(){var e=this.option("animation");return(0,S.isFunction)(e)&&(e=e.call(this)),e},_show:function(){var e=this,t=this,n=new y.Deferred;if(this._parentHidden=this._isParentHidden(),n.done(function(){delete t._parentHidden}),this._parentHidden)return this._isHidden=!0,n.resolve();if(this._currentVisible)return(new y.Deferred).resolve().promise();this._currentVisible=!0,this._isShown=!1,this._normalizePosition();var i=t._getAnimationConfig()||{},o=this._normalizeAnimation(i.show,"to"),a=o&&o.start||v.noop,r=o&&o.complete||v.noop;if(this._isHidingActionCanceled)delete this._isHidingActionCanceled,n.resolve();else{var s=function(){return e._renderVisibility(!0),e._isShowingActionCanceled?(delete e._isShowingActionCanceled,void n.resolve()):void e._animate(o,function(){t.option("focusStateEnabled")&&O.default.trigger(t._focusTarget(),"focus"),r.apply(this,arguments),t._showAnimationProcessing=!1,t._isShown=!0,t._actions.onShown(),t._toggleSafariScrolling(!1),n.resolve()},function(){a.apply(this,arguments),t._showAnimationProcessing=!0})};this.option("templatesRenderAsynchronously")?(this._stopShowTimer(),this._asyncShowTimeout=setTimeout(s)):s()}return n.promise()},_normalizeAnimation:function(e,t){return e&&(e=(0,x.extend)({type:"slide"},e),e[t]&&"object"===r(e[t])&&(0,x.extend)(e[t],{position:this._position})),e},_hide:function(){if(!this._currentVisible)return(new y.Deferred).resolve().promise();this._currentVisible=!1;var e=this,t=new y.Deferred,n=e._getAnimationConfig()||{},i=this._normalizeAnimation(n.hide,"from"),o=i&&i.start||v.noop,a=i&&i.complete||v.noop,r={cancel:!1};return this._isShowingActionCanceled?t.resolve():(this._actions.onHiding(r),e._toggleSafariScrolling(!0),r.cancel?(this._isHidingActionCanceled=!0,this.option("visible",!0),t.resolve()):(this._forceFocusLost(),this._toggleShading(!1),this._toggleSubscriptions(!1),this._stopShowTimer(),this._animate(i,function(){var n;e._$content.css("pointerEvents",""),e._renderVisibility(!1),a.apply(this,arguments),null===(n=e._actions)||void 0===n?void 0:n.onHidden(),t.resolve()},function(){e._$content.css("pointerEvents","none"),o.apply(this,arguments)}))),t.promise()},_forceFocusLost:function(){var e=h.default.getActiveElement(),t=!!this._$content.find(e).length;t&&(0,b.resetActiveElement)()},_animate:function(e,t,n){e?(n=n||e.start||v.noop,s.default.animate(this._$content,(0,x.extend)({},e,{start:n,complete:t}))):t()},_stopAnimation:function(){s.default.stop(this._$content,!0)},_renderVisibility:function(e){if(!e||!this._isParentHidden()){if(this._currentVisible=e,this._stopAnimation(),e||(0,M.triggerHidingEvent)(this._$content),this._toggleVisibility(e),this._$content.toggleClass(X,!e),this._updateZIndexStackPosition(e),e){this._renderContent();var t={cancel:!1};if(this._actions.onShowing(t),t.cancel)return this._toggleVisibility(!1),this._$content.toggleClass(X,!0),this._updateZIndexStackPosition(!1),this._moveFromContainer(),this._isShowingActionCanceled=!0,void this.option("visible",!1);this._moveToContainer(),this._renderGeometry(),(0,M.triggerShownEvent)(this._$content),(0,M.triggerResizeEvent)(this._$content)}else this._moveFromContainer();this._toggleShading(e),this._toggleSubscriptions(e)}},_updateZIndexStackPosition:function(e){var t=this._overlayStack(),n=(0,_.inArray)(this,t);e?(n===-1&&(this._zIndex=z.create(this._zIndexInitValue()),t.push(this)),this._$wrapper.css("zIndex",this._zIndex),this._$content.css("zIndex",this._zIndex)):n!==-1&&(t.splice(n,1),z.remove(this._zIndex))},_toggleShading:function(e){this._$wrapper.toggleClass(Y,this.option("shading")&&!this.option("container")),this._$wrapper.toggleClass(U,e&&this.option("shading")),this._$wrapper.css("backgroundColor",this.option("shading")?this.option("shadingColor"):""),this._toggleTabTerminator(e&&this.option("shading"))},_initTabTerminatorHandler:function(){var e=this;this._proxiedTabTerminatorHandler=function(){e._tabKeyHandler.apply(e,arguments)}},_toggleTabTerminator:function(e){var t=(0,R.addNamespace)("keydown",this.NAME);e?O.default.on(h.default.getDocument(),t,this._proxiedTabTerminatorHandler):O.default.off(h.default.getDocument(),t,this._proxiedTabTerminatorHandler)},_findTabbableBounds:function(){for(var e=this._$wrapper.find("*"),t=e.length-1,n={first:null,last:null},i=0;i<=t&&(!n.first&&e.eq(i).is(F.tabbable)&&(n.first=e.eq(i)),!n.last&&e.eq(t-i).is(F.tabbable)&&(n.last=e.eq(t-i)),!n.first||!n.last);i++);return n},_tabKeyHandler:function(e){if((0,R.normalizeKeyName)(e)===oe&&this._isTopOverlay()){var t=this._findTabbableBounds(),n=t.first,i=t.last,o=!e.shiftKey&&e.target===i.get(0),a=e.shiftKey&&e.target===n.get(0),r=0===t.length,s=!(0,b.contains)(this._$wrapper.get(0),e.target);if(o||a||r||s){e.preventDefault();var l=e.shiftKey?i:n;O.default.trigger(l,"focusin"),O.default.trigger(l,"focus")}}},_toggleSubscriptions:function(e){(0,T.hasWindow)()&&(this._toggleHideTopOverlayCallback(e),this._toggleParentsScrollSubscription(e))},_toggleHideTopOverlayCallback:function(e){this._hideTopOverlayHandler&&(e?B.hideCallback.add(this._hideTopOverlayHandler):B.hideCallback.remove(this._hideTopOverlayHandler))},_toggleParentsScrollSubscription:function(e){var t=this;if(this._position){var n=this._position.of||(0,p.default)(),i=this.option("closeOnTargetScroll"),o=fe(n).parents(),a=(0,R.addNamespace)("scroll",this.NAME);"desktop"===d.default.real().deviceType&&(o=o.add(j)),this._proxiedTargetParentsScrollHandler=this._proxiedTargetParentsScrollHandler||function(e){t._targetParentsScrollHandler(e)},O.default.off((0,p.default)().add(this._$prevTargetParents),a,this._proxiedTargetParentsScrollHandler),e&&i&&(O.default.on(o,a,this._proxiedTargetParentsScrollHandler),this._$prevTargetParents=o)}},_targetParentsScrollHandler:function(e){var t=!1,n=this.option("closeOnTargetScroll");(0,S.isFunction)(n)&&(t=n(e)),t||this._showAnimationProcessing||this.hide()},_render:function(){this.callBase(),this._appendContentToElement(),this._renderVisibilityAnimate(this.option("visible"))},_appendContentToElement:function(){this._$content.parent().is(this.$element())||this._$content.appendTo(this.$element())},_renderContent:function(){var e=!this._currentVisible&&this.option("deferRendering"),t=this.option("visible")&&this._isParentHidden();return t?void(this._isHidden=!0):void(this._contentAlreadyRendered||e||(this._contentAlreadyRendered=!0,this._appendContentToElement(),this.callBase()))},_isParentHidden:function(){if(!this.option("_checkParentVisibility"))return!1;if(void 0!==this._parentHidden)return this._parentHidden;var e=this.$element().parent();if(e.is(":visible"))return!1;var t=!1;return e.add(e.parents()).each(function(){var e=(0,p.default)(this);if("none"===e.css("display"))return t=!0,!1}),t||!h.default.getBody().contains(e.get(0))},_renderContentImpl:function(){var e=this,t=new y.Deferred,n=this.option("contentTemplate"),i=this._getTemplate(n),o=this._templateManager.anonymousTemplateName===n;return i&&i.render({container:(0,f.getPublicElement)(this.$content()),noModel:!0,transclude:o,onRendered:function(){t.resolve()}}),this._renderDrag(),this._renderResize(),this._renderScrollTerminator(),t.done(function(){e.option("visible")&&e._moveToContainer()}),t.promise()},_renderDrag:function(){var e=this._getDragTarget();if(e){var t=(0,R.addNamespace)(E.start,this.NAME),n=(0,R.addNamespace)(E.move,this.NAME);O.default.off(e,t),O.default.off(e,n),this.option("dragEnabled")&&(O.default.on(e,t,this._dragStartHandler.bind(this)),O.default.on(e,n,this._dragUpdateHandler.bind(this)))}},_renderResize:function(){this._resizable=this._createComponent(this._$content,V.default,{handles:this.option("resizeEnabled")?"all":"none",onResizeEnd:this._resizeEndHandler.bind(this),onResize:this._actions.onResize.bind(this),onResizeStart:this._actions.onResizeStart.bind(this),minHeight:100,minWidth:100,area:this._getDragResizeContainer()})},_resizeEndHandler:function(){this._positionChangeHandled=!0;var e=this._resizable.option("width"),t=this._resizable.option("height");e&&this.option("width",e),t&&this.option("height",t),this._actions.onResizeEnd()},_renderScrollTerminator:function(){var e=this._wrapper(),t=(0,R.addNamespace)(E.move,this.NAME);O.default.off(e,t),O.default.on(e,t,{validate:function(){return!0},getDirection:function(){return"both"},_toggleGestureCover:function(e){e||this._toggleGestureCoverImpl(e)},_clearSelection:v.noop,isNative:!0},function(e){var t=e.originalEvent.originalEvent;e._cancelPreventDefault=!0,t&&"mousemove"!==t.type&&e.cancelable!==!1&&e.preventDefault()})},_getDragTarget:function(){return this.$content()},_dragStartHandler:function(e){e.targetElements=[],this._prevOffset={x:0,y:0};var t=this._allowedOffsets();e.maxTopOffset=t.top,e.maxBottomOffset=t.bottom,e.maxLeftOffset=t.left,e.maxRightOffset=t.right},_getDragResizeContainer:function(){var e=(0,I.originalViewPort)().get(0)||this.option("container"),t=e?this._$container:(0,p.default)(j);return t},_deltaSize:function(){var e=this._$content,t=this._getDragResizeContainer(),n=e.outerWidth(),i=e.outerHeight(),o=t.outerWidth(),a=t.outerHeight();if(this._isWindow(t)){var r=h.default.getDocument(),s=Math.max((0,p.default)(r).outerHeight(),a),l=Math.max((0,p.default)(r).outerWidth(),o);a=s,o=l}return{width:o-n,height:a-i}},_dragUpdateHandler:function(e){var t=e.offset,n=this._prevOffset,i={top:t.y-n.y,left:t.x-n.x};this._changePosition(i),this._prevOffset=t},_changePosition:function(e){var t=(0,u.locate)(this._$content);(0,u.move)(this._$content,{left:t.left+e.left,top:t.top+e.top}),this._positionChangeHandled=!0},_allowedOffsets:function(){var e=(0,u.locate)(this._$content),t=this._deltaSize(),n=t.height>=0&&t.width>=0,i=!this.option("shading")||this.option("container")||this._isWindow(this._getContainer())?{top:0,left:0}:(0,u.locate)(this._$wrapper),o=this.option("boundaryOffset");return{top:n?e.top+i.top+o.v:0,bottom:n?-e.top-i.top+t.height-o.v:0,left:n?e.left+i.left+o.h:0,right:n?-e.left-i.left+t.width-o.h:0}},_moveFromContainer:function(){this._$content.appendTo(this.$element()),this._detachWrapperToContainer()},_detachWrapperToContainer:function(){this._$wrapper.detach()},_moveToContainer:function(){this._attachWrapperToContainer(),this._$content.appendTo(this._$wrapper)},_attachWrapperToContainer:function(){var e=this.$element(),t=void 0!==this.option("container"),n=t?this._$container:L.default.getSwatchContainer(e);n&&n[0]===e.parent()[0]&&(n=e),this._$wrapper.appendTo(n)},_fixHeightAfterSafariAddressBarResizing:function(){this._isWindow(this._getContainer())&&ce&&this._$wrapper.css("minHeight",j.innerHeight)},_renderGeometry:function(e){this.option("visible")&&(0,T.hasWindow)()&&this._renderGeometryImpl(e)},_renderGeometryImpl:function(e){this._stopAnimation(),this._normalizePosition(),this._renderWrapper(),this._fixHeightAfterSafariAddressBarResizing(),this._renderDimensions();var t=this._renderPosition();this._actions.onPositioned({position:t})},_fixWrapperPosition:function(){this._$wrapper.css("position",this._useFixedPosition()?"fixed":"absolute")},_useFixedPosition:function(){return this._shouldFixBodyPosition()||this.option("_fixedPosition")},_shouldFixBodyPosition:function(){var e=this._getContainer();return this._isWindow(e)&&(!ue||void 0!==this._bodyScrollTop)},_toggleSafariScrolling:function(e){if(ue&&this._shouldFixBodyPosition()){var t=h.default.getBody();e?((0,p.default)(t).removeClass(ie),j.scrollTo(0,this._bodyScrollTop),this._bodyScrollTop=void 0):this.option("visible")&&(this._bodyScrollTop=j.pageYOffset,(0,p.default)(t).addClass(ie))}},_renderWrapper:function(){this._fixWrapperPosition(),this._renderWrapperDimensions(),this._renderWrapperPosition()},_renderWrapperDimensions:function(){var e,t,n=this._getContainer();if(n){var i=this._isWindow(n);e=i?"":n.outerWidth(),t=i?"":n.outerHeight(),this._$wrapper.css({width:e,height:t})}},_isWindow:function(e){return!!e&&(0,S.isWindow)(e.get(0))},_renderWrapperPosition:function(){var e=this._getContainer();e&&l.default.setup(this._$wrapper,{my:"top left",at:"top left",of:e})},_getContainer:function(){var e=this._position,t=this.option("container"),n=null;return!t&&e&&(n=(0,S.isEvent)(e.of)?j:e.of||j),fe(t||n)},_renderDimensions:function(){var e=this._$content.get(0);this._$content.css({minWidth:this._getOptionValue("minWidth",e),maxWidth:this._getOptionValue("maxWidth",e),minHeight:this._getOptionValue("minHeight",e),maxHeight:this._getOptionValue("maxHeight",e),width:this._getOptionValue("width",e),height:this._getOptionValue("height",e)})},_renderPosition:function(){if(!this._positionChangeHandled){this._renderOverlayBoundaryOffset(),(0,u.resetPosition)(this._$content);var e=this._transformStringPosition(this._position,ae),t=l.default.setup(this._$content,e);return he(this._$content),this._actions.onPositioning(),t}var n=this._allowedOffsets();this._changePosition({top:(0,C.fitIntoRange)(0,-n.top,n.bottom),left:(0,C.fitIntoRange)(0,-n.left,n.right)})},_transformStringPosition:function(e,t){return(0,S.isString)(e)&&(e=(0,x.extend)({},t[e])),e},_renderOverlayBoundaryOffset:function(){var e=this.option("boundaryOffset");this._$content.css("margin",e.v+"px "+e.h+"px")},_focusTarget:function(){return this._$content},_attachKeyboardEvents:function(){var e=this;this._keyboardListenerId=P.keyboard.on(this._$content,null,function(t){return e._keyboardHandler(t)})},_keyboardHandler:function(e){var t=e.originalEvent,n=(0,p.default)(t.target);!n.is(this._$content)&&this.option("ignoreChildEvents")||this.callBase.apply(this,arguments)},_isVisible:function(){return this.option("visible")},_visibilityChanged:function(e){e?this.option("visible")&&this._renderVisibilityAnimate(e):this._renderVisibilityAnimate(e)},_dimensionChanged:function(){this._renderGeometry(!0)},_clean:function(){this._contentAlreadyRendered||this.$content().empty(),this._renderVisibility(!1),this._stopShowTimer(),this._cleanFocusState()},_stopShowTimer:function(){this._asyncShowTimeout&&clearTimeout(this._asyncShowTimeout),this._asyncShowTimeout=null},_dispose:function(){s.default.stop(this._$content,!1),clearTimeout(this._deferShowTimer),this._toggleViewPortSubscription(!1),this._toggleSubscriptions(!1),this._updateZIndexStackPosition(!1),this._toggleTabTerminator(!1),this._toggleSafariScrolling(!0),this._actions=null,this.callBase(),z.remove(this._zIndex),this._$wrapper.remove(),this._$content.remove()},_toggleDisabledState:function(e){this.callBase.apply(this,arguments),this._$content.toggleClass(ne,Boolean(e))},_toggleRTLDirection:function(e){this._$content.toggleClass(J,e)},_optionChanged:function(e){var t=this,n=e.value;if((0,_.inArray)(e.name,ee)>-1)return void this._initActions();switch(e.name){case"dragEnabled":this._renderDrag(),this._renderGeometry();break;case"resizeEnabled":this._renderResize(),this._renderGeometry();break;case"shading":case"shadingColor":this._toggleShading(this.option("visible"));break;case"width":case"height":case"minWidth":case"maxWidth":case"minHeight":case"maxHeight":case"boundaryOffset":this._renderGeometry();break;case"position":this._positionChangeHandled=!1,this._renderGeometry();break;case"visible":this._renderVisibilityAnimate(n).done(function(){t._animateDeferred&&t._animateDeferred.resolveWith(t)});break;case"target":this._initTarget(n),this._invalidate();break;case"container":this._initContainer(n),this._invalidate();break;case"innerOverlay":this._initInnerOverlayClass();break;case"deferRendering":case"contentTemplate":this._contentAlreadyRendered=!1,this._clean(),this._invalidate();break;case"hideTopOverlayHandler":this._toggleHideTopOverlayCallback(!1),this._initHideTopOverlayHandler(e.value),this._toggleHideTopOverlayCallback(this.option("visible"));break;case"closeOnTargetScroll":this._toggleParentsScrollSubscription(this.option("visible"));break;case"closeOnOutsideClick":case"animation":case"propagateOutsideClick":break;case"rtlEnabled":this._contentAlreadyRendered=!1,this.callBase(e);break;case"_fixedPosition":this._fixWrapperPosition();break;default:this.callBase(e)}},toggle:function(e){var t=this;e=void 0===e?!this.option("visible"):e;var n=new y.Deferred;if(e===this.option("visible"))return n.resolveWith(this,[e]).promise();var i=new y.Deferred;return this._animateDeferred=i,this.option("visible",e),i.promise().done(function(){delete t._animateDeferred,n.resolveWith(t,[t.option("visible")])}),n.promise()},$content:function(){return this._$content},show:function(){return this.toggle(!0)},hide:function(){return this.toggle(!1)},content:function(){return(0,f.getPublicElement)(this._$content)},repaint:function(){this._contentAlreadyRendered?(this._renderGeometry(),(0,M.triggerResizeEvent)(this._$content)):this.callBase()}});pe.baseZIndex=function(e){return z.base(e)},(0,c.default)("dxOverlay",pe);var ge=pe;t.default=ge,e.exports=t.default},function(e,t,n){t.clearStack=t.remove=t.create=t.base=void 0;var i=n(4),o=1500,a=[],r=function(e){return o=(0,i.ensureDefined)(e,o)};t.base=r;var s=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:o,t=a.length,n=(t?a[t-1]:e)+1;return a.push(n),n};t.create=s;var l=function(e){var t=a.indexOf(e);t>=0&&a.splice(t,1)};t.remove=l;var u=function(){a=[]};t.clearStack=u},function(e,t,n){t.SelectionFilterCreator=void 0;var i=n(4),o=n(1),a=function(e,t){function n(n,a,r,s,c){var d,h,f=a(c);if(!r&&(d=(0,i.getKeyHash)(f),!(0,o.isObject)(d))){var p=l(u(e,a,s));return p[d]?!t:!!t}for(h=0;h<e.length;h++)if(n(e[h],f))return!t;return!!t}function a(e,n){if(void 0!==n)return[e,t?"<>":"=",n]}function r(e,n){for(var i=[],o=0,r=e.length;o<r;o++){var s=e[o],l=n&&n[s],u=a(s,l);if(!u)break;o>0&&i.push(t?"or":"and"),i.push(u)}return i}this.getLocalFilter=function(e,t,o,a){return t=void 0===t?i.equalByValue:t,n.bind(this,t,e,o,a)},this.getExpr=function(n){if(n){var i;return e.forEach(function(e,s){i=i||[];var l;s>0&&i.push(t?"and":"or"),l=(0,o.isString)(n)?a(n,e):r(n,e),i.push(l)}),i&&1===i.length&&(i=i[0]),i}},this.getCombinedFilter=function(e,n){var i=this.getExpr(e),o=i;return t&&n&&(i?(o=[],o.push(i),o.push(n)):o=n),o};var s,l=function(e){if(!s){s={};for(var t=0;t<e.length;t++)s[(0,i.getKeyHash)(e[t])]=!0}return s},u=function(e,t,n){return Array.isArray(n)?e.map(function(e){return t(e)}):e}};t.SelectionFilterCreator=a},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return a=function(){return e},e}function r(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=a();if(t&&t.has(e))return t.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=o?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}t.rotateend=t.rotate=t.rotatestart=t.pinchend=t.pinch=t.pinchstart=t.zoomend=t.zoom=t.zoomstart=t.translateend=t.translate=t.translatestart=t.transformend=t.transform=t.transformstart=void 0;var s=n(29),l=r(n(3)),u=n(8),c=o(n(121)),d=o(n(99)),h="dx",f="transform",p="translate",g="pinch",_="rotate",m="start",v="",y="end",b=[],x=function(e,t){b.push({name:e,args:t})};x(f,{scale:!0,deltaScale:!0,rotation:!0,deltaRotation:!0,translation:!0,deltaTranslation:!0}),x(p,{translation:!0,deltaTranslation:!0}),x(g,{scale:!0,deltaScale:!0}),x(_,{rotation:!0,deltaRotation:!0});var w=function(e,t){return{x:t.pageX-e.pageX,y:-t.pageY+e.pageY,centerX:.5*(t.pageX+e.pageX),centerY:.5*(t.pageY+e.pageY)}},C=function(e){var t=e.pointers;return w(t[0],t[1])},k=function(e){return Math.sqrt(e.x*e.x+e.y*e.y)},S=function(e,t){return k(e)/k(t)},D=function(e,t){var n=e.x*t.x+e.y*t.y,i=k(e)*k(t);if(0===i)return 0;var o=(0,s.sign)(e.x*t.y-t.x*e.y),a=Math.acos((0,s.fitIntoRange)(n/i,-1,1));return o*a},I=function(e,t){return{x:e.centerX-t.centerX,y:e.centerY-t.centerY}},T=c.default.inherit({validatePointers:function(e){return(0,u.hasTouches)(e)>1},start:function(e){this._accept(e);var t=C(e);this._startVector=t,this._prevVector=t,this._fireEventAliases(m,e)},move:function(e){var t=C(e),n=this._getEventArgs(t);this._fireEventAliases(v,e,n),this._prevVector=t},end:function(e){var t=this._getEventArgs(this._prevVector);this._fireEventAliases(y,e,t)},_getEventArgs:function(e){return{scale:S(e,this._startVector),deltaScale:S(e,this._prevVector),rotation:D(e,this._startVector),deltaRotation:D(e,this._prevVector),translation:I(e,this._startVector),deltaTranslation:I(e,this._prevVector)}},_fireEventAliases:function(e,t,n){n=n||{},l.each(b,function(i,o){var a={};l.each(o.args,function(e){e in n&&(a[e]=n[e])}),this._fireEvent(h+o.name+e,t,a)}.bind(this))}}),O=b.reduce(function(e,t){return[m,v,y].forEach(function(n){e.push(h+t.name+n)}),e},[]);(0,d.default)({emitter:T,events:O});var E={};l.each(O,function(e,t){E[t.substring(h.length)]=t});var A=E.transformstart,P=E.transform,R=E.transformend,M=E.translatestart,B=E.translate,V=E.translateend,F=E.zoomstart,L=E.zoom,H=E.zoomend,z=E.pinchstart,N=E.pinch,j=E.pinchend,W=E.rotatestart,G=E.rotate,q=E.rotateend;t.rotateend=q,t.rotate=G,t.rotatestart=W,t.pinchend=j,t.pinch=N,t.pinchstart=z,t.zoomend=H,t.zoom=L,t.zoomstart=F,t.translateend=V,t.translate=B,t.translatestart=M,t.transformend=R,t.transform=P,t.transformstart=A},function(e,t,n){t.default=void 0;var i=n(69),o=i.DataSource;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return s(e)||r(e)||a(e)||o()}function o(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a(e,t){if(e){if("string"==typeof e)return l(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?l(e,t):void 0}}function r(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function s(e){if(Array.isArray(e))return l(e)}function l(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function u(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function c(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function d(e,t,n){return t&&c(e.prototype,t),n&&c(e,n),e}t.default=void 0;var h=n(1),f=n(278),p=function(){function e(){u(this,e);var t=(0,h.isString)(arguments[0])?this._publicCtor:this._internalCtor;t.apply(this,arguments)}return d(e,[{key:"_internalCtor",value:function(e,t,n){this.name=t||"",this.pathInfo=e&&i(e)||[],this.parentPath=this._getPathByPathInfo(this.pathInfo),this.relativeName=(0,f.pathCombine)(this.parentPath,t),this.key=this._getPathByPathInfo(this.getFullPathInfo(),!0),this.path=(0,f.pathCombine)(this.parentPath,t),this.pathKeys=this.pathInfo.map(function(e){var t=e.key;return t}),this.isRoot()||this.pathKeys.push(this.key),this._initialize(n)}},{key:"_publicCtor",value:function(e,t,n){this.path=e||"",this.pathKeys=n||[];for(var i=[],o=(0,f.getPathParts)(e,!0),a=0;a<o.length-1;a++){
var r=o[a],s={key:this.pathKeys[a]||r,name:(0,f.getName)(r)};i.push(s)}this.pathInfo=i,this.relativeName=e,this.name=(0,f.getName)(e),this.key=this.pathKeys.length?this.pathKeys[this.pathKeys.length-1]:e,this.parentPath=o.length>1?o[o.length-2]:"",this._initialize(t)}},{key:"_initialize",value:function(e){this.isDirectory=!!e,this.size=0,this.dateModified=new Date,this.thumbnail="",this.tooltipText=""}},{key:"getFullPathInfo",value:function(){var e=i(this.pathInfo);return this.isRoot()||e.push({key:this.key,name:this.name}),e}},{key:"isRoot",value:function(){return""===this.path}},{key:"getFileExtension",value:function(){return this.isDirectory?"":(0,f.getFileExtension)(this.name)}},{key:"equals",value:function(e){return e&&this.key===e.key}},{key:"createClone",value:function(){var t=new e(this.pathInfo,this.name,this.isDirectory);return t.key=this.key,t.size=this.size,t.dateModified=this.dateModified,t.thumbnail=this.thumbnail,t.tooltipText=this.tooltipText,t.hasSubDirectories=this.hasSubDirectories,t.dataItem=this.dataItem,t}},{key:"_getPathByPathInfo",value:function(e,t){return e.map(function(e){return t?(0,f.getEscapedFileName)(e.name):e.name}).join(f.PATH_SEPARATOR)}}]),e}(),g=p;t.default=g,e.exports=t.default},function(e,t,n){t.pathCombine=t.getEscapedFileName=t.getPathParts=t.getParentPath=t.getName=t.getFileExtension=t.PATH_SEPARATOR=void 0;var i=n(3),o="/";t.PATH_SEPARATOR=o;var a=function(e){var t=e.lastIndexOf(".");return t!==-1?e.substr(t):""};t.getFileExtension=a;var r=function(e){var t=e.lastIndexOf(o);return t!==-1?e.substr(t+o.length):e};t.getName=r;var s=function(e){var t=e.lastIndexOf(o);return t!==-1?e.substr(0,t):""};t.getParentPath=s;var l=function(e,t){if(!e||"/"===e)return[];for(var n=[],i="",a=0;a<e.length;a++){var r=e.charAt(a);if(r===o){var s=e.charAt(a+1);s!==o&&(i&&(n.push(i),i=""),r=s),a++}i+=r}if(i&&n.push(i),t)for(var l=0;l<n.length;l++)n[l]=c(0===l?"":n[l-1],u(n[l]));return n};t.getPathParts=l;var u=function(e){return e.replace(/\//g,"//")};t.getEscapedFileName=u;var c=function(){var e="";return(0,i.each)(arguments,function(t,n){n&&(e&&(e+=o),e+=n)}),e};t.pathCombine=c},function(e,t,n){t.default=void 0;var i={NoAccess:0,FileExists:1,FileNotFound:2,DirectoryExists:3,DirectoryNotFound:4,WrongFileExtension:5,MaxFileSizeExceeded:6,InvalidSymbols:7,Other:32767},o=i;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.confirm=t.alert=t.custom=t.title=t.FakeDialogComponent=void 0;var o=i(n(2)),a=i(n(117)),r=i(n(108)),s=i(n(13)),l=i(n(28)),u=n(33),c=n(6),d=n(1),h=n(3),f=n(0),p=n(7),g=i(n(5)),_=n(78),m=i(n(10)),v=i(n(17)),y=i(n(48)),b=n(4),x=(0,p.getWindow)(),w={text:"OK",onClick:function(){return!0}},C="dx-dialog",k="".concat(C,"-wrapper"),S="".concat(C,"-root"),D="".concat(C,"-content"),I="".concat(C,"-message"),T="".concat(C,"-buttons"),O="".concat(C,"-button"),E="dx-button",A=a.default.inherit({ctor:function(e,t){this.callBase(t)},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios"},options:{width:276}},{device:{platform:"android"},options:{lWidth:"60%",pWidth:"80%"}}])}});t.FakeDialogComponent=A;var P="";t.title=P;var R=function(e){function t(){return F.show(),i.promise()}function n(e){i.resolve(e),F.hide().done(function(){F.$element().remove()})}var i=new c.Deferred,a=(new A).option();e=(0,f.extend)(a,e);var p=(0,o.default)("<div>").addClass(C).appendTo((0,_.value)()),m="message"in e,R="messageHtml"in e;m&&v.default.log("W1013");var M=String(R?e.messageHtml:e.message),B=(0,o.default)("<div>").addClass(I).html(M),V=[];(0,h.each)(e.buttons||[w],function(){var e=new r.default(this.onClick,{context:F});V.push({toolbar:"bottom",location:s.default.current().android?"after":"center",widget:"dxButton",options:(0,f.extend)({},this,{onClick:function(){var t=e.execute.apply(e,arguments);n(t)}})})});var F=new y.default(p,(0,f.extend)({title:e.title||P,showTitle:(0,b.ensureDefined)(e.showTitle,!0),dragEnabled:(0,b.ensureDefined)(e.dragEnabled,!0),height:"auto",width:function(){var t=(0,o.default)(x).height()>(0,o.default)(x).width(),n=(t?"p":"l")+"Width",i=Object.prototype.hasOwnProperty.call(e,n)?e[n]:e.width;return(0,d.isFunction)(i)?i():i},showCloseButton:e.showCloseButton||!1,ignoreChildEvents:!1,onContentReady:function(e){e.component.$content().addClass(D).append(B)},onShowing:function(e){e.component.bottomToolbar().addClass(T).find(".".concat(E)).addClass(O),(0,u.resetActiveElement)()},onShown:function(e){var t=e.component.bottomToolbar().find(".".concat(E)).first();g.default.trigger(t,"focus")},onHiding:function(){i.reject()},toolbarItems:V,animation:{show:{type:"pop",duration:400},hide:{type:"pop",duration:400,to:{opacity:0,scale:0},from:{opacity:1,scale:1}}},rtlEnabled:(0,l.default)().rtlEnabled,boundaryOffset:{h:10,v:0}},e.popupOptions));return F._wrapper().addClass(k),e.position&&F.option("position",e.position),F._wrapper().addClass(S),{show:t,hide:n}};t.custom=R;var M=function(e,t,n){var i=(0,d.isPlainObject)(e)?e:{title:t,messageHtml:e,showTitle:n,dragEnabled:n};return R(i).show()};t.alert=M;var B=function(e,t,n){var i=(0,d.isPlainObject)(e)?e:{title:t,messageHtml:e,showTitle:n,buttons:[{text:m.default.format("Yes"),onClick:function(){return!0}},{text:m.default.format("No"),onClick:function(){return!1}}],dragEnabled:n};return R(i).show()};t.confirm=B},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=n(4),s=i(n(10)),l=i(n(9)),u=n(0),c=i(n(38)),d=i(n(58)),h=i(n(48)),f=i(n(143)),p=n(71),g=n(6),_=(0,a.getWindow)(),m="dx-actionsheet",v="dx-actionsheet-container",y="dx-actionsheet-popup-wrapper",b="dx-actionsheet-popover-wrapper",x="dx-actionsheet-cancel",w="dx-actionsheet-item",C="dxActionSheetItemData",k="dx-actionsheet-without-title",S=d.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{usePopover:!1,target:null,title:"",showTitle:!0,showCancelButton:!0,cancelText:s.default.format("Cancel"),onCancelClick:null,visible:!1,noDataText:"",focusStateEnabled:!1,selectionByClick:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios",tablet:!0},options:{usePopover:!0}}])},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({item:new p.BindableTemplate(function(e,t){var n=new c.default((0,o.default)("<div>"),(0,u.extend)({onClick:t&&t.click},t));e.append(n.$element())},["disabled","icon","text","type","onClick","click"],this.option("integrationOptions.watchMethod"))})},_itemContainer:function(){return this._$itemContainer},_itemClass:function(){return w},_itemDataKey:function(){return C},_toggleVisibility:r.noop,_renderDimensions:r.noop,_initMarkup:function(){this.callBase(),this.$element().addClass(m),this._createItemContainer()},_render:function(){this._renderPopup()},_createItemContainer:function(){this._$itemContainer=(0,o.default)("<div>").addClass(v),this._renderDisabled()},_renderDisabled:function(){this._$itemContainer.toggleClass("dx-state-disabled",this.option("disabled"))},_renderPopup:function(){this._$popup=(0,o.default)("<div>").appendTo(this.$element()),this._isPopoverMode()?this._createPopover():this._createPopup(),this._renderPopupTitle(),this._mapPopupOption("visible")},_mapPopupOption:function(e){this._popup&&this._popup.option(e,this.option(e))},_isPopoverMode:function(){return this.option("usePopover")&&this.option("target")},_renderPopupTitle:function(){this._mapPopupOption("showTitle"),this._popup&&this._popup._wrapper().toggleClass(k,!this.option("showTitle"))},_clean:function(){this._$popup&&this._$popup.remove(),this.callBase()},_overlayConfig:function(){return{onInitialized:function(e){this._popup=e.component}.bind(this),disabled:!1,showTitle:!0,title:this.option("title"),deferRendering:!_.angular,onContentReady:this._popupContentReadyAction.bind(this),onHidden:this.hide.bind(this)}},_createPopover:function(){this._createComponent(this._$popup,f.default,(0,u.extend)(this._overlayConfig(),{width:this.option("width")||200,height:this.option("height")||"auto",target:this.option("target")})),this._popup._wrapper().addClass(b)},_createPopup:function(){this._createComponent(this._$popup,h.default,(0,u.extend)(this._overlayConfig(),{dragEnabled:!1,width:this.option("width")||"100%",height:this.option("height")||"auto",showCloseButton:!1,position:{my:"bottom",at:"bottom",of:_},animation:{show:{type:"slide",duration:400,from:{position:{my:"top",at:"bottom",of:_}},to:{position:{my:"bottom",at:"bottom",of:_}}},hide:{type:"slide",duration:400,from:{position:{my:"bottom",at:"bottom",of:_}},to:{position:{my:"top",at:"bottom",of:_}}}}})),this._popup._wrapper().addClass(y)},_popupContentReadyAction:function(){this._popup.$content().append(this._$itemContainer),this._attachClickEvent(),this._attachHoldEvent(),this._prepareContent(),this._renderContent(),this._renderCancelButton()},_renderCancelButton:function(){if(!this._isPopoverMode()&&(this._$cancelButton&&this._$cancelButton.remove(),this.option("showCancelButton"))){var e=this._createActionByOption("onCancelClick")||r.noop,t=this;this._$cancelButton=(0,o.default)("<div>").addClass(x).appendTo(this._popup&&this._popup.$content()),this._createComponent(this._$cancelButton,c.default,{disabled:!1,text:this.option("cancelText"),onClick:function(n){var i={event:n,cancel:!1};e(i),i.cancel||t.hide()},integrationOptions:{}})}},_attachItemClickEvent:r.noop,_itemClickHandler:function(e){this.callBase(e),(0,o.default)(e.target).is(".dx-state-disabled, .dx-state-disabled *")||this.hide()},_itemHoldHandler:function(e){this.callBase(e),(0,o.default)(e.target).is(".dx-state-disabled, .dx-state-disabled *")||this.hide()},_optionChanged:function(e){switch(e.name){case"width":case"height":case"visible":case"title":this._mapPopupOption(e.name);break;case"disabled":this._renderDisabled();break;case"showTitle":this._renderPopupTitle();break;case"showCancelButton":case"onCancelClick":case"cancelText":this._renderCancelButton();break;case"target":case"usePopover":case"items":this._invalidate();break;default:this.callBase(e)}},toggle:function(e){var t=this,n=new g.Deferred;return t._popup.toggle(e).done(function(){t.option("visible",e),n.resolveWith(t)}),n.promise()},show:function(){return this.toggle(!0)},hide:function(){return this.toggle(!1)}});(0,l.default)("dxActionSheet",S);var D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(2)),r=n(7),s=i(n(5)),l=i(n(35)),u=i(n(9)),c=n(4),d=n(1),h=n(0),f=n(12),p=i(n(192)),g=i(n(115)),_=i(n(17)),m=n(8),v=i(n(13)),y=i(n(47)),b=n(3),x=i(n(288)),w=i(n(10)),C=n(154),k=n(6),S=i(n(334)),D=(0,r.getWindow)(),I=".dx-list-item",T="dxListItemData",O="dx-dropdownlist-popup-wrapper",E="dx-skip-gesture-event",A="input",P=["startswith","contains","endwith","notcontains"],R=p.default.inherit({_supportedKeys:function(){var e=this.callBase();return(0,h.extend)({},e,{tab:function(t){if(this._allowSelectItemByTab()){this._saveValueChangeEvent(t);var n=(0,a.default)(this._list.option("focusedElement"));n.length&&this._setSelectedElement(n)}e.tab.apply(this,arguments)},space:c.noop,home:c.noop,end:c.noop})},_allowSelectItemByTab:function(){return this.option("opened")&&"instantly"===this.option("applyValueMode")},_setSelectedElement:function(e){var t=this._valueGetter(this._list._getItemData(e));this._setValue(t)},_setValue:function(e){this.option("value",e)},_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),(0,h.extend)(x.default._dataExpressionDefaultOptions(),{displayValue:void 0,searchEnabled:!1,searchMode:"contains",searchTimeout:500,minSearchLength:0,searchExpr:null,valueChangeEvent:"input change keyup",selectedItem:null,noDataText:w.default.format("dxCollectionWidget-noDataText"),onSelectionChanged:null,onItemClick:c.noop,showDataBeforeSearch:!1,grouped:!1,groupTemplate:"group",popupPosition:{my:"left top",at:"left bottom",offset:{h:0,v:0},collision:"flip"},wrapItemText:!1}))},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios"},options:{popupPosition:{offset:{v:-1}}}},{device:{platform:"generic"},options:{buttonsLocation:"bottom center"}}])},_setOptionsByReference:function(){this.callBase(),(0,h.extend)(this._optionsByReference,{value:!0,selectedItem:!0,displayValue:!0})},_init:function(){this.callBase(),this._initDataExpressions(),this._initActions(),this._setListDataSource(),this._validateSearchMode(),this._clearSelectedItem(),this._initItems()},_setListFocusedElementOptionChange:function(){this._list._updateParentActiveDescendant=this._updateActiveDescendant.bind(this)},_initItems:function(){var e=this.option().items;e&&!e.length&&this._dataSource&&(this.option().items=this._dataSource.items())},_initActions:function(){this._initContentReadyAction(),this._initSelectionChangedAction(),this._initItemClickAction()},_initContentReadyAction:function(){this._contentReadyAction=this._createActionByOption("onContentReady",{excludeValidators:["disabled","readOnly"]})},_initSelectionChangedAction:function(){this._selectionChangedAction=this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})},_initItemClickAction:function(){this._itemClickAction=this._createActionByOption("onItemClick")},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({item:new C.ChildDefaultTemplate("item")})},_saveFocusOnWidget:function(e){this._list&&this._list.initialOption("focusStateEnabled")&&this._focusInput()},_fitIntoRange:function(e,t,n){return e>n?t:e<t?n:e},_items:function(){var e=this._getPlainItems(!this._list&&this._dataSource.items()),t=new y.default(e).filter("disabled","<>",!0).toArray();return t},_calcNextItem:function(e){var t=this._items(),n=this._fitIntoRange(this._getSelectedIndex()+e,0,t.length-1);return t[n]},_getSelectedIndex:function(){var e=this._items(),t=this.option("selectedItem"),n=-1;return(0,b.each)(e,function(e,i){if(this._isValueEquals(i,t))return n=e,!1}.bind(this)),n},_createPopup:function(){this.callBase(),this._updateCustomBoundaryContainer(),this._popup._wrapper().addClass(this._popupWrapperClass());var e=this._popup.$content();s.default.off(e,"mouseup"),s.default.on(e,"mouseup",this._saveFocusOnWidget.bind(this));var t=this;this._popup.on({shown:function(){t.$element().addClass(E)},hidden:function(){t.$element().removeClass(E)}})},_updateCustomBoundaryContainer:function(){var e=this.option("dropDownOptions.container"),t=e&&(0,a.default)(e);if(t&&t.length&&!(0,d.isWindow)(t.get(0))){var n=[].slice.call(t.parents());n.unshift(t.get(0)),(0,b.each)(n,function(e,t){return t!==(0,a.default)("body").get(0)&&("hidden"===D.getComputedStyle(t).overflowY?(this._$customBoundaryContainer=(0,a.default)(t),!1):void 0)}.bind(this))}},_popupWrapperClass:function(){return O},_renderInputValue:function(){var e=this._getCurrentValue();return this._loadInputValue(e,this._setSelectedItem.bind(this)).always(this.callBase.bind(this,e))},_loadInputValue:function(e,t){return this._loadItem(e).always(t)},_getItemFromPlain:function(e,t){var n,i;return t&&"object"!==o(e)&&(t.itemByValue||(t.itemByValue={},n=this._getPlainItems(),n.forEach(function(e){t.itemByValue[this._valueGetter(e)]=e},this)),i=t.itemByValue[e]),i||(n=this._getPlainItems(),i=(0,c.grep)(n,function(t){return this._isValueEquals(this._valueGetter(t),e)}.bind(this))[0]),i},_loadItem:function(e,t){var n=this._getItemFromPlain(e,t);return void 0!==n?(new k.Deferred).resolve(n).promise():this._loadValue(e)},_getPlainItems:function(e){var t=[];e=e||this.option("items")||this._dataSource.items()||[];for(var n=0;n<e.length;n++)e[n]&&e[n].items?t=t.concat(e[n].items):t.push(e[n]);return t},_updateActiveDescendant:function(){var e,t=this.option("opened"),n=null===(e=this._list)||void 0===e?void 0:e.getFocusedItemId(),i=(0,a.default)("#".concat(n)).length>0,o=t&&i&&n;this.setAria({activedescendant:o||null})},_setSelectedItem:function(e){var t=this._displayValue(e);this.option("selectedItem",(0,c.ensureDefined)(e,null)),this.option("displayValue",t)},_displayValue:function(e){return this._displayGetter(e)},_refreshSelected:function(){var e={};this._listItemElements().each(function(t,n){var i=(0,a.default)(n),o=this._valueGetter(i.data(T)),r=this._isSelectedValue(o,e);r?this._list.selectItem(i):this._list.unselectItem(i)}.bind(this))},_popupShownHandler:function(){this.callBase(),this._setFocusPolicy()},_setFocusPolicy:function(){this.option("focusStateEnabled")&&this._list&&this._list.option("focusedElement",null)},_isSelectedValue:function(e){return this._isValueEquals(e,this.option("value"))},_validateSearchMode:function(){var e=this.option("searchMode"),t=e.toLowerCase();if((0,f.inArray)(t,P)<0)throw _.default.Error("E1019",e)},_clearSelectedItem:function(){this.option("selectedItem",null)},_processDataSourceChanging:function(){this._setListDataSource(),this._renderInputValue().fail(function(){this._isCustomValueAllowed()||this._clearSelectedItem()}.bind(this))},_isCustomValueAllowed:function(){return this.option("displayCustomValue")},reset:function(){this.callBase(),this._clearFilter(),this._clearSelectedItem(),this._preventFiltering=!0},_listItemElements:function(){return this._$list?this._$list.find(I):(0,a.default)()},_popupConfig:function(){return(0,h.extend)(this.callBase(),{templatesRenderAsynchronously:!1,autoResizeEnabled:!1,maxHeight:this._getMaxHeight.bind(this)})},_renderPopupContent:function(){this.callBase(),this._renderList()},_getKeyboardListeners:function(){var e=this._canListHaveFocus();return this.callBase().concat([!e&&this._list])},_fireContentReadyAction:c.noop,_setAriaTargetForList:function(){this._list._getAriaTarget=this._getAriaTarget.bind(this)},_renderList:function(){this._listId="dx-"+(new l.default)._value;var e=this._$list=(0,a.default)("<div>").attr("id",this._listId).appendTo(this._popup.$content());this._list=this._createComponent(e,g.default,this._listConfig()),this._refreshList(),this._setAriaTargetForList(),this._list.option("_listAttributes",{role:"combobox"}),this._renderPreventBlur(this._$list),this._setListFocusedElementOptionChange()},_renderPreventBlur:function(e){var t=(0,m.addNamespace)("mousedown","dxDropDownList");s.default.off(e,t),s.default.on(e,t,function(e){e.preventDefault()}.bind(this))},_renderOpenedState:function(){this.callBase(),this._list&&this._updateActiveDescendant(),this.setAria({controls:this._list&&this._listId,owns:this._popup&&this._popupContentId})},_setDefaultAria:function(){this.setAria({haspopup:"listbox",autocomplete:"list"})},_refreshList:function(){this._list&&this._shouldRefreshDataSource()&&this._setListDataSource()},_shouldRefreshDataSource:function(){var e=!!this._list.option("dataSource");return e!==this._needPassDataSourceToList()},_isDesktopDevice:function(){return"desktop"===v.default.real().deviceType},_listConfig:function(){var e={selectionMode:"single",_templates:this.option("_templates"),templateProvider:this.option("templateProvider"),noDataText:this.option("noDataText"),grouped:this.option("grouped"),wrapItemText:this.option("wrapItemText"),onContentReady:this._listContentReadyHandler.bind(this),itemTemplate:this.option("itemTemplate"),indicateLoading:!1,keyExpr:this._getCollectionKeyExpr(),displayExpr:this._displayGetterExpr(),groupTemplate:this.option("groupTemplate"),onItemClick:this._listItemClickAction.bind(this),dataSource:this._getDataSource(),_revertPageOnEmptyLoad:!0,hoverStateEnabled:!!this._isDesktopDevice()&&this.option("hoverStateEnabled"),focusStateEnabled:!!this._isDesktopDevice()&&this.option("focusStateEnabled")};return this._canListHaveFocus()||(e.tabIndex=null),e},_canListHaveFocus:function(){return!1},_getDataSource:function(){return this._needPassDataSourceToList()?this._dataSource:null},_dataSourceOptions:function(){return{paginate:!1}},_getGroupedOption:function(){return this.option("grouped")},_dataSourceFromUrlLoadMode:function(){return"raw"},_listContentReadyHandler:function(){this._list=this._list||this._$list.dxList("instance"),this.option("deferRendering")||this._refreshSelected(),this._dimensionChanged(),this._contentReadyAction()},_setListOption:function(e,t){this._setWidgetOption("_list",arguments)},_listItemClickAction:function(e){this._listItemClickHandler(e),this._itemClickAction(e)},_listItemClickHandler:c.noop,_setListDataSource:function(){this._list&&(this._setListOption("dataSource",this._getDataSource()),this._needPassDataSourceToList()||this._setListOption("items",[]))},_needPassDataSourceToList:function(){return this.option("showDataBeforeSearch")||this._isMinSearchLengthExceeded()},_isMinSearchLengthExceeded:function(){return this._searchValue().toString().length>=this.option("minSearchLength")},_needClearFilter:function(){return!this._canKeepDataSource()&&this._needPassDataSourceToList()},_canKeepDataSource:function(){var e,t=this._isMinSearchLengthExceeded();return(null===(e=this._dataSource)||void 0===e?void 0:e.isLoaded())&&this.option("showDataBeforeSearch")&&this.option("minSearchLength")&&!t&&!this._isLastMinSearchLengthExceeded},_searchValue:function(){return this._input().val()||""},_getSearchEvent:function(){return(0,m.addNamespace)(A,this.NAME+"Search")},_getSetFocusPolicyEvent:function(){return(0,m.addNamespace)("input",this.NAME+"FocusPolicy")},_renderEvents:function(){this.callBase(),s.default.on(this._input(),this._getSetFocusPolicyEvent(),this._setFocusPolicy.bind(this)),this._shouldRenderSearchEvent()&&s.default.on(this._input(),this._getSearchEvent(),this._searchHandler.bind(this))},_shouldRenderSearchEvent:function(){return this.option("searchEnabled")},_refreshEvents:function(){s.default.off(this._input(),this._getSearchEvent()),s.default.off(this._input(),this._getSetFocusPolicyEvent()),this.callBase()},_searchHandler:function(){if(!this._isMinSearchLengthExceeded())return void this._searchCanceled();var e=this.option("searchTimeout");e?(this._clearSearchTimer(),this._searchTimer=setTimeout(this._searchDataSource.bind(this),e)):this._searchDataSource()},_searchCanceled:function(){this._clearSearchTimer(),this._needClearFilter()&&this._filterDataSource(null),this._refreshList()},_searchDataSource:function(){this._filterDataSource(this._searchValue())},_filterDataSource:function(e){this._clearSearchTimer();var t=this._dataSource;t&&(t.searchExpr(this.option("searchExpr")||this._displayGetterExpr()),t.searchOperation(this.option("searchMode")),t.searchValue(e),t.load().done(this._dataSourceFiltered.bind(this,e)))},_clearFilter:function(){var e=this._dataSource;e&&e.searchValue()&&e.searchValue(null)},_dataSourceFiltered:function(){this._isLastMinSearchLengthExceeded=this._isMinSearchLengthExceeded(),this._refreshList(),this._refreshPopupVisibility()},_shouldOpenPopup:function(){return this._hasItemsToShow()},_refreshPopupVisibility:function(){if(!this.option("readOnly")&&this._searchValue()){var e=this._shouldOpenPopup();e&&!this._isFocused()||(this.option("opened",e),e&&this._dimensionChanged())}},_dataSourceChangedHandler:function(e){0===this._dataSource.pageIndex()?this.option().items=e:this.option().items=this.option().items.concat(e)},_hasItemsToShow:function(){var e=this._dataSource&&this._dataSource.items()||[],t=e.length,n=this._needPassDataSourceToList();return!(!n||!t)},_clearSearchTimer:function(){clearTimeout(this._searchTimer),delete this._searchTimer},_popupShowingHandler:function(){this._dimensionChanged()},_dimensionChanged:function(){this.callBase(arguments),this._popup&&this._updatePopupDimensions()},_needPopupRepaint:function(){if(!this._dataSource)return!1;var e=this._dataSource.pageIndex(),t=(0,d.isDefined)(this._pageIndex)&&e<=this._pageIndex;return this._pageIndex=e,t},_updatePopupDimensions:function(){this._needPopupRepaint()&&this._popup.repaint(),this._list&&this._list.updateDimensions()},_getMaxHeight:function(){var e=this.$element(),t=this._$customBoundaryContainer,n=e.offset().top-(t?t.offset().top:0),i=(0,a.default)(D).outerHeight(),o=t?Math.min(t.outerHeight(),i):i,r=Math.max(n,o-n-e.outerHeight());return Math.min(.5*o,r)},_clean:function(){this._list&&delete this._list,delete this._isLastMinSearchLengthExceeded,this.callBase()},_dispose:function(){this._clearSearchTimer(),this.callBase()},_setCollectionWidgetOption:function(){this._setListOption.apply(this,arguments)},_setSubmitValue:function(){var e=this.option("value"),t=this._shouldUseDisplayValue(e)?this._displayGetter(e):e;this._getSubmitElement().val(t)},_shouldUseDisplayValue:function(e){return"this"===this.option("valueExpr")&&(0,d.isObject)(e)},_optionChanged:function(e){switch(this._dataExpressionOptionChanged(e),e.name){case"hoverStateEnabled":case"focusStateEnabled":this._isDesktopDevice()&&this._setListOption(e.name,e.value),this.callBase(e);break;case"items":this.option("dataSource")||this._processDataSourceChanging();break;case"dataSource":this._processDataSourceChanging();break;case"valueExpr":this._renderValue(),this._setListOption("keyExpr",this._getCollectionKeyExpr());break;case"displayExpr":this._renderValue(),this._setListOption("displayExpr",this._displayGetterExpr());break;case"searchMode":this._validateSearchMode();break;case"minSearchLength":this._refreshList();break;case"searchEnabled":case"showDataBeforeSearch":case"searchExpr":this._invalidate();break;case"onContentReady":this._initContentReadyAction();break;case"onSelectionChanged":this._initSelectionChangedAction();break;case"onItemClick":this._initItemClickAction();break;case"grouped":case"groupTemplate":case"wrapItemText":case"noDataText":this._setListOption(e.name);break;case"displayValue":this.option("text",e.value);break;case"itemTemplate":case"searchTimeout":break;case"selectedItem":e.previousValue!==e.value&&this._selectionChangedAction({selectedItem:e.value});break;default:this.callBase(e)}}}).include(x.default,S.default);(0,u.default)("dxDropDownList",R);var M=R;t.default=M,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){return r||T.default}function a(e){r=e}t.setScrollView=a,t.ListBase=void 0;var r,s=i(n(2)),l=i(n(5)),u=n(4),c=n(1),d=n(59),h=n(18),f=n(3),p=n(19),g=n(0),_=i(n(43)),m=n(21),v=n(191),y=n(46),b=i(n(10)),x=n(87),w=i(n(13)),C=i(n(602)),k=i(n(38)),S=n(8),D=n(36),I=n(7),T=i(n(96)),O=n(431),E=i(n(214)),A=n(71),P=n(6),R=i(n(334)),M="dx-list",B="dx-list-item",V="."+B,F="dx-list-item-icon-container",L="dx-list-item-icon",H="dx-list-group",z="dx-list-group-header",N="dx-list-group-body",j="dx-list-collapsible-groups",W="dx-list-group-collapsed",G="dx-list-group-header-indicator",q="dx-has-next",$="dx-list-next-button",K="dx-wrap-item-text",U=".dx-list-select-all",Y="dxListItemData",Z=70,X=(0,p.compileGetter)("items"),Q=E.default.inherit({_activeStateUnit:[V,U].join(","),_supportedKeys:function(){function e(e){var t=n.scrollTop(),i=n.$element().height(),o=(0,s.default)(n.option("focusedElement")),a=!0;if(!o.length)return(0,s.default)();for(;a;){var r=o[e]();if(!r.length)break;var l=r.position().top+r.outerHeight()/2;a=l<i+t&&l>t,a&&(o=r)}return o}function t(e,t){var i=e.position().top;"prev"===t&&(i=e.position().top-n.$element().height()+e.outerHeight()),n.scrollTo(i)}var n=this,i=function(i){var o=e(i),a=o.is(n.option("focusedElement"));a&&(t(o,i),o=e(i)),n.option("focusedElement",(0,h.getPublicElement)(o)),n.scrollToItem(o)};return(0,g.extend)(this.callBase(),{leftArrow:u.noop,rightArrow:u.noop,pageUp:function(){return i("prev"),!1},pageDown:function(){return i("next"),!1}})},_getDefaultOptions:function(){return(0,g.extend)(this.callBase(),{hoverStateEnabled:!0,pullRefreshEnabled:!1,scrollingEnabled:!0,showScrollbar:"onScroll",useNativeScrolling:!0,bounceEnabled:!0,scrollByContent:!0,scrollByThumb:!1,pullingDownText:b.default.format("dxList-pullingDownText"),pulledDownText:b.default.format("dxList-pulledDownText"),refreshingText:b.default.format("dxList-refreshingText"),pageLoadingText:b.default.format("dxList-pageLoadingText"),onScroll:null,onPullRefresh:null,onPageLoading:null,pageLoadMode:"scrollBottom",nextButtonText:b.default.format("dxList-nextButtonText"),onItemSwipe:null,grouped:!1,onGroupRendered:null,collapsibleGroups:!1,groupTemplate:"group",indicateLoading:!0,activeStateEnabled:!0,_itemAttributes:{role:"option"},_listAttributes:{role:"listbox"},useInkRipple:!1,wrapItemText:!1,_swipeEnabled:!0,_revertPageOnEmptyLoad:!1,showChevronExpr:function(e){return e?e.showChevron:void 0},badgeExpr:function(e){return e?e.badge:void 0}})},_defaultOptionsRules:function(){var e=(0,D.current)();return this.callBase().concat((0,O.deviceDependentOptions)(),[{device:function(){return!y.nativeScrolling},options:{useNativeScrolling:!1}},{device:function(e){return!y.nativeScrolling&&!w.default.isSimulator()&&"desktop"===w.default.real().deviceType&&"generic"===e.platform},options:{showScrollbar:"onHover",pageLoadMode:"nextButton"}},{device:function(){return"desktop"===w.default.real().deviceType&&!w.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,D.isMaterial)(e)},options:{pullingDownText:"",pulledDownText:"",refreshingText:"",pageLoadingText:"",useInkRipple:!0}}])},_visibilityChanged:function(e){e&&this._updateLoadingState(!0)},_itemClass:function(){return B},_itemDataKey:function(){return Y},_itemContainer:function(){return this._$container},_saveSelectionChangeEvent:function(e){this._selectionChangeEventInstance=e},_getSelectionChangeEvent:function(){return this._selectionChangeEventInstance},_refreshItemElements:function(){this.option("grouped")?this._itemElementsCache=this._itemContainer().children("."+H).children("."+N).children(this._itemSelector()):this._itemElementsCache=this._itemContainer().children(this._itemSelector())},_modifyByChanges:function(){this.callBase.apply(this,arguments),this._refreshItemElements(),this._updateLoadingState(!0)},reorderItem:function(e,t){var n=this.callBase(e,t);return n.done(function(){this._refreshItemElements()})},deleteItem:function(e){var t=this.callBase(e);return t.done(function(){this._refreshItemElements()})},_itemElements:function(){return this._itemElementsCache},_itemSelectHandler:function(e){"single"===this.option("selectionMode")&&this.isItemSelected(e.currentTarget)||this.callBase(e)},_allowDynamicItemsAppend:function(){return!0},_resetDataSourcePageIndex:function(){var e=this.getDataSource();e&&0!==e.pageIndex()&&(e.pageIndex(0),e.load())},_init:function(){this.callBase(),this._resetDataSourcePageIndex(),this._$container=this.$element(),this._initScrollView(),this._feedbackShowTimeout=Z,this._createGroupRenderAction()},_scrollBottomMode:function(){return"scrollBottom"===this.option("pageLoadMode")},_nextButtonMode:function(){return"nextButton"===this.option("pageLoadMode")},_dataSourceOptions:function(){var e=this._scrollBottomMode(),t=this._nextButtonMode();return(0,g.extend)(this.callBase(),{paginate:(0,u.ensureDefined)(e||t,!0)})},_getGroupedOption:function(){return this.option("grouped")},_dataSourceFromUrlLoadMode:function(){return"raw"},_initScrollView:function(){var e=this.option("scrollingEnabled"),t=e&&this.option("pullRefreshEnabled"),n=e&&this._scrollBottomMode()&&!!this._dataSource;this._scrollView=this._createComponent(this.$element(),o(),{disabled:this.option("disabled")||!e,onScroll:this._scrollHandler.bind(this),onPullDown:t?this._pullDownHandler.bind(this):null,onReachBottom:n?this._scrollBottomHandler.bind(this):null,showScrollbar:this.option("showScrollbar"),useNative:this.option("useNativeScrolling"),bounceEnabled:this.option("bounceEnabled"),scrollByContent:this.option("scrollByContent"),scrollByThumb:this.option("scrollByThumb"),pullingDownText:this.option("pullingDownText"),pulledDownText:this.option("pulledDownText"),
refreshingText:this.option("refreshingText"),reachBottomText:this.option("pageLoadingText"),useKeyboard:!1}),this._$container=(0,s.default)(this._scrollView.content()),this.option("wrapItemText")&&this._$container.addClass(K),this._createScrollViewActions()},_createScrollViewActions:function(){this._scrollAction=this._createActionByOption("onScroll"),this._pullRefreshAction=this._createActionByOption("onPullRefresh"),this._pageLoadingAction=this._createActionByOption("onPageLoading")},_scrollHandler:function(e){this._scrollAction&&this._scrollAction(e)},_initTemplates:function(){this._templateManager.addDefaultTemplates({group:new A.BindableTemplate(function(e,t){(0,c.isPlainObject)(t)?t.key&&e.text(t.key):e.text(String(t))},["key"],this.option("integrationOptions.watchMethod"))}),this.callBase()},_prepareDefaultItemTemplate:function(e,t){if(this.callBase(e,t),e.icon){var n=(0,d.getImageContainer)(e.icon).addClass(L),i=(0,s.default)("<div>").addClass(F);i.append(n),t.prepend(i)}},_getBindableFields:function(){return["text","html","icon"]},_updateLoadingState:function(e){var t=!e||this._isLastPage(),n=this._scrollBottomMode(),i=t||!n,o=i&&!this._isDataSourceLoading();i||this._scrollViewIsFull()?(this._scrollView.release(o),this._toggleNextButton(this._shouldRenderNextButton()&&!this._isLastPage()),this._loadIndicationSuppressed(!1)):this._infiniteDataLoading()},_shouldRenderNextButton:function(){return this._nextButtonMode()&&this._dataSource&&this._dataSource.isLoaded()},_dataSourceLoadingChangedHandler:function(e){this._loadIndicationSuppressed()||(e&&this.option("indicateLoading")?this._showLoadingIndicatorTimer=setTimeout(function(){var e=!this._itemElements().length;this._scrollView&&!e&&this._scrollView.startLoading()}.bind(this)):(clearTimeout(this._showLoadingIndicatorTimer),this._scrollView&&this._scrollView.finishLoading()))},_dataSourceChangedHandler:function(e){!this._shouldAppendItems()&&(0,I.hasWindow)()&&this._scrollView&&this._scrollView.scrollTo(0),this.callBase.apply(this,arguments)},_refreshContent:function(){this._prepareContent(),this._fireContentReadyAction()},_hideLoadingIfLoadIndicationOff:function(){this.option("indicateLoading")||this._dataSourceLoadingChangedHandler(!1)},_loadIndicationSuppressed:function(e){return arguments.length?void(this._isLoadIndicationSuppressed=e):this._isLoadIndicationSuppressed},_scrollViewIsFull:function(){return!this._scrollView||this._scrollView.isFull()},_pullDownHandler:function(e){this._pullRefreshAction(e),this._dataSource&&!this._isDataSourceLoading()?(this._clearSelectedItems(),this._dataSource.pageIndex(0),this._dataSource.reload()):this._updateLoadingState()},_infiniteDataLoading:function(){var e=this,t=this.$element().is(":visible");!t||this._scrollViewIsFull()||this._isDataSourceLoading()||this._isLastPage()||(clearTimeout(this._loadNextPageTimer),this._loadNextPageTimer=setTimeout(function(){e._loadNextPage().done(e._setPreviousPageIfNewIsEmpty.bind(e))}))},_setPreviousPageIfNewIsEmpty:function(e){if(this.option("_revertPageOnEmptyLoad")){var t=this.getDataSource(),n=null===t||void 0===t?void 0:t.pageIndex();0===(null===e||void 0===e?void 0:e.length)&&n>0&&(this._fireContentReadyAction(),t.pageIndex(n-1))}},_scrollBottomHandler:function(e){this._pageLoadingAction(e),this._isDataSourceLoading()||this._isLastPage()?this._updateLoadingState():this._loadNextPage()},_renderItems:function(e){this.option("grouped")?((0,f.each)(e,this._renderGroup.bind(this)),this._attachGroupCollapseEvent(),this._renderEmptyMessage(),(0,D.isMaterial)()&&this.attachGroupHeaderInkRippleEvents()):this.callBase.apply(this,arguments),this._refreshItemElements(),this._updateLoadingState(!0)},_attachGroupCollapseEvent:function(){var e=(0,S.addNamespace)(m.name,this.NAME),t="."+z,n=this.$element(),i=this.option("collapsibleGroups");n.toggleClass(j,i),l.default.off(n,e,t),i&&l.default.on(n,e,t,function(e){this._createAction(function(e){var t=(0,s.default)(e.event.currentTarget).parent();this._collapseGroupHandler(t),this.option("focusStateEnabled")&&this.option("focusedElement",(0,h.getPublicElement)(t.find("."+B).eq(0)))}.bind(this),{validatingTargetName:"element"})({event:e})}.bind(this))},_collapseGroupHandler:function(e,t){var n=new P.Deferred;if(e.hasClass(W)===t)return n.resolve();var i=e.children("."+N),o=i.outerHeight(),a=0===o?i.height("auto").outerHeight():0;return e.toggleClass(W,t),_.default.animate(i,{type:"custom",from:{height:o},to:{height:a},duration:200,complete:function(){this.updateDimensions(),this._updateLoadingState(),n.resolve()}.bind(this)}),n.promise()},_dataSourceLoadErrorHandler:function(){this._forgetNextPageLoading(),this._initialized&&(this._renderEmptyMessage(),this._updateLoadingState())},_initMarkup:function(){this._itemElementsCache=(0,s.default)(),this.$element().addClass(M),this.callBase(),this.option("useInkRipple")&&this._renderInkRipple(),this.setAria("role",this.option("_listAttributes").role)},_renderInkRipple:function(){this._inkRipple=(0,x.render)()},_toggleActiveState:function(e,t,n){this.callBase.apply(this,arguments);var i=this;if(this._inkRipple){var o={element:e,event:n};t?(0,D.isMaterial)()?this._inkRippleTimer=setTimeout(function(){i._inkRipple.showWave(o)},Z/2):i._inkRipple.showWave(o):(clearTimeout(this._inkRippleTimer),this._inkRipple.hideWave(o))}},_postprocessRenderItem:function(e){this._refreshItemElements(),this.callBase.apply(this,arguments),this.option("_swipeEnabled")&&this._attachSwipeEvent((0,s.default)(e.itemElement))},_attachSwipeEvent:function(e){var t=(0,S.addNamespace)(v.end,this.NAME);l.default.on(e,t,this._itemSwipeEndHandler.bind(this))},_itemSwipeEndHandler:function(e){this._itemDXEventHandler(e,"onItemSwipe",{direction:e.offset<0?"left":"right"})},_nextButtonHandler:function(e){this._pageLoadingAction(e);var t=this._dataSource;t&&!t.isLoading()&&(this._scrollView.toggleLoading(!0),this._$nextButton.detach(),this._loadIndicationSuppressed(!0),this._loadNextPage())},_renderGroup:function(e,t){var n=(0,s.default)("<div>").addClass(H).appendTo(this._itemContainer()),i=(0,s.default)("<div>").addClass(z).appendTo(n),o=this.option("groupTemplate"),a=this._getTemplate(t.template||o,t,e,i),r={index:e,itemData:t,container:(0,h.getPublicElement)(i)};this._createItemByTemplate(a,r),(0,D.isMaterial)()&&(0,s.default)("<div>").addClass(G).prependTo(i),this._renderingGroupIndex=e;var l=(0,s.default)("<div>").addClass(N).appendTo(n);(0,f.each)(X(t)||[],function(e,t){this._renderItem(e,t,l)}.bind(this)),this._groupRenderAction({groupElement:(0,h.getPublicElement)(n),groupIndex:e,groupData:t})},downInkRippleHandler:function(e){this._toggleActiveState((0,s.default)(e.currentTarget),!0,e)},upInkRippleHandler:function(e){this._toggleActiveState((0,s.default)(e.currentTarget),!1)},attachGroupHeaderInkRippleEvents:function(){var e="."+z,t=this.$element();this._downInkRippleHandler=this._downInkRippleHandler||this.downInkRippleHandler.bind(this),this._upInkRippleHandler=this._upInkRippleHandler||this.upInkRippleHandler.bind(this);var n=[t,"dxpointerdown",e,this._downInkRippleHandler],i=[t,"dxpointerup dxpointerout",e,this._upInkRippleHandler];l.default.off.apply(l.default,n),l.default.on.apply(l.default,n),l.default.off.apply(l.default,i),l.default.on.apply(l.default,i)},_createGroupRenderAction:function(){this._groupRenderAction=this._createActionByOption("onGroupRendered")},_clean:function(){clearTimeout(this._inkRippleTimer),this._$nextButton&&(this._$nextButton.remove(),this._$nextButton=null),delete this._inkRipple,this.callBase.apply(this,arguments)},_dispose:function(){clearTimeout(this._holdTimer),clearTimeout(this._loadNextPageTimer),clearTimeout(this._showLoadingIndicatorTimer),this.callBase()},_toggleDisabledState:function(e){this.callBase(e),this._scrollView.option("disabled",e||!this.option("scrollingEnabled"))},_toggleNextButton:function(e){var t=this._dataSource,n=this._getNextButton();this.$element().toggleClass(q,e),e&&t&&t.isLoaded()&&n.appendTo(this._itemContainer()),e||n.detach()},_getNextButton:function(){return this._$nextButton||(this._$nextButton=this._createNextButton()),this._$nextButton},_createNextButton:function(){var e=(0,s.default)("<div>").addClass($),t=(0,s.default)("<div>").appendTo(e);return this._createComponent(t,k.default,{text:this.option("nextButtonText"),onClick:this._nextButtonHandler.bind(this),type:(0,D.isMaterial)()?"default":void 0,integrationOptions:{}}),e},_moveFocus:function(){this.callBase.apply(this,arguments),this.scrollToItem(this.option("focusedElement"))},_refresh:function(){if((0,I.hasWindow)()){var e=this._scrollView.scrollTop();this.callBase(),e&&this._scrollView.scrollTo(e)}else this.callBase()},_optionChanged:function(e){switch(e.name){case"pageLoadMode":this._toggleNextButton(e.value),this._initScrollView();break;case"dataSource":this.callBase(e),this._initScrollView();break;case"pullingDownText":case"pulledDownText":case"refreshingText":case"pageLoadingText":case"showScrollbar":case"bounceEnabled":case"scrollByContent":case"scrollByThumb":case"useNativeScrolling":case"scrollingEnabled":case"pullRefreshEnabled":this._initScrollView(),this._updateLoadingState();break;case"nextButtonText":case"onItemSwipe":case"useInkRipple":this._invalidate();break;case"onScroll":case"onPullRefresh":case"onPageLoading":this._createScrollViewActions();break;case"grouped":case"collapsibleGroups":case"groupTemplate":this._invalidate();break;case"wrapItemText":this._$container.toggleClass(K,e.value);break;case"onGroupRendered":this._createGroupRenderAction();break;case"width":case"height":this.callBase(e),this._scrollView.update();break;case"indicateLoading":this._hideLoadingIfLoadIndicationOff();break;case"visible":this.callBase(e),this._scrollView.update();break;case"rtlEnabled":this._initScrollView(),this.callBase(e);break;case"showChevronExpr":case"badgeExpr":this._invalidate();break;case"_swipeEnabled":case"_revertPageOnEmptyLoad":break;case"_listAttributes":break;default:this.callBase(e)}},_extendActionArgs:function(e){if(!this.option("grouped"))return this.callBase(e);var t=e.closest("."+H),n=t.find("."+B);return(0,g.extend)(this.callBase(e),{itemIndex:{group:t.index(),item:n.index(e)}})},expandGroup:function(e){var t=new P.Deferred,n=this._itemContainer().find("."+H).eq(e);return this._collapseGroupHandler(n,!1).done(function(){t.resolveWith(this)}.bind(this)),t.promise()},collapseGroup:function(e){var t=new P.Deferred,n=this._itemContainer().find("."+H).eq(e);return this._collapseGroupHandler(n,!0).done(function(){t.resolveWith(this)}.bind(this)),t},updateDimensions:function(){var e=this,t=new P.Deferred;return e._scrollView?e._scrollView.update().done(function(){!e._scrollViewIsFull()&&e._updateLoadingState(!0),t.resolveWith(e)}):t.resolveWith(e),t.promise()},reload:function(){this.callBase(),this.scrollTo(0),this._pullDownHandler()},repaint:function(){this.scrollTo(0),this.callBase()},scrollTop:function(){return this._scrollView.scrollOffset().top},clientHeight:function(){return this._scrollView.clientHeight()},scrollHeight:function(){return this._scrollView.scrollHeight()},scrollBy:function(e){this._scrollView.scrollBy(e)},scrollTo:function(e){this._scrollView.scrollTo(e)},scrollToItem:function(e){var t=this._editStrategy.getItemElement(e);this._scrollView.scrollToElement(t)}}).include(R.default);t.ListBase=Q,Q.ItemClass=C.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(8),s=n(4),l=n(3),u=i(n(13)),c=i(n(15)),d=i(n(428)),h="dxNativeScrollable",f="dx-scrollable-native",p="dx-scrollable-scrollbar-simulated",g="dx-scrollable-scrollbars-hidden",_="vertical",m="horizontal",v=500,y=c.default.inherit({ctor:function(e){this._init(e)},_init:function(e){this._component=e,this._$element=e.$element(),this._$container=e._$container,this._$content=e._$content,this._direction=e.option("direction"),this._useSimulatedScrollbar=e.option("useSimulatedScrollbar"),this._showScrollbar=e.option("showScrollbar"),this.option=e.option.bind(e),this._createActionByOption=e._createActionByOption.bind(e),this._isLocked=e._isLocked.bind(e),this._isDirection=e._isDirection.bind(e),this._allowedDirection=e._allowedDirection.bind(e),this._getScrollOffset=e._getScrollOffset.bind(e),this._getMaxOffset=e._getMaxOffset.bind(e)},render:function(){this._renderPushBackOffset();var e=u.default.real(),t=e.platform;this._$element.addClass(f).addClass(f+"-"+t).toggleClass(g,!this._showScrollbar),this._showScrollbar&&this._useSimulatedScrollbar&&this._renderScrollbars()},updateBounds:s.noop,_renderPushBackOffset:function(){var e=this.option("pushBackValue");(e||this._component._lastPushBackValue)&&(this._$content.css({paddingTop:e,paddingBottom:e}),this._component._lastPushBackValue=e)},_renderScrollbars:function(){this._scrollbars={},this._hideScrollbarTimeout=0,this._$element.addClass(p),this._renderScrollbar(_),this._renderScrollbar(m)},_renderScrollbar:function(e){this._isDirection(e)&&(this._scrollbars[e]=new d.default((0,o.default)("<div>").appendTo(this._$element),{direction:e,expandable:this._component.option("scrollByThumb")}))},handleInit:s.noop,handleStart:function(){this._disablePushBack=!0},handleMove:function(e){return this._isLocked()?void(e.cancel=!0):void(this._allowedDirection()&&(e.originalEvent.isScrollingEvent=!0))},handleEnd:function(){this._disablePushBack=!1},handleCancel:s.noop,handleStop:s.noop,_eachScrollbar:function(e){e=e.bind(this),(0,l.each)(this._scrollbars||{},function(t,n){e(n,t)})},createActions:function(){this._scrollAction=this._createActionByOption("onScroll"),this._updateAction=this._createActionByOption("onUpdated")},_createActionArgs:function(){var e=this.location(),t=e.left,n=e.top;return{event:this._eventForUserAction,scrollOffset:this._getScrollOffset(),reachedLeft:this._isReachedLeft(t),reachedRight:this._isReachedRight(t),reachedTop:this._isDirection(_)?n>=0:void 0,reachedBottom:this._isDirection(_)?Math.abs(n)>=this._getMaxOffset().top-2*this.option("pushBackValue"):void 0}},_isReachedLeft:function(){return this._isDirection(m)?this.location().left>=0:void 0},_isReachedRight:function(){return this._isDirection(m)?Math.abs(this.location().left)>=this._getMaxOffset().left:void 0},handleScroll:function(e){return this._component._updateRtlConfig(),this._isScrollLocationChanged()?(this._eventForUserAction=e,this._moveScrollbars(),this._scrollAction(this._createActionArgs()),this._lastLocation=this.location(),void this._pushBackFromBoundary()):void e.stopImmediatePropagation()},_pushBackFromBoundary:function(){var e=this.option("pushBackValue");if(e&&!this._disablePushBack){var t=this._containerSize.height-this._contentSize.height,n=this._$container.scrollTop(),i=t+n-2*e;n?i||this._$container.scrollTop(e-t):this._$container.scrollTop(e)}},_isScrollLocationChanged:function(){var e=this.location(),t=this._lastLocation||{},n=t.top!==e.top,i=t.left!==e.left;return n||i},_moveScrollbars:function(){this._eachScrollbar(function(e){e.moveTo(this.location()),e.option("visible",!0)}),this._hideScrollbars()},_hideScrollbars:function(){clearTimeout(this._hideScrollbarTimeout),this._hideScrollbarTimeout=setTimeout(function(){this._eachScrollbar(function(e){e.option("visible",!1)})}.bind(this),v)},location:function(){return{left:-this._$container.scrollLeft(),top:this.option("pushBackValue")-this._$container.scrollTop()}},disabledChanged:s.noop,update:function(){this._update(),this._updateAction(this._createActionArgs())},_update:function(){this._updateDimensions(),this._updateScrollbars()},_updateDimensions:function(){this._containerSize={height:this._$container.height(),width:this._$container.width()},this._componentContentSize={height:this._component.$content().height(),width:this._component.$content().width()},this._contentSize={height:this._$content.height(),width:this._$content.width()},this._pushBackFromBoundary()},_updateScrollbars:function(){this._eachScrollbar(function(e,t){var n=t===_?"height":"width";e.option({containerSize:this._containerSize[n],contentSize:this._componentContentSize[n]}),e.update()})},_allowedDirections:function(){return{vertical:this._isDirection(_)&&this._contentSize.height>this._containerSize.height,horizontal:this._isDirection(m)&&this._contentSize.width>this._containerSize.width}},dispose:function(){var e=this._$element.get(0).className,t=new RegExp(f+"\\S*","g");t.test(e)&&this._$element.removeClass(e.match(t).join(" ")),a.default.off(this._$element,"."+h),a.default.off(this._$container,"."+h),this._removeScrollbars(),clearTimeout(this._hideScrollbarTimeout)},_removeScrollbars:function(){this._eachScrollbar(function(e){e.$element().remove()})},scrollBy:function(e){var t=this.location();this._$container.scrollTop(Math.round(-t.top-e.top+this.option("pushBackValue"))),this._$container.scrollLeft(Math.round(-t.left-e.left))},validate:function(e){return!this.option("disabled")&&((!(0,r.isDxMouseWheelEvent)(e)||!this._isScrolledInMaxDirection(e))&&!!this._allowedDirection())},_isScrolledInMaxDirection:function(e){var t,n=this._$container.get(0);return t=e.delta>0?e.shiftKey?!n.scrollLeft:!n.scrollTop:e.shiftKey?n.scrollLeft>=this._getMaxOffset().left:n.scrollTop>=this._getMaxOffset().top},getDirection:function(){return this._allowedDirection()},verticalOffset:function(){return this.option("pushBackValue")}}),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(5)),a=i(n(15)),r=n(8),s=i(n(163)),l=i(n(99)),u=n(120),c=i(n(13)),d=n(60),h=a.default.abstract,f=c.default.real(),p="dxscrollinit",g="dxscrollstart",_="dxscroll",m="dxscrollend",v="dxscrollstop",y="dxscrollcancel",b=a.default.inherit(function(){var e=(0,r.addNamespace)("scroll","dxScrollEmitter");return{ctor:function(t){this._element=t,this._locked=!1;var n=this;this._proxiedScroll=function(e){n._scroll(e)},o.default.on(this._element,e,this._proxiedScroll)},_scroll:h,check:function(e,t){this._locked&&t()},dispose:function(){o.default.off(this._element,e,this._proxiedScroll)}}}()),x=b.inherit(function(){return{ctor:function(e,t){this.callBase(e),this._timeout=t},_scroll:function(){this._prepare(),this._forget()},_prepare:function(){this._timer&&this._clearTimer(),this._locked=!0},_clearTimer:function(){clearTimeout(this._timer),this._locked=!1,this._timer=null},_forget:function(){var e=this;this._timer=setTimeout(function(){e._clearTimer()},this._timeout)},dispose:function(){this.callBase(),this._clearTimer()}}}()),w=x.inherit(function(){var e=400;return{ctor:function(t){this.callBase(t,e),this._lastWheelDirection=null},check:function(e,t){this._checkDirectionChanged(e),this.callBase(e,t)},_checkDirectionChanged:function(e){if(!(0,r.isDxMouseWheelEvent)(e))return void(this._lastWheelDirection=null);var t=e.shiftKey||!1,n=null!==this._lastWheelDirection&&t!==this._lastWheelDirection;this._lastWheelDirection=t,this._locked=this._locked&&!n}}}()),C=x.inherit(function(){var e=400;return{ctor:function(t){this.callBase(t,e)}}}());!function(){var e=f.ios&&(0,d.compare)(f.version,[8])>=0,t=f.android&&(0,d.compare)(f.version,[5])>=0;(e||t)&&(C=b.inherit(function(){return{_scroll:function(){this._locked=!0;var e=this;(0,u.cancelAnimationFrame)(this._scrollFrame),this._scrollFrame=(0,u.requestAnimationFrame)(function(){e._locked=!1})},check:function(e,t){(0,u.cancelAnimationFrame)(this._scrollFrame),(0,u.cancelAnimationFrame)(this._checkFrame);var n=this,i=this.callBase;this._checkFrame=(0,u.requestAnimationFrame)(function(){i.call(n,e,t),n._locked=!1})},dispose:function(){this.callBase(),(0,u.cancelAnimationFrame)(this._scrollFrame),(0,u.cancelAnimationFrame)(this._checkFrame)}}}()))}();var k=s.default.inherit(function(){var e=100,t=200,n=Math.round(1e3/60);return{ctor:function(e){this.callBase.apply(this,arguments),this.direction="both",this._pointerLocker=new C(e),this._wheelLocker=new w(e)},validate:function(){return!0},configure:function(e){e.scrollTarget&&(this._pointerLocker.dispose(),this._wheelLocker.dispose(),this._pointerLocker=new C(e.scrollTarget),this._wheelLocker=new w(e.scrollTarget)),this.callBase(e)},_init:function(e){this._wheelLocker.check(e,function(){(0,r.isDxMouseWheelEvent)(e)&&this._accept(e)}.bind(this)),this._pointerLocker.check(e,function(){var t=this.isNative&&(0,r.isMouseEvent)(e);(0,r.isDxMouseWheelEvent)(e)||t||this._accept(e)}.bind(this)),this._fireEvent(p,e),this._prevEventData=(0,r.eventData)(e)},move:function(e){this.callBase.apply(this,arguments),e.isScrollingEvent=this.isNative||e.isScrollingEvent},_start:function(e){this._savedEventData=(0,r.eventData)(e),this._fireEvent(g,e),this._prevEventData=(0,r.eventData)(e)},_move:function(e){var n=(0,r.eventData)(e);this._fireEvent(_,e,{delta:(0,r.eventDelta)(this._prevEventData,n)});var i=(0,r.eventDelta)(this._savedEventData,n);i.time>t&&(this._savedEventData=this._prevEventData),this._prevEventData=(0,r.eventData)(e)},_end:function(t){var i=(0,r.eventDelta)(this._prevEventData,(0,r.eventData)(t)),o={x:0,y:0};if(!(0,r.isDxMouseWheelEvent)(t)&&i.time<e){var a=(0,r.eventDelta)(this._savedEventData,this._prevEventData),s=n/a.time;o={x:a.x*s,y:a.y*s}}this._fireEvent(m,t,{velocity:o})},_stop:function(e){this._fireEvent(v,e)},cancel:function(e){this.callBase.apply(this,arguments),this._fireEvent(y,e)},dispose:function(){this.callBase.apply(this,arguments),this._pointerLocker.dispose(),this._wheelLocker.dispose()},_clearSelection:function(){if(!this.isNative)return this.callBase.apply(this,arguments)},_toggleGestureCover:function(){if(!this.isNative)return this.callBase.apply(this,arguments)}}}());(0,l.default)({emitter:k,events:[p,g,_,m,v,y]});var S={init:p,start:g,move:_,end:m,stop:v,cancel:y};t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var a=i(n(2)),r=i(n(5)),s=i(n(9)),l=n(0),u=i(n(215)),c=n(18),d=n(7),h=n(24),f=n(30),p=i(n(43)),g=n(6),_="dxSortable",m="placeholder",v="clone",y=function(e){return(0,a.default)(e).is(":visible")},b=function(e,t){var n,i;if(e){var o=(null===(n=t.to)||void 0===n?void 0:n.left)||0,a=(null===(i=t.to)||void 0===i?void 0:i.top)||0;e.style.transform="translate(".concat(o,"px,").concat(a,"px)"),e.style.transition=p.default.off?"":"transform ".concat(t.duration,"ms ").concat(t.easing)}},x=function(e){e&&(e.style.transform="",e.style.transition="")},w=u.default.inherit({_init:function(){this.callBase(),this._sourceScrollHandler=this._handleSourceScroll.bind(this),this._sourceScrollableInfo=null},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{clone:!0,filter:"> *",itemOrientation:"vertical",dropFeedbackMode:"push",allowDropInsideItem:!1,allowReordering:!0,moveItemOnDrop:!1,onDragChange:null,onAdd:null,onRemove:null,onReorder:null,onPlaceholderPrepared:null,animation:{type:"slide",duration:300,easing:"ease"},fromIndex:null,toIndex:null,dropInsideItem:!1,itemPoints:null,fromIndexOffset:0,offset:0,autoUpdate:!1,draggableElementSize:0})},reset:function(){this.option({dropInsideItem:!1,toIndex:null,fromIndex:null,itemPoints:null,fromIndexOffset:0,draggableElementSize:0}),this._$placeholderElement&&this._$placeholderElement.remove(),this._$placeholderElement=null,!this._isIndicateMode()&&this._$modifiedItem&&(this._$modifiedItem.css("marginBottom",this._modifiedItemMargin),this._$modifiedItem=null)},_getPrevVisibleItem:function(e,t){return e.slice(0,t).reverse().filter(y)[0]},_dragStartHandler:function(e){if(this.callBase.apply(this,arguments),e.cancel!==!0){var t=this._getSourceElement();this._updateItemPoints(),this._subscribeToSourceScroll(e),this.option("fromIndex",this._getElementIndex(t)),this.option("fromIndexOffset",this.option("offset"))}},_dragEndHandler:function(){this.callBase.apply(this,arguments),this._unsubscribeFromSourceScroll()},_subscribeToSourceScroll:function(e){var t=this._getScrollable((0,a.default)(e.target));t&&(this._sourceScrollableInfo={element:t,scrollLeft:t.scrollLeft(),scrollTop:t.scrollTop()},r.default.on(t,"scroll",this._sourceScrollHandler))},_unsubscribeFromSourceScroll:function(){this._sourceScrollableInfo&&(r.default.off(this._sourceScrollableInfo.element,"scroll",this._sourceScrollHandler),this._sourceScrollableInfo=null)},_handleSourceScroll:function(e){var t=this,n=this._sourceScrollableInfo;n&&["scrollLeft","scrollTop"].forEach(function(i){if(e.target[i]!==n[i]){var o=e.target[i]-n[i];t._correctItemPoints(o),n[i]=e.target[i]}})},_dragEnterHandler:function(){if(this.callBase.apply(this,arguments),this!==this._getSourceDraggable()&&(this._updateItemPoints(),this.option("fromIndex",-1),!this._isIndicateMode())){var e=this.option("itemPoints"),t=e[e.length-1];if(t){var n=this.$element(),i=this._getSourceElement(),o=this._isVerticalOrientation(),r=o?i.outerHeight(!0):i.outerWidth(!0),s=n.get(0)[o?"scrollHeight":"scrollWidth"],l=n.get(0)[o?"scrollTop":"scrollLeft"],u=o?"top":"left",c=t[u],d=n.offset()[u],h=d+s-l-c;if(h<r&&o){var f=this._getItems(),p=(0,a.default)(this._getPrevVisibleItem(f));this._$modifiedItem=p,this._modifiedItemMargin=p.get(0).style.marginBottom,p.css("marginBottom",r-h);var g=p.closest(".dx-sortable"),_=g.data("dxScrollable")||g.data("dxScrollView");_&&_.update()}}}},dragEnter:function(){this!==this._getTargetDraggable()&&this.option("toIndex",-1)},dragLeave:function(){this!==this._getTargetDraggable()&&this.option("toIndex",this.option("fromIndex"))},_isInsideTargetDraggable:function(e){var t=this._getTargetDraggable().$element(),n=this._getScrollable(t);if(n){var i=n.offset(),o=i.top+n.height()>=e.pageY&&i.top<=e.pageY,a=i.left+n.width()>=e.pageX&&i.left<=e.pageX;return o&&a}return!0},dragEnd:function(e){var t=this._getSourceElement(),n=this._getSourceDraggable(),i=n.NAME!==this.NAME,o=this.option("toIndex"),a=this._isInsideTargetDraggable(e.event);if(null!==o&&o>=0&&a){var r,s;if(n!==this&&(r=this._fireAddEvent(e),r||(s=this._fireRemoveEvent(e))),i&&(0,f.resetPosition)(t),this.option("moveItemOnDrop")&&!r&&this._moveItem(t,o,s),n===this)return this._fireReorderEvent(e)}return(new g.Deferred).resolve()},dragMove:function(e){var t=this.option("itemPoints");if(t){for(var n,i=this._isVerticalOrientation(),o=i?"top":"left",a=i?e.pageY:e.pageX,r=this.option("rtlEnabled"),s=t.length-1;s>=0;s--){var l=t[s+1]&&(t[s][o]+t[s+1][o])/2;if(!(!i&&r?a>l:l>a)&&void 0!==l)break;n=t[s]}n&&(this._updatePlaceholderPosition(e,n),this._verticalScrollHelper.isScrolling()&&this._isIndicateMode()&&this._movePlaceholder())}},_isIndicateMode:function(){return"indicate"===this.option("dropFeedbackMode")||this.option("allowDropInsideItem")},_createPlaceholder:function(){var e;return this._isIndicateMode()&&(e=(0,a.default)("<div>").addClass(this._addWidgetPrefix(m)).insertBefore(this._getSourceDraggable()._$dragElement)),this._$placeholderElement=e,e},_getItems:function(){var e=this._getItemsSelector();return this._$content().find(e).not("."+this._addWidgetPrefix(m)).not("."+this._addWidgetPrefix(v)).toArray()},_allowReordering:function(){var e=this._getSourceDraggable(),t=this._getTargetDraggable();return e!==t||this.option("allowReordering")},_isValidPoint:function(e,t,n){var i=this.option("allowDropInsideItem"),o=n||this._allowReordering();return!!(o||0===e&&i)&&(!this._isIndicateMode()||(t===-1||e!==t&&(n||e!==t+1)))},_getItemPoints:function(){var e,t,n,i=this,o=[],r=i.option("rtlEnabled"),s=i._isVerticalOrientation(),u=i._getItems(),c=u.filter(y),d=c.length,h=this._getDraggableElement(),f=c.indexOf(h.get(0));if(d){for(var p=0;p<=d;p++){var g=!s&&r^p===d,_=s&&p===d;p<d&&(e=(0,a.default)(c[p]),t=e.offset(),n=e.outerWidth()),o.push({dropInsideItem:!1,left:t.left+(g?n:0),top:t.top+(_?o[p-1].height:0),index:p===d?u.length:u.indexOf(e.get(0)),$item:e,width:e.outerWidth(),height:e.outerHeight(),isValid:i._isValidPoint(p,f)})}if(this.option("allowDropInsideItem")){var m=o;o=[];for(var v=0;v<m.length;v++)o.push(m[v]),m[v+1]&&o.push((0,l.extend)({},m[v],{dropInsideItem:!0,top:Math.floor((m[v].top+m[v+1].top)/2),left:Math.floor((m[v].left+m[v+1].left)/2),isValid:this._isValidPoint(v,f,!0)}))}}else o.push({dropInsideItem:!1,index:0,isValid:!0});return o},_updateItemPoints:function(e){(e||this.option("autoUpdate")||!this.option("itemPoints"))&&this.option("itemPoints",this._getItemPoints())},_correctItemPoints:function(e){var t=this.option("itemPoints");if(e&&t&&!this.option("autoUpdate")){var n=this._isVerticalOrientation(),i=n?"top":"left";t.forEach(function(t){t[i]-=e})}},_getElementIndex:function(e){return this._getItems().indexOf(e.get(0))},_getDragTemplateArgs:function(e){var t=this.callBase.apply(this,arguments);return t.model.fromIndex=this._getElementIndex(e),t},_togglePlaceholder:function(e){this._$placeholderElement&&this._$placeholderElement.toggle(e)},_isVerticalOrientation:function(){return"vertical"===this.option("itemOrientation")},_normalizeToIndex:function(e,t){var n=this._getSourceDraggable()!==this._getTargetDraggable(),i=this.option("fromIndex");return null===e?i:Math.max(n||i>=e||t?e:e-1,0)},_updatePlaceholderPosition:function(e,t){var n=this._getSourceDraggable(),i=this._normalizeToIndex(t.index,t.dropInsideItem),o=(0,l.extend)(this._getEventArgs(e),{toIndex:i,dropInsideItem:t.dropInsideItem});return t.isValid&&this._getAction("onDragChange")(o),o.cancel||!t.isValid?void(t.isValid||this.option({dropInsideItem:!1,toIndex:null})):(this.option({dropInsideItem:t.dropInsideItem,toIndex:t.index}),this._getAction("onPlaceholderPrepared")((0,l.extend)(this._getEventArgs(e),{placeholderElement:(0,c.getPublicElement)(this._$placeholderElement),dragElement:(0,c.getPublicElement)(n._$dragElement)})),void this._updateItemPoints())},_makeWidthCorrection:function(e,t){if(this._$scrollable=this._getScrollable(e),this._$scrollable&&this._$scrollable.width()<t){var n=this._$scrollable.width(),i=e.offset().left-this._$scrollable.offset().left,o=n-e.outerWidth()-i;t=i>0?n-i:o>0?n-o:n}return t},_updatePlaceholderSizes:function(e,t){var n=this,i=n.option("dropInsideItem"),o=(0,a.default)(t),r=n._isVerticalOrientation(),s="",l="";e.toggleClass(n._addWidgetPrefix("placeholder-inside"),i),(r||i)&&(s=o.outerWidth()),r&&!i||(l=o.outerHeight()),s=n._makeWidthCorrection(o,s),e.css({width:s,height:l})},_moveItem:function(e,t,n){var i,o=this._getItems(),a=o[t],r=this._getSourceDraggable();n&&(e=e.clone(),r._toggleDragSourceClass(!1,e)),a||(i=o[t-1]),this._moveItemCore(e,a,i)},_moveItemCore:function(e,t,n){t||n?n?e.insertAfter((0,a.default)(n)):e.insertBefore((0,a.default)(t)):e.appendTo(this.$element())},_getDragStartArgs:function(e,t){return(0,l.extend)(this.callBase.apply(this,arguments),{fromIndex:this._getElementIndex(t)})},_getEventArgs:function(e){var t=this._getSourceDraggable(),n=this._getTargetDraggable(),i=n.option("dropInsideItem");return(0,l.extend)(this.callBase.apply(this,arguments),{fromIndex:t.option("fromIndex"),toIndex:this._normalizeToIndex(n.option("toIndex"),i),dropInsideItem:i})},_optionChanged:function(e){var t=this,n=e.name;switch(n){case"onDragChange":case"onPlaceholderPrepared":case"onAdd":case"onRemove":case"onReorder":this["_"+n+"Action"]=this._createActionByOption(n);break;case"itemOrientation":case"allowDropInsideItem":case"moveItemOnDrop":case"dropFeedbackMode":case"itemPoints":case"animation":case"allowReordering":case"fromIndexOffset":case"offset":case"draggableElementSize":case"autoUpdate":break;case"fromIndex":[!1,!0].forEach(function(n){var i=n?e.value:e.previousValue;if(null!==i){var o=(0,a.default)(t._getItems()[i]);t._toggleDragSourceClass(n,o)}});break;case"dropInsideItem":this._optionChangedDropInsideItem(e);break;case"toIndex":this._optionChangedToIndex(e);break;default:this.callBase(e)}},_optionChangedDropInsideItem:function(){this._isIndicateMode()&&this._$placeholderElement&&this._movePlaceholder()},_isPositionVisible:function(e){var t,n=this.$element();if("hidden"!==n.css("overflow")?t=n.get(0):n.parents().each(function(){
if("visible"!==(0,a.default)(this).css("overflow"))return t=this,!1}),t){var i=(0,h.getBoundingRect)(t),o=this._isVerticalOrientation(),r=o?"top":"left",s=o?"bottom":"right",l=(0,d.getWindow)(),u=o?l.pageYOffset:l.pageXOffset;if(e[r]<i[r]+u||e[r]>i[s]+u)return!1}return!0},_optionChangedToIndex:function(e){var t=e.value;if(this._isIndicateMode()){var n=null!==t&&t>=0;this._togglePlaceholder(n),n&&this._movePlaceholder()}else this._moveItems(e.previousValue,e.value,e.fullUpdate)},update:function(){if(null!==this.option("fromIndex")||null!==this.option("toIndex")){this._updateItemPoints(!0),this._updateDragSourceClass();var e=this.option("toIndex");this._optionChangedToIndex({value:e,fullUpdate:!0})}},_updateDragSourceClass:function(){var e=this._getActualFromIndex(),t=(0,a.default)(this._getItems()[e]);t.length&&(this._$sourceElement=t,this._toggleDragSourceClass(!0,t))},_makeLeftCorrection:function(e,t){var n=this,i=n._$scrollable;return i&&n._isVerticalOrientation()&&i.scrollLeft()>t&&(e+=i.scrollLeft()-t),e},_movePlaceholder:function(){var e=this,t=e._$placeholderElement||e._createPlaceholder(),n=e._getItems(),i=e.option("toIndex"),o=e._isVerticalOrientation(),r=this.option("rtlEnabled"),s=e.option("dropInsideItem"),l=null,u=0,c=n[i];if(c){var d=(0,a.default)(c);l=d.offset(),u=parseFloat(d.css("marginLeft")),o||!r||s||(l.left+=d.outerWidth(!0))}else{var h=c=this._getPrevVisibleItem(n,i);h&&(l=(0,a.default)(h).offset(),o?l.top+=(0,a.default)(h).outerHeight(!0):r||(l.left+=(0,a.default)(h).outerWidth(!0)))}e._updatePlaceholderSizes(t,c),l&&!e._isPositionVisible(l)&&(l=null),l&&(l.left=e._makeLeftCorrection(l.left,u),e._move(l,t)),t.toggle(!!l)},_getPositions:function(e,t,n,i){for(var o=[],a=0;a<e.length;a++){var r=0;null!==i&&null!==n?(n===-1?a>=i&&(r=t):i===-1?a>n&&(r=-t):n<i?a>n&&a<i&&(r=-t):n>i&&a>=i&&a<n&&(r=t),o.push(r)):o.push(r)}return o},_getDraggableElementSize:function(e){var t=this._getDraggableElement(),n=this.option("draggableElementSize");return n||(n=e?(t.outerHeight()+t.outerHeight(!0))/2:(t.outerWidth()+t.outerWidth(!0))/2,this.option("autoUpdate")||this.option("draggableElementSize",n)),n},_getActualFromIndex:function(){var e=this.option(),t=e.fromIndex,n=e.fromIndexOffset,i=e.offset;return null==t?null:t+n-i},_moveItems:function(e,t,n){for(var i=this._getActualFromIndex(),a=this._isVerticalOrientation(),r=a?"top":"left",s=this._getDraggableElementSize(a),u=this._getItems(),c=this._getPositions(u,s,i,e),d=this._getPositions(u,s,i,t),h=this.option("animation"),f=this.option("rtlEnabled"),p=0;p<u.length;p++){var g=u[p],_=c[p],m=d[p];null===t||null===i?x(g):(_!==m||n&&m)&&b(g,(0,l.extend)({},h,{to:o({},r,!a&&f?-m:m)}))}},_toggleDragSourceClass:function(e,t){var n=t||this._$sourceElement;this.callBase.apply(this,arguments),this._isIndicateMode()||n&&n.toggleClass(this._addWidgetPrefix("source-hidden"),e)},_dispose:function(){this.reset(),this.callBase()},_fireAddEvent:function(e){var t=this._getEventArgs(e);return this._getAction("onAdd")(t),t.cancel},_fireRemoveEvent:function(e){var t=this._getSourceDraggable(),n=this._getEventArgs(e);return t._getAction("onRemove")(n),n.cancel},_fireReorderEvent:function(e){var t=this._getEventArgs(e);return this._getAction("onReorder")(t),t.promise||(new g.Deferred).resolve()}});(0,s.default)(_,w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(0),r=i(n(10)),s=i(n(104)),l=i(n(17)),u=n(6),c={_getDefaultOptions:function(){return(0,a.extend)(this.callBase(),{searchMode:"",searchExpr:null,searchValue:"",searchEnabled:!1,searchEditorOptions:{}})},_initMarkup:function(){this._renderSearch(),this.callBase()},_renderSearch:function(){var e=this.$element(),t=this.option("searchEnabled"),n=this._addWidgetPrefix("search"),i=this._addWidgetPrefix("with-search");if(!t)return e.removeClass(i),void this._removeSearchBox();var a=this._getSearchEditorOptions();this._searchEditor?this._searchEditor.option(a):(e.addClass(i),this._$searchEditorElement=(0,o.default)("<div>").addClass(n).prependTo(e),this._searchEditor=this._createComponent(this._$searchEditorElement,s.default,a))},_removeSearchBox:function(){this._$searchEditorElement&&this._$searchEditorElement.remove(),delete this._$searchEditorElement,delete this._searchEditor},_getSearchEditorOptions:function(){var e=this,t=e.option("searchEditorOptions"),n=r.default.format("Search");return(0,a.extend)({mode:"search",placeholder:n,tabIndex:e.option("tabIndex"),value:e.option("searchValue"),valueChangeEvent:"input",inputAttr:{"aria-label":n},onValueChanged:function(t){var n=e.option("searchTimeout");e._valueChangeDeferred=new u.Deferred,clearTimeout(e._valueChangeTimeout),e._valueChangeDeferred.done(function(){this.option("searchValue",t.value)}.bind(e)),t.event&&"input"===t.event.type&&n?e._valueChangeTimeout=setTimeout(function(){e._valueChangeDeferred.resolve()},n):e._valueChangeDeferred.resolve()}},t)},_getAriaTarget:function(){return this.option("searchEnabled")?this._itemContainer(!0):this.$element()},_focusTarget:function(){return this.option("searchEnabled")?this._itemContainer(!0):this.callBase()},_updateFocusState:function(e,t){this.option("searchEnabled")&&this._toggleFocusClass(t,this.$element()),this.callBase(e,t)},getOperationBySearchMode:function(e){return"equals"===e?"=":e},_cleanAria:function(e){this.setAria({role:null,activedescendant:null},e),e.attr("tabIndex",null)},_optionChanged:function(e){switch(e.name){case"searchEnabled":case"searchEditorOptions":this._cleanAria(this.option("searchEnabled")?this.$element():this._itemContainer()),this._invalidate();break;case"searchExpr":case"searchMode":case"searchValue":if(!this._dataSource)return void l.default.log("W1009");"searchMode"===e.name?this._dataSource.searchOperation(this.getOperationBySearchMode(e.value)):this._dataSource[e.name](e.value),this._dataSource.load();break;case"searchTimeout":break;default:this.callBase(e)}},focus:function(){return!this.option("focusedElement")&&this.option("searchEnabled")?void(this._searchEditor&&this._searchEditor.focus()):void this.callBase()},_refresh:function(){this._valueChangeDeferred&&this._valueChangeDeferred.resolve(),this.callBase()}};t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(101)),r=n(19),s=n(4),l=n(1),u=n(0),c=i(n(94)),d=n(69),h=i(n(81)),f=n(6),p=(0,u.extend)({},c.default,{_dataExpressionDefaultOptions:function(){return{items:[],dataSource:null,itemTemplate:"item",value:null,valueExpr:"this",displayExpr:void 0}},_initDataExpressions:function(){this._compileValueGetter(),this._compileDisplayGetter(),this._initDynamicTemplates(),this._initDataSource(),this._itemsToDataSource()},_itemsToDataSource:function(){this.option("dataSource")||(this._dataSource=new d.DataSource({store:new h.default(this.option("items")),pageSize:0}))},_compileDisplayGetter:function(){this._displayGetter=(0,r.compileGetter)(this._displayGetterExpr())},_displayGetterExpr:function(){return this.option("displayExpr")},_compileValueGetter:function(){this._valueGetter=(0,r.compileGetter)(this._valueGetterExpr())},_valueGetterExpr:function(){return this.option("valueExpr")||"this"},_loadValue:function(e){var t=new f.Deferred;return e=this._unwrappedValue(e),(0,l.isDefined)(e)?(this._loadSingle(this._valueGetterExpr(),e).done(function(n){this._isValueEquals(this._valueGetter(n),e)?t.resolve(n):t.reject()}.bind(this)).fail(function(){t.reject()}),t.promise()):t.reject().promise()},_getCurrentValue:function(){return this.option("value")},_unwrappedValue:function(e){return e=(0,l.isDefined)(e)?e:this._getCurrentValue(),e&&this._dataSource&&"this"===this._valueGetterExpr()&&(e=this._getItemKey(e)),a.default.unwrap(e)},_getItemKey:function(e){var t=this._dataSource.key();if(Array.isArray(t)){for(var n={},i=0,a=t.length;i<a;i++)n[t[i]]=e[t[i]];return n}return t&&"object"===o(e)&&(e=e[t]),e},_isValueEquals:function(e,t){var n=this._dataSource&&this._dataSource.key(),i=this._compareValues(e,t);return!i&&n&&(0,l.isDefined)(e)&&(0,l.isDefined)(t)&&(i=Array.isArray(n)?this._compareByCompositeKey(e,t,n):this._compareByKey(e,t,n)),i},_compareByCompositeKey:function(e,t,n){var i=l.isObject;if(!i(e)||!i(t))return!1;for(var o=0,a=n.length;o<a;o++)if(e[n[o]]!==t[n[o]])return!1;return!0},_compareByKey:function(e,t,n){var i=a.default.unwrap,o=(0,s.ensureDefined)(i(e[n]),e),r=(0,s.ensureDefined)(i(t[n]),t);return this._compareValues(o,r)},_compareValues:function(e,t){return(0,r.toComparable)(e,!0)===(0,r.toComparable)(t,!0)},_initDynamicTemplates:s.noop,_setCollectionWidgetItemTemplate:function(){this._initDynamicTemplates(),this._setCollectionWidgetOption("itemTemplate",this.option("itemTemplate"))},_getCollectionKeyExpr:function(){var e=this.option("valueExpr"),t=(0,l.isString)(e)&&"this"!==e||(0,l.isFunction)(e);return t?e:null},_dataExpressionOptionChanged:function(e){switch(e.name){case"items":this._itemsToDataSource(),this._setCollectionWidgetOption("items");break;case"dataSource":this._initDataSource();break;case"itemTemplate":this._setCollectionWidgetItemTemplate();break;case"valueExpr":this._compileValueGetter();break;case"displayExpr":this._compileDisplayGetter(),this._initDynamicTemplates(),this._setCollectionWidgetOption("displayExpr")}}}),g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(16)),r=i(n(38)),s=i(n(58)),l=i(n(9)),u=n(0),c=n(1),d=n(71),h="dx-buttongroup",f=h+"-wrapper",p=h+"-item",g=h+"-first-item",_=h+"-last-item",m=p+"-has-width",v="dx-shape-standard",y=s.default.inherit({_initTemplates:function(){var e=this;this.callBase(),this._templateManager.addDefaultTemplates({item:new d.BindableTemplate(function(t,n,i){e._prepareItemStyles(t),e._createComponent(t,r.default,(0,u.extend)({},i,n,e._getBasicButtonOptions(),{_templateData:i,template:i.template||e.option("buttonTemplate")}))},["text","type","icon","disabled","visible","hint"],this.option("integrationOptions.watchMethod"))})},_getBasicButtonOptions:function(){return{focusStateEnabled:!1,onClick:null,hoverStateEnabled:this.option("hoverStateEnabled"),activeStateEnabled:this.option("activeStateEnabled"),stylingMode:this.option("stylingMode")}},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{itemTemplateProperty:null})},_prepareItemStyles:function(e){var t=e.data("dxItemIndex");0===t&&e.addClass(g);var n=this.option("items");n&&t===n.length-1&&e.addClass(_),e.addClass(v)},_renderItemContent:function(e){return e.container=(0,o.default)(e.container).parent(),this.callBase(e)},_renderItemContentByNode:function(e,t){return e.container=(0,o.default)(e.container.children().first()),this.callBase(e,t)},_focusTarget:function(){return this.$element().parent()},_keyboardEventBindingTarget:function(){return this._focusTarget()},_refreshContent:function(){this._prepareContent(),this._renderContent()},_itemClass:function(){return p},_itemSelectHandler:function(e){"single"===this.option("selectionMode")&&this.isItemSelected(e.currentTarget)||this.callBase(e)}}),b=a.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{hoverStateEnabled:!0,focusStateEnabled:!0,selectionMode:"single",selectedItems:[],selectedItemKeys:[],stylingMode:"contained",keyExpr:"text",items:[],buttonTemplate:"content",onSelectionChanged:null,onItemClick:null})},_init:function(){this.callBase(),this._createItemClickAction()},_createItemClickAction:function(){this._itemClickAction=this._createActionByOption("onItemClick")},_initMarkup:function(){this.setAria("role","group"),this.$element().addClass(h),this._renderButtons(),this._syncSelectionOptions(),this.callBase()},_fireSelectionChangeEvent:function(e,t){this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})({addedItems:e,removedItems:t})},_renderButtons:function(){var e=this,t=(0,o.default)("<div>").addClass(f).appendTo(this.$element()),n=this.option("selectedItems"),i={selectionMode:this.option("selectionMode"),items:this.option("items"),keyExpr:this.option("keyExpr"),buttonTemplate:this.option("buttonTemplate"),scrollingEnabled:!1,selectedItemKeys:this.option("selectedItemKeys"),focusStateEnabled:this.option("focusStateEnabled"),hoverStateEnabled:this.option("hoverStateEnabled"),activeStateEnabled:this.option("activeStateEnabled"),stylingMode:this.option("stylingMode"),accessKey:this.option("accessKey"),tabIndex:this.option("tabIndex"),noDataText:"",selectionRequired:!1,onItemRendered:function(t){var n=e.option("width");(0,c.isDefined)(n)&&(0,o.default)(t.itemElement).addClass(m)},onSelectionChanged:function(t){e._syncSelectionOptions(),e._fireSelectionChangeEvent(t.addedItems,t.removedItems)},onItemClick:function(t){e._itemClickAction(t)}};(0,c.isDefined)(n)&&n.length&&(i.selectedItems=n),this._buttonsCollection=this._createComponent(t,y,i)},_syncSelectionOptions:function(){this._setOptionWithoutOptionChange("selectedItems",this._buttonsCollection.option("selectedItems")),this._setOptionWithoutOptionChange("selectedItemKeys",this._buttonsCollection.option("selectedItemKeys"))},_optionChanged:function(e){switch(e.name){case"stylingMode":case"selectionMode":case"keyExpr":case"buttonTemplate":case"items":case"activeStateEnabled":case"focusStateEnabled":case"hoverStateEnabled":case"tabIndex":this._invalidate();break;case"selectedItemKeys":case"selectedItems":this._buttonsCollection.option(e.name,e.value);break;case"onItemClick":this._createItemClickAction();break;case"onSelectionChanged":break;case"width":this.callBase(e),this._buttonsCollection.itemElements().toggleClass(m,!!e.value);break;default:this.callBase(e)}}});(0,l.default)("dxButtonGroup",b);var x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(610)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(1),s=n(18),l=i(n(9)),u=n(0),c=n(24),d=i(n(16)),h=n(93),f=n(7),p=i(n(633)),g=i(n(634)),_=i(n(635)),m=n(194),v=n(21),y=i(n(43)),b=n(6),x=n(55),w="dx-drawer",C="dx-drawer-wrapper",k="dx-drawer-panel-content",S="dx-drawer-content",D="dx-drawer-shader",I="dx-state-invisible",T="dx-drawer-opened",O="content",E="panel",A=d.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{position:"left",opened:!1,minSize:null,maxSize:null,shading:!1,template:E,openedStateMode:"shrink",revealMode:"slide",animationEnabled:!0,animationDuration:400,closeOnOutsideClick:!1,contentTemplate:O,target:void 0})},_setDeprecatedOptions:function(){this.callBase(),(0,u.extend)(this._deprecatedOptions,{target:{since:"20.1",message:"Functionality associated with this option is not intended for the Drawer widget."}})},_init:function(){this.callBase(),this._initStrategy(),this.$element().addClass(w),this._animations=[],this._whenAnimationCompleted=void 0,this._whenPanelContentRendered=void 0,this._whenPanelContentRefreshed=void 0,this._$wrapper=(0,o.default)("<div>").addClass(C),this._$viewContentWrapper=(0,o.default)("<div>").addClass(S),this._$wrapper.append(this._$viewContentWrapper),this.$element().append(this._$wrapper)},_initStrategy:function(){switch(this.option("openedStateMode")){case"push":this._strategy=new p.default(this);break;case"shrink":this._strategy=new g.default(this);break;case"overlap":this._strategy=new _.default(this);break;default:this._strategy=new p.default(this)}},_getAnonymousTemplateName:function(){return O},_initTemplates:function(){var e={};e[E]=new h.EmptyTemplate,e[O]=new h.EmptyTemplate,this._templateManager.addDefaultTemplates(e),this.callBase()},_viewContentWrapperClickHandler:function(e){var t=this.option("closeOnOutsideClick");(0,r.isFunction)(t)&&(t=t(e)),t&&this.option("opened")&&(this.stopAnimations(),this.option("shading")&&e.preventDefault(),this.hide())},_initMarkup:function(){this.callBase(),this._toggleOpenedStateClass(this.option("opened")),this._renderPanelContentWrapper(),this._refreshOpenedStateModeClass(),this._refreshRevealModeClass(),this._renderShader(),this._whenPanelContentRendered=new b.Deferred,this._strategy.renderPanelContent(this._whenPanelContentRendered),this._strategy.onPanelContentRendered(),this._renderViewContent(),a.default.off(this._$viewContentWrapper,v.name),a.default.on(this._$viewContentWrapper,v.name,this._viewContentWrapperClickHandler.bind(this)),this._refreshPositionClass(),this._refreshWrapperChildrenOrder()},_render:function(){var e=this;this._initMinMaxSize(),this.callBase(),this._whenPanelContentRendered.always(function(){e._initMinMaxSize(),e._strategy.refreshPanelElementSize("slide"===e.option("revealMode")||!e.isHorizontalDirection()),e._renderPosition(e.option("opened"),!1),e._removePanelManualPosition()})},_removePanelManualPosition:function(){this._$panelContentWrapper.attr("manualposition")&&(this._$panelContentWrapper.removeAttr("manualPosition"),this._$panelContentWrapper.css({position:"",top:"",left:"",right:"",bottom:""}))},_renderPanelContentWrapper:function(){this._$panelContentWrapper=(0,o.default)("<div>").addClass(k);var e=this.calcTargetPosition();"push"===this.option("openedStateMode")&&["top","bottom"].indexOf(e)>-1&&this._$panelContentWrapper.addClass(k+"-push-top-or-bottom"),"overlap"===this.option("openedStateMode")||this.option("opened")||this.option("minSize")||(this._$panelContentWrapper.attr("manualposition",!0),this._$panelContentWrapper.css({position:"absolute",top:"-10000px",left:"-10000px",right:"auto",bottom:"auto"})),this._$wrapper.append(this._$panelContentWrapper)},_refreshOpenedStateModeClass:function(e){e&&this.$element().removeClass(w+"-"+e),this.$element().addClass(w+"-"+this.option("openedStateMode"))},_refreshPositionClass:function(e){e&&this.$element().removeClass(w+"-"+e),this.$element().addClass(w+"-"+this.calcTargetPosition())},_refreshWrapperChildrenOrder:function(){var e=this.calcTargetPosition();this._strategy.isViewContentFirst(e,this.option("rtlEnabled"))?this._$wrapper.prepend(this._$viewContentWrapper):this._$wrapper.prepend(this._$panelContentWrapper)},_refreshRevealModeClass:function(e){e&&this.$element().removeClass(w+"-"+e),this.$element().addClass(w+"-"+this.option("revealMode"))},_renderViewContent:function(){var e=this.option("contentTemplate"),t=this._getTemplate(e);if(t){var n=t.render({container:this.viewContent(),noModel:!0,transclude:this._templateManager.anonymousTemplateName===e});n.hasClass("ng-scope")&&(0,o.default)(this._$viewContentWrapper).children().not(".".concat(D)).replaceWith(n)}},_renderShader:function(){this._$shader=this._$shader||(0,o.default)("<div>").addClass(D),this._$shader.appendTo(this.viewContent()),this._toggleShaderVisibility(this.option("opened"))},_initSize:function(){this._initMinMaxSize()},_initMinMaxSize:function(){var e=this.isHorizontalDirection()?this.getRealPanelWidth():this.getRealPanelHeight();this._maxSize=this.option("maxSize")||e,this._minSize=this.option("minSize")||0},calcTargetPosition:function(){var e=this.option("position"),t=this.option("rtlEnabled"),n=e;return"before"===e?n=t?"right":"left":"after"===e&&(n=t?"left":"right"),n},getOverlayTarget:function(){return this._options.silent("target")||this._$wrapper},getOverlay:function(){return this._overlay},getMaxSize:function(){return this._maxSize},getMinSize:function(){return this._minSize},getRealPanelWidth:function(){return(0,f.hasWindow)()?(0,r.isDefined)(this.option("templateSize"))?this.option("templateSize"):(0,c.getBoundingRect)(this._getPanelTemplateElement()).width:0},getRealPanelHeight:function(){return(0,f.hasWindow)()?(0,r.isDefined)(this.option("templateSize"))?this.option("templateSize"):(0,c.getBoundingRect)(this._getPanelTemplateElement()).height:0},_getPanelTemplateElement:function(){var e=this._strategy.getPanelContent(),t=e;return e.children().length&&(t=e.children().eq(0),e.hasClass("dx-overlay-content")&&t.hasClass("dx-template-wrapper")&&t.children().length&&(t=t.children().eq(0))),t.get(0)},getElementHeight:function(e){var t=e.children();return t.length?(0,c.getBoundingRect)(t.eq(0).get(0)).height:(0,c.getBoundingRect)(e.get(0)).height},isHorizontalDirection:function(){var e=this.calcTargetPosition();return"left"===e||"right"===e},stopAnimations:function(e){y.default.stop(this._$shader,e),y.default.stop((0,o.default)(this.content()),e),y.default.stop((0,o.default)(this.viewContent()),e);var t=this.getOverlay();t&&y.default.stop((0,o.default)(t.$content()),e)},setZIndex:function(e){this._$shader.css("zIndex",e-1),this._$panelContentWrapper.css("zIndex",e)},resizeContent:function(){this.resizeViewContent},resizeViewContent:function(){(0,x.triggerResizeEvent)(this.viewContent())},_isInvertedPosition:function(){var e=this.calcTargetPosition();return"right"===e||"bottom"===e},_renderPosition:function(e,t,n){this.stopAnimations(n),this._animations=[],(0,f.hasWindow)()&&(t=(0,r.isDefined)(t)?t&&this.option("animationEnabled"):this.option("animationEnabled"),e&&this._toggleShaderVisibility(e),this._strategy.renderPosition(e,t),this._strategy.renderShaderVisibility(e,t,this.option("animationDuration")))},_animationCompleteHandler:function(){this.resizeViewContent(),this._whenAnimationCompleted&&(this._whenAnimationCompleted.resolve(),this._animations=[])},_getPositionCorrection:function(){return this._isInvertedPosition()?-1:1},_dispose:function(){m.animation.complete((0,o.default)(this.viewContent())),this.callBase()},_visibilityChanged:function(e){e&&this._dimensionChanged()},_dimensionChanged:function(){this._initMinMaxSize(),this._strategy.refreshPanelElementSize("slide"===this.option("revealMode")),this._renderPosition(this.option("opened"),!1)},_toggleShaderVisibility:function(e){this.option("shading")?(this._$shader.toggleClass(I,!e),this._$shader.css("visibility",e?"visible":"hidden")):this._$shader.toggleClass(I,!0)},_toggleOpenedStateClass:function(e){this.$element().toggleClass(T,e)},_refreshPanel:function(){var e=this;(0,o.default)(this.viewContent()).css("paddingLeft",0),(0,o.default)(this.viewContent()).css("paddingRight",0),(0,o.default)(this.viewContent()).css("paddingTop",0),(0,o.default)(this.viewContent()).css("paddingBottom",0),(0,o.default)(this.viewContent()).css("left",0),(0,o.default)(this.viewContent()).css("transform","translate(0px, 0px)"),(0,o.default)(this.viewContent()).removeClass("dx-theme-background-color"),this._removePanelContentWrapper(),this._removeOverlay(),this._renderPanelContentWrapper(),this._refreshWrapperChildrenOrder(),this._whenPanelContentRefreshed=new b.Deferred,this._strategy.renderPanelContent(this._whenPanelContentRefreshed),this._strategy.onPanelContentRendered(),(0,f.hasWindow)()&&this._whenPanelContentRefreshed.always(function(){e._strategy.refreshPanelElementSize("slide"===e.option("revealMode")),e._renderPosition(e.option("opened"),!1,!0),e._removePanelManualPosition()})},_clean:function(){this._cleanFocusState(),this._removePanelContentWrapper(),this._removeOverlay()},_removePanelContentWrapper:function(){this._$panelContentWrapper&&this._$panelContentWrapper.remove()},_removeOverlay:function(){this._overlay&&(this._overlay.dispose(),delete this._overlay,delete this._$panelContentWrapper)},_optionChanged:function(e){switch(e.name){case"width":this.callBase(e),this._dimensionChanged();break;case"opened":this._renderPosition(e.value),this._toggleOpenedStateClass(e.value);break;case"position":this._refreshPositionClass(e.previousValue),this._refreshWrapperChildrenOrder(),this._invalidate();break;case"contentTemplate":case"template":this._invalidate();break;case"openedStateMode":case"target":this._initStrategy(),this._refreshOpenedStateModeClass(e.previousValue),this._refreshPanel();break;case"minSize":case"maxSize":this._initMinMaxSize(),this._renderPosition(this.option("opened"),!1);break;case"revealMode":this._refreshRevealModeClass(e.previousValue),this._refreshPanel();break;case"shading":this._toggleShaderVisibility(this.option("opened"));break;case"animationEnabled":case"animationDuration":case"closeOnOutsideClick":break;default:this.callBase(e)}},content:function(){return(0,s.getPublicElement)(this._$panelContentWrapper)},viewContent:function(){return(0,s.getPublicElement)(this._$viewContentWrapper)},show:function(){return this.toggle(!0)},hide:function(){return this.toggle(!1)},toggle:function(e){var t=void 0===e?!this.option("opened"):e;return this._whenAnimationCompleted=new b.Deferred,this.option("opened",t),this._whenAnimationCompleted.promise()}});(0,l.default)("dxDrawer",A);var P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=n(194),u=n(6),c=function(){function e(t){o(this,e),this._drawer=t}return r(e,[{key:"getDrawerInstance",value:function(){return this._drawer}},{key:"renderPanelContent",value:function(e){var t=this.getDrawerInstance(),n=t._getTemplate(t.option("template"));n&&n.render({container:t.content(),onRendered:function(){e.resolve()}})}},{key:"renderPosition",value:function(e,t){this._prepareAnimationDeferreds(t);var n=this._getPositionRenderingConfig(e);if(this._useDefaultAnimation())this._defaultPositionRendering(n,e,t);else{var i=this.getDrawerInstance().option("revealMode");"slide"===i?this._slidePositionRendering(n,e,t):"expand"===i&&this._expandPositionRendering(n,e,t)}}},{key:"_prepareAnimationDeferreds",value:function(e){var t=this.getDrawerInstance();this._contentAnimation=new u.Deferred,this._panelAnimation=new u.Deferred,this._shaderAnimation=new u.Deferred,t._animations.push(this._contentAnimation,this._panelAnimation,this._shaderAnimation),e?u.when.apply(s.default,t._animations).done(function(){t._animationCompleteHandler()}):t.resizeViewContent()}},{key:"_getPositionRenderingConfig",value:function(e){var t=this.getDrawerInstance();return{direction:t.calcTargetPosition(),$panel:(0,s.default)(t.content()),$content:(0,s.default)(t.viewContent()),defaultAnimationConfig:this._defaultAnimationConfig(),size:this._getPanelSize(e)}}},{key:"_useDefaultAnimation",value:function(){return!1}},{key:"_elementsAnimationCompleteHandler",value:function(){this._contentAnimation.resolve(),this._panelAnimation.resolve()}},{key:"_defaultAnimationConfig",value:function(){var e=this;return{complete:function(){e._elementsAnimationCompleteHandler()}}}},{key:"_getPanelOffset",value:function(e){var t=this.getDrawerInstance(),n=t.isHorizontalDirection()?t.getRealPanelWidth():t.getRealPanelHeight();return e?-(n-t.getMaxSize()):-(n-t.getMinSize())}},{key:"_getPanelSize",value:function(e){return e?this.getDrawerInstance().getMaxSize():this.getDrawerInstance().getMinSize()}},{key:"renderShaderVisibility",value:function(e,t,n){var i=this,o=this.getDrawerInstance(),a=e?{from:0,to:1}:{from:1,to:0};t?l.animation.fade((0,s.default)(o._$shader),a,n,function(){i._drawer._toggleShaderVisibility(e),i._shaderAnimation.resolve()}):(o._toggleShaderVisibility(e),o._$shader.css("opacity",a.to))}},{key:"getPanelContent",value:function(){return(0,s.default)(this.getDrawerInstance().content())}},{key:"setPanelSize",value:function(e){this.refreshPanelElementSize(e)}},{key:"refreshPanelElementSize",value:function(e){var t=this.getDrawerInstance(),n=this._getPanelSize(t.option("opened"));t.isHorizontalDirection()?(0,s.default)(t.content()).width(e?t.getRealPanelWidth():n):(0,s.default)(t.content()).height(e?t.getRealPanelHeight():n)}},{key:"isViewContentFirst",value:function(){return!1}},{key:"onPanelContentRendered",value:function(){}}]),e}(),d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getElementMaxHeightByWindow=void 0;var o=i(n(2)),a=n(7),r=n(1),s=.9,l=function(e,t){var n,i=(0,o.default)((0,a.getWindow)()),l=e.offset(),u=l.top;if((0,r.isNumeric)(t)){if(t<u)return u-t;n=i.innerHeight()-t+i.scrollTop()}else{var c=u-i.scrollTop(),d=i.innerHeight()-c-e.outerHeight();n=Math.max(c,d)}return n*s};t.getElementMaxHeightByWindow=l},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(35)),y=n(7),b=o(n(5)),x=o(n(9)),w=o(n(27)),C=n(1),k=n(3),S=n(0),D=n(12),I=n(6),T=o(n(76)),O=o(n(52)),E=o(n(38)),A=o(n(295)),P=o(n(23)),R=o(n(13)),M=n(8),B=n(21),V=o(n(10)),F=n(36),L=(0,y.getWindow)(),H="dx-fileuploader",z="dx-fileuploader-empty",N="dx-fileuploader-show-file-list",j="dx-fileuploader-dragover",W="dx-fileuploader-wrapper",G="dx-fileuploader-container",q="dx-fileuploader-content",$="dx-fileuploader-input-wrapper",K="dx-fileuploader-input-container",U="dx-fileuploader-input-label",Y="dx-fileuploader-input",Z="dx-fileuploader-files-container",X="dx-fileuploader-file-container",Q="dx-fileuploader-file-info",J="dx-fileuploader-file-status-message",ee="dx-fileuploader-file",te="dx-fileuploader-file-name",ne="dx-fileuploader-file-size",ie="dx-fileuploader-button",oe="dx-fileuploader-button-container",ae="dx-fileuploader-cancel-button",re="dx-fileuploader-upload-button",se="dx-fileuploader-invalid",le=400,ue="chunkMetadata",ce=function(){return(0,m.default)("<input>").attr("type","file")},de=function(){return!!L.FormData},he=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_supportedKeys",value:function(){var e=this,n=function(t){t.preventDefault();var n=e._selectButton.$element();b.default.trigger(n,B.name);
};return(0,S.extend)(l(_(t.prototype),"_supportedKeys",this).call(this),{space:n,enter:n})}},{key:"_setOptionsByReference",value:function(){l(_(t.prototype),"_setOptionsByReference",this).call(this),(0,S.extend)(this._optionsByReference,{value:!0})}},{key:"_getDefaultOptions",value:function(){return(0,S.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{chunkSize:0,value:[],selectButtonText:V.default.format("dxFileUploader-selectFile"),uploadButtonText:V.default.format("dxFileUploader-upload"),labelText:V.default.format("dxFileUploader-dropFile"),name:"files[]",multiple:!1,accept:"",uploadUrl:"/",allowCanceling:!0,showFileList:!0,progress:0,dialogTrigger:void 0,dropZone:void 0,readyToUploadMessage:V.default.format("dxFileUploader-readyToUpload"),uploadedMessage:V.default.format("dxFileUploader-uploaded"),uploadFailedMessage:V.default.format("dxFileUploader-uploadFailedMessage"),uploadAbortedMessage:V.default.format("dxFileUploader-uploadAbortedMessage"),uploadMode:"instantly",uploadMethod:"POST",uploadHeaders:{},uploadCustomData:{},onBeforeSend:null,onUploadStarted:null,onUploaded:null,onFilesUploaded:null,onProgress:null,onUploadError:null,onUploadAborted:null,onDropZoneEnter:null,onDropZoneLeave:null,allowedFileExtensions:[],maxFileSize:0,minFileSize:0,inputAttr:{},invalidFileExtensionMessage:V.default.format("dxFileUploader-invalidFileExtension"),invalidMaxFileSizeMessage:V.default.format("dxFileUploader-invalidMaxFileSize"),invalidMinFileSizeMessage:V.default.format("dxFileUploader-invalidMinFileSize"),extendSelection:!0,validationMessageMode:"always",uploadFile:null,uploadChunk:null,abortUpload:null,validationMessageOffset:{h:0,v:0},useNativeInputClick:!1,useDragOver:!0,nativeDropSupported:!0,_uploadButtonType:"normal"})}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return"desktop"===R.default.real().deviceType&&!R.default.isSimulator()},options:{focusStateEnabled:!0}},{device:[{platform:"android"}],options:{validationMessageOffset:{v:0}}},{device:function(){return"desktop"!==R.default.real().deviceType},options:{useDragOver:!1}},{device:function(){return!de()},options:{uploadMode:"useForm"}},{device:function(){return P.default.msie||"desktop"!==R.default.real().deviceType},options:{nativeDropSupported:!1}},{device:function(){return(0,F.isMaterial)()},options:{_uploadButtonType:"default"}}])}},{key:"_initOptions",value:function(e){var n="labelText"in e;l(_(t.prototype),"_initOptions",this).call(this,e),n||this._shouldDragOverBeRendered()||this.option("labelText","")}},{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._initFileInput(),this._initLabel(),this._setUploadStrategy(),this._createFiles(),this._createBeforeSendAction(),this._createUploadStartedAction(),this._createUploadedAction(),this._createFilesUploadedAction(),this._createProgressAction(),this._createUploadErrorAction(),this._createUploadAbortedAction(),this._createDropZoneEnterAction(),this._createDropZoneLeaveAction()}},{key:"_setUploadStrategy",value:function(){var e=null;if(this.option("chunkSize")>0){var t=this.option("uploadChunk");e=t&&(0,C.isFunction)(t)?new me(this):new _e(this)}else{var n=this.option("uploadFile");e=n&&(0,C.isFunction)(n)?new be(this):new ye(this)}this._uploadStrategy=e}},{key:"_initFileInput",value:function(){var e=this;this._isCustomClickEvent=!1,this._$fileInput||(this._$fileInput=ce(),b.default.on(this._$fileInput,"change",this._inputChangeHandler.bind(this)),b.default.on(this._$fileInput,"click",function(t){return t.stopPropagation(),e.option("useNativeInputClick")||e._isCustomClickEvent})),this._$fileInput.prop({multiple:this.option("multiple"),accept:this.option("accept"),tabIndex:-1})}},{key:"_inputChangeHandler",value:function(){if(!this._doPreventInputChange){var e=this._$fileInput.val().replace(/^.*\\/,""),t=this._$fileInput.prop("files");if(!t||t.length||"useForm"===this.option("uploadMode")){var n=t?this._getFiles(t):[{name:e}];this._changeValue(n),"instantly"===this.option("uploadMode")&&this._uploadFiles()}}}},{key:"_shouldFileListBeExtended",value:function(){return"useForm"!==this.option("uploadMode")&&this.option("extendSelection")&&this.option("multiple")}},{key:"_removeDuplicates",value:function(e,t){for(var n=[],i=0;i<t.length;i++)this._isFileInArray(e,t[i])||n.push(t[i]);return n}},{key:"_isFileInArray",value:function(e,t){for(var n=0;n<e.length;n++){var i=e[n];if(i.size===t.size&&i.name===t.name)return!0}return!1}},{key:"_changeValue",value:function(e){var t=this._shouldFileListBeExtended()?this.option("value").slice():[];"instantly"!==this.option("uploadMode")&&(e=this._removeDuplicates(t,e)),this.option("value",t.concat(e))}},{key:"_getFiles",value:function(e){var t=[];return(0,k.each)(e,function(e,n){return t.push(n)}),t}},{key:"_getFile",value:function(e){var t=(0,C.isNumeric)(e)?this.option("value")[e]:e;return this._files.filter(function(e){return e.value===t})[0]}},{key:"_initLabel",value:function(){this._$inputLabel||(this._$inputLabel=(0,m.default)("<div>")),this._updateInputLabelText()}},{key:"_updateInputLabelText",value:function(){var e=this._isInteractionDisabled()?"":this.option("labelText");this._$inputLabel.text(e)}},{key:"_focusTarget",value:function(){return this.$element().find("."+ie)}},{key:"_getSubmitElement",value:function(){return this._$fileInput}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this.$element().addClass(H),this._renderWrapper(),this._renderInputWrapper(),this._renderSelectButton(),this._renderInputContainer(),this._renderUploadButton(),this._preventRecreatingFiles=!0}},{key:"_render",value:function(){this._preventRecreatingFiles=!1,this._attachDragEventHandlers(this._$inputWrapper),this._attachDragEventHandlers(this.option("dropZone")),this._renderFiles(),l(_(t.prototype),"_render",this).call(this)}},{key:"_createFileProgressBar",value:function(e){e.progressBar=this._createProgressBar(e.value.size),e.progressBar.$element().appendTo(e.$file),this._initStatusMessage(e),this._ensureCancelButtonInitialized(e)}},{key:"_setStatusMessage",value:function(e,t){var n=this;setTimeout(function(){n.option("showFileList")&&e.$statusMessage&&(e.$statusMessage.text(t),e.$statusMessage.css("display",""),e.progressBar.$element().remove())},le)}},{key:"_getUploadAbortedStatusMessage",value:function(){return"instantly"===this.option("uploadMode")?this.option("uploadAbortedMessage"):this.option("readyToUploadMessage")}},{key:"_createFiles",value:function(){var e=this,t=this.option("value");!this._files||0!==t.length&&this._shouldFileListBeExtended()||(this._preventFilesUploading(this._files),this._files=null),this._files||(this._files=[]),(0,k.each)(t.slice(this._files.length),function(t,n){var i=e._createFile(n);e._validateFile(i),e._files.push(i)})}},{key:"_preventFilesUploading",value:function(e){var t=this;e.forEach(function(e){return t._uploadStrategy.abortUpload(e)})}},{key:"_validateFile",value:function(e){e.isValidFileExtension=this._validateFileExtension(e),e.isValidMinSize=this._validateMinFileSize(e),e.isValidMaxSize=this._validateMaxFileSize(e)}},{key:"_validateFileExtension",value:function(e){var t=this.option("allowedFileExtensions"),n=e.value.name.substring(e.value.name.lastIndexOf(".")).toLowerCase();if(0===t.length)return!0;for(var i=0;i<t.length;i++)if(n===t[i].toLowerCase())return!0;return!1}},{key:"_validateMaxFileSize",value:function(e){var t=e.value.size,n=this.option("maxFileSize");return!(n>0)||t<=n}},{key:"_validateMinFileSize",value:function(e){var t=e.value.size,n=this.option("minFileSize");return!(n>0)||t>=n}},{key:"_createBeforeSendAction",value:function(){this._beforeSendAction=this._createActionByOption("onBeforeSend",{excludeValidators:["readOnly"]})}},{key:"_createUploadStartedAction",value:function(){this._uploadStartedAction=this._createActionByOption("onUploadStarted",{excludeValidators:["readOnly"]})}},{key:"_createUploadedAction",value:function(){this._uploadedAction=this._createActionByOption("onUploaded",{excludeValidators:["readOnly"]})}},{key:"_createFilesUploadedAction",value:function(){this._filesUploadedAction=this._createActionByOption("onFilesUploaded",{excludeValidators:["readOnly"]})}},{key:"_createProgressAction",value:function(){this._progressAction=this._createActionByOption("onProgress",{excludeValidators:["readOnly"]})}},{key:"_createUploadAbortedAction",value:function(){this._uploadAbortedAction=this._createActionByOption("onUploadAborted",{excludeValidators:["readOnly"]})}},{key:"_createUploadErrorAction",value:function(){this._uploadErrorAction=this._createActionByOption("onUploadError",{excludeValidators:["readOnly"]})}},{key:"_createDropZoneEnterAction",value:function(){this._dropZoneEnterAction=this._createActionByOption("onDropZoneEnter")}},{key:"_createDropZoneLeaveAction",value:function(){this._dropZoneLeaveAction=this._createActionByOption("onDropZoneLeave")}},{key:"_createFile",value:function(e){return{value:e,loadedSize:0,onProgress:(0,w.default)(),onAbort:(0,w.default)(),onLoad:(0,w.default)(),onError:(0,w.default)(),onLoadStart:(0,w.default)(),isValidFileExtension:!0,isValidMaxSize:!0,isValidMinSize:!0,isValid:function(){return this.isValidFileExtension&&this.isValidMaxSize&&this.isValidMinSize},isInitialized:!1}}},{key:"_resetFileState",value:function(e){e.isAborted=!1,e.uploadStarted=!1,e.isStartLoad=!1,e.loadedSize=0,e.chunksData=void 0,e.request=void 0}},{key:"_renderFiles",value:function(){var e,t=this,n=this.option("value");this._$filesContainer?this._shouldFileListBeExtended()&&0!==n.length||this._$filesContainer.empty():this._$filesContainer=(0,m.default)("<div>").addClass(Z).appendTo(this._$content);var i=this.option("showFileList");i&&(0,k.each)(this._files,function(e,n){n.$file||t._renderFile(n)}),this.$element().toggleClass(N,i),this._toggleFileUploaderEmptyClassName(),this._updateFileNameMaxWidth(),null===(e=this._validationMessage)||void 0===e?void 0:e.repaint()}},{key:"_renderFile",value:function(e){var t=e.value,n=(0,m.default)("<div>").addClass(X).appendTo(this._$filesContainer);this._renderFileButtons(e,n),e.$file=(0,m.default)("<div>").addClass(ee).appendTo(n);var i=(0,m.default)("<div>").addClass(Q).appendTo(e.$file);e.$statusMessage=(0,m.default)("<div>").addClass(J).appendTo(e.$file),(0,m.default)("<div>").addClass(te).text(t.name).appendTo(i),(0,C.isDefined)(t.size)&&(0,m.default)("<div>").addClass(ne).text(this._getFileSize(t.size)).appendTo(i),e.isValid()?e.$statusMessage.text(this.option("readyToUploadMessage")):(e.isValidFileExtension||e.$statusMessage.append(this._createValidationElement("invalidFileExtensionMessage")),e.isValidMaxSize||e.$statusMessage.append(this._createValidationElement("invalidMaxFileSizeMessage")),e.isValidMinSize||e.$statusMessage.append(this._createValidationElement("invalidMinFileSizeMessage")),n.addClass(se))}},{key:"_createValidationElement",value:function(e){return(0,m.default)("<span>").text(this.option(e))}},{key:"_updateFileNameMaxWidth",value:function(){var e=this.option("allowCanceling")&&"useForm"!==this.option("uploadMode")?1:0,t="useButtons"===this.option("uploadMode")?1:0,n=this._$filesContainer.find("."+X).first().width()||this._$filesContainer.width(),i=this._$filesContainer.find("."+oe).eq(0),o=i.width()*(e+t),a=this._$filesContainer.find("."+ne).eq(0),r=a.text();a.text("1000 Mb");var s=a.width();a.text(r),this._$filesContainer.find("."+te).css("maxWidth",n-o-s)}},{key:"_renderFileButtons",value:function(e,t){var n=this._getCancelButton(e);n&&t.append(n);var i=this._getUploadButton(e);i&&t.append(i)}},{key:"_getCancelButton",value:function(e){var t=this;return"useForm"===this.option("uploadMode")?null:(e.cancelButton=this._createComponent((0,m.default)("<div>").addClass(ie+" "+ae),E.default,{onClick:function(){return t._removeFile(e)},icon:"close",visible:this.option("allowCanceling"),disabled:this.option("readOnly"),integrationOptions:{}}),(0,m.default)("<div>").addClass(oe).append(e.cancelButton.$element()))}},{key:"_getUploadButton",value:function(e){var t=this;return e.isValid()&&"useButtons"===this.option("uploadMode")?(e.uploadButton=this._createComponent((0,m.default)("<div>").addClass(ie+" "+re),E.default,{onClick:function(){return t._uploadFile(e)},icon:"upload"}),e.onLoadStart.add(function(){return e.uploadButton.option({visible:!1,disabled:!0})}),e.onAbort.add(function(){return e.uploadButton.option({visible:!0,disabled:!1})}),(0,m.default)("<div>").addClass(oe).append(e.uploadButton.$element())):null}},{key:"_removeFile",value:function(e){var t;null===(t=e.$file)||void 0===t?void 0:t.parent().remove(),this._files.splice((0,D.inArray)(e,this._files),1);var n=this.option("value").slice();n.splice((0,D.inArray)(e.value,n),1),this._preventRecreatingFiles=!0,this.option("value",n),this._preventRecreatingFiles=!1,this._toggleFileUploaderEmptyClassName(),this._doPreventInputChange=!0,this._$fileInput.val(""),this._doPreventInputChange=!1}},{key:"removeFile",value:function(e){if("useForm"!==this.option("uploadMode")&&(0,C.isDefined)(e)){var t=this._getFile(e);t&&(t.uploadStarted&&this._preventFilesUploading([t]),this._removeFile(t))}}},{key:"_toggleFileUploaderEmptyClassName",value:function(){this.$element().toggleClass(z,!this._files.length||this._hasInvalidFile(this._files))}},{key:"_hasInvalidFile",value:function(e){for(var t=0;t<e.length;t++)if(!e[t].isValid())return!0;return!1}},{key:"_getFileSize",value:function(e){for(var t=0,n=[V.default.format("dxFileUploader-bytes"),V.default.format("dxFileUploader-kb"),V.default.format("dxFileUploader-Mb"),V.default.format("dxFileUploader-Gb")],i=n.length-1;t<i&&e>=1024;)e/=1024,t++;return Math.round(e)+" "+n[t]}},{key:"_renderSelectButton",value:function(){var e=(0,m.default)("<div>").addClass(ie).appendTo(this._$inputWrapper);this._selectButton=this._createComponent(e,E.default,{text:this.option("selectButtonText"),focusStateEnabled:!1,integrationOptions:{},disabled:this.option("readOnly")}),this._selectFileDialogHandler=this._selectButtonClickHandler.bind(this),"desktop"===R.default.real().deviceType?this._selectButton.option("onClick",this._selectFileDialogHandler):this._attachSelectFileDialogHandler(this._selectButton.$element()),this._attachSelectFileDialogHandler(this.option("dialogTrigger"))}},{key:"_selectButtonClickHandler",value:function(){if(!this.option("useNativeInputClick")){if(this._isInteractionDisabled())return!1;this._isCustomClickEvent=!0,b.default.trigger(this._$fileInput,"click"),this._isCustomClickEvent=!1}}},{key:"_attachSelectFileDialogHandler",value:function(e){(0,C.isDefined)(e)&&(this._detachSelectFileDialogHandler(e),b.default.on((0,m.default)(e),"click",this._selectFileDialogHandler))}},{key:"_detachSelectFileDialogHandler",value:function(e){(0,C.isDefined)(e)&&b.default.off((0,m.default)(e),"click",this._selectFileDialogHandler)}},{key:"_renderUploadButton",value:function(){if("useButtons"===this.option("uploadMode")){var e=(0,m.default)("<div>").addClass(ie).addClass(re).appendTo(this._$content);this._uploadButton=this._createComponent(e,E.default,{text:this.option("uploadButtonText"),onClick:this._uploadButtonClickHandler.bind(this),type:this.option("_uploadButtonType"),integrationOptions:{}})}}},{key:"_uploadButtonClickHandler",value:function(){this._uploadFiles()}},{key:"_shouldDragOverBeRendered",value:function(){return!this.option("readOnly")&&("useForm"!==this.option("uploadMode")||this.option("nativeDropSupported"))}},{key:"_isInteractionDisabled",value:function(){return this.option("readOnly")||this.option("disabled")}},{key:"_renderInputContainer",value:function(){this._$inputContainer=(0,m.default)("<div>").addClass(K).appendTo(this._$inputWrapper),this._$fileInput.addClass(Y),this._renderInput();var e="dx-fileuploader-input-label-".concat(new v.default);this._$inputLabel.attr("id",e).addClass(U).appendTo(this._$inputContainer),this.setAria("labelledby",e,this._$fileInput)}},{key:"_renderInput",value:function(){this.option("useNativeInputClick")?this._selectButton.option("template",this._selectButtonInputTemplate.bind(this)):(this._$fileInput.appendTo(this._$inputContainer),this._selectButton.option("template","content")),this._applyInputAttributes(this.option("inputAttr"))}},{key:"_selectButtonInputTemplate",value:function(e,t){var n=(0,m.default)(t),i=(0,m.default)("<span>").addClass("dx-button-text").text(e.text);return n.append(i).append(this._$fileInput),n}},{key:"_renderInputWrapper",value:function(){this._$inputWrapper=(0,m.default)("<div>").addClass($).appendTo(this._$content)}},{key:"_detachDragEventHandlers",value:function(e){(0,C.isDefined)(e)&&b.default.off((0,m.default)(e),(0,M.addNamespace)("",this.NAME))}},{key:"_attachDragEventHandlers",value:function(e){var t=e!==this._$inputWrapper;(0,C.isDefined)(e)&&this._shouldDragOverBeRendered()&&(this._detachDragEventHandlers(e),e=(0,m.default)(e),this._dragEventsTargets=[],b.default.on(e,(0,M.addNamespace)("dragenter",this.NAME),this._dragEnterHandler.bind(this,t)),b.default.on(e,(0,M.addNamespace)("dragover",this.NAME),this._dragOverHandler.bind(this)),b.default.on(e,(0,M.addNamespace)("dragleave",this.NAME),this._dragLeaveHandler.bind(this,t)),b.default.on(e,(0,M.addNamespace)("drop",this.NAME),this._dropHandler.bind(this,t)))}},{key:"_applyInputAttributes",value:function(e){this._$fileInput.attr(e)}},{key:"_useInputForDrop",value:function(){return this.option("nativeDropSupported")&&"useForm"===this.option("uploadMode")}},{key:"_dragEnterHandler",value:function(e,t){return!this.option("disabled")&&(this._useInputForDrop()||t.preventDefault(),this._tryToggleDropZoneActive(!0,e,t),void this._updateEventTargets(t))}},{key:"_dragOverHandler",value:function(e){this._useInputForDrop()||e.preventDefault(),e.originalEvent.dataTransfer.dropEffect="copy"}},{key:"_dragLeaveHandler",value:function(e,t){this._useInputForDrop()||t.preventDefault(),this._updateEventTargets(t),this._tryToggleDropZoneActive(!1,e,t)}},{key:"_updateEventTargets",value:function(e){var t=this._dragEventsTargets.indexOf(e.target),n=t!==-1;"dragenter"===e.type?!n&&this._dragEventsTargets.push(e.target):n&&this._dragEventsTargets.splice(t,1)}},{key:"_tryToggleDropZoneActive",value:function(e,t,n){var i=e?"addClass":"removeClass",o=e?"_dropZoneEnterAction":"_dropZoneLeaveAction";this._dragEventsTargets.length||(this[o]({event:n,dropZoneElement:n.currentTarget}),t||this.$element()[i](j))}},{key:"_dropHandler",value:function(e,t){if(this._dragEventsTargets=[],e||this.$element().removeClass(j),!(this._useInputForDrop()||e&&this._isInteractionDisabled())){t.preventDefault();var n=t.originalEvent.dataTransfer.files,i=this._getFiles(n);!this.option("multiple")&&i.length>1||(this._changeValue(this._filterFiles(i)),"instantly"===this.option("uploadMode")&&this._uploadFiles())}}},{key:"_handleAllFilesUploaded",value:function(){var e=this._files.every(function(e){return!e.isValid()||e._isError||e._isLoaded||e.isAborted});e&&this._filesUploadedAction()}},{key:"_filterFiles",value:function(e){if(!e.length)return e;var t=this.option("accept");if(!t.length)return e;for(var n=[],i=this._getAllowedFileTypes(t),o=0,a=e.length;o<a;o++)this._isFileTypeAllowed(e[o],i)&&n.push(e[o]);return n}},{key:"_getAllowedFileTypes",value:function(e){return e.length?e.split(",").map(function(e){return e.trim()}):[]}},{key:"_isFileTypeAllowed",value:function(e,t){for(var n=0,i=t.length;n<i;n++){var o=t[n];if("."===o[0]){if(o=o.replace(".","\\."),e.name.match(new RegExp(o+"$","i")))return!0}else if(o=o.replace("*",""),e.type.match(new RegExp(o,"i")))return!0}return!1}},{key:"_renderWrapper",value:function(){var e=(0,m.default)("<div>").addClass(W).appendTo(this.$element()),t=(0,m.default)("<div>").addClass(G).appendTo(e);this._$content=(0,m.default)("<div>").addClass(q).appendTo(t)}},{key:"_clean",value:function(){this._$fileInput.detach(),delete this._$filesContainer,this._files&&this._files.forEach(function(e){e.$file=null,e.$statusMessage=null}),l(_(t.prototype),"_clean",this).call(this)}},{key:"abortUpload",value:function(e){if("useForm"!==this.option("uploadMode"))if((0,C.isDefined)(e)){var t=this._getFile(e);t&&this._preventFilesUploading([t])}else this._preventFilesUploading(this._files)}},{key:"upload",value:function(e){if("useForm"!==this.option("uploadMode"))if((0,C.isDefined)(e)){var t=this._getFile(e);t&&de()&&this._uploadFile(t)}else this._uploadFiles()}},{key:"_uploadFiles",value:function(){var e=this;de()&&(0,k.each)(this._files,function(t,n){return e._uploadFile(n)})}},{key:"_uploadFile",value:function(e){this._uploadStrategy.upload(e)}},{key:"_updateProgressBar",value:function(e,t){e.progressBar&&e.progressBar.option({value:t.loaded,showStatus:!0}),this._progressAction({file:e.value,segmentSize:t.currentSegmentSize,bytesLoaded:t.loaded,bytesTotal:t.total,event:t.event,request:e.request})}},{key:"_updateTotalProgress",value:function(e,t){var n=e?this._getProgressValue(t/e):0;this.option("progress",n),this._setLoadedSize(t)}},{key:"_getProgressValue",value:function(e){return Math.floor(100*e)}},{key:"_initStatusMessage",value:function(e){e.$statusMessage.css("display","none")}},{key:"_ensureCancelButtonInitialized",value:function(e){var t=this;if(!e.isInitialized){e.cancelButton.option("onClick",function(){t._preventFilesUploading([e]),t._removeFile(e)});var n=function(){setTimeout(function(){e.cancelButton.option({visible:!1})},le)};e.onLoad.add(n),e.onError.add(n)}}},{key:"_createProgressBar",value:function(e){var t=this;return this._createComponent((0,m.default)("<div>"),A.default,{value:void 0,min:0,max:e,statusFormat:function(e){return t._getProgressValue(e)+"%"},showStatus:!1,statusPosition:"right"})}},{key:"_getTotalFilesSize",value:function(){var e=this;return this._totalFilesSize||(this._totalFilesSize=0,(0,k.each)(this._files,function(t,n){e._totalFilesSize+=n.value.size})),this._totalFilesSize}},{key:"_getTotalLoadedFilesSize",value:function(){var e=this;return this._totalLoadedFilesSize||(this._totalLoadedFilesSize=0,(0,k.each)(this._files,function(t,n){e._totalLoadedFilesSize+=n.loadedSize})),this._totalLoadedFilesSize}},{key:"_setLoadedSize",value:function(e){this._totalLoadedFilesSize=e}},{key:"_recalculateProgress",value:function(){this._totalFilesSize=0,this._totalLoadedFilesSize=0,this._updateTotalProgress(this._getTotalFilesSize(),this._getTotalLoadedFilesSize())}},{key:"_updateReadOnlyState",value:function(){var e=this.option("readOnly");this._selectButton.option("disabled",e),this._files.forEach(function(t){var n;return null===(n=t.cancelButton)||void 0===n?void 0:n.option("disabled",e)}),this._updateInputLabelText(),this._attachDragEventHandlers(this._$inputWrapper)}},{key:"_optionChanged",value:function(e){var n=e.name,i=e.value,o=e.previousValue;switch(n){case"height":case"width":this._updateFileNameMaxWidth(),l(_(t.prototype),"_optionChanged",this).call(this,e);break;case"value":!i.length&&this._$fileInput.val(""),this._preventRecreatingFiles||(this._createFiles(),this._renderFiles()),this._recalculateProgress(),l(_(t.prototype),"_optionChanged",this).call(this,e);break;case"name":this._initFileInput(),l(_(t.prototype),"_optionChanged",this).call(this,e);break;case"accept":this._initFileInput();break;case"multiple":this._initFileInput(),e.value||this.reset();break;case"readOnly":this._updateReadOnlyState(),l(_(t.prototype),"_optionChanged",this).call(this,e);break;case"selectButtonText":this._selectButton.option("text",i);break;case"uploadButtonText":this._uploadButton&&this._uploadButton.option("text",i);break;case"_uploadButtonType":this._uploadButton&&this._uploadButton.option("type",i);break;case"dialogTrigger":this._detachSelectFileDialogHandler(o),this._attachSelectFileDialogHandler(i);break;case"dropZone":this._detachDragEventHandlers(o),this._attachDragEventHandlers(i);break;case"maxFileSize":case"minFileSize":case"allowedFileExtensions":case"invalidFileExtensionMessage":case"invalidMaxFileSizeMessage":case"invalidMinFileSizeMessage":case"readyToUploadMessage":case"uploadedMessage":case"uploadFailedMessage":case"uploadAbortedMessage":this._invalidate();break;case"labelText":this._updateInputLabelText();break;case"showFileList":this._preventRecreatingFiles||this._renderFiles();break;case"uploadFile":case"uploadChunk":case"chunkSize":this._setUploadStrategy();break;case"abortUpload":case"uploadUrl":case"progress":case"uploadMethod":case"uploadHeaders":case"uploadCustomData":case"extendSelection":break;case"allowCanceling":case"uploadMode":this.reset(),this._invalidate();break;case"onBeforeSend":this._createBeforeSendAction();break;case"onUploadStarted":this._createUploadStartedAction();break;case"onUploaded":this._createUploadedAction();break;case"onFilesUploaded":this._createFilesUploadedAction();break;case"onProgress":this._createProgressAction();break;case"onUploadError":this._createUploadErrorAction();break;case"onUploadAborted":this._createUploadAbortedAction();break;case"onDropZoneEnter":this._createDropZoneEnterAction();break;case"onDropZoneLeave":this._createDropZoneLeaveAction();break;case"useNativeInputClick":this._renderInput();break;case"useDragOver":this._attachDragEventHandlers(this._$inputWrapper);break;case"nativeDropSupported":this._invalidate();break;case"inputAttr":this._applyInputAttributes(this.option(n));break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"reset",value:function(){this.option("value",[])}}]),t}(O.default),fe=function(){function e(t,n){a(this,e),this.file=t,this.chunkSize=n,this.index=0}return s(e,[{key:"read",value:function(){if(!this.file)return null;var e=this.createBlobResult(this.file,this.index,this.chunkSize);return e.isCompleted&&(this.file=null),this.index++,e}},{key:"createBlobResult",value:function(e,t,n){var i=t*n;return{blob:this.sliceFile(e,i,n),index:t,isCompleted:i+n>=e.size}}},{key:"sliceFile",value:function(e,t,n){return e.slice?e.slice(t,t+n):e.webkitSlice?e.webkitSlice(t,t+n):null}}]),e}(),pe=function(){function e(t){a(this,e),this.fileUploader=t}return s(e,[{key:"upload",value:function(e){e.isInitialized&&e.isAborted&&this.fileUploader._resetFileState(e),e.isValid()&&!e.uploadStarted&&(this._prepareFileBeforeUpload(e),this._uploadCore(e))}},{key:"abortUpload",value:function e(t){var n=this;if(!(t._isError||t._isLoaded||t.isAborted)&&(t.isAborted=!0,t.request&&t.request.abort(),this._isCustomCallback("abortUpload"))){var e=this.fileUploader.option("abortUpload"),i=this._createUploadArgument(t),o=null;try{var a=e(t.value,i);o=(0,I.fromPromise)(a)}catch(e){o=(new I.Deferred).reject(e).promise()}o.done(function(){return t.onAbort.fire()}).fail(function(e){return n._handleFileError(t,e)})}}},{key:"_beforeSend",value:function(e,t){var n=this._createUploadArgument(t);this.fileUploader._beforeSendAction({request:e,file:t.value,uploadInfo:n}),t.request=e}},{key:"_createUploadArgument",value:function(e){}},{key:"_uploadCore",value:function(e){}},{key:"_isCustomCallback",value:function(e){var t=this.fileUploader.option(e);return t&&(0,C.isFunction)(t)}},{key:"_handleFileError",value:function(e,t){e._isError=!0,e.onError.fire(t)}},{key:"_prepareFileBeforeUpload",value:function(e){if(e.$file){var t;null===(t=e.progressBar)||void 0===t?void 0:t.dispose(),this.fileUploader._createFileProgressBar(e)}e.isInitialized||(e.onLoadStart.add(this._onUploadStarted.bind(this,e)),e.onLoad.add(this._onLoadedHandler.bind(this,e)),e.onError.add(this._onErrorHandler.bind(this,e)),e.onAbort.add(this._onAbortHandler.bind(this,e)),e.onProgress.add(this._onProgressHandler.bind(this,e)),e.isInitialized=!0)}},{key:"_isStatusError",value:function(e){return 400<=e&&e<500||500<=e&&e<600}},{key:"_onUploadStarted",value:function(e,t){e.uploadStarted=!0,this.fileUploader._uploadStartedAction({file:e.value,event:t,request:e.request})}},{key:"_onAbortHandler",value:function(e,t){var n={file:e.value,event:t,request:e.request,message:this.fileUploader._getUploadAbortedStatusMessage()};this.fileUploader._uploadAbortedAction(n),this.fileUploader._setStatusMessage(e,n.message),this.fileUploader._handleAllFilesUploaded()}},{key:"_onErrorHandler",value:function(e,t){var n={file:e.value,event:void 0,request:e.request,error:t,message:this.fileUploader.option("uploadFailedMessage")};this.fileUploader._uploadErrorAction(n),this.fileUploader._setStatusMessage(e,n.message),this.fileUploader._handleAllFilesUploaded()}},{key:"_onLoadedHandler",value:function(e,t){var n={file:e.value,event:t,request:e.request,message:this.fileUploader.option("uploadedMessage")};e._isLoaded=!0,this.fileUploader._uploadedAction(n),this.fileUploader._setStatusMessage(e,n.message),this.fileUploader._handleAllFilesUploaded()}},{key:"_onProgressHandler",value:function(e,t){if(e){var n=this.fileUploader._getTotalFilesSize(),i=this.fileUploader._getTotalLoadedFilesSize(),o=Math.min(t.loaded,e.value.size),a=o-e.loadedSize;e.loadedSize=o,this.fileUploader._updateTotalProgress(n,i+a),this.fileUploader._updateProgressBar(e,this._getLoadedData(o,t.total,a,t))}}},{key:"_getLoadedData",value:function(e,t,n,i){return{loaded:e,total:t,currentSegmentSize:n}}},{key:"_extendFormData",value:function(e){var t=this.fileUploader.option("uploadCustomData");for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(0,C.isDefined)(t[n])&&e.append(n,t[n])}}]),e}(),ge=function(e){function t(e){var i;return a(this,t),i=n.call(this,e),i.chunkSize=i.fileUploader.option("chunkSize"),i}c(t,e);var n=h(t);return s(t,[{key:"_uploadCore",value:function(e){var t=e.value,n={name:t.name,loadedBytes:0,type:t.type,blobReader:new fe(t,this.chunkSize),guid:new v.default,fileSize:t.size,count:Math.ceil(t.size/this.chunkSize),customData:{}};e.chunksData=n,this._sendChunk(e,n)}},{key:"_sendChunk",value:function(e,t){var n=this,i=t.blobReader.read();t.currentChunk=i,i&&this._sendChunkCore(e,t,i).done(function(){e.isAborted||(t.loadedBytes+=i.blob.size,e.onProgress.fire({loaded:t.loadedBytes,total:e.value.size}),i.isCompleted&&e.onLoad.fire(),setTimeout(function(){return n._sendChunk(e,t)}))}).fail(function(t){n._shouldHandleError(t)&&n._handleFileError(e,t)})}},{key:"_sendChunkCore",value:function(e,t,n){}},{key:"_shouldHandleError",value:function(e){}},{key:"_tryRaiseStartLoad",value:function(e){e.isStartLoad||(e.isStartLoad=!0,e.onLoadStart.fire())}},{key:"_getEvent",value:function(e){return null}},{key:"_createUploadArgument",value:function(e){return this._createChunksInfo(e.chunksData)}},{key:"_createChunksInfo",value:function(e){return{bytesUploaded:e.loadedBytes,chunkCount:e.count,customData:e.customData,chunkBlob:e.currentChunk.blob,chunkIndex:e.currentChunk.index}}}]),t}(pe),_e=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_sendChunkCore",value:function(e,t,n){var i=this;return T.default.sendRequest({url:this.fileUploader.option("uploadUrl"),method:this.fileUploader.option("uploadMethod"),headers:this.fileUploader.option("uploadHeaders"),beforeSend:function(t){return i._beforeSend(t,e)},upload:{onloadstart:function(){return i._tryRaiseStartLoad(e)},onabort:function(){return e.onAbort.fire()}},data:this._createFormData({fileName:t.name,blobName:this.fileUploader.option("name"),blob:n.blob,index:n.index,count:t.count,type:t.type,guid:t.guid,size:t.fileSize})})}},{key:"_shouldHandleError",value:function(e){return this._isStatusError(e.status)}},{key:"_createFormData",value:function(e){var t=new L.FormData;return t.append(e.blobName,e.blob),t.append(ue,JSON.stringify({FileName:e.fileName,Index:e.index,TotalCount:e.count,FileSize:e.size,FileType:e.type,FileGuid:e.guid})),this._extendFormData(t),t}}]),t}(ge),me=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_sendChunkCore",value:function(e,t){this._tryRaiseStartLoad(e);var n=this._createChunksInfo(t),i=this.fileUploader.option("uploadChunk");try{var o=i(e.value,n);return(0,I.fromPromise)(o)}catch(e){return(new I.Deferred).reject(e).promise();
}}},{key:"_shouldHandleError",value:function(e){return!0}}]),t}(ge),ve=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_uploadCore",value:function(e){var t=this;e.loadedSize=0,this._uploadFile(e).done(function(){e.isAborted||e.onLoad.fire()}).fail(function(n){t._shouldHandleError(e,n)&&t._handleFileError(e,n)})}},{key:"_uploadFile",value:function(e){}},{key:"_shouldHandleError",value:function(e,t){}},{key:"_handleProgress",value:function(e,t){e._isError||(e._isProgressStarted=!0,e.onProgress.fire(t))}},{key:"_getLoadedData",value:function(e,n,i,o){var a=l(_(t.prototype),"_getLoadedData",this).call(this,e,n,i,o);return a.event=o,a}}]),t}(pe),ye=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_uploadFile",value:function(e){var t=this;return T.default.sendRequest({url:this.fileUploader.option("uploadUrl"),method:this.fileUploader.option("uploadMethod"),headers:this.fileUploader.option("uploadHeaders"),beforeSend:function(n){return t._beforeSend(n,e)},upload:{onprogress:function(n){return t._handleProgress(e,n)},onloadstart:function(){return e.onLoadStart.fire()},onabort:function(){return e.onAbort.fire()}},data:this._createFormData(this.fileUploader.option("name"),e.value)})}},{key:"_shouldHandleError",value:function(e,t){return this._isStatusError(t.status)||!e._isProgressStarted}},{key:"_createFormData",value:function(e,t){var n=new L.FormData;return n.append(e,t,t.name),this._extendFormData(n),n}}]),t}(ve),be=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_uploadFile",value:function(e){var t=this;e.onLoadStart.fire();var n=function(n){var i={loaded:n,total:e.size};t._handleProgress(e,i)},i=this.fileUploader.option("uploadFile");try{var o=i(e.value,n);return(0,I.fromPromise)(o)}catch(e){return(new I.Deferred).reject(e).promise()}}},{key:"_shouldHandleError",value:function(e,t){return!0}}]),t}(ve);(0,x.default)("dxFileUploader",he);var xe=he;t.default=xe,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(296)),r=n(0),s=n(1),l=i(n(9)),u="dx-progressbar",c="dx-progressbar-container",d="dx-progressbar-range-container",h="dx-progressbar-range",f="dx-progressbar-wrapper",p="dx-progressbar-status",g="dx-progressbar-animating-container",_="dx-progressbar-animating-segment",m=a.default.inherit({_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{value:0,statusFormat:function(e){return"Progress: "+Math.round(100*e)+"%"},showStatus:!0,onComplete:null,activeStateEnabled:!1,statusPosition:"bottom left",_animatingSegmentCount:0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(e){return"android"===e.platform},options:{_animatingSegmentCount:2}}])},_initMarkup:function(){this._renderStatus(),this._createCompleteAction(),this.callBase(),this.$element().addClass(u),this._$wrapper.addClass(f),this._$bar.addClass(c),this.setAria("role","progressbar"),(0,o.default)("<div>").addClass(d).appendTo(this._$wrapper).append(this._$bar),this._$range.addClass(h),this._toggleStatus(this.option("showStatus"))},_createCompleteAction:function(){this._completeAction=this._createActionByOption("onComplete")},_renderStatus:function(){this._$status=(0,o.default)("<div>").addClass(p)},_renderIndeterminateState:function(){this._$segmentContainer=(0,o.default)("<div>").addClass(g);for(var e=this.option("_animatingSegmentCount"),t=0;t<e;t++)(0,o.default)("<div>").addClass(_).addClass(_+"-"+(t+1)).appendTo(this._$segmentContainer);this._$segmentContainer.appendTo(this._$wrapper)},_toggleStatus:function(e){var t=this.option("statusPosition").split(" ");e?"top"===t[0]||"left"===t[0]?this._$status.prependTo(this._$wrapper):this._$status.appendTo(this._$wrapper):this._$status.detach(),this._togglePositionClass()},_togglePositionClass:function(){var e=this.option("statusPosition"),t=e.split(" ");this._$wrapper.removeClass("dx-position-top-left dx-position-top-right dx-position-bottom-left dx-position-bottom-right dx-position-left dx-position-right");var n="dx-position-"+t[0];t[1]&&(n+="-"+t[1]),this._$wrapper.addClass(n)},_toggleIndeterminateState:function(e){e?(this._renderIndeterminateState(),this._$bar.toggle(!1)):(this._$bar.toggle(!0),this._$segmentContainer.remove(),delete this._$segmentContainer)},_renderValue:function(){var e=this.option("value"),t=this.option("max");return e||0===e?(this._$segmentContainer&&this._toggleIndeterminateState(!1),e===t&&this._completeAction(),this.callBase(),void this._setStatus()):void this._toggleIndeterminateState(!0)},_setStatus:function(){var e=this.option("statusFormat");e=(0,s.isFunction)(e)?e.bind(this):function(e){return e};var t=e(this._currentRatio,this.option("value"));this._$status.text(t)},_dispose:function(){this._$status.remove(),this.callBase()},_optionChanged:function(e){switch(e.name){case"statusFormat":this._setStatus();break;case"showStatus":this._toggleStatus(e.value);break;case"statusPosition":this._toggleStatus(this.option("showStatus"));break;case"onComplete":this._createCompleteAction();break;case"_animatingSegmentCount":break;default:this.callBase(e)}}});(0,l.default)("dxProgressBar",m);var v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(52)),r=i(n(9)),s=n(0),l=n(7),u=i(n(43)),c="dx-trackbar",d="dx-trackbar-container",h="dx-trackbar-range",f="dx-trackbar-wrapper",p=a.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{min:0,max:100,value:0})},_initMarkup:function(){this.$element().addClass(c),this._renderWrapper(),this._renderContainer(),this._renderRange(),this._renderValue(),this._setRangeStyles(),this.callBase()},_render:function(){this.callBase(),this._setRangeStyles(this._rangeStylesConfig())},_renderWrapper:function(){this._$wrapper=(0,o.default)("<div>").addClass(f).appendTo(this.$element())},_renderContainer:function(){this._$bar=(0,o.default)("<div>").addClass(d).appendTo(this._$wrapper)},_renderRange:function(){this._$range=(0,o.default)("<div>").addClass(h).appendTo(this._$bar)},_renderValue:function(){var e=this.option("value"),t=this.option("min"),n=this.option("max");if(!(t>n)){if(e<t)return this.option("value",t),void(this._currentRatio=0);if(e>n)return this.option("value",n),void(this._currentRatio=1);var i=t===n?0:(e-t)/(n-t);!this._needPreventAnimation&&this._setRangeStyles({width:100*i+"%"}),this.setAria({valuemin:this.option("min"),valuemax:n,valuenow:e}),this._currentRatio=i}},_rangeStylesConfig:function(){return{width:100*this._currentRatio+"%"}},_setRangeStyles:function(e){return u.default.stop(this._$range),e?void(!this._needPreventAnimation&&(0,l.hasWindow)()&&u.default.animate(this._$range,{type:"custom",duration:100,to:e})):void this._$range.css({width:0})},_optionChanged:function(e){switch(e.name){case"value":this._renderValue(),this.callBase(e);break;case"max":case"min":this._renderValue();break;default:this.callBase(e)}},_dispose:function(){u.default.stop(this._$range),this.callBase()}});(0,r.default)("dxTrackBar",p);var g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(46),r=n(0),s=i(n(13)),l=i(n(11)),u=i(n(9)),c=i(n(438)),d=i(n(220)),h=i(n(640)),f=n(59),p=n(18),g=n(1),_=n(71),m=n(7),v="dx-tabpanel",y="dx-tabpanel-tabs",b="dx-tabpanel-container",x="dx-tab-text",w=c.default.inherit({_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{itemTitleTemplate:"title",hoverStateEnabled:!0,showNavButtons:!1,scrollByContent:!0,scrollingEnabled:!0,onTitleClick:null,onTitleHold:null,onTitleRendered:null,badgeExpr:function(e){return e?e.badge:void 0}})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===s.default.real().deviceType&&!s.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return!a.touch},options:{swipeEnabled:!1}},{device:{platform:"generic"},options:{animationEnabled:!1}}])},_init:function(){this.callBase(),this.$element().addClass(v),this.setAria("role","tabpanel")},_initMarkup:function(){this.callBase(),this._createTitleActions(),this._renderLayout()},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({title:new _.BindableTemplate(function(e,t){if((0,g.isPlainObject)(t)){var n=(0,f.getImageContainer)(t.icon);n&&e.append(n),(0,g.isDefined)(t.title)&&!(0,g.isPlainObject)(t.title)&&e.append(l.default.createTextNode(t.title))}else(0,g.isDefined)(t)&&e.text(String(t));e.wrapInner((0,o.default)("<span>").addClass(x))},["title","icon"],this.option("integrationOptions.watchMethod"))})},_createTitleActions:function(){this._createTitleClickAction(),this._createTitleHoldAction(),this._createTitleRenderedAction()},_createTitleClickAction:function(){this._titleClickAction=this._createActionByOption("onTitleClick")},_createTitleHoldAction:function(){this._titleHoldAction=this._createActionByOption("onTitleHold")},_createTitleRenderedAction:function(){this._titleRenderedAction=this._createActionByOption("onTitleRendered")},_renderContent:function(){var e=this;this.callBase(),this.option("templatesRenderAsynchronously")&&(this._resizeEventTimer=setTimeout(function(){e._updateLayout()},0))},_renderLayout:function(){if(this._tabs)return void this._updateLayout();var e=this.$element();this._$tabContainer=(0,o.default)("<div>").addClass(y).appendTo(e);var t=(0,o.default)("<div>").appendTo(this._$tabContainer);this._tabs=this._createComponent(t,d.default,this._tabConfig()),this._$container=(0,o.default)("<div>").addClass(b).appendTo(e),this._$container.append(this._$wrapper),this._updateLayout()},_updateLayout:function(){if((0,m.hasWindow)()){var e=this._$tabContainer.outerHeight();this._$container.css({marginTop:-e,paddingTop:e})}},_refreshActiveDescendant:function(){if(this._tabs){var e=this._tabs,t=e.itemElements(),n=(0,o.default)(t[e.option("selectedIndex")]),i=this.getFocusedItemId();this.setAria("controls",void 0,(0,o.default)(t)),this.setAria("controls",i,n)}},_tabConfig:function(){return{selectOnFocus:!0,focusStateEnabled:this.option("focusStateEnabled"),hoverStateEnabled:this.option("hoverStateEnabled"),repaintChangesOnly:this.option("repaintChangesOnly"),tabIndex:this.option("tabIndex"),selectedIndex:this.option("selectedIndex"),badgeExpr:this.option("badgeExpr"),onItemClick:this._titleClickAction.bind(this),onItemHold:this._titleHoldAction.bind(this),itemHoldTimeout:this.option("itemHoldTimeout"),onSelectionChanged:function(e){this.option("selectedIndex",e.component.option("selectedIndex")),this._refreshActiveDescendant()}.bind(this),onItemRendered:this._titleRenderedAction.bind(this),itemTemplate:this._getTemplateByOption("itemTitleTemplate"),items:this.option("items"),noDataText:null,scrollingEnabled:this.option("scrollingEnabled"),scrollByContent:this.option("scrollByContent"),showNavButtons:this.option("showNavButtons"),itemTemplateProperty:"tabTemplate",loopItemFocus:this.option("loop"),selectionRequired:!0,onOptionChanged:function(e){if("focusedElement"===e.name)if(e.value){var t=(0,o.default)(e.value),n=this._itemElements().eq(t.index());this.option("focusedElement",(0,p.getPublicElement)(n))}else this.option("focusedElement",e.value)}.bind(this),onFocusIn:function(e){this._focusInHandler(e.event)}.bind(this),onFocusOut:function(e){this._isFocusOutHandlerExecuting||this._focusOutHandler(e.event)}.bind(this)}},_renderFocusTarget:function(){this._focusTarget().attr("tabIndex",-1)},_updateFocusState:function(e,t){this.callBase(e,t),e.target===this._tabs._focusTarget().get(0)&&this._toggleFocusClass(t,this._focusTarget())},_focusOutHandler:function(e){this._isFocusOutHandlerExecuting=!0,this.callBase.apply(this,arguments),this._tabs._focusOutHandler(e),this._isFocusOutHandlerExecuting=!1},_setTabsOption:function(e,t){this._tabs&&this._tabs.option(e,t)},_visibilityChanged:function(e){e&&(this._tabs._dimensionChanged(),this._updateLayout())},registerKeyHandler:function(e,t){this.callBase(e,t),this._tabs&&this._tabs.registerKeyHandler(e,t)},repaint:function(){this.callBase(),this._tabs.repaint()},_optionChanged:function(e){var t=e.name,n=e.value,i=e.fullName;switch(t){case"dataSource":this.callBase(e);break;case"items":this._setTabsOption(t,this.option(t)),this._updateLayout(),this.option("repaintChangesOnly")||this._tabs.repaint(),this.callBase(e);break;case"width":this.callBase(e),this._tabs.repaint();break;case"selectedIndex":case"selectedItem":if(this._setTabsOption(i,n),this.callBase(e),this.option("focusStateEnabled")===!0){var a=this.option("selectedIndex"),r=this._itemElements().eq(a);this.option("focusedElement",(0,p.getPublicElement)(r))}break;case"itemHoldTimeout":case"focusStateEnabled":case"hoverStateEnabled":this._setTabsOption(i,n),this.callBase(e);break;case"scrollingEnabled":case"scrollByContent":case"showNavButtons":this._setTabsOption(i,n);break;case"focusedElement":var s=n?(0,o.default)(n).index():n,l=n?this._tabs._itemElements().eq(s):n;this._setTabsOption("focusedElement",(0,p.getPublicElement)(l)),this.callBase(e);break;case"itemTitleTemplate":this._setTabsOption("itemTemplate",this._getTemplateByOption("itemTitleTemplate"));break;case"onTitleClick":this._createTitleClickAction(),this._setTabsOption("onItemClick",this._titleClickAction.bind(this));break;case"onTitleHold":this._createTitleHoldAction(),this._setTabsOption("onItemHold",this._titleHoldAction.bind(this));break;case"onTitleRendered":this._createTitleRenderedAction(),this._setTabsOption("onItemRendered",this._titleRenderedAction.bind(this));break;case"loop":this._setTabsOption("loopItemFocus",n);break;case"badgeExpr":this._invalidate();break;default:this.callBase(e)}},_clean:function(){clearTimeout(this._resizeEventTimer),this.callBase()}});w.ItemClass=h.default,(0,u.default)("dxTabPanel",w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=i(n(5)),r=n(4),s=n(0),l=n(3),u=i(n(100)),c=i(n(58)),d="dx-validationsummary",h=d+"-item",f=d+"-item-data",p=c.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{focusStateEnabled:!1,noDataText:null})},_setOptionsByReference:function(){this.callBase(),(0,s.extend)(this._optionsByReference,{validationGroup:!0})},_init:function(){this.callBase(),this._initGroupRegistration()},_initGroupRegistration:function(){var e=this.$element(),t=this.option("validationGroup")||u.default.findGroup(e,this._modelByElement(e)),n=u.default.addGroup(t);this._unsubscribeGroup(),this._groupWasInit=!0,this._validationGroup=t,this.groupSubscription=this._groupValidationHandler.bind(this),n.on("validated",this.groupSubscription)},_unsubscribeGroup:function(){var e=u.default.getGroupConfig(this._validationGroup);e&&e.off("validated",this.groupSubscription)},_getOrderedItems:function(e,t){var n=[];return(0,l.each)(e,function(e,i){var o=(0,r.grep)(t,function(e){if(e.validator===i)return!0});o.length&&(n=n.concat(o))}),n},_groupValidationHandler:function(e){var t=this,n=this._getOrderedItems(e.validators,(0,l.map)(e.brokenRules,function(e){return{text:e.message,validator:e.validator,index:e.index}}));this.validators=e.validators,(0,l.each)(this.validators,function(e,n){if(n._validationSummary!==t){var i=t._itemValidationHandler.bind(t),o=function(){n.off("validated",i),n._validationSummary=null,i=null};n.on("validated",i),n.on("disposing",o),n._validationSummary=t}}),this.option("items",n)},_itemValidationHandler:function(e){for(var t=e.isValid,n=e.validator,i=e.brokenRules,o=this.option("items"),a=!1,s=0,u=function(){var e=o[s];if(e.validator===n){var l=(0,r.grep)(i||[],function(t){return t.index===e.index})[0];if(t||!l)return o.splice(s,1),a=!0,"continue";l.message!==e.text&&(e.text=l.message,a=!0)}s++};s<o.length;){u()}(0,l.each)(i,function(e,t){var i=(0,r.grep)(o,function(e){return e.validator===n&&e.index===t.index})[0];i||(o.push({text:t.message,validator:n,index:t.index}),a=!0)}),a&&(o=this._getOrderedItems(this.validators,o),this.option("items",o))},_initMarkup:function(){this.$element().addClass(d),this.callBase()},_optionChanged:function(e){switch(e.name){case"validationGroup":this._initGroupRegistration();break;default:this.callBase(e)}},_itemClass:function(){return h},_itemDataKey:function(){return f},_postprocessRenderItem:function(e){a.default.on(e.itemElement,"click",function(){e.itemData.validator&&e.itemData.validator.focus&&e.itemData.validator.focus()})},_dispose:function(){this.callBase(),this._unsubscribeGroup()}});(0,o.default)("dxValidationSummary",p);var g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.prepareScrollData=t.allowScroll=void 0;var o=i(n(2)),a=n(8),r=function(e,t,n){var i=(0,o.default)(e),a=n?i.scrollLeft():i.scrollTop(),r=n?"Width":"Height",s=i.prop("scroll".concat(r))-i.prop("client".concat(r))-a;if(0===a&&0===s)return!1;var l=0===a&&t>=0,u=0===s&&t<=0,c=a>0&&s>0;return!!(l||u||c)||void 0};t.allowScroll=r;var s=function(e,t){var n=(0,o.default)(e),i=function(n){return!t||(0,o.default)(n).is(e)};return{validate:function(e){if((0,a.isDxMouseWheelEvent)(e)&&i(e.target))return!!r(n,-e.delta,e.shiftKey)&&(e._needSkipEvent=!0,!0)}}};t.prepareScrollData=s},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=function(){function e(){i(this,e),this._converters={}}return a(e,[{key:"addConverter",value:function(e,t){this._converters[e]=t}},{key:"getConverter",value:function(e){return this._converters[e]}}]),e}(),s=new r,l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(2)),r=i(n(13)),s=n(39),l=i(n(5)),u=i(n(9)),c=n(4),d=n(274),h=n(6),f=n(33),p=n(18),g=n(1),_=n(7),m=n(0),v=n(12),y=n(3),b=i(n(10)),x=n(8),w=n(21),C=i(n(423)),k=n(70),S=i(n(681)),D=i(n(127)),I=n(71),T=n(299),O="dxTagData",E="dx-tagbox",A="dx-tag-container",P="dx-tag",R="dx-tagbox-multi-tag",M="dx-tag-custom",B="dx-tag-remove-button",V="dx-tagbox-only-select",F="dx-tagbox-single-line",L="dx-tagbox-popup-wrapper",H="dx-tag-content",z="dx-tagbox-default-template",N="dx-tagbox-custom-template",j="dx-native-click",W="dx-texteditor-input-container",G=-.3,q=D.default.inherit({_supportedKeys:function(){var e=this,t=this.callBase(),n=function(t){return e._list._keyboardHandler(t)},i=this.option("rtlEnabled");return(0,m.extend)({},t,{backspace:function(e){if(this._isCaretAtTheStart()){this._processKeyboardEvent(e),this._isTagRemoved=!0;var t=this._$focusedTag||this._tagElements().last();this._$focusedTag&&this._moveTagFocus("prev",!0),0!==t.length&&(this._preserveFocusedTag=!0,this._removeTagElement(t),delete this._preserveFocusedTag)}},upArrow:function(i,o){return i.altKey||!e._list?t.upArrow.call(e,i):n(o)},downArrow:function(i,o){return i.altKey||!e._list?t.downArrow.call(e,i):n(o)},del:function(e){if(this._$focusedTag&&this._isCaretAtTheStart()){this._processKeyboardEvent(e),this._isTagRemoved=!0;var t=this._$focusedTag;this._moveTagFocus("next",!0),this._preserveFocusedTag=!0,this._removeTagElement(t),delete this._preserveFocusedTag}},enter:function(e,t){var i=this._list&&null!==this._list.option("focusedElement"),o=this.option("acceptCustomValue")&&!i;return o?(e.preventDefault(),void(""!==this._searchValue()&&this._customItemAddedHandler(e))):void(this.option("opened")&&(this._saveValueChangeEvent(e),n(t),e.preventDefault()))},space:function(e,t){var i=this.option("opened"),o=this._shouldRenderSearchEvent();i&&!o&&(this._saveValueChangeEvent(e),n(t),e.preventDefault())},leftArrow:function(e){if(!(!this._isCaretAtTheStart()||this._isEmpty()||this._isEditable()&&i&&!this._$focusedTag)){e.preventDefault();var t=i?"next":"prev";this._moveTagFocus(t),!this.option("multiline")&&this._scrollContainer(t)}},rightArrow:function(e){if(this._isCaretAtTheStart()&&!this._isEmpty()&&(!this._isEditable()||i||this._$focusedTag)){e.preventDefault();var t=i?"prev":"next";this._moveTagFocus(t),!this.option("multiline")&&this._scrollContainer(t)}}})},_processKeyboardEvent:function(e){e.preventDefault(),e.stopPropagation(),this._saveValueChangeEvent(e)},_isEmpty:function(){return 0===this._getValue().length},_updateTagsContainer:function(e){this._$tagsContainer=e.addClass(A).addClass(j),this._$tagsContainer.parent().addClass(j)},_allowSelectItemByTab:function(){return!1},_isCaretAtTheStart:function(){var e=(0,C.default)(this._input());return 0===e.start&&0===e.end},_moveTagFocus:function(e,t){if(!this._$focusedTag){var n=this._tagElements();return this._$focusedTag="next"===e?n.first():n.last(),void this._toggleFocusClass(!0,this._$focusedTag)}var i=this._$focusedTag[e](".".concat(P));i.length>0?this._replaceFocusedTag(i):(t||"next"===e&&this._isEditable())&&this._clearTagFocus()},_replaceFocusedTag:function(e){this._toggleFocusClass(!1,this._$focusedTag),this._$focusedTag=e,this._toggleFocusClass(!0,this._$focusedTag)},_clearTagFocus:function(){this._$focusedTag&&(this._toggleFocusClass(!1,this._$focusedTag),delete this._$focusedTag)},_focusClassTarget:function(e){return e&&e.length&&e[0]!==this._focusTarget()[0]?e:this.callBase()},_scrollContainer:function(e){if(!this.option("multiline")&&(0,_.hasWindow)()&&this._$tagsContainer){var t=this._getScrollPosition(e);this._$tagsContainer.scrollLeft(t)}},_getScrollPosition:function(e){return"start"===e||"end"===e?this._getBorderPosition(e):this._$focusedTag?this._getFocusedTagPosition(e):this._getBorderPosition("end")},_getBorderPosition:function(e){var t=this.option("rtlEnabled"),n="end"===e^t,i=(0,S.default)(),o=t&&i.decreasing^i.positive,a=!t||i.positive?1:-1;return n^!o?0:a*(this._$tagsContainer.get(0).scrollWidth-this._$tagsContainer.outerWidth())},_getFocusedTagPosition:function(e){var t=this.option("rtlEnabled"),n="next"===e^t,i=this._$focusedTag.position(),o=i.left,a=this._$tagsContainer.scrollLeft();if(n&&(o+=this._$focusedTag.outerWidth(!0)-this._$tagsContainer.outerWidth()),n^o<0){var r=(0,S.default)(),s=t&&!r.decreasing&&r.positive?-1:1;a+=o*s}return a},_setNextValue:c.noop,_getDefaultOptions:function(){return(0,m.extend)(this.callBase(),{value:[],showDropDownButton:!1,maxFilterLength:1500,tagTemplate:"tag",selectAllText:b.default.format("dxList-selectAll"),hideSelectedItems:!1,selectedItems:[],selectAllMode:"page",onSelectAllValueChanged:null,maxDisplayedTags:void 0,showMultiTagOnly:!0,onMultiTagPreparing:null,multiline:!0,useSubmitBehavior:!0})},_init:function(){this.callBase(),this._selectedItems=[],this._initSelectAllValueChangedAction()},_initActions:function(){this.callBase(),this._initMultiTagPreparingAction()},_initMultiTagPreparingAction:function(){this._multiTagPreparingAction=this._createActionByOption("onMultiTagPreparing",{beforeExecute:function(e){this._multiTagPreparingHandler(e.args[0])}.bind(this),excludeValidators:["disabled","readOnly"]})},_multiTagPreparingHandler:function(e){var t=this._getValue(),n=t.length;this.option("showMultiTagOnly")?e.text=b.default.getFormatter("dxTagBox-selected")(n):e.text=b.default.getFormatter("dxTagBox-moreSelected")(n-this.option("maxDisplayedTags")+1)},_initDynamicTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({tag:new I.BindableTemplate(function(e,t){var n=(0,a.default)("<div>").addClass(H);(0,a.default)("<span>").text(t.text||t).appendTo(n),(0,a.default)("<div>").addClass(B).appendTo(n),e.append(n)},["text"],this.option("integrationOptions.watchMethod"),{text:this._displayGetter})})},_toggleSubmitElement:function(e){e?(this._renderSubmitElement(),this._setSubmitValue()):(this._$submitElement&&this._$submitElement.remove(),delete this._$submitElement)},_renderSubmitElement:function(){this.option("useSubmitBehavior")&&(this._$submitElement=(0,a.default)("<select>").attr("multiple","multiple").css("display","none").appendTo(this.$element()))},_setSubmitValue:function(){if(this.option("useSubmitBehavior")){for(var e=this._getValue(),t=[],n=0,i=e.length;n<i;n++){var o=this._shouldUseDisplayValue(e[n]);t.push((0,a.default)("<option>").val(o?this._displayGetter(e[n]):e[n]).attr("selected","selected"))}this._getSubmitElement().empty().append(t)}},_initMarkup:function(){this._tagElementsCache=(0,a.default)();var e=!this.option("multiline");this.$element().addClass(E).toggleClass(V,!(this.option("searchEnabled")||this.option("acceptCustomValue"))).toggleClass(F,e),this._initTagTemplate(),this.callBase()},_render:function(){this.callBase(),this._renderTagRemoveAction(),this._renderSingleLineScroll(),this._scrollContainer("start")},_initTagTemplate:function(){this._tagTemplate=this._getTemplateByOption("tagTemplate")},_renderField:function(){var e=!(0,g.isDefined)(this.option("fieldTemplate"));this.$element().toggleClass(z,e).toggleClass(N,!e),this.callBase()},_renderTagRemoveAction:function(){var e=this._createAction(this._removeTagHandler.bind(this)),t=(0,x.addNamespace)(w.name,"dxTagBoxTagRemove");l.default.off(this._$tagsContainer,t),l.default.on(this._$tagsContainer,t,".".concat(B),function(t){e({event:t})}),this._renderTypingEvent()},_renderSingleLineScroll:function(){var e=(0,x.addNamespace)("dxmousewheel",this.NAME),t=this.$element(),n=this.option("multiline");return l.default.off(t,e),"desktop"!==r.default.real().deviceType?void(this._$tagsContainer&&this._$tagsContainer.css("overflowX",n?"":"auto")):void(n||l.default.on(t,e,this._tagContainerMouseWheelHandler.bind(this)))},_tagContainerMouseWheelHandler:function(e){var t=this._$tagsContainer.scrollLeft(),n=e.delta*G;if((0,T.allowScroll)(this._$tagsContainer,n,!0))return this._$tagsContainer.scrollLeft(t+n),!1},_renderTypingEvent:function(){var e=this;l.default.on(this._input(),(0,x.addNamespace)("keydown",this.NAME),function(t){var n=(0,x.normalizeKeyName)(t);!e._isControlKey(n)&&e._isEditable()&&e._clearTagFocus()})},_popupWrapperClass:function(){return this.callBase()+" "+L},_renderInput:function(){this.callBase(),this._renderPreventBlur(this._inputWrapper())},_renderInputValueImpl:function(){return this._renderMultiSelect()},_loadInputValue:function(){return(0,h.when)()},_clearTextValue:function(){this._input().val(""),this._toggleEmptinessEventHandler(),this.option("text","")},_focusInHandler:function(e){this._preventNestedFocusEvent(e)||this._scrollContainer("end"),this.callBase(e)},_renderInputValue:function(){return this.option("displayValue",this._searchValue()),this.callBase()},_restoreInputText:function(e){e||this._clearTextValue()},_focusOutHandler:function(e){this._preventNestedFocusEvent(e)||(this._clearTagFocus(),this._scrollContainer("start")),this.callBase(e)},_getFirstPopupElement:function(){return this.option("showSelectionControls")?this._list.$element():this.callBase()},_initSelectAllValueChangedAction:function(){this._selectAllValueChangeAction=this._createActionByOption("onSelectAllValueChanged")},_renderList:function(){var e=this;this.callBase(),this._setListDataSourceFilter(),this.option("showSelectionControls")&&(this._list.registerKeyHandler("tab",function(t){return e._popupElementTabHandler(t)}),this._list.registerKeyHandler("escape",function(t){return e._popupElementEscHandler(t)}))},_canListHaveFocus:function(){return"useButtons"===this.option("applyValueMode")},_listConfig:function(){var e=this,t=this.option("showSelectionControls")?"all":"multiple";return(0,m.extend)(this.callBase(),{selectionMode:t,selectAllText:this.option("selectAllText"),onSelectAllValueChanged:function(t){var n=t.value;e._selectAllValueChangeAction({value:n})},selectAllMode:this.option("selectAllMode"),selectedItems:this._selectedItems,onFocusedItemChanged:null})},_renderMultiSelect:function(){var e=this,t=new h.Deferred;return this._updateTagsContainer(this._$textEditorInputContainer),this._renderInputSize(),this._renderTags().done(function(){e._popup&&e._popup.refreshPosition(),t.resolve()}).fail(t.reject),t.promise()},_listItemClickHandler:function(e){!this.option("showSelectionControls")&&this._clearTextValue(),"useButtons"!==this.option("applyValueMode")&&(this.callBase(e),this._saveValueChangeEvent(void 0))},_shouldClearFilter:function(){var e=this.callBase(),t=this.option("showSelectionControls");return!t&&e},_renderInputSize:function(){var e=this._input(),t=e.val(),n=(0,g.isString)(t)&&t,i=5,o="",a="",r=this.option("searchEnabled")||this.option("acceptCustomValue");if(n&&r){var s=(0,f.createTextElementHiddenCopy)(e,t,{includePaddings:!0});s.insertAfter(e),o=s.outerWidth()+i,s.remove()}else t||(a=1);e.css("width",o),e.attr("size",a)},_renderInputSubstitution:function(){this.callBase(),this._renderInputSize()},_getValue:function(){return this.option("value")||[]},_multiTagRequired:function(){var e=this._getValue(),t=this.option("maxDisplayedTags");return(0,g.isDefined)(t)&&e.length>t},_renderMultiTag:function(e){var t=(0,a.default)("<div>").addClass(P).addClass(R),n={multiTagElement:(0,p.getPublicElement)(t),selectedItems:this.option("selectedItems")};return this._multiTagPreparingAction(n),!n.cancel&&(t.data(O,n.text),t.insertBefore(e),this._tagTemplate.render({model:n.text,container:(0,p.getPublicElement)(t)}),t)},_getFilteredItems:function(e){var t=this,n=new d.SelectionFilterCreator(e),i=this._list&&this._list.option("selectedItems")||this.option("selectedItems"),o=n.getLocalFilter(this._valueGetter),a=i.filter(o),r=a.length===e.length,s=new h.Deferred;if(!this._isDataSourceChanged&&r)return s.resolve(a).promise();var l=this._dataSource,u=l.filter(),c=n.getCombinedFilter(this.option("valueExpr"),u),f=encodeURI(JSON.stringify(c)).length,p=f>this.option("maxFilterLength")?void 0:c,g=l.loadOptions(),_=g.customQueryParams,m=g.expand;return l.store().load({filter:p,customQueryParams:_,expand:m}).done(function(e,n){if(t._isDataSourceChanged=!1,t._disposed)return void s.reject();var i=(0,k.normalizeLoadResult)(e,n),a=i.data,r=l._applyMapFunction(a);s.resolve(r.filter(o))}).fail(s.reject),s.promise()},_createTagsData:function(e,t){var n=this,i=[],o={},a="this"===this._valueGetterExpr(),r={};t.forEach(function(e){var t=a?JSON.stringify(e):n._valueGetter(e);r[t]=e});var s=[];e.forEach(function(e,t){var l=r[a?JSON.stringify(e):e];if(a&&!(0,g.isDefined)(l))s.push(n._loadItem(e,o).always(function(o){var a=n._createTagData(i,o,e,t);i.splice(t,0,a)}));else{var u=n._createTagData(i,l,e,t);i.splice(t,0,u)}});var l=new h.Deferred;return h.when.apply(this,s).always(function(){l.resolve(i)}),l.promise()},_createTagData:function(e,t,n,i){if((0,g.isDefined)(t))return this._selectedItems.push(t),t;var o=this.option("selectedItem"),a=this._valueGetter(o)===n?o:n;return a},_isGroupedData:function(){return this.option("grouped")&&!this._dataSource.group()},_getItemsByValues:function(e){var t=[];return e.forEach(function(e){var n=this._getItemFromPlain(e);(0,g.isDefined)(n)&&t.push(n)}.bind(this)),t},_getFilteredGroupedItems:function(e){var t=new h.Deferred;return this._dataSource.items().length?t.resolve(this._getItemsByValues(e)):this._dataSource.load().done(function(){t.resolve(this._getItemsByValues(e))}.bind(this)).fail(t.resolve([])),t.promise()},_loadTagsData:function(){var e=this,t=this._getValue(),n=new h.Deferred;this._selectedItems=[];var i=this._isGroupedData()?this._getFilteredGroupedItems(t):this._getFilteredItems(t);return i.done(function(i){var o=e._createTagsData(t,i);o.always(function(e){
n.resolve(e)})}).fail(n.reject.bind(this)),n.promise()},_renderTags:function(){var e=this,t=new h.Deferred,n=!1;return this._shouldGetItemsFromPlain(this._valuesToUpdate)&&(this._selectedItems=this._getItemsFromPlain(this._valuesToUpdate),this._selectedItems.length===this._valuesToUpdate.length&&(this._renderTagsImpl(this._selectedItems),n=!0,t.resolve())),n||this._loadTagsData().done(function(n){return e._disposed?void t.reject():(e._renderTagsImpl(n),void t.resolve())}).fail(t.reject),t.promise()},_renderTagsImpl:function(e){this._renderTagsCore(e),this._renderEmptyState(),this._preserveFocusedTag||this._clearTagFocus()},_shouldGetItemsFromPlain:function(e){return e&&this._dataSource.isLoaded()&&e.length<=this._getPlainItems().length},_getItemsFromPlain:function(e){var t=this,n=this._getPlainItems(),i=n.filter(function(n){for(var i,o=0;o<e.length;o++)if(i=e[o],(0,g.isObject)(i)){if(t._isValueEquals(n,i))return!0}else if(t._isValueEquals(t._valueGetter(n),i))return!0;return!1},this);return i},_integrateInput:function(){this.callBase(),this._updateTagsContainer((0,a.default)(".".concat(W))),this._renderTagRemoveAction()},_renderTagsCore:function(e){var t=this;this._renderField(),this.option("selectedItems",this._selectedItems.slice()),this._cleanTags();var n=this._multiTagRequired()&&this._renderMultiTag(this._input()),i=this.option("showMultiTagOnly"),o=this.option("maxDisplayedTags");e.forEach(function(e,a){return!(n&&i||n&&!i&&a-o>=-1)&&void t._renderTag(e,n||t._input())}),this._isFocused()&&this._scrollContainer("end"),this._refreshTagElements()},_cleanTags:function(){if(this._multiTagRequired())this._tagElements().remove();else{var e=this._tagElements(),t=this._getValue();(0,y.each)(e,function(e,n){var i=(0,a.default)(n),o=(0,v.inArray)(i.data(O),t);o<0&&i.remove()})}},_renderEmptyState:function(){var e=!(this._getValue().length||this._selectedItems.length||this._searchValue());this._toggleEmptiness(e),this._renderDisplayText()},_renderDisplayText:function(){this._renderInputSize()},_refreshTagElements:function(){this._tagElementsCache=this.$element().find(".".concat(P))},_tagElements:function(){return this._tagElementsCache},_applyTagTemplate:function(e,t){this._tagTemplate.render({model:e,container:(0,p.getPublicElement)(t)})},_renderTag:function(e,t){var n=this._valueGetter(e);if((0,g.isDefined)(n)){var i=this._getTag(n),o=this._displayGetter(e),a=this._getItemModel(e,o);i?((0,g.isDefined)(o)&&(i.empty(),this._applyTagTemplate(a,i)),i.removeClass(M)):(i=this._createTag(n,t),(0,g.isDefined)(e)?this._applyTagTemplate(a,i):(i.addClass(M),this._applyTagTemplate(n,i)))}},_getItemModel:function(e,t){return(0,g.isObject)(e)&&t?e:(0,c.ensureDefined)(t,"")},_getTag:function(e){for(var t=this._tagElements(),n=t.length,i=!1,o=0;o<n;o++){var r=t[o],l=(0,s.data)(r,O);if(e===l||(0,c.equalByValue)(e,l)){i=(0,a.default)(r);break}}return i},_createTag:function(e,t){return(0,a.default)("<div>").addClass(P).data(O,e).insertBefore(t)},_toggleEmptinessEventHandler:function(){this._toggleEmptiness(!this._getValue().length&&!this._searchValue().length)},_customItemAddedHandler:function(e){this.callBase(e),this._clearTextValue()},_removeTagHandler:function(e){var t=e.event;t.stopPropagation(),this._saveValueChangeEvent(t);var n=(0,a.default)(t.target).closest(".".concat(P));this._removeTagElement(n)},_removeTagElement:function(e){if(e.hasClass(R))return void(this.option("showMultiTagOnly")?this.reset():this.option("value",this._getValue().slice(0,this.option("maxDisplayedTags"))));var t=e.data(O);this._removeTagWithUpdate(t),this._refreshTagElements()},_updateField:c.noop,_removeTagWithUpdate:function(e){var t=this._getValue().slice();this._removeTag(t,e),this.option("value",t),0===t.length&&this._clearTagFocus()},_getCurrentValue:function(){return this._lastValue()},_selectionChangeHandler:function(e){var t=this;if("useButtons"!==this.option("applyValueMode")){var n=this._getValue().slice();if((0,y.each)(e.removedItems||[],function(e,i){t._removeTag(n,t._valueGetter(i))}),(0,y.each)(e.addedItems||[],function(e,i){t._addTag(n,t._valueGetter(i))}),this._updateWidgetHeight(),!(0,c.equalByValue)(this._list.option("selectedItemKeys"),this.option("value"))){var i=this._list._getSelectionChangeEvent();i&&this._saveValueChangeEvent(i),this.option("value",n)}this._list._saveSelectionChangeEvent(void 0)}},_removeTag:function(e,t){var n=this._valueIndex(t,e);n>=0&&e.splice(n,1)},_addTag:function(e,t){var n=this._valueIndex(t);n<0&&e.push(t)},_fieldRenderData:function(){return this._selectedItems.slice()},_completeSelection:function(e){this.option("showSelectionControls")||this._setValue(e)},_setValue:function(e){if(null!==e){var t="useButtons"===this.option("applyValueMode"),n=this._valueIndex(e),i=(t?this._list.option("selectedItemKeys"):this._getValue()).slice();n>=0?i.splice(n,1):i.push(e),"useButtons"===this.option("applyValueMode")?this._list.option("selectedItemKeys",i):this.option("value",i)}},_isSelectedValue:function(e,t){return this._valueIndex(e,null,t)>-1},_valueIndex:function(e,t,n){var i=this,a=-1;return n&&"object"!==o(e)&&(n.indexByValues||(n.indexByValues={},t=t||this._getValue(),t.forEach(function(e,t){n.indexByValues[e]=t})),e in n.indexByValues)?n.indexByValues[e]:(t=t||this._getValue(),(0,y.each)(t,function(t,n){if(i._isValueEquals(e,n))return a=t,!1}),a)},_lastValue:function(){var e=this._getValue(),t=e[e.length-1];return(0,g.isDefined)(t)?t:null},_valueChangeEventHandler:c.noop,_shouldRenderSearchEvent:function(){return this.option("searchEnabled")||this.option("acceptCustomValue")},_searchHandler:function(e){this.option("searchEnabled")&&e&&!this._isTagRemoved&&(this.callBase(e),this._setListDataSourceFilter()),this._updateWidgetHeight(),delete this._isTagRemoved},_updateWidgetHeight:function(){var e=this.$element(),t=e.height();this._renderInputSize();var n=e.height();this._popup&&this.option("opened")&&this._isEditable()&&n!==t&&this._popup.repaint()},_refreshSelected:function(){var e;(null===(e=this._list)||void 0===e?void 0:e.getDataSource())&&this._list.option("selectedItems",this._selectedItems)},_resetListDataSourceFilter:function(){var e=this._getDataSource();e&&(delete this._userFilter,e.filter(null),e.reload())},_setListDataSourceFilter:function(){if(this.option("hideSelectedItems")&&this._list){var e=this._getDataSource();if(e){var t=this._valueGetterExpr();if((0,g.isString)(t)&&"this"!==t){var n=this._dataSourceFilterExpr();void 0===this._userFilter&&(this._userFilter=e.filter()||null),this._userFilter&&n.push(this._userFilter),n.length?e.filter(n):e.filter(null)}else e.filter(this._dataSourceFilterFunction.bind(this));e.load()}}},_dataSourceFilterExpr:function(){var e=this,t=[];return(0,y.each)(this._getValue(),function(n,i){t.push(["!",[e._valueGetterExpr(),i]])}),t},_dataSourceFilterFunction:function(e){var t=this,n=this._valueGetter(e),i=!0;return(0,y.each)(this._getValue(),function(e,o){if(t._isValueEquals(o,n))return i=!1,!1}),i},_dataSourceChangedHandler:function(){this._list&&(this._isDataSourceChanged=!0),this.callBase.apply(this,arguments)},_applyButtonHandler:function(){this.option("value",this._getSortedListValues()),this._clearTextValue(),this._clearFilter(),this.callBase()},_getSortedListValues:function(){var e=this._getListValues(),t=this.option("value")||[],n=e.length?t.filter(function(t){return e.indexOf(t)!==-1}):[],i=n.length?e.filter(function(e){return t.indexOf(e)===-1}):e;return n.concat(i)},_getListValues:function(){var e=this;if(!this._list)return[];var t=this._getPlainItems(this._list.option("selectedItems")),n=[];return(0,y.each)(t,function(t,i){n[t]=e._valueGetter(i)}),n},_setListDataSource:function(){var e=this._getValue();this.callBase(),e!==this.option("value")&&this.option("value",e),this._refreshSelected()},_renderOpenedState:function(){this.callBase(),"useButtons"!==this.option("applyValueMode")||this.option("opened")||this._refreshSelected()},reset:function(){this._restoreInputText();var e=this._getDefaultOptions().value,t=this.option("value");e&&0===e.length&&t&&e.length===t.length||this.callBase()},_clean:function(){this.callBase(),delete this._defaultTagTemplate,delete this._valuesToUpdate,delete this._tagTemplate},_removeDuplicates:function(e,t){var n=this,i=[];return(0,y.each)(e,function(e,o){var a=t.filter(function(e){return n._valueGetter(o)===n._valueGetter(e)});a.length||i.push(o)}),i},_optionChanged:function(e){switch(e.name){case"onSelectAllValueChanged":this._initSelectAllValueChangedAction();break;case"onMultiTagPreparing":this._initMultiTagPreparingAction(),this._renderTags();break;case"hideSelectedItems":e.value?this._setListDataSourceFilter():this._resetListDataSourceFilter();break;case"useSubmitBehavior":this._toggleSubmitElement(e.value);break;case"displayExpr":this.callBase(e),this._initTemplates(),this._invalidate();break;case"tagTemplate":this._initTagTemplate(),this._invalidate();break;case"selectAllText":this._setListOption("selectAllText",this.option("selectAllText"));break;case"value":this._valuesToUpdate=null===e||void 0===e?void 0:e.value,this.callBase(e),this._valuesToUpdate=void 0,this._setListDataSourceFilter();break;case"maxDisplayedTags":case"showMultiTagOnly":this._renderTags();break;case"selectAllMode":this._setListOption(e.name,e.value);break;case"selectedItem":break;case"selectedItems":this._selectionChangedAction({addedItems:this._removeDuplicates(e.value,e.previousValue),removedItems:this._removeDuplicates(e.previousValue,e.value)});break;case"multiline":this.$element().toggleClass(F,!e.value),this._renderSingleLineScroll();break;case"maxFilterLength":break;default:this.callBase(e)}},_getActualSearchValue:function(){return this.callBase()||this._searchValue()},_popupHidingHandler:function(){this.callBase(),this._clearFilter()}});(0,u.default)("dxTagBox",q);var $=q;t.default=$,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}t.registerKeyboardAction=void 0;var r=a(n(128)),s=function(e,t,n,i,o){var a=t.getController("keyboardNavigation");if(!(t.option("useLegacyKeyboardNavigation")||a&&!a.isKeyboardEnabled())){var s=function(e){t.executeAction("onKeyDown",e)};t.createAction("onKeyDown"),r.registerKeyboardAction(e,t,n,i,o,s)}};t.registerKeyboardAction=s},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){for(var n=r.default.normalizeSortingInfo(t.group),i=[],o=0;o<e.length;o++)i.push([n[o].selector,"=",e[o]]);return t.filter&&i.push(t.filter),a.default.combineFilters(i)}t.createGroupFilter=o;var a=i(n(37)),r=i(n(41))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=e.dataField||e.groupName||"",n=(e.summaryType||"").toLowerCase();return(0,f.isString)(e.groupInterval)&&(t+="_"+e.groupInterval),n&&"custom"!==n?(n=n.replace(/^./,n[0].toUpperCase()),t.length&&(n=" ("+n+")")):n="",(0,b.titleize)(t)+n}function a(e,t){var n=e._initProperties||{};(0,_.each)(t,function(t,i){Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])})}function r(e,t){a(e,t),(0,f.isDefined)(e.caption)||(0,D.setFieldProperty)(e,"caption",o(e))}function s(e){return e.some(function(e){return e.summaryDisplayMode||e.calculateSummaryValue})}function l(e){return e.some(function(e){return!!e.runningTotal})}function u(e){return e.rows.length||e.columns.length||e.values.length}t.default=void 0;var c=n(70),d=i(n(102)),h=n(4),f=n(1),p=n(0),g=n(12),_=n(3),m=n(6),v=i(n(15)),y=n(92),b=n(40),x=n(772),w=i(n(773)),C=n(775),k=n(508),S=n(776),D=n(89),I={row:"rows",column:"columns",data:"values",filter:"filters"},T=["area","areaIndex","sortOrder","filterType","filterValues","sortBy","sortBySummaryField","sortBySummaryPath","expanded","summaryType","summaryDisplayMode"],O=["format","selector","customizeText","caption"],E=O.concat(["allowSorting","allowSortingBySummary","allowFiltering","allowExpandAll"]),A=v.default.inherit(function(){function e(e,t){var n=e.remoteOperations||e.paginate?w.default:x.LocalStore;return new n((0,p.extend)((0,c.normalizeDataSourceOptions)(e),{onChanged:null,onLoadingChanged:null,onProgressChanged:t}))}function t(t,n){var i,o;return(0,f.isPlainObject)(t)&&t.load?i=e(t,n):(t&&!t.store&&(t={store:t}),o=t.store,"xmla"===o.type?i=new k.XmlaStore(o):(0,f.isPlainObject)(o)&&o.type||o instanceof d.default||Array.isArray(o)?i=e(t,n):o instanceof v.default&&(i=o)),i}function n(e,t,n){for(var i=0;i<n;i++)if(!e[i]||!t[i]||e[i].index!==t[i].index)return!1;return!0}function i(e,t,i,o){var a=[],r=t&&t[i]||[],s=o&&o[i]||[];return(0,D.foreachTree)(e[i],function(e){var t=e[0],i=(0,D.createPath)(e);t.children&&r[i.length-1]&&!r[i.length-1].expanded&&i.length<r.length&&(!o||n(r,s,i.length))&&a.push(i.slice())},!0),a}function o(e,t,n,i){return t?(0,_.each)(i,function(i,o){if(n)e[o]=t[o];else{if(("summaryType"===o||"summaryDisplayMode"===o)&&void 0===t[o])return;(0,D.setFieldProperty)(e,o,t[o])}}):a(e,i),e}function b(e,t){var n=[];return(0,_.each)(e,function(e,i){n.push(o({dataField:i.dataField,name:i.name},i,!0,t))}),n}function A(e){return e.name?e.name:e.dataField+""}function P(e,t){var n=[];return(0,_.each)(e||[],function(e,i){A(i)===t&&n.push(i)}),n}function R(e,t){return e=e||[],(0,_.each)(t,function(t,n){o(n,e[t],!1,T),r(n,O)}),t}function M(e,t){e=e||[];var n,i={};return(0,_.each)(t,function(e,o){n=A(o),i[n]||(i[n]=P(t,A(o)))}),(0,_.each)(i,function(t,n){R(P(e,t),n)}),t}function B(e,t){return e.filter(function(e){return e.groupName===t.groupName&&(0,f.isNumeric)(e.groupIndex)&&e.visible!==!1}).map(function(e){return(0,p.extend)(e,{areaIndex:t.areaIndex,area:t.area,expanded:(0,f.isDefined)(e.expanded)?e.expanded:t.expanded,dataField:e.dataField||t.dataField,dataType:e.dataType||t.dataType,sortBy:e.sortBy||t.sortBy,sortOrder:e.sortOrder||t.sortOrder,sortBySummaryField:e.sortBySummaryField||t.sortBySummaryField,sortBySummaryPath:e.sortBySummaryPath||t.sortBySummaryPath,visible:e.visible||t.visible,showTotals:(0,f.isDefined)(e.showTotals)?e.showTotals:t.showTotals,showGrandTotals:(0,f.isDefined)(e.showGrandTotals)?e.showGrandTotals:t.showGrandTotals})}).sort(function(e,t){return e.groupIndex-t.groupIndex})}function V(e){e.sort(function(e,t){return e.areaIndex-t.areaIndex||e.groupIndex-t.groupIndex})}function F(e,t){var n="data"===t||e.visible!==!1;return e.area===t&&!(0,f.isDefined)(e.groupIndex)&&n}function L(e,t){var n=e.groupName||"";return(e.dataField||n)+(e.groupInterval?n+e.groupInterval:"NOGROUP")+(t?"":n)}function H(e,t,n){var i=[],o={},r={},s=[],l=(0,D.getFieldsDataType)(e);return t?((0,_.each)(t,function(e,t){o[L(t,n)]=t}),(0,_.each)(e,function(e,t){var s,u=L(t,n),c=o[u]||r[u];c?(c._initProperties&&a(c,E),s=(0,p.extend)({},c,t,{_initProperties:null})):o[u]=s=t,(0,p.extend)(s,{dataType:l[t.dataField]}),delete o[u],r[u]=c,i.push(s)}),n&&(0,_.each)(o,function(e,t){i.push(t)})):i=e,i.push.apply(i,s),z(i),i}function z(e){e.forEach(function(t){if(t.groupName&&t.groupInterval&&void 0===t.groupIndex){var n=e.filter(function(e){return e.groupName===t.groupName&&(0,f.isNumeric)(e.groupIndex)}).map(function(e){return e.groupIndex}).reduce(function(e,t){return Math.max(e,t)},-1);t.groupIndex=n+1}})}function N(e){var t,n=new m.Deferred,i=e._store,o=i&&i.getFields(e._fields);return(0,m.when)(o).done(function(i){e._storeFields=i,t=H(e._fields,i,e._retrieveFields),n.resolve(t)}).fail(n.reject),n}function j(e,t,n){return(0,D.foreachTreeAsync)(e[n],function(e){var i=e[0];i.text=i.text||(0,D.formatValue)(i.value,t[n][(0,D.createPath)(e).length-1])})}function W(e,t){return(0,m.when)(j(t,e,"columns"),j(t,e,"rows"))}function G(e){var t=new m.Deferred,n={};return(0,m.when)((0,D.foreachTreeAsync)(e,function(e){var t=(0,D.createPath)(e).join(".");n[t]=e[0]})).done(t.resolve),e._cacheByPath=n,t}function q(e,t){var n=[];return(0,_.each)(e,function(){F(this,t)&&n.push(this)}),n}var $=function(e,t){if(e._cacheByPath)return e._cacheByPath[t.join(".")]||null},K=function e(t,n){var i,o,a=-1;if(t)for(i=0;i<t.length;i++)o=t[i],void 0!==o.index&&(a=Math.max(a,o.index)),o.children?a=Math.max(a,e(o.children)):o.collapsedChildren&&(a=Math.max(a,e(o.collapsedChildren)));return(0,f.isDefined)(n)&&(a=Math.max(a,n)),a},U=function(e,t,n,i){var o,a=K(n)+1,r=K(e,i)+1,s=[],l=!1,u=new m.Deferred;if(t.children&&t.children.length===n.length)for(var c=0;c<n.length;c++){var d=n[c];void 0!==d.index&&(void 0===t.children[c].index?(d.index=s[d.index]=r++,t.children[c]=d):s[d.index]=t.children[c].index)}else{for(l=!0,o=0;o<a;o++)s[o]=r++;t.children=n}return(0,m.when)((0,D.foreachTreeAsync)(t.children,function(e){l&&(e[0].index=s[e[0].index])})).done(function(){u.resolve(s)}),u},Y=function(e,t,n){var i=new m.Deferred,o=n>=0&&K(e,n)+1,a=[];return(0,m.when)((0,D.foreachTreeAsync)(e,function(e){delete e[0].collapsedChildren})).done(function(){(0,m.when)((0,D.foreachTreeAsync)(t,function(t,n){var i=t[0];if(i.index>=0){var r=$(e,(0,D.createPath)(t));if(r&&r.index>=0)a[i.index]=r.index;else if(o){var s=(0,D.createPath)(t.slice(1));r=$(e,s);var l=s.length?r&&r.children:e;l&&(l[n]=i,i.index=a[i.index]=o++)}}})).done(function(){i.resolve(a)})}),i},Z=function(e,t,n,i){var o,a,r,s,l,u,c=e.values;if(t)for(o=0;o<=t.length;o++)if(r=t[o],l=n[o],(0,f.isDefined)(l)||(l=e.grandTotalRowIndex),r&&(0,f.isDefined)(l))for(c[l]||(c[l]=[]),a=0;a<=r.length;a++)s=r[a],u=i[a],(0,f.isDefined)(u)||(u=e.grandTotalColumnIndex),(0,f.isDefined)(s)&&(0,f.isDefined)(u)&&(c[l][u]=s)};return{ctor:function(e){e=e||{},this._eventsStrategy=new y.EventsStrategy(this);var n=this,i=t(e,function(e){n._eventsStrategy.fireEvent("progressChanged",[e])});n._store=i,n._paginate=!!e.paginate,n._pageSize=e.pageSize||40,n._data={rows:[],columns:[],values:[]},n._loadingCount=0,n._isFieldsModified=!1,(0,_.each)(["changed","loadError","loadingChanged","progressChanged","fieldsPrepared","expandValueChanging"],function(t,n){var i="on"+n[0].toUpperCase()+n.slice(1);Object.prototype.hasOwnProperty.call(e,i)&&this.on(n,e[i])}.bind(this)),n._retrieveFields=!(0,f.isDefined)(e.retrieveFields)||e.retrieveFields,n._fields=e.fields||[],n._descriptions=e.descriptions?(0,p.extend)(n._createDescriptions(),e.descriptions):void 0,i||(0,p.extend)(!0,n._data,e.store||e)},getData:function(){return this._data},getAreaFields:function(e,t){var n,i=[];return t||"data"===e?(i=q(this._fields,e),V(i)):(n=this._descriptions||{},i=n[I[e]]||[]),i},fields:function(e){var t=this;return e&&(t._fields=H(e,t._storeFields,t._retrieveFields),t._fieldsPrepared(t._fields)),t._fields},field:function e(t,n){var i,o=this,a=o._fields,e=a&&a[(0,f.isNumeric)(t)?t:(0,D.findField)(a,t)];return e&&n&&((0,_.each)(n,function(t,n){var o=(0,g.inArray)(t,T)<0;if((0,D.setFieldProperty)(e,t,n,o),"sortOrder"===t){i=e.levels||[];for(var a=0;a<i.length;a++)i[a][t]=n}}),r(e,O),o._descriptions=o._createDescriptions(e),o._isFieldsModified=!0,o._eventsStrategy.fireEvent("fieldChanged",[e])),e},getFieldValues:function(e,t,n){var i,o=this,a=this._fields&&this._fields[e],r=this.store(),s=[],l={columns:s,rows:[],values:this.getAreaFields("data"),filters:t?this._fields.filter(function(e){return e!==a&&e.area&&e.filterValues&&e.filterValues.length}):[],skipValues:!0},u=new m.Deferred;return n&&(i=n.searchValue,l.columnSkip=n.skip,l.columnTake=n.take),a&&r?((0,_.each)(a.levels||[a],function(){s.push((0,p.extend)({},this,{expanded:!0,filterValues:null,sortOrder:"asc",sortBySummaryField:null,searchValue:i}))}),r.load(l).done(function(e){l.columnSkip&&(e.columns=e.columns.slice(l.columnSkip)),l.columnTake&&(e.columns=e.columns.slice(0,l.columnTake)),W(l,e),l.columnTake||o._sort(l,e),u.resolve(e.columns)}).fail(u)):u.reject(),u},reload:function(){return this.load({reload:!0})},filter:function(){var e=this._store;return e.filter.apply(e,arguments)},load:function(e){function t(){n._delayedLoadTask=void 0,n._descriptions?n._loadCore(e,i):(0,m.when)(N(n)).done(function(t){n._fieldsPrepared(t),n._loadCore(e,i)}).fail(i.reject).fail(n._loadErrorHandler)}var n=this,i=new m.Deferred;return e=e||{},n.beginLoading(),i.fail(function(e){n._eventsStrategy.fireEvent("loadError",[e])}).always(function(){n.endLoading()}),n.store()?n._delayedLoadTask=(0,h.executeAsync)(t):t(),i},createDrillDownDataSource:function(e){return this._store.createDrillDownDataSource(this._descriptions,e)},_createDescriptions:function(e){var t=this,n=t.fields(),i={rows:[],columns:[],values:[],filters:[]};(0,_.each)(["row","column","data","filter"],function(t,i){(0,g.normalizeIndexes)(q(n,i),"areaIndex",e)}),(0,_.each)(n||[],function(e,t){var o=I[t.area],a=i[o],r=t.groupName;r&&!(0,f.isNumeric)(t.groupIndex)&&(t.levels=B(n,t)),!a||r&&(0,f.isNumeric)(t.groupIndex)||t.visible===!1&&"data"!==t.area&&"filter"!==t.area||(t.levels&&a!==i.filters&&a!==i.values?(a.push.apply(a,t.levels),t.filterValues&&t.filterValues.length&&i.filters.push(t)):a.push(t))}),(0,_.each)(i,function(e,t){V(t)});var o={};return(0,_.each)(i.values,function(e,t){var a=t.calculateSummaryValue;if((0,f.isFunction)(a)){var r=(0,S.createMockSummaryCell)(i,n,o);a(r)}}),i},_fieldsPrepared:function(e){var t=this;t._fields=e,(0,_.each)(e,function(e,t){t.index=e,r(t,E)});var n=b(e,["caption"]);t._eventsStrategy.fireEvent("fieldsPrepared",[e]);for(var i=0;i<e.length;i++)e[i].caption!==n[i].caption&&(0,D.setFieldProperty)(e[i],"caption",e[i].caption,!0);t._descriptions=t._createDescriptions()},isLoading:function(){return this._loadingCount>0},state:function(e,t){var n=this;return arguments.length?(e=(0,p.extend)({rowExpandedPaths:[],columnExpandedPaths:[]},e),void(n._descriptions?(n._fields=M(e.fields,n._fields),n._descriptions=n._createDescriptions(),!t&&n.load(e)):(n.beginLoading(),(0,m.when)(N(n)).done(function(i){n._fields=M(e.fields,i),n._fieldsPrepared(i),!t&&n.load(e)}).always(function(){n.endLoading()})))):{fields:b(n._fields,T),columnExpandedPaths:i(n._data,n._descriptions,"columns",n._lastLoadOptions),rowExpandedPaths:i(n._data,n._descriptions,"rows",n._lastLoadOptions)}},beginLoading:function(){this._changeLoadingCount(1)},endLoading:function(){this._changeLoadingCount(-1)},_changeLoadingCount:function(e){var t=this.isLoading();this._loadingCount+=e;var n=this.isLoading();t^n&&this._eventsStrategy.fireEvent("loadingChanged",[n])},_hasPagingValues:function(e,t,n){var i=t+"Take",o=t+"Skip",a=this._data.values,r=this._data[t+"s"],s="row"===t?"column":"row",l=[];if(e.path&&e.area===t){var u=$(r,e.path);if(r=u&&u.children,!r)return!1}if(e.oppositePath&&e.area===s){var c=$(r,e.oppositePath);if(r=c&&c.children,!r)return!1}for(var d=e[o];d<e[o]+e[i];d++)r[d]&&l.push(r[d].index);return l.every(function(e){if(void 0!==e)return"row"===t?(a[e]||[])[n]:(a[n]||[])[e]})},_processPagingCacheByArea:function(e,t,n){var i,o=n+"Take",a=n+"Skip",r=this._data[n+"s"],s="row"===n?"column":"row";if(e[o]){if(e.path&&e.area===n){var l=$(r,e.path);r=l&&l.children||[]}if(e.oppositePath&&e.area===s){var u=$(r,e.oppositePath);r=u&&u.children||[]}do if(i=r[e[a]],i&&void 0!==i.index){if(!this._hasPagingValues(e,s,i.index))break;e[a]++,e[o]--}while(i&&void 0!==i.index&&e[o]);if(e[o]){var c=Math.floor(e[a]/t)*t,d=Math.ceil((e[a]+e[o])/t)*t;e[a]=c,e[o]=d-c}}},_processPagingCache:function(e){var t=this._pageSize;if(!(t<0))for(var n=0;n<e.length;n++)this._processPagingCacheByArea(e[n],t,"row"),this._processPagingCacheByArea(e[n],t,"column")},_loadCore:function(e,t){var n=this,o=this._store,a=this._descriptions,r=e.reload||this.paginate()&&n._isFieldsModified,s=this.paginate(),l=I[e.area];if(e=e||{},o){(0,p.extend)(e,a),e.columnExpandedPaths=e.columnExpandedPaths||i(this._data,e,"columns",n._lastLoadOptions),e.rowExpandedPaths=e.rowExpandedPaths||i(this._data,e,"rows",n._lastLoadOptions),s&&(e.pageSize=this._pageSize),l&&(e.headerName=l),n.beginLoading(),t.always(function(){n.endLoading()});var c=[e];if(n._eventsStrategy.fireEvent("customizeStoreLoadOptions",[c,r]),r||n._processPagingCache(c),c=c.filter(function(e){return!(e.rows.length&&0===e.rowTake||e.columns.length&&0===e.columnTake)}),!c.length)return void n._update(t);var d=c.map(function(e){return o.load(e)});m.when.apply(null,d).done(function(){for(var e=arguments,i=0;i<e.length;i++){var o=c[i],a=e[i],l=i===e.length-1;o.path?n.applyPartialDataSource(o.area,o.path,a,!!l&&t,o.oppositePath):s&&!r&&u(n._data)?n.mergePartialDataSource(a,!!l&&t):((0,p.extend)(n._data,a),n._lastLoadOptions=o,n._update(!!l&&t))}}).fail(t.reject)}else n._update(t)},_sort:function(e,t,n){var i=this._store;i&&!this._paginate&&(0,C.sort)(e,t,n)},paginate:function(){return this._paginate&&this._store&&this._store.supportPaging()},isEmpty:function(){var e=this.getAreaFields("data"),t=this.getData();return!e.length||!t.values.length},_update:function(e){var t=this,n=t._descriptions,i=t._data,o=n.values,a=s(o);return(0,m.when)(W(n,i),G(i.rows),G(i.columns)).done(function(){a&&(t._sort(n,i,a),!t.isEmpty()&&(0,S.applyDisplaySummaryMode)(n,i)),t._sort(n,i),!t.isEmpty()&&l(o)&&(0,S.applyRunningTotal)(n,i),t._data=i,e!==!1&&(0,m.when)(e).done(function(){t._isFieldsModified=!1,t._eventsStrategy.fireEvent("changed"),(0,f.isDefined)(t._data.grandTotalRowIndex)&&(i.grandTotalRowIndex=t._data.grandTotalRowIndex),(0,f.isDefined)(t._data.grandTotalColumnIndex)&&(i.grandTotalColumnIndex=t._data.grandTotalColumnIndex)}),e&&e.resolve(t._data)}),e},store:function(){return this._store},collapseHeaderItem:function(e,t){var n=this,i="column"===e?n._data.columns:n._data.rows,o=$(i,t),a=n.getAreaFields(e)[t.length-1];return!(!o||!o.children)&&(n._eventsStrategy.fireEvent("expandValueChanging",[{area:e,path:t,expanded:!1}]),a&&(a.expanded=!1),o.collapsedChildren=o.children,delete o.children,n._update(),n.paginate()&&n.load(),!0)},collapseAll:function(e){var t=this,n=!1,i=this.field(e)||{},o=[(0,g.inArray)(i,this.getAreaFields(i.area))];i.expanded=!1,i&&i.levels&&(o=[],i.levels.forEach(function(e){o.push((0,g.inArray)(e,t.getAreaFields(i.area))),e.expanded=!1})),(0,D.foreachTree)(this._data[i.area+"s"],function(e){var t=e[0],i=(0,D.createPath)(e);t&&t.children&&o.indexOf(i.length-1)!==-1&&(t.collapsedChildren=t.children,delete t.children,n=!0)},!0),n&&this._update()},expandAll:function(e){var t=this.field(e);t&&t.area&&(t.expanded=!0,t&&t.levels&&t.levels.forEach(function(e){e.expanded=!0}),this.load())},expandHeaderItem:function(e,t){var n=this,i="column"===e?n._data.columns:n._data.rows,o=$(i,t);if(o&&!o.children){var a=!!o.collapsedChildren,r={area:e,path:t,expanded:!0,needExpandData:!a};return n._eventsStrategy.fireEvent("expandValueChanging",[r]),a?(o.children=o.collapsedChildren,delete o.collapsedChildren,n._update()):this.store()&&n.load(r),a}return!1},mergePartialDataSource:function(e,t){var n,i,o=this,a=o._data;e&&e.values&&(e.rows=e.rows||[],e.columns=e.columns||[],n=Y(a.rows,e.rows,a.grandTotalColumnIndex),i=Y(a.columns,e.columns,a.grandTotalColumnIndex),(0,m.when)(n,i).done(function(n,i){(n.length||i.length)&&Z(a,e.values,n,i),o._update(t)}))},applyPartialDataSource:function(e,t,n,i,o){var a,r,s,l,u=this,c=u._data,d="column"===e?c.columns:c.rows,h="column"===e?c.rows:c.columns;n&&n.values&&(n.rows=n.rows||[],n.columns=n.columns||[],a=$(d,t),r=o&&$(h,o),a&&("column"===e?(l=U(d,a,n.columns,c.grandTotalColumnIndex),s=r?U(h,r,n.rows,c.grandTotalRowIndex):Y(c.rows,n.rows,c.grandTotalRowIndex)):(s=U(d,a,n.rows,c.grandTotalRowIndex),l=r?U(h,r,n.columns,c.grandTotalColumnIndex):Y(c.columns,n.columns,c.grandTotalColumnIndex)),(0,m.when)(s,l).done(function(t,o){("row"===e&&t.length||"column"===e&&o.length)&&Z(c,n.values,t,o),u._update(i)})))},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this},dispose:function(){var e=this,t=e._delayedLoadTask;this._eventsStrategy.dispose(),t&&t.abort(),this._isDisposed=!0},isDisposed:function(){return!!this._isDisposed}}}());t.default=A,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){var o=0,a=0,r=n/2;return e+i-(t+n)>1?(e>=t+n+r&&(a=parseInt((e-(t+n))/r,10)),o=t+n+r*a):e<t?(e<=t-r&&(a=parseInt((e-(t-r))/r,10)),o=t-(n-r*a)):o=t,o}t.AreaItem=void 0;var a=i(n(2)),r=i(n(15)),s=n(18),l=n(0),u=n(24),c=n(1),d="dx-expand",h=function(e){var t=0,n=e.offsetWidth;if(e.getBoundingClientRect){var i=(0,u.getBoundingRect)(e);t=i.width,t||(t=i.right-i.left),t<=n-1&&(t=n)}return t>0?t:n},f=r.default.inherit({_getRowElement:function(e){var t=this;return t._tableElement&&t._tableElement.length>0?t._tableElement[0].rows[e]:null},_createGroupElement:function(){return(0,a.default)("<div>")},_createTableElement:function(){return(0,a.default)("<table>")},_getCellText:function(e,t){var n=e.isWhiteSpace?"&nbsp":e.text||"&nbsp";return!t||n.indexOf("<")===-1&&n.indexOf(">")===-1||(n=(0,a.default)("<div>").text(n).html()),n},_getRowClassNames:function(){},_applyCustomStyles:function(e){e.cell.width&&e.cssArray.push("min-width:"+e.cell.width+"px"),e.cell.sorted&&e.classArray.push("dx-pivotgrid-sorted")},_getMainElementMarkup:function(){return"<tbody>"},_getCloseMainElementMarkup:function(){return"</tbody>"},_renderTableContent:function(e,t){var n,i,o,a,r,s,l,u,h=this,f=t.length,p=h.option("rtlEnabled"),g=[],_=h.option("encodeHtml"),m="colspan='",v="rowspan='";for(e.data("area",h._getAreaName()),e.data("data",t),e.css("width",""),g.push(h._getMainElementMarkup()),o=0;o<f;o++){n=t[o];var y=[];for(u=[],g.push("<tr "),a=0;a<n.length;a++){if(i=n[a],this._getRowClassNames(o,i,u),y.push("<td "),i){i.rowspan&&y.push(v+(i.rowspan||1)+"'"),i.colspan&&y.push(m+(i.colspan||1)+"'");var b={cellElement:s,cell:i,cellsCount:n.length,cellIndex:a,rowElement:r,rowIndex:o,rowsCount:f,rtlEnabled:p,classArray:[],cssArray:[]};h._applyCustomStyles(b),b.cssArray.length&&(y.push("style='"),y.push(b.cssArray.join(";")),y.push("'")),b.classArray.length&&(y.push("class='"),y.push(b.classArray.join(" ")),y.push("'")),y.push(">"),(0,c.isDefined)(i.expanded)&&y.push("<div class='dx-expand-icon-container'><span class='"+d+"'></span></div>"),l=this._getCellText(i,_)}else l="";y.push("<span "),(0,c.isDefined)(i.wordWrapEnabled)&&y.push("style='white-space:",i.wordWrapEnabled?"normal":"nowrap",";'"),y.push(">"+l+"</span>"),i.sorted&&y.push("<span class='dx-icon-sorted'></span>"),y.push("</td>")}u.length&&(g.push("class='"),g.push(u.join(" ")),g.push("'")),g.push(">"),g.push(y.join("")),g.push("</tr>")}g.push(this._getCloseMainElementMarkup()),e.append(g.join("")),this._triggerOnCellPrepared(e,t)},_triggerOnCellPrepared:function(e,t){var n,i,o,a,r,u,c,d=this,h=e.find("tr"),f=d._getAreaName(),p=d.option("onCellPrepared"),g=d.component._eventsStrategy.hasEvent("cellPrepared"),_=this.component._defaultActionArgs();if(p||g)for(u=0;u<t.length;u++)for(a=t[u],n=h.eq(u),c=0;c<a.length;c++)r=a[c],i=n.children().eq(c),o={area:f,rowIndex:u,columnIndex:c,cellElement:(0,s.getPublicElement)(i),cell:r},g?d.component._trigger("onCellPrepared",o):p((0,l.extend)(o,_))},_getRowHeight:function(e){var t=this._getRowElement(e),n=0,i=t.offsetHeight;if(t&&t.lastChild){if(t.getBoundingClientRect){var o=(0,u.getBoundingRect)(t);n=o.height,n<=i-1&&(n=i)}return n>0?n:i}return 0},_setRowHeight:function(e,t){var n=this._getRowElement(e);n&&(n.style.height=t+"px")},ctor:function(e){this.component=e},option:function(){return this.component.option.apply(this.component,arguments)},getRowsLength:function(){var e=this;return e._tableElement&&e._tableElement.length>0?e._tableElement[0].rows.length:0;
},getRowsHeight:function(){var e,t=this,n=[],i=t.getRowsLength();for(e=0;e<i;e++)n.push(t._getRowHeight(e));return n},setRowsHeight:function(e){var t,n=this,i=0,o=e.length;for(t=0;t<o;t++)i+=e[t],n._setRowHeight(t,e[t]);this._tableHeight=i,this._tableElement[0].style.height=i+"px"},getColumnsWidth:function(){var e,t,n,i,o=this.getRowsLength(),a=[],r=[],s=function(e,t,n,i,o){var a,r;for(a=0;a<i;a++)for(r=0;r<o;r++)e[t+a]=e[t+a]||[],e[t+a][n+r]=!0};if(o)for(e=0;e<o;e++)for(a[e]=a[e]||[],t=this._getRowElement(e),n=0;n<t.cells.length;n++){for(i=0;a[e][i];i++);s(a,e,i,t.cells[n].rowSpan,t.cells[n].colSpan),1===t.cells[n].colSpan&&(r[i]=r[i]||h(t.cells[n]))}return r},setColumnsWidth:function(e){var t,n=this._tableElement[0],i="",o=this.getColumnsCount(),a=[];for(t=0;t<o;t++)a.push(e[t]||0);for(t=o;t<e.length&&e;t++)a[o-1]+=e[t];for(t=0;t<o;t++)i+='<col style="width: '+a[t]+'px">';this._colgroupElement.html(i),this._tableWidth=a.reduce(function(e,t){return e+t},0),n.style.width=this._tableWidth+"px",n.style.tableLayout="fixed"},resetColumnsWidth:function(){this._colgroupElement.find("col").width("auto"),this._tableElement.css({width:"",tableLayout:""})},groupWidth:function(e){return void 0===e?this._groupElement.width():e>=0?(this._groupWidth=e,this._groupElement[0].style.width=e+"px"):this._groupElement[0].style.width=e},groupHeight:function(e){return void 0===e?this._groupElement.height():(this._groupHeight=null,void(e>=0?(this._groupHeight=e,this._groupElement[0].style.height=e+"px"):this._groupElement[0].style.height=e))},groupElement:function(){return this._groupElement},tableElement:function(){return this._tableElement},element:function(){return this._rootElement},headElement:function(){return this._tableElement.find("thead")},_setTableCss:function(e){this.option("rtlEnabled")&&(e.right=e.left,delete e.left),this.tableElement().css(e)},setVirtualContentParams:function(e){this._virtualContent.css({width:e.width,height:e.height}),this.groupElement().addClass("dx-virtual-mode")},disableVirtualMode:function(){this.groupElement().removeClass("dx-virtual-mode")},_renderVirtualContent:function(){var e=this;e._virtualContent||"virtual"!==e.option("scrolling.mode")||(e._virtualContent=(0,a.default)("<div>").addClass("dx-virtual-content").insertBefore(e._tableElement))},reset:function(){var e=this,t=e._tableElement[0];if(e._fakeTable&&e._fakeTable.detach(),e._fakeTable=null,e.disableVirtualMode(),e.groupWidth("100%"),e.groupHeight("auto"),e.resetColumnsWidth(),t){for(var n=0;n<t.rows.length;n++)t.rows[n].style.height="";t.style.height="",t.style.width="100%"}},_updateFakeTableVisibility:function(){var e=this,t=e.tableElement()[0],n=e.option("rtlEnabled")?"right":"left",i=e._fakeTable[0];t.style.top===i.style.top&&i.style[n]===t.style[n]?e._fakeTable.addClass("dx-hidden"):e._fakeTable.removeClass("dx-hidden")},_moveFakeTableHorizontally:function(e){var t=this,n=t.option("rtlEnabled"),i=n?"right":"left",a=parseFloat(t.tableElement()[0].style[i]),r=o(e,a,t._tableWidth,t._groupWidth);parseFloat(t._fakeTable[0].style[i])!==r&&(t._fakeTable[0].style[i]=r+"px")},_moveFakeTableTop:function(e){var t=this,n=parseFloat(t.tableElement()[0].style.top),i=o(e,n,t._tableHeight,t._groupHeight);parseFloat(t._fakeTable[0].style.top)!==i&&(t._fakeTable[0].style.top=i+"px")},_moveFakeTable:function(){this._updateFakeTableVisibility()},_createFakeTable:function(){var e=this;e._fakeTable||(e._fakeTable=e.tableElement().clone().addClass("dx-pivot-grid-fake-table").appendTo(e._virtualContent))},render:function(e,t){var n=this;if(n._tableElement){try{n._tableElement[0].innerHTML=""}catch(e){n._tableElement.empty()}n._tableElement.attr("style","")}else n._groupElement=n._createGroupElement(),n._tableElement=n._createTableElement(),n._tableElement.appendTo(n._groupElement),n._groupElement.appendTo(e),n._rootElement=e;n._colgroupElement=(0,a.default)("<colgroup>").appendTo(n._tableElement),n._renderTableContent(n._tableElement,t),n._renderVirtualContent()},_getScrollable:function(){return this.groupElement().data("dxScrollable")},on:function(e,t){var n=this,i=n._getScrollable();return i&&i.on(e,function(e){n.option("rtlEnabled")&&(0,c.isDefined)(e.scrollOffset.left)&&(e.scrollOffset.left=i.$content().width()-i._container().width()-e.scrollOffset.left),t(e)}),this},off:function(e){var t=this._getScrollable();return t&&t.off(e),this},scrollTo:function(e){var t=this._getScrollable(),n=e;t&&(this.option("rtlEnabled")&&("column"===this._getAreaName()?n=t.$content().width()-t._container().width()-e:"data"===this._getAreaName()&&(n={x:t.$content().width()-t._container().width()-e.x,y:e.y})),t.scrollTo(n),this._virtualContent&&(this._createFakeTable(),this._moveFakeTable(e)))},updateScrollable:function(){var e=this._getScrollable();if(e)return e.update()},getColumnsCount:function(){var e,t=0,n=this._getRowElement(0);if(n){e=n.cells;for(var i=0,o=e.length;i<o;++i)t+=e[i].colSpan}return t},getData:function(){var e=this._tableElement;return e?e.data("data"):[]}});t.AreaItem=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=t;return(0,d.isDefined)(t.groupIndex)&&(n=e.getAreaFields(t.area,!0)[t.areaIndex]),n}function a(e){return e=e||{},JSON.stringify([e.fields,e.columnExpandedPaths,e.rowExpandedPaths])}t.default=void 0;var r=i(n(2)),s=i(n(5)),l=i(n(81)),u=n(21),c=n(4),d=n(1),h=n(12),f=n(0),p=n(3),g=i(n(10)),_=i(n(9)),m=i(n(16)),v=n(490),y=i(n(462)),b=i(n(467)),x=n(89),w=i(n(780)),C=n(6),k=1,S="<div>",D=v.HeaderFilterView.inherit({_getSearchExpr:function(e){return e.useDefaultSearchExpr=!0,this.callBase(e)}}),I=function(e,t){var n=[],i=!!t.groupName,o="exclude"===t.filterType;t.filterValues&&(0,p.each)(t.filterValues,function(e,t){n.push(Array.isArray(t)?t.join("/"):t&&t.valueOf())}),(0,x.foreachTree)(e,function(e){var t=e[0],a=(0,x.createPath)(e),r=i?(0,p.map)(e,function(e){return e.text}).reverse().join("/"):t.text;t.value=i?a.slice(0):t.key||t.value;var s=i?a.join("/"):t.value&&t.value.valueOf();t.children&&(t.items=t.children,t.children=null),(0,v.updateHeaderFilterItemSelectionState)(t,t.key&&(0,h.inArray)(r,n)>-1||(0,h.inArray)(s,n)>-1,o)})},T=m.default.inherit(y.default).inherit(b.default).inherit(v.headerFilterMixin).inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{allowFieldDragging:!0,applyChangesMode:"instantly",state:null,headerFilter:{width:252,height:325,searchTimeout:500,texts:{emptyValue:g.default.format("dxDataGrid-headerFilterEmptyValue"),ok:g.default.format("dxDataGrid-headerFilterOK"),cancel:g.default.format("dxDataGrid-headerFilterCancel")}}})},_init:function(){this.callBase(),this._headerFilterView=new D(this),this._refreshDataSource(),this.subscribeToEvents()},_refreshDataSource:function(){var e=this.option("dataSource");e&&e.fields&&e.load&&(this._dataSource=e)},_optionChanged:function(e){switch(e.name){case"dataSource":this._refreshDataSource();break;case"applyChangesMode":break;case"state":if(this._skipStateChange||!this._dataSource)break;"instantly"===this.option("applyChangesMode")&&a(this._dataSource.state())!==a(e.value)?this._dataSource.state(e.value):(this._clean(!0),this._renderComponent());break;case"headerFilter":case"allowFieldDragging":this._invalidate();break;default:this.callBase(e)}},renderField:function(e,t){var n=this,i=(0,r.default)(S).addClass("dx-area-field-content").text(e.caption||e.dataField),a=(0,r.default)(S).addClass("dx-area-field").addClass("dx-area-box").data("field",e).append(i),s=o(n._dataSource,e);return"data"!==e.area&&(e.allowSorting&&n._applyColumnState({name:"sort",rootElement:a,column:{alignment:n.option("rtlEnabled")?"right":"left",sortOrder:"desc"===e.sortOrder?"desc":"asc"},showColumnLines:t}),n._applyColumnState({name:"headerFilter",rootElement:a,column:{alignment:n.option("rtlEnabled")?"right":"left",filterValues:s.filterValues,allowFiltering:s.allowFiltering&&!e.groupIndex},showColumnLines:t})),e.groupName&&a.attr("item-group",e.groupName),a},_clean:function(){},_render:function(){this.callBase(),this._headerFilterView.render(this.$element())},renderSortable:function(){var e=this;e._createComponent(e.$element(),w.default,(0,f.extend)({allowDragging:e.option("allowFieldDragging"),itemSelector:".dx-area-field",itemContainerSelector:".dx-area-field-container",groupSelector:".dx-area-fields",groupFilter:function(){var t=e._dataSource,n=(0,r.default)(this).closest(".dx-sortable-old"),i=n.data("dxPivotGrid"),o=n.data("dxPivotGridFieldChooser");return i?i.getDataSource()===t:!!o&&o.option("dataSource")===t},itemRender:function(e,t){var n;if(e.hasClass("dx-area-box")?(n=e.clone(),"drag"===t&&(0,p.each)(e,function(e,t){n.eq(e).css("width",parseInt((0,r.default)(t).outerWidth(),10)+k)})):n=(0,r.default)(S).addClass("dx-area-field").addClass("dx-area-box").text(e.text()),"drag"===t){var i=(0,r.default)(S);return(0,p.each)(n,function(e,t){var n=(0,r.default)("<div>").addClass("dx-pivotgrid-fields-container").addClass("dx-widget").append((0,r.default)(t));i.append(n)}),i.children()}return n},onDragging:function(e){var t=e.sourceElement.data("field"),n=e.targetGroup;e.cancel=!1,t.isMeasure===!0?"column"!==n&&"row"!==n&&"filter"!==n||(e.cancel=!0):t.isMeasure===!1&&"data"===n&&(e.cancel=!0)},useIndicator:!0,onChanged:function(t){var n=e._dataSource,i=t.sourceElement.data("field");t.removeSourceElement=!!t.sourceGroup,e._adjustSortableOnChangedArgs(t),i&&e._applyChanges([o(n,i)],{area:t.targetGroup,areaIndex:t.targetIndex})}},e._getSortableOptions()))},_processDemandState:function(e){var t=this,n="instantly"===t.option("applyChangesMode"),i=t._dataSource;if(n)e(i,n);else{var o=i.state(),a=t.option("state");a&&i.state(a,!0),e(i,n),i.state(o,!0)}},_applyChanges:function(e,t){var n=this;n._processDemandState(function(i,o){e.forEach(function(e){var n=e.index;i.field(n,t)}),o?i.load():n._changedHandler()})},_adjustSortableOnChangedArgs:function(e){e.removeSourceElement=!1,e.removeTargetElement=!0,e.removeSourceClass=!1},_getSortableOptions:function(){return{direction:"auto"}},subscribeToEvents:function(e){var t=this,n=function(e){var n=(0,r.default)(e.currentTarget).data("field"),i=(0,f.extend)(!0,{},o(t._dataSource,n)),a=(0,r.default)(e.target).hasClass("dx-header-filter"),s=t._dataSource,u=i.groupName?"tree":"list",c=s.paginate()&&"list"===u;a?t._headerFilterView.showHeaderFilterMenu((0,r.default)(e.currentTarget),(0,f.extend)(i,{type:u,encodeHtml:t.option("encodeHtml"),dataSource:{useDefaultSearch:!c,load:function(e){var n=e.userData;if(n.store)return n.store.load(e);var o=new C.Deferred;return s.getFieldValues(i.index,t.option("headerFilter.showRelevantValues"),c?e:void 0).done(function(i){var a=t.option("headerFilter.texts.emptyValue");i.forEach(function(e){e.text||(e.text=a)}),c?o.resolve(i):(n.store=new l.default(i),n.store.load(e).done(o.resolve).fail(o.reject))}).fail(o.reject),o},postProcess:function(e){return I(e,i),e}},apply:function(){t._applyChanges([i],{filterValues:this.filterValues,filterType:this.filterType})}})):n.allowSorting&&"data"!==n.area&&t._applyChanges([n],{sortOrder:"desc"===n.sortOrder?"asc":"desc"})};return e?void s.default.on(e,u.name,".dx-area-field.dx-area-box",n):void s.default.on(t.$element(),u.name,".dx-area-field.dx-area-box",n)},_initTemplates:c.noop,addWidgetPrefix:function(e){return"dx-pivotgrid-"+e}});(0,_.default)("dxPivotGridFieldChooserBase",T);var O=T;t.default=O,e.exports=t.default},function(e,t,n){var i=function(){function e(e,t){this.minDate=null,this.maxDate=null,this._result=[],this.total=0,this.method=e,this.args=t,"between"===e?(this.maxDate=t.inc?t.before:new Date(t.before.getTime()-1),this.minDate=t.inc?t.after:new Date(t.after.getTime()+1)):"before"===e?this.maxDate=t.inc?t.dt:new Date(t.dt.getTime()-1):"after"===e&&(this.minDate=t.inc?t.dt:new Date(t.dt.getTime()+1))}return e.prototype.accept=function(e){++this.total;var t=this.minDate&&e<this.minDate,n=this.maxDate&&e>this.maxDate;if("between"===this.method){if(t)return!0;if(n)return!1}else if("before"===this.method){if(n)return!1}else if("after"===this.method)return!!t||(this.add(e),!1);return this.add(e)},e.prototype.add=function(e){return this._result.push(e),!0},e.prototype.getValue=function(){var e=this._result;switch(this.method){case"all":case"between":return e;case"before":case"after":default:return e.length?e[e.length-1]:null}},e.prototype.clone=function(){return new e(this.method,this.args)},e}();t.a=i},function(e,t,n){n.d(t,"b",function(){return s}),n.d(t,"a",function(){return l});var i=n(130),o=n(148),a=n(74),r=n(73),s=function(){function e(e,t,n,i){this.hour=e,this.minute=t,this.second=n,this.millisecond=i||0}return e.prototype.getHours=function(){return this.hour},e.prototype.getMinutes=function(){return this.minute},e.prototype.getSeconds=function(){return this.second},e.prototype.getMilliseconds=function(){return this.millisecond},e.prototype.getTime=function(){return 1e3*(60*this.hour*60+60*this.minute+this.second)+this.millisecond},e}(),l=function(e){function t(t,n,i,o,a,r,s){var l=e.call(this,o,a,r,s)||this;return l.year=t,l.month=n,l.day=i,l}return Object(i.b)(t,e),t.fromDate=function(e){return new this(e.getUTCFullYear(),e.getUTCMonth()+1,e.getUTCDate(),e.getUTCHours(),e.getUTCMinutes(),e.getUTCSeconds(),e.valueOf()%1e3)},t.prototype.getWeekday=function(){return r.a.getWeekday(new Date(this.getTime()))},t.prototype.getTime=function(){return new Date(Date.UTC(this.year,this.month-1,this.day,this.hour,this.minute,this.second,this.millisecond)).getTime()},t.prototype.getDay=function(){return this.day},t.prototype.getMonth=function(){return this.month},t.prototype.getYear=function(){return this.year},t.prototype.addYears=function(e){this.year+=e},t.prototype.addMonths=function(e){if(this.month+=e,this.month>12){var t=Math.floor(this.month/12),n=Object(a.j)(this.month,12);this.month=n,this.year+=t,0===this.month&&(this.month=12,--this.year)}},t.prototype.addWeekly=function(e,t){t>this.getWeekday()?this.day+=-(this.getWeekday()+1+(6-t))+7*e:this.day+=-(this.getWeekday()-t)+7*e,this.fixDay()},t.prototype.addDaily=function(e){this.day+=e,this.fixDay()},t.prototype.addHours=function(e,t,n){for(t&&(this.hour+=Math.floor((23-this.hour)/e)*e);;){this.hour+=e;var i=Object(a.a)(this.hour,24),o=i.div,r=i.mod;if(o&&(this.hour=r,this.addDaily(o)),Object(a.b)(n)||Object(a.c)(n,this.hour))break}},t.prototype.addMinutes=function(e,t,n,i){for(t&&(this.minute+=Math.floor((1439-(60*this.hour+this.minute))/e)*e);;){this.minute+=e;var o=Object(a.a)(this.minute,60),r=o.div,s=o.mod;if(r&&(this.minute=s,this.addHours(r,!1,n)),(Object(a.b)(n)||Object(a.c)(n,this.hour))&&(Object(a.b)(i)||Object(a.c)(i,this.minute)))break}},t.prototype.addSeconds=function(e,t,n,i,o){for(t&&(this.second+=Math.floor((86399-(3600*this.hour+60*this.minute+this.second))/e)*e);;){this.second+=e;var r=Object(a.a)(this.second,60),s=r.div,l=r.mod;if(s&&(this.second=l,this.addMinutes(s,!1,n,i)),(Object(a.b)(n)||Object(a.c)(n,this.hour))&&(Object(a.b)(i)||Object(a.c)(i,this.minute))&&(Object(a.b)(o)||Object(a.c)(o,this.second)))break}},t.prototype.fixDay=function(){if(!(this.day<=28)){var e=r.a.monthRange(this.year,this.month-1)[1];if(!(this.day<=e))for(;this.day>e;){if(this.day-=e,++this.month,13===this.month&&(this.month=1,++this.year,this.year>r.a.MAXYEAR))return;e=r.a.monthRange(this.year,this.month-1)[1]}}},t.prototype.add=function(e,t){var n=e.freq,i=e.interval,a=e.wkst,r=e.byhour,s=e.byminute,l=e.bysecond;switch(n){case o.a.YEARLY:return this.addYears(i);case o.a.MONTHLY:return this.addMonths(i);case o.a.WEEKLY:return this.addWeekly(i,a);case o.a.DAILY:return this.addDaily(i);case o.a.HOURLY:return this.addHours(i,t,r);case o.a.MINUTELY:return this.addMinutes(i,t,r,s);case o.a.SECONDLY:return this.addSeconds(i,t,r,s,l)}},t}(s)},function(e,t,n){n.d(t,"a",function(){return a});var i=n(73),o=n(791),a=(n.n(o),function(){function e(e,t){this.date=e,this.tzid=t}return Object.defineProperty(e.prototype,"isUTC",{get:function(){return!this.tzid||"UTC"===this.tzid.toUpperCase()},enumerable:!0,configurable:!0}),e.prototype.toString=function(){var e=i.b.timeToUntilString(this.date.getTime(),this.isUTC);return this.isUTC?":"+e:";TZID="+this.tzid+":"+e},e.prototype.getTime=function(){return this.date.getTime()},e.prototype.rezonedDate=function(){if(this.isUTC)return this.date;try{var e=o.DateTime.fromJSDate(this.date),t=e.setZone(this.tzid,{keepLocalTime:!0});return t.toJSDate()}catch(e){return e instanceof TypeError&&console.error("Using TZID without Luxon available is unsupported. Returned times are in UTC, not the requested time zone"),this.date}},e}())},function(e,t,n){var i={dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],tokens:{SKIP:/^[ \r\n\t]+|^\.$/,number:/^[1-9][0-9]*/,numberAsText:/^(one|two|three)/i,every:/^every/i,"day(s)":/^days?/i,"weekday(s)":/^weekdays?/i,"week(s)":/^weeks?/i,"hour(s)":/^hours?/i,"minute(s)":/^minutes?/i,"month(s)":/^months?/i,"year(s)":/^years?/i,on:/^(on|in)/i,at:/^(at)/i,the:/^the/i,first:/^first/i,second:/^second/i,third:/^third/i,nth:/^([1-9][0-9]*)(\.|th|nd|rd|st)/i,last:/^last/i,"for":/^for/i,"time(s)":/^times?/i,until:/^(un)?til/i,monday:/^mo(n(day)?)?/i,tuesday:/^tu(e(s(day)?)?)?/i,wednesday:/^we(d(n(esday)?)?)?/i,thursday:/^th(u(r(sday)?)?)?/i,friday:/^fr(i(day)?)?/i,saturday:/^sa(t(urday)?)?/i,sunday:/^su(n(day)?)?/i,january:/^jan(uary)?/i,february:/^feb(ruary)?/i,march:/^mar(ch)?/i,april:/^apr(il)?/i,may:/^may/i,june:/^june?/i,july:/^july?/i,august:/^aug(ust)?/i,september:/^sep(t(ember)?)?/i,october:/^oct(ober)?/i,november:/^nov(ember)?/i,december:/^dec(ember)?/i,comma:/^(,\s*|(and|or)\s*)+/i}};t.a=i},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(519)),l=i(n(811)),u=n(0),c=i(n(20)),d=n(1),h=n(36),f=i(n(86)),p=c.default.dateToMilliseconds,g=2,_=20,m=18,v=28,y=function(){function e(t){o(this,e),this.instance=t,this._initPositioningStrategy()}return r(e,[{key:"_isAdaptive",value:function(){return this.instance.fire("isAdaptive")}},{key:"_correctCollectorCoordinatesInAdaptive",value:function(e,t){e.top=e.top+this.getCollectorTopOffset(t),e.left=e.left+this.getCollectorLeftOffset()}},{key:"_initPositioningStrategy",value:function(){this._positioningStrategy=this._isAdaptive()?new l.default(this):new s.default(this)}},{key:"getPositioningStrategy",value:function(){return this._positioningStrategy}},{key:"getAppointmentMinSize",value:function(){return g}},{key:"keepAppointmentSettings",value:function(){return!1}},{key:"getDeltaTime",value:function(){}},{key:"getAppointmentGeometry",value:function(e){return e}},{key:"needCorrectAppointmentDates",value:function(){return!0}},{key:"getDirection",value:function(){return"horizontal"}},{key:"createTaskPositionMap",value:function(e){delete this._maxAppointmentCountPerCell;var t=e&&e.length;if(t){this._defaultWidth=this.instance.fire("getCellWidth"),this._defaultHeight=this.instance.fire("getCellHeight"),this._allDayHeight=this.instance._allDayCellHeight;for(var n=[],i=0;i<t;i++){var o=this._getItemPosition(e[i]);this._isRtl()&&(o=this._correctRtlCoordinates(o)),n.push(o)}var a=this._getSortedPositions(n),r=this._getResultPositions(a);return this._getExtendedPositionMap(n,r)}}},{key:"_getDeltaWidth",value:function(e,t){var n=this.instance.fire("getResizableStep")||this.getAppointmentMinSize(),i=t.width;return Math.round((e.width-i)/n)}},{key:"_correctRtlCoordinates",value:function(e){var t=e[0].width||this._getAppointmentMaxWidth();return e.forEach(function(e){e.appointmentReduced||(e.left-=t)}),e}},{key:"_getAppointmentMaxWidth",value:function(){return this.getDefaultCellWidth()}},{key:"_getItemPosition",value:function(e){for(var t=this.instance.createAppointmentAdapter(e),n=this._getAppointmentCoordinates(e),i=this.isAllDay(e),o=new Date(t.startDate),a=[],r=0;r<n.length;r++){var s=this.calculateAppointmentHeight(e,n[r]),l=this.calculateAppointmentWidth(e,n[r]),c=l,d=null,h=[],f=n[r].rowIndex,p=n[r].cellIndex;if(this._needVerifyItemSize()||i){var g=n[r].hMax;this.isAppointmentGreaterThan(g,{left:n[r].left,width:l})&&(d="head",f=n[r].rowIndex,p=n[r].cellIndex,c=this._reduceMultiWeekAppointment(l,{left:n[r].left,right:g}),h=this._getAppointmentParts({sourceAppointmentWidth:l,reducedWidth:c,height:s},n[r],o),this._isRtl()&&(n[r].left=g))}(0,u.extend)(n[r],{height:s,width:c,allDay:i,rowIndex:f,cellIndex:p,appointmentReduced:d}),a=this._getAppointmentPartsPosition(h,n[r],a)}return a}},{key:"_getAppointmentPartsPosition",value:function(e,t,n){return e.length?(e.unshift(t),n=n.concat(e)):n.push(t),n}},{key:"_getAppointmentCoordinates",value:function(e){return this.instance.fire("createAppointmentSettings",e)}},{key:"_isRtl",value:function(){return this.instance.option("rtlEnabled")}},{key:"_getAppointmentParts",value:function(){return[]}},{key:"_getCompactAppointmentParts",value:function(e){var t=this.getDefaultCellWidth()||this.getAppointmentMinSize();return Math.round(e/t)}},{key:"_reduceMultiWeekAppointment",value:function(e,t){return e=this._isRtl()?Math.floor(t.left-t.right):t.right-Math.floor(t.left)}},{key:"calculateAppointmentHeight",value:function(){return 0}},{key:"calculateAppointmentWidth",value:function(){return 0}},{key:"isAppointmentGreaterThan",value:function(e,t){var n=t.left+t.width-e;return this._isRtl()&&(n=e+t.width-t.left),n>this.getDefaultCellWidth()/2}},{key:"isAllDay",value:function(){return!1}},{key:"cropAppointmentWidth",value:function(e,t){return this.instance.fire("isGroupedByDate")&&(e=t),e}},{key:"_getSortedPositions",value:function(e){for(var t=this,n=[],i=function(e){return Math.round(100*e)/100},o=function(e,t,n,o,a,r,s,l){return{i:e,j:t,top:i(n),left:i(o),bottom:i(a),right:i(r),cellPosition:s,allDay:l}},a=0,r=e.length;a<r;a++)for(var s=0,l=e[a].length;s<l;s++){var u=e[a][s],c=u.top,d=u.left,h=u.height,f=u.width,p=u.cellPosition,g=u.allDay;n.push(o(a,s,c,d,c+h,d+f,p,g))}return n.sort(function(e,n){return t._sortCondition(e,n)})}},{key:"_sortCondition",value:function(){}},{key:"_getConditions",value:function(e,t){var n=this._isSomeEdge(e,t);return{columnCondition:n||this._normalizeCondition(e.left,t.left),rowCondition:n||this._normalizeCondition(e.top,t.top),cellPositionCondition:n||this._normalizeCondition(e.cellPosition,t.cellPosition)}}},{key:"_rowCondition",value:function(e,t){var n=this._getConditions(e,t);return n.columnCondition||n.rowCondition}},{key:"_columnCondition",value:function(e,t){var n=this._getConditions(e,t);return n.rowCondition||n.columnCondition}},{key:"_isSomeEdge",value:function(e,t){return e.i===t.i&&e.j===t.j}},{key:"_normalizeCondition",value:function(e,t){var n=e-t;return Math.abs(n)>1?n:0}},{key:"_isItemsCross",value:function(e,t){var n=!!e.allDay==!!t.allDay,i=e.allDay&&t.allDay;if(n){var o=this._getOrientation(i);return this._checkItemsCrossing(e,t,o)}return!1}},{key:"_checkItemsCrossing",value:function(e,t,n){var i=Math.floor(e[n[0]]),o=Math.floor(e[n[1]]),a=Math.ceil(t[n[0]]),r=Math.ceil(t[n[1]]),s=Math.abs(e[n[2]]-t[n[2]])<=1;return s&&(i<=a&&o>a||i<r&&o>=r||i===a&&o===r)}},{key:"_getOrientation",value:function(e){return e?["left","right","top"]:["top","bottom","left"]}},{key:"_getResultPositions",value:function(e){var t,n,i,o,a=this,r=[],s=0,l=0,u={},c=function e(t,n){var i=t.some(function(e){return e===n});return i?e(t,++n):n},d=function(e,t){var n=t||0;return{index:n,i:e.i,j:e.j,left:e.left,right:e.right,top:e.top,bottom:e.bottom,allDay:e.allDay,sortedIndex:a._skipSortedIndex(n)?null:s++}},h=function(e){u.items=[d(e)],u.left=e.left,u.right=e.right,u.top=e.top,u.bottom=e.bottom,u.allDay=e.allDay},f=function(e){e.forEach(function(e){r.push({index:e.index,count:l+1,i:e.i,j:e.j,sortedIndex:e.sortedIndex})})};for(t=0;t<e.length;t++)n=e[t],i=[],u.items?this._isItemsCross(u,n)?(u.items.forEach(function(e,t){a._isItemsCross(e,n)&&i.push(e.index)}),o=i.length?c(i,0):0,u.items.push(d(n,o)),l=Math.max(o,l),u.left=Math.min(u.left,n.left),u.right=Math.max(u.right,n.right),u.top=Math.min(u.top,n.top),u.bottom=Math.max(u.bottom,n.bottom),u.allDay=n.allDay):(f(u.items),u={},h(n),l=0):h(n);return u.items&&f(u.items),r.sort(function(e,t){var n=e.j-t.j,i=e.i-t.i;return i?i:n})}},{key:"_skipSortedIndex",value:function(e){return e>this._getMaxAppointmentCountPerCell()-1}},{key:"_findIndexByKey",value:function(e,t,n,i,o){for(var a=0,r=0,s=e.length;r<s;r++)if(e[r][t]===i&&e[r][n]===o){a=r;break}return a}},{key:"_getExtendedPositionMap",value:function(e,t){for(var n=0,i=[],o=0,a=e.length;o<a;o++){for(var r=[],s=0,l=e[o].length;s<l;s++)e[o][s].index=t[n].index,e[o][s].sortedIndex=t[n].sortedIndex,e[o][s].count=t[n++].count,r.push(e[o][s]),this._checkLongCompactAppointment(e[o][s],r);i.push(r)}return i}},{key:"_checkLongCompactAppointment",value:function(e,t){return this._splitLongCompactAppointment(e,t),t}},{key:"_splitLongCompactAppointment",value:function(e,t){var n=this._getMaxAppointmentCountPerCellByType(e.allDay),i=0;if(void 0!==n&&e.index>n-1){e.isCompact=!0,i=this._getCompactAppointmentParts(e.width);for(var o=1;o<i;o++){var a=(0,u.extend)(!0,{},e);a.left=this._getCompactLeftCoordinate(e.left,o),a.cellIndex=a.cellIndex+o,a.sortedIndex=null,t.push(a)}}return t}},{key:"normalizeEndDateByViewEnd",value:function(e,t){var n=new Date(t.getTime());if(!this.isAllDay(e)){var i=c.default.roundToHour(this.instance.fire("getEndViewDate"));n>i&&(n=i)}var o=this.instance._getCurrentViewOption("endDayHour"),a=this.instance.fire("getField","allDay",e),r=new Date(new Date(t.getTime()).setHours(o,0,0,0));return(n.getTime()>r.getTime()||a&&n.getHours()<o)&&(n=r),n}},{key:"_adjustDurationByDaylightDiff",value:function(e,t,n){var i=f.default.getDaylightOffset(t,n);return this._needAdjustDuration(i)?this._calculateDurationByDaylightDiff(e,i):e}},{key:"_needAdjustDuration",value:function(e){return 0!==e}},{key:"_calculateDurationByDaylightDiff",value:function(e,t){return e+t*p("minute")}},{key:"_getAppointmentDurationInMs",value:function(e,t,n){return this.instance.fire("getAppointmentDurationInMs",{startDate:e,endDate:t,allDay:n})}},{key:"_markAppointmentAsVirtual",value:function(e,t){var n=this._getMaxAppointmentCountPerCellByType(t);e.count-n>0&&(e.virtual={top:e.top,left:e.left,index:"tail"===e.appointmentReduced?e.groupIndex+"-"+e.rowIndex+"-"+e.cellIndex:e.groupIndex+"-"+e.rowIndex+"-"+e.cellIndex+"-tail",isAllDay:t})}},{key:"_getMaxAppointmentCountPerCellByType",value:function(e){var t=this._getMaxAppointmentCountPerCell();return(0,d.isObject)(t)?e?this._getMaxAppointmentCountPerCell().allDay:this._getMaxAppointmentCountPerCell().simple:t}},{key:"getDropDownAppointmentWidth",value:function(e,t){return this.getPositioningStrategy().getDropDownAppointmentWidth(e,t)}},{key:"getDropDownAppointmentHeight",value:function(){return this.getPositioningStrategy().getDropDownAppointmentHeight()}},{key:"getDropDownButtonAdaptiveSize",value:function(){return v}},{key:"getDefaultCellWidth",value:function(){return this._defaultWidth}},{key:"getDefaultCellHeight",value:function(){return this._defaultHeight}},{key:"getDefaultAllDayCellHeight",value:function(){return this._allDayHeight}},{key:"getCollectorTopOffset",value:function(e){return this.getPositioningStrategy().getCollectorTopOffset(e)}},{key:"getCollectorLeftOffset",value:function(){return this.getPositioningStrategy().getCollectorLeftOffset()}},{key:"getAppointmentDataCalculator",value:function(){}},{key:"_customizeCoordinates",value:function(e,t,n,i,o){var a=e.index,r=t/n,s=e.top+a*r,l=s+i,u=e.width,c=e.left;return e.isCompact&&(this._isAdaptive()&&this._correctCollectorCoordinatesInAdaptive(e,o),this._markAppointmentAsVirtual(e,o)),{height:r,width:u,top:l,left:c,empty:this._isAppointmentEmpty(t,u)}}},{key:"_isAppointmentEmpty",value:function(e,t){return e<this._getAppointmentMinHeight()||t<this._getAppointmentMinWidth()}},{key:"_calculateGeometryConfig",value:function(e){var t=this.instance.fire("getMaxAppointmentsPerCell"),n=this._getOffsets(),i=this._getAppointmentDefaultOffset(),o=this._getAppointmentCount(t,e),a=this._getDefaultRatio(e,o),r=this._getMaxHeight();(0,d.isNumeric)(o)||(o=e.count,a=(r-n.unlimited)/r);var s=(1-a)*r;return("auto"===t||(0,d.isNumeric)(t))&&(a=1,r-=i,s=i),{height:a*r,appointmentCountPerCell:o,offset:s}}},{key:"_getAppointmentCount",value:function(){}},{key:"_getDefaultRatio",value:function(){}},{key:"_getOffsets",value:function(){}},{key:"_getMaxHeight",value:function(){}},{key:"_needVerifyItemSize",value:function(){return!1}},{key:"needSeparateAppointment",value:function(e){return this.instance.fire("isGroupedByDate")&&e}},{key:"_getMaxAppointmentCountPerCell",value:function(){if(!this._maxAppointmentCountPerCell){var e,t=this.instance.fire("getMaxAppointmentsPerCell");(0,d.isNumeric)(t)&&(e=t),"auto"===t&&(e=this._getDynamicAppointmentCountPerCell()),"unlimited"===t&&(e=void 0),this._maxAppointmentCountPerCell=e}return this._maxAppointmentCountPerCell}},{key:"_getDynamicAppointmentCountPerCell",value:function(){return this.getPositioningStrategy().getDynamicAppointmentCountPerCell()}},{key:"hasAllDayAppointments",value:function(){return!1}},{key:"_isCompactTheme",value:function(){return"compact"===((0,h.current)()||"").split(".").pop()}},{key:"_getAppointmentDefaultOffset",value:function(){return this.getPositioningStrategy().getAppointmentDefaultOffset()}},{key:"_getAppointmentDefaultHeight",value:function(){return this._getAppointmentHeightByTheme()}},{key:"_getAppointmentMinHeight",value:function(){return this._getAppointmentDefaultHeight()}},{key:"_getAppointmentHeightByTheme",value:function(){return this._isCompactTheme()?m:_}},{key:"_getAppointmentDefaultWidth",value:function(){return this.getPositioningStrategy()._getAppointmentDefaultWidth()}},{key:"_getAppointmentMinWidth",value:function(){return this._getAppointmentDefaultWidth()}},{key:"_needVerticalGroupBounds",value:function(){return!1}},{key:"_needHorizontalGroupBounds",value:function(){return!1}},{key:"isVirtualScrolling",get:function(){return this.instance.fire("isVirtualScrolling")}}]),e}(),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=i(n(11)),u=n(39),c=n(1),d=n(18),h="tr",f=function(){function e(){o(this,e),this.VERTICAL="vertical",this.HORIZONTAL="horizontal"}return r(e,[{key:"insertAllDayRow",value:function(e,t,n){if(e[n]){var i=e[n].find(h);i.length||(i=(0,s.default)(l.default.createElement(h)),i.append(e[n].get(0))),t.appendChild(i.get?i.get(0):i)}}},{key:"makeTable",value:function(e){var t,n=l.default.createElement("tbody"),i=[],o=e.groupCount?e.rowCount/e.groupCount:e.rowCount,a=0,r=e.allDayElements,f=e.groupIndex,p=e.rowCount;(0,s.default)(e.container).append(n),r&&(this.insertAllDayRow(r,n,0),a++);for(var g=0;g<p;g++){t=l.default.createElement(h),n.appendChild(t);var _=(g+1)%o===0;e.rowClass&&(t.className=e.rowClass);for(var m=0;m<e.cellCount;m++){var v=l.default.createElement("td");t.appendChild(v),e.cellClass&&((0,c.isFunction)(e.cellClass)?v.className=e.cellClass(g,m):v.className=e.cellClass);var y=void 0,b=void 0,x=void 0;if(e.getCellData&&(y=e.getCellData(v,g,m,f),b=y.key,x=y.value,b&&(0,u.data)(v,b,x)),e.cellTemplate&&e.cellTemplate.render){var w={model:{text:e.getCellText?e.getCellText(g,m):"",
date:e.getCellDate?e.getCellDate(g):void 0},container:(0,d.getPublicElement)((0,s.default)(v)),index:g*e.cellCount+m};x&&(x.startDate&&(w.model.startDate=x.startDate),x.endDate&&(w.model.endDate=x.endDate),x.groups&&(w.model.groups=x.groups),x.allDay&&(w.model.allDay=x.allDay)),i.push(e.cellTemplate.render.bind(e.cellTemplate,w))}else e.getCellText&&(v.innerHTML="<div>"+e.getCellText(g,m)+"</div>")}r&&_&&(this.insertAllDayRow(r,n,a),a++)}return i}},{key:"makeGroupedTable",value:function(e,t,n,i,o,a,r){var s=[];return s=e===this.VERTICAL?this._makeVerticalGroupedRows(t,n,o,a):this._makeHorizontalGroupedRows(t,n,i,o,r)}},{key:"makeGroupedTableFromJSON",value:function(e,t,n){function i(){u=l.default.createElement("table"),_&&(u.className=_)}function o(e){return e[p]?e[p].length:0}function a(e,t,n,i){var o={element:l.default.createElement(f),childCount:t};v&&(o.element.className=v);var a=l.default.createTextNode(e);return"function"==typeof y?y(o.element,a,n,i):o.element.appendChild(a),o}function r(e){for(var t=0;t<e.length;t++){var n=o(e[t]),i=a(e[t][g],n,t,e[t]);c[d]||(c[d]=[]),c[d].push(i),n?r(e[t][p]):d++}}function s(){c.forEach(function(e){var t=l.default.createElement(h);m&&(t.className=m);for(var n=[],i=e.length-1;i>=0;i--){var o=e[i+1],a=e[i].childCount;o&&o.childCount&&(a*=o.childCount),n.push(a)}n.reverse(),e.forEach(function(e,i){n[i]&&e.element.setAttribute("rowSpan",n[i]),t.appendChild(e.element)}),u.appendChild(t)})}var u,c=[],d=0;n=n||{};var f=n.cellTag||"td",p=n.childrenField||"children",g=n.titleField||"title",_=n.groupTableClass,m=n.groupRowClass,v=n.groupCellClass,y=n.groupCellCustomContent;return i(),r(t),s(),u}},{key:"_makeFlexGroupedRowCells",value:function(e,t,n,i){for(var o=arguments.length>4&&void 0!==arguments[4]?arguments[4]:1,a=[],r=e.items,l=r.length,u=0;u<t*o;u++)for(var h=0;h<l;h++){var f=(0,s.default)("<div>"),p={};if(i&&i.render){var g={model:r[h],container:(0,d.getPublicElement)(f),index:u*l+h};e.data&&(g.model.data=e.data[h]),p.template=i.render.bind(i,g)}else f.text(r[h].text).attr("title",r[h].text).addClass("dx-scheduler-group-header-content"),f=(0,s.default)("<div>").append(f);var _=(0,c.isFunction)(n.groupHeaderClass)?n.groupHeaderClass(h):n.groupHeaderClass;p.element=f.addClass(_),a.push(p)}return a}},{key:"_makeVerticalGroupedRows",value:function(e,t,n){for(var i=[],o=1,a=[],r=function(e){e.template&&i.push(e.template)},l=0;l<e.length;l++){l>0&&(o=e[l-1].items.length*o);var u=this._makeFlexGroupedRowCells(e[l],o,t,n);u.forEach(r),a.push(u)}for(var c=[],d=a.length,h=0;h<d;h++)c.push((0,s.default)("<div>").addClass(t.groupHeaderRowClass));for(var f=d-1;f>=0;f--)for(var p=a[f].length,g=0;g<p;g++)c[f].append(a[f][g].element);return{elements:(0,s.default)("<div>").addClass("dx-scheduler-group-flex-container").append(c),cellTemplates:i}}},{key:"_makeHorizontalGroupedRows",value:function(e,t,n,i,o){for(var a=1,r=e.length,l=[],u=[],c=o?n:1,d=function(e){return e.template&&u.push(e.template),e.element},h=0;h<r;h++){h>0&&(a=e[h-1].items.length*a);var f=this._makeGroupedRowCells(e[h],a,t,i,c);l.push((0,s.default)("<tr>").addClass(t.groupRowClass).append(f.map(d)))}for(var p=l[r-1].find("th").length,g=0;g<r;g++){var _=l[g].find("th"),m=p/_.length;o||(m*=n),(m>1&&1===c||o&&r>1)&&_.attr("colSpan",m)}return{elements:l,cellTemplates:u}}},{key:"_makeGroupedRowCells",value:function(e,t,n,i,o){o=o||1,t*=o;for(var a=[],r=e.items,l=r.length,u=0;u<t;u++)for(var h=0;h<l;h++){var f=(0,s.default)("<div>"),p={};if(i&&i.render){var g={model:r[h],container:(0,d.getPublicElement)(f),index:u*l+h};e.data&&(g.model.data=e.data[h]),p.template=i.render.bind(i,g)}else f.text(r[h].text),f=(0,s.default)("<div>").append(f);f.addClass(n.groupHeaderContentClass);var _=void 0;_=(0,c.isFunction)(n.groupHeaderClass)?n.groupHeaderClass(h):n.groupHeaderClass,p.element=(0,s.default)("<th>").addClass(_).append(f),a.push(p)}return a}}]),e}(),p={tableCreator:new f};t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,_.useCallback)(function(){var t=e.height,n=o(),i=n.style;return(0,f.addHeightToStyle)(t,i)},[e.height]),n=(0,_.useCallback)(function(){var t=e.isVirtual,n=e.topVirtualRowHeight;return!!t&&!!n},[e.isVirtual,e.topVirtualRowHeight]),i=(0,_.useCallback)(function(){var t=e.bottomVirtualRowHeight,n=e.isVirtual;return!!n&&!!t},[e.bottomVirtualRowHeight,e.isVirtual]),o=(0,_.useCallback)(function(){var t=(e.bottomVirtualRowHeight,e.children,e.className,e.height,e.isVirtual,e.topVirtualRowHeight,e.virtualCellsCount,u(e,["bottomVirtualRowHeight","children","className","height","isVirtual","topVirtualRowHeight","virtualCellsCount"]));return t},[e]);return m({props:s({},e),style:t(),hasTopVirtualRow:n(),hasBottomVirtualRow:i(),restAttributes:o()})}t.Table=h,t.TableProps=t.viewFunction=void 0;var f=n(199),p=n(831),g=a(n(51)),_=n(53),m=function(e){var t=e.hasBottomVirtualRow,n=e.hasTopVirtualRow,i=e.props,o=i.bottomVirtualRowHeight,a=i.children,r=i.className,s=i.topVirtualRowHeight,l=i.virtualCellsCount,u=e.restAttributes,c=e.style;return g.h("table",d({},u,{className:r,style:c}),g.h("tbody",null,n&&g.h(p.VirtualRow,{height:s,cellsCount:l}),a,t&&g.h(p.VirtualRow,{height:o,cellsCount:l})))};t.viewFunction=m;var v={className:"",topVirtualRowHeight:0,bottomVirtualRowHeight:0,virtualCellsCount:0,isVirtual:!1};t.TableProps=v,h.defaultProps=s({},v)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,_.useCallback)(function(){var t=e.allDay,n=e.className;return(0,p.combineClasses)(c({"dx-scheduler-cell-sizes-horizontal":!0,"dx-scheduler-cell-sizes-vertical":!t,"dx-scheduler-date-table-cell":!t},n,!0))},[e.allDay,e.className]),n=(0,_.useCallback)(function(){var t=e.allDay,n=e.endDate,i=e.groupIndex,o=e.groups,a=e.index,r=e.startDate,s=e.text;return{data:{startDate:r,endDate:n,groups:o,groupIndex:o?i:void 0,text:s||"",allDay:t||void 0},index:a}},[e.allDay,e.endDate,e.groupIndex,e.groups,e.index,e.startDate,e.text]),i=(0,_.useCallback)(function(){var t=(e.allDay,e.children,e.className,e.contentTemplate,e.contentTemplateProps,e.dataCellTemplate,e.endDate,e.groupIndex,e.groups,e.index,e.isFirstGroupCell,e.isLastGroupCell,e.startDate,e.text,r(e,["allDay","children","className","contentTemplate","contentTemplateProps","dataCellTemplate","endDate","groupIndex","groups","index","isFirstGroupCell","isLastGroupCell","startDate","text"]));return t},[e]);return m({props:u(u({},e),{},{dataCellTemplate:y(e.dataCellTemplate),contentTemplate:y(e.contentTemplate)}),classes:t(),dataCellTemplateProps:n(),restAttributes:i()})}t.DateTableCellBase=h,t.DateTableCellBaseProps=t.viewFunction=void 0;var f=n(315),p=n(226),g=a(n(51)),_=n(53),m=function(e){return g.h(f.CellBase,d({},e.restAttributes,{isFirstGroupCell:e.props.isFirstGroupCell,isLastGroupCell:e.props.isLastGroupCell,contentTemplate:e.props.dataCellTemplate,contentTemplateProps:e.dataCellTemplateProps,className:e.classes}),e.props.children)};t.viewFunction=m;var v=u({},f.CellBaseProps);t.DateTableCellBaseProps=v;var y=function(e){return e&&(e.defaultProps?function(t){return g.h(e,d({},t))}:e)};h.defaultProps=u({},v)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,g.useCallback)(function(){var t=e.className,n=e.isFirstGroupCell,i=e.isLastGroupCell;return(0,f.getGroupCellClasses)(n,i,t)},[e.className,e.isFirstGroupCell,e.isLastGroupCell]),n=(0,g.useCallback)(function(){var t=(e.allDay,e.children,e.className,e.contentTemplate,e.contentTemplateProps,e.endDate,e.groupIndex,e.groups,e.index,e.isFirstGroupCell,e.isLastGroupCell,e.startDate,e.text,r(e,["allDay","children","className","contentTemplate","contentTemplateProps","endDate","groupIndex","groups","index","isFirstGroupCell","isLastGroupCell","startDate","text"]));return t},[e]);return _({props:u(u({},e),{},{contentTemplate:v(e.contentTemplate)}),classes:t(),restAttributes:n()})}t.CellBase=h,t.CellBaseProps=t.viewFunction=void 0;var f=n(199),p=a(n(51)),g=n(53),_=function(e){var t=e.props.contentTemplate;return p.h("td",d({},e.restAttributes,{className:e.classes}),!e.props.contentTemplate&&e.props.children,e.props.contentTemplate&&t(u({},e.props.contentTemplateProps)))};t.viewFunction=_;var m={className:"",isFirstGroupCell:!1,isLastGroupCell:!1,startDate:new Date,endDate:new Date,allDay:!1,text:"",index:0,contentTemplateProps:{data:{},index:0}};t.CellBaseProps=m;var v=function(e){return e&&(e.defaultProps?function(t){return p.h(e,d({},t))}:e)};h.defaultProps=u({},m)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(4),y=n(0),b=n(24),x=o(n(9)),w=o(n(317)),C=o(n(20)),k=o(n(312)),S=o(n(841)),D=o(n(86)),I=k.default.tableCreator,T="dx-scheduler-timeline",O="dx-scheduler-group-table",E="dx-scheduler-work-space-horizontal-grouped",A="dx-scheduler-header-panel-cell",P="dx-scheduler-header-panel-week-cell",R="dx-scheduler-header-row",M="horizontal",B=1,V=10,F=C.default.dateToMilliseconds,L=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this.$element().addClass(T),this._$sidebarTable=(0,m.default)("<div>").addClass(O)}},{key:"_getCellFromNextRow",value:function(e,n){return n?this._$focusedCell:l(_(t.prototype),"_getCellFromNextRow",this).call(this,e,n)}},{key:"_getDefaultGroupStrategy",value:function(){return"vertical"}},{key:"_toggleGroupingDirectionClass",value:function(){this.$element().toggleClass(E,this._isHorizontalGroupedWorkSpace())}},{key:"_getDefaultOptions",value:function(){return(0,y.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{groupOrientation:"vertical"})}},{key:"_getRowCount",value:function(){return 1}},{key:"_getCellCount",value:function(){return this._getCellCountInDay()*this.option("intervalCount")}},{key:"getGroupTableWidth",value:function(){return this._$sidebarTable?this._$sidebarTable.outerWidth():0}},{key:"_getTotalRowCount",value:function(e){return this._isHorizontalGroupedWorkSpace()?this._getRowCount():(e=e||1,this._getRowCount()*e)}},{key:"_getDateForHeaderText",value:function(e){var t=D.default.getDateWithoutTimezoneChange(this._firstViewDate);return this._getDateByIndexCore(t,e)}},{key:"_getDateByIndexCore",value:function(e,t){var n=new Date(e),i=Math.floor(t/this._getCellCountInDay());return n.setTime(e.getTime()+this._calculateCellIndex(0,t)*this._getInterval()+i*this._getHiddenInterval()),n}},{key:"_getDateByIndex",value:function(e){var t=D.default.getDateWithoutTimezoneChange(this._firstViewDate),n=this._getDateByIndexCore(t,e);return D.default.isTimezoneChangeInDate(this._firstViewDate)&&n.setDate(n.getDate()-1),n}},{key:"_getFormat",value:function(){return"shorttime"}},{key:"_needApplyLastGroupCellClass",value:function(){return!0}},{key:"_calculateHiddenInterval",value:function(e,t){var n=Math.floor(t/this._getCellCountInDay());return n*this._getHiddenInterval()}},{key:"_getMillisecondsOffset",value:function(e,t){return t=this._calculateCellIndex(e,t),this._getInterval()*t+this._calculateHiddenInterval(e,t)}},{key:"_createWorkSpaceElements",value:function(){this._createWorkSpaceScrollableElements()}},{key:"_getWorkSpaceHeight",value:function(){return this.option("crossScrollingEnabled")?(0,b.getBoundingRect)(this._$dateTable.get(0)).height:(0,b.getBoundingRect)(this.$element().get(0)).height}},{key:"_dateTableScrollableConfig",value:function(){var e=l(_(t.prototype),"_dateTableScrollableConfig",this).call(this),n={direction:M};return this.option("crossScrollingEnabled")?e:(0,y.extend)(e,n)}},{key:"_needCreateCrossScrolling",value:function(){return!0}},{key:"_headerScrollableConfig",value:function(){var e=l(_(t.prototype),"_headerScrollableConfig",this).call(this);return(0,y.extend)(e,{scrollByContent:!0})}},{key:"_renderTimePanel",value:function(){return(0,v.noop)()}},{key:"_renderAllDayPanel",value:function(){return(0,v.noop)()}},{key:"_getTableAllDay",value:function(){return!1}},{key:"_getDateHeaderTemplate",value:function(){return this.option("timeCellTemplate")}},{key:"_toggleAllDayVisibility",value:function(){return(0,v.noop)()}},{key:"_changeAllDayVisibility",value:function(){return(0,v.noop)()}},{key:"supportAllDayRow",value:function(){return!1}},{key:"_getGroupHeaderContainer",value:function(){return this._isHorizontalGroupedWorkSpace()?this._$thead:this._$sidebarTable}},{key:"_insertAllDayRowsIntoDateTable",value:function(){return!1}},{key:"_createAllDayPanelElements",value:function(){return(0,v.noop)()}},{key:"_renderDateHeader",value:function(){var e=l(_(t.prototype),"_renderDateHeader",this).call(this);if(this._needRenderWeekHeader()){for(var n=new Date(this._firstViewDate),i=[],o=this._getCellCountInDay(),a=this.option("dateCellTemplate"),r=0;r<this._getWeekDuration()*this.option("intervalCount");r++){var s=(0,m.default)("<th>"),u=this._formatWeekdayAndDay(n);if(a){var c={model:{text:u,date:new Date(n)},container:s,index:r};a.render(c)}else s.text(u);s.addClass(A).addClass(P).attr("colSpan",o),i.push(s),this._incrementDate(n)}var d=(0,m.default)("<tr>").addClass(R).append(i);e.before(d)}}},{key:"_needRenderWeekHeader",value:function(){return!1}},{key:"_incrementDate",value:function(e){e.setDate(e.getDate()+1)}},{key:"_getWeekDuration",value:function(){return 1}},{key:"_renderView",value:function(){this._setFirstViewDate();var e=this._renderGroupHeader();this._renderDateHeader(),this._renderAllDayPanel(),this._renderTimePanel(),this._renderDateTable(),this._shader=new S.default(this),this._updateGroupTableHeight(),this._$sidebarTable.appendTo(this._sidebarScrollable.$content()),this._applyCellTemplates(e)}},{key:"_setHorizontalGroupHeaderCellsHeight",value:function(){return(0,v.noop)()}},{key:"getIndicationCellCount",value:function(){var e=this._getTimeDiff();return this._calculateDurationInCells(e)}},{key:"_getTimeDiff",value:function(){var e=this._getToday(),t=this._getIndicationFirstViewDate();return e.getTime()-t.getTime()}},{key:"_calculateDurationInCells",value:function(e){var t=this._getToday(),n=Math.floor(e/F("day")),i=(e-n*F("day")-this.option("startDayHour")*F("hour"))/this.getCellDuration();return t.getHours()>this.option("endDayHour")&&(i=this._getCellCountInDay()),i<0&&(i=0),n*this._getCellCountInDay()+i}},{key:"getIndicationWidth",value:function(){if(this.isGroupedByDate()){var e=this.getIndicationCellCount(),t=Math.floor(e),n=e-t;return this.getCellWidth()*(t*this._getGroupCount()+n)}return this.getIndicationCellCount()*this.getCellWidth()}},{key:"_renderIndicator",value:function(e,t,n,i){var o,a=this.getIndicationWidth();if("vertical"===this.option("groupOrientation"))o=this._createIndicator(n),o.height((0,b.getBoundingRect)(n.get(0)).height),o.css("left",t?t-a:a);else for(var r=0;r<i;r++){var s=this.isGroupedByDate()?r*this.getCellWidth():this._getCellCount()*this.getCellWidth()*r;o=this._createIndicator(n),o.height((0,b.getBoundingRect)(n.get(0)).height),o.css("left",t?t-a-s:a+s)}}},{key:"_isVerticalShader",value:function(){return!1}},{key:"_isCurrentTimeHeaderCell",value:function(e){var t=!1;if(this.isIndicationOnView()){var n=this._getDateByIndex(e),i=this._getToday();if(n=new Date(n),C.default.sameDate(i,n)){var o=new Date(n),a=new Date(n);a=a.setMilliseconds(n.getMilliseconds()+this.getCellDuration()),t=C.default.dateInRange(i,o,a)}}return t}},{key:"_cleanView",value:function(){l(_(t.prototype),"_cleanView",this).call(this),this._$sidebarTable.empty()}},{key:"_visibilityChanged",value:function(e){l(_(t.prototype),"_visibilityChanged",this).call(this,e)}},{key:"_setTableSizes",value:function(){var e=this.getCellHeight(),n=this._getWorkSpaceMinHeight(),i=this._$sidebarTable.find("tr"),o=e*i.length;o<n&&(o=n),this._$sidebarTable.height(o),this._$dateTable.height(o),l(_(t.prototype),"_setTableSizes",this).call(this)}},{key:"_getWorkSpaceMinHeight",value:function(){var e=this._getWorkSpaceHeight(),t=this.$element().outerHeight(!0)-this.getHeaderPanelHeight()-2*B-V;return e<t&&(e=t),e}},{key:"_makeGroupRows",value:function(e,t){var n="vertical"===this.option("groupOrientation")?I.VERTICAL:I.HORIZONTAL;return I.makeGroupedTable(n,e,{groupRowClass:this._getGroupRowClass(),groupHeaderRowClass:this._getGroupRowClass(),groupHeaderClass:this._getGroupHeaderClass.bind(this),groupHeaderContentClass:this._getGroupHeaderContentClass()},this._getCellCount()||1,this.option("resourceCellTemplate"),this._getTotalRowCount(this._getGroupCount()),t)}},{key:"_ensureGroupHeaderCellsHeight",value:function(e){var t=this._calculateMinCellHeight();return e<t?t:e}},{key:"_calculateMinCellHeight",value:function(){var e=this._getDateTable(),t="."+this._getDateTableRowClass();return(0,b.getBoundingRect)(e).height/e.find(t).length-2*B}},{key:"_getCellCoordinatesByIndex",value:function(e){return{cellIndex:e%this._getCellCount(),rowIndex:0}}},{key:"_getCellByCoordinates",value:function(e,t){var n=this._groupedStrategy.prepareCellIndexes(e,t);return this._$dateTable.find("tr").eq(n.rowIndex).find("td").eq(n.cellIndex)}},{key:"_getWorkSpaceWidth",value:function(){return this._$dateTable.outerWidth(!0)}},{key:"_getIndicationFirstViewDate",value:function(){return C.default.trimTime(new Date(this._firstViewDate))}},{key:"_getIntervalBetween",value:function(e,t){var n=this.option("startDayHour"),i=this.option("endDayHour"),o=this.getStartViewDate(),a=o.getTime(),r=(24-i+n)*F("hour"),s=C.default.getTimezonesDifference(o,e),l=e.getTime(),u=l-a-s,c=Math.floor(u/F("day")),d=u-c*F("day"),h=0,f=this._getCellCountInDay()*(c-this._getWeekendsCount(c)),p=l-C.default.trimTime(new Date(e)).getTime(),g=f*this.option("hoursInterval")*F("hour");return t||(e.getHours()<n?h=d-r+p:e.getHours()>=n&&e.getHours()<i?h=d:e.getHours()>=n&&e.getHours()>=i?h=d-(p-i*F("hour")):c||(g=u),g+=h),g}},{key:"_getWeekendsCount",value:function(){return 0}},{key:"getAllDayContainer",value:function(){return null}},{key:"getTimePanelWidth",value:function(){return 0}},{key:"getPositionShift",value:function(e){var n=l(_(t.prototype),"getPositionShift",this).call(this,e),i=this.getCellWidth()*e;return this.option("rtlEnabled")&&(i*=-1),i+=n.left,{top:0,left:i,cellPosition:i}}},{key:"getVisibleBounds",value:function(){var e=this.option("rtlEnabled"),t={},n=this.getScrollable().$element(),i=this.getCellWidth(),o=e?this.getScrollableOuterWidth()-this.getScrollableScrollLeft():this.getScrollableScrollLeft(),a=o/i,r=n.width()/i,s=e?a-r:a+r,l=this._getDateByIndex(a),u=this._getDateByIndex(s);return e&&(l=this._getDateByIndex(s),u=this._getDateByIndex(a)),t.left={hours:l.getHours(),minutes:l.getMinutes()>=30?30:0,date:C.default.trimTime(l)},t.right={hours:u.getHours(),minutes:u.getMinutes()>=30?30:0,date:C.default.trimTime(u)},t}},{key:"getIntervalDuration",value:function(e){return this.getCellDuration()}},{key:"_supportCompactDropDownAppointments",value:function(){return!1}},{key:"getCellMinWidth",value:function(){return 0}},{key:"getWorkSpaceLeftOffset",value:function(){return 0}},{key:"scrollToTime",value:function(e,t,n){var i=this._getScrollCoordinates(e,t,n),o=this.getScrollable(),a=this.option("rtlEnabled")?(0,b.getBoundingRect)(this.getScrollableContainer().get(0)).width:0;this.option("templatesRenderAsynchronously")?setTimeout(function(){o.scrollBy({left:i.left-o.scrollLeft()-a,top:0})}):o.scrollBy({left:i.left-o.scrollLeft()-a,top:0})}},{key:"_getRowCountWithAllDayRows",value:function(){return this._getRowCount()}}]),t}(w.default);(0,x.default)("dxSchedulerTimeline",L);var H=L;t.default=H,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(523)),y=o(n(9)),b=o(n(20)),x=n(0),w=n(24),C=n(7),k=b.default.dateToMilliseconds,S="dx-scheduler-date-time-indicator",D="dx-scheduler-time-panel-current-time-cell",I="dx-scheduler-header-panel-current-time-cell",T=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getTimeZoneCalculator",value:function(){return this.invoke("getTimeZoneCalculator")}},{key:"_getToday",value:function(){var e=this.option("indicatorTime")||new Date,t=this._getTimeZoneCalculator();return(null===t||void 0===t?void 0:t.createDate(e,{path:"toGrid"}))||e}},{key:"isIndicationOnView",value:function(){if(this.option("showCurrentTimeIndicator")){var e=this._getToday(),t=b.default.trimTime(this.getEndViewDate());return b.default.dateInRange(e,this._firstViewDate,new Date(t.getTime()+k("day")))}return!1}},{key:"isIndicationAvailable",value:function(){if(!(0,C.hasWindow)())return!1;var e=this._getToday();return e>=b.default.trimTime(new Date(this.getStartViewDate()))}},{key:"isIndicatorVisible",value:function(){var e=this._getToday(),t=new Date(this.getEndViewDate()),n=new Date(this.getStartViewDate());return n.setFullYear(e.getFullYear(),e.getMonth(),e.getDate()),t.setFullYear(e.getFullYear(),e.getMonth(),e.getDate()),b.default.dateInRange(e,n,t)}},{key:"_renderDateTimeIndication",value:function(){if(this.isIndicationAvailable()&&(this.option("shadeUntilCurrentTime")&&this._shader.render(),this.isIndicationOnView()&&this.isIndicatorVisible())){var e=this._getGroupCount()||1,t=this._dateTableScrollable.$content(),n=this.getIndicationHeight(),i=this._getRtlOffset(this.getCellWidth());this._renderIndicator(n,i,t,e)}}},{key:"_renderIndicator",value:function(e,t,n,i){for(var o=this.isGroupedByDate(),a=o?1:i,r=0;r<a;r++){var s=this._createIndicator(n);s.width(o?this.getCellWidth()*i:this.getCellWidth()),this._groupedStrategy.shiftIndicator(s,e,t,r)}}},{key:"_createIndicator",value:function(e){var t=(0,m.default)("<div>").addClass(S);return e.append(t),t}},{key:"_getRtlOffset",value:function(e){return this.option("rtlEnabled")?(0,w.getBoundingRect)(this._dateTableScrollable.$content().get(0)).width-this.getTimePanelWidth()-e:0}},{key:"_setIndicationUpdateInterval",value:function(){this.option("showCurrentTimeIndicator")&&0!==this.option("indicatorUpdateInterval")&&(this._clearIndicatorUpdateInterval(),this._indicatorInterval=setInterval(function(){this._refreshDateTimeIndication()}.bind(this),this.option("indicatorUpdateInterval")));
}},{key:"_clearIndicatorUpdateInterval",value:function(){this._indicatorInterval&&(clearInterval(this._indicatorInterval),delete this._indicatorInterval)}},{key:"_isVerticalShader",value:function(){return!0}},{key:"getIndicationWidth",value:function(e){var t=this.getCellWidth()*this._getCellCount(),n=this._getIndicatorDuration();n>this._getCellCount()&&(n=this._getCellCount());var i=n*this.getRoundedCellWidth(e,e*this._getCellCount(),n);return t<i?t:i}},{key:"getIndicatorOffset",value:function(e){var t=this._getIndicatorDuration()-1,n=t*this.getRoundedCellWidth(e,e*this._getCellCount(),t);return n}},{key:"_getIndicatorDuration",value:function(){var e=this._getToday(),t=new Date(this._firstViewDate),n=e.getTime()-t.getTime();return"workWeek"===this.option("type")&&(n-=this._getWeekendsCount(Math.round(n/k("day")))*k("day")),Math.ceil((n+1)/k("day"))}},{key:"getIndicationHeight",value:function(){var e=this._getToday(),t=this.getCellHeight(),n=new Date(this._firstViewDate);this.isIndicationOnView()&&n.setFullYear(e.getFullYear(),e.getMonth(),e.getDate());var i=e.getTime()-n.getTime(),o=i/this.getCellDuration();return o*t}},{key:"_dispose",value:function(){this._clearIndicatorUpdateInterval(),l(_(t.prototype),"_dispose",this).apply(this,arguments)}},{key:"_refreshDateTimeIndication",value:function(){this._cleanDateTimeIndicator(),this._shader&&this._shader.clean(),this._renderDateTimeIndication()}},{key:"_isCurrentTime",value:function(e){if(this.isIndicationOnView()){var t=this._getToday(),n=!1;e=new Date(e),e.setFullYear(t.getFullYear(),t.getMonth(),t.getDate());var i=new Date(e),o=new Date(e);return b.default.sameDate(t,e)&&(i=i.setMilliseconds(e.getMilliseconds()-this.getCellDuration()+1),o=o.setMilliseconds(e.getMilliseconds()+this.getCellDuration()),n=b.default.dateInRange(t,i,o)),n}}},{key:"_isCurrentTimeHeaderCell",value:function(e){if(this.isIndicationOnView()){var t=this._getDateByIndex(e);return b.default.sameDate(t,this._getToday())}return!1}},{key:"_getTimeCellClass",value:function(e){var n=this._getTimeCellDate(e),i=l(_(t.prototype),"_getTimeCellClass",this).call(this,e);return this._isCurrentTime(n)?i+" "+D:i}},{key:"_getHeaderPanelCellClass",value:function(e){var n=l(_(t.prototype),"_getHeaderPanelCellClass",this).call(this,e);return this._isCurrentTimeHeaderCell(e)?n+" "+I:n}},{key:"_cleanView",value:function(){l(_(t.prototype),"_cleanView",this).call(this),this._cleanDateTimeIndicator()}},{key:"_dimensionChanged",value:function(){l(_(t.prototype),"_dimensionChanged",this).call(this),this._refreshDateTimeIndication()}},{key:"_cleanDateTimeIndicator",value:function(){this.$element().find("."+S).remove()}},{key:"_cleanWorkSpace",value:function(){l(_(t.prototype),"_cleanWorkSpace",this).call(this),this._renderDateTimeIndication(),this._setIndicationUpdateInterval()}},{key:"_optionChanged",value:function(e){switch(e.name){case"showCurrentTimeIndicator":case"indicatorTime":this._cleanWorkSpace();break;case"indicatorUpdateInterval":this._setIndicationUpdateInterval();break;case"showAllDayPanel":l(_(t.prototype),"_optionChanged",this).call(this,e),this._refreshDateTimeIndication();break;case"allDayExpanded":l(_(t.prototype),"_optionChanged",this).call(this,e),this._refreshDateTimeIndication();break;case"crossScrollingEnabled":l(_(t.prototype),"_optionChanged",this).call(this,e),this._refreshDateTimeIndication();break;case"shadeUntilCurrentTime":this._refreshDateTimeIndication();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_getDefaultOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{showCurrentTimeIndicator:!0,indicatorTime:new Date,indicatorUpdateInterval:5*k("minute"),shadeUntilCurrentTime:!0})}}]),t}(v.default);(0,y.default)("dxSchedulerWorkSpace",T);var O=T;t.default=O,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(1),b=o(n(10)),x=o(n(16)),w=o(n(48)),C="dx-filemanager-dialog",k="dx-filemanager-dialog-popup",S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;l(_(t.prototype),"_initMarkup",this).call(this),this._createOnClosedAction();var n=this._getDialogOptions(),i=(0,m.default)("<div>").addClass(k).appendTo(this.$element());n.popupCssClass&&i.addClass(n.popupCssClass);var o={showTitle:!0,title:n.title,visible:!1,closeOnOutsideClick:!0,contentTemplate:this._createContentTemplate.bind(this),toolbarItems:[{widget:"dxButton",toolbar:"bottom",location:"after",options:{text:n.buttonText,onClick:this._applyDialogChanges.bind(this)}},{widget:"dxButton",toolbar:"bottom",location:"after",options:{text:b.default.format("dxFileManager-dialogButtonCancel"),onClick:this._closeDialog.bind(this)}}],onInitialized:function(t){var n=t.component;n.registerKeyHandler("enter",e._applyDialogChanges.bind(e))},onHidden:this._onPopupHidden.bind(this),onShown:this._onPopupShown.bind(this)};(0,y.isDefined)(n.height)&&(o.height=n.height),(0,y.isDefined)(n.maxHeight)&&(o.maxHeight=n.maxHeight),this._popup=this._createComponent(i,w.default,o)}},{key:"show",value:function(){this._dialogResult=null,this._popup.show()}},{key:"_getDialogOptions",value:function(){return{title:"Title",buttonText:"ButtonText",contentCssClass:"",popupCssClass:""}}},{key:"_createContentTemplate",value:function(e){this._$contentElement=(0,m.default)("<div>").appendTo(e).addClass(C);var t=this._getDialogOptions().contentCssClass;t&&this._$contentElement.addClass(t)}},{key:"_getDialogResult",value:function(){return null}},{key:"_applyDialogChanges",value:function(){var e=this._getDialogResult();e&&(this._dialogResult=e,this._popup.hide())}},{key:"_closeDialog",value:function(){this._popup.hide()}},{key:"_onPopupHidden",value:function(){this._onClosedAction({dialogResult:this._dialogResult})}},{key:"_onPopupShown",value:function(){}},{key:"_createOnClosedAction",value:function(){this._onClosedAction=this._createActionByOption("onClosed")}},{key:"_setTitle",value:function(e){this._popup.option("title",e)}},{key:"_setButtonText",value:function(e){this._popup.option("toolbarItems[0].options.text",e)}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{onClosed:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"onClosed":this._createOnPathChangedAction();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(x.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=n(105),s=function(){function e(t){i(this,e);var n=(0,r.getDiagram)(),o=n.EventDispatcher;this.onChanged=new o,this._owner=t}return a(e,[{key:"raiseBarCommandExecuted",value:function(e,t){this.onChanged.raise("notifyBarCommandExecuted",parseInt(e),t)}},{key:"getCommandKeys",value:function(){throw"Not Implemented"}},{key:"setItemValue",value:function(e,t){}},{key:"setItemEnabled",value:function(e,t){}},{key:"setItemVisible",value:function(e,t){}},{key:"setEnabled",value:function(e){}},{key:"setItemSubItems",value:function(e,t){}},{key:"isVisible",value:function(){return!0}},{key:"_getKeys",value:function(e){var t=this,n=e.reduce(function(e,n){return void 0!==n.command&&e.push(n.command),n.items&&(e=e.concat(t._getKeys(n.items))),e},[]);return n}}]),e}(),l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}t.default=void 0;var r=a(n(111)),s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){var n=t.end-t.start,i=0;for(e.forEach(function(e){i+=e.getBoundingRect().width});n<i;)i-=l(e)}function l(e){var t=e.reduce(function(e,t,n){if(!t)return e;var i=t.value();return i<e.value?{value:i,rollingStock:t,index:n}:e},{rollingStock:void 0,value:1/0,index:void 0});t.rollingStock.getLabels()[0].draw(!1);var n=t.rollingStock.getBoundingRect().width;return e[t.index]=null,n}function u(e){var t,n,i,o,a=!1;for(t=0,i=e.length-1;t<i;t++){for(n=t+1,o=e.length;n<o;n++)if(t!==n&&d(e[t],e[n],!0)){a=!0;break}if(a)break}return a}function c(e,t,n,i){var o=arguments.length>4&&void 0!==arguments[4]?arguments[4]:function(){return 0},a=[],r={start:n?t.left:t.top,end:n?t.width-t.right:t.height-t.bottom},l=!1;if(e.forEach(function(e){e&&(l=l||e.series.isStackedSeries()||e.series.isFullStackedSeries(),e.getLabels().forEach(function(e){e.isVisible()&&a.push(new g(e,n,i))}))}),l)!n&&a.reverse();else{var c=a.slice();a.sort(function(e,t){return o(e,t)||e.getInitialPosition()-t.getInitialPosition()||c.indexOf(e)-c.indexOf(t)})}return!!u(a)&&(s(a,r),h(a),a.reverse(),f(a,r),!0)}function d(e,t,n){if(e&&t){var i=e.getBoundingRect(),o=t.getBoundingRect(),a=!n||(i.oppositeStart<=o.oppositeStart&&i.oppositeEnd>o.oppositeStart||o.oppositeStart<=i.oppositeStart&&o.oppositeEnd>i.oppositeStart);return i.end>o.start&&a}}function h(e){var t,n,i;for(t=0;t<e.length-1;t++)n=i||e[t],d(n,e[t+1])?(n.toChain(e[t+1]),e[t+1]=null,i=n):i=e[t+1]||n}function f(e,t){var n,i,o,a,r,s;for(n=0;n<e.length;n++){if(o=e[n],p(o,t))for(r=o.getBoundingRect(),i=n+1;i<e.length;i++)if(a=e[i],a&&(s=a.getBoundingRect(),s.end>r.start-(r.end-t.end))){a.toChain(o),e[n]=o=null;break}o&&o.setRollingStockInCanvas(t)}}function p(e,t){return e&&e.getBoundingRect().end>t.end}function g(e,t,n){var i=e.getBoundingRect(),o=i.x,a=i.y,r=i.x+i.width,s=i.y+i.height;return this.labels=[e],this.shiftFunction=n,this._bBox={start:t?o:a,width:t?i.width:i.height,end:t?r:s,oppositeStart:t?a:o,oppositeEnd:t?s:r},this._initialPosition=t?i.x:i.y,this}function _(e){return{nameField:e+"Name",colorField:e+"Color",indexField:e+"Index"}}function m(e){var t=_(e);return{getFormatObject:function(e){var n={};return n[t.indexField]=e.id,n[t.colorField]=e.states.normal.fill,n[t.nameField]=e.text,n},textField:t.nameField}}function v(e,t){return(e.x<=t.x&&t.x<=e.x+e.width||e.x>=t.x&&e.x<=t.x+t.width)&&(e.y<=t.y&&t.y<=e.y+e.height||e.y>=t.y&&e.y<=t.y+t.height)}t.BaseChart=t.overlapping=void 0;var y=n(4),b=r(n(5)),x=n(1),w=n(3),C=n(0),k=n(12),S=n(8),D=r(n(112)),I=n(228),T=n(324),O=n(326),E=n(547),A=n(548),P=a(n(929)),R=n(14),M=n(111),B=n(131),V=n(174),F=n(133),L=n(153),H=Array.isArray,z="_reinit",N="_updateDataSource",j="_dataInit",W="_forceRender",G="_resize",q=[z,N,j,W,G],$=.3,K=["series","commonSeriesSettings","dataPrepareSettings","seriesSelectionMode","pointSelectionMode","synchronizeMultiAxes","resolveLabelsOverlapping"],U=["minBubbleSize","maxBubbleSize","barGroupPadding","barGroupWidth","negativesAsZeroes","negativesAsZeros"],Y=["adaptiveLayout","crosshair","resolveLabelOverlapping","adjustOnZoom","stickyHovering"],Z="font";g.prototype={toChain:function(e){var t=e.getBoundingRect();e.shift(t.start-this._bBox.end),this._changeBoxWidth(t.width),this.labels=this.labels.concat(e.labels)},getBoundingRect:function(){return this._bBox},shift:function(e){var t=this.shiftFunction;(0,w.each)(this.labels,function(n,i){var o=i.getBoundingRect(),a=t(o,e);i.hideInsideLabel(a)||i.shift(a.x,a.y)}),this._bBox.end-=e,this._bBox.start-=e},setRollingStockInCanvas:function(e){this._bBox.end>e.end&&this.shift(this._bBox.end-e.end)},getLabels:function(){return this.labels},value:function(){return this.labels[0].getData().value},getInitialPosition:function(){return this._initialPosition},_changeBoxWidth:function(e){this._bBox.end+=e,this._bBox.width+=e}};var X={resolveLabelOverlappingInOneDirection:c};t.overlapping=X;var Q=D.default.inherit({_eventsMap:{onSeriesClick:{name:"seriesClick"},onPointClick:{name:"pointClick"},onArgumentAxisClick:{name:"argumentAxisClick"},onLegendClick:{name:"legendClick"},onSeriesSelectionChanged:{name:"seriesSelectionChanged"},onPointSelectionChanged:{name:"pointSelectionChanged"},onSeriesHoverChanged:{name:"seriesHoverChanged"},onPointHoverChanged:{name:"pointHoverChanged"},onDone:{name:"done"},onZoomStart:{name:"zoomStart"},onZoomEnd:{name:"zoomEnd"}},_fontFields:["legend."+Z,"legend.title."+Z,"legend.title.subtitle."+Z,"commonSeriesSettings.label."+Z],_rootClassPrefix:"dxc",_rootClass:"dxc-chart",_initialChanges:["INIT"],_themeDependentChanges:["REFRESH_SERIES_REINIT"],_getThemeManagerOptions:function(){var e=this.callBase.apply(this,arguments);return e.options=this.option(),e},_createThemeManager:function(){var e=this.option(),t=new E.ThemeManager(this._getThemeManagerOptions());return t.setTheme(e.theme,e.rtlEnabled),t},_initCore:function(){var e=this;e._canvasClipRect=e._renderer.clipRect(),e._createHtmlStructure(),e._createLegend(),e._createTracker(),e._needHandleRenderComplete=!0,e.layoutManager=new A.LayoutManager,e._createScrollBar(),b.default.on(e._$element,"contextmenu",function(e){((0,S.isTouchEvent)(e)||(0,S.isPointerEvent)(e))&&e.preventDefault()}),b.default.on(e._$element,"MSHoldVisual",function(e){e.preventDefault()})},_getLayoutItems:y.noop,_layoutManagerOptions:function(){return this._themeManager.getOptions("adaptiveLayout")},_reinit:function(){var e=this;(0,R.setCanvasValues)(e._canvas),e._reinitAxes(),e._requestChange(["DATA_SOURCE","DATA_INIT","CORRECT_AXIS","FULL_RENDER"])},_correctAxes:y.noop,_createHtmlStructure:function(){var e=this,t=e._renderer,n=t.root,i=function(){return t.g().attr({"class":"dxc-constant-lines-group"}).linkOn(n,"constant-lines")};e._constantLinesGroup={dispose:function(){this.under.dispose(),this.above.dispose()},linkOff:function(){this.under.linkOff(),this.above.linkOff()},clear:function(){this.under.linkRemove().clear(),this.above.linkRemove().clear()},linkAppend:function(){this.under.linkAppend(),this.above.linkAppend()}},e._backgroundRect=t.rect().attr({fill:"gray",opacity:1e-4}).append(n),e._panesBackgroundGroup=t.g().attr({"class":"dxc-background"}).append(n),e._stripsGroup=t.g().attr({"class":"dxc-strips-group"}).linkOn(n,"strips"),e._gridGroup=t.g().attr({"class":"dxc-grids-group"}).linkOn(n,"grids"),e._panesBorderGroup=t.g().attr({"class":"dxc-border"}).linkOn(n,"border"),e._axesGroup=t.g().attr({"class":"dxc-axes-group"}).linkOn(n,"axes"),e._labelAxesGroup=t.g().attr({"class":"dxc-strips-labels-group"}).linkOn(n,"strips-labels"),e._constantLinesGroup.under=i(),e._seriesGroup=t.g().attr({"class":"dxc-series-group"}).linkOn(n,"series"),e._constantLinesGroup.above=i(),e._scaleBreaksGroup=t.g().attr({"class":"dxc-scale-breaks"}).linkOn(n,"scale-breaks"),e._labelsGroup=t.g().attr({"class":"dxc-labels-group"}).linkOn(n,"labels"),e._crosshairCursorGroup=t.g().attr({"class":"dxc-crosshair-cursor"}).linkOn(n,"crosshair"),e._legendGroup=t.g().attr({"class":"dxc-legend","clip-path":e._getCanvasClipRectID()}).linkOn(n,"legend").linkAppend(n).enableLinks(),e._scrollBarGroup=t.g().attr({"class":"dxc-scroll-bar"}).linkOn(n,"scroll-bar")},_disposeObjectsInArray:function(e,t){(0,w.each)(this[e]||[],function(e,n){t&&n?(0,w.each)(t,function(e,t){n[t]&&n[t].dispose()}):n&&n.dispose()}),this[e]=null},_disposeCore:function(){var e=this,t=function(t){e[t]&&(e[t].dispose(),e[t]=null)},n=function(t){e[t].linkOff()},i=this._disposeObjectsInArray;e._renderer.stopAllAnimations(),i.call(e,"series"),t("_tracker"),t("_crosshair"),e.layoutManager=e._userOptions=e._canvas=e._groupsData=null,n("_stripsGroup"),n("_gridGroup"),n("_axesGroup"),n("_constantLinesGroup"),n("_labelAxesGroup"),n("_panesBorderGroup"),n("_seriesGroup"),n("_labelsGroup"),n("_crosshairCursorGroup"),n("_legendGroup"),n("_scrollBarGroup"),n("_scaleBreaksGroup"),t("_canvasClipRect"),t("_panesBackgroundGroup"),t("_backgroundRect"),t("_stripsGroup"),t("_gridGroup"),t("_axesGroup"),t("_constantLinesGroup"),t("_labelAxesGroup"),t("_panesBorderGroup"),t("_seriesGroup"),t("_labelsGroup"),t("_crosshairCursorGroup"),t("_legendGroup"),t("_scrollBarGroup"),t("_scaleBreaksGroup")},_getAnimationOptions:function(){return this._themeManager.getOptions("animation")},_getDefaultSize:function(){return{width:400,height:400}},_getOption:function(e){return this._themeManager.getOptions(e)},_applySize:function(e){this._rect=e.slice(),this._changes.has("FULL_RENDER")||this._processRefreshData(G)},_resize:function(){this._doRender(this.__renderOptions||{animate:!1,isResize:!0})},_trackerType:"ChartTracker",_createTracker:function(){var e=this;e._tracker=new P[e._trackerType]({seriesGroup:e._seriesGroup,renderer:e._renderer,tooltip:e._tooltip,legend:e._legend,eventTrigger:e._eventTrigger})},_getTrackerSettings:function(){return(0,C.extend)({chart:this},this._getSelectionModes())},_getSelectionModes:function(){var e=this._themeManager;return{seriesSelectionMode:e.getOptions("seriesSelectionMode"),pointSelectionMode:e.getOptions("pointSelectionMode")}},_updateTracker:function(e){var t=this;t._tracker.update(t._getTrackerSettings()),t._tracker.setCanvases({left:0,right:t._canvas.width,top:0,bottom:t._canvas.height},e)},_createCanvasFromRect:function(e){var t=this._canvas;return(0,R.setCanvasValues)({left:e[0],top:e[1],right:t.width-e[2],bottom:t.height-e[3],width:t.width,height:t.height})},_doRender:function(e){var t=this;if(0!==t._canvas.width||0!==t._canvas.height){t._resetIsReady();var n=t._prepareDrawOptions(e),i=n.recreateCanvas;t._preserveOriginalCanvas(),i?t.__currentCanvas=t._canvas:t._canvas=t.__currentCanvas,i&&t._updateCanvasClipRect(t._canvas),this._canvas=this._createCanvasFromRect(this._rect),t._renderer.stopAllAnimations(!0),t._cleanGroups();var o=new Date;t._renderElements(n),t._lastRenderingTime=new Date-o}},_preserveOriginalCanvas:function(){this.__originalCanvas=this._canvas,this._canvas=(0,C.extend)({},this._canvas)},_layoutAxes:y.noop,_renderElements:function(e){var t,n,i,o=this,a=o._prepareToRender(e),r=o._isRotated(),s=o._isLegendInside(),l=[];(0,C.extend)({},o._canvas);o._renderer.lock(),e.drawLegend&&o._legend&&o._legendGroup.linkAppend(),o.layoutManager.setOptions(o._layoutManagerOptions());var u=o._getLayoutTargets();this._layoutAxes(function(t){var n=t?(0,C.extend)({},e,{animate:!1,recreateCanvas:!0}):e,i=o._renderAxes(n,a);o._shrinkAxes(t,i)}),o._applyClipRects(a),o._appendSeriesGroups(),o._createCrosshairCursor(),u.forEach(function(e){var t=e.canvas;l.push({left:t.left,right:t.width-t.right,top:t.top,bottom:t.height-t.bottom})}),o._scrollBar&&(t=o._argumentAxes[0].getTranslator().getBusinessRange(),"discrete"===t.axisType&&t.categories&&t.categories.length<=1||"discrete"!==t.axisType&&t.min===t.max?n=i=void 0:(n=t.minVisible,i=t.maxVisible),o._scrollBar.init(t,!o._argumentAxes[0].getOptions().valueMarginsEnabled).setPosition(n,i)),o._updateTracker(l),o._updateLegendPosition(e,s),o._applyPointMarkersAutoHiding(),o._renderSeries(e,r,s),o._renderer.unlock()},_updateLegendPosition:y.noop,_createCrosshairCursor:y.noop,_appendSeriesGroups:function(){this._seriesGroup.linkAppend(),this._labelsGroup.linkAppend(),this._appendAdditionalSeriesGroups()},_renderSeries:function(e,t,n){this._calculateSeriesLayout(e,t),this._renderSeriesElements(e,n)},_calculateSeriesLayout:function(e,t){e.hideLayoutLabels=this.layoutManager.needMoreSpaceForPanesCanvas(this._getLayoutTargets(),t)&&!this._themeManager.getOptions("adaptiveLayout").keepLabels,this._updateSeriesDimensions(e)},_getArgFilter:function(){return function(){return!0}},_getValFilter:function(e){return function(){return!0}},_getPointsToAnimation:function(e){var t=this,n=this._getArgFilter();return e.map(function(e){var i=t._getValFilter(e);return e.getPoints().filter(function(e){return e.getOptions().visible&&n(e.argument)&&(i(e.getMinValue(!0))||i(e.getMaxValue(!0)))}).length})},_renderSeriesElements:function(e,t){var n,i,o=this,a=o.series,r=a.length,s=o._themeManager.getOptions("resolveLabelOverlapping"),l=o._getPointsToAnimation(a);for(n=0;n<r;n++)i=a[n],o._applyExtraSettings(i,e),i.draw(e.animate&&l[n]<=e.animationPointsLimit&&o._renderer.animationEnabled(),e.hideLayoutLabels,o._getLegendCallBack(i));"none"===s?o._adjustSeriesLabels(!1):o._locateLabels(s),o._renderTrackers(t),o._tracker.repairTooltip(),o._renderExtraElements(),o._clearCanvas(),o._seriesElementsDrawn=!0},_changesApplied:function(){var e=this;e._seriesElementsDrawn&&(e._seriesElementsDrawn=!1,e._drawn(),e._renderCompleteHandler())},_locateLabels:function(e){this._resolveLabelOverlapping(e)},_renderExtraElements:function(){},_clearCanvas:function(){this._canvas=this.__originalCanvas},_resolveLabelOverlapping:function(e){var t;switch(e){case"stack":t=this._resolveLabelOverlappingStack;break;case"hide":t=this._resolveLabelOverlappingHide;break;case"shift":t=this._resolveLabelOverlappingShift}return(0,x.isFunction)(t)&&t.call(this)},_getVisibleSeries:function(){return(0,y.grep)(this.getAllSeries(),function(e){return e.isVisible()})},_resolveLabelOverlappingHide:function(){var e,t,n,i,o,a,r,s=[],l=this._getVisibleSeries();for(o=0;o<l.length;o++)for(r=l[o].getVisiblePoints(),a=0;a<r.length;a++)s.push.apply(s,r[a].getLabels());for(o=0;o<s.length;o++)if(e=s[o],e.isVisible())for(n=e.getBoundingRect(),a=o+1;a<s.length;a++)t=s[a],i=t.getBoundingRect(),v(n,i)&&t.draw(!1)},_cleanGroups:function(){var e=this;e._stripsGroup.linkRemove().clear(),e._gridGroup.linkRemove().clear(),e._axesGroup.linkRemove().clear(),e._constantLinesGroup.above.clear(),e._labelAxesGroup.linkRemove().clear(),e._labelsGroup.linkRemove().clear(),e._crosshairCursorGroup.linkRemove().clear(),e._scaleBreaksGroup.linkRemove().clear()},_allowLegendInsidePosition:function(){return!1},_createLegend:function(){var e=this,t=m(e._legendDataField);e._legend=new I.Legend({renderer:e._renderer,widget:e,group:e._legendGroup,backgroundClass:"dxc-border",itemGroupClass:"dxc-item",titleGroupClass:"dxc-title",textField:t.textField,getFormatObject:t.getFormatObject,allowInsidePosition:e._allowLegendInsidePosition()}),e._updateLegend(),e._layout.add(e._legend)},_updateLegend:function(){var e=this,t=e._themeManager,n=t.getOptions("legend"),i=e._getLegendData();n.containerBackgroundColor=t.getOptions("containerBackgroundColor"),n._incidentOccurred=e._incidentOccurred,e._legend.update(i,n,t.theme("legend").title),this._change(["LAYOUT"])},_prepareDrawOptions:function(e){var t=this._getAnimationOptions(),n=(0,C.extend)({},{force:!1,adjustAxes:!0,drawLegend:!0,drawTitle:!0,animate:t.enabled,animationPointsLimit:t.maxPointCountSupported},e,this.__renderOptions);return(0,x.isDefined)(n.recreateCanvas)||(n.recreateCanvas=n.adjustAxes&&n.drawLegend&&n.drawTitle),n},_processRefreshData:function(e){var t=(0,k.inArray)(this._currentRefreshData,q),n=(0,k.inArray)(e,q);(!this._currentRefreshData||t>=0&&n<t)&&(this._currentRefreshData=e),this._requestChange(["REFRESH"])},_getLegendData:function(){return(0,R.map)(this._getLegendTargets(),function(e){var t=e.legendData,n=e.getLegendStyles,i=n.normal.opacity;e.visible||((!(0,x.isDefined)(i)||i>$)&&(i=$),t.textOpacity=$);var o={opacity:i};return t.states={hover:(0,C.extend)({},n.hover,o),selection:(0,C.extend)({},n.selection,o),normal:(0,C.extend)({},n.normal,o)},t})},_getLegendOptions:function(e){return{legendData:{text:e[this._legendItemTextField],id:e.index,visible:!0},getLegendStyles:e.getLegendStyles(),visible:e.isVisible()}},_disposeSeries:function(e){var t,n=this;n.series&&((0,x.isDefined)(e)?(n.series[e].dispose(),n.series.splice(e,1)):((0,w.each)(n.series,function(e,t){return t.dispose()}),n.series.length=0)),null!==(t=n.series)&&void 0!==t&&t.length||(n.series=[])},_disposeSeriesFamilies:function(){var e=this;(0,w.each)(e.seriesFamilies||[],function(e,t){t.dispose()}),e.seriesFamilies=null,e._needHandleRenderComplete=!0},_optionChanged:function(e){this._themeManager.resetOptions(e.name),this.callBase.apply(this,arguments)},_applyChanges:function(){var e=this;e._themeManager.update(e._options.silent()),e.callBase.apply(e,arguments)},_optionChangesMap:{animation:"ANIMATION",dataSource:"DATA_SOURCE",palette:"PALETTE",paletteExtensionMode:"PALETTE",legend:"FORCE_DATA_INIT",seriesTemplate:"FORCE_DATA_INIT","export":"FORCE_RENDER",valueAxis:"AXES_AND_PANES",argumentAxis:"AXES_AND_PANES",commonAxisSettings:"AXES_AND_PANES",panes:"AXES_AND_PANES",commonPaneSettings:"AXES_AND_PANES",defaultPane:"AXES_AND_PANES",containerBackgroundColor:"AXES_AND_PANES",rotated:"ROTATED",autoHidePointMarkers:"REFRESH_SERIES_REINIT",customizePoint:"REFRESH_SERIES_REINIT",customizeLabel:"REFRESH_SERIES_REINIT",scrollBar:"SCROLL_BAR"},_optionChangesOrder:["ROTATED","PALETTE","REFRESH_SERIES_REINIT","AXES_AND_PANES","INIT","REINIT","DATA_SOURCE","REFRESH_SERIES_DATA_INIT","DATA_INIT","FORCE_DATA_INIT","REFRESH_AXES","CORRECT_AXIS"],_customChangesOrder:["ANIMATION","REFRESH_SERIES_FAMILIES","FORCE_FIRST_DRAWING","FORCE_DRAWING","FORCE_RENDER","VISUAL_RANGE","SCROLL_BAR","REINIT","REFRESH","FULL_RENDER"],_change_ANIMATION:function(){this._renderer.updateAnimationOptions(this._getAnimationOptions())},_change_DATA_SOURCE:function(){this._needHandleRenderComplete=!0,this._updateDataSource()},_change_PALETTE:function(){this._themeManager.updatePalette(),this._refreshSeries("DATA_INIT")},_change_REFRESH_SERIES_DATA_INIT:function(){this._refreshSeries("DATA_INIT")},_change_DATA_INIT:function(){this.series&&!this.needToPopulateSeries||this._changes.has("FORCE_DATA_INIT")||this._dataInit()},_change_FORCE_DATA_INIT:function(){this._dataInit()},_change_REFRESH_SERIES_FAMILIES:function(){this._processSeriesFamilies(),this._populateBusinessRange(),this._processRefreshData(W)},_change_FORCE_RENDER:function(){this._processRefreshData(W)},_change_AXES_AND_PANES:function(){this._refreshSeries("INIT")},_change_ROTATED:function(){this._createScrollBar(),this._refreshSeries("INIT")},_change_REFRESH_SERIES_REINIT:function(){this._refreshSeries("INIT")},_change_REFRESH_AXES:function(){var e=this;(0,R.setCanvasValues)(e._canvas),e._reinitAxes(),e._requestChange(["CORRECT_AXIS","FULL_RENDER"])},_change_SCROLL_BAR:function(){this._createScrollBar(),this._processRefreshData(W)},_change_REINIT:function(){this._processRefreshData(z)},_change_FORCE_DRAWING:function(){this._resetComponentsAnimation()},_change_FORCE_FIRST_DRAWING:function(){this._resetComponentsAnimation(!0)},_resetComponentsAnimation:function(e){this.series.forEach(function(t){t.resetApplyingAnimation(e)}),this._resetAxesAnimation(e)},_resetAxesAnimation:y.noop,_refreshSeries:function(e){this.needToPopulateSeries=!0,this._requestChange([e])},_change_CORRECT_AXIS:function(){this._correctAxes()},_doRefresh:function(){var e=this._currentRefreshData;e&&(this._currentRefreshData=null,this._renderer.stopAllAnimations(!0),this[e]())},_updateCanvasClipRect:function(e){var t=this,n=Math.max(e.width-e.left-e.right,0),i=Math.max(e.height-e.top-e.bottom,0);t._canvasClipRect.attr({x:e.left,y:e.top,width:n,height:i}),t._backgroundRect.attr({x:e.left,y:e.top,width:n,height:i})},_getCanvasClipRectID:function(){return this._canvasClipRect.id},_dataSourceChangedHandler:function(){this._changes.has("INIT")?this._requestChange(["DATA_INIT"]):this._requestChange(["FORCE_DATA_INIT"])},_dataInit:function(){this._dataSpecificInit(!0)},_processSingleSeries:function(e){e.createPoints(!1)},_handleSeriesDataUpdated:function(){var e=this;this._getVisibleSeries().some(function(e){return e.useAggregation()})&&this._populateMarginOptions(),this.series.forEach(function(t){return e._processSingleSeries(t)},this)},_dataSpecificInit:function(e){var t=this;t.series&&!t.needToPopulateSeries||(t.series=t._populateSeries()),t._repopulateSeries(),t._seriesPopulatedHandlerCore(),t._populateBusinessRange(),t._tracker.updateSeries(t.series,this._changes.has("INIT")),t._updateLegend(),e&&this._requestChange(["FULL_RENDER"])},_forceRender:function(){this._doRender({force:!0})},_repopulateSeries:function(){var e=this,t=e._themeManager,n=e._dataSourceItems(),i=t.getOptions("dataPrepareSettings"),o=t.getOptions("seriesTemplate");o&&e._populateSeries(n),e._groupSeries();var a=(0,T.validateData)(n,e._groupsData,e._incidentOccurred,i);t.resetPalette(),e.series.forEach(function(e){e.updateData(a[e.getArgumentField()])}),e._handleSeriesDataUpdated()},_renderCompleteHandler:function(){var e=this,t=!0;e._needHandleRenderComplete&&((0,w.each)(e.series,function(e,n){t=t&&n.canRenderCompleteHandle()}),t&&(e._needHandleRenderComplete=!1,e._eventTrigger("done",{target:e})))},_dataIsReady:function(){return(0,x.isDefined)(this.option("dataSource"))&&this._dataIsLoaded();
},_populateSeriesOptions:function(e){for(var t,n,i=this,o=i._themeManager,a=o.getOptions("seriesTemplate"),r=a?(0,R.processSeriesTemplate)(a,e||[]):i.option("series"),s=H(r)?r:r?[r]:[],l=i._getExtraOptions(),u=[],c=function(e){i._specialProcessSeries(),i._populateBusinessRange(e&&e.getValueAxis(),!0),i._renderer.stopAllAnimations(!0),i._updateLegend(),i._requestChange(["FULL_RENDER"])},d=0;d<s.length;d++)t=(0,C.extend)(!0,{},s[d],l),(0,x.isDefined)(t.name)&&""!==t.name||(t.name="Series "+(d+1).toString()),t.rotated=i._isRotated(),t.customizePoint=o.getOptions("customizePoint"),t.customizeLabel=o.getOptions("customizeLabel"),t.visibilityChanged=c,t.incidentOccurred=i._incidentOccurred,n=o.getOptions("series",t,s.length),i._checkPaneName(n)&&u.push(n);return u},_populateSeries:function(e){var t,n=this,i=[],o=n._incidentOccurred,a=n._populateSeriesOptions(e),r=!1;n.needToPopulateSeries=!1,(0,w.each)(a,function(e,t){var o=n.series&&n.series.filter(function(e){return e.name===t.name&&i.map(function(e){return e.series}).indexOf(e)===-1})[0];o&&o.type===t.type?i.push({series:o,options:t}):(i.push({options:t}),r=!0)}),n._tracker.clearHover(),(0,w.reverseEach)(n.series,function(e,t){i.some(function(e){return t===e.series})||(n._disposeSeries(e),r=!0)}),!r&&(r=i.some(function(e){return e.series.name!==a[e.series.index].name})),n.series=[],r&&n._disposeSeriesFamilies(),n._themeManager.resetPalette();var s=function(e){n.series.forEach(function(t){t.notify(e)})};return(0,w.each)(i,function(e,i){var a=i.options,r={commonSeriesModes:n._getSelectionModes(),argumentAxis:n.getArgumentAxis(),valueAxis:n._getValueAxis(a.pane,a.axis)};i.series?(t=i.series,t.updateOptions(a,r)):t=new O.Series((0,C.extend)({renderer:n._renderer,seriesGroup:n._seriesGroup,labelsGroup:n._labelsGroup,eventTrigger:n._eventTrigger,eventPipe:s,incidentOccurred:o},r),a),t.isUpdated?(t.index=n.series.length,n.series.push(t)):o("E2101",[a.type])}),n.series},getStackedPoints:function(e){var t=e.series.getStackName();return this._getVisibleSeries().reduce(function(n,i){return(0,x.isDefined)(i.getStackName())&&(0,x.isDefined)(t)&&t!==i.getStackName()||(n=n.concat(i.getPointsByArg(e.argument))),n},[])},getAllSeries:function(){return(this.series||[]).slice()},getSeriesByName:function(e){var t=null;return(0,w.each)(this.series,function(n,i){if(i.name===e)return t=i,!1}),t},getSeriesByPos:function(e){return(this.series||[])[e]},clearSelection:function(){this._tracker.clearSelection()},hideTooltip:function(){this._tracker._hideTooltip()},clearHover:function(){this._tracker.clearHover()},render:function(e){var t=this;return t.__renderOptions=e,t.__forceRender=e&&e.force,t.callBase.apply(t,arguments),t.__renderOptions=t.__forceRender=null,t},refresh:function(){this._disposeSeries(),this._disposeSeriesFamilies(),this._requestChange(["CONTAINER_SIZE","REFRESH_SERIES_REINIT"])},_getMinSize:function(){var e=this._layoutManagerOptions();return[e.width,e.height]},_change_REFRESH:function(){this._changes.has("INIT")?this._currentRefreshData=null:this._doRefresh()},_change_FULL_RENDER:function(){this._forceRender()},_change_INIT:function(){this._reinit()},_stopCurrentHandling:function(){this._tracker.stopCurrentHandling()}});t.BaseChart=Q,K.forEach(function(e){Q.prototype._optionChangesMap[e]="REFRESH_SERIES_DATA_INIT"}),Y.forEach(function(e){Q.prototype._optionChangesMap[e]="FORCE_RENDER"}),U.forEach(function(e){Q.prototype._optionChangesMap[e]="REFRESH_SERIES_FAMILIES"}),Q.addPlugin(M.plugin),Q.addPlugin(B.plugin),Q.addPlugin(V.plugin),Q.addPlugin(F.plugin),Q.addPlugin(L.plugin);var J=Q.prototype._change_TITLE;Q.prototype._change_TITLE=function(){J.apply(this,arguments),this._change(["FORCE_RENDER"])}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=e;return t&&_(t.split("."),function(e,t){return n=n[t]}),n}t.BaseThemeManager=void 0;var a=i(n(15)),r=n(0),s=n(1),l=n(3),u=n(201),c=n(14),d=n(57);n(908),n(909),n(910),n(911),n(912),n(913),n(914),n(915),n(916);var h=d.getTheme,f=d.addCacheItem,p=d.removeCacheItem,g=r.extend,_=l.each,m=a.default.inherit({ctor:function(e){this._themeSection=e.themeSection,this._fontFields=e.fontFields||[],f(this)},dispose:function(){var e=this;return p(e),e._callback=e._theme=e._font=null,e},setCallback:function(e){return this._callback=e,this},setTheme:function(e,t){return this._current=e,this._rtl=t,this.refresh()},refresh:function(){var e=this,t=e._current||{},n=h(t.name||t);return e._themeName=n.name,e._defaultPalette=n.defaultPalette,e._font=g({},n.font,t.font),e._themeSection&&_(e._themeSection.split("."),function(e,t){n=g(!0,{},n[t])}),e._theme=g(!0,{},n,(0,s.isString)(t)?{}:t),e._initializeTheme(),(0,c.parseScalar)(e._rtl,e._theme.rtlEnabled)&&g(!0,e._theme,e._theme._rtl),e._callback(),e},theme:function(e){return o(this._theme,e)},themeName:function(){return this._themeName},createPalette:function(e,t){return(0,u.createPalette)(e,t,this._defaultPalette)},createDiscretePalette:function(e,t){return(0,u.getDiscretePalette)(e,t,this._defaultPalette)},createGradientPalette:function(e){return(0,u.getGradientPalette)(e,this._defaultPalette)},getAccentColor:function(e){return(0,u.getAccentColor)(e,this._defaultPalette)},_initializeTheme:function(){var e=this;_(e._fontFields||[],function(t,n){e._initializeFont(o(e._theme,n))})},_initializeFont:function(e){g(e,this._font,g({},e))}});t.BaseThemeManager=m},function(e,t,n){function i(e){this._options=e}function o(e,t){this._renderElement=e,this._cacheBBox=t}t.LayoutElement=i,t.WrapperLayoutElement=o;var a=n(4),r=n(50),s=Math.round,l={horizontal:0,vertical:0},u={center:.5,right:1,bottom:1,left:0,top:0};i.prototype={constructor:i,position:function(e){var t=this,n=e.of.getLayoutOptions(),i=t.getLayoutOptions(),o=e.at,a=e.my,r=e.offset||l,c=-u[a.horizontal]*i.width+n.x+u[o.horizontal]*n.width+parseInt(r.horizontal),d=-u[a.vertical]*i.height+n.y+u[o.vertical]*n.height+parseInt(r.vertical);t.shift(s(c),s(d))},getLayoutOptions:a.noop};var c=o.prototype=(0,r.clone)(i.prototype);c.constructor=o,c.getLayoutOptions=function(){return this._cacheBBox||this._renderElement.getBBox()},c.shift=function(e,t){var n=this.getLayoutOptions();this._renderElement.move(s(e-n.x),s(t-n.y))}},function(e,t,n){function i(e,t,n,i){i>=0&&e.slice(i).forEach(function(e){(0,B.isDefined)(e[n])&&(t[n]+=e[n],e[n]=void 0)})}function o(e){e.forEach(function(e){e.valueType=e.valueAxisType=null,e.series.forEach(function(e){e.updateDataType({})}),e.valueAxis&&e.valueAxis.resetTypes($)})}function a(e,t,n){var i=t?function(e,t){return e-t}:function(e,t){return t-e};return e.sort(function(e,t){var o=n(e),a=n(t),r=(0,B.isDefined)(o)?1:0,s=(0,B.isDefined)(a)?1:0;return r&&s?i(o,a):i(r,s)}),e}function r(e){e&&e.forEach(function(e){e.resetTypes(K)})}function s(e,t){var n=[];return e.forEach(function(e){var i=t(e);void 0!==i&&n.push(i)}),n}function l(e,t){var n=e.argumentOptions&&e.argumentOptions.categories;e.groups.forEach(function(e,n){var i=e.valueOptions&&e.valueOptions.categories;i&&(e.valueOptions.categories=s(i,t[n+1]))}),n&&(e.argumentOptions.categories=s(n,t[0]))}function u(e){return e}function c(e,t){var n=t;return t===H||(0,B.isString)(e)?n=H:t===N||(0,B.isDate)(e)?n=N:(0,B.isNumeric)(e)&&(n=z),n}function d(e,t,n,i){return e!==H||t!==G&&t!==q&&t!==W||i("E2002"),t===q?q:n||t===j||e===H?j:t===W?W:G}function h(e,t,n){e&&n((0,B.isNumeric)(e)||(0,B.isDate)(e)||(0,B.isString)(e)?"E2004":"E2003",[t])}function f(e,t,n){var i=e?(0,L.getParser)(e):u,o=t!==j?function(e){return isFinite(e)||void 0===e?e:null}:u;return function(e,t){var a=o(i(e));return void 0===a&&h(e,t,n),a}}function p(e,t){var n,i,o=f(e.argumentType,e.argumentAxisType,t),a=[o],r={},s=[];e.groups.forEach(function(e,s){e.series.forEach(function(l){i=f(e.valueType,e.valueAxisType,t),n=f(z,G,t),r[l.getArgumentField()]=o,l.getValueFields().forEach(function(e){a[s+1]=i,r[e]=i}),l.getSizeField()&&(r[l.getSizeField()]=n)})});for(var u in r)s.push([u,r[u]]);return s.length&&l(e,a),s}function g(e,t){var n,i,o,a=t.length,r=(0,V.extend)({},e);for(n=0;n<a;++n)i=t[n][0],o=e[i],r[i]=t[n][1](o,i);return r}function _(e,t){var n,i=[],o=e.length;for(i.length=o,n=0;n<o;++n)i[n]=g(e[n],t);return i}function m(e,t,n){var i,o,a=e.length;for(i=0;i<a&&(o=e[i][t],!((0,B.isDefined)(o)&&n>o));++i);return i}function v(e,t,n,o){o=o||{};var r=o.mode,s={};if(r&&"none"!==r){s[t]=String(o.groupName||"others"),s[n]=0;var l=a(e.slice(),!1,function(e){return e[n]});i(l,s,n,"smallValueThreshold"===r?m(l,n,o.threshold):o.topCount),s[n]&&e.push(s)}}function y(e,t){var n=t.groups[0]&&t.groups[0].series[0],i=n&&("pie"===n.type||"doughnut"===n.type||"donut"===n.type);i&&t.groups.forEach(function(t){t.series.forEach(function(t){v(e,t.getArgumentField(),t.getValueFields()[0],t.getOptions().smallValuesGrouping)})})}function b(e,t,n){n[e]||(t.push(e),n[e]=!0)}function x(e){var t=[],n={};return e.groups.forEach(function(e){e.series.forEach(function(e){b(e.getArgumentField(),t,n)})}),t}function w(e,t){var n=e-t;return isNaN(n)?(0,B.isDefined)(e)?(0,B.isDefined)(t)?0:-1:1:n}function C(e,t){return e.slice().sort(function(e,n){return w(e[t],n[t])})}function k(e,t){return e.slice().sort(t)}function S(e,t){return e.series.forEach(function(n){n.getValueFields().forEach(function(n){e.valueType=c(t[n],e.valueType)})}),e.valueType}function D(e){var t={};return e.forEach(function(e,n){t[e]=n}),function(e,n){return a(e.slice(),!0,function(e){return t[e[n]]})}}function I(e,t,n,i){var o,a={},r=t.argumentAxisType===j,s=r&&t.argumentOptions&&t.argumentOptions.categories,l=function(e){return e},u=n.sortingMethod;return!s&&(0,B.isFunction)(u)&&(e=k(e,u)),r&&(t.categories=O(e,i,s)),s||!(0,B.isFunction)(u)&&t.argumentType===H&&!n._skipArgumentSorting?l=D(t.categories):u===!0&&t.argumentType!==H&&(l=C,o=r),i.forEach(function(t){a[t]=l(e,t)}),o&&(t.categories=t.categories.sort(w)),a}function T(e,t){return e.map(function(e){return e.valueOf()}).indexOf(t.valueOf())===-1}function O(e,t,n){var i=n?n.slice():[];return t.forEach(function(t){e.forEach(function(e){var n=e[t];(0,B.isDefined)(n)&&T(i,n)&&i.push(n)})}),i}function E(e,t,n){return e.forEach(function(e){n.argumentType=c(t[e.getArgumentField()],n.argumentType)}),n.argumentType}function A(e,t,n){var i,o=[],a=[],r=t.argumentOptions&&U(t.argumentOptions.argumentType);t.groups.forEach(function(e){if(e.series.length){var n=e.valueOptions&&U(e.valueOptions.valueType);e.valueType=n,t.argumentType=r,!n&&o.push(e),!r&&a.push(e)}}),(o.length||a.length)&&(i=o.map(function(e,t){return t}),e.some(function(e){var r;if(o.forEach(function(t,n){S(t,e)&&i.indexOf(n)>=0&&i.splice(n,1)}),r||a.forEach(function(n){r=E(n.series,e,t)}),!n&&r&&0===i.length)return!0}))}function P(e,t){var n=e.argumentOptions||{},i=n&&n.categories||[],o=d(e.argumentType,n.type,!!i.length,t);e.groups.forEach(function(n){var i=n.valueOptions||{},a=i.categories||[],r=d(n.valueType,i.type,!!a.length,t);n.series.forEach(function(t){var a={};a.argumentAxisType=o,a.valueAxisType=r,e.argumentAxisType=e.argumentAxisType||a.argumentAxisType,n.valueAxisType=n.valueAxisType||a.valueAxisType,a.argumentType=e.argumentType,a.valueType=n.valueType,a.showZero=i.showZero,t.updateDataType(a)}),n.valueAxisType=n.valueAxisType||r,n.valueAxis&&(n.valueAxis.setTypes(n.valueAxisType,n.valueType,$),n.valueAxis.validate())}),e.argumentAxisType=e.argumentAxisType||o,e.argumentAxes&&e.argumentAxes.forEach(function(t){t.setTypes(e.argumentAxisType,e.argumentType,K),t.validate()})}function R(e,t){var n,i,o,a,r=[],s=(0,B.isDefined)(e),l=s&&!Y(e);if(s&&!l)for(n=0,i=e.length,o=0;n<i;++n)a=e[n],(0,B.isObject)(a)?r[o++]=a:a&&(l=!0);return l&&t("E2001"),r}function M(e,t,n,i){e=R(e,n),t.argumentType=t.argumentAxisType=null,o(t.groups),r(t.argumentAxes),A(e,t,i.checkTypeForAllData),P(t,n),i.convertToAxisDataType&&(e=_(e,p(t,n))),y(e,t);var a=I(e,t,i,x(t));return a}t.validateData=M;var B=n(1),V=n(0),F=n(14),L=n(325),H="string",z="numeric",N="datetime",j="discrete",W="semidiscrete",G="continuous",q="logarithmic",$="valueType",K="argumentType",U=(0,F.enumParser)([H,z,N]),Y=Array.isArray},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return"numeric"===e||"datetime"===e||"string"===e?e:""}t.correctValueType=o,t.getParser=void 0;var a=n(4),r=i(n(62)),s=n(1),l={string:function(e){return(0,s.isDefined)(e)?""+e:e},numeric:function(e){if(!(0,s.isDefined)(e))return e;var t=Number(e);return isNaN(t)&&(t=void 0),t},datetime:function(e){if(!(0,s.isDefined)(e))return e;var t,n=Number(e);return t=isNaN(n)?r.default.deserializeDate(e):new Date(n),isNaN(Number(t))&&(t=void 0),t}},u=function(e){return l[o(e)]||a.noop};t.getParser=u},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t,n){e&&e.trigger(t,n)}function l(e){return e=(0,C.normalizeEnum)(e),e===U||e===ee}function u(e,t){var n=e||{},i=(0,b.extend)({},n.font)||{},o=n.border||{},a=n.connector||{},r={fill:n.backgroundColor||t,"stroke-width":o.visible?o.width||0:0,stroke:o.visible&&o.width?o.color:"none",dashStyle:o.dashStyle},s={stroke:a.visible&&a.width?a.color||t:"none","stroke-width":a.visible?a.width||0:0};return i.color="none"===n.backgroundColor&&"#ffffff"===(0,C.normalizeEnum)(i.color)&&"inside"!==n.position?t:i.color,{alignment:n.alignment,format:n.format,argumentFormat:n.argumentFormat,customizeText:(0,y.isFunction)(n.customizeText)?n.customizeText:void 0,attributes:{font:i},visible:0!==i.size&&n.visible,showForZeroValues:n.showForZeroValues,horizontalOffset:n.horizontalOffset,verticalOffset:n.verticalOffset,radialOffset:n.radialOffset,background:r,position:n.position,connector:s,rotationAngle:n.rotationAngle,wordWrap:n.wordWrap,textOverflow:n.textOverflow,cssClass:n.cssClass}}function c(e,t){e.fullState|=z,e.applyView(t)}function d(e,t){e.fullState&=~z,e.applyView(t),e.releaseHoverState()}function h(e,t){e.fullState|=H,e.applyView(t)}function f(e,t){e.fullState&=~H,e.applyView(t)}function p(e,t){var n=(0,b.extend)({},e,t);return n.border=(0,b.extend)({},e&&e.border,t&&t.border),n}function g(e,t){var n=p(e,t);return n.image=(0,b.extend)(!0,{},e.image,t.image),n.selectionStyle=p(e.selectionStyle,t.selectionStyle),n.hoverStyle=p(e.hoverStyle,t.hoverStyle),n}function _(e,t){var n=this;n.fullState=0,n._extGroups=e,n._renderer=e.renderer,n._group=e.renderer.g().attr({"class":"dxc-series"}),n._eventTrigger=e.eventTrigger,n._eventPipe=e.eventPipe,n._incidentOccurred=e.incidentOccurred,n._legendCallback=k.noop,n.updateOptions(t,e)}function m(e){return e.data}function v(e,t){return t&&"logarithmic"===e&&t.getOptions().allowNegatives===!1?function(e){return e>0}:function(){return!0}}t.Series=_,t.mixins=void 0;var y=n(1),b=n(0),x=n(3),w=n(919),C=n(14),k=n(4),S=r(n(151)),D=r(n(230)),I=a(n(132)),T=a(n(231)),O=a(n(202)),E=a(n(173)),A=n(924),P=n(925),R=a(n(926)),M=a(n(927)),B=a(n(928)),V={},F=S.default.states,L="discrete",H=F.selectedMark,z=F.hoverMark,N=F.hover,j=F.normal,W=F.selection,G=F.applySelected,q=F.applyHover,$=F.resetItem,K="none",U="includepoints",Y="nearestpoint",Z="seriesSelectionChanged",X="pointSelectionChanged",Q="seriesHoverChanged",J="pointHoverChanged",ee="allseriespoints",te="allargumentpoints",ne="pointHover",ie="clearPointHover",oe="seriesSelect",ae="pointSelect",re="pointDeselect",se=function(){return{arg:{},val:{}}};V.mixins={chart:{},pie:{},polar:{}},V.mixins.chart.scatter=I.chart,V.mixins.polar.scatter=I.polar,(0,b.extend)(V.mixins.pie,R),(0,b.extend)(V.mixins.chart,T.chart,O.chart,E.chart,A.chart,P.chart,M,B.chart),(0,b.extend)(V.mixins.polar,T.polar,O.polar,E.polar,B.polar),_.prototype={constructor:_,_createLegendState:k.noop,getLegendStyles:function(){return this._styles.legendStyles},_createStyles:function(e){var t=this,n=e.mainSeriesColor;t._styles={normal:t._parseStyle(e,n,n),hover:t._parseStyle(e.hoverStyle||{},n,n),selection:t._parseStyle(e.selectionStyle||{},n,n),legendStyles:{normal:t._createLegendState(e,n),hover:t._createLegendState(e.hoverStyle||{},n),selection:t._createLegendState(e.selectionStyle||{},n)}}},setClippingParams:function(e,t,n){var i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3];this._paneClipRectID=e,this._widePaneClipRectID=t,this._forceClipping=n,this._clipLabels=i},applyClip:function(){this._group.attr({"clip-path":this._paneClipRectID})},resetClip:function(){this._group.attr({"clip-path":null})},getTagField:function(){return this._options.tagField||"tag"},getValueFields:k.noop,getSizeField:k.noop,getArgumentField:k.noop,getPoints:function(){return this._points},getPointsInViewPort:function(){return D.default.getPointsInViewPort(this)},_createPoint:function(e,t,n){e.index=t;var i=this,o=i.pointsByArgument,a=i._getCreatingPointOptions(e),r=e.argument.valueOf(),s=n;s?s.update(e,a):(s=new w.Point(i,e,a),i.isSelected()&&l(i.lastSelectionMode)&&s.setView(W));var u=o[r];return u?u.push(s):o[r]=[s],s.hasValue()&&i.customizePoint(s,e),s},getRangeData:function(){return this._visible?this._getRangeData():se()},getArgumentRange:function(){return this._visible?D.default.getArgumentRange(this):se()},getViewport:function(){return D.default.getViewport(this)},_deleteGroup:function(e){var t=this[e];t&&(t.dispose(),this[e]=null)},updateOptions:function(e,t){var n=this,i=e.widgetType,o=n.type,a=e.type;return n.type=a&&(0,C.normalizeEnum)(a.toString()),!n._checkType(i)||n._checkPolarBarType(i,e)?(n.dispose(),void(n.isUpdated=!1)):(o!==n.type?(n._firstDrawing=!0,n._resetType(o,i),n._setType(n.type,i)):n._defineDrawingState(),n._options=e,n._pointOptions=null,n.name=e.name,n.pane=e.pane,n.tag=e.tag,t&&(n._seriesModes=t.commonSeriesModes||n._seriesModes,n._valueAxis=t.valueAxis||n._valueAxis,n.axis=n._valueAxis&&n._valueAxis.name,n._argumentAxis=t.argumentAxis||n._argumentAxis),n._createStyles(e),n._stackName=null,n._updateOptions(e),n._visible=e.visible,n.isUpdated=!0,n.stack=e.stack,n.barOverlapGroup=e.barOverlapGroup,n._createGroups(),void(n._processEmptyValue=e.ignoreEmptyPoints?function(e){return null===e?void 0:e}:function(e){return e}))},_defineDrawingState:function(){this._firstDrawing=!0},_disposePoints:function(e){(0,x.each)(e||[],function(e,t){t.dispose()})},updateDataType:function(e){var t=this;return t.argumentType=e.argumentType,t.valueType=e.valueType,t.argumentAxisType=e.argumentAxisType,t.valueAxisType=e.valueAxisType,t.showZero=e.showZero,this._argumentChecker=v(e.argumentAxisType,t.getArgumentAxis()),this._valueChecker=v(e.valueAxisType,t.getValueAxis()),t},_argumentChecker:function(){return!0},_valueChecker:function(){return!0},getOptions:function(){return this._options},_getOldPoint:function(e,t,n){var i=e.argument&&e.argument.valueOf(),o=(t[i]||[])[0];return o&&t[i].splice(0,1),o},updateData:function(e){var t=this,n=t._options,i=n.nameField;e=e||[],e.length&&(t._canRenderCompleteHandle=!0);var o=this._getPointDataSelector(),a=0;t._data=e.reduce(function(e,t,r){var s=o(t);return(0,y.isDefined)(s.argument)?i&&t[i]!==n.nameFieldValue||(s.index=r,e.push(s)):a++,e},[]),a&&a===e.length&&t._incidentOccurred("W2002",[t.name,t.getArgumentField()]),t._endUpdateData()},_getData:function(){var e=this._data||[];return this.useAggregation()&&(e=this._resample(this.getArgumentAxis().getAggregationInfo(this._useAllAggregatedPoints,this.argumentAxisType!==L?this.getArgumentRange():{}),e)),e},useAggregation:function(){var e=this.getOptions().aggregation;return e&&e.enabled},autoHidePointMarkersEnabled:k.noop,usePointsToDefineAutoHiding:k.noop,createPoints:function(e){this._normalizeUsingAllAggregatedPoints(e),this._createPoints()},_normalizeUsingAllAggregatedPoints:function(e){this._useAllAggregatedPoints=this.useAggregation()&&(this.argumentAxisType===L||(this._data||[]).length>1&&!!e)},_createPoints:function(){var e=this,t=e.pointsByArgument||{},n=e._getData();e.pointsByArgument={},e._calculateErrorBars(n);var i={},o=n.reduce(function(n,o){if(e._checkData(o,i)){var a=n.length,r=e._getOldPoint(o,t,a),s=e._createPoint(o,a,r);n.push(s)}return n},[]);for(var a in i)i[a]===n.length&&e._incidentOccurred("W2002",[e.name,a]);Object.keys(t).forEach(function(n){return e._disposePoints(t[n])}),e._points=o},_removeOldSegments:function(){var e=this,t=e._segments.length;(0,x.each)(e._graphics.splice(t,e._graphics.length)||[],function(t,n){e._removeElement(n)}),e._trackers&&(0,x.each)(e._trackers.splice(t,e._trackers.length)||[],function(e,t){t.remove()})},_drawElements:function(e,t,n){var i=this,o=i._points||[],a=o[0]&&o[0].hasValue()&&i._options.closed,r={markers:i._markersGroup,errorBars:i._errorBarGroup};i._drawnPoints=[],i._graphics=i._graphics||[],i._segments=[];var s=o.reduce(function(o,a){var s=o[o.length-1];return a.translated&&!n||(a.translate(),!n&&a.setDefaultCoords()),a.hasValue()&&a.hasCoords()?(n&&i._drawPoint({point:a,groups:r,hasAnimation:e,firstDrawing:t}),s.push(a)):a.hasValue()?a.setInvisibility():s.length&&o.push([]),o},[[]]);s.forEach(function(t,n){t.length&&i._drawSegment(t,e,n,a&&n===this.length-1)},s),i._firstDrawing=!o.length,i._removeOldSegments(),e&&i._animate(t)},draw:function(e,t,n){var i=this,o=i._firstDrawing;return i._legendCallback=n||i._legendCallback,i._visible?(i._appendInGroup(),i._applyVisibleArea(),i._setGroupsSettings(e,o),!o&&!i._resetApplyingAnimation&&i._drawElements(!1,o,!1),i._drawElements(e,o,!0),t&&i.hideLabels(),i.isSelected()?i._changeStyle(i.lastSelectionMode,void 0,!0):i.isHovered()?i._changeStyle(i.lastHoverMode,void 0,!0):i._applyStyle(i._styles.normal),void(i._resetApplyingAnimation=!1)):(e=!1,void i._group.remove())},_setLabelGroupSettings:function(e){var t={"class":"dxc-labels","pointer-events":"none"};this._clipLabels&&this._applyElementsClipRect(t),this._applyClearingSettings(t),e&&(t.opacity=.001),this._labelsGroup.attr(t).append(this._extGroups.labelsGroup)},_checkType:function(e){return!!V.mixins[e][this.type]},_checkPolarBarType:function(e,t){return"polar"===e&&t.spiderWidget&&this.type.indexOf("bar")!==-1},_resetType:function(e,t){var n,i;if(e){i=V.mixins[t][e];for(n in i)delete this[n]}},_setType:function(e,t){var n,i=V.mixins[t][e];for(n in i)this[n]=i[n]},_setPointsView:function(e,t){this.getPoints().forEach(function(n){t!==n&&n.setView(e)})},_resetPointsView:function(e,t){this.getPoints().forEach(function(n){t!==n&&n.resetView(e)})},_resetNearestPoint:function(){var e=this;e._nearestPoint&&null!==e._nearestPoint.series&&e._nearestPoint.resetView(N),e._nearestPoint=null},_setSelectedState:function(e){var t=this;t.lastSelectionMode=(0,C.normalizeEnum)(e||t._options.selectionMode),t.fullState=t.fullState|H,t._resetNearestPoint(),t._changeStyle(t.lastSelectionMode),t.lastSelectionMode!==K&&t.isHovered()&&l(t.lastHoverMode)&&t._resetPointsView(N)},_releaseSelectedState:function(){var e=this;e.fullState=e.fullState&~H,e._changeStyle(e.lastSelectionMode,W),e.lastSelectionMode!==K&&e.isHovered()&&l(e.lastHoverMode)&&e._setPointsView(N)},isFullStackedSeries:function(){return 0===this.type.indexOf("fullstacked")},isStackedSeries:function(){return 0===this.type.indexOf("stacked")},resetApplyingAnimation:function(e){this._resetApplyingAnimation=!0,e&&(this._firstDrawing=!0)},isFinancialSeries:function(){return"stock"===this.type||"candlestick"===this.type},_canChangeView:function(){return!this.isSelected()&&(0,C.normalizeEnum)(this._options.hoverMode)!==K},_changeStyle:function(e,t,n){var i=this,o=i.fullState,a=[j,N,W,W];"none"===i.lastHoverMode&&(o&=~z),"none"===i.lastSelectionMode&&(o&=~H),l(e)&&!n&&(t?i._resetPointsView(t):i._setPointsView(a[o])),i._legendCallback([$,q,G,G][o]),i._applyStyle(i._styles[a[o]])},updateHover:function(e,t){var n=this,i=n._nearestPoint,o=n.isHovered()&&n.lastHoverMode===Y&&n.getNeighborPoint(e,t);o===i||n.isSelected()&&n.lastSelectionMode!==K||(n._resetNearestPoint(),o&&(o.setView(N),n._nearestPoint=o))},_getMainAxisName:function(){return this._options.rotated?"X":"Y"},areLabelsVisible:function(){return!(0,y.isDefined)(this._options.maxLabelCount)||this._points.length<=this._options.maxLabelCount},getLabelVisibility:function(){return this.areLabelsVisible()&&this._options.label&&this._options.label.visible},customizePoint:function e(t,n){var i,o,a,r,s,l,u=this,c=u._options,e=c.customizePoint,d=c.customizeLabel;d&&d.call&&(i=(0,b.extend)({seriesName:u.name},n),i.series=u,a=d.call(i,i),s=a&&!(0,y.isEmptyObject)(a),a=s?(0,b.extend)(!0,{},c.label,a):null),e&&e.call&&(i=i||(0,b.extend)({seriesName:u.name},n),i.series=u,r=e.call(i,i),l=r&&!(0,y.isEmptyObject)(r)),(s||l)&&(o=u._parsePointOptions(u._preparePointOptions(r),a||c.label,n,t),o.styles.useLabelCustomOptions=s,o.styles.usePointCustomOptions=l,t.updateOptions(o))},show:function(){this._visible||this._changeVisibility(!0)},hide:function(){this._visible&&this._changeVisibility(!1)},_changeVisibility:function(e){var t=this;t._visible=t._options.visible=e,t._updatePointsVisibility(),t.hidePointTooltip(),t._options.visibilityChanged(t)},_updatePointsVisibility:k.noop,hideLabels:function(){(0,x.each)(this._points,function(e,t){t._label.draw(!1)})},_parsePointOptions:function(e,t,n,i){var o=this,a=o._options,r=o._createPointStyles(e,n,i),s=(0,b.extend)({},e,{type:a.type,rotated:a.rotated,styles:r,widgetType:a.widgetType,visibilityChanged:a.visibilityChanged});return s.label=u(t,r.normal.fill),o.areErrorBarsVisible()&&(s.errorBars=a.valueErrorBar),s},_preparePointOptions:function(e){var t=this._getOptionsForPoint();return e?g(t,e):t},_getMarkerGroupOptions:function(){return(0,b.extend)(!1,{},this._getOptionsForPoint(),{hoverStyle:{},selectionStyle:{}})},_getAggregationMethod:function(e,t){var n,i=this.getOptions().aggregation,o=(0,C.normalizeEnum)(i.method),a="custom"===o&&i.calculate;return n=e&&!t?function(e){var t=e.data;return t[0]}:this._aggregators[o]||this._aggregators[this._defaultAggregator],a||n},_resample:function(e,t){var n=e.interval,i=e.ticks,o=e.aggregateByCategory,a=this,r=a.argumentAxisType===L||a.valueAxisType===L,s=0,l=this._getPointDataSelector(),u=a.getOptions(),c=function(e,t,n){if(t){var i=function(t){var i=t&&l(t,u);i&&a._checkData(i)&&(i.aggregationInfo=n,e.push(i))};t.length?t.forEach(i):i(t)}},d=this._getAggregationMethod(r,o);if(r){if(o){var h=this.getArgumentAxis().getTranslator().getBusinessRange().categories,f=h.reduce(function(e,t){return e[t.valueOf()]=[],e},{});return t.forEach(function(e){f[e.argument.valueOf()].push(e)}),h.reduce(function(e,t){return c(e,d({aggregationInterval:null,intervalStart:t,intervalEnd:t,data:f[t.valueOf()].map(m)},a)),e},[])}return t.reduce(function(e,t,i,o){if(e[1].push(t),i===o.length-1||(i+1)%n===0){var r=e[1],s={aggregationInterval:n,data:r.map(m)};c(e[0],d(s,a)),e[1]=[]}return e},[[],[]])[0]}for(var p=[],g=1;g<i.length;g++){for(var _=i[g],v=i[g-1],y=[];t[s]&&t[s].argument<_;)t[s].argument>=v&&y.push(t[s]),s++;var b={intervalStart:v,intervalEnd:_,aggregationInterval:n,data:y.map(m)};c(p,d(b,a),b)}return a._endUpdateData(),p},canRenderCompleteHandle:function(){var e=this._canRenderCompleteHandle;return delete this._canRenderCompleteHandle,!!e},isHovered:function(){return!!(1&this.fullState)},isSelected:function(){return!!(2&this.fullState)},isVisible:function(){return this._visible},getAllPoints:function(){return this._createAllAggregatedPoints(),(this._points||[]).slice()},getPointByPos:function(e){return this._createAllAggregatedPoints(),(this._points||[])[e]},getVisiblePoints:function(){return(this._drawnPoints||[]).slice()},selectPoint:function(e){e.isSelected()||(h(e,this._legendCallback),this._eventPipe({action:ae,target:e}),this._eventTrigger(X,{target:e}))},deselectPoint:function(e){e.isSelected()&&(f(e,this._legendCallback),this._eventPipe({action:re,target:e}),this._eventTrigger(X,{target:e}))},hover:function(e){var t=this,n=t._eventTrigger;t.isHovered()||(t.lastHoverMode=(0,C.normalizeEnum)(e||t._options.hoverMode),t.fullState=t.fullState|z,t._changeStyle(t.lastHoverMode,void 0,t.isSelected()&&t.lastSelectionMode!==K),n(Q,{target:t}))},clearHover:function(){var e=this,t=e._eventTrigger;e.isHovered()&&(e._resetNearestPoint(),e.fullState=e.fullState&~z,e._changeStyle(e.lastHoverMode,N,e.isSelected()&&e.lastSelectionMode!==K),t(Q,{target:e}))},hoverPoint:function(e){var t=this;e.isHovered()||(e.clearHover(),c(e,t._legendCallback),t._canChangeView()&&t._applyStyle(t._styles.hover),t._eventPipe({action:ne,target:e}),t._eventTrigger(J,{target:e}))},clearPointHover:function(){var e=this;e.getPoints().some(function(t){return!!t.isHovered()&&(d(t,e._legendCallback),e._canChangeView()&&e._applyStyle(e._styles.normal),e._eventPipe({action:ie,target:t}),e._eventTrigger(J,{target:t}),!0)})},showPointTooltip:function(e){s(this._extGroups.seriesGroup,"showpointtooltip",e)},hidePointTooltip:function(e){s(this._extGroups.seriesGroup,"hidepointtooltip",e)},select:function(){var e=this;e.isSelected()||(e._setSelectedState(e._options.selectionMode),e._eventPipe({action:oe,target:e}),e._group.toForeground(),e._eventTrigger(Z,{target:e}))},clearSelection:function(){var e=this;e.isSelected()&&(e._releaseSelectedState(),e._eventTrigger(Z,{target:e}))},getPointsByArg:function(e,t){var n=this,i=e.valueOf(),o=n.pointsByArgument[i];return o||t||!n._createAllAggregatedPoints()||(o=n.pointsByArgument[i]),o||[]},_createAllAggregatedPoints:function(){return!(!this.useAggregation()||this._useAllAggregatedPoints)&&(this.createPoints(!0),!0)},getPointsByKeys:function(e){return this.getPointsByArg(e)},notify:function(e){var t=this,n=e.action,i=t._seriesModes,o=e.target,a=o.getOptions(),r=(0,C.normalizeEnum)(a.hoverMode),s=(0,C.normalizeEnum)(a.selectionMode);n===ne?t._hoverPointHandler(o,r,e.notifyLegend):n===ie?t._clearPointHoverHandler(o,r,e.notifyLegend):n===oe?o!==t&&"single"===i.seriesSelectionMode&&t.clearSelection():n===ae?("single"===i.pointSelectionMode&&t.getPoints().some(function(e){return!(e===o||!e.isSelected())&&(t.deselectPoint(e),!0)}),t._selectPointHandler(o,s)):n===re&&t._deselectPointHandler(o,s)},_selectPointHandler:function(e,t){var n=this;t===ee?e.series===n&&n._setPointsView(W,e):t===te&&n.getPointsByKeys(e.argument,e.argumentIndex).forEach(function(t){t!==e&&t.setView(W)})},_deselectPointHandler:function(e,t){t===ee?e.series===this&&this._resetPointsView(W,e):t===te&&this.getPointsByKeys(e.argument,e.argumentIndex).forEach(function(t){t!==e&&t.resetView(W)})},_hoverPointHandler:function(e,t,n){var i=this;e.series!==i&&t===te?(i.getPointsByKeys(e.argument,e.argumentIndex).forEach(function(e){e.setView(N)}),n&&i._legendCallback(e)):t===ee&&e.series===i&&i._setPointsView(N,e)},_clearPointHoverHandler:function(e,t,n){var i=this;t===te?(e.series!==i&&i.getPointsByKeys(e.argument,e.argumentIndex).forEach(function(e){e.resetView(N)}),n&&i._legendCallback(e)):t===ee&&e.series===i&&i._resetPointsView(N,e)},_deletePoints:function(){var e=this;e._disposePoints(e._points),e._points=e._drawnPoints=null},_deleteTrackers:function(){var e=this;(0,x.each)(e._trackers||[],function(e,t){t.remove()}),e._trackersGroup&&e._trackersGroup.dispose(),e._trackers=e._trackersGroup=null},dispose:function(){var e=this;e._deletePoints(),e._group.dispose(),e._labelsGroup&&e._labelsGroup.dispose(),e._errorBarGroup&&e._errorBarGroup.dispose(),e._deleteTrackers(),e._group=e._extGroups=e._markersGroup=e._elementsGroup=e._bordersGroup=e._labelsGroup=e._errorBarGroup=e._graphics=e._rangeData=e._renderer=e._styles=e._options=e._pointOptions=e._drawnPoints=e.pointsByArgument=e._segments=e._prevSeries=null;
},correctPosition:k.noop,drawTrackers:k.noop,getNeighborPoint:k.noop,areErrorBarsVisible:k.noop,getMarginOptions:function(){return this._patchMarginOptions({percentStick:this.isFullStackedSeries()})},getColor:function(){return this.getLegendStyles().normal.fill},getOpacity:function(){return this._options.opacity},getStackName:function(){return this._stackName},getBarOverlapGroup:function(){return this._options.barOverlapGroup},getPointByCoord:function(e,t){var n=this.getNeighborPoint(e,t);return null!==n&&void 0!==n&&n.coordsIn(e,t)?n:null},getValueAxis:function(){return this._valueAxis},getArgumentAxis:function(){return this._argumentAxis},getMarkersGroup:function(){return this._markersGroup},getRenderer:function(){return this._renderer}};var le=V.mixins;t.mixins=le},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i=1/0;return(0,p.each)(t,function(t,o){var a=e[0]-o[0],r=e[1]-o[1],s=a*a+r*r;s<i&&(i=s,n=o)}),[y(n[0]),y(n[1])]}function a(e,t,n){return(t-e[0+n])/(e[2+n]-e[0+n])*(e[3-n]-e[1-n])+e[1-n]}function r(e){return void 0!==e.angle&&D||void 0!==e.r&&S||k}function s(e,t){e[t]&&e[t].dispose(),e[t]=null}function l(e){return e&&(e.fill&&"none"!==e.fill||e["stroke-width"]>0&&e.stroke&&"none"!==e.stroke)}function u(e){return e&&e["stroke-width"]>0&&e.stroke&&"none"!==e.stroke}function c(e,t){var n=t.format;return e.valueText=_(e.value,n),e.argumentText=_(e.argument,t.argumentFormat),void 0!==e.percent&&(e.percentText=_(e.percent,{type:"percent",precision:n&&n.percentPrecision})),void 0!==e.total&&(e.totalText=_(e.total,n)),void 0!==e.openValue&&(e.openValueText=_(e.openValue,n)),void 0!==e.closeValue&&(e.closeValueText=_(e.closeValue,n)),void 0!==e.lowValue&&(e.lowValueText=_(e.lowValue,n)),void 0!==e.highValue&&(e.highValueText=_(e.highValue,n)),void 0!==e.reductionValue&&(e.reductionValueText=_(e.reductionValue,n)),t.customizeText?t.customizeText.call(e,e):e.valueText}function d(e){this._renderer=e.renderer,this._container=e.labelsGroup,this._point=e.point,this._strategy=e.strategy,this._rowCount=1}t.Label=d;var h=i(n(79)),f=n(14),p=n(3),g=n(0),_=h.default.format,m=Math,v=m.round,y=m.floor,b=m.abs,x=12,w=8,C=4,k={isLabelInside:function(e,t){var n=e.x+e.width/2,i=e.y+e.height/2;return t.x<=n&&n<=t.x+t.width&&t.y<=i&&i<=t.y+t.height},prepareLabelPoints:function(e,t,n,i,o){var r,s=t.x,l=s+t.width/2,u=s+t.width-1,c=t.y,d=c+t.height/2,h=c+t.height-1,f=b(i)%90===0;return r=o[0]>s&&o[0]<u?f?[[o[0],b(o[1]-c)<b(o[1]-h)?c:h]]:[[o[0],a([s,c,u,h],o[0],0)]]:o[1]>c&&o[1]<h?f?[[b(o[0]-s)<b(o[0]-u)?s:u,o[1]]]:[[a([s,c,u,h],o[1],1),o[1]]]:f?[[s,c],[n?s:l,n?d:c],[u,c],[s,h],[n?u:l,n?d:h],[u,h]]:[[l,d]]},isHorizontal:function(e,t){return e.x>t.x+t.width||e.x+e.width<t.x},getFigureCenter:function(e){return[y(e.x+e.width/2),y(e.y+e.height/2)]},findFigurePoint:function(e,t){var n=k.getFigureCenter(e),i=o(t,[[e.x,n[1]],[n[0],e.y+e.height],[e.x+e.width,n[1]],[n[0],e.y]]);return i},adjustPoints:function(e){var t=b(e[1]-e[3])<=1,n=b(e[0]-e[2])<=1;return n&&(e[0]=e[2]),t&&(e[1]=e[3]),e}},S={isLabelInside:function(){return!1},prepareLabelPoints:k.prepareLabelPoints,isHorizontal:function(e,t){return e.x>t.x+t.r||e.x+e.width<t.x-t.r},getFigureCenter:function(e){return[e.x,e.y]},findFigurePoint:function(e,t){var n=Math.atan2(e.y-t[1],t[0]-e.x);return[v(e.x+e.r*Math.cos(n)),v(e.y-e.r*Math.sin(n))]},adjustPoints:k.adjustPoints},D={isLabelInside:function(e,t,n){return!n},prepareLabelPoints:function(e,t,n,i){var o=e.x,a=o+e.width,r=o+v(e.width/2),s=e.y,l=s+e.height,u=s+v(e.height/2),c=[[[o,s],[a,s]],[[a,s],[a,l]],[[a,l],[o,l]],[[o,l],[o,s]]],d=(0,f.getCosAndSin)(i);return c=0===i?n?[[o,u],[a,u]]:[[r,s],[r,l]]:c.map(function(e){return e.map(function(e){return[v((e[0]-r)*d.cos+(e[1]-u)*d.sin+r),v(-(e[0]-r)*d.sin+(e[1]-u)*d.cos+u)]})}).reduce(function(e,t){var i=t[0][0],o=t[0][1],a=t[1][0],s=t[1][1];return n?(o>=u&&u>=s||o<=u&&u<=s)&&e.push([(u-o)*(a-i)/(s-o)+i,u]):(i>=r&&r>=a||i<=r&&r<=a)&&e.push([r,(r-i)*(s-o)/(a-i)+o]),e},[])},isHorizontal:function(e,t){return e.x>t.x||t.x>e.x+e.width},getFigureCenter:S.getFigureCenter,findFigurePoint:function(e,t,n){if(!n)return[e.x,e.y];var i=t[0],o=v(e.x+(e.y-t[1])/Math.tan((0,f.degreesToRadians)(e.angle))),a=[e.x,e.y,o,t[1]];return e.x<=o&&o<=i||i<=o&&o<=e.x||(b(e.x-i)<x?a=[e.x,e.y]:e.x<=i?a[2]=e.x+x:a[2]=e.x-x),a},adjustPoints:function(e){return e}};d.prototype={constructor:d,setColor:function(e){this._color=e},setOptions:function(e){this._options=e},setData:function(e){this._data=e},setDataField:function(e,t){this._data=this._data||{},this._data[e]=t},getData:function(){return this._data},setFigureToDrawConnector:function(e){this._figure=e},dispose:function(){var e=this;s(e,"_group"),e._data=e._options=e._textContent=e._visible=e._insideGroup=e._text=e._background=e._connector=e._figure=null},_setVisibility:function(e,t){this._group&&this._group.attr({visibility:e}),this._visible=t},isVisible:function(){return this._visible},hide:function(e){this._holdVisibility=!!e,this._hide()},_hide:function(){this._setVisibility("hidden",!1)},show:function(e){var t=!this._drawn;this._point.hasValue()&&(this._holdVisibility=!!e,this._show(),t&&this._point.correctLabelPosition(this))},_show:function(){var e=this,t=e._renderer,n=e._container,i=e._options||{},o=e._textContent=c(e._data,e._options)||null;o?(e._group||(e._group=t.g().append(n),e._insideGroup=t.g().append(e._group),e._text=t.text("",0,0).append(e._insideGroup)),e._text.css(i.attributes?(0,f.patchFontOptions)(i.attributes.font):{}),l(i.background)?(e._background=e._background||t.rect().append(e._insideGroup).toBackground(),e._background.attr(i.background),e._color&&e._background.attr({fill:e._color})):s(e,"_background"),u(i.connector)?(e._connector=e._connector||t.path([],"line").sharp().append(e._group).toBackground(),e._connector.attr(i.connector),e._color&&e._connector.attr({stroke:e._color})):s(e,"_connector"),e._text.attr({text:o,align:i.textAlignment,"class":i.cssClass}),e._updateBackground(e._text.getBBox()),e._setVisibility("visible",!0),e._drawn=!0):e._hide()},_getLabelVisibility:function(e){return this._holdVisibility?this.isVisible():e},draw:function(e){return this._getLabelVisibility(e)?(this._show(),this._point&&this._point.correctLabelPosition(this)):(this._drawn=!1,this._hide()),this},_updateBackground:function(e){var t=this;t._background&&(e.x-=w,e.y-=C,e.width+=2*w,e.height+=2*C,t._background.attr(e)),t._bBoxWithoutRotation=(0,g.extend)({},e);var n=t._options.rotationAngle||0;t._insideGroup.rotate(n,e.x+e.width/2,e.y+e.height/2),e=(0,f.rotateBBox)(e,[e.x+e.width/2,e.y+e.height/2],-n),t._bBox=e},getFigureCenter:function(){var e=this._figure,t=this._strategy||r(e);return t.getFigureCenter(e)},_getConnectorPoints:function(){var e,t,n=this,i=n._figure,a=n._options,s=n._strategy||r(i),l=n._shiftBBox(n._bBoxWithoutRotation),u=n.getBoundingRect(),c=[];if(!s.isLabelInside(l,i,"inside"!==a.position)){t=s.isHorizontal(l,i);var d=n.getFigureCenter();c=s.prepareLabelPoints(l,u,t,-a.rotationAngle||0,d),e=o(d,c),c=s.findFigurePoint(i,e,t),c=c.concat(e)}return s.adjustPoints(c)},fit:function(e){var t=this._background?2*w:0,n=!1;if(this._text){var i=this._text.setMaxSize(e-t,void 0,this._options),o=i.rowCount;0===o&&(o=1),o!==this._rowCount&&(n=!0,this._rowCount=o),i.textIsEmpty&&s(this,"_background")}return this._updateBackground(this._text.getBBox()),n},resetEllipsis:function(){this._text&&this._text.restoreText(),this._updateBackground(this._text.getBBox())},setTrackerData:function(e){this._text.data({"chart-data-point":e}),this._background&&this._background.data({"chart-data-point":e})},hideInsideLabel:function(e){return this._point.hideInsideLabel(this,e)},getPoint:function(){return this._point},shift:function(e,t){var n=this;return n._textContent&&(n._insideGroup.attr({translateX:n._x=v(e-n._bBox.x),translateY:n._y=v(t-n._bBox.y)}),n._connector&&n._connector.attr({points:n._getConnectorPoints()})),n},getBoundingRect:function(){return this._shiftBBox(this._bBox)},_shiftBBox:function(e){return this._textContent?{x:e.x+this._x,y:e.y+this._y,width:e.width,height:e.height}:{}},getLayoutOptions:function(){var e=this._options;return{alignment:e.alignment,background:l(e.background),horizontalOffset:e.horizontalOffset,verticalOffset:e.verticalOffset,radialOffset:e.radialOffset,position:e.position,connectorOffset:(u(e.connector)?x:0)+(l(e.background)?w:0)}}}},function(e,t,n){t.default=void 0;var i=n(14),o={logarithmic:"logarithmic",discrete:"discrete",numeric:"numeric",left:"left",right:"right",top:"top",bottom:"bottom",center:"center",horizontal:"horizontal",vertical:"vertical",convertTicksToValues:function(e){return(0,i.map)(e||[],function(e){return e.value})},validateOverlappingMode:function(e){return"ignore"===e||"none"===e?e:"hide"},getTicksCountInRange:function(e,t,n){var i=1;if(e.length>1)for(;i<e.length&&!(Math.abs(e[i].coords[t]-e[0].coords[t])>=n);i++);return i},areLabelsOverlap:function(e,t,n,i){var o=e.x>t.x,a=e.y>t.y,r=e.x,s=t.x,l=e.width,u=t.width;"left"===i?(r+=l/2,s+=u/2):"right"===i&&(r-=l/2,s-=u/2);var c=o?s+u+n>r:r+l+n>s,d=a?t.y+t.height>e.y:e.y+e.height>t.y;return c&&d}};t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i,o,a,r="logarithmic"===t.axisType?function(e){return(0,p.getLogExt)(e,t.base)}:function(e){return e},s=[],l=e.length,u=0;for(a=0;a<l;a++)n=e[a],i=r(n.from),o=r(n.to),u+=o-i,s.push({trFrom:i,trTo:o,from:n.from,to:n.to,length:u,cumulativeWidth:n.cumulativeWidth});return s}function a(e){var t=e.min,n=e.max,i=e.minVisible,o=e.maxVisible,a="logarithmic"===e.axisType;return a&&(o=(0,p.getLogExt)(o,e.base,e.allowNegatives,e.linearThreshold),i=(0,p.getLogExt)(i,e.base,e.allowNegatives,e.linearThreshold),t=(0,p.getLogExt)(t,e.base,e.allowNegatives,e.linearThreshold),n=(0,p.getLogExt)(n,e.base,e.allowNegatives,e.linearThreshold)),{base:e.base,rangeMin:t,rangeMax:n,rangeMinVisible:i,rangeMaxVisible:o}}function r(e){return{isStartSide:e?function(e,t,n,i){return e<=t[t.length-1][i]}:function(e,t,n,i){return e<t[0][n]},isEndSide:e?function(e,t,n,i){return e>t[0][n]}:function(e,t,n,i){return e>=t[t.length-1][i]},isInBreak:e?function(e,t,n,i){return e>t[i]&&e<=t[n]}:function(e,t,n,i){return e>=t[n]&&e<t[i]},isBetweenBreaks:e?function(e,t,n,i,o){return e>=t[o]&&e<n[i]}:function(e,t,n,i,o){return e<t[i]&&e>=n[o]},getLength:e?function(e,t){return t.length-e.length}:function(e){return e.length},getBreaksSize:e?function(e,t){return t.cumulativeWidth-e.cumulativeWidth}:function(e){return e.cumulativeWidth}}}t.Translator2D=void 0;var s=n(0),l=n(3),u=n(134),c=i(n(931)),d=i(n(932)),h=i(n(933)),f=i(n(934)),p=n(14),g=n(1),_=n(29),m=i(n(20)),v=Math.abs,y=["width","height","left","top","bottom","right"],b={to:function(e){var t=this._canvasOptions.startPoint+(this._options.conversionValue?e:Math.round(e));return t>this._canvasOptions.endPoint?this._canvasOptions.endPoint:t},from:function(e){return e-this._canvasOptions.startPoint}},x=function(e){return(0,l.each)(y,function(t,n){e[n]=parseInt(e[n])||0}),e},w=function(e){var t={};return e.forEach(function(e,n){t[e.valueOf()]=n}),t},C=function(e){function t(t,n){!(0,g.isDefined)(e[t])&&(0,g.isDefined)(e[n])&&(e[t]=e[n])}return e instanceof u.Range||(e=new u.Range(e)),t("minVisible","min"),t("maxVisible","max"),e},k=function(e,t,n){this.update(e,t,n)};t.Translator2D=k,k.prototype={constructor:k,reinit:function(){var e=this,t=e._options,n=e._businessRange,i=n.categories||[],o={},a=e._prepareCanvasOptions(),l=(0,p.getCategoriesInfo)(i,n.minVisible,n.maxVisible).categories,u=l.length;if(n.isEmpty())o=b;else switch(n.axisType){case"logarithmic":o=f.default;break;case"semidiscrete":o=d.default,a.ratioOfCanvasRange=a.canvasLength/(m.default.addInterval(a.rangeMaxVisible,t.interval)-a.rangeMinVisible);break;case"discrete":o=c.default,e._categories=i,a.interval=e._getDiscreteInterval(t.addSpiderCategory?u+1:u,a),e._categoriesToPoints=w(i,a.invert),u&&(a.startPointIndex=e._categoriesToPoints[l[0].valueOf()],e.visibleCategories=l);break;default:"datetime"===n.dataType&&(o=h.default)}(e._oldMethods||[]).forEach(function(t){delete e[t]}),e._oldMethods=Object.keys(o),(0,s.extend)(e,o),e._conversionValue=t.conversionValue?function(e){return e}:function(e){return Math.round(e)},e.sc={},e._checkingMethodsAboutBreaks=[r(!1),r(e.isInverted())],e._translateBreaks(),e._calculateSpecialValues()},_translateBreaks:function(){var e,t,n,i,o=this._breaks,a=this._options.breaksSize;if(void 0!==o)for(e=0,i=o.length;e<i;e++)t=o[e],n=this.translate(t.to),t.end=n,t.start=t.gapSize?n:this.isInverted()?n+a:n-a},_checkValueAboutBreaks:function(e,t,n,i,o){var a,r,s,l,u={length:0,breaksSize:void 0,inBreak:!1},c=e[e.length-1];if(o.isStartSide(t,e,n,i))return u;if(o.isEndSide(t,e,n,i))return{length:c.length,breaksSize:c.cumulativeWidth,inBreak:!1};for(a=0,r=e.length;a<r;a++){if(s=e[a],l=e[a-1],o.isInBreak(t,s,n,i)){u.inBreak=!0,u.break=s;break}if(l&&o.isBetweenBreaks(t,s,l,n,i)){u={length:o.getLength(l,c),breaksSize:o.getBreaksSize(l,c),inBreak:!1};break}}return u},isInverted:function(){return!(this._options.isHorizontal^this._businessRange.invert)},_getDiscreteInterval:function(e,t){var n=e-(this._options.stick?1:0);return n>0?t.canvasLength/n:t.canvasLength},_prepareCanvasOptions:function(){var e,t=this,n=t._businessRange,i=t._canvasOptions=a(n),o=t._canvas,r=t._breaks;return i.startPadding=o.startPadding||0,i.endPadding=o.endPadding||0,t._options.isHorizontal?(i.startPoint=o.left+i.startPadding,e=o.width,i.endPoint=o.width-o.right-i.endPadding,i.invert=n.invert):(i.startPoint=o.top+i.startPadding,e=o.height,i.endPoint=o.height-o.bottom-i.endPadding,i.invert=!n.invert),t.canvasLength=i.canvasLength=i.endPoint-i.startPoint,i.rangeDoubleError=Math.pow(10,(0,p.getPower)(i.rangeMax-i.rangeMin)-(0,p.getPower)(e)-2),i.ratioOfCanvasRange=i.canvasLength/(i.rangeMaxVisible-i.rangeMinVisible),void 0!==r&&(i.ratioOfCanvasRange=(i.canvasLength-r[r.length-1].cumulativeWidth)/(i.rangeMaxVisible-i.rangeMinVisible-r[r.length-1].length)),i},updateCanvas:function(e){this._canvas=x(e),this.reinit()},updateBusinessRange:function(e){var t=this,n=e.breaks||[];t._userBreaks=e.userBreaks||[],t._businessRange=C(e),t._breaks=n.length?o(n,t._businessRange):void 0,t.reinit()},update:function(e,t,n){var i=this;i._options=(0,s.extend)(i._options||{},n),i._canvas=x(t),i.updateBusinessRange(e)},getBusinessRange:function(){return this._businessRange},getEventScale:function(e){return e.deltaScale||1},getCanvasVisibleArea:function(){return{min:this._canvasOptions.startPoint,max:this._canvasOptions.endPoint}},_calculateSpecialValues:function(){var e,t=this,n=t._canvasOptions,i=n.startPoint-n.startPadding,o=n.endPoint+n.endPadding,a=t._businessRange,r=a.minVisible,s=a.maxVisible,l=i+n.canvasLength/2;if(r<0&&s>0&&r!==s&&(e=t.translate(0,1)),!(0,g.isDefined)(e)){var u=a.invert^(r<0&&s<=0);e=t._options.isHorizontal?u?o:i:u?i:o}t.sc={canvas_position_default:e,canvas_position_left:i,canvas_position_top:i,canvas_position_center:l,canvas_position_middle:l,canvas_position_right:o,canvas_position_bottom:o,canvas_position_start:n.invert?o:i,canvas_position_end:n.invert?i:o}},translateSpecialCase:function(e){return this.sc[e]},_calculateProjection:function(e){var t=this._canvasOptions;return t.invert?t.endPoint-e:t.startPoint+e},_calculateUnProjection:function(e){var t=this._canvasOptions;return"datetime"===this._businessRange.dataType&&(e=Math.round(e)),t.invert?t.rangeMaxVisible.valueOf()-e:t.rangeMinVisible.valueOf()+e},getMinBarSize:function(e){var t=this.getCanvasVisibleArea(),n=this.from(t.min+e);return v(this.from(t.min)-((0,g.isDefined)(n)?n:this.from(t.max)))},checkMinBarSize:function(e,t,n){return v(e)<t?e>=0?t:-t:e},translate:function(e,t){var n=this.translateSpecialCase(e);return(0,g.isDefined)(n)?Math.round(n):isNaN(e)?null:this.to(e,t)},getInterval:function(e){var t,n=this._canvasOptions;return e=null!==(t=e)&&void 0!==t?t:this._businessRange.interval,e?Math.round(n.ratioOfCanvasRange*e):Math.round(n.endPoint-n.startPoint)},zoom:function(e,t,n){var i=this._canvasOptions;if(i.rangeMinVisible.valueOf()===i.rangeMaxVisible.valueOf()&&0!==e)return this.zoomZeroLengthRange(e,t);var o=i.startPoint,a=i.endPoint,r=this.isInverted(),s=(o+e)/t,l=(a+e)/t;n=n||{};var u,c,d=this.to(r?n.endValue:n.startValue),h=this.to(r?n.startValue:n.endValue);return d>s&&(l-=s-d,s=d,u=r?n.endValue:n.startValue),h<l&&(s-=l-h,l=h,c=r?n.startValue:n.endValue),h-d<l-s&&(s=d,l=h),e=(a-o)*s/(l-s)-o,t=(o+e)/s||1,u=(0,g.isDefined)(u)?u:(0,_.adjust)(this.from(s,1)),c=(0,g.isDefined)(c)?c:(0,_.adjust)(this.from(l,-1)),t<=1&&(u=this._correctValueAboutBreaks(u,1===t?e:-1),c=this._correctValueAboutBreaks(c,1===t?e:1)),u>c?(u=u>n.endValue?n.endValue:u,c=c<n.startValue?n.startValue:c):(u=u<n.startValue?n.startValue:u,c=c>n.endValue?n.endValue:c),{min:u,max:c,translate:(0,_.adjust)(e),scale:(0,_.adjust)(t)}},_correctValueAboutBreaks:function(e,t){var n=this._userBreaks.filter(function(t){return e>=t.from&&e<=t.to});return n.length?t>0?n[0].to:n[0].from:e},zoomZeroLengthRange:function(e,t){var n=this._canvasOptions,i=n.rangeMin,o=n.rangeMax,a=(o.valueOf()!==i.valueOf()?o.valueOf()-i.valueOf():v(n.rangeMinVisible.valueOf()-i.valueOf()))/n.canvasLength,r=(0,g.isDate)(o)||(0,g.isDate)(i),s="logarithmic"===this._businessRange.axisType,l=n.rangeMinVisible.valueOf()-a,u=n.rangeMaxVisible.valueOf()+a;return l=s?(0,_.adjust)((0,p.raiseToExt)(l,n.base)):r?new Date(l):l,u=s?(0,_.adjust)((0,p.raiseToExt)(u,n.base)):r?new Date(u):u,{min:l,max:u,translate:e,scale:t}},getMinScale:function(e){var t=this._businessRange,n=t.dataType,i=t.interval;return"datetime"===n&&1===i?this.getDateTimeMinScale(e):e?1.1:.9},getDateTimeMinScale:function(e){var t=this._canvasOptions,n=t.canvasLength/t.ratioOfCanvasRange;return n+=(parseInt(.1*n)||1)*(e?-2:2),t.canvasLength/(Math.max(n,1)*t.ratioOfCanvasRange)},getScale:function(e,t){var n=this._canvasOptions;return n.rangeMax===n.rangeMin?1:(e=(0,g.isDefined)(e)?this._fromValue(e):n.rangeMin,t=(0,g.isDefined)(t)?this._fromValue(t):n.rangeMax,(n.rangeMax-n.rangeMin)/Math.abs(e-t))},isValid:function(e){var t=this._canvasOptions;return e=this._fromValue(e),null!==e&&!isNaN(e)&&e.valueOf()+t.rangeDoubleError>=t.rangeMin&&e.valueOf()-t.rangeDoubleError<=t.rangeMax},getCorrectValue:function(e,t){var n,i=this,o=i._breaks;return e=i._fromValue(e),i._breaks&&(n=i._checkValueAboutBreaks(o,e,"trFrom","trTo",i._checkingMethodsAboutBreaks[0]),n.inBreak===!0)?i._toValue(t>0?n.break.trTo:n.break.trFrom):i._toValue(e)},to:function(e,t){var n=this.getBusinessRange();if((0,g.isDefined)(n.maxVisible)&&(0,g.isDefined)(n.minVisible)&&n.maxVisible.valueOf()===n.minVisible.valueOf())return(0,g.isDefined)(e)&&n.maxVisible.valueOf()===e.valueOf()?this.translateSpecialCase(0===e&&this._options.shiftZeroValue?"canvas_position_default":"canvas_position_middle"):null;e=this._fromValue(e);var i=this,o=i._canvasOptions,a=i._breaks,r={length:0},s=0;return void 0!==a&&(r=i._checkValueAboutBreaks(a,e,"trFrom","trTo",i._checkingMethodsAboutBreaks[0]),s=(0,g.isDefined)(r.breaksSize)?r.breaksSize:0),r.inBreak===!0?t>0?r.break.start:t<0?r.break.end:null:i._conversionValue(i._calculateProjection((e-o.rangeMinVisible-r.length)*o.ratioOfCanvasRange+s))},from:function(e,t){var n=this,i=n._breaks,o={length:0},a=n._canvasOptions,r=a.startPoint,s=0;return void 0!==i&&(o=n._checkValueAboutBreaks(i,e,"start","end",n._checkingMethodsAboutBreaks[1]),s=(0,g.isDefined)(o.breaksSize)?o.breaksSize:0),o.inBreak===!0?t>0?n._toValue(o.break.trTo):t<0?n._toValue(o.break.trFrom):null:n._toValue(n._calculateUnProjection((e-r-s)/a.ratioOfCanvasRange+o.length))},isValueProlonged:!1,getRange:function(){return[this._toValue(this._canvasOptions.rangeMin),this._toValue(this._canvasOptions.rangeMax)]},getScreenRange:function(){return[this._canvasOptions.startPoint,this._canvasOptions.endPoint]},add:function(e,t,n){return this._add(e,t,(this._businessRange.invert?-1:1)*n)},_add:function(e,t,n){return this._toValue(this._fromValue(e)+t*n)},_fromValue:function(e){return null!==e?Number(e):null},_toValue:function(e){return null!==e?Number(e):null},ratioOfCanvasRange:function(){return this._canvasOptions.ratioOfCanvasRange},convert:function(e){return e},getRangeByMinZoomValue:function(e,t){return t.minVisible+e<=this._businessRange.max?[t.minVisible,t.minVisible+e]:[t.maxVisible-e,t.maxVisible]}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t,n,i=e.x,o=e.y,a=v[e.type],r=e.textWidth+2*e.horMargin,s=e.textHeight+2*e.verMargin,l=i,u=o;return t=n=e.tailLength,1&a[0]?n=Math.min(n,s/3):t=Math.min(t,r/3),{cx:f(l+a[0]*t+(a[0]+a[2])*r/2),cy:f(u+a[1]*n+(a[1]+a[3])*s/2),points:[f(i),f(o),f(i+=a[0]*(r+t)),f(o+=a[1]*(s+n)),f(i+=a[2]*r),f(o+=a[3]*s),f(i+=a[4]*r),f(o+=a[5]*s),f(i+=a[6]*(r-t)),f(o+=a[7]*(s-n))]}}t.BaseRangeBar=t.BaseTextCloudMarker=t.BaseIndicator=t.BaseElement=void 0;var a=n(4),r=n(3),s=n(203),l=n(14),u=n(0),c=i(n(15)),d=isFinite,h=Number,f=Math.round,p=s.formatValue,g=s.getSampleText,_=c.default.inherit({ctor:function(e){var t=this;(0,r.each)(e,function(e,n){t["_"+e]=n}),t._init()},dispose:function(){var e=this;return e._dispose(),(0,r.each)(e,function(t){e[t]=null}),e},getOffset:function(){return h(this._options.offset)||0}});t.BaseElement=_;var m=_.inherit({_init:function(){var e=this;e._rootElement=e._createRoot().linkOn(e._owner,{name:"value-indicator",after:"core"}),e._trackerElement=e._createTracker()},_dispose:function(){this._rootElement.linkOff()},_setupAnimation:function(){var e=this;e._options.animation&&(e._animation={step:function(t){e._actualValue=e._animation.start+e._animation.delta*t,e._actualPosition=e._translator.translate(e._actualValue),e._move()},duration:e._options.animation.duration>0?h(e._options.animation.duration):0,easing:e._options.animation.easing})},_runAnimation:function(e){var t=this,n=t._animation;n.start=t._actualValue,n.delta=e-t._actualValue,t._rootElement.animate({_:0},{step:n.step,duration:n.duration,easing:n.easing})},_createRoot:function(){return this._renderer.g().attr({"class":this._className})},_createTracker:function(){return this._renderer.path([],"area")},_getTrackerSettings:a.noop,clean:function(){var e=this;return e._animation&&e._rootElement.stopAnimation(),e._rootElement.linkRemove().clear(),e._clear(),e._tracker.detach(e._trackerElement),e._options=e.enabled=e._animation=null,e},render:function(e){var t=this;return t.type=e.type,t._options=e,t._actualValue=t._currentValue=t._translator.adjust(t._options.currentValue),t.enabled=t._isEnabled(),t.enabled&&(t._setupAnimation(),t._rootElement.attr({fill:t._options.color}).linkAppend(),t._tracker.attach(t._trackerElement,t,t._trackerInfo)),t},resize:function(e){var t=this;return t._rootElement.clear(),t._clear(),t.visible=t._isVisible(e),t.visible&&((0,u.extend)(t._options,e),t._actualPosition=t._translator.translate(t._actualValue),t._render(),t._trackerElement.attr(t._getTrackerSettings()),t._move()),t},value:function(e,t){var n,i=this,o=this._rootElement,a=null;return void 0===e?i._currentValue:(null===e?(a="hidden",i._currentValue=e):(n=i._translator.adjust(e),i._currentValue!==n&&d(n)&&(i._currentValue=n,i.visible&&(i._animation&&!t?i._runAnimation(n):(i._actualValue=n,i._actualPosition=i._translator.translate(n),i._move())))),o.attr({visibility:a}),i)},_isEnabled:null,_isVisible:null,_render:null,_clear:null,_move:null});t.BaseIndicator=m;var v={};v["right-bottom"]=v.rb=[0,-1,-1,0,0,1,1,0],v["bottom-right"]=v.br=[-1,0,0,-1,1,0,0,1],v["left-bottom"]=v.lb=[0,-1,1,0,0,1,-1,0],v["bottom-left"]=v.bl=[1,0,0,-1,-1,0,0,1],v["left-top"]=v.lt=[0,1,1,0,0,-1,-1,0],v["top-left"]=v.tl=[1,0,0,1,-1,0,0,-1],v["right-top"]=v.rt=[0,1,-1,0,0,-1,1,0],v["top-right"]=v.tr=[-1,0,0,1,1,0,0,-1];var y=m.inherit({_move:function(){var e=this,t=e._getTextCloudOptions(),n=p(e._actualValue,e._options.text);e._text.attr({text:n});var i=e._text.getBBox(),a=o({x:t.x,y:t.y,textWidth:i.width||n.length*e._textUnitWidth,textHeight:i.height||e._textHeight,horMargin:e._options.horizontalOffset,verMargin:e._options.verticalOffset,tailLength:e._options.arrowLength,type:t.type});e._text.attr({x:a.cx,y:a.cy+e._textVerticalOffset}),e._cloud.attr({points:a.points}),e._trackerElement&&e._trackerElement.attr({points:a.points})},_measureText:function(){var e,t,n,i,o=this;o._textVerticalOffset||(e=o._createRoot().append(o._owner),i=g(o._translator,o._options.text),t=o._renderer.text(i,0,0).attr({align:"center"}).css((0,l.patchFontOptions)(o._options.text.font)).append(e),n=t.getBBox(),e.remove(),o._textVerticalOffset=-n.y-n.height/2,o._textWidth=n.width,o._textHeight=n.height,o._textUnitWidth=o._textWidth/i.length,o._textFullWidth=o._textWidth+2*o._options.horizontalOffset,o._textFullHeight=o._textHeight+2*o._options.verticalOffset)},_render:function(){var e=this;e._measureText(),e._cloud=e._cloud||e._renderer.path([],"area").append(e._rootElement),e._text=e._text||e._renderer.text().append(e._rootElement),e._text.attr({align:"center"}).css((0,l.patchFontOptions)(e._options.text.font))},_clear:function(){delete this._cloud,delete this._text},getTooltipParameters:function(){var e=this._getTextCloudOptions();return{x:e.x,y:e.y,value:this._currentValue,color:this._options.color}}});t.BaseTextCloudMarker=y;var b=m.inherit({_measureText:function(){var e,t,n,i=this;i._hasText=i._isTextVisible(),i._hasText&&!i._textVerticalOffset&&(e=i._createRoot().append(i._owner),t=i._renderer.text(g(i._translator,i._options.text),0,0).attr({"class":"dxg-text",align:"center"}).css((0,l.patchFontOptions)(i._options.text.font)).append(e),n=t.getBBox(),e.remove(),i._textVerticalOffset=-n.y-n.height/2,i._textWidth=n.width,i._textHeight=n.height)},_move:function(){var e=this;e._updateBarItemsPositions(),e._hasText&&(e._text.attr({text:p(e._actualValue,e._options.text)}),e._updateTextPosition(),e._updateLinePosition())},_updateBarItems:function(){var e,t=this,n=t._options,i=t._translator;t._setBarSides(),t._startPosition=i.translate(i.getDomainStart()),t._endPosition=i.translate(i.getDomainEnd()),t._basePosition=i.translate(n.baseValue),t._space=t._getSpace();var o=n.backgroundColor||"none";"none"!==o&&t._space>0?e=n.containerBackgroundColor||"none":(t._space=0,e="none"),t._backItem1.attr({fill:o}),t._backItem2.attr({fill:o}),t._spaceItem1.attr({fill:e}),t._spaceItem2.attr({fill:e})},_getSpace:function(){return 0},_updateTextItems:function(){var e=this;e._hasText?(e._line=e._line||e._renderer.path([],"line").attr({"class":"dxg-main-bar","stroke-linecap":"square"}).append(e._rootElement),e._text=e._text||e._renderer.text("",0,0).attr({"class":"dxg-text"}).append(e._rootElement),e._text.attr({align:e._getTextAlign()}).css(e._getFontOptions()),e._setTextItemsSides()):(e._line&&(e._line.remove(),delete e._line),e._text&&(e._text.remove(),delete e._text))},_isTextVisible:function(){return!1},_getTextAlign:function(){return"center"},_getFontOptions:function(){var e=this._options,t=e.text.font;return t&&t.color||(t=(0,u.extend)({},t,{color:e.color})),(0,l.patchFontOptions)(t)},_updateBarItemsPositions:function(){var e=this,t=e._getPositions();e._backItem1.attr(e._buildItemSettings(t.start,t.back1)),e._backItem2.attr(e._buildItemSettings(t.back2,t.end)),e._spaceItem1.attr(e._buildItemSettings(t.back1,t.main1)),e._spaceItem2.attr(e._buildItemSettings(t.main2,t.back2)),e._mainItem.attr(e._buildItemSettings(t.main1,t.main2)),e._trackerElement&&e._trackerElement.attr(e._buildItemSettings(t.main1,t.main2))},_render:function(){var e=this;e._measureText(),e._backItem1||(e._backItem1=e._createBarItem(),e._backItem1.attr({"class":"dxg-back-bar"})),e._backItem2||(e._backItem2=e._createBarItem(),e._backItem2.attr({"class":"dxg-back-bar"})),e._spaceItem1||(e._spaceItem1=e._createBarItem(),e._spaceItem1.attr({"class":"dxg-space-bar"})),e._spaceItem2||(e._spaceItem2=e._createBarItem(),e._spaceItem2.attr({"class":"dxg-space-bar"})),e._mainItem||(e._mainItem=e._createBarItem(),e._mainItem.attr({"class":"dxg-main-bar"})),e._updateBarItems(),e._updateTextItems()},_clear:function(){var e=this;delete e._backItem1,delete e._backItem2,delete e._spaceItem1,delete e._spaceItem2,delete e._mainItem,delete e._hasText,delete e._line,delete e._text},getTooltipParameters:function(){var e=this._getTooltipPosition();return{x:e.x,y:e.y,value:this._currentValue,color:this._options.color,offset:0}}});t.BaseRangeBar=b},function(e,t,n){function i(e){var t=e.border||{};return{fill:e.color,opacity:e.opacity,stroke:t.color,"stroke-width":t.width,"stroke-opacity":t.opacity,hatching:e.hatching}}function o(e,t){return{attr:{filter:t},css:(0,a.patchFontOptions)(e.font)}}t.buildRectAppearance=i,t.buildTextAppearance=o;var a=n(14)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){this._initHandlers(e)}function a(e,t){var n=t.getData(e);n>=0&&t.click({node:t.getNode(n),coords:t.getCoords(e),event:e})}function r(e,t){var n=t.getData(e);n>=0?t.getNode(n).setHover():t.widget.clearHover()}function s(e,t){var n,i=t.getData(e,!0);i>=0?(n=(0,l.eventData)(e),t.getNode(i).showTooltip([n.x,n.y])):t.widget.hideTooltip()}t.Tracker=o;var l=n(8),u=i(n(11)),c=n(21),d=i(n(25)),h=i(n(5)),f=d.default.down,p=d.default.move;o.prototype={constructor:o,_initHandlers:function(e){function t(t){a(t,e)}function n(t){d?d=!1:(void 0!==e.getData(t)&&(d=!0),i(t))}function i(t){r(t,e),e.widget._getOption("tooltip").enabled&&s(t,e)}var o=u.default.getDocument();e.getCoords=function(t){var n=(0,l.eventData)(t),i=e.widget._renderer.getRootOffset();return[n.x-i.left,n.y-i.top]},e.root.on(c.name,t),e.root.on(f,n),h.default.on(o,f,n),h.default.on(o,p,i),this._disposeHandlers=function(){e.root.off(c.name,t),e.root.off(f,n),h.default.off(o,f,n),h.default.off(o,p,i)};var d=!1},dispose:function(){this._disposeHandlers()}}},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.Options=void 0;var r=n(1),s=n(4),l=n(358),u=n(50),c=n(185),d=n(0),h=function(){function e(t,n,o,a){var r=this;i(this,e),this._deprecatedCallback,this._startChangeCallback,this._endChangeCallback,this._default=n,this._deprecated=a,this._deprecatedNames=[],this._initDeprecatedNames(),this._optionManager=new l.OptionManager(t,o),this._optionManager.onRelevantNamesPrepared(function(e,t,n,i){return r._setRelevantNames(e,t,n,i)}),this._cachedOptions={},this._rules=[]}return a(e,[{key:"_initDeprecatedNames",value:function(){for(var e in this._deprecated)this._deprecatedNames.push(e)}},{key:"_getByRules",value:function(e){return e=Array.isArray(e)?this._rules.concat(e):this._rules,(0,c.convertRulesToOptions)(e)}},{key:"_notifyDeprecated",value:function(e){var t=this._deprecated[e];t&&this._deprecatedCallback(e,t)}},{key:"_setRelevantNames",value:function(e,t,n,i){if(t){var o=this._normalizeName(t,i);o&&o!==t&&(this._setField(e,o,n),this._clearField(e,t))}}},{key:"_setField",value:function(e,t,n){var i="",o=null;do i=i?".".concat(i):"",i=(0,c.getFieldName)(t)+i,t=(0,c.getParentName)(t),o=t?this._optionManager.get(e,t,!1):e;while(!o);o[i]=n}},{key:"_clearField",value:function(e,t){delete e[t];var n=(0,c.getParentName)(t),i=n?this._optionManager.get(e,n,!1):e;i&&delete i[(0,c.getFieldName)(t)]}},{key:"_normalizeName",value:function(e,t){if(this._deprecatedNames.length&&e)for(var n=0;n<this._deprecatedNames.length;n++)if(this._deprecatedNames[n]===e){var i=this._deprecated[e];if(i)return!t&&this._notifyDeprecated(e),i.alias||e}return e}},{key:"addRules",value:function(e){this._rules=e.concat(this._rules)}},{key:"applyRules",value:function(e){var t=this._getByRules(e);this.silent(t)}},{key:"dispose",value:function(){this._deprecatedCallback=s.noop,this._startChangeCallback=s.noop,this._endChangeCallback=s.noop,
this._optionManager.dispose()}},{key:"onChanging",value:function(e){this._optionManager.onChanging(e)}},{key:"onChanged",value:function(e){this._optionManager.onChanged(e)}},{key:"onDeprecated",value:function(e){this._deprecatedCallback=e}},{key:"onStartChange",value:function(e){this._startChangeCallback=e}},{key:"onEndChange",value:function(e){this._endChangeCallback=e}},{key:"isInitial",value:function(e){var t=this.silent(e),n=this.initial(e),i=(0,r.isFunction)(t)&&(0,r.isFunction)(n);return i?t.toString()===n.toString():(0,s.equalByValue)(t,n)}},{key:"initial",value:function(e){return(0,c.getNestedOptionValue)(this._initial,e)}},{key:"option",value:function(e,t){var n=arguments.length<2&&"object"!==(0,r.type)(e);if(n)return this._optionManager.get(void 0,this._normalizeName(e));this._startChangeCallback();try{this._optionManager.set(e,t)}finally{this._endChangeCallback()}}},{key:"silent",value:function(e,t){var n=arguments.length<2&&"object"!==(0,r.type)(e);return n?this._optionManager.get(void 0,e,void 0,!0):void this._optionManager.set(e,t,void 0,!0)}},{key:"reset",value:function(e){var t=this;if(e){var n=e.replace(/\[([^\]])\]/g,".$1").split("."),i=n.reduce(function(e,n){return e?e[n]:t.initial(n)},null),o=(0,r.isObject)(i)?(0,u.clone)(i):i;this._optionManager.set(e,o,!1)}}},{key:"getAliasesByName",value:function(e){var t=this;return Object.keys(this._deprecated).filter(function(n){return e===t._deprecated[n].alias})}},{key:"isDeprecated",value:function(e){return Object.prototype.hasOwnProperty.call(this._deprecated,e)}},{key:"cache",value:function(e,t){var n=arguments.length<2;return n?this._cachedOptions[e]:void(this._cachedOptions[e]=(0,d.extend)(this._cachedOptions[e],t))}},{key:"_initial",set:function(e){this._initialOptions=e},get:function(){if(!this._initialOptions){var e=this._getByRules(this.silent("defaultOptionsRules"));this._initialOptions=this._default,this._optionManager._setByReference(this._initialOptions,e)}return this._initialOptions}}]),e}();t.Options=h},function(e,t,n){t.default=void 0;var i=n(1),o=function(e){return Array.isArray(e)&&e.every(function(e){var t=2===Object.keys(e).length,n="key"in e&&"items"in e;return t&&n&&Array.isArray(e.items)})},a={_getSpecificDataSourceOption:function(){var e="key",t=this.option("dataSource"),n=!1,a={};return this._getGroupedOption()&&o(t)&&(a=t.reduce(function(t,o){var a=o.items.map(function(t){return(0,i.isObject)(t)||(t={text:t},n=!0),e in t||(t[e]=o.key),t});return t.concat(a)},[]),t={store:{type:"array",data:a},group:{selector:"key",keepInitialKeyOrder:!0}},n&&(t.searchExpr="text")),t}};t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){var n=new V(e,t);return n._checkZipState(),n.ready().then(function(){return n.getData((0,l.isFunction)((0,s.getWindow)().Blob))})}t.getData=a,t.ExcelCreator=void 0;var r=i(n(15)),s=n(7),l=n(1),u=n(0),c=i(n(17)),d=n(42),h=i(n(267)),f=n(177),p=i(n(239)),g=i(n(342)),_=n(6),m='<?xml version="1.0" encoding="utf-8"?>',v='<sheetPr><outlinePr summaryBelow="0"/></sheetPr>',y="<sheetPr/>",b='<borders count="1"><border><left style="thin"><color rgb="FFD3D3D3"/></left><right style="thin"><color rgb="FFD3D3D3"/></right><top style="thin"><color rgb="FFD3D3D3"/></top><bottom style="thin"><color rgb="FFD3D3D3"/></bottom></border></borders><cellStyleXfs count="1"><xf numFmtId="0" fontId="0" fillId="0" borderId="0"/></cellStyleXfs>',x="http://schemas.openxmlformats.org",w="rels",C="xl",k="workbook.xml",S="[Content_Types].xml",D="sharedStrings.xml",I="styles.xml",T="worksheets",O="sheet1.xml",E='<?xml version="1.0" encoding="UTF-8" standalone="yes"?><worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" mc:Ignorable="x14ac" xmlns:x14ac="http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac">',A={"boolean":"b",date:"d",number:"n",string:"s"},P=Date.UTC(1899,11,30),R=60,M=7,B={quarter:"shortDate",quarterAndYear:"shortDate",minute:"longTime",millisecond:"longTime"},V=r.default.inherit({_getXMLTag:function(e,t,n){var i,o,a="<"+e,r=t.length;for(i=0;i<r;i++)o=t[i],void 0!==o.value&&(a=a+" "+o.name+'="'+o.value+'"');return(0,l.isDefined)(n)?a+">"+n+"</"+e+">":a+" />"},_convertToExcelCellRef:function(e,t){for(var n,i,o="",a=26;!i;)n=65+(t>=a?t%a:Math.ceil(t)),o=String.fromCharCode(n)+o,t>=a?t=Math.floor(t/a)-1:i=!0;return o+(e+1)},_convertToExcelCellRefAndTrackMaxIndex:function(e,t){return this._maxRowIndex<Number(e)&&(this._maxRowIndex=Number(e)),this._maxColumnIndex<Number(t)&&(this._maxColumnIndex=Number(t)),this._convertToExcelCellRef(e,t)},_getDataType:function(e){return A[e]||A.string},_tryGetExcelCellDataType:function(e){if((0,l.isDefined)(e)){if("number"==typeof e)return isFinite(e)?A.number:A.string;if((0,l.isString)(e))return A.string;if((0,l.isDate)(e))return A.number;if((0,l.isBoolean)(e))return A.boolean}},_formatObjectConverter:function(e,t){var n={format:e,precision:e&&e.precision,dataType:t};return(0,l.isObject)(e)?(0,u.extend)(n,e,{format:e.formatter||e.type,currency:e.currency}):n},_tryConvertToExcelNumberFormat:function(e,t){var n=this._formatObjectConverter(e,t);e=n.format;var i=n.currency;return t=n.dataType,(0,l.isDefined)(e)&&"date"===t&&(e=B[e&&e.type||e]||e),p.default.convertFormat(e,n.precision,t,i)},_appendString:function(e){if((0,l.isDefined)(e)&&(e=String(e),e.length))return e=(0,d.encodeHtml)(e),void 0===this._stringHash[e]&&(this._stringHash[e]=this._stringArray.length,this._stringArray.push(e)),this._stringHash[e]},_tryGetExcelDateValue:function(e){var t,n;if((0,l.isDate)(e))return t=Math.floor((Date.UTC(e.getFullYear(),e.getMonth(),e.getDate())-P)/864e5),t<R&&t--,n=(3600*e.getHours()+60*e.getMinutes()+e.getSeconds())/86400,t+n},_prepareValue:function(e,t){var n,i=this._dataProvider,o=i.getCellData(e,t)||{},a=o.cellSourceData,r=i.getCellData(e,t)||{},s=r.value,u=this._getDataType(i.getCellType(e,t));switch(u!==A.date||(0,l.isDate)(s)||(u=A.string),u){case A.string:n=s,s=this._appendString(s);break;case A.date:n=s,s=this._tryGetExcelDateValue(s),u=A.number}return{value:s,type:u,sourceValue:n,cellSourceData:a}},_callCustomizeExcelCell:function(e){var t=e.dataProvider,n=e.value,i=e.style,o=e.sourceData,a=g.default.copyCellFormat(i),r={value:n,numberFormat:a.numberFormat,clearStyle:function(){this.horizontalAlignment=null,this.verticalAlignment=null,this.wrapTextEnabled=null,this.font=null,this.numberFormat=null}};(0,l.isDefined)(a)&&((0,l.isDefined)(a.alignment)&&(r.horizontalAlignment=a.alignment.horizontal,r.verticalAlignment=a.alignment.vertical,r.wrapTextEnabled=a.alignment.wrapText),r.backgroundColor=a.backgroundColor,r.fillPatternType=a.fillPatternType,r.fillPatternColor=a.fillPatternColor,r.font=a.font),t.customizeExcelCell(r,o);var s=a||{};return s.font=r.font,s.alignment=s.alignment||{},s.alignment.horizontal=r.horizontalAlignment,s.alignment.vertical=r.verticalAlignment,s.alignment.wrapText=r.wrapTextEnabled,s.backgroundColor=r.backgroundColor,s.fillPatternType=r.fillPatternType,s.fillPatternColor=r.fillPatternColor,s.numberFormat=r.numberFormat,{value:r.value,style:s}},_getDataArray:function(){var e,t,n,i,a,r=this,s=[],u=r._dataProvider,c=u.getRowsCount(),d=u.getColumns();for(e=0;e<c;e++){for(n=[],a=d.length,t=0;t!==a;t++){i=r._prepareValue(e,t);var h=u.getStyleId(e,t),f=this._styleArrayIndexToCellStyleIdMap[h];if(u.hasCustomizeExcelCell&&u.hasCustomizeExcelCell()){var p=i.sourceValue||i.value,g=this._callCustomizeExcelCell({dataProvider:u,value:p,style:r._styleArray[h],sourceData:i.cellSourceData});if(g.value!==p){if(o(g.value)!==o(p)||"number"==typeof g.value&&!isFinite(g.value)){var _=this._tryGetExcelCellDataType(g.value);(0,l.isDefined)(_)&&(i.type=_)}switch(i.type){case A.string:i.value=this._appendString(g.value);break;case A.date:i.value=g.value;break;case A.number:var m=g.value,v=this._tryGetExcelDateValue(m);(0,l.isDefined)(v)&&(m=v),i.value=m;break;default:i.value=g.value}}f=this._excelFile.registerCellFormat(g.style)}n.push({style:f,value:i.value,type:i.type})}!r._needSheetPr&&u.getGroupLevel(e)>0&&(r._needSheetPr=!0),s.push(n)}return s},_calculateWidth:function(e){return e=parseInt(e,10),(!e||e<5)&&(e=100),Math.min(255,Math.floor((e-5)/M*100+.5)/100)},_prepareStyleData:function(){var e=this,t=this,n=t._dataProvider.getStyles();t._dataProvider.getColumns().forEach(function(e){t._colsArray.push(t._calculateWidth(e.width))});var i=[{size:11,color:{theme:1},name:"Calibri",family:2,scheme:"minor",bold:!1},{size:11,color:{theme:1},name:"Calibri",family:2,scheme:"minor",bold:!0}];this._excelFile.registerFont(i[0]),this._excelFile.registerFont(i[1]),n.forEach(function(e){var n=t._tryConvertToExcelNumberFormat(e.format,e.dataType);(0,l.isDefined)(n)||(n=0),t._styleArray.push({font:i[Number(!!e.bold)],numberFormat:n,alignment:{vertical:"top",wrapText:!!e.wrapText,horizontal:e.alignment||"left"}})}),t._styleArrayIndexToCellStyleIdMap=t._styleArray.map(function(t){return e._excelFile.registerCellFormat(t)})},_prepareCellData:function(){this._cellsArray=this._getDataArray()},_createXMLRelationships:function(e){return this._getXMLTag("Relationships",[{name:"xmlns",value:x+"/package/2006/relationships"}],e)},_createXMLRelationship:function(e,t,n){return this._getXMLTag("Relationship",[{name:"Id",value:"rId"+e},{name:"Type",value:x+"/officeDocument/2006/relationships/"+t},{name:"Target",value:n}])},_getWorkbookContent:function(){var e='<bookViews><workbookView xWindow="0" yWindow="0" windowWidth="0" windowHeight="0"/></bookViews><sheets><sheet name="Sheet" sheetId="1" r:id="rId1" /></sheets><definedNames><definedName name="_xlnm.Print_Titles" localSheetId="0">Sheet!$1:$1</definedName><definedName name="_xlnm._FilterDatabase" hidden="0" localSheetId="0">Sheet!$A$1:$F$6332</definedName></definedNames>';return m+this._getXMLTag("workbook",[{name:"xmlns:r",value:x+"/officeDocument/2006/relationships"},{name:"xmlns",value:x+"/spreadsheetml/2006/main"}],e)},_getContentTypesContent:function(){return m+'<Types xmlns="'+x+'/package/2006/content-types"><Default Extension="rels" ContentType="application/vnd.openxmlformats-package.relationships+xml" /><Default Extension="xml" ContentType="application/xml" /><Override PartName="/xl/worksheets/sheet1.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml" /><Override PartName="/xl/styles.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml" /><Override PartName="/xl/sharedStrings.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml" /><Override PartName="/xl/workbook.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml" /></Types>'},_generateStylesXML:function(){var e=this,t=e._zip.folder(C),n="";n+=this._excelFile.generateNumberFormatsXml(),n+=this._excelFile.generateFontsXml(),n+=this._excelFile.generateFillsXml(),n+=b,n+=this._excelFile.generateCellFormatsXml(),n+=e._getXMLTag("cellStyles",[{name:"count",value:1}],e._getXMLTag("cellStyle",[{name:"name",value:"Normal"},{name:"xfId",value:0},{name:"builtinId",value:0}])),n=m+e._getXMLTag("styleSheet",[{name:"xmlns",value:x+"/spreadsheetml/2006/main"}],n),t.file(I,n),e._styleArray=[]},_generateStringsXML:function(){var e,t=this._zip.folder(C),n=this._stringArray.length,i=m;for(e=0;e<n;e++)this._stringArray[e]=this._getXMLTag("si",[],this._getXMLTag("t",[],this._stringArray[e]));i+=this._getXMLTag("sst",[{name:"xmlns",value:x+"/spreadsheetml/2006/main"},{name:"count",value:this._stringArray.length},{name:"uniqueCount",value:this._stringArray.length}],this._stringArray.join("")),t.file(D,i),this._stringArray=[]},_getPaneXML:function(){var e=[{name:"activePane",value:"bottomLeft"},{name:"state",value:"frozen"}],t=this._dataProvider.getFrozenArea();return t.x||t.y?(t.x&&e.push({name:"xSplit",value:t.x}),t.y&&e.push({name:"ySplit",value:t.y}),e.push({name:"topLeftCell",value:this._convertToExcelCellRefAndTrackMaxIndex(t.y,t.x)}),this._getXMLTag("pane",e)):""},_getAutoFilterXML:function(e){return this._options.autoFilterEnabled?'<autoFilter ref="A'+this._dataProvider.getHeaderRowCount()+":"+e+'" />':""},_getIgnoredErrorsXML:function(e){return this._options.ignoreErrors?'<ignoredErrors><ignoredError sqref="A1:'+e+'" numberStoredAsText="1" /></ignoredErrors>':""},_generateWorksheetXML:function(){var e,t,n,i,o,a=[],r=this._cellsArray.length,s=this._colsArray.length,u="1:"+s,c=this._dataProvider.getHeaderRowCount?this._dataProvider.getHeaderRowCount():1,d=[E];for(d.push(this._needSheetPr?v:y),d.push('<dimension ref="A1:C1"/>'),d.push("<sheetViews><sheetView "),d.push(this._rtlEnabled?'rightToLeft="1" ':""),d.push('tabSelected="1" workbookViewId="0">'),d.push(this._getPaneXML()),d.push("</sheetView></sheetViews>"),d.push('<sheetFormatPr defaultRowHeight="15"'),d.push(' outlineLevelRow="'+(this._dataProvider.getRowsCount()>0?this._dataProvider.getGroupLevel(0):0)+'"'),d.push(' x14ac:dyDescent="0.25"/>'),e=0;e<s;e++)this._colsArray[e]=this._getXMLTag("col",[{name:"width",value:this._colsArray[e]},{name:"min",value:Number(e)+1},{name:"max",value:Number(e)+1},{name:"customWidth",value:1}]);for(d.push(this._getXMLTag("cols",[],this._colsArray.join(""))+"<sheetData>"),t=0;t<r;t++){for(i=[],o=this._cellsArray[t].length,e=0;e<o;e++)t=Number(t),n=this._cellsArray[t][e],i.push(this._getXMLTag("c",[{name:"r",value:this._convertToExcelCellRefAndTrackMaxIndex(t,e)},{name:"s",value:n.style},{name:"t",value:n.type}],(0,l.isDefined)(n.value)?this._getXMLTag("v",[],n.value):null));a.push(this._getXMLTag("row",[{name:"r",value:Number(t)+1},{name:"spans",value:u},{name:"outlineLevel",value:t>=c?this._dataProvider.getGroupLevel(t):0},{name:"x14ac:dyDescent",value:"0.25"}],i.join(""))),this._cellsArray[t]=null,a.length>1e4&&(d.push(a.join("")),a=[])}d.push(a.join("")),a=[];var h=this._convertToExcelCellRef(this._maxRowIndex,this._maxColumnIndex);d.push("</sheetData>"+this._getAutoFilterXML(h)+this._generateMergingXML()+this._getIgnoredErrorsXML(h)+"</worksheet>"),this._zip.folder(C).folder(T).file(O,d.join("")),this._colsArray=[],this._cellsArray=[],d=[]},_generateMergingXML:function(){var e,t,n,i,o,a=(0,l.isDefined)(this._dataProvider.getHeaderRowCount)?this._dataProvider.getHeaderRowCount():this._dataProvider.getRowsCount(),r=this._dataProvider.getColumns().length,s=[],u=[],c="";for(i=0;i<a;i++)for(n=0;n!==r;n++)if(!(0,l.isDefined)(s[i])||!(0,l.isDefined)(s[i][n])){var d=this._dataProvider.getCellMerging(i,n);if(d.colspan||d.rowspan)for(u.push({start:this._convertToExcelCellRefAndTrackMaxIndex(i,n),end:this._convertToExcelCellRefAndTrackMaxIndex(i+(d.rowspan||0),n+(d.colspan||0))}),e=i;e<=i+d.rowspan;e++)for(t=n;t<=n+d.colspan;t++)(0,l.isDefined)(s[e])||(s[e]=[]),s[e][t]=!0}var h=u.length;for(o=0;o<h;o++)c+=this._getXMLTag("mergeCell",[{name:"ref",value:u[o].start+":"+u[o].end}]);return c.length?this._getXMLTag("mergeCells",[{name:"count",value:h}],c):""},_generateCommonXML:function(){var e=m+this._createXMLRelationships(this._createXMLRelationship(1,"officeDocument","xl/"+k)),t=this._zip.folder(C),n=m;this._zip.folder("_"+w).file("."+w,e);var i=this._createXMLRelationship(1,"worksheet","worksheets/"+O)+this._createXMLRelationship(2,"styles",I)+this._createXMLRelationship(3,"sharedStrings",D);n+=this._createXMLRelationships(i),t.folder("_"+w).file(k+".rels",n),t.file(k,this._getWorkbookContent()),this._zip.file(S,this._getContentTypesContent())},_generateContent:function(){this._prepareStyleData(),this._prepareCellData(),this._generateWorkXML(),this._generateCommonXML()},_generateWorkXML:function(){this._generateStylesXML(),this._generateStringsXML(),this._generateWorksheetXML()},ctor:function(e,t){this._rtlEnabled=t&&!!t.rtlEnabled,this._options=t,this._maxRowIndex=0,this._maxColumnIndex=0,this._stringArray=[],this._stringHash={},this._styleArray=[],this._colsArray=[],this._cellsArray=[],this._needSheetPr=!1,this._dataProvider=e,this._excelFile=new g.default,(0,l.isDefined)(V.JSZip)?this._zip=new V.JSZip:this._zip=null},_checkZipState:function(){if(!this._zip)throw c.default.Error("E1041","JSZip")},ready:function(){return this._dataProvider.ready()},getData:function(e){var t={type:e?"blob":"base64",compression:"DEFLATE",mimeType:f.MIME_TYPES.EXCEL},n=new _.Deferred;return this._checkZipState(),this._generateContent(),this._zip.generateAsync?this._zip.generateAsync(t).then(n.resolve):n.resolve(this._zip.generate(t)),n}});t.ExcelCreator=V,V.JSZip=h.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(28)),a=i(n(82)),r=i(n(241)),s=i(n(339)),l=/([^\s0]+)?(\s*)0*[.,]*0*(\s*)([^\s0]+)?/,u={},c=function(e){var t=a.default.locale()+"/"+JSON.stringify(e);return u[t]||(u[t]=new Intl.NumberFormat(a.default.locale(),e).format),u[t]},d=function(e){return new Intl.NumberFormat(a.default.locale(),{style:"currency",currency:e})},h={engine:function(){return"intl"},_formatNumberCore:function(e,t,n){return"exponential"===t?this.callBase.apply(this,arguments):c(this._normalizeFormatConfig(t,n,e))(e)},_normalizeFormatConfig:function(e,t,n){var i;return i="decimal"===e?{minimumIntegerDigits:t.precision||void 0,useGrouping:!1,maximumFractionDigits:String(n).length,round:n<0?"ceil":"floor"}:this._getPrecisionConfig(t.precision),"percent"===e?i.style="percent":"currency"===e&&(i.style="currency",i.currency=t.currency||(0,o.default)().defaultCurrency),i},_getPrecisionConfig:function(e){var t;return t=null===e?{minimumFractionDigits:0,maximumFractionDigits:20}:{minimumFractionDigits:e||0,maximumFractionDigits:e||0}},format:function(e,t){return"number"!=typeof e?e:(t=this._normalizeFormat(t),"default"===t.currency&&(t.currency=(0,o.default)().defaultCurrency),t&&("function"==typeof t||t.type||t.formatter)?this.callBase.apply(this,arguments):c(t)(e))},_getCurrencySymbolInfo:function(e){var t=d(e);return this._extractCurrencySymbolInfo(t.format(0))},_extractCurrencySymbolInfo:function(e){var t=l.exec(e)||[],n=t[1]?"before":"after",i=t[1]||t[4]||"",o=t[2]||t[3]||"";return{position:n,symbol:i,delimiter:o}},getCurrencySymbol:function(e){e||(e=(0,o.default)().defaultCurrency);var t=this._getCurrencySymbolInfo(e);return{symbol:t.symbol}},getOpenXmlCurrencyFormat:function(e){var t=e||(0,o.default)().defaultCurrency,n=this._getCurrencySymbolInfo(t).symbol,i=a.default.getValueByClosestLocale(function(e){return s.default[e]});return(0,r.default)(n,i)}};t.default=h,e.exports=t.default},function(e,t,n){t.default=void 0;var i={"en-150":"en-001","en-AG":"en-001","en-AI":"en-001","en-AU":"en-001","en-BB":"en-001","en-BM":"en-001","en-BS":"en-001","en-BW":"en-001","en-BZ":"en-001","en-CA":"en-001","en-CC":"en-001","en-CK":"en-001","en-CM":"en-001","en-CX":"en-001","en-CY":"en-001","en-DG":"en-001","en-DM":"en-001","en-ER":"en-001","en-FJ":"en-001","en-FK":"en-001","en-FM":"en-001","en-GB":"en-001","en-GD":"en-001","en-GG":"en-001","en-GH":"en-001","en-GI":"en-001","en-GM":"en-001","en-GY":"en-001","en-HK":"en-001","en-IE":"en-001","en-IL":"en-001","en-IM":"en-001","en-IN":"en-001","en-IO":"en-001","en-JE":"en-001","en-JM":"en-001","en-KE":"en-001","en-KI":"en-001","en-KN":"en-001","en-KY":"en-001","en-LC":"en-001","en-LR":"en-001","en-LS":"en-001","en-MG":"en-001","en-MO":"en-001","en-MS":"en-001","en-MT":"en-001","en-MU":"en-001","en-MW":"en-001","en-MY":"en-001","en-NA":"en-001","en-NF":"en-001","en-NG":"en-001","en-NR":"en-001","en-NU":"en-001","en-NZ":"en-001","en-PG":"en-001","en-PH":"en-001","en-PK":"en-001","en-PN":"en-001","en-PW":"en-001","en-RW":"en-001","en-SB":"en-001","en-SC":"en-001","en-SD":"en-001","en-SG":"en-001","en-SH":"en-001","en-SL":"en-001","en-SS":"en-001","en-SX":"en-001","en-SZ":"en-001","en-TC":"en-001","en-TK":"en-001","en-TO":"en-001","en-TT":"en-001","en-TV":"en-001","en-TZ":"en-001","en-UG":"en-001","en-VC":"en-001","en-VG":"en-001","en-VU":"en-001","en-WS":"en-001","en-ZA":"en-001","en-ZM":"en-001","en-ZW":"en-001","en-AT":"en-150","en-BE":"en-150","en-CH":"en-150","en-DE":"en-150","en-DK":"en-150","en-FI":"en-150","en-NL":"en-150","en-SE":"en-150","en-SI":"en-150","es-AR":"es-419","es-BO":"es-419","es-BR":"es-419","es-BZ":"es-419","es-CL":"es-419","es-CO":"es-419","es-CR":"es-419","es-CU":"es-419","es-DO":"es-419","es-EC":"es-419","es-GT":"es-419","es-HN":"es-419","es-MX":"es-419","es-NI":"es-419","es-PA":"es-419","es-PE":"es-419","es-PR":"es-419","es-PY":"es-419","es-SV":"es-419","es-US":"es-419","es-UY":"es-419","es-VE":"es-419","pt-AO":"pt-PT","pt-CH":"pt-PT","pt-CV":"pt-PT","pt-FR":"pt-PT","pt-GQ":"pt-PT","pt-GW":"pt-PT","pt-LU":"pt-PT","pt-MO":"pt-PT","pt-MZ":"pt-PT","pt-ST":"pt-PT","pt-TL":"pt-PT","az-Arab":"root","az-Cyrl":"root","blt-Latn":"root","bm-Nkoo":"root","bs-Cyrl":"root","byn-Latn":"root","cu-Glag":"root","dje-Arab":"root","dyo-Arab":"root","en-Dsrt":"root","en-Shaw":"root","ff-Adlm":"root","ff-Arab":"root","ha-Arab":"root","hi-Latn":"root","iu-Latn":"root","kk-Arab":"root","ks-Deva":"root","ku-Arab":"root","ky-Arab":"root","ky-Latn":"root","ml-Arab":"root","mn-Mong":"root","mni-Mtei":"root","ms-Arab":"root","pa-Arab":"root","sat-Deva":"root","sd-Deva":"root","sd-Khoj":"root","sd-Sind":"root","shi-Latn":"root","so-Arab":"root","sr-Latn":"root","sw-Arab":"root","tg-Arab":"root","ug-Cyrl":"root","uz-Arab":"root","uz-Cyrl":"root","vai-Latn":"root","wo-Arab":"root","yo-Arab":"root","yue-Hans":"root","zh-Hant":"root","zh-Hant-MO":"zh-Hant-HK"};t.default=i,e.exports=t.default},function(e,t,n){t.default=void 0;var i="-",o=function(e,t){var n=e[t];return n?"root"!==n&&n:t.substr(0,t.lastIndexOf(i))};t.default=o,e.exports=t.default},function(e,t,n){t.default=void 0;var i={af:"\xa4#,##0.00;(\xa4#,##0.00)","af-NA":"\xa4#,##0.00;(\xa4#,##0.00)",agq:"#,##0.00\xa4",ak:"\xa4#,##0.00",am:"\xa4#,##0.00;(\xa4#,##0.00)",ar:"\xa4#,##0.00;(\xa4#,##0.00)","ar-AE":"\xa4#,##0.00;(\xa4#,##0.00)","ar-BH":"\xa4#,##0.00;(\xa4#,##0.00)","ar-DJ":"\xa4#,##0.00;(\xa4#,##0.00)","ar-DZ":"\xa4#,##0.00;(\xa4#,##0.00)","ar-EG":"\xa4#,##0.00;(\xa4#,##0.00)","ar-EH":"\xa4#,##0.00;(\xa4#,##0.00)","ar-ER":"\xa4#,##0.00;(\xa4#,##0.00)","ar-IL":"\xa4#,##0.00;(\xa4#,##0.00)","ar-IQ":"\xa4#,##0.00;(\xa4#,##0.00)","ar-JO":"\xa4#,##0.00;(\xa4#,##0.00)","ar-KM":"\xa4#,##0.00;(\xa4#,##0.00)","ar-KW":"\xa4#,##0.00;(\xa4#,##0.00)","ar-LB":"\xa4#,##0.00;(\xa4#,##0.00)","ar-LY":"\xa4#,##0.00;(\xa4#,##0.00)","ar-MA":"\xa4#,##0.00;(\xa4#,##0.00)","ar-MR":"\xa4#,##0.00;(\xa4#,##0.00)","ar-OM":"\xa4#,##0.00;(\xa4#,##0.00)","ar-PS":"\xa4#,##0.00;(\xa4#,##0.00)","ar-QA":"\xa4#,##0.00;(\xa4#,##0.00)","ar-SA":"\xa4#,##0.00;(\xa4#,##0.00)","ar-SD":"\xa4#,##0.00;(\xa4#,##0.00)","ar-SO":"\xa4#,##0.00;(\xa4#,##0.00)","ar-SS":"\xa4#,##0.00;(\xa4#,##0.00)","ar-SY":"\xa4#,##0.00;(\xa4#,##0.00)","ar-TD":"\xa4#,##0.00;(\xa4#,##0.00)","ar-TN":"\xa4#,##0.00;(\xa4#,##0.00)","ar-YE":"\xa4#,##0.00;(\xa4#,##0.00)",as:"\xa4\xa0#,##,##0.00",asa:"#,##0.00\xa0\xa4",ast:"#,##0.00\xa0\xa4",az:"#,##0.00\xa0\xa4","az-Cyrl":"#,##0.00\xa0\xa4","az-Latn":"#,##0.00\xa0\xa4",bas:"#,##0.00\xa0\xa4",be:"#,##0.00\xa0\xa4",bem:"\xa4#,##0.00;(\xa4#,##0.00)",bez:"#,##0.00\xa4",bg:"0.00\xa0\xa4;(0.00\xa0\xa4)",bm:"\xa4#,##0.00;(\xa4#,##0.00)",bn:"#,##,##0.00\xa4;(#,##,##0.00\xa4)","bn-IN":"#,##,##0.00\xa4;(#,##,##0.00\xa4)",bo:"\xa4\xa0#,##0.00","bo-IN":"\xa4\xa0#,##0.00",br:"#,##0.00\xa0\xa4",brx:"\xa4\xa0#,##,##0.00",bs:"#,##0.00\xa0\xa4","bs-Cyrl":"#,##0.00\xa0\xa4","bs-Latn":"#,##0.00\xa0\xa4",ca:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","ca-AD":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","ca-ES-VALENCIA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","ca-FR":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","ca-IT":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",ccp:"#,##,##0.00\xa4;(#,##,##0.00\xa4)","ccp-IN":"#,##,##0.00\xa4;(#,##,##0.00\xa4)",ce:"#,##0.00\xa0\xa4",ceb:"\xa4#,##0.00;(\xa4#,##0.00)",cgg:"\xa4#,##0.00",chr:"\xa4#,##0.00;(\xa4#,##0.00)",ckb:"\xa4\xa0#,##0.00","ckb-IR":"\xa4\xa0#,##0.00",cs:"#,##0.00\xa0\xa4",cy:"\xa4#,##0.00;(\xa4#,##0.00)",da:"#,##0.00\xa0\xa4","da-GL":"#,##0.00\xa0\xa4",dav:"\xa4#,##0.00;(\xa4#,##0.00)",de:"#,##0.00\xa0\xa4","de-AT":"#,##0.00\xa0\xa4","de-BE":"#,##0.00\xa0\xa4","de-CH":"#,##0.00\xa0\xa4","de-IT":"#,##0.00\xa0\xa4","de-LI":"#,##0.00\xa0\xa4","de-LU":"#,##0.00\xa0\xa4",dje:"#,##0.00\xa4",doi:"\xa4#,##0.00",dsb:"#,##0.00\xa0\xa4",dua:"#,##0.00\xa0\xa4",dyo:"#,##0.00\xa0\xa4",dz:"\xa4#,##,##0.00",ebu:"\xa4#,##0.00;(\xa4#,##0.00)",ee:"\xa4#,##0.00;(\xa4#,##0.00)","ee-TG":"\xa4#,##0.00;(\xa4#,##0.00)",el:"#,##0.00\xa0\xa4","el-CY":"#,##0.00\xa0\xa4",en:"\xa4#,##0.00;(\xa4#,##0.00)","en-001":"\xa4#,##0.00;(\xa4#,##0.00)","en-150":"#,##0.00\xa0\xa4","en-AE":"\xa4#,##0.00;(\xa4#,##0.00)","en-AG":"\xa4#,##0.00;(\xa4#,##0.00)","en-AI":"\xa4#,##0.00;(\xa4#,##0.00)","en-AS":"\xa4#,##0.00;(\xa4#,##0.00)","en-AT":"\xa4\xa0#,##0.00","en-AU":"\xa4#,##0.00;(\xa4#,##0.00)","en-BB":"\xa4#,##0.00;(\xa4#,##0.00)","en-BE":"#,##0.00\xa0\xa4","en-BI":"\xa4#,##0.00;(\xa4#,##0.00)","en-BM":"\xa4#,##0.00;(\xa4#,##0.00)","en-BS":"\xa4#,##0.00;(\xa4#,##0.00)","en-BW":"\xa4#,##0.00;(\xa4#,##0.00)","en-BZ":"\xa4#,##0.00;(\xa4#,##0.00)","en-CA":"\xa4#,##0.00;(\xa4#,##0.00)","en-CC":"\xa4#,##0.00;(\xa4#,##0.00)","en-CH":"\xa4\xa0#,##0.00;\xa4-#,##0.00","en-CK":"\xa4#,##0.00;(\xa4#,##0.00)","en-CM":"\xa4#,##0.00;(\xa4#,##0.00)","en-CX":"\xa4#,##0.00;(\xa4#,##0.00)","en-CY":"\xa4#,##0.00;(\xa4#,##0.00)","en-DE":"#,##0.00\xa0\xa4","en-DG":"\xa4#,##0.00;(\xa4#,##0.00)","en-DK":"#,##0.00\xa0\xa4","en-DM":"\xa4#,##0.00;(\xa4#,##0.00)","en-ER":"\xa4#,##0.00;(\xa4#,##0.00)","en-FI":"#,##0.00\xa0\xa4","en-FJ":"\xa4#,##0.00;(\xa4#,##0.00)","en-FK":"\xa4#,##0.00;(\xa4#,##0.00)","en-FM":"\xa4#,##0.00;(\xa4#,##0.00)","en-GB":"\xa4#,##0.00;(\xa4#,##0.00)","en-GD":"\xa4#,##0.00;(\xa4#,##0.00)","en-GG":"\xa4#,##0.00;(\xa4#,##0.00)","en-GH":"\xa4#,##0.00;(\xa4#,##0.00)","en-GI":"\xa4#,##0.00;(\xa4#,##0.00)","en-GM":"\xa4#,##0.00;(\xa4#,##0.00)","en-GU":"\xa4#,##0.00;(\xa4#,##0.00)","en-GY":"\xa4#,##0.00;(\xa4#,##0.00)","en-HK":"\xa4#,##0.00;(\xa4#,##0.00)","en-IE":"\xa4#,##0.00;(\xa4#,##0.00)","en-IL":"\xa4#,##0.00;(\xa4#,##0.00)","en-IM":"\xa4#,##0.00;(\xa4#,##0.00)","en-IN":"\xa4#,##0.00;(\xa4#,##0.00)","en-IO":"\xa4#,##0.00;(\xa4#,##0.00)","en-JE":"\xa4#,##0.00;(\xa4#,##0.00)","en-JM":"\xa4#,##0.00;(\xa4#,##0.00)","en-KE":"\xa4#,##0.00;(\xa4#,##0.00)","en-KI":"\xa4#,##0.00;(\xa4#,##0.00)","en-KN":"\xa4#,##0.00;(\xa4#,##0.00)","en-KY":"\xa4#,##0.00;(\xa4#,##0.00)","en-LC":"\xa4#,##0.00;(\xa4#,##0.00)","en-LR":"\xa4#,##0.00;(\xa4#,##0.00)","en-LS":"\xa4#,##0.00;(\xa4#,##0.00)","en-MG":"\xa4#,##0.00;(\xa4#,##0.00)","en-MH":"\xa4#,##0.00;(\xa4#,##0.00)","en-MO":"\xa4#,##0.00;(\xa4#,##0.00)","en-MP":"\xa4#,##0.00;(\xa4#,##0.00)","en-MS":"\xa4#,##0.00;(\xa4#,##0.00)","en-MT":"\xa4#,##0.00;(\xa4#,##0.00)","en-MU":"\xa4#,##0.00;(\xa4#,##0.00)","en-MW":"\xa4#,##0.00;(\xa4#,##0.00)","en-MY":"\xa4#,##0.00;(\xa4#,##0.00)","en-NA":"\xa4#,##0.00;(\xa4#,##0.00)","en-NF":"\xa4#,##0.00;(\xa4#,##0.00)","en-NG":"\xa4#,##0.00;(\xa4#,##0.00)","en-NL":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","en-NR":"\xa4#,##0.00;(\xa4#,##0.00)","en-NU":"\xa4#,##0.00;(\xa4#,##0.00)","en-NZ":"\xa4#,##0.00;(\xa4#,##0.00)","en-PG":"\xa4#,##0.00;(\xa4#,##0.00)","en-PH":"\xa4#,##0.00;(\xa4#,##0.00)","en-PK":"\xa4#,##0.00;(\xa4#,##0.00)","en-PN":"\xa4#,##0.00;(\xa4#,##0.00)","en-PR":"\xa4#,##0.00;(\xa4#,##0.00)","en-PW":"\xa4#,##0.00;(\xa4#,##0.00)","en-RW":"\xa4#,##0.00;(\xa4#,##0.00)","en-SB":"\xa4#,##0.00;(\xa4#,##0.00)","en-SC":"\xa4#,##0.00;(\xa4#,##0.00)","en-SD":"\xa4#,##0.00;(\xa4#,##0.00)","en-SE":"#,##0.00\xa0\xa4","en-SG":"\xa4#,##0.00;(\xa4#,##0.00)","en-SH":"\xa4#,##0.00;(\xa4#,##0.00)","en-SI":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","en-SL":"\xa4#,##0.00;(\xa4#,##0.00)","en-SS":"\xa4#,##0.00;(\xa4#,##0.00)","en-SX":"\xa4#,##0.00;(\xa4#,##0.00)","en-SZ":"\xa4#,##0.00;(\xa4#,##0.00)","en-TC":"\xa4#,##0.00;(\xa4#,##0.00)","en-TK":"\xa4#,##0.00;(\xa4#,##0.00)","en-TO":"\xa4#,##0.00;(\xa4#,##0.00)","en-TT":"\xa4#,##0.00;(\xa4#,##0.00)","en-TV":"\xa4#,##0.00;(\xa4#,##0.00)","en-TZ":"\xa4#,##0.00;(\xa4#,##0.00)","en-UG":"\xa4#,##0.00;(\xa4#,##0.00)","en-UM":"\xa4#,##0.00;(\xa4#,##0.00)","en-US-POSIX":"\xa4#,##0.00;(\xa4#,##0.00)","en-VC":"\xa4#,##0.00;(\xa4#,##0.00)","en-VG":"\xa4#,##0.00;(\xa4#,##0.00)","en-VI":"\xa4#,##0.00;(\xa4#,##0.00)","en-VU":"\xa4#,##0.00;(\xa4#,##0.00)","en-WS":"\xa4#,##0.00;(\xa4#,##0.00)","en-ZA":"\xa4#,##0.00;(\xa4#,##0.00)","en-ZM":"\xa4#,##0.00;(\xa4#,##0.00)","en-ZW":"\xa4#,##0.00;(\xa4#,##0.00)",eo:"\xa4\xa0#,##0.00",es:"#,##0.00\xa0\xa4","es-419":"\xa4#,##0.00","es-AR":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","es-BO":"\xa4#,##0.00","es-BR":"\xa4#,##0.00","es-BZ":"\xa4#,##0.00","es-CL":"\xa4#,##0.00","es-CO":"\xa4#,##0.00","es-CR":"\xa4#,##0.00","es-CU":"\xa4#,##0.00","es-DO":"\xa4#,##0.00;(\xa4#,##0.00)","es-EA":"#,##0.00\xa0\xa4","es-EC":"\xa4#,##0.00","es-GQ":"#,##0.00\xa0\xa4","es-GT":"\xa4#,##0.00","es-HN":"\xa4#,##0.00","es-IC":"#,##0.00\xa0\xa4","es-MX":"\xa4#,##0.00","es-NI":"\xa4#,##0.00","es-PA":"\xa4#,##0.00","es-PE":"\xa4#,##0.00","es-PH":"#,##0.00\xa0\xa4","es-PR":"\xa4#,##0.00","es-PY":"\xa4#,##0.00","es-SV":"\xa4#,##0.00","es-US":"\xa4#,##0.00","es-UY":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","es-VE":"\xa4#,##0.00",et:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",eu:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",ewo:"#,##0.00\xa0\xa4",fa:"\u200e\xa4\xa0#,##0.00;\u200e(\xa4\xa0#,##0.00)","fa-AF":"\xa4\xa0#,##0.00;\u200e(\xa4\xa0#,##0.00)",ff:"#,##0.00\xa0\xa4","ff-Adlm":"\xa4\xa0#,##0.00","ff-Adlm-BF":"\xa4\xa0#,##0.00","ff-Adlm-CM":"\xa4\xa0#,##0.00","ff-Adlm-GH":"\xa4\xa0#,##0.00","ff-Adlm-GM":"\xa4\xa0#,##0.00","ff-Adlm-GW":"\xa4\xa0#,##0.00","ff-Adlm-LR":"\xa4\xa0#,##0.00","ff-Adlm-MR":"\xa4\xa0#,##0.00","ff-Adlm-NE":"\xa4\xa0#,##0.00","ff-Adlm-NG":"\xa4\xa0#,##0.00","ff-Adlm-SL":"\xa4\xa0#,##0.00","ff-Adlm-SN":"\xa4\xa0#,##0.00","ff-Latn":"#,##0.00\xa0\xa4","ff-Latn-BF":"#,##0.00\xa0\xa4","ff-Latn-CM":"#,##0.00\xa0\xa4","ff-Latn-GH":"#,##0.00\xa0\xa4","ff-Latn-GM":"#,##0.00\xa0\xa4","ff-Latn-GN":"#,##0.00\xa0\xa4","ff-Latn-GW":"#,##0.00\xa0\xa4","ff-Latn-LR":"#,##0.00\xa0\xa4","ff-Latn-MR":"#,##0.00\xa0\xa4","ff-Latn-NE":"#,##0.00\xa0\xa4","ff-Latn-NG":"#,##0.00\xa0\xa4","ff-Latn-SL":"#,##0.00\xa0\xa4",fi:"#,##0.00\xa0\xa4",fil:"\xa4#,##0.00;(\xa4#,##0.00)",fo:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fo-DK":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",fr:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-BE":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-BF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-BI":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-BJ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-BL":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CD":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CG":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CH":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CI":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-CM":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-DJ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-DZ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-GA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-GF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-GN":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-GP":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-GQ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-HT":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-KM":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-LU":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MC":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MG":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",
"fr-ML":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MQ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MR":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-MU":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-NC":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-NE":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-PF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-PM":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-RE":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-RW":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-SC":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-SN":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-SY":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-TD":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-TG":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-TN":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-VU":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-WF":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","fr-YT":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",fur:"\xa4\xa0#,##0.00",fy:"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)",ga:"\xa4#,##0.00;(\xa4#,##0.00)","ga-GB":"\xa4#,##0.00;(\xa4#,##0.00)",gd:"\xa4#,##0.00;(\xa4#,##0.00)",gl:"#,##0.00\xa0\xa4",gsw:"#,##0.00\xa0\xa4","gsw-FR":"#,##0.00\xa0\xa4","gsw-LI":"#,##0.00\xa0\xa4",gu:"\xa4#,##,##0.00;(\xa4#,##,##0.00)",guz:"\xa4#,##0.00;(\xa4#,##0.00)",gv:"\xa4#,##0.00",ha:"\xa4\xa0#,##0.00","ha-GH":"\xa4\xa0#,##0.00","ha-NE":"\xa4\xa0#,##0.00",haw:"\xa4#,##0.00;(\xa4#,##0.00)",he:"#,##0.00\xa0\xa4",hi:"\xa4#,##,##0.00",hr:"#,##0.00\xa0\xa4","hr-BA":"#,##0.00\xa0\xa4",hsb:"#,##0.00\xa0\xa4",hu:"#,##0.00\xa0\xa4",hy:"#,##0.00\xa0\xa4",ia:"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)",id:"\xa4#,##0.00",ig:"\xa4#,##0.00;(\xa4#,##0.00)",ii:"\xa4\xa0#,##0.00",is:"#,##0.00\xa0\xa4",it:"#,##0.00\xa0\xa4","it-CH":"#,##0.00\xa0\xa4","it-SM":"#,##0.00\xa0\xa4","it-VA":"#,##0.00\xa0\xa4",ja:"\xa4#,##0.00;(\xa4#,##0.00)",jgo:"\xa4\xa0#,##0.00",jmc:"\xa4#,##0.00",jv:"\xa4\xa0#,##0.00",ka:"#,##0.00\xa0\xa4",kab:"#,##0.00\xa4",kam:"\xa4#,##0.00;(\xa4#,##0.00)",kde:"\xa4#,##0.00;(\xa4#,##0.00)",kea:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",khq:"#,##0.00\xa4",ki:"\xa4#,##0.00;(\xa4#,##0.00)",kk:"#,##0.00\xa0\xa4",kkj:"\xa4\xa0#,##0.00",kl:"\xa4#,##0.00;\xa4-#,##0.00",kln:"\xa4#,##0.00;(\xa4#,##0.00)",km:"#,##0.00\xa4;(#,##0.00\xa4)",kn:"\xa4#,##0.00;(\xa4#,##0.00)",ko:"\xa4#,##0.00;(\xa4#,##0.00)","ko-KP":"\xa4#,##0.00;(\xa4#,##0.00)",kok:"\xa4#,##0.00;(\xa4#,##0.00)",ks:"\xa4\xa0#,##,##0.00","ks-Arab":"\xa4\xa0#,##,##0.00",ksb:"#,##0.00\xa4",ksf:"#,##0.00\xa0\xa4",ksh:"#,##0.00\xa0\xa4",ku:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",kw:"\xa4#,##0.00",ky:"#,##0.00\xa0\xa4",lag:"\xa4\xa0#,##0.00",lb:"#,##0.00\xa0\xa4",lg:"#,##0.00\xa4",lkt:"\xa4\xa0#,##0.00",ln:"#,##0.00\xa0\xa4","ln-AO":"#,##0.00\xa0\xa4","ln-CF":"#,##0.00\xa0\xa4","ln-CG":"#,##0.00\xa0\xa4",lo:"\xa4#,##0.00;\xa4-#,##0.00",lrc:"\xa4\xa0#,##0.00","lrc-IQ":"\xa4\xa0#,##0.00",lt:"#,##0.00\xa0\xa4",lu:"#,##0.00\xa4",luo:"#,##0.00\xa4",luy:"\xa4#,##0.00;\xa4-\xa0#,##0.00",lv:"#,##0.00\xa0\xa4",mai:"\xa4\xa0#,##0.00",mas:"\xa4#,##0.00;(\xa4#,##0.00)","mas-TZ":"\xa4#,##0.00;(\xa4#,##0.00)",mer:"\xa4#,##0.00;(\xa4#,##0.00)",mfe:"\xa4\xa0#,##0.00",mg:"\xa4#,##0.00",mgh:"\xa4\xa0#,##0.00",mgo:"\xa4\xa0#,##0.00",mi:"\xa4\xa0#,##0.00",mk:"#,##0.00\xa0\xa4",ml:"\xa4#,##0.00;(\xa4#,##0.00)",mn:"\xa4\xa0#,##0.00",mni:"\xa4\xa0#,##0.00","mni-Beng":"\xa4\xa0#,##0.00",mr:"\xa4#,##0.00;(\xa4#,##0.00)",ms:"\xa4#,##0.00;(\xa4#,##0.00)","ms-BN":"\xa4#,##0.00;(\xa4#,##0.00)","ms-ID":"\xa4#,##0.00","ms-SG":"\xa4#,##0.00;(\xa4#,##0.00)",mt:"\xa4#,##0.00",mua:"\xa4#,##0.00;(\xa4#,##0.00)",my:"\xa4\xa0#,##0.00",mzn:"\xa4\xa0#,##0.00",naq:"\xa4#,##0.00",nb:"\xa4\xa0#,##0.00","nb-SJ":"\xa4\xa0#,##0.00",nd:"\xa4#,##0.00;(\xa4#,##0.00)",nds:"\xa4\xa0#,##0.00","nds-NL":"\xa4\xa0#,##0.00",ne:"\xa4\xa0#,##,##0.00","ne-IN":"\xa4\xa0#,##,##0.00",nl:"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-AW":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-BE":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-BQ":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-CW":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-SR":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)","nl-SX":"\xa4\xa0#,##0.00;(\xa4\xa0#,##0.00)",nmg:"#,##0.00\xa0\xa4",nn:"#,##0.00\xa0\xa4",nnh:"\xa4\xa0#,##0.00",nus:"\xa4#,##0.00;(\xa4#,##0.00)",nyn:"\xa4#,##0.00",om:"\xa4#,##0.00","om-KE":"\xa4#,##0.00",or:"\xa4#,##0.00;(\xa4#,##0.00)",os:"\xa4\xa0#,##0.00","os-RU":"\xa4\xa0#,##0.00",pa:"\xa4\xa0#,##0.00","pa-Arab":"\xa4\xa0#,##0.00","pa-Guru":"\xa4\xa0#,##0.00",pcm:"\xa4#,##0.00",pl:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",ps:"\xa4#,##0.00;(\xa4#,##0.00)","ps-PK":"\xa4#,##0.00;(\xa4#,##0.00)",pt:"\xa4\xa0#,##0.00","pt-AO":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-CH":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-CV":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-GQ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-GW":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-LU":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-MO":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-MZ":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-PT":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-ST":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","pt-TL":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",qu:"\xa4\xa0#,##0.00","qu-BO":"\xa4\xa0#,##0.00","qu-EC":"\xa4\xa0#,##0.00",rm:"#,##0.00\xa0\xa4",rn:"#,##0.00\xa4",ro:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","ro-MD":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",rof:"\xa4#,##0.00",root:"\xa4\xa0#,##0.00",ru:"#,##0.00\xa0\xa4","ru-BY":"#,##0.00\xa0\xa4","ru-KG":"#,##0.00\xa0\xa4","ru-KZ":"#,##0.00\xa0\xa4","ru-MD":"#,##0.00\xa0\xa4","ru-UA":"#,##0.00\xa0\xa4",rw:"\xa4\xa0#,##0.00",rwk:"#,##0.00\xa4",sa:"\xa4\xa0#,##0.00",sah:"#,##0.00\xa0\xa4",saq:"\xa4#,##0.00;(\xa4#,##0.00)",sat:"\xa4\xa0#,##0.00","sat-Olck":"\xa4\xa0#,##0.00",sbp:"#,##0.00\xa4",sd:"\xa4\xa0#,##0.00","sd-Arab":"\xa4\xa0#,##0.00","sd-Deva":"\xa4\xa0#,##0.00",se:"#,##0.00\xa0\xa4","se-FI":"#,##0.00\xa0\xa4","se-SE":"#,##0.00\xa0\xa4",seh:"#,##0.00\xa4",ses:"#,##0.00\xa4",sg:"\xa4#,##0.00;\xa4-#,##0.00",shi:"#,##0.00\xa4","shi-Latn":"#,##0.00\xa4","shi-Tfng":"#,##0.00\xa4",si:"\xa4#,##0.00;(\xa4#,##0.00)",sk:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",sl:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",smn:"#,##0.00\xa0\xa4",sn:"\xa4#,##0.00;(\xa4#,##0.00)",so:"\xa4#,##0.00;(\xa4#,##0.00)","so-DJ":"\xa4#,##0.00;(\xa4#,##0.00)","so-ET":"\xa4#,##0.00;(\xa4#,##0.00)","so-KE":"\xa4#,##0.00;(\xa4#,##0.00)",sq:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sq-MK":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sq-XK":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",sr:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Cyrl":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Cyrl-BA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Cyrl-ME":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Cyrl-XK":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Latn":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Latn-BA":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Latn-ME":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)","sr-Latn-XK":"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",su:"\xa4#,##0.00","su-Latn":"\xa4#,##0.00",sv:"#,##0.00\xa0\xa4","sv-AX":"#,##0.00\xa0\xa4","sv-FI":"#,##0.00\xa0\xa4",sw:"\xa4\xa0#,##0.00","sw-CD":"\xa4\xa0#,##0.00","sw-KE":"\xa4\xa0#,##0.00","sw-UG":"\xa4\xa0#,##0.00",ta:"\xa4#,##0.00;(\xa4#,##0.00)","ta-LK":"\xa4#,##0.00;(\xa4#,##0.00)","ta-MY":"\xa4#,##0.00;(\xa4#,##0.00)","ta-SG":"\xa4#,##0.00;(\xa4#,##0.00)",te:"\xa4#,##0.00;(\xa4#,##0.00)",teo:"\xa4#,##0.00;(\xa4#,##0.00)","teo-KE":"\xa4#,##0.00;(\xa4#,##0.00)",tg:"#,##0.00\xa0\xa4",th:"\xa4#,##0.00;(\xa4#,##0.00)",ti:"\xa4#,##0.00","ti-ER":"\xa4#,##0.00",tk:"#,##0.00\xa0\xa4",to:"\xa4\xa0#,##0.00",tr:"\xa4#,##0.00;(\xa4#,##0.00)","tr-CY":"\xa4#,##0.00;(\xa4#,##0.00)",tt:"#,##0.00\xa0\xa4",twq:"#,##0.00\xa4",tzm:"#,##0.00\xa0\xa4",ug:"\xa4#,##0.00;(\xa4#,##0.00)",uk:"#,##0.00\xa0\xa4",ur:"\xa4#,##0.00;(\xa4#,##0.00)","ur-IN":"\xa4#,##0.00;(\xa4#,##0.00)",uz:"#,##0.00\xa0\xa4","uz-Arab":"\xa4\xa0#,##0.00","uz-Cyrl":"#,##0.00\xa0\xa4","uz-Latn":"#,##0.00\xa0\xa4",vai:"\xa4#,##0.00;(\xa4#,##0.00)","vai-Latn":"\xa4#,##0.00;(\xa4#,##0.00)","vai-Vaii":"\xa4#,##0.00;(\xa4#,##0.00)",vi:"#,##0.00\xa0\xa4",vun:"\xa4#,##0.00",wae:"\xa4\xa0#,##0.00",wo:"\xa4\xa0#,##0.00",xh:"\xa4#,##0.00",xog:"#,##0.00\xa0\xa4",yav:"#,##0.00\xa0\xa4;(#,##0.00\xa0\xa4)",yi:"\xa4\xa0#,##0.00",yo:"\xa4#,##0.00;(\xa4#,##0.00)","yo-BJ":"\xa4#,##0.00;(\xa4#,##0.00)",yue:"\xa4#,##0.00;(\xa4#,##0.00)","yue-Hans":"\xa4#,##0.00;(\xa4#,##0.00)","yue-Hant":"\xa4#,##0.00;(\xa4#,##0.00)",zgh:"#,##0.00\xa4",zh:"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hans":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hans-HK":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hans-MO":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hans-SG":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hant":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hant-HK":"\xa4#,##0.00;(\xa4#,##0.00)","zh-Hant-MO":"\xa4#,##0.00;(\xa4#,##0.00)",zu:"\xa4#,##0.00;(\xa4#,##0.00)"};t.default=i,e.exports=t.default},function(e,t,n){t.default=void 0;var i={"af-NA":1,agq:1,ak:1,ar:6,"ar-EH":1,"ar-ER":1,"ar-KM":1,"ar-LB":1,"ar-MA":1,"ar-MR":1,"ar-PS":1,"ar-SO":1,"ar-SS":1,"ar-TD":1,"ar-TN":1,asa:1,ast:1,az:1,"az-Cyrl":1,bas:1,be:1,bem:1,bez:1,bg:1,bm:1,br:1,bs:1,"bs-Cyrl":1,ca:1,ce:1,cgg:1,ckb:6,cs:1,cy:1,da:1,de:1,dje:1,dsb:1,dua:1,dyo:1,ee:1,el:1,"en-001":1,"en-AE":6,"en-BI":1,"en-MP":1,"en-SD":6,eo:1,es:1,et:1,eu:1,ewo:1,fa:6,ff:1,"ff-Adlm":1,fi:1,fo:1,fr:1,"fr-DJ":6,"fr-DZ":6,"fr-SY":6,fur:1,fy:1,ga:1,gd:1,gl:1,gsw:1,gv:1,ha:1,hr:1,hsb:1,hu:1,hy:1,ia:1,ig:1,is:1,it:1,jgo:1,jmc:1,ka:1,kab:6,kde:1,kea:1,khq:1,kk:1,kkj:1,kl:1,"ko-KP":1,ksb:1,ksf:1,ksh:1,ku:1,kw:1,ky:1,lag:1,lb:1,lg:1,ln:1,lrc:6,lt:1,lu:1,lv:1,"mas-TZ":1,mfe:1,mg:1,mgo:1,mi:1,mk:1,mn:1,ms:1,mua:1,mzn:6,naq:1,nb:1,nds:1,nl:1,nmg:1,nn:1,nnh:1,nus:1,nyn:1,os:1,pcm:1,pl:1,ps:6,"pt-AO":1,"pt-CH":1,"pt-CV":1,"pt-GQ":1,"pt-GW":1,"pt-LU":1,"pt-ST":1,"pt-TL":1,"qu-BO":1,"qu-EC":1,rm:1,rn:1,ro:1,rof:1,ru:1,rw:1,rwk:1,sah:1,sbp:1,se:1,ses:1,sg:1,shi:1,"shi-Latn":1,si:1,sk:1,sl:1,smn:1,so:1,"so-DJ":6,sq:1,sr:1,"sr-Latn":1,sv:1,sw:1,"ta-LK":1,"ta-MY":1,teo:1,tg:1,"ti-ER":1,tk:1,to:1,tr:1,tt:1,twq:1,tzm:1,uk:1,uz:1,"uz-Arab":6,"uz-Cyrl":1,vai:1,"vai-Latn":1,vi:1,vun:1,wae:1,wo:1,xog:1,yav:1,yi:1,yo:1,zgh:1};t.default=i,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){return d(t)(e).replace(l,"")}t.default=void 0;var r=n(0),s=i(n(82)),l=/[\u200E\u200F]/g,u=function(e){return function(t){if(!e.timeZoneName){var n=t.getFullYear(),i=String(n).length<3,o=400,s=i?n+o:n,l=new Date(Date.UTC(s,t.getMonth(),t.getDate(),t.getHours(),t.getMinutes(),t.getSeconds()));i&&l.setFullYear(n);var u=(0,r.extend)({timeZone:"UTC"},e);return a(l,u)}return a(t,e)}},c={},d=function(e){var t=s.default.locale()+"/"+JSON.stringify(e);return c[t]||(c[t]=new Intl.DateTimeFormat(s.default.locale(),e).format),c[t]},h=function(e){return new Intl.NumberFormat(s.default.locale()).format(e)},f=function(){var e={};return function(t){if(!(t in e)){if("0"===h(0))return e[t]=!1,!1;e[t]={};for(var n=0;n<10;++n)e[t][h(n)]=n}return e[t]}}(),p=function(e){var t=f(s.default.locale());return t?e.split("").map(function(e){return e in t?String(t[e]):e}).join(""):e},g=function(e){return e.replace(/(\D)0+(\d)/g,"$1$2")},_=function(e,t){return g(e)===g(t)},m=function(e){return e.replace("d\u2019","de ")},v={day:{day:"numeric"},dayofweek:{weekday:"long"},longdate:{weekday:"long",year:"numeric",month:"long",day:"numeric"},longdatelongtime:{weekday:"long",year:"numeric",month:"long",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"},longtime:{hour:"numeric",minute:"numeric",second:"numeric"},month:{month:"long"},monthandday:{month:"long",day:"numeric"},monthandyear:{year:"numeric",month:"long"},shortdate:{},shorttime:{hour:"numeric",minute:"numeric"},shortyear:{year:"2-digit"},year:{year:"numeric"}};Object.defineProperty(v,"shortdateshorttime",{get:function(){var e=Intl.DateTimeFormat(s.default.locale()).resolvedOptions();return{year:e.year,month:e.month,day:e.day,hour:"numeric",minute:"numeric"}}});var y=function(e){return"string"==typeof e&&v[e.toLowerCase()]},b={standalone:function(e,t){var n=new Date(1999,e,13,1),i=u({month:t})(n);return i},format:function(e,t){var n=new Date(0,e,13,1),i=m(u({day:"numeric",month:t})(n)),o=i.split(" ").filter(function(e){return e.indexOf("13")<0});return 1===o.length?o[0]:2===o.length?o[0].length>o[1].length?o[0]:o[1]:b.standalone(e,t)}},x={engine:function(){return"intl"},getMonthNames:function(e,t){var n={wide:"long",abbreviated:"short",narrow:"narrow"},i=n[e||"wide"];return t="format"===t?t:"standalone",Array.apply(null,new Array(12)).map(function(e,n){return b[t](n,i)})},getDayNames:function(e){var t={wide:"long",abbreviated:"short","short":"narrow",narrow:"narrow"},n=function(e){return Array.apply(null,new Array(7)).map(function(t,n){return u({weekday:e})(new Date(0,0,n))})},i=n(t[e||"wide"]);return i},getPeriodNames:function(){var e=u({hour:"numeric",hour12:!0});return[1,13].map(function(t){var n=h(1),i=e(new Date(0,0,1,t)).split(n);if(2!==i.length)return"";var o=i[0].length>i[1].length?i[0]:i[1];return o.trim()})},format:function(e,t){if(e){if(!t)return e;"function"==typeof t||t.formatter||(t=t.type||t);var n=y(t);if(n)return u(n)(e);var i=o(t);return t.formatter||"function"===i||"string"===i?this.callBase.apply(this,arguments):u(t)(e)}},parse:function(e,t){var n,i=this;return t&&!t.parser&&"string"==typeof e&&(e=m(e),n=function(e){return m(i.format(e,t))}),this.callBase(e,n||t)},_parseDateBySimpleFormat:function(e,t){var n=this;e=p(e);var i=this.getFormatParts(t),o=e.split(/\D+/).filter(function(e){return e.length>0});if(i.length===o.length){var a=this._generateDateArgs(i,o),r=function(e,t){var n=t?12:0;return new Date(e.year,e.month,e.day,(e.hours+n)%24,e.minutes,e.seconds)},s=function(i){var o=r(a,i);if(_(p(n.format(o,t)),e))return o};return s(!1)||s(!0)}},_generateDateArgs:function(e,t){var n=new Date,i={year:n.getFullYear(),month:n.getMonth(),day:n.getDate(),hours:0,minutes:0,seconds:0};return e.forEach(function(e,n){var o=t[n],a=parseInt(o,10);"month"===e&&(a-=1),i[e]=a}),i},formatUsesMonthName:function(e){return"object"!==o(e)||e.type||e.format?this.callBase.apply(this,arguments):"long"===e.month},formatUsesDayName:function(e){return"object"!==o(e)||e.type||e.format?this.callBase.apply(this,arguments):"long"===e.weekday},getFormatParts:function(e){if("string"==typeof e)return this.callBase(e);var t=(0,r.extend)({},v[e.toLowerCase()]),n=new Date(2001,2,4,5,6,7),i=u(t)(n);i=p(i);var o=[{name:"year",value:1},{name:"month",value:3},{name:"day",value:4},{name:"hours",value:5},{name:"minutes",value:6},{name:"seconds",value:7}];return o.map(function(e){return{name:e.name,index:i.indexOf(e.value)}}).filter(function(e){return e.index>-1}).sort(function(e,t){return e.index-t.index}).map(function(e){return e.name})}};t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(1),l=i(n(83)),u=i(n(343)),c=i(n(244)),d=i(n(246)),h=i(n(346)),f=function(){function e(){o(this,e),this._cellFormatTags=[],this._fillTags=[],this._fontTags=[],this._numberFormatTags=[],this._fillTags.push(c.default.tryCreateTag({patternFill:{patternType:"none"}}))}return r(e,[{key:"registerCellFormat",value:function(e){var t,n=u.default.tryCreateTag(e,{registerFill:this.registerFill.bind(this),registerFont:this.registerFont.bind(this),registerNumberFormat:this.registerNumberFormat.bind(this)});if((0,s.isDefined)(n)){for(var i=0;i<this._cellFormatTags.length;i++)if(u.default.areEqual(this._cellFormatTags[i],n)){t=i;break}void 0===t&&(t=this._cellFormatTags.push(n)-1)}return t}},{key:"generateCellFormatsXml",value:function(){var e=this._cellFormatTags.map(function(e){return u.default.toXml(e)});return l.default.toXml("cellXfs",{count:e.length},e.join(""))}},{key:"registerFill",value:function(e){var t,n=c.default.tryCreateTag(e);if((0,s.isDefined)(n)){for(var i=0;i<this._fillTags.length;i++)if(c.default.areEqual(this._fillTags[i],n)){t=i;break}void 0===t&&(this._fillTags.length<2&&this._fillTags.push(c.default.tryCreateTag({patternFill:{patternType:"Gray125"}})),t=this._fillTags.push(n)-1)}return t}},{key:"generateFillsXml",value:function(){var e=this._fillTags.map(function(e){return c.default.toXml(e)});return l.default.toXml("fills",{count:e.length},e.join(""))}},{key:"registerFont",value:function(e){var t,n=d.default.tryCreateTag(e);if((0,s.isDefined)(n)){for(var i=0;i<this._fontTags.length;i++)if(d.default.areEqual(this._fontTags[i],n)){t=i;break}void 0===t&&(t=this._fontTags.push(n)-1)}return t}},{key:"generateFontsXml",value:function(){var e=this._fontTags.map(function(e){return d.default.toXml(e)});return l.default.toXml("fonts",{count:e.length},e.join(""))}},{key:"_convertNumberFormatIndexToId",value:function(e){var t=165;return t+e}},{key:"registerNumberFormat",value:function(e){var t,n=h.default.tryCreateTag(e);if((0,s.isDefined)(n)){for(var i=0;i<this._numberFormatTags.length;i++)if(h.default.areEqual(this._numberFormatTags[i],n)){t=this._numberFormatTags[i][h.default.ID_PROPERTY_NAME];break}void 0===t&&(n[h.default.ID_PROPERTY_NAME]=this._convertNumberFormatIndexToId(this._numberFormatTags.length),t=n[h.default.ID_PROPERTY_NAME],this._numberFormatTags.push(n))}return t}},{key:"generateNumberFormatsXml",value:function(){if(this._numberFormatTags.length>0){var e=this._numberFormatTags.map(function(e){return h.default.toXml(e)});return l.default.toXml("numFmts",{count:e.length},e.join(""))}return""}}],[{key:"copyCellFormat",value:function(e){return u.default.copy(e)}}]),e}();t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r=i(n(344)),s=i(n(244)),l=i(n(246)),u={tryCreateTag:function(e,t){var n=null;if((0,o.isDefined)(e)){var i;i="number"==typeof e.numberFormat?e.numberFormat:t.registerNumberFormat(e.numberFormat);var a=e.fill;(0,o.isDefined)(a)||(a=s.default.tryCreateFillFromSimpleFormat(e)),n={numberFormatId:i,alignment:r.default.tryCreateTag(e.alignment),fontId:t.registerFont(e.font),fillId:t.registerFill(a)},u.isEmpty(n)&&(n=null)}return n},copy:function(e){var t;return null===e?t=null:(0,o.isDefined)(e)&&(t={},void 0!==e.numberFormat&&(t.numberFormat=e.numberFormat),void 0!==e.fill?t.fill=s.default.copy(e.fill):s.default.copySimpleFormat(e,t),void 0!==e.alignment&&(t.alignment=r.default.copy(e.alignment)),void 0!==e.font&&(t.font=l.default.copy(e.font))),t},areEqual:function(e,t){return u.isEmpty(e)&&u.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.fontId===t.fontId&&e.numberFormatId===t.numberFormatId&&e.fillId===t.fillId&&r.default.areEqual(e.alignment,t.alignment)},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.fontId)&&!(0,o.isDefined)(e.numberFormatId)&&!(0,o.isDefined)(e.fillId)&&r.default.isEmpty(e.alignment)},toXml:function(e){var t,n=r.default.isEmpty(e.alignment);return(0,o.isDefined)(e.numberFormatId)&&(t=e.numberFormatId>0?1:0),a.default.toXml("xf",{xfId:0,applyAlignment:n?null:1,fontId:e.fontId,applyNumberFormat:t,fillId:e.fillId,numFmtId:e.numberFormatId},n?null:r.default.toXml(e.alignment))}},c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r={tryCreateTag:function(e){var t=null;return(0,o.isDefined)(e)&&(t={vertical:e.vertical,wrapText:e.wrapText,horizontal:e.horizontal},r.isEmpty(t)&&(t=null)),t},copy:function(e){var t=null;return(0,o.isDefined)(e)&&(t={},void 0!==e.horizontal&&(t.horizontal=e.horizontal),void 0!==e.vertical&&(t.vertical=e.vertical),void 0!==e.wrapText&&(t.wrapText=e.wrapText)),t},areEqual:function(e,t){return r.isEmpty(e)&&r.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.vertical===t.vertical&&e.wrapText===t.wrapText&&e.horizontal===t.horizontal},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.vertical)&&!(0,o.isDefined)(e.wrapText)&&!(0,o.isDefined)(e.horizontal)},toXml:function(e){return a.default.toXml("alignment",{vertical:e.vertical,wrapText:(0,o.isDefined)(e.wrapText)?Number(e.wrapText):void 0,horizontal:e.horizontal})}},s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r=i(n(245)),s={tryCreateTag:function(e){var t=null;return(0,o.isDefined)(e)&&(t={patternType:e.patternType,backgroundColor:r.default.tryCreateTag(e.backgroundColor),foregroundColor:r.default.tryCreateTag(e.foregroundColor)},s.isEmpty(t)&&(t=null)),t},copy:function(e){var t=null;return(0,o.isDefined)(e)&&(t={},void 0!==e.patternType&&(t.patternType=e.patternType),void 0!==e.backgroundColor&&(t.backgroundColor=r.default.copy(e.backgroundColor)),void 0!==e.foregroundColor&&(t.foregroundColor=r.default.copy(e.foregroundColor))),t},areEqual:function(e,t){return s.isEmpty(e)&&s.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.patternType===t.patternType&&r.default.areEqual(e.backgroundColor,t.backgroundColor)&&r.default.areEqual(e.foregroundColor,t.foregroundColor)},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.patternType)},toXml:function(e){var t=[(0,o.isDefined)(e.foregroundColor)?r.default.toXml("fgColor",e.foregroundColor):"",(0,o.isDefined)(e.backgroundColor)?r.default.toXml("bgColor",e.backgroundColor):""].join("");return a.default.toXml("patternFill",{patternType:e.patternType},t)}},l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(83)),r={ID_PROPERTY_NAME:"id",tryCreateTag:function(e){var t=null;return"string"==typeof e&&(t={formatCode:e},r.isEmpty(t)&&(t=null)),t},areEqual:function(e,t){return r.isEmpty(e)&&r.isEmpty(t)||(0,o.isDefined)(e)&&(0,o.isDefined)(t)&&e.formatCode===t.formatCode},isEmpty:function(e){return!(0,o.isDefined)(e)||!(0,o.isDefined)(e.formatCode)||""===e.formatCode},toXml:function(e){return a.default.toXml("numFmt",{numFmtId:e[r.ID_PROPERTY_NAME],formatCode:e.formatCode})}},s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return f.getData(e,t)}t.getData=o,t.svgCreator=void 0;var a=i(n(2)),r=i(n(76)),s=n(7),l=n(1),u=n(3),c=n(206),d=n(6),h=(0,s.getWindow)(),f={_markup:"",_imageArray:{},_imageDeferreds:[],_getBinaryFile:function(e,t){r.default.sendRequest({url:e,method:"GET",responseType:"arraybuffer"}).done(t).fail(function(){t(!1)})},_loadImages:function(){var e=this;(0,u.each)(e._imageArray,function(t){var n=new d.Deferred;e._imageDeferreds.push(n),e._getBinaryFile(t,function(i){if(!i)return delete e._imageArray[t],void n.resolve();var o,a="",r=new Uint8Array(i),s=r.byteLength;for(o=0;o<s;o++)a+=String.fromCharCode(r[o]);e._imageArray[t]="data:image/png;base64,"+h.btoa(a),n.resolve()})})},_parseImages:function(e){var t,n=this;"image"===e.tagName&&(t=(0,a.default)(e).attr("href")||(0,a.default)(e).attr("xlink:href"),n._imageArray[t]||(n._imageArray[t]="")),(0,u.each)(e.childNodes,function(e,t){n._parseImages(t)})},_prepareImages:function(e){return this._parseImages(e),this._loadImages(),d.when.apply(a.default,this._imageDeferreds)},getData:function(e,t){var n,i=this,o='<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>',r=(0,c.getSvgElement)(e),s=(0,a.default)(r);return n=o+(0,c.getSvgMarkup)(s.get(0),t.backgroundColor),i._prepareImages(r).then(function(){return(0,u.each)(i._imageArray,function(e,t){var i="href=['|\"]".concat(e,"['|\"]");n=n.replace(new RegExp(i,"gi"),'href="'.concat(t,'"'))}),(0,l.isFunction)(h.Blob)?i._getBlob(n):i._getBase64(n)})},_getBlob:function(e){return new h.Blob([e],{type:"image/svg+xml"})},_getBase64:function(e){return h.btoa(e)}};t.svgCreator=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(7),a=i(n(54)),r=(0,o.getWindow)(),s={getXhr:function(){return new r.XMLHttpRequest}},l=(0,a.default)(s);t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return v.replace("_main_",y(e+"",10)).replace("_info_",y(e+t+"",10)).replace("_image_",y(e+t+n+"",10))}function a(e,t){return l.imageCreator.getImageData(e,(0,c.extend)({},t,{format:"JPEG"})).then(function(e){var n=b(e,t,x()),i=(0,u.isFunction)(d.Blob)?w(n):C(n);return i})}t.getData=a;var r=i(n(160)),s=n(7),l=n(247),u=n(1),c=n(0),d=(0,s.getWindow)(),h="%PDF-1.3\r\n2 0 obj\r\n<</ProcSet[/PDF/ImageB/ImageC/ImageI]/XObject<</I0 5 0 R>>>>\r\nendobj\r\n4 0 obj\r\n<</Type/Pages/Kids[1 0 R]/Count 1>>\r\nendobj\r\n7 0 obj\r\n<</OpenAction[1 0 R /FitH null]/Type/Catalog/Pages 4 0 R/PageLayout/OneColumn>>\r\nendobj\r\n1 0 obj\r\n<</Type/Page/Resources 2 0 R/MediaBox[0 0 _width_ _height_]/Contents 3 0 R/Parent 4 0 R>>\r\nendobj\r\n",f="3 0 obj\r\n<</Length 52>>stream\r\n0.20 w\n0 G\nq _width_ 0 0 _height_ 0.00 0.00 cm /I0 Do Q\r\nendstream\r\nendobj\r\n",p="6 0 obj\r\n<</CreationDate _date_/Producer(DevExtreme _version_)>>\r\nendobj\r\n",g="5 0 obj\r\n<</Type/XObject/Subtype/Image/Width _width_/Height _height_/ColorSpace/DeviceRGB/BitsPerComponent 8/Filter/DCTDecode/Length _length_>>stream\r\n",_="\r\nendstream\r\nendobj\r\n",m="trailer\r\n<<\r\n/Size 8\r\n/Root 7 0 R\r\n/Info 6 0 R\r\n>>\r\nstartxref\r\n_length_\r\n%%EOF",v="xref\r\n0 8\r\n0000000000 65535 f\r\n0000000241 00000 n\r\n0000000010 00000 n\r\n_main_ 00000 n\r\n0000000089 00000 n\r\n_image_ 00000 n\r\n_info_ 00000 n\r\n0000000143 00000 n\r\n",y=function e(t,n){return t.length<n?e("0"+t,n):t},b=function(e,t,n){var i=2*(t.margin||0),a=t.width+i,s=t.height+i,l=(.75*a).toFixed(2),u=(.75*s).toFixed(2),c=h.replace("_width_",l).replace("_height_",u),d=f.replace("_width_",l).replace("_height_",u),v=p.replace("_date_",n).replace("_version_",r.default),y=g.replace("_width_",a).replace("_height_",s).replace("_length_",e.length)+e+_,b=o(c.length,d.length,v.length),x=c+d+v+y,w=m.replace("_length_",x.length);return x+b+w},x=function(){return new Date},w=function(e){for(var t=0,n=new Uint8Array(e.length);t<e.length;t++)n[t]=e.charCodeAt(t);return new d.Blob([n.buffer],{type:"application/pdf"})},C=function(e){return d.btoa(e)}},function(e,t,n){Object.defineProperty(t,"exportDataGrid",{enumerable:!0,get:function(){return i.exportDataGrid}}),Object.defineProperty(t,"exportPivotGrid",{enumerable:!0,get:function(){return o.exportPivotGrid}});var i=n(351),o=n(354)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return s.Export.export(a(e),u)}function a(e){if(!(0,r.isDefined)(e)||!(0,r.isObject)(e))throw Error('The "exportDataGrid" method requires a configuration object.');if(!(0,r.isDefined)(e.component)||!(0,r.isObject)(e.component)||"dxDataGrid"!==e.component.NAME)throw Error('The "component" field must contain a DataGrid instance.');return(0,r.isDefined)(e.selectedRowsOnly)||(e.selectedRowsOnly=!1),(0,r.isDefined)(e.autoFilterEnabled)||(e.autoFilterEnabled=!1),s.Export.getFullOptions(e)}t.exportDataGrid=o;var r=n(1),s=n(248),l=i(n(22)),u={_trySetAutoFilter:function(e,t,n,i,o){if(o&&!(0,r.isDefined)(t.autoFilter)&&e.getRowsCount()>0){var a={from:{row:n.from.row+i-1,column:n.from.column},to:n.to};t.autoFilter=a}},_trySetFont:function(e,t){(0,r.isDefined)(t)&&(e.font=e.font||{},e.font.bold=t)},_getWorksheetFrozenState:function(e,t){return{state:"frozen",ySplit:t.from.row+e.getFrozenArea().y-1}},_trySetOutlineLevel:function(e,t,n,i){n>=i&&(t.outlineLevel=e.getGroupLevel(n))},_getCustomizeCellOptions:function(e,t){var n={excelCell:e,gridCell:t};return Object.defineProperty(n,"cell",{get:function(){return l.default.log("W0003","CustomizeCell handler argument","cell","20.1","Use the 'excelCell' field instead"),e}}),n},_needMergeRange:function(e,t){return e<t},_renderLoadPanel:function(e){var t=e.getView("rowsView");t._renderLoadPanel(t.element(),t.element().parent())}}},function(e,t,n){t.default=void 0;var i={en:{Yes:"Yes",No:"No",Cancel:"Cancel",Clear:"Clear",Done:"Done",Loading:"Loading...",Select:"Select...",Search:"Search",Back:"Back",OK:"OK","dxCollectionWidget-noDataText":"No data to display","dxDropDownEditor-selectLabel":"Select","validation-required":"Required","validation-required-formatted":"{0} is required","validation-numeric":"Value must be a number","validation-numeric-formatted":"{0} must be a number","validation-range":"Value is out of range","validation-range-formatted":"{0} is out of range","validation-stringLength":"The length of the value is not correct","validation-stringLength-formatted":"The length of {0} is not correct","validation-custom":"Value is invalid","validation-custom-formatted":"{0} is invalid","validation-async":"Value is invalid","validation-async-formatted":"{0} is invalid","validation-compare":"Values do not match","validation-compare-formatted":"{0} does not match","validation-pattern":"Value does not match pattern","validation-pattern-formatted":"{0} does not match pattern","validation-email":"Email is invalid","validation-email-formatted":"{0} is invalid","validation-mask":"Value is invalid","dxLookup-searchPlaceholder":"Minimum character number: {0}","dxList-pullingDownText":"Pull down to refresh...","dxList-pulledDownText":"Release to refresh...","dxList-refreshingText":"Refreshing...","dxList-pageLoadingText":"Loading...","dxList-nextButtonText":"More","dxList-selectAll":"Select All","dxListEditDecorator-delete":"Delete","dxListEditDecorator-more":"More","dxScrollView-pullingDownText":"Pull down to refresh...","dxScrollView-pulledDownText":"Release to refresh...","dxScrollView-refreshingText":"Refreshing...","dxScrollView-reachBottomText":"Loading...","dxDateBox-simulatedDataPickerTitleTime":"Select time","dxDateBox-simulatedDataPickerTitleDate":"Select date","dxDateBox-simulatedDataPickerTitleDateTime":"Select date and time","dxDateBox-validation-datetime":"Value must be a date or time","dxFileUploader-selectFile":"Select file","dxFileUploader-dropFile":"or Drop file here","dxFileUploader-bytes":"bytes","dxFileUploader-kb":"kb","dxFileUploader-Mb":"Mb","dxFileUploader-Gb":"Gb","dxFileUploader-upload":"Upload","dxFileUploader-uploaded":"Uploaded","dxFileUploader-readyToUpload":"Ready to upload","dxFileUploader-uploadAbortedMessage":"Upload cancelled","dxFileUploader-uploadFailedMessage":"Upload failed","dxFileUploader-invalidFileExtension":"File type is not allowed","dxFileUploader-invalidMaxFileSize":"File is too large","dxFileUploader-invalidMinFileSize":"File is too small","dxRangeSlider-ariaFrom":"From","dxRangeSlider-ariaTill":"Till","dxSwitch-switchedOnText":"ON","dxSwitch-switchedOffText":"OFF","dxForm-optionalMark":"optional","dxForm-requiredMessage":"{0} is required","dxNumberBox-invalidValueMessage":"Value must be a number","dxNumberBox-noDataText":"No data","dxDataGrid-columnChooserTitle":"Column Chooser","dxDataGrid-columnChooserEmptyText":"Drag a column here to hide it","dxDataGrid-groupContinuesMessage":"Continues on the next page","dxDataGrid-groupContinuedMessage":"Continued from the previous page","dxDataGrid-groupHeaderText":"Group by This Column","dxDataGrid-ungroupHeaderText":"Ungroup","dxDataGrid-ungroupAllText":"Ungroup All","dxDataGrid-editingEditRow":"Edit","dxDataGrid-editingSaveRowChanges":"Save","dxDataGrid-editingCancelRowChanges":"Cancel","dxDataGrid-editingDeleteRow":"Delete","dxDataGrid-editingUndeleteRow":"Undelete","dxDataGrid-editingConfirmDeleteMessage":"Are you sure you want to delete this record?","dxDataGrid-validationCancelChanges":"Cancel changes","dxDataGrid-groupPanelEmptyText":"Drag a column header here to group by that column","dxDataGrid-noDataText":"No data",
"dxDataGrid-searchPanelPlaceholder":"Search...","dxDataGrid-filterRowShowAllText":"(All)","dxDataGrid-filterRowResetOperationText":"Reset","dxDataGrid-filterRowOperationEquals":"Equals","dxDataGrid-filterRowOperationNotEquals":"Does not equal","dxDataGrid-filterRowOperationLess":"Less than","dxDataGrid-filterRowOperationLessOrEquals":"Less than or equal to","dxDataGrid-filterRowOperationGreater":"Greater than","dxDataGrid-filterRowOperationGreaterOrEquals":"Greater than or equal to","dxDataGrid-filterRowOperationStartsWith":"Starts with","dxDataGrid-filterRowOperationContains":"Contains","dxDataGrid-filterRowOperationNotContains":"Does not contain","dxDataGrid-filterRowOperationEndsWith":"Ends with","dxDataGrid-filterRowOperationBetween":"Between","dxDataGrid-filterRowOperationBetweenStartText":"Start","dxDataGrid-filterRowOperationBetweenEndText":"End","dxDataGrid-applyFilterText":"Apply filter","dxDataGrid-trueText":"true","dxDataGrid-falseText":"false","dxDataGrid-sortingAscendingText":"Sort Ascending","dxDataGrid-sortingDescendingText":"Sort Descending","dxDataGrid-sortingClearText":"Clear Sorting","dxDataGrid-editingSaveAllChanges":"Save changes","dxDataGrid-editingCancelAllChanges":"Discard changes","dxDataGrid-editingAddRow":"Add a row","dxDataGrid-summaryMin":"Min: {0}","dxDataGrid-summaryMinOtherColumn":"Min of {1} is {0}","dxDataGrid-summaryMax":"Max: {0}","dxDataGrid-summaryMaxOtherColumn":"Max of {1} is {0}","dxDataGrid-summaryAvg":"Avg: {0}","dxDataGrid-summaryAvgOtherColumn":"Avg of {1} is {0}","dxDataGrid-summarySum":"Sum: {0}","dxDataGrid-summarySumOtherColumn":"Sum of {1} is {0}","dxDataGrid-summaryCount":"Count: {0}","dxDataGrid-columnFixingFix":"Fix","dxDataGrid-columnFixingUnfix":"Unfix","dxDataGrid-columnFixingLeftPosition":"To the left","dxDataGrid-columnFixingRightPosition":"To the right","dxDataGrid-exportTo":"Export","dxDataGrid-exportToExcel":"Export to Excel file","dxDataGrid-exporting":"Exporting...","dxDataGrid-excelFormat":"Excel file","dxDataGrid-selectedRows":"Selected rows","dxDataGrid-exportSelectedRows":"Export selected rows","dxDataGrid-exportAll":"Export all data","dxDataGrid-headerFilterEmptyValue":"(Blanks)","dxDataGrid-headerFilterOK":"OK","dxDataGrid-headerFilterCancel":"Cancel","dxDataGrid-ariaColumn":"Column","dxDataGrid-ariaValue":"Value","dxDataGrid-ariaFilterCell":"Filter cell","dxDataGrid-ariaCollapse":"Collapse","dxDataGrid-ariaExpand":"Expand","dxDataGrid-ariaDataGrid":"Data grid","dxDataGrid-ariaSearchInGrid":"Search in data grid","dxDataGrid-ariaSelectAll":"Select all","dxDataGrid-ariaSelectRow":"Select row","dxDataGrid-filterBuilderPopupTitle":"Filter Builder","dxDataGrid-filterPanelCreateFilter":"Create Filter","dxDataGrid-filterPanelClearFilter":"Clear","dxDataGrid-filterPanelFilterEnabledHint":"Enable the filter","dxTreeList-ariaTreeList":"Tree list","dxTreeList-editingAddRowToNode":"Add","dxPager-infoText":"Page {0} of {1} ({2} items)","dxPager-pagesCountText":"of","dxPager-pageSizesAllText":"All","dxPivotGrid-grandTotal":"Grand Total","dxPivotGrid-total":"{0} Total","dxPivotGrid-fieldChooserTitle":"Field Chooser","dxPivotGrid-showFieldChooser":"Show Field Chooser","dxPivotGrid-expandAll":"Expand All","dxPivotGrid-collapseAll":"Collapse All","dxPivotGrid-sortColumnBySummary":'Sort "{0}" by This Column',"dxPivotGrid-sortRowBySummary":'Sort "{0}" by This Row',"dxPivotGrid-removeAllSorting":"Remove All Sorting","dxPivotGrid-dataNotAvailable":"N/A","dxPivotGrid-rowFields":"Row Fields","dxPivotGrid-columnFields":"Column Fields","dxPivotGrid-dataFields":"Data Fields","dxPivotGrid-filterFields":"Filter Fields","dxPivotGrid-allFields":"All Fields","dxPivotGrid-columnFieldArea":"Drop Column Fields Here","dxPivotGrid-dataFieldArea":"Drop Data Fields Here","dxPivotGrid-rowFieldArea":"Drop Row Fields Here","dxPivotGrid-filterFieldArea":"Drop Filter Fields Here","dxScheduler-editorLabelTitle":"Subject","dxScheduler-editorLabelStartDate":"Start Date","dxScheduler-editorLabelEndDate":"End Date","dxScheduler-editorLabelDescription":"Description","dxScheduler-editorLabelRecurrence":"Repeat","dxScheduler-openAppointment":"Open appointment","dxScheduler-recurrenceNever":"Never","dxScheduler-recurrenceMinutely":"Every minute","dxScheduler-recurrenceHourly":"Hourly","dxScheduler-recurrenceDaily":"Daily","dxScheduler-recurrenceWeekly":"Weekly","dxScheduler-recurrenceMonthly":"Monthly","dxScheduler-recurrenceYearly":"Yearly","dxScheduler-recurrenceRepeatEvery":"Repeat Every","dxScheduler-recurrenceRepeatOn":"Repeat On","dxScheduler-recurrenceEnd":"End repeat","dxScheduler-recurrenceAfter":"After","dxScheduler-recurrenceOn":"On","dxScheduler-recurrenceRepeatMinutely":"minute(s)","dxScheduler-recurrenceRepeatHourly":"hour(s)","dxScheduler-recurrenceRepeatDaily":"day(s)","dxScheduler-recurrenceRepeatWeekly":"week(s)","dxScheduler-recurrenceRepeatMonthly":"month(s)","dxScheduler-recurrenceRepeatYearly":"year(s)","dxScheduler-switcherDay":"Day","dxScheduler-switcherWeek":"Week","dxScheduler-switcherWorkWeek":"Work Week","dxScheduler-switcherMonth":"Month","dxScheduler-switcherAgenda":"Agenda","dxScheduler-switcherTimelineDay":"Timeline Day","dxScheduler-switcherTimelineWeek":"Timeline Week","dxScheduler-switcherTimelineWorkWeek":"Timeline Work Week","dxScheduler-switcherTimelineMonth":"Timeline Month","dxScheduler-recurrenceRepeatOnDate":"on date","dxScheduler-recurrenceRepeatCount":"occurrence(s)","dxScheduler-allDay":"All day","dxScheduler-confirmRecurrenceEditMessage":"Do you want to edit only this appointment or the whole series?","dxScheduler-confirmRecurrenceDeleteMessage":"Do you want to delete only this appointment or the whole series?","dxScheduler-confirmRecurrenceEditSeries":"Edit series","dxScheduler-confirmRecurrenceDeleteSeries":"Delete series","dxScheduler-confirmRecurrenceEditOccurrence":"Edit appointment","dxScheduler-confirmRecurrenceDeleteOccurrence":"Delete appointment","dxScheduler-noTimezoneTitle":"No timezone","dxScheduler-moreAppointments":"{0} more","dxCalendar-todayButtonText":"Today","dxCalendar-ariaWidgetName":"Calendar","dxColorView-ariaRed":"Red","dxColorView-ariaGreen":"Green","dxColorView-ariaBlue":"Blue","dxColorView-ariaAlpha":"Transparency","dxColorView-ariaHex":"Color code","dxTagBox-selected":"{0} selected","dxTagBox-allSelected":"All selected ({0})","dxTagBox-moreSelected":"{0} more","vizExport-printingButtonText":"Print","vizExport-titleMenuText":"Exporting/Printing","vizExport-exportButtonText":"{0} file","dxFilterBuilder-and":"And","dxFilterBuilder-or":"Or","dxFilterBuilder-notAnd":"Not And","dxFilterBuilder-notOr":"Not Or","dxFilterBuilder-addCondition":"Add Condition","dxFilterBuilder-addGroup":"Add Group","dxFilterBuilder-enterValueText":"<enter a value>","dxFilterBuilder-filterOperationEquals":"Equals","dxFilterBuilder-filterOperationNotEquals":"Does not equal","dxFilterBuilder-filterOperationLess":"Is less than","dxFilterBuilder-filterOperationLessOrEquals":"Is less than or equal to","dxFilterBuilder-filterOperationGreater":"Is greater than","dxFilterBuilder-filterOperationGreaterOrEquals":"Is greater than or equal to","dxFilterBuilder-filterOperationStartsWith":"Starts with","dxFilterBuilder-filterOperationContains":"Contains","dxFilterBuilder-filterOperationNotContains":"Does not contain","dxFilterBuilder-filterOperationEndsWith":"Ends with","dxFilterBuilder-filterOperationIsBlank":"Is blank","dxFilterBuilder-filterOperationIsNotBlank":"Is not blank","dxFilterBuilder-filterOperationBetween":"Is between","dxFilterBuilder-filterOperationAnyOf":"Is any of","dxFilterBuilder-filterOperationNoneOf":"Is none of","dxHtmlEditor-dialogColorCaption":"Change Font Color","dxHtmlEditor-dialogBackgroundCaption":"Change Background Color","dxHtmlEditor-dialogLinkCaption":"Add Link","dxHtmlEditor-dialogLinkUrlField":"URL","dxHtmlEditor-dialogLinkTextField":"Text","dxHtmlEditor-dialogLinkTargetField":"Open link in new window","dxHtmlEditor-dialogImageCaption":"Add Image","dxHtmlEditor-dialogImageUrlField":"URL","dxHtmlEditor-dialogImageAltField":"Alternate text","dxHtmlEditor-dialogImageWidthField":"Width (px)","dxHtmlEditor-dialogImageHeightField":"Height (px)","dxHtmlEditor-dialogInsertTableRowsField":"Rows","dxHtmlEditor-dialogInsertTableColumnsField":"Columns","dxHtmlEditor-dialogInsertTableCaption":"Insert Table","dxHtmlEditor-heading":"Heading","dxHtmlEditor-normalText":"Normal text","dxFileManager-newDirectoryName":"Untitled directory","dxFileManager-rootDirectoryName":"Files","dxFileManager-errorNoAccess":"Access Denied. Operation could not be completed.","dxFileManager-errorDirectoryExistsFormat":"Directory '{0}' already exists.","dxFileManager-errorFileExistsFormat":"File '{0}' already exists.","dxFileManager-errorFileNotFoundFormat":"File '{0}' not found.","dxFileManager-errorDirectoryNotFoundFormat":"Directory '{0}' not found.","dxFileManager-errorWrongFileExtension":"File extension is not allowed.","dxFileManager-errorMaxFileSizeExceeded":"File size exceeds the maximum allowed size.","dxFileManager-errorInvalidSymbols":"This name contains invalid characters.","dxFileManager-errorDefault":"Unspecified error.","dxFileManager-errorDirectoryOpenFailed":"The directory cannot be opened","dxFileManager-commandCreate":"New directory","dxFileManager-commandRename":"Rename","dxFileManager-commandMove":"Move to","dxFileManager-commandCopy":"Copy to","dxFileManager-commandDelete":"Delete","dxFileManager-commandDownload":"Download","dxFileManager-commandUpload":"Upload files","dxFileManager-commandRefresh":"Refresh","dxFileManager-commandThumbnails":"Thumbnails View","dxFileManager-commandDetails":"Details View","dxFileManager-commandClearSelection":"Clear selection","dxFileManager-commandShowNavPane":"Toggle navigation pane","dxFileManager-dialogDirectoryChooserMoveTitle":"Move to","dxFileManager-dialogDirectoryChooserMoveButtonText":"Move","dxFileManager-dialogDirectoryChooserCopyTitle":"Copy to","dxFileManager-dialogDirectoryChooserCopyButtonText":"Copy","dxFileManager-dialogRenameItemTitle":"Rename","dxFileManager-dialogRenameItemButtonText":"Save","dxFileManager-dialogCreateDirectoryTitle":"New directory","dxFileManager-dialogCreateDirectoryButtonText":"Create","dxFileManager-dialogDeleteItemTitle":"Delete","dxFileManager-dialogDeleteItemButtonText":"Delete","dxFileManager-dialogDeleteItemSingleItemConfirmation":"Are you sure you want to delete {0}?","dxFileManager-dialogDeleteItemMultipleItemsConfirmation":"Are you sure you want to delete {0} items?","dxFileManager-dialogButtonCancel":"Cancel","dxFileManager-editingCreateSingleItemProcessingMessage":"Creating a directory inside {0}","dxFileManager-editingCreateSingleItemSuccessMessage":"Created a directory inside {0}","dxFileManager-editingCreateSingleItemErrorMessage":"Directory was not created","dxFileManager-editingCreateCommonErrorMessage":"Directory was not created","dxFileManager-editingRenameSingleItemProcessingMessage":"Renaming an item inside {0}","dxFileManager-editingRenameSingleItemSuccessMessage":"Renamed an item inside {0}","dxFileManager-editingRenameSingleItemErrorMessage":"Item was not renamed","dxFileManager-editingRenameCommonErrorMessage":"Item was not renamed","dxFileManager-editingDeleteSingleItemProcessingMessage":"Deleting an item from {0}","dxFileManager-editingDeleteMultipleItemsProcessingMessage":"Deleting {0} items from {1}","dxFileManager-editingDeleteSingleItemSuccessMessage":"Deleted an item from {0}","dxFileManager-editingDeleteMultipleItemsSuccessMessage":"Deleted {0} items from {1}","dxFileManager-editingDeleteSingleItemErrorMessage":"Item was not deleted","dxFileManager-editingDeleteMultipleItemsErrorMessage":"{0} items were not deleted","dxFileManager-editingDeleteCommonErrorMessage":"Some items were not deleted","dxFileManager-editingMoveSingleItemProcessingMessage":"Moving an item to {0}","dxFileManager-editingMoveMultipleItemsProcessingMessage":"Moving {0} items to {1}","dxFileManager-editingMoveSingleItemSuccessMessage":"Moved an item to {0}","dxFileManager-editingMoveMultipleItemsSuccessMessage":"Moved {0} items to {1}","dxFileManager-editingMoveSingleItemErrorMessage":"Item was not moved","dxFileManager-editingMoveMultipleItemsErrorMessage":"{0} items were not moved","dxFileManager-editingMoveCommonErrorMessage":"Some items were not moved","dxFileManager-editingCopySingleItemProcessingMessage":"Copying an item to {0}","dxFileManager-editingCopyMultipleItemsProcessingMessage":"Copying {0} items to {1}","dxFileManager-editingCopySingleItemSuccessMessage":"Copied an item to {0}","dxFileManager-editingCopyMultipleItemsSuccessMessage":"Copied {0} items to {1}","dxFileManager-editingCopySingleItemErrorMessage":"Item was not copied","dxFileManager-editingCopyMultipleItemsErrorMessage":"{0} items were not copied","dxFileManager-editingCopyCommonErrorMessage":"Some items were not copied","dxFileManager-editingUploadSingleItemProcessingMessage":"Uploading an item to {0}","dxFileManager-editingUploadMultipleItemsProcessingMessage":"Uploading {0} items to {1}","dxFileManager-editingUploadSingleItemSuccessMessage":"Uploaded an item to {0}","dxFileManager-editingUploadMultipleItemsSuccessMessage":"Uploaded {0} items to {1}","dxFileManager-editingUploadSingleItemErrorMessage":"Item was not uploaded","dxFileManager-editingUploadMultipleItemsErrorMessage":"{0} items were not uploaded","dxFileManager-editingUploadCanceledMessage":"Canceled","dxFileManager-listDetailsColumnCaptionName":"Name","dxFileManager-listDetailsColumnCaptionDateModified":"Date Modified","dxFileManager-listDetailsColumnCaptionFileSize":"File Size","dxFileManager-listThumbnailsTooltipTextSize":"Size","dxFileManager-listThumbnailsTooltipTextDateModified":"Date Modified","dxFileManager-notificationProgressPanelTitle":"Progress","dxFileManager-notificationProgressPanelEmptyListText":"No operations","dxFileManager-notificationProgressPanelOperationCanceled":"Canceled","dxDiagram-categoryGeneral":"General","dxDiagram-categoryFlowchart":"Flowchart","dxDiagram-categoryOrgChart":"Org Chart","dxDiagram-categoryContainers":"Containers","dxDiagram-categoryCustom":"Custom","dxDiagram-commandExportToSvg":"Export to SVG","dxDiagram-commandExportToPng":"Export to PNG","dxDiagram-commandExportToJpg":"Export to JPEG","dxDiagram-commandUndo":"Undo","dxDiagram-commandRedo":"Redo","dxDiagram-commandFontName":"Font Name","dxDiagram-commandFontSize":"Font Size","dxDiagram-commandBold":"Bold","dxDiagram-commandItalic":"Italic","dxDiagram-commandUnderline":"Underline","dxDiagram-commandTextColor":"Font Color","dxDiagram-commandLineColor":"Line Color","dxDiagram-commandLineWidth":"Line Width","dxDiagram-commandLineStyle":"Line Style","dxDiagram-commandLineStyleSolid":"Solid","dxDiagram-commandLineStyleDotted":"Dotted","dxDiagram-commandLineStyleDashed":"Dashed","dxDiagram-commandFillColor":"Fill Color","dxDiagram-commandAlignLeft":"Align Left","dxDiagram-commandAlignCenter":"Align Center","dxDiagram-commandAlignRight":"Align Right","dxDiagram-commandConnectorLineType":"Connector Line Type","dxDiagram-commandConnectorLineStraight":"Straight","dxDiagram-commandConnectorLineOrthogonal":"Orthogonal","dxDiagram-commandConnectorLineStart":"Connector Line Start","dxDiagram-commandConnectorLineEnd":"Connector Line End","dxDiagram-commandConnectorLineNone":"None","dxDiagram-commandConnectorLineArrow":"Arrow","dxDiagram-commandFullscreen":"Full Screen","dxDiagram-commandUnits":"Units","dxDiagram-commandPageSize":"Page Size","dxDiagram-commandPageOrientation":"Page Orientation","dxDiagram-commandPageOrientationLandscape":"Landscape","dxDiagram-commandPageOrientationPortrait":"Portrait","dxDiagram-commandPageColor":"Page Color","dxDiagram-commandShowGrid":"Show Grid","dxDiagram-commandSnapToGrid":"Snap to Grid","dxDiagram-commandGridSize":"Grid Size","dxDiagram-commandZoomLevel":"Zoom Level","dxDiagram-commandAutoZoom":"Auto Zoom","dxDiagram-commandFitToContent":"Fit to Content","dxDiagram-commandFitToWidth":"Fit to Width","dxDiagram-commandAutoZoomByContent":"Auto Zoom by Content","dxDiagram-commandAutoZoomByWidth":"Auto Zoom by Width","dxDiagram-commandSimpleView":"Simple View","dxDiagram-commandCut":"Cut","dxDiagram-commandCopy":"Copy","dxDiagram-commandPaste":"Paste","dxDiagram-commandSelectAll":"Select All","dxDiagram-commandDelete":"Delete","dxDiagram-commandBringToFront":"Bring to Front","dxDiagram-commandSendToBack":"Send to Back","dxDiagram-commandLock":"Lock","dxDiagram-commandUnlock":"Unlock","dxDiagram-commandInsertShapeImage":"Insert Image...","dxDiagram-commandEditShapeImage":"Change Image...","dxDiagram-commandDeleteShapeImage":"Delete Image","dxDiagram-commandLayoutLeftToRight":"Left-to-right","dxDiagram-commandLayoutRightToLeft":"Right-to-left","dxDiagram-commandLayoutTopToBottom":"Top-to-bottom","dxDiagram-commandLayoutBottomToTop":"Bottom-to-top","dxDiagram-unitIn":"in","dxDiagram-unitCm":"cm","dxDiagram-unitPx":"px","dxDiagram-dialogButtonOK":"OK","dxDiagram-dialogButtonCancel":"Cancel","dxDiagram-dialogInsertShapeImageTitle":"Insert Image","dxDiagram-dialogEditShapeImageTitle":"Change Image","dxDiagram-dialogEditShapeImageSelectButton":"Select image","dxDiagram-dialogEditShapeImageLabelText":"or drop file here","dxDiagram-uiExport":"Export","dxDiagram-uiProperties":"Properties","dxDiagram-uiSettings":"Settings","dxDiagram-uiShowToolbox":"Show Toolbox","dxDiagram-uiSearch":"Search","dxDiagram-uiStyle":"Style","dxDiagram-uiLayout":"Layout","dxDiagram-uiLayoutTree":"Tree","dxDiagram-uiLayoutLayered":"Layered","dxDiagram-uiDiagram":"Diagram","dxDiagram-uiText":"Text","dxDiagram-uiObject":"Object","dxDiagram-uiConnector":"Connector","dxDiagram-uiPage":"Page","dxDiagram-shapeText":"Text","dxDiagram-shapeRectangle":"Rectangle","dxDiagram-shapeEllipse":"Ellipse","dxDiagram-shapeCross":"Cross","dxDiagram-shapeTriangle":"Triangle","dxDiagram-shapeDiamond":"Diamond","dxDiagram-shapeHeart":"Heart","dxDiagram-shapePentagon":"Pentagon","dxDiagram-shapeHexagon":"Hexagon","dxDiagram-shapeOctagon":"Octagon","dxDiagram-shapeStar":"Star","dxDiagram-shapeArrowLeft":"Left Arrow","dxDiagram-shapeArrowUp":"Up Arrow","dxDiagram-shapeArrowRight":"Right Arrow","dxDiagram-shapeArrowDown":"Down Arrow","dxDiagram-shapeArrowUpDown":"Up Down Arrow","dxDiagram-shapeArrowLeftRight":"Left Right Arrow","dxDiagram-shapeProcess":"Process","dxDiagram-shapeDecision":"Decision","dxDiagram-shapeTerminator":"Terminator","dxDiagram-shapePredefinedProcess":"Predefined Process","dxDiagram-shapeDocument":"Document","dxDiagram-shapeMultipleDocuments":"Multiple Documents","dxDiagram-shapeManualInput":"Manual Input","dxDiagram-shapePreparation":"Preparation","dxDiagram-shapeData":"Data","dxDiagram-shapeDatabase":"Database","dxDiagram-shapeHardDisk":"Hard Disk","dxDiagram-shapeInternalStorage":"Internal Storage","dxDiagram-shapePaperTape":"Paper Tape","dxDiagram-shapeManualOperation":"Manual Operation","dxDiagram-shapeDelay":"Delay","dxDiagram-shapeStoredData":"Stored Data","dxDiagram-shapeDisplay":"Display","dxDiagram-shapeMerge":"Merge","dxDiagram-shapeConnector":"Connector","dxDiagram-shapeOr":"Or","dxDiagram-shapeSummingJunction":"Summing Junction","dxDiagram-shapeContainerDefaultText":"Container","dxDiagram-shapeVerticalContainer":"Vertical Container","dxDiagram-shapeHorizontalContainer":"Horizontal Container","dxDiagram-shapeCardDefaultText":"Person's Name","dxDiagram-shapeCardWithImageOnLeft":"Card with Image on the Left","dxDiagram-shapeCardWithImageOnTop":"Card with Image on the Top","dxDiagram-shapeCardWithImageOnRight":"Card with Image on the Right","dxGantt-dialogTitle":"Title","dxGantt-dialogStartTitle":"Start","dxGantt-dialogEndTitle":"End","dxGantt-dialogProgressTitle":"Progress","dxGantt-dialogResourcesTitle":"Resources","dxGantt-dialogResourceManagerTitle":"Resource Manager","dxGantt-dialogTaskDetailsTitle":"Task Details","dxGantt-dialogEditResourceListHint":"Edit Resource List","dxGantt-dialogEditNoResources":"No resources","dxGantt-dialogButtonAdd":"Add","dxGantt-contextMenuNewTask":"New Task","dxGantt-contextMenuNewSubtask":"New Subtask","dxGantt-contextMenuDeleteTask":"Delete Task","dxGantt-contextMenuDeleteDependency":"Delete Dependency","dxGantt-dialogTaskDeleteConfirmation":"Deleting a task also deletes all its dependencies and subtasks. Are you sure you want to delete this task?","dxGantt-dialogDependencyDeleteConfirmation":"Are you sure you want to delete the dependency from the task?","dxGantt-dialogResourcesDeleteConfirmation":"Deleting a resource also deletes it from tasks to which this resource is assigned. Are you sure you want to delete these resources? Resource: {0}","dxGantt-dialogConstraintCriticalViolationMessage":"The task you are attempting to move is linked to a second task by a dependency relation. This change would conflict with dependency rules. How would you like to proceed?","dxGantt-dialogConstraintViolationMessage":"The task you are attempting to move is linked to a second task by a dependency relation. How would you like to proceed?","dxGantt-dialogCancelOperationMessage":"Cancel the operation","dxGantt-dialogDeleteDependencyMessage":"Delete the dependency","dxGantt-dialogMoveTaskAndKeepDependencyMessage":"Move the task and keep the dependency","dxGantt-undo":"Undo","dxGantt-redo":"Redo","dxGantt-expandAll":"Expand All","dxGantt-collapseAll":"Collapse All","dxGantt-addNewTask":"Add New Task","dxGantt-deleteSelectedTask":"Delete Selected Task","dxGantt-zoomIn":"Zoom In","dxGantt-zoomOut":"Zoom Out","dxGantt-fullScreen":"Full Screen"}};t.default=i,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i;if(t>0){for(n="decimal"!==e?".":"",i=0;i<t;i++)n+="0";return n}return""}function a(e){for(var t,n=0;n<e.length;n++)if(t=e.charCodeAt(n),t>=m&&t<m+10)return!0;return!1}function r(e){var t=(h.default.format(new Date(2009,8,8,6,5,4),e)||"").toString(),n=(0,p.getFormat)(function(t){return h.default.format(t,e)});return n&&(n=l(n),n=s(t)+n),n}function s(e){var t=(0,g.getLanguageId)(),n=t?t.toString(16):"",i="";if(a(e)){for(;n.length<3;)n="0"+n;i="[$-2010"+n+"]"}else n&&(i="[$-"+n+"]");return i}function l(e){return e.replace(k,"\\/").split("'").map(function(e,t){return t%2===0?e.replace(y,"AM/PM").replace(x,"d").replace(b,"d").replace(w,"M").replace(C,"H").replace(S,"\\[").replace(D,"\\]"):e?e.replace(I,"\\$&"):"'"}).join("")}function u(e,t,n){var i,a;return a="currency"===e?d.default.getOpenXmlCurrencyFormat(n):v[e.toLowerCase()],a&&(i=(0,c.format)(a,o(e,t))),i}t.ExportFormat=void 0;var c=n(42),d=i(n(45)),h=i(n(32)),f=n(1),p=n(161),g=n(205),_=n(0);n(107);var m=1632,v={thousands:"#,##0{0},&quot;K&quot;",millions:"#,##0{0},,&quot;M&quot;",billions:"#,##0{0},,,&quot;B&quot;",trillions:"#,##0{0},,,,&quot;T&quot;",percent:"0{0}%",decimal:"#{0}",fixedpoint:"#,##0{0}",exponential:"0{0}E+00",currency:" "},y=/a+/g,b=/E/g,x=/dE+/g,w=/L/g,C=/h/g,k=/\//g,S=/\[/g,D=/]/g,I=/./g,T={formatObjectConverter:function(e,t){var n={format:e,precision:e&&e.precision,dataType:t};return(0,f.isObject)(e)?(0,_.extend)(n,e,{format:e.formatter||e.type,currency:e.currency}):n},convertFormat:function(e,t,n,i){if((0,f.isDefined)(e)){if("date"===n)return r(e);if((0,f.isString)(e)&&v[e.toLowerCase()])return u(e,t,i)}}};t.ExportFormat=T},function(e,t,n){function i(e){return r.Export.export(o(e),l)}function o(e){if(!(0,a.isDefined)(e)||!(0,a.isObject)(e))throw Error('The "exportPivotGrid" method requires a configuration object.');if(!(0,a.isDefined)(e.component)||!(0,a.isObject)(e.component)||"dxPivotGrid"!==e.component.NAME)throw Error('The "component" field must contain a PivotGrid instance.');return r.Export.getFullOptions(e)}t.exportPivotGrid=i;var a=n(1),r=n(248),s=n(4),l={_getWorksheetFrozenState:function(e,t){return{state:"frozen",xSplit:t.from.column+e.getFrozenArea().x-1,ySplit:t.from.row+e.getFrozenArea().y-1}},_getCustomizeCellOptions:function(e,t){return{excelCell:e,pivotCell:t}},_needMergeRange:function(){return!0},_renderLoadPanel:function(e){e._renderLoadPanel(e._dataArea.groupElement(),e.$element())},_trySetAutoFilter:s.noop,_trySetFont:s.noop,_trySetOutlineLevel:s.noop}},function(e,t,n){Object.defineProperty(t,"exportDataGrid",{enumerable:!0,get:function(){return i.exportDataGrid}});var i=n(356)},function(e,t,n){function i(e){if(!(0,a.isDefined)(e)||!(0,a.isObject)(e))throw Error('The "exportDataGrid" method requires a configuration object.');if(!(0,a.isDefined)(e.component)||!(0,a.isObject)(e.component)||"dxDataGrid"!==e.component.NAME)throw Error('The "component" field must contain a DataGrid instance.');return(0,a.isDefined)(e.selectedRowsOnly)||(e.selectedRowsOnly=!1),r.Export.getFullOptions(e)}function o(e){return r.Export.export(i(e))}t.exportDataGrid=o;var a=n(1),r=n(357)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.Export=void 0;var o=n(1),a=n(0),r=i(n(32)),s=i(n(45)),l=i(n(10)),u=n(7),c={getFullOptions:function(e){var t=(0,a.extend)({},e);if(!(0,o.isDefined)(t.jsPDFDocument)||!(0,o.isObject)(t.jsPDFDocument))throw Error('The "jsPDFDocument" field must contain a jsPDF instance.');if(!(0,o.isDefined)(t.jsPDFDocument.autoTable)||!(0,o.isFunction)(t.jsPDFDocument.autoTable))throw Error('The "exportDataGrid" method requires a autoTable plugin for jsPDF object.');if((0,o.isDefined)(t.keepColumnWidths)||(t.keepColumnWidths=!0),(0,o.isDefined)(t.autoTableOptions)){if(!(0,o.isObject)(t.autoTableOptions))throw Error('The "autoTableOptions" option must be of object type.');t.autoTableOptions=(0,a.extend)(!0,{},this._getDefaultAutoTableOptions(),t.autoTableOptions)}else t.autoTableOptions=this._getDefaultAutoTableOptions();return(0,o.isDefined)(t.loadPanel)||(t.loadPanel={}),(0,o.isDefined)(t.loadPanel.enabled)||(t.loadPanel.enabled=!0),(0,o.isDefined)(t.loadPanel.text)||(t.loadPanel.text=l.default.format("dxDataGrid-exporting")),t},_getDefaultAutoTableOptions:function(){return{theme:"plain",tableLineColor:149,tableLineWidth:.1,styles:{textColor:51,lineColor:149,lineWidth:0},columnStyles:{},headStyles:{fontStyle:"normal",textColor:149,lineWidth:.1},bodyStyles:{lineWidth:.1},head:[],body:[]}},_setLoadPanelOptions:function(e,t){(0,u.hasWindow)()&&(e._setOptionWithoutOptionChange("loadPanel",t),this._renderLoadPanel(e))},_renderLoadPanel:function(e){var t=e.getView("rowsView");t._renderLoadPanel(t.element(),t.element().parent())},"export":function(e){var t=this,n=e.jsPDFDocument,i=e.autoTableOptions,r=e.component,s=e.customizeCell,l=e.keepColumnWidths,u=e.selectedRowsOnly,c=e.loadPanel,d=(0,a.extend)({},r.option("loadPanel"));"animation"in r.option("loadPanel")&&(c.animation=null),this._setLoadPanelOptions(r,c);var h=r.getDataProvider(u),f=!!r.option("wordWrapEnabled");return new Promise(function(e){h.ready().done(function(){var a=h.getColumns(),r=h.getStyles(),u=h.getRowsCount(),c=h.getHeaderRowCount(),d=[];if(l){var p=t._tryGetPdfColumnWidths(i.tableWidth,h.getColumnsWidths());(0,o.isDefined)(p)&&(0,o.isDefined)(i.columnStyles)&&t._setColumnWidths(i.columnStyles,p)}for(var g=0;g<u;g++){for(var _=[],m=0;m<a.length;m++){var v=h.getCellData(g,m,!0),y=v.value,b=v.cellSourceData,x=r[h.getStyleId(g,m)],w={content:t._getFormattedValue(y,x.format),styles:t._getPDFCellStyles(b.rowType,a[m].alignment,x,f)};if("header"===b.rowType){var C=t._tryGetMergeRange(g,m,d,h);C&&C.rowSpan>0&&(w.rowSpan=C.rowSpan+1),C&&C.colSpan>0&&(w.colSpan=C.colSpan+1);var k=d[g]&&d[g][m];(!k||w.rowSpan>1||w.colSpan>1)&&((0,o.isFunction)(s)&&s({gridCell:b,pdfCell:w}),_.push(w))}else if("group"!==b.rowType||(0,o.isDefined)(w.content)||1!==_.length){var S;w.content=null!==(S=w.content)&&void 0!==S?S:"",(0,o.isFunction)(s)&&s({gridCell:b,pdfCell:w}),_.push(w)}else{var D;_[0].colSpan=null!==(D=_[0].colSpan)&&void 0!==D?D:1,_[0].colSpan++}}g<c?i.head.push(_):i.body.push(_)}n.autoTable(i),e()}).always(function(){t._setLoadPanelOptions(r,d)})})},_getFormattedValue:function(e,t){if((0,o.isDefined)(t)){if((0,o.isDate)(e))return r.default.format(e,t);if((0,o.isNumeric)(e))return s.default.format(e,t)}return e},_getPDFCellStyles:function(e,t,n,i){var o=n.alignment,a=n.bold,r="header"===e?t:o,s={};return r&&(s.halign=r),a&&"header"!==e&&(s.fontStyle="bold"),i&&(s.cellWidth="wrap"),s},_tryGetMergeRange:function(e,t,n,i){if(!n[e]||!n[e][t]){var o=i.getCellMerging(e,t),a=o.colspan,r=o.rowspan;if(a||r){for(var s=e;s<=e+r;s++)for(var l=t;l<=t+a;l++)n[s]||(n[s]=[]),n[s][l]=!0;return{rowSpan:r,colSpan:a}}}},_tryGetPdfColumnWidths:function(e,t){if((0,o.isNumeric)(e)&&(0,o.isDefined)(t)){var n=t.reduce(function(e,t){return e+t},0);return t.map(function(t){return e*t/n})}},_setColumnWidths:function(e,t){t.forEach(function(t,n){e[n]=e[n]||{},e[n].cellWidth=t})}};t.Export=c},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.OptionManager=void 0;var r=n(19),s=n(4),l=n(249),u=n(0),c=n(1),d=n(185),h={},f={},p=function(){function e(t,n){i(this,e),this._options=t,this._optionsByReference=n,this._changingCallback,this._changedCallback,this._namePreparedCallbacks}return a(e,[{key:"_setByReference",value:function(e,t){(0,u.extend)(!0,e,t);for(var n in this._optionsByReference)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])}},{key:"_setPreparedValue",value:function(e,t,n,i){var o=this.get(this._options,e,!1);if(!(0,l.equals)(o,t)){var a=e.split(/[.[]/);!i&&this._changingCallback(e,o,t),f[e]=f[e]||(0,r.compileSetter)(e),f[e](this._options,t,{functionsAsIs:!0,merge:(0,c.isDefined)(n)?n:!this._optionsByReference[e],unwrapObservables:a.length>1&&!!this._optionsByReference[a[0]]}),!i&&this._changedCallback(e,t,o)}}},{key:"_prepareRelevantNames",value:function(e,t,n,i){if((0,c.isPlainObject)(n))for(var o in n)this._prepareRelevantNames(e,"".concat(t,".").concat(o),n[o]);this._namePreparedCallbacks(e,t,n,i)}},{key:"get",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this._options,t=arguments.length>1?arguments[1]:void 0,n=arguments.length>2?arguments[2]:void 0;return h[t]=h[t]||(0,r.compileGetter)(t),h[t](e,{functionsAsIs:!0,unwrapObservables:n})}},{key:"set",value:function(e,t,n,i){e=(0,d.normalizeOptions)(e,t);for(var o in e)this._prepareRelevantNames(e,o,e[o],i);for(var a in e)this._setPreparedValue(a,e[a],n,i)}},{key:"onRelevantNamesPrepared",value:function(e){this._namePreparedCallbacks=e}},{key:"onChanging",value:function(e){this._changingCallback=e}},{key:"onChanged",value:function(e){this._changedCallback=e}},{key:"dispose",value:function(){this._changingCallback=s.noop,this._changedCallback=s.noop}}]),e}();t.OptionManager=p},function(e,t,n){function i(e){return s(e)||r(e)||a(e)||o()}function o(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a(e,t){if(e){if("string"==typeof e)return l(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?l(e,t):void 0}}function r(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function s(e){if(Array.isArray(e))return l(e)}function l(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function u(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function c(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function d(e,t,n){return t&&c(e.prototype,t),n&&c(e,n),e}t.PostponedOperations=void 0;var h=n(6),f=n(1),p=function(){function e(){u(this,e),this._postponedOperations={};
}return d(e,[{key:"add",value:function(e,t,n){if(e in this._postponedOperations)n&&this._postponedOperations[e].promises.push(n);else{var i=new h.Deferred;this._postponedOperations[e]={fn:t,completePromise:i,promises:n?[n]:[]}}return this._postponedOperations[e].completePromise.promise()}},{key:"callPostponedOperations",value:function(){for(var e in this._postponedOperations){var t=this._postponedOperations[e];(0,f.isDefined)(t)&&(t.promises&&t.promises.length?h.when.apply(void 0,i(t.promises)).done(t.fn).then(t.completePromise.resolve):t.fn().done(t.completePromise.resolve))}this._postponedOperations={}}}]),e}();t.PostponedOperations=p},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.TemplateManager=void 0;var s=i(n(2)),l=n(1),u=n(4),c=n(0),d=n(118),h=n(93),f=n(250),p=3,g="template",_="dxTemplate",m="dx-template-wrapper",v=new d.FunctionTemplate(function(e){var t=e.model,n=e.parent,i=t.widget;if(!i)return(0,s.default)();var o=(0,s.default)("<div>"),a=t.options||{};return n?n._createComponent(o,i,a):o[i](a),o}),y=function(){function e(t,n){o(this,e),this._tempTemplates=[],this._defaultTemplates={},this._anonymousTemplateName=n||g,this._createElement=t||f.defaultCreateElement,this._createTemplateIfNeeded=this._createTemplateIfNeeded.bind(this)}return r(e,[{key:"addDefaultTemplates",value:function(e){this._defaultTemplates=(0,c.extend)({},this._defaultTemplates,e)}},{key:"dispose",value:function(){this._tempTemplates.forEach(function(e){e.template.dispose&&e.template.dispose()}),this._tempTemplates=[]}},{key:"extractTemplates",value:function(e){var t=this._extractTemplates(e),n=this._extractAnonymousTemplate(e);return{templates:t,anonymousTemplateMeta:n}}},{key:"_extractTemplates",value:function(e){var t=this,n=(0,f.findTemplates)(e,_),i=(0,f.suitableTemplatesByName)(n);return n.forEach(function(e){var t=e.element,n=e.options.name;t===i[n]?(0,s.default)(t).addClass(m).detach():(0,s.default)(t).remove()}),Object.keys(i).map(function(e){return{name:e,template:t._createTemplate(i[e])}})}},{key:"_extractAnonymousTemplate",value:function(e){var t=e.contents().detach(),n=t.filter(function(e,t){var n=t.nodeType===p,i=(0,s.default)(t).text().trim().length<1;return!(n&&i)});return n.length>0?{template:this._createTemplate(t),name:this._anonymousTemplateName}:{}}},{key:"_createTemplateIfNeeded",value:function(e){var t=this._tempTemplates.filter(function(t){return t.source===(0,f.templateKey)(e)})[0];if(t)return t.template;var n=this._createTemplate(e);return this._tempTemplates.push({template:n,source:(0,f.templateKey)(e)}),n}},{key:"_createTemplate",value:function(e){return this._createElement((0,f.validateTemplateSource)(e))}},{key:"getTemplate",value:function(e,t,n,i){var o=this,a=n.isAsyncTemplate,r=n.skipTemplates;return(0,l.isFunction)(e)?new d.FunctionTemplate(function(n){var u=e.apply(i,(0,f.getNormalizedTemplateArgs)(n));if(!(0,l.isDefined)(u))return new h.EmptyTemplate;var c=!1,p=(0,f.acquireTemplate)(u,function(e){return e.nodeType||(0,l.isRenderer)(e)&&!(0,s.default)(e).is("script")?new d.FunctionTemplate(function(){return e}):(c=!0,o._createTemplate(e))},t,a,r,o._defaultTemplates),g=p.render(n);return c&&p.dispose&&p.dispose(),g}):(0,f.acquireTemplate)(e,this._createTemplateIfNeeded,t,a,r,this._defaultTemplates)}},{key:"anonymousTemplateName",get:function(){return this._anonymousTemplateName}}],[{key:"createDefaultOptions",value:function(){return{integrationOptions:{watchMethod:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};return n.skipImmediate||t(e()),u.noop},templates:{"dx-polymorph-widget":v},useDeferUpdateForTemplates:!0}}}}]),e}();t.TemplateManager=y},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.Template=void 0;var g=o(n(2)),_=n(67),m=n(33),v=n(162);n(362),(0,v.registerTemplateEngine)("default",{compile:function(e){return(0,m.normalizeTemplateElement)(e)},render:function(e,t,n){return e.clone()}}),(0,v.setTemplateEngine)("default");var y=function(e){function t(e){var i;return a(this,t),i=n.call(this),i._element=e,i}l(t,e);var n=c(t);return s(t,[{key:"_renderCore",value:function(e){var t=e.transclude;return t||this._compiledTemplate||(this._compiledTemplate=(0,v.getCurrentTemplateEngine)().compile(this._element)),(0,g.default)("<div>").append(t?this._element:(0,v.getCurrentTemplateEngine)().render(this._compiledTemplate,e.model,e.index)).contents()}},{key:"source",value:function(){return(0,g.default)(this._element).clone()}}]),t}(_.TemplateBase);t.Template=y},function(e,t,n){var i=n(33),o=n(162);(0,o.registerTemplateEngine)("jquery-tmpl",{compile:function(e){return(0,i.extractTemplateMarkup)(e)},render:function(e,t){return jQuery.tmpl(e,t)}}),(0,o.registerTemplateEngine)("jsrender",{compile:function(e){return(jQuery?jQuery:jsrender).templates((0,i.extractTemplateMarkup)(e))},render:function(e,t){return e.render(t)}}),(0,o.registerTemplateEngine)("mustache",{compile:function(e){return(0,i.extractTemplateMarkup)(e)},render:function(e,t){return Mustache.render(e,t)}}),(0,o.registerTemplateEngine)("hogan",{compile:function(e){return Hogan.compile((0,i.extractTemplateMarkup)(e))},render:function(e,t){return e.render(t)}}),(0,o.registerTemplateEngine)("underscore",{compile:function(e){return _.template((0,i.extractTemplateMarkup)(e))},render:function(e,t){return e(t)}}),(0,o.registerTemplateEngine)("handlebars",{compile:function(e){return Handlebars.compile((0,i.extractTemplateMarkup)(e))},render:function(e,t){return e(t)}}),(0,o.registerTemplateEngine)("doT",{compile:function(e){return doT.template((0,i.extractTemplateMarkup)(e))},render:function(e,t){return e(t)}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(15)),s=n(12),l=n(8),u="compositionstart",c="compositionend",d="keydown",h="KeyboardProcessor",f=r.default.inherit({_keydown:(0,l.addNamespace)(d,h),_compositionStart:(0,l.addNamespace)(u,h),_compositionEnd:(0,l.addNamespace)(c,h),ctor:function(e){var t=this;e=e||{},e.element&&(this._element=(0,o.default)(e.element)),e.focusTarget&&(this._focusTarget=e.focusTarget),this._handler=e.handler,this._element&&(this._processFunction=function(e){var n=t._focusTarget&&t._focusTarget!==e.target&&(0,s.inArray)(e.target,t._focusTarget)<0,i=t._isComposingJustFinished&&229===e.which||t._isComposing||n;t._isComposingJustFinished=!1,i||t.process(e)},this._toggleProcessingWithContext=this.toggleProcessing.bind(this),a.default.on(this._element,this._keydown,this._processFunction),a.default.on(this._element,this._compositionStart,this._toggleProcessingWithContext),a.default.on(this._element,this._compositionEnd,this._toggleProcessingWithContext))},dispose:function(){this._element&&(a.default.off(this._element,this._keydown,this._processFunction),a.default.off(this._element,this._compositionStart,this._toggleProcessingWithContext),a.default.off(this._element,this._compositionEnd,this._toggleProcessingWithContext)),this._element=void 0,this._handler=void 0},process:function(e){this._handler({keyName:(0,l.normalizeKeyName)(e),key:e.key,code:e.code,ctrl:e.ctrlKey,location:e.location,metaKey:e.metaKey,shift:e.shiftKey,alt:e.altKey,which:e.which,originalEvent:e})},toggleProcessing:function(e){var t=e.type;this._isComposing=t===u,this._isComposingJustFinished=!this._isComposing}}),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(22)),a=function e(t,n){if(!n)throw o.default.Error("E0017");return Array.isArray(t)?t.map(function(t){return e(t,n)}).join(" "):t.indexOf(" ")!==-1?e(t.split(/\s+/g),n):"".concat(t,".").concat(n)},r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.initMobileViewport=void 0;var o=i(n(2)),a=i(n(11)),r=n(7),s=i(n(5)),l=n(0),u=i(n(114)),c=n(46),d=n(97),h=i(n(13)),f=(0,r.getWindow)(),p=function(e){e=(0,l.extend)({},e);var t=h.default.real(),n=e.allowZoom,i=e.allowPan,r="allowSelection"in e?e.allowSelection:"generic"===t.platform,p="meta[name=viewport]";(0,o.default)(p).length||(0,o.default)("<meta>").attr("name","viewport").appendTo("head");var g=["width=device-width"],_=[];if(n?_.push("pinch-zoom"):g.push("initial-scale=1.0","maximum-scale=1.0, user-scalable=no"),i&&_.push("pan-x","pan-y"),i||n?(0,o.default)("html").css("msOverflowStyle","-ms-autohiding-scrollbar"):(0,o.default)("html, body").css({msContentZooming:"none",msUserSelect:"none",overflow:"hidden"}),!r&&(0,c.supportProp)("userSelect")&&(0,o.default)(".dx-viewport").css((0,d.styleProp)("userSelect"),"none"),(0,o.default)(p).attr("content",g.join()),(0,o.default)("html").css("msTouchAction",_.join(" ")||"none"),t=h.default.real(),c.touch&&(s.default.off(a.default.getDocument(),".dxInitMobileViewport"),s.default.on(a.default.getDocument(),"dxpointermove.dxInitMobileViewport",function(e){var t=e.pointers.length,o="touch"===e.pointerType,a=!n&&t>1,r=!i&&1===t&&!e.isScrollingEvent;o&&(a||r)&&e.preventDefault()})),t.ios){var m="file:"===a.default.getLocation().protocol;m||u.default.add(function(){var e=(0,o.default)(f).width();(0,o.default)("body").width(e)})}t.android&&u.default.add(function(){setTimeout(function(){var e=a.default.getActiveElement();e.scrollIntoViewIfNeeded?e.scrollIntoViewIfNeeded():e.scrollIntoView(!1)})})};t.initMobileViewport=p},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.getTimeZones=void 0;var o=i(n(86)),a=o.default.getTimeZones;t.getTimeZones=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(186)),a=i(n(34)),r=n(3),s=n(1),l=n(6),u=i(n(156)),c=function e(t,n,i){i=i||[],n=n||{};var c=function(e,t){return{name:e,args:t}},d=function(e){function d(e){switch(e.name){case"sortBy":return m=[e.args],!0;case"thenBy":if(!m)throw a.default.errors.Error("E4004");return m.push(e.args),!0}return!1}function h(){var e=g[0],t=[];e&&"multiSort"===e.name&&(g.shift(),(0,r.each)(e.args[0],function(){t.push(c(t.length?"thenBy":"sortBy",this))})),g=t.concat(g)}var f,p,g,_,m,v=new l.Deferred,y=function(e){var t=n.errorHandler;t&&t(e),a.default._errorHandler(e),v.reject(e)};try{f=n.adapter,(0,s.isFunction)(f)||(f=o.default[f]),p=f(n),g=[].concat(i).concat(e);var b=p.optimize;for(b&&b(g);g.length;){if(_=g[0],!d(_)){if(m){g.unshift(c("multiSort",[m])),m=null;continue}if("enumerate"!==String(_.name)&&(!p[_.name]||p[_.name].apply(p,_.args)===!1))break}g.shift()}h(),p.exec(t).done(function(e,t){if(g.length){var i=(0,u.default)(e,{errorHandler:n.errorHandler});(0,r.each)(g,function(){i=i[this.name].apply(i,this.args)}),i.done(v.resolve).fail(v.reject)}else v.resolve(e,t)}).fail(y)}catch(e){y(e)}return v.promise()},h={};return(0,r.each)(["sortBy","thenBy","filter","slice","select","groupBy"],function(){var o=String(this);h[o]=function(){return e(t,n,i.concat(c(o,arguments)))}}),(0,r.each)(["count","min","max","sum","avg","aggregate","enumerate"],function(){var e=String(this);h[e]=function(){return d.call(this,c(e,arguments))}}),h},d=c;t.default=d,e.exports=t.default},function(e,t,n){t.default=void 0;var i={zones:[{id:"Africa/Abidjan",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Accra",untils:"-r507yk|1e3pak|681qo|cjvlc|681qo|cjvlc|681qo|cjvlc|681qo|clq9c|681qo|cjvlc|681qo|cjvlc|681qo|cjvlc|681qo|clq9c|681qo|cjvlc|681qo|cjvlc|681qo|cjvlc|681qo|clq9c|681qo|cjvlc|681qo|cjvlc|681qo|cjvlc|681qo|clq9c|681qo|cjvlc|681qo|cjvlc|681qo|cjvlc|681qo|clq9c|681qo|cjvlc|681qo|cjvlc|681qo|Infinity",offsets:"0.8667|0|-20",offsetIndices:"012121212121212121212121212121212121212121212121"},{id:"Africa/Addis_Ababa",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Algiers",untils:"-uozn3l|2qx1nl|5luo0|8y800|a4tc0|7vc00|auqo0|7idc0|b7pc0|6sg00|cyo00|7ayo0|53c00|9idxc0|3i040|51mw0|253uk0|9o2k0|92040|8l3s0|jutc0|4uy840|3rdzw0|46xc00|7x6o0|2xco40|8n180|7x9g0|9d440|kiqg0|9d440|9q2s0|9cyk0|Infinity",offsets:"-9.35|0|-60|-120",offsetIndices:"0121212121212121232321212122321212"},{id:"Africa/Asmara",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Asmera",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Bamako",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Bangui",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Banjul",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Bissau",untils:"-u9rek0|wvoyo0|Infinity",offsets:"62.3333|60|0",offsetIndices:"012"},{id:"Africa/Blantyre",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Brazzaville",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Bujumbura",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Cairo",untils:"-fdls80|40d80|a31g0|7x3w0|a4w40|aqyk0|80ys0|b07w0|7tk40|b07w0|8jhg0|a8fw0|60go40|7el80|awo40|7v980|awqw0|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7tk40|ayd80|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|f9x80|3i040|eluk0|462s0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|b5rw0|7m5g0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|aqvs0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7k580|b5xg0|6u7w0|bvus0|6h980|c8tg0|64ak0|cyqs0|5anw0|1jms0|12t80|1w22s0|25p80|1sw40|2vmk0|Infinity",offsets:"-120|-180",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Africa/Casablanca",untils:"-tblt9g|di7nxg|3huk0|51k40|2znuk0|2dp9g0|776k0|8nt2s0|657w0|3ifxg0|3jp80|va040|4qak0|e1ms0|7pp80|cnms0|3afw0|2xi840|xqqk0|bp56s0|4qak0|e1ms0|45x80|d2g40|51ek0|c8tg0|64ak0|e1sc0|47uo0|1leo0|23xc0|asw00|3lmo0|1qyo0|40g00|7x6o0|4mo00|1stc0|4deo0|7x6o0|3ylc0|1stc0|51hc0|7x6o0|3lmo0|1stc0|5reo0|7k800|2vpc0|25s00|64dc0|7k800|2iqo0|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|25s00|g7c00|1stc0|g7c00|25s00|Infinity",offsets:"30.3333|0|-60",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"Africa/Ceuta",untils:"-qyiys0|7x3w0|2vt440|8sqs0|ssyk0|8n6s0|9px80|905g0|a2yo0|902o0|k69dc0|657w0|3ifxg0|3jp80|va040|4qak0|e1ms0|7pp80|cnms0|3afw0|2xi840|129us0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"010101010101010101010121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Africa/Conakry",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Dakar",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Dar_es_Salaam",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Djibouti",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Douala",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/El_Aaiun",untils:"-isdxk0|m2g0c0|vek0|4qak0|e1ms0|7pp80|cnms0|3afw0|fke5g0|4qak0|e1ms0|45x80|d2g40|51ek0|c8tg0|64ak0|e1sc0|47uo0|1leo0|23xc0|asw00|3lmo0|1qyo0|40g00|7x6o0|4mo00|1stc0|4deo0|7x6o0|3ylc0|1stc0|51hc0|7x6o0|3lmo0|1stc0|5reo0|7k800|2vpc0|25s00|64dc0|7k800|2iqo0|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|1stc0|gkao0|1stc0|g7c00|25s00|g7c00|1stc0|g7c00|25s00|g7c00|25s00|g7c00|1stc0|g7c00|25s00|Infinity",offsets:"52.8|60|0|-60",offsetIndices:"012323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Africa/Freetown",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Gaborone",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Harare",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Johannesburg",untils:"-yvtdi0|kn7o60|9cyk0|9d440|9cyk0|Infinity",offsets:"-90|-120|-180",offsetIndices:"012121"},{id:"Africa/Juba",untils:"-kcrsis|kixuys|8l6k0|a4w40|8n180|a6qs0|8n180|a31g0|8ovw0|a16s0|8qqk0|9zc40|8sl80|9xhg0|8wak0|9ts40|8y580|a4w40|8n180|a31g0|8ovw0|a16s0|8sl80|9xhg0|8ufw0|9vms0|8wak0|9ts40|8y580|a4w40|8ovw0|a16s0|8qqk0|7frw40|Infinity",offsets:"-126.4667|-120|-180",offsetIndices:"01212121212121212121212121212121212"},{id:"Africa/Kampala",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Khartoum",untils:"-kcrsow|kixv4w|8l6k0|a4w40|8n180|a6qs0|8n180|a31g0|8ovw0|a16s0|8qqk0|9zc40|8sl80|9xhg0|8wak0|9ts40|8y580|a4w40|8n180|a31g0|8ovw0|a16s0|8sl80|9xhg0|8ufw0|9vms0|8wak0|9ts40|8y580|a4w40|8ovw0|a16s0|8qqk0|7frw40|9ac180|Infinity",offsets:"-130.1333|-120|-180",offsetIndices:"012121212121212121212121212121212121"},{id:"Africa/Kigali",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Kinshasa",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Lagos",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Libreville",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Lome",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Luanda",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Lubumbashi",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Lusaka",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Malabo",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Maputo",untils:"-yvtfd8|Infinity",offsets:"-130.3333|-120",offsetIndices:"01"},{id:"Africa/Maseru",untils:"-yvtdi0|kn7o60|9cyk0|9d440|9cyk0|Infinity",offsets:"-90|-120|-180",offsetIndices:"012121"},{id:"Africa/Mbabane",untils:"-yvtdi0|kn7o60|9cyk0|9d440|9cyk0|Infinity",offsets:"-90|-120|-180",offsetIndices:"012121"},{id:"Africa/Mogadishu",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Monrovia",untils:"-qj6zc4|rl202a|Infinity",offsets:"43.1333|44.5|0",offsetIndices:"012"},{id:"Africa/Nairobi",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Africa/Ndjamena",untils:"-u9rk4c|zdk5cc|7iak0|Infinity",offsets:"-60.2|-60|-120",offsetIndices:"0121"},{id:"Africa/Niamey",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Nouakchott",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Ouagadougou",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Porto-Novo",untils:"-q9qbao|Infinity",offsets:"-13.6|-60",offsetIndices:"01"},{id:"Africa/Sao_Tome",untils:"-u9rhc0|1jbm840|irxc0|Infinity",offsets:"36.75|0|-60",offsetIndices:"0121"},{id:"Africa/Timbuktu",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Africa/Tripoli",untils:"-q3gfrw|gl6ajw|422c0|xado0|4bbo0|wrpg0|4s580|1kdpg0|c05bw0|4mqs0|9et80|9d440|9et80|9eys0|9et80|9mdg0|95jw0|9io40|9cyk0|99es0|9et80|9eys0|9et80|9d440|9et80|b2840|3cf3w0|9kis0|9et80|7vqyw0|75eo0|asw00|Infinity",offsets:"-52.7333|-60|-120",offsetIndices:"012121212121212121212121212122122"},{id:"Africa/Tunis",untils:"-uozn3l|enxevl|b5uo0|53c00|u8w00|7x9g0|c8w80|7k800|z3w0|ew40|8bx80|9d440|9nx00|925o0|8l100|gi3440|7k800|b9k00|7vc00|51mw00|5ytc0|9d1c0|9d1c0|b9k00|7thc0|7m0tc0|7tk40|93us0|b5uo0|7k800|b5uo0|7x6o0|asw00|Infinity",offsets:"-9.35|-60|-120",offsetIndices:"0121212121212121212121212121212121"},{id:"Africa/Windhoek",untils:"-yvtdi0|kn7o60|9cyk0|oj2nw0|235k00|8lho0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|Infinity",offsets:"-90|-120|-180|-60",offsetIndices:"01211313131313131313131313131313131313131313131313131"},{id:"America/Adak",untils:"-ek1nw0|1tyug0|2e6s0|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l940|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"660|600|540",offsetIndices:"011001010101010101010101010101010111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Anchorage",untils:"-ek1qo0|1tyx80|2e400|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l940|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"600|540|480",offsetIndices:"011001010101010101010101010101010111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Anguilla",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Antigua",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Araguaina",untils:"-t85j2o|99k8mo|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|2yl440|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|51udg0|64ak0|Infinity",offsets:"192.8|180|120",offsetIndices:"0121212121212121212121212121212121212121212121212121"},{id:"America/Argentina/Buenos_Aires",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvus0|6u7w0|bvus0|776k0|7qcg40|3yik0|b5xg0|7k580|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323232323232"},{id:"America/Argentina/Catamarca",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|5v42s0|z9g0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132321232"},{id:"America/Argentina/ComodRivadavia",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|5v42s0|z9g0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132321232"},{id:"America/Argentina/Cordoba",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|7qcg40|3yik0|b5xg0|7k580|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132323232"},{id:"America/Argentina/Jujuy",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|c8w80|776k0|ag040|7k2g0|bvus0|776k0|7qcg40|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323121323232"},{id:"America/Argentina/La_Rioja",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6qik0|3g880|8jbw0|6u7w0|bvus0|776k0|5v42s0|z9g0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323231232321232"},{id:"America/Argentina/Mendoza",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bktk0|71mk0|bqas0|73h80|bvus0|773s0|5unes0|6hes0|1p7mk0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232312121321232"},{id:"America/Argentina/Rio_Gallegos",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvus0|6u7w0|bvus0|776k0|5v42s0|z9g0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323232321232"},{id:"America/Argentina/Salta",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|7qcg40|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323231323232"},{id:"America/Argentina/San_Juan",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6qik0|3g880|8jbw0|6u7w0|bvus0|776k0|5v2840|2txg0|1sgak0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323231232321232"},{id:"America/Argentina/San_Luis",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|7pp80|b2aw0|71mk0|4qg40|6s8ik0|2txg0|1sgak0|14nw0|2gys0|b5xg0|7k580|b5xg0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323121212321212"},{id:"America/Argentina/Tucuman",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|5v42s0|mas0|1um2k0|3yik0|b5xg0|7k580|Infinity",
offsets:"256.8|240|180|120",offsetIndices:"0121212121212121212121212121212121212121212323232313232123232"},{id:"America/Argentina/Ushuaia",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvus0|6u7w0|bvus0|776k0|5v0dg0|12ys0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323232321232"},{id:"America/Aruba",untils:"-u7lckd|rlo7qd|Infinity",offsets:"275.7833|270|240",offsetIndices:"012"},{id:"America/Asuncion",untils:"-jy93zk|ldwofk|s4vw0|s6w40|7tek0|b0dg0|7rjw0|b0dg0|7rjw0|b0dg0|9cyk0|9eys0|9et80|9eys0|9cyk0|9eys0|9cyk0|9eys0|9cyk0|9eys0|9et80|9eys0|9cyk0|9eys0|9cyk0|9eys0|9cyk0|9eys0|9et80|9eys0|9cyk0|ahus0|8a2k0|9eys0|9cyk0|9o840|7k580|b7s40|93p80|9gtg0|7nuk0|b42s0|7lzw0|b5xg0|7tek0|b9ms0|776k0|biw40|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|9cyk0|7kas0|b5rw0|7x9g0|ast80|a31g0|7k580|b5xg0|7k580|b5xg0|7k580|biw40|776k0|biw40|776k0|biw40|8zzw0|905g0|9px80|905g0|9px80|9d440|8n180|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|a31g0|8n180|a31g0|8n180|a31g0|Infinity",offsets:"230.6667|240|180",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Atikokan",untils:"-qzov40|a2vw0|bfxjw0|pmdk0|1tz8c0|2dsw0|Infinity",offsets:"360|300",offsetIndices:"0101111"},{id:"America/Atka",untils:"-ek1nw0|1tyug0|2e6s0|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l940|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"660|600|540",offsetIndices:"011001010101010101010101010101010111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Bahia_Banderas",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|591h80|3ie2s0|axvpg0|dpgw40|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|asqg0|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"421|420|360|480|300",offsetIndices:"0121212131212121212121212121212121212142424242424242424242424242424242424242424242424242424242"},{id:"America/Bahia",untils:"-t85kv8|99kaf8|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|cyqs0|64ak0|cls40|5rbw0|dbpg0|51ek0|dbpg0|6h980|c8tg0|6h980|c8tg0|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|4irc40|6u7w0|Infinity",offsets:"154.0667|180|120",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121"},{id:"America/Barbados",untils:"-o0aiaj|46b400|npv1mj|5rbw0|a31g0|8n180|a31g0|8n180|ag040|84ik0|Infinity",offsets:"238.4833|240|180",offsetIndices:"00121212121"},{id:"America/Belem",untils:"-t85j0s|99k8ks|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|Infinity",offsets:"193.9333|180|120",offsetIndices:"012121212121212121212121212121"},{id:"America/Belize",untils:"-u52ic0|3edkc0|6ham0|c8s20|6u9a0|bvte0|6u9a0|bvte0|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|bvte0|6u9a0|bvte0|6u9a0|bvte0|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|bvte0|6u9a0|bvte0|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|bvte0|6u9a0|bvte0|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|bvte0|6u9a0|g2t2q0|3e580|4mcys0|2vmk0|Infinity",offsets:"352.8|360|330|300",offsetIndices:"01212121212121212121212121212121212121212121212121213131"},{id:"America/Blanc-Sablon",untils:"-qzp0o0|a2vw0|c5jxg0|1tzdw0|2dnc0|Infinity",offsets:"240|180",offsetIndices:"010110"},{id:"America/Boa_Vista",untils:"-t85grk|99k93k|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|62xk40|7k580|biw40|cvw0|Infinity",offsets:"242.6667|240|180",offsetIndices:"0121212121212121212121212121212121"},{id:"America/Bogota",untils:"-srdoy8|14f1hi8|ha580|Infinity",offsets:"296.2667|300|240",offsetIndices:"0121"},{id:"America/Boise",untils:"-r0emw0|ast80|7x9g0|ast80|1um840|9s7jw0|1tz5k0|2dvo0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|51k40|doik0|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420|360",offsetIndices:"0101012212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Buenos_Aires",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvus0|6u7w0|bvus0|776k0|7qcg40|3yik0|b5xg0|7k580|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323232323232"},{id:"America/Cambridge_Bay",untils:"-q3gdc0|bjeec0|1tz5k0|2dvo0|a7n3w0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x6o0|ast80|ct40|7kj40|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|420|360|300",offsetIndices:"0122131212121212121212121212121212121212121212233221212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Campo_Grande",untils:"-t85hvw|99ka7w|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|cyqs0|64ak0|cls40|5rbw0|dbpg0|51ek0|dbpg0|6h980|c8tg0|6h980|c8tg0|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|cls40|64ak0|dfes0|5nmk0|c8tg0|6h980|dbpg0|5rbw0|bvus0|6h980|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6u7w0|c8tg0|64ak0|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|dbpg0|5ed80|Infinity",offsets:"218.4667|240|180",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Cancun",untils:"-p1u7c0|vauo00|7ggw40|afuk0|8a840|afuk0|8a840|64ak0|4bms0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|51k40|Infinity",offsets:"347.0667|360|300|240",offsetIndices:"0123232321212121212121212121212121212121212"},{id:"America/Caracas",untils:"-u7lcxw|rlo83w|meoxm0|4dps00|Infinity",offsets:"267.6667|270|240",offsetIndices:"01212"},{id:"America/Catamarca",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|5v42s0|z9g0|1u93w0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132321232"},{id:"America/Cayenne",untils:"-uj7yb4|tcw6r4|Infinity",offsets:"209.3333|240|180",offsetIndices:"012"},{id:"America/Cayman",untils:"-w757vc|Infinity",offsets:"319.6|300",offsetIndices:"01"},{id:"America/Chicago",untils:"-r0esg0|ast80|7x9g0|ast80|bvus0|776k0|7kas0|b5rw0|9d440|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|7x9g0|dbjw0|8a840|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"01010101010101010101010101010101010101010101010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Chihuahua",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|xes2s0|afuk0|8a840|afuk0|8aaw0|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"424.3333|420|360|300",offsetIndices:"0121212323221212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Coral_Harbour",untils:"-qzov40|a2vw0|bfxjw0|pmdk0|1tz8c0|2dsw0|Infinity",offsets:"360|300",offsetIndices:"0101111"},{id:"America/Cordoba",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|7qcg40|3yik0|b5xg0|7k580|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132323232"},{id:"America/Costa_Rica",untils:"-pjw8fn|ubtl3n|51ek0|doo40|51ek0|5jso40|8drw0|acas0|2xh80|Infinity",offsets:"336.2167|360|300",offsetIndices:"0121212121"},{id:"America/Creston",untils:"-rshz80|vbus0|Infinity",offsets:"420|480",offsetIndices:"010"},{id:"America/Cuiaba",untils:"-t85hm4|99k9y4|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|cyqs0|64ak0|cls40|5rbw0|dbpg0|51ek0|dbpg0|6h980|c8tg0|6h980|c8tg0|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|w5hg0|5nmk0|c8tg0|6h980|dbpg0|5rbw0|bvus0|6h980|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6u7w0|c8tg0|64ak0|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|dbpg0|5ed80|Infinity",offsets:"224.3333|240|180",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Curacao",untils:"-u7lckd|rlo7qd|Infinity",offsets:"275.7833|270|240",offsetIndices:"012"},{id:"America/Danmarkshavn",untils:"-rvusjk|x8nx3k|8zrk0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|53hk0|Infinity",offsets:"74.6667|180|120|0",offsetIndices:"01212121212121212121212121212121213"},{id:"America/Dawson_Creek",untils:"-qzopk0|a2vw0|c5jxg0|1tz2s0|2dyg0|tj1g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|69uk0|Infinity",offsets:"480|420",offsetIndices:"0101101010101010101010101010101010101010101010101010101011"},{id:"America/Dawson",untils:"-qzoms0|a2vw0|asys0|882c0|bmiwc0|1tz000|2e180|a7n3w0|9q000|465k00|3e2is0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|Infinity",offsets:"540|480|420",offsetIndices:"01010110201212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Denver",untils:"-r0epo0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|2vmk0|ataw40|1tz5k0|2dvo0|a7n9g0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360",offsetIndices:"01010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Detroit",untils:"-xx8dyd|5eraud|dyeyk0|1tzb40|2dq40|1c9440|7x3w0|9rlbxo|71s2c|9d440|9cyk0|2cmdg0|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"332.1833|360|300|240",offsetIndices:"0123323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/Dominica",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Edmonton",untils:"-x1yazk|629ink|a2vw0|8n6s0|29ek0|h6lg0|9px80|905g0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|9l0g40|1tz5k0|2dvo0|tj1g0|7x3w0|ctzk40|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"453.8667|420|360",offsetIndices:"0121212121212122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Eirunepe",untils:"-t85f28|99ka68|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|2yy2s0|6h980|7hg2s0|2t2t80|Infinity",offsets:"279.4667|300|240",offsetIndices:"0121212121212121212121212121212121"},{id:"America/El_Salvador",untils:"-pkm4tc|ymao5c|7k580|b5xg0|7k580|Infinity",offsets:"356.8|360|300",offsetIndices:"012121"},{id:"America/Ensenada",untils:"-p1u1s0|11jrw0|1sns00|1sgdc0|71s40|9cyk0|5iidg0|1q6700|4lfk0|190g40|eluk0|2r4o80|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|84qys0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"468.0667|420|480",offsetIndices:"012121211212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Fort_Nelson",untils:"-qzopk0|a2vw0|c5jxg0|1tz2s0|2dyg0|tj1g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|Infinity",offsets:"480|420",offsetIndices:"01011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"America/Fort_Wayne",untils:"-r0esg0|ast80|7x9g0|ast80|baw840|51ek0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|19q7w0|asys0|5qonw0|9cyk0|9d440|9cyk0|ihslg0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"010101011010101010101010101010121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Fortaleza",untils:"-t85kvc|99kafc|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|514g40|7k580|biw40|puk0|id6s0|6h980|Infinity",offsets:"154|180|120",offsetIndices:"0121212121212121212121212121212121212121"},{id:"America/Glace_Bay",untils:"-z94kwc|89fk8c|a2vw0|c5jxg0|1tzdw0|2dnc0|3y8g40|7x3w0|9pa5g0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"239.8|240|180",offsetIndices:"012122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Godthab",untils:"-rvumf4|x8nqz4|8zrk0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"206.9333|180|120",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Goose_Bay",untils:"-qzp20k|a2vw0|8kjbw0|kzjyk|7k580|b5xg0|7k580|b5xg0|7k580|biw40|776k0|biw40|7k580|b5xg0|7k580|b5xg0|1pb260|2dly0|biw40|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|biw40|7k580|ag040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|6y2s0|22420|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a2lo|afuk0|8a840|asqg0|7xc80|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8tec|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"210.8667|150.8667|210|150|240|180|120",offsetIndices:"010232323232323233232323232323232323232323232323232323232324545454545454545454545454545454545454545454546454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454"},{id:"America/Grand_Turk",untils:"-u85og2|z3brw2|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|18ais0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"307.1667|300|240",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121222121212121212121212121212121212121212121"},{id:"America/Grenada",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Guadeloupe",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Guatemala",untils:"-qqqskk|ss0akk|4ofw0|4tidg0|6djw0|3wwas0|8n180|7n5ms0|7x3w0|Infinity",offsets:"362.0667|360|300",offsetIndices:"0121212121"},{id:"America/Guayaquil",untils:"-kcr84o|wb620o|3jp80|Infinity",offsets:"314|300|240",offsetIndices:"0121"},{id:"America/Guyana",untils:"-smcak8|vj4sz8|81rf90|Infinity",offsets:"232.6667|225|180|240",offsetIndices:"0123"},{id:"America/Halifax",untils:"-z94k80|777go0|9et80|st9o0|a2vw0|ssyk0|5rbw0|cv1g0|69uk0|c6ys0|6kyk0|ci2s0|67zw0|ci2s0|6w2k0|bu040|7lzw0|bu040|66580|bu040|7lzw0|bu040|64ak0|cls40|5v180|cv1g0|6j3w0|c6ys0|79180|b42s0|7lzw0|b42s0|7yyk0|bu040|64ak0|dbpg0|66580|cls40|5ed80|bu040|7lzw0|b42s0|7lzw0|cjxg0|66580|bh1g0|7lzw0|b42s0|7lzw0|6uj00|1tzdw0|2dnc0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|tw040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|tw040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|1cm2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"254.4|240|180",offsetIndices:"0121212121212121212121212121212121212121212121212122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Havana",untils:"-n7762o|1icfyo|69uk0|62s040|4ofw0|e1ms0|51ek0|e1ms0|4ofw0|1fhs40|4ofw0|e1ms0|4ofw0|9s9k40|67zw0|cedg0|6h980|9o840|7yyk0|b5xg0|7k580|bvus0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|8a2k0|ag040|8bx80|ae5g0|8drw0|acas0|9cyk0|9d440|9px80|905g0|9px80|9q2s0|7x3w0|8a840|ast80|7x9g0|ast80|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|8a2k0|ag040|8a2k0|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|905g0|a2vw0|905g0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|8n400|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|7x6o0|1cm000|6uao0|bvs00|779c0|bitc0|6uao0|bvs00|779c0|bvs00|779c0|c8qo0|779c0|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|Infinity",offsets:"329.6|300|240",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Hermosillo",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|591h80|3ie2s0|axvpg0|dpgw40|afuk0|8a840|afuk0|8a840|afuk0|Infinity",offsets:"443.8667|420|360|480",offsetIndices:"0121212131212121"},{id:"America/Indiana/Indianapolis",untils:"-r0esg0|ast80|7x9g0|ast80|baw840|51ek0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|19q7w0|asys0|5qonw0|9cyk0|9d440|9cyk0|ihslg0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"010101011010101010101010101010121212121212121212121212121212121212121212121212121212121212121212121"
},{id:"America/Indiana/Knox",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|tj1g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|7x3w0|asys0|7x3w0|asys0|9cyk0|9d440|9px80|9d440|9cyk0|9d440|s3180|1twas0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|7j5400|asw00|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"0101011010101010101010101010101010101010101010101010101010101010101010101010101010101010111010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Indiana/Marengo",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|2wsas0|7x3w0|1c9440|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|465h80|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4g00|64dc0|clmk0|fvt9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"0101011010101010101010101212121212111212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Indiana/Petersburg",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|501ek0|7kas0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|sfzw0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|eu02o0|asw00|6udg0|c8nw0|6hc00|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"01010110101010101010101010101010101010101010101010111011212121212121212121212121212121212121212121212121212121212121"},{id:"America/Indiana/Tell_City",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|501ek0|7kas0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|1tw580|9d440|9cyk0|9d440|9cvs0|9d440|9cyk0|ihslg0|asw00|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"01010110101010101010101010101021211010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Indiana/Vevay",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|4gyis0|7txx80|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|hfzhg0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"010101101212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Indiana/Vincennes",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|asys0|7x3w0|3fidg0|7x3w0|asys0|7x3w0|b5rw0|7kas0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|7k580|b5xg0|9cyk0|9d440|9cyk0|9d440|2lz980|9cyk0|9d440|9cyk0|ihslg0|asw00|6udg0|c8nw0|6hc00|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"01010110101010101010101010101010121211011212121212121212121212121212121212121212121212121212121212121"},{id:"America/Indiana/Winamac",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|465h80|9cyk0|9d440|9cyk0|ihslg0|asw00|6udg0|c8l40|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"01010110101010101010101010101010101010121211021212121212121212121212121212121212121212121212121212121212121"},{id:"America/Indianapolis",untils:"-r0esg0|ast80|7x9g0|ast80|baw840|51ek0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|19q7w0|asys0|5qonw0|9cyk0|9d440|9cyk0|ihslg0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"010101011010101010101010101010121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Inuvik",untils:"-8ve5c0|6fce80|9q000|71i2w0|ipzw0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|480|360|420",offsetIndices:"0121323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"America/Iqaluit",untils:"-eb6ao0|1l3h80|2dq40|a7n3w0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7xc80|ast80|7x6o0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|240|300|180|360",offsetIndices:"01123212121212121212121212121212121212121212142212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Jamaica",untils:"-u85og2|wbl182|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|Infinity",offsets:"307.1667|300|240",offsetIndices:"0121212121212121212121"},{id:"America/Jujuy",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|c8w80|776k0|ag040|7k2g0|bvus0|776k0|7qcg40|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"012121212121212121212121212121212121212121232323121323232"},{id:"America/Juneau",untils:"-ek1w80|1tz2s0|2dyg0|cawis0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9d1c0|9d1c0|9cyk0|9d440|9px80|905g0|9px80|1leo0|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420|540",offsetIndices:"01101010101010101010101010001010122020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202"},{id:"America/Kentucky/Louisville",untils:"-r0esg0|ast80|7x9g0|ast80|sg5g0|6bp80|a98o40|7x3w0|6w840|1tz8c0|2dsw0|ast9o|1sw2c|21gis0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|4bh80|3j3xc0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4g00|64dc0|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"0101010101101010101010101010101010101121212121212111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Kentucky/Monticello",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|bs6g40|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x6o0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"0101011010101010101010101010101010101010101010101010101010101010101010101121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Knox_IN",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|tj1g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|7x3w0|asys0|7x3w0|asys0|9cyk0|9d440|9px80|9d440|9cyk0|9d440|s3180|1twas0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|7j5400|asw00|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"0101011010101010101010101010101010101010101010101010101010101010101010101010101010101010111010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Kralendijk",untils:"-u7lckd|rlo7qd|Infinity",offsets:"275.7833|270|240",offsetIndices:"012"},{id:"America/La_Paz",untils:"-jxzspo|84ik0|Infinity",offsets:"272.6|212.6|240",offsetIndices:"012"},{id:"America/Lima",untils:"-w25lpo|fcxjlo|4ml80|93us0|9cyk0|9d440|9cyk0|nw16s0|4ml80|e5c40|4ml80|1fr1g0|4ml80|1yiys0|4ml80|Infinity",offsets:"308.6|300|240",offsetIndices:"0121212121212121"},{id:"America/Los_Angeles",untils:"-r0emw0|ast80|7x9g0|ast80|bmtus0|1tz2s0|2dyg0|1a3c5o|f2iic|owao0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Louisville",untils:"-r0esg0|ast80|7x9g0|ast80|sg5g0|6bp80|a98o40|7x3w0|6w840|1tz8c0|2dsw0|ast9o|1sw2c|21gis0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|4bh80|3j3xc0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4g00|64dc0|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"0101010101101010101010101010101010101121212121212111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Lower_Princes",untils:"-u7lckd|rlo7qd|Infinity",offsets:"275.7833|270|240",offsetIndices:"012"},{id:"America/Maceio",untils:"-t85ldw|99kaxw|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|2yl440|64ak0|1wf1g0|7k580|biw40|puk0|id6s0|6h980|Infinity",offsets:"142.8667|180|120",offsetIndices:"012121212121212121212121212121212121212121"},{id:"America/Managua",untils:"-ijh6oo|ka1i0o|xqqk0|24p6s0|53980|dmtg0|53980|60itw0|dq240|53es0|235h80|4beis0|8zzw0|at4c0|7x140|Infinity",offsets:"345.2|360|300",offsetIndices:"0121212121212121"},{id:"America/Manaus",untils:"-t85gvw|99k97w|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|2yy2s0|6h980|Infinity",offsets:"240.0667|240|180",offsetIndices:"01212121212121212121212121212121"},{id:"America/Marigot",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Martinique",untils:"-umcvcs|zz5x4s|8zzw0|Infinity",offsets:"244.3333|240|180",offsetIndices:"0121"},{id:"America/Matamoros",untils:"-p1u7c0|ykt480|ast80|3vppg0|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"400|360|300",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Mazatlan",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|591h80|3ie2s0|axvpg0|dpgw40|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"425.6667|420|360|480",offsetIndices:"0121212131212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Mendoza",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bktk0|71mk0|bqas0|73h80|bvus0|773s0|5unes0|6hes0|1p7mk0|3yik0|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232312121321232"},{id:"America/Menominee",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|asys0|7x3w0|a7n9g0|9px80|1at9g0|2396k0|9d1c0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"01010110101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Merida",untils:"-p1u7c0|vauo00|hoyk0|6ys0c0|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"358.4667|360|300",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Metlakatla",untils:"-ek1w80|1tz2s0|2dyg0|cawis0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|gpc840|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|3ylc0|2itg0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420|540",offsetIndices:"01101010101010101010101010101010102020200202020202020202020202020202020202020202"},{id:"America/Mexico_City",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|3knek0|776k0|rf440|5t6k0|1evk40|71mk0|30p1g0|8n180|nufxo0|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"396.6|420|360|300",offsetIndices:"012121232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/Miquelon",untils:"-ulmyxk|zzqbdk|3m59g0|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"224.6667|240|180|120",offsetIndices:"012323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/Moncton",untils:"-z94i40|89fhg0|a2vw0|7mqqo0|4ofw0|e1ms0|4ofw0|e1ms0|4ofw0|e1ms0|4ofw0|e1ms0|4ofw0|e1ms0|4ofw0|dmtg0|64ak0|cao40|6fek0|bkqs0|7iak0|6y5k0|1tzdw0|2dnc0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|s36s0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a2lo|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6uiyc|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240|180",offsetIndices:"012121212121212121212122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Monterrey",untils:"-p1u7c0|ykt480|ast80|3vppg0|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"401.2667|360|300",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Montevideo",untils:"-w4mll9|67elc0|1s74p9|9et80|9exe0|9czy0|9exe0|9czy0|3ydyq0|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|7x5a0|b5w20|7k6m0|b5w20|7k6m0|9q1e0|9czy0|asxe0|7x5a0|6do20|ppvy0|4mmm0|8g9qq0|901a0|38pe0|2inw0|2nf9g0|8zzw0|1e3s40|9o3y0|q8he0|2kik0|yxhg0|4bh80|s36s0|2vl60|905g0|5rg20|51ek0|weqs0|3yik0|e1ms0|4ofw0|erk40|3yik0|2vs40|gk7w0|41iys0|3wnw0|erk40|4bh80|c8tg0|64ak0|c8tg0|6u7w0|c8tg0|6h980|bvus0|6u7w0|614qs0|9q2s0|a31g0|7x3w0|ag040|8a2k0|asys0|7x3w0|asys0|7x3w0|asys0|8a2k0|ag040|8a2k0|ag040|8a2k0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",offsets:"224.85|240|180|210|150|120|90",offsetIndices:"001232323232323232323232324242525242525264252525252525252525252525252525252525252525252"},{id:"America/Montreal",untils:"-qzoxw0|a2vw0|7yx60|aqzy0|9q8c0|7jzo0|bw0c0|6bp80|cedg0|6h980|c8tg0|6h980|bvus0|776k0|biw40|776k0|biw40|776k0|biw40|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|xjeo0|1tzb40|2dq40|asys0|7x3w0|ast80|7x3w0|asys0|7x3w0|asys0|b5rw0|7xf00|ast80|7x9g0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"01010101010101010101010101010101010101010101011101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Montserrat",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Nassau",untils:"-u6m4c6|r7u7s6|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"309.5|300|240",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/New_York",untils:"-r0ev80|ast80|7x9g0|ast80|7x9g0|b5rw0|905g0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|6w840|1tzb40|2dq40|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"01010101010101010101010101010101010101010101010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Nipigon",untils:"-qzoxw0|a2vw0|bfxjw0|pmdk0|1tzb40|2dq40|ewvus0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",
offsets:"300|240",offsetIndices:"010111010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Nome",untils:"-ek1nw0|1tyug0|2e6s0|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l6c0|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"660|600|540|480",offsetIndices:"011001010101010101010101010101010122323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/Noronha",untils:"-t85lzw|99k8rw|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|514g40|7k580|biw40|cvw0|iq5g0|6h980|Infinity",offsets:"129.6667|120|60",offsetIndices:"0121212121212121212121212121212121212121"},{id:"America/North_Dakota/Beulah",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|1tz5k0|2dvo0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hc00|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360|300",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101011212121212121212121212121212121212121212121212121212121"},{id:"America/North_Dakota/Center",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|1tz5k0|2dvo0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a5c0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360|300",offsetIndices:"010101101010101010101010101010101010101010101010101010101011212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/North_Dakota/New_Salem",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|1tz5k0|2dvo0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a5c0|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360|300",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Nuuk",untils:"-rvumf4|x8nqz4|8zrk0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"206.9333|180|120",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Ojinaga",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|xes2s0|afuk0|8a840|afuk0|8aaw0|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"417.6667|420|360|300",offsetIndices:"0121212323221212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Panama",untils:"-w757vc|Infinity",offsets:"319.6|300",offsetIndices:"01"},{id:"America/Pangnirtung",untils:"-pkmlc0|b0ke00|1tzdw0|2dnc0|a7n3w0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|asw00|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7xc80|ast80|7x6o0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|240|180|120|300|360",offsetIndices:"012213121212121212121212121212121212114141414154414141414141414141414141414141414141414141414141414141414141414141414141414"},{id:"America/Paramaribo",untils:"-usj4g8|cixc0c|5lydbk|kcrm6c|Infinity",offsets:"220.6667|220.8667|220.6|210|180",offsetIndices:"01234"},{id:"America/Phoenix",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|zjedo|4olg0|9et80|bs6lmc|9cyk0|Infinity",offsets:"420|360",offsetIndices:"01010101010"},{id:"America/Port_of_Spain",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Port-au-Prince",untils:"-rmk9ac|ylcf6c|8zzw0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8aaw0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|3vpjw0|ast80|7x9g0|ast80|2stv00|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|pkg40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"289|300|240",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Porto_Acre",untils:"-t85fg0|99kak0|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|amves0|2t2t80|Infinity",offsets:"271.2|300|240",offsetIndices:"01212121212121212121212121212121"},{id:"America/Porto_Velho",untils:"-t85g60|99k8i0|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|Infinity",offsets:"255.6|240|180",offsetIndices:"012121212121212121212121212121"},{id:"America/Puerto_Rico",untils:"-efsnk0|1ppu40|2dnc0|Infinity",offsets:"240|180",offsetIndices:"0110"},{id:"America/Punta_Arenas",untils:"-vauawq|3dlssq|157b7a|f4e0q|49hzba|aye0q|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|534ik0|351g0|2fnh80|2mg00|b73400|7k580|c8tg0|6h980|a31g0|7x3w0|asys0|7x3w0|b5xg0|7k580|ag040|8a2k0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|Infinity",offsets:"282.7667|300|240|180",offsetIndices:"0102021212121212121232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"America/Rainy_River",untils:"-qzov40|a2vw0|bfxjw0|pmdk0|1tz8c0|2dsw0|ewvus0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"010111010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Rankin_Inlet",untils:"-6s8lc0|4c6oo0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|360|240|300",offsetIndices:"012131313131313131313131313131313131313131313331313131313131313131313131313131313131313131313131313131313131313131313131"},{id:"America/Recife",untils:"-t85ljc|99kb3c|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|514g40|7k580|biw40|cvw0|iq5g0|6h980|Infinity",offsets:"139.6|180|120",offsetIndices:"0121212121212121212121212121212121212121"},{id:"America/Regina",untils:"-xkq9yc|6l1hmc|a2vw0|60enw0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|1b6840|9cyk0|9d440|8zzw0|9q2s0|9cyk0|9q2s0|9cyk0|9d440|9cyk0|66gc0|1tz5k0|2dvo0|a31g0|9cyk0|a31g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|tj1g0|9cyk0|9d440|Infinity",offsets:"418.6|420|360",offsetIndices:"012121212121212121212121221212121212121212121212121212"},{id:"America/Resolute",untils:"-bnp9c0|97nco0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|360|240|300",offsetIndices:"012131313131313131313131313131313131313131313331313131313331313131313131313131313131313131313131313131313131313131313131"},{id:"America/Rio_Branco",untils:"-t85fg0|99kak0|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|amves0|2t2t80|Infinity",offsets:"271.2|300|240",offsetIndices:"01212121212121212121212121212121"},{id:"America/Rosario",untils:"-px7ys0|5iv8k0|67zw0|a4w40|73h80|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|cls40|66580|cls40|66580|cls40|66580|cls40|67zw0|6a040|hy7w0|6a040|xovw0|3uys0|18nbw0|b0dg0|8ve2k0|3uys0|3yik0|bqas0|71mk0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|7m2qs0|4tzw0|biw40|776k0|bvus0|6u7w0|bvxk0|6u540|bvus0|776k0|7qcg40|3yik0|b5xg0|7k580|Infinity",offsets:"256.8|240|180|120",offsetIndices:"01212121212121212121212121212121212121212123232323132323232"},{id:"America/Santa_Isabel",untils:"-p1u1s0|11jrw0|1sns00|1sgdc0|71s40|9cyk0|5iidg0|1q6700|4lfk0|190g40|eluk0|2r4o80|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|84qys0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"468.0667|420|480",offsetIndices:"012121211212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Santarem",untils:"-t85hvc|99ka7c|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|amves0|Infinity",offsets:"218.8|240|180",offsetIndices:"0121212121212121212121212121212"},{id:"America/Santiago",untils:"-vauawq|3dlssq|157b7a|f4e0q|49hzba|aye0q|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|534ik0|351g0|229zw0|2gt80|awo40|2mg00|b73400|7k580|c8tg0|6h980|a31g0|7x3w0|asys0|7x3w0|b5xg0|7k580|ag040|8a2k0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|e1h80|4olg0|e1h80|4olg0|c8nw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|Infinity",offsets:"282.7667|300|240|180",offsetIndices:"010202121212121212321232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"America/Santo_Domingo",untils:"-j6hz1c|hiw29c|67zw0|1dy840|62ha0|cnle0|4h2m0|elyq0|47ta0|ei9e0|4bim0|eek20|4dda0|ecpe0|dkmtg0|1stc0|Infinity",offsets:"280|300|240|270",offsetIndices:"01213131313131212"},{id:"America/Sao_Paulo",untils:"-t85jd8|99k8x8|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5k02s0|6onw0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|cyqs0|64ak0|cls40|5rbw0|dbpg0|51ek0|dbpg0|6h980|c8tg0|6h980|c8tg0|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|cls40|64ak0|dfes0|5nmk0|c8tg0|6h980|dbpg0|5rbw0|bvus0|6h980|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6u7w0|c8tg0|64ak0|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|dbpg0|5ed80|Infinity",offsets:"186.4667|180|120",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Scoresbysund",untils:"-rvurxk|x8ntpk|902o0|9cvs0|9cyk0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"87.8667|120|60|0",offsetIndices:"0121323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/Shiprock",untils:"-r0epo0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|2vmk0|ataw40|1tz5k0|2dvo0|a7n9g0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360",offsetIndices:"01010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Sitka",untils:"-ek1w80|1tz2s0|2dyg0|cawis0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1leo0|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420|540",offsetIndices:"01101010101010101010101010101010122020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202"},{id:"America/St_Barthelemy",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/St_Johns",untils:"-ris3ck|8bx80|ar440|a2vw0|9tjs0|53980|dkys0|9cyk0|9d440|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|9cyk0|9d440|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|7tmw0|1wfuk|8zzw0|a3480|7k580|b5xg0|7k580|b5xg0|7k580|biw40|776k0|biw40|7k580|b5xg0|7k580|b5xg0|1pb260|2dly0|biw40|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|biw40|7k580|ag040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a2lo|afuk0|8a840|asqg0|7xc80|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8tec|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"210.8667|150.8667|210|150|90",offsetIndices:"01010101010101010101010101010101010102323232323232323323232323232323232323232323232323232323232323232323232323232323232323232323232323232324232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"America/St_Kitts",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/St_Lucia",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/St_Thomas",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/St_Vincent",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Swift_Current",untils:"-xkq9d4|6l1h14|a2vw0|c5jxg0|1tz5k0|2dvo0|asys0|8n180|a31g0|7x3w0|asys0|7x3w0|asys0|7x3w0|3yles0|9cyk0|s36s0|9cyk0|9d440|7x3w0|b5xg0|7k580|5j4lg0|Infinity",offsets:"431.3333|420|360",offsetIndices:"012122121212121212121212"},{id:"America/Tegucigalpa",untils:"-pfzh6k|yho0ik|7k580|b5xg0|7k580|96x1g0|4qak0|Infinity",offsets:"348.8667|360|300",offsetIndices:"01212121"},{id:"America/Thule",untils:"-rvuj9g|12yzilg|9cyk0|9d440|9cyk0|9q2s0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"275.1333|240|180",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Thunder_Bay",untils:"-vbavc0|gr8qs0|1tzb40|2dq40|ctmlg0|9cyk0|9d440|9px80|9d440|9cyk0|s36s0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"0122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"America/Tijuana",untils:"-p1u1s0|11jrw0|1sns00|1sgdc0|71s40|9cyk0|5iidg0|1q6700|4lfk0|190g40|eluk0|2r4o80|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|84qys0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"468.0667|420|480",offsetIndices:"012121211212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Toronto",untils:"-qzoxw0|a2vw0|7yx60|aqzy0|9q8c0|7jzo0|bw0c0|6bp80|cedg0|6h980|c8tg0|6h980|bvus0|776k0|biw40|776k0|biw40|776k0|biw40|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|xjeo0|1tzb40|2dq40|asys0|7x3w0|ast80|7x3w0|asys0|7x3w0|asys0|b5rw0|7xf00|ast80|7x9g0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"01010101010101010101010101010101010101010101011101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Tortola",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Vancouver",untils:"-qzopk0|a2vw0|c5jxg0|1tz2s0|2dyg0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"0101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Virgin",untils:"-u6m79w|Infinity",offsets:"246.0667|240",offsetIndices:"01"},{id:"America/Whitehorse",untils:"-qzoms0|a2vw0|asys0|882c0|bmiwc0|1tz000|2e180|a7n3w0|9q000|tiyo0|6qp440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|Infinity",offsets:"540|480|420",offsetIndices:"01010110201212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"America/Winnipeg",untils:"-s0s7c0|7k580|tj700|a2vw0|9ok840|6u7w0|2a5hg0|1tz8c0|2dsw0|biw40|7x3w0|a31g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b7s40|7tek0|autg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9cyk0|9d440|7x3w0|1cm2s0|7k580|1cm2s0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",
offsets:"360|300",offsetIndices:"010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Yakutat",untils:"-ek1tg0|1tz000|2e180|cawis0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1lbw0|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"540|480",offsetIndices:"01101010101010101010101010101010100101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"America/Yellowknife",untils:"-i9m2o0|3pk3o0|1tz5k0|2dvo0|a7n3w0|9q000|7k85k0|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"0|420|360|300",offsetIndices:"012213121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Antarctica/Casey",untils:"-irxc0|lag4o0|73bo0|uz1o0|60l80|2fnh80|pz9g0|Infinity",offsets:"0|-480|-660",offsetIndices:"01212121"},{id:"Antarctica/Davis",untils:"-6rmdc0|42jdw0|27wgs0|l8uss0|7eqs0|unmk0|60qs0|Infinity",offsets:"0|-420|-300",offsetIndices:"01012121"},{id:"Antarctica/DumontDUrville",untils:"-c05eo0|2mks80|2i72g0|Infinity",offsets:"0|-600",offsetIndices:"0101"},{id:"Antarctica/Macquarie",untils:"-rsj4w0|8zzw0|11wqk0|f4kh40|a6p8g0|9d1c0|asw00|6uao0|bvs00|6uao0|bvs00|779c0|bvs00|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|b5uo0|7k800|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|bvs00|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x6o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|7x6o0|asw00|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660|0",offsetIndices:"0102010101010101010101010101010101010101010101010101010101010101010101010101010101010101011"},{id:"Antarctica/Mawson",untils:"-8aelc0|t22y80|Infinity",offsets:"0|-360|-300",offsetIndices:"012"},{id:"Antarctica/McMurdo",untils:"-m01p20|64ak0|biw40|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|8a3y0|afyq0|8a3y0|afyq0|afvy0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|b5ta0|7k9e0|b5ta0|7x820|hsl2m0|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-690|-750|-720|-780",offsetIndices:"01020202020202020202020202023232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Antarctica/Palmer",untils:"-2lxhc0|31ho0|bqas0|71mk0|bqas0|8ovw0|9d440|9px80|9d440|9cyk0|9d440|28t6k0|51ek0|46b6s0|8c2s0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|Infinity",offsets:"0|180|240|120",offsetIndices:"0121212121213121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Antarctica/Rothera",untils:"3lxs00|Infinity",offsets:"0|180",offsetIndices:"01"},{id:"Antarctica/South_Pole",untils:"-m01p20|64ak0|biw40|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|8a3y0|afyq0|8a3y0|afyq0|afvy0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|b5ta0|7k9e0|b5ta0|7x820|hsl2m0|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-690|-750|-720|-780",offsetIndices:"01020202020202020202020202023232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Antarctica/Syowa",untils:"-6qsqo0|Infinity",offsets:"0|-180",offsetIndices:"01"},{id:"Antarctica/Troll",untils:"ibruo0|27pg0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-120",offsetIndices:"00101010101010101010101010101010101010101010101010101010101010101010"},{id:"Antarctica/Vostok",untils:"-6aaao0|Infinity",offsets:"0|-360",offsetIndices:"01"},{id:"Arctic/Longyearbyen",untils:"-rzayo0|6qfs0|cgcqo0|15tsc0|7k800|9q000|9d1c0|9d1c0|9d1c0|9d1c0|70q5c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|b5uo0|7k800|7law00|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Asia/Aden",untils:"-bwgbbg|Infinity",offsets:"-186.8667|-180",offsetIndices:"01"},{id:"Asia/Almaty",untils:"-nu1a90|37a0d0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|Infinity",offsets:"-307.8|-300|-360|-420",offsetIndices:"012323232323232323232321232323232323232323232323232"},{id:"Asia/Amman",untils:"-kcrtbk|m566fk|60l80|awo40|7v980|awo40|7v980|ayis0|9gnw0|9b9g0|7v980|autg0|7v980|3e6840|9et80|9io40|9cyk0|9d440|9cyk0|9d440|9px80|ayis0|7rjw0|ag040|8a2k0|9zc40|8drw0|a31g0|8zzw0|9d440|9cyk0|9d440|8n180|ag040|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|epmo0|4deo0|9o5c0|9ew00|9b6o0|9ew00|9d1c0|9d1c0|9d1c0|asw00|7x6o0|afxc0|8n400|9d1c0|9d1c0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|wel80|51k40|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|Infinity",offsets:"-143.7333|-120|-180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Anadyr",untils:"-nu1sv8|379zj8|qi27w0|9et80|is040|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|j3440|7k800|Infinity",offsets:"-709.9333|-720|-780|-840|-660",offsetIndices:"01232121212121212121214121212121212121212121212121212121212141"},{id:"Asia/Aqtau",untils:"-nu15b4|379y74|qrh3w0|iruk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|Infinity",offsets:"-201.0667|-240|-300|-360",offsetIndices:"012323232323232323232123232312121212121212121212"},{id:"Asia/Aqtobe",untils:"-nu16l4|379zh4|qi27w0|s6qk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|Infinity",offsets:"-228.6667|-240|-300|-360",offsetIndices:"0123232323232323232321232323232323232323232323232"},{id:"Asia/Ashgabat",untils:"-nu16t8|379zp8|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|Infinity",offsets:"-233.5333|-240|-300|-360",offsetIndices:"0123232323232323232323212"},{id:"Asia/Ashkhabad",untils:"-nu16t8|379zp8|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|Infinity",offsets:"-233.5333|-240|-300|-360",offsetIndices:"0123232323232323232323212"},{id:"Asia/Atyrau",untils:"-nu15m8|37a1a8|qrh140|iruk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|j3440|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|Infinity",offsets:"-207.7333|-180|-300|-360|-240",offsetIndices:"01232323232323232323242323232323232324242424242"},{id:"Asia/Baghdad",untils:"-r50g80|xkn3w0|7v980|9b9g0|9gnw0|9eys0|9et80|9d440|9b9g0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9f1k0|9ew00|9ew00|9ew00|9d1c0|9ew00|9d1c0|9ew00|9d1c0|9ew00|9ew00|9ew00|9d1c0|9ew00|9d1c0|9ew00|9d1c0|9ew00|9ew00|9ew00|9d1c0|9ew00|9d1c0|9ew00|9d1c0|9ew00|9ew00|9ew00|9d1c0|9ew00|9d1c0|9ew00|9d1c0|9ew00|Infinity",offsets:"-177.6|-180|-240",offsetIndices:"012121212121212121212121212121212121212121212121212121"},{id:"Asia/Bahrain",untils:"-q3gmvk|rctnrk|Infinity",offsets:"-206.1333|-240|-180",offsetIndices:"012"},{id:"Asia/Baku",untils:"-nu158c|h4tkwc|ckinw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|9d1c0|239ew0|asw00|7x3w0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-199.4|-180|-240|-300",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Bangkok",untils:"-pysda4|Infinity",offsets:"-402.0667|-420",offsetIndices:"01"},{id:"Asia/Barnaul",untils:"-q4ljic|5hu6uc|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|38fo0|64og0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|qnc40|Infinity",offsets:"-335|-360|-420|-480",offsetIndices:"0123232323232323232323212323232321212121212121212121212121212121212"},{id:"Asia/Beirut",untils:"-pyzew0|aunw0|88dg0|9et80|8yas0|a2vw0|a31g0|7k580|hjqo40|7v980|awo40|7v980|awo40|7v980|ayis0|7v980|awo40|7v980|5lhs40|56yk0|awo40|7v980|awo40|7v980|awo40|7v980|ayis0|7v980|awo40|7v980|autg0|7v980|2wxus0|8n180|a4w40|8n180|a4w40|8n180|a4w40|8n180|bs5g0|71mk0|alk40|86d80|a4w40|8n180|a4w40|8n180|a6qs0|80t80|905g0|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|Infinity",offsets:"-120|-180",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Asia/Bishkek",untils:"-nu19tc|379zxc|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|h8dc0|bkl80|8n180|a31g0|8n180|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|9db20|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|Infinity",offsets:"-298.4|-300|-360|-420",offsetIndices:"012323232323232323232321212121212121212121212121212"},{id:"Asia/Brunei",untils:"-mvofy4|3khxs4|Infinity",offsets:"-459.6667|-450|-480",offsetIndices:"012"},{id:"Asia/Calcutta",untils:"-xehava|innm9a|bmfw0|5lxg0|1mn180|Infinity",offsets:"-321.1667|-330|-390",offsetIndices:"012121"},{id:"Asia/Chita",untils:"-q4cfog|5hkxgg|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|qnew0|Infinity",offsets:"-453.8667|-480|-540|-600",offsetIndices:"012323232323232323232321232323232323232323232323232323232323232312"},{id:"Asia/Choibalsan",untils:"-xmct7c|11sndrc|2qk2k0|9eqg0|9eys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|1ckdo0|7x3w0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|s6qk0|3nc0c0|9ct00|9d9o0|9ct00|Infinity",offsets:"-458|-420|-480|-600|-540",offsetIndices:"0123434343434343434343434343434343434343434343424242"},{id:"Asia/Chongqing",untils:"-qh00w0|8sl80|asbpg0|6w2k0|7ves0|bxjw0|4mqs0|1vduk0|d4as0|75bw0|a31g0|aaak0|9d440|7v980|awo40|1dx80|j9xpo0|6u7w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010"},{id:"Asia/Chungking",untils:"-qh00w0|8sl80|asbpg0|6w2k0|7ves0|bxjw0|4mqs0|1vduk0|d4as0|75bw0|a31g0|aaak0|9d440|7v980|awo40|1dx80|j9xpo0|6u7w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010"},{id:"Asia/Colombo",untils:"-xehask|isle6k|cajy0|1mp2u0|qetjw0|7x5a0|4xvqq0|Infinity",offsets:"-319.5333|-330|-360|-390",offsetIndices:"01231321"},{id:"Asia/Dacca",untils:"-eqtpow|bmgyw|5lxg0|4qknw0|u4ijy0|a1400|Infinity",offsets:"-353.3333|-390|-330|-360|-420",offsetIndices:"0121343"},{id:"Asia/Damascus",untils:"-q3gk20|5k6q0|8n180|a31g0|8n180|a31g0|8n180|a31g0|8zzw0|k4hk40|7yyk0|awo40|7tek0|b0dg0|7v980|awo40|7tek0|alk40|887w0|awo40|7v980|ayis0|7v980|awo40|7v980|awo40|7v980|awo40|7v980|ayis0|7v980|awo40|7v980|awo40|7v980|awo40|7v980|ayis0|7v980|awo40|6bp80|cg840|6bp80|2eh1g0|8zzw0|9ts40|8zzw0|pvk40|c33w0|7cw40|cjrw0|6zxg0|btuk0|7rpg0|9gnw0|9d440|9cyk0|9et80|9et80|9rxg0|91uk0|92040|9et80|9o840|9et80|9d440|9et80|9eys0|9et80|9b9g0|9gnw0|99es0|9iik0|9d440|9et80|9eys0|9et80|9d440|9et80|9d440|9et80|9d440|9et80|9eys0|9et80|9d440|9et80|9d440|8y580|9q2s0|b5rw0|7x9g0|aunw0|7ig40|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|Infinity",offsets:"-145.2|-120|-180",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Dhaka",untils:"-eqtpow|bmgyw|5lxg0|4qknw0|u4ijy0|a1400|Infinity",offsets:"-353.3333|-390|-330|-360|-420",offsetIndices:"0121343"},{id:"Asia/Dili",untils:"-u9s4l8|fqcu98|hufs00|cpz440|Infinity",offsets:"-502.3333|-480|-540",offsetIndices:"01212"},{id:"Asia/Dubai",untils:"-q3gnko|Infinity",offsets:"-221.2|-240",offsetIndices:"01"},{id:"Asia/Dushanbe",untils:"-nu18qo|379yuo|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|hp440|Infinity",offsets:"-275.2|-300|-360|-420",offsetIndices:"012323232323232323232321"},{id:"Asia/Famagusta",untils:"-p4bqac|rvhy2c|9cyk0|b42s0|7nuk0|8yas0|8zzw0|9q2s0|9et80|9b9g0|9cyk0|9q2s0|8zzw0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|at4c0|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|8h8w0|leog0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-135.8|-120|-180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212212121212121212121212121212121212121212121"},{id:"Asia/Gaza",untils:"-ffv9k0|19f3w0|7rv00|b02c0|7tk40|b07w0|8jhg0|a8lg0|8jhg0|a8ac0|5hoqs0|7el80|awo40|7v980|awqw0|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7tk40|ayd80|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7ves0|awik0|1sns0|3p6is0|51ek0|9q2s0|6u7w0|2khpg0|25s00|1weyo0|5reo0|bxmo0|7x3w0|cls40|5rbw0|bbhg0|7rjw0|asys0|7k580|c8tg0|6h980|ag040|7x3w0|asys0|8a2k0|asys0|8a2k0|ap9g0|80t80|ap9g0|7nuk0|b2840|80t80|66as0|4vxc0|8n400|a2yo0|8n400|a2yo0|8n400|asw00|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|8n400|a2yo0|8ulg0|97ek0|8y580|9ts40|8hms0|a4qk0|7x3w0|asys0|8a5c0|ahs1o|71mic|bzk5o|69uic|cg840|902o0|9q000|9cyk0|9d440|ast80|7z440|aqyk0|7z6w0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|Infinity",offsets:"-120|-180",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010100101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Asia/Harbin",untils:"-qh00w0|8sl80|asbpg0|6w2k0|7ves0|bxjw0|4mqs0|1vduk0|d4as0|75bw0|a31g0|aaak0|9d440|7v980|awo40|1dx80|j9xpo0|6u7w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010"},{id:"Asia/Hebron",untils:"-ffv9k0|19f3w0|7rv00|b02c0|7tk40|b07w0|8jhg0|a8lg0|8jhg0|a8ac0|5hoqs0|7el80|awo40|7v980|awqw0|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7tk40|ayd80|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7ves0|awik0|1sns0|3p6is0|51ek0|9q2s0|6u7w0|2khpg0|25s00|1weyo0|5reo0|bxmo0|7x3w0|cls40|5rbw0|bbhg0|7rjw0|asys0|7k580|c8tg0|6h980|ag040|7x3w0|asys0|8a2k0|asys0|8a2k0|ap9g0|80t80|ap9g0|7nuk0|b2840|80t80|66as0|4vxc0|8n400|a2yo0|8n400|a2yo0|8n400|asw00|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|8n400|a2yo0|8ulg0|97ek0|8y580|9ts40|8hms0|a4qk0|82nw0|anes0|8a5c0|afxc0|73h80|bzk5o|69uic|1hs40|1lbw0|9d440|902o0|9q000|9cyk0|9d440|ast80|7z440|aqyk0|7z6w0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|7idc0|b7pc0|7vc00|auqo0|7vc00|auqo0|7vc00|auqo0|7vc00|b7pc0|Infinity",offsets:"-120|-180",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Asia/Ho_Chi_Minh",untils:"-x56934|2isioa|gj25iu|15ct80|8so00|tmtk0|4azjw0|2cmao0|8285c0|Infinity",offsets:"-426.6667|-426.5|-420|-480|-540",offsetIndices:"0123423232"},{id:"Asia/Hong_Kong",untils:"-y0i0s0|j44dk0|5k000|4d4y0|2195i0|7x3w0|bj320|6uao0|bvs00|7x6o0|9d1c0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|ast80|77c40|biqk0|77c40|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|bvp80|6udg0|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|8n6s0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9cyk0|1c9440|8a2k0|Infinity",offsets:"-456.7|-480|-540|-510",offsetIndices:"0123212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Hovd",untils:"-xmcoz0|11sncb0|2qk2k0|9et80|9eys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|1ckdo0|7x3w0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|4fio40|9ct00|9d9o0|9ct00|Infinity",offsets:"-366.6|-360|-420|-480",offsetIndices:"012323232323232323232323232323232323232323232323232"},{id:"Asia/Irkutsk",untils:"-q28gn5|5fh175|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-417.0833|-420|-480|-540",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Istanbul",untils:"-ux9xew|2wvx6w|7v980|1tjc40|aunw0|88dg0|9et80|8yas0|a2vw0|tzpg0|79180|awo40|7v980|7p4040|4zjw0|2vs40|f4d80|9vms0|1u5ek0|c5440|69uk0|acas0|8n180|a31g0|8n180|9q2s0|8zzw0|a31g0|8zzw0|a31g0|8n180|5md9g0|o9zw0|a6qs0|75bw0|4iwyw0|7x6o0|7kas0|b5rw0|75hg0|bkl80|77c40|biqk0|7x9g0|a2vw0|8n6s0|4iqc0|2nkw80|38l80|kdes0|8qtc0|8a5c0|9ew00|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|902o0|9q000|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7kdk0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7m2o0|b4000|7k800|b5uo0|7x6o0|asw00|7z1c0|ar1c0|7x6o0|bitc0|779c0|8fe80|Infinity",offsets:"-116.9333|-120|-180|-240",offsetIndices:"0121212121212121212121212121212121212121212121223212121212121212121212121212121212121212121212121212121212121212122"},{id:"Asia/Jakarta",untils:"-o0bdpc|4lzxc0|4wdzjc|1tu960|1cx860|11jta0|74uc20|Infinity",offsets:"-427.2|-440|-450|-540|-480|-420",offsetIndices:"01232425"},{id:"Asia/Jayapura",untils:"-jebm20|66bqe0|a37vy0|Infinity",offsets:"-562.8|-540|-570",offsetIndices:"0121"},{id:"Asia/Jerusalem",untils:"-r50eig|bp54yg|19f3w0|7rv00|b02c0|7tk40|b07w0|8jhg0|a8lg0|8jhg0|a8ac0|t9s40|56vs0|35700|9b3w0|9gtg0|8jbw0|7tmw0|a6ig0|biyw0|8a5c0|9d1c0|902o0|7x6o0|e1eg0|4ofw0|dzxo0|4q500|doo40|64iw0|auqo0|7i500|8rfms0|51ek0|9q2s0|6u7w0|2khpg0|25s00|1weyo0|5reo0|bxmo0|7x3w0|cls40|5rbw0|bbhg0|7rjw0|asys0|7k580|c8tg0|6h980|ag040|7x3w0|asys0|8a2k0|asys0|8a2k0|ap9g0|80t80|ap9g0|7nuk0|b2840|80t80|9zc40|9iik0|9kis0|93p80|9mdg0|8qqk0|apf00|7x3w0|biw40|8zx40|9io40|8n180|9kis0|9vh80|8ulg0|9px80|9mdg0|8n180|9tuw0|9tmk0|8wg40|9gnw0|99es0|8qqk0|9zc40|9tmk0|8wg40|9gnw0|99es0|8qqk0|acas0|9gnw0|99es0|93p80|9mdg0|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|Infinity",offsets:"-140.6667|-120|-180|-240",offsetIndices:"012121212121321212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Kabul",untils:"-d1pkg0|Infinity",offsets:"-240|-270",offsetIndices:"01"},{id:"Asia/Kamchatka",untils:"-olrupo|3z045o|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|j3440|7k800|Infinity",offsets:"-634.6|-660|-720|-780",offsetIndices:"012323232323232323232321232323232323232323232323232323232323212"},{id:"Asia/Karachi",untils:"-wvpb30|im3zt0|1mn180|33xpg0|a63o20|g72qo0|9cyk0|2y85g0|7v980|8hms0|aaak0|Infinity",offsets:"-268.2|-330|-390|-300|-360",offsetIndices:"012133434343"},{id:"Asia/Kashgar",untils:"-lx5pjw|Infinity",offsets:"-350.3333|-360",offsetIndices:"01"},{id:"Asia/Kathmandu",untils:"-q3gt4s|yg2lus|Infinity",offsets:"-341.2667|-330|-345",offsetIndices:"012"},{id:"Asia/Katmandu",untils:"-q3gt4s|yg2lus|Infinity",offsets:"-341.2667|-330|-345",offsetIndices:"012"},{id:"Asia/Khandyga",untils:"-q4cjrp|5hl1jp|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|3fx40|4h6s0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|8ql00|1mlho0|Infinity",offsets:"-542.2167|-480|-540|-600|-660",offsetIndices:"0123232323232323232323212323232323232323232323232343434343434343432"},{id:"Asia/Kolkata",untils:"-xehava|innm9a|bmfw0|5lxg0|1mn180|Infinity",offsets:"-321.1667|-330|-390",offsetIndices:"012121"},{id:"Asia/Krasnoyarsk",untils:"-q37l72|5gg8j2|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-371.4333|-360|-420|-480",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Kuala_Lumpur",untils:"-xphpwd|eeb94d|4it32o|8n3jc|1v2p60|iy3o60|Infinity",offsets:"-415.4167|-420|-440|-450|-540|-480",offsetIndices:"0123435"},{id:"Asia/Kuching",untils:"-mvof3k|3khwxk|1epvy0|4ohqo|e5a9c|4ohqo|e3flc|4ohqo|e3flc|4ohqo|e3flc|4ohqo|e5a9c|4ohqo|e3flc|4ohqo|3ajlc|1v2qk0|Infinity",offsets:"-441.3333|-450|-480|-500|-540",offsetIndices:"0123232323232323242"},{id:"Asia/Kuwait",untils:"-bwgbbg|Infinity",offsets:"-186.8667|-180",offsetIndices:"01"},{id:"Asia/Macao",untils:"-y0i2cy|jdvyoy|6onw0|ac580|8fs40|7v980|11luw0|awlc0|7vc00|ac800|bko00|7x6o0|9d1c0|7vc00|asw00|7x6o0|asw00|7x6o0|auqo0|88ao0|asw00|7x6o0|asw00|779c0|bitc0|779c0|bvs00|6uao0|bw1q0|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|bvp80|6udg0|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|8n6s0|9cvs0|9d6w0|9cvs0|9d6w0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9cyk0|1c9440|8a2k0|Infinity",offsets:"-454.1667|-480|-540|-600",offsetIndices:"012323212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Macau",untils:"-y0i2cy|jdvyoy|6onw0|ac580|8fs40|7v980|11luw0|awlc0|7vc00|ac800|bko00|7x6o0|9d1c0|7vc00|asw00|7x6o0|asw00|7x6o0|auqo0|88ao0|asw00|7x6o0|asw00|779c0|bitc0|779c0|bvs00|6uao0|bw1q0|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|bvp80|6udg0|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|8n6s0|9cvs0|9d6w0|9cvs0|9d6w0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9cyk0|1c9440|8a2k0|Infinity",offsets:"-454.1667|-480|-540|-600",offsetIndices:"012323212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Magadan",untils:"-nu1nxc|37a05c|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|s39k0|Infinity",offsets:"-603.2|-600|-660|-720",offsetIndices:"012323232323232323232321232323232323232323232323232323232323232312"},{id:"Asia/Makassar",untils:"-q3gzg0|6p5hc0|4u87w0|1w02k0|Infinity",offsets:"-477.6|-480|-540",offsetIndices:"00121"},{id:"Asia/Manila",untils:"-hb5y80|4qak0|2qidg0|1b2d80|4xf440|442k0|cdqdg0|9et80|Infinity",offsets:"-480|-540",offsetIndices:"010101010"},{id:"Asia/Muscat",untils:"-q3gnko|Infinity",offsets:"-221.2|-240",offsetIndices:"01"},{id:"Asia/Nicosia",untils:"-p4bq6g|rvhxyg|9cyk0|b42s0|7nuk0|8yas0|8zzw0|9q2s0|9et80|9b9g0|9cyk0|9q2s0|8zzw0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|at4c0|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",
offsets:"-133.4667|-120|-180",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Novokuznetsk",untils:"-nu36tc|37bu5c|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|j3440|7k800|Infinity",offsets:"-348.8|-360|-420|-480",offsetIndices:"012323232323232323232321232323232323232323232323232323232323212"},{id:"Asia/Novosibirsk",untils:"-q4do0s|5hmbcs|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|2vh00|6hn40|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|wrpg0|Infinity",offsets:"-331.6667|-360|-420|-480",offsetIndices:"0123232323232323232323212323212121212121212121212121212121212121212"},{id:"Asia/Omsk",untils:"-q5xmx6|5j6d16|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-293.5|-300|-360|-420",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Oral",untils:"-nu15ic|37a16c|qi2540|s6qk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9q000|9d1c0|9d1c0|5reo0|cyo00|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|Infinity",offsets:"-205.4|-180|-300|-360|-240",offsetIndices:"01232323232323232424242424242424242424242424242"},{id:"Asia/Phnom_Penh",untils:"-pysda4|Infinity",offsets:"-402.0667|-420",offsetIndices:"01"},{id:"Asia/Pontianak",untils:"-w6piww|cse2o0|4tnu2w|1wkei0|1cx860|11jta0|74uc20|cixam0|Infinity",offsets:"-437.3333|-450|-540|-480|-420",offsetIndices:"001213134"},{id:"Asia/Pyongyang",untils:"-w895yc|1yh10c|hk5da0|10ipmo0|1f4qo0|Infinity",offsets:"-503|-510|-540",offsetIndices:"012212"},{id:"Asia/Qatar",untils:"-q3gmvk|rctnrk|Infinity",offsets:"-206.1333|-240|-180",offsetIndices:"012"},{id:"Asia/Qostanay",untils:"-nu17s4|37a0o4|qi27w0|s6qk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|Infinity",offsets:"-254.4667|-240|-300|-360",offsetIndices:"012323232323232323232123232323232323232323232323"},{id:"Asia/Qyzylorda",untils:"-nu184g|37a10g|qi27w0|s6qk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|ohhc0|cyo00|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|7osl00|Infinity",offsets:"-261.8667|-240|-300|-360",offsetIndices:"01232323232323232323232323232323232323232323232"},{id:"Asia/Rangoon",untils:"-q3gv5b|bnjp3b|1kh520|Infinity",offsets:"-384.7833|-390|-540",offsetIndices:"0121"},{id:"Asia/Riyadh",untils:"-bwgbbg|Infinity",offsets:"-186.8667|-180",offsetIndices:"01"},{id:"Asia/Saigon",untils:"-x56934|2isioa|gj25iu|15ct80|8so00|tmtk0|4azjw0|2cmao0|8285c0|Infinity",offsets:"-426.6667|-426.5|-420|-480|-540",offsetIndices:"0123423232"},{id:"Asia/Sakhalin",untils:"-xl87rc|kvnarc|ikvh40|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|iq5g0|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|qnc40|Infinity",offsets:"-570.8|-540|-660|-720|-600",offsetIndices:"01232323232323232323232423232323232424242424242424242424242424242"},{id:"Asia/Samarkand",untils:"-nu18eh|37a1ah|qi27w0|s6qk0|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|Infinity",offsets:"-267.8833|-240|-300|-360",offsetIndices:"01232323232323232323232"},{id:"Asia/Seoul",untils:"-w8966g|1yh18g|hkx5a0|1faao0|5cik0|ae5g0|8a2k0|ae5g0|8bx80|c8tg0|6h980|1bj6s0|l3aq0|6j3w0|d2g40|6u7w0|b5xg0|776k0|biw40|776k0|biw40|776k0|biw40|776k0|grs40|dfqxi0|7x6o0|asw00|7x6o0|Infinity",offsets:"-507.8667|-510|-540|-600|-570",offsetIndices:"012232323232141414141414123232"},{id:"Asia/Shanghai",untils:"-qh00w0|8sl80|asbpg0|6w2k0|7ves0|bxjw0|4mqs0|1vduk0|d4as0|75bw0|a31g0|aaak0|9d440|7v980|awo40|1dx80|j9xpo0|6u7w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010"},{id:"Asia/Singapore",untils:"-xphpwd|eeb94d|4it32o|8n3jc|1v2p60|iy3o60|Infinity",offsets:"-415.4167|-420|-440|-450|-540|-480",offsetIndices:"0123435"},{id:"Asia/Srednekolymsk",untils:"-nu1ogs|37a0os|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-614.8667|-600|-660|-720",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Taipei",untils:"-gtzfk0|45slc0|c51c0|75bw0|a31g0|aaak0|9d440|7v980|awo40|7v980|awo40|7v980|awo40|7v980|7tk40|clmk0|7rpg0|b07w0|7rpg0|b07w0|7rpg0|9et80|9eys0|9et80|9d440|9et80|9d440|9et80|9d440|9et80|cjxg0|69uk0|ci2s0|69uk0|6its40|9et80|9d440|9et80|1yf9g0|4qak0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010101010101010"},{id:"Asia/Tashkent",untils:"-nu18tz|379yxz|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|Infinity",offsets:"-277.1833|-300|-360|-420",offsetIndices:"012323232323232323232321"},{id:"Asia/Tbilisi",untils:"-nu14an|h4tjyn|ckinw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|9cvs0|9cyk0|9d440|9cyk0|9d440|ipzw0|9cyk0|9q2s0|tivw0|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|4ofw0|6hn40|7k800|Infinity",offsets:"-179.1833|-180|-240|-300",offsetIndices:"0123232323232323232323212121232323232323232323212"},{id:"Asia/Tehran",untils:"-s6m6uw|fnolc0|gm3h4w|777y0|b07w0|3pes0|42c20|9cyk0|9gtg0|9kd80|5ja5g0|7avw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|1av440|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|Infinity",offsets:"-205.7333|-210|-240|-300|-270",offsetIndices:"00123214141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141"},{id:"Asia/Tel_Aviv",untils:"-r50eig|bp54yg|19f3w0|7rv00|b02c0|7tk40|b07w0|8jhg0|a8lg0|8jhg0|a8ac0|t9s40|56vs0|35700|9b3w0|9gtg0|8jbw0|7tmw0|a6ig0|biyw0|8a5c0|9d1c0|902o0|7x6o0|e1eg0|4ofw0|dzxo0|4q500|doo40|64iw0|auqo0|7i500|8rfms0|51ek0|9q2s0|6u7w0|2khpg0|25s00|1weyo0|5reo0|bxmo0|7x3w0|cls40|5rbw0|bbhg0|7rjw0|asys0|7k580|c8tg0|6h980|ag040|7x3w0|asys0|8a2k0|asys0|8a2k0|ap9g0|80t80|ap9g0|7nuk0|b2840|80t80|9zc40|9iik0|9kis0|93p80|9mdg0|8qqk0|apf00|7x3w0|biw40|8zx40|9io40|8n180|9kis0|9vh80|8ulg0|9px80|9mdg0|8n180|9tuw0|9tmk0|8wg40|9gnw0|99es0|8qqk0|9zc40|9tmk0|8wg40|9gnw0|99es0|8qqk0|acas0|9gnw0|99es0|93p80|9mdg0|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|Infinity",offsets:"-140.6667|-120|-180|-240",offsetIndices:"012121212121321212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Asia/Thimbu",untils:"-bojclo|kxymno|Infinity",offsets:"-358.6|-330|-360",offsetIndices:"012"},{id:"Asia/Thimphu",untils:"-bojclo|kxymno|Infinity",offsets:"-358.6|-330|-360",offsetIndices:"012"},{id:"Asia/Tokyo",untils:"-bb4900|6uao0|afxc0|8a5c0|c8qo0|6hc00|c8qo0|6hc00|Infinity",offsets:"-540|-600",offsetIndices:"010101010"},{id:"Asia/Tomsk",untils:"-q3zbqf|5h7z2f|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|1leo0|97k40|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|tw040|Infinity",offsets:"-339.85|-360|-420|-480",offsetIndices:"0123232323232323232323212323232323232323232323212121212121212121212"},{id:"Asia/Ujung_Pandang",untils:"-q3gzg0|6p5hc0|4u87w0|1w02k0|Infinity",offsets:"-477.6|-480|-540",offsetIndices:"00121"},{id:"Asia/Ulaanbaatar",untils:"-xmcrsk|11sncck|2qk2k0|9et80|9eys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|1ckdo0|7x3w0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|4fio40|9ct00|9d9o0|9ct00|Infinity",offsets:"-427.5333|-420|-480|-540",offsetIndices:"012323232323232323232323232323232323232323232323232"},{id:"Asia/Ulan_Bator",untils:"-xmcrsk|11sncck|2qk2k0|9et80|9eys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|1ckdo0|7x3w0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|4fio40|9ct00|9d9o0|9ct00|Infinity",offsets:"-427.5333|-420|-480|-540",offsetIndices:"012323232323232323232323232323232323232323232323232"},{id:"Asia/Urumqi",untils:"-lx5pjw|Infinity",offsets:"-350.3333|-360",offsetIndices:"01"},{id:"Asia/Ust-Nera",untils:"-q4cl6u|5hl2yu|qi27w0|9eno0|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|8ql00|1mlho0|Infinity",offsets:"-572.9|-480|-540|-720|-660|-600",offsetIndices:"012343434343434343434345434343434343434343434343434343434343434345"},{id:"Asia/Vientiane",untils:"-pysda4|Infinity",offsets:"-402.0667|-420",offsetIndices:"01"},{id:"Asia/Vladivostok",untils:"-oligf7|3yqvf7|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-527.5167|-540|-600|-660",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Yakutsk",untils:"-q4cioy|5hl0gy|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-518.9667|-480|-540|-600",offsetIndices:"01232323232323232323232123232323232323232323232323232323232323232"},{id:"Asia/Yangon",untils:"-q3gv5b|bnjp3b|1kh520|Infinity",offsets:"-384.7833|-390|-540",offsetIndices:"0121"},{id:"Asia/Yekaterinburg",untils:"-rx5hw9|1kybx4|5pfyv5|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-242.55|-225.0833|-240|-300|-360",offsetIndices:"012343434343434343434343234343434343434343434343434343434343434343"},{id:"Asia/Yerevan",untils:"-nu148o|h4tjwo|ckinw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|iq5g0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|11t180|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|Infinity",offsets:"-178|-180|-240|-300",offsetIndices:"0123232323232323232323212121212323232323232323232323232323232"},{id:"Atlantic/Azores",untils:"-u9rbs0|2bufw0|6zxg0|66580|bq800|73k00|bodc0|71pc0|bq800|73k00|bq800|71pc0|bq800|1b2g00|9b6o0|saio0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|st1c0|8n400|9d1c0|9d1c0|sg2o0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|bitc0|9d1c0|9ew00|88ao0|25p80|5reo0|3lpg0|779c0|1sqk0|6uao0|38qs0|6uao0|25p80|6hc00|38qs0|6uao0|25p80|6hc00|38qs0|8a5c0|9d1c0|9d9o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|s3400|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|5qbjo0|9d1c0|9q000|9d1c0|9d1c0|9d440|9cyk0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9cyk0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9cyk0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"114.5333|120|60|0",offsetIndices:"01212121212121212121212121212121212121212121232123212321232121212121212121212121212121212121212121232323232323232323232323232323233323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Atlantic/Bermuda",untils:"-kvj2fu|n4pr3u|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"259.3|240|180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Atlantic/Canary",untils:"-oytbtc|ctvupc|hhq7s0|905g0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"61.6|60|0|-60",offsetIndices:"01232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Atlantic/Cape_Verde",untils:"-u9rbs0|g06lc0|1mn180|fpqwc0|Infinity",offsets:"94.0667|120|60",offsetIndices:"01212"},{id:"Atlantic/Faeroe",untils:"-wcehew|127keuw|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"27.0667|0|-60",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Atlantic/Faroe",untils:"-wcehew|127keuw|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"27.0667|0|-60",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Atlantic/Jan_Mayen",untils:"-rzayo0|6qfs0|cgcqo0|15tsc0|7k800|9q000|9d1c0|9d1c0|9d1c0|9d1c0|70q5c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|b5uo0|7k800|7law00|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Atlantic/Madeira",untils:"-u9rek0|2bufw0|6zxg0|66580|bq800|73k00|bodc0|71pc0|bq800|73k00|bq800|71pc0|bq800|1b2g00|9b6o0|saio0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|st1c0|8n400|9d1c0|9d1c0|sg2o0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|bitc0|9d1c0|9ew00|88ao0|25p80|5reo0|3lpg0|779c0|1sqk0|6uao0|38qs0|6uao0|25p80|6hc00|38qs0|6uao0|25p80|6hc00|38qs0|8a5c0|9d1c0|9d9o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|s3400|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|5qbjo0|9d1c0|9q000|9d1c0|9d1c0|9d440|9cyk0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9cyk0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"67.6|60|0|-60",offsetIndices:"01212121212121212121212121212121212121212121232123212321232121212121212121212121212121212121212121232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Atlantic/Reykjavik",untils:"-wcwx9c|4rpd9c|ci2s0|69uk0|du840|4xp80|du840|p7bw0|4w040|9bdzw0|9d6w0|64g40|cyl80|64dc0|clpc0|6hc00|bvs00|6uao0|bvs00|6uao0|bvs00|6uao0|c8qo0|6hc00|c8qo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|Infinity",offsets:"88|60|0",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212"},{id:"Atlantic/South_Georgia",untils:"Infinity",offsets:"120",offsetIndices:"0"},{id:"Atlantic/St_Helena",untils:"-u9rgl4|Infinity",offsets:"16.1333|0",offsetIndices:"01"},{id:"Atlantic/Stanley",untils:"-u63pac|dbvxqc|8zzw0|9q2s0|8zzw0|a31g0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|4xp80|l1pus0|7k580|b5rw0|77c40|biqk0|id6s0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|biqk0|77c40|biqk0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|biqk0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5rw0|7kas0|b5xg0|77c40|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|bvp80|77c40|biqk0|77c40|biqk0|77c40|Infinity",offsets:"231.4|240|180|120",offsetIndices:"012121212121212323212121212121212121212121212121212121212121212121212"},{id:"Australia/ACT",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Adelaide",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|6hc00|c8qo0|7k800|b5uo0|6uao0|c8qo0|779c0|bitc0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-570|-630",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Brisbane",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|97zuo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|Infinity",offsets:"-600|-660",offsetIndices:"01010101010101010"},{id:"Australia/Broken_Hill",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-570|-630",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Canberra",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Currie",untils:"-rsj4w0|8zzw0|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|b5uo0|7k800|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|bvs00|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x6o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|7x6o0|asw00|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Darwin",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|Infinity",offsets:"-570|-630",offsetIndices:"010101010"},{id:"Australia/Eucla",untils:"-rnstlc|49s2c|cxfms0|4h180|9d440|9cyk0|ghf1g0|6hc00|4ir9c0|6hc00|40r400|5eg00|7p9hc0|5reo0|b5uo0|7x6o0|asw00|7x6o0|Infinity",offsets:"-525|-585",offsetIndices:"0101010101010101010"},{id:"Australia/Hobart",untils:"-rsj4w0|8zzw0|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|c9tms0|9d1c0|asw00|6uao0|bvs00|6uao0|bvs00|779c0|bvs00|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|b5uo0|7k800|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|bvs00|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x6o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|7x6o0|asw00|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/LHI",untils:"5tp880|c8uu0|6u7w0|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|777y0|b5w20|7k6m0|biuq0|7k6m0|biuq0|777y0|biuq0|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|7x5a0|b5w20|7k6m0|7x820|asum0|b5w20|7x5a0|asxe0|7x5a0|asxe0|7x5a0|b5w20|7k6m0|b5w20|7x5a0|asxe0|7k6m0|b5w20|8a3y0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9pym0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|Infinity",offsets:"-600|-630|-690|-660",offsetIndices:"0121212121313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313"},{id:"Australia/Lindeman",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|97zuo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|Infinity",offsets:"-600|-660",offsetIndices:"010101010101010101010"},{id:"Australia/Lord_Howe",untils:"5tp880|c8uu0|6u7w0|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|777y0|b5w20|7k6m0|biuq0|7k6m0|biuq0|777y0|biuq0|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|7x5a0|b5w20|7k6m0|7x820|asum0|b5w20|7x5a0|asxe0|7x5a0|asxe0|7x5a0|b5w20|7k6m0|b5w20|7x5a0|asxe0|7k6m0|b5w20|8a3y0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9pym0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9d2q0|9czy0|9q1e0|9czy0|9d2q0|9czy0|9d2q0|Infinity",
offsets:"-600|-630|-690|-660",offsetIndices:"0121212121313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313"},{id:"Australia/Melbourne",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|b5uo0|7x6o0|bitc0|779c0|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/North",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|Infinity",offsets:"-570|-630",offsetIndices:"010101010"},{id:"Australia/NSW",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Perth",untils:"-rnsric|49s2c|cxfms0|4h180|9d440|9cyk0|ghf1g0|6hc00|4ir9c0|6hc00|40r400|5eg00|7p9hc0|5reo0|b5uo0|7x6o0|asw00|7x6o0|Infinity",offsets:"-480|-540",offsetIndices:"0101010101010101010"},{id:"Australia/Queensland",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|97zuo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|Infinity",offsets:"-600|-660",offsetIndices:"01010101010101010"},{id:"Australia/South",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|6hc00|c8qo0|7k800|b5uo0|6uao0|c8qo0|779c0|bitc0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-570|-630",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Sydney",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Tasmania",untils:"-rsj4w0|8zzw0|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|c9tms0|9d1c0|asw00|6uao0|bvs00|6uao0|bvs00|779c0|bvs00|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|b5uo0|7k800|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|bvs00|7k800|bitc0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x6o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|7x6o0|asw00|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/Victoria",untils:"-rnsx2c|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|b5uo0|7x6o0|bitc0|779c0|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|7x6o0|asw00|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-600|-660",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Australia/West",untils:"-rnsric|49s2c|cxfms0|4h180|9d440|9cyk0|ghf1g0|6hc00|4ir9c0|6hc00|40r400|5eg00|7p9hc0|5reo0|b5uo0|7x6o0|asw00|7x6o0|Infinity",offsets:"-480|-540",offsetIndices:"0101010101010101010"},{id:"Australia/Yancowinna",untils:"-rnsvoc|49s2c|cxfms0|4h180|9d440|9cyk0|9q2s0|8zzw0|eeiqs0|64dc0|clpc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|8a5c0|asw00|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|779c0|b5uo0|7k800|bitc0|7k800|bitc0|779c0|bitc0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|8a5c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-570|-630",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101"},{id:"Brazil/Acre",untils:"-t85fg0|99kak0|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|amves0|2t2t80|Infinity",offsets:"271.2|300|240",offsetIndices:"01212121212121212121212121212121"},{id:"Brazil/DeNoronha",untils:"-t85lzw|99k8rw|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|514g40|7k580|biw40|cvw0|iq5g0|6h980|Infinity",offsets:"129.6667|120|60",offsetIndices:"0121212121212121212121212121212121212121"},{id:"Brazil/East",untils:"-t85jd8|99k8x8|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5k02s0|6onw0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|cyqs0|5ed80|dbpg0|64ak0|cyqs0|64ak0|cls40|5rbw0|dbpg0|51ek0|dbpg0|6h980|c8tg0|6h980|c8tg0|64ak0|c8tg0|6u7w0|bxpg0|7iak0|biw40|6u7w0|biw40|7k580|biw40|6u7w0|c8tg0|6h980|dbpg0|5ed80|cls40|64ak0|dfes0|5nmk0|c8tg0|6h980|dbpg0|5rbw0|bvus0|6h980|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6u7w0|c8tg0|64ak0|cls40|64ak0|cls40|6h980|c8tg0|6h980|c8tg0|6h980|c8tg0|6h980|dbpg0|5ed80|Infinity",offsets:"186.4667|180|120",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Brazil/West",untils:"-t85gvw|99k97w|9a9c0|9io40|99980|8p65g0|6zuo0|bs2o0|67zw0|cjxg0|69uk0|cjxg0|4ml80|5mf440|49mk0|haas0|316k0|cls40|4ml80|cls40|66580|cls40|67zw0|981s40|6u7w0|biw40|5rbw0|d0lg0|5ed80|2yy2s0|6h980|Infinity",offsets:"240.0667|240|180",offsetIndices:"01212121212121212121212121212121"},{id:"Canada/Atlantic",untils:"-z94k80|777go0|9et80|st9o0|a2vw0|ssyk0|5rbw0|cv1g0|69uk0|c6ys0|6kyk0|ci2s0|67zw0|ci2s0|6w2k0|bu040|7lzw0|bu040|66580|bu040|7lzw0|bu040|64ak0|cls40|5v180|cv1g0|6j3w0|c6ys0|79180|b42s0|7lzw0|b42s0|7yyk0|bu040|64ak0|dbpg0|66580|cls40|5ed80|bu040|7lzw0|b42s0|7lzw0|cjxg0|66580|bh1g0|7lzw0|b42s0|7lzw0|6uj00|1tzdw0|2dnc0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|tw040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|tw040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|1cm2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"254.4|240|180",offsetIndices:"0121212121212121212121212121212121212121212121212122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Canada/Central",untils:"-s0s7c0|7k580|tj700|a2vw0|9ok840|6u7w0|2a5hg0|1tz8c0|2dsw0|biw40|7x3w0|a31g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b7s40|7tek0|autg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9cyk0|9d440|7x3w0|1cm2s0|7k580|1cm2s0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|asw00|7x6o0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Canada/Eastern",untils:"-qzoxw0|a2vw0|7yx60|aqzy0|9q8c0|7jzo0|bw0c0|6bp80|cedg0|6h980|c8tg0|6h980|bvus0|776k0|biw40|776k0|biw40|776k0|biw40|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|xjeo0|1tzb40|2dq40|asys0|7x3w0|ast80|7x3w0|asys0|7x3w0|asys0|b5rw0|7xf00|ast80|7x9g0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"01010101010101010101010101010101010101010101011101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Canada/Mountain",untils:"-x1yazk|629ink|a2vw0|8n6s0|29ek0|h6lg0|9px80|905g0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|9l0g40|1tz5k0|2dvo0|tj1g0|7x3w0|ctzk40|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"453.8667|420|360",offsetIndices:"0121212121212122121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Canada/Newfoundland",untils:"-ris3ck|8bx80|ar440|a2vw0|9tjs0|53980|dkys0|9cyk0|9d440|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|9cyk0|9d440|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|8zzw0|9q2s0|9cyk0|9q2s0|8zzw0|9q2s0|8zzw0|7tmw0|1wfuk|8zzw0|a3480|7k580|b5xg0|7k580|b5xg0|7k580|biw40|776k0|biw40|7k580|b5xg0|7k580|b5xg0|1pb260|2dly0|biw40|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|biw40|7k580|ag040|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a2lo|afuk0|8a840|asqg0|7xc80|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8tec|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"210.8667|150.8667|210|150|90",offsetIndices:"01010101010101010101010101010101010102323232323232323323232323232323232323232323232323232323232323232323232323232323232323232323232323232324232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Canada/Pacific",untils:"-qzopk0|a2vw0|c5jxg0|1tz2s0|2dyg0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"0101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Canada/Saskatchewan",untils:"-xkq9yc|6l1hmc|a2vw0|60enw0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|1b6840|9cyk0|9d440|8zzw0|9q2s0|9cyk0|9q2s0|9cyk0|9d440|9cyk0|66gc0|1tz5k0|2dvo0|a31g0|9cyk0|a31g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|tj1g0|9cyk0|9d440|Infinity",offsets:"418.6|420|360",offsetIndices:"012121212121212121212121221212121212121212121212121212"},{id:"Canada/Yukon",untils:"-qzoms0|a2vw0|asys0|882c0|bmiwc0|1tz000|2e180|a7n3w0|9q000|tiyo0|6qp440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|Infinity",offsets:"540|480|420",offsetIndices:"01010110201212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"CET",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|8l9c0|ggp1c0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Chile/Continental",untils:"-vauawq|3dlssq|157b7a|f4e0q|49hzba|aye0q|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|534ik0|351g0|229zw0|2gt80|awo40|2mg00|b73400|7k580|c8tg0|6h980|a31g0|7x3w0|asys0|7x3w0|b5xg0|7k580|ag040|8a2k0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|e1h80|4olg0|e1h80|4olg0|c8nw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|Infinity",offsets:"282.7667|300|240|180",offsetIndices:"010202121212121212321232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Chile/EasterIsland",untils:"-jhfaew|ivmeuw|7k580|c8tg0|6h980|a31g0|7x3w0|asys0|7x3w0|b5xg0|7k580|ag040|8a2k0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|iq2o0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|e1h80|4olg0|e1h80|4olg0|c8nw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|Infinity",offsets:"437.4667|420|360|300",offsetIndices:"012121212121212121212121212123232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"CST6CDT",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Cuba",untils:"-n7762o|1icfyo|69uk0|62s040|4ofw0|e1ms0|51ek0|e1ms0|4ofw0|1fhs40|4ofw0|e1ms0|4ofw0|9s9k40|67zw0|cedg0|6h980|9o840|7yyk0|b5xg0|7k580|bvus0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|8a2k0|ag040|8bx80|ae5g0|8drw0|acas0|9cyk0|9d440|9px80|905g0|9px80|9q2s0|7x3w0|8a840|ast80|7x9g0|ast80|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|8a2k0|ag040|8a2k0|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|905g0|a2vw0|905g0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|8n400|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|8a5c0|afxc0|8a5c0|afxc0|7x6o0|1cm000|6uao0|bvs00|779c0|bitc0|6uao0|bvs00|779c0|bvs00|779c0|c8qo0|779c0|b5uo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|Infinity",offsets:"329.6|300|240",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"EET",untils:"3s9ms0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-120|-180",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Egypt",untils:"-fdls80|40d80|a31g0|7x3w0|a4w40|aqyk0|80ys0|b07w0|7tk40|b07w0|8jhg0|a8fw0|60go40|7el80|awo40|7v980|awqw0|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7tk40|ayd80|7tk40|b07w0|7tk40|ayd80|7tk40|ayd80|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|f9x80|3i040|eluk0|462s0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|b5rw0|7m5g0|awik0|7ves0|awik0|7ves0|ayd80|7ves0|awik0|7ves0|awik0|7ves0|aqvs0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7k580|b5xg0|6u7w0|bvus0|6h980|c8tg0|64ak0|cyqs0|5anw0|1jms0|12t80|1w22s0|25p80|1sw40|2vmk0|Infinity",offsets:"-120|-180",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Eire",untils:"-rzcmlr|6uao0|9pytr|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|3g8800|8a5c0|bvs00|8n400|a2yo0|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"25.35|-34.65|0|-60",offsetIndices:"01232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"EST",untils:"Infinity",offsets:"300",offsetIndices:"0"},{id:"EST5EDT",untils:"-r0ev80|ast80|7x9g0|ast80|bmtus0|1tzb40|2dq40|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Etc/GMT-0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/GMT-1",untils:"Infinity",offsets:"-60",offsetIndices:"0"},{id:"Etc/GMT-10",untils:"Infinity",offsets:"-600",offsetIndices:"0"},{id:"Etc/GMT-11",untils:"Infinity",offsets:"-660",offsetIndices:"0"},{id:"Etc/GMT-12",untils:"Infinity",offsets:"-720",offsetIndices:"0"},{id:"Etc/GMT-13",untils:"Infinity",offsets:"-780",offsetIndices:"0"},{id:"Etc/GMT-14",untils:"Infinity",offsets:"-840",offsetIndices:"0"},{id:"Etc/GMT-2",untils:"Infinity",offsets:"-120",offsetIndices:"0"},{id:"Etc/GMT-3",untils:"Infinity",offsets:"-180",offsetIndices:"0"},{id:"Etc/GMT-4",untils:"Infinity",offsets:"-240",offsetIndices:"0"},{id:"Etc/GMT-5",untils:"Infinity",offsets:"-300",offsetIndices:"0"},{id:"Etc/GMT-6",untils:"Infinity",offsets:"-360",offsetIndices:"0"},{id:"Etc/GMT-7",untils:"Infinity",offsets:"-420",offsetIndices:"0"},{id:"Etc/GMT-8",untils:"Infinity",offsets:"-480",offsetIndices:"0"},{id:"Etc/GMT-9",untils:"Infinity",offsets:"-540",offsetIndices:"0"},{id:"Etc/GMT",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/GMT+0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/GMT+1",untils:"Infinity",offsets:"60",offsetIndices:"0"},{id:"Etc/GMT+10",untils:"Infinity",offsets:"600",offsetIndices:"0"
},{id:"Etc/GMT+11",untils:"Infinity",offsets:"660",offsetIndices:"0"},{id:"Etc/GMT+12",untils:"Infinity",offsets:"720",offsetIndices:"0"},{id:"Etc/GMT+2",untils:"Infinity",offsets:"120",offsetIndices:"0"},{id:"Etc/GMT+3",untils:"Infinity",offsets:"180",offsetIndices:"0"},{id:"Etc/GMT+4",untils:"Infinity",offsets:"240",offsetIndices:"0"},{id:"Etc/GMT+5",untils:"Infinity",offsets:"300",offsetIndices:"0"},{id:"Etc/GMT+6",untils:"Infinity",offsets:"360",offsetIndices:"0"},{id:"Etc/GMT+7",untils:"Infinity",offsets:"420",offsetIndices:"0"},{id:"Etc/GMT+8",untils:"Infinity",offsets:"480",offsetIndices:"0"},{id:"Etc/GMT+9",untils:"Infinity",offsets:"540",offsetIndices:"0"},{id:"Etc/GMT0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/Greenwich",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/UCT",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/Universal",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/UTC",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Etc/Zulu",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Europe/Amsterdam",untils:"-s0dvkk|7v980|a51o0|7x6o0|a2yo0|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9b6o0|a2yo0|c51c0|6l1c0|902o0|9q000|ci000|682o0|bgyo0|79400|bitc0|779c0|bmio0|7gio0|bbeo0|7eo00|bd9c0|7ctc0|bf400|7ayo0|bvs00|6uao0|bko00|7idc0|b9k00|7gio0|bbeo0|7eo00|bf400|7ayo0|btxc0|21uc0|4uaz8|bitc0|779c0|bko00|7idc0|bd3s0|1aarpc|7k800|9q000|9d1c0|9d1c0|9d1c0|8l9c0|ggp1c0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-19.5333|-79.5333|-80|-20|-120|-60",offsetIndices:"010101010101010101010101010101010101010101012323234545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545"},{id:"Europe/Andorra",untils:"-c4xmo0|k3ctg0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Astrakhan",untils:"-nu2zkc|37bv8c|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9q000|9d1c0|s3400|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|qnc40|Infinity",offsets:"-192.2|-180|-240|-300",offsetIndices:"012323232323232323212121212121212121212121212121212121212121212"},{id:"Europe/Athens",untils:"-rvv0cg|8bjasg|2vmk0|4hiw40|16ik0|scog0|7lx40|9o2k0|9eys0|4atzw0|6djw0|bplus0|bq800|71uw0|9d1c0|902o0|91xc0|9o5c0|905g0|9qgo0|9akg0|9iik0|99980|9dcg0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-94.8667|-120|-180|-60",offsetIndices:"012121313121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Belfast",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Belgrade",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Berlin",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|2o7w0|6bs00|2txg0|7k800|91xc0|9b9g0|1sqk0|2inw0|51k40|a2yo0|8n400|9q000|902o0|fx91c0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120|-180",offsetIndices:"01010101010101210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Bratislava",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|9d1c0|b5uo0|7vc00|2vs40|4bk00|2vmk0|8n400|a2yo0|8n400|9o5c0|91xc0|fe6000|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120|0",offsetIndices:"01010101010101010201010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Brussels",untils:"-ss5uo0|rrx80|7vc00|a4yw0|7x6o0|asw00|7x6o0|2wh40|5omo0|b5uo0|6uao0|cyo00|7ayo0|bko00|7rmo0|a2yo0|a2yo0|8n400|902o0|9q000|9d1c0|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|90b00|a2yo0|8n400|9q000|902o0|a2yo0|8n400|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|4deo0|1a36k0|7k800|9q000|9d1c0|8l9c0|a4tc0|8l9c0|clpc0|79400|fwu800|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0121212101010101010101010101010101010101010101010101212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Bucharest",untils:"-k29zi0|fj8m0|6w5c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|kp0dc0|6h980|9q000|905g0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9cvs0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9cyk0|9d440|9cyk0|9q2s0|ast80|7xhs0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-104.4|-120|-180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Budapest",untils:"-s0e080|7ves0|a4yw0|7x6o0|a31g0|8n180|autg0|bgvw0|b5jeg0|th9k0|7k800|9q000|9d1c0|9d1c0|awd00|9ew00|7q0c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|a4tc0|9q000|1va2g0|6u7w0|bxpg0|6u7w0|cjxg0|64ak0|cluw0|64g40|br3ek0|905g0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Busingen",untils:"-eyh6o0|7x6o0|asw00|7x6o0|k2zus0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Chisinau",untils:"-r2p1bo|70f1to|fj8m0|6w5c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|geqo0|ha580|oc8g0|7k800|9q000|9d1c0|7cl00|j3pbw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|25p80|7kdk0|9d1c0|9d1c0|9cvs0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|ast80|7xf00|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-115|-104.4|-120|-180|-60|-240",offsetIndices:"012323232323232323232424235353535353535353535323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Copenhagen",untils:"-rzo2w0|75bw0|cbs2w0|1aco80|7k800|9q000|9d1c0|9d1c0|9d1c0|6y000|dbmo0|6bs00|clpc0|51hc0|e1k00|4oio0|giutc0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Dublin",untils:"-rzcmlr|6uao0|9pytr|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|3g8800|8a5c0|bvs00|8n400|a2yo0|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"25.35|-34.65|0|-60",offsetIndices:"01232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Gibraltar",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|d0tp80|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"010101010101010101010101010101010101010101010101012121212121010121010101010101010101012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Guernsey",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Helsinki",untils:"-peghyd|ax3tqd|9gqo0|k31s80|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-99.8167|-120|-180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Isle_of_Man",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Istanbul",untils:"-ux9xew|2wvx6w|7v980|1tjc40|aunw0|88dg0|9et80|8yas0|a2vw0|tzpg0|79180|awo40|7v980|7p4040|4zjw0|2vs40|f4d80|9vms0|1u5ek0|c5440|69uk0|acas0|8n180|a31g0|8n180|9q2s0|8zzw0|a31g0|8zzw0|a31g0|8n180|5md9g0|o9zw0|a6qs0|75bw0|4iwyw0|7x6o0|7kas0|b5rw0|75hg0|bkl80|77c40|biqk0|7x9g0|a2vw0|8n6s0|4iqc0|2nkw80|38l80|kdes0|8qtc0|8a5c0|9ew00|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|902o0|9q000|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7kdk0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7m2o0|b4000|7k800|b5uo0|7x6o0|asw00|7z1c0|ar1c0|7x6o0|bitc0|779c0|8fe80|Infinity",offsets:"-116.9333|-120|-180|-240",offsetIndices:"0121212121212121212121212121212121212121212121223212121212121212121212121212121212121212121212121212121212121212122"},{id:"Europe/Jersey",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Kaliningrad",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|el00|z6o0|9kd80|82tg0|i9avw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-60|-120|-180|-240",offsetIndices:"01010101010101121232323232323232322121212121212121212121212121212121212121212121"},{id:"Europe/Kiev",untils:"-nu11ng|37a03g|5vd6k0|kzv40|7k800|9q000|1oyg0|jipzs0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|51ek0|neqw0|9cvs0|9cyk0|9d440|9cyk0|9d440|9cyk0|9dcg0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-122.0667|-120|-180|-60|-240",offsetIndices:"0121313242424242424242424242121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Kirov",untils:"-qcx400|5q5zo0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9q000|9d1c0|s3400|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-198.8|-180|-240|-300",offsetIndices:"01232323232323232321212121212121212121212121212121212121212121"},{id:"Europe/Lisbon",untils:"-u9rhc0|2bufw0|6zxg0|66580|bq800|73k00|bodc0|71pc0|bq800|73k00|bq800|71pc0|bq800|1b2g00|9b6o0|saio0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|st1c0|8n400|9d1c0|9d1c0|sg2o0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|bitc0|9d1c0|9ew00|88ao0|25p80|5reo0|3lpg0|779c0|1sqk0|6uao0|38qs0|6uao0|25p80|6hc00|38qs0|6uao0|25p80|6hc00|38qs0|8a5c0|9d1c0|9d9o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|s3400|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|5gyl40|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d440|9cyk0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9cyk0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"36.75|0|-60|-120",offsetIndices:"012121212121212121212121212121212121212121212321232123212321212121212121212121212121212121212121212121212121212121212121212121212122323232212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Ljubljana",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/London",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",
offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Luxembourg",untils:"-y89550|68l290|75hg0|ast80|796s0|at1k0|7x6o0|3lh40|4zmo0|b6300|6u2c0|cytk0|7at40|bktk0|7rh40|a31g0|a2vw0|8n9k0|8zx40|9q2s0|9et80|9b9g0|a2vw0|8n6s0|9px80|905g0|a2vw0|905g0|a2vw0|8ncc0|9q000|902o0|a2yo0|8n400|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|42ao0|1aeak0|7k800|9q000|9d1c0|8n400|a2yo0|8l9c0|clpc0|79400|fwu800|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-24.6|-60|-120|0",offsetIndices:"0121212131313131313131313131313131313131313131313131212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Madrid",untils:"-qzlus0|8yas0|9cyk0|9eys0|2d2vw0|8sqs0|ssyk0|8n6s0|9px80|905g0|a2yo0|902o0|a2vw0|8n6s0|40lh80|5k2s0|9cyk0|1frw0|7z1c0|j1c80|8a2k0|13yt80|685g0|brzw0|8n6s0|a2vw0|8n6s0|a2vw0|8n6s0|a2vw0|8n6s0|1clx80|7x9g0|cswik0|905g0|9px80|905g0|8zzw0|9d440|9px80|905g0|9q5k0|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"010101010101010101210121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Malta",untils:"-rymys0|64ak0|9d440|9et80|88dg0|aunw0|7ig40|b5rw0|8n6s0|9cyk0|aau2s0|18r9k0|7k800|9q000|9b6o0|8n400|a4tc0|8j940|9f1k0|afxc0|89zs0|afxc0|7kdk0|b5uo0|979rs0|6h980|cls40|64dc0|clpc0|64dc0|cyo00|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|9b6o0|9d1c0|ahs00|7m2o0|b45k0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|a4w40|8y580|9q2s0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Mariehamn",untils:"-peghyd|ax3tqd|9gqo0|k31s80|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-99.8167|-120|-180",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Minsk",untils:"-nu113c|379zjc|5r1mk0|pbf40|7k800|9q000|9d1c0|4oac0|j6dmk0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|sg2o0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|Infinity",offsets:"-110|-120|-180|-60|-240",offsetIndices:"01213131242424242424242424221212121212121212121212121212121212121212"},{id:"Europe/Monaco",untils:"-uozn3l|2qx1nl|5luo0|8y800|a4tc0|7vc00|auqo0|7idc0|b7pc0|6sg00|cyo00|7ayo0|bko00|7rmo0|a2yo0|bvs00|6uao0|902o0|9q000|9d1c0|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51po0|mdbo0|7x3w0|7x9g0|c8w80|7k800|9q000|9d1c0|9nzs0|922w0|8l9c0|fxlx80|9cyk0|9q5k0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-9.35|0|-60|-120",offsetIndices:"01212121212121212121212121212121212121212121212121232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Moscow",untils:"-rx5dmh|ipzua|97hc0|7yyk0|5i840|d9p80|1jwk7|2cvk0|s8o00|1qvw0|8fpc0|1jms0|is040|412as0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d440|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-150.2833|-151.3167|-211.3167|-271.3167|-240|-180|-300|-120",offsetIndices:"012132345464575454545454545454545455754545454545454545454545454545454545454545"},{id:"Europe/Nicosia",untils:"-p4bq6g|rvhxyg|9cyk0|b42s0|7nuk0|8yas0|8zzw0|9q2s0|9et80|9b9g0|9cyk0|9q2s0|8zzw0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|9cyk0|9d440|9cyk0|9d440|at4c0|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-133.4667|-120|-180",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Oslo",untils:"-rzayo0|6qfs0|cgcqo0|15tsc0|7k800|9q000|9d1c0|9d1c0|9d1c0|9d1c0|70q5c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|b5uo0|7k800|7law00|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Paris",untils:"-uozn1x|2qx1lx|5luo0|8y800|a4tc0|7vc00|auqo0|7idc0|b7pc0|6sg00|cyo00|7ayo0|bko00|7rmo0|a2yo0|bvs00|6uao0|902o0|9q000|9d1c0|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|9d1c0|9d1c0|902o0|a2yo0|9d1c0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51po0|5p8w0|18rcc0|7k800|9q000|9d1c0|7efo0|29k40|922w0|8l9c0|fxlx80|9cyk0|9q5k0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-9.35|0|-60|-120",offsetIndices:"0121212121212121212121212121212121212121212121212123232332323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Podgorica",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Prague",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|9d1c0|b5uo0|7vc00|2vs40|4bk00|2vmk0|8n400|a2yo0|8n400|9o5c0|91xc0|fe6000|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120|0",offsetIndices:"01010101010101010201010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Riga",untils:"-qznlky|7x6o0|a4tc0|2mg00|3myns0|7fhlky|gz180|p5v40|7k800|9q000|9d1c0|9d1c0|k7s0|j14ns0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d440|asw00|7x6o0|asw00|7x6o0|b5uo0|qaao0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-96.5667|-156.5667|-120|-180|-60|-240",offsetIndices:"010102324242435353535353535353323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Rome",untils:"-rymys0|64ak0|9d440|9et80|88dg0|aunw0|7ig40|b5rw0|8n6s0|9cyk0|aau2s0|18r9k0|7k800|9q000|9d1c0|8l9c0|a4tc0|8j940|9f1k0|afxc0|89zs0|afxc0|7kdk0|b5uo0|979rs0|6h980|cls40|64dc0|clpc0|64dc0|cyo00|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|clpc0|64dc0|c8qo0|6hc00|clpc0|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|c8qo0|6hc00|9q5k0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Samara",untils:"-qcx400|5q5zo0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9q000|jt1g0|89zs0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|j3440|7k800|Infinity",offsets:"-200.3333|-180|-240|-300",offsetIndices:"0123232323232323232121232323232323232323232323232323232323212"},{id:"Europe/San_Marino",untils:"-rymys0|64ak0|9d440|9et80|88dg0|aunw0|7ig40|b5rw0|8n6s0|9cyk0|aau2s0|18r9k0|7k800|9q000|9d1c0|8l9c0|a4tc0|8j940|9f1k0|afxc0|89zs0|afxc0|7kdk0|b5uo0|979rs0|6h980|cls40|64dc0|clpc0|64dc0|cyo00|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|clpc0|64dc0|c8qo0|6hc00|clpc0|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|c8qo0|6hc00|9q5k0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Sarajevo",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Saratov",untils:"-qcx400|5q5zo0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|s3400|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|13m040|Infinity",offsets:"-184.3|-180|-240|-300",offsetIndices:"012323232323232321212121212121212121212121212121212121212121212"},{id:"Europe/Simferopol",untils:"-nu12ao|37a0qo|5xiyk0|iu340|7k800|9q000|9d1c0|iac0|jajmk0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|eeio0|wrjw0|9cyk0|9d440|9cyk0|9d440|1sqk0|7k580|9d440|9cyk0|9q2s0|at4c0|7x9g0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x3w0|asqg0|Infinity",offsets:"-136|-120|-180|-60|-240",offsetIndices:"012131312424242424242424242121212424242212121212121212121212121212121212142"},{id:"Europe/Skopje",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Sofia",untils:"-e6dzw0|7k800|9q000|9d1c0|9d1c0|9d440|hqq240|9eys0|9o2k0|92040|9o2k0|90880|9pug0|90b00|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9cvs0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|ast80|7xhs0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-120|-60|-180",offsetIndices:"01010102020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020"},{id:"Europe/Stockholm",untils:"-rzo2w0|75hg0|x5bew0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Tallinn",untils:"-r3exx0|3re10|7x6o0|et6g0|ygov0|a1zgd0|ktx80|l94g0|7k800|9q000|9d1c0|8uac0|j27mk0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asys0|7x6o0|b5uo0|19dc00|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-99|-60|-120|-180|-240",offsetIndices:"012102321212343434343434343433232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Tirane",untils:"-t85vo8|dt2gw8|18pew0|7k800|m800|g7ot40|7rjw0|autg0|7x3w0|ayis0|7x3w0|b5xg0|7k580|b42s0|7lzw0|b42s0|7lzw0|b42s0|7x3w0|ahus0|7x3w0|b5xg0|7x3w0|a4w40|8jbw0|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-79.3333|-60|-120",offsetIndices:"01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Tiraspol",untils:"-r2p1bo|70f1to|fj8m0|6w5c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|geqo0|ha580|oc8g0|7k800|9q000|9d1c0|7cl00|j3pbw0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|25p80|7kdk0|9d1c0|9d1c0|9cvs0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|ast80|7xf00|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-115|-104.4|-120|-180|-60|-240",offsetIndices:"012323232323232323232424235353535353535353535323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"Europe/Ulyanovsk",untils:"-qcx400|5q5zo0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9q000|iq5g0|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|qnc40|Infinity",offsets:"-193.6|-180|-240|-300|-120",offsetIndices:"01232323232323232321214121212121212121212121212121212121212121212"},{id:"Europe/Uzhgorod",untils:"-fizzw0|1cm000|7k800|9q000|9d1c0|al900|cnms0|int140|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|eeio0|e1sc0|iprk0|9cyk0|9d440|9cyk0|9d440|9cyk0|9dcg0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120|-180|-240",offsetIndices:"010101023232323232323232320121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Vaduz",untils:"-eyh6o0|7x6o0|asw00|7x6o0|k2zus0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Vatican",untils:"-rymys0|64ak0|9d440|9et80|88dg0|aunw0|7ig40|b5rw0|8n6s0|9cyk0|aau2s0|18r9k0|7k800|9q000|9d1c0|8l9c0|a4tc0|8j940|9f1k0|afxc0|89zs0|afxc0|7kdk0|b5uo0|979rs0|6h980|cls40|64dc0|clpc0|64dc0|cyo00|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|clpc0|64dc0|c8qo0|6hc00|clpc0|64dc0|clpc0|64dc0|c8qo0|6hc00|clpc0|6hc00|c8qo0|6hc00|9q5k0|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Vienna",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|t6000|8a5c0|a7a800|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|iio0|ivmo0|91xc0|9b6o0|9d1c0|a2yo0|8n400|gfyyg0|8zzw0|9d9o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Vilnius",untils:"-rns980|1g224o|e75nc|4kqk0|acbs40|gpp40|pits0|7k800|9q000|9d1c0|65zo0|j4vx80|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x9g0|asw00|7x6o0|b5uo0|1s3eo0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-84|-95.6|-60|-120|-180|-240",offsetIndices:"012324323234545454545454545443434343434343434332334343434343434343434343434343434343434343434343434343434343434343434343"},{id:"Europe/Volgograd",untils:"-q3cw84|5glrw4|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|iq5g0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|s3400|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|239c40|Infinity",offsets:"-177.6667|-180|-240|-300",offsetIndices:"012323232323232321212121212121212121212121212121212121212121212"},{id:"Europe/Warsaw",untils:"-se9yk0|dvyc0|7ves0|a4yw0|7x6o0|asw00|7x6o0|aunw0|7x6o0|1evbs0|9fcwc0|18cao0|7k800|9q000|9d1c0|9gnw0|an980|9kd80|8fs40|922w0|ar1c0|7x6o0|a2yo0|8n400|9q000|902o0|4013w0|64dc0|9d1c0|9d1c0|clpc0|6hc00|9d1c0|9d1c0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|clpc0|64dc0|6j4tc0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",
offsets:"-84|-60|-120|-180",offsetIndices:"012121223212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Zagreb",untils:"-ezayw0|swz00|7k800|9q000|9d1c0|9d1c0|b7pc0|6qlc0|jl1hc0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Europe/Zaporozhye",untils:"-nu12hc|37a0xc|5u1180|mc0g0|7k800|9q000|12qg0|jjc7s0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9cvs0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9dcg0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-140|-120|-180|-60|-240",offsetIndices:"01213132424242424242424242422121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Europe/Zurich",untils:"-eyh6o0|7x6o0|asw00|7x6o0|k2zus0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"GB-Eire",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"GB",untils:"-rzcns0|6uao0|9q000|8c000|9o5c0|9ruo0|9b6o0|9ew00|9b6o0|auqo0|88ao0|9ew00|8y800|a2yo0|a2yo0|7k800|asw00|8a5c0|asw00|8n400|a2yo0|8n400|9q000|902o0|afxc0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|9d1c0|a2yo0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|a2yo0|b5uo0|51hc0|mbmk0|51hc0|c8qo0|6hc00|c8qo0|6uao0|bvs00|8n400|a4tc0|5clc0|4bms0|9q000|902o0|8a5c0|1frw0|64dc0|4bms0|6uao0|bvs00|7x6o0|asw00|8n400|9q000|902o0|9q000|9d1c0|9q000|902o0|8n400|9q000|902o0|a2yo0|8n400|afxc0|8n400|9q000|902o0|a2yo0|8n400|a2yo0|8n400|9q000|902o0|902o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|5reo0|1xhuo0|779c0|bitc0|779c0|bitc0|779c0|bitc0|779c0|bitc0|7k800|b5uo0|7k800|b5uo0|7k800|bitc0|779c0|bitc0|779c0|bitc0|7x3w0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60|-120",offsetIndices:"0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"GMT-0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"GMT",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"GMT+0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"GMT0",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Greenwich",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Hongkong",untils:"-y0i0s0|j44dk0|5k000|4d4y0|2195i0|7x3w0|bj320|6uao0|bvs00|7x6o0|9d1c0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|8a5c0|asw00|7x6o0|ast80|77c40|biqk0|77c40|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|77c40|biqk0|77c40|bvp80|6udg0|bvp80|6udg0|bvp80|77c40|biqk0|77c40|biqk0|8n6s0|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9cyk0|1c9440|8a2k0|Infinity",offsets:"-456.7|-480|-540|-510",offsetIndices:"0123212121212121212121212121212121212121212121212121212121212121212121"},{id:"HST",untils:"Infinity",offsets:"600",offsetIndices:"0"},{id:"Iceland",untils:"-wcwx9c|4rpd9c|ci2s0|69uk0|du840|4xp80|du840|p7bw0|4w040|9bdzw0|9d6w0|64g40|cyl80|64dc0|clpc0|6hc00|bvs00|6uao0|bvs00|6uao0|bvs00|6uao0|c8qo0|6hc00|c8qo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|7x6o0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|asw00|8a5c0|Infinity",offsets:"88|60|0",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212"},{id:"Indian/Antananarivo",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Indian/Chagos",untils:"-wvpc2s|1ag64us|Infinity",offsets:"-289.6667|-300|-360",offsetIndices:"012"},{id:"Indian/Christmas",untils:"Infinity",offsets:"-420",offsetIndices:"0"},{id:"Indian/Cocos",untils:"Infinity",offsets:"-390",offsetIndices:"0"},{id:"Indian/Comoro",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Indian/Kerguelen",untils:"-afrs00|Infinity",offsets:"0|-300",offsetIndices:"01"},{id:"Indian/Mahe",untils:"-x6pjlo|Infinity",offsets:"-221.8|-240",offsetIndices:"01"},{id:"Indian/Maldives",untils:"-57x6y0|Infinity",offsets:"-294|-300",offsetIndices:"01"},{id:"Indian/Mauritius",untils:"-wvp9bc|13jnu7c|8bx80|dd0wc0|7x3w0|Infinity",offsets:"-230|-240|-300",offsetIndices:"012121"},{id:"Indian/Mayotte",untils:"-lnsetg|s8mhg|57v020|afrrb0|Infinity",offsets:"-147.2667|-180|-150|-165",offsetIndices:"01231"},{id:"Indian/Reunion",untils:"-uks29s|Infinity",offsets:"-221.8667|-240",offsetIndices:"01"},{id:"Iran",untils:"-s6m6uw|fnolc0|gm3h4w|777y0|b07w0|3pes0|42c20|9cyk0|9gtg0|9kd80|5ja5g0|7avw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|1av440|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9d440|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|9b9g0|9gnw0|Infinity",offsets:"-205.7333|-210|-240|-300|-270",offsetIndices:"00123214141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141"},{id:"Israel",untils:"-r50eig|bp54yg|19f3w0|7rv00|b02c0|7tk40|b07w0|8jhg0|a8lg0|8jhg0|a8ac0|t9s40|56vs0|35700|9b3w0|9gtg0|8jbw0|7tmw0|a6ig0|biyw0|8a5c0|9d1c0|902o0|7x6o0|e1eg0|4ofw0|dzxo0|4q500|doo40|64iw0|auqo0|7i500|8rfms0|51ek0|9q2s0|6u7w0|2khpg0|25s00|1weyo0|5reo0|bxmo0|7x3w0|cls40|5rbw0|bbhg0|7rjw0|asys0|7k580|c8tg0|6h980|ag040|7x3w0|asys0|8a2k0|asys0|8a2k0|ap9g0|80t80|ap9g0|7nuk0|b2840|80t80|9zc40|9iik0|9kis0|93p80|9mdg0|8qqk0|apf00|7x3w0|biw40|8zx40|9io40|8n180|9kis0|9vh80|8ulg0|9px80|9mdg0|8n180|9tuw0|9tmk0|8wg40|9gnw0|99es0|8qqk0|9zc40|9tmk0|8wg40|9gnw0|99es0|8qqk0|acas0|9gnw0|99es0|93p80|9mdg0|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|7tk40|b9h80|7glg0|b9h80|7glg0|b9h80|7glg0|b9h80|7tk40|awik0|7tk40|awik0|Infinity",offsets:"-140.6667|-120|-180|-240",offsetIndices:"012121212121321212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Jamaica",untils:"-u85og2|wbl182|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|Infinity",offsets:"307.1667|300|240",offsetIndices:"0121212121212121212121"},{id:"Japan",untils:"-bb4900|6uao0|afxc0|8a5c0|c8qo0|6hc00|c8qo0|6hc00|Infinity",offsets:"-540|-600",offsetIndices:"010101010"},{id:"Kwajalein",untils:"-h817w0|27sas0|1hjus0|ddxug0|cgv6k0|Infinity",offsets:"-660|-600|-540|720|-720",offsetIndices:"012034"},{id:"Libya",untils:"-q3gfrw|gl6ajw|422c0|xado0|4bbo0|wrpg0|4s580|1kdpg0|c05bw0|4mqs0|9et80|9d440|9et80|9eys0|9et80|9mdg0|95jw0|9io40|9cyk0|99es0|9et80|9eys0|9et80|9d440|9et80|b2840|3cf3w0|9kis0|9et80|7vqyw0|75eo0|asw00|Infinity",offsets:"-52.7333|-60|-120",offsetIndices:"012121212121212121212121212122122"},{id:"MET",untils:"-s0e080|7ves0|a4yw0|7x6o0|asw00|7x6o0|b8qdc0|1cm000|7k800|9q000|9d1c0|9d1c0|9d1c0|8l9c0|ggp1c0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-60|-120",offsetIndices:"01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Mexico/BajaNorte",untils:"-p1u1s0|11jrw0|1sns00|1sgdc0|71s40|9cyk0|5iidg0|1q6700|4lfk0|190g40|eluk0|2r4o80|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|84qys0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|77c40|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"468.0667|420|480",offsetIndices:"012121211212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"Mexico/BajaSur",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|591h80|3ie2s0|axvpg0|dpgw40|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"425.6667|420|360|480",offsetIndices:"0121212131212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Mexico/General",untils:"-p1u4k0|2u7jw0|1sgdc0|8n400|7thc0|9eys0|3knek0|776k0|rf440|5t6k0|1evk40|71mk0|30p1g0|8n180|nufxo0|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|9q2s0|7k580|9q2s0|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|Infinity",offsets:"396.6|420|360|300",offsetIndices:"012121232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"MST",untils:"Infinity",offsets:"420",offsetIndices:"0"},{id:"MST7MDT",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|1tz5k0|2dvo0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Navajo",untils:"-r0epo0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|2vmk0|ataw40|1tz5k0|2dvo0|a7n9g0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360",offsetIndices:"01010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"NZ-CHAT",untils:"-ciya10|f1tq90|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-735|-765|-825",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"NZ",untils:"-m01p20|64ak0|biw40|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|8a3y0|afyq0|8a3y0|afyq0|afvy0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|b5ta0|7k9e0|b5ta0|7x820|hsl2m0|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-690|-750|-720|-780",offsetIndices:"01020202020202020202020202023232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Pacific/Apia",untils:"-usiiv4|kcrmt4|vp3la0|9odo0|902o0|4zbk0|4qog0|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"686.9333|690|660|600|-840|-780",offsetIndices:"01232345454545454545454545454545454545454545454545454545454"},{id:"Pacific/Auckland",untils:"-m01p20|64ak0|biw40|7x5a0|asxe0|7x5a0|asxe0|7x5a0|asxe0|8a3y0|afyq0|8a3y0|afyq0|afvy0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|asum0|7x820|b5ta0|7k9e0|b5ta0|7x820|hsl2m0|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-690|-750|-720|-780",offsetIndices:"01020202020202020202020202023232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Pacific/Bougainville",untils:"-ecsh40|1n05g0|1071c40|Infinity",offsets:"-600|-540|-660",offsetIndices:"0102"},{id:"Pacific/Chatham",untils:"-ciya10|f1tq90|5reo0|clpc0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6uao0|c8qo0|6hc00|b5uo0|8a5c0|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|afxc0|8a5c0|afxc0|8a5c0|afxc0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|8n400|a2yo0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|a2yo0|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|902o0|9q000|9d1c0|9q000|902o0|9q000|902o0|Infinity",offsets:"-735|-765|-825",offsetIndices:"012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212"},{id:"Pacific/Chuuk",untils:"-su4zs0|29hes0|bkenw0|29fk40|Infinity",offsets:"-600|-540",offsetIndices:"01010"},{id:"Pacific/Easter",untils:"-jhfaew|ivmeuw|7k580|c8tg0|6h980|a31g0|7x3w0|asys0|7x3w0|b5xg0|7k580|ag040|8a2k0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|iq2o0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|9cyk0|9d440|7x3w0|asys0|7x3w0|b5xg0|7k580|9q2s0|8zzw0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|a31g0|9px80|9q2s0|7x3w0|b5xg0|7k580|b5xg0|7k580|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|8n180|a31g0|7x3w0|asys0|8zzw0|9q2s0|ast80|5eis0|cyl80|6hes0|c8nw0|6udg0|bvp80|6udg0|vonw0|4olg0|e1h80|4olg0|e1h80|4olg0|c8nw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|b5rw0|7x9g0|ast80|7x9g0|Infinity",offsets:"437.4667|420|360|300",offsetIndices:"012121212121212121212121212123232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323"},{id:"Pacific/Efate",untils:"-u964i4|11f4ba4|9cyk0|awo40|7tek0|9q2s0|8zzw0|9q2s0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9q2s0|64ak0|e1ms0|4ofw0|Infinity",offsets:"-673.2667|-660|-720",offsetIndices:"0121212121212121212121"},{id:"Pacific/Enderbury",untils:"535io0|7yiqk0|Infinity",offsets:"720|660|-780",offsetIndices:"012"},{id:"Pacific/Fakaofo",untils:"lx0jw0|Infinity",offsets:"660|-780",offsetIndices:"01"},{id:"Pacific/Fiji",untils:"-sa2x4w|17bs00w|64dc0|cyo00|5reo0|53a5c0|64dc0|asw00|6uao0|bvs00|4oio0|e1k00|4oio0|eeio0|4bh80|erk40|3ylc0|erhc0|3ylc0|f4g00|3lmo0|f4g00|3lmo0|f4g00|3lmo0|fheo0|38o00|fheo0|3lmo0|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|3lmo0|f4g00|3lmo0|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|3lmo0|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|38o00|fheo0|3lmo0|f4g00|3lmo0|Infinity",offsets:"-715.7333|-720|-780",offsetIndices:"0121212121212121212121212121212121212121212121212121212121212121"},{id:"Pacific/Funafuti",untils:"Infinity",offsets:"-720",offsetIndices:"0"},{id:"Pacific/Galapagos",untils:"-kcr62o|spdryo|3lsas0|3jp80|Infinity",offsets:"358.4|300|360",offsetIndices:"01212"},{id:"Pacific/Gambier",untils:"-tvndoc|Infinity",offsets:"539.8|540",offsetIndices:"01"},{id:"Pacific/Guadalcanal",untils:"-tvowac|Infinity",offsets:"-639.8|-660",offsetIndices:"01"},{id:"Pacific/Guam",untils:"-en8eg0|1dl9g0|7s1k40|txp80|3frms0|qdrpo|7kgac|3ljw0|c8tg0|6u7w0|bvus0|6u7w0|16uo40|3ljw0|16aas0|4ivxo|cls2c|6h980|c65zw0|Infinity",offsets:"-600|-540|-660",offsetIndices:"01020202020202020200"},{id:"Pacific/Honolulu",untils:"-j50la0|13l00|4jvb00|1tyvu0|2e5e0|votg0|Infinity",offsets:"630|570|600",offsetIndices:"0101102"},{id:"Pacific/Johnston",untils:"-j50la0|13l00|4jvb00|1tyvu0|2e5e0|votg0|Infinity",offsets:"630|570|600",offsetIndices:"0101102"},{id:"Pacific/Kiritimati",untils:"535eyo|7yirhc|Infinity",offsets:"640|600|-840",offsetIndices:"012"},{id:"Pacific/Kosrae",untils:"-su52k0|29hhk0|9cmd40|27sas0|29fk40|cm2540|f9l3w0|Infinity",offsets:"-660|-540|-600|-720",offsetIndices:"01021030"},{id:"Pacific/Kwajalein",untils:"-h817w0|27sas0|1hjus0|ddxug0|cgv6k0|Infinity",offsets:"-660|-600|-540|720|-720",offsetIndices:"012034"},{id:"Pacific/Majuro",untils:"-su52k0|29hhk0|9cmd40|27sas0|1h6w40|deat40|Infinity",offsets:"-660|-540|-600|-720",offsetIndices:"0102103"},{id:"Pacific/Marquesas",untils:"-tvncu0|Infinity",offsets:"558|570",offsetIndices:"01"},{id:"Pacific/Midway",untils:"-usij20|Infinity",offsets:"682.8|660",offsetIndices:"01"},{id:"Pacific/Nauru",untils:"-pjxiws|ba66ys|1kwca0|hfzda0|Infinity",offsets:"-667.6667|-690|-540|-720",offsetIndices:"01213"},{id:"Pacific/Niue",untils:"-9wyz6o|ehcj4o|Infinity",offsets:"680|690|660",offsetIndices:"012"},{id:"Pacific/Norfolk",untils:"-9x0ps0|cfj8q0|6hc00|l6nk00|239aq0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|Infinity",offsets:"-672|-690|-750|-660|-720",offsetIndices:"012134343434343434343434343434343434343434"},{id:"Pacific/Noumea",untils:"-u9645o|ye0ixo|4dbw0|ecqs0|4f6k0|99p700|4oio0|Infinity",offsets:"-665.8|-660|-720",offsetIndices:"01212121"},{id:"Pacific/Pago_Pago",untils:"-usij20|Infinity",offsets:"682.8|660",offsetIndices:"01"},{id:"Pacific/Palau",untils:"Infinity",offsets:"-540",offsetIndices:"0"},{id:"Pacific/Pitcairn",untils:"es2cy0|Infinity",offsets:"510|480",offsetIndices:"01"},{id:"Pacific/Pohnpei",untils:"-su52k0|29hhk0|9cmd40|27sas0|29fk40|Infinity",offsets:"-660|-540|-600",offsetIndices:"010210"},{id:"Pacific/Ponape",untils:"-su52k0|29hhk0|9cmd40|27sas0|29fk40|Infinity",offsets:"-660|-540|-600",offsetIndices:"010210"},{id:"Pacific/Port_Moresby",untils:"Infinity",offsets:"-600",offsetIndices:"0"},{id:"Pacific/Rarotonga",untils:"4mj960|5rbw0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6ham0|c8s20|6u9a0|c8s20|6ham0|c8s20|6ham0|c8s20|6ham0|Infinity",offsets:"630|570|600",offsetIndices:"012121212121212121212121212"},{id:"Pacific/Saipan",untils:"-en8eg0|1dl9g0|7s1k40|txp80|3frms0|qdrpo|7kgac|3ljw0|c8tg0|6u7w0|bvus0|6u7w0|16uo40|3ljw0|16aas0|4ivxo|cls2c|6h980|c65zw0|Infinity",offsets:"-600|-540|-660",offsetIndices:"01020202020202020200"},{id:"Pacific/Samoa",untils:"-usij20|Infinity",offsets:"682.8|660",offsetIndices:"01"},{id:"Pacific/Tahiti",untils:"-tvnayw|Infinity",offsets:"598.2667|600",offsetIndices:"01"},{id:"Pacific/Tarawa",untils:"Infinity",offsets:"-720",offsetIndices:"0"},{id:"Pacific/Tongatapu",untils:"-f4vrlc|uo2edc|8fpc0|bvs00|4bh80|eelg0|4bh80|7pmis0|3lmo0|Infinity",offsets:"-740|-780|-840",offsetIndices:"0121212121"},{id:"Pacific/Truk",untils:"-su4zs0|29hes0|bkenw0|29fk40|Infinity",offsets:"-600|-540",offsetIndices:"01010"},{id:"Pacific/Wake",untils:"Infinity",offsets:"-720",offsetIndices:"0"},{id:"Pacific/Wallis",untils:"Infinity",offsets:"-720",offsetIndices:"0"},{id:"Pacific/Yap",untils:"-su4zs0|29hes0|bkenw0|29fk40|Infinity",offsets:"-600|-540",offsetIndices:"01010"},{id:"Poland",untils:"-se9yk0|dvyc0|7ves0|a4yw0|7x6o0|asw00|7x6o0|aunw0|7x6o0|1evbs0|9fcwc0|18cao0|7k800|9q000|9d1c0|9gnw0|an980|9kd80|8fs40|922w0|ar1c0|7x6o0|a2yo0|8n400|9q000|902o0|4013w0|64dc0|9d1c0|9d1c0|clpc0|6hc00|9d1c0|9d1c0|c8qo0|6hc00|c8qo0|6hc00|c8qo0|6hc00|clpc0|64dc0|6j4tc0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"-84|-60|-120|-180",offsetIndices:"012121223212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"Portugal",untils:"-u9rhc0|2bufw0|6zxg0|66580|bq800|73k00|bodc0|71pc0|bq800|73k00|bq800|71pc0|bq800|1b2g00|9b6o0|saio0|8n400|9q000|902o0|a2yo0|902o0|a2yo0|8n400|st1c0|8n400|9d1c0|9d1c0|sg2o0|9d1c0|902o0|9q000|a2yo0|8n400|9d1c0|9d1c0|902o0|9q000|a2yo0|b5uo0|51hc0|bitc0|9d1c0|9ew00|88ao0|25p80|5reo0|3lpg0|779c0|1sqk0|6uao0|38qs0|6uao0|25p80|6hc00|38qs0|6uao0|25p80|6hc00|38qs0|8a5c0|9d1c0|9d9o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|s3400|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|5gyl40|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d440|9cyk0|9d440|9d1c0|9d1c0|9d1c0|9d1c0|9d440|9cyk0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"36.75|0|-60|-120",offsetIndices:"012121212121212121212121212121212121212121212321232123212321212121212121212121212121212121212121212121212121212121212121212121212122323232212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"PRC",untils:"-qh00w0|8sl80|asbpg0|6w2k0|7ves0|bxjw0|4mqs0|1vduk0|d4as0|75bw0|a31g0|aaak0|9d440|7v980|awo40|1dx80|j9xpo0|6u7w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|Infinity",
offsets:"-480|-540",offsetIndices:"01010101010101010101010101010"},{id:"PST8PDT",untils:"-r0emw0|ast80|7x9g0|ast80|bmtus0|1tz2s0|2dyg0|b9gdg0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"ROC",untils:"-gtzfk0|45slc0|c51c0|75bw0|a31g0|aaak0|9d440|7v980|awo40|7v980|awo40|7v980|awo40|7v980|7tk40|clmk0|7rpg0|b07w0|7rpg0|b07w0|7rpg0|9et80|9eys0|9et80|9d440|9et80|9d440|9et80|9d440|9et80|cjxg0|69uk0|ci2s0|69uk0|6its40|9et80|9d440|9et80|1yf9g0|4qak0|Infinity",offsets:"-480|-540",offsetIndices:"01010101010101010101010101010101010101010"},{id:"ROK",untils:"-w8966g|1yh18g|hkx5a0|1faao0|5cik0|ae5g0|8a2k0|ae5g0|8bx80|c8tg0|6h980|1bj6s0|l3aq0|6j3w0|d2g40|6u7w0|b5xg0|776k0|biw40|776k0|biw40|776k0|biw40|776k0|grs40|dfqxi0|7x6o0|asw00|7x6o0|Infinity",offsets:"-507.8667|-510|-540|-600|-570",offsetIndices:"012232323232141414141414123232"},{id:"Singapore",untils:"-xphpwd|eeb94d|4it32o|8n3jc|1v2p60|iy3o60|Infinity",offsets:"-415.4167|-420|-440|-450|-540|-480",offsetIndices:"0123435"},{id:"Turkey",untils:"-ux9xew|2wvx6w|7v980|1tjc40|aunw0|88dg0|9et80|8yas0|a2vw0|tzpg0|79180|awo40|7v980|7p4040|4zjw0|2vs40|f4d80|9vms0|1u5ek0|c5440|69uk0|acas0|8n180|a31g0|8n180|9q2s0|8zzw0|a31g0|8zzw0|a31g0|8n180|5md9g0|o9zw0|a6qs0|75bw0|4iwyw0|7x6o0|7kas0|b5rw0|75hg0|bkl80|77c40|biqk0|7x9g0|a2vw0|8n6s0|4iqc0|2nkw80|38l80|kdes0|8qtc0|8a5c0|9ew00|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|902o0|9q000|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7kdk0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7m2o0|b4000|7k800|b5uo0|7x6o0|asw00|7z1c0|ar1c0|7x6o0|bitc0|779c0|8fe80|Infinity",offsets:"-116.9333|-120|-180|-240",offsetIndices:"0121212121212121212121212121212121212121212121223212121212121212121212121212121212121212121212121212121212121212122"},{id:"UCT",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"Universal",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"US/Alaska",untils:"-ek1qo0|1tyx80|2e400|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l940|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"600|540|480",offsetIndices:"011001010101010101010101010101010111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"US/Aleutian",untils:"-ek1nw0|1tyug0|2e6s0|b7yik0|12y080|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|1l940|7rs80|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"660|600|540",offsetIndices:"011001010101010101010101010101010111212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121"},{id:"US/Arizona",untils:"-r0epo0|ast80|7x9g0|ast80|bmtus0|zjedo|4olg0|9et80|bs6lmc|9cyk0|Infinity",offsets:"420|360",offsetIndices:"01010101010"},{id:"US/Central",untils:"-r0esg0|ast80|7x9g0|ast80|bvus0|776k0|7kas0|b5rw0|9d440|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|7x9g0|dbjw0|8a840|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"01010101010101010101010101010101010101010101010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"US/East-Indiana",untils:"-r0esg0|ast80|7x9g0|ast80|baw840|51ek0|6w840|1tz8c0|2dsw0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|19q7w0|asys0|5qonw0|9cyk0|9d440|9cyk0|ihslg0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300|240",offsetIndices:"010101011010101010101010101010121212121212121212121212121212121212121212121212121212121212121212121"},{id:"US/Eastern",untils:"-r0ev80|ast80|7x9g0|ast80|7x9g0|b5rw0|905g0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|6w840|1tzb40|2dq40|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"300|240",offsetIndices:"01010101010101010101010101010101010101010101010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"US/Hawaii",untils:"-j50la0|13l00|4jvb00|1tyvu0|2e5e0|votg0|Infinity",offsets:"630|570|600",offsetIndices:"0101102"},{id:"US/Indiana-Starke",untils:"-r0esg0|ast80|7x9g0|ast80|bmtus0|1tz8c0|2dsw0|tj1g0|7x3w0|asys0|7x3w0|asys0|7x3w0|b5xg0|7k580|b5xg0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|7x3w0|asys0|9px80|9d440|9cyk0|9d440|7x3w0|asys0|7x3w0|asys0|9cyk0|9d440|9px80|9d440|9cyk0|9d440|s3180|1twas0|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|7j5400|asw00|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"360|300",offsetIndices:"0101011010101010101010101010101010101010101010101010101010101010101010101010101010101010111010101010101010101010101010101010101010101010101010101010101010"},{id:"US/Michigan",untils:"-xx8dyd|5eraud|dyeyk0|1tzb40|2dq40|1c9440|7x3w0|9rlbxo|71s2c|9d440|9cyk0|2cmdg0|9cyk0|3lpg0|f4d80|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"332.1833|360|300|240",offsetIndices:"0123323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232"},{id:"US/Mountain",untils:"-r0epo0|ast80|7x9g0|ast80|7x9g0|b5rw0|7kas0|2vmk0|ataw40|1tz5k0|2dvo0|a7n9g0|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"420|360",offsetIndices:"01010101011010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"US/Pacific-New",untils:"-r0emw0|ast80|7x9g0|ast80|bmtus0|1tz2s0|2dyg0|1a3c5o|f2iic|owao0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"US/Pacific",untils:"-r0emw0|ast80|7x9g0|ast80|bmtus0|1tz2s0|2dyg0|1a3c5o|f2iic|owao0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|902o0|9q000|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|9d440|9cyk0|9d440|9cyk0|3lpg0|f4d80|64g40|clmk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|9d440|9px80|905g0|9px80|9d440|9cyk0|9d440|9cyk0|9d440|9cyk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|8a840|afuk0|8a840|afuk0|8a840|ast80|7x9g0|ast80|7x9g0|ast80|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6udg0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|6hes0|c8nw0|Infinity",offsets:"480|420",offsetIndices:"010101101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"US/Samoa",untils:"-usij20|Infinity",offsets:"682.8|660",offsetIndices:"01"},{id:"UTC",untils:"Infinity",offsets:"0",offsetIndices:"0"},{id:"W-SU",untils:"-rx5dmh|ipzua|97hc0|7yyk0|5i840|d9p80|1jwk7|2cvk0|s8o00|1qvw0|8fpc0|1jms0|is040|412as0|qi27w0|9et80|9d440|9et80|9d440|9et80|9eys0|9d6w0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d440|5reo0|3ljw0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|1vbzw0|Infinity",offsets:"-150.2833|-151.3167|-211.3167|-271.3167|-240|-180|-300|-120",offsetIndices:"012132345464575454545454545454545455754545454545454545454545454545454545454545"},{id:"WET",untils:"3s9ms0|902o0|9q000|9d1c0|9d1c0|9d1c0|9q000|902o0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9d1c0|9q000|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|7x6o0|b5uo0|7k800|b5uo0|7k800|b5uo0|7k800|b5uo0|7x6o0|asw00|7x6o0|asw00|Infinity",offsets:"0|-60",offsetIndices:"010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010"},{id:"Zulu",untils:"Infinity",offsets:"0",offsetIndices:"0"}]};t.default=i,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(20)),l=s.default.dateToMilliseconds,u=function(){function e(t){o(this,e),this._source=new Date(t.getTime?t.getTime():t)}return r(e,[{key:"result",value:function(){return this._source}},{key:"getTimezoneOffset",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:void 0,t=this._source.getTimezoneOffset();return"minute"===e?t*l("minute"):t}},{key:"getTime",value:function(){return this._source.getTime()}},{key:"setTime",value:function(e){return this._source.setTime(e),this}},{key:"addTime",value:function(e){return this._source.setTime(this._source.getTime()+e),this}},{key:"setMinutes",value:function(e){return this._source.setMinutes(e),this}},{key:"addMinutes",value:function(e){return this._source.setMinutes(this._source.getMinutes()+e),this}},{key:"subtractMinutes",value:function(e){return this._source.setMinutes(this._source.getMinutes()-e),this}},{key:"source",get:function(){return this._source}}]),e}(),c=function(e){return new u(e)},d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(){return o.hideCallback.fire()}t.default=i;var o=n(209);e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.Event=t.triggerHandler=t.trigger=t.off=t.one=t.on=void 0;var o=i(n(5)),a=o.default.on;t.on=a;var r=o.default.one;t.one=r;var s=o.default.off;t.off=s;var l=o.default.trigger;t.trigger=l;var u=o.default.triggerHandler;t.triggerHandler=u;var c=o.default.Event;t.Event=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e&&[e.target,e.delegateTarget,e.relatedTarget,e.currentTarget].filter(function(e){return!!e})}t.unsubscribeNodesDisposing=t.subscribeNodesDisposing=void 0;var a=i(n(5)),r="dxremove",s=function(e,t){a.default.one(o(e),r,t)};t.subscribeNodesDisposing=s;var l=function(e,t){a.default.off(o(e),r,t)};t.unsubscribeNodesDisposing=l},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=i(n(157)),r=i(n(255)),s=n(0),l={dxpointerdown:"pointerdown",dxpointermove:"pointermove",dxpointerup:"pointerup",dxpointercancel:"pointercancel",dxpointerover:"pointerover",dxpointerout:"pointerout",dxpointerenter:"pointerenter",dxpointerleave:"pointerleave"},u=!1,c=function(){u||(o=new r.default(l,function(e,t){return e.pointerId===t.pointerId},function(e){e.isPrimary&&o.reset()}),u=!0)},d=a.default.inherit({ctor:function(){this.callBase.apply(this,arguments),c()},_fireEvent:function(e){return this.callBase((0,s.extend)({pointers:o.pointers(),pointerId:e.originalEvent.pointerId},e))}});d.map=l,d.resetObserver=function(){o.reset()};var h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(157)),r=i(n(256)),s=i(n(254)),l=n(8),u={dxpointerdown:"touchstart mousedown",dxpointermove:"touchmove mousemove",dxpointerup:"touchend mouseup",dxpointercancel:"touchcancel",dxpointerover:"mouseover",dxpointerout:"mouseout",dxpointerenter:"mouseenter",dxpointerleave:"mouseleave"},c=!1,d=function(){c||(r.default.activate(),c=!0)},h=a.default.inherit({EVENT_LOCK_TIMEOUT:100,ctor:function(){this.callBase.apply(this,arguments),d()},_handler:function(e){var t=(0,l.isMouseEvent)(e);if(t||(this._skipNextEvents=!0),!t||!this._mouseLocked){if(t&&this._skipNextEvents){this._skipNextEvents=!1,this._mouseLocked=!0,clearTimeout(this._unlockMouseTimer);var n=this;return void(this._unlockMouseTimer=setTimeout(function(){n._mouseLocked=!1},this.EVENT_LOCK_TIMEOUT))}return this.callBase(e)}},_fireEvent:function(e){var t=(0,l.isMouseEvent)(e.originalEvent)?r.default.normalize:s.default.normalize;return this.callBase((0,o.extend)(t(e.originalEvent),e))},dispose:function(){this.callBase(),this._skipNextEvents=!1,this._mouseLocked=!1,clearTimeout(this._unlockMouseTimer)}});h.map=u,h.resetObserver=r.default.resetObserver;var f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){"intl"===s.default.engine()&&s.default.resetInjection(),"intl"===l.default.engine()&&l.default.resetInjection()}t.disableIntl=o,Object.defineProperty(t,"message",{enumerable:!0,get:function(){return r.default}}),Object.defineProperty(t,"number",{enumerable:!0,get:function(){return s.default}}),Object.defineProperty(t,"date",{enumerable:!0,get:function(){return l.default}}),t.parseDate=t.formatDate=t.parseNumber=t.formatNumber=t.formatMessage=t.loadMessages=t.locale=void 0;var a=i(n(82)),r=i(n(10)),s=i(n(45)),l=i(n(32));n(107);var u=a.default.locale.bind(a.default);t.locale=u;var c=r.default.load.bind(r.default);t.loadMessages=c;var d=r.default.format.bind(r.default);t.formatMessage=d;var h=s.default.format.bind(s.default);t.formatNumber=h;var f=s.default.parse.bind(s.default);t.parseNumber=f;var p=l.default.format.bind(l.default);t.formatDate=p;var g=l.default.parse.bind(l.default);t.parseDate=g},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=i(n(236)),r=i(n(85)),s=(0,r.default)();s&&a.default.set(o.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=i(n(85)),r=n(60),s=n(3),l=n(1),u=n(8),c=i(n(66)),d=i(n(237)),h=(0,a.default)();if(h)if((0,r.compare)(o.default.fn.jquery,[3])<0){var f={2:"touch",3:"pen",4:"mouse"};(0,s.each)(["MSPointerDown","MSPointerMove","MSPointerUp","MSPointerCancel","MSPointerOver","MSPointerOut","mouseenter","mouseleave","pointerdown","pointermove","pointerup","pointercancel","pointerover","pointerout","pointerenter","pointerleave"],function(){o.default.event.fixHooks[this]={filter:function(e,t){var n=t.pointerType;return(0,l.isNumeric)(n)&&(e.pointerType=f[n]),e},props:o.default.event.mouseHooks.props.concat(["pointerId","pointerType","originalTarget","width","height","pressure","result","tiltX","charCode","tiltY","detail","isPrimary","prevValue"])}}),(0,s.each)(["touchstart","touchmove","touchend","touchcancel"],function(){o.default.event.fixHooks[this]={filter:function(e,t){return(0,d.default)(function(n,i){e[n]=i(t)}),e},props:o.default.event.mouseHooks.props.concat(["touches","changedTouches","targetTouches","detail","result","originalTarget","charCode","prevValue"])}}),o.default.event.fixHooks.wheel=o.default.event.mouseHooks;var p={props:o.default.event.mouseHooks.props.concat(["pointerType","pointerId","pointers"])};c.default.callbacks.add(function(e){o.default.event.fixHooks[e]=p});var g=function(e,t){for(var n=o.default.event.fixHooks[t.type]||o.default.event.mouseHooks,i=n.props?o.default.event.props.concat(n.props):o.default.event.props,a=i.length;a--;){var r=i[a];e[r]=t[r]}return n.filter?n.filter(e,t):e};(0,u.setEventFixMethod)(g)}else(0,d.default)(function(e,t){o.default.event.addProp(e,t)})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=n(6),r=n(60),s=i(n(85)),l=(0,s.default)();if(l){var u=o.default.Deferred,c={Deferred:u};c.when=(0,r.compare)(o.default.fn.jquery,[3])<0?o.default.when:function(e){return 0===arguments.length?(new u).resolve():1===arguments.length?e&&e.then?e:(new u).resolve(e):o.default.when.apply(o.default,arguments)},(0,a.setStrategy)(c)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=n(258),r=i(n(49));if(o.default&&!a.themeReadyCallback.fired()){var s=o.default.holdReady||o.default.fn.holdReady;s(!0),a.themeReadyCallback.add(function(){r.default.add(function(){s(!1)})})}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=i(n(5)),r=i(n(85)),s=i(n(116)),l=i(n(11)),u=(0,r.default)();u&&(s.default.add(function(e,t){o.default.event.special[e]=t}),a.default.passiveEventHandlersSupported()&&a.default.forcePassiveFalseEventNames.forEach(function(e){o.default.event.special[e]={setup:function(t,n,i){l.default.listen(this,e,i,{passive:!1})}}}),a.default.set({on:function(e){(0,o.default)(e).on.apply((0,o.default)(e),Array.prototype.slice.call(arguments,1))},one:function(e){(0,o.default)(e).one.apply((0,o.default)(e),Array.prototype.slice.call(arguments,1))},off:function(e){(0,o.default)(e).off.apply((0,o.default)(e),Array.prototype.slice.call(arguments,1))},trigger:function(e){(0,o.default)(e).trigger.apply((0,o.default)(e),Array.prototype.slice.call(arguments,1))},triggerHandler:function(e){(0,o.default)(e).triggerHandler.apply((0,o.default)(e),Array.prototype.slice.call(arguments,1))},Event:o.default.Event}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=n(253);o.default&&(0,a.setEasing)(o.default.easing)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=n(39),r=i(n(85)),s=(0,r.default)();s&&(0,a.setDataStrategy)(o.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=n(18),a=i(n(85)),r=(0,a.default)(),s=function(e){return e};r&&(0,o.setPublicElementWrapper)(s)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=i(n(155)),r=i(n(22));if(o.default){var s=function(e,t){o.default.fn[e]=function(n){var i,o="string"==typeof n;if(o){var a=n,s=[].slice.call(arguments).slice(1);this.each(function(){var n=t.getInstance(this);if(!n)throw r.default.Error("E0009",e);var o=n[a],l=o.apply(n,s);void 0===i&&(i=l)})}else this.each(function(){var e=t.getInstance(this);e?e.option(n):new t(this,n)}),i=this;return i}};a.default.add(s)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(56)),a=i(n(76)),r=i(n(85)),s=(0,r.default)();s&&a.default.inject({sendRequest:function(e){return e.responseType||e.upload?this.callBase.apply(this,[e]):o.default.ajax(e)}})},function(e,t,n){n(257),n(387),n(395),n(396),n(397)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var a=i(n(2)),r=i(n(122)),s=i(n(5)),l=i(n(28)),u=i(n(155)),c=i(n(15)),d=i(n(27)),h=n(1),f=n(3),p=n(12),g=i(n(259)),_=i(n(52)),m=n(389),v=i(n(189)),y=i(n(58)),b=n(19),x=n(0),w=n(249),C=n(40),k="dxItemAlias",S=["rendering"],D="value";if(r.default){var I=function(e,t){return t.$root.$$phase?e(t):t.$apply(function(){return e(t)})},T=function e(t,n){var i=Object.prototype.hasOwnProperty.bind(t)("parent"),o=!i&&t.parent;if(o){var a=Object.getPrototypeOf(t);return a.prototype[n]?function(){return a.prototype[n]()}:e(a,n)}var r=t.parent.prototype[n];return r?function(){return r()}:r&&t.parent.subclassOf?e(t.parent,n):function(){}},O=c.default.inherit({ctor:function(e){this._componentDisposing=(0,d.default)(),this._optionChangedCallbacks=(0,d.default)(),this._ngLocker=new g.default,this._scope=e.scope,this._$element=e.$element,this._$templates=e.$templates,this._componentClass=e.componentClass,this._parse=e.parse,this._compile=e.compile,this._itemAlias=e.itemAlias,this._transcludeFn=e.transcludeFn,this._digestCallbacks=e.dxDigestCallbacks,this._normalizeOptions(e.ngOptions),this._initComponentBindings(),this._initComponent(this._scope),e.ngOptions||this._addOptionsStringWatcher(e.ngOptionsString)},_addOptionsStringWatcher:function(e){var t=this,n=this._scope.$watch(e,function(e){e&&(n(),t._normalizeOptions(e),t._initComponentBindings(),t._component.option(t._evalOptions(t._scope)))});this._componentDisposing.add(n)},_normalizeOptions:function(e){var t=this;this._ngOptions=(0,x.extendFromObject)({},e),e&&(!Object.prototype.hasOwnProperty.call(e,"bindingOptions")&&e.bindingOptions&&(this._ngOptions.bindingOptions=e.bindingOptions),e.bindingOptions&&(0,f.each)(e.bindingOptions,function(e,n){"string"===(0,h.type)(n)&&(t._ngOptions.bindingOptions[e]={dataPath:n})}))},_initComponent:function(e){this._component=new this._componentClass(this._$element,this._evalOptions(e)),this._component._isHidden=!0,this._handleDigestPhase()},_handleDigestPhase:function(){var e=this,t=function(){e._component.beginUpdate()},n=function(){e._component.endUpdate()};this._digestCallbacks.begin.add(t),this._digestCallbacks.end.add(n),this._componentDisposing.add(function(){e._digestCallbacks.begin.remove(t),e._digestCallbacks.end.remove(n)})},_initComponentBindings:function(){var e=this,t={};this._ngOptions.bindingOptions&&((0,f.each)(this._ngOptions.bindingOptions,function(n,i){var o,a,r=n.search(/\[|\./),s=r>-1?n.substring(0,r):n,l=i.dataPath,u=!0,c=!1;void 0!==i.deep&&(c=u=!!i.deep),t[s]||(t[s]={}),t[s][n]=l;var d=function t(){var i=function(i,o){e._ngLocker.locked(n)||(e._ngLocker.obtain(n),e._component.option(n,i),t(),(0,w.equals)(o,i)&&e._ngLocker.locked(n)&&e._ngLocker.release(n))},r=Array.isArray(e._scope.$eval(l))&&!c?"$watchCollection":"$watch";o!==r&&(a&&a(),a=e._scope[r](l,i,u),o=r)};d(),e._componentDisposing.add(a)}),this._optionChangedCallbacks.add(function(n){var i=n.name,o=n.fullName,a=n.component;if(e._ngLocker.locked(o))return void e._ngLocker.release(o);if(t&&t[i]){var r=e._scope.$root.$$phase,s=function(){e._ngLocker.obtain(o)};r?e._digestCallbacks.begin.add(s):s(),I(function(){(0,f.each)(t[i],function(t,i){if(e._optionsAreLinked(o,t)){var r=a.option(t);e._parse(i).assign(e._scope,r);var s=e._parse(i)(e._scope);s!==r&&n.component.option(t,s)}})},e._scope);var l=function t(){e._ngLocker.locked(o)&&e._ngLocker.release(o),e._digestCallbacks.begin.remove(s),e._digestCallbacks.end.remove(t)};r?e._digestCallbacks.end.addPrioritized(l):l()}}))},_optionsAreNested:function(e,t){var n=e[t.length];return 0===e.indexOf(t)&&("."===n||"["===n)},_optionsAreLinked:function(e,t){return e===t||(e.length>t.length?this._optionsAreNested(e,t):this._optionsAreNested(t,e))},_compilerByTemplate:function(e){var t=this,n=this._getScopeItemsPath();return function(i){var o=(0,a.default)(e).clone(),r=i.model&&i.model.constructor===t._scope.$root.constructor,l=r?i.model:i.noModel?t._scope:t._createScopeWithData(i);n&&t._synchronizeScopes(l,n,i.index),
o.appendTo(i.container),i.noModel||s.default.on(o,"$destroy",function(){var e=!l.$parent;e||l.$destroy()});var u=t._compile(o,t._transcludeFn);return t._applyAsync(function(e){u(e,null,{parentBoundTranscludeFn:t._transcludeFn})},l),o}},_applyAsync:function(e,t){var n=this;if(e(t),!t.$root.$$phase&&!this._renderingTimer){var i=function(){clearTimeout(n._renderingTimer)};this._renderingTimer=setTimeout(function(){t.$apply(),n._renderingTimer=null,n._componentDisposing.remove(i)}),this._componentDisposing.add(i)}},_getScopeItemsPath:function(){if(this._componentClass.subclassOf(y.default)&&this._ngOptions.bindingOptions&&this._ngOptions.bindingOptions.items)return this._ngOptions.bindingOptions.items.dataPath},_createScopeWithData:function(e){var t=this._scope.$new();return this._itemAlias&&(t[this._itemAlias]=e.model),(0,h.isDefined)(e.index)&&(t.$index=e.index),t},_synchronizeScopes:function(e,t,n){this._itemAlias&&"object"!==o(e[this._itemAlias])&&this._synchronizeScopeField({parentScope:this._scope,childScope:e,fieldPath:this._itemAlias,parentPrefix:t,itemIndex:n})},_synchronizeScopeField:function(e){var t=e.parentScope,n=e.childScope,i=e.fieldPath,o=e.parentPrefix,a=e.itemIndex,r=i===this._itemAlias?"":"."+i,s=void 0!==a,l=[o];if(s){if(!(0,h.isNumeric)(a))return;l.push("[",a,"]")}l.push(r);var u=l.join(""),c=t.$watch(u,function(e,t){e!==t&&(0,b.compileSetter)(i)(n,e)}),d=n.$watch(i,function(e,n){if(e!==n){if(s&&!(0,b.compileGetter)(o)(t)[a])return void d();(0,b.compileSetter)(u)(t,e)}});this._componentDisposing.add([c,d])},_evalOptions:function(e){var t=this,n=(0,x.extendFromObject)({},this._ngOptions);return delete n.bindingOptions,this._ngOptions.bindingOptions&&(0,f.each)(this._ngOptions.bindingOptions,function(t,i){n[t]=e.$eval(i.dataPath)}),n._optionChangedCallbacks=this._optionChangedCallbacks,n._disposingCallbacks=this._componentDisposing,n.onActionCreated=function(t,n,i){if(i&&(0,p.inArray)(i.category,S)>-1)return n;var o=function(){var t=this,i=arguments;return e&&e.$root&&!e.$root.$$phase?I(function(){return n.apply(t,i)},e):n.apply(this,i)};return o},n.beforeActionExecute=n.onActionCreated,n.nestedComponentOptions=function(e){return{templatesRenderAsynchronously:e.option("templatesRenderAsynchronously"),forceApplyBindings:e.option("forceApplyBindings"),modelByElement:e.option("modelByElement"),onActionCreated:e.option("onActionCreated"),beforeActionExecute:e.option("beforeActionExecute"),nestedComponentOptions:e.option("nestedComponentOptions")}},n.templatesRenderAsynchronously=!0,(0,l.default)().wrapActionsBeforeExecute&&(n.forceApplyBindings=function(){I(function(){},e)}),n.integrationOptions={createTemplate:function(e){return new m.NgTemplate(e,t._compilerByTemplate.bind(t))},watchMethod:function(n,i,o){o=o||{};var a,r=o.skipImmediate,s=e.$watch(function(){var e=n();return e instanceof Date&&(e=e.valueOf()),e},function(e){var t=a===e;!r&&(!t||t&&o.deep)&&i(e),r=!1},o.deep);return r||(a=n(),i(a)),(0,l.default)().wrapActionsBeforeExecute&&t._applyAsync(function(){},e),s},templates:{"dx-polymorph-widget":{render:function(e){var n=e.model.widget;if(n){var i=(0,a.default)("<div>").attr((0,C.dasherize)(n),"options").get(0),o=t._scope.$new();o.options=e.model.options,e.container.append(i),t._compile(i)(o)}}}}},n.modelByElement=function(){return e},n}});O=O.inherit({ctor:function(e){this._componentName=e.componentName,this._ngModel=e.ngModel,this._ngModelController=e.ngModelController,this.callBase.apply(this,arguments)},_isNgModelRequired:function(){return(this._componentClass.subclassOf(_.default)||this._componentClass.prototype instanceof _.default)&&this._ngModel},_initComponentBindings:function(){this.callBase.apply(this,arguments),this._initNgModelBinding()},_initNgModelBinding:function(){var e=this;if(this._isNgModelRequired()){var t=this._scope.$watch(this._ngModel,function(t,n){e._ngLocker.locked(D)||t!==n&&e._component.option(D,t)});this._optionChangedCallbacks.add(function(t){e._ngLocker.obtain(D);try{if(t.name!==D)return;e._ngModelController.$setViewValue(t.value)}finally{e._ngLocker.locked(D)&&e._ngLocker.release(D)}}),this._componentDisposing.add(t)}},_evalOptions:function(){if(!this._isNgModelRequired())return this.callBase.apply(this,arguments);var e=this.callBase.apply(this,arguments);return e[D]=this._parse(this._ngModel)(this._scope),e}});var E={},A=function(e){var t="dxValidator"!==e?1:10;v.default.directive(e,["$compile","$parse","dxDigestCallbacks",function(n,i,o){return{restrict:"A",require:"^?ngModel",priority:t,compile:function(t){var a=E[e],r=a.prototype._useTemplates?a.prototype._useTemplates():T(a,"_useTemplates")(),s=r?t.contents().detach():null;return function(t,r,l,u,c){r.append(s),I(function(){new O({componentClass:a,componentName:e,compile:n,parse:i,$element:r,scope:t,ngOptionsString:l[e],ngOptions:l[e]?t.$eval(l[e]):{},ngModel:l.ngModel,ngModelController:u,transcludeFn:c,itemAlias:l[k],dxDigestCallbacks:o})},t)}}}}])};u.default.add(function(e,t){E[e]||A(e),E[e]=t})}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(9)),r=i(n(68)),s=n(0),l=n(42),u=n(24),c="dx-invalid-message",d="dx-invalid-message-auto",h="dx-invalid-message-always",f="dx-invalid-message-content",p=100,g=r.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{integrationOptions:{},templatesRenderAsynchronously:!1,shading:!1,width:"auto",height:"auto",closeOnOutsideClick:!1,closeOnTargetScroll:!1,animation:null,visible:!0,propagateOutsideClick:!0,_checkParentVisibility:!1,rtlEnabled:!1,contentTemplate:this._renderInnerHtml,maxWidth:"100%",mode:"auto",validationErrors:void 0,positionRequest:void 0,boundary:void 0,offset:{h:0,v:0}})},_init:function(){this.callBase(),this.updateMaxWidth(),this._updatePosition()},_initMarkup:function(){this.callBase(),this.$element().addClass(c),this._wrapper().addClass(c),this._toggleModeClass(),this._updateContentId()},_updateContentId:function(){var e=(0,o.default)(this.option("container")).attr("aria-describedby");this.$content().addClass(f).attr("id",e)},_renderInnerHtml:function(e){var t=e&&(0,o.default)(e),n=this.option("validationErrors")||[],i="";n.forEach(function(e){var t=i?"<br />":"";i+=t+(0,l.encodeHtml)((null===e||void 0===e?void 0:e.message)||"")}),null===t||void 0===t?void 0:t.html(i)},_toggleModeClass:function(){var e=this.option("mode");this._wrapper().toggleClass(d,"auto"===e).toggleClass(h,"always"===e)},updateMaxWidth:function(){var e,t=this.option("target"),n=(null===t||void 0===t?void 0:null===(e=t.outerWidth)||void 0===e?void 0:e.call(t))||(0,o.default)(t).outerWidth(),i="100%";n&&(i=Math.max(n,p)),this.option({maxWidth:i})},_updatePosition:function(){var e=this.option(),t=e.positionRequest,n=e.rtlEnabled,i=e.offset,o=e.boundary,a=(0,u.getDefaultAlignment)(n),r="below"===t?[" top"," bottom"]:[" bottom"," top"];n&&(i.h=-i.h),"below"!==t&&(i.v=-i.v),this.option("position",{offset:i,boundary:o,my:a+r[0],at:a+r[1],collision:"none flip"})},_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"target":this.updateMaxWidth(),this.callBase(e);break;case"boundary":this.option("position.boundary",n);break;case"mode":this._toggleModeClass(n);break;case"rtlEnabled":case"offset":case"positionRequest":this._updatePosition();break;case"validationErrors":this._renderInnerHtml(this.$content());break;default:this.callBase(e)}}});(0,a.default)("dxValidationMessage",g);var _=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.NgTemplate=void 0;var g=o(n(2)),_=n(67),m=n(1),v=n(33),y=function(e){function t(e,i){var o;return a(this,t),o=n.call(this),o._element=e,o._compiledTemplate=i((0,v.normalizeTemplateElement)(o._element)),o}l(t,e);var n=c(t);return s(t,[{key:"_renderCore",value:function(e){var t=this._compiledTemplate;return(0,m.isFunction)(t)?t(e):t}},{key:"source",value:function(){return(0,g.default)(this._element).clone()}}]),t}(_.TemplateBase);t.NgTemplate=y},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=n(250),l=n(18),u=i(n(11)),c=n(1),d=n(6),h=n(0),f=n(12),p=n(3),g=i(n(108)),_=i(n(35)),m=i(n(16)),v=n(8),y=i(n(25)),b=i(n(94)),x=i(n(141)),w=n(75),C=i(n(10)),k=i(n(103)),S=n(19),D=n(21),I=n(178),T=n(71),O="dx-collection",E="dx-item",A="-content",P="dx-item-content-placeholder",R="dxItemData",M="dxItemIndex",B="tmpl-",V="dxItem",F="dx-item-selected",L="dx-item-response-wait",H="dx-empty-collection",z="dx-template-wrapper",N=/^([^.]+\[\d+\]\.)+([\w.]+)$/,j="item",W="up",G="down",q="left",$="right",K="pageup",U="pagedown",Y="last",Z="first",X=m.default.inherit({_activeStateUnit:"."+E,_supportedKeys:function(){var e=function(e){var t=(0,o.default)(this.option("focusedElement"));t.length&&this._itemClickHandler((0,h.extend)({},e,{target:t,currentTarget:t}))},t=function(t){t.preventDefault(),e.call(this,t)},n=function(e,t){(0,v.isCommandKeyPressed)(t)||(t.preventDefault(),t.stopPropagation(),this._moveFocus(e,t))};return(0,h.extend)(this.callBase(),{space:t,enter:e,leftArrow:n.bind(this,q),rightArrow:n.bind(this,$),upArrow:n.bind(this,W),downArrow:n.bind(this,G),pageUp:n.bind(this,W),pageDown:n.bind(this,G),home:n.bind(this,Z),end:n.bind(this,Y)})},_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{selectOnFocus:!1,loopItemFocus:!0,items:[],itemTemplate:"item",onItemRendered:null,onItemClick:null,onItemHold:null,itemHoldTimeout:750,onItemContextMenu:null,onFocusedItemChanged:null,noDataText:C.default.format("dxCollectionWidget-noDataText"),dataSource:null,_itemAttributes:{},itemTemplateProperty:"template",focusOnSelectedItem:!0,focusedElement:null,displayExpr:void 0,disabledExpr:function(e){return e?e.disabled:void 0},visibleExpr:function(e){return e?e.visible:void 0}})},_init:function(){this._compileDisplayGetter(),this.callBase(),this._cleanRenderedItems(),this._refreshDataSource()},_compileDisplayGetter:function(){var e=this.option("displayExpr");this._displayGetter=e?(0,S.compileGetter)(this.option("displayExpr")):void 0},_initTemplates:function(){this._initItemsFromMarkup(),this._initDefaultItemTemplate(),this.callBase()},_getAnonymousTemplateName:function(){return j},_initDefaultItemTemplate:function(){var e=this._getFieldsMap();this._templateManager.addDefaultTemplates({item:new T.BindableTemplate(function(t,n){(0,c.isPlainObject)(n)?this._prepareDefaultItemTemplate(n,t):(e&&(0,c.isFunction)(e.text)&&(n=e.text(n)),t.text(String((0,r.ensureDefined)(n,""))))}.bind(this),this._getBindableFields(),this.option("integrationOptions.watchMethod"),e)})},_getBindableFields:function(){return["text","html"]},_getFieldsMap:function(){if(this._displayGetter)return{text:this._displayGetter}},_prepareDefaultItemTemplate:function(e,t){(0,c.isDefined)(e.text)&&t.text(e.text),(0,c.isDefined)(e.html)&&t.html(e.html)},_initItemsFromMarkup:function(){var e=this,t=(0,s.findTemplates)(this.$element(),V);if(t.length&&!this.option("items").length){var n=t.map(function(t){var n=t.element,i=t.options,a=/\S/.test(n.innerHTML)&&!i.template;return a?i.template=e._prepareItemTemplate(n):(0,o.default)(n).remove(),i});this.option("items",n)}},_prepareItemTemplate:function(e){var t=B+new _.default,n=(0,o.default)(e).detach().clone().removeAttr("data-options").addClass(z);return this._saveTemplate(t,n),t},_dataSourceOptions:function(){return{paginate:!1}},_cleanRenderedItems:function(){this._renderedItemsCount=0},_focusTarget:function(){return this.$element()},_focusInHandler:function(e){if(this.callBase.apply(this,arguments),(0,f.inArray)(e.target,this._focusTarget())!==-1){var t=(0,o.default)(this.option("focusedElement"));if(t.length)this._setFocusedItem(t);else{var n=this._getActiveItem();n.length&&this.option("focusedElement",(0,l.getPublicElement)(n))}}},_focusOutHandler:function(){this.callBase.apply(this,arguments);var e=(0,o.default)(this.option("focusedElement"));this._updateFocusedItemState(e,!1)},_getActiveItem:function(e){var t=(0,o.default)(this.option("focusedElement"));if(t.length)return t;var n=this.option("focusOnSelectedItem")?this.option("selectedIndex"):0,i=this._getActiveElement(),a=i.length-1;return n<0&&(n=e?a:0),i.eq(n)},_moveFocus:function(e){var t,n=this._getAvailableItems();switch(e){case K:case W:t=this._prevItem(n);break;case U:case G:t=this._nextItem(n);break;case $:t=this.option("rtlEnabled")?this._prevItem(n):this._nextItem(n);break;case q:t=this.option("rtlEnabled")?this._nextItem(n):this._prevItem(n);break;case Z:t=n.first();break;case Y:t=n.last();break;default:return!1}0!==t.length&&this.option("focusedElement",(0,l.getPublicElement)(t))},_getVisibleItems:function(e){return e=e||this._itemElements(),e.filter(":visible")},_getAvailableItems:function(e){return this._getVisibleItems(e).not(".dx-state-disabled")},_prevItem:function(e){var t=this._getActiveItem(),n=e.index(t),i=e.last(),a=(0,o.default)(e[n-1]),r=this.option("loopItemFocus");return 0===a.length&&r&&(a=i),a},_nextItem:function(e){var t=this._getActiveItem(!0),n=e.index(t),i=e.first(),a=(0,o.default)(e[n+1]),r=this.option("loopItemFocus");return 0===a.length&&r&&(a=i),a},_selectFocusedItem:function(e){this.selectItem(e)},_updateFocusedItemState:function(e,t,n){var i=(0,o.default)(e);i.length&&(this._refreshActiveDescendant(),this._refreshItemId(i,n),this._toggleFocusClass(t,i)),this._updateParentActiveDescendant()},_refreshActiveDescendant:function(e){this.setAria("activedescendant",(0,c.isDefined)(this.option("focusedElement"))?this.getFocusedItemId():null,e)},_refreshItemId:function(e,t){!t&&this.option("focusedElement")?this.setAria("id",this.getFocusedItemId(),e):this.setAria("id",null,e)},_setFocusedItem:function(e){e&&e.length&&(this._updateFocusedItemState(e,!0),this.onFocusedItemChanged(this.getFocusedItemId()),this.option("selectOnFocus")&&this._selectFocusedItem(e))},_findItemElementByItem:function(e){var t=(0,o.default)(),n=this;return this.itemElements().each(function(){var i=(0,o.default)(this);if(i.data(n._itemDataKey())===e)return t=i,!1}),t},_getIndexByItem:function(e){return this.option("items").indexOf(e)},_itemOptionChanged:function(e,t,n,i){var o=this._findItemElementByItem(e);if(o.length){this.constructor.ItemClass.getInstance(o).setDataField(t,n)||this._refreshItem(o,e);var a="disabled"===t&&n;a&&this._resetItemFocus(o)}},_resetItemFocus:function(e){e.is(this.option("focusedElement"))&&this.option("focusedElement",null)},_refreshItem:function(e){var t=this._getItemData(e),n=e.data(this._itemIndexKey());this._renderItem(this._renderedItemsCount+n,t,null,e)},_updateParentActiveDescendant:r.noop,_optionChanged:function(e){if("items"===e.name){var t=e.fullName.match(N);if(t&&t.length){var n=t[t.length-1],i=e.fullName.replace("."+n,""),a=this.option(i);return void this._itemOptionChanged(a,n,e.value,e.previousValue)}}switch(e.name){case"items":case"_itemAttributes":case"itemTemplateProperty":case"showItemDataTitle":this._cleanRenderedItems(),this._invalidate();break;case"dataSource":this._refreshDataSource(),this._renderEmptyMessage();break;case"noDataText":this._renderEmptyMessage();break;case"itemTemplate":this._invalidate();break;case"onItemRendered":this._createItemRenderAction();break;case"onItemClick":break;case"onItemHold":case"itemHoldTimeout":this._attachHoldEvent();break;case"onItemContextMenu":this._attachContextMenuEvent();break;case"onFocusedItemChanged":this.onFocusedItemChanged=this._createActionByOption("onFocusedItemChanged");break;case"selectOnFocus":case"loopItemFocus":case"focusOnSelectedItem":break;case"focusedElement":this._updateFocusedItemState(e.previousValue,!1,!0),this._setFocusedItem((0,o.default)(e.value));break;case"displayExpr":this._compileDisplayGetter(),this._initDefaultItemTemplate(),this._invalidate();break;case"visibleExpr":case"disabledExpr":this._invalidate();break;default:this.callBase(e)}},_invalidate:function(){return this.option("focusedElement",null),this.callBase.apply(this,arguments)},_loadNextPage:function(){var e=this._dataSource;return this._expectNextPageLoading(),e.pageIndex(1+e.pageIndex()),e.load()},_expectNextPageLoading:function(){this._startIndexForAppendedItems=0},_expectLastItemLoading:function(){this._startIndexForAppendedItems=-1},_forgetNextPageLoading:function(){this._startIndexForAppendedItems=null},_dataSourceChangedHandler:function(e){var t=this.option("items");this._initialized&&t&&this._shouldAppendItems()?(this._renderedItemsCount=t.length,this._isLastPage()&&this._startIndexForAppendedItems===-1||(this.option().items=t.concat(e.slice(this._startIndexForAppendedItems))),this._forgetNextPageLoading(),this._refreshContent()):this.option("items",e.slice())},_refreshContent:function(){this._prepareContent(),this._renderContent()},_dataSourceLoadErrorHandler:function(){this._forgetNextPageLoading(),this.option("items",this.option("items"))},_shouldAppendItems:function(){return null!=this._startIndexForAppendedItems&&this._allowDynamicItemsAppend()},_allowDynamicItemsAppend:function(){return!1},_clean:function(){this._cleanFocusState(),this._cleanItemContainer()},_cleanItemContainer:function(){(0,o.default)(this._itemContainer()).empty()},_dispose:function(){this.callBase(),clearTimeout(this._itemFocusTimeout)},_refresh:function(){this._cleanRenderedItems(),this.callBase.apply(this,arguments)},_itemContainer:function(){return this.$element()},_itemClass:function(){return E},_itemContentClass:function(){return this._itemClass()+A},_selectedItemClass:function(){return F},_itemResponseWaitClass:function(){return L},_itemSelector:function(){return"."+this._itemClass()},_itemDataKey:function(){return R},_itemIndexKey:function(){return M},_itemElements:function(){return this._itemContainer().find(this._itemSelector())},_initMarkup:function(){this.callBase(),this.onFocusedItemChanged=this._createActionByOption("onFocusedItemChanged"),this.$element().addClass(O),this._prepareContent()},_prepareContent:(0,r.deferRenderer)(function(){this._renderContentImpl()}),_renderContent:function(){this._fireContentReadyAction()},_render:function(){this.callBase(),this._attachClickEvent(),this._attachHoldEvent(),this._attachContextMenuEvent()},_attachClickEvent:function(){var e=this._itemSelector(),t=(0,v.addNamespace)(D.name,this.NAME),n=(0,v.addNamespace)(y.default.down,this.NAME),i=this,r=new g.default(function(e){var t=e.event;i._itemPointerDownHandler(t)});a.default.off(this._itemContainer(),t,e),a.default.off(this._itemContainer(),n,e),a.default.on(this._itemContainer(),t,e,function(e){this._itemClickHandler(e)}.bind(this)),a.default.on(this._itemContainer(),n,e,function(e){r.execute({element:(0,o.default)(e.target),event:e})})},_itemClickHandler:function(e,t,n){this._itemDXEventHandler(e,"onItemClick",t,n)},_itemPointerDownHandler:function(e){this.option("focusStateEnabled")&&(this._itemFocusHandler=function(){if(clearTimeout(this._itemFocusTimeout),this._itemFocusHandler=null,!e.isDefaultPrevented()){var t=(0,o.default)(e.target),n=t.closest(this._itemElements()),i=this._closestFocusable(t);n.length&&i&&(0,f.inArray)(i.get(0),this._focusTarget())!==-1&&this.option("focusedElement",(0,l.getPublicElement)(n))}}.bind(this),this._itemFocusTimeout=setTimeout(this._forcePointerDownFocus.bind(this)))},_closestFocusable:function(e){if(e.is(w.focusable))return e;for(e=e.parent();e.length&&!u.default.isDocument(e.get(0));){if(e.is(w.focusable))return e;e=e.parent()}},_forcePointerDownFocus:function(){this._itemFocusHandler&&this._itemFocusHandler()},_updateFocusState:function(){this.callBase.apply(this,arguments),this._forcePointerDownFocus()},_attachHoldEvent:function(){var e=this._itemContainer(),t=this._itemSelector(),n=(0,v.addNamespace)(k.default.name,this.NAME);a.default.off(e,n,t),a.default.on(e,n,t,{timeout:this._getHoldTimeout()},this._itemHoldHandler.bind(this))},_getHoldTimeout:function(){return this.option("itemHoldTimeout")},_shouldFireHoldEvent:function(){return this.hasActionSubscription("onItemHold")},_itemHoldHandler:function(e){this._shouldFireHoldEvent()?this._itemDXEventHandler(e,"onItemHold"):e.cancel=!0},_attachContextMenuEvent:function(){var e=this._itemContainer(),t=this._itemSelector(),n=(0,v.addNamespace)(I.name,this.NAME);a.default.off(e,n,t),a.default.on(e,n,t,this._itemContextMenuHandler.bind(this))},_shouldFireContextMenuEvent:function(){return this.hasActionSubscription("onItemContextMenu")},_itemContextMenuHandler:function(e){this._shouldFireContextMenuEvent()?this._itemDXEventHandler(e,"onItemContextMenu"):e.cancel=!0},_renderContentImpl:function(){var e=this.option("items")||[];this._renderedItemsCount?this._renderItems(e.slice(this._renderedItemsCount)):this._renderItems(e)},_renderItems:function(e){e.length&&(0,p.each)(e,function(e,t){this._renderItem(this._renderedItemsCount+e,t)}.bind(this)),this._renderEmptyMessage()},_renderItem:function(e,t,n,i){n=n||this._itemContainer();var o=this._renderItemFrame(e,t,n,i);this._setElementData(o,t,e),o.attr(this.option("_itemAttributes")),this._attachItemClickEvent(t,o);var a=this._getItemContent(o),r=this._renderItemContent({index:e,itemData:t,container:(0,l.getPublicElement)(a),contentClass:this._itemContentClass(),defaultTemplateName:this.option("itemTemplate")}),s=this;return(0,d.when)(r).done(function(n){s._postprocessRenderItem({itemElement:o,itemContent:n,itemData:t,itemIndex:e}),s._executeItemRenderAction(e,t,(0,l.getPublicElement)(o))}),o},_getItemContent:function(e){var t=e.find("."+P);return t.removeClass(P),t},_attachItemClickEvent:function(e,t){e&&e.onClick&&a.default.on(t,D.name,function(n){this._itemEventHandlerByHandler(t,e.onClick,{event:n})}.bind(this))},_renderItemContent:function(e){var t=this._getItemTemplateName(e),n=this._getTemplate(t);this._addItemContentClasses(e);var i=(0,o.default)(this._createItemByTemplate(n,e));return i.hasClass(z)?this._renderItemContentByNode(e,i):e.container},_renderItemContentByNode:function(e,t){return(0,o.default)(e.container).replaceWith(t),e.container=(0,l.getPublicElement)(t),this._addItemContentClasses(e),t},_addItemContentClasses:function(e){var t=[E+A,e.contentClass];(0,o.default)(e.container).addClass(t.join(" "))},_appendItemToContainer:function(e,t,n){t.appendTo(e)},_renderItemFrame:function(e,t,n,i){var a=(0,o.default)("<div>");if(new this.constructor.ItemClass(a,this._itemOptions(),t||{}),i&&i.length?i.replaceWith(a):this._appendItemToContainer.call(this,n,a,e),this.option("showItemDataTitle")){var r=this._displayGetter?this._displayGetter(t):t;a.attr("title",r)}return a},_itemOptions:function(){var e=this;return{watchMethod:function(){return e.option("integrationOptions.watchMethod")},owner:e,fieldGetter:function(t){var n=e.option(t+"Expr"),i=(0,S.compileGetter)(n);return i}}},_postprocessRenderItem:r.noop,_executeItemRenderAction:function(e,t,n){this._getItemRenderAction()({itemElement:n,itemIndex:e,itemData:t})},_setElementData:function(e,t,n){e.addClass([E,this._itemClass()].join(" ")).data(this._itemDataKey(),t).data(this._itemIndexKey(),n)},_createItemRenderAction:function(){return this._itemRenderAction=this._createActionByOption("onItemRendered",{element:this.element(),excludeValidators:["disabled","readOnly"],category:"rendering"})},_getItemRenderAction:function(){return this._itemRenderAction||this._createItemRenderAction()},_getItemTemplateName:function(e){var t=e.itemData,n=e.templateProperty||this.option("itemTemplateProperty"),i=t&&t[n];return i||e.defaultTemplateName},_createItemByTemplate:function(e,t){return e.render({model:t.itemData,container:t.container,index:t.index})},_emptyMessageContainer:function(){return this._itemContainer()},_renderEmptyMessage:function(e){e=e||this.option("items");var t=this.option("noDataText"),n=!t||e&&e.length||this._isDataSourceLoading();n&&this._$noData&&(this._$noData.remove(),this._$noData=null,this.setAria("label",void 0)),n||(this._$noData=this._$noData||(0,o.default)("<div>").addClass("dx-empty-message"),this._$noData.appendTo(this._emptyMessageContainer()).html(t),this.setAria("label",t)),this.$element().toggleClass(H,!n)},_itemDXEventHandler:function(e,t,n,i){this._itemEventHandler(e.target,t,(0,h.extend)(n,{event:e}),i)},_itemEventHandler:function(e,t,n,i){var o=this._createActionByOption(t,(0,h.extend)({validatingTargetName:"itemElement"},i));return this._itemEventHandlerImpl(e,o,n)},_itemEventHandlerByHandler:function(e,t,n,i){var o=this._createAction(t,(0,h.extend)({validatingTargetName:"itemElement"},i));return this._itemEventHandlerImpl(e,o,n)},_itemEventHandlerImpl:function(e,t,n){var i=this._closestItemElement((0,o.default)(e)),a=(0,h.extend)({},n);return t((0,h.extend)(n,this._extendActionArgs(i),a))},_extendActionArgs:function(e){return{itemElement:(0,l.getPublicElement)(e),itemIndex:this._itemElements().index(e),itemData:this._getItemData(e)}},_closestItemElement:function(e){return(0,o.default)(e).closest(this._itemSelector())},_getItemData:function(e){return(0,o.default)(e).data(this._itemDataKey())},_getSummaryItemsWidth:function(e,t){var n=0;return e&&(0,p.each)(e,function(e,i){n+=(0,o.default)(i).outerWidth(t||!1)}),n},getFocusedItemId:function(){return this._focusedItemId||(this._focusedItemId="dx-"+new _.default),this._focusedItemId},itemElements:function(){return this._itemElements()},itemsContainer:function(){return this._itemContainer()}}).include(b.default);X.ItemClass=x.default;var Q=X;t.default=Q,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=n(70),s=function(){function e(){i(this,e),this._counter=-1,this._deferreds={}}return a(e,[{key:"add",value:function(e){return this._counter++,this._deferreds[this._counter]=e,this._counter}},{key:"remove",value:function(e){return delete this._deferreds[e]}},{key:"cancel",value:function(e){return e in this._deferreds&&(this._deferreds[e].reject(r.CANCELED_TOKEN),!0)}},{key:"cancelAll",value:function(){for(;this._counter>-1;)this.cancel(this._counter),this._counter--}}]),e}();t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(15)),r=n(4),s=a.default.abstract,l=a.default.inherit({ctor:function(e){this._collectionWidget=e},getIndexByItemData:s,getItemDataByIndex:s,getKeysByItems:s,getItemsByKeys:s,itemsGetter:s,getKeyByIndex:function(e){var t=this._denormalizeItemIndex(e);return this.getKeysByItems([this.getItemDataByIndex(t)])[0]},_equalKeys:function(e,t){return this._collectionWidget._isKeySpecified()?(0,r.equalByValue)(e,t):e===t},beginCache:function(){this._cache={}},endCache:function(){this._cache=null},getIndexByKey:s,getNormalizedIndex:function(e){return this._isNormalizedItemIndex(e)?e:this._isItemIndex(e)?this._normalizeItemIndex(e):this._isDOMNode(e)?this._getNormalizedItemIndex(e):this._normalizeItemIndex(this.getIndexByItemData(e))},getIndex:function(e){return this._isNormalizedItemIndex(e)?this._denormalizeItemIndex(e):this._isItemIndex(e)?e:this._isDOMNode(e)?this._denormalizeItemIndex(this._getNormalizedItemIndex(e)):this.getIndexByItemData(e)},getItemElement:function(e){if(this._isNormalizedItemIndex(e))return this._getItemByNormalizedIndex(e);if(this._isItemIndex(e))return this._getItemByNormalizedIndex(this._normalizeItemIndex(e));if(this._isDOMNode(e))return(0,o.default)(e);var t=this._normalizeItemIndex(this.getIndexByItemData(e));return this._getItemByNormalizedIndex(t)},deleteItemAtIndex:s,itemPlacementFunc:function(e,t){return this._itemsFromSameParent(e,t)&&e<t?"after":"before"},moveItemAtIndexToIndex:s,_isNormalizedItemIndex:function(e){return"number"==typeof e&&Math.round(e)===e},_isDOMNode:function(e){var t;try{t=(0,o.default)(e)}catch(e){return!1}return t&&t.length&&t.get(0).nodeType},_isItemIndex:s,_getNormalizedItemIndex:s,_normalizeItemIndex:s,_denormalizeItemIndex:s,_getItemByNormalizedIndex:s,_itemsFromSameParent:s}),u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(260)),r=i(n(17)),s=i(n(47)),l=n(6),u=a.default.inherit({getSelectedItems:function(){return this._loadFilteredData(this.options.selectionFilter)},getSelectedItemKeys:function(){var e=new l.Deferred,t=this,n=this.options.key(),i=(0,o.isString)(n)?[n]:n;return this._loadFilteredData(this.options.selectionFilter,null,i).done(function(n){var i=n.map(function(e){return t.options.keyOf(e)});e.resolve(i)}).fail(e.reject),e.promise()},selectedItemKeys:function(e,t,n,i){if(i){var o=this.options.filter(),a=!o||JSON.stringify(o)===JSON.stringify(this.options.selectionFilter)&&n;a?this._setOption("selectionFilter",n?[]:null):this._addSelectionFilter(n,o,i)}else{t||this._setOption("selectionFilter",[]);for(var r=0;r<e.length;r++)n?this.removeSelectedItem(e[r]):this.addSelectedItem(e[r])}return this.onSelectionChanged(),(new l.Deferred).resolve()},setSelectedItems:function(e){this._setOption("selectionFilter",null);for(var t=0;t<e.length;t++)this.addSelectedItem(e[t])},isItemDataSelected:function(e){return this.isItemKeySelected(e)},isItemKeySelected:function(e){var t=this.options.selectionFilter;return!t||!!(0,s.default)([e]).filter(t).toArray().length},_getKeyExpr:function(){var e=this.options.key();return Array.isArray(e)&&1===e.length?e[0]:e},_normalizeKey:function(e){var t=this.options.key();return Array.isArray(t)&&1===t.length?e[t[0]]:e},_getFilterByKey:function(e){var t=this._getKeyExpr(),n=[t,"=",this._normalizeKey(e)];if(Array.isArray(t)){n=[];for(var i=0;i<t.length;i++)n.push([t[i],"=",e[t[i]]]),i!==t.length-1&&n.push("and")}return n},addSelectedItem:function(e){var t=this._getFilterByKey(e);this._addSelectionFilter(!1,t)},removeSelectedItem:function(e){var t=this._getFilterByKey(e);this._addSelectionFilter(!0,t)},validate:function(){var e=this.options.key;if(e&&void 0===e())throw r.default.Error("E1042","Deferred selection")},_findSubFilter:function(e,t){if(!e)return-1;for(var n=JSON.stringify(t),i=0;i<e.length;i++){var o=e[i];if(o&&JSON.stringify(o)===n)return i}return-1},_isLastSubFilter:function(e,t){return!(!e||!t)&&(this._findSubFilter(e,t)===e.length-1||0===this._findSubFilter([e],t));
},_addFilterOperator:function(e,t){return e.length>1&&(0,o.isString)(e[1])&&e[1]!==t&&(e=[e]),e.length&&e.push(t),e},_denormalizeFilter:function(e){return e&&(0,o.isString)(e[0])&&(e=[e]),e},_addSelectionFilter:function(e,t,n){var i=this,o=e?["!",t]:t,a=e?"and":"or",r=!0,s=i.options.selectionFilter||[];if(s=i._denormalizeFilter(s),s&&s.length){i._removeSameFilter(s,t,e,n);var l=i._removeSameFilter(s,t,!e),u=this._isKeyFilter(t)&&this._hasKeyFiltersOnlyStartingFromIndex(s,l);r=t.length&&!u,r&&(s=i._addFilterOperator(s,a))}r&&s.push(o),s=i._normalizeFilter(s),i._setOption("selectionFilter",e||s.length?s:null)},_normalizeFilter:function(e){return e&&1===e.length&&(e=e[0]),e},_removeFilterByIndex:function(e,t,n){var i=e[1];t>0?e.splice(t-1,2):e.splice(t,2),n&&"and"===i&&e.splice(0,e.length)},_isSimpleKeyFilter:function(e,t){return 3===e.length&&e[0]===t&&"="===e[1]},_isKeyFilter:function(e){if(2===e.length&&"!"===e[0])return this._isKeyFilter(e[1]);var t=this._getKeyExpr();if(Array.isArray(t)){if(e.length!==2*t.length-1)return!1;for(var n=0;n<t.length;n++){if(n>0&&"and"!==e[n])return!1;if(!this._isSimpleKeyFilter(e[2*n],t[n]))return!1}return!0}return this._isSimpleKeyFilter(e,t)},_hasKeyFiltersOnlyStartingFromIndex:function(e,t){if(t>=0){for(var n=t;n<e.length;n++)if("string"!=typeof e[n]&&!this._isKeyFilter(e[n]))return!1;return!0}return!1},_removeSameFilter:function(e,t,n,i){if(t=n?["!",t]:t,JSON.stringify(t)===JSON.stringify(e))return e.splice(0,e.length),0;var o=this._findSubFilter(e,t);if(o>=0)return this._removeFilterByIndex(e,o,i),o;for(var a=0;a<e.length;a++)if(Array.isArray(e[a])&&e[a].length>2){var r=this._removeSameFilter(e[a],t,!1,i);if(r>=0)return e[a].length?1===e[a].length&&(e[a]=e[a][0]):this._removeFilterByIndex(e,a,i),r}return-1},getSelectAllState:function(){var e=this.options.filter(),t=this.options.selectionFilter;return!t||!!t.length&&(e&&e.length?(t=this._denormalizeFilter(t),!!this._isLastSubFilter(t,e)||!this._isLastSubFilter(t,["!",e])&&void 0):void 0)}});t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(4),a=n(1),r=i(n(47)),s=n(6),l=n(274),u=i(n(17)),c=i(n(260)),d=c.default.inherit({ctor:function(e){this.callBase(e),this._initSelectedItemKeyHash()},_initSelectedItemKeyHash:function(){this._setOption("keyHashIndices",this.options.equalByReference?null:{})},getSelectedItemKeys:function(){return this.options.selectedItemKeys.slice(0)},getSelectedItems:function(){return this.options.selectedItems.slice(0)},_preserveSelectionUpdate:function(e,t){var n,i,o,a=this.options.keyOf;if(a){var r=t&&e.length>1&&!this.options.equalByReference;for(r&&(n={}),o=0;o<e.length;o++){var s=e[o],l=a(s);t?(i=this.removeSelectedItem(l,n),n&&i>=0&&(n[i]=!0)):this.addSelectedItem(l,s)}r&&this._batchRemoveSelectedItems(n)}},_batchRemoveSelectedItems:function(e){var t=this.options.selectedItemKeys.slice(0),n=this.options.selectedItems.slice(0);this.options.selectedItemKeys.length=0,this.options.selectedItems.length=0;for(var i=0;i<t.length;i++)e[i]||(this.options.selectedItemKeys.push(t[i]),this.options.selectedItems.push(n[i]));this._initSelectedItemKeyHash(),this.updateSelectedItemKeyHash(this.options.selectedItemKeys)},_loadSelectedItemsCore:function(e,t,n){var i=new s.Deferred,o=this.options.key();if(!e.length&&!n)return i.resolve([]),i;var a=this.options.filter();if(n&&t&&!a)return i.resolve(this.getSelectedItems()),i;var u=new l.SelectionFilterCreator(e,n),c=u.getCombinedFilter(o,a),d=[];t&&(d=c?(0,r.default)(this.options.selectedItems).filter(c).toArray():this.options.selectedItems.slice(0));var h=d.length?d:this.options.plainItems(!0).filter(this.options.isSelectableItem).map(this.options.getItemData),f=u.getLocalFilter(this.options.keyOf,this.equalKeys.bind(this),this.options.equalByReference,o);return h=h.filter(f),d.length||!n&&h.length===e.length?i.resolve(h):i=this._loadFilteredData(c,f,null,n),i},_replaceSelectionUpdate:function(e){var t=[],n=this.options.keyOf;if(n){for(var i=0;i<e.length;i++){var o=e[i],a=n(o);t.push(a)}this.setSelectedItems(t,e)}},_warnOnIncorrectKeys:function(e){for(var t=this.options.allowNullValue,n=0;n<e.length;n++){var i=e[n];t&&null===i||this.isItemKeySelected(i)||u.default.log("W1002",i)}},_loadSelectedItems:function(e,t,n){var i=this,o=new s.Deferred;return(0,s.when)(i._lastLoadDeferred).always(function(){i._loadSelectedItemsCore(e,t,n).done(o.resolve).fail(o.reject)}),i._lastLoadDeferred=o,o},selectedItemKeys:function(e,t,n,i){var o=this,a=o._loadSelectedItems(e,n,i);return a.done(function(e){t?o._preserveSelectionUpdate(e,n):o._replaceSelectionUpdate(e),o.onSelectionChanged()}),a},addSelectedItem:function(e,t){if((0,a.isDefined)(t)&&t.disabled)return void(this.options.disabledItemKeys.indexOf(e)===-1&&this.options.disabledItemKeys.push(e));var n=this._getKeyHash(e);this._indexOfSelectedItemKey(n)===-1&&(!(0,a.isObject)(n)&&this.options.keyHashIndices&&(this.options.keyHashIndices[n]=[this.options.selectedItemKeys.length]),this.options.selectedItemKeys.push(e),this.options.addedItemKeys.push(e),this.options.addedItems.push(t),this.options.selectedItems.push(t))},_getSelectedIndexByKey:function(e,t){for(var n=this.options.selectedItemKeys,i=0;i<n.length;i++)if((!t||!t[i])&&this.equalKeys(n[i],e))return i;return-1},_getSelectedIndexByHash:function(e,t){var n=this.options.keyHashIndices[e];return n&&n.length>1&&t&&(n=n.filter(function(e){return!t[e]})),n&&n[0]>=0?n[0]:-1},_indexOfSelectedItemKey:function(e,t){var n;return n=this.options.equalByReference?this.options.selectedItemKeys.indexOf(e):(0,a.isObject)(e)?this._getSelectedIndexByKey(e,t):this._getSelectedIndexByHash(e,t)},_shiftSelectedKeyIndices:function(e){for(var t=e;t<this.options.selectedItemKeys.length;t++){var n=this.options.selectedItemKeys[t],i=(0,o.getKeyHash)(n),a=this.options.keyHashIndices[i];if(a)for(var r=0;r<a.length;r++)a[r]>e&&a[r]--}},removeSelectedItem:function(e,t){var n=this._getKeyHash(e),i=!!t,o=this._indexOfSelectedItemKey(n,t);if(o<0)return o;if(this.options.removedItemKeys.push(e),this.options.removedItems.push(this.options.selectedItems[o]),i)return o;if(this.options.selectedItemKeys.splice(o,1),this.options.selectedItems.splice(o,1),(0,a.isObject)(n)||!this.options.keyHashIndices)return o;var r=this.options.keyHashIndices[n];return r?(r.shift(),r.length||delete this.options.keyHashIndices[n],this._shiftSelectedKeyIndices(o),o):o},_updateAddedItemKeys:function(e,t){for(var n=0;n<e.length;n++)this.isItemKeySelected(e[n])||(this.options.addedItemKeys.push(e[n]),this.options.addedItems.push(t[n]))},_updateRemovedItemKeys:function(e,t,n){for(var i=0;i<t.length;i++)this.isItemKeySelected(t[i])||(this.options.removedItemKeys.push(t[i]),this.options.removedItems.push(n[i]))},_getKeyHash:function(e){return this.options.equalByReference?e:(0,o.getKeyHash)(e)},setSelectedItems:function(e,t){this._updateAddedItemKeys(e,t);var n=this.options.selectedItemKeys,i=this.options.selectedItems;this.options.equalByReference||(this._initSelectedItemKeyHash(),this.updateSelectedItemKeyHash(e)),this._setOption("selectedItemKeys",e),this._setOption("selectedItems",t),this._updateRemovedItemKeys(e,n,i)},isItemDataSelected:function(e){var t=this.options.keyOf(e);return this.isItemKeySelected(t)},isItemKeySelected:function(e){var t=this._getKeyHash(e),n=this._indexOfSelectedItemKey(t);return n!==-1},getSelectAllState:function(e){return e?this._getVisibleSelectAllState():this._getFullSelectAllState()}});t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(116)),a=i(n(5)),r=i(n(189)),s=i(n(122));s.default&&o.default.add(function(e){var t=e.slice(0,2)+e.charAt(2).toUpperCase()+e.slice(3);r.default.directive(t,["$parse",function(n){return function(i,o,r){var s,l=r[t].trim(),u={};"{"===l.charAt(0)?(u=i.$eval(l),s=n(u.execute)):s=n(r[t]),a.default.on(o,e,u,function(e){i.$apply(function(){s(i,{$event:e})})})}}])})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(27)),a=i(n(189)),r=i(n(122));r.default&&a.default.service("dxDigestCallbacks",["$rootScope",function(e){var t=(0,o.default)(),n=(0,o.default)(),i=(0,o.default)(),a=!1;return e.$watch(function(){a||(a=!0,t.fire(),e.$$postDigest(function(){a=!1,n.fire(),i.fire()}))}),{begin:{add:function(e){a&&e(),t.add(e)},remove:t.remove.bind(t)},end:{add:i.add.bind(i),addPrioritized:n.add.bind(n),remove:i.remove.bind(i)}}}])},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(108)),a=i(n(122));a.default&&o.default.registerExecutor({ngExpression:{execute:function(e){"string"==typeof e.action&&e.context.$eval(e.action)}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(61)),a=i(n(22)),r=n(60);if(n(399),n(401),n(402),n(403),n(404),n(405),n(406),o.default&&(0,r.compare)(o.default.version,[2,3])<0)throw a.default.Error("E0013")},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(61)),r=i(n(27)),s=n(1),l=i(n(155)),u=i(n(16)),c=n(400),d=i(n(52)),h=i(n(259)),f=n(261),p=i(n(28));if(a.default){var g="dxKoLocks",_="dxKoCreation",m=[],v=function(e,t){t.subclassOf(d.default)&&m.push(e),a.default.bindingHandlers[e]={init:function(e,n){function i(e,t){for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&I(e,n,t?[t,n].join("."):n)}var l,d,m=(0,o.default)(e),v=(0,r.default)(),y={},b=(0,p.default)().knockout,x=b&&b.isBindingPropertyPredicateName,w={onInitializing:function(){y=this._getOptionsByReference(),a.default.computed(function(){var e=a.default.unwrap(n());l&&l.beginUpdate(),d=x&&e&&e[x],i(e),l&&l.endUpdate()},null,{disposeWhenNodeIsRemoved:e}),l=this},modelByElement:function(e){if(e.length){var t=(0,f.getClosestNodeWithContext)(e.get(0));return a.default.dataFor(t)}},nestedComponentOptions:function(e){return{modelByElement:e.option("modelByElement"),nestedComponentOptions:e.option("nestedComponentOptions")}},_optionChangedCallbacks:v,integrationOptions:{watchMethod:function(e,t,n){n=n||{};var i=n.skipImmediate,o=a.default.computed(function(){var n=a.default.unwrap(e());i||t(n),i=!1});return function(){o.dispose()}},templates:{"dx-polymorph-widget":{render:function(e){var t=a.default.utils.unwrapObservable(e.model.widget);if(t){var n=(0,o.default)("<div>").attr("data-bind",t+": options").get(0);(0,o.default)(e.container).append(n),a.default.applyBindings(e.model,n)}}}},createTemplate:function(e){return new c.KoTemplate(e)}}},C={},k=function(e,t,n){var i=m.data(g),o=n?a.default.unwrap(t):t;if(a.default.isWriteableObservable(t)&&(C[e]=t),l){if(i.locked(e))return;i.obtain(e);try{a.default.ignoreDependencies?a.default.ignoreDependencies(l.option,l,[e,o]):l.option(e,o)}finally{i.release(e)}}else w[e]=o},S=function(e){var t=e.fullName,n=e.value;if(t in C){var i=this._$element,o=i.data(g);if(!o.locked(t)){o.obtain(t);try{C[t](n)}finally{o.release(t)}}}},D=function(){v.add(S),m.data(_,!0).data(g,new h.default),new t(m,w),w=null},I=function(t,n,o){if(o!==x)if(!d||d(o,n,t)){var r;a.default.computed(function(){var e=t[n];k(o,e,!0),r=a.default.unwrap(e)},null,{disposeWhenNodeIsRemoved:e}),(0,s.isPlainObject)(r)&&(y[o]||i(r,o))}else k(o,t[n],!1)};return D(),{controlsDescendantBindings:t.subclassOf(u.default)}}},"dxValidator"===e&&(a.default.bindingHandlers.dxValidator.after=m)};l.default.add(function(e,t){v(e,t)})}},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.KoTemplate=void 0;var g=o(n(2)),_=o(n(11)),m=o(n(61)),v=n(1),y=n(67),b=n(33),x=n(261),w=function(e){var t=_.default.createElement("div");m.default.applyBindingsToNode(t,null,e);var n=m.default.contextFor(t);return m.default.cleanNode(t),n},C=function(e){function t(e){var i;return a(this,t),i=n.call(this),i._element=e,i._template=(0,g.default)("<div>").append((0,b.normalizeTemplateElement)(e)),i._registerKoTemplate(),i}l(t,e);var n=c(t);return s(t,[{key:"_registerKoTemplate",value:function(){var e=this._template.get(0);new m.default.templateSources.anonymousTemplate(e).nodes(e)}},{key:"_prepareDataForContainer",value:function(e,t){if(t&&t.length){var n=(0,x.getClosestNodeWithContext)(t.get(0)),i=m.default.contextFor(n);if(e=void 0!==e?e:m.default.dataFor(n)||{},i)return e===i.$data?i:i.createChildContext(e)}return w(e).createChildContext(e)}},{key:"_renderCore",value:function(e){var t=this._prepareDataForContainer(e.model,(0,g.default)(e.container));(0,v.isDefined)(e.index)&&(t.$index=e.index);var n,i=(0,g.default)("<div>").appendTo(e.container);return m.default.renderTemplate(this._template.get(0),t,{afterRender:function(e){n=(0,g.default)(e)}},i.get(0),"replaceNode"),n}},{key:"source",value:function(){return(0,g.default)(this._element).clone()}},{key:"dispose",value:function(){this._template.remove()}}]),t}(y.TemplateBase);t.KoTemplate=C},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(5)),r=i(n(61)),s=n(1),l=i(n(116)),u=n(8);r.default&&l.default.add(function(e){var t=(0,u.addNamespace)(e,e+"Binding");r.default.bindingHandlers[e]={update:function(e,n,i,l){var u=(0,o.default)(e),c=r.default.utils.unwrapObservable(n()),d=c.execute?c.execute:c;a.default.off(u,t),a.default.on(u,t,(0,s.isPlainObject)(c)?c:{},function(e){d.call(l,l,e)})}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(61)),a=n(59);o.default&&(o.default.bindingHandlers.dxControlsDescendantBindings={init:function(e,t){return{controlsDescendantBindings:o.default.unwrap(t())}}},o.default.bindingHandlers.dxIcon={init:function(e,t){var n=o.default.utils.unwrapObservable(t())||{},i=(0,a.getImageContainer)(n);o.default.virtualElements.emptyNode(e),i&&o.default.virtualElements.prepend(e,i.get(0))},update:function(e,t){var n=o.default.utils.unwrapObservable(t())||{},i=(0,a.getImageContainer)(n);o.default.virtualElements.emptyNode(e),i&&o.default.virtualElements.prepend(e,i.get(0))}},o.default.virtualElements.allowedBindings.dxIcon=!0)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=n(3),a=n(0),r=i(n(15)),s=n(92),l=i(n(100)),u=n(6),c=i(n(35)),d=i(n(61));if(d.default){var h="valid",f="pending",p=r.default.inherit({ctor:function(e,t){var n=this,i=t.name,r=t.validationRules;this.target=e,this.name=i,this.isValid=d.default.observable(!0),this.validationError=d.default.observable(),this.validationErrors=d.default.observable(),this.validationStatus=d.default.observable(h),this._eventsStrategy=new s.EventsStrategy(this),this.validationRules=(0,o.map)(r,function(e,t){return(0,a.extend)({},e,{validator:n,index:t})}),this._validationInfo={result:null,deferred:null}},_updateValidationResult:function(e){if(this._validationInfo.result&&this._validationInfo.result.id===e.id)for(var t in e)"id"!==t&&"complete"!==t&&(this._validationInfo.result[t]=e[t]);else{var n=this._validationInfo.deferred&&this._validationInfo.result.complete;this._validationInfo.result=(0,a.extend)({},e,{complete:n})}},validate:function(){var e=this,t=this._validationInfo&&this._validationInfo.result,n=this.target();if(t&&t.status===f&&t.value===n)return(0,a.extend)({},t);var i=l.default.validate(n,this.validationRules,this.name);return i.id=(new c.default).toString(),this._applyValidationResult(i),i.complete&&i.complete.then(function(t){t.id===e._validationInfo.result.id&&e._applyValidationResult(t)}),(0,a.extend)({},this._validationInfo.result)},reset:function(){this.target(null);var e={id:null,isValid:!0,brokenRule:null,pendingRules:null,status:h,complete:null};return this._applyValidationResult(e),e},_applyValidationResult:function(e){return e.validator=this,this._updateValidationResult(e),this.target.dxValidator.isValid(this._validationInfo.result.isValid),this.target.dxValidator.validationError(this._validationInfo.result.brokenRule),this.target.dxValidator.validationErrors(this._validationInfo.result.brokenRules),this.target.dxValidator.validationStatus(this._validationInfo.result.status),e.status===f?(this._validationInfo.deferred||(this._validationInfo.deferred=new u.Deferred,this._validationInfo.result.complete=this._validationInfo.deferred.promise()),void this._eventsStrategy.fireEvent("validating",[this._validationInfo.result])):void(e.status!==f&&(this._eventsStrategy.fireEvent("validated",[e]),this._validationInfo.deferred&&(this._validationInfo.deferred.resolve(e),this._validationInfo.deferred=null)))},on:function(e,t){return this._eventsStrategy.on(e,t),this},off:function(e,t){return this._eventsStrategy.off(e,t),this}});d.default.extenders.dxValidator=function(e,t){return e.dxValidator=new p(e,t),e.subscribe(e.dxValidator.validate.bind(e.dxValidator)),e},l.default.registerModelForValidation=function(e){(0,o.each)(e,function(t,n){d.default.isObservable(n)&&n.dxValidator&&l.default.registerValidatorInGroup(e,n.dxValidator)})},l.default.unregisterModelForValidation=function(e){(0,o.each)(e,function(t,n){d.default.isObservable(n)&&n.dxValidator&&l.default.removeRegisteredValidator(e,n.dxValidator)})},l.default.validateModel=l.default.validateGroup}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(61)),a=i(n(101));o.default&&a.default.inject({isWrapped:o.default.isObservable,isWritableWrapped:o.default.isWritableObservable,wrap:o.default.observable,unwrap:function(e){return o.default.isObservable(e)?o.default.utils.unwrapObservable(e):this.callBase(e)},assign:function(e,t){o.default.isObservable(e)?e(t):this.callBase(e,t)}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=n(39),a=i(n(61)),r=n(60);if(a.default){var s=a.default.utils.domNodeDisposal.cleanExternalData,l=function(){(0,o.afterCleanData)(function(e){var t;for(t=0;t<e.length;t++)e[t].cleanedByJquery=!0;for(t=0;t<e.length;t++)e[t].cleanedByKo||a.default.cleanNode(e[t]),delete e[t].cleanedByKo;for(t=0;t<e.length;t++)delete e[t].cleanedByJquery}),a.default.utils.domNodeDisposal.cleanExternalData=function(e){e.cleanedByKo=!0,e.cleanedByJquery||(0,o.cleanData)([e])}},u=function(){(0,o.afterCleanData)(function(){}),a.default.utils.domNodeDisposal.cleanExternalData=s};l(),o.strategyChanging.add(function(e){var t=!!e.fn;t&&(0,r.compare)(e.fn.jquery,[2,0])<0&&u()})}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(61)),a=n(60),r=n(39);if(o.default){var s=function(e){var t=function(e,t){var n=function(){o.default.cleanNode(this)};t?e.each(n):e.find("*").each(n)},n=e.fn.empty;e.fn.empty=function(){return t(this,!1),n.apply(this,arguments)};var i=e.fn.remove;e.fn.remove=function(e,n){if(!n){var o=this;e&&(o=o.filter(e)),t(o,!0)}return i.call(this,e,n)};var a=e.fn.html;e.fn.html=function(e){return"string"==typeof e&&t(this,!1),a.apply(this,arguments)};var r=e.fn.replaceWith;e.fn.replaceWith=function(){var e=r.apply(this,arguments);return this.parent().length||t(this,!0),e}};r.strategyChanging.add(function(e){var t=!!e.fn;t&&(0,a.compare)(e.fn.jquery,[2,0])<0&&s(e)})}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}n(124);var o=i(n(72)),a=i(n(10)),r=i(n(82));if(n(72),o.default&&o.default.formatMessage){var s="en",l=o.default.loadMessages;o.default.loadMessages=function(e){a.default.load(e)};var u={engine:function(){return"globalize"},ctor:function(){this.load(this._dictionary)},load:function(e){this.callBase(e),l(e)},getMessagesByLocales:function(){return o.default.cldr.get("globalize-messages")},getFormatter:function(e,t){var n=t||r.default.locale(),i=this._getFormatterBase(e,t);return i||(i=this._formatterByGlobalize(e,t)),i||n===s||(i=this.getFormatter(e,s)),i},_formatterByGlobalize:function(e,t){var n,i=t&&t!==r.default.locale()?new o.default(t):o.default;return this._messageLoaded(e,t)&&(n=i.messageFormatter(e)),n},_messageLoaded:function(e,t){var n=t?new o.default(t).cldr:o.default.locale(),i=n.get(["globalize-messages/{bundle}",e]);return!!i},_loadSingle:function(e,t,n){var i={};i[n]={},i[n][e]=t,this.load(i)}};a.default.inject(u)}},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}n(124),n(190),n(72);var s=r(n(72)),l=r(n(32)),u=n(1),c=n(12),d=a(n(3)),h={supplemental:{version:{_cldrVersion:"28",_unicodeVersion:"8.0.0",_number:"$Revision: 11969 $"},timeData:{"001":{_allowed:"H h",_preferred:"H"},DE:{_allowed:"H",_preferred:"H"},JP:{_allowed:"H K h",_preferred:"H"},RU:{_allowed:"H",_preferred:"H"},US:{_allowed:"H h",_preferred:"h"}}}},f={main:{en:{identity:{version:{_cldrVersion:"28",_number:"$Revision: 11972 $"},language:"en"},dates:{calendars:{gregorian:{months:{format:{abbreviated:{1:"Jan",2:"Feb",3:"Mar",4:"Apr",5:"May",6:"Jun",7:"Jul",8:"Aug",9:"Sep",10:"Oct",11:"Nov",12:"Dec"},narrow:{1:"J",2:"F",3:"M",4:"A",5:"M",6:"J",7:"J",8:"A",9:"S",10:"O",11:"N",12:"D"},wide:{1:"January",2:"February",3:"March",4:"April",5:"May",6:"June",7:"July",8:"August",9:"September",10:"October",11:"November",12:"December"}},"stand-alone":{abbreviated:{1:"Jan",2:"Feb",3:"Mar",4:"Apr",5:"May",6:"Jun",7:"Jul",8:"Aug",9:"Sep",10:"Oct",11:"Nov",12:"Dec"},narrow:{1:"J",2:"F",3:"M",4:"A",5:"M",6:"J",7:"J",8:"A",9:"S",10:"O",11:"N",12:"D"},wide:{1:"January",2:"February",3:"March",4:"April",5:"May",6:"June",7:"July",8:"August",9:"September",10:"October",11:"November",12:"December"}}},days:{format:{abbreviated:{sun:"Sun",mon:"Mon",tue:"Tue",wed:"Wed",thu:"Thu",fri:"Fri",sat:"Sat"},narrow:{sun:"S",mon:"M",tue:"T",wed:"W",thu:"T",fri:"F",sat:"S"},"short":{sun:"Su",mon:"Mo",tue:"Tu",wed:"We",thu:"Th",fri:"Fr",sat:"Sa"},wide:{sun:"Sunday",mon:"Monday",tue:"Tuesday",wed:"Wednesday",thu:"Thursday",fri:"Friday",sat:"Saturday"}},"stand-alone":{abbreviated:{sun:"Sun",mon:"Mon",tue:"Tue",wed:"Wed",thu:"Thu",fri:"Fri",sat:"Sat"},narrow:{sun:"S",mon:"M",tue:"T",wed:"W",thu:"T",fri:"F",sat:"S"},"short":{sun:"Su",mon:"Mo",tue:"Tu",wed:"We",thu:"Th",fri:"Fr",sat:"Sa"},wide:{sun:"Sunday",mon:"Monday",tue:"Tuesday",wed:"Wednesday",thu:"Thursday",fri:"Friday",sat:"Saturday"}}},quarters:{format:{abbreviated:{1:"Q1",2:"Q2",3:"Q3",4:"Q4"},narrow:{1:"1",2:"2",3:"3",4:"4"},wide:{1:"1st quarter",2:"2nd quarter",3:"3rd quarter",4:"4th quarter"}},"stand-alone":{abbreviated:{1:"Q1",2:"Q2",3:"Q3",4:"Q4"},narrow:{1:"1",2:"2",3:"3",4:"4"},wide:{1:"1st quarter",2:"2nd quarter",3:"3rd quarter",4:"4th quarter"}}},dayPeriods:{format:{abbreviated:{midnight:"midnight",am:"AM","am-alt-variant":"am",noon:"noon",pm:"PM","pm-alt-variant":"pm",morning1:"in the morning",afternoon1:"in the afternoon",evening1:"in the evening",night1:"at night"},narrow:{midnight:"mi",am:"a","am-alt-variant":"am",noon:"n",pm:"p","pm-alt-variant":"pm",morning1:"in the morning",afternoon1:"in the afternoon",evening1:"in the evening",night1:"at night"},wide:{midnight:"midnight",am:"AM","am-alt-variant":"am",noon:"noon",pm:"PM","pm-alt-variant":"pm",morning1:"in the morning",afternoon1:"in the afternoon",evening1:"in the evening",night1:"at night"}},"stand-alone":{abbreviated:{midnight:"midnight",am:"AM","am-alt-variant":"am",noon:"noon",pm:"PM","pm-alt-variant":"pm",morning1:"in the morning",afternoon1:"in the afternoon",evening1:"in the evening",night1:"at night"},narrow:{midnight:"midnight",am:"AM","am-alt-variant":"am",noon:"noon",pm:"PM","pm-alt-variant":"pm",morning1:"in the morning",afternoon1:"in the afternoon",evening1:"in the evening",night1:"at night"},wide:{midnight:"midnight",am:"AM","am-alt-variant":"am",noon:"noon",pm:"PM","pm-alt-variant":"pm",morning1:"morning",afternoon1:"afternoon",evening1:"evening",night1:"night"}}},eras:{eraNames:{0:"Before Christ","0-alt-variant":"Before Common Era",1:"Anno Domini","1-alt-variant":"Common Era"},eraAbbr:{0:"BC","0-alt-variant":"BCE",1:"AD","1-alt-variant":"CE"},eraNarrow:{0:"B","0-alt-variant":"BCE",1:"A","1-alt-variant":"CE"}},dateFormats:{full:"EEEE, MMMM d, y","long":"MMMM d, y",medium:"MMM d, y","short":"M/d/yy"},timeFormats:{full:"h:mm:ss a zzzz","long":"h:mm:ss a z",medium:"h:mm:ss a","short":"h:mm a"},dateTimeFormats:{full:"{1} 'at' {0}","long":"{1} 'at' {0}",medium:"{1}, {0}","short":"{1}, {0}",availableFormats:{d:"d",E:"ccc",Ed:"d E",Ehm:"E h:mm a",EHm:"E HH:mm",Ehms:"E h:mm:ss a",EHms:"E HH:mm:ss",Gy:"y G",GyMMM:"MMM y G",GyMMMd:"MMM d, y G",GyMMMEd:"E, MMM d, y G",h:"h a",H:"HH",hm:"h:mm a",Hm:"HH:mm",hms:"h:mm:ss a",Hms:"HH:mm:ss",hmsv:"h:mm:ss a v",Hmsv:"HH:mm:ss v",hmv:"h:mm a v",Hmv:"HH:mm v",M:"L",Md:"M/d",MEd:"E, M/d",MMM:"LLL",MMMd:"MMM d",MMMEd:"E, MMM d",MMMMd:"MMMM d",ms:"mm:ss",y:"y",yM:"M/y",yMd:"M/d/y",yMEd:"E, M/d/y",yMMM:"MMM y",yMMMd:"MMM d, y",yMMMEd:"E, MMM d, y",yMMMM:"MMMM y",yQQQ:"QQQ y",yQQQQ:"QQQQ y"},appendItems:{Day:"{0} ({2}: {1})","Day-Of-Week":"{0} {1}",Era:"{0} {1}",Hour:"{0} ({2}: {1})",Minute:"{0} ({2}: {1})",Month:"{0} ({2}: {1})",Quarter:"{0} ({2}: {1})",Second:"{0} ({2}: {1})",Timezone:"{0} {1}",Week:"{0} ({2}: {1})",Year:"{0} {1}"},intervalFormats:{intervalFormatFallback:"{0} \u2013 {1}",d:{d:"d \u2013 d"},h:{a:"h a \u2013 h a",h:"h \u2013 h a"},H:{H:"HH \u2013 HH"},hm:{a:"h:mm a \u2013 h:mm a",h:"h:mm \u2013 h:mm a",m:"h:mm \u2013 h:mm a"},Hm:{H:"HH:mm \u2013 HH:mm",m:"HH:mm \u2013 HH:mm"},hmv:{a:"h:mm a \u2013 h:mm a v",h:"h:mm \u2013 h:mm a v",m:"h:mm \u2013 h:mm a v"},Hmv:{H:"HH:mm \u2013 HH:mm v",m:"HH:mm \u2013 HH:mm v"},hv:{a:"h a \u2013 h a v",h:"h \u2013 h a v"},Hv:{H:"HH \u2013 HH v"},M:{M:"M \u2013 M"},Md:{d:"M/d \u2013 M/d",M:"M/d \u2013 M/d"},MEd:{d:"E, M/d \u2013 E, M/d",M:"E, M/d \u2013 E, M/d"},MMM:{M:"MMM \u2013 MMM"},MMMd:{d:"MMM d \u2013 d",M:"MMM d \u2013 MMM d"},MMMEd:{d:"E, MMM d \u2013 E, MMM d",M:"E, MMM d \u2013 E, MMM d"},y:{y:"y \u2013 y"},yM:{M:"M/y \u2013 M/y",y:"M/y \u2013 M/y"},yMd:{d:"M/d/y \u2013 M/d/y",M:"M/d/y \u2013 M/d/y",y:"M/d/y \u2013 M/d/y"},yMEd:{d:"E, M/d/y \u2013 E, M/d/y",M:"E, M/d/y \u2013 E, M/d/y",y:"E, M/d/y \u2013 E, M/d/y"},yMMM:{M:"MMM \u2013 MMM y",y:"MMM y \u2013 MMM y"},yMMMd:{d:"MMM d \u2013 d, y",M:"MMM d \u2013 MMM d, y",y:"MMM d, y \u2013 MMM d, y"},yMMMEd:{d:"E, MMM d \u2013 E, MMM d, y",M:"E, MMM d \u2013 E, MMM d, y",y:"E, MMM d, y \u2013 E, MMM d, y"},yMMMM:{M:"MMMM \u2013 MMMM y",y:"MMMM y \u2013 MMMM y"}}}}}}}}},p={supplemental:{version:{_cldrVersion:"28",_unicodeVersion:"8.0.0",_number:"$Revision: 11969 $"},weekData:{minDays:{"001":"1",US:"1",DE:"4"},firstDay:{"001":"mon",DE:"mon",RU:"mon",JP:"sun",US:"sun"},weekendStart:{"001":"sat"},weekendEnd:{"001":"sun"}}}},g=["skeleton","date","time","datetime","raw"],_=/[\u200E\u200F]/g;if(s.default&&s.default.formatDate){"en"===s.default.locale().locale&&(s.default.load(p,h,f),s.default.locale("en"));var m={},v={shortdate:{path:"dateTimeFormats/availableFormats/yMd"},shorttime:{path:"timeFormats/short"},longdate:{path:"dateFormats/full"},longtime:{path:"timeFormats/medium"},monthandday:{path:"dateTimeFormats/availableFormats/MMMMd"},monthandyear:{path:"dateTimeFormats/availableFormats/yMMMM"},quarterandyear:{path:"dateTimeFormats/availableFormats/yQQQ"},day:{path:"dateTimeFormats/availableFormats/d"},year:{path:"dateTimeFormats/availableFormats/y"},shortdateshorttime:{path:"dateTimeFormats/short",parts:["shorttime","shortdate"]},longdatelongtime:{path:"dateTimeFormats/medium",parts:["longtime","longdate"]},month:{pattern:"LLLL"},shortyear:{pattern:"yy"},dayofweek:{pattern:"EEEE"},quarter:{pattern:"QQQ"},millisecond:{pattern:"SSS"},hour:{pattern:"HH"},minute:{pattern:"mm"},second:{pattern:"ss"}},y={engine:function(){return"globalize"},_getPatternByFormat:function(e){var t=this,n=e.toLowerCase(),i=v[n];if("datetime-local"===n)return"yyyy-MM-ddTHH':'mm':'ss";if(i){var o=i.path&&t._getFormatStringByPath(i.path)||i.pattern;return i.parts&&d.each(i.parts,function(e,n){o=o.replace("{"+e+"}",t._getPatternByFormat(n))}),o}},_getFormatStringByPath:function(e){return s.default.locale().main("dates/calendars/gregorian/"+e)},getPeriodNames:function(){var e=s.default.locale().main("dates/calendars/gregorian/dayPeriods/stand-alone/wide");return[e.am,e.pm]},getMonthNames:function(e,t){var n=s.default.locale().main("dates/calendars/gregorian/months/"+("format"===t?t:"stand-alone")+"/"+(e||"wide"));return d.map(n,function(e){return e})},getDayNames:function(e){var t=s.default.locale().main("dates/calendars/gregorian/days/stand-alone/"+(e||"wide"));return d.map(t,function(e){return e})},getTimeSeparator:function(){return s.default.locale().main("numbers/symbols-numberSystem-latn/timeSeparator")},removeRtlMarks:function(e){return e.replace(_,"")},format:function(e,t){if(e){if(!t)return e;var n,i;if("function"==typeof t)return t(e);if(t.formatter)return t.formatter(e);if(t=t.type||t,"string"==typeof t)i=s.default.locale().locale+":"+t,n=m[i],n||(t={raw:this._getPatternByFormat(t)||t},n=m[i]=s.default.dateFormatter(t));else{if(!this._isAcceptableFormat(t))return;n=s.default.dateFormatter(t)}return this.removeRtlMarks(n(e))}},parse:function(e,t){if(e){if(!t||"function"==typeof t||(0,u.isObject)(t)&&!this._isAcceptableFormat(t)){if(t){var n=this.callBase(e,t);if(n)return n}return s.default.parseDate(e)}return t.parser?t.parser(e):("string"==typeof t&&(t={raw:this._getPatternByFormat(t)||t}),s.default.parseDate(e,t))}},_isAcceptableFormat:function(e){if(e.parser)return!0;for(var t=0;t<g.length;t++)if(Object.prototype.hasOwnProperty.call(e,g[t]))return!0},firstDayOfWeekIndex:function(){var e=s.default.locale().supplemental.weekData.firstDay();return(0,c.inArray)(e,this._getDayKeys())},_getDayKeys:function(){var e=s.default.locale().main("dates/calendars/gregorian/days/format/short");return d.map(e,function(e,t){return t})}};l.default.resetInjection(),l.default.inject(y)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";
return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var a=i(n(241));n(124),n(190),n(107),n(72);var r=i(n(72)),s=i(n(28)),l=i(n(45)),u={main:{en:{identity:{version:{_cldrVersion:"28",_number:"$Revision: 11972 $"},language:"en"},numbers:{currencies:{USD:{displayName:"US Dollar","displayName-count-one":"US dollar","displayName-count-other":"US dollars",symbol:"$","symbol-alt-narrow":"$"}}}}}},c={supplemental:{version:{_cldrVersion:"28",_unicodeVersion:"8.0.0",_number:"$Revision: 11969 $"},currencyData:{fractions:{DEFAULT:{_rounding:"0",_digits:"2"}}}}};if(r.default&&r.default.formatCurrency){"en"===r.default.locale().locale&&(r.default.load(u,c),r.default.locale("en"));var d={},h=function(e,t){var n,i;return i="object"===o(t)?r.default.locale().locale+":"+e+":"+JSON.stringify(t):r.default.locale().locale+":"+e+":"+t,n=d[i],n||(n=d[i]=r.default.currencyFormatter(e,t)),n},f={_formatNumberCore:function(e,t,n){if("currency"===t){var i=n&&n.currency||(0,s.default)().defaultCurrency;return h(i,this._normalizeFormatConfig(t,n,e))(e)}return this.callBase.apply(this,arguments)},_normalizeFormatConfig:function(e,t,n){var i=this.callBase(e,t,n);return"currency"===e&&(i.style="accounting"),i},format:function(e,t){if("number"!=typeof e)return e;if(t=this._normalizeFormat(t)){if("default"===t.currency&&(t.currency=(0,s.default)().defaultCurrency),"currency"===t.type)return this._formatNumber(e,this._parseNumberFormatString("currency"),t);if(!t.type&&t.currency)return h(t.currency,t)(e)}return this.callBase.apply(this,arguments)},getCurrencySymbol:function(e){return e||(e=(0,s.default)().defaultCurrency),r.default.cldr.main("numbers/currencies/"+e)},getOpenXmlCurrencyFormat:function(e){var t=this.getCurrencySymbol(e).symbol,n=r.default.cldr.main("numbers/currencyFormats-numberSystem-latn").accounting;return(0,a.default)(t,n)}};l.default.inject(f)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(5)),a=i(n(11)),r=n(7),s=i(n(15)),l=i(n(34)),u=i(n(81)),c=(0,r.getWindow)(),d=s.default.abstract,h=s.default.inherit({ctor:function(e,t){this._store=e,this._dirty=!!t.data,this.save();var n=this._immediate=t.immediate,i=Math.max(100,t.flushInterval||1e4);if(!n){var r=this.save.bind(this);setInterval(r,i),o.default.on(c,"beforeunload",r),c.cordova&&a.default.listen(a.default.getDocument(),"pause",r,!1)}},notifyChanged:function(){this._dirty=!0,this._immediate&&this.save()},load:function(){this._store._array=this._loadImpl(),this._dirty=!1},save:function(){this._dirty&&(this._saveImpl(this._store._array),this._dirty=!1)},_loadImpl:d,_saveImpl:d}),f=h.inherit({ctor:function(e,t){var n=t.name;if(!n)throw l.default.errors.Error("E4013");this._key="dx-data-localStore-"+n,this.callBase(e,t)},_loadImpl:function(){var e=c.localStorage.getItem(this._key);return e?JSON.parse(e):[]},_saveImpl:function(e){e.length?c.localStorage.setItem(this._key,JSON.stringify(e)):c.localStorage.removeItem(this._key)}}),p={dom:f},g=u.default.inherit({ctor:function(e){e="string"==typeof e?{name:e}:e||{},this.callBase(e),this._backend=new p[e.backend||"dom"](this,e),this._backend.load()},clear:function(){this.callBase(),this._backend.notifyChanged()},_insertImpl:function(e){var t=this._backend;return this.callBase(e).done(t.notifyChanged.bind(t))},_updateImpl:function(e,t){var n=this._backend;return this.callBase(e,t).done(n.notifyChanged.bind(n))},_removeImpl:function(e){var t=this._backend;return this.callBase(e).done(t.notifyChanged.bind(t))}},"local"),_=g;t.default=_,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(90),o=i.applyChanges;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return/^(localhost$|127\.)/i.test(e)}t.default=void 0;var a,r,s=i(n(22)),l=n(7),u=i(n(264)),c=(0,l.getWindow)(),d=function(e){this.config=e,a="ms-appx:"===c.location.protocol,r=o(c.location.hostname)};d.prototype={urlFor:function(e){var t=this.config[e];if(!t)throw s.default.Error("E0006");return u.default.isProxyUsed()?u.default.formatProxyUrl(t.local):t.production&&(a&&!Debug.debuggerEnabled||!a&&!r)?t.production:t.local}};var h=d;t.default=h,e.exports=t.default},function(e,t,n){n(263),DevExpress.data.ODataStore=n(265),DevExpress.data.ODataContext=n(414),DevExpress.data.utils=DevExpress.data.utils||{},DevExpress.data.utils.odata={},DevExpress.data.utils.odata.keyConverters=n(95).keyConverters,DevExpress.data.EdmLiteral=n(95).EdmLiteral;var i=n(95);DevExpress.data.utils.odata.serializePropName=i.serializePropName,DevExpress.data.utils.odata.serializeValue=i.serializeValue,DevExpress.data.utils.odata.serializeKey=i.serializeKey,DevExpress.data.utils.odata.sendRequest=i.sendRequest,DevExpress.data.queryAdapters=DevExpress.data.queryAdapters||{},DevExpress.data.queryAdapters.odata=n(159).odata},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=n(0),r=n(1),s=n(3),l=i(n(34)),u=i(n(265)),c=i(n(266)),d=n(95),h=n(6);n(159);var f=o.default.inherit({ctor:function(e){var t=this;this._requestDispatcher=new c.default(e),this._errorHandler=e.errorHandler,(0,s.each)(e.entities||[],function(n,i){t[n]=new u.default((0,a.extend)({},e,{url:"".concat(t._requestDispatcher.url,"/").concat(encodeURIComponent(i.name||n))},i))})},get:function(e,t){return this.invoke(e,t,"GET")},invoke:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"POST";n=n.toLowerCase();var i,o=new h.Deferred,a="".concat(this._requestDispatcher.url,"/").concat(encodeURIComponent(e));return 4===this.version()&&("get"===n?(a=(0,d.formatFunctionInvocationUrl)(a,(0,d.escapeServiceOperationParams)(t,this.version())),t=null):"post"===n&&(i=t,t=null)),(0,h.when)(this._requestDispatcher.sendRequest(a,n,(0,d.escapeServiceOperationParams)(t,this.version()),i)).done(function(t){(0,r.isPlainObject)(t)&&e in t&&(t=t[e]),o.resolve(t)}).fail(this._errorHandler).fail(l.default._errorHandler).fail(o.reject),o.promise()},objectLink:function(e,t){var n=this[e];if(!n)throw l.default.errors.Error("E4015",e);return(0,r.isDefined)(t)?{__metadata:{uri:n._byKeyUrl(t,!0)}}:null},version:function(){return this._requestDispatcher.version}}),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function g(){if(!D.default)throw S.default.Error("E1041","JSZip");return D.default}t.default=void 0;var _=n(12),m=n(4),v=n(19),y=o(n(35)),b=n(1),x=o(n(34)),w=n(6),C=n(7),k=n(177),S=o(n(17)),D=o(n(267)),I=o(n(211)),T=o(n(279)),O=n(278),E=(0,C.getWindow)(),A=function(e){function t(e){var i;a(this,t),e=(0,m.ensureDefined)(e,{}),i=n.call(this,e);var o=e.data;if(o&&!Array.isArray(o))throw x.default.errors.Error("E4006");var r=e.itemsExpr||"items";i._subFileItemsGetter=(0,v.compileGetter)(r),i._subFileItemsSetter=i._getSetter(r);var s=e.contentExpr||"content";i._contentGetter=(0,v.compileGetter)(s),i._contentSetter=i._getSetter(s);var l=i._getNameExpr(e);i._nameSetter=i._getSetter(l);var u=i._getIsDirExpr(e);i._getIsDirSetter=i._getSetter(u);var c=i._getKeyExpr(e);i._keySetter=i._getSetter(c);var d=i._getSizeExpr(e);i._sizeSetter=i._getSetter(d);var h=i._getDateModifiedExpr(e);return i._dateModifiedSetter=i._getSetter(h),i._data=o||[],i}l(t,e);var n=c(t);return s(t,[{key:"getItems",value:function(e){var t=this;return this._executeActionAsDeferred(function(){return t._getItems(e)},!0)}},{key:"renameItem",value:function(e,t){var n=this;return this._executeActionAsDeferred(function(){return n._renameItemCore(e,t)})}},{key:"_renameItemCore",value:function(e,t){if(e){var n=this._findDataObject(e);this._nameSetter(n,t),e.name=t,e.key=this._ensureDataObjectKey(n)}}},{key:"createDirectory",value:function(e,t){var n=this;return this._executeActionAsDeferred(function(){n._validateDirectoryExists(e),n._createDataObject(e,t,!0)})}},{key:"deleteItems",value:function(e){var t=this;return e.map(function(e){return t._executeActionAsDeferred(function(){return t._deleteItem(e)})})}},{key:"moveItems",value:function(e,t){var n=this,i=this._findDataObject(t),o=this._getDirectoryDataItems(i),a=e.map(function(e){return n._executeActionAsDeferred(function(){n._checkAbilityToMoveOrCopyItem(e,t);var i=n._findDataObject(e);n._deleteItem(e),o.push(i)})});return this._updateHasSubDirs(t),a}},{key:"copyItems",value:function(e,t){var n=this,i=this._findDataObject(t),o=this._getDirectoryDataItems(i),a=e.map(function(e){return n._executeActionAsDeferred(function(){n._checkAbilityToMoveOrCopyItem(e,t);var i=n._findDataObject(e),a=n._createCopy(i);o.push(a)})});return this._updateHasSubDirs(t),a}},{key:"uploadFileChunk",value:function(e,t,n){var i=this;if(t.chunkIndex>0)return t.customData.deferred;this._validateDirectoryExists(n);var o=t.customData.deferred=new w.Deferred,a=this._createFileReader();return a.readAsDataURL(e),a.onload=function(){var t=a.result.split(",")[1],r=i._createDataObject(n,e.name,!1);i._sizeSetter(r,e.size),i._dateModifiedSetter(r,e.lastModifiedDate),i._contentSetter(r,t),o.resolve()},a.onerror=function(e){return o.reject(e)},o}},{key:"downloadItems",value:function(e){1===e.length?this._downloadSingleFile(e[0]):this._downloadMultipleFiles(e)}},{key:"_downloadSingleFile",value:function(e){for(var t=this._getFileContent(e),n=E.atob(t),i=new ArrayBuffer(n.length),o=new Uint8Array(i),a=0;a<n.length;a++)o[a]=n.charCodeAt(a);var r=new E.Blob([i],{type:"application/octet-stream"});k.fileSaver.saveAs(e.name,null,r)}},{key:"_downloadMultipleFiles",value:function(e){var t=this,n=g(),i=new n;e.forEach(function(e){return i.file(e.name,t._getFileContent(e),{base64:!0})});var o={type:"blob",compression:"DEFLATE",mimeType:"application/zip"},a=new w.Deferred;i.generateAsync?i.generateAsync(o).then(a.resolve):a.resolve(i.generate(o)),a.done(function(e){return k.fileSaver.saveAs("files.zip",null,e)})}},{key:"_getFileContent",value:function(e){var t=this._findDataObject(e);return this._contentGetter(t)||""}},{key:"_validateDirectoryExists",value:function(e){if(!this._isFileItemExists(e)||this._isDirGetter(e.fileItem))throw{errorId:T.default.DirectoryNotFound,fileItem:e}}},{key:"_checkAbilityToMoveOrCopyItem",value:function(e,t){var n=this,i=this._findDataObject(e),o=this._getKeyFromDataObject(i,e.parentPath),a=t.getFullPathInfo(),r="";a.forEach(function(t){r=(0,O.pathCombine)(r,t.name);var i=n._getDataObjectKey(t.key,r);if(i===o)throw{errorId:T.default.Other,fileItem:e}})}},{key:"_createDataObject",value:function(e,t,n){var i={};this._nameSetter(i,t),this._getIsDirSetter(i,n),this._keySetter(i,String(new y.default));var o=this._findDataObject(e),a=this._getDirectoryDataItems(o);return a.push(i),n&&this._updateHasSubDirs(e),i}},{key:"_createCopy",value:function(e){var t=this,n={};this._nameSetter(n,this._nameGetter(e)),this._getIsDirSetter(n,this._isDirGetter(e));var i=this._subFileItemsGetter(e);if(Array.isArray(i)){var o=[];i.forEach(function(e){var n=t._createCopy(e);o.push(n)}),this._subFileItemsSetter(n,o)}return n}},{key:"_deleteItem",value:function(e){var t=this._findDataObject(e),n=this._findFileItemObj(e.pathInfo),i=this._getDirectoryDataItems(n),o=i.indexOf(t);i.splice(o,1)}},{key:"_getDirectoryDataItems",value:function(e){if(!e)return this._data;var t=this._subFileItemsGetter(e);return Array.isArray(t)||(t=[],this._subFileItemsSetter(e,t)),t}},{key:"_getItems",value:function(e){this._validateDirectoryExists(e);var t=e.getFullPathInfo(),n=t&&t.length>0?t[t.length-1].key:null,i=this._data;if(n){var o=this._findFileItemObj(t);i=o&&this._subFileItemsGetter(o)||[]}return this._ensureKeysForDuplicateNameItems(i),this._convertDataObjectsToFileItems(i,t)}},{key:"_ensureKeysForDuplicateNameItems",value:function(e){var t=this,n={};e.forEach(function(e){var i=t._nameGetter(e);n[i]?t._ensureDataObjectKey(e):n[i]=!0})}},{key:"_findDataObject",value:function(e){if(e.isRoot())return null;var t=this._findFileItemObj(e.getFullPathInfo());if(!t)throw{errorId:e.isDirectory?T.default.DirectoryNotFound:T.default.FileNotFound,fileItem:e};return t}},{key:"_findFileItemObj",value:function(e){var t=this;Array.isArray(e)||(e=[]);for(var n="",i=null,o=this._data,a=function(a){i=(0,_.find)(o,function(i){var o=t._isDirGetter(i)||a===e.length-1;return t._getKeyFromDataObject(i,n)===e[a].key&&t._nameGetter(i)===e[a].name&&o}),i&&(n=(0,O.pathCombine)(n,t._nameGetter(i)),o=t._subFileItemsGetter(i))},r=0;r<e.length&&(0===r||i);r++)a(r);return i}},{key:"_getKeyFromDataObject",value:function(e,t){var n=this._keyGetter(e),i=(0,O.pathCombine)(t,this._nameGetter(e));return this._getDataObjectKey(n,i)}},{key:"_getDataObjectKey",value:function(e,t){return e?e:t}},{key:"_ensureDataObjectKey",value:function(e){var t=this._keyGetter(e);return t||(t=String(new y.default),this._keySetter(e,t)),t}},{key:"_updateHasSubDirs",value:function(e){if(e&&!e.isRoot()){var t=this._findDataObject(e);e.hasSubDirectories=this._hasSubDirs(t)}}},{key:"_hasSubDirs",value:function(e){var t=(0,m.ensureDefined)(this._subFileItemsGetter(e),[]);if(!Array.isArray(t))return!0;for(var n=0;n<t.length;n++)if(this._isDirGetter(t[n])===!0)return!0;return!1}},{key:"_getSetter",value:function(e){return(0,b.isFunction)(e)?e:(0,v.compileSetter)(e)}},{key:"_isFileItemExists",value:function(e){return e.isDirectory&&e.isRoot()||!!this._findFileItemObj(e.getFullPathInfo())}},{key:"_createFileReader",value:function(){return new E.FileReader}}]),t}(I.default),P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(2)),_=o(n(76)),m=n(4),v=o(n(35)),y=n(7),b=n(3),x=n(6),w=o(n(5)),C=o(n(211)),k=n(19),S=(0,y.getWindow)(),D="chunk",I=function(e){function t(e){var i;return a(this,t),e=(0,m.ensureDefined)(e,{}),i=n.call(this,e),i._endpointUrl=e.endpointUrl,i._hasSubDirsGetter=(0,k.compileGetter)(e.hasSubDirectoriesExpr||"hasSubDirectories"),i}l(t,e);var n=c(t);return s(t,[{key:"getItems",value:function(e){var t=this,n=e.getFullPathInfo();return this._getEntriesByPath(n).then(function(e){return t._convertDataObjectsToFileItems(e.result,n)})}},{key:"renameItem",value:function(e,t){return this._executeRequest("Rename",{pathInfo:e.getFullPathInfo(),isDirectory:e.isDirectory,name:t})}},{key:"createDirectory",value:function(e,t){return this._executeRequest("CreateDir",{pathInfo:e.getFullPathInfo(),name:t}).done(function(){e&&!e.isRoot()&&(e.hasSubDirectories=!0)})}},{key:"deleteItems",value:function(e){var t=this;return e.map(function(e){return t._executeRequest("Remove",{pathInfo:e.getFullPathInfo(),isDirectory:e.isDirectory})})}},{key:"moveItems",value:function(e,t){var n=this;return e.map(function(e){return n._executeRequest("Move",{sourcePathInfo:e.getFullPathInfo(),sourceIsDirectory:e.isDirectory,destinationPathInfo:t.getFullPathInfo()})})}},{key:"copyItems",value:function(e,t){var n=this;return e.map(function(e){return n._executeRequest("Copy",{sourcePathInfo:e.getFullPathInfo(),sourceIsDirectory:e.isDirectory,destinationPathInfo:t.getFullPathInfo()})})}},{key:"uploadFileChunk",value:function(e,t,n){0===t.chunkIndex&&(t.customData.uploadId=new v.default);var i={destinationPathInfo:n.getFullPathInfo(),chunkMetadata:JSON.stringify({UploadId:t.customData.uploadId,FileName:e.name,Index:t.chunkIndex,TotalCount:t.chunkCount,FileSize:e.size})},o=new S.FormData;o.append(D,t.chunkBlob),o.append("arguments",JSON.stringify(i)),o.append("command","UploadChunk");var a=new x.Deferred;return _.default.sendRequest({url:this._endpointUrl,method:"POST",dataType:"json",data:o,upload:{onprogress:m.noop,onloadstart:m.noop,onabort:m.noop},cache:!1}).done(function(e){!e.success&&a.reject(e)||a.resolve()}).fail(a.reject),a.promise()}},{key:"abortFileUpload",value:function(e,t,n){return this._executeRequest("AbortUpload",{uploadId:t.customData.uploadId})}},{key:"downloadItems",value:function(e){var t=this._getDownloadArgs(e),n=(0,g.default)("<form>").css({display:"none"}).attr({method:"post",action:t.url});["command","arguments"].forEach(function(e){(0,g.default)("<input>").attr({type:"hidden",name:e,value:t[e]}).appendTo(n)}),n.appendTo("body"),w.default.trigger(n,"submit"),setTimeout(function(){return n.remove()})}},{key:"getItemsContent",value:function(e){var t=this._getDownloadArgs(e),n=new S.FormData;return n.append("command",t.command),n.append("arguments",t.arguments),_.default.sendRequest({url:t.url,method:"POST",responseType:"arraybuffer",data:n,upload:{onprogress:m.noop,onloadstart:m.noop,onabort:m.noop},cache:!1})}},{key:"_getDownloadArgs",value:function(e){var t=e.map(function(e){return e.getFullPathInfo()}),n={pathInfoList:t},i=JSON.stringify(n);return{url:this._endpointUrl,arguments:i,command:"Download"}}},{key:"_getItemsIds",value:function(e){return e.map(function(e){return e.relativeName})}},{key:"_getEntriesByPath",value:function(e){return this._executeRequest("GetDirContents",{pathInfo:e})}},{key:"_executeRequest",value:function(e,t){var n="GetDirContents"===e?"GET":"POST",i=new x.Deferred;return _.default.sendRequest({url:this._getEndpointUrl(e,t),method:n,dataType:"json",cache:!1}).then(function(e){!e.success&&i.reject(e)||i.resolve(e)},function(e){return i.reject(e)}),i.promise()}},{key:"_getEndpointUrl",value:function(e,t){var n=this._getQueryString({command:e,arguments:JSON.stringify(t)}),i=this._endpointUrl&&this._endpointUrl.indexOf("?")>0?"&":"?";return this._endpointUrl+i+n}},{key:"_getQueryString",value:function(e){for(var t=[],n=Object.keys(e),i=0;i<n.length;i++){var o=n[i],a=e[o];if(void 0!==a)if(null===a&&(a=""),Array.isArray(a))this._processQueryStringArrayParam(o,a,t);else{var r=this._getQueryStringPair(o,a);t.push(r)}}return t.join("&")}},{key:"_processQueryStringArrayParam",value:function(e,t,n){var i=this;(0,b.each)(t,function(t,o){var a=i._getQueryStringPair(e,o);n.push(a)})}},{key:"_getQueryStringPair",value:function(e,t){return encodeURIComponent(e)+"="+encodeURIComponent(t)}},{key:"_hasSubDirs",value:function(e){var t=this._hasSubDirsGetter(e);return"boolean"!=typeof t||t}},{key:"_getKeyExpr",value:function(e){return e.keyExpr||"key"}}]),t}(C.default),T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=n(4),_=n(1),m=n(19),v=o(n(211)),y=function(e){function t(e){var i;return a(this,t),e=(0,g.ensureDefined)(e,{}),i=n.call(this,e),i._hasSubDirsGetter=(0,m.compileGetter)(e.hasSubDirectoriesExpr||"hasSubDirectories"),i._getItemsFunction=i._ensureFunction(e.getItems,function(){return[]}),i._renameItemFunction=i._ensureFunction(e.renameItem),i._createDirectoryFunction=i._ensureFunction(e.createDirectory),i._deleteItemFunction=i._ensureFunction(e.deleteItem),i._moveItemFunction=i._ensureFunction(e.moveItem),i._copyItemFunction=i._ensureFunction(e.copyItem),i._uploadFileChunkFunction=i._ensureFunction(e.uploadFileChunk),i._abortFileUploadFunction=i._ensureFunction(e.abortFileUpload),i._downloadItemsFunction=i._ensureFunction(e.downloadItems),i._getItemsContentFunction=i._ensureFunction(e.getItemsContent),i}l(t,e);var n=c(t);return s(t,[{key:"getItems",value:function(e){var t=this,n=e.getFullPathInfo();return this._executeActionAsDeferred(function(){return t._getItemsFunction(e)},!0).then(function(e){return t._convertDataObjectsToFileItems(e,n)})}},{key:"renameItem",value:function(e,t){var n=this;return this._executeActionAsDeferred(function(){return n._renameItemFunction(e,t)})}},{key:"createDirectory",value:function(e,t){var n=this;return this._executeActionAsDeferred(function(){return n._createDirectoryFunction(e,t)})}},{key:"deleteItems",value:function(e){var t=this;return e.map(function(e){return t._executeActionAsDeferred(function(){return t._deleteItemFunction(e)})})}},{key:"moveItems",value:function(e,t){var n=this;return e.map(function(e){return n._executeActionAsDeferred(function(){return n._moveItemFunction(e,t)})})}},{key:"copyItems",value:function(e,t){var n=this;return e.map(function(e){return n._executeActionAsDeferred(function(){return n._copyItemFunction(e,t)})})}},{key:"uploadFileChunk",value:function(e,t,n){var i=this;return this._executeActionAsDeferred(function(){return i._uploadFileChunkFunction(e,t,n)})}},{key:"abortFileUpload",value:function(e,t,n){var i=this;return this._executeActionAsDeferred(function(){return i._abortFileUploadFunction(e,t,n)})}},{key:"downloadItems",value:function(e){return this._downloadItemsFunction(e)}},{key:"getItemsContent",value:function(e){var t=this;return this._executeActionAsDeferred(function(){return t._getItemsContentFunction(e)})}},{key:"_hasSubDirs",value:function(e){var t=this._hasSubDirsGetter(e);return"boolean"!=typeof t||t}},{key:"_getKeyExpr",value:function(e){return e.keyExpr||"key"}},{key:"_ensureFunction",value:function(e,t){return t=t||g.noop,(0,_.isFunction)(e)?e:t}}]),t}(v.default),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(36),r=n(4),s=n(1),l=i(n(9)),u=n(12),c=n(0),d=n(3),h=n(24),f=i(n(579)),p=i(n(77)),g=n(71),_=i(n(22)),m=i(n(43)),v="dx-toolbar",y="dx-toolbar-before",b="dx-toolbar-center",x="dx-toolbar-after",w="dx-toolbar-mini",C="dx-toolbar-item",k="dx-toolbar-label",S="dx-toolbar-button",D="dx-toolbar-items-container",I="dx-toolbar-group",T="dx-toolbar-compact",O="."+k,E="dx-toolbar-multiline",A="text",P="default",R="dxToolbarItemDataKey",M=f.default.inherit({compactMode:!1,ctor:function(e,t){this._userOptions=t||{},this.callBase(e,t),"height"in this._userOptions&&_.default.log("W0001",this.NAME,"height","20.1","Functionality associated with this option is not intended for the Toolbar widget.")},_getSynchronizableOptionsForCreateComponent:function(){return this.callBase().filter(function(e){return"disabled"!==e})},_initTemplates:function(){this.callBase();var e=new g.BindableTemplate(function(e,t,n){(0,s.isPlainObject)(t)?(t.text&&e.text(t.text).wrapInner("<div>"),t.html&&e.html(t.html),"dxButton"===t.widget&&(this.option("useFlatButtons")&&(t.options=t.options||{},t.options.stylingMode=t.options.stylingMode||A),this.option("useDefaultButtons")&&(t.options=t.options||{},t.options.type=t.options.type||P))):e.text(String(t)),this._getTemplate("dx-polymorph-widget").render({container:e,model:n,parent:this})}.bind(this),["text","html","widget","options"],this.option("integrationOptions.watchMethod"));this._templateManager.addDefaultTemplates({item:e,menuItem:e})},_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{renderAs:"topToolbar",grouped:!1,useFlatButtons:!1,useDefaultButtons:!1,multiline:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return(0,a.isMaterial)()},options:{useFlatButtons:!0}}])},_itemContainer:function(){return this._$toolbarItemsContainer.find(["."+y,"."+b,"."+x].join(","))},_itemClass:function(){return C},_itemDataKey:function(){return R},_buttonClass:function(){return S},_dimensionChanged:function(){this._arrangeItems(),this._applyCompactMode()},_initMarkup:function(){this._renderToolbar(),this._renderSections(),this.callBase(),this.setAria("role","toolbar")},_waitParentAnimationFinished:function(){var e=this,t=this.$element(),n=15;return new p.default(function(i){var a=function(){var e=!0;return t.parents().each(function(t,n){if(m.default.isAnimating((0,o.default)(n)))return e=!1,!1}),e&&i(),e},r=function t(){clearTimeout(e._waitParentAnimationTimeout),e._waitParentAnimationTimeout=setTimeout(function(){return a()||t()},n)};r()})},_render:function(){this.callBase(),this._renderItemsAsync(),(0,a.isMaterial)()&&p.default.all([this._waitParentAnimationFinished(),this._checkWebFontForLabelsLoaded()]).then(this._dimensionChanged.bind(this))},_postProcessRenderItems:function(){this._arrangeItems()},_renderToolbar:function(){this.$element().addClass(v).toggleClass(E,this.option("multiline")),this._$toolbarItemsContainer=(0,o.default)("<div>").addClass(D).appendTo(this.$element())},_renderSections:function(){var e=this._$toolbarItemsContainer,t=this;(0,d.each)(["before","center","after"],function(){var n="dx-toolbar-"+this,i=e.find("."+n);i.length||(t["_$"+this+"Section"]=i=(0,o.default)("<div>").addClass(n).appendTo(e))})},_checkWebFontForLabelsLoaded:function(){var e=this.$element().find(O),t=[];return e.each(function(e,n){var i=(0,o.default)(n).text(),r=(0,o.default)(n).css("fontWeight");t.push((0,a.waitWebFont)(i,r))}),p.default.all(t)},_arrangeItems:function(e){e=e||this.$element().width(),this._$centerSection.css({margin:"0 auto","float":"none"});var t=(0,h.getBoundingRect)(this._$beforeSection.get(0)),n=(0,h.getBoundingRect)(this._$afterSection.get(0));this._alignCenterSection(t,n,e);var i=this._$toolbarItemsContainer.find(O).eq(0),a=i.parent();if(i.length){var r=t.width?t.width:i.position().left,s=a.hasClass(y)?0:r,l=a.hasClass(x)?0:n.width,u=0;a.children().not(O).each(function(){u+=(0,o.default)(this).outerWidth()});var c=e-u,d=Math.max(c-s-l,0);if(a.hasClass(y))this._alignSection(this._$beforeSection,d);else{var f=i.outerWidth()-i.width();i.css("maxWidth",d-f)}}},_alignCenterSection:function(e,t,n){this._alignSection(this._$centerSection,n-e.width-t.width);var i=this.option("rtlEnabled"),o=i?t:e,a=i?e:t,r=(0,h.getBoundingRect)(this._$centerSection.get(0));(o.right>r.left||r.right>a.left)&&this._$centerSection.css({marginLeft:o.width,marginRight:a.width,"float":o.width>a.width?"none":"right"})},_alignSection:function(e,t){var n=e.find(O),i=n.toArray();t-=this._getCurrentLabelsPaddings(i);var o=this._getCurrentLabelsWidth(i),a=Math.abs(o-t);t<o?(i=i.reverse(),this._alignSectionLabels(i,a,!1)):this._alignSectionLabels(i,a,!0)},_alignSectionLabels:function(e,t,n){for(var i=function(e){return(0,h.getBoundingRect)(e).width},a=0;a<e.length;a++){var r=(0,o.default)(e[a]),s=Math.ceil(i(e[a])),l=void 0;n&&r.css("maxWidth","inherit");var u=Math.ceil(n?i(e[a]):s);if(!(u<t)){l=n?s+t:s-t,r.css("maxWidth",l);break}l=n?u:0,t-=u,r.css("maxWidth",l)}},_applyCompactMode:function(){var e=this.$element();e.removeClass(T),this.option("compactMode")&&this._getSummaryItemsWidth(this.itemElements(),!0)>e.width()&&e.addClass(T)},_getCurrentLabelsWidth:function(e){var t=0;return e.forEach(function(e,n){t+=(0,o.default)(e).outerWidth()}),t},_getCurrentLabelsPaddings:function(e){var t=0;return e.forEach(function(e,n){t+=(0,o.default)(e).outerWidth()-(0,o.default)(e).width()}),t},_renderItem:function(e,t,n,i){var o=t.location||"center",a=n||this["_$"+o+"Section"],r=!(!t.text&&!t.html),s=this.callBase(e,t,a,i);return s.toggleClass(this._buttonClass(),!r).toggleClass(k,r).addClass(t.cssClass),s},_renderGroupedItems:function(){var e=this;(0,d.each)(this.option("items"),function(t,n){var i=n.items,a=(0,o.default)("<div>").addClass(I),r=n.location||"center";i&&i.length&&((0,d.each)(i,function(t,n){e._renderItem(t,n,a,null)}),e._$toolbarItemsContainer.find(".dx-toolbar-"+r).append(a))})},_renderItems:function(e){var t=this.option("grouped")&&e.length&&e[0].items;
t?this._renderGroupedItems():this.callBase(e)},_getToolbarItems:function(){return this.option("items")||[]},_renderContentImpl:function(){var e=this._getToolbarItems();this.$element().toggleClass(w,0===e.length),this._renderedItemsCount?this._renderItems(e.slice(this._renderedItemsCount)):this._renderItems(e),this._applyCompactMode()},_renderEmptyMessage:r.noop,_clean:function(){this._$toolbarItemsContainer.children().empty(),this.$element().empty()},_visibilityChanged:function(e){e&&this._arrangeItems()},_isVisible:function(){return this.$element().width()>0&&this.$element().height()>0},_getIndexByItem:function(e){return(0,u.inArray)(e,this._getToolbarItems())},_itemOptionChanged:function(e,t,n){this.callBase.apply(this,[e,t,n]),this._arrangeItems()},_optionChanged:function(e){var t=e.name;switch(t){case"width":this.callBase.apply(this,arguments),this._dimensionChanged();break;case"multiline":this.$element().toggleClass(E,e.value);break;case"renderAs":case"useFlatButtons":case"useDefaultButtons":this._invalidate();break;case"compactMode":this._applyCompactMode();break;case"grouped":break;default:this.callBase.apply(this,arguments)}},_dispose:function(){this.callBase(),clearTimeout(this._waitParentAnimationTimeout)}});(0,l.default)("dxToolbarBase",M);var B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(108)),r=n(78),s=n(0),l=n(1),u=i(n(420)),c=null,d=function(e,t,n){var i=(0,l.isPlainObject)(e)?e:{message:e},d=i.onHidden;(0,s.extend)(i,{type:t,displayTime:n,onHidden:function(e){(0,o.default)(e.element).remove(),new a.default(d,{context:e.model}).execute(arguments)}}),c=(0,o.default)("<div>").appendTo((0,r.value)()),new u.default(c,i).show()},h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(11)),s=i(n(5)),l=i(n(49)),u=n(4),c=n(1),d=n(0),h=n(12),f=i(n(25)),p=i(n(9)),g=i(n(68)),_=n(36),m=(0,a.getWindow)(),v=l.default.add,y="dx-toast",b=y+"-",x=b+"wrapper",w=b+"content",C=b+"message",k=b+"icon",S="dxToast",D=["info","warning","error","success"],I=[],T=8e3,O=null,E={top:{my:"top",at:"top",of:null,offset:"0 0"},bottom:{my:"bottom",at:"bottom",of:null,offset:"0 -20"},center:{my:"center",at:"center",of:null,offset:"0 0"},right:{my:"center right",at:"center right",of:null,offset:"0 0"},left:{my:"center left",at:"center left",of:null,offset:"0 0"}};v(function(){s.default.subscribeGlobal(r.default.getDocument(),f.default.down,function(e){for(var t=I.length-1;t>=0;t--)if(!I[t]._proxiedDocumentDownHandler(e))return})});var A=g.default.inherit({_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{message:"",type:"info",displayTime:2e3,position:"bottom center",animation:{show:{type:"fade",duration:400,from:0,to:1},hide:{type:"fade",duration:400,to:0}},shading:!1,height:"auto",hideTopOverlayHandler:null,closeOnSwipe:!0,closeOnClick:!1,resizeEnabled:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"android"},options:{closeOnOutsideClick:!0,width:"auto",position:{at:"bottom left",my:"bottom left",offset:"20 -20"},animation:{show:{type:"slide",duration:200,from:{position:{my:"top",at:"bottom",of:m}}},hide:{type:"slide",duration:200,to:{position:{my:"top",at:"bottom",of:m}}}}}},{device:function(e){var t="phone"===e.deviceType,n="android"===e.platform;return t&&n},options:{width:function(){var e;return(null===m||void 0===m?void 0:null===(e=m.visualViewport)||void 0===e?void 0:e.width)||(0,o.default)(m).width()},position:{at:"bottom center",my:"bottom center",offset:"0 0"}}},{device:function(e){return"phone"===e.deviceType},options:{width:function(){var e;return(null===m||void 0===m?void 0:null===(e=m.visualViewport)||void 0===e?void 0:e.width)||(0,o.default)(m).width()}}},{device:function(){return(0,_.isMaterial)()},options:{minWidth:344,maxWidth:568,displayTime:4e3}}])},_init:function(){this.callBase(),this._posStringToObject()},_renderContentImpl:function(){this.option("message")&&(this._message=(0,o.default)("<div>").addClass(C).text(this.option("message")).appendTo(this.$content())),this.setAria("role","alert",this._message),(0,h.inArray)(this.option("type").toLowerCase(),D)>-1&&this.$content().prepend((0,o.default)("<div>").addClass(k)),this.callBase()},_render:function(){this.callBase(),this.$element().addClass(y),this._wrapper().addClass(x),this._$content.addClass(b+String(this.option("type")).toLowerCase()),this.$content().addClass(w),this._toggleCloseEvents("Swipe"),this._toggleCloseEvents("Click")},_renderScrollTerminator:u.noop,_toggleCloseEvents:function(e){var t="dx"+e.toLowerCase();s.default.off(this._$content,t),this.option("closeOn"+e)&&s.default.on(this._$content,t,this.hide.bind(this))},_posStringToObject:function(){if((0,c.isString)(this.option("position"))){var e=this.option("position").split(" ")[0],t=this.option("position").split(" ")[1];switch(this.option("position",(0,d.extend)({},E[e])),t){case"center":case"left":case"right":this.option("position").at+=" "+t,this.option("position").my+=" "+t}}},_show:function(){return O&&O!==this&&(clearTimeout(O._hideTimeout),O.hide()),O=this,this.callBase.apply(this,arguments).done(function(){clearTimeout(this._hideTimeout),this._hideTimeout=setTimeout(this.hide.bind(this),this.option("displayTime"))}.bind(this))},_hide:function(){return O=null,this.callBase.apply(this,arguments)},_overlayStack:function(){return I},_zIndexInitValue:function(){return this.callBase()+T},_dispose:function(){clearTimeout(this._hideTimeout),O=null,this.callBase()},_optionChanged:function(e){switch(e.name){case"type":this._$content.removeClass(b+e.previousValue),this._$content.addClass(b+String(e.value).toLowerCase());break;case"message":this._message&&this._message.text(e.value);break;case"closeOnSwipe":this._toggleCloseEvents("Swipe");break;case"closeOnClick":this._toggleCloseEvents("Click");break;case"displayTime":case"position":break;default:this.callBase(e)}}});(0,p.default)(S,A);var P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function m(e){e._options.silent("onInitializing",null);var t=!1;if(R){var n=R.option("actions");if(n.forEach(function(n){if(n._options.silent("id")===e._options.silent("id"))return t=!0,e}),delete R._options.silent("position"),t)1===n.length?R.option((0,w.extend)({},M(n[0]),{actions:n,position:R._getPosition()})):R.option((0,w.extend)(R._getCurrentOptions(n),{actions:n}));else{if(R._getVisibleActions(n).length>=R.option("maxSpeedDialActionCount"))return e.dispose(),void k.default.log("W1014");n.push(e),R.option((0,w.extend)(R._getCurrentOptions(n),{actions:n}))}}else{var i=(0,b.default)("<div>").appendTo(T(e.$element()));R=e._createComponent(i,B,(0,w.extend)({},M(e),{actions:[e]}))}}function v(e){if(R){var t=R.option("actions"),n=t.length;t=t.filter(function(t){return t._options.silent("id")!==e}),n!==t.length&&(t.length?1===t.length?R.option((0,w.extend)({},M(t[0]),{actions:t})):R.option({actions:t}):(R.dispose(),R.$element().remove(),R=null))}}function y(){if(R){var e=R._getVisibleActions(),t=1===e.length?e[0].option("icon"):R._getDefaultOptions().icon,n=1===e.length?e[0].option("label"):R._getDefaultOptions().label;R.option({actions:R.option("actions"),icon:t,closeIcon:R._getDefaultOptions().closeIcon,position:R._getPosition(),label:n,maxSpeedDialActionCount:R._getDefaultOptions().maxSpeedDialActionCount,direction:R._getDefaultOptions().direction})}}t.initAction=m,t.disposeAction=v,t.repaint=y;var b=o(n(2)),x=o(n(28)),w=n(0),C=o(n(5)),k=o(n(17)),S=o(n(164)),D=o(n(581)),I=n(36),T=S.default.getSwatchContainer,O="dx-fa-button-main",E="dx-fa-button-with-label",A="dx-fa-button-icon-close",P="dx-state-invisible",R=null,M=function(e){return(0,w.extend)({},e._options.silent(),{onInitialized:null,onDisposing:null})},B=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){var e={icon:"add",closeIcon:"close",position:{at:"right bottom",my:"right bottom",offset:{x:-16,y:-16}},maxSpeedDialActionCount:5,hint:"",label:"",direction:"auto",actions:[],activeStateEnabled:!0,hoverStateEnabled:!0,indent:55,childIndent:40,childOffset:9,callOverlayRenderShading:!0,closeOnOutsideClick:!0};return(0,w.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),(0,w.extend)(e,(0,x.default)().floatingActionButtonConfig,{shading:!1}))}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return(0,I.isMaterial)()},options:{indent:72,childIndent:56,childOffset:8}}])}},{key:"_render",value:function(){this.$element().addClass(O),l(_(t.prototype),"_render",this).call(this),this._moveToContainer(),this._renderCloseIcon(),this._renderClick()}},{key:"_renderLabel",value:function(){l(_(t.prototype),"_renderLabel",this).call(this),this.$element().toggleClass(E,!!this._$label)}},{key:"_renderCloseIcon",value:function(){this._$closeIcon=this._renderButtonIcon(this._$closeIcon,this._options.silent("closeIcon"),A),this._$closeIcon.addClass(P)}},{key:"_renderClick",value:function(){this._clickAction=1===this._getVisibleActions().length?this._getActionComponent()._createActionByOption("onClick"):this._createAction(this._clickHandler.bind(this)),this._setClickAction()}},{key:"_getVisibleActions",value:function(e){var t=e||this.option("actions");return t.filter(function(e){return e.option("visible")})}},{key:"_getCurrentOptions",value:function(e){var t=R._getVisibleActions(e);return 1===t.length?(0,w.extend)(t[0]._options.silent(),{position:this._getPosition()}):(0,w.extend)(this._getDefaultOptions(),{visible:0!==t.length})}},{key:"_clickHandler",value:function(){var e=this._actionItems.filter(function(e){return e.option("actionVisible")}).sort(function(e,t){return e.option("index")-t.option("index")});if(1!==e.length){for(var t=e.length-1,n=0;n<e.length;n++)e[n].option("animation",this._getActionAnimation(e[n],n,t)),e[n].option("position",this._getActionPosition(e,n)),e[n]._$wrapper.css("position",this._$wrapper.css("position")),e[n].toggle();(0,x.default)().floatingActionButtonConfig.shading&&(this._isShadingShown=!this.option("shading"),this.option("shading",this._isShadingShown)),this._$icon.toggleClass(P),this._$closeIcon.toggleClass(P)}}},{key:"_updateZIndexStackPosition",value:function(){l(_(t.prototype),"_updateZIndexStackPosition",this).call(this);var e=this._overlayStack();e.push(this)}},{key:"_renderActions",value:function(){var e=this,t=this.option("actions"),n=1;if(this._actionItems&&this._actionItems.length&&(this._actionItems.forEach(function(e){e.dispose(),e.$element().remove()}),this._actionItems=[]),this._actionItems=[],t.length!==n)for(var i=0;i<t.length;i++){var o=t[i],a=(0,b.default)("<div>").appendTo(T(o.$element()));C.default.off(a,"click"),C.default.on(a,"click",function(){e._clickHandler()}),o._options.silent("actionComponent",o),o._options.silent("parentPosition",this._getPosition()),o._options.silent("actionVisible",o._options.silent("visible")),this._actionItems.push(this._createComponent(a,D.default,(0,w.extend)({},M(o),{visible:!1})))}}},{key:"_getActionAnimation",value:function(e,t,n){var i=30;return e._options.silent("animation.show.delay",i*t),e._options.silent("animation.hide.delay",i*(n-t)),e._options.silent("animation")}},{key:"_getDirectionIndex",value:function(e,t){var n=1;if("auto"===t){var i=this.$content().height(),o=this.initialOption("indent")+this.initialOption("childIndent")*e.length-i,a=this.$content().offset().top;if(o<a)return-n;var r=this._getContainer().height()-i-a;return a>=r?-n:n}return"down"!==t?-n:n}},{key:"_getActionPosition",value:function(e,t){var n=e[t],i=this.initialOption("childOffset"),o=n._options.silent("label")&&!this._$label?this._isPositionLeft(this._getPosition())?i:-i:0,a=this.initialOption("indent")+this.initialOption("childIndent")*t,r=this._getDirectionIndex(e,this.option("direction"))*a,s=n._options.silent("label")?this._isPositionLeft(this._getPosition())?"left":"right":"center";return{of:this.$content(),at:s,my:s,offset:{x:o,y:r}}}},{key:"_outsideClickHandler",value:function(e){if(this._isShadingShown){var t=(0,b.default)(e.target)[0]===this._$wrapper[0];t&&this._clickHandler()}}},{key:"_setPosition",value:function(){this.option("visible")&&(this._hide(),this._show())}},{key:"_getPosition",value:function(){return this._getDefaultOptions().position}},{key:"_getInkRippleContainer",value:function(){return this.$content()}},{key:"_optionChanged",value:function(e){switch(e.name){case"actions":this._isVisible()&&this._renderIcon(),this._renderCloseIcon(),this._renderClick(),this._renderActions();break;case"maxSpeedDialActionCount":this._renderActions();break;case"closeIcon":this._renderCloseIcon();break;case"position":this._setPosition();break;case"label":this._isVisible()&&this._renderLabel(),this._setPosition();break;case"icon":this._isVisible()&&this._renderIcon();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(D.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=i(n(585));(0,o.default)("dxTextEditor",a.default);var r=a.default;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(1),r=i(n(23)),s=i(n(13)),l=i(n(11)),u=s.default.real(),c=u.ios,d=u.mac,h=r.default.msie||c||d,f=function(e){var t;try{t={start:e.selectionStart,end:e.selectionEnd}}catch(e){t={start:0,end:0}}return t},p=function(e,t){if(l.default.getBody().contains(e))try{e.selectionStart=t.start,e.selectionEnd=t.end}catch(e){}},g=function(e,t){return e=(0,o.default)(e).get(0),(0,a.isDefined)(t)?void(h&&l.default.getActiveElement()!==e||p(e,t)):f(e)},_=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(5)),l=n(8),u=i(n(23)),c=n(12),d=n(33),h="dxMask",f="blur beforedeactivate",p=" ",g=function(){function e(t){o(this,e),this.editor=t,this.DIRECTION={FORWARD:"forward",BACKWARD:"backward"},this.NAME=this._getStrategyName()}return r(e,[{key:"_getStrategyName",value:function(){return"base"}},{key:"editorOption",value:function(){var e;return(e=this.editor).option.apply(e,arguments)}},{key:"editorInput",value:function(){return this.editor._input()}},{key:"editorCaret",value:function(e){return e?void this.editor._caret(e):this.editor._caret()}},{key:"getHandler",value:function(e){var t=this["_".concat(e,"Handler")]||function(){};return t.bind(this)}},{key:"attachEvents",value:function(){var e=this,t=this.editorInput();this.getHandleEventNames().forEach(function(n){var i=(0,l.addNamespace)(n.toLowerCase(),h);s.default.on(t,i,e.getEventHandler(n))}),this._attachChangeEventHandlers()}},{key:"getHandleEventNames",value:function(){return["focusIn","focusOut","keyDown","input","paste","cut","drop"]}},{key:"getEventHandler",value:function(e){return this["_".concat(e,"Handler")].bind(this)}},{key:"detachEvents",value:function(){s.default.off(this.editorInput(),".".concat(h))}},{key:"_attachChangeEventHandlers",value:function(){(0,c.inArray)("change",this.editorOption("valueChangeEvent").split(" "))!==-1&&s.default.on(this.editorInput(),(0,l.addNamespace)(f,h),function(e){this._suppressCaretChanging(this._changeHandler,[e]),this._changeHandler(e)}.bind(this.editor))}},{key:"_focusInHandler",value:function(){if(this.editor._showMaskPlaceholder(),this.editor._direction(this.DIRECTION.FORWARD),!this.editor._isValueEmpty()&&this.editorOption("isValid"))this.editor._adjustCaret();else{var e=this.editor._maskRulesChain.first();this._caretTimeout=setTimeout(function(){this._caret({start:e,end:e})}.bind(this.editor),0)}}},{key:"_focusOutHandler",value:function(e){this.editor._changeHandler(e),"onFocus"===this.editorOption("showMaskMode")&&this.editor._isValueEmpty()&&(this.editorOption("text",""),this.editor._renderDisplayText(""))}},{key:"_cutHandler",value:function(e){var t=this.editorCaret(),n=this.editorInput().val().substring(t.start,t.end);this.editor._maskKeyHandler(e,function(){return(0,d.clipboardText)(e,n)})}},{key:"_dropHandler",value:function(){this._clearDragTimer(),this._dragTimer=setTimeout(function(){this.option("value",this._convertToValue(this._input().val()))}.bind(this.editor))}},{key:"_clearDragTimer",value:function(){clearTimeout(this._dragTimer)}},{key:"_keyDownHandler",value:function(){this._keyPressHandled=!1}},{key:"_pasteHandler",value:function(e){var t=this.editor;this._keyPressHandled=!0;var n=this.editorCaret();t._maskKeyHandler(e,function(){var i=(0,d.clipboardText)(e),o=t._maskRulesChain.text().substring(n.end),a=t._handleChain({text:i,start:n.start,length:i.length}),r=n.start+a;t._handleChain({text:o,start:r,length:o.length}),t._caret({start:r,end:r})})}},{key:"_autoFillHandler",value:function(e){var t=this,n=this.editor,i=this.editorInput().val();this._inputHandlerTimer=setTimeout(function(){t._keyPressHandled=!0,t._isAutoFill()&&(t._keyPressHandled=!0,n._maskKeyHandler(e,function(){n._handleChain({text:i,start:0,length:i.length})}),n._validateMask())})}},{key:"_isAutoFill",value:function(){var e=this.editor._input(),t=!1;if(u.default.msie&&u.default.version>11)t=e.hasClass("edge-autofilled");else if(u.default.webkit){var n=e.get(0);t=n&&n.matches(":-webkit-autofill")}return t}},{key:"runWithoutEventProcessing",value:function(e){var t=this._keyPressHandled;this._keyPressHandled=!0,e(),this._keyPressHandled=t}},{key:"_backspaceHandler",value:function(){}},{key:"_delHandler",value:function(e){var t=this.editor;this._keyPressHandled=!0,t._maskKeyHandler(e,function(){return!t._hasSelection()&&t._handleKey(p)})}},{key:"clean",value:function(){this._clearDragTimer(),clearTimeout(this._backspaceHandlerTimeout),clearTimeout(this._caretTimeout),clearTimeout(this._inputHandlerTimer)}}]),e}();t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(8),r=n(0),s=i(n(595)),l=i(n(10)),u=i(n(596)),c=n(283),d="dx-list-item-selected",h="dx-list-item-response-wait",f=c.ListBase.inherit({_supportedKeys:function(){var e=this,t=this,n=this.callBase(),i=function(e){t.option("allowItemDeleting")&&(e.preventDefault(),t.deleteItem(t.option("focusedElement")))},o=function(i,o){var a=e._editStrategy,r=e.option("focusedElement"),s=a.getNormalizedIndex(r),l=s===e._getLastItemIndex();if(!l||!e._isDataSourceLoading())if(i.shiftKey&&t.option("itemDragging.allowReordering")){var u=s+(o?-1:1),c=a.getItemElement(u);e.reorderItem(r,c),e.scrollToItem(r),i.preventDefault()}else{var d=e._editProvider,h=d.handleKeyboardEvents(s,o);h||(o?n.upArrow(i):n.downArrow(i))}},a=function(e){this._editProvider.handleEnterPressing()||n.enter.apply(this,arguments)},s=function(e){this._editProvider.handleEnterPressing()||n.space.apply(this,arguments)};return(0,r.extend)({},n,{del:i,upArrow:function(e){return o(e,!0)},downArrow:function(e){return o(e)},enter:a,space:s})},_updateSelection:function(){this._editProvider.afterItemsRendered(),this.callBase()},_getLastItemIndex:function(){return this._itemElements().length-1},_refreshItemElements:function(){this.callBase();var e=this._editProvider.getExcludedItemSelectors();e.length&&(this._itemElementsCache=this._itemElementsCache.not(e))},_isItemStrictEquals:function(e,t){var n=e&&e.__dx_key__;return!(n&&!this.key()&&this._selection.isItemSelected(n))&&this.callBase(e,t)},_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{showSelectionControls:!1,selectionMode:"none",selectAllMode:"page",onSelectAllValueChanged:null,selectAllText:l.default.format("dxList-selectAll"),menuItems:[],menuMode:"context",allowItemDeleting:!1,itemDeleteMode:"static",itemDragging:{}})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(e){return"ios"===e.platform},options:{menuMode:"slide",itemDeleteMode:"slideItem"}},{device:{platform:"android"},options:{itemDeleteMode:"swipe"}}])},_init:function(){this.callBase(),this._initEditProvider()},_initDataSource:function(){this.callBase(),this._isPageSelectAll()||this._dataSource&&this._dataSource.requireTotalCount(!0)},_isPageSelectAll:function(){return"page"===this.option("selectAllMode")},_initEditProvider:function(){this._editProvider=new u.default(this)},_disposeEditProvider:function(){this._editProvider&&this._editProvider.dispose()},_refreshEditProvider:function(){this._disposeEditProvider(),this._initEditProvider()},_initEditStrategy:function(){this.option("grouped")?this._editStrategy=new s.default(this):this.callBase()},_initMarkup:function(){this._refreshEditProvider(),this.callBase()},_renderItems:function(){this.callBase.apply(this,arguments),this._editProvider.afterItemsRendered()},_selectedItemClass:function(){return d},_itemResponseWaitClass:function(){return h},_itemClickHandler:function(e){var t=(0,o.default)(e.currentTarget);if(!t.is(".dx-state-disabled, .dx-state-disabled *")){var n=this._editProvider.handleClick(t,e);n||(this._saveSelectionChangeEvent(e),this.callBase.apply(this,arguments))}},_shouldFireContextMenuEvent:function(){return this.callBase.apply(this,arguments)||this._editProvider.contextMenuHandlerExists()},_itemHoldHandler:function(e){var t=(0,o.default)(e.currentTarget);if(!t.is(".dx-state-disabled, .dx-state-disabled *")){var n=(0,a.isTouchEvent)(e)&&this._editProvider.handleContextMenu(t,e);return n?void(e.handledByEditProvider=!0):void this.callBase.apply(this,arguments)}},_itemContextMenuHandler:function(e){var t=(0,o.default)(e.currentTarget);if(!t.is(".dx-state-disabled, .dx-state-disabled *")){var n=!e.handledByEditProvider&&this._editProvider.handleContextMenu(t,e);return n?void e.preventDefault():void this.callBase.apply(this,arguments)}},_postprocessRenderItem:function(e){this.callBase.apply(this,arguments),this._editProvider.modifyItemElement(e)},_clean:function(){this._disposeEditProvider(),this.callBase()},focusListItem:function(e){var t=this._editStrategy.getItemElement(e);this.option("focusedElement",t),this.focus(),this.scrollToItem(this.option("focusedElement"))},_optionChanged:function(e){switch(e.name){case"selectAllMode":this._initDataSource(),this._dataSource.pageIndex(0),this._dataSource.load();break;case"grouped":this._clearSelectedItems(),delete this._renderingGroupIndex,this._initEditStrategy(),this.callBase(e);break;case"showSelectionControls":case"menuItems":case"menuMode":case"allowItemDeleting":case"itemDeleteMode":case"itemDragging":case"selectAllText":this._invalidate();break;case"onSelectAllValueChanged":break;default:this.callBase(e)}},selectAll:function(){return this._selection.selectAll(this._isPageSelectAll())},unselectAll:function(){return this._selection.deselectAll(this._isPageSelectAll())},isSelectAll:function(){return this._selection.getSelectAllState(this._isPageSelectAll())},getFlatIndexByItemElement:function(e){return this._itemElements().index(e)},getItemElementByFlatIndex:function(e){var t=this._itemElements();return e<0||e>=t.length?(0,o.default)():t.eq(e)},getItemByIndex:function(e){return this._editStrategy.getItemDataByIndex(e)}}),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=i(n(165)),l=n(8),u=i(n(25)),c=n(140),d=s.default.abstract,h="dxListEditDecorator",f=(0,l.addNamespace)(u.default.down,h),p=(0,l.addNamespace)(c.active,h),g="dx-list-item-content",_="dx-list-switchable-delete-ready",m="dx-list-switchable-menu-shield-positioning",v="dx-list-switchable-delete-top-shield",y="dx-list-switchable-delete-bottom-shield",b="dx-list-switchable-menu-item-shield-positioning",x="dx-list-switchable-delete-item-content-shield",w="dx-list-switchable-delete-button-container",C=s.default.inherit({_init:function(){this._$topShield=(0,o.default)("<div>").addClass(v),this._$bottomShield=(0,o.default)("<div>").addClass(y),this._$itemContentShield=(0,o.default)("<div>").addClass(x),a.default.on(this._$topShield,f,this._cancelDeleteReadyItem.bind(this)),a.default.on(this._$bottomShield,f,this._cancelDeleteReadyItem.bind(this)),this._list.$element().append(this._$topShield.toggle(!1)).append(this._$bottomShield.toggle(!1))},handleClick:function(){return this._cancelDeleteReadyItem()},_cancelDeleteReadyItem:function(){return!!this._$readyToDeleteItem&&(this._cancelDelete(this._$readyToDeleteItem),!0)},_cancelDelete:function(e){this._toggleDeleteReady(e,!1)},_toggleDeleteReady:function(e,t){void 0===t&&(t=!this._isReadyToDelete(e)),this._toggleShields(e,t),this._toggleScrolling(t),this._cacheReadyToDeleteItem(e,t),this._animateToggleDelete(e,t)},_isReadyToDelete:function(e){return e.hasClass(_)},_toggleShields:function(e,t){this._list.$element().toggleClass(m,t),this._$topShield.toggle(t),this._$bottomShield.toggle(t),t&&this._updateShieldsHeight(e),this._toggleContentShield(e,t)},_updateShieldsHeight:function(e){var t=this._list.$element(),n=t.offset().top,i=t.outerHeight(),o=e.offset().top,a=e.outerHeight(),r=o-n,s=i-a-r;this._$topShield.height(Math.max(r,0)),this._$bottomShield.height(Math.max(s,0))},_toggleContentShield:function(e,t){t?e.find("."+g).first().append(this._$itemContentShield):this._$itemContentShield.detach()},_toggleScrolling:function(e){var t=this._list.$element().dxScrollView("instance");e?t.on("start",this._cancelScrolling):t.off("start",this._cancelScrolling)},_cancelScrolling:function(e){e.event.cancel=!0},_cacheReadyToDeleteItem:function(e,t){t?this._$readyToDeleteItem=e:delete this._$readyToDeleteItem},_animateToggleDelete:function(e,t){t?(this._enablePositioning(e),this._prepareDeleteReady(e),this._animatePrepareDeleteReady(e),a.default.off(e,u.default.up)):(this._forgetDeleteReady(e),this._animateForgetDeleteReady(e).done(this._disablePositioning.bind(this,e)))},_enablePositioning:function(e){e.addClass(b),a.default.on(e,p,r.noop),a.default.one(e,u.default.up,this._disablePositioning.bind(this,e))},_disablePositioning:function(e){e.removeClass(b),a.default.off(e,p)},_prepareDeleteReady:function(e){e.addClass(_)},_forgetDeleteReady:function(e){e.removeClass(_)},_animatePrepareDeleteReady:d,_animateForgetDeleteReady:d,_getDeleteButtonContainer:function(e){return e=e||this._$readyToDeleteItem,e.children("."+w)},_deleteItem:function(e){e=e||this._$readyToDeleteItem,this._getDeleteButtonContainer(e).detach(),e.is(".dx-state-disabled, .dx-state-disabled *")||this._list.deleteItem(e).always(this._cancelDelete.bind(this,e))},_isRtlEnabled:function(){return this._list.option("rtlEnabled")},dispose:function(){this._$topShield&&this._$topShield.remove(),this._$bottomShield&&this._$bottomShield.remove(),this.callBase.apply(this,arguments)}}),k=C;t.default=k,e.exports=t.default},function(e,t,n){t.default=void 0;var i={_menuEnabled:function(){return!!this._menuItems().length},_menuItems:function(){return this._list.option("menuItems")},_deleteEnabled:function(){return this._list.option("allowItemDeleting")},_fireMenuAction:function(e,t){this._list._itemEventHandlerByHandler(e,t,{},{excludeValidators:["disabled","readOnly"]})}},o=i;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=i(n(49)),l=n(30),u=i(n(16)),c=n(8),d=n(4),h=n(1),f=n(0),p=i(n(25)),g="dxScrollbar",_="dx-scrollable-scrollbar",m="".concat(_,"-active"),v="dx-scrollable-scroll",y="dx-scrollable-scroll-content",b="dx-scrollbar-hoverable",x="horizontal",w=15,C={onScroll:"onScroll",onHover:"onHover",always:"always",never:"never"},k=null,S=u.default.inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{direction:null,visible:!1,activeStateEnabled:!1,visibilityMode:C.onScroll,containerSize:0,contentSize:0,expandable:!0,scaleRatio:1})},_init:function(){this.callBase(),this._isHovered=!1},_initMarkup:function(){this._renderThumb(),this.callBase()},_render:function(){this.callBase(),this._renderDirection(),this._update(),this._attachPointerDownHandler(),this.option("hoverStateEnabled",this._isHoverMode()),this.$element().toggleClass(b,this.option("hoverStateEnabled"))},_renderThumb:function(){this._$thumb=(0,o.default)("<div>").addClass(v),(0,o.default)("<div>").addClass(y).appendTo(this._$thumb),this.$element().addClass(_).append(this._$thumb)},isThumb:function(e){return!!this.$element().find(e).length},_isHoverMode:function(){var e=this.option("visibilityMode");return(e===C.onHover||e===C.always)&&this.option("expandable")},_renderDirection:function(){var e=this.option("direction");this.$element().addClass("dx-scrollbar-"+e),this._dimension=e===x?"width":"height",this._prop=e===x?"left":"top"},_attachPointerDownHandler:function(){r.default.on(this._$thumb,(0,c.addNamespace)(p.default.down,g),this.feedbackOn.bind(this))},feedbackOn:function(){this.$element().addClass(m),k=this},feedbackOff:function(){this.$element().removeClass(m),k=null},cursorEnter:function(){this._isHovered=!0,this._needScrollbar()&&this.option("visible",!0)},cursorLeave:function(){this._isHovered=!1,this.option("visible",!1)},_renderDimensions:function(){this._$thumb.css({width:this.option("width"),height:this.option("height")})},_toggleVisibility:function(e){this.option("visibilityMode")===C.onScroll&&this._$thumb.css("opacity"),e=this._adjustVisibility(e),this.option().visible=e,this._$thumb.toggleClass("dx-state-invisible",!e)},_adjustVisibility:function(e){if(this._baseContainerToContentRatio&&!this._needScrollbar())return!1;switch(this.option("visibilityMode")){case C.onScroll:break;case C.onHover:e=e||!!this._isHovered;break;case C.never:e=!1;break;
case C.always:e=!0}return e},moveTo:function(e){if(!this._isHidden()){(0,h.isPlainObject)(e)&&(e=e[this._prop]||0);var t={};t[this._prop]=this._calculateScrollBarPosition(e),(0,l.move)(this._$thumb,t)}},_calculateScrollBarPosition:function(e){return-e*this._thumbRatio},_update:function(){var e=Math.round(this.option("containerSize")),t=Math.round(this.option("contentSize")),n=Math.round(this.option("baseContainerSize")),i=Math.round(this.option("baseContentSize"));isNaN(n)&&(n=e,i=t),this._baseContainerToContentRatio=i?n/i:n,this._realContainerToContentRatio=t?e/t:e;var o=Math.round(Math.max(Math.round(e*this._realContainerToContentRatio),w));this._thumbRatio=(e-o)/(this.option("scaleRatio")*(t-e)),this.option(this._dimension,o/this.option("scaleRatio")),this.$element().css("display",this._needScrollbar()?"":"none")},_isHidden:function(){return this.option("visibilityMode")===C.never},_needScrollbar:function(){return!this._isHidden()&&this._baseContainerToContentRatio<1},containerToContentRatio:function(){return this._realContainerToContentRatio},_normalizeSize:function(e){return(0,h.isPlainObject)(e)?e[this._dimension]||0:e},_clean:function(){this.callBase(),this===k&&(k=null),r.default.off(this._$thumb,"."+g)},_optionChanged:function(e){if(!this._isHidden())switch(e.name){case"containerSize":case"contentSize":this.option()[e.name]=this._normalizeSize(e.value),this._update();break;case"baseContentSize":case"baseContainerSize":this._update();break;case"visibilityMode":case"direction":this._invalidate();break;case"scaleRatio":this._update();break;default:this.callBase.apply(this,arguments)}},update:(0,d.deferRenderer)(function(){this._adjustVisibility()&&this.option("visible",!0)})});s.default.add(function(){r.default.subscribeGlobal(a.default.getDocument(),(0,c.addNamespace)(p.default.up,g),function(){k&&k.feedbackOff()})});var D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.SimulatedStrategy=t.Scroller=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=n(40),l=n(0),u=n(7),c=n(3),d=n(1),h=n(24),f=n(30),p=i(n(15)),g=i(n(430)),_=i(n(13)),m=n(8),v=n(4),y=i(n(428)),b=n(6),x=_.default.real,w="android"===x.platform,C="dxSimulatedScrollable",k="dxScrollableStrategy",S=C+"Cursor",D=C+"Keyboard",I="dx-scrollable-simulated",T="dx-scrollable-scrollbars-hidden",O="dx-scrollable-scrollbars-alwaysvisible",E="dx-scrollable-scrollbar",A="vertical",P="horizontal",R=w?.95:.92,M=.5,B=1,V=Math.round(1e3/60),F=40,L=500,H=B/5,z=w?300:400,N=z/V,j=(1-Math.pow(R,N))/(1-R),W={PAGE_UP:"pageUp",PAGE_DOWN:"pageDown",END:"end",HOME:"home",LEFT:"leftArrow",UP:"upArrow",RIGHT:"rightArrow",DOWN:"downArrow",TAB:"tab"},G=g.default.inherit({ctor:function(e){this.callBase(),this.scroller=e},VELOCITY_LIMIT:B,_isFinished:function(){return Math.abs(this.scroller._velocity)<=this.VELOCITY_LIMIT},_step:function(){this.scroller._scrollStep(this.scroller._velocity),this.scroller._velocity*=this._acceleration()},_acceleration:function(){return this.scroller._inBounds()?R:M},_complete:function(){this.scroller._scrollComplete()},_stop:function(){this.scroller._stopComplete()}}),q=G.inherit({VELOCITY_LIMIT:H,_isFinished:function(){return this.scroller._crossBoundOnNextStep()||this.callBase()},_acceleration:function(){return R},_complete:function(){this.scroller._move(this.scroller._bounceLocation),this.callBase()}}),$=p.default.inherit({ctor:function(e){this._initOptions(e),this._initAnimators(),this._initScrollbar()},_initOptions:function(e){var t=this;this._location=0,this._topReached=!1,this._bottomReached=!1,this._axis=e.direction===P?"x":"y",this._prop=e.direction===P?"left":"top",this._dimension=e.direction===P?"width":"height",this._scrollProp=e.direction===P?"scrollLeft":"scrollTop",(0,c.each)(e,function(e,n){t["_"+e]=n})},_initAnimators:function(){this._inertiaAnimator=new G(this),this._bounceAnimator=new q(this)},_initScrollbar:function(){this._scrollbar=new y.default((0,o.default)("<div>").appendTo(this._$container),{direction:this._direction,visible:this._scrollByThumb,visibilityMode:this._visibilityModeNormalize(this._scrollbarVisible),expandable:this._scrollByThumb}),this._$scrollbar=this._scrollbar.$element()},_visibilityModeNormalize:function(e){return e===!0?"onScroll":e===!1?"never":e},_scrollStep:function(e){var t=this._location;this._location+=e,this._suppressBounce(),this._move(),Math.abs(t-this._location)<1||r.default.triggerHandler(this._$container,{type:"scroll"})},_suppressBounce:function(){this._bounceEnabled||this._inBounds(this._location)||(this._velocity=0,this._location=this._boundLocation())},_boundLocation:function(e){return e=void 0!==e?e:this._location,Math.max(Math.min(e,this._maxOffset),this._minOffset)},_move:function(e){this._location=void 0!==e?e*this._getScaleRatio():this._location,this._moveContent(),this._moveScrollbar()},_moveContent:function(){var e=this._location;this._$container[this._scrollProp](-e/this._getScaleRatio()),this._moveContentByTranslator(e)},_getScaleRatio:function(){if((0,u.hasWindow)()&&!this._scaleRatio){var e=this._$element.get(0),t=this._getRealDimension(e,this._dimension),n=this._getBaseDimension(e,this._dimension);this._scaleRatio=Math.round(t/n*100)/100}return this._scaleRatio||1},_getRealDimension:function(e,t){return Math.round((0,h.getBoundingRect)(e)[t])},_getBaseDimension:function(e,t){var n="offset"+(0,s.titleize)(t);return e[n]},_moveContentByTranslator:function(e){var t,n=-this._maxScrollPropValue;if(t=e>0?e:e<=n?e-n:e%1,this._translateOffset!==t){var i={};return i[this._prop]=t,this._translateOffset=t,0===t?void(0,f.resetPosition)(this._$content):void(0,f.move)(this._$content,i)}},_moveScrollbar:function(){this._scrollbar.moveTo(this._location)},_scrollComplete:function(){this._inBounds()&&(this._hideScrollbar(),this._completeDeferred&&this._completeDeferred.resolve()),this._scrollToBounds()},_scrollToBounds:function(){this._inBounds()||(this._bounceAction(),this._setupBounce(),this._bounceAnimator.start())},_setupBounce:function(){var e=this._bounceLocation=this._boundLocation(),t=e-this._location;this._velocity=t/j},_inBounds:function(e){return e=void 0!==e?e:this._location,this._boundLocation(e)===e},_crossBoundOnNextStep:function(){var e=this._location,t=e+this._velocity;return e<this._minOffset&&t>=this._minOffset||e>this._maxOffset&&t<=this._maxOffset},_initHandler:function(e){return this._stopDeferred=new b.Deferred,this._stopScrolling(),this._prepareThumbScrolling(e),this._stopDeferred.promise()},_stopScrolling:(0,v.deferRenderer)(function(){this._hideScrollbar(),this._inertiaAnimator.stop(),this._bounceAnimator.stop()}),_prepareThumbScrolling:function(e){if(!(0,m.isDxMouseWheelEvent)(e.originalEvent)){var t=(0,o.default)(e.originalEvent.target),n=this._isScrollbar(t);n&&this._moveToMouseLocation(e),this._thumbScrolling=n||this._isThumb(t),this._crossThumbScrolling=!this._thumbScrolling&&this._isAnyThumbScrolling(t),this._thumbScrolling&&this._scrollbar.feedbackOn()}},_isThumbScrollingHandler:function(e){return this._isThumb(e)},_moveToMouseLocation:function(e){var t=e["page"+this._axis.toUpperCase()]-this._$element.offset()[this._prop],n=this._location+t/this._containerToContentRatio()-this._$container.height()/2;this._scrollStep(-Math.round(n))},_stopComplete:function(){this._stopDeferred&&this._stopDeferred.resolve()},_startHandler:function(){this._showScrollbar()},_moveHandler:function(e){this._crossThumbScrolling||(this._thumbScrolling&&(e[this._axis]=-Math.round(e[this._axis]/this._containerToContentRatio())),this._scrollBy(e))},_scrollBy:function(e){e=e[this._axis],this._inBounds()||(e*=M),this._scrollStep(e)},_scrollByHandler:function(e){this._scrollBy(e),this._scrollComplete()},_containerToContentRatio:function(){return this._scrollbar.containerToContentRatio()},_endHandler:function(e){return this._completeDeferred=new b.Deferred,this._velocity=e[this._axis],this._inertiaHandler(),this._resetThumbScrolling(),this._completeDeferred.promise()},_inertiaHandler:function(){this._suppressInertia(),this._inertiaAnimator.start()},_suppressInertia:function(){this._inertiaEnabled&&!this._thumbScrolling||(this._velocity=0)},_resetThumbScrolling:function(){this._thumbScrolling=!1,this._crossThumbScrolling=!1},_stopHandler:function(){this._thumbScrolling&&this._scrollComplete(),this._resetThumbScrolling(),this._scrollToBounds()},_disposeHandler:function(){this._stopScrolling(),this._$scrollbar.remove()},_updateHandler:function(){this._update(),this._moveToBounds()},_update:function(){var e=this;return this._stopScrolling(),(0,v.deferUpdate)(function(){e._resetScaleRatio(),e._updateLocation(),e._updateBounds(),e._updateScrollbar(),(0,v.deferRender)(function(){e._moveScrollbar(),e._scrollbar.update()})})},_resetScaleRatio:function(){this._scaleRatio=null},_updateLocation:function(){this._location=((0,f.locate)(this._$content)[this._prop]-this._$container[this._scrollProp]())*this._getScaleRatio()},_updateBounds:function(){this._maxOffset=Math.round(this._getMaxOffset()),this._minOffset=Math.round(this._getMinOffset())},_getMaxOffset:function(){return 0},_getMinOffset:function(){return this._maxScrollPropValue=Math.max(this._contentSize()-this._containerSize(),0),-this._maxScrollPropValue},_updateScrollbar:(0,v.deferUpdater)(function(){var e=this,t=this._containerSize(),n=this._contentSize(),i=this._getBaseDimension(this._$container.get(0),this._dimension),o=this._getBaseDimension(this._$content.get(0),this._dimension);(0,v.deferRender)(function(){e._scrollbar.option({containerSize:t,contentSize:n,baseContainerSize:i,baseContentSize:o,scaleRatio:e._getScaleRatio()})})}),_moveToBounds:(0,v.deferRenderer)((0,v.deferUpdater)((0,v.deferRenderer)(function(){var e=this._boundLocation(),t=e!==this._location;this._location=e,this._move(),t&&this._scrollAction()}))),_createActionsHandler:function(e){this._scrollAction=e.scroll,this._bounceAction=e.bounce},_showScrollbar:function(){this._scrollbar.option("visible",!0)},_hideScrollbar:function(){this._scrollbar.option("visible",!1)},_containerSize:function(){return this._getRealDimension(this._$container.get(0),this._dimension)},_contentSize:function(){var e="hidden"===this._$content.css("overflow"+this._axis.toUpperCase()),t=this._getRealDimension(this._$content.get(0),this._dimension);if(!e){var n=this._$content[0]["scroll"+(0,s.titleize)(this._dimension)]*this._getScaleRatio();t=Math.max(n,t)}return t},_validateEvent:function(e){var t=(0,o.default)(e.originalEvent.target);return this._isThumb(t)||this._isScrollbar(t)||this._isContent(t)},_isThumb:function(e){return this._scrollByThumb&&this._scrollbar.isThumb(e)},_isScrollbar:function(e){return this._scrollByThumb&&e&&e.is(this._$scrollbar)},_isContent:function(e){return this._scrollByContent&&!!e.closest(this._$element).length},_reachedMin:function(){return this._location<=this._minOffset},_reachedMax:function(){return this._location>=this._maxOffset},_cursorEnterHandler:function(){this._resetScaleRatio(),this._updateScrollbar(),this._scrollbar.cursorEnter()},_cursorLeaveHandler:function(){this._scrollbar.cursorLeave()},dispose:v.noop});t.Scroller=$;var K,U,Y=p.default.inherit({ctor:function(e){this._init(e)},_init:function(e){this._component=e,this._$element=e.$element(),this._$container=e._$container,this._$wrapper=e._$wrapper,this._$content=e._$content,this.option=e.option.bind(e),this._createActionByOption=e._createActionByOption.bind(e),this._isLocked=e._isLocked.bind(e),this._isDirection=e._isDirection.bind(e),this._allowedDirection=e._allowedDirection.bind(e),this._getScrollOffset=e._getScrollOffset.bind(e)},render:function(){this._$element.addClass(I),this._createScrollers(),this.option("useKeyboard")&&this._$container.prop("tabIndex",0),this._attachKeyboardHandler(),this._attachCursorHandlers()},_createScrollers:function(){this._scrollers={},this._isDirection(P)&&this._createScroller(P),this._isDirection(A)&&this._createScroller(A),this._$element.toggleClass(O,"always"===this.option("showScrollbar")),this._$element.toggleClass(T,!this.option("showScrollbar"))},_createScroller:function(e){this._scrollers[e]=new $(this._scrollerOptions(e))},_scrollerOptions:function(e){return{direction:e,$content:this._$content,$container:this._$container,$wrapper:this._$wrapper,$element:this._$element,scrollByContent:this.option("scrollByContent"),scrollByThumb:this.option("scrollByThumb"),scrollbarVisible:this.option("showScrollbar"),bounceEnabled:this.option("bounceEnabled"),inertiaEnabled:this.option("inertiaEnabled"),isAnyThumbScrolling:this._isAnyThumbScrolling.bind(this)}},_applyScaleRatio:function(e){for(var t in this._scrollers){var n=this._getPropByDirection(t);if((0,d.isDefined)(e[n])){var i=this._scrollers[t];e[n]*=i._getScaleRatio()}}return e},_isAnyThumbScrolling:function(e){var t=!1;return this._eventHandler("isThumbScrolling",e).done(function(e,n){t=e||n}),t},handleInit:function(e){this._suppressDirections(e),this._eventForUserAction=e,this._eventHandler("init",e).done(this._stopAction)},_suppressDirections:function(e){return(0,m.isDxMouseWheelEvent)(e.originalEvent)?void this._prepareDirections(!0):(this._prepareDirections(),void this._eachScroller(function(t,n){var i=t._validateEvent(e);this._validDirections[n]=i}))},_prepareDirections:function(e){e=e||!1,this._validDirections={},this._validDirections[P]=e,this._validDirections[A]=e},_eachScroller:function(e){e=e.bind(this),(0,c.each)(this._scrollers,function(t,n){e(n,t)})},handleStart:function(e){this._eventForUserAction=e,this._eventHandler("start").done(this._startAction)},_saveActive:function(){U=this},_resetActive:function(){U===this&&(U=null)},handleMove:function(e){return this._isLocked()?(e.cancel=!0,void this._resetActive()):(this._saveActive(),e.preventDefault&&e.preventDefault(),this._adjustDistance(e,e.delta),this._eventForUserAction=e,void this._eventHandler("move",e.delta))},_adjustDistance:function(e,t){t.x*=this._validDirections[P],t.y*=this._validDirections[A];var n=this._tryGetDevicePixelRatio();n&&(0,m.isDxMouseWheelEvent)(e.originalEvent)&&(t.x=Math.round(t.x/n*100)/100,t.y=Math.round(t.y/n*100)/100)},_tryGetDevicePixelRatio:function(){if((0,u.hasWindow)())return(0,u.getWindow)().devicePixelRatio},handleEnd:function(e){return this._resetActive(),this._refreshCursorState(e.originalEvent&&e.originalEvent.target),this._adjustDistance(e,e.velocity),this._eventForUserAction=e,this._eventHandler("end",e.velocity).done(this._endAction)},handleCancel:function(e){return this._resetActive(),this._eventForUserAction=e,this._eventHandler("end",{x:0,y:0})},handleStop:function(){this._resetActive(),this._eventHandler("stop")},handleScroll:function(){this._component._updateRtlConfig(),this._scrollAction()},_attachKeyboardHandler:function(){r.default.off(this._$element,".".concat(D)),!this.option("disabled")&&this.option("useKeyboard")&&r.default.on(this._$element,(0,m.addNamespace)("keydown",D),this._keyDownHandler.bind(this))},_keyDownHandler:function(e){var t=this;if(clearTimeout(this._updateHandlerTimeout),this._updateHandlerTimeout=setTimeout(function(){(0,m.normalizeKeyName)(e)===W.TAB&&t._eachScroller(function(e){e._updateHandler()})}),this._$container.is(a.default.getActiveElement())){var n=!0;switch((0,m.normalizeKeyName)(e)){case W.DOWN:this._scrollByLine({y:1});break;case W.UP:this._scrollByLine({y:-1});break;case W.RIGHT:this._scrollByLine({x:1});break;case W.LEFT:this._scrollByLine({x:-1});break;case W.PAGE_DOWN:this._scrollByPage(1);break;case W.PAGE_UP:this._scrollByPage(-1);break;case W.HOME:this._scrollToHome();break;case W.END:this._scrollToEnd();break;default:n=!1}n&&(e.stopPropagation(),e.preventDefault())}},_scrollByLine:function(e){var t=this._tryGetDevicePixelRatio(),n=F;t&&(n=Math.abs(n/t*100)/100),this.scrollBy({top:(e.y||0)*-n,left:(e.x||0)*-n})},_scrollByPage:function(e){var t=this._wheelProp(),n=this._dimensionByProp(t),i={};i[t]=e*-this._$container[n](),this.scrollBy(i)},_dimensionByProp:function(e){return"left"===e?"width":"height"},_getPropByDirection:function(e){return e===P?"left":"top"},_scrollToHome:function(){var e=this._wheelProp(),t={};t[e]=0,this._component.scrollTo(t)},_scrollToEnd:function(){var e=this._wheelProp(),t=this._dimensionByProp(e),n={};n[e]=this._$content[t]()-this._$container[t](),this._component.scrollTo(n)},createActions:function(){this._startAction=this._createActionHandler("onStart"),this._stopAction=this._createActionHandler("onStop"),this._endAction=this._createActionHandler("onEnd"),this._updateAction=this._createActionHandler("onUpdated"),this._createScrollerActions()},_createScrollerActions:function(){this._scrollAction=this._createActionHandler("onScroll"),this._bounceAction=this._createActionHandler("onBounce"),this._eventHandler("createActions",{scroll:this._scrollAction,bounce:this._bounceAction})},_createActionHandler:function(e){var t=arguments,n=this,i=this._createActionByOption(e);return function(){i((0,l.extend)(n._createActionArgs(),t))}},_createActionArgs:function(){var e=this._scrollers,t=e.horizontal,n=e.vertical,i=this._getScrollOffset();return this._scrollOffset={top:n&&i.top,left:t&&i.left},{event:this._eventForUserAction,scrollOffset:this._scrollOffset,reachedLeft:t&&t._reachedMax(),reachedRight:t&&t._reachedMin(),reachedTop:n&&n._reachedMax(),reachedBottom:n&&n._reachedMin()}},_eventHandler:function(e){var t=[].slice.call(arguments).slice(1),n=(0,c.map)(this._scrollers,function(n){return n["_"+e+"Handler"].apply(n,t)});return b.when.apply(o.default,n).promise()},location:function e(){var e=(0,f.locate)(this._$content);return e.top-=this._$container.scrollTop(),e.left-=this._$container.scrollLeft(),e},disabledChanged:function(){this._attachCursorHandlers()},_attachCursorHandlers:function(){r.default.off(this._$element,".".concat(S)),!this.option("disabled")&&this._isHoverMode()&&(r.default.on(this._$element,(0,m.addNamespace)("mouseenter",S),this._cursorEnterHandler.bind(this)),r.default.on(this._$element,(0,m.addNamespace)("mouseleave",S),this._cursorLeaveHandler.bind(this)))},_isHoverMode:function(){return"onHover"===this.option("showScrollbar")},_cursorEnterHandler:function(e){e=e||{},e.originalEvent=e.originalEvent||{},U||e.originalEvent._hoverHandled||(K&&K._cursorLeaveHandler(),K=this,this._eventHandler("cursorEnter"),e.originalEvent._hoverHandled=!0)},_cursorLeaveHandler:function(e){K===this&&U!==K&&(this._eventHandler("cursorLeave"),K=null,this._refreshCursorState(e&&e.relatedTarget))},_refreshCursorState:function(e){if(this._isHoverMode()||e&&!U){var t=(0,o.default)(e),n=t.closest(".".concat(I,":not(.dx-state-disabled)")),i=n.length&&n.data(k);K&&K!==i&&K._cursorLeaveHandler(),i&&i._cursorEnterHandler()}},update:function(){var e=this,t=this._eventHandler("update").done(this._updateAction);return(0,b.when)(t,(0,v.deferUpdate)(function(){var t=e._allowedDirections();return(0,v.deferRender)(function(){var n=t.vertical?"pan-x":"";n=t.horizontal?"pan-y":n,n=t.vertical&&t.horizontal?"none":n,e._$container.css("touchAction",n)}),(0,b.when)().promise()}))},_allowedDirections:function(){var e=this.option("bounceEnabled"),t=this._scrollers[A],n=this._scrollers[P];return{vertical:t&&(t._minOffset<0||e),horizontal:n&&(n._minOffset<0||e)}},updateBounds:function(){this._scrollers[P]&&this._scrollers[P]._updateBounds()},scrollBy:function(e){var t=this._scrollers[A],n=this._scrollers[P];t&&(e.top=t._boundLocation(e.top+t._location)-t._location),n&&(e.left=n._boundLocation(e.left+n._location)-n._location),this._prepareDirections(!0),this._startAction(),this._eventHandler("scrollBy",{x:e.left,y:e.top}),this._endAction()},validate:function(e){return!this.option("disabled")&&(!!this.option("bounceEnabled")||((0,m.isDxMouseWheelEvent)(e)?this._validateWheel(e):this._validateMove(e)))},_validateWheel:function(e){var t=this,n=this._scrollers[this._wheelDirection(e)],i=n._reachedMin(),o=n._reachedMax(),a=!i||!o,r=!i&&!o,s=i&&e.delta>0,l=o&&e.delta<0,u=a&&(r||s||l);return u=u||void 0!==this._validateWheelTimer,u&&(clearTimeout(this._validateWheelTimer),this._validateWheelTimer=setTimeout(function(){t._validateWheelTimer=void 0},L)),u},_validateMove:function(e){return!(!this.option("scrollByContent")&&!(0,o.default)(e.target).closest(".".concat(E)).length)&&this._allowedDirection()},getDirection:function(e){return(0,m.isDxMouseWheelEvent)(e)?this._wheelDirection(e):this._allowedDirection()},_wheelProp:function(){return this._wheelDirection()===P?"left":"top"},_wheelDirection:function(e){switch(this.option("direction")){case P:return P;case A:return A;default:return e&&e.shiftKey?P:A}},verticalOffset:function(){return 0},dispose:function(){this._resetActive(),K===this&&(K=null),this._eventHandler("dispose"),this._detachEventHandlers(),this._$element.removeClass(I),this._eventForUserAction=null,clearTimeout(this._validateWheelTimer),clearTimeout(this._updateHandlerTimeout)},_detachEventHandlers:function(){r.default.off(this._$element,".".concat(S)),r.default.off(this._$container,".".concat(D))}});t.SimulatedStrategy=Y},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(4),a=i(n(15)),r=n(120),s=a.default.abstract,l=a.default.inherit({ctor:function(){this._finished=!0,this._stopped=!1,this._proxiedStepCore=this._stepCore.bind(this)},start:function(){this._stopped=!1,this._finished=!1,this._stepCore()},stop:function(){this._stopped=!0,(0,r.cancelAnimationFrame)(this._stepAnimationFrame)},_stepCore:function(){return this._isStopped()?void this._stop():this._isFinished()?(this._finished=!0,void this._complete()):(this._step(),void(this._stepAnimationFrame=(0,r.requestAnimationFrame)(this._proxiedStepCore)))},_step:s,_isFinished:o.noop,_stop:o.noop,_complete:o.noop,_isStopped:function(){return this._stopped},inProgress:function(){return!(this._stopped||this._finished)}}),u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.deviceDependentOptions=void 0;var o=i(n(13)),a=n(46),r=function(){return[{device:function(){return!a.nativeScrolling},options:{useNative:!1}},{device:function(e){return!o.default.isSimulator()&&"desktop"===o.default.real().deviceType&&"generic"===e.platform},options:{bounceEnabled:!1,scrollByThumb:!0,scrollByContent:a.touch,showScrollbar:"onHover"}}]};t.deviceDependentOptions=r},function(e,t,n){t.findChanges=void 0;var i=n(1),o=function(e,t){var n=t(e);if((0,i.isObject)(n))try{return JSON.stringify(n)}catch(e){return n}return n},a=function(e,t,n,i){var a=o(e,i);return t[n[a]]},r=function(e,t,n,i){var r={},s={},l=0,u=0,c=[];e.forEach(function(e,t){var i=o(e,n);r[i]=t}),t.forEach(function(e,t){var i=o(e,n);s[i]=t});for(var d=Math.max(e.length,t.length),h=0;h<d+l;h++){var f=t[h],p=h-l+u,g=e[p],_=!f||g&&!a(g,t,s,n);if(_)g&&(c.push({type:"remove",key:n(g),index:h,oldItem:g}),u++,h--);else{var m=o(f,n),v=r[m],y=e[v];if(y){if(v!==p)return;i(y,f)||c.push({type:"update",data:f,key:n(f),index:h,oldItem:y})}else l++,c.push({type:"insert",data:f,index:h})}}return c};t.findChanges=r},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(16)),r=n(118),s=i(n(9)),l=i(n(289)),u=i(n(48)),c=i(n(115)),d=n(19),h=n(18),f=n(59),p=i(n(94)),g=n(69),_=i(n(81)),m=n(6),v=n(0),y=n(1),b=n(4),x=i(n(35)),w=n(213),C=i(n(10)),k="dx-dropdownbutton",S="dx-dropdownbutton-content",D="dx-dropdownbutton-action",I="dx-dropdownbutton-toggle",T="dx-dropdownbutton-has-arrow",O="dx-dropdownbutton-popup-wrapper",E="dx-button-text",A="dx-icon-right",P=a.default.inherit({_getDefaultOptions:function(){return(0,v.extend)(this.callBase(),{itemTemplate:"item",keyExpr:"this",displayExpr:"this",selectedItem:null,selectedItemKey:null,stylingMode:"outlined",deferRendering:!0,noDataText:C.default.format("dxCollectionWidget-noDataText"),useSelectMode:!1,splitButton:!1,showArrowIcon:!0,text:"",icon:void 0,onButtonClick:null,onSelectionChanged:null,onItemClick:null,opened:!1,items:null,dataSource:null,focusStateEnabled:!0,hoverStateEnabled:!0,dropDownOptions:{},dropDownContentTemplate:"content",wrapItemText:!1,grouped:!1,groupTemplate:"group",buttonGroupOptions:{}})},_setOptionsByReference:function(){this.callBase(),(0,v.extend)(this._optionsByReference,{selectedItem:!0})},_init:function(){this.callBase(),this._createItemClickAction(),this._createActionClickAction(),this._createSelectionChangedAction(),this._initDataSource(),this._compileKeyGetter(),this._compileDisplayGetter(),this._itemsToDataSource(this.option("items")),this._options.cache("buttonGroupOptions",this.option("buttonGroupOptions")),this._options.cache("dropDownOptions",this.option("dropDownOptions"))},_initTemplates:function(){var e=this;this._templateManager.addDefaultTemplates({content:new r.FunctionTemplate(function(t){var n=(0,o.default)(t.container),i=(0,o.default)("<div>").appendTo(n);e._list=e._createComponent(i,c.default,e._listOptions()),e._list.registerKeyHandler("escape",e._escHandler.bind(e)),e._list.registerKeyHandler("tab",e._escHandler.bind(e)),e._list.registerKeyHandler("leftArrow",e._escHandler.bind(e)),e._list.registerKeyHandler("rightArrow",e._escHandler.bind(e))})}),this.callBase()},_itemsToDataSource:function(e){this._dataSource||(this._dataSource=new g.DataSource({store:new _.default({key:this._getKey(),data:e}),pageSize:0}))},_getKey:function(){var e,t=this.option("keyExpr"),n=null===(e=this._dataSource)||void 0===e?void 0:e.key();return!(0,y.isDefined)(n)||(0,y.isDefined)(t)&&"this"!==t?t:n},_compileKeyGetter:function(){this._keyGetter=(0,d.compileGetter)(this._getKey())},_compileDisplayGetter:function(){this._displayGetter=(0,d.compileGetter)(this.option("displayExpr"))},_initMarkup:function(){this.callBase(),this.$element().addClass(k),this._renderButtonGroup(),this._updateArrowClass(),(0,y.isDefined)(this.option("selectedItemKey"))&&this._loadSelectedItem().done(this._updateActionButton.bind(this))},_render:function(){this.option("deferRendering")&&!this.option("opened")||this._renderPopup(),this.callBase()},_renderContentImpl:function(){return this._popup&&this._renderPopupContent(),this.callBase()},_loadSelectedItem:function(){var e=new m.Deferred;if(this._list&&void 0!==this._lastSelectedItemData){var t=this.option("useSelectMode")?this._list.option("selectedItem"):this._lastSelectedItemData;return e.resolve(t)}this._lastSelectedItemData=void 0;var n=this.option("selectedItemKey");return this._loadSingle(this._getKey(),n).done(e.resolve).fail(function(){e.resolve(null)}),e.promise()},_createActionClickAction:function(){this._actionClickAction=this._createActionByOption("onButtonClick")},_createSelectionChangedAction:function(){this._selectionChangedAction=this._createActionByOption("onSelectionChanged")},_createItemClickAction:function(){this._itemClickAction=this._createActionByOption("onItemClick")},_fireSelectionChangedAction:function(e){var t=e.previousValue,n=e.value;this._selectionChangedAction({item:n,previousItem:t})},_fireItemClickAction:function(e){var t=e.event,n=e.itemElement,i=e.itemData;return this._itemClickAction({event:t,itemElement:n,itemData:this._actionItem||i})},_actionButtonConfig:function(){return{text:this.option("text"),icon:this.option("icon"),elementAttr:{"class":D}}},_getButtonGroupItems:function(){var e=[];return e.push(this._actionButtonConfig()),this.option("splitButton")&&e.push({icon:"spindown",elementAttr:{"class":I}}),e},_buttonGroupItemClick:function(e){var t=e.event,n=e.itemData,i=n.elementAttr.class===D,o=n.elementAttr.class===I;o?this.toggle():i&&(this._actionClickAction({event:t,selectedItem:this.option("selectedItem")}),this.option("splitButton")||this.toggle())},_buttonGroupOptions:function(){var e=this;return(0,v.extend)({items:this._getButtonGroupItems(),focusStateEnabled:this.option("focusStateEnabled"),hoverStateEnabled:this.option("hoverStateEnabled"),onItemClick:this._buttonGroupItemClick.bind(this),width:"100%",height:"100%",stylingMode:this.option("stylingMode"),selectionMode:"none",buttonTemplate:function(t,n){var i=t.text,a=t.icon;if(e.option("splitButton")||!e.option("showArrowIcon"))return"content";var r=(0,f.getImageContainer)(a),s=i?(0,o.default)("<span>").text(i).addClass(E):void 0,l=(0,f.getImageContainer)("spindown").addClass(A);(0,o.default)(n).append(r,s,l)}},this._options.cache("buttonGroupOptions"))},_renderPopupContent:function(){var e=this._popup.$content(),t=this._getTemplateByOption("dropDownContentTemplate");return e.empty(),this._popupContentId="dx-"+new x.default,this.setAria("id",this._popupContentId,e),t.render({container:(0,h.getPublicElement)(e),model:this.option("items")||this._dataSource})},_popupOptions:function(){var e=this,t=this.option("rtlEnabled")?"right":"left";return(0,v.extend)({dragEnabled:!1,focusStateEnabled:!1,deferRendering:this.option("deferRendering"),closeOnOutsideClick:function(t){var n=e.$element(),i=(0,o.default)(t.target).closest(".".concat(k));return!i.is(n)},showTitle:!1,animation:{show:{type:"fade",duration:0,from:0,to:1},hide:{type:"fade",duration:400,from:1,to:0}},width:function(){return(0,w.getElementWidth)(e.$element())},height:"auto",shading:!1,position:{of:this.$element(),collision:"flipfit",my:"top "+t,at:"bottom "+t}},this._options.cache("dropDownOptions"),{visible:this.option("opened")})},_listOptions:function(){var e=this,t=this.option("selectedItemKey"),n=this.option("useSelectMode");return{selectionMode:n?"single":"none",wrapItemText:this.option("wrapItemText"),focusStateEnabled:this.option("focusStateEnabled"),hoverStateEnabled:this.option("hoverStateEnabled"),showItemDataTitle:!0,onContentReady:function(){return e._fireContentReadyAction()},selectedItemKeys:t&&n?[t]:[],grouped:this.option("grouped"),groupTemplate:this.option("groupTemplate"),keyExpr:this._getKey(),noDataText:this.option("noDataText"),displayExpr:this.option("displayExpr"),itemTemplate:this.option("itemTemplate"),items:this.option("items"),dataSource:this._dataSource,onItemClick:function(t){e.option("useSelectMode")||(e._lastSelectedItemData=t.itemData),e.option("selectedItemKey",e._keyGetter(t.itemData));var n=e._fireItemClickAction(t);n!==!1&&(e.toggle(!1),e._buttonGroup.focus())}}},_upDownKeyHandler:function(){this._popup&&this._popup.option("visible")&&this._list?this._list.focus():this.open()},_escHandler:function(){this.close(),this._buttonGroup.focus()},_renderPopup:function(){var e=(0,o.default)("<div>");this.$element().append(e),this._popup=this._createComponent(e,u.default,this._popupOptions()),this._popup.$content().addClass(S),this._popup._wrapper().addClass(O),this._popup.on("hiding",this._popupHidingHandler.bind(this)),this._popup.on("showing",this._popupShowingHandler.bind(this)),this._bindInnerWidgetOptions(this._popup,"dropDownOptions")},_popupHidingHandler:function(){this.option("opened",!1),this.setAria({expanded:!1,owns:void 0})},_popupOptionChanged:function(e){var t=a.default.getOptionsFromContainer(e);this._setPopupOption(t);var n=Object.keys(t);n.indexOf("width")===-1&&n.indexOf("height")===-1||this._dimensionChanged()},_dimensionChanged:function(){var e=this,t=(0,w.getSizeValue)(this.option("dropDownOptions.width"));void 0===t&&this._setPopupOption("width",function(){return(0,w.getElementWidth)(e.$element())})},_setPopupOption:function(e,t){this._setWidgetOption("_popup",arguments)},_popupShowingHandler:function(){this.option("opened",!0),this.setAria({expanded:!0,owns:this._popupContentId})},_renderButtonGroup:function(){var e=this._buttonGroup&&this._buttonGroup.$element()||(0,o.default)("<div>");this._buttonGroup||this.$element().append(e),this._buttonGroup=this._createComponent(e,l.default,this._buttonGroupOptions()),this._buttonGroup.registerKeyHandler("downArrow",this._upDownKeyHandler.bind(this)),this._buttonGroup.registerKeyHandler("tab",this.close.bind(this)),this._buttonGroup.registerKeyHandler("upArrow",this._upDownKeyHandler.bind(this)),this._buttonGroup.registerKeyHandler("escape",this._escHandler.bind(this)),this._bindInnerWidgetOptions(this._buttonGroup,"buttonGroupOptions");
},_updateArrowClass:function(){var e=this.option("splitButton")||this.option("showArrowIcon");this.$element().toggleClass(T,e)},toggle:function(e){return this._popup||(this._renderPopup(),this._renderContent()),this._popup.toggle(e)},open:function(){return this.toggle(!0)},close:function(){return this.toggle(!1)},_setListOption:function(e,t){this._list&&this._list.option(e,t)},_getDisplayValue:function(e){var t=!(0,y.isPlainObject)(e),n=t?e:this._displayGetter(e);return(0,y.isPlainObject)(n)?"":String((0,b.ensureDefined)(n,""))},_updateActionButton:function(e){this.option("useSelectMode")&&this.option({text:this._getDisplayValue(e),icon:(0,y.isPlainObject)(e)?e.icon:void 0}),this._setOptionWithoutOptionChange("selectedItem",e),this._setOptionWithoutOptionChange("selectedItemKey",this._keyGetter(e))},_clean:function(){this._list&&this._list.$element().remove(),this._popup&&this._popup.$element().remove()},_selectedItemKeyChanged:function(e){var t=this;this._setListOption("selectedItemKeys",this.option("useSelectMode")&&(0,y.isDefined)(e)?[e]:[]);var n=this.option("selectedItem");this._loadSelectedItem().done(function(e){t._updateActionButton(e),t._displayGetter(n)!==t._displayGetter(e)&&t._fireSelectionChangedAction({previousValue:n,value:e})})},_actionButtonOptionChanged:function(e){var t=e.name,n=e.value,i={};i[t]=n,this._buttonGroup.option("items[0]",(0,v.extend)({},this._actionButtonConfig(),i)),this._popup&&this._popup.repaint()},_selectModeChanged:function(e){if(e){this._setListOption("selectionMode","single");var t=this.option("selectedItemKey");this._setListOption("selectedItemKeys",t?[t]:[])}else this._setListOption("selectionMode","none"),this.option({selectedItemKey:void 0,selectedItem:void 0})},_updateItemCollection:function(e){var t=this,n=this.option("selectedItemKey");this._setListOption("selectedItem",null),this._setWidgetOption("_list",[e]),(0,y.isDefined)(n)&&this._loadSelectedItem().done(function(e){t._setListOption("selectedItemKeys",[n]),t._setListOption("selectedItem",e)}).fail(function(e){t._setListOption("selectedItemKeys",[])}).always(this._updateActionButton.bind(this))},_updateDataSource:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this._dataSource.items();this._dataSource=void 0,this._itemsToDataSource(e),this._updateKeyExpr()},_updateKeyExpr:function(){this._compileKeyGetter(),this._setListOption("keyExpr",this._getKey())},_optionChanged:function(e){var t,n=e.name,i=e.value;switch(n){case"useSelectMode":this._selectModeChanged(i);break;case"splitButton":this._updateArrowClass(),this._renderButtonGroup();break;case"displayExpr":this._compileDisplayGetter(),this._setListOption(n,i),this._updateActionButton(this.option("selectedItem"));break;case"keyExpr":this._updateDataSource();break;case"buttonGroupOptions":this._innerWidgetOptionChanged(this._buttonGroup,e);break;case"dropDownOptions":if("dropDownOptions.visible"===e.fullName)break;void 0!==e.value.visible&&delete e.value.visible,this._popupOptionChanged(e),this._innerWidgetOptionChanged(this._popup,e);break;case"opened":this.toggle(i);break;case"focusStateEnabled":case"hoverStateEnabled":this._setListOption(n,i),this._buttonGroup.option(n,i),this.callBase(e);break;case"items":this._updateDataSource(this.option("items")),this._updateItemCollection(n);break;case"dataSource":Array.isArray(i)?this._updateDataSource(this.option("dataSource")):(this._initDataSource(),this._updateKeyExpr()),this._updateItemCollection(n);break;case"icon":case"text":this._actionButtonOptionChanged(e);break;case"showArrowIcon":this._updateArrowClass(),this._buttonGroup.repaint(),this._popup&&this._popup.repaint();break;case"width":case"height":this.callBase(e),null===(t=this._popup)||void 0===t?void 0:t.repaint();break;case"stylingMode":this._buttonGroup.option(n,i);break;case"itemTemplate":case"grouped":case"noDataText":case"groupTemplate":case"wrapItemText":this._setListOption(n,i);break;case"dropDownContentTemplate":this._renderContent();break;case"selectedItemKey":this._selectedItemKeyChanged(i);break;case"selectedItem":break;case"onItemClick":this._createItemClickAction();break;case"onButtonClick":this._createActionClickAction();break;case"onSelectionChanged":this._createSelectionChangedAction();break;case"deferRendering":this.toggle(this.option("opened"));break;default:this.callBase(e)}}}).include(p.default);(0,s.default)("dxDropDownButton",P);var R=P;t.default=R,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(614)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(30),s=n(0),l=i(n(98)),u=i(n(10)),c=i(n(13)),d=i(n(9)),h=i(n(52)),f=i(n(126)),p=i(n(104)),g=i(n(215)),_=n(21),m="dx-colorview",v="dx-colorview-container",y="dx-colorview-container-row",b="dx-colorview-container-cell",x="dx-colorview-palette",w="dx-colorview-palette-cell",C="dx-colorview-palette-handle",k="dx-colorview-palette-gradient",S="dx-colorview-palette-gradient-white",D="dx-colorview-palette-gradient-black",I="dx-colorview-hue-scale",T="dx-colorview-hue-scale-cell",O="dx-colorview-hue-scale-handle",E="dx-colorview-hue-scale-wrapper",A="dx-colorview-controls-container",P="dx-colorview-label-red",R="dx-colorview-label-green",M="dx-colorview-label-blue",B="dx-colorview-label-hex",V="dx-colorview-alpha-channel-scale",F="dx-colorview-alpha-channel-row",L="dx-colorview-alpha-channel-wrapper",H="dx-colorview-alpha-channel-label",z="dx-colorview-alpha-channel-handle",N="dx-colorview-alpha-channel-cell",j="dx-colorview-alpha-channel-border",W="dx-colorview-color-preview",G="dx-colorview-color-preview-container",q="dx-colorview-color-preview-container-inner",$="dx-colorview-color-preview-color-current",K="dx-colorview-color-preview-color-new",U=h.default.inherit({_supportedKeys:function(){function e(){n._placePaletteHandle(),n._updateColorFromHsv(n._currentColor.hsv.h,n._currentColor.hsv.s,n._currentColor.hsv.v)}var t=this.option("rtlEnabled"),n=this,i=function(e){var t=100/n._paletteWidth;return e.shiftKey&&(t*=n.option("keyStep")),t=t>1?t:1,Math.round(t)},o=function(t){var i=n._currentColor.hsv.s+t;i>100?i=100:i<0&&(i=0),n._currentColor.hsv.s=i,e()},a=function(e){var t=100/n._paletteHeight;return e.shiftKey&&(t*=n.option("keyStep")),t=t>1?t:1,Math.round(t)},l=function(t){var i=n._currentColor.hsv.v+t;i>100?i=100:i<0&&(i=0),n._currentColor.hsv.v=i,e()},u=function(e){var t=360/(n._hueScaleWrapperHeight-n._hueScaleHandleHeight);return e.shiftKey&&(t*=n.option("keyStep")),t=t>1?t:1},c=function(e){n._currentColor.hsv.h+=e,n._placeHueScaleHandle();var t=(0,r.locate)(n._$hueScaleHandle);n._updateColorHue(t.top+n._hueScaleHandleHeight/2)},d=function(e){var i=1/n._alphaChannelScaleWorkWidth;return e.shiftKey&&(i*=n.option("keyStep")),i=i>.01?i:.01,i=t?-i:i},h=function(e){n._currentColor.a+=e,n._placeAlphaChannelHandle();var t=(0,r.locate)(n._$alphaChannelHandle);n._calculateColorTransparencyByScaleWidth(t.left+n._alphaChannelHandleWidth/2)};return(0,s.extend)(this.callBase(),{upArrow:function(e){e.preventDefault(),e.stopPropagation(),e.ctrlKey?this._currentColor.hsv.h<=360&&!this._isTopColorHue&&c(u(e)):this._currentColor.hsv.v<100&&l(a(e))},downArrow:function(e){e.preventDefault(),e.stopPropagation(),e.ctrlKey?this._currentColor.hsv.h>=0&&(this._isTopColorHue&&(this._currentColor.hsv.h=360),c(-u(e))):this._currentColor.hsv.v>0&&l(-a(e))},rightArrow:function(e){e.preventDefault(),e.stopPropagation(),e.ctrlKey?(t?this._currentColor.a<1:this._currentColor.a>0&&this.option("editAlphaChannel"))&&h(-d(e)):this._currentColor.hsv.s<100&&o(i(e))},leftArrow:function(e){e.preventDefault(),e.stopPropagation(),e.ctrlKey?(t?this._currentColor.a>0:this._currentColor.a<1&&this.option("editAlphaChannel"))&&h(d(e)):this._currentColor.hsv.s>0&&o(-i(e))},enter:function(e){this._fireEnterKeyPressed(e)}})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{value:null,matchValue:null,onEnterKeyPressed:void 0,editAlphaChannel:!1,keyStep:1,stylingMode:void 0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===c.default.real().deviceType&&!c.default.isSimulator()},options:{focusStateEnabled:!0}}])},_init:function(){this.callBase(),this._initColorAndOpacity(),this._initEnterKeyPressedAction()},_initEnterKeyPressedAction:function(){this._onEnterKeyPressedAction=this._createActionByOption("onEnterKeyPressed")},_fireEnterKeyPressed:function(e){this._onEnterKeyPressedAction&&this._onEnterKeyPressedAction({event:e})},_initColorAndOpacity:function(){this._setCurrentColor(this.option("value"))},_setCurrentColor:function(e){e=e||"#000000";var t=new l.default(e);t.colorIsInvalid?this.option("value",this._currentColor.baseColor):this._currentColor&&this._makeRgba(this._currentColor)===this._makeRgba(t)||(this._currentColor=t,this._$currentColor&&this._makeTransparentBackground(this._$currentColor,t))},_setBaseColor:function(e){var t=e||"#000000",n=new l.default(t);if(!n.colorIsInvalid){var i=this._makeRgba(this.option("matchValue")!==this._makeRgba(n));i&&this._$baseColor&&this._makeTransparentBackground(this._$baseColor,n)}},_initMarkup:function(){this.callBase(),this.$element().addClass(m),this._renderColorPickerContainer()},_render:function(){this.callBase(),this._renderPalette(),this._renderHueScale(),this._renderControlsContainer(),this._renderControls(),this._renderAlphaChannelElements()},_makeTransparentBackground:function(e,t){t instanceof l.default||(t=new l.default(t)),e.css("backgroundColor",this._makeRgba(t))},_makeRgba:function(e){return e instanceof l.default||(e=new l.default(e)),"rgba("+[e.r,e.g,e.b,e.a].join(", ")+")"},_renderValue:function(){this.callBase(this.option("editAlphaChannel")?this._makeRgba(this._currentColor):this.option("value"))},_renderColorPickerContainer:function(){var e=this.$element();this._$colorPickerContainer=(0,o.default)("<div>").addClass(v).appendTo(e),this._renderHtmlRows()},_renderHtmlRows:function(e){var t=this._$colorPickerContainer.find("."+y),n=t.length,i=this.option("editAlphaChannel")?2:1,a=n-i;if(a>0&&t.eq(-1).remove(),a<0){a=Math.abs(a);var r,s=[];for(r=0;r<a;r++)s.push((0,o.default)("<div>").addClass(y));if(n)for(r=0;r<s.length;r++)t.eq(0).after(s[r]);else this._$colorPickerContainer.append(s)}},_renderHtmlCellInsideRow:function(e,t,n){return(0,o.default)("<div>").addClass(b).addClass(n).appendTo(t.find("."+y).eq(e))},_renderPalette:function(){var e=this._renderHtmlCellInsideRow(0,this._$colorPickerContainer,w),t=(0,o.default)("<div>").addClass([k,S].join(" ")),n=(0,o.default)("<div>").addClass([k,D].join(" "));this._$palette=(0,o.default)("<div>").addClass(x).css("backgroundColor",this._currentColor.getPureColor().toHex()).appendTo(e),this._paletteHeight=this._$palette.height(),this._paletteWidth=this._$palette.width(),this._renderPaletteHandle(),this._$palette.append([t,n])},_renderPaletteHandle:function(){this._$paletteHandle=(0,o.default)("<div>").addClass(C).appendTo(this._$palette),this._createComponent(this._$paletteHandle,g.default,{contentTemplate:null,boundary:this._$palette,allowMoveByClick:!0,boundOffset:function(){return-this._paletteHandleHeight/2}.bind(this),onDragMove:function(){var e=(0,r.locate)(this._$paletteHandle);this._updateByDrag=!0,this._updateColorFromHsv(this._currentColor.hsv.h,this._calculateColorSaturation(e),this._calculateColorValue(e))}.bind(this)}),this._paletteHandleWidth=this._$paletteHandle.width(),this._paletteHandleHeight=this._$paletteHandle.height(),this._placePaletteHandle()},_placePaletteHandle:function(){(0,r.move)(this._$paletteHandle,{left:Math.round(this._paletteWidth*this._currentColor.hsv.s/100-this._paletteHandleWidth/2),top:Math.round(this._paletteHeight-this._paletteHeight*this._currentColor.hsv.v/100-this._paletteHandleHeight/2)})},_calculateColorValue:function(e){var t=Math.floor(e.top+this._paletteHandleHeight/2);return 100-Math.round(100*t/this._paletteHeight)},_calculateColorSaturation:function(e){var t=Math.floor(e.left+this._paletteHandleWidth/2);return Math.round(100*t/this._paletteWidth)},_updateColorFromHsv:function(e,t,n){var i=this._currentColor.a;this._currentColor=new l.default("hsv("+[e,t,n].join(",")+")"),this._currentColor.a=i,this._updateColorParamsAndColorPreview(),this.applyColor()},_renderHueScale:function(){var e=this._renderHtmlCellInsideRow(0,this._$colorPickerContainer,T);this._$hueScaleWrapper=(0,o.default)("<div>").addClass(E).appendTo(e),this._$hueScale=(0,o.default)("<div>").addClass(I).appendTo(this._$hueScaleWrapper),this._hueScaleHeight=this._$hueScale.height(),this._hueScaleWrapperHeight=this._$hueScaleWrapper.outerHeight(),this._renderHueScaleHandle()},_renderHueScaleHandle:function(){this._$hueScaleHandle=(0,o.default)("<div>").addClass(O).appendTo(this._$hueScaleWrapper),this._createComponent(this._$hueScaleHandle,g.default,{contentTemplate:null,boundary:this._$hueScaleWrapper,allowMoveByClick:!0,dragDirection:"vertical",onDragMove:function(){this._updateByDrag=!0,this._updateColorHue((0,r.locate)(this._$hueScaleHandle).top+this._hueScaleHandleHeight/2)}.bind(this)}),this._hueScaleHandleHeight=this._$hueScaleHandle.height(),this._placeHueScaleHandle()},_placeHueScaleHandle:function(){var e=this._hueScaleWrapperHeight,t=this._hueScaleHandleHeight,n=(e-t)*(360-this._currentColor.hsv.h)/360;e<n+t&&(n=e-t),n<0&&(n=0),(0,r.move)(this._$hueScaleHandle,{top:Math.round(n)})},_updateColorHue:function(e){var t=360-Math.round(360*(e-this._hueScaleHandleHeight/2)/(this._hueScaleWrapperHeight-this._hueScaleHandleHeight)),n=this._currentColor.hsv.s,i=this._currentColor.hsv.v;this._isTopColorHue=!1,t=t<0?0:t,t>=360&&(this._isTopColorHue=!0,t=0),this._updateColorFromHsv(t,n,i),this._$palette.css("backgroundColor",this._currentColor.getPureColor().toHex())},_renderControlsContainer:function(){var e=this._renderHtmlCellInsideRow(0,this._$colorPickerContainer);this._$controlsContainer=(0,o.default)("<div>").addClass(A).appendTo(e)},_renderControls:function(){this._renderColorsPreview(),this._renderRgbInputs(),this._renderHexInput()},_renderColorsPreview:function(){var e=(0,o.default)("<div>").addClass(G).appendTo(this._$controlsContainer),t=(0,o.default)("<div>").addClass(q).appendTo(e);this._$currentColor=(0,o.default)("<div>").addClass([W,K].join(" ")),this._$baseColor=(0,o.default)("<div>").addClass([W,$].join(" ")),this._makeTransparentBackground(this._$baseColor,this.option("matchValue")),this._makeTransparentBackground(this._$currentColor,this._currentColor),t.append([this._$baseColor,this._$currentColor])},_renderAlphaChannelElements:function(){this.option("editAlphaChannel")&&(this._$colorPickerContainer.find("."+y).eq(1).addClass(F),this._renderAlphaChannelScale(),this._renderAlphaChannelInput())},_renderRgbInputs:function(){this._rgbInputsWithLabels=[this._renderEditorWithLabel({editorType:f.default,value:this._currentColor.r,onValueChanged:this._updateColor.bind(this,!1),labelText:"R",labelAriaText:u.default.format("dxColorView-ariaRed"),labelClass:P}),this._renderEditorWithLabel({editorType:f.default,value:this._currentColor.g,onValueChanged:this._updateColor.bind(this,!1),labelText:"G",labelAriaText:u.default.format("dxColorView-ariaGreen"),labelClass:R}),this._renderEditorWithLabel({editorType:f.default,value:this._currentColor.b,onValueChanged:this._updateColor.bind(this,!1),labelText:"B",labelAriaText:u.default.format("dxColorView-ariaBlue"),labelClass:M})],this._$controlsContainer.append(this._rgbInputsWithLabels),this._rgbInputs=[this._rgbInputsWithLabels[0].find(".dx-numberbox").dxNumberBox("instance"),this._rgbInputsWithLabels[1].find(".dx-numberbox").dxNumberBox("instance"),this._rgbInputsWithLabels[2].find(".dx-numberbox").dxNumberBox("instance")]},_renderEditorWithLabel:function(e){var t=this,n=(0,o.default)("<div>"),i=(0,o.default)("<label>").addClass(e.labelClass).text(e.labelText+":").append(n);a.default.off(i,_.name),a.default.on(i,_.name,function(e){e.preventDefault()});var r=e.editorType,l=(0,s.extend)({value:e.value,onValueChanged:e.onValueChanged,onKeyboardHandled:function(e){return t._keyboardHandler(e)}},{stylingMode:this.option("stylingMode")});r===f.default&&(l.min=e.min||0,l.max=e.max||255,l.step=e.step||1);var u=new r(n,l);return u.registerKeyHandler("enter",function(e){this._fireEnterKeyPressed(e)}.bind(this)),this.setAria("label",e.labelAriaText,n),i},hexInputOptions:function(){return{editorType:p.default,value:this._currentColor.toHex().replace("#",""),onValueChanged:this._updateColor.bind(this,!0),labelClass:B,labelText:"#",labelAriaText:u.default.format("dxColorView-ariaHex")}},_renderHexInput:function(){this._hexInput=p.default.getInstance(this._renderEditorWithLabel(this.hexInputOptions()).appendTo(this._$controlsContainer).find(".dx-textbox"))},_renderAlphaChannelScale:function(){var e=this._renderHtmlCellInsideRow(1,this._$colorPickerContainer,N),t=(0,o.default)("<div>").addClass(j).appendTo(e),n=(0,o.default)("<div>").addClass(L).appendTo(t);this._$alphaChannelScale=(0,o.default)("<div>").addClass(V).appendTo(n),this._makeCSSLinearGradient(this._$alphaChannelScale),this._renderAlphaChannelHandle(e)},_makeCSSLinearGradient:function(e){var t=this._currentColor,n=[t.r,t.g,t.b].join(","),i=t.toHex().replace("#",""),o=function(e,t){var n=this.option("rtlEnabled"),i="rgba("+e+", "+(n?"1":"0")+")",o="rgba("+e+", "+(n?"0":"1")+")",a="'#"+(n?"00":"")+t+"'",r="'#"+(n?"":"00")+t+"'";return["background-image: -webkit-linear-gradient(180deg, "+i+", "+o+")","background-image: -moz-linear-gradient(-90deg, "+i+", "+o+")","background-image: -o-linear-gradient(-90deg, "+i+", "+o+")","background-image: linear-gradient(-90deg, "+i+", "+o+")","filter: progid:DXImageTransform.Microsoft.gradient(GradientType=1,startColorstr="+a+", endColorstr="+r+")"].join(";")};e.attr("style",o.call(this,n,i))},_renderAlphaChannelInput:function(){var e=this,t=this._renderHtmlCellInsideRow(1,this._$colorPickerContainer);e._alphaChannelInput=this._renderEditorWithLabel({editorType:f.default,value:this._currentColor.a,max:1,step:.1,onValueChanged:function(t){var n=t.value;n=e._currentColor.isValidAlpha(n)?n:e._currentColor.a,e._updateColorTransparency(n),e._placeAlphaChannelHandle()},labelClass:H,labelText:"Alpha",labelAriaText:u.default.format("dxColorView-ariaAlpha")}).appendTo(t).find(".dx-numberbox").dxNumberBox("instance")},_updateColorTransparency:function(e){this._currentColor.a=e,this.applyColor()},_renderAlphaChannelHandle:function(e){this._$alphaChannelHandle=(0,o.default)("<div>").addClass(z).appendTo(e),this._createComponent(this._$alphaChannelHandle,g.default,{contentTemplate:null,boundary:e,allowMoveByClick:!0,dragDirection:"horizontal",onDragMove:function(){this._updateByDrag=!0;var e=this._$alphaChannelHandle,t=(0,r.locate)(e).left+this._alphaChannelHandleWidth/2;this._calculateColorTransparencyByScaleWidth(t)}.bind(this)}),this._alphaChannelHandleWidth=this._$alphaChannelHandle.width(),this._alphaChannelScaleWorkWidth=e.width()-this._alphaChannelHandleWidth,this._placeAlphaChannelHandle()},_calculateColorTransparencyByScaleWidth:function(e){var t=(e-this._alphaChannelHandleWidth/2)/this._alphaChannelScaleWorkWidth,n=this.option("rtlEnabled");t=n?t:1-t,e>=this._alphaChannelScaleWorkWidth+this._alphaChannelHandleWidth/2?t=n?1:0:t<1&&(t=t.toFixed(2)),t=Math.max(t,0),t=Math.min(t,1),this._alphaChannelInput.option("value",t)},_placeAlphaChannelHandle:function(){var e=this._alphaChannelScaleWorkWidth*(1-this._currentColor.a);e<0&&(e=0),this._alphaChannelScaleWorkWidth<e&&(e=this._alphaChannelScaleWorkWidth),(0,r.move)(this._$alphaChannelHandle,{left:this.option("rtlEnabled")?this._alphaChannelScaleWorkWidth-e:e})},applyColor:function(){var e=this.option("editAlphaChannel")?this._makeRgba(this._currentColor):this._currentColor.toHex();this._makeTransparentBackground(this._$currentColor,this._currentColor),this.option("value",e)},cancelColor:function(){this._initColorAndOpacity(),this._refreshMarkup()},_updateColor:function(e,t){var n,i;e?i=this._validateHex("#"+this._hexInput.option("value")):(n=this._validateRgb(),this._alphaChannelInput?(n.push(this._alphaChannelInput.option("value")),i="rgba("+n.join(", ")+")"):i="rgb("+n.join(", ")+")"),this._suppressEditorsValueUpdating||(this._currentColor=new l.default(i),this.applyColor(),this._refreshMarkup())},_validateHex:function(e){return this._currentColor.isValidHex(e)?e:this._currentColor.toHex()},_validateRgb:function(){var e=this._rgbInputs[0].option("value"),t=this._rgbInputs[1].option("value"),n=this._rgbInputs[2].option("value");return this._currentColor.isValidRGB(e,t,n)||(e=this._currentColor.r,t=this._currentColor.g,n=this._currentColor.b),[e,t,n]},_refreshMarkup:function(){this._placeHueScaleHandle(),this._placePaletteHandle(),this._updateColorParamsAndColorPreview(),this._$palette.css("backgroundColor",this._currentColor.getPureColor().toHex()),this._$alphaChannelHandle&&(this._updateColorTransparency(this._currentColor.a),this._placeAlphaChannelHandle())},_updateColorParamsAndColorPreview:function(){this._suppressEditorsValueUpdating=!0,this._hexInput.option("value",this._currentColor.toHex().replace("#","")),this._rgbInputs[0].option("value",this._currentColor.r),this._rgbInputs[1].option("value",this._currentColor.g),this._rgbInputs[2].option("value",this._currentColor.b),this._suppressEditorsValueUpdating=!1,this.option("editAlphaChannel")&&(this._makeCSSLinearGradient.call(this,this._$alphaChannelScale),this._alphaChannelInput.option("value",this._currentColor.a))},_optionChanged:function(e){var t=e.value;switch(e.name){case"value":this._setCurrentColor(t),this._updateByDrag||this._refreshMarkup(),this._updateByDrag=!1,this.callBase(e);break;case"matchValue":this._setBaseColor(t);break;case"onEnterKeyPressed":this._initEnterKeyPressedAction();break;case"editAlphaChannel":this._$colorPickerContainer&&(this._renderHtmlRows("editAlphaChannel"),this._renderAlphaChannelElements());break;case"keyStep":break;case"stylingMode":this._renderControls();break;default:this.callBase(e)}}});(0,d.default)("dxColorView",U);var Y=U;t.default=Y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(290)),a=i(n(218)),r=i(n(20)),s=n(4),l=n(1),u=n(0),c=i(n(10)),d=a.default.inherit({NAME:"Calendar",supportedKeys:function(){var e=function(e){return!!this.option("opened")&&(e.preventDefault(),!0)};return{rightArrow:function(){if(this.option("opened"))return!0},leftArrow:function(){if(this.option("opened"))return!0},enter:function(e){if(this.dateBox.option("opened")){if(e.preventDefault(),this._widget.option("zoomLevel")!==this._widget.option("maxZoomLevel"))return!0;var t=this._widget._view.option("contouredDate"),n=this._lastActionElement;t&&"calendar"===n&&this.dateBoxValue(t,e),this.dateBox.close(),this.dateBox._valueChangeEventHandler(e)}else this.dateBox._valueChangeEventHandler(e)}.bind(this),home:e,end:e}},getDisplayFormat:function(e){return e||"shortdate"},_getWidgetName:function(){return o.default},getKeyboardListener:function(){return this._widget},_getWidgetOptions:function(){var e=this.dateBox.option("disabledDates");return(0,u.extend)(this.dateBox.option("calendarOptions"),{value:this.dateBoxValue()||null,dateSerializationFormat:null,min:this.dateBox.dateOption("min"),max:this.dateBox.dateOption("max"),onValueChanged:this._valueChangedHandler.bind(this),onCellClick:this._cellClickHandler.bind(this),tabIndex:null,disabledDates:(0,l.isFunction)(e)?this._injectComponent(e.bind(this.dateBox)):e,onContouredChanged:this._refreshActiveDescendant.bind(this),hasFocus:function(){return!0}})},_injectComponent:function(e){var t=this;return function(n){return(0,u.extend)(n,{component:t.dateBox}),e(n)}},_refreshActiveDescendant:function(e){this._lastActionElement="calendar",this.dateBox.setAria("activedescendant",e.actionValue)},popupConfig:function(e){var t=e.toolbarItems,n=this.dateBox.option("buttonsLocation"),i=[];return i="default"!==n?(0,s.splitPair)(n):["bottom","center"],"useButtons"===this.dateBox.option("applyValueMode")&&this._isCalendarVisible()&&t.unshift({widget:"dxButton",toolbar:i[0],location:"after"===i[1]?"before":i[1],options:{onInitialized:function(e){e.component.registerKeyHandler("escape",this._escapeHandler.bind(this))}.bind(this),onClick:function(){this._widget._toTodayView()}.bind(this),text:c.default.format("dxCalendar-todayButtonText"),type:"today"}}),(0,u.extend)(!0,e,{toolbarItems:t,position:{collision:"flipfit flip"},width:"auto"})},_isCalendarVisible:function(){return(0,l.isEmptyObject)(this.dateBox.option("calendarOptions"))||this.dateBox.option("calendarOptions.visible")!==!1},_escapeHandler:function(){this.dateBox.close(),this.dateBox.focus()},_valueChangedHandler:function(e){var t=this.dateBox,n=e.value,i=e.previousValue;r.default.sameDate(n,i)||"instantly"===t.option("applyValueMode")&&this.dateBoxValue(this.getValue(),e.event)},_updateValue:function(){this._widget&&this._widget.option("value",this.dateBoxValue())},textChangedHandler:function(){this._lastActionElement="input",this.dateBox.option("opened")&&this._widget&&this._updateValue(!0)},_cellClickHandler:function(e){var t=this.dateBox;"instantly"===t.option("applyValueMode")&&(t.option("opened",!1),this.dateBoxValue(this.getValue(),e.event))}}),h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(35)),l=n(3),u=n(0),c=n(1),d=function(){function e(){o(this,e),this._map={}}return r(e,[{key:"_findWidgetInstance",value:function(e){var t;return(0,l.each)(this._map,function(n,i){var o=i.widgetInstance,a=i.item;if(e(a))return t=o,!1}),t}},{key:"_findFieldByCondition",value:function(e,t){var n;return(0,l.each)(this._map,function(i,o){if(e(o))return n="guid"===t?i:o[t],!1}),n}},{key:"clear",value:function(){this._map={}}},{key:"removeItemsByItems",value:function(e){var t=this;(0,l.each)(e.getItems(),function(e){return t.removeItemByKey(e)})}},{key:"removeItemByKey",value:function(e){delete this._map[e]}},{key:"add",value:function(e){var t=e.guid||new s.default;return this._map[t]=e,t}},{key:"addItemsOrExtendFrom",value:function(e){var t=this;e.each(function(e,n){t._map[e]?(n.widgetInstance&&(t._map[e].widgetInstance=n.widgetInstance),t._map[e].$itemContainer=n.$itemContainer):t.add({item:n.item,widgetInstance:n.widgetInstance,guid:e,$itemContainer:n.$itemContainer})})}},{key:"extendRunTimeItemInfoByKey",value:function(e,t){this._map[e]&&(this._map[e]=(0,u.extend)(this._map[e],t))}},{key:"findWidgetInstanceByItem",value:function(e){return this._findWidgetInstance(function(t){return t===e})}},{key:"getGroupOrTabLayoutManagerByPath",value:function(e){return this._findFieldByCondition(function(t){var n=t.path;return n===e},"layoutManager")}},{key:"getKeyByPath",value:function(e){return this._findFieldByCondition(function(t){var n=t.path;return n===e},"guid")}},{key:"findWidgetInstanceByName",value:function(e){return this._findWidgetInstance(function(t){return e===t.name})}},{key:"findWidgetInstanceByDataField",value:function(e){return this._findWidgetInstance(function(t){return e===((0,c.isString)(t)?t:t.dataField)})}},{key:"findItemContainerByItem",value:function(e){for(var t in this._map)if(this._map[t].item===e)return this._map[t].$itemContainer;return null}},{key:"findItemIndexByItem",value:function(e){return this._findFieldByCondition(function(t){var n=t.item;return n===e},"itemIndex")}},{key:"getItems",value:function(){return this._map}},{key:"each",value:function(e){(0,l.each)(this._map,function(t,n){e(t,n)})}},{key:"removeItemsByPathStartWith",value:function(e){var t=this,n=Object.keys(this._map),i=n.filter(function(n){return t._map[n].path.indexOf(e,0)>-1});i.forEach(function(e){return t.removeItemByKey(e)})}}]),e}();t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(30),r=n(639),s=n(29),l=n(0),u=n(4),c=n(55),d=n(18),h=n(1),f=i(n(13)),p=i(n(9)),g=i(n(214)),_=i(n(179)),m=n(6),v="dx-multiview",y="dx-multiview-wrapper",b="dx-multiview-item-container",x="dx-multiview-item",w="dx-multiview-item-hidden",C="dxMultiViewItemData",k=200,S=function(e){return+e},D=function(e){return(0,a.locate)(e).left},I=g.default.inherit({_activeStateUnit:"."+x,_supportedKeys:function(){return(0,l.extend)(this.callBase(),{pageUp:u.noop,pageDown:u.noop})},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{selectedIndex:0,swipeEnabled:!0,animationEnabled:!0,loop:!1,deferRendering:!0,_itemAttributes:{role:"tabpanel"},loopItemFocus:!1,selectOnFocus:!0,selectionMode:"single",selectionRequired:!0,selectionByClick:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===f.default.real().deviceType&&!f.default.isSimulator()},options:{focusStateEnabled:!0}}])},_itemClass:function(){return x},_itemDataKey:function(){return C},_itemContainer:function(){return this._$itemContainer},_itemElements:function(){return this._itemContainer().children(this._itemSelector())},_itemWidth:function(){return this._itemWidthValue||(this._itemWidthValue=this._$wrapper.width()),this._itemWidthValue},_clearItemWidthCache:function(){delete this._itemWidthValue},_itemsCount:function(){return this.option("items").length},_normalizeIndex:function(e){var t=this._itemsCount();return e<0&&(e+=t),e>=t&&(e-=t),e},_getRTLSignCorrection:function(){return this.option("rtlEnabled")?-1:1},_init:function(){this.callBase.apply(this,arguments);var e=this.$element();e.addClass(v),this._$wrapper=(0,o.default)("<div>").addClass(y),this._$wrapper.appendTo(e),this._$itemContainer=(0,o.default)("<div>").addClass(b),this._$itemContainer.appendTo(this._$wrapper),this.option("loopItemFocus",this.option("loop")),this._initSwipeable()},_initMarkup:function(){this._deferredItems=[],this.callBase();var e=this._getSelectedItemIndices();this._updateItemsVisibility(e[0])},_afterItemElementDeleted:function(e,t){this.callBase(e,t),this._deferredItems&&this._deferredItems.splice(t.itemIndex,1)},_beforeItemElementInserted:function(e){this.callBase.apply(this,arguments),this._deferredItems&&this._deferredItems.splice(e.index,0,null)},_executeItemRenderAction:function(e,t,n){e=(this.option("items")||[]).indexOf(t),this.callBase(e,t,n)},_renderItemContent:function(e){var t=new m.Deferred,n=this,i=this.callBase,o=new m.Deferred;return o.done(function(){var o=i.call(n,e);t.resolve(o)}),this._deferredItems[e.index]=o,this.option("deferRendering")||o.resolve(),t.promise()},_render:function(){var e=this;this.callBase(),(0,u.deferRender)(function(){var t=e._getSelectedItemIndices();e._updateItems(t[0])})},_updateItems:function(e,t){this._updateItemsPosition(e,t),this._updateItemsVisibility(e,t)},_modifyByChanges:function(){this.callBase.apply(this,arguments);var e=this._getSelectedItemIndices();this._updateItemsVisibility(e[0])},_updateItemsPosition:function(e,t){var n=this._itemElements(),i=(0,h.isDefined)(t)?-this._animationDirection(t,e):void 0,o=n.eq(e);r._translator.move(o,0),(0,h.isDefined)(t)&&r._translator.move(n.eq(t),100*i+"%")},_updateItemsVisibility:function(e,t){var n=this._itemElements();n.each(function(n,i){var a=(0,o.default)(i),r=n!==e&&n!==t;r||this._renderSpecificItem(n),a.toggleClass(w,r),this.setAria("hidden",r||void 0,a)}.bind(this))},_renderSpecificItem:function(e){var t=this._itemElements().eq(e),n=t.find(this._itemContentClass()).length>0;(0,h.isDefined)(e)&&!n&&(this._deferredItems[e].resolve(),(0,c.triggerResizeEvent)(t))},_refreshItem:function(e,t){this.callBase(e,t),
this._updateItemsVisibility(this.option("selectedIndex"))},_setAriaSelected:u.noop,_updateSelection:function(e,t){var n=e[0],i=t[0];r.animation.complete(this._$itemContainer),this._updateItems(i,n);var o=this._animationDirection(n,i);this._animateItemContainer(o*this._itemWidth(),function(){r._translator.move(this._$itemContainer,0),this._updateItems(n),this._$itemContainer.width()}.bind(this))},_animateItemContainer:function(e,t){var n=this.option("animationEnabled")?k:0;r.animation.moveTo(this._$itemContainer,e,n,t)},_animationDirection:function(e,t){var n=D(this._$itemContainer),i=(t-e)*this._getRTLSignCorrection()*this._getItemFocusLoopSignCorrection(),o=0!==n,a=o?n:i;return(0,s.sign)(a)},_getSwipeDisabledState:function(){return!this.option("swipeEnabled")||this._itemsCount()<=1},_initSwipeable:function(){var e=this;this._createComponent(this.$element(),_.default,{disabled:this._getSwipeDisabledState(),elastic:!1,itemSizeFunc:this._itemWidth.bind(this),onStart:function(t){return e._swipeStartHandler(t.event)},onUpdated:function(t){return e._swipeUpdateHandler(t.event)},onEnd:function(t){return e._swipeEndHandler(t.event)}})},_swipeStartHandler:function(e){r.animation.complete(this._$itemContainer);var t=this.option("selectedIndex"),n=this.option("loop"),i=this._itemsCount()-1,o=this.option("rtlEnabled");e.maxLeftOffset=S(n||(o?t>0:t<i)),e.maxRightOffset=S(n||(o?t<i:t>0)),this._swipeDirection=null},_swipeUpdateHandler:function(e){var t=e.offset,n=(0,s.sign)(t)*this._getRTLSignCorrection();if(r._translator.move(this._$itemContainer,t*this._itemWidth()),n!==this._swipeDirection){this._swipeDirection=n;var i=this.option("selectedIndex"),o=this._normalizeIndex(i-n);this._updateItems(i,o)}},_swipeEndHandler:function(e){var t=e.targetOffset*this._getRTLSignCorrection();if(t){this.option("selectedIndex",this._normalizeIndex(this.option("selectedIndex")-t));var n=this.itemElements().filter(".dx-item-selected");this.option("focusStateEnabled")&&this.option("focusedElement",(0,d.getPublicElement)(n))}else this._animateItemContainer(0,u.noop)},_getItemFocusLoopSignCorrection:function(){return this._itemFocusLooped?-1:1},_moveFocus:function(){this.callBase.apply(this,arguments),this._itemFocusLooped=!1},_prevItem:function(e){var t=this.callBase.apply(this,arguments);return this._itemFocusLooped=t.is(e.last()),t},_nextItem:function(e){var t=this.callBase.apply(this,arguments);return this._itemFocusLooped=t.is(e.first()),t},_dimensionChanged:function(){this._clearItemWidthCache()},_visibilityChanged:function(e){e&&this._dimensionChanged()},_updateSwipeDisabledState:function(){var e=this._getSwipeDisabledState();_.default.getInstance(this.$element()).option("disabled",e)},_optionChanged:function(e){var t=e.value;switch(e.name){case"loop":this.option("loopItemFocus",t);break;case"animationEnabled":break;case"swipeEnabled":this._updateSwipeDisabledState();break;case"deferRendering":this._invalidate();break;case"items":this._updateSwipeDisabledState(),this.callBase(e);break;default:this.callBase(e)}}});(0,p.default)("dxMultiView",I);var T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(141)),r="dx-tabs-item-badge",s="dx-badge",l=a.default.inherit({_renderWatchers:function(){this.callBase(),this._startWatcher("badge",this._renderBadge.bind(this))},_renderBadge:function(e){if(this._$element.children("."+s).remove(),e){var t=(0,o.default)("<div>").addClass(r).addClass(s).text(e);this._$element.append(t)}}}),u=l;t.default=u,e.exports=t.default},function(e,t,n){t.TABS_EXPANDED_CLASS=void 0;var i="dx-tabs-expanded";t.TABS_EXPANDED_CLASS=i},function(e,t,n){function i(e){return s(e)||r(e)||a(e)||o()}function o(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a(e,t){if(e){if("string"==typeof e)return l(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?l(e,t):void 0}}function r(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function s(e){if(Array.isArray(e))return l(e)}function l(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.getItemPath=t.isFullPathContainsTabs=t.tryGetTabPath=t.getOptionNameFromFullName=t.getFullOptionName=t.isExpectedItem=t.getTextWithoutSpaces=t.concatPaths=t.createItemPathByIndex=void 0;var u=n(1),c=function(e,t){return"".concat(t?"tabs":"items","[").concat(e,"]")};t.createItemPathByIndex=c;var d=function(e,t){return(0,u.isDefined)(e)&&(0,u.isDefined)(t)?"".concat(e,".").concat(t):e||t};t.concatPaths=d;var h=function(e){return e?e.replace(/\s/g,""):void 0};t.getTextWithoutSpaces=h;var f=function(e,t){return e&&(e.dataField===t||e.name===t||h(e.title)===t||"group"===e.itemType&&h(e.caption)===t)};t.isExpectedItem=f;var p=function(e,t){return"".concat(e,".").concat(t)};t.getFullOptionName=p;var g=function(e){var t=e.split(".");return t[t.length-1].replace(/\[\d+]/,"")};t.getOptionNameFromFullName=g;var _=function(e){for(var t=e.split("."),n=i(t),o=t.length-1;o>=0;o--){if(m(t[o]))return n.join(".");n.splice(o,1)}return""};t.tryGetTabPath=_;var m=function(e){return e.indexOf("tabs")>-1};t.isFullPathContainsTabs=m;var v=function e(t,n,i){var o=t.indexOf(n);if(o>-1)return c(o,i);for(var a=0;a<t.length;a++){var r=t[a],s=r.tabs||r.items;if(s){var l=e(s,n,r.tabs);if(l)return d(c(a,i),l)}}};t.getItemPath=v},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=n(1),l=i(n(17)),u=n(7),c=n(3),d=n(0),h=i(n(9)),f=i(n(216)),p=i(n(58)),g=(0,u.getWindow)(),_="dx-responsivebox",m=_+"-screen-",v="dx-box-item",y="dxBoxItemData",b=1920,x=p.default.inherit({_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{rows:[],cols:[],screenByWidth:null,singleColumnScreen:"",height:"100%",width:"100%",activeStateEnabled:!1,focusStateEnabled:!1,onItemStateChanged:void 0,onLayoutChanged:null,currentScreenFactor:void 0,_layoutStrategy:void 0})},_init:function(){this.option("screenByWidth")||this._options.silent("screenByWidth",u.defaultScreenFactorFunc),this.callBase(),this._initLayoutChangedAction()},_initLayoutChangedAction:function(){this._layoutChangedAction=this._createActionByOption("onLayoutChanged",{excludeValidators:["disabled","readonly"]})},_itemClass:function(){return v},_itemDataKey:function(){return y},_initMarkup:function(){this.callBase(),this.$element().addClass(_),this._updateRootBox()},_updateRootBox:function(){clearTimeout(this._updateTimer),this._updateTimer=setTimeout(function(){this._$root&&a.default.triggerHandler(this._$root,"dxupdate")}.bind(this))},_renderItems:function(){this._setScreenSize(),this._screenItems=this._itemsByScreen(),this._prepareGrid(),this._spreadItems(),this._layoutItems(),this._linkNodeToItem()},_itemOptionChanged:function(e){var t=this._findItemElementByItem(e);t.length&&(this._refreshItem(t,e),this._clearItemNodeTemplates(),this._update(!0))},_setScreenSize:function(){var e=this._getCurrentScreen();this._removeScreenSizeClass(),this.$element().addClass(m+e),this.option("currentScreenFactor",e)},_removeScreenSizeClass:function(){var e=this.option("currentScreenFactor");e&&this.$element().removeClass(m+e)},_prepareGrid:function(){var e=this._grid=[];this._prepareRowsAndCols(),(0,c.each)(this._rows,function(){var t=[];e.push(t),(0,c.each)(this._cols,function(){t.push(this._createEmptyCell())}.bind(this))}.bind(this))},getSingleColumnRows:function(){var e=this.option("rows"),t=this._screenItems.length;if(e.length){for(var n=this._filterByScreen(e),i=[],o=0;o<t;o++){var a=this._defaultSizeConfig();o<n.length&&(0,s.isDefined)(n[o].shrink)&&(a.shrink=n[o].shrink),i.push(a)}return i}return this._defaultSizeConfig(t)},_prepareRowsAndCols:function(){this._isSingleColumnScreen()?(this._prepareSingleColumnScreenItems(),this._rows=this.getSingleColumnRows(),this._cols=this._defaultSizeConfig(1)):(this._rows=this._sizesByScreen(this.option("rows")),this._cols=this._sizesByScreen(this.option("cols")))},_isSingleColumnScreen:function(){return this._screenRegExp().test(this.option("singleColumnScreen"))||!this.option("rows").length||!this.option("cols").length},_prepareSingleColumnScreenItems:function(){this._screenItems.sort(function(e,t){return e.location.row-t.location.row||e.location.col-t.location.col}),(0,c.each)(this._screenItems,function(e,t){(0,d.extend)(t.location,{row:e,col:0,rowspan:1,colspan:1})})},_sizesByScreen:function(e){return(0,c.map)(this._filterByScreen(e),function(e){return(0,d.extend)(this._defaultSizeConfig(),e)}.bind(this))},_createDefaultSizeConfig:function(){return{ratio:1,baseSize:0,minSize:0,maxSize:0}},_defaultSizeConfig:function(e){var t=this._createDefaultSizeConfig();if(!arguments.length)return t;for(var n=[],i=0;i<e;i++)n.push(t);return n},_filterByScreen:function(e){var t=this._screenRegExp();return(0,r.grep)(e,function(e){return!e.screen||t.test(e.screen)})},_screenRegExp:function(){var e=this._getCurrentScreen();return new RegExp("(^|\\s)"+e+"($|\\s)","i")},_getCurrentScreen:function(){var e=this._screenWidth();return this.option("screenByWidth")(e)},_screenWidth:function(){return(0,u.hasWindow)()?(0,o.default)(g).width():b},_createEmptyCell:function(){return{item:{},location:{colspan:1,rowspan:1}}},_spreadItems:function(){(0,c.each)(this._screenItems,function(e,t){var n=t.location||{},i=n.col,o=n.row,a=this._grid[o],r=a&&a[i];this._occupyCells(r,t)}.bind(this))},_itemsByScreen:function(){var e=this;return this.option("items").reduce(function(t,n){var i=n.location||{};return i=(0,s.isPlainObject)(i)?[i]:i,e._filterByScreen(i).forEach(function(e){t.push({item:n,location:(0,d.extend)({rowspan:1,colspan:1},e)})}),t},[])},_occupyCells:function(e,t){e&&!this._isItemCellOccupied(e,t)&&((0,d.extend)(e,t),this._markSpanningCell(e))},_isItemCellOccupied:function(e,t){if(!(0,s.isEmptyObject)(e.item))return!0;var n=!1;return this._loopOverSpanning(t.location,function(e){n=n||!(0,s.isEmptyObject)(e.item)}),n},_loopOverSpanning:function(e,t){var n=e.row+e.rowspan-1,i=e.col+e.colspan-1,o=Math.min(n,this._rows.length-1),a=Math.min(i,this._cols.length-1);e.rowspan-=n-o,e.colspan-=i-a;for(var r=e.row;r<=o;r++)for(var s=e.col;s<=a;s++)r===e.row&&s===e.col||t(this._grid[r][s])},_markSpanningCell:function(e){this._loopOverSpanning(e.location,function(t){(0,d.extend)(t,{item:e.item,spanningCell:e})})},_linkNodeToItem:function(){(0,c.each)(this._itemElements(),function(e,t){var n=(0,o.default)(t),i=n.data(y);i.box||(i.node=n.children())})},_layoutItems:function(){var e=this._grid.length,t=e&&this._grid[0].length;if(e||t){var n=this._layoutBlock({direction:"col",row:{start:0,end:e-1},col:{start:0,end:t-1}}),i=this._prepareBoxConfig(n.box||{direction:"row",items:[(0,d.extend)(n,{ratio:1})]});(0,d.extend)(i,this._rootBoxConfig(i.items)),this._$root=(0,o.default)("<div>").appendTo(this._itemContainer()),this._createComponent(this._$root,f.default,i)}},_rootBoxConfig:function(e){var t=(0,c.each)(e,function(e,t){this._needApplyAutoBaseSize(t)&&(0,d.extend)(t,{baseSize:"auto"})}.bind(this));return(0,d.extend)({width:"100%",height:"100%",items:t,itemTemplate:this._getTemplateByOption("itemTemplate"),itemHoldTimeout:this.option("itemHoldTimeout"),onItemHold:this._createActionByOption("onItemHold"),onItemClick:this._createActionByOption("onItemClick"),onItemContextMenu:this._createActionByOption("onItemContextMenu"),onItemRendered:this._createActionByOption("onItemRendered")},{_layoutStrategy:this.option("_layoutStrategy")})},_needApplyAutoBaseSize:function(e){return!(e.baseSize||e.minSize&&"auto"!==e.minSize||e.maxSize&&"auto"!==e.maxSize)},_prepareBoxConfig:function(e){return(0,d.extend)(e||{},{crossAlign:"stretch",onItemStateChanged:this.option("onItemStateChanged")})},_layoutBlock:function(e){return this._isSingleItem(e)?this._itemByCell(e.row.start,e.col.start):this._layoutDirection(e)},_isSingleItem:function(e){var t=this._grid[e.row.start][e.col.start].location,n=e.row.end-e.row.start===t.rowspan-1,i=e.col.end-e.col.start===t.colspan-1;return n&&i},_itemByCell:function(e,t){var n=this._grid[e][t];return n.spanningCell?null:n.item},_layoutDirection:function(e){for(var t,n=[],i=e.direction,o=this._crossDirection(i);t=this._nextBlock(e);){if(this._isBlockIndivisible(e.prevBlockOptions,t))throw l.default.Error("E1025");var a=this._layoutBlock({direction:o,row:t.row,col:t.col,prevBlockOptions:e});a&&((0,d.extend)(a,this._blockSize(t,o)),n.push(a)),e[o].start=t[o].end+1}return{box:this._prepareBoxConfig({direction:i,items:n})}},_isBlockIndivisible:function(e,t){return e&&e.col.start===t.col.start&&e.col.end===t.col.end&&e.row.start===t.row.start&&e.row.end===t.row.end},_crossDirection:function(e){return"col"===e?"row":"col"},_nextBlock:function(e){var t=e.direction,n=this._crossDirection(t),i=e[t].start,o=e[t].end,a=e[n].start;if(a>e[n].end)return null;for(var r=1,s=a;s<a+r;s++){for(var l=1,u=i;u<=o;u++){var c=this._cellByDirection(t,u,s);l=Math.max(l,c.location[n+"span"])}var d=s+l,h=a+r;d>h&&(r+=d-h)}var f={};return f[t]={start:i,end:o},f[n]={start:a,end:a+r-1},f},_cellByDirection:function(e,t,n){return"col"===e?this._grid[n][t]:this._grid[t][n]},_blockSize:function(e,t){for(var n="row"===t?"auto":0,i="row"===t?this._rows:this._cols,o=(0,d.extend)(this._createDefaultSizeConfig(),{ratio:0}),a=e[t].start;a<=e[t].end;a++){var r=i[a];o.ratio+=r.ratio,o.baseSize+=r.baseSize,o.minSize+=r.minSize,o.maxSize+=r.maxSize,(0,s.isDefined)(r.shrink)&&(o.shrink=r.shrink)}return o.minSize=o.minSize?o.minSize:n,o.maxSize=o.maxSize?o.maxSize:"auto",this._isSingleColumnScreen()&&(o.baseSize="auto"),o},_update:function(e){var t=this._$root;this._renderItems(),t&&(e?t.remove():(t.detach(),this._saveAssistantRoot(t))),this._layoutChangedAction(),this._updateRootBox()},_saveAssistantRoot:function(e){this._assistantRoots=this._assistantRoots||[],this._assistantRoots.push(e)},_dispose:function(){clearTimeout(this._updateTimer),this._clearItemNodeTemplates(),this._cleanUnusedRoots(),this.callBase.apply(this,arguments)},_cleanUnusedRoots:function(){this._assistantRoots&&(0,c.each)(this._assistantRoots,function(e,t){(0,o.default)(t).remove()})},_clearItemNodeTemplates:function(){(0,c.each)(this.option("items"),function(){delete this.node})},_toggleVisibility:function(e){this.callBase(e),e&&this._updateRootBox()},_attachClickEvent:r.noop,_optionChanged:function(e){switch(e.name){case"rows":case"cols":case"screenByWidth":case"_layoutStrategy":case"singleColumnScreen":this._clearItemNodeTemplates(),this._invalidate();break;case"width":case"height":this.callBase(e),this._update();break;case"onLayoutChanged":this._initLayoutChangedAction();break;case"itemTemplate":this._clearItemNodeTemplates(),this.callBase(e);break;case"currentScreenFactor":break;default:this.callBase(e)}},_dimensionChanged:function(){this._getCurrentScreen()!==this.option("currentScreenFactor")&&this._update()},repaint:function(){this._update()}});(0,h.default)("dxResponsiveBox",x);var w=x;t.default=w,e.exports=t.default},function(e,t,n){t.FORM_VALIDATION_SUMMARY=t.GROUP_COL_COUNT_ATTR=t.GROUP_COL_COUNT_CLASS=t.FIELD_ITEM_TAB_CLASS=t.HIDDEN_LABEL_CLASS=t.FORM_GROUP_WITH_CAPTION_CLASS=t.FIELD_ITEM_CONTENT_HAS_TABS_CLASS=t.FIELD_ITEM_CONTENT_HAS_GROUP_CLASS=t.FORM_GROUP_CONTENT_CLASS=t.ROOT_SIMPLE_ITEM_CLASS=t.SINGLE_COLUMN_ITEM_CONTENT=t.FIELD_BUTTON_ITEM_CLASS=t.FIELD_EMPTY_ITEM_CLASS=t.FIELD_ITEM_CONTENT_CLASS=t.FIELD_ITEM_CONTENT_LOCATION_CLASS=t.FIELD_ITEM_LABEL_CLASS=t.FIELD_ITEM_LABEL_ALIGN_CLASS=t.FIELD_ITEM_LABEL_LOCATION_CLASS=t.LABEL_HORIZONTAL_ALIGNMENT_CLASS=t.LABEL_VERTICAL_ALIGNMENT_CLASS=t.FORM_LAYOUT_MANAGER_CLASS=t.FIELD_ITEM_CONTENT_WRAPPER_CLASS=t.FIELD_ITEM_HELP_TEXT_CLASS=t.FIELD_ITEM_LABEL_CONTENT_CLASS=t.FIELD_ITEM_LABEL_TEXT_CLASS=t.FIELD_ITEM_REQUIRED_CLASS=t.FIELD_ITEM_OPTIONAL_CLASS=t.FIELD_ITEM_REQUIRED_MARK_CLASS=t.FIELD_ITEM_OPTIONAL_MARK_CLASS=t.LAYOUT_MANAGER_ONE_COLUMN=t.FLEX_LAYOUT_CLASS=t.FIELD_ITEM_CLASS=t.FORM_FIELD_ITEM_COL_CLASS=t.FORM_GROUP_CAPTION_CLASS=t.FORM_GROUP_CLASS=t.FORM_CLASS=void 0;var i="dx-form";t.FORM_CLASS=i;var o="dx-form-group";t.FORM_GROUP_CLASS=o;var a="dx-form-group-caption";t.FORM_GROUP_CAPTION_CLASS=a;var r="dx-col-";t.FORM_FIELD_ITEM_COL_CLASS=r;var s="dx-field-item";t.FIELD_ITEM_CLASS=s;var l="dx-flex-layout";t.FLEX_LAYOUT_CLASS=l;var u="dx-layout-manager-one-col";t.LAYOUT_MANAGER_ONE_COLUMN=u;var c="dx-field-item-optional-mark";t.FIELD_ITEM_OPTIONAL_MARK_CLASS=c;var d="dx-field-item-required-mark";t.FIELD_ITEM_REQUIRED_MARK_CLASS=d;var h="dx-field-item-optional";t.FIELD_ITEM_OPTIONAL_CLASS=h;var f="dx-field-item-required";t.FIELD_ITEM_REQUIRED_CLASS=f;var p="dx-field-item-label-text";t.FIELD_ITEM_LABEL_TEXT_CLASS=p;var g="dx-field-item-label-content";t.FIELD_ITEM_LABEL_CONTENT_CLASS=g;var _="dx-field-item-help-text";t.FIELD_ITEM_HELP_TEXT_CLASS=_;var m="dx-field-item-content-wrapper";t.FIELD_ITEM_CONTENT_WRAPPER_CLASS=m;var v="dx-layout-manager";t.FORM_LAYOUT_MANAGER_CLASS=v;var y="dx-label-v-align";t.LABEL_VERTICAL_ALIGNMENT_CLASS=y;var b="dx-label-h-align";t.LABEL_HORIZONTAL_ALIGNMENT_CLASS=b;var x="dx-field-item-label-location-";t.FIELD_ITEM_LABEL_LOCATION_CLASS=x;var w="dx-field-item-label-align";t.FIELD_ITEM_LABEL_ALIGN_CLASS=w;var C="dx-field-item-label";t.FIELD_ITEM_LABEL_CLASS=C;var k="dx-field-item-content-location-";t.FIELD_ITEM_CONTENT_LOCATION_CLASS=k;var S="dx-field-item-content";t.FIELD_ITEM_CONTENT_CLASS=S;var D="dx-field-empty-item";t.FIELD_EMPTY_ITEM_CLASS=D;var I="dx-field-button-item";t.FIELD_BUTTON_ITEM_CLASS=I;var T="dx-single-column-item-content";t.SINGLE_COLUMN_ITEM_CONTENT=T;var O="dx-root-simple-item";t.ROOT_SIMPLE_ITEM_CLASS=O;var E="dx-form-group-content";t.FORM_GROUP_CONTENT_CLASS=E;var A="dx-field-item-has-group";t.FIELD_ITEM_CONTENT_HAS_GROUP_CLASS=A;var P="dx-field-item-has-tabs";t.FIELD_ITEM_CONTENT_HAS_TABS_CLASS=P;var R="dx-form-group-with-caption";t.FORM_GROUP_WITH_CAPTION_CLASS=R;var M="dx-layout-manager-hidden-label";t.HIDDEN_LABEL_CLASS=M;var B="dx-field-item-tab";t.FIELD_ITEM_TAB_CLASS=B;var V="dx-group-colcount-";t.GROUP_COL_COUNT_CLASS=V;var F="group-col-count";t.GROUP_COL_COUNT_ATTR=F;var L="dx-form-validation-summary";t.FORM_VALIDATION_SUMMARY=L},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(9)),y=o(n(65)),b=o(n(298)),x=o(n(100)),w=o(n(221)),C="dx-validationgroup",k="dx-validator",S="dx-validationsummary",D=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return l(_(t.prototype),"_getDefaultOptions",this).call(this)}},{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),x.default.addGroup(this)}},{key:"_initMarkup",value:function(){var e=this.$element();e.addClass(C),e.find(".".concat(k)).each(function(e,t){w.default.getInstance((0,m.default)(t))._initGroupRegistration()}),e.find(".".concat(S)).each(function(e,t){b.default.getInstance((0,m.default)(t))._initGroupRegistration()}),l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"validate",value:function(){return x.default.validateGroup(this)}},{key:"reset",value:function(){return x.default.resetGroup(this)}},{key:"_dispose",value:function(){x.default.removeGroup(this),this.$element().removeClass(C),l(_(t.prototype),"_dispose",this).call(this)}},{key:"_useTemplates",value:function(){return!1}}]),t}(y.default);(0,v.default)("dxValidationGroup",D);var I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){if(!r.default)throw a.default.Error("E1041","Quill");return r.default}t.getQuill=o;var a=i(n(17)),r=i(n(63))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=n(3),s=n(19),l=i(n(15)),u=i(n(38)),c=l.default.abstract,d="dx-toolbar-menu-container",h="dx-toolbar-menu-button",f=l.default.inherit({ctor:function(e){this._toolbar=e},render:function(){this._renderMenuButton(),this._renderWidget()},_widgetOptions:function(){var e=this._toolbar._createActionByOption("onItemClick");return{itemTemplate:this._getMenuItemTemplate.bind(this),onItemClick:function(t){this._toggleMenu(!1,!0),e(t)}.bind(this)}},_getMenuItemTemplate:function(){return this._toolbar._getTemplateByOption("menuItemTemplate")},_renderWidget:function(){var e=(0,o.default)("<div>").appendTo(this._menuContainer());this._menu=this._toolbar._createComponent(e,this._menuWidgetClass(),this._widgetOptions()),this.renderMenuItems()},_menuContainer:c,_menuWidgetClass:c,_hasVisibleMenuItems:function(e){var t=e||this._toolbar.option("items"),n=!1,i=(0,s.compileGetter)("visible"),o=(0,s.compileGetter)("locateInMenu");return(0,r.each)(t,function(e,t){var a=i(t,{functionsAsIs:!0}),r=o(t,{functionsAsIs:!0});(a===!1||"auto"!==r&&"always"!==r)&&"menu"!==t.location||(n=!0)}),n},_getMenuItems:function(){return this._toolbar._getMenuItems()},_updateMenuVisibility:a.noop,_renderMenuButton:function(){var e=this._menuButtonOptions();this._renderMenuButtonContainer(),this._$button=(0,o.default)("<div>").appendTo(this._$menuButtonContainer).addClass(h),this._toolbar._createComponent(this._$button,u.default,e)},_menuButtonOptions:function(){return{onClick:this._menuButtonClickHandler.bind(this)}},_menuButtonClickHandler:function(){this._toggleMenu(!this._menuShown,!0)},_renderMenuButtonContainer:function(){var e=this._toolbar._$afterSection;this._$menuButtonContainer=(0,o.default)("<div>").appendTo(e).addClass(this._toolbar._buttonClass()).addClass(d)},renderMenuItems:function(){this._menu&&this._menu.option("items",this._getMenuItems())},toggleMenuVisibility:function(e,t){this._menu&&this._toggleMenu(e,t)},_toggleMenu:function(e){this._menuShown=e},getMenuWidget:function(){return this._menu},widgetOption:function(e,t){this._menu&&this._menu.option(e,t)},handleToolbarVisibilityChange:a.noop}),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(63)),_=o(n(2)),m=n(0),v=n(7),y=o(n(48)),b=o(n(115)),x={};if(g.default){var w="dx-suggestion-list",C="dx-suggestion-list-wrapper",k=g.default.import("core/module"),S=100;x=function(e){function t(e,i){var o;return a(this,t),o=n.call(this,e,i),o.options=(0,m.extend)({},o._getDefaultOptions(),i),o._popup=o.renderPopup(),o._popup._wrapper().addClass(C),o}l(t,e);var n=c(t);return s(t,[{key:"_getDefaultOptions",value:function(){return{dataSource:null}}}]),s(t,[{key:"renderList",value:function(e,t){var n=(0,_.default)("<div>").addClass(w).appendTo(e);this._list=this.options.editorInstance._createComponent(n,b.default,t)}},{key:"renderPopup",value:function(){var e=this.options.editorInstance,t=(0,_.default)("<div>").appendTo(e.$element()),n=this._getPopupConfig();return e._createComponent(t,y.default,n)}},{key:"_getPopupConfig",value:function(){var e=this;return{contentTemplate:function(t){var n=e._getListConfig(e.options);e.renderList((0,_.default)(t),n)},deferRendering:!1,onShown:function(){e._list.focus()},onHidden:function(){e._list.unselectAll(),e._list.option("focusedElement",null)},showTitle:!1,width:"auto",height:"auto",shading:!1,closeOnTargetScroll:!0,closeOnOutsideClick:!0,animation:{show:{type:"fade",duration:0,from:0,to:1},hide:{type:"fade",duration:400,from:1,to:0}},fullScreen:!1,maxHeight:this.maxHeight}}},{key:"_getListConfig",value:function(e){return{dataSource:e.dataSource,onSelectionChanged:this.selectionChangedHandler.bind(this),selectionMode:"single",pageLoadMode:"scrollBottom"}}},{key:"selectionChangedHandler",value:function(e){this._popup.option("visible")&&(this._popup.hide(),this.insertEmbedContent(e))}},{key:"insertEmbedContent",value:function(e){}},{key:"showPopup",value:function(){this._popup&&this._popup.show()}},{key:"savePosition",value:function(e){this.caretPosition=e}},{key:"getPosition",value:function(){return this.caretPosition}},{key:"maxHeight",get:function(){var e=(0,v.getWindow)(),t=e&&(0,_.default)(e).height()||0;return Math.max(S,.5*t)}}]),t}(k)}var D=x;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(77)),a=i(n(15)),r=n(3),s=n(1),l=n(8),u=a.default.abstract,c=a.default.inherit({_defaultRouteWeight:function(){return 5},_defaultRouteOpacity:function(){return.5},_defaultRouteColor:function(){return"#0000FF"},ctor:function(e,t){this._mapWidget=e,this._$container=t},render:function(e,t){return this._renderImpl().then(function(){return o.default.all([this._applyFunctionIfNeeded("addMarkers",e),this._applyFunctionIfNeeded("addRoutes",t)]).then(function(){return!0})}.bind(this))},_renderImpl:u,updateDimensions:u,updateMapType:u,updateBounds:u,updateCenter:u,updateZoom:u,updateControls:u,updateMarkers:function(e,t){return new o.default(function(n){return this._applyFunctionIfNeeded("removeMarkers",e).then(function(e){this._applyFunctionIfNeeded("addMarkers",t).then(function(t){n(t?t:e)})}.bind(this))}.bind(this))},addMarkers:u,removeMarkers:u,adjustViewport:u,updateRoutes:function(e,t){return new o.default(function(n){return this._applyFunctionIfNeeded("removeRoutes",e).then(function(e){this._applyFunctionIfNeeded("addRoutes",t).then(function(t){n(t?t:e)})}.bind(this))}.bind(this))},addRoutes:u,removeRoutes:u,clean:u,map:function(){return this._map},isEventsCanceled:function(){return!1},_option:function(e,t){return void 0===t?this._mapWidget.option(e):void this._mapWidget.setOptionSilent(e,t)},_keyOption:function(e){var t=this._option("apiKey");return void 0===t[e]?t:t[e]},_parseTooltipOptions:function(e){return{text:e.text||e,visible:e.isShown||!1}},_getLatLng:function(e){if("string"==typeof e){var t=(0,r.map)(e.split(","),function(e){return e.trim()}),n=/^[-+]?[0-9]*\.?[0-9]*$/;if(2===t.length&&t[0].match(n)&&t[1].match(n))return{lat:parseFloat(t[0]),lng:parseFloat(t[1])}}else{if(Array.isArray(e)&&2===e.length)return{lat:e[0],lng:e[1]};if((0,s.isPlainObject)(e)&&(0,s.isNumeric)(e.lat)&&(0,s.isNumeric)(e.lng))return e}return null},_areBoundsSet:function(){return this._option("bounds.northEast")&&this._option("bounds.southWest")},_addEventNamespace:function(e){return(0,l.addNamespace)(e,this._mapWidget.NAME)},_applyFunctionIfNeeded:function(e,t){return t.length?this[e](t):o.default.resolve()},_fireAction:function(e,t){this._mapWidget._createActionByOption(e)(t)},_fireClickAction:function(e){this._fireAction("onClick",e)},_fireMarkerAddedAction:function(e){this._fireAction("onMarkerAdded",e)},_fireMarkerRemovedAction:function(e){this._fireAction("onMarkerRemoved",e)},_fireRouteAddedAction:function(e){this._fireAction("onRouteAdded",e)},_fireRouteRemovedAction:function(e){this._fireAction("onRouteRemoved",e)}}),d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(77)),a=n(0),r=n(3),s=i(n(448)),l=s.default.abstract,u=s.default.inherit({_geocodeLocation:function(e){return new o.default(function(t){var n=this._geocodedLocations,i=n[e];i?t(i):this._geocodeLocationImpl(e).then(function(i){n[e]=i,t(i)})}.bind(this))},_renderImpl:function(){return this._load().then(function(){return this._init()}.bind(this)).then(function(){return o.default.all([this.updateMapType(),this._areBoundsSet()?this.updateBounds():this.updateCenter()])}.bind(this)).then(function(){return this._attachHandlers(),new o.default(function(e){var t=setTimeout(function(){clearTimeout(t),e()})})}.bind(this))},_load:function(){return this._mapsLoader||(this._mapsLoader=this._loadImpl()),this._markers=[],this._routes=[],this._mapsLoader},_loadImpl:l,_init:l,_attachHandlers:l,addMarkers:function(e){return o.default.all((0,r.map)(e,function(e){return this._addMarker(e)}.bind(this))).then(function(e){return this._fitBounds(),[!1,(0,r.map)(e,function(e){return e.marker})]}.bind(this))},_addMarker:function(e){return this._renderMarker(e).then(function(t){return this._markers.push((0,a.extend)({options:e},t)),this._fireMarkerAddedAction({options:e,originalMarker:t.marker}),t}.bind(this))},_renderMarker:l,removeMarkers:function(e){var t=this;return(0,r.each)(e,function(e,n){t._removeMarker(n);
}),o.default.resolve()},_removeMarker:function(e){var t=this;(0,r.each)(this._markers,function(n,i){return i.options!==e||(t._destroyMarker(i),t._markers.splice(n,1),t._fireMarkerRemovedAction({options:i.options}),!1)})},_destroyMarker:l,_clearMarkers:function(){for(;this._markers.length>0;)this._removeMarker(this._markers[0].options)},addRoutes:function(e){return o.default.all((0,r.map)(e,function(e){return this._addRoute(e)}.bind(this))).then(function(e){return this._fitBounds(),[!1,(0,r.map)(e,function(e){return e.instance})]}.bind(this))},_addRoute:function(e){return this._renderRoute(e).then(function(t){return this._routes.push((0,a.extend)({options:e},t)),this._fireRouteAddedAction({options:e,originalRoute:t.instance}),t}.bind(this))},_renderRoute:l,removeRoutes:function(e){var t=this;return(0,r.each)(e,function(e,n){t._removeRoute(n)}),o.default.resolve()},_removeRoute:function(e){var t=this;(0,r.each)(this._routes,function(n,i){return i.options!==e||(t._destroyRoute(i),t._routes.splice(n,1),t._fireRouteRemovedAction({options:e}),!1)})},_destroyRoute:l,_clearRoutes:function(){for(;this._routes.length>0;)this._removeRoute(this._routes[0].options)},adjustViewport:function(){return this._fitBounds()},isEventsCanceled:function(){return!0},_fitBounds:l,_updateBounds:function(){var e=this;this._clearBounds(),this._option("autoAdjust")&&((0,r.each)(this._markers,function(t,n){e._extendBounds(n.location)}),(0,r.each)(this._routes,function(t,n){n.northEast&&e._extendBounds(n.northEast),n.southWest&&e._extendBounds(n.southWest)}))},_clearBounds:function(){this._bounds=null},_extendBounds:l}),c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(676)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(678)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(16)),r=i(n(195)),s=n(30),l=i(n(84)),u=n(29),c=n(1),d=n(0),h=n(24),f=i(n(45)),p="dx-slider",g="dx-slider-handle",_={top:{my:"bottom center",at:"top center",collision:"none"},bottom:{my:"top center",at:"bottom center",collision:"none"},right:{my:"left center",at:"right center",collision:"none"},left:{my:"right center",at:"left center",collision:"none"}},m=a.default.inherit({_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{hoverStateEnabled:!1,value:0,tooltipEnabled:!1,tooltipFormat:function(e){return e},tooltipPosition:"top",tooltipShowMode:"onHover",tooltipFitIn:null})},_initMarkup:function(){this.callBase(),this.$element().addClass(g),this.setAria({role:"slider",valuenow:this.option("value")})},_render:function(){this._renderTooltip(),this.callBase()},_renderTooltip:function(){this.option("tooltipEnabled")?(this._$tooltip||(this._$tooltip=(0,o.default)("<div>").appendTo(this.$element())),this._$slider=this.$element().closest("."+p),this._updateTooltip()):this._removeTooltip()},_createTooltip:function(){return!this._tooltip&&(this._tooltip=this._createComponent(this._$tooltip,r.default,{visible:!0,target:this.$element(),closeOnOutsideClick:!1,container:this.$element(),hideTopOverlayHandler:null,closeOnTargetScroll:!1,onPositioned:function(e){this._saveTooltipElements(e.component),this._saveTooltipLocation(),this._centeredTooltipPosition()}.bind(this),animation:null,arrowPosition:null,templatesRenderAsynchronously:!1,_fixedPosition:!1}),!0)},_removeTooltip:function(){this._$tooltip&&(this._$tooltip.remove(),delete this._$tooltip,delete this._tooltip)},_renderTooltipPosition:function(){if(this._tooltip){var e=this.option("tooltipPosition");this._saveTooltipElements(),this._resetTooltipPosition(),"string"===(0,c.type)(e)&&(e=(0,d.extend)({boundary:this._$slider,boundaryOffset:{h:1,v:1}},_[e])),this._tooltip.option("position",e),this._saveTooltipLocation()}},_saveTooltipElements:function(e){e=this._tooltip||e,this._$tooltipContent=e.$content().parent(),this._$tooltipArrow=this._$tooltipContent.find(".dx-popover-arrow")},_resetTooltipPosition:function(){(0,s.resetPosition)(this._$tooltipContent),(0,s.resetPosition)(this._$tooltipArrow)},_saveTooltipLocation:function(){this._contentLocate=(0,s.locate)(this._$tooltipContent)},_centeredTooltipPosition:function(){if(this._tooltip){this._$tooltipContent.outerWidth("auto");var e=(0,h.getBoundingRect)(this._$tooltipContent.get(0)).width,t=Math.ceil(e),n=t%2+t;this._$tooltipContent.outerWidth(n);var i=(n-this.$element().width())/2;this._contentLocate.left=-i,this._$tooltipArrow.css({marginLeft:-this._$tooltipArrow.outerWidth()/2,left:"50%"}),this._fitTooltip()}},_fitTooltip:function(){if(this._tooltip){var e=this.option("tooltipPosition");"string"===(0,c.type)(e)&&(e=(0,d.extend)({of:this.$element(),boundary:this._$slider,boundaryOffset:{h:2,v:1}},_[e],{collision:"fit none"}));var t=l.default.calculate(this._$tooltipContent,e),n="left"===t.h.collisionSide,i=(n?-1:1)*t.h.oversize,o=this._contentLocate.left,a=this._contentLocate.left+this._$tooltipContent.outerWidth()-this._$tooltipArrow.outerWidth();(0,s.move)(this._$tooltipContent,{left:this._contentLocate.left+(n?1:-1)*t.h.oversize}),(0,s.move)(this._$tooltipArrow,{left:(0,u.fitIntoRange)(i,o,a)})}},_getFormattedValue:function(e){return f.default.format(e,this.option("tooltipFormat"))},_renderValue:function(){if(this._tooltip){var e=this.option("value");this._tooltip.$content().html(this._getFormattedValue(e)),this._fitTooltip()}},_updateTooltip:function(){var e=/^onhover$/i.test(this.option("tooltipShowMode"));e||this._createTooltip(),this.$element().toggleClass("dx-slider-tooltip-on-hover",e),this._renderTooltipPosition(),this._renderValue(),this._centeredTooltipPosition()},_clean:function(){this.callBase(),delete this._$tooltip,delete this._tooltip},_ensureTooltipIsCentered:function(e,t){(0,c.isDefined)(e)&&(0,c.isDefined)(t)&&e.toString().length!==t.toString().length&&this._centeredTooltipPosition()},_optionChanged:function(e){switch(e.name){case"tooltipFormat":this._renderValue();break;case"value":this._renderValue();var t=this._getFormattedValue(e.value),n=this._getFormattedValue(e.previousValue);this._ensureTooltipIsCentered(t,n),this.setAria("valuenow",e.value);break;case"tooltipEnabled":this._renderTooltip();break;case"tooltipPosition":this._renderTooltipPosition(),this._centeredTooltipPosition();break;case"tooltipShowMode":this._updateTooltip();break;case"tooltipFitIn":this._fitTooltip();break;default:this.callBase(e)}},fitTooltipPosition:function(){this._fitTooltip()},updateTooltip:function(){this._createTooltip()&&(this._renderTooltipPosition(),this._renderValue(),this._centeredTooltipPosition())},repaint:function(){this._renderTooltipPosition(),this._centeredTooltipPosition(),this._tooltip&&this._tooltip._visibilityChanged(!0)}}),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(35)),r=i(n(9)),s=n(0),l=i(n(143)),u=n(1),c="dx-tooltip",d="dx-tooltip-wrapper",h=l.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{toolbarItems:[],showCloseButton:!1,showTitle:!1,title:null,titleTemplate:null,onTitleRendered:null,bottomTemplate:null,propagateOutsideClick:!0})},_render:function(){this.$element().addClass(c),this._wrapper().addClass(d),this.callBase()},_renderContent:function(){this.callBase(),this._contentId="dx-"+new a.default,this._$content.attr({id:this._contentId,role:"tooltip"}),this._toggleAriaDescription(!0)},_toggleAriaDescription:function(e){var t=(0,o.default)(this.option("target")),n=e?this._contentId:void 0;(0,u.isWindow)(t.get(0))||this.setAria("describedby",n,t)}});(0,r.default)("dxTooltip",h);var f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=n(0),l=n(87),u=i(n(9)),c=i(n(52)),d=n(8),h=n(140),f=n(24),p=i(n(43)),g=i(n(10)),_=n(21),m=i(n(179)),v=n(6),y="dx-switch",b=y+"-wrapper",x=y+"-container",w=y+"-inner",C=y+"-handle",k=y+"-on-value",S=y+"-on",D=y+"-off",I=100,T=c.default.inherit({_supportedKeys:function(){var e=this.option("rtlEnabled"),t=function(e){e.preventDefault(),this._clickAction({event:e})},n=function(e,t){t.preventDefault(),t.stopPropagation(),this._animateValue(e)};return(0,s.extend)(this.callBase(),{space:t,enter:t,leftArrow:n.bind(this,!!e),rightArrow:n.bind(this,!e)})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{hoverStateEnabled:!0,activeStateEnabled:!0,switchedOnText:g.default.format("dxSwitch-switchedOnText"),switchedOffText:g.default.format("dxSwitch-switchedOffText"),value:!1,useInkRipple:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}}])},_feedbackHideTimeout:0,_animating:!1,_initMarkup:function(){this._renderContainers(),this.option("useInkRipple")&&this._renderInkRipple(),this.$element().addClass(y).append(this._$switchWrapper),this._renderSubmitElement(),this._renderClick(),this.setAria("role","button"),this._renderSwipeable(),this.callBase(),this._renderSwitchInner(),this._renderLabels(),this._renderValue()},_getInnerOffset:function(e,t){var n=(t-this._offsetDirection()*Number(!e))/2;return 100*n+"%"},_getHandleOffset:function(e,t){if(this.option("rtlEnabled")&&(e=!e),e){var n=-100+100*-t;return n+"%"}return 100*-t+"%"},_renderSwitchInner:function(){this._$switchInner=(0,o.default)("<div>").addClass(w).appendTo(this._$switchContainer),this._$handle=(0,o.default)("<div>").addClass(C).appendTo(this._$switchInner)},_renderLabels:function(){this._$labelOn=(0,o.default)("<div>").addClass(S).prependTo(this._$switchInner),this._$labelOff=(0,o.default)("<div>").addClass(D).appendTo(this._$switchInner),this._setLabelsText()},_renderContainers:function(){this._$switchContainer=(0,o.default)("<div>").addClass(x),this._$switchWrapper=(0,o.default)("<div>").addClass(b).append(this._$switchContainer)},_renderSwipeable:function(){this._createComponent(this.$element(),m.default,{elastic:!1,immediate:!0,onStart:this._swipeStartHandler.bind(this),onUpdated:this._swipeUpdateHandler.bind(this),onEnd:this._swipeEndHandler.bind(this),itemSizeFunc:this._getItemSizeFunc.bind(this)})},_getItemSizeFunc:function(){return this._$switchContainer.outerWidth(!0)-(0,f.getBoundingRect)(this._$handle.get(0)).width},_renderSubmitElement:function(){this._$submitElement=(0,o.default)("<input>").attr("type","hidden").appendTo(this.$element())},_getSubmitElement:function(){return this._$submitElement},_renderInkRipple:function(){this._inkRipple=(0,l.render)({waveSizeCoefficient:1.7,isCentered:!0,useHoldAnimation:!1,wavesNumber:2})},_renderInkWave:function(e,t,n,i){if(this._inkRipple){var o={element:e,event:t,wave:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}},_updateFocusState:function(e,t){this.callBase.apply(this,arguments),this._renderInkWave(this._$handle,e,t,0)},_toggleActiveState:function(e,t,n){this.callBase.apply(this,arguments),this._renderInkWave(this._$handle,n,t,1)},_offsetDirection:function(){return this.option("rtlEnabled")?-1:1},_renderPosition:function(e,t){var n=this._getInnerOffset(e,t),i=this._getHandleOffset(e,t);this._$switchInner.css("transform"," translateX("+n+")"),this._$handle.css("transform"," translateX("+i+")")},_validateValue:function(){var e=this.option("value");"boolean"!=typeof e&&this._options.silent("value",!!e)},_renderClick:function(){var e=(0,d.addNamespace)(_.name,this.NAME),t=this.$element();this._clickAction=this._createAction(this._clickHandler.bind(this)),a.default.off(t,e),a.default.on(t,e,function(e){this._clickAction({event:e})}.bind(this))},_clickHandler:function(e){var t=e.event;this._saveValueChangeEvent(t),this._animating||this._swiping||this._animateValue(!this.option("value"))},_animateValue:function(e){var t=this.option("value"),n=e;if(t!==n){this._animating=!0;var i=this._getInnerOffset(t,0),o=this._getInnerOffset(n,0),a=this._getHandleOffset(t,0),r=this._getHandleOffset(n,0),s=this,l={},u={},c={},d={};l.transform=" translateX("+i+")",u.transform=" translateX("+o+")",c.transform=" translateX("+a+")",d.transform=" translateX("+r+")",this.$element().toggleClass(k,n),p.default.animate(this._$handle,{from:c,to:d,duration:I}),p.default.animate(this._$switchInner,{from:l,to:u,duration:I,complete:function(){s._animating=!1,s.option("value",n)}})}},_swipeStartHandler:function(e){var t=this.option("value"),n=this.option("rtlEnabled"),i=n?0:1,o=n?1:0;e.event.maxLeftOffset=t?i:o,e.event.maxRightOffset=t?o:i,this._swiping=!0,this._feedbackDeferred=new v.Deferred,(0,h.lock)(this._feedbackDeferred),this._toggleActiveState(this.$element(),this.option("activeStateEnabled"))},_swipeUpdateHandler:function(e){this._renderPosition(this.option("value"),e.event.offset)},_swipeEndHandler:function(e){var t=this,n=this._offsetDirection(),i={},o={},a=this._getInnerOffset(t.option("value"),e.event.targetOffset),r=this._getHandleOffset(t.option("value"),e.event.targetOffset);i.transform=" translateX("+a+")",o.transform=" translateX("+r+")",p.default.animate(this._$handle,{to:o,duration:I}),p.default.animate(this._$switchInner,{to:i,duration:I,complete:function(){t._swiping=!1;var i=t.option("value")+n*e.event.targetOffset;t._saveValueChangeEvent(e.event),t.option("value",Boolean(i)),t._feedbackDeferred.resolve(),t._toggleActiveState(t.$element(),!1)}})},_renderValue:function(){this._validateValue();var e=this.option("value");this._renderPosition(e,0),this.$element().toggleClass(k,e),this._getSubmitElement().val(e),this.setAria({pressed:e,label:e?this.option("switchedOnText"):this.option("switchedOffText")})},_setLabelsText:function(){this._$labelOn&&this._$labelOn.text(this.option("switchedOnText")),this._$labelOff&&this._$labelOff.text(this.option("switchedOffText"))},_visibilityChanged:function(e){e&&this.repaint()},_optionChanged:function(e){switch(e.name){case"useInkRipple":this._invalidate();break;case"width":delete this._marginBound,this._refresh();break;case"switchedOnText":case"switchedOffText":this._setLabelsText();break;case"value":this._renderValue(),this.callBase(e);break;default:this.callBase(e)}},_clean:function(){delete this._inkRipple,this.callBase()}});(0,u.default)("dxSwitch",T);var O=T;t.default=O,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=n(7),l=i(n(9)),u=n(0),c=n(1),d=n(8),h=i(n(25)),f=i(n(285)),p=n(184),g=n(299),_=i(n(104)),m="dx-textarea",v="dx-texteditor-input",y="dx-texteditor-input-auto-resize",b=_.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{spellcheck:!0,minHeight:void 0,maxHeight:void 0,autoResizeEnabled:!1})},_initMarkup:function(){this.$element().addClass(m),this.callBase(),this.setAria("multiline","true")},_renderContentImpl:function(){this._updateInputHeight(),this.callBase()},_renderInput:function(){this.callBase(),this._renderScrollHandler()},_createInput:function(){var e=(0,o.default)("<textarea>");return this._applyInputAttributes(e,this.option("inputAttr")),this._updateInputAutoResizeAppearance(e),e},_applyInputAttributes:function(e,t){e.attr(t).addClass(v)},_renderScrollHandler:function(){this._eventY=0;var e=this._input(),t=(0,g.prepareScrollData)(e,!0);a.default.on(e,(0,d.addNamespace)(f.default.init,this.NAME),t,r.noop),a.default.on(e,(0,d.addNamespace)(h.default.down,this.NAME),this._pointerDownHandler.bind(this)),a.default.on(e,(0,d.addNamespace)(h.default.move,this.NAME),this._pointerMoveHandler.bind(this))},_pointerDownHandler:function(e){this._eventY=(0,d.eventData)(e).y},_pointerMoveHandler:function(e){var t=(0,d.eventData)(e).y,n=this._eventY-t;(0,g.allowScroll)(this._input(),n)&&(e.isScrollingEvent=!0,e.stopPropagation()),this._eventY=t},_renderDimensions:function(){var e=this.$element(),t=e.get(0),n=this._getOptionValue("width",t),i=this._getOptionValue("height",t),o=this.option("minHeight"),a=this.option("maxHeight");e.css({minHeight:void 0!==o?o:"",maxHeight:void 0!==a?a:"",width:n,height:i})},_resetDimensions:function(){this.$element().css({height:"",minHeight:"",maxHeight:""})},_renderEvents:function(){this.option("autoResizeEnabled")&&a.default.on(this._input(),(0,d.addNamespace)("input paste",this.NAME),this._updateInputHeight.bind(this)),this.callBase()},_refreshEvents:function(){a.default.off(this._input(),(0,d.addNamespace)("input paste",this.NAME)),this.callBase()},_getHeightDifference:function(e){return(0,p.getVerticalOffsets)(this._$element.get(0),!1)+(0,p.getVerticalOffsets)(this._$textEditorContainer.get(0),!1)+(0,p.getVerticalOffsets)(this._$textEditorInputContainer.get(0),!1)+(0,p.getElementBoxParams)("height",(0,s.getWindow)().getComputedStyle(e.get(0))).margin},_updateInputHeight:function(){var e=this._input(),t=void 0===this.option("height")&&this.option("autoResizeEnabled");if(!t)return void e.css("height","");this._resetDimensions(),this._$element.css("height",this._$element.outerHeight()),e.css("height",0);var n=this._getHeightDifference(e);this._renderDimensions();var i=this._getBoundaryHeight("minHeight"),o=this._getBoundaryHeight("maxHeight"),a=e[0].scrollHeight;if(void 0!==i&&(a=Math.max(a,i-n)),void 0!==o){var r=o-n,s=a>r;a=Math.min(a,r),this._updateInputAutoResizeAppearance(e,!s)}e.css("height",a),t&&this._$element.css("height","auto")},_getBoundaryHeight:function(e){var t=this.option(e);if((0,c.isDefined)(t))return"number"==typeof t?t:(0,p.parseHeight)(t,this._$textEditorContainer.get(0))},_renderInputType:r.noop,_visibilityChanged:function(e){e&&this._updateInputHeight()},_updateInputAutoResizeAppearance:function(e,t){if(e){var n=(0,r.ensureDefined)(t,this.option("autoResizeEnabled"));e.toggleClass(y,n)}},_optionChanged:function(e){switch(e.name){case"autoResizeEnabled":this._updateInputAutoResizeAppearance(this._input(),e.value),this._refreshEvents(),this._updateInputHeight();break;case"value":case"height":this.callBase(e),this._updateInputHeight();break;case"minHeight":case"maxHeight":this._renderDimensions(),this._updateInputHeight();break;case"visible":this.callBase(e),e.value&&this._updateInputHeight();break;default:this.callBase(e)}}});(0,l.default)("dxTextArea",b);var x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var s=r(n(2)),l=r(n(5)),u=r(n(43)),c=n(21),d=r(n(13)),h=r(n(11)),f=n(0),p=n(4),g=n(18),_=a(n(3)),m=n(1),v=r(n(9)),y=n(8),b=r(n(214)),x=n(6),w=n(71),C=n(59),k=n(36),S="dx-accordion",D="dx-accordion-wrapper",I="dx-accordion-item",T="dx-accordion-item-opened",O="dx-accordion-item-closed",E="dx-accordion-item-title",A="dx-accordion-item-body",P="dx-accordion-item-title-caption",R="dxAccordionItemData",M=b.default.inherit({_activeStateUnit:"."+I,_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{hoverStateEnabled:!0,height:void 0,itemTitleTemplate:"title",onItemTitleClick:null,selectedIndex:0,collapsible:!1,multiple:!1,animationDuration:300,deferRendering:!0,selectionByClick:!0,activeStateEnabled:!0,_itemAttributes:{role:"tab"},_animationEasing:"ease"})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===d.default.real().deviceType&&!d.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,k.isMaterial)()},options:{animationDuration:200,_animationEasing:"cubic-bezier(0.4, 0, 0.2, 1)"}}])},_itemElements:function(){return this._itemContainer().children(this._itemSelector())},_init:function(){this.callBase(),this.option("selectionRequired",!this.option("collapsible")),this.option("selectionMode",this.option("multiple")?"multiple":"single");var e=this.$element();e.addClass(S),this._$container=(0,s.default)("<div>").addClass(D),e.append(this._$container)},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({title:new w.BindableTemplate(function(e,t){if((0,m.isPlainObject)(t)){var n=(0,C.getImageContainer)(t.icon);n&&e.append(n),(0,m.isDefined)(t.title)&&!(0,m.isPlainObject)(t.title)&&e.append(h.default.createTextNode(t.title))}else(0,m.isDefined)(t)&&e.text(String(t));e.wrapInner((0,s.default)("<div>").addClass(P))},["title","icon"],this.option("integrationOptions.watchMethod"))})},_initMarkup:function(){var e=this;this._deferredItems=[],this.callBase(),this.setAria({role:"tablist",multiselectable:this.option("multiple")}),(0,p.deferRender)(function(){var t=e._getSelectedItemIndices();e._renderSelection(t,[])})},_render:function(){this.callBase(),this._updateItemHeightsWrapper(!0)},_itemDataKey:function(){return R},_itemClass:function(){return I},_itemContainer:function(){return this._$container},_itemTitles:function(){return this._itemElements().find("."+E)},_itemContents:function(){return this._itemElements().find("."+A)},_getItemData:function(e){return(0,s.default)(e).parent().data(this._itemDataKey())||this.callBase.apply(this,arguments)},_executeItemRenderAction:function(e){e.type||this.callBase.apply(this,arguments)},_itemSelectHandler:function(e){(0,s.default)(e.target).closest(this._itemContents()).length||this.callBase.apply(this,arguments)},_afterItemElementDeleted:function(e,t){this._deferredItems.splice(t.itemIndex,1),this.callBase.apply(this,arguments)},_renderItemContent:function(e){var t=this.callBase((0,f.extend)({},e,{contentClass:E,templateProperty:"titleTemplate",defaultTemplateName:this.option("itemTitleTemplate")}));this._attachItemTitleClickAction(t);var n=new x.Deferred;(0,m.isDefined)(this._deferredItems[e.index])?this._deferredItems[e.index]=n:this._deferredItems.push(n),(!this.option("deferRendering")||this._getSelectedItemIndices().indexOf(e.index)>=0)&&n.resolve(),n.done(this.callBase.bind(this,(0,f.extend)({},e,{contentClass:A,container:(0,g.getPublicElement)((0,s.default)("<div>").appendTo((0,s.default)(t).parent()))})))},_attachItemTitleClickAction:function(e){var t=(0,y.addNamespace)(c.name,this.NAME);l.default.off(e,t),l.default.on(e,t,this._itemTitleClickHandler.bind(this))},_itemTitleClickHandler:function(e){this._itemDXEventHandler(e,"onItemTitleClick")},_renderSelection:function(e,t){this._itemElements().addClass(O),this.setAria("hidden",!0,this._itemContents()),this._updateItems(e,t)},_updateSelection:function(e,t){this._updateItems(e,t),this._updateItemHeightsWrapper(!1)},_updateItems:function(e,t){var n=this,i=this._itemElements();_.each(e,function(e,t){n._deferredItems[t].resolve();var o=i.eq(t).addClass(T).removeClass(O);n.setAria("hidden",!1,o.find("."+A))}),_.each(t,function(e,t){var o=i.eq(t).removeClass(T);n.setAria("hidden",!0,o.find("."+A))})},_updateItemHeightsWrapper:function(e){this.option("templatesRenderAsynchronously")?this._animationTimer=setTimeout(function(){this._updateItemHeights(e)}.bind(this)):this._updateItemHeights(e)},_updateItemHeights:function(e){var t=this,n=t._deferredAnimate,i=this._splitFreeSpace(this._calculateFreeSpace());return clearTimeout(this._animationTimer),x.when.apply(s.default,[].slice.call(this._itemElements()).map(function(n){return t._updateItemHeight((0,s.default)(n),i,e)})).done(function(){n&&n.resolveWith(t)})},_updateItemHeight:function(e,t,n){var i=e.children("."+E);u.default.isAnimating(e)&&u.default.stop(e);var o=e.outerHeight(),a=e.hasClass(T)?t+i.outerHeight()||e.height("auto").outerHeight():i.outerHeight();return this._animateItem(e,o,a,n,!!t)},_animateItem:function(e,t,n,i,o){var a;return i||t===n?(e.css("height",n),a=(new x.Deferred).resolve()):a=u.default.animate(e,{type:"custom",from:{height:t},to:{height:n},duration:this.option("animationDuration"),easing:this.option("_animationEasing")}),a.done(function(){e.hasClass(T)&&!o&&e.css("height",""),e.not("."+T).addClass(O)})},_splitFreeSpace:function(e){return e?e/this.option("selectedItems").length:e},_calculateFreeSpace:function(){var e=this.option("height");if(void 0!==e&&"auto"!==e){var t=this._itemTitles(),n=0;return _.each(t,function(e,t){n+=(0,s.default)(t).outerHeight()}),this.$element().height()-n}},_visibilityChanged:function(e){e&&this._dimensionChanged()},_dimensionChanged:function(){this._updateItemHeights(!0)},_clean:function(){clearTimeout(this._animationTimer),this.callBase()},_tryParseItemPropertyName:function(e){var t=e.match(/.*\.(.*)/);if((0,m.isDefined)(t)&&t.length>=1)return t[1]},_optionChanged:function(e){switch(e.name){case"items":this.callBase(e),"title"===this._tryParseItemPropertyName(e.fullName)&&this._renderSelection(this._getSelectedItemIndices(),[]),"visible"===this._tryParseItemPropertyName(e.fullName)&&this._updateItemHeightsWrapper(!0);break;case"animationDuration":case"onItemTitleClick":case"_animationEasing":break;case"collapsible":this.option("selectionRequired",!this.option("collapsible"));break;case"itemTitleTemplate":case"height":case"deferRendering":this._invalidate();break;case"multiple":this.option("selectionMode",e.value?"multiple":"single");break;default:this.callBase(e)}},expandItem:function(e){return this._deferredAnimate=new x.Deferred,this.selectItem(e),this._deferredAnimate.promise()},collapseItem:function(e){return this._deferredAnimate=new x.Deferred,this.unselectItem(e),this._deferredAnimate.promise()},updateDimensions:function(){return this._updateItemHeights(!1)}});(0,v.default)("dxAccordion",M);var B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(11)),y=o(n(5)),b=o(n(35)),x=o(n(9)),w=n(4),C=n(1),k=n(33),S=n(18),D=n(3),I=n(12),T=n(0),O=n(7),E=o(n(43)),A=o(n(84)),P=o(n(13)),R=n(8),M=o(n(68)),B=o(n(458)),V=n(6),F="dx-menu",L=F+"-item",H=L+"-expanded",z="dx-menu-phone-overlay",N=F+"-items-container",j=L+"-wrapper",W="dx-submenu",G="dx-context-menu",q="dx-has-context-menu",$="dx-state-disabled",K="dx-state-focused",U="dx-state-hover",Y="up",Z="down",X="left",Q="right",J="first",ee="last",te=["onShowing","onShown","onSubmenuCreated","onHiding","onHidden","onPositioning","onLeftFirstItem","onLeftLastItem","onCloseRootSubmenu","onExpandLastSubmenu"],ne=[Y,Z,J,ee],ie="dxcontextmenu",oe=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getShowEvent",value:function(e){var t=null;return(0,C.isObject)(e)?null!==e.name&&(t=e.name||ie):t=e,t}},{key:"getShowDelay",value:function(e){return(0,C.isObject)(e)&&e.delay}},{key:"_getDefaultOptions",value:function(){return(0,T.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{showEvent:ie,closeOnOutsideClick:!0,position:{at:"top left",my:"top left"},onShowing:null,onShown:null,onSubmenuCreated:null,onHiding:null,onHidden:null,onPositioning:null,submenuDirection:"auto",visible:!1,target:void 0,onLeftFirstItem:null,onLeftLastItem:null,onCloseRootSubmenu:null,onExpandLastSubmenu:null})}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return!(0,O.hasWindow)()},options:{animation:null}}])}},{key:"_initActions",value:function(){var e=this;this._actions={},(0,D.each)(te,function(t,n){e._actions[n]=e._createActionByOption(n)||w.noop})}},{key:"_setOptionsByReference",value:function(){l(_(t.prototype),"_setOptionsByReference",this).call(this),(0,T.extend)(this._optionsByReference,{animation:!0,selectedItem:!0})}},{key:"_focusInHandler",value:function(){}},{key:"_itemContainer",value:function(){return this._overlay?this._overlay.$content():(0,m.default)()}},{key:"_eventBindingTarget",value:function(){return this._itemContainer()}},{key:"itemsContainer",value:function(){return this._overlay?this._overlay.$content():void 0}},{key:"_supportedKeys",value:function(){var e=this,n=function(){var t=(0,m.default)(e.option("focusedElement"));e.hide(),t.length&&e._isSelectionEnabled()&&e.selectItem(t[0])};return(0,T.extend)(l(_(t.prototype),"_supportedKeys",this).call(this),{space:n,escape:this.hide})}},{key:"_getActiveItem",value:function(){var e=this._getAvailableItems(),n=e.filter(".".concat(K)),i=e.filter(".".concat(U)),o=i.closest(".".concat(N));return o.find(".".concat(L)).index(n)>=0?n:i.length?i:l(_(t.prototype),"_getActiveItem",this).call(this)}},{key:"_moveFocus",value:function(e){var n,i=this._getItemsByLocation(e),o=this._getActiveItem(!0),a=this.itemsContainer().find(".".concat(U)),r=(0,m.default)(this.option("focusedElement")),s=!(!r.length&&!a.length);switch(e){case Y:n=s?this._prevItem(i):o,this._setFocusedElement(n),o.is(i.first())&&this._actions.onLeftFirstItem(o);break;case Z:n=s?this._nextItem(i):o,this._setFocusedElement(n),o.is(i.last())&&this._actions.onLeftLastItem(o);break;case Q:n=this.option("rtlEnabled")?this._hideSubmenuHandler():this._expandSubmenuHandler(i,e),this._setFocusedElement(n);break;case X:n=this.option("rtlEnabled")?this._expandSubmenuHandler(i,e):this._hideSubmenuHandler(),this._setFocusedElement(n);break;case J:n=i.first(),this._setFocusedElement(n);break;case ee:n=i.last(),this._setFocusedElement(n);break;default:return l(_(t.prototype),"_moveFocus",this).call(this,e)}}},{key:"_setFocusedElement",value:function(e){e&&0!==e.length&&this.option("focusedElement",(0,
S.getPublicElement)(e))}},{key:"_getItemsByLocation",value:function(e){var t,n=this._getActiveItem(!0);return(0,I.inArray)(e,ne)>=0&&(t=n.closest(".".concat(N)).children().children()),t=this._getAvailableItems(t)}},{key:"_getAriaTarget",value:function(){return this.$element()}},{key:"_refreshActiveDescendant",value:function(){if((0,C.isDefined)(this._overlay)){var e=this._overlay.$content();l(_(t.prototype),"_refreshActiveDescendant",this).call(this,e)}}},{key:"_hideSubmenuHandler",value:function(){var e=this._getActiveItem(!0),t=e.parents(".".concat(H)).first();return t.length?(this._hideSubmenusOnSameLevel(t),this._hideSubmenu(e.closest(".".concat(W))),t):(this._actions.onCloseRootSubmenu(e),e)}},{key:"_expandSubmenuHandler",value:function(e,t){var n=this._getActiveItem(!0),i=this._getItemData(n),o=this._dataAdapter.getNodeByItem(i),a=this._hasSubmenu(o),r=n.children(".".concat(W));return a&&!n.hasClass($)?(r.length&&"hidden"!==r.css("visibility")||this._showSubmenu(n),this._nextItem(this._getItemsByLocation(t))):void this._actions.onExpandLastSubmenu(n)}},{key:"_clean",value:function(){this._overlay&&(this._overlay.$element().remove(),this._overlay=null),this._detachShowContextMenuEvents(this._getTarget()),l(_(t.prototype),"_clean",this).call(this)}},{key:"_initMarkup",value:function(){this.$element().addClass(q),l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._renderVisibility(this.option("visible")),this._addWidgetClass()}},{key:"_renderContentImpl",value:function(){this._detachShowContextMenuEvents(this._getTarget()),this._attachShowContextMenuEvents()}},{key:"_attachKeyboardEvents",value:function(){!this._keyboardListenerId&&this._focusTarget().length&&l(_(t.prototype),"_attachKeyboardEvents",this).call(this)}},{key:"_renderContextMenuOverlay",value:function(){if(!this._overlay){var e=this._getOverlayOptions();this._overlay=this._createComponent((0,m.default)("<div>").appendTo(this._$element),M.default,e);var t=this._overlay.$content();t.addClass(G),this._addCustomCssClass(t),this._addPlatformDependentClass(t),this._attachContextMenuEvent()}}},{key:"_itemContextMenuHandler",value:function(e){l(_(t.prototype),"_itemContextMenuHandler",this).call(this,e),e.stopPropagation()}},{key:"_addPlatformDependentClass",value:function(e){P.default.current().phone&&e.addClass(z)}},{key:"_detachShowContextMenuEvents",value:function(e){var t=this.getShowEvent(this.option("showEvent"));if(t){var n=(0,R.addNamespace)(t,this.NAME);this._showContextMenuEventHandler?y.default.off(v.default.getDocument(),n,e,this._showContextMenuEventHandler):y.default.off((0,m.default)(e),n)}}},{key:"_attachShowContextMenuEvents",value:function(){var e=this,t=this._getTarget(),n=this.getShowEvent(this.option("showEvent"));if(n){var i=(0,R.addNamespace)(n,this.NAME),o=this._createAction(function(t){var n=e.getShowDelay(e.option("showEvent"));n?setTimeout(function(){return e._show(t.event)},n):e._show(t.event)},{validatingTargetName:"target"}),a=function(e){return o({event:e,target:(0,m.default)(e.currentTarget)})};o=this._createAction(o),(0,C.isRenderer)(t)||t.nodeType||(0,C.isWindow)(t)?(this._showContextMenuEventHandler=void 0,y.default.on(t,i,a)):(this._showContextMenuEventHandler=a,y.default.on(v.default.getDocument(),i,t,this._showContextMenuEventHandler))}}},{key:"_hoverEndHandler",value:function(e){l(_(t.prototype),"_hoverEndHandler",this).call(this,e),e.stopPropagation()}},{key:"_renderDimensions",value:function(){}},{key:"_renderContainer",value:function(e,n){var i=n||this._itemContainer();e=(0,m.default)("<div>"),e.appendTo(i).addClass(W).css("visibility",n?"hidden":"visible");var o=l(_(t.prototype),"_renderContainer",this).call(this,e);return n?o:this.option("width")?o.css("minWidth",this.option("width")):this.option("height")?o.css("minHeight",this.option("height")):o}},{key:"_renderSubmenuItems",value:function(e,t){this._renderItems(this._getChildNodes(e),t),this._actions.onSubmenuCreated({itemElement:(0,S.getPublicElement)(t),itemData:e.internalFields.item,submenuElement:(0,S.getPublicElement)(t.children(".".concat(W)))})}},{key:"_getOverlayOptions",value:function(){var e=this.option("position"),t={focusStateEnabled:this.option("focusStateEnabled"),animation:this.option("animation"),innerOverlay:!0,closeOnOutsideClick:this._closeOnOutsideClickHandler.bind(this),propagateOutsideClick:!0,closeOnTargetScroll:!0,deferRendering:!1,position:{at:e.at,my:e.my,of:this._getTarget(),collision:"flipfit"},shading:!1,showTitle:!1,height:"auto",width:"auto",onShown:this._overlayShownActionHandler.bind(this),onHiding:this._overlayHidingActionHandler.bind(this),onHidden:this._overlayHiddenActionHandler.bind(this)};return t}},{key:"_overlayShownActionHandler",value:function(e){this._actions.onShown(e)}},{key:"_overlayHidingActionHandler",value:function(e){this._actions.onHiding(e),e.cancel||(this._hideAllShownSubmenus(),this._setOptionWithoutOptionChange("visible",!1))}},{key:"_overlayHiddenActionHandler",value:function(e){this._actions.onHidden(e)}},{key:"_closeOnOutsideClickHandler",value:function(e){var t=this.option("closeOnOutsideClick");if((0,C.isFunction)(t))return t(e);if(!t)return!1;if(v.default.isDocument(e.target))return!0;var n=this._getActiveItemsContainer(e.target),i=this._getItemsContainers(),o=this._searchActiveItem(e.target),a=this.$element().parents(".".concat(L)),r=o[0]===a[0]&&o.length&&a.length,s=this._isIncludeOverlay(n,i)&&o.length;return!s&&!r||("onClick"===this._getShowSubmenuMode()&&this._hideAllShownChildSubmenus(o),!1)}},{key:"_getActiveItemsContainer",value:function(e){return(0,m.default)(e).closest(".".concat(N))}},{key:"_getItemsContainers",value:function(){return this._overlay._$content.find(".".concat(N))}},{key:"_searchActiveItem",value:function(e){return(0,m.default)(e).closest(".".concat(L)).eq(0)}},{key:"_isIncludeOverlay",value:function(e,t){var n=!1;return(0,D.each)(t,function(t,i){e.is(i)&&!n&&(n=!0)}),n}},{key:"_hideAllShownChildSubmenus",value:function(e){var t=this,n=e.find(".".concat(W)),i=(0,T.extend)([],this._shownSubmenus);n.length>0&&(0,D.each)(i,function(n,i){var o=t._searchActiveItem(i.context).parent();o.parent().is(e.parent().parent())&&!o.is(e.parent())&&t._hideSubmenu(i)})}},{key:"_showSubmenu",value:function(e){var n=this._dataAdapter.getNodeByItem(this._getItemData(e));if(this._hideSubmenusOnSameLevel(e),this._hasSubmenu(n)){var i=e.children(".".concat(W)),o=i.length;l(_(t.prototype),"_showSubmenu",this).call(this,e),o||this._renderSubmenuItems(n,e),this._isSubmenuVisible(i)||this._drawSubmenu(e)}}},{key:"_hideSubmenusOnSameLevel",value:function(e){var t=e.parent(".".concat(j)).siblings().find(".".concat(H));t.length&&(t.removeClass(H),this._hideSubmenu(t.find(".".concat(W))))}},{key:"_hideSubmenuGroup",value:function(e){this._isSubmenuVisible(e)&&this._hideSubmenuCore(e)}},{key:"_isSubmenuVisible",value:function(e){return"visible"===e.css("visibility")}},{key:"_drawSubmenu",value:function(e){var t=this.option("animation")?this.option("animation").show:{},n=e.children(".".concat(W)),i=this._getSubmenuPosition(e);this._overlay&&this._overlay.option("visible")&&((0,C.isDefined)(this._shownSubmenus)||(this._shownSubmenus=[]),(0,I.inArray)(n,this._shownSubmenus)&&this._shownSubmenus.push(n),t&&E.default.stop(n),A.default.setup(n,i),t&&((0,C.isPlainObject)(t.to)&&(t.to.position=i),this._animate(n,t)),n.css("visibility","visible"))}},{key:"_animate",value:function(e,t){E.default.animate(e,t)}},{key:"_getSubmenuPosition",value:function(e){var t=this.option("submenuDirection").toLowerCase(),n=e.parent(".".concat(j)),i={collision:"flip",of:n,offset:{h:0,v:-1}};switch(t){case"left":i.at="left top",i.my="right top";break;case"right":i.at="right top",i.my="left top";break;default:this.option("rtlEnabled")?(i.at="left top",i.my="right top"):(i.at="right top",i.my="left top")}return i}},{key:"_updateSubmenuVisibilityOnClick",value:function(e){if(e.args.length){var t=e.args[0].itemData,n=this._dataAdapter.getNodeByItem(t);if(n){var i=(0,m.default)(e.args[0].itemElement),o=i.find(".".concat(W)),a=this._hasSubmenu(n)&&!o.length;if(a&&(this._renderSubmenuItems(n,i),o=i.find(".".concat(W))),i.context!==o.context||"visible"!==o.css("visibility")){var r=t&&t.closeMenuOnClick===!1;if(t&&!t.disabled&&!r)if(this._updateSelectedItemOnClick(e),0===o.length){var s=(0,m.default)(i.parents(".".concat(W))[0]);this._hideSubmenu(s),!e.canceled&&this._overlay&&this._overlay.option("visible")&&this.option("visible",!1)}else this._shownSubmenus&&this._shownSubmenus.length>0&&this._shownSubmenus[0].is(o)&&this._hideSubmenu(o),this._showSubmenu(i)}}}}},{key:"_hideSubmenu",value:function(e){var t=this,n=(0,T.extend)([],this._shownSubmenus);(0,D.each)(n,function(n,i){(e.is(i)||(0,k.contains)(e[0],i[0]))&&(i.parent().removeClass(H),t._hideSubmenuCore(i))})}},{key:"_hideSubmenuCore",value:function(e){var t=(0,I.inArray)(e,this._shownSubmenus),n=this.option("animation")?this.option("animation").hide:null;t>=0&&this._shownSubmenus.splice(t,1),this._stopAnimate(e),n&&this._animate(e,n),e.css("visibility","hidden")}},{key:"_stopAnimate",value:function(e){E.default.stop(e,!0)}},{key:"_hideAllShownSubmenus",value:function(){var e=this,t=(0,T.extend)([],this._shownSubmenus),n=this._overlay.$content().find(".".concat(H));n.removeClass(H),(0,D.each)(t,function(t,n){e._hideSubmenu(n)})}},{key:"_visibilityChanged",value:function(e){e&&this._renderContentImpl()}},{key:"_optionChanged",value:function(e){if((0,I.inArray)(e.name,te)>-1)return void this._initActions();switch(e.name){case"visible":this._renderVisibility(e.value);break;case"showEvent":case"position":case"submenuDirection":this._invalidate();break;case"target":e.previousValue&&this._detachShowContextMenuEvents(e.previousValue),this._invalidate();break;case"closeOnOutsideClick":break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_renderVisibility",value:function(e){return e?this._show():this._hide()}},{key:"_toggleVisibility",value:function(){}},{key:"_show",value:function(e){var t={jQEvent:e},n=(new V.Deferred).reject().promise();if(this._actions.onShowing(t),t.cancel)return n;var i=this._positionContextMenu(e);return i&&(this._overlay||(this._renderContextMenuOverlay(),this._overlay.$content().addClass(this._widgetClass()),this._renderFocusState(),this._attachHoverEvents(),this._attachClickEvent(),this._renderItems(this._dataAdapter.getRootNodes())),this._setOptionWithoutOptionChange("visible",!0),this._overlay.option("position",i),n=this._overlay.show(),e&&e.stopPropagation(),this._setAriaAttributes()),n}},{key:"_setAriaAttributes",value:function(){this._overlayContentId="dx-".concat(new b.default),this.setAria("owns",this._overlayContentId),this.setAria({id:this._overlayContentId,role:"menu"},this._overlay.$content())}},{key:"_cleanAriaAttributes",value:function(){this._overlay&&this.setAria("id",null,this._overlay.$content()),this.setAria("owns",void 0)}},{key:"_getTarget",value:function(){return this.option("target")||this.option("position").of||(0,m.default)(v.default.getDocument())}},{key:"_getContextMenuPosition",value:function(){return(0,T.extend)({},this.option("position"),{of:this._getTarget()})}},{key:"_positionContextMenu",value:function(e){var t=this._getContextMenuPosition(),n=this._isInitialOptionValue("position"),i=this._createActionByOption("onPositioning",o);e&&e.preventDefault&&n&&(t.of=e);var o={position:t,event:e};return i(o),o.cancel?t=null:o.event&&(o.event.cancel=!0,e.preventDefault()),t}},{key:"_refresh",value:function(){if((0,O.hasWindow)())if(this._overlay){var e=this._overlay.option("position");l(_(t.prototype),"_refresh",this).call(this),this._overlay&&this._overlay.option("position",e)}else l(_(t.prototype),"_refresh",this).call(this);else l(_(t.prototype),"_refresh",this).call(this)}},{key:"_hide",value:function(){var e;return this._overlay&&(e=this._overlay.hide(),this._setOptionWithoutOptionChange("visible",!1)),this._cleanAriaAttributes(),this.option("focusedElement",null),e||(new V.Deferred).reject().promise()}},{key:"toggle",value:function(e){var t=this.option("visible");return e=void 0===e?!t:e,this._renderVisibility(e)}},{key:"show",value:function(){return this.toggle(!0)}},{key:"hide",value:function(){return this.toggle(!1)}}]),t}(B.default);(0,x.default)("dxContextMenu",oe);var ae=oe;t.default=ae,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(4),y=n(1),b=n(3),x=n(0),w=n(87),C=o(n(459)),k=o(n(689)),S=o(n(13)),D=o(n(141)),I="dx-menu",T=I+"-no-icons",O="dx-menu-base",E=I+"-item",A=E+"-content",P=E+"-selected",R=E+"-wrapper",M=I+"-items-container",B=E+"-expanded",V=I+"-separator",F=I+"-last-group-item",L=E+"-has-text",H=E+"-has-icon",z=E+"-has-submenu",N=E+"-popout",j=N+"-container",W=E+"-text",G="single",q={show:50,hide:300},$=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{items:[],cssClass:"",activeStateEnabled:!0,showSubmenuMode:{name:"onHover",delay:{show:50,hide:300}},animation:{show:{type:"fade",from:0,to:1,duration:100},hide:{type:"fade",from:1,to:0,duration:100}},selectByClick:!1,focusOnSelectedItem:!1,keyExpr:null,_itemAttributes:{role:"menuitem"},useInkRipple:!1})}},{key:"_itemDataKey",value:function(){return"dxMenuItemDataKey"}},{key:"_itemClass",value:function(){return E}},{key:"_setAriaSelected",value:function(){}},{key:"_selectedItemClass",value:function(){return P}},{key:"_widgetClass",value:function(){return O}},{key:"_focusTarget",value:function(){return this._itemContainer()}},{key:"_clean",value:function(){this.option("focusedElement",null),l(_(t.prototype),"_clean",this).call(this)}},{key:"_supportedKeys",value:function(){var e=this,n=function(){var t=(0,m.default)(e.option("focusedElement"));t.length&&e._isSelectionEnabled()&&e.selectItem(t[0])};return(0,x.extend)(l(_(t.prototype),"_supportedKeys",this).call(this),{space:n,pageUp:v.noop,pageDown:v.noop})}},{key:"_isSelectionEnabled",value:function(){return this.option("selectionMode")===G}},{key:"_init",value:function(){this._activeStateUnit=".".concat(E),l(_(t.prototype),"_init",this).call(this),this._renderSelectedItem(),this._initActions()}},{key:"_getTextContainer",value:function(e){var t=e.text,n=(0,m.default)("<span>").addClass(W),i=(0,y.isPlainObject)(e)?t:String(e);return t&&n.text(i)}},{key:"_getPopoutContainer",value:function(e){var t,n=e.items;if(n&&n.length){var i=(0,m.default)("<div>").addClass(N);t=(0,m.default)("<span>").addClass(j).append(i)}return t}},{key:"_getDataAdapterOptions",value:function(){return{rootValue:0,multipleSelection:!1,recursiveSelection:!1,recursiveExpansion:!1,searchValue:""}}},{key:"_selectByItem",value:function(e){if(e){var t=this._dataAdapter.getNodeByItem(e);this._dataAdapter.toggleSelection(t.internalFields.key,!0)}}},{key:"_renderSelectedItem",value:function(){var e=this._dataAdapter.getSelectedNodesKeys(),t=e.length&&e[0],n=this.option("selectedItem");if(!t)return void this._selectByItem(n);var i=this._dataAdapter.getNodeByKey(t);return i.selectable!==!1?n?void(n!==i.internalFields.item&&(this._dataAdapter.toggleSelection(t,!1),this._selectByItem(n))):void this.option("selectedItem",i.internalFields.item):void 0}},{key:"_initActions",value:function(){}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this._addCustomCssClass(this.$element()),this.option("useInkRipple")&&this._renderInkRipple()}},{key:"_renderInkRipple",value:function(){this._inkRipple=(0,w.render)()}},{key:"_toggleActiveState",value:function(e,n,i){if(l(_(t.prototype),"_toggleActiveState",this).apply(this,arguments),this._inkRipple){var o={element:e,event:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}}},{key:"_getShowSubmenuMode",value:function(){var e="onClick",t=this.option("showSubmenuMode");return t=(0,y.isObject)(t)?t.name:t,this._isDesktopDevice()?t:e}},{key:"_initSelectedItems",value:function(){}},{key:"_isDesktopDevice",value:function(){return"desktop"===S.default.real().deviceType}},{key:"_initEditStrategy",value:function(){var e=k.default;this._editStrategy=new e(this)}},{key:"_addCustomCssClass",value:function(e){e.addClass(this.option("cssClass"))}},{key:"_itemWrapperSelector",value:function(){return".".concat(R)}},{key:"_hoverStartHandler",value:function(e){var t=this._getItemElementByEventArgs(e);t&&!this._isItemDisabled(t)&&(e.stopPropagation(),"onHover"===this._getShowSubmenuMode()&&(clearTimeout(this._showSubmenusTimeout),this._showSubmenusTimeout=setTimeout(this._showSubmenu.bind(this,t),this._getSubmenuDelay("show"))))}},{key:"_getAvailableItems",value:function(e){return l(_(t.prototype),"_getAvailableItems",this).call(this,e).filter(function(){return"hidden"!==(0,m.default)(this).css("visibility")})}},{key:"_isItemDisabled",value:function(e){return this._disabledGetter(e.data(this._itemDataKey()))}},{key:"_showSubmenu",value:function(e){this._addExpandedClass(e)}},{key:"_addExpandedClass",value:function(e){(0,m.default)(e).addClass(B)}},{key:"_getSubmenuDelay",value:function(e){var t=this.option("showSubmenuMode"),n=t.delay;return(0,y.isDefined)(n)?(0,y.isObject)(n)?n[e]:n:q[e]}},{key:"_getItemElementByEventArgs",value:function(e){var t=(0,m.default)(e.target);if(t.hasClass(this._itemClass())||t.get(0)===e.currentTarget)return t;for(;!t.hasClass(this._itemClass());)if(t=t.parent(),t.hasClass("dx-submenu"))return null;return t}},{key:"_hoverEndHandler",value:function(){clearTimeout(this._showSubmenusTimeout)}},{key:"_hasSubmenu",value:function(e){return e&&e.internalFields.childrenKeys.length}},{key:"_renderContentImpl",value:function(){this._renderItems(this._dataAdapter.getRootNodes())}},{key:"_renderItems",value:function(e,t){var n=this;if(e.length){this.hasIcons=!1;var i=this._renderContainer(this.$element(),t),o=-1,a=-1;(0,b.each)(e,function(e,t){var r=t.visible!==!1;r&&o<0&&(o=e);var s=o<e&&(t.beginGroup||e===a);s&&(a=r?e:e+1),e===a&&o<e&&n._renderSeparator(i),n._renderItem(e,t,i)}),this.hasIcons||i.addClass(T)}}},{key:"_renderContainer",value:function(e){var t=(0,m.default)("<ul>");return this.setAria("role","none",t),t.appendTo(e).addClass(M)}},{key:"_createDOMElement",value:function(e){var t=(0,m.default)("<li>");return this.setAria("role","none",t),t.appendTo(e).addClass(R)}},{key:"_renderItem",value:function(e,n,i,o){var a=this.option("items"),r=o||this._createDOMElement(i);a[e+1]&&a[e+1].beginGroup&&r.addClass(F);var s=l(_(t.prototype),"_renderItem",this).call(this,e,n.internalFields.item,r);n.internalFields.item===this.option("selectedItem")&&s.addClass(P),s.attr("tabIndex",-1),this._hasSubmenu(n)&&this.setAria("haspopup","true",s)}},{key:"_renderItemFrame",value:function(e,n,i){var o=i.children(".".concat(E));return o.length?o:l(_(t.prototype),"_renderItemFrame",this).apply(this,arguments)}},{key:"_refreshItem",value:function(e,t){var n=this._dataAdapter.getNodeByItem(t),i=e.data(this._itemIndexKey()),o=e.closest("ul"),a=e.closest("li");this._renderItem(i,n,o,a)}},{key:"_addContentClasses",value:function(e,t){var n=!!e.text&&!!e.text.length,i=!!e.icon,o=!!e.items&&!!e.items.length;t.toggleClass(L,n),t.toggleClass(H,i),this.hasIcons||(this.hasIcons=i),t.toggleClass(z,o)}},{key:"_getItemContent",value:function(e){var n=l(_(t.prototype),"_getItemContent",this).call(this,e);return n.length||(n=e.children(".".concat(A))),n}},{key:"_postprocessRenderItem",value:function(e){var t=(0,m.default)(e.itemElement),n=this._dataAdapter.getSelectedNodesKeys();if(!n.length||!this._selectedGetter(e.itemData)||!this._isItemSelectable(e.itemData))return void this._setAriaSelected(t,"false");var i=this._dataAdapter.getNodeByItem(e.itemData);i.internalFields.key===n[0]?(t.addClass(this._selectedItemClass()),this._setAriaSelected(t,"true")):this._setAriaSelected(t,"false")}},{key:"_isItemSelectable",value:function(e){return e.selectable!==!1}},{key:"_renderSeparator",value:function(e){(0,m.default)("<li>").appendTo(e).addClass(V)}},{key:"_itemClickHandler",value:function(e){if(!e._skipHandling){var t=this._createAction(this._updateSubmenuVisibilityOnClick.bind(this));this._itemDXEventHandler(e,"onItemClick",{},{afterExecute:t.bind(this)}),e._skipHandling=!0}}},{key:"_updateSubmenuVisibilityOnClick",value:function(e){this._updateSelectedItemOnClick(e),"onClick"===this._getShowSubmenuMode()&&this._addExpandedClass(e.args[0].itemElement)}},{key:"_updateSelectedItemOnClick",value:function(e){var t=e.args?e.args[0]:e;if(this._isItemSelectionAllowed(t.itemData)){var n=this._dataAdapter.getSelectedNodesKeys(),i=n.length&&this._dataAdapter.getNodeByKey(n[0]);i&&this._toggleItemSelection(i,!1),i&&i.internalFields.item===t.itemData?(this._fireSelectionChangeEvent(null,this.option("selectedItem")),this._setOptionWithoutOptionChange("selectedItem",null)):this.selectItem(t.itemData)}}},{key:"_isItemSelectionAllowed",value:function(e){var t=this._isSelectionEnabled()&&this.option("selectByClick");return!this._isContainerEmpty()&&t&&this._isItemSelectable(e)&&!this._itemsGetter(e)}},{key:"_isContainerEmpty",value:function(){return this._itemContainer().is(":empty")}},{key:"_syncSelectionOptions",value:function(){return(0,v.asyncNoop)()}},{key:"_optionChanged",value:function(e){switch(e.name){case"showSubmenuMode":break;case"selectedItem":var n=this._dataAdapter.getNodeByItem(e.value),i=this._dataAdapter.getSelectedNodesKeys()[0];if(n&&n.internalFields.key!==i){if(n.selectable===!1)break;i&&this._toggleItemSelection(this._dataAdapter.getNodeByKey(i),!1),this._toggleItemSelection(n,!0),this._updateSelectedItems()}break;case"cssClass":case"position":case"selectByClick":case"animation":case"useInkRipple":this._invalidate();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_toggleItemSelection",value:function(e,t){var n=this._getElementByItem(e.internalFields.item);n&&(0,m.default)(n).toggleClass(P),this._dataAdapter.toggleSelection(e.internalFields.key,t)}},{key:"_getElementByItem",value:function(e){var t,n=this;return(0,b.each)(this._itemElements(),function(i,o){return(0,m.default)(o).data(n._itemDataKey())!==e||(t=o,!1)}),t}},{key:"_updateSelectedItems",value:function(e,t){(e||t)&&this._fireSelectionChangeEvent(t,e)}},{key:"_fireSelectionChangeEvent",value:function(e,t){this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})({addedItems:[e],removedItems:[t]})}},{key:"selectItem",value:function(e){var t=e.nodeType?this._getItemData(e):e,n=this._dataAdapter.getSelectedNodesKeys()[0],i=this.option("selectedItem"),o=this._dataAdapter.getNodeByItem(t);o.internalFields.key!==n&&(n&&this._toggleItemSelection(this._dataAdapter.getNodeByKey(n),!1),this._toggleItemSelection(o,!0),this._updateSelectedItems(i,t),this._setOptionWithoutOptionChange("selectedItem",t))}},{key:"unselectItem",value:function(e){var t=e.nodeType?this._getItemData(e):e,n=this._dataAdapter.getNodeByItem(t),i=this.option("selectedItem");n.internalFields.selected&&(this._toggleItemSelection(n,!1),this._updateSelectedItems(i,null),this._setOptionWithoutOptionChange("selectedItem",null))}}]),t}(C.default);$.ItemClass=D.default;var K=$;t.default=K,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(19),r=n(0),s=n(3),l=i(n(13)),u=n(59),c=i(n(687)),d=i(n(58)),h=n(71),f=n(1),p=n(4),g="dx-state-disabled",_=d.default.inherit({_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{keyExpr:"id",displayExpr:"text",selectedExpr:"selected",disabledExpr:"disabled",itemsExpr:"items",hoverStateEnabled:!0,parentIdExpr:"parentId",expandedExpr:"expanded"})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===l.default.real().deviceType&&!l.default.isSimulator()},options:{focusStateEnabled:!0}}])},_init:function(){this.callBase(),this._initAccessors(),this._initDataAdapter(),this._initDynamicTemplates()},_initDataSource:function(){this.callBase(),this._dataSource&&this._dataSource.paginate(!1)},_initDataAdapter:function(){var e=this._createDataAdapterAccessors();this._dataAdapter=new c.default((0,r.extend)({dataAccessors:{getters:e.getters,setters:e.setters},items:this.option("items")},this._getDataAdapterOptions()))},_getDataAdapterOptions:p.noop,_initDynamicTemplates:function(){var e=this;this._templateManager.addDefaultTemplates({item:new h.BindableTemplate(function(t,n){t.html(n.html).append(this._getIconContainer(n)).append(this._getTextContainer(n)).append(this._getPopoutContainer(n)),e._addContentClasses(n,t.parent())}.bind(this),["text","html","items","icon"],this.option("integrationOptions.watchMethod"),{text:this._displayGetter,items:this._itemsGetter})})},_getIconContainer:function(e){return e.icon?(0,u.getImageContainer)(e.icon):void 0},_getTextContainer:function(e){return(0,o.default)("<span>").text(e.text)},_getPopoutContainer:p.noop,_addContentClasses:p.noop,_initAccessors:function(){var e=this;(0,s.each)(this._getAccessors(),function(t,n){e._compileAccessor(n)}),this._compileDisplayGetter()},_getAccessors:function(){return["key","selected","items","disabled","parentId","expanded"]},_getChildNodes:function(e){var t=this,n=[];return(0,s.each)(e.internalFields.childrenKeys,function(e,i){var o=t._dataAdapter.getNodeByKey(i);n.push(o)}),n},_hasChildren:function(e){return e&&e.internalFields.childrenKeys.length},_compileAccessor:function(e){var t="_"+e+"Getter",n="_"+e+"Setter",i=this.option(e+"Expr");return i?(0,f.isFunction)(i)?(this[n]=function(e,t){e[i()]=t},void(this[t]=function(e){return e[i()]})):(this[t]=(0,a.compileGetter)(i),void(this[n]=(0,a.compileSetter)(i))):(this[t]=p.noop,void(this[n]=p.noop))},_createDataAdapterAccessors:function(){var e=this,t={getters:{},setters:{}};return(0,s.each)(this._getAccessors(),function(n,i){var o="_"+i+"Getter",a="_"+i+"Setter",r="parentId"===i?"parentKey":i;t.getters[r]=e[o],t.setters[r]=e[a]}),t.getters.display=this._displayGetter?this._displayGetter:function(e){return e.text},t},_initMarkup:function(){this.callBase(),this._addWidgetClass()},_addWidgetClass:function(){this._focusTarget().addClass(this._widgetClass())},_widgetClass:p.noop,_renderItemFrame:function(e,t){var n=this.callBase.apply(this,arguments);return n.toggleClass(g,!!this._disabledGetter(t)),n},_optionChanged:function(e){switch(e.name){case"displayExpr":case"keyExpr":this._initAccessors(),this._initDynamicTemplates(),this.repaint();break;case"itemsExpr":case"selectedExpr":case"disabledExpr":case"expandedExpr":case"parentIdExpr":this._initAccessors(),this._initDataAdapter(),this.repaint();break;case"items":this._initDataAdapter(),this.callBase(e);break;default:this.callBase(e)}}}),m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(691));n(701),n(702),n(703),n(705),n(708),n(709),n(711),n(712),n(713),n(715),n(716),n(720),n(722),n(723),n(724),n(726),n(727),n(728),n(730),n(731),n(732),n(733),n(734),n(735);var a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(197),s=i(n(10)),l=n(1),u=n(3),c=n(0),d=n(302),h="text-content",f="headers",p="nowrap",g=".dx-row",_="dx-header-row",m="dx-column-lines",v="context-menu-sort-asc",y="context-menu-sort-desc",b="context-menu-sort-none",x="dx-cell-focus-disabled",w="dx-visibility-hidden",C="dx-text-content-alignment-",k="dx-sort-indicator",S="dx-sort-index-indicator",D=".dx-header-filter",I="dx-header-filter-indicator",T="dx-header-multi-row",O={defaultOptions:function(){return{showColumnHeaders:!0,cellHintEnabled:!0}},views:{columnHeadersView:r.ColumnsView.inherit(function(){function e(e,t,n,i){var o=e._getIndicatorElements(t,!0),a=e._getIndicatorElements(t),r=o&&o.length,s=e._getColumnAlignment(n.alignment),l=".".concat(e._getIndicatorClassName("sort")),u=".".concat(e._getIndicatorClassName("sortIndex")),c=a.filter(l),d=a.children().filter(u);i=i||t.children("."+e.addWidgetPrefix(h)),i.toggleClass(C+s,r>0).toggleClass(C+("left"===s?"right":"left"),r>0&&"center"===n.alignment).toggleClass(k,!!c.length).toggleClass(S,!!d.length).toggleClass(I,!!a.filter("."+e._getIndicatorClassName("headerFilter")).length)}var t=function(t,n,i){var a=(0,o.default)("<div>").addClass(t.addWidgetPrefix(h));t.setAria("role","presentation",a),e(t,n,i.column,a);var r=t.option("showColumnLines"),s=t.getController("columns").getHeaderContentAlignment(i.column.alignment);return a[r||"right"===s?"appendTo":"prependTo"](n)};return{_createTable:function(){var e=this.callBase.apply(this,arguments);return a.default.on(e,"mousedown selectstart",this.createAction(function(e){var t=e.event;t.shiftKey&&t.preventDefault()})),e},_isLegacyKeyboardNavigation:function(){return this.option("useLegacyKeyboardNavigation")},_getDefaultTemplate:function(e){var n=this;return function(i,o){var a=e.command?i:t(n,i,o),r="expand"!==e.command&&e.caption;r?a.text(r):e.command&&i.html("&nbsp;")}},_getHeaderTemplate:function(e){return e.headerCellTemplate||{allowRenderToDetachedContainer:!0,render:this._getDefaultTemplate(e)}},_processTemplate:function(e,n){var i,o=this,a=n.column,r=o.callBase(e);return i="header"===n.rowType&&r&&a.headerCellTemplate&&!a.command?{render:function(e){var n=t(o,e.container,e.model);r.render((0,c.extend)({},e,{container:n}))}}:r},_handleDataChanged:function(e){"refresh"===e.changeType&&(this._isGroupingChanged||this._requireReady)&&(this._isGroupingChanged=!1,this.render())},_renderCell:function(e,t){var n=this.callBase(e,t);return"header"===t.row.rowType&&(n.addClass(x),this._isLegacyKeyboardNavigation()||t.column&&!t.column.type&&n.attr("tabindex",this.option("tabindex")||0)),n},_setCellAriaAttributes:function(e,t){this.callBase(e,t),"header"===t.rowType&&(this.setAria("role","columnheader",e),!t.column||t.column.command||t.column.isBand||(e.attr("id",t.column.headerId),
this.setAria("label",s.default.format("dxDataGrid-ariaColumn")+" "+t.column.caption,e)))},_createRow:function(e){var t=this.callBase(e).toggleClass(m,this.option("showColumnLines"));return"header"===e.rowType&&(t.addClass(_),this._isLegacyKeyboardNavigation()||(0,d.registerKeyboardAction)("columnHeaders",this,t,"td",this._handleActionKeyDown.bind(this))),t},_handleActionKeyDown:function(e){var t=e.event,n=(0,o.default)(t.target);if(this._lastActionElement=t.target,n.is(D)){var i=this.getController("headerFilter"),a=n.closest("td"),r=this.getColumnIndexByElement(a);r>=0&&i.showHeaderFilterMenu(r,!1)}else{var s=n.closest(g);this._processHeaderAction(t,s)}t.preventDefault()},_renderCore:function(){var e=this,t=e.element();(!e._tableElement||e._dataController.isLoaded()||e._hasRowElements)&&(t.addClass(e.addWidgetPrefix(f)).toggleClass(e.addWidgetPrefix(p),!e.option("wordWrapEnabled")).empty(),e.setAria("role","presentation",t),e._updateContent(e._renderTable()),e.getRowCount()>1&&t.addClass(T),e.callBase.apply(e,arguments))},_renderRows:function(){var e=this;(e._dataController.isLoaded()||e._hasRowElements)&&(e.callBase.apply(e,arguments),e._hasRowElements=!0)},_getRowVisibleColumns:function(e){return this._columnsController.getVisibleColumns(e)},_renderRow:function(e,t){t.columns=this._getRowVisibleColumns(t.row.rowIndex),this.callBase(e,t)},_createCell:function(e){var t=e.column,n=this.callBase.apply(this,arguments);return t.rowspan>1&&"header"===e.rowType&&n.attr("rowSpan",t.rowspan),n},_getRows:function(){var e=[],t=this.getRowCount();if(this.option("showColumnHeaders"))for(var n=0;n<t;n++)e.push({rowType:"header",rowIndex:n});return e},_getCellTemplate:function(e){if("header"===e.rowType)return this._getHeaderTemplate(e.column)},_columnOptionChanged:function(e){var t=e.changeTypes,n=e.optionNames;return t.grouping?void(this._isGroupingChanged=!0):(this.callBase(e),void((n.width||n.visible)&&this.resizeCompleted.fire()))},_isElementVisible:function(e){return e&&e.visible},_alignCaptionByCenter:function(e){var t=this._getIndicatorContainer(e,!0);t&&t.length&&(t.filter("."+w).remove(),t=this._getIndicatorContainer(e),t.clone().addClass(w).css("float","").insertBefore(e.children("."+this.addWidgetPrefix(h))))},_updateCell:function(e,t){"header"===t.rowType&&"center"===t.column.alignment&&this._alignCaptionByCenter(e),this.callBase.apply(this,arguments)},_updateIndicator:function(t,n,i){var o=this.callBase.apply(this,arguments);return"center"===n.alignment&&this._alignCaptionByCenter(t),e(this,t,n),o},_getIndicatorContainer:function(e,t){var n=this.callBase(e);return t?n:n.filter(":not(."+w+")")},_isSortableElement:function(){return!0},getHeadersRowHeight:function(){var e=this._getTableElement(),t=e&&e.find("."+_);return t&&t.toArray().reduce(function(e,t){return e+(0,o.default)(t).height()},0)||0},getHeaderElement:function(e){var t=this.getColumnElements();return t&&t.eq(e)},getColumnElements:function(e,t){var n,i=this,a=i._columnsController,r=i.getRowCount();if(i.option("showColumnHeaders")){if(r>1&&(!(0,l.isDefined)(e)||(0,l.isDefined)(t))){var s=[],c=(0,l.isDefined)(t)?a.getChildrenByBandColumn(t,!0):a.getVisibleColumns();return(0,u.each)(c,function(t,o){var r=(0,l.isDefined)(e)?e:a.getRowIndex(o.index);n=i._getCellElement(r,a.getVisibleIndex(o.index,r)),n&&s.push(n.get(0))}),(0,o.default)(s)}if(!e||e<r)return i.getCellElements(e||0)}},getColumnIndexByElement:function(e){var t=this.getCellIndex(e),n=e.closest(".dx-row"),i=n[0].rowIndex,o=this.getColumns(i)[t];return o?o.index:-1},getVisibleColumnIndex:function(e,t){var n=this.getColumns()[e];return n?this._columnsController.getVisibleIndex(n.index,t):-1},getColumnWidths:function(){var e=this.getColumnElements();return e&&e.length?this._getWidths(e):this.callBase.apply(this,arguments)},allowDragging:function(e,t,n){var i,o=0,a=e&&this._columnsController.getRowIndex(e.index),r=this.getColumns(0===a?0:null),s=(null===e||void 0===e?void 0:e.allowHiding)&&r.length>1,l=function(e){return e.allowReordering||e.allowGrouping||e.allowHiding};for(i=0;i<r.length;i++)l(r[i])&&o++;if(o<=1&&!s)return!1;if(!n)return(this.option("allowColumnReordering")||this._columnsController.isColumnOptionUsed("allowReordering"))&&e&&e.allowReordering;for(i=0;i<n.length;i++){var u=n[i];if(u&&u.allowDragging(e,t))return!0}return!1},getBoundingRect:function(){var e=this,t=e.getColumnElements();if(t&&t.length){var n=e._getTableElement().offset();return{top:n.top}}return null},getName:function(){return"headers"},getColumnCount:function(){var e=this.getColumnElements();return e?e.length:0},isVisible:function(){return this.option("showColumnHeaders")},optionChanged:function(e){var t=this;switch(e.name){case"showColumnHeaders":case"wordWrapEnabled":case"showColumnLines":t._invalidate(!0,!0),e.handled=!0;break;default:t.callBase(e)}},getHeight:function(){return this.getElementHeight()},getContextMenuItems:function(e){var t=this,n=e.column;if(e.row&&("header"===e.row.rowType||"detailAdaptive"===e.row.rowType)){var i=t.option("sorting");if(i&&"none"!==i.mode&&n&&n.allowSorting){var o=function(e){setTimeout(function(){t._columnsController.changeSortOrder(n.index,e.itemData.value)})};return[{text:i.ascendingText,value:"asc",disabled:"asc"===n.sortOrder,icon:v,onItemClick:o},{text:i.descendingText,value:"desc",disabled:"desc"===n.sortOrder,icon:y,onItemClick:o},{text:i.clearText,value:"none",disabled:!n.sortOrder,icon:b,onItemClick:o}]}}},getRowCount:function(){return this._columnsController&&this._columnsController.getRowCount()},setRowsOpacity:function(e,t,n){var i,o,a=this,r=a.getRowCount(),s=a._columnsController.getColumns(),c=s&&s[e],d=c&&c.isBand&&c.index,h=function(e,n){n.ownerBand===d&&(o.eq(e).css({opacity:t}),n.isBand&&a.setRowsOpacity(n.index,t,i+1))};if((0,l.isDefined)(d))for(n=n||0,i=n;i<r;i++)o=a.getCellElements(i),(0,u.each)(a.getColumns(i),h)}}}())}};t.default=O,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=n(0),s=n(24),l="dx-column-indicators",u="dx-group-panel-item",c={_applyColumnState:function(e){var t,n=this,i=this.option("rtlEnabled"),a=n._getColumnAlignment(e.column.alignment,i),s=(0,r.extend)(!0,{columnAlignment:a},e),l=s.rootElement.hasClass(u),c=n._createIndicatorContainer(s,l),d=(0,o.default)("<span>").addClass(n._getIndicatorClassName(e.name)),h=null===(t=n.component)||void 0===t?void 0:t.getController("columns"),f=(null===h||void 0===h?void 0:h.getHeaderContentAlignment(a))||a;return s.container=c,s.indicator=d,n._renderIndicator(s),c[!l&&e.showColumnLines||"left"!==f?"prependTo":"appendTo"](e.rootElement),d},_getIndicatorClassName:a.noop,_getColumnAlignment:function(e,t){return t=t||this.option("rtlEnabled"),e&&"center"!==e?e:(0,s.getDefaultAlignment)(t)},_createIndicatorContainer:function(e,t){var n=this._getIndicatorContainer(e.rootElement),i="left"===e.columnAlignment?"right":"left";return n.length||(n=(0,o.default)("<div>").addClass(l)),this.setAria("role","presentation",n),n.css("float",e.showColumnLines&&!t?i:null)},_getIndicatorContainer:function(e){return e&&e.find("."+l)},_getIndicatorElements:function(e){var t=this._getIndicatorContainer(e);return t&&t.children()},_renderIndicator:function(e){var t=e.container,n=e.indicator;t&&n&&t.append(n)},_updateIndicators:function(e){var t,n=this,i=n.getColumns(),o=n.getColumnElements();if(o&&i.length===o.length)for(var a=0;a<i.length;a++){t=o.eq(a),n._updateIndicator(t,i[a],e);var r=t.parent().data("options");r&&r.cells&&(r.cells[t.index()].column=i[a])}},_updateIndicator:function(e,t,n){if(!t.command)return this._applyColumnState({name:n,rootElement:e,column:t,showColumnLines:this.option("showColumnLines")})}};t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?o(Object(n),!0).forEach(function(t){r(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):o(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var s=i(n(2)),l=i(n(27)),u=i(n(101)),c=n(19),d=n(4),h=n(1),f=n(3),p=n(24),g=n(0),_=n(12),m=i(n(28)),v=n(50),y=i(n(17)),b=i(n(44)),x=i(n(37)),w=n(40),C=i(n(62)),k=i(n(45)),S=i(n(32)),D=i(n(10)),I=n(6),T=i(n(102)),O=n(69),E=n(70),A=i(n(196)),P=["filterValues","filterType","fixed","fixedPosition"],R=["visibleIndex","dataField","name","dataType","width","visible","sortOrder","lastSortOrder","sortIndex","groupIndex","filterValue","selectedFilterOperation","added"].concat(P),M={visibleWidth:!0,bestFitWidth:!0,bufferedFilterValue:!0},B="dx-command-expand",V=Number.MAX_SAFE_INTEGER||9007199254740991,F="groupExpand",L=/columns\[(\d+)\]\.?/gi,H=1,z={defaultOptions:function(){return{commonColumnSettings:{allowFiltering:!0,allowHiding:!0,allowSorting:!0,allowEditing:!0,encodeHtml:!0,trueText:D.default.format("dxDataGrid-trueText"),falseText:D.default.format("dxDataGrid-falseText")},allowColumnReordering:!1,allowColumnResizing:!1,columnResizingMode:"nextColumn",columnMinWidth:void 0,columnWidth:void 0,adaptColumnWidthByRatio:!0,columns:void 0,regenerateColumnsByVisibleItems:!1,customizeColumns:null,dateSerializationFormat:void 0}},controllers:{columns:b.default.Controller.inherit(function(){function e(e,t){return e&&t&&(t.name===e.name||!e.name)&&(t.dataField===e.dataField||e.name)}function t(e,t){e._columns=t,ce(e),e.updateColumnDataTypes()}function n(e){return"asc"===e||"desc"===e}var i={visible:!0,showInColumnChooser:!0},o={number:["=","<>","<",">","<=",">=","between"],string:["contains","notcontains","startswith","endswith","=","<>"],date:["=","<>","<",">","<=",">=","between"],datetime:["=","<>","<",">","<=",">=","between"]},r={visibleIndex:!0,groupIndex:!0,grouped:!0,sortIndex:!0,sortOrder:!0},b="group",D="columnChooser",z=function(e){e.filterOperations=e.defaultFilterOperations},N=function(e,t,n,o){var a={};if(t){(0,h.isString)(t)&&(t={dataField:t}),e.setName(t);var r={};if(t.command)r=(0,v.deepExtendArraySafe)(a,t);else{a=e.getCommonSettings(t),n&&n.name&&n.dataField&&(t=(0,g.extend)({},t,{dataField:n.dataField}));var s=e._createCalculatedColumnOptions(t,o);t.type||(r={headerId:"dx-col-".concat(H++)}),r=(0,v.deepExtendArraySafe)(r,i),(0,v.deepExtendArraySafe)(r,a),(0,v.deepExtendArraySafe)(r,s),(0,v.deepExtendArraySafe)(r,t),(0,v.deepExtendArraySafe)(r,{selector:null})}return t.filterOperations===t.defaultFilterOperations&&z(r),r}},j=function t(n,i,o){var a=[];return i&&(0,f.each)(i,function(i,r){var s=n._columnsUserState&&e(r,n._columnsUserState[i])&&n._columnsUserState[i],l=N(n,r,s,o);l&&(o&&(l.ownerBand=o),a.push(l),l.columns&&(a=a.concat(t(n,l.columns,l)),delete l.columns,l.hasColumns=!0))}),a},W=function(e,t){for(var n=[],i=t[e];i;)n.unshift(i),e=i.index,i=t[e];return n},G=function(e,t,n){var i=[],o=t[e];if(o)for(var a=0;a<o.length;a++){var r=o[a];(0,h.isDefined)(r.groupIndex)&&!r.showWhenGrouped||(i.push(r),n&&r.isBand&&(i=i.concat(G(r.index,t,n))))}return i},q=function(e,t){var n,i,o=e.getBandColumnsCache(),a=function(e){var t=n?n.index:void 0;return e.ownerBand===t};if(o.isPlain)n=e._columns[t[0]];else{i=e._columns.filter(a);for(var r=0;r<t.length;r++)n=i[t[r]],n&&(i=e._columns.filter(a))}return n},$=function(e,t){var n,i=[],o=e.getBandColumnsCache(),a=function(e){return e.ownerBand===t.ownerBand};if(o.isPlain){var r=e._columns.indexOf(t);r>=0&&(i=["columns[".concat(r,"]")])}else for(n=e._columns.filter(a);n.length&&n.indexOf(t)!==-1;)i.unshift("columns[".concat(n.indexOf(t),"]")),t=o.columnParentByIndex[t.index],n=t?e._columns.filter(a):[];return i.join(".")},K=function e(t,n){var i=0,o=t.getChildrenByBandColumn(n,!0);return(0,f.each)(o,function(n,o){o.isBand?(o.colspan=o.colspan||e(t,o.index),i+=o.colspan||1):i+=1}),i},U=function(e,t,n){for(var i,o=0;o<t.length;o++){var a=t[o];(a.visible||a.command)&&(a.isBand&&(a.colspan=a.colspan||K(e,a.index)),a.isBand&&a.colspan||(i=e.getRowCount(),a.command||(0,h.isDefined)(a.groupIndex)&&!a.showWhenGrouped||(i-=W(a.index,n.columnParentByIndex).length),i>1&&(a.rowspan=i)))}},Y=function(e){var t=(0,h.type)(e);return"string"!==t&&"boolean"!==t&&"number"!==t&&"date"!==t&&"object"!==t&&(t=void 0),t},Z=function(e,t){switch(e){case"date":case"datetime":return C.default.getDateSerializationFormat(t);case"number":if((0,h.isString)(t))return"string";if((0,h.isNumeric)(t))return null}},X=function(e,t){e.deserializeValue||(x.default.isDateType(t)&&(e.deserializeValue=function(e){return C.default.deserializeDate(e)},e.serializeValue=function(e){return(0,h.isString)(e)?e:C.default.serializeDate(e,this.serializationFormat)}),"number"===t&&(e.deserializeValue=function(e){var t=parseFloat(e);return isNaN(t)?e:t},e.serializeValue=function(e,t){return"filter"===t?e:(0,h.isDefined)(e)&&"string"===this.serializationFormat?e.toString():e}))},Q=function(e,t){switch(e){case"number":return"right";case"boolean":return"center";default:return(0,p.getDefaultAlignment)(t)}},J=function(e){return e.value===!0?this.trueText||"true":e.value===!1?this.falseText||"false":e.valueText||""},ee=function(e){if("boolean"===e)return J},te=function(e,t){for(var n,i=e._getFirstItems(t),o={},a=[],r=0;r<i.length;r++)if(i[r])for(n in i[r])(0,h.isFunction)(i[r][n])&&!u.default.isWrapped(i[r][n])||(o[n]=!0);for(n in o)if(0!==n.indexOf("__")){var s=N(e,n);a.push(s)}return a},ne=function(e){(0,f.each)(e._columns,function(e,t){t.index=e}),(0,f.each)(e._columns,function(e,t){(0,h.isObject)(t.ownerBand)&&(t.ownerBand=t.ownerBand.index)}),(0,f.each)(e._commandColumns,function(e,t){t.index=-(e+1)})},ie=function(e,t){(0,_.normalizeIndexes)(e._columns,"groupIndex",t,function(e){var t=e.grouped;return delete e.grouped,t})},oe=function(e,t){(0,f.each)(e._columns,function(e,t){(0,h.isDefined)(t.sortIndex)&&!n(t.sortOrder)&&delete t.sortIndex}),(0,_.normalizeIndexes)(e._columns,"sortIndex",t,function(e){return!(0,h.isDefined)(e.groupIndex)&&n(e.sortOrder)})},ae=function(e,t){for(var n,i,o={},a=[],r=e.getBandColumnsCache(),s=e._columns.filter(function(e){return!e.command}),l=0;l<s.length;l++){i=s[l];var u=W(l,r.columnParentByIndex);if(u.length){var c=u[u.length-1].index;o[c]=o[c]||[],o[c].push(i)}else a.push(i)}for(n in o)(0,_.normalizeIndexes)(o[n],"visibleIndex",t);(0,_.normalizeIndexes)(a,"visibleIndex",t)},re=function(e,t,n){var i,o=(0,h.isObject)(t)?t.rowIndex:null,a=n===b?e.getGroupColumns():n===D?e.getChooserColumns():e.getVisibleColumns(o);return t=(0,h.isObject)(t)?t.columnIndex:t,i=a[t],i&&i.type===F&&(i=e._columns.filter(function(e){return i.type===e.type})[0]||i),i&&(0,h.isDefined)(i.index)?i.index:-1},se=function(e,t,n){var i,o=e.getGroupColumns();if(n>=0)for(i=0;i<o.length;i++)o[i].groupIndex>=n&&o[i].groupIndex++;else for(n=0,i=0;i<o.length;i++)n=Math.max(n,o[i].groupIndex+1);return n},le=function(n){function i(e,t){if(t)for(var n=0;n<R.length;n++){var i=R[n];(0,_.inArray)(i,u)>=0||("dataType"===i?e[i]=e[i]||t[i]:(0,_.inArray)(i,P)>=0?i in t&&(e[i]=t[i]):("selectedFilterOperation"===i&&t[i]&&(e.defaultSelectedFilterOperation=e[i]||null),e[i]=t[i]))}}function o(t,n){for(var i=n.name||n.dataField,o=d[i]||0,a=0;a<t.length;a++)if(e(n,t[a])){if(!o)return d[i]=d[i]||0,d[i]++,a;o--}return-1}var a,r,s,l=n._columnsUserState,u=n._ignoreColumnOptionNames||[],c=n._columns,d={},f=[],p=!0,g=[];if(l){for(s=0;s<c.length;s++)r=o(l,c[s]),p=p&&r>=0,g.push(r);for(s=0;s<c.length;s++)a=c[s],r=g[s],(n._hasUserState||p)&&i(a,l[r]),r>=0&&(0,h.isDefined)(l[r].initialIndex)?f[r]=a:f.push(a);var m=!1;for(s=0;s<l.length;s++){var v=l[s];v.added&&o(c,v)<0&&(a=N(n,v.added),i(a,v),f.push(a),v.added.columns&&(m=!0))}m&&(ne(n),f=j(n,f)),t(n,f)}},ue=function(e,t){ne(e),ie(e,t),oe(e,t),Te(e),ae(e,t)},ce=function(e){e.resetColumnsCache()},de=function(e,t,n,i){var o=e._columnChanges||{optionNames:{length:0},changeTypes:{length:0},columnIndex:i};n=n||"all",n=n.split(".")[0];var a=o.changeTypes;t&&!a[t]&&(a[t]=!0,a.length++);var r=o.optionNames;n&&!r[n]&&(r[n]=!0,r.length++),void 0!==i&&i===o.columnIndex||delete o.columnIndex,e._columnChanges=o,ce(e)},he=function(e){var t=e.option("onColumnsChanging"),n=e._columnChanges,i=["dataField","lookup","dataType","columns"],o=function(e){return e&&i.some(function(t){return e[t]})};e.isInitialized()&&!e._updateLockCount&&n&&(t&&(e._updateLockCount++,t((0,g.extend)({component:e.component},n)),e._updateLockCount--),e._columnChanges=void 0,o(n.optionNames)?e.reinit():e.columnsChanged.fire(n))},fe=function(e,t,n,i){var o=i>=0;if(n>=0)o||(t.lastSortOrder=t.sortOrder);else{var a=e.option("sorting.mode"),r=t.lastSortOrder;if("single"===a){var s=e._columns.some(function(e){return e!==t&&(0,h.isDefined)(e.sortIndex)});s&&(r=void 0)}t.sortOrder=r}},pe=function(e,t){var n=t.value,i=t.optionName,o=t.prevValue,a=t.fullOptionName,r="".concat(a,".").concat(i);M[i]||e._skipProcessingColumnsChange===r||(e._skipProcessingColumnsChange=r,e.component._notifyOptionChanged(r,n,o),e._skipProcessingColumnsChange=!1)},ge=function(t,n,i,o,a){var s,l,u,d=(0,c.compileGetter)(i),f=n.index;if(3===arguments.length)return d(n,{functionsAsIs:!0});var p=d(n,{functionsAsIs:!0});if(p!==o){"groupIndex"===i||"calculateGroupValue"===i?(l="grouping",fe(t,n,o,p)):l="sortIndex"===i||"sortOrder"===i||"calculateSortValue"===i?"sorting":"columns";var g=(0,c.compileSetter)(i);g(n,o,{functionsAsIs:!0});var m=$(t,n);r[i]&&(ue(t,n),o=d(n)),"name"!==i&&"allowEditing"!==i||t._checkColumns(),m&&pe(t,{fullOptionName:m,optionName:i,value:o,prevValue:p}),(0,h.isDefined)(p)||(0,h.isDefined)(o)||0===i.indexOf("buffer")||(a=!0),a?ce(t):((0,_.inArray)(i,R)<0&&"visibleWidth"!==i&&(s=t.option("columns"),u=t.getColumnByPath(m,s),(0,h.isString)(u)&&(u=s[f]={dataField:u}),u&&e(u,n)&&g(u,o,{functionsAsIs:!0})),de(t,l,i,f))}},_e=function(e){var t=e._getExpandColumnOptions();e.addCommandColumn(t)},me=function(e,t){var n=this.dataField.split("."),i=n.length-1;this.serializeValue&&(t=this.serializeValue(t));for(var o=0;o<i;o++){var a=n[o];e=e[a]=e[a]||{}}e[n[i]]=t},ve=function e(t,n,i){var o=[];return n=n||0,t[n]&&(0,f.each)(t[n],function(a,r){r.ownerBand!==i&&r.type!==F||(r.isBand&&r.colspan?o.push.apply(o,e(t,n+1,r.index)):(!r.command||n<1)&&o.push(r))}),o},ye=function(e){var t=1,n=e.getBandColumnsCache(),i=n.columnParentByIndex;return e._columns.forEach(function(e){var n=W(e.index,i),o=n.filter(function(e){return!e.visible});e.visible&&!o.length&&(t=Math.max(t,n.length+1))}),t},be=function(e,t){return!!e._columns.filter(function(e){return e.type===t.type}).length},xe=function(e,t){var n=e.option("rtlEnabled");return t.command&&!be(e,t)||!t.fixedPosition?n?"right":"left":t.fixedPosition},we=function(e,t,n,i){var o,a=this.getRowCount(),r=e[i]&&e[i].rowspan,s=t.filter(function(e){return e.type===n});return e.forEach(function(t,i){t.type===n&&(o=i,r=e[i+1]?e[i+1].rowspan:a)}),r>1&&(s=(0,f.map)(s,function(e){return(0,g.extend)({},e,{rowspan:r})})),s.unshift.apply(s,(0,h.isDefined)(o)?[o,1]:[i,0]),e.splice.apply(e,s),r||1},Ce=function(e){var t;for(t=0;e>1;t++)e/=10;return t},ke=function(e,t){for(var n=e?e.toString():"0";n.length<t;)n="0"+n;return n},Se=function(e,t,n,i){for(var o,a,r=t.slice().map(function(e){return(0,g.extend)({},e)}),s=e._isColumnFixing(),l=n.slice().map(function(e){return(0,g.extend)({fixed:s},e)}),u=function(e){return n.reduce(function(t,n,o){var a=i&&e.type===F?"expand":e.type;return n.type===a||n.command===e.command?o:t},-1)},c=function(e){return e.command!==n[a].command},d=0;d<t.length;d++)if(o=t[d],a=o&&(o.type||o.command)?u(o):-1,a>=0)if(i)r[d]=(0,g.extend)({fixed:s},n[a],o),o.type!==F&&(l=l.filter(c));else{var h={visibleIndex:o.visibleIndex,index:o.index,headerId:o.headerId,allowFixing:0===o.groupIndex,allowReordering:0===o.groupIndex,groupIndex:o.groupIndex};r[d]=(0,g.extend)({},o,n[a],o.type===F&&h)}return t.length&&i&&l.length&&(r=r.concat(l)),r},De=function(e,t){return(0,h.isDefined)(t.fixed)||!t.type?t.fixed:e._isColumnFixing()},Ie=function(e){e.forEach(function(t){(0,h.isDefined)(t.ownerBand)&&(t.ownerBand=e[t.ownerBand])})},Te=function(e){e._bandColumnsCache=void 0},Oe=function(e,t){var n,i=(0,h.isString)(t)&&t.substr(0,t.indexOf(":"));if(void 0!==t)return i&&(t=t.substr(i.length+1)),i?n=e.filter(function(e){return""+e[i]===t})[0]:["index","name","dataField","caption"].some(function(i){return n=e.filter(function(e){return e[i]===t})[0],!!n}),n};return{_getExpandColumnOptions:function(){return{type:"expand",command:"expand",width:"auto",cssClass:B,allowEditing:!1,allowGrouping:!1,allowSorting:!1,allowResizing:!1,allowReordering:!1,allowHiding:!1}},_getFirstItems:function(e){var t,n=[],i=function e(t,n){if(!t||!n)return t;for(var i=0;i<t.length;i++){var o=e(t[i].items||t[i].collapsedItems,n-1);if(o&&o.length)return o}};return e&&e.items().length>0&&(t=x.default.normalizeSortingInfo(e.group()).length,n=i(e.items(),t)||[]),n},_endUpdateCore:function(){!this._skipProcessingColumnsChange&&he(this)},init:function(){var e=this,n=e.option("columns");e._commandColumns=e._commandColumns||[],e._columns=e._columns||[],e._isColumnsFromOptions=!!n,e._isColumnsFromOptions?(t(e,n?j(e,n):[]),le(e)):t(e,e._columnsUserState?j(e,e._columnsUserState):e._columns),_e(e),e._dataSourceApplied?e.applyDataSource(e._dataSource,!0):ue(e),e._checkColumns()},callbackNames:function(){return["columnsChanged"]},getColumnByPath:function(e,t){var n,i=this,o=[];return e.replace(L,function(e,t){return o.push(parseInt(t)),""}),o.length&&(n=t?o.reduce(function(e,t){return e&&e.columns&&e.columns[t]},{columns:t}):q(i,o)),n},optionChanged:function(e){var t;switch(e.name){case"adaptColumnWidthByRatio":e.handled=!0;break;case"dataSource":e.value===e.previousValue||this.option("columns")||Array.isArray(e.value)&&Array.isArray(e.previousValue)||(this._columns=[]);break;case"columns":t=this._skipProcessingColumnsChange,e.handled=!0,this._skipProcessingColumnsChange||(e.name===e.fullName?(this._columnsUserState=null,this._ignoreColumnOptionNames=null,this.init()):(this._columnOptionChanged(e),t=!0)),t&&this._updateRequireResize(e);break;case"commonColumnSettings":case"columnAutoWidth":case"allowColumnResizing":case"allowColumnReordering":case"columnFixing":case"grouping":case"groupPanel":case"regenerateColumnsByVisibleItems":case"customizeColumns":case"editing":case"columnHidingEnabled":case"dateSerializationFormat":case"columnResizingMode":case"columnMinWidth":case"columnWidth":var n,i,o,a,r;e.handled=!0;var s="columnWidth"===e.fullName&&["width"],l=0===(null===(n=e.fullName)||void 0===n?void 0:n.indexOf("editing.popup")),u=0===(null===(i=e.fullName)||void 0===i?void 0:i.indexOf("editing.form")),c=0===(null===(o=e.fullName)||void 0===o?void 0:o.indexOf("editing.editRowKey")),d=0===(null===(a=e.fullName)||void 0===a?void 0:a.indexOf("editing.editColumnName")),h=0===(null===(r=e.fullName)||void 0===r?void 0:r.indexOf("editing.changes")),f=!(l||u||c||h||d);f&&this.reinit(s);break;case"rtlEnabled":this.reinit();break;default:this.callBase(e)}},_columnOptionChanged:function(e){var t={},n=this.getColumnByPath(e.fullName),i=e.fullName.replace(L,"");n&&(i?t[i]=e.value:t=e.value,this._skipProcessingColumnsChange=e.fullName,this.columnOption(n.index,t),this._skipProcessingColumnsChange=!1)},_updateRequireResize:function(e){var t=this.component;"width"===e.fullName.replace(L,"")&&t._updateLockCount&&(t._requireResize=!0)},publicMethods:function(){return["addColumn","deleteColumn","columnOption","columnCount","clearSorting","clearGrouping","getVisibleColumns","getVisibleColumnIndex"]},applyDataSource:function(e,n){var i=this,o=e&&e.isLoaded();if(i._dataSource=e,!i._dataSourceApplied||0===i._dataSourceColumnsCount||n||i.option("regenerateColumnsByVisibleItems")){if(o){if(!i._isColumnsFromOptions){var a=te(i,e);a.length&&(t(i,a),i._dataSourceColumnsCount=i._columns.length,le(i))}return i.updateColumns(e,n)}i._dataSourceApplied=!1}else if(o&&!i.isAllDataTypesDefined(!0)&&i.updateColumnDataTypes(e))return de(i,"columns"),he(i),(new I.Deferred).reject().promise()},reset:function(){this._dataSourceApplied=!1,this._dataSourceColumnsCount=void 0,this.reinit()},resetColumnsCache:function(){var e=this;e._visibleColumns=void 0,e._fixedColumns=void 0,e._rowCount=void 0,Te(e)},reinit:function(e){this._columnsUserState=this.getUserState(),this._ignoreColumnOptionNames=e||null,this.init(),e&&(this._ignoreColumnOptionNames=null)},isInitialized:function(){return!!this._columns.length||!!this.option("columns")},isDataSourceApplied:function(){return this._dataSourceApplied},getCommonSettings:function(e){var t=(!e||!e.type)&&this.option("commonColumnSettings")||{},n=this.option("grouping")||{},i=this.option("groupPanel")||{};return(0,g.extend)({allowFixing:this.option("columnFixing.enabled"),allowResizing:this.option("allowColumnResizing")||void 0,allowReordering:this.option("allowColumnReordering"),minWidth:this.option("columnMinWidth"),width:this.option("columnWidth"),autoExpandGroup:n.autoExpandAll,allowCollapsing:n.allowCollapsing,allowGrouping:i.allowColumnDragging&&i.visible||n.contextMenuEnabled},t)},isColumnOptionUsed:function(e){for(var t=0;t<this._columns.length;t++)if(this._columns[t][e])return!0},isAllDataTypesDefined:function(e){var t=this._columns;if(!t.length)return!1;for(var n=0;n<t.length;n++)if((t[n].dataField||t[n].calculateCellValue!==t[n].defaultCalculateCellValue)&&(!t[n].dataType||e&&t[n].deserializeValue&&void 0===t[n].serializationFormat))return!1;return!0},getColumns:function(){return this._columns},isBandColumnsUsed:function(){return this.getColumns().some(function(e){return e.isBand})},getGroupColumns:function(){var e=[];return(0,f.each)(this._columns,function(){var t=this;(0,h.isDefined)(t.groupIndex)&&(e[t.groupIndex]=t)}),e},getVisibleColumns:function(e){return this._visibleColumns=this._visibleColumns||this._getVisibleColumnsCore(),e=(0,h.isDefined)(e)?e:this._visibleColumns.length-1,this._visibleColumns[e]||[]},getFixedColumns:function(e){return this._fixedColumns=this._fixedColumns||this._getFixedColumnsCore(),e=(0,h.isDefined)(e)?e:this._fixedColumns.length-1,this._fixedColumns[e]||[]},getFilteringColumns:function(){return this.getColumns().filter(function(e){return(e.dataField||e.name)&&(e.allowFiltering||e.allowHeaderFiltering)}).map(function(e){var t=(0,g.extend)(!0,{},e);return(0,h.isDefined)(t.dataField)||(t.dataField=t.name),t.filterOperations=e.filterOperations!==e.defaultFilterOperations?t.filterOperations:null,t})},getColumnIndexOffset:function(){return 0},_getFixedColumnsCore:function(){var e,t,n,i=this,o=[],r=i.getRowCount(),s=i._isColumnFixing(),l={command:"transparent"},u=0;if(s)for(var c=0;c<=r;c++){e=0,n=null,t=null;for(var d=i.getVisibleColumns(c,!0),f=0;f<d.length;f++){var p=d[f-1],g=d[f];g.fixed?p&&p.fixed&&xe(i,p)!==xe(i,g)?(0,h.isDefined)(t)||(t=f):n=g.fixedPosition:(0===c&&(g.isBand&&g.colspan?u+=g.colspan:u++),e++,(0,h.isDefined)(t)||(t=f))}if(0===c&&(0===e||e>=d.length))return[];(0,h.isDefined)(t)||(t="right"===n?0:d.length),o[c]=d.slice(0),l.colspan||(l.colspan=u),o[c].splice(t,e,l)}return o.map(function(e){return e.map(function(e){var t=a({},e);return t.headerId&&(t.headerId+="-fixed"),t})})},_isColumnFixing:function(){var e=this.option("columnFixing.enabled");return!e&&(0,f.each)(this._columns,function(t,n){if(n.fixed)return e=!0,!1}),e},_getExpandColumnsCore:function(){return this.getGroupColumns()},getExpandColumns:function(){var e,t=this._getExpandColumnsCore(),n=t.filter(function(e){return 0===e.groupIndex})[0],i=n&&n.fixed,o=this._isColumnFixing();return t.length&&(e=this.columnOption("command:expand")),t=(0,f.map)(t,function(t){return(0,g.extend)({},t,{visibleWidth:null,minWidth:null,cellTemplate:(0,h.isDefined)(t.groupIndex)?null:t.cellTemplate,headerCellTemplate:null,fixed:!(!(0,h.isDefined)(t.groupIndex)||!i)||o},e,{index:t.index,type:t.type||F})})},getBandColumnsCache:function(){if(!this._bandColumnsCache){var e=this._columns,t={},n={},i=!0;e.forEach(function(o){var a=o.ownerBand,r=e[a];o.hasColumns&&(i=!1),o.colspan&&(o.colspan=void 0),o.rowspan&&(o.rowspan=void 0),r?n[o.index]=r:a=-1,t[a]=t[a]||[],t[a].push(o)}),this._bandColumnsCache={isPlain:i,columnChildrenByIndex:t,columnParentByIndex:n}}return this._bandColumnsCache},_isColumnVisible:function(e){return e.visible&&this.isParentColumnVisible(e.index)},_getVisibleColumnsCore:function(){var e,t,n=this,i=[],o=0,a=0,r=n.getRowCount(),s=[],l=[],u=0,c=n.option("rtlEnabled"),d=n.getBandColumnsCache(),p=Se(n,n.getExpandColumns(),n._columns),g=Se(n,n._columns,n._commandColumns,!0),_=Ce(g.length);for(U(n,g,d),e=0;e<r;e++)i[e]=[],l[e]=[{}],s[e]=[{},{},{}];return(0,f.each)(g,function(){var i,o=this,a=o.visibleIndex,r=W(o.index,d.columnParentByIndex),f=n._isColumnVisible(o);if(f&&(!(0,h.isDefined)(o.groupIndex)||o.showWhenGrouped)){var p=r.length;if(a<0?(a=-a,i=l[p]):(o.fixed=r.length?r[0].fixed:o.fixed,o.fixedPosition=r.length?r[0].fixedPosition:o.fixedPosition,o.fixed?(t="right"===o.fixedPosition,!c||o.command&&!be(n,o)||(t=!t),i=t?s[p][2]:s[p][0]):i=s[p][1]),r.length)for(a=ke(a,_),e=r.length-1;e>=0;e--)a=ke(r[e].visibleIndex,_)+a;i[a]=i[a]||[],i[a].push(o),u++}}),(0,f.each)(i,function(e){(0,v.orderEach)(l[e],function(t,n){i[e].unshift.apply(i[e],n)});var t=i[e].length;(0,f.each)(s[e],function(t,n){(0,v.orderEach)(n,function(t,n){i[e].push.apply(i[e],n)})}),a<e+1&&(a+=we.call(n,i[e],p,"detailExpand",t)),o<e+1&&(o+=we.call(n,i[e],p,F,t))}),i.push(ve(i)),!u&&n._columns.length&&i[r].push({command:"empty"}),i},getInvisibleColumns:function(e,t){var n,i=this,o=[];return e=e||i._columns,(0,f.each)(e,function(e,a){if(a.ownerBand===t)return a.isBand?(n=a.visible?i.getInvisibleColumns(i.getChildrenByBandColumn(a.index),a.index):i.getChildrenByBandColumn(a.index),void(n.length&&(o.push(a),o=o.concat(n)))):void(a.visible||o.push(a))}),o},getChooserColumns:function(e){var t=e?this.getColumns():this.getInvisibleColumns();return(0,d.grep)(t,function(e){return e.showInColumnChooser})},allowMoveColumn:function(e,t,n,i){var o=this,a=re(o,e,n),r=o._columns[a];return!(!r||!(r.allowReordering||r.allowGrouping||r.allowHiding))&&(n===i?n!==D&&(e=(0,h.isObject)(e)?e.columnIndex:e,t=(0,h.isObject)(t)?t.columnIndex:t,e!==t&&e+1!==t):n===b&&i!==D||i===b?r&&r.allowGrouping:n!==D&&i!==D||r&&r.allowHiding)},moveColumn:function(e,t,n,i){var o,a,r=this,s={},l=re(r,e,n),u=re(r,t,i);if(l>=0){var c=r._columns[l];if(t=(0,h.isObject)(t)?t.columnIndex:t,a=u>=0?r._columns[u].groupIndex:-1,(0,h.isDefined)(c.groupIndex)&&n===b&&(a>c.groupIndex&&a--,i!==b?s.groupIndex=void 0:(o=c.groupIndex,delete c.groupIndex,ie(r))),i===b)s.groupIndex=se(r,c,a),c.groupIndex=o;else if(t>=0){var d=r._columns[u];d&&c.ownerBand===d.ownerBand?De(r,c)^De(r,d)?s.visibleIndex=V:s.visibleIndex=d.visibleIndex:s.visibleIndex=V}var f=i!==D;c.visible!==f&&(s.visible=f),r.columnOption(c.index,s)}},changeSortOrder:function(e,t){var i=this,o={},a=i.option("sorting"),r=a&&a.mode,s="single"===r||!t,l="single"===r||"multiple"===r,u=i._columns[e],c=function(e){if("ctrl"===t){if(!("sortOrder"in e&&"sortIndex"in e))return!1;o.sortOrder=void 0,o.sortIndex=void 0}else(0,h.isDefined)(e.groupIndex)||(0,h.isDefined)(e.sortIndex)?o.sortOrder="desc"===e.sortOrder?"asc":"desc":o.sortOrder="asc";return!0};l&&u&&u.allowSorting&&(s&&!(0,h.isDefined)(u.groupIndex)&&(0,f.each)(i._columns,function(t){t!==e&&this.sortOrder&&((0,h.isDefined)(this.groupIndex)||delete this.sortOrder,delete this.sortIndex)}),n(t)?u.sortOrder!==t&&(o.sortOrder=t):"none"===t?u.sortOrder&&(o.sortIndex=void 0,o.sortOrder=void 0):c(u)),i.columnOption(u.index,o)},getSortDataSourceParameters:function(e){var t=this,i=[],o=[];return(0,f.each)(t._columns,function(){
(this.dataField||this.selector||this.calculateCellValue)&&(0,h.isDefined)(this.sortIndex)&&!(0,h.isDefined)(this.groupIndex)&&(i[this.sortIndex]=this)}),(0,f.each)(i,function(){var t=this&&this.sortOrder;if(n(t)){var i={selector:this.calculateSortValue||this.displayField||this.calculateDisplayValue||e&&this.selector||this.dataField||this.calculateCellValue,desc:"desc"===this.sortOrder};this.sortingMethod&&(i.compare=this.sortingMethod.bind(this)),o.push(i)}}),o.length>0?o:null},getGroupDataSourceParameters:function(e){var t=[];return(0,f.each)(this.getGroupColumns(),function(){var n=this.calculateGroupValue||this.displayField||this.calculateDisplayValue||e&&this.selector||this.dataField||this.calculateCellValue;if(n){var i={selector:n,desc:"desc"===this.sortOrder,isExpanded:!!this.autoExpandGroup};this.sortingMethod&&(i.compare=this.sortingMethod.bind(this)),t.push(i)}}),t.length>0?t:null},refresh:function(e){var t=[];return(0,f.each)(this._columns,function(){var n=this.lookup;if(n&&!this.calculateDisplayValue){if(e&&n.valueMap)return;n.update&&t.push(n.update())}}),I.when.apply(s.default,t).done(ce.bind(null,this))},_updateColumnOptions:function(e,t){e.selector=e.selector||function(t){return e.calculateCellValue(t)},(0,f.each)(["calculateSortValue","calculateGroupValue","calculateDisplayValue"],function(n,i){var o=e[i];(0,h.isFunction)(o)&&!o.originalCallback&&(e[i]=function(t){return o.call(e,t)},e[i].originalCallback=o,e[i].columnIndex=t)}),(0,h.isString)(e.calculateDisplayValue)&&(e.displayField=e.calculateDisplayValue,e.calculateDisplayValue=(0,c.compileGetter)(e.displayField)),e.calculateDisplayValue&&(e.displayValueMap=e.displayValueMap||{}),X(e,e.dataType);var n=e.lookup;n&&X(n,n.dataType);var i=n?n.dataType:e.dataType;i&&(e.alignment=e.alignment||Q(i,this.option("rtlEnabled")),e.format=e.format||x.default.getFormatByDataType(i),e.customizeText=e.customizeText||ee(i),e.defaultFilterOperations=e.defaultFilterOperations||!n&&o[i]||[],(0,h.isDefined)(e.filterOperations)||z(e),e.defaultFilterOperation=e.filterOperations&&e.filterOperations[0]||"=",e.showEditorAlways=(0,h.isDefined)(e.showEditorAlways)?e.showEditorAlways:"boolean"===i&&!e.cellTemplate)},updateColumnDataTypes:function(e){var t=this,n=t.option("dateSerializationFormat"),i=t._getFirstItems(e),o=!1;return(0,f.each)(t._columns,function(e,a){var r,s,l,u,c,d=a.lookup;if(x.default.isDateType(a.dataType)&&void 0===a.serializationFormat&&(a.serializationFormat=n),d&&x.default.isDateType(d.dataType)&&void 0===a.serializationFormat&&(d.serializationFormat=n),a.calculateCellValue&&i.length){if(!a.dataType||d&&!d.dataType){for(r=0;r<i.length;r++)s=a.calculateCellValue(i[r]),a.dataType||(c=Y(s),l=l||c,l&&c&&l!==c&&(l="string")),d&&!d.dataType&&(c=Y(x.default.getDisplayValue(a,s,i[r])),u=u||c,u&&c&&u!==c&&(u="string"));(l||u)&&(l&&(a.dataType=l),d&&u&&(d.dataType=u),o=!0)}if(void 0===a.serializationFormat||d&&void 0===d.serializationFormat)for(r=0;r<i.length;r++)s=a.calculateCellValue(i[r],!0),void 0===a.serializationFormat&&(a.serializationFormat=Z(a.dataType,s)),d&&void 0===d.serializationFormat&&(d.serializationFormat=Z(d.dataType,d.calculateCellValue(s,!0)))}t._updateColumnOptions(a,e)}),o},_customizeColumns:function(e){var n=this,i=n.option("customizeColumns");if(i){var o=e.some(function(e){return(0,h.isObject)(e.ownerBand)});o&&ue(n),i(e),t(n,j(n,e))}},updateColumns:function(e,t){var n=this;if(t||this.updateSortingGrouping(e),!e||e.isLoaded()){var i=e?e.sort()||[]:this.getSortDataSourceParameters(),o=e?e.group()||[]:this.getGroupDataSourceParameters(),a=null===e||void 0===e?void 0:e.lastLoadOptions().filter;this._customizeColumns(this._columns),ue(this);var r=this._columns;return(0,I.when)(this.refresh(!0)).always(function(){n._columns===r&&(n._updateChanges(e,{sorting:i,grouping:o,filtering:a}),he(n))})}},_updateChanges:function(e,t){e&&(this.updateColumnDataTypes(e),this._dataSourceApplied=!0),x.default.equalSortParameters(t.sorting,this.getSortDataSourceParameters())||de(this,"sorting"),x.default.equalSortParameters(t.grouping,this.getGroupDataSourceParameters())||de(this,"grouping");var n=this.getController("data");n&&!x.default.equalFilterParameters(t.filtering,n.getCombinedFilter())&&de(this,"filtering"),de(this,"columns")},updateSortingGrouping:function(e,n){var i,o,a=this,r=function(e,t,n){(0,f.each)(e,function(e,i){if(delete i[n],t)for(var o=0;o<t.length;o++){var a=t[o].selector,r=t[o].isExpanded;if(a===i.dataField||a===i.name||a===i.selector||a===i.calculateCellValue||a===i.calculateGroupValue||a===i.calculateDisplayValue){i.sortOrder=i.sortOrder||(t[o].desc?"desc":"asc"),void 0!==r&&(i.autoExpandGroup=r),i[n]=o;break}}})};if(e){i=x.default.normalizeSortingInfo(e.sort());var s=x.default.normalizeSortingInfo(e.group()),l=a.getGroupDataSourceParameters(),u=a.getSortDataSourceParameters();a._columns.length||((0,f.each)(s,function(e,t){a._columns.push(t.selector)}),(0,f.each)(i,function(e,t){a._columns.push(t.selector)}),t(a,j(a,a._columns))),!n&&(l||a._hasUserState)||x.default.equalSortParameters(s,l)||(r(a._columns,s,"groupIndex"),n&&(de(a,"grouping"),o=!0)),!n&&(u||a._hasUserState)||x.default.equalSortParameters(i,u)||(r(a._columns,i,"sortIndex"),n&&(de(a,"sorting"),o=!0)),o&&he(a)}},updateFilter:function(e,t,n,i){var o=this;if(!Array.isArray(e))return e;if(e=(0,g.extend)([],e),n=void 0!==e.columnIndex?e.columnIndex:n,i=void 0!==e.filterValue?e.filterValue:i,(0,h.isString)(e[0])&&"!"!==e[0]){var a=o.columnOption(e[0]);t?(0,m.default)().forceIsoDateParsing&&a&&a.serializeValue&&e.length>1&&(e[e.length-1]=a.serializeValue(e[e.length-1],"filter")):a&&a.selector&&(e[0]=a.selector,e[0].columnIndex=a.index)}else(0,h.isFunction)(e[0])&&(e[0].columnIndex=n,e[0].filterValue=i);for(var r=0;r<e.length;r++)e[r]=o.updateFilter(e[r],t,n,i);return e},columnCount:function(){return this._columns?this._columns.length:0},columnOption:function(e,t,n,i){var o=this,a=o._columns.concat(o._commandColumns),r=Oe(a,e);if(r){if(1===arguments.length)return(0,g.extend)({},r);if((0,h.isString)(t)){if(2===arguments.length)return ge(o,r,t);ge(o,r,t,n,i)}else(0,h.isObject)(t)&&(0,f.each)(t,function(e,t){ge(o,r,e,t,i)});he(o)}},clearSorting:function(){var e=this,t=this.columnCount();e.beginUpdate();for(var n=0;n<t;n++)e.columnOption(n,"sortOrder",void 0);e.endUpdate()},clearGrouping:function(){var e=this,t=this.columnCount();e.beginUpdate();for(var n=0;n<t;n++)e.columnOption(n,"groupIndex",void 0);e.endUpdate()},getVisibleIndex:function(e,t){for(var n=this.getVisibleColumns(t),i=n.length-1;i>=0;i--)if(n[i].index===e)return i;return-1},getVisibleColumnIndex:function(e,t){var n=this.columnOption(e,"index");return this.getVisibleIndex(n,t)},addColumn:function(e){var t=this,n=N(t,e),i=t._columns.length;t._columns.push(n),n.isBand&&(t._columns=j(t,t._columns),n=t._columns[i]),n.added=e,ue(t,n),t.updateColumns(t._dataSource),t._checkColumns()},deleteColumn:function(e){var t=this,n=t.columnOption(e);if(n&&n.index>=0){if(Ie(t._columns),t._columns.splice(n.index,1),n.isBand){var i=t.getChildrenByBandColumn(n.index).map(function(e){return e.index});t._columns=t._columns.filter(function(e){return i.indexOf(e.index)<0})}ue(t),t.updateColumns(t._dataSource)}},addCommandColumn:function(e){var t=this._commandColumns.filter(function(t){return t.command===e.command})[0];t||(t=e,this._commandColumns.push(t))},getUserState:function(){function e(e,o){void 0!==n[t][o]&&(i[t][o]=n[t][o])}var t,n=this._columns,i=[];for(t=0;t<n.length;t++)i[t]={},(0,f.each)(R,e);return i},setName:function(e){var t=e.dataField;!(0,h.isDefined)(e.name)&&(0,h.isDefined)(t)&&(e.name=t)},setUserState:function(e){var t=this,n=t._dataSource,i=t.option("stateStoring.ignoreColumnOptionNames");if(null===e||void 0===e?void 0:e.forEach(this.setName),!i){i=[];var o=t.getCommonSettings();t.option("columnChooser.enabled")||i.push("visible"),"none"===t.option("sorting.mode")&&i.push("sortIndex","sortOrder"),o.allowGrouping||i.push("groupIndex"),o.allowFixing||i.push("fixed","fixedPosition"),o.allowResizing||i.push("width","visibleWidth");var a=!t.option("filterPanel.visible");!t.option("filterRow.visible")&&a&&i.push("filterValue","selectedFilterOperation"),!t.option("headerFilter.visible")&&a&&i.push("filterValues","filterType")}t._columnsUserState=e,t._ignoreColumnOptionNames=i,t._hasUserState=!!e,de(t,"filtering"),t.init(),n&&(n.sort(t.getSortDataSourceParameters()),n.group(t.getGroupDataSourceParameters()))},_checkColumns:function(){var e={},t=!1,n=[];this._columns.forEach(function(i){var o,a=i.name,r=null===(o=i.columns)||void 0===o?void 0:o.length,s=i.allowEditing&&(i.dataField||i.setCellValue)&&!r;a?(e[a]&&n.push('"'.concat(a,'"')),e[a]=!0):s&&(t=!0)}),n.length&&y.default.log("E1059",n.join(", ")),t&&y.default.log("E1060")},_createCalculatedColumnOptions:function(e,t){var n={},i=e.dataField;if((Array.isArray(e.columns)&&e.columns.length||e.isBand)&&(n.isBand=!0,i=null),i){if((0,h.isString)(i)){var o=(0,c.compileGetter)(i);n={caption:(0,w.captionize)(i),calculateCellValue:function(e,t){var n=o(e);return this.deserializeValue&&!t?this.deserializeValue(n):n},setCellValue:me,parseValue:function(e){var t,n,i=this;return"number"===i.dataType?(0,h.isString)(e)&&i.format?(n=k.default.parse(e),(0,h.isNumeric)(n)&&(t=n)):(0,h.isDefined)(e)&&(0,h.isNumeric)(e)&&(t=Number(e)):"boolean"===i.dataType?e===i.trueText?t=!0:e===i.falseText&&(t=!1):x.default.isDateType(i.dataType)?(n=S.default.parse(e,i.format),n&&(t=n)):t=e,t}}}n.allowFiltering=!0}else n.allowFiltering=!!e.calculateFilterExpression;return n.calculateFilterExpression=function(){return A.default.defaultCalculateFilterExpression.apply(this,arguments)},n.createFilterExpression=function(e){var t;return this.calculateFilterExpression&&(t=this.calculateFilterExpression.apply(this,arguments)),(0,h.isFunction)(t)&&(t=[t,"=",!0]),t&&(t.columnIndex=this.index,t.filterValue=e),t},i&&(0,h.isString)(i)||(0,g.extend)(!0,n,{allowSorting:!1,allowGrouping:!1,calculateCellValue:function(){return null}}),t&&(n.allowFixing=!1),e.dataType&&(n.userDataType=e.dataType),!e.selectedFilterOperation||"defaultSelectedFilterOperation"in n||(n.defaultSelectedFilterOperation=e.selectedFilterOperation),e.lookup&&(n.lookup={calculateCellValue:function(e,t){return this.valueExpr&&(e=this.valueMap&&this.valueMap[e]),this.deserializeValue&&!t?this.deserializeValue(e):e},updateValueMap:function(){if(this.valueMap={},this.items)for(var e=(0,c.compileGetter)(this.valueExpr),t=(0,c.compileGetter)(this.displayExpr),n=0;n<this.items.length;n++){var i=this.items[n],o=t(i);this.valueMap[e(i)]=o,this.dataType=this.dataType||Y(o)}},update:function(){var e=this,t=e.dataSource;if(t)if((0,h.isFunction)(t)&&!u.default.isWrapped(t)&&(t=t({})),(0,h.isPlainObject)(t)||t instanceof T.default||Array.isArray(t)){if(e.valueExpr){var n=(0,E.normalizeDataSourceOptions)(t);return n.paginate=!1,t=new O.DataSource(n),t.load().done(function(t){e.items=t,e.updateValueMap&&e.updateValueMap()})}}else y.default.log("E1016");else e.updateValueMap&&e.updateValueMap()}}),n.resizedCallbacks=(0,l.default)(),e.resized&&n.resizedCallbacks.add(e.resized.bind(e)),(0,f.each)(n,function(e){if((0,h.isFunction)(n[e])&&0!==e.indexOf("default")){var t="default"+e.charAt(0).toUpperCase()+e.substr(1);n[t]=n[e]}}),n},getRowCount:function(){return this._rowCount=this._rowCount||ye(this),this._rowCount},getRowIndex:function(e,t){var n=this._columns[e],i=this.getBandColumnsCache();return n&&(t||n.visible&&!n.command&&!(0,h.isDefined)(n.groupIndex))?W(e,i.columnParentByIndex).length:0},getChildrenByBandColumn:function(e,t){var n=this,i=n.getBandColumnsCache(),o=G(e,i.columnChildrenByIndex,!t);return t?o.filter(function(e){return e.visible&&!e.command}).sort(function(e,t){return e.visibleIndex-t.visibleIndex}):o},isParentBandColumn:function(e,t){var n=!1,i=this._columns[e],o=this.getBandColumnsCache(),a=i&&W(e,o.columnParentByIndex);return a&&(0,f.each)(a,function(e,i){if(i.index===t)return n=!0,!1}),n},isParentColumnVisible:function(e){var t=!0,n=this.getBandColumnsCache(),i=e>=0&&W(e,n.columnParentByIndex);return i&&(0,f.each)(i,function(e,n){return t=t&&n.visible}),t},getColumnId:function(e){return e.command&&e.type===F?be(this,e)?"type:"+e.type:"command:"+e.command:e.index},getCustomizeTextByDataType:ee,getHeaderContentAlignment:function(e){var t=this.option("rtlEnabled");return t?"left"===e?"right":"left":e}}}())}};t.default=z,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(27)),a=i(n(26)),r=n(4),s=n(1),l=n(3),u=n(0),c=i(n(81)),d=n(90),h=n(6),f=a.default.Controller.inherit(function(){function e(t,n){if(t&&(t=t.slice(0),n))for(var i=0;i<t.length;i++)t[i]=(0,u.extend)({key:t[i].key},t[i]),t[i].items=e(t[i].items,n-1);return t}function t(e,t,n){var i={reload:!0,fullReload:!0};return t&&(i={sorting:!a.default.equalSortParameters(e.sort,t.sort),grouping:!a.default.equalSortParameters(e.group,t.group,!0),groupExpanding:!a.default.equalSortParameters(e.group,t.group)||t.groupExpand,filtering:!a.default.equalFilterParameters(e.filter,t.filter),pageIndex:e.pageIndex!==t.pageIndex,skip:e.skip!==t.skip,take:e.take!==t.take,fullReload:n},i.reload=n||i.sorting||i.grouping||i.filtering,i.paging=i.pageIndex||i.take),i}function n(e,t){(0,s.isDefined)(t)?(0,r.executeAsync)(e,t):e()}function i(){return{pages:{}}}function f(e){return e.cachedPagesData.pages[e.pageIndex]}function p(e,t){var n=e.pageIndex;void 0!==n&&(e.cachedPagesData.pages[n]=t)}return{init:function(e,t){var n=this;n._dataSource=e,n._remoteOperations=t||{},n._isLastPage=!e.isLastPage(),n._hasLastPage=!1,n._currentTotalCount=0,n._cachedPagesData=i(),n._lastOperationTypes={},n._eventsStrategy=e._eventsStrategy,n._skipCorrection=0,n._isLoadingAll=!1,n.changed=(0,o.default)(),n.loadingChanged=(0,o.default)(),n.loadError=(0,o.default)(),n.customizeStoreLoadOptions=(0,o.default)(),n.changing=(0,o.default)(),n._dataChangedHandler=n._handleDataChanged.bind(n),n._dataLoadingHandler=n._handleDataLoading.bind(n),n._dataLoadedHandler=n._handleDataLoaded.bind(n),n._loadingChangedHandler=n._handleLoadingChanged.bind(n),n._loadErrorHandler=n._handleLoadError.bind(n),n._pushHandler=n._handlePush.bind(n),n._changingHandler=n._handleChanging.bind(n),e.on("changed",n._dataChangedHandler),e.on("customizeStoreLoadOptions",n._dataLoadingHandler),e.on("customizeLoadResult",n._dataLoadedHandler),e.on("loadingChanged",n._loadingChangedHandler),e.on("loadError",n._loadErrorHandler),e.on("changing",n._changingHandler),e.store().on("push",n._pushHandler),(0,l.each)(e,function(e,t){!n[e]&&(0,s.isFunction)(t)&&(n[e]=function(){return this._dataSource[e].apply(this._dataSource,arguments)})})},remoteOperations:function(){return this._remoteOperations},dispose:function(e){var t=this,n=t._dataSource,i=n.store();n.off("changed",t._dataChangedHandler),n.off("customizeStoreLoadOptions",t._dataLoadingHandler),n.off("customizeLoadResult",t._dataLoadedHandler),n.off("loadingChanged",t._loadingChangedHandler),n.off("loadError",t._loadErrorHandler),n.off("changing",t._changingHandler),i&&i.off("push",t._pushHandler),e||n.dispose()},refresh:function(e,t){var n=this,i=n._dataSource;t.reload&&(n.resetCurrentTotalCount(),n._isLastPage=!i.paginate(),n._hasLastPage=n._isLastPage)},resetCurrentTotalCount:function(){this._currentTotalCount=0,this._skipCorrection=0},resetCache:function(){this._cachedStoreData=void 0,this._cachedPagingData=void 0},resetPagesCache:function(){this._cachedPagesData=i()},_needClearStoreDataCache:function(){var e=this.remoteOperations(),n=t(this._lastLoadOptions||{},{}),i=Object.keys(e).every(function(t){return!n[t]||!e[t]});return!i},push:function(e,t){var n=this.store();this._needClearStoreDataCache()&&(this._cachedStoreData=void 0),this._cachedPagingData=void 0,this.resetPagesCache(!0),this._cachedStoreData&&(0,d.applyBatch)({keyInfo:n,data:this._cachedStoreData,changes:e}),t||this._applyBatch(e)},getDataIndexGetter:function(){var e=this;if(!this._dataIndexGetter){var t,n,i=this.store();this._dataIndexGetter=function(o){var a=n&&n!==e._cachedStoreData;if(!t||a){n=e._cachedStoreData||[],t={};for(var s=0;s<n.length;s++)t[(0,r.getKeyHash)(i.keyOf(n[s]))]=s}return t[(0,r.getKeyHash)(i.keyOf(o))]}}return this._dataIndexGetter},_getKeyInfo:function(){return this.store()},_applyBatch:function(e){var t=this,n=this._getKeyInfo(),i=this._dataSource,o=a.default.normalizeSortingInfo(this.group()).length,r=this.totalCount(),s="virtual"===this.option("scrolling.mode");e=e.filter(function(e){return!i.paginate()||"insert"!==e.type||void 0!==e.index});var l=function(){return o?t.itemsCount():t._items.length},u=l();(0,d.applyBatch)({keyInfo:n,data:this._items,changes:e,groupCount:o,useInsertIndex:!0}),(0,d.applyBatch)({keyInfo:n,data:i.items(),changes:e,groupCount:o,useInsertIndex:!0}),(this._currentTotalCount>0||s&&r===u)&&(this._skipCorrection+=l()-u),e.splice(0,e.length)},_handlePush:function(e){this.push(e,!0)},_handleChanging:function(e){this.changing.fire(e),this._applyBatch(e.changes)},_needCleanCacheByOperation:function(e,t){var n=["filtering","sorting","paging"],i=n.indexOf(e),o=i>=0?n.slice(i):[e];return o.some(function(e){return t[e]})},_customizeRemoteOperations:function(e,t){var n=this,o=n._cachedStoreData,a=n._cachedPagingData,r=n._cachedPagesData;(e.storeLoadOptions.filter&&!e.remoteOperations.filtering||e.storeLoadOptions.sort&&!e.remoteOperations.sorting)&&(e.remoteOperations={filtering:e.remoteOperations.filtering}),t.fullReload?(o=void 0,a=void 0,r=i()):(t.reload?(a=void 0,r=i()):(t.take||t.groupExpanding)&&(r=i()),(0,l.each)(t,function(t,i){i&&n._needCleanCacheByOperation(t,e.remoteOperations)&&(o=void 0,a=void 0)})),a&&(e.remoteOperations.paging=!1),e.cachedStoreData=o,e.cachedPagingData=a,e.cachedPagesData=r,e.isCustomLoading||(n._cachedStoreData=o,n._cachedPagingData=a,n._cachedPagesData=r)},_handleDataLoading:function(e){var n=this,i=n._dataSource,o=n._lastLoadOptions;n.customizeStoreLoadOptions.fire(e),e.delay=this.option("loadingTimeout"),e.originalStoreLoadOptions=e.storeLoadOptions,e.remoteOperations=(0,u.extend)({},this.remoteOperations());var a=!n.isLoaded()&&!n._isRefreshing;n.option("integrationOptions.renderedOnServer")&&!n.isLoaded()&&(e.delay=void 0);var r=(0,u.extend)({pageIndex:n.pageIndex()},e.storeLoadOptions),s=t(r,o,a);if(n._customizeRemoteOperations(e,s),!e.isCustomLoading){var l=n._isRefreshing;e.pageIndex=i.pageIndex(),e.lastLoadOptions=r,e.operationTypes=s,n._loadingOperationTypes=s,n._isRefreshing=!0,(0,h.when)(l||n._isRefreshed||n.refresh(e,s)).done(function(){n._lastOperationId===e.operationId&&(n._isRefreshed=!0,n.load().always(function(){n._isRefreshed=!1}))}).fail(function(){i.cancel(e.operationId)}).always(function(){n._isRefreshing=!1}),i.cancel(n._lastOperationId),n._lastOperationId=e.operationId,n._isRefreshing&&i.cancel(n._lastOperationId)}this._handleDataLoadingCore(e)},_handleDataLoadingCore:function(e){var t=e.remoteOperations;e.loadOptions={};var n=e.cachedPagesData.extra,i={filter:!t.filtering,sort:!t.sorting,group:!t.grouping,summary:!t.summary,skip:!t.paging,take:!t.paging,requireTotalCount:n&&"totalCount"in n||!t.paging};(0,l.each)(e.storeLoadOptions,function(t,n){i[t]&&(e.loadOptions[t]=n,delete e.storeLoadOptions[t])}),n&&(e.extra=n),e.data=f(e)||e.cachedStoreData},_handleDataLoaded:function(t){var n=this,i=t.loadOptions,o=t.remoteOperations&&!t.remoteOperations.paging,r=t.cachedPagesData,l=t.storeLoadOptions,d=this.option("cacheEnabled")!==!1&&l,g=d&&!t.isCustomLoading&&r&&(!o||l.group)&&!this.option("legacyRendering"),_=d&&o,m=_&&!t.isCustomLoading;if(!i)return void this._dataSource.cancel(t.operationId);t.lastLoadOptions&&(this._lastLoadOptions=t.lastLoadOptions,Object.keys(t.operationTypes).forEach(function(e){n._lastOperationTypes[e]=n._lastOperationTypes[e]||t.operationTypes[e]})),o&&(t.skip=i.skip,t.take=i.take,delete i.skip,delete i.take),i.group&&(i.group=t.group||i.group);var v=a.default.normalizeSortingInfo(l.group||i.group).length;g&&f(t)||(_&&t.cachedPagingData?t.data=e(t.cachedPagingData,v):(m&&(this._cachedStoreData?t.mergeStoreLoadData&&(t.data=this._cachedStoreData=this._cachedStoreData.concat(t.data)):this._cachedStoreData=e(t.data,a.default.normalizeSortingInfo(l.group).length)),new c.default(t.data).load(i).done(function(i){t.data=i,m&&(n._cachedPagingData=e(t.data,v))}).fail(function(e){t.data=(new h.Deferred).reject(e)})),i.requireTotalCount&&o&&(t.extra=(0,s.isPlainObject)(t.extra)?t.extra:{},t.extra.totalCount=t.data.length),t.extra&&t.extra.totalCount>=0&&(l.requireTotalCount===!1||i.requireTotalCount===!1)&&(t.extra.totalCount=-1),this._handleDataLoadedCore(t),g&&(r.extra=r.extra||(0,u.extend)({},t.extra),(0,h.when)(t.data).done(function(n){p(t,e(n,v))}))),t.storeLoadOptions=t.originalStoreLoadOptions},_handleDataLoadedCore:function(e){e.remoteOperations&&!e.remoteOperations.paging&&Array.isArray(e.data)&&(void 0!==e.skip&&(e.data=e.data.slice(e.skip)),void 0!==e.take&&(e.data=e.data.slice(0,e.take)))},_handleLoadingChanged:function(e){this.loadingChanged.fire(e)},_handleLoadError:function(e){this.loadError.fire(e),this.changed.fire({changeType:"loadError",error:e})},_handleDataChanged:function(e){var t,n=this,i=n._dataSource,o=!1,a=n.itemsCount();n._isLastPage=!a||!n.pageSize()||a<n.pageSize(),n._isLastPage&&(n._hasLastPage=!0),i.totalCount()>=0?i.pageIndex()>=n.pageCount()&&(i.pageIndex(n.pageCount()-1),n.pageIndex(i.pageIndex()),n.resetPagesCache(),i.load(),o=!0):e&&!(0,s.isDefined)(e.changeType)||(t=i.pageIndex()*n.pageSize()+a,n._currentTotalCount=Math.max(n._currentTotalCount,t),0===a&&i.pageIndex()>=n.pageCount()&&(i.pageIndex(n.pageCount()-1),"infinite"!==n.option("scrolling.mode")&&(i.load(),o=!0))),o||(n._operationTypes=n._lastOperationTypes,n._lastOperationTypes={},n.component._optionCache={},n.changed.fire(e),n.component._optionCache=void 0)},_scheduleCustomLoadCallbacks:function(e){var t=this;t._isCustomLoading=!0,e.always(function(){t._isCustomLoading=!1})},loadingOperationTypes:function(){return this._loadingOperationTypes},operationTypes:function(){return this._operationTypes},lastLoadOptions:function(){return this._lastLoadOptions||{}},isLastPage:function(){return this._isLastPage},totalCount:function(){return parseInt((this._currentTotalCount||this._dataSource.totalCount())+this._skipCorrection)},itemsCount:function(){return this._dataSource.items().length},totalItemsCount:function(){return this.totalCount()},pageSize:function(){var e=this._dataSource;return arguments.length||e.paginate()?e.pageSize.apply(e,arguments):0},pageCount:function(){var e=this,t=e.totalItemsCount()-e._skipCorrection,n=e.pageSize();return n&&t>0?Math.max(1,Math.ceil(t/n)):1},hasKnownLastPage:function(){return this._hasLastPage||this._dataSource.totalCount()>=0},loadFromStore:function(e,t){var n=this._dataSource,i=new h.Deferred;if(n)return t=t||n.store(),t.load(e).done(function(e,t){e&&!Array.isArray(e)&&Array.isArray(e.data)&&(t=e,e=e.data),i.resolve(e,t)}).fail(i.reject),i},isCustomLoading:function(){return!!this._isCustomLoading},load:function(e){var t=this,i=this,o=i._dataSource,a=new h.Deferred;if(e){var r=o.store(),s=o.loadOptions(),u={storeLoadOptions:e,isCustomLoading:!0};return(0,l.each)(r._customLoadOptions()||[],function(e,t){t in u.storeLoadOptions||(u.storeLoadOptions[t]=s[t])}),this._isLoadingAll=e.isLoadingAll,i._scheduleCustomLoadCallbacks(a),o._scheduleLoadCallbacks(a),i._handleDataLoading(u),n(function(){return o.store()?void(0,h.when)(u.data||i.loadFromStore(u.storeLoadOptions)).done(function(t,n){u.data=t,u.extra=n||{},i._handleDataLoaded(u),e.requireTotalCount&&void 0===u.extra.totalCount&&(u.extra.totalCount=r.totalCount(u.storeLoadOptions)),(0,h.when)(u.data,u.extra.totalCount).done(function(e,t){u.extra.totalCount=t,a.resolve(e,u.extra)}).fail(a.reject)}).fail(a.reject):a.reject("canceled")},i.option("loadingTimeout")),a.fail(function(){i._eventsStrategy.fireEvent("loadError",arguments)}).always(function(){t._isLoadingAll=!1}).promise()}return o.load()},reload:function(e){return e?this._dataSource.reload():this._dataSource.load()},getCachedStoreData:function(){return this._cachedStoreData}}}());t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(44)),r=i(n(37)),s=i(n(81)),l=i(n(123)),u=i(n(17)),c=n(4),d=n(3),h=n(1),f=n(0),p=i(n(94)),g=n(6),_=n(432),m={defaultOptions:function(){return{loadingTimeout:0,dataSource:null,cacheEnabled:!0,repaintChangesOnly:!1,highlightChanges:!1,onDataErrorOccurred:null,remoteOperations:"auto",paging:{enabled:!0,pageSize:void 0,pageIndex:void 0}}},controllers:{data:a.default.Controller.inherit({}).include(p.default).inherit(function(){var e=function(e,t,n){var i=e._dataSource;return i?void 0!==n?i[t]()!==n?("pageSize"===t&&i.pageIndex(0),i[t](n),e._skipProcessingPagingChange=!0,e.option("paging."+t,n),e._skipProcessingPagingChange=!1,i["pageIndex"===t?"load":"reload"]().done(e.pageChanged.fire.bind(e.pageChanged))):(0,g.Deferred)().resolve().promise():i[t]():0},t={init:function(){var e=this,t=this;t._items=[],t._columnsController=t.getController("columns"),t._currentOperationTypes=null,t._dataChangedHandler=function(n){t._currentOperationTypes=e._dataSource.operationTypes(),t._handleDataChanged(n),t._currentOperationTypes=null},t._columnsChangedHandler=t._handleColumnsChanged.bind(t),t._loadingChangedHandler=t._handleLoadingChanged.bind(t),t._loadErrorHandler=t._handleLoadError.bind(t),t._customizeStoreLoadOptionsHandler=t._handleCustomizeStoreLoadOptions.bind(t),t._changingHandler=t._handleChanging.bind(t),t._columnsController.columnsChanged.add(t._columnsChangedHandler),t._isLoading=!1,t._isCustomLoading=!1,t._repaintChangesOnly=void 0,t._changes=[],t.createAction("onDataErrorOccurred"),t.dataErrorOccurred.add(function(e){return t.executeAction("onDataErrorOccurred",{error:e})}),t._refreshDataSource()},callbackNames:function(){return["changed","loadingChanged","dataErrorOccurred","pageChanged","dataSourceChanged"]},callbackFlags:function(e){if("dataErrorOccurred"===e)return{stopOnFalse:!0}},publicMethods:function(){return["beginCustomLoading","endCustomLoading","refresh","filter","clearFilter","getCombinedFilter","keyOf","byKey","getDataByKeys","pageIndex","pageSize","pageCount","totalCount","_disposeDataSource","getKeyByRowIndex","getRowIndexByKey","getDataSource","getVisibleRows","repaintRows"]},reset:function(){this._columnsController.reset(),this._items=[],this._refreshDataSource()},optionChanged:function(e){function t(){e.handled=!0}var n,i=this;if("dataSource"===e.name&&e.name===e.fullName&&(e.value===e.previousValue||i.option("columns")&&Array.isArray(e.value)&&Array.isArray(e.previousValue))){if(e.value!==e.previousValue){var o=i.store();o&&(o._array=e.value)}return t(),void i.refresh(i.option("repaintChangesOnly"))}switch(e.name){case"cacheEnabled":case"repaintChangesOnly":case"highlightChanges":case"loadingTimeout":t();break;case"remoteOperations":case"keyExpr":case"dataSource":case"scrolling":t(),i.reset();break;case"paging":n=i.dataSource(),n&&i._setPagingOptions(n)&&n.load().done(i.pageChanged.fire.bind(i.pageChanged)),t();break;case"rtlEnabled":i.reset();break;case"columns":n=i.dataSource(),n&&n.isLoading()&&e.name===e.fullName&&(this._useSortingGroupingFromColumns=!0,n.load());break;default:i.callBase(e)}},isReady:function(){return!this._isLoading},getDataSource:function(){return this._dataSource&&this._dataSource._dataSource},getCombinedFilter:function(e){return this.combinedFilter(void 0,e)},combinedFilter:function(e,t){var n=this,i=n._dataSource,o=n._columnsController;if(i){void 0===e&&(e=i.filter());var a=n._calculateAdditionalFilter();a&&(o.isDataSourceApplied()||o.isAllDataTypesDefined())&&(e=r.default.combineFilters([a,e])),e=o.updateFilter(e,t||i.remoteOperations().filtering)}return e},waitReady:function(){return this._updateLockCount?(this._readyDeferred=new g.Deferred,this._readyDeferred):(0,g.when)()},_endUpdateCore:function(){var e=this._changes;if(e.length){this._changes=[];var t=e.every(function(e){return e.repaintChangesOnly});this.updateItems(1===e.length?e[0]:{repaintChangesOnly:t})}this._readyDeferred&&(this._readyDeferred.resolve(),this._readyDeferred=null)},_handleCustomizeStoreLoadOptions:function(e){var t=this._columnsController,n=this._dataSource,i=e.storeLoadOptions;e.isCustomLoading&&!i.isLoadingAll||(i.filter=this.combinedFilter(i.filter),t.isDataSourceApplied()||t.updateColumnDataTypes(n),this._columnsUpdating=!0,t.updateSortingGrouping(n,!this._useSortingGroupingFromColumns),this._columnsUpdating=!1,i.sort=t.getSortDataSourceParameters(),i.group=t.getGroupDataSourceParameters(),n.sort(i.sort),n.group(i.group),i.sort=t.getSortDataSourceParameters(!n.remoteOperations().sorting),e.group=t.getGroupDataSourceParameters(!n.remoteOperations().grouping))},_handleColumnsChanged:function(e){var t,n,i,o=this,a=e.changeTypes,s=e.optionNames,l=function e(){o._columnsController.columnsChanged.remove(e),o.updateItems()};if(a.sorting||a.grouping)o._dataSource&&!o._columnsUpdating&&(o._dataSource.group(o._columnsController.getGroupDataSourceParameters()),o._dataSource.sort(o._columnsController.getSortDataSourceParameters()),o.reload());else if(a.columns&&((s.filterValues||s.filterValue||s.selectedFilterOperation)&&(t=o._columnsController.columnOption(e.columnIndex,"filterValue"),n=o._columnsController.columnOption(e.columnIndex,"filterValues"),(Array.isArray(n)||void 0===e.columnIndex||(0,h.isDefined)(t)||!s.selectedFilterOperation||s.filterValue)&&(o._applyFilter(),i=!0)),o._needApplyFilter||r.default.checkChanges(s,["width","visibleWidth","filterValue","bufferedFilterValue","selectedFilterOperation","filterValues","filterType"])||o._columnsController.columnsChanged.add(l),(0,h.isDefined)(s.visible))){var u=o._columnsController.columnOption(e.columnIndex);u&&((0,h.isDefined)(u.filterValue)||(0,h.isDefined)(u.filterValues))&&(o._applyFilter(),i=!0)}!i&&a.filtering&&o.reload()},_handleDataChanged:function(e){var t=this,n=t._dataSource,i=t._columnsController,o=!1;this._useSortingGroupingFromColumns=!1,n&&!t._isDataSourceApplying&&(t._isDataSourceApplying=!0,(0,g.when)(t._columnsController.applyDataSource(n)).done(function(){t._isLoading&&t._handleLoadingChanged(!1),o&&e&&e.isDelayed&&(e.isDelayed=!1),t._isDataSourceApplying=!1;var n=function(){var e=t._calculateAdditionalFilter();return e&&e.length},i=t._needApplyFilter;t._needApplyFilter=!1,i&&!t._isAllDataTypesDefined&&n()?(u.default.log("W1005",t.component.NAME),t._applyFilter()):t.updateItems(e,!0)}).fail(function(){t._isDataSourceApplying=!1}),t._isDataSourceApplying&&(o=!0,t._handleLoadingChanged(!0)),t._needApplyFilter=!t._columnsController.isDataSourceApplied(),t._isAllDataTypesDefined=i.isAllDataTypesDefined())},_handleLoadingChanged:function(e){this._isLoading=e,this._fireLoadingChanged()},_handleLoadError:function(e){this.dataErrorOccurred.fire(e)},fireError:function(){this.dataErrorOccurred.fire(u.default.Error.apply(u.default,arguments))},_setPagingOptions:function(e){var t=this.option("paging.pageIndex"),n=this.option("paging.pageSize"),i=this.option("paging.enabled"),o=this.option("scrolling.mode"),a="infinite"===o,r="virtual"===o,s=i||r||a,l=!1;return e.requireTotalCount(!a),void 0!==i&&e.paginate()!==s&&(e.paginate(s),l=!0),void 0!==n&&e.pageSize()!==n&&(e.pageSize(n),l=!0),void 0!==t&&e.pageIndex()!==t&&(e.pageIndex(t),l=!0),l},_getSpecificDataSourceOption:function(){var e=this.option("dataSource");return Array.isArray(e)?{store:{type:"array",data:e,key:this.option("keyExpr")}}:e},_initDataSource:function(){var e=this,t=this.option("dataSource"),n=this._dataSource;e.callBase(),t=e._dataSource,e._useSortingGroupingFromColumns=!0,t?(e._setPagingOptions(t),e.setDataSource(t)):n&&e.updateItems()},_loadDataSource:function(){var e=this,t=e._dataSource,n=new g.Deferred;return(0,g.when)(this._columnsController.refresh(!0)).always(function(){t?t.load().done(n.resolve).fail(n.reject):n.resolve();
}),n.promise()},_beforeProcessItems:function(e){return e.slice(0)},getRowIndexDelta:function(){return 0},_processItems:function(e,t){var n=this,i=n.getRowIndexDelta(),o=t.changeType,a=n._columnsController.getVisibleColumns(null,"loadingAll"===o),r=n._items,s="append"===o&&r.length>0?r[r.length-1]:null,l=(0,h.isDefined)(null===s||void 0===s?void 0:s.dataIndex)?s.dataIndex+1:0,u={visibleColumns:a,dataIndex:l},c=[];return(0,d.each)(e,function(e,t){(0,h.isDefined)(t)&&(u.rowIndex=e-i,t=n._processItem(t,u),c.push(t))}),c},_processItem:function(e,t){return e=this._generateDataItem(e,t),e=this._processDataItem(e,t),e.dataIndex=t.dataIndex++,e},_generateDataItem:function(e){return{rowType:"data",data:e,key:this.keyOf(e)}},_processDataItem:function(e,t){return e.values=this.generateDataValues(e.data,t.visibleColumns),e},generateDataValues:function(e,t,n){for(var i,o=[],a=0;a<t.length;a++){var r=t[a];i=n?void 0:null,r.command||(r.calculateCellValue?i=r.calculateCellValue(e):r.dataField&&(i=e[r.dataField])),o.push(i)}return o},_applyChange:function(e){var t=this;"update"===e.changeType?t._applyChangeUpdate(e):t.items().length&&e.repaintChangesOnly&&"refresh"===e.changeType?t._applyChangesOnly(e):"refresh"===e.changeType&&t._applyChangeFull(e)},_applyChangeFull:function(e){this._items=e.items.slice(0)},_getRowIndices:function(e){var t=e.rowIndices.slice(0),n=this.getRowIndexDelta();t.sort(function(e,t){return e-t});for(var i=0;i<t.length;i++){var o=t[i];e.allowInvisibleRowIndices&&(o+=n),o<0&&(t.splice(i,1),i--)}return t},_applyChangeUpdate:function(e){var t,n=this,i=e.items,o=n._getRowIndices(e),a=n.getRowIndexDelta(),r=n.option("repaintChangesOnly"),s=-1,l=0;e.items=[],e.rowIndices=[],e.columnIndices=[],e.changeTypes=[];var u=function(e,t,n){var i=e&&t&&(0,c.equalByValue)(e.key,t.key);return i&&n&&(i=e.rowType===t.rowType&&("detail"!==t.rowType||e.isEditing===t.isEditing)),i};(0,d.each)(o,function(o,c){var d;if(c+=l+a,s!==c){s=c;var h=n._items[c],f=n._items[c+1],p=i[c],g=i[c+1],_=u(h,f)||u(p,g);if(p&&(p.rowIndex=c,e.items.push(p)),h&&p&&u(h,p,_))t="update",n._items[c]=p,h.visible!==p.visible?e.items.splice(-1,1,{visible:p.visible}):r&&!e.isFullUpdate&&(d=n._partialUpdateRow(h,p,c-a));else if(p&&!h||g&&u(h,g,_))t="insert",n._items.splice(c,0,p),l++;else if(h&&!p||f&&u(p,f,_))t="remove",n._items.splice(c,1),l--,s=-1;else{if(!p)return;t="update",n._items[c]=p}e.rowIndices.push(c-a),e.changeTypes.push(t),e.columnIndices.push(d)}})},_isCellChanged:function(e,t,n,i,o){function a(e,t){return!!e.modifiedValues&&void 0!==e.modifiedValues[t]}return JSON.stringify(e.values[i])!==JSON.stringify(t.values[i])||a(e,i)!==a(t,i)},_getChangedColumnIndices:function(e,t,n,i){if(e.rowType===t.rowType&&"group"!==t.rowType&&"groupFooter"!==t.rowType){var o=[];if("detail"!==t.rowType)for(var a=0;a<e.values.length;a++)this._isCellChanged(e,t,n,a,i)&&o.push(a);return o}},_partialUpdateRow:function(e,t,n,i){var o=this._getChangedColumnIndices(e,t,n,i);return o&&(e.cells&&e.cells.forEach(function(e,n){var i=o.indexOf(n)>=0;!i&&e&&e.update&&e.update(t)}),t.update=e.update,t.watch=e.watch,t.cells=e.cells,i&&(t.oldValues=e.values),e.update&&e.update(t)),o},_isItemEquals:function(e,t){if(JSON.stringify(e.values)!==JSON.stringify(t.values))return!1;var n=["modified","isNewRow","removed","isEditing"];if(n.some(function(n){return e[n]!==t[n]}))return!1;if("group"===e.rowType||"groupFooter"===e.rowType){var i,o,a,r,s=e.isExpanded===t.isExpanded,l=JSON.stringify(e.summaryCells)===JSON.stringify(t.summaryCells),u=(null===(i=e.data)||void 0===i?void 0:i.isContinuation)===(null===(o=t.data)||void 0===o?void 0:o.isContinuation)&&(null===(a=e.data)||void 0===a?void 0:a.isContinuationOnNextPage)===(null===(r=t.data)||void 0===r?void 0:r.isContinuationOnNextPage);if(!s||!l||!u)return!1}return!0},_applyChangesOnly:function(e){function t(e){if(e)return e.rowType+","+JSON.stringify(e.key)}var n=this,i=[],o=[],a=[],r=[],s={},l=function(e,t){return!!n._isItemEquals(e,t)&&(e.cells&&(e.update&&e.update(t),e.cells.forEach(function(e){e&&e.update&&e.update(t)})),!0)},u=this._items.slice();e.items.forEach(function(e,n){var i=t(e);s[i]=n,e.rowIndex=n});var c=(0,_.findChanges)(u,e.items,t,l);return c?(c.forEach(function(e){switch(e.type){case"update":var t=e.index,s=e.data,l=e.oldItem,u=n._partialUpdateRow(l,s,t,!0);i.push(t),a.push("update"),r.push(s),n._items[t]=s,o.push(u);break;case"insert":i.push(e.index),a.push("insert"),r.push(e.data),o.push(void 0),n._items.splice(e.index,0,e.data);break;case"remove":i.push(e.index),a.push("remove"),n._items.splice(e.index,1),r.push(e.oldItem),o.push(void 0)}}),e.repaintChangesOnly=!0,e.changeType="update",e.rowIndices=i,e.columnIndices=o,e.changeTypes=a,e.items=r,u.length&&(e.isLiveUpdate=!0),void this._correctRowIndices(function(e){var n=u[e],i=t(n),o=s[i];return o>=0?o-e:0})):void this._applyChangeFull(e)},_correctRowIndices:c.noop,_updateItemsCore:function(e){var t,n=this,i=n._dataSource,o=e.changeType||"refresh";if(e.changeType=o,i){t=e.items||i.items(),t=n._beforeProcessItems(t),t=n._processItems(t,e),e.items=t;var a=n._items.length===t.length&&n._items;n._applyChange(e);var r=n.getRowIndexDelta();(0,d.each)(n._items,function(e,t){t.rowIndex=e-r,a&&(t.cells=a[e].cells||[])})}else n._items=[]},_handleChanging:function(e){var t=this,n=t.getVisibleRows(),i=t.dataSource();i&&e.changes.forEach(function(e){if("insert"===e.type&&e.index>=0){for(var t=0,i=0;i<e.index;i++){var o=n[i];!o||"data"!==o.rowType&&"group"!==o.rowType||t++}e.index=t}})},updateItems:function(e,t){e=e||{};var n=this;if(void 0!==n._repaintChangesOnly)e.repaintChangesOnly=n._repaintChangesOnly;else if(e.changes)e.repaintChangesOnly=n.option("repaintChangesOnly");else if(t){var i=n.dataSource().operationTypes();e.repaintChangesOnly=i&&!i.grouping&&!i.filtering&&n.option("repaintChangesOnly"),e.isDataChanged=!0,i&&(i.reload||i.paging||i.groupExpanding)&&(e.needUpdateDimensions=!0)}return n._updateLockCount?void n._changes.push(e):(n._updateItemsCore(e),void(e.cancel||n._fireChanged(e)))},loadingOperationTypes:function(){var e=this.dataSource();return e&&e.loadingOperationTypes()||{}},_fireChanged:function(e){var t=this;this._currentOperationTypes&&(e.operationTypes=this._currentOperationTypes,this._currentOperationTypes=null),(0,c.deferRender)(function(){t.changed.fire(e)})},isLoading:function(){return this._isLoading||this._isCustomLoading},_fireLoadingChanged:function(){this.loadingChanged.fire(this.isLoading(),this._loadingText)},_calculateAdditionalFilter:function(){return null},_applyFilter:function(){var e=this,t=e._dataSource;if(t)return t.pageIndex(0),e.reload().done(e.pageChanged.fire.bind(e.pageChanged))},filter:function e(t){var n=this._dataSource,e=n&&n.filter();return 0===arguments.length?e:(t=arguments.length>1?Array.prototype.slice.call(arguments,0):t,void(r.default.equalFilterParameters(e,t)||(n&&n.filter(t),this._applyFilter())))},clearFilter:function(e){var t=this,n=t._columnsController,i=function(e){for(var t=n.columnCount(),i=0;i<t;i++)n.columnOption(i,e,void 0)};if(t.component.beginUpdate(),arguments.length>0)switch(e){case"dataSource":t.filter(null);break;case"search":t.searchByText("");break;case"header":i("filterValues");break;case"row":i("filterValue")}else t.filter(null),t.searchByText(""),i("filterValue"),i("bufferedFilterValue"),i("filterValues");t.component.endUpdate()},_fireDataSourceChanged:function(){var e=this,t=function t(){e.changed.remove(t),e.dataSourceChanged.fire()};e.changed.add(t)},_getDataSourceAdapter:c.noop,_createDataSourceAdapterCore:function(e,t){var n=this._getDataSourceAdapter(),i=n.create(this.component);return i.init(e,t),i},isLocalStore:function(e){return e=e||this.store(),e instanceof s.default},isCustomStore:function(e){return e=e||this.store(),e instanceof l.default},_createDataSourceAdapter:function(e){var t=this.option("remoteOperations"),n=e.store(),i={filtering:!0,sorting:!0,paging:!0,grouping:!0,summary:!0};return t&&t.groupPaging&&(t=(0,f.extend)({},i,t)),"auto"===t&&(t=this.isLocalStore(n)||this.isCustomStore(n)?{}:{filtering:!0,sorting:!0,paging:!0}),t===!0&&(t=i),this._createDataSourceAdapterCore(e,t)},setDataSource:function(e){var t=this,n=t._dataSource;!e&&n&&(n.cancelAll(),n.changed.remove(t._dataChangedHandler),n.loadingChanged.remove(t._loadingChangedHandler),n.loadError.remove(t._loadErrorHandler),n.customizeStoreLoadOptions.remove(t._customizeStoreLoadOptionsHandler),n.changing.remove(t._changingHandler),n.dispose(t._isSharedDataSource)),e&&(e=t._createDataSourceAdapter(e)),t._dataSource=e,e&&(t._fireDataSourceChanged(),t._isLoading=!e.isLoaded(),t._needApplyFilter=!0,t._isAllDataTypesDefined=t._columnsController.isAllDataTypesDefined(),e.changed.add(t._dataChangedHandler),e.loadingChanged.add(t._loadingChangedHandler),e.loadError.add(t._loadErrorHandler),e.customizeStoreLoadOptions.add(t._customizeStoreLoadOptionsHandler),e.changing.add(t._changingHandler))},items:function(){return this._items},isEmpty:function(){return!this.items().length},pageCount:function(){return this._dataSource?this._dataSource.pageCount():1},dataSource:function(){return this._dataSource},store:function(){var e=this._dataSource;return e&&e.store()},loadAll:function(e){var t=this,n=new g.Deferred,i=t._dataSource;if(i)if(e){var o={data:e,isCustomLoading:!0,storeLoadOptions:{isLoadingAll:!0},loadOptions:{filter:t.getCombinedFilter(),group:i.group(),sort:i.sort()}};i._handleDataLoaded(o),(0,g.when)(o.data).done(function(e){e=t._beforeProcessItems(e),n.resolve(t._processItems(e,{changeType:"loadingAll"}),o.extra&&o.extra.summary)}).fail(n.reject)}else if(i.isLoading())n.reject();else{var a=(0,f.extend)({},i.loadOptions(),{isLoadingAll:!0,requireTotalCount:!1});i.load(a).done(function(e,i){e=t._beforeProcessItems(e),e=t._processItems(e,{changeType:"loadingAll"}),n.resolve(e,i&&i.summary)}).fail(n.reject)}else n.resolve([]);return n},getKeyByRowIndex:function(e){var t=this.items()[e];if(t)return t.key},getRowIndexByKey:function(e){return r.default.getIndexByKey(e,this.items())},keyOf:function(e){var t=this.store();if(t)return t.keyOf(e)},byKey:function(e){var t,n=this.store(),i=this.getRowIndexByKey(e);if(n)return i>=0&&(t=(new g.Deferred).resolve(this.items()[i].data)),t||n.byKey(e)},key:function(){var e=this.store();if(e)return e.key()},getRowIndexOffset:function(){return 0},getDataByKeys:function(e){var t=this,n=new g.Deferred,i=[],a=[];return(0,d.each)(e,function(e,n){i.push(t.byKey(n).done(function(t){a[e]=t}))}),g.when.apply(o.default,i).always(function(){n.resolve(a)}),n},pageIndex:function(t){return e(this,"pageIndex",t)},pageSize:function(t){return e(this,"pageSize",t)},beginCustomLoading:function(e){this._isCustomLoading=!0,this._loadingText=e||"",this._fireLoadingChanged()},endCustomLoading:function(){this._isCustomLoading=!1,this._loadingText=void 0,this._fireLoadingChanged()},refresh:function(e){e===!0?e={reload:!0,changesOnly:!0}:e||(e={lookup:!0,selection:!0,reload:!0});var t=this,n=t.getDataSource(),i=e.changesOnly,o=new g.Deferred,a=function(){t._repaintChangesOnly=!!i};return(0,g.when)(!e.lookup||t._columnsController.refresh()).always(function(){e.load||e.reload?(n&&n.on("customizeLoadResult",a),(0,g.when)(t.reload(e.reload,i)).always(function(){n&&n.off("customizeLoadResult",a),t._repaintChangesOnly=void 0}).done(o.resolve).fail(o.reject)):(t.updateItems({repaintChangesOnly:e.changesOnly}),o.resolve())}),o.promise()},getVisibleRows:function(){return this.items()},_disposeDataSource:function(){this.setDataSource(null)},dispose:function(){this._disposeDataSource(),this.callBase.apply(this,arguments)},repaintRows:function(e,t){e=Array.isArray(e)?e:[e],(e.length>1||(0,h.isDefined)(e[0]))&&this.updateItems({changeType:"update",rowIndices:e,isFullUpdate:!t})},skipProcessingPagingChange:function(e){return this._skipProcessingPagingChange&&("paging.pageIndex"===e||"paging.pageSize"===e)},getUserState:function(){return{searchText:this.option("searchPanel.text"),pageIndex:this.pageIndex(),pageSize:this.pageSize()}},getCachedStoreData:function(){return this._dataSource&&this._dataSource.getCachedStoreData()}};return r.default.proxyMethod(t,"load"),r.default.proxyMethod(t,"reload"),r.default.proxyMethod(t,"push"),r.default.proxyMethod(t,"itemsCount",0),r.default.proxyMethod(t,"totalItemsCount",0),r.default.proxyMethod(t,"hasKnownLastPage",!0),r.default.proxyMethod(t,"isLoaded",!0),r.default.proxyMethod(t,"totalCount",0),t}())}};t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(21),s=n(1),l=n(0),u=i(n(467)),c=i(n(10)),d=n(8),h="dxDataGridColumnHeadersView",f=(0,l.extend)({},u.default,{_createRow:function(e){var t=this,n=this.callBase(e);return"header"===e.rowType&&a.default.on(n,(0,d.addNamespace)(r.name,h),"td",this.createAction(function(e){t._processHeaderAction(e.event,n)})),n},_processHeaderAction:function(e,t){if((0,o.default)(e.currentTarget).parent().get(0)===t.get(0)){var n=this,i=null,a=(0,o.default)(e.currentTarget),r=a.parent().index(),l=-1;[].slice.call(n.getCellElements(r)).some(function(e,t){if(e===a.get(0))return l=t,!0});var u=n._columnsController.getVisibleColumns(r),c=u[l],d=n.getController("editing"),h=n.option("editing.mode"),f=d&&d.isEditing()&&("batch"===h||"cell"===h);!f&&n._isSortableElement((0,o.default)(e.target))&&(!c||(0,s.isDefined)(c.groupIndex)||c.command||(e.shiftKey?i="shift":e.ctrlKey&&(i="ctrl"),setTimeout(function(){n._columnsController.changeSortOrder(c.index,i)})))}},_renderCellContent:function(e,t){var n=this,i=t.column;i.command||"header"!==t.rowType||n._applyColumnState({name:"sort",rootElement:e,column:i,showColumnLines:n.option("showColumnLines")}),n.callBase(e,t)},_columnOptionChanged:function(e){var t=e.changeTypes;return 1===t.length&&t.sorting?void this._updateIndicators("sort"):void this.callBase(e)},optionChanged:function(e){var t=this;switch(e.name){case"sorting":t._invalidate(),e.handled=!0;break;default:t.callBase(e)}}}),p=(0,l.extend)({},u.default,{_createGroupPanelItem:function(e,t){var n=this,i=n.callBase.apply(n,arguments);return a.default.on(i,(0,d.addNamespace)(r.name,"dxDataGridHeaderPanel"),n.createAction(function(){n._processGroupItemAction(t.index)})),n._applyColumnState({name:"sort",rootElement:i,column:{alignment:n.option("rtlEnabled")?"right":"left",allowSorting:t.allowSorting,sortOrder:"desc"===t.sortOrder?"desc":"asc"},showColumnLines:!0}),i},_processGroupItemAction:function(e){var t=this;setTimeout(function(){return t.getController("columns").changeSortOrder(e)})},optionChanged:function(e){var t=this;switch(e.name){case"sorting":t._invalidate(),e.handled=!0;break;default:t.callBase(e)}}}),g={defaultOptions:function(){return{sorting:{mode:"single",ascendingText:c.default.format("dxDataGrid-sortingAscendingText"),descendingText:c.default.format("dxDataGrid-sortingDescendingText"),clearText:c.default.format("dxDataGrid-sortingClearText"),showSortIndexes:!0}}},extenders:{views:{columnHeadersView:f,headerPanel:p}}};t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(2)),r="dx-sort",s="dx-sort-none",l="dx-sort-up",u="dx-sort-down",c="dx-sort-index",d="dx-sort-index-icon",h="action",f={_applyColumnState:function(e){var t,n,i=this,s=i.option("sorting.mode"),f=e.rootElement,p=e.column,g=i._getIndicatorContainer(f);if("sort"===e.name){f.find("."+r).remove(),!g.children().length&&g.remove();var _=("single"===s||"multiple"===s)&&p.allowSorting;if(!(0,o.isDefined)(p.groupIndex)&&(_||(0,o.isDefined)(p.sortOrder))){t="asc"===p.sortOrder?"ascending":"descending",n=i.callBase(e).toggleClass(l,"asc"===p.sortOrder).toggleClass(u,"desc"===p.sortOrder);var m=i.getController&&!!i.getController("columns").columnOption("sortIndex:1");m&&i.option("sorting.showSortIndexes")&&p.sortIndex>=0&&((0,a.default)("<span>").addClass(d).text(p.sortIndex+1).appendTo(n),n.addClass(c)),e.rootElement.addClass(i.addWidgetPrefix(h))}return(0,o.isDefined)(p.sortOrder)?i.setAria("sort",t,f):i.setAria("sort","none",f),n}return i.callBase(e)},_getIndicatorClassName:function(e){return"sort"===e?r:"sortIndex"===e?d:this.callBase(e)},_renderIndicator:function(e){var t=e.column,n=e.container,i=e.indicator;if("sort"===e.name){var a=this.option("rtlEnabled");if((0,o.isDefined)(t.sortOrder)||i&&i.addClass(s),n.children().length&&(!a&&"left"===e.columnAlignment||a&&"right"===e.columnAlignment))return void n.prepend(i)}this.callBase(e)},_updateIndicator:function(e,t,n){if("sort"!==n||!(0,o.isDefined)(t.groupIndex))return this.callBase.apply(this,arguments)},_getIndicatorElements:function(e,t){var n=this.callBase(e);return t?n:n&&n.not("."+s)}};t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e?e.scrollWidth()-e.clientWidth():0}t.default=void 0;var a=i(n(2)),r=n(7),s=i(n(5)),l=n(4),u=n(97),c=n(1),d=n(3),h=n(0),f=n(24),p=n(42),g=n(19),_=i(n(37)),m=n(197),v=i(n(109)),y=i(n(138)),b=i(n(10)),x=i(n(23)),w="rowsview",C="content",k="nowrap",S="dx-group-row",D="dx-group-cell",I="dx-data-row",T="dx-freespace-row",O="dx-row-lines",E="dx-column-lines",A="dx-row-alt",P="dx-last-row-border",R="dx-empty",M="row-inserted-animation",B=200,V={defaultOptions:function(){return{hoverStateEnabled:!1,scrolling:{useNative:"auto"},loadPanel:{enabled:"auto",text:b.default.format("Loading"),width:200,height:90,showIndicator:!0,indicatorSrc:"",showPane:!0},rowTemplate:null,columnAutoWidth:!1,noDataText:b.default.format("dxDataGrid-noDataText"),wordWrapEnabled:!1,showColumnLines:!0,showRowLines:!1,rowAlternationEnabled:!1,activeStateEnabled:!1,twoWayBindingEnabled:!0}},views:{rowsView:m.ColumnsView.inherit(function(){var e=function(e,t){var n=(0,p.isEmpty)(t.text)&&"data"===t.rowType,i=t.text,o=e.get(0);n?_.default.setEmptyText(e):t.column.encodeHtml?o.textContent=i:o.innerHTML=i},t=function(e){var t=e.getScrollable();return t?Math.ceil(parseFloat(t.$content().css("paddingBottom"))):0};return{_getDefaultTemplate:function(t){switch(t.command){case"empty":return function(e){e.html("&nbsp;")};default:return e}},_getDefaultGroupTemplate:function(e){var t=this,n=t.option("summary.texts");return function(t,i){var o=i.data,a=i.column.caption+": "+i.text,r=t.get(0);i.summaryItems&&i.summaryItems.length&&(a+=" "+_.default.getGroupRowSummaryText(i.summaryItems,n)),o&&(i.groupContinuedMessage&&i.groupContinuesMessage?a+=" ("+i.groupContinuedMessage+". "+i.groupContinuesMessage+")":i.groupContinuesMessage?a+=" ("+i.groupContinuesMessage+")":i.groupContinuedMessage&&(a+=" ("+i.groupContinuedMessage+")")),t.addClass(D),e.encodeHtml?r.textContent=a:r.innerHTML=a}},_update:function(){},_getCellTemplate:function(e){var t,n=this,i=e.column;return t="group"!==e.rowType||!(0,c.isDefined)(i.groupIndex)||i.showWhenGrouped||i.command?("data"===e.rowType||i.command)&&i.cellTemplate?i.cellTemplate:{allowRenderToDetachedContainer:!0,render:n._getDefaultTemplate(i)}:i.groupCellTemplate||{allowRenderToDetachedContainer:!0,render:n._getDefaultGroupTemplate(i)}},_createRow:function(e){var t=this.callBase(e);if(e){var n="group"===e.rowType,i="data"===e.rowType;if(i&&t.addClass(I),i&&this.option("showRowLines")&&t.addClass(O),this.option("showColumnLines")&&t.addClass(E),e.visible===!1&&t.hide(),n){t.addClass(S);var o=e.isExpanded;this.setAria("role","row",t),this.setAria("expanded",(0,c.isDefined)(o)&&o.toString(),t)}}return t},_rowPrepared:function(e,t,n){var i=this;"data"===t.rowType&&(this.option("rowAlternationEnabled")&&(this._isAltRow(n)&&e.addClass(A),t.watch&&t.watch(function(){return i._isAltRow(n)},function(t){e.toggleClass(A,t)})),this._setAriaRowIndex(t,e),t.watch&&t.watch(function(){return t.rowIndex},function(){return i._setAriaRowIndex(t,e)})),this.callBase.apply(this,arguments)},_setAriaRowIndex:function(e,t){var n=this.component,i="standard"===n.option("scrolling.mode")&&"virtual"!==n.option("scrolling.rowRenderingMode"),o=e.rowIndex+1;i?o=n.pageIndex()*n.pageSize()+o:o+=this._dataController.getRowIndexOffset(),this.setAria("rowindex",o,t)},_afterRowPrepared:function(e){var t=this,n=e.args[0],i=this._dataController,o=i.getVisibleRows()[n.rowIndex],a=this.option("integrationOptions.watchMethod");if(n.data&&"data"===n.rowType&&!n.isNewRow&&this.option("twoWayBindingEnabled")&&a&&o){var r=a(function(){return i.generateDataValues(n.data,n.columns)},function(){i.repaintRows([o.rowIndex],t.option("repaintChangesOnly"))},{deep:!0,skipImmediate:!0});s.default.on(n.rowElement,y.default,r)}},_renderScrollable:function(e){var t=this,n=t.element();if(n.children().length||n.append("<div>"),!e&&t._loadPanel||t._renderLoadPanel(n,n.parent(),t._dataController.isLocalStore()),(e||!t.getScrollable())&&t._dataController.isLoaded()){for(var i=t.getColumns(),o=!0,a=0;a<i.length;a++)if(!i[a].width&&!i[a].minWidth){o=!1;break}(t.option("columnAutoWidth")||t._hasHeight||o||t._columnsController._isColumnFixing())&&t._renderScrollableCore(n)}},_handleScroll:function(e){var t=this,n=t.option("rtlEnabled");t._isScrollByEvent=!!e.event,t._scrollTop=e.scrollOffset.top,t._scrollLeft=e.scrollOffset.left,n&&(this._scrollRight=o(e.component)-this._scrollLeft),t.scrollChanged.fire(e.scrollOffset,t.name)},_renderScrollableCore:function(e){var t=this,n=t._createScrollableOptions(),i=t._handleScroll.bind(t);n.onScroll=i,n.onStop=i,t._scrollable=t._createComponent(e,v.default,n),t._scrollableContainer=t._scrollable&&t._scrollable._$container},_renderLoadPanel:_.default.renderLoadPanel,_renderContent:function(e,t){return e.empty().append(t),this._findContentElement()},_updateContent:function(e,t){var n=this,i=n._getTableElement(),o=n._findContentElement(),a=t&&t.changeType,r=[],s=this.option("highlightChanges"),l=this.addWidgetPrefix(M);switch(a){case"update":(0,d.each)(t.rowIndices,function(o,a){var u=n._getRowElements(e).eq(o),d=t.changeTypes&&t.changeTypes[o],h=t.items&&t.items[o];r.push(function(){var e=n._getRowElements(),r=e.eq(a);switch(d){case"update":if(h){var f=t.columnIndices&&t.columnIndices[o];(0,c.isDefined)(h.visible)&&h.visible!==r.is(":visible")?r.toggle(h.visible):f?n._updateCells(r,u,f):r.replaceWith(u)}break;case"insert":if(e.length)r.length?u.insertBefore(r):u.insertAfter(e.last());else if(i){var p=u.is("tbody")?i:i.children("tbody");u.prependTo(p)}s&&t.isLiveUpdate&&u.addClass(l);break;case"remove":r.remove()}})}),(0,d.each)(r,function(){this()}),e.remove();break;default:n._setTableElement(e),o.addClass(n.addWidgetPrefix(C)),n._renderContent(o,e)}},_createEmptyRow:function(e,t,n){var i,o=this,a=o._createRow(),r=t?this.getFixedColumns():this.getColumns();a.addClass(e).toggleClass(E,o.option("showColumnLines"));for(var s=0;s<r.length;s++)i=o._createCell({column:r[s],rowType:"freeSpace",columnIndex:s,columns:r}),(0,c.isNumeric)(n)&&i.css("height",n),a.append(i);return o.setAria("role","presentation",a),a},_appendEmptyRow:function(e,t,n){var i=this._getBodies(e),o=!i.length||t.is("tbody"),a=o?e:i;if("top"===n){if(a.first().prepend(t),o){var r=a.children("colgroup");a.prepend(r)}}else a.last().append(t)},_renderFreeSpaceRow:function(e){var t=this._createEmptyRow(T);t=this._wrapRowIfNeed(e,t),this._appendEmptyRow(e,t)},_checkRowKeys:function(e){var t=this,n=t._getRows(e),i=t._dataController.store()&&t._dataController.store().key();i&&n.some(function(e){if("data"===e.rowType&&void 0===e.key)return t._dataController.fireError("E1046",i),!0})},_needUpdateRowHeight:function(e){return e>0&&!this._rowHeight},_getRowsHeight:function(e){var t=e.children("tbody").children().not(".dx-virtual-row").not("."+T);return t.toArray().reduce(function(e,t){return e+(0,f.getBoundingRect)(t).height},0)},_updateRowHeight:function(){var e=this,t=e._getTableElement(),n=e._dataController.items().length;if(t&&e._needUpdateRowHeight(n)){var i=e._getRowsHeight(t);e._rowHeight=i/n}},_findContentElement:function(){var e=this.element(),t=this.getScrollable();if(e)return t&&(e=t.$content()),e.children().first()},_getRowElements:function(e){var t=this.callBase(e);return t&&t.not("."+T)},_getFreeSpaceRowElements:function(e){var t=e||this.getTableElements();return t&&t.children("tbody").children("."+T)},_getNoDataText:function(){return this.option("noDataText")},_rowClick:function(e){var t=this._dataController.items()[e.rowIndex]||{};this.executeAction("onRowClick",(0,h.extend)({evaluate:function(e){var n=(0,g.compileGetter)(e);return n(t.data)}},e,t))},_rowDblClick:function(e){var t=this._dataController.items()[e.rowIndex]||{};this.executeAction("onRowDblClick",(0,h.extend)({},e,t))},_getColumnsCountBeforeGroups:function(e){for(var t=0;t<e.length;t++)if("groupExpand"===e[t].type)return t;return 0},_getGroupCellOptions:function(e){var t=this._getColumnsCountBeforeGroups(e.columns),n=(e.row.groupIndex||0)+t;return{columnIndex:n,colspan:e.columns.length-n-1}},_renderCells:function(e,t){"group"===t.row.rowType?this._renderGroupedCells(e,t):t.row.values&&this.callBase(e,t)},_renderGroupedCells:function(e,t){for(var n,i,o=t.row,a=t.columns,r=o.rowIndex,s=this._getGroupCellOptions(t),l=0;l<=s.columnIndex;l++)l===s.columnIndex&&a[l].allowCollapsing&&"infinite"!==t.scrollingMode?(i=!!o.isExpanded,n=a[l]):(i=null,n={command:"expand",cssClass:a[l].cssClass}),this._renderCell(e,{value:i,row:o,rowIndex:r,column:n,columnIndex:l});var u=(0,f.getDefaultAlignment)(this.option("rtlEnabled")),c=(0,h.extend)({},a[s.columnIndex],{command:null,cssClass:null,width:null,showWhenGrouped:!1,alignment:u});s.colspan>1&&(c.colspan=s.colspan),this._renderCell(e,{value:o.values[o.groupIndex],row:o,rowIndex:r,column:c,columnIndex:s.columnIndex})},_renderRows:function(e,t){var n=this,i=n.option("scrolling.mode");n.callBase(e,(0,h.extend)({scrollingMode:i},t)),n._checkRowKeys(t.change),n._renderFreeSpaceRow(e),n._hasHeight||n.updateFreeSpaceRowHeight(e)},_renderRow:function(e,t){var n=this,i=t.row,o=n.option("rowTemplate");"data"!==i.rowType&&"group"!==i.rowType||(0,c.isDefined)(i.groupIndex)||!o?n.callBase(e,t):n.renderTemplate(e,o,(0,h.extend)({columns:t.columns},i),!0)},_renderTable:function(e){var t=this,n=t.callBase(e),i=function e(){var n=t.getScrollable();n&&t.element().closest((0,r.getWindow)().document).length&&(t.resizeCompleted.remove(e),n._visibilityChanged(!0))};return(0,c.isDefined)(t._getTableElement())?t._renderScrollable():(t._setTableElement(n),t._renderScrollable(!0),t.resizeCompleted.add(i)),n},_createTable:function(){var e=this.callBase.apply(this,arguments);return this.option("rowTemplate")&&e.appendTo(this.component.$element()),e},_renderCore:function(e){var t=this,n=t.element();n.addClass(t.addWidgetPrefix(w)).toggleClass(t.addWidgetPrefix(k),!t.option("wordWrapEnabled")),n.toggleClass(R,0===t._dataController.items().length),t.setAria("role","presentation",n);var i=t._renderTable({change:e});t._updateContent(i,e),t.callBase(e),t._lastColumnWidths=null},_getRows:function(e){return e&&e.items||this._dataController.items()},_getCellOptions:function(e){var t=this,n=e.column,i=e.row,o=i.data,a=i&&i.summaryCells,r=e.value,s=_.default.getDisplayValue(n,r,o,i.rowType),l=this.callBase(e);if(l.value=r,l.oldValue=e.oldValue,l.displayValue=s,l.row=i,l.key=i.key,l.data=o,l.rowType=i.rowType,l.values=i.values,l.text=n.command?"":_.default.formatValue(s,n),l.rowIndex=i.rowIndex,l.summaryItems=a&&a[e.columnIndex],l.resized=n.resizedCallbacks,(0,c.isDefined)(n.groupIndex)&&!n.command){var u=t.option("grouping.texts"),d=t.option("scrolling.mode");"virtual"!==d&&"infinite"!==d&&(l.groupContinuesMessage=o&&o.isContinuationOnNextPage&&u&&u.groupContinuesMessage,l.groupContinuedMessage=o&&o.isContinuation&&u&&u.groupContinuedMessage)}return l},_setRowsOpacityCore:function(e,t,n,i){var o=this._columnsController,r=o.getColumns(),s=r&&r[n],l=s&&s.isBand&&s.index;(0,d.each)(e,function(r,s){if(!(0,a.default)(s).hasClass(S))for(var u=0;u<t.length&&(!((0,c.isNumeric)(l)&&o.isParentBandColumn(t[u].index,l)||t[u].index===n)||(e.eq(r).children().eq(u).css({opacity:i}),(0,c.isNumeric)(l)));u++);})},_getDevicePixelRatio:function(){return(0,r.getWindow)().devicePixelRatio},renderNoDataText:_.default.renderNoDataText,getCellOptions:function(e,t){var n,i,o=this._dataController.items()[e];return o&&(i=(0,c.isString)(t)?this._columnsController.columnOption(t):this._columnsController.getVisibleColumns()[t],i&&(n=this._getCellOptions({value:i.calculateCellValue(o.data),rowIndex:o.rowIndex,row:o,column:i}))),n},getRow:function(e){if(e>=0){var t=this._getRowElements();if(t.length>e)return(0,a.default)(t[e])}},updateFreeSpaceRowHeight:function(e){var t=this,n=this._dataController,i=n.items(!0).length,o=this._findContentElement(),a=this._getFreeSpaceRowElements(e);if(a&&o&&n.totalCount()>=0){var r=!1;if(i>0)if(this._hasHeight)a.hide(),(0,l.deferUpdate)(function(){var n=t.getScrollbarWidth(!0),i=t.element().height()-n,s=o.outerHeight(),u=i-s>0,c=t._getRowsHeight(o.children().first()),d=e||t.getTableElements(),h=Math.ceil(parseFloat(d.css("borderTopWidth"))),f=t._getHeightCorrection(),p=i-c-h-f;u&&(0,l.deferRender)(function(){a.css("height",p),r=!0,a.show()}),(0,l.deferRender)(function(){return t._updateLastRowBorder(r)})});else{var s=n.pageSize()-i,c=this.option("scrolling.mode");s>0&&n.pageCount()>1&&"virtual"!==c&&"infinite"!==c&&((0,u.setHeight)(a,s*this._rowHeight),r=!0),!r&&e?(0,u.setHeight)(a,0):a.toggle(r),this._updateLastRowBorder(r)}else a.css("height",0),a.show(),this._updateLastRowBorder(!0)}},_getHeightCorrection:function(){var e=x.default.webkit&&this._getDevicePixelRatio()>=2,t=x.default.mozilla&&x.default.version>=70&&!this.option("showRowLines");return e||t?1:0},_columnOptionChanged:function(e){var t=e.optionNames;e.changeTypes.grouping||(t.width||t.visibleWidth)&&(this.callBase(e),this._fireColumnResizedCallbacks())},getScrollable:function(){return this._scrollable},init:function(){var e=this,t=e.getController("data");e.callBase(),e._editorFactoryController=e.getController("editorFactory"),e._rowHeight=0,e._scrollTop=0,e._scrollLeft=-1,e._scrollRight=0,e._hasHeight=!1,t.loadingChanged.add(function(t,n){e.setLoading(t,n)}),t.dataSourceChanged.add(function(){e._scrollLeft>=0&&e._handleScroll({component:e.getScrollable(),scrollOffset:{top:e._scrollTop,left:e._scrollLeft}})})},_handleDataChanged:function(e){var t=this;switch(e.changeType){case"refresh":case"prepend":case"append":case"update":t.render(null,e);break;default:t._update(e)}},publicMethods:function(){return["isScrollbarVisible","getTopVisibleRowData","getScrollbarWidth","getCellElement","getRowElement","getScrollable"]},contentWidth:function(){return this.element().width()-this.getScrollbarWidth()},getScrollbarWidth:function(e){var n=this._scrollableContainer&&this._scrollableContainer.get(0),i=0;return n&&(e?(i=n.clientHeight?n.offsetHeight-n.clientHeight:0,i+=t(this)):i=n.clientWidth?n.offsetWidth-n.clientWidth:0),i>0?i:0},_fireColumnResizedCallbacks:function(){for(var e=this,t=e._lastColumnWidths||[],n=[],i=e.getColumns(),o=0;o<i.length;o++)n[o]=i[o].visibleWidth,i[o].resizedCallbacks&&!(0,c.isDefined)(i[o].groupIndex)&&t[o]!==n[o]&&i[o].resizedCallbacks.fire(n[o]);e._lastColumnWidths=n},_updateLastRowBorder:function(e){this.option("showBorders")&&this.option("showRowLines")&&!e?this.element().addClass(P):this.element().removeClass(P)},_updateScrollable:function(){var e=v.default.getInstance(this.element());e&&(e.update(),this._updateHorizontalScrollPosition())},_updateHorizontalScrollPosition:function(){var e=this.getScrollable(),t=e&&e.scrollOffset().left,n=this.option("rtlEnabled");if(n){var i=o(e),a=i-t;a!==this._scrollRight&&(this._scrollLeft=i-this._scrollRight)}this._scrollLeft>=0&&t!==this._scrollLeft&&e.scrollTo({x:this._scrollLeft})},_resizeCore:function(){var e=this;e._fireColumnResizedCallbacks(),e._updateRowHeight(),(0,l.deferRender)(function(){e._renderScrollable(),e.renderNoDataText(),e.updateFreeSpaceRowHeight(),(0,l.deferUpdate)(function(){e._updateScrollable()})})},scrollTo:function(e){var t=this.element(),n=t&&v.default.getInstance(t);
n&&n.scrollTo(e)},height:function(e,t){var n=this,i=this.element();return 0===arguments.length?i?i.outerHeight(!0):0:(n._hasHeight=void 0===t?"auto"!==e:t,void((0,c.isDefined)(e)&&i&&(0,u.setHeight)(i,e)))},setLoading:function(e,t){var n=this,i=n._loadPanel,o=n._dataController,a=n.option("loadPanel")||{},s=o.isLoaded()?a.animation:null,l=n.element();if((0,r.hasWindow)()&&(!i&&void 0!==t&&o.isLocalStore()&&"auto"===a.enabled&&l&&(n._renderLoadPanel(l,l.parent()),i=n._loadPanel),i)){var u={message:t||a.text,animation:s,visible:e};clearTimeout(n._hideLoadingTimeoutID),i.option("visible")&&!e?n._hideLoadingTimeoutID=setTimeout(function(){i.option(u)},B):i.option(u)}},setRowsOpacity:function(e,t){var n=this._getRowElements().not("."+S)||[];this._setRowsOpacityCore(n,this.getColumns(),e,t)},_getCellElementsCore:function(e){var t=this.callBase(e);if(t){var n=t.filter("."+D).index();if(n>=0&&t.length>n+1)return t.slice(0,n+1)}return t},getTopVisibleItemIndex:function(e){var t=this,n=0,i=0,o=0,a=t._scrollTop,r=t._findContentElement(),s=r&&r.offset().top,l=t._dataController.items(),u=t._getTableElement();if(l.length&&u){var c=t._getRowElements(u).filter(":visible");for(n=0;n<l.length;n++){i=o;var d=c.eq(n);if(d.length&&(o=d.offset().top-s,o>a)){n&&(e||2*a<Math.round(o+i))&&n--;break}}n&&n===l.length&&n--}return n},getTopVisibleRowData:function(){var e=this.getTopVisibleItemIndex(),t=this._dataController.items();if(t[e])return t[e].data},_scrollToElement:function(e,t){var n=this.getScrollable();n&&n.scrollToElement(e,t)},optionChanged:function(e){var t=this;switch(t.callBase(e),e.name){case"wordWrapEnabled":case"showColumnLines":case"showRowLines":case"rowAlternationEnabled":case"rowTemplate":case"twoWayBindingEnabled":t._invalidate(!0,!0),e.handled=!0;break;case"scrolling":t._rowHeight=null,t._tableElement=null,e.handled=!0;break;case"rtlEnabled":t._rowHeight=null,t._tableElement=null;break;case"loadPanel":t._tableElement=null,t._invalidate(!0,"loadPanel.enabled"!==e.fullName),e.handled=!0;break;case"noDataText":t.renderNoDataText(),e.handled=!0}},dispose:function(){clearTimeout(this._hideLoadingTimeoutID),this._scrollable&&this._scrollable.dispose()},setScrollerSpacing:function(){}}}())}};t.default=V,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(18),r=n(4),s=n(3),l=i(n(44)),u=i(n(110)),c="dx-context-menu",d={columnHeadersView:"header",rowsView:"content",footerView:"footer",headerPanel:"headerPanel"},h=["columnHeadersView","rowsView","footerView","headerPanel"],f=l.default.ViewController.inherit({init:function(){this.createAction("onContextMenuPreparing")},getContextMenuItems:function(e){if(!e)return!1;var t,n,i,r,l=this,u=(0,o.default)(e.target);return(0,s.each)(h,function(){var o=l.getView(this);if(t=o&&o.element(),t&&(t.is(u)||t.find(u).length)){i=u.closest(".dx-row > td, .dx-row > tr"),n=i.parent();var s=o.getRowIndex(n),c=i[0]&&i[0].cellIndex,h=n.data("options"),f={event:e,targetElement:(0,a.getPublicElement)(u),target:d[this],rowIndex:s,row:o._getRows()[s],columnIndex:c,column:h&&h.cells[c].column};if(f.items=o.getContextMenuItems&&o.getContextMenuItems(f),l.executeAction("onContextMenuPreparing",f),l._contextMenuPrepared(f),r=f.items)return!1}}),r},_contextMenuPrepared:r.noop}),p=l.default.View.inherit({_renderCore:function(){var e=this,t=e.element().addClass(c);this.setAria("role","presentation",t),this._createComponent(t,u.default,{onPositioning:function(t){var n=t.event,i=t.component,o=e.getController("contextMenu").getContextMenuItems(n);o?(i.option("items",o),n.stopPropagation()):t.cancel=!0},onItemClick:function(e){e.itemData.onItemClick&&e.itemData.onItemClick(e)},cssClass:e.getWidgetContainerClass(),target:e.component.$element()})}}),g={defaultOptions:function(){return{onContextMenuPreparing:null}},controllers:{contextMenu:f},views:{contextMenuView:p}};t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(21),s=n(3),l=i(n(44)),u="dx-error-row",c="dx-error-message",d="dx-closebutton",h="action",f=l.default.ViewController.inherit({init:function(){var e=this;e._columnHeadersView=e.getView("columnHeadersView"),e._rowsView=e.getView("rowsView")},_createErrorRow:function(e,t){var n,i,l=this,c=this._renderErrorMessage(e);return t?(n=(0,o.default)("<tr>").addClass(u),i=(0,o.default)("<div>").addClass(d).addClass(l.addWidgetPrefix(h)),a.default.on(i,r.name,l.createAction(function(e){var n,i=e.event,a=(0,o.default)(i.currentTarget).closest("."+u).index();i.stopPropagation(),(0,s.each)(t,function(e,t){n=(0,o.default)(t).children("tbody").children("tr").eq(a),l.removeErrorRow(n)}),l.getController("resizing")&&l.getController("resizing").fireContentReadyAction()})),(0,o.default)("<td>").attr({colSpan:l.getController("columns").getVisibleColumns().length,role:"presentation"}).prepend(i).append(c).appendTo(n),n):c},_renderErrorMessage:function(e){var t=e.url?e.message.replace(e.url,""):e.message||e,n=(0,o.default)("<div>").addClass(c).text(t);return e.url&&(0,o.default)("<a>").attr("href",e.url).text(e.url).appendTo(n),n},renderErrorRow:function(e,t,n){var i,a,r=this;if(n)return n.find("."+c).remove(),i=r._createErrorRow(e),n.prepend(i),i;var l=t>=0||!r._columnHeadersView.isVisible()?r._rowsView:r._columnHeadersView,u=n||l.getTableElements();if((0,s.each)(u,function(n,s){if(i=r._createErrorRow(e,u),a=a||i,t>=0){var c=l._getRowElements((0,o.default)(s)).eq(t);r.removeErrorRow(c.next()),i.insertAfter(c)}else{var d=(0,o.default)(s).children("tbody"),h=d.children("tr");r._columnHeadersView.isVisible()?(r.removeErrorRow(h.last()),(0,o.default)(s).append(i)):(r.removeErrorRow(h.first()),d.first().prepend(i))}}),!n){var d=r.getController("resizing");d&&d.fireContentReadyAction()}return a},removeErrorRow:function(e){if(!e){var t=this._columnHeadersView&&this._columnHeadersView.element();if(e=t&&t.find("."+u),!e||!e.length){var n=this._rowsView.element();e=n&&n.find("."+u)}}e&&e.hasClass(u)&&e.remove()},optionChanged:function(e){var t=this;switch(e.name){case"errorRowEnabled":e.handled=!0;break;default:t.callBase(e)}}}),p={defaultOptions:function(){return{errorRowEnabled:!0}},controllers:{errorHandling:f},extenders:{controllers:{data:{init:function(){var e=this,t=e.getController("errorHandling");e.callBase(),e.dataErrorOccurred.add(function(n,i){e.option("errorRowEnabled")&&t.renderErrorRow(n,void 0,i)}),e.changed.add(function(t){if(!t||"loadError"!==t.changeType){var n=e.getController("errorHandling"),i=e.getController("editing");i&&!i.hasChanges()&&n&&n.removeErrorRow()}})}}}}};t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var s=r(n(2)),l=r(n(44)),u=n(4),c=n(7),d=n(3),h=n(1),f=n(24),p=r(n(37)),g=r(n(10)),_=n(6),m=r(n(11)),v=r(n(23)),y=a(n(128)),b="table",x="borders",w="table-fixed",C="important-margin",k="text-content",S="dx-hidden",D="dx-gridbase-container",I="adaptiveHidden",T="input:not([type='hidden'])",O=["columnsSeparatorView","blockSeparatorView","trackerView","headerPanel","columnHeadersView","rowsView","footerView","columnChooserView","filterPanelView","pagerView","draggingHeaderView","contextMenuView","errorView","headerFilterView","filterBuilderView"],E=function(e){return(0,h.isString)(e)&&"%"===e.slice(-1)},A=function(e){return(0,h.isString)(e)&&"px"===e.slice(-2)},P=function(e,t){var n=[];if(e&&t&&e.length&&e.length===t.length)for(var i=0;i<e.length;i++)n.push(e[i]>t[i]?e[i]:t[i]);else e&&e.length?n=e:t&&(n=t);return n},R=function(e){var t=e.get(0).clientHeight,n=parseFloat(e.css("paddingTop")),i=parseFloat(e.css("paddingBottom"));return t-n-i},M=function(e,t){var n=e._rowsView.contentWidth(),i=e._getTotalWidth(t,n);return n-i},B=function(e,t,n,i){return M(e,i.map(function(e,i){return i===t?n:e}))},V=function(e,t){y.hiddenFocus(e),p.default.setSelectionRange(e,t)},F=l.default.ViewController.inherit({_initPostRenderHandlers:function(){var e=this,t=e._dataController;e._refreshSizesHandler||(e._refreshSizesHandler=function(n){t.changed.remove(e._refreshSizesHandler);var i,o=n&&n.changeType,a=n&&n.isDelayed,r=t.items();n&&"refresh"!==o&&"prepend"!==o&&"append"!==o?"update"===o&&n.changeTypes&&(!(r.length>1||"insert"!==n.changeTypes[0])||0===r.length&&"remove"===n.changeTypes[0]||n.needUpdateDimensions?i=e.resize():(0,u.deferUpdate)(function(){e._rowsView.resize()})):a||(i=e.resize()),o&&"updateSelection"!==o&&"updateFocusedRow"!==o&&!a&&(0,_.when)(i).done(function(){e._setAriaRowColCount(),e.fireContentReadyAction()})},e._dataController.changed.add(function(){e._dataController.changed.add(e._refreshSizesHandler)}))},fireContentReadyAction:function(){this.component._fireContentReadyAction()},_setAriaRowColCount:function(){var e=this.component;e.setAria({rowCount:this._dataController.totalItemsCount(),colCount:e.columnCount()},e.$element().children("."+D))},_getBestFitWidths:function(){if(!this.option("legacyRendering"))return this._rowsView.getColumnWidths();var e=this._rowsView.getColumnWidths(),t=this._columnHeadersView&&this._columnHeadersView.getColumnWidths(),n=this._footerView&&this._footerView.getColumnWidths(),i=P(e,t);return i=P(i,n)},_setVisibleWidths:function(e,t){var n=this._columnsController;n.beginUpdate(),(0,d.each)(e,function(e,i){var o=n.getColumnId(i);n.columnOption(o,"visibleWidth",t[e])}),n.endUpdate()},_toggleBestFitModeForView:function(e,t,n){var i=this;if(e&&e.isVisible()){var o=this._rowsView.getTableElements(),a=e.getTableElements();(0,d.each)(o,function(e,o){var r,l=(0,s.default)(o),u=a.eq(e);u&&u.length&&(r=n?u.children("tbody").appendTo(l):l.children("."+t).appendTo(u),r.toggleClass(t,n),r.toggleClass(i.addWidgetPrefix("best-fit"),n))})}},_toggleBestFitMode:function(e){var t=this.component.$element(),n=this;if(n.option("legacyRendering"))t.find("."+this.addWidgetPrefix(b)).toggleClass(this.addWidgetPrefix(w),!e),t.find(T).toggleClass(S,e),t.find(".dx-group-cell").toggleClass(S,e),t.find(".dx-header-row ."+this.addWidgetPrefix(k)).css("maxWidth","");else{var i=n._rowsView._getTableElement(),o=n._rowsView.getTableElements().eq(1);if(!i)return;i.css("tableLayout",e?"auto":"fixed"),i.children("colgroup").css("display",e?"none":""),o.toggleClass(this.addWidgetPrefix(w),!e),n._toggleBestFitModeForView(n._columnHeadersView,"dx-header",e),n._toggleBestFitModeForView(n._footerView,"dx-footer",e),n._needStretch()&&(i.get(0).style.width=e?"auto":""),v.default.msie&&11===parseInt(v.default.version)&&i.find("."+this.addWidgetPrefix(w)).each(function(){this.style.width=e?"10px":""})}},_synchronizeColumns:function(){var e,t,n,i=this,o=i._columnsController,a=o.getVisibleColumns(),r=i.option("columnAutoWidth"),s=i.option("legacyRendering"),l=i._needBestFit(),c=!1,g=!1,_=[],y=function(){var e;(0,d.each)(a,function(t,n){"groupExpand"===n.type&&(e=_[t])}),(0,d.each)(a,function(t,n){"groupExpand"===n.type&&e&&(_[t]=e)})};!l&&(0,d.each)(a,function(e,t){if("auto"===t.width||s&&t.fixed)return l=!0,!1}),(0,d.each)(a,function(e,t){if(t.minWidth)return c=!0,!1}),i._setVisibleWidths(a,[]),l&&(t=m.default.getActiveElement(),n=p.default.getSelectionRange(t),i._toggleBestFitMode(!0),e=!0),(0,u.deferUpdate)(function(){if(l?(_=i._getBestFitWidths(),(0,d.each)(a,function(e,t){var n=o.getColumnId(t);o.columnOption(n,"bestFitWidth",_[e],!0)})):c&&(_=i._getBestFitWidths()),(0,d.each)(a,function(e){var t=this.width;"auto"!==t&&((0,h.isDefined)(t)?_[e]=(0,h.isNumeric)(t)||A(t)?parseFloat(t):t:r||(_[e]=void 0))}),e&&(i._toggleBestFitMode(!1),e=!1,t&&t!==m.default.getActiveElement())){var s=(0,f.getBoundingRect)(t).bottom<0;s||(v.default.msie?setTimeout(function(){V(t,n)}):V(t,n))}g=i._correctColumnWidths(_,a),r&&(y(),i._needStretch()&&i._processStretch(_,a)),(0,u.deferRender)(function(){(l||g)&&i._setVisibleWidths(a,_)})})},_needBestFit:function(){return this.option("columnAutoWidth")},_needStretch:function(){return this.option("legacyRendering")||this._columnsController.getVisibleColumns().some(function(e){return"auto"===e.width&&!e.command})},_getAverageColumnsWidth:function(e){var t=M(this,e),n=e.filter(function(e){return void 0===e}).length;return t/n},_correctColumnWidths:function(e,t){var n,i=this,o=!1,a=!1,r=!1,s=i.component.$element(),l=i._hasWidth,u=function(){var s=n,l=t[s],u=e[s]===I,c=e[s],d=l.minWidth;if(d)if(void 0===c){var f=i._getAverageColumnsWidth(e);c=f}else if(E(c)){var p=B(i,s,d,e);p<0&&(c=-1)}var g=i._getRealColumnWidth(s,e.map(function(e,t){return s===t?c:e}));d&&!u&&g<d&&(e[s]=d,r=!0,n=-1),(0,h.isDefined)(l.width)||(a=!0),E(l.width)&&(o=!0)};for(n=0;n<t.length;n++)u();if(s&&i._maxWidth&&(delete i._maxWidth,s.css("maxWidth","")),!a&&e.length){var c=i._rowsView.contentWidth(),d=i._rowsView.getScrollbarWidth(),f=i._getTotalWidth(e,c);if(f<c){var g=p.default.getLastResizableColumnIndex(t,e);g>=0&&(e[g]="auto",r=!0,l!==!1||o||(i._maxWidth=f+d+(i.option("showBorders")?2:0),s.css("maxWidth",i._maxWidth)))}}return r},_processStretch:function(e,t){var n=this._rowsView.contentWidth(),i=this._getTotalWidth(e,n),o={length:0};if(e.length){(0,d.each)(t,function(t){(this.width||e[t]===I)&&(o[t]=!0,o.length++)});var a=n-i,r=Math.floor(a/(e.length-o.length)),s=a-r*(e.length-o.length);if(a>=0)for(var l=0;l<e.length;l++)o[l]||(e[l]+=r,s>0&&(s<1?(e[l]+=s,s=0):(e[l]++,s--)))}},_getRealColumnWidth:function(e,t,n){var i=1,o=t[e];if(!E(o))return parseFloat(o);var a=t.reduce(function(e,t,n){return E(t)?e+parseFloat(t):e},0),r=t.reduce(function(e,t){return!t||t===I||E(t)?e:e+parseFloat(t)},0);n=n||this._rowsView.contentWidth();var s=n-r,l=a*n/100;return r>0&&l+r>=n&&(i=l>s?s/l:1),parseFloat(o)*n*i/100},_getTotalWidth:function(e,t){for(var n=0,i=0;i<e.length;i++){var o=e[i];o&&o!==I&&(n+=this._getRealColumnWidth(i,e,t))}return n},updateSize:function(e){var t,n,i=this,o=i.addWidgetPrefix(C);void 0===i._hasHeight&&e&&e.is(":visible")&&e.width()&&(t=e.children("."+i.getWidgetContainerClass()),t.length&&t.detach(),i._hasHeight=!!R(e),n=e.width(),e.addClass(o),i._hasWidth=e.width()===n,e.removeClass(o),t.length&&t.appendTo(e))},publicMethods:function(){return["resize","updateDimensions"]},resize:function(){return!this.component._requireResize&&this.updateDimensions()},updateDimensions:function(e){var t=this;if(t._initPostRenderHandlers(),t._checkSize(e)){var n=t._resizeDeferred,i=t._resizeDeferred=new _.Deferred;return(0,_.when)(n).always(function(){(0,u.deferRender)(function(){t._dataController.isLoaded()&&t._synchronizeColumns(),t._resetGroupElementHeight(),(0,u.deferUpdate)(function(){(0,u.deferRender)(function(){(0,u.deferUpdate)(function(){t._updateDimensionsCore()})})})}).done(i.resolve).fail(i.reject)}),i.promise()}},_resetGroupElementHeight:function(){var e=this.component.$element().children().get(0),t=this._rowsView.getScrollable();!e||!e.style.height||t&&t.scrollTop()||(e.style.height="")},_checkSize:function(e){var t=this.component.$element();return!(e&&(this._lastWidth===t.width()&&this._lastHeight===t.height()||!t.is(":visible")))},_setScrollerSpacingCore:function(e){var t=this,n=e?t._rowsView.getScrollbarWidth():0,i=t._rowsView.getScrollbarWidth(!0);(0,u.deferRender)(function(){t._columnHeadersView&&t._columnHeadersView.setScrollerSpacing(n),t._footerView&&t._footerView.setScrollerSpacing(n),t._rowsView.setScrollerSpacing(n,i)})},_setScrollerSpacing:function(e){var t=this;this.option("scrolling.useNative")===!0?(0,u.deferRender)(function(){(0,u.deferUpdate)(function(){t._setScrollerSpacingCore(e)})}):this._setScrollerSpacingCore(e)},_updateDimensionsCore:function(){var e,t=this,n=t._dataController,i=t._rowsView,o=t.component.$element(),a=o.children().get(0),r=o&&(o.get(0).clientHeight||o.height()),l=parseInt(o.css("maxHeight")),c=l&&r>=l,h=t.option("height")||o.get(0).style.height,f=t.getController("editorFactory"),p=c&&a.scrollHeight===a.offsetHeight;t.updateSize(o);var g=t._hasHeight||c;h&&t._hasHeight^"auto"!==h&&(e=(0,s.default)("<div>").height(h).appendTo(o),t._hasHeight=!!e.height(),e.remove()),(0,u.deferRender)(function(){return i.height(null,g),c&&!p&&(0,s.default)(a).css("height",l),n.isLoaded()?void(0,u.deferUpdate)(function(){t._updateLastSizes(o),t._setScrollerSpacing(g),(0,d.each)(O,function(e,n){var i=t.getView(n);i&&i.resize()}),f&&f.resize()}):void i.setLoading(n.isLoading())})},_updateLastSizes:function(e){this._lastWidth=e.width(),this._lastHeight=e.height()},optionChanged:function(e){switch(e.name){case"width":case"height":this.component._renderDimensions(),this.resize();case"legacyRendering":case"renderAsync":return void(e.handled=!0);default:this.callBase(e)}},init:function(){var e=this;e._dataController=e.getController("data"),e._columnsController=e.getController("columns"),e._columnHeadersView=e.getView("columnHeadersView"),e._footerView=e.getView("footerView"),e._rowsView=e.getView("rowsView")}}),L=l.default.ViewController.inherit({_scrollChangedHandler:function(e,t,n){for(var i=0;i<e.length;i++)e[i]&&e[i].name!==n&&e[i].scrollTo({left:t.left,top:t.top})},init:function(){for(var e=[this.getView("columnHeadersView"),this.getView("footerView"),this.getView("rowsView")],t=0;t<e.length;t++){var n=e[t];n&&n.scrollChanged.add(this._scrollChangedHandler.bind(this,e))}}}),H=l.default.View.inherit({_endUpdateCore:function(){this.component._requireResize&&(this.component._requireResize=!1,this._resizingController.resize())},_getWidgetAriaLabel:function(){return"dxDataGrid-ariaDataGrid"},init:function(){var e=this;e._resizingController=e.getController("resizing"),e._dataController=e.getController("data")},getView:function(e){return this.component._views[e]},element:function(){return this._groupElement},optionChanged:function(e){var t=this;(0,h.isDefined)(t._groupElement)&&"showBorders"===e.name?(t._groupElement.toggleClass(t.addWidgetPrefix(x),!!e.value),e.handled=!0):t.callBase(e)},_renderViews:function(e){var t=this;(0,d.each)(O,function(n,i){var o=t.getView(i);o&&o.render(e)})},_getTableRoleName:function(){return"grid"},render:function(e){var t=this,n=!t._groupElement,i=t._groupElement||(0,s.default)("<div>").addClass(t.getWidgetContainerClass());i.addClass(D),i.toggleClass(t.addWidgetPrefix(x),!!t.option("showBorders")),t.setAria("role","presentation",e),t.component.setAria({role:this._getTableRoleName(),label:g.default.format(t._getWidgetAriaLabel())},i),t._rootElement=e||t._rootElement,n&&(t._groupElement=i,(0,c.hasWindow)()&&t.getController("resizing").updateSize(e),i.appendTo(e)),t._renderViews(i)},update:function(){var e=this,t=e._rootElement,n=e._groupElement,i=e.getController("resizing");t&&n&&(i.resize(),e._dataController.isLoaded()&&e._resizingController.fireContentReadyAction())}}),z={defaultOptions:function(){return{showBorders:!1,renderAsync:!1,legacyRendering:!1}},controllers:{resizing:F,synchronizeScrolling:L},views:{gridView:H}};t.default=z,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(166)),r=n(197),s=n(4),l=n(1),u=n(55);n(219);var c="header-panel",d="toolbar-button",h=r.ColumnsView.inherit({_getToolbarItems:function(){return[]},_getButtonContainer:function(){return(0,o.default)("<div>").addClass(this.addWidgetPrefix(d))},_getToolbarButtonClass:function(e){var t=e?" "+e:"";return this.addWidgetPrefix(d)+t},_getToolbarOptions:function(){var e={toolbarOptions:{items:this._getToolbarItems(),onItemRendered:function(e){var t=e.itemData.onItemRendered;t&&t(e)}}};if(this.executeAction("onToolbarPreparing",e),e.toolbarOptions&&!(0,l.isDefined)(e.toolbarOptions.visible)){var t=e.toolbarOptions.items;e.toolbarOptions.visible=!(!t||!t.length)}return e.toolbarOptions},_renderCore:function(){this._toolbar?this._toolbar.option(this._toolbarOptions):(this.element().addClass(this.addWidgetPrefix(c)),this._toolbar=this._createComponent((0,o.default)("<div>").appendTo(this.element()),a.default,this._toolbarOptions))},_columnOptionChanged:s.noop,_handleDataChanged:function(){this._requireReady&&this.render()},init:function(){this.callBase(),this.createAction("onToolbarPreparing",{excludeValidators:["disabled","readOnly"]})},render:function(){this._toolbarOptions=this._getToolbarOptions(),this.callBase.apply(this,arguments)},setToolbarItemDisabled:function(e,t){var n=this._toolbar;if(n){var i=n.option("items")||[],o=i.indexOf(i.filter(function(t){return t.name===e})[0]);if(o>=0){var a="items["+o+"]";n.option(a+".options")?n.option(a+".options.disabled",t):n.option(a+".disabled",t)}}},updateToolbarDimensions:function(){this._toolbar&&(0,u.triggerResizeEvent)(this.getHeaderPanel())},getHeaderPanel:function(){return this.element()},getHeight:function(){return this.getElementHeight()},optionChanged:function(e){"onToolbarPreparing"===e.name&&(this._invalidate(),e.handled=!0),this.callBase(e)},isVisible:function(){return this._toolbarOptions&&this._toolbarOptions.visible},allowDragging:s.noop}),f={defaultOptions:function(){return{}},views:{headerPanel:h},extenders:{controllers:{resizing:{_updateDimensionsCore:function(){this.callBase.apply(this,arguments),this.getView("headerPanel").updateToolbarDimensions()}}}}};t.default=f,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(4),o=n(1),a=n(0),r=n(474),s=n(6),l=function(e){var t=e.getView("pagerView"),n=e.getController("data"),i={allowedPageSizes:t?t.getPageSizes():void 0,filterPanel:{filterEnabled:e.option("filterPanel.filterEnabled")},filterValue:e.option("filterValue"),focusedRowKey:e.option("focusedRowEnabled")?e.option("focusedRowKey"):void 0};return(0,a.extend)(i,n.getUserState())},u=function(e){var t=e.getController("columns"),n=e.getController("selection"),i=e.getController("export"),o=e.getController("data");t&&t.columnsChanged.add(function(){e.updateState({columns:t.getUserState()})}),n&&n.selectionChanged.add(function(t){e.updateState({selectedRowKeys:t.selectedRowKeys,selectionFilter:t.selectionFilter})}),o&&(e._initialPageSize=e.option("paging.pageSize"),e._initialFilterValue=e.option("filterValue"),o.changed.add(function(){var t=l(e);e.updateState(t)})),i&&i.selectionOnlyChanged.add(function(){e.updateState({exportSelectionOnly:i.selectionOnly()})})},c=null,d=function(e,t){var n=e.getController("filterSync"),i=e.getController("columns"),o=t.columns||void 0!==t.filterValue;return n?o?t.filterValue||n.getFilterValueFromColumns(t.columns):e._initialFilterValue||n.getFilterValueFromColumns(i.getColumns()):c},h={defaultOptions:function(){return{stateStoring:{enabled:!1,storageKey:null,type:"localStorage",customLoad:null,customSave:null,savingTimeout:2e3}}},controllers:{stateStoring:r.StateStoringController},extenders:{views:{rowsView:{init:function(){var e=this,t=e.getController("data");e.callBase(),t.stateLoaded.add(function(){if(t.isLoaded()&&!t.getDataSource()){e.setLoading(!1),e.renderNoDataText();var n=e.component.getView("columnHeadersView");n&&n.render(),e.component._fireContentReadyAction()}})}}},controllers:{stateStoring:{init:function(){this.callBase.apply(this,arguments),u(this)},isLoading:function(){return this.callBase()||this.getController("data").isStateLoading()},state:function(e){var t=this.callBase.apply(this,arguments);return void 0!==e&&this.applyState((0,a.extend)({},e)),t},updateState:function(e){if(this.isEnabled()){var t=this.state(),n=(0,a.extend)({},t,e),o=(0,i.getKeyHash)(t),r=(0,i.getKeyHash)(n);(0,i.equalByValue)(o,r)||((0,a.extend)(this._state,e),this.save())}else(0,a.extend)(this._state,e)},applyState:function(e){var t=this,n=e.allowedPageSizes,i=e.searchText,a=e.selectedRowKeys,r=e.selectionFilter,s=t.getController("export"),l=t.getController("columns"),u=t.getController("data"),c=t.option("scrolling.mode"),h="virtual"===c||"infinite"===c,f=t.option("pager.visible")===!0&&t.option("pager.showPageSizeSelector");t.component.beginUpdate(),l&&l.setUserState(e.columns),s&&s.selectionOnly(e.exportSelectionOnly),a&&t.option("selectedRowKeys",a),t.option("selectionFilter",r),n&&"auto"===t.option("pager.allowedPageSizes")&&(t.option("pager").allowedPageSizes=n),t.option("focusedRowEnabled")&&t.option("focusedRowKey",e.focusedRowKey),t.component.endUpdate(),i&&t.option("searchPanel.text",i),t.option("filterValue",d(t,e)),t.option("filterPanel.filterEnabled",!e.filterPanel||e.filterPanel.filterEnabled),t.option("paging.pageSize",h&&!f||!(0,o.isDefined)(e.pageSize)?t._initialPageSize:e.pageSize),t.option("paging.pageIndex",e.pageIndex||0),u&&u.reset()}},columns:{getVisibleColumns:function(){var e=this.callBase.apply(this,arguments),t=this.getController("stateStoring");return t.isEnabled()&&!t.isLoaded()?[]:e}},data:{callbackNames:function(){return this.callBase().concat(["stateLoaded"])},_refreshDataSource:function(){var e=this,t=this.callBase,n=this.getController("stateStoring");if(n.isEnabled()&&!n.isLoaded()){clearTimeout(this._restoreStateTimeoutID);var i=new s.Deferred;return this._restoreStateTimeoutID=setTimeout(function(){n.load().always(function(){e._restoreStateTimeoutID=null}).done(function(){t.call(e),e.stateLoaded.fire(),i.resolve()}).fail(function(t){e.stateLoaded.fire(),e._handleLoadError(t||"Unknown error"),i.reject()})}),i.promise()}this.isStateLoading()||t.call(this)},isLoading:function(){var e=this,t=e.getController("stateStoring");return this.callBase()||t.isLoading()},isStateLoading:function(){return(0,o.isDefined)(this._restoreStateTimeoutID)},isLoaded:function(){return this.callBase()&&!this.isStateLoading()},dispose:function(){clearTimeout(this._restoreStateTimeoutID),this.callBase()}},selection:{_fireSelectionChanged:function(e){var t=this.getController("stateStoring"),n=this.option("selection.deferred");t.isLoading()&&n||this.callBase.apply(this,arguments)}}}}};t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.StateStoringController=void 0;var o=i(n(5)),a=n(7),r=i(n(44)),s=i(n(17)),l=i(n(23)),u=n(269),c=n(0),d=n(3),h=n(1),f=n(6),p=/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/,g=function e(t){t&&(0,d.each)(t,function(n,i){if((0,h.isPlainObject)(i)||Array.isArray(i))e(i);else if("string"==typeof i){var o=p.exec(i);o&&(t[n]=new Date(Date.UTC(+o[1],+o[2]-1,+o[3],+o[4],+o[5],+o[6])))}})},_=r.default.ViewController.inherit(function(){var e=function(e){var t="sessionStorage"===e.type?(0,u.sessionStorage)():(0,a.getWindow)().localStorage;if(!t)throw"file:"===(0,a.getWindow)().location.protocol&&l.default.msie?new Error("E1038"):new Error("E1007");return t},t=function(e){return(0,h.isDefined)(e.storageKey)?e.storageKey:"storage"};return{_loadState:function(){var n=this.option("stateStoring");if("custom"===n.type)return n.customLoad&&n.customLoad();try{return JSON.parse(e(n).getItem(t(n)))}catch(e){s.default.log(e.message)}},_saveState:function(n){var i=this.option("stateStoring");if("custom"===i.type)return void(i.customSave&&i.customSave(n));try{e(i).setItem(t(i),JSON.stringify(n))}catch(e){s.default.log(e.message)}},publicMethods:function(){return["state"]},isEnabled:function(){return this.option("stateStoring.enabled")},init:function(){var e=this;return e._state={},e._isLoaded=!1,e._isLoading=!1,e._windowUnloadHandler=function(){void 0!==e._savingTimeoutID&&e._saveState(e.state())},o.default.on((0,a.getWindow)(),"unload",e._windowUnloadHandler),e},isLoaded:function(){return this._isLoaded},isLoading:function(){return this._isLoading},load:function(){var e=this;this._isLoading=!0;var t=(0,f.fromPromise)(this._loadState());return t.always(function(){e._isLoaded=!0,e._isLoading=!1}).done(function(t){e.state(t)}),t},state:function(e){var t=this;return arguments.length?(t._state=(0,c.extend)({},e),void g(t._state)):(0,c.extend)(!0,{},t._state)},save:function(){var e=this;clearTimeout(e._savingTimeoutID),e._savingTimeoutID=setTimeout(function(){e._saveState(e.state()),e._savingTimeoutID=void 0},e.option("stateStoring.savingTimeout"))},optionChanged:function(e){var t=this;switch(e.name){case"stateStoring":t.isEnabled()&&!t.isLoading()&&t.load(),e.handled=!0;break;default:t.callBase(e)}},dispose:function(){clearTimeout(this._savingTimeoutID),o.default.off((0,a.getWindow)(),"unload",this._windowUnloadHandler)}}}());t.StateStoringController=_},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(26)),s=i(n(37)),l=n(1),u=n(3),c=n(0),d=n(46),h=n(21),f=i(n(10)),p=n(8),g=i(n(103)),_=i(n(210)),m=n(6),v="dx-editor-cell",y="dx-row",b="dx-selection",x="dx-select-checkbox",w="dx-select-checkboxes-hidden",C="dx-command-select",k="dx-selection-disabled",S="dx-data-row",D="selection.showCheckBoxesMode",I="selection.mode",T=function(e,t){var n=e.getController("selection"),i=e.getView("rowsView"),a=(0,o.default)(t.target).closest("."+S),r=i.getRowIndex(a);r<0||("onLongTap"===e.option(D)?n.isSelectionWithCheckboxes()?n.stopSelectionWithCheckboxes():n.startSelectionWithCheckboxes():("onClick"===e.option(D)&&n.startSelectionWithCheckboxes(),"always"!==e.option(D)&&n.changeItemSelection(r,{control:!0})))},O=r.default.Controller.inherit(function(){var e=function(e,t){var n=0,i=e._dataController.store(),o=i&&i.key(),a=Array.isArray(o);if(!t.length)return!1;if(a&&Array.isArray(t[0])&&"and"===t[1]){for(var r=0;r<t.length;r++)if(Array.isArray(t[r])){if(t[r][0]!==o[n]||"="!==t[r][1])return!0;n++}return!1}return o!==t[0]},t=function(e,t){var n=t.component,i=n.getView("rowsView");n.option("renderAsync")&&!n.option("selection.deferred")&&(t.value=n.isRowSelected(t.row.key)),i.renderSelectCheckBoxContainer((0,o.default)(e),t)},n=function(e,t){var n=t.column,i=(0,o.default)(e),a=t.component.getView("columnHeadersView");i.addClass(v),a._renderSelectAllCheckBox(i,n),a._attachSelectAllCheckBoxClickEvent(i)};return{init:function(){this._dataController=this.getController("data"),this._selectionMode=this.option(I),this._isSelectionWithCheckboxes=!1,this._selection=this._createSelection(),this._updateSelectColumn(),this.createAction("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})},_getSelectionConfig:function(){var e=this,t=e._dataController,n=e.option("selection")||{};return{selectedKeys:e.option("selectedRowKeys"),mode:e._selectionMode,deferred:n.deferred,maxFilterLengthInRequest:n.maxFilterLengthInRequest,selectionFilter:e.option("selectionFilter"),key:function(){return t&&t.key()},keyOf:function(e){return t&&t.keyOf(e)},dataFields:function(){return t.dataSource()&&t.dataSource().select()},load:function(e){return t.dataSource()&&t.dataSource().load(e)||(new m.Deferred).resolve([])},plainItems:function(){return t.items(!0)},isItemSelected:function(e){return e.selected},isSelectableItem:function(e){return e&&"data"===e.rowType&&!e.isNewRow},getItemData:function(e){return e&&(e.oldData||e.data||e)},filter:function(){return t.getCombinedFilter()},totalCount:function(){return t.totalCount()},onSelectionChanged:e._updateSelectedItems.bind(this)}},_updateSelectColumn:function(){var e=this.getController("columns"),i=this.isSelectColumnVisible();e.addCommandColumn({type:"selection",command:"select",visible:i,visibleIndex:-1,dataType:"boolean",alignment:"center",cssClass:C,width:"auto",cellTemplate:t,headerCellTemplate:n}),e.columnOption("command:select","visible",i)},_createSelection:function(){
var e=this._getSelectionConfig();return new _.default(e)},_fireSelectionChanged:function(e){e&&this.executeAction("onSelectionChanged",e);var t=this.option("selection.deferred")?{selectionFilter:this.option("selectionFilter")}:{selectedRowKeys:this.option("selectedRowKeys")};this.selectionChanged.fire(t)},_updateCheckboxesState:function(t){var n=t.isDeferredMode,i=t.selectionFilter,o=t.selectedItemKeys,a=t.removedItemKeys;"onClick"===this.option(D)&&((n?i&&e(this,i):o.length>1)?this.startSelectionWithCheckboxes():(n?i&&!i.length:0===o.length&&a.length)&&this.stopSelectionWithCheckboxes())},_updateSelectedItems:function(e){var t,n=this,i=n.option("selection.deferred"),o=n._selection.selectionFilter(),a=n._dataController,r=a.items();if(r){var s=n.isSelectionWithCheckboxes(),l=n.getChangedItemIndexes(r);n._updateCheckboxesState({selectedItemKeys:e.selectedItemKeys,removedItemKeys:e.removedItemKeys,selectionFilter:o,isDeferredMode:i}),(l.length||s!==n.isSelectionWithCheckboxes())&&a.updateItems({changeType:"updateSelection",itemIndexes:l}),i?(n.option("selectionFilter",o),t={}):(e.addedItemKeys.length||e.removedItemKeys.length)&&(n._selectedItemsInternalChange=!0,n.option("selectedRowKeys",e.selectedItemKeys.slice(0)),n._selectedItemsInternalChange=!1,t={selectedRowsData:e.selectedItems.slice(0),selectedRowKeys:e.selectedItemKeys.slice(0),currentSelectedRowKeys:e.addedItemKeys.slice(0),currentDeselectedRowKeys:e.removedItemKeys.slice(0)}),n._fireSelectionChanged(t)}},getChangedItemIndexes:function(e){for(var t=this,n=[],i=this.option("selection.deferred"),o=0,a=e.length;o<a;o++){var r=e[o],s=t.isRowSelected(i?r.data:r.key);t._selection.isDataItem(r)&&r.isSelected!==s&&n.push(o)}return n},callbackNames:function(){return["selectionChanged"]},optionChanged:function(e){var t=this;switch(t.callBase(e),e.name){case"selection":var n=t._selectionMode;t.init();var i=t._selectionMode,o=t.option("selectedRowKeys");n!==i&&("single"===i?o.length>1&&(o=[o[0]]):"multiple"!==i&&(o=[])),t.selectRows(o).always(function(){t._fireSelectionChanged()}),t.getController("columns").updateColumns(),e.handled=!0;break;case"selectionFilter":this._selection.selectionFilter(e.value),e.handled=!0;break;case"selectedRowKeys":var a=e.value||[];!Array.isArray(a)||t._selectedItemsInternalChange||!t.component.getDataSource()&&a.length||t.selectRows(a),e.handled=!0}},publicMethods:function(){return["selectRows","deselectRows","selectRowsByIndexes","getSelectedRowKeys","getSelectedRowsData","clearSelection","selectAll","deselectAll","startSelectionWithCheckboxes","stopSelectionWithCheckboxes","isRowSelected"]},isRowSelected:function(e){return this._selection.isItemSelected(e)},isSelectColumnVisible:function(){return"multiple"===this.option(I)&&("always"===this.option(D)||"onClick"===this.option(D)||this._isSelectionWithCheckboxes)},_isOnePageSelectAll:function(){return"page"===this.option("selection.selectAllMode")},isSelectAll:function(){return this._selection.getSelectAllState(this._isOnePageSelectAll())},selectAll:function(){return"onClick"===this.option(D)&&this.startSelectionWithCheckboxes(),this._selection.selectAll(this._isOnePageSelectAll())},deselectAll:function(){return this._selection.deselectAll(this._isOnePageSelectAll())},clearSelection:function(){return this.selectedItemKeys([])},refresh:function(){var e=this.option("selectedRowKeys")||[];return!this.option("selection.deferred")&&e.length?this.selectedItemKeys(e):(new m.Deferred).resolve().promise()},selectedItemKeys:function(e,t,n,i){return this._selection.selectedItemKeys(e,t,n,i)},getSelectedRowKeys:function(){return this._selection.getSelectedItemKeys()},selectRows:function(e,t){return this.selectedItemKeys(e,t)},deselectRows:function(e){return this.selectedItemKeys(e,!0,!0)},selectRowsByIndexes:function(e){var t=this._dataController.items(),n=[];return Array.isArray(e)||(e=Array.prototype.slice.call(arguments,0)),(0,u.each)(e,function(){var e=t[this];e&&"data"===e.rowType&&n.push(e.key)}),this.selectRows(n)},getSelectedRowsData:function(){return this._selection.getSelectedItems()},changeItemSelection:function(e,t){return t=t||{},this.isSelectionWithCheckboxes()&&(t.control=!0),this._selection.changeItemSelection(this._dataController.getRowIndexDelta()+e,t)},focusedItemIndex:function(e){var t=this;return(0,l.isDefined)(e)?void(t._selection._focusedItemIndex=e):t._selection._focusedItemIndex},isSelectionWithCheckboxes:function(){return"multiple"===this.option(I)&&("always"===this.option(D)||this._isSelectionWithCheckboxes)},startSelectionWithCheckboxes:function(){var e=this;return"multiple"===e.option(I)&&!e.isSelectionWithCheckboxes()&&(e._isSelectionWithCheckboxes=!0,e._updateSelectColumn(),!0)},stopSelectionWithCheckboxes:function(){var e=this;return!!e._isSelectionWithCheckboxes&&(e._isSelectionWithCheckboxes=!1,e._updateSelectColumn(),!0)}}}()),E={defaultOptions:function(){return{selection:{mode:"none",showCheckBoxesMode:"onClick",allowSelectAll:!0,selectAllMode:"allPages",maxFilterLengthInRequest:1500,deferred:!1},selectionFilter:[],selectedRowKeys:[]}},controllers:{selection:O},extenders:{controllers:{data:{init:function(){var e=this.getController("selection"),t=this.option("selection.deferred");this.callBase.apply(this,arguments),t&&e._updateCheckboxesState({isDeferredMode:!0,selectionFilter:this.option("selectionFilter")})},_loadDataSource:function(){var e=this;return e.callBase().done(function(){e.getController("selection").refresh()})},_processDataItem:function(e,t){var n=this,i=n.getController("selection"),o=i.isSelectColumnVisible(),a=t.isDeferredSelection=void 0===t.isDeferredSelection?this.option("selection.deferred"):t.isDeferredSelection,r=this.callBase.apply(this,arguments);if(r.isSelected=i.isRowSelected(a?r.data:r.key),o&&r.values)for(var s=0;s<t.visibleColumns.length;s++)if("select"===t.visibleColumns[s].command){r.values[s]=r.isSelected;break}return r},refresh:function(e){var t=this,n=new m.Deferred;return this.callBase.apply(this,arguments).done(function(){!e||e.selection?t.getController("selection").refresh().done(n.resolve).fail(n.reject):n.resolve()}).fail(n.reject),n.promise()},_handleDataChanged:function(e){this.callBase.apply(this,arguments),e&&"refresh"!==e.changeType||this._repaintChangesOnly||this.getController("selection").focusedItemIndex(-1)},_applyChange:function(e){var t=this;return e&&"updateSelection"===e.changeType?void e.items.forEach(function(e,n){var i=t._items[n];i&&(i.isSelected=e.isSelected,i.values=e.values)}):this.callBase.apply(this,arguments)},_endUpdateCore:function(){var e=this._changes,t=e.length>1&&e.every(function(e){return"updateSelection"===e.changeType});if(t){var n=e.map(function(e){return e.itemIndexes||[]}).reduce(function(e,t){return e.concat(t)});this._changes=[{changeType:"updateSelection",itemIndexes:n}]}this.callBase.apply(this,arguments)},push:function(e){this.callBase.apply(this,arguments);var t=e.filter(function(e){return"remove"===e.type}).map(function(e){return e.key});t.length&&this.getController("selection").deselectRows(t)}},contextMenu:{_contextMenuPrepared:function(e){var t=e.event;t.originalEvent&&"dxhold"!==t.originalEvent.type||e.items&&e.items.length>0||T(this,t)}}},views:{columnHeadersView:{init:function(){var e=this;e.callBase(),e.getController("selection").selectionChanged.add(e._updateSelectAllValue.bind(e))},_updateSelectAllValue:function(){var e=this,t=e.element(),n=t&&t.find("."+x);t&&n.length&&"multiple"===e.option("selection.mode")&&n.dxCheckBox("instance").option({visible:!e.getController("data").isEmpty(),value:e.getController("selection").isSelectAll()})},_handleDataChanged:function(e){this.callBase(e),e&&"refresh"!==e.changeType||this._updateSelectAllValue()},_renderSelectAllCheckBox:function(e,t){var n=this,i=n.getController("selection"),a=n.getController("data").isEmpty(),r=(0,o.default)("<div>").appendTo(e).addClass(x);return n.setAria("label",f.default.format("dxDataGrid-ariaSelectAll"),e),n.getController("editorFactory").createEditor(r,(0,c.extend)({},t,{parentType:"headerRow",dataType:"boolean",value:i.isSelectAll(),editorOptions:{visible:!a&&(n.option("selection.allowSelectAll")||i.isSelectAll()!==!1)},tabIndex:n.option("useLegacyKeyboardNavigation")?-1:n.option("tabIndex")||0,setValue:function(e,t){var o=n.option("selection.allowSelectAll");t.component.option("visible",o||t.component.option("value")!==!1),t.event&&i.isSelectAll()!==e&&(t.value&&!o?t.component.option("value",!1):t.value?i.selectAll():i.deselectAll(),t.event.preventDefault())}})),r},_attachSelectAllCheckBoxClickEvent:function(e){a.default.on(e,h.name,this.createAction(function(e){var t=e.event;(0,o.default)(t.target).closest("."+x).length||a.default.trigger((0,o.default)(t.currentTarget).children("."+x),h.name),t.preventDefault()}))}},rowsView:{renderSelectCheckBoxContainer:function(e,t){"data"!==t.rowType||t.row.isNewRow?s.default.setEmptyText(e):(e.addClass(v),this._attachCheckBoxClickEvent(e),this.setAria("label",f.default.format("dxDataGrid-ariaSelectRow"),e),this._renderSelectCheckBox(e,t))},_renderSelectCheckBox:function(e,t){var n=(0,o.default)("<div>").addClass(x).appendTo(e);return this.getController("editorFactory").createEditor(n,(0,c.extend)({},t.column,{parentType:"dataRow",dataType:"boolean",lookup:null,value:t.value,setValue:function(e,t){var n;"keydown"===(null===t||void 0===t?void 0:null===(n=t.event)||void 0===n?void 0:n.type)&&a.default.trigger(t.element,h.name,t)},row:t.row})),n},_attachCheckBoxClickEvent:function(e){a.default.on(e,h.name,this.createAction(function(e){var t=this.getController("selection"),n=e.event,i=this.getRowIndex((0,o.default)(n.currentTarget).closest("."+y));i>=0&&(t.startSelectionWithCheckboxes(),t.changeItemSelection(i,{shift:n.shiftKey}),(0,o.default)(n.target).closest("."+x).length&&this.getController("data").updateItems({changeType:"updateSelection",itemIndexes:[i]}))}))},_update:function(e){var t=this,n=t.getTableElements();"updateSelection"===e.changeType?n.length>0&&((0,u.each)(n,function(n,i){(0,u.each)(e.itemIndexes||[],function(n,a){var r;if(e.items[a]&&(r=t._getRowElements((0,o.default)(i)).eq(a),r.length)){var s=e.items[a].isSelected;r.toggleClass(b,void 0!==s&&s).find("."+x).dxCheckBox("option","value",s),t.setAria("selected",s,r)}})}),t._updateCheckboxesClass()):t.callBase(e)},_createTable:function(){var e=this,t=e.option("selection.mode"),n=e.callBase.apply(e,arguments);return"none"!==t&&("onLongTap"!==e.option(D)&&d.touch||a.default.on(n,(0,p.addNamespace)(g.default.name,"dxDataGridRowsView"),"."+S,e.createAction(function(t){T(e.component,t.event),t.event.stopPropagation()})),a.default.on(n,"mousedown selectstart",e.createAction(function(e){var t=e.event;t.shiftKey&&t.preventDefault()}))),n},_createRow:function(e){var t=this.callBase(e);if(e){var n=!!e.isSelected;n&&t.addClass(b),this.setAria("selected",n,t)}return t},_rowClick:function(e){var t=this,n=e.event,i=(0,o.default)(n.target).closest("."+k).length;t.isClickableElement((0,o.default)(n.target))||(i||"multiple"===t.option(I)&&"always"===t.option(D)||t.getController("selection").changeItemSelection(e.rowIndex,{control:(0,p.isCommandKeyPressed)(n),shift:n.shiftKey})&&(n.preventDefault(),e.handled=!0),t.callBase(e))},isClickableElement:function(e){var t=e.closest("."+C).length;return!!t},_renderCore:function(e){this.callBase(e),this._updateCheckboxesClass()},_updateCheckboxesClass:function(){var e=this.getTableElements(),t=this.getController("selection"),n=t.isSelectColumnVisible()&&!t.isSelectionWithCheckboxes();(0,u.each)(e,function(e,t){(0,o.default)(t).toggleClass(w,n)})}}}}};t.default=E,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(44)),s=n(197),l=i(n(10)),u=n(36),c=i(n(38)),d=i(n(167)),h=i(n(13)),f=i(n(48)),p=n(1),g=n(0),_=n(3),m="column-chooser",v="column-chooser-button",y="notouch-action",b="column-chooser-list",x="column-chooser-plain",w="column-chooser-mode-drag",C="column-chooser-mode-select",k="column-chooser",S="dx-column-chooser-item",D=".dx-treeview-node",I=".dx-checkbox",T=300,O=function(e,t){var n=[],i="select"===e.option("columnChooser.mode");return t.length&&(0,_.each)(t,function(e,t){var o={text:t.caption,cssClass:t.cssClass,allowHiding:t.allowHiding,expanded:!0,id:t.index,disabled:!1,disableCheckBox:t.allowHiding===!1,parentId:(0,p.isDefined)(t.ownerBand)?t.ownerBand:null};i&&(o.selected=t.visible),n.push(o)}),n},E=r.default.ViewController.inherit({renderShowColumnChooserButton:function(e){var t,n=this,i=n.addWidgetPrefix(v),a=n.option("columnChooser.enabled"),r=e.find("."+i);a?r.length?r.show():(t=(0,o.default)("<div>").addClass(i).appendTo(e),n._createComponent(t,c.default,{icon:k,onClick:function(){n.getView("columnChooserView").showColumnChooser()},hint:n.option("columnChooser.title"),integrationOptions:{}})):r.hide()},getPosition:function(){var e=this.getView("rowsView");return{my:"right bottom",at:"right bottom",of:e&&e.element(),collision:"fit",offset:"-2 -2",boundaryOffset:"2 2"}}}),A=s.ColumnsView.inherit({_resizeCore:a.noop,_isWinDevice:function(){return!!h.default.real().win},_updateList:function(e){var t,n=this._popupContainer.$content(),i="select"===this.option("columnChooser.mode"),o=this._columnChooserList,a=this._columnsController.getChooserColumns(i);if(i&&o&&e&&"selection"===e.changeType){t=O(this,a);for(var r=0;r<t.length;r++){var s=t[r].selected,l=t[r].id;l===e.columnIndex&&(s?o.selectItem(l,s):o.unselectItem(l,s))}}else i&&o&&"full"!==e||(this._popupContainer._wrapper().toggleClass(this.addWidgetPrefix(w),!i).toggleClass(this.addWidgetPrefix(C),i),t=O(this,a),this._renderTreeView(n,t))},_initializePopupContainer:function(){var e=this,t=e.element().addClass(e.addWidgetPrefix(m)),n=e.option("columnChooser"),i=(0,u.current)(),a=(0,u.isGeneric)(i),r=(0,u.isMaterial)(i),s={visible:!1,shading:!1,showCloseButton:!1,dragEnabled:!0,resizeEnabled:!0,toolbarItems:[{text:n.title,toolbar:"top",location:a||r?"before":"center"}],position:e.getController("columnChooser").getPosition(),width:n.width,height:n.height,rtlEnabled:e.option("rtlEnabled"),onHidden:function(){e._isWinDevice()&&(0,o.default)("body").removeClass(e.addWidgetPrefix(y))},container:n.container};a||r?(0,g.extend)(s,{showCloseButton:!0}):s.toolbarItems[s.toolbarItems.length]={shortcut:"cancel"},(0,p.isDefined)(this._popupContainer)?this._popupContainer.option(s):(e._popupContainer=e._createComponent(t,f.default,s),e._popupContainer.on("optionChanged",function(t){"visible"===t.name&&e.renderCompleted.fire()}))},_renderCore:function(e){this._popupContainer&&this._updateList(e)},_renderTreeView:function(e,t){var n=this,i=this.option("columnChooser"),r="select"===i.mode,s={items:t,dataStructure:"plain",activeStateEnabled:!0,focusStateEnabled:!0,hoverStateEnabled:!0,itemTemplate:"item",showCheckBoxesMode:"none",rootValue:null,searchEnabled:i.allowSearch,searchTimeout:i.searchTimeout,onItemRendered:function(e){if(e.itemData.disableCheckBox){var t,n=(0,o.default)(e.itemElement).closest(D);if(n.length&&(t=n.find(I),t.length)){var i=t.data("dxCheckBox");i&&i.option("disabled",!0)}}}},l=e.find(".dx-scrollable").data("dxScrollable"),u=l&&l.scrollTop();r&&!this._columnsController.isBandColumnsUsed()&&e.addClass(this.addWidgetPrefix(x)),s.onContentReady=function(e){(0,a.deferUpdate)(function(){if(u){var t=(0,o.default)(e.element).find(".dx-scrollable").data("dxScrollable");t&&t.scrollTo({y:u})}n.renderCompleted.fire()})},this._isWinDevice()&&(s.useNativeScrolling=!1),(0,g.extend)(s,r?this._prepareSelectModeConfig():this._prepareDragModeConfig()),this._columnChooserList?(s.searchEnabled||(s.searchValue=""),this._columnChooserList.option(s)):(this._columnChooserList=this._createComponent(e,d.default,s),e.addClass(this.addWidgetPrefix(b)))},_prepareDragModeConfig:function(){var e=this.option("columnChooser");return{noDataText:e.emptyPanelText,activeStateEnabled:!1,focusStateEnabled:!1,hoverStateEnabled:!1,itemTemplate:function(e,t,n){(0,o.default)(n).text(e.text).parent().addClass(e.cssClass).addClass(S)}}},_prepareSelectModeConfig:function(){var e=this,t=function(t){var n=e._columnsController.getVisibleColumns().filter(function(e){return!e.command}),i=1===n.length&&!t.itemData.selected;i?t.component.selectItem(t.itemElement):setTimeout(function(){e._columnsController.columnOption(t.itemData.id,"visible",t.itemData.selected)},T)};return{selectNodesRecursive:!1,showCheckBoxesMode:"normal",onItemSelectionChanged:t}},_columnOptionChanged:function(e){var t=e.changeTypes,n=e.optionNames,i="select"===this.option("columnChooser.mode");if(this.callBase(e),i){var o=n.visible&&1===n.length&&void 0!==e.columnIndex,a=n.showInColumnChooser||n.caption||n.visible||t.columns&&n.all;o?this.render(null,{changeType:"selection",columnIndex:e.columnIndex}):a&&this.render(null,"full")}},optionChanged:function(e){switch(e.name){case"columnChooser":this._initializePopupContainer(),this.render(null,"full");break;default:this.callBase(e)}},getColumnElements:function(){var e,t=[],n="select"===this.option("columnChooser.mode"),i=this._columnsController.getChooserColumns(n),a=this._popupContainer&&this._popupContainer.$content(),r=a&&a.find(".dx-treeview-node");return r&&i.forEach(function(n){e=r.filter("[data-item-id = '"+n.index+"']");var i=e.length?e.children("."+S).get(0):null;t.push(i)}),(0,o.default)(t)},getName:function(){return"columnChooser"},getColumns:function(){return this._columnsController.getChooserColumns()},allowDragging:function(e,t){var n=e&&e.allowHiding&&("columnChooser"!==t||!e.visible&&this._columnsController.isParentColumnVisible(e.index));return this.isColumnChooserVisible()&&n},getBoundingRect:function(){var e=this,t=e._popupContainer&&e._popupContainer._container();if(t&&t.is(":visible")){var n=t.offset();return{left:n.left,top:n.top,right:n.left+t.outerWidth(),bottom:n.top+t.outerHeight()}}return null},showColumnChooser:function(){this._popupContainer||(this._initializePopupContainer(),this.render()),this._popupContainer.show(),this._isWinDevice()&&(0,o.default)("body").addClass(this.addWidgetPrefix(y))},hideColumnChooser:function(){this._popupContainer&&this._popupContainer.hide()},isColumnChooserVisible:function(){var e=this._popupContainer;return e&&e.option("visible")},publicMethods:function(){return["showColumnChooser","hideColumnChooser"]}}),P={defaultOptions:function(){return{columnChooser:{enabled:!1,allowSearch:!1,searchTimeout:500,mode:"dragAndDrop",width:250,height:260,title:l.default.format("dxDataGrid-columnChooserTitle"),emptyPanelText:l.default.format("dxDataGrid-columnChooserEmptyText"),container:void 0}}},controllers:{columnChooser:E},views:{columnChooserView:A},extenders:{views:{headerPanel:{_getToolbarItems:function(){var e=this.callBase();return this._appendColumnChooserItem(e)},_appendColumnChooserItem:function(e){var t=this,n=t.option("columnChooser.enabled");if(n){var i=function(){t.component.getView("columnChooserView").showColumnChooser()},a=function(e){(0,o.default)(e.element).addClass(t._getToolbarButtonClass(t.addWidgetPrefix(v)))},r=t.option("columnChooser.title"),s={widget:"dxButton",options:{icon:k,onClick:i,hint:r,text:r,onInitialized:a},showText:"inMenu",location:"after",name:"columnChooserButton",locateInMenu:"auto",sortIndex:40};e.push(s)}return e},optionChanged:function(e){switch(e.name){case"columnChooser":this._invalidate(),e.handled=!0;break;default:this.callBase(e)}},isVisible:function(){var e=this,t=e.option("columnChooser.enabled");return e.callBase()||t}}},controllers:{columns:{allowMoveColumn:function(e,t,n,i){var o=this.option("columnChooser.mode"),a="select"===o&&"columnChooser"===i;return!a&&this.callBase(e,t,n,i)}}}}};t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(9)),r=i(n(287)),s=n(0),l=i(n(704)),u="dx-treeview",c="".concat(u,"-node-container"),d=l.default.inherit(r.default).inherit({_addWidgetPrefix:function(e){return"".concat(u,"-").concat(e)},_optionChanged:function(e){switch(e.name){case"searchValue":this._showCheckboxes()&&this._isRecursiveSelection()&&this._removeSelection(),this._initDataAdapter(),this._updateSearch(),this._repaintContainer();break;case"searchExpr":this._initDataAdapter(),this.repaint();break;case"searchMode":this.option("expandNodesRecursive")?this._updateDataAdapter():this._initDataAdapter(),this.repaint();break;default:this.callBase(e)}},_updateDataAdapter:function(){this._setOptionWithoutOptionChange("expandNodesRecursive",!1),this._initDataAdapter(),this._setOptionWithoutOptionChange("expandNodesRecursive",!0)},_getDataAdapterOptions:function(){return(0,s.extend)(this.callBase(),{searchValue:this.option("searchValue"),searchMode:this.option("searchMode")||"contains",searchExpr:this.option("searchExpr")})},_updateSearch:function(){if(this._searchEditor){var e=this._getSearchEditorOptions();this._searchEditor.option(e)}},_repaintContainer:function(){var e,t=this.$element().find(".".concat(c)).first();t.length&&(t.empty(),e=this._dataAdapter.getRootNodes(),this._renderEmptyMessage(e),this._renderItems(t,e),this._fireContentReadyAction())},_itemContainer:function(e){return this._scrollableContainer&&e?(0,o.default)(this._scrollableContainer.content()):this.callBase()},_addWidgetClass:function(){this.$element().addClass(this._widgetClass())},_clean:function(){this.callBase(),this._removeSearchBox()}});(0,a.default)("dxTreeView",d);var h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){for(var n=l.default.normalizeSortingInfo(t.group),i=[],o=0;o<e.length;o++){for(var a=[],r=0;r<=o;r++){var u=n[r].selector;if(o!==r||null!==e[r]&&e[r]!==!1&&e[r]!==!0){var c=[u,o===r?n[r].desc?">":"<":"=",e[r]];"<"===c[1]?a.push([c,"or",[u,"=",null]]):a.push(c)}else e[r]===!1?a.push([u,"=",!!n[r].desc||null]):(e[r]?!n[r].desc:n[r].desc)?a.push([u,"<>",e[r]]):(a.push([u,"<>",null]),a.push([u,"=",null]))}i.push(s.default.combineFilters(a))}return i=s.default.combineFilters(i,"or"),s.default.combineFilters([i,t.filter])}t.createOffsetFilter=o,t.GroupingHelper=void 0;var a=i(n(2)),r=i(n(15)),s=i(n(26)),l=i(n(41)),u=n(6),c=r.default.inherit(function(){var e=function(e,t){var n=e.hash;return n&&n[JSON.stringify(t)]},t=function(e,t){var n=0,i=e.length-1;if(!e.length)return 0;do{var o=i+n>>1;e[o].offset>t?i=o:n=o}while(i-n>1);var a;for(a=n;a<=i&&!(e[a].offset>t);a++);return a},n=function e(t,n,i){for(var o=0;o<t.length;o++)n+1>=i?t[o].children=[]:e(t[o].children,n+1,i)},i=function e(t,n,i){var o=0;if(n)if(i)for(var a=0;a<n.length;a++)t.isGroupItemCountable(n[a])&&o++,o+=e(t,n[a].items,i-1);else o=n.length;return o};return{ctor:function(e){this._dataSource=e,this.reset()},reset:function(){this._groupsInfo=[],this._totalCountCorrection=0},totalCountCorrection:function(){return this._totalCountCorrection},updateTotalItemsCount:function(e){this._totalCountCorrection=e||0},isGroupItemCountable:function(e){return!this._isVirtualPaging()||!e.isContinuation},_isVirtualPaging:function(){var e=this._dataSource.option("scrolling.mode");return"virtual"===e||"infinite"===e},itemsCount:function e(){var t=this._dataSource,n=t._dataSource,o=s.default.normalizeSortingInfo(n.group()||[]).length,e=i(this,n.items(),o);return e},foreachGroups:function(e,t,n,i,o){function r(e,t,l,c){function d(e,t,n,i){var o=t&&e(t,n);return o&&i.push(o),o}for(var h=[],f=0;f<e.length;f++){if(c.push(e[f].data),!l&&d(t,e[f].data,c,h)===!1)return!1;if(!e[f].data||e[f].data.isExpanded||n){var p=e[f].children,g=p.length&&r(p,t,l,c);if(g&&h.push(g),g===!1)return!1}if(l&&d(t,e[f].data,c,h)===!1)return!1;e[f].data&&e[f].data.offset===e[f].offset||(i=!0),c.pop()}var _=o&&c.slice(0);return i&&u.when.apply(a.default,h).always(function(){s._updateGroupInfoOffsets(e,_)})}var s=this;return r(s._groupsInfo,e,t,[])},_updateGroupInfoOffsets:function(e,t){t=t||[];for(var n=0;n<e.length;n++){var i=e[n];if(i.data&&i.data.offset!==i.offset){i.offset=i.data.offset;for(var o=0;o<t.length;o++)t[o].offset=i.offset}}e.sort(function(e,t){return e.offset-t.offset})},findGroupInfo:function(t){for(var n,i=this,o=i._groupsInfo,a=0;o&&a<t.length;a++)n=e(o,t[a]),o=n&&n.children;return n&&n.data},addGroupInfo:function(n){for(var i,o=this,a=n.path,r=o._groupsInfo,s=0;s<a.length;s++){if(i=e(r,a[s]),!i){i={key:a[s],offset:n.offset,data:{offset:n.offset,isExpanded:!0,path:a.slice(0,s+1)},children:[]};var l=t(r,n.offset);r.splice(l,0,i),r.hash=r.hash||{},r.hash[JSON.stringify(i.key)]=i}s===a.length-1&&(i.data=n,i.offset!==n.offset&&o._updateGroupInfoOffsets(r)),r=i.children}},allowCollapseAll:function(){return!0},refresh:function(e){var t=this,i=e.storeLoadOptions,o=l.default.normalizeSortingInfo(i.group||[]),a="_group"in t?l.default.normalizeSortingInfo(t._group||[]):o,r=Math.min(a.length,o.length);t._group=i.group;for(var s=0;s<r;s++)if(a[s].selector!==o[s].selector){r=s;break}r?n(t._groupsInfo,0,r):t.reset()},handleDataLoading:function(){},handleDataLoaded:function(e,t){t(e)},handleDataLoadedCore:function(e,t){t(e)}}}());t.GroupingHelper=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(37)),r=n(4),s=n(3),l=n(1),u=n(6),c="dx-master-detail-cell",d="dx-master-detail-row",h="dx-cell-focus-disabled",f="dx-row-lines",p={defaultOptions:function(){return{masterDetail:{enabled:!1,autoExpandAll:!1,template:null}}},extenders:{controllers:{columns:{_getExpandColumnsCore:function(){var e=this.callBase();return this.option("masterDetail.enabled")&&e.push({type:"detailExpand",cellTemplate:a.default.getExpandCellTemplate()}),e}},data:function(){var e=function(e){e._expandedItems=[],e._isExpandAll=e.option("masterDetail.autoExpandAll")};return{init:function(){var t=this;e(t),t.callBase()},expandAll:function(e){var t=this;e<0?(t._isExpandAll=!0,t._expandedItems=[],t.updateItems()):t.callBase.apply(t,arguments)},collapseAll:function(e){var t=this;e<0?(t._isExpandAll=!1,t._expandedItems=[],t.updateItems()):t.callBase.apply(t,arguments)},isRowExpanded:function(e){var t=this,n=a.default.getIndexByKey(e,t._expandedItems);return Array.isArray(e)?t.callBase.apply(t,arguments):!!(t._isExpandAll^(n>=0&&t._expandedItems[n].visible))},_getRowIndicesForExpand:function(e){var t=this.getRowIndexByKey(e);return[t,t+1]},_changeRowExpandCore:function(e){var t,n=this;if(Array.isArray(e))t=n.callBase.apply(n,arguments);else{var i=a.default.getIndexByKey(e,n._expandedItems);if(i>=0){var o=n._expandedItems[i].visible;n._expandedItems[i].visible=!o}else n._expandedItems.push({key:e,visible:!0});n.updateItems({changeType:"update",rowIndices:n._getRowIndicesForExpand(e)}),t=(new u.Deferred).resolve()}return t},_processDataItem:function(e,t){var n=this,i=n.callBase.apply(n,arguments);return i.isExpanded=n.isRowExpanded(i.key),void 0===t.detailColumnIndex&&(t.detailColumnIndex=-1,(0,s.each)(t.visibleColumns,function(e,n){if("expand"===n.command&&!(0,l.isDefined)(n.groupIndex))return t.detailColumnIndex=e,!1})),t.detailColumnIndex>=0&&(i.values[t.detailColumnIndex]=i.isExpanded),i},_processItems:function(e,t){var n=this,i=t.changeType,o=[];return e=n.callBase.apply(n,arguments),"loadingAll"===i?e:("refresh"===i&&(n._expandedItems=(0,r.grep)(n._expandedItems,function(e){return e.visible})),(0,s.each)(e,function(e,t){o.push(t);var i=a.default.getIndexByKey(t.key,n._expandedItems);"data"===t.rowType&&(t.isExpanded||i>=0)&&!t.isNewRow&&o.push({visible:t.isExpanded,rowType:"detail",key:t.key,data:t.data,values:[]})}),o)},optionChanged:function(t){var n,i,o=this;if("masterDetail"===t.name){switch(t.name="dataSource",t.fullName){case"masterDetail":var a=t.value||{},r=t.previousValue||{};n=a.enabled!==r.enabled,i=a.autoExpandAll!==r.autoExpandAll;break;case"masterDetail.enabled":n=!0;break;case"masterDetail.autoExpandAll":i=!0}(n||i)&&e(o)}o.callBase(t)}}}(),resizing:{fireContentReadyAction:function(){this.callBase.apply(this,arguments),this._updateParentDataGrids(this.component.$element())},_updateParentDataGrids:function(e){var t=this,n=e.closest("."+d);n.length&&(0,u.when)(this._updateMasterDataGrid(n,e)).done(function(){t._updateParentDataGrids(n.parent())})},_updateMasterDataGrid:function(e,t){var n=(0,o.default)(e).data("options"),i=(0,o.default)(e).closest("."+this.getWidgetContainerClass()).parent().data("dxDataGrid");if(n&&i){if(!i.getView("rowsView").isFixedColumns()){var a=i.getScrollable();return a&&a.update()}this._updateFixedMasterDetailGrids(i,n.rowIndex,t)}},_updateFixedMasterDetailGrids:function(e,t,n){var i=this,a=(0,o.default)(e.getRowElement(t));if(a&&2===a.length&&a.eq(0).height()!==a.eq(1).height()){var r=n.width();return e.updateDimensions().done(function(){var t=i.option("columnAutoWidth")&&e.option("scrolling.useNative")===!0,o=t&&r!==n.width();t&&o&&i.updateDimensions()})}},_toggleBestFitMode:function(e){if(this.callBase.apply(this,arguments),this.option("masterDetail.template")){var t=this._rowsView._getTableElement();t&&t.find(".dx-master-detail-cell").toggleClass("dx-hidden",e)}}}},views:{rowsView:function(){return{_getCellTemplate:function(e){var t,n=this,i=e.column,o=n.getController("editing"),a=o&&o.isEditRow(e.rowIndex);return t="detail"!==i.command||a?n.callBase.apply(n,arguments):n.option("masterDetail.template")||{allowRenderToDetachedContainer:!1,render:n._getDefaultTemplate(i)}},_isDetailRow:function(e){return e&&e.rowType&&0===e.rowType.indexOf("detail")},_createRow:function(e){var t=this.callBase(e);return e&&this._isDetailRow(e)&&(this.option("showRowLines")&&t.addClass(f),t.addClass(d),(0,l.isDefined)(e.visible)&&t.toggle(e.visible)),t},_renderCells:function(e,t){var n,i=t.row,o=this._columnsController.getVisibleColumns();i.rowType&&this._isDetailRow(i)?this._needRenderCell(0,t.columnIndices)&&(n=this._renderCell(e,{value:null,row:i,rowIndex:i.rowIndex,column:{command:"detail"},columnIndex:0}),n.addClass(h).addClass(c).attr("colSpan",o.length)):this.callBase.apply(this,arguments)}}}()}}};t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=i(n(44)),l=n(21),u=i(n(25)),c=i(n(84)),d=n(8),h=i(n(23)),f=n(0),p=n(24),g=i(n(481)),_=i(n(37)),m="dx-editor-inline-block",v="dx-cell-focus-disabled",y="focus-overlay",b="content",x="dx-focused",w="dx-row",C="dxDataGridEditorFactory",k=(0,d.addNamespace)([u.default.down,"focusin",l.name].join(" "),C),S="dx-hidden",D=s.default.ViewController.inherit({_getFocusedElement:function(e){var t=this.option("focusedRowEnabled")?"tr[tabindex]:focus":"tr[tabindex]:not(.dx-data-row):focus",n="td[tabindex]:focus, ".concat(t,", input:focus, textarea:focus, .dx-lookup-field:focus, .dx-checkbox:focus, .dx-switch:focus, .dx-dropdownbutton:focus");return e.find(n)},_getFocusCellSelector:function(){return".dx-row > td"},_updateFocusCore:function(){var e,t,n=this._$focusedElement,i=this.component&&this.component.$element();return i&&(n=this._getFocusedElement(i),n.length&&(n.hasClass(v)||n.hasClass(w)||(e=n.closest(this._getFocusCellSelector()+", ."+v),t=e.get(0)!==n.get(0)&&e.hasClass(m),n=e),n.length&&!n.hasClass(v)))?void this.focus(n,t):void this.loseFocus()},_updateFocus:function(e){var t=this,n=e&&e.event&&(0,o.default)(e.event.target).hasClass(t.addWidgetPrefix(y));t._isFocusOverlay=t._isFocusOverlay||n,clearTimeout(t._updateFocusTimeoutID),t._updateFocusTimeoutID=setTimeout(function(){delete t._updateFocusTimeoutID,t._isFocusOverlay||t._updateFocusCore(),t._isFocusOverlay=!1})},_updateFocusOverlaySize:function(e,t){e.hide();var n=c.default.calculate(e,(0,f.extend)({collision:"fit"},t));n.h.oversize>0&&e.outerWidth(e.outerWidth()-n.h.oversize),n.v.oversize>0&&e.outerHeight(e.outerHeight()-n.v.oversize),e.show()},callbackNames:function(){return["focused"]},focus:function(e,t){var n=this;return void 0===e?n._$focusedElement:void(e&&(e.is(n._$focusedElement)||n._$focusedElement&&n._$focusedElement.removeClass(x),n._$focusedElement=e,
clearTimeout(n._focusTimeoutID),n._focusTimeoutID=setTimeout(function(){delete n._focusTimeoutID,n.renderFocusOverlay(e,t),e.addClass(x),n.focused.fire(e)})))},refocus:function(){var e=this.focus();this.focus(e)},renderFocusOverlay:function(e,t){var n=this;if(_.default.isElementInCurrentGrid(this,e))if(n._$focusOverlay||(n._$focusOverlay=(0,o.default)("<div>").addClass(n.addWidgetPrefix(y))),t)n._$focusOverlay.addClass(S);else if(e.length){var i=h.default.msie?"left bottom":h.default.mozilla?"right bottom":"left top",a=e.closest("."+n.addWidgetPrefix(b)),r=(0,p.getBoundingRect)(e.get(0));n._$focusOverlay.removeClass(S).appendTo(a).outerWidth(r.right-r.left+1).outerHeight(r.bottom-r.top+1);var s={precise:!0,my:i,at:i,of:e,boundary:a.length&&a};n._updateFocusOverlaySize(n._$focusOverlay,s),c.default.setup(n._$focusOverlay,s),n._$focusOverlay.css("visibility","visible")}},resize:function(){var e=this._$focusedElement;e&&this.focus(e)},loseFocus:function(){this._$focusedElement&&this._$focusedElement.removeClass(x),this._$focusedElement=null,this._$focusOverlay&&this._$focusOverlay.addClass(S)},init:function(){this.createAction("onEditorPreparing",{excludeValidators:["disabled","readOnly"],category:"rendering"}),this.createAction("onEditorPrepared",{excludeValidators:["disabled","readOnly"],category:"rendering"}),this._updateFocusHandler=this._updateFocusHandler||this.createAction(this._updateFocus.bind(this)),r.default.on(a.default.getDocument(),k,this._updateFocusHandler),this._attachContainerEventHandlers()},_attachContainerEventHandlers:function(){var e=this,t=e.component&&e.component.$element();t&&r.default.on(t,(0,d.addNamespace)("keydown",C),function(t){"tab"===(0,d.normalizeKeyName)(t)&&e._updateFocusHandler(t)})},dispose:function(){clearTimeout(this._focusTimeoutID),clearTimeout(this._updateFocusTimeoutID),r.default.off(a.default.getDocument(),k,this._updateFocusHandler)}}).include(g.default),I={defaultOptions:function(){return{}},controllers:{editorFactory:D}};t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(5)),s=n(1),l=i(n(101)),u=n(19),c=i(n(23)),d=n(0),h=i(n(13)),f=n(18),p=n(70),g=n(8);n(104),n(126),n(144),n(127),n(217);var _=l.default.isWrapped,m="checkbox-size",v="dx-editor-inline-block",y=function(){function e(e){if(e.row&&e.row.watch&&"dataRow"===e.parentType){var t=e.editorOptions||{};e.editorOptions=t;var n,i=t.onInitialized;t.onInitialized=function(e){i&&i.apply(this,arguments),n=e.component,n.on("disposing",a)};var o,a=e.row.watch(function(){return o=e.lookup.dataSource(e.row),o&&o.filter},function(){n.option("dataSource",o)},function(t){e.row=t})}}function t(t){var n,o,a,r=t.lookup,l="filterRow"===t.parentType;if(r){n=(0,u.compileGetter)(r.displayExpr),o=r.dataSource,(0,s.isFunction)(o)&&!_(o)&&(o=o(t.row||{}),e(t)),((0,s.isObject)(o)||Array.isArray(o))&&(o=(0,p.normalizeDataSourceOptions)(o),l&&(a=o.postProcess,o.postProcess=function(e){return 0===this.pageIndex()&&(e=e.slice(0),e.unshift(null)),a?a.call(this,e):e}));var c=Boolean(r.allowClearing&&!l);t.editorName="dxSelectBox",t.editorOptions=i({searchEnabled:!0,value:t.value,valueExpr:t.lookup.valueExpr,searchExpr:t.lookup.searchExpr||t.lookup.displayExpr,allowClearing:c,showClearButton:c,displayExpr:function(e){return null===e?t.showAllText:n(e)},dataSource:o,onValueChanged:function(e){var n=[e.value];!l&&n.push(e.component.option("text")),t.setValue.apply(this,n)}},t)}}function n(e){e.editorName="dxCheckBox",e.editorOptions=i({elementAttr:{id:e.id},value:(0,s.isDefined)(e.value)?e.value:void 0,hoverStateEnabled:!e.readOnly,focusStateEnabled:!e.readOnly,activeStateEnabled:!1,onValueChanged:function(t){e.setValue&&e.setValue(t.value,t)}},e)}var i=function(e,t){return(0,d.extend)(e,{readOnly:t.readOnly,placeholder:t.placeholder,inputAttr:{id:t.id},tabIndex:t.tabIndex},t.editorOptions)},l=function(){return c.default.msie||c.default.mozilla||h.default.real().ios},y=function(e){var t={},n=l(),a=e.sharedData||t;return i({placeholder:e.placeholder,width:e.width,value:e.value,onValueChanged:function(n){var i="filterRow"===e.parentType||"searchPanel"===e.parentType,o=n.event&&("input"===n.event.type||"keyup"===n.event.type),r=function(t,n){e&&e.setValue(t.value,n)};clearTimeout(t.valueChangeTimeout),o&&i?a.valueChangeTimeout=t.valueChangeTimeout=setTimeout(function(){r(n,t.valueChangeTimeout!==a.valueChangeTimeout)},(0,s.isDefined)(e.updateValueTimeout)?e.updateValueTimeout:0):r(n)},onKeyDown:function(e){n&&"enter"===(0,g.normalizeKeyName)(e.event)&&r.default.trigger((0,o.default)(e.component._input()),"change")},valueChangeEvent:"change"+("filterRow"===e.parentType?" keyup input":"")},e)},b=function(e){e.editorName="dxDateBox",e.editorOptions=i({value:e.value,onValueChanged:function(t){e.setValue(t.value)},onKeyDown:function(e){l()&&"enter"===(0,g.normalizeKeyName)(e.event)&&(e.component.blur(),e.component.focus())},displayFormat:e.format,type:e.dataType,dateSerializationFormat:null,width:"filterBuilder"===e.parentType?void 0:"auto"},e)},x=function(e){var t=y(e),n="searchPanel"===e.parentType,i=function(e){return(0,s.isDefined)(e)?e.toString():""};e.editorType&&"dxTextBox"!==e.editorType?t.value=e.value:t.value=i(e.value),t.valueChangeEvent+=n?" keyup input search":"",t.mode=t.mode||(n?"search":"text"),e.editorName="dxTextBox",e.editorOptions=t},w=function(e){var t=y(e);t.value=(0,s.isDefined)(e.value)?e.value:null,e.editorName="dxNumberBox",e.editorOptions=t},C=function(e){"filterRow"===e.parentType||"filterBuilder"===e.parentType?t((0,d.extend)(e,{lookup:{displayExpr:function(t){return t===!0?e.trueText||"true":t===!1?e.falseText||"false":void 0},dataSource:[!0,!1]}})):n(e)},k=function(e,t){var n=(0,o.default)(t.editorElement);t.editorName&&t.editorOptions&&n[t.editorName]&&("dxCheckBox"!==t.editorName&&"dxSwitch"!==t.editorName||t.isOnForm||(n.addClass(e.addWidgetPrefix(m)),n.parent().addClass(v)),e._createComponent(n,t.editorName,t.editorOptions),"dxTextBox"===t.editorName&&n.dxTextBox("instance").registerKeyHandler("enter",a.noop),"dxDateBox"===t.editorName&&n.dxDateBox("instance").registerKeyHandler("enter",function(){return!0}),"dxTextArea"===t.editorName&&n.dxTextArea("instance").registerKeyHandler("enter",function(e){"enter"!==(0,g.normalizeKeyName)(e)||e.ctrlKey||e.shiftKey||e.stopPropagation()}))};return{createEditor:function(e,n){if(n.cancel=!1,n.editorElement=(0,f.getPublicElement)(e),(0,s.isDefined)(n.tabIndex)||(n.tabIndex=this.option("tabIndex")),n.lookup)t(n);else switch(n.dataType){case"date":case"datetime":b(n);break;case"boolean":C(n);break;case"number":w(n);break;default:x(n)}var i=n.editorName;this.executeAction("onEditorPreparing",n),n.cancel||("dataRow"===n.parentType&&n.editorType&&i===n.editorName&&(n.editorName=n.editorType),"dataRow"!==n.parentType||n.isOnForm||(0,s.isDefined)(n.editorOptions.showValidationMark)||(n.editorOptions.showValidationMark=!1),k(this,n),this.executeAction("onEditorPrepared",n))}}}(),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function l(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?s(Object(n),!0).forEach(function(t){u(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):s(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function u(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function c(e){return p(e)||f(e)||h(e)||d()}function d(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function h(e,t){if(e){if("string"==typeof e)return g(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?g(e,t):void 0}}function f(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function p(e){if(Array.isArray(e))return g(e)}function g(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.default=void 0;var _=r(n(2)),m=r(n(11)),v=n(7),y=r(n(5)),b=r(n(35)),x=n(1),w=n(3),C=n(0),k=r(n(44)),S=n(21),D=n(142),I=r(n(25)),T=r(n(37)),O=n(90),E=n(8),A=n(280),P=r(n(10)),R=r(n(38)),M=r(n(48)),B=r(n(13)),V=r(n(146)),F=r(n(103)),L=n(6),H=n(4),z=a(n(59)),N=r(n(109)),j="edit-form",W="edit-form-item",G="focus-overlay",q="readonly",$="edit-popup",K="form-buttons-container",U="addrow-button",Y="dx-dropdowneditor-overlay",Z="dx-link",X="dx-editor-cell",Q="dx-selection",J="dx-edit-row",ee="dx-edit-button",te="dx-command-edit",ne=te+"-with-icons",ie="dx-scrollable-container",oe="dx-button",ae="__DX_INSERT_INDEX__",re="dx-row",se="dx-row-removed",le="dx-row-inserted",ue="dx-row-modified",ce="dx-cell-modified",de="dxDataGridEditing",he="dx-data-row",fe="dx-cell-focus-disabled",pe="input:not([type='hidden'])",ge="[tabindex], "+pe,_e="batch",me="row",ve="cell",ye="form",be="popup",xe="insert",we="update",Ce="remove",ke="click",Se=[_e,me,ve,ye,be],De=[me,ye,be],Ie=[_e,ve],Te=[ye,be],Oe=[me,ye],Ee="targetComponent",Ae={save:"dx-link-save",cancel:"dx-link-cancel",edit:"dx-link-edit",undelete:"dx-link-undelete","delete":"dx-link-delete",add:"dx-link-add"},Pe={save:"save",cancel:"revert",edit:"edit",undelete:"revert","delete":"trash",add:"add"},Re={edit:"editRow","delete":"deleteRow",undelete:"undeleteRow",save:"saveEditData",cancel:"cancelEditData",add:"addRowByRowIndex"},Me={add:"allowAdding",edit:"allowUpdating","delete":"allowDeleting"},Be=["edit","save","cancel","delete","undelete"],Ve="editing.popup",Fe="editing.changes",Le="editing.editRowKey",He="editing.editColumnName",ze=function(e){return function(t){var n=t instanceof Error?t:new Error(t&&String(t)||"Unknown error");e.reject(n)}},Ne=function(e){var t=e.option("editing.mode");return Se.indexOf(t)!==-1?t:me},je=function(e){var t=Ne(e);return De.indexOf(t)!==-1},We=function(e,t){return t.isEditing||e&&t.column.allowEditing},Ge=function(e,t){var n=!!t.column.command,i=We(e,t),o=!n&&(i||t.column.showEditorAlways);return"data"===t.rowType&&o},qe=k.default.ViewController.inherit(function(){function e(e){return(0,x.isObject)(e)?e.name:e}var t=function(e){return function(t,n){var i=(0,_.default)("<div>").appendTo(t);e.getController("editorFactory").createEditor(i,(0,C.extend)({},n.column,{value:n.value,setValue:n.setValue,row:n.row,parentType:"dataRow",width:null,readOnly:!n.setValue,isOnForm:n.isOnForm,id:n.id}))}},n=function(e){var t=e.component.option("editing.texts")||{};return{save:t.saveRowChanges,cancel:t.cancelRowChanges,edit:t.editRow,undelete:t.undeleteRow,"delete":t.deleteRow,add:t.addRowToNode}},i=function(t,n){var i=-1;return t.some(function(t,o){if(e(t)===n)return i=o,!0}),i},o=function(e){var t=e.column;return e.isCustomEditorType?e.editorType:t.formItem&&t.formItem.editorType},a=function e(t,n){t.forEach(function(t){t.items||t.tabs?e(t.items||t.tabs,n):n(t)})};return{init:function(){var e=this;if(e._columnsController=e.getController("columns"),e._dataController=e.getController("data"),e._rowsView=e.getView("rowsView"),e._editForm=null,e._updateEditFormDeferred=null,e._lastOperation=null,e._deferreds&&e._deferreds.forEach(function(e){return e.reject("cancel")}),e._deferreds=[],e._dataChangedHandler||(e._dataChangedHandler=e._handleDataChanged.bind(e),e._dataController.changed.add(e._dataChangedHandler)),!e._saveEditorHandler){e.createAction("onInitNewRow",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowInserting",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowInserted",{excludeValidators:["disabled","readOnly"]}),e.createAction("onEditingStart",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowUpdating",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowUpdated",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowRemoving",{excludeValidators:["disabled","readOnly"]}),e.createAction("onRowRemoved",{excludeValidators:["disabled","readOnly"]}),e.createAction("onSaved",{excludeValidators:["disabled","readOnly"]}),e.createAction("onSaving",{excludeValidators:["disabled","readOnly"]}),e.createAction("onEditCanceling",{excludeValidators:["disabled","readOnly"]}),e.createAction("onEditCanceled",{excludeValidators:["disabled","readOnly"]});var t,n;e._pointerUpEditorHandler=function(){var t;n=null===(t=e.getController("columnsResizer"))||void 0===t?void 0:t.isResizing()},e._pointerDownEditorHandler=function(e){return t=(0,_.default)(e.target)},e._saveEditorHandler=e.createAction(function(i){function o(e){return e&&!!e.closest(".".concat(Y)).length}var a=i.event,r=(0,_.default)(a.target),s=a[Ee];if((!t||!t.is("input")||t.is(r))&&!je(e)&&!e._editCellInProgress){var l=o(r)||o(null===s||void 0===s?void 0:s.$element()),u=!!r.closest((0,v.getWindow)().document).length,c=s&&!s._disposed&&s!==e.component,d=!!r.closest(".".concat(e.addWidgetPrefix(U))).length,h=r.hasClass(e.addWidgetPrefix(G)),f=Ne(e)===ve;n||l||h||d&&f&&e.isEditing()||!u&&!c||e._closeEditItem.bind(e)(r)}}),y.default.on(m.default.getDocument(),I.default.up,e._pointerUpEditorHandler),y.default.on(m.default.getDocument(),I.default.down,e._pointerDownEditorHandler),y.default.on(m.default.getDocument(),S.name,e._saveEditorHandler)}e._updateEditColumn(),e._updateEditButtons(),this._internalState||(this._internalState=[]),this.component._optionsByReference[Le]=!0,this.component._optionsByReference[Fe]=!0},getChanges:function(){return this.option(Fe)},resetChanges:function(){var e=this.getChanges(),t=null===e||void 0===e?void 0:e.length;t&&this._silentOption(Fe,[])},_getInternalData:function(e){return this._internalState.filter(function(t){return t.key===e})[0]},_addInternalData:function(e){var t=this._getInternalData(e.key);return t?(0,C.extend)(t,e):(this._internalState.push(e),e)},_getOldData:function(e){var t;return null===(t=this._getInternalData(e))||void 0===t?void 0:t.oldData},getUpdatedData:function(e){var t=this._dataController.keyOf(e),n=this.getChanges(),i=T.default.getIndexByKey(t,n);return n[i]?(0,O.createObjectWithChanges)(e,n[i].data):e},getInsertedData:function(){return this.getChanges().filter(function(e){return e.data&&e.type===xe}).map(function(e){return e.data})},getRemovedData:function(){var e=this;return this.getChanges().filter(function(t){return e._getOldData(t.key)&&t.type===Ce}).map(function(t){return e._getOldData(t.key)})},_fireDataErrorOccurred:function(e){if("cancel"!==e){var t=this.getPopupContent();this._dataController.dataErrorOccurred.fire(e,t)}},_needToCloseEditableCell:function(e){var t=this.component.$element(),n=this.isEditing(),i=!t||!!e.closest(t).length;if(i){var o=e.closest("."+he).length;if(o){var a=this.getView("rowsView"),r=e.closest("."+re+"> td"),s=a.getRowIndex(r.parent()),l=a.getCellElements(s).index(r),u=this._columnsController.getVisibleColumns(),c=u[l]&&u[l].allowEditing;n=n&&!c&&!this.isEditCell(s,l)}}return n},_closeEditItem:function(e){this._needToCloseEditableCell(e)&&this.closeEditCell()},_handleDataChanged:function(e){var t=this._editForm;"refresh"===e.changeType&&Ne(this)===be&&t&&t.option("visible")&&this._repaintEditPopup()},_isDefaultButtonVisible:function(e,t){var n=!0,i=je(this),o=Ne(this)===_e,a=t.row&&t.row.rowIndex===this._getVisibleEditRowIndex()&&i;switch(e.name){case"edit":n=!a&&this.allowUpdating(t)&&i;break;case"save":case"cancel":n=a;break;case"delete":n=!a&&this.allowDeleting(t)&&(!o||!t.row.removed);break;case"undelete":n=o&&this.allowDeleting(t)&&t.row.removed}return n},_isButtonVisible:function(e,t){var n=e.visible;return(0,x.isDefined)(n)?(0,x.isFunction)(n)?n.call(e,{component:t.component,row:t.row,column:t.column}):n:this._isDefaultButtonVisible(e,t)},_getButtonConfig:function(t,i){var o=this,a=(0,x.isObject)(t)?t:{},r=e(t),s=n(i),l=Re[r],u=this.option("editing"),c=Me[r],d=!c||u[c];return(0,C.extend)({name:r,text:s[r],cssClass:Ae[r],onClick:function(e){var t=e.event;t.stopPropagation(),t.preventDefault(),setTimeout(function(){i.row&&d&&o[l]&&o[l](i.row.rowIndex)})}},a)},_getEditingButtons:function(e){var t,n=this,o=!!e.column.buttons,a=(e.column.buttons||[]).slice();return o?(t=i(a,"edit"),t>=0&&(i(a,"save")<0&&a.splice(t+1,0,"save"),i(a,"cancel")<0&&a.splice(i(a,"save")+1,0,"cancel")),t=i(a,"delete"),t>=0&&i(a,"undelete")<0&&a.splice(t+1,0,"undelete")):a=Be.slice(),a.map(function(t){return n._getButtonConfig(t,e)})},_renderEditingButtons:function(e,t,n){var i=this;t.forEach(function(t){i._isButtonVisible(t,n)&&i._createButton(e,t,n)})},_getEditCommandCellTemplate:function(){var e=this;return function(t,n){var i=(0,_.default)(t);if("data"===n.rowType){var o=e._getEditingButtons(n);e._renderEditingButtons(i,o,n),n.watch&&n.watch(function(){return o.map(function(t){return e._isButtonVisible(t,n)})},function(){i.empty(),e._renderEditingButtons(i,o,n)})}else T.default.setEmptyText(i)}},isRowEditMode:function(){return je(this)},isFormEditMode:function(){var e=Ne(this);return Te.indexOf(e)!==-1},isCellOrBatchEditMode:function(){var e=this.getEditMode();return Ie.indexOf(e)!==-1},getEditMode:function(){return Ne(this)},getFirstEditableColumnIndex:function(){var e,t=this.getController("columns"),n=this._firstFormItem;if(Ne(this)===ye&&n){var i=this.option(Le),o=this._dataController.getRowIndexByKey(i),a=this._rowsView.getCellElements(o);e=this._rowsView._getEditFormEditorVisibleIndex(a,n.column)}else{var r=t.getVisibleColumns();(0,w.each)(r,function(t,n){if(n.allowEditing)return e=t,!1})}return e},getFirstEditableCellInRow:function(e){var t=this.getView("rowsView");return t&&t._getCellElement(e?e:0,this.getFirstEditableColumnIndex())},getFocusedCellInRow:function(e){return this.getFirstEditableCellInRow(e)},getIndexByKey:function(e,t){return T.default.getIndexByKey(e,t)},hasChanges:function(e){for(var t=this.getChanges(),n=!1,i=0;i<(null===t||void 0===t?void 0:t.length);i++)if(t[i].type&&(!(0,x.isDefined)(e)||this._dataController.getRowIndexByKey(t[i].key)===e)){n=!0;break}return n},dispose:function(){this.callBase(),clearTimeout(this._inputFocusTimeoutID),y.default.off(m.default.getDocument(),I.default.up,this._pointerUpEditorHandler),y.default.off(m.default.getDocument(),I.default.down,this._pointerDownEditorHandler),y.default.off(m.default.getDocument(),S.name,this._saveEditorHandler)},optionChanged:function(e){if("editing"===e.name){var t=e.fullName,n=this._editPopup;if(t&&0===t.indexOf(Ve)){if(n){var i=t.slice(Ve.length+1);i?n.option(i,e.value):n.option(e.value)}}else n&&n.option("visible")&&0===t.indexOf("editing.form")?this._repaintEditPopup():t===Le?this._handleEditRowKeyChange(e):t===He?this._handleEditColumnNameChange(e):t===Fe?this._handleChangesChange(e):(this.init(),this.resetChanges(),this._resetEditColumnName(),this._resetEditRowKey());e.handled=!0}else this.callBase(e)},_handleEditRowKeyChange:function(e){var t=this._dataController.getRowIndexByKey(e.value),n=this._getEditRowIndexCorrection(),i=this._dataController.getRowIndexByKey(e.previousValue)+n,o=this._getVisibleEditColumnIndex();this.isCellOrBatchEditMode()?(0,x.isDefined)(e.value)&&e.value!==e.previousValue&&this._editCellFromOptionChanged(o,o,i):(0,x.isDefined)(e.value)?e.value!==e.previousValue&&this._editRowFromOptionChanged(t,i):this.cancelEditData()},_handleEditColumnNameChange:function(e){var t=this._getVisibleEditRowIndex(e.previousValue);if(this.isCellOrBatchEditMode()&&t!==-1&&(0,x.isDefined)(e.value)&&e.value!==e.previousValue){var n=this._columnsController.getVisibleColumnIndex(e.value),i=this._columnsController.getVisibleColumnIndex(e.previousValue);this._editCellFromOptionChanged(n,i,t)}},_handleChangesChange:function(e){var t=this._dataController;(e.value.length||e.previousValue.length)&&t.updateItems({repaintChangesOnly:!0})},_editCellFromOptionChanged:function(e,t,n){var i=this,o=this._columnsController.getVisibleColumns();e>-1&&(0,H.deferRender)(function(){i._repaintEditCell(o[e],o[t],n)})},publicMethods:function(){return["addRow","deleteRow","undeleteRow","editRow","editCell","closeEditCell","saveEditData","cancelEditData","hasEditData"]},refresh:function(e){var t=Ne(this),n=t===_e||e&&"virtual"!==this.option("scrolling.mode");(0,x.isDefined)(this._pageIndex)&&(this.isCellOrBatchEditMode()?n&&(this._resetEditColumnName(),this._resetEditRowKey()):(this.resetChanges(),this.init(),(0,x.isDefined)(this.option(Le))&&this._resetEditRowKey()))},isEditing:function(){var e=(0,x.isDefined)(this.option(Le)),t=(0,x.isDefined)(this.option(He));return this.isCellOrBatchEditMode()?e&&t:e},isEditRow:function(e){var t=Ne(this);return this._getVisibleEditRowIndex()===e&&De.indexOf(t)!==-1},_setEditRowKey:function(e,t){t?this._silentOption(Le,e):this.option(Le,e)},_setEditRowKeyByIndex:function(e,t){var n=this._dataController.getKeyByRowIndex(e);return void 0===n?void this._dataController.fireError("E1043"):void this._setEditRowKey(n,t)},getEditRowIndex:function(){return this._getVisibleEditRowIndex()},getEditFormRowIndex:function(){var e=Ne(this);return e===ye||e===be?this._getVisibleEditRowIndex():-1},isEditCell:function(e,t){return this._getVisibleEditRowIndex()===e&&this._getVisibleEditColumnIndex()===t},getPopupContent:function(){var e=Ne(this),t=this._editPopup&&this._editPopup.option("visible");if(e===be&&t)return this._$popupContent},getEditForm:function(){return this._editForm},_needInsertItem:function(e,t){var n=this,i=n._dataController.dataSource(),o=n.option("scrolling.mode"),a=i.pageIndex(),r=i.beginPageIndex?i.beginPageIndex():a,s=i.endPageIndex?i.endPageIndex():a;if("standard"!==o)switch(t){case"append":return e.key.pageIndex===s;case"prepend":return e.key.pageIndex===r;case"refresh":e.key.rowIndex=0,e.key.dataRowIndex=0,e.key.pageIndex=0;break;default:return e.key.pageIndex>=r&&e.key.pageIndex<=s}return e.key.pageIndex===a},_generateNewItem:function(e){var t={key:e};return e&&e[ae]&&(t[ae]=e[ae]),t},_getLoadedRowIndexByInsertKey:function(e,t,n){var i=this._dataController,o=i.getRowIndexOffset(!0),a=n.dataRowIndex-o;"append"===t.changeType&&(a-=i.items(!0).length,t.removeCount&&(a+=t.removeCount));for(var r=0;r<a;r++)e[r]&&e[r][ae]&&a++;return a},processItems:function(e,t){var n=this,i=t.changeType;this.update(i);var o=this.getChanges();return o.forEach(function(o){var a=o.key;if((0,x.isDefined)(a)&&o.type===xe){var r=n._getLoadedRowIndexByInsertKey(e,t,a),s=n._generateNewItem(a);r>=0&&n._needInsertItem(o,i,e,s)&&e.splice(a.dataRowIndex?r:0,0,s)}}),e},processDataItem:function(e,t,n){var i=t.visibleColumns,o=e.data[ae]?e.data.key:e.key,a=this.getChanges(),r=T.default.getIndexByKey(o,a);if(e.isEditing=!1,r>=0){var s=Ne(this),l=a[r],u=l.data,c=l.type;switch(c){case xe:s===be&&(e.visible=!1),e.isNewRow=!0,e.key=o,e.data=u;break;case we:e.modified=!0,e.oldData=e.data,e.data=(0,O.createObjectWithChanges)(e.data,u),e.modifiedValues=n(u,i,!0);break;case Ce:s===_e&&(e.data=(0,O.createObjectWithChanges)(e.data,u)),e.removed=!0}}},_initNewRow:function(e){var t=this;if(this.executeAction("onInitNewRow",e),e.promise){var n=new L.Deferred;return(0,L.when)((0,L.fromPromise)(e.promise)).done(n.resolve).fail(ze(n)).fail(function(e){return t._fireDataErrorOccurred(e)}),n}},_getInsertKey:function(e){var t=this,n=t._dataController,i=n.items(),o=Ne(t),a={parentKey:e,pageIndex:n.pageIndex(),rowIndex:t._getInsertRowIndex(e)},r=i[a.rowIndex];return r&&(!r.isEditing&&"detail"===r.rowType||"detailAdaptive"===r.rowType)&&a.rowIndex++,a.dataRowIndex=n.getRowIndexOffset()+i.filter(function(e,t){return t<a.rowIndex&&("data"===e.rowType&&!e.isNewRow||"group"===e.rowType)}).length,o!==_e&&this._setEditRowKey(a,!0),a[ae]=t._getInsertIndex(),a},_getInsertRowIndex:function(e){var t=this,n=t.getView("rowsView"),i=t._dataController.getRowIndexByKey(e);return i>=0?i+1:n?n.getTopVisibleItemIndex(!0):0},_getInsertIndex:function(){var e=0;return this.getChanges().forEach(function(t){t.type===xe&&t.key[ae]>e&&(e=t.key[ae])}),e+1},addRow:function(e){var t=this,n=t._dataController,i=n.store(),o=i&&i.key(),a={data:{}},r=Ne(t),s=t._getVisibleEditRowIndex(),l=new L.Deferred;return i?r===ve&&t.hasChanges()?(t.saveEditData().done(function(){t.hasChanges()?l.reject("cancel"):t.addRow(e).done(l.resolve).fail(l.reject)}),l.promise()):(t.refresh(),t._allowRowAdding()?(o||(a.data.__KEY__=String(new b.default)),(0,L.when)(t._initNewRow(a,e)).done(function(){t._allowRowAdding()?(t._addRowCore(a.data,e,s),l.resolve()):l.reject("cancel")}).fail(l.reject),l.promise()):l.reject("cancel")):(n.fireError("E1052",this.component.NAME),l.reject())},_allowRowAdding:function(){var e=this,t=Ne(e),n=e._getInsertIndex();return!(t!==_e&&n>1)},_addRowCore:function(e,t,n){var i=this,o=i._getVisibleEditRowIndex(),a=i._getInsertKey(t),r=Ne(i);i._addChange({key:a,data:e,type:xe}),i._dataController.updateItems({changeType:"update",rowIndices:[n,o,a.rowIndex]}),r===be?i._showEditPopup(a.rowIndex):i._focusFirstEditableCellInRow(a.rowIndex),i._afterInsertRow({key:a,data:e})},_focusFirstEditableCellInRow:function(e){var t=this,n=t.getFirstEditableCellInRow(e);t._editCellInProgress=!0,t._delayedInputFocus(n,function(){t._editCellInProgress=!1;var n=t.getFirstEditableCellInRow(e),i="dblClick"===t.option("editing.startEditAction")?D.name:S.name;n&&y.default.trigger(n,i)})},_isEditingStart:function(e){return this.executeAction("onEditingStart",e),e.cancel},_beforeEditCell:function(e,t,n){var i=this;if(Ne(i)===ve&&!n.isNewRow&&i.hasChanges()){var o=new L.Deferred;return i.saveEditData().always(function(){o.resolve(i.hasChanges())}),o}},_beforeUpdateItems:function(){},_getVisibleEditColumnIndex:function(){var e=this.option(He);return(0,x.isDefined)(e)?this._columnsController.getVisibleColumnIndex(e):-1},_setEditColumnNameByIndex:function(e,t){var n,i=this._columnsController.getVisibleColumns();this._setEditColumnName(null===(n=i[e])||void 0===n?void 0:n.name,t)},_setEditColumnName:function(e,t){t?this._silentOption(He,e):this.option(He,e)},_resetEditColumnName:function(){this._setEditColumnName(null,!0)},_getEditColumn:function(){var e=this.option(He);return this._getColumnByName(e)},_getColumnByName:function(e){var t,n=this._columnsController.getVisibleColumns();return(0,x.isDefined)(e)&&n.some(function(n){if(n.name===e)return t=n,!0}),t},_getVisibleEditRowIndex:function(e){var t=this._dataController,n=this.option(Le),i=t.getRowIndexByKey(n);return i===-1?i:i+this._getEditRowIndexCorrection(e)},_getEditRowIndexCorrection:function(e){var t=e?this._getColumnByName(e):this._getEditColumn(),n="adaptiveHidden"===(null===t||void 0===t?void 0:t.visibleWidth);return n?1:0},_resetEditRowKey:function(){this._setEditRowKey(null,!0)},_resetEditIndices:function(){this._resetEditColumnName(),this._resetEditRowKey()},editRow:function(e){var t=this._dataController,n=t.items(),i=n[e],o={data:i&&i.data,cancel:!1},a=this._getVisibleEditRowIndex();if(i){if(e===a)return!0;if(void 0===i.key)return void this._dataController.fireError("E1043");i.isNewRow||(o.key=i.key),this._isEditingStart(o)||(this.resetChanges(),this.init(),this._resetEditColumnName(),this._pageIndex=t.pageIndex(),this._addInternalData({key:i.key,oldData:i.data}),this._setEditRowKey(i.key))}},_editRowFromOptionChanged:function(e,t){var n=[t,e],i=Ne(this);this._beforeUpdateItems(n,e,t),i===be?this._showEditPopup(e):(this._needFocusEditor=!0,this._dataController.updateItems({changeType:"update",rowIndices:n}))},_focusEditorIfNeed:function(){var e=this,t=Ne(this);if(this._needFocusEditor){if(Oe.indexOf(t)!==-1){var n=this.getFocusedCellInRow(this._getVisibleEditRowIndex());this._delayedInputFocus(n,function(){n&&e.component.focus(n)})}else if(Ie.indexOf(t)!==-1){var i,o=this._getVisibleEditColumnIndex(),a=null===(i=this._rowsView)||void 0===i?void 0:i._getCellElement(this._getVisibleEditRowIndex(),o);a&&!a.find(":focus").length?this._focusEditingCell(function(){e._editCellInProgress=!1},a,!0):this._editCellInProgress=!1}this._needFocusEditor=!1}},_showEditPopup:function(e,t){var n=this,i="desktop"!==B.default.current().deviceType,o=(0,C.extend)({showTitle:!1,fullScreen:i,toolbarItems:[{toolbar:"bottom",location:"after",widget:"dxButton",options:n._getSaveButtonConfig()},{toolbar:"bottom",location:"after",widget:"dxButton",options:n._getCancelButtonConfig()}],contentTemplate:n._getPopupEditFormTemplate(e)},n.option(Ve));if(!n._editPopup){var a=(0,_.default)("<div>").appendTo(n.component.$element()).addClass(n.addWidgetPrefix($));n._editPopup=n._createComponent(a,M.default,{}),n._editPopup.on("hiding",n._getEditPopupHiddenHandler()),n._editPopup.on("shown",function(e){y.default.trigger(e.component.$content().find(ge).not("."+ie).first(),"focus"),t&&n._editForm&&n._editForm.repaint()})}n._editPopup.option(o),n._editPopup.show()},_repaintEditPopup:function(){var e=this._getVisibleEditRowIndex();if(this._editPopup&&this._editPopup.option("visible")&&e>=0){var t=this._editPopup.option("animation");this._editPopup.option("animation",null),this._showEditPopup(e,!0),this._editPopup.option("animation",t)}},_getEditPopupHiddenHandler:function(){var e=this;return function(t){e.isEditing()&&e.cancelEditData()}},_getPopupEditFormTemplate:function(e){var t=this,n=t.component.getVisibleRows()[e],i={row:n,rowType:n.rowType,key:n.key};return function(e){var n=t.getEditFormTemplate(),o=t._createComponent((0,_.default)("<div>").appendTo(e),N.default);t._$popupContent=o.$content(),n(t._$popupContent,i,!0)}},_getSaveButtonConfig:function(){return{text:this.option("editing.texts.saveRowChanges"),onClick:this.saveEditData.bind(this)}},_getCancelButtonConfig:function(){return{text:this.option("editing.texts.cancelRowChanges"),onClick:this.cancelEditData.bind(this)}},_removeInternalData:function(e){var t=this._getInternalData(e),n=this._internalState.indexOf(t);n>-1&&this._internalState.splice(n,1)},_removeChange:function(e){if(e>=0){var t=c(this.getChanges());this._removeInternalData(t[e].key),t.splice(e,1),this._silentOption(Fe,t)}},executeOperation:function(e,t){var n=this;this._lastOperation&&this._lastOperation.reject(),this._lastOperation=e,this.waitForDeferredOperations().done(function(){"rejected"!==e.state()&&(t(),n._lastOperation=null)}).fail(function(){e.reject(),n._lastOperation=null})},waitForDeferredOperations:function(){return L.when.apply(void 0,c(this._deferreds))},editCell:function(e,t){return this._editCell({rowIndex:e,columnIndex:t})},_editCell:function(e){var t,n=this,i=new L.Deferred;return this.executeOperation(i,function(){t=n._editCellCore(e),(0,L.when)(t).done(i.resolve).fail(i.reject)}),void 0!==t?t:i.promise()},_getNormalizedEditCellOptions:function(e){var t,n=e.oldColumnIndex,i=e.oldRowIndex,o=e.columnIndex,a=e.rowIndex,r=this._columnsController,s=r.getVisibleColumns(),l=this._dataController.items(),u=l[a];t=(0,x.isDefined)(n)?s[n]:this._getEditColumn(),
(0,x.isDefined)(i)||(i=this._getVisibleEditRowIndex()),(0,x.isString)(o)&&(o=r.columnOption(o,"index"),o=r.getVisibleIndex(o));var c=s[o];return{oldColumn:t,columnIndex:o,oldRowIndex:i,rowIndex:a,column:c,item:u}},_editCellCore:function(e){var t=this,n=this._dataController,i=(0,x.isDefined)(e.oldColumnIndex)||(0,x.isDefined)(e.oldRowIndex),o=this._getNormalizedEditCellOptions(e),a=o.columnIndex,r=o.rowIndex,s=o.column,l=o.item,u={data:null===l||void 0===l?void 0:l.data,cancel:!1,column:s};if(void 0===l.key)return void this._dataController.fireError("E1043");if(s&&l&&("data"===l.rowType||"detailAdaptive"===l.rowType)&&!l.removed&&!je(this)){if(!i&&this.isEditCell(r,a))return!0;var c=r+n.getRowIndexOffset();return(0,L.when)(this._beforeEditCell(r,a,l)).done(function(e){e||t._prepareEditCell(u,l,a,c)||t._processCanceledEditingCell()})}return!1},_processCanceledEditingCell:function(){},_prepareEditCell:function(e,t,n,i){return t.isNewRow||(e.key=t.key),!this._isEditingStart(e)&&(this._pageIndex=this._dataController.pageIndex(),this._setEditRowKey(t.key),this._setEditColumnNameByIndex(n),e.column.showEditorAlways||this._addInternalData({key:t.key,oldData:t.data}),!0)},_repaintEditCell:function(e,t,n){var i=this;this._needFocusEditor=!0,!e||!e.showEditorAlways||t&&!t.showEditorAlways?(i._editCellInProgress=!0,i.getController("editorFactory").loseFocus(),i._dataController.updateItems({changeType:"update",rowIndices:[n,i._getVisibleEditRowIndex()]})):i._dataController.updateItems({changeType:"update",rowIndices:[]})},_delayedInputFocus:function(e,t,n){function i(){if(t&&t(),e){var n=e.find(ge).first();T.default.focusAndSelectElement(o,n)}o._beforeFocusCallback=null}var o=this;B.default.real().ios||B.default.real().android?i():(o._beforeFocusCallback&&o._beforeFocusCallback(),clearTimeout(o._inputFocusTimeoutID),n&&(o._beforeFocusCallback=t),o._inputFocusTimeoutID=setTimeout(i))},_focusEditingCell:function(e,t,n){var i=this,o=i.getView("rowsView"),a=this._getVisibleEditColumnIndex();t=t||o&&o._getCellElement(i._getVisibleEditRowIndex(),a),i._delayedInputFocus(t,e,n)},deleteRow:function(e){var t=this;if("cell"===this.option("editing.mode")&&this.isEditing()){var n=this._dataController.items()[e].isNewRow;this.closeEditCell(null,n).always(function(){t._checkAndDeleteRow(e)})}else this._checkAndDeleteRow(e)},_checkAndDeleteRow:function(e){var t=this,n=t.option("editing"),i=n&&n.texts,o=n&&n.mode===_e,a=n&&n.confirmDelete,r=i&&i.confirmDeleteMessage,s=t._dataController.items()[e],l=o||!t.isEditing()||s.isNewRow;if(s&&l)if(!o&&a&&r){var u=i&&i.confirmDeleteTitle,c=(0,x.isDefined)(u)&&u.length>0;(0,A.confirm)(r,u,c).done(function(n){n&&t._deleteRowCore(e)})}else t._deleteRowCore(e)},_deleteRowCore:function(e){var t=this._dataController,n=t.items()[e],i=n&&n.key,o=this._getVisibleEditRowIndex(),a=this.option("editing.mode")===_e;this.refresh();var r=this.getChanges(),s=T.default.getIndexByKey(i,r);s>=0?r[s].type===xe?this._removeChange(s):this._addChange({key:i,type:Ce}):this._addChange({key:i,oldData:n.data,type:Ce}),a?t.updateItems({changeType:"update",rowIndices:[o,e]}):this.saveEditData()},undeleteRow:function(e){var t=this,n=t._dataController,i=n.items()[e],o=t._getVisibleEditRowIndex(),a=i&&i.key,r=this.getChanges();if(i){var s=T.default.getIndexByKey(a,r);if(s>=0){var l=r[s].data;(0,x.isEmptyObject)(l)?t._removeChange(s):t._addChange({key:a,type:we}),n.updateItems({changeType:"update",rowIndices:[o,e]})}}},_fireOnSaving:function(){var e=this,t={cancel:!1,promise:null,changes:c(this.getChanges())};this.executeAction("onSaving",t);var n=new L.Deferred;return(0,L.when)((0,L.fromPromise)(t.promise)).done(function(){n.resolve(t)}).fail(function(t){ze(n),e._fireDataErrorOccurred(t),n.resolve({cancel:!0})}),n},_executeEditingAction:function(e,t,n){if(this.component._disposed)return null;var i=new L.Deferred;return this.executeAction(e,t),(0,L.when)((0,L.fromPromise)(t.cancel)).done(function(e){e?setTimeout(function(){i.resolve("cancel")}):n(t).done(i.resolve).fail(ze(i))}).fail(ze(i)),i},_processChanges:function(e,t,n,i){var o=this,a=this._dataController.store();(0,w.each)(i,function(r,s){var u,c,d=o._getOldData(s.key),h=s.data,f=s.type,p=l({},s);if(!o._beforeSaveEditData(s,r)){switch(f){case Ce:c={data:d,key:s.key,cancel:!1},u=o._executeEditingAction("onRowRemoving",c,function(){return a.remove(s.key).done(function(e){n.push({type:"remove",key:e})})});break;case xe:c={data:h,cancel:!1},u=o._executeEditingAction("onRowInserting",c,function(){return a.insert(c.data).done(function(e,t){(0,x.isDefined)(t)&&(p.key=t),e&&(0,x.isObject)(e)&&e!==c.data&&(p.data=e),n.push({type:"insert",data:e,index:0})})});break;case we:c={newData:h,oldData:d,key:s.key,cancel:!1},u=o._executeEditingAction("onRowUpdating",c,function(){return a.update(s.key,c.newData).done(function(e,t){e&&(0,x.isObject)(e)&&e!==c.newData&&(p.data=e),n.push({type:"update",key:t,data:e})})})}if(i[r]=p,u){var g=new L.Deferred;u.always(function(e){t.push({key:s.key,result:e})}).always(g.resolve),e.push(g.promise())}}})},_processSaveEditDataResult:function(e){for(var t=this,n=!1,i=Ne(t),o=c(this.getChanges()),a=o.length,r=0;r<e.length;r++){var s=e[r].result,l="cancel"===s,u=T.default.getIndexByKey(e[r].key,o),d=o[u],h=s&&s instanceof Error;if(h){if(d&&this._addInternalData({key:d.key,error:s}),t._fireDataErrorOccurred(s),i!==_e){(null===d||void 0===d?void 0:d.type)===Ce&&u>=0&&o.splice(u,1);break}}else l&&d&&(i===_e||d.type!==Ce)||(u>=0&&o.splice(u,1),n=!l)}return o.length<a&&this._silentOption(Fe,o),n},_fireSaveEditDataEvents:function(e){var t=this;(0,w.each)(e,function(e,n){var i=n.data,o=n.key,a=n.type,r=t._addInternalData({key:o}),s={key:o,data:i};switch(r.error&&(s.error=r.error),a){case Ce:t.executeAction("onRowRemoved",(0,C.extend)({},s,{data:r.oldData}));break;case xe:t.executeAction("onRowInserted",s);break;case we:t.executeAction("onRowUpdated",s)}}),this.executeAction("onSaved",{changes:e})},saveEditData:function(){var e=this,t=new L.Deferred;return this.waitForDeferredOperations().done(function(){return e.isSaving()?void e._resolveAfterSave(t):void(0,L.when)(e._beforeSaveEditData()).done(function(n){return n?void e._resolveAfterSave(t,{cancel:n}):(e._saving=!0,void e._saveEditDataInner().done(t.resolve).fail(t.reject).always(function(){e._saving=!1}))}).fail(t.reject)}).fail(t.reject),t.promise()},_resolveAfterSave:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.cancel,i=t.error;(0,L.when)(this._afterSaveEditData(n)).done(function(){e.resolve(i)}).fail(e.reject)},_saveEditDataInner:function(){var e=this,t=[],n=[],i=[],o=this._dataController,a=o.dataSource(),r=new L.Deferred;return(0,L.when)(this._fireOnSaving()).done(function(o){var s=o.cancel,l=o.changes;return s?r.resolve().promise():(e._processChanges(n,t,i,l),n.length?(null===a||void 0===a?void 0:a.beginLoading(),L.when.apply(void 0,n).done(function(){e._processSaveEditDataResult(t)?e._endSaving(i,l,r):(null===a||void 0===a?void 0:a.endLoading(),r.resolve())}).fail(function(e){null===a||void 0===a?void 0:a.endLoading(),r.resolve(e)}),r.always(function(){e._focusEditingCell()}).promise()):void e._cancelSaving(r))}).fail(r.reject),r.promise()},_resetModifiedClassCells:function(e){var t=this,n=Ne(this);if(n===_e){var i=this._columnsController.getVisibleColumns().length;e.forEach(function(e){var n=e.key,o=t._dataController.getRowIndexByKey(n);if(o!==-1)for(var a=0;a<i;a++)t._rowsView._getCellElement(o,a).removeClass(ce)})}},_endSaving:function(e,t,n){var i,o=Ne(this),a=this._dataController.dataSource();o!==ve?(this._resetModifiedClassCells(t),this._resetEditIndices()):"update"!==(null===(i=t[0])||void 0===i?void 0:i.type)&&this._resetEditIndices(),o===be&&this._editPopup&&this._editPopup.hide(),null===a||void 0===a?void 0:a.endLoading(),this._refreshDataAfterSave(e,t,n)},_cancelSaving:function(e){var t=Ne(this),n=this._dataController;je(this)?this.hasChanges()||this._cancelEditDataCore():this.isCellOrBatchEditMode()?(t!==ve&&this._resetEditIndices(),n.updateItems()):this._focusEditingCell(),this.executeAction("onSaved",{changes:[]}),this._resolveAfterSave(e)},_refreshDataAfterSave:function(e,t,n){var i=this,o=this._dataController,a=this.option("editing.refreshMode"),r="reshape"!==a&&"repaint"!==a;r||o.push(e),(0,L.when)(o.refresh({selection:r,reload:r,load:"reshape"===a,changesOnly:this.option("repaintChangesOnly")})).always(function(){i._fireSaveEditDataEvents(t)}).done(function(){i._resolveAfterSave(n)}).fail(function(e){i._resolveAfterSave(n,{error:e})})},isSaving:function(){return this._saving},_updateEditColumn:function(){var e=this,t=e._isEditColumnVisible(),n=e.option("editing.useIcons"),i=te+(n?" "+ne:"");e._columnsController.addCommandColumn({type:"buttons",command:"edit",visible:t,cssClass:i,width:"auto",alignment:"center",cellTemplate:e._getEditCommandCellTemplate(),fixedPosition:"right"}),e._columnsController.columnOption("command:edit",{visible:t,cssClass:i})},_isEditColumnVisible:function(){var e=this,t=e.option("editing");if(t){var n=Ne(e),i=!1;switch(n){case me:i=t.allowUpdating||t.allowAdding;break;case ye:case be:i=t.allowUpdating}return t.allowDeleting||i}},_updateEditButtons:function(){var e=this,t=e.getView("headerPanel"),n=e.hasChanges();t&&(t.setToolbarItemDisabled("saveButton",!n),t.setToolbarItemDisabled("revertButton",!n))},_applyModified:function(e){e&&e.addClass(ce)},_beforeCloseEditCellInBatchMode:function(){},cancelEditData:function(){var e=this.getChanges(),t={cancel:!1,changes:e};this.executeAction("onEditCanceling",t),t.cancel||(this._cancelEditDataCore(),this.executeAction("onEditCanceled",{changes:e}))},_cancelEditDataCore:function(){var e=Ne(this),t=this._getVisibleEditRowIndex(),n=this._dataController;this._beforeCancelEditData(),this.init(),this.resetChanges(),this._resetEditColumnName(),this._resetEditRowKey(),De.indexOf(e)!==-1&&t>=0?n.updateItems({changeType:"update",rowIndices:[t,t+1]}):n.updateItems({repaintChangesOnly:this.option("repaintChangesOnly")}),e===be&&this._hideEditPopup()},_hideEditPopup:function(){this._editPopup&&this._editPopup.option("visible",!1)},hasEditData:function(){return this.hasChanges()},closeEditCell:function(e,t){var n=this,i=this,o=(0,L.when)(),a=i._getVisibleEditRowIndex();return je(i)||(o=(0,L.Deferred)(),this.executeOperation(o,function(){n._closeEditCellCore(e,a,t),o.resolve()})),o.promise()},_closeEditCellCore:function(e,t,n){var i=this,o=Ne(i),a=i._dataController;if(o===ve&&i.hasChanges())n||i.saveEditData().done(function(e){i.hasChanges()||i.closeEditCell(!!e)});else if(t>=0){var r=[t];this._resetEditRowKey(),i._resetEditColumnName(),i._beforeCloseEditCellInBatchMode(r),e||a.updateItems({changeType:"update",rowIndices:r})}},update:function(e){var t=this,n=t._dataController;n&&t._pageIndex!==n.pageIndex()&&("refresh"===e&&t.refresh(!0),t._pageIndex=n.pageIndex()),t._updateEditButtons()},_getRowIndicesForCascadeUpdating:function(e,t){return t?[]:[e.rowIndex]},addDeferred:function(e){var t=this;this._deferreds.indexOf(e)<0&&(this._deferreds.push(e),e.always(function(){var n=t._deferreds.indexOf(e);n>=0&&t._deferreds.splice(n,1)}))},_prepareChange:function(e,t,n){var i,o=this,a={},r=null===(i=e.row)||void 0===i?void 0:i.data,s=e.key,l=(0,_.default)(e.cellElement),u=Ne(o),c=new L.Deferred;if(void 0!==s){u===_e&&o._applyModified(l,e),e.value=t;var d=(0,L.fromPromise)(e.column.setCellValue(a,t,(0,C.extend)(!0,{},r),n));d.done(function(){c.resolve({data:a,key:s,oldData:r,type:we})}).fail(ze(c)).fail(function(e){return o._fireDataErrorOccurred(e)}),(0,x.isDefined)(n)&&e.column.displayValueMap&&(e.column.displayValueMap[t]=n),o._updateRowValues(e),o.addDeferred(c)}return c},_updateRowValues:function(e){if(e.values){var t=this._dataController,n=t.getRowIndexByKey(e.key),i=t.getVisibleRows()[n];i&&(e.values=i.values),e.values[e.columnIndex]=e.value}},updateFieldValue:function(e,t,n,i){var o=this,a=e.key,r=new L.Deferred;return void 0===a&&this._dataController.fireError("E1043"),e.column.setCellValue?this._prepareChange(e,t,n).done(function(t){(0,L.when)(o._applyChange(e,t,i)).always(function(){r.resolve()})}):r.resolve(),r.promise()},_focusPreviousEditingCellIfNeed:function(e){var t=this;if(t.hasEditData()&&!t.isEditCell(e.rowIndex,e.columnIndex))return t._focusEditingCell(),t._updateEditRow(e.row,!0),!0},_needUpdateRow:function(e){var t=this._columnsController.getVisibleColumns();e||(e=this._getEditColumn());var n=e&&e.setCellValue!==e.defaultSetCellValue,i=t.some(function(e){return e.calculateCellValue!==e.defaultCalculateCellValue});return n||i},_applyChange:function(e,t,n){var i=this,o=Ne(i),a=e.column.setCellValue!==e.column.defaultSetCellValue,r=e.column.showEditorAlways,s=o===ve&&e.row&&!e.row.isNewRow,l=r&&!n&&s&&i.hasEditData()&&!i.isEditCell(e.rowIndex,e.columnIndex);if(l)return i._focusEditingCell(),void i._updateEditRow(e.row,!0,a);if(i._addChange(t,e.row),i._updateEditButtons(),r&&!n){if(s)return i._setEditRowKey(e.row.key,!0),i._setEditColumnNameByIndex(e.columnIndex,!0),i.saveEditData();o===_e&&(n=i._needUpdateRow(e.column))}var u=e.row;u&&(n||a?i._updateEditRow(u,n,a):u.update&&u.update())},_updateEditRowCore:function(e,t,n){var i=this,o=i._editForm,a=Ne(i);a===be?i.option("repaintChangesOnly")?e.update&&e.update(e):o&&(i._updateEditFormDeferred=(new L.Deferred).done(function(){return o.repaint()}),i._updateLockCount||i._updateEditFormDeferred.resolve()):i._dataController.updateItems({changeType:"update",rowIndices:i._getRowIndicesForCascadeUpdating(e,t)})},_endUpdateCore:function(){this._updateEditFormDeferred&&this._updateEditFormDeferred.resolve()},_updateEditRow:function(e,t,n){var i=this;if(t||!je(i))i._updateEditRowCore(e,!t,n),t||i._focusEditingCell();else{var o=new L.Deferred;i.addDeferred(o),setTimeout(function(){var t=(0,_.default)(m.default.getActiveElement()),a=i._rowsView.getCellIndex(t,e.rowIndex),r=t.get(0),s=T.default.getSelectionRange(r);if(i._updateEditRowCore(e,!1,n),a>=0){var l=i._rowsView._getCellElement(e.rowIndex,a);i._delayedInputFocus(l,function(){setTimeout(function(){r=m.default.getActiveElement(),s.selectionStart>=0&&T.default.setSelectionRange(r,s)})})}o.resolve()})}},_addChange:function(e,t){var n=c(this.getChanges()),i=T.default.getIndexByKey(e.key,n);i<0&&(i=n.length,this._addInternalData({key:e.key,oldData:e.oldData}),delete e.oldData,n.push(e));var o=l({},n[i]);return o&&(e.data&&(o.data=(0,O.createObjectWithChanges)(o.data,e.data)),o.type&&e.data||!e.type||(o.type=e.type),t&&(t.oldData=this._getOldData(t.key),t.data=(0,O.createObjectWithChanges)(t.data,e.data))),n[i]=o,this._silentOption(Fe,n),i},_getFormEditItemTemplate:function(e,n){return n.editCellTemplate||t(this)},renderFormEditTemplate:function(e,t,n,i,a){var r=this,s=(0,_.default)(i),l=t.column,u=o(t),c=null===e||void 0===e?void 0:e.row.data,d=(0,C.extend)({},e,{data:c,cellElement:null,isOnForm:!0,item:t,column:(0,C.extend)({},l,{editorType:u,editorOptions:t.editorOptions}),id:n.getItemID(t.name||t.dataField),columnIndex:l.index,setValue:!a&&l.allowEditing&&function(e){r.updateFieldValue(d,e)}});d.value=l.calculateCellValue(c);var h=r._getFormEditItemTemplate.bind(r)(d,l);return r._rowsView.renderTemplate(s,h,d,!!s.closest((0,v.getWindow)().document).length).done(function(){r._rowsView._updateCell(s,d)}),d},getFormEditorTemplate:function(e,t){var n=this,i=this.component.columnOption(t.dataField);return function(o,a){var r=(0,_.default)(a);e.row.watch&&e.row.watch(function(){return i.selector(e.row.data)},function(){var i,a=r.find(".dx-widget").first(),s=a.data("dxValidator"),l=null===(i=s)||void 0===i?void 0:i.option();r.contents().remove(),e=n.renderFormEditTemplate.bind(n)(e,t,o.component,r),a=r.find(".dx-widget").first(),s=a.data("dxValidator"),l&&!s&&a.dxValidator({validationRules:l.validationRules,validationGroup:l.validationGroup,dataGetter:l.dataGetter})}),e=n.renderFormEditTemplate.bind(n)(e,t,o.component,r)}},getEditFormOptions:function(e){var t=this.option("editing.form.customizeItem"),n=this.addWidgetPrefix(W),i=this.option("editing.form.items"),o={},r=this;if(i)a(i,function(e){var t=(null===e||void 0===e?void 0:e.name)||(null===e||void 0===e?void 0:e.dataField);t&&(o[t]=!!e.editorType)});else{var s=this.getController("columns").getColumns();i=[],(0,w.each)(s,function(e,t){t.isBand||t.type||i.push({column:t,name:t.name,dataField:t.dataField})})}return{items:i,formID:"dx-"+new b.default,customizeItem:function(i){var a,s=i.name||i.dataField;if((i.column||s)&&(a=i.column||r._columnsController.columnOption(i.name?"name:"+i.name:"dataField:"+i.dataField)),a){i.label=i.label||{},i.label.text=i.label.text||a.caption,i.template=i.template||r.getFormEditorTemplate(e,i),i.column=a,i.isCustomEditorType=o[s],a.formItem&&(0,C.extend)(i,a.formItem),void 0===i.isRequired&&a.validationRules&&(i.isRequired=a.validationRules.some(function(e){return"required"===e.type}),i.validationRules=[]);var l=!(0,x.isDefined)(i.visible)||i.visible;!r._firstFormItem&&l&&(r._firstFormItem=i)}t&&t.call(this,i),i.cssClass=(0,x.isString)(i.cssClass)?i.cssClass+" "+n:n}}},getEditFormTemplate:function(){var e=this;return function(t,n,i){var o=e.option("editing.form"),a=e.getEditFormOptions(n);if(e._firstFormItem=void 0,e._editForm=e._createComponent((0,_.default)("<div>").appendTo(t),V.default,(0,C.extend)({},o,a)),!i){var r=(0,_.default)("<div>").addClass(e.addWidgetPrefix(K)).appendTo(t);e._createComponent((0,_.default)("<div>").appendTo(r),R.default,e._getSaveButtonConfig()),e._createComponent((0,_.default)("<div>").appendTo(r),R.default,e._getCancelButtonConfig())}e._editForm.on("contentReady",function(){e._editPopup&&e._editPopup.repaint()})}},getColumnTemplate:function(e){var n,i,o=this,a=e.column,r=e.row&&e.row.rowIndex,s=je(o),l=o.isEditRow(r),u=o.isEditCell(r,e.columnIndex);if(!(a.showEditorAlways||a.setCellValue&&(l&&a.allowEditing||u))||"data"!==e.rowType&&"detailAdaptive"!==e.rowType||a.command)"detail"===a.command&&"detail"===e.rowType&&l&&(n=o.getEditFormTemplate(e));else{var c=o.allowUpdating(e);((c||l)&&a.allowEditing||u)&&(s&&l||!s)&&(a.showEditorAlways&&!s&&(i={cancel:!1,key:e.row.isNewRow?void 0:e.row.key,data:e.row.data,column:a},o._isEditingStart(i)),i&&i.cancel||(e.setValue=function(t,n){o.updateFieldValue(e,t,n)})),n=a.editCellTemplate||t(o)}return n},_createButton:function(e,t,n){var i=this,o=Pe[t.name],a=i.option("editing.useIcons"),r=(0,_.default)("<a>").attr("href","#").addClass(Z).addClass(t.cssClass);if(t.template)i._rowsView.renderTemplate(e,t.template,n,!0);else{if(a&&o||t.icon){o=t.icon||o;var s=z.getImageSourceType(o);"image"===s||"svg"===s?r=z.getImageContainer(o).addClass(t.cssClass):r.addClass("dx-icon"+("dxIcon"===s?"-":" ")+o).attr("title",t.text),r.addClass("dx-link-icon"),e.addClass(ne);var l=this.getButtonLocalizationNames()[t.name];l&&r.attr("aria-label",P.default.format(l))}else r.text(t.text);(0,x.isDefined)(t.hint)&&r.attr("title",t.hint),y.default.on(r,(0,E.addNamespace)("click",de),i.createAction(function(e){t.onClick.call(t,(0,C.extend)({},e,{row:n.row,column:n.column})),e.event.preventDefault(),e.event.stopPropagation()})),e.append(r,"&nbsp;")}},getButtonLocalizationNames:function(){return{edit:"dxDataGrid-editingEditRow",save:"dxDataGrid-editingSaveRowChanges","delete":"dxDataGrid-editingDeleteRow",undelete:"dxDataGrid-editingUndeleteRow",cancel:"dxDataGrid-editingCancelRowChanges"}},prepareEditButtons:function(e){var t=this,n=t.option("editing")||{},i=t.option("editing.texts")||{},o={revert:i.cancelAllChanges,save:i.saveAllChanges,addRow:i.addRow},a={revert:"cancel",save:"save",addRow:"addrow"},r=[],s=function(n,i,r){var s=a[n],l=function(n){(0,_.default)(n.element).addClass(e._getToolbarButtonClass(ee+" "+t.addWidgetPrefix(s)+"-button"))},u=o[n],c=("save"===s||"cancel"===s)&&!t.hasChanges();return{widget:"dxButton",options:{onInitialized:l,icon:"edit-button-"+s,disabled:c,onClick:function(){setTimeout(function(){t[i]()})},text:u,hint:u},showText:"inMenu",name:n+"Button",location:"after",locateInMenu:"auto",sortIndex:r}};return n.allowAdding&&r.push(s("addRow","addRow",20)),(n.allowUpdating||n.allowAdding||n.allowDeleting)&&Ne(t)===_e&&(r.push(s("save","saveEditData",21)),r.push(s("revert","cancelEditData",22))),r},highlightDataCell:function(e,t){var n=this.isCellModified(t);n&&t.column.setCellValue&&e.addClass(ce)},_afterInsertRow:function(){},_beforeSaveEditData:function(e){if(e&&!(0,x.isDefined)(e.key)&&(0,x.isDefined)(e.type))return!0},_afterSaveEditData:function(){},_beforeCancelEditData:function(){},_allowEditAction:function(e,t){var n=this.option("editing."+e);return(0,x.isFunction)(n)&&(n=n({component:this.component,row:t.row})),n},allowUpdating:function(e,t){var n=this.option("editing.startEditAction")||ke,i=!(arguments.length>1)||(n===t||"down"===t);return i&&this._allowEditAction("allowUpdating",e)},allowDeleting:function(e){return this._allowEditAction("allowDeleting",e)},isCellModified:function(e){var t=e.columnIndex,n=e.row&&(e.row.isNewRow?e.row.values:e.row.modifiedValues);return!!n&&void 0!==n[t]}}}()),$e={defaultOptions:function(){return{editing:{mode:"row",refreshMode:"full",allowAdding:!1,allowUpdating:!1,allowDeleting:!1,useIcons:!1,selectTextOnEditStart:!1,confirmDelete:!0,texts:{editRow:P.default.format("dxDataGrid-editingEditRow"),saveAllChanges:P.default.format("dxDataGrid-editingSaveAllChanges"),saveRowChanges:P.default.format("dxDataGrid-editingSaveRowChanges"),cancelAllChanges:P.default.format("dxDataGrid-editingCancelAllChanges"),cancelRowChanges:P.default.format("dxDataGrid-editingCancelRowChanges"),addRow:P.default.format("dxDataGrid-editingAddRow"),deleteRow:P.default.format("dxDataGrid-editingDeleteRow"),undeleteRow:P.default.format("dxDataGrid-editingUndeleteRow"),confirmDeleteMessage:P.default.format("dxDataGrid-editingConfirmDeleteMessage"),confirmDeleteTitle:""},form:{colCount:2},popup:{},startEditAction:"click",editRowKey:null,editColumnName:null,changes:[]}}},controllers:{editing:qe},extenders:{controllers:{data:{init:function(){this._editingController=this.getController("editing"),this.callBase()},reload:function(e,t){return!t&&this._editingController.refresh(),this.callBase.apply(this,arguments)},repaintRows:function(){if(!this.getController("editing").isSaving())return this.callBase.apply(this,arguments)},_updateEditRow:function(e){var t=this._editingController,n=this.option(Le),i=T.default.getIndexByKey(n,e),o=e[i];o&&(o.isEditing=!0,t.getEditMode()===ye&&(o.rowType="detail"))},_updateItemsCore:function(e){this.callBase(e),this._updateEditRow(this.items())},_applyChangeUpdate:function(e){this._updateEditRow(e.items),this.callBase(e)},_applyChangesOnly:function(e){this._updateEditRow(e.items),this.callBase(e)},_processItems:function(e,t){return e=this._editingController.processItems(e,t),this.callBase(e,t)},_processDataItem:function(e,t){return this._editingController.processDataItem(e,t,this.generateDataValues),this.callBase(e,t)},_processItem:function(e,t){return e=this.callBase(e,t),e.isNewRow&&(t.dataIndex--,delete e.dataIndex),e},_getChangedColumnIndices:function(e,t,n,i){var o=this.getController("editing"),a=o.isRowEditMode();if(!(e.isNewRow!==t.isNewRow||e.removed!==t.removed||a&&e.isEditing!==t.isEditing))return this.callBase.apply(this,arguments)},_isCellChanged:function(e,t,n,i,o){var a=this.getController("editing"),r=e.cells&&e.cells[i],s=a&&a.isEditCell(n,i);return(!o||!s)&&(!(!r||!r.column||r.column.showEditorAlways||r.isEditing===s)||this.callBase.apply(this,arguments))}}},views:{rowsView:{init:function(){this.callBase(),this._editingController=this.getController("editing")},getCellElements:function(e){var t=this.callBase(e),n=this._editingController,i=n.getEditForm(),o=n.getEditFormRowIndex();return o===e&&t&&i?i.$element().find("."+this.addWidgetPrefix(W)+", ."+oe):t},getCellIndex:function(e,t){if(!e.is("td")&&t>=0){var n=this.getCellElements(t),i=-1;return(0,w.each)(n,function(t,n){(0,_.default)(n).find(e).length&&(i=t)}),i}return this.callBase.apply(this,arguments)},_getVisibleColumnIndex:function(e,t,n){var i=this._editingController.getEditFormRowIndex();if(i===t&&(0,x.isString)(n)){var o=this._columnsController.columnOption(n);return this._getEditFormEditorVisibleIndex(e,o)}return this.callBase.apply(this,arguments)},_getEditFormEditorVisibleIndex:function(e,t){var n=-1;return(0,w.each)(e,function(e,i){var o=(0,_.default)(i).find(".dx-field-item-content").data("dx-form-item");if(o&&o.column&&t&&o.column.index===t.index)return n=e,!1}),n},publicMethods:function(){return this.callBase().concat(["cellValue"])},_getCellTemplate:function(e){var t=this,n=t._editingController.getColumnTemplate(e);return n||t.callBase(e)},_isNativeClick:function(){return(B.default.real().ios||B.default.real().android)&&this.option("editing.allowUpdating")},_createTable:function(){var e=this,t=e.callBase.apply(e,arguments);return!je(e)&&e.option("editing.allowUpdating")&&y.default.on(t,(0,E.addNamespace)(F.default.name,"dxDataGridRowsView"),"td:not(."+X+")",e.createAction(function(){var t=e._editingController;t.isEditing()&&t.closeEditCell()})),t},_createRow:function(e){var t=this.callBase(e);if(e){var n=this._editingController,i=n.isEditRow(e.rowIndex),o=!!e.removed,a=!!e.isNewRow,r=!!e.modified;Ne(this)===_e?o&&t.addClass(se):i&&t.addClass(J),a&&t.addClass(le),r&&t.addClass(ue),(i||a||o)&&t.removeClass(Q),i&&"detail"===e.rowType&&t.addClass(this.addWidgetPrefix(j))}return t},_getColumnIndexByElement:function(e){for(var t=e.closest("table"),n=this.getTableElements();t.length&&!n.filter(t).length;)e=t.closest("td"),t=e.closest("table");return this._getColumnIndexByElementCore(e)},_getColumnIndexByElementCore:function(e){var t=e.closest("."+re+"> td:not(.dx-master-detail-cell)");return this.getCellIndex(t)},_editCellByClick:function(e,t){var n=this,i=n._editingController,o=(0,_.default)(e.event.target),a=n._getColumnIndexByElement(o),r=n._dataController.items()[e.rowIndex],s=i.allowUpdating({row:r},t)||r&&r.isNewRow,l=n._columnsController.getVisibleColumns()[a],u=s&&l&&(l.allowEditing||i.isEditCell(e.rowIndex,a)),c=n.option("editing.startEditAction")||"click";return"down"===t?l&&l.showEditorAlways&&u&&i.editCell(e.rowIndex,a):("click"!==t||"dblClick"!==c||i.isEditCell(e.rowIndex,a)||i.closeEditCell(),u&&t===c?i.editCell(e.rowIndex,a)||i.isEditRow(e.rowIndex):void 0)},_rowPointerDown:function(e){var t=this;this._pointerDownTimeout=setTimeout(function(){t._editCellByClick(e,"down")})},_rowClick:function(e){var t=(0,_.default)(e.rowElement).hasClass(this.addWidgetPrefix(j));e.event[Ee]=this.component,this._editCellByClick(e,"click")||t||this.callBase.apply(this,arguments)},_rowDblClick:function(e){this._editCellByClick(e,"dblClick")||this.callBase.apply(this,arguments)},_cellPrepared:function(e,t){var n=this._editingController,i=!!t.column.command,o=t.setValue,a=n.isEditRow(t.rowIndex),r=We(a,t);if(Ge(a,t)){var s=t.column.alignment;e.toggleClass(this.addWidgetPrefix(q),!o).toggleClass(fe,!o),s&&e.find(pe).first().css("textAlign",s)}r&&this._editCellPrepared(e),t.column&&!i&&n.highlightDataCell(e,t),this.callBase.apply(this,arguments)},_editCellPrepared:function(e){},_formItemPrepared:function(){},_isFormItem:function(e){var t="detail"===e.rowType||"detailAdaptive"===e.rowType,n="data"===e.rowType&&"popup"===Ne(this);return(t||n)&&e.item},_updateCell:function(e,t){this._isFormItem(t)?this._formItemPrepared(t,e):this.callBase(e,t)},_update:function(e){this.callBase(e),"updateSelection"===e.changeType&&this.getTableElements().children("tbody").children("."+J).removeClass(Q)},_getCellOptions:function(e){var t=this.callBase(e);return t.isEditing=this._editingController.isEditCell(t.rowIndex,t.columnIndex),t},_createCell:function(e){var t=this.callBase(e),n=this._editingController.isEditRow(e.rowIndex);return Ge(n,e)&&t.addClass(X),t},_renderCellContent:function(e,t){"data"===t.rowType&&Ne(this)===be&&t.row.visible===!1||this.callBase.apply(this,arguments)},cellValue:function(e,t,n,i){var o=this.getCellOptions(e,t);if(o){if(void 0===n)return o.value;this._editingController.updateFieldValue(o,n,i,!0)}},dispose:function(){this.callBase.apply(this,arguments),clearTimeout(this._pointerDownTimeout)},_renderCore:function(){this.callBase.apply(this,arguments),this._editingController._focusEditorIfNeed()}},headerPanel:{_getToolbarItems:function(){var e=this.callBase(),t=this.getController("editing").prepareEditButtons(this);return t.concat(e)},optionChanged:function(e){var t=e.fullName;switch(e.name){case"editing":var n=[Ve,Fe,He,Le],i=t&&!n.some(function(e){return e===t});i&&this._invalidate(),this.callBase(e);break;default:this.callBase(e)}},isVisible:function(){var e=this,t=e.getController("editing").option("editing");return e.callBase()||t&&(t.allowAdding||(t.allowUpdating||t.allowDeleting)&&t.mode===_e)}}}}};t.default=$e,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.default=void 0;var c=i(n(2)),d=i(n(5)),h=i(n(44)),f=i(n(37)),p=n(90),g=n(4),_=n(3),m=n(1),v=n(0),y=n(75),b=i(n(10)),x=i(n(38)),w=i(n(25)),C=i(n(100)),k=i(n(221)),S=i(n(195)),D=i(n(68)),I=i(n(17)),T=n(6),O=i(n(88)),E=n(42),A=i(n(23)),P="invalid",R="revert-tooltip",M="rowsview",B="dx-invalid-message",V="invalid-message",F="dx-invalid-message-always",L="dx-revert-button",H="validator",z="dx-pending-indicator",N="dx-validation-pending",j="content",W="__DX_INSERT_INDEX__",G=2,q="row",$="form",K="batch",U="cell",Y="popup",Z="dx-group-cell",X=[Y,$],Q="transparent",J={valid:"valid",invalid:"invalid",pending:"pending"},ee="insert",te="remove",ne="cancel",ie=function(e){return(0,m.isDefined)(e)&&e!==ne},oe=function(e,t){return void 0!==e||void 0===e&&t&&!t.isNewRow},ae=h.default.Controller.inherit(function(){return{init:function(){this._editingController=this.getController("editing"),this.createAction("onRowValidating"),this._validationState||(this._validationState=[])},_rowIsValidated:function(e){var t=this._getValidationData(null===e||void 0===e?void 0:e.key);return!!t&&!!t.validated},_getValidationData:function(e,t){var n=this._validationState.filter(function(t){return t.key===e})[0];return!n&&t&&(n={key:e,isValid:!0},this._validationState.push(n)),n},_getBrokenRules:function(e,t){var n;return n=t?t.brokenRules||t.brokenRule&&[t.brokenRule]:e.brokenRules||[]},_rowValidating:function(e,t){var n=new T.Deferred,i=this._editingController.getChangeByKey(null===e||void 0===e?void 0:e.key),o=this._getBrokenRules(e,t),a=t?t.isValid:e.isValid,r={brokenRules:o,isValid:a,key:i.key,newData:i.data,oldData:this._editingController._getOldData(i.key),promise:null,errorText:this.getHiddenValidatorsErrorText(o)};return this.executeAction("onRowValidating",r),(0,T.when)(r.promise).always(function(){e.isValid=r.isValid,e.errorText=r.errorText,n.resolve(r)}),n.promise()},getHiddenValidatorsErrorText:function(e){var t=[];return(0,_.each)(e,function(e,n){var i=n.column,o=i&&void 0!==i.groupIndex&&!i.showWhenGrouped,a=i&&i.visible;n.validator.$element().parent().length||a&&!o||t.push(n.message)}),t.join(", ")},validate:function(e){var t=this,n=!0,i=this._editingController,o=new T.Deferred,a=[],r=i.getEditMode();if(e=e||r===q,this._isValidationInProgress)return o.resolve(!1).promise();if(this._isValidationInProgress=!0,e){
i.addDeferred(o);var s=i.getChanges();(0,_.each)(s,function(e,i){var o=i.type,r=i.key;if("remove"!==o){var s=t._getValidationData(r),l=t.validateGroup(s);a.push(l),l.done(function(e){s.validated=!0,n=n&&e.isValid})}})}else if(this._currentCellValidator){var l=this.validateGroup(this._currentCellValidator._findGroup());a.push(l),l.done(function(e){n=e.isValid})}return T.when.apply(void 0,a).done(function(){t._isValidationInProgress=!1,o.resolve(n)}),o.promise()},validateGroup:function e(t){var n,i,o=this,a=new T.Deferred,e=t&&C.default.getGroupConfig(t);return null!==e&&void 0!==e&&e.validators.length&&(this.resetRowValidationResults(t),i=C.default.validateGroup(t)),(0,T.when)((null===(n=i)||void 0===n?void 0:n.complete)||i).done(function(e){(0,T.when)(o._rowValidating(t,e)).done(a.resolve)}),a.promise()},isRowDataModified:function(e){return!(0,m.isEmptyObject)(e.data)},updateValidationState:function(e){var t=this._editingController.getEditMode(),n=e.key,i=this._getValidationData(n,!0);if(X.indexOf(t)===-1){if(e.type===ee&&!this.isRowDataModified(e))return void(i.isValid=!0);this.setDisableApplyValidationResults(!0);var o=C.default.getGroupConfig(i);if(o){var a=C.default.validateGroup(i);(0,T.when)(a.complete||a).done(function(e){i.isValid=e.isValid,i.brokenRules=e.brokenRules})}else i.brokenRules&&i.brokenRules.length||(i.isValid=!0);this.setDisableApplyValidationResults(!1)}else i.isValid=!0},setValidator:function(e){this._currentCellValidator=e},renderCellPendingIndicator:function(e){var t=e.find("."+z);if(!t.length){var n=e;t=(0,c.default)("<div>").appendTo(n).addClass(z),this._createComponent(t,O.default),e.addClass(N)}},disposeCellPendingIndicator:function(e){var t=e.find("."+z);if(t.length){var n=O.default.getInstance(t);n&&(n.dispose(),n.$element().remove()),e.removeClass(N)}},validationStatusChanged:function(e){var t=e.validator,n=t.option("validationGroup"),i=t.option("dataGetter")().column;this.updateCellValidationResult({rowKey:n.key,columnIndex:i.index,validationResult:e})},validatorInitialized:function(e){e.component.on("validating",this.validationStatusChanged.bind(this)),e.component.on("validated",this.validationStatusChanged.bind(this))},validatorDisposing:function(e){var t=e.component,n=t.option("validationGroup"),i=t.option("dataGetter")().column,o=this.getCellValidationResult({rowKey:null===n||void 0===n?void 0:n.key,columnIndex:i.index});ie(o)&&o.status===J.pending&&this.cancelCellValidationResult({change:n,columnIndex:i.index})},applyValidationResult:function(e,t){var n=t.validator,i=n.option("validationGroup"),o=n.option("dataGetter")().column;if(t.brokenRules&&t.brokenRules.forEach(function(e){e.columnIndex=o.index,e.column=o}),e){var a=this.getCellValidationResult({rowKey:i.key,columnIndex:o.index}),r=ie(a)&&a.disabledPendingId===t.id;if(this._disableApplyValidationResults||r)return;if(t.status===J.invalid){var s=e.find(":focus");(0,y.focused)(s)||(d.default.trigger(s,"focus"),d.default.trigger(s,w.default.down))}var l=!o.editCellTemplate&&this.getController("editorFactory").getEditorInstance(e);t.status===J.pending?l?l.option("validationStatus",J.pending):this.renderCellPendingIndicator(e):l?l.option("validationStatus",J.valid):this.disposeCellPendingIndicator(e),e.toggleClass(this.addWidgetPrefix(P),t.status===J.invalid)}},createValidator:function(e,t){var n=this,i=this._editingController,o=e.column,a=o.showEditorAlways;if(!(0,m.isDefined)(o.command)&&o.validationRules&&Array.isArray(o.validationRules)&&o.validationRules.length){var r=i.getIndexByKey(e.key,i.getChanges()),s=r>-1;if(!s){if(!a){var l=this.getController("columns"),u=(null===l||void 0===l?void 0:l.getVisibleColumns())||[];a=u.some(function(e){return e.showEditorAlways})}var d=(0,g.equalByValue)(this.option("editing.editRowKey"),e.key),h=i.isCellOrBatchEditMode()&&i.allowUpdating({row:e.row});s=d||h&&a,h&&a&&i._addInternalData({key:e.key,oldData:e.data})}if(s){if(t&&!t.length)return void I.default.log("E1050");var f=this._getValidationData(e.key,!0),_=function(){var t=i.getChangeByKey(null===f||void 0===f?void 0:f.key),n=o.calculateCellValue((null===t||void 0===t?void 0:t.data)||{});return void 0!==n?n:e.value},y=t&&t.hasClass("dx-widget");t&&t.addClass(this.addWidgetPrefix(H));var b=new k.default(t||(0,c.default)("<div>"),{name:o.caption,validationRules:(0,v.extend)(!0,[],o.validationRules),validationGroup:f,adapter:y?null:{getValue:_,applyValidationResults:function(e){n.applyValidationResult(t,e)}},dataGetter:function(){var e=null===f||void 0===f?void 0:f.key,t=i.getChangeByKey(e),n=i._getOldData(e);return{data:(0,p.createObjectWithChanges)(n,null===t||void 0===t?void 0:t.data),column:o}},onInitialized:this.validatorInitialized.bind(this),onDisposing:this.validatorDisposing.bind(this)});if(y){var x=b.option("adapter");x&&(x.getValue=_,x.validationRequestsCallbacks=[])}return b}}},setDisableApplyValidationResults:function(e){this._disableApplyValidationResults=e},getDisableApplyValidationResults:function(){return this._disableApplyValidationResults},isCurrentValidatorProcessing:function(e){var t=e.rowKey,n=e.columnIndex;return this._currentCellValidator&&this._currentCellValidator.option("validationGroup").key===t&&this._currentCellValidator.option("dataGetter")().column.index===n},validateCell:function(e){var t={rowKey:e.option("validationGroup").key,columnIndex:e.option("dataGetter")().column.index},n=this.getCellValidationResult(t),i=ie(n);i||(n=e.validate());var o=new T.Deferred,a=e.option("adapter");return i&&n.status===J.pending&&(this.updateCellValidationResult(t),a.applyValidationResults(n)),(0,T.when)(n.complete||n).done(function(e){i&&a.applyValidationResults(e),o.resolve(e)}),o.promise()},updateCellValidationResult:function(e){var t=e.rowKey,n=e.columnIndex,i=e.validationResult,o=this._getValidationData(t);if(o){o.validationResults||(o.validationResults={});var a;if(i){if(a=(0,v.extend)({},i),o.validationResults[n]=a,i.status===J.pending&&(this._editingController.getEditMode()===U&&(a.deferred=new T.Deferred,a.complete.always(function(){a.deferred.resolve()}),this._editingController.addDeferred(a.deferred)),this._disableApplyValidationResults))return void(a.disabledPendingId=i.id)}else a=o.validationResults[n];a&&a.disabledPendingId&&delete a.disabledPendingId}},getCellValidationResult:function(e){var t,n=e.rowKey,i=e.columnIndex,o=this._getValidationData(n,!0);return null===o||void 0===o?void 0:null===(t=o.validationResults)||void 0===t?void 0:t[i]},removeCellValidationResult:function(e){var t=e.change,n=e.columnIndex,i=this._getValidationData(null===t||void 0===t?void 0:t.key);i&&i.validationResults&&(this.cancelCellValidationResult({change:t,columnIndex:n}),delete i.validationResults[n])},cancelCellValidationResult:function(e){var t=e.change,n=e.columnIndex,i=this._getValidationData(t.key);if(t&&i.validationResults){var o=i.validationResults[n];o&&(o.deferred&&o.deferred.reject(ne),i.validationResults[n]=ne)}},resetRowValidationResults:function(e){e&&(e.validationResults&&delete e.validationResults,delete e.validated)},isInvalidCell:function(e){var t=e.rowKey,n=e.columnIndex,i=this.getCellValidationResult({rowKey:t,columnIndex:n});return ie(i)&&i.status===J.invalid},getCellValidator:function(e){var t=e.rowKey,n=e.columnIndex,i=this._getValidationData(t),o=i&&C.default.getGroupConfig(i),a=o&&o.validators;return a&&a.filter(function(e){var t=e.option("dataGetter")().column;return!!t&&t.index===n})[0]},setCellValidationStatus:function(e){var t=this.getCellValidationResult({rowKey:e.key,columnIndex:e.column.index});(0,m.isDefined)(t)?e.validationStatus=t!==ne?t.status:ne:delete e.validationStatus}}}()),re={defaultOptions:function(){return{editing:{texts:{validationCancelChanges:b.default.format("dxDataGrid-validationCancelChanges")}}}},controllers:{validating:ae},extenders:{controllers:{editing:{_addChange:function(e,t){var n=this.callBase(e,t),i=this.getController("validating");if(n>=0&&e.type!==te){var o=this.getChanges()[n];o&&i.updateValidationState(o)}return n},_handleChangesChange:function(e){this.callBase.apply(this,arguments);var t=this.getController("validating");e.value.forEach(function(e){void 0===t._getValidationData(e.key)&&t.updateValidationState(e)})},_updateRowAndPageIndices:function(){var e=this,t=this,n=t.getView("rowsView").getTopVisibleItemIndex(),i=n;(0,_.each)(t.getChanges(),function(o,a){var r=a.key,s=a.type,l=e.getController("validating")._getValidationData(r);l&&!l.isValid&&l.pageIndex!==t._pageIndex&&(l.pageIndex=t._pageIndex,s===ee?l.rowIndex=n:l.rowIndex=i,i++)})},getEditFormOptions:function(e){var t=this.callBase.apply(this,arguments),n=this.getController("validating"),i=n._getValidationData(e.key,!0);return(0,v.extend)({},t,{validationGroup:i})},_updateEditRowCore:function(e,t,n){this.callBase.apply(this,arguments),n&&this._editForm&&!e.isNewRow&&this._editForm.validate()},_needInsertItem:function(e){var t=e.key,n=this.callBase.apply(this,arguments),i=this.getController("validating")._getValidationData(t);return!n||null!==i&&void 0!==i&&i.isValid||(n=t.pageIndex===this._pageIndex),n},_prepareEditCell:function(e){var t=this.callBase.apply(this,arguments),n=this.getController("validating");return t&&e.column.showEditorAlways&&n.updateValidationState({key:e.key}),t},processItems:function(e,t){var n,i=this,o=i.getChanges(),a=i.getController("data"),r=this.getController("validating"),s=function(e,t){var n=-1,i=e.type===ee,o=e.key;return(0,_.each)(t,function(e,t){if((0,g.equalByValue)(o,i?t:a.keyOf(t)))return n=e,!1}),n};e=i.callBase(e,t);var l=e.length,u=function(t,n){var i={key:t.key},o=s(t,e);if(!(o>=0)){n.rowIndex=n.rowIndex>l?n.rowIndex%l:n.rowIndex;var a=n.rowIndex;i[W]=1,e.splice(a,0,i)}};if(i.getEditMode()===K&&"prepend"!==t&&"append"!==t)for(n=0;n<o.length;n++){var c=o[n].key,d=r._getValidationData(c);d&&o[n].type&&d.pageIndex===i._pageIndex&&c.pageIndex!==i._pageIndex&&u(o[n],d)}return e},processDataItem:function(e){var t=e.data[W],n=t?e.data.key:e.key,i=this.getEditMode();if(i===K&&t&&n){var o=this.getChanges(),a=f.default.getIndexByKey(n,o);if(a>=0){var r=o[a];if(r.type!==ee){var s=this._getOldData(r.key);e.data=(0,v.extend)(!0,{},s,r.data),e.key=n}}}this.callBase.apply(this,arguments)},_createInvisibleColumnValidators:function(e){var t=this,n=this,i=this.getController("validating"),a=this.getController("columns"),r=a.getColumns(),s=a.getInvisibleColumns().filter(function(e){return!e.isBand}),l=a.getGroupColumns().filter(function(e){return!e.showWhenGrouped&&s.indexOf(e)===-1}),u=[],c=function(e,n){return t._dataController.getRowIndexByKey(n)>=0&&s.indexOf(e)<0};return s.push.apply(s,o(l)),X.indexOf(this.getEditMode())===-1&&(0,_.each)(r,function(t,o){e.forEach(function(e){var t;if(!c(o,e.key)){if(e.type===ee)t=e.data;else if("update"===e.type){var a=n._getOldData(e.key);t=(0,p.createObjectWithChanges)(a,e.data)}if(t){var r=i.createValidator({column:o,key:e.key,value:o.calculateCellValue(t)});r&&u.push(r)}}})}),function(){u.forEach(function(e){e.dispose()})}},_beforeSaveEditData:function(e,t){var n=this,i=this.callBase.apply(this,arguments),o=this.getController("validating"),a=o._getValidationData(null===e||void 0===e?void 0:e.key);if(e){var r="remove"===e.type||a.isValid;i=i||!r}else{var s=this._createInvisibleColumnValidators(this.getChanges());i=new T.Deferred,this.executeOperation(i,function(){o.validate(!0).done(function(e){switch(s(),n._updateRowAndPageIndices(),n.getEditMode()){case U:e||n._focusEditingCell();break;case K:e||(n._resetEditRowKey(),n._resetEditColumnName(),n.getController("data").updateItems())}i.resolve(!e)})})}return i.promise?i.promise():i},_beforeEditCell:function(e,t,n){var i=this.callBase(e,t,n);if(this.getEditMode()===U){var o=this._rowsView._getCellElement(e,t),a=o&&o.data("dxValidator"),r=o&&o.closest(".dx-row").data("options"),s=a&&a.option("adapter").getValue();if(a&&oe(s,r)){var l=this.getController("validating"),u=new T.Deferred;return(0,T.when)(l.validateCell(a),i).done(function(e,t){u.resolve(e.status===J.valid&&t)}),u.promise()}if(!a)return i}},_afterSaveEditData:function(e){var t,n=this;if((0,_.each)(this.getChanges(),function(e,i){var o=n._showErrorRow(i);t=t||o}),t){var i=this._rowsView.getScrollable();i&&(i.update(),i.scrollToElement(t))}if(e&&this.getEditMode()===U&&this._needUpdateRow()){var o=this.getEditRowIndex();this._dataController.updateItems({changeType:"update",rowIndices:[o]}),this._focusEditingCell()}else e||(this.getController("validating")._validationState=[])},_handleDataChanged:function(e){var t=this.getController("validating")._validationState;"standard"===this.option("scrolling.mode")&&this.resetRowAndPageIndices(),"prepend"===e.changeType&&(0,_.each)(t,function(t,n){n.rowIndex+=e.items.length}),this.callBase(e)},resetRowAndPageIndices:function(){var e=this,t=this.getController("validating")._validationState;(0,_.each)(t,function(t,n){n.pageIndex!==e._pageIndex&&(delete n.pageIndex,delete n.rowIndex)})},_beforeCancelEditData:function(){var e=this.getController("validating");e._validationState=[],this.callBase()},_showErrorRow:function(e){var t,n=this.getController("errorHandling"),i=this.getController("data").items(),o=this.getIndexByKey(e.key,i),a=this.getController("validating")._getValidationData(e.key);if((null===a||void 0===a||!a.isValid)&&null!==a&&void 0!==a&&a.errorText&&o>=0)return t=this.getPopupContent(),n&&n.renderErrorRow(null===a||void 0===a?void 0:a.errorText,o,t)},updateFieldValue:function(e){var t=this,n=this.getController("validating"),i=new T.Deferred;return n.removeCellValidationResult({change:this.getChangeByKey(e.key),columnIndex:e.column.index}),this.callBase.apply(this,arguments).done(function(){var o=n.getCellValidator({rowKey:e.key,columnIndex:e.column.index});(0,T.when)(o&&n.validateCell(o)).done(function(e){t.getController("editorFactory").refocus(),i.resolve(e)})}),i.promise()},highlightDataCell:function(e,t){this.callBase.apply(this,arguments);var n=this.getController("validating");n.setCellValidationStatus(t);var i=!!t.setValue,o=this.isCellModified(t),a=(0,m.isDefined)(t.validationStatus),r=o&&t.column.setCellValue||i&&!o&&!(t.row.isNewRow||!a);if(r){var s=e.data("dxValidator");s&&(0,T.when)(this.getController("validating").validateCell(s)).done(function(){n.setCellValidationStatus(t)})}},getChangeByKey:function(e){var t=this.getChanges();return t[f.default.getIndexByKey(e,t)]},isCellModified:function(e){var t=this.callBase(e),n=this.getChangeByKey(e.key),i=!!e.row&&this.getController("validating").isInvalidCell({rowKey:e.key,columnIndex:e.column.index});return t||this.getController("validating")._rowIsValidated(n)&&i}},editorFactory:function(){var e=function(e,t){var n=(0,c.default)(t).closest("tr").index(),i=(0,c.default)(e._rowsView.getRowElement(n)).first().children().filter(":not(.dx-hidden-cell)");return e._rowsView._getWidths(i).reduce(function(e,t){return e+t},0)},t=function(e){var t,n;return e.some(function(i,o){if(i.command===Q)return t=0===o?-1:o,n=o===e.length-1?-1:o+i.colspan-1,!0}),{startColumnIndex:t,endColumnIndex:n}};return{_showRevertButton:function(e){var t=this;if(e&&e.length){var n=e.find("."+this.addWidgetPrefix(R));n&&n.remove(),n=(0,c.default)("<div>").addClass(this.addWidgetPrefix(R)).appendTo(e);var i={animation:null,visible:!0,target:e,container:e,closeOnOutsideClick:!1,closeOnTargetScroll:!1,contentTemplate:function(){var e=(0,c.default)("<div>").addClass(L),n={icon:"revert",hint:t.option("editing.texts.validationCancelChanges"),onClick:function(){t._editingController.cancelEditData()}};return new x.default(e,n).$element()},position:{my:"left top",at:"right top",of:e,offset:"1 0",collision:"flip",boundary:this._rowsView.element()},onPositioned:this._positionedHandler.bind(this)};return new S.default(n,i)}},_hideFixedGroupCell:function(e,t){var n,i,o=this._rowsView.isFixedColumns(),a=this._editingController.isFormEditMode();if(o&&!a){var r=e.closest(".dx-row").next().data("options");r&&"group"===r.rowType&&(n=(0,c.default)(this._rowsView.getRowElement(r.rowIndex)).last(),i=n.find("."+Z),i.length&&"hidden"!==i.get(0).style.visibility&&(i.css("visibility","hidden"),t.onDisposing=function(){i.css("visibility","")}))}},_positionedHandler:function(e,t){if(!e.component.__skipPositionProcessing){var n=(0,c.default)(e.element).hasClass(this.addWidgetPrefix(R)),i=!n&&this._rowsView.updateFreeSpaceRowHeight(),o=this._normalizeValidationMessagePositionAndMaxWidth(e,n,t);e.component.__skipPositionProcessing=!(!i&&!o),o?e.component.option(o):i&&e.component.repaint()}},_showValidationMessage:function(e,t,n,i){var o=this,a=e.find(".dx-dropdowneditor-overlay").data("dxPopup"),r=a&&a.option("visible"),s=r?"top right":"top "+n,l=r?"top left":"bottom "+n,u=e.closest(".".concat(this.addWidgetPrefix(j))),d="";t&&t.forEach(function(e){d+=(d.length?"<br/>":"")+(0,E.encodeHtml)(e)});var h=(0,c.default)("<div>").addClass(B).addClass(F).addClass(this.addWidgetPrefix(V)).html(d).appendTo(e),f={target:e,container:u,shading:!1,width:"auto",height:"auto",visible:!0,animation:!1,propagateOutsideClick:!0,closeOnOutsideClick:!1,closeOnTargetScroll:!1,position:{collision:"flip",boundary:this._rowsView.element(),boundaryOffset:"0 0",offset:{x:0,y:r||!A.default.mozilla&&!A.default.msie?0:-1},my:s,at:l},onPositioned:function(t){o._positionedHandler(t,r),o._shiftValidationMessageIfNeed(t.component.$content(),i&&i.$content(),e)}};this._hideFixedGroupCell(e,f),new D.default(h,f)},_normalizeValidationMessagePositionAndMaxWidth:function(n,i,o){var a=this._columnsController.getFixedColumns();if(a&&a.length){var r,s=!i&&e(this,n.element),l=i?n.component.overlayContent():n.component.$content(),u=l.outerWidth(!0),d=!i&&u>s,h=this._rowsView.getCellIndex((0,c.default)(n.element).closest("td")),f=t(a);return i||h!==f.startColumnIndex&&!d?h===f.endColumnIndex&&(r={collision:"none flip",my:"top right",at:i||o?"top left":"bottom right"},i&&(r.offset="-1 0")):r={collision:"none flip",my:"top left",at:o?"top right":"bottom left"},r&&{position:r,maxWidth:d?s-2:void 0}}},_shiftValidationMessageIfNeed:function(e,t,n){if(t){var i=e.offset(),o=t.offset();if(i.top===o.top&&i.left+e.width()>o.left){var a=t.width()+G;e.css("left",o.left<n.offset().left?-a:a)}}},_getTooltipsSelector:function(){var e=this.addWidgetPrefix(V),t=this.addWidgetPrefix(R);return".dx-editor-cell ."+t+", .dx-editor-cell ."+e+", .dx-cell-modified ."+e},init:function(){this.callBase(),this._editingController=this.getController("editing"),this._columnsController=this.getController("columns"),this._rowsView=this.getView("rowsView")},loseFocus:function(e){e||this.getController("validating").setValidator(null),this.callBase()},updateCellState:function(e,t,n){var i,o=null===e||void 0===e?void 0:e.closest(this._getFocusCellSelector()),a=null!==o&&void 0!==o&&o.is("td")?o:null,r=null===o||void 0===o?void 0:o.closest(".dx-row").data("options"),s=r?this.getController("editing").getChangeByKey(r.key):null,l=a&&this.getController("columns").getVisibleColumns()[a.index()];(t&&t.status===J.invalid||"update"===(null===s||void 0===s?void 0:s.type)&&!this._editingController.isSaving())&&this._editingController.getEditMode()===U&&(i=this._showRevertButton(o));var u=t&&t.status===J.invalid;if(u&&a&&l&&t&&t.brokenRules){var c=[];t.brokenRules.forEach(function(e){c.push(e.message)}),this._showValidationMessage(o,c,l.alignment||"left",i)}!n&&this._rowsView.element()&&this._rowsView.updateFreeSpaceRowHeight()},focus:function(e,t){var n=this;if(!arguments.length)return this.callBase();var i=e&&e.closest("."+this.addWidgetPrefix(M)).find(this._getTooltipsSelector());if(i&&i.remove(),null!==e&&void 0!==e&&e.hasClass("dx-row"))return this.callBase(e,t);var o,a=null===e||void 0===e?void 0:e.closest(this._getFocusCellSelector()),r=this.callBase,s=a&&(a.data("dxValidator")||e.find("."+this.addWidgetPrefix(H)).eq(0).data("dxValidator")),l=a&&a.closest(".dx-row").data("options"),u=this.getController("editing"),c=l?u.getChangeByKey(l.key):null,d=a&&a.is("td")?a:null,h=d&&this.getController("columns").getVisibleColumns()[d.index()],f=this.getController("validating");if(s){f.setValidator(s);var p=s.option("adapter").getValue();if(oe(p,l)||f._rowIsValidated(c))return u.waitForDeferredOperations().done(function(){(0,T.when)(f.validateCell(s)).done(function(i){o=i,c&&h&&!f.isCurrentValidatorProcessing({rowKey:c.key,columnIndex:h.index})||(o.status===J.invalid&&(t=!0),n.updateCellState(e,o,t),r.call(n,e,t))})}),this.callBase(e,t)}return this.updateCellState(e,o,t),this.callBase(e,t)},getEditorInstance:function(e){var t=e.find(".dx-texteditor").eq(0);return f.default.getWidgetInstance(t)}}}(),data:{_isCellChanged:function(e,t,n,i,o){var a=e.cells[i],r=a&&a.validationStatus,s=this.getController("validating"),l=s.getCellValidationResult({rowKey:e.key,columnIndex:i}),u=s._getValidationData(e.key),d=ie(l)?l.status:l,h=JSON.stringify(t.modifiedValues)!==JSON.stringify(e.modifiedValues),f=(0,c.default)(null===a||void 0===a?void 0:a.cellElement).hasClass(this.addWidgetPrefix(P));return!!(r!==d&&h||u.isValid&&f)||this.callBase.apply(this,arguments)}}},views:{rowsView:{updateFreeSpaceRowHeight:function(e){var t,n,i,o=this,a=o.element(),r=a&&a.find("."+o.addWidgetPrefix(V)+" .dx-overlay-content");if(o.callBase(e),r&&r.length&&(t=o._getRowElements(),i=o._getFreeSpaceRowElements(e),n=i.first(),n&&1===t.length&&(!n.is(":visible")||r.outerHeight()>n.outerHeight())))return i.show(),i.height(r.outerHeight()),!0},_formItemPrepared:function(e,t){var n=this;this.callBase.apply(this,arguments),(0,g.deferUpdate)(function(){var i=t.find(".dx-widget").first(),o=i.length&&!i.children().length;o||n.getController("validating").createValidator(e,i)})},_cellPrepared:function(e,t){this.getController("editing").isFormEditMode()||this.getController("validating").createValidator(t,e),this.callBase.apply(this,arguments)}}}}};t.default=re,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=n(4),s=n(485),l=i(n(37)),u=n(3),c=n(6),d=n(30),h=i(n(88)),f=i(n(23)),p=n(24),g=n(1),_="table",m="bottom-load-panel",v="table-content",y="group-space",b="content",x="dx-row",w="dx-freespace-row",C="dx-column-lines",k="dx-virtual-row",S="infinite",D="virtual",I="standard",T=25e4,O=300,E=function(e){return e.option("scrolling.mode")===D},A=function(e){return e.option("scrolling.mode")===S},P=function(e){var t=e.option("scrolling.rowRenderingMode");return t===D||t!==I&&void 0},R=function(e,t,n,i){for(var o=0;o<t+1;o++){var a=e[n?e.length-1-o:o];a&&!i(a,o===t,n)&&t++}return t},M=function(e,t){return"data"===e.rowType&&!e.isNewRow||"group"===e.rowType&&t.isGroupItemCountable(e.data)},B=function(){var e=function(e){var t=e._virtualScrollController.beginPageIndex(-1);E(e)&&(t<0||e.viewportSize()>=0&&e.getViewportItemIndex()>=0&&(t*e.pageSize()>e.getViewportItemIndex()||t*e.pageSize()+e.itemsCount()<e.getViewportItemIndex()+e.viewportSize())&&e._dataSource.isLoading()?e._isLoading||(e._isLoading=!0,e.loadingChanged.fire(!0)):e._isLoading&&(e._isLoading=!1,e.loadingChanged.fire(!1)))},t={init:function(t){var n=this;n.callBase.apply(n,arguments),n._items=[],n._isLoaded=!0,n._virtualScrollController=new s.VirtualScrollController(n.component,{pageSize:function(){return n.pageSize()},totalItemsCount:function(){return n.totalItemsCount()},hasKnownLastPage:function(){return n.hasKnownLastPage()},pageIndex:function(e){return t.pageIndex(e)},isLoading:function(){return t.isLoading()&&!n.isCustomLoading()},pageCount:function(){return n.pageCount()},load:function(){return t.load()},updateLoading:function(){e(n)},itemsCount:function(){return n.itemsCount(!0)},items:function(){return t.items()},viewportItems:function(e){return e&&(n._items=e),n._items},onChanged:function(e){n.changed.fire(e)},changingDuration:function(e){return n.isLoading()?O:n._renderTime||0}})},_handleLoadingChanged:function(e){E(this)&&!this._isLoadingAll||(this._isLoading=e,this.callBase.apply(this,arguments)),e?this._startLoadTime=new Date:this._startLoadTime=void 0},_handleLoadError:function(){var e=this;e._isLoading=!1,e.loadingChanged.fire(!1),e.callBase.apply(e,arguments)},_handleDataChanged:function(e){var t=this.callBase.bind(this);this._virtualScrollController.handleDataChanged(t,e)},_customizeRemoteOperations:function(e,t){var n=this;!n.option("legacyRendering")&&E(n)&&!t.reload&&t.skip&&n._renderTime<n.option("scrolling.renderingThreshold")&&(e.delay=void 0),n.callBase.apply(n,arguments)},items:function(){return this._items},itemsCount:function(e){return e?this.callBase():this._virtualScrollController.itemsCount()},load:function(e){return e?this.callBase(e):this._virtualScrollController.load()},isLoading:function(){return this._isLoading},isLoaded:function(){return this._dataSource.isLoaded()&&this._isLoaded},resetPagesCache:function(e){e||this._virtualScrollController.reset(!0),this.callBase.apply(this,arguments)},_changeRowExpandCore:function(){var t=this.callBase.apply(this,arguments);return this.resetPagesCache(),e(this),t},reload:function(){this._dataSource.pageIndex(this.pageIndex());var e=this._virtualScrollController;if(e){var t=new c.Deferred;return this.callBase.apply(this,arguments).done(function(n){var i=e._delayDeferred;i?i.done(t.resolve).fail(t.reject):t.resolve(n)}).fail(t.reject),t}return this.callBase.apply(this,arguments)},refresh:function(t,n){var i=this,o=t.storeLoadOptions,a=i._dataSource;return n.reload?(i._virtualScrollController.reset(),a.items().length=0,i._isLoaded=!1,e(i),i._isLoaded=!0,A(i)?(i.pageIndex(0),a.pageIndex(0),o.pageIndex=0,t.pageIndex=0,o.skip=0):(a.pageIndex(i.pageIndex()),a.paginate()&&(t.pageIndex=i.pageIndex(),o.skip=i.pageIndex()*i.pageSize()))):A(i)&&o.skip&&i._skipCorrection<0&&(o.skip+=i._skipCorrection),i.callBase.apply(i,arguments)},dispose:function(){this._virtualScrollController.dispose(),this.callBase.apply(this,arguments)}};return["virtualItemsCount","getContentOffset","getVirtualContentSize","setContentSize","setViewportPosition","getViewportItemIndex","setViewportItemIndex","getItemIndexByPosition","viewportSize","viewportItemSize","getItemSize","getItemSizes","pageIndex","beginPageIndex","endPageIndex","loadIfNeed"].forEach(function(e){t[e]=function(){var t=this._virtualScrollController;return t[e].apply(t,arguments)}}),t}(),V=function(){var e=function(e,t){var n=function(e){return(0,o.default)(e).parent("."+t).get(0)},i=e.toArray().map(n).filter(function(e){return e});i.length&&(e=(0,o.default)(i));for(var a=t===w?e.length-1:e.length,r=0;r<a;r++)e.eq(r).remove()};return{init:function(){var e,t=this,n=this.getController("data");this.callBase(),n.pageChanged.add(function(){t.scrollToPage(n.pageIndex())}),null===(e=n.stateLoaded)||void 0===e?void 0:e.add(function(){t._scrollToCurrentPageOnResize()}),this._scrollToCurrentPageOnResize()},_scrollToCurrentPageOnResize:function(){var e=this,t=this.getController("data");if(!this.option("legacyRendering")&&t.pageIndex()>0){var n=function n(){e.resizeCompleted.remove(n),e.scrollToPage(t.pageIndex())};this.resizeCompleted.add(n)}},scrollToPage:function(e){var t,n=this,i=n._dataController,o=i?i.pageSize():0;if(E(n)||A(n)){var a=i.getItemSize(),r=i.getItemSizes(),s=e*o;t=s*a;for(var l in r)l<s&&(t+=r[l]-a)}else t=0;n.scrollTo({y:t,x:n._scrollLeft})},renderDelayedTemplates:function(e){this._updateContentPosition(!0),this.callBase.apply(this,arguments)},_renderCore:function(e){var t=this,n=new Date;t.callBase.apply(t,arguments);var i=t._dataController._dataSource;if(i&&e){var o=e.items?e.items.length:20,a=t._dataController.viewportSize()||20;P(t)?i._renderTime=(new Date-n)*a/o:i._renderTime=new Date-n}},_getRowElements:function(e){var t=this.callBase(e);return t&&t.not("."+k)},_renderContent:function(e,t){var n=this,i=n._dataController.virtualItemsCount();return i&&n.option("legacyRendering")?((0,a.hasWindow)()&&t.addClass(n.addWidgetPrefix(v)),e.children().length?e.children().first().replaceWith(t):e.append(t),1===e.children("table").length&&(e.append(n._createTable()),n._contentHeight=0),e):n.callBase.apply(n,arguments)},_removeRowsElements:function(e,t,n){var i=this._getRowElements(e).toArray();i="append"===n?i.slice(0,t):i.slice(-t);var a=this.getController("errorHandling");i.map(function(e){var t=(0,o.default)(e);a&&a.removeErrorRow(t.next()),t.remove()})},_restoreErrorRow:function(e){var t=this.getController("editing");t&&t.hasChanges()&&this._getRowElements(e).each(function(e,n){var i=(0,o.default)(n).data("options");if(i){var a=t.getChangeByKey(i.key);a&&t._showErrorRow(a)}})},_updateContent:function(t,n){var i,o=this,a=o._findContentElement(),r=n&&n.changeType;if("append"===r||"prepend"===r){var s=a.children().first(),l=o._getBodies(t);o.option("legacyRendering")||1!==l.length?l["append"===r?"appendTo":"prependTo"](s):o._getBodies(s)["append"===r?"append":"prepend"](l.children()),t.remove(),i=o._getFreeSpaceRowElements(s),e(i,w),n.removeCount&&o._removeRowsElements(s,n.removeCount,r),o._restoreErrorRow(s)}else o.callBase.apply(o,arguments);o._updateBottomLoading()},_addVirtualRow:function(e,t,n,i){if(i){var o=this._createEmptyRow(k,t,i);o=this._wrapRowIfNeed(e,o),this._appendEmptyRow(e,o,n)}},_getRowHeights:function(){var e=this._getRowElements(this._tableElement).toArray().map(function(e){return(0,p.getBoundingRect)(e).height});return e},_correctRowHeights:function(e){for(var t=this._dataController,n=t._dataSource,i=[],o=t.getVisibleRows(),a=0,r=!0,s=0;s<e.length;s++){var l=o[s];(0,g.isDefined)(l)&&(M(l,n)&&(r?r=!1:(i.push(a),a=0)),a+=e[s])}return a>0&&i.push(a),i},_updateContentPosition:function(t){var n=this,i=n._dataController,a=n._rowHeight||20;if(i.viewportItemSize(a),n.option("legacyRendering")||!E(n)&&!P(n))(0,r.deferUpdate)(function(){n._updateContentPositionCore()});else{if(!t){var s=n._getRowHeights(),l=n._correctRowHeights(s);i.setContentSize(l)}var u=i.getContentOffset("begin"),c=i.getContentOffset("end"),d=n.getTableElements(),h=d.children("tbody").children("."+k);e(h,k),d.each(function(e){var t=e>0;n._isFixedTableRendering=t,n._addVirtualRow((0,o.default)(this),t,"top",u),n._addVirtualRow((0,o.default)(this),t,"bottom",c),n._isFixedTableRendering=!1})}},_updateContentPositionCore:function(){var e,t,n,i=this,o=i._rowHeight||20,a=i._dataController.virtualItemsCount();if(a){var s=i._findContentElement();t=s.children(),n=t.eq(0);var l=t.eq(1);i._contentTableHeight=n[0].offsetHeight,i._dataController.viewportItemSize(o),i._dataController.setContentSize(i._contentTableHeight),e=i._dataController.getVirtualContentSize();var u=i._dataController.getContentOffset();(0,r.deferRender)(function(){(0,d.move)(n,{left:0,top:u});var t=i._contentHeight!==e||0===e||!i._isTableLinesDisplaysCorrect(l)||!i._isColumnElementsEqual(n.find("col"),l.find("col"));t&&(i._contentHeight=e,i._renderVirtualTableContent(l,e))})}},_isTableLinesDisplaysCorrect:function(e){var t=e.find("."+C).length>0;return t===this.option("showColumnLines")},_isColumnElementsEqual:function(e,t){var n=e.length===t.length;return n&&(0,u.each)(e,function(e,i){if(i.style.width!==t[e].style.width)return n=!1}),n},_renderVirtualTableContent:function(e,t){for(var n=this,i=n._columnsController.getVisibleColumns(),o=n._createColGroup(i).prop("outerHTML"),a="",r=n.option("showColumnLines")?C:"",s=function(e){return"<tr style='height:"+e+"px;' class='"+w+" "+x+" "+r+"' >"+a+"</tr>"},l=0;l<i.length;l++){var u=n._getCellClasses(i[l]),c=u.length?" class='"+u.join(" ")+"'":"";a+="<td"+c+"/>"}for(;t>T;)o+=s(T),t-=T;o+=s(t),e.addClass(n.addWidgetPrefix(_)),e.html(o)},_getCellClasses:function(e){var t=[],n=e.cssClass,i="expand"===e.command;return n&&t.push(n),i&&t.push(this.addWidgetPrefix(y)),t},_findBottomLoadPanel:function(e){var t=e||this.element(),n=t&&t.find("."+this.addWidgetPrefix(m));if(n&&n.length)return n},_updateBottomLoading:function(){var e=this,t=e.option("scrolling.mode"),n=t===D,i=t===S,a=!e._dataController.hasKnownLastPage()&&e._dataController.isLoaded()&&(n||i),r=e._findContentElement(),s=e._findBottomLoadPanel(r);
a?s||(0,o.default)("<div>").addClass(e.addWidgetPrefix(m)).append(e._createComponent((0,o.default)("<div>"),h.default).$element()).appendTo(r):s&&s.remove()},_handleScroll:function(e){var t=this;t._hasHeight&&t._rowHeight&&t._dataController.setViewportPosition(e.scrollOffset.top),t.callBase.apply(t,arguments)},_needUpdateRowHeight:function(e){var t=this;return t.callBase.apply(t,arguments)||e>0&&t.option("scrolling.mode")===S&&t.option("scrolling.rowRenderingMode")!==D},_updateRowHeight:function(){var e=this;if(e.callBase.apply(e,arguments),e._rowHeight){e._updateContentPosition();var t=e._hasHeight?e.element().outerHeight():(0,o.default)((0,a.getWindow)()).outerHeight();e._dataController.viewportSize(Math.ceil(t/e._rowHeight))}},updateFreeSpaceRowHeight:function(){var e=this.callBase.apply(this,arguments);return e&&this._updateContentPosition(),e},setLoading:function(e,t){var n=this,i=n.callBase,o=n._dataController,a=o.pageIndex()>0&&o.isLoaded()&&!!n._findBottomLoadPanel();a&&(e=!1),i.call(n,e,t)},_resizeCore:function(){var e=this,t=e.element();e.callBase(),e.component.$element()&&!e._windowScroll&&t.closest((0,a.getWindow)().document).length&&(e._windowScroll=(0,s.subscribeToExternalScrollers)(t,function(t){!e._hasHeight&&e._rowHeight&&e._dataController.setViewportPosition(t)},e.component.$element()),e.on("disposing",function(){e._windowScroll.dispose()})),e.loadIfNeed()},loadIfNeed:function(){var e=this._dataController;e&&e.loadIfNeed&&e.loadIfNeed()},setColumnWidths:function(e){var t,n=this.getScrollable();this.callBase.apply(this,arguments),"virtual"===this.option("scrolling.mode")&&(t=n?n.$content():this.element(),this.callBase(e,t.children("."+this.addWidgetPrefix(b)).children(":not(."+this.addWidgetPrefix(v)+")")))},dispose:function(){clearTimeout(this._scrollTimeoutID),this.callBase()}}}(),F={defaultOptions:function(){return{scrolling:{timeout:300,updateTimeout:300,minTimeout:0,renderingThreshold:100,removeInvisiblePages:!0,rowPageSize:5,mode:"standard",preloadEnabled:!1,rowRenderingMode:"standard",loadTwoPagesOnStart:!1}}},extenders:{dataSourceAdapter:B,controllers:{data:function(){var e={_refreshDataSource:function(){var e=this.callBase.apply(this,arguments)||(new c.Deferred).resolve().promise();return e.done(this.initVirtualRows.bind(this)),e},getRowPageSize:function(){var e=this.option("scrolling.rowPageSize"),t=this.pageSize();return t&&t<e?t:e},_applyFilter:function(){return this.setViewportPosition(0),this.callBase.apply(this,arguments)},reload:function(){var e=this,t=e._rowsScrollController||e._dataSource,n=t&&t.getItemIndexByPosition(),i=this.callBase.apply(this,arguments);return i&&i.done(function(){if(E(e)||P(e)){var t=e.getRowIndexOffset(),i=Math.floor(n)-t,a=e.component,r=a.getScrollable&&a.getScrollable(),s=e.dataSource().operationTypes().sorting;if(r&&!e.option("legacyRendering")&&!s){var l=a.getRowElement(i),u=l&&l[0]&&(0,o.default)(l[0]),c=u&&u.position().top,d=f.default.mozilla||f.default.msie?1:0;c>d&&(c=Math.round(c+u.outerHeight()*(n%1)),r.scrollTo({y:c}))}}})},initVirtualRows:function(){var e=this,t=P(e);if("virtual"!==e.option("scrolling.mode")&&t!==!0||t===!1||e.option("legacyRendering")||!e.option("scrolling.rowPageSize"))return e._visibleItems=null,void(e._rowsScrollController=null);e._rowPageIndex=Math.ceil(e.pageIndex()*e.pageSize()/e.getRowPageSize()),e._visibleItems=[];var n=function(t){return M(t,e._dataSource)};e._rowsScrollController=new s.VirtualScrollController(e.component,{pageSize:function(){return e.getRowPageSize()},totalItemsCount:function(){return E(e)?e.totalItemsCount():e._items.filter(n).length},hasKnownLastPage:function(){return!0},pageIndex:function(t){return void 0!==t&&(e._rowPageIndex=t),e._rowPageIndex},isLoading:function(){return e.isLoading()},pageCount:function e(){var e=Math.ceil(this.totalItemsCount()/this.pageSize());return e?e:1},load:function(){e._rowsScrollController.pageIndex()>=this.pageCount()&&(e._rowPageIndex=this.pageCount()-1,e._rowsScrollController.pageIndex(e._rowPageIndex)),!e._rowsScrollController._dataSource.items().length&&this.totalItemsCount()||e._rowsScrollController.handleDataChanged(function(t){t=t||{},t.changeType=t.changeType||"refresh",t.items=t.items||e._visibleItems,e._visibleItems.forEach(function(e,t){e.rowIndex=t}),e._fireChanged(t)})},updateLoading:function(){},itemsCount:function(){return e._rowsScrollController._dataSource.items().filter(n).length},correctCount:function(e,t,i){return R(e,t,i,function(e,t,i){return e.isNewRow?t&&!i:t&&i?!e.isNewRow:n(e)})},items:function(t){var i=e.dataSource(),o=i&&i.virtualItemsCount(),a=o?o.begin:0,r=e.getRowPageSize(),s=e._rowPageIndex*r-a,l=r,u=e._items;return s<0?[]:(s&&(s=this.correctCount(u,s),u=u.slice(s)),l&&(l=this.correctCount(u,l),u=u.slice(0,l)),t?u.filter(n):u)},viewportItems:function(t){return t&&(e._visibleItems=t),e._visibleItems},onChanged:function(){},changingDuration:function(t){var n=e.dataSource();return n.isLoading()?O:(null===n||void 0===n?void 0:n._renderTime)||0}},(!0)),e._rowsScrollController.positionChanged.add(function(){var t;null===(t=e._dataSource)||void 0===t?void 0:t.setViewportItemIndex(e._rowsScrollController.getViewportItemIndex())}),e.isLoaded()&&e._rowsScrollController.load()},_updateItemsCore:function(e){var t=this,n=this.getRowIndexDelta();this.callBase.apply(this,arguments);var i=this._rowsScrollController;if(i){var o=this._visibleItems,a="refresh"===e.changeType||e.isLiveUpdate;if("append"===e.changeType&&e.items&&!e.items.length)return;a||"append"===e.changeType||"prepend"===e.changeType?(e.cancel=!0,a&&i.reset(!0),i.load()):("update"===e.changeType?e.rowIndices.forEach(function(t,n){var i=e.changeTypes[n],a=e.items[n];"update"===i?o[t]=a:"insert"===i?o.splice(t,0,a):"remove"===i&&o.splice(t,1)}):(o.forEach(function(e,i){o[i]=t._items[i+n]||o[i]}),e.items=o),o.forEach(function(e,t){e.rowIndex=t}))}},_applyChange:function(e){var t=this,n=e.items,i=e.changeType,o=e.removeCount;if(o){var a="prepend"===i;o=R(t._items,o,a,function(e,n){return"data"===e.rowType&&!e.isNewRow||"group"===e.rowType&&(t._dataSource.isGroupItemCountable(e.data)||n)}),e.removeCount=o}switch(i){case"prepend":t._items.unshift.apply(t._items,n),o&&t._items.splice(-o);break;case"append":t._items.push.apply(t._items,n),o&&t._items.splice(0,o);break;default:t.callBase(e)}},items:function(e){return e?this._items:this._visibleItems||this._items},getRowIndexDelta:function(){var e=this._visibleItems,t=0;return e&&e[0]&&(t=this._items.indexOf(e[0])),t<0?0:t},getRowIndexOffset:function(e){var t=0,n=this.dataSource(),i=this._rowsScrollController;return i&&!e?t=i.beginPageIndex()*i._dataSource.pageSize():"virtual"===this.option("scrolling.mode")&&n&&(t=n.beginPageIndex()*n.pageSize()),t},viewportSize:function(){var e=this._rowsScrollController;e&&e.viewportSize.apply(e,arguments);var t=this._dataSource;return t&&t.viewportSize.apply(t,arguments)},viewportItemSize:function(){var e=this._rowsScrollController;e&&e.viewportItemSize.apply(e,arguments);var t=this._dataSource;return t&&t.viewportItemSize.apply(t,arguments)},setViewportPosition:function(){var e=this._rowsScrollController,t=this._dataSource;e?e.setViewportPosition.apply(e,arguments):null===t||void 0===t?void 0:t.setViewportPosition.apply(t,arguments)},setContentSize:function(e){var t=this._rowsScrollController;t&&t.setContentSize(e);var n=this._dataSource;return n&&n.setContentSize(e)},loadIfNeed:function(){var e=this._rowsScrollController;e&&e.loadIfNeed();var t=this._dataSource;return t&&t.loadIfNeed()},getItemSize:function(){var e=this._rowsScrollController;if(e)return e.getItemSize.apply(e,arguments);var t=this._dataSource;return t&&t.getItemSize.apply(t,arguments)},getItemSizes:function(){var e=this._rowsScrollController;if(e)return e.getItemSizes.apply(e,arguments);var t=this._dataSource;return t&&t.getItemSizes.apply(t,arguments)},getContentOffset:function(){var e=this._rowsScrollController;if(e)return e.getContentOffset.apply(e,arguments);var t=this._dataSource;return t&&t.getContentOffset.apply(t,arguments)},refresh:function(e){var t=this._dataSource;return t&&e&&e.load&&A(this)&&t.resetCurrentTotalCount(),this.callBase.apply(this,arguments)},dispose:function(){var e=this._rowsScrollController;e&&e.dispose(),this.callBase.apply(this,arguments)}};return l.default.proxyMethod(e,"virtualItemsCount"),l.default.proxyMethod(e,"getVirtualContentSize"),l.default.proxyMethod(e,"setViewportItemIndex"),e}(),resizing:{resize:function(){var e,t=this,n=t.callBase;if(t.option("legacyRendering")||!E(t)&&!P(t))e=n.apply(t);else{clearTimeout(t._resizeTimeout);var i=new Date-t._lastTime,o=t.option("scrolling.updateTimeout");t._lastTime&&i<o?(e=new c.Deferred,t._resizeTimeout=setTimeout(function(){n.apply(t).done(e.resolve).fail(e.reject),t._lastTime=new Date},o),t._lastTime=new Date):(e=n.apply(t),t._dataController.isLoaded()&&(t._lastTime=new Date))}return e},dispose:function(){this.callBase.apply(this,arguments),clearTimeout(this._resizeTimeout)}}},views:{rowsView:V}}};t.default=F,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e.msie?4e6:e.mozilla?8e6:15e6/x((0,s.getWindow)())}function a(e,t,n){function i(t){var n=t.element?t.$element():t,i=d.default.offset(n);return i?t.scrollTop()-(i.top-e.offset().top):e.offset().top}function o(e){return function(){var n=e.scrollTop()-i(e);n=n>0?n:0,t(n)}}function a(e){var t="#document"===e.get(0).nodeName,n=e.data("dxScrollable"),a=g;if(n||(n=t&&(0,r.default)((0,s.getWindow)())||"auto"===e.css("overflowY")&&e,a=l.default,n)){var u=o(n);a.on(n,"scroll",u),f.push(function(e){var t=i(n),o=n.scrollTo?"scrollTo":"scrollTop";e-t>=0&&n[o](e+t)}),c.push(n),p.push(function(){a.off(n,"scroll",u)})}}var u,c=[],f=[],p=[];n=n||e;var g={on:function(e,t,n){e.on("scroll",n)},off:function(e,t,n){e.off("scroll",n)}};for(u=n.parent();u.length;u=u.parent())a(u);return{scrollTo:function(e){(0,h.each)(f,function(t,n){n(e)})},dispose:function(){(0,h.each)(p,function(e,t){t()})}}}t.getContentHeightLimit=o,t.subscribeToExternalScrollers=a,t.VirtualScrollController=t.getPixelRatio=void 0;var r=i(n(2)),s=n(7),l=i(n(5)),u=i(n(23)),c=n(1),d=i(n(84)),h=n(3),f=i(n(15)),p=n(6),g=i(n(27)),_="infinite",m="virtual",v=function(e){return e.option("scrolling.mode")===m||e._isVirtual},y=function(e){return e.option("scrolling.mode")===_&&!e._isVirtual},b=function(e){return e.option("scrolling.loadTwoPagesOnStart")||e._isVirtual||e._viewportItemIndex>0},x=function(e){return e.devicePixelRatio||1};t.getPixelRatio=x;var w=f.default.inherit(function(){function e(e){return e._cache.length?e._cache[0].pageIndex:-1}function t(e){return e._cache.length?e._cache[e._cache.length-1].pageIndex:-1}var n=function(e){var t=e._dataSource.pageSize(),n=e.option("scrolling.preventPreload");if(n)return 0;var i=e._viewportSize;if(v(e)&&!e.option("legacyRendering")&&e.option("scrolling.removeInvisiblePages")){i=0;for(var o=e._viewportSize*e._viewportItemSize,a=e.getContentOffset(),r=e._position||0,s=e.virtualItemsCount(),l=e._dataSource.totalItemsCount(),u=s.begin;u<l&&!(a>=r+o);u++){var c=e._itemSizes[u]||e._viewportItemSize;a+=c,a>=r&&i++}}return t&&i>0?Math.ceil(i/t):1},i=function(e,t){var i=e.option("scrolling.preloadEnabled"),o=n(e);return o&&(t?o=i?1:0:(i&&o++,!y(e)&&b(e)||o--)),o},r=function(n){var o=-1,a=e(n),r=n._dataSource;if(a<0)o=n._pageIndex;else if(n._cache[n._pageIndex-a]){if(a>=0&&n._viewportSize>=0){if(a>0){var s=t(n)+1===r.pageCount()&&n._cache.length<i(n)+1,l=s||n._pageIndex===a&&i(n,!0);l&&(o=a-1)}if(o<0){var u=a+n._cache.length<=n._pageIndex+i(n);u&&(o=a+n._cache.length)}}}else o=n._pageIndex;return n._loadingPageIndexes[o]&&(o=-1),o},s=function(e,t,n){e._isChangedFiring=!0,t(n),e._isChangedFiring=!1},l=function(e,t,n){if(e._isDelayChanged)return e._isDelayChanged=!1,s(e,t,n),!0},d=function(e,t,n,i,o){var a=e._dataSource,r=a.items().slice(),l=(0,c.isObject)(n)?n:void 0,u="prepend"===n,d=a.viewportItems();n&&(0,c.isString)(n)&&!e._isDelayChanged&&(l={changeType:n,items:r},o&&(l.removeCount=o.itemsCount,l.removeCount&&a.correctCount&&(l.removeCount=a.correctCount(d,l.removeCount,u))));var h=o?o.itemsLength:0;h&&a.correctCount&&(h=a.correctCount(d,h,u)),"append"===n?(d.push.apply(d,r),o&&d.splice(0,h)):u?(d.unshift.apply(d,r),o&&d.splice(-h)):e._dataSource.viewportItems(r),a.updateLoading(),e._lastPageIndex=e.pageIndex(),e._isDelayChanged=i,i||s(e,t,l)},f=function(e,t){var n=e._dataSource;if(t===e.pageIndex()||!n.isLoading()&&t<n.pageCount()||!n.hasKnownLastPage()&&t===n.pageCount())return n.pageIndex(t),e._loadingPageIndexes[t]=!0,(0,p.when)(n.load()).always(function(){e._loadingPageIndexes[t]=!1})};return{ctor:function(e,t,n){var i=this;i._dataSource=t,i.component=e,i._pageIndex=i._lastPageIndex=t.pageIndex(),i._viewportSize=0,i._viewportItemSize=20,i._viewportItemIndex=-1,i._itemSizes={},i._sizeRatio=1,i._items=[],i._cache=[],i._isVirtual=n,i._loadingPageIndexes={},i.positionChanged=(0,g.default)()},getItemSizes:function(){return this._itemSizes},option:function(){return this.component.option.apply(this.component,arguments)},virtualItemsCount:function(){var t,n=this,i=0;if(v(n)){t=e(n),t<0&&(t=n._dataSource.pageIndex());var o=t*n._dataSource.pageSize();i=n._cache.length*n._dataSource.pageSize();var a=Math.max(0,n._dataSource.totalItemsCount()-i-o);return{begin:o,end:a}}},setViewportPosition:function(e){var t=this,n=new p.Deferred,i=Math.min(t.option("scrolling.timeout")||0,t._dataSource.changingDuration());return i<t.option("scrolling.renderingThreshold")&&(i=t.option("scrolling.minTimeout")||0),clearTimeout(t._scrollTimeoutID),i>0?t._scrollTimeoutID=setTimeout(function(){t._setViewportPositionCore(e),n.resolve()},i):(t._setViewportPositionCore(e),n.resolve()),n.promise()},getViewportPosition:function(){return this._position||0},getItemIndexByPosition:function(){for(var e=this,t=e._position,n=e.getItemSize(),i=0,o=0,a=Object.keys(e._itemSizes).concat(-1),r=0;r<a.length&&i<t;r++){var s=parseInt(a[r]),l=(t-i)/n;if(s<0||o+l<s){o+=l;break}l=s-o,i+=l*n,o+=l;var u=e._itemSizes[s];i+=u,o+=i<t?1:(t-i+u)/u}return Math.round(50*o)/50},_setViewportPositionCore:function(e){this._position=e;var t=this.getItemIndexByPosition(),n=this.setViewportItemIndex(t);return this.positionChanged.fire(),n},setContentSize:function(e){var t=this,n=Array.isArray(e)&&e,i=t.virtualItemsCount();if(n&&(e=n.reduce(function(e,t){return e+t},0)),t._contentSize=e,i){n&&n.forEach(function(e,n){t._itemSizes[i.begin+n]=e});var a=(i.begin+i.end+t.itemsCount())*t._viewportItemSize,r=o(u.default);a>r?t._sizeRatio=r/a:t._sizeRatio=1}},getItemSize:function(){return this._viewportItemSize*this._sizeRatio},getItemOffset:function(e,t){var n=this,i=n.virtualItemsCount(),o=e;if(!i)return 0;var a=0,r=n._dataSource.totalItemsCount();return Object.keys(n._itemSizes).forEach(function(i){o&&(t?i>=r-e:i<e)&&(a+=n._itemSizes[i],o--)}),Math.floor(a+o*n._viewportItemSize*n._sizeRatio)},getContentOffset:function(e){var t="end"===e,n=this.virtualItemsCount();return n?this.getItemOffset(t?n.end:n.begin,t):0},getVirtualContentSize:function(){var e=this,t=e.virtualItemsCount();return t?e.getContentOffset("begin")+e.getContentOffset("end")+e._contentSize:0},getViewportItemIndex:function(){return this._viewportItemIndex},setViewportItemIndex:function(e){var t,n=this,i=n._dataSource.pageSize(),o=n._dataSource.pageCount(),a=v(n),r=y(n),s=n._dataSource.totalItemsCount();if(n._viewportItemIndex=e,i&&(a||r)&&s>=0){if(n._viewportSize&&e+n._viewportSize>=s&&!n._isVirtual)if(n._dataSource.hasKnownLastPage()){t=o-1;var l=s%i;t>0&&l>0&&l<n._viewportSize&&t--}else t=o;else{t=Math.floor(e/i);var u=o-1;t=Math.max(t,0),t=Math.min(t,u)}return n.pageIndex(t),n.load()}},viewportItemSize:function(e){return void 0!==e&&(this._viewportItemSize=e),this._viewportItemSize},viewportSize:function(e){return void 0!==e&&(this._viewportSize=e),this._viewportSize},pageIndex:function(e){return v(this)||y(this)?(void 0!==e&&(this._pageIndex=e),this._pageIndex):this._dataSource.pageIndex(e)},beginPageIndex:function t(n){var t=e(this);return t<0&&(t=void 0!==n?n:this.pageIndex()),t},endPageIndex:function e(){var e=t(this);return e>0?e:this._lastPageIndex},pageSize:function(){return this._dataSource.pageSize()},load:function(){var e,t=this,n=t._dataSource;if(v(t)||y(t)){var i=r(t);if(i>=0){var o=f(t,i);o&&(e=new p.Deferred,o.done(function(){var n=t._delayDeferred;n?n.done(e.resolve).fail(e.reject):e.resolve()}).fail(e.reject),n.updateLoading())}}else e=n.load();return e||t._lastPageIndex===t.pageIndex()||t._dataSource.onChanged({changeType:"pageIndex"}),e||(new p.Deferred).resolve()},loadIfNeed:function(){var e=this;if((v(e)||y(e))&&!e._dataSource.isLoading()&&(!e._isChangedFiring||e._isVirtual)){var t=e.getViewportPosition();t>0?e._setViewportPositionCore(t):e.load()}},handleDataChanged:function(n,o){var a,r,u=this,c=u._dataSource,h=u._cache.length;if(o&&o.changes)s(u,n,o);else if(v(u)||y(u)){var f=e(u);if(f>=0&&(v(u)&&f+u._cache.length!==c.pageIndex()&&f-1!==c.pageIndex()&&(h=0,u._cache=[]),y(u)))if(0===c.pageIndex())u._cache=[];else if(c.pageIndex()<t(u))return void s(u,n,{changeType:"append",items:[]});var p={pageIndex:c.pageIndex(),itemsLength:c.items(!0).length,itemsCount:u.itemsCount(!0)};!u.option("legacyRendering")&&u.option("scrolling.removeInvisiblePages")&&v(u)?r=u._cache.length>Math.max(i(this)+(u.option("scrolling.preloadEnabled")?1:0),2):l(u,n,{isDelayed:!0});var g;f===c.pageIndex()+1?(r&&(g=u._cache.pop()),a="prepend",u._cache.unshift(p)):(r&&(g=u._cache.shift()),a="append",u._cache.push(p));var _=v(u)&&0===h&&b(u);d(u,n,u._cache.length>1?a:void 0,_,g),u._delayDeferred=u.load().done(function(){l(u,n)&&u.load()})}else d(u,n,o)},itemsCount:function e(t){var e=0;return!t&&v(this)?(0,h.each)(this._cache,function(){e+=this.itemsCount}):e=this._dataSource.itemsCount(),e},reset:function(e){this._loadingPageIndexes={},this._cache=[],e||(this._itemSizes={})},subscribeToWindowScrollEvents:function(e){var t=this;t._windowScroll=t._windowScroll||a(e,function(e){t.viewportItemSize()&&t.setViewportPosition(e)})},dispose:function(){clearTimeout(this._scrollTimeoutID),this._windowScroll&&this._windowScroll.dispose(),this._windowScroll=null},scrollTo:function(e){this._windowScroll&&this._windowScroll.scrollTo(e)}}}());t.VirtualScrollController=w},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return"onClick"===e.option("filterRow.applyFilter")}t.default=void 0;var a=i(n(2)),r=i(n(5)),s=n(1),l=n(0),u=n(8),c=n(3),d=i(n(44)),h=i(n(37)),f=i(n(10)),p=i(n(52)),g=i(n(68)),_=i(n(487)),m=n(128),v={"=":"filter-operation-equals","<>":"filter-operation-not-equals","<":"filter-operation-less","<=":"filter-operation-less-equal",">":"filter-operation-greater",">=":"filter-operation-greater-equal","default":"filter-operation-default",notcontains:"filter-operation-not-contains",contains:"filter-operation-contains",startswith:"filter-operation-starts-with",endswith:"filter-operation-ends-with",between:"filter-operation-between"},y={"=":"equal","<>":"notEqual","<":"lessThan","<=":"lessThanOrEqual",">":"greaterThan",">=":"greaterThanOrEqual",startswith:"startsWith",contains:"contains",notcontains:"notContains",endswith:"endsWith",between:"between"},b=700,x=1,w="filter-row",C="filter-range-overlay",k="filter-range-start",S="filter-range-end",D="dx-menu",I="dx-editor-with-menu",T="dx-editor-container",O="dx-editor-cell",E="dx-filter-menu",A="dx-apply-button",P="dx-highlight-outline",R="dx-focused",M="dx-cell-focus-disabled",B="dx-filter-range-content",V="dx-filter-modified",F="input:not([type='hidden'])",L=["date","datetime","number"],H=function(){function e(e,t){if(t)return o(e)&&void 0!==t.bufferedFilterValue?t.bufferedFilterValue:t.filterValue}var t=function(e){var t=e&&e.children(),n=t&&t.data("dxComponents"),i=n&&n.length&&t.data(n[0]);if(i instanceof p.default)return i},n=function(t,n){var i="",o="",a=e(t,n),r=h.default.getFormatOptionsByColumn(n,"filterRow");return Array.isArray(a)?(i=h.default.formatValue(a[0],r),o=h.default.formatValue(a[1],r),""!==o&&(i+=" - "+o)):(0,s.isDefined)(a)&&(i=h.default.formatValue(a,r)),i},i=function(e,t){if(t)return o(e)&&void 0!==t.bufferedSelectedFilterOperation?t.bufferedSelectedFilterOperation:t.selectedFilterOperation},d=function(e,t){return!(t&&L.indexOf(t.dataType)>=0&&Array.isArray(e))&&void 0!==e},A=function(t,n,o){var a=t._columnsController.columnOption(n),r=e(t,a),s=o.closest("."+t.addWidgetPrefix(C)).length,l=o.hasClass(t.addWidgetPrefix(k));return r&&Array.isArray(r)&&"between"===i(t,a)?l?r[0]:r[1]:!s&&d(r,a)?r:null},P=function(t,n,o,a){if("between"===i(t,o)){var r=e(t,o);return a.hasClass(t.addWidgetPrefix(k))?[n,Array.isArray(r)?r[1]:void 0]:[Array.isArray(r)?r[0]:r,n]}return n},V=function(e,t){var n=""===t.value?null:t.value,i=t.container,a=e._columnsController.columnOption(t.column.index),r=A(e,a.index,i);((0,s.isDefined)(r)||(0,s.isDefined)(n))&&(e._applyFilterViewController.setHighLight(i,r!==n),e._columnsController.columnOption(a.index,o(e)?"bufferedFilterValue":"filterValue",P(e,n,a,i),t.notFireEvent))};return{_updateEditorValue:function(e,n){var i=this,o=t(n);o&&o.option("value",A(i,e.index,n))},_columnOptionChanged:function(e){var t,o,r,s,l=this,u=e.optionNames;if(h.default.checkChanges(u,["filterValue","bufferedFilterValue","selectedFilterOperation","bufferedSelectedFilterOperation"])&&void 0!==e.columnIndex){var c=l._columnsController.getVisibleIndex(e.columnIndex),d=l._columnsController.columnOption(e.columnIndex);if(t=l._getCellElement(l.element().find("."+l.addWidgetPrefix(w)).index(),c)||(0,a.default)(),o=t.find("."+T).first(),u.filterValue||u.bufferedFilterValue){l._updateEditorValue(d,o);var f=t.find("."+l.addWidgetPrefix(C)).data("dxOverlay");f&&(r=f.$content().find("."+T),l._updateEditorValue(d,r.first()),l._updateEditorValue(d,r.last())),f&&f.option("visible")||l._updateFilterRangeContent(t,n(l,d))}return void((u.selectedFilterOperation||u.bufferedSelectedFilterOperation)&&c>=0&&d&&(s=t.find("."+D),s.length&&(l._updateFilterOperationChooser(s,d,o),"between"===i(l,d)?l._renderFilterRangeContent(t,d):o.find("."+B).length&&(l._renderEditor(o,l._getEditorOptions(o,d)),l._hideFilterRange()))))}l.callBase(e)},_renderCore:function(){this._filterRangeOverlayInstance=null,this.callBase.apply(this,arguments)},_resizeCore:function(){this.callBase.apply(this,arguments),this._filterRangeOverlayInstance&&this._filterRangeOverlayInstance.repaint()},isFilterRowVisible:function(){return this._isElementVisible(this.option("filterRow"))},isVisible:function(){return this.callBase()||this.isFilterRowVisible()},init:function(){this.callBase(),this._applyFilterViewController=this.getController("applyFilter")},_initFilterRangeOverlay:function(e,t){var o=this,s={},l=e.find(".dx-editor-container"),c=(0,a.default)("<div>").addClass(o.addWidgetPrefix(C)).appendTo(e);return o._createComponent(c,g.default,{height:"auto",shading:!1,showTitle:!1,focusStateEnabled:!1,closeOnTargetScroll:!1,closeOnOutsideClick:!0,animation:!1,position:{my:"top",at:"top",of:l.length&&l||e,offset:"0 -1"},contentTemplate:function(n){var i,l=(0,a.default)("<div>").addClass(T+" "+o.addWidgetPrefix(k)).appendTo(n);return t=o._columnsController.columnOption(t.index),i=o._getEditorOptions(l,t),i.sharedData=s,o._renderEditor(l,i),r.default.on(l.find(F),"keydown",function(t){var n=e.find("[tabindex]").not(t.target).first();"tab"===(0,u.normalizeKeyName)(t)&&t.shiftKey&&(t.preventDefault(),o._hideFilterRange(),n.length||(n=e.prev().find("[tabindex]").last()),r.default.trigger(n,"focus"))}),l=(0,a.default)("<div>").addClass(T+" "+o.addWidgetPrefix(S)).appendTo(n),i=o._getEditorOptions(l,t),i.sharedData=s,o._renderEditor(l,i),r.default.on(l.find(F),"keydown",function(t){"tab"!==(0,u.normalizeKeyName)(t)||t.shiftKey||(t.preventDefault(),o._hideFilterRange(),r.default.trigger(e.next().find("[tabindex]").first(),"focus"))}),(0,a.default)(n).addClass(o.getWidgetContainerClass())},onShown:function(e){var t=e.component.$content().find("."+T).first();r.default.trigger(t.find(F),"focus")},onHidden:function(){t=o._columnsController.columnOption(t.index),e.find("."+D).parent().addClass(I),"between"===i(o,t)&&(o._updateFilterRangeContent(e,n(o,t)),o.component.updateDimensions())}})},_updateFilterRangeOverlay:function(e){var t=this._filterRangeOverlayInstance;t&&t.option(e)},_showFilterRange:function(e,t){var n=this,i=e.children("."+n.addWidgetPrefix(C)),o=i.length&&i.data("dxOverlay");!o&&t&&(o=n._initFilterRangeOverlay(e,t)),o.option("visible")||(n._filterRangeOverlayInstance&&n._filterRangeOverlayInstance.hide(),n._filterRangeOverlayInstance=o,n._updateFilterRangeOverlay({width:e.outerWidth(!0)+x}),n._filterRangeOverlayInstance&&n._filterRangeOverlayInstance.show())},_hideFilterRange:function(){var e=this._filterRangeOverlayInstance;e&&e.hide()},getFilterRangeOverlayInstance:function(){return this._filterRangeOverlayInstance},_createRow:function(e){var t=this,n=this.callBase(e);return"filter"===e.rowType&&(n.addClass(this.addWidgetPrefix(w)),this.option("useLegacyKeyboardNavigation")||r.default.on(n,"keydown",function(e){return(0,m.selectView)("filterRow",t,e)})),n},_getRows:function(){var e=this.callBase();return this.isFilterRowVisible()&&e.push({rowType:"filter"}),e},_renderFilterCell:function(e,t){var n=this,o=t.column,r=(0,a.default)(e);n.component.option("showColumnHeaders")&&n.setAria("describedby",o.headerId,r),n.setAria("label",f.default.format("dxDataGrid-ariaFilterCell"),r),r.addClass(O);var s=(0,a.default)("<div>").appendTo(r),l=(0,a.default)("<div>").addClass(T).appendTo(s);if("between"===i(n,o))n._renderFilterRangeContent(r,o);else{var u=n._getEditorOptions(l,o);n._renderEditor(l,u)}var c=o.alignment;c&&"center"!==c&&r.find(F).first().css("textAlign",o.alignment),o.filterOperations&&o.filterOperations.length&&n._renderFilterOperationChooser(s,o,l)},_renderCellContent:function(e,t){var n=this,i=t.column;if("filter"===t.rowType)if(i.command)e.html("&nbsp;");else if(i.allowFiltering)return void n.renderTemplate(e,n._renderFilterCell.bind(n),t).done(function(){n._updateCell(e,t)});n.callBase(e,t)},_getEditorOptions:function(e,t){var n=this,o={editorOptions:{inputAttr:n._getFilterInputAccessibilityAttributes(t)}},a=(0,l.extend)(o,t,{value:A(n,t.index,e),parentType:"filterRow",showAllText:n.option("filterRow.showAllText"),updateValueTimeout:"onClick"===n.option("filterRow.applyFilter")?0:b,width:null,setValue:function(i,o){V(n,{column:t,value:i,container:e,notFireEvent:o})}});return"between"===i(n,t)&&(e.hasClass(n.addWidgetPrefix(k))?a.placeholder=n.option("filterRow.betweenStartText"):a.placeholder=n.option("filterRow.betweenEndText")),a},_getFilterInputAccessibilityAttributes:function(e){var t=f.default.format("dxDataGrid-ariaFilterCell");return this.component.option("showColumnHeaders")?{"aria-label":t,"aria-describedby":e.headerId}:{"aria-label":t}},_renderEditor:function(e,t){return e.empty(),this.getController("editorFactory").createEditor((0,a.default)("<div>").appendTo(e),t)},_renderFilterRangeContent:function(e,t){var i=this,o=e.find("."+T).first();o.empty();var s=(0,a.default)("<div>").addClass(B).attr("tabindex",this.option("tabIndex"));r.default.on(s,"focusin",function(){i._showFilterRange(e,t)}),s.appendTo(o),i._updateFilterRangeContent(e,n(i,t))},_updateFilterRangeContent:function(e,t){var n=e.find("."+B);n.length&&(""===t?n.html("&nbsp;"):n.text(t))},_updateFilterOperationChooser:function(e,n,a){var s,l=this,u=function(){var t=_.default.getInstance(e);t&&t.option("focusedElement",null),s&&l._focusEditor(a)};l._createComponent(e,_.default,{integrationOptions:{},activeStateEnabled:!1,selectionMode:"single",cssClass:l.getWidgetContainerClass()+" "+M+" "+E,showFirstSubmenuMode:"onHover",hideSubmenuOnMouseLeave:!0,items:[{disabled:!n.filterOperations||!n.filterOperations.length,icon:v[i(l,n)||"default"],selectable:!1,items:l._getFilterOperationMenuItems(n)}],onItemClick:function(e){var r=e.itemData.name,s=i(l,n),u=!1,c=o(l),d={};if(!(e.itemData.items||r&&r===s)){if(r?(d[c?"bufferedSelectedFilterOperation":"selectedFilterOperation"]=r,"between"!==r&&"between"!==s||(u="between"===r,d[c?"bufferedFilterValue":"filterValue"]=null)):(d[c?"bufferedFilterValue":"filterValue"]=null,d[c?"bufferedSelectedFilterOperation":"selectedFilterOperation"]=n.defaultSelectedFilterOperation||null),l._columnsController.columnOption(n.index,d),l._applyFilterViewController.setHighLight(a,!0),!r){var h=t(a);h&&"dxDateBox"===h.NAME&&!h.option("isValid")&&(h.reset(),h.option("isValid",!0))}u?l._showFilterRange(a.closest("."+O),n):l._focusEditor(a)}},onSubmenuShown:function(){s=l._isEditorFocused(a),l.getController("editorFactory").loseFocus()},onSubmenuHiding:function(){r.default.trigger(e,"blur"),u()},onContentReady:function(t){r.default.on(e,"blur",function(){var e=t.component;e._hideSubmenu(e._visibleSubmenu),u()})},rtlEnabled:l.option("rtlEnabled")})},_isEditorFocused:function(e){return e.hasClass(R)||e.parents("."+R).length},_focusEditor:function(e){this.getController("editorFactory").focus(e),r.default.trigger(e.find(F),"focus")},_renderFilterOperationChooser:function(e,t,n){var i,o=this;o.option("filterRow.showOperationChooser")&&(e.addClass(I),i=(0,a.default)("<div>").prependTo(e),o._updateFilterOperationChooser(i,t,n))},_getFilterOperationMenuItems:function(e){var t=this,n=[{}],o=t.option("filterRow"),a=o&&o.operationDescriptions||{};if(e.filterOperations&&e.filterOperations.length){var r=e.filterOperations.filter(function(e){return(0,s.isDefined)(y[e])});n=(0,c.map)(r,function(n){var o=y[n];return{name:n,selected:(i(t,e)||e.defaultFilterOperation)===n,text:a[o],icon:v[n]}}),n.push({name:null,text:o&&o.resetOperationText,icon:v.default})}return n},optionChanged:function(e){var t=this;switch(e.name){case"filterRow":case"showColumnLines":this._invalidate(!0,!0),e.handled=!0;break;default:t.callBase(e)}}}}(),z={skipCalculateColumnFilters:function(){return!1},_calculateAdditionalFilter:function(){if(this.skipCalculateColumnFilters())return this.callBase();var e=[this.callBase()],t=this._columnsController.getVisibleColumns(null,!0);return(0,c.each)(t,function(){if(this.allowFiltering&&this.calculateFilterExpression&&(0,s.isDefined)(this.filterValue)){var t=this.createFilterExpression(this.filterValue,this.selectedFilterOperation||this.defaultFilterOperation,"filterRow");e.push(t)}}),h.default.combineFilters(e)}},N=d.default.ViewController.inherit({_getHeaderPanel:function(){return this._headerPanel||(this._headerPanel=this.getView("headerPanel")),this._headerPanel},setHighLight:function(e,t){o(this)&&(e&&e.toggleClass(P,t)&&e.closest("."+O).toggleClass(V,t),this._getHeaderPanel().enableApplyButton(t))},applyFilter:function(){var e=this.getController("columns"),t=e.getColumns();e.beginUpdate();for(var n=0;n<t.length;n++){var i=t[n];void 0!==i.bufferedFilterValue&&(e.columnOption(n,"filterValue",i.bufferedFilterValue),i.bufferedFilterValue=void 0),void 0!==i.bufferedSelectedFilterOperation&&(e.columnOption(n,"selectedFilterOperation",i.bufferedSelectedFilterOperation),i.bufferedSelectedFilterOperation=void 0)}e.endUpdate(),this.removeHighLights()},removeHighLights:function(){if(o(this)){var e=this.getView("columnHeadersView").element();e.find("."+this.addWidgetPrefix(w)+" ."+P).removeClass(P),e.find("."+this.addWidgetPrefix(w)+" ."+V).removeClass(V),this._getHeaderPanel().enableApplyButton(!1)}}}),j={defaultOptions:function(){return{filterRow:{visible:!1,showOperationChooser:!0,showAllText:f.default.format("dxDataGrid-filterRowShowAllText"),resetOperationText:f.default.format("dxDataGrid-filterRowResetOperationText"),applyFilter:"auto",applyFilterText:f.default.format("dxDataGrid-applyFilterText"),operationDescriptions:{equal:f.default.format("dxDataGrid-filterRowOperationEquals"),notEqual:f.default.format("dxDataGrid-filterRowOperationNotEquals"),lessThan:f.default.format("dxDataGrid-filterRowOperationLess"),lessThanOrEqual:f.default.format("dxDataGrid-filterRowOperationLessOrEquals"),
greaterThan:f.default.format("dxDataGrid-filterRowOperationGreater"),greaterThanOrEqual:f.default.format("dxDataGrid-filterRowOperationGreaterOrEquals"),startsWith:f.default.format("dxDataGrid-filterRowOperationStartsWith"),contains:f.default.format("dxDataGrid-filterRowOperationContains"),notContains:f.default.format("dxDataGrid-filterRowOperationNotContains"),endsWith:f.default.format("dxDataGrid-filterRowOperationEndsWith"),between:f.default.format("dxDataGrid-filterRowOperationBetween"),isBlank:f.default.format("dxFilterBuilder-filterOperationIsBlank"),isNotBlank:f.default.format("dxFilterBuilder-filterOperationIsNotBlank")},betweenStartText:f.default.format("dxDataGrid-filterRowOperationBetweenStartText"),betweenEndText:f.default.format("dxDataGrid-filterRowOperationBetweenEndText")}}},controllers:{applyFilter:N},extenders:{controllers:{data:z,columnsResizer:{_startResizing:function(){var e=this;if(e.callBase.apply(e,arguments),e.isResizing()){var t=e._columnHeadersView.getFilterRangeOverlayInstance();if(t){var n=t.$element().closest("td").index();n!==e._targetPoint.columnIndex&&n!==e._targetPoint.columnIndex+1||t.$content().hide()}}},_endResizing:function(){var e,t=this;if(t.isResizing()){var n=t._columnHeadersView.getFilterRangeOverlayInstance();n&&(e=n.$element().closest("td"),t._columnHeadersView._updateFilterRangeOverlay({width:e.outerWidth(!0)+x}),n.$content().show())}t.callBase.apply(t,arguments)}}},views:{columnHeadersView:H,headerPanel:{_getToolbarItems:function(){var e=this.callBase(),t=this._prepareFilterItem(e);return t.concat(e)},_prepareFilterItem:function(){var e=this,t=[];if(e._isShowApplyFilterButton()){var n=e.option("filterRow.applyFilterText"),i=e._columnsController.getColumns(),o=!i.filter(function(e){return void 0!==e.bufferedFilterValue}).length,r=function(t){(0,a.default)(t.element).addClass(e._getToolbarButtonClass(A))},s=function(){e._applyFilterViewController.applyFilter()},l={widget:"dxButton",options:{icon:"apply-filter",disabled:o,onClick:s,hint:n,text:n,onInitialized:r},showText:"inMenu",name:"applyFilterButton",location:"after",locateInMenu:"auto",sortIndex:10};t.push(l)}return t},_isShowApplyFilterButton:function(){var e=this.option("filterRow");return e&&e.visible&&"onClick"===e.applyFilter},init:function(){this.callBase(),this._dataController=this.getController("data"),this._applyFilterViewController=this.getController("applyFilter")},enableApplyButton:function(e){this.setToolbarItemDisabled("applyFilterButton",!e)},isVisible:function(){return this.callBase()||this._isShowApplyFilterButton()},optionChanged:function(e){"filterRow"===e.name?(this._invalidate(),e.handled=!0):this.callBase(e)}}}}};t.default=j,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(488)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(5)),y=o(n(9)),b=n(4),x=n(18),w=n(3),C=n(1),k=n(0),S=n(293),D=n(8),I=o(n(25)),T=n(158),O=o(n(458)),E=o(n(68)),A=o(n(714)),P=o(n(38)),R=o(n(167)),M="dx-menu",B=M+"-vertical",V=M+"-horizontal",F=M+"-item",L=M+"-items-container",H=F+"-expanded",z="dx-context-menu",N=z+"-container-border",j="dx-context-menu-content-delimiter",W="dx-submenu",G="dx-state-disabled",q="dx-state-hover",$="dx-state-active",K=M+"-adaptive-mode",U=M+"-hamburger-button",Y=K+"-overlay-wrapper",Z="up",X="down",Q="left",J="right",ee="showSubmenu",te="nextItem",ne="prevItem",ie={show:50,hide:300},oe=["onSubmenuShowing","onSubmenuShown","onSubmenuHiding","onSubmenuHidden","onItemContextMenu","onItemClick","onSelectionChanged","onItemRendered"],ae=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,k.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{orientation:"horizontal",submenuDirection:"auto",showFirstSubmenuMode:{name:"onClick",delay:{show:50,hide:300}},hideSubmenuOnMouseLeave:!1,onSubmenuShowing:null,onSubmenuShown:null,onSubmenuHiding:null,onSubmenuHidden:null,adaptivityEnabled:!1})}},{key:"_setOptionsByReference",value:function(){l(_(t.prototype),"_setOptionsByReference",this).call(this),(0,k.extend)(this._optionsByReference,{animation:!0,selectedItem:!0})}},{key:"_itemElements",value:function(){var e=l(_(t.prototype),"_itemElements",this).call(this),n=this._submenuItemElements();return e.add(n)}},{key:"_submenuItemElements",value:function(){var e=[],t=".".concat(F),n=this._submenus.length&&this._submenus[0];return n&&n.itemsContainer()&&(e=n.itemsContainer().find(t)),e}},{key:"_focusTarget",value:function(){return this.$element()}},{key:"_isMenuHorizontal",value:function(){return"horizontal"===this.option("orientation")}},{key:"_moveFocus",value:function(e){var n,i,o,a,r=this._getAvailableItems(),s=this._isMenuHorizontal(),u=this._getActiveItem(!0);switch(e){case Z:i=s?ee:this._getItemsNavigationOperation(ne),n=s?u:r,o=this._getKeyboardNavigationAction(i,n),a=o();break;case X:i=s?ee:this._getItemsNavigationOperation(te),n=s?u:r,o=this._getKeyboardNavigationAction(i,n),a=o();break;case J:i=s?this._getItemsNavigationOperation(te):ee,n=s?r:u,o=this._getKeyboardNavigationAction(i,n),a=o();break;case Q:i=s?this._getItemsNavigationOperation(ne):ee,n=s?r:u,o=this._getKeyboardNavigationAction(i,n),a=o();break;default:return l(_(t.prototype),"_moveFocus",this).call(this,e)}a&&0!==a.length&&this.option("focusedElement",(0,x.getPublicElement)(a))}},{key:"_getItemsNavigationOperation",value:function(e){var t=e;return this.option("rtlEnabled")&&(t=e===ne?te:ne),t}},{key:"_getKeyboardNavigationAction",value:function(e,t){var n=b.noop;switch(e){case ee:t.hasClass(G)||(n=this._showSubmenu.bind(this,t));break;case te:n=this._nextItem.bind(this,t);break;case ne:n=this._prevItem.bind(this,t)}return n}},{key:"_clean",value:function(){l(_(t.prototype),"_clean",this).call(this),this.option("templatesRenderAsynchronously")&&clearTimeout(this._resizeEventTimer)}},{key:"_visibilityChanged",value:function(e){e&&(this._menuItemsWidth||this._updateItemsWidthCache(),this._dimensionChanged())}},{key:"_isAdaptivityEnabled",value:function(){return this.option("adaptivityEnabled")&&"horizontal"===this.option("orientation")}},{key:"_updateItemsWidthCache",value:function(){var e=this.$element().find("ul").first().children("li").children(".".concat(F));this._menuItemsWidth=this._getSummaryItemsWidth(e,!0)}},{key:"_dimensionChanged",value:function(){if(this._isAdaptivityEnabled()){var e=this.$element().outerWidth();this._toggleAdaptiveMode(this._menuItemsWidth>e)}}},{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._submenus=[]}},{key:"_initActions",value:function(){var e=this;this._actions={},(0,w.each)(oe,function(t,n){e._actions[n]=e._createActionByOption(n)})}},{key:"_initMarkup",value:function(){this._visibleSubmenu=null,this.$element().addClass(M),l(_(t.prototype),"_initMarkup",this).call(this),this.setAria("role","menubar")}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._initAdaptivity()}},{key:"_renderHamburgerButton",value:function(){return this._hamburger=new P.default((0,m.default)("<div>").addClass(U),{icon:"menu",activeStateEnabled:!1,onClick:this._toggleTreeView.bind(this)}),this._hamburger.$element()}},{key:"_toggleTreeView",value:function(e){(0,C.isPlainObject)(e)&&(e=!this._overlay.option("visible")),this._overlay.option("visible",e),this._toggleHamburgerActiveState(e)}},{key:"_toggleHamburgerActiveState",value:function(e){this._hamburger&&this._hamburger.$element().toggleClass($,e)}},{key:"_toggleAdaptiveMode",value:function(e){var t=this.$element().find(".".concat(V)),n=this.$element().find(".".concat(K));e?this._hideVisibleSubmenu():(this._treeView&&this._treeView.collapseAll(),this._overlay&&this._toggleTreeView(e)),t.toggle(!e),n.toggle(e)}},{key:"_removeAdaptivity",value:function(){this._$adaptiveContainer&&(this._toggleAdaptiveMode(!1),this._$adaptiveContainer.remove(),this._$adaptiveContainer=null,this._treeView=null,this._hamburger=null,this._overlay=null)}},{key:"_treeviewItemClickHandler",value:function(e){this._actions.onItemClick(e),e.node.children.length||this._toggleTreeView(!1)}},{key:"_getAdaptiveOverlayOptions",value:function(){var e=this,t=this.option("rtlEnabled"),n=t?"right":"left";return{maxHeight:function(){return(0,S.getElementMaxHeightByWindow)(e.$element())},deferRendering:!1,shading:!1,animation:!1,closeOnTargetScroll:!0,onHidden:function(){e._toggleHamburgerActiveState(!1)},height:"auto",closeOnOutsideClick:function(e){return!(0,m.default)(e.target).closest(".".concat(U)).length},position:{collision:"flipfit",at:"bottom "+n,my:"top "+n,of:this._hamburger.$element()}}}},{key:"_getTreeViewOptions",value:function(){var e=this,t={},n=["rtlEnabled","width","accessKey","activeStateEnabled","animation","dataSource","disabled","displayExpr","displayExpr","focusStateEnabled","hint","hoverStateEnabled","itemsExpr","items","itemTemplate","selectedExpr","selectionMode","tabIndex","visible"],i=["onItemContextMenu","onSelectionChanged"];return(0,w.each)(n,function(n,i){t[i]=e.option(i)}),(0,w.each)(i,function(n,i){t[i]=function(t){e._actions[i](t)}}),(0,k.extend)(t,{dataSource:this.getDataSource(),animationEnabled:!!this.option("animation"),onItemClick:this._treeviewItemClickHandler.bind(this),onItemExpanded:function(t){e._overlay.repaint(),e._actions.onSubmenuShown(t)},onItemCollapsed:function(t){e._overlay.repaint(),e._actions.onSubmenuHidden(t)},selectNodesRecursive:!1,selectByClick:this.option("selectByClick"),expandEvent:"click"})}},{key:"_initAdaptivity",value:function(){if(this._isAdaptivityEnabled()){this._$adaptiveContainer=(0,m.default)("<div>").addClass(K);var e=this._renderHamburgerButton();this._treeView=this._createComponent((0,m.default)("<div>"),R.default,this._getTreeViewOptions()),this._overlay=this._createComponent((0,m.default)("<div>"),E.default,this._getAdaptiveOverlayOptions()),this._overlay.$content().append(this._treeView.$element()).addClass(K).addClass(this.option("cssClass")),this._overlay._wrapper().addClass(Y),this._$adaptiveContainer.append(e),this._$adaptiveContainer.append(this._overlay.$element()),this.$element().append(this._$adaptiveContainer),this._updateItemsWidthCache(),this._dimensionChanged()}}},{key:"_getDelay",value:function(e){var t=this.option("showFirstSubmenuMode").delay;return(0,C.isDefined)(t)?(0,C.isObject)(t)?t[e]:t:ie[e]}},{key:"_keyboardHandler",value:function(e){return l(_(t.prototype),"_keyboardHandler",this).call(this,e,!!this._visibleSubmenu)}},{key:"_renderContainer",value:function(){var e=(0,m.default)("<div>");return e.appendTo(this.$element()).addClass(this._isMenuHorizontal()?V:B),l(_(t.prototype),"_renderContainer",this).call(this,e)}},{key:"_renderSubmenuItems",value:function(e,t){var n=this._createSubmenu(e,t);return this._submenus.push(n),this._renderBorderElement(t),n}},{key:"_getKeyboardListeners",value:function(){return l(_(t.prototype),"_getKeyboardListeners",this).call(this).concat(this._visibleSubmenu)}},{key:"_createSubmenu",value:function(e,t){var n=(0,m.default)("<div>").addClass(z).appendTo(t),i=this._getChildNodes(e),o=this._createComponent(n,A.default,(0,k.extend)(this._getSubmenuOptions(),{_dataAdapter:this._dataAdapter,_parentKey:e.internalFields.key,items:i,onHoverStart:this._clearTimeouts.bind(this),position:this.getSubmenuPosition(t)}));return this._attachSubmenuHandlers(t,o),o}},{key:"_getSubmenuOptions",value:function(){var e=this,t=(0,m.default)("<div>"),n=this._isMenuHorizontal();return{itemTemplate:this.option("itemTemplate"),target:t,orientation:this.option("orientation"),selectionMode:this.option("selectionMode"),cssClass:this.option("cssClass"),selectByClick:this.option("selectByClick"),hoverStateEnabled:this.option("hoverStateEnabled"),activeStateEnabled:this.option("activeStateEnabled"),focusStateEnabled:this.option("focusStateEnabled"),animation:this.option("animation"),showSubmenuMode:this.option("showSubmenuMode"),displayExpr:this.option("displayExpr"),disabledExpr:this.option("disabledExpr"),selectedExpr:this.option("selectedExpr"),itemsExpr:this.option("itemsExpr"),onFocusedItemChanged:function(t){t.component.option("visible")&&e.option("focusedElement",t.component.option("focusedElement"))},onSelectionChanged:this._nestedItemOnSelectionChangedHandler.bind(this),onItemClick:this._nestedItemOnItemClickHandler.bind(this),onItemRendered:this._nestedItemOnItemRenderedHandler.bind(this),onLeftFirstItem:n?null:this._moveMainMenuFocus.bind(this,ne),onLeftLastItem:n?null:this._moveMainMenuFocus.bind(this,te),onCloseRootSubmenu:this._moveMainMenuFocus.bind(this,n?ne:null),onExpandLastSubmenu:n?this._moveMainMenuFocus.bind(this,te):null}}},{key:"_getShowFirstSubmenuMode",value:function(){if(!this._isDesktopDevice())return"onClick";var e=this.option("showFirstSubmenuMode");return(0,C.isObject)(e)?e.name:e}},{key:"_moveMainMenuFocus",value:function(e){var t=this._getAvailableItems(),n=t.length,i=t.filter(".".concat(H)).eq(0),o=t.index(i);this._hideSubmenu(this._visibleSubmenu),o+=e===ne?-1:1,o>=n?o=0:o<0&&(o=n-1);var a=t.eq(o);this.option("focusedElement",(0,x.getPublicElement)(a))}},{key:"_nestedItemOnSelectionChangedHandler",value:function(e){var t=e.addedItems.length&&e.addedItems[0],n=A.default.getInstance(e.element),i=this._actions.onSelectionChanged;i(e),t&&this._clearSelectionInSubmenus(t[0],n),this._clearRootSelection(),this._setOptionWithoutOptionChange("selectedItem",t)}},{key:"_clearSelectionInSubmenus",value:function(e,t){var n=this,i=!arguments.length;(0,w.each)(this._submenus,function(e,o){var a=o._itemContainer(),r=!a.is(t&&t._itemContainer()),s=a.find(".".concat(n._selectedItemClass()));if(r&&s.length||i){s.removeClass(n._selectedItemClass());var l=n._getItemData(s);l&&(l.selected=!1),o._clearSelectedItems()}})}},{key:"_clearRootSelection",value:function(){var e=this.$element().find(".".concat(L)).first().children().children().filter(".".concat(this._selectedItemClass()));if(e.length){var t=this._getItemData(e);t.selected=!1,e.removeClass(this._selectedItemClass())}}},{key:"_nestedItemOnItemClickHandler",value:function(e){this._actions.onItemClick(e)}},{key:"_nestedItemOnItemRenderedHandler",value:function(e){this._actions.onItemRendered(e)}},{key:"_attachSubmenuHandlers",value:function(e,t){var n=this,i=t.getOverlayContent(),o=i.find(".".concat(W)),a=(0,D.addNamespace)(T.end,this.NAME+"_submenu");t.option({onShowing:this._submenuOnShowingHandler.bind(this,e,t),onShown:this._submenuOnShownHandler.bind(this,e,t),onHiding:this._submenuOnHidingHandler.bind(this,e,t),onHidden:this._submenuOnHiddenHandler.bind(this,e,t)}),(0,w.each)(o,function(t,i){v.default.off(i,a),v.default.on(i,a,null,n._submenuMouseLeaveHandler.bind(n,e))})}},{key:"_submenuOnShowingHandler",value:function(e,t){var n=e.children(".".concat(N));this._actions.onSubmenuShowing({rootItem:(0,x.getPublicElement)(e),submenu:t}),n.show(),e.addClass(H)}},{key:"_submenuOnShownHandler",value:function(e,t){this._actions.onSubmenuShown({rootItem:(0,x.getPublicElement)(e),submenu:t})}},{key:"_submenuOnHidingHandler",value:function(e,t,n){var i=e.children(".".concat(N)),o=n;o.rootItem=(0,x.getPublicElement)(e),o.submenu=t,this._actions.onSubmenuHiding(o),n=o,n.cancel||(this._visibleSubmenu===t&&(this._visibleSubmenu=null),i.hide(),e.removeClass(H))}},{key:"_submenuOnHiddenHandler",value:function(e,t){this._actions.onSubmenuHidden({rootItem:(0,x.getPublicElement)(e),submenu:t})}},{key:"_submenuMouseLeaveHandler",value:function(e,t){var n=(0,m.default)(t.relatedTarget).parents(".".concat(z))[0],i=this._getSubmenuByRootElement(e).getOverlayContent()[0];this.option("hideSubmenuOnMouseLeave")&&n!==i&&(this._clearTimeouts(),setTimeout(this._hideSubmenuAfterTimeout.bind(this),this._getDelay("hide")))}},{key:"_hideSubmenuAfterTimeout",value:function(){if(this._visibleSubmenu){var e=(0,m.default)(this._visibleSubmenu.$element().context).hasClass(q),t=this._visibleSubmenu.getOverlayContent().find(".".concat(q)).length,n=this._visibleSubmenu.getOverlayContent().get(0).querySelector(":hover");n||t||e||this._visibleSubmenu.hide()}}},{key:"_getSubmenuByRootElement",value:function(e){if(!e)return!1;var t=e.children(".".concat(z));return t.length&&A.default.getInstance(t)}},{key:"getSubmenuPosition",value:function(e){var t=this._isMenuHorizontal(),n=this.option("submenuDirection").toLowerCase(),i=this.option("rtlEnabled"),o={collision:"flip",of:e};switch(n){case"leftortop":o.at="left top",o.my=t?"left bottom":"right top";break;case"rightorbottom":o.at=t?"left bottom":"right top",o.my="left top";break;default:t?(o.at=i?"right bottom":"left bottom",o.my=i?"right top":"left top"):(o.at=i?"left top":"right top",o.my=i?"right top":"left top")}return o}},{key:"_renderBorderElement",value:function(e){(0,m.default)("<div>").appendTo(e).addClass(N).hide()}},{key:"_itemPointerDownHandler",value:function(e){var n=(0,m.default)(e.target),i=n.closest(this._itemElements());return i.hasClass("dx-menu-item-has-submenu")?void this.option("focusedElement",null):void l(_(t.prototype),"_itemPointerDownHandler",this).call(this,e)}},{key:"_hoverStartHandler",value:function(e){var t=(0,D.addNamespace)(I.default.move,this.NAME),n=this._getItemElementByEventArgs(e),i=this._dataAdapter.getNodeByItem(this._getItemData(n)),o=(0,C.isDefined)(e.buttons)&&1===e.buttons||!(0,C.isDefined)(e.buttons)&&1===e.which;if(!this._isItemDisabled(n)){if(v.default.off(n,t),!this._hasChildren(i))return void(this._showSubmenuTimer=setTimeout(this._hideSubmenuAfterTimeout.bind(this),this._getDelay("hide")));if("onHover"===this._getShowFirstSubmenuMode()&&!o){var a=this._getSubmenuByElement(n);this._clearTimeouts(),a.isOverlayVisible()||(v.default.on(n,t,this._itemMouseMoveHandler.bind(this)),this._showSubmenuTimer=this._getDelay("hide"))}}}},{key:"_hoverEndHandler",value:function(e){var n=this,i=this._getItemElementByEventArgs(e),o=(0,m.default)(e.relatedTarget);l(_(t.prototype),"_hoverEndHandler",this).call(this,e),this._clearTimeouts(),this._isItemDisabled(i)||o.hasClass(j)||this.option("hideSubmenuOnMouseLeave")&&!o.hasClass(L)&&(this._hideSubmenuTimer=setTimeout(function(){n._hideSubmenuAfterTimeout()},this._getDelay("hide")))}},{key:"_hideVisibleSubmenu",value:function(){return!!this._visibleSubmenu&&(this._hideSubmenu(this._visibleSubmenu),!0)}},{key:"_showSubmenu",value:function(e){var t=this._getSubmenuByElement(e);this._visibleSubmenu!==t&&this._hideVisibleSubmenu(),t&&(this._clearTimeouts(),t.show(),this.option("focusedElement",t.option("focusedElement"))),this._visibleSubmenu=t,this._hoveredRootItem=e}},{key:"_hideSubmenu",value:function(e){e&&e.hide(),this._visibleSubmenu===e&&(this._visibleSubmenu=null),this._hoveredRootItem=null}},{key:"_itemMouseMoveHandler",value:function(e){var t=this;if(!e.pointers||!e.pointers.length){var n=(0,m.default)(e.currentTarget);(0,C.isDefined)(this._showSubmenuTimer)&&(this._clearTimeouts(),this._showSubmenuTimer=setTimeout(function(){var e=t._getSubmenuByElement(n);e&&!e.isOverlayVisible()&&t._showSubmenu(n)},this._getDelay("show")))}}},{key:"_clearTimeouts",value:function(){clearTimeout(this._hideSubmenuTimer),clearTimeout(this._showSubmenuTimer)}},{key:"_getSubmenuByElement",value:function(e,t){var n=this._getSubmenuByRootElement(e);if(n)return n;t=t||this._getItemData(e);var i=this._dataAdapter.getNodeByItem(t);return this._hasChildren(i)&&this._renderSubmenuItems(i,e)}},{key:"_updateSubmenuVisibilityOnClick",value:function(e){var t=e.args.length&&e.args[0];if(t&&!this._disabledGetter(t.itemData)){var n=(0,m.default)(t.itemElement),i=this._getSubmenuByElement(n,t.itemData);if(this._updateSelectedItemOnClick(e),this._visibleSubmenu){if(this._visibleSubmenu===i)return void("onClick"===this.option("showFirstSubmenuMode")&&this._hideSubmenu(this._visibleSubmenu));this._hideSubmenu(this._visibleSubmenu)}if(i)return i.isOverlayVisible()?void 0:void this._showSubmenu(n)}}},{key:"_optionChanged",value:function(e){if(oe.indexOf(e.name)>=0)return void this._initActions();switch(e.name){case"orientation":case"submenuDirection":this._invalidate();break;case"showFirstSubmenuMode":case"hideSubmenuOnMouseLeave":break;case"showSubmenuMode":this._changeSubmenusOption(e.name,e.value);break;case"adaptivityEnabled":e.value?this._initAdaptivity():this._removeAdaptivity();break;case"width":this._isAdaptivityEnabled()&&(this._treeView.option(e.name,e.value),this._overlay.option(e.name,e.value)),l(_(t.prototype),"_optionChanged",this).call(this,e),this._dimensionChanged();break;case"animation":this._isAdaptivityEnabled()&&this._treeView.option("animationEnabled",!!e.value),l(_(t.prototype),"_optionChanged",this).call(this,e);break;default:this._isAdaptivityEnabled()&&this._treeView.option(e.name,e.value),l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_changeSubmenusOption",value:function(e,t){(0,w.each)(this._submenus,function(n,i){i.option(e,t)})}},{key:"selectItem",value:function(e){this._hideSubmenu(this._visibleSubmenu),l(_(t.prototype),"selectItem",this).call(this,e)}},{key:"unselectItem",value:function(e){this._hideSubmenu(this._visibleSubmenu),l(_(t.prototype),"selectItem",this).call(this,e)}}]),t}(O.default);(0,y.default)("dxMenu",ae);var re=ae;t.default=re,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return["!",e]}t.default=void 0;var a=i(n(5)),r=i(n(44)),s=i(n(196)),l=i(n(37)),u=n(490),c=i(n(10)),d=n(21),h=n(19),f=n(3),p=n(1),g=n(24),_=n(0),m=n(70),v=i(n(32)),y=i(n(101)),b=n(6),x=n(128),w={month:function(e){return v.default.getMonthNames()[e-1]},quarter:function(e){return v.default.format(new Date(2e3,3*e-1),"quarter")}},C=r.default.ViewController.inherit(function(){var e=function(e,t,n){var i=s.default.getGroupInterval(t),o=l.default.getFormatOptionsByColumn(t,"headerFilter");return i&&(o.groupInterval=i[n],l.default.isDateType(t.dataType)?o.format=w[i[n]]:"number"===t.dataType&&(o.getDisplayFormat=function(){var o={format:t.format,target:"headerFilter"},a=l.default.formatValue(e,o),r=e+i[n],s=l.default.formatValue(r,o);return a&&s?a+" - "+s:""})),o};return{init:function(){this._columnsController=this.getController("columns"),this._dataController=this.getController("data"),this._headerFilterView=this.getView("headerFilterView")},_updateSelectedState:function(e,t){for(var n=e.length,i="exclude"===t.filterType;n--;){var o=e[n];"items"in e[n]&&this._updateSelectedState(e[n].items,t),(0,u.updateHeaderFilterItemSelectionState)(o,l.default.getIndexByKey(e[n].value,t.filterValues,null)>-1,i)}},_normalizeGroupItem:function(e,t,n){var i,o,a=n.path,r=n.valueSelector,s=n.displaySelector,l=n.column;return r&&s?(i=r(e),o=s(e)):(i=e.key,o=i),e=(0,p.isObject)(e)?(0,_.extend)({},e):{},a.push(i),1===a.length?e.value=a[0]:e.value=a.join("/"),e.text=this.getHeaderItemText(o,l,t,n.headerFilterOptions),e},getHeaderItemText:function(t,n,i,o){var a=l.default.formatValue(t,e(t,n,i));return a||(a=o.texts.emptyValue),a},_processGroupItems:function(e,t,n,i){var o,a,r=this,s=i.column,l=s.lookup,u=i.level;n=n||[],t=t||0,l&&(o=(0,h.compileGetter)(l.displayExpr),a=(0,h.compileGetter)(l.valueExpr));for(var c=0;c<e.length;c++)e[c]=r._normalizeGroupItem(e[c],t,{column:i.column,headerFilterOptions:i.headerFilterOptions,displaySelector:o,valueSelector:a,path:n}),"items"in e[c]&&(t!==u&&(0,p.isDefined)(e[c].value)?r._processGroupItems(e[c].items,t+1,n,i):delete e[c].items),n.pop()},getDataSource:function(e){var t=this,n=t._dataController.dataSource(),i=l.default.getHeaderFilterGroupParameters(e,n&&n.remoteOperations().grouping),o=e.headerFilter&&e.headerFilter.dataSource,a=t.option("headerFilter"),r=!1,s={component:t.component};if(n){if((0,p.isDefined)(o)&&!(0,p.isFunction)(o))s.dataSource=(0,m.normalizeDataSourceOptions)(o);else if(e.lookup){r=!0;var u;e.lookup.items?u=e.lookup.items:(u=e.lookup.dataSource,(0,p.isFunction)(u)&&!y.default.isWrapped(u)&&(u=u({}))),s.dataSource=(0,m.normalizeDataSourceOptions)(u)}else{var c=Array.isArray(i)?i.length-1:0;t._currentColumn=e;var d=t._dataController.getCombinedFilter();t._currentColumn=null,s.dataSource={filter:d,group:i,useDefaultSearch:!0,load:function(i){var o=new b.Deferred;return i.dataField=e.dataField||e.name,n.load(i).done(function(n){t._processGroupItems(n,null,null,{level:c,column:e,headerFilterOptions:a}),o.resolve(n)}).fail(o.reject),o}}}(0,p.isFunction)(o)&&o.call(e,s);var h=s.dataSource.postProcess;return s.dataSource.postProcess=function(n){var i=n;return r&&(0!==this.pageIndex()||this.searchValue()||(i=i.slice(0),i.unshift(null)),t._processGroupItems(i,null,null,{level:0,column:e,headerFilterOptions:a})),i=h&&h.call(this,i)||i,t._updateSelectedState(i,e),i},s.dataSource}},getCurrentColumn:function(){return this._currentColumn},showHeaderFilterMenu:function(e,t){var n=this._columnsController,i=(0,_.extend)(!0,{},this._columnsController.getColumns()[e]);if(i){var o=n.getVisibleIndex(e),a=t?this.getView("headerPanel"):this.getView("columnHeadersView"),r=r||a.getColumnElements().eq(t?i.groupIndex:o);this.showHeaderFilterMenuBase({columnElement:r,column:i,applyFilter:!0,apply:function(){n.columnOption(e,{filterValues:this.filterValues,filterType:this.filterType})}})}},showHeaderFilterMenuBase:function(e){var t=this,n=this,i=e.column;if(i){var o=s.default.getGroupInterval(i),a=n._dataController.dataSource(),r=a&&a.remoteOperations().filtering;(0,_.extend)(e,i,{type:o&&o.length>1?"tree":"list",remoteFiltering:r,onShowing:function(t){var i=t.component.overlayContent().dxResizable("instance");i&&i.option("onResizeEnd",function(t){var i=n.getController("columns"),o=i.columnOption(e.dataField,"headerFilter");o=o||{},o.width=t.width,o.height=t.height,i.columnOption(e.dataField,"headerFilter",o,!0)})},onHidden:function(){return(0,x.restoreFocus)(t)}}),e.dataSource=n.getDataSource(e),e.isFilterBuilder&&(e.dataSource.filter=null,e.alignment="right"),n._headerFilterView.showHeaderFilterMenu(e.columnElement,e)}},hideHeaderFilterMenu:function(){this._headerFilterView.hideHeaderFilterMenu()}}}()),k=(0,_.extend)({},u.headerFilterMixin,{_renderCellContent:function(e,t){var n,i=this,o=t.column;!o.command&&(0,u.allowHeaderFiltering)(o)&&i.option("headerFilter.visible")&&"header"===t.rowType&&(n=i._applyColumnState({name:"headerFilter",rootElement:e,column:o,showColumnLines:i.option("showColumnLines")}),n&&i._subscribeToIndicatorEvent(n,o,"headerFilter")),i.callBase(e,t)},_subscribeToIndicatorEvent:function(e,t,n){var i=this;"headerFilter"===n&&a.default.on(e,d.name,i.createAction(function(e){e.event.stopPropagation(),i.getController("headerFilter").showHeaderFilterMenu(t.index,!1)}))},_updateIndicator:function(e,t,n){var i=this.callBase(e,t,n);i&&this._subscribeToIndicatorEvent(i,t,n)},_updateHeaderFilterIndicators:function(){this.option("headerFilter.visible")&&this._updateIndicators("headerFilter")},_needUpdateFilterIndicators:function(){return!0},_columnOptionChanged:function(e){var t=e.optionNames;return l.default.checkChanges(t,["filterValues","filterType"])?void(this._needUpdateFilterIndicators()&&this._updateHeaderFilterIndicators()):void this.callBase(e)}}),S=(0,_.extend)({},u.headerFilterMixin,{_createGroupPanelItem:function(e,t){var n,i=this,o=i.callBase.apply(i,arguments);return!t.command&&(0,u.allowHeaderFiltering)(t)&&i.option("headerFilter.visible")&&(n=i._applyColumnState({name:"headerFilter",rootElement:o,column:{alignment:(0,g.getDefaultAlignment)(i.option("rtlEnabled")),filterValues:t.filterValues,allowHeaderFiltering:!0},showColumnLines:!0}),n&&a.default.on(n,d.name,i.createAction(function(e){var n=e.event;n.stopPropagation(),i.getController("headerFilter").showHeaderFilterMenu(t.index,!0)}))),o}}),D={skipCalculateColumnFilters:function(){return!1},_calculateAdditionalFilter:function(){if(this.skipCalculateColumnFilters())return this.callBase();var e=this,t=[e.callBase()],n=e._columnsController.getVisibleColumns(null,!0),i=e.getController("headerFilter"),o=i.getCurrentColumn();return(0,f.each)(n,function(e,n){var i;if((!o||o.index!==n.index)&&(0,u.allowHeaderFiltering)(n)&&n.calculateFilterExpression&&Array.isArray(n.filterValues)&&n.filterValues.length){var a=[];(0,f.each)(n.filterValues,function(e,t){Array.isArray(t)?i=t:(n.deserializeValue&&!l.default.isDateType(n.dataType)&&"number"!==n.dataType&&(t=n.deserializeValue(t)),i=n.createFilterExpression(t,"=","headerFilter")),i&&(i.columnIndex=n.index),a.push(i)}),a=l.default.combineFilters(a,"or"),t.push("exclude"===n.filterType?["!",a]:a)}}),l.default.combineFilters(t)}},I={invertFilterExpression:o,defaultOptions:function(){return{headerFilter:{visible:!1,width:252,height:325,allowSearch:!1,searchTimeout:500,texts:{emptyValue:c.default.format("dxDataGrid-headerFilterEmptyValue"),ok:c.default.format("dxDataGrid-headerFilterOK"),cancel:c.default.format("dxDataGrid-headerFilterCancel")}}}},controllers:{headerFilter:C},views:{headerFilterView:u.HeaderFilterView},extenders:{controllers:{data:D},views:{columnHeadersView:k,headerPanel:S}}};t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){e=e||[];for(var t=0;t<e.length;t++)e[t].selected=!1,o(e[t].items)}function a(e,t){if(!e.component.option("searchValue")){var n=(0,l.default)(e.element).find(".dx-list-select-all-checkbox").data("dxCheckBox");n&&t&&t.length&&n.option("value",void 0)}}function r(e,t){var n=t.headerFilter;return n&&(0,d.isDefined)(n.allowSearch)?n.allowSearch:e.option("headerFilter.allowSearch")}function s(e,t,n){if(t^n){if(e.selected=!0,n&&e.items)for(var i=0;i<e.items.length;i++)if(!e.items[i].selected){e.selected=void 0;break}}else(n||e.selected)&&(e.selected=!1,o(e.items))}t.updateHeaderFilterItemSelectionState=s,t.headerFilterMixin=t.allowHeaderFiltering=t.HeaderFilterView=void 0;var l=i(n(2)),u=i(n(44)),c=i(n(37)),d=n(1),h=n(3),f=n(0),p=i(n(48)),g=i(n(167)),_=i(n(115)),m="dx-header-filter",v="dx-header-filter-menu",y="text",b=u.default.View.inherit({
getPopupContainer:function(){return this._popupContainer},getListContainer:function(){return this._listContainer},applyHeaderFilter:function(e){var t=this,n=t.getListContainer(),i=n.option("searchValue"),o=!i&&!e.isFilterBuilder&&n.$element().find(".dx-checkbox").eq(0).hasClass("dx-checkbox-checked"),a=[],r=function e(t,o,a){(0,h.each)(o,function(o,r){if(void 0!==r.selected&&!!r.selected^a){var s=n._getNode(r),l=n._hasChildren(s),u=l&&r.items&&r.items.some(function(e){return e.selected});if(!i||!u)return void t.push(r.value)}r.items&&r.items.length&&e(t,r.items,a)})};o?("tree"===e.type&&(e.filterType="exclude"),Array.isArray(e.filterValues)&&(e.filterValues=[])):"tree"===e.type&&(e.filterType&&(e.filterType="include"),r(a,n.option("items"),!1),e.filterValues=a),e.filterValues&&!e.filterValues.length&&(e.filterValues=null),e.apply(),t.hideHeaderFilterMenu()},showHeaderFilterMenu:function(e,t){var n=this;if(t){n._initializePopupContainer(t);var i=n.getPopupContainer();n.hideHeaderFilterMenu(),n.updatePopup(e,t),i.show()}},hideHeaderFilterMenu:function(){var e=this.getPopupContainer();e&&e.hide()},updatePopup:function(e,t){var n=this,i="right"===t.alignment?"left":"right";n._popupContainer&&(n._cleanPopupContent(),n._popupContainer.option("position",{my:i+" top",at:i+" bottom",of:e,collision:"flip fit"}))},_getSearchExpr:function(e){var t=e.lookup,n=e.useDefaultSearchExpr,i=e.headerFilter&&e.headerFilter.dataSource;if(n||(0,d.isDefined)(i)&&!(0,d.isFunction)(i))return y;if(t)return t.displayExpr||"this";if(e.dataSource){var o=e.dataSource.group;if(Array.isArray(o)&&o.length>0)return o[0].selector;if((0,d.isFunction)(o)&&!e.remoteFiltering)return o}return e.dataField||e.selector},_cleanPopupContent:function(){this._popupContainer&&this._popupContainer.$content().empty()},_initializePopupContainer:function(e){var t=this,n=t.element(),i=t.option("headerFilter"),o=e.headerFilter&&e.headerFilter.width||i&&i.width,a=e.headerFilter&&e.headerFilter.height||i&&i.height,r={width:o,height:a,visible:!1,shading:!1,showTitle:!1,showCloseButton:!1,closeOnTargetScroll:!1,dragEnabled:!1,closeOnOutsideClick:!0,focusStateEnabled:!1,toolbarItems:[{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:i.texts.ok,onClick:function(){t.applyHeaderFilter(e)}}},{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:i.texts.cancel,onClick:function(){t.hideHeaderFilterMenu()}}}],resizeEnabled:!0,onShowing:function(n){n.component.$content().parent().addClass("dx-dropdowneditor-overlay"),t._initializeListContainer(e),e.onShowing&&e.onShowing(n)},onShown:function(){t.getListContainer().focus()},onHidden:e.onHidden,onInitialized:function(e){var t=e.component;t.option("animation",t._getDefaultOptions().animation)}};(0,d.isDefined)(t._popupContainer)?t._popupContainer.option(r):t._popupContainer=t._createComponent(n,p.default,r)},_initializeListContainer:function(e){function t(t){"searchValue"!==t.fullName||e.isFilterBuilder||n.option("headerFilter.hideSelectAllOnSearch")===!1||("tree"===e.type?t.component.option("showCheckBoxesMode",t.value?"normal":"selectAll"):t.component.option("selectionMode",t.value?"multiple":"all"))}var n=this,i=n._popupContainer.$content(),o={searchEnabled:r(n,e),searchTimeout:n.option("headerFilter.searchTimeout"),searchMode:e.headerFilter&&e.headerFilter.searchMode||"",dataSource:e.dataSource,onContentReady:function(){n.renderCompleted.fire()},itemTemplate:function(t,n,i){var o=(0,l.default)(i);return e.encodeHtml?o.text(t.text):o.html(t.text)}};"tree"===e.type?n._listContainer=n._createComponent((0,l.default)("<div>").appendTo(i),g.default,(0,f.extend)(o,{showCheckBoxesMode:e.isFilterBuilder?"normal":"selectAll",onOptionChanged:t,keyExpr:"id"})):n._listContainer=n._createComponent((0,l.default)("<div>").appendTo(i),_.default,(0,f.extend)(o,{searchExpr:n._getSearchExpr(e),pageLoadMode:"scrollBottom",showSelectionControls:!0,selectionMode:e.isFilterBuilder?"multiple":"all",onOptionChanged:t,onSelectionChanged:function(t){var n=t.component.option("items"),i=t.component.option("selectedItems");if(!t.component._selectedItemsUpdating&&!t.component.option("searchValue")&&!e.isFilterBuilder){var o=e.filterValues||[],r="exclude"===e.filterType;0===i.length&&n.length&&(o.length<=1||r&&o.length===n.length-1)?(e.filterType="include",e.filterValues=[]):i.length===n.length&&(e.filterType="exclude",e.filterValues=[])}(0,h.each)(n,function(t,n){var o=c.default.getIndexByKey(n,i,null)>=0,a=!!n.selected;if(a!==o){n.selected=o,e.filterValues=e.filterValues||[];var r=c.default.getIndexByKey(n.value,e.filterValues,null);r>=0&&e.filterValues.splice(r,1),o^"exclude"===e.filterType&&e.filterValues.push(n.value)}}),a(t,e.filterValues)},onContentReady:function(t){var n=t.component,i=n.option("items"),o=[];(0,h.each)(i,function(){this.selected&&o.push(this)}),n._selectedItemsUpdating=!0,n.option("selectedItems",o),n._selectedItemsUpdating=!1,a(t,e.filterValues)}}))},_renderCore:function(){this.element().addClass(v)}});t.HeaderFilterView=b;var x=function(e){return(0,d.isDefined)(e.allowHeaderFiltering)?e.allowHeaderFiltering:e.allowFiltering};t.allowHeaderFiltering=x;var w={_applyColumnState:function(e){var t,n=e.rootElement,i=e.column;return"headerFilter"===e.name?(n.find("."+m).remove(),x(i)&&(t=this.callBase(e).toggleClass("dx-header-filter-empty",this._isHeaderFilterEmpty(i)),this.option("useLegacyKeyboardNavigation")||t.attr("tabindex",this.option("tabindex")||0)),t):this.callBase(e)},_isHeaderFilterEmpty:function(e){return!e.filterValues||!e.filterValues.length},_getIndicatorClassName:function(e){return"headerFilter"===e?m:this.callBase(e)},_renderIndicator:function(e){var t=e.container,n=e.indicator;if("headerFilter"===e.name){var i=this.option("rtlEnabled");if(t.children().length&&(!i&&"right"===e.columnAlignment||i&&"left"===e.columnAlignment))return void t.prepend(n)}this.callBase(e)},optionChanged:function(e){if("headerFilter"===e.name){var t="columnHeadersView"===this.name;this._invalidate(t,t),e.handled=!0}else this.callBase(e)}};t.headerFilterMixin=w},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e.dataField||e.name}function a(e){e.forEach(function(e){var t=o(e);if(!(0,r.isDefined)(t)&&e.allowFiltering)throw new u.default.Error("E1049",e.caption)})}t.default=void 0;var r=n(1),s=i(n(44)),l=n(223),u=i(n(17)),c=i(n(37)),d=i(n(196)),h=n(719),f=["=","<>","<","<=",">",">=","notcontains","contains","startswith","endswith","between"],p="include",g="exclude",_=s.default.Controller.inherit(function(){var e=function(){return{filterType:p,filterValues:void 0}},t=function(e){return!(d.default.getGroupInterval(e)||e.headerFilter&&e.headerFilter.dataSource)},n=function(n,i){if(!n)return e();var o,a=n[1],r=n[2],s=Array.isArray(r);if(!s&&!t(i))return e();switch(a){case"anyof":case"=":o=p;break;case"noneof":case"<>":o=g;break;default:return e()}return{filterType:o,filterValues:s?r:[r]}},i=function(e){var t=e.filterValue;if((0,r.isDefined)(t)){var n=e.selectedFilterOperation||e.defaultFilterOperation||(0,l.getDefaultOperation)(e),i=[o(e),n,e.filterValue];return i}return null},s=function(e){var n,i,a=e.filterValues;return a?(t(e)&&1===e.filterValues.length&&!Array.isArray(a[0])?(n=e.filterType===g?"<>":"=",i=a[0]):(n=e.filterType===g?"noneof":"anyof",i=a),[o(e),n,i]):null},u=function(e,t,i){var a=n(i,t);e.columnOption(o(t),a)},c=function(e,t,n){var i,a=n&&n[1],s=t.filterOperations||t.defaultFilterOperations;(!s||s.indexOf(a)>=0||a===t.defaultFilterOperation)&&f.indexOf(a)>=0?(a!==t.defaultFilterOperation||(0,r.isDefined)(t.selectedFilterOperation)||(a=t.selectedFilterOperation),i={filterValue:n[2],selectedFilterOperation:a}):i={filterValue:void 0,selectedFilterOperation:void 0},e.columnOption(o(t),i)};return{syncFilterValue:function(){var e=this,t=e.getController("columns"),n=t.getFilteringColumns();this._skipSyncColumnOptions=!0,n.forEach(function(n){var i=(0,l.getMatchedConditions)(e.option("filterValue"),o(n));if(1===i.length){var a=i[0];u(t,n,a),c(t,n,a)}else(0,r.isDefined)(n.filterValues)&&u(t,n),(0,r.isDefined)(n.filterValue)&&c(t,n)}),this._skipSyncColumnOptions=!1},_initSync:function(){var e=this.getController("columns").getColumns(),t=this.getController("data"),n=t.pageIndex();if(a(e),!this.option("filterValue")){var i=this.getController("columns").getFilteringColumns(),o=this.getFilterValueFromColumns(i);this.option("filterValue",o)}this.syncFilterValue(),t.pageIndex(n)},init:function(){var e=this,t=this.getController("data");t.isFilterSyncActive()&&(this.getController("columns").isAllDataTypesDefined()?this._initSync():t.dataSourceChanged.add(function(){return e._initSync()}))},_getSyncFilterRow:function(e,t){var n=i(t);return(0,r.isDefined)(n)?(0,l.syncFilters)(e,n):(0,l.removeFieldConditionsFromFilter)(e,o(t))},_getSyncHeaderFilter:function(e,t){var n=s(t);return n?(0,l.syncFilters)(e,n):(0,l.removeFieldConditionsFromFilter)(e,o(t))},getFilterValueFromColumns:function(e){if(!this.getController("data").isFilterSyncActive())return null;var t=["and"];return e&&e.forEach(function(e){var n=s(e),o=i(e);n&&(0,l.addItem)(n,t),o&&(0,l.addItem)(o,t)}),(0,l.getNormalizedFilter)(t)},syncFilterRow:function(e,t){this.option("filterValue",this._getSyncFilterRow(this.option("filterValue"),e))},syncHeaderFilter:function(e){this.option("filterValue",this._getSyncHeaderFilter(this.option("filterValue"),e))},getCustomFilterOperations:function(){var e=this.option("filterBuilder.customOperations")||[];return[(0,h.anyOf)(this.component),(0,h.noneOf)(this.component)].concat(e)},publicMethods:function(){return["getCustomFilterOperations"]}}}()),m={isFilterSyncActive:function(){var e=this.option("filterSyncEnabled");return"auto"===e?this.option("filterPanel.visible"):e},skipCalculateColumnFilters:function(){return(0,r.isDefined)(this.option("filterValue"))&&this.isFilterSyncActive()},_calculateAdditionalFilter:function(){var e=this;if(e.option("filterPanel.filterEnabled")===!1)return e.callBase();var t=[e.callBase()],n=e.getController("columns").getFilteringColumns(),i=e.option("filterValue");if(e.isFilterSyncActive()){var a=e.getController("headerFilter").getCurrentColumn();a&&i&&(i=(0,l.removeFieldConditionsFromFilter)(i,o(a)))}var r=e.getController("filterSync").getCustomFilterOperations(),s=(0,l.getFilterExpression)(i,n,r,"filterBuilder");return s&&t.push(s),c.default.combineFilters(t)},_parseColumnPropertyName:function(e){var t=e.match(/.*\.(.*)/);return t[1]},clearFilter:function(e){this.component.beginUpdate(),arguments.length>0?("filterValue"===e&&this.option("filterValue",null),this.callBase(e)):(this.option("filterValue",null),this.callBase()),this.component.endUpdate()},optionChanged:function(e){switch(e.name){case"filterValue":this._applyFilter(),this.isFilterSyncActive()&&this.getController("filterSync").syncFilterValue(),e.handled=!0;break;case"filterSyncEnabled":e.handled=!0;break;case"columns":if(this.isFilterSyncActive()){var t=this.getController("columns").getColumnByPath(e.fullName),n=this.getController("filterSync");if(t&&!n._skipSyncColumnOptions){var i=this._parseColumnPropertyName(e.fullName);n._skipSyncColumnOptions=!0,"filterType"===i?g!==e.value&&g!==e.previousValue||n.syncHeaderFilter(t):"filterValues"===i?n.syncHeaderFilter(t):["filterValue","selectedFilterOperation"].indexOf(i)>-1&&n.syncFilterRow(t,t.filterValue),n._skipSyncColumnOptions=!1}}this.callBase(e);break;default:this.callBase(e)}}},v={_isHeaderFilterEmpty:function(e){return this.getController("data").isFilterSyncActive()?!(0,l.filterHasField)(this.option("filterValue"),o(e)):this.callBase(e)},_needUpdateFilterIndicators:function(){return!this.getController("data").isFilterSyncActive()},optionChanged:function(e){"filterValue"===e.name?this._updateHeaderFilterIndicators():this.callBase(e)}},y={defaultOptions:function(){return{filterValue:null,filterSyncEnabled:"auto"}},controllers:{filterSync:_},extenders:{controllers:{data:m},views:{columnHeadersView:v}}};t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(44)),r=n(0),s=i(n(493)),l=i(n(10)),u=i(n(96)),c=i(n(48)),d=n(128),h=a.default.View.inherit({_renderCore:function(){this._updatePopupOptions()},_updatePopupOptions:function(){this.option("filterBuilderPopup.visible")?this._initPopup():this._filterBuilderPopup&&this._filterBuilderPopup.hide()},_disposePopup:function(){this._filterBuilderPopup&&(this._filterBuilderPopup.dispose(),this._filterBuilderPopup=void 0),this._filterBuilder&&(this._filterBuilder.dispose(),this._filterBuilder=void 0)},_initPopup:function(){var e=this;e._disposePopup(),e._filterBuilderPopup=e._createComponent(e.element(),c.default,(0,r.extend)({title:l.default.format("dxDataGrid-filterBuilderPopupTitle"),contentTemplate:function(t){return e._getPopupContentTemplate(t)},onOptionChanged:function(t){"visible"===t.name&&e.option("filterBuilderPopup.visible",t.value)},toolbarItems:e._getPopupToolbarItems()},e.option("filterBuilderPopup"),{onHidden:function(t){(0,d.restoreFocus)(e),e._disposePopup()}}))},_getPopupContentTemplate:function(e){var t=(0,o.default)(e),n=(0,o.default)("<div>").appendTo((0,o.default)(e));this._filterBuilder=this._createComponent(n,s.default,(0,r.extend)({value:this.option("filterValue"),fields:this.getController("columns").getFilteringColumns()},this.option("filterBuilder"),{customOperations:this.getController("filterSync").getCustomFilterOperations()})),this._createComponent(t,u.default,{direction:"both"})},_getPopupToolbarItems:function(){var e=this;return[{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:l.default.format("OK"),onClick:function(t){var n=e._filterBuilder.option("value");e.option("filterValue",n),e._filterBuilderPopup.hide()}}},{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:l.default.format("Cancel"),onClick:function(t){e._filterBuilderPopup.hide()}}}]},optionChanged:function(e){switch(e.name){case"filterBuilder":case"filterBuilderPopup":this._invalidate(),e.handled=!0;break;default:this.callBase(e)}}}),f={defaultOptions:function(){return{filterBuilder:{groupOperationDescriptions:{and:l.default.format("dxFilterBuilder-and"),or:l.default.format("dxFilterBuilder-or"),notAnd:l.default.format("dxFilterBuilder-notAnd"),notOr:l.default.format("dxFilterBuilder-notOr")},filterOperationDescriptions:{between:l.default.format("dxFilterBuilder-filterOperationBetween"),equal:l.default.format("dxFilterBuilder-filterOperationEquals"),notEqual:l.default.format("dxFilterBuilder-filterOperationNotEquals"),lessThan:l.default.format("dxFilterBuilder-filterOperationLess"),lessThanOrEqual:l.default.format("dxFilterBuilder-filterOperationLessOrEquals"),greaterThan:l.default.format("dxFilterBuilder-filterOperationGreater"),greaterThanOrEqual:l.default.format("dxFilterBuilder-filterOperationGreaterOrEquals"),startsWith:l.default.format("dxFilterBuilder-filterOperationStartsWith"),contains:l.default.format("dxFilterBuilder-filterOperationContains"),notContains:l.default.format("dxFilterBuilder-filterOperationNotContains"),endsWith:l.default.format("dxFilterBuilder-filterOperationEndsWith"),isBlank:l.default.format("dxFilterBuilder-filterOperationIsBlank"),isNotBlank:l.default.format("dxFilterBuilder-filterOperationIsNotBlank")}},filterBuilderPopup:{}}},views:{filterBuilderView:h}};t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(721)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(1),r=i(n(44)),s=i(n(37)),l=i(n(5)),u=i(n(10)),c=i(n(144)),d=n(223),h=n(6),f=n(40),p=n(302),g="filter-panel",_=g+"-text",m=g+"-checkbox",v=g+"-clear-filter",y=g+"-left",b="filterPanel",x=r.default.View.inherit({isVisible:function(){return this.option("filterPanel.visible")&&this.getController("data").dataSource()},init:function(){var e=this;this.getController("data").dataSourceChanged.add(function(){return e.render()})},_renderCore:function(){var e=this,t=e.element();t.empty().addClass(e.addWidgetPrefix(g));var n=(0,o.default)("<div>").addClass(e.addWidgetPrefix(y)).appendTo(t);e.option("filterValue")||e._filterValueBuffer?(n.append(e._getCheckElement()).append(e._getFilterElement()).append(e._getTextElement()),t.append(e._getRemoveButtonElement())):n.append(e._getFilterElement()).append(e._getTextElement())},_getCheckElement:function(){var e=this,t=(0,o.default)("<div>").addClass(this.addWidgetPrefix(m));return e._createComponent(t,c.default,{value:e.option("filterPanel.filterEnabled"),onValueChanged:function(t){e.option("filterPanel.filterEnabled",t.value)}}),t.attr("title",this.option("filterPanel.texts.filterEnabledHint")),t},_getFilterElement:function(){var e=this,t=(0,o.default)("<div>").addClass("dx-icon-filter");return l.default.on(t,"click",function(){return e._showFilterBuilder()}),(0,p.registerKeyboardAction)("filterPanel",e,t,void 0,function(){return e._showFilterBuilder()}),e._addTabIndexToElement(t),t},_getTextElement:function(){var e,t=this,n=(0,o.default)("<div>").addClass(t.addWidgetPrefix(_)),i=t.option("filterValue");return i?(0,h.when)(t.getFilterText(i,t.getController("filterSync").getCustomFilterOperations())).done(function(e){var o=t.option("filterPanel.customizeText");if(o){var a=o({component:t.component,filterValue:i,text:e});"string"==typeof a&&(e=a)}n.text(e)}):(e=t.option("filterPanel.texts.createFilter"),n.text(e)),l.default.on(n,"click",function(){return t._showFilterBuilder()}),(0,p.registerKeyboardAction)("filterPanel",t,n,void 0,function(){return t._showFilterBuilder()}),t._addTabIndexToElement(n),n},_showFilterBuilder:function(){this.option("filterBuilderPopup.visible",!0)},_getRemoveButtonElement:function(){var e=this,t=function(){return e.option("filterValue",null)},n=(0,o.default)("<div>").addClass(e.addWidgetPrefix(v)).text(e.option("filterPanel.texts.clearFilter"));return l.default.on(n,"click",t),(0,p.registerKeyboardAction)("filterPanel",this,n,void 0,t),e._addTabIndexToElement(n),n},_addTabIndexToElement:function(e){if(!this.option("useLegacyKeyboardNavigation")){var t=this.option("tabindex")||0;e.attr("tabindex",t)}},optionChanged:function(e){switch(e.name){case"filterValue":this._invalidate(),this.option("filterPanel.filterEnabled",!0),e.handled=!0;break;case"filterPanel":this._invalidate(),e.handled=!0;break;default:this.callBase(e)}},_getConditionText:function(e,t,n){var i="[".concat(e,"] ").concat(t);return(0,a.isDefined)(n)&&(i+=n),i},_getValueMaskedText:function(e){return Array.isArray(e)?"('".concat(e.join("', '"),"')"):" '".concat(e,"'")},_getValueText:function(e,t,n){var i=this,o=new h.Deferred,r=t&&t.customizeText;if((0,a.isDefined)(n)||r)if(!r&&e.lookup)(0,d.getCurrentLookupValueText)(e,n,function(e){o.resolve(i._getValueMaskedText(e))});else{var l=Array.isArray(n)?n:s.default.getDisplayValue(e,n);(0,h.when)((0,d.getCurrentValueText)(e,l,t,b)).done(function(e){o.resolve(i._getValueMaskedText(e))})}else o.resolve("");return o.promise()},getConditionText:function(e,t){var n,i=this,o=e[1],a=new h.Deferred,r=(0,d.getCustomOperation)(t.customOperations,o),s=(0,d.getField)(e[0],t.columns),l=s.caption||"",u=e[2];return n=r?r.caption||(0,f.captionize)(r.name):null===u?(0,d.getCaptionByOperation)("="===o?"isblank":"isnotblank",t.filterOperationDescriptions):(0,d.getCaptionByOperation)(o,t.filterOperationDescriptions),this._getValueText(s,r,u).done(function(e){a.resolve(i._getConditionText(l,n,e))}),a},getGroupText:function(e,t,n){var i=this,o=new h.Deferred,a=[],r=(0,d.getGroupValue)(e);return e.forEach(function(e){(0,d.isCondition)(e)?a.push(i.getConditionText(e,t)):(0,d.isGroup)(e)&&a.push(i.getGroupText(e,t,!0))}),h.when.apply(this,a).done(function(){for(var e,i=arguments.length,a=new Array(i),s=0;s<i;s++)a[s]=arguments[s];if("!"===r[0]){var l=t.groupOperationDescriptions["not"+r.substring(1,2).toUpperCase()+r.substring(2)].split(" ");e="".concat(l[0]," ").concat(a[0])}else e=a.join(" ".concat(t.groupOperationDescriptions[r]," "));n&&(e="(".concat(e,")")),o.resolve(e)}),o},getFilterText:function(e,t){var n=this,i={customOperations:t,columns:n.getController("columns").getFilteringColumns(),filterOperationDescriptions:n.option("filterBuilder.filterOperationDescriptions"),groupOperationDescriptions:n.option("filterBuilder.groupOperationDescriptions")};return(0,d.isCondition)(e)?n.getConditionText(e,i):n.getGroupText(e,i)}}),w={defaultOptions:function(){return{filterPanel:{visible:!1,filterEnabled:!0,texts:{createFilter:u.default.format("dxDataGrid-filterPanelCreateFilter"),clearFilter:u.default.format("dxDataGrid-filterPanelClearFilter"),filterEnabledHint:u.default.format("dxDataGrid-filterPanelFilterEnabledHint")}}}},views:{filterPanelView:x},extenders:{controllers:{data:{optionChanged:function(e){switch(e.name){case"filterPanel":this._applyFilter(),e.handled=!0;break;default:this.callBase(e)}}}}}};t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return(0,l.isDefined)(e.allowSearch)?e.allowSearch:e.allowFiltering}function a(e,t){var n=e.lookup;return e.parseValue?n?e.parseValue.call(n,t):e.parseValue(t):t}t.default=void 0;var r=i(n(2)),s=i(n(11)),l=n(1),u=n(19),c=n(3),d=i(n(37)),h=i(n(10)),f=i(n(47)),p="search-panel",g="search-text",_="header-panel",m=700,v={defaultOptions:function(){return{searchPanel:{visible:!1,width:160,placeholder:h.default.format("dxDataGrid-searchPanelPlaceholder"),highlightSearchText:!0,highlightCaseSensitive:!1,text:"",searchVisibleColumnsOnly:!1}}},extenders:{controllers:{data:function(){var e=function(e,t){function n(e){for(var t=(0,u.compileGetter)(s.valueExpr),n=0;n<e.length;n++){var i=t(e[n]);h.push(r.createFilterExpression(i,null,"search"))}}var i,r,s,l=e._columnsController.getColumns(),c=e.option("searchPanel.searchVisibleColumnsOnly"),h=[];if(!t)return null;for(i=0;i<l.length;i++)if(r=l[i],(!c||r.visible)&&o(r)&&r.calculateFilterExpression){s=r.lookup;var p=a(r,t);s&&s.items?(0,f.default)(s.items).filter(r.createFilterExpression.call({dataField:s.displayExpr,dataType:s.dataType,calculateFilterExpression:r.calculateFilterExpression},p,null,"search")).enumerate().done(n):void 0!==p&&h.push(r.createFilterExpression(p,null,"search"))}return d.default.combineFilters(h,"or")};return{publicMethods:function(){return this.callBase().concat(["searchByText"])},_calculateAdditionalFilter:function(){var t=this,n=t.callBase(),i=e(t,t.option("searchPanel.text"));return d.default.combineFilters([n,i])},searchByText:function(e){this.option("searchPanel.text",e)},optionChanged:function(e){var t=this;switch(e.fullName){case"searchPanel.text":case"searchPanel":t._applyFilter(),e.handled=!0;break;default:t.callBase(e)}}}}()},views:{headerPanel:function(){var e=function(e){return e.option("searchPanel")};return{_getToolbarItems:function(){var e=this.callBase();return this._prepareSearchItem(e)},_prepareSearchItem:function(t){var n=this,i=n.getController("data"),o=e(n);if(o&&o.visible){var a={template:function(e,t,a){var s=(0,r.default)("<div>").addClass(n.addWidgetPrefix(p)).appendTo(a);n.getController("editorFactory").createEditor(s,{width:o.width,placeholder:o.placeholder,parentType:"searchPanel",value:n.option("searchPanel.text"),updateValueTimeout:m,setValue:function(e){i.searchByText(e)},editorOptions:{inputAttr:{"aria-label":h.default.format("dxDataGrid-ariaSearchInGrid")}}}),n.resize()},name:"searchPanel",location:"after",locateInMenu:"never",sortIndex:40};t.push(a)}return t},getSearchTextEditor:function(){var e=this,t=e.element(),n=t.find("."+e.addWidgetPrefix(p)).filter(function(){return(0,r.default)(this).closest("."+e.addWidgetPrefix(_)).is(t)});return n.length?n.dxTextBox("instance"):null},isVisible:function(){var t=e(this);return this.callBase()||t&&t.visible},optionChanged:function(e){if("searchPanel"===e.name){if("searchPanel.text"===e.fullName){var t=this.getSearchTextEditor();t&&t.option("value",e.value)}else this._invalidate();e.handled=!0}else this.callBase(e)}}}(),rowsView:{init:function(){this.callBase.apply(this,arguments),this._searchParams=[]},_getFormattedSearchText:function(e,t){var n=a(e,t),i=d.default.getFormatOptionsByColumn(e,"search");return d.default.formatValue(n,i)},_getStringNormalizer:function(){var e=this.option("searchPanel.highlightCaseSensitive");return function(t){return e?t:t.toLowerCase()}},_findHighlightingTextNodes:function(e,t,n){var i,o=this,a=t.parent(),s=this._getStringNormalizer(),l=s(n);if(a.length){if(e)if(e.groupIndex>=0&&!e.showWhenGrouped)i=t;else{var u=o._columnsController.getVisibleIndex(e.index);i=a.children("td").eq(u).find("*")}}else a=(0,r.default)("<div>").append(t);return i=i&&i.length?i:a.find("*"),i=i.filter(function(e,t){for(var n=(0,r.default)(t).contents(),i=0;i<n.length;i++){var o=n.get(i);return 3===o.nodeType&&s(o.textContent||o.nodeValue).indexOf(l)>-1}})},_highlightSearchTextCore:function(e,t){var n=this,i=(0,r.default)("<span>").addClass(n.addWidgetPrefix(g)),o=e.text(),a=e[0],l=this._getStringNormalizer(),u=l(o).indexOf(l(t));if(u>=0)return a.textContent?a.textContent=o.substr(0,u):a.nodeValue=o.substr(0,u),e.after(i.text(o.substr(u,t.length))),e=(0,r.default)(s.default.createTextNode(o.substr(u+t.length))).insertAfter(i),n._highlightSearchTextCore(e,t)},_highlightSearchText:function(e,t,n){var i=this,o=this._getStringNormalizer(),a=i.option("searchPanel.text");if(t&&n&&(a=a&&i._getFormattedSearchText(n,a)),a&&i.option("searchPanel.highlightSearchText")){var s=i._findHighlightingTextNodes(n,e,a);(0,c.each)(s,function(e,n){(0,c.each)((0,r.default)(n).contents(),function(e,n){t?o((0,r.default)(n).text())===o(a)&&(0,r.default)(this).replaceWith((0,r.default)("<span>").addClass(i.addWidgetPrefix(g)).text((0,r.default)(n).text())):i._highlightSearchTextCore((0,r.default)(n),a)})})}},_renderCore:function(){this.callBase.apply(this,arguments),this.option("rowTemplate")&&(this.option("templatesRenderAsynchronously")?(clearTimeout(this._highlightTimer),this._highlightTimer=setTimeout(function(){this._highlightSearchText(this._getTableElement())}.bind(this))):this._highlightSearchText(this._getTableElement()))},_updateCell:function(e,t){var n=t.column,i=n.lookup&&n.lookup.dataType||n.dataType,a="string"!==i;o(n)&&(this.option("templatesRenderAsynchronously")?(this._searchParams.length||(clearTimeout(this._highlightTimer),this._highlightTimer=setTimeout(function(){this._searchParams.forEach(function(e){this._highlightSearchText.apply(this,e)}.bind(this)),this._searchParams=[]}.bind(this))),this._searchParams.push([e,a,n])):this._highlightSearchText(e,a,n)),this.callBase(e,t)},dispose:function(){clearTimeout(this._highlightTimer),this.callBase()}}}}};t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(44)),a=i(n(725)),r=n(12),s=n(1),l=n(7),u="pager",c=10,d=function(e){return 1+(parseInt(e.pageIndex())||0)},h=o.default.View.inherit({init:function(){var e=this,t=e.getController("data");e._isVisible=!1,t.changed.add(function(n){if(n&&n.repaintChangesOnly){var i=e._getPager();i?i.option({pageIndex:d(t),pageSize:t.pageSize(),pageCount:t.pageCount(),totalCount:t.totalCount(),hasKnownLastPage:t.hasKnownLastPage()}):e.render()}else(!n||"update"!==n.changeType&&"updateSelection"!==n.changeType)&&e.render()})},_getPager:function(){var e=this.element();return e&&e.data("dxPager")},_renderCore:function(){var e=this,t=e.element().addClass(e.addWidgetPrefix(u)),n=e.option("pager")||{},i=e.getController("data"),o=e.getController("keyboardNavigation"),r={maxPagesCount:c,pageIndex:d(i),pageCount:i.pageCount(),pageSize:i.pageSize(),showPageSizes:n.showPageSizeSelector,showInfo:n.showInfo,displayMode:n.displayMode,pagesNavigatorVisible:n.visible,showNavigationButtons:n.showNavigationButtons,pageSizes:e.getPageSizes(),totalCount:i.totalCount(),hasKnownLastPage:i.hasKnownLastPage(),pageIndexChanged:function(e){i.pageIndex()!==e-1&&setTimeout(function(){i.pageIndex(e-1)})},pageSizeChanged:function(e){setTimeout(function(){i.pageSize(e)})},onKeyDown:function(e){return o&&o.executeAction("onKeyDown",e)},useLegacyKeyboardNavigation:this.option("useLegacyKeyboardNavigation"),useKeyboard:this.option("keyboardNavigation.enabled")};(0,s.isDefined)(n.infoText)&&(r.infoText=n.infoText),e._createComponent(t,a.default,r)},getPageSizes:function(){var e=this,t=e.getController("data"),n=e.option("pager"),i=n&&n.allowedPageSizes,o=t.pageSize();return(0,s.isDefined)(e._pageSizes)&&(0,r.inArray)(o,e._pageSizes)!==-1||(e._pageSizes=[],n&&(Array.isArray(i)?e._pageSizes=i:i&&o>1&&(e._pageSizes=[Math.floor(o/2),o,2*o]))),e._pageSizes},isVisible:function(){var e=this,t=e.getController("data"),n=e.option("pager"),i=n&&n.visible,o=e.option("scrolling");return!!e._isVisible||("auto"===i&&(i=(!o||"virtual"!==o.mode&&"infinite"!==o.mode)&&(t.pageCount()>1||t.isLoaded()&&!t.hasKnownLastPage())),e._isVisible=i,i)},getHeight:function(){return this.getElementHeight()},optionChanged:function(e){var t=this,n=e.name,i="pager"===n,o="paging"===n,a="dataSource"===n,r="scrolling"===n,s=t.getController("data");if(i||o||r||a){if(e.handled=!0,s.skipProcessingPagingChange(e.fullName))return;(i||o)&&(t._pageSizes=null),(i||o||r)&&(t._isVisible=!1),a||(t._invalidate(),(0,l.hasWindow)()&&i&&t.component&&t.component.resize())}}}),f={defaultOptions:function(){return{pager:{visible:"auto",showPageSizeSelector:!1,allowedPageSizes:"auto"}}},views:{pagerView:h}};t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=i(n(27)),l=n(1),u=n(3),c=n(0),d=n(24),h=n(8),f=i(n(25)),p=n(80),g=i(n(44)),_=i(n(37)),m=i(n(43)),v=i(n(164)),y="columns-separator",b="columns-separator-transparent",x="drag-header",w="text-content",C="drag-action",k="tracker",S="drop-highlight",D="dx-block-separator",I="dx-header-row",T="dx-widget",O="dx-drag-command-cell",E="dxDataGridResizingReordering",A=10,P=5,R=.5,M=function(e){return e.option("allowColumnResizing")||e.getController("columns").isColumnOptionUsed("allowResizing")},B=function(e){return e.option("allowColumnReordering")||e.getController("columns").isColumnOptionUsed("allowReordering")},V=g.default.View.inherit({_renderCore:function(){this.callBase(),this.element().addClass(this.addWidgetPrefix(k)),this.hide()},_unsubscribeFromCallback:function(){this._positionChanged&&this._tablePositionController.positionChanged.remove(this._positionChanged)},_subscribeToCallback:function(){var e=this;e._positionChanged=function(t){var n=e.element();n&&n.hasClass(e.addWidgetPrefix(k))&&(n.css({top:t.top}),n.height(t.height))},this._tablePositionController.positionChanged.add(e._positionChanged)},optionChanged:function(e){"allowColumnResizing"===e.name&&(this._unsubscribeFromCallback(),e.value&&(this._subscribeToCallback(),this._invalidate())),this.callBase(e)},init:function(){this.callBase(),this._tablePositionController=this.getController("tablePosition"),this._subscribeToCallback()},isVisible:function(){return M(this)},show:function(){this.element().show()},hide:function(){this.element()&&this.element().hide()},setHeight:function(e){this.element().height(e)},dispose:function(){this._unsubscribeFromCallback(),this.callBase()}}),F=g.default.View.inherit({_renderSeparator:function(){},_renderCore:function(e){this.callBase(e),this._isShown=!0,this._renderSeparator(),this.hide()},show:function(){this._isShown=!0},hide:function(){this._isShown=!1},height:function(e){var t=this.element();if(t){if(!(0,l.isDefined)(e))return t.height();t.height(e)}},width:function(e){var t=this.element();if(t){if(!(0,l.isDefined)(e))return t.width();t.width(e)}}}),L=F.inherit({_renderSeparator:function(){this.callBase();var e=this.element();e.addClass(this.addWidgetPrefix(y))},_subscribeToCallback:function(){var e,t=this;t._positionChanged=function(n){
e=t.element(),e&&(e.css({top:n.top}),e.height(n.height))},t._tablePositionController.positionChanged.add(t._positionChanged)},_unsubscribeFromCallback:function(){this._positionChanged&&this._tablePositionController.positionChanged.remove(this._positionChanged)},_init:function(){this._isTransparent=M(this),this.isVisible()&&this._subscribeToCallback()},isVisible:function(){return this.option("showColumnHeaders")&&(B(this)||M(this))},optionChanged:function(e){"allowColumnResizing"===e.name&&(e.value?(this._init(),this._invalidate(),this.hide(!0)):(this._unsubscribeFromCallback(),this._isTransparent=M(this),this.hide(!0))),this.callBase(e)},init:function(){this.callBase(),this._tablePositionController=this.getController("tablePosition"),this._init()},show:function(){var e=this,t=this.element();t&&!e._isShown&&(e._isTransparent?t.removeClass(e.addWidgetPrefix(b)):t.show()),this.callBase()},hide:function(e){var t=this.element(),n=this.addWidgetPrefix(b);t&&(this._isShown||e)&&(this._isTransparent?(t.addClass(n),t.css("left",""),t.show()):(t.hasClass(n)&&t.removeClass(n),t.hide())),this.callBase()},moveByX:function(e){var t=this.element();t&&t.css("left",e-this._parentElement().offset().left)},changeCursor:function(e){e=(0,l.isDefined)(e)?e:"";var t=this.element();t&&t.css("cursor",e)},dispose:function(){this._unsubscribeFromCallback(),this.callBase()}}),H=F.inherit({init:function(){var e=this;this.callBase(),this.getController("data").loadingChanged.add(function(t){t||e.hide()})},_renderSeparator:function(){this.callBase(),this.element().addClass(D).html("&nbsp;")},hide:function(){var e=this,t=this._parentElement(),n=this.element();n&&this._isShown&&n.css("display","none"),t&&!t.children("."+D).length&&t.prepend(e.element()),e.callBase()},isVisible:function(){var e=this.option("groupPanel"),t=this.option("columnChooser");return e&&e.visible||t&&t.enabled},show:function(e){var t=this,n=this.element(),i=function(e){m.default.stop(n,!0),m.default.animate(n,{type:"slide",from:{width:0,display:e.display},to:e,duration:300,easing:"swing"})};if(n&&!t._isShown)switch(e){case"group":i({width:"50px",display:"inline-block"});break;case"columnChooser":i({width:"100%",display:"block"});break;default:n.css("display","")}t.callBase()}}),z=g.default.View.inherit({_isDragging:!1,isDragging:function(){return this._isDragging},_getDraggingPanelByPos:function(e){var t,n=this;return(0,u.each)(n._dragOptions.draggingPanels,function(n,i){if(i){var o=i.getBoundingRect();if(o&&(void 0===o.bottom||e.y<o.bottom)&&(void 0===o.top||e.y>o.top)&&(void 0===o.left||e.x>o.left)&&(void 0===o.right||e.x<o.right))return t=i,!1}}),t},_renderCore:function(){this.element().addClass(this.addWidgetPrefix(x)+" "+this.addWidgetPrefix(w)+" "+T).hide()},_resetTargetColumnOptions:function(){var e=this._dropOptions;e.targetColumnIndex=-1,delete e.targetColumnElement,delete e.isLast,delete e.posX,delete e.posY},_getVisibleIndexObject:function(e,t){return(0,l.isDefined)(e)?{columnIndex:t,rowIndex:e}:t},dispose:function(){var e=this.element();this._dragOptions=null,e&&e.parent().find("."+this.addWidgetPrefix(x)).remove()},isVisible:function(){var e=this.getController("columns"),t=e.getCommonSettings();return this.option("showColumnHeaders")&&(B(this)||t.allowGrouping||t.allowHiding)},init:function(){var e=this;this.callBase(),this._controller=this.getController("draggingHeader"),this._columnsResizerViewController=this.getController("columnsResizer"),this.getController("data").loadingChanged.add(function(t){var n=e.element();!t&&n&&n.hide()})},dragHeader:function(e){var t=this,n=e.columnElement,i=!!e.sourceColumn.type;t._isDragging=!0,t._dragOptions=e,t._dropOptions={sourceIndex:e.index,sourceColumnIndex:t._getVisibleIndexObject(e.rowIndex,e.columnIndex),sourceColumnElement:e.columnElement,sourceLocation:e.sourceLocation};var o=a.default.getDocument();t._onSelectStart=o.onselectstart,o.onselectstart=function(){return!1},t._controller.drag(t._dropOptions),t.element().css({textAlign:n&&n.css("textAlign"),height:n&&(i&&n.get(0).clientHeight||n.height()),width:n&&(i&&n.get(0).clientWidth||n.width()),whiteSpace:n&&n.css("whiteSpace")}).addClass(t.addWidgetPrefix(C)).toggleClass(O,i).text(i?"":e.sourceColumn.caption),t.element().appendTo(v.default.getSwatchContainer(n))},moveHeader:function(e){var t=e.event,n=t.data.that,i=(0,h.eventData)(t),o=!!n._columnsResizerViewController&&n._columnsResizerViewController.isResizing(),a=n._dragOptions;if(n._isDragging&&!o){var r=n.element(),s=Math.abs(i.x-a.columnElement.offset().left-a.deltaX),l=Math.abs(i.y-a.columnElement.offset().top-a.deltaY);if(r.is(":visible")||s>P||l>P){r.show();var u=i.x-a.deltaX,c=i.y-a.deltaY;r.css({left:u,top:c}),n.dockHeader(i)}t.preventDefault()}},dockHeader:function(e){var t=this,n=t._getDraggingPanelByPos(e),i=t._controller,a=t._dropOptions,r=t._dragOptions;if(n){var s=t.option("rtlEnabled"),l="columnChooser"===n.getName(),u=l?"y":"x",d=n.getName(),h="headers"===d?r.rowIndex:void 0,f=r.sourceColumn,p=n.getColumnElements(h,f&&f.ownerBand)||[],g=r.pointsByTarget=r.pointsByTarget||{},_="columnChooser"===d?[]:g[d]||i._generatePointsByColumns((0,c.extend)({},r,{targetDraggingPanel:n,columns:n.getColumns(h),columnElements:p,isVerticalOrientation:l,startColumnIndex:"headers"===d&&(0,o.default)(p[0]).index()}));if(g[d]=_,a.targetLocation=d,_.length>0)for(var m=0;m<_.length;m++){var v=_[m+1]&&(_[m][u]+_[m+1][u])/2;if(void 0===v||(s&&"x"===u?e[u]>v:e[u]<v)){a.targetColumnIndex=t._getVisibleIndexObject(h,_[m].columnIndex),p[m]?(a.targetColumnElement=p.eq(m),a.isLast=!1):(a.targetColumnElement=p.last(),a.isLast=!0),a.posX=_[m].x,a.posY=_[m].y,i.dock(a);break}}else t._resetTargetColumnOptions(),i.dock(a)}},dropHeader:function(e){var t=e.event,n=t.data.that,i=n._controller;n.element().hide(),i&&n._isDragging&&i.drop(n._dropOptions),n.element().appendTo(n._parentElement()),n._dragOptions=null,n._dropOptions=null,n._isDragging=!1,a.default.getDocument().onselectstart=n._onSelectStart||null}}),N=function(e){return"widget"!==e.option("columnResizingMode")},j=g.default.ViewController.inherit({_isHeadersRowArea:function(e){if(this._columnHeadersView){var t=this._columnHeadersView.element();if(t){var n=t.offset().top,i=this._columnHeadersView.getHeadersRowHeight();return e>=n&&e<=n+i}}return!1},_isRtlParentStyle:function(){var e;return this.option("rtlEnabled")&&"rtl"===(null===(e=this._$parentContainer)||void 0===e?void 0:e.parent().css("direction"))},_pointCreated:function(e,t,n){var i=N(this),o=this.option("rtlEnabled"),a=this._isRtlParentStyle(),r=i||!o||a?1:0;if(e.index>=r&&e.index<t+(i||o&&!a?0:1)){e.columnIndex-=r;var s=n[e.columnIndex]||{},l=n[e.columnIndex+1]||{};return!(i?s.allowResizing&&l.allowResizing:s.allowResizing)}return!0},_getTargetPoint:function(e,t,n){if(e)for(var i=0;i<e.length;i++)if((e[i].x!==e[0].x||!e[i+1]||e[i].x!==e[i+1].x)&&e[i].x-n<=t&&t<=e[i].x+n)return e[i];return null},_moveSeparator:function(e){var t=e.event,n=t.data,i=n._columnsSeparatorView.width(),o=N(n),a=i/2,r=n._$parentContainer.offset(),s=r.left,l=(0,h.eventData)(t),u=n.option("rtlEnabled"),c=this._isRtlParentStyle();if(n._isResizing&&n._resizingInfo){if((s<=l.x||!o&&c)&&(!o||l.x<=s+n._$parentContainer.width())&&n._updateColumnsWidthIfNeeded(l.x)){var d=n._columnHeadersView.getColumnElements().eq(n._resizingInfo.currentColumnIndex),f=d[0];if(f){var p=f.getBoundingClientRect().width;n._columnsSeparatorView.moveByX(d.offset().left+((o||c)&&u?0:p)),n._tablePositionController.update(n._targetPoint.y),t.preventDefault()}}}else n._isHeadersRowArea(l.y)?(n._previousParentOffset&&(n._previousParentOffset.left===r.left&&n._previousParentOffset.top===r.top||n.pointsByColumns(null)),n._targetPoint=n._getTargetPoint(n.pointsByColumns(),l.x,i),n._previousParentOffset=r,n._isReadyResizing=!1,n._targetPoint?(n._columnsSeparatorView.changeCursor("col-resize"),n._columnsSeparatorView.moveByX(n._targetPoint.x-a),n._tablePositionController.update(n._targetPoint.y),n._isReadyResizing=!0,t.preventDefault()):n._columnsSeparatorView.changeCursor()):(n.pointsByColumns(null),n._isReadyResizing=!1,n._columnsSeparatorView.changeCursor())},_endResizing:function(e){var t=e.event,n=t.data;n._isResizing&&(n.pointsByColumns(null),n._resizingInfo=null,n._columnsSeparatorView.hide(),n._columnsSeparatorView.changeCursor(),n._trackerView.hide(),n._isReadyResizing=!1,n._isResizing=!1)},_getNextColumnIndex:function(e){return e+1},_setupResizingInfo:function(e){var t=this,n=t._targetPoint.columnIndex,i=t._getNextColumnIndex(n),o=t._columnHeadersView.getHeaderElement(n),a=t._columnHeadersView.getHeaderElement(i);t._resizingInfo={startPosX:e,currentColumnIndex:n,currentColumnWidth:o&&o.length>0?(0,d.getBoundingRect)(o[0]).width:0,nextColumnIndex:i,nextColumnWidth:a&&a.length>0?(0,d.getBoundingRect)(a[0]).width:0}},_startResizing:function(e){var t=e.event,n=t.data,i=(0,h.eventData)(t);if((0,h.isTouchEvent)(t)&&(n._isHeadersRowArea(i.y)?(n._targetPoint=n._getTargetPoint(n.pointsByColumns(),i.x,A),n._targetPoint&&(n._columnsSeparatorView.moveByX(n._targetPoint.x-n._columnsSeparatorView.width()/2),n._isReadyResizing=!0)):n._isReadyResizing=!1),n._isReadyResizing){n._setupResizingInfo(i.x),n._isResizing=!0,n._tablePositionController.update(n._targetPoint.y),n._columnsSeparatorView.show(),n._trackerView.show();var o=n.component.getScrollable();o&&n._isRtlParentStyle()&&(n._scrollRight=o.$content().width()-o._container().width()-o.scrollLeft()),t.preventDefault(),t.stopPropagation()}this.isResizing()&&this.getController("editorFactory").loseFocus()},_generatePointsByColumns:function(){var e=this,t=e._columnsController?e._columnsController.getVisibleColumns():[],n=e._columnHeadersView.getColumnElements(),i=[];n&&n.length>0&&(i=_.default.getPointsByColumns(n,function(i){return e._pointCreated(i,n.length,t)})),e._pointsByColumns=i},_unsubscribeFromEvents:function(){this._moveSeparatorHandler&&r.default.off(a.default.getDocument(),(0,h.addNamespace)(f.default.move,E),this._moveSeparatorHandler),this._startResizingHandler&&r.default.off(this._$parentContainer,(0,h.addNamespace)(f.default.down,E),this._startResizingHandler),this._endResizingHandler&&(r.default.off(this._columnsSeparatorView.element(),(0,h.addNamespace)(f.default.up,E),this._endResizingHandler),r.default.off(a.default.getDocument(),(0,h.addNamespace)(f.default.up,E),this._endResizingHandler))},_subscribeToEvents:function(){this._moveSeparatorHandler=this.createAction(this._moveSeparator),this._startResizingHandler=this.createAction(this._startResizing),this._endResizingHandler=this.createAction(this._endResizing),r.default.on(a.default.getDocument(),(0,h.addNamespace)(f.default.move,E),this,this._moveSeparatorHandler),r.default.on(this._$parentContainer,(0,h.addNamespace)(f.default.down,E),this,this._startResizingHandler),r.default.on(this._columnsSeparatorView.element(),(0,h.addNamespace)(f.default.up,E),this,this._endResizingHandler),r.default.on(a.default.getDocument(),(0,h.addNamespace)(f.default.up,E),this,this._endResizingHandler)},_updateColumnsWidthIfNeeded:function(e){function t(e){return(0,l.isString)(e)&&"%"===e.slice(-1)}function n(e,n,i,o){if(e){var a=e.width;a&&(o=t(a)),o?(e&&h.columnOption(e.index,"visibleWidth",n),e&&h.columnOption(e.index,"width",(n/i*100).toFixed(3)+"%")):(e&&h.columnOption(e.index,"visibleWidth",null),e&&h.columnOption(e.index,"width",n))}}function i(e,n){var i=n.every(function(e){return e.width});if(i){var o=n.reduce(function(e,n){return t(n.width)&&(e+=parseFloat(n.width)),e},0);o>100&&(e=e/o*100)}return e}var o,a,r,s,u,c=!1,d=this._resizingInfo,h=this._columnsController,f=h.getVisibleColumns(),p=this._columnsSeparatorView.width(),g=this._rowsView.contentWidth(),m=N(this),v=m&&this.option("adaptColumnWidthByRatio")&&!this.option("columnAutoWidth"),y=this.option("rtlEnabled"),b=this._isRtlParentStyle();o=e-d.startPosX,(m||b)&&y&&(o=-o),u=d.currentColumnWidth+o;var x=f[d.currentColumnIndex];if(r=x&&x.minWidth||p,c=u>=r,m&&(a=d.nextColumnWidth-o,s=f[d.nextColumnIndex],r=s&&s.minWidth||p,c=c&&a>=r),c){if(h.beginUpdate(),u=Math.floor(u),g=i(g,f),n(x,u,g,v),m)a=Math.floor(a),n(s,a,g,v);else{var w=this._columnHeadersView.getColumnWidths();w[d.currentColumnIndex]=u;var C=w.reduce(function(e,t){return e+t},0)>this._rowsView.contentWidth();if(!C){var k=_.default.getLastResizableColumnIndex(f);k>=0&&h.columnOption(f[k].index,"visibleWidth","auto")}for(var S=0;S<w.length;S++)f[S]&&f[S]!==x&&void 0===f[S].width&&h.columnOption(f[S].index,"width",w[S])}if(h.endUpdate(),!m){this.component.updateDimensions();var D=this.component.getScrollable();if(D&&b){var I=D.$content().width()-D._container().width()-this._scrollRight;D.scrollTo({left:I})}}}return c},_subscribeToCallback:function(e,t){e.add(t),this._subscribesToCallbacks.push({callback:e,handler:t})},_unsubscribeFromCallbacks:function(){for(var e=0;e<this._subscribesToCallbacks.length;e++){var t=this._subscribesToCallbacks[e];t.callback.remove(t.handler)}this._subscribesToCallbacks=[]},_unsubscribes:function(){this._unsubscribeFromEvents(),this._unsubscribeFromCallbacks()},_init:function(){var e=this,t=function(){e._isResizing||e.pointsByColumns(null)},n=function(t){e._scrollLeft!==t.left&&(e._scrollLeft=t.left,e.pointsByColumns(null))};e._columnsSeparatorView=e.getView("columnsSeparatorView"),e._columnHeadersView=e.getView("columnHeadersView"),e._trackerView=e.getView("trackerView"),e._rowsView=e.getView("rowsView"),e._columnsController=e.getController("columns"),e._tablePositionController=e.getController("tablePosition"),e._$parentContainer=e.component.$element(),e._subscribeToCallback(e._columnHeadersView.renderCompleted,t),e._subscribeToCallback(e._columnHeadersView.resizeCompleted,t),e._subscribeToCallback(e._columnsSeparatorView.renderCompleted,function(){e._unsubscribeFromEvents(),e._subscribeToEvents()}),e._subscribeToCallback(e._rowsView.renderCompleted,function(){e._rowsView.scrollChanged.remove(n),e._rowsView.scrollChanged.add(n)});var i=0!==e._rowsView.getScrollbarWidth(),o=0;e._subscribeToCallback(e.getController("tablePosition").positionChanged,function(t){if(e._isResizing&&!e._rowsView.isResizing){var n=0!==e._rowsView.getScrollbarWidth();i!==n||o&&o!==t.height?(i=n,o=t.height,e.component.updateDimensions()):e._rowsView.updateFreeSpaceRowHeight()}o=t.height})},optionChanged:function(e){this.callBase(e),"allowColumnResizing"===e.name&&(e.value?(this._init(),this._subscribeToEvents()):this._unsubscribes())},isResizing:function(){return this._isResizing},init:function(){this._subscribesToCallbacks=[],M(this)&&this._init()},pointsByColumns:function(e){return void 0===e?(this._pointsByColumns||this._generatePointsByColumns(),this._pointsByColumns):void(this._pointsByColumns=e)},dispose:function(){this._unsubscribes(),this.callBase()}}),W=g.default.ViewController.inherit({update:function(e){var t=this,n={},i=t._columnHeadersView.element(),o=i&&i.offset(),a=o&&o.top||0,r=(0,l.isDefined)(e)?Math.abs(e-a):0,s=t._columnHeadersView?t._columnHeadersView.getHeight():0,u=t._rowsView.getScrollbarWidth(!0),c=t._rowsView?t._rowsView.height()-u:0,d=t.component.getController("columnsResizer"),h=t.component.getView("draggingHeaderView");n.height=s;var f=d.isResizing()!==!1||h.isDragging();f&&(n.height+=c-r),null!==e&&i&&i.length&&(n.top=i[0].offsetTop+r),t.positionChanged.fire(n)},init:function(){var e=this;e.callBase(),e._columnHeadersView=this.getView("columnHeadersView"),e._rowsView=this.getView("rowsView"),e._pagerView=this.getView("pagerView"),e._rowsView.resizeCompleted.add(function(){if(e.option("allowColumnResizing")){var t=e.getController("columnsResizer")._targetPoint;e.update(t?t.y:null)}})},ctor:function(e){this.callBase(e),this.positionChanged=(0,s.default)()}}),G=g.default.ViewController.inherit({_generatePointsByColumns:function(e){var t=this;return _.default.getPointsByColumns(e.columnElements,function(n){return t._pointCreated(n,e.columns,e.targetDraggingPanel.getName(),e.sourceColumn)},e.isVerticalOrientation,e.startColumnIndex)},_pointCreated:function(e,t,n,i){var o=t[e.columnIndex],a=t[e.columnIndex-1];switch(n){case"columnChooser":return!0;case"headers":return i&&!i.allowReordering||(!o||!o.allowReordering)&&(!a||!a.allowReordering);default:return 0===t.length}},_subscribeToEvents:function(e,t){var n=this;(0,u.each)(t,function(i,a){if(a)for(var s,l=a.getRowCount?a.getRowCount():1,c=a.getName(),d=function(i,l){if(l){var u=(0,o.default)(l),d=s[i];a.allowDragging(d,c,t)&&(u.addClass(n.addWidgetPrefix(C)),r.default.on(u,(0,h.addNamespace)(p.start,E),n.createAction(function(a){var r=a.event,s=(0,h.eventData)(r);e.dragHeader({deltaX:s.x-(0,o.default)(r.currentTarget).offset().left,deltaY:s.y-(0,o.default)(r.currentTarget).offset().top,sourceColumn:d,index:d.index,columnIndex:i,columnElement:u,sourceLocation:c,draggingPanels:t,rowIndex:n._columnsController.getRowIndex(d.index,!0)})})),r.default.on(u,(0,h.addNamespace)(p.move,E),{that:e},n.createAction(e.moveHeader)),r.default.on(u,(0,h.addNamespace)(p.end,E),{that:e},n.createAction(e.dropHeader)))}},f=0;f<l;f++){var g=a.getColumnElements(f)||[];g.length&&(s=a.getColumns(f)||[],(0,u.each)(g,d))}})},_unsubscribeFromEvents:function(e,t){var n=this;(0,u.each)(t,function(e,t){if(t){var i=t.getColumnElements()||[];(0,u.each)(i,function(e,t){var i=(0,o.default)(t);r.default.off(i,(0,h.addNamespace)(p.start,E)),r.default.off(i,(0,h.addNamespace)(p.move,E)),r.default.off(i,(0,h.addNamespace)(p.end,E)),i.removeClass(n.addWidgetPrefix(C))})}})},_getSeparator:function(e){return"headers"===e?this._columnsSeparatorView:this._blockSeparatorView},hideSeparators:function(e){var t=this._blockSeparatorView,n=this._columnsSeparatorView;this._animationColumnIndex=null,t&&t.hide(),"block"!==e&&n&&n.hide()},init:function(){var e=this;e.callBase(),e._columnsController=e.getController("columns"),e._columnHeadersView=e.getView("columnHeadersView"),e._columnsSeparatorView=e.getView("columnsSeparatorView"),e._draggingHeaderView=e.getView("draggingHeaderView"),e._rowsView=e.getView("rowsView"),e._blockSeparatorView=e.getView("blockSeparatorView"),e._headerPanelView=e.getView("headerPanel"),e._columnChooserView=e.getView("columnChooserView");var t=function(){if(e._draggingHeaderView){var t=[e._columnChooserView,e._columnHeadersView,e._headerPanelView];e._unsubscribeFromEvents(e._draggingHeaderView,t),e._subscribeToEvents(e._draggingHeaderView,t)}};e._columnHeadersView.renderCompleted.add(t),e._headerPanelView&&e._headerPanelView.renderCompleted.add(t),e._columnChooserView&&e._columnChooserView.renderCompleted.add(t)},allowDrop:function(e){return this._columnsController.allowMoveColumn(e.sourceColumnIndex,e.targetColumnIndex,e.sourceLocation,e.targetLocation)},drag:function(e){var t=e.sourceIndex,n=e.sourceLocation,i=e.sourceColumnElement,o=this._columnHeadersView,a=this._rowsView;i&&(i.css({opacity:R}),"headers"===n&&(o&&o.setRowsOpacity(t,R),a&&a.setRowsOpacity(t,R)))},dock:function(e){var t=this,n=(0,l.isObject)(e.targetColumnIndex)?e.targetColumnIndex.columnIndex:e.targetColumnIndex,i=e.sourceLocation,o=e.targetLocation,a=t._getSeparator(o),r=n>=0,s=function(){t._animationColumnIndex!==n&&(t.hideSeparators(),a.element()[e.isLast?"insertAfter":"insertBefore"](e.targetColumnElement),t._animationColumnIndex=n,a.show(o))};t._columnHeadersView.element().find("."+I).toggleClass(t.addWidgetPrefix(S),"headers"!==i&&"headers"===o&&!r),a&&(t.allowDrop(e)&&r?"group"===o||"columnChooser"===o?s():(t.hideSeparators("block"),t.getController("tablePosition").update(e.posY),a.moveByX(e.posX-a.width()),a.show()):t.hideSeparators())},drop:function(e){var t=e.sourceColumnElement;if(t&&(t.css({opacity:""}),this._columnHeadersView.setRowsOpacity(e.sourceIndex,""),this._rowsView.setRowsOpacity(e.sourceIndex,""),this._columnHeadersView.element().find("."+I).removeClass(this.addWidgetPrefix(S))),this.allowDrop(e)){var n=this._getSeparator(e.targetLocation);n&&n.hide(),this._columnsController.moveColumn(e.sourceColumnIndex,e.targetColumnIndex,e.sourceLocation,e.targetLocation)}},dispose:function(){this._draggingHeaderView&&this._unsubscribeFromEvents(this._draggingHeaderView,[this._columnChooserView,this._columnHeadersView,this._headerPanelView])}}),q={views:{columnsSeparatorView:L,blockSeparatorView:H,draggingHeaderView:z,trackerView:V},controllers:{draggingHeader:G,tablePosition:W,columnsResizer:j},extenders:{views:{rowsView:{_needUpdateRowHeight:function(e){var t=this.option("wordWrapEnabled"),n=this.getController("columnsResizer"),i=n.isResizing();return this.callBase.apply(this,arguments)||e>0&&t&&i}}},controllers:{editorFactory:{renderFocusOverlay:function(){if(!this.getController("columnsResizer").isResizing())return this.callBase.apply(this,arguments)}}}}};t.default=q,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e){return e&&e.hasClass(L)}function l(e){return e&&e.hasClass(N)}function u(e){return e&&!s(e)&&!l(e)}function c(e){return!e||e.hasClass(j)||e.hasClass(W)}function d(e){return e.length&&"TD"===e[0].tagName}function h(e,t){return!e._isRowEditMode()&&t&&!t.hasClass(U)&&t.hasClass(q)}function f(e){return(0,w.isDefined)(e)&&e.length>0}function p(){return"desktop"!==P.default.current().deviceType}function g(e){return!!e.parent(".".concat(H)).length}function _(e,t){var n=e.option("rtlEnabled"),i=!1;return i=n?!("right"===t.fixedPosition||(0,w.isDefined)(t.command)&&!(0,w.isDefined)(t.fixedPosition)):!(!(0,w.isDefined)(t.fixedPosition)||"left"===t.fixedPosition)}t.default=void 0;var m=r(n(2)),v=r(n(11)),y=r(n(5)),b=r(n(44)),x=r(n(37)),w=n(1),C=n(12),k=n(75),S=n(8),D=r(n(25)),I=n(21),T=n(4),O=a(n(128)),E=r(n(23)),A=n(119),P=r(n(13)),R="rowsview",M="edit-form",B="group-footer",V="dx-row",F="dx-data-row",L="dx-group-row",H="dx-header-row",z="edit-form-item",N="dx-master-detail-row",j="dx-freespace-row",W="dx-virtual-row",G="dx-master-detail-cell",q="dx-editor-cell",$="dx-dropdowneditor-overlay",K="dx-command-expand",U="dx-command-select",Y="dx-command-edit",Z="[class^=dx-command]",X="dx-cell-focus-disabled",Q="dxDateBox",J="dx-state-focused",ee="dx-widget",te="dx-revert-button",ne="delete",ie="input:not([type='hidden']), textarea, a, select, button, [tabindex]",oe="row",ae="form",re="batch",se="cell",le="row",ue="cell",ce="columnHeadersView",de=b.default.ViewController.inherit({init:function(){var e=this;if(this._dataController=this.getController("data"),this._selectionController=this.getController("selection"),this._editingController=this.getController("editing"),this._headerPanel=this.getView("headerPanel"),this._columnsController=this.getController("columns"),this._editorFactory=this.getController("editorFactory"),this.isKeyboardEnabled()){O.subscribeVisibilityChange(),this._updateFocusTimeout=null,this._fastEditingStarted=!1,this._focusedCellPosition={},this._canceledCellPosition=null;var t=function(t){e.setupFocusedView(),e._isNeedScroll&&t.is(":visible")&&e._focusedView&&e._focusedView.getScrollable&&(e._focusedView._scrollToElement(t),e._isNeedScroll=!1)};this._editorFactory.focused.add(t),this._initViewHandlers(),this._initDocumentHandlers(),this.createAction("onKeyDown")}},_initViewHandlers:function(){var e=this,t=this.getView("rowsView"),n=function(n){var i=(0,m.default)(n.target),o=(0,m.default)(n.relatedTarget).closest(t.element()).length,a=i.hasClass("dx-link");if(a&&!o){var r=e._getFocusedCell();r=f(r)?r:t.getCellElements(0).filter("[tabindex]").eq(0),i.closest(r).length||(n.preventDefault(),y.default.trigger(r,"focus"))}};t.renderCompleted.add(function(i){var o=t.element(),a=!i||"refresh"===i.changeType,r=e._focusedView&&e._focusedView.name===t.name,s=!1,l=i&&("append"===i.changeType||"prepend"===i.changeType),u=(0,m.default)(":focus"),c=!u.length||u.closest(o).length||E.default.msie&&u.is("body");y.default.off(o,"focusin",n),y.default.on(o,"focusin",n),e._initPointerEventHandler(),e._initKeyDownHandler(),e._setRowsViewAttributes(),r&&c&&(s=e._isNeedFocus?!l:e._isHiddenFocus&&a,s&&e._updateFocus(!0))})},_initDocumentHandlers:function(){var e=this,t=v.default.getDocument();this._documentClickHandler=this.createAction(function(t){var n=(0,m.default)(t.event.target),i=e._isEventInCurrentGrid(t.event)&&n.closest(".".concat(e.addWidgetPrefix(R))).length,o=n.closest(".".concat($)).length,a=e.getController("columnsResizer"),r=!!a&&a.isResizing();i||o||r||e._resetFocusedView()}),y.default.on(t,(0,S.addNamespace)(D.default.down,"dxDataGridKeyboardNavigation"),this._documentClickHandler)},_setRowsViewAttributes:function(){var e=this._getRowsViewElement(),t=!this._dataController.getVisibleRows().length;t&&this._applyTabIndexToElement(e)},_initPointerEventHandler:function(){var e=p()?I.name:D.default.down,t=".".concat(V," > td, .").concat(V),n=this._getRowsViewElement();(0,w.isDefined)(this._pointerEventAction)||(this._pointerEventAction=this.createAction(this._pointerEventHandler)),y.default.off(n,(0,S.addNamespace)(e,"dxDataGridKeyboardNavigation"),this._pointerEventAction),y.default.on(n,(0,S.addNamespace)(e,"dxDataGridKeyboardNavigation"),t,this._pointerEventAction)},_initKeyDownHandler:function(){var e=this,t=this._getRowsViewElement();A.keyboard.off(this._keyDownListener),this._keyDownListener=A.keyboard.on(t,null,function(t){return e._keyDownHandler(t)})},dispose:function(){this.callBase(),this._resetFocusedView(),A.keyboard.off(this._keyDownListener),y.default.off(v.default.getDocument(),(0,S.addNamespace)(D.default.down,"dxDataGridKeyboardNavigation"),this._documentClickHandler),clearTimeout(this._updateFocusTimeout),O.unsubscribeVisibilityChange()},optionChanged:function(e){var t=this;switch(e.name){case"keyboardNavigation":case"useLegacyKeyboardNavigation":e.handled=!0;break;default:t.callBase(e)}},isRowFocusType:function(){return this.focusType===le},isCellFocusType:function(){return this.focusType===ue},setRowFocusType:function(){this.option("focusedRowEnabled")&&(this.focusType=le)},setCellFocusType:function(){this.focusType=ue},_keyDownHandler:function(e){var t=this._editingController.isEditing(),n=!0,i=e.originalEvent,o=this._processOnKeyDown(e);if(!i.isDefaultPrevented()&&(this._isNeedFocus=!0,this._isNeedScroll=!0,this._updateFocusedCellPositionByTarget(i.target),!o)){switch(e.keyName){case"leftArrow":case"rightArrow":this._leftRightKeysHandler(e,t);break;case"upArrow":case"downArrow":e.ctrl?O.selectView("rowsView",this,i):this._upDownKeysHandler(e,t);break;case"pageUp":case"pageDown":this._pageUpDownKeyHandler(e);break;case"space":this._spaceKeyHandler(e,t);break;case"A":e.ctrl?this._ctrlAKeyHandler(e,t):this._beginFastEditing(e.originalEvent);break;case"tab":this._tabKeyHandler(e,t);break;case"enter":this._enterKeyHandler(e,t);break;case"escape":this._escapeKeyHandler(e,t);break;case"F":e.ctrl?this._ctrlFKeyHandler(e):this._beginFastEditing(e.originalEvent);break;case"F2":this._f2KeyHandler();break;case"del":case"backspace":this._isFastEditingAllowed()&&!this._isFastEditingStarted()&&this._beginFastEditing(i,!0);break;default:this._beginFastEditing(i)||(this._isNeedFocus=!1,this._isNeedScroll=!1,n=!1)}n&&i.stopPropagation()}},_processOnKeyDown:function(e){var t=e.originalEvent,n={handled:!1,event:t};return this.executeAction("onKeyDown",n),e.ctrl=t.ctrlKey,e.alt=t.altKey,e.shift=t.shiftKey,!!n.handled},_closeEditCell:function(){var e=this;setTimeout(function(){e._editingController.closeEditCell()})},_leftRightKeysHandler:function(e,t){var n=this.getVisibleRowIndex(),i=e.originalEvent,o=this._focusedView&&this._focusedView.getRow(n),a=this._getDirectionCodeByKey(e.keyName),r=this._isFastEditingStarted(),s=(!t||r)&&u(o);if(s){this.setCellFocusType(),r&&this._closeEditCell(),this._isVirtualColumnRender()&&this._processVirtualHorizontalPosition(a);var l=this._getNextCell(a);f(l)&&this._arrowKeysHandlerFocusCell(i,l,a),i&&i.preventDefault()}},_upDownKeysHandler:function(e,t){var n=this._focusedCellPosition.rowIndex,i=this.getVisibleRowIndex(),o=this._focusedView&&this._focusedView.getRow(i),a=e.originalEvent,r="upArrow"===e.keyName,s=this._dataController.dataSource(),u=this._isFastEditingStarted(),c=(!t||u)&&o&&!l(o);if(c){if(u&&this._closeEditCell(),!this._navigateNextCell(a,e.keyName)&&this._isVirtualRowRender()&&r&&s&&!s.isLoading()){var d=o.outerHeight();n=this._focusedCellPosition.rowIndex-1,this._scrollBy(0,-d,n,a)}a&&a.preventDefault()}},_pageUpDownKeyHandler:function(e){var t=this._dataController.pageIndex(),n=this._dataController.pageCount(),i=this.option("paging.enabled"),o="pageUp"===e.keyName,a=o?-1:1,r=this.getView("rowsView").getScrollable();i&&!this._isVirtualScrolling()?(o?t>0:t<n-1)&&!this._isVirtualScrolling()&&(this._dataController.pageIndex(t+a),e.originalEvent.preventDefault()):r&&r._container().height()<r.$content().height()&&(this._scrollBy(0,r._container().height()*a),e.originalEvent.preventDefault())},_spaceKeyHandler:function(e,t){var n=this.getVisibleRowIndex(),i=(0,m.default)(e.originalEvent&&e.originalEvent.target);if(this.option("selection")&&"none"!==this.option("selection").mode&&!t){var o="row"===this._getElementType(i)&&this.isRowFocusType()&&u(i),a=i.hasClass(U);a&&"onClick"===this.option("selection.showCheckBoxesMode")&&this._selectionController.startSelectionWithCheckboxes(),(o||i.parent().hasClass(F)||i.hasClass(this.addWidgetPrefix(R)))&&(this._selectionController.changeItemSelection(n,{shift:e.shift,control:e.ctrl}),e.originalEvent.preventDefault())}else this._beginFastEditing(e.originalEvent)},_ctrlAKeyHandler:function(e,t){!t&&e.ctrl&&!e.alt&&"multiple"===this.option("selection.mode")&&this.option("selection.allowSelectAll")&&(this._selectionController.selectAll(),e.originalEvent.preventDefault())},_tabKeyHandler:function(e,t){var n=this.option("editing"),i=e.shift?"previous":"next",o=(0,w.isDefined)(this._focusedCellPosition)&&!(0,w.isEmptyObject)(this._focusedCellPosition),a=!o||!e.shift&&this._isLastValidCell(this._focusedCellPosition)||e.shift&&this._isFirstValidCell(this._focusedCellPosition),r=e.originalEvent.target,s=this._focusedView&&this._focusedView.element();if(!this._handleTabKeyOnMasterDetailCell(r,i)){if((0,m.default)(s).addClass(J),n&&r&&!a)if((0,m.default)(r).hasClass(this.addWidgetPrefix(R))&&this._resetFocusedCell(),this._isVirtualColumnRender()&&this._processVirtualHorizontalPosition(i),t){if(!this._editingCellTabHandler(e,i))return}else this._targetCellTabHandler(e,i)&&(a=!0);a?(this._editorFactory.loseFocus(),this._editingController.isEditing()&&!this._isRowEditMode()&&(this._resetFocusedView(),this._closeEditCell())):e.originalEvent.preventDefault()}},_getMaxHorizontalOffset:function(){var e=this.component.getScrollable(),t=this.getView("rowsView"),n=e?e.scrollWidth()-(0,m.default)(t.element()).width():0;return n},_isColumnRendered:function(e){var t=this._columnsController.getVisibleColumns(null,!0),n=this._columnsController.getVisibleColumns(),i=t[e],o=!1;return i&&(o=n.indexOf(i)>=0),o},_isFixedColumn:function(e){var t=this._columnsController.getVisibleColumns(null,!0),n=t[e];return!!n&&!!n.fixed},_isColumnVirtual:function(e){var t=e-this._columnsController.getColumnIndexOffset(),n=this._columnsController.getVisibleColumns(),i=n[t];return!!i&&"virtual"===i.command},_processVirtualHorizontalPosition:function(e){
var t,n=this.component.getScrollable(),i=this.getColumnIndex(),o=0,a=!1;switch(e){case"next":case"nextInRow":var r=this._getVisibleColumnCount();t=i+1,o=this.option("rtlEnabled")?this._getMaxHorizontalOffset():0,a="next"===e?r===t||this._isFixedColumn(i)&&!this._isColumnRendered(t):r>t&&this._isFixedColumn(i)&&!this._isColumnRendered(t);break;case"previous":case"previousInRow":if(t=i-1,o=this.option("rtlEnabled")?0:this._getMaxHorizontalOffset(),"previous"===e){var s=this._columnsController.getColumnIndexOffset(),l=t<0&&0===s;a=l||this._isFixedColumn(i)&&!this._isColumnRendered(t)}else a=t>=0&&this._isFixedColumn(i)&&!this._isColumnRendered(t)}a?n.scrollTo({left:o}):(0,w.isDefined)(t)&&(0,w.isDefined)(e)&&this._isColumnVirtual(t)&&(o=this._getHorizontalScrollPositionOffset(e),0!==o&&n.scrollBy({left:o,top:0}))},_getHorizontalScrollPositionOffset:function(e){var t=0,n=this._getCell(this._focusedCellPosition),i=n&&n.outerWidth();if(i>0){var o=this.option("rtlEnabled")?-1:1;t="nextInRow"===e||"next"===e?i*o:i*o*-1}return t},_editingCellTabHandler:function(e,t){var n,i=e.originalEvent.target,o=this._getCellElementFromTarget(i),a=e.originalEvent,r=this._getElementType(i);if(o.is(Z))return!this._targetCellTabHandler(e,t);this._updateFocusedCellPosition(o);var s=this._getNextCellByTabKey(a,t,r);if(o=s.$cell,!o||this._handleTabKeyOnMasterDetailCell(o,t))return!1;var l=this._columnsController,u=this.getView("rowsView").getCellIndex(o),c=u+l.getColumnIndexOffset(),d=l.getVisibleColumns(null,!0)[c],h=o&&o.parent(),f=this._getRowIndex(h),p=this._dataController.items()[f],g=this._editingController;if(d&&d.allowEditing){var _=!p||"data"===p.rowType;n=g.allowUpdating({row:p})?_:p&&p.isNewRow}return n||this._closeEditCell(),this._focusCell(o,!s.isHighlighted)&&(!this._isRowEditMode()&&n?this._editFocusedCell():this._focusInteractiveElement(o,e.shift)),!0},_targetCellTabHandler:function(e,t){var n,i=e.originalEvent,o=i.target,a=this._getCellElementFromTarget(o),r=this._getInteractiveElement(a,!e.shift),s=!1;if(!h(this,a)&&r.length&&o!==r.get(0))s=!0;else{void 0===this._focusedCellPosition.rowIndex&&(0,m.default)(o).hasClass(V)&&this._updateFocusedCellPosition(a),n=this._getElementType(o),this.isRowFocusType()&&(this.setCellFocusType(),"row"===n&&u((0,m.default)(o))&&(o=this.getFirstValidCellInRow((0,m.default)(o)),n=this._getElementType(o)));var l=this._getNextCellByTabKey(i,t,n);if(a=l.$cell,!a)return!1;if(a=this._checkNewLineTransition(i,a),!a)return!1;this._focusCell(a,!l.isHighlighted),h(this,a)||this._focusInteractiveElement(a,e.shift)}return s},_getNextCellByTabKey:function(e,t,n){var i=this._getNextCell(t,n),o=i&&this._fireFocusedCellChanging(e,i,!0);return!o||o.cancel?{}:(o.$newCellElement&&(i=o.$newCellElement),{$cell:i,isHighlighted:o.isHighlighted})},_checkNewLineTransition:function(e,t){var n=this.getVisibleRowIndex(),i=t.parent();if(n!==this._getRowIndex(i)){var o=this._getCellPosition(t),a=this._fireFocusedRowChanging(e,i);if(a.cancel)return;a.rowIndexChanged&&(this.setFocusedColumnIndex(o.columnIndex),t=this._getFocusedCell())}return t},_enterKeyHandler:function(e,t){var n=this._getFocusedCell(),i=this.getVisibleRowIndex(),o=this._focusedView&&this._focusedView.getRow(i);if(this.option("grouping.allowCollapsing")&&s(o)||this.option("masterDetail.enabled")&&n&&n.hasClass(K)){var a=this._dataController.getKeyByRowIndex(i),r=this._dataController.items()[i];void 0!==a&&r&&r.data&&!r.data.isContinuation&&this._dataController.changeRowExpand(a)}else this._processEnterKeyForDataCell(e,t)},_processEnterKeyForDataCell:function(e,t){var n=this._getEnterKeyDirection(e),i=this._allowEditingOnEnterKey();t||!i&&n?(this._handleEnterKeyEditingCell(e.originalEvent),"next"===n||"previous"===n?this._targetCellTabHandler(e,n):"upArrow"!==n&&"downArrow"!==n||this._navigateNextCell(e.originalEvent,n)):i&&this._startEditing(e)},_getEnterKeyDirection:function(e){var t=this.option("keyboardNavigation.enterKeyDirection"),n=e.shift;return"column"===t?n?"upArrow":"downArrow":"row"===t?n?"previous":"next":void 0},_handleEnterKeyEditingCell:function(e){var t=e.target,n=this._getCellElementFromTarget(t),i=this._isRowEditMode();this._updateFocusedCellPosition(n),i?(this._focusEditFormCell(n),setTimeout(this._editingController.saveEditData.bind(this._editingController))):(y.default.trigger((0,m.default)(t),"change"),this._closeEditCell(),e.preventDefault())},_escapeKeyHandler:function(e,t){var n=this._getCellElementFromTarget(e.originalEvent.target);t&&(this._updateFocusedCellPosition(n),this._isRowEditMode()?(this._focusEditFormCell(n),this._editingController.cancelEditData(),0===this._dataController.items().length&&(this._resetFocusedCell(),this._editorFactory.loseFocus())):"cell"===this._editingController.getEditMode()?this._editingController.cancelEditData():this._closeEditCell(),e.originalEvent.preventDefault())},_ctrlFKeyHandler:function(e){if(this.option("searchPanel.visible")){var t=this._headerPanel.getSearchTextEditor();t&&(t.focus(),e.originalEvent.preventDefault())}},_f2KeyHandler:function(){var e=this._editingController.isEditing(),t=this.getVisibleRowIndex(),n=this._focusedView&&this._focusedView.getRow(t);!e&&u(n)&&this._startEditing()},_navigateNextCell:function(e,t){var n=this._getNextCell(t),i=this._getDirectionCodeByKey(t),o=n&&this._isCellValid(n),a=!!o&&this._arrowKeysHandlerFocusCell(e,n,i);return a},_arrowKeysHandlerFocusCell:function(e,t,n){var i="prevRow"===n||"nextRow"===n,o=this._fireFocusChangingEvents(e,t,i,!0);return t=o.$newCellElement,!(o.cancel||!this._isCellValid(t))&&(this._focus(t,!o.isHighlighted),!0)},_beginFastEditing:function(e,t){if(!this._isFastEditingAllowed()||e.altKey||e.ctrlKey||this._editingController.isEditing())return!1;if(t)this._startEditing(e,ne);else{var n=e.key,i=e.keyCode||e.which,o=n||i&&String.fromCharCode(i);!o||1!==o.length&&o!==ne||this._startEditing(e,o)}return!0},_pointerEventHandler:function(e){var t=e.event||e,n=(0,m.default)(t.currentTarget),i=this.getView("rowsView"),o=i&&i.element(),a=n.parent(),r=(0,m.default)(t.target).is(ie),s=!!(0,m.default)(t.target).closest(".".concat(te)).length,l=n.hasClass(K);!s&&this._isEventInCurrentGrid(t)&&(this._isCellValid(n,!r)||l)?(n=this._isInsideEditForm(n)?(0,m.default)(t.target):n,this._focusView(),(0,m.default)(o).removeClass(J),a.hasClass(j)?(this._updateFocusedCellPosition(n),this._applyTabIndexToElement(this._focusedView.element()),this._focusedView.focus()):this._isMasterDetailCell(n)?this._updateFocusedCellPosition(n):this._clickTargetCellHandler(t,n)):n.is("td")&&this._resetFocusedCell()},_clickTargetCellHandler:function(e,t){var n=this.getView("rowsView").getCellIndex(t),i=this._columnsController.getVisibleColumns()[n],o=this._isCellEditMode();this.setCellFocusType();var a=this._fireFocusChangingEvents(e,t,!0);if(t=a.$newCellElement,a.cancel)this.setRowFocusType(),this.setFocusedRowIndex(a.prevRowIndex),t=this._getFocusedCell(),this._editingController.isEditing()&&o&&this._closeEditCell();else{if(a.resetFocusedRow)return void this.getController("focus")._resetFocusedRow();if(a.rowIndexChanged&&(t=this._getFocusedCell()),a.isHighlighted||o||this.setRowFocusType(),this._updateFocusedCellPosition(t),this._allowRowUpdating()&&o&&i&&i.allowEditing)this._isNeedFocus=!1,this._isHiddenFocus=!1;else{var r=e&&(0,m.default)(e.target).closest(ie+", td"),s=r&&r.not(t).is(ie),l=!!i&&!i.command&&t.hasClass(q),u=!l&&(!a.isHighlighted||s);this._focus(t,u,s)}}},_allowRowUpdating:function(){var e=this.getVisibleRowIndex(),t=this._dataController.items()[e];return this._editingController.allowUpdating({row:t},"click")},focus:function(e){var t,n=this.option("focusedRowEnabled"),i=d((0,m.default)(e));e||(t=".dx-datagrid-rowsview .dx-row[tabindex]",n||(t+=", .dx-datagrid-rowsview .dx-row > td[tabindex]"),e=this.component.$element().find(t).first()),e&&this._focusElement((0,m.default)(e),i)},getFocusedView:function(){return this._focusedView},setupFocusedView:function(){this.isKeyboardEnabled()&&!(0,w.isDefined)(this._focusedView)&&this._focusView()},_focusElement:function(e,t){var n=(0,m.default)(this._getRowsViewElement()),i=e.closest(n),o=this.isRowFocusType(),a={};!i.length||d(e)&&!this._isCellValid(e)||(this._focusView(),this._isNeedFocus=!0,this._isNeedScroll=!0,(d(e)||s(e))&&(this.setCellFocusType(),a=this._fireFocusChangingEvents(null,e,!1,t),e=a.$newCellElement,o&&!a.isHighlighted&&this.setRowFocusType()),a.cancel||(this._focus(e,!a.isHighlighted),this._focusInteractiveElement(e)))},_getFocusedViewByElement:function(e){var t=this.getFocusedView(),n=t&&(0,m.default)(t.element());return e&&0!==e.closest(n).length},_focusView:function(){this._focusedView=this.getView("rowsView")},_resetFocusedView:function(){var e=this._getFocusedCell();this._resetFocusedCell(e,!0),this.setRowFocusType(),this._focusedView=null},_focusInteractiveElement:function(e,t){if(e){var n=this._getInteractiveElement(e,t);x.default.focusAndSelectElement(this,n)}},_focus:function(e,t,n){var i=e&&!e.hasClass(V)?e.closest(".".concat(V)):e;if(!i||!c(i)){var o,a=this._focusedView,r=a&&a.element();this._isHiddenFocus=t,s(i)||this.isRowFocusType()?(o=i,a&&this.setFocusedRowIndex(this._getRowIndex(i))):d(e)&&(o=e,this._updateFocusedCellPosition(e)),o&&(r&&r.find(".dx-row[tabindex], .dx-row > td[tabindex]").not(o).removeClass(X).removeAttr("tabindex"),y.default.one(o,"blur",function(e){e.relatedTarget&&o.removeClass(X)}),n||(this._applyTabIndexToElement(o),y.default.trigger(o,"focus")),t?o.addClass(X):this._editorFactory.focus(o))}},_updateFocus:function(e){var t=this;this._updateFocusTimeout=setTimeout(function(){var n=t._getFocusedCell(),i=t._editingController.isEditing();if(n&&(!t._isMasterDetailCell(n)||t._isRowEditMode())){if(t._hasSkipRow(n.parent())){var o=t._focusedCellPosition&&t._focusedCellPosition.rowIndex>0?"upArrow":"downArrow";n=t._getNextCell(o)}if(f(n)){if(e&&!i&&t._checkCellOverlapped(n))return;if(n.is("td")||n.hasClass(t.addWidgetPrefix(z))){var a=n.is(Z);if(!e&&a||!t._editorFactory.focus())(t._isNeedFocus||t._isHiddenFocus)&&t._focus(n,t._isHiddenFocus);else{var r=n.find(":focus"),s=f(r);if(a&&s)return void x.default.focusAndSelectElement(t,r);!s&&t._focus(n)}i&&t._focusInteractiveElement.bind(t)(n)}else y.default.trigger(n,"focus")}}})},_checkCellOverlapped:function(e){var t=e.offset(),n=this.component.getScrollable&&this.component.getScrollable(),i=!1;return n&&(t.left<0?i=e.width()+t.left<=0:t.top<0&&(i=e.height()+t.top<=0)),i},_getFocusedCell:function(){return(0,m.default)(this._getCell(this._focusedCellPosition))},_updateFocusedCellPositionByTarget:function(e){var t,n=this._getElementType(e);if("row"===n&&(0,w.isDefined)(null===(t=this._focusedCellPosition)||void 0===t?void 0:t.columnIndex)){var i=(0,m.default)(e);this._focusedView&&s(i)&&this.setFocusedRowIndex(this._getRowIndex(i))}else this._updateFocusedCellPosition(this._getCellElementFromTarget(e))},_updateFocusedCellPosition:function(e,t){var n=this._getCellPosition(e,t);return n&&(!e.length||n.rowIndex>=0&&n.columnIndex>=0)&&this.setFocusedCellPosition(n.rowIndex,n.columnIndex),n},_getFocusedColumnIndexOffset:function(e){var t=0,n=this._columnsController.getVisibleColumns()[e];return n&&n.fixed?t=this._getFixedColumnIndexOffset(n):e>=0&&(t=this._columnsController.getColumnIndexOffset()),t},_getFixedColumnIndexOffset:function(e){var t=_(this,e)?this._getVisibleColumnCount()-this._columnsController.getVisibleColumns().length:0;return t},_getCellPosition:function(e,t){var n,i=f(e)&&e.closest("tr"),o=this.getView("rowsView");if(f(i)){var a=this._getRowIndex(i);return n=o.getCellIndex(e,a),n+=this._getFocusedColumnIndexOffset(n),t&&(n="previous"===t?n-1:n+1,n=this._applyColumnIndexBoundaries(n)),{rowIndex:a,columnIndex:n}}},_focusCell:function(e,t){if(this._isCellValid(e))return this._focus(e,t),!0},_focusEditFormCell:function(e){e.hasClass(G)&&this._editorFactory.focus(e,!0)},_resetFocusedCell:function(e,t){var n,i=f(e)?e:this._getFocusedCell();f(i)&&i.removeAttr("tabindex"),this._isNeedFocus=!1,this._isNeedScroll=!1,this._focusedCellPosition={},clearTimeout(this._updateFocusTimeout),null===(n=this._focusedView)||void 0===n?void 0:n.renderFocusState(t)},restoreFocusableElement:function(e,t){var n,i,o=this,a=(0,w.isDefined)(e),r=o.getView("rowsView"),s=r.element(),l=o._focusedCellPosition.columnIndex,u=o._dataController.getRowIndexOffset();e=a?e:r.getTopVisibleItemIndex()+u,a?(i=r.getRow(e-u),n=o._fireFocusedRowChanging(t,i),!n.cancel&&n.rowIndexChanged&&(e=n.newRowIndex)):(o._editorFactory.loseFocus(),o._applyTabIndexToElement(s),y.default.trigger(s,"focus")),a&&n.cancel||o.setFocusedCellPosition(e,l),a&&o._updateFocus()},_getNewPositionByCode:function(e,t,n){var i,o=e.columnIndex,a=e.rowIndex;if(void 0===e.rowIndex&&"next"===n)return{columnIndex:0,rowIndex:0};switch(n){case"nextInRow":case"next":i=this._getVisibleColumnCount(),o<i-1&&"row"!==t&&this._hasValidCellAfterPosition({columnIndex:o,rowIndex:a})?o++:this._isLastRow(a)||"next"!==n||(o=0,a++);break;case"previousInRow":case"previous":o>0&&"row"!==t&&this._hasValidCellBeforePosition({columnIndex:o,rowIndex:a})?o--:a>0&&"previous"===n&&(a--,i=this._getVisibleColumnCount(),o=i-1);break;case"upArrow":a=a>0?a-1:a;break;case"downArrow":a=this._isLastRow(a)?a:a+1}return{columnIndex:o,rowIndex:a}},setFocusedCellPosition:function(e,t){this.setFocusedRowIndex(e),this.setFocusedColumnIndex(t)},setFocusedRowIndex:function(e){this._focusedCellPosition||(this._focusedCellPosition={}),this._focusedCellPosition.rowIndex=e},setFocusedColumnIndex:function(e){this._focusedCellPosition||(this._focusedCellPosition={}),this._focusedCellPosition.columnIndex=e},getRowIndex:function(){return this._focusedCellPosition?this._focusedCellPosition.rowIndex:-1},getColumnIndex:function(){return this._focusedCellPosition?this._focusedCellPosition.columnIndex:-1},getVisibleRowIndex:function(){var e=this._focusedCellPosition&&this._focusedCellPosition.rowIndex;return(0,w.isDefined)(e)?e-this._dataController.getRowIndexOffset():-1},getVisibleColumnIndex:function(){var e=this._focusedCellPosition&&this._focusedCellPosition.columnIndex;return(0,w.isDefined)(e)?e-this._columnsController.getColumnIndexOffset():-1},_applyColumnIndexBoundaries:function(e){var t=this._getVisibleColumnCount();return e<0?e=0:e>=t&&(e=t-1),e},_isCellByPositionValid:function(e){var t=(0,m.default)(this._getCell(e));return this._isCellValid(t)},_isLastRow:function(e){return this._isVirtualRowRender()?e>=this._dataController.totalItemsCount()-1:e===this._dataController.items().length-1},_isFirstValidCell:function(e){var t=!1;return 0===e.rowIndex&&e.columnIndex>=0&&(t=t||!this._hasValidCellBeforePosition(e)),t},_hasValidCellBeforePosition:function(e){for(var t=e.columnIndex,n=!1;t>0&&!n;){var i={columnIndex:--t,rowIndex:e.rowIndex};n=this._isCellByPositionValid(i)}return n},_hasValidCellAfterPosition:function(e){for(var t=e.columnIndex,n=!1,i=this._getVisibleColumnCount();t<i-1&&!n;){var o={columnIndex:++t,rowIndex:e.rowIndex};n=this._isCellByPositionValid(o)}return n},_isLastValidCell:function(e){var t=e.columnIndex>=0?e.columnIndex+1:0,n=e.rowIndex,i={columnIndex:t,rowIndex:n},o=this._dataController.getVisibleRows(),a=o&&o[n],r=this._isLastRow(n);return!!r&&(!!(a&&"group"===a.rowType&&e.columnIndex>0)||(e.columnIndex===this._getVisibleColumnCount()-1||!this._isCellByPositionValid(i)&&this._isLastValidCell(i)))},_isCellValid:function(e,t){if(f(e)){var n=this.getView("rowsView"),i=e.parent(),o=this._columnsController,a=n.getCellIndex(e)+o.getColumnIndexOffset(),r=o.getVisibleColumns(null,!0)[a],s=this._getVisibleColumnCount(),c=this._editingController,d=l(i),h=r&&r.showWhenGrouped,p=r&&!e.hasClass(K)&&u(i),g=function(){return!d&&r&&(!(0,w.isDefined)(r.groupIndex)||h&&p)||parseInt(e.attr("colspan"))>1};if(this._isMasterDetailCell(e))return!0;if(s>a&&g()){var _=this._dataController.items(),m=n.getRowIndex(i),v=_[m],y=c&&this._isCellEditMode()&&c.isEditing(),b=c&&c.isEditRow(m),x=b||y;return r.command?this._isLegacyNavigation()?!x&&"expand"===r.command:!y&&(b?"select"!==r.command:!x):(!y||!v||"data"===v.rowType)&&(!x||r.allowEditing||t)}}},getFirstValidCellInRow:function(e,t){var n,i,o=this,a=e.find("> td");t=t||0;for(var r=t;r<a.length;++r)if(n=a.eq(r),o._isCellValid(n)){i=n;break}return i},_getNextCell:function(e,t,n){var i,o,a=n||this._focusedCellPosition,r=this.isRowFocusType(),s=r||(0,C.inArray)(e,["next","previous"])>-1;if(this._focusedView&&a){var l=this._getNewPositionByCode(a,t,e);i=(0,m.default)(this._getCell(l));var u="previous"===e?this._isFirstValidCell(l):this._isLastValidCell(l);if(f(i)&&!this._isCellValid(i)&&this._isCellInRow(l,s)&&!u&&(i=r?this.getFirstValidCellInRow(i.parent(),l.columnIndex):this._getNextCell(e,"cell",l)),o=f(i)&&i.parent(),this._hasSkipRow(o)){var c=this._getRowIndex(o);if(this._isLastRow(c))return null;i=this._getNextCell(e,"row",{columnIndex:a.columnIndex,rowIndex:c})}return f(i)?i:null}return null},_startEditing:function(e,t){var n=this._focusedCellPosition,i=this.getVisibleRowIndex(),o=this.getVisibleColumnIndex(),a=this._dataController.items()[i],r=this._columnsController.getVisibleColumns()[o];this._isAllowEditing(a,r)&&(this._isRowEditMode()?this._editingController.editRow(i):n&&this._startEditCell(e,t))},_isAllowEditing:function(e,t){return this._editingController.allowUpdating({row:e})&&t&&t.allowEditing},_editFocusedCell:function(){var e=this.getVisibleRowIndex(),t=this.getVisibleColumnIndex();return this._editingController.editCell(e,t)},_startEditCell:function(e,t){var n=this;this._fastEditingStarted=(0,w.isDefined)(t);var i=this._editFocusedCell();if(this._isFastEditingStarted())if(i===!0)this._editingCellHandler(e,t);else if(i&&i.done){var o=t!==ne?t:"";i.done(function(){return n._editingCellHandler(e,o)})}},_editingCellHandler:function(e,t){var n=this._getFocusedCell().find(".dx-texteditor-input").eq(0),i=(0,S.createEvent)(e,{type:"keydown",target:n.get(0)}),o=(0,S.createEvent)(e,{type:"keypress",target:n.get(0)}),a=(0,S.createEvent)(e,{type:"input",target:n.get(0)});if(y.default.trigger(n,i),!i.isDefaultPrevented()&&(y.default.trigger(n,o),!o.isDefaultPrevented())){var r=E.default.mozilla?25:0;setTimeout(function(){n.val(t);var e=n.closest(".".concat(ee));y.default.off(e,"focusout"),y.default.one(e,"focusout",function(){y.default.trigger(n,"change")}),y.default.trigger(n,a)},r)}},_fireFocusChangingEvents:function(e,t,n,i){var o={},a=this._getCellPosition(t)||{};return this.isCellFocusType()&&(o=this._fireFocusedCellChanging(e,t,i),o.cancel||(a.columnIndex=o.newColumnIndex,a.rowIndex=o.newRowIndex,i=o.isHighlighted,t=(0,m.default)(this._getCell(a)))),!o.cancel&&n&&t&&(o=this._fireFocusedRowChanging(e,t.parent()),o.cancel||(a.rowIndex=o.newRowIndex,o.isHighlighted=i)),o.$newCellElement=(0,m.default)(this._getCell(a)),o.$newCellElement.length||(o.$newCellElement=t),o},_fireFocusedCellChanging:function(e,t,n){var i=this,o=i.option("focusedColumnIndex"),a=i.option("focusedRowIndex"),r=i._getCellPosition(t),s=r?r.columnIndex:-1,l=r?r.rowIndex:-1,u={cellElement:t,prevColumnIndex:o,prevRowIndex:a,newColumnIndex:s,newRowIndex:l,rows:i._dataController.getVisibleRows(),columns:i._columnsController.getVisibleColumns(),event:e,isHighlighted:n||!1,cancel:!1};return this._canceledCellPosition=null,i.executeAction("onFocusedCellChanging",u),u.newColumnIndex===s&&u.newRowIndex===l||(u.$newCellElement=(0,m.default)(this._getCell({columnIndex:u.newColumnIndex,rowIndex:u.newRowIndex}))),u.cancel&&(this._canceledCellPosition={rowIndex:l,columnIndex:s}),u},_fireFocusedCellChanged:function(e,t,n){var i=this,o=i._dataController,a=i.getView("rowsView").getCellIndex(e),r=this._getRowIndex(e&&e.parent()),s=Math.min(r-o.getRowIndexOffset(),o.items().length-1),l=i._editingController.isEditCell(s,a),u=o.items()[s];l||t===a&&n===r||i.executeAction("onFocusedCellChanged",{cellElement:e,columnIndex:a,rowIndex:r,row:u,column:i._columnsController.getVisibleColumns()[a]})},_fireFocusedRowChanging:function(e,t){var n=this._getRowIndex(t),i=this._dataController,o=this.option("focusedRowIndex"),a=i.loadingOperationTypes(),r={rowElement:t,prevRowIndex:o,newRowIndex:n,event:e,rows:i.getVisibleRows(),cancel:!1};return!i||i.isLoading()&&(a.reload||a.paging)?(r.cancel=!0,r):(this.option("focusedRowEnabled")&&(this.executeAction("onFocusedRowChanging",r),r.cancel||r.newRowIndex===n||(r.resetFocusedRow=r.newRowIndex<0,r.resetFocusedRow||this.setFocusedRowIndex(r.newRowIndex),r.rowIndexChanged=!0)),r)},_fireFocusedRowChanged:function(e){var t,n=this.option("focusedRowKey"),i=this.getController("focus"),o=null===i||void 0===i?void 0:i.getFocusedRowIndexByKey(n);if(this.option("focusedRowEnabled")){if(o>=0){var a=this._dataController;t=o>=0&&a.getVisibleRows()[o-a.getRowIndexOffset()]}this.executeAction("onFocusedRowChanged",{rowElement:e,rowIndex:o,row:t})}},_isEventInCurrentGrid:function(e){return x.default.isElementInCurrentGrid(this,(0,m.default)(e.target))},_isRowEditMode:function(){var e=this._editingController.getEditMode();return e===oe||e===ae},_isCellEditMode:function(){var e=this._editingController.getEditMode();return e===se||e===re},_isFastEditingAllowed:function(){return this._isCellEditMode()&&this.option("keyboardNavigation.editOnKeyPress")},_getInteractiveElement:function(e,t){var n=e.find(ie).filter(":visible");return t?n.last():n.first()},_applyTabIndexToElement:function(e){var t=this.option("tabIndex")||0;e.attr("tabindex",(0,w.isDefined)(t)?t:0)},_getCell:function(e){if(this._focusedView&&e){var t=this._dataController.getRowIndexOffset(),n=this._columnsController.getVisibleColumns(null,!0)[e.columnIndex],i=n&&n.fixed?this._getFixedColumnIndexOffset(n):this._columnsController.getColumnIndexOffset(),o=e.rowIndex>=0?e.rowIndex-t:-1,a=e.columnIndex>=0?e.columnIndex-i:-1;return this._focusedView.getCell({rowIndex:o,columnIndex:a})}},_getRowIndex:function(e){var t=this.getView("rowsView"),n=t.getRowIndex(e);return n>=0&&(n+=this._dataController.getRowIndexOffset()),n},_hasSkipRow:function(e){var t=e&&e.get(0);return t&&("none"===t.style.display||e.hasClass(this.addWidgetPrefix(B))||l(e)&&!e.hasClass(this.addWidgetPrefix(M)))},_allowEditingOnEnterKey:function(){return"startEdit"===this.option("keyboardNavigation.enterKeyAction")},_isLegacyNavigation:function(){return this.option("useLegacyKeyboardNavigation")},_getDirectionCodeByKey:function(e){var t;switch(e){case"upArrow":t="prevRow";break;case"downArrow":t="nextRow";break;case"leftArrow":t=this.option("rtlEnabled")?"nextInRow":"previousInRow";break;case"rightArrow":t=this.option("rtlEnabled")?"previousInRow":"nextInRow"}return t},_isVirtualScrolling:function(){var e=this.option("scrolling.mode");return"virtual"===e||"infinite"===e},_isVirtualRowRender:function(){return this._isVirtualScrolling()||"virtual"===this.option("scrolling.rowRenderingMode")},_isVirtualColumnRender:function(){return"virtual"===this.option("scrolling.columnRenderingMode")},_scrollBy:function(e,t,n,i){var o=this,a=this.getView("rowsView").getScrollable();if(o._focusedCellPosition){var r=function e(){a.off("scroll",e),setTimeout(o.restoreFocusableElement.bind(o,n,i))};a.on("scroll",r)}return a.scrollBy({left:e,top:t})},_isInsideEditForm:function(e){return(0,m.default)(e).closest("."+this.addWidgetPrefix(M)).length>0},_isMasterDetailCell:function(e){var t=(0,m.default)(e).closest("."+G),n=t.closest("."+this.getWidgetContainerClass()).parent();return t.length&&n.is(this.component.$element())},_processNextCellInMasterDetail:function(e){!this._isInsideEditForm(e)&&e&&this._applyTabIndexToElement(e)},_handleTabKeyOnMasterDetailCell:function(e,t){if(this._isMasterDetailCell(e)){this._updateFocusedCellPosition((0,m.default)(e),t);var n=this._getNextCell(t,"row");return this._processNextCellInMasterDetail(n),!0}return!1},_getElementType:function(e){return(0,m.default)(e).is("tr")?"row":"cell"},_isFastEditingStarted:function(){return this._isFastEditingAllowed()&&this._fastEditingStarted},_getVisibleColumnCount:function(){return this._columnsController.getVisibleColumns(null,!0).length},_isCellInRow:function(e,t){var n=e.columnIndex,i=this._getVisibleColumnCount();return t?n>=0&&n<=i-1:n>0&&n<i-1},_getCellElementFromTarget:function(e){var t,n=this._getElementType(e),i=(0,m.default)(e);return t="cell"===n?i.closest(".".concat(V," > td")):i.children().not("."+K).first()},_getRowsViewElement:function(){var e=this.getView("rowsView");return e&&e.element()},isKeyboardEnabled:function(){return this.option("keyboardNavigation.enabled")},_processCanceledEditCellPosition:function(e,t){if(this._canceledCellPosition){var n=this._canceledCellPosition.rowIndex===e&&this._canceledCellPosition.columnIndex===t;return this._canceledCellPosition=null,n}}}),he={defaultOptions:function(){return{useLegacyKeyboardNavigation:!1,keyboardNavigation:{enabled:!0,enterKeyAction:"startEdit",enterKeyDirection:"none",editOnKeyPress:!1}}},controllers:{keyboardNavigation:de},extenders:{views:{rowsView:{_rowClick:function(e){var t=this.getController("editing").getEditRowIndex(),n=this.getController("keyboardNavigation");t===e.rowIndex&&n.setCellFocusType();var i=p()&&this.option("focusedRowEnabled");i&&this._triggerPointerDownEventHandler(e),this.callBase.apply(this,arguments)},_triggerPointerDownEventHandler:function(e){var t=e.event.originalEvent;if(t){var n=this.getController("keyboardNavigation"),i=(0,m.default)(t.target),o=this.getCellIndex(i),a=this.getController("columns").getVisibleColumns()[o],r=this.getController("data").items()[e.rowIndex];if(n._isAllowEditing(r,a)){var s=(0,S.createEvent)(t,{currentTarget:t.target});n._pointerEventHandler(s)}}},renderFocusState:function(e){var t=this.getController("keyboardNavigation"),n=this.element();n&&!(0,k.focused)(n)&&n.attr("tabindex",null);var i=t.getVisibleRowIndex();(!(0,w.isDefined)(i)||i<0)&&(i=0);var o=this.getCellElements(i);t.isKeyboardEnabled()&&o.length&&this.updateFocusElementTabIndex(o,e)},updateFocusElementTabIndex:function(e){var t=this.getController("keyboardNavigation"),n=e.eq(0).parent();if(s(n))t._applyTabIndexToElement(n);else{var i=t.getColumnIndex();(!(0,w.isDefined)(i)||i<0)&&(i=0),this._updateFocusedCellTabIndex(e,i)}},_updateFocusedCellTabIndex:function(e,t){var n=this.getController("keyboardNavigation"),i=e?e.length:-1,o=function(e){var t=n._isMasterDetailCell(e),i=n._isCellValid(e);if(!t&&i&&d(e))return n._applyTabIndexToElement(e),n.setCellFocusType(),!0},a=e.filter("[aria-colindex='".concat(t+1,"']"));if(a.length)o(a);else{i<=t&&(t=i-1);for(var r=t;r<i&&!o((0,m.default)(e[r]));++r);}},renderDelayedTemplates:function(e){this.callBase.apply(this,arguments),e&&e.repaintChangesOnly||this.renderFocusState()},_renderCore:function(e){this.callBase(e),e&&e.repaintChangesOnly||this.renderFocusState()},_editCellPrepared:function(e){var t=this._getEditorInstance(e),n=this.getController("keyboardNavigation"),i=n&&n._isFastEditingStarted();t&&i&&this._handleEditingNavigationMode(t),this.callBase.apply(this,arguments)},_handleEditingNavigationMode:function(e){["downArrow","upArrow"].forEach(function(t){var n=e._supportedKeys()[t];e.registerKeyHandler(t,function(t){var i="true"===e._input().attr("aria-expanded");if(i)return n&&n.call(e,t)})}),e.registerKeyHandler("leftArrow",T.noop),e.registerKeyHandler("rightArrow",T.noop);var t=e.NAME===Q&&e.option("useMaskBehavior");t&&e.registerKeyHandler("enter",T.noop)},_getEditorInstance:function(e){var t=e.find(".dx-texteditor").eq(0);return x.default.getWidgetInstance(t)}}},controllers:{editing:{editCell:function(e,t){var n=this.getController("keyboardNavigation");if(n._processCanceledEditCellPosition(e,t))return!1;var i=this.callBase(e,t);return i&&n.setupFocusedView(),i},editRow:function(e){var t=this.getController("keyboardNavigation"),n=t.getVisibleColumnIndex(),i=this._columnsController.getVisibleColumns()[n];(i&&i.type||this.option("editing.mode")===ae)&&t._resetFocusedCell(),this.callBase(e)},addRow:function(e){return this.getController("keyboardNavigation").setupFocusedView(),this.callBase.apply(this,arguments)},getFocusedCellInRow:function(e){var t=this.getController("keyboardNavigation"),n=this.callBase(e);if(t.isKeyboardEnabled()&&t._focusedCellPosition.rowIndex===e){var i=t._getFocusedCell();f(i)&&!i.hasClass(Y)&&(n=i)}return n},_processCanceledEditingCell:function(){var e=this;this.closeEditCell().done(function(){var t=e.getController("keyboardNavigation");t._updateFocus()})},init:function(){this.callBase(),this._keyboardNavigationController=this.getController("keyboardNavigation")},closeEditCell:function(){var e=this.getController("keyboardNavigation");e._fastEditingStarted=!1;var t=this.callBase.apply(this,arguments);return e._updateFocus(),t},_delayedInputFocus:function(){this._keyboardNavigationController._isNeedScroll=!0,this.callBase.apply(this,arguments)},_isEditingStart:function(){var e=this.getController("keyboardNavigation"),t=this.callBase.apply(this,arguments);if(t&&!e._isNeedFocus){var n=e._getFocusedCell();e._focus(n,!0)}return t}},data:{_correctRowIndices:function(e){var t=this,n=t.getController("keyboardNavigation"),i=t.getController("editorFactory"),o=n._focusedCellPosition;if(t.callBase.apply(t,arguments),o&&o.rowIndex>=0){var a=e(o.rowIndex);a&&(o.rowIndex+=a,i.refocus())}}},adaptiveColumns:{_showHiddenCellsInView:function(e){var t=e.viewName,n=e.$cells,i=e.isCommandColumn;this.callBase.apply(this,arguments),t===ce&&!i&&n.each(function(e,t){var n=(0,m.default)(t);g(n)&&n.attr("tabindex",0)})},_hideVisibleCellInView:function(e){var t=e.viewName,n=e.$cell,i=e.isCommandColumn;this.callBase.apply(this,arguments),t===ce&&!i&&g(n)&&n.removeAttr("tabindex")}}}}};t.default=he,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(139),s=i(n(10)),l=i(n(37)),u=n(1),c=n(0),d=n(3),h=i(n(23)),f=n(24),p=n(30),g="content",_="content-fixed",m="dx-master-detail-cell",v="dx-first-cell",y="dx-last-cell",b="dx-state-hover",x="dx-col-fixed",w="dx-fixed-columns",C="dx-pointer-events-none",k="transparent",S="dx-group-row",D=function(e){var t=-1;return(0,d.each)(e,function(e,n){if(n.command===k)return t=e,!1}),t},I=function(e,t,n){var i=0;if(e&&t&&n)for(var o=0;o<e.length;o++)e[o].command===k?i+=e[o].colspan:(t[i]<n[o]&&(t[i]=n[o]),i++);return t},T={init:function(){this.callBase(),this._isFixedTableRendering=!1,this._isFixedColumns=!1},_createCol:function(e){return this.callBase(e).toggleClass(x,!(!this._isFixedTableRendering||!(e.fixed||e.command&&e.command!==k)))},_correctColumnIndicesForFixedColumns:function(e,t){var n=D(e),i=e[n].colspan,o=t&&t.columnIndices;o&&(t.columnIndices=o.map(function(e){if(e)return e.map(function(e){return e<n?e:e>=n+i?e-i+1:-1}).filter(function(e){return e>=0})}))},_renderTable:function(e){var t,n=this,i=n.getFixedColumns();n._isFixedColumns=!!i.length;var o=n.callBase(e);if(n._isFixedColumns){n._isFixedTableRendering=!0;var a=e&&e.change,r=a&&a.columnIndices;n._correctColumnIndicesForFixedColumns(i,a),t=n._createTable(i),n._renderRows(t,(0,c.extend)({},e,{columns:i})),n._updateContent(t,a),r&&(a.columnIndices=r),n._isFixedTableRendering=!1}else n._fixedTableElement&&n._fixedTableElement.parent().remove(),n._fixedTableElement=null;return o},_renderRow:function(e,t){var n,i=t.row.cells;if(this.callBase.apply(this,arguments),this._isFixedTableRendering&&i&&i.length){n=0;var o=t.row.cells||[];i=i.slice(),t.row.cells=i;for(var a=0;a<o.length;a++)o[a].column&&o[a].column.command===k?n=(o[a].column.colspan||1)-1:i[a+n]=o[a]}},_createCell:function(e){var t,n,i,o=this,a=e.column,r=a&&a.command,s=e.rowType,u=o.callBase.apply(o,arguments);(o._isFixedTableRendering||"filter"===s)&&(t=o.getFixedColumns(),i=D(t),n=t[i-1]),o._isFixedTableRendering?r===k&&(u.addClass(C).toggleClass(v,0===i||n&&"expand"===n.command).toggleClass(y,t.length&&i===t.length-1),"freeSpace"!==s&&l.default.setEmptyText(u)):"filter"===s&&u.toggleClass(v,e.columnIndex===i);var c=o.option("rowAlternationEnabled")&&e.isAltRow;
return h.default.mozilla&&e.column.fixed&&"group"!==e.rowType&&!c&&u.addClass(x),u},_wrapTableInScrollContainer:function(){var e=this.callBase.apply(this,arguments);return this._isFixedTableRendering&&e.addClass(this.addWidgetPrefix(_)),e},_renderCellContent:function(e,t){var n,i=this,o=t.column,a=i._isFixedTableRendering,r="group"===t.rowType&&(0,u.isDefined)(o.groupIndex);if(a&&r&&!o.command&&!o.groupCellTemplate&&e.css("pointerEvents","none"),!a&&i._isFixedColumns){if(n=o.fixed||o.command&&o.fixed!==!1,r&&(n=!1,t.row.summaryCells&&t.row.summaryCells.length)){var s=i._columnsController.getVisibleColumns(),l=i._getAlignByColumnCellCount?i._getAlignByColumnCellCount(o.colspan,{columns:s,row:t.row,isFixed:!0}):0;if(l>0){var c=D(i._columnsController.getFixedColumns());n=s.length-l<c}}if(n){if(i.option("legacyRendering")||o.command&&"buttons"!==o.type||"group"===t.rowType)return void e.html("&nbsp;").addClass(o.cssClass);e.addClass("dx-hidden-cell")}}o.command!==k&&i.callBase(e,t)},_getCellElementsCore:function(e){var t=this,n=t.callBase(e),i=n.parent().hasClass(S),o="columnHeadersView"===t.name?e:void 0;if(t._fixedTableElement&&n){var a=t.getFixedColumns(o),r=t._getRowElements(t._fixedTableElement).eq(e).children("td");(0,d.each)(r,function(e,s){if(i)n[e]&&"hidden"!==s.style.visibility&&(n[e]=s);else{var l=a[e];if(l)if(l.command===k)r.eq(e).hasClass(m)&&(n[e]=s||n[e]);else{var u=t._columnsController.getVisibleIndex(l.index,o);n[u]=s||n[u]}}})}return n},getColumnWidths:function(){var e,t=this,n=t.callBase(),i=t.getFixedColumns();return t._fixedTableElement&&n.length&&(e=t.callBase(t._fixedTableElement)),I(i,n,e)},_getTableElement:function(){var e=this._isFixedTableRendering?this._fixedTableElement:this.callBase();return e},_setTableElement:function(e){this._isFixedTableRendering?this._fixedTableElement=e.addClass(C):this.callBase(e)},getColumns:function(e,t){return t=t||this._getTableElement(),this._isFixedTableRendering||t&&t.closest("table").parent("."+this.addWidgetPrefix(_)).length?this.getFixedColumns(e):this.callBase(e,t)},getRowIndex:function(e){var t=this._fixedTableElement;return t&&t.find(e).length?this._getRowElements(t).index(e):this.callBase(e)},getTableElements:function(){var e=this.callBase.apply(this,arguments);return this._fixedTableElement&&(e=(0,o.default)([e.get(0),this._fixedTableElement.get(0)])),e},getFixedColumns:function(e){return this._columnsController.getFixedColumns(e)},getFixedColumnsOffset:function(){var e,t={left:0,right:0};if(this._fixedTableElement){e=this.getTransparentColumnElement();var n=e.position();t={left:n.left,right:this.element().outerWidth(!0)-(e.outerWidth(!0)+n.left)}}return t},getTransparentColumnElement:function(){return this._fixedTableElement&&this._fixedTableElement.find("."+C).first()},getFixedTableElement:function(){return this._fixedTableElement},isFixedColumns:function(){return this._isFixedColumns},_resizeCore:function(){this.callBase(),this.synchronizeRows()},setColumnWidths:function(e){var t,n=this._columnsController.getVisibleColumns(),i=e.widths,o=i&&i.length&&(0,u.isDefined)(n[0].visibleWidth),a=e.optionNames,r=a&&a.width,s=!1;if(this.callBase.apply(this,arguments),this._fixedTableElement){if(this.option("legacyRendering"))s=i&&i.length&&!this.isScrollbarVisible(!0);else{var l=i&&i.some(function(e){return"auto"===e});s=l&&(!o||!this.isScrollbarVisible(!0))}s&&(t=n),this.callBase((0,c.extend)({},e,{$tableElement:this._fixedTableElement,columns:t,fixed:!0}))}(o||r&&this.option("wordWrapEnabled"))&&this.synchronizeRows()},_createColGroup:function(e){if(!this.option("legacyRendering")&&this._isFixedTableRendering&&!this.option("columnAutoWidth")){var t=this._columnsController.getVisibleColumns(),n=t.filter(function(e){return!e.width}).length;n&&(e=t)}return this.callBase(e)},_getClientHeight:function(e){var t=e.getBoundingClientRect&&(0,f.getBoundingRect)(e);return t&&t.height?t.height:e.clientHeight},synchronizeRows:function(){var e,t,n,i,o=this,a=[],r=[];if(o._isFixedColumns&&o._tableElement&&o._fixedTableElement){var s=o._getClientHeight(o._tableElement.get(0)),l=o._getClientHeight(o._fixedTableElement.get(0));if(t=o._getRowElements(o._tableElement),n=o._getRowElements(o._fixedTableElement),i=o._findContentElement(),s!==l){for(i&&i.css("height",s),t.css("height",""),n.css("height",""),e=0;e<t.length;e++)a.push(o._getClientHeight(t.get(e))),r.push(o._getClientHeight(n.get(e)));for(e=0;e<t.length;e++){var u=a[e],c=r[e];u>c?n.eq(e).css("height",u):u<c&&t.eq(e).css("height",c)}i&&i.css("height","")}}},setScrollerSpacing:function(e){var t=this.option("rtlEnabled");this.callBase(e),this.element().children("."+this.addWidgetPrefix(_)).css({paddingLeft:t?e:"",paddingRight:t?"":e})}},O=(0,c.extend)({},T,{_getRowVisibleColumns:function(e){return this._isFixedTableRendering?this.getFixedColumns(e):this.callBase(e)},getContextMenuItems:function(e){var t=this,n=e.column,i=t.option("columnFixing"),o=t.callBase(e);if(e.row&&"header"===e.row.rowType&&n&&n.allowFixing){var a=function(e){switch(e.itemData.value){case"none":t._columnsController.columnOption(n.index,"fixed",!1);break;case"left":t._columnsController.columnOption(n.index,{fixed:!0,fixedPosition:"left"});break;case"right":t._columnsController.columnOption(n.index,{fixed:!0,fixedPosition:"right"})}};o=o||[],o.push({text:i.texts.fix,beginGroup:!0,items:[{text:i.texts.leftPosition,value:"left",disabled:n.fixed&&(!n.fixedPosition||"left"===n.fixedPosition),onItemClick:a},{text:i.texts.rightPosition,value:"right",disabled:n.fixed&&"right"===n.fixedPosition,onItemClick:a}]},{text:i.texts.unfix,value:"none",disabled:!n.fixed,onItemClick:a})}return o},getFixedColumnElements:function(e){var t=this;if((0,u.isDefined)(e))return this._fixedTableElement&&this._getRowElements(this._fixedTableElement).eq(e).children();var n=t.getColumnElements(),i=t.getTransparentColumnElement();if(n&&i&&i.length){var o=D(t.getFixedColumns());n.splice(o,i.get(0).colSpan,i.get(0))}return n},getColumnWidths:function(){var e,t=this,n=t.callBase(),i=t.getFixedColumnElements(),o=t.getFixedColumns();return t._fixedTableElement&&(e=i&&i.length?t._getWidths(i):t.callBase(t._fixedTableElement)),I(o,n,e)}}),E=(0,c.extend)({},T,{_detachHoverEvents:function(){this._fixedTableElement&&a.default.off(this._fixedTableElement,"mouseover mouseout",".dx-data-row"),this._tableElement&&a.default.off(this._tableElement,"mouseover mouseout",".dx-data-row")},_attachHoverEvents:function(){var e=this,t=function(t){a.default.on(t,"mouseover mouseout",".dx-data-row",e.createAction(function(t){var n=t.event,i=e.getRowIndex((0,o.default)(n.target).closest(".dx-row")),a="mouseover"===n.type;i>=0&&(e._tableElement&&e._getRowElements(e._tableElement).eq(i).toggleClass(b,a),e._fixedTableElement&&e._getRowElements(e._fixedTableElement).eq(i).toggleClass(b,a))}))};e._fixedTableElement&&e._tableElement&&(t(e._fixedTableElement),t(e._tableElement))},_findContentElement:function(){var e,t,n=this,i=n.addWidgetPrefix(g),s=n.element(),l=h.default.mozilla?60:0;if(s&&n._isFixedTableRendering){e=s.children("."+i);var u=n.getScrollable();return!e.length&&u&&(e=(0,o.default)("<div>").addClass(i),a.default.on(e,"scroll",function(e){clearTimeout(n._fixedScrollTimeout),n._fixedScrollTimeout=setTimeout(function(){t=(0,o.default)(e.target).scrollTop(),u.scrollTo({y:t})},l)}),a.default.on(e,r.name,function(e){var i=(0,o.default)(e.target).closest(".dx-scrollable");if(u&&u.$element().is(i)&&(t=u.scrollTop(),u.scrollTo({y:t-e.delta}),u.scrollTop()>0&&u.scrollTop()+u.clientHeight()<u.scrollHeight()+n.getScrollbarWidth()))return!1}),e.appendTo(s)),e}return n.callBase()},_updateScrollable:function(){this.callBase();var e=this.getScrollable(),t=e&&e.scrollOffset().top;this._updateFixedTablePosition(t)},_renderContent:function(e,t){return this._isFixedTableRendering?e.empty().addClass(this.addWidgetPrefix(g)+" "+this.addWidgetPrefix(_)).append(t):this.callBase(e,t)},_getGroupCellOptions:function(e){return this._isFixedTableRendering?this.callBase((0,c.extend)({},e,{columns:this._columnsController.getVisibleColumns()})):this.callBase(e)},_renderGroupedCells:function(e,t){return this.callBase(e,(0,c.extend)({},t,{columns:this._columnsController.getVisibleColumns()}))},_renderGroupSummaryCells:function(e,t){this._isFixedTableRendering?this.callBase(e,(0,c.extend)({},t,{columns:this._columnsController.getVisibleColumns()})):this.callBase(e,t)},_hasAlignByColumnSummaryItems:function(e,t){var n=this.callBase.apply(this,arguments),i=t.columns[e];return t.isFixed?i.fixed&&(n||"right"===i.fixedPosition):n&&!i.fixed},_renderGroupSummaryCellsCore:function(e,t,n,i){var o;if(this._isFixedTableRendering){t.isFixed=!0,o=this._getAlignByColumnCellCount(n,t),t.isFixed=!1;var a=t.columns.length-o;t=(0,c.extend)({},t,{columns:this.getFixedColumns()});var r=D(t.columns);a<r?(o-=t.columns[r].colspan-1||0,n-=t.columns[r].colspan-1||0):i>0&&e.css("visibility","hidden"),i=o}this.callBase(e,t,n,i)},_getSummaryCellIndex:function(e,t){if(this._isFixedTableRendering){var n=D(t);return e>n&&(e+=t[n].colspan-1),e}return this.callBase.apply(this,arguments)},_renderCore:function(e){this._detachHoverEvents(),this.callBase(e);var t=this._isFixedColumns;this.element().toggleClass(w,t),this.option("hoverStateEnabled")&&t&&this._attachHoverEvents()},setRowsOpacity:function(e,t){this.callBase(e,t);var n=this._getRowElements(this._fixedTableElement);this._setRowsOpacityCore(n,this.getFixedColumns(),e,t)},optionChanged:function(e){var t=this;t.callBase(e),"hoverStateEnabled"===e.name&&t._isFixedColumns&&(e.value?this._attachHoverEvents():this._detachHoverEvents())},getCellIndex:function(e){var t=this._fixedTableElement,n=0;if(t&&e.is("td")&&e.closest(t).length){var i=this.getFixedColumns();return(0,d.each)(i,function(t,i){return t!==e[0].cellIndex&&(i.colspan?void(n+=i.colspan):void n++)}),n}return this.callBase.apply(this,arguments)},_updateFixedTablePosition:function(e,t){if(this._fixedTableElement&&this._tableElement){var n,i=this.getController("editorFactory");this._fixedTableElement.parent().scrollTop(e),t&&i&&(n=i.focus(),n&&i.focus(n))}},setScrollerSpacing:function(e,t){var n=this,i={marginBottom:0},o=n.element().children("."+this.addWidgetPrefix(_));if(o.length&&n._fixedTableElement){o.css(i),n._fixedTableElement.css(i),i[n.option("rtlEnabled")?"marginLeft":"marginRight"]=e,i.marginBottom=t;var a=n._scrollable&&n._scrollable.option("useNative");(a?o:n._fixedTableElement).css(i)}},_getElasticScrollTop:function(e){var t=0,n=this.getScrollbarWidth(!0);if(e.scrollOffset.top<0)t=-e.scrollOffset.top;else if(e.reachedBottom){var i=this._findContentElement(),o=e.component._container(),a=Math.max(i.height()+n-o.height(),0);t=a-e.scrollOffset.top}return t},_applyElasticScrolling:function(e){if(this._fixedTableElement){var t=this._getElasticScrollTop(e);0!==Math.ceil(t)?(0,p.move)(this._fixedTableElement,{top:t}):this._fixedTableElement.css("transform","")}},_handleScroll:function(e){this._updateFixedTablePosition(e.scrollOffset.top,!0),this._applyElasticScrolling(e),this.callBase(e)},_updateContentPosition:function(e){this.callBase.apply(this,arguments),e||this._updateFixedTablePosition(this._scrollTop)},_afterRowPrepared:function(e){this._isFixedTableRendering||this.callBase(e)},_scrollToElement:function(e){this.callBase(e,this.getFixedColumnsOffset())},dispose:function(){this.callBase.apply(this,arguments),clearTimeout(this._fixedScrollTimeout)}}),A=T,P={defaultOptions:function(){return{columnFixing:{enabled:!1,texts:{fix:s.default.format("dxDataGrid-columnFixingFix"),unfix:s.default.format("dxDataGrid-columnFixingUnfix"),leftPosition:s.default.format("dxDataGrid-columnFixingLeftPosition"),rightPosition:s.default.format("dxDataGrid-columnFixingRightPosition")}}}},extenders:{views:{columnHeadersView:O,rowsView:E,footerView:A},controllers:function(){var e=function(e,t,n){var i=D(t),o=e.length-t.length;return(0,d.each)(n,function(e,t){t.index>i&&(t.columnIndex+=o,t.index+=o)}),n};return{draggingHeader:{_generatePointsByColumns:function(t){var n=t.columns,i=t.targetDraggingPanel;if(i&&"headers"===i.getName()&&i.isFixedColumns()&&t.sourceColumn.fixed){t.rowIndex||(t.columnElements=i.getFixedColumnElements(0)),t.columns=i.getFixedColumns(t.rowIndex);var o=this.callBase(t);return e(n,t.columns,o),o}return this.callBase(t)},_pointCreated:function(e,t,n,i){var o=this.callBase.apply(this,arguments),a=this._columnHeadersView.getTransparentColumnElement();if(!o&&"headers"===n&&a&&a.length){var r=(0,f.getBoundingRect)(a.get(0));return i&&i.fixed?"right"===i.fixedPosition?e.x<r.right:e.x>r.left:e.x<r.left||e.x>r.right}return o}},columnsResizer:{_generatePointsByColumns:function(){var t=this,n=t._columnsController,i=n&&t._columnsController.getVisibleColumns(),o=n&&t._columnsController.getFixedColumns(),a=t._columnHeadersView.getFixedColumnElements(),r=[];t.callBase(),a&&a.length>0&&(r=l.default.getPointsByColumns(a,function(e){return t._pointCreated(e,a.length,o)}),t._pointsByFixedColumns=e(i,o,r))},_pointCreated:function(e,t,n){var i="widget"===this.option("columnResizingMode");if(e.index>0&&e.index<t){var o=n[e.columnIndex-1]||{},a=n[e.columnIndex]||{};if(o.fixed||a.fixed)return e.columnIndex-=1,!((o.allowResizing||o.command===k)&&(i||a.allowResizing||a.command===k))}return this.callBase.apply(this,arguments)},_getTargetPoint:function(e,t,n){var i=this._columnHeadersView.getTransparentColumnElement();if(i&&i.length){var o=(0,f.getBoundingRect)(i.get(0));if(t<=o.left||t>=o.right)return this.callBase(this._pointsByFixedColumns,t,n)}return this.callBase(e,t,n)}}}}()}};t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return e._columnsController.getColumnId(t)}function a(e){return e.find("td:not(.dx-datagrid-hidden-column):not([class*='dx-command-'])")}function r(e,t){var n,i=(0,s.default)(e),o=t.component.getController("adaptiveColumns");"data"===t.rowType?(n=(0,s.default)("<span>").addClass(o.addWidgetPrefix(R)),l.default.on(n,(0,u.addNamespace)(c.name,I),o.createAction(function(){o.toggleExpandAdaptiveDetailRow(t.key)})),n.appendTo(i)):_.default.setEmptyText(i)}t.default=void 0;var s=i(n(2)),l=i(n(5)),u=n(8),c=n(21),d=n(1),h=i(n(23)),f=i(n(35)),p=i(n(44)),g=i(n(146)),_=i(n(37)),m=n(36),v=n(7),y=n(4),b=n(3),x=n(0),w=n(6),C="columnHeadersView",k="rowsView",S="footerView",D=[C,k,S],I="dxDataGridAdaptivity",T="adaptiveHidden",O="detailAdaptive",E="dx-field-item-content",A="dx-item-modified",P="hidden-column",R="adaptive-more",M="dx-command-adaptive",B="dx-command-adaptive-hidden",V="dx-adaptive-detail-row",F="dx-adaptive-item-text",L="dx-master-detail-cell",H="dx-last-data-cell",z="adaptive",N="batch",j="row",W="form",G="popup",q="revert-tooltip",$="dx-group-cell",K="dx-group-row",U=p.default.ViewController.inherit({_isRowEditMode:function(){var e=this._getEditMode();return e===j},_isItemModified:function(e,t){var n=this._columnsController.getVisibleIndex(e.column.index),i=this._dataController.getRowIndexByKey(t.key),o=this._dataController.items()[i+1];return o&&o.modifiedValues&&(0,d.isDefined)(o.modifiedValues[n])},_renderFormViewTemplate:function(e,t,n){var i=this,o=e.column,a=i.createAction(function(){l.default.trigger(n,c.name)}),r=o.calculateCellValue(t.data),s=_.default.getDisplayValue(o,r,t.data,t.rowType),u=_.default.formatValue(s,o),h=this._editingController.isCellOrBatchEditMode(),f=i._rowsView;if(o.allowEditing&&i.getController("keyboardNavigation").isKeyboardEnabled()&&(n.attr("tabIndex",i.option("tabIndex")),h&&(l.default.off(n,"focus",a),l.default.on(n,"focus",a))),o.cellTemplate){var p=(0,x.extend)({},t,{value:r,displayValue:s,text:u,column:o}),g=!!n.closest((0,v.getWindow)().document).length;f.renderTemplate(n,o.cellTemplate,p,g).done(function(){f._cellPrepared(n,t)})}else{var m=n.get(0);o.encodeHtml?m.textContent=u:m.innerHTML=u,n.addClass(F),(0,d.isDefined)(u)&&""!==u||n.html("&nbsp;"),i._isRowEditMode()||i._isItemModified(e,t)&&n.addClass(A),f._cellPrepared(n,t)}},_getTemplate:function(e,t){var n=this,i=e.column,o=this.getController("editing");return function(a,r){var l=(0,s.default)(r),u=n._columnsController.getVisibleIndex(i.visibleIndex),c=(0,x.extend)({},t),d=function(){var r=n._isItemEdited(e);c.value=t.row.values[u],r||i.showEditorAlways?o.renderFormEditTemplate(c,e,a.component,l,!r):(c.column=i,c.columnIndex=u,n._renderFormViewTemplate(e,c,l))};d(),c.watch&&c.watch(function(){return{isItemEdited:n._isItemEdited(e),value:t.row.values[u]}},function(){l.contents().remove(),l.removeClass(F),d()})}},_isVisibleColumnsValid:function(e){var t=function(){for(var t=0,n=0;n<e.length;n++){var i=e[n];i.command&&t++}return t};return!(e<2)&&!(e.length-t()<=1)},_calculatePercentWidths:function(e,t){var n=this,i=0;return t.forEach(function(t,o){e[o]!==T&&(i+=n._getItemPercentWidth(t))}),i},_isPercentWidth:function(e){return(0,d.isString)(e)&&"%"===e.slice(-1)},_isColumnHidden:function(e){return this._hiddenColumns.filter(function(t){return t.index===e.index}).length>0},_getAverageColumnsWidth:function(e,t,n){var i=this,o=0,a=0;return t.forEach(function(r){if(!i._isColumnHidden(r)){var s=r.width;(0,d.isDefined)(s)&&!isNaN(parseFloat(s))?o+=i._isPercentWidth(s)?i._calculatePercentWidth({visibleIndex:r.visibleIndex,columnsCount:t.length,columnsCanFit:n,bestFitWidth:r.bestFitWidth,columnWidth:s,containerWidth:e}):parseFloat(s):a++}}),(e-o)/a},_calculateColumnWidth:function(e,t,n,i){var a,r=o(this,e),s=this._columnsController.columnOption(r,"width"),l=this._columnsController.columnOption(r,"bestFitWidth"),u=n.length;if(s&&"auto"!==s){if(!this._isPercentWidth(s))return s;a=this._calculatePercentWidth({visibleIndex:e.visibleIndex,columnsCount:u,columnsCanFit:i,bestFitWidth:l,columnWidth:s,containerWidth:t})}else{var c=this.option("columnAutoWidth");a=c||e.command?l:this._getAverageColumnsWidth(t,n,i)}return a},_calculatePercentWidth:function(e){var t=e.visibleIndex<e.columnsCount-1&&e.columnsCanFit,n=e.containerWidth*parseFloat(e.columnWidth)/100,i=e.columnsCanFit&&n<e.bestFitWidth?e.bestFitWidth:n;return t?e.containerWidth*parseFloat(e.columnWidth)/100:i},_getNotTruncatedColumnWidth:function(e,t,n,i){var a=o(this,e),r=this._columnsController.columnOption(a,"width"),s=this._columnsController.columnOption(a,"bestFitWidth");if(r&&"auto"!==r&&!this._isPercentWidth(r))return parseFloat(r);var l=this._calculateColumnWidth(e,t,n,i);return l<s?null:l},_getItemPercentWidth:function(e){var t=0;return e.width&&this._isPercentWidth(e.width)&&(t=parseFloat(e.width)),t},_getCommandColumnsWidth:function(){var e=this,t=e._columnsController.getVisibleColumns(),n=0;return(0,b.each)(t,function(t,i){(i.index<0||i.command)&&(n+=e._columnsController.columnOption(o(e,i),"bestFitWidth")||0)}),n},_isItemEdited:function(e){if(this.isFormEditMode())return!1;if(!this._isRowEditMode()){var t=this._dataController.getRowIndexByKey(this._dataController.adaptiveExpandedKey())+1,n=this._columnsController.getVisibleIndex(e.column.index);return this._editingController.isEditCell(t,n)}var i=this.option("editing.editRowKey");return!!(0,y.equalByValue)(i,this._dataController.adaptiveExpandedKey())||void 0},_getFormItemsByHiddenColumns:function(e){var t=[];return(0,b.each)(e,function(e,n){t.push({column:n,name:n.name,dataField:n.dataField,visibleIndex:n.visibleIndex})}),t},_getAdaptiveColumnVisibleIndex:function(e){for(var t=0;t<e.length;t++){var n=e[t];if(n.command===z)return t}},_hideAdaptiveColumn:function(e,t){var n=this._getAdaptiveColumnVisibleIndex(t);(0,d.isDefined)(n)&&(e[n]=T,this._hideVisibleColumn({isCommandColumn:!0,visibleIndex:n}))},_showHiddenCellsInView:function(e){var t=e.$cells,n=e.isCommandColumn,i=n?B:this.addWidgetPrefix(P);t.removeClass(i)},_showHiddenColumns:function(){for(var e=0;e<D.length;e++){var t=this.getView(D[e]);if(t&&t.isVisible()&&t.element()){var n=t.name,i=t.element().find("."+B);this._showHiddenCellsInView({viewName:n,$cells:i,isCommandColumn:!0});var o=t.element().find("."+this.addWidgetPrefix(P));this._showHiddenCellsInView({viewName:n,$cells:o})}}},_isCellValid:function(e){return e&&e.length&&!e.hasClass(L)&&!e.hasClass($)},_hideVisibleColumn:function(e){var t=e.isCommandColumn,n=e.visibleIndex,i=this;D.forEach(function(e){var o=i.getView(e);o&&i._hideVisibleColumnInView({view:o,isCommandColumn:t,visibleIndex:n})})},_hideVisibleColumnInView:function(e){var t,n=e.view,i=e.isCommandColumn,o=e.visibleIndex,a=n.name,r=this._columnsController.getVisibleColumns()[o],s=this._editingController&&this._editingController.getEditFormRowIndex();if(n&&n.isVisible()&&r)for(var l=n.getRowsCount(),u=n._getRowElements(),c=0;c<l;c++){var d=c===s&&a===k&&"popup"!==this.option("editing.mode");if(!d){var h=a===C?this._columnsController.getVisibleIndex(r.index,c):o;if(h>=0){var f=u.eq(c);t=this._findCellElementInRow(f,h),this._isCellValid(t)&&this._hideVisibleCellInView({viewName:a,isCommandColumn:i,$cell:t})}}}},_findCellElementInRow:function(e,t){var n=e.children(),i=t,o=!1;if(e.hasClass(K)){var a=e.find(".".concat($)),r=a.attr("colspan");if(a.length&&(0,d.isDefined)(r)){var s=parseInt(r),l=a.index()+s-1;t>l?i=t-s+1:o=!0}}var u=o?void 0:n.eq(i);return u},_hideVisibleCellInView:function(e){var t=e.$cell,n=e.isCommandColumn,i=n?B:this.addWidgetPrefix(P);t.addClass(i)},_getEditMode:function(){return this._editingController.getEditMode()},isFormEditMode:function(){var e=this._getEditMode();return e===W||e===G},hideRedundantColumns:function(e,t,n){var i=this;if(this._hiddenColumns=[],i._isVisibleColumnsValid(t)&&n.length){var a,r,s,l=0,u=i.component.$element(),c=u.width()-i._getCommandColumnsWidth(),d=function(){var e=this;return t.filter(function(t){return!t.command&&0===e._hiddenColumns.filter(function(e){return e.index===t.index}).length})}.bind(this),h=d(),f=h.length;do{s=!1,l=0;var p=i._calculatePercentWidths(e,t),g=p<100&&0!==p;for(a=0;a<t.length;a++){var _=t[a],m=i._getNotTruncatedColumnWidth(_,c,h,g),v=o(i,_),y=i._columnsController.columnOption(v,"width"),b=i._columnsController.columnOption(v,"minWidth"),x=i._columnsController.columnOption(v,"bestFitWidth");if(e[a]!==T){if(!m&&!_.command&&!_.fixed){s=!0;break}y&&"auto"!==y||(m=Math.max(x||0,b||0)),(_.command!==z||r)&&(l+=m)}else r=!0}if(s=s||l>u.width()){var w=n.pop(),C=i._columnsController.getVisibleIndex(w.index);c+=i._calculateColumnWidth(w,c,h,g),i._hideVisibleColumn({visibleIndex:C}),e[C]=T,this._hiddenColumns.push(w),h=d()}}while(s&&h.length>1&&n.length);f===h.length&&i._hideAdaptiveColumn(e,t)}else i._hideAdaptiveColumn(e,t)},getItemContentByColumnIndex:function(e){for(var t,n=0;n<this._$itemContents.length;n++){t=this._$itemContents.eq(n);var i=t.data("dx-form-item");if(i&&i.column&&this._columnsController.getVisibleIndex(i.column.index)===e)return t}},toggleExpandAdaptiveDetailRow:function(e,t){this.isFormEditMode()&&this._editingController.isEditing()||this.getController("data").toggleExpandAdaptiveDetailRow(e,t)},createFormByHiddenColumns:function(e,t){var n=this,i=(0,s.default)(e),o={items:n._getFormItemsByHiddenColumns(n._hiddenColumns),formID:"dx-"+new f.default},a=(0,m.isMaterial)()?{colCount:2}:{};this.executeAction("onAdaptiveDetailRowPreparing",{formOptions:o}),n._$itemContents=null,n._form=n._createComponent((0,s.default)("<div>").appendTo(i),g.default,(0,x.extend)(a,o,{customizeItem:function(e){var i=e.column||n._columnsController.columnOption(e.name||e.dataField);i&&(e.label=e.label||{},e.label.text=e.label.text||i.caption,e.column=i,e.template=n._getTemplate(e,t,n.updateForm.bind(n))),o.customizeItem&&o.customizeItem.call(this,e)},onContentReady:function(e){o.onContentReady&&o.onContentReady.call(this,e),n._$itemContents=i.find("."+E)}}))},hasAdaptiveDetailRowExpanded:function(){return(0,d.isDefined)(this._dataController.adaptiveExpandedKey())},updateForm:function(e){this.hasAdaptiveDetailRowExpanded()&&this._form&&(0,d.isDefined)(this._form._contentReadyAction)&&(e&&e.length?this._form.option("items",this._getFormItemsByHiddenColumns(e)):this._form.repaint())},updateHidingQueue:function(e){var t,n,i=this,o=e.filter(function(e){return e.visible&&!e.type&&!e.fixed&&!((0,d.isDefined)(e.groupIndex)&&e.groupIndex>=0)});if(i._hidingColumnsQueue=[],i.option("allowColumnResizing")&&"widget"===i.option("columnResizingMode"))return i._hidingColumnsQueue;for(n=0;n<o.length;n++)(0,d.isDefined)(o[n].hidingPriority)&&o[n].hidingPriority>=0&&(t=!0,i._hidingColumnsQueue[o[n].hidingPriority]=o[n]);if(t)i._hidingColumnsQueue.reverse();else if(i.option("columnHidingEnabled"))for(n=0;n<o.length;n++){var a=i._columnsController.getVisibleIndex(o[n].index);i._hidingColumnsQueue[a]=o[n]}return i._hidingColumnsQueue=i._hidingColumnsQueue.filter(Object),i._hidingColumnsQueue},getHiddenColumns:function(){return this._hiddenColumns},hasHiddenColumns:function(){return this._hiddenColumns.length>0},getHidingColumnsQueue:function(){return this._hidingColumnsQueue},init:function(){var e=this;e._columnsController=e.getController("columns"),e._dataController=e.getController("data"),e._rowsView=e.getView("rowsView"),e._columnsController.addCommandColumn({type:z,command:z,visible:!0,adaptiveHidden:!0,cssClass:M,alignment:"center",width:"auto",cellTemplate:r,fixedPosition:"right"}),e._columnsController.columnsChanged.add(function(){var t=!!e.updateHidingQueue(e._columnsController.getColumns()).length;e._columnsController.columnOption("command:adaptive","adaptiveHidden",!t,!0)}),e._editingController=e.getController("editing"),e._hidingColumnsQueue=[],e._hiddenColumns=[],e.createAction("onAdaptiveDetailRowPreparing"),e.callBase()},optionChanged:function(e){"columnHidingEnabled"===e.name&&this._columnsController.columnOption("command:adaptive","adaptiveHidden",!e.value),this.callBase(e)},publicMethods:function(){return["isAdaptiveDetailRowExpanded","expandAdaptiveDetailRow","collapseAdaptiveDetailRow"]},isAdaptiveDetailRowExpanded:function(e){return this._dataController.adaptiveExpandedKey()&&(0,y.equalByValue)(this._dataController.adaptiveExpandedKey(),e)},expandAdaptiveDetailRow:function(e){this.hasAdaptiveDetailRowExpanded()||this.toggleExpandAdaptiveDetailRow(e)},collapseAdaptiveDetailRow:function(){this.hasAdaptiveDetailRowExpanded()&&this.toggleExpandAdaptiveDetailRow()}}),Y={defaultOptions:function(){return{columnHidingEnabled:!1,onAdaptiveDetailRowPreparing:null}},controllers:{adaptiveColumns:U},extenders:{views:{rowsView:{_getCellTemplate:function(e){var t=this,n=e.column;return e.rowType===O&&"detail"===n.command?function(e,n){t._adaptiveColumnsController.createFormByHiddenColumns((0,s.default)(e),n)}:t.callBase(e)},_createRow:function(e){var t=this.callBase(e);return e&&e.rowType===O&&e.key===this._dataController.adaptiveExpandedKey()&&t.addClass(V),t},_renderCells:function(e,t){this.callBase(e,t);var n=this._adaptiveColumnsController.getHidingColumnsQueue().length,i=this._adaptiveColumnsController.getHiddenColumns().length;n&&!i&&a(e).last().addClass(H)},_getColumnIndexByElementCore:function(e){var t=e.closest("."+E);if(t.length&&t.closest(this.component.$element()).length){var n=t.length?t.first().data("dx-form-item"):null;return n&&n.column&&this._columnsController.getVisibleIndex(n.column.index)}return this.callBase(e)},_cellPrepared:function(e,t){this.callBase.apply(this,arguments),t.row.rowType!==O&&t.column.visibleWidth===T&&e.addClass(this.addWidgetPrefix(P))},_getCellElement:function(e,t){var n=this._dataController.items()[e];return n&&n.rowType===O?this._adaptiveColumnsController.getItemContentByColumnIndex(t):this.callBase(e,t)},getContextMenuItems:function(e){if(e.row&&"detailAdaptive"===e.row.rowType){var t=this.component.getView("columnHeadersView"),n=(0,s.default)(e.targetElement).closest(".dx-field-item-label").next().data("dx-form-item");return e.column=n?n.column:e.column,t.getContextMenuItems&&t.getContextMenuItems(e)}return this.callBase&&this.callBase(e)},isClickableElement:function(e){var t=!!this.callBase&&this.callBase(e);return t||!!e.closest("."+M).length},init:function(){this.callBase(),this._adaptiveColumnsController=this.getController("adaptiveColumns")}}},controllers:{"export":{_updateColumnWidth:function(e,t){this.callBase(e,e.visibleWidth===T?e.bestFitWidth:t)}},columnsResizer:{_pointCreated:function(e,t,n){var i=this.callBase(e,t,n),o=n[e.columnIndex]||{},a=this._getNextColumnIndex(e.columnIndex),r=n[a]||{},s=a!==e.columnIndex+1&&r.command,l=o.visibleWidth===T||s;return i||l},_getNextColumnIndex:function(e){for(var t=this._columnsController.getVisibleColumns(),n=this.callBase(e);t[n]&&t[n].visibleWidth===T;)n++;return n}},draggingHeader:{_pointCreated:function(e,t,n,i){var o=this.callBase(e,t,n,i),a=t[e.columnIndex-1]||{},r=a.visibleWidth===T;return o||r}},editing:{_isRowEditMode:function(){return this.getEditMode()===j},_getFormEditItemTemplate:function(e,t){return this.getEditMode()!==j&&"detailAdaptive"===e.rowType?(e.columnIndex=this._columnsController.getVisibleIndex(t.index),this.getColumnTemplate(e)):this.callBase(e,t)},_closeEditItem:function(e){var t=e.closest("."+E),n=this._dataController.getRowIndexByKey(this._dataController.adaptiveExpandedKey())+1,i=t.length?t.first().data("dx-form-item"):null,o=i&&i.column&&this._columnsController.getVisibleIndex(i.column.index);this.isEditCell(n,o)||this.callBase(e)},_beforeUpdateItems:function(e,t){if(!this._adaptiveController.isFormEditMode()&&this._adaptiveController.hasHiddenColumns()){var n=this._dataController.items(),i=n[t],o=_.default.getIndexByKey(this._dataController.adaptiveExpandedKey(),n);this._isForceRowAdaptiveExpand=!this._adaptiveController.hasAdaptiveDetailRowExpanded(),o>=0&&e.push(o+1),e.push(t+1),this._dataController.adaptiveExpandedKey(i.key)}},_afterInsertRow:function(e){this.callBase(e),this._adaptiveController.hasHiddenColumns()&&(this._adaptiveController.toggleExpandAdaptiveDetailRow(e.key,this.isRowEditMode()),this._isForceRowAdaptiveExpand=!0)},_collapseAdaptiveDetailRow:function(){this._isRowEditMode()&&this._isForceRowAdaptiveExpand&&(this._adaptiveController.collapseAdaptiveDetailRow(),this._isForceRowAdaptiveExpand=!1)},_cancelEditAdaptiveDetailRow:function(){this._adaptiveController.hasHiddenColumns()&&this._collapseAdaptiveDetailRow()},_afterSaveEditData:function(){var e=this;this.callBase.apply(this,arguments);var t=new w.Deferred;return this._isRowEditMode()&&this._adaptiveController.hasHiddenColumns()?(0,w.when)(this.getController("validating").validate(!0)).done(function(n){n&&e._cancelEditAdaptiveDetailRow(),t.resolve()}):t.resolve(),t.promise()},_beforeCancelEditData:function(){this.callBase(),this._cancelEditAdaptiveDetailRow()},_getRowIndicesForCascadeUpdating:function(e){var t=this.callBase.apply(this,arguments);return this._adaptiveController.isAdaptiveDetailRowExpanded(e.key)&&t.push(e.rowType===O?e.rowIndex-1:e.rowIndex+1),t},_beforeCloseEditCellInBatchMode:function(e){var t=this._dataController._adaptiveExpandedKey;if(t){var n=_.default.getIndexByKey(t,this._dataController.items());n>-1&&e.unshift(n)}},editRow:function(e){this._adaptiveController.isFormEditMode()&&this._adaptiveController.collapseAdaptiveDetailRow(),this.callBase(e)},deleteRow:function(e){var t=this._dataController.getKeyByRowIndex(e);this.getEditMode()===N&&this._adaptiveController.isAdaptiveDetailRowExpanded(t)&&this._adaptiveController.collapseAdaptiveDetailRow(),this.callBase(e)},init:function(){this.callBase(),this._adaptiveController=this.getController("adaptiveColumns")}},resizing:{_needBestFit:function(){return this.callBase()||!!this._adaptiveColumnsController.getHidingColumnsQueue().length},_updateScrollableForIE:function(){var e=this;h.default.msie&&parseInt(h.default.version)<=11&&(this._updateScrollableTimeoutID=setTimeout(function(){e.getView("rowsView")._updateScrollable()}))},_correctColumnWidths:function(e,t){var n=this._adaptiveColumnsController,i=this.option("columnAutoWidth"),o=n.getHiddenColumns(),a=n.updateHidingQueue(this._columnsController.getColumns());n.hideRedundantColumns(e,t,a);var r=n.getHiddenColumns();return n.hasAdaptiveDetailRowExpanded()&&o.length!==r.length&&n.updateForm(r),
!r.length&&n.collapseAdaptiveDetailRow(),i&&a.length&&!r.length&&this._updateScrollableForIE(),this.callBase.apply(this,arguments)},_toggleBestFitMode:function(e){e&&this._adaptiveColumnsController._showHiddenColumns(),this.callBase(e)},_needStretch:function(){var e=this._adaptiveColumnsController;return this.callBase.apply(this,arguments)||e.getHidingColumnsQueue().length||e.hasHiddenColumns()},init:function(){this._adaptiveColumnsController=this.getController("adaptiveColumns"),this.callBase()},dispose:function(){this.callBase.apply(this,arguments),clearTimeout(this._updateScrollableTimeoutID)}},data:{_processItems:function(e,t){var n=this,i=t.changeType;if(e=n.callBase.apply(n,arguments),"loadingAll"===i||!(0,d.isDefined)(n._adaptiveExpandedKey))return e;var o=_.default.getIndexByKey(n._adaptiveExpandedKey,e);if(o>=0){var a=e[o];e.splice(o+1,0,{visible:!0,rowType:O,key:a.key,data:a.data,node:a.node,modifiedValues:a.modifiedValues,isNewRow:a.isNewRow,values:a.values})}else"refresh"===i&&(n._adaptiveExpandedKey=void 0);return e},_getRowIndicesForExpand:function(e){var t=this.callBase.apply(this,arguments);if(this.getController("adaptiveColumns").isAdaptiveDetailRowExpanded(e)){var n=t[t.length-1];t.push(n+1)}return t},adaptiveExpandedKey:function(e){return(0,d.isDefined)(e)?void(this._adaptiveExpandedKey=e):this._adaptiveExpandedKey},toggleExpandAdaptiveDetailRow:function(e,t){var n=this,i=_.default.getIndexByKey(n._adaptiveExpandedKey,n._items),o=_.default.getIndexByKey(e,n._items);i>=0&&i===o&&!t&&(e=void 0,o=-1),n._adaptiveExpandedKey=e,i>=0&&i++,o>=0&&o++;var a=n.getRowIndexDelta();n.updateItems({allowInvisibleRowIndices:!0,changeType:"update",rowIndices:[i-a,o-a]})},init:function(){this.callBase(),this._adaptiveExpandedKey=void 0}},editorFactory:{_getFocusCellSelector:function(){return this.callBase()+", .dx-adaptive-detail-row .dx-field-item > .dx-field-item-content"},_getTooltipsSelector:function(){return this.callBase()+", .dx-field-item-content ."+this.addWidgetPrefix(q)}},columns:{_isColumnVisible:function(e){return this.callBase(e)&&!e.adaptiveHidden}},keyboardNavigation:{_isCellValid:function(e){return this.callBase.apply(this,arguments)&&!e.hasClass(this.addWidgetPrefix(P))},_processNextCellInMasterDetail:function(e){this.callBase(e);var t=this._editingController.isCellOrBatchEditMode();if(!this._isInsideEditForm(e)&&e&&t){var n=function t(){l.default.off(e,"focus",t),l.default.trigger(e,"dxclick")};l.default.on(e,"focus",n)}},_handleTabKeyOnMasterDetailCell:function(e,t){var n=this.callBase(e,t),i=this._getFocusedCell(),o=i&&i.parent();if(!n&&o&&o.length){var r=a(o),s="next"===t?r.last():r.first(),l=o.get(0).rowIndex,u=this._adaptiveController,c=this._dataController.getKeyByRowIndex("next"===t?l:l-1),d=i&&s&&i.get(0)===s.get(0);return u.isAdaptiveDetailRowExpanded(c)&&d}return n},init:function(){this.callBase(),this._adaptiveController=this.getController("adaptiveColumns")}}}}};t.default=Y,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(7),o=n(502),a=50,r={_resizeCore:function(){this.callBase.apply(this,arguments),this._columnsController.resize()},_handleScroll:function(e){var t=this,n=this.getScrollable(),i=e.scrollOffset.left;t.callBase.apply(t,arguments),t.option("rtlEnabled")&&n&&(i=n.$content().width()-n.$element().width()-i),t._columnsController.setScrollPosition(i)}},s={_renderCore:function(){var e=this,t=e._scrollLeft;e.callBase.apply(e,arguments),e._columnsController.isVirtualMode()&&t>=0&&(e._scrollLeft=0,e.scrollTo({left:t}))}},l=function(){var e=function(e){return e.map(function(e){return e.visibleWidth||parseFloat(e.width)||a})},t={init:function(){var e=this;e.callBase(),e._beginPageIndex=0,e._endPageIndex=0,e._position=0,e._virtualVisibleColumns={}},resetColumnsCache:function(){this.callBase(),this._virtualVisibleColumns={}},getBeginPageIndex:function(t){for(var n=this.getVisibleColumns(void 0,!0),i=e(n),o=0,a=0;a<i.length;a++){if(o>=t)return Math.floor(a/this.getColumnPageSize());o+=i[a]}return 0},getTotalWidth:function(){var e=this.option("width");return"number"==typeof e?e:this.getController("resizing")._lastWidth||this.component.$element().outerWidth()},getEndPageIndex:function(t){var n=this.getVisibleColumns(void 0,!0),i=e(n),o=0;t+=this.getTotalWidth();for(var a=0;a<i.length;a++){if(o>=t)return Math.ceil(a/this.getColumnPageSize());o+=i[a]}return Math.ceil(i.length/this.getColumnPageSize())},getColumnPageSize:function(){return this.option("scrolling.columnPageSize")},_fireColumnsChanged:function(){var e=new Date;this.columnsChanged.fire({optionNames:{all:!0,length:1},changeTypes:{columns:!0,length:1}}),this._renderTime=new Date-e},setScrollPosition:function(e){var t=this,n=t.option("scrolling.columnRenderingThreshold");t._renderTime>n?(clearTimeout(t._changedTimeout),t._changedTimeout=setTimeout(function(){t._setScrollPositionCore(e)},t.option("scrolling.timeout"))):t._setScrollPositionCore(e)},isVirtualMode:function(){return(0,i.hasWindow)()&&"virtual"===this.option("scrolling.columnRenderingMode")},resize:function(){this._setScrollPositionCore(this._position)},_setScrollPositionCore:function(e){var t=this;if(t.isVirtualMode()){var n=t.getBeginPageIndex(e),i=t.getEndPageIndex(e),o=e<t._position?t._beginPageIndex>n:t._endPageIndex<i;t._position=e,o&&(t._beginPageIndex=n,t._endPageIndex=i,t._fireColumnsChanged())}},getFixedColumns:function(e,t){var n=this.callBase(e);if(this.isVirtualMode()&&!t&&n.length){var i=n.map(function(e){return e.command}).indexOf("transparent");return n[i].colspan=this.getVisibleColumns().length-this.callBase().length+1,n}return n},getVisibleColumns:function(t,n){if(n||!this.isVirtualMode())return this.callBase(t);this._beginPageIndex||this._endPageIndex||(this._beginPageIndex=this.getBeginPageIndex(this._position),this._endPageIndex=this.getEndPageIndex(this._position));var i=this._beginPageIndex,a=this._endPageIndex,r=t+"-"+i+"-"+a;if(this._virtualVisibleColumns[r])return this._virtualVisibleColumns[r];var s=this.callBase(),l=this.getRowCount(),u=this.getColumnPageSize(),c=i*u,d=a*u,h=this.getFixedColumns(void 0,!0),f=h.map(function(e){return e.command}).indexOf("transparent"),p=h.length?f:0,g=s.slice(0,p),_=s.slice(p,c),m=e(_).reduce(function(e,t){return e+t},0);m||(c=0);var v=h.length?h.length-f-1:0,y=s.slice(s.length-v),b=s.slice(d,s.length-v),x=e(b).reduce(function(e,t){return e+t},0);if(x||(d=s.length),l>1&&"number"==typeof t){for(var w=[],C=0;C<l;C++)w.push(this.callBase(C));g=(0,o.createColumnsInfo)(w,0,g.length)[t]||[],y=(0,o.createColumnsInfo)(w,s.length-y.length,s.length)[t]||[],s=(0,o.createColumnsInfo)(w,c,d)[t]||[]}else s=s.slice(c,d);return m&&(s.unshift({command:"virtual",width:m}),s=g.concat(s)),x&&(s.push({command:"virtual",width:x}),s=s.concat(y)),this._virtualVisibleColumns[r]=s,s},getColumnIndexOffset:function(){var e=0;if(this._beginPageIndex>0){var t=this.getFixedColumns(),n=t.map(function(e){return e.command}).indexOf("transparent"),i=n>=0?n:0;e=this._beginPageIndex*this.getColumnPageSize()-i-1}return e>0?e:0},dispose:function(){clearTimeout(this._changedTimeout),this.callBase.apply(this,arguments)}};return t}(),u={defaultOptions:function(){return{scrolling:{columnRenderingMode:"standard",columnPageSize:5,columnRenderingThreshold:300}}},extenders:{controllers:{columns:l},views:{columnHeadersView:s,footerView:s,rowsView:r}}};t.default=u,e.exports=t.default},function(e,t,n){function i(e,t,n,o,a,r){n=n||0,o=o||[],r=r||[],o[n]=o[n]||0;var s,l=e[n],u=r[n]+1||0,c=0;if(l){for(s=u;s<l.length;s++){var d=l[s],h=s+o[n],f=d.colspan||1;if(i(e,t,n+(d.rowspan||1),o,f,r),o[n]+=f-1,c+=f,d.rowspan)for(var p=n+1;p<n+d.rowspan;p++)o[p]=o[p]||0,o[p]+=d.colspan||1;if(t(d,h,n,s)===!1)break;if(void 0!==a&&c>=a)break}r[n]=s}}function o(e,t,n){var o=[];i(e,function(e,i,r){var s,l=e,u=l.colspan||1,c=i+u-1>=t&&i<n;if(o[r]=o[r]||[],c)i<t?(s=u-(t-i),i=t):s=u,i+s>n&&(s=n-i),s!==u&&(l=(0,a.extend)({},l,{colspan:s})),o[r].push(l);else if(i>n)return!1});for(var r=0;r<o.length;r++)o[r]=o[r]||[];return o}t.foreachColumnInfo=i,t.createColumnsInfo=o;var a=n(0)},function(e,t,n){t.default=void 0;var i=n(0),o={_getEmptyCell:function(){return{text:"",value:void 0,colspan:1,rowspan:1}},_defaultSetter:function(e){return e?e:1},_cloneItem:function(e){return(0,i.extend)({},e,this._getEmptyCell())},_prepareItems:function(e){function t(e,o,a){var r=i[e]=i[e]||[];if(r[o]=a,a.colspan>1||a.rowspan>1){for(var s=n._cloneItem(a),l=1;l<a.colspan;l++)t(e,o+l,s);for(var u=1;u<a.rowspan;u++)for(var c=0;c<a.colspan;c++)t(e+u,o+c,s)}}for(var n=this,i=[],o=(e[0]||[]).reduce(function(e,t){return e+n._defaultSetter(t.colspan)},0),a=function(e){var t=0,i=0;return function(){var o=e[t]||[],a=o[i++];return i>=o.length&&(t++,i=0),a&&(a.colspan=n._defaultSetter(a.colspan),a.rowspan=n._defaultSetter(a.rowspan)),a}}(e),r=a(),s=0;r;){for(var l=0;l<o&&r;l++)i[s]&&i[s][l]||(t(s,l,r),l+=r.colspan-1,r=a());s++}return i}};t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(44)),r=n(3),s=i(n(37)),l=n(4),u=n(1),c=n(6),d="dx-row-focused",h=".dx-row."+d,f="table",p="dx-cell-focus-disabled",g=a.default.ViewController.inherit(function(){return{init:function(){this._dataController=this.getController("data"),this._keyboardController=this.getController("keyboardNavigation"),this.component._optionsByReference.focusedRowKey=!0},optionChanged:function(e){if("focusedRowIndex"===e.name){var t=this.option("focusedRowKey");this._focusRowByIndex(e.value),this._triggerFocusedRowChangedIfNeed(t,e.value),e.handled=!0}else if("focusedRowKey"===e.name){var n=this.option("focusedRowIndex");this._focusRowByKey(e.value),this._triggerFocusedRowChangedIfNeed(e.value,n),e.handled=!0}else"focusedColumnIndex"===e.name?e.handled=!0:"focusedRowEnabled"===e.name?e.handled=!0:"autoNavigateToFocusedRow"===e.name?e.handled=!0:this.callBase(e)},_triggerFocusedRowChangedIfNeed:function(e,t){var n=this.getFocusedRowIndexByKey(e);if(t===n){var i=this._dataController.getRowIndexByKey(e);if(i>=0){var a=(0,o.default)(this.getView("rowsView").getRowElement(i));this.getController("keyboardNavigation")._fireFocusedRowChanged(a,t)}}},isAutoNavigateToFocusedRow:function(){return"infinite"!==this.option("scrolling.mode")&&this.option("autoNavigateToFocusedRow")},_focusRowByIndex:function(e){this.option("focusedRowEnabled")&&(e=void 0!==e?e:this.option("focusedRowIndex"),e<0?this.isAutoNavigateToFocusedRow()&&this._resetFocusedRow():this._focusRowByIndexCore(e))},_focusRowByIndexCore:function(e){var t=this,n=this.getController("data"),i=n.pageSize(),o=function(){if(t._isValidFocusedRowIndex(e)){var i=e-n.getRowIndexOffset(),o=n._getLastItemIndex(),a=Math.min(i,o),r=n.getKeyByRowIndex(a);(0,u.isDefined)(r)&&!t.isRowFocused(r)&&t.option("focusedRowKey",r)}};if(i>=0)if(this._isLocalRowIndex(e))o();else{var a=Math.floor(e/n.pageSize());(0,c.when)(n.pageIndex(a),n.waitReady()).done(function(){o()})}},_isLocalRowIndex:function(e){var t=this.getController("data"),n=this.getController("keyboardNavigation")._isVirtualScrolling();if(n){var i=Math.floor(e/t.pageSize()),o=t.virtualItemsCount(),a=o?o.begin:-1,r=t.getVisibleRows().length+t.getRowIndexOffset(),s=Math.ceil(r/t.pageSize());return a<=e&&s>i}return!0},_setFocusedRowKeyByIndex:function(e){var t=this.getController("data");if(this._isValidFocusedRowIndex(e)){var n=Math.min(e-t.getRowIndexOffset(),t.items().length-1),i=t.getKeyByRowIndex(n);(0,u.isDefined)(i)&&!this.isRowFocused(i)&&this.option("focusedRowKey",i)}},_focusRowByKey:function(e){(0,u.isDefined)(e)?this._navigateToRow(e,!0):this._resetFocusedRow()},_resetFocusedRow:function(){var e=this.option("focusedRowKey"),t=(0,u.isDefined)(e);if(t||!(this.option("focusedRowIndex")<0)){var n=this.getController("keyboardNavigation");t&&this.option("focusedRowKey",void 0),n.setFocusedRowIndex(-1),this.option("focusedRowIndex",-1),this.getController("data").updateItems({changeType:"updateFocusedRow",focusedRowKey:void 0}),n._fireFocusedRowChanged(void 0,-1)}},_isValidFocusedRowIndex:function(e){var t=this.getController("data"),n=t.getVisibleRows()[e];return!n||"data"===n.rowType||"group"===n.rowType},publicMethods:function(){return["navigateToRow","isRowFocused"]},navigateToRow:function(e){this.isAutoNavigateToFocusedRow()||this.option("focusedRowIndex",-1),this._navigateToRow(e)},_navigateToRow:function(e,t){var n=this,i=n.getController("data"),o=n.isAutoNavigateToFocusedRow(),a=new c.Deferred;if(void 0===e||!i.dataSource())return a.reject().promise();var r=n.getFocusedRowIndexByKey(e),s=i.getDataSource().paginate();return!o&&t||!s||r>=0?n._navigateTo(e,a,t):i.getPageIndexByKey(e).done(function(o){return o<0?void a.resolve(-1):void(o===i.pageIndex()?i.reload().done(function(){n.isRowFocused(e)?a.resolve(n.getFocusedRowIndexByKey(e)):n._navigateTo(e,a,t)}).fail(a.reject):i.pageIndex(o).done(function(){n._navigateTo(e,a,t)}).fail(a.reject))}).fail(a.reject),a.promise()},_navigateTo:function(e,t,n){var i=this.getController("data").getRowIndexByKey(e),o="virtual"===this.option("scrolling.rowRenderingMode"),a=this.isAutoNavigateToFocusedRow();a&&o&&i<0?this._navigateToVirtualRow(e,t,n):this._navigateToVisibleRow(e,t,n)},_navigateToVisibleRow:function(e,t,n){n?this._triggerUpdateFocusedRow(e,t):this.getView("rowsView").scrollToRowElement(e)},_navigateToVirtualRow:function(e,t,n){var i=this,o=this.getController("data"),a=o._rowsScrollController,r=s.default.getIndexByKey(e,o.items(!0)),l=i.getView("rowsView").getScrollable();if(a&&l&&r>=0){var u=r+o.getRowIndexOffset()-o.getRowIndexDelta(),c=a.getItemOffset(u);if(n){var d=function n(){i.component.off("contentReady",n),i._triggerUpdateFocusedRow(e,t)};i.component.on("contentReady",d)}l.scrollTo({y:c})}},_triggerUpdateFocusedRow:function(e,t){var n=this.getController("data"),i=this.getFocusedRowIndexByKey(e);this._isValidFocusedRowIndex(i)?(this.option("focusedRowEnabled")?n.updateItems({changeType:"updateFocusedRow",focusedRowKey:e}):this.getView("rowsView").scrollToRowElement(e),this.getController("keyboardNavigation").setFocusedRowIndex(i),t&&t.resolve(i)):t&&t.resolve(-1)},getFocusedRowIndexByKey:function(e){var t=this.getController("data"),n=t.getRowIndexByKey(e);return n>=0?n+t.getRowIndexOffset():-1},_focusRowByKeyOrIndex:function(){var e=this,t=this.option("focusedRowKey"),n=this.option("focusedRowIndex"),i=this.getController("keyboardNavigation"),o=this.getController("data");if((0,u.isDefined)(t)){var a=o.getRowIndexByKey(t);a>=0?(i._isVirtualScrolling()&&(n=a+o.getRowIndexOffset()),i.setFocusedRowIndex(n),this._triggerUpdateFocusedRow(t)):this._navigateToRow(t,!0).done(function(t){n>=0&&t<0&&e._focusRowByIndex()})}else n>=0&&this.getController("focus")._focusRowByIndex(n)},isRowFocused:function(e){var t=this.option("focusedRowKey");if((0,u.isDefined)(t))return(0,l.equalByValue)(e,this.option("focusedRowKey"))},updateFocusedRow:function(e){var t,n=this,i=n._dataController.getRowIndexByKey(e.focusedRowKey),a=n.getView("rowsView");(0,r.each)(a.getTableElements(),function(a,r){var s,l=0===a;t=(0,o.default)(r),n._clearPreviousFocusedRow(t,i),n._prepareFocusedRow({changedItem:null===e||void 0===e?void 0:null===(s=e.items)||void 0===s?void 0:s[i],$tableElement:t,focusedRowIndex:i,isMainTable:l})})},_clearPreviousFocusedRow:function(e,t){var n=this,i=function(t,i){var a=(0,o.default)(i).closest(".".concat(n.addWidgetPrefix(f)));return e.is(a)},a=e.find(h).filter(i);if(a.removeClass(d).removeClass(p).removeAttr("tabindex"),a.children("td").removeAttr("tabindex"),0!==t){var r=(0,o.default)(this.getView("rowsView").getRowElement(0));r.removeClass(p).removeAttr("tabIndex")}},_prepareFocusedRow:function(e){var t,n=e.changedItem;if(n&&("data"===n.rowType||"group"===n.rowType)){var i=e.focusedRowIndex,a=e.$tableElement,r=e.isMainTable,s=this.option("tabindex")||0,l=this.getView("rowsView");t=(0,o.default)(l._getRowElements(a).eq(i)),t.addClass(d).attr("tabindex",s),r&&l.scrollToElementVertically(t)}return t}}}()),_={defaultOptions:function(){return{focusedRowEnabled:!1,autoNavigateToFocusedRow:!0,focusedRowKey:void 0,focusedRowIndex:-1,focusedColumnIndex:-1}},controllers:{focus:g},extenders:{controllers:{keyboardNavigation:{init:function(){var e=this.option("focusedRowIndex"),t=this.option("focusedColumnIndex");this.createAction("onFocusedRowChanging",{excludeValidators:["disabled","readOnly"]}),this.createAction("onFocusedRowChanged",{excludeValidators:["disabled","readOnly"]}),this.createAction("onFocusedCellChanging",{excludeValidators:["disabled","readOnly"]}),this.createAction("onFocusedCellChanged",{excludeValidators:["disabled","readOnly"]}),this.callBase(),this.setRowFocusType(),this._focusedCellPosition={},(0,u.isDefined)(e)&&(this._focusedCellPosition.rowIndex=this.option("focusedRowIndex")),(0,u.isDefined)(t)&&(this._focusedCellPosition.columnIndex=this.option("focusedColumnIndex"))},setFocusedRowIndex:function(e){var t=this.getController("data");this.callBase(e);var n=e-t.getRowIndexOffset(),i=t.getVisibleRows()[n];i&&i.isNewRow||this.option("focusedRowIndex",e)},setFocusedColumnIndex:function(e){this.callBase(e),this.option("focusedColumnIndex",e)},_escapeKeyHandler:function(e,t){return t||!this.option("focusedRowEnabled")?void this.callBase(e,t):void(this.isCellFocusType()&&(this.setRowFocusType(),this._focus(this._getCellElementFromTarget(e.originalEvent.target),!0)))},_updateFocusedCellPosition:function(e,t){var n=this.option("focusedRowIndex"),i=this.option("focusedColumnIndex"),o=this.callBase(e,t);o&&o.columnIndex>=0&&this._fireFocusedCellChanged(e,i,n)}},editorFactory:{renderFocusOverlay:function(e,t){var n,i=this.getController("keyboardNavigation"),o=this.option("focusedRowEnabled"),a=this.getController("editing"),r="row"===i._getElementType(e);o&&i.isRowFocusType()&&!a.isEditing()?o&&r&&!e.hasClass(d)&&(n=i.getFirstValidCellInRow(e),i.focus(n)):this.callBase(e,t)}},columns:{getSortDataSourceParameters:function(e,t){var n=this,i=this.callBase.apply(this,arguments),o=this.getController("data"),a=o._dataSource,r=o.store(),s=r&&r.key(),l=a&&a.remoteOperations()||{},u=Object.keys(l).every(function(e){return!l[e]});if(s&&(this.option("focusedRowEnabled")&&this.getController("focus").isAutoNavigateToFocusedRow()!==!1||t)){s=Array.isArray(s)?s:[s];var c=s.filter(function(e){return!n.columnOption(e,"sortOrder")});c.length&&(i=i||[],u?i.push({selector:a.getDataIndexGetter(),desc:!1}):c.forEach(function(e){return i.push({selector:e,desc:!1})}))}return i}},data:{_applyChange:function(e){if(!e||"updateFocusedRow"!==e.changeType)return this.callBase.apply(this,arguments)},_fireChanged:function(e){if(this.callBase(e),this.option("focusedRowEnabled")&&this._dataSource){var t="update"===e.changeType&&e.repaintChangesOnly,n=t&&e.changeTypes&&e.changeTypes.indexOf("remove")>=0;"refresh"===e.changeType&&e.items.length||n?(this._updatePageIndexes(),this.processUpdateFocusedRow(e)):"append"!==e.changeType&&"prepend"!==e.changeType||this._updatePageIndexes()}},_updatePageIndexes:function(){var e=this._lastRenderingPageIndex||0,t=this._rowsScrollController?this._rowsScrollController.pageIndex():0;this._lastRenderingPageIndex=t,this._isPagingByRendering=t!==e},isPagingByRendering:function(){return this._isPagingByRendering},processUpdateFocusedRow:function(e){var t=e.operationTypes||{},n=this.getController("focus"),i=t.reload,o=t.fullReload,a=this.getController("keyboardNavigation"),r=a._isVirtualScrolling(),s=this.option("focusedRowKey"),l=n.isAutoNavigateToFocusedRow();if(i&&!o&&(0,u.isDefined)(s))n._navigateToRow(s,!0).done(function(e){e<0&&n._focusRowByIndex()});else if(t.paging&&!r)if(l){var c=this.getRowIndexByKey(s),d=c>=0,h=this.option("focusedRowIndex"),f=h>=0&&(h===c||!d);f&&n._focusRowByIndex()}else this.getRowIndexByKey(s)<0&&this.option("focusedRowIndex",-1);else t.fullReload&&n._focusRowByKeyOrIndex()},getPageIndexByKey:function(e){var t=this,n=new c.Deferred;return t.getGlobalRowIndexByKey(e).done(function(e){n.resolve(e>=0?Math.floor(e/t.pageSize()):-1)}).fail(n.reject),n.promise()},getGlobalRowIndexByKey:function(e){return this._dataSource.group()?this._calculateGlobalRowIndexByGroupedData(e):this._calculateGlobalRowIndexByFlatData(e)},_calculateGlobalRowIndexByFlatData:function(e,t,n){var i=this,o=new c.Deferred,a=i._dataSource,r=i._generateFilterByKey(e);return a.load({filter:i._concatWithCombinedFilter(r),skip:0,take:1}).done(function(s){s.length>0?(r=i._generateOperationFilterByKey(e,s[0],n),a.load({filter:i._concatWithCombinedFilter(r,t),skip:0,take:1,requireTotalCount:!0}).done(function(e,t){o.resolve(t.totalCount)})):o.resolve(-1)}),o.promise()},_concatWithCombinedFilter:function(e,t){var n=this.getCombinedFilter();return s.default.combineFilters([e,n,t])},_generateBooleanFilter:function(e,t,n){var i;return t===!1?i=[e,"=",!!n.desc||null]:(t===!0?!n.desc:n.desc)&&(i=[e,"<>",t]),i},_generateOperationFilterByKey:function(e,t,n){var i=this,o=i._dataSource,a=i._generateFilterByKey(e,"<"),r=i._columnsController.getSortDataSourceParameters(!o.remoteOperations().filtering,!0);if(n){var s=i._columnsController.getGroupDataSourceParameters(!o.remoteOperations().filtering);s&&(r=r?s.concat(r):s)}return r&&r.slice().reverse().forEach(function(e){var n,o=e.selector;n="function"==typeof o?o:i._columnsController.columnOption(o,"selector");var r=n?n(t):t[o];if(a=[[o,"=",r],"and",a],null===r||(0,u.isBoolean)(r)){var s=i._generateBooleanFilter(o,r,e);s&&(a=[s,"or",a])}else{var l=e.desc?">":"<",c=[o,l,r];e.desc||(c=[c,"or",[o,"=",null]]),a=[c,"or",a]}}),a},_generateFilterByKey:function(e,t){var n=this._dataSource.key(),i=[];if(t||(t="="),Array.isArray(n))for(var o=0;o<n.length;++o){var a=e[n[o]];a&&(i.length>0&&i.push("and"),i.push([n[o],t,a]))}else i=[n,t,e];return i},_getLastItemIndex:function(){return this.items(!0).length-1}}},views:{rowsView:{_createRow:function(e){var t=this.callBase(e);return this.option("focusedRowEnabled")&&e&&this.getController("focus").isRowFocused(e.key)&&t.addClass(d),t},_checkRowKeys:function(e){if(this.callBase.apply(this,arguments),this.option("focusedRowEnabled")&&this.option("dataSource")){var t=this._dataController.store();t&&!t.key()&&this._dataController.fireError("E1042","Row focusing")}},_update:function(e){"updateFocusedRow"===e.changeType?this.option("focusedRowEnabled")&&this.getController("focus").updateFocusedRow(e):this.callBase(e)},updateFocusElementTabIndex:function(e,t){this.option("focusedRowEnabled")?this._setFocusedRowElementTabIndex(t):this.callBase(e)},_setFocusedRowElementTabIndex:function(e){var t=this,n=this.option("focusedRowKey"),i=this.option("tabIndex")||0,o=this._dataController,a=this._columnsController,r=o.getRowIndexByKey(n),s=this.option("focusedColumnIndex"),l=this._findRowElementForTabIndex();if((0,u.isDefined)(this._scrollToFocusOnResize)||(this._scrollToFocusOnResize=function(){t.scrollToElementVertically(t._findRowElementForTabIndex()),t.resizeCompleted.remove(t._scrollToFocusOnResize)}),l.attr("tabIndex",i),r>=0&&!e&&(s<0&&(s=0),r+=o.getRowIndexOffset(),s+=a.getColumnIndexOffset(),this.getController("keyboardNavigation").setFocusedCellPosition(r,s),this.getController("focus").isAutoNavigateToFocusedRow())){var c=o.dataSource(),d=c&&c.operationTypes();!d||d.paging||o.isPagingByRendering()||(this.resizeCompleted.remove(this._scrollToFocusOnResize),this.resizeCompleted.add(this._scrollToFocusOnResize))}},_findRowElementForTabIndex:function(){var e=this.option("focusedRowKey"),t=this._dataController.getRowIndexByKey(e);return(0,o.default)(this.getRowElement(t>=0?t:0))},scrollToRowElement:function(e){var t=this.getController("data").getRowIndexByKey(e),n=(0,o.default)(this.getRow(t));this.scrollToElementVertically(n)},scrollToElementVertically:function(e){var t=this.getScrollable();if(t){var n=t.getScrollElementPosition(e,"vertical");t.scrollTo({top:n})}}}}}};t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(0),r=i(n(286)),s=i(n(37)),l=i(n(23)),u="dx-command-drag",c="dx-cell-focus-disabled",d="drag-icon",h="rowsview",f="dx-sortable-without-handle",p={init:function(){this.callBase.apply(this,arguments),this._updateHandleColumn()},_allowReordering:function(){var e=this.option("rowDragging");return!(!e||!(e.allowReordering||e.allowDropInsideItem||e.group))},_updateHandleColumn:function(){var e=this.option("rowDragging"),t=this._allowReordering(),n=this._columnsController,i=t&&e.showDragIcons;n&&n.addCommandColumn({type:"drag",command:"drag",visibleIndex:-2,alignment:"center",cssClass:u,width:"auto",cellTemplate:this._getHandleTemplate(),visible:i}),n.columnOption("type:drag","visible",i)},_renderContent:function(){var e=this,t=this.option("rowDragging"),n=this._allowReordering(),i=this.callBase.apply(this,arguments),o=this._isFixedTableRendering,s="_sortable",c="_sortableFixed",d=o?c:s,h=o?s:c,p=function(t){var n;null===(n=e[c])||void 0===n?void 0:n.$element().css("pointerEvents",t?"auto":"")};return n&&i.length&&(this[d]=this._createComponent(i,r.default,(0,a.extend)({component:this.component,contentTemplate:null,filter:"> table > tbody > .dx-row:not(.dx-freespace-row):not(.dx-virtual-row)",dragTemplate:this._getDraggableRowTemplate(),handle:t.showDragIcons&&".".concat(u),dropFeedbackMode:"indicate"},t,{onDragStart:function(e){var n,i=e.component.getVisibleRows()[e.fromIndex];e.itemData=i&&i.data;var o=i&&"data"===i.rowType;e.cancel=!o,null===(n=t.onDragStart)||void 0===n?void 0:n.call(t,e)},onDragEnter:function(){p(!0)},onDragLeave:function(){p(!1)},onDragEnd:function(e){var n;p(!1),null===(n=t.onDragEnd)||void 0===n?void 0:n.call(t,e)},dropFeedbackMode:l.default.msie?"indicate":t.dropFeedbackMode,onOptionChanged:function(t){var n=e[c];n&&("fromIndex"!==t.name&&"toIndex"!==t.name||e[h].option(t.name,t.value))}})),i.toggleClass(f,!t.showDragIcons)),i},_resizeCore:function(){this.callBase.apply(this,arguments);var e=this._dataController.getRowIndexOffset();[this._sortable,this._sortableFixed].forEach(function(t){null===t||void 0===t?void 0:t.option("offset",e),null===t||void 0===t?void 0:t.update()})},_getDraggableGridOptions:function(e){var t=this.option(),n=this.getColumns(),i=(0,o.default)(this.getRowElement(e.rowIndex));return{dataSource:[{id:1,parentId:0}],showBorders:!0,showColumnHeaders:!1,scrolling:{useNative:!1,showScrollbar:!1},pager:{visible:!1},loadingTimeout:void 0,columnFixing:t.columnFixing,columnAutoWidth:t.columnAutoWidth,showColumnLines:t.showColumnLines,columns:n.map(function(e){return{width:e.width||e.visibleWidth,fixed:e.fixed,fixedPosition:e.fixedPosition}}),onRowPrepared:function(e){var t=e.component.getView("rowsView");(0,o.default)(e.rowElement).replaceWith(i.eq(t._isFixedTableRendering?1:0).clone())}}},_getDraggableRowTemplate:function(){var e=this;return function(t){var n=e.component.$element(),i=(0,o.default)("<div>").width(n.width()),a=e._dataController.items(),r=a&&a[t.fromIndex],s=e._getDraggableGridOptions(r);return e._createComponent(i,e.component.NAME,s),i.find(".dx-gridbase-container").children(":not(.".concat(e.addWidgetPrefix(h),")")).hide(),i}},_getHandleTemplate:function(){var e=this;return function(t,n){return"data"===n.rowType?((0,o.default)(t).addClass(c),(0,o.default)("<span>").addClass(e.addWidgetPrefix(d))):void s.default.setEmptyText((0,o.default)(t))}},optionChanged:function(e){"rowDragging"===e.name&&(this._updateHandleColumn(),this._invalidate(!0,!0),e.handled=!0),this.callBase.apply(this,arguments)}},g={defaultOptions:function(){return{rowDragging:{showDragIcons:!0,dropFeedbackMode:"indicate",allowReordering:!1,allowDropInsideItem:!1}}},extenders:{views:{rowsView:p}}};t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(736)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(31)),a=i(n(17)),r=n(4),s=n(1),l=n(3),u=n(19),c=n(0),d=i(n(37)),h=n(90),f=i(n(81)),p=i(n(47)),g=i(n(464)),_=n(6),m=i(n(91)),v=m.default.queryByOptions,y="id",b=function(e){return"fullBranch"===e.option("filterMode")},x=g.default.inherit(function(){var e=function(e,t){var n=[];return t.forEach(function(t){var i=e.getNodeByKey(t);i&&i.children.forEach(function(e){n.push(e.key)})}),n};return{_createKeyGetter:function(){var e=this.getKeyExpr();return(0,u.compileGetter)(e)},_createKeySetter:function(){var e=this.getKeyExpr();return(0,s.isFunction)(e)?e:(0,u.compileSetter)(e)},_createParentIdGetter:function(){return(0,u.compileGetter)(this.option("parentIdExpr"))},createParentIdSetter:function(){var e=this.option("parentIdExpr");return(0,s.isFunction)(e)?e:(0,u.compileSetter)(e)},_createItemsGetter:function(){return(0,u.compileGetter)(this.option("itemsExpr"))},_createHasItemsGetter:function(){var e=this.option("hasItemsExpr");return e&&(0,u.compileGetter)(e)},_createHasItemsSetter:function(){var e=this.option("hasItemsExpr");return(0,s.isFunction)(e)?e:e&&(0,u.compileSetter)(e)},_updateIndexByKeyObject:function(e){var t=this;t._indexByKey={},(0,l.each)(e,function(e,n){t._indexByKey[n.key]=e})},_calculateHasItems:function(e,t){var n,i=this,o=t.storeLoadOptions.parentIds,a=b(i);return i._hasItemsGetter&&(o||!t.storeLoadOptions.filter||a)&&(n=i._hasItemsGetter(e.data)),void 0===n&&(n=!(i._isChildrenLoaded[e.key]||!t.remoteOperations.filtering||!o&&!a)||(t.loadOptions.filter&&!t.remoteOperations.filtering&&a?e.children.length:e.hasChildren)),!!n},_createVisibleItemsByNodes:function(e,t){for(var n=this,i=[],o=0;o<e.length;o++)e[o].visible&&i.push(e[o]),(n.isRowExpanded(e[o].key,t)||!e[o].visible)&&e[o].hasChildren&&e[o].children.length&&(i=i.concat(n._createVisibleItemsByNodes(e[o].children,t)));return i},_convertItemToNode:function(e,t,n){var i=this._keyGetter(e),o=this._parentIdGetter(e);o=(0,s.isDefined)(o)?o:t;var a=n[o]=n[o]||{key:o,children:[]},r=n[i]=n[i]||{key:i,children:[]};return r.data=e,r.parent=a,r},_createNodesByItems:function(e,t){var n,i=this,o=i.option("rootValue"),a={},r=i._nodeByKey={};if(t)for(n=0;n<t.length;n++)a[this._keyGetter(t[n])]=!0;for(n=0;n<e.length;n++){var s=i._convertItemToNode(e[n],o,r);if(void 0===s.key)return;s.visible=!t||!!a[s.key],s.parent&&s.parent.children.push(s)}var l=r[o]||{key:o,children:[]};return l.level=-1,l},_convertDataToPlainStructure:function(e,t,n){var i;if(this._itemsGetter&&!e.isConverted){n=n||[];for(var o=0;o<e.length;o++){var a=(0,h.createObjectWithChanges)(e[o]);i=this._keyGetter(a),void 0===i&&(i=n.length+1,this._keySetter(a,i)),this._parentIdSetter(a,void 0===t?this.option("rootValue"):t),n.push(a);var r=this._itemsGetter(a);if(r&&r.length){this._convertDataToPlainStructure(r,i,n);var l=this.option("itemsExpr");(0,s.isFunction)(l)||delete a[l]}}return n.isConverted=!0,n}return e},_createIdFilter:function(e,t){for(var n=[],i=0;i<t.length;i++)n.push([e,"=",t[i]]);return d.default.combineFilters(n,"or")},_customizeRemoteOperations:function(e,t){this.callBase.apply(this,arguments),e.remoteOperations.paging=!1;var n=!1;this.option("autoExpandAll")&&(e.remoteOperations.sorting=!1,e.remoteOperations.filtering=!1,this._lastLoadOptions&&(!t.filtering||e.storeLoadOptions.filter)||e.isCustomLoading||(n=!0)),e.isCustomLoading||(this._isReload=this._isReload||t.reload,e.cachedStoreData||(this._isChildrenLoaded={},this._isReload&&(this._nodeByKey={})),this.option("expandNodesOnFiltering")&&(t.filtering||this._isReload&&e.storeLoadOptions.filter)&&(e.storeLoadOptions.filter?n=!0:e.collapseVisibleNodes=!0)),e.expandVisibleNodes=n},_getParentIdsToLoad:function(e){for(var t=[],n=0;n<e.length;n++){var i=this.getNodeByKey(e[n]);(!i||i.hasChildren&&!i.children.length)&&t.push(e[n])}return t},_handleDataLoading:function(e){var t=this.option("rootValue"),n=this.option("parentIdExpr"),i=e.storeLoadOptions.parentIds;if(i&&(e.isCustomLoading=!1),this.callBase.apply(this,arguments),e.remoteOperations.filtering&&!e.isCustomLoading&&(b(this)&&e.cachedStoreData||!e.storeLoadOptions.filter)){
var o=e.collapseVisibleNodes?[]:this.option("expandedRowKeys");i=[t].concat(o).concat(i||[]);var a=e.data?this._getParentIdsToLoad(i):i;a.length&&(e.cachedPagingData=void 0,e.data=void 0,e.mergeStoreLoadData=!0),e.storeLoadOptions.parentIds=a,e.storeLoadOptions.filter=this._createIdFilter(n,a)}},_generateInfoToLoad:function(e,t){var n,i,o=this,a={},r={},s=[],l=o.option("rootValue");for(i=0;i<e.length;i++)n=t?o._parentIdGetter(e[i]):o._keyGetter(e[i]),a[n]=!0;for(i=0;i<e.length;i++){n=t?o._keyGetter(e[i]):o._parentIdGetter(e[i]);var u=t?o.isRowExpanded(n):n!==l;a[n]||r[n]||!u||(r[n]=!0,s.push(n))}return{keyMap:r,keys:s}},_loadParentsOrChildren:function(e,t,n){function i(t){return g&&(s._cachedStoreData=s._cachedStoreData.concat(t)),e.concat(t)}var o,a,r=this,s=this,l=s._generateInfoToLoad(e,n),u=l.keys,d=l.keyMap,h=new _.Deferred,g=t.remoteOperations.filtering,m=s.option("maxFilterLengthInRequest"),v=g?t.storeLoadOptions:t.loadOptions;if(!u.length)return h.resolve(e);var y=u.map(function(e){return r.getNodeByKey(e)}).filter(function(e){return e&&e.data});if(y.length===u.length&&(n&&(y=y.reduce(function(e,t){return e.concat(t.children)},[])),y.length))return s._loadParentsOrChildren(i(y.map(function(e){return e.data})),t,n);var b=n?s.option("parentIdExpr"):s.getKeyExpr();o=s._createIdFilter(b,u);var x=encodeURI(JSON.stringify(o)).length;x>m&&(o=function(e){return d[s._keyGetter(e)]},a=g),v=(0,c.extend)({},v,{filter:a?null:o});var w=t.fullData?new f.default(t.fullData):s._dataSource.store();return s.loadFromStore(v,w).done(function(r){r.length?(a&&(r=(0,p.default)(r).filter(o).toArray()),s._loadParentsOrChildren(i(r),t,n).done(h.resolve).fail(h.reject)):h.resolve(e)}).fail(h.reject),h},_loadParents:function(e,t){return this._loadParentsOrChildren(e,t)},_loadChildrenIfNeed:function(e,t){return b(this)?this._loadParentsOrChildren(e,t,!0):(0,_.when)(e)},_updateHasItemsMap:function(e){var t=e.storeLoadOptions.parentIds;if(t)for(var n=0;n<t.length;n++)this._isChildrenLoaded[t[n]]=!0},_getKeyInfo:function(){return{key:function(){return"key"},keyOf:function(e){return e.key}}},_applyBatch:function(e){var t=this,n=[];e.forEach(function(e){"insert"===e.type?n=n.concat(t._applyInsert(e)):"remove"===e.type?n=n.concat(t._applyRemove(e)):"update"===e.type&&n.push({type:e.type,key:e.key,data:{data:e.data}})}),this.callBase(n)},_setHasItems:function(e,t){var n=this._hasItemsSetter;e.hasChildren=t,n&&e.data&&n(e.data,t)},_applyInsert:function(e){var t=this,n=[],i=t.parentKeyOf(e.data),o=t.getNodeByKey(i);if(o){var a=t.option("rootValue"),r=t._convertItemToNode(e.data,a,t._nodeByKey);if(r.hasChildren=!1,r.level=o.level+1,r.visible=!0,o.children.push(r),t._isChildrenLoaded[r.key]=!0,t._setHasItems(o,!0),(!o.parent||t.isRowExpanded(o.key))&&void 0!==e.index){var s=t.items().indexOf(o)+1;s+=e.index>=0?Math.min(e.index,o.children.length):o.children.length,n.push({type:e.type,data:r,index:s})}}return n},_applyRemove:function(e){var t=[],n=this.getNodeByKey(e.key),i=n&&n.parent;if(i){var o=i.children.indexOf(n);o>=0&&(i.children.splice(o,1),i.children.length||this._setHasItems(i,!1),t.push(e),t=t.concat(this.getChildNodeKeys(e.key).map(function(t){return{type:e.type,key:t}})))}return t},_handleDataLoaded:function(e){var t=e.data=this._convertDataToPlainStructure(e.data);!e.remoteOperations.filtering&&e.loadOptions.filter&&(e.fullData=v((0,p.default)(e.data),{sort:e.loadOptions&&e.loadOptions.sort}).toArray()),this._updateHasItemsMap(e),this.callBase(e),t.isConverted&&this._cachedStoreData&&(this._cachedStoreData.isConverted=!0)},_fillNodes:function(e,t,n,i){var a=b(this);i=i||0;for(var r=0;r<e.length;r++){var s=e[r],l=!1;this._fillNodes(e[r].children,t,n,i+1),s.level=i,s.hasChildren=this._calculateHasItems(s,t),s.visible&&s.hasChildren&&(a?s.children.filter(function(e){return e.visible}).length?l=!0:s.children.length&&o.default.foreachNodes(s.children,function(e){e.visible=!0}):l=!0,t.expandVisibleNodes&&l&&n.push(s.key)),(s.visible||s.hasChildren)&&(s.parent.hasChildren=!0)}},_processTreeStructure:function(e,t){var n=e.data,i=e.storeLoadOptions.parentIds,o=[];if(i&&i.length||this._isReload){if(e.fullData&&e.fullData.length>e.data.length&&(n=e.fullData,t=t||e.data),this._rootNode=this._createNodesByItems(n,t),!this._rootNode)return void(e.data=(new _.Deferred).reject(a.default.Error("E1046",this.getKeyExpr())));this._fillNodes(this._rootNode.children,e,o),this._isNodesInitializing=!0,(e.collapseVisibleNodes||o.length)&&this.option("expandedRowKeys",o),this._isReload=!1,this.executeAction("onNodesInitialized",{root:this._rootNode}),this._isNodesInitializing=!1}n=this._createVisibleItemsByNodes(this._rootNode.children,e),e.data=n,this._totalItemsCount=n.length},_handleDataLoadedCore:function(e){var t,n=this,i=e.data,o=n.callBase,a=e.storeLoadOptions.filter||e.loadOptions.filter,r=n.option("filterMode"),s=e.storeLoadOptions.parentIds,l=a&&(!s||!s.length)&&"standard"!==r;if(!e.isCustomLoading){if(l){var u=e.data=new _.Deferred;return"matchOnly"===r&&(t=i),n._loadParents(i,e).done(function(i){n._loadChildrenIfNeed(i,e).done(function(i){e.data=i,n._processTreeStructure(e,t),o.call(n,e),u.resolve(e.data)})}).fail(u.reject)}n._processTreeStructure(e)}n.callBase(e)},_handlePush:function(e){var t=this._dataSource._reshapeOnPush,n=t&&!!e.length;n&&(this._isReload=!0),this.callBase.apply(this,arguments)},init:function(e,t){this.callBase.apply(this,arguments);var n=this.option("dataStructure");this._keyGetter=this._createKeyGetter(),this._parentIdGetter=this._createParentIdGetter(),this._hasItemsGetter=this._createHasItemsGetter(),this._hasItemsSetter=this._createHasItemsSetter(),"tree"===n&&(this._itemsGetter=this._createItemsGetter(),this._keySetter=this._createKeySetter(),this._parentIdSetter=this.createParentIdSetter()),this._nodeByKey={},this._isChildrenLoaded={},this._totalItemsCount=0,this.createAction("onNodesInitialized")},getKeyExpr:function(){var e=this.store(),t=e&&e.key(),n=this.option("keyExpr");if((0,s.isDefined)(t)&&(0,s.isDefined)(n)&&!(0,r.equalByValue)(t,n))throw a.default.Error("E1044");return t||n||y},keyOf:function(e){return this._keyGetter&&this._keyGetter(e)},parentKeyOf:function(e){return this._parentIdGetter&&this._parentIdGetter(e)},getRootNode:function(){return this._rootNode},totalItemsCount:function(){return this._totalItemsCount},isRowExpanded:function(e,t){if(t){var n=t.isExpandedByKey;return n||(n=t.isExpandedByKey={},this.option("expandedRowKeys").forEach(function(e){n[e]=!0})),!!n[e]}var i=d.default.getIndexByKey(e,this.option("expandedRowKeys"),null);return i>=0},_changeRowExpandCore:function(e){var t=this.option("expandedRowKeys").slice(),n=d.default.getIndexByKey(e,t,null);n<0?t.push(e):t.splice(n,1),this.option("expandedRowKeys",t)},changeRowExpand:function(e){return this._changeRowExpandCore(e),this._isNodesInitializing?(new _.Deferred).resolve():this.load()},getNodeByKey:function(e){if(this._nodeByKey)return this._nodeByKey[e]},getNodeLeafKeys:function(){var e=this,t=[],n=e._rootNode?[e._rootNode.key]:[];return n.forEach(function(n){var i=e.getNodeByKey(n);i&&o.default.foreachNodes([i],function(e){!e.children.length&&t.push(e.key)})}),t},getChildNodeKeys:function(e){var t=this.getNodeByKey(e),n=[];return t&&o.default.foreachNodes(t.children,function(e){n.push(e.key)}),n},loadDescendants:function(t,n){var i=this,o=new _.Deferred,a=i.remoteOperations();if(t=(0,s.isDefined)(t)?Array.isArray(t)?t:[t]:i.getNodeLeafKeys(),!a.filtering||!t.length)return o.resolve();var r=i._dataSource._createStoreLoadOptions();return r.parentIds=t,i.load(r).done(function(){if(!n){var a=e(i,t);if(a.length)return void i.loadDescendants(a,n).done(o.resolve).fail(o.reject)}o.resolve()}).fail(o.reject),o.promise()},forEachNode:function(){var e,t=[];if(1===arguments.length){e=arguments[0];var n=this.getRootNode();t=n&&n.children||[]}else 2===arguments.length&&(e=arguments[1],t=arguments[0],t=Array.isArray(t)?t:[t]);o.default.foreachNodes(t,e)}}}()),w={extend:function(e){x=x.inherit(e)},create:function(e){return new x(e)}};t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.XmlaStore=void 0;var c=i(n(2)),d=n(7),h=i(n(15)),f=n(42),p=i(n(34)),g=n(4),_=n(0),m=n(1),v=n(3),y=n(12),b=n(89),x=n(6),w=n(205),C=p.default.errors,k=(0,d.getWindow)(),S=h.default.inherit(function(){function e(e,t){var n=new x.Deferred,i=e.beforeSend,o={url:e.url,dataType:"text",data:t,headers:{"Content-Type":"text/xml"},xhrFields:{},method:"POST"};return(0,m.isFunction)(i)&&i(o),(0,b.sendRequest)(o).fail(function(){n.reject(arguments)}).done(function(e){var t,i=new k.DOMParser;try{try{t=i.parseFromString(e,"text/xml")}catch(e){t=void 0}if(!t||t.getElementsByTagName("parsererror").length||0===t.childNodes.length)throw new C.Error("E4023",e)}catch(t){n.reject({statusText:t.message,stack:t.stack,responseText:e})}n.resolve(t)}),n}function t(){var e=(0,w.getLanguageId)();return void 0!==e?(0,f.format)("<LocaleIdentifier>{0}</LocaleIdentifier>",e):""}function n(e,t,n){var i=t?t:e;return"Descendants({"+i+"}, "+n+", SELF_AND_BEFORE)"}function i(e){return(e.hierarchyName||e.dataField)+".[All]"}function a(e){var t=e.dataField+".allMembers",n=e.searchValue;return n&&(n=n.replace(/'/g,"''"),t="Filter("+t+", instr("+e.dataField+".currentmember.member_caption,'"+n+"') > 0)"),t}function r(e){var t=e.join(",");return e.length>1?(0,f.format)(ce,t):t}function s(e){var t=e.join(",");return e.length>1?"Union("+t+")":t}function l(e,t,o,s,l,u,c,d){for(var h,p,g,_=[],m=u[c],v=[],y=s;y<=t;y++){var b=m[y],x=b.dataField,w=m[y-1]&&m[y-1].hierarchyName,C=b.hierarchyName,k=!C||!m[y+1]||m[y+1].hierarchyName!==C,S=e.length+o+s;if(h=null,v.push(b),y<e.length)k&&(h="("+x+"."+H(e[y],x)+")");else if(y<=S)if(0===y&&0===o){var D=i(m[s]);h=C?D+","+m[s].dataField:a(m[s])}else C?(g=H(l[l.length-1]),(k||y===S)&&(w===C?(l.length&&(p=m[l.length-1]),p&&p.hierarchyName===C||(p=m[y-1],g=""),h=n(p.dataField,g,x)):h=a(b))):h=a(b);else{var I=!C||w!==C;I&&(h="("+i(b)+")")}if(h){if(h=(0,f.format)(de,h),d){var T=(b.hierarchyName||b.dataField)+("displayText"===b.sortBy?".MEMBER_CAPTION":".MEMBER_VALUE");h=(0,f.format)(ae,h,T,"desc"===b.sortOrder?"DESC":"ASC")}_.push(h)}}return r(_)}function u(e,t,n,i,o,a,r,s,u,c){var d,h=-1,p=a[r];do{h++,d=t.length+h+i;var g=l(t,n,h,i,o,a,r,u);u||c||(g=(0,f.format)(le,g,s)),e.push(g)}while(p[d]&&p[d+1]&&p[d].expanded)}function d(e,t,n,i){return n=n||"[DX_Set_"+t.length+"]",i=i||"set",t.push((0,f.format)(re,i,n,e)),n}function h(e,t,n,i,o){var a=e[t],l=[],c=[],h=[],p=0,g=0,_=[],m=(0,f.format)(de,n.join(","));if(a&&a.length){e.headerName===t?(c=e.path,p=c.length):e.headerName&&e.oppositePath?(c=e.oppositePath,p=c.length):h=("columns"===t?e.columnExpandedPaths:e.rowExpandedPaths)||h,g=(0,b.getExpandedLevel)(e,t),u(l,[],g,p,c,e,t,m,"rows"===t?e.rowTake:e.columnTake,e.totalsOnly),(0,v.each)(h,function(n,i){u(l,i,g,p,i,e,t,m)});for(var y=g;y>=c.length;y--)a[y].hierarchyName&&(o.visibleLevels[a[y].hierarchyName]=o.visibleLevels[a[y].hierarchyName]||[],o.visibleLevels[a[y].hierarchyName].push(a[y].dataField))}if(l.length){var x=s(l);"rows"===t&&e.rowTake&&(x=(0,f.format)(oe,x,e.rowSkip>0?e.rowSkip+1:0,e.rowSkip>0?e.rowTake:e.rowTake+1)),"columns"===t&&e.columnTake&&(x=(0,f.format)(oe,x,e.columnSkip>0?e.columnSkip+1:0,e.columnSkip>0?e.columnTake:e.columnTake+1));var w="[DX_".concat(t,"]");_.push(d(x,i,w)),e.totalsOnly&&_.push(d("COUNT(".concat(w,")"),i,"[DX_".concat(t,"_count]"),"member"))}return"columns"===t&&n.length&&!e.skipValues&&_.push(m),(0,f.format)(ue,r(_),t)}function p(e){var t=[];return(0,v.each)(e,function(e,n){var i,o=n.dataField,r=[],s=n.filterValues||[];n.hierarchyName&&(0,m.isNumeric)(n.groupIndex)||((0,v.each)(s,function(e,t){var i=o+"."+H(Array.isArray(t)?t[t.length-1]:t,o);"exclude"===n.filterType&&(r.push(i+".parent"),i="Descendants("+i+")"),r.push(i)}),s.length&&(i=(0,f.format)(de,r.join(",")),"exclude"===n.filterType&&(i="Except("+a(n)+","+i+")"),t.push(i)))}),t.length?r(t):""}function S(e,t,n,i){var o="["+i+"]";return(0,v.each)([e,t,n],function(e,t){t&&(o=(0,f.format)(ie,t+"on 0",o))}),o}function D(e,t,n,i,o,a,r){var s=arguments.length>7&&void 0!==arguments[7]?arguments[7]:{},l="",u=(t.length?"with "+t.join(" "):"")+" ";if(e.length){var c;if(s.totalsOnly){var d=[];i.length&&d.push("[DX_rows_count]"),n.length&&d.push("[DX_columns_count]"),c="{".concat(d.join(","),"} on columns")}else c=e.join(",");l=u+(0,f.format)(ne,S(p(n),p(i),p(o||[]),r),a.length?(0,f.format)(se,a.join(",")):"",c)}return l}function I(e,t){return(0,v.map)(t,function(t){return(0,m.isString)(t.expression)&&d(t.expression,e,t.dataField,"member"),t.dataField})}function T(e,t,n,i){(0,v.each)(i,function(i,o){var a=t[n][i];a.hierarchyName&&a.hierarchyName===t[n][i+1].hierarchyName||e.push(a.dataField+"."+H(o,a.dataField))})}function O(e,t,n){var i=e.columns||[],o=e.rows||[],a=e.values&&e.values.length?e.values:[{dataField:"[Measures]"}],r=[],s=[],l=[],u=I(s,a);return n.measureCount=e.skipValues?1:a.length,n.visibleLevels={},e.headerName&&e.path&&T(r,e,e.headerName,e.path),e.headerName&&e.oppositePath&&T(r,e,"rows"===e.headerName?"columns":"rows",e.oppositePath),(i.length||u.length)&&l.push(h(e,"columns",u,s,n)),o.length&&l.push(h(e,"rows",u,s,n)),D(l,s,i,o,e.filters,r,t,e)}function E(e,t,n){(0,v.each)(n,function(n,i){var o=t[n];o.hierarchyName&&(t[n+1]||{}).hierarchyName===o.hierarchyName||e.push(o.dataField+"."+H(i,o.dataField))})}function A(e,t,n){var i=e.columns||[],o=e.rows||[],a=e.values&&e.values.length?e.values:[{dataField:"[Measures]"}],r=[],s=[],l=[],u=I(s,a),c=n.maxRowCount,d=n.customColumns||[],h=d.length>0?" return "+d.join(","):"";E(r,i,n.columnPath||[]),E(r,o,n.rowPath||[]),(i.length||i.length||u.length)&&l.push([(u[n.dataIndex]||u[0])+" on 0"]);var f=D(l,s,i,o,e.filters,r,t);return f?"drillthrough"+(c>0?" maxrows "+c:"")+f+h:f}function P(e){return parseInt(e,10)}function R(e){return(0,m.isNumeric)(e)?parseFloat(e):e}function M(e,t){return(e.getElementsByTagName(t)||[])[0]}function B(e,t){return F(M(e,t))}function V(e,t){var n=[];for((0,v.each)(e.getElementsByTagName("Axis"),function(e,i){var o=i.getAttribute("name"),a=[],r=0;0===o.indexOf("Axis")&&(0,m.isNumeric)(P(o.substr(4)))&&(n.push(a),(0,v.each)(i.getElementsByTagName("Tuple"),function(e,i){var o=i.childNodes,s=0,l=[],u=t?o.length:o.length-1,c=1===n.length;c&&u--,a.push(l);for(var d=u;d>=0;d--){var h=o[d],f=P(B(h,"LNum"));l[d]={caption:B(h,"Caption"),value:R(B(h,"MEMBER_VALUE")),level:f,index:r++,hasValue:!(s||!f&&0!==d),name:B(h,"UName"),hierarchyName:o[d].getAttribute("Hierarchy"),parentName:B(h,"PARENT_UNIQUE_NAME"),levelName:B(h,"LName")},s+=f}}))});n.length<2;)n.push([[{level:0}]]);return n}function F(e){return e&&e&&(e.textContent||e.text||e.innerHTML)||""}function L(e,t,n){for(var i=[],o=[],a=0,r=[],s=e.getElementsByTagName("Cell"),l={},u=0;u<s.length;u++){var c=s[u],d=c.getElementsByTagName("Value")[0],h=d&&d.getElementsByTagName("Error")||[],f=0===h.length?F(d):"#N/A",p=parseFloat(f),g=f-p+1>0,_=P(c.getAttribute("CellOrdinal"));h.length&&(l[F(h[0].getElementsByTagName("ErrorCode")[0])]=F(h[0].getElementsByTagName("Description")[0])),r[_]={value:g?p:f||null}}return(0,v.each)(t[1],function(){var e=[];i.push(e),(0,v.each)(t[0],function(){var t=a%n;0===t&&(o=[],e.push(o)),o.push(r[a]?r[a].value:null),a++})}),Object.keys(l).forEach(function(e){C.log("W4002",l[e])}),i}function H(e,t){return e&&(e=(0,m.isString)(e)&&e.indexOf("&")!==-1?e:"["+e+"]",t&&0===e.indexOf(t+".")&&(e=e.slice(t.length+1,e.length))),e}function z(e,t,n,i){var o=e[t];return o||(o={},e[t]=o),!(0,m.isDefined)(o.value)&&n&&(o.text=n.caption,o.value=n.value,o.key=t?t:"",o.levelName=n.levelName,o.hierarchyName=n.hierarchyName,o.parentName=n.parentName,o.index=i,o.level=n.level),o}function N(e,t){var n=[],i=e.children&&(e.children.length?e.children:Object.keys(e.children.grandTotalHash||{}).reduce(function(t,n){return t.concat(e.children.grandTotalHash[n].children)},[])),o=i&&i[0];if(o&&(t[o.hierarchyName]&&(0,y.inArray)(o.levelName,t[o.hierarchyName])!==-1||!t[o.hierarchyName]||0===o.level)){var a=i.filter(function(e){return e.hierarchyName===o.hierarchyName});return a.grandTotalHash=i.grandTotalHash,a}if(o)for(var r=0;r<i.length;r++)i[r].hierarchyName===o.hierarchyName&&n.push.apply(n,N(i[r],t));return n}function j(e,t,n){var i=n.children=n.children||[],o=i.hash=i.hash||{},a=i.grandTotalHash=i.grandTotalHash||{};t.parentName&&(n=z(o,t.parentName),i=n.children=n.children||[]);var r=z(o,t.name,t,e);return t.hasValue&&!r.added&&(r.index=e,r.added=!0,i.push(r)),n.value&&n.parentName||!t.parentName?a[n.name]&&delete a[t.parentName]:a[t.parentName]=n,r}function W(e,t){var n;if(1===e.children.length&&""===e.children[0].parentName){n=e.children[0].index;var i=e.children.grandTotalHash;e.children=e.children[0].children||[],e.children.grandTotalHash=i,e.children=N(e,t)}else 0===e.children.length&&(n=0);return n}function G(e,t,n,i){var o=[];(0,v.each)(t,function(e,t){var i={children:o},a=(0,m.isDefined)(n)?Math.floor(e/n):e;(0,v.each)(t,function(e,t){i=j(a,t,i)})});var a={children:o};a.children=N(a,i);var r=W(a,i);return(0,b.foreachTree)(a.children,function(e){var t=e[0],n=N(t,i);n.length?t.children=n:delete t.children,delete t.levelName,delete t.hierarchyName,delete t.added,delete t.parentName,delete t.level},!0),(0,v.each)(a.children||[],function(t,n){e.push(n)}),r}function q(e){var t=e.getElementsByTagName("soap:Fault"),n=e.getElementsByTagName("Fault"),i=(0,c.default)([].slice.call(n.length?n:t)).find("Error");if(i.length){var o=i.attr("Description"),a=new C.Error("E4000",o);return C.log("E4000",o),a}return null}function $(e,t){var n={columns:[],rows:[]},i=t.measureCount,o=V(e,t.skipValues);return n.grandTotalColumnIndex=G(n.columns,o[0],i,t.visibleLevels),n.grandTotalRowIndex=G(n.rows,o[1],void 0,t.visibleLevels),n.values=L(e,o,i),n}function K(e,t,n,i){var o=[],a="MEASURE"===t,r=a?"MEASUREGROUP_NAME":t+"_DISPLAY_FOLDER";return(0,v.each)(e.getElementsByTagName("row"),function(e,s){var l="LEVEL"===t?B(s,"HIERARCHY_UNIQUE_NAME"):void 0,u=B(s,"LEVEL_NUMBER"),c=B(s,r);if(a&&(c=i[c]||c),("0"!==u||"true"!==B(s,t+"_IS_VISIBLE"))&&B(s,"DIMENSION_TYPE")!==fe){var d=a?he:B(s,"DIMENSION_UNIQUE_NAME"),h=B(s,t+"_UNIQUE_NAME");o.push({dimension:n.names[d]||d,groupIndex:u?P(u)-1:void 0,dataField:h,caption:B(s,t+"_CAPTION"),hierarchyName:l,groupName:l,displayFolder:c,isMeasure:a,isDefault:!!n.defaultHierarchies[h]})}}),o}function U(e){var t={};return(0,v.each)(e.getElementsByTagName("row"),function(e,n){t[B(n,"MEASUREGROUP_NAME")]=B(n,"MEASUREGROUP_CAPTION")}),t}function Y(e){var t={names:{},defaultHierarchies:{}};return(0,v.each)((0,c.default)(e).find("row"),function(){var e=(0,c.default)(this),n=e.children("DIMENSION_TYPE").text(),i=n===fe?he:e.children("DIMENSION_UNIQUE_NAME").text();t.names[i]=e.children("DIMENSION_CAPTION").text(),t.defaultHierarchies[e.children("DEFAULT_HIERARCHY").text()]=!0}),t}function Z(e){e=e.replace(/_x(....)_/g,function(e,t){return String.fromCharCode(parseInt(t,16))});var t=e.match(/\[.+?\]/gi);return t&&t.length&&(e=t[t.length-1]),e.replace(/\[/gi,"").replace(/\]/gi,"").replace(/\$/gi,"").replace(/\./gi," ")}function X(e){for(var t=e.getElementsByTagName("row"),n=[],i={},o=0;o<t.length;o++){for(var a=t[o].childNodes,r={},s=0;s<a.length;s++){var l=a[s].tagName,u=i[l]=i[l]||Z(l);r[u]=F(a[s])}n.push(r)}return n}function Q(n,i){return i=(0,c.default)("<div>").text(i).html(),e(n,(0,f.format)(te,i,n.catalog,t()))}function J(e,t,n){var i=[],a=t.columns||[],r=t.rows||[];a.length&&i.push({}),r.length&&i.push({});var s=L(n,[[{}],[{},{}]],1);if(!a.length&&r.length&&(e.rowCount=Math.max(s[0][0][0]-1,0)),!r.length&&a.length&&(e.columnCount=Math.max(s[0][0][0]-1,0)),r.length&&a.length&&(e.rowCount=Math.max(s[0][0][0]-1,0),e.columnCount=Math.max(s[1][0][0]-1,0)),void 0!==e.rowCount&&t.rowTake){e.rows=o(Array(t.rowSkip)).concat(e.rows),e.rows.length=e.rowCount;for(var l=0;l<e.rows.length;l++)e.rows[l]=e.rows[l]||{}}if(void 0!==e.columnCount&&t.columnTake){e.columns=o(Array(t.columnSkip)).concat(e.columns),e.columns.length=e.columnCount;for(var u=0;u<e.columns.length;u++)e.columns[u]=e.columns[u]||{}}}var ee='<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"><Body><Discover xmlns="urn:schemas-microsoft-com:xml-analysis"><RequestType>{2}</RequestType><Restrictions><RestrictionList><CATALOG_NAME>{0}</CATALOG_NAME><CUBE_NAME>{1}</CUBE_NAME></RestrictionList></Restrictions><Properties><PropertyList><Catalog>{0}</Catalog>{3}</PropertyList></Properties></Discover></Body></Envelope>',te='<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"><Body><Execute xmlns="urn:schemas-microsoft-com:xml-analysis"><Command><Statement>{0}</Statement></Command><Properties><PropertyList><Catalog>{1}</Catalog><ShowHiddenCubes>True</ShowHiddenCubes><SspropInitAppName>Microsoft SQL Server Management Studio</SspropInitAppName><Timeout>3600</Timeout>{2}</PropertyList></Properties></Execute></Body></Envelope>',ne="SELECT {2} FROM {0} {1} CELL PROPERTIES VALUE, FORMAT_STRING, LANGUAGE, BACK_COLOR, FORE_COLOR, FONT_FLAGS",ie="(SELECT {0} FROM {1})",oe="Subset({0}, {1}, {2})",ae="Order({0}, {1}, {2})",re="{0} {1} as {2}",se="WHERE ({0})",le="NonEmpty({0}, {1})",ue="{0} DIMENSION PROPERTIES PARENT_UNIQUE_NAME,HIERARCHY_UNIQUE_NAME, MEMBER_VALUE ON {1}",ce="CrossJoin({0})",de="{{0}}",he="DX_MEASURES",fe="2";return{ctor:function(e){this._options=e},getFields:function(){var n=this._options,i=n.catalog,o=n.cube,a=t(),r=e(n,(0,f.format)(ee,i,o,"MDSCHEMA_DIMENSIONS",a)),s=e(n,(0,f.format)(ee,i,o,"MDSCHEMA_MEASURES",a)),l=e(n,(0,f.format)(ee,i,o,"MDSCHEMA_HIERARCHIES",a)),u=e(n,(0,f.format)(ee,i,o,"MDSCHEMA_LEVELS",a)),c=new x.Deferred;return(0,x.when)(r,s,l,u).then(function(t,r,s,l){e(n,(0,f.format)(ee,i,o,"MDSCHEMA_MEASUREGROUPS",a)).done(function(e){var n=Y(t),i=K(s,"HIERARCHY",n),o=K(l,"LEVEL",n),a=U(e),u=K(r,"MEASURE",n,a).concat(i),d={};(0,v.each)(o,function(e,t){d[t.hierarchyName]=d[t.hierarchyName]||[],d[t.hierarchyName].push(t)}),(0,v.each)(i,function(e,t){d[t.dataField]&&d[t.dataField].length>1&&(t.groupName=t.hierarchyName=t.dataField,u.push.apply(u,d[t.hierarchyName]))}),c.resolve(u)}).fail(c.reject)}).fail(c.reject),c},load:function e(t){var n,i=new x.Deferred,o=this._options,a={skipValues:t.skipValues},r=O(t,o.cube,a);(t.rowSkip||t.rowTake||t.columnTake||t.columnSkip)&&(n=O((0,_.extend)({},t,{totalsOnly:!0,rowSkip:null,rowTake:null,columnSkip:null,columnTake:null}),o.cube,{}));var e=function(){r?(0,x.when)(Q(o,r),n&&Q(o,n)).done(function(e,n){var o=q(e)||n&&q(n);if(o)i.reject(o);else{var r=$(e,a);n&&J(r,t,n),i.resolve(r)}}).fail(i.reject):i.resolve({columns:[],rows:[],values:[],grandTotalColumnIndex:0,grandTotalRowIndex:0})};return t.delay?setTimeout(e,t.delay):e(),i},supportPaging:function(){return!0},getDrillDownItems:function(e,t){var n=new x.Deferred,i=this._options,o=A(e,i.cube,t);return o?(0,x.when)(Q(i,o)).done(function(e){var t=q(e);t?n.reject(t):n.resolve(X(e))}).fail(n.reject):n.resolve([]),n},key:g.noop,filter:g.noop}}()).include(b.storeDrillDownMixin);t.XmlaStore=S},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=[];if(e.items)for(var i=0;i<e.items.length;i++)n.push.apply(n,o(e.items[i],t));else(0,d.isDefined)(e.index)&&n.push(t[e.index]);return n}function a(e,t){if(e.items)for(var n=0;n<e.items.length;n++){var i=a(e.items[n],t);if(i)return i}if(t(e))return e}function r(e,t){for(var n=0,i=0;!n&&I[i];)n=I[i++](e,t);return n}function s(e){return e.find(".dx-scrollable").dxScrollable("instance")}t.default=void 0;var l=i(n(2)),u=n(59),c=n(7),d=n(1),h=n(0),f=n(12),p=n(3),g=i(n(10)),_=i(n(9)),m=n(89),v=i(n(167)),y=i(n(110)),b=i(n(306));n(304);var x="<div>",w=(0,c.hasWindow)(),C="dx-pivotgridfieldchooser",k="dx-pivotgridfieldchooser-container",S="dx-pivotgrid-fields-container",D="dx-pivotgrid-drag-action",I=[function(e,t){var n=-!!e.isMeasure,i=+!!t.isMeasure;return n+i},function(e,t){var n=-!(!e.items||!e.items.length),i=+!(!t.items||!t.items.length);return n+i},function(e,t){var n=+!!(e.isMeasure===!1&&e.field&&e.field.levels&&e.field.levels.length),i=-!!(t.isMeasure===!1&&t.field&&t.field.levels&&t.field.levels.length);return n+i},(0,m.getCompareFunction)(function(e){return e.text})],T=b.default.inherit({_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{height:400,layout:0,dataSource:null,onContextMenuPreparing:null,allowSearch:!1,searchTimeout:500,texts:{columnFields:g.default.format("dxPivotGrid-columnFields"),rowFields:g.default.format("dxPivotGrid-rowFields"),dataFields:g.default.format("dxPivotGrid-dataFields"),filterFields:g.default.format("dxPivotGrid-filterFields"),allFields:g.default.format("dxPivotGrid-allFields")}})},_refreshDataSource:function(){var e=this;e._expandedPaths=[],e._changedHandler=e._changedHandler||function(){(0,p.each)(e._dataChangedHandlers,function(e,t){t()}),e._fireContentReadyAction(),e._skipStateChange=!0,e.option("state",e._dataSource.state()),e._skipStateChange=!1},e._disposeDataSource(),e.callBase(),e._dataSource&&e._dataSource.on("changed",e._changedHandler)},_disposeDataSource:function(){var e=this,t=e._dataSource;t&&(t.off("changed",e._changedHandler),e._dataSource=void 0)},_dispose:function(){this._disposeDataSource(),this.callBase.apply(this,arguments)},_init:function(){this.callBase(),this._refreshDataSource(),this._dataChangedHandlers=[],this._initActions()},_initActions:function(){this._actions={onContextMenuPreparing:this._createActionByOption("onContextMenuPreparing")}},_trigger:function(e,t){this._actions[e](t)},_setOptionsByReference:function(){this.callBase(),(0,h.extend)(this._optionsByReference,{dataSource:!0})},_optionChanged:function(e){var t=this;switch(e.name){case"dataSource":t._refreshDataSource(),t._invalidate();break;case"layout":case"texts":case"allowSearch":case"searchTimeout":t._invalidate();break;case"onContextMenuPreparing":t._actions[e.name]=t._createActionByOption(e.name);break;default:t.callBase(e)}},_clean:function(e){!e&&this._dataSource&&this.option("state",this._dataSource.state()),this.$element().children("."+k).remove()},_renderLayout0:function(e){var t=this;e.addClass("dx-layout-0");var n=(0,l.default)(x).addClass("dx-row").appendTo(e),i=(0,l.default)(x).addClass("dx-row").appendTo(e),o=(0,l.default)(x).addClass("dx-col").appendTo(n),a=(0,l.default)(x).addClass("dx-col").appendTo(n),r=(0,l.default)(x).addClass("dx-col").appendTo(i),s=(0,l.default)(x).addClass("dx-col").appendTo(i);t._renderArea(o,"all"),t._renderArea(a,"row"),t._renderArea(a,"column"),t._renderArea(r,"filter"),t._renderArea(s,"data")},_renderLayout1:function(e){var t=this,n=(0,l.default)(x).addClass("dx-col").appendTo(e),i=(0,l.default)(x).addClass("dx-col").appendTo(e);t._renderArea(n,"all"),t._renderArea(i,"filter"),t._renderArea(i,"row"),t._renderArea(i,"column"),t._renderArea(i,"data")},_renderLayout2:function(e){var t=this;e.addClass("dx-layout-2");var n=(0,l.default)(x).addClass("dx-row").appendTo(e);t._renderArea(n,"all");var i=(0,l.default)(x).addClass("dx-row").appendTo(e),o=(0,l.default)(x).addClass("dx-col").appendTo(i),a=(0,l.default)(x).addClass("dx-col").appendTo(i);t._renderArea(o,"filter"),t._renderArea(o,"row"),t._renderArea(a,"column"),t._renderArea(a,"data")},_initMarkup:function(){var e=this,t=this.$element(),n=(0,l.default)(x).addClass(k).appendTo(t),i=e.option("layout");e.callBase(),t.addClass(C).addClass(S),e._dataChangedHandlers=[];var o=this._dataSource,a="instantly"!==e.option("applyChangesMode")&&o&&o.state();a&&e.option("state")&&o.state(e.option("state"),!0),0===i?e._renderLayout0(n):1===i?e._renderLayout1(n):e._renderLayout2(n),a&&o.state(a,!0)},_renderContentImpl:function(){this.callBase(),this.renderSortable(),this._renderContextMenu(),this.updateDimensions()},_fireContentReadyAction:function(){this._dataSource&&this._dataSource.isLoading()||this.callBase()},_getContextMenuArgs:function(e){var t,n,i=(0,l.default)(e.target).closest(".dx-area-field"),o=(0,l.default)(e.target).closest(".dx-area-fields");if(i.length){var a=i.data("field");a&&(t=this.getDataSource().field(a.index)||a)}return o.length&&(n=o.attr("group")),{event:e,field:t,area:n,items:[]}},_renderContextMenu:function(){var e=this,t=e.$element();e._contextMenu&&e._contextMenu.$element().remove(),e._contextMenu=e._createComponent((0,l.default)(x).appendTo(t),y.default,{onPositioning:function(t){var n=t.event;if(n){var i=e._getContextMenuArgs(n);e._trigger("onContextMenuPreparing",i),i.items&&i.items.length?t.component.option("items",i.items):t.cancel=!0}},target:t,onItemClick:function(e){e.itemData.onItemClick&&e.itemData.onItemClick(e)},cssClass:"dx-pivotgridfieldchooser-context-menu"})},_createTreeItems:function(e,t,n){var i,o=this,a=[],r=[],s=t[0],l={};return s?((0,p.each)(e,function(e,t){var n=t[s]||"";l[n]=l[n]||[],l[n].push(t),void 0===i&&(i=!0),i=i&&t.isMeasure===!0}),(0,p.each)(l,function(e,i){var s=n?n+"."+e:e,l=o._createTreeItems(i,t.slice(1),s);e?r.push({key:e,text:e,path:s,isMeasure:l.isMeasure,expanded:(0,f.inArray)(s,o._expandedPaths)>=0,items:l}):a=l}),a=r.concat(a),a.isMeasure=i):(0,p.each)(e,function(e,t){var n;t.isMeasure===!0&&(n="measure"),t.isMeasure===!1&&(n=t.groupName?"hierarchy":"dimension"),a.push({index:t.index,field:t,key:t.dataField,selected:(0,d.isDefined)(t.area),text:t.caption||t.dataField,icon:n,isMeasure:t.isMeasure,isDefault:t.isDefault})}),a},_createFieldsDataSource:function(e){var t=e&&e.fields()||[];t=t.filter(function(e){return e.visible!==!1&&!(0,d.isDefined)(e.groupIndex)});var n=this._createTreeItems(t,["dimension","displayFolder"]);return(0,m.foreachDataLevel)(n,function(e){e.sort(r)},0,"items"),n},_renderFieldsTreeView:function(e){var t=this,n=t._dataSource,i=t._createComponent(e,v.default,{dataSource:t._createFieldsDataSource(n),showCheckBoxesMode:"normal",expandNodesRecursive:!1,searchEnabled:t.option("allowSearch"),searchTimeout:t.option("searchTimeout"),itemTemplate:function(e,t,n){e.icon&&(0,u.getImageContainer)(e.icon).appendTo(n),(0,l.default)("<span>").toggleClass("dx-area-field",!e.items).data("field",e.field).text(e.text).appendTo(n)},onItemCollapsed:function(e){var n=(0,f.inArray)(e.itemData.path,t._expandedPaths);n>=0&&t._expandedPaths.splice(n,1)},onItemExpanded:function(e){var n=(0,f.inArray)(e.itemData.path,t._expandedPaths);n<0&&t._expandedPaths.push(e.itemData.path)},onItemSelectionChanged:function(e){var r,s,l,u=e.itemData,c=!0;if(u.items){if(u.selected)return void i.unselectItem(u);if(t._processDemandState(function(){s=o(u,n.fields());for(var e=0;e<s.length;e++)if(s[e].area){c=!1;break}}),c){var h=a(u,function(e){return e.isDefault})||a(u,function(e){return(0,d.isDefined)(e.index)});return void(h&&i.selectItem(h))}}else r=n.fields()[u.index],u.selected&&(l=r.isMeasure?"data":"column"),r&&(s=[r]);t._applyChanges(s,{area:l,areaIndex:void 0})}}),r=function(){var o=s(e),a=o?o.scrollTop():0;i.option({dataSource:t._createFieldsDataSource(n)}),o=s(e),o&&(o.scrollTo({y:a}),o.update())};t._dataChangedHandlers.push(r)},_renderAreaFields:function(e,t){var n=this,i=n._dataSource,o=i?(0,h.extend)(!0,[],i.getAreaFields(t,!0)):[];e.empty(),(0,p.each)(o,function(t,i){i.visible!==!1&&n.renderField(i,!0).appendTo(e)})},_renderArea:function(e,t){
var n,i,o=this,a=(0,l.default)(x).addClass("dx-area").appendTo(e),r=(0,l.default)(x).addClass("dx-area-fields-header").appendTo(a),s=o.option("texts."+t+"Fields");(0,l.default)("<span>").addClass("dx-area-icon").addClass("dx-area-icon-"+t).appendTo(r),(0,l.default)("<span>").html("&nbsp;").appendTo(r),(0,l.default)("<span>").addClass("dx-area-caption").text(s).appendTo(r);var u=(0,l.default)(x).addClass("dx-area-fields").addClass(D).appendTo(a);"all"!==t?(u.attr("group",t).attr("allow-scrolling",!0),n=(0,l.default)(x).addClass("dx-area-field-container").appendTo(u),i=function(){o._renderAreaFields(n,t)},o._dataChangedHandlers.push(i),i(),u.dxScrollable()):(a.addClass("dx-all-fields"),u.addClass("dx-treeview-border-visible"),o._renderFieldsTreeView(u))},_getSortableOptions:function(){return{}},_adjustSortableOnChangedArgs:function(){},resetTreeView:function(){var e=this.$element().find(".dx-treeview").dxTreeView("instance");e&&(e.option("searchValue",""),e.collapseAll())},applyChanges:function(){var e=this.option("state");(0,d.isDefined)(e)&&this._dataSource.state(e)},cancelChanges:function(){var e=this._dataSource;return!e.isLoading()&&(this.option("state",e.state()),!0)},getDataSource:function(){return this._dataSource},updateDimensions:function(){var e=this.$element().find(".dx-area .dx-scrollable");e.dxScrollable("update")},_visibilityChanged:function(e){e&&w&&this.updateDimensions()}});(0,_.default)("dxPivotGridFieldChooser",T);var O=T;t.default=O,e.exports=t.default},function(e,t,n){function i(e){for(var t=[],n=Object.keys(e),i=0,o=n;i<o.length;i++){var a=o[i];Object(l.c)(u.d,a)||t.push(a),c.b.isDate(e[a])&&!c.b.isValidDate(e[a])&&t.push(a)}if(t.length)throw new Error("Invalid options: "+t.join(", "));return Object(r.a)({},e)}function o(e){var t=Object(r.a)(Object(r.a)({},u.a),i(e));if(Object(l.f)(t.byeaster)&&(t.freq=u.c.YEARLY),!Object(l.f)(t.freq)||!u.c.FREQUENCIES[t.freq])throw new Error("Invalid frequency: "+t.freq+" "+e.freq);if(t.dtstart||(t.dtstart=new Date((new Date).setMilliseconds(0))),Object(l.f)(t.wkst)?Object(l.e)(t.wkst)||(t.wkst=t.wkst.weekday):t.wkst=u.c.MO.weekday,Object(l.f)(t.bysetpos)){Object(l.e)(t.bysetpos)&&(t.bysetpos=[t.bysetpos]);for(var n=0;n<t.bysetpos.length;n++){var o=t.bysetpos[n];if(0===o||!(o>=-366&&o<=366))throw new Error("bysetpos must be between 1 and 366, or between -366 and -1")}}if(!(Boolean(t.byweekno)||Object(l.h)(t.byweekno)||Object(l.h)(t.byyearday)||Boolean(t.bymonthday)||Object(l.h)(t.bymonthday)||Object(l.f)(t.byweekday)||Object(l.f)(t.byeaster)))switch(t.freq){case u.c.YEARLY:t.bymonth||(t.bymonth=t.dtstart.getUTCMonth()+1),t.bymonthday=t.dtstart.getUTCDate();break;case u.c.MONTHLY:t.bymonthday=t.dtstart.getUTCDate();break;case u.c.WEEKLY:t.byweekday=[c.b.getWeekday(t.dtstart)]}if(Object(l.f)(t.bymonth)&&!Object(l.d)(t.bymonth)&&(t.bymonth=[t.bymonth]),Object(l.f)(t.byyearday)&&!Object(l.d)(t.byyearday)&&Object(l.e)(t.byyearday)&&(t.byyearday=[t.byyearday]),Object(l.f)(t.bymonthday))if(Object(l.d)(t.bymonthday)){for(var a=[],s=[],n=0;n<t.bymonthday.length;n++){var o=t.bymonthday[n];o>0?a.push(o):o<0&&s.push(o)}t.bymonthday=a,t.bynmonthday=s}else t.bymonthday<0?(t.bynmonthday=[t.bymonthday],t.bymonthday=[]):(t.bynmonthday=[],t.bymonthday=[t.bymonthday]);else t.bymonthday=[],t.bynmonthday=[];if(Object(l.f)(t.byweekno)&&!Object(l.d)(t.byweekno)&&(t.byweekno=[t.byweekno]),Object(l.f)(t.byweekday))if(Object(l.e)(t.byweekday))t.byweekday=[t.byweekday],t.bynweekday=null;else if(Object(l.g)(t.byweekday))t.byweekday=[d.b.fromStr(t.byweekday).weekday],t.bynweekday=null;else if(t.byweekday instanceof d.b)!t.byweekday.n||t.freq>u.c.MONTHLY?(t.byweekday=[t.byweekday.weekday],t.bynweekday=null):(t.bynweekday=[[t.byweekday.weekday,t.byweekday.n]],t.byweekday=null);else{for(var h=[],f=[],n=0;n<t.byweekday.length;n++){var p=t.byweekday[n];Object(l.e)(p)?h.push(p):Object(l.g)(p)?h.push(d.b.fromStr(p).weekday):!p.n||t.freq>u.c.MONTHLY?h.push(p.weekday):f.push([p.weekday,p.n])}t.byweekday=Object(l.h)(h)?h:null,t.bynweekday=Object(l.h)(f)?f:null}else t.bynweekday=null;return Object(l.f)(t.byhour)?Object(l.e)(t.byhour)&&(t.byhour=[t.byhour]):t.byhour=t.freq<u.c.HOURLY?[t.dtstart.getUTCHours()]:null,Object(l.f)(t.byminute)?Object(l.e)(t.byminute)&&(t.byminute=[t.byminute]):t.byminute=t.freq<u.c.MINUTELY?[t.dtstart.getUTCMinutes()]:null,Object(l.f)(t.bysecond)?Object(l.e)(t.bysecond)&&(t.bysecond=[t.bysecond]):t.bysecond=t.freq<u.c.SECONDLY?[t.dtstart.getUTCSeconds()]:null,{parsedOptions:t}}function a(e){var t=e.dtstart.getTime()%1e3;if(!Object(s.b)(e.freq))return[];var n=[];return e.byhour.forEach(function(i){e.byminute.forEach(function(o){e.bysecond.forEach(function(e){n.push(new h.b(i,o,e,t))})})}),n}t.b=i,t.c=o,t.a=a;var r=n(130),s=n(148),l=n(74),u=n(129),c=n(73),d=n(168),h=n(308)},function(e,t,n){function i(e){var t=e.split("\n").map(a).filter(function(e){return null!==e});return Object(c.a)(Object(c.a)({},t[0]),t[1])}function o(e){var t={},n=/DTSTART(?:;TZID=([^:=]+?))?(?::|=)([^;\s]+)/i.exec(e);if(!n)return t;var i=(n[0],n[1]),o=n[2];return i&&(t.tzid=i),t.dtstart=f.b.untilStringToDate(o),t}function a(e){if(e=e.replace(/^\s+|\s+$/,""),!e.length)return null;var t=/^([A-Z]+?)[:;]/.exec(e.toUpperCase());if(!t)return r(e);var n=(t[0],t[1]);switch(n.toUpperCase()){case"RRULE":case"EXRULE":return r(e);case"DTSTART":return o(e);default:throw new Error("Unsupported RFC prop "+n+" in "+e)}}function r(e){var t=e.replace(/^RRULE:/i,""),n=o(t),i=e.replace(/^(?:RRULE|EXRULE):/i,"").split(";");return i.forEach(function(t){var i=t.split("="),a=i[0],r=i[1];switch(a.toUpperCase()){case"FREQ":n.freq=d.a[r.toUpperCase()];break;case"WKST":n.wkst=p.b[r.toUpperCase()];break;case"COUNT":case"INTERVAL":case"BYSETPOS":case"BYMONTH":case"BYMONTHDAY":case"BYYEARDAY":case"BYWEEKNO":case"BYHOUR":case"BYMINUTE":case"BYSECOND":var l=s(r),c=a.toLowerCase();n[c]=l;break;case"BYWEEKDAY":case"BYDAY":n.byweekday=u(r);break;case"DTSTART":case"TZID":var h=o(e);n.tzid=h.tzid,n.dtstart=h.dtstart;break;case"UNTIL":n.until=f.b.untilStringToDate(r);break;case"BYEASTER":n.byeaster=Number(r);break;default:throw new Error("Unknown RRULE property '"+a+"'")}}),n}function s(e){if(e.indexOf(",")!==-1){var t=e.split(",");return t.map(l)}return l(e)}function l(e){return/^[+-]?\d+$/.test(e)?Number(e):e}function u(e){var t=e.split(",");return t.map(function(e){if(2===e.length)return p.b[e];var t=e.match(/^([+-]?\d{1,2})([A-Z]{2})$/),n=Number(t[1]),i=t[2],o=p.b[i].weekday;return new h.b(o,n)})}t.b=i,t.a=o;var c=n(130),d=n(148),h=n(168),f=n(73),p=n(129)},function(e,t,n){function i(e){for(var t=[],n="",i=Object.keys(e),u=Object.keys(a.a),c=0;c<i.length;c++)if("tzid"!==i[c]&&Object(r.c)(u,i[c])){var d=i[c].toUpperCase(),h=e[i[c]],f="";if(Object(r.f)(h)&&(!Object(r.d)(h)||h.length)){switch(d){case"FREQ":f=a.c.FREQUENCIES[e.freq];break;case"WKST":f=Object(r.e)(h)?new s.b(h).toString():h.toString();break;case"BYWEEKDAY":d="BYDAY",f=Object(r.n)(h).map(function(e){return e instanceof s.b?e:Object(r.d)(e)?new s.b(e[0],e[1]):new s.b(e)}).toString();break;case"DTSTART":n=o(h,e.tzid);break;case"UNTIL":f=l.b.timeToUntilString(h,!e.tzid);break;default:if(Object(r.d)(h)){for(var p=[],g=0;g<h.length;g++)p[g]=String(h[g]);f=p.toString()}else f=String(h)}f&&t.push([d,f])}}var _=t.map(function(e){var t=e[0],n=e[1];return t+"="+n.toString()}).join(";"),m="";return""!==_&&(m="RRULE:"+_),[n,m].filter(function(e){return!!e}).join("\n")}function o(e,t){return e?"DTSTART"+new u.a(new Date(e),t).toString():""}t.a=i;var a=n(129),r=n(74),s=n(168),l=n(73),u=n(309)},function(e,t,n){function i(e,t){var n=t.dtstart,i=t.freq,o=t.interval,h=t.until,f=t.bysetpos,g=t.count;if(0===g||0===o)return r(e);var v=m.a.fromDate(n),y=new d.a(t);y.rebuild(v.year,v.month);for(var b=l(y,v,t);;){var x=y.getdayset(i)(v.year,v.month,v.day),w=x[0],C=x[1],k=x[2],S=s(w,C,k,y,t);if(Object(p.h)(f))for(var D=Object(_.a)(f,b,C,k,y,w),I=0;I<D.length;I++){var T=D[I];if(h&&T>h)return r(e);if(T>=n){var O=a(T,t);if(!e.accept(O))return r(e);if(g&&(--g,!g))return r(e)}}else for(var I=C;I<k;I++){var E=w[I];if(Object(p.f)(E))for(var A=c.b.fromOrdinal(y.yearordinal+E),P=0;P<b.length;P++){var R=b[P],T=c.b.combine(A,R);if(h&&T>h)return r(e);if(T>=n){var O=a(T,t);if(!e.accept(O))return r(e);if(g&&(--g,!g))return r(e)}}}if(0===t.interval)return r(e);if(v.add(t,S),v.year>c.b.MAXYEAR)return r(e);Object(u.b)(i)||(b=y.gettimeset(i)(v.hour,v.minute,v.second,0)),y.rebuild(v.year,v.month)}}function o(e,t,n){var i=n.bymonth,o=n.byweekno,a=n.byweekday,r=n.byeaster,s=n.bymonthday,l=n.bynmonthday,u=n.byyearday;return Object(p.h)(i)&&!Object(p.c)(i,e.mmask[t])||Object(p.h)(o)&&!e.wnomask[t]||Object(p.h)(a)&&!Object(p.c)(a,e.wdaymask[t])||Object(p.h)(e.nwdaymask)&&!e.nwdaymask[t]||null!==r&&!Object(p.c)(e.eastermask,t)||(Object(p.h)(s)||Object(p.h)(l))&&!Object(p.c)(s,e.mdaymask[t])&&!Object(p.c)(l,e.nmdaymask[t])||Object(p.h)(u)&&(t<e.yearlen&&!Object(p.c)(u,t+1)&&!Object(p.c)(u,-e.yearlen+t)||t>=e.yearlen&&!Object(p.c)(u,t+1-e.yearlen)&&!Object(p.c)(u,-e.nextyearlen+t-e.yearlen))}function a(e,t){return new g.a(e,t.tzid).rezonedDate()}function r(e){return e.getValue()}function s(e,t,n,i,a){for(var r=!1,s=t;s<n;s++){var l=e[s];r=o(i,l,a),r&&(e[l]=null)}return r}function l(e,t,n){var i=n.freq,o=n.byhour,a=n.byminute,r=n.bysecond;return Object(u.b)(i)?Object(f.a)(n):i>=h.c.HOURLY&&Object(p.h)(o)&&!Object(p.c)(o,t.hour)||i>=h.c.MINUTELY&&Object(p.h)(a)&&!Object(p.c)(a,t.minute)||i>=h.c.SECONDLY&&Object(p.h)(r)&&!Object(p.c)(r,t.second)?[]:e.gettimeset(i)(t.hour,t.minute,t.second,t.millisecond)}t.a=i;var u=n(148),c=n(73),d=n(793),h=n(129),f=n(510),p=n(74),g=n(309),_=n(798),m=n(308)},function(e,t,n){function i(e){var t=this;return function(n){if(void 0!==n&&(t["_"+e]=n),void 0!==t["_"+e])return t["_"+e];for(var i=0;i<t._rrule.length;i++){var o=t._rrule[i].origOptions[e];if(o)return o}}}function o(e,t){if(!(e instanceof l.c))throw new TypeError(String(e)+" is not RRule instance");Object(c.c)(t.map(String),String(e))||t.push(e)}function a(e,t){if(!(e instanceof Date))throw new TypeError(String(e)+" is not Date instance");Object(c.c)(t.map(Number),Number(e))||(t.push(e),u.b.sort(t))}function r(e,t,n){var i=!n||"UTC"===n.toUpperCase(),o=i?e+":":e+";TZID="+n+":",a=t.map(function(e){return u.b.timeToUntilString(e.valueOf(),i)}).join(",");return""+o+a}var s=n(130),l=n(129),u=n(73),c=n(74),d=n(802),h=n(515),f=n(512),p=function(e){function t(t){void 0===t&&(t=!1);var n=e.call(this,{},t)||this;return n.dtstart=i.apply(n,["dtstart"]),n.tzid=i.apply(n,["tzid"]),n._rrule=[],n._rdate=[],n._exrule=[],n._exdate=[],n}return Object(s.b)(t,e),t.prototype._iter=function(e){return Object(d.a)(e,this._rrule,this._exrule,this._rdate,this._exdate,this.tzid())},t.prototype.rrule=function(e){o(e,this._rrule)},t.prototype.exrule=function(e){o(e,this._exrule)},t.prototype.rdate=function(e){a(e,this._rdate)},t.prototype.exdate=function(e){a(e,this._exdate)},t.prototype.rrules=function(){return this._rrule.map(function(e){return Object(h.a)(e.toString())})},t.prototype.exrules=function(){return this._exrule.map(function(e){return Object(h.a)(e.toString())})},t.prototype.rdates=function(){return this._rdate.map(function(e){return new Date(e.getTime())})},t.prototype.exdates=function(){return this._exdate.map(function(e){return new Date(e.getTime())})},t.prototype.valueOf=function(){var e=[];return!this._rrule.length&&this._dtstart&&(e=e.concat(Object(f.a)({dtstart:this._dtstart}))),this._rrule.forEach(function(t){e=e.concat(t.toString().split("\n"))}),this._exrule.forEach(function(t){e=e.concat(t.toString().split("\n").map(function(e){return e.replace(/^RRULE:/,"EXRULE:")}).filter(function(e){return!/^DTSTART/.test(e)}))}),this._rdate.length&&e.push(r("RDATE",this._rdate,this.tzid())),this._exdate.length&&e.push(r("EXDATE",this._exdate,this.tzid())),e},t.prototype.toString=function(){return this.valueOf().join("\n")},t.prototype.clone=function(){var e=new t((!!this._cache));return this._rrule.forEach(function(t){return e.rrule(t.clone())}),this._exrule.forEach(function(t){return e.exrule(t.clone())}),this._rdate.forEach(function(t){return e.rdate(new Date(t.getTime()))}),this._exdate.forEach(function(t){return e.exdate(new Date(t.getTime()))}),e},t}(l.c);t.a=p},function(e,t,n){function i(e,t){var n=[],i=[],o=[],a=[],r=Object(v.a)(e),s=r.dtstart,l=r.tzid,d=c(e,t.unfold);return d.forEach(function(e){if(e){var t=u(e),r=t.name,s=t.parms,c=t.value;switch(r.toUpperCase()){case"RRULE":if(s.length)throw new Error("unsupported RRULE parm: "+s.join(","));n.push(Object(v.b)(e));break;case"RDATE":var d=/RDATE(?:;TZID=([^:=]+))?/i.exec(e),f=(d[0],d[1]);f&&!l&&(l=f),i=i.concat(h(c,s));break;case"EXRULE":if(s.length)throw new Error("unsupported EXRULE parm: "+s.join(","));o.push(Object(v.b)(c));break;case"EXDATE":a=a.concat(h(c,s));break;case"DTSTART":break;default:throw new Error("unsupported property: "+r)}}}),{dtstart:s,tzid:l,rrulevals:n,rdatevals:i,exrulevals:o,exdatevals:a}}function o(e,t){var n=i(e,t),o=n.rrulevals,a=n.rdatevals,s=n.exrulevals,l=n.exdatevals,u=n.dtstart,c=n.tzid,d=t.cache===!1;if(t.compatible&&(t.forceset=!0,t.unfold=!0),t.forceset||o.length>1||a.length||s.length||l.length){var h=new g.a(d);return h.dtstart(u),h.tzid(c||void 0),o.forEach(function(e){h.rrule(new p.c(r(e,u,c),d))}),a.forEach(function(e){h.rdate(e)}),s.forEach(function(e){h.exrule(new p.c(r(e,u,c),d))}),l.forEach(function(e){h.exdate(e)}),t.compatible&&t.dtstart&&h.rdate(u),h}var f=o[0]||{};return new p.c(r(f,f.dtstart||t.dtstart||u,f.tzid||t.tzid||c),d)}function a(e,t){return void 0===t&&(t={}),o(e,s(t))}function r(e,t,n){return Object(f.a)(Object(f.a)({},e),{dtstart:t,tzid:n})}function s(e){var t=[],n=Object.keys(e),i=Object.keys(y);if(n.forEach(function(e){Object(m.c)(i,e)||t.push(e)}),t.length)throw new Error("Invalid options: "+t.join(", "));return Object(f.a)(Object(f.a)({},y),e)}function l(e){if(e.indexOf(":")===-1)return{name:"RRULE",value:e};var t=Object(m.m)(e,":",1),n=t[0],i=t[1];return{name:n,value:i}}function u(e){var t=l(e),n=t.name,i=t.value,o=n.split(";");if(!o)throw new Error("empty property name");return{name:o[0].toUpperCase(),parms:o.slice(1),value:i}}function c(e,t){if(void 0===t&&(t=!1),e=e&&e.trim(),!e)throw new Error("Invalid empty string");if(!t)return e.split(/\s/);for(var n=e.split("\n"),i=0;i<n.length;){var o=n[i]=n[i].replace(/\s+$/g,"");o?i>0&&" "===o[0]?(n[i-1]+=o.slice(1),n.splice(i,1)):i+=1:n.splice(i,1)}return n}function d(e){e.forEach(function(e){if(!/(VALUE=DATE(-TIME)?)|(TZID=)/.test(e))throw new Error("unsupported RDATE/EXDATE parm: "+e)})}function h(e,t){return d(t),e.split(",").map(function(e){return _.b.untilStringToDate(e)})}t.a=a;var f=n(130),p=n(129),g=n(514),_=n(73),m=n(74),v=n(511),y={dtstart:null,cache:!1,unfold:!1,forceset:!1,compatible:!1,tzid:null}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return h(),c=d(e),c.show()}function a(){return c?c.hide().done(h).promise():(new u.Deferred).resolve()}t.show=o,t.hide=a;var r=i(n(2)),s=n(78),l=i(n(193)),u=n(6),c=null,d=function(e){return new l.default((0,r.default)("<div>").appendTo(e&&e.container||(0,s.value)()),e)},h=function(){c&&(c.$element().remove(),c=null)}},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}t.AppointmentTooltipInfo=void 0;var o=function e(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:void 0,o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[],a=arguments.length>3&&void 0!==arguments[3]?arguments[3]:[];i(this,e),this.appointment=t,this.targetedAppointment=n,this.color=o,this.settings=a};t.AppointmentTooltipInfo=o},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}t.TooltipStrategyBase=void 0;var l=i(n(38)),u=n(118),c=i(n(2)),d=i(n(425)),h="dx-tooltip-appointment-item",f=h+"-content",p=h+"-content-subject",g=h+"-content-date",_=h+"-marker",m=h+"-marker-body",v=h+"-delete-button-container",y=h+"-delete-button",b=function(){function e(t){a(this,e),this._tooltip=null,this._options=t,this._extraOptions=null}return s(e,[{key:"show",value:function(e,t,n){this._canShowTooltip(t)&&(this.hide(),this._extraOptions=n,this._showCore(e,t))}},{key:"_showCore",value:function(e,t){this._tooltip?(this._shouldUseTarget()&&this._tooltip.option("target",e),this._list.option("dataSource",t)):this._tooltip=this._createTooltip(e,t),this._prepareBeforeVisibleChanged(t),this._tooltip.option("visible",!0)}},{key:"_prepareBeforeVisibleChanged",value:function(e){}},{key:"_getContentTemplate",value:function(e){var t=this;return function(n){var i=(0,c.default)("<div>");(0,c.default)(n).append(i),t._list=t._createList(i,e)}}},{key:"isAlreadyShown",value:function(e){if(this._tooltip&&this._tooltip.option("visible"))return this._tooltip.option("target")[0]===e[0]}},{key:"_onShown",value:function(){this._list.option("focusStateEnabled",this._extraOptions.focusStateEnabled)}},{key:"dispose",value:function(){}},{key:"hide",value:function(){this._tooltip&&this._tooltip.option("visible",!1)}},{key:"_shouldUseTarget",value:function(){return!0}},{key:"_createTooltip",value:function(){}},{key:"_canShowTooltip",value:function(e){return!!e.length}},{key:"_createListOption",value:function(e){var t=this;return{dataSource:e,onContentReady:this._onListRender.bind(this),onItemClick:function(e){return t._onListItemClick(e)},itemTemplate:function(e,n){return t._renderTemplate(e.appointment,e.targetedAppointment,n,e.color)},_swipeEnabled:!1}}},{key:"_onListRender",value:function(){}},{key:"_createTooltipElement",value:function(e){return(0,c.default)("<div>").appendTo(this._options.container).addClass(e)}},{key:"_createList",value:function(e,t){return this._options.createComponent(e,d.default,this._createListOption(t))}},{key:"_renderTemplate",value:function(e,t,n,i){var a=this._createItemListContent(e,t,i);this._options.addDefaultTemplates(o({},this._getItemListTemplateName(),new u.FunctionTemplate(function(e){var t=(0,c.default)(e.container);return t.append(a),t})));var r=this._options.getAppointmentTemplate(this._getItemListTemplateName()+"Template");return this._createFunctionTemplate(r,e,t,n)}},{key:"_createFunctionTemplate",value:function(e,t,n,i){var o=this._isEmptyDropDownAppointmentTemplate();return new u.FunctionTemplate(function(a){return e.render({model:o?{appointmentData:t,targetedAppointmentData:n}:t,container:a.container,index:i})})}},{key:"_getItemListTemplateName",value:function(){return this._isEmptyDropDownAppointmentTemplate()?"appointmentTooltip":"dropDownAppointment"}},{key:"_isEmptyDropDownAppointmentTemplate",value:function(){return!this._extraOptions.dropDownAppointmentTemplate||"dropDownAppointment"===this._extraOptions.dropDownAppointmentTemplate}},{key:"_onListItemClick",value:function(e){this.hide(),this._extraOptions.clickEvent&&this._extraOptions.clickEvent(e),this._options.showAppointmentPopup(e.itemData.appointment,!1,e.itemData.targetedAppointment)}},{key:"_createItemListContent",value:function(e,t,n){var i=this._extraOptions.editing,o=(0,c.default)("<div>").addClass(h);o.append(this._createItemListMarker(n)),o.append(this._createItemListInfo(this._options.createFormattedDateText(e,t)));var a=this._options.getAppointmentDisabled(e);return!a&&(i&&i.allowDeleting===!0||i===!0)&&o.append(this._createDeleteButton(e,t)),o}},{key:"_createItemListMarker",value:function(e){var t=(0,c.default)("<div>").addClass(_),n=(0,c.default)("<div>").addClass(m);return t.append(n),e&&e.done(function(e){return n.css("background",e)}),t}},{key:"_createItemListInfo",value:function(e){var t=(0,c.default)("<div>").addClass(f),n=(0,c.default)("<div>").addClass(p).text(e.text),i=(0,c.default)("<div>").addClass(g).text(e.formatDate);return t.append(n).append(i)}},{key:"_createDeleteButton",value:function(e,t){var n=this,i=(0,c.default)("<div>").addClass(v),o=(0,c.default)("<div>").addClass(y);return i.append(o),this._options.createComponent(o,l.default,{icon:"trash",stylingMode:"text",onClick:function(i){n.hide(),i.event.stopPropagation(),n._options.checkAndDeleteAppointment(e,t)}}),i}}]),e}();t.TooltipStrategyBase=b},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=n(1),s=24,l=3,u=22,c=1,d=40,h=75,f=50,p=function(){function e(t){i(this,e),this._renderingStrategy=t}return a(e,[{key:"getRenderingStrategy",value:function(){return this._renderingStrategy}},{key:"getDropDownAppointmentWidth",value:function(e,t){return t||!(0,r.isDefined)(t)?h*this.getRenderingStrategy().getDefaultCellWidth()/100:s}},{key:"getCollectorTopOffset",value:function(){return l}},{key:"getCollectorLeftOffset",value:function(){return l}},{key:"getAppointmentDefaultOffset",value:function(){return this.getRenderingStrategy()._isCompactTheme()?u:this.getRenderingStrategy().instance.option("_appointmentOffset")}},{key:"getDynamicAppointmentCountPerCell",value:function(){var e=this.getRenderingStrategy(),t=e.instance.fire("getCellHeight"),n=Math.floor((t-e._getAppointmentDefaultOffset())/e._getAppointmentDefaultHeight())||this._getAppointmentMinCount();return e.hasAllDayAppointments()?{allDay:"vertical"===e.instance._groupOrientation?n:e.instance.option("_appointmentCountPerCell"),simple:this._calculateDynamicAppointmentCountPerCell()||this._getAppointmentMinCount()}:n}},{key:"getDropDownAppointmentHeight",value:function(){}},{key:"_getAppointmentMinCount",value:function(){return c}},{key:"_calculateDynamicAppointmentCountPerCell",value:function(){return Math.floor(this.getRenderingStrategy()._getAppointmentMaxWidth()/f)}},{key:"_getAppointmentDefaultWidth",value:function(){return d}}]),e}(),g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(311)),v=o(n(20)),y=60,b=35,x=2,w=v.default.dateToMilliseconds,C=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_needVerifyItemSize",value:function(){return!0}},{key:"calculateAppointmentWidth",value:function(e,t){var n=this.getDefaultCellWidth()||this.getAppointmentMinSize(),i=this.instance.fire("getField","allDay",e),o=t.info.appointment.startDate,a=this.normalizeEndDateByViewEnd(e,t.info.appointment.endDate),r=this._getAppointmentDurationInMs(o,a,i);r=this._adjustDurationByDaylightDiff(r,o,a);var s=this.instance.getAppointmentDurationInMinutes()*w("minute"),l=r/s,u=this.cropAppointmentWidth(l*n,n);return u}},{key:"_needAdjustDuration",value:function(e){return e<0}},{key:"getAppointmentGeometry",value:function(e){var n=this._customizeAppointmentGeometry(e);return l(_(t.prototype),"getAppointmentGeometry",this).call(this,n)}},{key:"_customizeAppointmentGeometry",value:function(e){var t=this._calculateGeometryConfig(e);return this._customizeCoordinates(e,t.height,t.appointmentCountPerCell,t.offset)}},{key:"_getOffsets",value:function(){return{unlimited:0,auto:0}}},{key:"_getCompactLeftCoordinate",value:function(e,t){var n=this.getDefaultCellWidth()||this.getAppointmentMinSize();return e+n*t}},{key:"_getMaxHeight",value:function(){return this.getDefaultCellHeight()||this.getAppointmentMinSize()}},{key:"_getAppointmentCount",value:function(e,t){return this._getMaxAppointmentCountPerCellByType(!1)}},{key:"_getAppointmentDefaultHeight",value:function(){return y}},{key:"_getAppointmentMinHeight",value:function(){return b}},{key:"_sortCondition",value:function(e,t){return this._columnCondition(e,t)}},{key:"_getOrientation",value:function(){return["left","right","top"]}},{key:"_getMaxAppointmentWidth",value:function(e){return this.instance.fire("getMaxAppointmentWidth",{date:e})}},{key:"getDropDownAppointmentWidth",value:function(){return this.getDefaultCellWidth()-2*x}},{key:"getDeltaTime",value:function(e,t){var n=0,i=e.width-t.width;return n=w("minute")*Math.round(i/this.getDefaultCellWidth()*this.instance.getAppointmentDurationInMinutes())}},{key:"isAllDay",value:function(e){return this.instance.fire("getField","allDay",e)}},{key:"needSeparateAppointment",value:function(){return this.instance.fire("isGroupedByDate")}}]),t}(m.default),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(520)),v=o(n(20)),y=o(n(47)),b=24,x=60,w=6e4,C=1,k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"calculateAppointmentWidth",value:function(e,t){var n=v.default.trimTime(t.info.appointment.startDate),i=this.normalizeEndDateByViewEnd(e,t.info.appointment.endDate),o=this.getDefaultCellWidth()||this.getAppointmentMinSize(),a=this._getDurationInDays(n,i),r=this.cropAppointmentWidth(Math.ceil(a)*o,o);return r}},{key:"_getDurationInDays",value:function(e,t){var n=this._adjustDurationByDaylightDiff(t.getTime()-e.getTime(),e,t);return n/v.default.dateToMilliseconds("day")||C}},{key:"getDeltaTime",value:function(e,t){return b*x*w*this._getDeltaWidth(e,t)}},{key:"isAllDay",value:function(){return!1}},{key:"createTaskPositionMap",value:function(e,n){return n||this.instance.getAppointmentsInstance()._sortAppointmentsByStartDate(e),l(_(t.prototype),"createTaskPositionMap",this).call(this,e)}},{key:"_getSortedPositions",value:function(e,n){var i=l(_(t.prototype),"_getSortedPositions",this).call(this,e);return n||(i=(0,y.default)(i).sortBy("top").thenBy("left").thenBy("cellPosition").thenBy("i").toArray()),i}},{key:"needCorrectAppointmentDates",value:function(){return!1}}]),t}(m.default),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a={SETTINGS:"dxAppointmentSettings"},r={dataAccessors:{getAppointmentSettings:function(e){return(0,o.default)(e).data(a.SETTINGS)},getAppointmentInfo:function(e){var t=r.dataAccessors.getAppointmentSettings(e);return null===t||void 0===t?void 0:t.info}}},s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(null==e)return{};var n,i,o=r(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function r(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function s(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function l(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?s(Object(n),!0).forEach(function(t){u(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):s(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function u(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function c(e,t,n){return(c="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=d(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);
return o.get?o.get.call(n):o.value}})(e,t,n||e)}function d(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=v(e),null!==e););return e}function h(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&f(e,t)}function f(e,t){return(f=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function p(e){var t=m();return function(){var n,i=v(e);if(t){var o=v(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return g(this,n)}}function g(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?_(e):t}function _(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function m(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function v(e){return(v=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function y(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function b(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function x(e,t,n){return t&&b(e.prototype,t),n&&b(e,n),e}t.default=void 0;var w=o(n(2)),C=o(n(11)),k=o(n(5)),S=n(39),D=o(n(20)),I=n(7),T=n(18),O=n(0),E=n(3),A=n(24),P=o(n(10)),R=o(n(32)),M=n(4),B=n(1),V=n(8),F=o(n(25)),L=o(n(17)),H=n(21),z=n(178),N=n(80),j=o(n(109)),W=o(n(820)),G=o(n(821)),q=o(n(312)),$=o(n(822)),K=o(n(823)),U=n(169),Y=o(n(86)),Z=o(n(824)),X=n(30),Q=o(n(825)),J=o(n(826)),ee=o(n(827)),te=o(n(835)),ne=o(n(836)),ie=o(n(839)),oe=n(525),ae=q.default.tableCreator,re=Z.default.abstract,se=D.default.dateToMilliseconds,le="dx-scheduler-work-space",ue="dx-scheduler-work-space-grouped",ce="dx-scheduler-work-space-vertical-grouped",de="dx-scheduler-work-space-vertical-group-table",he="dx-scheduler-work-space-both-scrollbar",fe="dx-scheduler-work-space-count",pe="dx-scheduler-work-space-group-by-date",ge="dx-scheduler-work-space-odd-cells",_e="dx-scheduler-time-panel",me="dx-scheduler-time-panel-cell",ve="dx-scheduler-time-panel-row",ye="dx-scheduler-all-day-panel",be="dx-scheduler-all-day-table",xe="dx-scheduler-all-day-appointments",we="dx-scheduler-all-day-title",Ce="dx-scheduler-all-day-title-hidden",ke="dx-scheduler-all-day-table-cell",Se="dx-scheduler-all-day-table-row",De="dx-scheduler-work-space-all-day",Ie="dx-scheduler-work-space-all-day-collapsed",Te="dx-scheduler-work-space-mouse-selection",Oe="dx-scheduler-cell-sizes-horizontal",Ee="dx-scheduler-cell-sizes-vertical",Ae="dx-scheduler-header-panel",Pe="dx-scheduler-header-panel-cell",Re="dx-scheduler-header-row",Me="dx-scheduler-group-row",Be="dx-scheduler-group-header",Ve="dx-scheduler-group-header-content",Fe="dx-scheduler-date-table",Le="dx-scheduler-date-table-cell",He="dx-scheduler-date-table-row",ze="dx-scheduler-focused-cell",Ne="dx-scheduler-virtual-row",je="dx-scheduler-virtual-cell",We="dx-scheduler-date-table-droppable-cell",Ge="dx-scheduler-header-scrollable",qe="dx-scheduler-sidebar-scrollable",$e="dx-scheduler-date-table-scrollable",Ke=(0,V.addNamespace)(F.default.down,"dxSchedulerWorkSpace"),Ue=(0,V.addNamespace)(N.enter,"dxSchedulerDateTable"),Ye=(0,V.addNamespace)(N.drop,"dxSchedulerDateTable"),Ze=(0,V.addNamespace)(N.leave,"dxSchedulerDateTable"),Xe=(0,V.addNamespace)(H.name,"dxSchedulerDateTable"),Qe=(0,V.addNamespace)(F.default.down,"dxSchedulerDateTable"),Je=(0,V.addNamespace)(F.default.up,"dxSchedulerDateTable"),et=(0,V.addNamespace)(F.default.move,"dxSchedulerDateTable"),tt="dxCellData",nt=1,it=75,ot=se("day"),at=se("hour"),rt=".".concat(Fe," td, .").concat(be," td"),st=".".concat(Le,", .").concat(ke),lt=function(){function e(){y(this,e),this.counter=0}return x(e,[{key:"isFree",value:function(){return 0===this.counter}},{key:"take",value:function(){this.counter++}},{key:"release",value:function(){this.counter--,this.counter<0&&(this.counter=0)}}]),e}(),ut=function(e){return R.default.getDayNames("abbreviated")[e.getDay()]},ct=function(e){function t(){return y(this,t),n.apply(this,arguments)}h(t,e);var n=p(t);return x(t,[{key:"_supportedKeys",value:function(){var e=function(e){if(e.preventDefault(),e.stopPropagation(),this._selectedCells&&this._selectedCells.length){var t=(0,w.default)(this.option("focusedElement")),n=(0,w.default)(t.length?t:this._selectedCells);e.target=this._selectedCells,this._showPopup=!0,this._cellClickAction({event:e,cellElement:(0,w.default)(this._selectedCells),cellData:this.getCellData(n)})}},n=function(e,t){e.preventDefault(),e.stopPropagation(),this._moveToCell(t,e.shiftKey)};return(0,O.extend)(c(v(t.prototype),"_supportedKeys",this).call(this),{enter:e,space:e,downArrow:function(e){var t=this._getCellFromNextRow("next",e.shiftKey);n.call(this,e,t)},upArrow:function(e){var t=this._getCellFromNextRow("prev",e.shiftKey);n.call(this,e,t)},rightArrow:function(e){var t=this._getCellFromNextColumn("next",e.shiftKey);n.call(this,e,t)},leftArrow:function(e){var t=this._getCellFromNextColumn("prev",e.shiftKey);n.call(this,e,t)}})}},{key:"_dispose",value:function(){var e;c(v(t.prototype),"_dispose",this).call(this),null===(e=this.virtualScrollingDispatcher)||void 0===e?void 0:e.dispose(),this._disposeRenovatedComponents()}},{key:"_isRTL",value:function(){return this.option("rtlEnabled")}},{key:"_getFocusedCell",value:function(){return this._$focusedCell||this._$dateTable.find("."+Le).eq(0)}},{key:"_getAllFocusedCells",value:function(){return this._selectedCells||this._$dateTable.find("."+Le).eq(0)}},{key:"_getCellFromNextRow",value:function(e){var t=this._$focusedCell;if((0,B.isDefined)(t)){var n=t.index(),i=t.parent(),o=i[e]().children().eq(n);return o=this._checkForViewBounds(o)}}},{key:"_checkForViewBounds",value:function(e){return e.length||(e=this._$focusedCell),e}},{key:"_getCellFromNextColumn",value:function(e,t){var n=this._$focusedCell;if((0,B.isDefined)(n)){var i,o=n.parent(),a=e,r="next"===e,s=r?"prev":"next",l=this._isRTL(),u=this._getGroupCount(),c=this._isHorizontalGroupedWorkSpace(),d=this.isGroupedByDate(),h=this._getTotalCellCount(u),f=t&&!d?this._getCellCount():h,p=f-1,g=n.index(),_=d&&t?u:1,m=this._isEdgeCell(c?h-1:p,g,_,e),v=l?1:-1,y=r?1:-1,b=v*y;if(m||t&&this._isGroupEndCell(n,e)){var x=g-b*_+b*f,w=l?s:a;i=o[w]().children().eq(x),i=this._checkForViewBounds(i)}else i=o.children().eq(g-b*_);return i}}},{key:"_isEdgeCell",value:function(e,t,n,i){var o=this._isRTL(),a="next"===i,r=o?0:e,s=o?e:0,l=a?r:s,u=t+n>l,c=t-n<l,d=o?c:u,h=o?u:c;return a?d:h}},{key:"_isGroupEndCell",value:function(e,t){if(this.isGroupedByDate())return!1;var n="next"===t,i=this._getCellCount(),o=e.index(),a=o%i,r=n?i-1:0,s=n?0:i-1;return this._isRTL()?a===s:a===r}},{key:"_moveToCell",value:function(e,t){t=t&&this.option("allowMultipleCellSelection"),this._setSelectedAndFocusedCells(e,t),this._dateTableScrollable.scrollToElement(e)}},{key:"_setSelectedAndFocusedCells",value:function(e,t){if((0,B.isDefined)(e)&&e.length){var n=this.isVirtualScrolling(),i=e;t&&(i=this._correctCellForGroup(e)),i.hasClass(ze)||(this._setSelectedCells(i,t),this._setFocusedCell(i,n))}}},{key:"_setFocusedCell",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1];this._releaseFocusedCell();var n=e;if(t){var i=this._getCoordinatesByCell(e),o=i.rowIndex,a=i.columnIndex,r=this._hasAllDayClass(e);this.virtualSelectionState.setFocusedCell(o,a,r);var s=this.virtualSelectionState.getFocusedCell(this._isVerticalGroupedWorkSpace()),l=s.cellData,u=s.coordinates,c=l.allDay;n=c&&!this._isVerticalGroupedWorkSpace()?this._dom_getAllDayPanelCell(u.cellIndex):this._dom_getDateCell(u)}this._toggleFocusedCellClass(!0,n),this._$focusedCell=n}},{key:"_setSelectedCells",value:function(e,t){this._releaseSelectedCells(),this._selectedCells=[],this.isVirtualScrolling()?this._setSelectedCellsInVirtualMode(e,t):this._setSelectedCellsInStandardMode(e,t);var n=(0,w.default)(this._selectedCells);this._toggleFocusClass(!0,n),this.setAria("label","Add appointment",n);var i=this.getSelectedCellData().map(function(e){var t=e.startDate,n=e.endDate,i=e.allDay,o=e.groups,a=e.groupIndex;return{startDate:t,endDate:n,allDay:i,groups:o,groupIndex:a||0}});this.option("selectedCellData",i),this._selectionChangedAction({selectedCellData:i})}},{key:"_setSelectedCellsInStandardMode",value:function(e,t){if(t){var n=this._$prevCell,i="day"!==this.option("type")||this.option("groups").length&&"vertical"!==this.option("groupOrientation")?"horizontal":"vertical",o=this._getCellsBetween(e,n,i);this._selectedCells=o.toArray()}else this._selectedCells=[e.get(0)],this._$prevCell=e}},{key:"_setSelectedCellsInVirtualMode",value:function(e,t){if(t){var n=this._getCoordinatesByCell(e),i=n.rowIndex,o=n.columnIndex,a=this._hasAllDayClass(e),r={rowIndex:i,columnIndex:o,allDay:a};this.virtualSelectionState.setSelectedCells(r)}else{this._selectedCells=[e.get(0)],this._$prevCell=e;var s=this._getCoordinatesByCell(e),l=s.rowIndex,u=s.columnIndex,c=this._hasAllDayClass(e),d={rowIndex:l,columnIndex:u,allDay:c};this.virtualSelectionState.setSelectedCells(d,d)}this._setSelectedCellsByCellData(this.virtualSelectionState.getSelectedCells())}},{key:"_correctCellForGroup",value:function(e){if(this.isVirtualScrolling()){var t=this.getCellData(e),n=this.virtualSelectionState.isValidFocusedCell(t);return n?e:this._$focusedCell}var i=this._$focusedCell,o=this._getGroupIndexByCell(e),a=this._getGroupIndexByCell(i),r=this._hasAllDayClass(e)!==this._hasAllDayClass(i);return a!==o||r?i:e}},{key:"_getCellsBetween",value:function(e,t,n){var i=this._hasAllDayClass(t),o=this._getCells(i,n),a=o.index(e),r=o.index(t);if(a>r){var s=a;a=r,r=s}if(o=o.slice(a,r+1),this._getGroupCount()>1){var l=[],u=this._getGroupIndexByCell(e);(0,E.each)(o,function(e,t){var n=this._getGroupIndexByCell((0,w.default)(t));u===n&&l.push(t)}.bind(this)),o=(0,w.default)(l)}return o}},{key:"_hasAllDayClass",value:function(e){return e.hasClass(ke)}},{key:"_getGroupIndexByCell",value:function(e){if(this.isVirtualScrolling()){var t=this._getCoordinatesByCell(e),n=t.rowIndex,i=t.columnIndex,o=e.hasClass(ke);return this.viewDataProvider.getCellData(n,i,o).groupIndex}return this._groupedStrategy.getGroupIndexByCell(e)}},{key:"_toggleFocusedCellClass",value:function(e,t){var n=t&&t.length?t:this._focusTarget();n.toggleClass(ze,e)}},{key:"_releaseSelectedAndFocusedCells",value:function(){this._releaseFocusedCell(),this._releaseSelectedCells(),this.option("selectedCellData",[])}},{key:"_releaseFocusedCell",value:function(){var e=this._$focusedCell;(0,B.isDefined)(e)&&e.length&&(this._toggleFocusedCellClass(!1,e),this.setAria("label",void 0,e))}},{key:"_releaseSelectedCells",value:function(){var e=(0,w.default)(this._selectedCells);(0,B.isDefined)(e)&&e.length&&(this._toggleFocusClass(!1,e),this.setAria("label",void 0,e))}},{key:"_focusInHandler",value:function(e){if((0,w.default)(e.target).is(this._focusTarget())&&this._isCellClick!==!1){delete this._isCellClick,delete this._contextMenuHandled,c(v(t.prototype),"_focusInHandler",this).apply(this,arguments);var n=this._getFocusedCell();this._setSelectedAndFocusedCells(n)}}},{key:"_focusOutHandler",value:function(){if(c(v(t.prototype),"_focusOutHandler",this).apply(this,arguments),!this._contextMenuHandled){var e;this._releaseSelectedAndFocusedCells(),null===(e=this.virtualSelectionState)||void 0===e?void 0:e.releaseSelectedAndFocusedCells()}}},{key:"_focusTarget",value:function(){return this.$element()}},{key:"_getDefaultOptions",value:function(){return(0,O.extend)(c(v(t.prototype),"_getDefaultOptions",this).call(this),{currentDate:new Date,intervalCount:1,startDate:null,firstDayOfWeek:void 0,startDayHour:0,endDayHour:24,hoursInterval:.5,activeStateEnabled:!0,hoverStateEnabled:!0,groups:[],showAllDayPanel:!0,allDayExpanded:!1,onCellClick:null,crossScrollingEnabled:!1,dataCellTemplate:null,timeCellTemplate:null,resourceCellTemplate:null,dateCellTemplate:null,allowMultipleCellSelection:!0,indicatorTime:new Date,indicatorUpdateInterval:5*se("minute"),shadeUntilCurrentTime:!0,groupOrientation:"horizontal",selectedCellData:[],groupByDate:!1,scrolling:{mode:"standard"},renovateRender:!1,height:void 0})}},{key:"_optionChanged",value:function(e){switch(e.name){case"startDayHour":case"endDayHour":this.invoke("validateDayHours"),this._cleanWorkSpace();break;case"dateCellTemplate":case"resourceCellTemplate":case"dataCellTemplate":case"timeCellTemplate":case"hoursInterval":case"firstDayOfWeek":case"currentDate":case"startDate":this._cleanWorkSpace();break;case"groups":this._cleanView(),this._disposeRenovatedComponents(),this._removeAllDayElements(),this._initGrouping(),this.repaint();break;case"groupOrientation":this._initGroupedStrategy(),this._createAllDayPanelElements(),this._removeAllDayElements(),this._cleanWorkSpace(),this._toggleGroupByDateClass();break;case"showAllDayPanel":this._isVerticalGroupedWorkSpace()?(this._cleanView(),this._disposeRenovatedComponents(),this._removeAllDayElements(),this._initGrouping(),this.repaint()):this._toggleAllDayVisibility();break;case"allDayExpanded":this._changeAllDayVisibility(),this._attachTablesEvents(),this.headerPanelOffsetRecalculate(),this._updateScrollable();break;case"onSelectionChanged":this._createSelectionChangedAction();break;case"onCellClick":this._createCellClickAction();break;case"onCellContextMenu":this._attachContextMenuEvent();break;case"intervalCount":this._cleanWorkSpace(),this._toggleWorkSpaceCountClass(),this._toggleFixedScrollableClass();break;case"groupByDate":this._cleanWorkSpace(),this._toggleGroupByDateClass();break;case"crossScrollingEnabled":this._toggleHorizontalScrollClass(),this._dateTableScrollable.option(this._dateTableScrollableConfig());break;case"width":c(v(t.prototype),"_optionChanged",this).call(this,e),this._dimensionChanged();break;case"allowMultipleCellSelection":break;case"selectedCellData":break;case"scrolling":this.option("renovateRender",this._isVirtualModeOn());break;case"renovateRender":this._disposeRenovatedComponents(),this.repaint();break;default:c(v(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_cleanWorkSpace",value:function(){this._cleanView(),this._toggleGroupedClass(),this._toggleWorkSpaceWithOddCells(),this._renderView()}},{key:"_init",value:function(){this._headerSemaphore=new lt,this._sideBarSemaphore=new lt,this._dataTableSemaphore=new lt,this._viewDataProvider=null,this._virtualSelectionState=null,this._activeStateUnit=st,this._maxAllowedVerticalPosition=[],this._maxAllowedPosition=[],c(v(t.prototype),"_init",this).call(this),this._initGrouping(),this._toggleHorizontalScrollClass(),this._toggleWorkSpaceCountClass(),this._toggleGroupByDateClass(),this._toggleWorkSpaceWithOddCells(),this.$element().addClass(le).addClass(this._getElementClass())}},{key:"_initGrouping",value:function(){this._initGroupedStrategy(),this._toggleGroupingDirectionClass(),this._toggleGroupByDateClass()}},{key:"_initGroupedStrategy",value:function(){var e=this.option("groups").length?this.option("groupOrientation"):this._getDefaultGroupStrategy(),t="vertical"===e?G.default:W.default;this._groupedStrategy=new t(this)}},{key:"_getDefaultGroupStrategy",value:function(){return"horizontal"}},{key:"_isVerticalGroupedWorkSpace",value:function(){return!!this.option("groups").length&&"vertical"===this.option("groupOrientation")}},{key:"_isHorizontalGroupedWorkSpace",value:function(){return!!this.option("groups").length&&"horizontal"===this.option("groupOrientation")}},{key:"_toggleHorizontalScrollClass",value:function(){this.$element().toggleClass(he,this.option("crossScrollingEnabled"))}},{key:"_toggleGroupByDateClass",value:function(){this.$element().toggleClass(pe,this.isGroupedByDate())}},{key:"_toggleWorkSpaceCountClass",value:function(){this.$element().toggleClass(fe,this._isWorkSpaceWithCount())}},{key:"_isWorkSpaceWithCount",value:function(){return this.option("intervalCount")>1}},{key:"_toggleWorkSpaceWithOddCells",value:function(){this.$element().toggleClass(ge,this._isWorkspaceWithOddCells())}},{key:"_isWorkspaceWithOddCells",value:function(){return.5===this.option("hoursInterval")&&!this.isVirtualScrolling()}},{key:"_toggleGroupingDirectionClass",value:function(){this.$element().toggleClass(ce,this._isVerticalGroupedWorkSpace())}},{key:"_getRealGroupOrientation",value:function(){return this._isVerticalGroupedWorkSpace()?"vertical":"horizontal"}},{key:"_getTimePanelClass",value:function(){return _e}},{key:"_getDateTableClass",value:function(){return Fe}},{key:"_getDateTableRowClass",value:function(){return He}},{key:"_getDateTableCellClass",value:function(e,t){var n=Le+" "+Oe+" "+Ee;return this._needApplyLastGroupCellClass()?this._groupedStrategy.addAdditionalGroupCellClasses(n,t+1,e,t):n}},{key:"_needApplyLastGroupCellClass",value:function(){return!0}},{key:"_getGroupRowClass",value:function(){return Me}},{key:"_getGroupHeaderClass",value:function(e){var t=Be;return this._groupedStrategy.addAdditionalGroupCellClasses(t,e+1)}},{key:"_getGroupHeaderContentClass",value:function(){return Ve}},{key:"_initWorkSpaceUnits",value:function(){this._$headerPanel=(0,w.default)("<table>"),this._$thead=(0,w.default)("<thead>").appendTo(this._$headerPanel),this._$fixedContainer=(0,w.default)("<div>").addClass(U.FIXED_CONTAINER_CLASS),this._$allDayContainer=(0,w.default)("<div>").addClass(xe),this._initAllDayPanelElements(),this.isRenovatedRender()?this.createRAllDayPanelElements():this._createAllDayPanelElements(),this._$timePanel=(0,w.default)("<table>").addClass(this._getTimePanelClass()),this._$dateTable=(0,w.default)("<table>"),this._$groupTable=(0,w.default)("<div>").addClass(de)}},{key:"_initAllDayPanelElements",value:function(){this._allDayTitles=[],this._allDayTables=[],this._allDayPanels=[]}},{key:"createRAllDayPanelElements",value:function(){this._$allDayPanel=(0,w.default)("<div>"),this._$allDayTitle=(0,w.default)("<div>").appendTo(this.$element())}},{key:"_createAllDayPanelElements",value:function(){var e=this._getGroupCount();if(this._isVerticalGroupedWorkSpace()&&0!==e)for(var t=0;t<e;t++){var n=(0,w.default)("<div>").addClass(we).text(P.default.format("dxScheduler-allDay"));this._allDayTitles.push(n),this._$allDayTable=(0,w.default)("<table>"),this._allDayTables.push(this._$allDayTable),this._$allDayPanel=(0,w.default)("<div>").addClass(ye).append(this._$allDayTable),this._allDayPanels.push(this._$allDayPanel)}else this._$allDayTitle=(0,w.default)("<div>").addClass(we).text(P.default.format("dxScheduler-allDay")).appendTo(this.$element()),this._$allDayTable=(0,w.default)("<table>"),this._$allDayPanel=(0,w.default)("<div>").addClass(ye).append(this._$allDayTable)}},{key:"_initDateTableScrollable",value:function(){var e=(0,w.default)("<div>").addClass($e);this._dateTableScrollable=this._createComponent(e,j.default,this._dateTableScrollableConfig())}},{key:"_dateTableScrollableConfig",value:function(){var e={useKeyboard:!1,bounceEnabled:!1,updateManually:!0,pushBackValue:0};return this._needCreateCrossScrolling()&&(e=(0,O.extend)(e,this._createCrossScrollingConfig())),e}},{key:"_createCrossScrollingConfig",value:function(){var e=this,t={};return t.direction="both",t.onScroll=function(t){e._dataTableSemaphore.take(),e._sideBarSemaphore.isFree()&&e._sidebarScrollable&&e._sidebarScrollable.scrollTo({top:t.scrollOffset.top}),e._headerSemaphore.isFree()&&e._headerScrollable&&e._headerScrollable.scrollTo({left:t.scrollOffset.left}),e._dataTableSemaphore.release()},t.onEnd=function(){e.notifyObserver("updateResizableArea",{})},t}},{key:"_createWorkSpaceElements",value:function(){this.option("crossScrollingEnabled")?this._createWorkSpaceScrollableElements():this._createWorkSpaceStaticElements()}},{key:"_createWorkSpaceStaticElements",value:function(){this._isVerticalGroupedWorkSpace()?(this._dateTableScrollable.$content().append(this._$allDayContainer,this._$groupTable,this._$timePanel,this._$dateTable),this.$element().append(this._$fixedContainer,this._$headerPanel,this._dateTableScrollable.$element())):(this._dateTableScrollable.$content().append(this._$timePanel,this._$dateTable),this.$element().append(this._$fixedContainer,this._$headerPanel,this._$allDayContainer,this._$allDayPanel,this._dateTableScrollable.$element()))}},{key:"_createWorkSpaceScrollableElements",value:function(){this.$element().append(this._$fixedContainer),this._createHeaderScrollable(),this._createSidebarScrollable(),this.$element().append(this._dateTableScrollable.$element()),this._headerScrollable.$content().append(this._$headerPanel),this._dateTableScrollable.$content().append(this._$dateTable),this._isVerticalGroupedWorkSpace()?(this._dateTableScrollable.$content().prepend(this._$allDayContainer),this._sidebarScrollable.$content().append(this._$groupTable,this._$timePanel)):this._headerScrollable.$content().append(this._$allDayContainer,this._$allDayPanel),this._sidebarScrollable.$content().append(this._$timePanel)}},{key:"_createHeaderScrollable",value:function(){var e=(0,w.default)("<div>").addClass(Ge).appendTo(this.$element());this._headerScrollable=this._createComponent(e,j.default,this._headerScrollableConfig())}},{key:"_headerScrollableConfig",value:function(){var e=this,t={useKeyboard:!1,showScrollbar:!1,direction:"horizontal",useNative:!1,updateManually:!0,bounceEnabled:!1,pushBackValue:0,onScroll:function(t){e._headerSemaphore.take(),e._dataTableSemaphore.isFree()&&e._dateTableScrollable.scrollTo({left:t.scrollOffset.left}),e._headerSemaphore.release()}};return t}},{key:"_createSidebarScrollable",value:function(){var e=this,t=(0,w.default)("<div>").addClass(qe).appendTo(this.$element());this._sidebarScrollable=this._createComponent(t,j.default,{useKeyboard:!1,showScrollbar:!1,direction:"vertical",useNative:!1,updateManually:!0,bounceEnabled:!1,pushBackValue:0,onScroll:function(t){e._sideBarSemaphore.take(),e._dataTableSemaphore.isFree()&&e._dateTableScrollable.scrollTo({top:t.scrollOffset.top}),e._sideBarSemaphore.release()}})}},{key:"_visibilityChanged",value:function(e){this.cache.clear(),e&&this._isVerticalGroupedWorkSpace()&&this._setHorizontalGroupHeaderCellsHeight(),e&&this._needCreateCrossScrolling()&&this._setTableSizes()}},{key:"_attachTableClasses",value:function(){if(this._addTableClass(this._$dateTable,this._getDateTableClass()),this._isVerticalGroupedWorkSpace())for(var e=this._getGroupCount(),t=0;t<e;t++)this._addTableClass(this._allDayTables[t],be);else this._addTableClass(this._$allDayTable,be)}},{key:"_attachHeaderTableClasses",value:function(){this._addTableClass(this._$headerPanel,Ae)}},{key:"_addTableClass",value:function(e,t){e&&!e.hasClass(t)&&e.addClass(t)}},{key:"_setTableSizes",value:function(){this._attachTableClasses();var e=this.getCellWidth();e<this.getCellMinWidth()&&(e=this.getCellMinWidth());var t=this.getWorkSpaceMinWidth(),n=this._$headerPanel.find("tr").last().find("th"),i=e*n.length;i<t&&(i=t),this._$headerPanel.width(i),this._$dateTable.width(i),this._$allDayTable&&this._$allDayTable.width(i),this._attachHeaderTableClasses(),this._isVerticalGroupedWorkSpace()&&this._setHorizontalGroupHeaderCellsHeight()}},{key:"getWorkSpaceMinWidth",value:function(){return this._groupedStrategy.getWorkSpaceMinWidth()}},{key:"_dimensionChanged",value:function(){this.option("crossScrollingEnabled")&&this._setTableSizes(),this.headerPanelOffsetRecalculate(),this.cache.clear(),this._cleanAllowedPositions()}},{key:"_needCreateCrossScrolling",value:function(){return this.option("crossScrollingEnabled")}},{key:"_getElementClass",value:function(){return(0,M.noop)()}},{key:"_getRowCount",value:function(){return(0,M.noop)()}},{key:"_getRowCountWithAllDayRows",value:function(){var e=this._isShowAllDayPanel()?1:0;return this._getRowCount()+e}},{key:"_getCellCount",value:function(){return(0,M.noop)()}},{key:"_initMarkup",value:function(){this.cache.clear(),this._initWorkSpaceUnits(),this._initDateTableScrollable(),this._createWorkSpaceElements(),this._initVirtualScrolling(),c(v(t.prototype),"_initMarkup",this).call(this),this.option("crossScrollingEnabled")||(this._attachTableClasses(),this._attachHeaderTableClasses()),this._toggleGroupedClass(),this._toggleFixedScrollableClass(),this._renderView(),this._attachEvents(),this._setFocusOnCellByOption(this.option("selectedCellData"))}},{key:"isRenovatedRender",value:function(){return this.renovatedRenderSupported()&&this.option("renovateRender")}},{key:"_isVirtualModeOn",value:function(){return"virtual"===this.option("scrolling.mode")}},{key:"isVirtualScrolling",value:function(){return this.isRenovatedRender()&&this._isVirtualModeOn()}},{key:"_initVirtualScrolling",value:function(){this.virtualScrollingDispatcher&&(this.virtualScrollingDispatcher.dispose(),this.virtualScrollingDispatcher=null),this.isVirtualScrolling()&&(this.virtualScrollingDispatcher=new Q.default(this))}},{key:"_render",value:function(){c(v(t.prototype),"_render",this).call(this),this._renderDateTimeIndication(),this._setIndicationUpdateInterval()}},{key:"_toggleGroupedClass",value:function(){this.$element().toggleClass(ue,this._getGroupCount()>0)}},{key:"_toggleFixedScrollableClass",value:function(){return(0,M.noop)()}},{key:"_renderView",value:function(){this._setFirstViewDate(),this._applyCellTemplates(this._renderGroupHeader()),this._renderDateHeader(),this.isRenovatedRender()?this.renderRWorkspace():(this._renderTimePanel(),this._renderGroupAllDayPanel(),this._renderDateTable(),this._renderAllDayPanel()),this._updateGroupTableHeight(),this._shader=new $.default(this)}},{key:"isGroupedAllDayPanel",value:function(){return this._isShowAllDayPanel()&&this._isVerticalGroupedWorkSpace()}},{key:"generateRenderOptions",value:function(){var e=this._getGroupCount(),t=this._isVerticalGroupedWorkSpace()?e:1,n=this._isVerticalGroupedWorkSpace()?1:e,i=this._insertAllDayRowsIntoDateTable()?this._allDayTitles:void 0,o=this._getRowCount(),a=this._getTotalCellCount(e),r={horizontalGroupCount:n,verticalGroupCount:t,rowCountInGroup:o,cellCount:a,cellCountInGroupRow:this._getCellCount(),cellDataGetters:[this._getCellData.bind(this)],allDayElements:i,startRowIndex:0,groupOrientation:this.option("groupOrientation"),nonVirtualRowCount:this._getRowCount(),groupCount:e};return this.isVirtualScrolling()?(0,O.extend)(r,this.virtualScrollingDispatcher.renderState):r.rowCount=this._getTotalRowCount(e,this._isVerticalGroupedWorkSpace()),r}},{key:"renovatedRenderSupported",value:function(){return!1}},{key:"renderRWorkspace",value:function(){var e,t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];this._cleanAllowedPositions(),this.viewDataProvider.update(t),this.renderRAllDayPanel(),this.renderRTimeTable(),this.renderRDateTable(),this.updateRSelection(),null===(e=this.virtualScrollingDispatcher)||void 0===e?void 0:e.updateDimensions()}},{key:"renderRAllDayPanel",value:function(){var e=this._isShowAllDayPanel()&&!this.isGroupedAllDayPanel();if(e){var t={viewData:this.viewDataProvider.viewData,visible:e,dataCellTemplate:this.option("dataCellTemplate")};this.renderRComponent(this._$allDayPanel,ee.default,"renovatedAllDayPanel",t),this.renderRComponent(this._$allDayTitle,te.default,"renovatedAllDayPanelTitle",{visible:e}),this._$allDayTable=this.renovatedAllDayPanel.$element().find(".".concat(be))}this._toggleAllDayVisibility()}},{key:"renderRTimeTable",value:function(){this.renderRComponent(this._$timePanel,ne.default,"renovatedTimePanel",{viewData:this.viewDataProvider.viewData,timeCellTemplate:this.option("timeCellTemplate")})}},{key:"renderRDateTable",value:function(){}},{key:"renderRComponent",value:function(e,t,n,i){var o=this[n];if(o)Object.getOwnPropertyNames(i).forEach(function(e){o.option(e,i[e])});else{var a=(0,T.getPublicElement)(e);o=this._createComponent(a,t,l(l({},i),{},{groupOrientation:this.option("groupOrientation")})),this[n]=o}}},{key:"updateRSelection",value:function(){var e=this._isVerticalGroupedWorkSpace(),t=this.virtualSelectionState.getFocusedCell(e),n=this.virtualSelectionState.getSelectedCells();if(t){var i=t.coordinates,o=t.cellData,a=!e&&o.allDay?this._dom_getAllDayPanelCell(i.cellIndex):this._dom_getDateCell(i);a&&this._setFocusedCell(a)}n&&this._setSelectedCellsByCellData(n)}},{key:"_updateGroupTableHeight",value:function(){this._isVerticalGroupedWorkSpace()&&(0,I.hasWindow)()&&this._setHorizontalGroupHeaderCellsHeight()}},{key:"_renderDateTimeIndication",value:function(){return(0,M.noop)()}},{key:"_setIndicationUpdateInterval",value:function(){return(0,M.noop)()}},{key:"_refreshDateTimeIndication",value:function(){return(0,M.noop)()}},{key:"_setFocusOnCellByOption",value:function(e){this._releaseSelectedAndFocusedCells(),this._setSelectedCellsByCellData(e)}},{key:"_setSelectedCellsByCellData",value:function(e){var t,n=this,i=[],o=this._getAllCells(null===e||void 0===e?void 0:null===(t=e[0])||void 0===t?void 0:t.allDay),a=this._getTotalCellCount(this._getGroupCount());e.forEach(function(e){var t=e.groups,r=e.startDate,s=e.allDay,l=e.groupIndex;l||(l=n._isGroupsSpecified(t)?n._getGroupIndexByResourceId(t):0);var u=n.isVirtualScrolling()?n.viewDataProvider.findCellPositionInMap(l,r,s):n.getCoordinatesByDate(r,l,s);if(u){var c=u.rowIndex,d=u.cellIndex,h=c*a+d,f=o[h];(0,B.isDefined)(f)&&(n._toggleFocusClass(!0,(0,w.default)(f)),i.push(f))}}),this._selectedCells=i}},{key:"_isGroupsSpecified",value:function(e){return this.option("groups").length&&e}},{key:"_getGroupIndexByResourceId",value:function(e){var t=this.option("groups"),n=this.invoke("createResourcesTree",t);return n.length?this._getGroupIndexRecursively(n,e):0}},{key:"_getGroupIndexRecursively",value:function(e,t){var n=this,i=e[0].name,o=t[i];return e.reduce(function(e,i){var a=i.leafIndex,r=i.value,s=i.children,l=o===r;return l&&void 0!==a?a:l?n._getGroupIndexRecursively(s,t):e},0)}},{key:"_setFirstViewDate",value:function(){var e=(0,B.isDefined)(this._firstDayOfWeek())?this._firstDayOfWeek():R.default.firstDayOfWeekIndex();this._firstViewDate=D.default.getFirstWeekDate(this._getViewStartByOptions(),e),this._setStartDayHour(this._firstViewDate)}},{key:"_getViewStartByOptions",value:function(){if(this.option("startDate")){for(var e=D.default.trimTime(this._getStartViewDate()),t=this.option("currentDate"),n=e.getTime()<=t.getTime()?1:-1,i=new Date(e.getTime()+this._getIntervalDuration()*n);!this._dateInRange(t,e,i,n);)e=i,i=new Date(e.getTime()+this._getIntervalDuration()*n);return n>0?e:i}return this.option("currentDate")}},{key:"_getHeaderDate",value:function(){return this.getStartViewDate()}},{key:"_getStartViewDate",value:function(){return this.option("startDate")}},{key:"_dateInRange",value:function(e,t,n,i){return i>0?D.default.dateInRange(e,t,new Date(n.getTime()-1)):D.default.dateInRange(e,n,t,"date")}},{key:"_getIntervalDuration",value:function(){return se("day")*this.option("intervalCount")}},{key:"_setStartDayHour",value:function(e){var t=this.option("startDayHour");(0,B.isDefined)(t)&&e.setHours(t,t%1*60,0,0)}},{key:"_firstDayOfWeek",value:function(){return this.option("firstDayOfWeek")}},{key:"_attachEvents",value:function(){this._createSelectionChangedAction(),this._attachClickEvent(),this._attachContextMenuEvent()}},{key:"_attachClickEvent",value:function(){var e=this,t=this._createAction(function(t){e._pointerDownHandler(t.event)});this._createCellClickAction();var n="."+Le+",."+ke,i=this.$element();k.default.off(i,Ke),k.default.off(i,Xe),k.default.on(i,Ke,function(e){return(0,V.isMouseEvent)(e)&&e.which>1?void e.preventDefault():void t({event:e})}),k.default.on(i,Xe,n,function(t){var n=(0,w.default)(t.target);e._cellClickAction({event:t,cellElement:(0,T.getPublicElement)(n),cellData:e.getCellData(n)})})}},{key:"_createCellClickAction",value:function(){var e=this;this._cellClickAction=this._createActionByOption("onCellClick",{
afterExecute:function(t){return e._cellClickHandler(t.args[0].event)}})}},{key:"_createSelectionChangedAction",value:function(){this._selectionChangedAction=this._createActionByOption("onSelectionChanged")}},{key:"_cellClickHandler",value:function(e){var t=(0,w.default)(e.target);this._showPopup&&this._hasFocusClass(t)&&(delete this._showPopup,this._showAddAppointmentPopup(t))}},{key:"_pointerDownHandler",value:function(e){var t=(0,w.default)(e.target);return t.hasClass(Le)||t.hasClass(ke)?(this._isCellClick=!0,void(t.hasClass(ze)?this._showPopup=!0:this._setSelectedAndFocusedCells(t))):void(this._isCellClick=!1)}},{key:"_showAddAppointmentPopup",value:function(e){var t=this.getCellData(e.first()),n=this.getCellData(e.last());if(this.isVirtualScrolling()){var i=this.virtualSelectionState.getSelectedCells();t=i[0],n=i[i.length-1]}var o={startDate:t.startDate,endDate:n.endDate};void 0!==n.allDay&&(o.allDay=n.allDay),this.invoke("showAddAppointmentPopup",o,n.groups)}},{key:"_attachContextMenuEvent",value:function(){this._createContextMenuAction();var e="."+Le+",."+ke,t=this.$element(),n=(0,V.addNamespace)(z.name,this.NAME);k.default.off(t,n,e),k.default.on(t,n,e,this._contextMenuHandler.bind(this))}},{key:"_contextMenuHandler",value:function(e){var t=(0,w.default)(e.target);this._contextMenuAction({event:e,cellElement:(0,T.getPublicElement)(t),cellData:this.getCellData(t)}),this._contextMenuHandled=!0}},{key:"_createContextMenuAction",value:function(){this._contextMenuAction=this._createActionByOption("onCellContextMenu")}},{key:"_getGroupHeaderContainer",value:function(){return this._isVerticalGroupedWorkSpace()?this._$groupTable:this._$thead}},{key:"_getDateHeaderContainer",value:function(){return this._$thead}},{key:"_renderGroupHeader",value:function(){var e=this._getGroupHeaderContainer(),t=this._getGroupCount(),n=[];if(t){var i=this._makeGroupRows(this.option("groups"),this.option("groupByDate"));this._attachGroupCountAttr(t,i),e.append(i.elements),n=i.cellTemplates}else this._detachGroupCountAttr();return n}},{key:"_applyCellTemplates",value:function(e){e.forEach(function(e){e()})}},{key:"_detachGroupCountAttr",value:function(){var e=this._groupedStrategy.getGroupCountAttr();this.$element().removeAttr(e.attr)}},{key:"_attachGroupCountAttr",value:function(e,t){var n=this._groupedStrategy.getGroupCountAttr(e,t);this.$element().attr(n.attr,n.count)}},{key:"headerPanelOffsetRecalculate",value:function(){if(this.option("resourceCellTemplate")||this.option("dateCellTemplate")){var e=this.getHeaderPanelHeight(),t=this.invoke("getHeaderHeight"),n=this.supportAllDayRow()&&this._isShowAllDayPanel()?this._groupedStrategy.getAllDayTableHeight():0;e&&this._headerScrollable&&this._headerScrollable.$element().height(e+n),e&&this._dateTableScrollable.$element().css({paddingBottom:n+e+"px",marginBottom:-1*(parseInt(e,10)+n)+"px"}),e&&this._sidebarScrollable&&this._sidebarScrollable.$element().css({paddingBottom:n+e+"px",marginBottom:-1*(parseInt(e,10)+n)+"px"}),this._$allDayTitle&&this._$allDayTitle.css("top",t+e+"px")}}},{key:"_makeGroupRows",value:function(e,t){var n=this._isVerticalGroupedWorkSpace()?ae.VERTICAL:ae.HORIZONTAL;return ae.makeGroupedTable(n,e,{groupHeaderRowClass:this._getGroupRowClass(),groupRowClass:this._getGroupRowClass(),groupHeaderClass:this._getGroupHeaderClass.bind(this),groupHeaderContentClass:this._getGroupHeaderContentClass()},this._getCellCount()||1,this.option("resourceCellTemplate"),this._getGroupCount(),t)}},{key:"_getDateHeaderTemplate",value:function(){return this.option("dateCellTemplate")}},{key:"_renderDateHeader",value:function(){var e=this._getDateHeaderContainer(),t=(0,w.default)("<tr>").addClass(Re),n=this._getCellCount(),i=this._getDateHeaderTemplate(),o=this._getCalculateHeaderCellRepeatCount(),a=[],r=this.isGroupedByDate();if(r){for(var s=r?this._getGroupCount():1,l=0;l<n;l++){var u=l*o,c=this._renderDateHeaderTemplate(t,l,u,i,a);c.attr("colSpan",s)}e.prepend(t)}else{for(var d=0;d<o;d++)for(var h=0;h<n;h++){var f=d*o+h;this._renderDateHeaderTemplate(t,h,f,i,a)}e.append(t)}return this._applyCellTemplates(a),t}},{key:"_renderDateHeaderTemplate",value:function(e,t,n,i,o){var a=this._getHeaderText(t),r=(0,w.default)("<th>").addClass(this._getHeaderPanelCellClass(t)).attr("title",a);return null!==i&&void 0!==i&&i.render?o.push(i.render.bind(i,{model:{text:a,date:this._getDateByIndex(t)},index:n,container:(0,T.getPublicElement)(r)})):r.text(a),e.append(r),r}},{key:"_getHeaderPanelCellClass",value:function(e){var t=Pe+" "+Oe;return this._groupedStrategy.addAdditionalGroupCellClasses(t,e+1,void 0,void 0,this.isGroupedByDate())}},{key:"_getCalculateHeaderCellRepeatCount",value:function(){return this._groupedStrategy.calculateHeaderCellRepeatCount()}},{key:"_renderAllDayPanel",value:function(e){var t=this._getCellCount();this._isVerticalGroupedWorkSpace()||(t*=this._getGroupCount()||1);var n=this._renderTableBody({container:this._allDayPanels.length?(0,T.getPublicElement)(this._allDayTables[e]):(0,T.getPublicElement)(this._$allDayTable),rowCount:1,cellCount:t,cellClass:this._getAllDayPanelCellClass.bind(this),rowClass:Se,cellTemplate:this.option("dataCellTemplate"),getCellData:this._getAllDayCellData.bind(this),groupIndex:e},!0);this._toggleAllDayVisibility(),this._applyCellTemplates(n)}},{key:"_renderGroupAllDayPanel",value:function(){if(this._isVerticalGroupedWorkSpace())for(var e=this._getGroupCount(),t=0;t<e;t++)this._renderAllDayPanel(t)}},{key:"_getAllDayPanelCellClass",value:function(e,t){var n=ke+" "+Oe;return this._groupedStrategy.addAdditionalGroupCellClasses(n,t+1)}},{key:"_getAllDayCellData",value:function(e,t,n,i){var o=this._getDateByCellIndexes(t,n),a=i||this._getGroupIndex(t,n);o=D.default.trimTime(o);var r={startDate:o,endDate:o,allDay:!0,groupIndex:a},s=this._getCellGroups(a);s.length&&(r.groups={});for(var l=0;l<s.length;l++)r.groups[s[l].name]=s[l].id;return{key:tt,value:r}}},{key:"_toggleAllDayVisibility",value:function(){var e=this._isShowAllDayPanel();this._$allDayPanel.toggle(e),this._$allDayTitle&&this._$allDayTitle.toggleClass(Ce,!e),this.$element().toggleClass(De,e),this._changeAllDayVisibility(),this._updateScrollable()}},{key:"_changeAllDayVisibility",value:function(){this.$element().toggleClass(Ie,!this.option("allDayExpanded")&&this._isShowAllDayPanel())}},{key:"_updateScrollable",value:function(){this._dateTableScrollable.update(),this._headerScrollable&&this._headerScrollable.update(),this._sidebarScrollable&&this._sidebarScrollable.update()}},{key:"_renderTimePanel",value:function(){var e=this,t=this._groupedStrategy.calculateTimeCellRepeatCount(),n=Y.default.getDateWithoutTimezoneChange(this.getStartViewDate()),i=function(t){var i=t%e._getRowCount();return i%2===0?R.default.format(e._getTimeCellDateCore(n,t),"shorttime"):""};this._renderTableBody({container:(0,T.getPublicElement)(this._$timePanel),rowCount:this._getTimePanelRowCount()*t,cellCount:1,cellClass:this._getTimeCellClass.bind(this),rowClass:ve,cellTemplate:this.option("timeCellTemplate"),getCellText:i.bind(this),getCellDate:this._getTimeCellDate.bind(this),groupCount:this._getGroupCount(),allDayElements:this._insertAllDayRowsIntoDateTable()?this._allDayTitles:void 0})}},{key:"_getTimePanelRowCount",value:function(){return this._getCellCountInDay()}},{key:"_getCellCountInDay",value:function(e){var t=this._calculateDayDuration()/this.option("hoursInterval");return e?t:Math.ceil(t)}},{key:"_calculateDayDuration",value:function(){return this.option("endDayHour")-this.option("startDayHour")}},{key:"_getTimeCellClass",value:function(e){var t=me+" "+Ee;return this._isVerticalGroupedWorkSpace()?this._groupedStrategy.addAdditionalGroupCellClasses(t,e,e):t}},{key:"_getTimeCellDate",value:function(e){return this._getTimeCellDateCore(this.getStartViewDate(),e)}},{key:"_getTimeCellDateCore",value:function(e,t){var n=new Date(e),i=Math.round(this.getCellDuration()),o=this._getCellCountInDay(!0);return n.setMilliseconds(n.getMilliseconds()+i*(t%o)),n}},{key:"_renderDateTable",value:function(){var e=this._getGroupCount();this._renderTableBody({container:(0,T.getPublicElement)(this._$dateTable),rowCount:this._getTotalRowCount(e),cellCount:this._getTotalCellCount(e),cellClass:this._getDateTableCellClass.bind(this),rowClass:this._getDateTableRowClass(),cellTemplate:this.option("dataCellTemplate"),getCellData:this._getCellData.bind(this),allDayElements:this._insertAllDayRowsIntoDateTable()?this._allDayPanels:void 0,groupCount:e,groupByDate:this.option("groupByDate")})}},{key:"_insertAllDayRowsIntoDateTable",value:function(){return this._groupedStrategy.insertAllDayRowsIntoDateTable()}},{key:"_getTotalCellCount",value:function(e){return this._groupedStrategy.getTotalCellCount(e)}},{key:"_getTotalRowCount",value:function(e,t){var n=this._groupedStrategy.getTotalRowCount(e);return t&&e>1&&this._isShowAllDayPanel()&&(n+=e),n}},{key:"_getCellData",value:function(e,t,n){var i=this._prepareCellData(t,n,e);return{key:tt,value:i}}},{key:"_prepareCellData",value:function(e,t){var n=this._getDateByCellIndexes(e,t),i=this.calculateEndDate(n),o=this._getGroupIndex(e,t),a={startDate:n,endDate:i,allDay:this._getTableAllDay(),groupIndex:o},r=this._getCellGroups(o);r.length&&(a.groups={});for(var s=0;s<r.length;s++)a.groups[r[s].name]=r[s].id;return a}},{key:"_getGroupIndex",value:function(e,t){return this._groupedStrategy.getGroupIndex(e,t)}},{key:"_getTableAllDay",value:function(){return!1}},{key:"calculateEndDate",value:function(e){var t=new Date(e);return t.setMilliseconds(t.getMilliseconds()+Math.round(this._getInterval())),t}},{key:"_getGroupCount",value:function(){for(var e=this.option("groups"),t=0,n=0,i=e.length;n<i;n++)n?t*=e[n].items.length:t=e[n].items.length;return t}},{key:"_getPathToLeaf",value:function(e){function t(e,n){for(var i=0;i<e.length;i++){if(e[i].leafIndex===n)return e[i];var o=t(e[i].children,n);if(o)return o}}function n(e,t){return t=t||[],t.push(e.value),e.parent&&n(e.parent,t),t}var i=this.invoke("createResourcesTree",this.option("groups")),o=t(i,e);return n(o).reverse()}},{key:"_getCellGroups",value:function(e){var t=[];if(this._getGroupCount()){var n=this.option("groups");if(e<0)return;for(var i=this._getPathToLeaf(e),o=0;o<n.length;o++)t.push({name:n[o].name,id:i[o]})}return t}},{key:"_attachTablesEvents",value:function(){var e,t,n=this,i=this,o=!1,a=this.$element();k.default.off(a,Ue),k.default.off(a,Ze),k.default.off(a,Ye),k.default.off(a,et),k.default.off(a,Qe),k.default.on(a,Ue,rt,{itemSizeFunc:function(n){return e||(e=(0,A.getBoundingRect)(n.get(0)).height),t||(t=(0,A.getBoundingRect)(n.get(0)).width),{width:t,height:e}},checkDropTarget:function(e,t){return!n._isOutsideScrollable(e,t)}},function(e){i._$currentTableTarget&&i.removeDroppableCellClass(i._$currentTableTarget),i._$currentTableTarget=(0,w.default)(e.target),i._$currentTableTarget.addClass(We)}),k.default.on(a,Ze,function(e){a.find((0,w.default)(e.draggingElement)).length||i.removeDroppableCellClass()}),k.default.on(a,Ye,rt,function(n){i.removeDroppableCellClass((0,w.default)(n.target)),e=0,t=0}),k.default.on(a,Qe,rt,function(e){(0,V.isMouseEvent)(e)&&1===e.which&&(o=!0,i.$element().addClass(Te),k.default.off(C.default.getDocument(),Je),k.default.on(C.default.getDocument(),Je,function(){o=!1,i.$element().removeClass(Te)}))}),k.default.on(a,et,rt,function(e){o&&i._dateTableScrollable&&!i._dateTableScrollable.option("scrollByContent")&&(e.preventDefault(),e.stopPropagation(),i._moveToCell((0,w.default)(e.target),!0))})}},{key:"_getDateTables",value:function(){return this._$dateTable.add(this._$allDayTable)}},{key:"_getDateTable",value:function(){return this._$dateTable}},{key:"_getAllDayTable",value:function(){return this._$allDayTable}},{key:"_getInterval",value:function(){return void 0===this._interval&&(this._interval=this.option("hoursInterval")*at),this._interval}},{key:"_getHeaderText",value:function(e){return R.default.format(this._getDateForHeaderText(e),this._getFormat())}},{key:"_getDateForHeaderText",value:function(e){return this._getDateByIndex(e)}},{key:"_getDateByIndex",value:function(){return re()}},{key:"_getFormat",value:function(){return re()}},{key:"_calculateCellIndex",value:function(e,t){return this._groupedStrategy.calculateCellIndex(e,t)}},{key:"_renderTableBody",value:function(e,t){var n=[];return t?n=ae.makeTable(e):this._applyCellTemplates(ae.makeTable(e)),n}},{key:"_removeAllDayElements",value:function(){this._$allDayTable&&this._$allDayTable.remove(),this._$allDayTitle&&this._$allDayTitle.remove()}},{key:"_cleanView",value:function(){var e;this.cache.clear(),this._cleanAllowedPositions(),null===(e=this.virtualSelectionState)||void 0===e?void 0:e.releaseSelectedAndFocusedCells(),this._$thead.empty(),this._$dateTable.empty(),this._shader&&this._shader.clean(),this._$timePanel.empty(),this._$allDayTable&&this._$allDayTable.empty(),this._$groupTable.empty(),delete this._hiddenInterval,delete this._interval}},{key:"_clean",value:function(){k.default.off(C.default.getDocument(),Je),c(v(t.prototype),"_clean",this).call(this)}},{key:"_disposeRenovatedComponents",value:function(){var e,t,n;null===(e=this.renovatedAllDayPanel)||void 0===e?void 0:e.dispose(),this.renovatedAllDayPanel=void 0,null===(t=this.renovatedDateTable)||void 0===t?void 0:t.dispose(),this.renovatedDateTable=void 0,null===(n=this.renovatedTimePanel)||void 0===n?void 0:n.dispose(),this.renovatedTimePanel=void 0}},{key:"getWorkArea",value:function(){return this._dateTableScrollable.$content()}},{key:"getScrollable",value:function(){return this._dateTableScrollable}},{key:"getScrollableScrollTop",value:function(){return this._dateTableScrollable.scrollTop()}},{key:"getGroupedScrollableScrollTop",value:function(e){return this._groupedStrategy.getScrollableScrollTop(e)}},{key:"getScrollableScrollLeft",value:function(){return this._dateTableScrollable.scrollLeft()}},{key:"getScrollableOuterWidth",value:function(){return this._dateTableScrollable.scrollWidth()}},{key:"getScrollableContainer",value:function(){return this._dateTableScrollable._container()}},{key:"getHeaderPanelHeight",value:function(){return this._$headerPanel&&this._$headerPanel.outerHeight(!0)}},{key:"getTimePanelWidth",value:function(){return this._$timePanel&&(0,A.getBoundingRect)(this._$timePanel.get(0)).width}},{key:"getGroupTableWidth",value:function(){return this._$groupTable?this._$groupTable.outerWidth():0}},{key:"getWorkSpaceLeftOffset",value:function(){return this._groupedStrategy.getLeftOffset()}},{key:"getGroupedStrategy",value:function(){return this._groupedStrategy}},{key:"_getCellCoordinatesByIndex",value:function(e){var t=Math.floor(e/this._getRowCount()),n=e-this._getRowCount()*t;return{cellIndex:t,rowIndex:n}}},{key:"_getDateByCellIndexes",value:function(e,t,n){t=n?t:this._patchCellIndex(t);var i=this.getStartViewDate(),o=new Date(i.getTime()+this._getMillisecondsOffset(e,t)+this._getOffsetByCount(t));return o.setTime(o.getTime()+D.default.getTimezonesDifference(i,o)),o}},{key:"_patchCellIndex",value:function(e){return this.isGroupedByDate()&&(e=Math.floor(e/this._getGroupCount())),e}},{key:"_getOffsetByCount",value:function(){return 0}},{key:"_getMillisecondsOffset",value:function(e,t){return this._getInterval()*this._calculateCellIndex(e,t)+this._calculateHiddenInterval(e,t)}},{key:"_calculateHiddenInterval",value:function(e,t){var n=t%this._getCellCount();return n*this._getHiddenInterval()}},{key:"_getHiddenInterval",value:function(){return void 0===this._hiddenInterval&&(this._hiddenInterval=ot-this.getVisibleDayDuration()),this._hiddenInterval}},{key:"_getIntervalBetween",value:function(e,t){var n=this.getStartViewDate(),i=this.option("startDayHour")*at,o=D.default.getTimezonesDifference(n,e),a=e.getTime()-n.getTime()-o,r=this._getDaysOfInterval(a,i),s=this._getWeekendsCount(r),l=(r-s)*ot;return t||(l=a-r*this._getHiddenInterval()-s*this.getVisibleDayDuration()),l}},{key:"_getWeekendsCount",value:function(){return 0}},{key:"_getDaysOfInterval",value:function(e,t){return Math.floor((e+t)/ot)}},{key:"_getGroupIndexes",value:function(e){var t=[];if(this._isGroupsSpecified(e)){var n=this.invoke("createResourcesTree",this.option("groups"));t=this.invoke("getResourceTreeLeaves",n,e)}return t}},{key:"_updateIndex",value:function(e){return e*this._getRowCount()}},{key:"_getDroppableCell",value:function(){return this._getDateTables().find("."+We)}},{key:"_getWorkSpaceWidth",value:function(){var e=this;return this.cache.get("workspaceWidth",function(){return e._needCreateCrossScrolling()?(0,A.getBoundingRect)(e._$dateTable.get(0)).width:(0,A.getBoundingRect)(e.$element().get(0)).width-e.getTimePanelWidth()})}},{key:"_getCellPositionByIndex",value:function(e,t,n){var i=this._getCellCoordinatesByIndex(e),o=this._getCellByCoordinates(i,t,n);return this._getCellPositionWithCache(o,i,t)}},{key:"_getCellPositionWithCache",value:function(e,t,n){var i=this._getCellPosition(e);return this.setCellDataCache(t,n,e),i&&(i.rowIndex=t.rowIndex,i.cellIndex=t.cellIndex),i}},{key:"_getCellPosition",value:function(e){var t=e.position();return this.option("rtlEnabled")&&(t.left+=(0,A.getBoundingRect)(e.get(0)).width),t}},{key:"_getCellByCoordinates",value:function(e,t,n){var i=this._groupedStrategy.prepareCellIndexes(e,t,n);return this._dom_getDateCell(i)}},{key:"_dom_getDateCell",value:function(e){return this._$dateTable.find("tr:not(.".concat(Ne,")")).eq(e.rowIndex).find("td").eq(e.cellIndex)}},{key:"_dom_getAllDayPanelCell",value:function(e){return this._$allDayPanel.find("tr").eq(0).find("td").eq(e)}},{key:"_getCells",value:function(e,t){var n=e?ke:Le;if("vertical"===t){for(var i=[],o=1;;o++){var a=this.$element().find("tr .".concat(n,":nth-child(").concat(o,")"));if(!a.length)break;i=i.concat(a.toArray())}return(0,w.default)(i)}return this.$element().find("."+n)}},{key:"_getAllCells",value:function(e){if(this._isVerticalGroupedWorkSpace())return this._$dateTable.find("td:not(.".concat(je,")"));var t=e&&this.supportAllDayRow()?ke:Le;return this.$element().find("."+t)}},{key:"_setHorizontalGroupHeaderCellsHeight",value:function(){var e=(0,A.getBoundingRect)(this._$dateTable.get(0)).height;this._$groupTable.outerHeight(e)}},{key:"_getDateTableBorder",value:function(){return nt}},{key:"_getDateTableBorderOffset",value:function(){return 2*this._getDateTableBorder()}},{key:"_getGroupHeaderCellsContent",value:function(){return this.$element().find("."+Ve)}},{key:"_getGroupHeaderCells",value:function(){return this.$element().find("."+Be)}},{key:"_getScrollCoordinates",value:function(e,t,n,i,o){var a=n||new Date(this.option("currentDate")),r=this.option("startDayHour"),s=this.option("endDayHour");if(e<r&&(e=r),e>=s&&(e=s-1),a.setHours(e,t,0,0),!this.isVirtualScrolling())return this.getCoordinatesByDate(a,i,o);var l=this.viewDataProvider.findGlobalCellPosition(a,i,o),u=l.position,c=l.cellData;return this.virtualScrollingDispatcher.calculateCoordinatesByDataAndPosition(c,u,a)}},{key:"_isOutsideScrollable",value:function(e,t){var n=this._dateTableScrollable.$element(),i=(0,A.getBoundingRect)(n.get(0)),o=(0,I.getWindow)(),a=!(0,w.default)(e).closest(n).length,r=t.pageX<i.left||t.pageX>i.left+i.width+(o.scrollX||0),s=t.pageY<i.top||t.pageY>i.top+i.height+(o.scrollY||0);return!(a&&!r)&&(s||r)}},{key:"setCellDataCache",value:function(e,t,n){var i=JSON.stringify({rowIndex:e.rowIndex,cellIndex:e.cellIndex,groupIndex:t});this.cache.set(i,this.getCellData(n))}},{key:"setCellDataCacheAlias",value:function(e,t){var n=JSON.stringify({rowIndex:e.rowIndex,cellIndex:e.cellIndex,groupIndex:e.groupIndex}),i=JSON.stringify({top:t.top,left:t.left});this.cache.set(i,this.cache.get(n))}},{key:"_cleanAllowedPositions",value:function(){this._maxAllowedVerticalPosition=[],this._maxAllowedPosition=[]}},{key:"supportAllDayRow",value:function(){return!0}},{key:"keepOriginalHours",value:function(){return!1}},{key:"getSelectedCellData",value:function(){if(this.isVirtualScrolling())return this.virtualSelectionState.getSelectedCells();var e=this._getAllFocusedCells(),t=[];if(e.length>1)t=this._getMultipleCellsData(e);else{var n=this.getCellData((0,w.default)(e[0]));n&&t.push(n)}return t}},{key:"_getMultipleCellsData",value:function(e){for(var t=[],n=0;n<e.length;n++)t.push(this.getCellData((0,w.default)(e[n])));return t}},{key:"getCellData",value:function(e){var t,n=e[0];return n&&(t=this.isRenovatedRender()?this._getCellDataInRenovatedView(e):(0,S.data)(n,tt)),(0,O.extend)(!0,{},t)}},{key:"_getVirtualRowOffset",value:function(){return this.isVirtualScrolling()?this.virtualScrollingDispatcher.verticalScrollingState.virtualItemSizeBefore:0}},{key:"_getCellDataInRenovatedView",value:function(e){var t=e.parent().index();this.isVirtualScrolling()&&(t-=this.virtualScrollingDispatcher.topVirtualRowsCount);var n=e.index(),i=this.viewDataProvider,o=this._hasAllDayClass(e),a=i.getCellData(t,n,o);return a?{startDate:a.startDate,endDate:a.endDate,groups:a.groups,groupIndex:a.groupIndex,allDay:a.allDay,text:a.text}:void 0}},{key:"_getHorizontalMax",value:function(e){return e=this.isGroupedByDate()?this._getGroupCount()-1:e,this._groupedStrategy.getHorizontalMax(e)}},{key:"getCoordinatesByDate",value:function(e,t,n){t=t||0;var i,o=this.isVirtualScrolling()&&(!n||this._isVerticalGroupedWorkSpace());if(o){var a=this.viewDataProvider.findCellPositionInMap(t,e,n);if(!a)return;var r=this._dom_getDateCell(a);i=this._getCellPositionWithCache(r,a,t)}else i=this.calculateCellPositionByView(e,t,n);var s=this.getPositionShift(n?0:this.getTimeShift(e),n),l=this._getHorizontalMax(t,e);if(!i)throw L.default.Error("E1039");return{cellPosition:i.left+s.cellPosition,top:i.top+s.top,left:i.left+s.left,rowIndex:i.rowIndex,cellIndex:i.cellIndex,hMax:l,vMax:this.getVerticalMax(t),groupIndex:t}}},{key:"calculateCellPositionByView",value:function(e,t,n){var i=this.getCellIndexByDate(e,n),o=this._getCellPositionByIndex(i,t,n);return o&&(o.top-=this._getVirtualRowOffset()),o}},{key:"getVerticalMax",value:function(e){return this._groupedStrategy.getVerticalMax(e)}},{key:"_getOffsetByAllDayPanel",value:function(e){return this._groupedStrategy._getOffsetByAllDayPanel(e)}},{key:"_getGroupTop",value:function(e){return this._groupedStrategy._getGroupTop(e)}},{key:"isGroupedByDate",value:function(){return this.option("groupByDate")&&this._isHorizontalGroupedWorkSpace()&&this._getGroupCount()>0}},{key:"getCellIndexByDate",value:function(e,t){var n=t?864e5:this._getInterval(),i=this._getIntervalBetween(e,t),o=Math.floor(i/n);return t&&(o=this._updateIndex(o)),o<0&&(o=0),o}},{key:"getPositionShift",value:function(e,t){return{top:e*this.getCellHeight(),left:0,cellPosition:0}}},{key:"getTimeShift",value:function(e){var t=new Date(e),n=this.getCellDuration(),i=new Date(new Date(e).setHours(this.option("endDayHour"),0,0));e.getTime()<=i.getTime()&&t.setHours(this.option("startDayHour"),0,0,0);var o=e.getTime(),a=t.getTime(),r=this._firstViewDate.getTime();return o>r?(o-a)%n/n:0}},{key:"_isSkippedData",value:function(){return!1}},{key:"getCoordinatesByDateInGroup",value:function(e,t,n){var i=this,o=[];if(this._isSkippedData(e))return o;var a=this._getGroupCount()?this._getGroupIndexes(t):[0];return a.forEach(function(t){var a=i.getCoordinatesByDate(e,t,n);a&&o.push(a)}),o}},{key:"getDroppableCellIndex",value:function(){var e=this._getDroppableCell(),t=e.parent(),n=t.index();return n*t.find("td").length+e.index()}},{key:"getDataByDroppableCell",value:function(){var e=this.getCellData((0,w.default)(this._getDroppableCell())),t=e.allDay,n=e.startDate,i=n&&this.invoke("calculateAppointmentEndDate",t,n);return{startDate:n,endDate:i,allDay:t,groups:e.groups}}},{key:"getDateRange",value:function(){return[this.getStartViewDate(),this.getEndViewDateByEndDayHour()]}},{key:"getCellWidth",value:function(){var e=this;return this.cache.get("cellWidth",function(){var t=e._getCells().first().get(0);return t&&(0,A.getBoundingRect)(t).width})}},{key:"getCellMinWidth",value:function(){return it}},{key:"getRoundedCellWidth",value:function(e,t,n){if(e<0)return 0;var i=this.$element().find("."+this._getDateTableRowClass()).eq(0),o=0,a=i.find("."+Le),r=this._getCellCount()*e;n=n||this._getCellCount(),(0,B.isDefined)(t)||(t=r);for(var s=t;s<r+n;s++)o+=(0,A.getBoundingRect)((0,w.default)(a).eq(s).get(0)).width;return o/(r+n-t)}},{key:"getCellHeight",value:function(){var e=this,t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0],n=function(){var t=e._getCells().first().get(0);return t&&(0,A.getBoundingRect)(t).height};return t?this.cache.get("cellHeight",n):n()}},{key:"getAllDayHeight",value:function(){var e=this._getCells(!0).first().get(0);return this._isShowAllDayPanel()?e&&(0,A.getBoundingRect)(e).height||0:0}},{key:"getAllDayOffset",value:function(){return this._groupedStrategy.getAllDayOffset()}},{key:"getMaxAllowedPosition",value:function(){if(0===this._maxAllowedPosition.length){var e=this.option("rtlEnabled");this._maxAllowedPosition=[],this._$dateTable.find("tr:not(.".concat(Ne,")")).first().find("td:nth-child(".concat(this._getCellCount(),"n)")).each(function(t,n){var i=(0,w.default)(n).position().left;e||(i+=(0,A.getBoundingRect)(n).width),this._maxAllowedPosition.push(Math.round(i))}.bind(this))}return this._maxAllowedPosition}},{key:"getMaxAllowedVerticalPosition",value:function(e){return this.isVirtualScrolling()?this.getMaxAllowedVerticalPositionVirtual(e):this.getMaxAllowedVerticalPositionStandard(e)}},{key:"getMaxAllowedVerticalPositionStandard",value:function(e){var t=this;if(0===this._maxAllowedVerticalPosition.length){var n=this._getRowCount();this._$dateTable.find("tr:not(.".concat(Ne,"):nth-child(").concat(n,"n)")).each(function(e,n){var i=(0,w.default)(n).position().top+(0,A.getBoundingRect)(n).height;t._maxAllowedVerticalPosition.push(Math.round(i))})}return this._maxAllowedVerticalPosition[e]}},{key:"getMaxAllowedVerticalPositionVirtual",value:function(e){var t=this,n=function(n){var i=t._$dateTable.find("tr:not(.".concat(Ne,")")).get(n),o=(0,w.default)(i).position().top+(0,A.getBoundingRect)(i).height;t.isGroupedAllDayPanel()&&(o-=(e+1)*t.getAllDayHeight()),t._maxAllowedVerticalPosition[e]=Math.round(o)};if(!this._maxAllowedVerticalPosition[e]){var i=this.viewDataProvider.getLasGroupCellPosition(e),o=i.rowIndex;n(o)}return this._maxAllowedVerticalPosition[e]}},{key:"getFixedContainer",value:function(){return this._$fixedContainer}},{key:"getAllDayContainer",value:function(){return this._$allDayContainer}},{key:"getCellIndexByCoordinates",value:function(e,t){var n=this._getTotalCellCount(this._getGroupCount()),i=Math.floor(this._getWorkSpaceWidth()/n),o=t?this.getAllDayHeight():this.getCellHeight(),a=this._isRTL()||this.option("crossScrollingEnabled")?0:this.getWorkSpaceLeftOffset(),r=Math.floor(Math.floor(e.top)/Math.floor(o)),s=Math.floor((e.left+5-a)/i);return this._isRTL()&&(s=n-s-1),n*r+s}},{key:"getStartViewDate",value:function(){return this._firstViewDate}},{key:"getEndViewDate",value:function(){var e=this.getDateOfLastViewCell(),t=this.calculateEndViewDate(e);return this._adjustEndViewDateByDaylightDiff(e,t)}},{key:"getEndViewDateByEndDayHour",value:function(){var e=this.getDateOfLastViewCell(),t=D.default.dateTimeFromDecimal(this.option("endDayHour")),n=new Date(e.setHours(t.hours,t.minutes));return this._adjustEndViewDateByDaylightDiff(e,n)}},{key:"calculateEndViewDate",value:function(e){return new Date(e.getTime()+this.getCellDuration())}},{key:"_adjustEndViewDateByDaylightDiff",value:function(e,t){var n=Y.default.getDaylightOffsetInMs(e,t),i=new Date(t.getTime()-n);return new Date(i.getTime()-se("minute"))}},{key:"getDateOfLastViewCell",value:function(){var e=this._getRowCount()-1,t=this._getCellCount();return this.isGroupedByDate()?t=t*this._getGroupCount()-1:t-=1,this._getDateByCellIndexes(e,t,!0)}},{key:"getCellDuration",value:function(){return 36e5*this.option("hoursInterval")}},{key:"getIntervalDuration",value:function(e){return e?se("day"):this.getCellDuration()}},{key:"getVisibleDayDuration",value:function(){return this.option("hoursInterval")*this._getCellCountInDay()*at}},{key:"getGroupBounds",value:function(e){var t=this._getCellCount(),n=this._getCells(),i=this.getCellWidth(),o=this._groupedStrategy.getGroupBoundsOffset(t,n,i,e);if(this._isRTL()){var a=o.left;o.left=o.right-2*i,o.right=a+2*i}return o}},{key:"needRecalculateResizableArea",value:function(){return this._isVerticalGroupedWorkSpace()&&0!==this.getScrollable().scrollTop()}},{key:"getCellDataByCoordinates",value:function(e,t){var n=this,i=JSON.stringify({top:e.top,left:e.left});return this.cache.get(i,function(){var i=n._getCells(t),o=n.getCellIndexByCoordinates(e,t),a=i.eq(o);return n.getCellData(a)})}},{key:"getVisibleBounds",value:function(){var e={},t=this.getScrollable().$element(),n=this.getCellHeight(),i=this.getScrollableScrollTop()/n,o=i+t.height()/n;return e.top={hours:Math.floor(i*this.option("hoursInterval"))+this.option("startDayHour"),minutes:i%2?30:0},e.bottom={hours:Math.floor(o*this.option("hoursInterval"))+this.option("startDayHour"),minutes:Math.floor(o)%2?30:0},e}},{key:"updateScrollPosition",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=this.option("observer"),o=i.timeZoneCalculator.createDate(e,{path:"toGrid"}),a=n&&this.supportAllDayRow()&&this._isShowAllDayPanel();this.needUpdateScrollPosition(o,t,a)&&this.scrollTo(o,t,a,!1)}},{key:"needUpdateScrollPosition",value:function(e,t,n){var i=this,o=this._getCellsInViewport(n),a=this._isGroupsSpecified(t)?this._getGroupIndexByResourceId(t):0,r=e.getTime(),s=D.default.trimTime(e).getTime();return o.reduce(function(e,t){var o=i.getCellData(t),l=o.startDate,u=o.endDate,c=o.groupIndex,d=l.getTime(),h=u.getTime();return(!(!n&&d<=r&&r<h||n&&s===d)||a!==c)&&e},!0)}},{key:"_getCellsInViewport",value:function(e){var t=this.getScrollable().$element(),n=this.getCellHeight(),i=this.getCellWidth(),o=this._getTotalCellCount(this._getGroupCount()),a=this.getScrollableScrollTop(),r=this.getScrollableScrollLeft(),s=a/n;this.isVirtualScrolling()&&(s-=this.virtualScrollingDispatcher.topVirtualRowsCount);var l=Math.floor(s);a%n!==0&&(l+=1);var u=r/i,c=Math.floor(u);r%i!==0&&(c+=1);var d=Math.floor(s+t.height()/n),h=Math.floor(u+t.width()/i),f=this._getAllCells(e),p=[];return f.each(function(e){var t=(0,w.default)(this),n=e%o,i=e/o;c<=n&&n<h&&l<=i&&i<d&&p.push(t)}),p}},{key:"getGroupWidth",value:function(e){var t=this._getCellCount()*this.getCellWidth(),n=this.getMaxAllowedPosition(),i=n[e];return n.length&&i&&(t=this._isRTL()?i-n[e+1]:0===e?i:i-n[e-1]),t}},{key:"scrollToTime",value:function(e,t,n){if(this._isValidScrollDate(n)){var i=this._getScrollCoordinates(e,t,n),o=this.getScrollable();o.scrollBy({top:i.top-o.scrollTop(),left:0})}}},{key:"scrollTo",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3];if(this._isValidScrollDate(e,i)){var o=this._getGroupCount()&&t?this._getGroupIndexByResourceId(t):0,a=n&&this.supportAllDayRow()&&this._isShowAllDayPanel(),r=this._getScrollCoordinates(e.getHours(),e.getMinutes(),e,o,a),s=this.getScrollable(),l=s.$element(),u=this.option("rtlEnabled")?this.getCellWidth():0,c=l.height(),d=l.width(),h=this.getCellWidth(),f=this.getCellHeight(),p=(d-h)/2,g=(c-f)/2,_=r.left-s.scrollLeft()-p-u,m=r.top-s.scrollTop()-g;a&&!this._isVerticalGroupedWorkSpace()&&(m=0),this.option("templatesRenderAsynchronously")?setTimeout(function(){s.scrollBy({left:_,top:m})}):s.scrollBy({left:_,top:m})}}},{key:"_isValidScrollDate",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=this.getStartViewDate(),i=this.getEndViewDate();return!(e<n||e>i)||(t&&L.default.log("W1008",e),!1)}},{key:"getDistanceBetweenCells",value:function(e,t){var n=0;return this.$element().find("."+this._getDateTableRowClass()).first().find("."+Le).each(function(i){return i<e||i>t||void(n+=(0,A.getBoundingRect)(this).width)}),n}},{key:"needApplyCollectorOffset",value:function(){
return!1}},{key:"initDragBehavior",value:function(e){!this.dragBehavior&&e&&(this.dragBehavior=new K.default(e),this._createDragBehavior(this.getWorkArea()),this._createDragBehavior(this.getAllDayContainer()),this._createDragBehavior(this._$allDayPanel))}},{key:"_createDragBehavior",value:function(e){var t=function(e,t){return t._getItemData(e)},n=function(e){return e.data(U.APPOINTMENT_SETTINGS_KEY)};this._createDragBehaviorBase(e,t,n)}},{key:"_createDragBehaviorBase",value:function(e,t,n){var i,o,r=this,s=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},u=s.isSetCursorOffset,c=a(s,["isSetCursorOffset"]),d=this.dragBehavior;d.addTo(e,l({container:this.$element().find(".".concat(U.FIXED_CONTAINER_CLASS)),dragTemplate:function(){return i},onDragStart:function(e){var a=e.cancel,s=e.event,l=(0,w.default)(e.itemElement),u=e.component._appointments;o=t(e.itemElement,u);var c=n(l,e);o&&!o.disabled&&(s.data=s.data||{},a||(c.isCompact||d.updateDragSource(o,c),i=r._createDragAppointment(o,c,u),s.data.itemElement=i,s.data.initialPosition=(0,X.locate)((0,w.default)(i)),s.data.itemData=o,s.data.itemSettings=c,d.onDragStart(s.data),(0,X.resetPosition)((0,w.default)(i))))},onDragEnd:function(e){var t;o&&!o.disabled&&d.onDragEnd(e),null===(t=i)||void 0===t?void 0:t.remove()},cursorOffset:u?function(){var e=(0,w.default)(i);return{x:e.width()/2,y:e.height()/2}}:void 0},c))}},{key:"_createDragAppointment",value:function(e,t,n){var i=n.option("items").length;t.isCompact=!1,t.virtual=!1;var o=n._renderItem(i,{itemData:e,settings:[t]});return o[0]}},{key:"_isApplyCompactAppointmentOffset",value:function(){return this._supportCompactDropDownAppointments()}},{key:"_supportCompactDropDownAppointments",value:function(){return!0}},{key:"_formatWeekday",value:function(e){return ut(e)}},{key:"_formatWeekdayAndDay",value:function(e){return ut(e)+" "+R.default.format(e,"day")}},{key:"removeDroppableCellClass",value:function(e){(e||this._getDroppableCell()).removeClass(We)}},{key:"_getCoordinatesByCell",value:function(e){var t=e.index(),n=e.parent().index(),i=this._hasAllDayClass(e),o=this._isVerticalGroupedWorkSpace();return!this.isVirtualScrolling()||i&&!o||(n-=this.virtualScrollingDispatcher.topVirtualRowsCount),{rowIndex:n,columnIndex:t}}},{key:"_isShowAllDayPanel",value:function(){return this.option("showAllDayPanel")}},{key:"updateAppointments",value:function(){var e;this.invoke("renderAppointments"),null===(e=this.dragBehavior)||void 0===e?void 0:e.updateDragSource()}},{key:"viewDataProvider",get:function(){return this._viewDataProvider||(this._viewDataProvider=new J.default(this)),this._viewDataProvider}},{key:"cache",get:function(){return oe.cache}},{key:"virtualSelectionState",get:function(){return this._virtualSelectionState||(this._virtualSelectionState=new ie.default(this.viewDataProvider)),this._virtualSelectionState}}]),t}(Z.default),dt=ct;t.default=dt,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r="dx-scheduler-last-group-cell",s="dx-scheduler-first-group-cell",l=function(){function e(t){i(this,e),this._workSpace=t}return a(e,[{key:"getLastGroupCellClass",value:function(){return r}},{key:"getFirstGroupCellClass",value:function(){return s}},{key:"_getOffsetByAllDayPanel",value:function(){return 0}},{key:"_getGroupTop",value:function(){return 0}}]),e}(),u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.cache=t.Cache=void 0;var r=n(1),s=function(){function e(){i(this,e),this._init()}return a(e,[{key:"_init",value:function(){this._cache=new Map}},{key:"clear",value:function(){this._init()}},{key:"get",value:function(e,t){return!this._cache.has(e)&&t&&this.set(e,t()),this._cache.get(e)}},{key:"set",value:function(e,t){(0,r.isDefined)(t)&&this._cache.set(e,t)}},{key:"size",get:function(){return this._cache.size}}]),e}();t.Cache=s;var l=new s;t.cache=l},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(24),l=i(n(2)),u="dx-scheduler-date-time-shader",c=function(){function e(t){o(this,e),this._workSpace=t,this._$container=this._workSpace._dateTableScrollable.$content()}return r(e,[{key:"render",value:function(){var e=this;this.initShaderElements(),this.renderShader(),this.applyShaderMargin(this._$shader),this._shader.forEach(function(t,n){e._$container.append(t)})}},{key:"initShaderElements",value:function(){this._$shader=this.createShader(),this._shader=[],this._shader.push(this._$shader)}},{key:"renderShader",value:function(){}},{key:"applyShaderMargin",value:function(e){e&&this._workSpace.option("crossScrollingEnabled")&&(e.css("marginTop",-(0,s.getBoundingRect)(this._$container.get(0)).height),e.css("height",(0,s.getBoundingRect)(this._$container.get(0)).height))}},{key:"createShader",value:function(){return(0,l.default)("<div>").addClass(u)}},{key:"clean",value:function(){this._$container&&this._$container.find("."+u).remove()}}]),e}(),d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,_.useCallback)(function(){var t=(e.className,e.dataCellTemplate,e.isVerticalGroupOrientation,e.viewData,u(e,["className","dataCellTemplate","isVerticalGroupOrientation","viewData"]));return t},[e]);return m({props:s(s({},e),{},{dataCellTemplate:y(e.dataCellTemplate)}),restAttributes:t()})}t.AllDayPanelTableBody=h,t.AllDayPanelTableBodyProps=t.viewFunction=void 0;var f=n(227),p=n(833),g=a(n(51)),_=n(53),m=function(e){return g.h(f.Row,d({},e.restAttributes,{className:"dx-scheduler-all-day-table-row ".concat(e.props.className)}),e.props.viewData.map(function(t){var n=t.endDate,i=t.groupIndex,o=t.groups,a=t.index,r=t.isFirstGroupCell,s=t.isLastGroupCell,l=t.key,u=t.startDate;return g.h(p.AllDayPanelCell,{isFirstGroupCell:!e.props.isVerticalGroupOrientation&&r,isLastGroupCell:!e.props.isVerticalGroupOrientation&&s,startDate:u,endDate:n,groups:o,groupIndex:i,index:a,dataCellTemplate:e.props.dataCellTemplate,key:l})}))};t.viewFunction=m;var v={viewData:[],isVerticalGroupOrientation:!1,className:""};t.AllDayPanelTableBodyProps=v;var y=function(e){return e&&(e.defaultProps?function(t){return g.h(e,d({},t))}:e)};h.defaultProps=s({},v)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function l(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?s(Object(n),!0).forEach(function(t){d(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):s(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function h(){return h=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},h.apply(this,arguments)}function f(e){var t=(0,m.useCallback)(function(){return p.default.format("dxScheduler-allDay")},[]),n=(0,m.useCallback)(function(){return(0,g.combineClasses)(d({"dx-scheduler-all-day-title":!0,"dx-scheduler-all-day-title-hidden":!e.visible},e.className,!!e.className))},[e.visible,e.className]),i=(0,m.useCallback)(function(){var t=(e.className,e.visible,u(e,["className","visible"]));return t},[e]);return v({props:l({},e),text:t(),classes:n(),restAttributes:i()})}t.AllDayPanelTitle=f,t.AllDayPanelTitleProps=t.viewFunction=void 0;var p=r(n(10)),g=n(226),_=a(n(51)),m=n(53),v=function(e){return _.h("div",h({},e.restAttributes,{className:e.classes}),e.text)};t.viewFunction=v;var y={className:"",visible:!0};t.AllDayPanelTitleProps=y,f.defaultProps=l({},y)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(316)),y=n(24),b="dx-scheduler-timeline-week",x=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getElementClass",value:function(){return b}},{key:"_getCellCount",value:function(){return l(_(t.prototype),"_getCellCount",this).call(this)*this._getWeekDuration()}},{key:"_getHeaderPanelCellWidth",value:function(e){return(0,y.getBoundingRect)(e.children().first().get(0)).width}},{key:"_getWeekDuration",value:function(){return 7}},{key:"_needRenderWeekHeader",value:function(){return!0}},{key:"_incrementDate",value:function(e){e.setDate(e.getDate()+1)}}]),t}(v.default);t.default=x,(0,m.default)("dxSchedulerTimelineWeek",x),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(20)),l=1,u=6,c=0,d=function(){function e(){o(this,e)}return r(e,null,[{key:"isDataOnWeekend",value:function(e){var t=e.getDay();return t===u||t===c}},{key:"getFirstDayOfWeek",value:function(e){return e||l}},{key:"getWeekendsCount",value:function(e){return 2*Math.floor(e/7)}},{key:"getFirstViewDate",value:function(e,t){var n=s.default.getFirstWeekDate(e,t);return s.default.normalizeDateByWeek(n,e)}}]),e}(),h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(3),y=o(n(317)),b=o(n(32)),x=o(n(86)),w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getCellsBetween",value:function(e,n){if(this._hasAllDayClass(n))return l(_(t.prototype),"_getCellsBetween",this).call(this,e,n);for(var i=this._getCells(),o=e.index(),a=e.parent().index(),r=n.index(),s=n.parent().index(),u=this._getGroupCount(),c=u>0?this._getTotalCellCount(u):this._getCellCount(),d=this._getTotalRowCount(u),h=[],f=0;f<c;f++)for(var p=0;p<d;p++){var g=i.get(c*p+f);h.push(g)}var y=this.getCellData(n).groups,b=this.option("showAllDayPanel")&&this._isVerticalGroupedWorkSpace()?this._getGroupIndexByResourceId(y)+1:0,x=d*o+a-b,w=d*r+s-b;if(x>w){var C=x;x=w,w=C}if(i=(0,m.default)(h).slice(x,w+1),this._getGroupCount()){var k=[],S=this._getGroupIndexByCell(e);(0,v.each)(i,function(e,t){var n=this._getGroupIndexByCell((0,m.default)(t));S===n&&k.push(t)}.bind(this)),i=(0,m.default)(k)}return i}},{key:"_getCellFromNextColumn",value:function(e,n){var i=l(_(t.prototype),"_getCellFromNextColumn",this).call(this,e,n),o=this._$focusedCell;return o.parent().index()!==i.parent().index()&&n&&(i=o),i}},{key:"_getFormat",value:function(){return this._formatWeekdayAndDay}},{key:"renovatedRenderSupported",value:function(){return!0}},{key:"generateRenderOptions",value:function(){var e=this,n=x.default.getDateWithoutTimezoneChange(this.getStartViewDate()),i=function(t,i){var o=t%e._getRowCount();return o%2===0&&0===i?b.default.format(e._getTimeCellDateCore(n,t),"shorttime"):""},o=l(_(t.prototype),"generateRenderOptions",this).call(this);return o.cellDataGetters.push(function(e,t,n){return{value:{text:i(t,n)}}}),o}}]),t}(y.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,m.useCallback)(function(){return"dx-scheduler-date-table ".concat(e.className)},[e.className]),n=(0,m.useCallback)(function(){var t=e.viewData;return!!t.isVirtual},[e.viewData]),i=(0,m.useCallback)(function(){return e.viewData.topVirtualRowHeight||0},[e.viewData]),o=(0,m.useCallback)(function(){return e.viewData.bottomVirtualRowHeight||0},[e.viewData]),a=(0,m.useCallback)(function(){return e.viewData.groupedData[0].dateTable[0].length},[e.viewData]),s=(0,m.useCallback)(function(){var t=(e.className,e.dataCellTemplate,e.groupOrientation,e.viewData,e.viewType,r(e,["className","dataCellTemplate","groupOrientation","viewData","viewType"]));return t},[e]);return v({props:u(u({},e),{},{dataCellTemplate:b(e.dataCellTemplate)}),classes:t(),isVirtual:n(),topVirtualRowHeight:i(),bottomVirtualRowHeight:o(),virtualCellsCount:a(),restAttributes:s()})}t.DateTableLayoutBase=h,t.DateTableLayoutBaseProps=t.viewFunction=void 0;var f=n(313),p=n(847),g=n(170),_=a(n(51)),m=n(53),v=function(e){return _.h(f.Table,d({},e.restAttributes,{isVirtual:e.isVirtual,topVirtualRowHeight:e.topVirtualRowHeight,bottomVirtualRowHeight:e.bottomVirtualRowHeight,virtualCellsCount:e.virtualCellsCount,className:e.classes}),_.h(p.DateTableBody,{viewType:e.props.viewType,viewData:e.props.viewData,dataCellTemplate:e.props.dataCellTemplate}))};t.viewFunction=v;var y=u({},g.LayoutProps);t.DateTableLayoutBaseProps=y;var b=function(e){return e&&(e.defaultProps?function(t){return _.h(e,d({},t))}:e)};h.defaultProps=u({},y)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(20)),y=o(n(32)),b=o(n(531)),x=o(n(850)),w="dx-scheduler-work-space-week",C=v.default.dateToMilliseconds,k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getElementClass",value:function(){return w}},{key:"_getRowCount",value:function(){return this._getCellCountInDay()}},{key:"_getCellCount",value:function(){return 7*this.option("intervalCount")}},{key:"_getDateByIndex",value:function(e){var t=new Date(this._firstViewDate);return t.setDate(this._firstViewDate.getDate()+e),t}},{key:"_getStartViewDate",value:function(){return v.default.getFirstWeekDate(this.option("startDate"),this._firstDayOfWeek()||y.default.firstDayOfWeekIndex())}},{key:"_getIntervalDuration",value:function(){return 7*C("day")*this.option("intervalCount")}},{key:"getPositionShift",value:function(e,n){return!n&&this.invoke("isAdaptive")&&0===this.invoke("getMaxAppointmentCountPerCellByType")?{top:0,left:0,cellPosition:0}:l(_(t.prototype),"getPositionShift",this).call(this,e,n)}},{key:"_isApplyCompactAppointmentOffset",value:function(){return(!this.invoke("isAdaptive")||0!==this.invoke("getMaxAppointmentCountPerCellByType"))&&l(_(t.prototype),"_isApplyCompactAppointmentOffset",this).call(this)}},{key:"renderRDateTable",value:function(){this.renderRComponent(this._$dateTable,x.default,"renovatedDateTable",{viewData:this.viewDataProvider.viewData,dataCellTemplate:this.option("dataCellTemplate")})}}]),t}(b.default);(0,m.default)("dxSchedulerWorkSpaceWeek",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(5)),y=n(0),b=n(59),x=n(4),w=o(n(16)),C=o(n(477)),k=o(n(535)),S=n(6),D="dx-filemanager-dirs-tree",I="dx-filemanager-focused-item",T="dx-filemanager-dirs-tree-item-text",O="dx-treeview-item",E=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;this._initActions(),this._getCurrentDirectory=this.option("getCurrentDirectory"),this._createFileActionsButton=x.noop,this._storeExpandedState=this.option("storeExpandedState")||!1;var t=(0,m.default)("<div>").addClass(D).appendTo(this.$element()),n={dataStructure:"plain",rootValue:"",createChildren:this._onFilesTreeViewCreateSubDirectories.bind(this),itemTemplate:this._createFilesTreeViewItemTemplate.bind(this),keyExpr:"getInternalKey",parentIdExpr:"parentDirectory.getInternalKey",displayExpr:function(e){return e.getDisplayName()},hasItemsExpr:"fileItem.hasSubDirectories",onItemClick:function(t){return e._actions.onDirectoryClick(t)},onItemExpanded:function(t){return e._onFilesTreeViewItemExpanded(t)},onItemCollapsed:function(t){return e._onFilesTreeViewItemCollapsed(t)},onItemRendered:function(t){return e._onFilesTreeViewItemRendered(t)},onContentReady:function(){return e._actions.onFilesTreeViewContentReady()}};this._contextMenu&&(this._contextMenu.option("onContextMenuHidden",function(){return e._onContextMenuHidden()}),n.onItemContextMenu=function(t){return e._onFilesTreeViewItemContextMenu(t)},this._createFileActionsButton=function(t,n){return e._createComponent(t,k.default,n)}),this._filesTreeView=this._createComponent(t,C.default,n),v.default.on(t,"click",function(){return e._actions.onClick()})}},{key:"_initActions",value:function(){this._actions={onClick:this._createActionByOption("onClick"),onDirectoryClick:this._createActionByOption("onDirectoryClick"),onFilesTreeViewContentReady:this._createActionByOption("onFilesTreeViewContentReady")}}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this);var e=this;setTimeout(function(){e._updateFocusedElement()})}},{key:"_onFilesTreeViewCreateSubDirectories",value:function(e){var t=this.option("getDirectories"),n=e&&e.itemData||null;return t&&t(n,!0)}},{key:"_onFilesTreeViewItemRendered",value:function(e){var t=e.itemData,n=this._getCurrentDirectory();n&&n.fileItem.equals(t.fileItem)&&this._updateFocusedElement()}},{key:"_onFilesTreeViewItemExpanded",value:function(e){var t=e.itemData;this._storeExpandedState&&(t.expanded=!0)}},{key:"_onFilesTreeViewItemCollapsed",value:function(e){var t=e.itemData;this._storeExpandedState&&(t.expanded=!1)}},{key:"_createFilesTreeViewItemTemplate",value:function(e,t,n){var i=this,o=(0,m.default)(n),a=o.closest(this._filesTreeViewItemSelector);a.data("item",e);var r=(0,b.getImageContainer)(e.icon),s=(0,m.default)("<span>").text(e.getDisplayName()).addClass(T),l=(0,m.default)("<div>");o.append(r,s,l),this._createFileActionsButton(l,{onClick:function(e){return i._onFileItemActionButtonClick(e)}})}},{key:"_onFilesTreeViewItemContextMenu",value:function(e){var t=e.itemElement,n=e.event;n.preventDefault();var i=(0,m.default)(t).data("item");this._contextMenu.showAt([i],t,n)}},{key:"_onFileItemActionButtonClick",value:function(e){var t=e.component,n=e.element,i=e.event;i.stopPropagation();var o=t.$element().closest(this._filesTreeViewItemSelector),a=o.data("item");this._contextMenu.showAt([a],n),this._activeFileActionsButton=t,this._activeFileActionsButton.setActive(!0);
}},{key:"_onContextMenuHidden",value:function(){this._activeFileActionsButton&&this._activeFileActionsButton.setActive(!1)}},{key:"toggleNodeDisabledState",value:function(e,t){var n=this._getNodeByKey(e);if(n){var i=this._filesTreeView.option("items"),o=i.map(function(e){return e.getInternalKey()}).indexOf(n.getInternalKey());o!==-1&&this._filesTreeView.option("items[".concat(o,"].disabled"),t)}}},{key:"_updateFocusedElement",value:function(){var e=this._getCurrentDirectory(),t=this._getItemElementByKey(null===e||void 0===e?void 0:e.getInternalKey());this._$focusedElement&&this._$focusedElement.toggleClass(I,!1),this._$focusedElement=t||(0,m.default)(),this._$focusedElement.toggleClass(I,!0)}},{key:"_getNodeByKey",value:function(e){var t;return null===(t=this._filesTreeView)||void 0===t?void 0:t._getNode(e)}},{key:"_getItemElementByKey",value:function(e){var t=this._getNodeByKey(e);if(t){var n=this._filesTreeView._getNodeElement(t);if(n)return n.children(this._filesTreeViewItemSelector)}return null}},{key:"_getDefaultOptions",value:function(){return(0,y.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{storeExpandedState:!1,initialFolder:null,contextMenu:null,getItems:null,getCurrentDirectory:null,onDirectoryClick:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"storeExpandedState":this._storeExpandedState=this.option(n);break;case"getItems":case"rootFolderDisplayName":case"initialFolder":case"contextMenu":this.repaint();break;case"getCurrentDirectory":this.getCurrentDirectory=this.option(n);break;case"onClick":case"onDirectoryClick":case"onFilesTreeViewContentReady":this._actions[n]=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"toggleDirectoryExpandedState",value:function(e,t){var n=new S.Deferred,i=this._getNodeByKey(null===e||void 0===e?void 0:e.getInternalKey());if(!i)return n.reject().promise();if(i.expanded===t||i.itemsLoaded&&!i.fileItem.hasSubDirectories)return n.resolve().promise();var o=t?"expandItem":"collapseItem";return this._filesTreeView[o](e.getInternalKey())}},{key:"refresh",value:function(){this._$focusedElement=null,this._filesTreeView.option("dataSource",[])}},{key:"updateCurrentDirectory",value:function(){this._disposed||(this._updateFocusedElement(),this._storeExpandedState&&this._updateExpandedStateToCurrentDirectory())}},{key:"_updateExpandedStateToCurrentDirectory",value:function(){return this.toggleDirectoryExpandedStateRecursive(this._getCurrentDirectory(),!0)}},{key:"toggleDirectoryExpandedStateRecursive",value:function(e,t){for(var n=[],i=e;i;i=i.parentDirectory)n.unshift(i);return this.toggleDirectoryLineExpandedState(n,t)}},{key:"toggleDirectoryLineExpandedState",value:function(e,t){var n=this;return e.length?this.toggleDirectoryExpandedState(e.shift(),t).then(function(){return n.toggleDirectoryLineExpandedState(e,t)}):(new S.Deferred).resolve().promise()}},{key:"_filesTreeViewItemSelector",get:function(){return".".concat(O)}},{key:"_contextMenu",get:function(){return this.option("contextMenu")}}]),t}(w.default),A=E;t.default=A,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(16)),b=o(n(38)),x="dx-filemanager-file-actions-button",w="dx-filemanager-file-actions-button-activated",C="dx-state-active",k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;this._createClickAction();var n=(0,m.default)("<div>");this.$element().append(n).addClass(x),this._button=this._createComponent(n,b.default,{icon:"overflow",stylingMode:"text",onClick:function(t){return e._raiseClick(t)}}),l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"_createClickAction",value:function(){this._clickAction=this._createActionByOption("onClick")}},{key:"_raiseClick",value:function(e){this._clickAction(e)}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{cssClass:"",onClick:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"cssClass":this.repaint();break;case"onClick":this._createClickAction();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"setActive",value:function(e){var t=this;this.$element().toggleClass(w,e),setTimeout(function(){return t._button.$element().toggleClass(C,e)})}}]),t}(y.default),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=n(0),k=n(6),S=n(142),D=n(8),I=o(n(5)),T=n(59),O=o(n(13)),E=o(n(123)),A=o(n(16)),P="dx-filemanager-files-view",R="dxFileManager_open",M=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"_init",value:function(){this._initActions(),this._lockFocusedItemProcessing=!1,this._focusedItemKey=this.option("focusedItemKey"),p(w(t.prototype),"_init",this).call(this)}},{key:"_initMarkup",value:function(){this.$element().addClass(P);var e=(0,D.addNamespace)(S.name,R);I.default.on(this.$element(),e,this._getItemSelector(),this._onItemDblClick.bind(this)),p(w(t.prototype),"_initMarkup",this).call(this)}},{key:"_initActions",value:function(){this._actions={onError:this._createActionByOption("onError"),onSelectionChanged:this._createActionByOption("onSelectionChanged"),onFocusedItemChanged:this._createActionByOption("onFocusedItemChanged"),onSelectedItemOpened:this._createActionByOption("onSelectedItemOpened"),onContextMenuShowing:this._createActionByOption("onContextMenuShowing")}}},{key:"_getDefaultOptions",value:function(){return(0,C.extend)(p(w(t.prototype),"_getDefaultOptions",this).call(this),{selectionMode:"single",selectedItemKeys:[],focusedItemKey:void 0,contextMenu:null,getItems:null,getItemThumbnail:null,onError:null,onSelectionChanged:null,onFocusedItemChanged:null,onSelectedItemOpened:null,onContextMenuShowing:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"selectionMode":case"contextMenu":case"getItems":case"getItemThumbnail":this.repaint();break;case"selectedItemKeys":this._setSelectedItemKeys(e.value);break;case"focusedItemKey":this._lockFocusedItemProcessing||this._setFocusedItemKey(e.value);break;case"onError":case"onSelectedItemOpened":case"onSelectionChanged":case"onFocusedItemChanged":case"onContextMenuShowing":this._actions[n]=this._createActionByOption(n);break;default:p(w(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_getItems",value:function(){var e=this;return this._getItemsInternal().done(function(t){e._itemCount=t.length;var n=e._findParentDirectoryItem(t);e._hasParentDirectoryItem=!!n,e._parentDirectoryItemKey=n?n.fileItem.key:null})}},{key:"_getItemsInternal",value:function(){var e=this.option("getItems"),t=e?e():[];return(0,k.when)(t)}},{key:"_raiseOnError",value:function(e){this._actions.onError({error:e})}},{key:"_raiseSelectionChanged",value:function(e){this._actions.onSelectionChanged(e)}},{key:"_raiseFocusedItemChanged",value:function(e){this._actions.onFocusedItemChanged(e)}},{key:"_raiseSelectedItemOpened",value:function(e){this._actions.onSelectedItemOpened({fileItemInfo:e})}},{key:"_raiseContextMenuShowing",value:function(){this._actions.onContextMenuShowing()}},{key:"_tryRaiseSelectionChanged",value:function(e){var t=this,n=e.selectedItemInfos,i=e.selectedItems,o=e.selectedItemKeys,a=e.currentSelectedItemKeys,r=e.currentDeselectedItemKeys,s=this._findParentDirectoryItem(this.getSelectedItems());s&&this._deselectItem(s);var l=!this._hasParentDirectoryItem;l=l||this._hasValidKeys(a)||this._hasValidKeys(r),l&&(n=this._filterOutItemByPredicate(n,function(e){return e.fileItem.key===t._parentDirectoryItemKey}),i=this._filterOutParentDirectory(i),o=this._filterOutParentDirectoryKey(o,!0),a=this._filterOutParentDirectoryKey(a,!0),r=this._filterOutParentDirectoryKey(r,!0),this._raiseSelectionChanged({selectedItemInfos:n,selectedItems:i,selectedItemKeys:o,currentSelectedItemKeys:a,currentDeselectedItemKeys:r}))}},{key:"_onFocusedItemChanged",value:function(e){this._focusedItemKey!==e.itemKey&&(this._focusedItemKey=e.itemKey,this._lockFocusedItemProcessing=!0,this.option("focusedItemKey",e.itemKey),this._lockFocusedItemProcessing=!1,this._raiseFocusedItemChanged(e))}},{key:"_getItemThumbnail",value:function(e){var t=this.option("getItemThumbnail");return t?t(e):{thumbnail:""}}},{key:"_getItemThumbnailContainer",value:function(e){var t=this._getItemThumbnail(e),n=t.thumbnail,i=t.cssClass,o=(0,T.getImageContainer)(n).addClass(this._getItemThumbnailCssClass());return i&&o.addClass(i),o}},{key:"_getItemThumbnailCssClass",value:function(){return""}},{key:"_getItemSelector",value:function(){}},{key:"_onItemDblClick",value:function(e){}},{key:"_isDesktop",value:function(){return"desktop"===O.default.real().deviceType}},{key:"_showContextMenu",value:function(e,t,n,i){this._contextMenu.showAt(e,t,n,i)}},{key:"_findParentDirectoryItem",value:function(e){for(var t=0;t<e.length;t++){var n=e[t];if(this._isParentDirectoryItem(n))return n}return null}},{key:"_getFileItemsForContextMenu",value:function(e){var t=this.getSelectedItems();return this._isParentDirectoryItem(e)&&t.push(e),t}},{key:"_isParentDirectoryItem",value:function(e){return e.fileItem.isParentFolder}},{key:"_hasValidKeys",value:function(e){return e.length>1||1===e.length&&e[0]!==this._parentDirectoryItemKey}},{key:"_filterOutParentDirectory",value:function(e,t){var n=this;return this._filterOutItemByPredicate(e,function(e){return e.key===n._parentDirectoryItemKey},t)}},{key:"_filterOutParentDirectoryKey",value:function(e,t){var n=this;return this._filterOutItemByPredicate(e,function(e){return e===n._parentDirectoryItemKey},t)}},{key:"_filterOutItemByPredicate",value:function(e,t,n){for(var i=e,o=-1,r=0;r<e.length;r++)if(t(e[r])){o=r;break}return o!==-1&&(n&&(i=a(e)),i.splice(o,1)),i}},{key:"_isMultipleSelectionMode",value:function(){return"multiple"===this.option("selectionMode")}},{key:"_deselectItem",value:function(e){}},{key:"_setSelectedItemKeys",value:function(e){}},{key:"_setFocusedItemKey",value:function(e){}},{key:"_createDataSource",value:function(){return{store:new E.default({key:"fileItem.key",load:this._getItems.bind(this)})}}},{key:"getSelectedItems",value:function(){}},{key:"clearSelection",value:function(){}},{key:"selectItem",value:function(){}},{key:"_contextMenu",get:function(){return this.option("contextMenu")}}]),t}(A.default),B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(11)),b=o(n(5)),x=o(n(25)),w=n(7),C=n(8),k=n(1),S=(0,w.getWindow)(),D="dx-splitter",I="".concat(D,"-wrapper"),T="".concat(D,"-inactive"),O="".concat(D,"-border"),E="".concat(D,"-initial"),A="dx-state-disabled",P="dxSplitterResizing",R=(0,C.addNamespace)(x.default.down,P),M=(0,C.addNamespace)(x.default.move,P),B=(0,C.addNamespace)(x.default.up,P),V=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){this._$container=this.option("container"),this._$leftElement=this.option("leftElement"),this._$rightElement=this.option("rightElement"),this._onApplyPanelSize=this._createActionByOption("onApplyPanelSize"),this.$element().addClass(I).addClass(E),this._$splitterBorder=(0,m.default)("<div>").addClass(O).appendTo(this.$element()),this._$splitter=(0,m.default)("<div>").addClass(D).addClass(T).appendTo(this._$splitterBorder)}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._detachEventHandlers(),this._attachEventHandlers()}},{key:"_clean",value:function(){this._detachEventHandlers(),l(_(t.prototype),"_clean",this).call(this)}},{key:"_attachEventHandlers",value:function(){var e=y.default.getDocument();b.default.on(this._$splitterBorder,R,this._onMouseDownHandler.bind(this)),b.default.on(e,M,this._onMouseMoveHandler.bind(this)),b.default.on(e,B,this._onMouseUpHandler.bind(this))}},{key:"_detachEventHandlers",value:function(){var e=y.default.getDocument();b.default.off(this._$splitterBorder,R),b.default.off(e,M),b.default.off(e,B)}},{key:"_dimensionChanged",value:function(){if(void 0===this._leftPanelPercentageWidth){var e=this._$leftElement.get(0).clientWidth+this.getSplitterOffset();this._leftPanelPercentageWidth=this._convertLeftPanelWidthToPercentage(e)}var t=100-this._leftPanelPercentageWidth;this._onApplyPanelSize({leftPanelWidth:this._leftPanelPercentageWidth+"%",rightPanelWidth:t+"%"}),this.setSplitterPositionLeft(this._$leftElement.get(0).clientWidth-this.getSplitterOffset())}},{key:"_onMouseDownHandler",value:function(e){e.preventDefault(),this._offsetX=e.pageX-this._$splitterBorder.offset().left<=this._getSplitterBorderWidth()?e.pageX-this._$splitterBorder.offset().left:0,this._isSplitterActive=!0,this._containerWidth=this._$container.get(0).clientWidth,this.$element().removeClass(E),this._$splitter.removeClass(T),this.setSplitterPositionLeft(null,!0)}},{key:"_onMouseMoveHandler",value:function(e){this._isSplitterActive&&this.setSplitterPositionLeft(this._getNewSplitterPositionLeft(e),!0)}},{key:"_onMouseUpHandler",value:function(){this._isSplitterActive&&(this._$splitter.addClass(T),this._isSplitterActive=!1)}},{key:"_getNewSplitterPositionLeft",value:function(e){var t=e.pageX-this._getContainerLeftOffset()-this._offsetX;return t=Math.max(0-this.getSplitterOffset(),t),t=Math.min(this._containerWidth-this.getSplitterOffset()-this._getSplitterWidth(),t)}},{key:"_getContainerLeftOffset",value:function(){var e=this._$container.offset().left;if(S){var t=S.getComputedStyle(this._$container.get(0)),n=parseFloat(t.paddingLeft)||0,i=parseFloat(t.borderLeftWidth)||0;e+=n+i}return e}},{key:"_isDomElement",value:function(e){return e&&e.nodeType&&1===e.nodeType}},{key:"_isPercentValue",value:function(e){return(0,k.isString)(e)&&"%"===e.slice(-1)}},{key:"getSplitterOffset",value:function(){return(this._getSplitterBorderWidth()-this._getSplitterWidth())/2}},{key:"_getSplitterWidth",value:function(){return this._$splitter.get(0).clientWidth}},{key:"_getSplitterBorderWidth",value:function(){return this._$splitterBorder.get(0).clientWidth}},{key:"toggleState",value:function(e){var t=e?"removeClass":"addClass";this.$element()[t](A),this._$splitter[t](A)}},{key:"isSplitterMoved",value:function(){return!this.$element().hasClass(E)}},{key:"setSplitterPositionLeft",value:function(e,t){if(e=e||this._$leftElement.get(0).clientWidth-this.getSplitterOffset(),this.$element().css("left",e),t){var n=e+this.getSplitterOffset(),i=this._containerWidth-n;this._onApplyPanelSize({leftPanelWidth:n,rightPanelWidth:i}),this._leftPanelPercentageWidth=this._convertLeftPanelWidthToPercentage(n)}}},{key:"_optionChanged",value:function(e){switch(e.name){case"initialLeftPanelWidth":this._leftPanelPercentageWidth=this._convertLeftPanelWidthToPercentage(e.value),this._dimensionChanged();break;case"leftElement":this.repaint();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_convertLeftPanelWidthToPercentage",value:function(e){return e/this._$container.get(0).clientWidth*100}}]),t}(v.default);t.default=V,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(5)),b=n(8),x=o(n(25)),w=(0,b.addNamespace)(x.default.up,"dxDiagramPanel"),C=".dx-textbox",k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._createOnPointerUpAction()}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._attachPointerUpEvent()}},{key:"_getPointerUpElements",value:function(){return[this.$element()]}},{key:"_attachPointerUpEvent",value:function(){var e=this,t=this._getPointerUpElements();t.forEach(function(t){y.default.off(t,w),y.default.on(t,w,function(t){(0,m.default)(t.target).closest(C).length||e._onPointerUpAction()})})}},{key:"_createOnPointerUpAction",value:function(){this._onPointerUpAction=this._createActionByOption("onPointerUp")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onPointerUp":this._createOnPointerUpAction();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(v.default),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}t.default=void 0;var a=i(n(2)),r=n(59),s="dx-diagram-contextmenu",l={getContextMenuItemTemplate:function(e,t,n,i){var o=(0,a.default)(i);o.empty();var s=void 0!==t.rootCommand?t.rootCommand:-1;if(t.icon&&!t.checked){var l=(0,r.getImageContainer)(t.icon);o.append(l)}else if(e._menuHasCheckedItems&&e._menuHasCheckedItems[s]===!0){var u=(0,r.getImageContainer)("check");u.css("visibility",t.checked?"visible":"hidden"),o.append(u)}o.append('<span class="dx-menu-item-text">'+t.text+"</span>"),Array.isArray(t.items)&&t.items.length>0&&o.append('<span class="dx-menu-item-popout-container"><div class="dx-menu-item-popout"></div></span>')},getContextMenuCssClass:function(){return s},onContextMenuItemClick:function(e,t,n){if(void 0===t.command&&void 0===t.name||Array.isArray(t.items)&&t.items.length){if(void 0!==t.rootCommand&&void 0!==t.value){var i=l.getItemCommandParameter(e,t,t.value);n.call(this,t.rootCommand,void 0,i)}}else{var o=l.getItemCommandParameter(e,t);n.call(this,t.command,t.name,o)}},getItemValue:function(e){return"object"===o(e.value)?JSON.stringify(e.value):e.value},getItemOptionText:function(e,t){if(e){t=t.slice();var n=this._getParentItemOptionText(t);e&&e._originalItemsInfo&&e._originalItemsInfo[n]&&(t[t.length-1]+=e._originalItemsInfo[n].indexPathCorrection)}return this._getItemOptionTextCore(t)},_getParentItemOptionText:function(e){var t=e.slice(0,e.length-1);return this._getItemOptionTextCore(t)},_getItemOptionTextCore:function(e){return e.reduce(function(e,t){return e+"items[".concat(t,"].")},"")},getItemCommandParameter:function(e,t,n){return t.getParameter?t.getParameter(e):n},updateContextMenuItems:function(e,t,n,i){var o=this;e._originalItemsInfo||(e._originalItemsInfo={}),e._originalItemsInfo[t]||(e._originalItemsInfo[t]={items:e.option(t+"items")||[]}),i=i.map(function(t){return{value:o.getItemValue(t),text:t.text,checked:t.checked,widget:e,rootCommand:n}});var a=e._originalItemsInfo[t].items;e.option(t+"items",i.concat(a)),e._originalItemsInfo[t]&&a.length&&(e._originalItemsInfo[t].indexPathCorrection=i.length)},updateContextMenuItemVisible:function(e,t,n){e.option(t+"visible",n)},updateContextMenuItemValue:function(e,t,n,i){var o=e.option(t+"items");"boolean"!=typeof i||o&&o.length?void 0!==i&&(this._setContextMenuHasCheckedItems(e,n),Array.isArray(o)&&o.forEach(function(e,t){e.checked=e.value===i})):(this._setContextMenuHasCheckedItems(e,-1),e.option(t+"checked",i))},_setContextMenuHasCheckedItems:function(e,t){e._menuHasCheckedItems||(e._menuHasCheckedItems={}),e._menuHasCheckedItems[t]=!0}},u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(7),b=o(n(48)),x=o(n(538)),w="dx-diagram-mobile-popup",C=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._createOnVisibilityChangingAction(),this._createOnVisibilityChangedAction()}},{key:"isVisible",value:function(){return this.option("isVisible")}},{key:"isMobileView",value:function(){return this.option("isMobileView")}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this);var e=this.$element(),n=(0,m.default)("<div>").addClass(this._getPopupClass()).addClass(this.isMobileView()&&w).appendTo(e);this._popup=this._createComponent(n,b.default,this._getPopupOptions()),this._updatePopupVisible()}},{key:"show",value:function(){this.option("isVisible",!0)}},{key:"hide",value:function(){this.option("isVisible",!1)}},{key:"toggle",value:function(){this.option("isVisible",!this.isVisible())}},{key:"repaint",value:function(){this._popup.repaint()}},{key:"_getPopupContent",value:function(){return this._popup.content()}},{key:"_getPopupTitle",value:function(){var e=(0,m.default)(this._getPopupContent());return e.parent().find(".dx-popup-title")}},{key:"_getPointerUpElements",value:function(){return[this._getPopupContent(),this._getPopupTitle()]}},{key:"_getVerticalPaddingsAndBorders",value:function(){var e=(0,m.default)(this._getPopupContent());return e.outerHeight()-e.height()}},{key:"_getHorizontalPaddingsAndBorders",value:function(){var e=(0,m.default)(this._getPopupContent());return e.outerWidth()-e.width()}},{key:"_getPopupClass",value:function(){return""}},{key:"_getPopupWidth",value:function(){return this.option("width")||"auto"}},{key:"_getPopupMaxWidth",value:function(){return this.option("maxWidth")}},{key:"_getPopupMinWidth",value:function(){return this.option("minWidth")}},{key:"_getPopupHeight",value:function(){return this.option("height")||"auto"}},{key:"_getPopupMaxHeight",value:function(){return this.option("maxHeight")}},{key:"_getPopupMinHeight",value:function(){return this.option("minHeight")}},{key:"_getPopupPosition",value:function(){
return{}}},{key:"_getPopupContainer",value:function(){return this.option("container")}},{key:"_getPopupSlideAnimationObject",value:function(e){return(0,v.extend)({type:"slide",start:function(){(0,m.default)("body").css("overflow","hidden")},complete:function(){(0,m.default)("body").css("overflow","")}},e)}},{key:"_getPopupAnimation",value:function(){return{hide:{type:"fadeOut"},show:{type:"fadeIn"}}}},{key:"_getPopupOptions",value:function(){var e=this,t=this;return{animation:(0,y.hasWindow)()?this._getPopupAnimation():null,shading:!1,showTitle:!1,focusStateEnabled:!1,container:this._getPopupContainer(),width:this._getPopupWidth(),height:this._getPopupHeight(),maxWidth:this._getPopupMaxWidth(),maxHeight:this._getPopupMaxHeight(),minWidth:this._getPopupMinWidth(),minHeight:this._getPopupMinHeight(),position:this._getPopupPosition(),onContentReady:function(){t._renderPopupContent(t._popup.content())},onShowing:function(){e._onVisibilityChangingAction({visible:!0,component:e})},onShown:function(){e.option("isVisible",!0),e._onVisibilityChangedAction({visible:!0,component:e})},onHiding:function(){e._onVisibilityChangingAction({visible:!1,component:e})},onHidden:function(){e.option("isVisible",!1),e._onVisibilityChangedAction({visible:!1,component:e})}}}},{key:"_renderPopupContent",value:function(e){}},{key:"_updatePopupVisible",value:function(){this._popup.option("visible",this.isVisible())}},{key:"_createOnVisibilityChangingAction",value:function(){this._onVisibilityChangingAction=this._createActionByOption("onVisibilityChanging")}},{key:"_createOnVisibilityChangedAction",value:function(){this._onVisibilityChangedAction=this._createActionByOption("onVisibilityChanged")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onVisibilityChanging":this._createOnVisibilityChangingAction();break;case"onVisibilityChanged":this._createOnVisibilityChangedAction();break;case"container":this._popup.option("container",this._getPopupContainer());break;case"width":this._popup.option("width",this._getPopupWidth());break;case"height":this._popup.option("height",this._getPopupHeight());break;case"maxWidth":this._popup.option("maxWidth",this._getPopupMaxWidth());break;case"maxHeight":this._popup.option("maxHeight",this._getPopupMaxHeight());break;case"minWidth":this._popup.option("minWidth",this._getPopupMinWidth());break;case"minHeight":this._popup.option("minHeight",this._getPopupMinHeight());break;case"isMobileView":this._invalidate();break;case"isVisible":this._updatePopupVisible();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{isVisible:!0,isMobileView:!1,offsetX:0,offsetY:0})}}]),t}(x.default),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==o(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=n(0),_=i(n(117)),m=i(n(94)),v=function(e){function t(e){var i;return a(this,t),i=n.call(this),i._diagramWidget=e,i._resetCache(),i}l(t,e);var n=c(t);return s(t,[{key:"_dataSourceChangedHandler",value:function(e,t){if(this._resetCache(),this._items=e.map(function(e){return(0,g.extend)(!0,{},e)}),this._dataSourceItems=e.slice(),t&&t.changes){var n=t.changes.filter(function(e){return!e.internalChange});n.length&&this._reloadContentByChanges(n,!0)}else this._diagramWidget._onDataSourceChanged()}},{key:"_dataSourceLoadingChangedHandler",value:function(e){e&&!this._dataSource.isLoaded()?this._diagramWidget._showLoadingIndicator():this._diagramWidget._hideLoadingIndicator()}},{key:"_prepareData",value:function(e){for(var t in e)Object.prototype.hasOwnProperty.call(e,t)&&void 0===e[t]&&(e[t]=null);return e}},{key:"insert",value:function(e,t,n){var i=this;this._resetCache();var o=this._getStore();o.insert(this._prepareData(e)).done(function(e,n){var a=[{type:"insert",key:n,data:e,internalChange:!0}];o.push(a),i._reloadContentByChanges(a,!1),t&&t(e),i._resetCache()}).fail(function(e){n&&n(e),i._resetCache()})}},{key:"update",value:function(e,t,n,i){var o=this,a=this._getStore(),r=this._getStoreKey(a,e,t);a.update(r,this._prepareData(t)).done(function(e,t){var i=[{type:"update",key:t,data:e,internalChange:!0}];a.push(i),o._reloadContentByChanges(i,!1),n&&n(t,e)}).fail(function(e){i&&i(e)})}},{key:"remove",value:function(e,t,n,i){var o=this;this._resetCache();var a=this._getStore(),r=this._getStoreKey(a,e,t);a.remove(r).done(function(e){var t=[{type:"remove",key:e,internalChange:!0}];a.push(t),o._reloadContentByChanges(t,!1),n&&n(e),o._resetCache()}).fail(function(e){i&&i(e),o._resetCache()})}},{key:"findItem",value:function(e){return this._items?this._getItemByKey(e):null}},{key:"getItems",value:function(){return this._items}},{key:"hasItems",value:function(){return!!this._items}},{key:"_reloadContentByChanges",value:function(e,t){var n=this;e=e.map(function(e){return(0,g.extend)(e,{internalKey:n._getInternalKey(e.key)})}),this._diagramWidget._reloadContentByChanges(e,t)}},{key:"_getItemByKey",value:function(e){this._ensureCache();var t=this._cache,n=this._getIndexByKey(e);return t.items[n]}},{key:"_getIndexByKey",value:function(e){this._ensureCache();var t=this._cache;if("object"!==o(e)){var n=t.keySet||t.keys.reduce(function(e,t,n){return e[t]=n,e},{});return t.keySet||(t.keySet=n),n[e]}for(var i=0,a=t.keys.length;i<a;i++)if(t.keys[i]===e)return i;return-1}},{key:"_ensureCache",value:function(){var e=this._cache;e.keys||(e.keys=[],e.items=[],this._fillCache(e,this._items))}},{key:"_fillCache",value:function(e,t){var n=this;if(t&&t.length){var i=this._getKeyExpr();i&&t.forEach(function(t){e.keys.push(i(t)),e.items.push(t)});var o=this._getItemsExpr();o&&t.forEach(function(t){return n._fillCache(e,o(t))});var a=this._getContainerChildrenExpr();a&&t.forEach(function(t){return n._fillCache(e,a(t))})}}},{key:"_getKeyExpr",value:function(){throw"Not Implemented"}},{key:"_getItemsExpr",value:function(){}},{key:"_getContainerChildrenExpr",value:function(){}},{key:"_dataSourceOptions",value:function(){return{paginate:!1}}},{key:"_getStore",value:function(){return this._dataSource&&this._dataSource.store()}},{key:"_getStoreKey",value:function(e,t,n){var i=e.keyOf(n);if(i===n){var o=this._getKeyExpr();this._dataSourceItems.forEach(function(e){o(e)===t&&(i=e)})}return i}},{key:"_getInternalKey",value:function(e){if("object"===o(e)){var t=this._getKeyExpr();return t(e)}return e}},{key:"_resetCache",value:function(){this._cache={}}}]),t}(_.default);v.include(m.default);var y=v;t.default=y,e.exports=t.default},function(e,t,n){Object.defineProperty(t,"refreshPaths",{enumerable:!0,get:function(){return i.refreshPaths}}),t.prepareSegmentRectPoints=void 0;var i=n(171),o=n(3),a=function(e,t,n,i,a){var r=~~((n<i?n:i)/2),s=a.width||0,l=s<r?s:r;e+=l/2,t+=l/2,n-=l,i-=l;var u,c=e+n,d=t+i,h=[],f=[],p=0,g=0,_={top:[[e,t],[c,t]],right:[[c,t],[c,d]],bottom:[[c,d],[e,d]],left:[[e,d],[e,t]]};switch((0,o.each)(_,function(e){var t=!!a[e];p=2*p+~~t}),p){case 13:case 9:u=["left","top","right","bottom"];break;case 11:u=["bottom","left","top","right"];break;default:u=["top","right","bottom","left"]}return(0,o.each)(u,function(e,t){var n=!!a[t];!g&&f.length&&(h.push(f),f=[]),n&&(0,o.each)(_[t].slice(g),function(e,t){f=f.concat(t)}),g=~~n}),f.length&&h.push(f),1===h.length&&(h=h[0]),{points:h,pathType:15===p?"area":"line"}};t.prepareSegmentRectPoints=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(135)),a=i(n(22)),r=(0,o.default)(a.default.ERROR_MESSAGES,{E2001:"Invalid data source",E2002:"Axis type and data type are incompatible",E2003:'The "{0}" data source field contains data of unsupported type',E2004:'The "{0}" data source field is inconsistent',E2005:'The value field "{0}" is absent in the data source or all its values are negative',E2006:"A cycle is detected in provided data",E2007:'The value field "{0}" is absent in the data source',E2008:'The value field "{0}" must be a string',E2009:'The value field "{0}" must be a positive numeric value',E2101:"Unknown series type: {0}",E2102:"Ambiguity occurred between two value axes with the same name",E2103:'The "{0}" option is given an invalid value. Assign a function instead',E2104:"Invalid logarithm base",E2105:'Invalid value of a "{0}"',E2202:"Invalid {0} scale value",E2203:"The range you are trying to set is invalid",W2002:"The {0} series cannot be drawn because the {1} data field is missing",W2003:"Tick interval is too small",W2101:'The "{0}" pane does not exist; the last pane is used by default',W2102:'A value axis with the "{0}" name was created automatically',W2103:"The chart title was hidden due to the container size",W2104:"The legend was hidden due to the container size",W2105:'The title of the "{0}" axis was hidden due to the container size',W2106:'The labels of the "{0}" axis were hidden due to the container size',W2107:"The export menu was hidden due to the container size",W2108:"The browser does not support exporting images to {0} format.",W2301:"Invalid value range"});t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(152)),r=n(14),s=n(1),l=i(n(151)),u=o.extend,c=Math.round,d=Math.sqrt,h=Math.acos,f=180/Math.PI,p=Math.abs,g=l.default.radialLabelIndent,_=u({},a.default,{_updateData:function(e,t){var n=this;a.default._updateData.call(this,e),!t&&(0,s.isDefined)(n._visible)||(n._visible=!0),n.minValue=n.initialMinValue=n.originalMinValue=(0,s.isDefined)(e.minValue)?e.minValue:0},animate:function(e,t,n){var i=this;i.graphic.animate({x:i.centerX,y:i.centerY,outerRadius:i.radiusOuter,innerRadius:i.radiusInner,startAngle:i.toAngle,endAngle:i.fromAngle},{delay:n,partitionDuration:t},e)},correctPosition:function(e){var t=this;t.correctRadius(e),t.correctLabelRadius(e.radiusOuter+g),t.centerX=e.centerX,t.centerY=e.centerY},correctRadius:function(e){this.radiusInner=e.radiusInner,this.radiusOuter=e.radiusOuter},correctLabelRadius:function(e){this.radiusLabels=e},correctValue:function(e,t,n){var i=this;i.value=(n||i.normalInitialValue)+e,i.minValue=e,i.percent=t,i._label.setDataField("percent",t)},_updateLabelData:function(){this._label.setData(this._getLabelFormatObject())},_getShiftLabelCoords:function(){var e=this,t=e._label.getBoundingRect(),n=e._getLabelCoords(e._label),i=e._getVisibleArea();return e._isLabelDrawingWithoutPoints?e._checkLabelPosition(n,t,i):e._getLabelExtraCoord(n,e._checkVerticalLabelPosition(n,t,i),t)},_getLabelPosition:function(e){return e.position},getAnnotationCoords:function(e){return this._getElementCoords("edge"!==e?"inside":"outside",this.radiusOuter,0)},_getElementCoords:function(e,t,n){var i,o,a=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{x:0,y:0,width:0,height:0},s=this,l=(0,r.getCosAndSin)(s.middleAngle),u=s.radiusInner,d=s.radiusOuter,h="columns"===e;return"inside"===e?(i=u+(d-u)/2+n,o=s.centerX+i*l.cos-a.width/2):(i=t+n,o=l.cos>.1||h&&l.cos>=0?s.centerX+i*l.cos:l.cos<-.1||h&&l.cos<0?s.centerX+i*l.cos-a.width:s.centerX+i*l.cos-a.width/2),{x:o,y:c(s.centerY-i*l.sin-a.height/2)}},_getLabelCoords:function(e){var t=this,n=e.getBoundingRect(),i=e.getLayoutOptions(),o=t._getLabelPosition(i);return t._getElementCoords(o,t.radiusLabels,i.radialOffset,n)},_correctLabelCoord:function(e,t){var n=this,i=n._label,o=i.getBoundingRect(),a=o.width,s=i.getLayoutOptions(),l=n._getVisibleArea(),u=l.maxX-a,c=l.minX,d=(0,r.normalizeAngle)(n.middleAngle),h=n.centerX,f=s.connectorOffset,p=e.x;return"columns"===s.position?(p=d<=90||d>=270?u:c,e.x=p):"inside"!==s.position&&t&&(d<=90||d>=270?p-f<h&&(p=h+f):p+a+f>h&&(p=h-a-f),e.x=p),e},drawLabel:function(){this.translate(),this._isLabelDrawingWithoutPoints=!0,this._drawLabel(),this._isLabelDrawingWithoutPoints=!1},updateLabelCoord:function(e){var t=this,n=t._label.getBoundingRect(),i=t._correctLabelCoord(n,e);i=t._checkHorizontalLabelPosition(i,n,t._getVisibleArea()),t._label.shift(c(i.x),c(n.y))},_checkVerticalLabelPosition:function(e,t,n){var i=e.x,o=e.y;return e.y+t.height>n.maxY?o=n.maxY-t.height:e.y<n.minY&&(o=n.minY),{x:i,y:o}},_getLabelExtraCoord:function(e,t,n){return e.y!==t.y?(0,r.getVerticallyShiftedAngularCoords)({x:e.x,y:e.y,width:n.width,height:n.height},t.y-e.y,{x:this.centerX,y:this.centerY}):e},_checkHorizontalLabelPosition:function(e,t,n){var i=e.x,o=e.y;return e.x+t.width>n.maxX?i=n.maxX-t.width:e.x<n.minX&&(i=n.minX),{x:i,y:o}},applyWordWrap:function(e){var t=this,n=t._label,i=n.getBoundingRect(),o=t._getVisibleArea(),a=n.getLayoutOptions().position,r=i.width,s=!1;return"columns"===a&&t.series.index>0?r=o.maxX-t.centerX-t.radiusLabels:"inside"===a?r>o.maxX-o.minX&&(r=o.maxX-o.minX):e&&i.x<t.centerX&&i.width+i.x>t.centerX?r=Math.floor((o.maxX-o.minX)/2):i.x+r>o.maxX?r=o.maxX-i.x:i.x<o.minX&&(r=i.x+r-o.minX),r<i.width&&(s=n.fit(r)),s},setLabelTrackerData:function(){this._label.setTrackerData(this)},_checkLabelPosition:function(e,t,n){return e=this._checkHorizontalLabelPosition(e,t,n),this._checkVerticalLabelPosition(e,t,n)},_getLabelConnector:function(){var e=this,t=e.radiusOuter,n=e._options.styles.normal,i=n["stroke-width"]/2,o=c(e.series.getOptions().containerBackgroundColor===n.stroke?i:-i),a=(0,r.getCosAndSin)(c(e.middleAngle));return{x:c(e.centerX+(t-o)*a.cos),y:c(e.centerY-(t-o)*a.sin),angle:e.middleAngle}},_drawMarker:function(e,t,n,i){var o=this,a=o.radiusOuter,r=o.radiusInner,s=o.fromAngle,l=o.toAngle;n&&(r=a=0,i||(s=l=o.shiftedAngle)),o.graphic=e.arc(o.centerX,o.centerY,r,a,l,s).attr({"stroke-linejoin":"round"}).smartAttr(o._getStyle()).data({"chart-data-point":o}).sharp().append(t)},getTooltipParams:function(){var e=this,t=(0,r.getCosAndSin)(e.middleAngle),n=e.radiusInner,i=e.radiusOuter;return{x:e.centerX+(n+(i-n)/2)*t.cos,y:e.centerY-(n+(i-n)/2)*t.sin,offset:0}},_translate:function(){var e=this,t=e.shiftedAngle||0,n=e.value,i=e.minValue,o=e._getValTranslator();e.fromAngle=o.translate(i)+t,e.toAngle=o.translate(n)+t,e.middleAngle=o.translate((n-i)/2+i)+t,e.isVisible()||(e.middleAngle=e.toAngle=e.fromAngle=e.fromAngle||t)},getMarkerVisibility:function(){return!0},_updateMarker:function(e,t,n,i){var o=this;e||(t=u({x:o.centerX,y:o.centerY,outerRadius:o.radiusOuter,innerRadius:o.radiusInner,startAngle:o.toAngle,endAngle:o.fromAngle},t)),o.graphic.smartAttr(t).sharp(),i&&i()},getLegendStyles:function(){return this._styles.legendStyles},isInVisibleArea:function(){return!0},hide:function(){var e=this;e._visible&&(e._visible=!1,e.hideTooltip(),e._options.visibilityChanged())},show:function(){var e=this;e._visible||(e._visible=!0,e._options.visibilityChanged())},setInvisibility:function(){this._label.draw(!1)},isVisible:function(){return this._visible},_getFormatObject:function(e){var t=a.default._getFormatObject.call(this,e),n=this.percent;return t.percent=n,t.percentText=e.formatValue(n,"percent"),t},getColor:function(){return this._styles.normal.fill},coordsIn:function(e,t){var n,i=this,o=e-i.centerX,a=t-i.centerY,r=d(o*o+a*a),s=i.fromAngle%360,l=i.toAngle%360;return!(r<i.radiusInner||r>i.radiusOuter||0===r)&&(n=h(o/r)*f*(a>0?-1:1),n<0&&(n+=360),s===l&&p(i.toAngle-i.fromAngle)>1e-4||(s>=l?n<=s&&n>=l:!(n>=s&&n<=l)))}});t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=n(0),r=n(4),s=n(327),l=i(n(152)),u=n(1),c=a.extend,d=Math,h=d.abs,f=d.min,p=d.max,g=d.round,_=20,m=20,v=c({},l.default,{deleteLabel:function(){var e=this;e._topLabel.dispose(),e._topLabel=null,e._bottomLabel.dispose(),e._bottomLabel=null},hideMarker:function(e){var t=this.graphic,n=t&&t[e+"Marker"],i=this["_"+e+"Label"];n&&"hidden"!==n.attr("visibility")&&n.attr({visibility:"hidden"}),i.draw(!1)},setInvisibility:function(){this.hideMarker("top"),this.hideMarker("bottom")},clearVisibility:function(){var e=this,t=e.graphic,n=t&&t.topMarker,i=t&&t.bottomMarker;n&&n.attr("visibility")&&n.attr({visibility:null}),i&&i.attr("visibility")&&i.attr({visibility:null})},clearMarker:function(){var e=this,t=e.graphic,n=t&&t.topMarker,i=t&&t.bottomMarker,o=e._emptySettings;n&&n.attr(o),i&&i.attr(o)},_getLabelPosition:function(e){var t,n="inside"===this._options.label.position;return t=this._options.rotated?"top"===e^n?"right":"left":"top"===e^n?"top":"bottom"},_getLabelMinFormatObject:function(){var e=this;return{index:0,argument:e.initialArgument,value:e.initialMinValue,seriesName:e.series.name,originalValue:e.originalMinValue,originalArgument:e.originalArgument,point:e}},_updateLabelData:function(){var e=this._getLabelFormatObject();e.index=1,this._topLabel.setData(e),this._bottomLabel.setData(this._getLabelMinFormatObject())},_updateLabelOptions:function(){var e=this,t=this._options.label;(!e._topLabel||!e._bottomLabel)&&e._createLabel(),e._topLabel.setOptions(t),e._bottomLabel.setOptions(t)},_createLabel:function(){var e={renderer:this.series._renderer,labelsGroup:this.series._labelsGroup,point:this};this._topLabel=new s.Label(e),this._bottomLabel=new s.Label(e)},_getGraphicBBox:function(e){var t,n=this._options,i=this._getImage(n.image),o="top"===e?this._checkImage(i.top):this._checkImage(i.bottom),a=this._getPositionFromLocation(e);return t=n.visible?o?this._getImageBBox(a.x,a.y):this._getSymbolBBox(a.x,a.y,n.styles.normal.r):{x:a.x,y:a.y,width:0,height:0}},_getPositionFromLocation:function(e){var t,n,i="top"===e;return this._options.rotated?(t=i?p(this.x,this.minX):f(this.x,this.minX),n=this.y):(t=this.x,n=i?f(this.y,this.minY):p(this.y,this.minY)),{x:t,y:n}},_checkOverlay:function(e,t,n){return e<t+n},_getOverlayCorrections:function(e,t){var n=this._options.rotated,i=n?"x":"y",o=n?"width":"height",a=this.series.getValueAxis().getVisibleArea(),r=a[0],s=a[1],l=g((e[i]+e[o]-t[i])/2),u=e[i]-l,c=t[i]+l;return u<r?(l=r-u,u+=l,c+=l):c+t[o]>s&&(l=s-c-t[o],u+=l,c+=l),{coord1:u,coord2:c}},_checkLabelsOverlay:function(e){var t=this,n=t._topLabel.getBoundingRect(),i=t._bottomLabel.getBoundingRect(),o={};t._options.rotated?"top"===e?this._checkOverlay(n.x,i.x,i.width)&&(o=this._getOverlayCorrections(i,n),t._topLabel.shift(o.coord2,n.y),t._bottomLabel.shift(o.coord1,i.y)):this._checkOverlay(i.x,n.x,n.width)&&(o=this._getOverlayCorrections(n,i),t._topLabel.shift(o.coord1,n.y),t._bottomLabel.shift(o.coord2,i.y)):"top"===e?this._checkOverlay(i.y,n.y,n.height)&&(o=this._getOverlayCorrections(n,i),t._topLabel.shift(n.x,o.coord1),t._bottomLabel.shift(i.x,o.coord2)):this._checkOverlay(n.y,i.y,i.height)&&(o=this._getOverlayCorrections(i,n),t._topLabel.shift(n.x,o.coord2),t._bottomLabel.shift(i.x,o.coord1))},_drawLabel:function(){var e=this,t=[],n=e._options.rotated?e.x>=e.minX:e.y<e.minY,i=e._getCustomLabelVisibility(),a=e._topLabel,r=e._bottomLabel;a.pointPosition=n?"top":"bottom",r.pointPosition=n?"bottom":"top",(e.series.getLabelVisibility()||i)&&e.hasValue()&&i!==!1?(e.visibleTopMarker!==!1&&t.push(a),e.visibleBottomMarker!==!1&&t.push(r),(0,o.each)(t,function(e,t){t.draw(!0)}),e._checkLabelsOverlay(e._topLabel.pointPosition)):(a.draw(!1),r.draw(!1))},_getImage:function(e){var t={};return(0,u.isDefined)(e)&&("string"==typeof e?t.top=t.bottom=e:(t.top={url:"string"==typeof e.url?e.url:e.url&&e.url.rangeMaxPoint,width:"number"==typeof e.width?e.width:e.width&&e.width.rangeMaxPoint,height:"number"==typeof e.height?e.height:e.height&&e.height.rangeMaxPoint},t.bottom={url:"string"==typeof e.url?e.url:e.url&&e.url.rangeMinPoint,width:"number"==typeof e.width?e.width:e.width&&e.width.rangeMinPoint,height:"number"==typeof e.height?e.height:e.height&&e.height.rangeMinPoint})),t},_checkSymbol:function(e,t){var n=this,i=e.symbol,o=t.symbol,a="circle"===i&&"circle"!==o||"circle"!==i&&"circle"===o,r=n._getImage(e.image),s=n._getImage(t.image),l=n._checkImage(r.top)!==n._checkImage(s.top),u=n._checkImage(r.bottom)!==n._checkImage(s.bottom);return a||l||u},_getSettingsForTwoMarkers:function(e){var t=this,n=t._options,i={},o=n.rotated?f(t.x,t.minX):t.x,a=n.rotated?t.y:f(t.y,t.minY),r=e.r,s=t._populatePointShape(n.symbol,r);return i.top=c({translateX:o+t.width,translateY:a,r:r},e),i.bottom=c({translateX:o,translateY:a+t.height,r:r},e),s&&(i.top.points=i.bottom.points=s),i},_hasGraphic:function(){return this.graphic&&this.graphic.topMarker&&this.graphic.bottomMarker},_drawOneMarker:function(e,t,n,i){var o=this,a=o.graphic;a[t]?o._updateOneMarker(t,i):a[t]=o._createMarker(e,a,n,i)},_drawMarker:function(e,t,n,i,o){var a=this,r=a._getSettingsForTwoMarkers(o||a._getStyle()),s=a._getImage(a._options.image);a._checkImage(s.top)&&(r.top=a._getImageSettings(r.top,s.top)),a._checkImage(s.bottom)&&(r.bottom=a._getImageSettings(r.bottom,s.bottom)),a.graphic=a.graphic||e.g().append(t),a.visibleTopMarker&&a._drawOneMarker(e,"topMarker",s.top,r.top),a.visibleBottomMarker&&a._drawOneMarker(e,"bottomMarker",s.bottom,r.bottom)},_getSettingsForTracker:function(e){var t=this,n=t._options.rotated;return{translateX:n?f(t.x,t.minX)-e:t.x-e,translateY:n?t.y-e:f(t.y,t.minY)-e,width:t.width+2*e,height:t.height+2*e}},isInVisibleArea:function(){var e,t=this,n=t._options.rotated,i=n?t.y:t.x,o=n?p(t.minX,t.x):p(t.minY,t.y),a=n?f(t.minX,t.x):f(t.minY,t.y),r=!0,s=!0,l=!0,u=t.series.getArgumentAxis().getVisibleArea(),c=t.series.getValueAxis().getVisibleArea(),d=u[1]<i||u[0]>i,h=c[0]>a&&c[0]>o||c[1]<a&&c[1]<o;return d||h?r=s=l=!1:(r=c[0]<=a&&c[1]>a,s=c[0]<o&&c[1]>=o,n&&(e=r,r=s,s=e)),t.visibleTopMarker=r,t.visibleBottomMarker=s,l},getTooltipParams:function(){var e,t,n=this,i=n._options.rotated,o=i?f(n.x,n.minX):f(n.y,n.minY),a=i?"width":"height",r=n._getVisibleArea(),s=i?r.minX:r.minY,l=i?r.maxX:r.maxY,u=p(s,o),c=f(l,o+n[a]);return i?(t=n.y,e=u+(c-u)/2):(e=n.x,t=u+(c-u)/2),{x:e,y:t,offset:0}},_translate:function(){var e=this,t=e._options.rotated;l.default._translate.call(e),e.height=t?0:h(e.minY-e.y),e.width=t?h(e.x-e.minX):0},hasCoords:function(){return l.default.hasCoords.call(this)&&!(null===this.minX||null===this.minY)},_updateData:function(e){var t=this;l.default._updateData.call(t,e),t.minValue=t.initialMinValue=t.originalMinValue=e.minValue},_getImageSettings:function(e,t){return{href:t.url||t.toString(),width:t.width||_,height:t.height||m,translateX:e.translateX,translateY:e.translateY}},getCrosshairData:function(e,t){var n=this,i=n._options.rotated,o=n.minX,a=n.minY,r=n.vx,s=n.vy,l=n.value,u=n.minValue,c=n.argument,d={axis:n.series.axis,x:r,y:s,yValue:l,xValue:c};return i?(d.yValue=c,h(r-e)<h(o-e)?d.xValue=l:(d.x=o,d.xValue=u)):h(s-t)>=h(a-t)&&(d.y=a,d.yValue=u),d},_updateOneMarker:function(e,t){this.graphic&&this.graphic[e]&&this.graphic[e].attr(t)},_updateMarker:function(e,t){this._drawMarker(void 0,void 0,!1,!1,t)},_getFormatObject:function(e){var t=this,n=t.initialMinValue,i=t.initialValue,o=t.initialArgument,a=e.formatValue(n),r=e.formatValue(i);return{argument:o,argumentText:e.formatValue(o,"argument"),valueText:a+" - "+r,rangeValue1Text:a,rangeValue2Text:r,rangeValue1:n,rangeValue2:i,seriesName:t.series.name,point:t,originalMinValue:t.originalMinValue,originalValue:t.originalValue,originalArgument:t.originalArgument}},getLabel:function(){return[this._topLabel,this._bottomLabel]},getLabels:function(){return[this._topLabel,this._bottomLabel]},getBoundingRect:r.noop,coordsIn:function(e,t){var n=this._storeTrackerR(),i=e>=this.x-n&&e<=this.x+n,o=t>=this.y-n&&t<=this.y+n;return this._options.rotated?o&&(i||e>=this.minX-n&&e<=this.minX+n):i&&(o||t>=this.minY-n&&t<=this.minY+n)},getMaxValue:function(){return"discrete"!==this.series.valueAxisType&&this.minValue>this.value?this.minValue:this.value},getMinValue:function(){return"discrete"!==this.series.valueAxisType?this.minValue<this.value?this.minValue:this.value:this.minValue}});t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(152)),r=i(n(229)),s=Math,l=s.abs,u=s.min,c=s.max,d=s.round,h=2,f=(0,o.extend)({},r.default,{_getContinuousPoints:function(e,t){var n,i=this,o=i.x,a=i._options.rotated?function(e,t){return[t,e]}:function(e,t){return[e,t]},r=i.width,s=i.highY,u=l(s-e)<l(s-t)?e:t,c=u===t?e:t;return n=c===u?[].concat(a(o,i.highY)).concat(a(o,i.lowY)).concat(a(o,i.closeY)).concat(a(o-r/2,i.closeY)).concat(a(o+r/2,i.closeY)).concat(a(o,i.closeY)):[].concat(a(o,i.highY)).concat(a(o,u)).concat(a(o+r/2,u)).concat(a(o+r/2,c)).concat(a(o,c)).concat(a(o,i.lowY)).concat(a(o,c)).concat(a(o-r/2,c)).concat(a(o-r/2,u)).concat(a(o,u))},_getCrockPoints:function(e){var t=this,n=t.x,i=t._options.rotated?function(e,t){return[t,e]}:function(e,t){return[e,t]};return[].concat(i(n,t.highY)).concat(i(n,t.lowY)).concat(i(n,e)).concat(i(n-t.width/2,e)).concat(i(n+t.width/2,e)).concat(i(n,e))},_getPoints:function(){var e,t=this,n=t.closeY,i=t.openY;return e=null!==n&&null!==i?t._getContinuousPoints(i,n):i===n?[t.x,t.highY,t.x,t.lowY]:t._getCrockPoints(null!==i?i:n)},getColor:function(){var e=this;return e._isReduction?e._options.reduction.color:e._styles.normal.stroke||e.series.getColor()},_drawMarkerInGroup:function(e,t,n){var i=this;i.graphic=n.path(i._getPoints(),"area").attr({"stroke-linecap":"square"}).attr(t).data({"chart-data-point":i}).sharp().append(e)},_fillStyle:function(){var e=this,t=e._options.styles;e._isReduction&&e._isPositive?e._styles=t.reductionPositive:e._isReduction?e._styles=t.reduction:e._isPositive?e._styles=t.positive:e._styles=t},_getMinTrackerWidth:function(){return 2+2*this._styles.normal["stroke-width"]},correctCoordinates:function(e){var t=this._getMinTrackerWidth(),n=10,i=e.width;i=i<t?t:i>n?n:i,this.width=i+i%2,this.xCorrection=e.offset},_getMarkerGroup:function(e){var t,n=this;return t=n._isReduction&&n._isPositive?e.reductionPositiveMarkersGroup:n._isReduction?e.reductionMarkersGroup:n._isPositive?e.defaultPositiveMarkersGroup:e.defaultMarkersGroup},_drawMarker:function(e,t){this._drawMarkerInGroup(this._getMarkerGroup(t),this._getStyle(),e)},_getSettingsForTracker:function(){var e,t,n,i,o=this,a=o.highY,r=o.lowY,s=o._options.rotated;return a===r&&(a=s?a+h:a-h,r=s?r-h:r+h),s?(e=u(r,a),t=o.x-o.width/2,n=l(r-a),i=o.width):(e=o.x-o.width/2,t=u(r,a),n=o.width,i=l(r-a)),{x:e,y:t,width:n,height:i}},_getGraphicBBox:function(e){var t=this,n=t._options.rotated,i=t.x,o=t.width,a=t.lowY,r=t.highY;if(e){var s=t.series.getValueAxis().getVisibleArea();r=t._truncateCoord(r,s),a=t._truncateCoord(a,s)}var l={x:n?a:i-d(o/2),y:n?i-d(o/2):r,width:n?r-a:o,height:n?o:a-r};if(e){var u="top"===e;this._options.rotated?(l.x=u?l.x+l.width:l.x,l.width=0):(l.y=u?l.y:l.y+l.height,l.height=0)}return l},getTooltipParams:function(e){var t=this;if(t.graphic){var n=u(t.lowY,t.highY),i=c(t.lowY,t.highY),o=t._getVisibleArea(),a=t._options.rotated,r=a?o.minX:o.minY,s=a?o.maxX:o.maxY,l=c(r,n),d=u(s,i),h=t.getCenterCoord();return"edge"===e&&(h[a?"x":"y"]=a?d:l),h.offset=0,h}},getCenterCoord:function(){if(this.graphic){var e,t,n=this,i=u(n.lowY,n.highY),o=c(n.lowY,n.highY),a=n._getVisibleArea(),r=n._options.rotated,s=r?a.minX:a.minY,l=r?a.maxX:a.maxY,d=c(s,i),h=u(l,o),f=d+(h-d)/2;return r?(t=n.x,e=f):(e=n.x,t=f),{x:e,y:t}}},hasValue:function(){return null!==this.highValue&&null!==this.lowValue},hasCoords:function(){return null!==this.x&&null!==this.lowY&&null!==this.highY},_translate:function(){var e=this,t=e._options.rotated,n=e._getValTranslator(),i=e._getArgTranslator().translate(e.argument);e.vx=e.vy=e.x=null===i?i:i+(e.xCorrection||0),e.openY=null!==e.openValue?n.translate(e.openValue):null,e.highY=n.translate(e.highValue),e.lowY=n.translate(e.lowValue),e.closeY=null!==e.closeValue?n.translate(e.closeValue):null;var o=u(e.lowY,e.highY)+l(e.lowY-e.highY)/2;e._calculateVisibility(t?o:e.x,t?e.x:o)},getCrosshairData:function(e,t){var n,i,o=this,a=o._options.rotated,r=a?e:t,s=o.argument,u="low";return l(o.lowY-r)<l(o.closeY-r)?n=o.lowY:(n=o.closeY,u="close"),l(n-r)>=l(o.openY-r)&&(n=o.openY,u="open"),l(n-r)>=l(o.highY-r)&&(n=o.highY,u="high"),i=a?{y:o.vy,x:n,xValue:o[u+"Value"],yValue:s}:{x:o.vx,y:n,xValue:s,yValue:o[u+"Value"]},i.axis=o.series.axis,i},_updateData:function(e){var t=this,n=t._label,i=this._options.reduction.color;t.value=t.initialValue=e.reductionValue,t.originalValue=e.value,t.lowValue=t.originalLowValue=e.lowValue,t.highValue=t.originalHighValue=e.highValue,t.openValue=t.originalOpenValue=e.openValue,t.closeValue=t.originalCloseValue=e.closeValue,t._isPositive=e.openValue<e.closeValue,t._isReduction=e.isReduction,t._isReduction&&n.setColor(i)},_updateMarker:function(e,t,n){var i=this,o=i.graphic;o.attr({points:i._getPoints()}).smartAttr(t).sharp(),n&&o.append(i._getMarkerGroup(n))},_getLabelFormatObject:function(){var e=this;return{openValue:e.openValue,highValue:e.highValue,lowValue:e.lowValue,closeValue:e.closeValue,reductionValue:e.initialValue,argument:e.initialArgument,value:e.initialValue,seriesName:e.series.name,originalOpenValue:e.originalOpenValue,originalCloseValue:e.originalCloseValue,originalLowValue:e.originalLowValue,originalHighValue:e.originalHighValue,originalArgument:e.originalArgument,point:e}},_getFormatObject:function(e){var t=this,n=e.formatValue(t.highValue),i=e.formatValue(t.openValue),r=e.formatValue(t.closeValue),s=e.formatValue(t.lowValue),l=a.default,u=l._getFormatObject.call(t,e);return(0,o.extend)({},u,{valueText:"h: "+n+(""!==i?" o: "+i:"")+(""!==r?" c: "+r:"")+" l: "+s,highValueText:n,openValueText:i,closeValueText:r,lowValueText:s})},getMaxValue:function(){return this.highValue},getMinValue:function(){return this.lowValue}});t.default=f,e.exports=t.default},function(e,t,n){t.ThemeManager=void 0;var i=n(4),o=n(1),a=n(0),r=n(322),s=n(14),l=r.BaseThemeManager.inherit(function(){var e=function(e){var t=this;t.callBase.apply(t,arguments);var n=e.options||{};t._userOptions=n,t._mergeAxisTitleOptions=[],t._multiPieColors={},t._callback=i.noop},t=function(){var e=this;return e.palette&&e.palette.dispose(),e.palette=e._userOptions=e._mergedSettings=e._multiPieColors=null,e.callBase.apply(e,arguments)},n=function(){this.palette.reset(),this._multiPieColors={}},r=function(e){return(0,o.isString)(e)?{text:e}:e},l=function(e){return e?(e=(0,a.extend)(!0,{},e),e.title=r(e.title),("logarithmic"===e.type&&e.logarithmBase<=0||e.logarithmBase&&!(0,o.isNumeric)(e.logarithmBase))&&(e.logarithmBase=void 0,e.logarithmBaseError=!0),e.label&&e.label.alignment&&(e.label.userAlignment=!0),e):{}},u=function(e,t,n){var i=this._theme,r=n^"valueAxis"===e?"verticalAxis":"horizontalAxis",s=l(t,e),u=l(this._userOptions.commonAxisSettings,e),c=(0,
a.extend)(!0,{},i.commonAxisSettings,i[r],i[e],u,s);return c.workWeek=s.workWeek||i[e].workWeek,c.forceUserTickInterval|=(0,o.isDefined)(s.tickInterval)&&!(0,o.isDefined)(s.axisDivisionFactor),c},c=function(e,t){t=t||this._userOptions[e];var n=this._theme[e],i=this._mergedSettings[e];return i?i:(i=(0,o.isPlainObject)(n)&&(0,o.isPlainObject)(t)?(0,a.extend)(!0,{},n,t):(0,o.isDefined)(t)?t:n,this._mergedSettings[e]=i,i)},d={base:c,argumentAxis:u,valueAxisRangeSelector:function(){return c.call(this,"valueAxis")},valueAxis:u,series:function(e,t,n){var i,o,r=this,l=r._theme,u=r._userOptions.commonSeriesSettings||{},c=l.commonSeriesSettings,h=r._themeSection.split(".").slice(-1)[0],f=(0,s.normalizeEnum)(t.type||u.type||c.type||"pie"===h&&l.type),p=r.palette,g=~f.indexOf("bar"),_=~f.indexOf("line"),m=~f.indexOf("area"),v="bubble"===f,y=r.getOptions("resolveLabelsOverlapping"),b=r.getOptions("containerBackgroundColor"),x=d.seriesTemplate.call(this);(g||v)&&(t=(0,a.extend)(!0,{},u,u[f],t),o=t.visible,u={type:{}},(0,a.extend)(!0,t,t.point),t.visible=o);var w=(0,a.extend)(!0,{aggregation:{}},c,c[f],u,u[f],t);return w.aggregation.enabled="chart"===h&&!!w.aggregation.enabled,w.type=f,w.widgetType=h,w.containerBackgroundColor=b,i="pie"!==h?w.color||p.getNextColor(n):function(e,t,n){var i="".concat(e,"-").concat(t);return r._multiPieColors[i]||(r._multiPieColors[i]=p.getNextColor(n)),r._multiPieColors[i]},w.mainSeriesColor=i,w.resolveLabelsOverlapping=y,w.label&&(_||m&&"rangearea"!==f||"scatter"===f)&&(w.label.position="outside"),x&&(w.nameField=x.nameField),w},animation:function(e){var t=this._userOptions[e];return t=(0,o.isPlainObject)(t)?t:(0,o.isDefined)(t)?{enabled:!!t}:{},c.call(this,e,t)},seriesTemplate:function(){var e=c.call(this,"seriesTemplate");return e&&(e.nameField=e.nameField||"series"),e},zoomAndPan:function(){function e(e){e=(0,s.normalizeEnum)(e);var t="pan"===e||"both"===e,n="zoom"===e||"both"===e;return{pan:t,zoom:n,none:!t&&!n}}var t=c.call(this,"zoomAndPan");return{valueAxis:e(t.valueAxis),argumentAxis:e(t.argumentAxis),dragToZoom:!!t.dragToZoom,dragBoxStyle:{"class":"dxc-shutter",fill:t.dragBoxStyle.color,opacity:t.dragBoxStyle.opacity},panKey:t.panKey,allowMouseWheel:!!t.allowMouseWheel,allowTouchGestures:!!t.allowTouchGestures}}};return{_themeSection:"chart",ctor:e,dispose:t,resetPalette:n,getOptions:function(e){return(d[e]||d.base).apply(this,arguments)},refresh:function(){return this._mergedSettings={},this.callBase.apply(this,arguments)},_initializeTheme:function(){var e=this;e.callBase.apply(e,arguments),e.updatePalette()},resetOptions:function(e){this._mergedSettings[e]=null},update:function(e){this._userOptions=e},updatePalette:function(){var e=this;e.palette=e.createPalette(e.getOptions("palette"),{useHighlight:!0,extensionMode:e.getOptions("paletteExtensionMode")})}}}());t.ThemeManager=l},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i;return i=n<e?e:t<n?t:n}function a(e){if(e._label.isVisible()&&"inside"!==e._label.getLayoutOptions().position)return e._label.getBoundingRect()}function r(e,t,n,i,r){return e.some(function(e){return e.getVisiblePoints().reduce(function(e,s){var l=a(s);if(l){var c=o(l.x,l.x+l.width,t),d=o(l.y,l.y+l.height,n);i=x(w(u(c,d,t,n)-C,r),i),e=!0}return e},!1)}),i}function s(e){return e.reduce(function(e,t){var n=t.getVisiblePoints().reduce(function(e,t){var n=a(t);return n&&n.width>e&&(e=n.width),e},0),i=n;return n&&(e.outerLabelsCount++,e.outerLabelsCount>1&&(n+=m.default.pieLabelSpacing),i+=m.default.pieLabelSpacing),e.sizes.push(n),e.rSizes.push(i),e.common+=n,e},{sizes:[],rSizes:[],common:0,outerLabelsCount:0})}function l(e,t,n,i,o,a){var r,s,l=0,u=e.sizes,c=e.rSizes;for(s=0;s<n.length;s++)0!==u[s]?(r=y(r?r+c[s-1]:t),n[s].correctLabelRadius(r),l+=o||u[s],c[s]=o||c[s],n[s].setVisibleArea({left:y(a-t-l),right:y(i.width-(a+t+l)),top:i.top,bottom:i.bottom,width:i.width,height:i.height})):r&&(r+=c[s-1])}function u(e,t,n,i){return b((e-n)*(e-n)+(t-i)*(t-i))}function c(e){var t;return"pie"===e.type?t=0:(t=(0,_.isNumeric)(e.innerRadius)?Number(e.innerRadius):.5,t=t<.2?.2:t,t=t>.8?.8:t),t}function d(){}function h(e,t,n,i){return(e-t-C-n.left)/i.outerLabelsCount}function f(e,t,n){return e-t.left-(n.outerLabelsCount>0?n.common+C:0)}function p(e,t,n,i,o,a){var u,c=s(n),d=f(o,t,c);return d<i?(e=i,u=h(o,e,t,c)):e=x(r(n,o,a,e,i),d),l(c,e+C,n,t,u,o),e}function g(e){return new v.WrapperLayoutElement(null,{x:e.left,y:e.top,width:e.width-e.left-e.right,height:e.height-e.top-e.bottom})}t.LayoutManager=d;var _=n(1),m=i(n(151)),v=n(323),y=Math.floor,b=Math.sqrt,x=Math.min,w=Math.max,C=m.default.radialLabelIndent;d.prototype={constructor:d,setOptions:function(e){this._options=e},applyPieChartSeriesLayout:function(e,t,n){var i,o,a=e.height-e.top-e.bottom,r=e.width-e.left-e.right,s=r/2+e.left,l=a/2+e.top,u=this._options.piePercentage;return(0,_.isNumeric)(u)?i=o=u*x(e.height,e.width)/2:(i=x(r,a)/2,o=this._options.minPiePercentage*i),n||(i=p(i,e,t,o,s,l)),{centerX:y(s),centerY:y(l),radiusInner:y(i*c(t[0])),radiusOuter:y(i)}},applyEqualPieChartLayout:function(e,t){var n=t.radius;return{centerX:y(t.x),centerY:y(t.y),radiusInner:y(n*c(e[0])),radiusOuter:y(n)}},correctPieLabelRadius:function(e,t,n){var i,o=s(e),a=t.radiusOuter+C,r=t.centerX-n.left-a;o.common+C>r&&(i=h(t.centerX,t.radiusOuter,n,o)),l(o,a,e,n,i,t.centerX)},needMoreSpaceForPanesCanvas:function(e,t,n){var i=this._options,o=i.width,a=i.height,r=i.piePercentage,s=(0,_.isNumeric)(r),l=0,u=0;return e.forEach(function(e){var i=e.canvas,c=s?x(i.width,i.height)*r:void 0,d=n?n(e):{width:!1,height:!1},h=d.width?0:(s?c:o)-(i.width-i.left-i.right),f=d.height?0:(s?c:a)-(i.height-i.top-i.bottom);t?(l+=h>0?h:0,u=w(f>0?f:0,u)):(l=w(h>0?h:0,l),u+=f>0?f:0)}),(l>0||u>0)&&{width:l,height:u}},layoutInsideLegend:function(e,t){var n={left:"right",right:"left",top:"bottom",bottom:"top",center:"center"},i=e.getLayoutOptions();if(i){var o=i.position,a=i.cutSide,r={horizontal:o.horizontal,vertical:o.vertical};t[i.cutLayoutSide]+="horizontal"===i.cutSide?i.width:i.height,r[a]=n[r[a]],e.position({of:g(t),my:r,at:o})}}}},function(e,t,n){function i(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function o(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?i(Object(n),!0).forEach(function(t){a(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):i(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function a(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function u(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function c(e,t,n){return t&&u(e.prototype,t),n&&u(e,n),e}function d(e,t,n){return"a ".concat(e," ").concat(e," 0 0 1 ").concat(t*e," ").concat(n*e)}function h(e,t,n){return"A ".concat(e," ").concat(e," 0 0 1 ").concat(t," ").concat(n)}function f(e,t,n,i,o){return(e-i)*x(S(n))+(t-o)*x(k(n))+i}function p(e,t,n,i,o){return-(e-i)*x(k(n))+(t-o)*x(S(n))+o}function g(e,t){return t%90===0&&t%180!==0?{width:e.height,height:e.width}:e}function _(e,t,n,i,o){var a=e.width,r=e.height,s=a/2,l=r/2,u=Math.ceil(t+s),c=Math.floor(t-s),d=Math.floor(n-l),h=Math.ceil(n+l);return i<c&&o<d||i>=c&&i<=u&&o<d?270:i>u&&o>h||i>=c&&i<=u&&o>h?90:i<c&&o>h||i<c&&o>=d&&o<=h?180:0}function m(e,t,n,i,o,a,r){var s=e.width,l=e.height,u=a.arrowWidth,c=a.cornerRadius,f=void 0===c?0:c,p=u/2,g=s/2,_=l/2,m=Math.ceil(t+g),v=Math.floor(t-g),y=Math.floor(n-_),b=Math.ceil(n+_),x=[v,y],O=[m,y],E=[m,b],A=[v,b],P=i<=v?v:m<=i?m:i,R=o<=y?y:b<=o?b:o,M=C(R+p,b),B=w(R-p,y),V=w(P-p,v);f=Math.min(s/2,l/2,f);var F;if(x[1]+=f,O[0]-=f,E[1]-=f,A[0]+=f,!r||v<=i&&i<=m&&y<=o&&o<=b)F=T(x,d(f,1,-1),"L",O,d(f,1,1),"L",E,d(f,-1,1),"L",A,d(f,-1,-1));else if(i>m&&o<y){var L=u/f||0,H=I/4+L/2,z=I/4-L/2,N=O[0]+S(z)*f,j=O[1]+(1-k(z))*f,W=T("L",O,d(f,S(H),1-k(H)),"L",[i,o,N,j],h(f,O[0]+f,O[1]+f));Math.abs(H)>I/2&&(W=T("L",[V,y,i,o,m,M])),F=T(x,d(f,1,-1),W,"L",E,d(f,-1,1),"L",A,d(f,-1,-1))}else if(i>m&&o>=y&&o<=b){var G;if(B>=O[1]+f&&M<=E[1])G=T(d(f,1,1),"L",[m,B,i,o,m,M],"L",E,d(f,-1,1));else if(B<O[1]+f&&M>=O[1]+f&&M<=E[1]){var q=O[1]+f-B,$=q/f,K=O[0]+S($)*f,U=O[1]+(1-k($))*f;G=T(d(f,S($),1-k($)),"L",[K,U,i,o,m,M],"L",E,d(f,-1,1))}else if(B<O[1]+f&&M<O[1]+f){var Y=O[1]+f-B,Z=Y/f,X=Z,Q=O[0]+S(X)*f,J=O[1]+(1-k(X))*f,ee=Math.sin((O[1]+f-M)/f),te=O[0]+f*S(ee),ne=O[1]+f*(1-k(ee));G=T(d(f,S(X),1-k(X)),"L",[Q,J,i,o,te,ne],h(f,O[0]+f,O[1]+f),"L",E,d(f,-1,1))}else if(B<=O[1]+f&&M>=E[1]){var ie=D((O[1]+f-B)/f),oe=O[0]+f*S(ie),ae=O[1]+f*(1-k(ie)),re=D((M-E[1])/f),se=E[0]+f*(S(re)-1),le=E[1]+f*k(re);G=T(d(f,S(ie),1-k(ie)),"L",[oe,ae,i,o,se,le],h(f,E[0]-f,E[1]+f))}else if(B>O[1]+f&&B<=E[1]&&M>E[1]){var ue=D((M-E[1])/f),ce=E[0]+f*(S(ue)-1),de=E[1]+f*k(ue);G=T(d(f,1,1),"L",[m,B,i,o,ce,de],h(f,E[0]-f,E[1]+f))}else if(B>O[1]+f&&M>E[1]){var he=D((M-E[1])/f),fe=E[0]+f*(S(he)-1),pe=E[1]+f*k(he),ge=D((B-E[1])/f),_e=E[0]+f*(S(ge)-1),me=E[1]+f*k(ge);G=T(d(f,1,1),"L",E,d(f,S(ge)-1,k(ge)),"L",[_e,me,i,o,fe,pe],h(f,E[0]-f,E[1]+f))}F=T(x,d(f,1,-1),"L",O,G,"L",A,d(f,-1,-1))}return T("M",F,"Z")}t.Plaque=void 0;var v=n(0),y=n(1),b=Math,x=b.round,w=b.max,C=b.min,k=b.sin,S=b.cos,D=b.asin,I=b.PI,T=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return t.join("")},O=function(){function e(t,n,i,o){var a=!(arguments.length>4&&void 0!==arguments[4])||arguments[4],r=arguments.length>5&&void 0!==arguments[5]?arguments[5]:function(e,t){return t.getBBox()},s=arguments.length>6&&void 0!==arguments[6]?arguments[6]:function(e,t,n,i){return t.move(n,i)};l(this,e),this.widget=n,this.options=t,this.root=i,this.contentTemplate=o,this.bonded=a,this.measureContent=r,this.moveContentGroup=s}return c(e,[{key:"draw",value:function(e){var t=this,n=e.x,i=e.y,a=e.canvas,s=void 0===a?{}:a,l=e.offsetX,u=e.offsetY,c=e.offset,d=void 0===c?0:c,h=r(e,["x","y","canvas","offsetX","offsetY","offset"]),f=this.options,p=f.x,g=f.y,_={xl:s.left,xr:s.width-s.right,width:s.width-s.right-s.left,yt:s.top,yb:s.height-s.bottom,height:s.height-s.bottom-s.top};if(!((0,y.isDefined)(n)&&(0,y.isDefined)(i)||(0,y.isDefined)(p)&&(0,y.isDefined)(g)))return!1;if((0,y.isDefined)(n)&&(n<_.xl||_.xr<n||i<_.yt||_.yb<i))return!1;this._root||this._draw();var m=(0,v.extend)({x:"-50%",y:"-50%",width:"200%",height:"200%"},f.shadow),b=f.width>0?f.width:null,k=f.height>0?f.height:null,S=function(){var e,o=t._contentBBox=t.measureContent(t.widget,t._contentGroup),a=t._size={width:w(b,o.width)+2*f.paddingLeftRight,height:w(k,o.height)+2*f.paddingTopBottom,offset:d},r=m.offsetX,s=m.offsetY,c=2*m.blur+1,h=w(c-r,0),v=w(c+r,0),S=w(c-s,0),D=w(c+s,0);if(t.margins={lm:h,rm:v,tm:S,bm:D},(0,y.isDefined)(p)?(p+=l||0,(0,y.isDefined)(n)||(n=p)):p=(0,y.isDefined)(l)?n+l:_.width<a.width?x(_.xl+_.width/2):C(w(n,Math.ceil(_.xl+a.width/2+h)),Math.floor(_.xr-a.width/2-v)),(0,y.isDefined)(g))g+=u||0,(0,y.isDefined)(i)||(i=g+a.height/2);else if((0,y.isDefined)(u))g=i+u;else{var I=i-f.arrowLength-a.height/2-d,T=i+f.arrowLength+a.height/2+d;_.height<a.height+f.arrowLength?g=x(_.yt+a.height/2):I-a.height/2-S<_.yt?T+a.height/2+D<_.yb?(g=T,i+=d):g=x(_.yt+a.height/2):(g=I,i-=d)}t.anchorX=n,t.anchorY=i,t.move(p,g),null===(e=t._root)||void 0===e?void 0:e.append(t.root)};return this.contentTemplate.render?(this.contentTemplate.render({model:f,container:this._contentGroup.element,onRendered:S}),!0):this.contentTemplate(o({group:this._contentGroup,onRender:S},h))}},{key:"_draw",value:function(){var e=this.widget._renderer,t=this.options,n=(0,v.extend)({x:"-50%",y:"-50%",width:"200%",height:"200%"},t.shadow),i=this._shadow=e.shadowFilter().attr(n),o={opacity:t.opacity,"stroke-width":0,fill:t.color},a=t.border||{};a.visible&&(0,v.extend)(o,{"stroke-width":a.width,stroke:a.color,"stroke-opacity":a.opacity,dashStyle:a.dashStyle});var r=this._root=e.g().append(this.root);t.type&&r.attr({"class":"dxc-".concat(t.type,"-annotation")});var s=e.g().attr({filter:i.id}).append(r);this._cloud=e.path([],"area").attr(o).sharp().append(s),this._contentGroup=e.g().append(r)}},{key:"getBBox",value:function(){var e=this._size||{},t=this.margins||{},n=_(e,this.x,this.y,this.anchorX,this.anchorY);return{x:Math.floor(this.x-e.width/2-t.lm),y:Math.floor(this.y-e.height/2-t.tm-(270===n?this.options.arrowLength:0)),width:e.width+t.lm+t.rm,height:e.height+t.tm+t.bm+(90===n||270===n?this.options.arrowLength:0)}}},{key:"clear",value:function(){return this._root&&(this._root.remove(),this._shadow.remove(),this._root=null),this}},{key:"customizeCloud",value:function(e){this._cloud&&this._cloud.attr(e)}},{key:"moveRoot",value:function(e,t){this._root&&this._root.move(e,t)}},{key:"move",value:function(e,t){e=x(e),t=x(t),this.x=e,this.y=t;var n=_(this._size,e,t,this.anchorX,this.anchorY),i=n*I/180;this._cloud.attr({d:m(g(this._size,n),e,t,f(this.anchorX,this.anchorY,i,e,t),p(this.anchorX,this.anchorY,i,e,t),this.options,this.bonded)}).rotate(n,e,t),this.moveContentGroup(this.widget,this._contentGroup,e-this._contentBBox.x-this._contentBBox.width/2,t-this._contentBBox.y-this._contentBBox.height/2)}},{key:"hitTest",value:function(e,t){var n=this._size||{},i=n.width,o=n.height;return Math.abs(e-this.x)<=i/2&&Math.abs(t-this.y)<=o/2}}]),e}();t.Plaque=O},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?o(Object(n),!0).forEach(function(t){r(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):o(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function s(e){return k(e)?0===e.length?[{}]:e:[e]}function l(e,t){return e.processBubbleSize&&(e.size=t),e}function u(e,t,n,i){var o=i?e.width/t:e.width,a=i?e.height:e.height/t;return Math.min(o,a)*n}function c(e,t,n,i){var o,a;if(n){a=e.value,o={skipEventRising:!0};var s=e.fullName.split("."),l=s[s.length-1];"endValue"!==l&&"startValue"!==l||(o={allowPartialUpdate:!0},a=r({},l,e.value))}else a=((0,x.isDefined)(i)?e.value[i]:e.value).visualRange;t.visualRange(a,o)}function d(e,t,n){if(n)return{argumentAxisType:e.argumentAxisType,argumentType:e.argumentType};var i=e.groups.filter(function(e){return e.valueAxis===t})[0],o=i.valueAxisType,a=i.valueType;return{valueAxisType:o,valueType:a}}t.AdvancedChart=void 0;var h=n(0),f=n(12),p=n(3),g=n(134),_=n(232),m=n(555),v=n(321),y=n(556),b=i(n(230)),x=n(1),w=n(4),C=n(14),k=Array.isArray,S="defaultAxisName",D="font",I="commonAxisSettings",T="default",O="VISUAL_RANGE",E=v.BaseChart.inherit({_fontFields:[I+".label."+D,I+".title."+D],_partialOptionChangesMap:{visualRange:O,_customVisualRange:O,strips:"REFRESH_AXES",constantLines:"REFRESH_AXES"},_partialOptionChangesPath:{argumentAxis:["strips","constantLines","visualRange","_customVisualRange"],valueAxis:["strips","constantLines","visualRange","_customVisualRange"]},_initCore:function(){this._panesClipRects={},this.callBase()},_disposeCore:function(){var e=this._disposeObjectsInArray,t=this._panesClipRects;this.callBase(),e.call(t,"fixed"),e.call(t,"base"),e.call(t,"wide"),this._panesClipRects=null},_dispose:function(){var e=this,t=this._disposeObjectsInArray;e.callBase(),e.panes=null,e._legend&&(e._legend.dispose(),e._legend=null),t.call(e,"panesBackground"),t.call(e,"seriesFamilies"),e._disposeAxes()},_createPanes:function(){this._cleanPanesClipRects("fixed"),this._cleanPanesClipRects("base"),this._cleanPanesClipRects("wide")},_cleanPanesClipRects:function(e){var t=this._panesClipRects[e];(t||[]).forEach(function(e){return e&&e.dispose()}),this._panesClipRects[e]=[]},_getElementsClipRectID:function(e){var t=this._panesClipRects.fixed[this._getPaneIndex(e)];return t&&t.id},_getPaneIndex:function(e){var t,n=e||T;return(0,p.each)(this.panes,function(e,i){if(i.name===n)return t=e,!1}),t},_updateSize:function(){this.callBase(),(0,C.setCanvasValues)(this._canvas)},_reinitAxes:function(){this.panes=this._createPanes(),this._populateAxes(),this._axesReinitialized=!0},_getCrosshairMargins:function(){var e=this._getCrosshairOptions()||{},t=e.enabled,n=(0,y.getMargins)();return{x:t&&e.horizontalLine.visible?n.x:0,y:t&&e.verticalLine.visible?n.y:0}},_populateAxes:function(){function e(){return S+h++}var t,n=this,i=n.panes,o=n._isRotated(),a=s(n.option("argumentAxis")||{})[0],r=n.option("valueAxis"),l=s(r||{}),u=[],c=[],d=[],h=0,g=n._getCrosshairMargins();t=o?"right"===a.position?i[i.length-1].name:i[0].name:"top"===a.position?i[0].name:i[i.length-1].name,u=(0,C.map)(i,function(e){var i=e.name!==t;return n._populateAxesOptions("argumentAxis",a,{pane:e.name,name:null,optionPath:"argumentAxis",crosshairMargin:o?g.x:g.y},o,i)}),(0,p.each)(l,function(t,i){var a,s=[],l=i.name;return l&&(0,f.inArray)(l,d)!==-1?void n._incidentOccurred("E2102"):(l&&d.push(l),i.pane&&s.push(i.pane),null!==(a=i.panes)&&void 0!==a&&a.length&&(s=s.concat(i.panes.slice(0))),s=(0,C.unique)(s),s.length||s.push(void 0),void(0,p.each)(s,function(a,s){var u=k(r)?"valueAxis[".concat(t,"]"):"valueAxis";c.push(n._populateAxesOptions("valueAxis",i,{name:l||e(),pane:s,priority:t,optionPath:u,crosshairMargin:o?g.y:g.x},o))}))}),n._redesignAxes(u,!0,t),n._redesignAxes(c,!1)},_redesignAxes:function(e,t,n){var i=this,o=[],a=t?i._argumentAxes:i._valueAxes;(0,p.each)(e,function(e,n){var r=a&&a.filter(function(e){return e.name===n.name&&(!(0,x.isDefined)(n.pane)&&i.panes.some(function(t){return t.name===e.pane})||e.pane===n.pane)});r&&r.length>0?(0,p.each)(r,function(e,a){var r=d(i._groupsData,a,t);a.updateOptions(n),t?a.setTypes(r.argumentAxisType,r.argumentType,"argumentType"):a.setTypes(r.valueAxisType,r.valueType,"valueType"),a.validate(),o.push({axis:a})}):o.push({options:n})}),a?(0,p.reverseEach)(a,function(e,n){o.some(function(e){return e.axis&&e.axis===n})||i._disposeAxis(e,t)}):a=t?i._argumentAxes=[]:i._valueAxes=[],(0,p.each)(o,function(e,o){var r=o.axis;o.axis&&t?o.axis.isVirtual=o.axis.pane!==n:o.options&&(r=i._createAxis(t,o.options,t?o.options.pane!==n:void 0),a.push(r)),r.applyVisualRangeSetter(i._getVisualRangeSetter())})},_disposeAxis:function(e,t){var n=t?this._argumentAxes:this._valueAxes,i=n[e];i&&(i.dispose(),n.splice(e,1))},_disposeAxes:function(){var e=this,t=e._disposeObjectsInArray;t.call(e,"_argumentAxes"),t.call(e,"_valueAxes")},_appendAdditionalSeriesGroups:function(){this._crosshairCursorGroup.linkAppend(),this._scrollBar&&this._scrollBarGroup.linkAppend()},_getLegendTargets:function(){var e=this;return(this.series||[]).map(function(t){var n=e._getLegendOptions(t);return n.legendData.series=t,t.getOptions().showInLegend||(n.legendData.visible=!1),n})},_legendItemTextField:"name",_seriesPopulatedHandlerCore:function(){this._processSeriesFamilies(),this._processValueAxisFormat()},_renderTrackers:function(){var e,t=this;for(e=0;e<t.series.length;++e)t.series[e].drawTrackers()},_specialProcessSeries:function(){this._processSeriesFamilies()},_processSeriesFamilies:function(){var e,t,n=this,i=[],o=[],a=n._themeManager,r=a.getOptions("negativesAsZeroes"),s=a.getOptions("negativesAsZeros"),l={minBubbleSize:a.getOptions("minBubbleSize"),maxBubbleSize:a.getOptions("maxBubbleSize"),barGroupPadding:a.getOptions("barGroupPadding"),barGroupWidth:a.getOptions("barGroupWidth"),negativesAsZeroes:(0,x.isDefined)(r)?r:s};return null!==(e=n.seriesFamilies)&&void 0!==e&&e.length?void(0,p.each)(n.seriesFamilies,function(e,t){t.updateOptions(l),t.adjustSeriesValues()}):((0,p.each)(n.series,function(e,t){(0,f.inArray)(t.type,i)===-1&&i.push(t.type)}),(0,p.each)(n._getLayoutTargets(),function(e,a){t=n._getSeriesForPane(a.name),(0,p.each)(i,function(e,i){var r=new m.SeriesFamily({type:i,pane:a.name,minBubbleSize:l.minBubbleSize,maxBubbleSize:l.maxBubbleSize,barGroupPadding:l.barGroupPadding,barGroupWidth:l.barGroupWidth,negativesAsZeroes:l.negativesAsZeroes,rotated:n._isRotated()});r.add(t),r.adjustSeriesValues(),o.push(r)})}),void(n.seriesFamilies=o))},_updateSeriesDimensions:function(){var e,t=this,n=t.seriesFamilies||[];for(e=0;e<n.length;e++){var i=n[e];i.updateSeriesValues(),i.adjustSeriesDimensions()}},_getLegendCallBack:function(e){return this._legend&&this._legend.getActionCallback(e)},_appendAxesGroups:function(){var e=this;e._stripsGroup.linkAppend(),e._gridGroup.linkAppend(),e._axesGroup.linkAppend(),e._constantLinesGroup.linkAppend(),e._labelAxesGroup.linkAppend(),e._scaleBreaksGroup.linkAppend()},_populateMarginOptions:function(){var e=this,t=u(e.getSize(),e.panes.length,e._themeManager.getOptions("maxBubbleSize"),e._isRotated()),n={};e._valueAxes.forEach(function(i){var o=e.series.filter(function(e){return e.getValueAxis()===i}),a={};o.forEach(function(e){if(e.isVisible()){var i=l(e.getMarginOptions(),t);a=(0,C.mergeMarginOptions)(a,i),n=(0,C.mergeMarginOptions)(n,i)}}),i.setMarginOptions(a)}),e._argumentAxes.forEach(function(e){return e.setMarginOptions(n)})},_populateBusinessRange:function(e,t){var n=this,i=n._isRotated(),o=n._getVisibleSeries(),r={},s=new g.Range({rotated:!!i}),l=function(e){return e.pane||T};if(n.panes.forEach(function(e){return r[e.name]=new g.Range({rotated:!!i})}),n._valueAxes.forEach(function(a){var s=new g.Range({rotated:!!i,pane:a.pane,axis:a.name}),u=o.filter(function(e){return e.getValueAxis()===a});u.forEach(function(e){var t=e.getRangeData();s.addRange(t.val),r[l(a)].addRange(t.arg)}),(!e||e&&u.length&&a===e)&&(a.setGroupSeries(u),a.setBusinessRange(s,n._axesReinitialized||t,n._argumentAxes[0]._lastVisualRangeUpdateMode))}),!e||e&&o.length){Object.keys(r).forEach(function(e){return s.addRange(r[e])});var u=s.interval;n._argumentAxes.forEach(function(e){var t,i=null!==(t=r[l(e)].interval)&&void 0!==t?t:u;e.setBusinessRange(new g.Range(a(a({},s),{},{interval:i})),n._axesReinitialized,void 0,n._groupsData.categories)})}n._populateMarginOptions()},getArgumentAxis:function(){return(this._argumentAxes||[]).filter(function(e){return!e.isVirtual})[0]},getValueAxis:function(e){var t=this;return(this._valueAxes||[]).filter((0,x.isDefined)(e)?function(t){return t.name===e}:function(e){return e.pane===t.defaultPane})[0]},_getGroupsData:function(){var e=this,t=[];return e._valueAxes.forEach(function(n){t.push({series:e.series.filter(function(e){return e.getValueAxis()===n}),valueAxis:n,valueOptions:n.getOptions()})}),{groups:t,argumentAxes:e._argumentAxes,argumentOptions:e._argumentAxes[0].getOptions()}},_groupSeries:function(){var e=this;e._correctValueAxes(!1),e._groupsData=e._getGroupsData()},_processValueAxisFormat:function(){var e=[];this.series.forEach(function(t){var n=t.getValueAxis();t.isFullStackedSeries()&&(n.setPercentLabelFormat(),e.push(n))}),this._valueAxes.forEach(function(t){e.indexOf(t)===-1&&t.resetAutoLabelFormat()})},_populateAxesOptions:function(e,t,n,i,o){var a=this,r=a._prepareStripsAndConstantLines(e,t,i),s=(0,h.extend)(!0,{},r,n,a._prepareAxisOptions(e,r,i));return o&&(s.visible=s.tick.visible=s.minorTick.visible=s.label.visible=!1,s.title={}),s},_getValFilter:function(e){return b.default.getViewPortFilter(e.getValueAxis().visualRange()||{})},_createAxis:function(e,t,n){var i=this,o=e?"argumentAxis":"valueAxis",a=(0,h.extend)({renderer:i._renderer,incidentOccurred:i._incidentOccurred,eventTrigger:i._eventTrigger,axisClass:e?"arg":"val",widgetClass:"dxc",stripsGroup:i._stripsGroup,labelAxesGroup:i._labelAxesGroup,constantLinesGroup:i._constantLinesGroup,scaleBreaksGroup:i._scaleBreaksGroup,axesContainerGroup:i._axesGroup,gridGroup:i._gridGroup,isArgumentAxis:e,getTemplate:function(){return i._getTemplate(t.label.template)}},i._getAxisRenderingOptions(o)),r=new _.Axis(a);return r.updateOptions(t),r.isVirtual=n,r},_applyVisualRangeByVirtualAxes:function(e,t){return!1},_applyCustomVisualRangeOption:function(e,t){var n=this;e.getOptions().optionPath&&n._parseVisualRangeOption("".concat(e.getOptions().optionPath,".visualRange"),t)},_getVisualRangeSetter:function(){var e=this;return function(t,n){var i=n.skipEventRising,o=n.range;e._applyCustomVisualRangeOption(t,o),t.setCustomVisualRange(o),t.skipEventRising=i,e._applyVisualRangeByVirtualAxes(t,o)||(e._applyingChanges?e._change_VISUAL_RANGE():e._requestChange([O]))}},_getTrackerSettings:function(){return(0,h.extend)(this.callBase(),{argumentAxis:this.getArgumentAxis()})},_prepareStripsAndConstantLines:function(e,t,n){return t=this._themeManager.getOptions(e,t,n),t.strips&&(0,p.each)(t.strips,function(e){t.strips[e]=(0,h.extend)(!0,{},t.stripStyle,t.strips[e])}),t.constantLines&&(0,p.each)(t.constantLines,function(e,n){t.constantLines[e]=(0,h.extend)(!0,{},t.constantLineStyle,n)}),t},refresh:function(){this._disposeAxes(),this.callBase()},_layoutAxes:function(e){var t=this;e();var n=t.checkForMoreSpaceForPanesCanvas();if(n){var i=this._rect.slice(),o=this._layout.backward(i,i,[n.width,n.height]);n.width=Math.max(0,o[0]),n.height=Math.max(0,o[1]),this._canvas=this._createCanvasFromRect(i),e(n)}},checkForMoreSpaceForPanesCanvas:function(){return this.layoutManager.needMoreSpaceForPanesCanvas(this._getLayoutTargets(),this._isRotated())},_parseVisualRangeOption:function(e,t){var n=this,i=e.split(/[.[]/)[0],o=e.match(/\d+/g);o=(0,x.isDefined)(o)?parseInt(o[0]):o,e.indexOf("visualRange")>0?n._setCustomVisualRange(i,o,t):("object"===(0,x.type)(t)||k(t))&&i.indexOf("Axis")>0&&JSON.stringify(t).indexOf("visualRange")>0&&((0,x.isDefined)(t.visualRange)?n._setCustomVisualRange(i,o,t.visualRange):k(t)&&t.forEach(function(e,t){return(0,x.isDefined)(e.visualRange)&&n._setCustomVisualRange(i,t,e.visualRange)}))},_setCustomVisualRange:function(e,t,n){var i=this,o=i._options.silent(e);o&&((0,x.isDefined)(t)?o[t]._customVisualRange=n:o._customVisualRange=n,i._axesReinitialized=!0)},_raiseZoomEndHandlers:function(){this._valueAxes.forEach(function(e){return e.handleZoomEnd()})},_setOptionsByReference:function(){this.callBase(),(0,h.extend)(this._optionsByReference,{"valueAxis.visualRange":!0})},_notifyOptionChanged:function(e,t,n){this.callBase.apply(this,arguments),this._optionChangedLocker||this._parseVisualRangeOption(e,t)},_notifyVisualRange:function(){var e=this;e._valueAxes.forEach(function(t){if(t.getOptions().optionPath){var n="".concat(t.getOptions().optionPath,".visualRange"),i=(0,C.convertVisualRangeObject)(t.visualRange(),!k(e.option(n)));t.skipEventRising&&(0,C.rangesAreEqual)(i,e.option(n))?t.skipEventRising=null:e.option(n,i)}})},_notify:function(){this.callBase(),this._axesReinitialized=!1,this.option("disableTwoWayBinding")!==!0&&this._notifyVisualRange()},_getAxesForScaling:function(){return this._valueAxes},_getAxesByOptionPath:function(e,t,n){var i=this,o=i._getAxesForScaling(),a=[];if(t){var r;e.fullName&&(r=e.fullName.slice(0,e.fullName.indexOf("."))),a=o.filter(function(e){return e.getOptions().optionPath===r})}else"object"===(0,x.type)(e.value)?a=o.filter(function(t){return t.getOptions().optionPath===e.name}):k(e.value)&&e.value.forEach(function(t,i){var r=o.filter(function(t){return t.getOptions().optionPath==="".concat(e.name,"[").concat(i,"]")})[0];(0,x.isDefined)(t[n])&&(0,x.isDefined)(r)&&(a[i]=r)});return a},_optionChanged:function(e){var t=this;if(!t._optionChangedLocker){var n,i="visualRange",o=e.fullName.indexOf(i)>0||!(t.getPartialChangeOptionsName(e).indexOf(i)>-1)&&void 0;(0,x.isDefined)(o)&&(n=t._getAxesByOptionPath(e,o,i),n&&(n.length>1||k(e.value)?n.forEach(function(t,n){return c(e,t,o,n)}):1===n.length&&c(e,n[0],o)))}t.callBase(e)},_change_VISUAL_RANGE:function(){var e=this;if(e._recreateSizeDependentObjects(!1),!e._changes.has("FULL_RENDER")){var t=e.option("resizePanesOnZoom");e._doRender({force:!0,drawTitle:!1,drawLegend:!1,adjustAxes:null!==t&&void 0!==t?t:e.option("adjustAxesOnZoom")||!1,animate:!1}),e._raiseZoomEndHandlers()}},resetVisualRange:function(){var e=this;e._valueAxes.forEach(function(t){t.resetVisualRange(!1),e._applyCustomVisualRangeOption(t)}),e._requestChange([O])},_legendDataField:"series",_adjustSeriesLabels:w.noop,_correctValueAxes:w.noop});t.AdvancedChart=E},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){var o,l,u,c,d,h=T.length;"week"===i?i="day":"quarter"===i?i="month":"shorttime"===i?i="hour":"longtime"===i&&(i="second");var f=T.indexOf(i);if(n){if(l=b.default.getDatesDifferences(t,n),o=b.default.getDatesDifferences(t,e),l[i])for(c=h-1;c>=f;c--)if(u=T[c],c===f)s(l,f+(l.millisecond?2:1));else if(l[u]){r(l,c);break}}else for(o=b.default.getDatesDifferences(e,t),c=h-1;c>=f;c--)if(u=T[c],o[u]){if(c-f>1){for(d=f+1;d>=0;d--)r(o,d);break}if(a(t,u)){for(d=c-1;d>0;d--)r(o,d);break}}return n?l:o}function a(e,t){var n,i=[e.getMilliseconds(),e.getSeconds(),e.getMinutes(),e.getHours(),e.getDate(),e.getMonth()],o=T.indexOf(t);for(n=0;n<o;n++)if(4===n&&1!==i[n]||4!==n&&0!==i[n])return!1;return!0}function r(e,t){var n=T[t];e[n]&&(e[n]=!1,e.count--)}function s(e,t){var n=T[t];e[n]===!1&&(e[n]=!0,e.count++)}function l(e){return e.length-parseInt(e).toString().length}function u(e,t){var n,i,o,a=0;for(o=k(t-e[0]),n=1;n<e.length;n++)i=k(t-e[n]),i<o&&(o=i,a=n);return a}function c(e){return e.toString().split(".")}function d(e){var t;return(0,y.isFunction)(e)&&(t=e,e=null),{type:e,formatter:t}}function h(e,t){var n,i,a,r,s,h,f,p,g,_=t.tickInterval,m=k(e).toString(),T=0,O=0,E=0,A=-1,P=t.labelOptions.format,R=t.ticks,M="logarithmic"===t.type;if(1!==R.length||0!==R.indexOf(e)||(0,y.isDefined)(_)||(_=k(e)>=1?1:(0,x.adjust)(1-k(e),e)),!(0,y.isDefined)(P)&&"discrete"!==t.type&&e&&(10===t.logarithmBase||!M))if("datetime"!==t.dataType&&(0,y.isDefined)(_)){if(R.length&&R.indexOf(e)===-1&&(A=u(R,e),_=(0,x.adjust)(k(e-R[A]),e)),s=c(_),s<2&&(s=c(e)),M)if(f=(0,w.getAdjustedLog10)(k(e)),f>0)r=I[S(f/3)]||D;else{if(!(f<-4))return C((0,x.adjust)(e));r=D}else s.length>1&&!(0,y.isExponential)(_)?(T=s[1].length,r=I[E]):(0,y.isExponential)(_)&&(m.indexOf(".")!==-1||(0,y.isExponential)(e))?(r=D,T=(0,y.isExponential)(e)?Math.max(k((0,x.getExponent)(e)-(0,x.getExponent)(_)),k((0,x.getPrecision)(e)-(0,x.getPrecision)(_))):k(l(m.split(".")[1])-(0,x.getExponent)(_)+1)):(n=S((0,w.getAdjustedLog10)(_)),a=i=S((0,w.getAdjustedLog10)(k(e))),i-n>=2&&(a=n),E=S(a/3),O=3*E,E<5?(n-O===2&&i>=3&&(E++,O=3*E),r=I[E]):r=I[I.length-1],O>0&&(s=c(_/Math.pow(10,O)),s[1]&&(T=s[1].length)));void 0===r&&void 0===T||(P={type:r,precision:T})}else"datetime"===t.dataType&&(r=b.default.getDateFormatByTickInterval(_),t.showTransition&&R.length&&(A=R.map(Number).indexOf(+e),1===R.length&&0===A?r=v.default.getDateFormatByTicks(R):(A===-1?p=u(R,e):(p=0===A?R.length-1:A-1,g=0===A?1:-1),h=o(R[p],e,R[g],r),r=v.default.getDateFormatByDifferences(h,r))),P=d(r));return C(e,P)}function f(e){var t=!1;for(var n in e)e[n]===!0||"hour"===n||t?(e[n]=!1,t=!0):e[n]===!1&&(e[n]=!0);return d(v.default.getDateFormatByDifferences(e))}function p(e,t){var n=!1;for(var i in e)n?e[i]=!1:i===t?n=!0:e[i]=!0;return d(v.default.getDateFormatByDifferences(e));
}function g(e,t,n){var i=o(e,t),a=b.default.getDateFormatByTickInterval(n),r=v.default.getDateFormatByDifferences(i,a),s=d(r),l=[];if(n in i){var u=p(o(e,t),n),c=C(e,u);c&&l.push(c)}else{var h=f(o(e,t)),g=C(e,h);g&&l.push(g),l.push("".concat(C(e,s)," - ").concat(C(t,s)))}return l.join(", ")}function _(e){if((0,y.isObject)(e)){var t=Object.keys(e),n=t.reduce(function(t,n){return e[n]+t},0);if(1===n){var i=t.filter(function(t){return 1===e[t]})[0];return i.slice(0,i.length-1)}}return e}function m(e,t,n,i){var o=i.dataType,a=i.type,r=i.logarithmBase;if("discrete"===a)return"";if("datetime"===o)return g(e,t,_(n));var s={ticks:[],type:a,dataType:o,tickInterval:n,logarithmBase:r,labelOptions:{}};return"".concat(h(e,s)," - ").concat(h(t,s))}t.smartFormatter=h,t.formatRange=m;var v=i(n(79)),y=n(1),b=i(n(20)),x=n(29),w=n(14),C=v.default.format,k=Math.abs,S=Math.floor,D="exponential",I=["fixedPoint","thousands","millions","billions","trillions",D],T=["millisecond","second","minute","hour","day","month","year"]},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return u(e)||l(e,t)||r(e,t)||a()}function a(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function l(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function u(e){if(Array.isArray(e))return e}function c(e){return function(t,n,i,o){var a=te(n/e.axisDivisionFactor);a=a<1?1:a;var r=n/a;return{ticks:r>0?Array.apply(null,new Array(a+1)).map(function(e,t){return r*t}):[],tickInterval:r}}}function d(e){return function(t,n,i,o){var a=(0,$.getCategoriesInfo)(t.categories,t.min,t.max).categories;return{ticks:a,tickInterval:ne(a.length*e.axisDivisionFactor/n)}}}function h(e,t,n,i){return(0,U.isDefined)(i)?i?e:t:function(i,o,a,r){var s=e(i,o);return i-s===0||!(0,U.isDefined)(a)||n(i,s,o,a)||r?s:t(i,o)}}function f(e,t,n,i){var o=n-ee(t-e),a=J.max(ne(i.businessDelta/n),2),r=i.screenDelta/(a*n),s=J.round(r*o),l=(0,$.getLogExt)(r*n/i.axisDivisionFactor,2)+1,u=(i.isSpacedMargin?2:1)*l,c=J.round(ce*u);return i.businessDelta>i.interval&&s>=c}function p(e){return function(t,n,i,o){return f((0,$.getLogExt)(t,e),(0,$.getLogExt)(n,e),i,o)}}function g(e,t,n,i){return f(e.valueOf(),t.valueOf(),Q(n),i)}function _(e,t){var n=0;return t&&(n=t.reduce(function(e,t){return e+(t.to-t.from)},0)),ee(e.max-e.min-n)}function m(e,t,n){var i=fe(e,t,n);return function(e,t){var n=0;return t&&(n=t.reduce(function(e,t){return e+ee(i(t.to/t.from))},0)),ne(ee(i(e.max)-i(e.min))-n)}}function v(e,t,n,i){var o=t/n-(i||0);return o=o<1?1:o,e/o}function y(e,t){return ie(10,te((0,$.getLogExt)(e,10))+(t||0))}function b(e,t,n,i,o,a,r,s,l,u){var c=v(e,t,o,s),d=1,h=r===!1;return i&&n||((c>=1||!h&&c>0)&&(d=x(c,a,h)),(!n||!i&&n<d)&&(n=d)),!i&&u&&(u=x(u,a,h),u>n&&(n=u)),n}function x(e,t,n){var i=y(e,-1),o=1;return t=t||oe,e>0&&(e/=i,o=t.concat(10*t[0]).map(function(e){return 10*e}).reduce(function(t,o){return.1===i&&n&&25===o?t:t<e?o:t},0),o=(0,Y.adjust)(o*i,i)),o}function w(e,t,n,i){var o=v(e,t,i);return n||ue.reduce(function(t,n){var i=e/n;return i>=o?i:t},0)}function C(e){return function(t,n,i,o,a,r,s,l,u,c){var d=v(t,n,a),h=0,f=k(e);return o&&i||(d>0&&(h=f(d,r)),(!i||!o&&i<h)&&(i=h)),!o&&c&&(c=f(c,r),c>i&&(i=c)),i}}function k(e){return function(t,n){var i=y(t);return n=n||ae,!e&&i<1&&(i=1),n.concat(10*n[0]).reduce(function(e,n){return e<t?n*i:e},0)}}function S(e){return e&&e>2?se:re}function D(e,t){return function(n,i){return!n&&e<=X(t,i)&&(n={},n[t+"s"]=i),n}}function I(e,t){return function(n,i){var o=t*i;return!n&&e<=X("year",o)&&2.5!==o&&(n={years:o}),n}}function T(e,t,n,i,o,a,r,s,l,u){if(!i||!n){var c=O(v(e,t,o),a,null,l);(!n||!i&&Q(n)<=Q(c))&&(n=c)}return!i&&u&&(u=O(u,a,null,l),Q(u)>Q(n)&&(n=u)),n}function O(e,t,n,i){var o;t=t||S(i);for(var a in t)if(o=t[a].reduce(D(e,a),o))break;if(!o)for(var r=1;!(o=oe.reduce(I(e,r),o));r*=10);return o}function E(e,t,n,i){return T(e,t,n,!0,i,le)}function A(e,t){return function(n){return n?t(ee((0,Y.adjust)(e(n[1])-e(n[0]))))||void 0:void 0}}function P(e,t,n){return K.default.addInterval(e,t,n)}function R(e,t){return function(n,i,o){return t(P(e(n),i,o))}}function M(e,t,n){return P(e,t,n)}function B(e,t,n){return t=t.filter(function(e){return!e.gapSize}),function(i,o,a){var r;return i=e(i,o,a),t.every(function(t){return i>=e(t.from,o)&&e(i,o)<t.to&&(r=t.to-t.from-2*(e(t.from,o)-t.from)),!r})||(i=n(e(i,r),o)),i}}function V(e,t,n,i){return function(o,a,r,s,l,u,c,d,h){var f=H(e,s),p=o.min,g=o.max,_={screenDelta:c,businessDelta:u,axisDivisionFactor:d,isSpacedMargin:o.isSpacedMargin,interval:a},m=t(p,a,_),v=[];null!==l&&void 0!==l&&l.length&&(e=B(e,l,t)),m>g&&(m=t(p,n(u/2),_),m>g&&(r=!0,m=t(p,a,_,r))),m=f(m);for(var y;m<g&&m!==y||h&&m<=g;)v.push(m),y=m,m=f(e(m,a));return(r||m-g===0||!(0,U.isDefined)(r)&&i(g,m,a,_))&&v.push(m),v}}function F(e,t,n,i,o){return function(a,r,s,l,u,c,d){var h=u/l,f=s[s.length-1],p=s[0],g=d-1;if(null!==c&&void 0!==c&&c.length&&(t=B(t,c,n)),l=e(l,p,p,h),0===l)return[];var _=i(n(a,u,a),l);l=e(l,p,_,h);for(var m=[];_<p&&(!g||g>0);)_>=a&&m.push(_),g--,_=t(_,l);var v=s.reduce(function(n,o){if(g=d-1,null===n.prevTick)return n.prevTick=o,n;l=e(l,o,n.prevTick,h);for(var a=i(n.prevTick,l);a<o&&(!g||g>0);)a!==n.prevTick&&n.minors.push(a),g--,a=t(a,l);return n.prevTick=o,n},{prevTick:null,minors:[]});m=m.concat(v.minors);var y=o(r,u,a);l=e(l,y,y,h),_=i(f,l);for(var b;_<r&&_!==b;)m.push(_),b=_,_=t(_,l);return f-r!==0&&_-r===0&&m.push(_),m}}function L(e,t){if(t.length){var n=t.reduce(function(t,n){var i,o=[];for(i=t[1];i<e.length;i++){var a=e[i];if(a<n.from&&o.push(a),a>=n.to)break}return[t[0].concat(o),i]},[[],0]);return n[0].concat(e.slice(n[1]))}return e}function H(e,t){return function(n){var i;return t.every(function(e){return n>=e.from&&n<e.to&&(i=e.gapSize),!i})||(n=e(n,i)),n}}function z(e,t,n,i,o,a,r,s,l,u){function c(e){return{tickInterval:o(e.majors),ticks:e.majors||[],minorTickInterval:a(e.minors),minorTicks:e.minors||[]}}function d(t,n,i){return t&&n/r(t)>=i+1&&(e.incidentOccurred("W2003"),t=void 0),t}function h(t,i,o,a,r,l,c,h){if(c.majors)return t.breaks=h,t;var f,p=h.filter(function(e){return e.gapSize});return r=e.skipCalculationLimits?r:d(r,o,a),r=n(o,a,r,l,e.axisDivisionFactor,e.numberMultipliers,e.allowDecimals,h.length,p[0]&&p[0].gapSize.days,e.minTickInterval),e.skipTickGeneration||(f=s(i,r,e.endOnTick,p,h,o,a,e.axisDivisionFactor,e.generateExtraTick),h=u(h,f,r),f=L(f,h),t.breaks=h,t.ticks=t.ticks.concat(f)),t.tickInterval=r,t}function f(t,n,o,s,u,c,h){if(!e.calculateMinors)return t;if(h.minors)return t;var f=r(t.tickInterval),p=s*f/o,g=t.breaks;return!u&&c?u=a([f/(c+1),f/(c+1)*2]):c=void 0,u=d(u,f,p),u=i(f,p,u,e.minorAxisDivisionFactor),t.minorTicks=L(t.minorTicks.concat(l(n.min,n.max,t.ticks,u,t.tickInterval,g,c)),g),t.minorTickInterval=u,t}return function(n,i,o,a,r,s,l,u){r=r||{};var d=t(n,u),p=c(r);return isNaN(d)||(0!==d||r.majors?(p=h(p,n,d,i,o,a,r,u||[]),!e.skipTickGeneration&&d>0&&(p=f(p,n,d,i,s,l,r))):p.ticks=[n.min]),p}}function N(e,t,n,i){var a=o(t,2),r=a[0],s=a[1];return(!(0,U.isDefined)(r)||ee(i(e)-i(r))/n>.25)&&((0,U.isDefined)(s)?r=s:(0,U.isDefined)(r)||(r=e)),r}function j(e,t,n){return function(i,o,a){var r=e(a),s=.5*r;return i.reduce(function(e,i){var a=o.filter(function(e){return e<=i.from}),l=n(N(i.from,[].concat(a[a.length-1],o[a.length]),r,t),s);a=o.filter(function(e){return e>=i.to});var u=n(N(i.to,[].concat(a[0],o[o.length-a.length-1]),r,t),-s);return t(u)-t(l)<r&&!i.gapSize?e:i.gapSize?e.concat([i]):e.concat([{from:l,to:u,cumulativeWidth:i.cumulativeWidth}])},[])}}function W(e){var t=me(he,te,he),n=me(he,ne,he),i=A(he,he);return z(e,_,b,w,i,i,he,V(P,h(t,n,f,e.endOnTick),x,f),F(he,P,t,P,he),j(he,he,function(e,t){return e+t}))}function G(e){var t=e.logBase,n=pe(t,e.allowNegatives,e.linearThreshold),i=fe(t,e.allowNegatives,e.linearThreshold),o=_e(t),a=ge(t),r=function(e){return e<0?ne:te},s=function(e){return e<0?te:ne},l=ve(a,r,o),u=ve(a,s,o),c=me(he,ne,he);return z(e,m(t,e.allowNegatives,e.linearThreshold),C(e.skipCalculationLimits),w,A(i,he),A(he,he),he,V(R(i,n),h(l,u,p(t),e.endOnTick),k(e.skipCalculationLimits),p(t)),F(function(e,t,n,i){return Math.max(Math.abs(t),Math.abs(n))/i},P,l,c,u),j(he,i,function(e,t){return n(i(e)+t)}))}function q(e){function t(t,n){var i=me(he,te,he),o=(0,U.isString)(n)?K.default.getDateIntervalByString(n.toLowerCase()):n,a=Q(n);(o.days%7===0||n.quarters)&&(o=O(a));var r=function(t){return K.default.correctDateWithUnitBeginning(t,o,null,e.firstDayOfWeek)},s=function(e){return new Date(te((e.getTime()-e.getTimezoneOffset()*de)/a)*a+e.getTimezoneOffset()*de)};return t=r(t),"years"in o?t.setFullYear(i(t.getFullYear(),o.years,0)):"quarters"in o?t=r(s(t)):"months"in o?t.setMonth(i(t.getMonth(),o.months,0)):"weeks"in o||"days"in o?t=r(s(t)):"hours"in o?t.setHours(i(t.getHours(),o.hours,0)):"minutes"in o?t.setMinutes(i(t.getMinutes(),o.minutes,0)):"seconds"in o?t.setSeconds(i(t.getSeconds(),o.seconds,0)):"milliseconds"in o&&(t=s(t)),t}function n(e,n){for(var i=t(e,n);e-i>0;)i=M(i,n);return i}var i=A(he,K.default.convertMillisecondsToDateUnits);return z(e,_,T,E,i,i,Q,V(M,h(t,n,g,e.endOnTick),O,g),F(he,M,t,M,he),j(Q,he,function(e,t){return new Date(e.getTime()+t)}))}t.tickGenerator=void 0;var $=n(14),K=i(n(20)),U=n(1),Y=n(29),Z=n(0),X=K.default.convertDateUnitToMilliseconds,Q=K.default.dateToMilliseconds,J=Math,ee=J.abs,te=J.floor,ne=J.ceil,ie=J.pow,oe=[1,2,2.5,5],ae=[1,2,3,5],re={millisecond:[1,2,5,10,25,50,100,250,500],second:[1,2,3,5,10,15,20,30],minute:[1,2,3,5,10,15,20,30],hour:[1,2,3,4,6,8,12],day:[1,2],week:[1,2],month:[1,2,3,6]},se=(0,Z.extend)({},re,{day:[1]}),le={millisecond:[1,2,5,10,25,50,100,250,500],second:[1,2,3,5,10,15,20,30],minute:[1,2,3,5,10,15,20,30],hour:[1,2,3,4,6,8,12],day:[1,2,3,7,14],month:[1,2,3,6]},ue=[2,4,5,8,10],ce=3,de=6e4,he=function(e){return e},fe=function(e,t,n){return function(i){return(0,$.getLogExt)(i,e,t,n)}},pe=function(e,t,n){return function(i){return(0,$.raiseToExt)(i,e,t,n)}},ge=function(e){return function(t){return(0,$.raiseTo)(t,e)}},_e=function(e){return function(t){return 0===t?0:(0,$.getLog)(ee(t),e)}},me=function(e,t,n){return function(i,o){return(0,Y.adjust)(e(t((0,Y.adjust)(n(i)/o))*o))}},ve=function(e,t,n){return function(i,o){return(0,Y.sign)(i)*(0,Y.adjust)(e(t(i)((0,Y.adjust)(n(i)/o))*o))}},ye=function(e){var t;return t=e.rangeIsEmpty?c(e):"discrete"===e.axisType?d(e):"logarithmic"===e.axisType?G(e):"datetime"===e.dataType?q(e):W(e)};t.tickGenerator=ye},function(e,t,n){function i(e){return{stroke:e.color,"stroke-width":e.width,"stroke-opacity":e.opacity,opacity:1}}function o(e,t,n,o,l,u,c){function d(t){var n=e._textFontStyles,i=b.customizeColor;return i&&i.call&&(n=(0,r.extend)({},e._textFontStyles,{fill:i.call(t,t)})),n}function h(t,n){var i=e.formatHint(t.value,b,n);(0,a.isDefined)(i)&&""!==i&&t.getContentContainer().setTitle(i)}var f=c||e._tickOffset,p=e._axisLineGroup,g=e._axisElementsGroup,_=i(n),m=i(o),v=/^\s+$/,y=e.getOptions(),b=y.label,x=e._textOptions;return function(i){var r={value:i,updateValue:function(e){this.value=i=e},initCoords:function(){this.coords=e._getTranslatedValue(i,f),this.labelCoords=e._getTranslatedValue(i)},saveCoords:function(){this._lastStoredCoordinates={coords:this._storedCoords,labelCoords:this._storedLabelsCoords},this._storedCoords=this.coords,this._storedLabelsCoords=this.templateContainer?this._getTemplateCoords():this.labelCoords},resetCoordinates:function(){this._storedCoords=this._lastStoredCoordinates.coords,this._storedLabelsCoords=this._lastStoredCoordinates.labelCoords},drawMark:function(t){n.visible&&l!==i&&(e.areCoordsOutsideAxis(this.coords)||(this.mark?(this.mark.append(p),e.sharp(this.mark,e.getSharpDirectionByCoords(this.coords)),this.updateTickPosition(t)):(this.mark=e._createPathElement([],_,e.getSharpDirectionByCoords(this.coords)).append(p),this.updateTickPosition(t))))},setSkippedCategory:function(e){l=e},_updateLine:function(t,n,i,o,a){if(t){if(null===n.points||null===n.r)return void t.remove();o&&i&&null!==i.points?(n.opacity=1,t.attr(i),t.animate(n)):(n.opacity=o?0:1,t.attr(n),o&&t.animate({opacity:1},{delay:.5,partitionDuration:.5})),this.coords.angle&&e._rotateTick(t,this.coords,a)}},updateTickPosition:function(t,i){this._updateLine(this.mark,{points:e._getTickMarkPoints(r.coords,n.length,t)},this._storedCoords&&{points:e._getTickMarkPoints(r._storedCoords,n.length,t)},i,!1)},drawLabel:function(n,o){var r=this;if(this.templateContainer&&e.isRendered())return void this.updateLabelPosition();var l=b.visible&&!u&&!e.getTranslator().getBusinessRange().isEmpty()&&!e.areCoordsOutsideAxis(this.labelCoords);if(!l)return void(this.label&&this.removeLabel());var c=b.template,f=e.formatLabel(i,b,n);if(this.label)return this.label.attr({text:f,rotate:0}).append(g),h(this,n),void this.updateLabelPosition();c?(this.templateContainer=t.g().append(g),this._templateDef&&this._templateDef.reject(),this._templateDef=new s.Deferred,o.render({model:{valueText:f,value:this.value,labelFontStyle:d(this),labelStyle:x},container:this.templateContainer.element,onRendered:function(){r.updateLabelPosition(),r._templateDef.resolve()}})):(0,a.isDefined)(f)&&""!==f&&!v.test(f)&&(this.label=t.text(f).css(d(this)).attr(x).append(g),this.updateLabelPosition(),h(this,n));var p=this.getContentContainer();p&&p.data("chart-data-argument",this.value),this.templateContainer&&h(this,n)},getTemplateDeferred:function(){return this._templateDef},getContentContainer:function(){return this.templateContainer||this.label},fadeOutElements:function(){var t={opacity:1},n={opacity:0},i={partitionDuration:.5};this.getContentContainer()&&this._fadeOutLabel(),this.grid&&this.grid.append(e._axisGridGroup).attr(t).animate(n,i),this.mark&&this.mark.append(e._axisLineGroup).attr(t).animate(n,i)},_fadeInLabel:function(){var t=e._renderer.g().attr({opacity:0}).append(e._axisElementsGroup).animate({opacity:1},{delay:.5,partitionDuration:.5});this.getContentContainer().append(t)},_fadeOutLabel:function(){var t=e._renderer.g().attr({opacity:1}).animate({opacity:0},{partitionDuration:.5}).append(e._axisElementsGroup).toBackground();this.getContentContainer().append(t)},_getTemplateCoords:function(){return e._getLabelAdjustedCoord(this,(e._constantLabelOffset||0)+(r.labelOffset||0))},updateLabelPosition:function(e){var t=this.templateContainer;if(this.getContentContainer())if(e&&this._storedLabelsCoords)if(t){t.attr(this._storedLabelsCoords);var n=this._getTemplateCoords();t.animate(n)}else this.label.attr({x:this._storedLabelsCoords.x,y:this._storedLabelsCoords.y}),this.label.animate({x:this.labelCoords.x,y:this.labelCoords.y});else{if(t){var i=this._getTemplateCoords();t.attr(i)}else this.label.attr({x:this.labelCoords.x,y:this.labelCoords.y});e&&this._fadeInLabel()}},updateMultilineTextAlignment:function(){!b.template&&this.label&&this.label.attr({textsAlignment:this.labelAlignment||e.getOptions().label.alignment})},drawGrid:function(t){o.visible&&l!==this.value&&(this.grid?(this.grid.append(e._axisGridGroup),e.sharp(this.grid,e.getSharpDirectionByCoords(this.coords)),this.updateGridPosition()):(this.grid=t(this,m),this.grid&&this.grid.append(e._axisGridGroup)))},updateGridPosition:function(t){this._updateLine(this.grid,e._getGridPoints(r.coords),this._storedCoords&&e._getGridPoints(this._storedCoords),t,!0)},removeLabel:function(){var e=this.getContentContainer();e&&e.remove(),this._templateDef&&this._templateDef.reject(),this._templateDef=this.templateContainer=this.label=null}};return r}}t.tick=o;var a=n(1),r=n(0),s=n(6)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i;if("week"===t&&(t="day"),"quarter"===t&&(t="month"),e[t])for(i=0;i<k.default.dateUnitIntervals.length&&(n=k.default.dateUnitIntervals[i],e[n]&&(e[n]=!1,e.count--),n!==t);i++);}function a(e){return e.sort(function(e,t){return e.from-t.from})}function r(e,t,n){var i,o=e;return e=P(e,n),t=P(t,n),i=k.default.getSequenceByInterval(e,t,n),i.length&&o>i[0]&&(i=i.slice(1)),i}function s(e){var t="start";return"center"===e&&(t="center"),"right"===e&&(t="end"),t}function l(e){var t="start";return"center"===e&&(t="center"),"bottom"===e&&(t="end"),t}function u(e){var t=A(e);return"quarter"===t&&(t=A(t)),t}function c(e,t,n,i){var a=i,r=t&&k.default.getDatesDifferences(t,e);return t&&"year"!==n&&(o(r,n),a=C.default.getDateFormatByDifferences(r)),a}function d(e,t){return t.reduce(function(t,n){return M(t,e(n))},0)}function h(e,t){t=R.abs(t),t=t%180>=90?90-t%90:t%90;var n=t*(R.PI/180);return n>=R.atan(e.height/e.width)?e.height/R.abs(R.sin(n)):e.width}function f(e){return e.reduce(function(e,t){return M(e,t.paddingTopBottom)},0)}function p(e,t,n){return e.some(function(e){return e.label.verticalAlignment===t})&&n||0}function g(e){return R.abs(e.x)||0}function _(e){return R.abs(e.width-R.abs(e.x))||0}function m(e,t,n){var i,o,a,r,s,l=null,u=[];for(i=1,o=e.length;i<o;i++)a=e[i],r=e[i-1],s=n(a,r),t.indexOf(a)>=0?(!l||s>l.length)&&(l={start:a,end:r,length:s}):(l&&s<l.length?u.push(l):u.push({start:a,end:r,length:s}),l=null);return l&&u.push(l),u}function v(e,t,n){for(var i=e.logarithmBase,o=e.type,r=e.maxAutoBreakCount,s=n.minVisible,l=n.maxVisible,u=[],c="logarithmic"===o?function(e,t){return(0,T.getLog)(t/e,i)}:function(e,t){return t-e},d=c(s,l),h=t.reduce(function(e,t){var n=t.getPointsInViewPort();return e[0]=e[0].concat(n[0]),e[1]=e[1].concat(n[1]),e},[[],[]]),f=h[0].concat(h[1]).sort(function(e,t){return t-e}),p=h[1].filter(function(e){return h[0].indexOf(e)<0}),g=N*d,_=m(f,p,c).sort(function(e,t){return t.length-e.length}),v=R.min.apply(null,_.map(function(e){return e.length}))/1e3,y=(0,O.isDefined)(r)?R.min(r,_.length):_.length,b=0;b<y&&_[b].length>=g&&!(d<=_[b].length);b++)d-=_[b].length,(d>v||d<-v)&&(u.push({from:_[b].start,to:_[b].end}),g=N*d);return a(u),u}function y(e,t,n,i,o,a,r){var s=r?"bezier":"line",l=e.g().append(t);return function(t,u){e.path(n(b(i,o,a,t,r)),s).attr(u).append(l)}}function b(e,t,n,i,o){if(!o)return[e,t+i,n,t+i];t+=i;var a,r=t+W,s=t+j,l=t+G,u=[[e,s]];for(a=e;a<n+q;a+=q)u.push([a+6,r,a+6,r,a+12,s,a+18,l,a+18,l,a+24,s]);return[].concat.apply([],u)}function x(e){var t,n=[];for(t=0;t<e.length;t+=2)n.push(e[t+1]),n.push(e[t]);return n}t.default=void 0;var w=n(134),C=i(n(79)),k=i(n(20)),S=n(0),D=n(935),I=n(4),T=n(14),O=n(1),E=i(n(328)),A=k.default.getNextDateUnit,P=k.default.correctDateWithUnitBeginning,R=Math,M=R.max,B=E.default.top,V=E.default.bottom,F=E.default.left,L=E.default.right,H=E.default.center,z=3,N=.3,j=2,W=0,G=4,q=24,$={left:-1,top:-1,right:0,bottom:0,center:-.5},K={linear:{_getStep:function(e,t){var n=this._options.label.minSpacing,i=this._isHorizontal?function(e){return e.width+n}:function(e){return e.height},o=d(i,e);return t&&(o=h({width:o,height:this._getMaxLabelHeight(e,0)},t)),E.default.getTicksCountInRange(this._majorTicks,this._isHorizontal?"x":"y",o)},_getMaxLabelHeight:function(e,t){return d(function(e){return e.height},e)+t},_validateOverlappingMode:function(e,t){return this._isHorizontal&&("rotate"===t||"stagger"===t)||!this._isHorizontal?E.default.validateOverlappingMode(e):e},_validateDisplayMode:function(e){return this._isHorizontal?e:"standard"},getMarkerTrackers:function(){return this._markerTrackers},_getSharpParam:function(e){return this._isHorizontal^e?"h":"v"},_createAxisElement:function(){return this._renderer.path([],"line")},_updateAxisElementPosition:function(){var e=this._axisPosition,t=this._getCanvasStartEnd();this._axisElement.attr({points:this._isHorizontal?[t.start,e,t.end,e]:[e,t.start,e,t.end]})},_getTranslatedCoord:function(e,t){return this._translator.translate(e,t)},_initAxisPositions:function(){var e=this;e.customPositionIsAvailable()&&!(0,O.isDefined)(e._customBoundaryPosition)&&(e._customBoundaryPosition=e.getCustomBoundaryPosition()),!e.customPositionIsAvailable()||e.customPositionIsBoundary()?e._axisPosition=e.getPredefinedPosition(e.getResolvedBoundaryPosition()):e._axisPosition=e.getCustomPosition()},_getTickMarkPoints:function(e,t,n){var i,o=this._isHorizontal,a=this._options.tickOrientation,r=this._options.label.position;if((0,O.isDefined)(a))i=$[a]*t;else{var s=n.shift||0;(!o&&r===F||o&&r!==V)&&(s=-s),i=s+this.getTickStartPositionShift(t)}return[e.x+(o?0:i),e.y+(o?i:0),e.x+(o?0:i+t),e.y+(o?i+t:0)]},getTickStartPositionShift:function(e){var t=this._options.width,n=this.getResolvedBoundaryPosition();return e%2===1?(t%2!==0||n!==F&&n!==B)&&(t%2!==1||n!==L&&n!==V||this.hasCustomPosition())?-Math.floor(e/2):Math.floor(-e/2):-e/2+(t%2===0?0:n===V||n===L?-1:1)},_getTitleCoords:function(){var e=this,t=e._isHorizontal,n=e._axisPosition,i=e._axisPosition,o=e._options.title.alignment,a=e._getCanvasStartEnd(),r=t||e._options.position===F,s=r?a.start:a.end,l=r?a.end:a.start,u=o===F?s:o===L?l:a.start+(a.end-a.start)/2;return t?n=u:i=u,{x:n,y:i}},_drawTitleText:function(e,t){var n=this._options,i=n.title,o={opacity:i.opacity,align:i.alignment,"class":i.cssClass};if(i.text&&e){t=t||this._getTitleCoords(),this._isHorizontal||(o.rotate=n.position===F?270:90);var a=this._renderer.text(i.text,t.x,t.y).css((0,T.patchFontOptions)(i.font)).attr(o).append(e);return this._checkTitleOverflow(a),a}},_updateTitleCoords:function(){this._title&&this._title.element.attr(this._getTitleCoords())},_drawTitle:function(){var e=this._drawTitleText(this._axisTitleGroup);e&&(this._title={element:e})},_measureTitle:function(){this._title&&(this._title.bBox&&!this._title.originalSize&&(this._title.originalSize=this._title.bBox),this._title.bBox=this._title.element.getBBox())},_drawDateMarker:function(e,t,n){var i,o=this,a=o._options.marker,r=o._translator.getBusinessRange().invert,s=a.width+a.textLeftIndent;if(null!==t.x){t.withoutStick||(i=o._renderer.path([t.x,t.y,t.x,t.y+a.separatorHeight],"line").attr({"stroke-width":a.width,stroke:a.color,"stroke-opacity":a.opacity,sharp:"h"}).append(o._axisElementsGroup));var l=String(o.formatLabel(e,t.labelOptions,n));return{date:e,x:t.x,y:t.y,cropped:t.withoutStick,label:o._renderer.text(l,t.x,t.y).css((0,T.patchFontOptions)(a.label.font)).append(o._axisElementsGroup),line:i,getContentContainer:function(){return this.label},getEnd:function(){return this.x+(r?-1:1)*(s+this.labelBBox.width)},setTitle:function(){this.title=l},hideLabel:function(){this.label.dispose(),this.label=null,this.title=l},hide:function(){i&&(i.dispose(),i=null),this.label.dispose(),this.label=null,this.hidden=!0}}}},_drawDateMarkers:function(){function e(e,t,i){return n._drawDateMarker(e,{x:o.translate(e),y:d,labelOptions:n._getLabelFormatOptions(t),withoutStick:i},a)}var t,n=this,i=n._options,o=n._translator,a=n._getViewportRange(),s=a.minVisible,l=[];if(a.isEmpty()||!i.marker.visible||"datetime"!==i.argumentType||"discrete"===i.type||n._majorTicks.length<=1)return[];var d=n._axisPosition+i.marker.topIndent,h=k.default.getDateUnitInterval(this._tickInterval),f=u(h),p=r(s,a.maxVisible,f);return(p.length>1||1===p.length&&s<p[0])&&(l=p.reduce(function(t,n,i,o){var a=e(n,c(n,o[i-1]||s<n&&s,h,f));return a&&t.push(a),t},[]),s<p[0]&&(t=e(s,c(s,p[0],h,f),!0),t&&l.unshift(t))),l},_adjustDateMarkers:function(e){e=e||0;var t=this,n=this._options.marker,i=n.width+n.textLeftIndent,o=this._translator.getBusinessRange().invert,a=t._getCanvasStartEnd(),r=this._dateMarkers;if(!r.length)return e;r[0].cropped&&(this._checkMarkersPosition(o,r[1],r[0])||r[0].hideLabel());var s;return r.forEach(function(e,n,i){e.cropped||((o?e.getEnd()<a.end:e.getEnd()>a.end)?e.hideLabel():t._checkMarkersPosition(o,e,s)?s=e:e.hide())}),this._dateMarkers.forEach(function(t){if(t.label){var a=t.labelBBox,r=t.y+n.textTopIndent-a.y;t.label.attr({translateX:o?t.x-i-a.x-a.width:t.x+i-a.x,translateY:r+e})}t.line&&t.line.attr({translateY:e})}),t._initializeMarkersTrackers(e),e+n.topIndent+n.separatorHeight},_checkMarkersPosition:function(e,t,n){return void 0===n||(e?t.x<n.getEnd():t.x>n.getEnd())},_initializeMarkersTrackers:function(e){var t=this,n=t._options.marker.separatorHeight,i=t._renderer,o=this._translator.getBusinessRange(),a=t._getCanvasStartEnd(),r=t._axisElementsGroup;t._markerTrackers=this._dateMarkers.filter(function(e){return!e.hidden}).map(function(t,s,l){var u=l[s+1]||{x:a.end,date:o.max},c=t.x,d=t.y+e,h=i.path([c,d,c,d+n,u.x,d+n,u.x,d,c,d],"area").attr({"stroke-width":1,stroke:"grey",fill:"grey",opacity:1e-4}).append(r);return h.data("range",{startValue:t.date,endValue:u.date}),t.title&&h.setTitle(t.title),h})},_getLabelFormatOptions:function(e){var t=this,n=t._markerLabelOptions;return n||(t._markerLabelOptions=n=(0,S.extend)(!0,{},t._options.marker.label)),(0,O.isDefined)(t._options.marker.label.format)||(n.format=e),n},_adjustConstantLineLabels:function(e){var t=this,n=t._options.position,i=t.getCanvas(),o=i.left,a=i.width-i.right,r=i.top,s=i.height-i.bottom,l=r+(s-r)/2,u=o+(a-o)/2,c=0;return e.forEach(function(e){var i,d,h=t._isHorizontal,f=e.options,p=f.paddingTopBottom,g=f.paddingLeftRight,_=f.label,m=_.verticalAlignment,v=_.horizontalAlignment,y="inside"===_.position,b=e.label,x=e.labelBBox;if(null!==b&&!x.isEmpty){if(h)if(y)switch(i=v===F?e.coord-g-x.x-x.width:e.coord+g-x.x,m){case H:d=l-x.y-x.height/2;break;case V:d=s-p-x.y-x.height;break;default:d=r+p-x.y}else n===m&&(c=M(c,x.height+p)),i=e.coord-x.x-x.width/2,d=m===V?s+p-x.y:r-p-x.y-x.height;else if(y)switch(d=m===V?e.coord+p-x.y:e.coord-p-x.y-x.height,v){case H:i=u-x.x-x.width/2;break;case L:i=a-g-x.x-x.width;break;default:i=o+g-x.x}else n===v&&(c=M(c,x.width+g)),d=e.coord-x.y-x.height/2,i=v===L?a+g-x.x:o-g-x.x-x.width;b.attr({translateX:i,translateY:d})}}),c},_drawConstantLinesForEstimating:function(e){var t=this,n=this._renderer,i=n.g();return e.forEach(function(e){t._drawConstantLineLabelText(e.label.text,0,0,e.label,i).attr({align:"center"})}),i.append(n.root)},_estimateLabelHeight:function(e,t){var n=e.height,i=t.drawingType;if("stagger"!==this._validateDisplayMode(i)&&"stagger"!==this._validateOverlappingMode(t.overlappingBehavior,i)||(n=2*n+t.staggeringSpacing),"rotate"===this._validateDisplayMode(i)||"rotate"===this._validateOverlappingMode(t.overlappingBehavior,i)){var o=(0,T.getCosAndSin)(t.rotationAngle);n=n*o.cos+e.width*o.sin}return n&&(n+t.indentFromAxis||0)||0},estimateMargins:function(e){this.updateCanvas(e);var t=this,n=t._getViewportRange(),i=this._createTicksAndLabelFormat(n),o=i.ticks,a=i.tickInterval,r=this._options,s=t._outsideConstantLines.filter(function(e){return e.labelOptions.visible}).map(function(e){return e.options}),l=t._renderer.root,u=r.label.visible&&!n.isEmpty()&&o.length,c=u&&t.formatLabel(o[o.length-1],r.label,void 0,void 0,a,o),d=u&&t._renderer.text(c,0,0).css(t._textFontStyles).attr(t._textOptions).append(l),h=t._drawTitleText(l,{x:0,y:0}),m=t._drawConstantLinesForEstimating(s),v=!r.label.template&&d&&d.getBBox()||{x:0,y:0,width:0,height:0},y=h&&h.getBBox()||{x:0,y:0,width:0,height:0},b=m.getBBox(),x=y.height?y.height+r.title.margin:0,w=t._estimateLabelHeight(v,r.label),C=b.height?b.height+f(s):0,k=w+x,S={left:M(g(v),g(b)),right:M(_(v),_(b)),top:("top"===r.position?k:0)+p(s,"top",C),bottom:("top"!==r.position?k:0)+p(s,"bottom",C)};return d&&d.remove(),h&&h.remove(),m&&m.remove(),S},_checkAlignmentConstantLineLabels:function(e){var t=e.position,n=(e.verticalAlignment||"").toLowerCase(),i=(e.horizontalAlignment||"").toLowerCase();this._isHorizontal?"outside"===t?(n=n===V?V:B,i=H):(n=n===H?H:n===V?V:B,i=i===F?F:L):"outside"===t?(n=H,i=i===F?F:L):(n=n===V?V:B,i=i===L?L:i===H?H:F),e.verticalAlignment=n,e.horizontalAlignment=i},_getConstantLineLabelsCoords:function(e,t){var n=this,i=e,o=e;return n._isHorizontal?o=n._orthogonalPositions["top"===t.verticalAlignment?"start":"end"]:i=n._orthogonalPositions["right"===t.horizontalAlignment?"end":"start"],{x:i,y:o}},_getAdjustedStripLabelCoords:function(e){var t=e.options,n=t.paddingTopBottom,i=t.paddingLeftRight,o=t.label.horizontalAlignment,a=t.label.verticalAlignment,r=e.labelBBox,s=r.height,l=r.width,u=e.labelCoords,c=u.y-r.y,d=u.x-r.x;return a===B?c+=n:a===H?c-=s/2:a===V&&(c-=n+s),o===F?d+=i:o===H?d-=l/2:o===L&&(d-=i+l),{translateX:d,translateY:c}},_adjustTitle:function(e){if(e=e||0,this._title){var t=this,n=t._options,i=n.position,o=n.title.margin,a=t._title,r=a.bBox,s=r.x,l=r.y,u=r.width,c=r.height,d=t._axisPosition,h=d-o-e,f=d+o+e,p={};t._isHorizontal?i===B?p.translateY=h-(l+c):p.translateY=f-l:i===F?p.translateX=h-(s+u):p.translateX=f-s,a.element.attr(p)}},_checkTitleOverflow:function(e){if(this._title||e){var t=this._getScreenDelta(),n=e?{bBox:e.getBBox(),element:e}:this._title,i=this._options.title,o=n.bBox;if((this._isHorizontal?o.width:o.height)>t)n.element.setMaxSize(t,void 0,{wordWrap:i.wordWrap||"none",textOverflow:i.textOverflow||"ellipsis"}),this._wrapped=i.wordWrap&&"none"!==i.wordWrap;else{var a=n.originalSize&&t>(this._isHorizontal?n.originalSize.width:n.originalSize.height);!this._wrapped&&a&&n.element.restoreText()}}},coordsIn:function(e,t){var n=this.getCanvas(),i=this._options.isHorizontal,o=this._options.position,a=i?t:e;return!(i&&(e<n.left||e>n.width-n.right)||!i&&(t<n.top||t>n.height-n.bottom))&&(i&&o===E.default.top||!i&&o===E.default.left?a<n[o]:a>n[i?"height":"width"]-n[o])},_boundaryTicksVisibility:{min:!0,max:!0},adjust:function(e){var t=this,n=t._seriesData,i={min:n.min,max:n.max};e||(i=t._series.filter(function(e){return e.isVisible()}).reduce(function(e,t){var n=t.getViewport();return e.min=(0,O.isDefined)(n.min)?e.min<n.min?e.min:n.min:e.min,e.max=(0,O.isDefined)(n.max)?e.max>n.max?e.max:n.max:e.max,t.showZero&&(e=new w.Range(e),e.correctValueZeroLevel()),e},{})),(0,O.isDefined)(i.min)&&(0,O.isDefined)(i.max)&&(n.minVisible=i.min,n.maxVisible=i.max),n.userBreaks=t._getScaleBreaks(t._options,{minVisible:n.minVisible,maxVisible:n.maxVisible},t._series,t.isArgumentAxis),t._translator.updateBusinessRange(t._getViewportRange())},hasWrap:function(){return this._wrapped},getAxisPosition:function(){return this._axisPosition},_getStick:function(){return!this._options.valueMarginsEnabled},_getStripLabelCoords:function(e,t,n){var i,o,a=this,r=a._orthogonalPositions,u=a._isHorizontal,c=n.horizontalAlignment,d=n.verticalAlignment;return u?(c===H?i=e+(t-e)/2:c===F?i=e:c===L&&(i=t),o=r[l(d)]):(i=r[s(c)],d===B?o=e:d===H?o=t+(e-t)/2:d===V&&(o=t)),{x:i,y:o}},_getTranslatedValue:function(e,t){var n=this._translator.translate(e,t,"semidiscrete"===this._options.type&&this._options.tickInterval),i=this._axisPosition,o=this._isHorizontal;return{x:o?n:i,y:o?i:n}},areCoordsOutsideAxis:function(e){var t=this._isHorizontal?e.x:e.y,n=this.getVisibleArea();return t<n[0]||t>n[1]},_getSkippedCategory:function(e){var t;return this._options.type===E.default.discrete&&this._tickOffset&&0!==e.length&&(t=e[e.length-1]),t},_filterBreaks:function(e,t,n){var i=t.minVisible,o=t.maxVisible,a=n?n.width:0;return e.reduce(function(e,t){var n,r=t.from,s=t.to,l=e[e.length-1];if(!(0,O.isDefined)(r)||!(0,O.isDefined)(s))return e;if(r>s&&(s=[r,r=s][0]),e.length&&r<l.to)s>l.to&&(l.to=s>o?o:s,l.gapSize&&(l.gapSize=void 0,l.cumulativeWidth+=a));else if((r>=i&&r<o||s<=o&&s>i)&&(r=r>=i?r:i,s=s<=o?s:o,s-r<o-i)){var u;if(n={from:r,to:s,cumulativeWidth:(null!==(u=null===l||void 0===l?void 0:l.cumulativeWidth)&&void 0!==u?u:0)+a},t.gapSize){var c;n.gapSize=k.default.convertMillisecondsToDateUnits(s-r),n.cumulativeWidth=null!==(c=null===l||void 0===l?void 0:l.cumulativeWidth)&&void 0!==c?c:0;
}e.push(n)}return e},[])},_getScaleBreaks:function(e,t,n,i){var o=this,r=(e.breaks||[]).map(function(e){return{from:o.parser(e.startValue),to:o.parser(e.endValue)}});return"discrete"!==e.type&&"datetime"===e.dataType&&e.workdaysOnly&&(r=r.concat((0,D.generateDateBreaks)(t.minVisible,t.maxVisible,e.workWeek,e.singleWorkdays,e.holidays))),!i&&"discrete"!==e.type&&"datetime"!==e.dataType&&e.autoBreaksEnabled&&0!==e.maxAutoBreakCount&&(r=r.concat(v(e,n,t))),a(r)},_drawBreak:function(e,t,n,i,o,a){var r=this,s=e-(r._translator.isInverted()?0:i+1),l={"stroke-width":1,stroke:o.borderColor,sharp:o.isWaved?void 0:o.isHorizontal?"h":"v"},u={stroke:o.color,"stroke-width":i},c=r._isHorizontal?x:function(e){return e},d=y(r._renderer,a,c,t,s,n,o.isWaved);d(i/2,u),d(0,l),d(i,l)},_createBreakClipRect:function(e,t){var n,i=this,o=i._canvas,a=t-e;return n=i._isHorizontal?i._renderer.clipRect(o.left,e,o.width,a):i._renderer.clipRect(e,o.top,a,o.height),i._breaksElements=i._breaksElements||[],i._breaksElements.push(n),n.id},_createBreaksGroup:function(e,t){var n=this,i=n._renderer.g().attr({"class":n._axisCssPrefix+"breaks","clip-path":n._createBreakClipRect(e,t)}).append(n._scaleBreaksGroup);return n._breaksElements=n._breaksElements||[],n._breaksElements.push(i),i},_disposeBreaksGroup:function(){(this._breaksElements||[]).forEach(function(e){e.dispose()}),this._breaksElements=null},drawScaleBreaks:function(e){var t,n,i,o,a,r=this,s=r._options,l=s.breakStyle,u=s.position,c=r._translator.getBusinessRange().breaks||[];if(r._disposeBreaksGroup(),c&&c.length){var d={color:r._options.containerColor,borderColor:l.color,isHorizontal:r._isHorizontal,isWaved:"straight"!==l.line.toLowerCase()};e?(t=e.start,n=e.end):(t=r._orthogonalPositions.start-(!s.visible||r._axisShift||u!==F&&u!==B?0:z),n=r._orthogonalPositions.end+(!s.visible||u!==L&&u!==V?0:z));var h=r._createBreaksGroup(t,n);r._axisShift&&s.visible&&(o=r._axisPosition-r._axisShift-z,a=o+2*z,i=r._createBreaksGroup(o,a)),c.forEach(function(e){if(!e.gapSize){var u=r._getTranslatedCoord(e.to);r._drawBreak(u,t,n,l.width,d,h),r._axisShift&&s.visible&&r._drawBreak(u,o,a,l.width,d,i)}})}},_getSpiderCategoryOption:I.noop,shift:function(e){function t(t,n){var i={translateX:0,translateY:0},r=e[t]?e[t]+a:0;return i[o?"translateY":"translateX"]=(t===F||t===B?-1:1)*r,(n[t]||n).attr(i),r}var n=this,i=n._options,o=i.isHorizontal,a=n.getMultipleAxesSpacing(),r=n._axisConstantLineGroups;n._axisShift=t(i.position,n._axisGroup),(o?[B,V]:[F,L]).forEach(function(e){t(e,r.above),t(e,r.under)})},getCustomPosition:function(e){var t,n=this,i=n.getOrthogonalAxis(),o=null!==e&&void 0!==e?e:n.getResolvedPositionOption(),a=n.getOptions().offset,r=i.getTranslator(),s=i.getOptions().type,l=i.validateUnit(o);return"discrete"===s&&(!r._categories||r._categories.indexOf(l)<0)&&(l=void 0),t=n.positionIsBoundary(o)?n.getPredefinedPosition(o):(0,O.isDefined)(l)?r.to(l,-1):n.getPredefinedPosition(n.getOptions().position),isFinite(t)&&isFinite(a)&&(t+=a),t},getCustomBoundaryPosition:function(e){var t=this,n=t.getOrthogonalAxis(),i=null!==e&&void 0!==e?e:t.getResolvedPositionOption(),o=n.getTranslator(),a=o.getCanvasVisibleArea();if((0,O.isDefined)(n._orthogonalPositions)&&0!==o.canvasLength){var r=t.getCustomPosition(i);return(0,O.isDefined)(r)?r<=a.min?t._isHorizontal?B:F:r>=a.max?t._isHorizontal?V:L:r:t.getResolvedBoundaryPosition()}},getResolvedPositionOption:function(){var e,t=this.getOptions();return null!==(e=t.customPosition)&&void 0!==e?e:t.position},customPositionIsAvailable:function(){var e=this.getOptions();return(0,O.isDefined)(this.getOrthogonalAxis())&&((0,O.isDefined)(e.customPosition)||isFinite(e.offset))},hasCustomPosition:function(){return this.customPositionIsAvailable()&&!this.customPositionIsBoundary()},getResolvedBoundaryPosition:function(){return this.customPositionIsBoundary()?this._customBoundaryPosition:this.getOptions().position},customPositionEqualsToPredefined:function(){return this.customPositionIsBoundary()&&this._customBoundaryPosition===this.getOptions().position},customPositionIsBoundary:function(){return this.positionIsBoundary(this._customBoundaryPosition)},positionIsBoundary:function(e){return[B,F,V,L].indexOf(e)>=0},getPredefinedPosition:function(e){var t;return null===(t=this._orthogonalPositions)||void 0===t?void 0:t[e===B||e===F?"start":"end"]},resolveOverlappingForCustomPositioning:function(e){var t=this;if(t.hasCustomPosition()||t.customPositionIsBoundary()||e.some(function(e){return e.hasCustomPosition()})){var n={axes:[],ticks:[]};e.filter(function(e){return e.pane===t.pane}).forEach(function(e){for(var i=0;i<t._majorTicks.length;i++){var o=t._majorTicks[i],a=o.label;if(a){n.axes.indexOf(e)<0&&t._detectElementsOverlapping(a,e._axisElement)&&(n.axes.push(e),t._shiftThroughOrthogonalAxisOverlappedTick(a,e));for(var r=0;r<e._majorTicks.length;r++){var s=e._majorTicks[r],l=s.label;if(l&&t._detectElementsOverlapping(a,l)){n.ticks.push(o),t._shiftThroughAxisOverlappedTick(o),i=t._majorTicks.length;break}}}o.mark&&n.ticks.indexOf(o)<0&&(t._isHorizontal&&o.mark.attr("translateY")?o.mark.attr({translateY:0}):!t._isHorizontal&&o.mark.attr("translateX")&&o.mark.attr({translateX:0}))}})}},_shiftThroughOrthogonalAxisOverlappedTick:function(e,t){var n=this,i=e.getBBox(),o=t.getAxisPosition(),a=t.getOptions().label,r=a.position,s=a.indentFromAxis/2,l=n._isHorizontal?"translateX":"translateY",u=n._isHorizontal?F:B,c=e.attr(l),d=(n._isHorizontal?i.x:i.y)+c,h=n._isHorizontal?i.width:i.height,f=o-d,p=d+h-o,g={};g[l]=c,f>0&&p>0&&(p-f>1?g[l]+=f+s:f-p>1?g[l]-=p+s:g[l]+=r===u?f+s:-(p+s),e.attr(g))},_shiftThroughAxisOverlappedTick:function(e){var t,n=this,i=e.label;if(i){var o=i.getBBox(),a=null===(t=e.mark)||void 0===t?void 0:t.getBBox(),r=n.getAxisPosition(),s=n.getOptions().label,l=s.indentFromAxis,u=s.position,c=n._isHorizontal?B:F,d=n._isHorizontal?"translateY":"translateX",h=i.attr(d),f=(n._isHorizontal?o.y:o.x)+h,p=n._isHorizontal?o.height:o.width,g={};if(g[d]=h+(u===c?r-f+l:-(f-r+p+l)),i.attr(g),e.mark){var _=n._isHorizontal?a.y:a.x,m=n._isHorizontal?a.height:a.width;g[d]=2*(r-_)-m+1,e.mark.attr(g)}}},_detectElementsOverlapping:function(e,t){if(!e||!t)return!1;var n=e.getBBox(),i=n.x+e.attr("translateX"),o=n.y+e.attr("translateY"),a=t.getBBox(),r=a.x+t.attr("translateX"),s=a.y+t.attr("translateY");return(r>=i&&r<=i+n.width||i>=r&&i<=r+a.width)&&(s>=o&&s<=o+n.height||o>=s&&o<=s+a.height)}}};t.default=K,e.exports=t.default},function(e,t,n){function i(e){return e<0||e>1?void 0:e}function o(e){return e<0||e>1?F:e}function a(e,t){return e.some(function(e){return!e.getOptions().ignoreEmptyPoints||e.getPointsByArg(t,!0).some(function(e){return e.hasValue()})})}function r(e,t,n,o,a,r,s){e.forEach(function(e){var d,h=s(t.indexOf(o),t.length),f=e.getPointsByArg(n,!0),p=i(e.getOptions().barPadding),g=e.getOptions().barWidth,_=u(h,a),m=a.width;h!==-1&&(((0,D.isDefined)(p)||(0,D.isDefined)(g))&&(d=l(r,t.length,1-p,g),m=d.width,_=u(h,d)),c(f,m,_))})}function s(e,t,n){var i=[],s=[],u={},c=t.barGroupWidth,d=e[0]&&e[0].getArgumentAxis().getTranslator().getInterval(),h=c?d>c?c:d:d*(1-o(t.barGroupPadding));e.forEach(function(e,t){var n,o=e.getStackName()||e.getBarOverlapGroup()||t.toString();for(n in e.pointsByArgument)s.indexOf(n.valueOf())===-1&&s.push(n.valueOf());i.indexOf(o)===-1&&(i.push(o),u[o]=[]),u[o].push(e)}),s.forEach(function(e){var t=i.reduce(function(t,n){return a(u[n],e)&&t.push(n),t},[]),o=l(h,t.length);i.forEach(function(i){r(u[i],t,e,i,o,h,n)})})}function l(e,t,n,i){var o,a;return i?(a=V(i,P(e/t)),o=t>1?P((e-a*t)/(t-1)):0):(0,D.isDefined)(n)?(a=P(e*n/t),o=P(t>1?(e-e*n)/(t-1):0)):(o=P(e/t*.2),a=P((e-o*(t-1))/t)),{width:a>1?a:1,spacing:o,middleIndex:t/2}}function u(e,t){return(e-t.middleIndex+.5)*t.width-(t.middleIndex-e-.5)*t.spacing}function c(e,t,n){(0,T.each)(e,function(e,i){i.correctCoordinates({width:t,offset:n})})}function d(e){return e>=0?"positive":"negative"}function h(e){return e.series.filter(function(e){return e.isVisible()})}function f(e,t,n){var i=(e.positive[t]||{})[n]||0,o=-(e.negative[t]||{})[n]||0;return i+o}function p(e,t,n){var i=(e.positive[t]||{})[n]||0,o=(e.negative[t]||{})[n]||0;return i+o}function g(e){return e?function(e,t){return t-e-1}:function(e){return e}}function _(e){return e[0]&&e[0].getArgumentAxis().getTranslator().isInverted()}function m(){var e=h(this);s(e,this._options,g(_(e)))}function v(e){for(var t,n=e.getPoints(),i=0;i<n.length;i++){var o=n[i];if(t=o.initialValue&&o.initialValue.valueOf(),R(t)>0)break}return(0,O.sign)(t)}function y(){var e=this,t=e._options.negativesAsZeroes,n=h(e),i={positive:{},negative:{}},o={left:{},right:{}},a={},r={};n.forEach(function(e){var n=e.getStackName()||e.getBarOverlapGroup(),s=!1,l=v(e)<0?r:a;e._prevSeries=l[n],l[n]=e,e.holes=(0,I.extend)(!0,{},o),e.getPoints().forEach(function(a,r,l){var u=a.initialValue&&a.initialValue.valueOf(),c=a.argument.valueOf(),d=u>=0?i.positive:i.negative,h="bar"!==e.type;t&&u<0&&(d=i.positive,u=0,a.resetValue()),d[n]=d[n]||{};var f=d[n];if(f[c]?(h&&a.correctValue(f[c]),f[c]+=u):(f[c]=u,h&&a.resetCorrection()),a.hasValue())s&&(s=!1,o.left[c]=(o.left[c]||0)+(a.value.valueOf()-(isFinite(a.minValue)?a.minValue.valueOf():0)),a._skipSetLeftHole=!0);else{var p=l[r-1];!s&&p&&p.hasValue()&&(c=p.argument.valueOf(),p._skipSetRightHole=!0,o.right[c]=(o.right[c]||0)+(p.value.valueOf()-(isFinite(p.minValue)?p.minValue.valueOf():0))),s=!0}})}),n.forEach(function(e){var t=e.holes;e.getPoints().forEach(function(e){var n=e.argument.valueOf();e.resetHoles(),!e._skipSetLeftHole&&e.setHole(t.left[n]||o.left[n]&&0,"left"),!e._skipSetRightHole&&e.setHole(t.right[n]||o.right[n]&&0,"right"),e._skipSetLeftHole=null,e._skipSetRightHole=null})}),e._stackKeepers=i,n.forEach(function(e){e.getPoints().forEach(function(t){var n=t.argument.valueOf(),a=e.getStackName()||e.getBarOverlapGroup(),r=f(i,a,n),s=p(i,a,n);t.setPercentValue(r,s,o.left[n],o.right[n])})})}function b(){var e=this,t=h(e),n=e._stackKeepers,i={positive:{},negative:{}};(0,T.each)(t,function(t,o){var a=o.getOptions().minBarSize,r=o.getValueAxis().getTranslator(),s=a&&r.getMinBarSize(a),l=o.getStackName();(0,T.each)(o.getPoints(),function(t,o){if(o.hasValue()){var a=o.initialValue&&o.initialValue.valueOf(),u=o.argument.valueOf();e.fullStacked&&(a=a/f(n,l,u)||0);var c=r.checkMinBarSize(a,s,o.value),h=d(c),p=i[h][l]=i[h][l]||{};p[u]?(o.minValue=p[u],p[u]+=c):p[u]=c,o.value=p[u]}})}),e.fullStacked&&x(t,i)}function x(e,t){(0,T.each)(e,function(e,n){var i=n.getStackName?n.getStackName():"default";(0,T.each)(n.getPoints(),function(e,n){var o=f(t,i,n.argument.valueOf());0!==o&&(n.value=n.value/o,(0,D.isNumeric)(n.minValue)&&(n.minValue=n.minValue/o))})})}function w(){(0,T.each)(this.series,function(e,t){var n=t.getOptions().minBarSize,i=t.getValueAxis().getTranslator(),o=n&&i.getMinBarSize(n);o&&(0,T.each)(t.getPoints(),function(e,t){t.hasValue()&&(t.value=i.checkMinBarSize(t.initialValue,o))})})}function C(){var e=h(this);s(e,{barGroupPadding:.3},g(_(e)))}function k(){var e=h(this);if(e.length){var t,n,i,o=this._options,a=e[0].getArgumentAxis().getVisibleArea(),r=e[0].getValueAxis().getVisibleArea(),s=V(a[1]-a[0],r[1]-r[0]),l=M(o.minBubbleSize,2),u=M(s*o.maxBubbleSize,2),c=(s*o.maxBubbleSize+o.minBubbleSize)/2,d=1/0,f=-(1/0);(0,T.each)(e,function(e,t){(0,T.each)(t.getPoints(),function(e,t){f=f>t.size?f:t.size,d=d<t.size?d:t.size})});var p=f-d,g=R(u-l);(0,T.each)(e,function(e,o){(0,T.each)(o.getPoints(),function(e,o){f===d?t=P(c):(i=R(o.size-d)/p,n=g*i+l,t=P(B(n))),o.correctCoordinates(t)})})}}function S(e){var t=this;switch(t.type=(0,A.normalizeEnum)(e.type),t.pane=e.pane,t.series=[],t.updateOptions(e),t.type){case"bar":t.adjustSeriesDimensions=m,t.updateSeriesValues=w,t.adjustSeriesValues=y;break;case"rangebar":t.adjustSeriesDimensions=m;break;case"fullstackedbar":t.fullStacked=!0,t.adjustSeriesDimensions=m,t.adjustSeriesValues=y,t.updateSeriesValues=b;break;case"stackedbar":t.adjustSeriesDimensions=m,t.adjustSeriesValues=y,t.updateSeriesValues=b;break;case"fullstackedarea":case"fullstackedline":case"fullstackedspline":case"fullstackedsplinearea":t.fullStacked=!0,t.adjustSeriesValues=y;break;case"stackedarea":case"stackedsplinearea":case"stackedline":case"stackedspline":t.adjustSeriesValues=y;break;case"candlestick":case"stock":t.adjustSeriesDimensions=C;break;case"bubble":t.adjustSeriesDimensions=k}}t.SeriesFamily=S;var D=n(1),I=n(0),T=n(3),O=n(29),E=n(4),A=n(14),P=Math.round,R=Math.abs,M=Math.pow,B=Math.sqrt,V=Math.min,F=.3;S.prototype={constructor:S,adjustSeriesDimensions:E.noop,adjustSeriesValues:E.noop,updateSeriesValues:E.noop,updateOptions:function(e){this._options=e},dispose:function(){this.series=null},add:function(e){var t=this.type;this.series=(0,A.map)(e,function(e){return e.type===t?e:null})}}},function(e,t,n){function i(){return{x:_,y:m}}function o(e){return{x:e.x-_,y:e.y-m,width:e.width+2*_,height:e.height+2*m}}function a(e,t,n,i){var a=n?["x","width","y","height",t,0]:["y","height","x","width",e,1];return function(e,t,r){var s={x:r.x,y:r.y},l=o(e),u=n?r.y-e.y-e.height/2:r.y-e.y;return s.y=n||!n&&t===w?r.y+u:r.y,l[a[0]]<0?s[a[0]]-=l[a[0]]:l[a[0]]+l[a[1]]+u*a[5]>i[a[1]]&&(s[a[0]]-=l[a[0]]+l[a[1]]+u*a[5]-i[a[1]]),a[4]-l[a[3]]/2<0?s[a[2]]-=a[4]-l[a[3]]/2:a[4]+l[a[3]]/2>i[a[3]]&&(s[a[2]]-=a[4]+l[a[3]]/2-i[a[3]]),s}}function r(e,t,n,i){var o=this;o._renderer=e,o._crosshairGroup=i,o._options={},o.update(t,n)}t.getMargins=i,t.Crosshair=r;var s=n(14),l=n(0),u=Math,c=u.abs,d=u.min,h=u.max,f=u.floor,p="horizontal",g="vertical",_=8,m=4,v="center",y="right",b="left",x="top",w="bottom";r.prototype={constructor:r,update:function(e,t){var n=this,i=t.canvas;n._canvas={top:i.top,bottom:i.height-i.bottom,left:i.left,right:i.width-i.right,width:i.width,height:i.height},n._axes=t.axes,n._panes=t.panes,n._prepareOptions(e,p),n._prepareOptions(e,g)},dispose:function(){var e=this;e._renderer=e._crosshairGroup=e._options=e._axes=e._canvas=e._horizontalGroup=e._verticalGroup=e._horizontal=e._vertical=e._circle=e._panes=null},_prepareOptions:function(e,t){var n=e[t+"Line"];this._options[t]={visible:n.visible,line:{stroke:n.color||e.color,"stroke-width":n.width||e.width,dashStyle:n.dashStyle||e.dashStyle,opacity:n.opacity||e.opacity,"stroke-linecap":"butt"},label:(0,l.extend)(!0,{},e.label,n.label)}},_createLines:function(e,t,n){for(var i=[],o=this._canvas,a=[o.left,o.top,o.left,o.top],r=0;r<2;r++)i.push(this._renderer.path(a,"line").attr(e).sharp(t).append(n));return i},render:function(){var e=this,t=e._renderer,n=e._options,i=n.vertical,o=n.horizontal,a=o.visible?o.line:i.line,r={stroke:a.stroke,"stroke-width":a["stroke-width"],dashStyle:a.dashStyle,opacity:a.opacity},s=e._canvas;e._horizontal={},e._vertical={},e._circle=t.circle(s.left,s.top,0).attr(r).append(e._crosshairGroup),e._horizontalGroup=t.g().append(e._crosshairGroup),e._verticalGroup=t.g().append(e._crosshairGroup),i.visible&&(e._vertical.lines=e._createLines(i.line,"h",e._verticalGroup),e._vertical.labels=e._createLabels(e._axes[0],i,!1,e._verticalGroup)),o.visible&&(e._horizontal.lines=e._createLines(o.line,"v",e._horizontalGroup),e._horizontal.labels=e._createLabels(e._axes[1],o,!0,e._horizontalGroup)),e.hide()},_createLabels:function(e,t,n,i){var o,a,r,l,u,c=this,d=c._canvas,h=c._renderer,f=[],p=t.label;return p.visible&&e.forEach(function(e){var c=e.getOptions().position;if(!e.getTranslator().getBusinessRange().isEmpty()){u=e.getLabelsPosition(),n?(a=d.top,o=u):(o=d.left,a=u);var g=c===x||c===w?v:c===y?b:y;l=h.rect(0,0,0,0).attr({fill:p.backgroundColor||t.line.stroke}).append(i),r=h.text("0",0,0).css((0,s.patchFontOptions)(t.label.font)).attr({align:g,"class":p.cssClass}).append(i),f.push({text:r,background:l,axis:e,options:p,pos:{coord:u,side:c},startXY:{x:o,y:a}})}}),f},_updateText:function(e,t,n,i,a){var r=this;n.forEach(function(n){var s=n.axis,l=n.startXY,u=n.text,c=n.background,d="";s.name&&s.name!==t||(d=s.getFormattedValue(e,n.options,i)),d?(u.attr({text:d,x:l.x,y:l.y}),u.attr(a(u.getBBox(),n.pos.side,l)),r._updateLinesCanvas(n),c.attr(o(u.getBBox()))):(u.attr({text:""}),c.attr({x:0,y:0,width:0,height:0}))})},hide:function(){this._crosshairGroup.attr({visibility:"hidden"})},_updateLinesCanvas:function(e){var t=e.pos.side,n=e.pos.coord,i=this._linesCanvas,o=this._canvas;i[t]=i[t]!==o[t]&&c(i[t]-o[t])<c(n-o[t])?i[t]:n},_updateLines:function(e,t,n,i,o){for(var a=this._linesCanvas,r=this._canvas,s=o?[[d(t-i,a.left),r.top,t-i,r.top],[t+i,r.top,h(a.right,t+i),r.top]]:[[r.left,d(a.top,n-i),r.left,n-i],[r.left,n+i,r.left,h(a.bottom,n+i)]],l=0;l<2;l++)e[l].attr({points:s[l]}).sharp(o?"v":"h",o?n===r.bottom?-1:1:t===r.right?-1:1)},_resetLinesCanvas:function(){var e=this._canvas;this._linesCanvas={left:e.left,right:e.right,top:e.top,bottom:e.bottom}},_getClipRectForPane:function(e,t){var n,i,o=this._panes;for(n=0;n<o.length;n++)if(i=o[n].coords,i.left<=e&&i.right>=e&&i.top<=t&&i.bottom>=t)return o[n].clipRect;return{id:null}},show:function(e){var t=this,n=e.point,i=n.getCrosshairData(e.x,e.y),o=n.getPointRadius(),r=t._horizontal,s=t._vertical,l=o?o+3:0,u=t._canvas,c=f(i.x),d=f(i.y);c>=u.left&&c<=u.right&&d>=u.top&&d<=u.bottom?(t._crosshairGroup.attr({visibility:"visible"}),t._resetLinesCanvas(),t._circle.attr({cx:c,cy:d,r:l,"clip-path":t._getClipRectForPane(c,d).id}),r.lines&&(t._updateText(i.yValue,i.axis,r.labels,n,a(c,d,!0,u)),t._updateLines(r.lines,c,d,l,!0),t._horizontalGroup.attr({translateY:d-u.top})),s.lines&&(t._updateText(i.xValue,i.axis,s.labels,n,a(c,d,!1,u)),t._updateLines(s.lines,c,d,l,!1),t._verticalGroup.attr({translateX:c-u.left}))):t.hide()}}},function(e,t,n){function i(){this.setDomain(arguments[0],arguments[1]).setCodomain(arguments[2],arguments[3]).setInverted(!1)}t.Translator1D=i;var o=Number;i.prototype={constructor:i,setDomain:function(e,t){var n=this;return n._domain1=o(e),n._domain2=o(t),n._domainDelta=n._domain2-n._domain1,n},setCodomain:function(e,t){var n=this;return n._codomain1=o(e),n._codomain2=o(t),n._codomainDelta=n._codomain2-n._codomain1,n},setInverted:function(e){this.inverted=e},getDomain:function(){return[this._domain1,this._domain2]},getCodomain:function(){return[this._codomain1,this._codomain2]},getDomainStart:function(){return this._domain1},getDomainEnd:function(){return this._domain2},getCodomainStart:function(){return this._codomain1},getCodomainEnd:function(){return this._codomain2},getDomainRange:function(){return this._domainDelta},getCodomainRange:function(){return this._codomainDelta},translate:function(e){var t=(o(e)-this._domain1)/this._domainDelta;return this.inverted&&(t=1-t),0<=t&&t<=1?this._codomain1+t*this._codomainDelta:NaN},adjust:function(e){var t=(o(e)-this._domain1)/this._domainDelta,n=NaN;return t<0?n=this._domain1:t>1?n=this._domain2:0<=t&&t<=1&&(n=o(e)),n}}},function(e,t,n){function i(e,t){return null===e?e:D(e)?I(e):t}function o(e){return S(e)?e:(0,y.isNumeric)(e)?[e]:null}function a(e){return e?e.value:null}function r(e,t,n){for(var o=e[t],r=S(n)?(0,C.map)(n,a):[],s=0,l=r.length,u=[];s<l;++s)u.push(i(r[s],o[s]));e[t]=u}function s(e){void 0===e.option(R)&&void 0===e.option(M)&&void 0!==e.option("valueIndicators")&&(l(e),u(e))}function l(e){e.value=e.subvalues=k.noop,e._setupValue=e._setupSubvalues=e._updateValueIndicator=e._updateSubvalueIndicators=null}function u(e){e._indicatorValues=[],r(e,"_indicatorValues",e.option("valueIndicators")),e._valueIndicators=[];var t=e._applyMostlyTotalChange;e._applyMostlyTotalChange=function(){r(this,"_indicatorValues",this.option("valueIndicators")),t.call(this)},e._updateActiveElements=c,e._prepareValueIndicators=d,e._disposeValueIndicators=h,e._cleanValueIndicators=f,e.indicatorValue=p}function c(){var e=this;A(e._valueIndicators,function(t,n){n.value(e._indicatorValues[n.index],e._noAnimation)})}function d(){var e,t=this,n=t._valueIndicators||[],i=t.option("valueIndicators"),o=[],a=0;for(e=S(i)?i.length:0;a<e;++a)o.push(i[a]);for(e=n.length;a<e;++a)o.push(null);var r=[];return A(o,function(e,i){var o=n[e];if(!i)return void(o&&o.dispose());var a=t._prepareIndicatorSettings(i,"valueIndicatorType");o&&o.type!==a.type&&(o.dispose(),o=null),o||(o=t._createIndicator(a.type,t._renderer.root,"dxg-value-indicator","value-indicator",e,!0)),o&&(o.index=e,o.render(a),r.push(o))}),t._valueIndicators=r,t._valueIndicators}function h(){A(this._valueIndicators,function(e,t){t.dispose()}),this._valueIndicators=null}function f(){A(this._valueIndicators,function(e,t){t.clean()})}function p(e,t){return g(this,this._valueIndicators,this._indicatorValues,e,t)}function g(e,t,n,o,a){return void 0!==a?(void 0!==n[o]&&(n[o]=i(a,n[o]),t[o]&&t[o].value(n[o])),e):n[o]}function _(e){this._parameters=e,this._indicators=[]}function m(e){return function(t,n,i){var o=e[(0,C.normalizeEnum)(n)]||!i&&e._default;return o?new o(t):null}}t.createIndicatorCreator=m,t.dxGauge=void 0;var v=n(203),y=n(1),b=n(3),x=n(0),w=n(232),C=n(14),k=n(4),S=Array.isArray,D=isFinite,I=Number,T=Math.min,O=Math.max,E=x.extend,A=b.each,P=90,R="value",M="subvalues",B=5,V=[1,2,5],F=v.dxBaseGauge.inherit({_initCore:function(){var e=this,t=e._renderer;e._setupValue(e.option(R)),e.__subvalues=o(e.option(M)),e._setupSubvalues(e.__subvalues),s(e),e.callBase.apply(e,arguments),e._rangeContainer=new e._factory.RangeContainer({renderer:t,container:t.root,translator:e._translator,themeManager:e._themeManager}),e._initScale(),e._subvalueIndicatorContainer=e._renderer.g().attr({"class":"dxg-subvalue-indicators"}).linkOn(e._renderer.root,"valueIndicator").enableLinks()},_fontFields:["scale.label.font","valueIndicators.rangebar.text.font","valueIndicators.textcloud.text.font","indicator.text.font"],_initScale:function(){var e=this;e._scaleGroup=e._renderer.g().attr({"class":"dxg-scale"}).linkOn(e._renderer.root,"scale"),e._scale=new w.Axis({incidentOccurred:e._incidentOccurred,renderer:e._renderer,axesContainerGroup:e._scaleGroup,axisType:e._scaleTypes.type,drawingType:e._scaleTypes.drawingType,widgetClass:"dxg",getTemplate:function(){}})},_disposeCore:function(){var e=this;e.callBase.apply(e,arguments),e._scale.dispose(),e._scaleGroup.linkOff(),e._rangeContainer.dispose(),e._disposeValueIndicators(),e._subvalueIndicatorContainer.linkOff(),e._scale=e._scaleGroup=e._rangeContainer=null},_disposeValueIndicators:function(){var e=this;e._valueIndicator&&e._valueIndicator.dispose(),e._subvalueIndicatorsSet&&e._subvalueIndicatorsSet.dispose(),e._valueIndicator=e._subvalueIndicatorsSet=null},_setupDomainCore:function(){var e=this,t=e.option("scale")||{},n=e.option("startValue"),i=e.option("endValue");n=(0,y.isNumeric)(n)?I(n):(0,y.isNumeric)(t.startValue)?I(t.startValue):0,i=(0,y.isNumeric)(i)?I(i):(0,y.isNumeric)(t.endValue)?I(t.endValue):100,e._baseValue=n<i?n:i,e._translator.setDomain(n,i)},_cleanContent:function(){var e=this;e._rangeContainer.clean(),e._cleanValueIndicators()},_measureScale:function(e){var t=this,n=e.tick,i=n.visible&&n.length>0&&n.width>0,o=e.minorTick,a=o.visible&&o.length>0&&o.width>0,r=e.label,s=Number(r.indentFromTick);if(!i&&!a&&!r.visible)return{};var l=t._scale.measureLabels((0,x.extend)({},t._canvas)),u=t._getScaleLayoutValue(),c={min:u,max:u},d=t._getTicksCoefficients(e),h=d.inner,f=d.outer;return i&&(c.min=T(c.min,u-h*n.length),c.max=O(c.max,u+f*n.length)),a&&(c.min=T(c.min,u-h*o.length),c.max=O(c.max,u+f*o.length)),r.visible&&t._correctScaleIndents(c,s,l),c},_renderContent:function(){var e=this,t=e._prepareScaleSettings();e._rangeContainer.render(E(e._getOption("rangeContainer"),{vertical:e._area.vertical})),e._renderScale(t),e._subvalueIndicatorContainer.linkAppend();var n=(0,C.map)([e._rangeContainer].concat(e._prepareValueIndicators()),function(e){return e&&e.enabled?e:null});e._applyMainLayout(n,e._measureScale(t)),A(n,function(t,n){n.resize(e._getElementLayout(n.getOffset()))}),e._shiftScale(e._getElementLayout(0),t),e._beginValueChanging(),e._updateActiveElements(),e._endValueChanging()},_prepareScaleSettings:function(){var e=this,t=e.option("scale"),n=(0,x.extend)(!0,{},e._themeManager.theme("scale"),t);return n.label.indentFromAxis=0,n.isHorizontal=!e._area.vertical,n.forceUserTickInterval|=(0,y.isDefined)(t)&&(0,y.isDefined)(t.tickInterval)&&!(0,y.isDefined)(t.scaleDivisionFactor),n.axisDivisionFactor=n.scaleDivisionFactor||e._gridSpacingFactor,n.minorAxisDivisionFactor=n.minorScaleDivisionFactor||B,n.numberMultipliers=V,n.tickOrientation=e._getTicksOrientation(n),n.label.useRangeColors&&(n.label.customizeColor=function(){return e._rangeContainer.getColorForValue(this.value)}),n},_renderScale:function(e){var t=this,n=t._translator.getDomain(),i=n[0],o=n[1],a=t._translator.getCodomain(),r=!!(i>o^e.inverted),s=T(i,o),l=O(i,o);e.min=s,e.max=l,e.startAngle=P-a[0],e.endAngle=P-a[1],e.skipViewportExtending=!0,e.inverted=r,t._scale.updateOptions(e),t._scale.setBusinessRange({axisType:"continuous",dataType:"numeric",min:s,max:l,invert:r}),t._updateScaleTickIndent(e),t._scaleGroup.linkAppend(),t._scale.draw((0,x.extend)({},t._canvas))},_updateIndicatorSettings:function(e){var t=this;e.currentValue=e.baseValue=D(t._translator.translate(e.baseValue))?I(e.baseValue):t._baseValue,e.vertical=t._area.vertical,e.text&&!e.text.format&&(e.text.format=t._defaultFormatOptions)},_prepareIndicatorSettings:function(e,t){var n=this,i=n._themeManager.theme("valueIndicators"),o=(0,C.normalizeEnum)(e.type||n._themeManager.theme(t)),a=E(!0,{},i._default,i[o],e);return a.type=o,a.animation=n._animationSettings,a.containerBackgroundColor=n._containerBackgroundColor,n._updateIndicatorSettings(a),a},_cleanValueIndicators:function(){this._valueIndicator&&this._valueIndicator.clean(),this._subvalueIndicatorsSet&&this._subvalueIndicatorsSet.clean()},_prepareValueIndicators:function(){var e=this;return e._prepareValueIndicator(),null!==e.__subvalues&&e._prepareSubvalueIndicators(),[e._valueIndicator,e._subvalueIndicatorsSet]},_updateActiveElements:function(){this._updateValueIndicator(),this._updateSubvalueIndicators()},_prepareValueIndicator:function(){var e=this,t=e._valueIndicator,n=e._prepareIndicatorSettings(e.option("valueIndicator")||{},"valueIndicatorType");t&&t.type!==n.type&&(t.dispose(),t=null),t||(t=e._valueIndicator=e._createIndicator(n.type,e._renderer.root,"dxg-value-indicator","value-indicator")),t.render(n)},_createSubvalueIndicatorsSet:function(){var e=this,t=e._subvalueIndicatorContainer;return new _({createIndicator:function(n,i){return e._createIndicator(n,t,"dxg-subvalue-indicator","subvalue-indicator",i)},createPalette:function(t){return e._themeManager.createPalette(t)}})},_prepareSubvalueIndicators:function(){var e=this,t=e._subvalueIndicatorsSet,n=e._prepareIndicatorSettings(e.option("subvalueIndicator")||{},"subvalueIndicatorType");t||(t=e._subvalueIndicatorsSet=e._createSubvalueIndicatorsSet());var i=n.type!==t.type;t.type=n.type;var o=e._createIndicator(n.type,e._renderer.root);o&&(o.dispose(),t.render(n,i))},_setupValue:function(e){this.__value=i(e,this.__value)},_setupSubvalues:function(e){var t,n,a,r=void 0===e?this.__subvalues:o(e);if(null!==r){for(t=0,n=r.length,a=[];t<n;++t)a.push(i(r[t],this.__subvalues[t]));this.__subvalues=a}},_updateValueIndicator:function(){var e=this;e._valueIndicator&&e._valueIndicator.value(e.__value,e._noAnimation)},_updateSubvalueIndicators:function(){var e=this;e._subvalueIndicatorsSet&&e._subvalueIndicatorsSet.values(e.__subvalues,e._noAnimation)},value:function(e){return void 0!==e?(this._changeValue(e),this):this.__value},subvalues:function(e){return void 0!==e?(this._changeSubvalues(e),this):null!==this.__subvalues?this.__subvalues.slice():void 0},_changeValue:function(e){var t=this;t._setupValue(e),t._beginValueChanging(),t._updateValueIndicator(),t.__value!==t.option(R)&&t.option(R,t.__value),t._endValueChanging()},_changeSubvalues:function(e){var t=this;null!==t.__subvalues?(t._setupSubvalues(e),t._beginValueChanging(),t._updateSubvalueIndicators(),t._endValueChanging()):(t.__subvalues=o(e),t._setContentSize(),t._renderContent()),(0,v.compareArrays)(t.__subvalues,t.option(M))||t.option(M,t.__subvalues)},_optionChangesMap:{scale:"DOMAIN",rangeContainer:"MOSTLY_TOTAL",valueIndicator:"MOSTLY_TOTAL",subvalueIndicator:"MOSTLY_TOTAL",containerBackgroundColor:"MOSTLY_TOTAL",value:"VALUE",subvalues:"SUBVALUES",valueIndicators:"MOSTLY_TOTAL"},_customChangesOrder:["VALUE","SUBVALUES"],_change_VALUE:function(){this._changeValue(this.option(R))},_change_SUBVALUES:function(){this._changeSubvalues(this.option(M))},_applyMainLayout:null,_getElementLayout:null,_createIndicator:function(e,t,n,i,o,a){var r=this,s=r._factory.createIndicator({renderer:r._renderer,translator:r._translator,owner:t,tracker:r._tracker,className:n},e,a);return s&&(s.type=e,s._trackerInfo={type:i,index:o}),s},_getApproximateScreenRange:null});t.dxGauge=F,_.prototype={constructor:_,dispose:function(){var e=this;return A(e._indicators,function(e,t){t.dispose()}),e._parameters=e._options=e._indicators=e._colorPalette=e._palette=null,e},clean:function(){var e=this;return e._sample&&e._sample.clean().dispose(),A(e._indicators,function(e,t){t.clean()}),e._sample=e._options=e._palette=null,e},render:function(e,t){var n=this;return n._options=e,n._sample=n._parameters.createIndicator(n.type),n._sample.render(e),n.enabled=n._sample.enabled,n._palette=(0,y.isDefined)(e.palette)?n._parameters.createPalette(e.palette):null,n.enabled&&(n._generatePalette(n._indicators.length),n._indicators=(0,C.map)(n._indicators,function(e,i){return t&&(e.dispose(),e=n._parameters.createIndicator(n.type,i)),e.render(n._getIndicatorOptions(i)),e})),n},getOffset:function(){return this._sample.getOffset()},resize:function(e){var t=this;return t._layout=e,A(t._indicators,function(t,n){n.resize(e)}),t},measure:function(e){return this._sample.measure(e)},_getIndicatorOptions:function(e){var t=this._options;return this._colorPalette&&(t=E({},t,{color:this._colorPalette[e]})),t},_generatePalette:function(e){var t=this,n=null;t._palette&&(t._palette.reset(),n=t._palette.generateColors(e,{repeat:!0})),t._colorPalette=n},_adjustIndicatorsCount:function(e){var t,n,i,o=this,a=o._indicators,r=a.length;if(r>e){for(t=e,n=r;t<n;++t)a[t].clean().dispose();o._indicators=a.slice(0,e),o._generatePalette(a.length)}else if(r<e)for(o._generatePalette(e),t=r,n=e;t<n;++t)i=o._parameters.createIndicator(o.type,t),i.render(o._getIndicatorOptions(t)).resize(o._layout),a.push(i)},values:function(e,t){var n=this;if(n.enabled)return void 0!==e?(S(e)||(e=D(e)?[Number(e)]:null),e&&(n._adjustIndicatorsCount(e.length),A(n._indicators,function(n,i){i.value(e[n],t)})),n):(0,C.map)(n._indicators,function(e){return e.value()})}}},function(e,t,n){function i(e,t,n,i){var o;return n>e&&i<t?o=[{start:e,end:n},{start:i,end:t}]:n>=t||i<=e?o=[{start:e,end:t}]:n<=e&&i>=t?o=[]:n>e?o=[{start:e,end:n}]:i<t&&(o=[{start:i,end:t}]),o}function o(e,t,n,i){var o;return n<e&&i>t?o=[{start:e,end:n},{start:i,end:t}]:n<=t||i>=e?o=[{start:e,end:t}]:n>=e&&i<=t?o=[]:n<e?o=[{start:e,end:n}]:i>t&&(o=[{start:i,end:t}]),o}function a(e,t,n){var i=n.startValue,o=n.endValue;return o===i&&i===e&&t===e}function r(e,t,n){return t-e>0||a(e,t,n)}function s(e,t,n){return e-t>0||a(e,t,n)}t.default=void 0;var l=n(3),u=n(330),c=n(1),d=Number,h=Array.isArray,f=isFinite,p=u.BaseElement.inherit({_init:function(){this._root=this._renderer.g().attr({"class":"dxg-range-container"}).linkOn(this._container,"range-container")},_dispose:function(){this._root.linkOff()},clean:function(){return this._root.linkRemove().clear(),this._options=this.enabled=null,this},_getRanges:function(){var e=this,t=e._options,n=e._translator,a=n.getDomain()[0],u=n.getDomain()[1],p=u-a,g=p>=0?r:s,_=p>=0?i:o,m=[],v=[],y=[{start:a,end:u}],b=(0,c.isString)(t.backgroundColor)?t.backgroundColor:"none",x=t.width||{},w=d(x>0?x:x.start),C=d(x>0?x:x.end),k=C-w;if(void 0!==t.ranges&&!h(t.ranges))return null;if(!(w>=0&&C>=0&&w+C>0))return null;m=(h(t.ranges)?t.ranges:[]).reduce(function(e,t,i){t=t||{};var o=n.adjust(t.startValue),a=n.adjust(t.endValue);return f(o)&&f(a)&&g(o,a,t)&&e.push({
start:o,end:a,color:t.color,classIndex:i}),e},[]);var S=e._themeManager.createPalette(t.palette,{type:"indicatingSet",extensionMode:t.paletteExtensionMode,keepLastColorInEnd:!0,count:m.length});return(0,l.each)(m,function(e,t){var n=S.getNextColor();t.color=(0,c.isString)(t.color)&&t.color||n||"none",t.className="dxg-range dxg-range-"+t.classIndex,delete t.classIndex}),(0,l.each)(m,function(e,t){var n,i,o,a,r,s=[],l=[];for(n=0,i=v.length;n<i;++n)r=v[n],a=_(r.start,r.end,t.start,t.end),(o=a[0])&&(o.color=r.color)&&(o.className=r.className)&&s.push(o),(o=a[1])&&(o.color=r.color)&&(o.className=r.className)&&s.push(o);for(s.push(t),v=s,n=0,i=y.length;n<i;++n)r=y[n],a=_(r.start,r.end,t.start,t.end),(o=a[0])&&l.push(o),(o=a[1])&&l.push(o);y=l}),(0,l.each)(y,function(e,t){t.color=b,t.className="dxg-range dxg-background-range",v.push(t)}),(0,l.each)(v,function(e,t){t.startWidth=(t.start-a)/p*k+w,t.endWidth=(t.end-a)/p*k+w}),v},render:function(e){var t=this;return t._options=e,t._processOptions(),t._ranges=t._getRanges(),t._ranges&&(t.enabled=!0,t._root.linkAppend()),t},resize:function(e){var t=this;return t._root.clear(),t._isVisible(e)&&(0,l.each)(t._ranges,function(n,i){t._createRange(i,e).attr({fill:i.color,"class":i.className}).append(t._root)}),t},_processOptions:null,_isVisible:null,_createRange:null,getColorForValue:function(e){var t=null;return(0,l.each)(this._ranges,function(n,i){if(i.start<=e&&e<=i.end||i.start>=e&&e>=i.end)return t=i.color,!1}),t}}),g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){var n=w(e),i=w(t),o=n.cos,a=n.sin,r=i.cos,s=i.sin;return{left:a<=0&&s>=0||a<=0&&s<=0&&o<=r||a>=0&&s>=0&&o>=r?-1:S(o,r,0),right:a>=0&&s<=0||a>=0&&s>=0&&o>=r||a<=0&&s<=0&&o<=r?1:k(o,r,0),up:o<=0&&r>=0||o<=0&&r<=0&&a>=s||o>=0&&r>=0&&a<=s?-1:-k(a,s,0),down:o>=0&&r<=0||o>=0&&r>=0&&a<=s||o<=0&&r<=0&&a>=s?1:-S(a,s,0)}}function l(e){return e.right-e.left}function u(e){return e.bottom-e.top}function c(e,t,n){var i,o=(0,f.extend)({},e),a=0,r=0;return n=n||{},t>0&&(o.left+=n.left||0,o.right-=n.right||0,o.top+=n.top||0,o.bottom-=n.bottom||0,l(o)>0&&u(o)>0?(i=u(o)/l(o),i>1?t<i?a=l(o):r=u(o):t>i?r=u(o):a=l(o),a>0||(a=r/t),r>0||(r=a*t),a=(l(o)-a)/2,r=(u(o)-r)/2,o.left+=a,o.right-=a,o.top+=r,o.bottom-=r):(o.left=o.right=(o.left+o.right)/2,o.top=o.bottom=(o.top+o.bottom)/2)),o}t.default=void 0;var d=r(n(9)),h=n(50),f=n(0),p=n(3),g=n(203),_=n(558),m=n(14),v=a(n(951)),y=r(n(952)),b=isFinite,x=m.normalizeAngle,w=m.getCosAndSin,C=Math.abs,k=Math.max,S=Math.min,D=Math.round,I=p.each,T=Math.PI,O=_.dxGauge.inherit({_rootClass:"dxg-circular-gauge",_factoryMethods:{rangeContainer:"createCircularRangeContainer",indicator:"createCircularIndicator"},_gridSpacingFactor:17,_scaleTypes:{type:"polarAxes",drawingType:"circular"},_getThemeManagerOptions:function(){var e=this.callBase.apply(this,arguments);return e.subTheme="_circular",e},_updateScaleTickIndent:function(e){var t=e.label.indentFromTick,n=e.tick.visible?e.tick.length:0,i=this._scale.measureLabels((0,f.extend)({},this._canvas)),o=n;"inside"===e.orientation?o=0:"center"===e.orientation&&(o=.5*n),e.label.indentFromAxis=t>=0?t+o:t-o-k(i.width,i.height),this._scale.updateOptions(e)},_setupCodomain:function(){var e,t=this,n=t.option("geometry")||{},i=n.startAngle,o=n.endAngle;i=b(i)?x(i):225,o=b(o)?x(o):-45,C(i-o)<1?(o-=360,e={left:-1,up:-1,right:1,down:1}):(i<o&&(o-=360),e=s(i,o)),t._area={x:0,y:0,radius:100,startCoord:i,endCoord:o,sides:e},t._translator.setCodomain(i,o)},_shiftScale:function(e){var t=this._scale,n=t.getCanvas();n.width=n.height=2*e.radius,t.draw(n);var i=t.getCenter();t.shift({right:e.x-i.x,bottom:e.y-i.y})},_getScaleLayoutValue:function(){return this._area.radius},_getTicksOrientation:function(e){return e.orientation},_getTicksCoefficients:function(e){var t={inner:0,outer:1};return"inside"===e.orientation?(t.inner=1,t.outer=0):"center"===e.orientation&&(t.inner=t.outer=.5),t},_correctScaleIndents:function(e,t,n){t>=0?(e.horizontalOffset=t+n.width,e.verticalOffset=t+n.height):(e.horizontalOffset=e.verticalOffset=0,e.min-=-t+k(n.width,n.height)),e.inverseHorizontalOffset=n.width/2,e.inverseVerticalOffset=n.height/2},_measureMainElements:function(e,t){var n=this,i=n._area.radius,o=0,a=1/0,r=0,s=0,l=0,u=0,c=n._scale;return I(e.concat(c),function(e,n){var c=n.measure?n.measure({radius:i-n.getOffset()}):t;c.min>0&&(a=S(a,c.min)),c.max>0&&(o=k(o,c.max)),c.horizontalOffset>0&&(r=k(r,c.max+c.horizontalOffset)),c.verticalOffset>0&&(s=k(s,c.max+c.verticalOffset)),c.inverseHorizontalOffset>0&&(l=k(l,c.inverseHorizontalOffset)),c.inverseVerticalOffset>0&&(u=k(u,c.inverseVerticalOffset))}),r=k(r-o,0),s=k(s-o,0),{minRadius:a,maxRadius:o,horizontalMargin:r,verticalMargin:s,inverseHorizontalMargin:l,inverseVerticalMargin:u}},_applyMainLayout:function(e,t){var n=this._measureMainElements(e,t),i=this._area,o=i.sides,a={left:(o.left<-.1?n.horizontalMargin:n.inverseHorizontalMargin)||0,right:(o.right>.1?n.horizontalMargin:n.inverseHorizontalMargin)||0,top:(o.up<-.1?n.verticalMargin:n.inverseVerticalMargin)||0,bottom:(o.down>.1?n.verticalMargin:n.inverseVerticalMargin)||0},r=c(this._innerRect,(o.down-o.up)/(o.right-o.left),a),s=S(l(r)/(o.right-o.left),u(r)/(o.down-o.up));s=s-n.maxRadius+i.radius;var d=r.left-l(r)*o.left/(o.right-o.left),h=r.top-u(r)*o.up/(o.down-o.up);i.x=D(d),i.y=D(h),i.radius=s,r.left-=a.left,r.right+=a.right,r.top-=a.top,r.bottom+=a.bottom,this._innerRect=r},_getElementLayout:function(e){return{x:this._area.x,y:this._area.y,radius:D(this._area.radius-e)}},_getApproximateScreenRange:function(){var e=this,t=e._area,n=S(e._canvas.width/(t.sides.right-t.sides.left),e._canvas.height/(t.sides.down-t.sides.up));return n>t.totalRadius&&(n=t.totalRadius),n=.8*n,-e._translator.getCodomainRange()*n*T/180},_getDefaultSize:function(){return{width:300,height:300}},_factory:(0,h.clone)(g.dxBaseGauge.prototype._factory)}),E=O.prototype._factory.indicators={};O.prototype._factory.createIndicator=(0,_.createIndicatorCreator)(E),E._default=v._default,E.rectangleneedle=v.rectangleneedle,E.triangleneedle=v.triangleneedle,E.twocolorneedle=v.twocolorneedle,E.trianglemarker=v.trianglemarker,E.textcloud=v.textcloud,E.rangebar=v.rangebar,O.prototype._factory.RangeContainer=y.default,(0,d.default)("dxCircularGauge",O);var A=O;t.default=A,e.exports=t.default},function(e,t,n){function i(e,t){return l(e)||s(e,t)||a(e,t)||o()}function o(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a(e,t){if(e){if("string"==typeof e)return r(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?r(e,t):void 0}}function r(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function s(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function l(e){if(Array.isArray(e))return e}function u(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function c(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function d(e,t,n){return t&&c(e.prototype,t),n&&c(e,n),e}function h(e,t){return R(e-t)<1e-8}function f(e,t){return h(e[0],t[0])&&h(e[1],t[1])}function p(e,t,n,i){var o=E(e);return isFinite(o)?A(P(o,t),n):i}function g(e,t,n,i){return[p(e[0],t[0],n[0],i[0]),p(e[1],t[1],n[1],i[1])]}function _(e){return e instanceof G&&e||q.get(e)||q(e)||q.get(j)}function m(e,t,n,i){var o=(n+i)/2;return R(e-o)>R(t-o)?e:t}function v(e,t,n,i){var o=(n+i)/2;return R(e-o)<R(t-o)?e:t}function y(e,t,n){var i=e([n[0],n[3]]),o=e([n[0],n[1]]),a=e([n[2],n[3]]),r=e([n[2],n[1]]),s=v(i[0],o[0],a[0],r[0]),l=v(a[0],r[0],i[0],o[0]),u=v(i[1],a[1],o[1],r[1]),c=v(o[1],r[1],i[1],a[1]);return[2/P(R(s-l),R(u-c)),t([(s+l)/2,(u+c)/2])]}function b(e,t,n){var i=D(t,n),o=i.min,a=i.max;e.min=S(o),e.max=S(a)}function x(e){return[e[0],-e[1]]}function w(e){return function(t){return x(e(t))}}function C(e){return function(t){return e(x(t))}}function k(e){return function(){return e}}function S(e){return function(){return e.slice()}}function D(e,t){return{min:[A(e[0],t[0]),A(e[1],t[1])],max:[P(e[0],t[0]),P(e[1],t[1])]}}function I(e,t,n,o,a){var r=(n[0]+o[0])/2-a/2,s=(n[1]+o[1])/2-a/2,l=2/a;return{to:function(t){var n=e(t),o=i(n,2),a=o[0],u=o[1];return[-1+(a-r)*l,-1+(u-s)*l]},from:function(e){return t([r+(e[0]+1)/l,s+(e[1]+1)/l])}}}t.projection=t.Projection=void 0;var T=n(0),O=n(562),E=Number,A=Math.min,P=Math.max,R=Math.abs,M=Math.round,B=Math.log,V=Math.pow,F=2/Math.LN2,L=1/3600/180/10,H=1,z=256,N=[NaN,NaN],j="mercator",W=function(e){var t=this;t._initEvents(),t._params=e,t._engine=_(),t._center=t._engine.center(),t._adjustCenter()};t.Projection=W,W.prototype={constructor:W,_minZoom:H,_maxZoom:z,_zoom:H,_center:N,_canvas:{},_scale:[],dispose:function(){this._disposeEvents()},setEngine:function(e){var t=this,n=_(e);t._engine!==n&&(t._engine=n,t._fire("engine"),t._changeCenter(n.center())&&t._triggerCenterChanged(),t._changeZoom(t._minZoom)&&t._triggerZoomChanged(),t._adjustCenter(),t._setupScreen())},setBounds:function(e){void 0!==e&&this.setEngine(this._engine.original().bounds(e))},_setupScreen:function(){var e=this,t=e._canvas,n=t.width,i=t.height,o=e._engine,a=o.ar();e._x0=t.left+n/2,e._y0=t.top+i/2;var r,s=[e.project([o.min()[0],0])[0],e.project([0,o.min()[1]])[1]],l=[e.project([o.max()[0],0])[0],e.project([0,o.max()[1]])[1]],u=n/i,c=R(l[0]-s[0])/R(l[1]-s[1]);r=isNaN(c)||0===c||A(u,a)<=a*c&&a*c<=P(u,a)?1:c>1?c:1/c,a*c>=u?(e._xRadius=n/2/r,e._yRadius=n/2/(a*r)):(e._xRadius=i/2*(a/r),e._yRadius=i/2/r),e._fire("screen")},setSize:function(e){this._canvas=e,this._setupScreen()},getCanvas:function(){return this._canvas},_toScreen:function(e){return[this._x0+this._xRadius*e[0],this._y0+this._yRadius*e[1]]},_fromScreen:function(e){return[(e[0]-this._x0)/this._xRadius,(e[1]-this._y0)/this._yRadius]},_toTransformed:function(e){return[e[0]*this._zoom+this._xCenter,e[1]*this._zoom+this._yCenter]},_toTransformedFast:function(e){return[e[0]*this._zoom,e[1]*this._zoom]},_fromTransformed:function(e){return[(e[0]-this._xCenter)/this._zoom,(e[1]-this._yCenter)/this._zoom]},_adjustCenter:function(){var e=this,t=e._engine.project(e._center);e._xCenter=-t[0]*e._zoom||0,e._yCenter=-t[1]*e._zoom||0},project:function(e){return this._engine.project(e)},transform:function(e){return this._toScreen(this._toTransformedFast(e))},isInvertible:function(){return this._engine.isInvertible()},getSquareSize:function(e){return[e[0]*this._zoom*this._xRadius,e[1]*this._zoom*this._yRadius]},getZoom:function(){return this._zoom},_changeZoom:function(e){var t=this,n=t._zoom,i=t._zoom=p(e,t._minZoom,t._maxZoom,t._minZoom),o=!h(n,i);return o&&(t._adjustCenter(),t._fire("zoom")),o},setZoom:function(e){this._engine.isInvertible()&&this._changeZoom(e)&&this._triggerZoomChanged()},getScaledZoom:function(){return M((this._scale.length-1)*B(this._zoom)/B(this._maxZoom))},setScaledZoom:function(e){this.setZoom(this._scale[M(e)])},changeScaledZoom:function(e){this.setZoom(this._scale[P(A(M(this.getScaledZoom()+e),this._scale.length-1),0)])},getZoomScalePartition:function(){return this._scale.length-1},_setupScaling:function(){var e=this,t=P(M(F*B(e._maxZoom)),4),n=V(e._maxZoom,1/t),i=e._minZoom;e._scale=[i];for(var o=1;o<=t;++o)e._scale.push(i*=n)},setMaxZoom:function(e){var t=this;t._minZoom=H,t._maxZoom=p(e,t._minZoom,E.MAX_VALUE,z),t._setupScaling(),t._zoom>t._maxZoom&&t.setZoom(t._maxZoom),t._fire("max-zoom")},getCenter:function(){return this._center.slice()},setCenter:function(e){this._engine.isInvertible()&&this._changeCenter(e||[])&&this._triggerCenterChanged()},_changeCenter:function(e){var t=this,n=t._engine,i=t._center,o=t._center=g(e,n.min(),n.max(),n.center()),a=!f(i,o);return a&&(t._adjustCenter(),t._fire("center")),a},_triggerCenterChanged:function(){this._params.centerChanged(this.getCenter())},_triggerZoomChanged:function(){this._params.zoomChanged(this.getZoom())},setCenterByPoint:function(e,t){var n=this,i=n._engine.project(e),o=n._fromScreen(t);n.setCenter(n._engine.unproject([-o[0]/n._zoom+i[0],-o[1]/n._zoom+i[1]]))},beginMoveCenter:function(){this._engine.isInvertible()&&(this._moveCenter=this._center)},endMoveCenter:function(){var e=this;e._moveCenter&&(f(e._moveCenter,e._center)||e._triggerCenterChanged(),e._moveCenter=null)},moveCenter:function(e){var t=this;if(t._moveCenter){var n=t.toScreenPoint(t._center);t._changeCenter(t.fromScreenPoint([n[0]+e[0],n[1]+e[1]]))}},getViewport:function(){var e=this,t=e._engine.unproject,n=t(e._fromTransformed([-1,-1])),i=t(e._fromTransformed([-1,1])),o=t(e._fromTransformed([1,-1])),a=t(e._fromTransformed([1,1])),r=D([m(n[0],i[0],o[0],a[0]),m(n[1],o[1],i[1],a[1])],[m(o[0],a[0],n[0],i[0]),m(i[1],a[1],n[1],o[1])]);return[].concat(r.min,r.max)},setViewport:function(e){var t=this._engine,n=e?y(t.project,t.unproject,e):[this._minZoom,t.center()];this.setZoom(n[0]),this.setCenter(n[1])},getTransform:function(){return{translateX:this._xCenter*this._xRadius,translateY:this._yCenter*this._yRadius}},fromScreenPoint:function(e){return this._engine.unproject(this._fromTransformed(this._fromScreen(e)))},toScreenPoint:function(e){return this._toScreen(this._toTransformed(this._engine.project(e)))},_eventNames:["engine","screen","center","zoom","max-zoom"]},(0,O.makeEventEmitter)(W);var G=function(){function e(t){u(this,e);var n=this,i=w(t.to),o=t.from?C(t.from):k(N);n.project=i,n.unproject=o,n.original=k(n),n.source=function(){return(0,T.extend)({},t)},n.isInvertible=k(!!t.from),n.ar=k(t.aspectRatio>0?E(t.aspectRatio):1),n.center=S(o([0,0])),b(n,[o([-1,0])[0],o([0,1])[1]],[o([1,0])[0],o([0,-1])[1]])}return d(e,[{key:"aspectRatio",value:function(t){var n=new e((0,T.extend)(this.source(),{aspectRatio:t}));return n.original=this.original,n.min=this.min,n.max=this.max,n}},{key:"bounds",value:function(t){t=t||[];var n=this.source(),i=this.min(),o=this.max(),a=g([t[0],t[1]],i,o,i),r=g([t[2],t[3]],i,o,o),s=n.to(a),l=n.to(r),u=A(R(l[0]-s[0])>L?R(l[0]-s[0]):2,R(l[1]-s[1])>L?R(l[1]-s[1]):2);u<2&&(0,T.extend)(n,I(n.to,n.from,s,l,u));var c=new e(n);return c.original=this.original,b(c,a,r),c}}]),e}(),q=function(e){return e&&e.to?new G(e):null};t.projection=q;var $={};q.get=function(e){return $[e]||null},q.add=function(e,t){return t=t instanceof G&&t||q(t),!$[e]&&t&&($[e]=t),q}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t,n=e.prototype;for(t in r)n[t]=r[t]}t.makeEventEmitter=o;var a=i(n(27)),r={_initEvents:function(){var e,t=this._eventNames,n=t.length,i=this._events={};for(e=0;e<n;++e)i[t[e]]=(0,a.default)()},_disposeEvents:function(){var e,t=this._events;for(e in t)t[e].empty();this._events=null},on:function(e){function t(){for(n in e)i[n].remove(e[n])}var n,i=this._events;for(n in e)i[n].add(e[n]);return t},_fire:function(e,t){this._events[e].fire(t)}}},function(e,t,n){function i(e,t){return r(a(e,+t),-t)}Object.defineProperty(t,"projection",{enumerable:!0,get:function(){return o.projection}});var o=n(561),a=Math.min,r=Math.max,s=Math.sin,l=Math.asin,u=Math.tan,c=Math.atan,d=Math.exp,h=Math.log,f=Math.PI,p=f/4,g=180,_=90,m=f/180,v=(2*c(d(f))-f/2)/m,y=(2.5*c(d(.8*f))-.625*f)/m;o.projection.add("mercator",(0,o.projection)({aspectRatio:1,to:function(e){return[e[0]/g,h(u(p+i(e[1],v)*m/2))/f]},from:function(e){return[e[0]*g,(2*c(d(e[1]*f))-f/2)/m]}})),o.projection.add("equirectangular",(0,o.projection)({aspectRatio:2,to:function(e){return[e[0]/g,e[1]/_]},from:function(e){return[e[0]*g,e[1]*_]}})),o.projection.add("lambert",(0,o.projection)({aspectRatio:2,to:function(e){return[e[0]/g,s(i(e[1],_)*m)]},from:function(e){return[e[0]*g,l(i(e[1],1))/m]}})),o.projection.add("miller",(0,o.projection)({aspectRatio:1,to:function(e){return[e[0]/g,1.25*h(u(p+i(e[1],y)*m*.4))/f]},from:function(e){return[e[0]*g,(2.5*c(d(.8*e[1]*f))-.625*f)/m]}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=e.left,o=e.top,a=e.bottom,r=e.right,s=e.width,l=e.height;return(0,m.pointInCanvas)({left:i,top:o,right:s-r,bottom:l-a,width:s,height:l},t,n)}function a(e){var t=e.data,n=t.widget;n._enableOutHandler(),n._showTooltip()}function r(e,t){var n=e.lineSpacing,i=(void 0!==n&&null!==n?n:w)+e.size;return function(e){for(var n="",o=e.valueText,a=0;a<o.length;a+=2)n+="<tr><td>"+o[a]+"</td><td style='width: 15px'></td><td style='text-align: "+(t?"left":"right")+"'>"+o[a+1]+"</td></tr>";return{html:"<table style='border-spacing:0px; line-height: "+i+"px'>"+n+"</table>"}}}function s(e,t,n){var i=r(t,n);return(0,h.isFunction)(e)?function(t){var n=e.call(t,t);return"html"in n||"text"in n||S(n,i.call(t,t)),n}:i}function l(e){var t=new v.Translator2D({},{},{shiftZeroValue:!e,isHorizontal:!!e});return{getTranslator:function(){return t},update:function(e,n,i){t.update(e,n,i)},getVisibleArea:function(){var e=t.getCanvasVisibleArea();return[e.min,e.max]},visualRange:y.noop,calculateInterval:y.noop,getMarginOptions:function(){return{}}}}t.default=void 0;var u,c=i(n(5)),d=i(n(11)),h=n(1),f=i(n(112)),p=n(0),g=n(8),_=i(n(25)),m=n(14),v=n(329),y=n(4),b=n(133),x=n(111),w=2,C="sparkline-tooltip",k=(0,g.addNamespace)([_.default.down,_.default.move],C),S=p.extend,D=Math.floor,I=f.default.inherit({_getLayoutItems:y.noop,_useLinks:!1,_themeDependentChanges:["OPTIONS"],_initCore:function(){var e=this;e._tooltipTracker=e._renderer.root,e._tooltipTracker.attr({"pointer-events":"visible"}),e._createHtmlElements(),e._initTooltipEvents(),e._argumentAxis=l(!0),e._valueAxis=l()},_getDefaultSize:function(){return this._defaultSize},_disposeCore:function(){this._disposeWidgetElements(),this._disposeTooltipEvents(),this._ranges=null},_optionChangesOrder:["OPTIONS"],_change_OPTIONS:function(){this._prepareOptions(),this._change(["UPDATE"])},_customChangesOrder:["UPDATE"],_change_UPDATE:function(){this._update()},_update:function(){var e=this;e._tooltipShown&&(e._tooltipShown=!1,e._tooltip.hide()),e._cleanWidgetElements(),e._updateWidgetElements(),e._drawWidgetElements()},_updateWidgetElements:function(){var e=this._getCorrectCanvas();this._updateRange(),this._argumentAxis.update(this._ranges.arg,e,this._getStick()),this._valueAxis.update(this._ranges.val,e)},_getStick:function(){},_applySize:function(e){this._allOptions.size={width:e[2]-e[0],height:e[3]-e[1]},this._change(["UPDATE"])},_setupResizeHandler:y.noop,_prepareOptions:function(){return S(!0,{},this._themeManager.theme(),this.option())},_getTooltipCoords:function(){var e=this._canvas,t=this._renderer.getRootOffset();return{x:e.width/2+t.left,y:e.height/2+t.top}},_initTooltipEvents:function(){var e={widget:this};this._renderer.root.off("."+C).on(k,e,a)},_showTooltip:function(){var e,t=this;t._tooltipShown||(t._tooltipShown=!0,e=t._getTooltip(),e.isEnabled()&&t._tooltip.show(t._getTooltipData(),t._getTooltipCoords(),{}))},_hideTooltip:function(){this._tooltipShown&&(this._tooltipShown=!1,this._tooltip.hide())},_stopCurrentHandling:function(){this._hideTooltip()},_enableOutHandler:function(){var e=this;if(!e._outHandler){var t=function(t){var n=t.pageX,i=t.pageY,a=e._renderer.getRootOffset(),r=a.left,s=a.top,l=D(n-r),u=D(i-s);o(e._canvas,l,u)||(e._hideTooltip(),e._disableOutHandler())};c.default.on(d.default.getDocument(),k,t),this._outHandler=t}},_disableOutHandler:function(){this._outHandler&&c.default.off(d.default.getDocument(),k,this._outHandler),this._outHandler=null},_disposeTooltipEvents:function(){this._tooltipTracker.off(),this._disableOutHandler(),this._renderer.root.off("."+C)},_getTooltip:function(){var e=this;return e._tooltip||(u.apply(this,arguments),e._setTooltipRendererOptions(e._tooltipRendererOptions),e._tooltipRendererOptions=null,e._setTooltipOptions()),e._tooltip}}),T=I;t.default=T,I.addPlugin(b.plugin),u=I.prototype._initTooltip,I.prototype._initTooltip=y.noop;var O=I.prototype._disposeTooltip;I.prototype._disposeTooltip=function(){this._tooltip&&O.apply(this,arguments)},I.prototype._setTooltipRendererOptions=function(){var e=this._getRendererOptions();this._tooltip?this._tooltip.setRendererOptions(e):this._tooltipRendererOptions=e},I.prototype._setTooltipOptions=function(){var e=this._tooltip,t=e&&this._getOption("tooltip");e&&e.update(S({},t,{customizeTooltip:s(t.customizeTooltip,t.font,this.option("rtlEnabled")),enabled:t.enabled&&this._isTooltipEnabled()}))};var E=(0,p.extend)(!0,{},x.plugin,{init:y.noop,dispose:y.noop,customize:null,members:{_getExportMenuOptions:null}});I.addPlugin(E),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return l(e,t)}function a(e){return(0,r.default)(e,o,!1)}t.default=void 0;var r=i(n(566)),s=n(175),l=Math.max;(0,s.addAlgorithm)("squarified",a);var u=a;t.default=u,e.exports=t.default},function(e,t,n){function i(e,t){return t.value-e.value}function o(e){return c(e,1/e)}function a(e,t,n){var i,a,r,s,l,u=1/0,c=0,h=e.length,f=n.areaToValue/n.staticSide;for(r=t;r<h;){for(a=c+e[r].value,l=n.staticSide/f/a,i=0,s=t;s<=r;++s)i=n.accumulate(i,o(l*e[s].value/a),s-t+1);if(!(i<u))break;u=i,c=a,++r}return{sum:c,count:r-t,side:d(f*c)}}function r(e){return(e[2]-e[0])*(e[3]-e[1])}function s(e,t,n){var i=(0,u.buildSidesData)(n.rect,n.directions,n.staticSideIndex),o=r(n.rect),s=o>0?a(e,t,{areaToValue:o/n.sum,accumulate:n.accumulate,staticSide:i.staticSide}):{sum:1,side:i.variedSide,count:e.length-t};return(0,u.calculateRectangles)(e,t,n.rect,i,s),n.sum-=s.sum,t+s.count}function l(e,t,n){var o,a=e.items,r=a.length,l={sum:e.sum,rect:e.rect,directions:e.directions,accumulate:t};for(n&&(l.staticSideIndex=(0,u.getStaticSideIndex)(l.rect)),a.sort(i),o=0;o<r;)o=s(a,o,l)}t.default=l;var u=n(175),c=Math.max,d=Math.round;e.exports=t.default},function(e,t,n){function i(e,t,n){var i=t.createPalette(e.palette,{useHighlight:!0,extensionMode:e.paletteExtensionMode,count:e.colorizeGroups?r(n):o(n)});return(e.colorizeGroups?l:a)(i,n)}function o(e){var t,n,i=e.nodes.slice(),a=i.length,r=0;for(t=0;t<a;++t)n=i[t],n.isNode()?r=Math.max(r,o(n)):r+=1;return r}function a(e){var t=e.generateColors();return function(e){return t[e.index]}}function r(e){var t,n,i=e.nodes.slice(),o=i.length,a=0;for(t=0;t<o;++t)n=i[t],n.isNode()&&(a+=r(n)+1);return a}function s(e,t){var n,i,o={},a=t.nodes.slice(),r=a.length;for(n=0;n<r;++n)i=a[n],i.isNode()?(a=a.concat(i.nodes),r=a.length):o[i.parent._id]||(o[i.parent._id]=e.getNextColor());return o}function l(e,t){var n=s(e,t);return function(e){return n[e._id]}}t.default=void 0;var u=n(235);(0,u.addColorizer)("discrete",i);var c=i;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i,a=e.isNode()&&e.nodes,r=a&&a.length;for(e.setState(t,n),i=0;i<r;++i)o(a[i],t,n)}var a=i(n(113)),r=i(n(204)),s=n(172),l=n(331),u=n(14);n(176),n(569);var c=a.default.prototype,d=r.default.prototype,h=1;c._eventsMap.onHoverChanged={name:"hoverChanged"},(0,s.expand)(c._handlers,"calculateAdditionalStates",function(e,t){e[1]=t.hoverStyle?(0,l.buildRectAppearance)(t.hoverStyle):{}}),a.default.addChange({code:"HOVER_ENABLED",handler:function(){var e=(0,u.parseScalar)(this._getOption("hoverEnabled",!0),!0);e||this.clearHover(),this._hoverEnabled=e},isThemeDependent:!0,isOptionChange:!0,option:"hoverEnabled"}),d.statesMap[1]=1,d.additionalStates.push(1),(0,s.expand)(c,"_extendProxyType",function(e){var t=this;e.setHover=function(){t._hoverNode(this._id)},e.isHovered=function(){return t._hoverIndex===this._id}}),(0,s.expand)(c,"_onNodesCreated",function(){this._hoverIndex=-1}),(0,s.expand)(c,"_changeGroupSettings",function(){var e=this;e._groupHoverEnabled=(0,u.parseScalar)(e._getOption("group").hoverEnabled,!0),e._groupHoverEnabled||e.clearHover()}),c._applyHoverState=function(e,t){o(this._nodes[e],h,t),this._eventTrigger("hoverChanged",{node:this._nodes[e].proxy})},c._hoverNode=function(e){var t=this,n=t._hoverIndex;if(t._hoverEnabled&&n!==e){if(!t._groupHoverEnabled&&e>=0&&t._nodes[e].isNode())return void t.clearHover();t._context.suspend(),t._hoverIndex=-1,n>=0&&t._applyHoverState(n,!1),t._hoverIndex=e,e>=0&&t._applyHoverState(e,!0),t._context.resume()}},c.clearHover=function(){this._hoverNode(-1)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){var o,a=i.length;for(o=0;o<a;++o)e[i[o]]=(0,l.extend)({},t,n[i[o]])}var a=i(n(113)),r=i(n(204)),s=n(4),l=n(0),u=a.default.prototype,c=r.default.prototype,d=u._handlers,h=d.calculateState,f=c._buildState;d.calculateState=function(e){var t={0:h(e)};return d.calculateAdditionalStates(t,e),t},d.calculateAdditionalStates=s.noop,c.code=0,c.statesMap={0:0},c.additionalStates=[],c._buildState=function(e,t){var n={0:f(e[0],t)};return this.additionalStates.length&&o(n,n[0],e,this.additionalStates),n},c._getState=function(){return this.state[this.statesMap[this.code]]},c.setState=function(e,t){t?this.code|=e:this.code&=~e,this.ctx.change(["TILES"])}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=t.getRootOffset();return[(e[0]+e[2])/2+n.left,(e[1]+e[3])/2+n.top]}var a=n(172),r=i(n(113));n(176);var s=n(133),l=r.default.prototype;(0,a.expand)(l,"_extendProxyType",function(e){var t=this;e.showTooltip=function(e){t._showTooltip(this._id,e)}}),(0,a.expand)(l,"_onNodesCreated",function(){this._tooltipIndex>=0&&this._tooltip.hide(),this._tooltipIndex=-1}),(0,a.expand)(l,"_onTilingPerformed",function(){this._tooltipIndex>=0&&this._moveTooltip(this._nodes[this._tooltipIndex])}),l._showTooltip=function(e,t){var n=this,i=n._tooltip,o=n._nodes[e];if(n._tooltipIndex===e)return void n._moveTooltip(o,t);var a=function(a){void 0!==a&&(a?n._moveTooltip(o,t):i.hide(),n._tooltipIndex=a?e:-1)};a(i.show({value:o.value,valueText:i.formatValue(o.value),node:o.proxy},{x:0,y:0,offset:0},{node:o.proxy},void 0,a))},l._moveTooltip=function(e,t){var n=t||e.rect&&o(e.rect,this._renderer)||[-1e3,-1e3];this._tooltip.move(n[0],n[1],0)},l.hideTooltip=function(){this._tooltipIndex>=0&&(this._tooltipIndex=-1,this._tooltip.hide())},r.default.addPlugin(s.plugin)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e.map(function(e,t){return t%2?1-e:e})}function a(e){return{fill:e.fill,hatching:e.hatching}}t.default=void 0;var r=n(990),s=i(n(991)),l=i(n(992)),u=n(4),c=i(n(112)),d=i(n(9)),h=i(n(993)),f=n(174),p="NODES_CREATE";(0,r.addAlgorithm)("dynamicslope",s.default,!0),(0,r.addAlgorithm)("dynamicheight",l.default);var g=c.default.inherit({_rootClass:"dxf-funnel",_rootClassPrefix:"dxf",_proxyData:[],_optionChangesMap:{dataSource:"DATA_SOURCE",neckWidth:p,neckHeight:p,inverted:p,algorithm:p,item:p,valueField:p,argumentField:p,colorField:p,palette:p,paletteExtensionMode:p,sortData:p},_themeDependentChanges:[p],_getDefaultSize:function(){return{width:400,height:400}},_themeSection:"funnel",_fontFields:["legend.title.font","legend.title.subtitle.font","legend.font"],_optionChangesOrder:["DATA_SOURCE"],_initialChanges:["DATA_SOURCE"],_initCore:function(){this._group=this._renderer.g().append(this._renderer.root),this._items=[]},_eventsMap:{onHoverChanged:{name:"hoverChanged"},onSelectionChanged:{name:"selectionChanged"}},_disposeCore:u.noop,_applySize:function(e){return this._rect=e.slice(),this._change(["TILING"]),this._rect},_getAlignmentRect:function(){return this._rect},_change_TILING:function(){var e=this,t=e._items,n=e._rect,i=function(e,t){var i=t%2;return n[0+i]+(n[2+i]-n[0+i])*e};this._group.clear(),t.forEach(function(t,n){var o=t.figure.map(i),a=e._renderer.path([],"area").attr({points:o}).append(e._group);t.coords=o,t.element=a}),this._requestChange(["TILES"])},_customChangesOrder:[p,"LAYOUT","TILING","TILES","DRAWN"],_dataSourceChangedHandler:function(){this._requestChange([p])},_change_DRAWN:function(){this._drawn()},_change_DATA_SOURCE:function(){this._change(["DRAWN"]),this._updateDataSource()},_change_NODES_CREATE:function(){this._buildNodes()},_change_TILES:function(){this._applyTilesAppearance()},_suspend:function(){this._applyingChanges||this._suspendChanges()},_resume:function(){this._applyingChanges||this._resumeChanges()},_applyTilesAppearance:function(){this._items.forEach(function(e){var t=e.getState();e.element.smartAttr(e.states[t])})},_hitTestTargets:function(e,t){var n,i=this;return this._proxyData.some(function(o){if(n=o.call(i,e,t))return!0}),n},clearHover:function(){this._suspend(),this._items.forEach(function(e){e.isHovered()&&e.hover(!1)}),this._resume()},clearSelection:function(){this._suspend(),this._items.forEach(function(e){e.isSelected()&&e.select(!1)}),this._resume()},_getData:function(){var e=this,t=e._dataSourceItems()||[],n=e._getOption("valueField",!0),i=e._getOption("argumentField",!0),o=e._getOption("colorField",!0),a=t.reduce(function(e,t){var a=Number(t[n]);return a>=0&&(e[0].push({value:a,color:t[o],argument:t[i],dataItem:t}),e[1]+=a),e},[[],0]),r=a[0];return a[1]||(r=r.map(function(e){return e.value+=1,e})),t.length>0&&0===r.length&&e._incidentOccurred("E2005",n),e._getOption("sortData",!0)&&r.sort(function(e,t){return t.value-e.value}),r},_buildNodes:function(){var e=this,t=e._getData(),n=(0,r.getAlgorithm)(e._getOption("algorithm",!0)),i=n.normalizeValues(t),a=e._getOption("item"),s=n.getFigures(i,e._getOption("neckWidth",!0),e._getOption("neckHeight",!0)),l=e._themeManager.createPalette(e._getOption("palette",!0),{useHighlight:!0,extensionMode:e._getOption("paletteExtensionMode",!0),count:s.length});e._items=s.map(function(n,o){var r=t[o],s=new h.default(e,{figure:n,data:r,percent:i[o],id:o,color:r.color||l.getNextColor(),itemOptions:a});return s}),e._getOption("inverted",!0)&&e._items.forEach(function(e){e.figure=o(e.figure)}),e._renderer.initHatching(),e._change(["TILING","DRAWN"])},_showTooltip:u.noop,hideTooltip:u.noop,getAllItems:function(){return this._items.slice()},_getLegendData:function(){return this._items.map(function(e){var t=e.states;return{id:e.id,visible:!0,text:e.argument,item:e,states:{normal:a(t.normal),hover:a(t.hover),selection:a(t.selection)}}})},_getMinSize:function(){var e=this._getOption("adaptiveLayout");return[e.width,e.height]}});(0,d.default)("dxFunnel",g);var _=g;t.default=_,g.addPlugin(f.plugin),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){e._label.getBBox().width>n&&e.labelText.applyEllipsis(n);var o=e._label.getBBox(),a=t.verticalOffset,r=t.horizontalOffset,s=Math.round(e.rect.y+e.rect.height/2-o.y-o.height/2)+a,l=e.rect.x+r+e.rect.width-o.x;l+o.width>=i[2]-i[0]&&(l=e.rect.x-r-o.x-o.width),s>=i[3]&&(s=i[3]),s-o.height<i[1]&&(s=e.rect.y-o.y+a),e.labelText.attr({translateX:l,translateY:s})}function a(e,t,n){var i=[],o="in"===n?"_to":"_from",a="in"===n?"_from":"_to";
return e.links.map(function(e){return e[o]._name===t}).forEach(function(t,n){t&&i.push({index:n,weight:e.links[n]._weight,node:e.links[n][a]._name})}),i}t.default=void 0;var r=n(573),s=n(4),l=i(n(998)),u=i(n(999)),c=i(n(1e3)),d=n(1),h=i(n(112)),f=i(n(9)),p=n(174),g=h.default.inherit({_rootClass:"dxs-sankey",_rootClassPrefix:"dxs",_proxyData:[],_optionChangesMap:{dataSource:"DATA_SOURCE",sortData:"DATA_SOURCE",alignment:"DATA_SOURCE",node:"BUILD_LAYOUT",link:"BUILD_LAYOUT",palette:"BUILD_LAYOUT",paletteExtensionMode:"BUILD_LAYOUT"},_themeDependentChanges:["BUILD_LAYOUT"],_getDefaultSize:function(){return{width:400,height:400}},_themeSection:"sankey",_fontFields:["label.font"],_optionChangesOrder:["DATA_SOURCE"],_initialChanges:["DATA_SOURCE"],_initCore:function(){this._groupLinks=this._renderer.g().append(this._renderer.root),this._groupNodes=this._renderer.g().append(this._renderer.root),this._groupLabels=this._renderer.g().attr({"class":this._rootClassPrefix+"-labels"}).append(this._renderer.root),this._drawLabels=!0,this._nodes=[],this._links=[],this._gradients=[]},_disposeCore:s.noop,_applySize:function(e){this._rect=e.slice();var t=this._getOption("adaptiveLayout");return t.keepLabels||this._rect[2]-this._rect[0]>t.width?this._drawLabels=!0:this._drawLabels=!1,this._change(["BUILD_LAYOUT"]),this._rect},_eventsMap:{onNodeHoverChanged:{name:"nodeHoverChanged"},onLinkHoverChanged:{name:"linkHoverChanged"}},_customChangesOrder:["BUILD_LAYOUT","NODES_DRAW","LINKS_DRAW","LABELS","DRAWN"],_dataSourceChangedHandler:function(){this._requestChange(["BUILD_LAYOUT"])},_change_DRAWN:function(){this._drawn()},_change_DATA_SOURCE:function(){this._change(["DRAWN"]),this._updateDataSource()},_change_LABELS:function(){this._applyLabelsAppearance()},_change_BUILD_LAYOUT:function(){this._groupNodes.clear(),this._groupLinks.clear(),this._groupLabels.clear(),this._buildLayout()},_change_NODES_DRAW:function(){var e=this,t=e._nodes;t.forEach(function(t,n){var i=e._renderer.rect().attr(t.rect).append(e._groupNodes);t.element=i}),this._applyNodesAppearance()},_change_LINKS_DRAW:function(){var e=this,t=e._links;t.forEach(function(t,n){var i=e._renderer.g().attr({"class":"link","data-link-idx":n}).append(e._groupLinks);t.overlayElement=e._renderer.path([],"area").attr({d:t.d}).append(i),t.element=e._renderer.path([],"area").attr({d:t.d}).append(i)}),this._applyLinksAppearance()},_suspend:function(){this._applyingChanges||this._suspendChanges()},_resume:function(){this._applyingChanges||this._resumeChanges()},_showTooltip:s.noop,hideTooltip:s.noop,clearHover:function(){this._suspend(),this._nodes.forEach(function(e){e.isHovered()&&e.hover(!1)}),this._links.forEach(function(e){e.isHovered()&&e.hover(!1),e.isAdjacentNodeHovered()&&e.adjacentNodeHover(!1)}),this._resume()},_applyNodesAppearance:function(){this._nodes.forEach(function(e){var t=e.getState();e.element.smartAttr(e.states[t])})},_applyLinksAppearance:function(){this._links.forEach(function(e){var t=e.getState();e.element.smartAttr(e.states[t]),e.overlayElement.smartAttr(e.overlayStates[t])})},_hitTestTargets:function(e,t){var n,i=this;return this._proxyData.some(function(o){if(n=o.call(i,e,t))return!0}),n},_getData:function(){var e=this,t=e._dataSourceItems()||[],n=e._getOption("sourceField",!0),i=e._getOption("targetField",!0),o=e._getOption("weightField",!0),a=[];return t.forEach(function(t){var r=Object.prototype.hasOwnProperty.bind(t);r(n)?r(i)?r(o)?(0,d.isString)(t[n])?(0,d.isString)(t[i])?!(0,d.isNumeric)(t[o])||t[o]<=0?e._incidentOccurred("E2009",o):a.push([t[n],t[i],t[o]]):e._incidentOccurred("E2008",i):e._incidentOccurred("E2008",n):e._incidentOccurred("E2007",o):e._incidentOccurred("E2007",i):e._incidentOccurred("E2007",n)}),a},_buildLayout:function(){var e=this,t=this,n=t._getData(),i=this._rect,o=t._getOption("node"),s=t._getOption("sortData"),d=t._getOption("layoutBuilder",!0)||c.default,h={x:i[0],y:i[1],width:i[2]-i[0],height:i[3]-i[1]},f=d.computeLayout(n,s,{availableRect:h,nodePadding:o.padding,nodeWidth:o.width,nodeAlign:t._getOption("alignment",!0)},t._incidentOccurred);if(t._layoutMap=f,!Object.prototype.hasOwnProperty.call(f,"error")){var p={},g=0,_=t._getOption("link"),m=f.nodes.map(function(e){return e.length}).reduce(function(e,t){return e+t},0),v=t._themeManager.createPalette(t._getOption("palette",!0),{useHighlight:!0,extensionMode:t._getOption("paletteExtensionMode",!0),count:m});t._nodes=[],t._links=[],t._gradients.forEach(function(e){e.dispose()}),t._gradients=[],t._shadowFilter&&t._shadowFilter.dispose(),f.nodes.forEach(function(e){e.forEach(function(e){var n=o.color||v.getNextColor(),i=new l.default(t,{id:g,color:n,rect:e,options:o,linksIn:a(f,e._name,"in"),linksOut:a(f,e._name,"out")});t._nodes.push(i),g++,p[e._name]=n})}),f.links.forEach(function(n){var i=null;_.colorMode===r.COLOR_MODE_GRADIENT&&(i=t._renderer.linearGradient([{offset:"0%","stop-color":p[n._from._name]},{offset:"100%","stop-color":p[n._to._name]}]),e._gradients.push(i));var o=_.color;_.colorMode===r.COLOR_MODE_SOURCE?o=p[n._from._name]:_.colorMode===r.COLOR_MODE_TARGET&&(o=p[n._to._name]);var a=new u.default(t,{d:n.d,boundingRect:n._boundingRect,color:o,options:_,connection:{source:n._from._name,target:n._to._name,weight:n._weight},gradient:i});t._links.push(a)}),t._renderer.initHatching(),t._change(["NODES_DRAW","LINKS_DRAW","LABELS"])}t._change(["DRAWN"])},_applyLabelsAppearance:function(){var e=this,t=e._getOption("label"),n=e._rect[2]-e._rect[0],i=e._getOption("node");if(e._shadowFilter=e._renderer.shadowFilter("-50%","-50%","200%","200%").attr(t.shadow),e._groupLabels.clear(),e._drawLabels&&t.visible){var a=(n-(i.width+t.horizontalOffset)-e._layoutMap.cascades.length*i.width)/(e._layoutMap.cascades.length-1)-t.horizontalOffset;e._nodes.forEach(function(n){e._createLabel(n,t,e._shadowFilter.id),o(n,t,a,e._rect)}),"none"!==t.overlappingBehavior&&e._nodes.forEach(function(n){var i=n._label.getBBox();e._nodes.forEach(function(e){var o=e._label.getBBox();n.id!==e.id&&c.default.overlap(i,o)&&("ellipsis"===t.overlappingBehavior?n.labelText.applyEllipsis(o.x-i.x):"hide"===t.overlappingBehavior&&n.labelText.remove())})})}},_createLabel:function(e,t,n){var i=t.customizeText(e),o=e.getLabelAttributes(t,n);i&&(e._label=this._renderer.g().append(this._groupLabels),e.labelText=this._renderer.text(i).attr(o.attr).css(o.css),e.labelText.append(e._label))},_getMinSize:function(){var e=this._getOption("adaptiveLayout");return[e.width,e.height]},getAllNodes:function(){return this._nodes.slice()},getAllLinks:function(){return this._links.slice()}});(0,f.default)("dxSankey",g);var _=g;t.default=_,g.addPlugin(p.plugin),e.exports=t.default},function(e,t,n){t.COLOR_MODE_NONE=t.COLOR_MODE_TARGET=t.COLOR_MODE_SOURCE=t.COLOR_MODE_GRADIENT=void 0;var i="gradient";t.COLOR_MODE_GRADIENT=i;var o="source";t.COLOR_MODE_SOURCE=o;var a="target";t.COLOR_MODE_TARGET=a;var r="none";t.COLOR_MODE_NONE=r},function(e,t,n){t.default=void 0;var i="white",o="gray",a="black",r={maxOfArray:function(e,t){var n=0,i=function(e){return e};t&&(i=t);for(var o=0;o<e.length;o++)i(e[o])>n&&(n=i(e[o]));return n}},s=function(e){var t=[];return e.forEach(function(e){t.indexOf(e[0])===-1&&t.push(e[0]),t.indexOf(e[1])===-1&&t.push(e[1])}),t},l=function(e,t){var n=[];return e.forEach(function(e){e[0]===t&&n.indexOf(e[1])===-1&&n.push(e[1])}),n},u=function(e,t){var n=[];return e.forEach(function(e){e[1]===t&&n.indexOf(e[0])===-1&&n.push(e[0])}),n},c={_hasCycle:!1,_sortedList:[],hasCycle:function(e){var t=this;this._hasCycle=!1,this._sortedList=[];var n={},o=s(e);return o.forEach(function(e){n[e]={color:i}}),o.forEach(function(o){n[o].color===i&&t._depthFirstSearch(e,n,o)}),this._sortedList.reverse(),this._hasCycle},_depthFirstSearch:function(e,t,n){t[n].color=o;for(var r=l(e,n),s=0;s<r.length;s++)t[r[s]].color===i?this._depthFirstSearch(e,t,r[s]):t[r[s]].color===o&&(this._hasCycle=!0);this._sortedList.push({name:n,lp:null,incoming:u(e,n),outgoing:l(e,n)}),t[n].color=a},computeLongestPaths:function(e){var t=this._sortedList;return t.forEach(function(n){var i=u(e,n.name);if(0===i.length)n.lp=0;else{var o=[];i.forEach(function(e){o.push(t.filter(function(t){return t.name===e})[0].lp)}),n.lp=r.maxOfArray(o)+1}}),this._sortedList}},d={struct:c,routines:r,getVertices:s,getAdjacentVertices:l,getReverseAdjacentVertices:u};t.default=d,e.exports=t.default},function(e,t,n){var i=n(180);n(262),n(576);var o=i.ui=n(578);o.themes=n(36),o.setTemplateEngine=n(162).setTemplateEngine,o.dialog=n(280),o.notify=n(419),o.repaintFloatingActionButton=n(580),o.dxActionSheet=n(281),o.dxAutocomplete=n(582),o.dxBox=n(216),o.dxButton=n(38),o.dxDropDownButton=n(433),o.dxButtonGroup=n(289),o.dxCalendar=n(290),o.dxCheckBox=n(144),o.dxColorBox=n(434),o.dxDateBox=n(217),o.dxDrawer=n(632),o.dxDeferRendering=n(636),o.dxDropDownBox=n(637),o.dxDropDownMenu=n(219),o.dxFileUploader=n(294),o.dxForm=n(146),o.dxGallery=n(645),o.dxHtmlEditor=n(646),o.dxList=n(115),o.dxLoadIndicator=n(88),o.dxLoadPanel=n(193),o.dxLookup=n(669),o.dxMap=n(670),o.dxMultiView=n(438),o.dxNavBar=n(674),o.dxNumberBox=n(126),o.dxOverlay=n(68),o.dxPopover=n(143),o.dxPopup=n(48),o.dxProgressBar=n(295),o.dxRadioGroup=n(450),o.dxRangeSlider=n(677),o.dxResizable=n(188),o.dxResponsiveBox=n(442),o.dxScrollView=n(96),o.dxSelectBox=n(127),o.dxSlider=n(451),o.dxSpeedDialAction=n(679),o.dxSwitch=n(454),o.dxTabPanel=n(297),o.dxTabs=n(220),o.dxTagBox=n(301),o.dxTextArea=n(455),o.dxTextBox=n(104),o.dxTileView=n(682),o.dxToast=n(420),o.dxToolbar=n(166),o.dxTooltip=n(195),o.dxTrackBar=n(296),o.dxDraggable=n(215),o.dxSortable=n(286),i.validationEngine=n(100),o.dxValidationSummary=n(298),o.dxValidationGroup=n(444),o.dxValidator=n(221),n(683),o.CollectionWidget=n(58),o.dxDropDownEditor=n(192),e.exports=o},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(180)),a=n(577);o.default.fileManagement=a,e.exports=a},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(137)),a=i(n(277)),r=i(n(415)),s=i(n(416)),l=i(n(417));e.exports=o.default.fileManagement=o.default.fileManagement||{},o.default.fileManagement.FileSystemItem=a.default,o.default.fileManagement.ObjectFileSystemProvider=r.default,o.default.fileManagement.RemoteFileSystemProvider=s.default,o.default.fileManagement.CustomFileSystemProvider=l.default},function(e,t,n){n(137),e.exports=DevExpress.ui={}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(58)),a=n(6),r=n(4),s=o.default.inherit({_initMarkup:function(){this._deferredItems=[],this.callBase()},_renderItemContent:function(e){var t=new a.Deferred,n=new a.Deferred,i=this;this._deferredItems[e.index]=n;var o=this.callBase.call(i,e);return n.done(function(){t.resolve(o)}),t.promise()},_createItemByTemplate:function(e,t){var n=this;return e.render({model:t.itemData,container:t.container,index:t.index,onRendered:function(){n._deferredItems[t.index].resolve()}})},_postProcessRenderItems:r.noop,_renderItemsAsync:function(){var e=this,t=new a.Deferred;return a.when.apply(this,this._deferredItems).done(function(){e._postProcessRenderItems(),t.resolve()}),t.promise()},_clean:function(){this.callBase(),this._deferredItems=[]}}),l=s;t.default=l,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(421),o=i.repaint;t.default=o,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(5)),b=n(8),x=n(21),w=n(59),C=o(n(68)),k=n(87),S=n(36),D="dx-fa-button",I="dx-fa-button-icon",T="dx-fa-button-label",O="dx-fa-button-label-wrapper",E="dx-fa-button-content-reverse",A=".dx-overlay-content",P=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{shading:!1,useInkRipple:!1,callOverlayRenderShading:!1,width:"auto",zIndex:1500})}},{key:"_defaultOptionsRules",value:function(){return l(_(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return(0,S.isMaterial)()},options:{useInkRipple:!0}}])}},{key:"_render",value:function(){this.$element().addClass(D),this._renderIcon(),this._renderLabel(),l(_(t.prototype),"_render",this).call(this),this.option("useInkRipple")&&this._renderInkRipple(),this._renderClick()}},{key:"_renderLabel",value:function(){!!this._$label&&this._$label.remove();var e=this.option("label");if(!e)return void(this._$label=null);var t=(0,m.default)("<div>").addClass(T),n=(0,m.default)("<div>").addClass(O);this._$label=n.prependTo(this.$content()).append(t.text(e)),this.$content().toggleClass(E,this._isPositionLeft(this.option("parentPosition")))}},{key:"_isPositionLeft",value:function(e){var t=e?e.at?e.at.x?e.at.x:e.at:"string"==typeof e?e:"":"";return"left"===t.split(" ")[0]}},{key:"_renderButtonIcon",value:function(e,t,n){!!e&&e.remove(),e=(0,m.default)("<div>").addClass(n);var i=(0,w.getImageContainer)(t);return e.append(i).appendTo(this.$content()),e}},{key:"_renderIcon",value:function(){this._$icon=this._renderButtonIcon(this._$icon,this._options.silent("icon"),I)}},{key:"_renderWrapper",value:function(){this._options.silent("callOverlayRenderShading")&&l(_(t.prototype),"_renderWrapper",this).call(this)}},{key:"_getVisibleActions",value:function(e){var t=e||this.option("actions")||[];return t.filter(function(e){return e.option("visible")})}},{key:"_getActionComponent",value:function(){return 1===this._getVisibleActions().length?this._getVisibleActions()[0]:this.option("actionComponent")||this.option("actions")[0]}},{key:"_initContentReadyAction",value:function(){this._contentReadyAction=this._getActionComponent()._createActionByOption("onContentReady",{excludeValidators:["disabled","readOnly"]},!0)}},{key:"_fireContentReadyAction",value:function(){this._contentReadyAction({actionElement:this.$element()})}},{key:"_updateZIndexStackPosition",value:function(){var e=this.option("zIndex");this._$wrapper.css("zIndex",e),this._$content.css("zIndex",e)}},{key:"_fixWrapperPosition",value:function(){var e=this._$wrapper,t=this._getContainer();e.css("position",this._isWindow(t)?"fixed":"absolute")}},{key:"_setClickAction",value:function(){var e=this,t=(0,b.addNamespace)(x.name,this.NAME),n=this.$element().find(A);y.default.off(n,t),y.default.on(n,t,function(t){var n={event:t,actionElement:e.element(),element:e._getActionComponent().$element()};e._clickAction(n)})}},{key:"_defaultActionArgs",value:function(){return{component:this._getActionComponent()}}},{key:"_renderClick",value:function(){this._clickAction=this._getActionComponent()._createActionByOption("onClick"),this._setClickAction()}},{key:"_renderInkRipple",value:function(){this._inkRipple=(0,k.render)()}},{key:"_getInkRippleContainer",value:function(){return this._$icon}},{key:"_toggleActiveState",value:function(e,n,i){if(l(_(t.prototype),"_toggleActiveState",this).apply(this,arguments),this._inkRipple){var o={element:this._getInkRippleContainer(),event:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}}},{key:"_optionChanged",value:function(e){switch(e.name){case"icon":this._renderIcon();break;case"onClick":this._renderClick();break;case"label":this._renderLabel();break;case"visible":this._currentVisible=e.previousValue,e.value?this._show():this._hide();break;case"useInkRipple":this._render();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(C.default),R=P;t.default=R,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(9)),s=n(0),l=i(n(282)),u=n(6),c=n(8),d="dx-autocomplete",h="dx-autocomplete-popup-wrapper",f=l.default.inherit({_supportedKeys:function(){var e=this._list?this._list.option("focusedElement"):null,t=this.callBase();return e=e&&(0,o.default)(e),(0,s.extend)({},t,{upArrow:function(t){return!(!(0,c.isCommandKeyPressed)(t)&&(t.preventDefault(),t.stopPropagation(),e&&!this._calcNextItem(-1)))||(this._clearFocusedItem(),!1)},downArrow:function(t){return!(!(0,c.isCommandKeyPressed)(t)&&(t.preventDefault(),t.stopPropagation(),e&&!this._calcNextItem(1)))||(this._clearFocusedItem(),!1)},enter:function(){return e||this.close(),t.enter.apply(this,arguments),this.option("opened")}})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{minSearchLength:1,maxItemCount:10,noDataText:"",showDropDownButton:!1,searchEnabled:!0})},_initMarkup:function(){this.callBase(),this.$element().addClass(d),this.setAria("autocomplete","inline")},_displayGetterExpr:function(){return this.option("valueExpr")},_popupConfig:function(){return(0,s.extend)(this.callBase(),{closeOnOutsideClick:function(e){return!(0,o.default)(e.target).closest(this.$element()).length}.bind(this)})},_renderDimensions:function(){this.callBase(),this._dimensionChanged()},_popupWrapperClass:function(){return this.callBase()+" "+h},_listConfig:function(){return(0,s.extend)(this.callBase(),{pageLoadMode:"none"})},_listItemClickHandler:function(e){this._saveValueChangeEvent(e.event);var t=this._displayGetter(e.itemData);this.option("value",t),this.close()},_setListDataSource:function(){this._list&&(this._list.option("selectedItems",[]),this.callBase())},_refreshSelected:a.noop,_searchCanceled:function(){this.callBase(),this.close()},_loadItem:function(e,t){var n=this._getItemFromPlain(e,t);return(new u.Deferred).resolve(n).promise()},_dataSourceOptions:function(){return{paginate:!0,pageSize:this.option("maxItemCount")}},_searchDataSource:function(){this._dataSource.pageSize(this.option("maxItemCount")),this.callBase(),this._clearFocusedItem()},_clearFocusedItem:function(){this._list&&(this._list.option("focusedElement",null),this._list.option("selectedIndex",-1))},_renderValueEventName:function(){return"input keyup"},_valueChangeEventHandler:function(e){var t=this._input().val()||null;return this.callBase(e,t)},_optionChanged:function(e){switch(e.name){case"maxItemCount":this._searchDataSource();break;case"valueExpr":this._compileDisplayGetter(),this._setListOption("displayExpr",this._displayGetterExpr()),this.callBase(e);break;default:this.callBase(e)}},reset:function(){this.callBase(),this.close()}});(0,r.default)("dxAutocomplete",f);var p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(5)),b=o(n(10)),x=o(n(212)),w=o(n(38)),C="dx-dropdowneditor-button",k="dx-dropdowneditor-button-visible",S="dxDropDownEditor-selectLabel",D=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_attachEvents",value:function(e){var t=this.editor;e.option("onClick",function(e){!t.option("openOnFieldClick")&&t._openHandler(e)}),y.default.on(e.$element(),"mousedown",function(e){t.$element().is(".dx-state-focused")&&e.preventDefault()})}},{key:"_create",value:function(){var e=this.editor,t=(0,m.default)("<div>"),n=this._getOptions();this._addToContainer(t);var i=e._createComponent(t,w.default,(0,v.extend)({},n,{elementAttr:{"aria-label":b.default.format(S)}}));return this._legacyRender(e.$element(),t,n.visible),{$element:t,instance:i}}},{key:"_getOptions",value:function(){var e=this.editor,t=this._isVisible(),n=e.option("readOnly"),i=e._getTemplateByOption("dropDownButtonTemplate");return{focusStateEnabled:!1,hoverStateEnabled:!1,activeStateEnabled:!1,useInkRipple:!1,disabled:n,visible:t,template:i}}},{key:"_isVisible",value:function(){var e=this.editor;return l(_(t.prototype),"_isVisible",this).call(this)&&e.option("showDropDownButton")}},{key:"_legacyRender",value:function(e,t,n){e.toggleClass(k,n),t&&(t.removeClass("dx-button"),t.addClass(C))}},{key:"update",value:function(){var e=l(_(t.prototype),"update",this).call(this);if(e){var n=this.editor,i=this.instance,o=n.$element(),a=this._getOptions();i&&i.option(a),this._legacyRender(o,i&&i.$element(),a.visible)}}}]),t}(x.default);t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(23)),s=i(n(5)),l=i(n(13)),u=n(12),c=n(0),d=i(n(9)),h=i(n(422)),f=n(8),p=(0,a.getWindow)(),g=(0,a.getNavigator)(),_=g.userAgent,m=["backspace","tab","enter","pageUp","pageDown","end","home","leftArrow","rightArrow","downArrow","upArrow","del"],v="dx-textbox",y="dx-searchbox",b="dx-icon",x="dx-icon-search",w=h.default.inherit({ctor:function(e,t){t&&(this._showClearButton=t.showClearButton),this.callBase.apply(this,arguments)},_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{value:"",mode:"text",maxLength:null})},_initMarkup:function(){this.$element().addClass(v),this.callBase(),this.setAria("role","textbox")},_renderContentImpl:function(){this._renderMaxLengthHandlers(),this.callBase()},_renderInputType:function(){this.callBase(),this._renderSearchMode()},_renderMaxLengthHandlers:function(){this._isAndroidOrIE()&&(s.default.on(this._input(),(0,f.addNamespace)("keydown",this.NAME),this._onKeyDownCutOffHandler.bind(this)),s.default.on(this._input(),(0,f.addNamespace)("change",this.NAME),this._onChangeCutOffHandler.bind(this)))},_renderProps:function(){this.callBase(),this._toggleMaxLengthProp()},_toggleMaxLengthProp:function(){var e=this._getMaxLength();e&&e>0?this._input().attr("maxLength",e):this._input().removeAttr("maxLength")},_renderSearchMode:function(){var e=this._$element;"search"===this.option("mode")?(e.addClass(y),this._renderSearchIcon(),void 0===this._showClearButton&&(this._showClearButton=this.option("showClearButton"),this.option("showClearButton",!0))):(e.removeClass(y),this._$searchIcon&&this._$searchIcon.remove(),this.option("showClearButton",void 0===this._showClearButton?this.option("showClearButton"):this._showClearButton),delete this._showClearButton)},_renderSearchIcon:function(){var e=(0,o.default)("<div>").addClass(b).addClass(x);e.prependTo(this._input().parent()),this._$searchIcon=e},_optionChanged:function(e){switch(e.name){case"maxLength":this._toggleMaxLengthProp(),this._renderMaxLengthHandlers();break;case"mask":this.callBase(e),this._toggleMaxLengthProp();break;default:this.callBase(e)}},_onKeyDownCutOffHandler:function(e){var t=this._getMaxLength();if(!t||e.ctrlKey||this._hasSelection())return!0;var n=(0,o.default)(e.target),i=(0,f.normalizeKeyName)(e);return this._cutOffExtraChar(n),n.val().length<t||(0,u.inArray)(i,m)!==-1||""!==p.getSelection().toString()},_onChangeCutOffHandler:function(e){var t=(0,o.default)(e.target);this.option("maxLength")&&this._cutOffExtraChar(t)},_cutOffExtraChar:function(e){var t=this._getMaxLength(),n=e.val();t&&n.length>t&&e.val(n.substr(0,t))},_getMaxLength:function(){var e=!!this.option("mask");return e?null:this.option("maxLength")},_isAndroidOrIE:function(){var e=l.default.real(),t=e.version.join(".");return r.default.msie||"android"===e.platform&&t&&/^(2\.|4\.1)/.test(t)&&!/chrome/i.test(_)}});(0,d.default)("dxTextBox",w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return/[0-9]/.test(e)}function a(e){var t=e.charCodeAt();return 64<t&&t<91||96<t&&t<123||t>127}function r(e){return" "===e}t.default=void 0;var s=i(n(2)),l=i(n(423)),u=n(586),c=n(3),d=n(8),h=i(n(5)),f=n(0),p=n(75),g=n(1),_=i(n(10)),m=n(4),v=n(42),y=n(139),b=n(587),x=i(n(588)),w=i(n(592)),C=i(n(593)),k=function(){return{}},S=l.default,D=" ",I="\\",T="dx-texteditor-masked",O="forward",E="backward",A={0:/[0-9]/,9:/[0-9\s]/,"#":/[-+0-9\s]/,L:function(e){return a(e)},l:function(e){return a(e)||r(e)},C:/\S/,c:/./,A:function(e){return a(e)||o(e)},a:function(e){return a(e)||o(e)||r(e)}},P=x.default.inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{mask:"",maskChar:"_",maskRules:{},maskInvalidMessage:_.default.format("validation-mask"),useMaskedValue:!1,showMaskMode:"always"})},_supportedKeys:function(){var e=this,t={backspace:e._maskStrategy.getHandler("backspace"),del:e._maskStrategy.getHandler("del"),enter:e._changeHandler},n=e.callBase();return(0,c.each)(t,function(t,i){var o=n[t];n[t]=function(t){e.option("mask")&&i.call(e,t),o&&o(t)}}),n},_getSubmitElement:function(){return this.option("mask")?this._$hiddenElement:this.callBase()},_init:function(){this.callBase(),this._initMaskStrategy()},_initMaskStrategy:function(){this._maskStrategy=(0,u.isInputEventsL2Supported)()?new C.default(this):new w.default(this)},_initMarkup:function(){this._renderHiddenElement(),this.callBase()},_attachMouseWheelEventHandlers:function(){var e=this._onMouseWheel!==m.noop;if(e){var t=this._input(),n=(0,d.addNamespace)(y.name,this.NAME),i=this._createAction(function(e){if((0,p.focused)(t)){var n=e.event;this._onMouseWheel(n),n.preventDefault(),n.stopPropagation()}}.bind(this));h.default.off(t,n),h.default.on(t,n,function(e){i({event:e})})}},_onMouseWheel:m.noop,_render:function(){this._renderMask(),this.callBase(),this._attachMouseWheelEventHandlers()},_renderHiddenElement:function(){this.option("mask")&&(this._$hiddenElement=(0,s.default)("<input>").attr("type","hidden").appendTo(this._inputWrapper()))},_removeHiddenElement:function(){this._$hiddenElement&&this._$hiddenElement.remove()},_renderMask:function(){this.$element().removeClass(T),this._maskRulesChain=null,this._maskStrategy.detachEvents(),this.option("mask")&&(this.$element().addClass(T),this._maskStrategy.attachEvents(),this._parseMask(),this._renderMaskedValue())},_suppressCaretChanging:function(e,t){S=k;try{e.apply(this,t)}finally{S=l.default}},_changeHandler:function(e){var t=this._input(),n=t.val();if(n!==this._changedValue){this._changedValue=n;var i=(0,d.createEvent)(e,{type:"change"});h.default.trigger(t,i)}},_parseMask:function(){this._maskRules=(0,f.extend)({},A,this.option("maskRules")),this._maskRulesChain=this._parseMaskRule(0)},_parseMaskRule:function(e){var t=this.option("mask");if(e>=t.length)return new b.EmptyMaskRule;var n=t[e],i=n===I,o=i?new b.StubMaskRule({maskChar:t[e+1]}):this._getMaskRule(n);return o.next(this._parseMaskRule(e+1+i)),o},_getMaskRule:function(e){var t;return(0,c.each)(this._maskRules,function(n,i){if(n===e)return t={pattern:n,allowedChars:i},!1}),(0,g.isDefined)(t)?new b.MaskRule((0,f.extend)({maskChar:this.option("maskChar")},t)):new b.StubMaskRule({maskChar:e})},_renderMaskedValue:function(){if(this._maskRulesChain){var e=this.option("value")||"";this._maskRulesChain.clear(this._normalizeChainArguments());var t={length:e.length};t[this._isMaskedValueMode()?"text":"value"]=e,this._handleChain(t),this._displayMask()}},_replaceSelectedText:function(e,t,n){if(void 0===n)return e;var i=e.slice(0,t.start),o=e.slice(t.end),a=i+n+o;return a},_isMaskedValueMode:function(){return this.option("useMaskedValue")},_displayMask:function(e){e=e||this._caret(),this._renderValue(),this._caret(e)},_isValueEmpty:function(){return(0,v.isEmpty)(this._value)},_shouldShowMask:function(){var e=this.option("showMaskMode");return"onFocus"!==e||((0,p.focused)(this._input())||!this._isValueEmpty())},_showMaskPlaceholder:function(){if(this._shouldShowMask()){var e=this._maskRulesChain.text();this.option("text",e),"onFocus"===this.option("showMaskMode")&&this._renderDisplayText(e)}},_renderValue:function(){if(this._maskRulesChain){var e=this._maskRulesChain.text();if(this._showMaskPlaceholder(),this._$hiddenElement){var t=this._maskRulesChain.value(),n=this._isMaskedValueMode()?e:t;this._$hiddenElement.val((0,v.isEmpty)(t)?"":n)}}return this.callBase()},_valueChangeEventHandler:function(e){return this._maskRulesChain?(this._saveValueChangeEvent(e),void this.option("value",this._convertToValue().replace(/\s+$/,""))):void this.callBase.apply(this,arguments)},_isControlKeyFired:function(e){return this._isControlKey((0,d.normalizeKeyName)(e))||(0,d.isCommandKeyPressed)(e)},_handleChain:function(e){var t=this._maskRulesChain.handle(this._normalizeChainArguments(e));return this._value=this._maskRulesChain.value(),this._textValue=this._maskRulesChain.text(),t},_normalizeChainArguments:function(e){return e=e||{},e.index=0,e.fullText=this._maskRulesChain.text(),e},_convertToValue:function(e){
return e=this._isMaskedValueMode()?this._replaceMaskCharWithEmpty(e||this._textValue||""):e||this._value||""},_replaceMaskCharWithEmpty:function(e){return e.replace(new RegExp(this.option("maskChar"),"g"),D)},_maskKeyHandler:function(e,t){var n=this;if(!this.option("readOnly")){this.setForwardDirection(),e.preventDefault(),this._handleSelection();var i=this._input().val(),o=function(){i!==n._input().val()&&n._maskStrategy.runWithoutEventProcessing(function(){return h.default.trigger(n._input(),"input")})},a=t();a?a.then(o):(this.setForwardDirection(),this._adjustCaret(),this._displayMask(),this._maskRulesChain.reset(),o())}},_handleKey:function(e,t){this._direction(t||O),this._adjustCaret(e),this._handleKeyChain(e),this._moveCaret()},_handleSelection:function(){if(this._hasSelection()){var e=this._caret(),t=new Array(e.end-e.start+1).join(D);this._handleKeyChain(t)}},_handleKeyChain:function(e){var t=this._caret(),n=this.isForwardDirection()?t.start:t.start-1,i=this.isForwardDirection()?t.end:t.end-1,o=n===i?1:i-n;this._handleChain({text:e,start:n,length:o})},_tryMoveCaretBackward:function(){this.setBackwardDirection();var e=this._caret().start;return this._adjustCaret(),!e||e!==this._caret().start},_adjustCaret:function(e){var t=this._maskRulesChain.adjustedCaret(this._caret().start,this.isForwardDirection(),e);this._caret({start:t,end:t})},_moveCaret:function(){var e=this._caret().start,t=e+(this.isForwardDirection()?0:-1),n=this._maskRulesChain.isAccepted(t)?e+(this.isForwardDirection()?1:-1):e;this._caret({start:n,end:n})},_caret:function(e){var t=this._input();if(t.length)return arguments.length?void S(t,e):S(t)},_hasSelection:function(){var e=this._caret();return e.start!==e.end},_direction:function(e){return arguments.length?void(this._typingDirection=e):this._typingDirection},setForwardDirection:function(){this._direction(O)},setBackwardDirection:function(){this._direction(E)},isForwardDirection:function(){return this._direction()===O},_clean:function(){this._maskStrategy&&this._maskStrategy.clean(),this.callBase()},_validateMask:function(){if(this._maskRulesChain){var e=(0,v.isEmpty)(this.option("value"))||this._maskRulesChain.isValid(this._normalizeChainArguments());this.option({isValid:e,validationError:e?null:{editorSpecific:!0,message:this.option("maskInvalidMessage")}})}},_updateHiddenElement:function(){this._removeHiddenElement(),this.option("mask")&&(this._input().removeAttr("name"),this._renderHiddenElement()),this._setSubmitElementName(this.option("name"))},_updateMaskOption:function(){this._updateHiddenElement(),this._renderMask(),this._validateMask()},_processEmptyMask:function(e){if(!e){var t=this.option("value");this.option({text:t,isValid:!0}),this.validationRequest.fire({value:t,editor:this}),this._renderValue()}},_optionChanged:function(e){switch(e.name){case"mask":this._updateMaskOption(),this._processEmptyMask(e.value);break;case"maskChar":case"maskRules":case"useMaskedValue":this._updateMaskOption();break;case"value":this._renderMaskedValue(),this._validateMask(),this.callBase(e),this._changedValue=this._input().val();break;case"maskInvalidMessage":break;case"showMaskMode":this.option("text",""),this._renderValue();break;default:this.callBase(e)}}}),R=P;t.default=R,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){var e=s.default.real(),t=e.android,n=e.version;return t&&n[0]>4}function a(){return"onbeforeinput"in r.default.createElement("input")||o()}t.isInputEventsL2Supported=a;var r=i(n(11)),s=i(n(13))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.StubMaskRule=t.MaskRule=t.EmptyMaskRule=void 0;var o=i(n(15)),a=n(0),r=n(12),s=n(1),l=n(4),u=" ",c=o.default.inherit({ctor:function(e){this._value=u,(0,a.extend)(this,e)},next:function(e){return arguments.length?void(this._next=e):this._next},text:l.noop,value:l.noop,rawValue:l.noop,handle:l.noop,_prepareHandlingArgs:function(e,t){t=t||{};var n=Object.prototype.hasOwnProperty.call(e,"value")?"value":"text";return e[n]=(0,s.isDefined)(t.str)?t.str:e[n],e.start=(0,s.isDefined)(t.start)?t.start:e.start,e.length=(0,s.isDefined)(t.length)?t.length:e.length,e.index=e.index+1,e},reset:l.noop,clear:l.noop,first:function(e){return e=e||0,this.next().first(e+1)},isAccepted:function(){return!1},adjustedCaret:function(e,t,n){return t?this._adjustedForward(e,0,n):this._adjustedBackward(e,0,n)},_adjustedForward:l.noop,_adjustedBackward:l.noop,isValid:l.noop}),d=c.inherit({next:l.noop,handle:function(){return 0},text:function(){return""},value:function(){return""},first:function(){return 0},rawValue:function(){return""},adjustedCaret:function(){return 0},isValid:function(){return!0}});t.EmptyMaskRule=d;var h=c.inherit({text:function(){return(this._value!==u?this._value:this.maskChar)+this.next().text()},value:function(){return this._value+this.next().value()},rawValue:function(){return this._value+this.next().rawValue()},handle:function(e){var t=Object.prototype.hasOwnProperty.call(e,"value")?e.value:e.text;if(!t||!t.length||!e.length)return 0;if(e.start)return this.next().handle(this._prepareHandlingArgs(e,{start:e.start-1}));var n=t[0],i=t.substring(1);return this._tryAcceptChar(n,e),this._accepted()?this.next().handle(this._prepareHandlingArgs(e,{str:i,length:e.length-1}))+1:this.handle(this._prepareHandlingArgs(e,{str:i,length:e.length-1}))},clear:function(e){this._tryAcceptChar(u,e),this.next().clear(this._prepareHandlingArgs(e))},reset:function(){this._accepted(!1),this.next().reset()},_tryAcceptChar:function(e,t){if(this._accepted(!1),this._isAllowed(e,t)){var n=e===u?this.maskChar:e;t.fullText=t.fullText.substring(0,t.index)+n+t.fullText.substring(t.index+1),this._accepted(!0),this._value=e}},_accepted:function(e){return arguments.length?void(this._isAccepted=!!e):!!this._isAccepted},first:function(e){return this._value===u?e||0:this.callBase(e)},_isAllowed:function(e,t){return e===u||this._isValid(e,t)},_isValid:function(e,t){var n=this.allowedChars;return n instanceof RegExp?n.test(e):(0,s.isFunction)(n)?n(e,t.index,t.fullText):Array.isArray(n)?(0,r.inArray)(e,n)>-1:n===e},isAccepted:function(e){return 0===e?this._accepted():this.next().isAccepted(e-1)},_adjustedForward:function(e,t,n){return t>=e?t:this.next()._adjustedForward(e,t+1,n)||t+1},_adjustedBackward:function(e,t){return t>=e-1?e:this.next()._adjustedBackward(e,t+1)||t+1},isValid:function(e){return this._isValid(this._value,e)&&this.next().isValid(this._prepareHandlingArgs(e))}});t.MaskRule=h;var f=h.inherit({value:function(){return this.next().value()},handle:function(e){var t=Object.prototype.hasOwnProperty.call(e,"value"),n=t?e.value:e.text;if(!n.length||!e.length)return 0;if(e.start||t)return this.next().handle(this._prepareHandlingArgs(e,{start:e.start&&e.start-1}));var i=n[0],o=n.substring(1);this._tryAcceptChar(i);var a=this._isAllowed(i)?this._prepareHandlingArgs(e,{str:o,length:e.length-1}):e;return this.next().handle(a)+1},clear:function(e){this._accepted(!1),this.next().clear(this._prepareHandlingArgs(e))},_tryAcceptChar:function(e){this._accepted(this._isValid(e))},_isValid:function(e){return e===this.maskChar},first:function(e){return e=e||0,this.next().first(e+1)},_adjustedForward:function(e,t,n){return t>=e&&n===this.maskChar?t:e===t+1&&this._accepted()?e:this.next()._adjustedForward(e,t+1,n)},_adjustedBackward:function(e,t){return t>=e-1?0:this.next()._adjustedBackward(e,t+1)},isValid:function(e){return this.next().isValid(this._prepareHandlingArgs(e))}});t.StubMaskRule=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){if((0,c.isDefined)(e)&&!Array.isArray(e))throw w.default.Error("E1053")}t.default=void 0;var a=i(n(2)),r=i(n(11)),s=i(n(5)),l=n(33),u=n(75),c=n(1),d=n(0),h=n(12),f=n(3),p=n(36),g=i(n(13)),_=i(n(52)),m=n(8),v=i(n(25)),y=i(n(589)),b=i(n(590)),x=i(n(28)),w=i(n(17)),C=n(6),k=i(n(88)),S="dx-texteditor",D="dx-texteditor-input-container",I="dx-texteditor-input",T="."+I,O="dx-texteditor-container",E="dx-texteditor-buttons-container",A="dx-placeholder",P="dx-texteditor-empty",R="dx-editor-",M=[R+"outlined",R+"filled",R+"underlined"],B="dx-state-invisible",V="dx-pending-indicator",F="dx-validation-pending",L="dx-valid",H=["KeyDown","KeyPress","KeyUp","Change","Cut","Copy","Paste","Input"],z=["tab","enter","shift","control","alt","escape","pageUp","pageDown","end","home","leftArrow","upArrow","rightArrow","downArrow"],N=_.default.inherit({ctor:function(e,t){t&&o(t.buttons),this._buttonCollection=new b.default(this,this._getDefaultButtons()),this._$beforeButtonsContainer=null,this._$afterButtonsContainer=null,this.callBase.apply(this,arguments)},_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{buttons:void 0,value:"",spellcheck:!1,showClearButton:!1,valueChangeEvent:"change",placeholder:"",inputAttr:{},onFocusIn:null,onFocusOut:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onChange:null,onInput:null,onCut:null,onCopy:null,onPaste:null,onEnterKey:null,mode:"text",hoverStateEnabled:!0,focusStateEnabled:!0,text:void 0,displayValueFormatter:function(e){return(0,c.isDefined)(e)&&e!==!1?e:""},stylingMode:(0,x.default)().editorStylingMode||"outlined",showValidationMark:!0})},_defaultOptionsRules:function(){var e=(0,p.current)();return this.callBase().concat([{device:function(){return(0,p.isMaterial)(e)},options:{stylingMode:(0,x.default)().editorStylingMode||"underlined"}}])},_setDeprecatedOptions:function(){this.callBase(),(0,d.extend)(this._deprecatedOptions,{onKeyPress:{since:"20.1",message:"This event is removed from the web standards and will be deprecated in modern browsers soon."}})},_getDefaultButtons:function(){return[{name:"clear",Ctor:y.default}]},_isClearButtonVisible:function(){return this.option("showClearButton")&&!this.option("readOnly")},_input:function(){return this.$element().find(T).first()},_isFocused:function(){return(0,u.focused)(this._input())||this.callBase()},_inputWrapper:function(){return this.$element()},_buttonsContainer:function(){return this._inputWrapper().find("."+E).eq(0)},_isControlKey:function(e){return z.indexOf(e)!==-1},_renderStylingMode:function(){var e=this,t="stylingMode",n=this.option(t);M.forEach(function(t){return e.$element().removeClass(t)});var i=R+n;if(M.indexOf(i)===-1){var o=this._getDefaultOptions()[t],a=this._convertRulesToOptions(this._defaultOptionsRules())[t];i=R+(a||o)}this.$element().addClass(i),this._updateButtonsStyling(n)},_initMarkup:function(){this.$element().addClass(S),this._renderInput(),this._renderStylingMode(),this._renderInputType(),this._renderPlaceholder(),this._renderProps(),this.callBase(),this._renderValue()},_render:function(){this._renderPlaceholder(),this._refreshValueChangeEvent(),this._renderEvents(),this._renderEnterKeyAction(),this._renderEmptinessEvent(),this.callBase()},_renderInput:function(){this._$buttonsContainer=this._$textEditorContainer=(0,a.default)("<div>").addClass(O).appendTo(this.$element()),this._$textEditorInputContainer=(0,a.default)("<div>").addClass(D).appendTo(this._$textEditorContainer),this._$textEditorInputContainer.append(this._createInput()),this._renderButtonContainers()},_getInputContainer:function(){return this._$textEditorInputContainer},_renderPendingIndicator:function(){this.$element().addClass(F);var e=this._getInputContainer(),t=(0,a.default)("<div>").addClass(V).appendTo(e);this._pendingIndicator=this._createComponent(t,k.default)},_disposePendingIndicator:function(){this._pendingIndicator&&(this._pendingIndicator.dispose(),this._pendingIndicator.$element().remove(),this._pendingIndicator=null,this.$element().removeClass(F))},_renderValidationState:function(){this.callBase();var e="pending"===this.option("validationStatus"),t=this.$element();e?(!this._pendingIndicator&&this._renderPendingIndicator(),this._showValidMark=!1):("invalid"===this.option("validationStatus")&&(this._showValidMark=!1),this._showValidMark||this.option("showValidationMark")!==!0||(this._showValidMark="valid"===this.option("validationStatus")&&!!this._pendingIndicator),this._disposePendingIndicator()),t.toggleClass(L,!!this._showValidMark)},_renderButtonContainers:function(){var e=this.option("buttons");this._$beforeButtonsContainer=this._buttonCollection.renderBeforeButtons(e,this._$buttonsContainer),this._$afterButtonsContainer=this._buttonCollection.renderAfterButtons(e,this._$buttonsContainer)},_clean:function(){this._buttonCollection.clean(),this._disposePendingIndicator(),this._$beforeButtonsContainer=null,this._$afterButtonsContainer=null,this._$textEditorContainer=null,this._$buttonsContainer=null,this.callBase()},_createInput:function(){var e=(0,a.default)("<input>");return this._applyInputAttributes(e,this.option("inputAttr")),e},_setSubmitElementName:function(e){var t=this.option("inputAttr.name");return this.callBase(e||t||"")},_applyInputAttributes:function(e,t){var n=(0,d.extend)(this._getDefaultAttributes(),t);e.attr(n).addClass(I).css("minHeight",this.option("height")?"0":"")},_getDefaultAttributes:function(){var e={autocomplete:"off"},t=g.default.real(),n=t.ios,i=t.mac;return(n||i)&&(e.placeholder=" "),e},_updateButtons:function(e){this._buttonCollection.updateButtons(e)},_updateButtonsStyling:function(e){var t=this;(0,f.each)(this.option("buttons"),function(n,i){if(i.options&&!i.options.stylingMode){var o=t.getButton(i.name);o.option&&o.option("stylingMode","underlined"===e?"text":"contained")}})},_renderValue:function(){var e=this._renderInputValue();return e.promise()},_renderInputValue:function(e){e=e||this.option("value");var t=this.option("text"),n=this.option("displayValue"),i=this.option("displayValueFormatter");return void 0!==n&&null!==e?t=i(n):(0,c.isDefined)(t)||(t=i(e)),this.option("text",t),this._input().val()!==((0,c.isDefined)(t)?t:"")?this._renderDisplayText(t):this._toggleEmptinessEventHandler(),(new C.Deferred).resolve()},_renderDisplayText:function(e){this._input().val(e),this._toggleEmptinessEventHandler()},_isValueValid:function(){if(this._input().length){var e=this._input().get(0).validity;if(e)return e.valid}return!0},_toggleEmptiness:function(e){this.$element().toggleClass(P,e),this._togglePlaceholder(e)},_togglePlaceholder:function(e){this.$element().find(".".concat(A)).toggleClass(B,!e)},_renderProps:function(){this._toggleReadOnlyState(),this._toggleSpellcheckState(),this._toggleTabIndex()},_toggleDisabledState:function(e){this.callBase.apply(this,arguments);var t=this._input();t.prop("disabled",e)},_toggleTabIndex:function(){var e=this._input(),t=this.option("disabled"),n=this.option("focusStateEnabled");t||!n?e.attr("tabIndex",-1):e.removeAttr("tabIndex")},_toggleReadOnlyState:function(){this._input().prop("readOnly",this._readOnlyPropValue()),this.callBase()},_readOnlyPropValue:function(){return this.option("readOnly")},_toggleSpellcheckState:function(){this._input().prop("spellcheck",this.option("spellcheck"))},_renderPlaceholder:function(){this._renderPlaceholderMarkup(),this._attachPlaceholderEvents()},_renderPlaceholderMarkup:function(){this._$placeholder&&(this._$placeholder.remove(),this._$placeholder=null);var e=this._input(),t=this.option("placeholder"),n=this._$placeholder=(0,a.default)("<div>").attr("data-dx_placeholder",t);n.insertAfter(e),n.addClass(A)},_attachPlaceholderEvents:function(){var e=this,t=(0,m.addNamespace)(v.default.up,this.NAME);s.default.on(this._$placeholder,t,function(){s.default.trigger(e._input(),"focus")}),this._toggleEmptinessEventHandler()},_placeholder:function(){return this._$placeholder||(0,a.default)()},_clearValueHandler:function(e){var t=this._input();e.stopPropagation(),this._saveValueChangeEvent(e),this._clearValue(),!this._isFocused()&&s.default.trigger(t,"focus"),s.default.trigger(t,"input")},_clearValue:function(){this.reset()},_renderEvents:function(){var e=this,t=this._input();(0,f.each)(H,function(n,i){if(e.hasActionSubscription("on"+i)){var o=e._createActionByOption("on"+i,{excludeValidators:["readOnly"]});s.default.on(t,(0,m.addNamespace)(i.toLowerCase(),e.NAME),function(t){e._disposed||o({event:t})})}})},_refreshEvents:function(){var e=this,t=this._input();(0,f.each)(H,function(n,i){s.default.off(t,(0,m.addNamespace)(i.toLowerCase(),e.NAME))}),this._renderEvents()},_keyPressHandler:function(){this.option("text",this._input().val())},_keyDownHandler:function(e){var t=this._input(),n=e.ctrlKey&&"enter"===(0,m.normalizeKeyName)(e),i=t.val()!==this.option("value");n&&i&&s.default.trigger(t,"change")},_renderValueChangeEvent:function(){var e=(0,m.addNamespace)(this._renderValueEventName(),"".concat(this.NAME,"TextChange")),t=(0,m.addNamespace)(this.option("valueChangeEvent"),"".concat(this.NAME,"ValueChange")),n=(0,m.addNamespace)("keydown","".concat(this.NAME,"TextChange")),i=this._input();s.default.on(i,e,this._keyPressHandler.bind(this)),s.default.on(i,t,this._valueChangeEventHandler.bind(this)),s.default.on(i,n,this._keyDownHandler.bind(this))},_cleanValueChangeEvent:function(){var e=".".concat(this.NAME,"ValueChange"),t=".".concat(this.NAME,"TextChange");s.default.off(this._input(),e),s.default.off(this._input(),t)},_refreshValueChangeEvent:function(){this._cleanValueChangeEvent(),this._renderValueChangeEvent()},_renderValueEventName:function(){return"input change keypress"},_focusTarget:function(){return this._input()},_focusEventTarget:function(){return this.element()},_preventNestedFocusEvent:function(e){if(e.isDefaultPrevented())return!0;var t=this._isNestedTarget(e.relatedTarget);return"focusin"===e.type&&(t=t&&this._isNestedTarget(e.target)),t&&e.preventDefault(),t},_isNestedTarget:function(e){return!!this.$element().find(e).length},_focusClassTarget:function(){return this.$element()},_focusInHandler:function(e){this._preventNestedFocusEvent(e),this.callBase.apply(this,arguments)},_focusOutHandler:function(e){this._preventNestedFocusEvent(e),this.callBase.apply(this,arguments)},_toggleFocusClass:function(e,t){this.callBase(e,this._focusClassTarget(t))},_hasFocusClass:function(e){return this.callBase((0,a.default)(e||this.$element()))},_renderEmptinessEvent:function(){var e=this._input();s.default.on(e,"input blur",this._toggleEmptinessEventHandler.bind(this))},_toggleEmptinessEventHandler:function(){var e=this._input().val(),t=(""===e||null===e)&&this._isValueValid();this._toggleEmptiness(t)},_valueChangeEventHandler:function(e,t){this._saveValueChangeEvent(e),this.option("value",arguments.length>1?t:this._input().val()),this._saveValueChangeEvent(void 0)},_renderEnterKeyAction:function(){this._enterKeyAction=this._createActionByOption("onEnterKey",{excludeValidators:["readOnly"]}),s.default.off(this._input(),"keyup.onEnterKey.dxTextEditor"),s.default.on(this._input(),"keyup.onEnterKey.dxTextEditor",this._enterKeyHandlerUp.bind(this))},_enterKeyHandlerUp:function(e){this._disposed||"enter"===(0,m.normalizeKeyName)(e)&&this._enterKeyAction({event:e})},_updateValue:function(){this._options.silent("text",null),this._renderValue()},_dispose:function(){this._enterKeyAction=void 0,this.callBase()},_getSubmitElement:function(){return this._input()},_optionChanged:function(e){var t=e.name,n=e.fullName,i=e.value;if((0,h.inArray)(t.replace("on",""),H)>-1)return void this._refreshEvents();switch(t){case"valueChangeEvent":this._refreshValueChangeEvent(),this._refreshFocusEvent(),this._refreshEvents();break;case"onValueChanged":this._createValueChangeAction();break;case"focusStateEnabled":this.callBase(e),this._toggleTabIndex();break;case"spellcheck":this._toggleSpellcheckState();break;case"mode":this._renderInputType();break;case"onEnterKey":this._renderEnterKeyAction();break;case"placeholder":this._renderPlaceholder();break;case"readOnly":case"disabled":this._updateButtons(),this.callBase(e);break;case"showClearButton":this._updateButtons(["clear"]);break;case"text":break;case"value":this._updateValue(),this.callBase(e);break;case"inputAttr":this._applyInputAttributes(this._input(),this.option(t));break;case"stylingMode":this._renderStylingMode();break;case"buttons":n===t&&o(i),this._$beforeButtonsContainer&&this._$beforeButtonsContainer.remove(),this._$afterButtonsContainer&&this._$afterButtonsContainer.remove(),this._buttonCollection.clean(),this._renderButtonContainers();break;case"displayValueFormatter":this._invalidate();break;case"showValidationMark":break;default:this.callBase(e)}},_renderInputType:function(){this._setInputType(this.option("mode"))},_setInputType:function(e){var t=this._input();"search"===e&&(e="text");try{t.prop("type",e)}catch(e){t.prop("type","text")}},getButton:function(e){return this._buttonCollection.getButton(e)},focus:function(){s.default.trigger(this._input(),"focus")},blur:function(){this._input().is(r.default.getActiveElement())&&(0,l.resetActiveElement)()},reset:function(){this._showValidMark&&(this._showValidMark=!1,this._renderValidationState());var e=this._getDefaultOptions();this.option("value")===e.value?(this._options.silent("text",""),this._renderValue()):this.option("value",e.value)},on:function(e,t){var n=this.callBase(e,t),i=e.charAt(0).toUpperCase()+e.substr(1);return H.indexOf(i)>=0&&this._refreshEvents(),n}}),j=N;t.default=j,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(5)),y=o(n(212)),b=n(8),x=o(n(25)),w=n(21),C=x.default.down,k="dx-state-invisible",S="dx-clear-button-area",D="dx-icon-clear",I="dx-icon",T="dx-show-clear-button",O=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_create",value:function(){var e=(0,m.default)("<span>").addClass(S).append((0,m.default)("<span>").addClass(I).addClass(D));return this._addToContainer(e),this.update(!0),{instance:e,$element:e}}},{key:"_isVisible",value:function(){var e=this.editor;return e._isClearButtonVisible()}},{key:"_attachEvents",value:function(e,t){var n=this.editor,i=n.NAME;v.default.on(t,(0,b.addNamespace)(C,i),function(e){e.preventDefault(),"mouse"!==e.pointerType&&n._clearValueHandler(e)}),v.default.on(t,(0,b.addNamespace)(w.name,i),function(e){return n._clearValueHandler(e)})}},{key:"_legacyRender",value:function(e,t){e.toggleClass(T,t)}},{key:"update",value:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];!e&&l(_(t.prototype),"update",this).call(this);var n=this.editor,i=this.instance,o=n.$element(),a=this._isVisible();i&&i.toggleClass(k,!a),this._legacyRender(o,a)}}]),t}(y.default);t.default=O,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}function s(e){"@babel/helpers - typeof";return(s="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function l(e){var t=function(){if(!e||"object"!==s(e)||Array.isArray(e))throw g.default.Error("E1053")},n=function(){var t=e.location;"location"in e&&"after"!==t&&"before"!==t&&(e.location="after")},i=function(){if(!("name"in e))throw g.default.Error("E1054")},o=function(){var t=e.name;if("string"!=typeof t)throw g.default.Error("E1055")};t(),i(),o(),n()}function u(e,t){if(e.indexOf(t)!==-1)throw g.default.Error("E1055",t);e.push(t)}function c(e,t){return!!(0,p.find)(t,function(t){return t.name===e})}t.default=void 0;var d=i(n(2)),h=i(n(591)),f=n(0),p=n(12),g=i(n(17)),_="dx-texteditor-buttons-container",m=function(){function e(t,n){o(this,e),this.buttons=[],this.defaultButtonsInfo=n,this.editor=t}return r(e,[{key:"_compileButtonInfo",value:function(e){var t=this,n=[];return e.map(function(e){var i="string"==typeof e;i||l(e);var o=i||c(e.name,t.defaultButtonsInfo);if(o){var a=(0,p.find)(t.defaultButtonsInfo,function(t){var n=t.name;return n===e||n===e.name});if(!a)throw g.default.Error("E1056",t.editor.NAME,e);return u(n,e),a}var r=e.name;return u(n,r),(0,f.extend)(e,{Ctor:h.default})})}},{key:"_createButton",value:function(e){var t=e.Ctor,n=e.options,i=e.name,o=new t(i,this.editor,n);return this.buttons.push(o),o}},{key:"_renderButtons",value:function(e,t,n){var i=this,o=null,a=e?this._compileButtonInfo(e):this.defaultButtonsInfo,r=function(){return o=o||(0,d.default)("<div>").addClass(_),"before"===n?t.prepend(o):t.append(o),o};return a.forEach(function(e){var t=e.location,o=void 0===t?"after":t;o===n&&i._createButton(e).render(r())}),o}},{key:"clean",value:function(){this.buttons.forEach(function(e){return e.dispose()}),this.buttons=[]}},{key:"getButton",value:function(e){var t=(0,p.find)(this.buttons,function(t){var n=t.name;return n===e});return t&&t.instance}},{key:"renderAfterButtons",value:function(e,t){return this._renderButtons(e,t,"after")}},{key:"renderBeforeButtons",value:function(e,t){return this._renderButtons(e,t,"before")}},{key:"updateButtons",value:function(e){this.buttons.forEach(function(t){e&&e.indexOf(t.name)===-1||t.update()})}}]),e}();t.default=m,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(212)),y=o(n(38)),b=n(0),x=o(n(5)),w=n(158),C=n(21),k="dx-custom-button-hovered",S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_attachEvents",value:function(e,t){var n=this.editor;x.default.on(t,w.start,function(){n.$element().addClass(k)}),x.default.on(t,w.end,function(){n.$element().removeClass(k)}),x.default.on(t,C.name,function(e){e.stopPropagation()})}},{key:"_create",value:function(){var e=this.editor,t=(0,m.default)("<div>");this._addToContainer(t);var n=e._createComponent(t,y.default,(0,b.extend)({},this.options,{ignoreParentReadOnly:!0,disabled:this._isDisabled(),integrationOptions:this._prepareIntegrationOptions(e)}));return{$element:t,instance:n}}},{key:"_prepareIntegrationOptions",value:function(e){return(0,b.extend)({},e.option("integrationOptions"),{skipTemplates:["content"]})}},{key:"update",value:function(){var e=l(_(t.prototype),"update",this).call(this);return this.instance&&this.instance.option("disabled",this._isDisabled()),e}},{key:"_isVisible",value:function(){var e=this.editor;return e.option("visible")}},{key:"_isDisabled",value:function(){var e=void 0!==this.options.disabled;return e?this.instance?this.instance.option("disabled"):this.options.disabled:this.editor.option("readOnly")}}]),t}(v.default);t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){
var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=o(n(424)),k=n(8),S=o(n(77)),D="deleteContentBackward",I=" ",T=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"_getStrategyName",value:function(){return"default"}},{key:"getHandleEventNames",value:function(){return[].concat(a(p(w(t.prototype),"getHandleEventNames",this).call(this)),["keyPress"])}},{key:"_keyPressHandler",value:function(e){if(!this._keyPressHandled&&(this._keyPressHandled=!0,!this.editor._isControlKeyFired(e))){var t=this.editor;t._maskKeyHandler(e,function(){return t._handleKey((0,k.getChar)(e))})}}},{key:"_inputHandler",value:function(e){if(this._backspaceInputHandled(e.originalEvent&&e.originalEvent.inputType)&&this._handleBackspaceInput(e),e.originalEvent&&this._autoFillHandler(e),!this._keyPressHandled){this._keyPressHandled=!0;var t=this.editorInput().val(),n=this.editorCaret();if(n.end){n.start=n.end-1;var i=t.substring(0,n.start)+t.substring(n.end),o=t[n.start],a=this.editor;this.editorInput().val(i),a._caret({start:n.start,end:n.start}),a._maskKeyHandler(e,function(){return a._handleKey(o)})}}}},{key:"_backspaceHandler",value:function(e){var t=this,n=this.editor;this._keyPressHandled=!0;var i=function(e,i){e&&(n._direction(t.DIRECTION.FORWARD),n._adjustCaret());var o=t.editorCaret();return new S.default(function(e){clearTimeout(t._backspaceHandlerTimeout),t._backspaceHandlerTimeout=setTimeout(function(){i(o),e()})})};n._maskKeyHandler(e,function(){return n._hasSelection()?i(!0,function(e){n._displayMask(e),n._maskRulesChain.reset()}):n._tryMoveCaretBackward()?i(!1,function(e){t.editorCaret(e)}):(n._handleKey(I,t.DIRECTION.BACKWARD),i(!0,function(e){n._displayMask(e),n._maskRulesChain.reset()}))})}},{key:"_backspaceInputHandled",value:function(e){return e===D&&!this._keyPressHandled}},{key:"_handleBackspaceInput",value:function(e){var t=this.editorCaret(),n=t.start,i=t.end;this.editorCaret({start:n+1,end:i+1}),this._backspaceHandler(e)}}]),t}(C.default),O=T;t.default=O,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=o(n(424)),k="deleteContentBackward",S=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"_getStrategyName",value:function(){return"inputEvents"}},{key:"getHandleEventNames",value:function(){return[].concat(a(p(w(t.prototype),"getHandleEventNames",this).call(this)),["beforeInput"])}},{key:"_beforeInputHandler",value:function(){this._prevCaret=this.editorCaret()}},{key:"_inputHandler",value:function(e){var t=e.originalEvent;if(t){var n=t.inputType,i=t.data,o=this.editorCaret();if(n===k){var a=this._prevCaret.end-this._prevCaret.start||1;this.editor.setBackwardDirection(),this._updateEditorMask({start:o.start,length:a,text:this._getEmptyString(a)})}else{if(!o.end)return;this._autoFillHandler(t),this.editorCaret(o);var r=this._prevCaret.end-this._prevCaret.start,s=i+(r?this._getEmptyString(r-i.length):"");this.editor.setForwardDirection();var l=this._updateEditorMask({start:this._prevCaret.start,length:r||s.length,text:s});l||this.editorCaret(this._prevCaret)}}}},{key:"_getEmptyString",value:function(e){return Array(e+1).join(" ")}},{key:"_updateEditorMask",value:function(e){var t=e.text.length,n=this.editor._handleChain(e);if(this.editor.isForwardDirection()){var i=this.editorCaret(),o=i.start,a=i.end,r=n-t;o<=n&&n>1&&this.editorCaret({start:o+r,end:a+r}),this.editor.isForwardDirection()&&this.editor._adjustCaret()}return this.editor._displayMask(),!!n}}]),t}(C.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(425)),a=i(n(287)),r=o.default.inherit(a.default).inherit({_addWidgetPrefix:function(e){return"dx-list-"+e},_getCombinedFilter:function(){var e,t,n=this._dataSource;return n&&(t={filter:n.filter()},n._addSearchFilter(t),e=t.filter),e},_initDataSource:function(){var e=this.option("searchValue"),t=this.option("searchExpr"),n=this.option("searchMode");this.callBase(),this._dataSource&&(e&&e.length&&this._dataSource.searchValue(e),n.length&&this._dataSource.searchOperation(a.default.getOperationBySearchMode(n)),t&&this._dataSource.searchExpr(t))}}),s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(1),r=n(3),s=i(n(91)),l=i(n(47)),u=i(n(207)),c="dx-list-item",d="dx-list-group",h=20,f=2303,p=function(e){return(e.group<<h)+e.item},g=function(e){return{group:e>>h,item:e&f}},_=u.default.inherit({_groupElements:function(){return this._collectionWidget._itemContainer().find("."+d)},_groupItemElements:function(e){return e.find("."+c)},getIndexByItemData:function(e){var t=this._collectionWidget.option("items"),n=!1;return!!e&&(e.items&&e.items.length&&(e=e.items[0]),(0,r.each)(t,function(t,i){return!!i.items&&((0,r.each)(i.items,function(i,o){return o!==e||(n={group:t,item:i},!1)}),!n&&void 0)}),n)},getItemDataByIndex:function(e){var t=this._collectionWidget.option("items");return(0,a.isNumeric)(e)?this.itemsGetter()[e]:e&&t[e.group]&&t[e.group].items[e.item]||null},itemsGetter:function(){for(var e=[],t=this._collectionWidget.option("items"),n=0;n<t.length;n++)t[n]&&t[n].items?e=e.concat(t[n].items):e.push(t[n]);return e},deleteItemAtIndex:function(e){var t=g(e),n=this._collectionWidget.option("items")[t.group].items;n.splice(t.item,1)},getKeysByItems:function(e){var t,n=[];for(t=0;t<e.length;t++)e[t]&&e[t].items?n=n.concat(e[t].items):n.push(e[t]);var i=[];for(t=0;t<n.length;t++)i.push(this._collectionWidget.keyOf(n[t]));return i},getIndexByKey:function(e,t){var n=t||this._collectionWidget.option("items"),i=-1,o=this;return(0,r.each)(n,function(t,n){if(n.items){var a=o.getKeysByItems(n.items);return(0,r.each)(a,function(n,a){if(o._equalKeys(a,e))return i={group:t,item:n},!1}),i===-1&&void 0}}),i},_getGroups:function(e){var t=this._collectionWidget.getDataSource(),n=t&&t.group();return n?s.default.queryByOptions((0,l.default)(e),{group:n}).toArray():this._collectionWidget.option("items")},getItemsByKeys:function(e,t){var n=[];return(0,r.each)(e,function(e,i){var o=function(e){var t=this.getIndexByKey(i,e),n=t&&e[t.group];if(n)return{groupKey:n.key,item:n.items[t.item]}}.bind(this),a=o(this._getGroups(t));if(a){var s,l=a.groupKey,u=a.item;(0,r.each)(n,function(e,t){if(t.key===l)return s=t,!1}),s||(s={key:l,items:[]},n.push(s)),s.items.push(u)}}.bind(this)),n},moveItemAtIndexToIndex:function(e,t){var n=this._collectionWidget.option("items"),i=g(e),o=g(t),a=n[i.group].items,r=n[o.group].items,s=a[i.item];a.splice(i.item,1),r.splice(o.item,0,s)},_isItemIndex:function(e){return e&&(0,a.isNumeric)(e.group)&&(0,a.isNumeric)(e.item)},_getNormalizedItemIndex:function(e){var t=(0,o.default)(e),n=t.closest("."+d);return n.length?p({group:this._groupElements().index(n),item:this._groupItemElements(n).index(t)}):-1},_normalizeItemIndex:function(e){return p(e)},_denormalizeItemIndex:function(e){return g(e)},_getItemByNormalizedIndex:function(e){var t=g(e),n=this._groupElements().eq(t.group);return this._groupItemElements(n).eq(t.item)},_itemsFromSameParent:function(e,t){return g(e).group===g(t).group}}),m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=i(n(15)),s=n(0),l=n(3),u=i(n(17)),c=n(125);n(597),n(598),n(599),n(600),n(601),n(607),n(609);var d=[],h=function(e,t,n){d.push({enabled:e,decoratorType:t,decoratorSubType:n})};h(function(){return this.option("menuItems").length},function(){return"menu"},function(){return this.option("menuMode")}),h(function(){return!this.option("menuItems").length&&this.option("allowItemDeleting")},function(){var e=this.option("itemDeleteMode");return"toggle"===e||"slideButton"===e||"swipe"===e||"static"===e?"delete":"menu"},function(){var e=this.option("itemDeleteMode");return"slideItem"===e&&(e="slide"),"hold"===e&&(e="context"),e}),h(function(){return"none"!==this.option("selectionMode")&&this.option("showSelectionControls")},function(){return"selection"},function(){return"default"}),h(function(){return this.option("itemDragging.allowReordering")||this.option("itemDragging.allowDropInsideItem")||this.option("itemDragging.group")},function(){return"reorder"},function(){return"default"});var f="dx-list-item-before-bag",p="dx-list-item-after-bag",g="beforeBag",_="afterBag",m="modifyElement",v="afterRender",y="getExcludedSelectors",b=r.default.inherit({ctor:function(e){this._list=e,this._fetchRequiredDecorators()},dispose:function(){this._decorators&&this._decorators.length&&(0,l.each)(this._decorators,function(e,t){t.dispose()})},_fetchRequiredDecorators:function(){this._decorators=[],(0,l.each)(d,function(e,t){var n=t.enabled.call(this._list);if(n){var i=t.decoratorType.call(this._list),o=t.decoratorSubType.call(this._list),a=this._createDecorator(i,o);this._decorators.push(a)}}.bind(this))},_createDecorator:function(e,t){var n=this._findDecorator(e,t);return new n(this._list)},_findDecorator:function(e,t){var n=c.registry[e][t];if(!n)throw u.default.Error("E1012",e,t);return n},modifyItemElement:function(e){var t=(0,o.default)(e.itemElement),n={$itemElement:t};this._prependBeforeBags(t,n),this._appendAfterBags(t,n),this._applyDecorators(m,n)},afterItemsRendered:function(){this._applyDecorators(v)},_prependBeforeBags:function(e,t){var n=this._collectDecoratorsMarkup(g,t,f);e.prepend(n)},_appendAfterBags:function(e,t){var n=this._collectDecoratorsMarkup(_,t,p);e.append(n)},_collectDecoratorsMarkup:function(e,t,n){var i=(0,o.default)("<div>");return(0,l.each)(this._decorators,function(){var a=(0,o.default)("<div>").addClass(n);this[e]((0,s.extend)({$container:a},t)),a.children().length&&i.append(a)}),i.children()},_applyDecorators:function(e,t){(0,l.each)(this._decorators,function(){this[e](t)})},_handlerExists:function(e){if(!this._decorators)return!1;for(var t=this._decorators,n=t.length,i=0;i<n;i++)if(t[i][e]!==a.noop)return!0;return!1},_eventHandler:function(e,t,n){if(!this._decorators)return!1;for(var i=!1,o=this._decorators,a=o.length,r=0;r<a&&!(i=o[r][e](t,n));r++);return i},handleClick:function(e,t){return this._eventHandler("handleClick",e,t)},handleKeyboardEvents:function(e,t){return this._eventHandler("handleKeyboardEvents",e,t)},handleEnterPressing:function(){return this._eventHandler("handleEnterPressing")},contextMenuHandlerExists:function(){return this._handlerExists("handleContextMenu")},handleContextMenu:function(e,t){return this._eventHandler("handleContextMenu",e,t)},getExcludedItemSelectors:function(){var e=[];return this._applyDecorators(y,e),e.join(",")}}),x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(38)),r=n(125),s=i(n(165)),l="dx-list-static-delete-button-container",u="dx-list-static-delete-button";(0,r.register)("delete","static",s.default.inherit({afterBag:function(e){var t=e.$itemElement,n=e.$container,i=(0,o.default)("<div>").addClass(u);this._list._createComponent(i,a.default,{icon:"remove",onClick:function(e){e.event.stopPropagation(),this._deleteItem(t)}.bind(this),integrationOptions:{}}),n.addClass(l).append(i)},_deleteItem:function(e){e.is(".dx-state-disabled, .dx-state-disabled *")||this._list.deleteItem(e)}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(43)),r=i(n(38)),s=i(n(10)),l=n(125),u=i(n(426)),c="dx-list-switchable-delete-button-container",d="dx-list-switchable-delete-button-wrapper",h="dx-list-switchable-delete-button-inner-wrapper",f="dx-list-switchable-delete-button",p=200,g=u.default.inherit({_init:function(){this.callBase.apply(this,arguments);var e=(0,o.default)("<div>").addClass(c),t=(0,o.default)("<div>").addClass(d),n=(0,o.default)("<div>").addClass(h),i=(0,o.default)("<div>").addClass(f);this._list._createComponent(i,r.default,{text:s.default.format("dxListEditDecorator-delete"),type:"danger",onClick:function(e){this._deleteItem(),e.event.stopPropagation()}.bind(this),integrationOptions:{}}),e.append(t),t.append(n),n.append(i),this._$buttonContainer=e},_enablePositioning:function(e){this.callBase.apply(this,arguments),a.default.stop(this._$buttonContainer,!0),this._$buttonContainer.appendTo(e)},_disablePositioning:function(){this.callBase.apply(this,arguments),this._$buttonContainer.detach()},_animatePrepareDeleteReady:function(){var e=this._isRtlEnabled(),t=this._list.$element().width(),n=this._buttonWidth(),i=e?t:-n,o=e?t-n:0;return a.default.animate(this._$buttonContainer,{type:"custom",duration:p,from:{right:i},to:{right:o}})},_animateForgetDeleteReady:function(){var e=this._isRtlEnabled(),t=this._list.$element().width(),n=this._buttonWidth(),i=e?t-n:0,o=e?t:-n;return a.default.animate(this._$buttonContainer,{type:"custom",duration:p,from:{right:i},to:{right:o}})},_buttonWidth:function(){return this._buttonContainerWidth||(this._buttonContainerWidth=this._$buttonContainer.outerWidth()),this._buttonContainerWidth},dispose:function(){this._$buttonContainer&&this._$buttonContainer.remove(),this.callBase.apply(this,arguments)}}),_="dx-list-toggle-delete-switch-container",m="dx-list-toggle-delete-switch";(0,l.register)("delete","toggle",g.inherit({beforeBag:function(e){var t=e.$itemElement,n=e.$container,i=(0,o.default)("<div>").addClass(m);this._list._createComponent(i,r.default,{icon:"toggle-delete",onClick:function(e){a.default.stop(this._$buttonContainer,!1),this._toggleDeleteReady(t),e.event.stopPropagation()}.bind(this),integrationOptions:{}}),n.addClass(_),n.append(i)}})),(0,l.register)("delete","slideButton",g.inherit({_shouldHandleSwipe:!0,_swipeEndHandler:function(e,t){return 0!==t.targetOffset&&(a.default.stop(this._$buttonContainer,!1),this._toggleDeleteReady(e)),!0}}));var v=g;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(5)),r=n(4),s=n(21),l=i(n(10)),u=n(30),c=n(8),d=n(140),h=i(n(427)),f=n(125),p=i(n(426)),g=i(n(43)),_=n(36),m=i(n(281)),v="dxListEditDecorator",y=(0,c.addNamespace)(s.name,v),b=(0,c.addNamespace)(d.active,v),x="dx-list-slide-menu",w="dx-list-slide-menu-wrapper",C="dx-list-slide-menu-content",k="dx-list-slide-menu-buttons-container",S="dx-list-slide-menu-buttons",D="dx-list-slide-menu-button",I="dx-list-slide-menu-button-menu",T="dx-list-slide-menu-button-delete",O=400,E="cubic-bezier(0.075, 0.82, 0.165, 1)";(0,f.register)("menu","slide",p.default.inherit({_shouldHandleSwipe:!0,_init:function(){this.callBase.apply(this,arguments),this._$buttonsContainer=(0,o.default)("<div>").addClass(k),a.default.on(this._$buttonsContainer,b,r.noop),this._$buttons=(0,o.default)("<div>").addClass(S).appendTo(this._$buttonsContainer),this._renderMenu(),this._renderDeleteButton()},_renderMenu:function(){if(this._menuEnabled()){var e=this._menuItems();if(1===e.length){var t=e[0];this._renderMenuButton(t.text,function(e){e.stopPropagation(),this._fireAction(t)}.bind(this))}else{var n=(0,o.default)("<div>").addClass(x);this._menu=this._list._createComponent(n,m.default,{showTitle:!1,items:e,onItemClick:function(e){this._fireAction(e.itemData)}.bind(this),integrationOptions:{}}),n.appendTo(this._list.$element());var i=this._renderMenuButton(l.default.format("dxListEditDecorator-more"),function(e){e.stopPropagation(),this._menu.show()}.bind(this));this._menu.option("target",i)}}},_renderMenuButton:function(e,t){var n=(0,o.default)("<div>").addClass(D).addClass(I).text(e);return this._$buttons.append(n),a.default.on(n,y,t),n},_renderDeleteButton:function(){if(this._deleteEnabled()){var e=(0,o.default)("<div>").addClass(D).addClass(T).text((0,_.isMaterial)()?"":l.default.format("dxListEditDecorator-delete"));a.default.on(e,y,function(e){e.stopPropagation(),this._deleteItem()}.bind(this)),this._$buttons.append(e)}},_fireAction:function(e){this._fireMenuAction((0,o.default)(this._cachedNode),e.action),this._cancelDeleteReadyItem()},modifyElement:function(e){this.callBase.apply(this,arguments);var t=e.$itemElement;t.addClass(w);var n=(0,o.default)("<div>").addClass(C);t.wrapInner(n)},_getDeleteButtonContainer:function(){return this._$buttonsContainer},handleClick:function(e,t){return!!(0,o.default)(t.target).closest("."+C).length&&this.callBase.apply(this,arguments)},_swipeStartHandler:function(e){this._enablePositioning(e),this._cacheItemData(e),this._setPositions(this._getPositions(0))},_swipeUpdateHandler:function(e,t){var n=this._isRtlEnabled(),i=n?-1:1,o=this._isReadyToDelete(e),a=this._getCurrentPositions().content===this._getStartPositions().content;if(a&&!o&&t.offset*i>0)return void(t.cancel=!0);var r=this._cachedItemWidth*t.offset,s=o?-this._cachedButtonWidth*i:0,l=(r+s)*i,u=l<0?Math.abs((r+s)/this._cachedButtonWidth):0;return this._setPositions(this._getPositions(u)),!0},_getStartPositions:function(){var e=this._isRtlEnabled(),t=e?-1:1;return{content:0,buttonsContainer:e?-this._cachedButtonWidth:this._cachedItemWidth,buttons:-this._cachedButtonWidth*t}},_getPositions:function(e){var t=this._isRtlEnabled(),n=t?-1:1,i=this._getStartPositions();return{content:i.content-e*this._cachedButtonWidth*n,buttonsContainer:i.buttonsContainer-Math.min(e,1)*this._cachedButtonWidth*n,buttons:i.buttons+Math.min(e,1)*this._cachedButtonWidth*n}},_getCurrentPositions:function(){return{content:(0,u.locate)(this._$cachedContent).left,buttonsContainer:(0,u.locate)(this._$buttonsContainer).left,buttons:(0,u.locate)(this._$buttons).left}},_setPositions:function(e){(0,u.move)(this._$cachedContent,{left:e.content}),(0,u.move)(this._$buttonsContainer,{left:e.buttonsContainer}),(0,u.move)(this._$buttons,{left:e.buttons})},_cacheItemData:function(e){e[0]!==this._cachedNode&&(this._$cachedContent=e.find("."+C),this._cachedItemWidth=e.outerWidth(),this._cachedButtonWidth=this._cachedButtonWidth||this._$buttons.outerWidth(),this._$buttonsContainer.width(this._cachedButtonWidth),this._$cachedContent.length&&(this._cachedNode=e[0]))},_minButtonContainerLeftOffset:function(){return this._cachedItemWidth-this._cachedButtonWidth},_swipeEndHandler:function(e,t){this._cacheItemData(e);var n=this._isRtlEnabled()?1:-1,i=this._cachedItemWidth*t.offset,o=!this._isReadyToDelete(e)&&i*n>.2*this._cachedButtonWidth,a=t.targetOffset===n&&o;return this._toggleDeleteReady(e,a),!0},_enablePositioning:function(e){g.default.stop(this._$cachedContent,!0),this.callBase.apply(this,arguments),this._$buttonsContainer.appendTo(e)},_disablePositioning:function(){this.callBase.apply(this,arguments),this._$buttonsContainer.detach()},_animatePrepareDeleteReady:function(){return this._animateToPositions(this._getPositions(1))},_animateForgetDeleteReady:function(e){return this._cacheItemData(e),this._animateToPositions(this._getPositions(0))},_animateToPositions:function(e){var t=this,n=this._getCurrentPositions(),i=Math.min(Math.abs(n.content-e.content)/this._cachedButtonWidth,1);return g.default.animate(this._$cachedContent,{from:n,to:e,easing:E,duration:O*i,strategy:"frame",draw:function(e){t._setPositions(e)}})},dispose:function(){this._menu&&this._menu.$element().remove(),this._$buttonsContainer&&this._$buttonsContainer.remove(),this.callBase.apply(this,arguments)}}).include(h.default))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=n(30),a=i(n(43)),r=n(125),s=i(n(165)),l=n(6);(0,r.register)("delete","swipe",s.default.inherit({_shouldHandleSwipe:!0,_renderItemPosition:function(e,t,n){var i=new l.Deferred,r=t*this._itemElementWidth;return n?a.default.animate(e,{to:{left:r},type:"slide",complete:function(){i.resolve(e,t)}}):((0,o.move)(e,{left:r}),i.resolve()),i.promise()},_swipeStartHandler:function(e){return this._itemElementWidth=e.width(),!0},_swipeUpdateHandler:function(e,t){return this._renderItemPosition(e,t.offset),!0},_swipeEndHandler:function(e,t){var n=t.targetOffset;return this._renderItemPosition(e,n,!0).done(function(e,t){Math.abs(t)&&this._list.deleteItem(e).fail(function(){this._renderItemPosition(e,0,!0)}.bind(this))}.bind(this)),!0}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(427)),r=i(n(10)),s=n(125),l=i(n(165)),u=i(n(68)),c=n(283),d="dx-list-context-menu",h="dx-list-context-menucontent";(0,s.register)("menu","context",l.default.inherit({_init:function(){var e=(0,o.default)("<div>").addClass(d);this._list.$element().append(e),this._menu=this._renderOverlay(e)},_renderOverlay:function(e){return this._list._createComponent(e,u.default,{shading:!1,deferRendering:!0,closeOnTargetScroll:!0,closeOnOutsideClick:function(e){return!(0,o.default)(e.target).closest("."+d).length},animation:{show:{type:"slide",duration:300,from:{height:0,opacity:1},to:{height:function(){return this._$menuList.outerHeight()}.bind(this),opacity:1}},hide:{type:"slide",duration:0,from:{opacity:1},to:{opacity:0}}},height:function(){return this._$menuList?this._$menuList.outerHeight():0}.bind(this),width:function(){return this._list.$element().outerWidth()}.bind(this),onContentReady:this._renderMenuContent.bind(this)})},_renderMenuContent:function(e){var t=e.component.$content(),n=this._menuItems().slice();this._deleteEnabled()&&n.push({text:r.default.format("dxListEditDecorator-delete"),action:this._deleteItem.bind(this)}),this._$menuList=(0,o.default)("<div>"),this._list._createComponent(this._$menuList,c.ListBase,{items:n,onItemClick:this._menuItemClickHandler.bind(this),height:"auto",integrationOptions:{}}),t.addClass(h),t.append(this._$menuList)},_menuItemClickHandler:function(e){this._menu.hide(),this._fireMenuAction(this._$itemWithMenu,e.itemData.action)},_deleteItem:function(){this._list.deleteItem(this._$itemWithMenu)},handleContextMenu:function(e){return this._$itemWithMenu=e,this._menu.option({position:{my:"top",at:"bottom",of:e,collision:"flip"}}),this._menu.show(),!0},dispose:function(){this._menu&&this._menu.$element().remove(),this.callBase.apply(this,arguments)}}).include(a.default))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(141)),r="dx-list-item-badge-container",s="dx-list-item-badge",l="dx-badge",u="dx-list-item-chevron-container",c="dx-list-item-chevron",d=a.default.inherit({_renderWatchers:function(){this.callBase(),this._startWatcher("badge",this._renderBadge.bind(this)),this._startWatcher("showChevron",this._renderShowChevron.bind(this))},_renderBadge:function(e){if(this._$element.children("."+r).remove(),e){var t=(0,o.default)("<div>").addClass(r).append((0,o.default)("<div>").addClass(s).addClass(l).text(e)),n=this._$element.children("."+u).first();n.length>0?t.insertBefore(n):t.appendTo(this._$element)}},_renderShowChevron:function(e){if(this._$element.children("."+u).remove(),e){var t=(0,o.default)("<div>").addClass(u),n=(0,o.default)("<div>").addClass(c);t.append(n).appendTo(this._$element)}}}),h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(13)),r=n(7),s=i(n(10)),l=i(n(9)),u=n(18),c=n(0),d=n(4),h=i(n(604)),f=i(n(605)),p=i(n(606)),g=i(n(109)),_=i(n(88)),m=n(36),v=i(n(193)),y="dx-scrollview",b=y+"-content",x=y+"-top-pocket",w=y+"-bottom-pocket",C=y+"-pull-down",k=y+"-scrollbottom",S=k+"-indicator",D=k+"-text",I=y+"-loadpanel",T={pullDown:h.default,swipeDown:f.default,simulated:p.default},O=!(0,r.hasWindow)(),E={finishLoading:d.noop,release:d.noop,refresh:d.noop,_optionChanged:function(e){if("onUpdated"!==e.name)return this.callBase.apply(this,arguments)}},A=g.default.inherit(O?E:{_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{pullingDownText:s.default.format("dxScrollView-pullingDownText"),pulledDownText:s.default.format("dxScrollView-pulledDownText"),refreshingText:s.default.format("dxScrollView-refreshingText"),reachBottomText:s.default.format("dxScrollView-reachBottomText"),onPullDown:null,onReachBottom:null,refreshStrategy:"pullDown"})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){var e=a.default.real();return"android"===e.platform},options:{refreshStrategy:"swipeDown"}},{device:function(){return(0,m.isMaterial)()},options:{pullingDownText:"",pulledDownText:"",refreshingText:"",reachBottomText:""}}])},_init:function(){this.callBase(),this._loadingIndicatorEnabled=!0},_initScrollableMarkup:function(){this.callBase(),this.$element().addClass(y),this._initContent(),this._initTopPocket(),this._initBottomPocket(),this._initLoadPanel()},_initContent:function(){var e=(0,o.default)("<div>").addClass(b);this._$content.wrapInner(e)},_initTopPocket:function(){var e=this._$topPocket=(0,o.default)("<div>").addClass(x),t=this._$pullDown=(0,o.default)("<div>").addClass(C);e.append(t),this._$content.prepend(e)},_initBottomPocket:function(){var e=this._$bottomPocket=(0,o.default)("<div>").addClass(w),t=this._$reachBottom=(0,o.default)("<div>").addClass(k),n=(0,o.default)("<div>").addClass(S),i=new _.default((0,o.default)("<div>")).$element(),a=this._$reachBottomText=(0,o.default)("<div>").addClass(D);this._updateReachBottomText(),t.append(n.append(i)).append(a),e.append(t),this._$content.append(e)},_initLoadPanel:function(){var e=(0,o.default)("<div>").addClass(I).appendTo(this.$element()),t={shading:!1,delay:400,message:this.option("refreshingText"),position:{of:this.$element()}};this._loadPanel=this._createComponent(e,v.default,t)},_updateReachBottomText:function(){this._$reachBottomText.text(this.option("reachBottomText"))},_createStrategy:function(){var e=this.option("useNative")?this.option("refreshStrategy"):"simulated",t=T[e];if(!t)throw Error("E1030",this.option("refreshStrategy"));this._strategy=new t(this),this._strategy.pullDownCallbacks.add(this._pullDownHandler.bind(this)),this._strategy.releaseCallbacks.add(this._releaseHandler.bind(this)),this._strategy.reachBottomCallbacks.add(this._reachBottomHandler.bind(this))},_createActions:function(){this.callBase(),this._pullDownAction=this._createActionByOption("onPullDown"),this._reachBottomAction=this._createActionByOption("onReachBottom"),this._tryRefreshPocketState()},_tryRefreshPocketState:function(){this._pullDownEnable(this.hasActionSubscription("onPullDown")),this._reachBottomEnable(this.hasActionSubscription("onReachBottom"))},on:function(e){var t=this.callBase.apply(this,arguments);return"pullDown"!==e&&"reachBottom"!==e||this._tryRefreshPocketState(),t},_pullDownEnable:function(e){return 0===arguments.length?this._pullDownEnabled:void(this._$pullDown&&this._strategy&&(this._$pullDown.toggle(e),this._strategy.pullDownEnable(e),this._pullDownEnabled=e))},_reachBottomEnable:function(e){return 0===arguments.length?this._reachBottomEnabled:void(this._$reachBottom&&this._strategy&&(this._$reachBottom.toggle(e),this._strategy.reachBottomEnable(e),this._reachBottomEnabled=e))},_pullDownHandler:function(){this._loadingIndicator(!1),this._pullDownLoading()},_loadingIndicator:function(e){return arguments.length<1?this._loadingIndicatorEnabled:void(this._loadingIndicatorEnabled=e)},_pullDownLoading:function(){this.startLoading(),this._pullDownAction()},_reachBottomHandler:function(){this._loadingIndicator(!1),this._reachBottomLoading()},_reachBottomLoading:function(){this.startLoading(),this._reachBottomAction()},_releaseHandler:function(){this.finishLoading(),this._loadingIndicator(!0)},_optionChanged:function(e){switch(e.name){case"onPullDown":case"onReachBottom":this._createActions();break;case"pullingDownText":case"pulledDownText":case"refreshingText":case"refreshStrategy":this._invalidate();break;case"reachBottomText":this._updateReachBottomText();break;default:this.callBase(e)}},isEmpty:function(){return!(0,o.default)(this.content()).children().length},content:function(){return(0,u.getPublicElement)(this._$content.children().eq(1))},release:function(e){return void 0!==e&&this.toggleLoading(!e),this._strategy.release()},toggleLoading:function(e){this._reachBottomEnable(e)},isFull:function(){return(0,o.default)(this.content()).height()>this._$container.height();
},refresh:function(){this.hasActionSubscription("onPullDown")&&(this._strategy.pendingRelease(),this._pullDownLoading())},startLoading:function(){this._loadingIndicator()&&this.$element().is(":visible")&&this._loadPanel.show(),this._lock()},finishLoading:function(){this._loadPanel.hide(),this._unlock()},_dispose:function(){this._strategy.dispose(),this.callBase(),this._loadPanel&&this._loadPanel.$element().remove()}});(0,l.default)("dxScrollView",A);var P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(27)),r=n(30),s=i(n(284)),l=i(n(88)),u=n(3),c=i(n(23)),d=n(6),h="dx-scrollview-pull-down-loading",f="dx-scrollview-pull-down-ready",p="dx-scrollview-pull-down-image",g="dx-scrollview-pull-down-indicator",_="dx-scrollview-pull-down-text",m="dx-scrollview-pull-down-text-visible",v=0,y=1,b=2,x=3,w=400,C=s.default.inherit({_init:function(e){this.callBase(e),this._$topPocket=e._$topPocket,this._$pullDown=e._$pullDown,this._$bottomPocket=e._$bottomPocket,this._$refreshingText=e._$refreshingText,this._$scrollViewContent=(0,o.default)(e.content()),this._initCallbacks()},_initCallbacks:function(){this.pullDownCallbacks=(0,a.default)(),this.releaseCallbacks=(0,a.default)(),this.reachBottomCallbacks=(0,a.default)()},render:function(){this.callBase(),this._renderPullDown(),this._releaseState()},_renderPullDown:function(){var e=(0,o.default)("<div>").addClass(p),t=(0,o.default)("<div>").addClass(g),n=new l.default((0,o.default)("<div>")).$element(),i=this._$pullDownText=(0,o.default)("<div>").addClass(_);this._$pullingDownText=(0,o.default)("<div>").text(this.option("pullingDownText")).appendTo(i),this._$pulledDownText=(0,o.default)("<div>").text(this.option("pulledDownText")).appendTo(i),this._$refreshingText=(0,o.default)("<div>").text(this.option("refreshingText")).appendTo(i),this._$pullDown.empty().append(e).append(t.append(n)).append(i)},_releaseState:function(){this._state=v,this._refreshPullDownText()},_pushBackFromBoundary:function(){this._isLocked()||this._component.isEmpty()||this.callBase()},_refreshPullDownText:function(){var e=this,t=[{element:this._$pullingDownText,visibleState:v},{element:this._$pulledDownText,visibleState:y},{element:this._$refreshingText,visibleState:b}];(0,u.each)(t,function(t,n){var i=e._state===n.visibleState?"addClass":"removeClass";n.element[i](m)})},update:function(){this.callBase(),this._setTopPocketOffset()},_updateDimensions:function(){this.callBase(),this._topPocketSize=this._$topPocket.height(),this._bottomPocketSize=this._$bottomPocket.height(),c.default.msie?this._scrollOffset=Math.round(100*(this._$container.height()-this._$content.height()))/100:this._scrollOffset=this._$container.height()-this._$content.height()},_allowedDirections:function(){var e=this.callBase();return e.vertical=e.vertical||this._pullDownEnabled,e},_setTopPocketOffset:function(){this._$topPocket.css({top:-this._topPocketSize})},handleEnd:function(){this.callBase(),this._complete()},handleStop:function(){this.callBase(),this._complete()},_complete:function(){this._state===y&&(this._setPullDownOffset(this._topPocketSize),clearTimeout(this._pullDownRefreshTimeout),this._pullDownRefreshTimeout=setTimeout(function(){this._pullDownRefreshing()}.bind(this),400))},_setPullDownOffset:function(e){(0,r.move)(this._$topPocket,{top:e}),(0,r.move)(this._$scrollViewContent,{top:e})},handleScroll:function(e){if(this.callBase(e),this._state!==b){var t=this.location().top,n=(this._location||0)-t;this._location=t,this._isPullDown()?this._pullDownReady():n>0&&this._isReachBottom()?this._reachBottom():this._stateReleased()}},_isPullDown:function(){return this._pullDownEnabled&&this._location>=this._topPocketSize},_isReachBottom:function(){return this._reachBottomEnabled&&this._location-(this._scrollOffset+this._bottomPocketSize)<=.5},_reachBottom:function(){this._state!==x&&(this._state=x,this.reachBottomCallbacks.fire())},_pullDownReady:function(){this._state!==y&&(this._state=y,this._$pullDown.addClass(f),this._refreshPullDownText())},_stateReleased:function(){this._state!==v&&(this._$pullDown.removeClass(h).removeClass(f),this._releaseState())},_pullDownRefreshing:function(){this._state!==b&&(this._state=b,this._$pullDown.addClass(h).removeClass(f),this._refreshPullDownText(),this.pullDownCallbacks.fire())},pullDownEnable:function(e){e&&(this._updateDimensions(),this._setTopPocketOffset()),this._pullDownEnabled=e},reachBottomEnable:function(e){this._reachBottomEnabled=e},pendingRelease:function(){this._state=y},release:function(){var e=new d.Deferred;return this._updateDimensions(),clearTimeout(this._releaseTimeout),this._state===x&&(this._state=v),this._releaseTimeout=setTimeout(function(){this._setPullDownOffset(0),this._stateReleased(),this.releaseCallbacks.fire(),this._updateAction(),e.resolve()}.bind(this),w),e.promise()},dispose:function(){clearTimeout(this._pullDownRefreshTimeout),clearTimeout(this._releaseTimeout),this.callBase()}}),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(27)),r=n(30),s=n(8),l=i(n(284)),u=i(n(88)),c=n(6),d="dx-scrollview-pull-down-loading",h="dx-scrollview-pull-down-indicator",f="dx-scrollview-pull-down-refreshing",p="dx-icon-pulldown",g=0,_=1,m=2,v=4,y=5,b=l.default.inherit({_init:function(e){this.callBase(e),this._$topPocket=e._$topPocket,this._$bottomPocket=e._$bottomPocket,this._$pullDown=e._$pullDown,this._$scrollViewContent=e.content(),this._initCallbacks(),this._location=0},_initCallbacks:function(){this.pullDownCallbacks=(0,a.default)(),this.releaseCallbacks=(0,a.default)(),this.reachBottomCallbacks=(0,a.default)()},render:function(){this.callBase(),this._renderPullDown(),this._releaseState()},_renderPullDown:function(){var e=(0,o.default)("<div>").addClass(h),t=new u.default((0,o.default)("<div>")).$element();this._$icon=(0,o.default)("<div>").addClass(p),this._$pullDown.empty().append(this._$icon).append(e.append(t))},_releaseState:function(){this._state=g,this._releasePullDown(),this._updateDimensions()},_releasePullDown:function(){this._$pullDown.css({opacity:0})},_updateDimensions:function(){this.callBase(),this._topPocketSize=this._$topPocket.height(),this._bottomPocketSize=this._$bottomPocket.height(),this._scrollOffset=this._$container.height()-this._$content.height()},_allowedDirections:function(){var e=this.callBase();return e.vertical=e.vertical||this._pullDownEnabled,e},handleInit:function(e){this.callBase(e),this._state===g&&0===this._location&&(this._startClientY=(0,s.eventData)(e.originalEvent).y,this._state=v)},handleMove:function(e){this.callBase(e),this._deltaY=(0,s.eventData)(e.originalEvent).y-this._startClientY,this._state===v&&(this._pullDownEnabled&&this._deltaY>0?this._state=y:this._complete()),this._state===y&&(e.preventDefault(),this._movePullDown())},_movePullDown:function(){var e=this._getPullDownHeight(),t=Math.min(3*e,this._deltaY+this._getPullDownStartPosition()),n=180*t/e/3;this._$pullDown.css({opacity:1}).toggleClass(f,t<e),(0,r.move)(this._$pullDown,{top:t}),this._$icon.css({transform:"rotate("+n+"deg)"})},_isPullDown:function(){return this._pullDownEnabled&&this._state===y&&this._deltaY>=this._getPullDownHeight()-this._getPullDownStartPosition()},_getPullDownHeight:function(){return Math.round(.05*this._$element.outerHeight())},_getPullDownStartPosition:function(){return-Math.round(1.5*this._$pullDown.outerHeight())},handleEnd:function(){this._isPullDown()&&this._pullDownRefreshing(),this._complete()},handleStop:function(){this._complete()},_complete:function(){this._state!==v&&this._state!==y||this._releaseState()},handleScroll:function(e){if(this.callBase(e),this._state!==m){var t=this.location().top,n=this._location-t;this._location=t,n>0&&this._isReachBottom()?this._reachBottom():this._stateReleased()}},_isReachBottom:function(){return this._reachBottomEnabled&&this._location<=this._scrollOffset+this._bottomPocketSize},_reachBottom:function(){this.reachBottomCallbacks.fire()},_stateReleased:function(){this._state!==g&&(this._$pullDown.removeClass(d),this._releaseState())},_pullDownRefreshing:function(){this._state=m,this._pullDownRefreshHandler()},_pullDownRefreshHandler:function(){this._refreshPullDown(),this.pullDownCallbacks.fire()},_refreshPullDown:function(){this._$pullDown.addClass(d),(0,r.move)(this._$pullDown,{top:this._getPullDownHeight()})},pullDownEnable:function(e){this._$topPocket.toggle(e),this._pullDownEnabled=e},reachBottomEnable:function(e){this._reachBottomEnabled=e},pendingRelease:function(){this._state=_},release:function(){var e=new c.Deferred;return this._updateDimensions(),clearTimeout(this._releaseTimeout),this._releaseTimeout=setTimeout(function(){this._stateReleased(),this.releaseCallbacks.fire(),this._updateAction(),e.resolve()}.bind(this),800),e.promise()},dispose:function(){clearTimeout(this._pullDownRefreshTimeout),clearTimeout(this._releaseTimeout),this.callBase()}}),x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(27)),r=n(3),s=n(4),l=n(0),u=n(429),c=i(n(88)),d=Math,h="dx-scrollview-pull-down-loading",f="dx-scrollview-pull-down-ready",p="dx-scrollview-pull-down-image",g="dx-scrollview-pull-down-indicator",_="dx-scrollview-pull-down-text",m="dx-scrollview-pull-down-text-visible",v=0,y=1,b=2,x=3,w=u.Scroller.inherit({ctor:function(){this._topPocketSize=0,this.callBase.apply(this,arguments),this._initCallbacks(),this._releaseState()},_releaseState:function(){this._state=v,this._refreshPullDownText()},_refreshPullDownText:function(){var e=this,t=[{element:this._$pullingDownText,visibleState:v},{element:this._$pulledDownText,visibleState:y},{element:this._$refreshingText,visibleState:b}];(0,r.each)(t,function(t,n){var i=e._state===n.visibleState?"addClass":"removeClass";n.element[i](m)})},_initCallbacks:function(){this.pullDownCallbacks=(0,a.default)(),this.releaseCallbacks=(0,a.default)(),this.reachBottomCallbacks=(0,a.default)()},_updateBounds:function(){var e="horizontal"!==this._direction;this._topPocketSize=e?Math.round(this._$topPocket[this._dimension]()):0,this._bottomPocketSize=e?Math.round(this._$bottomPocket[this._dimension]()):0,this.callBase(),this._bottomBound=this._minOffset+this._bottomPocketSize},_updateScrollbar:function(){this._scrollbar.option({containerSize:this._containerSize(),contentSize:this._contentSize()-this._topPocketSize-this._bottomPocketSize,scaleRatio:this._getScaleRatio()})},_moveContent:function(){this.callBase(),this._isPullDown()?this._pullDownReady():this._isReachBottom()?this._reachBottomReady():this._state!==v&&this._stateReleased()},_moveScrollbar:function(){this._scrollbar.moveTo(this._topPocketSize+this._location)},_isPullDown:function(){return this._pullDownEnabled&&this._location>=0},_isReachBottom:function(){return this._reachBottomEnabled&&this._location-this._bottomBound<=.5},_scrollComplete:function(){this._inBounds()&&this._state===y?this._pullDownRefreshing():this._inBounds()&&this._state===x?this._reachBottomLoading():this.callBase()},_reachBottomReady:function(){this._state!==x&&(this._state=x,this._minOffset=this._getMinOffset())},_getMaxOffset:function(){return-this._topPocketSize},_getMinOffset:function(){return d.min(this.callBase(),-this._topPocketSize)},_reachBottomLoading:function(){this.reachBottomCallbacks.fire()},_pullDownReady:function(){this._state!==y&&(this._state=y,this._maxOffset=0,this._$pullDown.addClass(f),this._refreshPullDownText())},_stateReleased:function(){this._state!==v&&(this._releaseState(),this._updateBounds(),this._$pullDown.removeClass(h).removeClass(f),this.releaseCallbacks.fire())},_pullDownRefreshing:function(){this._state!==b&&(this._state=b,this._$pullDown.addClass(h).removeClass(f),this._refreshPullDownText(),this.pullDownCallbacks.fire())},_releaseHandler:function(){return this._state===v&&this._moveToBounds(),this._update(),this._releaseTask&&this._releaseTask.abort(),this._releaseTask=(0,s.executeAsync)(this._release.bind(this)),this._releaseTask.promise},_release:function(){this._stateReleased(),this._scrollComplete()},_reachBottomEnablingHandler:function(e){this._reachBottomEnabled!==e&&(this._reachBottomEnabled=e,this._updateBounds())},_pullDownEnablingHandler:function(e){this._pullDownEnabled!==e&&(this._pullDownEnabled=e,this._considerTopPocketChange(),this._updateHandler())},_considerTopPocketChange:function(){this._location-=this._$topPocket.height()||-this._topPocketSize,this._maxOffset=0,this._move()},_pendingReleaseHandler:function(){this._state=y},dispose:function(){this._releaseTask&&this._releaseTask.abort(),this.callBase()}}),C=u.SimulatedStrategy.inherit({_init:function(e){this.callBase(e),this._$pullDown=e._$pullDown,this._$topPocket=e._$topPocket,this._$bottomPocket=e._$bottomPocket,this._initCallbacks()},_initCallbacks:function(){this.pullDownCallbacks=(0,a.default)(),this.releaseCallbacks=(0,a.default)(),this.reachBottomCallbacks=(0,a.default)()},render:function(){this._renderPullDown(),this.callBase()},_renderPullDown:function(){var e=(0,o.default)("<div>").addClass(p),t=(0,o.default)("<div>").addClass(g),n=new c.default((0,o.default)("<div>")).$element(),i=this._$pullDownText=(0,o.default)("<div>").addClass(_);this._$pullingDownText=(0,o.default)("<div>").text(this.option("pullingDownText")).appendTo(i),this._$pulledDownText=(0,o.default)("<div>").text(this.option("pulledDownText")).appendTo(i),this._$refreshingText=(0,o.default)("<div>").text(this.option("refreshingText")).appendTo(i),this._$pullDown.empty().append(e).append(t.append(n)).append(i)},pullDownEnable:function(e){this._eventHandler("pullDownEnabling",e)},reachBottomEnable:function(e){this._eventHandler("reachBottomEnabling",e)},_createScroller:function(e){var t=this,n=t._scrollers[e]=new w(t._scrollerOptions(e));n.pullDownCallbacks.add(function(){t.pullDownCallbacks.fire()}),n.releaseCallbacks.add(function(){t.releaseCallbacks.fire()}),n.reachBottomCallbacks.add(function(){t.reachBottomCallbacks.fire()})},_scrollerOptions:function(e){return(0,l.extend)(this.callBase(e),{$topPocket:this._$topPocket,$bottomPocket:this._$bottomPocket,$pullDown:this._$pullDown,$pullDownText:this._$pullDownText,$pullingDownText:this._$pullingDownText,$pulledDownText:this._$pulledDownText,$refreshingText:this._$refreshingText})},pendingRelease:function(){this._eventHandler("pendingRelease")},release:function(){return this._eventHandler("release").done(this._updateAction)},location:function e(){var e=this.callBase();return e.top+=this._$topPocket.height(),e},dispose:function(){(0,r.each)(this._scrollers,function(){this.dispose()}),this.callBase()}}),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(5)),r=n(21),s=n(0),l=i(n(17)),u=i(n(144)),c=i(n(608)),d=n(8),h=n(125),f=i(n(165)),p="dx-list-select-decorator-enabled",g="dx-list-select-all",_="dx-list-select-all-checkbox",m="dx-list-select-all-label",v="dx-list-select-checkbox-container",y="dx-list-select-checkbox",b="dx-list-select-radiobutton-container",x="dx-list-select-radiobutton",w="dx-state-focused",C=(0,d.addNamespace)(r.name,"dxListEditDecorator");(0,h.register)("selection","default",f.default.inherit({_init:function(){this.callBase.apply(this,arguments);var e=this._list.option("selectionMode");this._singleStrategy="single"===e,this._containerClass=this._singleStrategy?b:v,this._controlClass=this._singleStrategy?x:y,this._controlWidget=this._singleStrategy?c.default:u.default,this._list.$element().addClass(p)},beforeBag:function(e){var t=e.$itemElement,n=e.$container.addClass(this._containerClass),i=(0,o.default)("<div>").addClass(this._controlClass).appendTo(n);new this._controlWidget(i,(0,s.extend)(this._commonOptions(),{value:this._isSelected(t),focusStateEnabled:!1,hoverStateEnabled:!1,onValueChanged:function(e){e.event&&this._list._saveSelectionChangeEvent(e.event),this._processCheckedState(t,e.value),e.event&&e.event.stopPropagation()}.bind(this)}))},modifyElement:function(e){this.callBase.apply(this,arguments);var t=e.$itemElement,n=this._controlWidget.getInstance(t.find("."+this._controlClass));a.default.on(t,"stateChanged",function(e,t){n.option("value",t)}.bind(this))},_updateSelectAllState:function(){this._$selectAll&&this._selectAllCheckBox.option("value",this._list.isSelectAll())},afterRender:function(){"all"===this._list.option("selectionMode")&&(this._$selectAll?this._updateSelectAllState():this._renderSelectAll())},handleKeyboardEvents:function(e,t){var n=!t,i=this._list,o=this._$selectAll,a=i._getLastItemIndex(),r=t&&0===e||n&&e===a,s=!!o;return!(!s||!r)&&(i.option("focusedElement",o),i.scrollToItem(i.option("focusedElement")),!0)},handleEnterPressing:function(){if(this._$selectAll&&this._$selectAll.hasClass(w))return this._selectAllCheckBox.option("value",!this._selectAllCheckBox.option("value")),!0},_renderSelectAll:function(){var e=this._$selectAll=(0,o.default)("<div>").addClass(g),t=this._list,n=t._supportedKeys().downArrow.bind(t);this._selectAllCheckBox=t._createComponent((0,o.default)("<div>").addClass(_).appendTo(e),u.default,{focusStateEnabled:!1,hoverStateEnabled:!1}),this._selectAllCheckBox.registerKeyHandler("downArrow",n),(0,o.default)("<div>").addClass(m).text(this._list.option("selectAllText")).appendTo(e),this._list.itemsContainer().prepend(e),this._updateSelectAllState(),this._attachSelectAllHandler()},_attachSelectAllHandler:function(){this._selectAllCheckBox.option("onValueChanged",this._selectAllHandler.bind(this)),a.default.off(this._$selectAll,C),a.default.on(this._$selectAll,C,this._selectAllClickHandler.bind(this))},_selectAllHandler:function(e){e.event&&e.event.stopPropagation();var t=this._selectAllCheckBox.option("value"),n=this._list._createActionByOption("onSelectAllValueChanged")({value:t});n!==!1&&(e.event&&this._list._saveSelectionChangeEvent(e.event),t===!0?this._selectAllItems():t===!1&&this._unselectAllItems())},_checkSelectAllCapability:function(){var e=this._list,t=e.getDataSource();return!!("allPages"!==e.option("selectAllMode")||!e.option("grouped")||t&&t.group())||(l.default.log("W1010"),!1)},_selectAllItems:function(){this._checkSelectAllCapability()&&this._list._selection.selectAll("page"===this._list.option("selectAllMode"))},_unselectAllItems:function(){this._checkSelectAllCapability()&&this._list._selection.deselectAll("page"===this._list.option("selectAllMode"))},_selectAllClickHandler:function(e){this._list._saveSelectionChangeEvent(e),this._selectAllCheckBox.option("value",!this._selectAllCheckBox.option("value"))},_isSelected:function(e){return this._list.isItemSelected(e)},_processCheckedState:function(e,t){t?this._list.selectItem(e):this._list.unselectItem(e)},dispose:function(){this._disposeSelectAll(),this._list.$element().removeClass(p),this.callBase.apply(this,arguments)},_disposeSelectAll:function(){this._$selectAll&&(this._$selectAll.remove(),this._$selectAll=null)}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(13)),s=n(0),l=n(87),u=i(n(9)),c=i(n(52)),d=n(8),h=n(21),f="dx-radiobutton",p="dx-radiobutton-icon",g="dx-radiobutton-icon-dot",_="dx-radiobutton-checked",m="dx-radiobutton-icon-checked",v=c.default.inherit({_supportedKeys:function(){var e=function(e){e.preventDefault(),this._clickAction({event:e})};return(0,s.extend)(this.callBase(),{space:e})},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{hoverStateEnabled:!0,activeStateEnabled:!0,value:!1,useInkRipple:!1})},_canValueBeChangedByClick:function(){return!0},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===r.default.real().deviceType&&!r.default.isSimulator()},options:{focusStateEnabled:!0}}])},_init:function(){this.callBase(),this.$element().addClass(f)},_initMarkup:function(){this.callBase(),this._renderIcon(),this.option("useInkRipple")&&this._renderInkRipple(),this._renderCheckedState(this.option("value")),this._renderClick(),this.setAria("role","radio")},_renderInkRipple:function(){this._inkRipple=(0,l.render)({waveSizeCoefficient:3.3,useHoldAnimation:!1,wavesNumber:2,isCentered:!0})},_renderInkWave:function(e,t,n,i){if(this._inkRipple){var o={element:e,event:t,wave:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}},_updateFocusState:function(e,t){this.callBase.apply(this,arguments),this._renderInkWave(this._$icon,e,t,0)},_toggleActiveState:function(e,t,n){this.callBase.apply(this,arguments),this._renderInkWave(this._$icon,n,t,1)},_renderIcon:function(){this._$icon=(0,o.default)("<div>").addClass(p),(0,o.default)("<div>").addClass(g).appendTo(this._$icon),this.$element().append(this._$icon)},_renderCheckedState:function(e){this.$element().toggleClass(_,e).find("."+p).toggleClass(m,e),this.setAria("checked",e)},_renderClick:function(){var e=(0,d.addNamespace)(h.name,this.NAME);this._clickAction=this._createAction(function(e){this._clickHandler(e.event)}.bind(this)),a.default.off(this.$element(),e),a.default.on(this.$element(),e,function(e){this._clickAction({event:e})}.bind(this))},_clickHandler:function(e){this._saveValueChangeEvent(e),this.option("value",!0)},_optionChanged:function(e){switch(e.name){case"useInkRipple":this._invalidate();break;case"value":this._renderCheckedState(e.value),this.callBase(e);break;default:this.callBase(e)}},_clean:function(){delete this._inkRipple,this.callBase()}});(0,u.default)("dxRadioButton",v);var y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(5)),r=n(0),s=n(8),l=n(125),u=i(n(165)),c=i(n(286)),d="dx-list-reorder-handle-container",h="dx-list-reorder-handle",f="dx-list-item-ghost-reordering",p="dx-state-hover";(0,l.register)("reorder","default",u.default.inherit({_init:function(){var e=this._list;this._groupedEnabled=this._list.option("grouped"),this._lockedDrag=!1;var t=this._groupedEnabled?"> .dx-list-group > .dx-list-group-body > .dx-list-item":"> .dx-list-item";this._sortable=e._createComponent(e._scrollView.content(),c.default,(0,r.extend)({component:e,contentTemplate:null,allowReordering:!1,filter:t,container:e.$element(),dragDirection:e.option("itemDragging.group")?"both":"vertical",handle:".".concat(h),dragTemplate:this._dragTemplate,onDragStart:this._dragStartHandler.bind(this),onDragChange:this._dragChangeHandler.bind(this),onReorder:this._reorderHandler.bind(this)},e.option("itemDragging")))},afterRender:function(){this._sortable.update()},_dragTemplate:function(e){return(0,o.default)(e.itemElement).clone().width((0,o.default)(e.itemElement).width()).addClass(f).addClass(p)},_dragStartHandler:function(e){if(this._lockedDrag)return void(e.cancel=!0)},_dragChangeHandler:function(e){if(this._groupedEnabled&&!this._sameParent(e.fromIndex,e.toIndex))return void(e.cancel=!0)},_sameParent:function(e,t){var n=this._list.getItemElementByFlatIndex(e),i=this._list.getItemElementByFlatIndex(t);return i.parent().get(0)===n.parent().get(0)},_reorderHandler:function(e){var t=this._list.getItemElementByFlatIndex(e.toIndex);this._list.reorderItem((0,o.default)(e.itemElement),t)},afterBag:function(e){var t=this,n=(0,o.default)("<div>").addClass(h);a.default.on(n,"dxpointerdown",function(e){t._lockedDrag=!(0,s.isMouseEvent)(e)}),a.default.on(n,"dxhold",{timeout:30},function(e){e.cancel=!0,t._lockedDrag=!1}),e.$container.addClass(d).append(n)}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(35)),r=i(n(9)),s=n(4),l=n(1),u=n(29),c=n(0),d=i(n(38)),h=i(n(52)),f=i(n(179)),p=i(n(611)),g=i(n(612)),_=n(30),m=i(n(23)),v=i(n(20)),y=i(n(62)),b=i(n(13)),x=i(n(43)),w=n(7),C=i(n(10)),k=n(118),S="dx-calendar",D="dx-calendar-body",I="dx-calendar-cell",T="dx-calendar-footer",O="dx-calendar-today-button",E="dx-calendar-with-footer",A="dx-calendar-views-wrapper",P="dx-calendar-view",R="dx-state-focused",M=250,B=.6,V=1,F="yyyy-MM-dd",L="dxDateValueKey",H={month:3,year:2,decade:1,century:0},z={MONTH:"month",YEAR:"year",DECADE:"decade",CENTURY:"century"},N=m.default.msie&&parseInt(m.default.version)<=11,j=h.default.inherit({_activeStateUnit:"."+I,_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{hoverStateEnabled:!0,activeStateEnabled:!0,currentDate:new Date,value:null,dateSerializationFormat:void 0,min:new Date(1e3,0),max:new Date(3e3,0),firstDayOfWeek:void 0,zoomLevel:z.MONTH,maxZoomLevel:z.MONTH,minZoomLevel:z.CENTURY,showTodayButton:!1,cellTemplate:"cell",disabledDates:null,onCellClick:null,onContouredChanged:null,hasFocus:function(e){return e.hasClass(R)},_todayDate:function(){return new Date}})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===b.default.real().deviceType&&!b.default.isSimulator()},options:{focusStateEnabled:!0}}])},_supportedKeys:function(){return(0,c.extend)(this.callBase(),{rightArrow:function(e){e.preventDefault(),e.ctrlKey?this._waitRenderView(1):this._moveCurrentDateByOffset(1*this._getRtlCorrection())},leftArrow:function(e){e.preventDefault(),e.ctrlKey?this._waitRenderView(-1):this._moveCurrentDateByOffset(-1*this._getRtlCorrection())},upArrow:function(e){if(e.preventDefault(),e.ctrlKey)this._navigateUp();else{if(x.default.isAnimating(this._view.$element()))return;this._moveCurrentDateByOffset(-1*this._view.option("colCount"))}},downArrow:function(e){if(e.preventDefault(),e.ctrlKey)this._navigateDown();else{if(x.default.isAnimating(this._view.$element()))return;this._moveCurrentDateByOffset(1*this._view.option("colCount"))}},home:function(e){e.preventDefault();var t=this.option("zoomLevel"),n=this.option("currentDate"),i=this._dateOption("min");if(!this._view.isDateDisabled(n)){var o=v.default.sameView(t,n,i)?i:v.default.getViewFirstCellDate(t,n);this._moveToClosestAvailableDate(o)}},end:function(e){e.preventDefault();var t=this.option("zoomLevel"),n=this.option("currentDate"),i=this._dateOption("max");if(!this._view.isDateDisabled(n)){var o=v.default.sameView(t,n,i)?i:v.default.getViewLastCellDate(t,n);this._moveToClosestAvailableDate(o)}},pageUp:function(e){e.preventDefault(),this._waitRenderView(-1*this._getRtlCorrection())},pageDown:function(e){e.preventDefault(),this._waitRenderView(1*this._getRtlCorrection())},tab:s.noop,enter:function(e){if(this._isMaxZoomLevel()){if(!this._view.isDateDisabled(this.option("currentDate"))){var t=this._updateTimeComponent(this.option("currentDate"));this._dateValue(t,e)}}else this._navigateDown()}})},_getSerializationFormat:function(e){var t=this.option(e||"value");if(this.option("dateSerializationFormat"))return this.option("dateSerializationFormat");if((0,l.isNumeric)(t))return"number";if((0,l.isString)(t))return y.default.getDateSerializationFormat(t)},_convertToDate:function(e,t){return y.default.deserializeDate(e)},_dateValue:function(e,t){t&&this._saveValueChangeEvent(t),this._dateOption("value",e)},_dateOption:function(e,t){if(1===arguments.length)return this._convertToDate(this.option(e),e);var n=this._getSerializationFormat(e);this.option(e,y.default.serializeDate(t,n))},_shiftDate:function(e,t,n,i){switch(e){case z.MONTH:t.setDate(t.getDate()+n*i);break;case z.YEAR:t.setMonth(t.getMonth()+n*i);break;case z.DECADE:t.setFullYear(t.getFullYear()+n*i);break;case z.CENTURY:t.setFullYear(t.getFullYear()+10*n*i)}},_moveCurrentDateByOffset:function(e){var t=this.option("currentDate"),n=new Date(t),i=this.option("zoomLevel");this._shiftDate(i,n,e,1);for(var o=this._getMaxDate(),a=this._getMinDate(),r=this._areDatesInNeighborView(i,n,t),s=(0,u.inRange)(n,a,o)&&r,l=new Date(n);s;){if(s&&!this._view.isDateDisabled(l)){n=l;break}this._shiftDate(i,l,e,1),r=this._areDatesInNeighborView(i,l,t),s=(0,u.inRange)(l,a,o)&&r}this._view.isDateDisabled(t)||this._view.isDateDisabled(n)?this._waitRenderView(e>0?1:-1):this.option("currentDate",n)},_areDatesInSameView:function(e,t,n){switch(e){case z.MONTH:return t.getMonth()===n.getMonth();case z.YEAR:return t.getYear()===n.getYear();case z.DECADE:return parseInt(t.getYear()/10)===parseInt(n.getYear()/10);case z.CENTURY:return parseInt(t.getYear()/100)===parseInt(n.getYear()/100)}},_areDatesInNeighborView:function(e,t,n){var i=function(e,t){var n=Math.abs(e-t);return Math.min(n,12-n)};switch(e){case z.MONTH:return i(t.getMonth(),n.getMonth())<=1;case z.YEAR:return Math.abs(t.getYear()-n.getYear())<=1;case z.DECADE:return Math.abs(t.getYear()-n.getYear())<=10;case z.CENTURY:return Math.abs(t.getYear()-n.getYear())<=100}},_moveToClosestAvailableDate:function(){var e,t,n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.option("currentDate"),i=v.default.createDate(n),o=this.option("zoomLevel"),a=!this._isDateNotAvailable(i),r=a,s=a,l=v.default.createDate(i),u=v.default.createDate(i);do{if(r){i=l;break}if(s){i=u;break}this._shiftDate(o,l,1,1),this._shiftDate(o,u,1,-1),e=this._areDatesInSameView(o,l,n),t=this._areDatesInSameView(o,u,n),r=e&&!this._isDateNotAvailable(l),s=t&&!this._isDateNotAvailable(u)}while(e||t);this.option("currentDate",i)},_isDateNotAvailable:function(e){var t=this._getMaxDate(),n=this._getMinDate();return!(0,u.inRange)(e,n,t)||this._view.isDateDisabled(e)},_init:function(){this.callBase(),this._correctZoomLevel(),this._initCurrentDate(),this._initActions()},_correctZoomLevel:function(){var e=this.option("minZoomLevel"),t=this.option("maxZoomLevel"),n=this.option("zoomLevel");H[t]<H[e]||(H[n]>H[t]?this.option("zoomLevel",t):H[n]<H[e]&&this.option("zoomLevel",e))},_initCurrentDate:function(){var e=this._getNormalizedDate(this._dateOption("value"))||this._getNormalizedDate(this.option("currentDate"));this.option("currentDate",e)},_getNormalizedDate:function(e){return e=v.default.normalizeDate(e,this._getMinDate(),this._getMaxDate()),(0,l.isDefined)(e)?this._getDate(e):e},_initActions:function(){this._cellClickAction=this._createActionByOption("onCellClick"),this._onContouredChanged=this._createActionByOption("onContouredChanged")},_initTemplates:function(){this._templateManager.addDefaultTemplates({cell:new k.FunctionTemplate(function(e){var t=e.model;(0,o.default)(e.container).append((0,o.default)("<span>").text(t&&t.text||String(t)))})}),this.callBase()},_updateCurrentDate:function(e){x.default.isAnimating(this._$viewsWrapper)&&x.default.stop(this._$viewsWrapper,!0);var t=this._getMinDate(),n=this._getMaxDate();if(t>n)return void this.option("currentDate",new Date);var i=this._getNormalizedDate(e);if(e.getTime()!==i.getTime())return void this.option("currentDate",v.default.createDate(i));var o=this._getViewsOffset(this._view.option("date"),i);0!==o&&!this._isMaxZoomLevel()&&this._isOtherViewCellClicked&&(o=0),this._view&&0!==o&&!this._suppressNavigation?this._navigate(o,i):(this._renderNavigator(),this._setViewContoured(i),this._updateAriaId(i))},_setViewContoured:function(e){this.option("hasFocus")(this._focusTarget())&&this._view.option("contouredDate",e)},_getMinDate:function(){return this.min?this.min:(this.min=this._dateOption("min")||new Date(1e3,0),this.min)},_getMaxDate:function(){return this.max?this.max:(this.max=this._dateOption("max")||new Date(3e3,0),this.max)},_getViewsOffset:function(e,t){var n=this.option("zoomLevel");if(n===z.MONTH)return this._getMonthsOffset(e,t);var i;switch(n){case z.CENTURY:i=100;break;case z.DECADE:i=10;break;default:i=1}return parseInt(t.getFullYear()/i)-parseInt(e.getFullYear()/i)},_getMonthsOffset:function(e,t){var n=t.getFullYear()-e.getFullYear(),i=t.getMonth()-e.getMonth();return 12*n+i},_waitRenderView:function(e){var t=this;if(!this._alreadyViewRender){this._alreadyViewRender=!0;var n=this._getDateByOffset(e*this._getRtlCorrection());this._moveToClosestAvailableDate(n),this._waitRenderViewTimeout=setTimeout(function(){t._alreadyViewRender=!1})}},_getRtlCorrection:function(){return this.option("rtlEnabled")?-1:1},_getDateByOffset:function(e,t){t=this._getDate(t||this.option("currentDate"));
var n=t.getDate(),i=v.default.getDifferenceInMonth(this.option("zoomLevel"))*e;t.setDate(1),t.setMonth(t.getMonth()+i);var o=v.default.getLastMonthDate(t).getDate();return t.setDate(n>o?o:n),t},_focusTarget:function(){return this.$element()},_initMarkup:function(){this._renderSubmitElement(),this.callBase();var e=this.$element();e.addClass(S),this._renderBody(),e.append(this.$body),this._renderViews(),this._renderNavigator(),e.append(this._navigator.$element()),this._renderSwipeable(),this._renderFooter(),this._updateAriaSelected(),this._updateAriaId(),this._moveToClosestAvailableDate()},_render:function(){this.callBase(),this._setViewContoured(this.option("currentDate"))},_renderBody:function(){this._$viewsWrapper||(this.$body=(0,o.default)("<div>").addClass(D),this._$viewsWrapper=(0,o.default)("<div>").addClass(A),this.$body.append(this._$viewsWrapper))},_getKeyboardListeners:function(){return this.callBase().concat([this._view])},_renderViews:function(){this.$element().addClass(P+"-"+this.option("zoomLevel"));var e=this.option("currentDate");if(this._view=this._renderSpecificView(e),(0,w.hasWindow)()){var t=this._getDateByOffset(-1,e);this._beforeView=this._isViewAvailable(t)?this._renderSpecificView(t):null;var n=this._getDateByOffset(1,e);n.setDate(1),this._afterView=this._isViewAvailable(n)?this._renderSpecificView(n):null}this._translateViews()},_renderSpecificView:function(e){var t=g.default[this.option("zoomLevel")],n=(0,o.default)("<div>").appendTo(this._$viewsWrapper),i=this._viewConfig(e);return this._createComponent(n,t,i)},_viewConfig:function(e){var t=this.option("disabledDates");return t=(0,l.isFunction)(t)?this._injectComponent(t.bind(this)):t,{date:e,min:this._getMinDate(),max:this._getMaxDate(),firstDayOfWeek:this.option("firstDayOfWeek"),value:this._dateOption("value"),tabIndex:void 0,focusStateEnabled:this.option("focusStateEnabled"),hoverStateEnabled:this.option("hoverStateEnabled"),disabledDates:t,onCellClick:this._cellClickHandler.bind(this),cellTemplate:this._getTemplateByOption("cellTemplate"),allowValueSelection:this._isMaxZoomLevel(),_todayDate:this.option("_todayDate")}},_injectComponent:function(e){var t=this;return function(n){return(0,c.extend)(n,{component:t}),e(n)}},_isViewAvailable:function(e){var t=this.option("zoomLevel"),n=v.default.getViewMinBoundaryDate(t,this._getMinDate()),i=v.default.getViewMaxBoundaryDate(t,this._getMaxDate());return v.default.dateInRange(e,n,i)},_translateViews:function(){(0,_.move)(this._view.$element(),{left:0,top:0}),this._beforeView&&(0,_.move)(this._beforeView.$element(),{left:this._getViewPosition(-1),top:0}),this._afterView&&(0,_.move)(this._afterView.$element(),{left:this._getViewPosition(1),top:0})},_getViewPosition:function(e){var t=this.option("rtlEnabled")&&!m.default.msie?-1:1;return 100*e*t+"%"},_cellClickHandler:function(e){var t=this.option("zoomLevel"),n=v.default.getViewDown(t),i=this._isMaxZoomLevel();if(n&&!i)this._navigateDown(e.event.currentTarget);else{var o=this._updateTimeComponent(e.value);this._dateValue(o,e.event),this._cellClickAction(e)}},_updateTimeComponent:function(e){var t=v.default.createDate(e),n=this._dateOption("value");return n&&(t.setHours(n.getHours()),t.setMinutes(n.getMinutes()),t.setSeconds(n.getSeconds()),t.setMilliseconds(n.getMilliseconds())),t},_isMaxZoomLevel:function(){return this.option("zoomLevel")===this.option("maxZoomLevel")},_navigateDown:function(e){var t=this.option("zoomLevel");if(!this._isMaxZoomLevel()){var n=v.default.getViewDown(t);if(n){var i=this._view.option("contouredDate")||this._view.option("date");e&&(i=(0,o.default)(e).data(L)),this._isOtherViewCellClicked=!0,this.option("currentDate",i),this.option("zoomLevel",n),this._isOtherViewCellClicked=!1,this._renderNavigator(),this._animateShowView(),this._moveToClosestAvailableDate(),this._setViewContoured(this._getNormalizedDate(this.option("currentDate")))}}},_renderNavigator:function(){this._navigator||(this._navigator=new p.default((0,o.default)("<div>"),this._navigatorConfig())),this._navigator.option("text",this._view.getNavigatorCaption()),this._updateButtonsVisibility()},_navigatorConfig:function(){return{text:this._view.getNavigatorCaption(),onClick:this._navigatorClickHandler.bind(this),onCaptionClick:this._navigateUp.bind(this),rtlEnabled:this.option("rtlEnabled")}},_navigatorClickHandler:function(e){var t=this._getDateByOffset(e.direction,this.option("currentDate"));this._moveToClosestAvailableDate(t)},_navigateUp:function(){var e=this.option("zoomLevel"),t=v.default.getViewUp(e);t&&!this._isMinZoomLevel(e)&&(this.option("zoomLevel",t),this._renderNavigator(),this._animateShowView(),this._moveToClosestAvailableDate(),this._setViewContoured(this._getNormalizedDate(this.option("currentDate"))))},_isMinZoomLevel:function(e){var t=this._getMinDate(),n=this._getMaxDate();return v.default.sameView(e,t,n)||this.option("minZoomLevel")===e},_updateButtonsVisibility:function(){this._navigator.toggleButton("next",!(0,l.isDefined)(this._getRequiredView("next"))),this._navigator.toggleButton("prev",!(0,l.isDefined)(this._getRequiredView("prev")))},_renderSwipeable:function(){this._swipeable||(this._swipeable=this._createComponent(this.$element(),f.default,{onStart:this._swipeStartHandler.bind(this),onUpdated:this._swipeUpdateHandler.bind(this),onEnd:this._swipeEndHandler.bind(this),itemSizeFunc:this._viewWidth.bind(this)}))},_swipeStartHandler:function(e){x.default.stop(this._$viewsWrapper,!0),e.event.maxLeftOffset=this._getRequiredView("next")?1:0,e.event.maxRightOffset=this._getRequiredView("prev")?1:0},_getRequiredView:function(e){var t,n=this.option("rtlEnabled");return"next"===e?t=n?this._beforeView:this._afterView:"prev"===e&&(t=n?this._afterView:this._beforeView),t},_swipeUpdateHandler:function(e){var t=e.event.offset;(0,_.move)(this._$viewsWrapper,{left:t*this._viewWidth(),top:0}),this._updateNavigatorCaption(t)},_swipeEndHandler:function(e){var t=e.event.targetOffset,n=t?t/Math.abs(t):0;if(0===n)return void this._animateWrapper(0,M);var i=this._getDateByOffset(-n*this._getRtlCorrection());this._isDateInInvalidRange(i)&&(i=n>=0?new Date(this._getMinDate()):new Date(this._getMaxDate())),this.option("currentDate",i)},_viewWidth:function(){return this._viewWidthValue||(this._viewWidthValue=this.$element().width()),this._viewWidthValue},_updateNavigatorCaption:function(e){e*=this._getRtlCorrection();var t=this._view;e>.5&&this._beforeView?t=this._beforeView:e<-.5&&this._afterView&&(t=this._afterView),this._navigator.option("text",t.getNavigatorCaption())},_isDateInInvalidRange:function(e){if(!this._view.isBoundary(e)){var t=this._getMinDate(),n=this._getMaxDate(),i=v.default.normalizeDate(e,t,n);return i===t||i===n}},_renderFooter:function(){var e=this.option("showTodayButton");if(e){var t=this._createComponent((0,o.default)("<a>"),d.default,{focusStateEnabled:!1,text:C.default.format("dxCalendar-todayButtonText"),onClick:function(){this._toTodayView()}.bind(this),integrationOptions:{}}).$element().addClass(O);this._$footer=(0,o.default)("<div>").addClass(T).append(t),this.$element().append(this._$footer)}this.$element().toggleClass(E,e)},_renderSubmitElement:function(){this._$submitElement=(0,o.default)("<input>").attr("type","hidden").appendTo(this.$element()),this._setSubmitValue(this.option("value"))},_setSubmitValue:function(e){var t=this._convertToDate(e);this._getSubmitElement().val(y.default.serializeDate(t,F))},_getSubmitElement:function(){return this._$submitElement},_animateShowView:function(){return x.default.stop(this._view.$element(),!0),this._popAnimationView(this._view,B,V,M).promise()},_popAnimationView:function(e,t,n,i){return x.default.animate(e.$element(),{type:"pop",from:{scale:t,opacity:t},to:{scale:n,opacity:n},duration:i})},_navigate:function(e,t){if(0!==e&&1!==Math.abs(e)&&this._isViewAvailable(t)){var n=this._renderSpecificView(t);e>0?(this._afterView&&this._afterView.$element().remove(),this._afterView=n):(this._beforeView&&this._beforeView.$element().remove(),this._beforeView=n),this._translateViews()}var i=this._getRtlCorrection(),o=e>0?1:e<0?-1:0,a=-i*o*this._viewWidth(),r=this._$viewsWrapper.position().left;r!==a&&(this._preventViewChangeAnimation?this._wrapperAnimationEndHandler(e,t):this._animateWrapper(a,M).done(this._wrapperAnimationEndHandler.bind(this,e,t)))},_animateWrapper:function(e,t){return x.default.animate(this._$viewsWrapper,{type:"slide",from:{left:this._$viewsWrapper.position().left},to:{left:e},duration:t})},_getDate:function(e){var t=v.default.createDate(e);return N&&(0,l.isDate)(e)&&t.setMilliseconds(0),t},_toTodayView:function(){var e=new Date;return this._isMaxZoomLevel()?void this._dateOption("value",e):(this._preventViewChangeAnimation=!0,this.option("zoomLevel",this.option("maxZoomLevel")),this._dateOption("value",e),this._animateShowView(),void(this._preventViewChangeAnimation=!1))},_wrapperAnimationEndHandler:function(e,t){this._rearrangeViews(e),this._translateViews(),this._resetLocation(),this._renderNavigator(),this._setViewContoured(t),this._updateAriaId(t)},_rearrangeViews:function(e){if(0!==e){var t,n,i;if(e<0?(t=1,n="_beforeView",i="_afterView"):(t=-1,n="_afterView",i="_beforeView"),this[n]){var o=this[n].option("date");this[i]&&this[i].$element().remove(),e===t?this[i]=this._view:(this[i]=this._renderSpecificView(this._getDateByOffset(t,o)),this._view.$element().remove()),this._view=this[n];var a=this._getDateByOffset(-t,o);this[n]=this._isViewAvailable(a)?this._renderSpecificView(a):null}}},_resetLocation:function(){(0,_.move)(this._$viewsWrapper,{left:0,top:0})},_clean:function(){this.callBase(),this._clearViewWidthCache(),delete this._$viewsWrapper,delete this._navigator,delete this._$footer},_clearViewWidthCache:function(){delete this._viewWidthValue},_disposeViews:function(){this._view.$element().remove(),this._beforeView&&this._beforeView.$element().remove(),this._afterView&&this._afterView.$element().remove(),delete this._view,delete this._beforeView,delete this._afterView},_dispose:function(){clearTimeout(this._waitRenderViewTimeout),this.callBase()},_refreshViews:function(){this._disposeViews(),this._renderViews()},_visibilityChanged:function(){this._translateViews()},_focusInHandler:function(){this.callBase.apply(this,arguments),this._view.option("contouredDate",this.option("currentDate"))},_focusOutHandler:function(){this.callBase.apply(this,arguments),this._view.option("contouredDate",null)},_updateViewsValue:function(e){var t=e?new Date(e):null;this._view.option("value",t),this._beforeView&&this._beforeView.option("value",t),this._afterView&&this._afterView.option("value",t)},_updateAriaSelected:function(e,t){e=e||this._dateOption("value");var n=this._view._getCellByDate(t),i=this._view._getCellByDate(e);this.setAria("selected",void 0,n),this.setAria("selected",!0,i),e&&this.option("currentDate").getTime()===e.getTime()&&this._updateAriaId(e)},_updateAriaId:function(e){e=e||this.option("currentDate");var t="dx-"+new a.default,n=this._view._getCellByDate(e);this.setAria("id",t,n),this.setAria("activedescendant",t),this._onContouredChanged(t)},_suppressingNavigation:function(e,t){this._suppressNavigation=!0,e.apply(this,t),delete this._suppressNavigation},_optionChanged:function(e){var t=e.value,n=e.previousValue;switch(e.name){case"width":this.callBase(e),this._clearViewWidthCache();break;case"min":case"max":this.min=void 0,this.max=void 0,this._suppressingNavigation(this._updateCurrentDate,[this.option("currentDate")]),this._refreshViews(),this._renderNavigator();break;case"firstDayOfWeek":this._refreshViews(),this._updateButtonsVisibility();break;case"currentDate":this.setAria("id",void 0,this._view._getCellByDate(n)),this._updateCurrentDate(t);break;case"zoomLevel":this.$element().removeClass(P+"-"+n),this._correctZoomLevel(),this._refreshViews(),this._renderNavigator(),this._updateAriaId();break;case"minZoomLevel":case"maxZoomLevel":this._correctZoomLevel(),this._updateButtonsVisibility();break;case"value":t=this._convertToDate(t),n=this._convertToDate(n),this._updateAriaSelected(t,n),this.option("currentDate",(0,l.isDefined)(t)?v.default.createDate(t):new Date),this._updateViewsValue(t),this._setSubmitValue(t),this.callBase(e);break;case"onCellClick":this._view.option("onCellClick",t);break;case"onContouredChanged":this._onContouredChanged=this._createActionByOption("onContouredChanged");break;case"disabledDates":case"dateSerializationFormat":case"cellTemplate":case"showTodayButton":this._invalidate();break;case"hasFocus":break;case"_todayDate":this._refreshViews();break;default:this.callBase(e)}}});(0,r.default)("dxCalendar",j);var W=j;t.default=W,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(0),r=i(n(16)),s=i(n(38)),l="dx-calendar-navigator",u="dx-calendar-navigator-previous-month",c="dx-calendar-navigator-next-month",d="dx-calendar-navigator-previous-view",h="dx-calendar-navigator-next-view",f="dx-calendar-disabled-navigator-link",p="dx-calendar-caption-button",g=r.default.inherit({_getDefaultOptions:function(){return(0,a.extend)(this.callBase(),{onClick:null,onCaptionClick:null,text:""})},_init:function(){this.callBase(),this._initActions()},_initActions:function(){this._clickAction=this._createActionByOption("onClick"),this._captionClickAction=this._createActionByOption("onCaptionClick")},_initMarkup:function(){this.callBase(),this.$element().addClass(l),this._renderButtons(),this._renderCaption()},_renderButtons:function(){var e=this,t=this.option("rtlEnabled")?-1:1;this._prevButton=this._createComponent((0,o.default)("<a>"),s.default,{focusStateEnabled:!1,icon:"chevronleft",onClick:function(n){e._clickAction({direction:-t,event:n})},integrationOptions:{}});var n=this._prevButton.$element().addClass(d).addClass(u);this._nextButton=this._createComponent((0,o.default)("<a>"),s.default,{focusStateEnabled:!1,icon:"chevronright",onClick:function(n){e._clickAction({direction:t,event:n})},integrationOptions:{}});var i=this._nextButton.$element().addClass(h).addClass(c);this._caption=this._createComponent((0,o.default)("<a>").addClass(p),s.default,{focusStateEnabled:!1,onClick:function(t){e._captionClickAction({event:t})},integrationOptions:{}});var a=this._caption.$element();this.$element().append(n,a,i)},_renderCaption:function(){this._caption.option("text",this.option("text"))},toggleButton:function(e,t){var n="_"+e+"Button",i=this[n];i&&(i.option("disabled",t),i.$element().toggleClass(f,t))},_optionChanged:function(e){switch(e.name){case"text":this._renderCaption();break;default:this.callBase(e)}}}),_=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(613)),r=n(4),s=i(n(20)),l=n(0),u=i(n(32)),c=i(n(62)),d=n(1),h="dx-calendar-other-month",f="dx-calendar-other-view",p={month:a.default.inherit({_getViewName:function(){return"month"},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{firstDayOfWeek:void 0,rowCount:6,colCount:7})},_renderImpl:function(){this.callBase(),this._renderHeader()},_renderBody:function(){this.callBase(),this._$table.find(".".concat(f)).addClass(h)},_renderFocusTarget:r.noop,getCellAriaLabel:function(e){return u.default.format(e,"longdate")},_renderHeader:function(){var e=(0,o.default)("<tr>"),t=(0,o.default)("<thead>").append(e);this._$table.prepend(t);for(var n=0,i=this.option("colCount");n<i;n++)this._renderHeaderCell(n,e)},_renderHeaderCell:function(e,t){var n=this._getDayCaption(this._getFirstDayOfWeek()+e),i=n.full,a=n.abbreviated,r=(0,o.default)("<th>").attr({scope:"col",abbr:i}).text(a);this._appendCell(t,r)},getNavigatorCaption:function(){return u.default.format(this.option("date"),"monthandyear")},_isTodayCell:function(e){var t=this.option("_todayDate")();return s.default.sameDate(e,t)},_isDateOutOfRange:function(e){var t=this.option("min"),n=this.option("max");return!s.default.dateInRange(e,t,n,"date")},_isOtherView:function(e){return e.getMonth()!==this.option("date").getMonth()},_getCellText:function(e){return u.default.format(e,"d")},_getDayCaption:function(e){var t=this.option("colCount"),n=e%t;return{full:u.default.getDayNames()[n],abbreviated:u.default.getDayNames("abbreviated")[n]}},_getFirstCellData:function(){var e=s.default.getFirstMonthDate(this.option("date")),t=this._getFirstDayOfWeek()-e.getDay(),n=this.option("colCount");return t>=0&&(t-=n),e.setDate(e.getDate()+t),e},_getNextCellData:function(e){return e=s.default.createDate(e),e.setDate(e.getDate()+1),e},_getFirstDayOfWeek:function(){return(0,d.isDefined)(this.option("firstDayOfWeek"))?this.option("firstDayOfWeek"):u.default.firstDayOfWeekIndex()},_getCellByDate:function(e){return this._$table.find("td[data-value='".concat(c.default.serializeDate(e,s.default.getShortDateFormat()),"']"))},isBoundary:function(e){return s.default.sameMonthAndYear(e,this.option("min"))||s.default.sameMonthAndYear(e,this.option("max"))},_getDefaultDisabledDatesHandler:function(e){return function(t){var n=e.some(function(e){return s.default.sameDate(e,t.date)});if(n)return!0}}}),year:a.default.inherit({_getViewName:function(){return"year"},_isTodayCell:function(e){var t=this.option("_todayDate")();return s.default.sameMonthAndYear(e,t)},_isDateOutOfRange:function(e){return!s.default.dateInRange(e,s.default.getFirstMonthDate(this.option("min")),s.default.getLastMonthDate(this.option("max")))},_isOtherView:function(){return!1},_getCellText:function(e){return u.default.getMonthNames("abbreviated")[e.getMonth()]},_getFirstCellData:function(){var e=this.option("date"),t=s.default.createDate(e);return t.setDate(1),t.setMonth(0),t},_getNextCellData:function(e){return e=s.default.createDate(e),e.setMonth(e.getMonth()+1),e},_getCellByDate:function(e){var t=s.default.createDate(e);return t.setDate(1),this._$table.find("td[data-value='".concat(c.default.serializeDate(t,s.default.getShortDateFormat()),"']"))},getCellAriaLabel:function(e){return u.default.format(e,"monthandyear")},getNavigatorCaption:function(){return u.default.format(this.option("date"),"yyyy")},isBoundary:function(e){return s.default.sameYear(e,this.option("min"))||s.default.sameYear(e,this.option("max"))}}),decade:a.default.inherit({_getViewName:function(){return"decade"},_isTodayCell:function(e){var t=this.option("_todayDate")();return s.default.sameYear(e,t)},_isDateOutOfRange:function(e){var t=this.option("min"),n=this.option("max");return!s.default.dateInRange(e.getFullYear(),t&&t.getFullYear(),n&&n.getFullYear())},_isOtherView:function(e){var t=s.default.createDate(e);return t.setMonth(1),!s.default.sameDecade(t,this.option("date"))},_getCellText:function(e){return u.default.format(e,"yyyy")},_getFirstCellData:function(){var e=s.default.getFirstYearInDecade(this.option("date"))-1;return s.default.createDateWithFullYear(e,0,1)},_getNextCellData:function(e){return e=s.default.createDate(e),e.setFullYear(e.getFullYear()+1),e},getNavigatorCaption:function(){var e=this.option("date"),t=s.default.getFirstYearInDecade(e),n=s.default.createDate(e),i=s.default.createDate(e);return n.setFullYear(t),i.setFullYear(t+9),u.default.format(n,"yyyy")+"-"+u.default.format(i,"yyyy")},_isValueOnCurrentView:function(e,t){return s.default.sameDecade(e,t)},_getCellByDate:function(e){var t=s.default.createDate(e);return t.setDate(1),t.setMonth(0),this._$table.find("td[data-value='".concat(c.default.serializeDate(t,s.default.getShortDateFormat()),"']"))},isBoundary:function(e){return s.default.sameDecade(e,this.option("min"))||s.default.sameDecade(e,this.option("max"))}}),century:a.default.inherit({_getViewName:function(){return"century"},_isTodayCell:function(e){var t=this.option("_todayDate")();return s.default.sameDecade(e,t)},_isDateOutOfRange:function(e){var t=s.default.getFirstYearInDecade(e),n=s.default.getFirstYearInDecade(this.option("min")),i=s.default.getFirstYearInDecade(this.option("max"));return!s.default.dateInRange(t,n,i)},_isOtherView:function(e){var t=s.default.createDate(e);return t.setMonth(1),!s.default.sameCentury(t,this.option("date"))},_getCellText:function(e){var t=u.default.format(e,"yyyy"),n=s.default.createDate(e);return n.setFullYear(n.getFullYear()+9),t+" - "+u.default.format(n,"yyyy")},_getFirstCellData:function(){var e=s.default.getFirstDecadeInCentury(this.option("date"))-10;return s.default.createDateWithFullYear(e,0,1)},_getNextCellData:function(e){return e=s.default.createDate(e),e.setFullYear(e.getFullYear()+10),e},_getCellByDate:function(e){var t=s.default.createDate(e);return t.setDate(1),t.setMonth(0),t.setFullYear(s.default.getFirstYearInDecade(t)),this._$table.find("td[data-value='".concat(c.default.serializeDate(t,s.default.getShortDateFormat()),"']"))},getNavigatorCaption:function(){var e=this.option("date"),t=s.default.getFirstDecadeInCentury(e),n=s.default.createDate(e),i=s.default.createDate(e);return n.setFullYear(t),i.setFullYear(t+99),u.default.format(n,"yyyy")+"-"+u.default.format(i,"yyyy")},isBoundary:function(e){return s.default.sameCentury(e,this.option("min"))||s.default.sameCentury(e,this.option("max"))}})},g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=n(39),l=n(18),u=i(n(16)),c=i(n(20)),d=n(0),h=n(4),f=i(n(62)),p=i(n(10)),g=n(8),_=n(21),m=u.default.abstract,v="dx-calendar-other-view",y="dx-calendar-cell",b="dx-calendar-empty-cell",x="dx-calendar-today",w="dx-calendar-selected-date",C="dx-calendar-contoured-date",k=(0,g.addNamespace)(_.name,"dxCalendar"),S="dxDateValueKey",D=u.default.inherit({_getViewName:function(){return"base"},_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{date:new Date,focusStateEnabled:!1,cellTemplate:null,disabledDates:null,onCellClick:null,rowCount:3,colCount:4,allowValueSelection:!0,_todayDate:function(){return new Date}})},_init:function(){this.callBase();var e=this.option("value");this.option("value",new Date(e)),this.option("value").valueOf()||this.option("value",new Date(0,0,0,0,0,0))},_initMarkup:function(){this.callBase(),this._renderImpl()},_renderImpl:function(){this.$element().append(this._createTable()),this._createDisabledDatesHandler(),this._renderBody(),this._renderContouredDate(),this._renderValue(),this._renderEvents()},_createTable:function(){return this._$table=(0,o.default)("<table>"),this.setAria({label:p.default.format("dxCalendar-ariaWidgetName"),role:"grid"},this._$table),this._$table},_renderBody:function(){this.$body=(0,o.default)("<tbody>").appendTo(this._$table);for(var e={cellDate:this._getFirstCellData(),prevCellDate:null},t=0,n=this.option("rowCount");t<n;t++){e.row=this._createRow();for(var i=0,a=this.option("colCount");i<a;i++)this._renderCell(e,i)}},_createRow:function(){var e=a.default.createElement("tr");return this.setAria("role","row",(0,o.default)(e)),this.$body.get(0).appendChild(e),e},_appendCell:function(e,t){this._appendMethodName||this._cacheAppendMethodName(),(0,o.default)(e)[this._appendMethodName](t)},_cacheAppendMethodName:function(e){this._appendMethodName=(null!==e&&void 0!==e?e:this.option("rtlEnabled"))?"prepend":"append"},_createCell:function(e){var t=a.default.createElement("td"),n=(0,o.default)(t);return t.className=this._getClassNameByDate(e),t.setAttribute("data-value",f.default.serializeDate(e,c.default.getShortDateFormat())),(0,s.data)(t,S,e),this.setAria({role:"gridcell",label:this.getCellAriaLabel(e)},n),{cell:t,$cell:n}},_renderCell:function(e,t){var n=e.cellDate,i=e.prevCellDate,o=e.row;i&&c.default.fixTimezoneGap(i,n),e.prevCellDate=n;var a=this._createCell(n),r=a.cell,s=a.$cell,l=this.option("cellTemplate");this._appendCell(o,r),l?l.render(this._prepareCellTemplateData(n,t,s)):r.innerHTML=this._getCellText(n),e.cellDate=this._getNextCellData(n)},_getClassNameByDate:function(e){var t=y;return this._isTodayCell(e)&&(t+=" ".concat(x)),(this._isDateOutOfRange(e)||this.isDateDisabled(e))&&(t+=" ".concat(b)),this._isOtherView(e)&&(t+=" ".concat(v)),t},_prepareCellTemplateData:function(e,t,n){return{model:{text:this._getCellText(e),date:e,view:this._getViewName()},container:(0,l.getPublicElement)(n),index:t}},_renderEvents:function(){var e=this;this._createCellClickAction(),r.default.off(this._$table,k),r.default.on(this._$table,k,"td",function(t){(0,o.default)(t.currentTarget).hasClass(b)||e._cellClickAction({event:t,value:(0,o.default)(t.currentTarget).data(S)})})},_createCellClickAction:function(){this._cellClickAction=this._createActionByOption("onCellClick")},_createDisabledDatesHandler:function(){var e=this.option("disabledDates");this._disabledDatesHandler=Array.isArray(e)?this._getDefaultDisabledDatesHandler(e):e||h.noop},_getDefaultDisabledDatesHandler:function(e){return h.noop},_isTodayCell:m,_isDateOutOfRange:m,isDateDisabled:function(e){var t={date:e,view:this._getViewName()};return this._disabledDatesHandler(t)},_isOtherView:m,_getCellText:m,_getFirstCellData:m,_getNextCellData:m,_renderContouredDate:function(e){if(this.option("focusStateEnabled")){e=e||this.option("contouredDate");var t=this._$table.find(".".concat(C)),n=this._getCellByDate(e);t.removeClass(C),n.addClass(C)}},_changeValue:function(e){if(e){var t=this.option("value"),n=t?new Date(t):new Date;n.setDate(e.getDate()),n.setMonth(e.getMonth()),n.setFullYear(e.getFullYear()),n.setDate(e.getDate()),this.option("value",n)}else this.option("value",null)},_renderValue:function(){if(this.option("allowValueSelection")){var e=this.option("value"),t=this._getCellByDate(e);this._selectedCell&&this._selectedCell.removeClass(w),t.addClass(w),this._selectedCell=t}},getCellAriaLabel:function(e){return this._getCellText(e)},_getFirstAvailableDate:function(){var e=this.option("date"),t=this.option("min");return e=c.default.getFirstDateView(this._getViewName(),e),new Date(t&&e<t?t:e)},_getCellByDate:m,isBoundary:m,_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"value":this._renderValue();break;case"contouredDate":this._renderContouredDate(n);break;case"onCellClick":this._createCellClickAction();break;case"disabledDates":case"cellTemplate":this._invalidate();break;case"rtlEnabled":this._cacheAppendMethodName(n),this.callBase(e);break;case"_todayDate":this._renderBody();break;default:this.callBase(e)}}}),I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(98)),s=i(n(435)),l=n(0),u=i(n(9)),c=i(n(192)),d="dx-colorbox",h=d+"-input",f=h+"-container",p=d+"-color-result-preview",g=d+"-color-is-not-defined",_=d+"-overlay",m="dx-colorview-container-cell",v="dx-colorview-button-cell",y="dx-colorview-buttons-container",b="dx-colorview-apply-button",x="dx-colorview-cancel-button",w=s.default.prototype,C={makeTransparentBackground:w._makeTransparentBackground.bind(w),makeRgba:w._makeRgba.bind(w)},k=c.default.inherit({_supportedKeys:function(){var e=function(e){if(e.stopPropagation(),this.option("opened"))return e.preventDefault(),!0},t=function(e){return this.option("opened")?!e.altKey||(this.close(),!1):(e.preventDefault(),!1)},n=function(e){return this.option("opened")||e.altKey?!(!this.option("opened")&&e.altKey)||(this._validatedOpening(),!1):(e.preventDefault(),!1)};return(0,l.extend)(this.callBase(),{tab:function(e){this.option("opened")&&(e.preventDefault(),this._colorView._rgbInputs[0].focus())},enter:this._enterKeyHandler,leftArrow:e,rightArrow:e,upArrow:t,downArrow:n})},_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{editAlphaChannel:!1,applyValueMode:"useButtons",keyStep:1,fieldTemplate:null,buttonsLocation:"bottom after"})},_popupHidingHandler:function(){this.callBase(),"useButtons"===this.option("applyValueMode")&&this._updateColorViewValue(this.option("value"))},_popupConfig:function(){return(0,l.extend)(this.callBase(),{width:""})},_contentReadyHandler:function(){this._createColorView(),this._addPopupBottomClasses()},_addPopupBottomClasses:function(){var e=this._popup.bottomToolbar();e&&(e.addClass(m).addClass(v).find(".dx-toolbar-items-container").addClass(y),e.find(".dx-popup-done").addClass(b),e.find(".dx-popup-cancel").addClass(x))},_createColorView:function(){this._popup.overlayContent().addClass(_);var e=(0,o.default)("<div>").appendTo(this._popup.$content());this._colorView=this._createComponent(e,s.default,this._colorViewConfig()),this._colorView.registerKeyHandler("escape",this._escapeHandler.bind(this)),a.default.on(e,"focus",function(){this.focus()}.bind(this))},_escapeHandler:function(){this.close(),this.focus()},_applyNewColor:function(e){this.option("value",e),e&&C.makeTransparentBackground(this._$colorResultPreview,e),this._colorViewEnterKeyPressed&&(this.close(),this._colorViewEnterKeyPressed=!1)},_colorViewConfig:function(){var e=this;return{value:e.option("value"),matchValue:e.option("value"),editAlphaChannel:e.option("editAlphaChannel"),applyValueMode:e.option("applyValueMode"),focusStateEnabled:e.option("focusStateEnabled"),stylingMode:this.option("stylingMode"),onEnterKeyPressed:function(){e._colorViewEnterKeyPressed=!0,e._colorView.option("value")!==e.option("value")&&(e._applyNewColor(e._colorView.option("value")),e.close())},onValueChanged:function(t){if(C.makeRgba(t.value)!==t.previousValue){var n="instantly"===e.option("applyValueMode");(n||e._colorViewEnterKeyPressed)&&e._applyNewColor(t.value)}}}},_enterKeyHandler:function(){var e=this._input().val(),t=this.option("value"),n=this.option("editAlphaChannel")?C.makeRgba(t):t;if(!e)return!1;var i=new r.default(e);if(i.colorIsInvalid)return void this._input().val(n);if(e!==n&&(this._applyColorFromInput(e),this.option("value",this.option("editAlphaChannel")?C.makeRgba(e):e)),this._colorView){var o=this._colorView.option("value");t!==o&&this.option("value",o)}return this.close(),!1},_applyButtonHandler:function(){this._applyNewColor(this._colorView.option("value")),this.callBase()},_cancelButtonHandler:function(){this._resetInputValue(),this.callBase()},_getKeyboardListeners:function(){return this.callBase().concat([this._colorView])},_init:function(){this.callBase()},_initMarkup:function(){this.$element().addClass(d),this.callBase()},_renderInput:function(){this.callBase(),this._input().addClass(h),this._renderColorPreview()},_renderColorPreview:function(){this.$element().wrapInner((0,o.default)("<div>").addClass(f)),this._$colorBoxInputContainer=this.$element().children().eq(0),this._$colorResultPreview=(0,o.default)("<div>").addClass(p).appendTo(this._$textEditorInputContainer),this.option("value")?C.makeTransparentBackground(this._$colorResultPreview,this.option("value")):this._$colorBoxInputContainer.addClass(g)},_renderValue:function(){var e=this.option("value");return this.option("text",this.option("editAlphaChannel")?C.makeRgba(e):e),this.callBase()},_resetInputValue:function(){var e=this._input(),t=this.option("value");e.val(t),this._updateColorViewValue(t)},_updateColorViewValue:function(e){this._colorView&&this._colorView.option({value:e,matchValue:e})},_valueChangeEventHandler:function(e){var t=this._input().val();t&&(t=this._applyColorFromInput(t),this._updateColorViewValue(t)),this.callBase(e,t)},_applyColorFromInput:function(e){var t=new r.default(e);return t.colorIsInvalid&&(this._resetInputValue(),e=this.option("value")),e},_optionChanged:function(e){var t=e.value,n=e.name;switch(n){case"value":this._$colorBoxInputContainer.toggleClass(g,!t),t?C.makeTransparentBackground(this._$colorResultPreview,t):this._$colorResultPreview.removeAttr("style"),this._updateColorViewValue(t),this.callBase(e);break;case"applyButtonText":case"cancelButtonText":this.callBase(e),this._popup&&this._addPopupBottomClasses();break;case"editAlphaChannel":case"keyStep":this._colorView&&this._colorView.option(n,t);break;default:this.callBase(e)}}});(0,u.default)("dxColorBox",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=i(n(616));
(0,o.default)("dxNumberBox",a.default);var r=a.default;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(5)),a=n(0),r=n(1),s=i(n(23)),l=i(n(13)),u=n(29),c=i(n(45)),d=n(617),h=n(268),f=i(n(618)),p=n(8),g=n(4),_="dxNumberFormatter",m=1,v=-1,y="-",b="minus",x="Subtract",w="input",C=s.default.msie?300:0,k=f.default.inherit({_getDefaultOptions:function(){return(0,a.extend)(this.callBase(),{useMaskBehavior:!0,format:null})},_isDeleteKey:function(e){return"del"===e},_supportedKeys:function(){if(!this._useMaskBehavior())return this.callBase();var e=this;return(0,a.extend)(this.callBase(),{minus:e._revertSign.bind(e),del:e._removeHandler.bind(e),backspace:e._removeHandler.bind(e),leftArrow:e._arrowHandler.bind(e,v),rightArrow:e._arrowHandler.bind(e,m),home:e._moveCaretToBoundaryEventHandler.bind(e,m),enter:e._updateFormattedValue.bind(e),end:e._moveCaretToBoundaryEventHandler.bind(e,v)})},_focusInHandler:function(e){this._preventNestedFocusEvent(e)||(this.clearCaretTimeout(),this._caretTimeout=setTimeout(function(){this._caretTimeout=null;var t=this._caret();if(t.start===t.end&&this._useMaskBehavior()){var n=this._getInputVal(),i=c.default.getDecimalSeparator(),o=n.indexOf(i);o>=0?this._caret({start:o,end:o}):this._moveCaretToBoundaryEventHandler(v,e)}}.bind(this),C)),this.callBase(e)},_focusOutHandler:function(e){var t=!this._preventNestedFocusEvent(e);t&&(this._focusOutOccurs=!0,this._useMaskBehavior()&&this._updateFormattedValue()),this.callBase(e),t&&(this._focusOutOccurs=!1)},_hasValueBeenChanged:function(e){var t=this._getFormatPattern(),n=this.option("value"),i=this._format(n,t)||"";return i!==e},_updateFormattedValue:function(){var e=this._getInputVal();this._hasValueBeenChanged(e)&&(this._updateParsedValue(),this._adjustParsedValue(),this._setTextByParsedValue(),this._parsedValue!==this.option("value")&&o.default.trigger(this._input(),"change"))},_arrowHandler:function(e,t){if(this._useMaskBehavior()){var n=this._getInputVal(),i=this._getFormatPattern(),o=(0,d.getCaretWithOffset)(this._caret(),e);(0,d.isCaretInBoundaries)(o,n,i)||(o=e===m?o.end:o.start,t.preventDefault(),this._caret((0,d.getCaretInBoundaries)(o,n,i)))}},_moveCaretToBoundary:function(e){var t=(0,d.getCaretBoundaries)(this._getInputVal(),this._getFormatPattern()),n=(0,d.getCaretWithOffset)(e===m?t.start:t.end,0);this._caret(n)},_moveCaretToBoundaryEventHandler:function(e,t){!this._useMaskBehavior()||t&&t.shiftKey||(this._moveCaretToBoundary(e),t&&t.preventDefault())},_shouldMoveCaret:function(e,t){var n=c.default.getDecimalSeparator(),i=e.charAt(t.end)===n,o="0"===e.charAt(t.end),a=(this._lastKey===n||"."===this._lastKey)&&i,r="0"===this._lastKey&&o;return a||r},_getInputVal:function(){return c.default.convertDigits(this._input().val(),!0)},_keyboardHandler:function(e){if(this.clearCaretTimeout(),this._lastKey=c.default.convertDigits((0,p.getChar)(e),!0),this._lastKeyName=(0,p.normalizeKeyName)(e),!this._shouldHandleKey(e.originalEvent))return this.callBase(e);var t=this._getInputVal(),n=this._caret(),i=this._lastKeyName===b?"":this._lastKey,o=this._tryParse(t,n,i);return this._shouldMoveCaret(t,n)&&(this._moveCaret(1),e.originalEvent.preventDefault()),void 0===o?this._lastKeyName!==b&&e.originalEvent.preventDefault():this._parsedValue=o,this.callBase(e)},_keyPressHandler:function(e){this._useMaskBehavior()||this.callBase(e)},_removeHandler:function(e){var t=this._caret(),n=this._getInputVal(),i=t.start,o=t.end;this._lastKey=(0,p.getChar)(e),this._lastKeyName=(0,p.normalizeKeyName)(e);var a=this._isDeleteKey(this._lastKeyName),r=!a;if(i===o){var s=i,l=r&&s>0||a&&s<n.length;if(!l)return void e.preventDefault();a&&o++,r&&i--}var u=n.slice(i,o);if(this._isStub(u))return this._moveCaret(a?1:-1),(this._parsedValue<0||1/this._parsedValue===-(1/0))&&(this._revertSign(e),this._setTextByParsedValue()),void e.preventDefault();var d=c.default.getDecimalSeparator();if(u===d){var h=n.indexOf(d);return void(this._isNonStubAfter(h+1)&&(this._moveCaret(a?1:-1),e.preventDefault()))}if(o-i<n.length){var f=this._replaceSelectedText(n,{start:i,end:o},""),g=f.search(/[0-9]/)<0;if(g&&this._isValueInRange(0))return void(this._parsedValue=this._parsedValue<0||1/this._parsedValue===-(1/0)?-0:0)}var _=this._tryParse(n,{start:i,end:o},"");void 0===_?e.preventDefault():this._parsedValue=_},_isPercentFormat:function(){var e=this._getFormatPattern(),t=e.replace(/'[^']+'/g,"");return t.indexOf("%")!==-1},_parse:function(e,t){var n=this.option("format"),i=(0,r.isFunction)(n.parser),o=i?n.parser:c.default.parse;if(!i){var a=t.indexOf("."),s=e.indexOf(c.default.getDecimalSeparator()),l=a!==-1?a:t.length,u=s!==-1?s:e.length;u>l&&t.indexOf("#")===-1&&(e=e.substr(u-l))}return o(e,t)},_format:function(e,t){var n=this.option("format"),i=(0,r.isFunction)(null===n||void 0===n?void 0:n.formatter),o=i?n.formatter:c.default.format;return o(e,t)},_getFormatPattern:function(){return this._currentFormat||this._updateFormat(),this._currentFormat},_updateFormat:function(){var e=this,t=this.option("format"),n=(0,r.isFunction)(null===t||void 0===t?void 0:t.parser),i=(0,r.isString)(t)&&(t.indexOf("0")>=0||t.indexOf("#")>=0);this._currentFormat=n||i?t:(0,h.getFormat)(function(n){var i=e._format(n,t);return c.default.convertDigits(i,!0)})},_getFormatForSign:function(e){var t=this._getFormatPattern();if((0,r.isString)(t)){var n=t.split(";"),i=c.default.getSign(e,t);return n[1]=n[1]||"-"+n[0],i<0?n[1]:n[0]}var o=c.default.getSign(e);return o<0?"-":""},_removeStubs:function(e,t){var n=this._getFormatForSign(e),i=c.default.getThousandsSeparator(),o=this._getStubs(n),a=e;if(o.length){var r=o[0],s=new RegExp("("+(0,g.escapeRegExp)(o[1]||"")+")$","g"),l=new RegExp("[-"+(0,g.escapeRegExp)(t?"":i)+"]","g");a=a.replace(r,"").replace(s,"").replace(l,"")}return a},_getStubs:function(e){var t=/[^']([#0.,]+)/g.exec(e),n=t&&t[0].trim();return e.split(n).map(function(e){return e.replace(/'/g,"")})},_truncateToPrecision:function(e,t){if((0,r.isDefined)(e)){var n=e.toString(),i=n.indexOf(".");if(n&&i>-1){var o=parseFloat(n.substr(0,i+t+1));return isNaN(o)?e:o}}return e},_tryParse:function(e,t,n){var i=this._replaceSelectedText(e,t,n),o=this._getFormatPattern(),a=t.start!==t.end,r=this._getParsedValue(i,o),s=!o.parser&&this._getPrecisionLimits(i).max,l=r!==this._parsedValue,u=c.default.getDecimalSeparator(),d=n===u&&0===s,h=!a&&!l&&n!==y&&!this._isValueIncomplete(i)&&this._isStub(n);if(!d&&!h&&(""===this._removeStubs(i)&&(r=Math.abs(0*this._parsedValue)),!isNaN(r))){var f=null===r?this._parsedValue:r;return r=s?this._truncateToPrecision(f,s):r,!o.parser&&this._isPercentFormat()?r&&r/100:r}},_getParsedValue:function(e,t){var n=c.default.getSign(e,(null===t||void 0===t?void 0:t.formatter)||t),i=this._removeStubs(e,!0),o=this._parse(i,t),a=o<0?-1:1,s=(0,r.isNumeric)(o)&&n!==a?n*o:o;return s},_isValueIncomplete:function(e){if(!this._useMaskBehavior())return this.callBase(e);var t=this._caret(),n=c.default.getDecimalSeparator(),i=e.indexOf(n),o=i>=0&&i<t.start,a=this._removeStubs(e,!0).split(n);if(!o||2!==a.length)return!1;var r=a[1].length,s=this._getFormatPattern(),l=!!s.parser,d=!l&&this._getPrecisionLimits(this._getFormatPattern(),e),h=!!l||(0,u.inRange)(r,d.min,d.max),f="0"===a[1].charAt(r-1);return h&&(f||!r)},_isValueInRange:function(e){var t=(0,g.ensureDefined)(this.option("min"),-(1/0)),n=(0,g.ensureDefined)(this.option("max"),1/0);return(0,u.inRange)(e,t,n)},_setInputText:function(e){var t=c.default.convertDigits(e,!0),n=(0,d.getCaretAfterFormat)(this._getInputVal(),t,this._caret(),this._getFormatPattern());this._input().val(e),this._toggleEmptinessEventHandler(),this._formattedValue=e,this._focusOutOccurs||this._caret(n)},_useMaskBehavior:function(){return!!this.option("format")&&this.option("useMaskBehavior")},_renderInputType:function(){var e="number"===this.option("mode"),t="desktop"===l.default.real().deviceType;this._useMaskBehavior()&&e?this._setInputType(t||this._isSupportInputMode()?"text":"tel"):this.callBase()},_isChar:function(e){return(0,r.isString)(e)&&1===e.length},_moveCaret:function(e){if(e){var t=(0,d.getCaretWithOffset)(this._caret(),e),n=(0,d.getCaretInBoundaries)(t,this._getInputVal(),this._getFormatPattern());this._caret(n)}},_shouldHandleKey:function(e){var t=(0,p.normalizeKeyName)(e),n=e.ctrlKey||e.shiftKey||e.altKey||!this._isChar(t),i=t===b,o=this._useMaskBehavior();return o&&!n&&!i},_renderInput:function(){this.callBase(),this._renderFormatter()},_renderFormatter:function(){this._clearCache(),this._detachFormatterEvents(),this._useMaskBehavior()&&this._attachFormatterEvents()},_detachFormatterEvents:function(){o.default.off(this._input(),"."+_)},_isInputFromPaste:function(e){var t=e.originalEvent&&e.originalEvent.inputType;return(0,r.isDefined)(t)?"insertFromPaste"===t:this._isValuePasted},_attachFormatterEvents:function(){var e=this._input();o.default.on(e,(0,p.addNamespace)(w,_),function(e){this._formatValue(e),this._isValuePasted=!1}.bind(this)),s.default.msie&&s.default.version<12&&o.default.on(e,(0,p.addNamespace)("paste",_),function(){this._isValuePasted=!0}.bind(this)),o.default.on(e,(0,p.addNamespace)("dxclick",_),function(){this._caretTimeout||(this._caretTimeout=setTimeout(function(){this._caret((0,d.getCaretInBoundaries)(this._caret(),this._getInputVal(),this._getFormatPattern()))}.bind(this),C))}.bind(this)),o.default.on(e,"dxdblclick",function(){this.clearCaretTimeout()}.bind(this))},clearCaretTimeout:function(){clearTimeout(this._caretTimeout),this._caretTimeout=null},_forceRefreshInputValue:function(){if(!this._useMaskBehavior())return this.callBase()},_isNonStubAfter:function(e,t){return t=(t||this._getInputVal()).slice(e),t&&!this._isStub(t,!0)},_isStub:function(e,t){var n=(0,g.escapeRegExp)(c.default.getDecimalSeparator()),i="^[^0-9"+n+"]+$",o=new RegExp(i,"g");return o.test(e)&&(t||this._isChar(e))},_parseValue:function(e){return this._useMaskBehavior()?this._parsedValue:this.callBase(e)},_getPrecisionLimits:function(e){var t=this._getFormatForSign(e),n=(t.split(".")[1]||"").replace(/[^#0]/g,""),i=n.replace(/^(0*)#*/,"$1").length,o=n.length;return{min:i,max:o}},_revertSign:function(e){if(this._useMaskBehavior()){var t=this._caret();if(t.start!==t.end){if((0,p.normalizeKeyName)(e)===b)return void this._applyRevertedSign(e,t,!0);this._caret((0,d.getCaretInBoundaries)(0,this._getInputVal(),this._getFormatPattern()))}this._applyRevertedSign(e,t)}},_applyRevertedSign:function(e,t,n){var i=-1*(0,g.ensureDefined)(this._parsedValue,null);if(this._isValueInRange(i)){if(this._parsedValue=i,n){var a=this._getFormatPattern(),r=this._getInputVal();this._setTextByParsedValue(),e.preventDefault();var l=this._getInputVal(),u=(0,d.getCaretOffset)(r,l,a);t=(0,d.getCaretWithOffset)(t,u);var c=(0,d.getCaretInBoundaries)(t,l,a);s.default.msie?(clearTimeout(this._caretTimeout),this._caretTimeout=setTimeout(this._caret.bind(this,c))):this._caret(c)}e.key===x&&o.default.trigger(this._input(),w)}},_removeMinusFromText:function(e,t){var n=this._lastKeyName===b&&e.charAt(t.start-1)===y;return n?this._replaceSelectedText(e,{start:t.start-1,end:t.start},""):e},_setTextByParsedValue:function(){var e=this._getFormatPattern(),t=this._parseValue(),n=this._format(t,e)||"";this._setInputText(n)},_formatValue:function(e){var t=this._getInputVal(),n=this._caret(),i=this._removeMinusFromText(t,n),o=i!==t;if(t=i,!this._isInputFromPaste(e)&&this._isValueIncomplete(i))return this._formattedValue=t,void(o&&this._setTextByParsedValue());var a=c.default.convertDigits(this._formattedValue,!0)!==t;if(a){var s=this._tryParse(t,n,"");(0,r.isDefined)(s)&&(this._parsedValue=s)}this._setTextByParsedValue()},_renderDisplayText:function(){this._useMaskBehavior()?this._toggleEmptinessEventHandler():this.callBase.apply(this,arguments)},_renderValue:function(){return this._useMaskBehavior()&&(this._parsedValue=this.option("value"),this._setTextByParsedValue()),this.callBase()},_updateParsedValue:function(){var e=this._getInputVal();this._parsedValue=this._tryParse(e,this._caret())},_adjustParsedValue:function(){if(this._useMaskBehavior()){var e=this._removeStubs(this._getInputVal()),t=e?this._parseValue():null;return(0,r.isNumeric)(t)?void(this._parsedValue=(0,u.fitIntoRange)(t,this.option("min"),this.option("max"))):void(this._parsedValue=t)}},_valueChangeEventHandler:function(e){if(!this._useMaskBehavior())return this.callBase(e);var t=this._caret();this._saveValueChangeEvent(e),this._lastKey=null,this._lastKeyName=null,this._updateParsedValue(),this._adjustParsedValue(),this.option("value",this._parsedValue),t&&this._caret(t)},_optionChanged:function(e){switch(e.name){case"format":case"useMaskBehavior":this._updateFormat(),this._renderFormatter(),this._renderValue();break;case"min":case"max":this._adjustParsedValue(),this.callBase(e);break;default:this.callBase(e)}},_clearCache:function(){delete this._formattedValue,delete this._lastKey,delete this._lastKeyName,delete this._parsedValue,delete this._focusOutOccurs,clearTimeout(this._caretTimeout),delete this._caretTimeout},_clean:function(){this._clearCache(),this.callBase()}}),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return 1===e.length&&!!e.match(/^[,.][0-9]*$/g)}function a(e,t,n){e=p(e,0);var i=u(t,n),o={start:(0,r.fitIntoRange)(e.start,i.start,i.end),end:(0,r.fitIntoRange)(e.end,i.start,i.end)};return o}t.getCaretInBoundaries=a,t.getCaretOffset=t.isCaretInBoundaries=t.getCaretAfterFormat=t.getCaretWithOffset=t.getCaretBoundaries=void 0;var r=n(29),s=n(4),l=i(n(45)),u=function(e,t){if("string"==typeof t){var n=t.split(";"),i=l.default.getSign(e,t);n[1]=n[1]||"-"+n[0],t=n[i<0?1:0];var o=function(e){return e.replace(/'([^']*)'/g,function(e){return e.split("").map(function(){return" "}).join("").substr(2)})};t=o(t);var a=/^[^#0.,]*/.exec(t)[0].length,r=/[^#0.,]*$/.exec(t)[0].length;return{start:a,end:e.length-r}}return{start:0,end:e.length}};t.getCaretBoundaries=u;var c=function(e,t){var n=l.default.getDecimalSeparator(),i=new RegExp("[^0-9"+(0,s.escapeRegExp)(n)+"]","g"),o=t.slice(0,e);return o.replace(i,"").length},d=function(e){return e.split("").reverse().join("")},h=function(e,t){if(!e)return-1;for(var n=/[0-9]/g,i=1,o=null,a=n.exec(t);a;){if(o=a.index,!e||i>=e)return o;i++,a=n.exec(t)}return null===o?t.length:o},f=function(e){return e.replace(/[^0-9e]+$/,"")},p=function(e,t){return void 0===e.start&&(e={start:e,end:e}),{start:e.start+t,end:e.end+t}};t.getCaretWithOffset=p;var g=function(e,t,n,i){n=p(n,0);var r=l.default.getDecimalSeparator(),s=o(e),u=s?0:e.indexOf(r),g=t.indexOf(r),_=s?e.split(e[u]):e.split(r),m=t.split(r),v=u!==-1&&n.start>u;if(v){var y=n.start-u-1,b=c(y,_[1]),x=m[1]?g+1+h(b,m[1])+1:t.length;return a(x,t,i)}var w=f(m[0]),C=_[0].length-n.start,k=c(C,d(_[0])),S=h(k,d(w)),D=w.length-(S+1);return a(D,t,i)};t.getCaretAfterFormat=g;var _=function(e,t,n){e=p(e,0);var i=a(e,t,n);return e.start>=i.start&&e.end<=i.end};t.isCaretInBoundaries=_;var m=function(e,t,n){var i=u(e,n),o=u(t,n);return o.start-i.start};t.getCaretOffset=m},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=n(4),l=n(1),u=n(29),c=n(0),d=n(12),h=i(n(13)),f=i(n(23)),p=i(n(422)),g=n(8),_=i(n(619)),m=i(n(10)),v=n(6),y=Math,b="dx-numberbox",x=["tab","del","backspace","leftArrow","rightArrow","home","end","enter"],w="NumberBoxForceValueChange",C=p.default.inherit({_supportedKeys:function(){return(0,c.extend)(this.callBase(),{upArrow:function(e){(0,g.isCommandKeyPressed)(e)||(e.preventDefault(),e.stopPropagation(),this._spinUpChangeHandler(e))},downArrow:function(e){(0,g.isCommandKeyPressed)(e)||(e.preventDefault(),e.stopPropagation(),this._spinDownChangeHandler(e))},enter:function(){}})},_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{value:0,min:void 0,max:void 0,step:1,showSpinButtons:!1,useLargeSpinButtons:!0,mode:"text",invalidValueMessage:m.default.format("dxNumberBox-invalidValueMessage"),buttons:void 0})},_getDefaultButtons:function(){return this.callBase().concat([{name:"spins",Ctor:_.default}])},_isSupportInputMode:function(){var e=parseFloat(f.default.version);return f.default.chrome&&e>=66||f.default.safari&&e>=12||f.default.msie&&e>=75},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return h.default.real().generic&&!h.default.isSimulator()},options:{useLargeSpinButtons:!1}},{device:function(){return"desktop"!==h.default.real().deviceType&&!this._isSupportInputMode()}.bind(this),options:{mode:"number"}}])},_initMarkup:function(){this._renderSubmitElement(),this.$element().addClass(b),this.callBase()},_getDefaultAttributes:function(){var e=this.callBase();return e.inputmode="decimal",e},_renderContentImpl:function(){this.option("isValid")&&this._validateValue(this.option("value")),this.setAria("role","spinbutton")},_renderSubmitElement:function(){this._$submitElement=(0,o.default)("<input>").attr("type","hidden").appendTo(this.$element()),this._setSubmitValue(this.option("value"))},_setSubmitValue:function(e){this._getSubmitElement().val((0,s.applyServerDecimalSeparator)(e))},_getSubmitElement:function(){return this._$submitElement},_keyPressHandler:function(e){this.callBase(e);var t=(0,g.getChar)(e),n=/[\d.,eE\-+]|Subtract/,i=n.test(t);if(!i){var o=(0,g.normalizeKeyName)(e);if((0,g.isCommandKeyPressed)(e)||o&&(0,d.inArray)(o,x)>=0)return;return e.preventDefault(),!1}this._keyPressed=!0},_onMouseWheel:function(e){e.delta>0?this._spinValueChange(1,e):this._spinValueChange(-1,e)},_renderValue:function(){var e=this._input().val(),t=this.option("value");e.length&&Number(e)===t||(this._forceValueRender(),this._toggleEmptinessEventHandler());var n=(0,l.isDefined)(t)?null:m.default.format("dxNumberBox-noDataText");return this.setAria({valuenow:(0,s.ensureDefined)(t,""),valuetext:n}),this.option("text",this._input().val()),this._updateButtons(),(new v.Deferred).resolve()},_forceValueRender:function(){var e=this.option("value"),t=Number(e),n=isNaN(t)?"":this._applyDisplayValueFormatter(e);this._renderDisplayText(n)},_applyDisplayValueFormatter:function(e){return this.option("displayValueFormatter")(e)},_renderProps:function(){this.callBase(),this._input().prop({min:this.option("min"),max:this.option("max"),step:this.option("step")}),this.setAria({valuemin:(0,s.ensureDefined)(this.option("min"),""),valuemax:(0,s.ensureDefined)(this.option("max"),"")})},_spinButtonsPointerDownHandler:function(){var e=this._input();this.option("useLargeSpinButtons")||a.default.getActiveElement()===e[0]||r.default.trigger(e,"focus")},_spinUpChangeHandler:function(e){this.option("readOnly")||this._spinValueChange(1,e.event||e)},_spinDownChangeHandler:function(e){this.option("readOnly")||this._spinValueChange(-1,e.event||e)},_spinValueChange:function(e,t){var n=parseFloat(this.option("step"));if(0!==n){var i=parseFloat(this._normalizeInputValue())||0;i=this._correctRounding(i,n*e);var o=this.option("min"),a=this.option("max");(0,l.isDefined)(o)&&(i=Math.max(o,i)),(0,l.isDefined)(a)&&(i=Math.min(a,i)),this._saveValueChangeEvent(t),this.option("value",i)}},_correctRounding:function(e,t){var n=/[,.](.*)/,i=n.test(e),o=n.test(t);if(i||o){var a=i?n.exec(e)[0].length:0,r=o?n.exec(t)[0].length:0,s=y.max(a,r);return e=this._round(e+t,s)}return e+t},_round:function(e,t){t=t||0;var n=Math.pow(10,t);return e*=n,e=Math.round(e)/n},_renderValueChangeEvent:function(){this.callBase();var e=(0,g.addNamespace)("focusout",w);r.default.off(this.element(),e),r.default.on(this.element(),e,this._forceRefreshInputValue.bind(this))},_forceRefreshInputValue:function(){if("number"!==this.option("mode")){var e=this._input(),t=this._applyDisplayValueFormatter(this.option("value"));e.val(null),e.val(t)}},_valueChangeEventHandler:function(e){var t=this._input(),n=this._normalizeText(),i=this._parseValue(n),o="."!==n&&"-"!==n;return this._isValueValid()&&!this._validateValue(i)?void t.val(this._applyDisplayValueFormatter(i)):(o&&this.callBase(e,isNaN(i)?null:i),this._applyValueBoundaries(n,i),void this.validationRequest.fire({value:i,editor:this}))},_applyValueBoundaries:function(e,t){var n=this._isValueIncomplete(e),i=this._isValueInRange(e);n||i||null===t||Number(e)!==t&&this._input().val(this._applyDisplayValueFormatter(t))},_replaceCommaWithPoint:function(e){return e.replace(",",".")},_inputIsInvalid:function(){var e="number"===this.option("mode"),t=this._input().get(0).validity;return e&&t&&t.badInput},_renderDisplayText:function(e){this._inputIsInvalid()||this.callBase(e)},_isValueIncomplete:function(e){var t=/(^-$)|(^-?\d*\.$)|(\d+e-?$)/i;return t.test(e)},_isValueInRange:function(e){return(0,u.inRange)(e,this.option("min"),this.option("max"))},_isNumber:function(e){return null!==this._parseValue(e)},_validateValue:function(e){var t=this._normalizeText(),n=this._isValueValid(),i=!0,o=this._isNumber(t);return isNaN(Number(e))&&(i=!1),!e&&n?i=!0:o||n||(i=!1),this.option({isValid:i,validationError:i?null:{editorSpecific:!0,message:this.option("invalidValueMessage")}}),i},_normalizeInputValue:function(){return this._parseValue(this._normalizeText())},_normalizeText:function(){var e=this._input().val().trim();return this._replaceCommaWithPoint(e)},_parseValue:function(e){var t=parseFloat(e);return isNaN(t)?null:(0,u.fitIntoRange)(t,this.option("min"),this.option("max"))},_clearValue:function(){this._inputIsInvalid()&&(this._input().val(""),this._validateValue()),this.callBase()},reset:function(){null===this.option("value")?(this.option("text",""),this._renderValue()):this.option("value",null)},_optionChanged:function(e){switch(e.name){case"value":this._validateValue(e.value),this._setSubmitValue(e.value),this.callBase(e),this._resumeValueChangeAction();break;case"step":this._renderProps();break;case"min":case"max":this._renderProps(),this.option("value",this._parseValue(this.option("value")));break;case"showSpinButtons":case"useLargeSpinButtons":this._updateButtons(["spins"]);break;case"invalidValueMessage":break;default:this.callBase(e)}}}),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(5)),y=o(n(212)),b=o(n(620)),x=n(8),w=o(n(25)),C=n(0),k="dx-numberbox-spin",S="dx-numberbox-spin-container",D="dx-numberbox-spin-touch-friendly",I=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_attachEvents",value:function(e,t){var n=this.editor,i=(0,x.addNamespace)(w.default.down,n.NAME),o=t.children(),a=n._createAction(function(e){return n._spinButtonsPointerDownHandler(e)});v.default.off(t,i),v.default.on(t,i,function(e){return a({event:e})}),b.default.getInstance(o.eq(0)).option("onChange",function(e){return n._spinUpChangeHandler(e)}),b.default.getInstance(o.eq(1)).option("onChange",function(e){return n._spinDownChangeHandler(e)})}},{key:"_create",value:function(){var e=this.editor,t=(0,m.default)("<div>").addClass(S),n=(0,m.default)("<div>").appendTo(t),i=(0,m.default)("<div>").appendTo(t),o=this._getOptions();return this._addToContainer(t),e._createComponent(n,b.default,(0,C.extend)({direction:"up"},o)),e._createComponent(i,b.default,(0,C.extend)({direction:"down"},o)),this._legacyRender(e.$element(),this._isTouchFriendly(),o.visible),{instance:t,$element:t}}},{key:"_getOptions",value:function(){var e=this.editor,t=this._isVisible(),n=e.option("disabled");return{visible:t,disabled:n}}},{key:"_isVisible",value:function(){var e=this.editor;return l(_(t.prototype),"_isVisible",this).call(this)&&e.option("showSpinButtons")}},{key:"_isTouchFriendly",value:function(){var e=this.editor;return e.option("showSpinButtons")&&e.option("useLargeSpinButtons")}},{key:"_legacyRender",value:function(e,t,n){e.toggleClass(D,t),e.toggleClass(k,n)}},{key:"update",value:function(){var e=l(_(t.prototype),"update",this).call(this);if(e){var n=this.editor,i=this.instance,o=n.$element(),a=this._isVisible(),r=this._isTouchFriendly(),s=i.children(),u=b.default.getInstance(s.eq(0)),c=b.default.getInstance(s.eq(1)),d=this._getOptions();u.option(d),c.option(d),this._legacyRender(o,r,a)}}}]),t}(y.default);t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=i(n(16)),l=n(0),u=n(8),c=i(n(25)),d=n(140),h=i(n(103)),f=n(6),p="dx-numberbox-spin",g="dx-numberbox-spin-button",_=100,m="dxNumberBox",v=(0,u.addNamespace)(c.default.up,m),y=(0,u.addNamespace)(c.default.cancel,m),b=s.default.inherit({_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{direction:"up",onChange:null,activeStateEnabled:!0,hoverStateEnabled:!0})},_initMarkup:function(){this.callBase();var e=p+"-"+this.option("direction");this.$element().addClass(g).addClass(e),this._spinIcon=(0,o.default)("<div>").addClass(e+"-icon").appendTo(this.$element())},_render:function(){this.callBase();var e=(0,u.addNamespace)(c.default.down,this.NAME),t=this.$element();r.default.off(t,e),r.default.on(t,e,this._spinDownHandler.bind(this)),this._spinChangeHandler=this._createActionByOption("onChange")},_spinDownHandler:function(e){e.preventDefault(),this._clearTimer(),r.default.on(this.$element(),h.default.name,function(){this._feedBackDeferred=new f.Deferred,(0,d.lock)(this._feedBackDeferred),this._spinChangeHandler({event:e}),this._holdTimer=setInterval(this._spinChangeHandler,_,{event:e})}.bind(this));var t=a.default.getDocument();r.default.on(t,v,this._clearTimer.bind(this)),r.default.on(t,y,this._clearTimer.bind(this)),this._spinChangeHandler({event:e})},_dispose:function(){this._clearTimer(),this.callBase()},_clearTimer:function(){r.default.off(this.$element(),h.default.name);var e=a.default.getDocument();r.default.off(e,v),r.default.off(e,y),this._feedBackDeferred&&this._feedBackDeferred.resolve(),this._holdTimer&&clearInterval(this._holdTimer)},_optionChanged:function(e){switch(e.name){case"onChange":case"direction":this._invalidate();break;default:this.callBase(e)}}}),x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=i(n(622));(0,o.default)("dxDateBox",a.default);var r=a.default;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(8),a=n(1),r=n(33),s=n(0),l=n(29),u=i(n(5)),c=n(623),d=i(n(32)),h=n(208),f=n(161),p=i(n(624)),g=i(n(45)),_=i(n(13)),m=i(n(23)),v="dateBoxMask",y=1,b=-1,x=p.default.inherit({_supportedKeys:function(e){var t=this,n=this.callBase(e),i=function(e){var i=n[(0,o.normalizeKeyName)(e)];return i&&i.apply(t,[e])},a=function(e,n){return t._shouldUseOriginalHandler(e)?i.apply(t,[e]):n.apply(t,[e])};return(0,s.extend)({},n,{del:function(e){return a(e,function(e){t._revertPart(y),t._isAllSelected()||e.preventDefault()})},backspace:function(e){return a(e,function(e){t._revertPart(b),t._isAllSelected()||e.preventDefault()})},home:function(e){return a(e,function(e){t._selectFirstPart(),e.preventDefault()})},end:function(e){return a(e,function(e){t._selectLastPart(),e.preventDefault()})},escape:function(e){return a(e,function(e){t._revertChanges(e)})},enter:function(e){return a(e,function(e){t._enterHandler(e)})},leftArrow:function(e){return a(e,function(e){t._selectNextPart(b),e.preventDefault()})},rightArrow:function(e){return a(e,function(e){t._selectNextPart(y),e.preventDefault()})},upArrow:function(e){return a(e,function(e){t._upDownArrowHandler(y),e.preventDefault()})},downArrow:function(e){return a(e,function(e){t._upDownArrowHandler(b),e.preventDefault()})}})},_shouldUseOriginalHandler:function(e){var t=["backspace","del"],n=this.option("opened")&&e&&t.indexOf((0,o.normalizeKeyName)(e))===-1;return!this._useMaskBehavior()||n||e&&e.altKey},_upDownArrowHandler:function(e){this._setNewDateIfEmpty();var t=this._getActivePartValue(this._initialMaskValue),n=this._getActivePartValue(),i=n-t;this._loadMaskValue(this._initialMaskValue),this._partIncrease(i+e,!0)},_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{useMaskBehavior:!1,emptyDateValue:new Date(2e3,0,1,0,0,0)})},_isSingleCharKey:function(e){var t=e.originalEvent.data||e.originalEvent.key;return"string"==typeof t&&1===t.length&&!e.ctrl&&!e.alt},_isSingleDigitKey:function(e){var t,n=null===(t=e.originalEvent)||void 0===t?void 0:t.data;return 1===(null===n||void 0===n?void 0:n.length)&&parseInt(n,10)},_useBeforeInputEvent:function(){var e=_.default.real();return e.android&&e.version[0]>4},_keyboardHandler:function(e){var t=this,n=e.originalEvent.key,i=this.callBase(e);return!this._useMaskBehavior()||this._useBeforeInputEvent()?i:(m.default.chrome&&"Process"===e.key&&0===e.code.indexOf("Digit")?(n=e.code.replace("Digit",""),this._processInputKey(n),this._maskInputHandler=function(){t._renderSelectedPart()}):this._isSingleCharKey(e)&&(this._processInputKey(n),e.originalEvent.preventDefault()),i)},_maskBeforeInputHandler:function(e){var t=this;this._maskInputHandler=null;var n=e.originalEvent.inputType;"insertCompositionText"===n&&(this._maskInputHandler=function(){t._renderSelectedPart()});var i="deleteContentBackward"===n,o="deleteContentForward"===n;if(i||o){var a=i?b:y;this._maskInputHandler=function(){t._revertPart(),t._selectNextPart(a)}}if(this._useMaskBehavior()&&this._isSingleCharKey(e)){var r=e.originalEvent.data;return this._processInputKey(r),e.preventDefault(),!0}},_keyPressHandler:function(e){var t=e.originalEvent;"insertCompositionText"===(null===t||void 0===t?void 0:t.inputType)&&this._isSingleDigitKey(e)&&(this._processInputKey(t.data),this._renderDisplayText(this._getDisplayedText(this._maskValue)),this._selectNextPart()),this.callBase(e),this._maskInputHandler&&(this._maskInputHandler(),this._maskInputHandler=null)},_processInputKey:function(e){this._isAllSelected()&&(this._activePartIndex=0),this._setNewDateIfEmpty(),isNaN(parseInt(e))?this._searchString(e):this._searchNumber(e)},_isAllSelected:function(){var e=this._caret();
return e.end-e.start===this.option("text").length},_getFormatPattern:function(){if(this._formatPattern)return this._formatPattern;var e=this._strategy.getDisplayFormat(this.option("displayFormat")),t=(0,a.isString)(e)&&!d.default._getPatternByFormat(e);return t?this._formatPattern=e:this._formatPattern=(0,f.getFormat)(function(t){return d.default.format(t,e)}),this._formatPattern},_setNewDateIfEmpty:function(){if(!this._maskValue){var e="time"===this.option("type")?new Date(null):new Date;this._maskValue=e,this._initialMaskValue=e,this._renderDateParts()}},_partLimitsReached:function(e){var t=String(e).length,n=this._getActivePartProp("pattern").length,i=1===n,o=i?t:Math.min(n,t),a=this._searchValue.length===o,r=parseInt(this._searchValue+"0")>e;return a||r},_searchNumber:function(e){var t=this._getActivePartLimits(),n=t.max,i=String(n).length;this._searchValue=(this._searchValue+e).substr(-i),isNaN(this._searchValue)&&(this._searchValue=e),this._setActivePartValue(this._searchValue),this._partLimitsReached(n)&&this._selectNextPart(y)},_searchString:function(e){if(isNaN(parseInt(this._getActivePartProp("text")))){for(var t=this._getActivePartProp("limits")(this._maskValue),n=this._searchValue+e.toLowerCase(),i=t.max-t.min,o=0;o<=i;o++)if(this._loadMaskValue(this._initialMaskValue),this._partIncrease(o+1),0===this._getActivePartProp("text").toLowerCase().indexOf(n))return void(this._searchValue=n);this._setNewDateIfEmpty(),this._searchValue&&(this._clearSearchValue(),this._searchString(e))}},_clearSearchValue:function(){this._searchValue=""},_revertPart:function(e){if(!this._isAllSelected()){var t=this._getActivePartValue(this.option("emptyDateValue"));this._setActivePartValue(t),this._selectNextPart(e)}this._clearSearchValue()},_useMaskBehavior:function(){return this.option("useMaskBehavior")&&"text"===this.option("mode")},_prepareRegExpInfo:function(){this._regExpInfo=(0,h.getRegExpInfo)(this._getFormatPattern(),d.default);var e=this._regExpInfo.regexp,t=e.flags,n=g.default.convertDigits(this._regExpInfo.regexp.source,!1);this._regExpInfo.regexp=RegExp(n,t)},_initMaskState:function(){this._activePartIndex=0,this._formatPattern=null,this._prepareRegExpInfo(),this._loadMaskValue()},_renderMask:function(){this.callBase(),this._detachMaskEvents(),this._clearMaskState(),this._useMaskBehavior()&&(this._attachMaskEvents(),this._initMaskState(),this._renderDateParts())},_renderDateParts:function(){if(this._useMaskBehavior()){var e=this.option("text")||this._getDisplayedText(this._maskValue);e&&(this._dateParts=(0,c.renderDateParts)(e,this._regExpInfo),this._isFocused()&&this._selectNextPart())}},_detachMaskEvents:function(){u.default.off(this._input(),"."+v)},_attachMaskEvents:function(){var e=this;u.default.on(this._input(),(0,o.addNamespace)("dxclick",v),this._maskClickHandler.bind(this)),u.default.on(this._input(),(0,o.addNamespace)("paste",v),this._maskPasteHandler.bind(this)),u.default.on(this._input(),(0,o.addNamespace)("drop",v),function(){e._renderSelectedPart()}),u.default.on(this._input(),(0,o.addNamespace)("compositionend",v),this._maskCompositionEndHandler.bind(this)),this._useBeforeInputEvent()&&u.default.on(this._input(),(0,o.addNamespace)("beforeinput",v),this._maskBeforeInputHandler.bind(this))},_renderSelectedPart:function(){this._renderDisplayText(this._getDisplayedText(this._maskValue)),this._selectNextPart()},_selectLastPart:function(){this.option("text")&&(this._activePartIndex=this._dateParts.length,this._selectNextPart(b))},_selectFirstPart:function(){this.option("text")&&(this._activePartIndex=-1,this._selectNextPart(y))},_onMouseWheel:function(e){this._useMaskBehavior()&&this._partIncrease(e.delta>0?y:b,e)},_selectNextPart:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;if(this.option("text")&&!this._disposed){e&&(this._initialMaskValue=new Date(this._maskValue));var t=(0,l.fitIntoRange)(this._activePartIndex+e,0,this._dateParts.length-1);if(this._dateParts[t].isStub){var n=0===t&&e<0||t===this._dateParts.length-1&&e>0;if(!n)return void this._selectNextPart(e>=0?e+1:e-1);t=this._activePartIndex}this._activePartIndex!==t&&this._clearSearchValue(),this._activePartIndex=t,this._caret(this._getActivePartProp("caret"))}},_getRealLimitsPattern:function(){if("d"===this._getActivePartProp("pattern")[0])return"dM"},_getActivePartLimits:function(e){var t=this._getActivePartProp("limits");return t(this._maskValue,e&&this._getRealLimitsPattern())},_getActivePartValue:function(e){e=e||this._maskValue;var t=this._getActivePartProp("getter");return(0,a.isFunction)(t)?t(e):e[t]()},_addLeadingZeroes:function(e){var t=this._searchValue.match(/^0+/),n=this._getActivePartLimits(),i=String(n.max).length;return((t&&t[0]||"")+String(e)).substr(-i)},_setActivePartValue:function(e,t){t=t||this._maskValue;var n=this._getActivePartProp("setter"),i=this._getActivePartLimits();e=(0,l.inRange)(e,i.min,i.max)?e:e%10,e=this._addLeadingZeroes((0,l.fitIntoRange)(e,i.min,i.max)),(0,a.isFunction)(n)?n(t,e):t[n](e),this._renderDisplayText(this._getDisplayedText(t)),this._renderDateParts()},_getActivePartProp:function(e){if(this._dateParts&&this._dateParts[this._activePartIndex])return this._dateParts[this._activePartIndex][e]},_loadMaskValue:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.dateOption("value");this._maskValue=e&&new Date(e),this._initialMaskValue=e&&new Date(e)},_saveMaskValue:function(){var e=this._maskValue&&new Date(this._maskValue);e&&"date"===this.option("type")&&e.setHours(0,0,0,0),this._initialMaskValue=new Date(e),this.dateOption("value",e)},_revertChanges:function(){this._loadMaskValue(),this._renderDisplayText(this._getDisplayedText(this._maskValue)),this._renderDateParts()},_renderDisplayText:function(e){this.callBase(e),this._useMaskBehavior()&&this.option("text",e)},_partIncrease:function(e,t){this._setNewDateIfEmpty();var n=this._getActivePartLimits(t),i=n.max,o=n.min,a=i-o;1===a&&a++;var r=e+this._getActivePartValue();r>i?r=this._applyLimits(r,{limitBase:o,limitClosest:i,limitDelta:a}):r<o&&(r=this._applyLimits(r,{limitBase:i,limitClosest:o,limitDelta:a})),this._setActivePartValue(r)},_applyLimits:function(e,t){var n=t.limitBase,i=t.limitClosest,o=t.limitDelta,a=(e-i)%o;return a?n+a-1*(0,l.sign)(a):i},_maskClickHandler:function(){this.option("text")&&(this._activePartIndex=(0,c.getDatePartIndexByPosition)(this._dateParts,this._caret().start),(0,a.isDefined)(this._activePartIndex)?this._caret(this._getActivePartProp("caret")):this._selectLastPart())},_maskCompositionEndHandler:function(e){var t=this;if(m.default.msie&&this._isSingleDigitKey(e)){var n=e.originalEvent.data;this._processInputKey(n)}else this._input().val(this._getDisplayedText(this._maskValue)),this._selectNextPart(),this._maskInputHandler=function(){t._renderSelectedPart()}},_maskPasteHandler:function(e){var t=this._replaceSelectedText(this.option("text"),this._caret(),(0,r.clipboardText)(e)),n=d.default.parse(t,this._getFormatPattern());n&&this._isDateValid(n)&&(this._maskValue=n,this._renderDisplayText(this._getDisplayedText(this._maskValue)),this._renderDateParts(),this._selectNextPart()),e.preventDefault()},_isDateValid:function(e){return(0,a.isDate)(e)&&!isNaN(e)},_isValueDirty:function(){var e=this.dateOption("value");return(this._maskValue&&this._maskValue.getTime())!==(e&&e.getTime())},_fireChangeEvent:function(){this._clearSearchValue(),this._isValueDirty()&&u.default.trigger(this._input(),"change")},_enterHandler:function(e){this._fireChangeEvent(),this._selectNextPart(y),e.preventDefault()},_focusOutHandler:function(e){this.callBase(e),this._useMaskBehavior()&&!e.isDefaultPrevented()&&(this._fireChangeEvent(),this._selectFirstPart(e))},_valueChangeEventHandler:function(e){var t=this.option("text");this._useMaskBehavior()?(this._saveValueChangeEvent(e),t?null===this._maskValue&&this._loadMaskValue(t):this._maskValue=null,this._saveMaskValue()):this.callBase(e)},_optionChanged:function(e){switch(e.name){case"useMaskBehavior":this._renderMask();break;case"displayFormat":case"mode":this.callBase(e),this._renderMask();break;case"value":this._loadMaskValue(),this.callBase(e),this._renderDateParts();break;case"emptyDateValue":break;default:this.callBase(e)}},_clearMaskState:function(){this._clearSearchValue(),delete this._dateParts,delete this._activePartIndex,delete this._maskValue},reset:function(){this.callBase(),this._clearMaskState(),this._activePartIndex=0},_clean:function(){this.callBase(),this._detachMaskEvents(),this._clearMaskState()}}),w=x;t.default=w,e.exports=t.default},function(e,t,n){t.getDatePartIndexByPosition=t.renderDateParts=void 0;var i=n(208),o=n(0),a=n(29),r=n(4),s=void 0,l=function(e){return e.getMonth()+1},u=function(e,t){var n=e.getDate(),i=p("M",e),o=(0,a.fitIntoRange)(parseInt(t),i.min,i.max);e.setMonth(o-1,1);var r=p("dM",e),s=r.min,l=r.max,u=(0,a.fitIntoRange)(n,s,l);e.setDate(u)},c={a:function(e){return e.getHours()<12?0:1},E:"getDay",y:"getFullYear",M:l,L:l,d:"getDate",H:"getHours",h:"getHours",m:"getMinutes",s:"getSeconds",S:"getMilliseconds"},d=(0,o.extend)({},(0,i.getPatternSetters)(),{a:function(e,t){var n=e.getHours(),i=n>=12;i!==!!parseInt(t)&&e.setHours((n+12)%24)},d:function(e,t){var n=p("dM",e).max;t>n&&e.setMonth(e.getMonth()+1),e.setDate(t)},h:function(e,t){var n=e.getHours()>=12;e.setHours(+t%12+(n?12:0))},M:u,L:u,E:function(e,t){t<0||e.setDate(e.getDate()-e.getDay()+parseInt(t))},y:function(e,t){var n=e.getFullYear(),i=String(t).length,o=String(p("y",e).max).length,a=parseInt(String(n).substr(0,o-i)+t);e.setFullYear(a)}}),h=function(e){var t=function(){return e};return c[e]||t},f=function(e,t){for(var n=t.regexp.exec(e),i=0,o=0,a=[],l=1;l<n.length;l++){i=o,o=i+n[l].length;var u=t.patterns[l-1].replace(/^'|'$/g,""),c=h(u[0]);a.push({index:l-1,isStub:u===n[l],caret:{start:i,end:o},pattern:u,text:n[l],limits:p.bind(s,u[0]),setter:d[u[0]]||r.noop,getter:c})}return a};t.renderDateParts=f;var p=function(e,t,n){var i={y:{min:0,max:9999},M:{min:1,max:12},L:{min:1,max:12},d:{min:1,max:31},dM:{min:1,max:new Date(t.getFullYear(),t.getMonth()+1,0).getDate()},E:{min:0,max:6},H:{min:0,max:23},h:{min:0,max:23},m:{min:0,max:59},s:{min:0,max:59},S:{min:0,max:999},a:{min:0,max:1}};return i[n||e]||i.getAmPm},g=function(e,t){for(var n=0;n<e.length;n++){var i=e[n].caret.end>=t;if(!e[n].isStub&&i)return n}return null};t.getDatePartIndexByPosition=g},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(7),a=i(n(9)),r=n(1),s=n(33),l=n(3),u=n(60),c=n(0),d=n(46),h=i(n(13)),f=i(n(28)),p=i(n(20)),g=i(n(145)),_=i(n(62)),m=i(n(192)),v=i(n(32)),y=i(n(10)),b=i(n(436)),x=i(n(625)),w=i(n(628)),C=i(n(629)),k=i(n(631)),S=(0,o.getWindow)(),D="dx-datebox",I="dx-auto-width",T="dx-show-invalid-badge",O="dx-clear-button-area",E="dx-datebox-wrapper",A={calendar:"calendar",rollers:"rollers",list:"list","native":"native"},P={date:"date",datetime:"datetime",time:"time"},R={calendar:"Calendar",dateView:"DateView","native":"Native",calendarWithTime:"CalendarWithTime",list:"List"},M={Calendar:b.default,DateView:x.default,Native:w.default,CalendarWithTime:C.default,List:k.default},B=m.default.inherit({_supportedKeys:function(){return(0,c.extend)(this.callBase(),this._strategy.supportedKeys())},_renderButtonContainers:function(){this.callBase.apply(this,arguments),this._strategy.customizeButtons()},_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{type:"date",showAnalogClock:!0,value:null,dateSerializationFormat:void 0,min:void 0,max:void 0,displayFormat:null,interval:30,disabledDates:null,pickerType:A.calendar,invalidDateMessage:y.default.format("dxDateBox-validation-datetime"),dateOutOfRangeMessage:y.default.format("validation-range"),applyButtonText:y.default.format("OK"),adaptivityEnabled:!1,calendarOptions:{},useHiddenSubmitElement:!0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios"},options:{"dropDownOptions.showTitle":!0}},{device:{platform:"android"},options:{buttonsLocation:"bottom after"}},{device:function(){var e=h.default.real(),t=e.platform;return"ios"===t||"android"===t},options:{pickerType:A.native}},{device:function(e){var t=h.default.real(),n=t.platform,i=t.version;return"generic"===n&&"desktop"!==e.deviceType||"android"===n&&(0,u.compare)(i,[4,4])<0},options:{pickerType:A.rollers}},{device:{platform:"generic",deviceType:"desktop"},options:{buttonsLocation:"bottom after"}}])},_initOptions:function(e){this._userOptions=(0,c.extend)({},e),this.callBase(e),this._updatePickerOptions()},_updatePickerOptions:function(){var e=this.option("pickerType"),t=this.option("type");e!==A.list||t!==P.datetime&&t!==P.date||(e=A.calendar),t===P.time&&e===A.calendar&&(e=A.list),this.option("showDropDownButton","generic"!==h.default.real().platform||e!==A.native),this._pickerType=e},_init:function(){this._initStrategy(),this.option((0,c.extend)({},this._strategy.getDefaultOptions(),this._userOptions)),delete this._userOptions,this.callBase()},_toLowerCaseFirstLetter:function(e){return e.charAt(0).toLowerCase()+e.substr(1)},_initStrategy:function(){var e=this._getStrategyName(this._getFormatType()),t=M[e];this._strategy&&this._strategy.NAME===e||(this._strategy=new t(this))},_getFormatType:function(){var e=this.option("type"),t=/h|m|s/g.test(e),n=/d|M|Y/g.test(e),i="";return n&&(i+=P.date),t&&(i+=P.time),i},_getStrategyName:function(e){var t=this._pickerType;return t===A.rollers?R.dateView:t===A.native?R.native:e===P.date?R.calendar:e===P.datetime?R.calendarWithTime:R.list},_initMarkup:function(){this.$element().addClass(D),this.callBase(),this._refreshFormatClass(),this._refreshPickerTypeClass(),this._strategy.renderInputMinMax(this._input())},_render:function(){this.callBase(),this._formatValidationIcon()},_renderDimensions:function(){this.callBase(),this.$element().toggleClass(I,!this.option("width")),this._dimensionChanged()},_dimensionChanged:function(){if(this.callBase(arguments),this._popup){var e,t;null===(e=(t=this._strategy)._updatePopupHeight)||void 0===e?void 0:e.call(t)}},_refreshFormatClass:function(){var e=this.$element();(0,l.each)(P,function(t,n){e.removeClass(D+"-"+n)}),e.addClass(D+"-"+this.option("type"))},_refreshPickerTypeClass:function(){var e=this.$element();(0,l.each)(A,function(t,n){e.removeClass(D+"-"+n)}),e.addClass(D+"-"+this._pickerType)},_formatValidationIcon:function(){if((0,o.hasWindow)()){var e=this._input().get(0),t=this.option("rtlEnabled"),n=this._getClearButtonWidth(),i=this._getLongestElementDimensions(),a=parseFloat(S.getComputedStyle(e).width)-n,r=i.width>a,s=e.style;this.$element().toggleClass(T,!r),r?(void 0===this._storedPadding&&(this._storedPadding=t?i.leftPadding:i.rightPadding),t?s.paddingLeft=0:s.paddingRight=0):t?s.paddingLeft=this._storedPadding+"px":s.paddingRight=this._storedPadding+"px"}},_getClearButtonWidth:function(){var e=0;if(this._isClearButtonVisible()&&""===this._input().val()){var t=this.$element().find("."+O).get(0);e=parseFloat(S.getComputedStyle(t).width)}return e},_getLongestElementDimensions:function(){var e=this._strategy.getDisplayFormat(this.option("displayFormat")),t=v.default.format(g.default.getLongestDate(e,v.default.getMonthNames(),v.default.getDayNames()),e),n=this._input(),i=n.get(0),o=(0,s.createTextElementHiddenCopy)(n,t),a=void 0!==this._storedPadding,r=a?this._storedPadding:0;o.appendTo(this.$element());var l=parseFloat(S.getComputedStyle(o.get(0)).width),u=parseFloat(S.getComputedStyle(i).paddingRight),c=parseFloat(S.getComputedStyle(i).paddingLeft),d=l+c+u+r;return o.remove(),{width:d,leftPadding:c,rightPadding:u}},_getKeyboardListeners:function(){return this.callBase().concat([this._strategy&&this._strategy.getKeyboardListener()])},_renderPopup:function(){this.callBase(),this._popup._wrapper().addClass(E),this._renderPopupWrapper()},_popupConfig:function(){var e=this.callBase();return(0,c.extend)(this._strategy.popupConfig(e),{title:this._getPopupTitle(),dragEnabled:!1})},_renderPopupWrapper:function(){if(this._popup){var e=this.$element(),t=(0,c.extend)({},P,A);(0,l.each)(t,function(t,n){e.removeClass(E+"-"+n)}.bind(this)),this._popup._wrapper().addClass(E+"-"+this.option("type")).addClass(E+"-"+this._pickerType)}},_renderPopupContent:function(){this.callBase(),this._strategy.renderPopupContent()},_getFirstPopupElement:function(){return this._strategy.getFirstPopupElement()||this.callBase()},_getLastPopupElement:function(){return this._strategy.getLastPopupElement()||this.callBase()},_popupShowingHandler:function(){this.callBase(),this._strategy.popupShowingHandler()},_popupShownHandler:function(){this.callBase(),this._strategy.renderOpenedState()},_popupHiddenHandler:function(){this.callBase(),this._strategy.renderOpenedState(),this._strategy.popupHiddenHandler()},_visibilityChanged:function(e){e&&this._formatValidationIcon()},_clearValueHandler:function(e){this.option("text",""),this.callBase(e)},_readOnlyPropValue:function(){if(this._pickerType===A.rollers)return!0;var e=h.default.real().platform,t=this._isNativeType()&&("ios"===e||"android"===e);return t?this.option("readOnly"):this.callBase()},_isClearButtonVisible:function(){return this.callBase()&&!this._isNativeType()},_renderValue:function(){var e=this.dateOption("value");return this.option("text",this._getDisplayedText(e)),this._strategy.renderValue(),this.callBase()},_setSubmitValue:function(){var e=this.dateOption("value"),t=this.option("dateSerializationFormat"),n=g.default.SUBMIT_FORMATS_MAP[this.option("type")],i=t?_.default.serializeDate(e,t):g.default.toStandardDateFormat(e,n);this._getSubmitElement().val(i)},_getDisplayedText:function(e){var t,n=this.option("mode");if("text"===n){var i=this._strategy.getDisplayFormat(this.option("displayFormat"));t=v.default.format(e,i)}else{var o=this._getFormatByMode(n);t=o?v.default.format(e,o):g.default.toStandardDateFormat(e,n)}return t},_getFormatByMode:function(e){return(0,d.inputType)(e)?null:g.default.FORMATS_MAP[e]},_valueChangeEventHandler:function(e){var t=this.option("text"),n=this.dateOption("value");if(t===this._getDisplayedText(n))return void this._applyInternalValidation(n);var i=this._getParsedDate(t),o=n||this._getDateByDefault(),a=this.option("type"),r=g.default.mergeDates(o,i,a),s=i&&"time"===a?r:i;if(this._applyInternalValidation(s).isValid){var l=this._getDisplayedText(r);o&&r&&o.getTime()===r.getTime()&&l!==t?this._renderValue():this.dateValue(r,e)}},_getDateByDefault:function(){return this._strategy.useCurrentDateByDefault()&&this._strategy.getDefaultDate()},_getParsedDate:function(e){var t=this._strategy.getDisplayFormat(this.option("displayFormat")),n=this._strategy.getParsedText(e,t);return(0,r.isDefined)(n)?n:void 0},_applyInternalValidation:function(e){var t=this.option("text"),n=!!t&&null!==e,i=!!e&&(0,r.isDate)(e)&&!isNaN(e.getTime()),o=i&&p.default.dateInRange(e,this.dateOption("min"),this.dateOption("max"),this.option("type")),a=!n&&!e||o,s="";return i?o||(s=this.option("dateOutOfRangeMessage")):s=this.option("invalidDateMessage"),this.option({isValid:a,validationError:a?null:{editorSpecific:!0,message:s}}),{isValid:a,isDate:i}},_applyCustomValidation:function(e){this.validationRequest.fire({editor:this,value:this._serializeDate(e)})},_isValueChanged:function(e){var t=this.dateOption("value"),n=t&&t.getTime(),i=e&&e.getTime();return n!==i},_isTextChanged:function(e){var t=this.option("text"),n=e&&this._getDisplayedText(e)||"";return t!==n},_renderProps:function(){this.callBase(),this._input().attr("autocomplete","off")},_renderOpenedState:function(){this._isNativeType()||this.callBase(),this._strategy.isAdaptivityChanged()&&this._refreshStrategy()},_getPopupTitle:function(){var e=this.option("placeholder");if(e)return e;var t=this.option("type");return t===P.time?y.default.format("dxDateBox-simulatedDataPickerTitleTime"):t===P.date||t===P.datetime?y.default.format("dxDateBox-simulatedDataPickerTitleDate"):""},_renderPlaceholder:function(){this._popup&&this._popup.option("title",this._getPopupTitle()),this.callBase()},_refreshStrategy:function(){this._strategy.dispose(),this._initStrategy(),this.option(this._strategy.getDefaultOptions()),this._refresh()},_applyButtonHandler:function(e){var t=this._strategy.getValue(),n=this._applyInternalValidation(t),i=n.isValid,o=n.isDate;if(i)this.dateValue(t,e.event);else if(o){var a=this._getDisplayedText(t);this.option("text",a),this._renderDisplayText(a)}this.callBase()},_dispose:function(){var e;this.callBase(),null===(e=this._strategy)||void 0===e?void 0:e.dispose()},_isNativeType:function(){return this._pickerType===A.native},_optionChanged:function(e){switch(e.name){case"showClearButton":case"buttons":this.callBase.apply(this,arguments),this._formatValidationIcon();break;case"pickerType":this._updatePickerOptions({pickerType:e.value}),this._refreshStrategy(),this._refreshPickerTypeClass(),this._invalidate();break;case"type":this._updatePickerOptions({format:e.value}),this._refreshStrategy(),this._refreshFormatClass(),this._renderPopupWrapper(),this._formatValidationIcon(),this._updateValue();break;case"placeholder":this._renderPlaceholder();break;case"min":case"max":var t=this.option("isValid");this._applyInternalValidation(this.dateOption("value")),t||this._applyCustomValidation(this.dateOption("value")),this._invalidate();break;case"dateSerializationFormat":case"interval":case"disabledDates":case"calendarOptions":this._invalidate();break;case"displayFormat":this.option("text",this._getDisplayedText(this.dateOption("value"))),this._renderInputValue();break;case"text":this._strategy.textChangedHandler(e.value),this.callBase.apply(this,arguments);break;case"isValid":this.callBase.apply(this,arguments),this._formatValidationIcon();break;case"showDropDownButton":this._formatValidationIcon(),this.callBase.apply(this,arguments);break;case"readOnly":this.callBase.apply(this,arguments),this._formatValidationIcon();break;case"invalidDateMessage":case"dateOutOfRangeMessage":case"adaptivityEnabled":case"showAnalogClock":break;default:this.callBase.apply(this,arguments)}},_getSerializationFormat:function(){var e=this.option("value");if(this.option("dateSerializationFormat")&&(0,f.default)().forceIsoDateParsing)return this.option("dateSerializationFormat");if((0,r.isNumeric)(e))return"number";if((0,r.isString)(e))return _.default.getDateSerializationFormat(e)},_updateValue:function(e){this.callBase(),this._applyInternalValidation(e||this.dateOption("value"))},dateValue:function(e,t){var n=this._isValueChanged(e);return n&&t&&this._saveValueChangeEvent(t),n||(this._isTextChanged(e)?this._updateValue(e):""===this.option("text")&&this._applyCustomValidation(e)),this.dateOption("value",e)},dateOption:function(e,t){return 1===arguments.length?_.default.deserializeDate(this.option(e)):void this.option(e,this._serializeDate(t))},_serializeDate:function(e){var t=this._getSerializationFormat();return _.default.serializeDate(e,t)},reset:function(){this.callBase(),this._updateValue(this.dateOption("value"))}});(0,a.default)("dxDateBox",B);var V=B;t.default=V,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(626)),s=i(n(218)),l=n(46),u=n(0),c=i(n(145)),d=i(n(10)),h=(0,a.getWindow)(),f=s.default.inherit({NAME:"DateView",getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{openOnFieldClick:!0,applyButtonText:d.default.format("OK"),"dropDownOptions.showTitle":!0})},getDisplayFormat:function(e){return e||c.default.FORMATS_MAP[this.dateBox.option("type")]},popupConfig:function(e){return{toolbarItems:this.dateBox._popupToolbarItemsConfig(),onInitialized:e.onInitialized,defaultOptionsRules:[{device:{platform:"android"},options:{width:333,height:331}},{device:function(e){var t=e.platform;return"generic"===t||"ios"===t},options:{width:"auto",height:"auto"}},{device:function(e){var t=e.platform,n=e.phone;return"generic"===t&&n},options:{width:333,maxWidth:"100%",maxHeight:"100%",height:"auto",position:{collision:"flipfit flip"}}},{device:{platform:"ios",phone:!0},options:{width:"100%",position:{my:"bottom",at:"bottom",of:h}}}]}},_renderWidget:function(){if((0,l.inputType)(this.dateBox.option("mode"))&&this.dateBox._isNativeType()||this.dateBox.option("readOnly"))return void(this._widget&&(this._widget.$element().remove(),this._widget=null));var e=this._getPopup();if(this._widget)this._widget.option(this._getWidgetOptions());else{var t=(0,o.default)("<div>").appendTo(e.$content());this._widget=this._createWidget(t)}this._widget.$element().appendTo(this._getWidgetContainer())},_getWidgetName:function(){return r.default},renderOpenedState:function(){this.callBase(),this._widget&&this._widget.option("value",this._widget._getCurrentDate())},_getWidgetOptions:function(){return{value:this.dateBoxValue()||new Date,type:this.dateBox.option("type"),minDate:this.dateBox.dateOption("min")||new Date(1900,0,1),maxDate:this.dateBox.dateOption("max")||new Date(Date.now()+50*c.default.ONE_YEAR),onDisposing:function(){this._widget=null}.bind(this)}}}),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(52)),r=i(n(627)),s=i(n(20)),l=n(3),u=n(0),c=i(n(145)),d=i(n(9)),h=i(n(32)),f="dx-dateview",p="dx-dateview-compact",g="dx-dateview-wrapper",_="dx-dateview-rollers",m="dx-dateviewroller",v={date:"date",datetime:"datetime",time:"time"},y={year:"year",month:"month",day:"day",hours:"hours"},b=a.default.inherit({_valueOption:function(){var e=this.option("value"),t=new Date(e);return!e||isNaN(t)?this._getDefaultDate():t},_getDefaultDate:function(){var e=new Date;return this.option("type")===v.date?new Date(e.getFullYear(),e.getMonth(),e.getDate()):e},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{minDate:c.default.MIN_DATEVIEW_DEFAULT_DATE,maxDate:c.default.MAX_DATEVIEW_DEFAULT_DATE,type:v.date,value:new Date,applyCompactClass:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(e){return"desktop"!==e.deviceType},options:{applyCompactClass:!0}}])},_render:function(){this.callBase(),this.$element().addClass(f),this._toggleFormatClasses(this.option("type")),this._toggleCompactClass()},_toggleFormatClasses:function(e,t){this.$element().addClass(f+"-"+e),t&&this.$element().removeClass(f+"-"+t)},_toggleCompactClass:function(){this.$element().toggleClass(p,this.option("applyCompactClass"))},_wrapper:function(){return this._$wrapper},_renderContentImpl:function(){this._$wrapper=(0,o.default)("<div>").addClass(g),this._renderRollers(),this._$wrapper.appendTo(this.$element())},_renderRollers:function(){this._$rollersContainer||(this._$rollersContainer=(0,o.default)("<div>").addClass(_)),this._$rollersContainer.empty(),this._createRollerConfigs(),this._rollers={};var e=this;(0,l.each)(e._rollerConfigs,function(t){var n=(0,o.default)("<div>").appendTo(e._$rollersContainer).addClass(m+"-"+e._rollerConfigs[t].type);e._rollers[e._rollerConfigs[t].type]=e._createComponent(n,r.default,{items:e._rollerConfigs[t].displayItems,selectedIndex:e._rollerConfigs[t].selectedIndex,showScrollbar:!1,onStart:function(n){var i=n.component;i._toggleActive(!0),e._setActiveRoller(e._rollerConfigs[t],i.option("selectedIndex"))},onEnd:function(e){var t=e.component;t._toggleActive(!1)},onClick:function(n){var i=n.component;i._toggleActive(!0),e._setActiveRoller(e._rollerConfigs[t],i.option("selectedIndex")),e._setRollerState(e._rollerConfigs[t],i.option("selectedIndex")),i._toggleActive(!1)},onSelectedIndexChanged:function(n){var i=n.component;e._setRollerState(e._rollerConfigs[t],i.option("selectedIndex"))}})}),e._$rollersContainer.appendTo(e._wrapper())},_createRollerConfigs:function(e){var t=this;e=e||t.option("type"),t._rollerConfigs={},h.default.getFormatParts(c.default.FORMATS_MAP[e]).forEach(function(e){t._createRollerConfig(e)})},_createRollerConfig:function(e){for(var t=c.default.DATE_COMPONENTS_INFO[e],n=this._calculateRollerConfigValueRange(e),i=n.startValue,o=n.endValue,a=t.formatter,r=this._getCurrentDate(),s={type:e,setValue:t.setter,valueItems:[],displayItems:[],getIndex:function(e){return e[t.getter]()-i}},l=i;l<=o;l++)s.valueItems.push(l),s.displayItems.push(a(l,r));s.selectedIndex=s.getIndex(r),this._rollerConfigs[e]=s},_setActiveRoller:function(e){var t=e&&this._rollers[e.type];(0,l.each)(this._rollers,function(){this.toggleActiveState(this===t)})},_updateRollersPosition:function(){var e=this;(0,l.each)(this._rollers,function(t){var n=e._rollerConfigs[t].getIndex(e._getCurrentDate());this.option("selectedIndex",n)})},_setRollerState:function(e,t){if(t!==e.selectedIndex){var n=e.valueItems[t],i=e.setValue,o=new Date(this._getCurrentDate()),a=o.getDate(),r=this.option("minDate"),l=this.option("maxDate");e.type===y.month?a=Math.min(a,c.default.getMaxMonthDay(o.getFullYear(),n)):e.type===y.year&&(a=Math.min(a,c.default.getMaxMonthDay(n,o.getMonth()))),o.setDate(a),o[i](n);var u=s.default.normalizeDate(o,r,l);o=c.default.mergeDates(u,o,"time"),o=s.default.normalizeDate(o,r,l),this.option("value",o),e.selectedIndex=t}e.type===y.year&&this._refreshRollers(),e.type===y.month&&(this._refreshRoller(y.day),this._refreshRoller(y.hours))},_refreshRoller:function(e){var t=this._rollers[e];if(t){this._createRollerConfig(e);var n=this._rollerConfigs[e];e!==y.day&&n.displayItems.toString()===t.option("items").toString()||t.option({items:n.displayItems,selectedIndex:n.selectedIndex})}},_getCurrentDate:function(){var e=this._valueOption(),t=this.option("minDate"),n=this.option("maxDate");return s.default.normalizeDate(e,t,n)},_calculateRollerConfigValueRange:function(e){var t=this._getCurrentDate(),n=this.option("minDate"),i=this.option("maxDate"),o=s.default.sameYear(t,n),a=o&&t.getMonth()===n.getMonth(),r=s.default.sameYear(t,i),l=r&&t.getMonth()===i.getMonth(),u=a&&t.getDate()===n.getDate(),d=l&&t.getDate()===i.getDate(),h=c.default.DATE_COMPONENTS_INFO[e],f=h.startValue,p=h.endValue;return e===y.year&&(f=n.getFullYear(),p=i.getFullYear()),e===y.month&&(o&&(f=n.getMonth()),r&&(p=i.getMonth())),e===y.day&&(p=c.default.getMaxMonthDay(t.getFullYear(),t.getMonth()),o&&a&&(f=n.getDate()),r&&l&&(p=i.getDate())),e===y.hours&&(f=u?n.getHours():f,p=d?i.getHours():p),{startValue:f,endValue:p}},_refreshRollers:function(){this._refreshRoller(y.month),this._refreshRoller(y.day),this._refreshRoller(y.hours)},_optionChanged:function(e){switch(e.name){case"minDate":case"maxDate":case"type":this._renderRollers(),this._toggleFormatClasses(e.value,e.previousValue);break;case"visible":this.callBase(e),e.value&&this._renderRollers();break;case"value":this.option("value",this._valueOption()),this._refreshRollers(),this._updateRollersPosition();break;default:this.callBase(e)}},_clean:function(){this.callBase(),delete this._$rollersContainer}});(0,d.default)("dxDateView",b);var x=b;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(9)),s=n(0),l=n(3),u=n(8),c=n(21),d=i(n(109)),h=i(n(13)),f=i(n(43)),p=n(30),g="dx-dateviewroller",_="dx-state-active",m="dx-dateviewroller-current",v="dx-dateview-item",y="dx-dateview-item-selected",b="dx-dateview-item-selected-frame",x="dx-dateview-item-selected-border",w=d.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{showScrollbar:!1,useNative:!1,selectedIndex:0,bounceEnabled:!1,items:[],showOnClick:!1,onClick:null,onSelectedIndexChanged:null})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"generic"},options:{scrollByContent:!0}}])},_init:function(){this.callBase(),this._renderSelectedItemFrame()},_render:function(){this.callBase(),this.$element().addClass(g),this._renderContainerClick(),this._renderItems(),this._renderSelectedValue(),this._renderItemsClick(),this._renderWheelEvent(),this._wrapAction("_endAction",this._endActionHandler.bind(this)),this._renderSelectedIndexChanged()},
_renderSelectedIndexChanged:function(){this._selectedIndexChanged=this._createActionByOption("onSelectedIndexChanged")},_renderWheelEvent:function(){var e=this;a.default.on(this._$container,"dxmousewheel",function(t){e._isWheelScrolled=!0})},_renderContainerClick:function(){if(this.option("showOnClick")){var e=(0,u.addNamespace)(c.name,this.NAME),t=this._createActionByOption("onClick");a.default.off(this._$container,e),a.default.on(this._$container,e,function(e){t({event:e})})}},_wrapAction:function(e,t){var n=this._strategy,i=n[e];n[e]=function(){return t.apply(this,arguments),i.apply(this,arguments)}},_renderItems:function(){var e=this.option("items")||[],t=(0,o.default)();this._$content.empty(),e.forEach(function(e){t=t.add((0,o.default)("<div>").addClass(v).append(e))}),this._$content.append(t),this._$items=t,this.update()},_renderSelectedItemFrame:function(){(0,o.default)("<div>").addClass(b).append((0,o.default)("<div>").addClass(x)).appendTo(this._$container)},_renderSelectedValue:function(e){var t=this._fitIndex(e||this.option("selectedIndex"));this._moveTo({top:this._getItemPosition(t)}),this._renderActiveStateItem()},_fitIndex:function(e){var t=this.option("items")||[],n=t.length;return e>=n?n-1:e<0?0:e},_getItemPosition:function(e){return Math.round(this._itemHeight()*e)},_renderItemsClick:function(){var e=this._getItemSelector(),t=(0,u.addNamespace)(c.name,this.NAME);a.default.off(this.$element(),t,e),a.default.on(this.$element(),t,e,this._itemClickHandler.bind(this))},_getItemSelector:function(){return"."+v},_itemClickHandler:function(e){this.option("selectedIndex",this._itemElementIndex(e.currentTarget))},_itemElementIndex:function(e){return this._itemElements().index(e)},_itemElements:function(){return this.$element().find(this._getItemSelector())},_renderActiveStateItem:function(){var e=this.option("selectedIndex");(0,l.each)(this._$items,function(t){(0,o.default)(this).toggleClass(y,e===t)})},_shouldScrollToNeighborItem:function(){return"desktop"===h.default.real().deviceType&&this._isWheelScrolled},_moveTo:function(e){e=this._normalizeLocation(e);var t=this._location(),n={x:-(t.left-e.left),y:-(t.top-e.top)};if(this._isVisible()&&(n.x||n.y))if(this._strategy._prepareDirections(!0),this._animation&&!this._shouldScrollToNeighborItem()){var i=this;f.default.stop(this._$content),f.default.animate(this._$content,{duration:200,type:"slide",to:{top:Math.floor(n.y)},complete:function(){(0,p.resetPosition)(i._$content),i._strategy.handleMove({delta:n})}}),delete this._animation}else this._strategy.handleMove({delta:n})},_validate:function(e){return this._strategy.validate(e)},_fitSelectedIndexInRange:function(e){var t=this.option("items").length;return Math.max(Math.min(e,t-1),0)},_isInNullNeighborhood:function(e){var t=.1;return-t<=e&&e<=t},_getSelectedIndexAfterScroll:function(e){var t=-this._location().top,n=e*this._itemHeight(),i=t-n;if(this._isInNullNeighborhood(i))return e;var o=i>0?1:-1,a=this._fitSelectedIndexInRange(e+o);return a},_getNewSelectedIndex:function(e){if(this._shouldScrollToNeighborItem())return this._getSelectedIndexAfterScroll(e);this._animation=!0;var t=-this._location().top/this._itemHeight();return Math.round(t)},_endActionHandler:function(){var e=this.option("selectedIndex"),t=this._getNewSelectedIndex(e);t===e?this._renderSelectedValue(t):this.option("selectedIndex",t),this._isWheelScrolled=!1},_itemHeight:function(){var e=this._$items.first();return e.height()},_toggleActive:function(e){this.$element().toggleClass(_,e)},_isVisible:function(){return this._$container.is(":visible")},_fireSelectedIndexChanged:function(e,t){this._selectedIndexChanged({value:e,previousValue:t,event:void 0})},_visibilityChanged:function(e){this.callBase(e),e&&this._renderSelectedValue(this.option("selectedIndex")),this.toggleActiveState(!1)},toggleActiveState:function(e){this.$element().toggleClass(m,e)},_refreshSelectedIndex:function(){var e=this.option("selectedIndex"),t=this._fitIndex(e);t===e?this._renderActiveStateItem():this.option("selectedIndex",t)},_optionChanged:function(e){switch(e.name){case"selectedIndex":this._fireSelectedIndexChanged(e.value,e.previousValue),this._renderSelectedValue(e.value);break;case"items":this._renderItems(),this._refreshSelectedIndex();break;case"onClick":case"showOnClick":this._renderContainerClick();break;case"onSelectedIndexChanged":this._renderSelectedIndexChanged();break;default:this.callBase(e)}}});(0,r.default)("dxDateViewRoller",w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(4),a=i(n(218)),r=n(46),s=n(12),l=i(n(145)),u=i(n(62)),c=n(0),d=i(n(13)),h=a.default.inherit({NAME:"Native",popupConfig:function(e){return(0,c.extend)({},e,{width:"auto"})},getParsedText:function(e,t){return e?"datetime"===this.dateBox.option("type")?new Date(e.replace(/-/g,"/").replace("T"," ").split(".")[0]):this._isTextInput()?this.callBase(e,t):l.default.fromStandardDateFormat(e):null},_isTextInput:function(){return"text"===this.dateBox._input().prop("type")},renderPopupContent:o.noop,_getWidgetName:o.noop,_getWidgetOptions:o.noop,_getDateBoxType:function(){var e=this.dateBox.option("type");return(0,s.inArray)(e,l.default.SUPPORTED_FORMATS)===-1?e="date":"datetime"!==e||(0,r.inputType)(e)||(e="datetime-local"),e},customizeButtons:function(){var e=this.dateBox.getButton("dropDown");d.default.real().android&&e&&e.on("click",function(){this.dateBox._input().get(0).click()}.bind(this))},getDefaultOptions:function(){return{mode:this._getDateBoxType()}},getDisplayFormat:function(e){var t=this._getDateBoxType();return e||l.default.FORMATS_MAP[t]},renderInputMinMax:function(e){e.attr({min:u.default.serializeDate(this.dateBox.dateOption("min"),"yyyy-MM-dd"),max:u.default.serializeDate(this.dateBox.dateOption("max"),"yyyy-MM-dd")})}}),f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(436)),s=i(n(630)),l=i(n(32)),u=n(0),c=i(n(216)),d=i(n(145)),h=(0,a.getWindow)(),f=573,p="dx-datebox-adaptivity-mode",g=r.default.inherit({NAME:"CalendarWithTime",getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{applyValueMode:"useButtons",buttonsLocation:"bottom after","dropDownOptions.showTitle":!1})},getDisplayFormat:function(e){return e||"shortdateshorttime"},_is24HourFormat:function(){return l.default.is24HourFormat(this.getDisplayFormat(this.dateBox.option("displayFormat")))},_renderWidget:function(){this.callBase(),this._timeView=this.dateBox._createComponent((0,o.default)("<div>"),s.default,{value:this.dateBoxValue(),_showClock:!this._isShrinkView(),use24HourFormat:this._is24HourFormat(),onValueChanged:this._valueChangedHandler.bind(this),stylingMode:this.dateBox.option("stylingMode")}),this._timeView.registerKeyHandler("escape",this._escapeHandler.bind(this))},renderOpenedState:function(){this.callBase();var e=this._getPopup();e&&e._wrapper().toggleClass(p,this._isSmallScreen()),clearTimeout(this._repaintTimer),this._repaintTimer=setTimeout(function(){this._getPopup()&&this._getPopup().repaint()}.bind(this),0)},isAdaptivityChanged:function(){var e=this._isShrinkView(),t=this._currentAdaptiveMode;return e!==t?(this._currentAdaptiveMode=e,void 0!==t):this.callBase()},_updateValue:function(e){var t=this.dateBoxValue();t||e||(t=new Date,d.default.normalizeTime(t)),this.callBase(),this._timeView&&(t&&this._timeView.option("value",t),this._timeView.option("use24HourFormat",this._is24HourFormat()))},_isSmallScreen:function(){return(0,o.default)(h).width()<=f},_isShrinkView:function(){return!this.dateBox.option("showAnalogClock")||this.dateBox.option("adaptivityEnabled")&&this._isSmallScreen()},_getBoxItems:function(){var e=[{ratio:0,shrink:0,baseSize:"auto",name:"calendar"}];return this._isShrinkView()||e.push({ratio:0,shrink:0,baseSize:"auto",name:"time"}),e},renderPopupContent:function(){this.callBase(),this._currentAdaptiveMode=this._isShrinkView();var e=this._getPopup().$content();this._box=this.dateBox._createComponent((0,o.default)("<div>").appendTo(e),c.default,{direction:"row",crossAlign:"start",items:this._getBoxItems(),itemTemplate:function(e){var t=(0,o.default)("<div>");switch(e.name){case"calendar":t.append(this._widget.$element()),this._isShrinkView()&&t.append(this._timeView.$element());break;case"time":t.append(this._timeView.$element())}return t}.bind(this)}),this._attachTabHandler()},popupConfig:function(e){var t=this.callBase(e),n=(0,u.extend)(t,{width:"auto",onShowing:function(){if("fallback"===this._box.option("_layoutStrategy")){var e=this._getPopup().$content().find(".dx-timeview-clock").css("minWidth");this._timeView.$element().css("maxWidth",e)}}.bind(this)});return n},getFirstPopupElement:function(){return this._timeView._hourBox.$element().find("input")},_attachTabHandler:function(){var e=this.dateBox,t=function(t){t.shiftKey&&(t.preventDefault(),e.focus())};this._timeView._hourBox.registerKeyHandler("tab",t)},_preventFocusOnPopup:function(e){(0,o.default)(e.target).hasClass("dx-texteditor-input")||(this.callBase.apply(this,arguments),this.dateBox._hasFocusClass()||this.dateBox.focus())},getValue:function(){var e=this._widget.option("value"),t=this._timeView.option("value");return e=e?new Date(e):new Date,e.setHours(t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds()),e},dispose:function(){clearTimeout(this._removeMinWidthTimer),clearTimeout(this._repaintTimer),this.callBase()}}),_=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(52)),r=i(n(126)),s=i(n(127)),l=i(n(216)),u=n(0),c=i(n(9)),d=i(n(32)),h=i(n(145)),f="dx-timeview",p="dx-timeview-clock",g="dx-timeview-field",_="dx-timeview-hourarrow",m="dx-timeview-time-separator",v="dx-timeview-format12",y=-1,b=1,x="dx-timeview-minutearrow",w=function(e,t,n){C(e,t,n)},C=function(e,t,n){e.css("transform","rotate("+t+"deg) translate(0,"+n+"px)")},k=a.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{value:new Date(Date.now()),use24HourFormat:!0,_showClock:!0,_arrowOffset:0,stylingMode:void 0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"android"},options:{_arrowOffset:15}},{device:{platform:"generic"},options:{_arrowOffset:5}}])},_getValue:function(){return this.option("value")||new Date},_init:function(){this.callBase(),this.$element().addClass(f)},_render:function(){this.callBase(),this._renderBox(),this._updateTime()},_renderBox:function(){var e=(0,o.default)("<div>").appendTo(this.$element()),t=[];this.option("_showClock")&&t.push({ratio:1,shrink:0,baseSize:"auto",template:this._renderClock.bind(this)}),t.push({ratio:0,shrink:0,baseSize:50,template:this._renderField.bind(this)}),this._createComponent(e,l.default,{height:"100%",width:"100%",direction:"col",items:t})},_renderClock:function(e,t,n){this._$hourArrow=(0,o.default)("<div>").addClass(_),this._$minuteArrow=(0,o.default)("<div>").addClass(x);var i=(0,o.default)(n);i.addClass(p).append(this._$hourArrow).append(this._$minuteArrow),this.setAria("role","presentation",i)},_updateClock:function(){var e=this._getValue(),t=e.getHours()/12*360+e.getMinutes()/60*30,n=e.getMinutes()/60*360;w(this._$hourArrow,t,this.option("_arrowOffset")),w(this._$minuteArrow,n,this.option("_arrowOffset"))},_getBoxItems:function(e){var t=this,n=[{ratio:0,shrink:0,baseSize:"auto",template:function(){return t._hourBox.$element()}},{ratio:0,shrink:0,baseSize:"auto",template:(0,o.default)("<div>").addClass(m).text(d.default.getTimeSeparator())},{ratio:0,shrink:0,baseSize:"auto",template:function(){return t._minuteBox.$element()}}];return e&&n.push({ratio:0,shrink:0,baseSize:"auto",template:function(){return t._format12.$element()}}),n},_renderField:function(){var e=!this.option("use24HourFormat");return this._createHourBox(e),this._createMinuteBox(),e&&this._createFormat12Box(),this._createComponent((0,o.default)("<div>").addClass(g),l.default,{direction:"row",align:"center",crossAlign:"center",items:this._getBoxItems(e)}).$element()},_createHourBox:function(e){var t=this,n=this._hourBox=this._createComponent((0,o.default)("<div>"),r.default,(0,u.extend)({min:-1,max:e?12:24,value:this._getValue().getHours(),onValueChanged:this._onHourBoxValueChanged.bind(this),onKeyboardHandled:function(e){return t._keyboardHandler(e)}},this._getNumberBoxConfig()));n.setAria("label","hours")},_isPM:function(){return!this.option("use24HourFormat")&&1===this._format12.option("value")},_onHourBoxValueChanged:function(e){var t=this._getValue(),n=new Date(t),i=this._convertMaxHourToMin(e.value);this._isPM()&&(i+=12),n.setHours(i),h.default.normalizeTime(n),this.option("value",n)},_convertMaxHourToMin:function(e){var t=this.option("use24HourFormat")?24:12;return(t+e)%t},_createMinuteBox:function(){var e=this,t=this._minuteBox=this._createComponent((0,o.default)("<div>"),r.default,(0,u.extend)({min:-1,max:60,value:this._getValue().getMinutes(),onKeyboardHandled:function(t){return e._keyboardHandler(t)},onValueChanged:function(t){var n=t.value,i=t.component,o=(60+n)%60;i.option("value",o);var a=new Date(e._getValue());a.setMinutes(o),h.default.normalizeTime(a),e.option("value",a)}},this._getNumberBoxConfig()));t.setAria("label","minutes")},_createFormat12Box:function(){var e=this,t=d.default.getPeriodNames(),n=this._format12=this._createComponent((0,o.default)("<div>").addClass(v),s.default,{items:[{value:y,text:t[0]},{value:b,text:t[1]}],valueExpr:"value",displayExpr:"text",onKeyboardHandled:function(t){return e._keyboardHandler(t)},onValueChanged:function(t){var n=t.value,i=e._getValue().getHours(),o=new Date(e._getValue()),a=(i+12*n)%24;o.setHours(a),e.option("value",o)},value:this._getValue().getHours()>=12?b:y,stylingMode:this.option("stylingMode")});n.setAria("label","type")},_refreshFormat12:function(){if(!this.option("use24HourFormat")){var e=this._getValue(),t=e.getHours(),n=t>=12,i=n?b:y;this._silentEditorValueUpdate(this._format12,i)}},_silentEditorValueUpdate:function(e,t){e&&(e._suppressValueChangeAction(),e.option("value",t),e._resumeValueChangeAction())},_getNumberBoxConfig:function(){return{showSpinButtons:!0,displayValueFormatter:function(e){return(e<10?"0":"")+e},stylingMode:this.option("stylingMode")}},_normalizeHours:function(e){return this.option("use24HourFormat")?e:e%12||12},_updateField:function(){var e=this._normalizeHours(this._getValue().getHours());this._silentEditorValueUpdate(this._hourBox,e),this._silentEditorValueUpdate(this._minuteBox,this._getValue().getMinutes()),this._refreshFormat12()},_updateTime:function(){this.option("_showClock")&&this._updateClock(),this._updateField()},_visibilityChanged:function(e){e&&this._updateTime()},_optionChanged:function(e){switch(e.name){case"value":this._updateTime(),this.callBase(e);break;case"_arrowOffset":break;case"use24HourFormat":case"_showClock":case"stylingMode":this._invalidate();break;default:this.callBase(e)}}});(0,c.default)("dxTimeView",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(7),r=i(n(115)),s=i(n(218)),l=n(4),u=n(1),c=n(0),d=i(n(145)),h=i(n(32)),f=i(n(62)),p=n(213),g=(0,a.getWindow)(),_="date",m={min:new Date(0,0,0,0,0),max:new Date(0,0,0,23,59)},v=s.default.inherit({NAME:"List",supportedKeys:function(){return{tab:function(){this.option("opened")&&this.close()},space:l.noop,home:l.noop,end:l.noop}},getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{applyValueMode:"instantly"})},getDisplayFormat:function(e){return e||"shorttime"},popupConfig:function(e){return e},useCurrentDateByDefault:function(){return!0},getDefaultDate:function(){return new Date(null)},popupShowingHandler:function(){this.dateBox._dimensionChanged()},_renderWidget:function(){this.callBase(),this._refreshItems()},_getWidgetName:function(){return r.default},_getWidgetOptions:function(){return{itemTemplate:this._timeListItemTemplate.bind(this),onItemClick:this._listItemClickHandler.bind(this),tabIndex:-1,onFocusedItemChanged:this._refreshActiveDescendant.bind(this),selectionMode:"single"}},_refreshActiveDescendant:function(e){this.dateBox.setAria("activedescendant",""),this.dateBox.setAria("activedescendant",e.actionValue)},_refreshItems:function(){this._widgetItems=this._getTimeListItems(),this._widget.option("items",this._widgetItems)},renderOpenedState:function(){this._widget&&(this._widget.option("focusedElement",null),this._setSelectedItemsByValue(),this._widget.option("templatesRenderAsynchronously")?this._asyncScrollTimeout=setTimeout(this._scrollToSelectedItem.bind(this)):this._scrollToSelectedItem())},dispose:function(){this.callBase(),clearTimeout(this._asyncScrollTimeout)},_updateValue:function(){this._widget&&(this._refreshItems(),this._setSelectedItemsByValue(),this._scrollToSelectedItem())},_setSelectedItemsByValue:function(){var e=this.dateBoxValue(),t=this._getDateIndex(e);t===-1?this._widget.option("selectedItems",[]):this._widget.option("selectedIndex",t)},_scrollToSelectedItem:function(){this._widget.scrollToItem(this._widget.option("selectedIndex"))},_getDateIndex:function(e){for(var t=-1,n=0,i=this._widgetItems.length;n<i;n++)if(this._areDatesEqual(e,this._widgetItems[n])){t=n;break}return t},_areDatesEqual:function(e,t){return(0,u.isDate)(e)&&(0,u.isDate)(t)&&e.getHours()===t.getHours()&&e.getMinutes()===t.getMinutes()},_getTimeListItems:function(){var e=this.dateBox.dateOption("min")||this._getBoundaryDate("min"),t=this.dateBox.dateOption("max")||this._getBoundaryDate("max"),n=this.dateBox.dateOption("value")||null,i=t-e,o=e.getMinutes()%this.dateBox.option("interval");return i<0?[]:(i>d.default.ONE_DAY&&(i=d.default.ONE_DAY),n-e<d.default.ONE_DAY?this._getRangeItems(e,new Date(e),i):(e=this._getBoundaryDate("min"),e.setMinutes(o),n&&Math.abs(n-t)<d.default.ONE_DAY&&(i=(60*t.getHours()+Math.abs(t.getMinutes()-o))*d.default.ONE_MINUTE),this._getRangeItems(e,new Date(e),i)))},_getRangeItems:function(e,t,n){for(var i=[],o=this.dateBox.option("interval");t-e<=n;)i.push(new Date(t)),t.setMinutes(t.getMinutes()+o);return i},_getBoundaryDate:function(e){var t=m[e],n=new Date((0,l.ensureDefined)(this.dateBox.dateOption("value"),0));return new Date(n.getFullYear(),n.getMonth(),n.getDate(),t.getHours(),t.getMinutes())},_timeListItemTemplate:function(e){var t=this.dateBox.option("displayFormat");return h.default.format(e,this.getDisplayFormat(t))},_listItemClickHandler:function(e){this.dateBox.option("opened",!1);var t=this.dateBox.option("value"),n=e.itemData,i=n.getHours(),o=n.getMinutes(),a=n.getSeconds(),r=n.getFullYear(),s=n.getMonth(),l=n.getDate();t?(t=this.dateBox.option("dateSerializationFormat")?f.default.deserializeDate(t):new Date(t),t.setHours(i),t.setMinutes(o),t.setSeconds(a),t.setFullYear(r),t.setMonth(s),t.setDate(l)):t=new Date(r,s,l,i,o,0,0),this.dateBoxValue(t,e.event)},getKeyboardListener:function(){return this._widget},_updatePopupHeight:function(){var e=(0,p.getSizeValue)(this.dateBox.option("dropDownOptions.height"));if(void 0===e||"auto"===e){this.dateBox._setPopupOption("height","auto");var t=this._widget.$element().outerHeight(),n=.45*(0,o.default)(g).height();this.dateBox._setPopupOption("height",Math.min(t,n))}this.dateBox._timeList&&this.dateBox._timeList.updateDimensions()},getParsedText:function(e,t){var n=this.callBase(e,t);return n&&(n=d.default.mergeDates(n,new Date(null),_)),n}}),y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(291)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=n(194),v=o(n(292)),y=o(n(2)),b=n(30),x=n(0),w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_useDefaultAnimation",value:function(){return!0}},{key:"_defaultPositionRendering",value:function(e,t,n){var i=this,o=this.getDrawerInstance();if((0,y.default)(o.content()).css(o.isHorizontalDirection()?"width":"height",e.maxSize),o.getMinSize()){var a="padding";switch(o.calcTargetPosition()){case"left":a+="Right";break;case"right":a+="Left";break;case"top":a+="Bottom";break;case"bottom":a+="Top"}(0,y.default)(o.viewContent()).css(a,o.getMinSize())}if(n){var r={$element:(0,y.default)(o.viewContent()),position:e.contentPosition,direction:o.calcTargetPosition(),duration:o.option("animationDuration"),complete:function(){i._elementsAnimationCompleteHandler()}};m.animation.moveTo(r)}else o.isHorizontalDirection()?(0,b.move)((0,y.default)(o.viewContent()),{left:e.contentPosition}):(0,b.move)((0,y.default)(o.viewContent()),{top:e.contentPosition})}},{key:"_getPositionRenderingConfig",value:function(e){return(0,x.extend)(l(_(t.prototype),"_getPositionRenderingConfig",this).call(this,e),{contentPosition:this._getPanelSize(e)*this.getDrawerInstance()._getPositionCorrection(),maxSize:this._getPanelSize(!0)})}},{key:"onPanelContentRendered",value:function(){(0,y.default)(this.getDrawerInstance().viewContent()).addClass("dx-theme-background-color")}}]),t}(v.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=n(194),v=o(n(292)),y=o(n(2)),b=n(0),x=n(40),w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_slidePositionRendering",value:function(e,t,n){if(n){var i=(0,b.extend)(e.defaultAnimationConfig,{$element:e.$panel,margin:e.panelOffset,duration:this.getDrawerInstance().option("animationDuration"),direction:e.direction});m.animation.margin(i)}else e.$panel.css("margin"+(0,x.camelize)(e.direction,!0),e.panelOffset)}},{key:"_expandPositionRendering",value:function(e,t,n){var i=this.getDrawerInstance();if(n){var o=(0,b.extend)(e.defaultAnimationConfig,{$element:e.$panel,size:e.size,duration:i.option("animationDuration"),direction:e.direction});m.animation.size(o)}else i.isHorizontalDirection()?(0,y.default)(e.$panel).css("width",e.size):(0,y.default)(e.$panel).css("height",e.size)}},{key:"_getPositionRenderingConfig",value:function(e){return(0,b.extend)(l(_(t.prototype),"_getPositionRenderingConfig",this).call(this,e),{panelOffset:this._getPanelOffset(e)})}},{key:"isViewContentFirst",value:function(e,t){return(t?"left"===e:"right"===e)||"bottom"===e}}]),t}(v.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=n(194),v=o(n(292)),y=o(n(2)),b=n(30),x=o(n(68)),w=n(4),C=n(0),k=n(40),S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"renderPanelContent",value:function(e){var t=this;delete this._initialPosition;var n=this.getDrawerInstance(),i=n.option(),o=i.opened,a=i.minSize;n._overlay=n._createComponent(n.content(),x.default,{shading:!1,container:n.getOverlayTarget(),position:this._getOverlayPosition(),width:o?"auto":a||0,height:"100%",templatesRenderAsynchronously:n.option("templatesRenderAsynchronously"),animation:{show:{duration:0}},onPositioned:function(e){this._fixOverlayPosition(e.component.$content())}.bind(this),contentTemplate:n.option("template"),onContentReady:function(n){e.resolve(),t._processOverlayZIndex(n.component.content())},visible:!0,propagateOutsideClick:!0})}},{key:"_fixOverlayPosition",value:function(e){var t=(0,w.ensureDefined)(this._initialPosition,{left:0,top:0});(0,b.move)(e,t),"right"===this.getDrawerInstance().calcTargetPosition()&&e.css("left","auto"),"bottom"===this.getDrawerInstance().calcTargetPosition()&&(e.css("top","auto"),e.css("bottom","0px"))}},{key:"_getOverlayPosition",value:function(){var e=this.getDrawerInstance(),t=e.calcTargetPosition(),n={};switch(t){case"left":n={my:"top left",at:"top left"};break;case"right":n={my:e.option("rtlEnabled")?"top left":"top right",at:"top right"};break;case"top":case"bottom":n={my:t,at:t}}return n.of=e.getOverlayTarget(),n}},{key:"refreshPanelElementSize",value:function(e){var t=this.getDrawerInstance(),n=t.getOverlay();t.isHorizontalDirection()?(n.option("height","100%"),n.option("width",e?t.getRealPanelWidth():this._getPanelSize(t.option("opened")))):(n.option("width",n.option("container").width()),n.option("height",e?t.getRealPanelHeight():this._getPanelSize(t.option("opened"))))}},{key:"onPanelContentRendered",value:function(){this._updateViewContentStyles()}},{key:"_updateViewContentStyles",value:function(){var e=this.getDrawerInstance();(0,y.default)(e.viewContent()).css("padding"+(0,k.camelize)(e.calcTargetPosition(),!0),e.option("minSize")),(0,y.default)(e.viewContent()).css("transform","inherit")}},{key:"_slidePositionRendering",value:function(e,t,n){var i=this.getDrawerInstance();this._initialPosition=i.isHorizontalDirection()?{left:e.panelOffset}:{top:e.panelOffset};var o=i.calcTargetPosition();if(this._updateViewContentStyles(),n){var a=(0,C.extend)(e.defaultAnimationConfig,{$element:e.$panel,position:e.panelOffset,duration:i.option("animationDuration"),direction:o});m.animation.moveTo(a)}else i.isHorizontalDirection()?(0,b.move)(e.$panel,{left:e.panelOffset}):(0,b.move)(e.$panel,{top:e.panelOffset})}},{key:"_expandPositionRendering",value:function(e,t,n){var i=this.getDrawerInstance();this._initialPosition={left:0};var o=i.calcTargetPosition();if(this._updateViewContentStyles(),(0,b.move)(e.$panelOverlayContent,{left:0}),n){var a=(0,C.extend)(e.defaultAnimationConfig,{$element:e.$panelOverlayContent,size:e.size,duration:i.option("animationDuration"),direction:o,marginTop:e.marginTop});m.animation.size(a)}else i.isHorizontalDirection()?(0,y.default)(e.$panelOverlayContent).css("width",e.size):((0,y.default)(e.$panelOverlayContent).css("height",e.size),"bottom"===o&&(0,y.default)(e.$panelOverlayContent).css("marginTop",e.marginTop))}},{key:"_getPositionRenderingConfig",value:function(e){var n=this.getDrawerInstance(),i=l(_(t.prototype),"_getPositionRenderingConfig",this).call(this,e);return(0,C.extend)(i,{panelOffset:this._getPanelOffset(e)*this.getDrawerInstance()._getPositionCorrection(),$panelOverlayContent:n.getOverlay().$content(),
marginTop:n.getRealPanelHeight()-i.size})}},{key:"getPanelContent",value:function(){return(0,y.default)(this.getDrawerInstance().getOverlay().content())}},{key:"_processOverlayZIndex",value:function(e){var t=(0,y.default)(e).get(0).style,n=t.zIndex||1;this.getDrawerInstance().setZIndex(n)}},{key:"isViewContentFirst",value:function(e){return"right"===e||"bottom"===e}}]),t}(v.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(271),a=i(n(9)),r=i(n(11)),s=i(n(2)),l=n(4),u=n(6),c=n(0),d=n(3),h=n(1),f=n(7),p=i(n(5)),g=n(55),_=i(n(88)),m=i(n(16)),v=n(24),y=(0,f.getWindow)(),b="dx-widget",x="dx-deferrendering",w="dx-pending-rendering",C="dx-pending-rendering-manual",k="dx-pending-rendering-active",S="dx-visible-while-pending-rendering",D="dx-invisible-while-pending-rendering",I="dx-loadindicator-container",T="dx-deferrendering-loadindicator-container",O="dx-deferrendering-load-indicator",E="content",A=["onRendered","onShown"],P=m.default.inherit({_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{showLoadIndicator:!1,renderWhen:void 0,animation:void 0,staggerItemSelector:void 0,onRendered:null,onShown:null})},_getAnonymousTemplateName:function(){return E},_init:function(){this.transitionExecutor=new o.TransitionExecutor,this._initElement(),this._initRender(),this._$initialContent=this.$element().clone().contents(),this._initActions(),this.callBase()},_initElement:function(){this.$element().addClass(x)},_initRender:function(){var e=this,t=this.$element(),n=this.option("renderWhen"),i=function(){return e._renderDeferredContent()};(0,h.isPromise)(n)?(0,u.fromPromise)(n).done(i):(t.data("dx-render-delegate",i),void 0===n&&t.addClass(C))},_initActions:function(){var e=this;this._actions={},(0,d.each)(A,function(t,n){e._actions[n]=e._createActionByOption(n)||l.noop})},_initMarkup:function(){this.callBase(),this._initContent||(this._initContent=this._renderContent,this._renderContent=function(){}),this._initContent()},_renderContentImpl:function(){this.$element().removeClass(b),this.$element().append(this._$initialContent),this._setLoadingState()},_renderDeferredContent:function(){var e=this,t=this.$element(),n=new u.Deferred;return t.removeClass(C),t.addClass(k),this._abortRenderTask(),this._renderTask=(0,l.executeAsync)(function(){e._renderImpl().done(function(){var i={element:t};e._actions.onShown([i]),n.resolve(i)}).fail(function(){n.rejectWith(n,arguments)})}),n.promise()},_isElementInViewport:function(e){var t=(0,v.getBoundingRect)(e);return t.bottom>=0&&t.right>=0&&t.top<=(y.innerHeight||r.default.getDocumentElement().clientHeight)&&t.left<=(y.innerWidth||r.default.getDocumentElement().clientWidth)},_animate:function(){var e,t=this,n=this.$element(),i=(0,f.hasWindow)()&&this.option("animation"),o=this.option("staggerItemSelector");return t.transitionExecutor.stop(),i?(o?n.find(o).each(function(){t._isElementInViewport(this)&&t.transitionExecutor.enter((0,s.default)(this),i)}):t.transitionExecutor.enter(n,i),e=t.transitionExecutor.start()):e=(new u.Deferred).resolve().promise(),e},_renderImpl:function(){var e=this.$element(),t={element:e},n=this._getTemplate(this._templateManager.anonymousTemplateName);return n&&n.render({container:e.empty(),noModel:!0}),this._setRenderedState(e),p.default.trigger(e,"dxcontentrendered"),this._actions.onRendered([t]),this._isRendered=!0,this._animate()},_setLoadingState:function(){var e=this.$element(),t=!!e.find("."+S).length;e.addClass(w),t||e.children().addClass(D),this.option("showLoadIndicator")&&this._showLoadIndicator(e)},_showLoadIndicator:function(e){this._$loadIndicator=new _.default((0,s.default)("<div>"),{visible:!0}).$element().addClass(O),(0,s.default)("<div>").addClass(I).addClass(T).append(this._$loadIndicator).appendTo(e)},_setRenderedState:function(){var e=this.$element();this._$loadIndicator&&this._$loadIndicator.remove(),e.removeClass(w),e.removeClass(k),(0,g.triggerShownEvent)(e.children())},_optionChanged:function(e){var t=e.value,n=e.previousValue;switch(e.name){case"renderWhen":n===!1&&t===!0?this._renderOrAnimate():n===!0&&t===!1&&(this.transitionExecutor.stop(),this._setLoadingState());break;case"showLoadIndicator":case"onRendered":case"onShown":break;default:this.callBase(e)}},_renderOrAnimate:function(){var e;return this._isRendered?(this._setRenderedState(),e=this._animate()):e=this._renderDeferredContent(),e},renderContent:function(){return this._renderOrAnimate()},_abortRenderTask:function(){this._renderTask&&(this._renderTask.abort(),this._renderTask=void 0)},_dispose:function(){this.transitionExecutor.stop(!0),this._abortRenderTask(),this._actions=void 0,this._$initialContent=void 0,this.callBase()}});(0,a.default)("dxDeferRendering",P);var R=P;t.default=R,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(192)),a=i(n(288)),r=n(4),s=n(1),l=n(3),u=n(75),c=n(6),d=i(n(2)),h=i(n(5)),f=n(0),p=n(293),g=i(n(9)),_=n(8),m=n(119),v=i(n(13)),y=i(n(11)),b=n(18),x=y.default.getActiveElement,w="dx-dropdownbox",C="content",k=v.default.real(),S=o.default.inherit({_supportedKeys:function(){return(0,f.extend)({},this.callBase(),{tab:function(e){if(this.option("opened")){var t=this._getTabbableElements(),n=e.shiftKey?t.last():t.first();n&&h.default.trigger(n,"focus"),e.preventDefault()}}})},_getTabbableElements:function(){return this._getElements().filter(u.tabbable)},_getElements:function(){return(0,d.default)(this.content()).find("*")},_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{acceptCustomValue:!1,contentTemplate:C,openOnFieldClick:!0,displayValueFormatter:function(e){return Array.isArray(e)?e.join(", "):e},useHiddenSubmitElement:!0})},_getAnonymousTemplateName:function(){return C},_initTemplates:function(){this.callBase()},_initMarkup:function(){this._initDataExpressions(),this.$element().addClass(w),this.callBase()},_setSubmitValue:function(){var e=this.option("value"),t=this._shouldUseDisplayValue(e)?this._displayGetter(e):e;this._getSubmitElement().val(t)},_shouldUseDisplayValue:function(e){return"this"===this.option("valueExpr")&&(0,s.isObject)(e)},_renderInputValue:function(){var e=this.callBase.bind(this),t=[];if(!this._dataSource)return e(t),(new c.Deferred).resolve();var n=this._getCurrentValue(),i=(0,r.ensureDefined)(n,[]);i=Array.isArray(i)?i:[i];var o=(0,l.map)(i,function(e){return this._loadItem(e).always(function(n){var i=this._displayGetter(n);t.push((0,r.ensureDefined)(i,e))}.bind(this))}.bind(this));return c.when.apply(this,o).always(function(){this.option("displayValue",t),e(t.length&&t)}.bind(this))},_loadItem:function(e){var t=new c.Deferred,n=this,i=(0,r.grep)(this.option("items")||[],function(t){return this._isValueEquals(this._valueGetter(t),e)}.bind(this))[0];return void 0!==i?t.resolve(i):this._loadValue(e).done(function(e){t.resolve(e)}).fail(function(i){n.option("acceptCustomValue")?t.resolve(e):t.reject()}),t.promise()},_popupElementTabHandler:function(e){if("tab"===(0,_.normalizeKeyName)(e)){var t=this._getTabbableElements().first().get(0),n=this._getTabbableElements().last().get(0),i=e.originalEvent.target,o=!(i!==t||!e.shift),a=!(i!==n||e.shift);(o||a)&&(this.close(),h.default.trigger(this._input(),"focus"),o&&e.originalEvent.preventDefault())}},_renderPopup:function(e){var t=this;this.callBase(),this.option("focusStateEnabled")&&m.keyboard.on(this.content(),null,function(e){return t._popupElementTabHandler(e)})},_renderPopupContent:function(){if(this.option("contentTemplate")!==C){var e=this._getTemplateByOption("contentTemplate");if(e&&this.option("contentTemplate")){var t=this._popup.$content(),n={value:this._fieldRenderData(),component:this};t.empty(),e.render({container:(0,b.getPublicElement)(t),model:n})}}},_canShowVirtualKeyboard:function(){return k.mac},_isNestedElementActive:function(){var e=x();return e&&this._popup.$content().get(0).contains(e)},_shouldCloseOnTargetScroll:function(){return"desktop"===k.deviceType&&this._canShowVirtualKeyboard()&&this._isNestedElementActive()},_popupHiddenHandler:function(){this.callBase(),this._popupPosition=void 0},_popupPositionedHandler:function(e){this.callBase(e),this._popupPosition=e.position},_getDefaultPopupPosition:function(e){var t=this.callBase(e),n=t.my,i=t.at;return{my:n,at:i,offset:{v:-1},collision:"flipfit"}},_popupConfig:function(){var e=this,t=this.option(),n=t.focusStateEnabled;return(0,f.extend)(this.callBase(),{tabIndex:-1,dragEnabled:!1,focusStateEnabled:n,contentTemplate:C,closeOnTargetScroll:this._shouldCloseOnTargetScroll.bind(this),position:(0,f.extend)(this.option("popupPosition"),{of:this.$element()}),onKeyboardHandled:function(t){return e.option("focusStateEnabled")&&e._popupElementTabHandler(t)},maxHeight:function(){var e,t=null===(e=this._popupPosition)||void 0===e?void 0:e.v.location;return(0,p.getElementMaxHeightByWindow)(this.$element(),t)}.bind(this)})},_popupShownHandler:function(){this.callBase();var e=this._getTabbableElements().first();h.default.trigger(e,"focus")},_setCollectionWidgetOption:r.noop,_optionChanged:function(e){switch(this._dataExpressionOptionChanged(e),e.name){case"dataSource":this._renderInputValue();break;case"displayValue":this.option("text",e.value);break;case"displayExpr":this._renderValue();break;case"contentTemplate":this._invalidate();break;default:this.callBase(e)}}}).include(a.default);(0,g.default)("dxDropDownBox",S);var D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.default=void 0;var c=i(n(2)),d=i(n(5)),h=i(n(9)),f=i(n(35)),p=n(4),g=n(1),_=n(3),m=n(12),v=n(0),y=n(42),b=i(n(23)),x=n(55),w=n(18),C=i(n(10)),k=i(n(16)),S=i(n(52)),D=n(7),I=i(n(100)),T=i(n(437)),O=i(n(297)),E=i(n(109)),A=n(6),P=n(36),R=i(n(641));n(643);var M=n(441);n(298),n(444);var B=n(443),V="dx-widget",F="dx-state-focused",L=["items","isRequired","validationRules","visible"],H=k.default.inherit({_init:function(){this.callBase(),this._cachedColCountOptions=[],this._itemsRunTimeInfo=new T.default,this._groupsColCount=[],this._attachSyncSubscriptions()},_initOptions:function(e){"screenByWidth"in e||(e.screenByWidth=D.defaultScreenFactorFunc),this.callBase(e)},_getDefaultOptions:function(){return(0,v.extend)(this.callBase(),{formID:"dx-"+new f.default,formData:{},colCount:1,screenByWidth:null,colCountByScreen:void 0,labelLocation:"left",readOnly:!1,onFieldDataChanged:null,customizeItem:null,onEditorEnterKey:null,minColWidth:200,alignItemLabels:!0,alignItemLabelsInAllGroups:!0,alignRootItemLabels:!0,showColonAfterLabel:!0,showRequiredMark:!0,showOptionalMark:!1,requiredMark:"*",optionalMark:C.default.format("dxForm-optionalMark"),requiredMessage:C.default.getFormatter("dxForm-requiredMessage"),showValidationSummary:!1,items:void 0,scrollingEnabled:!1,validationGroup:void 0,stylingMode:void 0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return(0,P.isMaterial)()},options:{showColonAfterLabel:!1,labelLocation:"top"}}])},_setOptionsByReference:function(){this.callBase(),(0,v.extend)(this._optionsByReference,{formData:!0,validationGroup:!0})},_getGroupColCount:function(e){return parseInt(e.attr(B.GROUP_COL_COUNT_ATTR))},_createHiddenElement:function(e){this._$hiddenElement=(0,c.default)("<div>").addClass(V).addClass(B.HIDDEN_LABEL_CLASS).appendTo("body");var t=e._renderLabel({text:" ",location:this._labelLocation()}).appendTo(this._$hiddenElement);this._hiddenLabelText=t.find("."+B.FIELD_ITEM_LABEL_TEXT_CLASS)[0]},_removeHiddenElement:function(){this._$hiddenElement.remove(),this._hiddenLabelText=null},_getLabelWidthByText:function(e){return this._hiddenLabelText.innerHTML=e,this._hiddenLabelText.offsetWidth},_getLabelsSelectorByCol:function(e,t){t=t||{};var n=t.inOneColumn?B.FIELD_ITEM_CLASS:B.FORM_FIELD_ITEM_COL_CLASS+e,i=t.excludeTabbed?":not(."+B.FIELD_ITEM_TAB_CLASS+")":"",o="> ."+B.FIELD_ITEM_LABEL_CLASS+" > ."+B.FIELD_ITEM_LABEL_CONTENT_CLASS;return"."+n+i+o},_getLabelText:function(e){var t,n,i=e.children.length,o="";for(n=0;n<i;n++)t=e.children[n],o+=(0,y.isEmpty)(t.innerText)?t.innerHTML:t.innerText;return o},_applyLabelsWidthByCol:function(e,t,n){var i,o,a=e.find(this._getLabelsSelectorByCol(t,n)),r=a.length,s=0;for(o=0;o<r;o++)i=this._getLabelWidthByText(this._getLabelText(a[o])),i>s&&(s=i);for(o=0;o<r;o++)a[o].style.width=s+"px"},_applyLabelsWidth:function(e,t,n,i){i=n?1:i||this._getGroupColCount(e);var o,a={excludeTabbed:t,inOneColumn:n};for(o=0;o<i;o++)this._applyLabelsWidthByCol(e,o,a)},_getGroupElementsInColumn:function(e,t,n){var i=(0,g.isDefined)(n)?"."+B.GROUP_COL_COUNT_CLASS+n:"",o="."+B.FORM_FIELD_ITEM_COL_CLASS+t+" > ."+B.FIELD_ITEM_CONTENT_CLASS+" > ."+B.FORM_GROUP_CLASS+i;return e.find(o)},_applyLabelsWidthWithGroups:function(e,t,n){this.option("alignRootItemLabels")===!0&&this._alignRootSimpleItems(e,t,n);var i=this.option("alignItemLabelsInAllGroups");if(i)this._applyLabelsWidthWithNestedGroups(e,t,n);else{var o,a=this.$element().find("."+B.FORM_GROUP_CLASS);for(o=0;o<a.length;o++)this._applyLabelsWidth(a.eq(o),n)}},_alignRootSimpleItems:function(e,t,n){for(var i=e.find(".".concat(B.ROOT_SIMPLE_ITEM_CLASS)),o=0;o<t;o++)this._applyLabelsWidthByCol(i,o,n)},_applyLabelsWidthWithNestedGroups:function(e,t,n){var i,o,a,r,s={excludeTabbed:n};for(i=0;i<t;i++)for(r=this._getGroupElementsInColumn(e,i),this._applyLabelsWidthByCol(r,0,s),o=0;o<this._groupsColCount.length;o++){r=this._getGroupElementsInColumn(e,i,this._groupsColCount[o]);var l=this._getGroupColCount(r);for(a=1;a<l;a++)this._applyLabelsWidthByCol(r,a,s)}},_labelLocation:function(){return this.option("labelLocation")},_alignLabelsInColumn:function(e){var t=e.layoutManager,n=e.inOneColumn,i=e.$container,o=e.excludeTabbed,a=e.items;(0,D.hasWindow)()&&"top"!==this._labelLocation()&&(this._createHiddenElement(t),n?this._applyLabelsWidth(i,o,!0):this._checkGrouping(a)?this._applyLabelsWidthWithGroups(i,t._getColCount(),o):this._applyLabelsWidth(i,o,!1,t._getColCount()),this._removeHiddenElement())},_prepareFormData:function(){(0,g.isDefined)(this.option("formData"))||this.option("formData",{})},_initMarkup:function(){I.default.addGroup(this._getValidationGroup()),this._clearCachedInstances(),this._prepareFormData(),this.$element().addClass(B.FORM_CLASS),this.callBase(),this.setAria("role","form",this.$element()),this.option("scrollingEnabled")&&this._renderScrollable(),this._renderLayout(),this._renderValidationSummary(),this._lastMarkupScreenFactor=this._targetScreenFactor||this._getCurrentScreenFactor()},_getCurrentScreenFactor:function(){return(0,D.hasWindow)()?(0,D.getCurrentScreenFactor)(this.option("screenByWidth")):"lg"},_clearCachedInstances:function(){this._itemsRunTimeInfo.clear(),this._cachedLayoutManagers=[]},_alignLabels:function(e,t){this._alignLabelsInColumn({$container:this.$element(),layoutManager:e,excludeTabbed:!0,items:this.option("items"),inOneColumn:t})},_clean:function(){this.callBase(),this._groupsColCount=[],this._cachedColCountOptions=[],this._lastMarkupScreenFactor=void 0},_renderScrollable:function(){var e=this.option("useNativeScrolling");this._scrollable=new E.default(this.$element(),{useNative:!!e,useSimulatedScrollbar:!e,useKeyboard:!1,direction:"both",bounceEnabled:!1})},_getContent:function(){return this.option("scrollingEnabled")?this._scrollable.$content():this.$element()},_renderValidationSummary:function(){var e=this.$element().find("."+B.FORM_VALIDATION_SUMMARY);if(e.length>0&&e.remove(),this.option("showValidationSummary")){var t=(0,c.default)("<div>").addClass(B.FORM_VALIDATION_SUMMARY).appendTo(this._getContent());this._validationSummary=t.dxValidationSummary({validationGroup:this._getValidationGroup()}).dxValidationSummary("instance")}},_prepareItems:function(e,t,n,i){if(e){for(var o=[],a=0;a<e.length;a++){var r=e[a],s=(0,M.concatPaths)(n,(0,M.createItemPathByIndex)(a,i)),l=this._itemsRunTimeInfo.add({item:r,itemIndex:a,path:s});if((0,g.isString)(r)&&(r={dataField:r}),(0,g.isObject)(r)){var u=(0,v.extend)({},r);u.guid=l,this._tryPrepareGroupItem(u),this._tryPrepareTabbedItem(u,s),this._tryPrepareItemTemplate(u),t&&(u.cssItemClass=B.FIELD_ITEM_TAB_CLASS),u.items&&(u.items=this._prepareItems(u.items,t,s)),o.push(u)}else o.push(r)}return o}},_tryPrepareGroupItem:function(e){"group"===e.itemType&&(e.alignItemLabels=(0,p.ensureDefined)(e.alignItemLabels,!0),e.template&&(e.groupContentTemplate=this._getTemplate(e.template)),e.template=this._itemGroupTemplate.bind(this,e))},_tryPrepareTabbedItem:function(e,t){"tabbed"===e.itemType&&(e.template=this._itemTabbedTemplate.bind(this,e),e.tabs=this._prepareItems(e.tabs,!0,t,!0))},_tryPrepareItemTemplate:function(e){e.template&&(e.template=this._getTemplate(e.template))},_checkGrouping:function(e){if(e)for(var t=0;t<e.length;t++){var n=e[t];if("group"===n.itemType)return!0}},_renderLayout:function(){var e=this,t=e.option("items"),n=e._getContent();t=e._prepareItems(t),e._rootLayoutManager=e._renderLayoutManager(t,n,{isRoot:!0,colCount:e.option("colCount"),alignItemLabels:e.option("alignItemLabels"),screenByWidth:this.option("screenByWidth"),colCountByScreen:this.option("colCountByScreen"),onLayoutChanged:function(t){e._alignLabels.bind(e)(e._rootLayoutManager,t)},onContentReady:function(t){e._alignLabels(t.component,t.component.isSingleColumnMode())}})},_tryGetItemsForTemplate:function(e){return e.items||[]},_itemTabbedTemplate:function(e,t,n){var i=this,a=(0,c.default)("<div>").appendTo(n),r=(0,v.extend)({},e.tabPanelOptions,{dataSource:e.tabs,onItemRendered:function(e){return(0,x.triggerShownEvent)(e.itemElement)},itemTemplate:function(e,t,n){var o=(0,c.default)(n),a=(0,p.ensureDefined)(e.alignItemLabels,!0),r=i._renderLayoutManager(i._tryGetItemsForTemplate(e),o,{colCount:e.colCount,alignItemLabels:a,screenByWidth:i.option("screenByWidth"),colCountByScreen:e.colCountByScreen,cssItemClass:e.cssItemClass,onLayoutChanged:function(t){i._alignLabelsInColumn({$container:o,layoutManager:r,items:e.items,inOneColumn:t})}});i._itemsRunTimeInfo&&i._itemsRunTimeInfo.extendRunTimeItemInfoByKey(e.guid,{layoutManager:r}),a&&i._alignLabelsInColumn({$container:o,layoutManager:r,items:e.items,inOneColumn:r.isSingleColumnMode()})}}),s=function(e,t){Array.isArray(e)&&e.forEach(function(e){return i._itemsRunTimeInfo.extendRunTimeItemInfoByKey(e.guid,{widgetInstance:t})})},l=this._createComponent(a,O.default,r);(0,c.default)(n).parent().addClass(B.FIELD_ITEM_CONTENT_HAS_TABS_CLASS),l.on("optionChanged",function(e){"dataSource"===e.fullName&&s(e.value,e.component)}),s([{guid:e.guid}].concat(o(e.tabs)),l)},_itemGroupTemplate:function(e,t,n){var i=(0,c.default)("<div>").toggleClass(B.FORM_GROUP_WITH_CAPTION_CLASS,(0,g.isDefined)(e.caption)&&e.caption.length).addClass(B.FORM_GROUP_CLASS).appendTo(n);(0,c.default)(n).parent().addClass(B.FIELD_ITEM_CONTENT_HAS_GROUP_CLASS);var o,a;e.caption&&(0,c.default)("<span>").addClass(B.FORM_GROUP_CAPTION_CLASS).text(e.caption).appendTo(i);var r=(0,c.default)("<div>").addClass(B.FORM_GROUP_CONTENT_CLASS).appendTo(i);if(e.groupContentTemplate){var s={formData:this.option("formData"),component:this};e.groupContentTemplate.render({model:s,container:(0,w.getPublicElement)(r)})}else a=this._renderLayoutManager(this._tryGetItemsForTemplate(e),r,{colCount:e.colCount,colCountByScreen:e.colCountByScreen,alignItemLabels:e.alignItemLabels,cssItemClass:e.cssItemClass}),this._itemsRunTimeInfo&&this._itemsRunTimeInfo.extendRunTimeItemInfoByKey(e.guid,{layoutManager:a}),o=a._getColCount(),(0,m.inArray)(o,this._groupsColCount)===-1&&this._groupsColCount.push(o),i.addClass(B.GROUP_COL_COUNT_CLASS+o),i.attr(B.GROUP_COL_COUNT_ATTR,o)},_renderLayoutManager:function(e,t,n){var i=(0,c.default)("<div>"),o=this,a=o._getLayoutManagerConfig(e,n),r={lg:n.colCount,md:n.colCount,sm:n.colCount,xs:1};o._cachedColCountOptions.push({colCountByScreen:(0,v.extend)(r,n.colCountByScreen)}),i.appendTo(t);var s=o._createComponent(i,"dxLayoutManager",a);return s.on("autoColCountChanged",function(){o._refresh()}),o._cachedLayoutManagers.push(s),s},_getValidationGroup:function(){return this.option("validationGroup")||this},_getLayoutManagerConfig:function(e,t){var n=this,i={form:this,isRoot:t.isRoot,validationGroup:this._getValidationGroup(),showRequiredMark:this.option("showRequiredMark"),showOptionalMark:this.option("showOptionalMark"),requiredMark:this.option("requiredMark"),optionalMark:this.option("optionalMark"),requiredMessage:this.option("requiredMessage"),screenByWidth:this.option("screenByWidth"),layoutData:this.option("formData"),labelLocation:this.option("labelLocation"),customizeItem:this.option("customizeItem"),minColWidth:this.option("minColWidth"),showColonAfterLabel:this.option("showColonAfterLabel"),onEditorEnterKey:this.option("onEditorEnterKey"),onFieldDataChanged:function(e){n._isDataUpdating||n._triggerOnFieldDataChanged(e)},validationBoundary:this.option("scrollingEnabled")?this.$element():void 0};return(0,v.extend)(i,{items:e,onContentReady:function(e){n._itemsRunTimeInfo.addItemsOrExtendFrom(e.component._itemsRunTimeInfo),t.onContentReady&&t.onContentReady(e)},onDisposing:function(e){var t=e.component,i=t.getItemsRunTimeInfo();n._itemsRunTimeInfo.removeItemsByItems(i)},colCount:t.colCount,alignItemLabels:t.alignItemLabels,cssItemClass:t.cssItemClass,colCountByScreen:t.colCountByScreen,onLayoutChanged:t.onLayoutChanged,width:t.width})},_createComponent:function(e,t,n){var i=this;return n=n||{},i._extendConfig(n,{readOnly:i.option("readOnly")}),i.callBase(e,t,n)},_attachSyncSubscriptions:function(){var e=this;e.on("optionChanged",function(t){var n=t.fullName;"formData"===n&&((0,g.isDefined)(t.value)||e._options.silent("formData",t.value={}),e._triggerOnFieldDataChangedByDataSet(t.value)),e._cachedLayoutManagers.length&&(0,_.each)(e._cachedLayoutManagers,function(i,o){"formData"===n&&(e._isDataUpdating=!0,o.option("layoutData",t.value),e._isDataUpdating=!1),"readOnly"!==t.name&&"disabled"!==t.name||o.option(n,t.value)})})},_optionChanged:function(e){var t=this._getRootLevelOfExpectedComplexOption(e.fullName,["formData","items"]);if(t)return void this._customHandlerOfComplexOption(e,t);switch(e.name){case"formData":this.option("items")?(0,g.isEmptyObject)(e.value)&&this._resetValues():this._invalidate();break;case"items":case"colCount":case"onFieldDataChanged":case"onEditorEnterKey":case"labelLocation":case"alignItemLabels":case"showColonAfterLabel":case"customizeItem":case"alignItemLabelsInAllGroups":case"showRequiredMark":case"showOptionalMark":case"requiredMark":case"optionalMark":case"requiredMessage":case"scrollingEnabled":case"formID":case"colCountByScreen":case"screenByWidth":case"stylingMode":this._invalidate();break;case"showValidationSummary":this._renderValidationSummary();break;case"minColWidth":"auto"===this.option("colCount")&&this._invalidate();break;case"alignRootItemLabels":case"readOnly":break;case"width":this.callBase(e),this._rootLayoutManager.option(e.name,e.value),this._alignLabels(this._rootLayoutManager,this._rootLayoutManager.isSingleColumnMode());break;case"visible":this.callBase(e),e.value&&(0,x.triggerShownEvent)(this.$element());break;case"validationGroup":I.default.removeGroup(e.previousValue||this),this._invalidate();break;default:this.callBase(e)}},_getRootLevelOfExpectedComplexOption:function(e,t){var n,i=e.split(".");if(i.length>1){var o,a=i[0];for(o=0;o<t.length;o++)a.search(t[o])!==-1&&(n=t[o])}return n},_tryCreateItemOptionAction:function(e,t,n,i,o){return"tabs"===e&&(this._itemsRunTimeInfo.removeItemsByPathStartWith("".concat(o,".tabs")),n=this._prepareItems(n,!0,o,!0)),(0,R.default)(e,{item:t,value:n,previousValue:i,itemsRunTimeInfo:this._itemsRunTimeInfo})},_tryExecuteItemOptionAction:function(e){return e&&e.tryExecute()},_updateValidationGroupAndSummaryIfNeeded:function(e){var t=(0,M.getOptionNameFromFullName)(e);L.indexOf(t)>-1&&(I.default.addGroup(this._getValidationGroup()),this.option("showValidationSummary")&&this._validationSummary&&this._validationSummary._initGroupRegistration())},_setLayoutManagerItemOption:function(e,t,n,i){var o=this;if(this._updateLockCount>0){!e._updateLockCount&&e.beginUpdate();var a=this._itemsRunTimeInfo.getKeyByPath(i);this.postponedOperations.add(a,function(){return!e._disposed&&e.endUpdate(),(new A.Deferred).resolve()})}var r=function e(t){if(t.component.off("contentReady",e),(0,M.isFullPathContainsTabs)(i)){var n=(0,M.tryGetTabPath)(i),a=o._itemsRunTimeInfo.getGroupOrTabLayoutManagerByPath(n);o._alignLabelsInColumn({items:a.option("items"),layoutManager:a,$container:a.$element(),inOneColumn:a.isSingleColumnMode()})}else o._alignLabels(o._rootLayoutManager,o._rootLayoutManager.isSingleColumnMode())};e.on("contentReady",r),e.option(t,n),this._updateValidationGroupAndSummaryIfNeeded(t)},_tryChangeLayoutManagerItemOption:function(e,t){var n=e.split("."),i=(0,M.getOptionNameFromFullName)(e);if("items"===i&&n.length>1){var o=this._getItemPath(n),a=this._itemsRunTimeInfo.getGroupOrTabLayoutManagerByPath(o);if(a){this._itemsRunTimeInfo.removeItemsByItems(a.getItemsRunTimeInfo());var r=this._prepareItems(t,!1,o);return this._setLayoutManagerItemOption(a,i,r,o),!0}}else if(n.length>2){var s=n.length-2,l=this._getItemPath(n.slice(0,s)),u=this._itemsRunTimeInfo.getGroupOrTabLayoutManagerByPath(l);if(u){var c=(0,M.getFullOptionName)(n[s],i);if("editorType"===i&&u.option(c)!==t)return!1;if("visible"===i){var d=this.option((0,M.getFullOptionName)(l,"items"));if(d&&d.length){var h=u.option("items");d.forEach(function(e,t){var n=h[t];n.visibleIndex=e.visibleIndex})}}return this._setLayoutManagerItemOption(u,c,t,l),!0}}return!1},_tryChangeLayoutManagerItemOptions:function(e,t){var n,i=this;return this.beginUpdate(),(0,_.each)(t,function(t,o){if(n=i._tryChangeLayoutManagerItemOption((0,M.getFullOptionName)(e,t),o),!n)return!1}),this.endUpdate(),n},_customHandlerOfComplexOption:function(e,t){var n=e.fullName.split("."),i=e.value;if("items"===t){var o=this._getItemPath(n),a=this.option(o),r=e.fullName.replace(o+".",""),s=r.split(".")[0].replace(/\[\d+]/,""),l=this._tryCreateItemOptionAction(s,a,a[s],e.previousValue,o);if(!this._tryExecuteItemOptionAction(l)&&!this._tryChangeLayoutManagerItemOption(e.fullName,i)&&a){this._changeItemOption(a,r,i);var u=this._generateItemsFromData(this.option("items"));this.option("items",u)}}if("formData"===t){var c=n.slice(1).join("."),d=this.getEditor(c);d?d.option("value",i):this._triggerOnFieldDataChanged({dataField:c,value:i})}},_getItemPath:function(e){var t,n=e[0];for(t=1;t<e.length&&e[t].search(/items\[\d+]|tabs\[\d+]/)!==-1;t++)n+="."+e[t];return n},_triggerOnFieldDataChanged:function(e){this._createActionByOption("onFieldDataChanged")(e)},_triggerOnFieldDataChangedByDataSet:function(e){var t=this;e&&(0,g.isObject)(e)&&(0,_.each)(e,function(e,n){t._triggerOnFieldDataChanged({dataField:e,value:n})})},_updateFieldValue:function(e,t){if((0,g.isDefined)(this.option("formData"))){var n=this.getEditor(e);if(this.option("formData."+e,t),n){var i=n.option("value");i!==t&&n.option("value",t)}}},_generateItemsFromData:function(e){var t=this.option("formData"),n=[];return!e&&(0,g.isDefined)(t)&&(0,_.each)(t,function(e){n.push({dataField:e})}),e&&(0,_.each)(e,function(e,t){(0,g.isObject)(t)?n.push(t):n.push({dataField:t})}),n},_getItemByField:function(e,t){var n,i=this,o=(0,g.isObject)(e)?e:i._getFieldParts(e),a=o.fieldName,r=o.fieldPath;return t.length&&(0,_.each)(t,function(e,t){var o=t.itemType;if(r.length){var s=r.slice();t=i._getItemByFieldPath(s,a,t)}else if("group"===o&&!t.caption&&!t.name||"tabbed"===o&&!t.name){var l=i._getSubItemField(o);t.items=i._generateItemsFromData(t.items),t=i._getItemByField({fieldName:a,fieldPath:r},t[l])}if((0,M.isExpectedItem)(t,a))return n=t,!1}),n},_getFieldParts:function(e){for(var t=".",n=e,i=n.indexOf(t),o=[];i!==-1;)o.push(n.substr(0,i)),n=n.substr(i+1),i=n.indexOf(t);return{fieldName:n,fieldPath:o.reverse()}},_getItemByFieldPath:function(e,t,n){var i,o=this,a=n.itemType,r=o._getSubItemField(a),s="group"===a||"tabbed"===a||n.title;do{if(!s)break;var l=n.name||n.caption||n.title,u=(0,g.isDefined)(l),c=(0,M.getTextWithoutSpaces)(l),d=void 0;if(n[r]=o._generateItemsFromData(n[r]),u&&(d=e.pop()),!e.length&&(i=o._getItemByField(t,n[r])))break;(!u||u&&c===d)&&e.length&&(i=o._searchItemInEverySubItem(e,t,n[r]))}while(e.length&&!(0,g.isDefined)(i));return i},_getSubItemField:function(e){return"tabbed"===e?"tabs":"items"},_searchItemInEverySubItem:function(e,t,n){var i,o=this;return(0,_.each)(n,function(n,a){if(i=o._getItemByFieldPath(e.slice(),t,a))return!1}),i||(i=!1),i},_changeItemOption:function(e,t,n){(0,g.isObject)(e)&&(e[t]=n)},_dimensionChanged:function(){var e=this._getCurrentScreenFactor();this._lastMarkupScreenFactor!==e&&(this._isColCountChanged(this._lastMarkupScreenFactor,e)&&(this._targetScreenFactor=e,this._refresh(),this._targetScreenFactor=void 0),this._lastMarkupScreenFactor=e)},_isColCountChanged:function(e,t){var n=!1;return(0,_.each)(this._cachedColCountOptions,function(i,o){if(o.colCountByScreen[e]!==o.colCountByScreen[t])return n=!0,!1}),n},_refresh:function(){var e="."+F+" input, ."+F+" textarea";d.default.trigger(this.$element().find(e),"change"),this.callBase()},_resetValues:function(){this._itemsRunTimeInfo.each(function(e,t){(0,g.isDefined)(t.widgetInstance)&&t.widgetInstance instanceof S.default&&(t.widgetInstance.reset(),t.widgetInstance.option("isValid",!0))}),I.default.resetGroup(this._getValidationGroup())},_updateData:function(e,t,n){var i=this,o=n?t:e;(0,g.isObject)(o)?(0,_.each)(o,function(t,o){i._updateData(n?e+"."+t:t,o,(0,g.isObject)(o))}):(0,g.isString)(e)&&i._updateFieldValue(e,t)},registerKeyHandler:function(e,t){this.callBase(e,t),this._itemsRunTimeInfo.each(function(n,i){(0,g.isDefined)(i.widgetInstance)&&i.widgetInstance.registerKeyHandler(e,t)})},_focusTarget:function(){return this.$element().find("."+B.FIELD_ITEM_CONTENT_CLASS+" [tabindex]").first()},_visibilityChanged:function(e){e&&b.default.msie&&this._refresh()},_dispose:function(){I.default.removeGroup(this._getValidationGroup()),this.callBase()},resetValues:function(){this._resetValues()},updateData:function(e,t){this._updateData(e,t)},getEditor:function(e){return this._itemsRunTimeInfo.findWidgetInstanceByDataField(e)||this._itemsRunTimeInfo.findWidgetInstanceByName(e)},getButton:function(e){return this._itemsRunTimeInfo.findWidgetInstanceByName(e)},updateDimensions:function(){var e=this,t=new A.Deferred;return e._scrollable?e._scrollable.update().done(function(){t.resolveWith(e)}):t.resolveWith(e),t.promise()},itemOption:function(e,t,n){var i=this,o=this._generateItemsFromData(this.option("items")),a=this._getItemByField(e,o),r=(0,M.getItemPath)(o,a);if(a)switch(arguments.length){case 1:return a;case 3:var s=this._tryCreateItemOptionAction(t,a,n,a[t],r);this._changeItemOption(a,t,n);var l=(0,M.getFullOptionName)(r,t);this._tryExecuteItemOptionAction(s)||this._tryChangeLayoutManagerItemOption(l,n)||this.option("items",o);break;default:if((0,g.isObject)(t)&&!this._tryChangeLayoutManagerItemOptions(r,t)){var u;(0,_.each)(t,function(e,t){var n=i._tryCreateItemOptionAction(e,a,t,a[e],r);i._changeItemOption(a,e,t),u||i._tryExecuteItemOptionAction(n)||(u=!0)}),u&&this.option("items",o);
}}},validate:function(){return I.default.validateGroup(this._getValidationGroup())},getItemID:function(e){return"dx_"+this.option("formID")+"_"+(e||new f.default)},getTargetScreenFactor:function(){return this._targetScreenFactor}});(0,h.default)("dxForm",H);var z=H;t.default=z,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.animation=t._translator=void 0;var o=i(n(43)),a=n(30),r={move:function(e,t){(0,a.move)(e,{left:t})}};t._translator=r;var s={moveTo:function(e,t,n,i){o.default.animate(e,{type:"slide",to:{left:t},duration:n,complete:i})},complete:function(e){o.default.stop(e,!0)}};t.animation=s},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(141)),v=n(4),y=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_renderWatchers",value:function(){return this._startWatcher("badge",v.noop),l(_(t.prototype),"_renderWatchers",this).call(this)}}]),t}(m.default);t.default=y,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(642)),_=n(39),m=n(0),v=n(441),y=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"tryExecute",value:function(){var e=this._options.value,t=this.findInstance();return!!t&&(t.option(e),!0)}}]),t}(g.default),b=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"tryExecute",value:function(){var e=this.findInstance();if(e){var t=this._options,n=t.optionName,i=t.item,o=t.value,a=this._itemsRunTimeInfo.findItemIndexByItem(i);if(a>=0)return e.option((0,v.getFullOptionName)("items[".concat(a,"]"),n),o),!0}return!1}}]),t}(g.default),x=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"tryExecute",value:function(){var e=this.findInstance();if(e){var t=this._options.value;return e.option("dataSource",t),!0}return!1}}]),t}(g.default),w=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"tryExecute",value:function(){var e=this._options.item,t=this.findInstance(),n=t&&(0,_.data)(t.$element()[0],"dxValidator");if(n&&e){var i=function(e){return"required"===e.type},o=(n.option("validationRules")||[]).some(i),a=(e.validationRules||[]).some(i);if(!o&&!a||o&&a)return n.option("validationRules",e.validationRules),!0}return!1}}]),t}(g.default),C=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"tryExecute",value:function(){var e=this.findItemContainer(),t=this._options,n=t.previousValue,i=t.value;return!!e&&(e.removeClass(n).addClass(i),!0)}}]),t}(g.default),k=function(e,t){switch(e){case"editorOptions":case"buttonOptions":return new y(t);case"validationRules":return new w(t);case"cssClass":return new C(t);case"badge":case"disabled":case"icon":case"template":case"tabTemplate":case"title":return new b((0,m.extend)(t,{optionName:e}));case"tabs":return new x(t);default:return null}},S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(15)),l=function(){function e(t){o(this,e),this._options=t,this._itemsRunTimeInfo=this._options.itemsRunTimeInfo}return r(e,[{key:"findInstance",value:function(){return this._itemsRunTimeInfo.findWidgetInstanceByItem(this._options.item)}},{key:"findItemContainer",value:function(){return this._itemsRunTimeInfo.findItemContainerByItem(this._options.item)}},{key:"tryExecute",value:function(){s.default.abstract()}}]),e}();t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}t.default=void 0;var d=i(n(2)),h=i(n(5)),f=i(n(35)),p=i(n(437)),g=i(n(9)),_=n(1),m=n(18),v=i(n(101)),y=n(7),b=n(42),x=n(3),w=n(0),C=n(12),k=n(19),S=i(n(138)),D=n(21),I=i(n(17)),T=i(n(10)),O=n(97),E=n(40),A=i(n(16)),P=i(n(221)),R=i(n(442)),M=n(36),B=n(443);n(104),n(126),n(144),n(217),n(38);var V="dxTextBox",F="dx-first-row",L="dx-last-row",H="dx-first-col",z="dx-last-col",N="dx-invalid",j="flex",W="fallback",G="simple",q="dx-template-wrapper",$=["dataSource","items"],K=["dxTagBox","dxRangeSlider"],U=A.default.inherit({_getDefaultOptions:function(){return(0,w.extend)(this.callBase(),{layoutData:{},readOnly:!1,colCount:1,colCountByScreen:void 0,labelLocation:"left",onFieldDataChanged:null,onEditorEnterKey:null,customizeItem:null,alignItemLabels:!0,minColWidth:200,showRequiredMark:!0,screenByWidth:null,showOptionalMark:!1,requiredMark:"*",optionalMark:T.default.format("dxForm-optionalMark"),requiredMessage:T.default.getFormatter("dxForm-requiredMessage")})},_setOptionsByReference:function(){this.callBase(),(0,w.extend)(this._optionsByReference,{layoutData:!0,validationGroup:!0})},_init:function(){var e=this.option("layoutData");this.callBase(),this._itemWatchers=[],this._itemsRunTimeInfo=new p.default,this._updateReferencedOptions(e),this._initDataAndItems(e)},_dispose:function(){this.callBase(),this._cleanItemWatchers()},_initDataAndItems:function(e){this._syncDataWithItems(),this._updateItems(e)},_syncDataWithItems:function(){var e=this,t=this.option("layoutData"),n=this.option("items");(0,_.isDefined)(n)&&n.forEach(function(n){if(n.dataField&&void 0===e._getDataByField(n.dataField)){var i;n.editorOptions&&(i=n.editorOptions.value),((0,_.isDefined)(i)||n.dataField in t)&&e._updateFieldValue(n.dataField,i)}})},_getDataByField:function(e){return e?this.option("layoutData."+e):null},_isCheckboxUndefinedStateEnabled:function(e){if(e.allowIndeterminateState===!0&&"dxCheckBox"===e.editorType){var t=["layoutData"].concat(a(e.dataField.split("."))),n=t.pop(),i=this.option(t.join("."));return i&&n in i}return!1},_updateFieldValue:function(e,t){var n=this.option("layoutData"),i=t;!v.default.isWrapped(n[e])&&(0,_.isDefined)(e)?this.option("layoutData."+e,i):v.default.isWritableWrapped(n[e])&&(i=(0,_.isFunction)(i)?i():i,n[e](i)),this._triggerOnFieldDataChanged({dataField:e,value:i})},_triggerOnFieldDataChanged:function(e){this._createActionByOption("onFieldDataChanged")(e)},_updateItems:function(e){var t=this,n=this.option("items"),i=(0,_.isDefined)(n),o=t.option("customizeItem"),a=i?n:this._generateItemsByData(e);if((0,_.isDefined)(a)){var r=[];(0,x.each)(a,function(e,n){t._isAcceptableItem(n)&&(n=t._processItem(n),o&&o(n),(0,_.isObject)(n)&&v.default.unwrap(n.visible)!==!1&&r.push(n))}),t._itemWatchers.length&&i||t._updateItemWatchers(a),this._setItems(r),this._sortItems()}},_cleanItemWatchers:function(){this._itemWatchers.forEach(function(e){e()}),this._itemWatchers=[]},_updateItemWatchers:function(e){var t=this,n=t._getWatch();e.forEach(function(e){(0,_.isObject)(e)&&(0,_.isDefined)(e.visible)&&(0,_.isFunction)(n)&&t._itemWatchers.push(n(function(){return v.default.unwrap(e.visible)},function(){t._updateItems(t.option("layoutData")),t.repaint()},{skipImmediate:!0}))})},_generateItemsByData:function(e){var t=[];return(0,_.isDefined)(e)&&(0,x.each)(e,function(e){t.push({dataField:e})}),t},_isAcceptableItem:function(e){var t=e.dataField||e,n=this._getDataByField(t);return!((0,_.isFunction)(n)&&!v.default.isWrapped(n))},_processItem:function(e){if("string"==typeof e&&(e={dataField:e}),"object"!==o(e)||e.itemType||(e.itemType=G),!(0,_.isDefined)(e.editorType)&&(0,_.isDefined)(e.dataField)){var t=this._getDataByField(e.dataField);e.editorType=(0,_.isDefined)(t)?this._getEditorTypeByDataType((0,_.type)(t)):V}if("dxCheckBox"===e.editorType){var n;e.allowIndeterminateState=null===(n=e.allowIndeterminateState)||void 0===n||n}return e},_getEditorTypeByDataType:function(e){switch(e){case"number":return"dxNumberBox";case"date":return"dxDateBox";case"boolean":return"dxCheckBox";default:return"dxTextBox"}},_sortItems:function(){(0,C.normalizeIndexes)(this._items,"visibleIndex"),this._sortIndexes()},_sortIndexes:function(){this._items.sort(function(e,t){var n,i=e.visibleIndex,o=t.visibleIndex;return n=i>o?1:i<o?-1:0})},_initMarkup:function(){this._itemsRunTimeInfo.clear(),this.$element().addClass(B.FORM_LAYOUT_MANAGER_CLASS),this.callBase(),this._renderResponsiveBox()},_hasBrowserFlex:function(){return(0,O.styleProp)(j)===j},_renderResponsiveBox:function(){var e=this,t=[];if(e._items&&e._items.length){var n=e._getColCount(),i=(0,d.default)("<div>").appendTo(e.$element());e._prepareItemsWithMerging(n);var o=e._generateLayoutItems();e._extendItemsWithDefaultTemplateOptions(o,e._items),e._responsiveBox=e._createComponent(i,R.default,e._getResponsiveBoxConfig(o,n,t)),(0,y.hasWindow)()||e._renderTemplates(t)}},_extendItemsWithDefaultTemplateOptions:function(e,t){t.forEach(function(t){t.merged||((0,_.isDefined)(t.disabled)&&(e[t.visibleIndex].disabled=t.disabled),(0,_.isDefined)(t.visible)&&(e[t.visibleIndex].visible=t.visible))})},_itemStateChangedHandler:function(e){this._refresh()},_renderTemplate:function(e,t){switch(t.itemType){case"empty":this._renderEmptyItem(e);break;case"button":this._renderButtonItem(t,e);break;default:this._renderFieldItem(t,e)}},_renderTemplates:function(e){var t=this;(0,x.each)(e,function(e,n){t._renderTemplate(n.container,n.formItem)})},_getResponsiveBoxConfig:function(e,t,n){var i=this,o=i.option("colCountByScreen"),a=o&&o.xs;return{onItemStateChanged:this._itemStateChangedHandler.bind(this),_layoutStrategy:i._hasBrowserFlex()?j:W,onLayoutChanged:function e(){var e=i.option("onLayoutChanged"),t=i.isSingleColumnMode();e&&(i.$element().toggleClass(B.LAYOUT_MANAGER_ONE_COLUMN,t),e(t))},onContentReady:function(e){(0,y.hasWindow)()&&i._renderTemplates(n),i.option("onLayoutChanged")&&i.$element().toggleClass(B.LAYOUT_MANAGER_ONE_COLUMN,i.isSingleColumnMode(e.component))},itemTemplate:function(e,o,a){if(e.location){var r=(0,d.default)(a),s=e.location.row*t,l=i._items[e.location.col+s],u=(0,d.default)("<div>").addClass(l.cssClass).appendTo(r);n.push({container:u,formItem:l}),r.toggleClass(B.SINGLE_COLUMN_ITEM_CONTENT,i.isSingleColumnMode(this)),0===e.location.row&&u.addClass(F),0===e.location.col&&u.addClass(H),l.itemType===G&&i.option("isRoot")&&r.addClass(B.ROOT_SIMPLE_ITEM_CLASS);var c=e.location.col===t-1||e.location.col+e.location.colspan===t,h=i._getRowsCount(),f=e.location.row===h-1;c&&u.addClass(z),f&&u.addClass(L)}},cols:i._generateRatio(t),rows:i._generateRatio(i._getRowsCount(),!0),dataSource:e,screenByWidth:i.option("screenByWidth"),singleColumnScreen:!a&&"xs"}},_getColCount:function(){var e=this.option("colCount"),t=this.option("colCountByScreen");if(t){var n=this.option("form").getTargetScreenFactor();n||(n=(0,y.hasWindow)()?(0,y.getCurrentScreenFactor)(this.option("screenByWidth")):"lg"),e=t[n]||e}if("auto"===e){if(this._cashedColCount)return this._cashedColCount;this._cashedColCount=e=this._getMaxColCount()}return e<1?1:e},_getMaxColCount:function(){if(!(0,y.hasWindow)())return 1;var e=this.option("minColWidth"),t=this.$element().width(),n=this._items.length,i=Math.floor(t/e)||1;return n<i?n:i},isCachedColCountObsolete:function(){return this._cashedColCount&&this._getMaxColCount()!==this._cashedColCount},_prepareItemsWithMerging:function(e){var t,n,i,o,a=this._items.slice(0),r=[];for(o=0;o<a.length;o++)if(t=a[o],r.push(t),(this.option("alignItemLabels")||t.alignItemLabels||t.colSpan)&&(t.col=this._getColByIndex(r.length-1,e)),t.colSpan>1&&t.col+t.colSpan<=e){for(n=[],i=0;i<t.colSpan-1;i++)n.push({merged:!0});r=r.concat(n)}else delete t.colSpan;this._setItems(r)},_getColByIndex:function(e,t){return e%t},_setItems:function(e){this._items=e,this._cashedColCount=null},_generateLayoutItems:function(){var e,t,n=this._items,i=this._getColCount(),o=[];for(t=0;t<n.length;t++)if(e=n[t],!e.merged){var a={location:{row:parseInt(t/i),col:this._getColByIndex(t,i)}};(0,_.isDefined)(e.colSpan)&&(a.location.colspan=e.colSpan),(0,_.isDefined)(e.rowSpan)&&(a.location.rowspan=e.rowSpan),o.push(a)}return o},_renderEmptyItem:function(e){return e.addClass(B.FIELD_EMPTY_ITEM_CLASS).html("&nbsp;")},_getButtonHorizontalAlignment:function(e){return(0,_.isDefined)(e.horizontalAlignment)?e.horizontalAlignment:"right"},_getButtonVerticalAlignment:function(e){switch(e.verticalAlignment){case"center":return"center";case"bottom":return"flex-end";default:return"flex-start"}},_renderButtonItem:function(e,t){var n=(0,d.default)("<div>").appendTo(t),i={validationGroup:this.option("validationGroup")};t.addClass(B.FIELD_BUTTON_ITEM_CLASS).css("textAlign",this._getButtonHorizontalAlignment(e)),t.parent().css("justifyContent",this._getButtonVerticalAlignment(e));var o=this._createComponent(n,"dxButton",(0,w.extend)(i,e.buttonOptions));return this._itemsRunTimeInfo.add({item:e,widgetInstance:o,guid:e.guid,$itemContainer:t}),this._addItemClasses(t,e.col),n},_addItemClasses:function(e,t){e.addClass(B.FIELD_ITEM_CLASS).addClass(this.option("cssItemClass")).addClass((0,_.isDefined)(t)?"dx-col-"+t:"")},_renderFieldItem:function(e,t){var n,i=this,o=i._getName(e),a=i.getItemID(o),r=(0,_.isDefined)(e.isRequired)?e.isRequired:!!i._hasRequiredRuleInSet(e.validationRules),s=i._getLabelOptions(e,a,r),l=(0,d.default)("<div>"),u=e.helpText?"dx-"+new f.default:null;this._addItemClasses(t,e.col),t.addClass(r?B.FIELD_ITEM_REQUIRED_CLASS:B.FIELD_ITEM_OPTIONAL_CLASS),s.visible&&s.text&&(n=i._renderLabel(s).appendTo(t)),e.itemType===G&&(i._isLabelNeedBaselineAlign(e)&&"top"!==s.location&&t.addClass(B.FIELD_ITEM_LABEL_ALIGN_CLASS),i._hasBrowserFlex()&&t.addClass(B.FLEX_LAYOUT_CLASS)),l.data("dx-form-item",e),i._appendEditorToField({$fieldItem:t,$label:n,$editor:l,labelOptions:s});var c=i._renderEditor({$container:l,dataField:e.dataField,name:e.name,editorType:e.editorType,editorOptions:e.editorOptions,template:i._getTemplateByFieldItem(e),isRequired:r,helpID:u,labelID:s.labelID,id:a,validationBoundary:i.option("validationBoundary"),allowIndeterminateState:e.allowIndeterminateState});this._itemsRunTimeInfo.add({item:e,widgetInstance:c,guid:e.guid,$itemContainer:t});var h=l.children().first(),p=h.hasClass(q)?h.children().first():h,g=p&&p.data("dx-validation-target");g&&(i._renderValidator(p,e),(0,M.isMaterial)()&&i._addWrapperInvalidClass(g)),i._renderHelpText(e,l,u),i._attachClickHandler(n,l,e.editorType)},_hasRequiredRuleInSet:function(e){var t;return e&&e.length&&(0,x.each)(e,function(e,n){if("required"===n.type)return t=!0,!1}),t},_getName:function(e){return e.dataField||e.name},_isLabelNeedBaselineAlign:function(e){var t=["dxTextArea","dxRadioGroup","dxCalendar","dxHtmlEditor"];return!!e.helpText&&!this._hasBrowserFlex()||(0,C.inArray)(e.editorType,t)!==-1},_isLabelNeedId:function(e){var t=["dxRadioGroup","dxCheckBox","dxLookup","dxSlider","dxRangeSlider","dxSwitch","dxHtmlEditor"];return(0,C.inArray)(e.editorType,t)!==-1},_getLabelOptions:function(e,t,n){var i=(0,w.extend)({showColon:this.option("showColonAfterLabel"),location:this.option("labelLocation"),id:t,visible:!0,isRequired:n},e?e.label:{});return this._isLabelNeedId(e)&&(i.labelID="dx-label-".concat(new f.default)),!i.text&&e.dataField&&(i.text=(0,E.captionize)(e.dataField)),i.text&&(i.text+=i.showColon?":":""),i},_renderLabel:function(e){var t=e.text,n=e.id,i=e.location,o=e.alignment,a=e.isRequired,r=e.labelID,s=void 0===r?null:r;if((0,_.isDefined)(t)&&t.length>0){var l=B.FIELD_ITEM_LABEL_CLASS+" "+B.FIELD_ITEM_LABEL_LOCATION_CLASS+i,u=(0,d.default)("<label>").addClass(l).attr("for",n).attr("id",s),c=(0,d.default)("<span>").addClass(B.FIELD_ITEM_LABEL_CONTENT_CLASS).appendTo(u);return(0,d.default)("<span>").addClass(B.FIELD_ITEM_LABEL_TEXT_CLASS).text(t).appendTo(c),o&&u.css("textAlign",o),c.append(this._renderLabelMark(a)),u}},_renderLabelMark:function(e){var t,n=this._getRequiredMarksConfig(),i=n.showRequiredMark&&e,o=n.showOptionalMark&&!e;if(i||o){var a=i?B.FIELD_ITEM_REQUIRED_MARK_CLASS:B.FIELD_ITEM_OPTIONAL_MARK_CLASS,r=i?n.requiredMark:n.optionalMark;t=(0,d.default)("<span>").addClass(a).html("&nbsp"+r)}return t},_getRequiredMarksConfig:function(){return this._cashedRequiredConfig||(this._cashedRequiredConfig={showRequiredMark:this.option("showRequiredMark"),showOptionalMark:this.option("showOptionalMark"),requiredMark:this.option("requiredMark"),optionalMark:this.option("optionalMark")}),this._cashedRequiredConfig},_renderEditor:function(e){var t=this._getDataByField(e.dataField),n=void 0!==t||this._isCheckboxUndefinedStateEnabled(e)?{value:t}:{},i=!0;K.indexOf(e.editorType)!==-1&&(n.value=n.value||[]);var o=this.option("form"),a=(0,w.extend)(i,n,e.editorOptions,{inputAttr:{id:e.id},validationBoundary:e.validationBoundary,stylingMode:o&&o.option("stylingMode")});this._replaceDataOptions(e.editorOptions,a);var r={editorType:e.editorType,dataField:e.dataField,template:e.template,name:e.name,helpID:e.helpID,labelID:e.labelID,isRequired:e.isRequired};return this._createEditor(e.$container,r,a)},_replaceDataOptions:function(e,t){e&&$.forEach(function(n){t[n]&&(t[n]=e[n])})},_renderValidator:function(e,t){var n=this._getFieldLabelName(t),i=this._prepareValidationRules(t.validationRules,t.isRequired,t.itemType,n);Array.isArray(i)&&i.length&&this._createComponent(e,P.default,{validationRules:i,validationGroup:this.option("validationGroup"),dataGetter:function(){return{formItem:t}}})},_getFieldLabelName:function(e){var t=e.label&&e.label.text,n=t?null:this._getName(e);return t?e.label.text:n&&(0,E.captionize)(n)},_prepareValidationRules:function(e,t,n,i){var o,a=n===G;if(a)if(e)o=e;else{var r=(0,b.format)(this.option("requiredMessage"),i||"");o=t?[{type:"required",message:r}]:null}return o},_addWrapperInvalidClass:function(e){var t="."+B.FIELD_ITEM_CONTENT_WRAPPER_CLASS,n=function(e){(0,d.default)(e.element).parents(t).toggleClass(N,e.component._isFocused()&&e.component.option("isValid")===!1)};e.on("focusIn",n).on("focusOut",n).on("enterKey",n)},_createEditor:function(e,t,n){var i,o=this,a=t.template;if(t.dataField&&!n.name&&(n.name=t.dataField),o._addItemContentClasses(e),a){var r={dataField:t.dataField,editorType:t.editorType,editorOptions:n,component:o._getComponentOwner(),name:t.name};a.render({model:r,container:(0,m.getPublicElement)(e)})}else{var s=(0,d.default)("<div>").appendTo(e);try{i=o._createComponent(s,t.editorType,n),i.setAria("describedby",t.helpID),i.setAria("labelledby",t.labelID),i.setAria("required",t.isRequired),t.dataField&&o._bindDataField(i,t,e)}catch(e){I.default.log("E1035",e.message)}}return i},_getComponentOwner:function(){return this.option("form")||this},_bindDataField:function(e,t,n){var i=this._getComponentOwner();e.on("enterKey",function(e){i._createActionByOption("onEditorEnterKey")((0,w.extend)(e,{dataField:t.dataField}))}),this._createWatcher(e,n,t),this.linkEditorToDataField(e,t.dataField,t.editorType)},_createWatcher:function(e,t,n){var i=this,o=i._getWatch();if((0,_.isFunction)(o)){var a=o(function(){return i._getDataByField(n.dataField)},function(){e.option("value",i._getDataByField(n.dataField))},{deep:!0,skipImmediate:!0});h.default.on(t,S.default,a)}},_getWatch:function(){if(!(0,_.isDefined)(this._watch)){var e=this.option("form");this._watch=e&&e.option("integrationOptions.watchMethod")}return this._watch},_addItemContentClasses:function(e){var t=this._getItemContentLocationSpecificClass();e.addClass([B.FIELD_ITEM_CONTENT_CLASS,t].join(" "))},_getItemContentLocationSpecificClass:function(){var e=this.option("labelLocation"),t={right:"left",left:"right",top:"bottom"};return B.FIELD_ITEM_CONTENT_LOCATION_CLASS+t[e]},_createComponent:function(e,t,n){var i=this,o=this.option("readOnly"),a=i.callBase(e,t,n);return o&&a.option("readOnly",o),i.on("optionChanged",function(e){"readOnly"!==e.name||(0,_.isDefined)(n.readOnly)||a.option(e.name,e.value)}),a},_getTemplateByFieldItem:function(e){return e.template?this._getTemplate(e.template):null},_appendEditorToField:function(e){if(e.$label){var t=e.labelOptions.location;"top"!==t&&"left"!==t||e.$fieldItem.append(e.$editor),"right"===t&&e.$fieldItem.prepend(e.$editor),this._addInnerItemAlignmentClass(e.$fieldItem,t)}else e.$fieldItem.append(e.$editor)},_addInnerItemAlignmentClass:function(e,t){"top"===t?e.addClass(B.LABEL_VERTICAL_ALIGNMENT_CLASS):e.addClass(B.LABEL_HORIZONTAL_ALIGNMENT_CLASS)},_renderHelpText:function(e,t,n){var i=e.helpText,o=e.itemType===G;if(i&&o){var a=(0,d.default)("<div>").addClass(B.FIELD_ITEM_CONTENT_WRAPPER_CLASS);t.wrap(a),(0,d.default)("<div>").addClass(B.FIELD_ITEM_HELP_TEXT_CLASS).attr("id",n).text(i).appendTo(t.parent())}},_attachClickHandler:function(e,t,n){var i="dxCheckBox"===n||"dxSwitch"===n;e&&i&&h.default.on(e,D.name,function(){h.default.trigger(t.children(),D.name)})},_generateRatio:function(e,t){var n,i,o=[];for(i=0;i<e;i++)n={ratio:1},t&&(n.baseSize="auto"),o.push(n);return o},_getRowsCount:function(){return Math.ceil(this._items.length/this._getColCount())},_updateReferencedOptions:function(e){var t=this,n=this.option("layoutData");(0,_.isObject)(n)&&Object.getOwnPropertyNames(n).forEach(function(e){return delete t._optionsByReference["layoutData."+e]}),(0,_.isObject)(e)&&Object.getOwnPropertyNames(e).forEach(function(e){return t._optionsByReference["layoutData."+e]=!0})},_resetWidget:function(e){this._disableEditorValueChangedHandler=!0,e.reset(),this._disableEditorValueChangedHandler=!1,e.option("isValid",!0)},_optionChanged:function(e){var t=this;if(0!==e.fullName.search("layoutData."))switch(e.name){case"showRequiredMark":case"showOptionalMark":case"requiredMark":case"optionalMark":this._cashedRequiredConfig=null,this._invalidate();break;case"layoutData":this._updateReferencedOptions(e.value),this.option("items")?(0,_.isEmptyObject)(e.value)||this._itemsRunTimeInfo.each(function(n,i){if((0,_.isDefined)(i.item)){var o=i.item.dataField;if(o&&(0,_.isDefined)(i.widgetInstance)){var a=(0,k.compileGetter)(o),r=a(e.value);void 0!==r||t._isCheckboxUndefinedStateEnabled(i.item)?i.widgetInstance.option("value",r):t._resetWidget(i.widgetInstance)}}}):(this._initDataAndItems(e.value),this._invalidate());break;case"items":this._cleanItemWatchers(),this._initDataAndItems(e.value),this._invalidate();break;case"alignItemLabels":case"labelLocation":case"requiredMessage":this._invalidate();break;case"customizeItem":this._updateItems(this.option("layoutData")),this._invalidate();break;case"colCount":this._resetColCount();break;case"minColWidth":"auto"===this.option("colCount")&&this._resetColCount();break;case"readOnly":break;case"width":this.callBase(e),"auto"===this.option("colCount")&&this._resetColCount();break;case"onFieldDataChanged":break;default:this.callBase(e)}},_resetColCount:function(){this._cashedColCount=null,this._invalidate()},linkEditorToDataField:function(e,t){var n=this;this.on("optionChanged",function(n){n.fullName==="layoutData.".concat(t)&&e._setOptionWithoutOptionChange("value",n.value)}),e.on("valueChanged",function(e){var i=(0,_.isObject)(e.value)||Array.isArray(e.value);n._disableEditorValueChangedHandler||i&&e.value===e.previousValue||n._updateFieldValue(t,e.value)})},_dimensionChanged:function(){"auto"===this.option("colCount")&&this.isCachedColCountObsolete()&&this._eventsStrategy.fireEvent("autoColCountChanged")},getItemID:function(e){var t=this.option("form");return t&&t.getItemID(e)},updateData:function(e,t){var n=this;(0,_.isObject)(e)?(0,x.each)(e,function(e,t){n._updateFieldValue(e,t)}):"string"==typeof e&&n._updateFieldValue(e,t)},getEditor:function(e){return this._itemsRunTimeInfo.findWidgetInstanceByDataField(e)||this._itemsRunTimeInfo.findWidgetInstanceByName(e)},isSingleColumnMode:function(e){var t=this._responsiveBox||e;if(t)return t.option("currentScreenFactor")===t.option("singleColumnScreen")},getItemsRunTimeInfo:function(){return this._itemsRunTimeInfo}});(0,g.default)("dxLayoutManager",U);var Y=U;t.default=Y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=o.default.inherit({ctor:function(e,t){var n=this;this.editor=e,this.validator=t,this.validationRequestsCallbacks=[];var i=function(e){n.validationRequestsCallbacks.forEach(function(t){return t(e)})};e.validationRequest.add(i),e.on("disposing",function(){e.validationRequest.remove(i)})},getValue:function(){return this.editor.option("value")},getCurrentValidationError:function(){return this.editor.option("validationError")},bypass:function(){return this.editor.option("disabled")},applyValidationResults:function(e){this.editor.option({validationErrors:e.brokenRules,validationStatus:e.status})},reset:function(){this.editor.reset()},focus:function(){this.editor.focus()}}),r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(9)),s=n(4),l=n(1),u=n(7),c=n(0),d=n(18),h=i(n(43)),f=n(21),p=n(30),g=i(n(13)),_=i(n(16)),m=n(8),v=i(n(58)),y=i(n(179)),b=n(71),x=n(6),w="dx-gallery",C=w+"-wrapper",k="dx-gallery-loop",S=w+"-container",D=w+"-active",I=w+"-item",T=w+"-item-invisible",O=I+"-loop",E="."+I,A=I+"-selected",P=w+"-indicator",R=P+"-item",M="."+R,B=R+"-selected",V="dx-gallery-item-image",F="dxGalleryItemData",L=1,H=_.default.inherit({_supportedKeys:function(){return(0,c.extend)(this.callBase(),{pageUp:s.noop,pageDown:s.noop})},_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{direction:"next",onClick:null,hoverStateEnabled:!0,activeStateEnabled:!0})},_render:function(){this.callBase();var e=this,t=this.$element(),n=(0,m.addNamespace)(f.name,this.NAME);t.addClass(w+"-nav-button-"+this.option("direction")),a.default.off(t,n),a.default.on(t,n,function(t){e._createActionByOption("onClick")({event:t})})},_optionChanged:function(e){switch(e.name){case"onClick":case"direction":this._invalidate();break;default:this.callBase(e)}}}),z=v.default.inherit({_activeStateUnit:E,_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{activeStateEnabled:!1,animationDuration:400,animationEnabled:!0,loop:!1,swipeEnabled:!0,indicatorEnabled:!0,showIndicator:!0,selectedIndex:0,slideshowDelay:0,showNavButtons:!1,wrapAround:!1,initialItemWidth:void 0,stretchImages:!1,_itemAttributes:{role:"option"},loopItemFocus:!1,selectOnFocus:!0,selectionMode:"single",selectionRequired:!0,selectionByClick:!1})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===g.default.real().deviceType&&!g.default.isSimulator()},options:{focusStateEnabled:!0}}])},_init:function(){this.callBase(),this.option("loopItemFocus",this.option("loop"))},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({item:new b.BindableTemplate(function(e,t){var n=(0,o.default)("<img>").addClass(V);(0,l.isPlainObject)(t)?(this._prepareDefaultItemTemplate(t,e),n.attr({src:t.imageSrc,alt:t.imageAlt}).appendTo(e)):n.attr("src",String(t)).appendTo(e)}.bind(this),["imageSrc","imageAlt","text","html"],this.option("integrationOptions.watchMethod"))})},_dataSourceOptions:function(){return{paginate:!1}},_itemContainer:function(){return this._$container},_itemClass:function(){return I},_itemDataKey:function(){return F},_actualItemWidth:function(){var e=this.option("wrapAround");if(this.option("stretchImages")){var t=e?this._itemsPerPage()+1:this._itemsPerPage();
return 1/t}return e?this._itemPercentWidth()*this._itemsPerPage()/(this._itemsPerPage()+1):this._itemPercentWidth()},_itemPercentWidth:function(){var e,t=this.$element().outerWidth(),n=this.option("initialItemWidth");return e=n&&n<=t?n/t:1},_itemsPerPage:function(){var e=(0,u.hasWindow)()?Math.floor(1/this._itemPercentWidth()):1;return Math.min(e,this._itemsCount())},_pagesCount:function(){return Math.ceil(this._itemsCount()/this._itemsPerPage())},_itemsCount:function(){return(this.option("items")||[]).length},_offsetDirection:function(){return this.option("rtlEnabled")?-1:1},_initMarkup:function(){this._renderWrapper(),this._renderItemsContainer(),this.$element().addClass(w),this.$element().toggleClass(k,this.option("loop")),this.callBase(),this.setAria({role:"listbox",label:"gallery"})},_render:function(){this._renderDragHandler(),this._renderContainerPosition(),this._renderItemSizes(),this._renderItemPositions(),this._renderNavButtons(),this._renderIndicator(),this._renderSelectedItem(),this._renderItemVisibility(),this._renderUserInteraction(),this._setupSlideShow(),this._reviseDimensions(),this.callBase()},_dimensionChanged:function(){var e=this.option("selectedIndex")||0;this._stopItemAnimations(),this._clearCacheWidth(),this._cloneDuplicateItems(),this._renderItemSizes(),this._renderItemPositions(),this._renderIndicator(),this._renderContainerPosition(this._calculateIndexOffset(e),!0),this._renderItemVisibility()},_renderDragHandler:function(){var e=(0,m.addNamespace)("dragstart",this.NAME);a.default.off(this.$element(),e),a.default.on(this.$element(),e,"img",function(){return!1})},_renderWrapper:function(){this._$wrapper||(this._$wrapper=(0,o.default)("<div>").addClass(C).appendTo(this.$element()))},_renderItems:function(e){if(!(0,u.hasWindow)()){var t=this.option("selectedIndex");e=e.length>t?e.slice(t,t+1):e.slice(0,1)}this.callBase(e),this._loadNextPageIfNeeded()},_renderItemsContainer:function(){this._$container||(this._$container=(0,o.default)("<div>").addClass(S).appendTo(this._$wrapper))},_cloneDuplicateItems:function(){if(this.option("loop")){var e,t=this.option("items")||[],n=t.length,i=n-1;if(n){this._getLoopedItems().remove();var o=Math.min(this._itemsPerPage(),n),a=this._getRealItems(),r=this._itemContainer();for(e=0;e<o;e++)this._cloneItemForDuplicate(a[e],r);for(e=0;e<o;e++)this._cloneItemForDuplicate(a[i-e],r)}}},_cloneItemForDuplicate:function(e,t){e&&(0,o.default)(e).clone(!0).addClass(O).css("margin",0).appendTo(t)},_getRealItems:function(){var e="."+I+":not(."+O+")";return this.$element().find(e)},_getLoopedItems:function(){return this.$element().find("."+O)},_emptyMessageContainer:function(){return this._$wrapper},_renderItemSizes:function(e){var t=this._itemElements(),n=this._actualItemWidth();void 0!==e&&(t=t.slice(e)),t.each(function(e){(0,o.default)(t[e]).outerWidth(100*n+"%")})},_renderItemPositions:function(){var e=this._actualItemWidth(),t=this._itemsCount(),n=this._itemsPerPage(),i=this.$element().find("."+O).length,a=t+i-1,r=this.option("wrapAround")?.5:0,s=this._itemFreeSpace(),l=!!s,u=this.option("rtlEnabled"),c=this.option("selectedIndex"),d=u?"Right":"Left";this._itemElements().each(function(i){var u=i,c=(0,o.default)(this).hasClass(O);if(i>t+n-1&&(u=a-u-n),!c&&0!==u)return void(l&&(0,o.default)(this).css("margin"+d,100*s+"%"));var h=e*(u+r)+s*(u+1-r),f=c?d.toLowerCase():"margin"+d;(0,o.default)(this).css(f,100*h+"%")}),this._relocateItems(c,c,!0)},_itemFreeSpace:function(){var e=this._itemsPerPage();return this.option("wrapAround")&&(e+=1),(1-this._actualItemWidth()*e)/(e+1)},_renderContainerPosition:function(e,t,n){this._releaseInvisibleItems(),e=e||0;var i,o=this,a=this._actualItemWidth(),r=e,s=this._offsetDirection()*r*(a+this._itemFreeSpace());return(0,l.isDefined)(this._animationOverride)&&(n=this._animationOverride,delete this._animationOverride),n?(o._startSwipe(),i=o._animate(s).done(o._endSwipe.bind(o))):((0,p.move)(this._$container,{left:s*this._elementWidth(),top:0}),i=(new x.Deferred).resolveWith(o)),i.done(function(){this._deferredAnimate&&o._deferredAnimate.resolveWith(o),t&&this._renderItemVisibility()}),i.promise()},_startSwipe:function(){this.$element().addClass(D)},_endSwipe:function(){this.$element().removeClass(D)},_animate:function(e,t){var n=this,i=this._$container,o=new x.Deferred;return h.default.animate(this._$container,(0,c.extend)({type:"slide",to:{left:e*this._elementWidth()},duration:n.option("animationDuration"),complete:function(){n._needMoveContainerForward()&&(0,p.move)(i,{left:0,top:0}),n._needMoveContainerBack()&&(0,p.move)(i,{left:n._maxContainerOffset()*n._elementWidth(),top:0}),o.resolveWith(n)}},t||{})),o},_needMoveContainerForward:function(){var e=this._$container.position().left*this._offsetDirection(),t=-this._maxItemWidth()*this._elementWidth()*this._itemsCount();return e<=t+L},_needMoveContainerBack:function(){var e=this._$container.position().left*this._offsetDirection(),t=this._actualItemWidth()*this._elementWidth();return e>=t-L},_maxContainerOffset:function(){return-this._maxItemWidth()*(this._itemsCount()-this._itemsPerPage())*this._offsetDirection()},_maxItemWidth:function(){return this._actualItemWidth()+this._itemFreeSpace()},_reviseDimensions:function(){var e=this,t=e._itemElements().first().find(".dx-item-content");t&&!t.is(":hidden")&&(e.option("height")||e.option("height",t.outerHeight()),e.option("width")||e.option("width",t.outerWidth()),this._dimensionChanged())},_renderIndicator:function(){if(this._cleanIndicators(),this.option("showIndicator")){for(var e=this._$indicator=(0,o.default)("<div>").addClass(P).appendTo(this._$wrapper),t=this.option("indicatorEnabled"),n=0;n<this._pagesCount();n++){var i=(0,o.default)("<div>").addClass(R).appendTo(e);t&&this._attachIndicatorClickHandler(i,n)}this._renderSelectedPageIndicator()}},_attachIndicatorClickHandler:function(e,t){a.default.on(e,(0,m.addNamespace)(f.name,this.NAME),function(e){this._indicatorSelectHandler(e,t)}.bind(this))},_detachIndicatorClickHandler:function(e){a.default.off(e,(0,m.addNamespace)(f.name,this.NAME))},_toggleIndicatorInteraction:function(e){var t,n=(null===(t=this._$indicator)||void 0===t?void 0:t.find(M))||[];n.length&&n.each(function(t,n){e?this._attachIndicatorClickHandler((0,o.default)(n),t):this._detachIndicatorClickHandler((0,o.default)(n))}.bind(this))},_cleanIndicators:function(){this._$indicator&&this._$indicator.remove()},_renderSelectedItem:function(){var e=this.option("selectedIndex");this._itemElements().removeClass(A).eq(e).addClass(A)},_renderItemVisibility:function(){return this.option("initialItemWidth")||this.option("wrapAround")?void this._releaseInvisibleItems():(this._itemElements().each(function(e,t){this.option("selectedIndex")===e?(0,o.default)(t).removeClass(T):(0,o.default)(t).addClass(T)}.bind(this)),void this._getLoopedItems().addClass(T))},_releaseInvisibleItems:function(){this._itemElements().removeClass(T),this._getLoopedItems().removeClass(T)},_renderSelectedPageIndicator:function(){if(this._$indicator){var e=this.option("selectedIndex"),t=this._pagesCount()-1,n=Math.ceil(e/this._itemsPerPage());n=Math.min(t,n),this._$indicator.find(M).removeClass(B).eq(n).addClass(B)}},_renderUserInteraction:function(){var e=this.$element(),t=this.option("swipeEnabled")&&this._itemsCount()>1;this._createComponent(e,y.default,{disabled:this.option("disabled")||!t,onStart:this._swipeStartHandler.bind(this),onUpdated:this._swipeUpdateHandler.bind(this),onEnd:this._swipeEndHandler.bind(this),itemSizeFunc:this._elementWidth.bind(this)})},_indicatorSelectHandler:function(e,t){if(this.option("indicatorEnabled")){var n=this._fitPaginatedIndex(t*this._itemsPerPage());this._needLongMove=!0,this.option("selectedIndex",n),this._loadNextPageIfNeeded(n)}},_renderNavButtons:function(){var e=this;return e.option("showNavButtons")?(e._prevNavButton=(0,o.default)("<div>").appendTo(this._$wrapper),e._createComponent(e._prevNavButton,H,{direction:"prev",onClick:function(){e._prevPage()}}),e._nextNavButton=(0,o.default)("<div>").appendTo(this._$wrapper),e._createComponent(e._nextNavButton,H,{direction:"next",onClick:function(){e._nextPage()}}),void this._renderNavButtonsVisibility()):void e._cleanNavButtons()},_prevPage:function(){var e=this._itemsPerPage(),t=this.option("selectedIndex")-e;return t===-e&&e===this._itemsCount()?this._relocateItems(t,0):this.goToItem(this._fitPaginatedIndex(t))},_nextPage:function(){var e=this._itemsPerPage(),t=this.option("selectedIndex")+e;return t===e&&e===this._itemsCount()?this._relocateItems(t,0):this.goToItem(this._fitPaginatedIndex(t)).done(this._loadNextPageIfNeeded)},_loadNextPageIfNeeded:function(e){e=void 0===e?this.option("selectedIndex"):e,this._dataSource&&this._dataSource.paginate()&&this._shouldLoadNextPage(e)&&!this._isDataSourceLoading()&&!this._isLastPage()&&this._loadNextPage().done(function(){this._renderIndicator(),this._cloneDuplicateItems(),this._renderItemPositions(),this._renderNavButtonsVisibility(),this._renderItemSizes(e)}.bind(this))},_shouldLoadNextPage:function(e){var t=this._itemsPerPage();return e+2*t>this.option("items").length},_allowDynamicItemsAppend:function(){return!0},_fitPaginatedIndex:function(e){var t=this._itemsPerPage(),n=e<0?t+e:this._itemsCount()-e;return e>this._itemsCount()-1?(e=0,this._goToGhostItem=!0):n<t&&n>0&&(e>0?e-=t-n:e+=t-n),e},_cleanNavButtons:function(){this._prevNavButton&&(this._prevNavButton.remove(),delete this._prevNavButton),this._nextNavButton&&(this._nextNavButton.remove(),delete this._nextNavButton)},_renderNavButtonsVisibility:function(){if(this.option("showNavButtons")&&this._prevNavButton&&this._nextNavButton){var e=this.option("selectedIndex"),t=this.option("loop"),n=this._itemsCount();if(this._prevNavButton.show(),this._nextNavButton.show(),0===n&&(this._prevNavButton.hide(),this._nextNavButton.hide()),!t){var i=e===n-this._itemsPerPage(),o=n<2||0===e;i=this._dataSource&&this._dataSource.paginate()?i&&this._isLastPage():i||n<2,o&&this._prevNavButton.hide(),i&&this._nextNavButton.hide()}}},_setupSlideShow:function(){var e=this,t=e.option("slideshowDelay");clearTimeout(e._slideshowTimer),t&&(e._slideshowTimer=setTimeout(function(){return e._userInteraction?void e._setupSlideShow():void e.nextItem(!0).done(e._setupSlideShow)},t))},_elementWidth:function(){return this._cacheElementWidth||(this._cacheElementWidth=this.$element().width()),this._cacheElementWidth},_clearCacheWidth:function(){delete this._cacheElementWidth},_swipeStartHandler:function(e){this._releaseInvisibleItems(),this._clearCacheWidth(),this._elementWidth();var t=this._itemsCount();if(!t)return void(e.event.cancel=!0);if(this._stopItemAnimations(),this._startSwipe(),this._userInteraction=!0,!this.option("loop")){var n=this.option("selectedIndex"),i=t-n-this._itemsPerPage(),o=n,a=this.option("rtlEnabled");e.event.maxLeftOffset=a?o:i,e.event.maxRightOffset=a?i:o}},_stopItemAnimations:function(){h.default.stop(this._$container,!0)},_swipeUpdateHandler:function(e){var t=this.option("wrapAround")?1:0,n=this._offsetDirection()*e.event.offset*(this._itemsPerPage()+t)-this.option("selectedIndex");n<0&&this._loadNextPageIfNeeded(Math.ceil(Math.abs(n))),this._renderContainerPosition(n)},_swipeEndHandler:function(e){var t=e.event.targetOffset*this._offsetDirection()*this._itemsPerPage(),n=this.option("selectedIndex"),i=this._fitIndex(n-t),o=this._fitPaginatedIndex(i);return Math.abs(t)<this._itemsPerPage()?void this._relocateItems(n):this._itemsPerPage()===this._itemsCount()?void(t>0?this._relocateItems(-t):this._relocateItems(0)):void this.option("selectedIndex",o)},_setFocusOnSelect:function(){this._userInteraction=!0;var e=this.itemElements().filter("."+A);this.option("focusedElement",(0,d.getPublicElement)(e)),this._userInteraction=!1},_flipIndex:function(e){var t=this._itemsCount();return e%=t,e>(t+1)/2&&(e-=t),e<-(t-1)/2&&(e+=t),e},_fitIndex:function(e){if(!this.option("loop"))return e;var t=this._itemsCount();return(e>=t||e<0)&&(this._goToGhostItem=!0),e>=t&&(e=t-e),e%=t,e<0&&(e+=t),e},_clean:function(){this.callBase(),this._cleanIndicators(),this._cleanNavButtons()},_dispose:function(){clearTimeout(this._slideshowTimer),this.callBase()},_updateSelection:function(e,t){this._stopItemAnimations(),this._renderNavButtonsVisibility(),this._renderSelectedItem(),this._relocateItems(e[0],t[0]),this._renderSelectedPageIndicator()},_relocateItems:function(e,t,n){void 0===t&&(t=e);var i=this._calculateIndexOffset(e,t);this._renderContainerPosition(i,!0,this.option("animationEnabled")&&!n).done(function(){this._setFocusOnSelect(),this._userInteraction=!1,this._setupSlideShow()})},_focusInHandler:function(){h.default.isAnimating(this._$container)||this._userInteraction||this.callBase.apply(this,arguments)},_focusOutHandler:function(){h.default.isAnimating(this._$container)||this._userInteraction||this.callBase.apply(this,arguments)},_selectFocusedItem:s.noop,_moveFocus:function(){this._stopItemAnimations(),this.callBase.apply(this,arguments);var e=this.itemElements().index((0,o.default)(this.option("focusedElement")));this.goToItem(e,this.option("animationEnabled"))},_visibilityChanged:function(e){e&&this._reviseDimensions()},_calculateIndexOffset:function(e,t){void 0===t&&(t=e);var n=t-e;return this.option("loop")&&!this._needLongMove&&this._goToGhostItem&&(this._isItemOnFirstPage(e)&&this._isItemOnLastPage(t)?n=-this._itemsPerPage():this._isItemOnLastPage(e)&&this._isItemOnFirstPage(t)&&(n=this._itemsPerPage()),this._goToGhostItem=!1),this._needLongMove=!1,n-=t},_isItemOnLastPage:function(e){return e>=this._itemsCount()-this._itemsPerPage()},_isItemOnFirstPage:function(e){return e<=this._itemsPerPage()},_optionChanged:function(e){switch(e.name){case"width":case"initialItemWidth":this.callBase.apply(this,arguments),this._dimensionChanged();break;case"animationDuration":this._renderNavButtonsVisibility();break;case"animationEnabled":break;case"loop":this.$element().toggleClass(k,e.value),this.option("loopItemFocus",e.value),(0,u.hasWindow)()&&(this._cloneDuplicateItems(),this._renderItemPositions(),this._renderNavButtonsVisibility());break;case"showIndicator":this._renderIndicator();break;case"showNavButtons":this._renderNavButtons();break;case"slideshowDelay":this._setupSlideShow();break;case"wrapAround":case"stretchImages":(0,u.hasWindow)()&&(this._renderItemSizes(),this._renderItemPositions(),this._renderItemVisibility());break;case"swipeEnabled":this._renderUserInteraction();break;case"indicatorEnabled":this._toggleIndicatorInteraction(e.value);break;default:this.callBase(e)}},goToItem:function(e,t){var n=this.option("selectedIndex"),i=this._itemsCount();return void 0!==t&&(this._animationOverride=t),e=this._fitIndex(e),this._deferredAnimate=new x.Deferred,e>i-1||e<0||n===e?this._deferredAnimate.resolveWith(this).promise():(this.option("selectedIndex",e),this._deferredAnimate.promise())},prevItem:function(e){return this.goToItem(this.option("selectedIndex")-1,e)},nextItem:function(e){return this.goToItem(this.option("selectedIndex")+1,e)}});(0,r.default)("dxGallery",z);var N=z;t.default=N,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(647)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.default=void 0;var a=i(n(2)),r=n(0),s=n(1),l=n(18),u=n(4),c=i(n(9)),d=n(93),h=i(n(52)),f=i(n(17)),p=i(n(27)),g=n(6),_=i(n(5)),m=n(8),v=i(n(285)),y=n(299),b=i(n(648));n(666);var x=i(n(300)),w=i(n(667)),C=i(n(668)),k="dx-htmleditor",S="dx-quill-container",D="ql-clipboard",I="dx-htmleditor-submit-element",T="dx-htmleditor-content",O="markdown",E="htmlContent",A=h.default.inherit({_getDefaultOptions:function(){return(0,r.extend)(this.callBase(),{focusStateEnabled:!0,valueType:"html",placeholder:"",toolbar:null,variables:null,mediaResizing:null,mentions:null,customizeModules:null,formDialogOptions:null})},_init:function(){this.callBase(),this._cleanCallback=(0,p.default)(),this._contentInitializedCallback=(0,p.default)()},_getAnonymousTemplateName:function(){return E},_initTemplates:function(){this._templateManager.addDefaultTemplates(o({},E,new d.EmptyTemplate)),this.callBase()},_focusTarget:function(){return this._getContent()},_getContent:function(){return this.$element().find(".".concat(T))},_focusInHandler:function(e){var t=e.relatedTarget;this._shouldSkipFocusEvent(t)||(this._toggleFocusClass(!0,this.$element()),this.callBase.apply(this,arguments))},_focusOutHandler:function(e){var t=e.relatedTarget;this._shouldSkipFocusEvent(t)||(this._toggleFocusClass(!1,this.$element()),this.callBase.apply(this,arguments))},_shouldSkipFocusEvent:function(e){return(0,a.default)(e).hasClass(D)},_initMarkup:function(){this._$htmlContainer=(0,a.default)("<div>").addClass(S),this.$element().addClass(k).wrapInner(this._$htmlContainer);var e=this._getTemplate(E),t=!0;this._$templateResult=e&&e.render({container:(0,l.getPublicElement)(this._$htmlContainer),noModel:!0,transclude:t}),this._renderSubmitElement(),this.callBase(),this._updateContainerMarkup()},_renderSubmitElement:function(){this._$submitElement=(0,a.default)("<textarea>").addClass(I).attr("hidden",!0).appendTo(this.$element()),this._setSubmitValue(this.option("value"))},_setSubmitValue:function(e){this._getSubmitElement().val(e)},_getSubmitElement:function(){return this._$submitElement},_updateContainerMarkup:function(){var e=this.option("value");this._isMarkdownValue()&&(this._prepareMarkdownConverter(),e=this._markdownConverter.toHtml(e)),e&&this._$htmlContainer.html(e)},_prepareMarkdownConverter:function(){var e=x.default.getConverter("markdown");if(!e)throw f.default.Error("E1051","markdown");this._markdownConverter=new e},_render:function(){this._prepareConverters(),this.callBase()},_prepareQuillRegistrator:function(){this._quillRegistrator||(this._quillRegistrator=new b.default)},_getRegistrator:function(){return this._prepareQuillRegistrator(),this._quillRegistrator},_prepareConverters:function(){if(!this._deltaConverter){var e=x.default.getConverter("delta");e&&(this._deltaConverter=new e)}this.option("valueType")!==O||this._markdownConverter||this._prepareMarkdownConverter()},_renderContentImpl:function(){this._contentRenderedDeferred=new g.Deferred;var e=this._contentRenderedDeferred.promise();return this.callBase(),this._renderHtmlEditor(),this._renderFormDialog(),this._addKeyPressHandler(),e},_attachFocusEvents:function(){(0,u.deferRender)(this.callBase.bind(this))},_addKeyPressHandler:function(){var e=(0,m.addNamespace)("keydown","".concat(this.NAME,"TextChange"));_.default.on(this._$htmlContainer,e,this._keyDownHandler.bind(this))},_keyDownHandler:function(e){this._saveValueChangeEvent(e)},_renderHtmlEditor:function(){var e=this,t=this.option("customizeModules"),n=this._getModulesConfig();(0,s.isFunction)(t)&&t(n),this._quillInstance=this._getRegistrator().createEditor(this._$htmlContainer[0],{placeholder:this.option("placeholder"),readOnly:this.option("readOnly")||this.option("disabled"),modules:n,theme:"basic"}),this._deltaConverter.setQuillInstance(this._quillInstance),this._textChangeHandlerWithContext=this._textChangeHandler.bind(this),this._quillInstance.on("text-change",this._textChangeHandlerWithContext),this._renderScrollHandler(),this._hasTranscludedContent()?this._updateContentTask=(0,u.executeAsync)(function(){e._applyTranscludedContent()}):this._finalizeContentRendering()},_renderScrollHandler:function(){var e=this._getContent(),t=(0,y.prepareScrollData)(e);_.default.on(e,(0,m.addNamespace)(v.default.init,this.NAME),t,u.noop)},_applyTranscludedContent:function(){var e=this.option("value");if(!(0,s.isDefined)(e)){var t=this._deltaConverter.toHtml(),n=this._quillInstance.clipboard.convert({html:t});if(n.ops.length)return void this._quillInstance.setContents(n)}this._finalizeContentRendering()},_hasTranscludedContent:function(){return this._$templateResult&&this._$templateResult.length},_getModulesConfig:function(){var e=this,t=this._getRegistrator().getQuill(),n=(0,w.default)(t),i=(0,r.extend)({},{table:!0,toolbar:this._getModuleConfigByOption("toolbar"),variables:this._getModuleConfigByOption("variables"),resizing:this._getModuleConfigByOption("mediaResizing"),mentions:this._getModuleConfigByOption("mentions"),uploader:{onDrop:function(t){return e._saveValueChangeEvent(t)},imageBlot:"extendedImage"},clipboard:{onPaste:function(t){return e._saveValueChangeEvent(t)},onCut:function(t){return e._saveValueChangeEvent(t)},matchers:[["p.MsoListParagraphCxSpFirst",n],["p.MsoListParagraphCxSpMiddle",n],["p.MsoListParagraphCxSpLast",n]]}},this._getCustomModules());return i},_getModuleConfigByOption:function(e){var t=this.option(e),n={};if((0,s.isDefined)(t))return Array.isArray(t)?n[e]=t:n=t,(0,r.extend)(this._getBaseModuleConfig(),n)},_getBaseModuleConfig:function(){return{editorInstance:this}},_getCustomModules:function(){var e=this,t={},n=this._getRegistrator().getRegisteredModuleNames();return n.forEach(function(n){t[n]=e._getBaseModuleConfig()}),t},_textChangeHandler:function(e,t,n){var i=this._deltaConverter.toHtml(),o=this._isMarkdownValue()?this._updateValueByType(O,i):i,a=this.option("value");a===o||this._isNullValueConverted(a,o)||(this._isEditorUpdating=!0,this.option("value",o)),this._finalizeContentRendering()},_isNullValueConverted:function(e,t){return null===e&&""===t},_finalizeContentRendering:function(){this._contentRenderedDeferred&&(this.clearHistory(),this._contentInitializedCallback.fire(),this._contentRenderedDeferred.resolve(),this._contentRenderedDeferred=void 0)},_updateValueByType:function(e,t){var n=this._markdownConverter;if((0,s.isDefined)(n)){var i=(0,u.ensureDefined)(t,this.option("value"));return e===O?n.toMarkdown(i):n.toHtml(i)}},_isMarkdownValue:function(){return this.option("valueType")===O},_resetEnabledState:function(){if(this._quillInstance){var e=!(this.option("readOnly")||this.option("disabled"));this._quillInstance.enable(e)}},_renderFormDialog:function(){var e=(0,r.extend)(!0,{width:"auto",height:"auto",closeOnOutsideClick:!0},this.option("formDialogOptions"));this._formDialog=new C.default(this,e)},_getQuillContainer:function(){return this._$htmlContainer},_optionChanged:function(e){switch(e.name){case"value":if(this._quillInstance)if(this._isEditorUpdating)this._isEditorUpdating=!1;else{var t=this._isMarkdownValue()?this._updateValueByType("HTML",e.value):e.value;this._updateHtmlContent(t)}else this._$htmlContainer.html(e.value);this._setSubmitValue(e.value),this.callBase(e);break;case"placeholder":case"variables":case"toolbar":case"mentions":case"customizeModules":this._invalidate();break;case"valueType":this._prepareConverters();var n=this._updateValueByType(e.value);"html"===e.value&&this._quillInstance?this._updateHtmlContent(n):this.option("value",n);break;case"readOnly":case"disabled":this.callBase(e),this._resetEnabledState();break;case"formDialogOptions":this._renderFormDialog();break;case"mediaResizing":e.previousValue&&e.value?this._quillInstance.getModule("resizing").option(e.name,e.value):this._invalidate();break;case"width":this.callBase(e),this._repaintToolbar();break;default:this.callBase(e)}},_repaintToolbar:function(){var e=this._quillInstance.getModule("toolbar");e&&e.repaint()},_updateHtmlContent:function(e){var t=this._quillInstance.clipboard.convert({html:e});this._quillInstance.setContents(t)},_clean:function(){this._quillInstance&&(_.default.off(this._getContent(),".".concat(this.NAME)),this._quillInstance.off("text-change",this._textChangeHandlerWithContext),this._cleanCallback.fire()),this._abortUpdateContentTask(),this._cleanCallback.empty(),this._contentInitializedCallback.empty(),this.callBase()},_abortUpdateContentTask:function(){this._updateContentTask&&(this._updateContentTask.abort(),this._updateContentTask=void 0)},_applyQuillMethod:function(e,t){if(this._quillInstance)return this._quillInstance[e].apply(this._quillInstance,t)},_applyQuillHistoryMethod:function(e){this._quillInstance&&this._quillInstance.history&&this._quillInstance.history[e]()},addCleanCallback:function(e){this._cleanCallback.add(e)},addContentInitializedCallback:function(e){this._contentInitializedCallback.add(e)},register:function(e){this._getRegistrator().registerModules(e),this._quillInstance&&this.repaint()},get:function(e){return this._getRegistrator().getQuill().import(e)},getModule:function(e){return this._applyQuillMethod("getModule",arguments)},getQuillInstance:function(){return this._quillInstance},getSelection:function(){return this._applyQuillMethod("getSelection")},setSelection:function(e,t){this._applyQuillMethod("setSelection",arguments)},format:function(e,t){this._applyQuillMethod("format",arguments)},formatText:function(e,t,n,i){this._applyQuillMethod("formatText",arguments)},formatLine:function(e,t,n,i){this._applyQuillMethod("formatLine",arguments)},getFormat:function(e,t){return this._applyQuillMethod("getFormat",arguments)},removeFormat:function(e,t){return this._applyQuillMethod("removeFormat",arguments)},clearHistory:function(){this._applyQuillHistoryMethod("clear")},undo:function(){this._applyQuillHistoryMethod("undo")},redo:function(){this._applyQuillHistoryMethod("redo")},getLength:function(){return this._applyQuillMethod("getLength")},"delete":function(e,t){this._applyQuillMethod("deleteText",arguments)},insertText:function(e,t,n){this._applyQuillMethod("insertText",arguments)},insertEmbed:function(e,t,n){this._applyQuillMethod("insertEmbed",arguments)},showFormDialog:function(e){return this._formDialog.show(e)},formDialogOption:function(e,t){return this._formDialog.popupOption.apply(this._formDialog,arguments)},focus:function(){this.callBase(),this._applyQuillMethod("focus")}});(0,c.default)("dxHtmlEditor",A);var P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(445),l=i(n(649)),u=i(n(650)),c=i(n(651)),d=i(n(652)),h=i(n(653)),f=i(n(654)),p=i(n(655)),g=i(n(660)),_=i(n(661)),m=i(n(663)),v=i(n(664)),y=function(){function e(){if(o(this,e),!e.initialized){var t=this.getQuill(),n=t.import("attributors/style/direction");t.register({"formats/align":f.default,"formats/direction":n,"formats/font":d.default,"formats/size":h.default,"formats/extendedImage":u.default,"formats/link":c.default,"modules/toolbar":p.default,"modules/dropImage":g.default,"modules/variables":_.default,"modules/resizing":m.default,"modules/mentions":v.default,"themes/basic":l.default},!0),this._customModules=[],e._initialized=!0}}return r(e,[{key:"createEditor",value:function(e,t){var n=this.getQuill();return new n(e,t)}},{key:"registerModules",value:function(e){var t=RegExp("modules/*"),n=this.getQuill(),i=function(e){return!!n.imports[e]};for(var o in e)t.test(o)&&!i(o)&&this._customModules.push(o.slice(8));n.register(e,!0)}},{key:"getRegisteredModuleNames",value:function(){return this._customModules}},{key:"getQuill",value:function(){return(0,s.getQuill)()}}]),e}(),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&s(e,t)}function s(e,t){return(s=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function l(e){var t=d();return function(){var n,i=h(e);if(t){var o=h(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return u(this,n)}}function u(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?c(e):t}function c(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function d(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function h(e){return(h=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var f,p=o(n(63));if(p.default){var g=p.default.import("core/theme");f=function(e){function t(e,i){var o;return a(this,t),o=n.call(this,e,i),o.quill.root.classList.add("dx-htmleditor-content"),o}r(t,e);var n=l(t);return t}(g)}else f={};var _=f;t.default=_,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(63)),v=n(1),y={};if(m.default){var b=m.default.import("formats/image");y=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"formats",value:function e(){var e=l(_(t.prototype),"formats",this).call(this),n=this.domNode.style.float;return n&&(e.float=n),e}},{key:"format",value:function(e,n){"float"===e?this.domNode.style[e]=n:l(_(t.prototype),"format",this).call(this,e,n)}}],[{key:"create",value:function(e){var n=e&&e.src||e,i=l(_(t),"create",this).call(this,n);if((0,v.isObject)(e)){var o=function(t,n){e[t]&&i.setAttribute(t,n)};o("alt",e.alt),o("width",e.width),o("height",e.height)}return i}},{key:"formats",value:function e(n){var e=l(_(t),"formats",this).call(this,n);
return e.imageSrc=n.getAttribute("src"),e}},{key:"value",value:function(e){return{src:e.getAttribute("src"),width:e.getAttribute("width"),height:e.getAttribute("height"),alt:e.getAttribute("alt")}}}]),t}(b),y.blotName="extendedImage"}var x=y;t.default=x,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(63)),v=n(1),y={};if(m.default){var b=m.default.import("formats/link");y=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"formats",value:function e(){var e=l(_(t.prototype),"formats",this).call(this),n=t.formats(this.domNode),i=n.href,o=n.target;return e.link=i,e.target=o,e}},{key:"format",value:function(e,n){"link"===e&&(0,v.isObject)(n)?(n.text&&(this.domNode.innerText=n.text),n.target?this.domNode.setAttribute("target","_blank"):this.domNode.removeAttribute("target"),this.domNode.setAttribute("href",n.href)):l(_(t.prototype),"format",this).call(this,e,n)}}],[{key:"create",value:function(e){var n=e&&e.href||e,i=l(_(t),"create",this).call(this,n);return(0,v.isObject)(e)&&(e.text&&(i.innerText=e.text),e.target||i.removeAttribute("target")),i}},{key:"formats",value:function(e){return{href:e.getAttribute("href"),target:e.getAttribute("target")}}},{key:"value",value:function(e){return{href:e.getAttribute("href"),text:e.innerText,target:!!e.getAttribute("target")}}}]),t}(b)}var x=y;t.default=x,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(63)),a={};o.default&&(a=o.default.import("attributors/style/font"),a.whitelist=null);var r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(63)),a={};o.default&&(a=o.default.import("attributors/style/size"),a.whitelist=null);var r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(63)),a={};o.default&&(a=o.default.import("attributors/style/align"),a.whitelist.push("left"));var r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&g(e,t)}function g(e,t){return(g=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function _(e){var t=y();return function(){var n,i=b(e);if(t){var o=b(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return m(this,n)}}function m(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?v(e):t}function v(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function y(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function b(e){return(b=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var x=o(n(63)),w=o(n(2)),C=o(n(166));n(127),n(435),n(126);var k=o(n(659)),S=n(3),D=n(1),I=n(0),T=o(n(10)),O=n(40),E=o(n(5)),A=n(8),P={};if(x.default){var R=x.default.import("core/module"),M="dx-htmleditor-toolbar-wrapper",B="dx-htmleditor-toolbar",V="dx-htmleditor-toolbar-format",F="dx-htmleditor-toolbar-separator",L="dx-htmleditor-toolbar-menu-separator",H="dx-format-active",z="dx-box-item-content",N="dx-icon",j="selection-change",W="dxHtmlEditor-dialogColorCaption",G="dxHtmlEditor-dialogBackgroundCaption",q="dxHtmlEditor-dialogLinkCaption",$="dxHtmlEditor-dialogLinkUrlField",K="dxHtmlEditor-dialogLinkTextField",U="dxHtmlEditor-dialogLinkTargetField",Y="dx-formdialog-field-target",Z="dxHtmlEditor-dialogImageCaption",X="dxHtmlEditor-dialogImageUrlField",Q="dxHtmlEditor-dialogImageAltField",J="dxHtmlEditor-dialogImageWidthField",ee="dxHtmlEditor-dialogImageHeightField",te="dxHtmlEditor-dialogInsertTableRowsField",ne="dxHtmlEditor-dialogInsertTableColumnsField",ie="dxHtmlEditor-dialogInsertTableCaption",oe=["insertTable","insertRowAbove","insertRowBelow","insertColumnLeft","insertColumnRight","deleteColumn","deleteRow","deleteTable"],ae="user",re="silent",se=T.default.format("dxHtmlEditor-heading"),le=T.default.format("dxHtmlEditor-normalText");P=function(e){function t(e,i){var o;return d(this,t),o=n.call(this,e,i),o._editorInstance=i.editorInstance,o._toolbarWidgets=new k.default,o._formatHandlers=o._getFormatHandlers(),(0,D.isDefined)(i.items)&&(o._addCallbacks(),o._renderToolbar(),o.quill.on("editor-change",function(e){var t=e===j;o._updateToolbar(t)})),o}p(t,e);var n=_(t);return f(t,[{key:"_applyFormat",value:function(e,t){var n;this._editorInstance._saveValueChangeEvent(t),(n=this.quill).format.apply(n,a(e))}},{key:"_addCallbacks",value:function(){this._editorInstance.addCleanCallback(this.clean.bind(this)),this._editorInstance.addContentInitializedCallback(this.updateHistoryWidgets.bind(this))}},{key:"_updateToolbar",value:function(e){this.updateFormatWidgets(e),this.updateHistoryWidgets(),this.updateTableWidgets()}},{key:"_getDefaultClickHandler",value:function(e){var t=this;return function(n){var i=n.event,o=t.quill.getFormat(),a=o[e],r=!((0,D.isBoolean)(a)?a:(0,D.isDefined)(a));t._applyFormat([e,r,ae],i),t._updateFormatWidget(e,r,o)}}},{key:"_updateFormatWidget",value:function(e,t,n){var i=this._toolbarWidgets.getByName(e);i&&(t?this._markActiveFormatWidget(e,i,n):(this._resetFormatWidget(e,i),Object.prototype.hasOwnProperty.call(e)&&delete n[e]),this._toggleClearFormatting(t||!(0,D.isEmptyObject)(n)))}},{key:"_getFormatHandlers",value:function(){var e=this;return{clear:function(t){var n=e.quill.getSelection();n&&(e.quill.removeFormat(n),e.updateFormatWidgets())},link:this._prepareLinkHandler(),image:this._prepareImageHandler(),color:this._prepareColorClickHandler("color"),background:this._prepareColorClickHandler("background"),orderedList:this._prepareShortcutHandler("list","ordered"),bulletList:this._prepareShortcutHandler("list","bullet"),alignLeft:this._prepareShortcutHandler("align","left"),alignCenter:this._prepareShortcutHandler("align","center"),alignRight:this._prepareShortcutHandler("align","right"),alignJustify:this._prepareShortcutHandler("align","justify"),codeBlock:this._getDefaultClickHandler("code-block"),undo:function(t){var n=t.event;e._editorInstance._saveValueChangeEvent(n),e.quill.history.undo()},redo:function(t){var n=t.event;e._editorInstance._saveValueChangeEvent(n),e.quill.history.redo()},increaseIndent:function(t){var n=t.event;e._applyFormat(["indent","+1",ae],n)},decreaseIndent:function(t){var n=t.event;e._applyFormat(["indent","-1",ae],n)},superscript:this._prepareShortcutHandler("script","super"),subscript:this._prepareShortcutHandler("script","sub"),insertTable:this._prepareInsertTableHandler(),insertRowAbove:this._getTableOperationHandler("insertRowAbove"),insertRowBelow:this._getTableOperationHandler("insertRowBelow"),insertColumnLeft:this._getTableOperationHandler("insertColumnLeft"),insertColumnRight:this._getTableOperationHandler("insertColumnRight"),deleteColumn:this._getTableOperationHandler("deleteColumn"),deleteRow:this._getTableOperationHandler("deleteRow"),deleteTable:this._getTableOperationHandler("deleteTable")}}},{key:"_prepareShortcutHandler",value:function(e,t){var n=this;return function(i){var o=i.event,a=n.quill.getFormat(),r=a[e]!==t&&t;n._applyFormat([e,r,ae],o),n.updateFormatWidgets(!0)}}},{key:"_prepareLinkHandler",value:function(){var e=this;return function(){e.quill.focus();var t=e.quill.getSelection(),n=e._hasEmbedContent(t),i=t?e.quill.getFormat():{},o={href:i.link||"",text:t&&!n?e.quill.getText(t):"",target:!Object.prototype.hasOwnProperty.call(i,"target")||!!i.target};e._editorInstance.formDialogOption("title",T.default.format(q));var a=e._editorInstance.showFormDialog({formData:o,items:e._getLinkFormItems(t)});a.done(function(i,o){if(t&&!n){var a=i.text||i.href,r=t.index,s=t.length;i.text=void 0,e._editorInstance._saveValueChangeEvent(o),s&&e.quill.deleteText(r,s,re),e.quill.insertText(r,a,"link",i,ae),e.quill.setSelection(r+a.length,0,ae)}else i.text=t||i.text?i.text:i.href,e._applyFormat(["link",i,ae],o)}),a.fail(function(){e.quill.focus()})}}},{key:"_hasEmbedContent",value:function(e){return!!e&&this.quill.getText(e).trim().length<e.length}},{key:"_getLinkFormItems",value:function(e){return[{dataField:"href",label:{text:T.default.format($)}},{dataField:"text",label:{text:T.default.format(K)},visible:!this._hasEmbedContent(e)},{dataField:"target",editorType:"dxCheckBox",editorOptions:{text:T.default.format(U)},cssClass:Y,label:{visible:!1}}]}},{key:"_prepareImageHandler",value:function(){var e=this;return function(){var t=e.quill.getFormat(),n=Object.prototype.hasOwnProperty.call(t,"imageSrc"),i=e._defaultPasteIndex;if(n){var o=e.quill.getFormat(i-1,1),a=o.imageSrc;t.src=t.imageSrc,delete t.imageSrc,a&&0!==i||e.quill.setSelection(i+1,0,re)}var r=e._embedFormatIndex;e._editorInstance.formDialogOption("title",T.default.format(Z));var s=e._editorInstance.showFormDialog({formData:t,items:e._imageFormItems});s.done(function(t,o){var a=i;e._editorInstance._saveValueChangeEvent(o),n&&(a=r,e.quill.deleteText(a,1,re)),e.quill.insertEmbed(a,"extendedImage",t,ae),e.quill.setSelection(a+1,0,ae)}).always(function(){e.quill.focus()})}}},{key:"_prepareInsertTableHandler",value:function(){var e=this;return function(){var t=e.quill.getFormat(),n=Object.prototype.hasOwnProperty.call(t,"table"),i={rows:1,columns:1};if(n)return void e.quill.focus();e._editorInstance.formDialogOption("title",T.default.format(ie));var o=e._editorInstance.showFormDialog({formData:i,items:e._insertTableFormItems});o.done(function(t,n){e.quill.focus();var i=e.quill.getModule("table");if(i){e._editorInstance._saveValueChangeEvent(n);var o=t.columns,a=t.rows;i.insertTable(o,a)}}).always(function(){e.quill.focus()})}}},{key:"_getTableOperationHandler",value:function(e){for(var t=this,n=arguments.length,i=new Array(n>1?n-1:0),o=1;o<n;o++)i[o-1]=arguments[o];return function(){var n=t.quill.getModule("table");if(n)return t.quill.focus(),n[e].apply(n,i)}}},{key:"_renderToolbar",value:function(){var e=this,t=this.options.container||this._getContainer();this._$toolbar=(0,w.default)("<div>").addClass(B).appendTo(t),this._$toolbarContainer=(0,w.default)(t).addClass(M),E.default.on(this._$toolbarContainer,(0,A.addNamespace)("mousedown",this._editorInstance.NAME),function(e){e.preventDefault()}),this.toolbarInstance=this._editorInstance._createComponent(this._$toolbar,C.default,this.toolbarConfig),this._editorInstance.on("optionChanged",function(t){var n=t.name;"readOnly"!==n&&"disabled"!==n||e.toolbarInstance.option("disabled",e.isInteractionDisabled)})}},{key:"isMultilineMode",value:function(){var e;return null===(e=this.options.multiline)||void 0===e||e}},{key:"clean",value:function(){this._toolbarWidgets.clear(),this._$toolbarContainer&&this._$toolbarContainer.empty().removeClass(M)}},{key:"repaint",value:function(){this.toolbarInstance&&this.toolbarInstance.repaint()}},{key:"_getContainer",value:function(){var e=(0,w.default)("<div>");return this._editorInstance.$element().prepend(e),e}},{key:"_prepareToolbarItems",value:function(){var e=this,t=[];return(0,S.each)(this.options.items,function(n,i){var o;if((0,D.isObject)(i))o=e._handleObjectItem(i);else if((0,D.isString)(i)){var a=e._prepareButtonItemConfig(i);o=e._getToolbarItem(a)}o&&t.push(o)}),t}},{key:"_handleObjectItem",value:function(e){if(e.formatName&&e.formatValues&&this._isAcceptableItem(e.widget,"dxSelectBox")){var t=this._prepareSelectItemConfig(e);return this._getToolbarItem(t)}if(e.formatName&&this._isAcceptableItem(e.widget,"dxButton")){var n=this._prepareButtonItemConfig(e.formatName),i=(0,I.extend)(!0,n,e);return this._getToolbarItem(i)}return this._getToolbarItem(e)}},{key:"_isAcceptableItem",value:function(e,t){return!e||e===t}},{key:"_prepareButtonItemConfig",value:function(e){var t="clear"===e?"clearformat":e,n=(0,O.titleize)(e);return{widget:"dxButton",formatName:e,options:{hint:n,text:n,icon:t.toLowerCase(),onClick:this._formatHandlers[e]||this._getDefaultClickHandler(e),stylingMode:"text"},showText:"inMenu"}}},{key:"_prepareSelectItemConfig",value:function(e){var t=this;return(0,I.extend)(!0,{widget:"dxSelectBox",formatName:e.formatName,options:{stylingMode:"filled",dataSource:e.formatValues,placeholder:(0,O.titleize)(e.formatName),onValueChanged:function(n){t._isReset||(t._applyFormat([e.formatName,n.value,ae],n.event),t._setValueSilent(n.component,n.value))}}},e)}},{key:"_prepareColorClickHandler",value:function(e){var t=this;return function(){var n=t.quill.getFormat(),i="color"===e?W:G;t._editorInstance.formDialogOption("title",T.default.format(i));var o=t._editorInstance.showFormDialog({formData:n,items:[{dataField:e,editorType:"dxColorView",editorOptions:{onContentReady:function(e){(0,w.default)(e.element).closest(".".concat(z)).css("flexBasis","auto")},focusStateEnabled:!1},label:{visible:!1}}]});o.done(function(n,i){t._applyFormat([e,n[e],ae],i)}),o.fail(function(){t.quill.focus()})}}},{key:"_getToolbarItem",value:function(e){var t=this,n={options:{onInitialized:function(n){e.formatName&&(n.component.$element().addClass(V),n.component.$element().toggleClass("dx-".concat(e.formatName.toLowerCase(),"-format"),!!e.formatName),t._toolbarWidgets.add(e.formatName,n.component))}}},i=this.isMultilineMode()?{location:"before",locateInMenu:"never"}:{};return(0,I.extend)(!0,{location:"before",locateInMenu:"auto"},this._getDefaultConfig(e.formatName),e,n,i)}},{key:"_getDefaultItemsConfig",value:function(){return{header:{options:{displayExpr:function(e){var t=(0,D.isDefined)(e)&&e!==!1;return t?"".concat(se," ").concat(e):le}}},clear:{options:{disabled:!0}},undo:{options:{disabled:!0}},redo:{options:{disabled:!0}},insertRowAbove:{options:{disabled:!0}},insertRowBelow:{options:{disabled:!0}},insertColumnLeft:{options:{disabled:!0}},insertColumnRight:{options:{disabled:!0}},deleteRow:{options:{disabled:!0}},deleteColumn:{options:{disabled:!0}},deleteTable:{options:{disabled:!0}},separator:{template:function(e,t,n){(0,w.default)(n).addClass(F)},menuItemTemplate:function(e,t,n){(0,w.default)(n).addClass(L)}}}}},{key:"_getDefaultConfig",value:function(e){return this._getDefaultItemsConfig()[e]}},{key:"updateHistoryWidgets",value:function(){var e=this.quill.history;if(e){var t=e.stack,n=t.undo,i=t.redo;this._updateManipulationWidget(this._toolbarWidgets.getByName("undo"),Boolean(n.length)),this._updateManipulationWidget(this._toolbarWidgets.getByName("redo"),Boolean(i.length))}}},{key:"updateTableWidgets",value:function(){var e,t=this,n=this.quill.getModule("table");if(n){var i=this.quill.getSelection(),o=i&&Boolean(null===(e=this.quill.getFormat(i))||void 0===e?void 0:e.table);oe.forEach(function(e){var n="insertTable"===e,i=t._toolbarWidgets.getByName(e);t._updateManipulationWidget(i,n?!o:o)})}}},{key:"_updateManipulationWidget",value:function(e,t){e&&e.option("disabled",!t)}},{key:"updateFormatWidgets",value:function(e){var t=this.quill.getSelection();if(t){var n=this.quill.getFormat(t),i=!(0,D.isEmptyObject)(n);i&&!e||this._resetFormatWidgets();for(var o in n){var a=this._getFormatWidgetName(o,n),r=this._toolbarWidgets.getByName(a)||this._toolbarWidgets.getByName(o);r&&this._markActiveFormatWidget(o,r,n)}this._toggleClearFormatting(i||t.length>1)}}},{key:"_markActiveFormatWidget",value:function(e,t,n){this._isColorFormat(e)&&this._updateColorWidget(e,n[e]),"value"in t.option()?this._setValueSilent(t,n[e]):t.$element().addClass(H)}},{key:"_toggleClearFormatting",value:function(e){var t=this._toolbarWidgets.getByName("clear");t&&t.option("disabled",!e)}},{key:"_isColorFormat",value:function(e){return"color"===e||"background"===e}},{key:"_updateColorWidget",value:function(e,t){var n=this._toolbarWidgets.getByName(e);n&&n.$element().find(".".concat(N)).css("borderBottomColor",t||"transparent")}},{key:"_getFormatWidgetName",value:function(e,t){var n;switch(e){case"align":n=e+(0,O.titleize)(t[e]);break;case"list":n=t[e]+(0,O.titleize)(e);break;case"code-block":n="codeBlock";break;case"script":n=t[e]+e;break;case"imageSrc":n="image";break;default:n=e}return n}},{key:"_setValueSilent",value:function(e,t){this._isReset=!0,e.option("value",t),this._isReset=!1}},{key:"_resetFormatWidgets",value:function(){var e=this;this._toolbarWidgets.each(function(t,n){e._resetFormatWidget(t,n)})}},{key:"_resetFormatWidget",value:function(e,t){t.$element().removeClass(H),this._isColorFormat(e)&&this._updateColorWidget(e),"clear"===e&&t.option("disabled",!0),"dxSelectBox"===t.NAME&&this._setValueSilent(t,null)}},{key:"addClickHandler",value:function(e,t){this._formatHandlers[e]=t;var n=this._toolbarWidgets.getByName(e);n&&"dxButton"===n.NAME&&n.option("onClick",t)}},{key:"_insertTableFormItems",get:function(){return[{dataField:"columns",editorType:"dxNumberBox",editorOptions:{min:1},label:{text:T.default.format(te)}},{dataField:"rows",editorType:"dxNumberBox",editorOptions:{min:1},label:{text:T.default.format(ne)}}]}},{key:"_embedFormatIndex",get:function(){var e=this.quill.getSelection();return e?e.length?e.index:e.index-1:this.quill.getLength()}},{key:"_defaultPasteIndex",get:function(){var e=this.quill.getSelection();return(0,D.isDefined)(null===e||void 0===e?void 0:e.index)?e.index:this.quill.getLength()}},{key:"_imageFormItems",get:function(){return[{dataField:"src",label:{text:T.default.format(X)}},{dataField:"width",label:{text:T.default.format(J)}},{dataField:"height",label:{text:T.default.format(ee)}},{dataField:"alt",label:{text:T.default.format(Q)}}]}},{key:"toolbarConfig",get:function(){return{dataSource:this._prepareToolbarItems(),disabled:this.isInteractionDisabled,menuContainer:this._$toolbarContainer,multiline:this.isMultilineMode()}}},{key:"isInteractionDisabled",get:function(){return this._editorInstance.option("readOnly")||this._editorInstance.option("disabled")}}]),t}(R)}var ue=P;t.default=ue,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(446)),a=n(0),r=i(n(281)),s=o.default.inherit({NAME:"actionSheet",_getMenuItemTemplate:function(){return this._toolbar._getTemplate("actionSheetItem")},render:function(){this._hasVisibleMenuItems()&&this.callBase()},_menuWidgetClass:function(){return r.default},_menuContainer:function(){return this._toolbar.$element()},_widgetOptions:function(){return(0,a.extend)({},this.callBase(),{target:this._$button,showTitle:!1})},_menuButtonOptions:function(){return(0,a.extend)({},this.callBase(),{icon:"overflow"})},_toggleMenu:function(){this.callBase.apply(this,arguments),this._menu.toggle(this._menuShown),this._menuShown=!1}}),l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(446)),r=i(n(658)),s=i(n(219)),l="dx-state-invisible",u=a.default.inherit({NAME:"dropDownMenu",render:function(){this._hasVisibleMenuItems()&&(this._renderMenuButtonContainer(),this._renderWidget())},renderMenuItems:function(){this._menu||this.render(),this.callBase(),this._menu&&!this._menu.option("items").length&&this._menu.close()},_menuWidgetClass:function(){return s.default},_widgetOptions:function(){var e=this;return(0,o.extend)(this.callBase(),{deferRendering:!0,container:e._toolbar.option("menuContainer"),menuWidget:r.default,onOptionChanged:function(t){"items"===t.name&&e._updateMenuVisibility(t.value)},popupPosition:{at:"bottom right",my:"top right"}})},_updateMenuVisibility:function(e){var t=e||this._getMenuItems(),n=t.length&&this._hasVisibleMenuItems(t);this._toggleMenuVisibility(n)},_toggleMenuVisibility:function(e){this._menuContainer()&&this._menuContainer().toggleClass(l,!e)},_menuContainer:function(){return this._$menuButtonContainer}}),c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(9)),r=n(3),s=n(283),l="dx-toolbar-menu-action",u="dx-toolbar-hidden-button",c="dx-toolbar-hidden-button-group",d="dx-toolbar-menu-section",h="dx-toolbar-menu-last-section",f=s.ListBase.inherit({_activeStateUnit:"."+l,_initMarkup:function(){this._renderSections(),this.callBase()},_getSections:function(){return this._itemContainer().children()},_itemElements:function(){return this._getSections().children(this._itemSelector())},_renderSections:function(){var e=this,t=this._itemContainer();(0,r.each)(["before","center","after","menu"],function(){var n="_$"+this+"Section",i=e[n];i||(e[n]=i=(0,o.default)("<div>").addClass(d)),i.appendTo(t)})},_renderItems:function(){this.callBase.apply(this,arguments),this._updateSections()},_updateSections:function(){var e=this.$element().find("."+d);e.removeClass(h),e.not(":empty").eq(-1).addClass(h)},_renderItem:function(e,t,n,i){var o=t.location||"menu",a=this["_$"+o+"Section"],r=this.callBase(e,t,a,i);return this._getItemTemplateName({itemData:t})&&r.addClass("dx-toolbar-menu-custom"),("menu"===o||"dxButton"===t.widget||t.isAction)&&r.addClass(l),"dxButton"===t.widget&&r.addClass(u),"dxButtonGroup"===t.widget&&r.addClass(c),r.addClass(t.cssClass),r},_getItemTemplateName:function(e){var t=this.callBase(e),n=e.itemData,i=n&&n.menuItemTemplate;return i||t},_itemClickHandler:function(e,t,n){(0,o.default)(e.target).closest("."+l).length&&this.callBase(e,t,n)},_clean:function(){this._getSections().empty(),this.callBase()}});(0,a.default)("dxToolbarMenu",f);var p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.default=void 0;var r=n(3),s=function(){function e(){i(this,e),this._collection=[]}return a(e,[{key:"clear",value:function(){this._collection=[]}},{key:"add",value:function(e,t){this._collection.push({name:e,instance:t})}},{key:"getByName",value:function(e){var t=null;return(0,r.each)(this._collection,function(n,i){var o=i.name,a=i.instance;if(o===e)return t=a,!1}),t}},{key:"each",value:function(e){this._collection.forEach(function(t){var n=t.name,i=t.instance;return i&&e(n,i)})}}]),e}();t.default=s,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(63)),_=o(n(5)),m=n(8),v=n(3),y=o(n(23)),b=n(7),x={};if(g.default){var w=g.default.import("core/module");x=function(e){function t(e,i){var o;a(this,t),o=n.call(this,e,i),o.editorInstance=i.editorInstance;var r=o.editorInstance.NAME;return _.default.on(o.quill.root,(0,m.addNamespace)("dragover",r),o._dragOverHandler.bind(h(o))),_.default.on(o.quill.root,(0,m.addNamespace)("drop",r),o._dropHandler.bind(h(o))),_.default.on(o.quill.root,(0,m.addNamespace)("paste",r),o._pasteHandler.bind(h(o))),o}l(t,e);var n=c(t);return s(t,[{key:"_dragOverHandler",value:function(e){y.default.msie&&e.preventDefault()}},{key:"_dropHandler",value:function(e){var t,n=e.originalEvent.dataTransfer,i=null===n||void 0===n?void 0:null===(t=n.files)||void 0===t?void 0:t.length;this.editorInstance._saveValueChangeEvent(e),e.preventDefault(),i&&this._getImage(n.files,this._addImage.bind(this))}},{key:"_pasteHandler",value:function(e){var t,n=this,i=e.originalEvent.clipboardData;if(this.editorInstance._saveValueChangeEvent(e),i){var o=null===(t=i.items)||void 0===t?void 0:t.length,a=i.getData("text/html");!a&&o&&this._getImage(i.items,function(e){n._isBrowserSupportImagePaste(y.default)||(y.default.msie?setTimeout(function(){n._addImage(e)}):n._addImage(e))})}}},{key:"_isBrowserSupportImagePaste",value:function(e){var t=e.mozilla,n=e.chrome,i=e.version;return t||n&&i>82}},{key:"_isImage",value:function(e){return!!e.type.match(/^image\/(a?png|bmp|gif|p?jpe?g|svg|vnd\.microsoft\.icon|webp)/i)}},{key:"_getImage",value:function(e,t){var n=this,i=(0,b.getWindow)();(0,v.each)(e,function(e,o){if(n._isImage(o)){var a=new i.FileReader;a.onload=function(e){var n=e.target;t(n.result)};var r=o.getAsFile?o.getAsFile():o;r instanceof i.Blob&&a.readAsDataURL(r)}})}},{key:"_addImage",value:function(e){var t=this.quill.getSelection(),n=t?t.index:this.quill.getLength();this.quill.insertEmbed(n,"extendedImage",e,"user")}}]),t}(w)}var C=x;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(63)),v=o(n(2)),y=n(24),b=o(n(447)),x=o(n(662)),w=n(0),C={};if(m.default){var k="dx-variable-format",S="dx-format-active";m.default.register({"formats/variable":x.default},!0),C=function(e){function t(e,i){var o;a(this,t),o=n.call(this,e,i);var r=e.getModule("toolbar");return r&&r.addClickHandler("variable",o.showPopup.bind(h(o))),e.keyboard.addBinding({key:"P",altKey:!0},o.showPopup.bind(h(o))),o._popup.on("shown",function(e){
var t=(0,v.default)(e.component.option("position").of);t.hasClass(k)&&t.addClass(S)}),o}l(t,e);var n=c(t);return s(t,[{key:"_getDefaultOptions",value:function(){var e=p(_(t.prototype),"_getDefaultOptions",this).call(this);return(0,w.extend)(e,{escapeChar:""})}}]),s(t,[{key:"showPopup",value:function(e){var n=this.quill.getSelection(),i=n?n.index:this.quill.getLength();this.savePosition(i),this._resetPopupPosition(e,i),p(_(t.prototype),"showPopup",this).call(this)}},{key:"_resetPopupPosition",value:function(e,t){if(e&&e.element)this._popup.option("position",{of:e.element,offset:{h:0,v:0},my:"top center",at:"bottom center",collision:"fit"});else{var n=this.quill.getBounds(t),i=(0,y.getBoundingRect)(this.quill.root);this._popup.option("position",{of:this.quill.root,offset:{h:n.left,v:n.bottom-i.height},my:"top center",at:"bottom left",collision:"fit flip"})}}},{key:"insertEmbedContent",value:function(e){var t=this.getPosition(),n=e.component.option("selectedItem"),i=(0,w.extend)({},{value:n,escapeChar:this.options.escapeChar});setTimeout(function(){this.quill.insertEmbed(t,"variable",i),this.quill.setSelection(t+1)}.bind(this))}}]),t}(b.default)}var D=C;t.default=D,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(63)),v=n(4),y=n(0),b={};if(m.default){var x=m.default.import("blots/embed"),w="dx-variable";b=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,null,[{key:"create",value:function(e){var n,i,o=l(_(t),"create",this).call(this),a=e.value;return Array.isArray(e.escapeChar)?(n=(0,v.ensureDefined)(e.escapeChar[0],""),i=(0,v.ensureDefined)(e.escapeChar[1],"")):n=i=e.escapeChar,o.innerText=n+a+i,o.dataset.varStartEscChar=n,o.dataset.varEndEscChar=i,o.dataset.varValue=e.value,o}},{key:"value",value:function(e){return(0,y.extend)({},{value:e.dataset.varValue,escapeChar:[e.dataset.varStartEscChar||"",e.dataset.varEndEscChar||""]})}}]),t}(x),b.blotName="variable",b.tagName="span",b.className=w}var C=b;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=i(n(5)),u=n(21),c=n(8),d=n(30),h=i(n(13)),f=i(n(188)),p=n(24),g=i(n(63)),_="dx-resize-frame",m="dx-touch-device",v="dxHtmlResizingModule",y=(0,c.addNamespace)("keydown",v),b=(0,c.addNamespace)("scroll",v),x=(0,c.addNamespace)("mousedown",v),w=1,C=function(){function e(t,n){o(this,e),this.quill=t,this.editorInstance=n.editorInstance,this.allowedTargets=n.allowedTargets||["image"],this.enabled=!!n.enabled,this.enabled&&(this._attachEvents(),this._createResizeFrame())}return r(e,[{key:"_attachEvents",value:function(){l.default.on(this.quill.root,(0,c.addNamespace)(u.name,v),this._clickHandler.bind(this)),l.default.on(this.quill.root,b,this._scrollHandler.bind(this))}},{key:"_detachEvents",value:function(){l.default.off(this.quill.root,v)}},{key:"_clickHandler",value:function(e){if(this._isAllowedTarget(e.target)){if(this._$target===e.target)return;this._$target=e.target,this.updateFramePosition(),this.showFrame()}else this._$target&&this.hideFrame()}},{key:"_scrollHandler",value:function(e){this._$target&&this.updateFramePosition()}},{key:"_isAllowedTarget",value:function(e){return this._isImage(e)}},{key:"_isImage",value:function(e){return this.allowedTargets.indexOf("image")!==-1&&"IMG"===e.tagName.toUpperCase()}},{key:"showFrame",value:function(){this._$resizeFrame.show(),l.default.on(this.quill.root,y,this._handleFrameKeyDown.bind(this))}},{key:"_handleFrameKeyDown",value:function(e){var t=(0,c.normalizeKeyName)(e);"del"!==t&&"backspace"!==t||this._deleteImage(),this.hideFrame()}},{key:"hideFrame",value:function(){this._$target=null,this._$resizeFrame.hide(),l.default.off(this.quill.root,y)}},{key:"updateFramePosition",value:function(){var e=(0,p.getBoundingRect)(this._$target),t=e.height,n=e.width,i=e.top,o=e.left,a=(0,p.getBoundingRect)(this.quill.root),r=a.top,s=a.left,l=this._getBorderWidth();this._$resizeFrame.css({height:t,width:n,padding:w,top:i-r-l-w,left:o-s-l-w}),(0,d.move)(this._$resizeFrame,{left:0,top:0})}},{key:"_getBorderWidth",value:function(){return parseInt(this._$resizeFrame.css("borderTopWidth"))}},{key:"_createResizeFrame",value:function(){var e=this;if(!this._$resizeFrame){var t=h.default.current(),n=t.deviceType;this._$resizeFrame=(0,s.default)("<div>").addClass(_).toggleClass(m,"desktop"!==n).appendTo(this.editorInstance._getQuillContainer()).hide(),l.default.on(this._$resizeFrame,x,function(e){e.preventDefault()}),this.editorInstance._createComponent(this._$resizeFrame,f.default,{onResize:function(t){if(e._$target){var n=2*(w+e._getBorderWidth());(0,s.default)(e._$target).attr({height:t.height-n,width:t.width-n}),e.updateFramePosition()}}})}}},{key:"_deleteImage",value:function(){this._isAllowedTarget(this._$target)&&g.default.find(this._$target).deleteAt(0)}},{key:"option",value:function(e,t){var n=this;return"mediaResizing"===e?void Object.keys(t).forEach(function(e){return n.option(e,t[e])}):void("enabled"===e?(this.enabled=t,t?this._attachEvents():this._detachEvents()):"allowedTargets"===e&&Array.isArray(t)&&(this.allowedTargets=t))}},{key:"clean",value:function(){this._detachEvents(),this._$resizeFrame.remove(),this._$resizeFrame=void 0}}]),e}();t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(63)),y=n(19),b=n(1),x=n(0),w=n(18),C=o(n(5)),k=o(n(447)),S=o(n(665)),D={};if(v.default){var I="user",T="silent",O="@",E={ARROW_UP:"upArrow",ARROW_DOWN:"downArrow",ARROW_LEFT:"leftArrow",ARROW_RIGHT:"rightArrow",ENTER:"enter",ESCAPE:"escape",SPACE:"space",PAGE_UP:"pageUp",PAGE_DOWN:"pageDown",END:"end",HOME:"home"},A=[E.ARROW_LEFT,E.ARROW_RIGHT,E.PAGE_UP,E.PAGE_DOWN,E.END,E.HOME],P=[" ","\n"],R="dx-state-disabled";v.default.register({"formats/mention":S.default},!0),D=function(e){function t(e,i){var o;return a(this,t),o=n.call(this,e,i),o._mentions={},o.editorInstance=i.editorInstance,i.mentions.forEach(function(e){var t=e.marker;t||(e.marker=t=O);var n=e.template;if(n){var i=o.editorInstance._getTemplate(n);i&&S.default.addTemplate(t,i)}o._mentions[t]=(0,x.extend)({},o._getDefaultOptions(),e)}),o._attachKeyboardHandlers(),o.editorInstance.addCleanCallback(o.clean.bind(h(o))),o.quill.on("text-change",o.onTextChange.bind(h(o))),o}l(t,e);var n=c(t);return s(t,[{key:"_getDefaultOptions",value:function(){var e=p(_(t.prototype),"_getDefaultOptions",this).call(this);return(0,x.extend)(e,{itemTemplate:"item",valueExpr:"this",displayExpr:"this",template:null,searchExpr:null,searchTimeout:500,minSearchLength:0})}}]),s(t,[{key:"_attachKeyboardHandlers",value:function(){this.quill.keyboard.addBinding({key:E.ARROW_UP},this._moveToItem.bind(this,"prev")),this.quill.keyboard.addBinding({key:E.ARROW_DOWN},this._moveToItem.bind(this,"next")),this.quill.keyboard.addBinding({key:[E.ENTER,E.SPACE]},this._selectItemHandler.bind(this));var e=this.quill.keyboard.bindings[E.ENTER];e.unshift(e.pop()),this.quill.keyboard.addBinding({key:E.ESCAPE},this._escapeKeyHandler.bind(this)),this.quill.keyboard.addBinding({key:[E.ARROW_LEFT,E.ARROW_RIGHT],shiftKey:!0},this._ignoreKeyHandler.bind(this)),this.quill.keyboard.addBinding({key:A},this._ignoreKeyHandler.bind(this))}},{key:"_moveToItem",value:function(e){var t=this._list.getDataSource();if(this._isMentionActive&&!t.isLoading()){var n=(0,m.default)(this._list.option("focusedElement")),i="next"===e?"first":"last",o=n[e]();o=o.length?o:this._activeListItems[i](),this._list.option("focusedElement",(0,w.getPublicElement)(o)),this._list.scrollToItem(o)}return!this._isMentionActive}},{key:"_ignoreKeyHandler",value:function(){return!this._isMentionActive}},{key:"_fitIntoRange",value:function(e,t,n){return e>n?t:e<t?n:e}},{key:"_selectItemHandler",value:function(){return this._isMentionActive&&this._list.selectItem(this._list.option("focusedElement")),!this._isMentionActive}},{key:"_escapeKeyHandler",value:function(){return this._isMentionActive&&this._popup.hide(),!this._isMentionActive}},{key:"renderList",value:function(e,n){this.compileGetters(this.options),p(_(t.prototype),"renderList",this).call(this,e,n)}},{key:"compileGetters",value:function(e){var t=e.displayExpr,n=e.valueExpr;this._valueGetter=(0,y.compileGetter)(t),this._idGetter=(0,y.compileGetter)(n)}},{key:"_getListConfig",value:function(e){var n=this,i=p(_(t.prototype),"_getListConfig",this).call(this,e);return(0,x.extend)(i,{itemTemplate:this.options.itemTemplate,onContentReady:function(){n._hasSearch&&(n._popup.repaint(),n._focusFirstElement(),n._hasSearch=!1)}})}},{key:"insertEmbedContent",value:function(){var e=this._activeMentionConfig.marker.length,t=e+this._searchValue.length,n=this.getPosition(),i=Math.max(0,n-e),o=this._list.option("selectedItem"),a={value:this._valueGetter(o),id:this._idGetter(o),marker:this._activeMentionConfig.marker};setTimeout(function(){this.quill.insertText(i," ",T),this.quill.deleteText(i+1,t,T),this.quill.insertEmbed(i,"mention",a),this.quill.setSelection(i+2)}.bind(this))}},{key:"_getLastInsertOperation",value:function(e){var t=e[e.length-1],n="insert"in t;if(n)return t;var i="delete"in t;if(i&&e.length>=2){var o=e[e.length-2],a="insert"in o,r=i&&a;if(r)return o}return null}},{key:"onTextChange",value:function(e,t,n){if(n===I){var i=e.ops[e.ops.length-1];if(this._isMentionActive&&this._isPopupVisible)this._processSearchValue(i)&&this._filterList(this._searchValue);else{var o=e.ops,a=this._getLastInsertOperation(o);a&&this.checkMentionRequest(a,o)}}}},{key:"_processSearchValue",value:function(e){var t="insert"in e;if(t)this._searchValue+=e.insert;else{if(!this._searchValue.length||e.delete>1)return this._popup.hide(),!1;this._searchValue=this._searchValue.slice(0,-1)}return!0}},{key:"checkMentionRequest",value:function(e,t){var n=e.insert,i=this.quill.getSelection();n&&(0,b.isString)(n)&&i&&!this._isMarkerPartOfText(t[0].retain)&&(this._activeMentionConfig=this._mentions[n],this._activeMentionConfig&&(this._updateList(this._activeMentionConfig),this.savePosition(i.index),this._popup.option("position",this._popupPosition),this._searchValue="",this._popup.show()))}},{key:"_isMarkerPartOfText",value:function(e){return!(!e||P.indexOf(this._getCharByIndex(e-1))!==-1)}},{key:"_getCharByIndex",value:function(e){return this.quill.getContents(e,1).ops[0].insert}},{key:"_updateList",value:function(e){var t=e.dataSource,n=e.displayExpr,i=e.valueExpr,o=e.itemTemplate,a=e.searchExpr;this.compileGetters({displayExpr:n,valueExpr:i}),this._list.unselectAll(),this._list.option({dataSource:t,displayExpr:n,itemTemplate:o,searchExpr:a})}},{key:"_filterList",value:function(e){var t=this;if(!this._isMinSearchLengthExceeded(e))return void this._resetFilter();var n=this._activeMentionConfig.searchTimeout;n?(clearTimeout(this._searchTimer),this._searchTimer=setTimeout(function(){t._search(e)},n)):this._search(e)}},{key:"_isMinSearchLengthExceeded",value:function(e){return e.length>=this._activeMentionConfig.minSearchLength}},{key:"_resetFilter",value:function(){clearTimeout(this._searchTimer),this._search(null)}},{key:"_search",value:function(e){this._hasSearch=!0,this._list.option("searchValue",e)}},{key:"_focusFirstElement",value:function(){if(this._list){var e=this._activeListItems.first();this._list.option("focusedElement",(0,w.getPublicElement)(e)),this._list.scrollToItem(e)}}},{key:"_getPopupConfig",value:function(){var e=this;return(0,x.extend)(p(_(t.prototype),"_getPopupConfig",this).call(this),{closeOnTargetScroll:!1,onShown:function(){e._isMentionActive=!0,e._hasSearch=!1,e._focusFirstElement()},onHidden:function(){e._list.unselectAll(),e._list.option("focusedElement",null),e._isMentionActive=!1,e._search(null)},focusStateEnabled:!1})}},{key:"clean",value:function(){var e=this;Object.keys(this._mentions).forEach(function(t){e._mentions[t].template&&S.default.removeTemplate(t)})}},{key:"_isPopupVisible",get:function(){var e;return null===(e=this._popup)||void 0===e?void 0:e.option("visible")}},{key:"_popupPosition",get:function(){var e=this.getPosition(),t=this.quill.getBounds(e?e-1:e),n=t.left,i=t.top,o=t.height,a=(0,m.default)(this.quill.root).offset(),r=a.left,s=a.top,l=C.default.Event("positionEvent",{pageX:r+n,pageY:s+i});return{of:l,offset:{v:o},my:"top left",at:"top left",collision:{y:"flip",x:"flipfit"}}}},{key:"_activeListItems",get:function(){return this._list.itemElements().filter(":not(.".concat(R,")"))}}]),t}(k.default)}var M=D;t.default=M,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(63)),v=o(n(2)),y={};if(m.default){var b=m.default.import("blots/embed"),x="dx-mention";y=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,null,[{key:"create",value:function(e){var n=l(_(t),"create",this).call(this);return n.setAttribute("spellcheck",!1),n.dataset.marker=e.marker,n.dataset.mentionValue=e.value,n.dataset.id=e.id,this.renderContent(n,e),n}},{key:"value",value:function(e){return{marker:e.dataset.marker,id:e.dataset.id,value:e.dataset.mentionValue}}},{key:"renderContent",value:function(e,t){var n=this._templates.get(t.marker);n?n.render({model:t,container:e}):this.baseContentRender(e,t)}},{key:"baseContentRender",value:function(e,t){var n=(0,v.default)("<span>").text(t.marker);(0,v.default)(e).append(n).append(t.value)}},{key:"addTemplate",value:function(e,t){this._templates.set(e,t)}},{key:"removeTemplate",value:function(e){this._templates.delete(e)}}]),t}(b),y.blotName="mention",y.tagName="span",y.className=x,y._templates=new Map}var w=y;t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(300)),l=n(445),u=function(){function e(){o(this,e),this.TextBlot=(0,l.getQuill)().import("blots/text"),this.BreakBlot=(0,l.getQuill)().import("blots/break")}return r(e,[{key:"setQuillInstance",value:function(e){this.quillInstance=e}},{key:"toHtml",value:function(){if(this.quillInstance)return this._isQuillEmpty()?"":this.quillInstance.getSemanticHTML(0,this.quillInstance.getLength()+1)}},{key:"_isQuillEmpty",value:function(){var e=this.quillInstance.getContents();return 1===e.length()&&this._isDeltaEmpty(e)}},{key:"_isDeltaEmpty",value:function(e){return e.reduce(function(e,t){var n=t.insert;return n.indexOf("\n")!==-1})}}]),e}();s.default.addConverter("delta",u);var c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){var t=e[1];return t.match(/\S+\./)?"ordered":"bullet"}function o(e){var t=e.getAttribute("style");if(t){var n=t.replace(/\n+/g,"").match(/level(\d+)/);return n?n[1]-1:0}return!1}function a(e){var t=e[e.length-1];t.insert=t.insert.trim()}t.default=void 0;var r=function(e){var t=e.import("delta");return function(e,n){var r=n.ops.slice(),s=r[0];s.insert=s.insert.replace(/^\s+/,"");var l=s.insert.match(/^(\S+)\s+/),u=l&&o(e);return l&&u!==!1?(s.insert=s.insert.substring(l[0].length,s.insert.length),a(r),r.push({insert:"\n",attributes:{list:i(l),indent:u}}),new t(r)):n}},s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=n(0),u=i(n(48)),c=i(n(146)),d=i(n(11)),h=n(33),f=n(6),p=i(n(10)),g=i(n(23)),_=d.default.getActiveElement,m="dx-formdialog",v="dx-formdialog-form",y=function(){function e(t,n){o(this,e),this._editorInstance=t,this._popupUserConfig=n,this._renderPopup()}return r(e,[{key:"_renderPopup",value:function(){var e=this._editorInstance,t=(0,s.default)("<div>").addClass(m).appendTo(e.$element()),n=this._getPopupConfig();return e._createComponent(t,u.default,n)}},{key:"_escKeyHandler",value:function(){this._popup.hide()}},{key:"_addEscapeHandler",value:function(e){e.component.registerKeyHandler("escape",this._escKeyHandler.bind(this))}},{key:"_getPopupConfig",value:function(){var e=this;return(0,l.extend)({onInitialized:function(t){e._popup=t.component,e._popup.on("hiding",function(){e.deferred.reject()}),e._popup.on("shown",function(){e._form.focus()})},deferRendering:!1,focusStateEnabled:!1,showCloseButton:!1,contentTemplate:function(t){var n=(0,s.default)("<div>").appendTo(t);e._renderForm(n,{onEditorEnterKey:function(t){var n=t.component,i=t.dataField,o=t.event;e._updateEditorValue(n,i),e.hide(n.option("formData"),o)},customizeItem:function(t){"simple"===t.itemType&&(t.editorOptions=(0,l.extend)(!0,{},t.editorOptions,{onInitialized:e._addEscapeHandler.bind(e)}))}})},toolbarItems:[{toolbar:"bottom",location:"after",widget:"dxButton",options:{onInitialized:this._addEscapeHandler.bind(this),text:p.default.format("OK"),onClick:function(t){var n=t.event;e.hide(e._form.option("formData"),n)}}},{toolbar:"bottom",location:"after",widget:"dxButton",options:{onInitialized:this._addEscapeHandler.bind(this),text:p.default.format("Cancel"),onClick:function(){e._popup.hide()}}}]},this._popupUserConfig)}},{key:"_updateEditorValue",value:function(e,t){if(g.default.msie&&parseInt(g.default.version)<=11){var n=e.getEditor(t),i=_();n.$element().find(i).length&&(0,h.resetActiveElement)()}}},{key:"_renderForm",value:function(e,t){e.addClass(v),this._form=this._editorInstance._createComponent(e,c.default,t)}},{key:"show",value:function(e){if(!this._popup.option("visible")){this.deferred=new f.Deferred;var t=(0,l.extend)({},e);return this._form.option(t),this._popup.show(),this.deferred.promise()}}},{key:"hide",value:function(e,t){this.deferred.resolve(e,t),this._popup.hide()}},{key:"popupOption",value:function(e,t){return this._popup.option.apply(this._popup,arguments)}}]),e}(),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(7),s=n(46),l=n(4),u=n(18),c=n(3),d=n(0),h=n(87),f=i(n(10)),p=i(n(13)),g=i(n(9)),_=n(8),m=i(n(282)),v=n(36),y=n(21),b=i(n(143)),x=i(n(104)),w=n(154),C=n(30),k=n(1),S=n(213),D=(0,r.getWindow)(),I="dx-lookup",T="dx-lookup-search",O="dx-lookup-search-wrapper",E="dx-lookup-field",A="dx-lookup-arrow",P="dx-lookup-field-wrapper",R="dx-lookup-popup",M="dx-lookup-popup-wrapper",B="dx-lookup-popup-search",V="dx-lookup-popover-mode",F="dx-lookup-empty",L="dx-popover-flipped-vertical",H="dx-texteditor-input",z="dx-list-item",N="dx-list-item-selected",j="dx-list-group-header",W=5,G=8,q=m.default.inherit({_supportedKeys:function(){return(0,d.extend)(this.callBase(),{space:function(e){e.preventDefault(),this._validatedOpening()},enter:function(){this._validatedOpening()}})},_getDefaultOptions:function(){return(0,d.extend)(this.callBase(),{placeholder:f.default.format("Select"),searchPlaceholder:f.default.format("Search"),searchEnabled:!0,cleanSearchOnOpening:!0,showCancelButton:!0,showClearButton:!1,clearButtonText:f.default.format("Clear"),applyButtonText:f.default.format("OK"),pullRefreshEnabled:!1,useNativeScrolling:!0,pullingDownText:f.default.format("dxList-pullingDownText"),pulledDownText:f.default.format("dxList-pulledDownText"),refreshingText:f.default.format("dxList-refreshingText"),pageLoadingText:f.default.format("dxList-pageLoadingText"),onScroll:null,onPullRefresh:null,onPageLoading:null,pageLoadMode:"scrollBottom",nextButtonText:f.default.format("dxList-nextButtonText"),grouped:!1,groupTemplate:"group",usePopover:!1,showDropDownButton:!1,focusStateEnabled:!1,animation:{},dropDownOptions:{showTitle:!0,width:function(){return.8*(0,o.default)(D).width()},height:function(){return.8*(0,o.default)(D).height()},shading:!0,closeOnOutsideClick:!1,position:void 0,animation:{},title:"",titleTemplate:"title",onTitleRendered:null,fullScreen:!1},dropDownCentered:!1,_scrollToSelectedItemEnabled:!1,useHiddenSubmitElement:!0})},_defaultOptionsRules:function(){var e=this,t=(0,v.current)();return this.callBase().concat([{device:function(){return!s.nativeScrolling},options:{useNativeScrolling:!1}},{device:function(e){return!p.default.isSimulator()&&"desktop"===p.default.real().deviceType&&"generic"===e.platform},options:{usePopover:!0,dropDownOptions:{height:"auto"}}},{device:{platform:"ios",phone:!0},options:{dropDownOptions:{fullScreen:!0}}},{device:{platform:"ios",tablet:!0},options:{dropDownOptions:{width:function(){return.4*Math.min((0,o.default)(D).width(),(0,o.default)(D).height())},height:"auto"},usePopover:!0,useInkRipple:!1}},{device:function(){return"desktop"===p.default.real().deviceType&&!p.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return(0,v.isMaterial)(t)},options:{usePopover:!1,searchEnabled:!1,showCancelButton:!1,dropDownCentered:!0,_scrollToSelectedItemEnabled:!0,dropDownOptions:{closeOnOutsideClick:!0,width:function(){return(0,S.getElementWidth)(e.$element())},height:function(){return this._getPopupHeight()}.bind(this),showTitle:!1,shading:!1}}}])},_init:function(){this.callBase(),this._initActions()},_initActions:function(){this.callBase(),this._initScrollAction(),this._initPageLoadingAction(),this._initPullRefreshAction()},_initPageLoadingAction:function(){this._pageLoadingAction=this._createActionByOption("onPageLoading")},_initPullRefreshAction:function(){this._pullRefreshAction=this._createActionByOption("onPullRefresh")},_initScrollAction:function(){this._scrollAction=this._createActionByOption("onScroll")},_scrollHandler:function(e){this._scrollAction(e)},_pullRefreshHandler:function(e){this._pullRefreshAction(e)},_pageLoadingHandler:function(e){this._pageLoadingAction(e)},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({group:new w.ChildDefaultTemplate("group"),title:new w.ChildDefaultTemplate("title")})},_initMarkup:function(){this.$element().addClass(I).toggleClass(V,this.option("usePopover")),this.callBase()},_inputWrapper:function(){return this.$element().find("."+P)},_dataSourceOptions:function(){return(0,d.extend)(this.callBase(),{paginate:!0})},_fireContentReadyAction:l.noop,_popupWrapperClass:function(){return""},_renderInput:function(){var e=this,t=this._createAction(function(){e._toggleOpenState()});this._$field=(0,o.default)("<div>").addClass(E),a.default.on(this._$field,(0,_.addNamespace)(y.name,this.NAME),function(e){t({event:e})});var n=(0,o.default)("<div>").addClass(A);this._$fieldWrapper=(0,o.default)("<div>").addClass(P).append(this._$field).append(n).appendTo(this.$element()),this.option("useInkRipple")&&this._renderInkRipple()},_getInputContainer:function(){return this._$fieldWrapper},_renderInkRipple:function(){this._inkRipple=(0,h.render)()},_toggleOpenState:function(){this.callBase(),!this.option("dropDownOptions.fullScreen")&&this.option("_scrollToSelectedItemEnabled")&&this._setPopupPosition()},_toggleActiveState:function(e,t,n){if(this.callBase.apply(this,arguments),this._inkRipple){var i={element:this._inputWrapper(),event:n};t?this._inkRipple.showWave(i):this._inkRipple.hideWave(i)}},_renderField:function(){var e=this._getTemplateByOption("fieldTemplate");if(e&&this.option("fieldTemplate"))return void this._renderFieldTemplate(e);var t=this.option("displayValue");this._updateField((0,k.isDefined)(t)&&String(t)||this.option("placeholder")),this.$element().toggleClass(F,!this.option("selectedItem"))},_renderDisplayText:function(e){this._input().length?this.callBase(e):this._updateField(e)},_updateField:function(e){this._$field.text(e)},_renderFieldTemplate:function(e){this._$field.empty();var t=this._fieldRenderData();e.render({model:t,container:(0,u.getPublicElement)(this._$field)})},_fieldRenderData:function(){return this.option("selectedItem")},_popupShowingHandler:function(){this.callBase.apply(this,arguments),this.option("cleanSearchOnOpening")&&(this.option("searchEnabled")&&this._searchBox.option("value")&&(this._searchBox.option("value",""),this._searchCanceled()),this._list&&this._list.option("focusedElement",null)),this.option("dropDownOptions.fullScreen")&&this.option("_scrollToSelectedItemEnabled")&&(this._popup.option("position").of=(0,o.default)(D))},_scrollToSelectedItem:function(){var e=this._list.option("selectedIndex"),t=this._list.option("items"),n=t.length;0!==n&&(this._list.option("grouped")?this._list.scrollToItem({group:n-1,item:t[n-1].items.length-1}):this._list.scrollToItem(n-1),this._list.scrollToItem(e))},_getDifferenceOffsets:function(e){return e.offset().top-(0,o.default)(this.element()).offset().top},_isCenteringEnabled:function(e,t){return 1<e&&e<t-2},_getPopupOffset:function(){var e=this._listItemElements().length;if(0!==e){var t=(0,o.default)(this._list.element()).find("."+N),n=this._listItemElements().index(t),i=(t.height()-(0,o.default)(this.element()).height())/2,a=(0,o.default)(this._list.element()).offset().top,r=this.option("dropDownOptions.height"),s="function"==typeof r?r():r,l=(0,o.default)(D).height(),u=0;if(n!==-1){if(this._isCenteringEnabled(n,e)){this._scrollToSelectedItem();var c=(s-t.height())/2-this._getDifferenceOffsets(t);this._list.scrollTo(this._list.scrollTop()+G/2-c),u=i+this._getDifferenceOffsets(t),a<u&&n!==e-3&&(this._list.scrollTo(this._list.scrollTop()+this._getDifferenceOffsets(t)/2),u=i+this._getDifferenceOffsets(t))}else n<=1?(this._list.scrollTo(0),u=i+this._getDifferenceOffsets(t)):n>=e-2&&(this._scrollToSelectedItem(),u=i+this._getDifferenceOffsets(t));a<u&&(this._scrollToSelectedItem(),
u=i+G)}var d=s-u-(0,o.default)(this.element()).height();return l-a<d&&(this._list.scrollTo(this._list.scrollTop()+i-d),u=s-(0,o.default)(this.element()).height()-G),u}},_setPopupPosition:function(){if(this.option("dropDownCentered")){var e=this._popup._$wrapper.hasClass(L);if(!e){var t=(0,o.default)(this._popup.content()).parent(),n=this._getPopupOffset(),i=(0,C.locate)(t);(0,C.move)(t,{top:i.top-n})}}},_listItemGroupedElements:function(){var e=this._list._itemContainer().children(),t=[];return e.each(function(e,n){t.push((0,o.default)(n).find("."+j)[0]);var i=(0,o.default)(n).find("."+z);i.each(function(e,n){t.push(n)})}),(0,o.default)(t)},_calculateListHeight:function(e){var t=e?this._listItemGroupedElements():this._listItemElements(),n=(0,o.default)("."+N),i=t.index(n),a=0,r=[];return 0===t.length?a+=G:t.length<W?t.each(function(e,t){a+=(0,o.default)(t).outerHeight()}):(r=i<=1?t.slice(0,W):this._isCenteringEnabled(i,t.length)?t.slice(i-2,i+3):t.slice(t.length-W,t.length),r.each(function(e,t){a+=(0,o.default)(t).outerHeight()})),a+(e?G:2*G)},_getPopupHeight:function(){return this._list&&this._list.itemElements()?this._calculateListHeight(this.option("grouped"))+(this._$searchWrapper?this._$searchWrapper.outerHeight():0)+(this._popup._$bottom?this._popup._$bottom.outerHeight():0)+(this._popup._$title?this._popup._$title.outerHeight():0):"auto"},_renderPopup:function(){this.option("usePopover")&&!this.option("dropDownOptions.fullScreen")?this.option("_scrollToSelectedItemEnabled")?this.callBase():this._renderPopover():this.callBase(),this._$popup.addClass(R),this._popup._wrapper().addClass(M)},_renderPopover:function(){this._popup=this._createComponent(this._$popup,b.default,(0,d.extend)(this._popupConfig(),this._options.cache("dropDownOptions"),{showEvent:null,hideEvent:null,target:this.$element(),_fixedPosition:!1,fullScreen:!1,shading:!1,closeOnTargetScroll:!0,width:this._isInitialOptionValue("dropDownOptions.width")?function(){return this.$element().outerWidth()}.bind(this):this._popupConfig().width})),this._popup.on({showing:this._popupShowingHandler.bind(this),shown:this._popupShownHandler.bind(this),hiding:this._popupHidingHandler.bind(this),hidden:this._popupHiddenHandler.bind(this),contentReady:this._contentReadyHandler.bind(this)}),this.option("_scrollToSelectedItemEnabled")&&this._popup._$arrow.remove(),this._setPopupContentId(this._popup.$content()),this._contentReadyHandler()},_popupHidingHandler:function(){this.callBase(),this.option("focusStateEnabled")&&this.focus()},_popupHiddenHandler:function(){this.callBase(),this.option("_scrollToSelectedItemEnabled")&&(0,C.resetPosition)((0,o.default)(this._popup.content()).parent())},_preventFocusOnPopup:l.noop,_popupConfig:function(){var e=this,t=(0,d.extend)(this.callBase(),{toolbarItems:this._getPopupToolbarItems(),closeOnTargetScroll:!1,onPositioned:null,maxHeight:function(){return(0,o.default)(D).height()},showTitle:this.option("dropDownOptions.showTitle"),title:this.option("dropDownOptions.title"),titleTemplate:this._getTemplateByOption("dropDownOptions.titleTemplate"),onTitleRendered:this.option("dropDownOptions.onTitleRendered"),fullScreen:this.option("dropDownOptions.fullScreen"),shading:this.option("dropDownOptions.shading"),closeOnOutsideClick:this.option("dropDownOptions.closeOnOutsideClick")});return delete t.animation,delete t.position,this.option("_scrollToSelectedItemEnabled")&&(t.position=this.option("dropDownCentered")?{my:"left top",at:"left top",of:this.element()}:{my:"left top",at:"left bottom",of:this.element()}),(0,c.each)(["position","animation","width","height"],function(n,i){var o=e.option("dropDownOptions.".concat(i));void 0!==o&&(t[i]=o)}),t},_getPopupToolbarItems:function(){var e="useButtons"===this.option("applyValueMode")?this._popupToolbarItemsConfig():[],t=this._getCancelButtonConfig();t&&e.push(t);var n=this._getClearButtonConfig();return n&&e.push(n),this._applyButtonsLocation(e)},_popupToolbarItemsConfig:function(){return[{shortcut:"done",options:{onClick:this._applyButtonHandler.bind(this),text:this.option("applyButtonText")}}]},_getCancelButtonConfig:function(){var e=this;return this.option("showCancelButton")?{shortcut:"cancel",onClick:this._cancelButtonHandler.bind(this),options:{onInitialized:function(t){t.component.registerKeyHandler("escape",e.close.bind(e))},text:this.option("cancelButtonText")}}:null},_getClearButtonConfig:function(){return this.option("showClearButton")?{shortcut:"clear",onClick:this._resetValue.bind(this),options:{text:this.option("clearButtonText")}}:null},_applyButtonHandler:function(){this.option("value",this._valueGetter(this._currentSelectedItem())),this.callBase()},_cancelButtonHandler:function(){this._refreshSelected(),this.callBase()},_refreshPopupVisibility:function(){this.option("opened")&&this._updatePopupDimensions()},_dimensionChanged:function(){this.option("usePopover")&&!this.option("dropDownOptions.width")&&this.option("dropDownOptions.width",this.$element().width()),this._popup&&this._updatePopupDimensions()},_input:function(){return this._$searchBox||this.callBase()},_renderPopupContent:function(){this.callBase(),this._renderSearch()},_renderSearch:function(){var e=this,t=this.option("searchEnabled");if(this._toggleSearchClass(t),t){var n=this._$searchWrapper=(0,o.default)("<div>").addClass(O),i=this._$searchBox=(0,o.default)("<div>").addClass(T).appendTo(n),a=p.default.current(),r=a.android&&a.version[0]>=5?"text":"search",s=!1;this._searchBox=this._createComponent(i,x.default,{onDisposing:function(){return s=!1},onFocusIn:function(){return s=!0},onFocusOut:function(){return s=!1},onKeyboardHandled:function(t){return s&&e._list._keyboardHandler(t)},mode:r,showClearButton:!0,valueChangeEvent:this.option("valueChangeEvent"),onValueChanged:this._searchHandler.bind(this)}),this._registerSearchKeyHandlers(),n.insertBefore(this._$list),this._setSearchPlaceholder()}},_removeSearch:function(){this._$searchWrapper&&this._$searchWrapper.remove(),delete this._$searchWrapper,this._$searchBox&&this._$searchBox.remove(),delete this._$searchBox,delete this._searchBox},_selectListItemHandler:function(e){var t=(0,o.default)(this._list.option("focusedElement"));t.length&&(e.preventDefault(),this._saveValueChangeEvent(e),this._selectListItem(e.itemData,t))},_registerSearchKeyHandlers:function(){this._searchBox.registerKeyHandler("escape",this.close.bind(this)),this._searchBox.registerKeyHandler("enter",this._selectListItemHandler.bind(this)),this._searchBox.registerKeyHandler("space",this._selectListItemHandler.bind(this)),this._searchBox.registerKeyHandler("end",l.noop),this._searchBox.registerKeyHandler("home",l.noop)},_toggleSearchClass:function(e){this._popup&&this._popup._wrapper().toggleClass(B,e)},_setSearchPlaceholder:function(){if(this._$searchBox){var e=this.option("minSearchLength"),t=this.option("searchPlaceholder");e&&t===f.default.format("Search")&&(t=f.default.getFormatter("dxLookup-searchPlaceholder")(e)),this._searchBox.option("placeholder",t)}},_setAriaTargetForList:l.noop,_renderList:function(){var e=this;this.callBase(),this._list.registerKeyHandler("escape",function(){e.close()})},_listConfig:function(){return(0,d.extend)(this.callBase(),{tabIndex:0,grouped:this.option("grouped"),groupTemplate:this._getTemplateByOption("groupTemplate"),pullRefreshEnabled:this.option("pullRefreshEnabled"),useNativeScrolling:this.option("useNativeScrolling"),pullingDownText:this.option("pullingDownText"),pulledDownText:this.option("pulledDownText"),refreshingText:this.option("refreshingText"),pageLoadingText:this.option("pageLoadingText"),onScroll:this._scrollHandler.bind(this),onPullRefresh:this._pullRefreshHandler.bind(this),onPageLoading:this._pageLoadingHandler.bind(this),pageLoadMode:this.option("pageLoadMode"),nextButtonText:this.option("nextButtonText"),indicateLoading:this.option("searchEnabled"),onSelectionChanged:this._getSelectionChangedHandler()})},_getSelectionChangedHandler:function(){return this.option("showSelectionControls")?this._selectionChangeHandler.bind(this):l.noop},_listContentReadyHandler:function(){this.callBase.apply(this,arguments),this._refreshSelected()},_setFocusPolicy:function(){this.option("focusStateEnabled")&&(this.option("searchEnabled")?this._searchBox.focus():a.default.trigger(this._$list,"focus"))},_focusTarget:function(){return this._$field},_keyboardEventBindingTarget:function(){return this._$field},_listItemClickHandler:function(e){this._saveValueChangeEvent(e.event),this._selectListItem(e.itemData,e.event.currentTarget)},_selectListItem:function(e,t){this._list.selectItem(t),"instantly"===this.option("applyValueMode")&&this._applyButtonHandler()},_currentSelectedItem:function(){return this.option("grouped")?this._list.option("selectedItems[0]").items[0]:this._list.option("selectedItems[0]")},_resetValue:function(e){this._saveValueChangeEvent(e.event),this.option("value",null),this.option("opened",!1)},_searchValue:function(){return this.option("searchEnabled")&&this._searchBox?this._searchBox.option("value"):""},_renderInputValue:function(){var e=this;return this.callBase().always(function(){e._refreshSelected()})},_renderPlaceholder:function(){0!==this.$element().find("."+H).length&&this.callBase()},_clean:function(){this._$fieldWrapper.remove(),this._$searchBox=null,delete this._inkRipple,this.callBase()},_setDeprecatedOptions:function(){this.callBase(),(0,d.extend)(this._deprecatedOptions,{title:{since:"20.1",alias:"dropDownOptions.title"},titleTemplate:{since:"20.1",alias:"dropDownOptions.titleTemplate"},onTitleRendered:{since:"20.1",alias:"dropDownOptions.onTitleRendered"},fullScreen:{since:"20.1",alias:"dropDownOptions.fullScreen"},popupWidth:{since:"20.1",alias:"dropDownOptions.width"},popupHeight:{since:"20.1",alias:"dropDownOptions.height"},shading:{since:"20.1",alias:"dropDownOptions.shading"},closeOnOutsideClick:{since:"20.1",alias:"dropDownOptions.closeOnOutsideClick"},position:{since:"20.1",alias:"dropDownOptions.position"},animation:{since:"20.1",alias:"dropDownOptions.animation"}})},_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"dataSource":this.callBase.apply(this,arguments),this._renderField();break;case"searchEnabled":this._popup&&(this._removeSearch(),this._renderSearch());break;case"searchPlaceholder":this._setSearchPlaceholder();break;case"minSearchLength":this._setSearchPlaceholder(),this.callBase.apply(this,arguments);break;case"title":case"titleTemplate":case"onTitleRendered":case"shading":case"animation":case"position":case"closeOnOutsideClick":case"fullScreen":this._setPopupOption(t,n);break;case"usePopover":case"placeholder":this._invalidate();break;case"clearButtonText":case"showClearButton":case"showCancelButton":this._setPopupOption("toolbarItems",this._getPopupToolbarItems());break;case"applyValueMode":this.callBase.apply(this,arguments);break;case"popupWidth":this._setPopupOption("width","auto"===n?this.initialOption("dropDownOptions").width:n);break;case"popupHeight":this._setPopupOption("height","auto"===n?this.initialOption("dropDownOptions").height:n);break;case"onPageLoading":this._initPageLoadingAction();break;case"onPullRefresh":this._initPullRefreshAction();break;case"pullRefreshEnabled":case"useNativeScrolling":case"pullingDownText":case"pulledDownText":case"refreshingText":case"pageLoadingText":case"nextButtonText":case"grouped":case"groupTemplate":this._setListOption(t);break;case"onScroll":this._initScrollAction();break;case"pageLoadMode":this._setListOption("pageLoadMode",this.option("pageLoadMode"));break;case"cleanSearchOnOpening":case"_scrollToSelectedItemEnabled":break;case"dropDownCentered":this.option("_scrollToSelectedItemEnabled")&&(this.option("dropDownOptions.position",void 0),this._renderPopup());break;default:this.callBase.apply(this,arguments)}},focus:function(){this.option("opened")?this._setFocusPolicy():a.default.trigger(this._focusTarget(),"focus")},field:function(){return this._$field}});(0,g.default)("dxLookup",q);var $=q;t.default=$,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(77)),s=n(6),l=i(n(9)),u=i(n(17)),c=i(n(13)),d=i(n(16)),h=n(40),f=n(3),p=n(0),g=n(12),_=n(1),m=n(8),v=i(n(25)),y=i(n(671)),b=i(n(672)),x=i(n(673)),w={googleStatic:y.default,google:b.default,bing:x.default},C="dx-map",k="dx-map-container",S="dx-map-shield",D="dx-native-click",I=d.default.inherit({_getDefaultOptions:function(){return(0,p.extend)(this.callBase(),{bounds:{northEast:null,southWest:null},center:{lat:0,lng:0},zoom:1,width:300,height:300,type:"roadmap",provider:"google",autoAdjust:!0,markers:[],markerIconSrc:null,onMarkerAdded:null,onMarkerRemoved:null,routes:[],onRouteAdded:null,onRouteRemoved:null,key:{bing:"",google:"",googleStatic:""},apiKey:{bing:"",google:"",googleStatic:""},controls:!1,onReady:null,onUpdated:null,onClick:null})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===c.default.real().deviceType&&!c.default.isSimulator()},options:{focusStateEnabled:!0}}])},_setDeprecatedOptions:function(){this.callBase(),(0,p.extend)(this._deprecatedOptions,{key:{since:"20.2",alias:"apiKey"}})},_init:function(){this.callBase(),this.$element().addClass(C).addClass(D),this._lastAsyncAction=r.default.resolve(),this._checkOption("provider"),this._checkOption("markers"),this._checkOption("routes"),this._initContainer(),this._grabEvents(),this._rendered={}},_checkOption:function(e){var t=this.option(e);if("markers"===e&&!Array.isArray(t))throw u.default.Error("E1022");if("routes"===e&&!Array.isArray(t))throw u.default.Error("E1023")},_initContainer:function(){this._$container=(0,o.default)("<div>").addClass(k),this.$element().append(this._$container)},_grabEvents:function(){var e=(0,m.addNamespace)(v.default.down,this.NAME);a.default.on(this.$element(),e,this._cancelEvent.bind(this))},_cancelEvent:function(e){var t=this._provider&&this._provider.isEventsCanceled(e)&&!this.option("disabled");t&&e.stopPropagation()},_saveRendered:function(e){var t=this.option(e);this._rendered[e]=t.slice()},_render:function(){this.callBase(),this._renderShield(),this._saveRendered("markers"),this._saveRendered("routes"),this._provider=new(w[this.option("provider")])(this,this._$container),this._queueAsyncAction("render",this._rendered.markers,this._rendered.routes)},_renderShield:function(){var e;this.option("disabled")?(e=(0,o.default)("<div>").addClass(S),this.$element().append(e)):(e=this.$element().find("."+S),e.remove())},_clean:function(){this._cleanFocusState(),this._provider&&this._provider.clean(),this._provider=null,this._lastAsyncAction=r.default.resolve(),this.setOptionSilent("bounds",{northEast:null,southWest:null}),delete this._suppressAsyncAction},_optionChanged:function(e){var t=e.name,n=this._optionChangeBag;switch(this._optionChangeBag=null,t){case"disabled":this._renderShield(),this.callBase(e);break;case"width":case"height":this.callBase(e),this._dimensionChanged();break;case"provider":this._suppressAsyncAction=!0,this._invalidate();break;case"key":case"apiKey":u.default.log("W1001");break;case"bounds":this._queueAsyncAction("updateBounds");break;case"center":this._queueAsyncAction("updateCenter");break;case"zoom":this._queueAsyncAction("updateZoom");break;case"type":this._queueAsyncAction("updateMapType");break;case"controls":this._queueAsyncAction("updateControls",this._rendered.markers,this._rendered.routes);break;case"autoAdjust":this._queueAsyncAction("adjustViewport");break;case"markers":case"routes":this._checkOption(t);var i=this._rendered[t];this._saveRendered(t),this._queueAsyncAction("update"+(0,h.titleize)(t),n?n.removed:i,n?n.added:this._rendered[t]).then(function(e){n&&n.resolve(e)});break;case"markerIconSrc":this._queueAsyncAction("updateMarkers",this._rendered.markers,this._rendered.markers);break;case"onReady":case"onUpdated":case"onMarkerAdded":case"onMarkerRemoved":case"onRouteAdded":case"onRouteRemoved":case"onClick":break;default:this.callBase.apply(this,arguments)}},_visibilityChanged:function(e){e&&this._dimensionChanged()},_dimensionChanged:function(){this._queueAsyncAction("updateDimensions")},_queueAsyncAction:function(e){var t=[].slice.call(arguments).slice(1),n=this._suppressAsyncAction;return this._lastAsyncAction=this._lastAsyncAction.then(function(){return!this._provider||n?r.default.resolve():this._provider[e].apply(this._provider,t).then(function(e){e=(0,g.wrapToArray)(e);var t=e[0];return t&&!this._disposed&&this._triggerReadyAction(),e[1]}.bind(this))}.bind(this)),this._lastAsyncAction},_triggerReadyAction:function(){this._createActionByOption("onReady")({originalMap:this._provider.map()})},_triggerUpdateAction:function(){this._createActionByOption("onUpdated")()},setOptionSilent:function(e,t){this._setOptionWithoutOptionChange(e,t)},addMarker:function(e){return this._addFunction("markers",e)},removeMarker:function(e){return this._removeFunction("markers",e)},addRoute:function(e){return this._addFunction("routes",e)},removeRoute:function(e){return this._removeFunction("routes",e)},_addFunction:function(e,t){var n=this.option(e),i=(0,g.wrapToArray)(t);return n.push.apply(n,i),this._partialArrayOptionChange(e,n,i,[])},_removeFunction:function(e,t){var n=this.option(e),i=(0,g.wrapToArray)(t);return(0,f.each)(i,function(t,o){var a=(0,_.isNumeric)(o)?o:(0,g.inArray)(o,n);if(a===-1)throw u.default.log("E1021",(0,h.titleize)(e.substring(0,e.length-1)),o);var r=n.splice(a,1)[0];i.splice(t,1,r)}),this._partialArrayOptionChange(e,n,[],i)},_partialArrayOptionChange:function(e,t,n,i){return(0,s.fromPromise)(new r.default(function(o){this._optionChangeBag={resolve:o,added:n,removed:i},this.option(e,t)}.bind(this)).then(function(e){return e&&1===e.length?e[0]:e}),this)}});(0,l.default)("dxMap",I);var T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=i(n(5)),r=i(n(77)),s=i(n(448)),l=i(n(98)),u=n(21),c="https://maps.google.com/maps/api/staticmap?",d=s.default.inherit({_locationToString:function(e){var t=this._getLatLng(e);return t?t.lat+","+t.lng:e.toString().replace(/ /g,"+")},_renderImpl:function(){return this._updateMap()},updateDimensions:function(){return this._updateMap()},updateMapType:function(){return this._updateMap()},updateBounds:function(){return r.default.resolve()},updateCenter:function(){return this._updateMap()},updateZoom:function(){return this._updateMap()},updateControls:function(){return r.default.resolve()},addMarkers:function(e){var t=this;return this._updateMap().then(function(n){return(0,o.each)(e,function(e,n){t._fireMarkerAddedAction({options:n})}),n})},removeMarkers:function(e){var t=this;return this._updateMap().then(function(n){return(0,o.each)(e,function(e,n){t._fireMarkerRemovedAction({options:n})}),n})},adjustViewport:function(){return r.default.resolve()},addRoutes:function(e){var t=this;return this._updateMap().then(function(n){return(0,o.each)(e,function(e,n){t._fireRouteAddedAction({options:n})}),n})},removeRoutes:function(e){var t=this;return this._updateMap().then(function(n){return(0,o.each)(e,function(e,n){t._fireRouteRemovedAction({options:n})}),n})},clean:function(){return this._$container.css("backgroundImage","none"),a.default.off(this._$container,this._addEventNamespace(u.name)),r.default.resolve()},mapRendered:function(){return!0},_updateMap:function(){var e=this._keyOption("googleStatic"),t=this._$container,n=["sensor=false","size="+Math.round(t.width())+"x"+Math.round(t.height()),"maptype="+this._option("type"),"center="+this._locationToString(this._option("center")),"zoom="+this._option("zoom"),this._markersSubstring()];n.push.apply(n,this._routeSubstrings()),e&&n.push("key="+e);var i=c+n.join("&");return this._$container.css("background",'url("'+i+'") no-repeat 0 0'),this._attachClickEvent(),r.default.resolve(!0)},_markersSubstring:function(){var e=this,t=[],n=this._option("markerIconSrc");return n&&t.push("icon:"+n),(0,o.each)(this._option("markers"),function(n,i){t.push(e._locationToString(i.location))}),"markers="+t.join("|")},_routeSubstrings:function(){var e=this,t=[];return(0,o.each)(this._option("routes"),function(n,i){var a=new l.default(i.color||e._defaultRouteColor()).toHex().replace("#","0x"),r=Math.round(255*(i.opacity||e._defaultRouteOpacity())).toString(16),s=i.weight||e._defaultRouteWeight(),u=[];(0,o.each)(i.locations,function(t,n){u.push(e._locationToString(n))}),t.push("path=color:"+a+r+"|weight:"+s+"|"+u.join("|"))}),t},_attachClickEvent:function(){var e=this,t=this._addEventNamespace(u.name);a.default.off(this._$container,t),a.default.on(this._$container,t,function(t){e._fireClickAction({event:t})})}}),h=d;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a,r=i(n(2)),s=n(7),l=n(4),u=i(n(13)),c=i(n(77)),d=n(0),h=n(3),f=i(n(449)),p=i(n(17)),g=i(n(98)),_=i(n(76)),m=n(1),v=(0,s.getWindow)(),y="_googleScriptReady",b="https://maps.googleapis.com/maps/api/js?callback="+y,x="gm-style-iw",w=function(){o=function(e){this._position=e.position,this._offset=e.offset,this._$overlayContainer=(0,r.default)("<div>").css({position:"absolute",display:"none",cursor:"pointer"}).append(e.html),this.setMap(e.map)},o.prototype=new google.maps.OverlayView,o.prototype.onAdd=function(){var e=(0,r.default)(this.getPanes().overlayMouseTarget);e.append(this._$overlayContainer),this._clickListener=google.maps.event.addDomListener(this._$overlayContainer.get(0),"click",function(e){google.maps.event.trigger(this,"click"),e.preventDefault()}.bind(this)),this.draw()},o.prototype.onRemove=function(){google.maps.event.removeListener(this._clickListener),this._$overlayContainer.remove()},o.prototype.draw=function(){var e=this.getProjection().fromLatLngToDivPixel(this._position);this._$overlayContainer.css({left:e.x+this._offset.left,top:e.y+this._offset.top,display:"block"})}},C=function(){return v.google&&v.google.maps},k=f.default.inherit({_mapType:function(e){var t={hybrid:google.maps.MapTypeId.HYBRID,roadmap:google.maps.MapTypeId.ROADMAP,satellite:google.maps.MapTypeId.SATELLITE};return t[e]||t.hybrid},_movementMode:function(e){var t={driving:google.maps.TravelMode.DRIVING,walking:google.maps.TravelMode.WALKING};return t[e]||t.driving},_resolveLocation:function(e){return new c.default(function(t){var n=this._getLatLng(e);n?t(new google.maps.LatLng(n.lat,n.lng)):this._geocodeLocation(e).then(function(e){t(e)})}.bind(this))},_geocodedLocations:{},_geocodeLocationImpl:function(e){return new c.default(function(t){if(!(0,m.isDefined)(e))return void t(new google.maps.LatLng(0,0));var n=new google.maps.Geocoder;n.geocode({address:e},function(e,n){n===google.maps.GeocoderStatus.OK?t(e[0].geometry.location):(p.default.log("W1006",n),t(new google.maps.LatLng(0,0)))})})},_normalizeLocation:function(e){return{lat:e.lat(),lng:e.lng()}},_normalizeLocationRect:function(e){return{northEast:this._normalizeLocation(e.getNorthEast()),southWest:this._normalizeLocation(e.getSouthWest())}},_loadImpl:function(){return new c.default(function(e){C()?e():(a||(a=this._loadMapScript()),a.then(function(){return C()?void e():void this._loadMapScript().then(e)}.bind(this)))}.bind(this)).then(function(){w()})},_loadMapScript:function(){return new c.default(function(e){var t=this._keyOption("google");v[y]=e,_.default.sendRequest({url:b+(t?"&key="+t:""),dataType:"script"})}.bind(this)).then(function(){try{delete v[y]}catch(e){v[y]=void 0}})},_init:function(){return new c.default(function(e){this._resolveLocation(this._option("center")).then(function(t){var n=this._option("controls");this._map=new google.maps.Map(this._$container[0],{zoom:this._option("zoom"),center:t,disableDefaultUI:!n});var i=google.maps.event.addListener(this._map,"idle",function(){e(i)})}.bind(this))}.bind(this)).then(function(e){google.maps.event.removeListener(e)})},_attachHandlers:function(){this._boundsChangeListener=google.maps.event.addListener(this._map,"bounds_changed",this._boundsChangeHandler.bind(this)),this._clickListener=google.maps.event.addListener(this._map,"click",this._clickActionHandler.bind(this))},_boundsChangeHandler:function(){var e=this._map.getBounds();this._option("bounds",this._normalizeLocationRect(e));var t=this._map.getCenter();this._option("center",this._normalizeLocation(t)),this._preventZoomChangeEvent||this._option("zoom",this._map.getZoom())},_clickActionHandler:function(e){this._fireClickAction({location:this._normalizeLocation(e.latLng)})},updateDimensions:function(){var e=this._option("center");return google.maps.event.trigger(this._map,"resize"),this._option("center",e),this.updateCenter()},updateMapType:function(){return this._map.setMapTypeId(this._mapType(this._option("type"))),c.default.resolve()},updateBounds:function(){return c.default.all([this._resolveLocation(this._option("bounds.northEast")),this._resolveLocation(this._option("bounds.southWest"))]).then(function(e){var t=new google.maps.LatLngBounds;t.extend(e[0]),t.extend(e[1]),this._map.fitBounds(t)}.bind(this))},updateCenter:function(){return this._resolveLocation(this._option("center")).then(function(e){this._map.setCenter(e),this._option("center",this._normalizeLocation(e))}.bind(this))},updateZoom:function(){return this._map.setZoom(this._option("zoom")),c.default.resolve()},updateControls:function(){var e=this._option("controls");return this._map.setOptions({disableDefaultUI:!e}),c.default.resolve()},isEventsCanceled:function(e){var t=this._map&&this._map.get("gestureHandling"),n=(0,r.default)(e.target).closest(".".concat(x)).length>0;return!(n||"desktop"!==u.default.real().deviceType&&"cooperative"===t)&&this.callBase()},_renderMarker:function(e){return this._resolveLocation(e.location).then(function(t){var n;n=e.html?new o({map:this._map,position:t,html:e.html,offset:(0,d.extend)({top:0,left:0},e.htmlOffset)}):new google.maps.Marker({position:t,map:this._map,icon:e.iconSrc||this._option("markerIconSrc")});var i,a=this._renderTooltip(n,e.tooltip);if(e.onClick||e.tooltip){var r=this._mapWidget._createAction(e.onClick||l.noop),s=this._normalizeLocation(t);i=google.maps.event.addListener(n,"click",function(){r({location:s}),a&&a.open(this._map,n)}.bind(this))}return{location:t,marker:n,listener:i}}.bind(this))},_renderTooltip:function(e,t){if(t){t=this._parseTooltipOptions(t);var n=new google.maps.InfoWindow({content:t.text});return t.visible&&n.open(this._map,e),n}},_destroyMarker:function(e){e.marker.setMap(null),e.listener&&google.maps.event.removeListener(e.listener)},_renderRoute:function(e){return c.default.all((0,h.map)(e.locations,function(e){return this._resolveLocation(e)}.bind(this))).then(function(t){return new c.default(function(n){var i=t.shift(),o=t.pop(),a=(0,h.map)(t,function(e){return{location:e,stopover:!0}}),r={origin:i,destination:o,waypoints:a,optimizeWaypoints:!0,travelMode:this._movementMode(e.mode)};(new google.maps.DirectionsService).route(r,function(t,i){if(i===google.maps.DirectionsStatus.OK){var o=new g.default(e.color||this._defaultRouteColor()).toHex(),a={directions:t,map:this._map,suppressMarkers:!0,preserveViewport:!0,polylineOptions:{strokeWeight:e.weight||this._defaultRouteWeight(),strokeOpacity:e.opacity||this._defaultRouteOpacity(),strokeColor:o}},r=new google.maps.DirectionsRenderer(a),s=t.routes[0].bounds;n({instance:r,northEast:s.getNorthEast(),southWest:s.getSouthWest()})}else p.default.log("W1006",i),n({instance:new google.maps.DirectionsRenderer({})})}.bind(this))}.bind(this))}.bind(this))},_destroyRoute:function(e){e.instance.setMap(null)},_fitBounds:function(){if(this._updateBounds(),this._bounds&&this._option("autoAdjust")){var e=this._map.getZoom();this._preventZoomChangeEvent=!0,this._map.fitBounds(this._bounds),this._boundsChangeHandler();var t=this._map.getZoom();e<t?this._map.setZoom(e):this._option("zoom",t),delete this._preventZoomChangeEvent}return c.default.resolve()},_extendBounds:function(e){this._bounds?this._bounds.extend(e):(this._bounds=new google.maps.LatLngBounds,this._bounds.extend(e))},clean:function(){return this._map&&(google.maps.event.removeListener(this._boundsChangeListener),google.maps.event.removeListener(this._clickListener),this._clearMarkers(),this._clearRoutes(),delete this._map,this._$container.empty()),c.default.resolve()}}),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o,a=n(4),r=n(7),s=i(n(77)),l=n(0),u=i(n(17)),c=n(3),d=i(n(449)),h=i(n(98)),f=i(n(76)),p=n(1),g=(0,r.getWindow)(),_="_bingScriptReady",m="https://www.bing.com/api/maps/mapcontrol?callback="+_,v=13,y="AhuxC0dQ1DBTNo8L-H9ToVMQStmizZzBJdraTSgCzDSWPsA1Qd8uIvFSflzxdaLH",b=1e-16,x=function(){return g.Microsoft&&g.Microsoft.Maps},w=d.default.inherit({_mapType:function(e){var t={roadmap:Microsoft.Maps.MapTypeId.road,hybrid:Microsoft.Maps.MapTypeId.aerial,satellite:Microsoft.Maps.MapTypeId.aerial};return t[e]||t.road},_movementMode:function(e){var t={driving:Microsoft.Maps.Directions.RouteMode.driving,walking:Microsoft.Maps.Directions.RouteMode.walking};return t[e]||t.driving},_resolveLocation:function(e){return new s.default(function(t){var n=this._getLatLng(e);n?t(new Microsoft.Maps.Location(n.lat,n.lng)):this._geocodeLocation(e).then(function(e){t(e)})}.bind(this))},_geocodedLocations:{},_geocodeLocationImpl:function(e){return new s.default(function(t){if(!(0,p.isDefined)(e))return void t(new Microsoft.Maps.Location(0,0));var n=new Microsoft.Maps.Search.SearchManager(this._map),i={where:e,count:1,callback:function(e){var n=e.results[0];if(n){var i=e.results[0].location;t(new Microsoft.Maps.Location(i.latitude,i.longitude))}else t(new Microsoft.Maps.Location(0,0))}};n.geocode(i)}.bind(this))},_normalizeLocation:function(e){return{lat:e.latitude,lng:e.longitude}},_normalizeLocationRect:function(e){var t=this._normalizeLocation(e.getNorthwest()),n=this._normalizeLocation(e.getSoutheast());return{northEast:{lat:t.lat,lng:n.lng},southWest:{lat:n.lat,lng:t.lng}}},_loadImpl:function(){return new s.default(function(e){x()?e():(o||(o=this._loadMapScript()),o.then(function(){return x()?void e():void this._loadMapScript().then(e)}.bind(this)))}.bind(this)).then(function(){return s.default.all([new s.default(function(e){Microsoft.Maps.loadModule("Microsoft.Maps.Search",{callback:e})}),new s.default(function(e){Microsoft.Maps.loadModule("Microsoft.Maps.Directions",{callback:e})})])})},_loadMapScript:function(){return new s.default(function(e){g[_]=e,f.default.sendRequest({url:m,dataType:"script"})}).then(function(){try{delete g[_]}catch(e){g[_]=void 0}})},_init:function(){return this._createMap(),s.default.resolve()},_createMap:function(){var e=this._option("controls");this._map=new Microsoft.Maps.Map(this._$container[0],{credentials:this._keyOption("bing")||y,zoom:this._option("zoom"),showDashboard:e,showMapTypeSelector:e,showScalebar:e})},_attachHandlers:function(){this._providerViewChangeHandler=Microsoft.Maps.Events.addHandler(this._map,"viewchange",this._viewChangeHandler.bind(this)),this._providerClickHandler=Microsoft.Maps.Events.addHandler(this._map,"click",this._clickActionHandler.bind(this))},_viewChangeHandler:function(){var e=this._map.getBounds();this._option("bounds",this._normalizeLocationRect(e));var t=this._map.getCenter();this._option("center",this._normalizeLocation(t)),this._preventZoomChangeEvent||this._option("zoom",this._map.getZoom())},_clickActionHandler:function(e){"map"===e.targetType&&this._fireClickAction({location:this._normalizeLocation(e.location)})},updateDimensions:function(){var e=this._$container;return this._map.setOptions({width:e.width(),height:e.height()}),s.default.resolve()},updateMapType:function(){var e=this._option("type"),t=Microsoft.Maps.LabelOverlay;return this._map.setView({animate:!1,mapTypeId:this._mapType(e),labelOverlay:"satellite"===e?t.hidden:t.visible}),s.default.resolve()},updateBounds:function(){return s.default.all([this._resolveLocation(this._option("bounds.northEast")),this._resolveLocation(this._option("bounds.southWest"))]).then(function(e){
var t=new Microsoft.Maps.LocationRect.fromLocations(e[0],e[1]);this._map.setView({animate:!1,bounds:t})}.bind(this))},updateCenter:function(){return this._resolveLocation(this._option("center")).then(function(e){this._map.setView({animate:!1,center:e})}.bind(this))},updateZoom:function(){return this._map.setView({animate:!1,zoom:this._option("zoom")}),s.default.resolve()},updateControls:function(){return this.clean(),this.render.apply(this,arguments)},_renderMarker:function(e){return this._resolveLocation(e.location).then(function(t){var n={icon:e.iconSrc||this._option("markerIconSrc")};if(e.html){(0,l.extend)(n,{htmlContent:e.html,width:null,height:null});var i=e.htmlOffset;i&&(n.anchor=new Microsoft.Maps.Point((-i.left),(-i.top)))}var o=new Microsoft.Maps.Pushpin(t,n);this._map.entities.push(o);var r,s=this._renderTooltip(t,e.tooltip);if(e.onClick||e.tooltip){var u=this._mapWidget._createAction(e.onClick||a.noop),c=this._normalizeLocation(t);r=Microsoft.Maps.Events.addHandler(o,"click",function(){u({location:c}),s&&s.setOptions({visible:!0})})}return{location:t,marker:o,infobox:s,handler:r}}.bind(this))},_renderTooltip:function(e,t){if(t){t=this._parseTooltipOptions(t);var n=new Microsoft.Maps.Infobox(e,{description:t.text,offset:new Microsoft.Maps.Point(0,v),visible:t.visible});return n.setMap(this._map),n}},_destroyMarker:function(e){this._map.entities.remove(e.marker),e.infobox&&e.infobox.setMap(null),e.handler&&Microsoft.Maps.Events.removeHandler(e.handler)},_renderRoute:function(e){return s.default.all((0,c.map)(e.locations,function(e){return this._resolveLocation(e)}.bind(this))).then(function(t){return new s.default(function(n){var i=new Microsoft.Maps.Directions.DirectionsManager(this._map),o=new h.default(e.color||this._defaultRouteColor()).toHex(),a=new Microsoft.Maps.Color.fromHex(o);a.a=255*(e.opacity||this._defaultRouteOpacity()),i.setRenderOptions({autoUpdateMapView:!1,displayRouteSelector:!1,waypointPushpinOptions:{visible:!1},drivingPolylineOptions:{strokeColor:a,strokeThickness:e.weight||this._defaultRouteWeight()},walkingPolylineOptions:{strokeColor:a,strokeThickness:e.weight||this._defaultRouteWeight()}}),i.setRequestOptions({routeMode:this._movementMode(e.mode),routeDraggable:!1}),(0,c.each)(t,function(e,t){var n=new Microsoft.Maps.Directions.Waypoint({location:t});i.addWaypoint(n)});var r=[];r.push(Microsoft.Maps.Events.addHandler(i,"directionsUpdated",function(e){for(;r.length;)Microsoft.Maps.Events.removeHandler(r.pop());var t=e.routeSummary[0];n({instance:i,northEast:t.northEast,southWest:t.southWest})})),r.push(Microsoft.Maps.Events.addHandler(i,"directionsError",function(e){for(;r.length;)Microsoft.Maps.Events.removeHandler(r.pop());var t="RouteResponseCode: "+e.responseCode+" - "+e.message;u.default.log("W1006",t),n({instance:i})})),i.calculateDirections()}.bind(this))}.bind(this))},_destroyRoute:function(e){e.instance.dispose()},_fitBounds:function(){if(this._updateBounds(),this._bounds&&this._option("autoAdjust")){var e=this._map.getZoom();this._preventZoomChangeEvent=!0;var t=this._bounds.clone();t.height=1.1*t.height,t.width=1.1*t.width,this._map.setView({animate:!1,bounds:t,zoom:e});var n=this._map.getZoom();e<n?this._map.setView({animate:!1,zoom:e}):this._option("zoom",n),delete this._preventZoomChangeEvent}return s.default.resolve()},_extendBounds:function(e){this._bounds?this._bounds=new Microsoft.Maps.LocationRect.fromLocations(this._bounds.getNorthwest(),this._bounds.getSoutheast(),e):this._bounds=new Microsoft.Maps.LocationRect(e,b,b)},clean:function(){return this._map&&(Microsoft.Maps.Events.removeHandler(this._providerViewChangeHandler),Microsoft.Maps.Events.removeHandler(this._providerClickHandler),this._clearMarkers(),this._clearRoutes(),this._map.dispose()),s.default.resolve()}}),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=n(0),r=i(n(675)),s=i(n(220)),l="dx-navbar",u="dx-item-content",c="dx-nav-item",d="dx-nav-item-content",h=s.default.inherit({ctor:function(e,t){this.callBase(e,t),this._logDeprecatedComponentWarning("20.1","dxTabs")},_getDefaultOptions:function(){return(0,a.extend)(this.callBase(),{scrollingEnabled:!1})},_render:function(){this.callBase(),this.$element().addClass(l)},_postprocessRenderItem:function(e){this.callBase(e);var t=e.itemElement,n=e.itemData;t.addClass(c),t.find("."+u).addClass(d),n.icon||t.addClass("dx-navbar-text-item")}});h.ItemClass=r.default,(0,o.default)("dxNavBar",h);var f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(439)),a="dx-tabs-item-badge",r="dx-navbar-item-badge",s=o.default.inherit({_renderBadge:function(e){this.callBase(e),this._$element.children("."+a).removeClass(a).addClass(r)}}),l=s;t.default=l,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function l(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function u(e,t,n){return t&&l(e.prototype,t),n&&l(e,n),e}function c(e,t,n){return(c="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=d(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function d(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=v(e),null!==e););return e}function h(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&f(e,t)}function f(e,t){return(f=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function p(e){var t=m();return function(){var n,i=v(e);if(t){var o=v(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return g(this,n)}}function g(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?_(e):t}function _(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function m(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function v(e){return(v=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var y=r(n(2)),b=n(0),x=r(n(13)),w=n(4),C=n(1),k=a(n(87)),S=r(n(9)),D=r(n(58)),I=r(n(288)),T=r(n(52)),O=n(6),E="dx-radiobutton-checked",A="dx-radiobutton",P="dx-radiobutton-icon-checked",R="dx-radiobutton-icon",M="dx-radiobutton-icon-dot",B="dx-radiogroup-horizontal",V="dx-radiogroup-vertical",F="dx-radio-value-container",L="dx-radiogroup",H=100,z=function(e){function t(){return s(this,t),n.apply(this,arguments)}h(t,e);var n=p(t);return u(t,[{key:"_focusTarget",value:function(){return this.$element().parent()}},{key:"_nullValueSelectionSupported",value:function(){return!0}},{key:"_getDefaultOptions",value:function(){var e=c(v(t.prototype),"_getDefaultOptions",this).call(this);return(0,b.extend)(e,I.default._dataExpressionDefaultOptions(),{_itemAttributes:{role:"radio"}})}},{key:"_initMarkup",value:function(){var e=this;c(v(t.prototype),"_initMarkup",this).call(this),(0,w.deferRender)(function(){e.itemElements().addClass(A)})}},{key:"_keyboardEventBindingTarget",value:function(){return this._focusTarget()}},{key:"_postprocessRenderItem",value:function(e){var n=e.itemData.html,i=e.itemElement;if(!n){var o=(0,y.default)("<div>").addClass(R);(0,y.default)("<div>").addClass(M).appendTo(o);var a=(0,y.default)("<div>").append(o).addClass(F);(0,y.default)(i).prepend(a)}c(v(t.prototype),"_postprocessRenderItem",this).call(this,e)}},{key:"_processSelectableItem",value:function(e,n){c(v(t.prototype),"_processSelectableItem",this).call(this,e,n),e.toggleClass(E,n).find(".".concat(R)).first().toggleClass(P,n),this.setAria("checked",n,e)}},{key:"_refreshContent",value:function(){this._prepareContent(),this._renderContent()}},{key:"_supportedKeys",value:function(){var e=c(v(t.prototype),"_supportedKeys",this).call(this);return(0,b.extend)({},e,{enter:function(t){return t.preventDefault(),e.enter.apply(this,arguments)},space:function(t){return t.preventDefault(),e.space.apply(this,arguments)}})}},{key:"_itemElements",value:function(){return this._itemContainer().children(this._itemSelector())}}]),t}(D.default),N=function(e){function t(){return s(this,t),n.apply(this,arguments)}h(t,e);var n=p(t);return u(t,[{key:"_clean",value:function(){delete this._inkRipple,c(v(t.prototype),"_clean",this).call(this)}},{key:"_dataSourceOptions",value:function(){return{paginate:!1}}},{key:"_defaultOptionsRules",value:function(){var e=c(v(t.prototype),"_defaultOptionsRules",this).call(this);return e.concat([{device:{tablet:!0},options:{layout:"horizontal"}},{device:function(){return"desktop"===x.default.real().deviceType&&!x.default.isSimulator()},options:{focusStateEnabled:!0}}])}},{key:"_fireContentReadyAction",value:function(e){e&&c(v(t.prototype),"_fireContentReadyAction",this).call(this)}},{key:"_focusTarget",value:function(){return this.$element()}},{key:"_getAriaTarget",value:function(){return this.$element()}},{key:"_getDefaultOptions",value:function(){var e=c(v(t.prototype),"_getDefaultOptions",this).call(this);return(0,b.extend)(e,(0,b.extend)(I.default._dataExpressionDefaultOptions(),{hoverStateEnabled:!0,activeStateEnabled:!0,layout:"vertical",useInkRipple:!1}))}},{key:"_getItemValue",value:function(e){return this._valueGetter?this._valueGetter(e):e.text}},{key:"_getSubmitElement",value:function(){return this._$submitElement}},{key:"_init",value:function(){c(v(t.prototype),"_init",this).call(this),this._activeStateUnit=".".concat(A),this._feedbackHideTimeout=H,this._initDataExpressions()}},{key:"_initMarkup",value:function(){this.$element().addClass(L),this._renderSubmitElement(),this.setAria("role","radiogroup"),this._renderRadios(),this.option("useInkRipple")&&this._renderInkRipple(),this._renderLayout(),c(v(t.prototype),"_initMarkup",this).call(this)}},{key:"_itemClickHandler",value:function(e){var t=e.itemElement,n=e.event,i=e.itemData;if(this.itemElements().is(t)){var o=this._getItemValue(i);o!==this.option("value")&&(this._saveValueChangeEvent(n),this.option("value",o))}}},{key:"_setSelection",value:function(e){this._setCollectionWidgetOption("selectedItemKeys",[this._unwrappedValue(e)])}},{key:"_optionChanged",value:function(e){var n=e.name,i=e.value;switch(this._dataExpressionOptionChanged(e),n){case"useInkRipple":case"dataSource":this._invalidate();break;case"focusStateEnabled":case"accessKey":case"tabIndex":this._setCollectionWidgetOption(n,i);break;case"disabled":c(v(t.prototype),"_optionChanged",this).call(this,e),this._setCollectionWidgetOption(n,i);break;case"valueExpr":this._setCollectionWidgetOption("keyExpr",this._getCollectionKeyExpr());break;case"value":this._setSelection(i),this._setSubmitValue(i),c(v(t.prototype),"_optionChanged",this).call(this,e);break;case"items":this._setSelection(this.option("value"));break;case"itemTemplate":case"displayExpr":break;case"layout":this._renderLayout(),this._updateItemsSize();break;default:c(v(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_render",value:function(){c(v(t.prototype),"_render",this).call(this),this._updateItemsSize()}},{key:"_renderInkRipple",value:function(){this._inkRipple=k.render({waveSizeCoefficient:3.3,useHoldAnimation:!1,isCentered:!0})}},{key:"_renderLayout",value:function(){var e=this.option("layout"),t=this.$element();t.toggleClass(V,"vertical"===e),t.toggleClass(B,"horizontal"===e)}},{key:"_renderRadios",value:function(){var e=this;this._areRadiosCreated=new O.Deferred;var t=(0,y.default)("<div>").appendTo(this.$element()),n=this.option(),i=n.value,o=n.displayExpr,a=n.accessKey,r=n.focusStateEnabled,s=n.itemTemplate,l=n.tabIndex,u=n.valueExpr,c="this"!==u;this._createComponent(t,z,{onInitialized:function(t){var n=t.component;e._radios=n},onContentReady:function(t){e._fireContentReadyAction(!0)},onItemClick:this._itemClickHandler.bind(this),displayExpr:o,accessKey:a,dataSource:this._dataSource,focusStateEnabled:r,itemTemplate:s,keyExpr:this._getCollectionKeyExpr(),noDataText:"",scrollingEnabled:!1,selectionByClick:!1,selectionMode:"single",selectedItemKeys:c||(0,C.isDefined)(i)?[i]:[],tabIndex:l}),this._areRadiosCreated.resolve()}},{key:"_renderSubmitElement",value:function(){this._$submitElement=(0,y.default)("<input>").attr("type","hidden").appendTo(this.$element()),this._setSubmitValue()}},{key:"_setOptionsByReference",value:function(){c(v(t.prototype),"_setOptionsByReference",this).call(this),(0,b.extend)(this._optionsByReference,{value:!0})}},{key:"_setSubmitValue",value:function(e){e=e||this.option("value");var t="this"===this.option("valueExpr")?this._displayGetter(e):e;this._$submitElement.val(t)}},{key:"_setCollectionWidgetOption",value:function(){this._areRadiosCreated.done(this._setWidgetOption.bind(this,"_radios",arguments))}},{key:"_toggleActiveState",value:function(e,n,i){if(c(v(t.prototype),"_toggleActiveState",this).call(this,e,n,i),this._inkRipple){var o={element:e.find(".".concat(R)),event:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}}},{key:"_updateItemsSize",value:function(){if("horizontal"===this.option("layout"))this.itemElements().css("height","auto");else{var e=this.option("items").length;this.itemElements().css("height",100/e+"%")}}},{key:"focus",value:function(){var e;null===(e=this._radios)||void 0===e?void 0:e.focus()}},{key:"itemElements",value:function(){var e;return null===(e=this._radios)||void 0===e?void 0:e.itemElements()}}]),t}(T.default);N.include(I.default),(0,S.default)("dxRadioGroup",N);var j=N;t.default=j,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(451)),s=i(n(452)),l=i(n(9)),u=n(0),c=n(4),d=n(8),h=i(n(10)),f="dx-rangeslider",p=f+"-start-handle",g=f+"-end-handle",_=r.default.inherit({_supportedKeys:function(){var e=this.option("rtlEnabled"),t=this,n=function(e,n){t.option("start")===t.option("end")&&(t._capturedHandle=n,e.target=t._capturedHandle,a.default.trigger(t._capturedHandle,"focus"))},i=function(n,i,a){var r=(0,o.default)(n.target).hasClass(p),s=r?"start":"end",l=t.option(s);i=t._valueStep(i),l+=a*(e?-i:i),t.option(s,l)},r=function(o,a){n(o,e?t._$handleStart:t._$handleEnd),i(o,a,1)},s=function(o,a){n(o,e?t._$handleEnd:t._$handleStart),i(o,a,-1)};return(0,u.extend)(this.callBase(),{leftArrow:function(e){this._processKeyboardEvent(e),s(e,this.option("step"))},rightArrow:function(e){this._processKeyboardEvent(e),r(e,this.option("step"))},pageUp:function(e){this._processKeyboardEvent(e),r(e,this.option("step")*this.option("keyStep"))},pageDown:function(e){this._processKeyboardEvent(e),s(e,this.option("step")*this.option("keyStep"))},home:function(e){this._processKeyboardEvent(e);var t=(0,o.default)(e.target).hasClass(p),n=t?"start":"end",i=t?"min":"start",a=this.option(i);this.option(n,a)},end:function(e){this._processKeyboardEvent(e);var t=(0,o.default)(e.target).hasClass(p),n=t?"start":"end",i=t?"end":"max",a=this.option(i);this.option(n,a)}})},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{start:40,end:60,value:[40,60],startName:"",endName:""})},_renderSubmitElement:function(){var e=this.$element();this._$submitStartElement=(0,o.default)("<input>").attr("type","hidden").attr("name",this.option("startName")).appendTo(e),this._$submitEndElement=(0,o.default)("<input>").attr("type","hidden").attr("name",this.option("endName")).appendTo(e)},_initOptions:function(e){this.callBase(e);var t=this.initialOption("value"),n=this.option("value");n[0]===t[0]&&n[1]===t[1]?this.option("value",[this.option("start"),this.option("end")]):this.option({start:n[0],end:n[1]})},_initMarkup:function(){this.$element().addClass(f),this.callBase()},_renderContentImpl:function(){this._callHandlerMethod("repaint"),this.callBase()},_renderHandle:function(){this._$handleStart=this._renderHandleImpl(this.option("start"),this._$handleStart).addClass(p),this._$handleEnd=this._renderHandleImpl(this.option("end"),this._$handleEnd).addClass(g),this._updateHandleAriaLabels()},_startHandler:function(e){var t=e.event,n=this._$range,i=n.width(),o=(0,d.eventData)(t).x-this._$bar.offset().left,a=n.position().left,r=n.position().left+i,s=this.option("rtlEnabled"),l=(s?-1:1)*((a+r)/2-o)>0;this._capturedHandle=l?this._$handleStart:this._$handleEnd,this.callBase(e)},_updateHandleAriaLabels:function(){this.setAria("label",h.default.getFormatter("dxRangeSlider-ariaFrom")(this.option("dxRangeSlider-ariaFrom")),this._$handleStart),this.setAria("label",h.default.getFormatter("dxRangeSlider-ariaTill")(this.option("dxRangeSlider-ariaTill")),this._$handleEnd)},_activeHandle:function(){return this._capturedHandle},_updateHandlePosition:function(e){var t=this.option("rtlEnabled"),n=t?-1:1,i=this.option("max"),o=this.option("min"),r=this._startOffset+n*e.event.offset/this._swipePixelRatio();r=r.toPrecision(12);var l=r*(i-o)+o;this._updateSelectedRangePosition(r,r),s.default.getInstance(this._activeHandle()).fitTooltipPosition,this._changeValueOnSwipe(r);var u,c=this.option("start"),d=this.option("end");c===d&&(u=l<c?this._$handleStart:this._$handleEnd,a.default.trigger(u,"focus"),u&&u!==this._capturedHandle&&(this._updateSelectedRangePosition((c-o)/(i-o),(d-o)/(i-o)),this._toggleActiveState(this._activeHandle(),!1),this._toggleActiveState(u,!0),this._capturedHandle=u),this._updateSelectedRangePosition(r,r),this._changeValueOnSwipe(r))},_updateSelectedRangePosition:function(e,t){var n=this.option("rtlEnabled"),i=this._capturedHandle===this._$handleStart&&n||this._capturedHandle===this._$handleEnd&&!n,o=i?"right":"left";n^i?this._$range.css(o,100-100*t+"%"):this._$range.css(o,100*e+"%")},_setValueOnSwipe:function(e){var t=this._capturedHandle===this._$handleStart?"start":"end",n=this.option("start"),i=this.option("end"),o=this.option("max"),a=this.option("min");n=Math.min(Math.max(n,a),o),i=Math.min(Math.max(i,a),o),"start"===t?n=e>i?i:e:i=e<n?n:e,this.option("value",[n,i])},_renderValue:function(){var e=this.option("start"),t=this.option("end"),n=this.option("min"),i=this.option("max"),o=this.option("rtlEnabled");e=Math.max(n,Math.min(e,i)),t=Math.max(e,Math.min(t,i)),this._setOptionWithoutOptionChange("start",e),this._setOptionWithoutOptionChange("end",t),this._setOptionWithoutOptionChange("value",[e,t]),this._$submitStartElement.val((0,c.applyServerDecimalSeparator)(e)),this._$submitEndElement.val((0,c.applyServerDecimalSeparator)(t));var a=i===n?0:(e-n)/(i-n),r=i===n?0:(t-n)/(i-n),l=parseFloat((100*a).toPrecision(12))+"%",u=parseFloat((100*(1-r)).toPrecision(12))+"%";!this._needPreventAnimation&&this._setRangeStyles({right:o?l:u,left:o?u:l}),s.default.getInstance(this._$handleStart).option("value",e),s.default.getInstance(this._$handleEnd).option("value",t)},_callHandlerMethod:function(e,t){s.default.getInstance(this._$handleStart)[e](t),s.default.getInstance(this._$handleEnd)[e](t)},_setValueOption:function(){var e=this.option("start"),t=this.option("end");this.option("value",[e,t])},_optionChanged:function(e){switch(e.name){case"value":if(e.value[0]===e.previousValue[0]&&e.value[1]===e.previousValue[1])break;this._setOptionWithoutOptionChange("start",e.value[0]),this._setOptionWithoutOptionChange("end",e.value[1]),this._renderValue();var t=this.option("start"),n=this.option("end");this._createActionByOption("onValueChanged",{excludeValidators:["disabled","readOnly"]})({start:t,end:n,value:[t,n],event:this._valueChangeEventInstance}),this.validationRequest.fire({value:[t,n],editor:this}),this._saveValueChangeEvent(void 0);break;case"start":case"end":this._setValueOption();break;case"startName":this._$submitStartElement.attr("name",e.value);break;case"endName":this._$submitEndElement.attr("name",e.value);break;case"name":break;default:this.callBase(e)}}});(0,l.default)("dxRangeSlider",_);var m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=i(n(13)),r=i(n(2)),s=n(4),l=n(6),u=n(0),c=n(21),d=n(140),h=i(n(5)),f=i(n(179)),p=i(n(25)),g=n(8),_=n(55),m=i(n(45)),v=n(36),y=i(n(296)),b=n(87),x=i(n(452)),w=n(29),C="dx-slider",k="dx-slider-wrapper",S=".dx-slider-handle",D="dx-slider-bar",I="dx-slider-range",T="dx-slider-range-visible",O="dx-slider-label",E="dx-slider-label-position-",A="dx-slider-tooltip-position-",P="dx-invalid-message-visible",R="Validation",M=y.default.inherit({_activeStateUnit:S,_supportedKeys:function(){var e=this,t=this.option("rtlEnabled"),n=function(t,n){t=e._valueStep(t);var i=e.option("step"),o=e.option("value"),a=o-e.option("min"),r=(0,w.getRemainderByDivision)(a,i,e._getValueExponentLength()),s=n?o-t+(r?i-r:0):o+t-r,l=e.option("min"),u=e.option("max");return s<l?s=l:s>u&&(s=u),e._roundToExponentLength(s)},i=function(i){e.option("value",n(i,t))},o=function(i){e.option("value",n(i,!t))};return(0,u.extend)(this.callBase(),{leftArrow:function(e){this._processKeyboardEvent(e),o(this.option("step"))},rightArrow:function(e){this._processKeyboardEvent(e),i(this.option("step"))},pageUp:function(e){this._processKeyboardEvent(e),i(this.option("step")*this.option("keyStep"))},pageDown:function(e){this._processKeyboardEvent(e),o(this.option("step")*this.option("keyStep"))},home:function(e){this._processKeyboardEvent(e);var t=this.option("min");this.option("value",t)},end:function(e){this._processKeyboardEvent(e);var t=this.option("max");this.option("value",t)}})},_processKeyboardEvent:function(e){e.preventDefault(),e.stopPropagation(),this._saveValueChangeEvent(e)},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{value:50,hoverStateEnabled:!0,activeStateEnabled:!0,step:1,showRange:!0,tooltip:{enabled:!1,format:function(e){return e},position:"top",showMode:"onHover"},label:{visible:!1,position:"bottom",format:function(e){return e}},keyStep:1,useInkRipple:!1,validationMessageOffset:(0,v.isMaterial)()?{h:18,v:0}:{h:7,v:4},focusStateEnabled:!0})},_toggleValidationMessage:function(e){this.option("isValid")||this.$element().toggleClass(P,e)},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===a.default.real().deviceType&&!a.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){var e=(0,v.current)();return(0,v.isMaterial)(e)},options:{useInkRipple:!0}}])},_initMarkup:function(){this.$element().addClass(C),this._renderSubmitElement(),this.option("useInkRipple")&&this._renderInkRipple(),this.callBase(),this._renderLabels(),this._renderStartHandler(),this._renderAriaMinAndMax()},_attachFocusEvents:function(){this.callBase();var e=this.NAME+R,t=(0,g.addNamespace)("focusin",e),n=(0,g.addNamespace)("focusout",e),i=this._focusTarget();h.default.on(i,t,this._toggleValidationMessage.bind(this,!0)),h.default.on(i,n,this._toggleValidationMessage.bind(this,!1))},_detachFocusEvents:function(){this.callBase();var e=this._focusTarget();this._toggleValidationMessage(!1),h.default.off(e,this.NAME+R)},_render:function(){this.callBase(),this._repaintHandle()},_renderSubmitElement:function(){this._$submitElement=(0,r.default)("<input>").attr("type","hidden").appendTo(this.$element())},_getSubmitElement:function(){return this._$submitElement},_renderInkRipple:function(){this._inkRipple=(0,b.render)({waveSizeCoefficient:.7,isCentered:!0,wavesNumber:2,useHoldAnimation:!1})},_renderInkWave:function(e,t,n,i){if(this._inkRipple){var o={element:e,event:t,wave:i};n?this._inkRipple.showWave(o):this._inkRipple.hideWave(o)}},_visibilityChanged:function(){this.repaint()},_renderWrapper:function(){this.callBase(),this._$wrapper.addClass(k),this._createComponent(this._$wrapper,f.default,{elastic:!1,immediate:!0,onStart:this._swipeStartHandler.bind(this),onUpdated:this._swipeUpdateHandler.bind(this),onEnd:this._swipeEndHandler.bind(this),itemSizeFunc:this._itemWidthFunc.bind(this)})},_renderContainer:function(){this.callBase(),this._$bar.addClass(D)},_renderRange:function(){this.callBase(),this._$range.addClass(I),this._renderHandle(),this._renderRangeVisibility()},_renderRangeVisibility:function(){this._$range.toggleClass(T,Boolean(this.option("showRange")))},_renderHandle:function(){this._$handle=this._renderHandleImpl(this.option("value"),this._$handle)},_renderHandleImpl:function(e,t){var n=t||(0,r.default)("<div>").appendTo(this._$range),i=this.option("tooltip.format"),o=this.option("tooltip.enabled"),a=this.option("tooltip.position");return this.$element().toggleClass(A+"bottom",o&&"bottom"===a).toggleClass(A+"top",o&&"top"===a),this._createComponent(n,x.default,{value:e,tooltipEnabled:o,tooltipPosition:a,tooltipFormat:i,tooltipShowMode:this.option("tooltip.showMode"),tooltipFitIn:this.$element()}),n},_renderAriaMinAndMax:function(){this.setAria({valuemin:this.option("min"),valuemax:this.option("max")},this._$handle)},_hoverStartHandler:function(e){x.default.getInstance((0,r.default)(e.currentTarget)).updateTooltip()},_toggleActiveState:function(e,t){this.callBase(e,t),t&&x.default.getInstance(e).updateTooltip(),this._renderInkWave(e,null,!!t,1)},_toggleFocusClass:function(e,t){if(this.callBase(e,t),!this._disposed){var n=(0,r.default)(t||this._focusTarget());this._renderInkWave(n,null,e,0)}},_renderLabels:function(){if(this.$element().removeClass(E+"bottom").removeClass(E+"top"),this.option("label.visible")){var e=this.option("min"),t=this.option("max"),n=this.option("label.position"),i=this.option("label.format");this._$minLabel||(this._$minLabel=(0,r.default)("<div>").addClass(O).appendTo(this._$wrapper)),this._$minLabel.html(m.default.format(e,i)),this._$maxLabel||(this._$maxLabel=(0,r.default)("<div>").addClass(O).appendTo(this._$wrapper)),this._$maxLabel.html(m.default.format(t,i)),this.$element().addClass(E+n)}else this._$minLabel&&(this._$minLabel.remove(),delete this._$minLabel),this._$maxLabel&&(this._$maxLabel.remove(),delete this._$maxLabel)},_renderStartHandler:function(){var e=this,t=(0,g.addNamespace)(p.default.down,this.NAME),n=(0,g.addNamespace)(c.name,this.NAME),i=this._createAction(this._startHandler.bind(this)),o=this.$element();h.default.off(o,t),h.default.on(o,t,function(e){(0,g.isMouseEvent)(e)&&i({event:e})}),h.default.off(o,n),h.default.on(o,n,function(t){var n=e._activeHandle();n&&(h.default.trigger(n,"focusin"),h.default.trigger(n,"focus")),i({event:t})})},_itemWidthFunc:function(){return this._itemWidthRatio},_swipeStartHandler:function(e){var t=this.option("rtlEnabled");(0,g.isTouchEvent)(e.event)&&this._createAction(this._startHandler.bind(this))({event:e.event}),this._feedbackDeferred=new l.Deferred,(0,d.lock)(this._feedbackDeferred),this._toggleActiveState(this._activeHandle(),this.option("activeStateEnabled")),this._startOffset=this._currentRatio;var n=this._startOffset*this._swipePixelRatio(),i=(1-this._startOffset)*this._swipePixelRatio();e.event.maxLeftOffset=t?i:n,e.event.maxRightOffset=t?n:i,this._itemWidthRatio=this.$element().width()/this._swipePixelRatio(),this._needPreventAnimation=!0},_swipeEndHandler:function(e){this._feedbackDeferred.resolve(),this._toggleActiveState(this._activeHandle(),!1);var t=this.option("rtlEnabled")?-1:1;delete this._needPreventAnimation,this._saveValueChangeEvent(e),this._changeValueOnSwipe(this._startOffset+t*e.event.targetOffset/this._swipePixelRatio()),delete this._startOffset,this._renderValue()},_activeHandle:function(){return this._$handle},_swipeUpdateHandler:function(e){this._saveValueChangeEvent(e),this._updateHandlePosition(e)},_updateHandlePosition:function(e){var t=this.option("rtlEnabled")?-1:1,n=Math.min(this._startOffset+t*e.event.offset/this._swipePixelRatio(),1);this._$range.width(100*n+"%"),x.default.getInstance(this._activeHandle()).fitTooltipPosition,this._changeValueOnSwipe(n)},_swipePixelRatio:function(){var e=this.option("min"),t=this.option("max"),n=this._valueStep(this.option("step"));return(t-e)/n},_valueStep:function(e){return e&&!isNaN(e)||(e=1),0===e&&(e=1e-5),e},_getValueExponentLength:function(){var e=this.option(),t=e.step,n=e.min;return Math.max((0,w.getExponentLength)(t),(0,w.getExponentLength)(n))},_roundToExponentLength:function(e){var t=this._getValueExponentLength();return(0,w.roundFloatPart)(e,t)},_changeValueOnSwipe:function(e){var t=this.option("min"),n=this.option("max"),i=this._valueStep(this.option("step")),o=e*(n-t),a=t+o;if(!(i<0))if(a===n||a===t)this._setValueOnSwipe(a);else{var r=Math.round((a-t)/i);a=this._roundToExponentLength(r*i+t),this._setValueOnSwipe(Math.max(Math.min(a,n),t))}},_setValueOnSwipe:function(e){this.option("value",e),this._saveValueChangeEvent(void 0)},_startHandler:function(e){var t=e.event;this._currentRatio=((0,g.eventData)(t).x-this._$bar.offset().left)/this._$bar.width(),this.option("rtlEnabled")&&(this._currentRatio=1-this._currentRatio),this._saveValueChangeEvent(t),this._changeValueOnSwipe(this._currentRatio)},_renderValue:function(){this.callBase();var e=this.option("value");this._getSubmitElement().val((0,s.applyServerDecimalSeparator)(e)),x.default.getInstance(this._activeHandle()).option("value",e)},_setRangeStyles:function(e){e&&this._$range.css(e)},_callHandlerMethod:function(e,t){x.default.getInstance(this._$handle)[e](t)},_repaintHandle:function(){this._callHandlerMethod("repaint")},_fitTooltip:function(){this._callHandlerMethod("fitTooltipPosition")},_optionChanged:function(e){switch(e.name){case"visible":this.callBase(e),this._renderHandle(),this._repaintHandle(),(0,_.triggerShownEvent)(this.$element());break;case"min":case"max":this._renderValue(),this.callBase(e),this._renderLabels(),this._renderAriaMinAndMax(),this._fitTooltip();break;case"step":this._renderValue();break;case"keyStep":break;case"showRange":this._renderRangeVisibility();break;case"tooltip":this._renderHandle();break;case"label":this._renderLabels();break;case"useInkRipple":this._invalidate();break;default:this.callBase(e)}},_refresh:function(){this._toggleRTLDirection(this.option("rtlEnabled")),this._renderDimensions(),this._renderValue(),this._renderHandle(),this._repaintHandle()},_clean:function(){delete this._inkRipple,this.callBase()}});(0,o.default)("dxSlider",M);var B=M;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(680)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,
"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=n(0),y=o(n(35)),b=o(n(49)),x=o(n(16)),w=n(421),C=o(n(164)),k=C.default.getSwatchContainer,S=b.default.add,D=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{icon:"",onClick:null,label:"",visible:!0,index:0,onContentReady:null,activeStateEnabled:!0,hoverStateEnabled:!0,animation:{show:{type:"pop",duration:200,easing:"cubic-bezier(0.4, 0, 0.2, 1)",from:{scale:0,opacity:0},to:{scale:1,opacity:1}},hide:{type:"pop",duration:200,easing:"cubic-bezier(0.4, 0, 0.2, 1)",from:{scale:1,opacity:1},to:{scale:0,opacity:0}}},id:new y.default})}},{key:"_optionChanged",value:function(e){switch(e.name){case"onClick":case"icon":case"label":case"visible":case"index":case"onInitializing":(0,w.initAction)(this);break;case"animation":case"id":break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_render",value:function(){var e=this;this._toggleVisibility(!1),k(this.$element())?(0,w.initAction)(this):S(function(){return(0,w.initAction)(e)})}},{key:"_dispose",value:function(){(0,w.disposeAction)(this._options.silent("id")),l(_(t.prototype),"_dispose",this).call(this)}}]),t}(x.default);(0,m.default)("dxSpeedDialAction",D);var I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(11)),a=i(n(64)),r=(0,a.default)(function(){var e=o.default.getDocument();e.body.insertAdjacentHTML("beforeend","<div style='direction: rtl;\n       position: absolute; left: 0; top: -1; overflow: hidden; width: 1px;\n       height: 1px;'><div style='width: 2px; height: 1px;'></div></div>");var t=e.body.lastElementChild,n=t.scrollLeft>0;t.scrollLeft=-1;var i=t.scrollLeft<0,a={decreasing:i||n,positive:!i};return e.body.removeChild(t),a}),s=r;t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(13)),r=i(n(9)),s=n(40),l=n(3),u=n(1),c=n(0),d=n(7),h=n(18),f=n(4),p=i(n(96)),g=i(n(58)),_="dx-tileview",m="dx-tileview-wrapper",v="dx-tile",y="."+v,b="dxTileData",x={horizontal:{itemMainRatio:"widthRatio",itemCrossRatio:"heightRatio",baseItemMainDimension:"baseItemWidth",baseItemCrossDimension:"baseItemHeight",mainDimension:"width",crossDimension:"height",mainPosition:"left",crossPosition:"top"},vertical:{itemMainRatio:"heightRatio",itemCrossRatio:"widthRatio",baseItemMainDimension:"baseItemHeight",baseItemCrossDimension:"baseItemWidth",mainDimension:"height",crossDimension:"width",mainPosition:"top",crossPosition:"left"}},w=g.default.inherit({_activeStateUnit:y,_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{items:null,direction:"horizontal",hoverStateEnabled:!0,showScrollbar:!1,height:500,baseItemWidth:100,baseItemHeight:100,itemMargin:20,activeStateEnabled:!0,indicateLoading:!0})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return"desktop"===a.default.real().deviceType&&!a.default.isSimulator()},options:{focusStateEnabled:!0}}])},_itemClass:function(){return v},_itemDataKey:function(){return b},_itemContainer:function(){return this._$container},_init:function(){this.callBase(),this.$element().addClass(_),this._initScrollView()},_dataSourceLoadingChangedHandler:function(e){var t=this._scrollView;t&&t.startLoading&&(e&&this.option("indicateLoading")?t.startLoading():t.finishLoading())},_hideLoadingIfLoadIndicationOff:function(){this.option("indicateLoading")||this._dataSourceLoadingChangedHandler(!1)},_initScrollView:function(){this._scrollView=this._createComponent(this.$element(),p.default,{direction:this.option("direction"),scrollByContent:!0,useKeyboard:!1,showScrollbar:this.option("showScrollbar")}),this._$container=(0,o.default)(this._scrollView.content()),this._$container.addClass(m),this._scrollView.option("onUpdated",this._renderGeometry.bind(this))},_initMarkup:function(){this.callBase(),(0,f.deferRender)(function(){this._cellsPerDimension=1,this._renderGeometry(),this._updateScrollView(),this._fireContentReadyAction()}.bind(this))},_updateScrollView:function(){this._scrollView.option("direction",this.option("direction")),this._scrollView.update(),this._indicateLoadingIfAlreadyStarted()},_indicateLoadingIfAlreadyStarted:function(){this._isDataSourceLoading()&&this._dataSourceLoadingChangedHandler(!0)},_renderGeometry:function(){this._config=x[this.option("direction")];var e=this.option("items")||[],t=this._config,n=this.option("itemMargin"),i=Math.max.apply(Math,(0,l.map)(e||[],function(e){return Math.round(e[t.itemCrossRatio]||1)})),o=(0,d.hasWindow)()?this.$element()[t.crossDimension]():parseInt(this.$element().get(0).style[t.crossDimension]);this._cellsPerDimension=Math.floor(o/(this.option(t.baseItemCrossDimension)+n)),this._cellsPerDimension=Math.max(this._cellsPerDimension,i),this._cells=[],this._cells.push(new Array(this._cellsPerDimension)),this._arrangeItems(e),this._renderContentSize(t,n)},_renderContentSize:function(e,t){var n=e.mainDimension,i=e.baseItemMainDimension;if((0,d.hasWindow)()){var o=this._cells.length*this.option(i)+(this._cells.length+1)*t,a=this.$element()[n]();this._$container[n](Math.max(o,a))}},_arrangeItems:function(e){var t=this._config,n=t.itemMainRatio,i=t.itemCrossRatio,o=t.mainPosition;this._itemsPositions=[],(0,l.each)(e,function(e,a){var r={};r[n]=a[n]||1,r[i]=a[i]||1,r.index=e,r[n]=r[n]<=0?0:Math.round(r[t.itemMainRatio]),r[i]=r[i]<=0?0:Math.round(r[t.itemCrossRatio]);var s=this._getItemPosition(r);s[o]===-1&&(s[o]=this._cells.push(new Array(this._cellsPerDimension))-1),this._occupyCells(r,s),this._arrangeItem(r,s),this._itemsPositions.push(s)}.bind(this))},_getItemPosition:function(e){var t=this._config,n=t.mainPosition,i=t.crossPosition,o={};o[n]=-1,o[i]=0;for(var a=0;a<this._cells.length;a++){for(var r=0;r<this._cellsPerDimension;r++)if(this._itemFit(a,r,e)){o[n]=a,o[i]=r;break}if(o[n]>-1)break}return o},_itemFit:function(e,t,n){var i=!0,o=this._config,a=n[o.itemMainRatio],r=n[o.itemCrossRatio];if(t+r>this._cellsPerDimension)return!1;for(var s=e;s<e+a;s++)for(var l=t;l<t+r;l++)if(this._cells.length-1<s)this._cells.push(new Array(this._cellsPerDimension));else if(void 0!==this._cells[s][l]){i=!1;break}return i},_occupyCells:function(e,t){for(var n=this._config,i=t[n.mainPosition],o=t[n.crossPosition],a=e[n.itemMainRatio],r=e[n.itemCrossRatio],s=i;s<i+a;s++)for(var l=o;l<o+r;l++)this._cells[s][l]=e.index},_arrangeItem:function(e,t){var n=this._config,i=t[n.mainPosition],o=t[n.crossPosition],a=e[n.itemMainRatio],r=e[n.itemCrossRatio],s=this.option(n.baseItemCrossDimension),l=this.option(n.baseItemMainDimension),u=this.option("itemMargin"),c={display:a<=0||r<=0?"none":""},d=a*l+(a-1)*u,h=r*s+(r-1)*u;if(c[n.mainDimension]=d<0?0:d,c[n.crossDimension]=h<0?0:h,c[n.mainPosition]=i*l+(i+1)*u,c[n.crossPosition]=o*s+(o+1)*u,this.option("rtlEnabled")){var f=this._$container.width(),p=this.option("baseItemWidth"),g=t.left,_=g*p,m=p+u,v=m*e.widthRatio,y=g*u;c.left=f-(_+v+y)}this._itemElements().eq(e.index).css(c)},_moveFocus:function(e){var t="up",n="down",i=this.option("rtlEnabled")?"right":"left",a=this.option("rtlEnabled")?"left":"right",r="pageup",s="pagedown",l="horizontal"===this.option("direction"),c=this._cells,d=(0,o.default)(this.option("focusedElement")).index(),f=this._itemsPositions[d].left,p=this._itemsPositions[d].top,g=(l?c:c[0]).length,_=(l?c[0]:c).length,m=function(e,t){return l?c[e][t]:c[t][e]};switch(e){case r:case t:for(;p>0&&d===m(f,p);)p--;p<0&&(p=0);break;case s:case n:for(;p<_&&d===m(f,p);)p++;p===_&&(p=_-1);break;case a:for(;f<g&&d===m(f,p);)f++;f===g&&(f=g-1);break;case i:for(;f>=0&&d===m(f,p);)f--;f<0&&(f=0);break;default:return void this.callBase.apply(this,arguments)}var v=m(f,p);if((0,u.isDefined)(v)){var y=this._itemElements().eq(v);this.option("focusedElement",(0,h.getPublicElement)(y)),this._scrollToItem(y)}},_scrollToItem:function(e){if(e.length){var t=this._config,n="outer"+(0,s.captionize)(t.mainDimension),i=this.option("itemMargin"),o=e.position()[t.mainPosition],a=e[n](),r=o+a,l=this.scrollPosition(),u=this.$element()[n]();l<=o&&r<=l+u||(l>o?this._scrollView.scrollTo(o-i):this._scrollView.scrollTo(o+a-u+i))}},_optionChanged:function(e){switch(e.name){case"items":this.callBase(e),this._renderGeometry(),this._updateScrollView();break;case"showScrollbar":this._initScrollView();break;case"disabled":this._scrollView.option("disabled",e.value),this.callBase(e);break;case"baseItemWidth":case"baseItemHeight":case"itemMargin":this._renderGeometry();break;case"width":case"height":this.callBase(e),this._renderGeometry(),this._updateScrollView();break;case"direction":this._renderGeometry(),this._updateScrollView();break;case"indicateLoading":this._hideLoadingIfLoadIndicationOff();break;default:this.callBase(e)}},scrollPosition:function(){return this._scrollView.scrollOffset()[this._config.mainPosition]}});(0,r.default)("dxTileView",w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(684)),l=i(n(685)),u=n(7),c=i(n(17)),d=i(n(300)),h=function(){function e(){var t;o(this,e);var n=(0,u.getWindow)(),i=n&&n.TurndownService||s.default,a=n&&n.showdown||l.default;if(!i)throw c.default.Error("E1041","Turndown");if(!a)throw c.default.Error("E1041","Showdown");this._html2Markdown=new i,null!==(t=this._html2Markdown)&&void 0!==t&&t.addRule&&this._html2Markdown.addRule("emptyLine",{filter:function(e){return"p"===e.nodeName.toLowerCase()&&"<br>"===e.innerHTML},replacement:function(){return"<br>"}}),this._markdown2Html=new a.Converter({simpleLineBreaks:!0,strikethrough:!0})}return r(e,[{key:"toMarkdown",value:function(e){return this._html2Markdown.turndown(e||"")}},{key:"toHtml",value:function(e){var t=this._markdown2Html.makeHtml(e);return t&&(t=t.replace(new RegExp("\\r?\\n","g"),"")),t}}]),e}();d.default.addConverter("markdown",h);var f=h;t.default=f,e.exports=t.default},function(e,t){e.exports=window.TurndownService},function(e,t){e.exports=window.showdown},function(e,t,n){var i=n(262),o=n(575);o.dxAccordion=n(456),o.dxContextMenu=n(110),o.dxDataGrid=n(690),o.dxTreeList=n(506),o.dxMenu=n(487),o.dxPivotGrid=n(769),o.dxPivotGridFieldChooser=n(783),i.PivotGridDataSource=n(304),i.XmlaStore=n(784),o.dxScheduler=n(785),o.dxTreeView=n(167),o.dxFilterBuilder=n(493),o.dxFileManager=n(856),o.dxDiagram=n(876),o.dxGantt=n(894)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=n(4),r=n(3),s=n(1),l=n(0),u=i(n(17)),c=i(n(287)),d=n(12),h=i(n(47)),f=i(n(91)),p=i(n(688)),g="expanded",_="selected",m="disabled",v=o.default.inherit({ctor:function(e){this.options={},(0,l.extend)(this.options,this._defaultOptions(),e),this.options.dataConverter.setDataAccessors(this.options.dataAccessors),this._selectedNodesKeys=[],this._expandedNodesKeys=[],this._dataStructure=[],this._createInternalDataStructure(),this.getTreeNodes()},setOption:function(e,t){this.options[e]=t,"recursiveSelection"===e&&this._updateSelection()},_defaultOptions:function(){return{dataAccessors:void 0,items:[],multipleSelection:!0,recursiveSelection:!1,recursiveExpansion:!1,rootValue:0,searchValue:"",dataType:"tree",searchMode:"contains",dataConverter:new p.default,onNodeChanged:a.noop,sort:null}},_createInternalDataStructure:function(){this._initialDataStructure=this.options.dataConverter.createPlainStructure(this.options.items,this.options.rootValue,this.options.dataType),this._dataStructure=this.options.searchValue.length?this.search(this.options.searchValue):this._initialDataStructure,this.options.dataConverter._dataStructure=this._dataStructure,this._updateSelection(),this._updateExpansion()},_updateSelection:function(){this.options.recursiveSelection&&(this._setChildrenSelection(),this._setParentSelection()),this._selectedNodesKeys=this._updateNodesKeysArray(_)},_updateExpansion:function(e){this.options.recursiveExpansion&&(e?this._updateOneBranch(e):this._setParentExpansion()),this._expandedNodesKeys=this._updateNodesKeysArray(g)},_updateNodesKeysArray:function(e){var t=this,n=[];return(0,r.each)(t._getDataBySelectionMode(),function(i,o){t._isNodeVisible(o)&&o.internalFields[e]&&(e===g||t.options.multipleSelection?n.push(o.internalFields.key):(n.length&&t.toggleSelection(n[0],!1,!0),n=[o.internalFields.key]))}),n},_getDataBySelectionMode:function(){return this.options.multipleSelection?this.getData():this.getFullData()},_isNodeVisible:function(e){return e.internalFields.item.visible!==!1},_getByKey:function(e,t){return e===this._dataStructure?this.options.dataConverter._getByKey(t):this.options.dataConverter.getByKey(e,t)},_setChildrenSelection:function(){var e=this;(0,r.each)(this._dataStructure,function(t,n){if(n.internalFields.childrenKeys.length){var i=n.internalFields.selected;i===!0&&e._toggleChildrenSelection(n,i)}})},_setParentSelection:function(){var e=this;(0,r.each)(this._dataStructure,function(t,n){var i=e.options.dataConverter.getParentNode(n);i&&n.internalFields.parentKey!==e.options.rootValue&&e._iterateParents(n,function(t){var n=e._calculateSelectedState(t);e._setFieldState(t,_,n)})})},_setParentExpansion:function(){var e=this;(0,r.each)(this._dataStructure,function(t,n){n.internalFields.expanded&&e._updateOneBranch(n.internalFields.key)})},_updateOneBranch:function(e){var t=this,n=this.getNodeByKey(e);t._iterateParents(n,function(e){t._setFieldState(e,g,!0)})},_iterateChildren:function(e,t,n,i){if((0,s.isFunction)(n)){var o=this,a=e.internalFields.key;i=i||[],i.indexOf(a)===-1&&(i.push(a),(0,r.each)(e.internalFields.childrenKeys,function(e,a){var r=o.getNodeByKey(a);n(r),r.internalFields.childrenKeys.length&&t&&o._iterateChildren(r,t,n,i)}))}},_iterateParents:function(e,t,n){if(e.internalFields.parentKey!==this.options.rootValue&&(0,s.isFunction)(t)){n=n||[];var i=e.internalFields.key;if(n.indexOf(i)===-1){n.push(i);var o=this.options.dataConverter.getParentNode(e);o&&(t(o),o.internalFields.parentKey!==this.options.rootValue&&this._iterateParents(o,t,n))}}},_calculateSelectedState:function(e){for(var t=e.internalFields.childrenKeys.length,n=0,i=0,o=!1,a=0;a<=t-1;a++){var r=this.getNodeByKey(e.internalFields.childrenKeys[a]),s=r.internalFields.item.visible===!1,l=r.internalFields.selected;s?i++:l?n++:void 0===l&&(n+=.5)}return n&&(o=n===t-i||void 0),o},_toggleChildrenSelection:function(e,t){var n=this;this._iterateChildren(e,!0,function(e){n._isNodeVisible(e)&&n._setFieldState(e,_,t)})},_setFieldState:function(e,t,n){e.internalFields[t]!==n&&(e.internalFields[t]=n,e.internalFields.publicNode&&(e.internalFields.publicNode[t]=n),this.options.dataAccessors.setters[t](e.internalFields.item,n),this.options.onNodeChanged(e))},_markChildren:function(e){var t=this;(0,r.each)(e,function(e,n){var i=t.getIndexByKey(n),o=t.getNodeByKey(n);t._dataStructure[i]=0,o.internalFields.childrenKeys.length&&t._markChildren(o.internalFields.childrenKeys)})},_removeNode:function(e){var t=this.getNodeByKey(e);this._dataStructure[this.getIndexByKey(e)]=0,this._markChildren(t.internalFields.childrenKeys);var n=this,i=0,o=(0,l.extend)([],this._dataStructure);(0,r.each)(o,function(e,t){t||(n._dataStructure.splice(e-i,1),i++)})},_addNode:function(e){var t=this.options.dataConverter,n=t._convertItemToNode(e,this.options.dataAccessors.getters.parentKey(e));this._dataStructure=this._dataStructure.concat(n),this._initialDataStructure=this._initialDataStructure.concat(n),t._dataStructure=t._dataStructure.concat(n)},_updateFields:function(){this.options.dataConverter.updateChildrenKeys(),this._updateSelection(),this._updateExpansion()},getSelectedNodesKeys:function(){return this._selectedNodesKeys},getExpandedNodesKeys:function(){return this._expandedNodesKeys},getData:function(){return this._dataStructure},getFullData:function(){return this._initialDataStructure},getNodeByItem:function(e){var t=null;return(0,r.each)(this._dataStructure,function(n,i){if(i.internalFields.item===e)return t=i,!1}),t},getNodesByItems:function(e){var t=this,n=[];return(0,r.each)(e,function(e,i){var o=t.getNodeByItem(i);o&&n.push(o)}),n},getNodeByKey:function(e,t){return this._getByKey(t||this._getDataBySelectionMode(),e)},getTreeNodes:function(){return this.options.dataConverter.convertToPublicNodes(this.getRootNodes())},getItemsCount:function(){return this.options.dataConverter.getItemsCount()},getVisibleItemsCount:function(){return this.options.dataConverter.getVisibleItemsCount()},getPublicNode:function(e){return e.internalFields.publicNode},getRootNodes:function(){return this.getChildrenNodes(this.options.rootValue)},getChildrenNodes:function(e){return(0,h.default)(this._dataStructure).filter(["internalFields.parentKey",e]).toArray()},getIndexByKey:function(e){return this.options.dataConverter.getIndexByKey(e)},addItem:function(e){this._addNode(e),this._updateFields()},removeItem:function(e){this._removeNode(e),this._updateFields()},toggleSelection:function(e,t,n){var i=this._isSingleModeUnselect(t),o=this._getByKey(n||i?this._initialDataStructure:this._dataStructure,e);this._setFieldState(o,_,t),this.options.recursiveSelection&&!n&&(t?this._setChildrenSelection():this._toggleChildrenSelection(o,t),this._setParentSelection()),this._selectedNodesKeys=this._updateNodesKeysArray(_)},_isSingleModeUnselect:function(e){return!this.options.multipleSelection&&!e},toggleNodeDisabledState:function(e,t){var n=this.getNodeByKey(e);this._setFieldState(n,m,t)},toggleSelectAll:function(e){if((0,s.isDefined)(e)){var t=this,n=t._selectedNodesKeys[t._selectedNodesKeys.length-1],i=t._isSingleModeUnselect(e)?this._initialDataStructure:this._dataStructure;(0,r.each)(i,function(n,i){t._isNodeVisible(i)&&t._setFieldState(i,_,e)}),t._selectedNodesKeys=t._updateNodesKeysArray(_),!e&&t.options.selectionRequired&&t.toggleSelection(n,!0)}},isAllSelected:function(){return!!this.getSelectedNodesKeys().length&&(this.getSelectedNodesKeys().length===this.getVisibleItemsCount()||void 0)},toggleExpansion:function(e,t){var n=this.getNodeByKey(e);this._setFieldState(n,g,t),t&&this._updateExpansion(e),this._expandedNodesKeys=this._updateNodesKeysArray(g)},isFiltered:function(e){return!this.options.searchValue.length||!!this._filterDataStructure(this.options.searchValue,[e]).length},_createCriteria:function(e,t,n){var i=[];return Array.isArray(e)?((0,r.each)(e,function(e,o){i.push([o,n,t],"or")}),i.pop(),i):[e,n,t]},_filterDataStructure:function(e,t){var n=this.options.searchExpr||this.options.dataAccessors.getters.display,i=c.default.getOperationBySearchMode(this.options.searchMode),o=this._createCriteria(n,e,i);return t=t||this._initialDataStructure,(0,h.default)(t).filter(o).toArray()},search:function(e){function t(e,i){for(var a=e.length;i<a;){var r=e[i];if(r.internalFields.parentKey!==n.options.rootValue){var s=o.getParentNode(r);s?(s.internalFields.expanded||n._setFieldState(s,g,!0),(0,d.inArray)(s,e)>-1?i++:(e.splice(i,0,s),t(e,i))):(u.default.log("W1007",r.internalFields.parentKey,r.internalFields.key),i++)}else i++}}var n=this,i=this._filterDataStructure(e),o=this.options.dataConverter;return t(i,0),this.options.sort&&(i=f.default.queryByOptions((0,h.default)(i),{sort:this.options.sort}).toArray()),o._indexByKey={},(0,r.each)(i,function(e,t){t.internalFields.childrenKeys=[],o._indexByKey[t.internalFields.key]=e}),o._dataStructure=i,o.setChildrenKeys(),o._dataStructure}}),y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(15)),a=n(0),r=i(n(17)),s=n(3),l=n(1),u=o.default.inherit({ctor:function(){this._dataStructure=[],this._itemsCount=0,this._visibleItemsCount=0},_indexByKey:{},_convertItemsToNodes:function(e,t){var n=this;(0,s.each)(e,function(e,i){var o=(0,l.isDefined)(t)?t:n._getParentId(i),a=n._convertItemToNode(i,o);n._dataStructure.push(a),n._checkForDuplicateId(a.internalFields.key),n._indexByKey[a.internalFields.key]=n._dataStructure.length-1,n._itemHasChildren(i)&&n._convertItemsToNodes(n._dataAccessors.getters.items(i),a.internalFields.key)})},_checkForDuplicateId:function(e){if((0,l.isDefined)(this._indexByKey[e]))throw r.default.Error("E1040",e)},_getParentId:function(e){return"plain"===this._dataType?this._dataAccessors.getters.parentKey(e):void 0},_itemHasChildren:function(e){if("plain"!==this._dataType){var t=this._dataAccessors.getters.items(e);return t&&t.length}},_getUniqueKey:function(e){var t=this._dataAccessors.getters.key,n=t(e),i=t&&(n||0===n)&&(0,l.isPrimitive)(n);return i?n:this.getItemsCount()},_convertItemToNode:function(e,t){this._itemsCount++,e.visible!==!1&&this._visibleItemsCount++;var n=this,i={internalFields:{disabled:n._dataAccessors.getters.disabled(e,{defaultValue:!1}),expanded:n._dataAccessors.getters.expanded(e,{defaultValue:!1}),selected:n._dataAccessors.getters.selected(e,{defaultValue:!1}),key:n._getUniqueKey(e),parentKey:(0,l.isDefined)(t)?t:n._rootValue,item:n._makeObjectFromPrimitive(e),childrenKeys:[]}};return(0,a.extend)(i,e),delete i.items,i},setChildrenKeys:function(){var e=this;(0,s.each)(this._dataStructure,function(t,n){if(n.internalFields.parentKey!==e._rootValue){var i=e.getParentNode(n);i&&i.internalFields.childrenKeys.push(n.internalFields.key)}})},_makeObjectFromPrimitive:function(e){if((0,l.isPrimitive)(e)){var t=e;e={},this._dataAccessors.setters.key(e,t)}return e},_convertToPublicNode:function(e,t){if(!e)return null;var n={text:this._dataAccessors.getters.display(e),key:e.internalFields.key,selected:e.internalFields.selected,expanded:e.internalFields.expanded,disabled:e.internalFields.disabled,parent:t||null,itemData:e.internalFields.item,children:[],items:[]};return n.parent&&(n.parent.children.push(n),n.parent.items.push(n)),n},convertToPublicNodes:function(e,t){if(!e.length)return[];var n=this,i=[];return(0,s.each)(e,function(e,o){o=(0,l.isPrimitive)(o)?n._getByKey(o):o;var a=n._convertToPublicNode(o,t);a.children=n.convertToPublicNodes(o.internalFields.childrenKeys,a),i.push(a),o.internalFields.publicNode=a}),i},setDataAccessors:function(e){this._dataAccessors=e},_getByKey:function(e){return this._dataStructure[this.getIndexByKey(e)]||null},getParentNode:function(e){return this._getByKey(e.internalFields.parentKey)},getByKey:function e(t,n){if(null===n||void 0===n)return null;var i=null,o=this,e=function(e,t){return(0,s.each)(e,function(e,n){var a=n.internalFields&&n.internalFields.key||o._dataAccessors.getters.key(n);if(a.toString()===t.toString())return i=n,!1}),i};return e(t,n)},getItemsCount:function(){return this._itemsCount},getVisibleItemsCount:function(){return this._visibleItemsCount},updateIndexByKey:function(){var e=this;this._indexByKey={},(0,s.each)(this._dataStructure,function(t,n){e._checkForDuplicateId(n.internalFields.key),e._indexByKey[n.internalFields.key]=t})},updateChildrenKeys:function(){this._indexByKey={},this.removeChildrenKeys(),this.updateIndexByKey(),this.setChildrenKeys()},removeChildrenKeys:function(){this._indexByKey={},(0,s.each)(this._dataStructure,function(e,t){t.internalFields.childrenKeys=[]})},getIndexByKey:function(e){return this._indexByKey[e]},createPlainStructure:function(e,t,n){return this._itemsCount=0,this._visibleItemsCount=0,this._rootValue=t,this._dataType=n,this._indexByKey={},this._convertItemsToNodes(e),this.setChildrenKeys(),this._dataStructure}}),c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==o(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=i(n(2)),_=n(3),m=i(n(207)),v=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getPlainItems",value:function(){return(0,_.map)(this._collectionWidget.option("items"),function e(t){return t.items?[t].concat((0,_.map)(t.items,e)):t})}},{key:"_stringifyItem",value:function(e){var t=this;return JSON.stringify(e,function(e,n){return"template"===e?t._getTemplateString(n):n})}},{key:"_getTemplateString",value:function(e){var t;return t="object"===o(e)?(0,g.default)(e).text():e.toString()}}]),t}(m.default),y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(460)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(9)),r=n(4),s=n(1),l=n(3),u=n(0),c=n(106),d=i(n(23)),h=i(n(16)),f=i(n(26)),p=n(36);n(692),n(693),n(694),n(695),n(696),n(697),n(698),n(699),n(700);var g=".dx-row",_="Specifying grid templates with the jQuery selector name is now deprecated. Use the DOM Node or the jQuery object that references this selector instead.";f.default.registerModulesOrder(["stateStoring","columns","selection","editorFactory","columnChooser","grouping","editing","masterDetail","validating","adaptivity","data","virtualScrolling","columnHeaders","filterRow","headerPanel","headerFilter","sorting","search","rows","pager","columnsResizingReordering","contextMenu","keyboardNavigation","errorHandling","summary","columnFixing","export","gridView"]);var m=h.default.inherit({_activeStateUnit:g,_getDefaultOptions:function(){var e=this,t=e.callBase();return(0,l.each)(f.default.modules,function(){(0,s.isFunction)(this.defaultOptions)&&(0,u.extend)(!0,t,this.defaultOptions())}),t},_setDeprecatedOptions:function(){this.callBase(),(0,u.extend)(this._deprecatedOptions,{useKeyboard:{since:"19.2",alias:"keyboardNavigation.enabled"}})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{platform:"ios"},options:{showRowLines:!0}},{device:function(){return(0,p.isMaterial)()},options:{showRowLines:!0,showColumnLines:!1,headerFilter:{height:315},editing:{useIcons:!0}}},{device:function(){return d.default.webkit},options:{loadingTimeout:30,loadPanel:{animation:{show:{easing:"cubic-bezier(1, 0, 1, 0)",duration:500,from:{opacity:0},to:{opacity:1}}}}}},{device:function(e){return"desktop"!==e.deviceType},options:{grouping:{expandMode:"rowClick"}}}])},_init:function(){var e=this;e.callBase(),f.default.processModules(e,f.default),f.default.callModuleItemsMethod(e,"init")},_clean:r.noop,_optionChanged:function(e){var t=this;f.default.callModuleItemsMethod(t,"optionChanged",[e]),e.handled||t.callBase(e)},_dimensionChanged:function(){this.updateDimensions(!0)},_visibilityChanged:function(e){e&&this.updateDimensions()},_initMarkup:function(){this.callBase.apply(this,arguments),this.getView("gridView").render(this.$element())},_renderContentImpl:function(){this.getView("gridView").update()},_renderContent:function(){var e=this;(0,r.deferRender)(function(){e._renderContentImpl()})},_getTemplate:function(e){var t=e;return(0,s.isString)(t)&&"#"===t[0]&&(t=(0,o.default)(e),c.logger.warn(_)),this.callBase(t)},_dispose:function(){var e=this;e.callBase(),f.default.callModuleItemsMethod(e,"dispose")},isReady:function(){return this.getController("data").isReady()},beginUpdate:function(){var e=this;e.callBase(),f.default.callModuleItemsMethod(e,"beginUpdate")},endUpdate:function(){var e=this;f.default.callModuleItemsMethod(e,"endUpdate"),e.callBase()},getController:function(e){return this._controllers[e]},getView:function(e){return this._views[e]},focus:function(e){this.getController("keyboardNavigation").focus(e)}});m.registerModule=f.default.registerModule.bind(f.default),(0,a.default)("dxDataGrid",m);var v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.ColumnHeadersView=void 0;var o=i(n(26)),a=i(n(461)),r=a.default.views.columnHeadersView;t.ColumnHeadersView=r,o.default.registerModule("columnHeaders",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(463)),r=n(0);o.default.registerModule("columns",{defaultOptions:function(){return(0,r.extend)(!0,{},a.default.defaultOptions(),{commonColumnSettings:{allowExporting:!0}})},controllers:a.default.controllers})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.DataController=void 0;var o=i(n(26)),a=i(n(17)),r=i(n(222)),s=i(n(465)),l=s.default.controllers.data.inherit(function(){return{_getDataSourceAdapter:function(){return r.default},_getSpecificDataSourceOption:function(){var e=this.option("dataSource");return e&&!Array.isArray(e)&&this.option("keyExpr")&&a.default.log("W1011"),this.callBase()}}}());t.DataController=l,o.default.registerModule("data",{defaultOptions:s.default.defaultOptions,controllers:{data:l}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{
"default":e}}var o=i(n(26)),a=i(n(466));o.default.registerModule("sorting",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.RowsView=void 0;var o=i(n(26)),a=i(n(468)),r=a.default.views.rowsView;t.RowsView=r,o.default.registerModule("rows",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(469));o.default.registerModule("contextMenu",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(470));o.default.registerModule("errorHandling",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(471));o.default.registerModule("gridView",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.HeaderPanel=void 0;var o=i(n(26)),a=i(n(472)),r=a.default.views.headerPanel;t.HeaderPanel=r,o.default.registerModule("headerPanel",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(473));o.default.registerModule("stateStoring",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(475));o.default.registerModule("selection",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.ColumnChooserView=t.ColumnChooserController=void 0;var o=i(n(26)),a=i(n(476)),r=a.default.controllers.columnChooser;t.ColumnChooserController=r;var s=a.default.views.columnChooserView;t.ColumnChooserView=s,o.default.registerModule("columnChooser",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=i(n(10)),l=n(21),u=n(4),c=n(7),d=n(1),h=n(0),f=n(3),p=n(18),g=i(n(144)),_=i(n(459)),m=n(8),v=i(n(25)),y=n(142),b=i(n(43)),x=i(n(109)),w=i(n(88)),C=n(6),k=i(n(17)),S="dx-treeview",D="".concat(S,"-node"),I="".concat(D,"-container"),T="".concat(D,"-loadindicator"),O="".concat(D,"-container-opened"),E="".concat(D,"-is-leaf"),A="".concat(S,"-item"),P="".concat(A,"-with-checkbox"),R="".concat(A,"-without-checkbox"),M="".concat(A,"-data"),B="".concat(S,"-toggle-item-visibility"),V="".concat(S,"-loadindicator"),F="".concat(S,"-loadindicator-wrapper"),L="".concat(S,"-toggle-item-visibility-opened"),H="".concat(S,"-select-all-item"),z="dx-state-invisible",N="dx-state-disabled",j="dx-state-selected",W="dxTreeView_expand",G="data-item-id",q=_.default.inherit({_supportedKeys:function(e){var t=this,n=function(e){var n=(0,o.default)(t.option("focusedElement"));if(n.length){e.target=n,e.currentTarget=n,t._itemClickHandler(e,n.children("."+A));var i=t._getEventNameByOption(t.option("expandEvent")),a=i===(0,m.addNamespace)(l.name,W);a&&t._expandEventHandler(e)}},i=function(e){e.preventDefault(),t._changeCheckBoxState((0,o.default)(t.option("focusedElement")))},a=function(e,t){if(this.option("expandAllEnabled")){t.preventDefault();var n=(0,o.default)(this.option("focusedElement"));if(n.length){var i=this._getItemData(n.find(".".concat(A)));this._toggleExpandedNestedItems([i],e)}}};return(0,h.extend)(this.callBase(),{enter:this._showCheckboxes()?i:n,space:this._showCheckboxes()?i:n,asterisk:a.bind(this,!0),minus:a.bind(this,!1)})},_changeCheckBoxState:function(e){var t=this._getCheckBoxInstance(e),n=t.option("value");t.option("disabled")||this._updateItemSelection(!n,e.find("."+A).get(0),!0,e)},_toggleExpandedNestedItems:function(e,t){if(e)for(var n=0,i=e.length;n<i;n++){var o=e[n],a=this._dataAdapter.getNodeByItem(o);this._toggleExpandedState(a,t),this._toggleExpandedNestedItems(o.items,t)}},_getNodeElement:function(e,t){var n=this._encodeString(e.internalFields.key);if(t)return t.$nodeByKey||(t.$nodeByKey={},this.$element().find(".".concat(D)).each(function(){var e=(0,o.default)(this),n=e.attr(G);t.$nodeByKey[n]=e})),t.$nodeByKey[n]||(0,o.default)();var i=this.$element().get(0).querySelector("[".concat(G,'="').concat(n,'"]'));return(0,o.default)(i)},_activeStateUnit:"."+A,_widgetClass:function(){return S},_getDefaultOptions:function(){var e=(0,h.extend)(this.callBase(),{animationEnabled:!0,dataStructure:"tree",deferRendering:!0,expandAllEnabled:!1,hasItemsExpr:"hasItems",selectNodesRecursive:!0,expandNodesRecursive:!0,showCheckBoxesMode:"none",selectAllText:s.default.format("dxList-selectAll"),onItemSelectionChanged:null,onItemExpanded:null,onItemCollapsed:null,scrollDirection:"vertical",virtualModeEnabled:!1,rootValue:0,focusStateEnabled:!1,selectionMode:"multiple",expandEvent:"dblclick",selectByClick:!1,createChildren:null,onSelectAllValueChanged:null});return(0,h.extend)(!0,e,{integrationOptions:{useDeferUpdateForTemplates:!1}})},_initSelectedItems:u.noop,_syncSelectionOptions:u.asyncNoop,_fireSelectionChanged:function(){var e=this._selectionChangePromise;(0,C.when)(e).done(function(){this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})()}.bind(this))},_createSelectAllValueChangedAction:function(){this._selectAllValueChangedAction=this._createActionByOption("onSelectAllValueChanged",{excludeValidators:["disabled","readOnly"]})},_fireSelectAllValueChanged:function(e){this._selectAllValueChangedAction({value:e})},_checkBoxModeChange:function(e,t){if("none"===t||"none"===e)return void this.repaint();var n=this._$selectAllItem&&this._$selectAllItem.length;switch(e){case"selectAll":!n&&this._renderSelectAllItem();break;case"normal":n&&(this._$selectAllItem.remove(),delete this._$selectAllItem)}},_removeSelection:function(){var e=this;(0,f.each)(this._dataAdapter.getFullData(),function(t,n){e._hasChildren(n)&&e._dataAdapter.toggleSelection(n.internalFields.key,!1,!0)})},_optionChanged:function(e){var t=e.name,n=e.value,i=e.previousValue;switch(t){case"selectAllText":this._$selectAllItem&&this._$selectAllItem.dxCheckBox("instance").option("text",n);break;case"showCheckBoxesMode":this._checkBoxModeChange(n,i);break;case"scrollDirection":this._scrollableContainer.option("direction",n);break;case"items":delete this._$selectAllItem,this.callBase(e);break;case"dataSource":this.callBase(e),this._initDataAdapter(),this._filter={};break;case"hasItemsExpr":this._initAccessors(),this.repaint();break;case"expandEvent":this._initExpandEvent();break;case"deferRendering":case"dataStructure":case"rootValue":case"createChildren":case"expandNodesRecursive":case"onItemSelectionChanged":case"onItemExpanded":case"onItemCollapsed":case"expandAllEnabled":case"animationEnabled":case"virtualModeEnabled":case"selectByClick":break;case"selectionMode":this._initDataAdapter(),this.callBase(e);break;case"onSelectAllValueChanged":this._createSelectAllValueChangedAction();break;case"selectNodesRecursive":this._dataAdapter.setOption("recursiveSelection",e.value),this.repaint();break;default:this.callBase(e)}},_initDataSource:function(){this._useCustomChildrenLoader()?this._loadChildrenByCustomLoader(null).done(function(e){e&&e.length&&this.option("items",e)}.bind(this)):(this.callBase(),this._isVirtualMode()&&this._initVirtualMode())},_initVirtualMode:function(){var e=this._filter;e.custom||(e.custom=this._dataSource.filter()),e.internal||(e.internal=[this.option("parentIdExpr"),this.option("rootValue")])},_useCustomChildrenLoader:function(){return(0,d.isFunction)(this.option("createChildren"))&&this._isDataStructurePlain()},_loadChildrenByCustomLoader:function(e){var t=this.option("createChildren").call(this,e);return Array.isArray(t)?(new C.Deferred).resolve(t).promise():t&&(0,d.isFunction)(t.then)?(0,C.fromPromise)(t):(new C.Deferred).resolve([]).promise()},_combineFilter:function(){return this._filter.custom&&this._filter.custom.length?[this._filter.custom,this._filter.internal]:this._filter.internal},_dataSourceLoadErrorHandler:function(){this._renderEmptyMessage()},_init:function(){this._filter={},this.callBase(),this._initStoreChangeHandlers()},_dataSourceChangedHandler:function(e){var t=this.option("items");this._initialized&&this._isVirtualMode()&&t.length||this.option("items",e)},_removeTreeViewLoadIndicator:function(){this._treeViewLoadIndicator&&(this._treeViewLoadIndicator.remove(),this._treeViewLoadIndicator=null)},_createTreeViewLoadIndicator:function(){return this._treeViewLoadIndicator=(0,o.default)("<div>").addClass(V),this._createComponent(this._treeViewLoadIndicator,w.default,{}),this._treeViewLoadIndicator},_dataSourceLoadingChangedHandler:function(e){var t;if(this._isVirtualMode()&&(t=this._combineFilter(),this._dataSource.filter(t)),e&&!this._dataSource.isLoaded()){this.option("items",[]);var n=(0,o.default)("<div>").addClass(F);this._createTreeViewLoadIndicator().appendTo(n),this.itemsContainer().append(n),this._isVirtualMode()&&this._dataSource.filter()!==t&&this._dataSource.filter([])}else this._removeTreeViewLoadIndicator()},_initStoreChangeHandlers:function(){var e=this;"plain"===this.option("dataStructure")&&this._dataSource&&this._dataSource.store().on("inserted",function(t){e.option().items=e.option("items").concat(t),e._dataAdapter.addItem(t),e._dataAdapter.isFiltered(t)&&e._updateLevel(e._parentIdGetter(t))}).on("removed",function(t){var n=e._dataAdapter.getNodeByKey(t);(0,d.isDefined)(n)&&(e.option("items")[e._dataAdapter.getIndexByKey(n.internalFields.key)]=0,e._markChildrenItemsToRemove(n),e._removeItems(),e._dataAdapter.removeItem(t),e._updateLevel(e._parentIdGetter(n)))})},_markChildrenItemsToRemove:function(e){var t=this,n=e.internalFields.childrenKeys;(0,f.each)(n,function(e,n){t.option("items")[t._dataAdapter.getIndexByKey(n)]=0,t._markChildrenItemsToRemove(t._dataAdapter.getNodeByKey(n))})},_removeItems:function(){var e=this,t=(0,h.extend)(!0,[],this.option("items")),n=0;(0,f.each)(t,function(t,i){i||(e.option("items").splice(t-n,1),n++)})},_updateLevel:function(e){var t=this._getContainerByParentKey(e);this._renderItems(t,this._dataAdapter.getChildrenNodes(e))},_getOldContainer:function(e){return e.length?e.children(".".concat(I)):this._scrollableContainer?this._scrollableContainer.$content().children():(0,o.default)()},_getContainerByParentKey:function(e){var t=this._dataAdapter.getNodeByKey(e),n=t?this._getNodeElement(t):[];this._getOldContainer(n).remove();var i=this._renderNodeContainer(n);return this._isRootLevel(e)&&(this._scrollableContainer||this._renderScrollableContainer(),this._scrollableContainer.$content().append(i)),i},_isRootLevel:function(e){return e===this.option("rootValue")},_getAccessors:function(){var e=this.callBase();return e.push("hasItems"),e},_getDataAdapterOptions:function(){return{rootValue:this.option("rootValue"),multipleSelection:!this._isSingleSelection(),recursiveSelection:this._isRecursiveSelection(),recursiveExpansion:this.option("expandNodesRecursive"),selectionRequired:this.option("selectionRequired"),dataType:this.option("dataStructure"),sort:this._dataSource&&this._dataSource.sort()}},_initMarkup:function(){this._renderScrollableContainer(),this._renderEmptyMessage(this._dataAdapter.getRootNodes()),this.callBase(),this.setAria("role","tree")},_renderContentImpl:function(){var e=this._renderNodeContainer();this._scrollableContainer.$content().append(e),this.option("items")&&this.option("items").length&&(this._renderItems(e,this._dataAdapter.getRootNodes()),this._initExpandEvent(),this._selectAllEnabled()&&(this._createSelectAllValueChangedAction(),this._renderSelectAllItem(e)))},_isVirtualMode:function(){return this.option("virtualModeEnabled")&&this._isDataStructurePlain()&&!!this.option("dataSource")},_isDataStructurePlain:function(){return"plain"===this.option("dataStructure")},_fireContentReadyAction:function(){var e=this.getDataSource(),t=e&&!e.isLoaded();this._scrollableContainer&&(0,c.hasWindow)()&&this._scrollableContainer.update(),t||this.callBase(),this._scrollableContainer&&(0,c.hasWindow)()&&this._scrollableContainer.update()},_renderScrollableContainer:function(){this._scrollableContainer=this._createComponent((0,o.default)("<div>").appendTo(this.$element()),x.default,{direction:this.option("scrollDirection"),useKeyboard:!1})},_renderNodeContainer:function(e){var t=(0,o.default)("<ul>").addClass(I);if(this.setAria("role","group",t),e&&e.length){var n=this._getItemData(e.children("."+A));this._expandedGetter(n)&&t.addClass(O),t.appendTo(e)}return t},_createDOMElement:function(e,t){var n=(0,o.default)("<li>").addClass(D).attr(G,this._encodeString(t.internalFields.key)).prependTo(e);return this.setAria({role:"treeitem",label:this._displayGetter(t.internalFields.item)||"",expanded:t.internalFields.expanded||!1,level:this._getLevel(e)},n),n},_getLevel:function(e){var t=e.parent();return t.hasClass("dx-scrollable-content")?1:parseInt(t.attr("aria-level"))+1},_showCheckboxes:function(){return"none"!==this.option("showCheckBoxesMode")},_selectAllEnabled:function(){return"selectAll"===this.option("showCheckBoxesMode")&&!this._isSingleSelection()},_renderItems:function(e,t){for(var n=t.length-1,i=n;i>=0;i--)this._renderItem(i,t[i],e);this._renderedItemsCount+=t.length},_renderItem:function(e,t,n){var i=this._createDOMElement(n,t),o=t.internalFields,a=this._showCheckboxes();i.addClass(a?P:R),i.toggleClass(z,o.item.visible===!1),a&&this._renderCheckBox(i,t),this.setAria("selected",o.selected,i),this._toggleSelectedClass(i,o.selected),this.callBase(this._renderedItemsCount+e,o.item,i),o.item.visible!==!1&&this._renderChildren(i,t)},_setAriaSelected:function(){},_renderChildren:function(e,t){var n=this;return this._hasChildren(t)?(this._renderToggleItemVisibilityIcon(e,t),void(this.option("deferRendering")&&!t.internalFields.expanded||this._loadSublevel(t).done(function(i){n._renderSublevel(e,n._getActualNode(t),i)}))):void this._addLeafClass(e)},_getActualNode:function(e){return this._dataAdapter.getNodeByKey(e.internalFields.key)},_hasChildren:function(e){return this._isVirtualMode()||this._useCustomChildrenLoader()?this._hasItemsGetter(e.internalFields.item)!==!1:this.callBase(e)},_loadSublevel:function(e){var t=this,n=new C.Deferred,i=this._getChildNodes(e);return i.length?n.resolve(i):this._loadNestedItems(e).done(function(e){n.resolve(t._dataAdapter.getNodesByItems(e))}),n.promise()},_renderSublevel:function(e,t,n){var i=this._renderNodeContainer(e,t),o=n.filter(function(e){return t.internalFields.childrenKeys.indexOf(e.internalFields.key)!==-1});if(this._renderItems(i,o),o.length&&!t.internalFields.selected){var a=o[0];this._updateParentsState(a,this._getNodeElement(a))}this._normalizeIconState(e,o.length),t.internalFields.expanded&&i.addClass(O)},_executeItemRenderAction:function(e,t,n){var i=this._getNode(n);this._getItemRenderAction()({itemElement:n,itemIndex:e,itemData:t,node:this._dataAdapter.getPublicNode(i)})},_addLeafClass:function(e){e.addClass(E)},_expandEventHandler:function(e){var t=(0,o.default)(e.currentTarget.parentNode);t.hasClass(E)||this._toggleExpandedState(e.currentTarget,void 0,e)},_initExpandEvent:function(){var e=this._getEventNameByOption(this.option("expandEvent")),t=this._itemContainer(),n=this._itemSelector();r.default.off(t,"."+W,n),r.default.on(t,e,n,this._expandEventHandler.bind(this))},_getEventNameByOption:function(e){var t="click"===e?l.name:y.name;return(0,m.addNamespace)(t,W)},_getNode:function(e){if(!(0,d.isDefined)(e))return null;if(e.internalFields)return e;if((0,d.isPrimitive)(e))return this._dataAdapter.getNodeByKey(e);var t=(0,o.default)(e).get(0);return t?a.default.isElementNode(t)?this._getNodeByElement(t):this._dataAdapter.getNodeByItem(t):null},_getNodeByElement:function(e){var t=(0,o.default)(e).closest("."+D),n=this._decodeString(t.attr(G));return this._dataAdapter.getNodeByKey(n)},_toggleExpandedState:function(e,t,n){var i=this._getNode(e);if(!i)return(new C.Deferred).reject().promise();if(i.internalFields.disabled)return(new C.Deferred).reject().promise();var o=i.internalFields.expanded;if(o===t)return(new C.Deferred).resolve().promise();if(this._hasChildren(i)){var a=this._getNodeElement(i);if(a.find(".".concat(T,":not(.").concat(z,")")).length)return(new C.Deferred).reject().promise();this._createLoadIndicator(a)}return(0,d.isDefined)(t)||(t=!o),this._dataAdapter.toggleExpansion(i.internalFields.key,t),this._updateExpandedItemsUI(i,t,n)},_createLoadIndicator:function(e){var t=e.children("."+B),n=e.children(".".concat(I));t.hasClass(L)||n.not(":empty").length||(this._createComponent((0,o.default)("<div>").addClass(T),w.default,{}).$element().appendTo(e),t.hide())},_renderToggleItemVisibilityIcon:function(e,t){var n=(0,o.default)("<div>").addClass(B).appendTo(e);t.internalFields.expanded&&(n.addClass(L),e.parent().addClass(O)),t.internalFields.disabled&&n.addClass(N),this._renderToggleItemVisibilityIconClick(n,t)},_renderToggleItemVisibilityIconClick:function(e,t){var n=this,i=(0,m.addNamespace)(l.name,this.NAME);r.default.off(e,i),r.default.on(e,i,function(e){n._toggleExpandedState(t.internalFields.key,void 0,e)})},_updateExpandedItemsUI:function(e,t,n){var i=this._getNodeElement(e),o=!i.length||t&&i.is(":hidden");if(this.option("expandNodesRecursive")&&o){var a=this._getNode(e.internalFields.parentKey);a&&this._updateExpandedItemsUI(a,t,n)}var r=i.children("."+B),s=i.children(".".concat(I));r.toggleClass(L,t);var l=s.length>0,u=new C.Deferred;return!t||l&&!s.is(":empty")?(this._animateNodeContainer(e,t,n,u),u.promise()):0===e.internalFields.childrenKeys.length&&(this._isVirtualMode()||this._useCustomChildrenLoader())?(this._loadNestedItemsWithUpdate(e,t,n,u),u.promise()):(this._renderSublevel(i,e,this._getChildNodes(e)),this._fireContentReadyAction(),this._animateNodeContainer(e,t,n,u),u.promise())},_loadNestedItemsWithUpdate:function(e,t,n,i){var o=this,a=this._getNodeElement(e);this._loadNestedItems(e).done(function(r){var s=o._getActualNode(e);o._renderSublevel(a,s,o._dataAdapter.getNodesByItems(r)),r&&r.length&&(o._fireContentReadyAction(),o._animateNodeContainer(s,t,n,i))})},_loadNestedItems:function(e){var t=this;if(this._useCustomChildrenLoader()){var n=this._dataAdapter.getPublicNode(e);return this._loadChildrenByCustomLoader(n).done(function(e){t._areNodesExists(e)||t._appendItems(e)})}return this._isVirtualMode()?(this._filter.internal=[this.option("parentIdExpr"),e.internalFields.key],this._dataSource.filter(this._combineFilter()),this._dataSource.load().done(function(e){t._areNodesExists(e)||t._appendItems(e)})):(new C.Deferred).resolve([]).promise()},_areNodesExists:function(e,t){var n=this.keyOf(e[0]),i=this._dataAdapter.getFullData();return!!this._dataAdapter.getNodeByKey(n,i)},_appendItems:function(e){this.option().items=this.option("items").concat(e),this._initDataAdapter()},_animateNodeContainer:function(e,t,n,i){var o=this._getNodeElement(e),a=o.children(".".concat(I));e&&i&&0===a.length&&i.resolve(),a.addClass(O);var r=a.height();b.default.stop(a,!0),b.default.animate(a,{type:"custom",duration:this.option("animationEnabled")?400:0,from:{maxHeight:t?0:r},to:{maxHeight:t?r:0},complete:function(){a.css("maxHeight","none"),a.toggleClass(O,t),this.setAria("expanded",t,o),this._scrollableContainer.update(),this._fireExpandedStateUpdatedEvent(t,e,n),i&&i.resolve()}.bind(this)})},_fireExpandedStateUpdatedEvent:function(e,t,n){if(this._hasChildren(t)){var i=e?"onItemExpanded":"onItemCollapsed";if((0,d.isDefined)(n))this._itemDXEventHandler(n,i,{node:this._dataAdapter.getPublicNode(t)});else{var o=this._getNodeElement(t);this._itemEventHandler(o,i,{event:n,node:this._dataAdapter.getPublicNode(t)})}}},_normalizeIconState:function(e,t){var n=e.find(".".concat(T));if(n.length&&w.default.getInstance(n).option("visible",!1),t){var i=e.find("."+B);return void i.show()}e.find("."+B).removeClass(B),e.addClass(E)},_emptyMessageContainer:function(){return this._scrollableContainer?this._scrollableContainer.content():this.callBase()},_renderContent:function(){var e=this.option("items");e&&e.length&&(this._contentAlreadyRendered=!0),this.callBase()},_renderSelectAllItem:function(e){e=e||this.$element().find(".".concat(I)).first(),this._$selectAllItem=(0,o.default)("<div>").addClass(H);var t=this._dataAdapter.isAllSelected();this._createComponent(this._$selectAllItem,g.default,{value:t,text:this.option("selectAllText"),onValueChanged:function(e){this._toggleSelectAll(e),this._fireSelectAllValueChanged(e.value)}.bind(this)}),this._toggleSelectedClass(this._$selectAllItem,t),e.before(this._$selectAllItem)},_toggleSelectAll:function(e){this._dataAdapter.toggleSelectAll(e.value),this._updateItemsUI(),this._fireSelectionChanged()},_renderCheckBox:function(e,t){var n=(0,o.default)("<div>").appendTo(e);this._createComponent(n,g.default,{value:t.internalFields.selected,onValueChanged:this._changeCheckboxValue.bind(this),focusStateEnabled:!1,disabled:this._disabledGetter(t)})},_toggleSelectedClass:function(e,t){e.toggleClass(j,!!t)},_toggleNodeDisabledState:function(e,t){var n=this._getNodeElement(e),i=n.find("."+A).eq(0);if(this._dataAdapter.toggleNodeDisabledState(e.internalFields.key,t),i.toggleClass(N,!!t),this._showCheckboxes()){var o=this._getCheckBoxInstance(n);o.option("disabled",!!t)}},_itemOptionChanged:function(e,t,n){var i=this._dataAdapter.getNodeByItem(e);t===this.option("disabledExpr")&&this._toggleNodeDisabledState(i,n)},_changeCheckboxValue:function(e){var t=(0,o.default)(e.element).parent("."+D),n=t.children("."+A),i=this._getItemData(n),a=this._getNodeByElement(n),r=e.value;a&&a.internalFields.selected===r||this._updateItemSelection(r,i,e.event)},_isSingleSelection:function(){return"single"===this.option("selectionMode")},_isRecursiveSelection:function(){return this.option("selectNodesRecursive")&&"single"!==this.option("selectionMode")},_isLastSelectedBranch:function(e,t,n){var i=t.indexOf(e.key);return i>=0&&t.splice(i,1),n&&(0,f.each)(e.children,function(e,n){this._isLastSelectedBranch(n,t,!0)}.bind(this)),e.parent&&this._isLastSelectedBranch(e.parent,t),0===t.length},_isLastRequired:function(e){var t=this.option("selectionRequired"),n=this._isSingleSelection(),i=this.getSelectedNodeKeys();if(t)return n?1===i.length:this._isLastSelectedBranch(e.internalFields.publicNode,i.slice(),!0)},_updateItemSelection:function(e,t,n){var i=this,o=this._getNode(t);if(!o)return!1;if(o.internalFields.selected===e)return!0;if(!e&&this._isLastRequired(o)){if(this._showCheckboxes()){var a=this._getNodeElement(o),r=this._getCheckBoxInstance(a);r&&r.option("value",!0)}return!1}var s=this.getSelectedNodeKeys();this._isSingleSelection()&&e&&(0,f.each)(s,function(e,t){i.unselectItem(t)}),this._dataAdapter.toggleSelection(o.internalFields.key,e),this._updateItemsUI();var l=n||this._findItemElementByItem(o.internalFields.item),u=n?this._itemDXEventHandler:this._itemEventHandler;return u.call(this,l,"onItemSelectionChanged",{node:this._dataAdapter.getPublicNode(o),itemData:o.internalFields.item}),this._fireSelectionChanged(),!0},_getCheckBoxInstance:function(e){return e.children(".dx-checkbox").dxCheckBox("instance")},_updateItemsUI:function(){var e=this,t={};(0,f.each)(this._dataAdapter.getData(),function(n,i){var o=e._getNodeElement(i,t),a=i.internalFields.selected;if(o.length&&(e._toggleSelectedClass(o,a),e.setAria("selected",a,o),e._showCheckboxes())){var r=e._getCheckBoxInstance(o);r.option("value",a)}}),this._selectAllEnabled()&&this._$selectAllItem.dxCheckBox("instance").option("value",this._dataAdapter.isAllSelected())},_updateParentsState:function(e,t){if(t){var n=this._dataAdapter.getNodeByKey(e.internalFields.parentKey),i=(0,o.default)(t.parents("."+D)[0]);if(this._showCheckboxes()){var a=n.internalFields.selected;this._getCheckBoxInstance(i).option("value",a),this._toggleSelectedClass(i,a)}n.internalFields.parentKey!==this.option("rootValue")&&this._updateParentsState(n,i)}},_itemEventHandlerImpl:function(e,t,n){var i=(0,o.default)(e).closest("."+D).children("."+A);return t((0,h.extend)(this._extendActionArgs(i),n))},_itemContextMenuHandler:function(e){this._createEventHandler("onItemContextMenu",e)},_itemHoldHandler:function(e){this._createEventHandler("onItemHold",e)},_createEventHandler:function(e,t){var n=this._getNodeByElement(t.currentTarget);this._itemDXEventHandler(t,e,{node:this._dataAdapter.getPublicNode(n)})},_itemClass:function(){return A},_itemDataKey:function(){return M},_attachClickEvent:function(){var e="."+this._itemClass(),t="."+D+", ."+H,n=(0,m.addNamespace)(l.name,this.NAME),i=(0,m.addNamespace)(v.default.down,this.NAME),a=this._itemContainer(),s=this;r.default.off(a,n,e),r.default.off(a,i,t),r.default.on(a,n,e,function(e){s._itemClickHandler(e,(0,o.default)(this))}),r.default.on(a,i,t,function(e){s._itemPointerDownHandler(e)})},_itemClickHandler:function(e,t){var n=this._getItemData(t),i=this._getNodeByElement(t);this._itemDXEventHandler(e,"onItemClick",{node:this._dataAdapter.getPublicNode(i)}),this.option("selectByClick")&&!e.isDefaultPrevented()&&this._updateItemSelection(!i.internalFields.selected,n,e)},_updateSelectionToFirstItem:function(e,t){for(var n=t;n>=0;){var i=(0,o.default)(e[n]);this._updateItemSelection(!0,i.find("."+A).get(0)),n--}},_updateSelectionToLastItem:function(e,t){for(var n=e.length,i=t;i<n;){var a=(0,o.default)(e[i]);this._updateItemSelection(!0,a.find("."+A).get(0)),i++}},_focusInHandler:function(e){var t=this;if(this._updateFocusState(e,!0),this.option("focusedElement"))return clearTimeout(this._setFocusedItemTimeout),void(this._setFocusedItemTimeout=setTimeout(function(){t._setFocusedItem((0,o.default)(t.option("focusedElement")))}));var n=this._getActiveItem();this.option("focusedElement",(0,p.getPublicElement)(n.closest("."+D)))},_setFocusedItem:function(e){e&&e.length&&(e.children().hasClass(N)||this.callBase(e))},_itemPointerDownHandler:function(e){if(this.option("focusStateEnabled")){var t=(0,o.default)(e.target).closest("."+D+", ."+H);if(t.length){var n=t.hasClass(N)?null:t;this.option("focusedElement",(0,p.getPublicElement)(n))}}},_findNonDisabledNodes:function(e){return e.not(function(){return(0,o.default)(this).children("."+A).hasClass(N)})},_moveFocus:function(e,t){var n="up",i="down",o="first",a="last",r=this.option("rtlEnabled")?"right":"left",s=this.option("rtlEnabled")?"left":"right";this.$element().find(".".concat(I)).each(function(){b.default.stop(this,!0)});var l=this._findNonDisabledNodes(this._nodeElements());if(l&&l.length)switch(e){case n:var u=this._prevItem(l);this.option("focusedElement",(0,p.getPublicElement)(u));var c=this._getNodeItemElement(u);this._scrollableContainer.scrollToElement(c),t.shiftKey&&this._showCheckboxes()&&this._updateItemSelection(!0,c);break;case i:var d=this._nextItem(l);this.option("focusedElement",(0,p.getPublicElement)(d));var h=this._getNodeItemElement(d);this._scrollableContainer.scrollToElement(h),t.shiftKey&&this._showCheckboxes()&&this._updateItemSelection(!0,h);break;case o:var f=l.first();t.shiftKey&&this._showCheckboxes()&&this._updateSelectionToFirstItem(l,l.index(this._prevItem(l))),this.option("focusedElement",(0,p.getPublicElement)(f)),this._scrollableContainer.scrollToElement(this._getNodeItemElement(f));break;case a:var g=l.last();t.shiftKey&&this._showCheckboxes()&&this._updateSelectionToLastItem(l,l.index(this._nextItem(l))),this.option("focusedElement",(0,p.getPublicElement)(g)),this._scrollableContainer.scrollToElement(this._getNodeItemElement(g));break;case s:this._expandFocusedContainer();break;case r:this._collapseFocusedContainer();break;default:return void this.callBase.apply(this,arguments)}},_getNodeItemElement:function(e){return e.find("."+A).get(0)},_nodeElements:function(){return this.$element().find("."+D).not(":hidden")},_expandFocusedContainer:function(){var e=(0,o.default)(this.option("focusedElement"));if(e.length&&!e.hasClass(E)){var t=e.find(".".concat(I)).eq(0);if(t.hasClass(O)){var n=this._nextItem(this._findNonDisabledNodes(this._nodeElements()));return this.option("focusedElement",(0,p.getPublicElement)(n)),void this._scrollableContainer.scrollToElement(this._getNodeItemElement(n))}var i=this._getNodeByElement(e.children("."+A));this._toggleExpandedState(i,!0)}},_getClosestNonDisabledNode:function(e){do e=e.parent().closest("."+D);while(e.children(".dx-treeview-item.dx-state-disabled").length);return e},_collapseFocusedContainer:function(){var e=(0,o.default)(this.option("focusedElement"));if(e.length){var t=e.find(".".concat(I)).eq(0);if(!e.hasClass(E)&&t.hasClass(O)){var n=this._getNodeByElement(e.children("."+A));this._toggleExpandedState(n,!1)}else{var i=this._getClosestNonDisabledNode(e);i.length&&this.option("focusedElement",(0,p.getPublicElement)(i)),this._scrollableContainer.scrollToElement(this._getNodeItemElement(i))}}},_encodeString:function(e){return(0,d.isString)(e)?encodeURI(e):e},_decodeString:function(e){return(0,d.isString)(e)?decodeURI(e):e},updateDimensions:function(){var e=this,t=new C.Deferred;return this._scrollableContainer?this._scrollableContainer.update().done(function(){t.resolveWith(e)}):t.resolveWith(this),t.promise()},selectItem:function(e){return this._updateItemSelection(!0,e)},unselectItem:function(e){return this._updateItemSelection(!1,e)},expandItem:function(e){return this._toggleExpandedState(e,!0)},collapseItem:function(e){return this._toggleExpandedState(e,!1)},getNodes:function(){return this._dataAdapter.getTreeNodes()},getSelectedNodes:function(){var e=this;return this.getSelectedNodeKeys().map(function(t){var n=e._dataAdapter.getNodeByKey(t);return e._dataAdapter.getPublicNode(n)})},getSelectedNodesKeys:function(){return k.default.log("W0002","dxTreeView","getSelectedNodesKeys","20.1","Use the 'getSelectedNodeKeys' method instead"),this.getSelectedNodeKeys()},getSelectedNodeKeys:function(){return this._dataAdapter.getSelectedNodesKeys()},selectAll:function(){this._selectAllEnabled()?this._$selectAllItem.dxCheckBox("instance").option("value",!0):this._toggleSelectAll({value:!0})},unselectAll:function(){this._selectAllEnabled()?this._$selectAllItem.dxCheckBox("instance").option("value",!1):this._toggleSelectAll({value:!1})},expandAll:function(){var e=this._dataAdapter;(0,f.each)(e.getData(),function(t,n){return e.toggleExpansion(n.internalFields.key,!0)}),this.repaint()},collapseAll:function(){(0,f.each)(this._dataAdapter.getExpandedNodesKeys(),function(e,t){this._toggleExpandedState(t,!1)}.bind(this))},scrollToItem:function(e){var t=this,n=this._getNode(e);if(!n)return(new C.Deferred).reject().promise();for(var i=[],o=n.internalFields.publicNode.parent;null!=o;)o.expanded||i.push(o.key),o=o.parent;var a=new C.Deferred;return this._expandNodes(i.reverse()).always(function(){var e=t._getNodeElement(n);e&&e.length?(t._scrollableContainer.scrollToElementTopLeft(e),a.resolve()):a.reject()}),a.promise()},_expandNodes:function(e){var t=this;if(!e||0===e.length)return(new C.Deferred).resolve().promise();var n=new C.Deferred,i=e.map(function(e){return t.expandItem(e)});return C.when.apply(o.default,i).done(function(){return n.resolve()}).fail(function(){return n.reject()}),n.promise()},_dispose:function(){this.callBase(),clearTimeout(this._setFocusedItemTimeout)}}),$=q;t.default=$,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.GroupingHeaderPanelExtender=void 0;var o=i(n(2)),a=i(n(26)),r=n(706),s=n(707),l=i(n(10)),u=i(n(222)),c=n(1),d=n(3),h=i(n(13)),f=n(6),p=n(302),g=n(128),_="dx-datagrid-group-panel",m="dx-group-panel-message",v="dx-group-panel-item",y="dx-toolbar-label",b="dx-datagrid-expand",x="dx-group-row",w=".dx-header-filter",C=function(){return{init:function(){this.callBase.apply(this,arguments),this._initGroupingHelper()},_initGroupingHelper:function(e){var t=this._grouping,n=this.option("grouping.autoExpandAll"),i=this.option("focusedRowEnabled"),o=e?e.remoteOperations:this.remoteOperations(),a=o.filtering&&o.sorting&&o.paging;!a||o.grouping||!n&&i?(!t||t instanceof r.GroupingHelper)&&(this._grouping=new s.GroupingHelper(this)):(!t||t instanceof s.GroupingHelper)&&(this._grouping=new r.GroupingHelper(this))},totalItemsCount:function(){var e=this,t=e.callBase();return t>0&&e._dataSource.group()&&e._dataSource.requireTotalCount()?t+e._grouping.totalCountCorrection():t},itemsCount:function(){
return this._dataSource.group()?this._grouping.itemsCount()||0:this.callBase.apply(this,arguments)},allowCollapseAll:function(){return this._grouping.allowCollapseAll()},isGroupItemCountable:function(e){return this._grouping.isGroupItemCountable(e)},isRowExpanded:function(e){var t=this._grouping.findGroupInfo(e);return t?t.isExpanded:!this._grouping.allowCollapseAll()},collapseAll:function(e){return this._collapseExpandAll(e,!1)},expandAll:function(e){return this._collapseExpandAll(e,!0)},_collapseExpandAll:function(e,t){var n=this,i=n._dataSource,o=i.group(),r=a.default.normalizeSortingInfo(o||[]);if(r.length){for(var s=0;s<r.length;s++)void 0===e||e===s?r[s].isExpanded=t:o&&o[s]&&(r[s].isExpanded=o[s].isExpanded);i.group(r),n._grouping.foreachGroups(function(n,i){void 0!==e&&e!==i.length-1||(n.isExpanded=t)},!1,!0),n.resetPagesCache()}return!0},refresh:function(){return this.callBase.apply(this,arguments),this._grouping.refresh.apply(this._grouping,arguments)},changeRowExpand:function(e){var t=this,n=t._dataSource;if(n.group())return n.beginLoading(),t._lastLoadOptions&&(t._lastLoadOptions.groupExpand=!0),t._changeRowExpandCore(e).always(function(){n.endLoading()})},_changeRowExpandCore:function(e){return this._grouping.changeRowExpand(e)},_hasGroupLevelsExpandState:function(e,t){if(e&&Array.isArray(e))for(var n=0;n<e.length;n++)if(e[n].isExpanded===t)return!0},_customizeRemoteOperations:function(e,t){var n=e.remoteOperations;e.storeLoadOptions.group?(n.grouping&&!e.isCustomLoading&&(n.groupPaging&&!this._hasGroupLevelsExpandState(e.storeLoadOptions.group,!0)||(n.paging=!1)),n.grouping||n.sorting&&n.filtering&&!e.isCustomLoading&&!this._hasGroupLevelsExpandState(e.storeLoadOptions.group,!1)||(n.paging=!1)):!e.isCustomLoading&&n.paging&&t.grouping&&this.resetCache(),this.callBase.apply(this,arguments)},_handleDataLoading:function(e){return this.callBase(e),this._initGroupingHelper(e),this._grouping.handleDataLoading(e)},_handleDataLoaded:function(e){return this._grouping.handleDataLoaded(e,this.callBase.bind(this))},_handleDataLoadedCore:function(e){return this._grouping.handleDataLoadedCore(e,this.callBase.bind(this))}}}();u.default.extend(C);var k=function(){return{init:function(){var e=this;e.callBase(),e.createAction("onRowExpanding"),e.createAction("onRowExpanded"),e.createAction("onRowCollapsing"),e.createAction("onRowCollapsed")},_beforeProcessItems:function(e){var t=this._columnsController.getGroupColumns();return e=this.callBase(e),e.length&&t.length&&(e=this._processGroupItems(e,t.length)),e},_processItem:function(e,t){return(0,c.isDefined)(e.groupIndex)&&(0,c.isString)(e.rowType)&&0===e.rowType.indexOf("group")?(e=this._processGroupItem(e,t),t.dataIndex=0):e=this.callBase.apply(this,arguments),e},_processGroupItem:function(e){return e},_processGroupItems:function(e,t,n){var i=this,o=i._columnsController.getGroupColumns(),a=o[o.length-t];if(!n){var r=i.option("scrolling.mode");n={collectContinuationItems:"virtual"!==r&&"infinite"!==r,resultItems:[],path:[],values:[]}}var s=n.resultItems;if(n.data&&(!n.collectContinuationItems&&n.data.isContinuation||s.push({rowType:"group",data:n.data,groupIndex:n.path.length-1,isExpanded:!!n.data.items,key:n.path.slice(0),values:n.values.slice(0)})),e)if(0===t)s.push.apply(s,e);else for(var l=0;l<e.length;l++){var u=e[l];u&&"items"in u?(n.data=u,n.path.push(u.key),n.values.push(a&&a.deserializeValue&&!a.calculateDisplayValue?a.deserializeValue(u.key):u.key),i._processGroupItems(u.items,t-1,n),n.data=void 0,n.path.pop(),n.values.pop()):s.push(u)}return s},publicMethods:function(){return this.callBase().concat(["collapseAll","expandAll","isRowExpanded","expandRow","collapseRow"])},collapseAll:function(e){var t=this._dataSource;t&&t.collapseAll(e)&&(t.pageIndex(0),t.reload())},expandAll:function(e){var t=this._dataSource;t&&t.expandAll(e)&&(t.pageIndex(0),t.reload())},changeRowExpand:function(e){var t=this,n=t.isRowExpanded(e),i={key:e,expanded:n};return t.executeAction(n?"onRowCollapsing":"onRowExpanding",i),i.cancel?(new f.Deferred).resolve():(0,f.when)(t._changeRowExpandCore(e)).done(function(){i.expanded=!n,t.executeAction(n?"onRowCollapsed":"onRowExpanded",i)})},_changeRowExpandCore:function(e){var t=this,n=this._dataSource,i=new f.Deferred;return n?(0,f.when)(n.changeRowExpand(e)).done(function(){t.load().done(i.resolve).fail(i.reject)}).fail(i.reject):i.resolve(),i},isRowExpanded:function(e){var t=this._dataSource;return t&&t.isRowExpanded(e)},expandRow:function(e){return this.isRowExpanded(e)?(new f.Deferred).resolve():this.changeRowExpand(e)},collapseRow:function(e){return this.isRowExpanded(e)?this.changeRowExpand(e):(new f.Deferred).resolve()},optionChanged:function(e){"grouping"===e.name&&(e.name="dataSource"),this.callBase(e)}}}(),S=function(e,t){var n=this._columnsController;switch(t.itemData.value){case"group":var i=n._dataSource.group()||[];n.columnOption(e.dataField,"groupIndex",i.length);break;case"ungroup":n.columnOption(e.dataField,"groupIndex",-1);break;case"ungroupAll":this.component.clearGrouping()}},D=function(){return{_getToolbarItems:function(){var e=this.callBase();return this._appendGroupingItem(e)},_appendGroupingItem:function(e){var t=this;if(this._isGroupPanelVisible()){var n=!1,i={template:function(){var e=(0,o.default)("<div>").addClass(_);return t._updateGroupPanelContent(e),(0,p.registerKeyboardAction)("groupPanel",t,e,void 0,t._handleActionKeyDown.bind(t)),e},name:"groupPanel",onItemRendered:function(){n&&t.renderCompleted.fire(),n=!0},location:"before",locateInMenu:"never",sortIndex:1};e.push(i)}return e},_handleActionKeyDown:function(e){var t=e.event,n=(0,o.default)(t.target),i=n.closest(".".concat(v)).index(),a=this._columnsController.getGroupColumns()[i],r=a&&a.index;n.is(w)?this.getController("headerFilter").showHeaderFilterMenu(r,!0):this._processGroupItemAction(r),t.preventDefault()},_isGroupPanelVisible:function(){var e,t=this.option("groupPanel");return t&&(e=t.visible,"auto"===e&&(e="desktop"===h.default.current().deviceType)),e},_renderGroupPanelItems:function(e,t){var n=this;e.empty(),(0,d.each)(t,function(t,i){n._createGroupPanelItem(e,i)}),(0,g.restoreFocus)(this)},_createGroupPanelItem:function(e,t){var n=(0,o.default)("<div>").addClass(t.cssClass).addClass(v).data("columnData",t).appendTo(e).text(t.caption);return(0,g.setTabIndex)(this,n),n},_columnOptionChanged:function(e){if(!this._requireReady&&!a.default.checkChanges(e.optionNames,["width","visibleWidth"])){var t=this.element(),n=t&&t.find("."+_);n&&n.length&&(this._updateGroupPanelContent(n),this.renderCompleted.fire())}this.callBase()},_updateGroupPanelContent:function(e){var t=this,n=t.getController("columns").getGroupColumns(),i=t.option("groupPanel");t._renderGroupPanelItems(e,n),i.allowColumnDragging&&!n.length&&((0,o.default)("<div>").addClass(m).text(i.emptyPanelText).appendTo(e),e.closest("."+y).css("maxWidth","none"),t.updateToolbarDimensions())},allowDragging:function(e){var t=this.option("groupPanel");return this._isGroupPanelVisible()&&t.allowColumnDragging&&e&&e.allowGrouping},getColumnElements:function(){var e=this.element();return e&&e.find("."+v)},getColumns:function(){return this.getController("columns").getGroupColumns()},getBoundingRect:function(){var e=this,t=e.element();if(t&&t.find("."+_).length){var n=t.offset();return{top:n.top,bottom:n.top+t.height()}}return null},getName:function(){return"group"},getContextMenuItems:function(e){var t,n=this,i=n.option("grouping.contextMenuEnabled"),a=(0,o.default)(e.targetElement).closest("."+v);if(a.length&&(e.column=a.data("columnData")),i&&e.column){var r=e.column,s=!(0,c.isDefined)(r.allowGrouping)||r.allowGrouping;if(s){var l=(0,c.isDefined)(r.groupIndex)&&r.groupIndex>-1,u=n.option("grouping.texts"),d=S.bind(n,r);t=[{text:u.ungroup,value:"ungroup",disabled:!l,onItemClick:d},{text:u.ungroupAll,value:"ungroupAll",onItemClick:d}]}}return t},isVisible:function(){return this.callBase()||this._isGroupPanelVisible()},optionChanged:function(e){"groupPanel"===e.name?(this._invalidate(),e.handled=!0):this.callBase(e)}}}();t.GroupingHeaderPanelExtender=D;var I=function(){return{getContextMenuItems:function(e){var t,n=this,i=n.option("grouping.contextMenuEnabled");if(i&&e.row&&"group"===e.row.rowType){var o=n._columnsController,a=o.columnOption("groupIndex:"+e.row.groupIndex);if(a&&a.allowGrouping){var r=n.option("grouping.texts"),s=S.bind(n,a);t=[],t.push({text:r.ungroup,value:"ungroup",onItemClick:s},{text:r.ungroupAll,value:"ungroupAll",onItemClick:s})}}return t},_rowClick:function(e){var t=this,n=t.option("grouping.expandMode"),i=t.option("scrolling.mode"),a="infinite"!==i&&"rowClick"===n&&(0,o.default)(e.event.target).closest("."+x).length,r=(0,o.default)(e.event.target).closest("."+b).length;(a||r)&&t._changeGroupRowState(e),t.callBase(e)},_changeGroupRowState:function(e){var t=this.getController("data"),n=t.items()[e.rowIndex],i=this._columnsController.columnOption("groupIndex:"+n.groupIndex,"allowCollapsing");("data"===n.rowType||"group"===n.rowType&&i!==!1)&&(t.changeRowExpand(n.key,!0),e.event.preventDefault(),e.handled=!0)}}}(),T=function(){return{getContextMenuItems:function(e){var t=this,n=t.option("grouping.contextMenuEnabled"),i=t.callBase(e);if(n&&e.row&&("header"===e.row.rowType||"detailAdaptive"===e.row.rowType)){var o=e.column;if(!o.command&&(!(0,c.isDefined)(o.allowGrouping)||o.allowGrouping)){var a=t.option("grouping.texts"),r=(0,c.isDefined)(o.groupIndex)&&o.groupIndex>-1,s=S.bind(t,o);i=i||[],i.push({text:a.groupByThisColumn,value:"group",beginGroup:!0,disabled:r,onItemClick:s}),o.showWhenGrouped&&i.push({text:a.ungroup,value:"ungroup",disabled:!r,onItemClick:s}),i.push({text:a.ungroupAll,value:"ungroupAll",onItemClick:s})}}return i}}}();a.default.registerModule("grouping",{defaultOptions:function(){return{grouping:{autoExpandAll:!0,allowCollapsing:!0,contextMenuEnabled:!1,expandMode:"buttonClick",texts:{groupContinuesMessage:l.default.format("dxDataGrid-groupContinuesMessage"),groupContinuedMessage:l.default.format("dxDataGrid-groupContinuedMessage"),groupByThisColumn:l.default.format("dxDataGrid-groupHeaderText"),ungroup:l.default.format("dxDataGrid-ungroupHeaderText"),ungroupAll:l.default.format("dxDataGrid-ungroupAllText")}},groupPanel:{visible:!1,emptyPanelText:l.default.format("dxDataGrid-groupPanelEmptyText"),allowColumnDragging:!0}}},extenders:{controllers:{data:k,columns:{_getExpandColumnOptions:function(){var e=this.callBase.apply(this,arguments);return e.cellTemplate=a.default.getExpandCellTemplate(),e}}},views:{headerPanel:D,rowsView:I,columnHeadersView:T}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.GroupingHelper=void 0;var o=n(19),a=i(n(41)),r=n(3),s=n(0),l=i(n(91)),u=i(n(26)),c=n(478),d=n(303),h=i(n(47)),f=n(6),p=function(e,t){var n=new f.Deferred,i=(0,s.extend)({skip:0,take:1,requireTotalCount:!0},t);return e.load(i).done(function(e,t){n.resolve(t&&t.totalCount)}).fail(n.reject.bind(n)),n},g=c.GroupingHelper.inherit(function(){var e=function(e,t,n){return e.foreachGroups(function(e){if(!e.isExpanded)return t(e)},!1,!1,n,!0)},t=function(t,n){var i=0,o=n||0;return n&&(e(t,function(e){return!(e.offset-i>=n)&&void(i+=e.count-1)}),o+=i),o},n=function e(t,n,i,o,a,r){for(var s=0,l=!1,u=0;u<n.length;u++){var c=n[u];if(void 0!==c.items){i.push(c.key);var d=t.findGroupInfo(i);if(d&&!d.isExpanded)c.collapsedItems=c.items,c.items=null,o+=d.count,r--,r<0&&s++,a&&(l=!0);else if(c.items){var h=e(t,c.items,i,o,a,r);a&&(h.offset-o>1?c.isContinuation=!0:l=!0),o=h.offset,r=h.take,r<0&&(c.items.length?c.isContinuationOnNextPage=!0:s++)}i.pop()}else a&&(l=!0),o++,r--,r<0&&s++;a=!1}return l&&n.splice(0,1),s&&n.splice(-s,s),{offset:o,take:r}},i=function(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(!a.default.keysEqual(null,e[n],t[n]))return!1;return!0},g=function e(t,n,o,a,r){if(n){for(var s=0;s<n.length;s++){var l=n[s];if("key"in l&&void 0!==l.items){o.push(l.key),r&&i(r.path,o)&&!l.isContinuation&&(r.offset=a);var u=t.findGroupInfo(o);u&&!l.isContinuation&&(u.offset=a),u&&!u.isExpanded?a+=u.count:a=e(t,l.items,o,a,r),o.pop()}else a++}return a}},_=function(e,t){if(t.group){var n=u.default.normalizeSortingInfo(t.group),i=u.default.normalizeSortingInfo(e.sort);e.sort=l.default.arrangeSortingInfo(n,i),delete t.group}},m=function(e,t,n){for(var i=u.default.normalizeSortingInfo(n||t.group),o=[],a=0;a<e.length;a++){for(var r=[],s=0;s<=a;s++)r.push([i[s].selector,a===s?"<>":"=",e[s]]);o.push(u.default.combineFilters(r))}return o=u.default.combineFilters(o,"or"),u.default.combineFilters([o,t.filter])},v=function e(t,n){var i=t.count||t.items.length;if(!t.count&&n>1){i=0;for(var o=0;o<t.items.length;o++)i+=e(t.items[o],n-1)}return i};return{handleDataLoading:function(n){var i,o=this,a=n.storeLoadOptions,l=[],u=0,c=!1,d=n.loadOptions.group,h=0;if(_(a,n.loadOptions),n.group=n.group||d,!n.isCustomLoading){var f=(0,s.extend)({},a);f.skip=t(o,a.skip),f.skip&&f.take&&d&&(f.skip--,f.take++,c=!0),f.take&&d&&(i=f.take,f.take++),e(o,function(e){return!(e.offset>=f.skip+f.take+h)&&void(e.offset>=f.skip+h&&e.count&&(h+=e.count-1,l.push(e),u+=e.count))}),(0,r.each)(l,function(){f.filter=m(this.path,f,d)}),n.storeLoadOptions=f,n.collapsedGroups=l,n.collapsedItemsCount=u,n.skip=f.skip||0,n.skipFirstItem=c,n.take=i}},handleDataLoaded:function(e,t){function i(e,t,n,a,r){if(e&&t.length&&n.length){var s,l,u=(0,o.toComparable)(t[0],!0);for(l=0;l<e.length&&(s=(0,o.toComparable)(e[l].key,!0),!(r>=a.offset||u===s));l++)r+=v(e[l],n.length);e.length&&u===s||e.splice(l,0,{key:t[0],items:[],count:1===t.length?a.count:void 0}),i(e[l].items,t.slice(1),n.slice(1),a,r)}}var a=this,r=e.data,s=e.collapsedGroups,c=u.default.normalizeSortingInfo(e.group),d=c.length;if(e.collapsedItemsCount&&e.extra&&e.extra.totalCount>=0&&(e.extra.totalCount+=e.collapsedItemsCount),t(e),d){var f=(0,h.default)(r);if(l.default.multiLevelGroup(f,c).enumerate().done(function(e){r=e}),s)for(var p=0;p<s.length;p++)i(r,s[p].path,c,s[p],e.skip);e.isCustomLoading||n(a,r,[],e.skip,e.skipFirstItem,e.take),e.data=r}},isGroupItemCountable:function(e){return null===e.items},updateTotalItemsCount:function(){var t=0;e(this,function(e){e.count&&(t-=e.count-1)}),this.callBase(t)},changeRowExpand:function(e){var n,i=this,o=i._dataSource,a=o.beginPageIndex?o.beginPageIndex():o.pageIndex(),r=o.items(),s=t(i,a*o.pageSize()),l=i.findGroupInfo(e);return n=l&&!l.isExpanded?(new f.Deferred).resolve(l.count):p(o,{filter:(0,d.createGroupFilter)(e,{filter:o.filter(),group:o.group()})}),(0,f.when)(n).done(function(t){t=parseInt(t.length?t[0]:t),l?(g(i,r,[],s),l.isExpanded=!l.isExpanded,l.count=t):(l={offset:-1,count:t,path:e,isExpanded:!1},g(i,r,[],s,l),l.offset>=0&&i.addGroupInfo(l)),i.updateTotalItemsCount()}).fail(function(){o._eventsStrategy.fireEvent("loadError",arguments)})},allowCollapseAll:function(){return!1},refresh:function(t,n){var i=this,o=t.storeLoadOptions,a=i._dataSource;if(this.callBase.apply(this,arguments),n.reload)return e(i,function(e){var t=p(a,{filter:(0,d.createGroupFilter)(e.path,o)}),n=p(a,{filter:(0,c.createOffsetFilter)(e.path,o)});return(0,f.when)(n,t).done(function(t,n){t=parseInt(t.length?t[0]:t),n=parseInt(n.length?n[0]:n),e.offset=t,e.count!==n&&(e.count=n,i.updateTotalItemsCount())})},!0)}}}());t.GroupingHelper=g},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){if(i=i||0,t>1&&n>0){var o=e-Math.floor(e/t)*t||t;return o+=n-i-2,o<0&&(o+=t),Math.floor(o/(t-i-1))}return 0}t.GroupingHelper=void 0;var a=n(0),r=n(3),s=i(n(26)),l=n(478),u=n(303),c=i(n(17)),d=i(n(34)),h=n(6),f=l.GroupingHelper.inherit(function(){function e(e,t,n,i){var o=t.items[i],a=e.skips&&e.skips[n],r=e.takes&&e.takes[n],s=i===t.items.length-1,l=0===i,u=l&&a||0,c=o.count>r+u;l&&void 0!==a&&(o.isContinuation=!0),s&&void 0!==r&&c&&(o.isContinuationOnNextPage=!0)}function t(e,t,n){var i=n-1,o=e.group?e.group.length:0;t.skip=e.skips&&e.skips[i],e.takes&&void 0!==e.takes[i]&&(o===n?t.take=t.count?t.count-(t.skip||0):0:t.take=0,t.take+=e.takes[i])}function n(e){return!Array.isArray(e)}function i(e){n(e.data)||(e.data=new h.Deferred)}function f(e,o,a,r,s,l){o.isCustomLoading||(r={},m(e,l,a,r,[]),t(o,r,a));var u=o.group?o.group.length:0;r.paths.length&&u-a>0?(i(o),p(e,o,r,a,s,l)):r.paths.length&&o.storeLoadOptions.group?(i(o),g(e,o,r,l)):n(o.data)&&o.data.resolve(l)}function p(t,n,i,o,a,s){var l=n.group||[],c=l[a+1],d=[];(0,r.each)(i.paths,function(o){var r={requireTotalCount:!1,requireGroupCount:!0,group:[c],groupSummary:n.storeLoadOptions.groupSummary,filter:(0,u.createGroupFilter)(i.paths[o],{filter:n.storeLoadOptions.filter,group:l}),select:n.storeLoadOptions.select};0===o&&(r.skip=i.skip||0),o===i.paths.length-1&&(r.take=i.take);var s=0===r.take?[]:t._dataSource.loadFromStore(r);(0,h.when)(s).done(function(t){var r=i.items[o];e(n,i,a,o),r.items=t}),d.push(s)}),h.when.apply(null,d).done(function(){y(t,n,s,o+1),f(t,n,o+1,i,a+1,s)})}function g(t,n,i,o){var l=[],c=n.group||[];(0,r.each)(i.paths,function(e,t){l.push((0,u.createGroupFilter)(t,{group:n.isCustomLoading?n.storeLoadOptions.group:c}))});var d=n.storeLoadOptions.filter;n.storeLoadOptions.isLoadingAll||(d=s.default.combineFilters([d,s.default.combineFilters(l,"or")]));var f=(0,a.extend)({},n.storeLoadOptions,{requireTotalCount:!1,requireGroupCount:!1,group:null,sort:c.concat(s.default.normalizeSortingInfo(n.storeLoadOptions.sort||[])),filter:d}),p=t._dataSource.isLastLevelGroupItemsPagingLocal();p||(f.skip=i.skip,f.take=i.take),(0,h.when)(0===i.take?[]:t._dataSource.loadFromStore(f)).done(function(a,s){p&&(a=t._dataSource.sortLastLevelGroupItems(a,c,i.paths),a=i.skip?a.slice(i.skip):a,a=i.take?a.slice(0,i.take):a),(0,r.each)(i.items,function(t,o){var r=o.count-(0===t&&i.skip||0),s=a.splice(0,r);e(n,i,c.length-1,t),o.items=s}),n.data.resolve(o)}).fail(n.data.reject)}var _=function(e,t,n){return e.foreachGroups(function(e,n){if(e.isExpanded)return t(e,n)},!0,!1,n,n)},m=function e(t,n,i,o,a,r,s){var l;if(o.items=o.items||[],o.paths=o.paths||[],o.count=o.count||0,o.lastCount=o.lastCount||0,i)for(var u=0;u<n.length;u++){var c=n[u];if(void 0!==c.items){if(a.push(c.key),r)l=!0;else{var d=t.findGroupInfo(a);l=d&&d.isExpanded}l?c.items?e(t,c.items,i-1,o,a,r,s):1!==i||!c.count||r&&!s||(o.items.push(c),o.paths.push(a.slice(0)),o.count+=o.lastCount,o.lastCount=c.count):(c.collapsedItems=c.items,c.items=null),a.pop()}}},v=function(e,t,n,i,o){var a,r=e.findGroupInfo(i);r?(n?r.count=t.count>0?t.count:t.items&&t.items.length||0:t.count=r.count||t.count,r.offset=o):(n&&(a=t.count>0?t.count:t.items.length),e.addGroupInfo({isExpanded:e._isGroupExpanded(i.length-1),path:i.slice(0),offset:o,count:a||0}))},y=function e(t,n,i,o,a,r,s){var l,u=n.group?n.group.length:0,c=u===o,d=n.remoteOperations.paging,h=0,f=0;if(a=a||0,r=r||[],d&&!s&&(h=0===a?n.skip||0:n.skips[a-1]||0),a>=o)return i.length;for(var p=0;p<i.length;p++){var g=i[p];if(g){if(r.push(g.key),!g.count&&!g.items||void 0===g.items)return-1;if(v(t,g,c,r,h+p),l=g.items?e(t,n,g.items,o,a+1,r,p):g.count||-1,l<0)return-1;f+=l,r.pop()}}return f},b=function(e,t){return e&&e.length&&e[t]&&!!e[t].isExpanded},x=function(e,t,n){for(var i,a=n,r=0;r<e.length;r++)i=e[r].offset+1,r>0&&(i+=e[r-1].childrenTotalCount,t&&(i+=o(a,t,i,r-1)*r)),a+=i;return a},w=function(e,t){var n=new h.Deferred,i=!(!t.group||!t.group.length),o=(0,a.extend)({skip:0,take:1,requireGroupCount:i,requireTotalCount:!i},t,{group:i?t.group:null});return e.load(o).done(function(e,t){var o=t&&(i?t.groupCount:t.totalCount);return isFinite(o)?void n.resolve(o):void n.reject(d.default.errors.Error(i?"E4022":"E4021"))}).fail(n.reject.bind(n)),n};return{updateTotalItemsCount:function(e){var t=0,n=e.extra&&e.extra.totalCount||0,i=e.extra&&e.extra.groupCount||0,a=this._dataSource.pageSize(),r=this._isVirtualPaging();_(this,function(e){e.childrenTotalCount=0}),_(this,function(e,n){var i=x(n,r?0:a,t),s=e.count+e.childrenTotalCount;r||(s+=o(i,a,s,n.length-1)),n[n.length-2]?n[n.length-2].childrenTotalCount+=s:t+=s}),this.callBase(t-n+i)},_isGroupExpanded:function(e){var t=this._dataSource.group();return b(t,e)},_updatePagingOptions:function(e,t){var n=this,i=n._isVirtualPaging(),a=n._dataSource.pageSize(),r=[],s=[],l=0,u=0;e.take&&(_(this,function(e){e.childrenTotalCount=0,e.skipChildrenTotalCount=0}),_(n,function(n,c){var d,h=0,f=0,p=x(c,i?0:a,u),g=0,_=0,m=n.count+n.childrenTotalCount,v=m;t&&t(n,p);var y=e.skip-p;p<=e.skip+e.take&&m&&(d=e.take,i||(g=o(p,a,m,c.length-1),m+=g*c.length,v+=g,a&&y>=0&&(h=c.length,f=c.length-1,_=Math.floor(y/a))),y>=0?(p+m>e.skip&&r.unshift(y-_*h-n.skipChildrenTotalCount),p+m>=e.skip+d&&s.unshift(d-h-n.childrenTotalCount+n.skipChildrenTotalCount)):p+m>=e.skip+d&&s.unshift(d+y-n.childrenTotalCount)),p<=e.skip&&(c[c.length-2]?c[c.length-2].skipChildrenTotalCount+=Math.min(v,y+1-_*f):l+=Math.min(v,y+1)),p<=e.skip+d&&(m=Math.min(v,y+d-(_+1)*f),c[c.length-2]?c[c.length-2].childrenTotalCount+=m:u+=m)}),e.skip-=l,e.take-=u-l),e.skips=r,e.takes=s},changeRowExpand:function(e){var t=this,n=t.findGroupInfo(e),i=t._dataSource,o=i.remoteOperations().groupPaging,a=s.default.normalizeSortingInfo(i.group());return n?(n.isExpanded=!n.isExpanded,o&&n.isExpanded&&e.length<a.length?w(i,{filter:(0,u.createGroupFilter)(e,{filter:i.lastLoadOptions().filter,group:i.group()}),group:[a[e.length]],select:i.select()}).done(function(e){n.count=e}):(new h.Deferred).resolve()):(new h.Deferred).reject()},handleDataLoading:function(e){var t=this,n=e.storeLoadOptions,i=s.default.normalizeSortingInfo(n.group||e.loadOptions.group);if(!e.isCustomLoading&&i.length){if(e.remoteOperations.grouping){var o=t._dataSource.remoteOperations().paging;n.group=s.default.normalizeSortingInfo(n.group),n.group.forEach(function(e,t){var i=t===n.group.length-1;e.isExpanded=!o||!i})}e.group=e.group||i,e.remoteOperations.paging&&(e.skip=n.skip,e.take=n.take,n.requireGroupCount=!0,n.group=i.slice(0,1),t._updatePagingOptions(e),n.skip=e.skip,n.take=e.take)}},handleDataLoadedCore:function(e,t){var n,i=this,o=s.default.normalizeSortingInfo(e.storeLoadOptions.group||e.loadOptions.group).length,a=e.group?e.group.length:0,r={};if(e.isCustomLoading)t(e),m(i,e.data,o,r,[],e.isCustomLoading,e.storeLoadOptions.isLoadingAll);else{if(e.remoteOperations.paging||i.foreachGroups(function(e){e.count=0}),n=y(i,e,e.data,o),n<0)return void(e.data=(new h.Deferred).reject(c.default.Error("E1037")));if(e.remoteOperations.paging||o&&e.extra&&e.loadOptions.requireTotalCount&&(e.extra.totalCount=n,e.extra.groupCount=e.data.length),a&&e.storeLoadOptions.requireGroupCount&&!isFinite(e.extra.groupCount))return void(e.data=(new h.Deferred).reject(d.default.errors.Error("E4022")));i.updateTotalItemsCount(e),e.remoteOperations.paging||i._updatePagingOptions(e),t(e),e.remoteOperations.paging||i._processPaging(e,o)}f(i,e,o,r,0,e.data)},_processSkips:function(e,t,n){if(n){var i=e[0],o=t[0],a=i&&i.items;void 0!==o&&(i.isContinuation=!0,a&&(i.items=a.slice(o),this._processSkips(i.items,t.slice(1),n-1)))}},_processTakes:function(e,t,n,i,o){if(i&&e){o=o||[];var a=e[e.length-1],r=a&&a.items,s=n[0],l=t[0];if(a){var u=a.count-(a.isContinuation&&l||0)||r.length;void 0!==s&&u>s&&(a.isContinuationOnNextPage=!0,o.forEach(function(e){e.isContinuationOnNextPage=!0}),r&&(r=r.slice(0,s),a.items=r)),o.push(a),this._processTakes(r,t.slice(1),n.slice(1),i-1,o)}}},_processPaging:function(e,t){this._processSkips(e.data,e.skips,t),this._processTakes(e.data,e.skips,e.takes,t)},isLastLevelGroupItemsPagingLocal:function(){return!1},sortLastLevelGroupItems:function(e){return e},refresh:function(e,t){function n(e,t){t.length===o+1&&(e.isExpanded=i)}var i,o,a=this,r=a._dataSource,c=e.storeLoadOptions,d=e.group||e.storeLoadOptions.group,f=s.default.normalizeSortingInfo(a._group);for(o=0;o<f.length;o++)i=b(d,o),b(a._group,o)!==i&&a.foreachGroups(n);if(a.callBase.apply(this,arguments),d&&e.remoteOperations.paging&&t.reload)return _(a,function(t){var n=w(r,{filter:(0,u.createGroupFilter)(t.path,{filter:c.filter,group:d}),group:d.slice(t.path.length),select:c.select}),i=w(r,{filter:(0,l.createOffsetFilter)(t.path,{filter:c.filter,group:d}),group:d.slice(t.path.length-1,t.path.length),select:c.select});return(0,h.when)(i,n).done(function(n,i){n=parseInt(n.length?n[0]:n),i=parseInt(i.length?i[0]:i),t.offset=n,t.count!==i&&(t.count=i,a.updateTotalItemsCount(e))})},!0)}}}());t.GroupingHelper=f},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(479));o.default.registerModule("masterDetail",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}n(710);var o=i(n(26)),a=i(n(482)),r=n(0);o.default.registerModule("editing",(0,r.extend)(!0,{},a.default,{extenders:{controllers:{data:{_changeRowExpandCore:function(e){var t=this._editingController;return Array.isArray(e)&&t&&t.refresh(),this.callBase.apply(this,arguments)}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(480));o.default.registerModule("editorFactory",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(483));o.default.registerModule("validating",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(222)),r=i(n(484));o.default.registerModule("virtualScrolling",r.default),a.default.extend(r.default.extenders.dataSourceAdapter)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(486));o.default.registerModule("filterRow",a.default)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(4),y=n(18),b=o(n(84)),x=n(0),w=o(n(110)),C="dx-context-menu-content-delimiter",k="dx-submenu",S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getDefaultOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{orientation:"horizontal",tabIndex:null,onHoverStart:v.noop})}},{key:"_initDataAdapter",value:function(){this._dataAdapter=this.option("_dataAdapter"),this._dataAdapter||l(_(t.prototype),"_initDataAdapter",this).call(this)}},{key:"_renderContentImpl",value:function(){this._renderContextMenuOverlay(),l(_(t.prototype),"_renderContentImpl",this).call(this);var e=this._dataAdapter.getNodeByKey(this.option("_parentKey"));e&&this._renderItems(this._getChildNodes(e)),this._renderDelimiter()}},{key:"_renderDelimiter",value:function(){this.$contentDelimiter=(0,m.default)("<div>").appendTo(this._itemContainer()).addClass(C)}},{key:"_getOverlayOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getOverlayOptions",this).call(this),{onPositioned:this._overlayPositionedActionHandler.bind(this)})}},{key:"_overlayPositionedActionHandler",value:function(e){this._showDelimiter(e)}},{key:"_hoverEndHandler",value:function(e){l(_(t.prototype),"_hoverEndHandler",this).call(this,e),this._toggleFocusClass(!1,e.currentTarget)}},{key:"_isMenuHorizontal",value:function(){return"horizontal"===this.option("orientation")}},{key:"_hoverStartHandler",value:function(e){var n=this.option("onHoverStart");n(e),l(_(t.prototype),"_hoverStartHandler",this).call(this,e),this._toggleFocusClass(!0,e.currentTarget)}},{key:"_drawSubmenu",value:function(e){this._actions.onShowing({rootItem:(0,y.getPublicElement)(e),submenu:this}),l(_(t.prototype),"_drawSubmenu",this).call(this,e),this._actions.onShown({rootItem:(0,y.getPublicElement)(e),submenu:this})}},{key:"_hideSubmenu",value:function(e){this._actions.onHiding({cancel:!0,rootItem:(0,y.getPublicElement)(e),submenu:this}),l(_(t.prototype),"_hideSubmenu",this).call(this,e),this._actions.onHidden({rootItem:(0,y.getPublicElement)(e),submenu:this})}},{key:"_showDelimiter",value:function(e){if(this.$contentDelimiter){var t=this._itemContainer().children(".".concat(k)).eq(0),n=this.option("position").of,i={of:t},o=e.position,a=o.v.location,r=o.h.location,s=n.offset(),l=Math.round(s.left),u=Math.round(s.top),c=n.width(),d=n.height(),h=t.width(),f=t.height();this.$contentDelimiter.css("display","block"),this.$contentDelimiter.width(this._isMenuHorizontal()?c<h?c-2:h:2),this.$contentDelimiter.height(this._isMenuHorizontal()?2:d<f?d-2:f),this._isMenuHorizontal()?a>u?Math.round(r)===l?(i.offset="1 -1",i.at=i.my="left top"):(i.offset="-1 -1",i.at=i.my="right top"):(this.$contentDelimiter.height(5),Math.round(r)===l?(i.offset="1 4",i.at=i.my="left bottom"):(i.offset="-1 2",i.at=i.my="right bottom")):r>l?Math.round(a)===u?(i.offset="-1 1",i.at=i.my="left top"):(i.offset="-1 -1",i.at=i.my="left bottom"):Math.round(a)===u?(i.offset="1 1",i.at=i.my="right top"):(i.offset="1 -1",i.at=i.my="right bottom"),b.default.setup(this.$contentDelimiter,i)}}},{key:"_getContextMenuPosition",value:function(){return this.option("position")}},{key:"isOverlayVisible",value:function(){return this._overlay.option("visible")}},{key:"getOverlayContent",value:function(){return this._overlay.$content()}}]),t}(w.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(489));o.default.registerModule("headerFilter",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(491));o.default.registerModule("filterSync",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=(0,r.default)("<div>").addClass(u),i=(0,r.default)("<div>").addClass(c),o=e.value||[],a=function(e){return e&&e.length>0?e[0]:null},f=function(e){return e&&2===e.length?e[1]:null};t.append(n),t.append((0,r.default)("<span>").addClass(d).text(h)),t.append(i),t.addClass(l),this._editorFactory.createEditor.call(this,n,(0,s.extend)({},e.field,e,{value:a(o),parentType:"filterBuilder",setValue:function(t){o=[t,f(o)],e.setValue(o)}})),this._editorFactory.createEditor.call(this,i,(0,s.extend)({},e.field,e,{value:f(o),parentType:"filterBuilder",setValue:function(t){o=[a(o),t],e.setValue(o)}}))}function a(e,t){
return{name:"between",caption:e,icon:"range",valueSeparator:h,dataTypes:["number","date","datetime"],editorTemplate:o.bind(t),notForLookup:!0}}t.getConfig=a;var r=i(n(2)),s=n(0),l="dx-filterbuilder-range",u=l+"-start",c=l+"-end",d=l+"-separator",h="\u2013"},function(e,t,n){t.default=void 0;var i={"=":"equal","<>":"notequal","<":"less","<=":"lessorequal",">":"greater",">=":"greaterorequal",notcontains:"doesnotcontain",contains:"contains",startswith:"startswith",endswith:"endswith",isblank:"isblank",isnotblank:"isnotblank"},o={"=":"equal","<>":"notEqual","<":"lessThan","<=":"lessThanOrEqual",">":"greaterThan",">=":"greaterThanOrEqual",startswith:"startsWith",contains:"contains",notcontains:"notContains",endswith:"endsWith",isblank:"isBlank",isnotblank:"isNotBlank",between:"between"},a={getIconByFilterOperation:function(e){return i[e]},getNameByFilterOperation:function(e){return o[e]}};t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=function(e,t,n){var i=[],o=e.length-1;return e&&e.forEach(function(e,a){if((0,h.isCondition)(e)||(0,h.isGroup)(e)){var r=(0,h.getFilterExpression)(e,n,[],"headerFilter");i.push(r)}else i.push((0,h.getFilterExpression)([t.dataField,"=",e],n,[],"headerFilter"));a!==o&&i.push("or")}),1===i.length&&(i=i[0]),i},n=function(e,t){return t?t+"/"+e:e},i=function e(t,i){var o=[];return t.forEach(function(t){if(t.items){var a=e(t.items,n(t.text,i));o=o.concat(a)}t.selected&&o.push(n(t.text,i))}),o},o=e&&e.getController("headerFilter"),a=function(t){var n=t.value,a=e.columnOption(t.field.dataField),r=a&&a.headerFilter,s=a&&a.lookup;if(r&&r.dataSource||s&&s.dataSource){a=(0,u.extend)({},a,{filterType:"include",filterValues:[n]});var l=o.getDataSource(a);l.paginate=!1;var h=new c.DataSource(l),f=new d.Deferred,p=h.store().key();return p&&h.filter([p,"=",t.value]),h.load().done(function(e){f.resolve(i(e)[0])}),f}var g=o.getHeaderItemText(n,a,0,e.option("headerFilter"));return g};return{dataTypes:["string","date","datetime","number","boolean","object"],calculateFilterExpression:t,editorTemplate:function(t,n){var i=(0,s.default)("<div>").addClass("dx-filterbuilder-item-value-text").appendTo(n),a=(0,u.extend)(!0,{},e.columnOption(t.field.dataField));(0,h.renderValueText)(i,t.text&&t.text.split("|"));var r=function(e){t.setValue(e)};return a.filterType="include",a.filterValues=t.value?t.value.slice():[],o.showHeaderFilterMenuBase({columnElement:i,column:a,apply:function(){r(this.filterValues),o.hideHeaderFilterMenu(),t.closeEditor()},onHidden:function(){t.closeEditor()},isFilterBuilder:!0}),n},customizeText:a}}function a(e){return(0,u.extend)(o(e),{name:"anyof",icon:"selectall",caption:l.default.format("dxFilterBuilder-filterOperationAnyOf")})}function r(e){var t=o(e);return(0,u.extend)({},t,{calculateFilterExpression:function(e,n,i){var o=t.calculateFilterExpression(e,n,i);return o&&0!==o.length?"!"===o[0]?o:["!",o]:null},name:"noneof",icon:"unselectall",caption:l.default.format("dxFilterBuilder-filterOperationNoneOf")})}t.anyOf=a,t.noneOf=r;var s=i(n(2)),l=i(n(10)),u=n(0),c=n(69),d=n(6),h=n(223)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(492));o.default.registerModule("filterBuilder",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(15)),s=i(n(5)),l=i(n(16)),u=i(n(9)),c=n(0),d=i(n(10)),h=n(6),f=n(1),p=i(n(167)),g=i(n(48)),_=n(293),m=i(n(481)),v=n(8),y=n(223),b="dx-filterbuilder",x=b+"-group",w=x+"-item",C=x+"-content",k=x+"-operations",S=x+"-operation",D=b+"-action",I=D+"-icon",T="dx-icon-plus",O="dx-icon-remove",E=b+"-text",A=b+"-item-field",P=b+"-item-operation",R=b+"-item-value",M=b+"-item-value-text",B=b+"-overlay",V=b+"-operations",F=b+"-fields",L=b+"-add-condition",H="dx-state-active",z=b+"-menu-custom-operation",N="filterBuilder",j="dx-state-disabled",W="tab",G="enter",q="escape",$=[{name:"onEditorPreparing",config:{excludeValidators:["disabled","readOnly"],category:"rendering"}},{name:"onEditorPrepared",config:{excludeValidators:["disabled","readOnly"],category:"rendering"}},{name:"onValueChanged",config:{excludeValidators:["disabled","readOnly"]}}],K={and:"and",or:"or",notAnd:"!and",notOr:"!or"},U=r.default.inherit(m.default),Y=l.default.inherit({_getDefaultOptions:function(){return(0,c.extend)(this.callBase(),{onEditorPreparing:null,onEditorPrepared:null,onValueChanged:null,fields:[],defaultGroupOperation:"and",groupOperations:["and","or","notAnd","notOr"],maxGroupLevel:void 0,value:null,allowHierarchicalFields:!1,groupOperationDescriptions:{and:d.default.format("dxFilterBuilder-and"),or:d.default.format("dxFilterBuilder-or"),notAnd:d.default.format("dxFilterBuilder-notAnd"),notOr:d.default.format("dxFilterBuilder-notOr")},customOperations:[],closePopupOnTargetScroll:!0,filterOperationDescriptions:{between:d.default.format("dxFilterBuilder-filterOperationBetween"),equal:d.default.format("dxFilterBuilder-filterOperationEquals"),notEqual:d.default.format("dxFilterBuilder-filterOperationNotEquals"),lessThan:d.default.format("dxFilterBuilder-filterOperationLess"),lessThanOrEqual:d.default.format("dxFilterBuilder-filterOperationLessOrEquals"),greaterThan:d.default.format("dxFilterBuilder-filterOperationGreater"),greaterThanOrEqual:d.default.format("dxFilterBuilder-filterOperationGreaterOrEquals"),startsWith:d.default.format("dxFilterBuilder-filterOperationStartsWith"),contains:d.default.format("dxFilterBuilder-filterOperationContains"),notContains:d.default.format("dxFilterBuilder-filterOperationNotContains"),endsWith:d.default.format("dxFilterBuilder-filterOperationEndsWith"),isBlank:d.default.format("dxFilterBuilder-filterOperationIsBlank"),isNotBlank:d.default.format("dxFilterBuilder-filterOperationIsNotBlank")}})},_optionChanged:function(e){switch(e.name){case"closePopupOnTargetScroll":break;case"onEditorPreparing":case"onEditorPrepared":case"onValueChanged":this._initActions();break;case"customOperations":this._initCustomOperations(),this._invalidate();break;case"fields":case"defaultGroupOperation":case"maxGroupLevel":case"groupOperations":case"allowHierarchicalFields":case"groupOperationDescriptions":case"filterOperationDescriptions":this._invalidate();break;case"value":if(e.value!==e.previousValue){var t=this._disableInvalidateForValue;t||(this._initModel(),this._invalidate()),this._disableInvalidateForValue=!1,this.executeAction("onValueChanged",{value:e.value,previousValue:e.previousValue}),this._disableInvalidateForValue=t}break;default:this.callBase(e)}},getFilterExpression:function(){var e=this._getNormalizedFields(),t=(0,c.extend)(!0,[],this._model);return(0,y.getFilterExpression)((0,y.getNormalizedFilter)(t),e,this._customOperations,N)},_getNormalizedFields:function(){return(0,y.getNormalizedFields)(this.option("fields"))},_updateFilter:function(){this._disableInvalidateForValue=!0;var e=(0,c.extend)(!0,[],this._model),t=(0,y.getNormalizedFilter)(e),n=(0,y.getNormalizedFilter)(this._getModel(this.option("value")));JSON.stringify(n)!==JSON.stringify(t)&&this.option("value",t),this._disableInvalidateForValue=!1,this._fireContentReadyAction()},_init:function(){this._initCustomOperations(),this._initModel(),this._initEditorFactory(),this._initActions(),this.callBase()},_initEditorFactory:function(){this._editorFactory=new U},_initCustomOperations:function(){this._customOperations=(0,y.getMergedOperations)(this.option("customOperations"),this.option("filterOperationDescriptions.between"),this)},_getModel:function(e){return(0,y.convertToInnerStructure)(e,this._customOperations)},_initModel:function(){this._model=this._getModel(this.option("value"))},_initActions:function(){var e=this;e._actions={},$.forEach(function(t){var n=(0,c.extend)({},t.config);e._actions[t.name]=e._createActionByOption(t.name,n)})},executeAction:function(e,t){var n=this._actions[e];return n&&n(t)},_initMarkup:function(){this.$element().addClass(b),this.callBase(),this._createGroupElementByCriteria(this._model).appendTo(this.$element())},_createConditionElement:function(e,t){return(0,o.default)("<div>").addClass(x).append(this._createConditionItem(e,t))},_createGroupElementByCriteria:function(e,t){for(var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,i=this._createGroupElement(e,t,n),o=i.find("."+C),a=(0,y.getGroupCriteria)(e),r=0;r<a.length;r++){var s=a[r];(0,y.isGroup)(s)?this._createGroupElementByCriteria(s,a,n+1).appendTo(o):(0,y.isCondition)(s)&&this._createConditionElement(s,a).appendTo(o)}return i},_createGroupElement:function(e,t,n){var i=this,a=(0,o.default)("<div>").addClass(w),r=(0,o.default)("<div>").addClass(C),s=(0,o.default)("<div>").addClass(x).append(a).append(r);return null!=t&&this._createRemoveButton(function(){(0,y.removeItem)(t,e),s.remove(),i._updateFilter()}).appendTo(a),this._createGroupOperationButton(e).appendTo(a),this._createAddButton(function(){var t=(0,y.createEmptyGroup)(i.option("defaultGroupOperation"));(0,y.addItem)(t,e),i._createGroupElement(t,e,n+1).appendTo(r),i._updateFilter()},function(){var t=i.option("fields")[0],n=(0,y.createCondition)(t,i._customOperations);(0,y.addItem)(n,e),i._createConditionElement(n,e).appendTo(r),i._updateFilter()},n).appendTo(a),s},_createButton:function(e){return(0,o.default)("<div>").text(e)},_createGroupOperationButton:function(e){var t=this,n=this._getGroupOperations(e),i=(0,y.getGroupMenuItem)(e,n),o=i.text,a=n&&n.length<2?this._createButton(o).addClass(j):this._createButtonWithMenu({caption:o,menu:{items:n,displayExpr:"text",keyExpr:"value",onItemClick:function(n){i!==n.itemData&&((0,y.setGroupValue)(e,n.itemData.value),a.html(n.itemData.text),i=n.itemData,t._updateFilter())},onContentReady:function(e){e.component.selectItem(i)},cssClass:k}});return a.addClass(E).addClass(S).attr("tabindex",0)},_createButtonWithMenu:function(e){var t=this,n=function(){t.$element().find("."+H).removeClass(H),t.$element().find(".dx-overlay .dx-treeview").remove(),t.$element().find(".dx-overlay").remove()},i=this.option("rtlEnabled"),a=function(e){return function(t){e(t),"dxclick"===t.event.type&&n()}},r=i?"right":"left",l=this._createButton(e.caption);return(0,c.extend)(e.menu,{focusStateEnabled:!0,selectionMode:"single",onItemClick:a(e.menu.onItemClick),onHiding:function(e){l.removeClass(H)},position:{my:r+" top",at:r+" bottom",offset:"0 1",of:l,collision:"flip"},animation:null,onHidden:function(){n()},cssClass:B+" "+e.menu.cssClass,rtlEnabled:i}),e.popup={onShown:function(t){var n=(0,o.default)(t.component.content()).find(".dx-treeview"),i=n.dxTreeView("instance");s.default.on(n,"keyup keydown",function(n){var i=(0,v.normalizeKeyName)(n);("keydown"===n.type&&i===W||"keyup"===n.type&&(i===q||i===G))&&(t.component.hide(),s.default.trigger(e.menu.position.of,"focus"))}),i.focus(),i.option("focusedElement",null)}},this._subscribeOnClickAndEnterKey(l,function(){n(),t._createPopupWithTreeView(e,t.$element()),l.addClass(H)}),l},_hasValueButton:function(e){var t=(0,y.getCustomOperation)(this._customOperations,e[1]);return t?t.hasValue!==!1:null!==e[2]},_createOperationButtonWithMenu:function(e,t){var n=this,i=this,a=(0,y.getAvailableOperations)(t,this.option("filterOperationDescriptions"),this._customOperations),r=(0,y.getOperationFromAvailable)((0,y.getOperationValue)(e),a),s=this._createButtonWithMenu({caption:r.text,menu:{items:a,displayExpr:"text",onItemRendered:function(e){e.itemData.isCustom&&(0,o.default)(e.itemElement).addClass(z)},onContentReady:function(e){e.component.selectItem(r)},onItemClick:function(o){if(r!==o.itemData){r=o.itemData,(0,y.updateConditionByOperation)(e,r.value,i._customOperations);var a=s.siblings().filter("."+R);i._hasValueButton(e)?(0!==a.length&&a.remove(),i._createValueButton(e,t).appendTo(s.parent())):a.remove(),s.html(r.text),n._updateFilter()}},cssClass:V}}).addClass(E).addClass(P).attr("tabindex",0);return s},_createOperationAndValueButtons:function(e,t,n){this._createOperationButtonWithMenu(e,t).appendTo(n),this._hasValueButton(e)&&this._createValueButton(e,t).appendTo(n)},_createFieldButtonWithMenu:function(e,t,n){var i=this,o=this,a=this.option("allowHierarchicalFields"),r=(0,y.getItems)(e,a),s=(0,y.getField)(n.name||n.dataField,r),l=function(e,t){return a?(0,y.getCaptionWithParents)(e,t):e.caption},u=this._createButtonWithMenu({caption:l(s,r),menu:{items:r,dataStructure:"plain",keyExpr:"id",parentId:"parentId",displayExpr:"caption",onItemClick:function(e){if(s!==e.itemData){s=e.itemData,t[0]=s.name||s.dataField,t[2]="object"===s.dataType?null:"",(0,y.updateConditionByOperation)(t,(0,y.getDefaultOperation)(s),o._customOperations),u.siblings().filter("."+E).remove(),o._createOperationAndValueButtons(t,s,u.parent());var n=l(s,e.component.option("items"));u.html(n),i._updateFilter()}},onContentReady:function(e){e.component.selectItem(s)},cssClass:F}}).addClass(E).addClass(A).attr("tabindex",0);return u},_createConditionItem:function(e,t){var n=this,i=(0,o.default)("<div>").addClass(w),a=this._getNormalizedFields(),r=(0,y.getField)(e[0],a);return this._createRemoveButton(function(){(0,y.removeItem)(t,e);var o=1===i.parent().children().length;o?i.parent().remove():i.remove(),n._updateFilter()}).appendTo(i),this._createFieldButtonWithMenu(a,e,r).appendTo(i),this._createOperationAndValueButtons(e,r,i),i},_getGroupOperations:function(e){var t=this.option("groupOperations"),n=this.option("groupOperationDescriptions");return t&&t.length||(t=[(0,y.getGroupValue)(e).replace("!","not")]),t.map(function(e){return{text:n[e],value:K[e]}})},_createRemoveButton:function(e){var t=(0,o.default)("<div>").addClass(I).addClass(O).addClass(D).attr("tabindex",0);return this._subscribeOnClickAndEnterKey(t,e),t},_createAddButton:function(e,t,n){var i,o=this.option("maxGroupLevel");return(0,f.isDefined)(o)&&n>=o?(i=this._createButton(),this._subscribeOnClickAndEnterKey(i,t)):i=this._createButtonWithMenu({menu:{items:[{caption:d.default.format("dxFilterBuilder-addCondition"),click:t},{caption:d.default.format("dxFilterBuilder-addGroup"),click:e}],displayExpr:"caption",onItemClick:function(e){e.itemData.click()},cssClass:L}}),i.addClass(I).addClass(T).addClass(D).attr("tabindex",0)},_createValueText:function(e,t,n){var i=this,a=(0,o.default)("<div>").html("&nbsp;").addClass(M).attr("tabindex",0).appendTo(n),r=e[2],s=(0,y.getCustomOperation)(i._customOperations,e[1]);return!s&&t.lookup?(0,y.getCurrentLookupValueText)(t,r,function(e){(0,y.renderValueText)(a,e)}):(0,h.when)((0,y.getCurrentValueText)(t,r,s)).done(function(e){(0,y.renderValueText)(a,e,s)}),i._subscribeOnClickAndEnterKey(a,function(o){"keyup"===o.type&&o.stopPropagation(),i._createValueEditorWithEvents(e,t,n)}),a},_updateConditionValue:function(e,t,n){var i=e[2]!==t;i&&(e[2]=t),n(),this._updateFilter()},_addDocumentKeyUp:function(e,t){var n=!1,i=!1,o=a.default.getDocument(),r=function(e){return n||i?void(i=!1):void t(e)};s.default.on(o,"keyup",r);var l=e.find("input");s.default.on(l,"compositionstart",function(){n=!0}),s.default.on(l,"compositionend",function(){n=!1,i=!0}),s.default.on(l,"keydown",function(e){229!==e.which&&(i=!1)}),this._documentKeyUpHandler=r},_addDocumentClick:function(e,t){var n=this,i=a.default.getDocument(),o=function(i){n._isFocusOnEditorParts(e,i.target)||(s.default.trigger(e.find("input"),"change"),t())};s.default.on(i,"dxpointerdown",o),this._documentClickHandler=o},_isFocusOnEditorParts:function(e,t){var n=t||a.default.getActiveElement();return(0,o.default)(n).closest(e.children()).length||(0,o.default)(n).closest(".dx-dropdowneditor-overlay").length},_removeEvents:function(){var e=a.default.getDocument();(0,f.isDefined)(this._documentKeyUpHandler)&&s.default.off(e,"keyup",this._documentKeyUpHandler),(0,f.isDefined)(this._documentClickHandler)&&s.default.off(e,"dxpointerdown",this._documentClickHandler)},_dispose:function(){this._removeEvents(),this.callBase()},_createValueEditorWithEvents:function(e,t,n){var i=this,o=e[2],a=function(){return n.empty(),i._removeEvents(),i._createValueText(e,t,n)},r=function(){i._updateConditionValue(e,o,function(){a()})},l={value:""===o?null:o,filterOperation:(0,y.getOperationValue)(e),setValue:function(e){o=null===e?"":e},closeEditor:r,text:n.text()};n.empty();var u=this._createValueEditor(n,t,l);s.default.trigger(u.find("input").not(":hidden").eq(0),"focus"),this._removeEvents(),this._addDocumentClick(u,r),this._addDocumentKeyUp(u,function(t){var r=(0,v.normalizeKeyName)(t);if(r===W){if(i._isFocusOnEditorParts(u))return;i._updateConditionValue(e,o,function(){a(),t.shiftKey&&s.default.trigger(n.prev(),"focus")})}r===q&&s.default.trigger(a(),"focus"),r===G&&i._updateConditionValue(e,o,function(){s.default.trigger(a(),"focus")})}),this._fireContentReadyAction()},_createValueButton:function(e,t){var n=(0,o.default)("<div>").addClass(E).addClass(R);return this._createValueText(e,t,n),n},_createValueEditor:function(e,t,n){var i=(0,o.default)("<div>").attr("tabindex",0).appendTo(e),a=(0,y.getCustomOperation)(this._customOperations,n.filterOperation),r=a&&a.editorTemplate?a.editorTemplate:t.editorTemplate;if(r){var s=this._getTemplate(r);s.render({model:(0,c.extend)({field:t},n),container:i})}else this._editorFactory.createEditor.call(this,i,(0,c.extend)({},t,n,{parentType:N}));return i},_createPopupWithTreeView:function(e,t){var n=this,i=(0,o.default)("<div>").addClass(e.menu.cssClass).appendTo(t);this._createComponent(i,g.default,{onHiding:e.menu.onHiding,onHidden:e.menu.onHidden,rtlEnabled:e.menu.rtlEnabled,position:e.menu.position,animation:e.menu.animation,contentTemplate:function(t){var i=(0,o.default)("<div>").appendTo(t);n._createComponent(i,p.default,e.menu),this.repaint()},maxHeight:function(){return(0,_.getElementMaxHeightByWindow)(e.menu.position.of)},visible:!0,focusStateEnabled:!1,closeOnTargetScroll:this.option("closePopupOnTargetScroll"),closeOnOutsideClick:!0,onShown:e.popup.onShown,shading:!1,width:"auto",height:"auto",showTitle:!1})},_subscribeOnClickAndEnterKey:function(e,t){s.default.on(e,"dxclick",t),s.default.on(e,"keyup",function(e){(0,v.normalizeKeyName)(e)===G&&t(e)})}});(0,u.default)("dxFilterBuilder",Y);var Z=Y;t.default=Z,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(494));o.default.registerModule("filterPanel",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(495));o.default.registerModule("search",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(496));o.default.registerModule("pager",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=i(n(15)),s=n(42),l=i(n(9)),u=n(4),c=n(3),d=n(1),h=n(0),f=n(21),p=i(n(25)),g=i(n(10)),_=i(n(16)),m=i(n(127)),v=i(n(126)),y=n(8),b=n(128),x=4,w="dx-pager",C="dx-page",k="."+C,S="dx-pages",D="dx-light-mode",I="dx-light-pages",T="dx-page-index",O="dx-pages-count",E="dx-selection",A="dx-separator",P="dx-page-sizes",R="dx-page-size",M="."+R,B="dx-navigate-button",V="dx-prev-button",F="dx-next-button",L="dx-info",H="dx-info-text",z="dx-button-disable",N=r.default.inherit({ctor:function(e,t){var n=this;n.index=t,n._$page=(0,o.default)("<div>").text(e).addClass(C)},value:function(e){var t=this;if(!(0,d.isDefined)(e)){var n=t._$page.text();return(0,d.isNumeric)(n)?parseInt(n):n}t._$page.text(e)},element:function(){return this._$page},select:function(e){this._$page.toggleClass(E,e)},render:function(e,t){t?this._$page.prependTo(e):this._$page.appendTo(e)}}),j=_.default.inherit({_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{visible:!0,pagesNavigatorVisible:"auto",pageIndex:1,maxPagesCount:10,pageCount:10,totalCount:0,pageSize:5,showPageSizes:!0,pageSizes:[5,10],hasKnownLastPage:!0,showNavigationButtons:!1,showInfo:!1,infoText:g.default.getFormatter("dxPager-infoText"),pagesCountText:g.default.getFormatter("dxPager-pagesCountText"),rtlEnabled:!1,lightModeEnabled:!1,pageIndexChanged:u.noop,pageSizeChanged:u.noop})},_toggleVisibility:function(e){var t=this.$element();t&&t.css("display",e?"":"none")},_getPages:function(e,t){var n,i,o=[],a=!this.option("hasKnownLastPage");if(t>0||a)if(t<=this.option("maxPagesCount")){for(i=1;i<=t;i++)o.push(new N(i,i-1));a&&o.push(new N(">",i-1))}else{for(o.push(new N(1,0)),n=e?e.value()-e.index:1,i=1;i<=x;i++)o.push(new N(n+i,i));o.push(new N(t,x+1)),a&&o.push(new N(">",x+1))}return o},_getPageByValue:function(e){var t,n,i=this;for(n=0;n<i._pages.length;n++)if(t=i._pages[n],t.value()===e)return t},_processSelectedPage:function(e,t,n){var i,o=this,a=!1;o._pages&&((0,c.each)(o._pages,function(e,n){t===n.value()&&(a=!0)}),a||(o.selectedPage=null)),(0,d.isDefined)(o.selectedPage)?t===n&&n>e&&o.selectedPage.index!==x+1&&(o.selectedPage.index=x+1):t>x&&t<n&&(i=n-x<t?x-(n-t)+1:2,o.selectedPage=new N(t,i))},_selectPageByValue:function(e){var t,n,i,o=this,a=o._getPageByValue(e),r=o._pages,s=r.length;if((0,d.isDefined)(a)){var l=o._pages[a.index-1];if(n=o._pages[a.index+1],n&&">"===n.value()&&(i=n,n=void 0,s--,r.pop()),o.selectedPage&&o.selectedPage.select(!1),a.select(!0),o.selectedPage=a,n&&n.value()-e>1)if(0!==a.index)for(l.value(e+1),o._pages.splice(a.index,1),o._pages.splice(a.index-1,0,a),o._pages[a.index].index=a.index,a.index=a.index-1,t=a.index-1;t>0;t--)o._pages[t].value(o._pages[t+1].value()-1);else for(t=0;t<s-1;t++)o._pages[t].value(t+1);if(l&&e-l.value()>1)if(a.index!==s-1)for(n.value(e-1),o._pages.splice(a.index,1),o._pages.splice(a.index+1,0,a),o._pages[a.index].index=a.index,a.index=a.index+1,t=a.index+1;t<s-1;t++)o._pages[t].value(o._pages[t-1].value()+1);else for(t=1;t<=s-2;t++)o._pages[s-1-t].value(o._pages[s-1].value()-t);i&&r.push(i)}},_updatePagesTabIndices:function(){var e=this,t=this.selectedPage._$page,n=function n(){var i=(0,o.default)(e.element()).find("[role=button]:not(.dx-button-disable)");(0,c.each)(i,function(e,t){return(0,o.default)(t).attr("tabindex",0)}),a.default.off(t,"focus",n)};a.default.on(t,"focus",n)},_nextPage:function(e){var t=this.option("pageIndex"),n=this.option("pageCount");(0,d.isDefined)(t)&&(t="next"===e?++t:--t,t>0&&t<=n&&this.option("pageIndex",t))},_wrapClickAction:function(e){var t=this;return function(n){if("dxpointerup"===n.type)t._pointerUpHappened=!0;else if(t._pointerUpHappened)return void(t._pointerUpHappened=!1);e({event:n})}},_renderPages:function(e){var t,n,i=this,r=e.length,s=i._createAction(function(e){var t=e.event,n=(0,o.default)(t.target).text(),a=">"===n?i.option("pageCount")+1:Number(n);i.option("pageIndex",a)});r>1&&(i._pageClickHandler=this._wrapClickAction(s),a.default.on(i._$pagesChooser,(0,y.addNamespace)([p.default.up,f.name],i.Name+"Pages"),k,i._pageClickHandler),(0,b.registerKeyboardAction)("pager",i,i._$pagesChooser,k,s));for(var l=0;l<r;l++)n=e[l],n.render(i._$pagesChooser,i.option("rtlEnabled")),i.setAria({role:"button",label:"Page "+n.value()},n.element()),(0,b.setTabIndex)(i,n.element()),e[l+1]&&e[l+1].value()-n.value()>1&&(t=(0,o.default)("<div>").text(". . .").addClass(A),i.option("rtlEnabled")?t.prependTo(i._$pagesChooser):t.appendTo(i._$pagesChooser))},_calculateLightPagesWidth:function(e,t){return Number(e.css("minWidth").replace("px",""))+10*t.toString().length},_renderLightPages:function(){var e=this,t=this.option("pageCount"),n=this.option("pageIndex"),i=e._createAction(function(){e.option("pageIndex",t)}),r=this.option("pagesCountText"),s=(0,o.default)("<div>").addClass(I).appendTo(this._$pagesChooser),l=(0,o.default)("<div>").addClass(T).appendTo(s);e._pageIndexEditor=e._createComponent(l,v.default,{value:n,min:1,max:t,width:e._calculateLightPagesWidth(l,t),onValueChanged:function(t){null!==t.value&&e.option("pageIndex",t.value)}}),(0,o.default)("<span>").text(r).addClass(H+" "+L).appendTo(s);var u=(0,o.default)("<span>").addClass(O).text(t);a.default.on(u,(0,y.addNamespace)(f.name,e.Name+"PagesCount"),function(e){i({event:e})}),(0,b.registerKeyboardAction)("pager",e,u,void 0,i),u.appendTo(s),e.setAria({role:"button",label:"Navigates to the last page"},u)},_renderPagesChooser:function(){var e=this,t=e.option("lightModeEnabled"),n=e.option("pagesNavigatorVisible"),i=e.$element();if(e._$pagesChooser&&e._$pagesChooser.remove(),n){if(e._pages&&0===e._pages.length)return void(e.selectedPage=null);e._$pagesChooser=(0,o.default)("<div>").addClass(S).appendTo(i),"auto"===n&&e._$pagesChooser.css("visibility",1===e.option("pageCount")?"hidden":""),t||e._renderInfo(),e._renderNavigateButton("prev"),t?e._renderLightPages():e._renderPages(e._pages),e._renderNavigateButton("next"),e._updatePagesChooserWidth()}},_renderPageSizes:function(){var e,t,n,i=this,r=i.option("pageSizes"),s=r&&r.length,l=i.option("pageSize"),u=i._createAction(function(e){var n=e.event;t=parseInt((0,o.default)(n.target).text()),i.option("pageSize",t)});for(a.default.on(i._$pagesSizeChooser,(0,y.addNamespace)(f.name,i.Name+"PageSize"),M,function(e){u({event:e})}),(0,b.registerKeyboardAction)("pager",i,i._$pagesSizeChooser,M,u),e=0;e<s;e++)n=(0,o.default)("<div>").text(r[e]).addClass(R),i.setAria({role:"button",label:"Display "+r[e]+" items on page"},n),(0,b.setTabIndex)(i,n),l===r[e]&&n.addClass(E),i._$pagesSizeChooser.append(n)},_calculateLightPageSizesWidth:function(e){return Number(this._$pagesSizeChooser.css("minWidth").replace("px",""))+10*Math.max.apply(Math,e).toString().length},_renderLightPageSizes:function(){var e=this,t=e.option("pageSizes"),n=(0,o.default)("<div>").appendTo(e._$pagesSizeChooser);e._pageSizeEditor=e._createComponent(n,m.default,{dataSource:t,value:e.option("pageSize"),onSelectionChanged:function(t){e.option("pageSize",t.selectedItem)},width:e._calculateLightPageSizesWidth(t)})},_renderPagesSizeChooser:function(){var e=this,t=e.option("pageSizes"),n=e.option("showPageSizes"),i=t&&t.length,a=e.$element();e._$pagesSizeChooser&&e._$pagesSizeChooser.remove(),n&&i&&(e._$pagesSizeChooser=(0,o.default)("<div>").addClass(P).appendTo(a),e.option("lightModeEnabled")?e._renderLightPageSizes():e._renderPageSizes(),e._pagesSizeChooserWidth=e._$pagesSizeChooser.width())},_renderInfo:function(){var e=this.option("infoText");this.option("showInfo")&&(0,d.isDefined)(e)&&(this._$info=(0,o.default)("<div>").css("display",this._isInfoHide?"none":"").addClass(L).text((0,s.format)(e,this.selectedPage&&this.selectedPage.value(),this.option("pageCount"),this.option("totalCount"))).appendTo(this._$pagesChooser),this._isInfoHide||(this._infoWidth=this._$info.outerWidth(!0)))},_renderNavigateButton:function(e){var t,n=this,i=n._createAction(function(){n._nextPage(e)});(n.option("showNavigationButtons")||n.option("lightModeEnabled"))&&(t=(0,o.default)("<div>").addClass(B),a.default.on(t,(0,y.addNamespace)([p.default.up,f.name],n.Name+"Pages"),n._wrapClickAction(i)),(0,b.registerKeyboardAction)("pager",n,t,void 0,i),n.setAria({role:"button",label:"prev"===e?"Previous page":" Next page"},t),(0,b.setTabIndex)(n,t),n.option("rtlEnabled")?(t.addClass("prev"===e?F:V),t.prependTo(this._$pagesChooser)):(t.addClass("prev"===e?V:F),t.appendTo(this._$pagesChooser)))},_renderContentImpl:function(){this.$element().toggleClass(D,this.option("lightModeEnabled")),this._toggleVisibility(this.option("visible")),this._updatePageSizes(!0),this._updatePages(!0),(0,b.restoreFocus)(this)},_initMarkup:function(){var e=this.$element();e.addClass(w);var t=(0,o.default)("<div>").addClass(C);this._$pagesChooser=(0,o.default)("<div>").addClass(S).append(t).appendTo(e)},_render:function(){this.option().lightModeEnabled=!1,this.callBase(),this._updateLightMode()},_updatePageSizes:function(e){var t=this.option("lightModeEnabled"),n=this.option("pageSize"),i=this.option("pageSizes");t&&this._pageSizeEditor&&this._pageSizeEditor.option({value:n,dataSource:i,width:this._calculateLightPageSizesWidth(i)}),t&&!e||this._renderPagesSizeChooser()},_updatePages:function(e){var t=this.option("pageCount"),n=this.option("pageIndex"),i=this.option("lightModeEnabled");i?this._pageIndexEditor&&this._pageIndexEditor.option({value:n,width:this._calculateLightPagesWidth(this._pageIndexEditor.$element(),t)}):(this._processSelectedPage(this.option("maxPagesCount"),n,t),this._pages=this._getPages(this.selectedPage,t),this._selectPageByValue(n)),i&&!e||this._renderPagesChooser(),this._updateButtonsState(n)},_isPageIndexInvalid:function(e,t){var n="next"===e,i=this.option("rtlEnabled");return i&&n||!i&&!n?t<=1:t>=this.option("pageCount")},_updateButtonsState:function(e){var t=this.$element().find("."+F),n=this.$element().find("."+V);t.toggleClass(z,this._isPageIndexInvalid("next",e)),n.toggleClass(z,this._isPageIndexInvalid("prev",e))},_optionChanged:function(e){switch(e.name){case"visible":this._toggleVisibility(e.value);break;case"pageIndex":var t=this.option("pageIndexChanged");t&&t(e.value),this._updatePages();break;case"maxPagesCount":case"pageCount":case"totalCount":case"hasKnownLastPage":case"pagesNavigatorVisible":case"showNavigationButtons":this._updatePages();break;case"pageSize":var n=this.option("pageSizeChanged");n&&n(e.value),this._updatePageSizes();break;case"pageSizes":this._updatePageSizes();break;case"lightModeEnabled":this._renderContentImpl(),!e.value&&this._updateLightMode();break;default:this._invalidate()}},_clean:function(){this._$pagesChooser&&(a.default.off(this._$pagesChooser,(0,y.addNamespace)([p.default.up,f.name],this.Name+"Pages"),k,this._pageClickHandler),(0,b.registerKeyboardAction)("pager",this,this._$pagesChooser,k,this._pageKeyDownHandler)),this.callBase()},_getMinPagerWidth:function(){var e=(0,d.isDefined)(this._pagesChooserWidth)?this._pagesChooserWidth:0,t=(0,d.isDefined)(this._pagesSizeChooserWidth)?this._pagesSizeChooserWidth:0;return e+t},_updatePagesChooserWidth:(0,u.deferUpdater)(function(){var e=this._pages&&this._pages.length>0?this._pages[this._pages.length-1]._$page.width():0;this._pagesChooserWidth=this._$pagesChooser.width()+e}),_updateLightMode:(0,u.deferUpdater)(function(){var e=this,t=this.$element().width(),n=(0,d.isDefined)(this._infoWidth)?this._infoWidth:0;(0,u.deferRender)(function(){e._isInfoHide&&t>e._getMinPagerWidth()+n&&(e._$info.show(),e._updatePagesChooserWidth(),e._isInfoHide=!1),!e._isInfoHide&&t>e._getMinPagerWidth()-n&&t<e._getMinPagerWidth()&&(e._$info.hide(),e._updatePagesChooserWidth(),e._isInfoHide=!0),(0,u.deferUpdate)(function(){(0,u.deferRender)(function(){e.option("lightModeEnabled")&&t>e._previousWidth?e.option("lightModeEnabled",!1):t<e._getMinPagerWidth()&&e.option("lightModeEnabled",!0),e._previousWidth=t})})})}),_dimensionChanged:function(){this._updateLightMode()},getHeight:function(){return this.option("visible")?this.$element().outerHeight():0}}),W=j;t.default=W,(0,l.default)("dxPager",j),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.TrackerView=t.ColumnsResizerViewController=t.TablePositionViewController=t.ColumnsSeparatorView=t.DraggingHeaderViewController=t.DraggingHeaderView=void 0;var o=i(n(26)),a=i(n(497)),r=a.default.views.draggingHeaderView;t.DraggingHeaderView=r;var s=a.default.controllers.draggingHeader;t.DraggingHeaderViewController=s;var l=a.default.views.columnsSeparatorView;t.ColumnsSeparatorView=l;var u=a.default.controllers.tablePosition;t.TablePositionViewController=u;var c=a.default.controllers.columnsResizer;t.ColumnsResizerViewController=c;var d=a.default.views.trackerView;t.TrackerView=d,o.default.registerModule("columnsResizingReordering",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(498));o.default.registerModule("keyboardNavigation",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.FooterView=t.renderSummaryCell=void 0;var o=i(n(2)),a=n(4),r=n(1),s=n(3),l=n(0),u=n(19),c=i(n(17)),d=i(n(26)),h=i(n(10)),f=i(n(222)),p=n(197),g=i(n(729)),_=i(n(47)),m=i(n(91)),v=i(n(41)),y="dx-datagrid-total-footer",b="dx-datagrid-summary-item",x="dx-datagrid-text-content",w="dx-datagrid-group-footer",C="dx-datagrid-group-text-content",k="dx-datagrid-nowrap",S="groupFooter",D=function(e,t){
var n=(0,o.default)(e),i=t.column,a=t.summaryItems,r=[];if(!i.command&&a){for(var s=0;s<a.length;s++){var l=a[s];r.push((0,o.default)("<div>").css("textAlign",l.alignment||i.alignment).addClass(b).addClass(x).addClass(l.cssClass).toggleClass(C,"group"===t.rowType).text(d.default.getSummaryText(l,t.summaryTexts)))}n.append(r)}};t.renderSummaryCell=D;var I=function(e,t){var n=e.option("summary.texts")||{};return{totalItem:t.row,summaryItems:t.row.summaryCells[t.columnIndex],summaryTexts:n}},T=function(e){return e.summary||e.aggregates||[]},O=function(e){return e.option("summary.recalculateWhileEditing")},E=p.ColumnsView.inherit(function(){return{_getRows:function(){return this._dataController.footerItems()},_getCellOptions:function(e){return(0,l.extend)(this.callBase(e),I(this,e))},_renderCellContent:function(e,t){D(e,t),this.callBase(e,t)},_renderCore:function(e){var t=this._dataController.footerItems()[0];e&&e.columnIndices||this.element().empty().addClass(y).toggleClass(k,!this.option("wordWrapEnabled")),t&&t.summaryCells&&t.summaryCells.length&&this._updateContent(this._renderTable({change:e}),e)},_updateContent:function(e,t){if(!t||"update"!==t.changeType||!t.columnIndices)return this.callBase.apply(this,arguments);var n=this._getTableElement().find(".dx-row"),i=e.find(".dx-row");this._updateCells(n,i,t.columnIndices[0])},_rowClick:function(e){var t=this._dataController.footerItems()[e.rowIndex]||{};this.executeAction("onRowClick",(0,l.extend)({},e,t))},_columnOptionChanged:function(e){var t=e.optionNames;e.changeTypes.grouping||(t.width||t.visibleWidth)&&this.callBase(e)},_handleDataChanged:function(e){var t=e.changeType;"update"===e.changeType&&e.repaintChangesOnly?e.totalColumnIndices?e.totalColumnIndices.length&&this.render(null,{changeType:"update",columnIndices:[e.totalColumnIndices]}):this.render():"refresh"!==t&&"append"!==t&&"prepend"!==t||this.render()},getHeight:function(){return this.getElementHeight()},isVisible:function(){return!!this._dataController.footerItems().length}}}());t.FooterView=E;var A=function(){function e(t,n,i,o){o=o||[];for(var a=0;a<t.length;a++)o.push(t[a].key),1===n?i(o,t[a].items):e(t[a].items,n-1,i,o),o.pop()}return{init:function(){this.callBase.apply(this,arguments),this._totalAggregates=[],this._summaryGetter=a.noop},summaryGetter:function(e){return arguments.length?void((0,r.isFunction)(e)&&(this._summaryGetter=e)):this._summaryGetter},summary:function(e){return arguments.length?void(this._summaryGetter=function(){return e}):this._summaryGetter()},totalAggregates:function(){return this._totalAggregates},isLastLevelGroupItemsPagingLocal:function(){var e=this.summary(),t=e&&e.sortByGroups();return t&&t.length},sortLastLevelGroupItems:function(t,n,i){var o=m.default.multiLevelGroup((0,_.default)(t),n).toArray(),a=[];return i.forEach(function(t){e(o,n.length,function(e,n){t.toString()===e.toString()&&(a=a.concat(n))})}),a}}}(),P=function(){var e=function e(t,n,i){return i?e(t,n.map(function(e){return{items:[e]}},i-1)):t.concat(n)},t=function e(t,n,i){return i?t.map(function(t){var o={},a=e(t.items||[],n,i-1);return Object.defineProperty(o,"aggregates",{get:function(){return t.aggregates},set:function(e){t.aggregates=e}}),(0,l.extend)(o,t,{items:a})}):t.filter(function(e){return n.indexOf(e)<0})},n=function(n,i,o,a){var r;if(O(n)){var s=n.getController("editing");if(s){var l=s.getInsertedData();l.length&&(o=e(o,l,a));var u=s.getRemovedData();u.length&&(o=t(o,u,a))}}return i&&(r=new g.default({totalAggregates:i.totalAggregates,groupAggregates:i.groupAggregates,data:o,groupLevel:a}),r.calculate()),r?r.totalAggregates():[]},i=function e(t,n,i){if(!t||!n.length)return t;var o,a=n[0],r=i[0];return a&&r&&r.length&&(o=(0,_.default)(t),(0,s.each)(r,function(e){o=0===e?o.sortBy(this.selector,this.desc):o.thenBy(this.selector,this.desc)}),o.enumerate().done(function(e){t=e})),n=n.slice(1),i=i.slice(1),n.length&&i.length&&(0,s.each)(t,function(){this.items=e(this.items,n,i)}),t},o=function(e,t,n){var o=n&&n.sortByGroups&&n.sortByGroups();return o&&o.length?i(e,t,o):e};return{_customizeRemoteOperations:function(e){var t=this.summary();t&&(e.remoteOperations.summary?e.isCustomLoading&&!e.storeLoadOptions.isLoadingAll||(e.storeLoadOptions.group&&(e.remoteOperations.grouping?e.storeLoadOptions.groupSummary=t.groupAggregates:t.groupAggregates.length&&(e.remoteOperations.paging=!1)),e.storeLoadOptions.totalSummary=t.totalAggregates):(t.totalAggregates.length||t.groupAggregates.length&&e.storeLoadOptions.group)&&(e.remoteOperations.paging=!1)),this.callBase.apply(this,arguments);var n=e.cachedPagesData.extra;n&&n.summary&&!e.isCustomLoading&&(e.storeLoadOptions.totalSummary=void 0)},_handleDataLoadedCore:function(e){var t,i=this,a=v.default.normalizeSortingInfo(e.storeLoadOptions.group||e.loadOptions.group||[]),s=e.remoteOperations||{},l=i.summaryGetter()(s);e.isCustomLoading&&!e.storeLoadOptions.isLoadingAll||(s.summary?!s.paging&&a.length&&l&&(s.grouping||n(i,{groupAggregates:l.groupAggregates},e.data,a.length),e.data=o(e.data,a,l)):s.paging||(t=n(i,l,e.data,a.length),e.data=o(e.data,a,l),e.extra=(0,r.isPlainObject)(e.extra)?e.extra:{},e.extra.summary=t)),e.isCustomLoading||(i._totalAggregates=e.extra&&e.extra.summary||i._totalAggregates),i.callBase(e)}}}();f.default.extend(A),f.default.extend(P),d.default.registerModule("summary",{defaultOptions:function(){return{summary:{groupItems:void 0,totalItems:void 0,calculateCustomSummary:void 0,skipEmptyValues:!0,recalculateWhileEditing:!1,texts:{sum:h.default.getFormatter("dxDataGrid-summarySum"),sumOtherColumn:h.default.getFormatter("dxDataGrid-summarySumOtherColumn"),min:h.default.getFormatter("dxDataGrid-summaryMin"),minOtherColumn:h.default.getFormatter("dxDataGrid-summaryMinOtherColumn"),max:h.default.getFormatter("dxDataGrid-summaryMax"),maxOtherColumn:h.default.getFormatter("dxDataGrid-summaryMaxOtherColumn"),avg:h.default.getFormatter("dxDataGrid-summaryAvg"),avgOtherColumn:h.default.getFormatter("dxDataGrid-summaryAvgOtherColumn"),count:h.default.getFormatter("dxDataGrid-summaryCount")}},sortByGroupSummaryInfo:void 0}},views:{footerView:E},extenders:{controllers:{data:function(){return{_isDataColumn:function(e){return e&&(!(0,r.isDefined)(e.groupIndex)||e.showWhenGrouped)},_isGroupFooterVisible:function(){for(var e=this.option("summary.groupItems")||[],t=0;t<e.length;t++){var n=e[t],i=this._columnsController.columnOption(n.showInColumn||n.column);if(n.showInGroupFooter&&this._isDataColumn(i))return!0}return!1},_processGroupItems:function(e,t,n){var i=n&&n.data,o=this.callBase.apply(this,arguments);return n&&(void 0===n.isGroupFooterVisible&&(n.isGroupFooterVisible=this._isGroupFooterVisible()),i&&i.items&&n.isGroupFooterVisible&&(n.collectContinuationItems||!i.isContinuationOnNextPage)&&o.push({rowType:S,key:n.path.slice(),data:i,groupIndex:n.path.length-1,values:[]})),o},_processGroupItem:function(e,t){var n=this;if(t.summaryGroupItems||(t.summaryGroupItems=n.option("summary.groupItems")||[]),"group"===e.rowType){var i=-1,o=-1;(0,s.each)(t.visibleColumns,function(n){var a=t.visibleColumns[n-1];e.groupIndex===this.groupIndex&&(i=this.index),n>0&&"expand"===a.command&&"expand"!==this.command&&(o=this.index)}),e.summaryCells=this._calculateSummaryCells(t.summaryGroupItems,T(e.data),t.visibleColumns,function(e,t){return e.showInGroupFooter?-1:e.alignByColumn&&t&&!(0,r.isDefined)(t.groupIndex)&&t.index!==o?t.index:i})}return e.rowType===S&&(e.summaryCells=this._calculateSummaryCells(t.summaryGroupItems,T(e.data),t.visibleColumns,function(e,t){return e.showInGroupFooter&&n._isDataColumn(t)?t.index:-1})),e},_calculateSummaryCells:function(e,t,n,i){var o=this,a=[],u={};return(0,s.each)(e,function(e,n){var a=o._columnsController.columnOption(n.column),s=n.showInColumn&&o._columnsController.columnOption(n.showInColumn)||a,c=i(n,s);if(c>=0){u[c]||(u[c]=[]);var h=t[e];if(h===h){var f;(0,r.isDefined)(n.valueFormat)?f=n.valueFormat:"count"!==n.summaryType&&(f=d.default.getFormatByDataType(a&&a.dataType)),u[c].push((0,l.extend)({},n,{value:(0,r.isString)(h)&&a&&a.deserializeValue?a.deserializeValue(h):h,valueFormat:f,columnCaption:a&&a.index!==c?a.caption:void 0}))}}}),(0,r.isEmptyObject)(u)||(0,s.each)(n,function(){a.push(u[this.index]||[])}),a},_getSummaryCells:function(e,t){var n=this,i=n._columnsController;return n._calculateSummaryCells(e,t,i.getVisibleColumns(),function(e,t){return n._isDataColumn(t)?t.index:-1})},_updateItemsCore:function(e){var t,n=this,i=n._dataSource,o=n._footerItems,a=o&&o[0]&&o[0].summaryCells,r=n.option("summary.totalItems");if(n._footerItems=[],i&&r&&r.length){var s=i.totalAggregates();t=n._getSummaryCells(r,s),e&&e.repaintChangesOnly&&a&&(e.totalColumnIndices=t.map(function(e,t){return JSON.stringify(e)!==JSON.stringify(a[t])?t:-1}).filter(function(e){return e>=0})),t.length&&n._footerItems.push({rowType:"totalFooter",summaryCells:t})}n.callBase(e)},_prepareUnsavedDataSelector:function(e){var t=this;if(O(t)){var n=t.getController("editing");if(n)return function(t){return t=n.getUpdatedData(t),e(t)}}return e},_prepareAggregateSelector:function(e,t){return e=this._prepareUnsavedDataSelector(e),"avg"===t||"sum"===t?function(t){var n=e(t);return(0,r.isDefined)(n)?Number(n):n}:e},_getAggregates:function(e,t){var n=this,i=n.getController("columns"),o=n.option("summary.calculateCustomSummary"),a=n.option("summary.skipEmptyValues");return(0,s.map)(e||[],function(e){var s=i.columnOption(e.column),l=s&&s.calculateCellValue?s.calculateCellValue.bind(s):(0,u.compileGetter)(s?s.dataField:e.column),d=e.summaryType||"count",h=e.column,f=(0,r.isDefined)(e.skipEmptyValues)?e.skipEmptyValues:a;if(t)return{selector:e.column,summaryType:d};if(h=n._prepareAggregateSelector(l,d),"custom"===d){o||(c.default.log("E1026"),o=function(){});var p={component:n.component,name:e.name};o(p),p.summaryProcess="calculate",d={seed:function(e){return p.summaryProcess="start",p.totalValue=void 0,p.groupIndex=e,delete p.value,o(p),p.totalValue},step:function(e,t){return p.summaryProcess="calculate",p.totalValue=e,p.value=t,o(p),p.totalValue},finalize:function(e){return p.summaryProcess="finalize",p.totalValue=e,delete p.value,o(p),p.totalValue}}}return{selector:h,aggregator:d,skipEmptyValues:f}})},_addSortInfo:function(e,t,n,i){if(t){var o=t.groupIndex;i=i||t.sortOrder,(0,r.isDefined)(o)&&(e[o]=e[o]||[],e[o].push({selector:n,desc:"desc"===i}))}},_findSummaryItem:function(e,t){var n=-1,i=function(e){var t=e.summaryType,n=e.column;return t&&n&&t+"_"+n};return(0,r.isDefined)(t)&&(0,s.each)(e||[],function(e){if(this.name===t||e===t||this.summaryType===t||this.column===t||i(this)===t)return n=e,!1}),n},_getSummarySortByGroups:function(e,t){var n=this,i=n._columnsController,o=i.getGroupColumns(),a=[];if(t&&t.length)return(0,s.each)(e||[],function(){var e=this.sortOrder,l=this.groupColumn,u=n._findSummaryItem(t,this.summaryItem);if(!(u<0)){var c=function(e){return T(e)[u]};(0,r.isDefined)(l)?(l=i.columnOption(l),n._addSortInfo(a,l,c,e)):(0,s.each)(o,function(t,i){n._addSortInfo(a,i,c,e)})}}),a},_createDataSourceAdapterCore:function(e,t){var n=this,i=this.callBase(e,t);return i.summaryGetter(function(e){return n._getSummaryOptions(e||t)}),i},_getSummaryOptions:function(e){var t=this,n=t.option("summary.groupItems"),i=t.option("summary.totalItems"),o=t.option("sortByGroupSummaryInfo"),a=t._getAggregates(n,e&&e.grouping&&e.summary),r=t._getAggregates(i,e&&e.summary),s=function(){return t._getSummarySortByGroups(o,n)};if(a.length||r.length)return{groupAggregates:a,totalAggregates:r,sortByGroups:s}},publicMethods:function(){var e=this.callBase();return e.push("getTotalSummaryValue"),e},getTotalSummaryValue:function(e){var t=this._findSummaryItem(this.option("summary.totalItems"),e),n=this._dataSource.totalAggregates();if(n.length&&t>-1)return n[t]},optionChanged:function(e){"summary"!==e.name&&"sortByGroupSummaryInfo"!==e.name||(e.name="dataSource"),this.callBase(e)},init:function(){this._footerItems=[],this.callBase()},footerItems:function(){return this._footerItems}}}(),editing:function(){return{_refreshSummary:function(){O(this)&&!this.isSaving()&&this._dataController.refresh({load:!0,changesOnly:!0})},_addChange:function(e){var t=this.callBase.apply(this,arguments);return e.type&&this._refreshSummary(),t},_removeChange:function(){var e=this.callBase.apply(this,arguments);return this._refreshSummary(),e},cancelEditData:function(){var e=this.callBase.apply(this,arguments);return this._refreshSummary(),e}}}()},views:{rowsView:function(){return{_createRow:function(e){var t=this.callBase(e);return e&&t.addClass(e.rowType===S?w:""),t},_renderCells:function(e,t){this.callBase.apply(this,arguments),"group"===t.row.rowType&&t.row.summaryCells&&t.row.summaryCells.length&&this._renderGroupSummaryCells(e,t)},_hasAlignByColumnSummaryItems:function(e,t){return!(0,r.isDefined)(t.columns[e].groupIndex)&&t.row.summaryCells[e].length},_getAlignByColumnCellCount:function(e,t){for(var n=0,i=1;i<e;i++){var o=t.row.summaryCells.length-i;n=this._hasAlignByColumnSummaryItems(o,t)?i:n}return n},_renderGroupSummaryCells:function(e,t){var n=e.children().last(),i=Number(n.attr("colSpan"))||1,o=this._getAlignByColumnCellCount(i,t);this._renderGroupSummaryCellsCore(n,t,i,o)},_renderGroupSummaryCellsCore:function(e,t,n,i){if(i>0){e.attr("colSpan",n-i);for(var o=0;o<i;o++){var a=t.columns.length-i+o;this._renderCell(e.parent(),(0,l.extend)({column:t.columns[a],columnIndex:this._getSummaryCellIndex(a,t.columns)},t))}}},_getSummaryCellIndex:function(e){return e},_getCellTemplate:function(e){return!e.column.command&&!(0,r.isDefined)(e.column.groupIndex)&&e.summaryItems&&e.summaryItems.length?D:this.callBase(e)},_getCellOptions:function(e){var t=this,n=t.callBase(e);return e.row.summaryCells?(0,l.extend)(n,I(t,e)):n}}}()}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){var a=0;if(e<t)for(;a<n.items.length;a++)o(e+1,t,n.items[a],i);e===t&&i(n)}function a(e,t){var n;if("map"in e)return e.map(t);var i=new Array(e.length);for(n in e)i[n]=t(e[n],n);return i}function r(e){return e!==e||""===e||null===e||void 0===e}function s(e){return e===f.default.aggregators.count}function l(e){var t=(0,c.compileGetter)(e.selector),n=!("skipEmptyValues"in e)||e.skipEmptyValues,i=e.aggregator;if("string"==typeof i&&(i=f.default.aggregators[i],!i))throw h.default.errors.Error("E4001",e.aggregator);return{selector:t,aggregator:i,skipEmptyValues:n}}t.default=void 0;var u=i(n(15)),c=n(19),d=n(1),h=i(n(34)),f=i(n(41)),p=u.default.inherit({ctor:function(e){this._data=e.data,this._groupLevel=e.groupLevel||0,this._totalAggregates=a(e.totalAggregates||[],l),this._groupAggregates=a(e.groupAggregates||[],l),this._totals=[]},calculate:function(){this._totalAggregates.length&&this._calculateTotals(0,{items:this._data}),this._groupAggregates.length&&this._groupLevel>0&&this._calculateGroups({items:this._data})},totalAggregates:function(){return this._totals},_aggregate:function(e,t,n){for(var i=t.items?t.items.length:0,o=0;o<e.length;o++)if(s(e[o].aggregator))n[o]=(n[o]||0)+i;else for(var a=0;a<i;a++)this._accumulate(o,e[o],n,t.items[a])},_calculateTotals:function(e,t){if(0===e&&(this._totals=this._seed(this._totalAggregates)),e===this._groupLevel)this._aggregate(this._totalAggregates,t,this._totals);else for(var n=0;n<t.items.length;n++)this._calculateTotals(e+1,t.items[n]);0===e&&(this._totals=this._finalize(this._totalAggregates,this._totals))},_calculateGroups:function(e){function t(e){e.aggregates=a(i-1),i===n?r(e,e.aggregates):o(i,n,e,function(t){r(t,e.aggregates)}),e.aggregates=s(e.aggregates)}for(var n=this._groupLevel,i=n+1,a=this._seed.bind(this,this._groupAggregates),r=this._aggregate.bind(this,this._groupAggregates),s=this._finalize.bind(this,this._groupAggregates);--i>0;)o(0,i,e,t)},_seed:function(e,t){return a(e,function(e){var n=e.aggregator,i="seed"in n?(0,d.isFunction)(n.seed)?n.seed(t):n.seed:NaN;return i})},_accumulate:function(e,t,n,i){var o=t.selector(i),a=t.aggregator,s=t.skipEmptyValues;s&&r(o)||(n[e]!==n[e]?n[e]=o:n[e]=a.step(n[e],o))},_finalize:function(e,t){return a(e,function(e,n){var i=e.aggregator.finalize;return i?i(t[n]):t[n]})}});t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(499));o.default.registerModule("columnFixing",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(500));o.default.registerModule("adaptivity",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(501));o.default.registerModule("virtualColumns",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.ExportController=t.DataProvider=void 0;var o=i(n(2)),a=i(n(15)),r=n(1),s=n(0),l=n(24),u=n(12),c=i(n(26)),d=i(n(503)),h=n(181),f=i(n(10)),p=i(n(38)),g=i(n(115)),_=i(n(110)),m=n(6),v="dx-datagrid-export-menu",y="dx-datagrid-export-button",b="export-to",x="xlsxfile",w="exportselected",C="export-excel-button",k="dx-toolbar-item-auto-hide",S="dx-toolbar-hidden-button",D="dx-button",I=3,T=a.default.inherit({_getGroupValue:function(e){var t=e.key,n=e.data,i=e.rowType,o=e.groupIndex,a=e.summaryCells,r=this._options.groupColumns[o],s=c.default.getDisplayValue(r,r.deserializeValue?r.deserializeValue(t[o]):t[o],n,i),l=r.caption+": "+c.default.formatValue(s,r);return a&&a[0]&&a[0].length&&(l+=" "+c.default.getGroupRowSummaryText(a[0],this._options.summaryTexts)),l},_correctCellIndex:function(e){return e},_initOptions:function(){var e=this._exportController,t=e._columnsController.getGroupColumns(),n=e.option("export.excelWrapTextEnabled");this._options={columns:e._getColumns(this._initialColumnWidthsByColumnIndex),groupColumns:t,items:this._selectedRowsOnly||e._selectionOnly?e._getSelectedItems():e._getAllItems(),getVisibleIndex:e._columnsController.getVisibleIndex.bind(e._columnsController),isHeadersVisible:e.option("showColumnHeaders"),summaryTexts:e.option("summary.texts"),customizeExportData:e.option("customizeExportData"),rtlEnabled:e.option("rtlEnabled"),wrapTextEnabled:(0,r.isDefined)(n)?n:!!e.option("wordWrapEnabled"),customizeExcelCell:e.option("export.customizeExcelCell")}},hasCustomizeExcelCell:function(){return(0,r.isDefined)(this._options.customizeExcelCell)},customizeExcelCell:function(e,t){this._options.customizeExcelCell&&(e.gridCell=t,(0,r.isDefined)(this._exportController)&&(0,r.isDefined)(this._exportController.component)&&(e.component=this._exportController.component),this._options.customizeExcelCell(e))},ctor:function(e,t,n){this._exportController=e,this._initialColumnWidthsByColumnIndex=t,this._selectedRowsOnly=n},getStyles:function(){var e=this._options.wrapTextEnabled,t=["center","left","right"].map(function(e){return{bold:!0,alignment:e,wrapText:!0}});return this.getColumns().forEach(function(n){t.push({alignment:n.alignment||"left",format:n.format,wrapText:e,dataType:n.dataType})}),t.push({bold:!0,wrapText:!1,alignment:(0,l.getDefaultAlignment)(this._options.rtlEnabled)}),t},_getTotalCellStyleId:function(e){var t=this.getColumns()[e]&&this.getColumns()[e].alignment||"right";return["center","left","right"].indexOf(t)},getStyleId:function(e,t){return e<this.getHeaderRowCount()?0:this.isTotalCell(e-this.getHeaderRowCount(),t)?this._getTotalCellStyleId(t):this.isGroupRow(e-this.getHeaderRowCount())?I+this.getColumns().length:t+I},getColumns:function(e){var t=this._options.columns;return e?t:t[t.length-1]},getColumnsWidths:function(){var e=this.getColumns();return(0,r.isDefined)(e)?e.map(function(e){return e.width}):void 0},getRowsCount:function(){return this._options.items.length+this.getHeaderRowCount()},getHeaderRowCount:function(){return this.isHeadersVisible()?this._options.columns.length-1:0},isGroupRow:function(e){return e<this._options.items.length&&"group"===this._options.items[e].rowType},getGroupLevel:function(e){var t=this._options.items[e-this.getHeaderRowCount()],n=t&&t.groupIndex;return t&&"totalFooter"===t.rowType?0:(0,r.isDefined)(n)?n:this._options.groupColumns.length},getCellType:function(e,t){var n=this.getColumns();if(e<this.getHeaderRowCount())return"string";if(e-=this.getHeaderRowCount(),t<n.length){var i=this._options.items.length&&this._options.items[e],o=n[t];return i&&"data"===i.rowType&&isFinite(i.values[this._correctCellIndex(t)])&&!(0,r.isDefined)(o.customizeText)?(0,r.isDefined)(o.lookup)?o.lookup.dataType:o.dataType:"string"}},ready:function(){var e=this;e._initOptions();var t=e._options;return(0,m.when)(t.items).done(function(n){t.customizeExportData&&t.customizeExportData(e.getColumns(e.getHeaderRowCount()>1),n),t.items=n}).fail(function(){t.items=[]})},_convertFromGridGroupSummaryItems:function(e){if((0,r.isDefined)(e)&&e.length>0)return e.map(function(e){return{value:e.value,name:e.name}})},getCellData:function(e,t,n){var i,o,a={cellSourceData:{},value:o},s=this.getColumns(),l=this._correctCellIndex(t);if(e<this.getHeaderRowCount()){var u=this.getColumns(!0)[e];i=u[t],a.cellSourceData.rowType="header",a.cellSourceData.column=i&&i.gridColumn,a.value=i&&i.caption}else{e-=this.getHeaderRowCount();var d=this._options.items.length&&this._options.items[e];if(d){var h=d.values;switch(a.cellSourceData.rowType=d.rowType,a.cellSourceData.column=s[t]&&s[t].gridColumn,d.rowType){case"groupFooter":case"totalFooter":l<h.length&&(o=h[l],(0,r.isDefined)(o)?(a.cellSourceData.value=o.value,a.cellSourceData.totalSummaryItemName=o.name,a.value=c.default.getSummaryText(o,this._options.summaryTexts)):a.cellSourceData.value=void 0);break;case"group":if(a.cellSourceData.groupIndex=d.groupIndex,t<1)a.cellSourceData.column=this._options.groupColumns[d.groupIndex],a.cellSourceData.value=d.key[d.groupIndex],a.cellSourceData.groupSummaryItems=this._convertFromGridGroupSummaryItems(d.summaryCells[0]),a.value=this._getGroupValue(d);else{var f=d.values[l];if(Array.isArray(f)){a.cellSourceData.groupSummaryItems=this._convertFromGridGroupSummaryItems(f),o="";for(var p=0;p<f.length;p++)o+=(p>0?n?"\n":" \n ":"")+c.default.getSummaryText(f[p],this._options.summaryTexts);a.value=o}else a.cellSourceData.value=void 0}break;default:if(i=s[t]){var g=h[l],_=c.default.getDisplayValue(i,g,d.data,d.rowType);!isFinite(_)||(0,r.isDefined)(i.customizeText)?n&&(0,r.isDefined)(i.customizeText)&&i.customizeText===this._exportController._columnsController.getCustomizeTextByDataType("boolean")?a.value=_:a.value=c.default.formatValue(_,i):a.value=_,a.cellSourceData.value=g}a.cellSourceData.data=d.data}}}return a},isHeadersVisible:function(){return this._options.isHeadersVisible},isTotalCell:function(e,t){var n=this._options.items,i=n[e],o=this._correctCellIndex(t),a=i.summaryCells&&i.summaryCells[o]&&i.summaryCells[o].length>0&&i.summaryCells[o][0].alignByColumn;return i&&"groupFooter"===i.rowType||"totalFooter"===i.rowType||a},getCellMerging:function(e,t){var n=this._options.columns,i=n[e]&&n[e][t];return i?{colspan:(i.exportColspan||1)-1,rowspan:(i.rowspan||1)-1}:{colspan:0,rowspan:0}},getFrozenArea:function(){var e=this;return{x:0,y:e.getHeaderRowCount()}}});t.DataProvider=T;var O=c.default.ViewController.inherit({}).include(d.default).inherit({_getEmptyCell:function(){return{caption:"",colspan:1,rowspan:1}},_updateColumnWidth:function(e,t){e.width=t},_getColumns:function(e){var t,n,i=[],o=this._columnsController,a=o.getRowCount();for(t=0;t<=a;t++){var l=[];n=o.getVisibleColumns(t,!0);var u=void 0;if(t===a)if(this._updateLockCount)u=e;else{var c=this._getColumnWidths(this._headersView,this._rowsView);if(c&&c.length){u={};for(var d=0;d<n.length;d++)u[n[d].index]=c[d]}}for(var h=0;h<n.length;h++){var f=(0,s.extend)({},n[h],{dataType:"datetime"===n[h].dataType?"date":n[h].dataType,gridColumn:n[h]});if(this._needColumnExporting(f)){var p=this._calculateExportColspan(f);(0,r.isDefined)(p)&&(f.exportColspan=p),u&&this._updateColumnWidth(f,u[f.index]),l.push(f)}}i.push(l)}return n=i[a],i=this._prepareItems(i.slice(0,-1)),i.push(n),i},_calculateExportColspan:function(e){var t=this;if(e.isBand){var n=this._columnsController.getChildrenByBandColumn(e.index,!0);if((0,r.isDefined)(n))return n.reduce(function(e,n){return t._needColumnExporting(n)?e+(t._calculateExportColspan(n)||1):e},0)}},_needColumnExporting:function(e){return!e.command&&(e.allowExporting||void 0===e.allowExporting)},_getFooterSummaryItems:function(e,t){var n=[],i=1,o=0;do{for(var a=[],r=0;r<e.length;r++){var s=e[r],l=s.length;i<l&&(i=l),a.push(s[o])}n.push({values:a,rowType:t?"totalFooter":"groupFooter"})}while(o++<i-1);return n},_hasSummaryGroupFooters:function(){var e=this.option("summary.groupItems");if((0,r.isDefined)(e))for(var t=0;t<e.length;t++)if(e[t].showInGroupFooter)return!0;return!1},_getItemsWithSummaryGroupFooters:function(e){for(var t=[],n=[],i=[],o=0;o<e.length;o++){var a=e[o];"groupFooter"===a.rowType?(i=this._getFooterSummaryItems(a.summaryCells),t=t.concat(n,i),n=[]):n.push(a)}return t.length?t:n},_updateGroupValuesWithSummaryByColumn:function(e){for(var t=[],n=0;n<e.length;n++){var i=e[n],o=i.summaryCells;if("group"===i.rowType&&o&&o.length>1){for(var a=i.values.length,r=1;r<o.length;r++)for(var s=0;s<o[r].length;s++){var l=o[r][s];l&&l.alignByColumn&&(Array.isArray(t[r-a])||(t[r-a]=[]),t[r-a].push(l))}t.length>0&&((0,u.merge)(i.values,t),t=[])}}},_processUnExportedItems:function(e){for(var t,n,i=this._columnsController.getVisibleColumns(null,!0),o=this._columnsController.getGroupColumns(),a=0;a<e.length;a++){var r=e[a],s=!1;t=[],n=[];for(var l=0;l<i.length;l++){var u=i[l];if(s=s||"detailExpand"===u.type,this._needColumnExporting(u)&&(r.values&&("group"!==r.rowType||t.length?t.push(r.values[l]):t.push(r.key[r.groupIndex])),r.summaryCells))if("group"!==r.rowType||n.length)n.push(r.summaryCells[l]);else{var c=l-o.length+r.groupIndex;n.push(r.summaryCells[s?c-1:c])}}t.length&&(r.values=t),n.length&&(r.summaryCells=n)}},_getAllItems:function(e){var t,n=this,i=new m.Deferred,o=this.getController("data"),a=o.footerItems(),s=a.length&&a[0],l=n.option("summary.totalItems");return(0,m.when)(e).done(function(e){o.loadAll(e).done(function(e,o){n._updateGroupValuesWithSummaryByColumn(e),n._hasSummaryGroupFooters()&&(e=n._getItemsWithSummaryGroupFooters(e)),t=s&&s.summaryCells,(0,r.isDefined)(o)&&l&&(t=n._getSummaryCells(l,o));var a=s&&n._getFooterSummaryItems(t,!0);a&&(e=e.concat(a)),n._processUnExportedItems(e),i.resolve(e)}).fail(i.reject)}).fail(i.reject),i},_getSummaryCells:function(e,t){var n=this.getController("data"),i=n._columnsController;return n._calculateSummaryCells(e,t,i.getVisibleColumns(null,!0),function(e,t){return n._isDataColumn(t)?t.index:-1})},_getSelectedItems:function(){var e=this.getController("selection"),t=e.getSelectedRowsData();return this._getAllItems(t)},_getColumnWidths:function(e,t){return e&&e.isVisible()?e.getColumnWidths():t.getColumnWidths()},init:function(){this._columnsController=this.getController("columns"),this._rowsView=this.getView("rowsView"),this._headersView=this.getView("columnHeadersView"),this.createAction("onExporting",{excludeValidators:["disabled","readOnly"]}),this.createAction("onExported",{excludeValidators:["disabled","readOnly"]}),this.createAction("onFileSaving",{excludeValidators:["disabled","readOnly"]})},callbackNames:function(){return["selectionOnlyChanged"]},getExportFormat:function(){return["EXCEL"]},getDataProvider:function(e){var t,n=this._getColumnWidths(this._headersView,this._rowsView);if(n&&n.length){t={};for(var i=this._columnsController.getVisibleColumns(this._columnsController.getRowCount(),!0),o=0;o<i.length;o++)t[i[o].index]=n[o]}return new T(this,t,e)},exportToExcel:function(e){var t=this;t._selectionOnly=e,(0,h.export)(t.component.getDataProvider(),{fileName:t.option("export.fileName"),proxyUrl:t.option("export.proxyUrl"),format:"EXCEL",autoFilterEnabled:!!t.option("export.excelFilterEnabled"),rtlEnabled:t.option("rtlEnabled"),ignoreErrors:t.option("export.ignoreExcelErrors"),exportingAction:t.getAction("onExporting"),exportedAction:t.getAction("onExported"),fileSavingAction:t.getAction("onFileSaving")},h.excel.getData)},publicMethods:function(){return["getDataProvider","getExportFormat","exportToExcel"]},selectionOnly:function(e){return(0,r.isDefined)(e)?(this._isSelectedRows=e,void this.selectionOnlyChanged.fire()):this._isSelectedRows}});t.ExportController=O,c.default.registerModule("export",{defaultOptions:function(){return{"export":{enabled:!1,fileName:"DataGrid",excelFilterEnabled:!1,excelWrapTextEnabled:void 0,proxyUrl:void 0,allowExportSelectedData:!1,ignoreExcelErrors:!0,texts:{exportTo:f.default.format("dxDataGrid-exportTo"),exportAll:f.default.format("dxDataGrid-exportAll"),exportSelectedRows:f.default.format("dxDataGrid-exportSelectedRows")}}}},controllers:{"export":O},extenders:{controllers:{editing:{callbackNames:function(){var e=this.callBase();return(0,r.isDefined)(e)?e.push("editingChanged"):["editingChanged"]},_updateEditButtons:function(){this.callBase(),this.editingChanged.fire(this.hasChanges())}}},views:{headerPanel:{_getToolbarItems:function(){var e=this.callBase();return this._appendExportItems(e)},_appendExportItems:function(e){var t=this,n=t.option("export");if(n.enabled){var i=[];n.allowExportSelectedData?i.push({template:function(e,n,i){var a=(0,o.default)(i);t._renderButton(e,a),t._renderExportMenu(a)},menuItemTemplate:function(e,n,i){t._renderList(e,(0,o.default)(i))},name:"exportButton",allowExportSelected:!0,location:"after",locateInMenu:"auto",sortIndex:30}):i.push({template:function(e,n,i){t._renderButton(e,(0,o.default)(i))},menuItemTemplate:function(e,n,i){t._renderButton(e,(0,o.default)(i),!0)},name:"exportButton",location:"after",locateInMenu:"auto",sortIndex:30}),e=e.concat(i),t._correctItemsPosition(e)}return e},_renderButton:function(e,t,n){var i=this,a=i._getButtonOptions(e.allowExportSelected),r=i._getButtonContainer().addClass(y).appendTo(t);if(n){var s=(0,o.default)("<div>").addClass(k);t.wrapInner(s).parent().addClass("dx-toolbar-menu-action dx-toolbar-menu-button "+S),a.text=a.hint}i._createComponent(r,p.default,a)},_renderList:function(e,t){var n=this,i=n.option("export.texts"),a=[{template:function(e,t,i){n._renderFakeButton(e,(0,o.default)(i),x)},text:i.exportAll},{template:function(e,t,i){n._renderFakeButton(e,(0,o.default)(i),w)},text:i.exportSelectedRows,exportSelected:!0}];n._createComponent(t,g.default,{items:a,onItemClick:function(e){n._exportController.exportToExcel(e.itemData.exportSelected)},scrollingEnabled:!1})},_renderFakeButton:function(e,t,n){var i=(0,o.default)("<div>").addClass("dx-icon dx-icon-"+n),a=(0,o.default)("<span>").addClass("dx-button-text").text(e.text),r=(0,o.default)("<div>").addClass("dx-button-content").append(i).append(a),s=(0,o.default)("<div>").addClass(D+" dx-button-has-text dx-button-has-icon dx-datagrid-toolbar-button").append(r),l=(0,o.default)("<div>").addClass(k).append(s);t.append(l).parent().addClass("dx-toolbar-menu-custom "+S)},_correctItemsPosition:function(e){e.sort(function(e,t){return e.sortIndex-t.sortIndex})},_renderExportMenu:function(e){var t=this,n=e.find("."+D),i=t.option("export.texts"),a=[{text:i.exportAll,icon:x},{text:i.exportSelectedRows,exportSelected:!0,icon:w}],r=(0,o.default)("<div>").appendTo(e);t._contextMenu=t._createComponent(r,_.default,{showEvent:"dxclick",items:a,cssClass:v,onItemClick:function(e){t._exportController.exportToExcel(e.itemData.exportSelected)},target:n,position:{at:"left bottom",my:"left top",offset:"0 3",collision:"fit",boundary:t._$parent,boundaryOffset:"1 1"}})},_isExportButtonVisible:function(){return this.option("export.enabled")},_getButtonOptions:function(e){var t,n=this,i=n.option("export.texts");return t=e?{hint:i.exportTo,icon:b}:{hint:i.exportAll,icon:C,onClick:function(){n._exportController.exportToExcel()}}},optionChanged:function(e){this.callBase(e),"export"===e.name&&(e.handled=!0,this._invalidate())},init:function(){var e=this;this.callBase(),this._exportController=this.getController("export"),
this._editingController=this.getController("editing"),this._editingController.editingChanged.add(function(t){e.setToolbarItemDisabled("exportButton",t)})},isVisible:function(){return this.callBase()||this._isExportButtonVisible()}}}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(504)),r=n(6),s=n(1),l=n(4),u=n(303),c=n(19),d=n(0),h=Number.MAX_SAFE_INTEGER||9007199254740991;o.default.registerModule("focus",(0,d.extend)(!0,{},a.default,{extenders:{controllers:{data:{changeRowExpand:function(e,t){if(this.option("focusedRowEnabled")&&Array.isArray(e)&&this.isRowExpanded(e)){var n=this.getController("keyboardNavigation");t&&n.isKeyboardEnabled()||!this._isFocusedRowInsideGroup(e)||this.option("focusedRowKey",e)}return this.callBase.apply(this,arguments)},_isFocusedRowInsideGroup:function(e){var t=this.getController("columns"),n=this.option("focusedRowKey"),i=this.getRowIndexByKey(n),o=i>=0&&this.getVisibleRows()[i],a=t.getGroupDataSourceParameters(!0);if(o)for(var r=0;r<e.length;++r){var s=(0,c.compileGetter)(a[r]&&a[r].selector);if(s(o.data)!==e[r])return!1}return!0},_getGroupPath:function(e){for(var t=[e.key],n=e.items;n&&n[0];){var i=n[0];void 0!==i.key&&t.push(i.key),n=i.items}return t},_expandGroupByPath:function(e,t,n){var i=new r.Deferred;return n++,e.expandRow(t.slice(0,n)).done(function(){n===t.length?i.resolve():e._expandGroupByPath(e,t,n).done(i.resolve).fail(i.reject)}).fail(i.reject),i.promise()},_calculateGlobalRowIndexByGroupedData:function(e){var t=this,n=t._dataSource,i=t._generateFilterByKey(e),o=new r.Deferred,a=Array.isArray(e),l=n.group();return a?o.resolve(-1).promise():n._grouping._updatePagingOptions?(n.load({filter:t._concatWithCombinedFilter(i),group:l}).done(function(n){if(!n||0===n.length||!(0,s.isDefined)(n[0].key)||n[0].key===-1)return o.resolve(-1).promise();var i=t._getGroupPath(n[0]);t._expandGroupByPath(t,i,0).done(function(){t._calculateExpandedRowGlobalIndex(o,e,i,l)}).fail(o.reject)}).fail(o.reject),o.promise()):(t._calculateGlobalRowIndexByFlatData(e,null,!0).done(o.resolve).fail(o.reject),o)},_calculateExpandedRowGlobalIndex:function(e,t,n,i){var o,a=(0,u.createGroupFilter)(n,{group:i}),r=this._dataSource,s=this.option("scrolling.mode"),c="virtual"===s||"infinite"===s,d=r.pageSize();r._grouping._updatePagingOptions({skip:0,take:h},function(e,t){(0,l.equalByValue)(e.path,n)&&(o=t)}),this._calculateGlobalRowIndexByFlatData(t,a).done(function(t){var i,a;if(t<0)return void e.resolve(-1);var r=o%d||d;i=r+t-n.length,a=c?0:Math.floor(i/(d-n.length))*n.length,i=o+t+a,e.resolve(i)}).fail(e.reject)}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(26)),a=i(n(505));o.default.registerModule("rowDragging",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(737));n(747),n(748),n(749),n(750),n(752),n(753),n(754),n(755),n(756),n(757),n(758),n(759),n(760),n(761),n(762),n(763),n(764),n(765),n(766),n(767),n(768);var a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(9)),a=n(4),r=n(1),s=n(3),l=n(0),u=i(n(16)),c=i(n(31)),d=n(36);n(738),n(739),n(740),n(741),n(742),n(743),n(744),n(745),n(746);var h=c.default.callModuleItemsMethod,f=".dx-row",p="dx-treelist";c.default.registerModulesOrder(["stateStoring","columns","selection","editorFactory","columnChooser","editing","grouping","masterDetail","validating","adaptivity","data","virtualScrolling","columnHeaders","filterRow","headerPanel","headerFilter","sorting","search","rows","pager","columnsResizingReordering","contextMenu","keyboardNavigation","errorHandling","summary","columnFixing","export","gridView"]);var g=u.default.inherit({_activeStateUnit:f,_getDefaultOptions:function(){var e=this,t=e.callBase();return(0,s.each)(c.default.modules,function(){(0,r.isFunction)(this.defaultOptions)&&(0,l.extend)(!0,t,this.defaultOptions())}),t},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return(0,d.isMaterial)()},options:{showRowLines:!0,showColumnLines:!1,headerFilter:{height:315},editing:{useIcons:!0}}}])},_init:function(){var e=this;e.callBase(),c.default.processModules(e,c.default),h(e,"init")},_clean:a.noop,_optionChanged:function(e){var t=this;h(t,"optionChanged",[e]),e.handled||t.callBase(e)},_dimensionChanged:function(){this.updateDimensions(!0)},_visibilityChanged:function(e){e&&this.updateDimensions()},_initMarkup:function(){this.callBase.apply(this,arguments),this.$element().addClass(p),this.getView("gridView").render(this.$element())},_renderContentImpl:function(){this.getView("gridView").update()},_renderContent:function(){var e=this;(0,a.deferRender)(function(){e._renderContentImpl()})},_dispose:function(){var e=this;e.callBase(),h(e,"dispose")},isReady:function(){return this.getController("data").isReady()},beginUpdate:function(){var e=this;e.callBase(),h(e,"beginUpdate")},endUpdate:function(){var e=this;h(e,"endUpdate"),e.callBase()},getController:function(e){return this._controllers[e]},getView:function(e){return this._views[e]},focus:function(e){this.callBase(),(0,r.isDefined)(e)&&this.getController("keyboardNavigation").focus(e)}});g.registerModule=c.default.registerModule.bind(c.default),(0,o.default)("dxTreeList",g);var _=g;t.default=_,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(461));o.default.registerModule("columnHeaders",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.ColumnsController=void 0;var o=n(1),a=i(n(31)),r=i(n(463)),s=r.default.controllers.columns.inherit(function(){return{_getFirstItems:function(e){return this.callBase(e).map(function(e){return e.data})},getFirstDataColumnIndex:function(){for(var e=this.getVisibleColumns(),t=e.length,n=0,i=0;i<=t-1;i++)if(!(0,o.isDefined)(e[i].command)){n=e[i].index;break}return n}}}());t.ColumnsController=s,a.default.registerModule("columns",{defaultOptions:r.default.defaultOptions,controllers:{columns:s}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.DataController=void 0;var o=n(0),a=n(6),r=i(n(31)),s=n(4),l=i(n(507)),u=i(n(465)),c=u.default.controllers.data.inherit(function(){return{_getDataSourceAdapter:function(){return l.default},_getNodeLevel:function(e){for(var t=-1;e.parent;)e.visible&&t++,e=e.parent;return t},_generateDataItem:function(e,t){return{rowType:"data",node:e,key:e.key,data:e.data,isExpanded:this.isRowExpanded(e.key,t),level:this._getNodeLevel(e)}},_loadOnOptionChange:function(){this._dataSource.load()},_isItemEquals:function(e,t){return!!this.callBase.apply(this,arguments)&&((!e.node||!t.node||e.node.hasChildren===t.node.hasChildren)&&e.level===t.level)},init:function(){this.createAction("onRowExpanding"),this.createAction("onRowExpanded"),this.createAction("onRowCollapsing"),this.createAction("onRowCollapsed"),this.callBase.apply(this,arguments)},keyOf:function(e){var t=this._dataSource;if(t)return t.keyOf(e)},key:function(){var e=this._dataSource;if(e)return e.getKeyExpr()},publicMethods:function(){return this.callBase().concat(["expandRow","collapseRow","isRowExpanded","getRootNode","getNodeByKey","loadDescendants","forEachNode"])},changeRowExpand:function(e){if(this._dataSource){var t=this,n={key:e},i=this.isRowExpanded(e);if(t.executeAction(i?"onRowCollapsing":"onRowExpanding",n),!n.cancel)return t._dataSource.changeRowExpand(e).done(function(){t.executeAction(i?"onRowCollapsed":"onRowExpanded",n)})}return(new a.Deferred).resolve()},isRowExpanded:function(e,t){return this._dataSource&&this._dataSource.isRowExpanded(e,t)},expandRow:function(e){return this.isRowExpanded(e)?(new a.Deferred).resolve():this.changeRowExpand(e)},collapseRow:function(e){return this.isRowExpanded(e)?this.changeRowExpand(e):(new a.Deferred).resolve()},getRootNode:function(){return this._dataSource&&this._dataSource.getRootNode()},optionChanged:function(e){switch(e.name){case"rootValue":case"parentIdExpr":case"itemsExpr":case"filterMode":case"expandNodesOnFiltering":case"autoExpandAll":case"hasItemsExpr":case"dataStructure":this._columnsController.reset(),this._items=[],this._refreshDataSource(),e.handled=!0;break;case"expandedRowKeys":case"onNodesInitialized":!this._dataSource||this._dataSource._isNodesInitializing||(0,s.equalByValue)(e.value,e.previousValue)||this._loadOnOptionChange(),e.handled=!0;break;case"maxFilterLengthInRequest":e.handled=!0;break;default:this.callBase(e)}},getNodeByKey:function(e){if(this._dataSource)return this._dataSource.getNodeByKey(e)},getChildNodeKeys:function(e){if(this._dataSource)return this._dataSource.getChildNodeKeys(e)},loadDescendants:function(e,t){if(this._dataSource)return this._dataSource.loadDescendants(e,t)},forEachNode:function(){this._dataSource.forEachNode.apply(this,arguments)}}}());t.DataController=c,r.default.registerModule("data",{defaultOptions:function(){return(0,o.extend)({},u.default.defaultOptions(),{itemsExpr:"items",parentIdExpr:"parentId",rootValue:0,dataStructure:"plain",expandedRowKeys:[],filterMode:"withAncestors",expandNodesOnFiltering:!0,autoExpandAll:!1,onNodesInitialized:null,maxFilterLengthInRequest:1500,paging:{enabled:!1}})},controllers:{data:c}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(466));o.default.registerModule("sorting",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.RowsView=void 0;var o=i(n(2)),a=i(n(31)),r=i(n(468)),s="dx-treelist-text-content",l="dx-treelist-icon-container",u="dx-treelist-cell-expandable",c="dx-treelist-empty-space",d="dx-treelist-expanded",h="dx-treelist-collapsed",f=r.default.views.rowsView.inherit(function(){var e=function(e){return(0,o.default)("<div>").addClass(s).appendTo(e)},t=function(e,t){var n=(0,o.default)("<div>").addClass(c);return e&&n.toggleClass(d,t).toggleClass(h,!t).append((0,o.default)("<span>")),n};return{_renderIconContainer:function(e,t){var n=this,i=(0,o.default)("<div>").addClass(l).appendTo(e);return t.watch&&t.watch(function(){return[t.row.level,t.row.isExpanded,t.row.node.hasChildren]},function(){i.empty(),n._renderIcons(i,t)}),e.addClass(u),this._renderIcons(i,t)},_renderIcons:function(e,n){for(var i=n.row,o=i.level,a=0;a<=o;a++)e.append(t(a===o&&i.node.hasChildren,i.isExpanded));return e},_renderCellCommandContent:function(e,t){return this._renderIconContainer(e,t),!0},_processTemplate:function(t,n){var i,o=this,a=this.callBase(t),r=o._columnsController.getFirstDataColumnIndex();return i=a&&n.column.index===r?{render:function(t){var n=t.container;o._renderCellCommandContent(n,t.model)&&(t.container=e(n)),a.render(t)}}:a},_updateCell:function(e,t){e=e.hasClass(s)?e.parent():e,this.callBase(e,t)},_rowClick:function(e){var t=this._dataController,n=(0,o.default)(e.event.target),i=this.isExpandIcon(n),a=t&&t.items()[e.rowIndex];i&&a&&t.changeRowExpand(a.key),this.callBase(e)},_createRow:function(e){var t=e&&e.node,n=this.callBase.apply(this,arguments);return t&&(this.setAria("level",e.level,n),t.hasChildren&&this.setAria("expanded",e.isExpanded,n)),n},isExpandIcon:function(e){return!!e.closest("."+d+", ."+h).length}}}());t.RowsView=f,a.default.registerModule("rows",{defaultOptions:r.default.defaultOptions,views:{rowsView:f}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(469));o.default.registerModule("contextMenu",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(470));o.default.registerModule("errorHandling",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(471)),r=a.default.views.gridView.inherit(function(){return{_getWidgetAriaLabel:function(){return"dxTreeList-ariaTreeList"},_getTableRoleName:function(){return"treegrid"}}}());o.default.registerModule("gridView",{defaultOptions:a.default.defaultOptions,controllers:a.default.controllers,views:{gridView:r},extenders:{controllers:{resizing:{_toggleBestFitMode:function(e){if(this.callBase(e),!this.option("legacyRendering")){var t=this._rowsView._getTableElement();t.find(".dx-treelist-cell-expandable").toggleClass(this.addWidgetPrefix("best-fit"),e)}}}}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(472));o.default.registerModule("headerPanel",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=n(0),r=i(n(473)),s=r.default.extenders.controllers.stateStoring.applyState;o.default.registerModule("stateStoring",(0,a.extend)(!0,{},r.default,{extenders:{controllers:{stateStoring:{applyState:function(e){s.apply(this,arguments),Object.prototype.hasOwnProperty.call(e,"expandedRowKeys")&&this.option("expandedRowKeys",e.expandedRowKeys&&e.expandedRowKeys.slice())}},data:{getUserState:function(){var e=this.callBase.apply(this,arguments);return this.option("autoExpandAll")||(e.expandedRowKeys=this.option("expandedRowKeys")),e}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(476));o.default.registerModule("columnChooser",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(479)),r=n(0);o.default.registerModule("masterDetail",(0,r.extend)(!0,{},a.default,{extenders:{controllers:{data:{isRowExpanded:function(){return this.callBase.apply(this,arguments)},_processItems:function(){return this.callBase.apply(this,arguments)},_processDataItem:function(){return this.callBase.apply(this,arguments)}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=(0,a.default)(e.event.target);return!n.closest("."+p).length&&(t.call(this,e),!0)}n(751);var a=i(n(2)),r=i(n(17)),s=n(1),l=n(0),u=n(6),c=i(n(10)),d=i(n(31)),h=i(n(37)),f=i(n(482)),p="dx-treelist-icon-container",g="dx-select-checkbox",_="insert",m=f.default.controllers.editing.inherit(function(){return{_generateNewItem:function(e){var t=this.callBase(e);return t.data={key:e},t.children=[],t.level=0,t.parentKey=this.option("rootValue"),t},_needInsertItem:function(e,t,n,i){var o=e.key.parentKey;if(void 0!==o&&o!==this.option("rootValue")){var a=h.default.getIndexByKey(o,n);return a>=0&&this._dataController.isRowExpanded(o)&&n.splice(a+1,0,i),!1}return this.callBase.apply(this,arguments)},_isEditColumnVisible:function(){var e=this.callBase.apply(this,arguments),t=this.option("editing");return e||t&&t.allowAdding},_isDefaultButtonVisible:function(e,t){var n=this.callBase.apply(this,arguments),i=t.row;return"add"===e.name?this.allowAdding(t)&&i.rowIndex!==this._getVisibleEditRowIndex()&&!(i.removed||i.isNewRow):n},_getEditingButtons:function(e){var t=this.callBase.apply(this,arguments);return e.column.buttons||t.unshift(this._getButtonConfig("add",e)),t},_beforeSaveEditData:function(e){var t=this._dataController,n=this.callBase.apply(this,arguments);if(e&&e.type!==_){var i=null===t||void 0===t?void 0:t.store(),o=null===i||void 0===i?void 0:i.key();if(!(0,s.isDefined)(o))throw r.default.Error("E1045")}return n},addRowByRowIndex:function(e){var t=this.getController("data"),n=t.getVisibleRows()[e];return this.addRow(n?n.key:void 0)},addRow:function(e){var t=this,n=t.callBase,i=t.getController("data");if(void 0!==e&&!i.isRowExpanded(e)){var o=new u.Deferred;return i.expandRow(e).done(function(){setTimeout(function(){n.call(t,e).done(o.resolve).fail(o.reject)})}).fail(o.reject),o.promise()}return void 0===e&&(e=t.option("rootValue")),n.call(t,e)},_initNewRow:function(e,t){var n=this.getController("data"),i=n.dataSource(),o=i.createParentIdSetter();return o(e.data,t),this.callBase.apply(this,arguments)},allowAdding:function(e){return this._allowEditAction("allowAdding",e)},_needToCloseEditableCell:function(e){return this.callBase.apply(this,arguments)||e.closest("."+p).length&&this.isEditing()},getButtonLocalizationNames:function(){var e=this.callBase.apply(this);return e.add="dxTreeList-editingAddRowToNode",e}}}()),v=f.default.extenders.views.rowsView._rowClick,y=f.default.extenders.views.rowsView._rowDblClick,b=function(e){var t=(0,a.default)(e.event.target),n="dxdblclick"===e.event.type?y:v;return!t.closest("."+g).length&&!o.call(this,e,n)},x=(0,l.extend)({},f.default.extenders.views.rowsView,{_renderCellCommandContent:function(e,t){var n=this._editingController,i=t.row&&n.isEditRow(t.row.rowIndex),o=t.isEditing||i;return!o&&this.callBase.apply(this,arguments)},_rowClick:function(e){b.call(this,e)&&this.callBase.apply(this,arguments)},_rowDblClick:function(e){b.call(this,e)&&this.callBase.apply(this,arguments)}});d.default.registerModule("editing",{defaultOptions:function(){return(0,l.extend)(!0,f.default.defaultOptions(),{editing:{texts:{addRowToNode:c.default.format("dxTreeList-editingAddRowToNode")}}})},controllers:{editing:m},extenders:{controllers:(0,l.extend)(!0,{},f.default.extenders.controllers,{data:{changeRowExpand:function(){return this._editingController.refresh(),this.callBase.apply(this,arguments)}}}),views:{rowsView:x,headerPanel:f.default.extenders.views.headerPanel}}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(480));o.default.registerModule("editorFactory",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=n(0),a=i(n(31)),r=i(n(483)),s=(0,o.extend)({},r.default.extenders.controllers.editing);delete s.processItems,delete s.processDataItem,a.default.registerModule("validating",{defaultOptions:r.default.defaultOptions,controllers:r.default.controllers,extenders:{controllers:{editing:s,editorFactory:r.default.extenders.controllers.editorFactory},views:r.default.extenders.views}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(507)),r=i(n(484)),s=n(0),l=r.default.defaultOptions,u=r.default.extenders.controllers.data,c=r.default.extenders.dataSourceAdapter;r.default.extenders.controllers.data=(0,s.extend)({},u,{_loadOnOptionChange:function(){var e=this._dataSource&&this._dataSource._virtualScrollController;e&&e.reset(),this.callBase()}}),r.default.extenders.dataSourceAdapter=(0,s.extend)({},c,{changeRowExpand:function(){var e=this;return this.callBase.apply(this,arguments).done(function(){var t=e.getViewportItemIndex();t>=0&&e.setViewportItemIndex(t)})}}),o.default.registerModule("virtualScrolling",(0,s.extend)({},r.default,{defaultOptions:function(){return(0,s.extend)(!0,l(),{scrolling:{mode:"virtual"}})}})),a.default.extend(r.default.extenders.dataSourceAdapter)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(486));o.default.registerModule("filterRow",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(489));o.default.registerModule("headerFilter",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(491));o.default.registerModule("filterSync",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(492));o.default.registerModule("filterBuilder",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(494));o.default.registerModule("filterPanel",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(496));o.default.registerModule("pager",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(497));o.default.registerModule("columnsResizingReordering",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(499));o.default.registerModule("columnFixing",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(500));o.default.registerModule("adaptivity",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(2)),a=i(n(31)),r=n(4),s=i(n(475)),l=n(0),u=n(1),c="dx-treelist-select-all",d="dx-cell-focus-disabled",h="dx-select-checkbox",f=s.default.extenders.views.rowsView._rowClick,p=s.default.extenders.controllers.data._handleDataChanged,g=function(e,t){return!!e.filter(function(e){return e===t}).length};a.default.registerModule("selection",(0,l.extend)(!0,{},s.default,{defaultOptions:function(){return(0,l.extend)(!0,s.default.defaultOptions(),{selection:{showCheckBoxesMode:"always",recursive:!1}})},extenders:{controllers:{data:{_handleDataChanged:function(e){var t=this.getController("selection"),n=t.isRecursiveSelection();!n||e&&"updateSelectionState"===e.changeType||t.updateSelectionState({selectedItemKeys:this.option("selectedRowKeys")}),p.apply(this,arguments)},loadDescendants:function(){var e=this,t=e.callBase.apply(e,arguments),n=e.getController("selection"),i=n.isRecursiveSelection();return i&&t.done(function(){n.updateSelectionState({selectedItemKeys:e.option("selectedRowKeys")})}),t}},selection:{init:function(){this.callBase.apply(this,arguments),this._selectionStateByKey={}},_getSelectionConfig:function(){var e=arguments,t=this,n=this.callBase.apply(this,arguments),i=n.plainItems;return n.plainItems=function(n){return n?t._dataController.getCachedStoreData()||[]:i.apply(t,e).map(function(e){return e.data})},n.isItemSelected=function(e){var n=t._dataController.keyOf(e);return t.isRowSelected(n)},n.isSelectableItem=function(){return!0},n.getItemData=function(e){return e},n},renderSelectCheckBoxContainer:function(e,t){var n=this,i=n.component.getView("rowsView");e.addClass(d);var o=i._renderSelectCheckBox(e,{value:t.row.isSelected,row:t.row,column:t.column});i._attachCheckBoxClickEvent(o)},_updateSelectColumn:r.noop,_getSelectAllNodeKeys:function(){var e=this.component,t=e.getRootNode(),n={},i=[],o=this.isRecursiveSelection();return t&&a.default.foreachNodes(t.children,function(t){return void 0!==t.key&&(t.visible||o)&&i.push(t.key),!t.visible||!o&&e.isRowExpanded(t.key,n)}),i},isSelectAll:function(){var e=this.component,t=this._getSelectAllNodeKeys(),n=t.filter(function(t){return e.isRowSelected(t)});if(!n.length){var i=t.some(function(t){return void 0===e.isRowSelected(t)});return!!i&&void 0}if(n.length===t.length)return!0},selectAll:function(){var e=this,t=e._getSelectAllNodeKeys().filter(function(t){return!e.isRowSelected(t)});return e.selectRows(t,!0)},deselectAll:function(){var e=this._getSelectAllNodeKeys();return this.deselectRows(e)},selectedItemKeys:function(e,t,n,i){var o=this,a=o.option("selectedRowKeys"),s=this.isRecursiveSelection(),l=s&&o._normalizeSelectionArgs({keys:(0,u.isDefined)(e)?e:[]},!n);return l&&!(0,r.equalByValue)(l.selectedRowKeys,a)?(o._isSelectionNormalizing=!0,this.callBase(l.selectedRowKeys,!1,!1,!1).always(function(){o._isSelectionNormalizing=!1}).done(function(e){l.selectedRowsData=e,o._fireSelectionChanged(l)})):this.callBase(e,t,n,i)},changeItemSelection:function(e,t){var n=this.isRecursiveSelection();if(n&&!t.shift){var i=this._dataController.getKeyByRowIndex(e);return this.selectedItemKeys(i,!0,this.isRowSelected(i))}return this.callBase.apply(this,arguments)},_updateParentSelectionState:function(e,t){var n=this,i=t,o=e.parent;if(o){if(o.children.length>1)if(t===!1){var a=o.children.some(function(e,t,i){return n._selectionStateByKey[e.key]});i=!!a&&void 0}else if(t===!0){var r=o.children.some(function(e){return!n._selectionStateByKey[e.key]});i=!r||void 0}this._selectionStateByKey[o.key]=i,o.parent&&o.parent.level>=0&&this._updateParentSelectionState(o,i)}},_updateChildrenSelectionState:function(e,t){var n=this,i=e.children;i&&i.forEach(function(e){n._selectionStateByKey[e.key]=t,e.children.length>0&&n._updateChildrenSelectionState(e,t)})},_updateSelectionStateCore:function(e,t){for(var n=this._dataController,i=0;i<e.length;i++){this._selectionStateByKey[e[i]]=t;var o=n.getNodeByKey(e[i]);o&&(this._updateParentSelectionState(o,t),this._updateChildrenSelectionState(o,t))}},_getSelectedParentKeys:function(e,t,n){for(var i,o=this._dataController.getNodeByKey(e),a=o&&o.parent,r=[];a&&a.level>=0;){r.unshift(a.key);var s=n?!g(t,a.key)&&this.isRowSelected(a.key):t.indexOf(a.key)>=0;if(s){i=a,r=this._getSelectedParentKeys(i.key,t,n).concat(r);break}if(n)break;a=a.parent}return i&&r||[]},_getSelectedChildKeys:function(e,t){var n=this,i=[];return e&&a.default.foreachNodes(e.children,function(e){var o=t.indexOf(e.key);return o<0&&i.push(e.key),o>0||o<0&&void 0===n._selectionStateByKey[e.key]}),i},_normalizeParentKeys:function(e,t){var n=this,i=[e],o=n._getSelectedParentKeys(e,t.selectedRowKeys);if(o.length){i=i.concat(o),i.forEach(function(e){var n=t.selectedRowKeys.indexOf(e);n>=0&&t.selectedRowKeys.splice(n,1)});var a=n._dataController.getNodeByKey(o[0]),r=n._getSelectedChildKeys(a,i);t.selectedRowKeys=t.selectedRowKeys.concat(r)}},_normalizeChildrenKeys:function(e,t){var n=this,i=n._dataController.getNodeByKey(e);i&&i.children.forEach(function(e){var i=t.selectedRowKeys.indexOf(e.key);i>=0&&t.selectedRowKeys.splice(i,1),n._normalizeChildrenKeys(e.key,t)})},_normalizeSelectedRowKeysCore:function(e,t,n){var i=this;e.forEach(function(e){if(i.isRowSelected(e)!==n){i._normalizeChildrenKeys(e,t);var o=t.selectedRowKeys.indexOf(e);n?(o<0&&t.selectedRowKeys.push(e),t.currentSelectedRowKeys.push(e)):(o>=0&&t.selectedRowKeys.splice(o,1),t.currentDeselectedRowKeys.push(e),i._normalizeParentKeys(e,t))}})},_normalizeSelectionArgs:function(e,t){var n,i=Array.isArray(e.keys)?e.keys:[e.keys],o=this.option("selectedRowKeys")||[];return i.length&&(n={currentSelectedRowKeys:[],currentDeselectedRowKeys:[],selectedRowKeys:o.slice(0)},this._normalizeSelectedRowKeysCore(i,n,t)),n},_updateSelectedItems:function(e){this.updateSelectionState(e),this.callBase(e)},_fireSelectionChanged:function(){this._isSelectionNormalizing||this.callBase.apply(this,arguments)},_isModeLeavesOnly:function(e){return"leavesOnly"===e},_getAllSelectedRowKeys:function(e){var t=this,n=[];return e.forEach(function(e){var i=n.length,o=t._getSelectedParentKeys(e,n,!0),a=t._dataController.getChildNodeKeys(e);n.splice.apply(n,[i,0].concat(o)),n.push(e),n=n.concat(a)}),n},_getParentSelectedRowKeys:function(e){var t=this,n=[];return e.forEach(function(i){var o=t._getSelectedParentKeys(i,e);!o.length&&n.push(i)}),n},_getLeafSelectedRowKeys:function(e){var t=this,n=[],i=t._dataController;return e.forEach(function(e){var t=i.getNodeByKey(e);t&&!t.hasChildren&&n.push(e)}),n},isRecursiveSelection:function(){var e=this.option("selection.mode"),t=this.option("selection.recursive");return"multiple"===e&&t},updateSelectionState:function(e){var t=e.removedItemKeys||[],n=e.selectedItemKeys||[];this._updateSelectionStateCore(t,!1),this._updateSelectionStateCore(n,!0)},isRowSelected:function(e){var t=this.callBase.apply(this,arguments),n=this.isRecursiveSelection();return!t&&n?e in this._selectionStateByKey&&this._selectionStateByKey[e]:t},getSelectedRowKeys:function(e){var t=this;if(!t._dataController)return[];var n=t.callBase.apply(t,arguments);return e&&(this.isRecursiveSelection()&&(n=this._getAllSelectedRowKeys(n)),"all"!==e&&("excludeRecursive"===e?n=t._getParentSelectedRowKeys(n):t._isModeLeavesOnly(e)&&(n=t._getLeafSelectedRowKeys(n)))),n},getSelectedRowsData:function(e){var t=this,n=t._dataController,i=this.getSelectedRowKeys(e)||[],o=[];return i.forEach(function(e){var t=n.getNodeByKey(e);t&&o.push(t.data)}),o},refresh:function(){return this._selectionStateByKey={},this.callBase.apply(this,arguments)}}},views:{columnHeadersView:{_processTemplate:function(e,t){var n,i=this,o=this.callBase(e,t),a=i._columnsController.getFirstDataColumnIndex();return n=o&&"header"===t.rowType&&t.column.index===a?{render:function(e){"multiple"===i.option("selection.mode")&&i.renderSelectAll(e.container,e.model),o.render(e)}}:o},renderSelectAll:function(e,t){e.addClass(c),this._renderSelectAllCheckBox(e)},_isSortableElement:function(e){return this.callBase(e)&&!e.closest("."+h).length}},rowsView:{_renderIcons:function(e,t){return this.callBase.apply(this,arguments),"multiple"===this.option("selection.mode")&&this.getController("selection").renderSelectCheckBoxContainer(e,t),e},_rowClick:function(e){var t=(0,o.default)(e.event.target);this.isExpandIcon(t)?this.callBase.apply(this,arguments):f.apply(this,arguments)}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(495));o.default.registerModule("search",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(498)),r=n(0);o.default.registerModule("keyboardNavigation",(0,r.extend)(!0,{},a.default,{extenders:{controllers:{keyboardNavigation:{_leftRightKeysHandler:function(e,t){var n=this.getVisibleRowIndex(),i=this._dataController;if(!e.ctrl)return this.callBase.apply(this,arguments);var o=this._getDirectionCodeByKey(e.keyName),a=i.getKeyByRowIndex(n);"nextInRow"===o?i.expandRow(a):i.collapseRow(a)}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(501));o.default.registerModule("virtualColumns",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=-1;return e.forEach(function(e,i){t(e)&&(n=i)}),n}var a=i(n(31)),r=n(0),s=n(6),l=i(n(504));a.default.registerModule("focus",(0,r.extend)(!0,{},l.default,{extenders:{controllers:{data:{changeRowExpand:function(e){return this.option("focusedRowEnabled")&&this.isRowExpanded(e)&&this._isFocusedRowInside(e)&&this.option("focusedRowKey",e),this.callBase.apply(this,arguments)},_isFocusedRowInside:function(e){for(var t=this.option("focusedRowKey"),n=this.getRowIndexByKey(t),i=n>=0&&this.getVisibleRows()[n],o=i&&i.node.parent;o;){if(o.key===e)return!0;o=o.parent}return!1},getParentKey:function(e){var t=this,n=t._dataSource,i=t.getNodeByKey(e),o=new s.Deferred;return i?o.resolve(i.parent?i.parent.key:void 0):n.load({filter:[n.getKeyExpr(),"=",e]}).done(function(e){var t=e[0];t?o.resolve(n.parentKeyOf(t)):o.reject()}).fail(o.reject),o.promise()},expandAscendants:function(e){var t=this,n=t._dataSource,i=new s.Deferred;return t.getParentKey(e).done(function(e){n&&void 0!==e&&e!==t.option("rootValue")?(n._isNodesInitializing=!0,t.expandRow(e),n._isNodesInitializing=!1,t.expandAscendants(e).done(i.resolve).fail(i.reject)):i.resolve()}).fail(i.reject),i.promise()},getPageIndexByKey:function(e){var t=this,n=t._dataSource,i=new s.Deferred;return t.expandAscendants(e).done(function(){n.load({filter:t.getCombinedFilter(),sort:t.getController("columns").getSortDataSourceParameters(!n.remoteOperations().sorting),parentIds:[]}).done(function(n){var a=o(n,function(n){return t.keyOf(n.data)===e}),r=t.pageIndex();a>=0&&(r=Math.floor(a/t.pageSize())),i.resolve(r)}).fail(i.reject)}).fail(i.reject),i.promise()}}}}}))},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(31)),a=i(n(505));o.default.registerModule("rowDragging",a.default)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(770)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=0;return(0,y.each)(e,function(e,n){t+=n||0}),t}function a(e,t){for(var n=t/e.length,i=0;i<e.length;i++)e[i]-=n}function r(e){e.off("scroll").off("stop")}function s(e,t){r(e),e.on("scroll",t).on("stop",t)}function l(e){if(re[e])return re[e];var t=0,n={},i=(0,d.default)(ie).css({position:"absolute",visibility:"hidden",top:-1e3,left:-1e3,width:100,height:100}).appendTo("body"),o=(0,d.default)("<p>").css({width:"100%",height:200}).appendTo(i);"auto"!==e&&(n.useNative=!!e,n.useSimulatedScrollbar=!e),i.dxScrollable(n);var a=i.dxScrollable("instance").option("useNative");return t=a?i.width()-o.width():0,i.remove(),re[e]={scrollBarWidth:t,scrollBarUseNative:a},re[e]}function u(e,t){var n="width"===t?["borderLeftWidth","borderRightWidth"]:["borderTopWidth","borderBottomWidth"],i=0;
return(0,y.each)(e,function(e,t){var o=z.getComputedStyle(t.get(0));n.forEach(function(e){i+=parseFloat(o[e])||0})}),i}function c(e){return e.closest("."+X).length||e.find("."+X).length}t.default=void 0;var d=i(n(2)),h=n(7),f=i(n(23)),p=i(n(5)),g=i(n(9)),_=n(18),m=n(42),v=n(4),y=n(3),b=n(1),x=n(0),w=n(21),C=i(n(10)),k=i(n(16)),S=n(8),D=i(n(37)),I=n(89),T=n(771),O=n(777),E=n(778),A=n(184),P=n(779),R=i(n(509)),M=i(n(306)),B=n(781),V=i(n(782)),F=i(n(48)),L=i(n(110)),H=n(6),z=(0,h.getWindow)(),N="dx-area-data-cell",j="dx-area-row-cell",W="dx-area-column-cell",G="dx-area-description-cell",q="dx-pivotgrid-border",$="dx-pivotgrid",K="dx-row-lines",U="dx-bottom-row",Y="dx-bottom-border",Z="dx-pivotgrid-fields-container",X="dx-area-fields",Q="dx-fieldchooser-popup",J="dx-incompressible-fields",ee="dx-overflow-hidden",te="<tr>",ne="<td>",ie="<div>",oe=66666,ae=["allowSorting","allowSortingBySummary","allowFiltering","allowExpandAll"],re={},se=k.default.inherit({_getDefaultOptions:function(){return(0,x.extend)(this.callBase(),{scrolling:{timeout:300,renderingThreshold:150,minTimeout:10,mode:"standard",useNative:"auto",removeInvisiblePages:!0,virtualRowHeight:50,virtualColumnWidth:100,loadTwoPagesOnStart:!0},encodeHtml:!0,dataSource:null,activeStateEnabled:!1,fieldChooser:{minWidth:250,minHeight:250,enabled:!0,allowSearch:!1,searchTimeout:500,layout:0,title:C.default.format("dxPivotGrid-fieldChooserTitle"),width:600,height:600,applyChangesMode:"instantly"},onContextMenuPreparing:null,allowSorting:!1,allowSortingBySummary:!1,allowFiltering:!1,allowExpandAll:!1,wordWrapEnabled:!0,fieldPanel:{showColumnFields:!0,showFilterFields:!0,showDataFields:!0,showRowFields:!0,allowFieldDragging:!0,visible:!1,texts:{columnFieldArea:C.default.format("dxPivotGrid-columnFieldArea"),rowFieldArea:C.default.format("dxPivotGrid-rowFieldArea"),filterFieldArea:C.default.format("dxPivotGrid-filterFieldArea"),dataFieldArea:C.default.format("dxPivotGrid-dataFieldArea")}},dataFieldArea:"column","export":{enabled:!1,fileName:"PivotGrid",proxyUrl:void 0,ignoreExcelErrors:!0},showRowTotals:!0,showRowGrandTotals:!0,showColumnTotals:!0,showColumnGrandTotals:!0,hideEmptySummaryCells:!0,showTotalsPrior:"none",rowHeaderLayout:"standard",loadPanel:{enabled:!0,text:C.default.format("Loading"),width:200,height:70,showIndicator:!0,indicatorSrc:"",showPane:!0},texts:{grandTotal:C.default.format("dxPivotGrid-grandTotal"),total:C.default.getFormatter("dxPivotGrid-total"),noData:C.default.format("dxDataGrid-noDataText"),showFieldChooser:C.default.format("dxPivotGrid-showFieldChooser"),expandAll:C.default.format("dxPivotGrid-expandAll"),collapseAll:C.default.format("dxPivotGrid-collapseAll"),sortColumnBySummary:C.default.getFormatter("dxPivotGrid-sortColumnBySummary"),sortRowBySummary:C.default.getFormatter("dxPivotGrid-sortRowBySummary"),removeAllSorting:C.default.format("dxPivotGrid-removeAllSorting"),exportToExcel:C.default.format("dxDataGrid-exportToExcel"),dataNotAvailable:C.default.format("dxPivotGrid-dataNotAvailable")},onCellClick:null,onCellPrepared:null,showBorders:!1,stateStoring:{enabled:!1,storageKey:null,type:"localStorage",customLoad:null,customSave:null,savingTimeout:2e3},onExpandValueChanging:null,renderCellCountLimit:2e4,onExporting:null,onExported:null,onFileSaving:null,headerFilter:{width:252,height:325,allowSearch:!1,showRelevantValues:!1,searchTimeout:500,texts:{emptyValue:C.default.format("dxDataGrid-headerFilterEmptyValue"),ok:C.default.format("dxDataGrid-headerFilterOK"),cancel:C.default.format("dxDataGrid-headerFilterCancel")}}})},_updateCalculatedOptions:function(e){var t=this;(0,y.each)(e,function(e,n){(0,y.each)(ae,function(e,i){var o=n._initProperties&&i in n._initProperties&&void 0===n._initProperties[i],a=void 0===n[i]||o;a&&(0,I.setFieldProperty)(n,i,t.option(i))})})},_getDataControllerOptions:function(){var e=this;return{component:e,dataSource:e.option("dataSource"),texts:e.option("texts"),showRowTotals:e.option("showRowTotals"),showRowGrandTotals:e.option("showRowGrandTotals"),showColumnTotals:e.option("showColumnTotals"),showTotalsPrior:e.option("showTotalsPrior"),showColumnGrandTotals:e.option("showColumnGrandTotals"),dataFieldArea:e.option("dataFieldArea"),rowHeaderLayout:e.option("rowHeaderLayout"),hideEmptySummaryCells:e.option("hideEmptySummaryCells"),onFieldsPrepared:function(t){e._updateCalculatedOptions(t)}}},_initDataController:function(){var e=this;e._dataController&&e._dataController.dispose(),e._dataController=new T.DataController(e._getDataControllerOptions()),(0,h.hasWindow)()&&e._dataController.changed.add(function(){e._render()}),e._dataController.scrollChanged.add(function(t){e._scrollLeft=t.left,e._scrollTop=t.top}),e._dataController.loadingChanged.add(function(t){e._updateLoading()}),e._dataController.progressChanged.add(e._updateLoading.bind(e)),e._dataController.dataSourceChanged.add(function(){e._trigger("onChanged")});var t=e.option("onExpandValueChanging");t&&e._dataController.expandValueChanging.add(function(e){t(e)})},_init:function(){var e=this;e.callBase(),e._initDataController(),e._scrollLeft=e._scrollTop=null,e._initActions()},_initActions:function(){var e=this;e._actions={onChanged:e._createActionByOption("onChanged"),onContextMenuPreparing:e._createActionByOption("onContextMenuPreparing"),onCellClick:e._createActionByOption("onCellClick"),onExporting:e._createActionByOption("onExporting"),onExported:e._createActionByOption("onExported"),onFileSaving:e._createActionByOption("onFileSaving"),onCellPrepared:e._createActionByOption("onCellPrepared")}},_trigger:function(e,t){this._actions[e](t)},_optionChanged:function(e){var t=this;if(ae.indexOf(e.name)>=0){var n=this.getDataSource().fields();this._updateCalculatedOptions(n)}switch(e.name){case"dataSource":case"allowSorting":case"allowFiltering":case"allowExpandAll":case"allowSortingBySummary":case"scrolling":case"stateStoring":t._initDataController(),t._fieldChooserPopup.hide(),t._renderFieldChooser(),t._invalidate();break;case"texts":case"showTotalsPrior":case"showRowTotals":case"showRowGrandTotals":case"showColumnTotals":case"showColumnGrandTotals":case"hideEmptySummaryCells":case"dataFieldArea":t._dataController.updateViewOptions(t._getDataControllerOptions());break;case"useNativeScrolling":case"encodeHtml":case"renderCellCountLimit":break;case"rtlEnabled":t.callBase(e),t._renderFieldChooser(),t._renderContextMenu(),(0,h.hasWindow)()&&t._renderLoadPanel(t._dataArea.groupElement(),t.$element()),t._invalidate();break;case"export":t._renderDescriptionArea();break;case"onExpandValueChanging":break;case"onCellClick":case"onContextMenuPreparing":case"onExporting":case"onExported":case"onFileSaving":case"onCellPrepared":t._actions[e.name]=t._createActionByOption(e.name);break;case"fieldChooser":t._renderFieldChooser(),t._renderDescriptionArea();break;case"loadPanel":(0,h.hasWindow)()&&(t._renderLoadPanel(t._dataArea.groupElement(),t.$element()),t._invalidate());break;case"fieldPanel":t._renderDescriptionArea(),t._invalidate();break;case"headerFilter":t._renderFieldChooser(),t._invalidate();break;case"showBorders":t._tableElement().toggleClass(q,!!e.value),t.updateDimensions();break;case"wordWrapEnabled":t._tableElement().toggleClass("dx-word-wrap",!!e.value),t.updateDimensions();break;case"rowHeaderLayout":t._tableElement().find("."+j).toggleClass("dx-area-tree-view","tree"===e.value),t._dataController.updateViewOptions(t._getDataControllerOptions());break;case"height":case"width":t._hasHeight=null,t.callBase(e),t.resize();break;default:t.callBase(e)}},_updateScrollPosition:function(e,t,n){var i,o,a=this,r=a._scrollTop||a._scrollLeft;a._scrollUpdating||(a._scrollUpdating=!0,t&&!t.hasScroll()&&a._hasHeight&&(a._scrollTop=null),e&&!e.hasScroll()&&(a._scrollLeft=null),(null!==a._scrollTop||null!==a._scrollLeft||r||a.option("rtlEnabled"))&&(i=a._scrollTop||0,o=a._scrollLeft||0,n.scrollTo({x:o,y:i}),e.scrollTo(o),t.scrollTo(i),a._dataController.updateWindowScrollPosition(a._scrollTop)),a._scrollUpdating=!1)},_subscribeToEvents:function(e,t,n){var i=this,o=function(o){var a=o.scrollOffset,r=(0,b.isDefined)(a.left)?a.left:i._scrollLeft,s=(0,b.isDefined)(a.top)&&i._hasHeight?a.top:i._scrollTop;(i._scrollLeft||0)===(r||0)&&(i._scrollTop||0)===(s||0)||(i._scrollLeft=r,i._scrollTop=s,i._updateScrollPosition(e,t,n),"virtual"===i.option("scrolling.mode")&&i._dataController.setViewportPosition(i._scrollLeft,i._scrollTop))};(0,y.each)([e,t,n],function(e,t){s(t,o)}),!i._hasHeight&&i._dataController.subscribeToWindowScrollEvents(n.groupElement())},_clean:v.noop,_needDelayResizing:function(e){var t=e.length*(e.length?e[0].length:0);return t>this.option("renderCellCountLimit")},_renderFieldChooser:function(){var e=this,t=e._pivotGridContainer,n=e.option("fieldChooser")||{},i="onDemand"===n.applyChangesMode?[{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:C.default.format("OK"),onClick:function(t){e._fieldChooserPopup.$content().dxPivotGridFieldChooser("applyChanges"),e._fieldChooserPopup.hide()}}},{toolbar:"bottom",location:"after",widget:"dxButton",options:{text:C.default.format("Cancel"),onClick:function(t){e._fieldChooserPopup.hide()}}}]:[],o={layout:n.layout,texts:n.texts||{},dataSource:e.getDataSource(),allowSearch:n.allowSearch,searchTimeout:n.searchTimeout,width:void 0,height:void 0,headerFilter:e.option("headerFilter"),encodeHtml:e.option("encodeHtml"),applyChangesMode:n.applyChangesMode,onContextMenuPreparing:function(t){e._trigger("onContextMenuPreparing",t)}},a={shading:!1,title:n.title,width:n.width,height:n.height,showCloseButton:!0,resizeEnabled:!0,minWidth:n.minWidth,minHeight:n.minHeight,toolbarItems:i,onResize:function(e){e.component.$content().dxPivotGridFieldChooser("updateDimensions")},onShown:function(t){e._createComponent(t.component.content(),R.default,o)},onHidden:function(e){var t=e.component.$content().dxPivotGridFieldChooser("instance");t.resetTreeView(),t.cancelChanges()}};e._fieldChooserPopup?(e._fieldChooserPopup.option(a),e._fieldChooserPopup.$content().dxPivotGridFieldChooser(o)):e._fieldChooserPopup=e._createComponent((0,d.default)(ie).addClass(Q).appendTo(t),F.default,a)},_renderContextMenu:function(){var e=this,t=e._pivotGridContainer;e._contextMenu&&e._contextMenu.$element().remove(),e._contextMenu=e._createComponent((0,d.default)(ie).appendTo(t),L.default,{onPositioning:function(t){var n=t.event;if(t.cancel=!0,n){var i=n.target.cellIndex>=0?n.target:(0,d.default)(n.target).closest("td").get(0);if(i){var o=e._createEventArgs(i,n),a=e._getContextMenuItems(o);return a?(t.component.option("items",a),void(t.cancel=!1)):void 0}}},onItemClick:function(e){e.itemData.onItemClick&&e.itemData.onItemClick(e)},cssClass:$,target:e.$element()})},_getContextMenuItems:function(e){var t=this,n=[],i=t.option("texts");if("row"===e.area||"column"===e.area){var o=e[e.area+"Fields"],a=e["column"===e.area?"rowFields":"columnFields"],r=e.cell.path&&o[e.cell.path.length-1],s=t.getDataSource();if(r&&r.allowExpandAll&&e.cell.path.length<e[e.area+"Fields"].length&&!s.paginate()&&(n.push({beginGroup:!0,icon:"none",text:i.expandAll,onItemClick:function(){s.expandAll(r.index)}}),n.push({text:i.collapseAll,icon:"none",onItemClick:function(){s.collapseAll(r.index)}})),e.cell.isLast&&!s.paginate()){var l=0;(0,y.each)(a,function(t,o){o.allowSortingBySummary&&(0,y.each)(e.dataFields,function(t,a){if(!(0,b.isDefined)(e.cell.dataIndex)||e.cell.dataIndex===t){var r=!(0,b.isDefined)(e.cell.dataIndex)&&e.dataFields.length>1,u="column"===e.area?i.sortColumnBySummary:i.sortRowBySummary,c=(0,I.findField)(e.dataFields,o.sortBySummaryField)===t&&(e.cell.path||[]).join("/")===(o.sortBySummaryPath||[]).join("/"),d=(0,m.format)(u,r?o.caption+" - "+a.caption:o.caption);n.push({beginGroup:0===l,icon:c?"desc"===o.sortOrder?"sortdowntext":"sortuptext":"none",text:d,onItemClick:function(){s.field(o.index,{sortBySummaryField:a.name||a.caption||a.dataField,sortBySummaryPath:e.cell.path,sortOrder:"desc"===o.sortOrder?"asc":"desc"}),s.load()}}),l++}})}),(0,y.each)(a,function(e,t){if(t.allowSortingBySummary&&(0,b.isDefined)(t.sortBySummaryField))return n.push({beginGroup:0===l,icon:"none",text:i.removeAllSorting,onItemClick:function(){(0,y.each)(a,function(e,t){s.field(t.index,{sortBySummaryField:void 0,sortBySummaryPath:void 0,sortOrder:void 0})}),s.load()}}),!1})}}if(t.option("fieldChooser.enabled")&&n.push({beginGroup:!0,icon:"columnchooser",text:i.showFieldChooser,onItemClick:function(){t._fieldChooserPopup.show()}}),t.option("export.enabled")&&n.push({beginGroup:!0,icon:"xlsxfile",text:i.exportToExcel,onItemClick:function(){t.exportToExcel()}}),e.items=n,t._trigger("onContextMenuPreparing",e),n=e.items,n&&n.length)return n},_createEventArgs:function(e,t){var n=this,i=n.getDataSource(),o={rowFields:i.getAreaFields("row"),columnFields:i.getAreaFields("column"),dataFields:i.getAreaFields("data"),event:t};return c((0,d.default)(e))?(0,x.extend)(n._createFieldArgs(e),o):(0,x.extend)(n._createCellArgs(e),o)},_createFieldArgs:function(e){var t=(0,d.default)(e).children().data("field"),n={field:t};return(0,b.isDefined)(t)?n:{}},_createCellArgs:function(e){var t=(0,d.default)(e),n=e.cellIndex,i=e.parentElement.rowIndex,o=t.closest("table"),a=o.data("data"),r=a&&a[i]&&a[i][n],s={area:o.data("area"),rowIndex:i,columnIndex:n,cellElement:(0,_.getPublicElement)(t),cell:r};return s},_handleCellClick:function(e){var t=this,n=t._createEventArgs(e.currentTarget,e),i=n.cell;i&&(n.area||!n.rowIndex&&!n.columnIndex)&&(t._trigger("onCellClick",n),i&&!n.cancel&&(0,b.isDefined)(i.expanded)&&setTimeout(function(){t._dataController[i.expanded?"collapseHeaderItem":"expandHeaderItem"](n.area,i.path)}))},_getNoDataText:function(){return this.option("texts.noData")},_renderNoDataText:D.default.renderNoDataText,_renderLoadPanel:D.default.renderLoadPanel,_updateLoading:function(e){var t=this,n=t._dataController.isLoading();if(t._loadPanel){var i=t._loadPanel.option("visible");i||(t._startLoadingTime=new Date),n&&(e?new Date-t._startLoadingTime>=1e3&&t._loadPanel.option("message",Math.floor(100*e)+"%"):t._loadPanel.option("message",t.option("loadPanel.text"))),clearTimeout(t._hideLoadingTimeoutID),i&&!n?t._hideLoadingTimeoutID=setTimeout(function(){t._loadPanel.option("visible",!1),t.$element().removeClass(ee)}):(t._loadPanel.option("visible",n),t.$element().toggleClass(ee,!n))}},_renderDescriptionArea:function(){var e,t=this,n=this.$element(),i=n.find("."+G),o=(0,d.default)(ie).addClass("dx-pivotgrid-toolbar"),a=this.option("fieldPanel"),r=n.find(".dx-filter-header"),s=n.find(".dx-column-header");if(e=a.visible&&a.showFilterFields?r:a.visible&&(a.showDataFields||a.showColumnFields)?s:i,s.toggleClass(Y,!(!a.visible||!a.showDataFields&&!a.showColumnFields)),r.toggleClass(Y,!(!a.visible||!a.showFilterFields)),i.toggleClass("dx-pivotgrid-background",a.visible&&(a.showDataFields||a.showColumnFields||a.showRowFields)),this.$element().find(".dx-pivotgrid-toolbar").remove(),o.prependTo(e),this.option("fieldChooser.enabled")){var l=(0,d.default)(ie).appendTo(o).addClass("dx-pivotgrid-field-chooser-button"),u={icon:"columnchooser",hint:this.option("texts.showFieldChooser"),onClick:function(){t.getFieldChooserPopup().show()}};this._createComponent(l,"dxButton",u)}if(this.option("export.enabled")){var c=(0,d.default)(ie).appendTo(o).addClass("dx-pivotgrid-export-button"),h={icon:"xlsxfile",hint:this.option("texts.exportToExcel"),onClick:function(){t.exportToExcel()}};this._createComponent(c,"dxButton",h)}},_detectHasContainerHeight:function(){var e=this,t=e.$element();if((0,b.isDefined)(e._hasHeight)){var n=e.option("height")||e.$element().get(0).style.height;n&&e._hasHeight^"auto"!==n&&(e._hasHeight=null)}if(!(0,b.isDefined)(e._hasHeight)&&!t.is(":hidden")){e._pivotGridContainer.addClass("dx-hidden");var i=(0,d.default)(ie).height(oe);t.append(i),e._hasHeight=t.height()!==oe,e._pivotGridContainer.removeClass("dx-hidden"),i.remove()}},_renderHeaders:function(e,t,n,i){var o=this,a=o.getDataSource();o._rowFields=o._rowFields||new P.FieldsArea(o,"row"),o._rowFields.render(e,a.getAreaFields("row")),o._columnFields=o._columnFields||new P.FieldsArea(o,"column"),o._columnFields.render(t,a.getAreaFields("column")),o._filterFields=o._filterFields||new P.FieldsArea(o,"filter"),o._filterFields.render(n,a.getAreaFields("filter")),o._dataFields=o._dataFields||new P.FieldsArea(o,"data"),o._dataFields.render(i,a.getAreaFields("data")),o.$element().dxPivotGridFieldChooserBase("instance").renderSortable()},_createTableElement:function(){var e=this,t=(0,d.default)("<table>").css({width:"100%"}).toggleClass(q,!!e.option("showBorders")).toggleClass("dx-word-wrap",!!e.option("wordWrapEnabled"));return p.default.on(t,(0,S.addNamespace)(w.name,"dxPivotGrid"),"td",e._handleCellClick.bind(e)),t},_renderDataArea:function(e){var t=this,n=t._dataArea||new O.DataArea(t);return t._dataArea=n,n.render(e,t._dataController.getCellsInfo()),n},_renderRowsArea:function(e){var t=this,n=t._rowsArea||new E.VerticalHeadersArea(t);return t._rowsArea=n,n.render(e,t._dataController.getRowsInfo()),n},_renderColumnsArea:function(e){var t=this,n=t._columnsArea||new E.HorizontalHeadersArea(t);return t._columnsArea=n,n.render(e,t._dataController.getColumnsInfo()),n},_initMarkup:function(){var e=this;e.callBase.apply(this,arguments),e.$element().addClass($)},_renderContentImpl:function(){var e,t,n,i,o,a,s,l,u=this,c=!u._pivotGridContainer;i=!c&&u._tableElement(),i||(u.$element().addClass(K).addClass(Z),u._pivotGridContainer=(0,d.default)(ie).addClass("dx-pivotgrid-container"),u._renderFieldChooser(),u._renderContextMenu(),e=(0,d.default)(ne).addClass(W),t=(0,d.default)(ne).addClass(j),n=(0,d.default)(ne).addClass(N),i=u._createTableElement(),l=(0,d.default)(ne).addClass("dx-data-header"),s=(0,d.default)("<td>").attr("colspan","2").addClass("dx-filter-header"),a=(0,d.default)(ne).addClass("dx-column-header"),o=(0,d.default)(ne).addClass(G),(0,d.default)(te).append(s).appendTo(i),(0,d.default)(te).append(l).append(a).appendTo(i),(0,d.default)(te).toggleClass("dx-ie",f.default.msie===!0).append(o).append(e).appendTo(i),(0,d.default)(te).addClass(U).append(t).append(n).appendTo(i),u._pivotGridContainer.append(i),u.$element().append(u._pivotGridContainer),"tree"===u.option("rowHeaderLayout")&&t.addClass("dx-area-tree-view")),u.$element().addClass(ee),u._createComponent(u.$element(),M.default,{dataSource:u.getDataSource(),encodeHtml:u.option("encodeHtml"),allowFieldDragging:u.option("fieldPanel.allowFieldDragging"),headerFilter:u.option("headerFilter"),visible:u.option("visible")});var h=u._renderDataArea(n),p=u._renderRowsArea(t),g=u._renderColumnsArea(e);h.tableElement().prepend(g.headElement()),c&&(u._renderLoadPanel(h.groupElement().parent(),u.$element()),u._renderDescriptionArea(),p.processScroll(),g.processScroll()),[h,p,g].forEach(function(e){r(e)}),u._renderHeaders(o,a,s,l),u._update(c)},_update:function(e){var t=this,n=function(){t.updateDimensions()};t._needDelayResizing(t._dataArea.getData())&&e?setTimeout(n):n()},_fireContentReadyAction:function(){this._dataController.isLoading()||this.callBase()},getScrollPath:function(e){var t=this;return"column"===e?t._columnsArea.getScrollPath(t._scrollLeft):t._rowsArea.getScrollPath(t._scrollTop)},getDataSource:function(){return this._dataController.getDataSource()},getFieldChooserPopup:function(){return this._fieldChooserPopup},hasScroll:function(e){var t=this;return"column"===e?t._columnsArea.hasScroll():t._rowsArea.hasScroll()},_dimensionChanged:function(){this.updateDimensions()},_visibilityChanged:function(e){e&&this.updateDimensions()},_dispose:function(){var e=this;clearTimeout(e._hideLoadingTimeoutID),e.callBase.apply(e,arguments),e._dataController&&e._dataController.dispose()},_tableElement:function(){return this.$element().find("table").first()},addWidgetPrefix:function(e){return"dx-pivotgrid-"+e},resize:function(){this.updateDimensions()},isReady:function(){return this.callBase()&&!this._dataController.isLoading()},updateDimensions:function(){var e,t,n,i,r=this,s=r._tableElement(),c=r._rowsArea,p=r._columnsArea,g=r._dataArea,_=0,m=0,b=0,x=r.option("scrolling")||{},w=l(x.useNative),C=w.scrollBarWidth,k=s.find("."+N),S=s.find("."+j),D=s.find("."+W),T=s.find("."+G),O=s.find(".dx-filter-header"),E=s.find(".dx-column-header"),P=r._rowFields,R=new H.Deferred;if((0,h.hasWindow)()){var M=P.isVisible()&&"tree"!==r.option("rowHeaderLayout");r._detectHasContainerHeight(),g.headElement().length||g.tableElement().prepend(p.headElement()),M&&(c.updateColspans(P.getColumnsCount()),c.tableElement().prepend(P.headElement())),s.addClass(J),g.reset(),c.reset(),p.reset(),P.reset();var B=function(e,t){return t-e>=1},V=function(e,t,n,i,o){return n?e:t+(i?o:0)};return(0,v.deferUpdate)(function(){var l=g.getColumnsWidth(),h=c.getRowsHeight(),F=M?h.slice(1):h,L=g.getRowsHeight(),N=(0,A.getSize)(T[0],"height",{paddings:!0,borders:!0,margins:!0})+(M?h[0]:0),j=r._dataController.getColumnsInfo().length,W=(0,I.mergeArraysByMaxValue)(F,L.slice(j)),G=L.slice(0,j),q=o(G),$=c.getColumnsWidth(),K=0,U=0;if(r._hasHeight){K=O.height();var Z=s.find(".dx-data-header"),X=f.default.msie?(0,A.getSize)(Z.get(0),"height",{paddings:!1,borders:!1,margins:!1}):Z.height();t=u([D,k,s,E,O],"height"),U=r.$element().height()-K-X-(Math.max(g.headElement().height(),D.height(),N)+t)}if(_=g.tableElement().width(),m=o(W),!_||!m)return void R.resolve();b=o($);var Q=r.$element().width();t=u([S,k,s],"width"),e=Q-b-t,e=e>0?e:_;var ee=_-e,te=ee>=0&&ee<=2;te&&(a(l,ee),_=e),n=r._hasHeight&&B(U,m),i=B(e,_);var ne=V(U,m,n,i,C);(0,v.deferRender)(function(){if(p.tableElement().append(g.headElement()),P.tableElement().append(c.headElement()),!i&&n&&C&&(a(l,C),_-=C),N>q&&(a(G,q-N),p.setRowsHeight(G)),s.removeClass(J),E.children().css("maxWidth",e),p.groupWidth(e),p.processScrollBarSpacing(n?C:0),p.setColumnsWidth(l),c.groupHeight(r._hasHeight?ne:"auto"),c.processScrollBarSpacing(i?C:0),c.setColumnsWidth($),c.setRowsHeight(W),g.setColumnsWidth(l),g.setRowsHeight(W),g.groupWidth(e),g.groupHeight(r._hasHeight?ne:"auto"),M&&P.setColumnsWidth($),k.toggleClass(Y,!n),S.toggleClass(Y,!n),!r._hasHeight&&Q!==r.$element().width()){var t=Q-r.$element().width();i||(a(l,t),p.setColumnsWidth(l),g.setColumnsWidth(l)),g.groupWidth(e-t),p.groupWidth(e-t)}if(r._hasHeight&&r._filterFields.isVisible()&&O.height()!==K){var o=O.height()-K;if(o>0){n=B(U-o,m);var u=V(U-o,m,n,i,C);g.groupHeight(u),c.groupHeight(u)}}if("virtual"===x.mode){var h=r._dataController.calculateVirtualContentParams({virtualRowHeight:x.virtualRowHeight,virtualColumnWidth:x.virtualColumnWidth,itemWidths:l,itemHeights:W,rowCount:W.length,columnCount:l.length,viewportWidth:e,viewportHeight:r._hasHeight?ne:(0,d.default)(z).outerHeight()});g.setVirtualContentParams({top:h.contentTop,left:h.contentLeft,width:h.width,height:h.height}),c.setVirtualContentParams({top:h.contentTop,width:b,height:h.height}),p.setVirtualContentParams({left:h.contentLeft,width:h.width,height:p.groupElement().height()})}var f=[];g.processScroll(w.scrollBarUseNative,i,n),(0,y.each)([p,c,g],function(e,t){f.push(t&&t.updateScrollable())}),r._updateLoading(),r._renderNoDataText(k),H.when.apply(d.default,f).done(function(){r._updateScrollPosition(p,c,g),r._subscribeToEvents(p,c,g),R.resolve()})})}),R}},applyPartialDataSource:function(e,t,n){this._dataController.applyPartialDataSource(e,t,n)}}).inherit(B.ExportMixin).include(V.default);(0,g.default)("dxPivotGrid",se);var le=se;t.default=le,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.DataController=void 0;var a=i(n(27)),r=n(6),s=n(0),l=n(12),u=n(3),c=i(n(15)),d=n(42),h=n(4),f=n(1),p=n(485),g=n(502),_=n(474),m=i(n(304)),v=n(89),y=Math,b="GT",x="T",w="D",C="#N/A",k=300,S=function(e,t,n){e[t]||(e[t]=function(){var e=this._dataSource;return e?e[t].apply(e,arguments):n})},D=c.default.inherit(function(){function e(e,t,n){var i=e.text;return(0,f.isDefined)(e.displayText)?i=e.displayText:(0,f.isDefined)(e.caption)?i=e.caption:e.type===b&&(i=n.texts.grandTotal),e.isAdditionalTotal&&(i=(0,d.format)(n.texts.total||"",i)),i}function t(e,t,n){return e===C?n:(0,v.formatValue)(e,t)}function n(e,t){var n=[];return(0,u.each)(e,function(e,i){var o=(0,v.findField)(t,i.sortBySummaryField);o>=0&&n.push((i.sortBySummaryPath||[]).concat([o]))}),n}function i(e,t){for(var n=0,i=[],o=0;o<e.length;o++)for(var a=0;a<e[o].length;a++){var r=(e[o][a].rowspan||1)-1,s=o+r;if(i[o]&&(n-=i[o],i[o]=0),t(e[o][a],s,o,a,n)===!1)break;i[o+(e[o][a].rowspan||1)]=(i[o+(e[o][a].rowspan||1)]||0)+1,n++}}function c(e,n,o,a,r,s){var l=[],u="row"===r,c=o.values;return c.length&&i(e,function(e,i){var r=l[i]=[],d=c[e.dataSourceIndex>=0?e.dataSourceIndex:o.grandTotalRowIndex]||[];e.isLast&&(0,g.foreachColumnInfo)(n,function(n,i){var l=(u?e.dataIndex:n.dataIndex)||0,c=a[l];if(n.isLast&&c){var h=d[n.dataSourceIndex>=0?n.dataSourceIndex:o.grandTotalColumnIndex];Array.isArray(h)||(h=[h]);var f=h[l];r[i]={text:t(f,c,s),value:f,format:c.format,dataType:c.dataType,columnType:n.type,rowType:e.type,rowPath:e.path||[],columnPath:n.path||[],dataIndex:l},c.width&&(r[i].width=c.width)}})}),l}function D(e,t){var n=0,i=[];return(0,v.foreachTree)(e,function(e){var o=e[0],a=(0,v.createPath)(e);if(!o.children||t.showTotals!==!1){var r=(0,s.extend)(!0,{},o,{visibleIndex:n++,path:a});(0,f.isDefined)(r.index)?i[r.index]=r:i.push(r)}}),i}function I(e,t,n){if(t&&"virtual"===t.option("scrolling.mode"))return new p.VirtualScrollController(t,(0,s.extend)({hasKnownLastPage:function(){return!0},pageCount:function(){return y.ceil(this.totalItemsCount()/this.pageSize())},updateLoading:function(){},itemsCount:function(){return this.pageIndex()<this.pageCount()-1?this.pageSize():this.totalItemsCount()%this.pageSize()},items:function(){return[]},viewportItems:function(){return[]},onChanged:function(){},isLoading:function(){return e.isLoading()},changingDuration:function(){var t=e._dataSource;return t.paginate()?k:e._changingDuration||0}},n))}function T(e){var t=[];return(0,u.each)(e,function(e,n){n.showTotals===!1&&t.push(e)}),t}function O(e){var t=[];return e.forEach(function(e,n){(void 0===e.showValues&&e.showTotals===!1||e.showValues===!1)&&t.push(n)}),t}function E(e,t){var n=[];return(0,u.each)(e,function(e,t){t.showGrandTotals===!1&&n.push(e)}),0===t.length&&n.length===e.length&&(n=[]),n}var A=function(){function t(e,t,n){var i=e[t]=e[t]||[],o=i[n]={};if(e[t+1]){o.children=e[t+1];for(var a=t+1;a<e.length;a++)e[a]=void 0;e.length=t+1}return o}function n(e,n){var i,o=n&&n.length||0,a=[],s=new r.Deferred;return(0,r.when)((0,v.foreachTreeAsync)(e,function(e,n){var r=e[0],s=(0,v.createPath)(e);i=t(a,s.length,n),i.type=w,i.value=r.value,i.path=s,i.text=r.text,i.index=r.index,i.displayText=r.displayText,i.key=r.key,i.isEmpty=r.isEmpty,s.length<o&&(!r.children||0!==r.children.length)&&(i.expanded=!!r.children)})).done(function(){s.resolve(t(a,0,0).children||[])}),s}function i(e,t,n){(0,v.foreachTree)(e,function(e){var i,o=e[0];if(!o.children||0===o.children.length)for(o.children=[],i=0;i<t.length;i++){var a=o.type===b,r=o.type===x,s=o.type===w,u=t[i].visible===!1||a&&(0,l.inArray)(i,n.hiddenGrandTotals)!==-1||r&&(0,l.inArray)(i,n.hiddenTotals)!==-1||s&&(0,l.inArray)(i,n.hiddenValues)!==-1;u||o.children.push({caption:t[i].caption,path:o.path,type:o.type,value:i,index:o.index,dataIndex:i,isMetric:!0,isEmpty:o.isEmpty&&o.isEmpty[i]})}})}function o(e,t,n,i){n=n||i,(0,v.foreachTree)(e,function(o,a){var r=o[0],l=(o[1]?o[1].children:e)||[],u=t[o.length-1];r.type===w&&r.expanded&&(u.showTotals!==!1||i)&&(a!==-1&&l.splice(n?a:a+1,0,(0,s.extend)({},r,{children:null,type:x,expanded:!!n||null,isAdditionalTotal:!0})),n&&(r.expanded=null))})}function a(e){(0,v.foreachTree)([{children:e}],function(t,n){var i=t[0],o=(t[1]?t[1].children:e)||[],a=i.isEmpty;a&&a.length&&(a=i.isEmpty.filter(function(e){return e}).length===a.length),i&&!i.children&&a&&(o.splice(n,1),_(t,1))})}var c=function(e){var t=0;return(0,v.foreachTree)(e,function(e){t=y.max(t,e.length)}),t},d=function(e,t,n,i){var o={type:e.type,text:e.text};return e.path&&(o.path=e.path),e.width&&(o.width=e.width),(0,f.isDefined)(e.wordWrapEnabled)&&(o.wordWrapEnabled=e.wordWrapEnabled),e.isLast&&(o.isLast=!0),e.sorted&&(o.sorted=!0),e.isMetric&&(o.dataIndex=e.dataIndex),(0,f.isDefined)(e.expanded)&&(o.expanded=e.expanded),t>1&&(o[n?"colspan":"rowspan"]=t),e.depthSize&&e.depthSize>1&&(o[n?"rowspan":"colspan"]=e.depthSize),e.index>=0&&(o.dataSourceIndex=e.index),i&&e.children&&e.children.length&&!e.children[0].isMetric&&(o.width=null,o.isWhiteSpace=!0),o},h=function(e,t){var n=t.lastIndex-t.index||1,i=function(e,t,n,i,o){for(var a=o?i:n;!e[a];)e.push([]);o?e[a].push(t):e[a].unshift(t)},o=d(t.headerItem,n,t.isHorizontal,t.isTree);return i(e,o,t.index,t.depth,t.isHorizontal),t.headerItem.children&&0!==t.headerItem.children.length?t.lastIndex:t.lastIndex+1},p=function(e,t){var n,i=e[0],o=(0,f.isString)(t[0]),a=i.dataIndex>=0?e[1]:i;return n=o&&t[0].indexOf("&[")!==-1&&a.key||!a.key?(0,v.createPath)(e):(0,u.map)(e,function(e){return e.dataIndex>=0?e.value:e.text}).reverse(),i.type===b&&(n=n.slice(1)),n.join("/")===t.join("/")},g=function(t,s,c,d,h){var g=c.length,_=n(t,s),m=h.dataFields,y=new r.Deferred;return(0,r.when)(_).done(function(t){h.notifyProgress(.5),h.showGrandTotals&&t[h.showTotalsPrior?"unshift":"push"]({type:b,isEmpty:h.isEmptyGrandTotal});var n=h.showTotals===!1||m.length>0&&m.length===h.hiddenTotals.length,_=m.length>0&&h.hiddenValues.length===m.length;_&&n&&(d=1),n&&"tree"!==h.layout||o(t,s,h.showTotalsPrior,"tree"===h.layout),(0,r.when)((0,v.foreachTreeAsync)(t,function(e){var t=e[0];t.children&&0!==t.children.length||(t.depthSize=d-e.length+1)})).done(function(){g>1&&i(t,c,h),!h.showEmpty&&a(t),h.notifyProgress(.75),(0,r.when)((0,v.foreachTreeAsync)(t,function(n){var i=n[0],o=i.isMetric,a=s[n.length-1]||{};if(i.type!==w||o||(i.width=a.width),_===!0&&i.type===w){var r=(n[1]?n[1].children:t)||[];return void r.splice((0,l.inArray)(i,r),1)}o?i.wordWrapEnabled=c[i.dataIndex].wordWrapEnabled:i.wordWrapEnabled=a.wordWrapEnabled,i.isLast=!i.children||!i.children.length,i.isLast&&(0,u.each)(h.sortBySummaryPaths,function(e,t){if((0,f.isDefined)(i.dataIndex)||(t=t.slice(0),t.pop()),p(n,t))return i.sorted=!0,!1}),i.text=e(i,a,h)})).done(function(){t.length||t.push({}),h.notifyProgress(1),y.resolve(t)})})}),y},_=function e(t,n){var i=t[n+1];!t[n].children.length&&i&&i.children&&(i.children.splice((0,l.inArray)(t[n],i.children),1),e(t,n+1))},m=function(e,t,n,i,o){var a,r,s=0,l=[0];(0,v.foreachTree)(t,function(t){var u=t[0];for(r=u.isMetric?n:t.length-1;l.length-1<r;)l.push(l[l.length-1]);a=l[r]||0,s=h(e,{headerItem:u,index:a,lastIndex:s,depth:r,isHorizontal:i,isTree:o}),l.length=r,l.push(s)})};return function(e,t,n,i,o){var a=[],s=c(e)||1,l=new r.Deferred;return g(e,t,n,s,o).done(function(e){m(a,e,s,i,"tree"===o.layout),o.notifyProgress(1),l.resolve(a)}),l}}(),P={ctor:function(e){var t=this,n=t._fireChanged.bind(t);e=t._options=e||{},t.dataSourceChanged=(0,a.default)(),t._dataSource=t._createDataSource(e),t._rowsScrollController=I(t,e.component,{totalItemsCount:function(){return t.totalRowCount()},pageIndex:function(e){return t.rowPageIndex(e)},pageSize:function(){return t.rowPageSize()},load:function(){return t._rowsScrollController.pageIndex()>=this.pageCount()&&t._rowsScrollController.pageIndex(this.pageCount()-1),t._rowsScrollController.handleDataChanged(function(){t._dataSource.paginate()?t._dataSource.load():n.apply(this,arguments)})}}),t._columnsScrollController=I(t,e.component,{totalItemsCount:function(){return t.totalColumnCount()},pageIndex:function(e){return t.columnPageIndex(e)},pageSize:function(){return t.columnPageSize()},load:function(){return t._columnsScrollController.pageIndex()>=this.pageCount()&&t._columnsScrollController.pageIndex(this.pageCount()-1),
t._columnsScrollController.handleDataChanged(function(){t._dataSource.paginate()?t._dataSource.load():n.apply(this,arguments)})}}),t._stateStoringController=new _.StateStoringController(e.component).init(),t._columnsInfo=[],t._rowsInfo=[],t._cellsInfo=[],t.expandValueChanging=(0,a.default)(),t.loadingChanged=(0,a.default)(),t.progressChanged=(0,a.default)(),t.scrollChanged=(0,a.default)(),t.load(),t._update(),t.changed=(0,a.default)()},_fireChanged:function(){var e=this,t=new Date;e.changed&&!e._lockChanged&&e.changed.fire(),e._changingDuration=new Date-t},_correctSkipsTakes:function(e,t,n,i,o,a){var r=n?e+n-1:e;o[i.length]=o[i.length]||0,a[i.length]=a[i.length]||0,r<t?o[i.length]++:a[i.length]++},_calculatePagingForRowExpandedPaths:function(e,t,n,i,o){var a,r,s,l=this._rowsInfo,u=Math.min(e.rowSkip+e.rowTake,l.length),c=e.rowExpandedPaths,d=[],h={};for(c.forEach(function(e,t){h[e]=t}),a=0;a<u;a++){for(n.length=t.length=d.length+1,r=0;r<l[a].length;r++){var f=l[a][r];if("D"===f.type){this._correctSkipsTakes(a,e.rowSkip,f.rowspan,d,t,n),s=f.path||s;var p=s&&s.length>1?h[s.slice(0,-1)]:-1;p>=0&&(i[p]=t[d.length]||0,o[p]=n[d.length]||0),f.rowspan&&d.push(f.rowspan)}}d=d.map(function(e){return e-1}).filter(function(e){return e>0})}},_calculatePagingForColumnExpandedPaths:function(e,t,n,i,o){var a={},r={};(0,g.foreachColumnInfo)(this._columnsInfo,function(t,n){if("D"===t.type&&t.path&&void 0===t.dataIndex){var i=t.colspan||1,o=t.path.slice(0,-1).toString();a[o]=a[o]||0,r[o]=r[o]||0,n+i<=e.columnSkip?a[o]++:n<e.columnSkip+e.columnTake&&r[o]++}}),t[0]=a[[]],n[0]=r[[]],e.columnExpandedPaths.forEach(function(e,t){var n=a[e],s=r[e];void 0!==n&&(i[t]=n),void 0!==s&&(o[t]=s)})},_processPagingForExpandedPaths:function(e,t,n,i){var o=e[t+"ExpandedPaths"],a=o.map(function(){return 0}),r=o.map(function(){return i?e.pageSize:0}),s=[],l=[];i||("row"===t?this._calculatePagingForRowExpandedPaths(e,s,l,a,r):this._calculatePagingForColumnExpandedPaths(e,s,l,a,r)),this._savePagingForExpandedPaths(e,t,n,s[0],l[0],a,r)},_savePagingForExpandedPaths:function(e,t,n,i,a,r,l){var u=e[t+"ExpandedPaths"];e[t+"ExpandedPaths"]=[],e[t+"Skip"]=void 0!==i?i:e[t+"Skip"],e[t+"Take"]=void 0!==a?a:e[t+"Take"];for(var c=0;c<u.length;c++)if(l[c]){var d,h=e.area&&e.area!==t;n.push((0,s.extend)({area:t,headerName:t+"s"},e,(d={},o(d,t+"Skip",r[c]),o(d,t+"Take",l[c]),o(d,h?"oppositePath":"path",u[c]),d)))}},_handleCustomizeStoreLoadOptions:function(e,t){var n=this,i=e[0],o=this._rowsScrollController;if(this._dataSource.paginate()&&o){var a=o._dataSource.pageSize();"rows"===i.headerName?(i.rowSkip=0,i.rowTake=a,i.rowExpandedPaths=[]):(i.rowSkip=o.beginPageIndex()*a,i.rowTake=(o.endPageIndex()-o.beginPageIndex()+1)*a,this._processPagingForExpandedPaths(i,"row",e,t))}var r=this._columnsScrollController;if(this._dataSource.paginate()&&r){var s=r._dataSource.pageSize();e.forEach(function(i,o){"columns"===i.headerName?(i.columnSkip=0,i.columnTake=s,i.columnExpandedPaths=[]):(i.columnSkip=r.beginPageIndex()*s,i.columnTake=(r.endPageIndex()-r.beginPageIndex()+1)*s,n._processPagingForExpandedPaths(i,"column",e,t))})}},load:function(){var e=this,t=this._stateStoringController;t.isEnabled()&&!t.isLoaded()?t.load().always(function(t){t?e._dataSource.state(t):e._dataSource.load()}):e._dataSource.load()},calculateVirtualContentParams:function(e){var t=this,n=t._rowsScrollController,i=t._columnsScrollController;if(n&&i)return n.viewportItemSize(e.virtualRowHeight),n.viewportSize(e.viewportHeight/n.viewportItemSize()),n.setContentSize(e.itemHeights),i.viewportItemSize(e.virtualColumnWidth),i.viewportSize(e.viewportWidth/i.viewportItemSize()),i.setContentSize(e.itemWidths),(0,h.deferUpdate)(function(){i.loadIfNeed(),n.loadIfNeed()}),t.scrollChanged.fire({left:i.getViewportPosition(),top:n.getViewportPosition()}),{contentTop:n.getContentOffset(),contentLeft:i.getContentOffset(),width:i.getVirtualContentSize(),height:n.getVirtualContentSize()}},setViewportPosition:function(e,t){this._rowsScrollController.setViewportPosition(t||0),this._columnsScrollController.setViewportPosition(e||0)},subscribeToWindowScrollEvents:function(e){this._rowsScrollController&&this._rowsScrollController.subscribeToWindowScrollEvents(e)},updateWindowScrollPosition:function(e){this._rowsScrollController&&this._rowsScrollController.scrollTo(e)},updateViewOptions:function(e){(0,s.extend)(this._options,e),this._update()},_handleExpandValueChanging:function(e){this.expandValueChanging.fire(e)},_handleLoadingChanged:function(e){this.loadingChanged.fire(e)},_handleProgressChanged:function(e){this.progressChanged.fire(e)},_handleFieldsPrepared:function(e){this._options.onFieldsPrepared&&this._options.onFieldsPrepared(e)},_createDataSource:function(e){var t,n=this,i=e.dataSource;return n._isSharedDataSource=i instanceof m.default,t=n._isSharedDataSource?i:new m.default(i),n._expandValueChangingHandler=n._handleExpandValueChanging.bind(n),n._loadingChangedHandler=n._handleLoadingChanged.bind(n),n._fieldsPreparedHandler=n._handleFieldsPrepared.bind(n),n._customizeStoreLoadOptionsHandler=n._handleCustomizeStoreLoadOptions.bind(n),n._changedHandler=function(){n._update(),n.dataSourceChanged.fire()},n._progressChangedHandler=function(e){n._handleProgressChanged(.8*e)},t.on("changed",n._changedHandler),t.on("expandValueChanging",n._expandValueChangingHandler),t.on("loadingChanged",n._loadingChangedHandler),t.on("progressChanged",n._progressChangedHandler),t.on("fieldsPrepared",n._fieldsPreparedHandler),t.on("customizeStoreLoadOptions",n._customizeStoreLoadOptionsHandler),t},getDataSource:function(){return this._dataSource},isLoading:function(){return this._dataSource.isLoading()},beginLoading:function(){this._dataSource.beginLoading()},endLoading:function(){this._dataSource.endLoading()},_update:function(){var e=this,t=e._dataSource,i=e._options,o=t.getAreaFields("column"),a=t.getAreaFields("row"),s=t.getAreaFields("data"),l="row"===i.dataFieldArea?s:[],u="row"!==i.dataFieldArea?s:[],c=t.getData(),d=T(s),h=O(s),p=E(s,o),g=!(s.length>0)||p.length!==s.length,_={isEmptyGrandTotal:c.isEmptyGrandTotalRow,texts:i.texts||{},hiddenTotals:d,hiddenValues:h,hiddenGrandTotals:[],showTotals:i.showRowTotals,showGrandTotals:i.showRowGrandTotals!==!1&&g,sortBySummaryPaths:n(o,s),showTotalsPrior:"rows"===i.showTotalsPrior||"both"===i.showTotalsPrior,showEmpty:!i.hideEmptySummaryCells,layout:i.rowHeaderLayout,fields:a,dataFields:s,progress:0},m={isEmptyGrandTotal:c.isEmptyGrandTotalColumn,texts:i.texts||{},hiddenTotals:d,hiddenValues:h,hiddenGrandTotals:p,showTotals:i.showColumnTotals,showTotalsPrior:"columns"===i.showTotalsPrior||"both"===i.showTotalsPrior,showGrandTotals:i.showColumnGrandTotals!==!1&&g,sortBySummaryPaths:n(a,s),showEmpty:!i.hideEmptySummaryCells,fields:o,dataFields:s,progress:0},v=function(t){this.progress=t,e._handleProgressChanged(.8+.1*_.progress+.1*m.progress)};_.notifyProgress=v,m.notifyProgress=v,(0,f.isDefined)(c.grandTotalRowIndex)||(c.grandTotalRowIndex=D(c.rows,_).length),(0,f.isDefined)(c.grandTotalColumnIndex)||(c.grandTotalColumnIndex=D(c.columns,m).length),t._changeLoadingCount(1),(0,r.when)(A(c.columns,o,u,!0,m),A(c.rows,a,l,!1,_)).always(function(){t._changeLoadingCount(-1)}).done(function(t,n){e._columnsInfo=t,e._rowsInfo=n,e._rowsScrollController&&e._columnsScrollController&&e.changed&&!e._dataSource.paginate()&&(e._rowsScrollController.reset(),e._columnsScrollController.reset(),e._lockChanged=!0,e._rowsScrollController.load(),e._columnsScrollController.load(),e._lockChanged=!1)}).done(function(){e._fireChanged(),e._stateStoringController.isEnabled()&&!e._dataSource.isLoading()&&(e._stateStoringController.state(e._dataSource.state()),e._stateStoringController.save())})},getRowsInfo:function(e){var t,n,o=this,a=o._rowsInfo,r=o._rowsScrollController;if(r&&!e){var l=r.beginPageIndex()*o.rowPageSize(),u=r.endPageIndex()*o.rowPageSize()+o.rowPageSize(),c=[],d=1;return i(a,function(e,i,o,a,r){var h=i>=l&&o<u,f=o<l?0:o-l,p=e;if(h)c[f]=c[f]||[],t=o<l?e.rowspan-(l-o)||1:e.rowspan,l+f+t>u&&(t=u-(f+l)||1),t!==e.rowspan&&(p=(0,s.extend)({},p,{rowspan:t})),c[f].push(p),d=y.max(d,r+1);else if(n>u)return!1}),i(c,function(e,t,n,i,o){var a=e.colspan||1;o+a>d&&(c[n][i]=(0,s.extend)({},e,{colspan:d-o||1}))}),c}return a},getColumnsInfo:function(e){var t=this,n=t._columnsInfo,i=t._columnsScrollController;if(i&&!e){var o=i.beginPageIndex()*t.columnPageSize(),a=i.endPageIndex()*t.columnPageSize()+t.columnPageSize();n=(0,g.createColumnsInfo)(n,o,a)}return n},totalRowCount:function(){return this._rowsInfo.length},rowPageIndex:function(e){return void 0!==e&&(this._rowPageIndex=e),this._rowPageIndex||0},totalColumnCount:function(){var e=0;if(this._columnsInfo&&this._columnsInfo.length)for(var t=0;t<this._columnsInfo[0].length;t++)e+=this._columnsInfo[0][t].colspan||1;return e},rowPageSize:function(e){return void 0!==e&&(this._rowPageSize=e),this._rowPageSize||20},columnPageSize:function(e){return void 0!==e&&(this._columnPageSize=e),this._columnPageSize||20},columnPageIndex:function(e){return void 0!==e&&(this._columnPageIndex=e),this._columnPageIndex||0},getCellsInfo:function(e){var t=this.getRowsInfo(e),n=this.getColumnsInfo(e),i=this._dataSource.getData(),o=this._options.texts||{};return c(t,n,i,this._dataSource.getAreaFields("data"),this._options.dataFieldArea,o.dataNotAvailable)},dispose:function(){var e=this;e._isSharedDataSource?(e._dataSource.off("changed",e._changedHandler),e._dataSource.off("expandValueChanging",e._expandValueChangingHandler),e._dataSource.off("loadingChanged",e._loadingChangedHandler),e._dataSource.off("progressChanged",e._progressChangedHandler),e._dataSource.off("fieldsPrepared",e._fieldsPreparedHandler),e._dataSource.off("customizeStoreLoadOptions",e._customizeStoreLoadOptionsHandler)):e._dataSource.dispose(),e._columnsScrollController&&e._columnsScrollController.dispose(),e._rowsScrollController&&e._rowsScrollController.dispose(),e._stateStoringController.dispose(),e.expandValueChanging.empty(),e.changed.empty(),e.loadingChanged.empty(),e.progressChanged.empty(),e.scrollChanged.empty(),e.dataSourceChanged.empty()}};return S(P,"applyPartialDataSource"),S(P,"collapseHeaderItem"),S(P,"expandHeaderItem"),S(P,"getData"),S(P,"isEmpty"),P}());t.DataController=D},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.LocalStore=void 0;var o=n(6),a=i(n(41)),r=i(n(47)),s=i(n(62)),l=n(69),u=i(n(123)),c=n(19),d=i(n(15)),h=n(4),f=n(1),p=n(3),g=n(89),_=i(n(81)),m="/./",v=d.default.inherit(function(){function e(e){return e.indexOf(".")!==-1?(0,c.compileGetter)(e):function(t){return t[e]}}function t(e){return function(t){var n=e(t);return!n||n instanceof Date||(n=s.default.deserializeDate(n)),n}}function n(i){(0,p.each)(i||[],function(i,o){var a,r,s,l,u=o.dataField,c=o.levels;if(!o.selector){if(l=u?e(u):function(e){return e},c&&n(c),"date"===o.dataType){r=M[o.groupInterval];var d=t(l);a=function(e){var t=d(e);return r?r(t):t}}else"number"===o.dataType?(s=(0,f.isNumeric)(o.groupInterval)&&o.groupInterval>0&&o.groupInterval,a=function(e){var t=l(e);return(0,f.isString)(t)&&(t=Number(t)),s?Math.floor(t/s)*s:t}):a=l;(0,g.setDefaultFieldValueFormatting)(o),(0,g.setFieldProperty)(o,"selector",a)}})}function i(e,t,n,o,a){var r,s,l=t.dimensions[o],u=t.expandedPathsHash;l&&(r=l.selector(t.data),a=void 0!==a?a+m+r:r+"",s=B(r,n,a,t.childrenHash),e.push(s.index),(u&&u[a]||l.expanded)&&(s.children||(s.children=[]),i(e,t,s.children,o+1,a)))}function d(e,t,n,o){var a=[0],r=t.headerName===o?t.path.length:0,s="rows"===o?t.rowExpandedPaths:t.columnExpandedPaths,l={data:e,childrenHash:n[o+"Hash"],dimensions:t[o],expandedPathsHash:t.headerName!==o&&s&&s.hash};return i(a,l,n[o],r),a}function v(e,t,n,i){var o,a,r,s,l=[],u=d(e,i,n,"rows"),c=d(e,i,n,"columns");for(a=0;a<u.length;a++)for(r=u[a],t[r]=t[r]||[],o=0;o<c.length;o++)s=c[o],l.push(t[r][s]=t[r][s]||[]);return l}function y(e){if(e){var t=e.hash={};e.forEach(function(e){var n=e.map(function(e){return e+""}).join(m);t[n]=!0})}}function b(e){e.rows=e.rows||[],e.columns=e.columns||[],e.filters=e.filters||[],y(e.columnExpandedPaths),y(e.rowExpandedPaths),n(e.columns),n(e.rows),n(e.values),n(e.filters)}function x(e){return"custom"===e.summaryType?(e.calculateCustomSummary=e.calculateCustomSummary||h.noop,{seed:function(){var t={summaryProcess:"start",totalValue:void 0};return e.calculateCustomSummary(t),t},step:function(t,n){return t.summaryProcess="calculate",t.value=n,e.calculateCustomSummary(t),t},finalize:function(t){return t.summaryProcess="finalize",delete t.value,e.calculateCustomSummary(t),t.totalValue}}):a.default.aggregators[e.summaryType]||a.default.aggregators.count}function w(e,t,n){for(var i=0;i<e.length;i++)for(var o=e[i],a=o.selector(n),r=x(o),s="function"==typeof r.seed,l=0;l<t.length;l++){var u=t[l];u.length<=i&&(u[i]=s?r.seed():r.seed),void 0===u[i]?u[i]=a:(0,f.isDefined)(a)&&(u[i]=r.step(u[i],a))}}function C(e,t){(0,p.each)(e,function(e,n){var i=x(n);i.finalize&&(0,p.each)(t,function(t,n){(0,p.each)(n,function(t,n){n&&void 0!==n[e]&&(n[e]=i.finalize(n[e]))})})})}function k(e,t){var n=e&&e.valueOf(),i=t&&t.valueOf();if(Array.isArray(e)){t=t||[];for(var o=0;o<e.length;o++)if(n=e[o]&&e[o].valueOf(),i=t[o]&&t[o].valueOf(),n!==i)return!1;return!0}return n===i}function S(e,t){var n=[];return(0,p.each)(e,function(e,i){n.push(i.selector(t))}),n}function D(e){var t=[];return(0,p.each)(e,function(e,n){var i=n.filterValues||[],o=n.groupName;if(!o||!(0,f.isNumeric)(n.groupIndex)){var a=function(e){for(var t=n.levels?S(n.levels,e):n.selector(e),o=!1,a=0;a<i.length;a++)if(k(i[a],t)){o=!0;break}return"exclude"===n.filterType?!o:o};i.length&&t.push(a)}}),t}function I(e){var t=D(e.rows).concat(D(e.columns)).concat(D(e.filters)),n=e[e.headerName],i=e.path;return n&&t.push(function(e){for(var t,o=0;o<i.length;o++)if(t=n[o].selector(e),(0,c.toComparable)(t,!0)!==(0,c.toComparable)(i[o],!0))return!1;return!0}),function(e){for(var n=0;n<t.length;n++)if(!t[n](e))return!1;return!0}}function T(e,t,n){function i(){for(var o=new Date,h=c;c<e.length;c++){if(c>h&&c%1e4===0&&new Date-o>=300)return n(c/e.length),void setTimeout(i,0);r=e[c],d(r)&&(a=v(r,l,s,t),w(t.values,a,r))}C(t.values,l),n(1),u.resolve({rows:s.rows,columns:s.columns,values:l,grandTotalRowIndex:0,grandTotalColumnIndex:0})}var a,r,s={columns:[],rows:[],columnsHash:{length:1},rowsHash:{length:1}},l=[],u=new o.Deferred,c=0,d=I(t);return i(),u}function O(e,t){var n=e.filter();return e.store()instanceof u.default&&n?(n=R(n,t),(0,r.default)(e.items()).filter(n).toArray()):e.items()}function E(e,t,n){var i=new o.Deferred,a=function(n){e.store()instanceof _.default&&(n.storeLoadOptions.filter=R(n.storeLoadOptions.filter,t))};if(e.on("customizeStoreLoadOptions",a),!e.isLoaded()||n){var r=n?e.load():e.reload();(0,o.when)(r).done(function(){E(e,t).done(function(){i.resolve(O(e,t))}).fail(i.reject)}).fail(i.reject)}else i.resolve(O(e,t));return i.always(function(){e.off("customizeStoreLoadOptions",a)})}function A(n,i){i.forEach(function(i){if(i.dataField&&"date"===i.dataType){var o=t(e(i.dataField));n[i.dataField]=function(e){return o(e)}}})}function P(e){var t={};return Array.isArray(e)?A(t,e):e&&["rows","columns","filters"].forEach(function(n){e[n]&&A(t,e[n])}),t}function R(e,t){if(!Array.isArray(e))return e;e=e.slice(0),(0,f.isString)(e[0])&&(e[1]instanceof Date||e[2]instanceof Date)&&(e[0]=t[e[0]]);for(var n=0;n<e.length;n++)e[n]=R(e[n],t);return e}var M={year:function(e){return e&&e.getFullYear()},quarter:function(e){return e&&Math.floor(e.getMonth()/3)+1},month:function(e){return e&&e.getMonth()+1},day:function(e){return e&&e.getDate()},dayOfWeek:function(e){return e&&e.getDay()}},B=function(e,t,n,i){var o=i[n];return o||(o={value:e,index:i.length++},i[n]=o,t.push(o)),o};return{ctor:function(e){this._progressChanged=e.onProgressChanged||h.noop,this._dataSource=new l.DataSource(e),this._dataSource.paginate(!1)},getFields:function(e){var t=this,n=t._dataSource,i=new o.Deferred;return E(n,P(e)).done(function(t){i.resolve((0,g.discoverObjectFields)(t,e))}).fail(i.reject),i},key:function(){return this._dataSource.key()},load:function(e){var t=this,n=t._dataSource,i=new o.Deferred;return b(e),E(n,P(e),e.reload).done(function(n){(0,o.when)(T(n,e,t._progressChanged)).done(i.resolve)}).fail(i.reject),i},filter:function(){var e=this._dataSource;return e.filter.apply(e,arguments)},supportPaging:function(){return!1},getDrillDownItems:function(e,t){e=e||{},t=t||{},b(e);for(var n,i=[],o=this._dataSource.items(),a=t.maxRowCount,r=t.customColumns,s=I(e),l=I({rows:(0,g.getFiltersByPath)(e.rows,t.rowPath),columns:(0,g.getFiltersByPath)(e.columns,t.columnPath),filters:[]}),u=0;u<o.length;u++){if(l(o[u])&&s(o[u])){if(r){n={};for(var c=0;c<r.length;c++)n[r[c]]=o[u][r[c]]}else n=o[u];i.push(n)}if(a>0&&i.length===a)break}return i}}}()).include(g.storeDrillDownMixin);t.LocalStore=v},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function c(e,t){var n=[];return(0,P.each)(e,function(i,o){n.push({selector:o.dataField,groupInterval:o.groupInterval,desc:t&&"desc"===o.sortOrder,isExpanded:i<e.length-1})}),n}function d(e){var t=e.dataField,n=e.groupInterval;return"date"===e.dataType&&"string"==typeof n&&("quarter"===n.toLowerCase()&&(n="Month"),t=t+"."+(0,B.capitalizeFirstLetter)(n)),t}function h(e,t,n,i){var o=[e,i?"<":">=",n],a=[e,i?">=":"<",n+t];return[o,i?"or":"and",a]}function f(e,t){var n=d(e),i="exclude"===e.filterType,o=[n,i?"<>":"=",t];return(0,O.isDefined)(e.groupInterval)&&("string"==typeof e.groupInterval&&"quarter"===e.groupInterval.toLowerCase()?o=h(n,3,3*(t-1)+1,i):"number"==typeof e.groupInterval&&"date"!==e.dataType&&(o=h(n,e.groupInterval,t,i))),o}function p(e,t){var n=[];return e.searchValue?[e.dataField,"contains",e.searchValue]:(t="exclude"===e.filterType?t||"and":t||"or",(0,P.each)(e.filterValues,function(i,o){var a=[];if(Array.isArray(o)){var r=e.levels&&e.levels.length;r&&(a=p({filterValues:o,filterType:e.filterType,levels:e.levels},"and"))}else{var s=e.levels?e.levels[i]:e;a=f(s,o)}a.length&&(n.length&&n.push(t),n.push(a))}),n)}function g(e){var t=[];return(0,P.each)(e,function(e,n){var i=p(n);return i.length?(t.length&&t.push("and"),void t.push(i)):[]}),1===t.length&&(t=t[0]),t}function _(e,t){var n,i=function(e){return e&&e.length};return n=i(e)&&i(t)?[e,"and",t]:i(e)?e:t}function m(e,t,n){var i=g(e.filters),o=c(e.rows,e.rowTake).concat(c(e.columns,e.columnTake)),a={groupSummary:[],totalSummary:[],group:o.length?o:void 0,take:o.length?void 0:1};return e.rows.length&&e.rowTake?(a.skip=e.rowSkip,a.take=e.rowTake,a.requireGroupCount=!0):e.columns.length&&e.columnTake&&!n&&(a.skip=e.columnSkip,a.take=e.columnTake,a.requireGroupCount=!0),t&&(i=_(i,t)),i.length&&(a.filter=i),(0,P.each)(e.values,function(t,n){var i={selector:n.dataField,summaryType:n.summaryType||"count"};a.groupSummary.push(i),e.includeTotalSummary&&a.totalSummary.push(i)}),a}function v(e,t,n,i,o){e[n]=e[n]||[],e[n][i]=e[n][i]||[],(0,O.isDefined)(e[n][i][o])||(e[n][i][o]=t)}function y(e,t){return t&&"number"===t.dataType&&(0,O.isString)(e)?Number(e):!t||"date"!==t.dataType||t.groupInterval||e instanceof Date?e:F.default.deserializeDate(e)}function b(e,t,n,i){function a(e,t,n,o,a){var r,s,l,u,c=i[t+"Hash"],d=n.slice(0,o+1).join("/");return void 0!==c[d]?l=c[d]:(l={value:y(e.key,a),index:i[t+"Index"]++},u=n.slice(0,o).join("/"),o>0&&void 0!==c[u]?(r=c[u],s=r.children=r.children||[]):s=i[t+"s"],s.push(l),c[d]=l),l}var r=[],s=[],l=i.rowHash,u=i.columnHash;if(t&&t.summary&&(0,P.each)(t.summary,function(e,t){v(i.values,t,i.grandTotalRowIndex,i.grandTotalColumnIndex,e)}),t&&t.groupCount>=0){var c=n.rows.length?n.rowSkip:n.columnSkip;e=o(Array(c)).concat(e),e.length=t.groupCount}return(0,V.forEachGroup)(e,function(e,t){var o,c,d=t>=n.rows.length?n.rows.length:t,h=t>=n.rows.length?t-n.rows.length:0;if(!(t>=n.rows.length&&h>=n.columns.length)){t<n.rows.length&&(s=[]),t>=n.rows.length?e?(s[h]=e.key+"",o=a(e,"column",s,h,n.columns[h]),c=l[r.slice(0,d+1).join("/")]):i.columns.push({}):e?(r[d]=e.key+"",c=a(e,"row",r,d,n.rows[d]),o=u[s.slice(0,h+1).join("/")]):i.rows.push({});var f=c&&c.index||i.grandTotalRowIndex,p=o&&o.index||i.grandTotalColumnIndex;(0,P.each)(e&&e.summary||[],function(e,t){v(i.values,t,f,p,e)})}}),i}function x(e){return(e||[]).filter(function(e){return e.filterValues&&e.filterValues.length||e.searchValue})}function w(e,t){if(e.headerName){if(t===e.headerName)return e.path.length;if(e.oppositePath)return e.oppositePath.length}return 0}function C(e){return(0,B.getFiltersByPath)(e[e.headerName],e.path).concat((0,B.getFiltersByPath)(e["rows"===e.headerName?"columns":"rows"],e.oppositePath||[]))}function k(e,t,n,i){var o=[],a=n>i?0:i,r=e.headerName!==t?e[t].slice(a,n):[],s="rows"===t?e.rowExpandedPaths:e.columnExpandedPaths;return(0,P.each)(r,function(e,t){var i=[];(0,P.each)(s,function(t,o){if(o=o.slice(a,n),e<o.length){var r=o[e];i.indexOf(r)===-1&&i.push(r)}}),i.length&&o.push((0,A.extend)({},t,{filterType:"include",filterValues:i}))}),o}function S(e,t,n,i,o,a){var r,s=("columns"===t?e.columnExpandedPaths:e.rowExpandedPaths)||[],l="columns"===t?"rows":"columns",u=e[t],c=[];if(s.length)for(var d=n;d<i+1;d++)r={filters:o.concat(k(e,t,d,a))},r[t]=u.slice(n,d+1),r[l]=[],c.push((0,A.extend)({},e,r));else r={filters:o},r[t]=u.slice(n,i+1),r[l]=[],c.push((0,A.extend)({},e,r));return c[0].includeTotalSummary=!0,c}function D(e){var t=0;return(0,P.each)(e,function(e,n){if(!n.expanded)return t=e,!1}),t}function I(e){var t=(0,B.getExpandedLevel)(e,"rows"),n=(0,B.getExpandedLevel)(e,"columns"),i=e.filters||[],o=w(e,"columns"),a=D(e.columns),r=D(e.rows),s=w(e,"rows"),l=[];i=i.concat(x(e.rows)).concat(x(e.columns)).concat(C(e));var u=S(e,"columns",o,n,i,a);if(e.rows.length&&e.columns.length){"rows"!==e.headerName&&(l=l.concat(u));for(var c=s;c<t+1;c++)for(var d=e.rows.slice(s,c+1),h=k(e,"rows",c,r),f=o;f<n+1;f++){var p=(0,A.extend)({},e,{columns:e.columns.slice(o,f+1),rows:d,filters:i.concat(k(e,"columns",f,a)).concat(h)});l.push(p)}}else l=e.columns.length?u:S(e,"rows",s,t,i,r);return l}function T(e){(0,P.each)(e||[],function(e,t){var n=t.levels;n&&T(n),(0,B.setDefaultFieldValueFormatting)(t)})}t.default=void 0;var O=n(1),E=i(n(15)),A=n(0),P=n(3),R=n(69),M=n(6),B=n(89),V=n(774),F=i(n(62)),L=E.default.inherit(function(){return{ctor:function(e){this._dataSource=new R.DataSource(e),this._store=this._dataSource.store()},getFields:function(e){var t=new M.Deferred;return this._store.load({skip:0,take:20}).done(function(n){t.resolve((0,B.discoverObjectFields)(n,e))}).fail(t.reject),t},key:function(){return this._store.key()},load:function(e){var t=this,n=new M.Deferred,i={rows:[],columns:[],values:[],grandTotalRowIndex:0,grandTotalColumnIndex:0,rowHash:{},columnHash:{},rowIndex:1,columnIndex:1},o=I(e),a=[];return T(e.rows),T(e.columns),T(e.filters),(0,P.each)(o,function(n,i){a.push(t._store.load(m(i,t.filter(),e.rows.length)))}),M.when.apply(null,a).done(function(){var e=a.length>1?arguments:[arguments];(0,P.each)(e,function(e,t){b(t[0],t[1],o[e],i)}),n.resolve({rows:i.rows,columns:i.columns,values:i.values,grandTotalRowIndex:i.grandTotalRowIndex,grandTotalColumnIndex:i.grandTotalColumnIndex})}).fail(n.reject),n},filter:function(){return this._dataSource.filter.apply(this._dataSource,arguments)},supportPaging:function(){return!1},createDrillDownDataSource:function(e,t){e=e||{},t=t||{};var n=this._store,i=(0,B.getFiltersByPath)(e.rows,t.rowPath).concat((0,B.getFiltersByPath)(e.columns,t.columnPath)).concat(x(e.rows)).concat(e.filters||[]).concat(x(e.columns)),o=g(i);return new R.DataSource({load:function(e){return n.load((0,A.extend)({},e,{filter:_(o,e.filter),select:t.customColumns}))}})}}}());t.default=L,e.exports=t.default},function(e,t,n){t.forEachGroup=void 0;var i=function e(t,n,i){t=t||[],i=i||0;for(var o=0;o<t.length;o++){var a=t[o];n(a,i),a&&a.items&&a.items.length&&e(a.items,n,i+1)}};t.forEachGroup=i},function(e,t,n){function i(e,t,n){o(t,e,"rows",n),o(t,e,"columns",n)}function o(e,t,n,i){var o=t[n]||[],r=t.headerName===n?t.path.length:0,s=[];(0,u.foreachDataLevel)(e[n],function(r,l){var u=o[l]||{},c=s[l]=s[l]||a(u,e,t,n,i);r.sort(c)},r)}function a(e,t,n,i,o){var a=o?"asc":e.sortOrder,l=s(e.sortBy,o),c=e.sortingMethod?function(t,n){return e.sortingMethod(t,n)}:(0,u.getCompareFunction)(function(e){return e[l]}),d=!o&&r(e,t,n,i),h=d&&(0,u.getCompareFunction)(d),f=function(e,t){var n=h&&h(e,t)||c(e,t);return"desc"===a?-n:n};return f}function r(e,t,n,i){var o=t.values,a=(0,u.findField)(n.values,e.sortBySummaryField),r="rows"===i,s=r?t.columns:t.rows,d=r?t.grandTotalRowIndex:t.grandTotalColumnIndex,h=e.sortBySummaryPath||[],f=h.length?l(s,h):d;if(o&&o.length&&a>=0&&(0,c.isDefined)(f))return function(e){var t=r?e.index:f,n=r?f:e.index,i=((o[t]||[[]])[n]||[])[a];return(0,c.isDefined)(i)?i:null}}function s(e,t){var n="text";return"none"===e?n="index":(t||"displayText"!==e)&&(n="value"),n}function l(e,t){var n=null,i=(t||[]).join(".");return i.length&&(0,u.foreachTree)(e,function(e){var t=e[0],o=(0,u.createPath)(e).join("."),a=(0,d.map)(e,function(e){return e.text}).reverse().join(".");if(i===o||t.key&&a===i)return n=e[0].index,!1}),n}t.sort=i;var u=n(89),c=n(1),d=n(3)},function(e,t,n){function i(){return{fields:{},positions:{}}}function o(e,t,n){var i={index:-1};if(!(0,d.isObject)(t))if(n.fields[t])t=n[t];else{var o=e.columns.concat(e.rows).concat(e.values),a=(0,p.findField)(o,t);t=n[t]=o[a]}if(t){var r=t.area||"data";i=n.positions[t.index]=n.positions[t.index]||{area:r,index:(0,f.inArray)(t,e["data"===r?"values":r+"s"])}}return i}function a(e){return e===_?"_rowPath":"_columnPath"}function r(e){var t=e.summaryDisplayMode,n=e.allowCrossGroupCalculation,i=m;return(0,d.isFunction)(e.calculateSummaryValue)?i=e.calculateSummaryValue:t&&(i="absoluteVariation"===t?x(n):"percentVariation"===t?w(n):C[t],i&&!e.format&&t.indexOf("percent")!==-1&&(0,p.setFieldProperty)(e,"format","percent")),i}function s(e,t,n,i){var o=e.values[t][n]=e.values[t][n]||[],a=o.originalCell;a&&(!o.allowResetting&&i||(e.values[t][n]=a.slice()),e.values[t][n].allowResetting=i)}function l(e,t){var n=[],o=[{index:t.grandTotalColumnIndex,children:t.columns}],a=[{index:t.grandTotalRowIndex,children:t.rows}],l=e.values,u=i();t.values=t.values||[],(0,p.foreachTree)(o,function(e){e[0].isEmpty=[]},!1),(0,p.foreachTree)(a,function(i){var a=i[0];a.isEmpty=[],t.values[a.index]=t.values[a.index]||[],(0,p.foreachTree)(o,function(o){var c,d=o[0];s(t,a.index,d.index,!1);for(var h=0;h<l.length;h++){var f=l[h],p=n[h]=void 0===n[h]?r(f):n[h];if(c=!1,p){var g=new D(o,i,t,e,h,u),_=g.cell(),m=_[h]=p(g);_.postProcessedFlags[h]=!0,c=null===m||void 0===m}void 0===d.isEmpty[h]&&(d.isEmpty[h]=!0),void 0===a.isEmpty[h]&&(a.isEmpty[h]=!0),c||(a.isEmpty[h]=d.isEmpty[h]=!1)}},!1)},!1),t.isEmptyGrandTotalRow=a[0].isEmpty,t.isEmptyGrandTotalColumn=o[0].isEmpty}function u(e,t){var n=[],o=[{index:t.grandTotalColumnIndex,children:t.columns}],a=[{index:t.grandTotalRowIndex,children:t.rows}],r=e.values,l=i();t.values=t.values||[],(0,p.foreachTree)(a,function(i){var a=i[0];t.values[a.index]=t.values[a.index]||[],(0,p.foreachTree)(o,function(o){var u=o[0];s(t,a.index,u.index,!0);for(var c=0;c<r.length;c++){var d=r[c],h=n[c]=void 0===n[c]?S(d):n[c];if(h){var f=new D(o,i,t,e,c,l),p=f.cell();p[c]=h(f),p.postProcessedFlags[c]=!0}}},!1)},!1)}function c(e,t,n){var i=new D([],[],{},e,0);return i.value=function(i){if((0,d.isDefined)(i)){var o=(0,p.findField)(t,i),a=t[o];n[o]||!a||(0,d.isDefined)(a.area)||(e.values.push(a),n[o]=!0)}},i.grandTotal=function(){return this},i.children=function(){return[]},i}t.applyDisplaySummaryMode=l,t.applyRunningTotal=u,t.createMockSummaryCell=c;var d=n(1),h=n(0),f=n(12),p=n(89),g="column",_="row",m=null,v=function(e,t){var n=e/t;return(0,d.isDefined)(e)&&!isNaN(n)||(n=m),n},y=function(e,t){return v(e.value(),e.grandTotal(t).value())},b=function(e,t){var n=e.parent(t),i=n?n.value():e.value();return v(e.value(),i)},x=function(e){return function(t){var n=t.prev(g,e),i=n&&n.value();return(0,d.isDefined)(i)&&(0,d.isDefined)(t.value())?t.value()-i:m}},w=function(e){var t=x(e);return function(n){var i=t(n),o=n.prev(g,e),a=o&&o.value();return i!==m&&a?i/a:m}},C={percentOfColumnTotal:function(e){return b(e,_)},percentOfRowTotal:function(e){return b(e,g)},percentOfColumnGrandTotal:function(e){return y(e,_)},percentOfRowGrandTotal:function(e){return y(e,g)},percentOfGrandTotal:function(e){return y(e)}},k=function e(t,n){if(t&&t.parent(n)){var i=t.prev(n);return i||(i=e(t.parent(n),n)),i}},S=function(e){if(e.runningTotal){var t=e.runningTotal===g?_:g;return function(n){var i=e.allowCrossGroupCalculation?k(n,t):n.prev(t,!1),o=n.value(!0),a=i&&i.value(!0);return(0,d.isDefined)(a)&&(0,d.isDefined)(o)?o=a+o:(0,d.isDefined)(a)&&(o=a),o}}},D=function(e,t,n,o,a,r){this._columnPath=e,this._rowPath=t,this._fieldIndex=a,this._fieldsCache=r||i(),this._data=n,this._descriptions=o;var s=n.values&&n.values[t[0].index]&&n.values[t[0].index][e[0].index];s&&(s.originalCell=s.originalCell||s.slice(),s.postProcessedFlags=s.postProcessedFlags||[],this._cell=s)};D.prototype=(0,h.extend)(D.prototype,{_getPath:function(e){return this[a(e)]},_getDimension:function(e){return e=e===_?"rows":"columns",this._descriptions[e]},_createCell:function(e){var t=this;return new D(e._columnPath||t._columnPath,e._rowPath||t._rowPath,t._data,t._descriptions,t._fieldIndex)},parent:function(e){var t=this._getPath(e).slice(),n={};return t.shift(),t.length?(n[a(e)]=t,this._createCell(n)):m},children:function(e){var t=this._getPath(e).slice(),n=t[0],i=[],o={};if(n.children)for(var r=0;r<n.children.length;r++)o[a(e)]=[n.children[r]].concat(t.slice()),i.push(this._createCell(o));return i},grandTotal:function(e){var t={},n=this._rowPath,i=this._columnPath,o=this._getPath(e),r=a(e);return e?t[r]=[o[o.length-1]]:(t._rowPath=[n[n.length-1]],t._columnPath=[i[i.length-1]]),this._createCell(t)},next:function(e,t){var n,i=this._getPath(e),o=i[0],a=this.parent(e);if(a){var r=(0,f.inArray)(o,i[1].children);if(n=a.children(e),n[r+1])return n[r+1]}if(t&&a){do a=a.next(e,t),n=a?a.children(e):[];while(a&&!n.length);return n[0]||m}return m},prev:function(e,t){var n,i=this._getPath(e),o=i[0],a=this.parent(e);if(a){var r=(0,f.inArray)(o,i[1].children);if(n=a.children(e),n[r-1])return n[r-1]}if(t&&a){do a=a.prev(e,t),n=a?a.children(e):[];while(a&&!n.length);return n[n.length-1]||m}return m},cell:function(){return this._cell},field:function e(t){if("data"===t)return this._descriptions.values[this._fieldIndex];var n=this._getPath(t),i=this._getDimension(t),e=i[n.length-2];return e||m},child:function(e,t){for(var n=this.children(e),i=0;i<n.length;i++){var o=o||n[i].field(e);if(n[i].value(o)===t)return n[i]}return m},slice:function(e,t){var n=this,i={},r=o(this._descriptions,e,this._fieldsCache),s=r.area,l=r.index,u=m,c=[];if(s===_||s===g){var d=this._getPath(s).slice(),h=l!==-1&&d.length-2-l;if(d[h]){c[d.length-1]=d[d.length-1];for(var f=h;f>=0;f--){if(d[f+1]){var p=d[f+1].children||[],v=f===h?t:d[f].value;d[f]=void 0;for(var y=0;y<p.length;y++)if(p[y].value===v){d[f]=p[y];break}}if(void 0===d[f])return u}i[a(s)]=d,u=n._createCell(i)}}return u},value:function(e,t){var n=this._cell,i=this._fieldIndex,a=e===!0||e===!1,r=a?m:e,s=a&&e||t;
if((0,d.isDefined)(r)){var l=o(this._descriptions,r,this._fieldsCache);if(i=l.index,"data"!==l.area){var u=this._getPath(l.area),c=i!==-1&&u.length-2-i;return u[c]&&u[c].value}}return n&&n.originalCell?s?n[i]:n.originalCell[i]:m},isPostProcessed:function(e){var t=this._fieldIndex;if((0,d.isDefined)(e)){var n=o(this._descriptions,e,this._fieldsCache);if(t=n.index,"data"!==n.area)return!1}return!(!this._cell||!this._cell.postProcessedFlags[t])}})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.DataArea=void 0;var o=i(n(2)),a=n(305),r="dx-pivotgrid-area",s="dx-pivotgrid-area-data",l="dx-total",u="dx-grandtotal",c="dx-row-total",d=a.AreaItem.inherit({_getAreaName:function(){return"data"},_createGroupElement:function(){return(0,o.default)("<div>").addClass(r).addClass(s)},_applyCustomStyles:function(e){var t=e.cell,n=e.classArray;"T"!==t.rowType&&"T"!==t.columnType||n.push(l),"GT"!==t.rowType&&"GT"!==t.columnType||n.push(u),"T"!==t.rowType&&"GT"!==t.rowType||n.push(c),e.rowIndex===e.rowsCount-1&&e.cssArray.push("border-bottom: 0px"),this.callBase(e)},_moveFakeTable:function(e){this._moveFakeTableHorizontally(e.x),this._moveFakeTableTop(e.y),this.callBase()},processScroll:function(e,t,n){var i="both";t&&!n?i="horizontal":!t&&n&&(i="vertical"),this._groupElement.css("borderTopWidth",0).dxScrollable({useNative:!!e,useSimulatedScrollbar:!e,direction:i,bounceEnabled:!1,updateManually:!0})},reset:function(){this.callBase(),this._virtualContent&&this._virtualContent.parent().css("height","auto")},setVirtualContentParams:function(e){this.callBase(e),this._virtualContent.parent().css("height",e.height),this._setTableCss({top:e.top,left:e.left})}});t.DataArea=d},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(t){var n=e.data().data,i=t.parentNode.rowIndex,o=t.cellIndex;return n[i]&&n[i][o]&&n[i][o].path}}t.VerticalHeadersArea=t.HorizontalHeadersArea=void 0;var a=i(n(2)),r=n(1),s=n(12),l=n(3),u=n(305),c="dx-pivotgrid-area",d="dx-pivotgrid-horizontal-headers",h="dx-pivotgrid-vertical-headers",f="dx-total",p="dx-grandtotal",g="dx-row-total",_="dx-pivotgrid-expanded",m="dx-pivotgrid-collapsed",v="dx-last-cell",y="dx-vertical-scroll",b="dx-expand-border",x=u.AreaItem.inherit({_getAreaName:function(){return"column"},_getAreaClassName:function(){return d},_createGroupElement:function(){return(0,a.default)("<div>").addClass(this._getAreaClassName()).addClass(c)},_applyCustomStyles:function(e){var t=e.cssArray,n=e.cell,i=e.rowsCount,o=e.classArray;e.cellIndex===e.cellsCount-1&&t.push((e.rtlEnabled?"border-left:":"border-right:")+"0px"),n.rowspan!==i-e.rowIndex&&e.rowIndex+1!==i||t.push("border-bottom-width:0px"),"T"!==n.type&&"GT"!==n.type||o.push(g),"T"===e.cell.type&&o.push(f),"GT"===e.cell.type&&o.push(p),(0,r.isDefined)(n.expanded)&&o.push(n.expanded?_:m),this.callBase(e)},_getMainElementMarkup:function(){return"<thead class='"+this._getAreaClassName()+"'>"},_getCloseMainElementMarkup:function(){return"</thead>"},setVirtualContentParams:function(e){this.callBase(e),this._setTableCss({left:e.left,top:0}),this._virtualContentWidth=e.width},hasScroll:function(){var e=this._virtualContent?this._virtualContentWidth:this._tableWidth;return!(!this._groupWidth||!e)&&e-this._groupWidth>=1},processScroll:function(){this._getScrollable()||this._groupElement.dxScrollable({useNative:!1,useSimulatedScrollbar:!1,showScrollbar:!1,bounceEnabled:!1,direction:"horizontal",updateManually:!0})},processScrollBarSpacing:function(e){var t=this,n=t.option("rtlEnabled")?"right":"left";t._groupWidth&&t.groupWidth(t._groupWidth-e),t._scrollBarWidth&&t._groupElement.next().remove(),t._groupElement.toggleClass(y,e>0),t._groupElement.css("float",n).width(t._groupHeight),t._scrollBarWidth=e},ctor:function(e){this.callBase(e),this._scrollBarWidth=0},getScrollPath:function(e){var t,n=this.tableElement();return e-=parseInt(n[0].style.left,10)||0,(0,l.each)(n.find("td"),function(n,i){if(1===i.colSpan&&i.offsetLeft<=e&&i.offsetWidth+i.offsetLeft>e)return t=i,!1}),o(n,t)},_moveFakeTable:function(e){this._moveFakeTableHorizontally(e),this.callBase()}});t.HorizontalHeadersArea=x;var w=x.inherit({_getAreaClassName:function(){return h},_applyCustomStyles:function(e){this.callBase(e),e.cellIndex===e.cellsCount-1&&e.classArray.push(v),e.rowIndex===e.rowsCount-1&&e.cssArray.push("border-bottom: 0px"),e.cell.isWhiteSpace&&e.classArray.push("dx-white-space-column")},_getAreaName:function(){return"row"},setVirtualContentParams:function(e){this.callBase(e),this._setTableCss({top:e.top,left:0}),this._virtualContentHeight=e.height},hasScroll:function(){var e=this._virtualContent?this._virtualContentHeight:this._tableHeight;return!(!this._groupHeight||!e)&&e-this._groupHeight>=1},processScroll:function(){this._getScrollable()||this._groupElement.dxScrollable({useNative:!1,useSimulatedScrollbar:!1,showScrollbar:!1,bounceEnabled:!1,direction:"vertical",updateManually:!0})},processScrollBarSpacing:function(e){var t=this;t._groupHeight&&t.groupHeight(t._groupHeight-e),t._scrollBarWidth&&t._groupElement.next().remove(),e&&t._groupElement.after((0,a.default)("<div>").width("100%").height(e-1)),t._scrollBarWidth=e},getScrollPath:function(e){var t,n=this.tableElement();return e-=parseInt(n[0].style.top,10)||0,(0,l.each)(n.find("tr"),function(n,i){var o=i.childNodes[i.childNodes.length-1];if(o&&1===o.rowSpan&&o.offsetTop<=e&&o.offsetHeight+o.offsetTop>e)return t=o,!1}),o(n,t)},_moveFakeTable:function(e){this._moveFakeTableTop(e),this.callBase()},_getRowClassNames:function(e,t,n){0!==e&t.expanded&&(0,s.inArray)(b,n)===-1&&n.push(b)},_getMainElementMarkup:function(){return"<tbody class='"+this._getAreaClassName()+"'>"},_getCloseMainElementMarkup:function(){return"</tbody>"},updateColspans:function(e){var t=this.tableElement()[0].rows,n=0,i=[];if(!(this.getColumnsCount()-e>0))for(var o=0;o<t.length;o++)for(var a=0;a<t[o].cells.length;a++){var r=t[o].cells[a],s=r.rowSpan;i[o]&&(n-=i[o],i[o]=0);var l=e-(n+r.colSpan);a===t[o].cells.length-1&&l>0&&(r.colSpan=r.colSpan+l),i[o+s]=(i[o+s]||0)+r.colSpan,n+=r.colSpan}}});t.VerticalHeadersArea=w},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){n&&n.groupName&&n.groupName===e.groupName&&(0,a.default)(h).addClass("dx-group-connector").addClass("dx-group-connector-prev").appendTo(i),t&&t.groupName&&t.groupName===e.groupName&&(0,a.default)(h).addClass("dx-group-connector").addClass("dx-group-connector-next").appendTo(i)}t.FieldsArea=void 0;var a=i(n(2)),r=n(4),s=n(3),l=n(305),u=n(89),c=i(n(48)),d=i(n(38));n(306);var h="<div>",f="dx-pivotgrid-drag-action",p=l.AreaItem.inherit({ctor:function(e,t){this.callBase(e),this._area=t},_getAreaName:function(){return"fields"},_createGroupElement:function(){return(0,a.default)(h).addClass("dx-pivotgrid-fields-area").addClass("dx-area-fields").addClass(f).attr("group",this._area)},isVisible:function(){return!!this.option("fieldPanel.visible")&&this.option("fieldPanel.show"+(0,u.capitalizeFirstLetter)(this._area)+"Fields")},_renderButton:function(e){var t=this,n=(0,a.default)("<td>").appendTo((0,a.default)("<tr>").appendTo(e)),i=t.component._createComponent((0,a.default)(h).appendTo(n),d.default,{text:"Fields",icon:"menu",width:"auto",onClick:function(){var e=t.tableElement().find(".dx-fields-area-popup").dxPopup("instance");e.option("visible")||e.show()}});i.$element().addClass("dx-pivotgrid-fields-area-hamburger")},_getPopupOptions:function(e,t){return{contentTemplate:function(){return(0,a.default)("<table>").addClass("dx-area-field-container").append((0,a.default)("<thead>").addClass("dx-pivotgrid-fields-area-head").append(e))},height:"auto",width:"auto",position:{at:"left",my:"left",of:t},dragEnabled:!1,animation:{show:{type:"pop",duration:200}},shading:!1,showTitle:!1,closeOnOutsideClick:!0,container:t.parent()}},_renderPopup:function(e,t){var n=this,i=e.find(".dx-button"),o=n._getPopupOptions(t,i),r=n.component.$element().dxPivotGridFieldChooserBase("instance");n._rowPopup&&n._rowPopup.$element().remove(),n._rowPopup=n.component._createComponent((0,a.default)(h).appendTo(e),c.default,o),n._rowPopup.$element().addClass("dx-fields-area-popup"),n._rowPopup.content().addClass("dx-pivotgrid-fields-container"),n._rowPopup.content().parent().attr("group","row"),r.subscribeToEvents(n._rowPopup.content()),r.renderSortable(n._rowPopup.content())},_shouldCreateButton:function(){return!1},_renderTableContent:function(e,t){var n=this,i=this.groupElement(),r=this.isVisible(),l=n.component.$element().dxPivotGridFieldChooserBase("instance"),u=(0,a.default)("<thead>").addClass("dx-pivotgrid-fields-area-head").appendTo(e),c=n._area,d=(0,a.default)("<tr>");i.toggleClass("dx-hidden",!r),e.addClass("dx-area-field-container"),r&&((0,s.each)(t,function(e,i){if(i.area===c&&i.visible!==!1){var r=(0,a.default)("<td>").append(l.renderField(i,"row"===i.area)),s=r.find(".dx-column-indicators");s.length&&n._shouldCreateButton()&&s.insertAfter(s.next()),r.appendTo(d),o(i,t[e+1],t[e-1],r)}}),d.children().length||(0,a.default)("<td>").append((0,a.default)(h).addClass("dx-empty-area-text").text(this.option("fieldPanel.texts."+c+"FieldArea"))).appendTo(d),n._shouldCreateButton()?(n._renderButton(u),n._renderPopup(e,d)):u.append(d))},reset:function(){this.callBase(),this.groupElement().css("marginTop",0)},_renderVirtualContent:r.noop});t.FieldsArea=p},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=(0,c.default)(e),o=i.offset();if(t>=o.left&&t<=o.left+i.outerWidth(!0)&&n>=o.top&&n<=o.top+i.outerHeight(!0))return!0}function a(e,t,n){return!(0,h.isDefined)(t.posHorizontal)||(n?e>t.posHorizontal:e<t.posHorizontal)}function r(e,t){var n=-1,i=t.get(0);return(0,p.each)(e,function(t,o){var a=(0,c.default)(o);if(a.attr("item-group")&&a.attr("item-group")===e.eq(t-1).attr("item-group")||n++,o===i)return!1}),n===e.length?-1:n}function s(e,t){var n;return(0,p.each)(t,function(){o(this,e.pageX,e.pageY)&&(n=(0,c.default)(this))}),n}function l(e,t,n){for(var i=[],o=[],a=0;a<e.length;a+=o.length)if(o=e.eq(a),o.attr("item-group")&&(o=n.find("[item-group='"+o.attr("item-group")+"']")),o.is(":visible")){var r={item:o,index:i.length,posHorizontal:t?void 0:(o.last().outerWidth(!0)+o.last().offset().left+o.offset().left)/2};t?r.posVertical=(o.last().offset().top+o.offset().top+o.last().outerHeight(!0))/2:r.posVertical=o.last().outerHeight(!0)+o.last().offset().top,i.push(r)}return i}function u(e){function t(e){r=e.scrollOffset.top}function n(){i(),e.scrollTo(r+=c),a=setTimeout(n,S)}function i(){clearTimeout(a)}function o(e){if(e.pageY<=l+k)c=-C;else{if(!(e.pageY>=l+u-k))return c=0,void i();c=C}n()}var a=null,r=e.scrollTop(),s=e.$element(),l=s.offset().top,u=s.height(),c=0;return e.on("scroll",t),{moveIfNeed:o,element:function(){return s},dispose:function(){i(),e.off("scroll",t)}}}t.default=void 0;var c=i(n(2)),d=i(n(5)),h=n(1),f=n(0),p=n(3),g=n(8),_=i(n(9)),m=i(n(65)),v=n(80),y=i(n(164)),b=y.default.getSwatchContainer,x="dxSortable",w="dx-sortable-old",C=2,k=20,S=10,D=m.default.inherit({_getDefaultOptions:function(){return(0,f.extend)(this.callBase(),{onChanged:null,onDragging:null,itemRender:null,groupSelector:null,itemSelector:".dx-sort-item",itemContainerSelector:".dx-sortable-old",sourceClass:"dx-drag-source",dragClass:"dx-drag",targetClass:"dx-drag-target",direction:"vertical",allowDragging:!0,groupFilter:null,useIndicator:!1})},_renderItem:function(e,t){var n,i=this.option("itemRender");return i?n=i(e,t):(n=e.clone(),n.css({width:e.width(),height:e.height()})),n},_renderIndicator:function(e,t,n,i){var o=e.outerHeight(!0),a=e.outerWidth(!0),r=e.offset().top-n.offset().top,s=e.offset().left-n.offset().left;this._indicator.css({position:"absolute",top:i&&t?r+o:r,left:i&&!t?s+a:s}).toggleClass("dx-position-indicator-horizontal",!t).toggleClass("dx-position-indicator-vertical",!!t).toggleClass("dx-position-indicator-last",!!i).height("").width("").appendTo(n),t?this._indicator.width(a):this._indicator.height(o)},_renderDraggable:function(e){this._$draggable&&this._$draggable.remove(),this._$draggable=this._renderItem(e,"drag").addClass(this.option("dragClass")).appendTo(b(e)).css({zIndex:1e6,position:"absolute"})},_detachEventHandlers:function(){var e=[v.move,v.start,v.end,v.enter,v.leave,v.drop].join(" ");d.default.off(this._getEventListener(),(0,g.addNamespace)(e,x))},_getItemOffset:function(e,t,n){for(var i=0;i<t.length;i++){var o=void 0,r=n.pageY<t[i].posVertical;if(e?o=r:r&&(o=a(n.pageX,t[i],this.option("rtlEnabled")),!o&&t[i+1]&&t[i+1].posVertical>t[i].posVertical&&(o=!0)),o)return t[i]}},_getEventListener:function(){var e=this.option("groupSelector"),t=this.$element();return e?t.find(e):t},_attachEventHandlers:function(){var e,t,n,i,o,a,h,f=this,_=f.option("itemSelector"),m=f.option("itemContainerSelector"),y=f.option("groupSelector"),b=f.option("sourceClass"),C=f.option("targetClass"),k=f.option("onDragging"),S=f.option("groupFilter"),D=f.$element(),I=null,T=-1,O=function(){o=[],(0,p.each)(e,function(e,t){o.push((0,c.default)(t).offset())})},E=function(){return y?S?(0,c.default)(y).filter(S):D.find(y):D},A=function(){I&&I.dispose(),I=null},P=function(){var o={sourceGroup:a,sourceIndex:t,sourceElement:e,targetGroup:i.attr("group"),targetIndex:i.find(_).index(n)};k&&k(o),o.cancel&&(i=void 0)};if(f._detachEventHandlers(),f.option("allowDragging")){var R=f._getEventListener();d.default.on(R,(0,g.addNamespace)(v.start,x),_,function(i){e=(0,c.default)(i.currentTarget);var o=e.closest(y);a=o.attr("group"),t=r((y?o:D).find(_),e),e.attr("item-group")&&(e=o.find("[item-group='"+e.attr("item-group")+"']")),f._renderDraggable(e),n=f._renderItem(e,"target").addClass(C),e.addClass(b),O(),h=E(),f._indicator=(0,c.default)("<div>").addClass("dx-position-indicator")}),d.default.on(R,(0,g.addNamespace)(v.move,x),function(r){var d,g,v;if(e){if(T=-1,f._indicator.detach(),(0,p.each)(f._$draggable,function(e,t){(0,c.default)(t).css({top:o[e].top+r.offset.y,left:o[e].left+r.offset.x})}),i&&i.removeClass(C),i=s(r,h),i&&P(),i&&I&&i.get(0)!==I.element().get(0)&&A(),I&&I.moveIfNeed(r),!i)return void n.detach();!I&&i.attr("allow-scrolling")&&(I=u(i.dxScrollable("instance"))),i.addClass(C);var y=i.find(m),x=y.find(_),k=i.closest("."+w).data("dxSortableOld"),S=k.option("useIndicator"),D="vertical"===(k||f).option("direction"),O=l(x,D,y),E=f._getItemOffset(D,O,r);if(E){if(d=E.item,v=O[E.index-1]&&O[E.index-1].item,d.hasClass(b)||v&&v.hasClass(b)&&v.is(":visible"))return void n.detach();if(T=E.index,!S)return void n.insertBefore(d);var R=i.attr("group")!==a,M=T===t,B=T===t+1;if(R)return void f._renderIndicator(d,D,i,f.option("rtlEnabled")&&!D);M||B||f._renderIndicator(d,D,i,f.option("rtlEnabled")&&!D)}else{if(g=x.last(),g.is(":visible")&&g.hasClass(b))return;y.length&&(T=O.length?O[O.length-1].index+1:0),S?x.length&&f._renderIndicator(g,D,i,!f.option("rtlEnabled")||D):n.appendTo(y)}}}),d.default.on(R,(0,g.addNamespace)(v.end,x),function(){if(A(),e){var o=f.option("onChanged"),r={sourceIndex:t,sourceElement:e,sourceGroup:a,targetIndex:T,removeSourceElement:!0,removeTargetElement:!1,removeSourceClass:!0};i&&(i.removeClass(C),r.targetGroup=i.attr("group"),(a!==r.targetGroup||T>-1)&&(o&&o(r),r.removeSourceElement&&e.remove())),f._indicator.detach(),r.removeSourceClass&&e.removeClass(b),e=null,f._$draggable.remove(),f._$draggable=null,r.removeTargetElement&&n.remove(),n.removeClass(C),n=null}})}},_init:function(){this.callBase(),this._attachEventHandlers()},_render:function(){this.callBase(),this.$element().addClass(w)},_dispose:function(){var e=this;e.callBase.apply(e,arguments),e._$draggable&&e._$draggable.detach(),e._indicator&&e._indicator.detach()},_optionChanged:function(e){var t=this;switch(e.name){case"onDragging":case"onChanged":case"itemRender":case"groupSelector":case"itemSelector":case"itemContainerSelector":case"sourceClass":case"targetClass":case"dragClass":case"allowDragging":case"groupFilter":case"useIndicator":t._attachEventHandlers();break;case"direction":break;default:t.callBase(e)}},_useTemplates:function(){return!1}});(0,_.default)("dxSortableOld",D);var I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){if(e&&e.customizeText)return"string";if(e.dataType)return e.dataType;if(e.format){if(1===d.default.parse(c.default.format(1,e.format)))return"number";if(c.default.format(new Date,e.format))return"date"}return g}t.DataProvider=t.ExportMixin=void 0;var a=i(n(15)),r=n(1),s=n(0),l=n(3),u=n(7),c=i(n(79)),d=i(n(45)),h=n(181),f=i(n(503)),p=n(6),g="string",_=0,m=1,v=2,y=100,b=(0,s.extend)({},f.default,{exportToExcel:function(){var e=this;(0,h.export)(e.getDataProvider(),{fileName:e.option("export.fileName"),proxyUrl:e.option("export.proxyUrl"),format:"EXCEL",rtlEnabled:e.option("rtlEnabled"),ignoreErrors:e.option("export.ignoreExcelErrors"),exportingAction:e._actions.onExporting,exportedAction:e._actions.onExported,fileSavingAction:e._actions.onFileSaving},h.excel.getData)},_getLength:function(e){var t,n=e[0].length,i=0;for(t=0;t<n;t++)i+=e[0][t].colspan||1;return i},_correctCellsInfoItemLengths:function(e,t){for(var n=0;n<e.length;n++)for(;e[n].length<t;)e[n].push({});return e},_calculateCellInfoItemLength:function(e){for(var t=0,n=0;n<e.length;n++)t+=(0,r.isDefined)(e[n].colspan)?e[n].colspan:1;return t},_getAllItems:function(e,t,n){var i,o,a=n,l=this._getLength(t),u=e.length;if(e.length>0&&e[0].length>0&&n.length>0&&0===n[0].length){var c=this._calculateCellInfoItemLength(e[0]);c>0&&(a=this._correctCellsInfoItemLengths(n,c))}var d=e.concat(a);for(o=0;o<t.length;o++)for(i=t[o].length-1;i>=0;i--)(0,r.isDefined)(d[o+u])||(d[o+u]=[]),d[o+u].splice(0,0,(0,s.extend)({},t[o][i]));return d[0].splice(0,0,(0,s.extend)({},this._getEmptyCell(),{alignment:this._options.rtlEnabled?"right":"left",colspan:l,rowspan:u})),this._prepareItems(d)},getDataProvider:function(){var e=this,t=this._dataController,n=new p.Deferred;return t.beginLoading(),setTimeout(function(){var i=(0,s.extend)(!0,[],t.getColumnsInfo(!0)),o=(0,s.extend)(!0,[],t.getRowsInfo(!0)),a=t.getCellsInfo(!0);n.resolve(e._getAllItems(i,o,a)),t.endLoading()}),new x({items:n,rtlEnabled:this.option("rtlEnabled"),dataFields:this.getDataSource().getAreaFields("data"),customizeExcelCell:this.option("export.customizeExcelCell"),rowsArea:this._rowsArea,columnsArea:this._columnsArea})}});t.ExportMixin=b;var x=a.default.inherit({ctor:function(e){this._options=e,this._styles=[]},ready:function(){var e=this,t=e._options,n=t.dataFields;return(0,p.when)(t.items).done(function(i){var a=i[0][0].rowspan,r=i[a-1],u={alignment:t.rtlEnabled?"left":"right"};e._styles=[{alignment:"center",dataType:"string"},{alignment:t.rtlEnabled?"right":"left",dataType:"string"}],n.length?n.forEach(function(t){e._styles.push((0,s.extend)({},u,{format:t.format,dataType:o(t)}))}):e._styles.push(u),(0,l.each)(r,function(e,t){t.width=y}),t.columns=r,t.items=i})},getColumns:function(){return this._options.columns},getColumnsWidths:function(){var e=this._options.columnsArea,t=this._options.rowsArea,n=this._options.columns,i=!(0,u.hasWindow)()||"virtual"===e.option("scrolling.mode")||e.element().is(":hidden");return i?n.map(function(e){return y}):t.getColumnsWidth().concat(e.getColumnsWidth())},getRowsCount:function(){return this._options.items.length},getGroupLevel:function(){return 0},getCellMerging:function(e,t){var n=this._options.items,i=n[e]&&n[e][t];return i?{colspan:i.colspan-1,rowspan:i.rowspan-1}:{colspan:0,rowspan:0}},getFrozenArea:function(){var e=this._options.items;return{x:e[0][0].colspan,y:e[0][0].rowspan}},getCellType:function(e,t){var n=this._styles[this.getStyleId(e,t)];return n&&n.dataType||"string"},getCellData:function(e,t,n){var i={},o=this._options.items,a=o[e]&&o[e][t]||{};if(n){i.cellSourceData=a;var r=this._tryGetAreaName(o,a,e,t);r&&(i.cellSourceData.area=r),i.cellSourceData.rowIndex=e,i.cellSourceData.columnIndex=t}return"string"===this.getCellType(e,t)?i.value=a.text:i.value=a.value,i.cellSourceData&&i.cellSourceData.isWhiteSpace&&(i.value=""),i},_tryGetAreaName:function(e,t,n,i){var o=e[0][0].rowspan,a=e[0][0].colspan;return i>=a&&n<o?"column":n>=o&&i<a?"row":(0,r.isDefined)(t.dataIndex)?"data":void 0},getStyles:function(){return this._styles},getStyleId:function(e,t){var n=this._options.items,i=n[0][0].rowspan,o=n[0][0].colspan,a=n[e]&&n[e][t]||{};return 0===t&&0===e?_:t>=o&&e<i?_:e>=i&&t<o?m:v+(a.dataIndex||0)},hasCustomizeExcelCell:function(){return(0,r.isDefined)(this._options.customizeExcelCell)},customizeExcelCell:function(e){this._options.customizeExcelCell&&this._options.customizeExcelCell(e)}});t.DataProvider=x},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=[],i=t.length-1;return(0,_.each)(e,function(e,o){n.push(o.text||(0,g.formatValue)(o.value,t[i-e]))}),n.reverse()}function a(e){var t=0;return(0,g.foreachTree)(e,function(e){t=Math.max(t,e.length-1)}),t}function r(e,t){var n=e.chartDataItem,i=t&&t(e);return i?(n=(0,p.extend)({},n,i.chartDataItem),i=(0,p.extend)({},e,i,{chartDataItem:n})):e}function s(e,t,n){function i(){var e,i=(x.values[d]||[])[m]||[],o=i[u],a={rowPath:p,maxRowLevel:T,rowPathFormatted:f,rowFields:k,columnPathFormatted:b,maxColumnLevel:O,columnPath:y,columnFields:S,dataFields:C,dataIndex:u,dataValues:i,visible:v&&h},s=(t.inverted?b:f).join(" - "),l=(t.inverted?f:b).join("/");C.length>1&&("args"!==t.putDataFieldsInto&&"both"!==t.putDataFieldsInto||(l+=" | "+c.caption),"args"!==t.putDataFieldsInto&&(s+=" | "+c.caption,"singleAxis"!==t.dataFieldsDisplayMode&&(e=c.caption))),a.chartDataItem={val:void 0===o?null:o,series:s,arg:l},a=r(a,t.processCell),a.visible&&(n[a.chartDataItem.series]=n[a.chartDataItem.series]||e,w.push(a.chartDataItem))}function s(e){(0,g.foreachTree)(I,function(n){d=n[0].index,n=n.slice(0,n.length-1),h=T===n.length,p=(0,g.createPath)(n),f=o(n,k),0===p.length&&(f=[t.grandTotalText]),(0,g.foreachTree)(D,function(n){m=n[0].index,n=n.slice(0,n.length-1),v=O===n.length,y=(0,g.createPath)(n),b=o(n,S),0===y.length&&(b=[t.grandTotalText]),e()})})}function l(e){(0,_.each)(C,function(t,n){c=n,u=t,e()})}var u,c,d,h,f,p,m,v,y,b,x=e.getData(),w=[],C=e.getAreaFields("data"),k=e.getAreaFields("row"),S=e.getAreaFields("column"),D=[{index:x.grandTotalColumnIndex,children:x.columns}],I=[{index:x.grandTotalRowIndex,children:x.rows}],T=a(I),O=a(D);return t.alternateDataFields===!1?l(function(){s(i)}):s(function(){l(i)}),w}function l(e,t){var n=e.getAreaFields("data");if("args"!==t.putDataFieldsInto&&"singleAxis"!==t.dataFieldsDisplayMode||1===n.length){var i=[];return(0,_.each)(n,function(e,n){var o={name:n.caption,title:n.caption,valueType:m[n.dataType]||n.dataType,label:{format:n.format}};n.customizeText&&(o.label.customizeText=function(e){return n.customizeText.call(n,e)}),"splitPanes"===t.dataFieldsDisplayMode&&(o.pane=n.caption),i.push(o)}),i}return[{}]}function u(e,t){var n=[],i=e.getAreaFields("data");return i.length>1&&"splitPanes"===t.dataFieldsDisplayMode&&"args"!==t.putDataFieldsInto&&(0,_.each)(i,function(e,t){n.push({name:t.caption})}),n.length||n.push({}),n}function c(e,t){var n=t.customizeSeries,i=t.customizeChart,o={valueAxis:l(e,t),panes:u(e,t)},a={};return i&&(o=(0,p.extend)(!0,{},o,i(o))),o.dataSource=s(e,t,a),o.seriesTemplate={nameField:"series",customizeSeries:function(e){var i={};return"splitPanes"===t.dataFieldsDisplayMode?i.pane=a[e]:"singleAxis"!==t.dataFieldsDisplayMode&&(i.axis=a[e]),n&&(i=(0,p.extend)(i,n(e,i))),i}},o}function d(e){if(!e)return!1;if(e.NAME)return"dxChart"===e.NAME&&e;var t=(0,f.default)(e);return t.data("dxChart")&&t.dxChart("instance")}function h(e){var t=e.$element().data(v);t&&t()}t.default=void 0;var f=i(n(2)),p=n(0),g=n(89),_=n(3),m={number:"numeric",date:"datetime"},v="dxPivotGridUnbinding",y={bindChart:function(e,t){t=(0,p.extend)({},t);var n=this,i=function(){t.grandTotalText=n.option("texts.grandTotal");var i=c(n.getDataSource(),t);e.option(i)};if(e=d(e),!e)return null;h(e),n.on("changed",i),i();var o=function(){e.$element().removeData(v),n.off("changed",i)};return e.on("disposing",o),this.on("disposing",o),e.$element().data(v,o),o}};t.default=y,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(509)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(508),o=i.XmlaStore;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(786)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function r(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?a(Object(n),!0).forEach(function(t){s(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):a(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function s(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function l(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function u(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function c(e,t,n){return t&&u(e.prototype,t),n&&u(e,n),e}function d(e,t,n){return(d="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=h(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function h(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=y(e),null!==e););return e}function f(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&p(e,t)}function p(e,t){return(p=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function g(e){var t=v();return function(){var n,i=y(e);if(t){var o=y(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return _(this,n)}}function _(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?m(e):t}function m(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function v(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function y(e){return(y=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var b=o(n(9)),x=o(n(28)),w=o(n(13)),C=o(n(2)),k=n(71),S=n(93),D=n(12),I=o(n(23)),T=o(n(27)),O=n(4),E=n(19),A=n(24),P=o(n(20)),R=o(n(62)),M=n(6),B=n(0),V=n(3),F=n(46),L=n(1),H=n(7),z=o(n(94)),N=n(55),j=o(n(32)),W=o(n(10)),G=n(280),q=n(36),$=o(n(17)),K=o(n(16)),U=o(n(787)),Y=n(803),Z=n(804),X=n(805),Q=n(516),J=o(n(806)),ee=o(n(809)),te=o(n(814)),ne=o(n(815)),ie=o(n(817)),oe=o(n(818)),ae=n(147),re=o(n(86)),se=o(n(819)),le=o(n(840)),ue=o(n(842)),ce=o(n(529)),de=o(n(843)),he=o(n(844)),fe=o(n(849)),pe=o(n(533)),ge=o(n(852)),_e=o(n(853)),me=n(854),ve=n(517),ye=n(855),be=o(n(522)),xe=60,we="dx-scheduler",Ce="".concat(we,"-small"),ke="".concat(we,"-adaptive"),Se="".concat(we,"-win-no-touch"),De="".concat(we,"-readonly"),Ie=400,Te="yyyyMMddTHHmmss",Oe=Te+"Z",Ee={day:{workSpace:he.default,renderingStrategy:"vertical"},week:{workSpace:pe.default,renderingStrategy:"vertical"},workWeek:{workSpace:ge.default,renderingStrategy:"vertical"},month:{workSpace:fe.default,renderingStrategy:"horizontalMonth"},timelineDay:{workSpace:le.default,renderingStrategy:"horizontal"},timelineWeek:{workSpace:ce.default,renderingStrategy:"horizontal"},timelineWorkWeek:{workSpace:de.default,renderingStrategy:"horizontal"},timelineMonth:{workSpace:ue.default,renderingStrategy:"horizontalMonthLine"},agenda:{workSpace:se.default,renderingStrategy:"agenda"}},Ae={ADDING:"onAppointmentAdding",ADDED:"onAppointmentAdded",DELETING:"onAppointmentDeleting",DELETED:"onAppointmentDeleted",UPDATING:"onAppointmentUpdating",UPDATED:"onAppointmentUpdated"},Pe=function(e){function t(){return l(this,t),n.apply(this,arguments)}f(t,e);var n=g(t);return c(t,[{key:"_getDefaultOptions",value:function(){var e=(0,B.extend)(d(y(t.prototype),"_getDefaultOptions",this).call(this),{views:["day","week"],currentView:"day",currentDate:P.default.trimTime(new Date),min:void 0,max:void 0,dateSerializationFormat:void 0,firstDayOfWeek:void 0,groups:[],resources:[],dataSource:null,customizeDateNavigatorText:void 0,appointmentTemplate:"item",dropDownAppointmentTemplate:"dropDownAppointment",appointmentCollectorTemplate:"appointmentCollector",dataCellTemplate:null,timeCellTemplate:null,resourceCellTemplate:null,dateCellTemplate:null,startDayHour:0,endDayHour:24,editing:{allowAdding:!0,allowDeleting:!0,allowDragging:!0,allowResizing:!0,allowUpdating:!0,allowTimeZoneEditing:!1,allowEditingTimeZones:!1},showAllDayPanel:!0,showCurrentTimeIndicator:!0,shadeUntilCurrentTime:!1,indicatorUpdateInterval:3e5,indicatorTime:void 0,recurrenceEditMode:"dialog",cellDuration:30,maxAppointmentsPerCell:"auto",selectedCellData:[],groupByDate:!1,onAppointmentRendered:null,onAppointmentClick:null,onAppointmentDblClick:null,onAppointmentContextMenu:null,onCellClick:null,onCellContextMenu:null,onAppointmentAdding:null,onAppointmentAdded:null,onAppointmentUpdating:null,onAppointmentUpdated:null,onAppointmentDeleting:null,onAppointmentDeleted:null,onAppointmentFormOpening:null,appointmentTooltipTemplate:"appointmentTooltip",appointmentPopupTemplate:"appointmentPopup",crossScrollingEnabled:!1,useDropDownViewSwitcher:!1,startDateExpr:"startDate",endDateExpr:"endDate",textExpr:"text",descriptionExpr:"description",allDayExpr:"allDay",recurrenceRuleExpr:"recurrenceRule",recurrenceExceptionExpr:"recurrenceException",disabledExpr:"disabled",remoteFiltering:!1,timeZone:"",startDateTimeZoneExpr:"startDateTimeZone",endDateTimeZoneExpr:"endDateTimeZone",noDataText:W.default.format("dxCollectionWidget-noDataText"),adaptivityEnabled:!1,allowMultipleCellSelection:!0,scrolling:{mode:"standard"},renovateRender:!1,_appointmentTooltipOffset:{x:0,y:0},_appointmentTooltipButtonsPosition:"bottom",_appointmentTooltipOpenButtonText:W.default.format("dxScheduler-openAppointment"),_dropDownButtonIcon:"overflow",_appointmentCountPerCell:2,_collectorOffset:0,_appointmentOffset:26});return(0,B.extend)(!0,e,{integrationOptions:{useDeferUpdateForTemplates:!1}})}},{key:"_defaultOptionsRules",value:function(){return d(y(t.prototype),"_defaultOptionsRules",this).call(this).concat([{device:function(){return"desktop"===w.default.real().deviceType&&!w.default.isSimulator()},options:{focusStateEnabled:!0}},{device:function(){return!w.default.current().generic},options:{useDropDownViewSwitcher:!0,editing:{allowDragging:!1,allowResizing:!1}}},{device:function(){return(0,q.isMaterial)()},options:{useDropDownViewSwitcher:!0,dateCellTemplate:function(e,t,n){var i=e.text;i.split(" ").forEach(function(e,t){var i=(0,C.default)("<span>").text(e).addClass("dx-scheduler-header-panel-cell-date");(0,C.default)(n).append(i),t||(0,C.default)(n).append(" ")})},_appointmentTooltipOffset:{x:0,y:11},_appointmentTooltipButtonsPosition:"top",_appointmentTooltipOpenButtonText:null,
_dropDownButtonIcon:"chevrondown",_appointmentCountPerCell:1,_collectorOffset:20,_appointmentOffset:30}}])}},{key:"_setDeprecatedOptions",value:function(){d(y(t.prototype),"_setDeprecatedOptions",this).call(this),(0,B.extend)(this._deprecatedOptions,{dropDownAppointmentTemplate:{since:"19.2",message:"appointmentTooltipTemplate"},allowEditingTimeZones:{since:"20.1",alias:"allowTimeZoneEditing"}})}},{key:"_getAppointmentSettingsGenerator",value:function(){return new ye.AppointmentSettingsGenerator(this)}},{key:"_postponeDataSourceLoading",value:function(e){this.postponedOperations.add("_reloadDataSource",this._reloadDataSource.bind(this),e)}},{key:"_postponeResourceLoading",value:function(){var e=this,t=this.postponedOperations.add("_loadResources",function(){return e._loadResources()}),n=new M.Deferred;return t.done(function(e){n.resolve(e)}),this._postponeDataSourceLoading(t),n.promise()}},{key:"_optionChanged",value:function(e){var n=this,i=e.value,o=e.name;switch(e.name){case"customizeDateNavigatorText":this._updateOption("header",o,i);break;case"firstDayOfWeek":this._updateOption("workSpace",o,i),this._updateOption("header",o,i);break;case"currentDate":i=this._dateOption(o),i=P.default.trimTime(new Date(i)),this.option("selectedCellData",[]),this._workSpace.option(o,new Date(i)),this._header.option(o,new Date(i)),this._header.option("displayedDate",this._workSpace._getViewStartByOptions()),this._appointments.option("items",[]),this._filterAppointmentsByDate(),this._postponeDataSourceLoading();break;case"dataSource":this._initDataSource(),this._appointmentModel.setDataSource(this._dataSource),this._postponeResourceLoading().done(function(e){n._filterAppointmentsByDate(),n._updateOption("workSpace","showAllDayPanel",n.option("showAllDayPanel"))});break;case"min":case"max":i=this._dateOption(o),this._updateOption("header",o,new Date(i)),this._updateOption("workSpace",o,new Date(i));break;case"views":this._processCurrentView(),this._getCurrentViewOptions()?this.repaint():this._header.option(o,i);break;case"useDropDownViewSwitcher":this._header.option(o,i);break;case"currentView":this._processCurrentView(),this.fire("validateDayHours"),this.getLayoutManager().initRenderingStrategy(this._getAppointmentsRenderingStrategy()),this._validateCellDuration(),this._appointments.option({items:[],allowDrag:this._allowDragging(),allowResize:this._allowResizing(),itemTemplate:this._getAppointmentTemplate("appointmentTemplate")}),this._postponeResourceLoading().done(function(e){n._refreshWorkSpace(e),n._updateHeader(),n._filterAppointmentsByDate(),n._appointments.option("allowAllDayResize","day"!==i)});break;case"appointmentTemplate":this._appointments.option("itemTemplate",i);break;case"dateCellTemplate":case"resourceCellTemplate":case"dataCellTemplate":case"timeCellTemplate":this._updateOption("workSpace",o,i),this.repaint();break;case"groups":this._postponeResourceLoading().done(function(e){n._refreshWorkSpace(e),n._filterAppointmentsByDate()});break;case"resources":this._resourcesManager.setResources(this.option("resources")),this._appointmentModel.setDataAccessors(this._combineDataAccessors()),this._postponeResourceLoading().done(function(e){n._appointments.option("items",[]),n._refreshWorkSpace(e),n._filterAppointmentsByDate()});break;case"startDayHour":case"endDayHour":this.fire("validateDayHours"),this._appointments.option("items",[]),this._updateOption("workSpace",o,i),this._appointments.repaint(),this._filterAppointmentsByDate(),this._postponeDataSourceLoading();break;case Ae.ADDING:case Ae.ADDED:case Ae.UPDATING:case Ae.UPDATED:case Ae.DELETING:case Ae.DELETED:case"onAppointmentFormOpening":this._actions[o]=this._createActionByOption(o);break;case"onAppointmentRendered":this._appointments.option("onItemRendered",this._getAppointmentRenderedAction());break;case"onAppointmentClick":this._appointments.option("onItemClick",this._createActionByOption(o));break;case"onAppointmentDblClick":this._appointments.option(o,this._createActionByOption(o));break;case"onAppointmentContextMenu":this._appointments.option("onItemContextMenu",this._createActionByOption(o));break;case"noDataText":case"allowMultipleCellSelection":case"selectedCellData":case"accessKey":case"onCellClick":this._workSpace.option(o,i);break;case"onCellContextMenu":this._workSpace.option(o,i);break;case"crossScrollingEnabled":this._postponeResourceLoading().done(function(e){n._appointments.option("items",[]),n._refreshWorkSpace(e),n._readyToRenderAppointments&&n._appointments.option("items",n._getAppointmentsToRepaint())});break;case"cellDuration":this._validateCellDuration(),this._appointments.option("items",[]),this._readyToRenderAppointments&&(this._updateOption("workSpace","hoursInterval",i/60),this._appointments.option("items",this._getAppointmentsToRepaint()));break;case"tabIndex":case"focusStateEnabled":this._updateOption("header",o,i),this._updateOption("workSpace",o,i),this._appointments.option(o,i),d(y(t.prototype),"_optionChanged",this).call(this,e);break;case"width":this._updateOption("header",o,i),this.option("crossScrollingEnabled")&&this._updateOption("workSpace","width",i),d(y(t.prototype),"_optionChanged",this).call(this,e),this._dimensionChanged();break;case"height":d(y(t.prototype),"_optionChanged",this).call(this,e),this._dimensionChanged();break;case"editing":this._initEditing();var a=this._editing;this._bringEditingModeToAppointments(a),this.hideAppointmentTooltip(),this._cleanPopup();break;case"showAllDayPanel":this._postponeResourceLoading().done(function(e){n._filterAppointmentsByDate(),n._updateOption("workSpace","allDayExpanded",i),n._updateOption("workSpace",o,i)});break;case"showCurrentTimeIndicator":case"indicatorTime":case"indicatorUpdateInterval":case"shadeUntilCurrentTime":case"groupByDate":this._updateOption("workSpace",o,i),this.repaint();break;case"appointmentDragging":case"appointmentTooltipTemplate":case"appointmentPopupTemplate":case"recurrenceEditMode":case"remoteFiltering":case"timeZone":case"dropDownAppointmentTemplate":case"appointmentCollectorTemplate":case"_appointmentTooltipOffset":case"_appointmentTooltipButtonsPosition":case"_appointmentTooltipOpenButtonText":case"_dropDownButtonIcon":case"_appointmentCountPerCell":case"_collectorOffset":case"_appointmentOffset":this.repaint();break;case"dateSerializationFormat":break;case"maxAppointmentsPerCell":break;case"startDateExpr":case"endDateExpr":case"startDateTimeZoneExpr":case"endDateTimeZoneExpr":case"textExpr":case"descriptionExpr":case"allDayExpr":case"recurrenceRuleExpr":case"recurrenceExceptionExpr":case"disabledExpr":this._updateExpression(o,i),this._appointmentModel.setDataAccessors(this._combineDataAccessors()),this._initAppointmentTemplate(),this.repaint();break;case"adaptivityEnabled":this._toggleAdaptiveClass(),this.repaint();break;case"scrolling":this._updateOption("workSpace",e.fullName,i);break;case"renovateRender":this._updateOption("workSpace",o,i);break;default:d(y(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_updateHeader",value:function(){var e=this._getViewCountConfig();this._header.option("intervalCount",e.intervalCount),this._header.option("displayedDate",this._workSpace._getViewStartByOptions()),this._header.option("min",this._dateOption("min")),this._header.option("max",this._dateOption("max")),this._header.option("currentDate",this._dateOption("currentDate")),this._header.option("firstDayOfWeek",this._getCurrentViewOption("firstDayOfWeek")),this._header.option("currentView",this._currentView)}},{key:"_dateOption",value:function(e){var t=this._getCurrentViewOption(e);return R.default.deserializeDate(t)}},{key:"_getSerializationFormat",value:function(e){var t=this._getCurrentViewOption(e);if("number"==typeof t)return"number";if((0,L.isString)(t))return R.default.getDateSerializationFormat(t)}},{key:"_bringEditingModeToAppointments",value:function(e){var t={allowDelete:e.allowUpdating&&e.allowDeleting};this._isAgenda()||(t.allowDrag=e.allowDragging,t.allowResize=e.allowResizing,t.allowAllDayResize=e.allowResizing&&this._supportAllDayResizing()),this._appointments.option(t),this.repaint()}},{key:"_isAgenda",value:function(){return"agenda"===this._getAppointmentsRenderingStrategy()}},{key:"_allowDragging",value:function(){return this._editing.allowDragging&&!this._isAgenda()}},{key:"_allowResizing",value:function(){return this._editing.allowResizing&&!this._isAgenda()}},{key:"_allowAllDayResizing",value:function(){return this._editing.allowResizing&&this._supportAllDayResizing()}},{key:"_supportAllDayResizing",value:function(){return"day"!==this._getCurrentViewType()||this._currentView.intervalCount>1}},{key:"_isAllDayExpanded",value:function(e){return this.option("showAllDayPanel")&&this._appointmentModel.hasAllDayAppointments(e,this._getCurrentViewOption("startDayHour"),this._getCurrentViewOption("endDayHour"))}},{key:"_getTimezoneOffsetByOption",value:function(e){return re.default.calculateTimezoneByValue(this.option("timeZone"),e)}},{key:"_filterAppointmentsByDate",value:function(){var e=this._workSpace.getDateRange();this._appointmentModel.filterByDate(e[0],e[1],this.option("remoteFiltering"),this.option("dateSerializationFormat"))}},{key:"_loadResources",value:function(){var e=this._getCurrentViewOption("groups"),t=new M.Deferred;return this._resourcesManager.loadResources(e).done(function(e){this._loadedResources=e,t.resolve(e)}.bind(this)),t.promise()}},{key:"_reloadDataSource",value:function(){var e=new M.Deferred;return this._dataSource?(this._dataSource.load().done(function(){(0,Q.hide)(),this._fireContentReadyAction(e)}.bind(this)).fail(function(){(0,Q.hide)(),e.reject()}),this._dataSource.isLoading()&&(0,Q.show)({container:this.$element(),position:{of:this.$element()}})):this._fireContentReadyAction(e),e.promise()}},{key:"_fireContentReadyAction",value:function(e){var n=d(y(t.prototype),"_fireContentReadyAction",this).bind(this),i=function(){n(),null===e||void 0===e?void 0:e.resolve()};if(this._workSpaceRecalculation){var o;null===(o=this._workSpaceRecalculation)||void 0===o?void 0:o.done(function(){i()})}else i()}},{key:"_dimensionChanged",value:function(){var e=this.getFilteredItems();if(this._toggleSmallClass(),!this._isAgenda()&&e&&this._isVisible()){this._workSpace._cleanAllowedPositions(),this._workSpace.option("allDayExpanded",this._isAllDayExpanded(e)),this._workSpace._dimensionChanged();var t=this._layoutManager.createAppointmentsMap(e);this._appointments.option("items",t)}this.hideAppointmentTooltip(),this._appointmentPopup.triggerResize(),this._appointmentPopup.updatePopupFullScreenMode()}},{key:"_clean",value:function(){this._cleanPopup(),d(y(t.prototype),"_clean",this).call(this)}},{key:"_toggleSmallClass",value:function(){var e=(0,A.getBoundingRect)(this.$element().get(0)).width;this.$element().toggleClass(Ce,e<Ie)}},{key:"_toggleAdaptiveClass",value:function(){this.$element().toggleClass(ke,this.option("adaptivityEnabled"))}},{key:"_visibilityChanged",value:function(e){e&&this._dimensionChanged()}},{key:"_dataSourceOptions",value:function(){return{paginate:!1}}},{key:"_init",value:function(){var e=this;this._initExpressions({startDate:this.option("startDateExpr"),endDate:this.option("endDateExpr"),startDateTimeZone:this.option("startDateTimeZoneExpr"),endDateTimeZone:this.option("endDateTimeZoneExpr"),allDay:this.option("allDayExpr"),text:this.option("textExpr"),description:this.option("descriptionExpr"),recurrenceRule:this.option("recurrenceRuleExpr"),recurrenceException:this.option("recurrenceExceptionExpr"),disabled:this.option("disabledExpr")}),d(y(t.prototype),"_init",this).call(this),this._initDataSource(),this._loadedResources=[],this.$element().addClass(we).toggleClass(Se,!(!I.default.msie||!F.touch)),this._initEditing(),this._resourcesManager=new ie.default(this.option("resources"));var n=this._combineDataAccessors();this._appointmentModel=new te.default(this._dataSource,n,this.getAppointmentDurationInMinutes()),this._initActions(),this._compactAppointmentsHelper=new Y.CompactAppointmentsHelper(this),this._asyncTemplatesTimers=[],this._dataSourceLoadedCallback=(0,T.default)(),this._subscribes=oe.default,this.timeZoneCalculator=new me.TimeZoneCalculator({getClientOffset:function(e){return re.default.getClientTimezoneOffset(e)},getCommonOffset:function(t,n){return re.default.calculateTimezoneByValue(n||e.option("timeZone"),t)},getAppointmentOffset:function(e,t){return re.default.calculateTimezoneByValue(t,e)}})}},{key:"_initTemplates",value:function(){this._initAppointmentTemplate(),this._templateManager.addDefaultTemplates({appointmentTooltip:new S.EmptyTemplate,dropDownAppointment:new S.EmptyTemplate}),d(y(t.prototype),"_initTemplates",this).call(this)}},{key:"_initAppointmentTemplate",value:function(){var e=this,t=this._dataAccessors.expr,n=function(e){return(0,E.compileGetter)("appointmentData.".concat(e))};this._templateManager.addDefaultTemplates(s({},"item",new k.BindableTemplate(function(t,n,i){e.getAppointmentsInstance()._renderAppointmentTemplate(t,n,i)},["html","text","startDate","endDate","allDay","description","recurrenceRule","recurrenceException","startDateTimeZone","endDateTimeZone"],this.option("integrationOptions.watchMethod"),{text:n(t.textExpr),startDate:n(t.startDateExpr),endDate:n(t.endDateExpr),startDateTimeZone:n(t.startDateTimeZoneExpr),endDateTimeZone:n(t.endDateTimeZoneExpr),allDay:n(t.allDayExpr),recurrenceRule:n(t.recurrenceRuleExpr)})))}},{key:"_combineDataAccessors",value:function(){var e=this._resourcesManager._dataAccessors,t=(0,B.extend)(!0,{},this._dataAccessors);return(0,V.each)(e,function(e,n){t[e].resources=n}.bind(this)),t}},{key:"_renderContent",value:function(){this._renderContentImpl()}},{key:"_dataSourceChangedHandler",value:function(e){this._readyToRenderAppointments&&this._workSpaceRecalculation.done(function(){this._renderAppointments(),this._isAgenda()&&(this._workSpace._renderView(),this._dataSourceLoadedCallback.fireWith(this,[this.getFilteredItems()]))}.bind(this))}},{key:"isVirtualScrolling",value:function(){var e;return null===(e=this.getWorkSpace())||void 0===e?void 0:e.isVirtualScrolling()}},{key:"_filterAppointments",value:function(){var e=this.isVirtualScrolling()?"prerenderFilterVirtual":"prerenderFilter";return this.fire(e)}},{key:"_renderAppointments",value:function(){this._filteredItems=this._filterAppointments(),this._workSpace.option("allDayExpanded",this._isAllDayExpanded(this._filteredItems)),this._isAgenda()&&this.getRenderingStrategyInstance().calculateRows(this._filteredItems,7,this.option("currentDate"),!0),this._filteredItems.length&&this._isVisible()?(this._appointments.option("items",this._getAppointmentsToRepaint()),this._appointmentModel.cleanModelState()):this._appointments.option("items",[])}},{key:"_getAppointmentsToRepaint",value:function(){var e=this._layoutManager.createAppointmentsMap(this._filteredItems);return this._layoutManager.getRepaintedAppointments(e,this.getAppointmentsInstance().option("items"))}},{key:"_initExpressions",value:function(e){var t=function(e){return"startDate"===e||"endDate"===e};this._dataAccessors||(this._dataAccessors={getter:{},setter:{},expr:{}}),(0,V.each)(e,function(e,n){if(n){var i,o,a=(0,E.compileGetter)(n),r=(0,E.compileSetter)(n);if(t(e)){var s=this;i=function(){var e=a.apply(this,arguments);if((0,x.default)().forceIsoDateParsing){if(!s.option("dateSerializationFormat")){var t=R.default.getDateSerializationFormat(e);t&&s.option("dateSerializationFormat",t)}e=R.default.deserializeDate(e)}return e},o=function(e,t){((0,x.default)().forceIsoDateParsing||s.option("dateSerializationFormat"))&&(t=R.default.serializeDate(t,s.option("dateSerializationFormat"))),r.call(this,e,t)}}this._dataAccessors.getter[e]=i||a,this._dataAccessors.setter[e]=o||r,this._dataAccessors.expr[e+"Expr"]=n}else delete this._dataAccessors.getter[e],delete this._dataAccessors.setter[e],delete this._dataAccessors.expr[e+"Expr"]}.bind(this))}},{key:"_updateExpression",value:function(e,t){var n={};n[e.replace("Expr","")]=t,this._initExpressions(n)}},{key:"_initEditing",value:function(){var e=this.option("editing");this._editing={allowAdding:!!e,allowUpdating:!!e,allowDeleting:!!e,allowResizing:!!e,allowDragging:!!e},(0,L.isObject)(e)&&(this._editing=(0,B.extend)(this._editing,e)),this._editing.allowDragging=this._editing.allowDragging&&this._editing.allowUpdating,this._editing.allowResizing=this._editing.allowResizing&&this._editing.allowUpdating,this.$element().toggleClass(De,this._isReadOnly())}},{key:"_isReadOnly",value:function(){var e=!0,t=this._editing;for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e=e&&!t[n]);return e}},{key:"_dispose",value:function(){this._appointmentTooltip&&this._appointmentTooltip.dispose(),this.hideAppointmentPopup(),this.hideAppointmentTooltip(),this._asyncTemplatesTimers.forEach(clearTimeout),this._asyncTemplatesTimers=[],d(y(t.prototype),"_dispose",this).call(this)}},{key:"_initActions",value:function(){this._actions={onAppointmentAdding:this._createActionByOption(Ae.ADDING),onAppointmentAdded:this._createActionByOption(Ae.ADDED),onAppointmentUpdating:this._createActionByOption(Ae.UPDATING),onAppointmentUpdated:this._createActionByOption(Ae.UPDATED),onAppointmentDeleting:this._createActionByOption(Ae.DELETING),onAppointmentDeleted:this._createActionByOption(Ae.DELETED),onAppointmentFormOpening:this._createActionByOption("onAppointmentFormOpening")}}},{key:"_getAppointmentRenderedAction",value:function(){return this._createActionByOption("onAppointmentRendered",{excludeValidators:["disabled","readOnly"]})}},{key:"_renderFocusTarget",value:function(){return(0,O.noop)()}},{key:"_initMarkup",value:function(){d(y(t.prototype),"_initMarkup",this).call(this),this.fire("validateDayHours"),this._validateCellDuration(),this._processCurrentView(),this._renderHeader(),this._layoutManager=new ee.default(this,this._getAppointmentsRenderingStrategy()),this._appointments=this._createComponent("<div>",J.default,this._appointmentsConfig()),this._appointments.option("itemTemplate",this._getAppointmentTemplate("appointmentTemplate")),this._appointmentTooltip=new(this.option("adaptivityEnabled")?X.MobileTooltipStrategy:Z.DesktopTooltipStrategy)(this._getAppointmentTooltipOptions()),this._appointmentPopup=new U.default(this),this._isLoaded()||this._isDataSourceLoading()?(this._initMarkupCore(this._loadedResources),this._dataSourceChangedHandler(this._dataSource.items()),this._fireContentReadyAction()):this._loadResources().done(function(e){this._initMarkupCore(e),this._reloadDataSource()}.bind(this))}},{key:"_getAppointmentTooltipOptions",value:function(){var e=this,t=this;return{createComponent:t._createComponent.bind(t),container:t.$element(),getScrollableContainer:t.getWorkSpaceScrollableContainer.bind(t),addDefaultTemplates:t._templateManager.addDefaultTemplates.bind(t._templateManager),getAppointmentTemplate:t._getAppointmentTemplate.bind(t),showAppointmentPopup:t.showAppointmentPopup.bind(t),checkAndDeleteAppointment:t.checkAndDeleteAppointment.bind(t),isAppointmentInAllDayPanel:t.isAppointmentInAllDayPanel.bind(t),createFormattedDateText:function(t,n,i){return e.fire("getTextAndFormatDate",t,n,i)},getAppointmentDisabled:function(t){return e.createAppointmentAdapter(t).disabled}}}},{key:"checkAndDeleteAppointment",value:function(e,t){var n=this,i=this.createAppointmentAdapter(t);this._checkRecurringAppointment(e,t,i.startDate,function(){n.deleteAppointment(e)},!0)}},{key:"_getExtraAppointmentTooltipOptions",value:function(){return{rtlEnabled:this.option("rtlEnabled"),focusStateEnabled:this.option("focusStateEnabled"),editing:this.option("editing"),offset:this.option("_appointmentTooltipOffset")}}},{key:"isAppointmentInAllDayPanel",value:function(e){var t=this._workSpace,n=this.appointmentTakesAllDay(e);return n&&t.supportAllDayRow()&&t.option("showAllDayPanel")}},{key:"_initMarkupCore",value:function(e){var t=this;this._readyToRenderAppointments=(0,H.hasWindow)(),this._workSpace&&this._cleanWorkspace(),this._renderWorkSpace(e),this._appointments.option({fixedContainer:this._workSpace.getFixedContainer(),allDayContainer:this._workSpace.getAllDayContainer()}),this._waitAsyncTemplate(function(){var e;return null===(e=t._workSpaceRecalculation)||void 0===e?void 0:e.resolve()}),this._filterAppointmentsByDate()}},{key:"_isLoaded",value:function(){return this._isResourcesLoaded()&&this._isDataSourceLoaded()}},{key:"_isResourcesLoaded",value:function(){return(0,L.isDefined)(this._loadedResources)}},{key:"_isDataSourceLoaded",value:function(){return this._dataSource&&this._dataSource.isLoaded()}},{key:"_render",value:function(){this._toggleSmallClass(),this._toggleAdaptiveClass(),d(y(t.prototype),"_render",this).call(this)}},{key:"_renderHeader",value:function(){var e=(0,C.default)("<div>").appendTo(this.$element());this._header=this._createComponent(e,ne.default,this._headerConfig())}},{key:"_headerConfig",value:function(){var e=this,t=this._getCurrentViewOptions(),n=this._getViewCountConfig(),i=(0,B.extend)({isAdaptive:this.option("adaptivityEnabled"),firstDayOfWeek:this.option("firstDayOfWeek"),currentView:this._currentView,tabIndex:this.option("tabIndex"),focusStateEnabled:this.option("focusStateEnabled"),width:this.option("width"),rtlEnabled:this.option("rtlEnabled"),useDropDownViewSwitcher:this.option("useDropDownViewSwitcher"),_dropDownButtonIcon:this.option("_dropDownButtonIcon"),customizeDateNavigatorText:this.option("customizeDateNavigatorText")},t);return i.observer=this,i.intervalCount=n.intervalCount,i.views=this.option("views"),i.min=new Date(this._dateOption("min")),i.max=new Date(this._dateOption("max")),i.currentDate=P.default.trimTime(new Date(this._dateOption("currentDate"))),i.todayDate=function(){var t=e.timeZoneCalculator.createDate(new Date,{path:"toGrid"});return t},i}},{key:"_appointmentsConfig",value:function(){var e=this,t={observer:this,onItemRendered:this._getAppointmentRenderedAction(),onItemClick:this._createActionByOption("onAppointmentClick"),onItemContextMenu:this._createActionByOption("onAppointmentContextMenu"),onAppointmentDblClick:this._createActionByOption("onAppointmentDblClick"),tabIndex:this.option("tabIndex"),focusStateEnabled:this.option("focusStateEnabled"),allowDrag:this._allowDragging(),allowDelete:this._editing.allowUpdating&&this._editing.allowDeleting,allowResize:this._allowResizing(),allowAllDayResize:this._allowAllDayResizing(),rtlEnabled:this.option("rtlEnabled"),currentView:this.option("currentView"),onContentReady:function(){e._workSpace&&e._workSpace.option("allDayExpanded",e._isAllDayExpanded(e.getFilteredItems()))}};return t}},{key:"getCollectorOffset",value:function(){return this._workSpace.needApplyCollectorOffset()&&!this.option("adaptivityEnabled")?this.option("_collectorOffset"):0}},{key:"getAppointmentDurationInMinutes",value:function(){return this._getCurrentViewOption("cellDuration")}},{key:"_processCurrentView",value:function(){var e=this.option("views"),t=this.option("currentView"),n=this;this._currentView=t,(0,V.each)(e,function(e,i){var o=(0,L.isObject)(i),a=o?i.name:i,r=i.type;if(t===a||t===r)return n._currentView=i,!1})}},{key:"_validateCellDuration",value:function(){var e=this._getCurrentViewOption("endDayHour"),t=this._getCurrentViewOption("startDayHour"),n=this._getCurrentViewOption("cellDuration");(e-t)*xe%n!==0&&$.default.log("W1015")}},{key:"_getCurrentViewType",value:function(){return this._currentView.type||this._currentView}},{key:"_getAppointmentsRenderingStrategy",value:function(){return Ee[this._getCurrentViewType()].renderingStrategy}},{key:"_renderWorkSpace",value:function(e){this._readyToRenderAppointments&&this._toggleSmallClass();var t=(0,C.default)("<div>").appendTo(this.$element()),n=this._getViewCountConfig(),i=Ee[this._getCurrentViewType()].workSpace,o=this._workSpaceConfig(e,n);this._workSpace=this._createComponent(t,i,o),this._allowDragging()&&this._workSpace.initDragBehavior(this,this._all),this._workSpace._attachTablesEvents(),this._workSpace.getWorkArea().append(this._appointments.$element()),this._recalculateWorkspace(),n.startDate&&this._header&&this._header.option("currentDate",this._workSpace._getHeaderDate()),this._appointments.option("_collectorOffset",this.getCollectorOffset())}},{key:"_getViewCountConfig",value:function(){var e=this.option("currentView"),t=this._getViewByName(e),n=t&&t.intervalCount||1,i=t&&t.startDate||null;return{intervalCount:n,startDate:i}}},{key:"_getViewByName",value:function(e){for(var t=this.option("views"),n=0;n<t.length;n++)if(t[n].name===e||t[n].type===e||t[n]===e)return t[n]}},{key:"_recalculateWorkspace",value:function(){var e=this;this._workSpaceRecalculation=new M.Deferred,this._waitAsyncTemplate(function(){(0,N.triggerResizeEvent)(e._workSpace.$element()),e._workSpace._refreshDateTimeIndication()})}},{key:"_workSpaceConfig",value:function(e,t){var n,i=this,o=this._getCurrentViewOptions(),a=this.option("scrolling"),r=(0,B.extend)({noDataText:this.option("noDataText"),firstDayOfWeek:this.option("firstDayOfWeek"),startDayHour:this.option("startDayHour"),endDayHour:this.option("endDayHour"),tabIndex:this.option("tabIndex"),accessKey:this.option("accessKey"),focusStateEnabled:this.option("focusStateEnabled"),cellDuration:this.option("cellDuration"),showAllDayPanel:this.option("showAllDayPanel"),showCurrentTimeIndicator:this.option("showCurrentTimeIndicator"),indicatorTime:this.option("indicatorTime"),indicatorUpdateInterval:this.option("indicatorUpdateInterval"),shadeUntilCurrentTime:this.option("shadeUntilCurrentTime"),allDayExpanded:this._appointments.option("items"),crossScrollingEnabled:this.option("crossScrollingEnabled"),dataCellTemplate:this.option("dataCellTemplate"),timeCellTemplate:this.option("timeCellTemplate"),resourceCellTemplate:this.option("resourceCellTemplate"),dateCellTemplate:this.option("dateCellTemplate"),allowMultipleCellSelection:this.option("allowMultipleCellSelection"),selectedCellData:this.option("selectedCellData"),onSelectionChanged:function(e){i.option("selectedCellData",e.selectedCellData)},groupByDate:this._getCurrentViewOption("groupByDate"),scrolling:a,renovateRender:this.option("renovateRender")||"virtual"===a.mode||"virtual"===(null===(n=o.scrolling)||void 0===n?void 0:n.mode)},o);return r.observer=this,r.intervalCount=t.intervalCount,r.startDate=t.startDate,r.groups=e,r.onCellClick=this._createActionByOption("onCellClick"),r.onCellContextMenu=this._createActionByOption("onCellContextMenu"),r.min=new Date(this._dateOption("min")),r.max=new Date(this._dateOption("max")),r.currentDate=P.default.trimTime(new Date(this._dateOption("currentDate"))),r.hoursInterval=r.cellDuration/60,r.allDayExpanded=this._isAllDayExpanded(this.getFilteredItems()),r.dataCellTemplate=r.dataCellTemplate?this._getTemplate(r.dataCellTemplate):null,r.timeCellTemplate=r.timeCellTemplate?this._getTemplate(r.timeCellTemplate):null,r.resourceCellTemplate=r.resourceCellTemplate?this._getTemplate(r.resourceCellTemplate):null,r.dateCellTemplate=r.dateCellTemplate?this._getTemplate(r.dateCellTemplate):null,r}},{key:"_waitAsyncTemplate",value:function(e){if(this._options.silent("templatesRenderAsynchronously")){var t=setTimeout(function(){e(),clearTimeout(t)});this._asyncTemplatesTimers.push(t)}else e()}},{key:"_getCurrentViewOptions",value:function(){return this._currentView}},{key:"_getCurrentViewOption",value:function(e){var t=this._getCurrentViewOptions();return t&&void 0!==t[e]?t[e]:this.option(e)}},{key:"_getAppointmentTemplate",value:function(e){var t=this._getCurrentViewOptions();return t&&t[e]?this._getTemplate(t[e]):this._getTemplateByOption(e)}},{key:"_updateOption",value:function(e,t,n){var i=this._getCurrentViewOptions();i&&(0,L.isDefined)(i[t])||this["_"+e].option(t,n)}},{key:"_refreshWorkSpace",value:function(e){var t=this;this._cleanWorkspace(),delete this._workSpace,this._renderWorkSpace(e),this._readyToRenderAppointments&&(this._appointments.option({fixedContainer:this._workSpace.getFixedContainer(),allDayContainer:this._workSpace.getAllDayContainer()}),this._waitAsyncTemplate(function(){return t._workSpaceRecalculation.resolve()}))}},{key:"_cleanWorkspace",value:function(){this._appointments.$element().detach(),this._workSpace._dispose(),this._workSpace.$element().remove(),this.option("selectedCellData",[])}},{key:"getWorkSpaceScrollable",value:function(){return this._workSpace.getScrollable()}},{key:"getWorkSpaceScrollableScrollTop",value:function(e){return this._workSpace.getGroupedScrollableScrollTop(e)}},{key:"getWorkSpaceScrollableScrollLeft",value:function(){return this._workSpace.getScrollableScrollLeft()}},{key:"getWorkSpaceScrollableContainer",value:function(){return this._workSpace.getScrollableContainer()}},{key:"getWorkSpaceAllDayHeight",value:function(){return this._workSpace.getAllDayHeight()}},{key:"getWorkSpaceAllDayOffset",value:function(){return this._workSpace.getAllDayOffset()}},{key:"getWorkSpaceHeaderPanelHeight",value:function(){return this._workSpace.getHeaderPanelHeight()}},{key:"getWorkSpaceDateTableOffset",value:function(){return!this.option("crossScrollingEnabled")||this.option("rtlEnabled")?this._workSpace.getWorkSpaceLeftOffset():0}},{key:"getWorkSpace",value:function(){return this._workSpace}},{key:"getAppointmentModel",value:function(){return this._appointmentModel}},{key:"getHeader",value:function(){return this._header}},{key:"getMaxAppointmentsPerCell",value:function(){return this._getCurrentViewOption("maxAppointmentsPerCell")}},{key:"_cleanPopup",value:function(){this._appointmentPopup&&this._appointmentPopup.dispose()}},{key:"_checkRecurringAppointment",value:function(e,t,n,i,o,a,r){var s=this;delete this._updatedRecAppointment;var l=this.fire("getField","recurrenceRule",e);if(!(0,ae.getRecurrenceProcessor)().evalRecurrenceRule(l).isValid||!this._editing.allowUpdating)return void i();var u=this.option("recurrenceEditMode");switch(u){case"series":i();break;case"occurrence":this._excludeAppointmentFromSeries(e,t,n,o,a,r);break;default:r&&(r.cancel=new M.Deferred),this._showRecurrenceChangeConfirm(o).done(function(l){l&&i(),!l&&s._excludeAppointmentFromSeries(e,t,n,o,a,r)}).fail(function(){return s._appointments.moveAppointmentBack(r)})}}},{key:"_excludeAppointmentFromSeries",value:function(e,t,n,i,o,a){var s=this,l=this.createAppointmentAdapter(r({},e)),u=this.createAppointmentAdapter(t);u.recurrenceRule="",u.recurrenceException="";var c=!i&&!o;if(c){var d=this._appointmentModel.keyName;delete t[d],this.addAppointment(t)}l.recurrenceException=this._createRecurrenceException(l,n),o?(this._updatedRecAppointment=l.source(),this._appointmentPopup.show(t,!0),this._editAppointmentData=e):this._updateAppointment(e,l.source(),function(){s._appointments.moveAppointmentBack(a)},a)}},{key:"_createRecurrenceException",value:function(e,t){var n=[];return e.recurrenceException&&n.push(e.recurrenceException),n.push(this._getSerializedDate(t,e.startDate,e.allDay)),n.join()}},{key:"_getSerializedDate",value:function(e,t,n){return n&&e.setHours(t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds()),R.default.serializeDate(e,Oe)}},{key:"_showRecurrenceChangeConfirm",value:function(e){var t=W.default.format(e?"dxScheduler-confirmRecurrenceDeleteMessage":"dxScheduler-confirmRecurrenceEditMessage"),n=W.default.format(e?"dxScheduler-confirmRecurrenceDeleteSeries":"dxScheduler-confirmRecurrenceEditSeries"),i=W.default.format(e?"dxScheduler-confirmRecurrenceDeleteOccurrence":"dxScheduler-confirmRecurrenceEditOccurrence");return(0,G.custom)({messageHtml:t,showCloseButton:!0,showTitle:!0,buttons:[{text:n,onClick:function(){return!0}},{text:i,onClick:function(){return!1}}]}).show()}},{key:"_getUpdatedData",value:function(e){var t=this,n=function(e){return e?t.timeZoneCalculator.createDate(e,{path:"fromGrid"}):void 0},i=function(e){return!isNaN(new Date(e).getTime())},o=this.getTargetCellData(),a=this.createAppointmentAdapter(e),r=n(o.startDate),s=n(o.endDate),l=new Date(a.startDate),u=new Date(a.endDate),c=r||l;
i(l)||(l=c),i(u)||(u=s);var d=u.getTime()-l.getTime(),h=this._workSpace.keepOriginalHours()&&i(a.startDate)&&i(r);if(h){var f=P.default.trimTime,p=this.timeZoneCalculator.createDate(a.startDate,{path:"toGrid"}),g=p.getTime()-f(p).getTime();c=new Date(f(o.startDate).getTime()+g),c=this.timeZoneCalculator.createDate(c,{path:"fromGrid"})}var _=this.createAppointmentAdapter({});void 0!==o.allDay&&(_.allDay=o.allDay),_.startDate=c;var m=new Date(c.getTime()+d);if(this.appointmentTakesAllDay(e)&&!_.allDay&&this._workSpace.supportAllDayRow()&&(m=this._workSpace.calculateEndDate(c)),a.allDay&&!this._workSpace.supportAllDayRow()&&!this._workSpace.keepOriginalHours()){var v=new Date(c);v.setHours(0),m=new Date(v.getTime()+d),0!==m.getHours()&&m.setHours(this._getCurrentViewOption("endDayHour"))}var y=re.default.getTimezoneOffsetChangeInMs(l,u,c,m);_.endDate=new Date(m.getTime()-y);var b=_.source();return this._resourcesManager.setResourcesToItem(b,o.groups),b}},{key:"getTargetedAppointment",value:function(e,t){var n=be.default.dataAccessors.getAppointmentSettings(t),i=be.default.dataAccessors.getAppointmentInfo(t),o=(0,C.default)(t).data(this._appointments._itemIndexKey()),a=this.createAppointmentAdapter(e),r=a.clone();if(this._isAgenda()&&a.isRecurrent){var s=this.getRenderingStrategyInstance().getAppointmentDataCalculator(),l=s((0,C.default)(t),a.startDate).startDate;r.startDate=l,r.endDate=new Date(l.getTime()+a.duration)}else n&&(r.startDate=i?i.sourceAppointment.startDate:a.startDate,r.endDate=i?i.sourceAppointment.endDate:a.endDate);var u=r.source();return t&&this.setTargetedAppointmentResources(u,t,o),u}},{key:"subscribe",value:function(e,t){this._subscribes[e]=oe.default[e]=t}},{key:"fire",value:function(e){var t=this._subscribes[e],n=Array.prototype.slice.call(arguments);if(!(0,L.isFunction)(t))throw $.default.Error("E1031",e);return t.apply(this,n.slice(1))}},{key:"getTargetCellData",value:function(){return this._workSpace.getDataByDroppableCell()}},{key:"_updateAppointment",value:function(e,t,n,i){var o={newData:t,oldData:(0,B.extend)({},e),cancel:!1},a=function(e){if(n&&n.call(this),e&&"Error"===e.name)throw e}.bind(this);return this._actions[Ae.UPDATING](o),i&&!(0,L.isDeferred)(i.cancel)&&(i.cancel=new M.Deferred),this._processActionResult(o,function(n){var o=this,r=new M.Deferred;if(n)a(),r.resolve();else{this._expandAllDayPanel(t);try{r=this._appointmentModel.update(e,t).done(function(){i&&i.cancel.resolve(!1)}).always(function(e){return o._onDataPromiseCompleted(Ae.UPDATED,t,e)}).fail(function(){return a()})}catch(e){a(e),r.resolve()}}return r.promise()})}},{key:"_processActionResult",value:function(e,t){var n=this,i=new M.Deferred,o=function(e){(0,M.when)((0,M.fromPromise)(e)).always(i.resolve)};return(0,L.isPromise)(e.cancel)?(0,M.when)((0,M.fromPromise)(e.cancel)).always(function(i){(0,L.isDefined)(i)||(i="rejected"===e.cancel.state()),o(t.call(n,i))}):o(t.call(this,e.cancel)),i.promise()}},{key:"_expandAllDayPanel",value:function(e){!this._isAllDayExpanded(this.getFilteredItems())&&this.appointmentTakesAllDay(e)&&this._workSpace.option("allDayExpanded",!0)}},{key:"_onDataPromiseCompleted",value:function(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]&&arguments[3],o={appointmentData:t};n instanceof Error?o.error=n:(i||(o.appointmentData=n[0]||n),this._appointmentPopup.isVisible()&&this._appointmentPopup.hide()),this._actions[e](o),this._fireContentReadyAction()}},{key:"getAppointmentPopup",value:function(){return this._appointmentPopup.getPopup()}},{key:"getUpdatedAppointment",value:function(){return this._appointmentModel.getUpdatedAppointment()}},{key:"getUpdatedAppointmentKeys",value:function(){return this._appointmentModel.getUpdatedAppointmentKeys()}},{key:"getAppointmentsInstance",value:function(){return this._appointments}},{key:"getResourceManager",value:function(){return this._resourcesManager}},{key:"getLayoutManager",value:function(){return this._layoutManager}},{key:"getRenderingStrategyInstance",value:function(){return this._layoutManager.getRenderingStrategyInstance()}},{key:"getFilteredItems",value:function(){return this._filteredItems}},{key:"getActions",value:function(){return this._actions}},{key:"appointmentTakesAllDay",value:function(e){return this._appointmentModel.appointmentTakesAllDay(e,this._getCurrentViewOption("startDayHour"),this._getCurrentViewOption("endDayHour"))}},{key:"_getRecurrenceException",value:function(e){var t=this.createAppointmentAdapter(e),n=t.recurrenceException;if(n){for(var i=n.split(","),o=0;o<i.length;o++)i[o]=this._convertRecurrenceException(i[o],t.startDate);return i.join()}return n}},{key:"_convertRecurrenceException",value:function(e,t){var n=this;e=e.replace(/\s/g,"");var i=function(e){return n.timeZoneCalculator.createDate(e,{path:"toGrid"})},o=R.default.deserializeDate(e),a=i(t),r=i(o);return r=re.default.correctRecurrenceExceptionByTimezone(r,a,this.option("timeZone")),e=R.default.serializeDate(r,Te)}},{key:"dayHasAppointment",value:function(e,t,n){var i=this,o=function(e){return i.timeZoneCalculator.createDate(e,{path:"toGrid"})},a=this.createAppointmentAdapter(t),r=new Date(a.startDate),s=new Date(a.endDate);if(r=o(r),s=o(s),e.getTime()===s.getTime())return r.getTime()===s.getTime();n&&(e=P.default.trimTime(e),r=P.default.trimTime(r),s=P.default.trimTime(s));var l=e.getTime(),u=r.getTime(),c=s.getTime();return(0,D.inArray)(l,[u,c])>-1||u<l&&c>l}},{key:"setTargetedAppointmentResources",value:function(e,t,n){var i=this._getCurrentViewOption("groups");if(null!==i&&void 0!==i&&i.length){var o,a,r=this._resourcesManager._dataAccessors.setter,s=this._workSpace;this._isAgenda()?(o=function(){var e=this.getLayoutManager()._positionMap[n];return s._getCellGroups(e[0].groupIndex)},a=function(t,n){r[n.name](e,n.id)}):(o=function(){var e=be.default.dataAccessors.getAppointmentSettings(t)||{};return s.getCellDataByCoordinates({left:e.left,top:e.top}).groups},a=function(t,n){r[t](e,n)}),(0,V.each)(o.call(this),a)}}},{key:"getStartViewDate",value:function(){return this._workSpace.getStartViewDate()}},{key:"getEndViewDate",value:function(){return this._workSpace.getEndViewDate()}},{key:"showAppointmentPopup",value:function(e,t,n){var i=this,o=this.createAppointmentAdapter(n||e),a=(0,B.extend)({},e,n);this._checkRecurringAppointment(e,a,o.startDate,function(){t||(0,L.isEmptyObject)(e)?(delete i._editAppointmentData,i._editing.allowAdding&&i._appointmentPopup.show(e,!0)):(i._editAppointmentData=e,i._appointmentPopup.show(e,i._editing.allowUpdating))},!1,!0)}},{key:"hideAppointmentPopup",value:function(e){this._appointmentPopup&&this._appointmentPopup.isVisible()&&(e&&this._appointmentPopup.saveChanges(),this._appointmentPopup.hide())}},{key:"showAppointmentTooltip",value:function(e,t,n){if(e){var i=new ve.AppointmentTooltipInfo(e,n,this._appointments._tryGetAppointmentColor(t));this.showAppointmentTooltipCore(t,[i])}}},{key:"showAppointmentTooltipCore",value:function(e,t,n){this._appointmentTooltip.isAlreadyShown(e)?this.hideAppointmentTooltip():this._appointmentTooltip.show(e,t,(0,B.extend)(this._getExtraAppointmentTooltipOptions(),n))}},{key:"hideAppointmentTooltip",value:function(){this._appointmentTooltip&&this._appointmentTooltip.hide()}},{key:"scrollToTime",value:function(e,t,n){this._workSpace.scrollToTime(e,t,n)}},{key:"scrollTo",value:function(e,t,n){this._workSpace.scrollTo(e,t,n)}},{key:"addAppointment",value:function(e){var t=this,n=this.createAppointmentAdapter(e);n.text=n.text||"";var i=n.source(!0),o={appointmentData:i,cancel:!1};return this._actions[Ae.ADDING](o),this._processActionResult(o,function(e){return e?(new M.Deferred).resolve():(t._expandAllDayPanel(i),t._appointmentModel.add(i).always(function(e){return t._onDataPromiseCompleted(Ae.ADDED,i,e)}))})}},{key:"updateAppointment",value:function(e,t){return this._updateAppointment(e,t)}},{key:"deleteAppointment",value:function(e){var t={appointmentData:e,cancel:!1};this._actions[Ae.DELETING](t),this._processActionResult(t,function(t){var n=this;t||this._appointmentModel.remove(e).always(function(t){return n._onDataPromiseCompleted(Ae.DELETED,e,t,!0)})})}},{key:"focus",value:function(){this._editAppointmentData?this._appointments.focus():this._workSpace.focus()}},{key:"getFirstDayOfWeek",value:function(){return(0,L.isDefined)(this.option("firstDayOfWeek"))?this.option("firstDayOfWeek"):j.default.firstDayOfWeekIndex()}},{key:"createAppointmentAdapter",value:function(e){var t=this,n={getField:function(e,n){return t.fire("getField",n,e)},setField:function(e,n,i){return t.fire("setField",n,e,i)},getTimeZoneCalculator:function(){return t.timeZoneCalculator}};return new _e.default(e,n)}}]),t}(K.default);Pe.include(z.default),(0,b.default)("dxScheduler",Pe);var Re=Pe;t.default=Re,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(13)),l=i(n(2)),u=i(n(20)),c=n(6),d=n(0),h=n(3),f=n(1),p=n(7),g=n(55),_=i(n(10)),m=i(n(48)),v=n(788),y=n(516),b=u.default.dateToMilliseconds,x="dx-scheduler",w="".concat(x,"-appointment-popup"),C=485,k=970,S=1e3,D=500,I=350,T="after",O="before",E=b("day"),A=function(){function e(t){o(this,e),this.scheduler=t,this._popup=null,this._appointmentForm=null,this.state={lastEditData:null,saveChangesLocker:!1,appointment:{data:null,isEmptyText:!1,isEmptyDescription:!1}}}return r(e,[{key:"show",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1?arguments[1]:void 0;if((0,f.isEmptyObject)(e)){var n=this.scheduler.option("currentDate"),i=new Date(n.getTime()+this.scheduler.option("cellDuration")*b("minute"));this.scheduler.fire("setField","startDate",e,n),this.scheduler.fire("setField","endDate",e,i)}if(this.state.appointment.data=e,!this._popup){var o=this._createPopupConfig();this._popup=this._createPopup(o)}this._popup.option("toolbarItems",this._createPopupToolbarItems(t)),this._popup.show()}},{key:"hide",value:function(){this._popup.hide()}},{key:"isVisible",value:function(){return!!this._popup&&this._popup.option("visible")}},{key:"dispose",value:function(){this._$popup&&(this._popup.$element().remove(),this._$popup=null)}},{key:"_createPopup",value:function(e){var t=(0,l.default)("<div>").addClass(w).appendTo(this.scheduler.$element());return this.scheduler._createComponent(t,m.default,e)}},{key:"_createPopupConfig",value:function(){var e=this;return{height:"auto",maxHeight:"100%",showCloseButton:!1,showTitle:!1,onHiding:function(){e.scheduler.focus()},contentTemplate:function(){return e._createPopupContent()},onShowing:function(t){return e._onShowing(t)},defaultOptionsRules:[{device:function(){return s.default.current().android},options:{showTitle:!1}}]}}},{key:"_onShowing",value:function(e){var t=this;this._updateForm();var n={form:this._appointmentForm,popup:this._popup,appointmentData:this.state.appointment.data,cancel:!1};this.scheduler._actions.onAppointmentFormOpening(n),this.scheduler._processActionResult(n,function(n){n?e.cancel=!0:t.updatePopupFullScreenMode()})}},{key:"_createPopupContent",value:function(){var e=(0,l.default)("<div>");return this._appointmentForm=this._createForm(e),e}},{key:"_createAppointmentFormData",value:function(e){var t=this._createAppointmentAdapter(e),n=(0,d.extend)(!0,{repeat:!!t.recurrenceRule},e);return(0,h.each)(this.scheduler._resourcesManager.getResourcesFromItem(n,!0)||{},function(e,t){return n[e]=t}),n}},{key:"_createForm",value:function(e){var t=this.scheduler._dataAccessors.expr,n=this.scheduler.option("resources"),i=this._getAllowTimeZoneEditing(),o=this.state.appointment.data,a=this._createAppointmentFormData(o),r=this._isReadOnly(o);return v.AppointmentForm.prepareAppointmentFormEditors(t,this.scheduler,this.triggerResize.bind(this),this.changeSize.bind(this),a,i,r),n&&n.length&&v.AppointmentForm.concatResources(this.scheduler._resourcesManager.getEditors()),v.AppointmentForm.create(this.scheduler._createComponent.bind(this.scheduler),e,r,a)}},{key:"_getAllowTimeZoneEditing",value:function(){var e=this.scheduler;return e.option("editing.allowTimeZoneEditing")||e.option("editing.allowEditingTimeZones")}},{key:"_isReadOnly",value:function(e){var t=this.scheduler.createAppointmentAdapter(e);return!(!e||!t.disabled)||!!this.scheduler._editAppointmentData&&!this.scheduler._editing.allowUpdating}},{key:"_createAppointmentAdapter",value:function(e){return this.scheduler.createAppointmentAdapter(e)}},{key:"_updateForm",value:function(){var e=this.state.appointment.data,t=this._createAppointmentAdapter(e),n=t.allDay,i=t.startDate&&t.calculateStartDate("toAppointment"),o=t.endDate&&t.calculateEndDate("toAppointment");this.state.appointment.isEmptyText=void 0===e||void 0===t.text,this.state.appointment.isEmptyDescription=void 0===e||void 0===t.description;var a=this._createAppointmentAdapter(this._createAppointmentFormData(e));void 0===a.text&&(a.text=""),void 0===a.description&&(a.description=""),void 0===a.recurrenceRule&&(a.recurrenceRule="");var r=a.source();i&&this.scheduler.fire("setField","startDate",r,i),o&&this.scheduler.fire("setField","endDate",r,o);var s=this.scheduler._dataAccessors.expr,l=s.startDateExpr,u=s.endDateExpr;this._appointmentForm.option("readOnly",this._isReadOnly(e)),v.AppointmentForm.updateFormData(this._appointmentForm,r,this.scheduler._dataAccessors.expr),v.AppointmentForm.setEditorsType(this._appointmentForm,l,u,n)}},{key:"_isDeviceMobile",value:function(){return"desktop"!==s.default.current().deviceType}},{key:"_isPopupFullScreenNeeded",value:function(){var e=this._tryGetWindowWidth();return!!e&&(this._isDeviceMobile()?e<D:e<S)}},{key:"_tryGetWindowWidth",value:function(){if((0,p.hasWindow)()){var e=(0,p.getWindow)();return(0,l.default)(e).width()}}},{key:"triggerResize",value:function(){this._popup&&(0,g.triggerResizeEvent)(this._popup.$element())}},{key:"_getMaxWidth",value:function(e){return this._isDeviceMobile()?I:e?k:C}},{key:"changeSize",value:function(e){var t=this._isPopupFullScreenNeeded();this._popup.option({maxWidth:t?"100%":this._getMaxWidth(e),fullScreen:t})}},{key:"updatePopupFullScreenMode",value:function(){if(this._appointmentForm){var e=v.AppointmentForm.getRecurrenceRule(this._appointmentForm.option("formData"),this.scheduler._dataAccessors.expr);this.isVisible()&&this.changeSize(e)}}},{key:"_createPopupToolbarItems",value:function(e){var t=this,n=[],i="ios"===s.default.current().platform;return e&&n.push({shortcut:"done",options:{text:_.default.format("Done")},location:T,onClick:function(e){return t._doneButtonClickHandler(e)}}),n.push({shortcut:"cancel",location:i?O:T}),n}},{key:"saveChanges",value:function(e){var t=this,n=new c.Deferred,i=this._appointmentForm.validate(),o=this.state.appointment;return e&&this._showLoadPanel(),(0,c.when)(i&&i.complete||i).done(function(e){if(e&&!e.isValid)return t._hideLoadPanel(),void n.resolve(!1);var i=t._appointmentForm.option("formData"),a=t.scheduler.createAppointmentAdapter(i),r=a.clone({pathTimeZone:"fromAppointment"}).source(),s=t.scheduler._editAppointmentData,l=t.scheduler._updatedRecAppointment;o.isEmptyText&&""===a.text&&delete r.text,o.isEmptyDescription&&""===a.description&&delete r.description,void 0===o.data.recurrenceRule&&""===a.recurrenceRule&&delete r.recurrenceRule,(0,f.isDefined)(r.repeat)&&delete r.repeat,s&&!l?t.scheduler.updateAppointment(s,r).done(n.resolve):(l&&(t.scheduler.updateAppointment(s,l),delete t.scheduler._updatedRecAppointment),t.scheduler.addAppointment(r).done(n.resolve)),n.done(function(){t._hideLoadPanel(),t.state.lastEditData=r})}),n.promise()}},{key:"_doneButtonClickHandler",value:function(e){e.cancel=!0,this.saveEditData()}},{key:"saveEditData",value:function(){var e=this,t=new c.Deferred;return this._tryLockSaveChanges()&&(0,c.when)(this.saveChanges(!0)).done(function(){if(e.state.lastEditData){var n=e.scheduler.createAppointmentAdapter(e.state.lastEditData),i=n.startDate,o=n.endDate,a=n.allDay,r=i.getTime(),s=o.getTime(),l=a||s-r>=E;e.scheduler._workSpace.updateScrollPosition(i,e.scheduler._resourcesManager.getResourcesFromItem(e.state.lastEditData,!0),l),e.state.lastEditData=null}e._unlockSaveChanges(),t.resolve()}),t.promise()}},{key:"_hideLoadPanel",value:function(){(0,y.hide)()}},{key:"_showLoadPanel",value:function(){var e=this._popup.overlayContent();(0,y.show)({container:e,position:{of:e}})}},{key:"_tryLockSaveChanges",value:function(){return this.state.saveChangesLocker===!1&&(this.state.saveChangesLocker=!0,!0)}},{key:"_unlockSaveChanges",value:function(){this.state.saveChangesLocker=!1}}]),e}();t.default=A,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.AppointmentForm=t.APPOINTMENT_FORM_GROUP_NAMES=void 0;var o=i(n(146)),a=i(n(62)),r=i(n(10)),s=i(n(13)),l=i(n(276)),u=i(n(270)),c=n(0);n(789),n(455),n(301),n(454),n(127);var d=600,h={Main:"mainGroup",Recurrence:"recurrenceGroup"};t.APPOINTMENT_FORM_GROUP_NAMES=h;var f={_appointmentForm:{},_lockDateShiftFlag:!1,_validateAppointmentFormDate:function(e,t,n){var i=null===t||!!t,o=null===n||!!n;!i&&o&&e.option("value",n)},_getAllDayStartDate:function(e){return new Date(new Date(e).setHours(0,0,0,0))},_getAllDayEndDate:function(e){return new Date(new Date(e).setDate(e.getDate()+1))},_getStartDateWithStartHour:function(e,t){return new Date(new Date(e).setHours(t))},create:function(e,t,n,i){return this._appointmentForm=e(t,o.default,{items:this._editors,readOnly:n,showValidationSummary:!0,scrollingEnabled:!0,colCount:"auto",colCountByScreen:{lg:2,xs:1},formData:i,showColonAfterLabel:!1,labelLocation:"top",screenByWidth:function(e){return e<d||"desktop"!==s.default.current().deviceType?"xs":"lg"}}),this._appointmentForm},_dateBoxValueChanged:function(e,t,n){this._validateAppointmentFormDate(e.component,e.value,e.previousValue);var i=a.default.deserializeDate(e.value),o=a.default.deserializeDate(e.previousValue),r=this._appointmentForm.getEditor(t),s=a.default.deserializeDate(r.option("value"));if(!this._appointmentForm._lockDateShiftFlag&&s&&i&&n(s,i)){var l=o?s.getTime()-o.getTime():0;r.option("value",new Date(i.getTime()+l))}},_createTimezoneEditor:function(e,t,n,i,o){var a=this,s=arguments.length>5&&void 0!==arguments[5]&&arguments[5],l=r.default.format("dxScheduler-noTimezoneTitle");return{dataField:e,editorType:"dxSelectBox",visibleIndex:n,colSpan:i,label:{text:" "},editorOptions:{displayExpr:"title",valueExpr:"id",placeholder:l,searchEnabled:!0,onValueChanged:function(e){var n=a._appointmentForm,i=n.getEditor(t);o&&i.option("value",e.value)}},visible:s}},_createDateBoxEditor:function(e,t,n,i,o){return{dataField:e,editorType:"dxDateBox",colSpan:t,label:{text:r.default.format(i)},validationRules:[{type:"required"}],editorOptions:{width:"100%",calendarOptions:{firstDayOfWeek:n},onValueChanged:o}}},_createDateBoxItems:function(e,t,n){var i=this,o=n?2:1,a=t.option("firstDayOfWeek");return[this._createDateBoxEditor(e.startDateExpr,o,a,"dxScheduler-editorLabelStartDate",function(t){i._dateBoxValueChanged(t,e.endDateExpr,function(e,t){return e<t})}),this._createTimezoneEditor(e.startDateTimeZoneExpr,e.endDateTimeZoneExpr,1,o,!0,n),this._createDateBoxEditor(e.endDateExpr,o,a,"dxScheduler-editorLabelEndDate",function(t){i._dateBoxValueChanged(t,e.startDateExpr,function(e,t){return t<e})}),this._createTimezoneEditor(e.endDateTimeZoneExpr,e.startDateTimeZoneExpr,3,o,!1,n)]},_createMainItems:function(e,t,n,i,o){var s=this;return[{dataField:e.textExpr,editorType:"dxTextBox",colSpan:2,label:{text:r.default.format("dxScheduler-editorLabelTitle")}},{itemType:"group",colSpan:2,colCountByScreen:{lg:2,xs:1},items:this._createDateBoxItems(e,t,o)},{itemType:"group",colCountByScreen:{lg:3,xs:3},colSpan:2,items:[{dataField:e.allDayExpr,cssClass:"dx-appointment-form-switch",editorType:"dxSwitch",label:{text:r.default.format("dxScheduler-allDay"),location:"right"},editorOptions:{onValueChanged:function(n){var i=n.value,o=s._appointmentForm.getEditor(e.startDateExpr),r=s._appointmentForm.getEditor(e.endDateExpr),l=a.default.deserializeDate(o.option("value"));if(!s._appointmentForm._lockDateShiftFlag&&l)if(i){var u=s._getAllDayStartDate(l);o.option("value",u),r.option("value",s._getAllDayEndDate(u))}else{var c=s._getStartDateWithStartHour(l,t.option("startDayHour")),d=t._workSpace.calculateEndDate(c);o.option("value",c),r.option("value",d)}o.option("type",i?"date":"datetime"),r.option("type",i?"date":"datetime")}}},{editorType:"dxSwitch",dataField:"repeat",cssClass:"dx-appointment-form-switch",name:"visibilityChanged",label:{text:r.default.format("dxScheduler-editorLabelRecurrence"),location:"right"},editorOptions:{onValueChanged:function(t){var o=s._appointmentForm,a=t.value?1:2;o.itemOption(h.Main,"colSpan",a),o.itemOption(h.Recurrence,"colSpan",a),s._updateRecurrenceItemVisibility(e.recurrenceRuleExpr,t.value,o),i(t.value),n()}}}]},{itemType:"empty",colSpan:2},{dataField:e.descriptionExpr,editorType:"dxTextArea",colSpan:2,label:{text:r.default.format("dxScheduler-editorLabelDescription")}},{itemType:"empty",colSpan:2}]},_updateRecurrenceItemVisibility:function(e,t,n){var i;n.itemOption(h.Recurrence,"visible",t),!t&&n.updateData(e,""),null===(i=n.getEditor(e))||void 0===i?void 0:i.changeValueByVisibility(t)},prepareAppointmentFormEditors:function(e,t,n,i,o,a,r){var s=!!this.getRecurrenceRule(o,e);return i(s),this._editors=[{itemType:"group",name:h.Main,colCountByScreen:{lg:2,xs:1},colSpan:s?1:2,items:this._createMainItems(e,t,n,i,a)},{itemType:"group",name:h.Recurrence,visible:s,colSpan:s?1:2,items:this._createRecurrenceEditor(e,t,r)}],this._editors},_createRecurrenceEditor:function(e,t,n){return[{dataField:e.recurrenceRuleExpr,editorType:"dxRecurrenceEditor",editorOptions:{readOnly:n,firstDayOfWeek:t.option("firstDayOfWeek")},label:{text:" ",visible:!1}}]},getRecurrenceRule:function(e,t){return e[t.recurrenceRuleExpr]},concatResources:function(e){this._editors[0].items=this._editors[0].items.concat(e)},setEditorsType:function(e,t,n,i){var o="".concat(h.Main,".").concat(t),a="".concat(h.Recurrence,".").concat(n),r=e.itemOption(o),s=e.itemOption(a);if(r&&s){var l=r.editorOptions,u=s.editorOptions;l.type=u.type=i?"date":"datetime",e.itemOption(o,"editorOptions",l),e.itemOption(a,"editorOptions",u)}},updateTimeZoneEditorDataSource:function(e,t){var n=new l.default({store:u.default.getDisplayedTimeZones(e),paginate:!0,pageSize:10}),i={dataSource:n};this.setEditorOptions(t,"Main",i)},updateRecurrenceEditorStartDate:function(e,t){var n={startDate:e};this.setEditorOptions(t,"Recurrence",n)},setEditorOptions:function(e,t,n){var i="".concat(h.groupName,".").concat(e),o=this._appointmentForm.itemOption(i);o&&this._appointmentForm.itemOption(i,"editorOptions",(0,c.extend)({},o.editorOptions,n))},updateFormData:function(e,t,n){e._lockDateShiftFlag=!0;var i=new Date(t[n.startDateExpr]),o=new Date(t[n.endDateExpr]);this.updateTimeZoneEditorDataSource(i,n.startDateTimeZoneExpr),this.updateTimeZoneEditorDataSource(o,n.endDateTimeZoneExpr),this.updateRecurrenceEditorStartDate(i,n.recurrenceRuleExpr),e.option("formData",t),e._lockDateShiftFlag=!1}};t.AppointmentForm=f},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t,n){return(d="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=h(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function h(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=y(e),null!==e););return e}function f(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&p(e,t)}function p(e,t){return(p=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function g(e){var t=v();return function(){var n,i=y(e);if(t){var o=y(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return _(this,n)}}function _(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?m(e):t}function m(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function v(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function y(e){return(y=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function b(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function x(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function w(e,t,n){return t&&x(e.prototype,t),n&&x(e,n),e}t.default=void 0;var C=o(n(9)),k=o(n(35)),S=o(n(2)),D=o(n(20)),I=n(0),T=n(1),O=o(n(32)),E=o(n(10)),A=o(n(146)),P=o(n(289)),R=o(n(217)),M=o(n(52)),B=o(n(126)),V=n(147);n(450);var F="dx-recurrence-editor",L="-label",H="-wrapper",z="dx-recurrence-editor-container",N="dx-recurrence-repeat-end",j="dx-recurrence-radiogroup-repeat-type",W="dx-recurrence-numberbox-repeat-count",G="dx-recurrence-datebox-until-date",q="dx-recurrence-button-group",$="dx-recurrence-selectbox-freq",K="dx-recurrence-numberbox-interval",U="dx-recurrence-repeat-on",Y="dx-recurrence-numberbox-day-of-month",Z="dx-recurrence-selectbox-month-of-year",X=70,Q=120,J=1,ee=[{recurrence:"dxScheduler-recurrenceHourly",value:"hourly"},{recurrence:"dxScheduler-recurrenceDaily",value:"daily"},{recurrence:"dxScheduler-recurrenceWeekly",value:"weekly"},{recurrence:"dxScheduler-recurrenceMonthly",value:"monthly"},{recurrence:"dxScheduler-recurrenceYearly",value:"yearly"}],te=ee.map(function(e){return{text:function(){return E.default.format(e.recurrence)},value:e.value}}),ne=[{type:"never"},{type:"until"},{type:"count"}],ie=["SU","MO","TU","WE","TH","FR","SA"],oe=function(){function e(t){b(this,e),this._recurrenceProcessor=(0,V.getRecurrenceProcessor)(),this._recurrenceRule=this._recurrenceProcessor.evalRecurrenceRule(t).rule}return w(e,[{key:"makeRules",value:function(e){this._recurrenceRule=this._recurrenceProcessor.evalRecurrenceRule(e).rule}},{key:"makeRule",value:function(e,t){return!t||Array.isArray(t)&&!t.length?void delete this._recurrenceRule[e]:void((0,T.isDefined)(e)&&("until"===e&&delete this._recurrenceRule.count,"count"===e&&delete this._recurrenceRule.until,this._recurrenceRule[e]=t))}},{key:"getRepeatEndRule",value:function(){var e=this._recurrenceRule;return"count"in e?"count":"until"in e?"until":"never"}},{key:"getRecurrenceString",value:function(){return this._recurrenceProcessor.getRecurrenceString(this._recurrenceRule)}},{key:"getRules",value:function(){return this._recurrenceRule}},{key:"getDaysFromByDayRule",value:function(){return this._recurrenceProcessor.daysFromByDayRule(this._recurrenceRule)}}]),e}(),ae=function(e){function t(){return b(this,t),n.apply(this,arguments)}f(t,e);var n=g(t);return w(t,[{key:"_getDefaultOptions",value:function(){var e=d(y(t.prototype),"_getDefaultOptions",this).call(this);return(0,I.extend)(e,{value:null,startDate:new Date,firstDayOfWeek:void 0})}},{key:"_getFirstDayOfWeek",value:function(){var e=this.option("firstDayOfWeek");return(0,T.isDefined)(e)?e:O.default.firstDayOfWeekIndex()}},{key:"_createComponent",value:function(e,n){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};return this._extendConfig(i,{readOnly:this.option("readOnly")}),d(y(t.prototype),"_createComponent",this).call(this,e,n,i)}},{key:"_init",value:function(){d(y(t.prototype),"_init",this).call(this),this._recurrenceRule=new oe(this.option("value"))}},{key:"_render",value:function(){d(y(t.prototype),"_render",this).call(this),this.$element().addClass(F),this._$container=(0,S.default)("<div>").addClass(z).appendTo(this.$element()),this._prepareEditors(),this._renderEditors(this._$container)}},{key:"getEditorByField",value:function(e){var t=this.getRecurrenceForm().getEditor(e);if(!(0,T.isDefined)(t))switch(e){case"byday":t=this._weekEditor;break;case"count":t=this._repeatCountEditor;break;case"until":t=this._repeatUntilDate}return t}},{key:"_prepareEditors",value:function(){var e=(this._recurrenceRule.getRules().freq||ee[J].value).toLowerCase();return this._editors=[this._createFreqEditor(e),this._createIntervalEditor(e),this._createRepeatOnLabel(e),{itemType:"group",cssClass:U,colCount:2,colCountByScreen:{xs:2},items:this._createRepeatOnEditor(e)},{itemType:"group",items:this._createRepeatEndEditor()}],this._editors}},{key:"_createFreqEditor",value:function(e){var t=this;return{dataField:"freq",name:"FREQ",editorType:"dxSelectBox",cssClass:$,editorOptions:{items:te,value:e,field:"freq",valueExpr:"value",displayExpr:"text",layout:"horizontal",elementAttr:{"class":$},onValueChanged:function(e){return t._valueChangedHandler(e)}},label:{text:E.default.format("dxScheduler-editorLabelRecurrence")}}}},{key:"_createIntervalEditor",value:function(e){var t=this,n=this._recurrenceRule.getRules().interval||1;return{itemType:"group",colCount:2,cssClass:"".concat(K).concat(H),colCountByScreen:{xs:2},items:[{dataField:"interval",editorType:"dxNumberBox",editorOptions:{width:X,min:1,field:"interval",value:n,showSpinButtons:!0,useLargeSpinButtons:!1,elementAttr:{"class":K},onValueChanged:function(e){return t._valueChangedHandler(e)}},label:{text:E.default.format("dxScheduler-recurrenceRepeatEvery")}},{name:"intervalLabel",cssClass:"".concat(K).concat(L),template:function(){return E.default.format("dxScheduler-recurrenceRepeat".concat(e.charAt(0).toUpperCase()).concat(e.substr(1).toLowerCase()))}}]}}},{key:"_createRepeatOnLabel",value:function(e){return{itemType:"group",cssClass:"".concat(U).concat(L),items:[{name:"repeatOnLabel",colSpan:2,template:function(){return E.default.format("dxScheduler-recurrenceRepeatOn")},visible:e&&"daily"!==e&&"hourly"!==e}]}}},{key:"_createRepeatOnEditor",value:function(e){return[this._createByDayEditor(e),this._createByMonthEditor(e),this._createByMonthDayEditor(e)]}},{key:"_createByDayEditor",value:function(e){var t=this;return{dataField:"byday",colSpan:2,template:function(e,n){var i=t._getFirstDayOfWeek(),o=t._daysOfWeekByRules(),a=O.default.getDayNames("abbreviated"),r=ie.slice(i).concat(ie.slice(0,i)),s=a.slice(i).concat(a.slice(0,i)).map(function(e,t){return{text:e,key:r[t]}});t._$repeatOnWeek=(0,S.default)("<div>").addClass(q).appendTo(n),t._weekEditor=t._createComponent(t._$repeatOnWeek,P.default,{items:s,field:"byday",selectionMode:"multiple",selectedItemKeys:o,keyExpr:"key",onSelectionChanged:function(e){var n=e.component.option("selectedItemKeys");
t._recurrenceRule.makeRule("byday",n),t._changeEditorValue()}})},visible:"weekly"===e,label:{visible:!1}}}},{key:"_createByMonthEditor",value:function(e){var t=this,n=O.default.getMonthNames("wide"),i=a(Array(12)).map(function(e,t){return{value:"".concat(t+1),text:n[t]}});return{dataField:"bymonth",editorType:"dxSelectBox",editorOptions:{field:"bymonth",items:i,value:this._monthOfYearByRules(),width:Q,displayExpr:"text",valueExpr:"value",elementAttr:{"class":Z},onValueChanged:function(e){return t._valueChangedHandler(e)}},visible:"yearly"===e,label:{visible:!1}}}},{key:"_createByMonthDayEditor",value:function(e){var t=this;return{dataField:"bymonthday",editorType:"dxNumberBox",editorOptions:{min:1,max:31,width:X,field:"bymonthday",showSpinButtons:!0,useLargeSpinButtons:!1,value:this._dayOfMonthByRules(),elementAttr:{"class":Y},onValueChanged:function(e){return t._valueChangedHandler(e)}},visible:"monthly"===e||"yearly"===e,label:{visible:!1}}}},{key:"_createRepeatEndEditor",value:function(){var e=this,t=this._recurrenceRule.getRepeatEndRule();return[{dataField:"repeatEnd",editorType:"dxRadioGroup",editorOptions:{items:ne,value:t,valueExpr:"type",field:"repeatEnd",itemTemplate:function(t){return"count"===t.type?e._renderRepeatCountEditor():"until"===t.type?e._renderRepeatUntilEditor():e._renderDefaultRepeatEnd()},layout:"vertical",elementAttr:{"class":j},onValueChanged:function(t){return e._repeatEndValueChangedHandler(t)}},label:{text:E.default.format("dxScheduler-recurrenceEnd")}}]}},{key:"_renderEditors",value:function(e){this._recurrenceForm=this._createComponent(e,A.default,{items:this._editors,showValidationSummary:!0,scrollingEnabled:!0,showColonAfterLabel:!1,labelLocation:"top"}),this._disableRepeatEndParts()}},{key:"_setAriaDescribedBy",value:function(e,t){var n="label-".concat(new k.default);e.setAria("describedby",n),e.setAria("id",n,t)}},{key:"getRecurrenceForm",value:function(){return this._recurrenceForm}},{key:"changeValueByVisibility",value:function(e){e?this.option("value")||this._handleDefaults():(this._recurrenceRule.makeRules(""),this.option("value",""))}},{key:"_handleDefaults",value:function(){this._recurrenceRule.makeRule("freq",ee[J].value),this._changeEditorValue()}},{key:"_changeEditorValue",value:function(){this.option("value",this._recurrenceRule.getRecurrenceString()||"")}},{key:"_daysOfWeekByRules",value:function(){var e=this._recurrenceRule.getDaysFromByDayRule();return e.length||(e=[ie[this.option("startDate").getDay()]]),e}},{key:"_dayOfMonthByRules",value:function(){var e=this._recurrenceRule.getRules().bymonthday;return e||(e=this.option("startDate").getDate()),e}},{key:"_monthOfYearByRules",value:function(){var e=this._recurrenceRule.getRules().bymonth;return e||(e=this.option("startDate").getMonth()+1),String(e)}},{key:"_renderDefaultRepeatEnd",value:function(){var e=(0,S.default)("<div>").addClass(N+H);return(0,S.default)("<div>").text(E.default.format("dxScheduler-recurrenceNever")).addClass(N+L).appendTo(e),e}},{key:"_repeatEndValueChangedHandler",value:function(e){var t=e.value;this._disableRepeatEndParts(t),"until"===t&&this._recurrenceRule.makeRule(t,this._getUntilValue()),"count"===t&&this._recurrenceRule.makeRule(t,this._repeatCountEditor.option("value")),"never"===t&&(this._recurrenceRule.makeRule("count",""),this._recurrenceRule.makeRule("until","")),this._changeEditorValue()}},{key:"_disableRepeatEndParts",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this._recurrenceRule.getRepeatEndRule();"until"===e&&(this._repeatCountEditor.option("disabled",!0),this._repeatUntilDate.option("disabled",!1)),"count"===e&&(this._repeatCountEditor.option("disabled",!1),this._repeatUntilDate.option("disabled",!0)),"never"===e&&(this._repeatCountEditor.option("disabled",!0),this._repeatUntilDate.option("disabled",!0))}},{key:"_renderRepeatCountEditor",value:function(){var e=this._recurrenceRule.getRules().count||1,t=(0,S.default)("<div>").addClass(N+H);return(0,S.default)("<div>").text(E.default.format("dxScheduler-recurrenceAfter")).addClass(N+L).appendTo(t),this._$repeatCountEditor=(0,S.default)("<div>").addClass(W).appendTo(t),(0,S.default)("<div>").text(E.default.format("dxScheduler-recurrenceRepeatCount")).addClass(N+L).appendTo(t),this._repeatCountEditor=this._createComponent(this._$repeatCountEditor,B.default,{field:"count",width:X,min:1,showSpinButtons:!0,useLargeSpinButtons:!1,value:e,onValueChanged:this._repeatCountValueChangeHandler.bind(this)}),t}},{key:"_repeatCountValueChangeHandler",value:function(e){if("count"===this._recurrenceRule.getRepeatEndRule()){var t=e.value;this._recurrenceRule.makeRule("count",t),this._changeEditorValue()}}},{key:"_formatUntilDate",value:function(e){return this._recurrenceRule.getRules().until&&D.default.sameDate(this._recurrenceRule.getRules().until,e)?e:D.default.setToDayEnd(e)}},{key:"_renderRepeatUntilEditor",value:function(){var e=this._recurrenceRule.getRules().until||this._formatUntilDate(new Date),t=(0,S.default)("<div>").addClass(N+H);return(0,S.default)("<div>").text(E.default.format("dxScheduler-recurrenceOn")).addClass(N+L).appendTo(t),this._$repeatDateEditor=(0,S.default)("<div>").addClass(G).appendTo(t),this._repeatUntilDate=this._createComponent(this._$repeatDateEditor,R.default,{field:"until",value:e,type:"date",onValueChanged:this._repeatUntilValueChangeHandler.bind(this),calendarOptions:{firstDayOfWeek:this._getFirstDayOfWeek()}}),t}},{key:"_repeatUntilValueChangeHandler",value:function(e){if("until"===this._recurrenceRule.getRepeatEndRule()){var t=this._formatUntilDate(new Date(e.value));this._repeatUntilDate.option("value",t),this._recurrenceRule.makeRule("until",t),this._changeEditorValue()}}},{key:"_valueChangedHandler",value:function(e){var t=e.value,n=e.previousValue,i=e.component.option("field");this.option("visible")?(this._recurrenceRule.makeRule(i,t),"freq"===i&&(this._makeRepeatOnRule(t),this._changeRepeatOnVisibility(t,n)),this._changeEditorValue()):this.option("value","")}},{key:"_makeRepeatOnRule",value:function(e){"daily"!==e&&"hourly"!==e||(this._recurrenceRule.makeRule("byday",""),this._recurrenceRule.makeRule("bymonth",""),this._recurrenceRule.makeRule("bymonthday","")),"weekly"===e&&(this._recurrenceRule.makeRule("byday",this._daysOfWeekByRules()),this._recurrenceRule.makeRule("bymonth",""),this._recurrenceRule.makeRule("bymonthday","")),"monthly"===e&&(this._recurrenceRule.makeRule("bymonthday",this._dayOfMonthByRules()),this._recurrenceRule.makeRule("bymonth",""),this._recurrenceRule.makeRule("byday","")),"yearly"===e&&(this._recurrenceRule.makeRule("bymonthday",this._dayOfMonthByRules()),this._recurrenceRule.makeRule("bymonth",this._monthOfYearByRules()),this._recurrenceRule.makeRule("byday",""))}},{key:"_optionChanged",value:function(e){switch(e.name){case"value":this._recurrenceRule.makeRules(e.value),this._changeRepeatIntervalLabel(),this._disableRepeatEndParts(),this._changeEditorsValue(this._recurrenceRule.getRules()),d(y(t.prototype),"_optionChanged",this).call(this,e);break;case"startDate":this._makeRepeatOnRule(this._recurrenceRule.getRules().freq),(0,T.isDefined)(this._recurrenceRule.getRecurrenceString())&&this._changeEditorValue();break;case"firstDayOfWeek":if(this._weekEditor){var n=O.default.getDayNames("abbreviated"),i=ie.slice(e.value).concat(ie.slice(0,e.value)),o=n.slice(e.value).concat(n.slice(0,e.value)).map(function(e,t){return{text:e,key:i[t]}});this._weekEditor.option("items",o)}this._$repeatDateEditor&&this._repeatUntilDate.option("calendarOptions.firstDayOfWeek",this._getFirstDayOfWeek());break;default:d(y(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_changeRepeatOnVisibility",value:function(e,t){e!==t&&(this._recurrenceForm.itemOption("byday","visible",!1),this._recurrenceForm.itemOption("bymonthday","visible",!1),this._recurrenceForm.itemOption("bymonth","visible",!1),this._recurrenceForm.itemOption("repeatOnLabel","visible",e&&"daily"!==e&&"hourly"!==e),"weekly"===e&&this._recurrenceForm.itemOption("byday","visible",!0),"monthly"===e&&this._recurrenceForm.itemOption("bymonthday","visible",!0),"yearly"===e&&(this._recurrenceForm.itemOption("bymonthday","visible",!0),this._recurrenceForm.itemOption("bymonth","visible",!0)))}},{key:"_changeRepeatIntervalLabel",value:function(){var e=this._recurrenceRule.getRules().freq;e&&this._recurrenceForm.itemOption("intervalLabel","template",E.default.format("dxScheduler-recurrenceRepeat".concat(e.charAt(0).toUpperCase()).concat(e.substr(1).toLowerCase())))}},{key:"_changeEditorsValue",value:function(e){this._recurrenceForm.getEditor("freq").option("value",(e.freq||ee[J].value).toLowerCase()),this._changeDayOfWeekValue(),this._changeDayOfMonthValue(),this._changeMonthOfYearValue(),this._changeIntervalValue(e.interval),this._changeRepeatCountValue(),this._changeRepeatEndValue(),this._changeRepeatUntilValue()}},{key:"_changeIntervalValue",value:function(e){this._recurrenceForm.getEditor("interval").option("value",e||1)}},{key:"_changeRepeatEndValue",value:function(){var e=this._recurrenceRule.getRepeatEndRule();this._recurrenceForm.getEditor("repeatEnd").option("value",e)}},{key:"_changeDayOfWeekValue",value:function(){var e=this._recurrenceForm.itemOption("byday").visible;if(e){var t=this._daysOfWeekByRules();this.getEditorByField("byday").option("selectedItemKeys",t)}}},{key:"_changeDayOfMonthValue",value:function(){var e=this._recurrenceForm.itemOption("bymonthday").visible;if(e){var t=this._dayOfMonthByRules();this._recurrenceForm.getEditor("bymonthday").option("value",t)}}},{key:"_changeMonthOfYearValue",value:function(){var e=this._recurrenceForm.itemOption("bymonth").visible;if(e){var t=this._monthOfYearByRules();this._recurrenceForm.getEditor("bymonth").option("value",t)}}},{key:"_changeRepeatCountValue",value:function(){var e=this._recurrenceRule.getRules().count||1;this._repeatCountEditor.option("value",e)}},{key:"_changeRepeatUntilValue",value:function(){this._repeatUntilDate.option("value",this._getUntilValue())}},{key:"_getUntilValue",value:function(){return this._recurrenceRule.getRules().until||this._formatUntilDate(new Date)}},{key:"toggle",value:function(){this._freqEditor.focus()}},{key:"setAria",value:function(){this._switchEditor&&this._switchEditor.setAria(arguments.length<=0?void 0:arguments[0],arguments.length<=1?void 0:arguments[1])}}]),t}(M.default);(0,C.default)("dxRecurrenceEditor",ae);var re=ae;t.default=re,e.exports=t.default},function(e,t,n){var i=n(130),o=n(307),a=function(e){function t(t,n,i){var o=e.call(this,t,n)||this;return o.iterator=i,o}return Object(i.b)(t,e),t.prototype.add=function(e){return!!this.iterator(e,this._result.length)&&(this._result.push(e),!0)},t}(o.a);t.a=a},function(e,t){e.exports=window.luxon},function(e,t,n){n.d(t,"a",function(){return r});var i=n(307),o=n(73),a=n(74),r=function(){function e(){this.all=!1,this.before=[],this.after=[],this.between=[]}return e.prototype._cacheAdd=function(e,t,n){t&&(t=t instanceof Date?o.b.clone(t):o.b.cloneDates(t)),"all"===e?this.all=t:(n._value=t,this[e].push(n))},e.prototype._cacheGet=function(e,t){var n=!1,r=t?Object.keys(t):[],s=function(e){for(var n=0;n<r.length;n++){var i=r[n];if(String(t[i])!==String(e[i]))return!0}return!1},l=this[e];if("all"===e)n=this.all;else if(Object(a.d)(l))for(var u=0;u<l.length;u++){var c=l[u];if(!r.length||!s(c)){n=c._value;break}}if(!n&&this.all){for(var d=new i.a(e,t),u=0;u<this.all.length&&d.accept(this.all[u]);u++);n=d.getValue(),this._cacheAdd(e,n,t)}return Object(a.d)(n)?o.b.cloneDates(n):n instanceof Date?o.b.clone(n):n},e}()},function(e,t,n){var i=n(73),o=n(74),a=n(148),r=n(794),s=n(796),l=n(797),u=n(308),c=function(){function e(e){this.options=e}return e.prototype.rebuild=function(e,t){var n=this.options;if(e!==this.lastyear&&(this.yearinfo=Object(r.a)(e,n)),Object(o.h)(n.bynweekday)&&(t!==this.lastmonth||e!==this.lastyear)){var i=this.yearinfo,a=i.yearlen,u=i.mrange,c=i.wdaymask;this.monthinfo=Object(s.a)(e,t,a,u,c,n)}Object(o.f)(n.byeaster)&&(this.eastermask=Object(l.a)(e,n.byeaster))},Object.defineProperty(e.prototype,"lastyear",{get:function(){return this.monthinfo?this.monthinfo.lastyear:null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"lastmonth",{get:function(){return this.monthinfo?this.monthinfo.lastmonth:null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"yearlen",{get:function(){return this.yearinfo.yearlen},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"yearordinal",{get:function(){return this.yearinfo.yearordinal},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mrange",{get:function(){return this.yearinfo.mrange},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"wdaymask",{get:function(){return this.yearinfo.wdaymask},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mmask",{get:function(){return this.yearinfo.mmask},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"wnomask",{get:function(){return this.yearinfo.wnomask},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nwdaymask",{get:function(){return this.monthinfo?this.monthinfo.nwdaymask:[]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nextyearlen",{get:function(){return this.yearinfo.nextyearlen},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mdaymask",{get:function(){return this.yearinfo.mdaymask},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nmdaymask",{get:function(){return this.yearinfo.nmdaymask},enumerable:!0,configurable:!0}),e.prototype.ydayset=function(){return[Object(o.k)(this.yearlen),0,this.yearlen]},e.prototype.mdayset=function(e,t,n){for(var i=this.mrange[t-1],a=this.mrange[t],r=Object(o.l)(null,this.yearlen),s=i;s<a;s++)r[s]=s;return[r,i,a]},e.prototype.wdayset=function(e,t,n){for(var a=Object(o.l)(null,this.yearlen+7),r=i.b.toOrdinal(new Date(Date.UTC(e,t-1,n)))-this.yearordinal,s=r,l=0;l<7&&(a[r]=r,++r,this.wdaymask[r]!==this.options.wkst);l++);return[a,s,r]},e.prototype.ddayset=function(e,t,n){var a=Object(o.l)(null,this.yearlen),r=i.b.toOrdinal(new Date(Date.UTC(e,t-1,n)))-this.yearordinal;return a[r]=r,[a,r,r+1]},e.prototype.htimeset=function(e,t,n,o){var a=this,r=[];return this.options.byminute.forEach(function(t){r=r.concat(a.mtimeset(e,t,n,o))}),i.b.sort(r),r},e.prototype.mtimeset=function(e,t,n,o){var a=this.options.bysecond.map(function(n){return new u.b(e,t,n,o)});return i.b.sort(a),a},e.prototype.stimeset=function(e,t,n,i){return[new u.b(e,t,n,i)]},e.prototype.getdayset=function(e){switch(e){case a.a.YEARLY:return this.ydayset.bind(this);case a.a.MONTHLY:return this.mdayset.bind(this);case a.a.WEEKLY:return this.wdayset.bind(this);case a.a.DAILY:return this.ddayset.bind(this);default:return this.ddayset.bind(this)}},e.prototype.gettimeset=function(e){switch(e){case a.a.HOURLY:return this.htimeset.bind(this);case a.a.MINUTELY:return this.mtimeset.bind(this);case a.a.SECONDLY:return this.stimeset.bind(this)}},e}();t.a=c},function(e,t,n){function i(e,t){var n=new Date(Date.UTC(e,0,1)),i=r.b.isLeapYear(e)?366:365,l=r.b.isLeapYear(e+1)?366:365,u=r.b.toOrdinal(n),c=r.b.getWeekday(n),d=Object(a.a)(Object(a.a)({yearlen:i,nextyearlen:l,yearordinal:u,yearweekday:c},o(e)),{wnomask:null});if(Object(s.b)(t.byweekno))return d;d.wnomask=Object(s.l)(0,i+7);var h,f,p=h=Object(s.j)(7-c+t.wkst,7);p>=4?(p=0,f=d.yearlen+Object(s.j)(c-t.wkst,7)):f=i-p;for(var g=Math.floor(f/7),_=Object(s.j)(f,7),m=Math.floor(g+_/4),v=0;v<t.byweekno.length;v++){var y=t.byweekno[v];if(y<0&&(y+=m+1),y>0&&y<=m){var b=void 0;y>1?(b=p+7*(y-1),p!==h&&(b-=7-h)):b=p;for(var x=0;x<7&&(d.wnomask[b]=1,b++,d.wdaymask[b]!==t.wkst);x++);}}if(Object(s.c)(t.byweekno,1)){var b=p+7*m;if(p!==h&&(b-=7-h),b<i)for(var v=0;v<7&&(d.wnomask[b]=1,b+=1,d.wdaymask[b]!==t.wkst);v++);}if(p){var w=void 0;if(Object(s.c)(t.byweekno,-1))w=-1;else{var C=r.b.getWeekday(new Date(Date.UTC(e-1,0,1))),k=Object(s.j)(7-C.valueOf()+t.wkst,7),S=r.b.isLeapYear(e-1)?366:365,D=void 0;k>=4?(k=0,D=S+Object(s.j)(C-t.wkst,7)):D=i-p,w=Math.floor(52+Object(s.j)(D,7)/4)}if(Object(s.c)(t.byweekno,w))for(var b=0;b<p;b++)d.wnomask[b]=1}return d}function o(e){var t=r.b.isLeapYear(e)?366:365,n=new Date(Date.UTC(e,0,1)),i=r.b.getWeekday(n);return 365===t?{mmask:l.a,mdaymask:l.e,nmdaymask:l.g,wdaymask:l.i.slice(i),mrange:l.b}:{mmask:l.c,mdaymask:l.f,nmdaymask:l.h,wdaymask:l.i.slice(i),mrange:l.d}}t.a=i;var a=n(130),r=n(73),s=n(74),l=n(795)},function(e,t,n){n.d(t,"i",function(){return x}),n.d(t,"a",function(){return a}),n.d(t,"b",function(){return b}),n.d(t,"c",function(){return r}),n.d(t,"d",function(){return y}),n.d(t,"e",function(){return h}),n.d(t,"f",function(){return d}),n.d(t,"g",function(){return v}),n.d(t,"h",function(){return m});var i=n(130),o=n(74),a=Object(i.c)(Object(o.l)(1,31),Object(o.l)(2,28),Object(o.l)(3,31),Object(o.l)(4,30),Object(o.l)(5,31),Object(o.l)(6,30),Object(o.l)(7,31),Object(o.l)(8,31),Object(o.l)(9,30),Object(o.l)(10,31),Object(o.l)(11,30),Object(o.l)(12,31),Object(o.l)(1,7)),r=Object(i.c)(Object(o.l)(1,31),Object(o.l)(2,29),Object(o.l)(3,31),Object(o.l)(4,30),Object(o.l)(5,31),Object(o.l)(6,30),Object(o.l)(7,31),Object(o.l)(8,31),Object(o.l)(9,30),Object(o.l)(10,31),Object(o.l)(11,30),Object(o.l)(12,31),Object(o.l)(1,7)),s=Object(o.k)(1,29),l=Object(o.k)(1,30),u=Object(o.k)(1,31),c=Object(o.k)(1,32),d=Object(i.c)(c,l,c,u,c,u,c,c,u,c,u,c,c.slice(0,7)),h=Object(i.c)(c,s,c,u,c,u,c,c,u,c,u,c,c.slice(0,7)),f=Object(o.k)(-28,0),p=Object(o.k)(-29,0),g=Object(o.k)(-30,0),_=Object(o.k)(-31,0),m=Object(i.c)(_,p,_,g,_,g,_,_,g,_,g,_,_.slice(0,7)),v=Object(i.c)(_,f,_,g,_,g,_,_,g,_,g,_,_.slice(0,7)),y=[0,31,60,91,121,152,182,213,244,274,305,335,366],b=[0,31,59,90,120,151,181,212,243,273,304,334,365],x=function(){for(var e=[],t=0;t<55;t++)e=e.concat(Object(o.k)(7));return e}()},function(e,t,n){function i(e,t,n,i,r,s){var l={lastyear:e,lastmonth:t,nwdaymask:[]},u=[];if(s.freq===o.c.YEARLY)if(Object(a.b)(s.bymonth))u=[[0,n]];else for(var c=0;c<s.bymonth.length;c++)t=s.bymonth[c],u.push(i.slice(t-1,t+1));else s.freq===o.c.MONTHLY&&(u=[i.slice(t-1,t+1)]);if(Object(a.b)(u))return l;l.nwdaymask=Object(a.l)(0,n);for(var c=0;c<u.length;c++)for(var d=u[c],h=d[0],f=d[1]-1,p=0;p<s.bynweekday.length;p++){var g=void 0,_=s.bynweekday[p],m=_[0],v=_[1];v<0?(g=f+7*(v+1),g-=Object(a.j)(r[g]-m,7)):(g=h+7*(v-1),g+=Object(a.j)(7-r[g]+m,7)),h<=g&&g<=f&&(l.nwdaymask[g]=1)}return l}t.a=i;var o=n(129),a=n(74)},function(e,t,n){function i(e,t){void 0===t&&(t=0);var n=e%19,i=Math.floor(e/100),o=e%100,a=Math.floor(i/4),r=i%4,s=Math.floor((i+8)/25),l=Math.floor((i-s+1)/3),u=Math.floor(19*n+i-a-l+15)%30,c=Math.floor(o/4),d=o%4,h=Math.floor(32+2*r+2*c-u-d)%7,f=Math.floor((n+11*u+22*h)/451),p=Math.floor((u+h-7*f+114)/31),g=(u+h-7*f+114)%31+1,_=Date.UTC(e,p-1,g+t),m=Date.UTC(e,0,1);return[Math.ceil((_-m)/864e5)]}t.a=i},function(e,t,n){function i(e,t,n,i,r,s){for(var l=[],u=0;u<e.length;u++){var c=void 0,d=void 0,h=e[u];h<0?(c=Math.floor(h/t.length),d=Object(a.j)(h,t.length)):(c=Math.floor((h-1)/t.length),d=Object(a.j)(h-1,t.length));for(var f=[],p=n;p<i;p++){var g=s[p];Object(a.f)(g)&&f.push(g)}var _=void 0;_=c<0?f.slice(c)[0]:f[c];var m=t[d],v=o.b.fromOrdinal(r.yearordinal+_),y=o.b.combine(v,m);Object(a.c)(l,y)||l.push(y)}return o.b.sort(l),l}t.a=i;var o=n(73),a=n(74)},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),n.d(t,"fromText",function(){return s}),n.d(t,"isFullyConvertible",function(){return c}),n.d(t,"toText",function(){return u});var i=n(800),o=n(801),a=n(224),r=n(310);n.d(t,"parseText",function(){return o.a});/*!
* rrule.js - Library for working with recurrence rules for calendar dates.
* https://github.com/jakubroztocil/rrule
*
* Copyright 2010, Jakub Roztocil and Lars Schoning
* Licenced under the BSD licence.
* https://github.com/jakubroztocil/rrule/blob/master/LICENCE
*
*/
var s=function(e,t){return void 0===t&&(t=r.a),new a.default(Object(o.a)(e,t)||void 0)},l=["count","until","interval","byweekday","bymonthday","bymonth"];i.a.IMPLEMENTED=[],i.a.IMPLEMENTED[a.default.HOURLY]=l,i.a.IMPLEMENTED[a.default.MINUTELY]=l,i.a.IMPLEMENTED[a.default.DAILY]=["byhour"].concat(l),i.a.IMPLEMENTED[a.default.WEEKLY]=l,i.a.IMPLEMENTED[a.default.MONTHLY]=l,i.a.IMPLEMENTED[a.default.YEARLY]=["byweekno","byyearday"].concat(l);var u=function(e,t,n,o){return new i.a(e,t,n,o).toString()},c=i.a.isFullyConvertible},function(e,t,n){var i=n(310),o=n(224),a=n(74),r=function(e,t){return e.indexOf(t)!==-1},s=function(e){return e.toString()},l=function(e,t,n){return t+" "+n+", "+e},u=function(){function e(e,t,n,o){if(void 0===t&&(t=s),void 0===n&&(n=i.a),void 0===o&&(o=l),this.text=[],this.language=n||i.a,this.gettext=t,this.dateFormatter=o,this.rrule=e,this.options=e.options,this.origOptions=e.origOptions,this.origOptions.bymonthday){var r=[].concat(this.options.bymonthday),u=[].concat(this.options.bynmonthday);r.sort(function(e,t){return e-t}),u.sort(function(e,t){return t-e}),this.bymonthday=r.concat(u),this.bymonthday.length||(this.bymonthday=null)}if(Object(a.f)(this.origOptions.byweekday)){var c=Object(a.d)(this.origOptions.byweekday)?this.origOptions.byweekday:[this.origOptions.byweekday],d=String(c);this.byweekday={allWeeks:c.filter(function(e){return!e.n}),someWeeks:c.filter(function(e){return Boolean(e.n)}),isWeekdays:d.indexOf("MO")!==-1&&d.indexOf("TU")!==-1&&d.indexOf("WE")!==-1&&d.indexOf("TH")!==-1&&d.indexOf("FR")!==-1&&d.indexOf("SA")===-1&&d.indexOf("SU")===-1,isEveryDay:d.indexOf("MO")!==-1&&d.indexOf("TU")!==-1&&d.indexOf("WE")!==-1&&d.indexOf("TH")!==-1&&d.indexOf("FR")!==-1&&d.indexOf("SA")!==-1&&d.indexOf("SU")!==-1};var h=function(e,t){return e.weekday-t.weekday};this.byweekday.allWeeks.sort(h),this.byweekday.someWeeks.sort(h),this.byweekday.allWeeks.length||(this.byweekday.allWeeks=null),this.byweekday.someWeeks.length||(this.byweekday.someWeeks=null)}else this.byweekday=null}return e.isFullyConvertible=function(t){var n=!0;if(!(t.options.freq in e.IMPLEMENTED))return!1;if(t.origOptions.until&&t.origOptions.count)return!1;for(var i in t.origOptions){if(r(["dtstart","wkst","freq"],i))return!0;if(!r(e.IMPLEMENTED[t.options.freq],i))return!1}return n},e.prototype.isFullyConvertible=function(){return e.isFullyConvertible(this.rrule)},e.prototype.toString=function(){var t=this.gettext;if(!(this.options.freq in e.IMPLEMENTED))return t("RRule error: Unable to fully convert this rrule to text");if(this.text=[t("every")],this[o.default.FREQUENCIES[this.options.freq]](),this.options.until){this.add(t("until"));var n=this.options.until;this.add(this.dateFormatter(n.getUTCFullYear(),this.language.monthNames[n.getUTCMonth()],n.getUTCDate()))}else this.options.count&&this.add(t("for")).add(this.options.count.toString()).add(t(this.plural(this.options.count)?"times":"time"));return this.isFullyConvertible()||this.add(t("(~ approximate)")),this.text.join("")},e.prototype.HOURLY=function(){var e=this.gettext;1!==this.options.interval&&this.add(this.options.interval.toString()),this.add(e(this.plural(this.options.interval)?"hours":"hour"))},e.prototype.MINUTELY=function(){var e=this.gettext;1!==this.options.interval&&this.add(this.options.interval.toString()),this.add(e(this.plural(this.options.interval)?"minutes":"minute"))},e.prototype.DAILY=function(){var e=this.gettext;1!==this.options.interval&&this.add(this.options.interval.toString()),this.byweekday&&this.byweekday.isWeekdays?this.add(e(this.plural(this.options.interval)?"weekdays":"weekday")):this.add(e(this.plural(this.options.interval)?"days":"day")),this.origOptions.bymonth&&(this.add(e("in")),this._bymonth()),this.bymonthday?this._bymonthday():this.byweekday?this._byweekday():this.origOptions.byhour&&this._byhour()},e.prototype.WEEKLY=function(){var e=this.gettext;1!==this.options.interval&&this.add(this.options.interval.toString()).add(e(this.plural(this.options.interval)?"weeks":"week")),this.byweekday&&this.byweekday.isWeekdays?1===this.options.interval?this.add(e(this.plural(this.options.interval)?"weekdays":"weekday")):this.add(e("on")).add(e("weekdays")):this.byweekday&&this.byweekday.isEveryDay?this.add(e(this.plural(this.options.interval)?"days":"day")):(1===this.options.interval&&this.add(e("week")),this.origOptions.bymonth&&(this.add(e("in")),this._bymonth()),this.bymonthday?this._bymonthday():this.byweekday&&this._byweekday())},e.prototype.MONTHLY=function(){var e=this.gettext;this.origOptions.bymonth?(1!==this.options.interval&&(this.add(this.options.interval.toString()).add(e("months")),this.plural(this.options.interval)&&this.add(e("in"))),this._bymonth()):(1!==this.options.interval&&this.add(this.options.interval.toString()),this.add(e(this.plural(this.options.interval)?"months":"month"))),this.bymonthday?this._bymonthday():this.byweekday&&this.byweekday.isWeekdays?this.add(e("on")).add(e("weekdays")):this.byweekday&&this._byweekday()},e.prototype.YEARLY=function(){var e=this.gettext;this.origOptions.bymonth?(1!==this.options.interval&&(this.add(this.options.interval.toString()),this.add(e("years"))),this._bymonth()):(1!==this.options.interval&&this.add(this.options.interval.toString()),this.add(e(this.plural(this.options.interval)?"years":"year"))),this.bymonthday?this._bymonthday():this.byweekday&&this._byweekday(),this.options.byyearday&&this.add(e("on the")).add(this.list(this.options.byyearday,this.nth,e("and"))).add(e("day")),this.options.byweekno&&this.add(e("in")).add(e(this.plural(this.options.byweekno.length)?"weeks":"week")).add(this.list(this.options.byweekno,void 0,e("and")))},e.prototype._bymonthday=function(){var e=this.gettext;this.byweekday&&this.byweekday.allWeeks?this.add(e("on")).add(this.list(this.byweekday.allWeeks,this.weekdaytext,e("or"))).add(e("the")).add(this.list(this.bymonthday,this.nth,e("or"))):this.add(e("on the")).add(this.list(this.bymonthday,this.nth,e("and")))},e.prototype._byweekday=function(){var e=this.gettext;this.byweekday.allWeeks&&!this.byweekday.isWeekdays&&this.add(e("on")).add(this.list(this.byweekday.allWeeks,this.weekdaytext)),this.byweekday.someWeeks&&(this.byweekday.allWeeks&&this.add(e("and")),this.add(e("on the")).add(this.list(this.byweekday.someWeeks,this.weekdaytext,e("and"))))},e.prototype._byhour=function(){var e=this.gettext;this.add(e("at")).add(this.list(this.origOptions.byhour,void 0,e("and")))},e.prototype._bymonth=function(){this.add(this.list(this.options.bymonth,this.monthtext,this.gettext("and")))},e.prototype.nth=function(e){e=parseInt(e.toString(),10);var t,n,i=this.gettext;if(e===-1)return i("last");switch(n=Math.abs(e)){case 1:case 21:case 31:t=n+i("st");break;case 2:case 22:t=n+i("nd");break;case 3:case 23:t=n+i("rd");break;default:t=n+i("th")}return e<0?t+" "+i("last"):t},e.prototype.monthtext=function(e){return this.language.monthNames[e-1]},e.prototype.weekdaytext=function(e){var t=Object(a.e)(e)?(e+1)%7:e.getJsWeekday();return(e.n?this.nth(e.n)+" ":"")+this.language.dayNames[t]},e.prototype.plural=function(e){return e%100!==1},e.prototype.add=function(e){return this.text.push(" "),this.text.push(e),this},e.prototype.list=function(e,t,n,i){void 0===i&&(i=","),Object(a.d)(e)||(e=[e]);var o=function(e,t,n){for(var i="",o=0;o<e.length;o++)0!==o&&(i+=o===e.length-1?" "+n+" ":t+" "),i+=e[o];return i};t=t||function(e){return e.toString()};var r=this,s=function(e){return t&&t.call(r,e)};return n?o(e.map(s),i,n):e.map(s).join(i+" ")},e}();t.a=u},function(e,t,n){function i(e,t){function n(){p.expect("every");var e=p.acceptNumber();if(e&&(f.interval=parseInt(e[0],10)),p.isDone())throw new Error("Unexpected end");switch(p.symbol){case"day(s)":f.freq=a.default.DAILY,p.nextSymbol()&&(s(),h());break;case"weekday(s)":f.freq=a.default.WEEKLY,f.byweekday=[a.default.MO,a.default.TU,a.default.WE,a.default.TH,a.default.FR],p.nextSymbol(),h();break;case"week(s)":f.freq=a.default.WEEKLY,p.nextSymbol()&&(i(),h());break;case"hour(s)":f.freq=a.default.HOURLY,p.nextSymbol()&&(i(),h());break;case"minute(s)":f.freq=a.default.MINUTELY,p.nextSymbol()&&(i(),h());break;case"month(s)":f.freq=a.default.MONTHLY,p.nextSymbol()&&(i(),h());break;case"year(s)":f.freq=a.default.YEARLY,p.nextSymbol()&&(i(),h());break;case"monday":case"tuesday":case"wednesday":case"thursday":case"friday":case"saturday":case"sunday":f.freq=a.default.WEEKLY;var t=p.symbol.substr(0,2).toUpperCase();if(f.byweekday=[a.default[t]],!p.nextSymbol())return;for(;p.accept("comma");){if(p.isDone())throw new Error("Unexpected end");var n=u();if(!n)throw new Error("Unexpected symbol "+p.symbol+", expected weekday");f.byweekday.push(a.default[n]),p.nextSymbol()}d(),h();break;case"january":case"february":case"march":case"april":case"may":case"june":case"july":case"august":case"september":case"october":case"november":case"december":if(f.freq=a.default.YEARLY,f.bymonth=[l()],!p.nextSymbol())return;for(;p.accept("comma");){if(p.isDone())throw new Error("Unexpected end");var o=l();if(!o)throw new Error("Unexpected symbol "+p.symbol+", expected month");f.bymonth.push(o),p.nextSymbol()}i(),h();break;default:throw new Error("Unknown symbol")}}function i(){var e=p.accept("on"),t=p.accept("the");if(e||t)do{var n=c(),i=u(),o=l();if(n)i?(p.nextSymbol(),f.byweekday||(f.byweekday=[]),f.byweekday.push(a.default[i].nth(n))):(f.bymonthday||(f.bymonthday=[]),f.bymonthday.push(n),p.accept("day(s)"));else if(i)p.nextSymbol(),f.byweekday||(f.byweekday=[]),f.byweekday.push(a.default[i]);else if("weekday(s)"===p.symbol)p.nextSymbol(),f.byweekday||(f.byweekday=[a.default.MO,a.default.TU,a.default.WE,a.default.TH,a.default.FR]);else if("week(s)"===p.symbol){p.nextSymbol();var r=p.acceptNumber();if(!r)throw new Error("Unexpected symbol "+p.symbol+", expected week number");for(f.byweekno=[parseInt(r[0],10)];p.accept("comma");){if(r=p.acceptNumber(),!r)throw new Error("Unexpected symbol "+p.symbol+"; expected monthday");f.byweekno.push(parseInt(r[0],10))}}else{if(!o)return;p.nextSymbol(),f.bymonth||(f.bymonth=[]),f.bymonth.push(o)}}while(p.accept("comma")||p.accept("the")||p.accept("on"))}function s(){var e=p.accept("at");if(e)do{var t=p.acceptNumber();if(!t)throw new Error("Unexpected symbol "+p.symbol+", expected hour");for(f.byhour=[parseInt(t[0],10)];p.accept("comma");){if(t=p.acceptNumber(),!t)throw new Error("Unexpected symbol "+p.symbol+"; expected hour");f.byhour.push(parseInt(t[0],10))}}while(p.accept("comma")||p.accept("at"))}function l(){switch(p.symbol){case"january":return 1;case"february":return 2;case"march":return 3;case"april":return 4;case"may":return 5;case"june":return 6;case"july":return 7;case"august":return 8;case"september":return 9;case"october":return 10;case"november":return 11;case"december":return 12;default:return!1}}function u(){switch(p.symbol){case"monday":case"tuesday":case"wednesday":case"thursday":case"friday":case"saturday":case"sunday":return p.symbol.substr(0,2).toUpperCase();default:return!1}}function c(){switch(p.symbol){case"last":return p.nextSymbol(),-1;case"first":return p.nextSymbol(),1;case"second":return p.nextSymbol(),p.accept("last")?-2:2;case"third":return p.nextSymbol(),p.accept("last")?-3:3;case"nth":var e=parseInt(p.value[1],10);if(e<-366||e>366)throw new Error("Nth out of range: "+e);return p.nextSymbol(),p.accept("last")?-e:e;default:return!1}}function d(){p.accept("on"),p.accept("the");var e=c();if(e)for(f.bymonthday=[e],p.nextSymbol();p.accept("comma");){if(e=c(),!e)throw new Error("Unexpected symbol "+p.symbol+"; expected monthday");f.bymonthday.push(e),p.nextSymbol()}}function h(){if("until"===p.symbol){var e=Date.parse(p.text);if(!e)throw new Error("Cannot parse until date:"+p.text);f.until=new Date(e)}else p.accept("for")&&(f.count=parseInt(p.value[0],10),p.expect("number"))}void 0===t&&(t=o.a);var f={},p=new r(t.tokens);return p.start(e)?(n(),f):null}t.a=i;var o=n(310),a=n(224),r=function(){function e(e){this.done=!0,this.rules=e}return e.prototype.start=function(e){return this.text=e,this.done=!1,this.nextSymbol()},e.prototype.isDone=function(){return this.done&&null===this.symbol},e.prototype.nextSymbol=function(){var e,t,n=this;this.symbol=null,this.value=null;do{if(this.done)return!1;var i=void 0;e=null;for(var o in this.rules){i=this.rules[o];var a=i.exec(n.text);a&&(null===e||a[0].length>e[0].length)&&(e=a,t=o)}if(null!=e&&(this.text=this.text.substr(e[0].length),""===this.text&&(this.done=!0)),null==e)return this.done=!0,this.symbol=null,void(this.value=null)}while("SKIP"===t);return this.symbol=t,this.value=e,!0},e.prototype.accept=function(e){if(this.symbol===e){if(this.value){var t=this.value;return this.nextSymbol(),t}return this.nextSymbol(),!0}return!1},e.prototype.acceptNumber=function(){return this.accept("number")},e.prototype.expect=function(e){if(this.accept(e))return!0;throw new Error("expected "+e+" but found "+this.symbol)},e}()},function(e,t,n){function i(e,t,n,i,s,l){function u(e,t){n.forEach(function(n){n.between(e,t,!0).forEach(function(e){c[Number(e)]=!0})})}var c={},d=e.accept;s.forEach(function(e){var t=new o.a(e,l).rezonedDate();c[Number(t)]=!0}),e.accept=function(e){var t=Number(e);return isNaN(t)?d.call(this,e):!(!c[t]&&(u(new Date(t-1),new Date(t+1)),!c[t]))||(c[t]=!0,d.call(this,e))},"between"===e.method&&(u(e.args.after,e.args.before),e.accept=function(e){var t=Number(e);return!!c[t]||(c[t]=!0,d.call(this,e))});for(var h=0;h<i.length;h++){var f=new o.a(i[h],l).rezonedDate();if(!e.accept(new Date(f.getTime())))break}t.forEach(function(t){Object(a.a)(e,t.options)});var p=e._result;switch(r.b.sort(p),e.method){case"all":case"between":return p;case"before":return p.length&&p[p.length-1]||null;case"after":default:return p.length&&p[0]||null}}t.a=i;var o=n(309),a=n(513),r=n(73)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.CompactAppointmentsHelper=void 0;var s=i(n(2)),l=i(n(38)),u=n(30),c=i(n(10)),d=n(118),h=n(6),f=n(0),p=n(24),g=n(517),_=n(169),m="dx-scheduler-appointment-collector",v=m+"-compact",y=m+"-content",b=5,x=1,w=function(){function e(t){o(this,e),this.instance=t,this.elements=[]}return r(e,[{key:"render",value:function(e){var t=e.isCompact,n=e.items,i=e.buttonColor,o=this._createTemplate(n.data.length,t),a=this._createCompactButton(o,e),r=a.$element();return this._makeBackgroundColor(r,n.colors,i),this._makeBackgroundDarker(r),this.elements.push(r),r.data("items",this._createTooltipInfos(n)),r}},{key:"clear",value:function(){this.elements.forEach(function(e){e.detach(),e.remove()}),this.elements=[]}},{key:"_createTooltipInfos",value:function(e){var t=this;return e.data.map(function(n,i){var o,a=t.instance.createAppointmentAdapter(n).clone();if((null===(o=e.settings)||void 0===o?void 0:o.length)>0){var r=e.settings[i].info;a.startDate=r.sourceAppointment.startDate,a.endDate=r.sourceAppointment.endDate}return new g.AppointmentTooltipInfo(n,a.source(),e.colors[i],e.settings[i])})}},{key:"_onButtonClick",value:function(e,t){var n=(0,s.default)(e.element);this.instance.showAppointmentTooltipCore(n,n.data("items"),this._getExtraOptionsForTooltip(t))}},{key:"_getExtraOptionsForTooltip",value:function(e){return{clickEvent:this._clickEvent(e.onAppointmentClick).bind(this),dragBehavior:e.allowDrag&&this._createTooltipDragBehavior().bind(this),dropDownAppointmentTemplate:this.instance.option().dropDownAppointmentTemplate,isButtonClick:!0}}},{key:"_clickEvent",value:function(e){var t=this;return function(n){var i={itemData:n.itemData.appointment,itemElement:n.itemElement},o=(0,f.extendFromObject)(t.instance.fire("mapAppointmentFields",i),n,!1);delete o.itemData,delete o.itemIndex,delete o.itemElement,e(o)}}},{key:"_createTooltipDragBehavior",value:function(){var e=this;return function(t){var n=(0,s.default)(t.element),i=e.instance.getWorkSpace(),o=function(e){var t;return null===(t=(0,s.default)(e).data(_.LIST_ITEM_DATA_KEY))||void 0===t?void 0:t.appointment},a=function(e,t){return t.itemSettings},r={filter:".".concat(_.LIST_ITEM_CLASS),isSetCursorOffset:!0};i._createDragBehaviorBase(n,o,a,r)}}},{key:"_getCollectorOffset",value:function(e,t){return t-e-this._getCollectorRightOffset()}},{key:"_getCollectorRightOffset",value:function(){return this.instance.getRenderingStrategyInstance()._isCompactTheme()?x:b}},{key:"_makeBackgroundDarker",value:function(e){e.css("boxShadow","inset ".concat((0,p.getBoundingRect)(e.get(0)).width,"px 0 0 0 rgba(0, 0, 0, 0.3)"))}},{key:"_makeBackgroundColor",value:function(e,t,n){h.when.apply(null,t).done(function(){this._makeBackgroundColorCore(e,n,arguments)}.bind(this))}},{key:"_makeBackgroundColorCore",value:function(e,t,n){var i,o=!0;t&&t.done(function(t){if(n.length){i=n[0];for(var a=1;a<n.length;a++){if(i!==n[a]){o=!1;break}i=t}}t&&o&&e.css("backgroundColor",t)}.bind(this))}},{key:"_setPosition",value:function(e,t){(0,u.move)(e,{top:t.top,left:t.left})}},{key:"_createCompactButton",value:function(e,t){var n=this,i=this._createCompactButtonElement(t);return this.instance._createComponent(i,l.default,{type:"default",width:t.width,height:t.height,onClick:function(e){return n._onButtonClick(e,t)},template:this._renderTemplate(e,t.items,t.isCompact)})}},{key:"_createCompactButtonElement",value:function(e){var t=e.isCompact,n=e.$container,i=e.width,o=e.coordinates,a=e.applyOffset,r=e.cellWidth,l=(0,s.default)("<div>").addClass(m).toggleClass(v,t).appendTo(n),u=a?this._getCollectorOffset(i,r):0;return this._setPosition(l,{top:o.top,left:o.left+u}),l}},{key:"_renderTemplate",value:function(e,t,n){return new d.FunctionTemplate(function(i){return e.render({model:{appointmentCount:t.data.length,isCompact:n},container:i.container})})}},{key:"_createTemplate",value:function(e,t){return this._initButtonTemplate(e,t),this.instance._getAppointmentTemplate("appointmentCollectorTemplate")}},{key:"_initButtonTemplate",value:function(e,t){var n=this;this.instance._templateManager.addDefaultTemplates({appointmentCollector:new d.FunctionTemplate(function(i){return n._createButtonTemplate(e,(0,s.default)(i.container),t)})})}},{key:"_createButtonTemplate",value:function(e,t,n){var i=n?e:c.default.getFormatter("dxScheduler-moreAppointments")(e);return t.append((0,s.default)("<span>").text(i)).addClass(y)}}]),e}();t.CompactAppointmentsHelper=w},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.DesktopTooltipStrategy=void 0;var m=n(518),v=o(n(195)),y=n(46),b="dx-scheduler-appointment-tooltip-wrapper",x=200,w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_prepareBeforeVisibleChanged",value:function(e){this._tooltip.option("position",{my:"bottom",at:"top",boundary:this._getBoundary(e),offset:this._extraOptions.offset,collision:"fit flipfit"})}},{key:"_getBoundary",value:function(e){return this._options.isAppointmentInAllDayPanel(e[0].appointment)?this._options.container:this._options.getScrollableContainer()}},{key:"_onShown",value:function(){l(_(t.prototype),"_onShown",this).call(this),this._extraOptions.isButtonClick&&(this._list.focus(),this._list.option("focusedElement",null))}},{key:"_createListOption",value:function(e,n){var i=l(_(t.prototype),"_createListOption",this).call(this,e,n);return i.showScrollbar=y.touch?"always":"onHover",i}},{key:"_createTooltip",value:function(e,t){var n=this._createTooltipElement(b);return this._options.createComponent(n,v.default,{target:e,maxHeight:x,rtlEnabled:this._extraOptions.rtlEnabled,onShown:this._onShown.bind(this),contentTemplate:this._getContentTemplate(t)})}},{key:"_onListRender",value:function(e){return this._extraOptions.dragBehavior&&this._extraOptions.dragBehavior(e)}}]),t}(m.TooltipStrategyBase);t.DesktopTooltipStrategy=w},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.MobileTooltipStrategy=void 0;var g=o(n(68)),_=n(518),m=n(7),v=o(n(2)),y="dx-scheduler-overlay-panel",b=.9,x={PHONE:250,TABLET:"90%",DEFAULT:"auto"},w={PHONE:"100%",TABLET:"80%"},C={show:{type:"slide",duration:300,from:{position:{my:"top",at:"bottom",of:(0,m.getWindow)()}},to:{position:{my:"center",at:"center",of:(0,m.getWindow)()}}},hide:{type:"slide",duration:300,to:{position:{my:"top",at:"bottom",of:(0,m.getWindow)()}},from:{position:{my:"center",at:"center",of:(0,m.getWindow)()}}}},k=function(e){return{shading:!1,width:w.PHONE,height:e>x.PHONE?x.PHONE:x.DEFAULT,position:{my:"bottom",at:"bottom",of:(0,m.getWindow)()}}},S=function(e){var t=(0,v.default)((0,m.getWindow)()).height()*b;return{shading:!0,width:w.TABLET,height:e>t?x.TABLET:x.DEFAULT,position:{my:"center",at:"center",of:(0,m.getWindow)()}}},D=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_shouldUseTarget",value:function(){return!1}},{key:"_onShowing",value:function(){var e=(0,v.default)((0,m.getWindow)()).width()>700;this._tooltip.option("height",x.DEFAULT);var t=this._list.$element().outerHeight();this._tooltip.option(e?S(t):k(t))}},{key:"_createTooltip",value:function(e,t){var n=this,i=this._createTooltipElement(y);return this._options.createComponent(i,g.default,{target:(0,m.getWindow)(),closeOnOutsideClick:!0,animation:C,onShowing:function(){return n._onShowing()},onShown:this._onShown.bind(this),contentTemplate:this._getContentTemplate(t)})}}]),t}(_.TooltipStrategyBase);t.MobileTooltipStrategy=D},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(11)),r=i(n(5)),s=n(39),l=n(30),u=i(n(20)),c=n(4),d=n(1),h=n(3),f=n(50),p=n(12),g=n(0),_=n(18),m=n(147),v=i(n(9)),y=i(n(225)),b=i(n(807)),x=n(8),w=n(142),C=i(n(10)),k=i(n(58)),S=n(6),D=i(n(86)),I=n(169),T="dx-scheduler-scrollable-appointments",O="dx-scheduler-appointment",E="dx-scheduler-appointment-title",A="dx-scheduler-appointment-content-details",P="dx-scheduler-appointment-content-date",R="dx-scheduler-appointment-recurrence-icon",M="dx-scheduler-appointment-content-allday",B=(0,x.addNamespace)(w.name,"dxSchedulerAppointment"),V=u.default.dateToMilliseconds,F=k.default.inherit({ctor:function(e,t){this.callBase(e,t)},_supportedKeys:function(){var e=this.callBase(),t=function(e){var t=this._getAccessAppointments(),n=t.filter(".dx-state-focused"),i=n.data(I.APPOINTMENT_SETTINGS_KEY).sortedIndex,o=t.length-1;if(i>0&&e.shiftKey||i<o&&!e.shiftKey){e.preventDefault(),e.shiftKey?i--:i++;var a=this._getAppointmentByIndex(i);this._resetTabIndex(a),r.default.trigger(a,"focus")}};return(0,g.extend)(e,{escape:function(){this.moveAppointmentBack(),this._escPressed=!0}.bind(this),del:function(e){if(this.option("allowDelete")){e.preventDefault();var t=this._getItemData(e.target);this.notifyObserver("onDeleteButtonPress",{data:t,target:e.target})}}.bind(this),tab:t})},_getAppointmentByIndex:function(e){var t=this._getAccessAppointments();return t.filter(function(t,n){return(0,s.data)(n,I.APPOINTMENT_SETTINGS_KEY).sortedIndex===e}).eq(0)},_getAccessAppointments:function(){return this._itemElements().filter(":visible").not(".dx-state-disabled")},_resetTabIndex:function(e){this._focusTarget().attr("tabIndex",-1),e.attr("tabIndex",this.option("tabIndex"))},_moveFocus:c.noop,_focusTarget:function(){return this._itemElements()},_renderFocusTarget:function(){var e=this._getAppointmentByIndex(0);this._resetTabIndex(e)},_focusInHandler:function(e){this.callBase.apply(this,arguments),this._$currentAppointment=(0,o.default)(e.target),this.option("focusedElement",(0,_.getPublicElement)((0,o.default)(e.target)))},_focusOutHandler:function(){var e=this._getAppointmentByIndex(0);this.option("focusedElement",(0,_.getPublicElement)(e)),this.callBase.apply(this,arguments)},_eventBindingTarget:function(){return this._itemContainer()},_getDefaultOptions:function(){return(0,g.extend)(this.callBase(),{noDataText:null,activeStateEnabled:!0,hoverStateEnabled:!0,tabIndex:0,fixedContainer:null,allDayContainer:null,allowDrag:!0,allowResize:!0,allowAllDayResize:!0,onAppointmentDblClick:null,_collectorOffset:0})},_optionChanged:function(e){switch(e.name){case"items":this._cleanFocusState(),this._clearDropDownItems(),this._clearDropDownItemsElements(),this._repaintAppointments(e.value),this._renderDropDownAppointments(),this._attachAppointmentsEvents();break;case"fixedContainer":case"allDayContainer":case"onAppointmentDblClick":break;case"allowDrag":case"allowResize":case"allowAllDayResize":this._invalidate();break;case"focusedElement":this._resetTabIndex((0,o.default)(e.value)),this.callBase(e);break;case"allowDelete":break;case"focusStateEnabled":this._clearDropDownItemsElements(),this._renderDropDownAppointments(),this.callBase(e);break;default:this.callBase(e)}},_isAllDayAppointment:function(e){return e.settings.length&&e.settings[0].allDay||!1},_isRepaintAppointment:function(e){return!(0,d.isDefined)(e.needRepaint)||e.needRepaint===!0},_isRepaintAll:function(e){if(this.invoke("isVirtualScrolling"))return!0;if(this.invoke("isCurrentViewAgenda"))return!0;for(var t=0;t<e.length;t++){var n=e[t];if(!this._isRepaintAppointment(n))return!1}return!0},_applyFragment:function(e,t){e.children().length>0&&this._getAppointmentContainer(t).append(e)},_onEachAppointment:function(e,t,n,i){var o=this,a=function(){e.needRepaint=!1,o._clearItem(e),o._renderItem(t,e,n)};(null===e||void 0===e?void 0:e.needRemove)===!0?this._clearItem(e):(i||this._isRepaintAppointment(e))&&a()},_repaintAppointments:function(e){var t=this;this._renderByFragments(function(n,i){var o=t._isRepaintAll(e);o&&(t._getAppointmentContainer(!0).html(""),t._getAppointmentContainer(!1).html("")),!e.length&&t._cleanItemContainer(),e.forEach(function(e,a){var r=t._isAllDayAppointment(e)?i:n;t._onEachAppointment(e,a,r,o)})})},_renderByFragments:function(e){var t=this.invoke("isVirtualScrolling");if(t){var n=(0,o.default)(a.default.createDocumentFragment()),i=(0,o.default)(a.default.createDocumentFragment());e(n,i),this._applyFragment(n,!1),this._applyFragment(i,!0)}else e(this._getAppointmentContainer(!1),this._getAppointmentContainer(!0))},_attachAppointmentsEvents:function(){this._attachClickEvent(),this._attachHoldEvent(),this._attachContextMenuEvent(),this._attachAppointmentDblClick(),this._renderFocusState(),this._attachFeedbackEvents(),this._attachHoverEvents()},_clearItem:function(e){var t=this._findItemElementByItem(e.itemData);t.length&&(0,h.each)(t,function(e,t){t.detach(),t.remove()})},_clearDropDownItems:function(){this._virtualAppointments={}},_clearDropDownItemsElements:function(){this.invoke("clearCompactAppointments")},_findItemElementByItem:function(e){var t=[],n=this;return this.itemElements().each(function(){var i=(0,o.default)(this);i.data(n._itemDataKey())===e&&t.push(i)}),t},_itemClass:function(){return O},_itemContainer:function(){var e=this.callBase(),t=e,n=this.option("allDayContainer");return n&&(t=e.add(n)),t},_cleanItemContainer:function(){this.callBase();var e=this.option("allDayContainer");e&&e.empty(),this._virtualAppointments={}},_clean:function(){this.callBase(),delete this._$currentAppointment,delete this._initialSize,delete this._initialCoordinates},_init:function(){this.callBase(),this.$element().addClass(T),this._preventSingleAppointmentClick=!1},_renderAppointmentTemplate:function(e,t,n){var i=this.invoke("getTextAndFormatDate",n.appointmentData,n.appointmentData.settings||n.targetedAppointmentData,"TIME");(0,o.default)("<div>").text(i.text).addClass(E).appendTo(e),(0,d.isPlainObject)(t)&&t.html&&e.html(t.html);var a=(0,o.default)("<div>").addClass(A);(0,o.default)("<div>").addClass(P).text(i.formatDate).appendTo(a),a.appendTo(e),t.recurrenceRule&&(0,o.default)("<span>").addClass(R+" dx-icon-repeat").appendTo(e),t.allDay&&(0,o.default)("<div>").text(" "+C.default.format("dxScheduler-allDay")+": ").addClass(M).prependTo(a)},_executeItemRenderAction:function(e,t,n){var i=this._getItemRenderAction();i&&i(this.invoke("mapAppointmentFields",{itemData:t,itemElement:n})),delete this._currentAppointmentSettings},_itemClickHandler:function(e){this.callBase(e,{},{afterExecute:function(e){this._processItemClick(e.args[0].event)}.bind(this)})},_processItemClick:function(e){var t=(0,o.default)(e.currentTarget),n=this._getItemData(t);
return"keydown"===e.type||(0,x.isFakeClickEvent)(e)?void this.notifyObserver("showEditAppointmentPopup",{data:n,target:t}):void(this._appointmentClickTimeout=setTimeout(function(){!this._preventSingleAppointmentClick&&a.default.getBody().contains(t[0])&&this.notifyObserver("showAppointmentTooltip",{data:n,target:t}),this._preventSingleAppointmentClick=!1}.bind(this),300))},_extendActionArgs:function(){var e=this.callBase.apply(this,arguments);return this.invoke("mapAppointmentFields",e)},_render:function(){this.callBase.apply(this,arguments),this._attachAppointmentDblClick()},_attachAppointmentDblClick:function(){var e=this,t=e._itemSelector(),n=this._itemContainer();r.default.off(n,B,t),r.default.on(n,B,t,function(t){e._itemDXEventHandler(t,"onAppointmentDblClick",{},{afterExecute:function(t){e._dblClickHandler(t.args[0].event)}})})},_dblClickHandler:function(e){var t=(0,o.default)(e.currentTarget),n=this._getItemData(t);clearTimeout(this._appointmentClickTimeout),this._preventSingleAppointmentClick=!0,this.notifyObserver("showEditAppointmentPopup",{data:n,target:t})},_renderItem:function(e,t,n){for(var i=t.itemData,o=[],a=0;a<t.settings.length;a++){var r=t.settings[a];this._currentAppointmentSettings=r;var s=this.callBase(e,i,n);s.data(I.APPOINTMENT_SETTINGS_KEY,r),o.push(s)}return o},_getItemContent:function(e){e.data(I.APPOINTMENT_SETTINGS_KEY,this._currentAppointmentSettings);var t=this.callBase(e);return t},_createItemByTemplate:function(e,t){var n=t.itemData,i=t.container,a=t.index;return e.render({model:{appointmentData:n,targetedAppointmentData:this.invoke("getTargetedAppointmentData",n,(0,o.default)(i).parent())},container:i,index:a})},_getAppointmentContainer:function(e){var t=this.option("allDayContainer"),n=this.itemsContainer().not(t);return e&&t&&(n=t),n},_postprocessRenderItem:function(e){this._renderAppointment(e.itemElement,this._currentAppointmentSettings)},_renderAppointment:function(e,t){e.data(I.APPOINTMENT_SETTINGS_KEY,t),this._applyResourceDataAttr(e);var n=this._getItemData(e),i=this.invoke("getAppointmentGeometry",t),o=this.option("allowResize")&&(!(0,d.isDefined)(t.skipResizing)||(0,d.isString)(t.skipResizing)),a=this.option("allowDrag"),r=t.allDay;this.invoke("setCellDataCacheAlias",this._currentAppointmentSettings,i);var s=this._getAppointmentColor(e,t.groupIndex);if(t.virtual)this._processVirtualAppointment(t,e,n,s);else{var l=t.info;this._createComponent(e,b.default,{observer:this.option("observer"),data:n,geometry:i,direction:t.direction||"vertical",allowResize:o,allowDrag:a,allDay:r,reduced:t.appointmentReduced,isCompact:t.isCompact,startDate:new Date(null===l||void 0===l?void 0:l.appointment.startDate),cellWidth:this.invoke("getCellWidth"),cellHeight:this.invoke("getCellHeight"),resizableConfig:this._resizableConfig(n,t)}),s.done(function(t){t&&e.css("backgroundColor",t)})}},_applyResourceDataAttr:function(e){var t=this.invoke("getResourcesFromItem",this._getItemData(e));t&&(0,h.each)(t,function(t,n){for(var i="data-"+(0,c.normalizeKey)(t.toLowerCase())+"-",o=0;o<n.length;o++)e.attr(i+(0,c.normalizeKey)(n[o]),!0)})},_resizableConfig:function(e,t){return{area:this._calculateResizableArea(t,e),onResizeStart:function(e){if(this._$currentAppointment=(0,o.default)(e.element),this.invoke("needRecalculateResizableArea")){var t=this._calculateResizableArea(this._$currentAppointment.data(I.APPOINTMENT_SETTINGS_KEY),this._$currentAppointment.data("dxItemData"));e.component.option("area",t),e.component._renderDragOffsets(e.event)}this._initialSize={width:e.width,height:e.height},this._initialCoordinates=(0,l.locate)(this._$currentAppointment)}.bind(this),onResizeEnd:function(e){return this._escPressed?void(e.event.cancel=!0):void this._resizeEndHandler(e)}.bind(this)}},_calculateResizableArea:function(e,t){var n=this.$element().closest(".dx-scrollable-content");return this.invoke("getResizableAppointmentArea",{coordinates:{left:e.left,top:0,groupIndex:e.groupIndex},allDay:e.allDay})||n},_resizeEndHandler:function(e){var t=this.option("observer"),n=(0,o.default)(e.element),i=n.data("dxAppointmentSettings"),a=i.info,r=this._getItemData(n),s=t.createAppointmentAdapter(r).clone(),l=this._getEndResizeAppointmentStartDate(e,r,a.appointment),u=a.appointment.endDate,c=this._getDateRange(e,l,u);s.startDate=new Date(c[0]),s.endDate=new Date(c[1]),this.notifyObserver("updateAppointmentAfterResize",{target:r,data:s.clone({pathTimeZone:"fromGrid"}).source(),$appointment:n})},_getEndResizeAppointmentStartDate:function(e,t,n){var i=this.option("observer"),o=i.createAppointmentAdapter(t),a=n.startDate,r=(0,m.getRecurrenceProcessor)(),s=o.recurrenceRule,l=o.startDateTimeZone,u=this.invoke("isAllDay",t),c=r.isValidRecurrenceRule(s);return e.handles.top||c||u||(a=i.timeZoneCalculator.createDate(o.startDate,{appointmentTimeZone:l,path:"toGrid"})),a},_getDateRange:function(e,t,n){var i,o,a=this._getItemData(e.element),r=this.invoke("getDeltaTime",e,this._initialSize,a),s=this.invoke("getRenderingStrategyDirection"),l=!1,u=this.invoke("isAllDay",a),c=this.invoke("needCorrectAppointmentDates")&&!u;return l="vertical"!==s||u?this.option("rtlEnabled")?e.handles.right:e.handles.left:e.handles.top,l?(i=c?this._correctStartDateByDelta(t,r):t.getTime()-r,i+=D.default.getTimezoneOffsetChangeInMs(t,n,i,n),o=n.getTime()):(i=t.getTime(),o=c?this._correctEndDateByDelta(n,r):n.getTime()+r,o-=D.default.getTimezoneOffsetChangeInMs(t,n,t,o)),[i,o]},_correctEndDateByDelta:function(e,t){var n=this.invoke("getEndDayHour"),i=this.invoke("getStartDayHour"),o=e.getTime()+t,a=(n-i)*V("hour"),r=t>0?Math.ceil(t/a):Math.floor(t/a),s=new Date(e),l=new Date(e);if(l.setHours(i,0,0,0),s.setHours(n,0,0,0),o>s.getTime()||o<=l.getTime()){var u=s.getTime()-e.getTime(),c=t-u,d=new Date(e.setDate(e.getDate()+r));d.setHours(i,0,0,0),o=d.getTime()+c-a*(r-1)}return o},_correctStartDateByDelta:function(e,t){var n=this.invoke("getEndDayHour"),i=this.invoke("getStartDayHour"),o=e.getTime()-t,a=(n-i)*V("hour"),r=t>0?Math.ceil(t/a):Math.floor(t/a),s=new Date(e),l=new Date(e);if(l.setHours(i,0,0,0),s.setHours(n,0,0,0),o<l.getTime()||o>=s.getTime()){var u=e.getTime()-l.getTime(),c=t-u,d=new Date(e.setDate(e.getDate()-r));d.setHours(n,0,0,0),o=d.getTime()-c+a*(r-1)}return o},_tryGetAppointmentColor:function(e){var t=(0,o.default)(e).data(I.APPOINTMENT_SETTINGS_KEY);if(t)return this._getAppointmentColor(e,t.groupIndex)},_getAppointmentColor:function(e,t){var n=new S.Deferred,i=this.invoke("getAppointmentColor",{itemData:this._getItemData(e),groupIndex:t});return i.done(function(e){return n.resolve(e)}),n.promise()},_calculateBoundOffset:function(){return this.invoke("getBoundOffset")},_virtualAppointments:{},_processVirtualAppointment:function(e,t,n,i){var o=e.virtual,a=o.index;(0,d.isDefined)(this._virtualAppointments[a])||(this._virtualAppointments[a]={coordinates:{top:o.top,left:o.left},items:{data:[],colors:[],settings:[]},isAllDay:!!o.isAllDay,buttonColor:i}),e.targetedAppointmentData=this.invoke("getTargetedAppointmentData",n,t),this._virtualAppointments[a].items.settings.push(e),this._virtualAppointments[a].items.data.push(n),this._virtualAppointments[a].items.colors.push(i),t.remove()},_renderContentImpl:function(){this.callBase(),this._renderDropDownAppointments()},_renderDropDownAppointments:function(){var e=this;this._renderByFragments(function(t,n){(0,h.each)(e._virtualAppointments,function(e){var i=this._virtualAppointments[e],o=i.items,a=i.coordinates,r=i.isAllDay?n:t,s=a.left,l=this.invoke("getDropDownAppointmentWidth",i.isAllDay),u=this.invoke("getDropDownAppointmentHeight"),c=this.option("rtlEnabled")?l:0;this.notifyObserver("renderCompactAppointments",{$container:r,coordinates:{top:a.top,left:s+c},items:o,buttonColor:i.buttonColor,width:l-this.option("_collectorOffset"),height:u,onAppointmentClick:this.option("onItemClick"),allowDrag:this.option("allowDrag"),cellWidth:this.invoke("getCellWidth"),isCompact:this.invoke("isAdaptive")||this._isGroupCompact(i),applyOffset:!i.isAllDay&&this.invoke("isApplyCompactAppointmentOffset")})}.bind(e))})},_isGroupCompact:function(e){return!e.isAllDay&&this.invoke("supportCompactDropDownAppointments")},_sortAppointmentsByStartDate:function(e){e.sort(function(e,t){var n=0,i=new Date(this.invoke("getField","startDate",e.settings||e)).getTime(),o=new Date(this.invoke("getField","startDate",t.settings||t)).getTime();return i<o&&(n=-1),i>o&&(n=1),n}.bind(this))},_processRecurrenceAppointment:function(e,t,n){var i=this.invoke("getField","recurrenceRule",e),o={parts:[],indexes:[]};if(i){for(var a=e.settings||e,r=new Date(this.invoke("getField","startDate",a)),s=new Date(this.invoke("getField","endDate",a)),l=s.getTime()-r.getTime(),u=this.invoke("getField","recurrenceException",e),c=this.invoke("getStartViewDate"),d=this.invoke("getEndViewDate"),h=(0,m.getRecurrenceProcessor)().generateDates({rule:i,exception:u,start:r,end:s,min:c,max:d}),f=e.settings?1:h.length,p=0;p<f;p++){var _=(0,g.extend)({},e,!0);if(h[p]){var v=this._applyStartDateToObj(h[p],{});this._applyEndDateToObj(new Date(h[p].getTime()+l),v),_.settings=v}else _.settings=a;o.parts.push(_),n||this._processLongAppointment(_,o)}o.indexes.push(t)}return o},_processLongAppointment:function(e,t){var n=this.splitAppointmentByDay(e),i=n.length,o=this.invoke("getEndViewDate").getTime(),a=this.invoke("getStartViewDate").getTime(),r=this.invoke("getTimeZoneCalculator");if(t=t||{parts:[]},i>1){(0,g.extend)(e,n[0]);for(var s=1;s<i;s++){var l=this.invoke("getField","startDate",n[s].settings).getTime();l=r.createDate(l,{path:"toGrid"}),l<o&&l>a&&t.parts.push(n[s])}}return t},_reduceRecurrenceAppointments:function(e,t){(0,h.each)(e,function(e,n){t.splice(n-e,1)})},_combineAppointments:function(e,t){t.length&&(0,p.merge)(e,t),this._sortAppointmentsByStartDate(e)},_applyStartDateToObj:function(e,t){return this.invoke("setField","startDate",t,e),t},_applyEndDateToObj:function(e,t){return this.invoke("setField","endDate",t,e),t},moveAppointmentBack:function(e){var t=this._$currentAppointment,n=this._initialSize,i=this._initialCoordinates;e&&(this._removeDragSourceClassFromDraggedAppointment(),(0,d.isDeferred)(e.cancel)?e.cancel.resolve(!0):e.cancel=!0),t&&!e&&(i&&((0,l.move)(t,i),delete this._initialSize),n&&(t.outerWidth(n.width),t.outerHeight(n.height),delete this._initialCoordinates))},focus:function(){if(this._$currentAppointment){var e=(0,_.getPublicElement)(this._$currentAppointment);this.option("focusedElement",e),r.default.trigger(e,"focus")}},splitAppointmentByDay:function(e){var t=e.settings||e,n=new Date(this.invoke("getField","startDate",t)),i=u.default.makeDate(n),o=u.default.makeDate(this.invoke("getField","endDate",t)),a=this.invoke("getEndViewDate"),r=this.invoke("getStartDayHour"),s=this.invoke("getEndDayHour"),l=this.invoke("appointmentTakesSeveralDays",e),c=[],d=this.invoke("getTimeZoneCalculator");for(i=d.createDate(i,{path:"toGrid"}),o=d.createDate(o,{path:"toGrid"}),i.getHours()<=s&&i.getHours()>=r&&!l&&(c.push(this._applyStartDateToObj(new Date(i),{appointmentData:e})),i.setDate(i.getDate()+1));l&&i.getTime()<o.getTime()&&i<a;){var h=new Date(i),p=new Date(i);this._checkStartDate(h,n,r),this._checkEndDate(p,o,s);var g=(0,f.deepExtendArraySafe)({},e,!0),_={};this._applyStartDateToObj(h,_),this._applyEndDateToObj(p,_),g.settings=_,c.push(g),i=u.default.trimTime(i),i.setDate(i.getDate()+1),i.setHours(r)}return c},_checkStartDate:function(e,t,n){!u.default.sameDate(e,t)||e.getHours()<=n?e.setHours(n,0,0,0):e.setHours(t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds())},_checkEndDate:function(e,t,n){!u.default.sameDate(e,t)||e.getHours()>n?e.setHours(n,0,0,0):e.setHours(t.getHours(),t.getMinutes(),t.getSeconds(),t.getMilliseconds())},_removeDragSourceClassFromDraggedAppointment:function(){var e=this._itemElements().filter(".".concat(I.APPOINTMENT_DRAG_SOURCE_CLASS));e.each(function(e,t){var n=(0,o.default)(t).dxSchedulerAppointment("instance");n.option("isDragSource",!1)})},_setDragSourceAppointment:function(e,t){var n=this._findItemElementByItem(e),i=t.info.sourceAppointment,o=i.startDate,a=i.endDate,r=t.groupIndex;n.forEach(function(e){var t=e.data(I.APPOINTMENT_SETTINGS_KEY),n=t.info,i=t.groupIndex,s=n.sourceAppointment,l=s.startDate,u=s.endDate,c=e.dxSchedulerAppointment("instance"),d=o.getTime()===l.getTime()&&a.getTime()===u.getTime()&&r===i;c.option("isDragSource",d)})}}).include(y.default);(0,v.default)("dxSchedulerAppointments",F);var L=F;t.default=L,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(30),s=n(147),l=n(0),u=i(n(9)),c=n(808),d=i(n(225)),h=n(8),f=i(n(25)),p=i(n(65)),g=i(n(188)),_=i(n(10)),m=i(n(32)),v=n(169),y="left right",b="top bottom",x=(0,h.addNamespace)(f.default.enter,"dxSchedulerAppointment"),w=(0,h.addNamespace)(f.default.leave,"dxSchedulerAppointment"),C=p.default.inherit({_getDefaultOptions:function(){return(0,l.extend)(this.callBase(),{data:{},geometry:{top:0,left:0,width:0,height:0},allowDrag:!0,allowResize:!0,reduced:null,isCompact:!1,direction:"vertical",resizableConfig:{},cellHeight:0,cellWidth:0,isDragSource:!1})},_optionChanged:function(e){switch(e.name){case"data":case"geometry":case"allowDrag":case"allowResize":case"reduced":case"sortedIndex":case"isCompact":case"direction":case"resizableConfig":case"cellHeight":case"cellWidth":this._invalidate();break;case"isDragSource":this._renderDragSourceClass();break;default:this.callBase(e)}},_getHorizontalResizingRule:function(){var e={head:this.option("rtlEnabled")?"right":"left",body:"",tail:this.option("rtlEnabled")?"left":"right"};return{handles:this.option("reduced")?e[this.option("reduced")]:y,minHeight:0,minWidth:this.invoke("getCellWidth"),step:this.invoke("getResizableStep"),roundStepValue:!1}},_getVerticalResizingRule:function(){var e=this.invoke("getCellHeight");return{handles:b,minWidth:0,minHeight:e,step:e,roundStepValue:!0}},_render:function(){this.callBase(),this._renderAppointmentGeometry(),this._renderEmptyClass(),this._renderReducedAppointment(),this._renderAllDayClass(),this._renderDragSourceClass(),this._renderDirection(),this.$element().data("dxAppointmentStartDate",this.option("startDate")),this.$element().attr("title",this.invoke("getField","text",this.option("data"))),this.$element().attr("role","button"),this._renderRecurrenceClass(),this._renderResizable()},_renderAppointmentGeometry:function(){var e=this.option("geometry"),t=this.$element();(0,r.move)(t,{top:e.top,left:e.left}),t.css({width:e.width<0?0:e.width,height:e.height<0?0:e.height})},_renderEmptyClass:function(){var e=this.option("geometry");(e.empty||this.option("isCompact"))&&this.$element().addClass(v.EMPTY_APPOINTMENT_CLASS)},_renderReducedAppointment:function(){var e=this.option("reduced");e&&(this.$element().toggleClass(v.REDUCED_APPOINTMENT_CLASS,!0).toggleClass(v.REDUCED_APPOINTMENT_PARTS_CLASSES[e],!0),this._renderAppointmentReducedIcon())},_renderAppointmentReducedIcon:function(){var e=(0,o.default)("<div>").addClass(v.REDUCED_APPOINTMENT_ICON).appendTo(this.$element()),t=this._getEndDate(),n=_.default.format("dxScheduler-editorLabelEndDate"),i=[n,": ",m.default.format(t,"monthAndDay"),", ",m.default.format(t,"year")].join("");a.default.off(e,x),a.default.on(e,x,function(){(0,c.show)({target:e,content:i})}),a.default.off(e,w),a.default.on(e,w,function(){(0,c.hide)()})},_getEndDate:function(){var e=this.invoke("getField","endDate",this.option("data"));return e?new Date(e):e},_renderAllDayClass:function(){this.$element().toggleClass(v.ALL_DAY_APPOINTMENT_CLASS,!!this.option("allDay"))},_renderDragSourceClass:function(){this.$element().toggleClass(v.APPOINTMENT_DRAG_SOURCE_CLASS,!!this.option("isDragSource"))},_renderRecurrenceClass:function(){var e=this.invoke("getField","recurrenceRule",this.option("data"));(0,s.getRecurrenceProcessor)().isValidRecurrenceRule(e)&&this.$element().addClass(v.RECURRENCE_APPOINTMENT_CLASS)},_renderDirection:function(){this.$element().addClass(v.DIRECTION_APPOINTMENT_CLASSES[this.option("direction")])},_createResizingConfig:function(){var e="vertical"===this.option("direction")?this._getVerticalResizingRule():this._getHorizontalResizingRule();return this.invoke("isGroupedByDate")||(e.stepPrecision="strict"),e},_renderResizable:function(){this.option("allowResize")&&this._createComponent(this.$element(),g.default,(0,l.extend)(this._createResizingConfig(),this.option("resizableConfig")))},_useTemplates:function(){return!1}}).include(d.default);(0,u.default)("dxSchedulerAppointment",C);var k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return p(),f(e),d.show()}function a(){return d?d.hide().done(p).promise():(new u.Deferred).resolve()}t.show=o,t.hide=a;var r=i(n(2)),s=i(n(453)),l=n(0),u=n(6),c=n(78),d=null,h=null,f=function(e){e=(0,l.extend)({position:"top"},e);var t=e.content;delete e.content;var n=(0,r.default)("<div>").html(t).appendTo((0,c.value)());h=function(){n.remove()},d=new s.default(n,e)},p=function(){d&&(h(),d=null)}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(4),l=i(n(810)),u=i(n(520)),c=i(n(521)),d=i(n(812)),h=i(n(813)),f={horizontal:u.default,horizontalMonth:d.default,horizontalMonthLine:c.default,vertical:l.default,agenda:h.default},p=function(){function e(t,n){o(this,e),this.instance=t,n&&this.initRenderingStrategy(n)}return r(e,[{key:"getCellDimensions",value:function(e){if(this.instance._workSpace)return{width:this.instance._workSpace.getCellWidth(),height:this.instance._workSpace.getCellHeight(),allDayHeight:this.instance._workSpace.getAllDayHeight()}}},{key:"getGroupOrientation",value:function(e){this.instance._workSpace&&e.callback(this.instance._workSpace._getRealGroupOrientation())}},{key:"initRenderingStrategy",value:function(e){var t=f[e];this._renderingStrategyInstance=new t(this.instance),this.renderingStrategy=e}},{key:"createAppointmentsMap",value:function(e){var t=this,n=this.getCellDimensions(),i=n.allDayHeight;this.instance._allDayCellHeight=i,this.getGroupOrientation({callback:function(e){return t.instance._groupOrientation=e}});var o=e?e.slice():[];return this._positionMap=this._renderingStrategyInstance.createTaskPositionMap(o),this._createAppointmentsMapCore(o,this._positionMap)}},{key:"_createAppointmentsMapCore",value:function(e,t){var n=this;return e.map(function(e,i){n._renderingStrategyInstance.keepAppointmentSettings()||delete e.settings;var o=t[i];return o.forEach(function(e){e.direction="vertical"!==n.renderingStrategy||e.allDay?"horizontal":"vertical"}),{itemData:e,settings:o,needRepaint:!0,needRemove:!1}})}},{key:"_isDataChanged",value:function(e){var t=this.instance.getUpdatedAppointment();return t===e||this.instance.getUpdatedAppointmentKeys().some(function(t){return e[t.key]===t.value})}},{key:"_isAppointmentShouldAppear",value:function(e,t){return e.needRepaint&&t.needRemove}},{key:"_isSettingChanged",value:function(e,t){if(e.length!==t.length)return!0;for(var n=0;n<e.length;n++){var i=e[n],o=t[n];if(o&&(o.sortedIndex=i.sortedIndex),!(0,s.equalByValue)(i,o))return!0}return!1}},{key:"_getAssociatedSourceAppointment",value:function(e,t){for(var n=0;n<t.length;n++){var i=t[n];if(i.itemData===e.itemData)return i}return null}},{key:"_getDeletedAppointments",value:function(e,t){for(var n=[],i=0;i<t.length;i++){var o=t[i],a=this._getAssociatedSourceAppointment(o,e);a||(o.needRemove=!0,n.push(o))}return n}},{key:"getRepaintedAppointments",value:function(e,t){var n=this;return 0===t.length||"agenda"===this.renderingStrategy?e:(e.forEach(function(e){var i=n._getAssociatedSourceAppointment(e,t);i&&(e.needRepaint=n._isDataChanged(e.itemData)||n._isSettingChanged(e.settings,i.settings)||n._isAppointmentShouldAppear(e,i))}),e.concat(this._getDeletedAppointments(e,t)))}},{key:"getRenderingStrategyInstance",value:function(){return this._renderingStrategyInstance}}]),e}(),g=p;t.default=g,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(311)),v=n(0),y=n(1),b=o(n(20)),x=o(n(86)),w=5,C=20,k=b.default.dateToMilliseconds,S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getDeltaTime",value:function(e,t,n){var i=0;if(this.isAllDay(n))i=this._getDeltaWidth(e,t)*k("day");else{var o=e.height-t.height;i=k("minute")*Math.round(o/this.getDefaultCellHeight()*this.instance.getAppointmentDurationInMinutes())}return i}},{key:"_correctCollectorCoordinatesInAdaptive",value:function(e,n){if(n)l(_(t.prototype),"_correctCollectorCoordinatesInAdaptive",this).call(this,e,n);else if(0===this._getMaxAppointmentCountPerCellByType()){var i=this.getDefaultCellHeight(),o=this.getDefaultCellWidth();e.top+=(i-this.getDropDownButtonAdaptiveSize())/2,e.left+=(o-this.getDropDownButtonAdaptiveSize())/2}}},{key:"getAppointmentGeometry",value:function(e){var n=null;return n=e.allDay?this._getAllDayAppointmentGeometry(e):this._isAdaptive()&&e.isCompact?this._getAdaptiveGeometry(e):this._getVerticalAppointmentGeometry(e),l(_(t.prototype),"getAppointmentGeometry",this).call(this,n)}},{key:"_getAdaptiveGeometry",value:function(e){var t=this._calculateGeometryConfig(e);return this._customizeCoordinates(e,t.height,t.appointmentCountPerCell,t.offset)}},{key:"_getItemPosition",value:function(e){var n=this.instance.createAppointmentAdapter(e),i=this.isAllDay(e),o=!!n.recurrenceRule,a=n.calculateStartDate("toGrid"),r=n.calculateEndDate("toGrid"),s=!x.default.isSameAppointmentDates(a,r);if(i)return l(_(t.prototype),"_getItemPosition",this).call(this,e);for(var u=this._getAppointmentCoordinates(e),c=[],d=0;d<u.length;d++){var h=u[d],f=this.calculateAppointmentHeight(e,h),p=this.calculateAppointmentWidth(e,h),g=f,m=null,y=[],w=h.vMax;if(this._isMultiViewAppointment(h,f)||s&&!o){var C=b.default.sameDate(a,h.info.appointment.startDate)||o;C&&(g=this._reduceMultiDayAppointment(f,{top:h.top,bottom:w}),y=this._getAppointmentParts({sourceAppointmentHeight:f,reducedHeight:g,width:p},h));var k=this._isMultiDayAppointment(h,f);k&&(m=C?"head":"tail")}(0,v.extend)(h,{height:g,width:p,allDay:i,appointmentReduced:m}),c=this._getAppointmentPartsPosition(y,h,c)}return c}},{key:"_isMultiDayAppointment",value:function(e,t){if(this.isVirtualScrolling){var n=this._getGroupHeight()-this._getGroupTopOffset(e);return t>n}return!1}},{key:"_isMultiViewAppointment",value:function(e,t){return t>e.vMax-e.top}},{key:"_reduceMultiDayAppointment",value:function(e,t){return e=t.bottom-Math.floor(t.top)}},{key:"_getGroupHeight",value:function(){var e=this.instance.getWorkSpace();return e.getCellHeight()*e._getRowCount()}},{key:"_getGroupTopOffset",value:function(e){var t=Math.max(0,this.instance.fire("getGroupTop",e.groupIndex)),n=this.instance.fire("getOffsetByAllDayPanel",e.groupIndex),i=e.top-t-n;return i}},{key:"_getTailHeight",value:function(e,t){if(!this.isVirtualScrolling)return e.sourceAppointmentHeight-e.reducedHeight;var n=this._getGroupTopOffset(t),i=e.sourceAppointmentHeight,o=this._getGroupHeight(),a=n+i-o;return a}},{key:"_getAppointmentParts",value:function(e,t){var n=this._getTailHeight(e,t),i=e.width,o=[],a=Math.max(0,this.instance.fire("getGroupTop",t.groupIndex)),r=this.instance.fire("isGroupedByDate")?this.getDefaultCellWidth()*this.instance.fire("getGroupCount"):this.getDefaultCellWidth(),s=t.left+r;if(n>0){var l=this.getAppointmentMinSize();n<l&&(n=l),a+=this.instance.fire("getOffsetByAllDayPanel",t.groupIndex),o.push((0,v.extend)(!0,{},t,{top:a,left:s,height:n,width:i,appointmentReduced:"tail",rowIndex:++t.rowIndex}))}return o}},{key:"_getMinuteHeight",value:function(){return this.getDefaultCellHeight()/this.instance.getAppointmentDurationInMinutes()}},{key:"_getCompactLeftCoordinate",value:function(e,t){var n=1,i=this.getDefaultCellWidth()||this.getAppointmentMinSize();return e+(n+i)*t}},{key:"_getVerticalAppointmentGeometry",value:function(e){var t=this._calculateVerticalGeometryConfig(e);return this._customizeVerticalCoordinates(e,t.width,t.appointmentCountPerCell,t.offset)}},{key:"_customizeVerticalCoordinates",value:function(e,t,n,i,o){var a=Math.max(t/n,t/e.count),r=e.height,s=e.left+e.index*a,l=e.top;return e.isCompact&&this._markAppointmentAsVirtual(e,o),{height:r,width:a,top:l,left:s,empty:this._isAppointmentEmpty(r,t)}}},{key:"_calculateVerticalGeometryConfig",value:function(e){var t=this.instance.fire("getMaxAppointmentsPerCell"),n=this._getOffsets(),i=this._getAppointmentDefaultOffset(),o=this._getAppointmentCount(t,e),a=this._getDefaultRatio(e,o),r=this._getMaxWidth();o||(o=e.count,a=(r-n.unlimited)/r);var s=(1-a)*r;return("auto"===t||(0,y.isNumeric)(t))&&(a=1,r-=i,s=0),{width:a*r,appointmentCountPerCell:o,offset:s}}},{key:"_getMaxWidth",value:function(){return this.getDefaultCellWidth()||this.invoke("getCellWidth")}},{key:"isAllDay",value:function(e){var t=this.instance.fire("getField","allDay",e);return!!t||this.instance.appointmentTakesAllDay(e)}},{key:"_getAppointmentMaxWidth",value:function(){return this.getDefaultCellWidth()-this._getAppointmentDefaultOffset()}},{key:"calculateAppointmentWidth",value:function(e,t){if(!this.isAllDay(e))return 0;var n=b.default.trimTime(t.info.appointment.startDate),i=this.normalizeEndDateByViewEnd(e,t.info.appointment.endDate),o=this.getDefaultCellWidth()||this.getAppointmentMinSize(),a=(i.getTime()-n.getTime())/k("hour"),r=Math.ceil(a/24)*o;return r=this.cropAppointmentWidth(r,o)}},{key:"calculateAppointmentHeight",value:function(e,t){if(this.isAllDay(e))return 0;var n=t.info.appointment.startDate,i=this.normalizeEndDateByViewEnd(e,t.info.appointment.endDate),o=this.instance.fire("getField","allDay",e),a=this._getAppointmentDurationInMs(n,i,o),r=this._adjustDurationByDaylightDiff(a,n,i)/k("minute"),s=r*this._getMinuteHeight();return s}},{key:"getDirection",value:function(){return"vertical"}},{key:"_sortCondition",value:function(e,t){var n=e.allDay-t.allDay,i=e.allDay&&t.allDay,o="vertical"===this.instance._groupOrientation&&i?this._columnCondition(e,t):this._rowCondition(e,t);return n?n:o}},{key:"hasAllDayAppointments",value:function(){return!0}},{key:"_getAllDayAppointmentGeometry",value:function(e){var t=this._calculateGeometryConfig(e);return this._customizeCoordinates(e,t.height,t.appointmentCountPerCell,t.offset,!0)}},{key:"_calculateGeometryConfig",value:function(e){this.instance._allowResizing()&&this.instance._allowAllDayResizing()||(e.skipResizing=!0);var n=l(_(t.prototype),"_calculateGeometryConfig",this).call(this,e);return e.count<=this._getDynamicAppointmentCountPerCell().allDay&&(n.offset=0),n}},{key:"_getAppointmentCount",value:function(e,t){return"auto"===e||1!==t.count||(0,y.isNumeric)(e)?this._getMaxAppointmentCountPerCellByType(t.allDay):t.count}},{key:"_getDefaultRatio",value:function(e,t){return e.count>this.instance.option("_appointmentCountPerCell")?.65:1}},{key:"_getOffsets",value:function(){return{unlimited:w,auto:C}}},{key:"_getMaxHeight",value:function(){return this.getDefaultAllDayCellHeight()||this.getAppointmentMinSize()}},{key:"_needVerticalGroupBounds",value:function(e){return!e}},{key:"_needHorizontalGroupBounds",value:function(){return!1}}]),t}(m.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(519)),v=28,y=40,b=35,x=30,w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getDropDownAppointmentWidth",value:function(e,t){return this.getDropDownButtonAdaptiveSize()}},{key:"getDropDownButtonAdaptiveSize",value:function(){return v}},{key:"getCollectorTopOffset",value:function(e){var t=this.getRenderingStrategy();return t.hasAllDayAppointments()&&e?(t.getDefaultAllDayCellHeight()-t.getDropDownButtonAdaptiveSize())/2:this.getRenderingStrategy().getDefaultCellHeight()-y;
}},{key:"getCollectorLeftOffset",value:function(){return(this.getRenderingStrategy().getDefaultCellWidth()-v)/2}},{key:"getAppointmentDefaultOffset",value:function(){return b}},{key:"getDynamicAppointmentCountPerCell",value:function(){var e=this.getRenderingStrategy();return e.hasAllDayAppointments()?{allDay:0,simple:this._calculateDynamicAppointmentCountPerCell()||this._getAppointmentMinCount()}:0}},{key:"getDropDownAppointmentHeight",value:function(){return v}},{key:"_getAppointmentMinCount",value:function(){return 0}},{key:"_getAppointmentDefaultWidth",value:function(){var e=this.getRenderingStrategy();return e.hasAllDayAppointments()?x:l(_(t.prototype),"_getAppointmentDefaultWidth",this).call(this)}},{key:"_calculateDynamicAppointmentCountPerCell",value:function(){return Math.floor(this.getRenderingStrategy()._getAppointmentMaxWidth()/this.getRenderingStrategy()._getAppointmentDefaultWidth())}}]),t}(m.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(521)),v=n(0),y=.6,b=26,x=30,w=36,C=60,k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getAppointmentParts",value:function(e,t,n){var i=e.sourceAppointmentWidth-e.reducedWidth,o=e.height,a=this._getFullWeekAppointmentWidth(t.groupIndex),r=this._getMaxAppointmentWidth(n),s=Math.ceil(i/a)-1,l=Math.floor(i%a),u=s?l:l||a,c=[],d=e.reducedWidth+u,h=t.top+this.getDefaultCellHeight(),f=this._calculateMultiWeekAppointmentLeftOffset(t.hMax,a);"vertical"===this.instance._groupOrientation&&(f+=this.instance.fire("getWorkSpaceDateTableOffset"));for(var p=0;p<s&&!(d>r);p++)c.push((0,v.extend)(!0,{},t,{top:h,left:f,height:o,width:a,appointmentReduced:"body",rowIndex:++t.rowIndex,cellIndex:0})),h+=this.getDefaultCellHeight(),d+=a;return u&&(this._isRtl()&&(f+=a-u),c.push((0,v.extend)(!0,{},t,{top:h,left:f,height:o,width:u,appointmentReduced:"tail",rowIndex:++t.rowIndex,cellIndex:0}))),c}},{key:"_calculateMultiWeekAppointmentLeftOffset",value:function(e,t){return this._isRtl()?e:e-t}},{key:"_getFullWeekAppointmentWidth",value:function(e){return this._maxFullWeekAppointmentWidth=this.instance.fire("getFullWeekAppointmentWidth",{groupIndex:e}),this._maxFullWeekAppointmentWidth}},{key:"_getAppointmentDefaultHeight",value:function(){return this._getAppointmentHeightByTheme()}},{key:"_getAppointmentMinHeight",value:function(){return this._getAppointmentDefaultHeight()}},{key:"_columnCondition",value:function(e,t){var n=this._getConditions(e,t);return n.rowCondition||n.columnCondition||n.cellPositionCondition}},{key:"createTaskPositionMap",value:function(e){return l(_(t.prototype),"createTaskPositionMap",this).call(this,e,!0)}},{key:"_getSortedPositions",value:function(e){return l(_(t.prototype),"_getSortedPositions",this).call(this,e,!0)}},{key:"_getDefaultRatio",value:function(){return y}},{key:"_getOffsets",value:function(){return{unlimited:b,auto:x}}},{key:"getDropDownAppointmentWidth",value:function(e){if(this.instance.fire("isAdaptive"))return this.getDropDownButtonAdaptiveSize();var t=e>1?C:w;return this.getDefaultCellWidth()-t}},{key:"needCorrectAppointmentDates",value:function(){return!1}},{key:"_needVerticalGroupBounds",value:function(){return!1}},{key:"_needHorizontalGroupBounds",value:function(){return!0}}]),t}(m.default),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(20)),_=n(3),m=n(12),v=o(n(311)),y=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"getAppointmentMinSize",value:function(){}},{key:"getDeltaTime",value:function(){}},{key:"keepAppointmentSettings",value:function(){return!0}},{key:"getAppointmentGeometry",value:function(e){return e}},{key:"createTaskPositionMap",value:function(e){var t,n;if(e.length){t=this.instance.fire("getAgendaVerticalStepHeight"),n=this.instance.fire("groupAppointmentsByResources",e);var i=[];(0,_.each)(n,function(e,t){var n=[],o=[];(0,_.each)(t,function(e,t){var i=this.instance.getAppointmentsInstance()._processRecurrenceAppointment(t,e),a=null;i.indexes.length||(a={parts:[]},a=this.instance.getAppointmentsInstance()._processLongAppointment(t),n=n.concat(a.parts)),n=n.concat(i.parts),o=o.concat(i.indexes)}.bind(this)),this.instance.getAppointmentsInstance()._reduceRecurrenceAppointments(o,t),this.instance.getAppointmentsInstance()._combineAppointments(t,n),i=i.concat(t)}.bind(this)),Array.prototype.splice.apply(e,[0,e.length].concat(i))}var o=[],a=0;return e.forEach(function(e,i){o.push([{height:t,width:"100%",sortedIndex:a++,groupIndex:this._calculateGroupIndex(i,n)}])}.bind(this)),o}},{key:"_calculateGroupIndex",value:function(e,t){var n,i=0;for(var o in t){var a=t[o].length;if(e>=i&&e<i+a){n=Number(o);break}i+=a}return n}},{key:"_getDeltaWidth",value:function(){}},{key:"_getAppointmentMaxWidth",value:function(){return this.getDefaultCellWidth()}},{key:"_needVerifyItemSize",value:function(){return!1}},{key:"_isRtl",value:function(){return this.instance.option("rtlEnabled")}},{key:"_getAppointmentParts",value:function(){}},{key:"_reduceMultiWeekAppointment",value:function(){}},{key:"calculateAppointmentHeight",value:function(){return 0}},{key:"calculateAppointmentWidth",value:function(){return 0}},{key:"isAppointmentGreaterThan",value:function(){}},{key:"isAllDay",value:function(){return!1}},{key:"_sortCondition",value:function(){}},{key:"_rowCondition",value:function(){}},{key:"_columnCondition",value:function(){}},{key:"_findIndexByKey",value:function(){}},{key:"_markAppointmentAsVirtual",value:function(){}},{key:"getDropDownAppointmentWidth",value:function(){}},{key:"getDefaultCellWidth",value:function(){return this._defaultWidth}},{key:"getCollectorLeftOffset",value:function(){}},{key:"getCollectorTopOffset",value:function(){}},{key:"calculateRows",value:function(e,t,n,i){this._rows=[];var o=this.instance.fire("groupAppointmentsByResources",e);return n=g.default.trimTime(new Date(n)),(0,_.each)(o,function(e,o){var a=[],r={indexes:[],parts:[]};if(!o.length)return this._rows.push([]),!0;(0,_.each)(o,function(e,t){var n=this.instance.fire("getField","startDate",t),o=this.instance.fire("getField","endDate",t);this.instance.fire("replaceWrongEndDate",t,n,o),i&&delete t.settings;var a=this.instance.getAppointmentsInstance()._processRecurrenceAppointment(t,e,!1);r.parts=r.parts.concat(a.parts),r.indexes=r.indexes.concat(a.indexes)}.bind(this)),this.instance.getAppointmentsInstance()._reduceRecurrenceAppointments(r.indexes,o),(0,m.merge)(o,r.parts);for(var s=o.length,l=0;l<t;l++){var u=new Date(n);u.setMilliseconds(u.getMilliseconds()+864e5*l),void 0===a[l]&&(a[l]=0);for(var c=0;c<s;c++){var d=o[c].settings||o[c],h=this.instance.fire("appointmentTakesSeveralDays",o[c]),f=this.instance.fire("getField","recurrenceRule",o[c]);(this.instance.fire("dayHasAppointment",u,d,!0)||!f&&h&&this.instance.fire("dayHasAppointment",u,o[c],!0))&&(a[l]+=1)}}this._rows.push(a)}.bind(this)),this._rows}},{key:"_iterateRow",value:function(e,t,n){for(var i=0;i<e.length;i++)if(t.counter=t.counter+e[i],t.counter>=n){t.indexInRow=i;break}}},{key:"getDateByIndex",value:function(e,t,n){var i={counter:0,indexInRow:0};e++;for(var o=0;o<t.length&&(this._iterateRow(t[o],i,e),!i.indexInRow);o++);return new Date(new Date(n).setDate(n.getDate()+i.indexInRow))}},{key:"getAppointmentDataCalculator",value:function(){return function(e,t){var n=e.index(),i=this.instance.getStartViewDate(),o=this.getDateByIndex(n,this._rows,i),a=new Date(t);return{startDate:new Date(o.setHours(a.getHours(),a.getMinutes(),a.getSeconds(),a.getMilliseconds()))}}.bind(this)}}]),t}(v.default),b=y;t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(28)),l=n(3),u=i(n(62)),c=n(147),d=i(n(20)),h=n(4),f=n(1),p=n(12),g=n(0),_=i(n(47)),m=n(6),v=d.default.dateToMilliseconds,y=0,b=1,x=function(){function e(t){o(this,e),this._filterRegistry=null,this._dataAccessors=t}return r(e,[{key:"isRegistered",value:function(){return!!this._filterRegistry}},{key:"clearRegistry",value:function(){delete this._filterRegistry}},{key:"make",value:function(e,t){this._filterRegistry||(this._filterRegistry={}),this._make(e).apply(this,t)}},{key:"_make",value:function(e){var t=this;switch(e){case"date":return function(e,n,i){var o=i?t._dataAccessors.getter.startDate:t._dataAccessors.expr.startDateExpr,a=i?t._dataAccessors.getter.endDate:t._dataAccessors.expr.endDateExpr,r=t._dataAccessors.expr.recurrenceRuleExpr;t._filterRegistry.date=[[[a,">",e],[o,"<",n]],"or",[r,"startswith","freq"],"or",[[a,e],[o,e]]],r||t._filterRegistry.date.splice(1,2)};case"user":return function(e){t._filterRegistry.user=e}}}},{key:"combine",value:function(){var e=[];return this._filterRegistry.date&&e.push(this._filterRegistry.date),this._filterRegistry.user&&e.push(this._filterRegistry.user),e}},{key:"dateFilter",value:function(){return this._filterRegistry.date}}]),e}(),w=function(e,t,n,i,o){var a=d.default.dateTimeFromDecimal(n),r=e.getHours()>=a.hours&&e.getMinutes()>=a.minutes||t.getHours()===a.hours&&t.getMinutes()>a.minutes||t.getHours()>a.hours||o||i;return r},C=function(e,t,n,i,o,a,r,s){var l,u=(24-i+n)*v("hour"),c=t.getTime()-e.getTime(),h=(u-c)/v("hour"),f=e.getHours(),p=e.getMinutes(),g=d.default.dateTimeFromDecimal(i),_=d.default.dateTimeFromDecimal(n);return l=f<g.hours||f===g.hours&&p<g.minutes||o&&e<=r||a&&e<r&&t>s&&(f<g.hours||60*t.getHours()+t.getMinutes()>60*_.hours),c<u&&f>g.hours&&p>g.minutes&&h<=f-i&&(l=!1),l},k=function(){function e(t,n,i){o(this,e),this.setDataAccessors(n),this.setDataSource(t),this._updatedAppointmentKeys=[],this._filterMaker=new x(n),this._baseAppointmentDuration=i}return r(e,[{key:"_createFilter",value:function(e,t,n,i){this._filterMaker.make("date",[e,t]);var o=this._excessFiltering()?this._dataSource.filter()[b]:this._dataSource.filter();this._filterMaker.make("user",[o]),n&&this._dataSource.filter(this._combineRemoteFilter(i))}},{key:"_excessFiltering",value:function(){var e=this._filterMaker.dateFilter(),t=this._dataSource.filter();return t&&((0,h.equalByValue)(t,e)||t.length&&(0,h.equalByValue)(t[y],e))}},{key:"_combineFilter",value:function(){return this._filterMaker.combine()}},{key:"_getStoreKey",value:function(e){var t=this._dataSource.store();return t.keyOf(e)}},{key:"_filterAppointmentByResources",value:function(e,t){var n,i,o,a=this,r=!1,s=function(){var i,r=a._dataAccessors.getter.resources[o];(0,f.isFunction)(r)&&(i=r(e));for(var s=(0,p.wrapToArray)(i),u=(0,l.map)(t[n].items,function(e){return e.id}),c=0,d=s.length;c<d;c++)if((0,p.inArray)(s[c],u)>-1)return!0;return!1};for(n=0,i=t.length;n<i;n++)if(o=t[n].name,r=s.call(this),!r)return!1;return r}},{key:"_filterAppointmentByRRule",value:function(e,t,n,i,o,a){var r=e.recurrenceRule,s=e.recurrenceException,l=e.allDay,u=!0,d=e.startDate,h=e.endDate,f=(0,c.getRecurrenceProcessor)();if(l||this._appointmentPartInInterval(d,h,i,o)){var p=this._trimDates(t,n);t=p.min,n=new Date(p.max.getTime()-v("minute"))}return r&&!f.isValidRecurrenceRule(r)&&(u=h>t&&d<=n),u&&f.isValidRecurrenceRule(r)&&(u=f.hasRecurrence({rule:r,exception:s,start:d,end:h,min:t,max:n,firstDayOfWeek:a})),u}},{key:"_appointmentPartInInterval",value:function(e,t,n,i){var o=e.getHours(),a=t.getHours();return o<=n&&a<=i&&a>=n||a>=i&&o<=i&&o>=n}},{key:"_createCombinedFilter",value:function(e,t){var n=this._dataAccessors,i=new Date(e.min),o=new Date(e.max),a=e.recurrenceException,r=e.startDayHour,s=e.endDayHour,l=e.viewStartDayHour,u=e.viewEndDayHour,c=e.resources,d=e.firstDayOfWeek,h=this;return[[function(p){var g,_=!0,m=new Date(n.getter.startDate(p)),v=new Date(n.getter.endDate(p)),y=h.appointmentTakesAllDay(p,l,u),b=h.appointmentTakesSeveralDays(p),x=n.getter.allDay(p),k=b||y,S=(0,f.isDefined)(n.getter.recurrenceRule);S&&(g=n.getter.recurrenceRule(p)),c&&c.length&&(_=h._filterAppointmentByResources(p,c)),y&&e.allDay===!1&&(_=!1);var D=n.getter.startDateTimeZone(p),I=n.getter.endDateTimeZone(p),T=t.createDate(m,{appointmentTimeZone:D,path:"toGrid"}),O=t.createDate(v,{appointmentTimeZone:I,path:"toGrid"});if(_&&S){var E=a?a(p):n.getter.recurrenceException(p);_=h._filterAppointmentByRRule({startDate:T,endDate:O,recurrenceRule:g,recurrenceException:E,allDay:y},i,o,r,s,d)}return _&&O<i&&k&&!x&&(!S||S&&!g)&&(_=!1),!_||void 0===r||S&&e.isVirtualScrolling||(_=w(T,O,r,y,b)),_&&void 0!==s&&(_=C(T,O,r,s,y,b,o,i)),_&&S&&!g&&O<i&&!x&&(_=!1),_}]]}},{key:"setDataSource",value:function(e){this._dataSource=e,this.cleanModelState(),this._initStoreChangeHandlers(),this._filterMaker&&this._filterMaker.clearRegistry()}},{key:"_initStoreChangeHandlers",value:function(){var e=this,t=this._dataSource,n=null===t||void 0===t?void 0:t.store();n&&(n.on("updating",function(t){e._updatedAppointment=t}),n.on("push",function(i){var o=t.items(),a=n.key();i.forEach(function(t){var n=0!==o.filter(function(e){return e[a]===t.key}).length;n?e._updatedAppointmentKeys.push({key:a,value:t.key}):o.push(t.data)})}))}},{key:"getUpdatedAppointment",value:function(){return this._updatedAppointment}},{key:"getUpdatedAppointmentKeys",value:function(){return this._updatedAppointmentKeys}},{key:"cleanModelState",value:function(){this._updatedAppointment=null,this._updatedAppointmentKeys=[]}},{key:"setDataAccessors",value:function(e){this._dataAccessors=e,this._filterMaker=new x(e)}},{key:"filterByDate",value:function(e,t,n,i){if(this._dataSource){var o=this._trimDates(e,t);if(this._filterMaker.isRegistered()){var a;if(this._filterMaker.make("date",[o.min,o.max]),(null===(a=this._dataSource.filter())||void 0===a?void 0:a.length)>1){var r=this._serializeRemoteFilter([this._dataSource.filter()[1]],i);this._filterMaker.make("user",r)}n&&this._dataSource.filter(this._combineRemoteFilter(i))}else this._createFilter(o.min,o.max,n,i)}}},{key:"_combineRemoteFilter",value:function(e){var t=this._filterMaker.combine();return this._serializeRemoteFilter(t,e)}},{key:"_serializeRemoteFilter",value:function(e,t){if(!Array.isArray(e))return e;e=(0,g.extend)([],e);var n=this._dataAccessors.expr.startDateExpr,i=this._dataAccessors.expr.endDateExpr;(0,f.isString)(e[0])&&(0,s.default)().forceIsoDateParsing&&e.length>1&&(e[0]!==n&&e[0]!==i||(e[e.length-1]=u.default.serializeDate(new Date(e[e.length-1]),t)));for(var o=0;o<e.length;o++)e[o]=this._serializeRemoteFilter(e[o],t);return e}},{key:"_createAppointmentFilter",value:function(e,t){var n=this._createCombinedFilter(e,t);if(this._filterMaker.isRegistered()){this._filterMaker.make("user",void 0);var i=this._trimDates(e.min,e.max);this._filterMaker.make("date",[i.min,i.max,!0]);var o=this.customizeDateFilter(this._filterMaker.combine(),t);n.push([o])}return n}},{key:"filterLoadedAppointments",value:function(e,t){var n=this._createAppointmentFilter(e,t);return(0,_.default)(this.getPreparedDataItems()).filter(n).toArray()}},{key:"getPreparedDataItems",value:function(){var e=this,t=this._dataSource.items();return(0,l.map)(t,function(t){var n=new Date(e._dataAccessors.getter.startDate(t)),i=new Date(e._dataAccessors.getter.endDate(t));return e.replaceWrongEndDate(t,n,i),t})}},{key:"replaceWrongEndDate",value:function(e,t,n){if(this._isEndDateWrong(t,n)){var i=this._dataAccessors.getter.allDay(e),o=this._calculateAppointmentEndDate(i,t);this._dataAccessors.setter.endDate(e,o)}}},{key:"filterLoadedVirtualAppointments",value:function(e,t,n){var i=this,o=[],a=this.getPreparedDataItems(),r=n>0;return r&&(a=a.filter(function(t){for(var n=0;n<e.length;++n){var o=e[n].resources;if(i._filterAppointmentByResources(t,o))return!0}})),e.forEach(function(e){o.length&&o.push("or");var n=i._createAppointmentFilter(e,t);o.push(n)}),(0,_.default)(a).filter(o).toArray()}},{key:"_trimDates",value:function(e,t){var n=d.default.trimTime(new Date(e)),i=d.default.trimTime(new Date(t));return i.setDate(i.getDate()+1),{min:n,max:i}}},{key:"hasAllDayAppointments",value:function(e,t,n){if(!e)return!1;var i=this,o=!1;return(0,l.each)(e,function(e,a){if(i.appointmentTakesAllDay(a,t,n))return o=!0,!1}),o}},{key:"appointmentTakesAllDay",value:function(e,t,n){var i=this._dataAccessors,o=i.getter.startDate(e),a=i.getter.endDate(e),r=i.getter.allDay(e);return r||this._appointmentHasAllDayDuration(o,a,t,n)}},{key:"_appointmentHasAllDayDuration",value:function(e,t,n,i){e=new Date(e),t=new Date(t);var o=24,a=this._getAppointmentDurationInHours(e,t);return a>=o||this._appointmentHasShortDayDuration(e,t,n,i)}},{key:"_appointmentHasShortDayDuration",value:function(e,t,n,i){var o=this._getAppointmentDurationInHours(e,t),a=i-n;return o>=a&&e.getHours()===n&&t.getHours()===i}},{key:"_getAppointmentDurationInHours",value:function(e,t){return(t.getTime()-e.getTime())/v("hour")}},{key:"appointmentTakesSeveralDays",value:function(e){var t=this._dataAccessors,n=new Date(t.getter.startDate(e)),i=new Date(t.getter.endDate(e));return!d.default.sameDate(n,i)}},{key:"customizeDateFilter",value:function(e,t){var n=this,i=(0,g.extend)(!0,[],e);return function(e){var o=new Date(n._dataAccessors.getter.startDate(e)),a=new Date(n._dataAccessors.getter.endDate(e));e=(0,g.extend)(!0,{},e);var r=n._dataAccessors.getter.startDateTimeZone(e),s=n._dataAccessors.getter.endDateTimeZone(e),l=t.createDate(o,{appointmentTimeZone:r,path:"toGrid"}),u=t.createDate(a,{appointmentTimeZone:s,path:"toGrid"});return n._dataAccessors.setter.startDate(e,l),n._dataAccessors.setter.endDate(e,u),(0,_.default)([e]).filter(i).toArray().length>0}.bind(this)}},{key:"_calculateAppointmentEndDate",value:function(e,t){return e?d.default.setToDayEnd(new Date(t)):new Date(t.getTime()+this._baseAppointmentDuration*v("minute"))}},{key:"_isEndDateWrong",value:function(e,t){return!t||isNaN(t.getTime())||e.getTime()>t.getTime()}},{key:"add",value:function(e){var t=this;return this._dataSource.store().insert(e).done(function(){t._dataSource.load()}.bind(this))}},{key:"update",value:function(e,t){var n=this,i=this._getStoreKey(e),o=new m.Deferred;return this._dataSource.store().update(i,t).done(function(){n._dataSource.load().done(o.resolve).fail(o.reject)}).fail(o.reject),o.promise()}},{key:"remove",value:function(e){var t=this,n=this._getStoreKey(e);return this._dataSource.store().remove(n).done(function(){t._dataSource.load()}.bind(this))}},{key:"keyName",get:function(){var e=this._dataSource.store();return e.key()}}]),e}(),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(1),r=n(4),s=n(0),l=n(3),u=n(12),c=n(40),d=i(n(9)),h=i(n(16)),f=i(n(225)),p=i(n(816)),g=i(n(219)),_=i(n(220)),m=n(440),v=i(n(22)),y=i(n(10)),b="dx-scheduler-header",x="dx-scheduler-view-switcher",w="dx-scheduler-view-switcher-label",C={day:"day",week:"week",workWeek:"workWeek",month:"month",timelineDay:"day",timelineWeek:"week",timelineWorkWeek:"workWeek",timelineMonth:"month",agenda:"agenda"},k=["day","week","workWeek","month","timelineDay","timelineWeek","timelineWorkWeek","timelineMonth","agenda"],S=h.default.inherit({_getDefaultOptions:function(){return(0,s.extend)(this.callBase(),{views:[],isAdaptive:!1,intervalCount:1,currentView:"day",firstDayOfWeek:void 0,currentDate:new Date,min:void 0,max:void 0,useDropDownViewSwitcher:!1,_dropDownButtonIcon:"overlay"})},_setOptionsByReference:function(){this.callBase(),(0,s.extend)(this._optionsByReference,{currentView:!0})},_optionChanged:function(e){var t=e.value;switch(e.name){case"views":this._validateViews(),this._viewSwitcher.option({items:t,selectedItem:this.option("currentView")});break;case"customizeDateNavigatorText":this._navigator.option(e.name,t);break;case"currentView":this._viewSwitcher.option("selectedItem",t),this._navigator.option("step",C[this._getCurrentViewType()]),this._changeViewSwitcherLabelText();break;case"currentDate":this._navigator.option("date",t);break;case"displayedDate":this._navigator.option("displayedDate",t);break;case"min":case"max":case"firstDayOfWeek":case"intervalCount":this._navigator.option(e.name,t);break;case"tabIndex":case"focusStateEnabled":this._viewSwitcher.option(e.name,t),this._navigator.option(e.name,t),this.callBase(e);break;case"useDropDownViewSwitcher":this._refreshViewSwitcher();break;default:this.callBase(e)}},_init:function(){this.callBase(),this.$element().addClass(b)},_initMarkup:function(){this.callBase(),this._renderNavigator(),this._renderViewSwitcher()},_renderNavigator:function(){this._navigator=this._createComponent("<div>",p.default,{min:this.option("min"),max:this.option("max"),intervalCount:this.option("intervalCount"),date:this.option("currentDate"),step:C[this._getCurrentViewType()],firstDayOfWeek:this.option("firstDayOfWeek"),tabIndex:this.option("tabIndex"),focusStateEnabled:this.option("focusStateEnabled"),observer:this.option("observer"),customizeDateNavigatorText:this.option("customizeDateNavigatorText"),todayDate:this.option("todayDate")}),this._navigator.$element().appendTo(this.$element())},_renderViewSwitcher:function(){this._validateViews();var e=(0,o.default)("<div>").addClass(x).appendTo(this.$element());this.option("useDropDownViewSwitcher")?this._renderViewSwitcherDropDownMenu(e):this._renderViewSwitcherTabs(e)},_validateViews:function(){var e=this.option("views");(0,l.each)(e,function(e,t){var n=(0,a.isObject)(t),i=n&&t.type?t.type:t;(0,u.inArray)(i,k)===-1&&v.default.log("W0008",i)})},_getCurrentViewType:function(){var e=this.option("currentView");return e.type||e},_renderViewSwitcherTabs:function(e){var t=this;e.addClass(m.TABS_EXPANDED_CLASS),this._viewSwitcher=this._createComponent(e,_.default,{selectionRequired:!0,scrollingEnabled:!0,onSelectionChanged:this._updateCurrentView.bind(this),items:this.option("views"),itemTemplate:function(e){return(0,o.default)("<span>").addClass("dx-tab-text").text(t._getItemText(e))},selectedItem:this.option("currentView"),tabIndex:this.option("tabIndex"),focusStateEnabled:this.option("focusStateEnabled")})},_getItemText:function(e){return e.name||y.default.format("dxScheduler-switcher"+(0,c.camelize)(e.type||e,!0))},_refreshViewSwitcher:function(){this._viewSwitcher._dispose(),this._viewSwitcher.$element().remove(),delete this._viewSwitcher,this._removeViewSwitcherLabel(),this._renderViewSwitcher()},_removeViewSwitcherLabel:function(){(0,a.isDefined)(this._$viewSwitcherLabel)&&(this._$viewSwitcherLabel.detach(),this._$viewSwitcherLabel.remove(),delete this._$viewSwitcherLabel)},_renderViewSwitcherDropDownMenu:function(e){var t=this;this._$viewSwitcherLabel=(0,o.default)("<div>").addClass(w).appendTo(this.$element()),this._changeViewSwitcherLabelText(),this._viewSwitcher=this._createComponent(e,g.default,{onItemClick:this._updateCurrentView.bind(this),buttonIcon:this.option("_dropDownButtonIcon"),items:this.option("views"),selectionMode:this.option("isAdaptive")?"single":"none",selectedItemKeys:[this.option("currentView")],itemTemplate:function(e){return(0,o.default)("<span>").addClass("dx-dropdownmenu-item-text").text(t._getItemText(e))}})},_changeViewSwitcherLabelText:function(){if((0,a.isDefined)(this._$viewSwitcherLabel)){var e=this.option("currentView"),t=this._getItemText(e);this._$viewSwitcherLabel.text(t)}},_getCurrentViewName:function(e){return(0,a.isObject)(e)?e.name||e.type:e},_updateCurrentView:function(e){var t=e.itemData||e.component.option("selectedItem"),n=this._getCurrentViewName(t);this.notifyObserver("currentViewUpdated",n)},_renderFocusTarget:r.noop}).include(f.default);(0,d.default)("dxSchedulerHeader",S);var D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(4),r=n(1),s=i(n(17)),l=i(n(20)),u=n(0),c=i(n(9)),d=i(n(13)),h=i(n(16)),f=i(n(38)),p=i(n(290)),g=i(n(143)),_=i(n(48)),m=i(n(225)),v=i(n(32)),y=i(n(109)),b="dx-scheduler-navigator",x="dx-scheduler-navigator-calendar",w="dx-scheduler-navigator-next",C="dx-scheduler-navigator-caption",k="dx-scheduler-navigator-previous",S="dx-scheduler-navigator-calendar-popover",D=1,I=function(e){return e?D:v.default.firstDayOfWeekIndex()},T=function(e){return function(t){var n=v.default.getMonthNames(e?"abbreviated":"wide")[t.getMonth()];return[v.default.format(t,"day"),n].join(" ")}},O=function(e){return v.default.getMonthNames("abbreviated")[e.getMonth()]+" "+v.default.format(e,"year")},E=function e(t,n,i){var o=T(t);return function(a){if(n>1){var r=new Date(a),s=i;r.setDate(a.getDate()+s-1);var l=a.getMonth()!==r.getMonth(),u=l||t,c=v.default.format(a,l?T(u):"d"),d=v.default.format(r,e(u));return c+"-"+d}return[o(a),v.default.format(a,"year")].join(" ")}},A=function(e,t,n){var i=this.option("firstDayOfWeek"),o=(0,r.isDefined)(i)?i:I(t);0===o&&n&&(o=D);var a=l.default.getFirstWeekDate(e,o),s=2;n&&(a=l.default.normalizeDateByWeek(a,e)),i>=6&&n&&a.setDate(a.getDate()+(7-i+1));var u=new Date(a),c=this.option("intervalCount");return t=t||6,u=new Date(u.setDate(u.getDate()+(c>1?7*(c-1)+t:t))),u.getDay()%6===0&&n&&u.setDate(u.getDate()+s),{text:P.call(this,u,a),startDate:a,endDate:u}},P=function(e,t){var n,i,o=t.getMonth()!==e.getMonth(),a=t.getFullYear()!==e.getFullYear(),r=o||this.option("_useShortDateFormat");return a?(i=v.default.format(t,E(!0)),n=v.default.format(e,E(!0))):(i=v.default.format(t,o?T(r):"d"),n=v.default.format(e,E(r))),i+"-"+n},R=function(e){var t,n=new Date(l.default.getFirstMonthDate(e)),i=new Date(l.default.getLastMonthDate(n));if(this.option("intervalCount")>1){i=new Date(n),i.setMonth(n.getMonth()+this.option("intervalCount")-1),i=new Date(l.default.getLastMonthDate(i));var o=n.getYear()===i.getYear(),a=O(i),r=o?v.default.getMonthNames("abbreviated")[n.getMonth()]:O(n);t=r+"-"+a}else t=v.default.format(e,"monthandyear");return{text:t,startDate:n,endDate:i}},M=function(e,t){return new Date(e[this.setter](e[this.getter]()+t))},B=function(e){var t;switch(e){case"day":return{duration:1*this.option("intervalCount"),setter:"setDate",getter:"getDate",getDate:M,getCaption:function(e){var t=E(!1,this.option("intervalCount"),this._getConfig().duration);return{text:v.default.format(e,t),startDate:e,endDate:e}}};case"week":return{duration:7*this.option("intervalCount"),setter:"setDate",getter:"getDate",getDate:M,getCaption:A};case"workWeek":return{duration:7*this.option("intervalCount"),setter:"setDate",getter:"getDate",getDate:M,getCaption:function(e){return A.call(this,e,4,!0)}};case"month":return{duration:1*this.option("intervalCount"),setter:"setMonth",getter:"getMonth",getDate:function(e,t){var n=e.getDate();e.setDate(1),e=M.call(this,e,t);var i=l.default.getLastMonthDay(e);return e.setDate(n<i?n:i),e},getCaption:R};case"agenda":return t=this.invoke("getAgendaDuration"),t=(0,r.isNumeric)(t)&&t>0?t:7,{duration:t,setter:"setDate",getter:"getDate",getDate:M,getCaption:function(e){var n,i=E(this.option("_useShortDateFormat")),o=new Date(e),a=new Date(e);return t>1?(a.setDate(a.getDate()+t-1),n=P.call(this,a,e)):n=v.default.format(e,i),{text:n,startDate:o,endDate:a}}}}},V=h.default.inherit({_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{date:new Date,displayedDate:void 0,step:"day",intervalCount:1,min:void 0,max:void 0,firstDayOfWeek:void 0,_useShortDateFormat:!1,todayDate:function(){return new Date}})},_defaultOptionsRules:function(){return this.callBase().concat([{device:function(){return!d.default.real().generic||d.default.isSimulator()},options:{_useShortDateFormat:!0}}])},_optionChanged:function(e){switch(e.name){case"step":case"date":case"intervalCount":case"displayedDate":this._updateButtonsState(),this._renderCaption(),this._setCalendarOption("value",this.option("date"));break;case"min":case"max":this._updateButtonsState(),this._setCalendarOption(e.name,e.value);break;case"firstDayOfWeek":this._setCalendarOption(e.name,e.value);break;case"customizeDateNavigatorText":this._renderCaption();break;case"tabIndex":case"focusStateEnabled":this._next.option(e.name,e.value),this._caption.option(e.name,e.value),this._prev.option(e.name,e.value),this._setCalendarOption(e.name,e.value),this.callBase(e);break;case"_useShortDateFormat":
break;default:this.callBase(e)}},_init:function(){this.callBase(),this.$element().addClass(b),this._initButtons()},_initButtons:function(){var e=(0,o.default)("<div>").addClass(w);this._next=this._createComponent(e,f.default,{icon:"chevronnext",onClick:this._updateCurrentDate.bind(this,1),focusStateEnabled:this.option("focusStateEnabled"),tabIndex:this.option("tabIndex"),integrationOptions:{}});var t=(0,o.default)("<div>").addClass(C);this._caption=this._createComponent(t,f.default,{focusStateEnabled:this.option("focusStateEnabled"),tabIndex:this.option("tabIndex"),integrationOptions:{}});var n=(0,o.default)("<div>").addClass(k);this._prev=this._createComponent(n,f.default,{icon:"chevronprev",onClick:this._updateCurrentDate.bind(this,-1),focusStateEnabled:this.option("focusStateEnabled"),tabIndex:this.option("tabIndex"),integrationOptions:{}}),this.setAria("label","Next period",e),this.setAria("label","Previous period",n),this._updateButtonsState(),this.$element().append(n,t,e)},_updateButtonsState:function(){var e=this.option("min"),t=this.option("max"),n=this._getConfig().getCaption.call(this,this.option("displayedDate")||this.option("date"));e=e?l.default.trimTime(e):e,t=t?l.default.trimTime(t):t,t&&t.setHours(23,59,59),this._prev.option("disabled",e&&!isNaN(e.getTime())&&this._getNextDate(-1,n.endDate)<e),this._next.option("disabled",t&&!isNaN(t.getTime())&&this._getNextDate(1,n.startDate)>t)},_updateCurrentDate:function(e){var t=this._getNextDate(e);l.default.normalizeDate(t,this.option("min"),this.option("max")),this.notifyObserver("currentDateUpdated",t)},_getNextDate:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n=this._getConfig(),i=n.duration*e,o=n.getDate(new Date(t||this.option("date")),i);return o},_renderFocusTarget:a.noop,_initMarkup:function(){this.callBase(),this._renderCaption()},_render:function(){this.callBase(),this._renderPopover(),this._renderCaptionKeys()},_isMobileLayout:function(){return!d.default.current().generic},_renderPopover:function(){var e=this,t=this._isMobileLayout()?_.default:g.default,n=(0,o.default)("<div>").addClass(S);this._popover=this._createComponent(n,t,{contentTemplate:function(){return e._createPopupContent()},defaultOptionsRules:[{device:function(){return!d.default.current().generic},options:{fullScreen:!0,showCloseButton:!1,toolbarItems:[{shortcut:"cancel"}]}},{device:function(){return d.default.current().generic},options:{target:this._caption.$element()}}]}),this._popover.$element().appendTo(this.$element())},_createScrollable:function(e){var t=this._createComponent((0,o.default)("<div>"),y.default,{direction:"vertical"});return t.$content().append(e),t},_createPopupContent:function(){var e=(0,o.default)("<div>").addClass(x);if(this._calendar=this._createComponent(e,p.default,this._calendarOptions()),this._isMobileLayout()){var t=this._createScrollable(e);return t.$element()}return e},_calendarOptions:function(){return{min:this.option("min"),max:this.option("max"),firstDayOfWeek:this.option("firstDayOfWeek"),value:this.option("date"),_todayDate:this.option("todayDate"),focusStateEnabled:this.option("focusStateEnabled"),onValueChanged:function(e){this.option("visible")&&(this.notifyObserver("currentDateUpdated",e.value),this._popover.hide())}.bind(this),hasFocus:function(){return!0},tabIndex:null}},_renderCaption:function(){var e=this,t=this.option("displayedDate")||this.option("date"),n=this._getConfig().getCaption.call(this,t),i=this.option("customizeDateNavigatorText"),o=(0,r.isFunction)(i)?i(n):n.text;this._caption.option({text:o,onKeyboardHandled:function(t){e.option("focusStateEnabled")&&!e.option("disabled")&&e._calendar._keyboardHandler(t)},onClick:function(){return e._popover.toggle()}})},_renderCaptionKeys:function(){if(this.option("focusStateEnabled")&&!this.option("disabled")){var e=this,t=function(){return!e._popover.$content().is(":hidden")||void e._popover.show()},n=function(){e._popover.hide()};this._caption.registerKeyHandler("enter",t),this._caption.registerKeyHandler("space",t),this._caption.registerKeyHandler("tab",n)}},_setCalendarOption:function(e,t){this._calendar&&this._calendar.option(e,t)},_getConfig:function(){var e=this.option("step"),t=B.call(this,e);if(!t)throw s.default.Error("E1033",e);return t}}).include(m.default);(0,c.default)("dxSchedulerNavigator",V);var F=V;t.default=F,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(12),l=n(4),u=n(1),c=n(50),d=n(3),h=n(0),f=i(n(47)),p=n(19),g=n(69),_=n(6),m=n(70),v=function(e){return e.valueExpr||"id"},y=function(e){return e.displayExpr||"text"},b=function(){function e(t){o(this,e),this._resourceLoader={},this.setResources(t)}return r(e,[{key:"_createWrappedDataSource",value:function(e){if(e instanceof g.DataSource)return e;var t={store:(0,m.normalizeDataSourceOptions)(e).store,pageSize:0};return Array.isArray(e)||(t.filter=e.filter),new g.DataSource(t)}},{key:"_mapResourceData",value:function(e,t){var n=(0,p.compileGetter)(v(e)),i=(0,p.compileGetter)(y(e));return(0,d.map)(t,function(e){var t={id:n(e),text:i(e)};return e.color&&(t.color=e.color),t})}},{key:"_isMultipleResource",value:function(e){var t=!1;return(0,d.each)(this.getResources(),function(n,i){var o=this.getField(i);if(o===e)return t=i.allowMultiple,!1}.bind(this)),t}},{key:"getDataAccessors",value:function(e,t){var n=null;return(0,d.each)(this._dataAccessors[t],function(t,i){if(e===t)return n=i,!1}),n}},{key:"getField",value:function(e){return e.fieldExpr||e.field}},{key:"setResources",value:function(e){this._resources=e,this._dataAccessors={getter:{},setter:{}},this._resourceFields=(0,d.map)(e||[],function(e){var t=this.getField(e);return this._dataAccessors.getter[t]=(0,p.compileGetter)(t),this._dataAccessors.setter[t]=(0,p.compileSetter)(t),t}.bind(this))}},{key:"getResources",value:function(){return this._resources||[]}},{key:"getResourcesData",value:function(){return this._resourcesData||[]}},{key:"getEditors",value:function(){var e=[],t=this;return(0,d.each)(this.getResources(),function(n,i){var o=t.getField(i),a=t._getResourceDataByField(o);e.push({editorOptions:{dataSource:a.length?a:t._createWrappedDataSource(i.dataSource),displayExpr:y(i),valueExpr:v(i)},dataField:o,editorType:i.allowMultiple?"dxTagBox":"dxSelectBox",label:{text:i.label||o}})}),e}},{key:"getResourceDataByValue",value:function(e,t){var n=this,i=new _.Deferred;return(0,d.each)(this.getResources(),function(o,a){var r=n.getField(a);if(r===e){var s=n._createWrappedDataSource(a.dataSource),l=v(a);return n._resourceLoader[e]||(n._resourceLoader[e]=s.load()),n._resourceLoader[e].done(function(o){var a=(0,f.default)(o).filter(l,t).toArray();delete n._resourceLoader[e],i.resolve(a[0])}).fail(function(){delete n._resourceLoader[e],i.reject()}),!1}}),i.promise()}},{key:"setResourcesToItem",value:function(e,t){var n=this._dataAccessors.setter;for(var i in t)if(Object.prototype.hasOwnProperty.call(t,i)){var o=t[i];n[i](e,this._isMultipleResource(i)?(0,s.wrapToArray)(o):o)}}},{key:"getResourcesFromItem",value:function(e,t){var n=this,i=null;return(0,u.isDefined)(t)||(t=!1),this._resourceFields.forEach(function(o){(0,d.each)(e,function(e,a){var r={};r[e]=a;var l=n.getDataAccessors(o,"getter")(r);if((0,u.isDefined)(l))return i||(i={}),1===l.length&&(l=l[0]),!t||t&&n._isMultipleResource(o)?n.getDataAccessors(o,"setter")(r,(0,s.wrapToArray)(l)):n.getDataAccessors(o,"setter")(r,l),(0,h.extend)(i,r),!0})}),i}},{key:"loadResources",value:function(e){var t=new _.Deferred,n=this,i=[];return(0,d.each)(this.getResourcesByFields(e),function(e,t){var o=new _.Deferred,a=n.getField(t);i.push(o),n._createWrappedDataSource(t.dataSource).load().done(function(e){o.resolve({name:a,items:n._mapResourceData(t,e),data:e})}).fail(function(){o.reject()})}),i.length?(_.when.apply(null,i).done(function(){var e=Array.prototype.slice.call(arguments),i=function(e){return{name:e.name,items:e.items,data:e.data}};n._resourcesData=e,t.resolve(e.map(i))}).fail(function(){t.reject()}),t.promise()):(n._resourcesData=[],t.resolve([]))}},{key:"getResourcesByFields",value:function(e){return(0,l.grep)(this.getResources(),function(t){var n=this.getField(t);return(0,s.inArray)(n,e)>-1}.bind(this))}},{key:"getResourceByField",value:function(e){return this.getResourcesByFields([e])[0]||{}}},{key:"getResourceColor",value:function(e,t){var n,i=this.getResourceByField(e).valueExpr||"id",o=(0,p.compileGetter)(i),a=this.getResourceByField(e).colorExpr||"color",r=(0,p.compileGetter)(a),s=new _.Deferred,l=this._getResourceDataByField(e),u=l.length;if(u){for(var c=0;c<u;c++)if(o(l[c])===t){n=r(l[c]);break}s.resolve(n)}else this.getResourceDataByValue(e,t).done(function(e){e&&(n=r(e)),s.resolve(n)}).fail(function(){s.reject()});return s.promise()}},{key:"getResourceForPainting",value:function(e){var t,n=this.getResources();return(0,d.each)(n,function(e,n){if(n.useColorAsDefault)return t=n,!1}),t||(Array.isArray(e)&&e.length&&(n=this.getResourcesByFields(e)),t=n[n.length-1]),t}},{key:"createResourcesTree",value:function(e){function t(i,o,a,r){a=a||[];for(var s=0;s<i.items.length;s++){var l=i.items[s],u={name:i.name,value:l.id,title:l.text,data:i.data&&i.data[s],children:[],parent:r?r:null};a.push(u);var c=o+1;e[c]&&t.call(this,e[c],c,u.children,u),u.children.length||(u.leafIndex=n,n++)}return a}var n=0,i=i||0;return t.call(this,e[0],0)}},{key:"_hasGroupItem",value:function(e,t,n){var i=this.getDataAccessors(t,"getter")(e);return!!(i&&(0,s.inArray)(n,i)>-1)}},{key:"_getResourceDataByField",value:function(e){for(var t=this.getResourcesData(),n=[],i=0,o=t.length;i<o;i++)if(t[i].name===e){n=t[i].data;break}return n}},{key:"getResourceTreeLeaves",value:function(e,t,n){n=n||[];for(var i=0;i<e.length;i++)this._hasGroupItem(t,e[i].name,e[i].value)&&((0,u.isDefined)(e[i].leafIndex)&&n.push(e[i].leafIndex),e[i].children&&this.getResourceTreeLeaves(e[i].children,t,n));return n}},{key:"groupAppointmentsByResources",value:function(e,t){var n=this.createResourcesTree(t),i={};return(0,d.each)(e,function(e,t){for(var o=this.getResourcesFromItem(t),a=this.getResourceTreeLeaves(n,o),r=0;r<a.length;r++)i[a[r]]||(i[a[r]]=[]),i[a[r]].push((0,c.deepExtendArraySafe)({},t,!0))}.bind(this)),i}},{key:"reduceResourcesTree",value:function(e,t,n){n=n?n.children:[];var i=this;return e.forEach(function(e,o){var a=!1,r=e.name,s=e.value,l=e.title,u=e.data,c=i.getDataAccessors(r,"getter");t.forEach(function(e){if(!a){var t=c(e);Array.isArray(t)?t.indexOf(s)>-1&&(n.push({name:r,value:s,title:l,data:u,children:[]}),a=!0):t===s&&(n.push({name:r,value:s,title:l,data:u,children:[]}),a=!0)}}),a&&e.children&&e.children.length&&i.reduceResourcesTree(e.children,t,n[o])}),n}},{key:"getResourcesDataByGroups",value:function(e){var t=this,n=this.getResourcesData();if(!e)return n;var i=Object.getOwnPropertyNames(e),o=n.filter(function(e){return i.indexOf(e.name)!==-1}),a=[];return o.forEach(function(e){return a.push((0,h.extend)({},e))}),(0,d.each)(e,function(e,n){a.forEach(function(e){var i=e.items,o=e.data,a=e.name,r=t.getResourceByField(a),s=v(r),l=i.filter(function(e){return e.id===n}),u=o.filter(function(e){return e[s]===n});e.items=l,e.data=u})}),a}}]),e}();t.default=b,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(12),r=n(1),s=i(n(20)),l=n(3),u=i(n(17)),c=n(30),d=n(4),h=n(0),f=n(6),p=i(n(32)),g=i(n(86)),_=n(169),m=i(n(522)),v=24,y=s.default.dateToMilliseconds,b=y("hour"),x={getTimeZoneCalculator:function(){return this.timeZoneCalculator},isCurrentViewAgenda:function(){return"agenda"===this.option("currentView")},currentViewUpdated:function(e){this.option("currentView",e)},currentDateUpdated:function(e){this.option("currentDate",e)},getOption:function(e){return this.option(e)},isVirtualScrolling:function(){return this.isVirtualScrolling()},setCellDataCacheAlias:function(e,t){this._workSpace.setCellDataCacheAlias(e,t)},createAppointmentSettings:function(e){return this._getAppointmentSettingsGenerator().create(e)},isGroupedByDate:function(){return this.getWorkSpace().isGroupedByDate()},showAppointmentTooltip:function(e){var t=this.getTargetedAppointment(e.data,e.target);this.showAppointmentTooltip(e.data,e.target,t)},hideAppointmentTooltip:function(){this.hideAppointmentTooltip()},showAddAppointmentPopup:function(e,t){var n=this.createAppointmentAdapter({});n.allDay=e.allDay,n.startDate=this.timeZoneCalculator.createDate(e.startDate,{path:"fromGrid"}),n.endDate=this.timeZoneCalculator.createDate(e.endDate,{path:"fromGrid"});var i=(0,h.extend)(n.source(),t);this.showAppointmentPopup(i,!0)},showEditAppointmentPopup:function(e){var t=this.getTargetedAppointment(e.data,e.target);this.showAppointmentPopup(e.data,!1,t)},updateAppointmentAfterResize:function(e){var t=m.default.dataAccessors.getAppointmentInfo(e.$appointment),n=t.sourceAppointment.startDate;this._checkRecurringAppointment(e.target,e.data,n,function(){this._updateAppointment(e.target,e.data,function(){this._appointments.moveAppointmentBack()})}.bind(this))},getUpdatedData:function(e){return this._getUpdatedData(e)},updateAppointmentAfterDrag:function(e){var t=e.event,n=e.element,i=e.rawAppointment,o=e.coordinates,a=m.default.dataAccessors.getAppointmentInfo(n),r=this.createAppointmentAdapter(i),s=this.createAppointmentAdapter((0,h.extend)({},i,this._getUpdatedData(i))),l=s.source(),u=this._workSpace.getDroppableCellIndex(),c=this._workSpace.getCellIndexByCoordinates(o),d=s.allDay,f=r.allDay,p=this._workSpace.supportAllDayRow()&&(f&&!d||!f&&d);u!==c||p?this._checkRecurringAppointment(i,l,a.sourceAppointment.startDate,function(){this._updateAppointment(i,l,function(){this._appointments.moveAppointmentBack(t)},t)}.bind(this),void 0,void 0,t):this._appointments.moveAppointmentBack(t)},onDeleteButtonPress:function(e){var t=this.getTargetedAppointment(e.data,(0,o.default)(e.target));this.checkAndDeleteAppointment(e.data,t),this.hideAppointmentTooltip()},getAppointmentColor:function(e){var t=this._resourcesManager,n=t.getResourceForPainting(this._getCurrentViewOption("groups")),i=(new f.Deferred).resolve().promise();if(n){for(var o=t.getField(n),r=e.groupIndex,s=this._workSpace._getCellGroups(r),l=(0,a.wrapToArray)(t.getDataAccessors(o,"getter")(e.itemData)),u=l.length?l[0]:void 0,c=0;c<s.length;c++)if(s[c].name===o){u=s[c].id;break}i=t.getResourceColor(o,u)}return i},getHeaderHeight:function(){return this._header._$element&&parseInt(this._header._$element.outerHeight(),10)},getResourcesFromItem:function(e){return this._resourcesManager.getResourcesFromItem(e)},getBoundOffset:function(){return{top:-this.getWorkSpaceAllDayHeight()}},appointmentTakesSeveralDays:function(e){return this._appointmentModel.appointmentTakesSeveralDays(e)},getTextAndFormatDate:function(e,t,n){var i=this.createAppointmentAdapter(e),o=this.createAppointmentAdapter(t||e),a=this.timeZoneCalculator.createDate(o.startDate,{path:"toGrid"}),r=this.timeZoneCalculator.createDate(o.endDate,{path:"toGrid"}),s=n||this.fire("_getTypeFormat",a,r,o.allDay);return{text:o.text||i.text,formatDate:this.fire("_formatDates",a,r,s)}},_getTypeFormat:function(e,t,n){return n?"DATE":"month"!==this.option("currentView")&&s.default.sameDate(e,t)?"TIME":"DATETIME"},_createAppointmentTitle:function(e){return(0,r.isPlainObject)(e)?e.text:String(e)},_formatDates:function(e,t,n){var i="monthandday",o="shorttime",a=e.getDate()===t.getDate();switch(n){case"DATETIME":return[p.default.format(e,i)," ",p.default.format(e,o)," - ",a?"":p.default.format(t,i)+" ",p.default.format(t,o)].join("");case"TIME":return"".concat(p.default.format(e,o)," - ").concat(p.default.format(t,o));case"DATE":return"".concat(p.default.format(e,i)).concat(a?"":" - "+p.default.format(t,i))}},getResizableAppointmentArea:function(e){var t=e.allDay,n=this._getCurrentViewOption("groups");if(n&&n.length){if(t||this.getLayoutManager().getRenderingStrategyInstance()._needHorizontalGroupBounds()){var i=this._workSpace.getGroupBounds(e.coordinates);return{left:i.left,right:i.right,top:0,bottom:0}}if(this.getLayoutManager().getRenderingStrategyInstance()._needVerticalGroupBounds(t)&&this._workSpace._isVerticalGroupedWorkSpace()){var o=this._workSpace.getGroupBounds(e.coordinates);return{left:0,right:0,top:o.top,bottom:o.bottom}}}},needRecalculateResizableArea:function(){return this.getWorkSpace().needRecalculateResizableArea()},getAppointmentGeometry:function(e){return this.getLayoutManager().getRenderingStrategyInstance().getAppointmentGeometry(e)},isAllDay:function(e){return this.getLayoutManager().getRenderingStrategyInstance().isAllDay(e)},getDeltaTime:function(e,t,n){return this.getLayoutManager().getRenderingStrategyInstance().getDeltaTime(e,t,n)},getDropDownAppointmentWidth:function(e){return this.getLayoutManager().getRenderingStrategyInstance().getDropDownAppointmentWidth(this._getViewCountConfig().intervalCount,e)},getDropDownAppointmentHeight:function(){return this.getLayoutManager().getRenderingStrategyInstance().getDropDownAppointmentHeight()},getCellWidth:function(){return this.getWorkSpace().getCellWidth()},getCellHeight:function(){return this.getWorkSpace().getCellHeight()},getResizableStep:function(){var e=this.getWorkSpace(),t=e.getCellWidth();return e.isGroupedByDate()?e._getGroupCount()*t:t},getRenderingStrategy:function(){return this._getAppointmentsRenderingStrategy()},getMaxAppointmentCountPerCellByType:function(e){return this.getRenderingStrategyInstance()._getMaxAppointmentCountPerCellByType(e)},needCorrectAppointmentDates:function(){return this.getRenderingStrategyInstance().needCorrectAppointmentDates()},getRenderingStrategyDirection:function(){return this.getRenderingStrategyInstance().getDirection()},getWorkSpaceDateTableOffset:function(){return this.getWorkSpaceDateTableOffset()},getFullWeekAppointmentWidth:function(e){var t=e.groupIndex;return this._workSpace.getGroupWidth(t)},getMaxAppointmentWidth:function(e){var t=this._workSpace;return t.getCellCountToLastViewDate(e.date)*t.getCellWidth()},updateAppointmentStartDate:function(e){var t,n=e.appointment,i=this._workSpace.getStartViewDate(),o=new Date(e.startDate),a=this._getCurrentViewOption("startDayHour");return this.appointmentTakesAllDay(n)?t=s.default.normalizeDate(o,i):(o<i&&(o=i),t=s.default.normalizeDate(e.startDate,new Date(o))),s.default.roundDateByStartDayHour(t,a)},updateAppointmentEndDate:function(e){var t=e.endDate,n=this._getCurrentViewOption("endDayHour"),i=this._getCurrentViewOption("startDayHour"),o=t;return t.getHours()>=n?o.setHours(n,0,0,0):!e.isSameDate&&i>0&&60*t.getHours()+t.getMinutes()<60*i&&(o=new Date(o.getTime()-y("day")),o.setHours(n,0,0,0)),o},renderCompactAppointments:function(e){this._compactAppointmentsHelper.render(e)},clearCompactAppointments:function(){this._compactAppointmentsHelper.clear()},supportCompactDropDownAppointments:function(){return this._workSpace._supportCompactDropDownAppointments()},isApplyCompactAppointmentOffset:function(){return this._workSpace._isApplyCompactAppointmentOffset()},getGroupCount:function(){return this._workSpace._getGroupCount()},mapAppointmentFields:function(e){var t=this.getTargetedAppointment(e.itemData,e.itemElement);return{appointmentData:e.itemData,appointmentElement:e.itemElement,targetedAppointmentData:t}},getOffsetByAllDayPanel:function(e){return this._workSpace._getOffsetByAllDayPanel(e)},getGroupTop:function(e){return this._workSpace._getGroupTop(e)},updateResizableArea:function(){var e=this.$element().find(".dx-scheduler-appointment.dx-resizable"),t=(0,d.grep)(e,function(e){var t=(0,o.default)(e),n=t.dxResizable("instance"),i=n.option("area");return(0,a.inArray)(n.option("handles"),["right left","left right"])>-1&&(0,r.isPlainObject)(i)});(0,l.each)(t,function(e,t){var n=(0,o.default)(t),i=(0,c.locate)(n),a=this._appointments._getItemData(n),r=this._appointments._calculateResizableArea({left:i.left},a);n.dxResizable("instance").option("area",r)}.bind(this))},getField:function(e,t){if((0,r.isDefined)(this._dataAccessors.getter[e]))return this._dataAccessors.getter[e](t)},setField:function(e,t,n){if((0,r.isDefined)(this._dataAccessors.setter[e])){var i=this.option(e+"Expr").split("."),o=i[0];if(void 0===t[o]&&i.length>1){var a=function(e){for(var t={},n=t,i=e.length-1,o=1;o<i;o++)n=n[e[o]]={};return t}(i);t[o]=a}return this._dataAccessors.setter[e](t,n),t}},renderAppointments:function(){this._renderAppointments()},prerenderFilter:function(){var e,t=this.getWorkSpace().getDateRange(),n=this._resourcesManager.getResourcesData(),i=this._getCurrentViewOption("startDayHour"),o=this._getCurrentViewOption("endDayHour");return!this.option("showAllDayPanel")&&this._workSpace.supportAllDayRow()&&(e=!1),this._appointmentModel.filterLoadedAppointments({startDayHour:i,endDayHour:o,viewStartDayHour:i,viewEndDayHour:o,min:t[0],max:t[1],resources:n,allDay:e,firstDayOfWeek:this.getFirstDayOfWeek(),recurrenceException:this._getRecurrenceException.bind(this)},this.timeZoneCalculator)},prerenderFilterVirtual:function(){var e=this,t=this.getWorkSpace(),n=this._resourcesManager,i=this.option("showAllDayPanel")||!this._workSpace.supportAllDayRow(),o=t.viewDataProvider,a=o.viewData.groupedData,r=a.filter(function(e){var t=e.dateTable;return t.length>0}),s=t._isVerticalGroupedWorkSpace(),l=t.getEndViewDateByEndDayHour(),u=[];r.forEach(function(t){var a=t.groupIndex,r=t.allDayPanel,c=o.getGroupStartDate(a),d=new Date(Math.min(o.getGroupEndDate(a),l)),h=c.getHours(),f=(h+(d-c)/b)%v,p=i!==!1&&(null===r||void 0===r?void 0:r.length)>0,g=o.getCellsGroup(a),_=s?n.getResourcesDataByGroups(g):n.getResourcesData();u.push({isVirtualScrolling:!0,startDayHour:h,endDayHour:f,viewStartDayHour:e._getCurrentViewOption("startDayHour"),viewEndDayHour:e._getCurrentViewOption("endDayHour"),min:c,max:d,resources:_,allDay:p,firstDayOfWeek:e.getFirstDayOfWeek(),recurrenceException:e._getRecurrenceException.bind(e)})});var c=this._appointmentModel.filterLoadedVirtualAppointments(u,this.timeZoneCalculator,t._getGroupCount());return c},dayHasAppointment:function(e,t,n){return this.dayHasAppointment(e,t,n)},createResourcesTree:function(){return this._resourcesManager.createResourcesTree(this._loadedResources)},getResourceTreeLeaves:function(e,t){return this._resourcesManager.getResourceTreeLeaves(e,t)},createReducedResourcesTree:function(){var e=this._resourcesManager.createResourcesTree(this._loadedResources);return this._resourcesManager.reduceResourcesTree(e,this.getFilteredItems())},groupAppointmentsByResources:function(e){var t={0:e},n=this._getCurrentViewOption("groups");n&&n.length&&this._resourcesManager.getResourcesData().length&&(t=this._resourcesManager.groupAppointmentsByResources(e,this._loadedResources));var i=0;(0,l.each)(this._loadedResources,function(e,t){e?i*=t.items.length:i=t.items.length});for(var o=0;o<i;o++){var a=o.toString();t[a]||(t[a]=[])}return t},getAgendaRows:function(e){function t(n){var a=i(n,e.agendaDuration,e.currentDate);this._dataSourceLoadedCallback.remove(t),o.resolve(a)}var n=this._layoutManager.getRenderingStrategyInstance(),i=n.calculateRows.bind(n),o=new f.Deferred;return this._dataSourceLoadedCallback.add(t),o.promise()},getAgendaVerticalStepHeight:function(){return this.getWorkSpace().getAgendaVerticalStepHeight()},getAgendaDuration:function(){return this._getCurrentViewOption("agendaDuration")},getStartViewDate:function(){return this.getStartViewDate()},getEndViewDate:function(){return this.getEndViewDate()},getMaxAppointmentsPerCell:function(){return this.getMaxAppointmentsPerCell()},forceMaxAppointmentPerCell:function(){return this.forceMaxAppointmentPerCell()},onAgendaReady:function(e){for(var t=this.getAppointmentsInstance()._itemElements(),n=0,i=function(e,i){var o=i+n-1;t.eq(o).addClass(_.AGENDA_LAST_IN_DATE_APPOINTMENT_CLASS),n+=i},o=0;o<e.length;o++)(0,l.each)(e[o],i)},getTimezone:function(){return this._getTimezoneOffsetByOption()},getTargetedAppointmentData:function(e,t){return this.getTargetedAppointment(e,t)},getAppointmentDurationInMs:function(e){var t=e.startDate,n=e.endDate,i=e.allDay,o=n.getTime()-t.getTime(),a=y("day"),r=this._workSpace.getVisibleDayDuration(),l=0;if(i){var u=Math.ceil(o/a);l=u*r}else{var c,d=!g.default.isSameAppointmentDates(t,n),h=Math.floor(o/a);if(d){var f=new Date(new Date(t).setHours(this.option("endDayHour"),0,0)),p=a-r-(t.getTime()>f.getTime()?t.getTime()-f.getTime():0);c=o-(h?h*a:p);var _=this.option("startDayHour")*y("hour"),m=n-s.default.trimTime(n);m<_&&(h&&(c-=p),c+=_-m)}else c=o%a;c>r&&(c=r),l=h*r+c||y("minute")}return l},replaceWrongEndDate:function(e,t,n){this._appointmentModel.replaceWrongEndDate(e,t,n)},calculateAppointmentEndDate:function(e,t){return this._appointmentModel._calculateAppointmentEndDate(e,t)},getEndDayHour:function(){return this._workSpace.option("endDayHour")||this.option("endDayHour")},getStartDayHour:function(){return this._workSpace.option("startDayHour")||this.option("startDayHour")},isAdaptive:function(){return this.option("adaptivityEnabled")},validateDayHours:function(){var e=this._getCurrentViewOption("endDayHour"),t=this._getCurrentViewOption("startDayHour");if(t>=e)throw u.default.Error("E1058")},removeDroppableCellClass:function(){this._workSpace.removeDroppableCellClass()}},w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(11)),y=n(4),b=n(3),x=n(18),w=o(n(9)),C=o(n(523)),k=n(0),S=o(n(32)),D=o(n(312)),I=D.default.tableCreator,T="dx-scheduler-agenda",O="dx-scheduler-agenda-date",E="dx-scheduler-group-table",A="dx-group-column-count",P="dx-scheduler-time-panel-row",R="dx-scheduler-time-panel-cell",M="dx-scheduler-agenda-nodata",B="dx-scheduler-date-table-last-row",V=5,F=20,L=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._activeStateUnit=void 0}},{key:"_getDefaultOptions",value:function(){return(0,k.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{agendaDuration:7,rowHeight:60,noDataText:""})}},{key:"_optionChanged",value:function(e){var n=e.name,i=e.value;switch(n){case"agendaDuration":break;case"noDataText":case"rowHeight":this._recalculateAgenda(this._rows);break;case"groups":i&&i.length?this._$groupTable||(this._initGroupTable(),this._dateTableScrollable.$content().prepend(this._$groupTable)):this._$groupTable&&(this._$groupTable.remove(),this._$groupTable=null,this._detachGroupCountAttr()),l(_(t.prototype),"_optionChanged",this).call(this,e);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_renderFocusState",value:function(){return(0,y.noop)()}},{key:"_renderFocusTarget",value:function(){return(0,y.noop)()}},{key:"_cleanFocusState",value:function(){return(0,y.noop)()}},{key:"supportAllDayRow",value:function(){return!1}},{key:"_isVerticalGroupedWorkSpace",value:function(){return!1}},{key:"_getElementClass",value:function(){return T}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=new Date(this.option("currentDate")),this._setStartDayHour(this._firstViewDate)}},{key:"_getRowCount",value:function(){return this.option("agendaDuration")}},{key:"_getCellCount",value:function(){return 1}},{key:"_getTimePanelRowCount",value:function(){return this.option("agendaDuration")}},{key:"_getDateByIndex",value:function(){return(0,y.noop)()}},{key:"_getFormat",value:function(){return"d ddd"}},{key:"_renderAllDayPanel",value:function(){return(0,y.noop)()}},{key:"_toggleAllDayVisibility",value:function(){return(0,y.noop)()}},{key:"_initWorkSpaceUnits",value:function(){this._initGroupTable(),this._$timePanel=(0,m.default)("<table>").addClass(this._getTimePanelClass()),this._$dateTable=(0,m.default)("<table>").addClass(this._getDateTableClass())}},{key:"_initGroupTable",value:function(){var e=this.option("groups");e&&e.length&&(this._$groupTable=(0,m.default)("<table>").addClass(E))}},{key:"_renderView",value:function(){this._setFirstViewDate(),this._rows=[],this.invoke("getAgendaRows",{agendaDuration:this.option("agendaDuration"),currentDate:new Date(this.option("currentDate"))}).done(function(e){this._recalculateAgenda(e)}.bind(this))}},{key:"_recalculateAgenda",value:function(e){var t=[];return this._cleanView(),this._rowsIsEmpty(e)?void this._renderNoData():(this._rows=e,this._$groupTable&&(t=this._renderGroupHeader(),this._setGroupHeaderCellsHeight()),this._renderTimePanel(),this._renderDateTable(),this.invoke("onAgendaReady",e),this._applyCellTemplates(t),void this._dateTableScrollable.update())}},{key:"_renderNoData",value:function(){this._$noDataContainer=(0,m.default)("<div>").addClass(M).html(this.option("noDataText")),this._dateTableScrollable.$content().append(this._$noDataContainer)}},{key:"_setTableSizes",value:function(){return(0,y.noop)()}},{key:"_toggleHorizontalScrollClass",value:function(){return(0,y.noop)()}},{key:"_createCrossScrollingConfig",value:function(){return(0,y.noop)()}},{key:"_setGroupHeaderCellsHeight",value:function(){var e=this._getGroupHeaderCells().filter(function(e,t){return!t.getAttribute("rowSpan")}),t=this._removeEmptyRows(this._rows);if(t.length)for(var n=0;n<e.length;n++){var i=e.eq(n).find(".dx-scheduler-group-header-content");i.outerHeight(this._getGroupRowHeight(t[n]))}}},{key:"_rowsIsEmpty",value:function(e){for(var t=!0,n=0;n<e.length;n++)for(var i=e[n],o=0;o<i.length;o++)if(i[o]){t=!1;break}return t}},{key:"_detachGroupCountAttr",value:function(){this.$element().removeAttr(A)}},{key:"_attachGroupCountAttr",value:function(){this.$element().attr(A,this.option("groups").length)}},{key:"_removeEmptyRows",value:function(e){for(var t=[],n=function(e){return!e.some(function(e){
return e>0})},i=0;i<e.length;i++)e[i].length&&!n(e[i])&&t.push(e[i]);return t}},{key:"_getGroupHeaderContainer",value:function(){return this._$groupTable}},{key:"_makeGroupRows",value:function(){var e=this.invoke("createReducedResourcesTree"),t=this.option("resourceCellTemplate"),n=this._getGroupHeaderContentClass(),i=[],o=I.makeGroupedTableFromJSON(I.VERTICAL,e,{cellTag:"th",groupTableClass:E,groupRowClass:this._getGroupRowClass(),groupCellClass:this._getGroupHeaderClass(),groupCellCustomContent:function(e,o,a,r){var s=v.default.createElement("div"),l=v.default.createElement("div");s.className=n,l.appendChild(o),s.appendChild(l),s.className=n,t&&t.render?i.push(t.render.bind(t,{model:{data:r.data,id:r.value,color:r.color,text:o.textContent},container:(0,x.getPublicElement)((0,m.default)(s)),index:a})):(l.appendChild(o),s.appendChild(l)),e.appendChild(s)},cellTemplate:t});return{elements:(0,m.default)(o).find("."+this._getGroupRowClass()),cellTemplates:i}}},{key:"_cleanView",value:function(){this._$dateTable.empty(),this._$timePanel.empty(),this._$groupTable&&this._$groupTable.empty(),this._$noDataContainer&&(this._$noDataContainer.empty(),this._$noDataContainer.remove(),delete this._$noDataContainer)}},{key:"_createWorkSpaceElements",value:function(){this._createWorkSpaceStaticElements()}},{key:"_createWorkSpaceStaticElements",value:function(){this._$groupTable&&this._dateTableScrollable.$content().prepend(this._$groupTable),this._dateTableScrollable.$content().append(this._$timePanel,this._$dateTable),this.$element().append(this._dateTableScrollable.$element())}},{key:"_renderDateTable",value:function(){this._renderTableBody({container:(0,x.getPublicElement)(this._$dateTable),rowClass:this._getDateTableRowClass(),cellClass:this._getDateTableCellClass()})}},{key:"_attachTablesEvents",value:function(){return(0,y.noop)()}},{key:"_attachEvents",value:function(){return(0,y.noop)()}},{key:"_cleanCellDataCache",value:function(){return(0,y.noop)()}},{key:"isIndicationAvailable",value:function(){return!1}},{key:"_prepareCellTemplateOptions",value:function(e,t,n,i){var o=this.option("groups"),a={},r=o.length&&this._getPathToLeaf(n)||[];return r.forEach(function(e,t){var n=o[t].name;a[n]=e}),{model:{text:e,date:t,groups:a},container:(0,x.getPublicElement)(i),index:n}}},{key:"_renderTableBody",value:function(e){var t=[],n=e.cellTemplate;this._$rows=[];var i,o=function(o,a){if(a){var r,s,l,u=(0,m.default)("<tr>"),c=(0,m.default)("<td>").height(this._getRowHeight(a));if(e.getStartDate&&(r=e.getStartDate&&e.getStartDate(o),s=S.default.format(r,"d"),l=S.default.format(r,this._formatWeekday)),n&&n.render){var d=this._prepareCellTemplateOptions(s+" "+l,r,i,c);t.push(n.render.bind(n,d))}else s&&l&&c.addClass(O).text(s+" "+l);e.rowClass&&u.addClass(e.rowClass),e.cellClass&&c.addClass(e.cellClass),u.append(c),this._$rows.push(u)}}.bind(this);for(i=0;i<this._rows.length;i++)(0,b.each)(this._rows[i],o),this._setLastRowClass();(0,m.default)(e.container).append((0,m.default)("<tbody>").append(this._$rows)),this._applyCellTemplates(t)}},{key:"_setLastRowClass",value:function(){if(this._rows.length>1&&this._$rows.length){var e=this._$rows[this._$rows.length-1];e.addClass(B)}}},{key:"_renderTimePanel",value:function(){this._renderTableBody({container:(0,x.getPublicElement)(this._$timePanel),rowCount:this._getTimePanelRowCount(),cellCount:1,rowClass:P,cellClass:R,cellTemplate:this.option("dateCellTemplate"),getStartDate:this._getTimePanelStartDate.bind(this)})}},{key:"_getTimePanelStartDate",value:function(e){var t=new Date(this.option("currentDate")),n=new Date(t.setDate(t.getDate()+e));return n}},{key:"_getRowHeight",value:function(e){var t=this.option("rowHeight"),n=(e-1)*V;return e?t*e+n+F:0}},{key:"_getGroupRowHeight",value:function(e){if(e){for(var t=0,n=0;n<e.length;n++)t+=this._getRowHeight(e[n]);return t}}},{key:"getAgendaVerticalStepHeight",value:function(){return this.option("rowHeight")}},{key:"getEndViewDate",value:function(){var e=new Date(this.option("currentDate")),t=this.option("agendaDuration");e.setHours(this.option("endDayHour"));var n=e.setDate(e.getDate()+t-1)-6e4;return new Date(n)}},{key:"getEndViewDateByEndDayHour",value:function(){return this.getEndViewDate()}},{key:"getCoordinatesByDate",value:function(){return{top:0,left:0,max:0,groupIndex:0}}},{key:"getCellDataByCoordinates",value:function(){return{startDate:null,endDate:null}}},{key:"updateScrollPosition",value:function(e){var t=this.option("observer"),n=t.timeZoneCalculator.createDate(e,{path:"toGrid"}),i=this.getVisibleBounds(),o=n.getHours(),a=n.getMinutes();this.needUpdateScrollPosition(o,a,i,n)&&this.scrollToTime(o,a,n)}},{key:"needUpdateScrollPosition",value:function(e,t,n){var i=!1;return(e<n.top.hours||e>n.bottom.hours)&&(i=!0),e===n.top.hours&&t<n.top.minutes&&(i=!0),e===n.bottom.hours&&t>n.top.minutes&&(i=!0),i}}]),t}(C.default);(0,w.default)("dxSchedulerAgenda",L);var H=L;t.default=H,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=n(24),_=o(n(524)),m="dx-group-row-count",v=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"prepareCellIndexes",value:function(e,t,n){var i=this._workSpace.isGroupedByDate();return i?{rowIndex:e.rowIndex,cellIndex:e.cellIndex*this._workSpace._getGroupCount()+t}:{rowIndex:e.rowIndex,cellIndex:e.cellIndex+t*this._workSpace._getCellCount()}}},{key:"calculateCellIndex",value:function(e,t){return t%=this._workSpace._getCellCount(),this._workSpace._getRowCount()*t+e}},{key:"getGroupIndex",value:function(e,t){var n=this._workSpace.isGroupedByDate(),i=this._workSpace._getGroupCount();return n?t%i:Math.floor(t/this._workSpace._getCellCount())}},{key:"calculateHeaderCellRepeatCount",value:function(){return this._workSpace._getGroupCount()||1}},{key:"insertAllDayRowsIntoDateTable",value:function(){return!1}},{key:"getTotalCellCount",value:function(e){return e=e||1,this._workSpace._getCellCount()*e}},{key:"getTotalRowCount",value:function(){return this._workSpace._getRowCount()}},{key:"addAdditionalGroupCellClasses",value:function(e,t,n,i){var o=arguments.length>4&&void 0!==arguments[4]&&arguments[4];return e=this._addLastGroupCellClass(e,t,o),this._addFirstGroupCellClass(e,t,o)}},{key:"_addLastGroupCellClass",value:function(e,t,n){if(n)return"".concat(e," ").concat(this.getLastGroupCellClass());var i=this._workSpace.isGroupedByDate();if(i){if(t%this._workSpace._getGroupCount()===0)return"".concat(e," ").concat(this.getLastGroupCellClass())}else if(t%this._workSpace._getCellCount()===0)return"".concat(e," ").concat(this.getLastGroupCellClass());return e}},{key:"_addFirstGroupCellClass",value:function(e,t,n){if(n)return"".concat(e," ").concat(this.getFirstGroupCellClass());var i=this._workSpace.isGroupedByDate();if(i){if((t-1)%this._workSpace._getGroupCount()===0)return"".concat(e," ").concat(this.getFirstGroupCellClass())}else if((t-1)%this._workSpace._getCellCount()===0)return"".concat(e," ").concat(this.getFirstGroupCellClass());return e}},{key:"getHorizontalMax",value:function(e){return this._workSpace.getMaxAllowedPosition()[e]}},{key:"getVerticalMax",value:function(e){return this._workSpace.getMaxAllowedVerticalPosition(0)}},{key:"calculateTimeCellRepeatCount",value:function(){return 1}},{key:"getWorkSpaceMinWidth",value:function(){return(0,g.getBoundingRect)(this._workSpace.$element().get(0)).width-this._workSpace.getTimePanelWidth()}},{key:"getAllDayOffset",value:function(){return this._workSpace.getAllDayHeight()}},{key:"getAllDayTableHeight",value:function(){return(0,g.getBoundingRect)(this._workSpace._$allDayTable.get(0)).height||0}},{key:"getGroupCountAttr",value:function(e,t){return{attr:m,count:t&&t.elements.length}}},{key:"getLeftOffset",value:function(){return this._workSpace.getTimePanelWidth()}},{key:"getGroupBoundsOffset",value:function(e,t,n,i){var o,a,r,s,l;return this._workSpace.isGroupedByDate()?(r=0,s=t.eq(r).offset().left-n/2,l=t.eq(e*this._workSpace._getGroupCount()-1).offset().left+n+n/2):(a=this._workSpace.getCellIndexByCoordinates(i),o=i.groupIndex||Math.floor(a/e),r=o*e,s=t.eq(r).offset().left-n/2,l=t.eq(r+e-1).offset().left+n+n/2),{left:s,right:l,top:0,bottom:0}}},{key:"shiftIndicator",value:function(e,t,n,i){var o=this._getIndicatorOffset(i),a=n?n-o:o;e.css("left",a),e.css("top",t)}},{key:"_getIndicatorOffset",value:function(e){var t=this._workSpace.isGroupedByDate();return t?this._calculateGroupByDateOffset(e):this._calculateOffset(e)}},{key:"_calculateOffset",value:function(e){return this._workSpace._getCellCount()*this._workSpace.getRoundedCellWidth(e-1,0)*e+this._workSpace.getIndicatorOffset(e)+e}},{key:"_calculateGroupByDateOffset",value:function(e){return this._workSpace.getIndicatorOffset(0)*this._workSpace._getGroupCount()+this._workSpace.getRoundedCellWidth(e-1,0)*e}},{key:"getShaderOffset",value:function(e,t){var n=this._workSpace._getCellCount()*this._workSpace.getRoundedCellWidth(e-1)*e;return this._workSpace.option("rtlEnabled")?(0,g.getBoundingRect)(this._workSpace._dateTableScrollable.$content().get(0)).width-n-this._workSpace.getTimePanelWidth()-t:n}},{key:"getShaderTopOffset",value:function(e){return-this.getShaderMaxHeight()*(e>0?1:0)}},{key:"getShaderHeight",value:function(){var e=this._workSpace.getIndicationHeight();return e}},{key:"getShaderMaxHeight",value:function(){return(0,g.getBoundingRect)(this._workSpace._dateTableScrollable.$content().get(0)).height}},{key:"getShaderWidth",value:function(e){return this._workSpace.getIndicationWidth(e)}},{key:"getScrollableScrollTop",value:function(e){return e?0:this._workSpace.getScrollable().scrollTop()}},{key:"getGroupIndexByCell",value:function(e){var t=e.parent().index(),n=e.index();return this.getGroupIndex(t,n)}}]),t}(_.default),y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=n(24),_=o(n(524)),m=n(525),v="dx-group-column-count",y=10,b=1,x=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"prepareCellIndexes",value:function(e,t,n){var i=e.rowIndex+t*this._workSpace._getRowCount();return this._workSpace.supportAllDayRow()&&this._workSpace.option("showAllDayPanel")&&(i+=t,n||(i+=1)),{rowIndex:i,cellIndex:e.cellIndex}}},{key:"calculateCellIndex",value:function(e,t){return e%=this._workSpace._getRowCount(),this._workSpace._getRowCount()*t+e}},{key:"getGroupIndex",value:function(e){return Math.floor(e/this._workSpace._getRowCount())}},{key:"calculateHeaderCellRepeatCount",value:function(){return 1}},{key:"insertAllDayRowsIntoDateTable",value:function(){return this._workSpace.option("showAllDayPanel")}},{key:"getTotalCellCount",value:function(){return this._workSpace._getCellCount()}},{key:"getTotalRowCount",value:function(){return this._workSpace._getRowCount()*this._workSpace._getGroupCount()}},{key:"addAdditionalGroupCellClasses",value:function(e,t,n,i){return e=this._addLastGroupCellClass(e,n+1),this._addFirstGroupCellClass(e,n+1)}},{key:"_addLastGroupCellClass",value:function(e,t){return t%this._workSpace._getRowCount()===0?e+" "+this.getLastGroupCellClass():e}},{key:"_addFirstGroupCellClass",value:function(e,t){return(t-1)%this._workSpace._getRowCount()===0?e+" "+this.getFirstGroupCellClass():e}},{key:"getHorizontalMax",value:function(){return this._workSpace.getMaxAllowedPosition()[0]}},{key:"getVerticalMax",value:function(e){var t=this._workSpace.getMaxAllowedVerticalPosition(e);return t+=this._getOffsetByAllDayPanel(e)}},{key:"_getOffsetByAllDayPanel",value:function(e){var t=0;return this._workSpace.supportAllDayRow()&&this._workSpace.option("showAllDayPanel")&&(t=this._workSpace.getAllDayHeight()*(e+1)),t}},{key:"_getGroupTop",value:function(e){var t=this._workSpace,n=t.isVirtualScrolling()?t.viewDataProvider.getRowCountInGroup(e):t._getRowCount();return t.getMaxAllowedVerticalPosition(e)-t.getCellHeight()*n}},{key:"calculateTimeCellRepeatCount",value:function(){return this._workSpace._getGroupCount()||1}},{key:"getWorkSpaceMinWidth",value:function(){var e=this._workSpace._getWorkSpaceWidth(),t=(0,g.getBoundingRect)(this._workSpace.$element().get(0)).width-this._workSpace.getTimePanelWidth()-this._workSpace.getGroupTableWidth()-2*b;return e<t&&(e=t),e}},{key:"getAllDayOffset",value:function(){return 0}},{key:"getAllDayTableHeight",value:function(){return 0}},{key:"getGroupCountAttr",value:function(){return{attr:v,count:this._workSpace.option("groups")&&this._workSpace.option("groups").length}}},{key:"getLeftOffset",value:function(){return this._workSpace.getTimePanelWidth()+this._workSpace.getGroupTableWidth()}},{key:"getGroupBoundsOffset",value:function(e,t,n,i){var o=this;return m.cache.get("groupBoundsOffset",function(){var a=i.groupIndex,r=t.eq(0).offset().left,s=t.eq(e-1).offset().left+n,l=o._workSpace._calculateDayDuration()/o._workSpace.option("hoursInterval")*o._workSpace.getCellHeight(),u=o.getScrollableScrollTop(),c=a*l+(0,g.getBoundingRect)(o._workSpace._$thead.get(0)).height+o._workSpace.invoke("getHeaderHeight")+y-u;o._workSpace.option("showAllDayPanel")&&o._workSpace.supportAllDayRow()&&(c+=o._workSpace.getCellHeight()*(a+1));var d=c+l;return o._groupBoundsOffset={left:r,right:s,top:c,bottom:d}})}},{key:"shiftIndicator",value:function(e,t,n,i){var o=this._workSpace.getIndicatorOffset(0),a=this._workSpace.option("crossScrollingEnabled")?0:this._workSpace.getGroupTableWidth(),r=n?n-o:o,s=this._workSpace._getRowCount()*this._workSpace.getCellHeight()*i;this._workSpace.supportAllDayRow()&&this._workSpace.option("showAllDayPanel")&&(s+=this._workSpace.getAllDayHeight()*(i+1)),e.css("left",r+a),e.css("top",t+s)}},{key:"getShaderOffset",value:function(e,t){var n=this._workSpace.option("crossScrollingEnabled")?0:this._workSpace.getGroupTableWidth();return this._workSpace.option("rtlEnabled")?(0,g.getBoundingRect)(this._$container.get(0)).width-n-this._workSpace.getWorkSpaceLeftOffset()-t:n}},{key:"getShaderTopOffset",value:function(e){return 0}},{key:"getShaderHeight",value:function(){var e=this._workSpace.getIndicationHeight();return this._workSpace.supportAllDayRow()&&this._workSpace.option("showAllDayPanel")&&(e+=this._workSpace.getCellHeight()),e}},{key:"getShaderMaxHeight",value:function(){var e=this._workSpace._getRowCount()*this._workSpace.getCellHeight();return this._workSpace.supportAllDayRow()&&this._workSpace.option("showAllDayPanel")&&(e+=this._workSpace.getCellHeight()),e}},{key:"getShaderWidth",value:function(){return this._workSpace.getIndicationWidth(0)}},{key:"getScrollableScrollTop",value:function(){return this._workSpace.getScrollable().scrollTop()}},{key:"getGroupIndexByCell",value:function(e){var t=e.parent().index(),n=this._workSpace._getRowCountWithAllDayRows();return Math.ceil((t+1)/n)}}]),t}(_.default),w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(526)),y="dx-scheduler-date-time-shader-all-day",b="dx-scheduler-date-time-shader-top",x="dx-scheduler-date-time-shader-bottom",w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"renderShader",value:function(){var e=this._getShaderHeight(),t=this._getShaderMaxHeight(),n=e>t;e>t&&(e=t),this._$shader.height(e);var i=this._workSpace._getGroupCount()||1;this._workSpace.isGroupedByDate()?this._renderGroupedByDateShaderParts(i,e,t,n):this._renderShaderParts(i,e,t,n)}},{key:"_renderShaderParts",value:function(e,t,n,i){for(var o=0;o<e;o++){var a=this._getShaderWidth(o);this._renderTopShader(this._$shader,t,a,o),!i&&this._renderBottomShader(this._$shader,n,t,a,o),this._renderAllDayShader(a,o)}}},{key:"_renderGroupedByDateShaderParts",value:function(e,t,n,i){var o=this._getShaderWidth(0),a=o-this._workSpace.getCellWidth();t<0&&(t=0,a=o),this._renderTopShader(this._$shader,t,o*e,0),!i&&this._renderBottomShader(this._$shader,n,t,a*e+this._workSpace.getCellWidth(),0),this._renderAllDayShader(o*e,0)}},{key:"_renderTopShader",value:function(e,t,n,i){this._$topShader=(0,m.default)("<div>").addClass(b),n&&this._$topShader.width(n)&&this._$topShader.height(t),this._$topShader.css("marginTop",this._getShaderTopOffset(i)),this._$topShader.css("left",this._getShaderOffset(i,n)),e.append(this._$topShader)}},{key:"_renderBottomShader",value:function(e,t,n,i,o){this._$bottomShader=(0,m.default)("<div>").addClass(x);var a=n<0?i:i-this._workSpace.getCellWidth(),r=n<0?t:t-n;this._$bottomShader.width(a)&&this._$bottomShader.height(r),this._$bottomShader.css("left",this._getShaderOffset(o,i-this._workSpace.getCellWidth())),e.append(this._$bottomShader)}},{key:"_renderAllDayShader",value:function(e,t){this._workSpace.option("showAllDayPanel")&&(this._$allDayIndicator=(0,m.default)("<div>").addClass(y),this._$allDayIndicator.height(this._workSpace.getAllDayHeight()),this._$allDayIndicator.width(e),this._$allDayIndicator.css("left",this._getShaderOffset(t,e)),this._workSpace._$allDayPanel.prepend(this._$allDayIndicator))}},{key:"_getShaderOffset",value:function(e,t){return this._workSpace.getGroupedStrategy().getShaderOffset(e,t)}},{key:"_getShaderTopOffset",value:function(e){return this._workSpace.getGroupedStrategy().getShaderTopOffset(e)}},{key:"_getShaderHeight",value:function(e,t){return this._workSpace.getGroupedStrategy().getShaderHeight()}},{key:"_getShaderMaxHeight",value:function(e,t){return this._workSpace.getGroupedStrategy().getShaderMaxHeight()}},{key:"_getShaderWidth",value:function(e){return this._workSpace.getGroupedStrategy().getShaderWidth(e)}},{key:"clean",value:function(){l(_(t.prototype),"clean",this).call(this),this._workSpace&&this._workSpace._$allDayPanel&&this._workSpace._$allDayPanel.find("."+y).remove()}}]),t}(v.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=i(n(215)),u=n(0),c=n(169),d="dx-scheduler-appointment",h=function(){function e(t){o(this,e),this.scheduler=t,this.appointments=t._appointments,this.initialPosition={left:0,top:0},this.appointmentInfo=null}return r(e,[{key:"isAllDay",value:function(e){return e.data("dxAppointmentSettings").allDay}},{key:"onDragStart",value:function(e){var t=e.itemSettings,n=e.itemData,i=e.initialPosition;this.initialPosition=i,this.appointmentInfo={appointment:n,settings:t},this.appointments.notifyObserver("hideAppointmentTooltip")}},{key:"onDragMove",value:function(e){e.fromComponent!==e.toComponent&&this.appointments.notifyObserver("removeDroppableCellClass")}},{key:"getAppointmentElement",value:function(e){var t=e.event.data&&e.event.data.itemElement||e.itemElement;return(0,s.default)(t)}},{key:"onDragEnd",value:function(e){var t=this.getAppointmentElement(e),n=this.appointments._getItemData(t),i=this.appointments._getAppointmentContainer(this.isAllDay(t));i.append(t),this.appointments.notifyObserver("updateAppointmentAfterDrag",{event:e,element:t,rawAppointment:n,coordinates:this.initialPosition})}},{key:"getItemData",value:function(e){var t=(0,s.default)(e).data(c.LIST_ITEM_DATA_KEY),n=null===t||void 0===t?void 0:t.appointment,i=this.appointments._getItemData(e);return n||i}},{key:"getItemSettings",value:function(e){var t=(0,s.default)(e).data(c.LIST_ITEM_DATA_KEY);return t&&t.settings||[]}},{key:"createDragStartHandler",value:function(e,t){var n=this;return function(i){i.itemData=n.getItemData(i.itemElement),i.itemSettings=n.getItemSettings(i.itemElement),t.onDragStart&&t.onDragStart(i),i.cancel||e.onDragStart(i)}}},{key:"createDragMoveHandler",value:function(e,t){return function(n){t.onDragMove&&t.onDragMove(n),n.cancel||e.onDragMove(n)}}},{key:"createDragEndHandler",value:function(e,t){var n=this;return function(i){n.appointmentInfo=null,t.onDragEnd&&t.onDragEnd(i),i.cancel||(e.onDragEnd(i),i.fromComponent!==i.toComponent&&t.onRemove&&t.onRemove(i))}}},{key:"createDropHandler",value:function(e){var t=this;return function(n){var i=t.appointments.invoke("getUpdatedData",n.itemData);n.itemData=(0,u.extend)({},n.itemData,i),n.fromComponent!==n.toComponent&&e.onAdd&&e.onAdd(n)}}},{key:"addTo",value:function(e,t){var n=this.scheduler.option("appointmentDragging")||{},i=(0,u.extend)({component:this.scheduler,contentTemplate:null,filter:".".concat(d),immediate:!1,onDragStart:this.onDragStart.bind(this),onDragMove:this.onDragMove.bind(this),onDragEnd:this.onDragEnd.bind(this)},t);this.appointments._createComponent(e,l.default,(0,u.extend)({},i,n,{onDragStart:this.createDragStartHandler(i,n),onDragMove:this.createDragMoveHandler(i,n),onDragEnd:this.createDragEndHandler(i,n),onDrop:this.createDropHandler(n)}))}},{key:"updateDragSource",value:function(e,t){var n=this.appointmentInfo;if(n||e){var i=e||n.appointment,o=t||n.settings;this.appointments._setDragSourceAppointment(i,o)}}}]),e}();t.default=h,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(16)),_=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"notifyObserver",value:function(e,t){var n=this.option("observer");n&&n.fire(e,t)}},{key:"invoke",value:function(){var e=this.option("observer");if(e)return e.fire.apply(e,arguments)}}]),t}(g.default),m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&r(e,t)}function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function s(e){var t=c();return function(){var n,i=d(e);if(t){var o=d(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return l(this,n)}}function l(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?u(e):t}function u(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function c(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function h(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function f(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?h(Object(n),!0).forEach(function(t){p(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):h(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function p(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function g(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function _(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function m(e,t,n){return t&&_(e.prototype,t),n&&_(e,n),e}t.default=void 0;var v=o(n(11)),y=o(n(5)),b=n(7),x=n(8),w=50,C=150,k=10,S=15,D=(0,x.addNamespace)("scroll","dxSchedulerVirtualScrolling"),I={vertical:"vertical",horizontal:"horizontal",both:"both"},T=I.vertical,O=function(){function e(t){g(this,e),this._workspace=t,this._rowHeight=w,this._cellWidth=C,this._renderer=new R(this.workspace),this._createVirtualScrolling(),this._attachScrollableEvents()}return m(e,[{key:"calculateCoordinatesByDataAndPosition",value:function(e,t,n){var i=this._workspace,o=t.rowIndex,a=t.columnIndex,r=e.startDate,s=e.endDate,l=e.allDay,u=n.getTime(),c=r.getTime(),d=s.getTime(),h=l?0:(u-c)/(d-c),f=i.getCellWidth(),p=(o+h)*this.rowHeight,g=f*a;return i.option("rtlEnabled")&&(g=i.getScrollableOuterWidth()-g),{top:p,left:g}}},{key:"dispose",value:function(){this._onScrollHandler&&y.default.off(this.document,D,this._onScrollHandler)}},{key:"_createVirtualScrolling",value:function(){this.verticalScrollingAllowed&&(this.verticalVirtualScrolling=new A({workspace:this.workspace,viewportHeight:this.viewportHeight,rowHeight:this.rowHeight})),this.horizontalScrollingAllowed&&(this.horizontalVirtualScrolling=new P({
workspace:this.workspace,viewportWidth:this.viewportWidth,cellWidth:this.cellWidth}))}},{key:"_attachScrollableEvents",value:function(){(this.height||this.width)&&this._attachScrollableScroll(),this.height&&this.width||this._attachWindowScroll()}},{key:"_attachScrollableScroll",value:function(){var e=this,t=this.workspace.getScrollable(),n=t.option("onScroll");t.option("onScroll",function(i){null===n||void 0===n?void 0:n.apply(t,[i]),e._process(null===i||void 0===i?void 0:i.scrollOffset)})}},{key:"_attachWindowScroll",value:function(){var e=this,t=(0,b.getWindow)();this._onScrollHandler=this.workspace._createAction(function(){var n=t.scrollX,i=t.scrollY;(n>=e.minScrollOffset||i>=e.minScrollOffset)&&e._process({left:n,top:i})}),y.default.on(this.document,D,this._onScrollHandler)}},{key:"_process",value:function(e){if(e){var t,n,i=e.left,o=e.top;null===(t=this.verticalVirtualScrolling)||void 0===t?void 0:t.updateState(o),null===(n=this.horizontalVirtualScrolling)||void 0===n?void 0:n.updateState(i),this.renderer.updateRender()}}},{key:"updateDimensions",value:function(){var e=this.workspace.getCellHeight(!1),t=this.workspace.getCellWidth(),n=e!==this.rowHeight||t!==this.cellWidth;n&&(this.rowHeight=e,this.cellWidth=t,this._createVirtualScrolling(),this.renderer._renderDateTable())}},{key:"workspace",get:function(){return this._workspace}},{key:"renderer",get:function(){return this._renderer}},{key:"isVirtualScrolling",get:function(){return this.workspace.isVirtualScrolling()}},{key:"minScrollOffset",get:function(){return k}},{key:"verticalVirtualScrolling",get:function(){return this._verticalVirtualScrolling},set:function(e){this._verticalVirtualScrolling=e}},{key:"horizontalVirtualScrolling",get:function(){return this._horizontalVirtualScrolling},set:function(e){this._horizontalVirtualScrolling=e}},{key:"document",get:function(){return v.default.getDocument()}},{key:"height",get:function(){return this.workspace.invoke("getOption","height")}},{key:"width",get:function(){return this.workspace.invoke("getOption","width")}},{key:"rowHeight",get:function(){return this._rowHeight},set:function(e){this._rowHeight=e}},{key:"viewportHeight",get:function(){return this.height?this.workspace.$element().height():(0,b.getWindow)().innerHeight}},{key:"cellWidth",get:function(){return this._cellWidth},set:function(e){this._cellWidth=e}},{key:"viewportWidth",get:function(){return this.width?this.workspace.$element().width():(0,b.getWindow)().innerWidth}},{key:"topVirtualRowsCount",get:function(){return this.verticalScrollingState.virtualItemCountBefore>0?1:0}},{key:"scrollingState",get:function(){var e,t;return{vertical:null===(e=this.verticalVirtualScrolling)||void 0===e?void 0:e.state,horizontal:null===(t=this.horizontalVirtualScrolling)||void 0===t?void 0:t.state}}},{key:"verticalScrollingState",get:function(){return this.scrollingState.vertical}},{key:"horizontalScrollingState",get:function(){return this.scrollingState.horizontal}},{key:"renderState",get:function(){var e,t,n=(null===(e=this.verticalVirtualScrolling)||void 0===e?void 0:e.getRenderState())||{},i=(null===(t=this.horizontalVirtualScrolling)||void 0===t?void 0:t.getRenderState())||{};return f(f({},n),i)}},{key:"scrollingType",get:function(){return this.workspace.option("scrolling.type")||T}},{key:"verticalScrollingAllowed",get:function(){return this.scrollingType===I.vertical||this.scrollingType===I.both}},{key:"horizontalScrollingAllowed",get:function(){return this.scrollingType===I.horizontal||this.scrollingType===I.both}}]),e}();t.default=O;var E=function(){function e(t){g(this,e),this._workspace=t.workspace,this._state=this.defaultState,this._viewportSize=t.viewportSize,this._itemSize=t.itemSize,this.updateState(0)}return m(e,[{key:"needUpdateState",value:function(e){var t=this.state,n=t.prevPosition,i=t.startIndex,o=i<0;if(!o&&(0===e||e===this.maxScrollPosition))return!0;var a=n,r=Math.floor(a/this.itemSize),s=Math.floor(e/this.itemSize),l=Math.abs(r-s)>=this.outlineCount;return o||l}},{key:"_correctPosition",value:function(e){return e<0?0:Math.min(e,this.maxScrollPosition)}},{key:"updateState",value:function(e){if(e=this._correctPosition(e),!this.needUpdateState(e))return!1;var t=this._calcItemInfoBefore(e),n=this._calcItemDeltaBefore(t),i=this._calcItemInfoAfter(n),o=i.outlineCountAfter,a=i.virtualItemCountAfter,r=i.itemCountWithAfter,s=t.virtualItemCountBefore,l=t.outlineCountBefore,u=l+r+o,c=Math.floor(e/this.itemSize);return this.state.prevPosition=c*this.itemSize,this.state.startIndex=c-l,this.state.virtualItemCountBefore=s,this.state.outlineCountBefore=l,this.state.itemCount=u,this.state.outlineCountAfter=o,this.state.virtualItemCountAfter=a,this._updateStateCore(),!0}},{key:"_calcItemInfoBefore",value:function(e){var t=Math.floor(e/this.itemSize),n=Math.min(t,this.outlineCount);return t-=n,{virtualItemCountBefore:t,outlineCountBefore:n}}},{key:"_calcItemDeltaBefore",value:function(e){var t=e.virtualItemCountBefore,n=e.outlineCountBefore,i=this.getTotalItemCount();return i-t-n}},{key:"getTotalItemCount",value:function(){throw"getTotalItemCount method should be implemented"}},{key:"getRenderState",value:function(){throw"getRenderState method should be implemented"}},{key:"_calcItemInfoAfter",value:function(e){var t=e>=this.pageSize?this.pageSize:e,n=e-t,i=n>0?Math.min(n,this.outlineCount):0;return n>0&&(n-=i),{virtualItemCountAfter:n,outlineCountAfter:i,itemCountWithAfter:t}}},{key:"_updateStateCore",value:function(){var e=this.state,t=e.virtualItemCountBefore,n=e.virtualItemCountAfter,i=e.outlineCountBefore,o=e.outlineCountAfter,a=e.virtualItemSizeBefore,r=e.virtualItemSizeAfter,s=e.outlineSizeBefore,l=e.outlineSizeAfter,u=this.itemSize*t,c=this.itemSize*n,d=this.itemSize*i,h=this.itemSize*o,f=a+s,p=u+d,g=r+l,_=c+h,m=f<p,v=g<_,y=m||v;y&&(e.virtualItemSizeBefore=u,e.virtualItemSizeAfter=c)}},{key:"viewportSize",get:function(){return this._viewportSize}},{key:"itemSize",get:function(){return this._itemSize}},{key:"state",get:function(){return this._state},set:function(e){this._state=e}},{key:"startIndex",get:function(){return this.state.startIndex}},{key:"pageSize",get:function(){return Math.ceil(this.viewportSize/this.itemSize)}},{key:"outlineCount",get:function(){return Math.floor(this.pageSize/2)}},{key:"workspace",get:function(){return this._workspace}},{key:"groupCount",get:function(){return this.workspace._getGroupCount()}},{key:"isVerticalGrouping",get:function(){return this.workspace._isVerticalGroupedWorkSpace()}},{key:"defaultState",get:function(){return{prevPosition:0,startIndex:-1,itemCount:0,virtualItemCountBefore:0,virtualItemCountAfter:0,outlineCountBefore:0,outlineCountAfter:0,virtualItemSizeBefore:0,virtualItemSizeAfter:0,outlineSizeBefore:0,outlineSizeAfter:0}}},{key:"maxScrollPosition",get:function(){return this.getTotalItemCount()*this.itemSize-this.viewportSize}}]),e}(),A=function(e){function t(e){return g(this,t),n.call(this,{workspace:e.workspace,viewportSize:e.viewportHeight,itemSize:e.rowHeight})}a(t,e);var n=s(t);return m(t,[{key:"getTotalItemCount",value:function(){return this.workspace._getTotalRowCount(this.groupCount,this.isVerticalGrouping)}},{key:"getRenderState",value:function(){return{topVirtualRowHeight:this.state.virtualItemSizeBefore,bottomVirtualRowHeight:this.state.virtualItemSizeAfter,startRowIndex:this.state.startIndex,rowCount:this.state.itemCount,startIndex:this.state.startIndex}}},{key:"prevTopPosition",get:function(){return this.state.prevPosition}},{key:"rowCount",get:function(){return this.state.itemCount}},{key:"topVirtualRowCount",get:function(){return this.state.virtualItemCountBefore}},{key:"bottomVirtualRowCount",get:function(){return this.state.virtualItemCountAfter}}]),t}(E),P=function(e){function t(e){return g(this,t),n.call(this,{workspace:e.workspace,viewportSize:e.viewportWidth,itemSize:e.cellWidth})}a(t,e);var n=s(t);return m(t,[{key:"getTotalItemCount",value:function(){return this.workspace._getTotalCellCount(this.groupCount,this.isVerticalGrouping)}},{key:"getRenderState",value:function(){return{leftVirtualCellWidth:this.state.virtualItemSizeBefore,rightVirtualCellWidth:this.state.virtualItemSizeAfter,startCellIndex:this.state.startIndex,cellCount:this.state.itemCount,cellWidth:this.state.itemSize}}}]),t}(E),R=function(){function e(t){g(this,e),this._workspace=t,this._renderAppointmentTimeout=null}return m(e,[{key:"getRenderTimeout",value:function(){return S}},{key:"updateRender",value:function(){this._renderDateTable(),this._renderAppointments()}},{key:"_renderDateTable",value:function(){this.workspace.renderRWorkspace(!1)}},{key:"_renderAppointments",value:function(){var e=this,t=this.getRenderTimeout();t>=0?(clearTimeout(this._renderAppointmentTimeout),this._renderAppointmentTimeout=setTimeout(function(){return e.workspace.updateAppointments()},t)):this.workspace.updateAppointments()}},{key:"workspace",get:function(){return this._workspace}}]),e}();e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?o(Object(n),!0).forEach(function(t){r(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):o(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function s(e){return d(e)||c(e)||u(e)||l()}function l(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function u(e,t){if(e){if("string"==typeof e)return h(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?h(e,t):void 0}}function c(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function d(e){if(Array.isArray(e))return h(e)}function h(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function f(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function p(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function g(e,t,n){return t&&p(e.prototype,t),n&&p(e,n),e}t.default=void 0;var _=i(n(20)),m=function(){function e(t){f(this,e),this.workspace=t}return g(e,[{key:"_getCompleteViewDataMap",value:function(e){for(var t=e.nonVirtualRowCount,n=e.cellCount,i=e.verticalGroupCount,o=[],a=0;a<i;a+=1){var r=this._generateAllDayPanelData(e,a,t,n),l=this._generateViewCellsData(e,t,0,t*a);r&&o.push(r),o.push.apply(o,s(l))}return o}},{key:"_generateViewDataMap",value:function(e,t){var n=t.startRowIndex,i=t.rowCount,o=this.workspace._isVerticalGroupedWorkSpace(),a=this.workspace._isShowAllDayPanel(),r=o||!a?0:1,s=n+r;return e.slice(s,s+i).map(function(e,t){return e.map(function(e,n){return{cellData:e,position:{rowIndex:t,cellIndex:n}}})})}},{key:"_getViewDataFromMap",value:function(e,t,n){var i=n.topVirtualRowHeight,o=n.bottomVirtualRowHeight,a=n.cellCountInGroupRow,r=this.workspace.isGroupedAllDayPanel(),s=e.reduce(function(e,t){var n=e.previousGroupIndex,i=e.previousGroupedData,o=t.map(function(e){var t=e.cellData;return t}),a=o[0],s=a.allDay,l=a.groupIndex;return l!==n&&i.push({dateTable:[],isGroupedAllDayPanel:r,groupIndex:l}),s?i[i.length-1].allDayPanel=o:i[i.length-1].dateTable.push(o),{previousGroupedData:i,previousGroupIndex:l}},{previousGroupIndex:-1,previousGroupedData:[]}),l=s.previousGroupedData,u=this.workspace.isVirtualScrolling(),c=this.workspace._isVerticalGroupedWorkSpace(),d=this.workspace._isShowAllDayPanel();return!c&&d&&(l[0].allDayPanel=t[0]),{groupedData:l,isVirtual:u,topVirtualRowHeight:i,bottomVirtualRowHeight:o,cellCountInGroupRow:a}}},{key:"_generateViewCellsData",value:function(e,t,n,i){for(var o=e.cellCount,a=e.cellDataGetters,r=e.rowCountInGroup,s=[],l=0;l<t;++l){var u=n+i+l,c=u%r;s.push(this._generateCellsRow(e,a,u,o,c))}return s}},{key:"_generateAllDayPanelData",value:function(e,t,n,i){var o=this.workspace;if(!o._isShowAllDayPanel())return null;var a=Math.max(t*n,0);return this._generateCellsRow(e,[o._getAllDayCellData.bind(o)],a,i,0,t)}},{key:"_generateCellsRow",value:function(e,t,n,i,o,r){for(var s=this,l=[],u=e.horizontalGroupCount,c=e.groupOrientation,d=e.rowCountInGroup,h=e.cellCountInGroupRow,f=e.groupCount,p=function(e){var p=t.reduce(function(t,i){return a(a({},t),i(void 0,n,e,r).value)},{});p.index=s._calculateCellIndex(u,c,s._workspace.isGroupedByDate(),o,e,i),p.isFirstGroupCell=s._isFirstGroupCell(n,e,d,h,f),p.isLastGroupCell=s._isLastGroupCell(n,e,d,h,f),p.key=s._getKeyByRowAndColumn(n,e,i),l.push(p)},g=0;g<i;++g)p(g);return l}},{key:"_calculateCellIndex",value:function(e,t,n,i,o,a){var r=e||1,s=i*a+o,l=a/r;if("horizontal"===t){var u=o%l;n&&(u=Math.floor(o/r)),s=i*l+u}return s}},{key:"_getKeyByRowAndColumn",value:function(e,t,n){return e*n+t}},{key:"generateGroupedDataMap",value:function(e){var t=e.reduce(function(e,t){var n=e.previousGroupedDataMap,i=e.previousRowIndex,o=e.previousGroupIndex,a=t[0].cellData.groupIndex,r=a===o?i+1:0;return t.forEach(function(e){var t=e.cellData.groupIndex;n[t]||(n[t]=[]),n[t][r]||(n[t][r]=[]),n[t][r].push(e)}),{previousGroupedDataMap:n,previousRowIndex:r,previousGroupIndex:a}},{previousGroupedDataMap:[],previousRowIndex:-1,previousGroupIndex:-1}),n=t.previousGroupedDataMap;return n}},{key:"_isFirstGroupCell",value:function(e,t,n,i,o){return this.workspace.isGroupedByDate()?t%o===0:this.workspace._isHorizontalGroupedWorkSpace()||0===o?t%i===0:e%n===0}},{key:"_isLastGroupCell",value:function(e,t,n,i,o){return this.workspace.isGroupedByDate()?(t+1)%o===0:this.workspace._isHorizontalGroupedWorkSpace()||0===o?(t+1)%i===0:(e+1)%n===0}},{key:"workspace",get:function(){return this._workspace},set:function(e){this._workspace=e}}]),e}(),v=function(){function e(t){f(this,e),this._viewDataGenerator=null,this._viewData=[],this._completeViewDataMap=[],this._completeGroupedViewDataMap=[],this._viewDataMap=[],this._groupedDataMap=[],this._workspace=t}return g(e,[{key:"update",value:function(e){var t=this.viewDataGenerator,n=this._workspace,i=n.generateRenderOptions();e&&(this.completeViewDataMap=t._getCompleteViewDataMap(i)),this.viewDataMap=t._generateViewDataMap(this.completeViewDataMap,i),this.viewData=t._getViewDataFromMap(this.viewDataMap,this.completeViewDataMap,i),this.groupedDataMap=t.generateGroupedDataMap(this.viewDataMap)}},{key:"getStartDate",value:function(){var e=this.viewData.groupedData,t=e[0].dateTable;return t[0][0].startDate}},{key:"getGroupStartDate",value:function(e){var t=this.getGroupData(e),n=t.dateTable;return n[0][0].startDate}},{key:"getGroupEndDate",value:function(e){var t=this.getGroupData(e),n=t.dateTable,i=n.length-1,o=n[i].length-1;return n[i][o].endDate}},{key:"findGroupCellStartDate",value:function(e,t,n,i){if(i)return this.findAllDayGroupCellStartDate(e,t);var o=this.getGroupData(e),a=o.dateTable;if(a.length)for(var r=0;r<a[0].length;++r){var s=a[0][r],l=a.length-1;if(_.default.sameDate(s.startDate,t)){var u,c=a[l][r];if(c.endDate<=t&&n.getDate()>t.getDate()&&(s=a[0][r+1],c=a[l][r+1]),(null===(u=c)||void 0===u?void 0:u.endDate)>t)return s.startDate}}}},{key:"findAllDayGroupCellStartDate",value:function(e,t){var n=this.getGroupStartDate(e);return n>t?n:t}},{key:"getCellsGroup",value:function(e){var t=this.getGroupData(e),n=t.dateTable;return n[0][0].groups}},{key:"getCellData",value:function(e,t,n){if(n&&!this._workspace._isVerticalGroupedWorkSpace())return this._viewData.groupedData[0].allDayPanel[t];var i=this.viewDataMap[e][t].cellData;return i}},{key:"getCellsByGroupIndexAndAllDay",value:function(e,t){var n=this._workspace,i=n._getRowCountWithAllDayRows(),o=n._isVerticalGroupedWorkSpace(),a=n._isShowAllDayPanel(),r=o?e*i:0,s=o?(e+1)*i-1:i,l=a&&!t?r+1:r,u=t?l:s;return this.completeViewDataMap.slice(l,u+1).map(function(t){return t.filter(function(t){var n=t.groupIndex;return e===n})})}},{key:"findCellPositionInMap",value:function(e,t,n){for(var i=n?_.default.trimTime(t).getTime():t.getTime(),o=function(e){var t=e.startDate.getTime(),o=e.endDate.getTime();return n?e.allDay&&i>=t&&i<=o:i>=t&&i<o},a=n&&!this._workspace._isVerticalGroupedWorkSpace()?[this.completeViewDataMap[0].map(function(e,t){return{cellData:e,position:{cellIndex:t,rowIndex:0}}})]:this.groupedDataMap[e]||[],r=0;r<a.length;++r)for(var s=a[r],l=0;l<s.length;++l){var u=s[l],c=u.cellData;if(c.groupIndex===e&&o(c))return u.position}}},{key:"getGroupIndices",value:function(){var e=this.viewData.groupedData;return e.map(function(e){var t=e.groupIndex;return t})}},{key:"_getLastGroupRow",value:function(e){var t=this.groupedDataMap[e],n=t.length-1;return t[n]}},{key:"getLasGroupCellPosition",value:function(e){var t=this._getLastGroupRow(e);return t[0].position}},{key:"getLasGroupCellIndex",value:function(e){var t=this.groupedDataMap[e];return t.length-1}},{key:"getRowCountInGroup",value:function(e){var t=this._getLastGroupRow(e),n=t.length,i=t[n-1].cellData,o=i.index;return(o+1)/t.length}},{key:"getGroupData",value:function(e){var t=this.viewData.groupedData;return t.filter(function(t){return t.groupIndex===e})[0]}},{key:"isGroupIntersectDateInterval",value:function(e,t,n){var i=this.getGroupStartDate(e),o=this.getGroupEndDate(e);return t<o&&n>i}},{key:"findGlobalCellPosition",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=this.completeViewDataMap,o=this._workspace,a=o._isShowAllDayPanel(),r=o._isVerticalGroupedWorkSpace(),s=0;s<i.length;s+=1)for(var l=i[s],u=0;u<l.length;u+=1){var c=l[u],d=c.startDate,h=c.endDate,f=c.groupIndex,p=c.allDay;if(t===f&&n===p&&this._compareDatesAndAllDay(e,d,h,n))return{position:{columnIndex:u,rowIndex:a&&!r?s-1:s},cellData:c}}}},{key:"_compareDatesAndAllDay",value:function(e,t,n,i){var o=e.getTime(),a=_.default.trimTime(e).getTime(),r=t.getTime(),s=n.getTime();return!i&&o>=r&&o<s||i&&a===r}},{key:"viewDataGenerator",get:function(){return this._viewDataGenerator||(this._viewDataGenerator=new m(this._workspace)),this._viewDataGenerator}},{key:"completeViewDataMap",get:function(){return this._completeViewDataMap},set:function(e){this._completeViewDataMap=e}},{key:"completeGroupedViewDataMap",get:function(){return this._completeGroupedViewDataMap},set:function(e){this._completeGroupedViewDataMap=e}},{key:"viewData",get:function(){return this._viewData},set:function(e){this._viewData=e}},{key:"viewDataMap",get:function(){return this._viewDataMap},set:function(e){this._viewDataMap=e}},{key:"groupedDataMap",get:function(){return this._groupedDataMap},set:function(e){this._groupedDataMap=e}}]),e}();t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(198)),y=n(829),b=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getProps",value:function(){var e=l(_(t.prototype),"getProps",this).call(this);return e.dataCellTemplate=this._createTemplateComponent(e,e.dataCellTemplate),e}},{key:"_propsInfo",get:function(){return{twoWay:[],allowNull:[],elements:[]}}},{key:"_viewComponent",get:function(){return y.AllDayPanelLayout}}]),t}(v.default);t.default=b,(0,m.default)("dxAllDayPanelLayout",b),e.exports=t.default},function(e,t,n){t.removeDifferentElements=t.wrapElement=void 0;var i=n(3),o=function(e,t){(0,i.each)(t,function(t,n){var i=n.name,o=n.value;"class"===i?e.addClass(o):e.attr(i,o)})},a=function(e,t){var n=t.get(0),a=n.attributes,r=t.contents();return o(e,a),t.remove(),(0,i.each)(r,function(t,n){e.append(n)}),r};t.wrapElement=a;var r=function(e,t){(0,i.each)(t,function(t,n){var o=!1;(0,i.each)(e,function(e,t){n===t&&(o=!0)}),!o&&n.parentNode&&n.parentNode.removeChild(n)})};t.removeDifferentElements=r},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,y.useCallback)(function(){return e.viewData.groupedData[0].allDayPanel},[e.viewData]),n=(0,y.useCallback)(function(){return t()?void 0:m.DefaultSizes.allDayPanelHeight},[e.viewData]),i=(0,y.useCallback)(function(){return(0,f.combineClasses)(c({"dx-scheduler-all-day-panel":!0,"dx-hidden":!e.visible},e.className,!!e.className))},[e.visible,e.className]),o=(0,y.useCallback)(function(){var t=(e.className,e.dataCellTemplate,e.groupOrientation,e.viewData,e.visible,r(e,["className","dataCellTemplate","groupOrientation","viewData","visible"]));return t},[e]);return b({props:u(u({},e),{},{dataCellTemplate:w(e.dataCellTemplate)}),allDayPanelData:t(),emptyTableHeight:n(),classes:i(),restAttributes:o()})}t.AllDayPanelLayout=h,t.AllDayPanelLayoutProps=t.viewFunction=void 0;var f=n(226),p=n(313),g=n(527),_=n(170),m=n(834),v=a(n(51)),y=n(53),b=function(e){return v.h("div",d({},e.restAttributes,{className:e.classes}),e.props.visible&&v.h(p.Table,{className:"dx-scheduler-all-day-table",height:e.emptyTableHeight},v.h(g.AllDayPanelTableBody,{viewData:e.allDayPanelData,dataCellTemplate:e.props.dataCellTemplate})))};t.viewFunction=b;var x=u(u({},_.LayoutProps),{},{className:"",visible:!0});t.AllDayPanelLayoutProps=x;var w=function(e){return e&&(e.defaultProps?function(t){return v.h(e,d({},t))}:e)};h.defaultProps=u({},x)},function(e,t,n){t.HORIZONTAL_GROUP_ORIENTATION=t.VERTICAL_GROUP_ORIENTATION=void 0;var i="vertical";t.VERTICAL_GROUP_ORIENTATION=i;var o="horizontal";t.HORIZONTAL_GROUP_ORIENTATION=o},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e){return h(e)||d(e)||c(e)||u()}function u(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function c(e,t){if(e){if("string"==typeof e)return f(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?f(e,t):void 0}}function d(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function h(e){if(Array.isArray(e))return f(e)}function f(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function p(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function g(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?p(Object(n),!0).forEach(function(t){_(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):p(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function _(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function m(){return m=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},m.apply(this,arguments)}function v(e){var t=(0,C.useCallback)(function(){var t=e.height,n=o(),i=n.style;return(0,y.addHeightToStyle)(t,i)},[e.height]),n=(0,C.useCallback)(function(){var t=e.className;return"dx-scheduler-virtual-row ".concat(t)},[e.className]),i=(0,C.useCallback)(function(){var t=e.cellsCount;return l(Array(t))},[e.cellsCount]),o=(0,C.useCallback)(function(){var t=(e.cellsCount,e.children,e.className,e.height,r(e,["cellsCount","children","className","height"]));return t},[e]);return k({props:g({},e),style:t(),classes:n(),virtualCells:i(),restAttributes:o()})}t.VirtualRow=v,t.VirtualRowProps=t.viewFunction=void 0;var y=n(199),b=n(227),x=n(832),w=a(n(51)),C=n(53),k=function(e){return w.h(b.Row,m({},e.restAttributes,{style:e.style,className:e.classes}),e.virtualCells.map(function(e,t){return w.h(x.VirtualCell,{key:t.toString()})}))};t.viewFunction=k;var S=g(g({},b.RowProps),{},{cellsCount:1});t.VirtualRowProps=S,v.defaultProps=g({},S)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,p.useCallback)(function(){var t=(e.className,u(e,["className"]));return t},[e]);return g({props:s({},e),restAttributes:t()})}t.VirtualCell=h,t.VirtualCellProps=t.viewFunction=void 0;var f=a(n(51)),p=n(53),g=function(e){return f.h("td",d({},e.restAttributes,{
className:"dx-scheduler-virtual-cell ".concat(e.props.className)}))};t.viewFunction=g;var _={className:""};t.VirtualCellProps=_,h.defaultProps=s({},_)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,g.useCallback)(function(){var t=(e.allDay,e.children,e.className,e.contentTemplate,e.contentTemplateProps,e.dataCellTemplate,e.endDate,e.groupIndex,e.groups,e.index,e.isFirstGroupCell,e.isLastGroupCell,e.startDate,e.text,u(e,["allDay","children","className","contentTemplate","contentTemplateProps","dataCellTemplate","endDate","groupIndex","groups","index","isFirstGroupCell","isLastGroupCell","startDate","text"]));return t},[e]);return _({props:s(s({},e),{},{dataCellTemplate:m(e.dataCellTemplate),contentTemplate:m(e.contentTemplate)}),restAttributes:t()})}t.AllDayPanelCell=h,t.viewFunction=void 0;var f=n(314),p=a(n(51)),g=n(53),_=function(e){return p.h(f.DateTableCellBase,d({},e.restAttributes,{className:"dx-scheduler-all-day-table-cell ".concat(e.props.className),startDate:e.props.startDate,endDate:e.props.endDate,groups:e.props.groups,groupIndex:e.props.groupIndex,allDay:!0,isFirstGroupCell:e.props.isFirstGroupCell,isLastGroupCell:e.props.isLastGroupCell,index:e.props.index,dataCellTemplate:e.props.dataCellTemplate}))};t.viewFunction=_;var m=function(e){return e&&(e.defaultProps?function(t){return p.h(e,d({},t))}:e)};h.defaultProps=s({},f.DateTableCellBaseProps)},function(e,t,n){t.DefaultSizes=void 0;var i={allDayPanelHeight:25};t.DefaultSizes=i},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(9)),_=o(n(198)),m=n(528),v=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_propsInfo",get:function(){return{twoWay:[],allowNull:[],elements:[]}}},{key:"_viewComponent",get:function(){return m.AllDayPanelTitle}}]),t}(_.default);t.default=v,(0,g.default)("dxAllDayPanelTitle",v),e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(198)),y=n(837),b=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getProps",value:function(){var e=l(_(t.prototype),"getProps",this).call(this);return e.timeCellTemplate=this._createTemplateComponent(e,e.timeCellTemplate),e.dataCellTemplate=this._createTemplateComponent(e,e.dataCellTemplate),e}},{key:"_propsInfo",get:function(){return{twoWay:[],allowNull:[],elements:[]}}},{key:"_viewComponent",get:function(){return y.TimePanelTableLayout}}]),t}(v.default);t.default=b,(0,m.default)("dxTimePanelTableLayout",b),e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,x.useCallback)(function(){var t=e.viewData;return!!t.isVirtual},[e.viewData]),n=(0,x.useCallback)(function(){return e.viewData.topVirtualRowHeight||0},[e.viewData]),i=(0,x.useCallback)(function(){return e.viewData.bottomVirtualRowHeight||0},[e.viewData]),o=(0,x.useCallback)(function(){var t=e.groupOrientation;return(0,_.isVerticalGroupOrientation)(t)},[e.groupOrientation]),a=(0,x.useCallback)(function(){var t=(e.allDayPanelVisible,e.className,e.dataCellTemplate,e.groupOrientation,e.timeCellTemplate,e.viewData,r(e,["allDayPanelVisible","className","dataCellTemplate","groupOrientation","timeCellTemplate","viewData"]));return t},[e]);return w({props:u(u({},e),{},{timeCellTemplate:k(e.timeCellTemplate),dataCellTemplate:k(e.dataCellTemplate)}),isVirtual:t(),topVirtualRowHeight:n(),bottomVirtualRowHeight:i(),isVerticalGroupOrientation:o(),restAttributes:a()})}t.TimePanelTableLayout=h,t.TimePanelTableLayoutProps=t.viewFunction=void 0;var f=n(227),p=n(838),g=n(315),_=n(199),m=n(313),v=n(170),y=n(528),b=a(n(51)),x=n(53),w=function(e){var t=e.bottomVirtualRowHeight,n=e.isVerticalGroupOrientation,i=e.isVirtual,o=e.props,a=e.restAttributes,r=e.topVirtualRowHeight,s=o.timeCellTemplate,l=o.viewData;return b.h(m.Table,d({},a,{isVirtual:i,topVirtualRowHeight:r,bottomVirtualRowHeight:t,virtualCellsCount:1,className:"dx-scheduler-time-panel"}),l.groupedData.map(function(e,t){var i=e.dateTable,o=e.groupIndex;return b.h(b.Fragment,{key:(0,_.getKeyByGroup)(o)},(0,_.getIsGroupedAllDayPanel)(l,t)&&b.h(f.Row,null,b.h(g.CellBase,{className:"dx-scheduler-time-panel-title-cell"},b.h(y.AllDayPanelTitle,null))),i.map(function(e){var t=l.cellCountInGroupRow,i=e[0],a=i.groups,r=i.index,u=i.isFirstGroupCell,c=i.isLastGroupCell,d=i.key,h=i.startDate,g=i.text;return b.h(f.Row,{className:"dx-scheduler-time-panel-row",key:d},b.h(p.TimePanelCell,{startDate:h,text:g,groups:n?a:void 0,groupIndex:n?o:void 0,isFirstGroupCell:n&&u,isLastGroupCell:n&&c,index:Math.floor(r/t),timeCellTemplate:s}))}))}))};t.viewFunction=w;var C=u(u({},v.LayoutProps),{},{className:"",allDayPanelVisible:!1});t.TimePanelTableLayoutProps=C;var k=function(e){return e&&(e.defaultProps?function(t){return b.h(e,d({},t))}:e)};h.defaultProps=u({},C)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,g.useCallback)(function(){var t=e.groupIndex,n=e.groups,i=e.index,o=e.startDate,a=e.text;return{data:{date:o,groups:n,groupIndex:t,text:a},index:i}},[e.groupIndex,e.groups,e.index,e.startDate,e.text]),n=(0,g.useCallback)(function(){var t=(e.allDay,e.children,e.className,e.contentTemplate,e.contentTemplateProps,e.endDate,e.groupIndex,e.groups,e.index,e.isFirstGroupCell,e.isLastGroupCell,e.startDate,e.text,e.timeCellTemplate,r(e,["allDay","children","className","contentTemplate","contentTemplateProps","endDate","groupIndex","groups","index","isFirstGroupCell","isLastGroupCell","startDate","text","timeCellTemplate"]));return t},[e]);return _({props:u(u({},e),{},{timeCellTemplate:v(e.timeCellTemplate),contentTemplate:v(e.contentTemplate)}),timeCellTemplateProps:t(),restAttributes:n()})}t.TimePanelCell=h,t.TimePanelCellProps=t.viewFunction=void 0;var f=n(315),p=a(n(51)),g=n(53),_=function(e){return p.h(f.CellBase,d({},e.restAttributes,{isFirstGroupCell:e.props.isFirstGroupCell,isLastGroupCell:e.props.isLastGroupCell,contentTemplate:e.props.timeCellTemplate,contentTemplateProps:e.timeCellTemplateProps,className:"dx-scheduler-time-panel-cell dx-scheduler-cell-sizes-vertical ".concat(e.props.className)}),p.h("div",null,e.props.text))};t.viewFunction=_;var m=u({},f.CellBaseProps);t.TimePanelCellProps=m;var v=function(e){return e&&(e.defaultProps?function(t){return p.h(e,d({},t))}:e)};h.defaultProps=u({},m)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function c(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function d(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function h(e,t,n){return t&&d(e.prototype,t),n&&d(e,n),e}t.default=void 0;var f=i(n(20)),p=function(){function e(t){c(this,e),this._viewDataProvider=t,this._focusedCell=null,this._selectedCells=null,this._firstSelectedCell=null}return h(e,[{key:"setFocusedCell",value:function(e,t,n){if(e>=0){var i=this._viewDataProvider.getCellData(e,t,n);this._focusedCell=i}}},{key:"getFocusedCell",value:function(e){var t=this._focusedCell;if(t){var n=this._getColumnIndexByCellData(t,e),i=this._getRowIndexByColumnAndData(t,n,e);return{coordinates:{cellIndex:n,rowIndex:i},cellData:t}}}},{key:"setSelectedCells",value:function(e){var t=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:void 0,i=this._viewDataProvider,a=e.rowIndex,r=e.columnIndex,s=e.allDay;if(!(a<0)){var l=n?i.getCellData(n.rowIndex,n.columnIndex,n.allDay):this._firstSelectedCell,u=i.getCellData(a,r,s);if(this._firstSelectedCell=l,l.startDate.getTime()>u.startDate.getTime()){var c=[u,l];l=c[0],u=c[1]}var d=l,h=d.startDate,f=d.groupIndex,p=u,g=p.startDate,_=i.getCellsByGroupIndexAndAllDay(f,s),m=_.reduce(function(e,n){var i=t._filterCellsByDate(n,h,g);return e.push.apply(e,o(i)),e},[]);this._selectedCells=m.sort(function(e,t){return e.startDate.getTime()-t.startDate.getTime()})}}},{key:"getSelectedCells",value:function(){return this._selectedCells}},{key:"releaseSelectedAndFocusedCells",value:function(){this.releaseSelectedCells(),this.releaseFocusedCell()}},{key:"releaseSelectedCells",value:function(){this._selectedCells=null,this._firstSelectedCell=null}},{key:"releaseFocusedCell",value:function(){this._focusedCell=null}},{key:"isValidFocusedCell",value:function(e){var t=this._focusedCell;if(!t)return!0;var n=t.groupIndex,i=t.allDay,o=e.groupIndex,a=e.allDay;return n===o&&i===a}},{key:"_getColumnIndexByCellData",value:function(e,t){for(var n=this._viewDataProvider.viewDataMap,i=e.startDate,o=e.groupIndex,a=n[0],r=f.default.trimTime(i).getTime(),s=0;s<a.length;s+=1){var l=a[s].cellData,u=l.startDate,c=l.groupIndex;if(r===f.default.trimTime(u).getTime()&&(o===c||t))return s}}},{key:"_getRowIndexByColumnAndData",value:function(e,t,n){var i=this._viewDataProvider.viewDataMap,o=e.startDate,a=e.groupIndex,r=e.allDay;if(r&&!n)return 0;for(var s=0;s<i.length;s+=1){var l=i[s][t].cellData,u=l.startDate,c=l.groupIndex,d=l.allDay;if(o.getTime()===u.getTime()&&a===c&&r===d)return s}}},{key:"_filterCellsByDate",value:function(e,t,n){var i=t.getTime(),o=n.getTime();return e.filter(function(e){var t=e.startDate,n=t.getTime();return i<=n&&n<=o})}}]),e}();t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(9)),_=o(n(316)),m="dx-scheduler-timeline-day",v=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getElementClass",value:function(){return m}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=this.option("currentDate"),this._setStartDayHour(this._firstViewDate)}},{key:"_needRenderWeekHeader",value:function(){return this._isWorkSpaceWithCount()}}]),t}(_.default);(0,g.default)("dxSchedulerTimelineDay",v);var y=v;t.default=y,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=n(24),_=o(n(526)),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"renderShader",value:function(){for(var e=this._workSpace._isHorizontalGroupedWorkSpace()?this._workSpace._getGroupCount():1,t=0;t<e;t++){var n=0===t,i=n?this._$shader:this.createShader();this.applyShaderMargin(i),this._workSpace.isGroupedByDate()?this._customizeGroupedByDateShader(i,t):this._customizeShader(i,t),!n&&this._shader.push(i)}}},{key:"_customizeShader",value:function(e,t){var n=this._workSpace.getIndicationWidth();if(this._applyShaderWidth(e,n),t>=1){var i=this._workSpace,o=i._getCellCount()*i.getCellWidth();e.css("left",o)}else e.css("left",0)}},{key:"_applyShaderWidth",value:function(e,t){var n=(0,g.getBoundingRect)(this._$container.get(0)).width;t>n&&(t=n),t>0&&e.width(t)}},{key:"_customizeGroupedByDateShader",value:function(e,t){var n,i=this._workSpace.getIndicationCellCount(),o=Math.floor(i),a=i-o,r=0===t,s=this._workSpace,l=r?s.getIndicationWidth():a*s.getCellWidth();this._applyShaderWidth(e,l),n=r?s._getCellCount()*s.getCellWidth()*t:s.getCellWidth()*o*s._getGroupCount()+t*s.getCellWidth(),e.css("left",n)}}]),t}(_.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(316)),y=o(n(20)),b="dx-scheduler-timeline-month",x=864e5,w=y.default.dateToMilliseconds,C=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_renderView",value:function(){l(_(t.prototype),"_renderView",this).call(this),this._updateScrollable()}},{key:"_getElementClass",value:function(){return b}},{key:"_getDateHeaderTemplate",value:function(){return this.option("dateCellTemplate")}},{key:"_getHiddenInterval",value:function(){return 0}},{key:"_calculateDurationInCells",value:function(e){return e/this.getCellDuration()}},{key:"getCellDuration",value:function(){return w("day")}},{key:"calculateEndViewDate",value:function(e){return new Date(e.getTime()+this._calculateDayDuration()*w("hour"))}},{key:"isIndicatorVisible",value:function(){return!0}},{key:"_getCellCount",value:function(){var e=this.option("currentDate"),t=0;if(this._isWorkSpaceWithCount())for(var n=this.option("intervalCount"),i=1;i<=n;i++)t+=new Date(e.getFullYear(),e.getMonth()+i,0).getDate();else t=new Date(e.getFullYear(),e.getMonth()+1,0).getDate();return t}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=y.default.getFirstMonthDate(this.option("currentDate")),this._setStartDayHour(this._firstViewDate)}},{key:"_getFormat",value:function(){return this._formatWeekdayAndDay}},{key:"_getDateByIndex",value:function(e){var t=new Date(this._firstViewDate);return t.setDate(this._firstViewDate.getDate()+e),t}},{key:"_getInterval",value:function(){return x}},{key:"_getIntervalBetween",value:function(e){var t=this.getStartViewDate(),n=y.default.getTimezonesDifference(t,e);return e.getTime()-(t.getTime()-36e5*this.option("startDayHour"))-n}},{key:"calculateEndDate",value:function(e){var t=new Date(e);return new Date(t.setHours(this.option("endDayHour")))}},{key:"_calculateHiddenInterval",value:function(){return 0}},{key:"_getDateByCellIndexes",value:function(e,n){var i=l(_(t.prototype),"_getDateByCellIndexes",this).call(this,e,n);return this._setStartDayHour(i),i}},{key:"getPositionShift",value:function(){return{top:0,left:0,cellPosition:0}}}]),t}(v.default);(0,m.default)("dxSchedulerTimelineMonth",C);var k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(529)),y=o(n(20)),b=o(n(530)),x=y.default.dateToMilliseconds,w="dx-scheduler-timeline-work-week",C=5,k=function(e){function t(){var e;a(this,t);for(var i=arguments.length,o=new Array(i),r=0;r<i;r++)o[r]=arguments[r];return e=n.call.apply(n,[this].concat(o)),e._getWeekendsCount=b.default.getWeekendsCount,e._isSkippedData=b.default.isDataOnWeekend,e}c(t,e);var n=h(t);return s(t,[{key:"_getElementClass",value:function(){return w}},{key:"_getWeekDuration",value:function(){return 5}},{key:"_firstDayOfWeek",value:function(){return b.default.getFirstDayOfWeek(this.option("firstDayOfWeek"))}},{key:"_isSkippedData",value:function(){return b.default.isDataOnWeekend}},{key:"_incrementDate",value:function(e){var n=e.getDay();n===C&&e.setDate(e.getDate()+2),l(_(t.prototype),"_incrementDate",this).call(this,e)}},{key:"_getOffsetByCount",value:function(e){
var t=Math.floor(e/(5*this._getCellCountInDay()));return x("day")*t*2}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=b.default.getFirstViewDate(this.option("currentDate"),this._firstDayOfWeek()),this._setStartDayHour(this._firstViewDate)}}]),t}(v.default);(0,m.default)("dxSchedulerTimelineWorkWeek",k);var S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(531)),y=o(n(845)),b="dx-scheduler-work-space-day",x=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_getElementClass",value:function(){return b}},{key:"_getRowCount",value:function(){return this._getCellCountInDay()}},{key:"_getCellCount",value:function(){return this.option("intervalCount")}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=this._getViewStartByOptions(),this._setStartDayHour(this._firstViewDate)}},{key:"_getDateByIndex",value:function(e){if(1===this.option("intervalCount"))return this._firstViewDate;var t=new Date(this._firstViewDate);return t.setDate(this._firstViewDate.getDate()+e),t}},{key:"_renderDateHeader",value:function(){return 1===this.option("intervalCount")?null:l(_(t.prototype),"_renderDateHeader",this).call(this)}},{key:"renderRDateTable",value:function(){this.renderRComponent(this._$dateTable,y.default,"renovatedDateTable",{viewData:this.viewDataProvider.viewData,dataCellTemplate:this.option("dataCellTemplate")})}}]),t}(v.default);(0,m.default)("dxSchedulerWorkSpaceDay",x);var w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(198)),y=n(846),b=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getProps",value:function(){var e=l(_(t.prototype),"getProps",this).call(this);return e.dataCellTemplate=this._createTemplateComponent(e,e.dataCellTemplate),e}},{key:"_propsInfo",get:function(){return{twoWay:[],allowNull:[],elements:[]}}},{key:"_viewComponent",get:function(){return y.DayDateTableLayout}}]),t}(v.default);t.default=b,(0,m.default)("dxDayDateTableLayout",b),e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,_.useCallback)(function(){var t=(e.dataCellTemplate,e.groupOrientation,e.viewData,u(e,["dataCellTemplate","groupOrientation","viewData"]));return t},[e]);return m({props:s(s({},e),{},{dataCellTemplate:v(e.dataCellTemplate)}),restAttributes:t()})}t.DayDateTableLayout=h,t.viewFunction=void 0;var f=n(532),p=n(170),g=a(n(51)),_=n(53),m=function(e){return g.h(f.DateTableLayoutBase,d({viewData:e.props.viewData,viewType:"day",dataCellTemplate:e.props.dataCellTemplate},e.restAttributes))};t.viewFunction=m;var v=function(e){return e&&(e.defaultProps?function(t){return g.h(e,d({},t))}:e)};h.defaultProps=s({},p.LayoutProps)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(){return l=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},l.apply(this,arguments)}function u(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function c(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?u(Object(n),!0).forEach(function(t){d(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):u(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function d(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function h(e){var t=(0,b.useCallback)(function(){var t=e.viewType;return"month"===t?m.MonthDateTableCell:v.DateTableCellBase},[e.viewType]),n=(0,b.useCallback)(function(){var t=(e.dataCellTemplate,e.groupOrientation,e.viewData,e.viewType,r(e,["dataCellTemplate","groupOrientation","viewData","viewType"]));return t},[e]);return x({props:c(c({},e),{},{dataCellTemplate:C(e.dataCellTemplate)}),cell:t(),restAttributes:n()})}t.DateTableBody=h,t.DateTableBodyProps=t.viewFunction=void 0;var f=n(227),p=n(199),g=n(170),_=n(527),m=n(848),v=n(314),y=a(n(51)),b=n(53),x=function(e){var t=e.cell,n=e.props,i=n.dataCellTemplate,o=n.viewData;return y.h(y.Fragment,null,o.groupedData.map(function(e,n){var a=e.allDayPanel,r=e.dateTable;return y.h(y.Fragment,{key:(0,p.getKeyByGroup)(n)},(0,p.getIsGroupedAllDayPanel)(o,n)&&y.h(_.AllDayPanelTableBody,{viewData:a,dataCellTemplate:i,isVerticalGroupOrientation:!0}),r.map(function(e){return y.h(f.Row,{className:"dx-scheduler-date-table-row",key:e[0].key},e.map(function(e){var n=e.endDate,o=e.groupIndex,a=e.groups,r=e.index,s=e.isFirstGroupCell,l=e.isLastGroupCell,u=e.key,c=e.startDate;return y.h(t,{isFirstGroupCell:s,isLastGroupCell:l,startDate:c,endDate:n,groups:a,groupIndex:o,index:r,dataCellTemplate:i,key:u})}))}))}))};t.viewFunction=x;var w=c({},g.LayoutProps);t.DateTableBodyProps=w;var C=function(e){return e&&(e.defaultProps?function(t){return y.h(e,l({},t))}:e)};h.defaultProps=c({},w)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){if(null==e)return{};var n,i,o=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function s(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach(function(t){c(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,g.useCallback)(function(){var t=e.className,n=e.otherMonth,i=e.today,o=[];return n&&o.push("dx-scheduler-date-table-other-month"),i&&o.push("dx-scheduler-date-table-current-date"),t&&o.push(t),0!==o.length?o.join(" "):void 0},[e.className,e.otherMonth,e.today]),n=(0,g.useCallback)(function(){var t=(e.allDay,e.children,e.className,e.contentTemplate,e.contentTemplateProps,e.dataCellTemplate,e.endDate,e.groupIndex,e.groups,e.index,e.isFirstGroupCell,e.isLastGroupCell,e.otherMonth,e.startDate,e.text,e.today,r(e,["allDay","children","className","contentTemplate","contentTemplateProps","dataCellTemplate","endDate","groupIndex","groups","index","isFirstGroupCell","isLastGroupCell","otherMonth","startDate","text","today"]));return t},[e]);return _({props:u(u({},e),{},{dataCellTemplate:v(e.dataCellTemplate),contentTemplate:v(e.contentTemplate)}),classes:t(),restAttributes:n()})}t.MonthDateTableCell=h,t.MonthDateTableCellProps=t.viewFunction=void 0;var f=n(314),p=a(n(51)),g=n(53),_=function(e){return p.h(f.DateTableCellBase,d({},e.restAttributes,{className:e.classes,dataCellTemplate:e.props.dataCellTemplate,startDate:e.props.startDate,endDate:e.props.endDate,groups:e.props.groups,groupIndex:e.props.groupIndex,index:e.props.index}),p.h("div",null,e.props.startDate.getDate()))};t.viewFunction=_;var m=u(u({},f.DateTableCellBaseProps),{},{otherMonth:!1,today:!1});t.MonthDateTableCellProps=m;var v=function(e){return e&&(e.defaultProps?function(t){return p.h(e,d({},t))}:e)};h.defaultProps=u({},m)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(4),y=o(n(9)),b=o(n(317)),x=o(n(20)),w=n(24),C=o(n(32)),k="dx-scheduler-work-space-month",S="dx-scheduler-date-table-current-date",D="dx-scheduler-date-table-first-of-month",I="dx-scheduler-date-table-other-month",T="dx-scheduler-scrollable-fixed-content",O=7,E=864e5,A=x.default.dateToMilliseconds,P=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_toggleFixedScrollableClass",value:function(){this._dateTableScrollable.$content().toggleClass(T,!this._isWorkSpaceWithCount()&&!this._isVerticalGroupedWorkSpace())}},{key:"_getElementClass",value:function(){return k}},{key:"_getRowCount",value:function(){return this._isWorkSpaceWithCount()?4*this.option("intervalCount")+2:6}},{key:"_getCellCount",value:function(){return O}},{key:"_getDateByIndex",value:function(e){var t=new Date(this._firstViewDate);return t.setDate(this._firstViewDate.getDate()+e),t}},{key:"_getFormat",value:function(){return this._formatWeekday}},{key:"_calculateCellIndex",value:function(e,t){return this._isVerticalGroupedWorkSpace()?e%=this._getRowCount():t%=this._getCellCount(),e*this._getCellCount()+t}},{key:"_getInterval",value:function(){return E}},{key:"_getIntervalBetween",value:function(e){var t=this.getStartViewDate(),n=x.default.getTimezonesDifference(t,e);return e.getTime()-(t.getTime()-36e5*this.option("startDayHour"))-n}},{key:"_getDateByCellIndexes",value:function(e,n){var i=l(_(t.prototype),"_getDateByCellIndexes",this).call(this,e,n);return this._setStartDayHour(i),i}},{key:"getCellWidth",value:function(){var e=this;return this.cache.get("cellWidth",function(){var t=7,n=0;return e._getCells().slice(0,t).each(function(e,t){return n+=(0,w.getBoundingRect)(t).width}),n/t})}},{key:"_calculateHiddenInterval",value:function(){return 0}},{key:"_insertAllDayRowsIntoDateTable",value:function(){return!1}},{key:"_getCellCoordinatesByIndex",value:function(e){var t=Math.floor(e/this._getCellCount()),n=e-this._getCellCount()*t;return{rowIndex:t,cellIndex:n}}},{key:"_createWorkSpaceElements",value:function(){this._isVerticalGroupedWorkSpace()?this._createWorkSpaceScrollableElements():l(_(t.prototype),"_createWorkSpaceElements",this).call(this)}},{key:"_needCreateCrossScrolling",value:function(){return this.option("crossScrollingEnabled")||this._isVerticalGroupedWorkSpace()}},{key:"_renderTimePanel",value:function(){return(0,v.noop)()}},{key:"_renderAllDayPanel",value:function(){return(0,v.noop)()}},{key:"_getTableAllDay",value:function(){return(0,v.noop)()}},{key:"_toggleAllDayVisibility",value:function(){return(0,v.noop)()}},{key:"_changeAllDayVisibility",value:function(){return(0,v.noop)()}},{key:"_setFirstViewDate",value:function(){var e=x.default.getFirstMonthDate(this._getViewStartByOptions());this._firstViewDate=x.default.getFirstWeekDate(e,this.option("firstDayOfWeek")||C.default.firstDayOfWeekIndex()),this._setStartDayHour(this._firstViewDate);var t=this._getViewStartByOptions();this._minVisibleDate=new Date(t.setDate(1)),this._maxVisibleDate=new Date(new Date(t.setMonth(t.getMonth()+this.option("intervalCount"))).setDate(0))}},{key:"_getViewStartByOptions",value:function(){if(this.option("startDate")){for(var e=this._getStartViewDate(),t=this.option("currentDate"),n=e.getTime()<=t.getTime()?1:-1,i=new Date(new Date(this._getStartViewDate().setMonth(this._getStartViewDate().getMonth()+n*this.option("intervalCount"))));!this._dateInRange(t,e,i,n);)e=new Date(i),n>0&&e.setDate(1),i=new Date(new Date(i.setMonth(i.getMonth()+n*this.option("intervalCount"))));return n>0?e:i}return new Date(this.option("currentDate").getTime())}},{key:"_getStartViewDate",value:function(){var e=x.default.getFirstMonthDate(this.option("startDate"));return e}},{key:"_renderTableBody",value:function(e){e.getCellText=this._getCellText.bind(this),l(_(t.prototype),"_renderTableBody",this).call(this,e)}},{key:"_getCellText",value:function(e,t){this.isGroupedByDate()?t=Math.floor(t/this._getGroupCount()):t%=this._getCellCount();var n=this._getDate(e,t);return this._isWorkSpaceWithCount()&&this._isFirstDayOfMonth(n)?this._formatMonthAndDay(n):C.default.format(n,"dd")}},{key:"_formatMonthAndDay",value:function(e){var t=C.default.getMonthNames("abbreviated")[e.getMonth()];return[t,C.default.format(e,"day")].join(" ")}},{key:"_getDate",value:function(e,t){var n=new Date(this._firstViewDate),i=this._getRowCount();return n.setDate(n.getDate()+e%i*O+t),n}},{key:"_updateIndex",value:function(e){return e}},{key:"_prepareCellData",value:function(e,n,i){var o=l(_(t.prototype),"_prepareCellData",this).call(this,e,n,i),a=(0,m.default)(i);return a.toggleClass(S,this._isCurrentDate(o.startDate)).toggleClass(D,this._isFirstDayOfMonth(o.startDate)).toggleClass(I,this._isOtherMonth(o.startDate)),o}},{key:"_isCurrentDate",value:function(e){return x.default.sameDate(e,this._getToday())}},{key:"_isFirstDayOfMonth",value:function(e){return this._isWorkSpaceWithCount()&&1===e.getDate()}},{key:"_isOtherMonth",value:function(e){return!x.default.dateInRange(e,this._minVisibleDate,this._maxVisibleDate,"date")}},{key:"isIndicationAvailable",value:function(){return!1}},{key:"getCellDuration",value:function(){return 36e5*this._calculateDayDuration()}},{key:"getIntervalDuration",value:function(){return A("day")}},{key:"getTimePanelWidth",value:function(){return 0}},{key:"getPositionShift",value:function(e){return{cellPosition:e*this.getCellWidth(),top:0,left:0}}},{key:"getCellCountToLastViewDate",value:function(e){var t=e.getTime(),n=this.getEndViewDate().getTime(),i=this.getCellDuration();return Math.ceil((n-t)/i)}},{key:"supportAllDayRow",value:function(){return!1}},{key:"keepOriginalHours",value:function(){return!0}},{key:"calculateEndDate",value:function(e){var t=new Date(e);return new Date(t.setHours(this.option("endDayHour")))}},{key:"getWorkSpaceLeftOffset",value:function(){return 0}},{key:"needApplyCollectorOffset",value:function(){return!0}},{key:"_getDateTableBorderOffset",value:function(){return this._getDateTableBorder()}},{key:"_getCellPositionByIndex",value:function(e,n){var i,o=l(_(t.prototype),"_getCellPositionByIndex",this).call(this,e,n),a=this._getCellCoordinatesByIndex(e).rowIndex;return i=this._isVerticalGroupedWorkSpace()?this.getCellHeight()*(a+n*this._getRowCount()):this.getCellHeight()*a,i&&(o.top=i),o}},{key:"_getHeaderDate",value:function(){return this._getViewStartByOptions()}},{key:"_supportCompactDropDownAppointments",value:function(){return!1}},{key:"scrollToTime",value:function(){return(0,v.noop)()}},{key:"_getRowCountWithAllDayRows",value:function(){return this._getRowCount()}}]),t}(b.default);(0,y.default)("dxSchedulerWorkSpaceMonth",P);var R=P;t.default=R,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(198)),y=n(851),b=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"getProps",value:function(){var e=l(_(t.prototype),"getProps",this).call(this);return e.dataCellTemplate=this._createTemplateComponent(e,e.dataCellTemplate),e}},{key:"_propsInfo",get:function(){return{twoWay:[],allowNull:[],elements:[]}}},{key:"_viewComponent",get:function(){return y.WeekTableLayout}}]),t}(v.default);t.default=b,(0,m.default)("dxWeekTableLayout",b),e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);t&&(i=i.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,i)}return n}function s(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?r(Object(n),!0).forEach(function(t){l(e,t,n[t])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):r(Object(n)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))})}return e}function l(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e,t){if(null==e)return{};var n,i,o=c(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}function c(e,t){if(null==e)return{};var n,i,o={},a=Object.keys(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||(o[n]=e[n]);return o}function d(){return d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},d.apply(this,arguments)}function h(e){var t=(0,_.useCallback)(function(){var t=(e.dataCellTemplate,e.groupOrientation,e.viewData,u(e,["dataCellTemplate","groupOrientation","viewData"]));return t},[e]);return m({props:s(s({},e),{},{dataCellTemplate:v(e.dataCellTemplate)}),restAttributes:t()})}t.WeekTableLayout=h,t.viewFunction=void 0;var f=n(532),p=n(170),g=a(n(51)),_=n(53),m=function(e){return g.h(f.DateTableLayoutBase,d({viewData:e.props.viewData,viewType:"week",dataCellTemplate:e.props.dataCellTemplate},e.restAttributes))};t.viewFunction=m;var v=function(e){return e&&(e.defaultProps?function(t){return g.h(e,d({},t))}:e)};h.defaultProps=s({},p.LayoutProps)},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(9)),v=o(n(20)),y=o(n(530)),b=o(n(533)),x=v.default.dateToMilliseconds,w="dx-scheduler-work-space-work-week",C=[1,2,3,4,5],k=0,S=function(e){
function t(){var e;a(this,t);for(var i=arguments.length,o=new Array(i),r=0;r<i;r++)o[r]=arguments[r];return e=n.call.apply(n,[this].concat(o)),e._isSkippedData=y.default.isDataOnWeekend,e._getWeekendsCount=y.default.getWeekendsCount,e}c(t,e);var n=h(t);return s(t,[{key:"_getElementClass",value:function(){return w}},{key:"_getCellCount",value:function(){return 5*this.option("intervalCount")}},{key:"_firstDayOfWeek",value:function(){return y.default.getFirstDayOfWeek(this.option("firstDayOfWeek"))}},{key:"_getDateByIndex",value:function(e){var t=new Date(this._firstViewDate);e%this._getCellCount()===0&&(k=0),t.setDate(this._firstViewDate.getDate()+e+k);for(var n=t.getDay();C.indexOf(n)===-1;)t.setDate(t.getDate()+1),n=t.getDay(),k++;return t}},{key:"_renderView",value:function(){k=0,l(_(t.prototype),"_renderView",this).call(this)}},{key:"_setFirstViewDate",value:function(){this._firstViewDate=y.default.getFirstViewDate(this._getViewStartByOptions(),this._firstDayOfWeek()),this._setStartDayHour(this._firstViewDate)}},{key:"_getOffsetByCount",value:function(e){var t=this._getCellCount(),n=Math.floor(e/t);e-=t*n;var i=Math.floor(e/5);return x("day")*i*2}}]),t}(b.default);(0,m.default)("dxSchedulerWorkSpaceWorkWeek",S);var D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=n(0),l=i(n(17)),u=n(50),c=n(147),d={startDate:"startDate",endDate:"endDate",allDay:"allDay",text:"text",description:"description",startDateTimeZone:"startDateTimeZone",endDateTimeZone:"endDateTimeZone",recurrenceRule:"recurrenceRule",recurrenceException:"recurrenceException",disabled:"disabled"},h=function(){function e(t,n){o(this,e),this.rawAppointment=t,this.options=n}return r(e,[{key:"getField",value:function(e){return this.options.getField(this.rawAppointment,e)}},{key:"setField",value:function(e,t){return this.options.setField(this.rawAppointment,e,t)}},{key:"calculateStartDate",value:function(e){if(!this.startDate||isNaN(this.startDate.getTime()))throw l.default.Error("E1032",this.text);return this.calculateDate(this.startDate,this.startDateTimeZone,e)}},{key:"calculateEndDate",value:function(e){return this.calculateDate(this.endDate,this.endDateTimeZone,e)}},{key:"calculateDate",value:function(e,t,n){if(e)return this.timeZoneCalculator.createDate(e,{appointmentTimeZone:t,path:n})}},{key:"clone",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:void 0,n=new e((0,u.deepExtendArraySafe)({},this.rawAppointment),this.options);return null!==t&&void 0!==t&&t.pathTimeZone&&(n.startDate=n.calculateStartDate(t.pathTimeZone),n.endDate=n.calculateEndDate(t.pathTimeZone)),n}},{key:"source",value:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];if(e){var t=this.clone();return t.startDate=this.startDate,t.endDate=this.endDate,t.source()}return(0,s.extend)({},this.rawAppointment)}},{key:"duration",get:function(){return this.endDate?this.endDate-this.startDate:0}},{key:"startDate",get:function(){var e=this.getField(d.startDate);return void 0===e?e:new Date(e)},set:function(e){this.setField(d.startDate,e)}},{key:"endDate",get:function(){var e=this.getField(d.endDate);return void 0===e?e:new Date(e)},set:function(e){this.setField(d.endDate,e)}},{key:"allDay",get:function(){return this.getField(d.allDay)},set:function(e){this.setField(d.allDay,e)}},{key:"text",get:function(){return this.getField(d.text)},set:function(e){this.setField(d.text,e)}},{key:"description",get:function(){return this.getField(d.description)},set:function(e){this.setField(d.description,e)}},{key:"startDateTimeZone",get:function(){return this.getField(d.startDateTimeZone)}},{key:"endDateTimeZone",get:function(){return this.getField(d.endDateTimeZone)}},{key:"recurrenceRule",get:function(){return this.getField(d.recurrenceRule)},set:function(e){this.setField(d.recurrenceRule,e)}},{key:"recurrenceException",get:function(){return this.getField(d.recurrenceException)},set:function(e){this.setField(d.recurrenceException,e)}},{key:"disabled",get:function(){return!!this.getField(d.disabled)}},{key:"timeZoneCalculator",get:function(){return this.options.getTimeZoneCalculator()}},{key:"isRecurrent",get:function(){return(0,c.getRecurrenceProcessor)().isValidRecurrenceRule(this.recurrenceRule)}}]),e}(),f=h;t.default=f,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.TimeZoneCalculator=t.PathTimeZoneConversion=void 0;var s=n(1),l=i(n(20)),u=l.default.dateToMilliseconds,c={fromSourceToAppointment:"toAppointment",fromAppointmentToSource:"fromAppointment",fromSourceToGrid:"toGrid",fromGridToSource:"fromGrid"};t.PathTimeZoneConversion=c;var d=function(){function e(t){o(this,e),this.options=t}return r(e,[{key:"createDate",value:function(e,t){var n=new Date(e);switch(t.path){case c.fromSourceToAppointment:return this._getConvertedDate(n,t.appointmentTimeZone,!0);case c.fromAppointmentToSource:return this._getConvertedDate(n,t.appointmentTimeZone,!0,!0);case c.fromSourceToGrid:return this._getConvertedDate(n,t.appointmentTimeZone,!1);case c.fromGridToSource:return this._getConvertedDate(n,t.appointmentTimeZone,!1,!0)}throw new Error("not specified pathTimeZoneConversion")}},{key:"getOffsets",value:function(e,t){var n=-this._getClientOffset(e)/u("hour"),i=this._getCommonOffset(e),o=this._getAppointmentOffset(e,t);return{client:n,common:(0,s.isDefined)(i)?i:n,appointment:"number"!=typeof o?n:o}}},{key:"_getClientOffset",value:function(e){return this.options.getClientOffset(e)}},{key:"_getCommonOffset",value:function(e){return this.options.getCommonOffset(e)}},{key:"_getAppointmentOffset",value:function(e,t){return this.options.getAppointmentOffset(e,t)}},{key:"_getConvertedDate",value:function(e,t,n,i){var o=new Date(e.getTime()),a=this.getOffsets(o,t);return n&&t?this._getConvertedDateByOffsets(e,a.client,a.appointment,i):this._getConvertedDateByOffsets(e,a.client,a.common,i)}},{key:"_getConvertedDateByOffsets",value:function(e,t,n,i){var o=i?-1:1,a=e.getTime()-o*t*u("hour");return new Date(a+o*n*u("hour"))}}]),e}();t.TimeZoneCalculator=d},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return l(e)||s(e)||b(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return x(e)}function u(e,t,n){return(u="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=c(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function c(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=m(e),null!==e););return e}function d(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&h(e,t)}function h(e,t){return(h=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function f(e){var t=_();return function(){var n,i=m(e);if(t){var o=m(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return p(this,n)}}function p(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?g(e):t}function g(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function _(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function m(e){return(m=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function v(e,t){return C(e)||w(e,t)||b(e,t)||y()}function y(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function b(e,t){if(e){if("string"==typeof e)return x(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?x(e,t):void 0}}function x(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function w(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function C(e){if(Array.isArray(e))return e}function k(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function S(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function D(e,t,n){return t&&S(e.prototype,t),n&&S(e,n),e}t.AppointmentSettingsGeneratorVirtualStrategy=t.AppointmentSettingsGeneratorBaseStrategy=t.AppointmentSettingsGenerator=void 0;var I=o(n(20)),T=n(1),O=n(0),E=n(147),A=o(n(86)),P=I.default.dateToMilliseconds,R=function(){function e(t){k(this,e),this.scheduler=t,this.settingsStrategy=this.scheduler.isVirtualScrolling()?new B(this.scheduler):new M(this.scheduler)}return D(e,[{key:"create",value:function(e){return this.settingsStrategy.create(e)}}]),e}();t.AppointmentSettingsGenerator=R;var M=function(){function e(t){k(this,e),this.scheduler=t}return D(e,[{key:"create",value:function(e){var t=this.scheduler,n=t.createAppointmentAdapter(e),i=t._resourcesManager.getResourcesFromItem(e),o=this._isAllDayAppointment(e),a=this._createAppointments(n,i);this._canProcessNotNativeTimezoneDates(a,n)&&(a=this._getProcessedNotNativeTimezoneDates(a,n));var r=this._createGridAppointmentList(a);r=this._cropAppointmentsByStartDayHour(r,e,o),r=this._getProcessedLongAppointmentsIfRequired(r,n);var s=this._createAppointmentInfos(r,i,o,n.isRecurrent);return s}},{key:"_isAllDayAppointment",value:function(e){return this.scheduler.appointmentTakesAllDay(e)&&this.workspace.supportAllDayRow()}},{key:"_createAppointments",value:function(e,t){var n=this._createRecurrenceAppointments(e,t);return e.isRecurrent||0!==n.length||n.push({startDate:e.startDate,endDate:e.endDate}),n=n.map(function(e){var t=e.startDate,n=e.endDate,i=null===n||void 0===n?void 0:n.getTime();return t.getTime()===i&&n.setTime(i+P("minute")),e})}},{key:"_canProcessNotNativeTimezoneDates",value:function(e,t){var n=this.scheduler.option("timeZone"),i=A.default.isEqualLocalTimeZone,o=A.default.hasDSTInLocalTimeZone,a=e.length>1,r=!(0,T.isEmptyObject)(n);return!!a&&(!(!r&&o())&&(r&&!i(n)))}},{key:"_getProcessedNotNativeDateIfCrossDST",value:function(e,t){if(t<0){var n=new Date(e),i=new Date(n);i.setHours(i.getHours()-1);var o=this.timeZoneCalculator.getOffsets(n).common,a=this.timeZoneCalculator.getOffsets(i).common;if(o!==a)return 0}return t}},{key:"_getProcessedNotNativeTimezoneDates",value:function(e,t){var n=this,i=t.startDate,o=e[e.length-1].endDate,a=this.timeZoneCalculator.getOffsets(i).common,r=this.timeZoneCalculator.getOffsets(o).common,s=a!==r;return s?e.map(function(e){var i=n.timeZoneCalculator.getOffsets(t.startDate).common-n.timeZoneCalculator.getOffsets(e.startDate).common,o=n.timeZoneCalculator.getOffsets(t.endDate).common-n.timeZoneCalculator.getOffsets(e.endDate).common;i=n._getProcessedNotNativeDateIfCrossDST(e.startDate,i),o=n._getProcessedNotNativeDateIfCrossDST(e.endDate,o);var a=new Date(e.startDate.getTime()+i*P("hour")),r=new Date(e.endDate.getTime()+o*P("hour")),s=n.timeZoneCalculator.createDate(a,{path:"toGrid"}),l=n.timeZoneCalculator.createDate(r,{path:"toGrid"});return t.duration>l.getTime()-s.getTime()&&(r=new Date(a.getTime()+t.duration)),{startDate:a,endDate:r}}):e}},{key:"_getProcessedLongAppointmentsIfRequired",value:function(e,t){var n=this,i=t.source(),o=this.scheduler.appointmentTakesAllDay(i),a=this.workspace.getDateRange(),r=this.scheduler.getLayoutManager().getRenderingStrategyInstance();if(r.needSeparateAppointment(o)){var s=[],l=[];e.forEach(function(e){var u=new Date(a[1]),c=r.normalizeEndDateByViewEnd(i,e.endDate);s=I.default.getDatesOfInterval(e.startDate,c,{milliseconds:n.scheduler.getWorkSpace().getIntervalDuration(o)});var d=s.filter(function(e){return new Date(e)<u}).map(function(n){return{startDate:n,endDate:new Date(new Date(n).setMilliseconds(t.duration)),source:e.source}});l=l.concat(d)}),e=l}return e}},{key:"_createGridAppointmentList",value:function(e){var t=this;return e.map(function(e){var n=t.timeZoneCalculator.createDate(e.startDate,{path:"toGrid"}),i=t.timeZoneCalculator.createDate(e.endDate,{path:"toGrid"});return{startDate:n,endDate:i,source:e}})}},{key:"_createExtremeRecurrenceDates",value:function(e){var t=this.workspace.getDateRange(),n=this.scheduler.appointmentTakesAllDay(e)?I.default.trimTime(t[0]):t[0],i=this.scheduler.option("timeZone"),o=i?this.timeZoneCalculator.createDate(n,{path:"fromGrid"}):n,a=i?this.timeZoneCalculator.createDate(t[1],{path:"fromGrid"}):t[1];return[o,a]}},{key:"_createRecurrenceOptions",value:function(e,t){var n=this,i=this._createExtremeRecurrenceDates(e.source(),t),o=v(i,2),a=o[0],r=o[1];return{rule:e.recurrenceRule,exception:e.recurrenceException,min:a,max:r,firstDayOfWeek:this.scheduler.getFirstDayOfWeek(),start:e.startDate,end:e.endDate,getPostProcessedException:function(t){var i=n.scheduler.option("timeZone");if((0,T.isEmptyObject)(i)||A.default.isEqualLocalTimeZone(i))return t;var o=n.timeZoneCalculator.getOffsets(e.startDate).common,a=n.timeZoneCalculator.getOffsets(t).common,r=o-a;return r=n._getProcessedNotNativeDateIfCrossDST(t,r),new Date(t.getTime()-r*I.default.dateToMilliseconds("hour"))}}}},{key:"_createRecurrenceAppointments",value:function(e,t){var n=e.duration,i=this._createRecurrenceOptions(e),o=(0,E.getRecurrenceProcessor)().generateDates(i);return o.map(function(e){var t=A.default.createUTCDateWithLocalOffset(e);t.setTime(t.getTime()+n);var i=A.default.createDateFromUTCWithLocalOffset(t);return{startDate:new Date(e),endDate:i}})}},{key:"_getGroupIndices",value:function(e){var t=this.scheduler._workSpace;return t._getGroupIndexes(e)}},{key:"_cropAppointmentsByStartDayHour",value:function(e,t,n){var i=this;return e.map(function(e){var n=new Date(e.startDate),o=i._getAppointmentFirstViewDate(e,t),a=i._getViewStartDayHour(o);return e.startDate=i._getAppointmentResultDate({appointment:e,rawAppointment:t,startDate:n,startDayHour:a,firstViewDate:o}),e})}},{key:"_getAppointmentFirstViewDate",value:function(){return this.scheduler.getStartViewDate()}},{key:"_getViewStartDayHour",value:function(){return this.scheduler._getCurrentViewOption("startDayHour")}},{key:"_getAppointmentResultDate",value:function(e){var t=e.appointment,n=e.rawAppointment,i=e.startDayHour,o=e.firstViewDate,a=e.startDate,r=new Date(t.startDate);return this.scheduler.appointmentTakesAllDay(n)?r=I.default.normalizeDate(a,o):(a<o&&(a=o),r=I.default.normalizeDate(t.startDate,a)),I.default.roundDateByStartDayHour(r,i)}},{key:"_createAppointmentInfos",value:function(e,t,n,i){for(var o=this,a=[],r=function(i){var r=o.scheduler._workSpace.getCoordinatesByDateInGroup(e[i].startDate,t,n);r.forEach(function(t){(0,O.extend)(t,{info:{appointment:e[i],sourceAppointment:e[i].source}})}),a=a.concat(r)},s=0;s<e.length;s++)r(s);return a}},{key:"timeZoneCalculator",get:function(){return this.scheduler.timeZoneCalculator}},{key:"workspace",get:function(){return this.scheduler.getWorkSpace()}},{key:"viewDataProvider",get:function(){return this.workspace.viewDataProvider}}]),e}();t.AppointmentSettingsGeneratorBaseStrategy=M;var B=function(e){function t(){return k(this,t),n.apply(this,arguments)}d(t,e);var n=f(t);return D(t,[{key:"_createAppointmentInfos",value:function(e,n,i,o){var a=this,r=i?e:e.filter(function(e){var t=e.source,n=e.startDate,i=e.endDate,o=t.groupIndex;return a.viewDataProvider.isGroupIntersectDateInterval(o,n,i)});return o&&this.isVerticalGrouping?this._createRecurrentAppointmentInfos(r,n,i):u(m(t.prototype),"_createAppointmentInfos",this).call(this,r,n,i,o)}},{key:"_createRecurrentAppointmentInfos",value:function(e,t,n){var i=this,o=[];return e.forEach(function(e){var t=e.source,a=t.groupIndex,r=i.workspace.getCoordinatesByDate(e.startDate,a,n);r&&((0,O.extend)(r,{info:{appointment:e,sourceAppointment:t}}),o.push(r))}),o}},{key:"_cropAppointmentsByStartDayHour",value:function(e,t,n){var i=this;return e.filter(function(e){var o=i._getAppointmentFirstViewDate(e,t);if(!o)return!1;var a=i._getViewStartDayHour(o),r=new Date(e.startDate);return e.startDate=i._getAppointmentResultDate({appointment:e,rawAppointment:t,startDate:r,startDayHour:a,firstViewDate:o}),!!n||e.endDate>e.startDate})}},{key:"_createRecurrenceAppointments",value:function(e,t){var n=this,i=e.duration,o=[],r=this.isVerticalGrouping&&this.workspace._getGroupCount()?this._getGroupIndices(t):[0];return r.forEach(function(t){var r=n._createRecurrenceOptions(e,t),s=(0,E.getRecurrenceProcessor)().generateDates(r),l=s.map(function(e){var n=new Date(e),o=A.default.createUTCDateWithLocalOffset(e);o.setTime(o.getTime()+i);var a=A.default.createDateFromUTCWithLocalOffset(o);return{startDate:n,endDate:a,groupIndex:t}});o.push.apply(o,a(l))}),o}},{key:"_getViewStartDayHour",value:function(e){return e.getHours()}},{key:"_getAppointmentFirstViewDate",value:function(e,t){var n=this.scheduler.getWorkSpace(),i=n.viewDataProvider,o=e.source.groupIndex,a=e.startDate,r=e.endDate,s=this._isAllDayAppointment(t);return i.findGroupCellStartDate(o,a,r,s)}},{key:"_updateGroupIndices",value:function(e,t){var n=this,i=this.isVerticalGrouping?this._getGroupIndices(t):[0],o=[];return i.forEach(function(t){var i=n.viewDataProvider.getGroupStartDate(t);i&&e.forEach(function(e){var n=(0,O.extend)({},e);n.groupIndex=t,o.push(n)})}),o}},{key:"_getGroupIndices",value:function(e){var n=u(m(t.prototype),"_getGroupIndices",this).call(this,e),i=this.scheduler.getWorkSpace(),o=i.viewDataProvider,a=o.getGroupIndices(),r=n.filter(function(e){return a.indexOf(e)!==-1});return r}},{key:"_createAppointments",value:function(e,n){var i=u(m(t.prototype),"_createAppointments",this).call(this,e,n);return e.isRecurrent?i:this._updateGroupIndices(i,n)}},{key:"viewDataProvider",get:function(){return this.workspace.viewDataProvider}},{key:"isVerticalGrouping",get:function(){return this.workspace._isVerticalGroupedWorkSpace()}}]),t}(M);t.AppointmentSettingsGeneratorVirtualStrategy=B},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(857)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=o(n(2)),k=o(n(5)),S=n(0),D=n(1),I=n(6),T=n(4),O=o(n(10)),E=o(n(9)),A=o(n(16)),P=o(n(419)),R=n(149),M=o(n(858)),B=n(859),V=o(n(860)),F=o(n(534)),L=o(n(861)),H=o(n(862)),z=o(n(864)),N=o(n(865)),j=o(n(867)),W=o(n(874)),G=o(n(875)),q="dx-filemanager",$=q+"-wrapper",K=q+"-container",U=q+"-dirs-panel",Y=q+"-inactive-area",Z=q+"-editing-container",X=q+"-items-panel",Q=q+"-item-custom-thumbnail",J="[*DXPDK*]$40F96F03-FBD8-43DF-91BE-F55F4B8BA871$",ee={folders:"navPane",items:"itemView"},te=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"_initTemplates",value:function(){}},{key:"_initMarkup",value:function(){p(w(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._firstItemViewLoad=!0,this._lockSelectionProcessing=!1,this._lockFocusedItemProcessing=!1,this._itemKeyToFocus=void 0,this._controller=new M.default({currentPath:this.option("currentPath"),currentPathKeys:this.option("currentPathKeys"),rootText:this.option("rootFolderName"),fileProvider:this.option("fileSystemProvider"),allowedFileExtensions:this.option("allowedFileExtensions"),uploadMaxFileSize:this.option("upload").maxFileSize,uploadChunkSize:this.option("upload").chunkSize,onInitialized:this._onControllerInitialized.bind(this),onDataLoading:this._onDataLoading.bind(this),onSelectedDirectoryChanged:this._onSelectedDirectoryChanged.bind(this)}),this._commandManager=new B.FileManagerCommandManager(this.option("permissions")),this.$element().addClass(q),this._createNotificationControl(),this._initCommandManager(),this._setItemsViewAreaActive(!1)}},{key:"_createNotificationControl",value:function(){var e=this,t=(0,C.default)("<div>").addClass("dx-filemanager-notification-container").appendTo(this.$element());this._notificationControl=this._createComponent(t,N.default,{progressPanelContainer:this.$element(),contentTemplate:function(t){return e._createWrapper(t)},onActionProgress:function(t){return e._onActionProgress(t)},positionTarget:".".concat(K)}),this._editing.option("notificationControl",this._notificationControl)}},{key:"_createWrapper",value:function(e){var t=this;this._$wrapper=(0,C.default)("<div>").addClass($).appendTo(e),this._createEditing();var n=(0,C.default)("<div>").appendTo(this._$wrapper);this._toolbar=this._createComponent(n,z.default,{commandManager:this._commandManager,generalItems:this.option("toolbar.items"),fileItems:this.option("toolbar.fileSelectionItems"),itemViewMode:this.option("itemView").mode,onItemClick:function(e){return t._actions.onToolbarItemClick(e)}}),this._createAdaptivityControl()}},{key:"_createAdaptivityControl",value:function(){var e=this,t=(0,C.default)("<div>").addClass(K).appendTo(this._$wrapper);this._adaptivityControl=this._createComponent(t,G.default,{drawerTemplate:function(t){return e._createFilesTreeView(t)},contentTemplate:function(t){return e._createItemsPanel(t)},onAdaptiveStateChanged:function(t){return e._onAdaptiveStateChanged(t)}})}},{key:"_createEditing",value:function(){var e=this,t=(0,C.default)("<div>").addClass(Z).appendTo(this.$element());this._editing=this._createComponent(t,j.default,{controller:this._controller,model:{getMultipleSelectedItems:this._getMultipleSelectedItems.bind(this)},getItemThumbnail:this._getItemThumbnailInfo.bind(this),uploadDropZonePlaceholderContainer:this.$element(),onSuccess:function(t){var n=t.updatedOnlyFiles;return e._redrawComponent(n)},onCreating:function(){return e._setItemsViewAreaActive(!1)},onError:function(t){return e._onEditingError(t)}})}},{key:"_createItemsPanel",value:function(e){this._$itemsPanel=(0,C.default)("<div>").addClass(X).appendTo(e),this._createBreadcrumbs(this._$itemsPanel),this._createItemView(this._$itemsPanel),this._commandManager.isCommandAvailable("upload")&&this._editing.setUploaderDropZone(this._$itemsPanel)}},{key:"_createFilesTreeView",value:function(e){var t=this;this._filesTreeViewContextMenu=this._createContextMenu(!1,ee.folders);var n=(0,C.default)("<div>").addClass(U).appendTo(e);this._filesTreeView=this._createComponent(n,F.default,{storeExpandedState:!0,contextMenu:this._filesTreeViewContextMenu,getDirectories:this.getDirectories.bind(this),getCurrentDirectory:this._getCurrentDirectory.bind(this),onDirectoryClick:this._onFilesTreeViewDirectoryClick.bind(this),onClick:function(){return t._setItemsViewAreaActive(!1)}})}},{key:"_createItemView",value:function(e,t){var n=this;this._itemViewContextMenu=this._createContextMenu(!0,ee.items);var i=this.option("itemView"),o={selectionMode:this.option("selectionMode"),selectedItemKeys:this.option("selectedItemKeys"),focusedItemKey:this.option("focusedItemKey"),contextMenu:this._itemViewContextMenu,getItems:this._getItemViewItems.bind(this),onError:function(e){var t=e.error;return n._showError(t)},onSelectionChanged:this._onItemViewSelectionChanged.bind(this),onFocusedItemChanged:this._onItemViewFocusedItemChanged.bind(this),onSelectedItemOpened:this._onSelectedItemOpened.bind(this),onContextMenuShowing:function(){return n._onContextMenuShowing(ee.items)},getItemThumbnail:this._getItemThumbnailInfo.bind(this),customizeDetailColumns:this.option("customizeDetailColumns"),detailColumns:this.option("itemView.details.columns")},a=(0,C.default)("<div>").appendTo(e);t=t||i.mode;var r="thumbnails"===t?H.default:L.default;this._itemView=this._createComponent(a,r,o),k.default.on(a,"click",this._onItemViewClick.bind(this))}},{key:"_createBreadcrumbs",value:function(e){var t=this,n=(0,C.default)("<div>").appendTo(e);this._breadcrumbs=this._createComponent(n,W.default,{rootFolderDisplayName:this.option("rootFolderName"),onCurrentDirectoryChanging:function(e){var n=e.currentDirectory;return t._setCurrentDirectory(n)}}),this._breadcrumbs.setCurrentDirectory(this._getCurrentDirectory())}},{key:"_createContextMenu",value:function(e,t){var n=this,i=(0,C.default)("<div>").appendTo(this._$wrapper);return this._createComponent(i,V.default,{commandManager:this._commandManager,items:this.option("contextMenu.items"),onItemClick:function(e){return n._actions.onContextMenuItemClick(e)},onContextMenuShowing:function(){return n._onContextMenuShowing(t)},isolateCreationItemCommands:e,viewArea:t})}},{key:"_initCommandManager",value:function(){var e=this,t=(0,S.extend)(this._editing.getCommandActions(),{refresh:function(){return e._refreshAndShowProgress()},thumbnails:function(){return e.option("itemView.mode","thumbnails")},details:function(){return e.option("itemView.mode","details")},clearSelection:function(){return e._clearSelection()},showNavPane:function(){return e._adaptivityControl.toggleDrawer()}});this._commandManager.registerActions(t)}},{key:"_onFilesTreeViewDirectoryClick",value:function(e){var t=e.itemData;this._setCurrentDirectory(t),this._setItemsViewAreaActive(!1)}},{key:"_onItemViewSelectionChanged",value:function(e){var t=e.selectedItemInfos,n=e.selectedItems,i=e.selectedItemKeys,o=e.currentSelectedItemKeys,a=e.currentDeselectedItemKeys;this._lockSelectionProcessing=!0,this.option("selectedItemKeys",i),this._lockSelectionProcessing=!1,this._actions.onSelectionChanged({selectedItems:n,selectedItemKeys:i,currentSelectedItemKeys:o,currentDeselectedItemKeys:a}),this._updateToolbar(t),this._setItemsViewAreaActive(!0)}},{key:"_onItemViewFocusedItemChanged",value:function(e){this._lockFocusedItemProcessing=!0,this.option("focusedItemKey",e.itemKey),this._lockFocusedItemProcessing=!1,this._actions.onFocusedItemChanged({item:e.item,itemElement:e.itemElement})}},{key:"_onAdaptiveStateChanged",value:function(e){var t=e.enabled;this._commandManager.setCommandEnabled("showNavPane",t),this._updateToolbar()}},{key:"_onActionProgress",value:function(e){var t=e.message,n=e.status;this._toolbar.updateRefreshItem(t,n),this._updateToolbar()}},{key:"_onEditingError",value:function(e){var t=(0,R.extendAttributes)({},e,["errorCode","errorText","fileSystemItem"]);this._actions.onErrorOccurred(t),e.errorText=t.errorText}},{key:"_refreshAndShowProgress",value:function(){var e=this;return(0,I.when)(this._notificationControl.tryShowProgressPanel(),this._controller.refresh()).then(function(){return e._filesTreeView.refresh()})}},{key:"_updateToolbar",value:function(e){var t=e||this._getSelectedItemInfos();this._toolbar.update(t)}},{key:"_setItemsViewAreaActive",value:function(e){if(this._itemsViewAreaActive!==e){this._itemsViewAreaActive=e;var t=null,n=null;e?(t=this._itemView.$element(),n=this._filesTreeView.$element()):(t=this._filesTreeView.$element(),n=this._itemView.$element()),t.removeClass(Y),n.addClass(Y)}}},{key:"_switchView",value:function(e){this._disposeWidget(this._itemView.option("contextMenu")),this._disposeWidget(this._itemView),this._createItemView(this._$itemsPanel,e),this._toolbar.option({itemViewMode:e})}},{key:"_disposeWidget",value:function(e){e.dispose(),e.$element().remove()}},{key:"_clearSelection",value:function(){this._itemView.clearSelection()}},{key:"_getMultipleSelectedItems",value:function(){return this._itemsViewAreaActive?this._getSelectedItemInfos():[this._getCurrentDirectory()]}},{key:"_showError",value:function(e){this._showNotification(e,!1)}},{key:"_showNotification",value:function(e,t){(0,P.default)({message:e,width:450},t?"success":"error",5e3)}},{key:"_redrawComponent",value:function(e){!e&&this._filesTreeView.refresh(),this._itemView.refresh()}},{key:"_getItemViewItems",value:function(){var e=this,t=this.option("itemView").showFolders,n=this._controller.getCurrentItems(!t);
return this._updateToolbarWithSelectionOnFirstLoad(n),this.option("itemView.showParentFolder")&&(n=(0,I.when)(n).then(function(t){return e._getPreparedItemViewItems(t)})),n}},{key:"_updateToolbarWithSelectionOnFirstLoad",value:function(e){var t=this;if(this._firstItemViewLoad){this._firstItemViewLoad=!1;var n=this.option("selectedItemKeys");n.length>0&&(0,I.when)(e).done(function(e){var i=(0,R.findItemsByKeys)(e,n);i.length>0&&t._updateToolbar(i)})}}},{key:"_getPreparedItemViewItems",value:function(e){var t=this._getCurrentDirectory();if(t.fileItem.isRoot())return e;var n=t.fileItem.createClone();n.isParentFolder=!0,n.name="..",n.relativeName="..",n.key="".concat(J).concat(t.fileItem.key);var i=a(e);return i.unshift({fileItem:n,icon:"parentfolder"}),i}},{key:"_onItemViewClick",value:function(){this._setItemsViewAreaActive(!0)}},{key:"_onContextMenuShowing",value:function(e){this._setItemsViewAreaActive(e===ee.items)}},{key:"_getItemThumbnailInfo",value:function(e){var t=this.option("customizeThumbnail"),n=(0,D.isFunction)(t)?t(e.fileItem):e.fileItem.thumbnail;return n?{thumbnail:n,cssClass:Q}:{thumbnail:e.icon}}},{key:"_getDefaultOptions",value:function(){return(0,S.extend)(p(w(t.prototype),"_getDefaultOptions",this).call(this),{fileSystemProvider:null,currentPath:"",currentPathKeys:[],rootFolderName:O.default.format("dxFileManager-rootDirectoryName"),selectionMode:"multiple",selectedItemKeys:[],focusedItemKey:void 0,toolbar:{items:["showNavPane","create","upload","switchView",{name:"separator",location:"after"},"refresh"],fileSelectionItems:["download","separator","move","copy","rename","separator","delete","clearSelection",{name:"separator",location:"after"},"refresh"]},contextMenu:{items:["create","upload","rename","move","copy","delete","refresh","download"]},itemView:{details:{columns:["thumbnail","name","dateModified","size"]},mode:"details",showFolders:!0,showParentFolder:!0},customizeThumbnail:null,customizeDetailColumns:null,onContextMenuItemClick:null,onCurrentDirectoryChanged:null,onSelectedFileOpened:null,onSelectionChanged:null,onFocusedItemChanged:null,onToolbarItemClick:null,onErrorOccurred:null,allowedFileExtensions:[],upload:{maxFileSize:0,chunkSize:2e5},permissions:{create:!1,copy:!1,move:!1,"delete":!1,rename:!1,upload:!1,download:!1}})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"currentPath":this._controller.setCurrentPath(e.value);break;case"currentPathKeys":this._controller.setCurrentPathByKeys(e.value);break;case"selectedItemKeys":!this._lockSelectionProcessing&&this._itemView&&this._itemView.option("selectedItemKeys",e.value);break;case"focusedItemKey":!this._lockFocusedItemProcessing&&this._itemView&&this._itemView.option("focusedItemKey",e.value);break;case"fileSystemProvider":case"selectionMode":case"customizeThumbnail":case"customizeDetailColumns":case"rootFolderName":case"allowedFileExtensions":case"permissions":case"upload":this.repaint();break;case"itemView":"itemView.mode"===e.fullName?this._switchView(e.value):this.repaint();break;case"toolbar":var i={};"toolbar"===e.fullName&&(e.value.items&&(i.generalItems=e.value.items),e.value.fileSelectionItems&&(i.fileItems=e.value.fileSelectionItems)),0===e.fullName.indexOf("toolbar.items")&&(i.generalItems=this.option("toolbar.items")),0===e.fullName.indexOf("toolbar.fileSelectionItems")&&(i.fileItems=this.option("toolbar.fileSelectionItems")),this._toolbar.option(i);break;case"contextMenu":if("contextMenu"===e.fullName&&e.value.items||0===e.fullName.indexOf("contextMenu.items")){var o=this.option("contextMenu.items");this._filesTreeViewContextMenu.option("items",o),this._itemViewContextMenu.option("items",o)}break;case"onContextMenuItemClick":case"onCurrentDirectoryChanged":case"onSelectedFileOpened":case"onSelectionChanged":case"onFocusedItemChanged":case"onToolbarItemClick":case"onErrorOccurred":this._actions[n]=this._createActionByOption(n);break;default:p(w(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_initActions",value:function(){this._actions={onContextMenuItemClick:this._createActionByOption("onContextMenuItemClick"),onCurrentDirectoryChanged:this._createActionByOption("onCurrentDirectoryChanged"),onSelectedFileOpened:this._createActionByOption("onSelectedFileOpened"),onSelectionChanged:this._createActionByOption("onSelectionChanged"),onFocusedItemChanged:this._createActionByOption("onFocusedItemChanged"),onToolbarItemClick:this._createActionByOption("onToolbarItemClick"),onErrorOccurred:this._createActionByOption("onErrorOccurred")}}},{key:"executeCommand",value:function(e){return this._commandManager.executeCommand(e)}},{key:"_setCurrentDirectory",value:function(e){this._controller.setCurrentDirectory(e)}},{key:"_getCurrentDirectory",value:function(){return this._controller.getCurrentDirectory()}},{key:"_onControllerInitialized",value:function(e){var t=e.controller;this._controller=this._controller||t;var n=t.getCurrentDirectory();n.fileItem.isRoot()||this._syncToCurrentDirectory()}},{key:"_onDataLoading",value:function(e){var t=e.operation,n=null;"navigation"===t&&(n={focusedItemKey:this._itemKeyToFocus},this._itemKeyToFocus=void 0),this._itemView.refresh(n)}},{key:"_onSelectedDirectoryChanged",value:function(){var e=this._getCurrentDirectory();this._syncToCurrentDirectory(),this._actions.onCurrentDirectoryChanged({directory:e.fileItem})}},{key:"_syncToCurrentDirectory",value:function(){var e=this._getCurrentDirectory(),t=this._controller.getCurrentPath(),n=e.fileItem.pathKeys;this._filesTreeView&&this._filesTreeView.updateCurrentDirectory(),this._breadcrumbs&&this._breadcrumbs.setCurrentDirectory(e);var i={currentPath:t};(0,T.equalByValue)(this.option("currentPathKeys"),n)||(i.currentPathKeys=n),this.option(i)}},{key:"getDirectories",value:function(e,t){return this._controller.getDirectories(e,t)}},{key:"_getSelectedItemInfos",value:function(){return this._itemView.getSelectedItems()}},{key:"refresh",value:function(){return this.executeCommand("refresh")}},{key:"getCurrentDirectory",value:function(){var e=this._getCurrentDirectory();return e&&e.fileItem||null}},{key:"getSelectedItems",value:function(){return this._getSelectedItemInfos().map(function(e){return e.fileItem})}},{key:"_onSelectedItemOpened",value:function(e){var t=e.fileItemInfo,n=t.fileItem;if(!n.isDirectory)return void this._actions.onSelectedFileOpened({file:n});n.isParentFolder&&(this._itemKeyToFocus=this._getCurrentDirectory().fileItem.key);var i=n.isParentFolder?this._getCurrentDirectory().parentDirectory:t;this._setCurrentDirectory(i),i&&this._filesTreeView.toggleDirectoryExpandedState(i.parentDirectory,!0)}}]),t}(A.default);(0,E.default)("dxFileManager",te);var ne=te;t.default=ne,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l(e)||s(e)||r(e)||a()}function a(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function r(e,t){if(e){if("string"==typeof e)return u(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?u(e,t):void 0}}function s(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function l(e){if(Array.isArray(e))return u(e)}function u(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function c(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function d(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function h(e,t,n){return t&&d(e.prototype,t),n&&d(e,n),e}t.default=void 0;var f=i(n(211)),p=i(n(277)),g=i(n(415)),_=i(n(416)),m=i(n(417)),v=i(n(279)),y=n(278),b=n(149),x=n(6),w=n(12),C=n(0),k=n(4),S="Files",D=function(){function e(t){c(this,e),t=t||{},this._options=(0,C.extend)({},t),this._isInitialized=!1,this._dataLoading=!1,this._dataLoadingDeferred=null,this._rootDirectoryInfo=this._createRootDirectoryInfo(t.rootText),this._currentDirectoryInfo=this._rootDirectoryInfo,this._defaultIconMap=this._createDefaultIconMap(),this._securityController=new I({allowedFileExtensions:this._options.allowedFileExtensions,maxFileSize:this._options.uploadMaxFileSize}),this._setProvider(t.fileProvider),this._initialize()}return h(e,[{key:"_setProvider",value:function(e){this._fileProvider=this._createFileProvider(e),this._resetState()}},{key:"_createFileProvider",value:function(e){if(e||(e=[]),Array.isArray(e))return new g.default({data:e});if(e instanceof f.default)return e;switch(e.type){case"remote":return new _.default(e);case"custom":return new m.default(e)}return new g.default(e)}},{key:"setCurrentPath",value:function(e){var t=(0,y.getPathParts)(e),n=y.pathCombine.apply(void 0,o(t));if(this.getCurrentDirectory().fileItem.relativeName!==n)return this._setCurrentDirectoryByPathParts(t)}},{key:"setCurrentPathByKeys",value:function(e){if(!(0,k.equalByValue)(this.getCurrentDirectory().fileItem.pathKeys,e,0,!0))return this._setCurrentDirectoryByPathParts(e,!0)}},{key:"getCurrentPath",value:function(){for(var e="",t=this.getCurrentDirectory();t&&!t.fileItem.isRoot();){var n=(0,y.getEscapedFileName)(t.fileItem.name);e=(0,y.pathCombine)(n,e),t=t.parentDirectory}return e}},{key:"getCurrentDirectory",value:function(){return this._currentDirectoryInfo}},{key:"setCurrentDirectory",value:function(e){if(e&&(!this._currentDirectoryInfo||this._currentDirectoryInfo!==e)){var t=this._currentDirectoryInfo.fileItem.key!==e.fileItem.key;this._currentDirectoryInfo=e,t&&this._isInitialized&&(this._dataLoading||this._raiseDataLoading("navigation"),this._raiseSelectedDirectoryChanged(e))}}},{key:"getCurrentItems",value:function(e){var t=this;return this._dataLoadingDeferred?this._dataLoadingDeferred.then(function(){return t._getCurrentItemsInternal(e)}):this._getCurrentItemsInternal(e)}},{key:"_getCurrentItemsInternal",value:function(e){var t=this.getCurrentDirectory();return e?this.getFiles(t):this.getDirectoryContents(t)}},{key:"getDirectories",value:function(e,t){return this.getDirectoryContents(e,t).then(function(e){return e.filter(function(e){return e.fileItem.isDirectory})})}},{key:"getFiles",value:function(e){return this.getDirectoryContents(e).then(function(e){return e.filter(function(e){return!e.fileItem.isDirectory})})}},{key:"getDirectoryContents",value:function(e,t){var n=this;if(!e)return(new x.Deferred).resolve([this._rootDirectoryInfo]).promise();if(e.itemsLoaded)return(new x.Deferred).resolve(e.items).promise();var i=e.getInternalKey(),o=this._loadedItems[i];return o?o:(o=this._getFileItems(e,t).then(function(t){return t=t||[],e.items=t.map(function(t){return t.isDirectory&&n._createDirectoryInfo(t,e)||n._createFileInfo(t,e)}),e.itemsLoaded=!0,e.items}),this._loadedItems[i]=o,o.always(function(){delete n._loadedItems[i]}),o)}},{key:"_getFileItems",value:function(e,t){var n=this,i=null;try{i=this._fileProvider.getItems(e.fileItem)}catch(n){return this._handleItemLoadError(e,n,t)}return(0,x.when)(i).then(function(e){return n._securityController.getAllowedItems(e)},function(i){return n._handleItemLoadError(e,i,t)})}},{key:"createDirectory",value:function(e,t){var n=this,i=this._createDirInfoByName(t,e),o=this._createEditActionInfo("create",i,e);return this._processEditAction(o,function(){return n._fileProvider.createDirectory(e.fileItem,t)},function(){return n._resetDirectoryState(e,!0)})}},{key:"renameItem",value:function(e,t){var n=this,i=this._createEditActionInfo("rename",e,e.parentDirectory,{itemNewName:t});return this._processEditAction(i,function(){return e.fileItem.isDirectory||n._securityController.validateExtension(t),n._fileProvider.renameItem(e.fileItem,t)},function(){var t=n._getActualDirectoryInfo(e.parentDirectory);n._resetDirectoryState(t),n.setCurrentDirectory(t)})}},{key:"moveItems",value:function(e,t){var n=this,i=e.map(function(e){return e.fileItem}),o=this._createEditActionInfo("move",e,t);return this._processEditAction(o,function(){return n._fileProvider.moveItems(i,t.fileItem)},function(){t=n._getActualDirectoryInfo(t),e.forEach(function(e){return n._resetDirectoryState(e.parentDirectory,!0)}),n._resetDirectoryState(t),n.setCurrentDirectory(t),t.expanded=!0})}},{key:"copyItems",value:function(e,t){var n=this,i=e.map(function(e){return e.fileItem}),o=this._createEditActionInfo("copy",e,t);return this._processEditAction(o,function(){return n._fileProvider.copyItems(i,t.fileItem)},function(){t=n._getActualDirectoryInfo(t),n._resetDirectoryState(t),n.setCurrentDirectory(t),t.expanded=!0})}},{key:"deleteItems",value:function(e){var t=this,n=e.map(function(e){return e.fileItem}),i=e.length>0?e[0].parentDirectory:null,o=this._createEditActionInfo("delete",e,i);return this._processEditAction(o,function(){return t._fileProvider.deleteItems(n)},function(){e.forEach(function(e){var n=t._getActualDirectoryInfo(e.parentDirectory);t._resetDirectoryState(n),t.setCurrentDirectory(n)})})}},{key:"processUploadSession",value:function(e,t){var n=this,i=this._getItemInfosForUploaderFiles(e.files,t),o=this._createEditActionInfo("upload",i,t,{sessionInfo:e});return this._processEditAction(o,function(){return e.deferreds},function(){return n._resetDirectoryState(t,!0)})}},{key:"uploadFileChunk",value:function(e,t,n){return this._securityController.validateMaxFileSize(e.size),this._securityController.validateExtension(e.name),(0,x.when)(this._fileProvider.uploadFileChunk(e,t,n))}},{key:"abortFileUpload",value:function(e,t,n){return(0,x.when)(this._fileProvider.abortFileUpload(e,t,n))}},{key:"getFileUploadChunkSize",value:function(){var e=this._options.uploadChunkSize;return e&&e>0?e:this._fileProvider.getFileUploadChunkSize()}},{key:"downloadItems",value:function(e){var t=e.map(function(e){return e.fileItem});this._fileProvider.downloadItems(t)}},{key:"getItemContent",value:function(e){var t=e.map(function(e){return e.fileItem});return(0,x.when)(this._fileProvider.getItemsContent(t))}},{key:"_handleItemLoadError",value:function(e,t,n){e=this._getActualDirectoryInfo(e);var i=this._createEditActionInfo("getItems",e,e);return this._raiseEditActionStarting(i),this._raiseEditActionResultAcquired(i),this._raiseEditActionError(i,{errorId:t.errorId,fileItem:e.fileItem,index:0}),this._resetDirectoryState(e),e.expanded=!1,n||this.setCurrentDirectory(e.parentDirectory),(new x.Deferred).reject().promise()}},{key:"_processEditAction",value:function(e,t,n){var i=this,o=null;this._raiseEditActionStarting(e);try{o=t()}catch(t){return this._raiseEditActionError(e,t),(new x.Deferred).reject().promise()}return Array.isArray(o)?o.length>1&&(e.singleRequest=!1):o=[o],this._raiseEditActionResultAcquired(e),(0,b.whenSome)(o,function(t){return i._raiseCompleteEditActionItem(e,t)},function(t){return i._raiseEditActionItemError(e,t)}).then(function(){n(),i._raiseCompleteEditAction(e)})}},{key:"_createEditActionInfo",value:function(e,t,n,i){t=Array.isArray(t)?t:[t],i=i||{};var o=t.map(function(e){return e.fileItem});return{name:e,itemInfos:t,items:o,directory:n,customData:i,singleRequest:!0}}},{key:"_getItemInfosForUploaderFiles",value:function(e,t){for(var n=this._getPathInfo(t),i=[],o=0;o<e.length;o++){var a=e[o],r=new p.default(n,a.name,(!1)),s=this._createFileInfo(r,t);i.push(s)}return i}},{key:"refresh",value:function(){var e=this;return this._lockRefresh?this._refreshDeferred:(this._lockRefresh=!0,this._executeDataLoad(function(){return e._refreshDeferred=e._refreshInternal()},"refresh"))}},{key:"_refreshInternal",value:function(){var e=this,t={items:this._rootDirectoryInfo.items},n=this._getDirectoryPathKeyParts(this.getCurrentDirectory());return this._resetDirectoryState(this._rootDirectoryInfo),this._loadItemsRecursive(this._rootDirectoryInfo,t).then(function(){var t=e._findDirectoryByPathKeyParts(n);e.setCurrentDirectory(t),delete e._lockRefresh})}},{key:"_loadItemsRecursive",value:function(e,t){var n=this;return this.getDirectories(e).then(function(e){for(var i=[],o=function(o){var a=(0,w.find)(t.items,function(t){return e[o].fileItem.key===t.fileItem.key});return a?(e[o].expanded=a.expanded,void(e[o].expanded&&i.push(n._loadItemsRecursive(e[o],a)))):"continue"},a=0;a<e.length;a++){o(a)}return(0,b.whenSome)(i)},function(){return null})}},{key:"_initialize",value:function(){var e=this,t=this._options.currentPathKeys&&this._options.currentPathKeys.length?this.setCurrentPathByKeys(this._options.currentPathKeys):this.setCurrentPath(this._options.currentPath),n=function(){e._isInitialized=!0,e._raiseInitialized()};t?(0,x.when)(t).always(n):n()}},{key:"_setCurrentDirectoryByPathParts",value:function(e,t){var n=this;return this._executeDataLoad(function(){return n._setCurrentDirectoryByPathPartsInternal(e,t)},"navigation")}},{key:"_setCurrentDirectoryByPathPartsInternal",value:function(e,t){var n=this;return this._getDirectoryByPathParts(this._rootDirectoryInfo,e,t).then(function(e){for(var t=e.parentDirectory;t;t=t.parentDirectory)t.expanded=!0;n.setCurrentDirectory(e)})}},{key:"_executeDataLoad",value:function(e,t){var n=this;return this._dataLoading=!0,this._dataLoadingDeferred=new x.Deferred,this._isInitialized&&this._raiseDataLoading(t),e().always(function(){n._dataLoadingDeferred.resolve(),n._dataLoadingDeferred=null,n._dataLoading=!1})}},{key:"_getDirectoryByPathParts",value:function(e,t,n){var i=this;if(t.length<1)return(new x.Deferred).resolve(e).promise();var a=n?"key":"name";return this.getDirectories(e).then(function(e){var r=(0,w.find)(e,function(e){return e.fileItem[a]===t[0]});if(!r)return(new x.Deferred).reject().promise();var s=o(t).splice(1);return i._getDirectoryByPathParts(r,s,n)})}},{key:"_getDirectoryPathKeyParts",value:function(e){for(var t=[];e&&e.parentDirectory;)t.unshift(e.fileItem.key),e=e.parentDirectory;return t}},{key:"_findDirectoryByPathKeyParts",value:function(e){var t=this._rootDirectoryInfo;if(0===e.length)return t;for(var n=0,i=t;i&&n<e.length;)i=(0,w.find)(t.items,function(t){return t.fileItem.key===e[n]}),i&&(t=i),n++;return t}},{key:"_getActualDirectoryInfo",value:function(e){var t=this._getDirectoryPathKeyParts(e);return this._findDirectoryByPathKeyParts(t)}},{key:"_createDirInfoByName",value:function(e,t){var n=this._getPathInfo(t),i=new p.default(n,e,(!0));return this._createDirectoryInfo(i,t)}},{key:"_createDirectoryInfo",value:function(e,t){return(0,C.extend)(this._createFileInfo(e,t),{icon:"folder",expanded:e.isRoot(),items:[]})}},{key:"_createFileInfo",value:function(e,t){return{fileItem:e,parentDirectory:t,icon:this._getFileItemDefaultIcon(e),getInternalKey:function(){return"FIK_".concat(this.fileItem.key)},getDisplayName:function(){return this.displayName||this.fileItem.name}}}},{key:"_resetDirectoryState",value:function(e,t){t&&(e=this._getActualDirectoryInfo(e)),e.itemsLoaded=!1,e.items=[]}},{key:"_getFileItemDefaultIcon",value:function(e){if(e.isDirectory)return"folder";var t=e.getFileExtension(),n=this._defaultIconMap[t];return n||"doc"}},{key:"_createDefaultIconMap",value:function(){var e={".txt":"txtfile",".rtf":"rtffile",".doc":"docfile",".docx":"docxfile",".xls":"xlsfile",".xlsx":"xlsxfile",".ppt":"pptfile",".pptx":"pptxfile",".pdf":"pdffile"};return[".png",".gif",".jpg",".jpeg",".ico",".bmp"].forEach(function(t){e[t]="image"}),e}},{key:"_createRootDirectoryInfo",value:function(e){var t=new p.default(null,"",(!0)),n=this._createDirectoryInfo(t,null);return n.displayName=e||S,n}},{key:"_raiseInitialized",value:function(){var e={controller:this};this._options.onInitialized&&this._options.onInitialized(e)}},{key:"_raiseDataLoading",value:function(e){this._options.onDataLoading&&this._options.onDataLoading({operation:e})}},{key:"_raiseSelectedDirectoryChanged",value:function(e){var t={selectedDirectoryInfo:e};this._options.onSelectedDirectoryChanged&&this._options.onSelectedDirectoryChanged(t)}},{key:"_raiseEditActionStarting",value:function(e){this._options.onEditActionStarting&&this._options.onEditActionStarting(e)}},{key:"_raiseEditActionResultAcquired",value:function(e){this._options.onEditActionResultAcquired&&this._options.onEditActionResultAcquired(e)}},{key:"_raiseEditActionError",value:function(e,t){this._options.onEditActionError&&this._options.onEditActionError(e,t)}},{key:"_raiseEditActionItemError",value:function(e,t){this._options.onEditActionItemError&&this._options.onEditActionItemError(e,t)}},{key:"_raiseCompleteEditActionItem",value:function(e,t){this._options.onCompleteEditActionItem&&this._options.onCompleteEditActionItem(e,t)}},{key:"_raiseCompleteEditAction",value:function(e){this._options.onCompleteEditAction&&this._options.onCompleteEditAction(e)}},{key:"_resetState",value:function(){this._selectedDirectory=null,this._rootDirectoryInfo.items=[],this._loadedItems={}}},{key:"_getPathInfo",value:function(e){for(var t=[],n=e;n&&!n.fileItem.isRoot();n=n.parentDirectory)t.unshift({key:n.fileItem.key,name:n.fileItem.name});return t}},{key:"on",value:function(e,t){var n="on".concat(e);this._options[n]=t}}]),e}();t.default=D;var I=function(){function e(t){var n=this;c(this,e);var i={allowedFileExtensions:[],maxFileSize:0};this._options=(0,C.extend)(i,t),this._extensionsMap={},this._allowedFileExtensions.forEach(function(e){n._extensionsMap[e.toUpperCase()]=!0})}return h(e,[{key:"getAllowedItems",value:function(e){var t=this;return 0===this._allowedFileExtensions.length?e:e.filter(function(e){return e.isDirectory||t._isValidExtension(e.name)})}},{key:"validateExtension",value:function(e){this._isValidExtension(e)||this._throwError(v.default.WrongFileExtension)}},{key:"validateMaxFileSize",value:function(e){this._maxFileSize&&e>this._maxFileSize&&this._throwError(v.default.MaxFileSizeExceeded)}},{key:"_isValidExtension",value:function(e){if(0===this._allowedFileExtensions.length)return!0;var t=(0,y.getFileExtension)(e).toUpperCase();return this._extensionsMap[t]}},{key:"_throwError",value:function(e){throw{errorId:e}}},{key:"_allowedFileExtensions",get:function(){return this._options.allowedFileExtensions}},{key:"_maxFileSize",get:function(){return this._options.maxFileSize}}]),e}();e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.FileManagerCommandManager=void 0;var s=n(0),l=n(1),u=i(n(10)),c=function(){function e(t){o(this,e),this._actions={},this._permissions=t||{},this._initCommands()}return r(e,[{key:"_initCommands",value:function(){var e=this;this._commands=[{name:"create",text:u.default.format("dxFileManager-commandCreate"),icon:"newfolder",enabled:this._permissions.create,noFileItemRequired:!0},{name:"rename",text:u.default.format("dxFileManager-commandRename"),icon:"rename",enabled:this._permissions.rename,isSingleFileItemCommand:!0},{name:"move",text:u.default.format("dxFileManager-commandMove"),icon:"movetofolder",enabled:this._permissions.move},{name:"copy",text:u.default.format("dxFileManager-commandCopy"),icon:"copy",enabled:this._permissions.copy},{name:"delete",text:u.default.format("dxFileManager-commandDelete"),icon:"trash",enabled:this._permissions.delete},{name:"download",text:u.default.format("dxFileManager-commandDownload"),icon:"download",enabled:this._permissions.download},{name:"upload",text:u.default.format("dxFileManager-commandUpload"),icon:"upload",enabled:this._permissions.upload,noFileItemRequired:!0},{name:"refresh",text:u.default.format("dxFileManager-commandRefresh"),icon:"dx-filemanager-i dx-filemanager-i-refresh",enabled:!0,noFileItemRequired:!0},{name:"thumbnails",text:u.default.format("dxFileManager-commandThumbnails"),icon:"mediumiconslayout",enabled:!0,noFileItemRequired:!0},{name:"details",text:u.default.format("dxFileManager-commandDetails"),icon:"detailslayout",enabled:!0,noFileItemRequired:!0},{name:"clearSelection",text:u.default.format("dxFileManager-commandClearSelection"),icon:"remove",enabled:!0},{name:"showNavPane",hint:u.default.format("dxFileManager-commandShowNavPane"),icon:"menu",enabled:!1,noFileItemRequired:!0}],this._commandMap={},this._commands.forEach(function(t){e._commandMap[t.name]=t})}},{key:"registerActions",value:function(e){this._actions=(0,s.extend)(this._actions,e)}},{key:"executeCommand",value:function(e,t){var n=(0,l.isString)(e)?e:e.name,i=this._actions[n];if(i)return i(t)}},{key:"setCommandEnabled",value:function(e,t){var n=this.getCommandByName(e);n&&(n.enabled=t)}},{key:"getCommandByName",value:function(e){return this._commandMap[e]}},{key:"isCommandAvailable",value:function(e,t){var n=this.getCommandByName(e);if(!n||!n.enabled)return!1;if(n.noFileItemRequired)return!0;var i=t&&t.length||0;return 0!==i&&!t.some(function(e){return e.fileItem.isRoot()||e.fileItem.isParentFolder})&&("download"===e?t.every(function(e){return!e.fileItem.isDirectory}):!n.isSingleFileItemCommand||1===i)}}]),e}();t.FileManagerCommandManager=c},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function s(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function l(e,t,n){return t&&s(e.prototype,t),n&&s(e,n),e}function u(e,t,n){return(u="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=c(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function c(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=m(e),null!==e););return e}function d(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&h(e,t)}function h(e,t){return(h=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function f(e){var t=_();return function(){var n,i=m(e);if(t){var o=m(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return p(this,n)}}function p(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?g(e):t}function g(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function _(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function m(e){return(m=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var v=o(n(2)),y=n(0),b=n(1),x=n(4),w=o(n(16)),C=o(n(457)),k="dx-filemanager-context-menu",S={create:{},upload:{},download:{},rename:{},move:{},copy:{},"delete":{},refresh:{beginGroup:!0}},D=function(e){function t(){return r(this,t),n.apply(this,arguments)}d(t,e);var n=f(t);return l(t,[{key:"_initMarkup",value:function(){var e=this;this._initActions(),this._isVisible=!1;var n=(0,v.default)("<div>").appendTo(this.$element());this._contextMenu=this._createComponent(n,C.default,{cssClass:k,showEvent:"",onItemClick:function(t){return e._onContextMenuItemClick(t.itemData.name,t)},onShowing:function(){return e._actions.onContextMenuShowing()},onHidden:function(){return e._onContextMenuHidden()}}),u(m(t.prototype),"_initMarkup",this).call(this)}},{key:"showAt",value:function(e,t,n,i){this._isVisible&&this._raiseContextMenuHidden(),this._isVisible=!0;var o=this.createContextMenuItems(e,null,i),a={of:t,at:"top left",my:"top left",offset:""};n?a.offset=n.offsetX+" "+n.offsetY:(a.my="left top",a.at="left bottom",a.boundaryOffset="1"),this._contextMenu.option({dataSource:o,target:t,position:a}),this._contextMenu.show()}},{key:"createContextMenuItems",value:function(e,t,n){var i=this;this._targetFileItems=e,this._targetFileItem=(0,b.isDefined)(n)?n:null===e||void 0===e?void 0:e[0];var o=[],a=t||this.option("items");return a.forEach(function(t){var n=(0,b.isString)(t)?t:t.name,a=i._configureItemByCommandName(n,t,e,i._targetFileItem);i._isContextMenuItemAvailable(a,e)&&o.push(a)}),o}},{key:"_isContextMenuItemAvailable",value:function(e,t){return this._isDefaultItem(e.name)&&e._autoHide?!(this._isIsolatedCreationItemCommand(e.name)&&t&&t.length)&&this._commandManager.isCommandAvailable(e.name,t):(0,x.ensureDefined)(e.visible,!0)}},{key:"_isIsolatedCreationItemCommand",value:function(e){return("create"===e||"upload"===e)&&this.option("isolateCreationItemCommands")}},{key:"_isDefaultItem",value:function(e){return!!S[e]}},{key:"_extendAttributes",value:function(e,t,n){n.forEach(function(n){(0,y.extend)(e,(0,b.isDefined)(t[n])?a({},n,t[n]):{})})}},{key:"_configureItemByCommandName",value:function(e,t,n,i){if(!this._isDefaultItem(e)){var o=(0,y.extend)(!0,{},t);return o.originalItemData=t,this._addItemClickHandler(e,o),Array.isArray(t.items)&&(o.items=this.createContextMenuItems(n,t.items,i)),o}var a=this._createMenuItemByCommandName(e),r=S[e];return(0,y.extend)(a,r),a.originalItemData=t,this._extendAttributes(a,t,["visible","beginGroup","text","icon"]),(0,b.isDefined)(a.visible)?this._extendAttributes(a,t,["visible","disabled"]):a._autoHide=!0,e&&!a.name&&(0,y.extend)(a,{name:e}),a}},{key:"_createMenuItemByCommandName",value:function(e){var t=this._commandManager.getCommandByName(e),n=t.text,i=t.icon,o={name:e,text:n,icon:i};return this._addItemClickHandler(e,o),o}},{key:"_addItemClickHandler",value:function(e,t){var n=this;t.onItemClick=function(t){return n._onContextMenuItemClick(e,t)}}},{key:"_onContextMenuItemClick",value:function(e,t){var n,i=(0,y.extend)(!0,{},t);if(i.itemData=t.itemData.originalItemData,i.fileSystemItem=null===(n=this._targetFileItem)||void 0===n?void 0:n.fileItem,i.viewArea=this.option("viewArea"),this._actions.onItemClick(i),this._isDefaultItem(e)){var o=this._isIsolatedCreationItemCommand(e)?null:this._targetFileItems;this._commandManager.executeCommand(e,o)}}},{key:"_initActions",value:function(){this._actions={onContextMenuHidden:this._createActionByOption("onContextMenuHidden"),onContextMenuShowing:this._createActionByOption("onContextMenuShowing"),onItemClick:this._createActionByOption("onItemClick")}}},{key:"_onContextMenuHidden",value:function(){this._isVisible=!1,this._raiseContextMenuHidden()}},{key:"_raiseContextMenuHidden",value:function(){this._actions.onContextMenuHidden()}},{key:"_getDefaultOptions",value:function(){return(0,y.extend)(u(m(t.prototype),"_getDefaultOptions",this).call(this),{commandManager:null,onContextMenuHidden:null,onItemClick:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"commandManager":this.repaint();break;case"items":if(this._isVisible){var i=this.createContextMenuItems(this._targetFileItems);this._contextMenu.option("dataSource",i)}break;case"onItemClick":case"onContextMenuShowing":case"onContextMenuHidden":this._actions[n]=this._createActionByOption(n);break;default:u(m(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_commandManager",get:function(){return this.option("commandManager")}}]),t}(w.default),I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){
return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(149),b=n(1),x=o(n(10)),w=o(n(460)),C=o(n(536)),k=o(n(535)),S="dx-filemanager-details",D="dx-filemanager-details-item-thumbnail",I="dx-filemanager-details-item-name",T="dx-filemanager-details-item-name-wrapper",O="dx-filemanager-details-item-is-directory",E="dx-filemanager-parent-directory-item",A="dx-data-row",P={thumbnail:{caption:"",calculateSortValue:"isDirectory",width:36,alignment:"center",cssClass:O},name:{caption:x.default.format("dxFileManager-listDetailsColumnCaptionName")},dateModified:{caption:x.default.format("dxFileManager-listDetailsColumnCaptionDateModified"),width:110,hidingPriority:1},size:{caption:x.default.format("dxFileManager-listDetailsColumnCaptionFileSize"),width:90,alignment:"right",hidingPriority:0},isParentFolder:{caption:"isParentFolder",visible:!1,sortIndex:0,sortOrder:"asc"}},R=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;this._itemCount=0,this._focusedItem=null,this._hasParentDirectoryItem=!1,this._parentDirectoryItemKey=null,this._selectAllCheckBox=null,this._selectAllCheckBoxUpdating=!1,this.$element().addClass(S),this._createFilesView(),this._contextMenu.option("onContextMenuHidden",function(){return e._onContextMenuHidden()}),l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"_createFilesView",value:function(){var e=(0,m.default)("<div>").appendTo(this.$element()),t=this._isMultipleSelectionMode()?"multiple":"none";this._filesView=this._createComponent(e,w.default,{dataSource:this._createDataSource(),hoverStateEnabled:!0,selection:{mode:t,showCheckBoxesMode:this._isDesktop()?"onClick":"none"},selectedRowKeys:this.option("selectedItemKeys"),focusedRowKey:this.option("focusedItemKey"),focusedRowEnabled:!0,allowColumnResizing:!0,scrolling:{mode:"virtual"},sorting:{mode:"single",showSortIndexes:!1},showColumnLines:!1,showRowLines:!1,columnHidingEnabled:!0,columns:this._createColumns(),onEditorPreparing:this._onEditorPreparing.bind(this),onRowPrepared:this._onRowPrepared.bind(this),onContextMenuPreparing:this._onContextMenuPreparing.bind(this),onSelectionChanged:this._onFilesViewSelectionChanged.bind(this),onFocusedRowChanged:this._onFilesViewFocusedRowChanged.bind(this),onOptionChanged:this._onFilesViewOptionChanged.bind(this)})}},{key:"_createColumns",value:function(){var e=this,t=this.option("detailColumns");t=t.slice(0),t=t.map(function(t){var n=t;return(0,b.isString)(t)&&(n={dataField:t}),e._getPreparedColumn(n)});var n=this.option("customizeDetailColumns");return(0,b.isFunction)(n)&&(t=n(t)),t.push(this._getPreparedColumn({dataField:"isParentFolder"})),t.forEach(function(t){return e._updateColumnDataField(t)}),t}},{key:"_getPreparedColumn",value:function(e){var t={},n="";if(this._isDefaultColumn(e.dataField)){var i=(0,v.extend)(!0,{},P[e.dataField]);n=i.cssClass||"","thumbnail"===e.dataField&&(i.cellTemplate=this._createThumbnailColumnCell.bind(this),i.calculateSortValue="fileItem.".concat(i.calculateSortValue)),"name"===e.dataField&&(i.cellTemplate=this._createNameColumnCell.bind(this)),"size"===e.dataField&&(i.calculateCellValue=this._calculateSizeColumnCellValue.bind(this)),(0,v.extend)(!0,t,i)}return(0,y.extendAttributes)(t,e,["alignment","caption","dataField","dataType","hidingPriority","sortIndex","sortOrder","visible","visibleIndex","width"]),e.cssClass&&(n=n?"".concat(n," ").concat(e.cssClass):e.cssClass),n&&(t.cssClass=n),t}},{key:"_updateColumnDataField",value:function(e){var t=this._isDefaultColumn(e.dataField)?"":"dataItem.";return e.dataField="fileItem."+t+e.dataField,e}},{key:"_isDefaultColumn",value:function(e){return!!P[e]}},{key:"_onFileItemActionButtonClick",value:function(e){var t=e.component,n=e.element,i=e.event;i.stopPropagation();var o=t.$element().closest(this._getItemSelector()),a=o.data("item");this._selectItem(a),this._showContextMenu(this._getFileItemsForContextMenu(a),n,a),this._activeFileActionsButton=t,this._activeFileActionsButton.setActive(!0)}},{key:"_onContextMenuHidden",value:function(){this._activeFileActionsButton&&this._activeFileActionsButton.setActive(!1)}},{key:"_getItemThumbnailCssClass",value:function(){return D}},{key:"_getItemSelector",value:function(){return".".concat(A)}},{key:"_onItemDblClick",value:function(e){var t=(0,m.default)(e.currentTarget),n=t.data("item");this._raiseSelectedItemOpened(n)}},{key:"_isAllItemsSelected",value:function(){var e=this._hasParentDirectoryItem?this._itemCount-1:this._itemCount,t=this._filesView.option("selectedRowKeys");return!!t.length&&(t.length>=e||void 0)}},{key:"_onEditorPreparing",value:function(e){var t=this,n=e.component,i=e.command,o=e.row,a=e.parentType,r=e.editorOptions;this._filesView||(this._filesView=n),"select"===i&&o?this._isParentDirectoryItem(o.data)&&(r.disabled=!0):"headerRow"===a&&(r.onInitialized=function(e){var n=e.component;t._selectAllCheckBox=n},r.value=this._isAllItemsSelected(),r.onValueChanged=function(e){return t._onSelectAllCheckBoxValueChanged(e)})}},{key:"_onSelectAllCheckBoxValueChanged",value:function(e){var t=e.event,n=e.previousValue,i=e.value;return t?void(this._isAllItemsSelected()!==i&&(i?this._filesView.selectAll():this._filesView.deselectAll(),t.preventDefault())):void(n&&!this._selectAllCheckBoxUpdating&&this._selectAllCheckBox&&this._selectAllCheckBox.option("value",n))}},{key:"_onRowPrepared",value:function(e){var t=e.rowType,n=e.rowElement,i=e.data;if("data"===t){var o=(0,m.default)(n);o.data("item",i),this._isParentDirectoryItem(i)&&o.addClass(E)}}},{key:"_onContextMenuPreparing",value:function(e){if(this._isDesktop()){var t=null,n=null;e.row&&"data"===e.row.rowType&&(n=e.row.data,this._selectItem(n),t=this._getFileItemsForContextMenu(n)),e.items=this._contextMenu.createContextMenuItems(t,null,n),this._raiseContextMenuShowing()}}},{key:"_onFilesViewSelectionChanged",value:function(e){var t=e.component,n=e.selectedRowsData,i=e.selectedRowKeys,o=e.currentSelectedRowKeys,a=e.currentDeselectedRowKeys;this._filesView=this._filesView||t,this._selectAllCheckBox&&(this._selectAllCheckBoxUpdating=!0,this._selectAllCheckBox.option("value",this._isAllItemsSelected()),this._selectAllCheckBoxUpdating=!1);var r=n.map(function(e){return e.fileItem});this._tryRaiseSelectionChanged({selectedItemInfos:n,selectedItems:r,selectedItemKeys:i,currentSelectedItemKeys:o,currentDeselectedItemKeys:a})}},{key:"_onFilesViewFocusedRowChanged",value:function(e){var t;if(!this._isMultipleSelectionMode()){var n;this._selectItemSingleSelection(null===(n=e.row)||void 0===n?void 0:n.data)}var i=(null===(t=e.row)||void 0===t?void 0:t.data.fileItem)||null;this._onFocusedItemChanged({item:i,itemKey:null===i||void 0===i?void 0:i.key,itemElement:e.rowElement})}},{key:"_onFilesViewOptionChanged",value:function(e){var t=e.fullName;t.indexOf("sortOrder")>-1&&this._filesView.columnOption("isParentFolder",{sortOrder:"asc",sortIndex:0})}},{key:"_createThumbnailColumnCell",value:function(e,t){this._getItemThumbnailContainer(t.data).appendTo(e)}},{key:"_createNameColumnCell",value:function(e,t){var n=this,i=(0,m.default)("<div>"),o=(0,m.default)("<span>").text(t.data.fileItem.name).addClass(I),a=(0,m.default)("<div>").append(o,i).addClass(T);(0,m.default)(e).append(a),this._createComponent(i,k.default,{onClick:function(e){return n._onFileItemActionButtonClick(e)}})}},{key:"_calculateSizeColumnCellValue",value:function(e){return e.fileItem.isDirectory?"":(0,y.getDisplayFileSize)(e.fileItem.size)}},{key:"_selectItem",value:function(e){var t=this._isMultipleSelectionMode()?this._selectItemMultipleSelection:this._selectItemSingleSelection;t.call(this,e)}},{key:"_deselectItem",value:function(e){this._filesView.deselectRows([e.fileItem.key])}},{key:"_selectItemSingleSelection",value:function(e){if(!this._focusedItem||!e||this._focusedItem.fileItem.key!==e.fileItem.key){var t=this._focusedItem;this._focusedItem=e;var n=[];t&&n.push(t.fileItem.key);var i=[],o=[];e&&(i.push(e.fileItem),o.push(e.fileItem.key)),this._raiseSelectionChanged({selectedItems:i,selectedItemKeys:o,currentSelectedItemKeys:[].concat(o),currentDeselectedItemKeys:n})}}},{key:"_selectItemMultipleSelection",value:function(e){var t=e.fileItem;if(!this._filesView.isRowSelected(t.key)){var n=this._filesView.getController("selection"),i=n.isSelectionWithCheckboxes();this._filesView.selectRows([t.key],i)}}},{key:"_setSelectedItemKeys",value:function(e){this._filesView.option("selectedRowKeys",e)}},{key:"_setFocusedItemKey",value:function(e){this._filesView.option("focusedRowKey",e)}},{key:"clearSelection",value:function(){this._isMultipleSelectionMode()?this._filesView.clearSelection():this._filesView.option("focusedRowIndex",-1)}},{key:"refresh",value:function(e){var t={dataSource:this._createDataSource()};e&&Object.prototype.hasOwnProperty.call(e,"focusedItemKey")&&((0,b.isDefined)(e.focusedItemKey)?t.focusedRowKey=e.focusedItemKey:t.focusedRowIndex=-1),this._filesView.option(t)}},{key:"getSelectedItems",value:function(){return this._isMultipleSelectionMode()?this._filesView.getSelectedRowsData():this._focusedItem&&!this._isParentDirectoryItem(this._focusedItem)?[this._focusedItem]:[]}}]),t}(C.default),M=R;t.default=M,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(6),b=o(n(5)),x=n(8),w=n(178),C=n(149),k=o(n(10)),S=o(n(863)),D=o(n(536)),I="dx-filemanager-thumbnails",T="dx-filemanager-thumbnails-item",O="dx-filemanager-thumbnails-item-thumbnail",E="dxFileManager_thumbnails",A=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this.$element().addClass(I);var e=(0,x.addNamespace)(w.name,E);b.default.on(this.$element(),e,this._onContextMenu.bind(this)),this._createItemList()}},{key:"_createItemList",value:function(){var e=this._isMultipleSelectionMode()?"multiple":"single",t=(0,m.default)("<div>").appendTo(this.$element());this._itemList=this._createComponent(t,S.default,{dataSource:this._createDataSource(),selectionMode:e,selectedItemKeys:this.option("selectedItemKeys"),focusedItemKey:this.option("focusedItemKey"),activeStateEnabled:!0,hoverStateEnabled:!0,loopItemFocus:!1,focusStateEnabled:!0,onItemEnterKeyPressed:this._tryOpen.bind(this),itemThumbnailTemplate:this._getItemThumbnailContainer.bind(this),getTooltipText:this._getTooltipText.bind(this),onSelectionChanged:this._onItemListSelectionChanged.bind(this),onFocusedItemChanged:this._onItemListFocusedItemChanged.bind(this)})}},{key:"_onContextMenu",value:function(e){if(e.preventDefault(),this._isDesktop()){var t=null,n=(0,m.default)(e.target).closest(this._getItemSelector()),i=null;n.length>0&&(i=this._itemList.getItemByItemElement(n),this._itemList.selectItem(i),t=this._getFileItemsForContextMenu(i)),this._showContextMenu(t,e.target,e,i)}}},{key:"_getItemThumbnailCssClass",value:function(){return O}},{key:"_getItemSelector",value:function(){return".".concat(T)}},{key:"_getTooltipText",value:function(e){var t=e.fileItem;if(t.tooltipText)return t.tooltipText;var n="".concat(t.name,"\r\n");return t.isDirectory||(n+="".concat(k.default.format("dxFileManager-listThumbnailsTooltipTextSize"),": ").concat((0,C.getDisplayFileSize)(t.size),"\r\n")),n+="".concat(k.default.format("dxFileManager-listThumbnailsTooltipTextDateModified"),": ").concat(t.dateModified)}},{key:"_onItemDblClick",value:function(e){var t=(0,m.default)(e.currentTarget),n=this._itemList.getItemByItemElement(t);this._tryOpen(n)}},{key:"_tryOpen",value:function(e){e&&this._raiseSelectedItemOpened(e)}},{key:"_getItemsInternal",value:function(){return l(_(t.prototype),"_getItemsInternal",this).call(this).then(function(e){var t=new y.Deferred;return setTimeout(function(){return t.resolve(e)}),t.promise()})}},{key:"_disableDragging",value:function(){return!1}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{focusStateEnabled:!0})}},{key:"_onItemListSelectionChanged",value:function(e){var t=e.addedItemKeys,n=e.removedItemKeys,i=this.getSelectedItems(),o=i.map(function(e){return e.fileItem}),a=o.map(function(e){return e.key});this._tryRaiseSelectionChanged({selectedItemInfos:i,selectedItems:o,selectedItemKeys:a,currentSelectedItemKeys:t,currentDeselectedItemKeys:n})}},{key:"_onItemListFocusedItemChanged",value:function(e){var t=e.item,n=e.itemElement;this._isMultipleSelectionMode()||this._selectItemSingleSelection(t);var i=(null===t||void 0===t?void 0:t.fileItem)||null;this._onFocusedItemChanged({item:i,itemKey:null===i||void 0===i?void 0:i.key,itemElement:n||void 0})}},{key:"_setSelectedItemKeys",value:function(e){this._itemList.option("selectedItemKeys",e)}},{key:"_setFocusedItemKey",value:function(e){this._itemList.option("focusedItemKey",e)}},{key:"refresh",value:function(e){var t={dataSource:this._createDataSource()};e&&Object.prototype.hasOwnProperty.call(e,"focusedItemKey")&&(t.focusedItemKey=e.focusedItemKey),this._itemList.option(t)}},{key:"_deselectItem",value:function(e){var t=this._itemList.getItemElementByItem(e);this._itemList.unselectItem(t)}},{key:"_selectItemSingleSelection",value:function(e){e?this._itemList.selectItem(e):this._itemList.clearSelection()}},{key:"clearSelection",value:function(){this._itemList.clearSelection()}},{key:"getSelectedItems",value:function(){return this._itemList.getSelectedItems()}}]),t}(D.default),P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(12),b=n(1),x=n(6),w=o(n(103)),C=n(8),k=o(n(5)),S=n(71),D=o(n(96)),I=o(n(58)),T=o(n(210)),O="dx-filemanager-thumbnails-view-port",E="dx-filemanager-thumbnails-container",A="dx-filemanager-thumbnails-item",P="dx-filemanager-thumbnails-item-name",R="dx-filemanager-thumbnails-item-spacer",M="dxFileManagerItemData",B="dxFileManagerThumbnailsListBox",V=(0,C.addNamespace)(w.default.name,B),F=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){this._initActions(),this._lockFocusedItemProcessing=!1,this.$element().addClass(O),this._renderScrollView(),this._renderItemsContainer(),this._createScrollViewControl(),l(_(t.prototype),"_initMarkup",this).call(this),this.onFocusedItemChanged=this._onFocusedItemChanged.bind(this),this._layoutUtils=new L(this._scrollView,this.$element(),this._$itemContainer,this.itemElements().first()),this._syncFocusedItemKey()}},{key:"_initActions",value:function(){this._actions={onItemEnterKeyPressed:this._createActionByOption("onItemEnterKeyPressed"),onFocusedItemChanged:this._createActionByOption("onFocusedItemChanged")}}},{key:"_initTemplates",value:function(){l(_(t.prototype),"_initTemplates",this).call(this),this._itemThumbnailTemplate=this.option("itemThumbnailTemplate"),this._getTooltipText=this.option("getTooltipText"),this._templateManager.addDefaultTemplates({item:new S.BindableTemplate(function(e,t,n){var i=this._getDefaultItemTemplate(n,e);e.append(i)}.bind(this),["fileItem"],this.option("integrationOptions.watchMethod"))})}},{key:"_createScrollViewControl",value:function(){this._scrollView||(this._scrollView=this._createComponent(this._$scrollView,D.default,{scrollByContent:!0,scrollByThumb:!0,useKeyboard:!1,showScrollbar:"onHover"}))}},{key:"_renderScrollView",value:function(){this._$scrollView||(this._$scrollView=(0,m.default)("<div>").appendTo(this.$element()))}},{key:"_renderItemsContainer",value:function(){this._$itemContainer||(this._$itemContainer=(0,m.default)("<div>").addClass(E).appendTo(this._$scrollView))}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._detachEventHandlers(),this._attachEventHandlers()}},{key:"_clean",value:function(){this._detachEventHandlers(),l(_(t.prototype),"_clean",this).call(this)}},{key:"_supportedKeys",value:function(){return(0,v.extend)(l(_(t.prototype),"_supportedKeys",this).call(this),{upArrow:function(e){this._beforeKeyProcessing(e),this._processArrowKeys(-1,!1,e)},downArrow:function(e){this._beforeKeyProcessing(e),this._processArrowKeys(1,!1,e)},home:function(e){this._beforeKeyProcessing(e),this._processHomeEndKeys(0,!0,e)},end:function(e){this._beforeKeyProcessing(e),this._processHomeEndKeys(this._getItemsLength()-1,!0,e)},pageUp:function(e){this._beforeKeyProcessing(e),this._processPageChange(!0,e)},pageDown:function(e){this._beforeKeyProcessing(e),this._processPageChange(!1,e)},enter:function(e){this._beforeKeyProcessing(e),this._actions.onItemEnterKeyPressed(this._getFocusedItem())},A:function(e){this._beforeKeyProcessing(e),(0,C.isCommandKeyPressed)(e)&&this.selectAll()}})}},{key:"_beforeKeyProcessing",value:function(e){e.preventDefault(),this._layoutUtils.reset()}},{key:"_processArrowKeys",value:function(e,t,n){var i=this._getFocusedItem();if(i){if(!t){var o=this._layoutUtils.getLayoutModel();if(!o)return;e*=o.itemPerRowCount}var a=this._getIndexByItem(i)+e;this._focusItemByIndex(a,!0,n)}}},{key:"_processHomeEndKeys",value:function(e,t,n){this._focusItemByIndex(e,t,n)}},{key:"_processPageChange",value:function(e,t){var n=this._getFocusedItem();if(n){var i=this._layoutUtils.getLayoutModel();if(i){var o=this._layoutUtils.createItemLayoutModel(this._getIndexByItem(n)),a=e?i.rowPerPageRate:-i.rowPerPageRate,r=o.itemRowIndex-a,s=e?Math.ceil:Math.floor,l=s(r),u=l*i.itemPerRowCount+o.itemColumnIndex;u<0?u=0:u>=this._getItemsLength()&&(u=this._getItemsLength()-1),this._focusItemByIndex(u,!0,t)}}}},{key:"_processLongTap",value:function(e){var t=this._closestItemElement((0,m.default)(e.target)),n=this._getIndexByItemElement(t);this._selection.changeItemSelection(n,{control:!0})}},{key:"_attachEventHandlers",value:function(){var e=this;"multiple"===this.option("selectionMode")&&k.default.on(this._itemContainer(),V,".".concat(this._itemContentClass()),function(t){e._processLongTap(t),t.stopPropagation()}),k.default.on(this._itemContainer(),"mousedown selectstart",function(e){e.shiftKey&&e.preventDefault()})}},{key:"_detachEventHandlers",value:function(){k.default.off(this._itemContainer(),V),k.default.off(this._itemContainer(),"mousedown selectstart")}},{key:"_itemContainer",value:function(){return this._$itemContainer}},{key:"_itemClass",value:function(){return A}},{key:"_itemDataKey",value:function(){return M}},{key:"_getDefaultItemTemplate",value:function(e,t){t.attr("title",this._getTooltipText(e));var n=this._itemThumbnailTemplate(e),i=(0,m.default)("<div>").addClass(R),o=(0,m.default)("<div>").addClass(P).text(e.fileItem.name);t.append(n,i,o)}},{key:"_itemSelectHandler",value:function(e){var t={};"multiple"===this.option("selectionMode")&&(this._isPreserveSelectionMode||(this._isPreserveSelectionMode=(0,C.isCommandKeyPressed)(e)||e.shiftKey),t={control:this._isPreserveSelectionMode,shift:e.shiftKey});var n=this._getIndexByItemElement(e.currentTarget);this._selection.changeItemSelection(n,t)}},{key:"_initSelectionModule",value:function(){var e=this;l(_(t.prototype),"_initSelectionModule",this).call(this);var n=(0,v.extend)(this._selection.options,{selectedKeys:this.option("selectedItemKeys"),onSelectionChanged:function(t){e.option("selectedItems",e._getItemsByKeys(t.selectedItemKeys,t.selectedItems)),e._updateSelectedItems(t)}});this._selection=new T.default(n)}},{key:"_updateSelectedItems",value:function(e){var t=this,n=e.addedItemKeys,i=e.removedItemKeys;if(this._rendered&&(n.length||i.length)){var o=this._selectionChangePromise;if(!this._rendering){var a,r=[],s=[];this._editStrategy.beginCache();for(var l=0;l<i.length;l++)a=this._getIndexByKey(i[l]),s.push(a),this._removeSelection(a);for(var u=0;u<n.length;u++)a=this._getIndexByKey(n[u]),r.push(a),this._addSelection(a);this._editStrategy.endCache(),this._updateSelection(r,s)}(0,x.when)(o).done(function(){return t._fireSelectionChangeEvent(e)})}}},{key:"_fireSelectionChangeEvent",value:function(e){this._createActionByOption("onSelectionChanged",{excludeValidators:["disabled","readOnly"]})(e)}},{key:"_updateSelection",value:function(e,t){var n=this.getSelectedItems().length;0===n&&(this._isPreserveSelectionMode=!1)}},{key:"_normalizeSelectedItems",value:function(){var e=this._getKeysByItems(this.option("selectedItems")),t=this._selection.getSelectedItemKeys();return this._compareKeys(t,e)||this._selection.setSelection(e),(new x.Deferred).resolve().promise()}},{key:"_focusOutHandler",value:function(){}},{key:"_getItems",value:function(){return this.option("items")||[]}},{key:"_getItemsLength",value:function(){return this._getItems().length}},{key:"_getIndexByItemElement",value:function(e){return this._editStrategy.getNormalizedIndex(e)}},{key:"_getItemByIndex",value:function(e){return this._getItems()[e]}},{key:"_getFocusedItem",value:function(){return this.getItemByItemElement(this.option("focusedElement"))}},{key:"_focusItem",value:function(e,t){this.option("focusedElement",this.getItemElementByItem(e)),t&&this._layoutUtils.scrollToItem(this._getIndexByItem(e))}},{key:"_focusItemByIndex",value:function(e,t,n){if(e>=0&&e<this._getItemsLength()){var i=this._getItemByIndex(e);this._focusItem(i,t,n)}}},{key:"_syncFocusedItemKey",value:function(){var e=this;this._syncFocusedItemKeyDeferred||(this._syncFocusedItemKeyDeferred=new x.Deferred);var t=this._syncFocusedItemKeyDeferred;if(this._dataSource&&this._dataSource.isLoading())return t.promise();var n=this.option("focusedItemKey");if((0,b.isDefined)(n)){var i=this.option("items"),o=(0,y.find)(i,function(t){return e.keyOf(t)===n});o?(this._focusItem(o,!0),t.resolve()):(this.option("focusedItemKey",void 0),t.reject())}else t.resolve();return this._syncFocusedItemKeyDeferred=null,t.promise()}},{key:"_onFocusedItemChanged",value:function(){var e=this._getFocusedItem(),t=this.keyOf(e),n=this.option("focusedItemKey");t!==n&&(this._lockFocusedItemProcessing=!0,this.option("focusedItemKey",t),this._lockFocusedItemProcessing=!1,this._raiseFocusedItemChanged(e))}},{key:"_raiseFocusedItemChanged",value:function(e){var t={item:e,itemElement:this.option("focusedElement")};this._actions.onFocusedItemChanged(t)}},{key:"_changeItemSelection",value:function(e,t){if(this.isItemSelected(e)!==t){var n=this.getItemElementByItem(e),i=this._getIndexByItemElement(n);this._selection.changeItemSelection(i,{control:this._isPreserveSelectionMode})}}},{key:"_chooseSelectOption",value:function(){return"selectedItemKeys"}},{key:"getSelectedItems",value:function(){return this._selection.getSelectedItems()}},{key:"getItemElementByItem",value:function(e){return this._editStrategy.getItemElement(e)}},{key:"getItemByItemElement",value:function(e){return this._getItemByIndex(this._getIndexByItemElement(e))}},{key:"selectAll",value:function(){"multiple"===this.option("selectionMode")&&(this._selection.selectAll(),this._isPreserveSelectionMode=!0)}},{key:"selectItem",value:function(e){this._changeItemSelection(e,!0)}},{key:"deselectItem",value:function(e){this._changeItemSelection(e,!1)}},{key:"clearSelection",value:function(){this._selection.deselectAll()}},{key:"_optionChanged",value:function(e){var n=this;switch(e.name){case"items":this._layoutUtils&&this._layoutUtils.updateItems(this.itemElements().first()),l(_(t.prototype),"_optionChanged",this).call(this,e);break;case"focusedItemKey":if(this._lockFocusedItemProcessing)break;(0,b.isDefined)(e.value)?this._syncFocusedItemKey().done(function(){var e=n._getFocusedItem();n._raiseFocusedItemChanged(e)}):(this.option("focusedElement",null),this._raiseFocusedItemChanged(null));break;case"onItemEnterKeyPressed":case"onFocusedItemChanged":this._actions[e.name]=this._createActionByOption(e.name);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(I.default),L=function(){function e(t,n,i,o){a(this,e),this._layoutModel=null,this._scrollView=t,this._$viewPort=n,this._$itemContainer=i,this._$item=o}return s(e,[{key:"updateItems",value:function(e){this._$item=e}},{key:"reset",value:function(){this._layoutModel=null}},{key:"getLayoutModel",value:function(){return this._layoutModel||(this._layoutModel=this._createLayoutModel()),this._layoutModel}},{key:"_createLayoutModel",value:function(){if(!this._$item)return null;var e=this._$item.outerWidth(!0);if(0===e)return null;var t=this._$item.outerHeight(!0),n=this._$itemContainer.innerWidth(),i=this._$viewPort.innerHeight(),o=this._scrollView.scrollTop(),a=o+i,r=Math.floor(n/e),s=i/t;return{itemWidth:e,itemHeight:t,viewPortWidth:n,viewPortHeight:i,viewPortScrollTop:o,viewPortScrollBottom:a,itemPerRowCount:r,rowPerPageRate:s}}},{key:"createItemLayoutModel",value:function(e){var t=this.getLayoutModel();if(!t)return null;var n=Math.floor(e/t.itemPerRowCount),i=e%t.itemPerRowCount,o=n*t.itemHeight,a=o+t.itemHeight;return{itemRowIndex:n,itemColumnIndex:i,itemTop:o,itemBottom:a}}},{key:"scrollToItem",value:function(e){var t=this.getLayoutModel();if(t){var n=Math.floor(e/t.itemPerRowCount),i=n*t.itemHeight,o=i+t.itemHeight,a=t.viewPortScrollTop;i<t.viewPortScrollTop?a=i:o>t.viewPortScrollBottom&&(a=o-t.viewPortHeight),this._scrollView.scrollTo(a)}}}]),e}(),H=F;t.default=H,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{
constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(1),b=n(4),x=o(n(10)),w=n(149),C=o(n(16)),k=o(n(166));n(433);var S="dx-filemanager-toolbar",D="dx-filemanager-general-toolbar",I="dx-filemanager-file-toolbar",T=S+"-separator-item",O=S+"-viewmode-item",E=S+"-has-large-icon",A="dx-filemanager-view-switcher-popup",P={showNavPane:{location:"before"},create:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},upload:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},refresh:{location:"after",showText:"inMenu",cssClass:E,compactMode:{showText:"inMenu",locateInMenu:"auto"}},switchView:{location:"after"},download:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},move:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},copy:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},rename:{location:"before",compactMode:{showText:"inMenu",locateInMenu:"auto"}},"delete":{location:"before",compactMode:{showText:"inMenu"}},clearSelection:{location:"after",locateInMenu:"never",compactMode:{showText:"inMenu"}},separator:{location:"before"}},R=["separator","switchView"],M={"default":"dx-filemanager-i dx-filemanager-i-refresh",progress:"dx-filemanager-i dx-filemanager-i-progress",success:"dx-filemanager-i dx-filemanager-i-done",error:"dx-filemanager-i dx-filemanager-i-danger"},B=500,V=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){this._commandManager=this.option("commandManager"),this._createItemClickedAction(),this._generalToolbarVisible=!0,this._$viewSwitcherPopup=(0,m.default)("<div>").addClass(A),this._generalToolbar=this._createToolbar(this.option("generalItems")),this._fileToolbar=this._createToolbar(this.option("fileItems"),!0),this._$viewSwitcherPopup.appendTo(this.$element()),this.$element().addClass(S+" "+D)}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this);var e=this._getVisibleToolbar();this._checkCompactMode(e)}},{key:"_dimensionChanged",value:function(e){if(!e||"height"!==e){var t=this._getVisibleToolbar();this._checkCompactMode(t)}}},{key:"_getVisibleToolbar",value:function(){return this._generalToolbarVisible?this._generalToolbar:this._fileToolbar}},{key:"_createToolbar",value:function(e,t){var n=this,i=this._getPreparedItems(e),o=(0,m.default)("<div>").appendTo(this.$element()),a=this._createComponent(o,k.default,{items:i,visible:!t,onItemClick:function(e){return n._raiseItemClicked(e)}});return a.compactMode=!1,a}},{key:"_getPreparedItems",value:function(e){var t=this;return e=e.map(function(e){var n=e;(0,y.isString)(e)&&(n={name:e});var i=n.name,o=t._configureItemByCommandName(i,n);return o.originalItemData=e,"separator"!==i&&t._setItemVisibleAvailable(o),o}),this._updateSeparatorsVisibility(e),e}},{key:"_updateSeparatorsVisibility",value:function(e,t){var n=this,i=!1,o=this._getMenuItems(t),a={before:!1,center:!1,after:!1},r={before:this._getItemsInGroup(e,o,"before"),center:this._getItemsInGroup(e,o,"center"),after:this._getItemsInGroup(e,o,"after")};return e.forEach(function(e,t){var s=e.location;if("separator"===e.name){var l=a[s]&&n._groupHasItemsAfter(r[s]);e.visible!==l&&(i=!0,e.visible=l),a[s]=!1}else n._isItemInMenu(o,e)||(a[s]=a[s]||e.visible),r[s].shift()}),t&&i&&t.repaint(),i}},{key:"_getMenuItems",value:function(e){var t=e?e._getMenuItems():[];return t.map(function(e){return e.originalItemData})}},{key:"_isItemInMenu",value:function(e,t){return!!e.length&&"never"!==(0,b.ensureDefined)(t.locateInMenu,"never")&&e.indexOf(t.originalItemData)!==-1}},{key:"_getItemsInGroup",value:function(e,t,n){var i=this;return e.filter(function(e){return e.location===n&&!i._isItemInMenu(t,e)})}},{key:"_groupHasItemsAfter",value:function(e){for(var t=0;t<e.length;t++)if("separator"!==e[t].name&&e[t].visible)return!0;return!1}},{key:"_configureItemByCommandName",value:function(e,t){var n={},i=this._commandManager.getCommandByName(e);switch(i&&(n=this._createCommandItem(i)),e){case"separator":n=this._createSeparatorItem();break;case"switchView":n=this._createViewModeItem()}if(this._isDefaultItem(e)){var o=P[e];(0,v.extend)(!0,n,o),(0,w.extendAttributes)(n,t,["visible","location","locateInMenu"]),(0,y.isDefined)(t.visible)?(0,w.extendAttributes)(n,t,["disabled"]):n._autoHide=!0,(0,w.extendAttributes)(n.options,t,["text","icon"]),"dxButton"===n.widget&&("inMenu"!==n.showText||(0,y.isDefined)(n.options.hint)||(n.options.hint=n.options.text),n.compactMode&&!(0,y.isDefined)(n.options.hint)&&this._configureHintForCompactMode(n))}else(0,v.extend)(!0,n,t),n.widget||(n.widget="dxButton"),"dxButton"===n.widget&&!n.compactMode&&!n.showText&&n.options.icon&&n.options.text&&(n.compactMode={showText:"inMenu"});return e&&!n.name&&(0,v.extend)(n,{name:e}),n.location=(0,b.ensureDefined)(n.location,"before"),"dxButton"===n.widget&&(0,v.extend)(!0,n,{options:{stylingMode:"text"}}),"dxSelectBox"===n.widget&&(0,v.extend)(!0,n,{options:{stylingMode:"filled"}}),n}},{key:"_isDefaultItem",value:function(e){return!!P[e]}},{key:"_createCommandItem",value:function(e){var t=this;return{widget:"dxButton",options:{text:e.text,hint:e.hint,commandText:e.text,icon:e.icon,stylingMode:"text",onClick:function(n){return t._executeCommand(e)}}}}},{key:"_createSeparatorItem",value:function(){return{template:function(e,t,n){(0,m.default)(n).addClass(T)}}}},{key:"_createViewModeItem",value:function(){var e=this,t=["details","thumbnails"].map(function(t){var n=e._commandManager.getCommandByName(t),i=n.text,o=n.icon;return{name:t,text:i,icon:o}}),n="thumbnails"===this.option("itemViewMode")?1:0;return{cssClass:O,widget:"dxDropDownButton",options:{items:t,keyExpr:"name",selectedItemKey:this.option("itemViewMode"),displayExpr:" ",hint:t[n].text,stylingMode:"text",showArrowIcon:!1,useSelectMode:!0,dropDownOptions:{container:this._$viewSwitcherPopup},onItemClick:function(t){return e._executeCommand(t.itemData.name)}}}}},{key:"_configureHintForCompactMode",value:function(e){e.options.hint="",e.compactMode.options=e.compactMode.options||{},e.compactMode.options.hint=e.options.text}},{key:"_checkCompactMode",value:function(e){e.compactMode&&this._toggleCompactMode(e,!1);var t=this._toolbarHasItemsOverflow(e);e.compactMode!==t?(e.compactMode||this._toggleCompactMode(e,t),e.compactMode=t):e.compactMode&&this._toggleCompactMode(e,!0)}},{key:"_toolbarHasItemsOverflow",value:function(e){var t=e.$element().width(),n=e._getItemsWidth();return t<n}},{key:"_toggleCompactMode",value:function(e,t){var n=this,i=!1,o=e.option("items");o.forEach(function(e){if(e.compactMode){var o=null;t?(e.saved=n._getCompactModeOptions(e,e._available),o=e.compactMode):o=e.saved;var a=n._getCompactModeOptions(o,e._available);(0,v.extend)(!0,e,a),i=!0}}),i=this._updateSeparatorsVisibility(o)||i,i&&e.repaint(),this._updateSeparatorsVisibility(o,e)}},{key:"_getCompactModeOptions",value:function(e,t){var n=e.showText,i=e.locateInMenu,o=e.options;return{visible:t,showText:(0,b.ensureDefined)(n,"always"),locateInMenu:(0,b.ensureDefined)(i,"never"),options:{hint:null===o||void 0===o?void 0:o.hint}}}},{key:"_ensureAvailableCommandsVisible",value:function(e,t){var n=this,i=!1,o=e.option("items");o.forEach(function(e){if("separator"!==e.name){var o=e._available;n._setItemVisibleAvailable(e,t),e._available!==o&&(i=!0)}}),i=this._updateSeparatorsVisibility(o)||i,i&&e.repaint(),this._updateSeparatorsVisibility(o,e)}},{key:"_setItemVisibleAvailable",value:function(e,t){var n,i=null===(n=e.originalItemData)||void 0===n?void 0:n.visible;e._available=this._isToolbarItemAvailable(e,t),e.visible=(0,y.isDefined)(i)?i:e._available}},{key:"_fileToolbarHasEffectiveItems",value:function(e){var t=this,n=this._fileToolbar.option("items");return n.some(function(n){return t._isFileToolbarItemAvailable(n,e)})}},{key:"_executeCommand",value:function(e){this._commandManager.executeCommand(e)}},{key:"_isToolbarItemAvailable",value:function(e,t){return this._isDefaultItem(e.name)&&e._autoHide?"refresh"===e.name?this._generalToolbarVisible||!!this._isRefreshVisibleInFileToolbar:R.indexOf(e.name)>-1||this._commandManager.isCommandAvailable(e.name,t):(0,b.ensureDefined)(e.visible,!0)}},{key:"_isFileToolbarItemAvailable",value:function(e,t){var n=e.name,i=e.visible;return!this._isDefaultItem(n)&&(0,b.ensureDefined)(i,!0)||"clearSelection"!==n&&"refresh"!==n&&this._commandManager.isCommandAvailable(n,t)}},{key:"_updateItemInToolbar",value:function(e,t,n){e.beginUpdate();for(var i=e.option("items"),o=0;o<i.length;o++){var a=i[o];if(a.name===t){e.option("items[".concat(o,"]"),n);break}}e.endUpdate()}},{key:"_raiseItemClicked",value:function(e){var t=(0,v.extend)(!0,{},e);t.itemData=e.itemData.originalItemData,this._itemClickedAction(t)}},{key:"_createItemClickedAction",value:function(){this._itemClickedAction=this._createActionByOption("onItemClick")}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{commandManager:null,generalItems:[],fileItems:[],itemViewMode:"details",onItemClick:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"commandManager":case"itemViewMode":case"generalItems":case"fileItems":this.repaint();break;case"onItemClick":this._itemClickedAction=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"updateRefreshItem",value:function(e,t){var n=null,i=x.default.format("dxFileManager-commandRefresh"),o="inMenu";this._isRefreshVisibleInFileToolbar=!1,"default"===t?n={options:{icon:M.default}}:(n={options:{icon:M[t]}},this._isRefreshVisibleInFileToolbar=!0,i=e,o="always");var a=(0,v.extend)({},n,{visible:this._isRefreshVisibleInFileToolbar});this._applyRefreshItemOptions(n,a),this._refreshItemTextTimeout=this._updateRefreshItemText("progress"===t,i,o)}},{key:"_updateRefreshItemText",value:function(e,t,n){var i=this,o={showText:n,options:{text:t}};return e?setTimeout(function(){i._applyRefreshItemOptions(o),i._refreshItemTextTimeout=void 0},B):(this._refreshItemTextTimeout&&clearTimeout(this._refreshItemTextTimeout),void this._applyRefreshItemOptions(o))}},{key:"_applyRefreshItemOptions",value:function(e,t){t||(t=(0,v.extend)({},e)),this._updateItemInToolbar(this._generalToolbar,"refresh",e),this._updateItemInToolbar(this._fileToolbar,"refresh",t)}},{key:"update",value:function(e){e=(0,b.ensureDefined)(e,[]);var t=0===e.length||!this._fileToolbarHasEffectiveItems(e);this._generalToolbarVisible!==t&&(this._generalToolbar.option("visible",t),this._fileToolbar.option("visible",!t),this._generalToolbarVisible=t,this.$element().toggleClass(D,t),this.$element().toggleClass(I,!t));var n=this._getVisibleToolbar();this._ensureAvailableCommandsVisible(n,e),this._checkCompactMode(n)}}]),t}(C.default),F=V;t.default=F,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(1),b=n(6),x=n(7),w=o(n(16)),C=o(n(48)),k=o(n(291)),S=o(n(866)),D=(0,x.getWindow)(),I=1e3,T="dx-filemanager-notification",O="".concat(T,"-drawer"),E="".concat(O,"-panel"),A="".concat(T,"-popup"),P="".concat(T,"-popup-error"),R="".concat(T,"-common"),M="".concat(T,"-separator"),B="".concat(T,"-details"),V="".concat(T,"-common-no-item"),F=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;l(_(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._actionProgressStatus="default",this._operationInProgressCount=0,this._failedOperationCount=0,this._isInAdaptiveState=this._isSmallScreen();var n=this.option("progressPanelContainer"),i=(0,m.default)("<div>").addClass(O).appendTo(n);(0,m.default)("<div>").addClass(E).appendTo(i);var o=(0,v.extend)({opened:!1,position:"right",template:function(t){return e._ensureProgressPanelCreated(t)}},this._getProgressDrawerAdaptiveOptions());this._progressDrawer=this._createComponent(i,k.default,o);var a=i.find(".".concat(E)).first(),r=this.option("contentTemplate");(0,y.isFunction)(r)&&r(a)}},{key:"tryShowProgressPanel",value:function(){var e=this,t=new b.Deferred;return"default"===this._actionProgressStatus||this._isProgressDrawerOpened()?t.resolve().promise():(setTimeout(function(){e._progressDrawer.show().done(t.resolve),e._getNotificationPopup().hide(),e._tryHideActionProgress()}),t.promise())}},{key:"addOperation",value:function(e,t,n){this._operationInProgressCount++;var i=this._progressPanel.addOperation(e,t,n);return this._updateActionProgress(e,"progress"),i}},{key:"addOperationDetails",value:function(e,t,n){this._progressPanel.addOperationDetails(e,t,n)}},{key:"updateOperationItemProgress",value:function(e,t,n,i){this._progressPanel.updateOperationItemProgress(e,t,n,i)}},{key:"completeOperationItem",value:function(e,t,n){this._progressPanel.completeOperationItem(e,t,n)}},{key:"completeOperation",value:function(e,t,n,i){if(this._operationInProgressCount--,n?this._failedOperationCount++:this._showPopup(t),this._progressPanel.completeOperation(e,t,n,i),!this._isProgressDrawerOpened()||!this._tryHideActionProgress()){var o=0===this._failedOperationCount?"success":"error";this._updateActionProgress("",o)}}},{key:"completeSingleOperationWithError",value:function(e,t){this._progressPanel.completeSingleOperationWithError(e,t.detailErrorText),this._notifyError(t)}},{key:"addOperationDetailsError",value:function(e,t){this._progressPanel.addOperationDetailsError(e,t.itemIndex,t.detailErrorText),this._notifyError(t)}},{key:"_hideProgressPanel",value:function(){var e=this;setTimeout(function(){return e._progressDrawer.hide()})}},{key:"_tryHideActionProgress",value:function(){return 0===this._operationInProgressCount&&0===this._failedOperationCount&&(this._updateActionProgress("","default"),!0)}},{key:"_updateActionProgress",value:function(e,t){this._actionProgressStatus=t,this._raiseActionProgress(e,t)}},{key:"_isSmallScreen",value:function(){return!!(0,x.hasWindow)()&&(0,m.default)(D).width()<=I}},{key:"_dimensionChanged",value:function(e){e&&"height"===e||this._checkAdaptiveState()}},{key:"_checkAdaptiveState",value:function(){var e=this._isInAdaptiveState;if(this._isInAdaptiveState=this._isSmallScreen(),this._progressDrawer&&e!==this._isInAdaptiveState){this._progressPanel&&this._progressPanel.$element().detach();var t=this._getProgressDrawerAdaptiveOptions();this._progressDrawer.option(t)}}},{key:"_getProgressDrawerAdaptiveOptions",value:function(){return this._isInAdaptiveState?{openedStateMode:"overlap",shading:!0,closeOnOutsideClick:!0}:{openedStateMode:"shrink",shading:!1,closeOnOutsideClick:!1}}},{key:"_ensureProgressPanelCreated",value:function(e){var t=this;if(this._progressPanel)this._progressPanel.$element().appendTo(e);else{var n=(0,m.default)("<div>").appendTo(e);this._progressPanel=this._createComponent(n,this._getProgressPanelComponent(),{onOperationClosed:function(e){var n=e.info;return t._onProgressPanelOperationClosed(n)},onOperationCanceled:function(e){var n=e.info;return t._raiseOperationCanceled(n)},onOperationItemCanceled:function(e){var n=e.item,i=e.itemIndex;return t._raiseOperationItemCanceled(n,i)},onPanelClosed:function(){return t._hideProgressPanel()}})}}},{key:"_getProgressPanelComponent",value:function(){return S.default}},{key:"_notifyError",value:function(e){this._showPopupError(e),this._updateActionProgress(e.commonErrorText,"error")}},{key:"_onProgressPanelOperationClosed",value:function(e){e.hasError&&(this._failedOperationCount--,this._tryHideActionProgress())}},{key:"_isProgressDrawerOpened",value:function(){return this._progressDrawer.option("opened")}},{key:"_showPopup",value:function(e,t){this._isProgressDrawerOpened()||(this._getNotificationPopup()._wrapper().toggleClass(P,!!t),this._getNotificationPopup().option("contentTemplate",e),this._getNotificationPopup().option("visible")||this._getNotificationPopup().show())}},{key:"_showPopupError",value:function(e){var t=(0,m.default)("<div>"),n=(0,m.default)("<div>").addClass(R).text(e.commonErrorText),i=(0,m.default)("<div>").addClass(M);(0,m.default)("<div>").appendTo(i);var o=(0,m.default)("<div>").addClass(B);e.item?this._progressPanel.createErrorDetailsProgressBox(o,e.item,e.detailErrorText):(n.addClass(V),this._progressPanel.renderError(o,i,e.detailErrorText)),t.append(n,i,o),this._showPopup(t,!0)}},{key:"_getNotificationPopup",value:function(){if(!this._notificationPopup){var e=(0,m.default)("<div>").addClass(A).appendTo(this.$element());this._notificationPopup=this._createComponent(e,C.default,{container:this.$element(),width:"auto",height:"auto",showTitle:!1,dragEnabled:!1,shading:!1,visible:!1,closeOnOutsideClick:!0,animation:{duration:0},position:{my:"right top",at:"right top",of:this.option("positionTarget"),offset:"-10 -5"}})}return this._notificationPopup}},{key:"_raiseActionProgress",value:function(e,t){this._actions.onActionProgress({message:e,status:t})}},{key:"_raiseOperationCanceled",value:function(e){this._actions.onOperationCanceled({info:e})}},{key:"_raiseOperationItemCanceled",value:function(e,t){this._actions.onOperationItemCanceled({item:e,itemIndex:t})}},{key:"_initActions",value:function(){this._actions={onActionProgress:this._createActionByOption("onActionProgress"),onOperationCanceled:this._createActionByOption("onOperationCanceled"),onOperationItemCanceled:this._createActionByOption("onOperationItemCanceled")}}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{progressPanelContainer:null,contentTemplate:null,onActionProgress:null,onOperationCanceled:null,onOperationItemCanceled:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"progressPanelContainer":case"contentTemplate":break;case"onActionProgress":case"onOperationCanceled":case"onOperationItemCanceled":this._actions[n]=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(w.default);t.default=F,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(4),b=n(59),x=o(n(10)),w=o(n(16)),C=o(n(295)),k=o(n(38)),S=o(n(96)),D="dx-filemanager-progress-panel",I="".concat(D,"-container"),T="".concat(D,"-title"),O="".concat(D,"-title-text"),E="".concat(D,"-close-button"),A="".concat(D,"-infos-container"),P="".concat(D,"-separator"),R="".concat(D,"-info"),M="".concat(D,"-common"),B="".concat(D,"-info-with-details"),V="".concat(D,"-details"),F="dx-filemanager-progress-box",L="".concat(F,"-error"),H="".concat(F,"-without-close-button"),z="".concat(F,"-image"),N="".concat(F,"-wrapper"),j="".concat(F,"-common"),W="".concat(F,"-progress-bar"),G="".concat(F,"-close-button"),q="dx-card",$=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){var e=this;l(_(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._operationCount=0,this.$element().addClass(D);var n=(0,m.default)("<div>").appendTo(this.$element()),i=(0,m.default)("<div>").addClass(I).appendTo(n);this._scrollView=this._createComponent(n,S.default,{scrollByContent:!0,scrollByThumb:!0,showScrollbar:"onScroll"});var o=(0,m.default)("<div>").addClass(T).appendTo(i);(0,m.default)("<div>").text(x.default.format("dxFileManager-notificationProgressPanelTitle")).addClass(O).appendTo(o);var a=(0,m.default)("<div>").addClass(E).appendTo(o);this._createComponent(a,k.default,{icon:"close",stylingMode:"text",onClick:function(){return e._raisePanelClosed()}}),this._$infosContainer=(0,m.default)("<div>").addClass(A).appendTo(i),this._renderEmptyListText()}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{onOperationClosed:null,onOperationCanceled:null,onOperationItemCanceled:null,onPanelClosed:null})}},{key:"_initActions",value:function(){this._actions={onOperationClosed:this._createActionByOption("onOperationClosed"),onOperationCanceled:this._createActionByOption("onOperationCanceled"),onOperationItemCanceled:this._createActionByOption("onOperationItemCanceled"),onPanelClosed:this._createActionByOption("onPanelClosed")}}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"test":break;case"onOperationClosed":case"onOperationCanceled":case"onOperationItemCanceled":this._actions[n]=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"addOperation",value:function(e,t,n){var i=this;this._operationCount?(0,m.default)("<div>").addClass(P).prependTo(this._$infosContainer):this._$infosContainer.empty(),this._operationCount++;var o={customCloseHandling:t,allowProgressAutoUpdate:(0,y.ensureDefined)(n,!0)},a=(0,m.default)("<div>").addClass(R).prependTo(this._$infosContainer);o.$info=a;var r=(0,m.default)("<div>").addClass(M).appendTo(a);return o.common=this._createProgressBox(r,{commonText:e,showCloseButton:!0,showCloseButtonAlways:t,onCloseButtonClick:function(){return i._closeOperation(o)}}),o}},{key:"addOperationDetails",value:function(e,t,n){var i=this;e.$info.addClass(B);var o=(0,m.default)("<div>").addClass(V).appendTo(e.$info);e.details=t.map(function(t,a){return t.info=e,i._createDetailsItem(o,t,a,!1,n)})}},{key:"_createDetailsItem",value:function(e,t,n,i,o){var a=this,r=(0,m.default)("<div>").appendTo(e);return n!==-1&&r.addClass(q),this._createProgressBox(r,{commonText:t.commonText,imageUrl:t.imageUrl,skipProgressBox:i,showCloseButton:o,showCloseButtonAlways:o,onCloseButtonClick:function(){return a._cancelOperationItem(t,n)}})}},{key:"completeOperationItem",value:function(e,t,n){e.allowProgressAutoUpdate&&this.updateOperationItemProgress(e,t,100,n),this._setCloseButtonVisible(e.details[t],!1)}},{key:"updateOperationItemProgress",value:function(e,t,n,i){if(e.common.progressBar.option("value",i),e.details){var o=e.details[t];o.progressBar.option("value",n)}}},{key:"completeOperation",value:function(e,t,n,i){e.completed=!0,e.common.$commonText.text(t),n?this._removeProgressBar(e.common):e.allowProgressAutoUpdate&&e.common.progressBar.option("value",100),i&&this._setProgressBarText(e.common,i),this._setCloseButtonVisible(e.common,!0)}},{key:"completeSingleOperationWithError",value:function(e,t){var n,i=null===(n=e.details)||void 0===n?void 0:n[0];e.completed=!0,this._renderOperationError(i||e.common,t),this._setCloseButtonVisible(e.common,!0),i&&this._setCloseButtonVisible(i,!1)}},{key:"addOperationDetailsError",value:function(e,t,n){var i=e.details[t];this._renderOperationError(i,n),this._setCloseButtonVisible(i,!1)}},{key:"renderError",value:function(e,t,n){(0,m.default)("<div>").text(n).addClass(L).appendTo(e)}},{key:"createErrorDetailsProgressBox",value:function(e,t,n){var i=this._createDetailsItem(e,t,-1,!0);this._renderOperationError(i,n)}},{key:"_renderEmptyListText",value:function(){this._$infosContainer.text(x.default.format("dxFileManager-notificationProgressPanelEmptyListText"))}},{key:"_renderOperationError",value:function(e,t){this._removeProgressBar(e),this.renderError(e.$wrapper,e.$commonText,t)}},{key:"_removeProgressBar",value:function(e){e.progressBar&&(e.progressBar.dispose(),e.progressBar.$element().remove(),e.progressBar=null)}},{key:"_createProgressBox",value:function(e,t){var n=this;e.addClass(F),t.showCloseButtonAlways||e.addClass(H),t.imageUrl&&(0,b.getImageContainer)(t.imageUrl).addClass(z).appendTo(e);var i=(0,m.default)("<div>").addClass(N).appendTo(e),o=(0,m.default)("<div>").addClass(j).text(t.commonText).appendTo(i),a=null;if(!t.skipProgressBox){var r=(0,m.default)("<div>").addClass(W).appendTo(i);a=this._createComponent(r,C.default,{min:0,max:100,width:"100%",validationMessageMode:"always",statusFormat:function(e,t){return n._getStatusString(e,t)}})}var s=null;if(t.showCloseButton){var l=(0,m.default)("<div>").addClass(G).appendTo(e);s=this._createComponent(l,k.default,{icon:"dx-filemanager-i dx-filemanager-i-cancel",stylingMode:"text",visible:t.showCloseButtonAlways,onClick:t.onCloseButtonClick})}return{$commonText:o,progressBar:a,$element:e,$wrapper:i,closeButton:s}}},{key:"_setCloseButtonVisible",value:function(e,t){e.closeButton&&(e.$element.toggleClass(H,!t),e.closeButton.option("visible",t))}},{key:"_setProgressBarText",value:function(e,t){e.progressBar.option("statusFormat",function(){return t})}},{key:"_closeOperation",value:function(e){var t=this;e.customCloseHandling&&!e.completed?(this._raiseOperationCanceled(e),this._setCloseButtonVisible(e.common,!1),e.details.forEach(function(e){return t._displayClosedOperationItem(e)})):(this._raiseOperationClosed(e),e.$info.next(".".concat(P)).remove(),e.$info.remove(),this._operationCount--,this._operationCount||this._renderEmptyListText())}},{key:"_cancelOperationItem",value:function(e,t){this._raiseOperationItemCanceled(e,t);var n=e.info.details[t];this._displayClosedOperationItem(n)}},{key:"_displayClosedOperationItem",value:function(e){this._setProgressBarText(e,x.default.format("dxFileManager-notificationProgressPanelOperationCanceled")),this._setCloseButtonVisible(e,!1)}},{key:"_getStatusString",value:function(e,t){return 1===e?x.default.format("Done"):Math.round(100*e)+"%"}},{key:"_raiseOperationClosed",value:function(e){this._actions.onOperationClosed({info:e})}},{key:"_raiseOperationCanceled",value:function(e){this._actions.onOperationCanceled({info:e})}},{key:"_raiseOperationItemCanceled",value:function(e,t){this._actions.onOperationItemCanceled({item:e,itemIndex:t})}},{key:"_raisePanelClosed",value:function(){this._actions.onPanelClosed()}}]),t}(w.default),K=$;t.default=K,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);
return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=o(n(2)),k=n(0),S=n(6),D=n(3),I=n(42),T=o(n(10)),O=o(n(16)),E=o(n(868)),A=o(n(872)),P=n(873),R=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"_initMarkup",value:function(){p(w(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._controller=this.option("controller"),this._controller.on("EditActionStarting",this._onEditActionStarting.bind(this)),this._controller.on("EditActionResultAcquired",this._onEditActionResultAcquired.bind(this)),this._controller.on("EditActionItemError",this._onEditActionItemError.bind(this)),this._controller.on("EditActionError",this._onEditActionError.bind(this)),this._controller.on("CompleteEditActionItem",this._onCompleteEditActionItem.bind(this)),this._controller.on("CompleteEditAction",this._onCompleteEditAction.bind(this)),this._model=this.option("model"),this._uploadOperationInfoMap={},this._dialogManager=new E.default(this.$element(),{chooseDirectoryDialog:{provider:this._controller._fileProvider,getDirectories:this._controller.getDirectories.bind(this._controller),getCurrentDirectory:this._controller.getCurrentDirectory.bind(this._controller)},onDialogClosed:this._onDialogClosed.bind(this)}),this._fileUploader=this._createFileUploader(),this._createMetadataMap()}},{key:"_initNotificationControl",value:function(e){var t=this;this._notificationControl=e,this._notificationControl.option({onOperationCanceled:function(e){var n=e.info;return t._onCancelUploadSession(n)},onOperationItemCanceled:function(e){var n=e.item,i=e.itemIndex;return t._onCancelFileUpload(n,i)}})}},{key:"_getFileUploaderComponent",value:function(){return A.default}},{key:"_createFileUploader",value:function(){var e=this,t=(0,C.default)("<div>").appendTo(this.$element());return this._createComponent(t,this._getFileUploaderComponent(),{getController:this._getFileUploaderController.bind(this),dropZonePlaceholderContainer:this.option("uploadDropZonePlaceholderContainer"),onUploadSessionStarted:function(t){return e._onUploadSessionStarted(t)},onUploadProgress:function(t){return e._onUploadProgress(t)}})}},{key:"setUploaderDropZone",value:function(e){this._fileUploader.option("dropZone",e)}},{key:"_getFileUploaderController",value:function(){var e=this,t=this.uploadDirectoryInfo.fileItem;return{chunkSize:this._controller.getFileUploadChunkSize(),uploadFileChunk:function(n,i){return e._controller.uploadFileChunk(n,i,t)},abortFileUpload:function(n,i){return e._controller.abortFileUpload(n,i,t)}}}},{key:"_createMetadataMap",value:function(){var e=this;this._metadataMap={create:{action:function(t){return e._tryCreate(t)},affectsAllItems:!0,singleItemProcessingMessage:T.default.format("dxFileManager-editingCreateSingleItemProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingCreateSingleItemSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingCreateSingleItemErrorMessage"),commonErrorMessage:T.default.format("dxFileManager-editingCreateCommonErrorMessage")},rename:{action:function(t){return e._tryRename(t)},singleItemProcessingMessage:T.default.format("dxFileManager-editingRenameSingleItemProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingRenameSingleItemSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingRenameSingleItemErrorMessage"),commonErrorMessage:T.default.format("dxFileManager-editingRenameCommonErrorMessage")},"delete":{action:function(t){return e._tryDelete(t)},singleItemProcessingMessage:T.default.format("dxFileManager-editingDeleteSingleItemProcessingMessage"),multipleItemsProcessingMessage:T.default.format("dxFileManager-editingDeleteMultipleItemsProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingDeleteSingleItemSuccessMessage"),multipleItemsSuccessMessage:T.default.format("dxFileManager-editingDeleteMultipleItemsSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingDeleteSingleItemErrorMessage"),multipleItemsErrorMessage:T.default.format("dxFileManager-editingDeleteMultipleItemsErrorMessage"),commonErrorMessage:T.default.format("dxFileManager-editingDeleteCommonErrorMessage")},move:{action:function(t){return e._tryMove(t)},singleItemProcessingMessage:T.default.format("dxFileManager-editingMoveSingleItemProcessingMessage"),multipleItemsProcessingMessage:T.default.format("dxFileManager-editingMoveMultipleItemsProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingMoveSingleItemSuccessMessage"),multipleItemsSuccessMessage:T.default.format("dxFileManager-editingMoveMultipleItemsSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingMoveSingleItemErrorMessage"),multipleItemsErrorMessage:T.default.format("dxFileManager-editingMoveMultipleItemsErrorMessage"),commonErrorMessage:T.default.format("dxFileManager-editingMoveCommonErrorMessage")},copy:{action:function(t){return e._tryCopy(t)},singleItemProcessingMessage:T.default.format("dxFileManager-editingCopySingleItemProcessingMessage"),multipleItemsProcessingMessage:T.default.format("dxFileManager-editingCopyMultipleItemsProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingCopySingleItemSuccessMessage"),multipleItemsSuccessMessage:T.default.format("dxFileManager-editingCopyMultipleItemsSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingCopySingleItemErrorMessage"),multipleItemsErrorMessage:T.default.format("dxFileManager-editingCopyMultipleItemsErrorMessage"),commonErrorMessage:T.default.format("dxFileManager-editingCopyCommonErrorMessage")},upload:{action:function(t){return e._tryUpload(t)},allowCancel:!0,allowItemProgress:!0,singleItemProcessingMessage:T.default.format("dxFileManager-editingUploadSingleItemProcessingMessage"),multipleItemsProcessingMessage:T.default.format("dxFileManager-editingUploadMultipleItemsProcessingMessage"),singleItemSuccessMessage:T.default.format("dxFileManager-editingUploadSingleItemSuccessMessage"),multipleItemsSuccessMessage:T.default.format("dxFileManager-editingUploadMultipleItemsSuccessMessage"),singleItemErrorMessage:T.default.format("dxFileManager-editingUploadSingleItemErrorMessage"),multipleItemsErrorMessage:T.default.format("dxFileManager-editingUploadMultipleItemsErrorMessage"),canceledMessage:T.default.format("dxFileManager-editingUploadCanceledMessage")},download:{action:function(t){return e._download(t)}},getItemContent:{action:function(t){return e._getItemContent(t)}},getItems:{singleItemProcessingMessage:"",singleItemErrorMessage:T.default.format("dxFileManager-errorDirectoryOpenFailed"),commonErrorMessage:T.default.format("dxFileManager-errorDirectoryOpenFailed")}}}},{key:"getCommandActions",value:function(){var e=this,t={};return(0,D.each)(this._metadataMap,function(n){Object.prototype.hasOwnProperty.call(e._metadataMap,n)&&(t[n]=function(t){return e._executeAction(n,t)})}),t}},{key:"_executeAction",value:function(e,t){var n=this._metadataMap[e];return n?n.action(t):null}},{key:"_onCancelUploadSession",value:function(e){this._fileUploader.cancelUpload(e.uploadSessionId)}},{key:"_onCancelFileUpload",value:function(e,t){this._fileUploader.cancelFileUpload(e.info.uploadSessionId,t)}},{key:"_onUploadProgress",value:function(e){var t=e.sessionId,n=e.fileIndex,i=e.commonValue,o=e.fileValue,a=this._uploadOperationInfoMap[t];this._notificationControl.updateOperationItemProgress(a,n,100*o,100*i)}},{key:"_onUploadSessionStarted",value:function(e){var t=e.sessionInfo;this._controller.processUploadSession(t,this.uploadDirectoryInfo)}},{key:"_onEditActionStarting",value:function(e){var t=this._metadataMap[e.name],n=new M(t,e.itemInfos,e.directory),i=this._notificationControl.addOperation(n.processingMessage,t.allowCancel,!t.allowItemProgress);switch((0,k.extend)(e.customData,{context:n,operationInfo:i}),e.name){case"upload":var o=e.customData.sessionInfo.sessionId;i.uploadSessionId=o,this._uploadOperationInfoMap[o]=i;break;case"rename":e.customData.context.itemNewName=e.customData.itemNewName}}},{key:"_onEditActionResultAcquired",value:function(e){var t=this,n=e.customData,i=n.context,o=n.operationInfo;i.singleRequest=e.singleRequest;var a=i.itemInfos.map(function(e){return t._getItemProgressDisplayInfo(e)});this._notificationControl.addOperationDetails(o,a,i.actionMetadata.allowCancel)}},{key:"_onEditActionError",value:function(e,t){var n=e.customData,i=n.context,o=n.operationInfo;i.singleRequest=e.singleRequest,this._handleActionError(o,i,t),this._completeAction(o,i)}},{key:"_onEditActionItemError",value:function(e,t){var n=e.customData,i=n.context,o=n.operationInfo;this._handleActionError(o,i,t)}},{key:"_onCompleteEditActionItem",value:function(e,t){var n=e.customData,i=n.context,o=n.operationInfo;t.result&&t.result.canceled||(i.completeOperationItem(t.index),this._notificationControl.completeOperationItem(o,t.index,i.commonProgress))}},{key:"_onCompleteEditAction",value:function(e){var t=e.customData,n=t.context,i=t.operationInfo;this._completeAction(i,n),"upload"===e.name&&delete this._uploadOperationInfoMap[e.customData.sessionInfo.sessionId]}},{key:"_tryCreate",value:function(e){var t=this,n=e&&e[0]||this._getCurrentDirectory(),i=T.default.format("dxFileManager-newDirectoryName");return this._showDialog(this._dialogManager.getCreateItemDialog(),i).then(function(e){var i=e.name;return t._controller.createDirectory(n,i)})}},{key:"_tryRename",value:function(e){var t=this,n=e&&e[0]||this._model.getMultipleSelectedItems()[0];return this._showDialog(this._dialogManager.getRenameItemDialog(),n.fileItem.name).then(function(e){var i=e.name;return t._controller.renameItem(n,i)})}},{key:"_tryDelete",value:function(e){var t=this;e=e||this._model.getMultipleSelectedItems();var n=e[0].fileItem.name,i=e.length;return this._showDialog(this._dialogManager.getDeleteItemDialog(),{itemName:n,itemCount:i}).then(function(){return t._controller.deleteItems(e)})}},{key:"_tryMove",value:function(e){var t=this;return e=e||this._model.getMultipleSelectedItems(),this._showDialog(this._dialogManager.getMoveDialog(e)).then(function(n){var i=n.folder;return t._controller.moveItems(e,i)})}},{key:"_tryCopy",value:function(e){var t=this;return e=e||this._model.getMultipleSelectedItems(),this._showDialog(this._dialogManager.getCopyDialog(e)).then(function(n){var i=n.folder;return t._controller.copyItems(e,i)})}},{key:"_tryUpload",value:function(e){this._uploadDirectoryInfo=null===e||void 0===e?void 0:e[0],this._fileUploader.tryUpload()}},{key:"_download",value:function(e){return e=e||this._model.getMultipleSelectedItems(),this._controller.downloadItems(e)}},{key:"_getItemContent",value:function(e){return e=e||this._model.getMultipleSelectedItems(),this._controller.getItemContent(e)}},{key:"_completeAction",value:function(e,t){this._notificationControl.completeOperation(e,t.completionMessage,!t.success,t.statusText),t.hasModifiedItems()&&this._raiseOnSuccess(t.onlyFiles)}},{key:"_handleActionError",value:function(e,t,n){e.hasError=!0,t.singleRequest?this._handleSingleRequestActionError(e,t,n):this._handleMultipleRequestActionError(e,t,n)}},{key:"_handleSingleRequestActionError",value:function(e,t,n){var i=t.getItemForSingleRequestError(),o=t.itemNewName,a=this._getErrorText(n,i,o);t.processSingleRequestError(a);var r=this._getOperationErrorInfo(t);this._notificationControl.completeSingleOperationWithError(e,r),t.multipleItems&&this._raiseOnSuccess(t.onlyFiles)}},{key:"_handleMultipleRequestActionError",value:function(e,t,n){var i=t.getItemForMultipleRequestError(n.index),o=this._getErrorText(n,i);t.processMultipleRequestError(n.index,o);var a=this._getOperationErrorInfo(t);this._notificationControl.addOperationDetailsError(e,a)}},{key:"_getOperationErrorInfo",value:function(e){var t=e.errorState.currentDetailError;return{commonErrorText:e.errorState.commonErrorText,item:t.itemInfo?this._getItemProgressDisplayInfo(t.itemInfo):null,itemIndex:t.itemIndex,detailErrorText:t.errorText}}},{key:"_getErrorText",value:function(e,t,n){n=n||(null===t||void 0===t?void 0:t.fileItem.name);var i=e.errorText||P.FileManagerMessages.get(e.errorId,n),o={fileSystemItem:null===t||void 0===t?void 0:t.fileItem,errorCode:e.errorId,errorText:i};return this._raiseOnError(o),o.errorText}},{key:"_getItemProgressDisplayInfo",value:function(e){return{commonText:e.fileItem.name,imageUrl:this._getItemThumbnail(e)}}},{key:"_showDialog",value:function(e,t){return this._dialogDeferred=new S.Deferred,e.show(t),this._dialogDeferred.promise()}},{key:"_onDialogClosed",value:function(e){var t=e.dialogResult;t?this._dialogDeferred.resolve(t):this._dialogDeferred.reject()}},{key:"_getItemThumbnail",value:function(e){var t=this.option("getItemThumbnail");if(!t)return null;var n=t(e);return n?n.thumbnail:null}},{key:"_initActions",value:function(){this._actions={onSuccess:this._createActionByOption("onSuccess"),onError:this._createActionByOption("onError"),onCreating:this._createActionByOption("onCreating")}}},{key:"_getDefaultOptions",value:function(){return(0,k.extend)(p(w(t.prototype),"_getDefaultOptions",this).call(this),{model:{getMultipleSelectedItems:null},notificationControl:null,getItemThumbnail:null,onSuccess:null,onError:null,onCreating:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"model":this.repaint();break;case"notificationControl":this._initNotificationControl(e.value);break;case"getItemThumbnail":break;case"uploadDropZonePlaceholderContainer":this._fileUploader.option("dropZonePlaceholderContainer",e.value);break;case"onSuccess":case"onError":case"onCreating":this._actions[n]=this._createActionByOption(n);break;default:p(w(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_raiseOnSuccess",value:function(e){this._actions.onSuccess({updatedOnlyFiles:e})}},{key:"_raiseOnError",value:function(e){this._actions.onError(e)}},{key:"_getCurrentDirectory",value:function(){return this._controller.getCurrentDirectory()}},{key:"uploadDirectoryInfo",get:function(){return this._uploadDirectoryInfo||this._getCurrentDirectory()}}]),t}(O.default),M=function(){function e(t,n,i){d(this,e),this._actionMetadata=t,this._itemInfos=n,this._onlyFiles=!this._actionMetadata.affectsAllItems&&this._itemInfos.every(function(e){return!e.fileItem.isDirectory}),this._items=this._itemInfos.map(function(e){return e.fileItem}),this._multipleItems=this._items.length>1,this._location=i.getDisplayName(),this._singleRequest=!0,this._completedItems=[],this._commonProgress=0,this._errorState={failedCount:0},this._itemNewName=""}return f(e,[{key:"completeOperationItem",value:function(e){if(this._singleRequest)this._completedItems=a(this._items);else{var t=this._items[e];this._completedItems.push(t)}this._actionMetadata.allowItemProgress||(this._commonProgress=this._completedItems.length/this._items.length*100)}},{key:"processSingleRequestError",value:function(e){this._errorState.failedCount=1,this._errorState.commonErrorText=this._multipleItems?this._actionMetadata.commonErrorMessage:this._actionMetadata.singleItemErrorMessage;var t=this._multipleItems?-1:1,n=this.getItemForSingleRequestError();this._setCurrentDetailError(t,n,e)}},{key:"processMultipleRequestError",value:function(e,t){this._errorState.failedCount++,this._errorState.commonErrorText=this._errorState.failedCount>1?(0,I.format)(this._actionMetadata.multipleItemsErrorMessage,this._errorState.failedCount):this._actionMetadata.singleItemErrorMessage;var n=this.getItemForMultipleRequestError(e);this._setCurrentDetailError(e,n,t)}},{key:"hasModifiedItems",value:function(){return this._hasCompletedItems()||this._singleRequest&&!this.success&&this._multipleItems}},{key:"getItemForSingleRequestError",value:function(){return this._multipleItems?null:this._itemInfos[0]}},{key:"getItemForMultipleRequestError",value:function(e){return this._itemInfos[e]}},{key:"_setCurrentDetailError",value:function(e,t,n){this._errorState.currentDetailError={itemIndex:e,itemInfo:t,errorText:n}}},{key:"_hasCompletedItems",value:function(){return this._completedItems.length>0}},{key:"actionMetadata",get:function(){return this._actionMetadata}},{key:"itemInfos",get:function(){return this._itemInfos}},{key:"itemNewName",get:function(){return this._itemNewName},set:function(e){this._itemNewName=e}},{key:"errorState",get:function(){return this._errorState}},{key:"singleRequest",get:function(){return this._singleRequest},set:function(e){this._singleRequest=e}},{key:"multipleItems",get:function(){return this._multipleItems}},{key:"onlyFiles",get:function(){return this._onlyFiles}},{key:"processingMessage",get:function(){return this._multipleItems?(0,I.format)(this._actionMetadata.multipleItemsProcessingMessage,this._items.length,this._location):(0,I.format)(this._actionMetadata.singleItemProcessingMessage,this._location)}},{key:"successMessage",get:function(){return this._hasCompletedItems()?this._multipleItems?(0,I.format)(this._actionMetadata.multipleItemsSuccessMessage,this._completedItems.length,this._location):(0,I.format)(this._actionMetadata.singleItemSuccessMessage,this._location):this._multipleItems?(0,I.format)(this._actionMetadata.multipleItemsErrorMessage,this._items.length):this._actionMetadata.singleItemErrorMessage}},{key:"completionMessage",get:function(){return this.success?this.successMessage:this.errorState.commonErrorText}},{key:"statusText",get:function(){return this.success&&!this._hasCompletedItems()?this._actionMetadata.canceledMessage:void 0}},{key:"commonProgress",get:function(){return this._commonProgress}},{key:"success",get:function(){return!this._errorState.failedCount}}]),e}(),B=R;t.default=B,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.default=void 0;var s=i(n(2)),l=n(0),u=i(n(10)),c=i(n(869)),d=i(n(870)),h=i(n(871)),f=function(){function e(t,n){o(this,e),this._$element=t,this._options=n;var i=(0,s.default)("<div>").appendTo(this._$element);this._chooseDirectoryDialog=new d.default(i,(0,l.extend)(this._options.chooseDirectoryDialog,{onClosed:this._options.onDialogClosed}));var a=(0,s.default)("<div>").appendTo(this._$element);this._renameItemDialog=new c.default(a,{title:u.default.format("dxFileManager-dialogRenameItemTitle"),buttonText:u.default.format("dxFileManager-dialogRenameItemButtonText"),onClosed:this._options.onDialogClosed});var r=(0,s.default)("<div>").appendTo(this._$element);this._createItemDialog=new c.default(r,{title:u.default.format("dxFileManager-dialogCreateDirectoryTitle"),buttonText:u.default.format("dxFileManager-dialogCreateDirectoryButtonText"),onClosed:this._options.onDialogClosed});var f=(0,s.default)("<div>").appendTo(this._$element);this._deleteItemDialog=new h.default(f,{onClosed:this._options.onDialogClosed})}return r(e,[{key:"getCopyDialog",value:function(e){return this._chooseDirectoryDialog.switchToCopyDialog(e),this._chooseDirectoryDialog}},{key:"getMoveDialog",value:function(e){return this._chooseDirectoryDialog.switchToMoveDialog(e),this._chooseDirectoryDialog}},{key:"getRenameItemDialog",value:function(){return this._renameItemDialog}},{key:"getCreateItemDialog",value:function(){return this._createItemDialog}},{key:"getDeleteItemDialog",value:function(){return this._deleteItemDialog}}]),e}(),p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(104)),b=o(n(318)),x="dx-filemanager-dialog-name-editor",w="dx-filemanager-dialog-name-editor-popup",C=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"show",value:function(e){e=e||"",this._nameTextBox?this._nameTextBox.option("value",e):this._initialNameValue=e,l(_(t.prototype),"show",this).call(this)}},{key:"_onPopupShown",value:function(){if(this._nameTextBox){var e=this._nameTextBox._input();e.length&&e[0].select(),this._nameTextBox.focus()}}},{key:"_getDialogOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDialogOptions",this).call(this),{title:this.option("title"),buttonText:this.option("buttonText"),contentCssClass:x,popupCssClass:w})}},{key:"_createContentTemplate",value:function(e){l(_(t.prototype),"_createContentTemplate",this).call(this,e),this._nameTextBox=this._createComponent((0,m.default)("<div>"),y.default,{value:this._initialNameValue,onEnterKey:this._applyDialogChanges.bind(this)}),this._$contentElement.append(this._nameTextBox.$element())}},{key:"_getDialogResult",value:function(){var e=this._nameTextBox.option("value");return e?{name:e}:null}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{title:"",buttonText:""})}}]),t}(b.default),k=C;t.default=k,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e){return u(e)||l(e)||s(e)||r()}function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function s(e,t){if(e){if("string"==typeof e)return c(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?c(e,t):void 0}}function l(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function u(e){if(Array.isArray(e))return c(e)}function c(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function d(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function h(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function f(e,t,n){return t&&h(e.prototype,t),n&&h(e,n),e}function p(e,t,n){return(p="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=g(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function g(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=w(e),null!==e););return e}function _(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function v(e){var t=x();return function(){var n,i=w(e);if(t){var o=w(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return y(this,n)}}function y(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?b(e):t}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function x(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function w(e){return(w=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var C=o(n(2)),k=n(0),S=o(n(10)),D=n(149),I=o(n(318)),T=o(n(534)),O="dx-filemanager-dialog-folder-chooser",E="dx-filemanager-dialog-folder-chooser-popup",A=function(e){function t(){return d(this,t),n.apply(this,arguments)}_(t,e);var n=v(t);return f(t,[{key:"show",value:function(){var e;this._resetDialogSelectedDirectory(),null===(e=this._filesTreeView)||void 0===e?void 0:e.refresh(),p(w(t.prototype),"show",this).call(this)}},{key:"switchToCopyDialog",value:function(e){this._targetItemInfos=e,this._setTitle(S.default.format("dxFileManager-dialogDirectoryChooserCopyTitle")),this._setButtonText(S.default.format("dxFileManager-dialogDirectoryChooserCopyButtonText"))}},{key:"switchToMoveDialog",value:function(e){this._targetItemInfos=e,this._setTitle(S.default.format("dxFileManager-dialogDirectoryChooserMoveTitle")),this._setButtonText(S.default.format("dxFileManager-dialogDirectoryChooserMoveButtonText"))}},{key:"_getDialogOptions",value:function(){return(0,k.extend)(p(w(t.prototype),"_getDialogOptions",this).call(this),{contentCssClass:O,popupCssClass:E})}},{key:"_createContentTemplate",value:function(e){var n=this;p(w(t.prototype),"_createContentTemplate",this).call(this,e),this._filesTreeView=this._createComponent((0,C.default)("<div>"),T.default,{getDirectories:this.option("getDirectories"),getCurrentDirectory:function(){return n._getDialogSelectedDirectory()},onDirectoryClick:function(e){return n._onFilesTreeViewDirectoryClick(e)},onFilesTreeViewContentReady:function(){return n._toggleUnavailableLocationsDisabled(!0)}}),this._$contentElement.append(this._filesTreeView.$element())}},{key:"_getDialogResult",value:function(){var e=this._getDialogSelectedDirectory();return e?{folder:e}:e}},{key:"_getDefaultOptions",value:function(){return(0,k.extend)(p(w(t.prototype),"_getDefaultOptions",this).call(this),{getItems:null})}},{key:"_getDialogSelectedDirectory",value:function(){return this._selectedDirectoryInfo}},{key:"_resetDialogSelectedDirectory",value:function(){this._selectedDirectoryInfo=null}},{key:"_onFilesTreeViewDirectoryClick",value:function(e){var t=e.itemData;this._selectedDirectoryInfo=t,this._filesTreeView.updateCurrentDirectory()}},{key:"_onPopupShown",value:function(){this._toggleUnavailableLocationsDisabled(!0),p(w(t.prototype),"_onPopupShown",this).call(this)}},{key:"_onPopupHidden",value:function(){this._toggleUnavailableLocationsDisabled(!1),p(w(t.prototype),"_onPopupHidden",this).call(this)}},{key:"_toggleUnavailableLocationsDisabled",value:function(e){var t=this;if(this._filesTreeView){var n=this._getLocationsToProcess(e);this._filesTreeView.toggleDirectoryExpandedStateRecursive(n.locationsToExpand[0],e).then(function(){return t._filesTreeView.toggleDirectoryLineExpandedState(n.locationsToCollapse,!e).then(function(){return n.locationKeysToDisable.forEach(function(n){return t._filesTreeView.toggleNodeDisabledState(n,e)})})})}}},{key:"_getLocationsToProcess",value:function(e){var t,n={},i={};this._targetItemInfos.forEach(function(e){e.parentDirectory&&(n[e.parentDirectory.getInternalKey()]=e.parentDirectory),e.fileItem.isDirectory&&(i[e.getInternalKey()]=e)});var o=(0,D.getMapFromObject)(n),r=(0,D.getMapFromObject)(i);return{locationsToExpand:e?o.values:[],locationsToCollapse:e?r.values:[],locationKeysToDisable:(t=o.keys).concat.apply(t,a(r.keys))}}}]),t}(I.default),P=A;t.default=P,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(10)),b=o(n(96)),x=o(n(318)),w="dx-filemanager-dialog-delete-item",C="dx-filemanager-dialog-delete-item-popup",k=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"show",value:function(e){var n=e.itemName,i=e.itemCount,o=1===i?y.default.format("dxFileManager-dialogDeleteItemSingleItemConfirmation",n):y.default.format("dxFileManager-dialogDeleteItemMultipleItemsConfirmation",i);this._$text?this._$text.text(o):this._initialText=o,l(_(t.prototype),"show",this).call(this)}},{key:"_getDialogOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDialogOptions",this).call(this),{title:y.default.format("dxFileManager-dialogDeleteItemTitle"),buttonText:y.default.format("dxFileManager-dialogDeleteItemButtonText"),contentCssClass:w,popupCssClass:C,height:"auto",maxHeight:"80vh"})}},{key:"_createContentTemplate",value:function(e){l(_(t.prototype),"_createContentTemplate",this).call(this,e),this._$text=(0,m.default)("<div>").text(this._initialText).appendTo(this._$contentElement),this._createComponent(this._$contentElement,b.default,{width:"100%",height:"100%"})}},{key:"_getDialogResult",value:function(){return{}}}]),t}(x.default),S=k;t.default=S,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(6),b=n(7),x=o(n(35)),w=o(n(16)),C=o(n(294)),k=n(149),S="dx-filemanager-fileuploader",D="dx-filemanager-fileuploader-dropzone-placeholder",I=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){this._initActions(),this.$element().addClass(S),this._uploaderInfos=[],this._createInternalFileUploader(),this._createDropZonePlaceholder(),this._setDropZonePlaceholderVisible(!1),l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"_createInternalFileUploader",value:function(){var e=this,t=this._getController().chunkSize,n=(0,m.default)("<div>").appendTo(this.$element()),i=this._createComponent(n,C.default,{name:"file",multiple:!0,showFileList:!1,activeStateEnabled:!1,focusStateEnabled:!1,hoverStateEnabled:!1,labelText:"",readyToUploadMessage:"",accept:"*",chunkSize:t,dropZone:this.option("dropZone"),onValueChanged:function(t){return e._onFileUploaderValueChanged(t)},onProgress:function(t){return e._onFileUploaderProgress(t)},onUploaded:function(t){return e._onFileUploaderUploaded(t)},onUploadAborted:function(t){return e._onFileUploaderUploadAborted(t)},onUploadError:function(t){return e._onFileUploaderUploadError(t)},onDropZoneEnter:function(){return e._setDropZonePlaceholderVisible(!0)},onDropZoneLeave:function(){return e._setDropZonePlaceholderVisible(!1)}});i.option({uploadChunk:function(t,n){return e._fileUploaderUploadChunk(i,t,n)},abortUpload:function(t,n){return e._fileUploaderAbortUpload(i,t,n)}});var o={fileUploader:i};this._uploaderInfos.push(o)}},{key:"tryUpload",value:function(){var e=this._findAndUpdateAvailableUploaderInfo();e&&e.fileUploader._selectButtonClickHandler()}},{key:"cancelUpload",value:function(e){this._cancelUpload(e)}},{key:"cancelFileUpload",value:function(e,t){this._cancelUpload(e,t)}},{key:"_cancelUpload",value:function(e,t){var n=this._findUploaderInfoBySessionId(e),i=n.fileUploader;i.abortUpload(t)}},{key:"_fileUploaderUploadChunk",value:function(e,t,n){var i=this._findSessionByFile(e,t),o=i.session,a=i.fileIndex,r=o.controller;return n.fileIndex=a,r.uploadFileChunk(t,n)}},{key:"_fileUploaderAbortUpload",value:function(e,t,n){var i=this._findSessionByFile(e,t),o=i.session,a=i.fileIndex,r=o.controller;return n.fileIndex=a,r.abortFileUpload(t,n)}},{key:"_onFileUploaderValueChanged",value:function(e){var t=this,n=e.component,i=e.value;if(0!==i.length){var o=i.slice(),a=this._findUploaderInfo(n);this._uploadFiles(a,o),setTimeout(function(){t._findAndUpdateAvailableUploaderInfo()||t._createInternalFileUploader()})}}},{key:"_onFileUploaderProgress",value:function(e){var t=e.component,n=e.file,i=e.bytesLoaded,o=e.bytesTotal,a=this._findSessionByFile(t,n),r=a.session,s=a.fileIndex,l=0!==o?i/o:1,u=t.option("progress")/100,c={sessionId:r.id,fileIndex:s,commonValue:u,fileValue:l};this._raiseUploadProgress(c)}},{key:"_onFileUploaderUploaded",value:function(e){var t=e.component,n=e.file,i=this._getDeferredForFile(t,n);i.resolve()}},{key:"_onFileUploaderUploadAborted",value:function(e){var t=e.component,n=e.file,i=this._getDeferredForFile(t,n);i.resolve({canceled:!0})}},{key:"_onFileUploaderUploadError",value:function(e){var t=e.component,n=e.file,i=e.error,o=this._getDeferredForFile(t,n);o.reject(i)}},{key:"_createDropZonePlaceholder",value:function(){this._$dropZonePlaceholder=(0,m.default)("<div>").addClass(D).appendTo(this.option("dropZonePlaceholderContainer"))}},{key:"_adjustDropZonePlaceholder",value:function(){if((0,b.hasWindow)()){var e=this.option("dropZone"),t=parseFloat(this._$dropZonePlaceholder.css("borderTopWidth")),n=parseFloat(this._$dropZonePlaceholder.css("borderLeftWidth")),i=this.option("dropZonePlaceholderContainer"),o=parseFloat(i.css("borderBottomWidth")),a=parseFloat(i.css("borderLeftWidth")),r=i.innerHeight(),s=i.offset(),l=e.offset();this._$dropZonePlaceholder.css({top:l.top-s.top-r-o,left:l.left-s.left-a}),this._$dropZonePlaceholder.height(e.get(0).offsetHeight-2*t),this._$dropZonePlaceholder.width(e.get(0).offsetWidth-2*n)}}},{key:"_setDropZonePlaceholderVisible",value:function(e){e?(this._adjustDropZonePlaceholder(),this._$dropZonePlaceholder.css("display","")):this._$dropZonePlaceholder.css("display","none")}},{key:"_uploadFiles",value:function(e,t){this._setDropZonePlaceholderVisible(!1);var n=(new x.default).toString(),i=this._getController(),o=t.map(function(){return new y.Deferred}),a={id:n,controller:i,files:t,deferreds:o};e.session=a;var r={sessionId:n,deferreds:o,files:t};return this._raiseUploadSessionStarted(r),(0,k.whenSome)(o).always(function(){return setTimeout(function(){e.fileUploader.reset(),e.session=null})})}},{key:"_getDeferredForFile",value:function(e,t){var n=this._findSessionByFile(e,t),i=n.session,o=n.fileIndex;return i.deferreds[o]}},{key:"_findSessionByFile",value:function(e,t){var n=this._findUploaderInfo(e),i=n.session,o=i.files.indexOf(t);return{session:i,fileIndex:o}}},{key:"_findUploaderInfoBySessionId",value:function(e){for(var t=0;t<this._uploaderInfos.length;t++){var n=this._uploaderInfos[t],i=n.session;if(i&&i.id===e)return n}return null}},{key:"_findAndUpdateAvailableUploaderInfo",value:function(){for(var e,t=null,n=0;n<this._uploaderInfos.length;n++){var i=this._uploaderInfos[n];i.fileUploader.option("dropZone",""),t||i.session||(t=i)}return null===(e=t)||void 0===e?void 0:e.fileUploader.option("dropZone",this.option("dropZone")),t}},{key:"_findUploaderInfo",value:function(e){for(var t=0;t<this._uploaderInfos.length;t++){var n=this._uploaderInfos[t];if(n.fileUploader===e)return n}return null}},{key:"_getController",value:function(){var e=this.option("getController");return e()}},{key:"_raiseUploadSessionStarted",value:function(e){this._actions.onUploadSessionStarted({sessionInfo:e})}},{key:"_raiseUploadProgress",value:function(e){this._actions.onUploadProgress(e)}},{key:"_initActions",value:function(){this._actions={onUploadSessionStarted:this._createActionByOption("onUploadSessionStarted"),onUploadProgress:this._createActionByOption("onUploadProgress")}}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{getController:null,onUploadSessionStarted:null,onUploadProgress:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"getController":this.repaint();break;case"onUploadSessionStarted":case"onUploadProgress":this._actions[n]=this._createActionByOption(n);break;case"dropZone":this._findAndUpdateAvailableUploaderInfo(),this._adjustDropZonePlaceholder();break;case"dropZonePlaceholderContainer":this._$dropZonePlaceholder.detach(),this._$dropZonePlaceholder.appendTo(e.value);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(w.default),T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}Object.defineProperty(t,"ErrorCode",{enumerable:!0,get:function(){return a.default}}),t.FileManagerMessages=void 0;var o=i(n(10)),a=i(n(279)),r={get:function(e,t){switch(e){case a.default.NoAccess:return o.default.format("dxFileManager-errorNoAccess");case a.default.FileExists:return o.default.format("dxFileManager-errorFileExistsFormat",t);case a.default.FileNotFound:return o.default.format("dxFileManager-errorFileNotFoundFormat",t);case a.default.DirectoryExists:return o.default.format("dxFileManager-errorDirectoryExistsFormat",t);case a.default.DirectoryNotFound:return o.default.format("dxFileManager-errorDirectoryNotFoundFormat",t);case a.default.WrongFileExtension:return o.default.format("dxFileManager-errorWrongFileExtension");case a.default.MaxFileSizeExceeded:return o.default.format("dxFileManager-errorMaxFileSizeExceeded");case a.default.InvalidSymbols:return o.default.format("dxFileManager-errorInvalidSymbols")}return o.default.format("dxFileManager-errorDefault")}};t.FileManagerMessages=r},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(16)),b=o(n(488)),x="dx-filemanager-breadcrumbs",w=x+"-parent-folder-item",C=x+"-separator-item",k=x+"-path-separator-item",S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._currentDirectory=null}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._currentDirectory&&this._renderMenu(),this.$element().addClass(x)}},{key:"setCurrentDirectory",value:function(e){this._areDirsEqual(this._currentDirectory,e)||(this._currentDirectory=e,this.repaint())}},{key:"_renderMenu",value:function(){var e=(0,m.default)("<div>").appendTo(this.$element());this._menu=this._createComponent(e,b.default,{dataSource:this._getMenuItems(),onItemClick:this._onItemClick.bind(this),onItemRendered:this._onItemRendered.bind(this)})}},{key:"_getMenuItems",value:function(){var e=this._getParentDirsLine(),t=[{icon:"arrowup",directory:this._currentDirectory.parentDirectory,isPathItem:!0,cssClass:w},{text:"\xa0",cssClass:C}];return e.forEach(function(n,i){t.push({text:n.getDisplayName(),directory:n,isPathItem:!0}),i!==e.length-1&&t.push({icon:"spinnext",cssClass:k})}),t}},{key:"_onItemClick",value:function(e){var t=e.itemData;if(t.isPathItem){var n=t.directory;this._areDirsEqual(n,this._currentDirectory)||this._raiseCurrentDirectoryChanged(n)}}},{key:"_onItemRendered",value:function(e){var t=e.itemElement,n=e.itemData;n.cssClass&&(0,m.default)(t).addClass(n.cssClass)}},{key:"_getParentDirsLine",value:function(){for(var e=this._currentDirectory,t=[];e;)t.unshift(e),e=e.parentDirectory;return t}},{key:"_areDirsEqual",value:function(e,t){return e&&t&&e===t&&e.fileItem.key===t.fileItem.key}},{key:"_initActions",value:function(){this._actions={onCurrentDirectoryChanging:this._createActionByOption("onCurrentDirectoryChanging")}}},{key:"_raiseCurrentDirectoryChanged",value:function(e){this._actions.onCurrentDirectoryChanging({currentDirectory:e})}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{rootFolderDisplayName:"Files",onCurrentDirectoryChanging:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"rootFolderDisplayName":this.repaint();break;case"onCurrentDirectoryChanging":this._actions[n]=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(y.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(1),b=n(7),x=o(n(16)),w=o(n(291)),C=o(n(537)),k=(0,b.getWindow)(),S=573,D="dx-filemanager-adaptivity-drawer-panel",I="dx-drawer-panel-content-initial",T="dx-drawer-panel-content-adaptive",O=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this._initActions(),this._isInAdaptiveState=!1;var e=(0,m.default)("<div>").appendTo(this.$element());(0,m.default)("<div>").addClass(D).appendTo(e),this._drawer=this._createComponent(e,w.default),this._drawer.option({opened:!0,template:this._createDrawerTemplate.bind(this)}),(0,m.default)(this._drawer.content()).addClass(I);var n=e.find(".".concat(D)).first(),i=this.option("contentTemplate");(0,y.isFunction)(i)&&i(n),this._updateDrawerMaxSize()}},{key:"_createDrawerTemplate",value:function(e){this.option("drawerTemplate")(e),this._splitter=this._createComponent("<div>",C.default,{container:this.$element(),leftElement:(0,m.default)(this._drawer.content()),rightElement:(0,m.default)(this._drawer.viewContent()),onApplyPanelSize:this._onApplyPanelSize.bind(this)}),this._splitter.$element().appendTo(e)}},{key:"_render",value:function(){l(_(t.prototype),"_render",this).call(this),this._checkAdaptiveState()}},{key:"_onApplyPanelSize",value:function(e){if((0,b.hasWindow)()){if(!this._splitter.isSplitterMoved())return void this._setDrawerWidth("");(0,m.default)(this._drawer.content()).removeClass(I),this._setDrawerWidth(e.leftPanelWidth)}}},{key:"_setDrawerWidth",value:function(e){(0,m.default)(this._drawer.content()).css("width",e),this._updateDrawerMaxSize(),this._drawer.resizeViewContent()}},{key:"_updateDrawerMaxSize",value:function(){this._drawer.option("maxSize",this._drawer.getRealPanelWidth())}},{key:"_dimensionChanged",value:function(e){e&&"height"===e||this._checkAdaptiveState()}},{key:"_checkAdaptiveState",value:function(){var e=this._isInAdaptiveState;this._isInAdaptiveState=this._isSmallScreen(),e!==this._isInAdaptiveState&&(this.toggleDrawer(!this._isInAdaptiveState,!0),(0,m.default)(this._drawer.content()).toggleClass(T,this._isInAdaptiveState),this._raiseAdaptiveStateChanged(this._isInAdaptiveState)),this._isInAdaptiveState&&this._isDrawerOpened()&&this._updateDrawerMaxSize()}},{key:"_isSmallScreen",value:function(){return(0,m.default)(k).width()<=S}},{key:"_isDrawerOpened",value:function(){return this._drawer.option("opened")}},{key:"_initActions",value:function(){this._actions={onAdaptiveStateChanged:this._createActionByOption("onAdaptiveStateChanged")}}},{key:"_raiseAdaptiveStateChanged",value:function(e){this._actions.onAdaptiveStateChanged({enabled:e})}},{key:"_getDefaultOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{drawerTemplate:null,contentTemplate:null,onAdaptiveStateChanged:null})}},{key:"_optionChanged",value:function(e){var n=e.name;switch(n){case"drawerTemplate":case"contentTemplate":this.repaint();break;case"onAdaptiveStateChanged":this._actions[n]=this._createActionByOption(n);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"isInAdaptiveState",value:function(){return this._isInAdaptiveState}},{key:"toggleDrawer",value:function(e,t){this._updateDrawerMaxSize(),this._drawer.option("animationEnabled",!t),this._drawer.toggle(e);var n=this._isDrawerOpened()&&!this.isInAdaptiveState();this._splitter.toggleState(n)}}]),t}(x.default),E=O;t.default=E,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(877)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function l(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function u(e,t,n){return t&&l(e.prototype,t),n&&l(e,n),e}function c(e,t,n){return(c="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=d(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function d(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=v(e),null!==e););return e}function h(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&f(e,t)}function f(e,t){return(f=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function p(e){var t=m();return function(){var n,i=v(e);if(t){var o=v(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return g(this,n)}}function g(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?_(e):t}function _(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function m(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function v(e){return(v=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var y=r(n(2)),b=r(n(16)),x=r(n(88)),w=r(n(9)),C=n(0),k=n(1),S=n(19),D=r(n(84)),I=r(n(114)),T=n(105),O=n(7),E=n(18),A=r(n(5)),P=n(8),R=r(n(10)),M=r(n(45)),B=a(n(273)),V=r(n(272)),F=r(n(200)),L=r(n(879)),H=r(n(880)),z=r(n(881)),N=r(n(882)),j=r(n(883)),W=r(n(884)),G=r(n(885)),q=r(n(886)),$=r(n(887)),K=r(n(888)),U=r(n(889)),Y=r(n(890)),Z=r(n(891)),X=r(n(150)),Q=r(n(892)),J=r(n(893)),ee="dx-diagram",te="dx-diagram-fullscreen",ne=ee+"-toolbar-wrapper",ie=ee+"-content-wrapper",oe=ee+"-content",ae=ee+"-scroll-view",re=ee+"-floating-toolbar-container",se=ee+"-properties-panel-toolbar-container",le=ee+"-loading-indicator",ue=12,ce="in",de=1,he="disabled",fe="portrait",pe="#ffffff",ge=576,_e=12,me=3,ve=12,ye=4,be=152,xe="dxDiagramEvent",we=(0,P.addNamespace)("fullscreenchange",xe),Ce=(0,P.addNamespace)("msfullscreenchange",xe),ke=(0,P.addNamespace)("webkitfullscreenchange",xe),Se=(0,P.addNamespace)("mozfullscreenchange",xe),De=function(e){function t(){return s(this,t),n.apply(this,arguments)}h(t,e);var n=p(t);return u(t,[{key:"_init",value:function(){this._updateDiagramLockCount=0,this.toggleFullscreenLock=0,this._browserResizeTimer=-1,this._toolbars=[],c(v(t.prototype),"_init",this).call(this),this._initDiagram(),this._createCustomCommand()}},{key:"_initMarkup",value:function(){var e=this;c(v(t.prototype),"_initMarkup",this).call(this),this._toolbars=[],delete this._isMobileScreenSize;var n=!(0,O.hasWindow)();this.$element().addClass(ee),delete this._mainToolbar,this.option("mainToolbar.visible")&&this._renderMainToolbar();var i=(0,y.default)("<div>").addClass(ie).appendTo(this.$element());if(delete this._historyToolbar,delete this._historyToolbarResizeCallback,this._isHistoryToolbarVisible()&&this._renderHistoryToolbar(i),delete this._propertiesToolbar,delete this._propertiesToolbarResizeCallback,this._isPropertiesPanelEnabled()&&this._renderPropertiesToolbar(i),delete this._viewToolbar,delete this._viewToolbarResizeCallback,this.option("viewToolbar.visible")&&this._renderViewToolbar(i),delete this._toolbox,delete this._toolboxResizeCallback,this._isToolboxEnabled()&&this._renderToolbox(i),delete this._propertiesPanel,delete this._propertiesPanelResizeCallback,this._isPropertiesPanelEnabled()&&this._renderPropertiesPanel(i),this._$content=(0,y.default)("<div>").addClass(oe).appendTo(i),delete this._contextMenu,this.option("contextMenu.enabled")&&this._renderContextMenu(i),delete this._contextToolbox,this.option("contextToolbox.enabled")&&this._renderContextToolbox(i),this._renderDialog(i),!n){var o=(0,y.default)("<div>").addClass(ae).appendTo(this._$content);this._createComponent(o,q.default,{onCreateDiagram:function(t){e._diagramInstance.createDocument(t.$parent[0],t.scrollView)}})}(0,O.hasWindow)()&&I.default.add(function(){e._killBrowserResizeTimer(),e._browserResizeTimer=setTimeout(function(){return e._processBrowserResize()},100)}),this._setCustomCommandChecked(X.default.SHOW_PROPERTIES_PANEL_COMMAND_NAME,this._isPropertiesPanelVisible()),this._setCustomCommandChecked(X.default.SHOW_TOOLBOX_COMMAND_NAME,this._isToolboxVisible())}},{key:"_processBrowserResize",value:function(){this._isMobileScreenSize=void 0,this._processDiagramResize(),this._killBrowserResizeTimer()}},{key:"_processDiagramResize",value:function(){this._historyToolbarResizeCallback&&this._historyToolbarResizeCallback.call(this),this._propertiesToolbarResizeCallback&&this._propertiesToolbarResizeCallback.call(this),this._propertiesPanelResizeCallback&&this._propertiesPanelResizeCallback.call(this),this._viewToolbarResizeCallback&&this._viewToolbarResizeCallback.call(this),this._toolboxResizeCallback&&this._toolboxResizeCallback.call(this)}},{key:"_killBrowserResizeTimer",value:function(){this._browserResizeTimer>-1&&clearTimeout(this._browserResizeTimer),this._browserResizeTimer=-1}},{key:"isMobileScreenSize",value:function(){return void 0===this._isMobileScreenSize&&(this._isMobileScreenSize=(0,O.hasWindow)()&&this.$element().outerWidth()<ge),this._isMobileScreenSize}},{key:"_captureFocus",value:function(){this._diagramInstance&&this._diagramInstance.captureFocus()}},{key:"_captureFocusOnTimeout",value:function(){var e=this;this._captureFocusTimeout=setTimeout(function(){e._captureFocus(),delete e._captureFocusTimeout},100)}},{key:"_killCaptureFocusTimeout",value:function(){this._captureFocusTimeout&&(clearTimeout(this._captureFocusTimeout),delete this._captureFocusTimeout)}},{key:"notifyBarCommandExecuted",value:function(){this._captureFocusOnTimeout()}},{key:"_registerToolbar",value:function(e){this._registerBar(e),this._toolbars.push(e)}},{key:"_registerBar",value:function(e){e.bar.onChanged.add(this),this._diagramInstance.registerBar(e.bar)}},{key:"_getExcludeCommands",value:function(){var e=[];return this._isToolboxEnabled()||e.push(X.default.SHOW_TOOLBOX_COMMAND_NAME),this._isPropertiesPanelEnabled()||e.push(X.default.SHOW_PROPERTIES_PANEL_COMMAND_NAME),e}},{key:"_getToolbarBaseOptions",value:function(){var e=this;return{onContentReady:function(t){var n=t.component;return e._registerToolbar(n)},onSubMenuVisibilityChanging:function(t){var n=t.component;return e._diagramInstance.updateBarItemsState(n.bar)},onPointerUp:this._onPanelPointerUp.bind(this),"export":this.option("export"),container:this.$element(),excludeCommands:this._getExcludeCommands(),onInternalCommand:this._onInternalCommand.bind(this),onCustomCommand:this._onCustomCommand.bind(this),isMobileView:this.isMobileScreenSize()}}},{key:"_onInternalCommand",value:function(e){switch(e.command){case X.default.SHOW_TOOLBOX_COMMAND_NAME:this._toolbox&&this._toolbox.toggle();break;case X.default.SHOW_PROPERTIES_PANEL_COMMAND_NAME:this._propertiesPanel&&this._propertiesPanel.toggle()}}},{key:"_onCustomCommand",value:function(e){this._customCommandAction({name:e.name})}},{key:"_renderMainToolbar",value:function(){var e=(0,y.default)("<div>").addClass(ne).appendTo(this.$element());this._mainToolbar=this._createComponent(e,L.default,(0,C.extend)(this._getToolbarBaseOptions(),{commands:this.option("mainToolbar.commands"),skipAdjustSize:!0}))}},{key:"_isHistoryToolbarVisible",value:function(){return this.option("historyToolbar.visible")&&!this.isReadOnlyMode()}},{key:"_renderHistoryToolbar",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=(0,y.default)("<div>").addClass(re).appendTo(e);this._historyToolbar=this._createComponent(i,H.default,(0,C.extend)(this._getToolbarBaseOptions(),{commands:this.option("historyToolbar.commands"),locateInMenu:"never"})),this._updateHistoryToolbarPosition(i,e,n),this._historyToolbarResizeCallback=function(){t._historyToolbar.option("isMobileView",t.isMobileScreenSize())}}},{key:"_updateHistoryToolbarPosition",value:function(e,t,n){n||D.default.setup(e,{my:"left top",at:"left top",of:t,offset:ue+" "+ue})}},{key:"_isToolboxEnabled",value:function(){return"disabled"!==this.option("toolbox.visibility")&&!this.isReadOnlyMode()}},{key:"_isToolboxVisible",value:function(){return"visible"===this.option("toolbox.visibility")||"auto"===this.option("toolbox.visibility")&&!this.isMobileScreenSize()}},{key:"_renderToolbox",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=(0,y.default)("<div>").appendTo(e),o=this._getToolboxBounds(e,n);this._toolbox=this._createComponent(i,K.default,{isMobileView:this.isMobileScreenSize(),isVisible:this._isToolboxVisible(),container:this.$element(),height:o.height,offsetParent:e,offsetX:o.offsetX,offsetY:o.offsetY,showSearch:this.option("toolbox.showSearch"),toolboxGroups:this._getToolboxGroups(),toolboxWidth:this.option("toolbox.width"),onShapeCategoryRendered:function(e){n||t._diagramInstance.createToolbox(e.$element[0],"texts"===e.displayMode,e.shapes||e.category,{
shapeIconSpacing:_e,shapeIconCountInRow:t.option("toolbox.shapeIconsPerRow"),shapeIconAttributes:{"data-toggle":e.dataToggle}})},onFilterChanged:function(e){n||t._diagramInstance.applyToolboxFilter(e.text,e.filteringToolboxes)},onVisibilityChanging:function(e){n||(t._setCustomCommandChecked(X.default.SHOW_TOOLBOX_COMMAND_NAME,e.visible),t._propertiesPanel&&e.visible&&t.isMobileScreenSize()&&t._propertiesPanel.hide(),t._historyToolbar&&e.visible&&t.isMobileScreenSize()&&(t._historyToolbarZIndex=B.create(V.default.baseZIndex()),t._historyToolbar.$element().css("zIndex",t._historyToolbarZIndex),t._historyToolbar.$element().css("boxShadow","none")),t._viewToolbar&&(t._viewToolbar.$element().css("opacity",e.visible&&t.isMobileScreenSize()?"0":"1"),t._viewToolbar.$element().css("pointerEvents",e.visible&&t.isMobileScreenSize()?"none":"")))},onVisibilityChanged:function(e){e.visible||t._textInputStarted||t._captureFocus(),n||t._historyToolbar&&!e.visible&&t.isMobileScreenSize()&&t._historyToolbarZIndex&&(B.remove(t._historyToolbarZIndex),t._historyToolbar.$element().css("zIndex",""),t._historyToolbar.$element().css("boxShadow",""),t._historyToolbarZIndex=void 0)},onPointerUp:this._onPanelPointerUp.bind(this)}),this._toolboxResizeCallback=function(){var i=t._getToolboxBounds(e,n);t._toolbox.option("height",i.height);var o=t._toolbox.option("isMobileView");o!==t.isMobileScreenSize()&&(t._toolbox.option({isMobileView:t.isMobileScreenSize(),isVisible:t._isToolboxVisible()}),t._setCustomCommandChecked(X.default.SHOW_TOOLBOX_COMMAND_NAME,t._isToolboxVisible())),t._toolbox.updateMaxHeight()}}},{key:"_getToolboxBounds",value:function(e,t){var n={offsetX:ue,offsetY:ue,height:t?0:e.height()-2*ue};return this._historyToolbar&&!t&&(n.offsetY+=this._historyToolbar.$element().outerHeight()+ue,n.height-=this._historyToolbar.$element().outerHeight()+ue),this._viewToolbar&&!t&&(n.height-=this._viewToolbar.$element().outerHeight()+this._getViewToolbarYOffset(t)),n}},{key:"_renderViewToolbar",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=(0,y.default)("<div>").addClass(re).appendTo(e);this._viewToolbar=this._createComponent(i,z.default,(0,C.extend)(this._getToolbarBaseOptions(),{commands:this.option("viewToolbar.commands"),locateInMenu:"never"})),this._updateViewToolbarPosition(i,e,n),this._viewToolbarResizeCallback=function(){t._updateViewToolbarPosition(i,e,n)}}},{key:"_getViewToolbarYOffset",value:function(e){if(!e){var t=ue;return this._viewToolbar&&this._propertiesToolbar&&(t+=(this._propertiesToolbar.$element().outerHeight()-this._viewToolbar.$element().outerHeight())/2),t}}},{key:"_updateViewToolbarPosition",value:function(e,t,n){n||D.default.setup(e,{my:"left bottom",at:"left bottom",of:t,offset:ue+" -"+this._getViewToolbarYOffset(n)})}},{key:"_isPropertiesPanelEnabled",value:function(){return"disabled"!==this.option("propertiesPanel.visibility")&&!this.isReadOnlyMode()}},{key:"_isPropertiesPanelVisible",value:function(){return"visible"===this.option("propertiesPanel.visibility")}},{key:"_renderPropertiesToolbar",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=(0,y.default)("<div>").addClass(re).addClass(se).appendTo(e);this._propertiesToolbar=this._createComponent(i,N.default,(0,C.extend)(this._getToolbarBaseOptions(),{buttonStylingMode:"contained",buttonType:"default",locateInMenu:"never"})),this._updatePropertiesToolbarPosition(i,e,n),this._propertiesToolbarResizeCallback=function(){t._updatePropertiesToolbarPosition(i,e,n)}}},{key:"_updatePropertiesToolbarPosition",value:function(e,t,n){n||D.default.setup(e,{my:"right bottom",at:"right bottom",of:t,offset:"-"+ue+" -"+ue})}},{key:"_renderPropertiesPanel",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=(0,y.default)("<div>").appendTo(e),o=ue,a=2*ue+(n?0:this._propertiesToolbar.$element().outerHeight());this._propertiesPanel=this._createComponent(i,U.default,{isMobileView:this.isMobileScreenSize(),isVisible:this._isPropertiesPanelVisible(),container:this.$element(),offsetParent:e,offsetX:o,offsetY:a,propertyTabs:this.option("propertiesPanel.tabs"),onCreateToolbar:function(e){e.toolbar=t._createComponent(e.$parent,F.default,(0,C.extend)(t._getToolbarBaseOptions(),{commands:e.commands,locateInMenu:"never",editorStylingMode:"outlined"}))},onVisibilityChanging:function(e){n||(t._updatePropertiesPanelGroupBars(e.component),t._setCustomCommandChecked(X.default.SHOW_PROPERTIES_PANEL_COMMAND_NAME,e.visible),t._toolbox&&e.visible&&t.isMobileScreenSize()&&t._toolbox.hide())},onVisibilityChanged:function(e){e.visible||t._textInputStarted||t._captureFocus()},onSelectedGroupChanged:function(e){var n=e.component;return t._updatePropertiesPanelGroupBars(n)},onPointerUp:this._onPanelPointerUp.bind(this)}),this._propertiesPanelResizeCallback=function(){var e=t._propertiesPanel.option("isMobileView");e!==t.isMobileScreenSize()&&(t._propertiesPanel.option({isMobileView:t.isMobileScreenSize(),isVisible:t._isPropertiesPanelVisible()}),t._setCustomCommandChecked(X.default.SHOW_PROPERTIES_PANEL_COMMAND_NAME,t._isPropertiesPanelVisible()))}}},{key:"_updatePropertiesPanelGroupBars",value:function(e){var t=this;e.getActiveToolbars().forEach(function(e){t._diagramInstance.updateBarItemsState(e.bar)})}},{key:"_onPanelPointerUp",value:function(){this._captureFocusOnTimeout()}},{key:"_renderContextMenu",value:function(e){var t=this,n=(0,y.default)("<div>").appendTo(e);this._contextMenu=this._createComponent(n,j.default.DiagramContextMenuWrapper,{commands:this.option("contextMenu.commands"),onContentReady:function(e){var n=e.component;return t._registerBar(n)},onVisibilityChanging:function(e){var n=e.component;return t._diagramInstance.updateBarItemsState(n.bar)},onItemClick:function(e){return t._onBeforeCommandExecuted(e.command)},"export":this.option("export"),excludeCommands:this._getExcludeCommands(),onInternalCommand:this._onInternalCommand.bind(this),onCustomCommand:this._onCustomCommand.bind(this)})}},{key:"_renderContextToolbox",value:function(e){var t=this,n=!(0,O.hasWindow)(),i=this.option("contextToolbox.category"),o=this.option("contextToolbox.displayMode"),a=this.option("contextToolbox.shapes"),r=(0,y.default)("<div>").appendTo(e);this._contextToolbox=this._createComponent(r,W.default,{toolboxWidth:this.option("contextToolbox.width"),onShown:function(e){if(!n){var r=(0,y.default)(e.$element),s="texts"===o;if(!a&&!i&&!s){var l=t._getToolboxGroups().filter(function(t){return t.category===e.category})[0];l&&(s="texts"===l.displayMode)}t._diagramInstance.createContextToolbox(r[0],s,a||i||e.category,{shapeIconSpacing:ve,shapeIconCountInRow:t.option("contextToolbox.shapeIconsPerRow")},function(n){e.callback(n),t._captureFocus(),e.hide()})}}})}},{key:"_setCustomCommandChecked",value:function(e,t){this._toolbars.forEach(function(n){n.setCommandChecked(e,t)})}},{key:"_onBeforeCommandExecuted",value:function(e){var t=Z.default.getDialogParameters(e);return t&&this._showDialog(t),!!t}},{key:"_renderDialog",value:function(e){var t=(0,y.default)("<div>").appendTo(e);this._dialogInstance=this._createComponent(t,G.default,{})}},{key:"_showDialog",value:function(e){this._dialogInstance&&(this._dialogInstance.option("onGetContent",e.onGetContent),this._dialogInstance.option("onHidden",function(){this._captureFocus()}.bind(this)),this._dialogInstance.option("command",this._diagramInstance.getCommand(e.command)),this._dialogInstance.option("title",e.title),this._dialogInstance._show())}},{key:"_showLoadingIndicator",value:function(){this._loadingIndicator=(0,y.default)("<div>").addClass(le),this._createComponent(this._loadingIndicator,x.default,{});var e=this._$content||this.$element();e.append(this._loadingIndicator)}},{key:"_hideLoadingIndicator",value:function(){this._loadingIndicator&&(this._loadingIndicator.remove(),this._loadingIndicator=null)}},{key:"_initDiagram",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramControl;if(this._diagramInstance=new t,this._diagramInstance.onChanged=this._raiseDataChangeAction.bind(this),this._diagramInstance.onEdgeInserted=this._raiseEdgeInsertedAction.bind(this),this._diagramInstance.onEdgeUpdated=this._raiseEdgeUpdatedAction.bind(this),this._diagramInstance.onEdgeRemoved=this._raiseEdgeRemovedAction.bind(this),this._diagramInstance.onNodeInserted=this._raiseNodeInsertedAction.bind(this),this._diagramInstance.onNodeUpdated=this._raiseNodeUpdatedAction.bind(this),this._diagramInstance.onNodeRemoved=this._raiseNodeRemovedAction.bind(this),this._diagramInstance.onToolboxDragStart=this._raiseToolboxDragStart.bind(this),this._diagramInstance.onToolboxDragEnd=this._raiseToolboxDragEnd.bind(this),this._diagramInstance.onTextInputStart=this._raiseTextInputStart.bind(this),this._diagramInstance.onTextInputEnd=this._raiseTextInputEnd.bind(this),this._diagramInstance.onToggleFullscreen=this._onToggleFullScreen.bind(this),this._diagramInstance.onShowContextMenu=this._onShowContextMenu.bind(this),this._diagramInstance.onHideContextMenu=this._onHideContextMenu.bind(this),this._diagramInstance.onShowContextToolbox=this._onShowContextToolbox.bind(this),this._diagramInstance.onHideContextToolbox=this._onHideContextToolbox.bind(this),this._diagramInstance.onNativeAction.add({notifyItemClick:this._raiseItemClickAction.bind(this),notifyItemDblClick:this._raiseItemDblClickAction.bind(this),notifySelectionChanged:this._raiseSelectionChanged.bind(this)}),this._diagramInstance.onRequestOperation=this._raiseRequestEditOperation.bind(this),this._updateEventSubscriptionMethods(),this._updateDefaultItemProperties(),this._updateEditingSettings(),this._updateShapeTexts(),this._updateUnitItems(),this._updateFormatUnitsMethod(),this.option("units")!==ce&&this._updateUnitsState(),this.isReadOnlyMode()&&this._updateReadOnlyState(),this.option("pageSize")&&(this.option("pageSize.items")&&this._updatePageSizeItemsState(),this.option("pageSize.width")&&this.option("pageSize.height")&&this._updatePageSizeState()),this.option("pageOrientation")!==fe&&this._updatePageOrientationState(),this.option("pageColor")!==pe&&this._updatePageColorState(),this.option("viewUnits")!==ce&&this._updateViewUnitsState(),this.option("showGrid")||this._updateShowGridState(),this.option("snapToGrid")||this._updateSnapToGridState(),this.option("gridSize")&&(this.option("gridSize.items")&&this._updateGridSizeItemsState(),this._updateGridSizeState()),this.option("zoomLevel.items")&&this._updateZoomLevelItemsState(),this.option("simpleView")&&this._updateSimpleViewState(),this.option("zoomLevel")!==de&&this._updateZoomLevelState(),this.option("autoZoomMode")!==he&&this._updateAutoZoomState(),this.option("fullScreen")){var n=(0,O.getWindow)();n&&n.self!==n.top?this.option("fullScreen",!1):this._updateFullscreenState()}this.optionsUpdateBar=new Y.default(this),this._diagramInstance.registerBar(this.optionsUpdateBar),(0,O.hasWindow)()&&this._diagramInstance.initMeasurer(this.$element()[0]),this._updateCustomShapes(this._getCustomShapes()),this._refreshDataSources()}},{key:"_clean",value:function(){this._diagramInstance&&this._diagramInstance.cleanMarkup(function(e){(0,y.default)(e).empty()}),c(v(t.prototype),"_clean",this).call(this)}},{key:"_dispose",value:function(){this._killCaptureFocusTimeout(),c(v(t.prototype),"_dispose",this).call(this),this._diagramInstance=void 0}},{key:"_executeDiagramCommand",value:function(e,t){this._diagramInstance.getCommand(e).execute(t)}},{key:"getNodeDataSource",value:function(){return this._nodesOption&&this._nodesOption.getDataSource()}},{key:"getEdgeDataSource",value:function(){return this._edgesOption&&this._edgesOption.getDataSource()}},{key:"_refreshDataSources",value:function(){this._beginUpdateDiagram(),this._refreshNodesDataSource(),this._refreshEdgesDataSource(),this._endUpdateDiagram()}},{key:"_refreshNodesDataSource",value:function(){this._nodesOption&&(this._nodesOption._disposeDataSource(),delete this._nodesOption),this.option("nodes.dataSource")&&(this._nodesOption=new Q.default(this),this._nodesOption.option("dataSource",this.option("nodes.dataSource")),this._nodesOption._refreshDataSource())}},{key:"_refreshEdgesDataSource",value:function(){this._edgesOption&&(this._edgesOption._disposeDataSource(),delete this._edgesOption),this.option("edges.dataSource")&&(this._edgesOption=new J.default(this),this._edgesOption.option("dataSource",this.option("edges.dataSource")),this._edgesOption._refreshDataSource())}},{key:"_getDiagramData",value:function(){var e,t=(0,T.getDiagram)(),n=t.DiagramCommand;return this._executeDiagramCommand(n.Export,function(t){e=t}),e}},{key:"_setDiagramData",value:function(e,t){var n=(0,T.getDiagram)(),i=n.DiagramCommand;this._executeDiagramCommand(i.Import,{data:e,keepExistingItems:t})}},{key:"isReadOnlyMode",value:function(){return this.option("readOnly")||this.option("disabled")}},{key:"_onDataSourceChanged",value:function(){this._bindDiagramData()}},{key:"_getChangesKeys",value:function(e){return e.map(function(e){return e.internalKey||e.key}).filter(function(e){return!!e})}},{key:"_createOptionGetter",value:function(e){var t=this.option(e);return t&&(0,S.compileGetter)(t)}},{key:"_onRequestUpdateLayout",value:function(e){this._requestLayoutUpdateAction||this._createRequestLayoutUpdateAction();var t={changes:e,allowed:!1};return this._requestLayoutUpdateAction(t),t.allowed}},{key:"_createOptionSetter",value:function(e){var t=this.option(e);return(0,k.isFunction)(t)?t:t&&(0,S.compileSetter)(t)}},{key:"_bindDiagramData",value:function(){if(!this._updateDiagramLockCount&&this._isBindingMode()){var e,t,n,i,o,a,r,s,l=(0,T.getDiagram)(),u=l.DiagramCommand,c=l.ConnectorLineOption,d=l.ConnectorLineEnding,h={nodeDataSource:this._nodesOption&&this._nodesOption.getItems(),edgeDataSource:this._edgesOption&&this._edgesOption.getItems(),nodeDataImporter:{getKey:this._createOptionGetter("nodes.keyExpr"),setKey:this._createOptionSetter("nodes.keyExpr"),getCustomData:this._createOptionGetter("nodes.customDataExpr"),setCustomData:this._createOptionSetter("nodes.customDataExpr"),getLocked:this._createOptionGetter("nodes.lockedExpr"),setLocked:this._createOptionSetter("nodes.lockedExpr"),getStyle:this._createOptionGetter("nodes.styleExpr"),setStyle:this._createOptionSetter("nodes.styleExpr"),getStyleText:this._createOptionGetter("nodes.textStyleExpr"),setStyleText:this._createOptionSetter("nodes.textStyleExpr"),getZIndex:this._createOptionGetter("nodes.zIndexExpr"),setZIndex:this._createOptionSetter("nodes.zIndexExpr"),getType:this._createOptionGetter("nodes.typeExpr"),setType:this._createOptionSetter("nodes.typeExpr"),getText:this._createOptionGetter("nodes.textExpr"),setText:this._createOptionSetter("nodes.textExpr"),getImage:this._createOptionGetter("nodes.imageUrlExpr"),setImage:this._createOptionSetter("nodes.imageUrlExpr"),getLeft:this._createOptionGetter("nodes.leftExpr"),setLeft:this._createOptionSetter("nodes.leftExpr"),getTop:this._createOptionGetter("nodes.topExpr"),setTop:this._createOptionSetter("nodes.topExpr"),getWidth:this._createOptionGetter("nodes.widthExpr"),setWidth:this._createOptionSetter("nodes.widthExpr"),getHeight:this._createOptionGetter("nodes.heightExpr"),setHeight:this._createOptionSetter("nodes.heightExpr"),getParentKey:this._createOptionGetter("nodes.parentKeyExpr"),setParentKey:this._createOptionSetter("nodes.parentKeyExpr"),getItems:this._createOptionGetter("nodes.itemsExpr"),setItems:this._createOptionSetter("nodes.itemsExpr"),getContainerKey:r=this._createOptionGetter("nodes.containerKeyExpr"),setContainerKey:s=this._createOptionSetter("nodes.containerKeyExpr"),getChildren:!r&&!s&&this._createOptionGetter("nodes.containerChildrenExpr"),setChildren:!r&&!s&&this._createOptionSetter("nodes.containerChildrenExpr")},edgeDataImporter:{getKey:this._createOptionGetter("edges.keyExpr"),setKey:this._createOptionSetter("edges.keyExpr"),getCustomData:this._createOptionGetter("edges.customDataExpr"),setCustomData:this._createOptionSetter("edges.customDataExpr"),getLocked:this._createOptionGetter("edges.lockedExpr"),setLocked:this._createOptionSetter("edges.lockedExpr"),getStyle:this._createOptionGetter("edges.styleExpr"),setStyle:this._createOptionSetter("edges.styleExpr"),getStyleText:this._createOptionGetter("edges.textStyleExpr"),setStyleText:this._createOptionSetter("edges.textStyleExpr"),getZIndex:this._createOptionGetter("edges.zIndexExpr"),setZIndex:this._createOptionSetter("edges.zIndexExpr"),getFrom:this._createOptionGetter("edges.fromExpr"),setFrom:this._createOptionSetter("edges.fromExpr"),getFromPointIndex:this._createOptionGetter("edges.fromPointIndexExpr"),setFromPointIndex:this._createOptionSetter("edges.fromPointIndexExpr"),getTo:this._createOptionGetter("edges.toExpr"),setTo:this._createOptionSetter("edges.toExpr"),getToPointIndex:this._createOptionGetter("edges.toPointIndexExpr"),setToPointIndex:this._createOptionSetter("edges.toPointIndexExpr"),getPoints:this._createOptionGetter("edges.pointsExpr"),setPoints:this._createOptionSetter("edges.pointsExpr"),getText:this._createOptionGetter("edges.textExpr"),setText:this._createOptionSetter("edges.textExpr"),getLineOption:(e=this._createOptionGetter("edges.lineTypeExpr"))&&function(t){var n=e(t);return this._getConnectorLineOption(n)}.bind(this),setLineOption:(t=this._createOptionSetter("edges.lineTypeExpr"))&&function(e,n){switch(n){case c.Straight:n="straight";break;case c.Orthogonal:n="orthogonal"}t(e,n)}.bind(this),getStartLineEnding:(n=this._createOptionGetter("edges.fromLineEndExpr"))&&function(e){var t=n(e);return this._getConnectorLineEnding(t)}.bind(this),setStartLineEnding:(i=this._createOptionSetter("edges.fromLineEndExpr"))&&function(e,t){switch(t){case d.Arrow:t="arrow";break;case d.OutlinedTriangle:t="outlinedTriangle";break;case d.FilledTriangle:t="filledTriangle";break;case d.None:t="none"}i(e,t)}.bind(this),getEndLineEnding:(o=this._createOptionGetter("edges.toLineEndExpr"))&&function(e){var t=o(e);return this._getConnectorLineEnding(t)}.bind(this),setEndLineEnding:(a=this._createOptionSetter("edges.toLineEndExpr"))&&function(e,t){switch(t){case d.Arrow:t="arrow";break;case d.OutlinedTriangle:t="outlinedTriangle";break;case d.FilledTriangle:t="filledTriangle";break;case d.None:t="none"}a(e,t)}.bind(this)},layoutParameters:this._getDataBindingLayoutParameters()};this._executeDiagramCommand(u.BindDocument,h)}}},{key:"_reloadContentByChanges",value:function(e,t){var n=this._getChangesKeys(e),i=this._onRequestUpdateLayout(e);this._reloadContent(n,i,t)}},{key:"_reloadContent",value:function(e,t,n){var i=this,o=function(){var e,t;return i._nodesOption&&n&&(e=i._nodesOption.getItems()),i._edgesOption&&n&&(t=i._edgesOption.getItems()),{nodeDataSource:e,edgeDataSource:t}};this._diagramInstance.reloadContent(e,o,t&&this._getDataBindingLayoutParameters(),n)}},{key:"_getConnectorLineOption",value:function(e){var t=(0,T.getDiagram)(),n=t.ConnectorLineOption;switch(e){case"straight":return n.Straight;default:return n.Orthogonal}}},{key:"_getConnectorLineEnding",value:function(e){var t=(0,T.getDiagram)(),n=t.ConnectorLineEnding;switch(e){case"arrow":return n.Arrow;case"outlinedTriangle":return n.OutlinedTriangle;case"filledTriangle":return n.FilledTriangle;default:return n.None}}},{key:"_getDataBindingLayoutParameters",value:function(){var e=(0,T.getDiagram)(),t=e.DataLayoutType,n=e.DataLayoutOrientation,i=this.option("nodes.autoLayout")||"off",o=i.type||i,a={};if("off"!==o&&("auto"!==o||!this._hasNodePositionExprs())){switch(o){case"tree":a.type=t.Tree;break;default:a.type=t.Sugiyama}switch(i.orientation){case"vertical":a.orientation=n.Vertical;break;case"horizontal":a.orientation=n.Horizontal}(this.option("edges.fromPointIndexExpr")||this.option("edges.toPointIndexExpr"))&&(a.skipPointIndices=!0)}return a.autoSizeEnabled=!!this.option("nodes.autoSizeEnabled"),a}},{key:"_hasNodePositionExprs",value:function(){return this.option("nodes.topExpr")&&this.option("nodes.leftExpr")}},{key:"_getAutoZoomValue",value:function(e){var t=(0,T.getDiagram)(),n=t.AutoZoomMode;switch(e){case"fitContent":return n.FitContent;case"fitWidth":return n.FitToWidth;default:return n.Disabled}}},{key:"_isBindingMode",value:function(){return this._nodesOption&&this._nodesOption.hasItems()||this._edgesOption&&this._nodesOption.hasItems()}},{key:"_beginUpdateDiagram",value:function(){this._updateDiagramLockCount++}},{key:"_endUpdateDiagram",value:function(){this._updateDiagramLockCount=Math.max(this._updateDiagramLockCount-1,0),this._updateDiagramLockCount||this._bindDiagramData()}},{key:"_getCustomShapes",value:function(){return this.option("customShapes")||[]}},{key:"_getToolboxGroups",value:function(){return $.default.getGroups(this.option("toolbox.groups"))}},{key:"_updateCustomShapes",value:function(e,t){var n=this;Array.isArray(t)&&this._diagramInstance.removeCustomShapes(t.map(function(e){return e.type})),Array.isArray(e)&&this._diagramInstance.addCustomShapes(e.map(function(e){var t=e.template||n.option("customShapeTemplate"),i=t&&n._getTemplate(t),o=e.toolboxTemplate||n.option("customShapeToolboxTemplate"),a=o&&n._getTemplate(o);return{category:e.category,type:e.type,baseType:e.baseType,title:e.title,svgUrl:e.backgroundImageUrl,svgToolboxUrl:e.backgroundImageToolboxUrl,svgLeft:e.backgroundImageLeft,svgTop:e.backgroundImageTop,svgWidth:e.backgroundImageWidth,svgHeight:e.backgroundImageHeight,defaultWidth:e.defaultWidth,defaultHeight:e.defaultHeight,toolboxWidthToHeightRatio:e.toolboxWidthToHeightRatio,minWidth:e.minWidth,minHeight:e.minHeight,maxWidth:e.maxWidth,maxHeight:e.maxHeight,allowResize:e.allowResize,defaultText:e.defaultText,allowEditText:e.allowEditText,textLeft:e.textLeft,textTop:e.textTop,textWidth:e.textWidth,textHeight:e.textHeight,defaultImageUrl:e.defaultImageUrl,allowEditImage:e.allowEditImage,imageLeft:e.imageLeft,imageTop:e.imageTop,imageWidth:e.imageWidth,imageHeight:e.imageHeight,connectionPoints:e.connectionPoints&&e.connectionPoints.map(function(e){return{x:e.x,y:e.y}}),createTemplate:i&&function(e,t){i.render({model:n._nativeItemToDiagramItem(t),container:(0,E.getPublicElement)((0,y.default)(e))})},createToolboxTemplate:a&&function(e,t){a.render({model:n._nativeItemToDiagramItem(t),container:(0,E.getPublicElement)((0,y.default)(e))})},destroyTemplate:i&&function(e){(0,y.default)(e).empty()},templateLeft:e.templateLeft,templateTop:e.templateTop,templateWidth:e.templateWidth,templateHeight:e.templateHeight,keepRatioOnAutoSize:e.keepRatioOnAutoSize}}))}},{key:"_onToggleFullScreen",value:function(e){this.toggleFullscreenLock>0||(this._changeNativeFullscreen(e),this.$element().toggleClass(te,e),this._diagramInstance.updateLayout(!0),this._processDiagramResize(),this._toolbox&&this._toolbox.repaint(),this._propertiesPanel&&this._propertiesPanel.repaint())}},{key:"_changeNativeFullscreen",value:function(e){var t=(0,O.getWindow)();t.self!==t.top&&e!==this._inNativeFullscreen()&&(e?this._subscribeFullscreenNativeChanged():this._unsubscribeFullscreenNativeChanged(),this._setNativeFullscreen(e))}},{key:"_setNativeFullscreen",value:function(e){var t=(0,O.getWindow)(),n=t.self.document,i=t.self.document.body;e?i.requestFullscreen?i.requestFullscreen():i.mozRequestFullscreen?i.mozRequestFullscreen():i.webkitRequestFullscreen?i.webkitRequestFullscreen():i.msRequestFullscreen&&i.msRequestFullscreen():n.exitFullscreen?n.exitFullscreen():n.mozCancelFullscreen?n.mozCancelFullscreen():n.webkitExitFullscreen?n.webkitExitFullscreen():n.msExitFullscreen&&n.msExitFullscreen()}},{key:"_inNativeFullscreen",value:function(){var e=(0,O.getWindow)().document,t=e.fullscreenElement||e.msFullscreenElement||e.webkitFullscreenElement,n=t===e.body||e.webkitIsFullscreen;return!!n}},{key:"_subscribeFullscreenNativeChanged",value:function(){var e=(0,O.getWindow)().document,t=this._onNativeFullscreenChangeHandler.bind(this);A.default.on(e,we,t),A.default.on(e,Ce,t),A.default.on(e,ke,t),A.default.on(e,Se,t)}},{key:"_unsubscribeFullscreenNativeChanged",value:function(){var e=(0,O.getWindow)().document;A.default.off(e,we),A.default.off(e,Ce),A.default.off(e,ke),A.default.off(e,Se)}},{key:"_onNativeFullscreenChangeHandler",value:function(){this._inNativeFullscreen()||(this._unsubscribeFullscreenNativeChanged(),this.option("fullScreen",!1))}},{key:"_executeDiagramFullscreenCommand",value:function(e){var t=(0,T.getDiagram)(),n=t.DiagramCommand;this.toggleFullscreenLock++,this._executeDiagramCommand(n.Fullscreen,e),this.toggleFullscreenLock--}},{key:"_onShowContextMenu",value:function(e,t,n){this._contextMenu&&this._contextMenu._show(e,t,n)}},{key:"_onHideContextMenu",value:function(){this._contextMenu&&this._contextMenu._hide()}},{key:"_onShowContextToolbox",value:function(e,t,n,i,o){this._contextToolbox&&this._contextToolbox._show(e,t,n,i,o)}},{key:"_onHideContextToolbox",value:function(){this._contextToolbox&&this._contextToolbox._hide()}},{key:"_getDiagramUnitValue",value:function(e){var t=(0,T.getDiagram)(),n=t.DiagramUnit;switch(e){case"in":return n.In;case"cm":return n.Cm;case"px":return n.Px;default:return n.In}}},{key:"_updateReadOnlyState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand,n=this.isReadOnlyMode();this._executeDiagramCommand(t.ToggleReadOnly,n)}},{key:"_updateZoomLevelState",value:function(){var e=this.option("zoomLevel.value");e||(e=this.option("zoomLevel"));var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.ZoomLevel,e)}},{key:"_updateZoomLevelItemsState",value:function(){var e=this.option("zoomLevel.items");if(Array.isArray(e)){var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.ZoomLevelItems,e)}}},{key:"_updateAutoZoomState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.SwitchAutoZoom,this._getAutoZoomValue(this.option("autoZoomMode")))}},{key:"_updateSimpleViewState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.ToggleSimpleView,this.option("simpleView"))}},{key:"_updateFullscreenState",value:function(){var e=this.option("fullScreen");this._executeDiagramFullscreenCommand(e),this._onToggleFullScreen(e)}},{key:"_updateShowGridState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.ShowGrid,this.option("showGrid"))}},{key:"_updateSnapToGridState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.SnapToGrid,this.option("snapToGrid"))}},{key:"_updateGridSizeState",value:function(){var e=this.option("gridSize.value");e||(e=this.option("gridSize"));var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.GridSize,e)}},{key:"_updateGridSizeItemsState",value:function(){var e=this.option("gridSize.items");if(Array.isArray(e)){var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.GridSizeItems,e)}}},{key:"_updateUnitItems",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramLocalizationService,n=this._getUnitItems();this._unitItems!==n&&(this._unitItems=n,t.unitItems=n)}},{key:"_getUnitItems",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramUnit,n={};return n[t.In]=R.default.format("dxDiagram-unitIn"),n[t.Cm]=R.default.format("dxDiagram-unitCm"),n[t.Px]=R.default.format("dxDiagram-unitPx"),n}},{key:"_updateFormatUnitsMethod",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramLocalizationService;t.formatUnit=function(e){return M.default.format(e)}}},{key:"_updateViewUnitsState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.ViewUnits,this._getDiagramUnitValue(this.option("viewUnits")))}},{key:"_updateUnitsState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.Units,this._getDiagramUnitValue(this.option("units")))}},{key:"_updatePageSizeState",value:function(){var e=this.option("pageSize");if(e&&e.width&&e.height){var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.PageSize,e)}}},{key:"_updatePageSizeItemsState",value:function(){var e=this.option("pageSize.items");if(Array.isArray(e)){var t=(0,T.getDiagram)(),n=t.DiagramCommand;this._executeDiagramCommand(n.PageSizeItems,e)}}},{key:"_updatePageOrientationState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.PageLandscape,"landscape"===this.option("pageOrientation"))}},{key:"_updatePageColorState",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramCommand;this._executeDiagramCommand(t.PageColor,this.option("pageColor"))}},{key:"_updateShapeTexts",value:function(){var e=(0,T.getDiagram)(),t=e.DiagramLocalizationService,n=this._getShapeTexts();this._shapeTexts!==n&&(this._shapeTexts=n,t.shapeTexts=n)}},{key:"_getShapeTexts",value:function(){var e=(0,T.getDiagram)(),t=e.ShapeTypes,n={};return n[t.Text]=R.default.format("dxDiagram-shapeText"),n[t.Rectangle]=R.default.format("dxDiagram-shapeRectangle"),n[t.Ellipse]=R.default.format("dxDiagram-shapeEllipse"),n[t.Cross]=R.default.format("dxDiagram-shapeCross"),n[t.Triangle]=R.default.format("dxDiagram-shapeTriangle"),n[t.Diamond]=R.default.format("dxDiagram-shapeDiamond"),n[t.Heart]=R.default.format("dxDiagram-shapeHeart"),n[t.Pentagon]=R.default.format("dxDiagram-shapePentagon"),n[t.Hexagon]=R.default.format("dxDiagram-shapeHexagon"),n[t.Octagon]=R.default.format("dxDiagram-shapeOctagon"),n[t.Star]=R.default.format("dxDiagram-shapeStar"),n[t.ArrowLeft]=R.default.format("dxDiagram-shapeArrowLeft"),n[t.ArrowUp]=R.default.format("dxDiagram-shapeArrowUp"),n[t.ArrowRight]=R.default.format("dxDiagram-shapeArrowRight"),n[t.ArrowDown]=R.default.format("dxDiagram-shapeArrowDown"),n[t.ArrowUpDown]=R.default.format("dxDiagram-shapeArrowUpDown"),n[t.ArrowLeftRight]=R.default.format("dxDiagram-shapeArrowLeftRight"),n[t.Process]=R.default.format("dxDiagram-shapeProcess"),n[t.Decision]=R.default.format("dxDiagram-shapeDecision"),n[t.Terminator]=R.default.format("dxDiagram-shapeTerminator"),n[t.PredefinedProcess]=R.default.format("dxDiagram-shapePredefinedProcess"),n[t.Document]=R.default.format("dxDiagram-shapeDocument"),n[t.MultipleDocuments]=R.default.format("dxDiagram-shapeMultipleDocuments"),n[t.ManualInput]=R.default.format("dxDiagram-shapeManualInput"),n[t.Preparation]=R.default.format("dxDiagram-shapePreparation"),n[t.Data]=R.default.format("dxDiagram-shapeData"),n[t.Database]=R.default.format("dxDiagram-shapeDatabase"),n[t.HardDisk]=R.default.format("dxDiagram-shapeHardDisk"),n[t.InternalStorage]=R.default.format("dxDiagram-shapeInternalStorage"),n[t.PaperTape]=R.default.format("dxDiagram-shapePaperTape"),n[t.ManualOperation]=R.default.format("dxDiagram-shapeManualOperation"),n[t.Delay]=R.default.format("dxDiagram-shapeDelay"),n[t.StoredData]=R.default.format("dxDiagram-shapeStoredData"),n[t.Display]=R.default.format("dxDiagram-shapeDisplay"),n[t.Merge]=R.default.format("dxDiagram-shapeMerge"),n[t.Connector]=R.default.format("dxDiagram-shapeConnector"),n[t.Or]=R.default.format("dxDiagram-shapeOr"),n[t.SummingJunction]=R.default.format("dxDiagram-shapeSummingJunction"),n[t.Container]=R.default.format("dxDiagram-shapeContainerDefaultText"),n[t.VerticalContainer]=R.default.format("dxDiagram-shapeVerticalContainer"),n[t.HorizontalContainer]=R.default.format("dxDiagram-shapeHorizontalContainer"),n[t.Card]=R.default.format("dxDiagram-shapeCardDefaultText"),n[t.CardWithImageOnLeft]=R.default.format("dxDiagram-shapeCardWithImageOnLeft"),n[t.CardWithImageOnTop]=R.default.format("dxDiagram-shapeCardWithImageOnTop"),n[t.CardWithImageOnRight]=R.default.format("dxDiagram-shapeCardWithImageOnRight"),n}},{key:"_updateEventSubscriptionMethods",value:function(){var e=(0,T.getDiagram)(),t=e.RenderHelper;t.addEventListener=function(e,t,n){A.default.on(e,t,n)},t.removeEventListener=function(e,t,n){A.default.off(e,t,n)}}},{key:"_updateDefaultItemProperties",value:function(){this.option("defaultItemProperties.style")&&this._diagramInstance.setInitialStyleProperties(this.option("defaultItemProperties.style")),this.option("defaultItemProperties.textStyle")&&this._diagramInstance.setInitialTextStyleProperties(this.option("defaultItemProperties.textStyle")),
this._diagramInstance.setInitialConnectorProperties({lineOption:this._getConnectorLineOption(this.option("defaultItemProperties.connectorLineType")),startLineEnding:this._getConnectorLineEnding(this.option("defaultItemProperties.connectorLineStart")),endLineEnding:this._getConnectorLineEnding(this.option("defaultItemProperties.connectorLineEnd"))}),this._diagramInstance.applyShapeSizeSettings({shapeMinWidth:this.option("defaultItemProperties.shapeMinWidth"),shapeMaxWidth:this.option("defaultItemProperties.shapeMaxWidth"),shapeMinHeight:this.option("defaultItemProperties.shapeMinHeight"),shapeMaxHeight:this.option("defaultItemProperties.shapeMaxHeight")})}},{key:"_updateEditingSettings",value:function(){this._diagramInstance.applyOperationSettings({addShape:this.option("editing.allowAddShape"),addShapeFromToolbox:this.option("editing.allowAddShape"),deleteShape:this.option("editing.allowDeleteShape"),deleteConnector:this.option("editing.allowDeleteConnector"),changeConnection:this.option("editing.allowChangeConnection"),changeConnectorPoints:this.option("editing.allowChangeConnectorPoints"),changeShapeText:this.option("editing.allowChangeShapeText"),changeConnectorText:this.option("editing.allowChangeConnectorText"),resizeShape:this.option("editing.allowResizeShape"),moveShape:this.option("editing.allowMoveShape")})}},{key:"focus",value:function(){this._captureFocus()}},{key:"export",value:function(){return this._getDiagramData()}},{key:"exportTo",value:function(e,t){var n=this._getDiagramExportToCommand(e);this._executeDiagramCommand(n,t)}},{key:"_getDiagramExportToCommand",value:function(e){var t=(0,T.getDiagram)(),n=t.DiagramCommand;switch(e){case"png":return n.ExportPng;case"jpg":return n.ExportJpg;default:return n.ExportSvg}}},{key:"import",value:function(e,t){this._setDiagramData(e,t),this._raiseDataChangeAction()}},{key:"updateToolbox",value:function(){this._diagramInstance&&this._diagramInstance.refreshToolbox(),this._toolbox&&this._toolbox.updateMaxHeight()}},{key:"_getDefaultOptions",value:function(){return(0,C.extend)(c(v(t.prototype),"_getDefaultOptions",this).call(this),{readOnly:!1,zoomLevel:de,simpleView:!1,autoZoomMode:he,fullScreen:!1,showGrid:!0,snapToGrid:!0,units:ce,viewUnits:ce,pageOrientation:fe,pageColor:pe,hasChanges:!1,nodes:{dataSource:null,keyExpr:"id",customDataExpr:void 0,lockedExpr:void 0,styleExpr:void 0,textStyleExpr:void 0,zIndexExpr:void 0,typeExpr:"type",textExpr:"text",imageUrlExpr:void 0,parentKeyExpr:void 0,itemsExpr:void 0,leftExpr:void 0,topExpr:void 0,widthExpr:void 0,heightExpr:void 0,containerKeyExpr:void 0,containerChildrenExpr:"children",autoLayout:"auto",autoSizeEnabled:!0},edges:{dataSource:null,keyExpr:"id",customDataExpr:void 0,lockedExpr:void 0,styleExpr:void 0,textStyleExpr:void 0,zIndexExpr:void 0,fromExpr:"from",fromPointIndexExpr:void 0,toExpr:"to",toPointIndexExpr:void 0,pointsExpr:void 0,textExpr:void 0,lineTypeExpr:void 0,fromLineEndExpr:void 0,toLineEndExpr:void 0},customShapes:[],toolbox:{visibility:"auto",shapeIconsPerRow:me,showSearch:!0},mainToolbar:{visible:!1},historyToolbar:{visible:!0},viewToolbar:{visible:!0},contextMenu:{enabled:!0},contextToolbox:{enabled:!0,shapeIconsPerRow:ye,width:be},propertiesPanel:{visibility:"auto"},defaultItemProperties:{connectorLineType:"orthogonal",connectorLineStart:"none",connectorLineEnd:"arrow"},editing:{allowAddShape:!0,allowDeleteShape:!0,allowDeleteConnector:!0,allowChangeConnection:!0,allowChangeConnectorPoints:!0,allowChangeShapeText:!0,allowChangeConnectorText:!0,allowResizeShape:!0,allowMoveShape:!0},"export":{fileName:"Diagram",proxyUrl:void 0},onItemClick:null,onItemDblClick:null,onSelectionChanged:null,onRequestEditOperation:null,onRequestLayoutUpdate:null})}},{key:"_raiseDataChangeAction",value:function(){this._initialized&&this.option("hasChanges",!0)}},{key:"_raiseEdgeInsertedAction",value:function(e,t,n){this._edgesOption&&this._edgesOption.insert(e,t,n)}},{key:"_raiseEdgeUpdatedAction",value:function(e,t,n,i){this._edgesOption&&this._edgesOption.update(e,t,n,i)}},{key:"_raiseEdgeRemovedAction",value:function(e,t,n,i){this._edgesOption&&this._edgesOption.remove(e,t,n,i)}},{key:"_raiseNodeInsertedAction",value:function(e,t,n){this._nodesOption&&this._nodesOption.insert(e,t,n)}},{key:"_raiseNodeUpdatedAction",value:function(e,t,n,i){this._nodesOption&&this._nodesOption.update(e,t,n,i)}},{key:"_raiseNodeRemovedAction",value:function(e,t,n,i){this._nodesOption&&this._nodesOption.remove(e,t,n,i)}},{key:"_raiseToolboxDragStart",value:function(){this._toolbox&&(this._toolbox._raiseToolboxDragStart(),this.isMobileScreenSize()&&(this._toolbox.hide(),this._toolboxDragHidden=!0))}},{key:"_raiseToolboxDragEnd",value:function(){this._toolbox&&(this._toolbox._raiseToolboxDragEnd(),this._toolboxDragHidden&&(this._toolbox.show(),delete this._toolboxDragHidden))}},{key:"_raiseTextInputStart",value:function(){this._textInputStarted=!0,this._propertiesPanel&&this.isMobileScreenSize()&&this._propertiesPanel.isVisible()&&(this._propertiesPanel.hide(),this._propertiesPanelTextInputHidden=!0),this._toolbox&&this.isMobileScreenSize()&&this._toolbox.isVisible()&&(this._toolbox.hide(),this._toolboxTextInputHidden=!0)}},{key:"_raiseTextInputEnd",value:function(){this._propertiesPanel&&this._propertiesPanelTextInputHidden&&(this._propertiesPanel.show(),delete this._propertiesPanelTextInputHidden),this._toolbox&&this._toolboxTextInputHidden&&(this._toolbox.show(),delete this._toolboxTextInputHidden),this._textInputStarted=!1}},{key:"_createItemClickAction",value:function(){this._itemClickAction=this._createActionByOption("onItemClick")}},{key:"_createItemDblClickAction",value:function(){this._itemDblClickAction=this._createActionByOption("onItemDblClick")}},{key:"_createSelectionChangedAction",value:function(){this._selectionChangedAction=this._createActionByOption("onSelectionChanged")}},{key:"_createRequestEditOperationAction",value:function(){this._requestEditOperationAction=this._createActionByOption("onRequestEditOperation")}},{key:"_createRequestLayoutUpdateAction",value:function(){this._requestLayoutUpdateAction=this._createActionByOption("onRequestLayoutUpdate")}},{key:"_createCustomCommand",value:function(){this._customCommandAction=this._createActionByOption("onCustomCommand")}},{key:"_raiseItemClickAction",value:function(e){this._itemClickAction||this._createItemClickAction(),this._itemClickAction({item:this._nativeItemToDiagramItem(e)})}},{key:"_raiseItemDblClickAction",value:function(e){this._itemDblClickAction||this._createItemDblClickAction(),this._itemDblClickAction({item:this._nativeItemToDiagramItem(e)})}},{key:"_raiseSelectionChanged",value:function(e){this._selectionChangedAction||this._createSelectionChangedAction(),this._selectionChangedAction({items:e.map(this._nativeItemToDiagramItem.bind(this))})}},{key:"_raiseRequestEditOperation",value:function(e,t){this._requestEditOperationAction||this._createRequestEditOperationAction();var n=this._getRequestEditOperationEventArgs(e,t);this._requestEditOperationAction(n),t.allowed=n.allowed}},{key:"_getModelOperation",value:function(e){var t=(0,T.getDiagram)(),n=t.DiagramModelOperation;switch(e){case n.AddShape:return"addShape";case n.AddShapeFromToolbox:return"addShapeFromToolbox";case n.DeleteShape:return"deleteShape";case n.DeleteConnector:return"deleteConnector";case n.ChangeConnection:return"changeConnection";case n.ChangeConnectorPoints:return"changeConnectorPoints";case n.BeforeChangeShapeText:return"beforeChangeShapeText";case n.ChangeShapeText:return"changeShapeText";case n.BeforeChangeConnectorText:return"beforeChangeConnectorText";case n.ChangeConnectorText:return"changeConnectorText";case n.ResizeShape:return"resizeShape";case n.MoveShape:return"moveShape"}}},{key:"_getRequestEditOperationEventArgs",value:function(e,t){var n=(0,T.getDiagram)(),i=n.DiagramModelOperation,o=n.ConnectorPosition,a={operation:this._getModelOperation(e),allowed:t.allowed,updateUI:t.updateUI,reason:t.updateUI?"checkUIElementAvailability":"modelModification"};switch(e){case i.AddShape:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape),position:t.position&&{x:t.position.x,y:t.position.y}};break;case i.AddShapeFromToolbox:a.args={shapeType:t.shapeType};break;case i.DeleteShape:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape)};break;case i.DeleteConnector:a.args={connector:t.connector&&this._nativeItemToDiagramItem(t.connector)};break;case i.ChangeConnection:a.args={newShape:t.shape&&this._nativeItemToDiagramItem(t.shape),oldShape:t.oldShape&&this._nativeItemToDiagramItem(t.oldShape),connector:t.connector&&this._nativeItemToDiagramItem(t.connector),connectionPointIndex:t.connectionPointIndex,connectorPosition:t.position===o.Begin?"start":"end"};break;case i.ChangeConnectorPoints:a.args={connector:t.connector&&this._nativeItemToDiagramItem(t.connector),newPoints:t.points&&t.points.map(function(e){return{x:e.x,y:e.y}}),oldPoints:t.oldPoints&&t.oldPoints.map(function(e){return{x:e.x,y:e.y}})};break;case i.BeforeChangeShapeText:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape)};break;case i.ChangeShapeText:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape),text:t.text};break;case i.BeforeChangeConnectorText:a.args={connector:t.connector&&this._nativeItemToDiagramItem(t.connector),index:t.index};break;case i.ChangeConnectorText:a.args={connector:t.connector&&this._nativeItemToDiagramItem(t.connector),index:t.index,text:t.text};break;case i.ResizeShape:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape),newSize:t.size&&{width:t.size.width,height:t.size.height},oldSize:t.oldSize&&{width:t.oldSize.width,height:t.oldSize.height}};break;case i.MoveShape:a.args={shape:t.shape&&this._nativeItemToDiagramItem(t.shape),newPosition:t.position&&{x:t.position.x,y:t.position.y},oldPosition:t.oldPosition&&{x:t.oldPosition.x,y:t.oldPosition.y}}}return a}},{key:"_nativeItemToDiagramItem",value:function(e){var t=(0,T.getDiagram)(),n=t.NativeShape,i=e instanceof n?this._nativeShapeToDiagramShape.bind(this):this._nativeConnectorToDiagramConnector.bind(this);return(0,C.extend)({id:e.id,key:e.key,dataItem:void 0},i(e))}},{key:"_nativeShapeToDiagramShape",value:function(e){return{dataItem:this._nodesOption&&this._nodesOption.findItem(e.key),itemType:"shape",text:e.text,type:e.type,position:{x:e.position.x,y:e.position.y},size:{width:e.size.width,height:e.size.height},attachedConnectorIds:e.attachedConnectorIds}}},{key:"_nativeConnectorToDiagramConnector",value:function(e){return{dataItem:this._edgesOption&&this._edgesOption.findItem(e.key),itemType:"connector",texts:e.texts,fromKey:e.fromKey,toKey:e.toKey,fromId:e.fromId,fromPointIndex:e.fromPointIndex,toId:e.toId,toPointIndex:e.toPointIndex,points:e.points.map(function(e){return{x:e.x,y:e.y}})}}},{key:"getItemByKey",value:function(e){var t=this._diagramInstance&&this._diagramInstance.getNativeItemByDataKey(e);return t&&this._nativeItemToDiagramItem(t)}},{key:"getItemById",value:function(e){var t=this._diagramInstance&&this._diagramInstance.getNativeItemByKey(e);return t&&this._nativeItemToDiagramItem(t)}},{key:"_invalidateContextMenuCommands",value:function(){this._contextMenu&&this._contextMenu.option({commands:this.option("contextMenu.commands")})}},{key:"_invalidatePropertiesPanelTabs",value:function(){this._propertiesPanel&&this._propertiesPanel.option({propertyTabs:this.option("propertiesPanel.tabs")})}},{key:"_invalidateMainToolbarCommands",value:function(){this._mainToolbar&&this._mainToolbar.option({commands:this.option("mainToolbar.commands")})}},{key:"_invalidateHistoryToolbarCommands",value:function(){this._historyToolbar&&this._historyToolbar.option({commands:this.option("historyToolbar.commands")})}},{key:"_invalidateViewToolbarCommands",value:function(){this._viewToolbar&&this._viewToolbar.option({commands:this.option("viewToolbar.commands")})}},{key:"_invalidateToolboxGroups",value:function(){this._toolbox&&this._toolbox.option({toolboxGroups:this._getToolboxGroups()})}},{key:"_optionChanged",value:function(e){if(!this.optionsUpdateBar.isUpdateLocked()){this.optionsUpdateBar.beginUpdate();try{this._optionChangedCore(e)}finally{this.optionsUpdateBar.endUpdate()}}}},{key:"_optionChangedCore",value:function(e){var n=this;switch(e.name){case"readOnly":case"disabled":this._updateReadOnlyState(),this._invalidate();break;case"zoomLevel":"zoomLevel"!==e.fullName&&"zoomLevel.items"!==e.fullName||this._updateZoomLevelItemsState(),"zoomLevel"!==e.fullName&&"zoomLevel.value"!==e.fullName||this._updateZoomLevelState();break;case"autoZoomMode":this._updateAutoZoomState();break;case"simpleView":this._updateSimpleViewState();break;case"fullScreen":this._updateFullscreenState();break;case"showGrid":this._updateShowGridState();break;case"snapToGrid":this._updateSnapToGridState();break;case"gridSize":"gridSize"!==e.fullName&&"gridSize.items"!==e.fullName||this._updateGridSizeItemsState(),"gridSize"!==e.fullName&&"gridSize.value"!==e.fullName||this._updateGridSizeState();break;case"viewUnits":this._updateViewUnitsState();break;case"units":this._updateUnitsState();break;case"pageSize":"pageSize"!==e.fullName&&"pageSize.items"!==e.fullName||this._updatePageSizeItemsState(),"pageSize"!==e.fullName&&"pageSize.width"!==e.fullName&&"pageSize.height"!==e.fullName||this._updatePageSizeState();break;case"pageOrientation":this._updatePageOrientationState();break;case"pageColor":this._updatePageColorState();break;case"nodes":"nodes.autoLayout"===e.fullName?this._refreshDataSources():this._refreshNodesDataSource();break;case"edges":this._refreshEdgesDataSource();break;case"customShapes":this._updateCustomShapes(e.value,e.previousValue),this._invalidate();break;case"contextMenu":"contextMenu.commands"===e.fullName?this._invalidateContextMenuCommands():this._invalidate();break;case"contextToolbox":this._invalidate();break;case"propertiesPanel":"propertiesPanel.tabs"===e.name?this._invalidatePropertiesPanelTabs():this._invalidate();break;case"toolbox":"toolbox.groups"===e.fullName?this._invalidateToolboxGroups():this._invalidate();break;case"mainToolbar":"mainToolbar.commands"===e.fullName?this._invalidateMainToolbarCommands():this._invalidate();break;case"historyToolbar":"historyToolbar.commands"===e.fullName?this._invalidateHistoryToolbarCommands():this._invalidate();break;case"viewToolbar":"viewToolbar.commands"===e.fullName?this._invalidateViewToolbarCommands():this._invalidate();break;case"onItemClick":this._createItemClickAction();break;case"onItemDblClick":this._createItemDblClickAction();break;case"onSelectionChanged":this._createSelectionChangedAction();break;case"onRequestEditOperation":this._createRequestEditOperationAction();break;case"onRequestLayoutUpdate":this._createRequestLayoutUpdateAction();break;case"onCustomCommand":this._createCustomCommand();break;case"defaultItemProperties":this._updateDefaultItemProperties();break;case"editing":this._updateEditingSettings();break;case"export":this._toolbars.forEach(function(e){e.option("export",n.option("export"))}),this._contextMenu&&this._contextMenu.option("export",this.option("export"));break;case"hasChanges":break;default:c(v(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(b.default);(0,w.default)("dxDiagram",De);var Ie=De;t.default=Ie,e.exports=t.default},function(e,t){e.exports=window.DevExpress.diagram},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(200)),_=o(n(150)),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getCommands",value:function(){return _.default.getMainToolbarCommands(this.option("commands"),this.option("excludeCommands"))}}]),t}(g.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(200)),_=o(n(150)),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getCommands",value:function(){return _.default.getHistoryToolbarCommands(this.option("commands"),this._getExcludeCommands())}},{key:"_getExcludeCommands",value:function(){var e=[].concat(this.option("excludeCommands"));return this.option("isMobileView")||e.push(_.default.SHOW_TOOLBOX_COMMAND_NAME),e}}]),t}(g.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(200)),_=o(n(150)),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getCommands",value:function(){return _.default.getViewToolbarCommands(this.option("commands"),this.option("excludeCommands"))}}]),t}(g.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(200)),_=o(n(150)),m=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getCommands",value:function(){return _.default.getPropertiesToolbarCommands()}}]),t}(g.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(110)),b=o(n(150)),x=o(n(539)),w=o(n(319)),C=n(105),k="dx-diagram-touchbar",S="dx-diagram-touchbar-overlay",D="dx-diagram-touchbar-target",I=800,T=32,O=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._createOnVisibilityChangingAction(),this._createOnInternalCommand(),this._createOnCustomCommand(),this._createOnItemClickAction(),this._tempState=void 0,this._commands=[],this._commandToIndexMap={},this.bar=new A(this)}},{key:"_initMarkup",value:function(){var e=this;l(_(t.prototype),"_initMarkup",this).call(this),this._commands=this._getCommands(),this._commandToIndexMap={},this._fillCommandToIndexMap(this._commands,[]),this._$contextMenuTargetElement=(0,m.default)("<div>").addClass(D).appendTo(this.$element());var n=(0,m.default)("<div>").appendTo(this.$element());this._contextMenuInstance=this._createComponent(n,E,{isTouchBarMode:this._isTouchBarMode(),cssClass:this._isTouchBarMode()?k:x.default.getContextMenuCssClass(),closeOnOutsideClick:!1,showEvent:"",focusStateEnabled:!1,items:this._commands,position:this._isTouchBarMode()?{my:{x:"center",y:"bottom"},at:{x:"center",y:"top"},of:this._$contextMenuTargetElement}:{},itemTemplate:function(e,t,n){x.default.getContextMenuItemTemplate(this,e,t,n)},onItemClick:function(t){var n=t.itemData;return e._onItemClick(n)},onShowing:function(t){e._inOnShowing!==!0&&(e._inOnShowing=!0,e._onVisibilityChangingAction({visible:!0,component:e}),t.component.option("items",t.component.option("items")),delete e._inOnShowing)}})}},{key:"_show",value:function(e,t,n){if(this._contextMenuInstance.hide(),this._isTouchBarMode()){this._$contextMenuTargetElement.show(),n||(n={x:e,y:t,width:0,height:0});var i=n.width>I?0:(I-n.width)/2;this._$contextMenuTargetElement.css({left:n.x-i,top:n.y-T,width:n.width+2*i,height:n.height+2*T}),this._contextMenuInstance.show()}else this._contextMenuInstance.option("position",{offset:e+" "+t}),this._contextMenuInstance.show()}},{key:"_hide",value:function(){this._$contextMenuTargetElement.hide(),this._contextMenuInstance.hide()}},{key:"_isTouchBarMode",value:function(){var e=(0,C.getDiagram)(),t=e.Browser;return t.TouchUI}},{key:"_onItemClick",value:function(e){var t=!1;this._onItemClickAction&&(t=this._onItemClickAction(e)),t||(x.default.onContextMenuItemClick(this,e,this._executeCommand.bind(this)),this._contextMenuInstance.hide())}},{key:"_executeCommand",value:function(e,t,n){"number"==typeof e?this.bar.raiseBarCommandExecuted(e,n):"string"==typeof e&&this._onInternalCommandAction({command:e}),void 0!==t&&this._onCustomCommandAction({name:t})}},{key:"_createOnInternalCommand",value:function(){this._onInternalCommandAction=this._createActionByOption("onInternalCommand")}},{key:"_createOnCustomCommand",value:function(){this._onCustomCommandAction=this._createActionByOption("onCustomCommand")}},{key:"_getCommands",value:function(){return b.default.getContextMenuCommands(this.option("commands"))}},{key:"_fillCommandToIndexMap",value:function(e,t){var n=this;e.forEach(function(e,i){var o=t.concat([i]);void 0!==e.command&&(n._commandToIndexMap[e.command]=o),Array.isArray(e.items)&&n._fillCommandToIndexMap(e.items,o)})}},{key:"_setItemEnabled",value:function(e,t){this._setItemVisible(e,t)}},{key:"_setItemVisible",value:function(e,t){var n=x.default.getItemOptionText(this._contextMenuInstance,this._commandToIndexMap[e]);x.default.updateContextMenuItemVisible(this._contextMenuInstance,n,t)}},{key:"_setItemValue",value:function(e,t){var n=x.default.getItemOptionText(this._contextMenuInstance,this._commandToIndexMap[e]);x.default.updateContextMenuItemValue(this._contextMenuInstance,n,e,t)}},{key:"_setItemSubItems",value:function(e,t){var n=x.default.getItemOptionText(this._contextMenuInstance,this._commandToIndexMap[e]);x.default.updateContextMenuItems(this._contextMenuInstance,n,e,t)}},{key:"_setEnabled",value:function(e){this._contextMenuInstance.option("disabled",!e)}},{key:"isVisible",value:function(){return this._inOnShowing}},{key:"_createOnVisibilityChangingAction",value:function(){this._onVisibilityChangingAction=this._createActionByOption("onVisibilityChanging")}},{key:"_createOnItemClickAction",value:function(){this._onItemClickAction=this._createActionByOption("onItemClick")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onVisibilityChanging":this._createOnVisibilityChangingAction();break;case"onInternalCommand":this._createOnInternalCommand();break;case"onCustomCommand":this._createOnCustomCommand();break;case"onItemClick":this._createOnItemClickAction();break;case"commands":this._invalidate();break;case"export":break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(v.default),E=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_renderContextMenuOverlay",value:function(){var e=this;l(_(t.prototype),"_renderContextMenuOverlay",this).call(this),this._overlay&&this.option("isTouchBarMode")&&this._overlay&&this._overlay.option("onShown",function(){var t=(0,m.default)(e._overlay.$content());t.parent().addClass(S)})}}]),t}(y.default),A=function(e){function t(e){return a(this,t),n.call(this,e)}c(t,e);var n=h(t);return s(t,[{key:"getCommandKeys",value:function(){return this._getKeys(this._owner._commands)}},{key:"setItemValue",value:function(e,t){this._owner._setItemValue(e,t)}},{key:"setItemEnabled",value:function(e,t){this._owner._setItemEnabled(e,t)}},{key:"setItemVisible",value:function(e,t){this._owner._setItemVisible(e,t)}},{key:"setItemSubItems",value:function(e,t){this._owner._setItemSubItems(e,t)}},{key:"setEnabled",value:function(e){this._owner._setEnabled(e)}},{key:"isVisible",value:function(){return this._owner.isVisible()}}]),t}(w.default),P={DiagramContextMenuWrapper:O,DiagramContextMenu:E};t.default=P,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function");
}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(143)),b=n(105),x="dx-diagram-context-toolbox-target",w="dx-diagram-context-toolbox",C="dx-diagram-touch-context-toolbox",k="dx-diagram-context-toolbox-content",S=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._onShownAction=this._createActionByOption("onShown"),this._popoverPositionData=[{my:{x:"center",y:"top"},at:{x:"center",y:"bottom"},offset:{x:0,y:5}},{my:{x:"right",y:"center"},at:{x:"left",y:"center"},offset:{x:-5,y:0}},{my:{x:"center",y:"bottom"},at:{x:"center",y:"top"},offset:{x:0,y:-5}},{my:{x:"left",y:"center"},at:{x:"right",y:"center"},offset:{x:5,y:0}}]}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this._$popoverTargetElement=(0,m.default)("<div>").addClass(x).appendTo(this.$element());var e=(0,m.default)("<div>").appendTo(this.$element()),n=(0,b.getDiagram)(),i=n.Browser,o=w;i.TouchUI&&(o+=" "+C),this._popoverInstance=this._createComponent(e,y.default,{closeOnOutsideClick:!1,container:this.$element(),elementAttr:{"class":o}})}},{key:"_show",value:function(e,t,n,i,o){this._popoverInstance.hide();var a=(0,m.default)("<div>").addClass(k);void 0!==this.option("toolboxWidth")&&a.css("width",this.option("toolboxWidth")),this._$popoverTargetElement.css({left:e+this._popoverPositionData[n].offset.x,top:t+this._popoverPositionData[n].offset.y}).show(),this._popoverInstance.option({position:{my:this._popoverPositionData[n].my,at:this._popoverPositionData[n].at,of:this._$popoverTargetElement},contentTemplate:a,onContentReady:function(){var e=this,t=this.$element().find("."+k);this._onShownAction({category:i,callback:o,$element:t,hide:function(){return e._popoverInstance.hide()}})}.bind(this)}),this._popoverInstance.show()}},{key:"_hide",value:function(){this._$popoverTargetElement.hide(),this._popoverInstance.hide()}}]),t}(v.default),D=S;t.default=D,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(10)),b=o(n(48)),x=n(0),w=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._command=void 0,this._isShown=!1,this._createOnGetContentOption(),this._createOnHiddenOption()}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this._command=this.option("command"),this._$popupElement=(0,m.default)("<div>").appendTo(this.$element()),this._popup=this._createComponent(this._$popupElement,b.default,{title:this.option("title"),maxWidth:this.option("maxWidth"),height:this.option("height"),toolbarItems:this.option("toolbarItems"),onHidden:this._onHiddenAction})}},{key:"_clean",value:function(){delete this._popup,this._$popupElement&&this._$popupElement.remove()}},{key:"_getDefaultOptions",value:function(){return(0,x.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{title:"",maxWidth:500,height:"auto",toolbarItems:this._getToolbarItems()})}},{key:"_getToolbarItems",value:function(){return[this._getOkToolbarItem(),this._getCancelToolbarItem()]}},{key:"_getOkToolbarItem",value:function(){return{widget:"dxButton",location:"after",toolbar:"bottom",options:{text:y.default.format("dxDiagram-dialogButtonOK"),onClick:function(){this._command.execute(this._commandParameter),this._hide()}.bind(this)}}}},{key:"_getCancelToolbarItem",value:function(){return{widget:"dxButton",location:"after",toolbar:"bottom",options:{text:y.default.format("dxDiagram-dialogButtonCancel"),onClick:this._hide.bind(this)}}}},{key:"_optionChanged",value:function(e){switch(e.name){case"title":case"maxWidth":case"height":case"toolbarItems":this._popup.option(e.name,e.value);break;case"command":this._command=e.value;break;case"onGetContent":this._createOnGetContentOption();break;case"onHidden":this._createOnHiddenOption();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"_createOnGetContentOption",value:function(){this._onGetContentAction=this._createActionByOption("onGetContent")}},{key:"_createOnHiddenOption",value:function(){this._onHiddenAction=this._createActionByOption("onHidden")}},{key:"_hide",value:function(){this._popup.hide(),this._isShown=!1}},{key:"_show",value:function(){this._popup.$content().empty().append(this._onGetContentAction()),this._popup.show(),this._isShown=!0}},{key:"isVisible",value:function(){return this._isShown}}]),t}(v.default),C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=o(n(16)),y=o(n(96)),b=n(105),x=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this);var e=(0,b.getDiagram)(),n=e.EventDispatcher;this.onScroll=new n,this._createOnCreateDiagramAction()}},{key:"_initMarkup",value:function(){var e=this;l(_(t.prototype),"_initMarkup",this).call(this);var n=(0,m.default)("<div>").appendTo(this.$element());this._scrollView=this._createComponent(n,y.default,{direction:"both",bounceEnabled:!1,onScroll:function(t){var n=t.scrollOffset;e._raiseOnScroll(n.left,n.top)}}),this._onCreateDiagramAction({$parent:(0,m.default)(this._scrollView.content()),scrollView:this})}},{key:"setScroll",value:function(e,t){this._scrollView.scrollTo({left:e,top:t}),this._raiseOnScrollWithoutPoint()}},{key:"offsetScroll",value:function(e,t){this._scrollView.scrollBy({left:e,top:t}),this._raiseOnScrollWithoutPoint()}},{key:"getSize",value:function(){var e=(0,b.getDiagram)(),t=e.Size,n=this._scrollView.$element();return new t(Math.floor(n.width()),Math.floor(n.height()))}},{key:"getScrollContainer",value:function(){return this._scrollView.$element()[0]}},{key:"getScrollBarWidth",value:function(){return 0}},{key:"detachEvents",value:function(){}},{key:"_raiseOnScroll",value:function(e,t){var n=(0,b.getDiagram)(),i=n.Point;this.onScroll.raise("notifyScrollChanged",function(){return new i(e,t)})}},{key:"_raiseOnScrollWithoutPoint",value:function(){var e=this,t=(0,b.getDiagram)(),n=t.Point;this.onScroll.raise("notifyScrollChanged",function(){return new n(e._scrollView.scrollLeft(),e._scrollView.scrollTop())})}},{key:"_createOnCreateDiagramAction",value:function(){this._onCreateDiagramAction=this._createActionByOption("onCreateDiagram")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onCreateDiagram":this._createOnCreateDiagramAction();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(v.default),w=x;t.default=w,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(10)),a={getDefaultGroups:function(){return this._groups||(this._groups={general:{category:"general",title:o.default.format("dxDiagram-categoryGeneral")},flowchart:{category:"flowchart",title:o.default.format("dxDiagram-categoryFlowchart")},orgChart:{category:"orgChart",title:o.default.format("dxDiagram-categoryOrgChart")},containers:{category:"containers",title:o.default.format("dxDiagram-categoryContainers")},custom:{category:"custom",title:o.default.format("dxDiagram-categoryCustom")}})},getGroups:function(e){var t=this.getDefaultGroups();return e?e.map(function(e){return"string"==typeof e?{category:e,title:t[e]&&t[e].title||e}:e}).filter(function(e){return e}):[t.general,t.flowchart,t.orgChart,t.containers]}},r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=n(7),b=n(6),x=o(n(10)),w=o(n(104)),C=o(n(456)),k=o(n(96)),S=o(n(195)),D=n(105),I=o(n(540)),T=130,O="dx-diagram-toolbox-popup",E="dx-diagram-toolbox-panel",A="dx-diagram-toolbox-input-container",P="dx-diagram-toolbox-input",R="shape-toolbox-tooltip",M="dx-skip-gesture-event",B=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._toolboxes=[],this._filterText="",this._createOnShapeCategoryRenderedAction(),this._createOnFilterChangedAction()}},{key:"_getPopupClass",value:function(){return O}},{key:"_getPopupHeight",value:function(){return this.isMobileView()?"100%":l(_(t.prototype),"_getPopupHeight",this).call(this)}},{key:"_getPopupMaxHeight",value:function(){return this.isMobileView()?"100%":l(_(t.prototype),"_getPopupMaxHeight",this).call(this)}},{key:"_getPopupMinHeight",value:function(){return T}},{key:"_getPopupPosition",value:function(){var e=this.option("offsetParent"),t={my:"left top",at:"left top",of:e};return this.isMobileView()?t:(0,v.extend)(t,{offset:this.option("offsetX")+" "+this.option("offsetY")})}},{key:"_getPopupAnimation",value:function(){var e=this.option("offsetParent");return this.isMobileView()?{hide:this._getPopupSlideAnimationObject({direction:"left",from:{position:{my:"left top",at:"left top",of:e}},to:{position:{my:"right top",at:"left top",of:e}}}),show:this._getPopupSlideAnimationObject({direction:"right",from:{position:{my:"right top",at:"left top",of:e}},to:{position:{my:"left top",at:"left top",of:e}}})}:l(_(t.prototype),"_getPopupAnimation",this).call(this)}},{key:"_getPopupOptions",value:function(){var e=l(_(t.prototype),"_getPopupOptions",this).call(this);return this.isMobileView()?e:(0,v.extend)(e,{showTitle:!0,toolbarItems:[{widget:"dxButton",location:"center",options:{activeStateEnabled:!1,focusStateEnabled:!1,hoverStateEnabled:!1,icon:"diagram-toolbox-drag",stylingMode:"outlined",type:"normal"}}]})}},{key:"_renderPopupContent",value:function(e){var t="100%";if(this.option("showSearch")){var n=(0,m.default)("<div>").addClass(A).appendTo(e);this._updateElementWidth(n),this._renderSearchInput(n),(0,y.hasWindow)()&&(t="calc(100% - "+this._searchInput.$element().height()+"px)")}var i=(0,m.default)("<div>").addClass(E).appendTo(e).height(t);this._updateElementWidth(i),this._renderScrollView(i)}},{key:"_updateElementWidth",value:function(e){void 0!==this.option("toolboxWidth")&&e.css("width",this.option("toolboxWidth"))}},{key:"updateMaxHeight",value:function(){if(!this.isMobileView()){var e=6;if(this._popup){var t=this._getPopupTitle();e+=t.outerHeight()}this._accordion&&(e+=this._accordion.$element().outerHeight()),this._searchInput&&(e+=this._searchInput.$element().outerHeight()),this.option("maxHeight",e)}}},{key:"_renderSearchInput",value:function(e){var t=this,n=(0,m.default)("<div>").addClass(P).appendTo(e);this._searchInput=this._createComponent(n,w.default,{stylingMode:"outlined",placeholder:x.default.format("dxDiagram-uiSearch"),onValueChanged:function(e){t._onInputChanged(e.value)},valueChangeEvent:"keyup",buttons:[{name:"search",location:"after",options:{activeStateEnabled:!1,focusStateEnabled:!1,hoverStateEnabled:!1,icon:"search",stylingMode:"outlined",type:"normal",onClick:function(){t._searchInput.focus()}}}]})}},{key:"_renderScrollView",value:function(e){var t=(0,m.default)("<div>").appendTo(e);this._scrollView=this._createComponent(t,k.default);var n=(0,m.default)("<div>").appendTo(this._scrollView.content());this._updateElementWidth(n),this._renderAccordion(n)}},{key:"_getAccordionDataSource",value:function(){for(var e=this,t=[],n=this.option("toolboxGroups"),i=0;i<n.length;i++){var o=n[i].category,a=n[i].title,r={category:o,title:a||o,expanded:n[i].expanded,displayMode:n[i].displayMode,shapes:n[i].shapes,onTemplate:function(t,n,i){var o=(0,m.default)(n);e._onShapeCategoryRenderedAction({category:i.category,displayMode:i.displayMode,dataToggle:R,shapes:i.shapes,$element:o}),e._toolboxes.push(o),""!==e._filterText&&e._onFilterChangedAction({text:e._filterText,filteringToolboxes:e._toolboxes.length-1}),e._createTooltips(o.find('[data-toggle="'+R+'"]'))}};t.push(r)}return t}},{key:"_createTooltips",value:function(e){var t=this,n=(0,D.getDiagram)(),i=n.Browser;if(!i.TouchUI){var o=this.$element();e.each(function(e,n){var i=(0,m.default)(n),a=i.attr("title");if(a){var r=(0,m.default)("<div>").html(a).appendTo(o);t._createComponent(r,S.default,{target:i.get(0),showEvent:"mouseenter",hideEvent:"mouseleave",position:"top",animation:{show:{type:"fade",from:0,to:1,delay:500},hide:{type:"fade",from:1,to:0,delay:100}}})}})}}},{key:"_renderAccordion",value:function(e){var t=this,n=this._getAccordionDataSource();this._accordion=this._createComponent(e,C.default,{multiple:!0,animationDuration:0,activeStateEnabled:!1,focusStateEnabled:!1,hoverStateEnabled:!1,collapsible:!0,displayExpr:"title",dataSource:n,disabled:this.option("disabled"),itemTemplate:function(e,n,i){e.onTemplate(t,i,e)},onSelectionChanged:function(e){t._updateScrollAnimateSubscription(e.component)},onContentReady:function(e){for(var i=0;i<n.length;i++)n[i].expanded===!1?e.component.collapseItem(i):n[i].expanded===!0&&e.component.expandItem(i);t._updateScrollAnimateSubscription(e.component)}})}},{key:"_updateScrollAnimateSubscription",value:function(e){var t=this;e._deferredAnimate=new b.Deferred,e._deferredAnimate.done(function(){t.updateMaxHeight(),t._scrollView.update(),t._updateScrollAnimateSubscription(e)})}},{key:"_raiseToolboxDragStart",value:function(){this._scrollView.$element().addClass(M)}},{key:"_raiseToolboxDragEnd",value:function(){this._scrollView.$element().removeClass(M)}},{key:"_onInputChanged",value:function(e){var t=this;this._filterText=e,this._onFilterChangedAction({text:this._filterText,filteringToolboxes:this._toolboxes.map(function(e,t){return t})}),this._toolboxes.forEach(function(e){var n=(0,m.default)(e);t._createTooltips(n.find('[data-toggle="'+R+'"]'))}),this.updateMaxHeight(),this._scrollView.update()}},{key:"_createOnShapeCategoryRenderedAction",value:function(){this._onShapeCategoryRenderedAction=this._createActionByOption("onShapeCategoryRendered")}},{key:"_createOnFilterChangedAction",value:function(){this._onFilterChangedAction=this._createActionByOption("onFilterChanged")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onShapeCategoryRendered":this._createOnShapeCategoryRenderedAction();break;case"onFilterChanged":this._createOnFilterChangedAction();break;case"showSearch":case"toolboxWidth":this._invalidate();break;case"toolboxGroups":this._accordion.option("dataSource",this._getAccordionDataSource());break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(I.default),V=B;t.default=V,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(0),y=o(n(96)),b=o(n(297)),x=o(n(540)),w=o(n(150)),C=420,k=340,S="dx-diagram-properties-popup",D="dx-diagram-properties-popup-notabs",I="dx-diagram-properties-panel",T="dx-diagram-properties-panel-group-title",O="dx-diagram-properties-panel-group-toolbar",E=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._commandTabs=w.default.getPropertyPanelCommandTabs(this.option("propertyTabs")),this._createOnCreateToolbar(),this._createOnSelectedGroupChanged()}},{key:"_initMarkup",value:function(){this._toolbars=[],this._selectedToolbar=void 0,l(_(t.prototype),"_initMarkup",this).call(this)}},{key:"_getPopupClass",value:function(){var e=S;return this._hasTabPanel()||(e+=" "+D),e}},{key:"_getPopupWidth",value:function(){return this.isMobileView()?"100%":C}},{key:"_getPopupHeight",value:function(){return k}},{key:"_getPopupPosition",value:function(){var e=this.option("offsetParent");return this.isMobileView()?{my:"left bottom",at:"left bottom",of:e}:{my:"right bottom",at:"right bottom",of:e,offset:"-"+this.option("offsetX")+" -"+this.option("offsetY")}}},{key:"_getPopupAnimation",value:function(){var e=this.option("offsetParent");return this.isMobileView()?{hide:this._getPopupSlideAnimationObject({direction:"bottom",from:{position:{my:"left bottom",at:"left bottom",of:e}},to:{position:{my:"left top",at:"left bottom",of:e}}}),show:this._getPopupSlideAnimationObject({direction:"top",from:{position:{my:"left top",at:"left bottom",of:e}},to:{position:{my:"left bottom",at:"left bottom",of:e}}})}:l(_(t.prototype),"_getPopupAnimation",this).call(this)}},{key:"_getPopupOptions",value:function(){return(0,v.extend)(l(_(t.prototype),"_getPopupOptions",this).call(this),{showTitle:this.isMobileView(),showCloseButton:this.isMobileView()})}},{key:"_renderPopupContent",value:function(e){if(this._commandTabs.length){var t=(0,m.default)("<div>").addClass(I).appendTo(e);this._hasTabPanel()?this._renderTabPanel(t):this._renderTabContent(t,this._commandTabs[0],0,!0)}}},{key:"_hasTabPanel",value:function(){return this._commandTabs.length>1}},{key:"_renderTabPanel",value:function(e){var t=this,n=(0,m.default)("<div>").appendTo(e);this._tabPanel=this._createComponent(n,b.default,{focusStateEnabled:!1,dataSource:this._commandTabs,itemTemplate:function(e,n,i){t._renderTabContent(i,e,n)},onSelectionChanged:function(e){t._onSelectedGroupChangedAction(),t._onPointerUpAction()},onContentReady:function(e){t._popup.option("height",e.component.$element().height()+t._getVerticalPaddingsAndBorders()),t._firstScrollView&&(t._scrollViewHeight=t._firstScrollView.$element().outerHeight(),t._firstScrollView.option("height",t._scrollViewHeight))}})}},{key:"_renderTabContent",value:function(e,t,n,i){var o=(0,m.default)("<div>").appendTo(e),a=this._createComponent(o,y.default,{height:this._scrollViewHeight});this._renderTabInnerContent(a.content(),t,n),i?this._popup.option("height",a.$element().height()+this._getVerticalPaddingsAndBorders()):this._firstScrollView=this._firstScrollView||a}},{key:"_renderTabInnerContent",value:function(e,t,n){var i=this;t.groups?t.groups.forEach(function(t,o){i._renderTabGroupContent(e,n,t.title,t.commands)}):t.commands&&this._renderTabGroupContent(e,n,void 0,t.commands)}},{key:"_renderTabGroupContent",value:function(e,t,n,i){n&&(0,m.default)("<div>").addClass(T).appendTo(e).text(n);var o=(0,m.default)("<div>").addClass(O).appendTo(e),a={$parent:o,commands:i};this._onCreateToolbarAction(a),this._toolbars[t]||(this._toolbars[t]=[]),this._toolbars[t].push(a.toolbar),this._selectedToolbar=a.toolbar}},{key:"getActiveToolbars",value:function(){var e=this._tabPanel?this._tabPanel.option("selectedIndex"):0;return this._toolbars[e]}},{key:"_createOnCreateToolbar",value:function(){this._onCreateToolbarAction=this._createActionByOption("onCreateToolbar")}},{key:"_createOnSelectedGroupChanged",value:function(){this._onSelectedGroupChangedAction=this._createActionByOption("onSelectedGroupChanged")}},{key:"_optionChanged",value:function(e){switch(e.name){case"onCreateToolbar":this._createOnCreateToolbar();break;case"onSelectedGroupChanged":this._createOnSelectedGroupChanged();break;case"propertyTabs":this._invalidate();break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(x.default),A=E;t.default=A,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){"@babel/helpers - typeof";return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==o(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=i(n(319)),_=n(105),m=function(e){function t(e){var i;a(this,t),i=n.call(this,e);var r=(0,_.getDiagram)(),s=r.DiagramCommand;return i.commandOptions={},i.commandOptions[s.Fullscreen]="fullScreen",i.commandOptions[s.ZoomLevel]=function(e){"object"===o(this._getOption("zoomLevel"))?this._setOption("zoomLevel.value",e):this._setOption("zoomLevel",e)},i.commandOptions[s.SwitchAutoZoom]=function(e){var t=(0,_.getDiagram)(),n=t.AutoZoomMode;switch(e){case n.FitContent:this._setOption("autoZoomMode","fitContent");break;case n.FitToWidth:this._setOption("autoZoomMode","fitWidth");break;case n.Disabled:this._setOption("autoZoomMode","disabled")}},i.commandOptions[s.ToggleSimpleView]="simpleView",i.commandOptions[s.ShowGrid]="showGrid",i.commandOptions[s.SnapToGrid]="snapToGrid",i.commandOptions[s.GridSize]=function(e){"object"===o(this._getOption("gridSize"))?this._setOption("gridSize.value",e):this._setOption("gridSize",e)},i.commandOptions[s.ViewUnits]="viewUnits",i.commandOptions[s.PageSize]=function(e){var t=this._getOption("pageSize");void 0!==t&&t.width===e.width&&t.height===e.height||this._setOption("pageSize",e)},i.commandOptions[s.PageLandscape]=function(e){this._setOption("pageOrientation",e?"landscape":"portrait")},i.commandOptions[s.ViewUnits]=function(e){var t=(0,_.getDiagram)(),n=t.DiagramUnit;switch(e){case n.In:this._setOption("viewUnits","in");break;case n.Cm:this._setOption("viewUnits","cm");break;case n.Px:this._setOption("viewUnits","px")}},i.commandOptions[s.PageColor]="pageColor",i._updateLock=0,i}l(t,e);var n=c(t);return s(t,[{key:"getCommandKeys",value:function(){return Object.keys(this.commandOptions).map(function(e){return parseInt(e)})}},{key:"setItemValue",value:function(e,t){if(!this.isUpdateLocked()){this.beginUpdate();try{"function"==typeof this.commandOptions[e]?this.commandOptions[e].call(this,t):this._setOption(this.commandOptions[e],t)}finally{this.endUpdate()}}}},{key:"beginUpdate",
value:function(){this._updateLock++}},{key:"endUpdate",value:function(){this._updateLock--}},{key:"isUpdateLocked",value:function(){return this._updateLock>0}},{key:"_getOption",value:function(e){return this._owner.option(e)}},{key:"_setOption",value:function(e,t){this._owner.option(e,t)}}]),t}(g.default),v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(105),r=i(n(10)),s=i(n(294)),l=n(7),u={getConfigurations:function(){var e=(0,a.getDiagram)(),t=e.DiagramCommand;return this.dialogList||(this.dialogList=[{command:t.InsertShapeImage,title:r.default.format("dxDiagram-dialogInsertShapeImageTitle"),onGetContent:this.getChangeImageDialogContent},{command:t.EditShapeImage,title:r.default.format("dxDiagram-dialogEditShapeImageTitle"),onGetContent:this.getChangeImageDialogContent}])},getChangeImageDialogContent:function(e){var t=(0,o.default)("<div>");return e.component._createComponent(t,s.default,{selectButtonText:r.default.format("dxDiagram-dialogEditShapeImageSelectButton"),accept:"image/*",uploadMode:"useForm",onValueChanged:function(t){var n=(0,l.getWindow)(),i=new n.FileReader;i.onload=function(t){e.component._commandParameter=t.target.result},i.readAsDataURL(t.value[0])}}),t},getDialogParameters:function(e){var t=this.getConfigurations().map(function(e){return e.command}).indexOf(e);return t>=0?this.getConfigurations()[t]:null}},c=u;t.default=c,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(541)),_=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getKeyExpr",value:function(){return this._diagramWidget._createOptionGetter("nodes.keyExpr")}},{key:"_getItemsExpr",value:function(){return this._diagramWidget._createOptionGetter("nodes.itemsExpr")}},{key:"_getContainerChildrenExpr",value:function(){return this._diagramWidget._createOptionGetter("nodes.containerChildrenExpr")}}]),t}(g.default),m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(541)),_=function(e){function t(){return a(this,t),n.apply(this,arguments)}l(t,e);var n=c(t);return s(t,[{key:"_getKeyExpr",value:function(){return this._diagramWidget._createOptionGetter("edges.keyExpr")}}]),t}(g.default),m=_;t.default=m,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(895)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var m=o(n(2)),v=n(1),y=o(n(16)),b=o(n(9)),x=n(19),w=n(896),C=n(900),k=o(n(506)),S=n(0),D=n(24),I=n(7),T=o(n(901)),O=o(n(537)),E=n(902),A=o(n(193)),P=n(18),R=n(903),M="dx-gantt",B="dx-gantt-view",V="dx-gantt-collapsable-row",F="dx-gantt-treelist-wrapper",L="dx-gantt-toolbar-wrapper",H="dx-gantt-main-wrapper",z="tasks",N="dependencies",j="resources",W="resourceAssignments",G="gantt_new_task_key",q=34,$=/(\w*)Expr/,K=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._cache=new R.GanttDataCache}},{key:"_initMarkup",value:function(){l(_(t.prototype),"_initMarkup",this).call(this),this.$element().addClass(M),this._$toolbarWrapper=(0,m.default)("<div>").addClass(L).appendTo(this.$element()),this._$toolbar=(0,m.default)("<div>").appendTo(this._$toolbarWrapper),this._$mainWrapper=(0,m.default)("<div>").addClass(H).appendTo(this.$element()),this._$treeListWrapper=(0,m.default)("<div>").addClass(F).appendTo(this._$mainWrapper),this._$treeList=(0,m.default)("<div>").appendTo(this._$treeListWrapper),this._$splitter=(0,m.default)("<div>").appendTo(this._$mainWrapper),this._$ganttView=(0,m.default)("<div>").addClass(B).appendTo(this._$mainWrapper),this._$dialog=(0,m.default)("<div>").appendTo(this.$element()),this._$loadPanel=(0,m.default)("<div>").appendTo(this.$element()),this._$contextMenu=(0,m.default)("<div>").appendTo(this.$element()),this._refreshDataSource(z),this._refreshDataSource(N),this._refreshDataSource(j),this._refreshDataSource(W)}},{key:"_renderContent",value:function(){this._renderBars(),this._renderTreeList(),this._renderSplitter()}},{key:"_renderTreeList",value:function(){var e=this,t=this.option(z),n=t.keyExpr,i=t.parentIdExpr;this._treeList=this._createComponent(this._$treeList,k.default,{dataSource:this._tasksRaw,keyExpr:n,parentIdExpr:i,columns:this._getTreeListColumns(),columnResizingMode:"nextColumn",height:this._getTreeListHeight(),width:this.option("taskListWidth"),selection:{mode:this._getSelectionMode(this.option("allowSelection"))},selectedRowKeys:this._getArrayFromOneElement(this.option("selectedRowKey")),sorting:{mode:"none"},scrolling:{showScrollbar:"onHover",mode:"virtual"},allowColumnResizing:!0,autoExpandAll:!0,showRowLines:this.option("showRowLines"),rootValue:this.option("rootValue"),onContentReady:function(t){e._onTreeListContentReady(t)},onSelectionChanged:function(t){e._onTreeListSelectionChanged(t)},onRowCollapsed:function(t){e._onTreeListRowCollapsed(t)},onRowExpanded:function(t){e._onTreeListRowExpanded(t)},onRowPrepared:function(t){e._onTreeListRowPrepared(t)},onContextMenuPreparing:function(t){e._onTreeListContextMenuPreparing(t)},onRowClick:function(t){e._onTreeListRowClick(t)},onRowDblClick:function(t){e._onTreeListRowDblClick(t)}})}},{key:"_renderSplitter",value:function(){this._splitter=this._createComponent(this._$splitter,O.default,{container:this.$element(),leftElement:this._$treeListWrapper,rightElement:this._$ganttView,onApplyPanelSize:this._onApplyPanelSize.bind(this)}),this._splitter.option("initialLeftPanelWidth",this.option("taskListWidth"))}},{key:"_renderBars",value:function(){this._bars=[],this._toolbar=new C.GanttToolbar(this._$toolbar,this),this._updateToolbarContent(),this._bars.push(this._toolbar),this._contextMenuBar=new C.GanttContextMenuBar(this._$contextMenu,this),this._updateContextMenu(),this._bars.push(this._contextMenuBar)}},{key:"_initGanttView",value:function(){var e=this;this._ganttView||(this._ganttView=this._createComponent(this._$ganttView,w.GanttView,{width:"100%",height:this._treeList._$element.get(0).offsetHeight,rowHeight:this._getTreeListRowHeight(),headerHeight:this._getTreeListHeaderHeight(),tasks:this._tasks,dependencies:this._dependencies,resources:this._resources,resourceAssignments:this._resourceAssignments,allowSelection:this.option("allowSelection"),selectedRowKey:this.option("selectedRowKey"),showResources:this.option("showResources"),taskTitlePosition:this.option("taskTitlePosition"),firstDayOfWeek:this.option("firstDayOfWeek"),showRowLines:this.option("showRowLines"),scaleType:this.option("scaleType"),editing:this.option("editing"),validation:this.option("validation"),stripLines:this.option("stripLines"),bars:this._bars,mainElement:this.$element(),onSelectionChanged:this._onGanttViewSelectionChanged.bind(this),onScroll:this._onGanttViewScroll.bind(this),onDialogShowing:this._showDialog.bind(this),onPopupMenuShowing:this._showPopupMenu.bind(this),onExpandAll:this._expandAll.bind(this),onCollapseAll:this._collapseAll.bind(this),modelChangesListener:this._createModelChangesListener(),taskTooltipContentTemplate:this._getTaskTooltipContentTemplateFunc(this.option("taskTooltipContentTemplate")),onTaskClick:function(t){e._onTreeListRowClick(t)},onTaskDblClick:function(t){e._onTreeListRowDblClick(t)}}),this._fireContentReadyAction())}},{key:"_onApplyPanelSize",value:function(e){var t;this._setInnerElementsWidth(e);var n=this._getTreeListRowHeight();null===(t=this._ganttView)||void 0===t?void 0:t._ganttViewCore.updateRowHeights(n)}},{key:"_onTreeListContentReady",value:function(e){e.component.getDataSource()&&(this._initGanttView(),this._initScrollSync(e.component))}},{key:"_onTreeListRowPrepared",value:function(e){"data"===e.rowType&&e.node.children.length>0&&(0,m.default)(e.rowElement).addClass(V)}},{key:"_onTreeListContextMenuPreparing",value:function(e){var t,n;"data"===(null===(t=e.row)||void 0===t?void 0:t.rowType)&&this._setTreeListOption("selectedRowKeys",[e.row.data[this.option("tasks.keyExpr")]]),e.items=[];var i={cancel:!1,event:e.event,type:"task",key:null===(n=e.row)||void 0===n?void 0:n.key,position:{x:e.event.pageX,y:e.event.pageY}};this._showPopupMenu(i)}},{key:"_onTreeListRowClick",value:function(e){this._raiseTaskClickAction(e.key,e.event)}},{key:"_onTreeListRowDblClick",value:function(e){this._raiseTaskDblClickAction(e.key,e.event)&&this._ganttView._ganttViewCore.commandManager.showTaskEditDialog.execute()}},{key:"_onTreeListSelectionChanged",value:function(e){var t=e.currentSelectedRowKeys[0];this._setGanttViewOption("selectedRowKey",t),this.option("selectedRowKey",t),this._raiseSelectionChangedAction(t)}},{key:"_onTreeListRowCollapsed",value:function(e){this._ganttView.changeTaskExpanded(e.key,!1),this._adjustHeight()}},{key:"_onTreeListRowExpanded",value:function(e){this._ganttView.changeTaskExpanded(e.key,!0),this._adjustHeight()}},{key:"_adjustHeight",value:function(){this._hasHeight||(this._setGanttViewOption("height",0),this._setGanttViewOption("height",this._treeList._$element.get(0).offsetHeight))}},{key:"_getTreeListHeight",value:function(){return this._$treeList.height()?this._$treeList.height():(this._hasHeight=(0,v.isDefined)(this.option("height"))&&""!==this.option("height"),this._hasHeight?"100%":"")}},{key:"_getTreeListColumns",value:function(){var e=this.option("columns");if(e)for(var t=0;t<e.length;t++){var n=e[t],i=n.dataField===this.option("".concat(z,".keyExpr"))||n.dataField===this.option("".concat(z,".parentIdExpr"));i&&!n.dataType&&(n.dataType="object")}return e}},{key:"_onGanttViewSelectionChanged",value:function(e){this._setTreeListOption("selectedRowKeys",this._getArrayFromOneElement(e.id))}},{key:"_onGanttViewScroll",value:function(e){var t=this._treeList.getScrollable();if(t){var n=e.scrollTop-t.scrollTop();0!==n&&t.scrollBy({left:0,top:n})}}},{key:"_onTreeListScroll",value:function(e){var t=this._ganttView.getTaskAreaContainer();t.scrollTop!==e.component.scrollTop()&&(t.scrollTop=e.component.scrollTop())}},{key:"_expandAll",value:function(){var e=this;this._treeList.forEachNode(function(t){t.children&&t.children.length&&(e._treeList.expandRow(t.key),e._ganttView.changeTaskExpanded(t.key,!0))})}},{key:"_collapseAll",value:function(){var e=this;this._treeList.forEachNode(function(t){t.children&&t.children.length&&(e._treeList.collapseRow(t.key),e._ganttView.changeTaskExpanded(t.key,!1))})}},{key:"_initScrollSync",value:function(e){var t=this,n=e.getScrollable();n&&(n.off("scroll"),n.on("scroll",function(e){t._onTreeListScroll(e)}))}},{key:"_getTreeListRowHeight",value:function(){var e=this._treeList._$element.find(".dx-data-row"),t=e.length?(0,D.getBoundingRect)(e.last().get(0)).height:q;return t||(t=q),this._correctRowsViewRowHeight(t),t}},{key:"_correctRowsViewRowHeight",value:function(e){var t=this._treeList._views&&this._treeList._views.rowsView;(null===t||void 0===t?void 0:t._rowHeight)!==e&&(t._rowHeight=e)}},{key:"_getTreeListHeaderHeight",value:function(){return(0,D.getBoundingRect)(this._treeList._$element.find(".dx-treelist-headers").get(0)).height}},{key:"_setInnerElementsWidth",value:function(e){if((0,I.hasWindow)()){e||(e=this._getPanelsWidthByOption());var t=e.leftPanelWidth,n=e.rightPanelWidth;this._$treeListWrapper.width(t);var i=(0,v.isString)(t)&&"%"===t.slice(-1);this._$treeList.width(i?"100%":t),this._$ganttView.width(n),this._setGanttViewOption("width",this._$ganttView.width())}}},{key:"_getPanelsWidthByOption",value:function(){return{leftPanelWidth:this.option("taskListWidth"),rightPanelWidth:this._$element.width()-this.option("taskListWidth")}}},{key:"_setGanttViewOption",value:function(e,t){this._ganttView&&this._ganttView.option(e,t)}},{key:"_setTreeListOption",value:function(e,t){this._treeList&&this._treeList.option(e,t)}},{key:"_refreshDataSource",value:function(e){var t=this,n=this["_".concat(e,"Option")];n&&(n._disposeDataSource(),delete this["_".concat(e,"Option")],delete this["_".concat(e)]),this.option("".concat(e,".dataSource"))&&(n=new T.default(e,this._getLoadPanel(),function(e,n){t._dataSourceChanged(e,n)}),n.option("dataSource",this._getSpecificDataSourceOption(e)),n._refreshDataSource(),this["_".concat(e,"Option")]=n)}},{key:"_getSpecificDataSourceOption",value:function(e){var t=this.option("".concat(e,".dataSource"));return Array.isArray(t)?{store:{type:"array",data:t,key:this.option("".concat(e,".keyExpr"))}}:t}},{key:"_compileGettersByOption",value:function(e){var t={},n=this.option(e);for(var i in n){var o=i.match(/(\w*)Expr/);o&&(t[o[1]]=(0,x.compileGetter)(n[o[0]]))}return t}},{key:"_compileSettersByOption",value:function(e){var t={},n=this.option(e);for(var i in n){var o=i.match(/(\w*)Expr/);o&&(t[o[1]]=(0,x.compileSetter)(n[o[0]]))}return t}},{key:"_getStoreObject",value:function(e,t){var n=this._compileSettersByOption(e);return Object.keys(n).reduce(function(e,i){return"key"!==i&&n[i](e,t[i]),e},{})}},{key:"_prepareSetterMapHandler",value:function(e){return function(t){return Object.keys(e).reduce(function(n,i){var o="key"===i?"id":i;return e[i](n,t[o]),n},{})}}},{key:"_prepareMapHandler",value:function(e){return function(t){return Object.keys(e).reduce(function(n,i){var o="key"===i?"id":i;return n[o]=e[i](t),n},{})}}},{key:"_dataSourceChanged",value:function(e,t){var n=this,i=this._compileGettersByOption(e),o=t.map(this._prepareMapHandler(i));if(this["_".concat(e)]=o,this._setGanttViewOption(e,o),e===z){this._tasksRaw=t;var a=t.map(function(e){return e[n.option("tasks.parentIdExpr")]}).filter(function(e,t,n){return e&&n.indexOf(e)===t});this._setTreeListOption("expandedRowKeys",a),this._setTreeListOption("dataSource",t)}}},{key:"_createModelChangesListener",value:function(){var e=this;return{NotifyTaskCreated:function(t,n,i){e._onRecordInserted(z,t,n)},NotifyTaskRemoved:function(t,n,i){e._onRecordRemoved(z,t,i)},NotifyTaskTitleChanged:function(t,n,i){e._onRecordUpdated(z,t,"title",n)},NotifyTaskDescriptionChanged:function(t,n,i){e._onRecordUpdated(z,t,"description",n)},NotifyTaskStartChanged:function(t,n,i){e._onRecordUpdated(z,t,"start",n)},NotifyTaskEndChanged:function(t,n,i){e._onRecordUpdated(z,t,"end",n)},NotifyTaskProgressChanged:function(t,n,i){e._onRecordUpdated(z,t,"progress",n)},NotifyDependencyInserted:function(t,n,i){e._onRecordInserted(N,t,n)},NotifyDependencyRemoved:function(t,n,i){e._onRecordRemoved(N,t,i)},NotifyResourceCreated:function(t,n,i){e._onRecordInserted(j,t,n)},NotifyResourceRemoved:function(t,n,i){e._onRecordRemoved(j,t,i)},NotifyResourceAssigned:function(t,n,i){e._onRecordInserted(W,t,n)},NotifyResourceUnassigned:function(t,n,i){e._onRecordRemoved(W,t,i)},NotifyParentDataRecalculated:function(t){e._onParentTasksRecalculated(t)},NotifyTaskCreating:function(t){e._raiseInsertingAction(z,t)},NotifyTaskRemoving:function(t){e._raiseDeletingAction(z,t)},NotifyTaskUpdating:function(t){e._raiseUpdatingAction(z,t)},NotifyTaskMoving:function(t){e._raiseUpdatingAction(z,t,e._getTaskMovingAction())},NotifyTaskEditDialogShowing:function(t){e._raiseTaskEditDialogShowingAction(t)},NotifyDependencyInserting:function(t){e._raiseInsertingAction(N,t)},NotifyDependencyRemoving:function(t){e._raiseDeletingAction(N,t)},NotifyResourceCreating:function(t){e._raiseInsertingAction(j,t)},NotifyResourceRemoving:function(t){e._raiseDeletingAction(j,t)},NotifyResourceAssigning:function(t){e._raiseInsertingAction(W,t)},NotifyResourceUnassigning:function(t){e._raiseDeletingAction(W,t)}}}},{key:"_onRecordInserted",value:function(e,t,n){var i=this,o=this["_".concat(e,"Option")];if(o){var a=this._getStoreObject(e,t);e===z&&this._addCustomFieldsDataFromCache(G,a),o.insert(a,function(o){var r=(0,x.compileGetter)(i.option("".concat(e,".keyExpr"))),s=r(o);if(n(s),e===z){i._updateTreeListDataSource();var l=t.parentId;if(void 0!==l){var u=i._treeList.option("expandedRowKeys");u.indexOf(l)===-1&&(u.push(l),i._treeList.option("expandedRowKeys",u))}i._setTreeListOption("selectedRowKeys",i._getArrayFromOneElement(s)),i._setTreeListOption("focusedRowKey",s)}i._raiseInsertedAction(e,a,s)})}}},{key:"_onRecordRemoved",value:function(e,t,n){var i=this,o=this["_".concat(e,"Option")];o&&o.remove(t,function(){e===z&&i._updateTreeListDataSource(),i._raiseDeletedAction(e,t,i._convertCoreToMappedData(e,n))})}},{key:"_onRecordUpdated",value:function(e,t,n,i){var o=this,a=this["_".concat(e,"Option")],r=e===z;if(a){var s=(0,x.compileSetter)(this.option("".concat(e,".").concat(n,"Expr"))),l={};s(l,i);var u=r&&this._cache.hasData(t);u&&this._addCustomFieldsDataFromCache(t,l),a.update(t,l,function(){r&&(u&&a._refreshDataSource(),o._updateTreeListDataSource()),o._raiseUpdatedAction(e,l,t)})}}},{key:"_onParentTasksRecalculated",value:function(e){var t=this._compileSettersByOption(z),n=this._appendCustomFields(e.map(this._prepareSetterMapHandler(t)));this._setTreeListOption("dataSource",n)}},{key:"_appendCustomFields",value:function(e){var t=this._tasksOption&&this._tasksOption._getItems(),n=(0,x.compileGetter)(this.option("".concat(z,".keyExpr")));return e.reduce(function(e,i){var o=t&&t.filter(function(e){return n(e)===n(i)})[0];if(o){var a={};for(var r in o)a[r]=Object.prototype.hasOwnProperty.call(i,r)?i[r]:o[r];e.push(a)}else e.push(i);return e},[])}},{key:"_updateTreeListDataSource",value:function(){if(!this._skipUpdateTreeListDataSource()){var e=this.option("tasks.dataSource"),t=this._tasksOption._getStore()._array||e.items&&e.items();this._setTreeListOption("dataSource",t?t:e)}}},{key:"_skipUpdateTreeListDataSource",value:function(){return this.option("validation.autoUpdateParentTasks")}},{key:"_addCustomFieldsDataFromCache",value:function(e,t){this._cache.pullDataFromCache(e,t)}},{key:"_saveCustomFieldsDataToCache",value:function(e,t){var n=this,i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],o=this._getCustomFieldsData(t);if(Object.keys(o).length>0){var a=function(e,t){var i=n["_".concat(z,"Option")];i&&t&&i.update(e,t,function(){n._updateTreeListDataSource(),i._refreshDataSource()})};this._cache.saveData(e,o,i?a:null)}}},{key:"_getLoadPanel",value:function(){return this._loadPanel||(this._loadPanel=this._createComponent(this._$loadPanel,A.default,{position:{of:this.$element()}})),this._loadPanel}},{key:"_createSelectionChangedAction",value:function(){this._selectionChangedAction=this._createActionByOption("onSelectionChanged")}},{key:"_createTaskClickAction",value:function(){this._taskClickAction=this._createActionByOption("onTaskClick")}},{key:"_createTaskDblClickAction",value:function(){this._taskDblClickAction=this._createActionByOption("onTaskDblClick")}},{key:"_createCustomCommandAction",value:function(){this._customCommandAction=this._createActionByOption("onCustomCommand")}},{key:"_createContextMenuPreparingAction",value:function(){this._contextMenuPreparingAction=this._createActionByOption("onContextMenuPreparing")}},{key:"_raiseSelectionChangedAction",value:function(e){this._selectionChangedAction||this._createSelectionChangedAction(),this._selectionChangedAction({selectedRowKey:e})}},{key:"_raiseCustomCommand",value:function(e){this._customCommandAction||this._createCustomCommandAction(),this._customCommandAction({name:e})}},{key:"_raiseContextMenuPreparing",value:function(e){this._contextMenuPreparingAction||this._createContextMenuPreparingAction(),this._contextMenuPreparingAction(e)}},{key:"_raiseInsertingAction",value:function(e,t){var n=this._getInsertingAction(e);if(n){var i={cancel:!1,values:this._convertCoreToMappedData(e,t.values)};n(i),t.cancel=i.cancel,t.values=this._convertMappedToCoreData(e,i.values),e===z&&this._saveCustomFieldsDataToCache(G,i.newValues)}}},{key:"_raiseInsertedAction",value:function(e,t,n){var i=this._getInsertedAction(e);if(i){var o={values:t,key:n};i(o)}}},{key:"_raiseDeletingAction",value:function(e,t){var n=this._getDeletingAction(e);if(n){var i={cancel:!1,key:t.key,values:this._convertCoreToMappedData(e,t.values)};n(i),t.cancel=i.cancel}}},{key:"_raiseDeletedAction",value:function(e,t,n){var i=this._getDeletedAction(e);if(i){var o={key:t,values:n};i(o)}}},{key:"_raiseUpdatingAction",value:function(e,t,n){if(n=n||this._getUpdatingAction(e)){var i={cancel:!1,key:t.key,newValues:this._convertCoreToMappedData(e,t.newValues),values:this._convertCoreToMappedData(e,t.values)};n(i),t.cancel=i.cancel,t.newValues=this._convertMappedToCoreData(e,i.newValues),e===z&&this._saveCustomFieldsDataToCache(i.key,i.newValues)}}},{key:"_raiseUpdatedAction",value:function(e,t,n){var i=this._getUpdatedAction(e);if(i){var o={values:t,key:n};i(o)}}},{key:"_raiseTaskEditDialogShowingAction",value:function(e){var t=this._getTaskEditDialogShowingAction();if(t){var n={cancel:!1,key:e.key,values:this._convertCoreToMappedData(z,e.values),readOnlyFields:this._convertCoreToMappedFields(z,e.readOnlyFields),hiddenFields:this._convertCoreToMappedFields(z,e.hiddenFields)};t(n),e.cancel=n.cancel,e.values=this._convertMappedToCoreData(z,n.values),e.readOnlyFields=this._convertMappedToCoreFields(z,n.readOnlyFields),e.hiddenFields=this._convertMappedToCoreFields(z,n.hiddenFields)}}},{key:"_raiseTaskClickAction",value:function(e,t){this._taskClickAction||this._createTaskClickAction();var n={key:e,event:t,data:this.getTaskData(e)};this._taskClickAction(n)}},{key:"_raiseTaskDblClickAction",value:function(e,t){this._taskDblClickAction||this._createTaskDblClickAction();var n={cancel:!1,data:this.getTaskData(e),event:t,key:e};return this._taskDblClickAction(n),!n.cancel}},{key:"_getInsertingAction",value:function(e){switch(e){case z:return this._getTaskInsertingAction();case N:return this._getDependencyInsertingAction();case j:return this._getResourceInsertingAction();case W:return this._getResourceAssigningAction()}return function(){}}},{key:"_getInsertedAction",value:function(e){switch(e){case z:return this._getTaskInsertedAction();case N:return this._getDependencyInsertedAction();case j:return this._getResourceInsertedAction();case W:return this._getResourceAssignedAction()}return function(){}}},{key:"_getDeletingAction",value:function(e){switch(e){case z:return this._getTaskDeletingAction();case N:return this._getDependencyDeletingAction();case j:return this._getResourceDeletingAction();case W:return this._getResourceUnassigningAction()}return function(){}}},{key:"_getDeletedAction",value:function(e){switch(e){case z:return this._getTaskDeletedAction();case N:return this._getDependencyDeletedAction();case j:return this._getResourceDeletedAction();case W:return this._getResourceUnassignedAction()}return function(){}}},{key:"_getUpdatingAction",value:function(e){switch(e){case z:return this._getTaskUpdatingAction()}return function(){}}},{key:"_getUpdatedAction",value:function(e){switch(e){case z:return this._getTaskUpdatedAction()}return function(){}}},{key:"_getTaskInsertingAction",value:function(){return this._taskInsertingAction||this._createTaskInsertingAction(),this._taskInsertingAction}},{key:"_getTaskInsertedAction",value:function(){return this._taskInsertedAction||this._createTaskInsertedAction(),this._taskInsertedAction}},{key:"_getTaskDeletingAction",value:function(){return this._taskDeletingAction||this._createTaskDeletingAction(),this._taskDeletingAction}},{key:"_getTaskDeletedAction",value:function(){return this._taskDeletedAction||this._createTaskDeletedAction(),this._taskDeletedAction}},{key:"_getTaskUpdatingAction",value:function(){return this._taskUpdatingAction||this._createTaskUpdatingAction(),this._taskUpdatingAction}},{key:"_getTaskUpdatedAction",value:function(){return this._taskUpdatedAction||this._createTaskUpdatedAction(),this._taskUpdatedAction}},{key:"_getTaskMovingAction",value:function(){return this._taskMovingAction||this._createTaskMovingAction(),this._taskMovingAction}},{key:"_getTaskEditDialogShowingAction",value:function(){return this._taskEditDialogShowingAction||this._createTaskEditDialogShowingAction(),this._taskEditDialogShowingAction}},{key:"_getDependencyInsertingAction",value:function(){return this._dependencyInsertingAction||this._createDependencyInsertingAction(),this._dependencyInsertingAction}},{key:"_getDependencyInsertedAction",value:function(){return this._dependencyInsertedAction||this._createDependencyInsertedAction(),this._dependencyInsertedAction}},{key:"_getDependencyDeletingAction",value:function(){return this._dependencyDeletingAction||this._createDependencyDeletingAction(),this._dependencyDeletingAction}},{key:"_getDependencyDeletedAction",value:function(){return this._dependencyDeletedAction||this._createDependencyDeletedAction(),this._dependencyDeletedAction}},{key:"_getResourceInsertingAction",value:function(){return this._resourceInsertingAction||this._createResourceInsertingAction(),this._resourceInsertingAction}},{key:"_getResourceInsertedAction",value:function(){return this._resourceInsertedAction||this._createResourceInsertedAction(),this._resourceInsertedAction}},{key:"_getResourceDeletingAction",value:function(){return this._resourceDeletingAction||this._createResourceDeletingAction(),this._resourceDeletingAction}},{key:"_getResourceDeletedAction",value:function(){return this._resourceDeletedAction||this._createResourceDeletedAction(),this._resourceDeletedAction}},{key:"_getResourceAssigningAction",value:function(){return this._resourceAssigningAction||this._createResourceAssigningAction(),this._resourceAssigningAction}},{key:"_getResourceAssignedAction",value:function(){return this._resourceAssignedAction||this._createResourceAssignedAction(),this._resourceAssignedAction}},{key:"_getResourceUnassigningAction",value:function(){return this._resourceUnassigningAction||this._createResourceUnassigningAction(),this._resourceUnassigningAction}},{key:"_getResourceUnassignedAction",value:function(){return this._resourceUnassignedAction||this._createResourceUnassignedAction(),this._resourceUnassignedAction}},{key:"_createResourceUnassigningAction",value:function(){this._resourceUnassigningAction=this._createActionByOption("onResourceUnassigning")}},{key:"_createResourceUnassignedAction",value:function(){this._resourceUnassignedAction=this._createActionByOption("onResourceUnassigned")}},{key:"_createTaskInsertingAction",value:function(){this._taskInsertingAction=this._createActionByOption("onTaskInserting")}},{key:"_createTaskInsertedAction",value:function(){this._taskInsertedAction=this._createActionByOption("onTaskInserted")}},{key:"_createTaskDeletingAction",value:function(){this._taskDeletingAction=this._createActionByOption("onTaskDeleting")}},{key:"_createTaskDeletedAction",value:function(){this._taskDeletedAction=this._createActionByOption("onTaskDeleted");
}},{key:"_createTaskUpdatingAction",value:function(){this._taskUpdatingAction=this._createActionByOption("onTaskUpdating")}},{key:"_createTaskUpdatedAction",value:function(){this._taskUpdatedAction=this._createActionByOption("onTaskUpdated")}},{key:"_createTaskMovingAction",value:function(){this._taskMovingAction=this._createActionByOption("onTaskMoving")}},{key:"_createTaskEditDialogShowingAction",value:function(){this._taskEditDialogShowingAction=this._createActionByOption("onTaskEditDialogShowing")}},{key:"_createDependencyInsertingAction",value:function(){this._dependencyInsertingAction=this._createActionByOption("onDependencyInserting")}},{key:"_createDependencyInsertedAction",value:function(){this._dependencyInsertedAction=this._createActionByOption("onDependencyInserted")}},{key:"_createDependencyDeletingAction",value:function(){this._dependencyDeletingAction=this._createActionByOption("onDependencyDeleting")}},{key:"_createDependencyDeletedAction",value:function(){this._dependencyDeletedAction=this._createActionByOption("onDependencyDeleted")}},{key:"_createResourceInsertingAction",value:function(){this._resourceInsertingAction=this._createActionByOption("onResourceInserting")}},{key:"_createResourceInsertedAction",value:function(){this._resourceInsertedAction=this._createActionByOption("onResourceInserted")}},{key:"_createResourceDeletingAction",value:function(){this._resourceDeletingAction=this._createActionByOption("onResourceDeleting")}},{key:"_createResourceDeletedAction",value:function(){this._resourceDeletedAction=this._createActionByOption("onResourceDeleted")}},{key:"_createResourceAssigningAction",value:function(){this._resourceAssigningAction=this._createActionByOption("onResourceAssigning")}},{key:"_createResourceAssignedAction",value:function(){this._resourceAssignedAction=this._createActionByOption("onResourceAssigned")}},{key:"_convertCoreToMappedData",value:function(e,t){var n=this;return Object.keys(t).reduce(function(i,o){var a=n._getMappedFieldName(e,o);if(a){var r=(0,x.compileSetter)(a);r(i,t[o])}return i},{})}},{key:"_convertMappedToCoreData",value:function(e,t){var n={};if(t){var i=this.option(e);for(var o in i){var a=o.match($),r=a&&i[a[0]];if(r&&void 0!==t[r]){var s=(0,x.compileGetter)(r),l=a[1];n[l]=s(t)}}}return n}},{key:"_getMappedFieldName",value:function(e,t){var n=t;return"id"===t&&(n="key"),this.option("".concat(e,".").concat(n,"Expr"))}},{key:"_convertCoreToMappedFields",value:function(e,t){var n=this;return t.reduce(function(t,i){var o=n._getMappedFieldName(e,i);return o&&t.push(o),t},[])}},{key:"_convertMappedToCoreFields",value:function(e,t){var n=[],i=this.option(e);for(var o in i){var a=o.match($),r=a&&i[a[0]];if(r&&t.indexOf(r)>-1){var s=a[1];n.push(s)}}return n}},{key:"_getTaskMappedFieldNames",value:function(){var e=[],t=this.option(z);for(var n in t){var i=n.match($),o=i&&t[i[0]];o&&e.push(o)}return e}},{key:"_getTaskCustomFields",value:function(){var e=this.option("columns"),t=e&&e.map(function(e){return e.dataField}),n=this._getTaskMappedFieldNames();return t?t.filter(function(e){return n.indexOf(e)<0}):[]}},{key:"_getCustomFieldsData",value:function(e){return this._getTaskCustomFields().reduce(function(t,n){return e&&void 0!==e[n]&&(t[n]=e[n]),t},{})}},{key:"_addCustomFieldsData",value:function(e,t){if(t)for(var n=this._tasksOption&&this._tasksOption._getItems(),i=(0,x.compileGetter)(this.option("".concat(z,".keyExpr"))),o=n&&n.filter(function(t){return i(t)===e})[0],a=this._getTaskCustomFields(),r=0;r<a.length;r++){var s=a[r];Object.prototype.hasOwnProperty.call(o,s)&&(t[s]=o[s])}}},{key:"_getSelectionMode",value:function(e){return e?"single":"none"}},{key:"_getArrayFromOneElement",value:function(e){return void 0===e||null===e?[]:[e]}},{key:"_getToolbarItems",value:function(){var e=this.option("toolbar.items");return e?e:[]}},{key:"_updateToolbarContent",value:function(){var e=this._getToolbarItems();e.length?this._$toolbarWrapper.show():this._$toolbarWrapper.hide(),this._toolbar&&this._toolbar.createItems(e),this._updateBarItemsState()}},{key:"_updateContextMenu",value:function(){var e=this.option("contextMenu");e.enabled&&this._contextMenuBar&&(this._contextMenuBar.createItems(e.items),this._updateBarItemsState())}},{key:"_updateBarItemsState",value:function(){this._ganttView&&this._ganttView.updateBarItemsState()}},{key:"_showDialog",value:function(e){this._dialogInstance||(this._dialogInstance=new E.GanttDialog(this,this._$dialog)),this._dialogInstance.show(e.name,e.parameters,e.callback,e.afterClosing,this.option("editing"))}},{key:"_showPopupMenu",value:function(e){if(this.option("contextMenu.enabled")){this._ganttView.getBarManager().updateContextMenu();var t={cancel:!1,event:e.event,targetType:e.type,targetKey:e.key,items:(0,S.extend)(!0,[],this._contextMenuBar._items),data:"task"===e.type?this.getTaskData(e.key):this.getDependencyData(e.key)};this._raiseContextMenuPreparing(t),t.cancel||this._contextMenuBar.show(e.position,t.items)}}},{key:"_executeCoreCommand",value:function(e){this._ganttView.executeCoreCommand(e)}},{key:"_clean",value:function(){delete this._ganttView,delete this._dialogInstance,l(_(t.prototype),"_clean",this).call(this)}},{key:"_getTaskTooltipContentTemplateFunc",value:function(e){var t=this,n=!0,i=e&&this._getTemplate(e),o=i&&function(e,o){return i.render({model:t.getTaskDataByCoreData(o),container:(0,P.getPublicElement)((0,m.default)(e))}),n};return o}},{key:"_getDefaultOptions",value:function(){return(0,S.extend)(l(_(t.prototype),"_getDefaultOptions",this).call(this),{tasks:{dataSource:null,keyExpr:"id",parentIdExpr:"parentId",startExpr:"start",endExpr:"end",progressExpr:"progress",titleExpr:"title",colorExpr:"color"},dependencies:{dataSource:null,keyExpr:"id",predecessorIdExpr:"predecessorId",successorIdExpr:"successorId",typeExpr:"type"},resources:{dataSource:null,keyExpr:"id",textExpr:"text",colorExpr:"color"},resourceAssignments:{dataSource:null,keyExpr:"id",taskIdExpr:"taskId",resourceIdExpr:"resourceId"},columns:void 0,taskListWidth:300,showResources:!0,taskTitlePosition:"inside",firstDayOfWeek:void 0,selectedRowKey:void 0,onSelectionChanged:null,onTaskClick:null,onTaskDblClick:null,onTaskInserting:null,onTaskInserted:null,onTaskDeleting:null,onTaskDeleted:null,onTaskUpdating:null,onTaskUpdated:null,onTaskMoving:null,onTaskEditDialogShowing:null,onDependencyInserting:null,onDependencyInserted:null,onDependencyDeleting:null,onDependencyDeleted:null,onResourceInserting:null,onResourceInserted:null,onResourceDeleting:null,onResourceDeleted:null,onResourceAssigning:null,onResourceAssigned:null,onResourceUnassigning:null,onResourceUnassigned:null,onCustomCommand:null,onContextMenuPreparing:null,allowSelection:!0,showRowLines:!0,stripLines:void 0,scaleType:"auto",editing:{enabled:!1,allowTaskAdding:!0,allowTaskDeleting:!0,allowTaskUpdating:!0,allowDependencyAdding:!0,allowDependencyDeleting:!0,allowResourceAdding:!0,allowResourceDeleting:!0,allowResourceUpdating:!0,allowTaskResourceUpdating:!0},validation:{validateDependencies:!1,autoUpdateParentTasks:!1},toolbar:null,contextMenu:{enabled:!0,items:void 0},taskTooltipContentTemplate:null,rootValue:0})}},{key:"getTaskResources",value:function(e){var t=this;if(!(0,v.isDefined)(e))return null;var n=this._ganttView._ganttViewCore.getTaskResources(e);return n.map(function(e){return t._convertCoreToMappedData(j,e)})}},{key:"getVisibleTaskKeys",value:function(){return this._ganttView._ganttViewCore.getVisibleTaskKeys()}},{key:"getVisibleDependencyKeys",value:function(){return this._ganttView._ganttViewCore.getVisibleDependencyKeys()}},{key:"getVisibleResourceKeys",value:function(){return this._ganttView._ganttViewCore.getVisibleResourceKeys()}},{key:"getVisibleResourceAssignmentKeys",value:function(){return this._ganttView._ganttViewCore.getVisibleResourceAssignmentKeys()}},{key:"getTaskData",value:function(e){if(!(0,v.isDefined)(e))return null;var t=this._ganttView._ganttViewCore.getTaskData(e),n=this.getTaskDataByCoreData(t);return n}},{key:"getTaskDataByCoreData",value:function(e){var t=e?this._convertCoreToMappedData(z,e):null;return this._addCustomFieldsData(e.id,t),t}},{key:"insertTask",value:function(e){this._saveCustomFieldsDataToCache(G,e),this._ganttView._ganttViewCore.insertTask(this._convertMappedToCoreData(z,e))}},{key:"deleteTask",value:function(e){this._ganttView._ganttViewCore.deleteTask(e)}},{key:"updateTask",value:function(e,t){this._saveCustomFieldsDataToCache(e,t,!0),this._ganttView._ganttViewCore.updateTask(e,this._convertMappedToCoreData(z,t))}},{key:"getDependencyData",value:function(e){if(!(0,v.isDefined)(e))return null;var t=this._ganttView._ganttViewCore.getDependencyData(e);return t?this._convertCoreToMappedData(N,t):null}},{key:"insertDependency",value:function(e){this._ganttView._ganttViewCore.insertDependency(this._convertMappedToCoreData(N,e))}},{key:"deleteDependency",value:function(e){this._ganttView._ganttViewCore.deleteDependency(e)}},{key:"getResourceData",value:function(e){var t=this._ganttView._ganttViewCore.getResourceData(e);return t?this._convertCoreToMappedData(j,t):null}},{key:"deleteResource",value:function(e){this._ganttView._ganttViewCore.deleteResource(e)}},{key:"insertResource",value:function(e,t){this._ganttView._ganttViewCore.insertResource(this._convertMappedToCoreData(j,e),t)}},{key:"getResourceAssignmentData",value:function(e){var t=this._ganttView._ganttViewCore.getResourceAssignmentData(e);return t?this._convertCoreToMappedData(W,t):null}},{key:"assignResourceToTask",value:function(e,t){this._ganttView._ganttViewCore.assignResourceToTask(e,t)}},{key:"unassignResourceFromTask",value:function(e,t){this._ganttView._ganttViewCore.unassignResourceFromTask(e,t)}},{key:"updateDimensions",value:function(){this._setInnerElementsWidth()}},{key:"_optionChanged",value:function(e){switch(e.name){case"tasks":this._refreshDataSource(z);break;case"dependencies":this._refreshDataSource(N);break;case"resources":this._refreshDataSource(j);break;case"resourceAssignments":this._refreshDataSource(W);break;case"columns":this._setTreeListOption("columns",this._getTreeListColumns());break;case"taskListWidth":this._setInnerElementsWidth();break;case"showResources":this._setGanttViewOption("showResources",e.value);break;case"taskTitlePosition":this._setGanttViewOption("taskTitlePosition",e.value);break;case"firstDayOfWeek":this._setGanttViewOption("firstDayOfWeek",e.value);break;case"selectedRowKey":this._setTreeListOption("selectedRowKeys",this._getArrayFromOneElement(e.value));break;case"onSelectionChanged":this._createSelectionChangedAction();break;case"onTaskClick":this._createTaskClickAction();break;case"onTaskDblClick":this._createTaskDblClickAction();break;case"onTaskInserting":this._createTaskInsertingAction();break;case"onTaskInserted":this._createTaskInsertedAction();break;case"onTaskDeleting":this._createTaskDeletingAction();break;case"onTaskDeleted":this._createTaskDeletedAction();break;case"onTaskUpdating":this._createTaskUpdatingAction();break;case"onTaskUpdated":this._createTaskUpdatedAction();break;case"onTaskMoving":this._createTaskMovingAction();break;case"onTaskEditDialogShowing":this._createTaskEditDialogShowingAction();break;case"onDependencyInserting":this._createDependencyInsertingAction();break;case"onDependencyInserted":this._createDependencyInsertedAction();break;case"onDependencyDeleting":this._createDependencyDeletingAction();break;case"onDependencyDeleted":this._createDependencyDeletedAction();break;case"onResourceInserting":this._createResourceInsertingAction();break;case"onResourceInserted":this._createResourceInsertedAction();break;case"onResourceDeleting":this._createResourceDeletingAction();break;case"onResourceDeleted":this._createResourceDeletedAction();break;case"onResourceAssigning":this._createResourceAssigningAction();break;case"onResourceAssigned":this._createResourceAssignedAction();break;case"onResourceUnassigning":this._createResourceUnassigningAction();break;case"onResourceUnassigned":this._createResourceUnassignedAction();break;case"onCustomCommand":this._createCustomCommandAction();break;case"onContextMenuPreparing":this._createContextMenuPreparingAction();break;case"allowSelection":this._setTreeListOption("selection.mode",this._getSelectionMode(e.value)),this._setGanttViewOption("allowSelection",e.value);break;case"showRowLines":this._setTreeListOption("showRowLines",e.value),this._setGanttViewOption("showRowLines",e.value);break;case"stripLines":this._setGanttViewOption("stripLines",e.value);break;case"scaleType":this._setGanttViewOption("scaleType",e.value);break;case"editing":this._setGanttViewOption("editing",this.option(e.name));break;case"validation":this._setGanttViewOption("validation",this.option(e.name));break;case"toolbar":this._updateToolbarContent();break;case"contextMenu":this._updateContextMenu();break;case"taskTooltipContentTemplate":this._setGanttViewOption("taskTooltipContentTemplate",this._getTaskTooltipContentTemplateFunc(e.value));break;case"rootValue":this._setTreeListOption("rootValue",e.value);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}}]),t}(y.default);(0,b.default)("dxGantt",K);var U=K;t.default=U,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t,n){return(l="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=u(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function u(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=_(e),null!==e););return e}function c(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&d(e,t)}function d(e,t){return(d=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function h(e){var t=g();return function(){var n,i=_(e);if(t){var o=_(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return f(this,n)}}function f(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?p(e):t}function p(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function g(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function _(e){return(_=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.GanttView=void 0;var m=o(n(2)),v=o(n(16)),y=n(897),b=n(899),x=o(n(32)),w=n(1),C=function(e){function t(){return a(this,t),n.apply(this,arguments)}c(t,e);var n=h(t);return s(t,[{key:"_init",value:function(){l(_(t.prototype),"_init",this).call(this),this._onSelectionChanged=this._createActionByOption("onSelectionChanged"),this._onScroll=this._createActionByOption("onScroll"),this._onDialogShowing=this._createActionByOption("onDialogShowing"),this._onPopupMenuShowing=this._createActionByOption("onPopupMenuShowing"),this._expandAll=this._createActionByOption("onExpandAll"),this._collapseAll=this._createActionByOption("onCollapseAll"),this._taskClick=this._createActionByOption("onTaskClick"),this._taskDblClick=this._createActionByOption("onTaskDblClick")}},{key:"_initMarkup",value:function(){var e=(0,y.getGanttViewCore)();this._ganttViewCore=new e(this.$element().get(0),this,{showResources:this.option("showResources"),taskTitlePosition:this._getTaskTitlePosition(this.option("taskTitlePosition")),firstDayOfWeek:this._getFirstDayOfWeek(this.option("firstDayOfWeek")),allowSelectTask:this.option("allowSelection"),editing:this._parseEditingSettings(this.option("editing")),validation:this.option("validation"),stripLines:{stripLines:this.option("stripLines")},areHorizontalBordersEnabled:this.option("showRowLines"),areAlternateRowsEnabled:!1,viewType:this._getViewTypeByScaleType(this.option("scaleType")),cultureInfo:this._getCultureInfo(),taskTooltipContentTemplate:this.option("taskTooltipContentTemplate")}),this._selectTask(this.option("selectedRowKey")),this.updateBarItemsState()}},{key:"_getFirstDayOfWeek",value:function(e){return(0,w.isDefined)(e)?e:x.default.firstDayOfWeekIndex()}},{key:"getTaskAreaContainer",value:function(){return this._ganttViewCore.taskAreaContainer}},{key:"getBarManager",value:function(){return this._ganttViewCore.barManager}},{key:"executeCoreCommand",value:function(e){var t=this._ganttViewCore.commandManager.getCommand(e);t&&t.execute()}},{key:"changeTaskExpanded",value:function(e,t){this._ganttViewCore.changeTaskExpanded(e,t)}},{key:"updateView",value:function(){this._ganttViewCore.updateView()}},{key:"updateBarItemsState",value:function(){this._ganttViewCore.barManager.updateItemsState([])}},{key:"setWidth",value:function(e){this._ganttViewCore.setWidth(e)}},{key:"_selectTask",value:function(e){this._ganttViewCore.selectTaskById(e)}},{key:"_update",value:function(){this._ganttViewCore.loadOptionsFromGanttOwner(),this._ganttViewCore.resetAndUpdate()}},{key:"_getCultureInfo",value:function(){return{monthNames:x.default.getMonthNames("wide"),dayNames:x.default.getDayNames("wide"),abbrMonthNames:x.default.getMonthNames("abbreviated"),abbrDayNames:x.default.getDayNames("abbreviated"),quarterNames:x.default.getQuarterNames(),amText:x.default.getPeriodNames()[0],pmText:x.default.getPeriodNames()[1]}}},{key:"_getTaskTitlePosition",value:function(e){switch(e){case"outside":return 1;case"none":return 2;default:return 0}}},{key:"_getViewTypeByScaleType",value:function(e){switch(e){case"minutes":return 0;case"hours":return 1;case"days":return 3;case"weeks":return 4;case"months":return 5;case"quarters":return 6;case"years":return 7;default:return}}},{key:"_parseEditingSettings",value:function(e){return{enabled:e.enabled,allowDependencyDelete:e.allowDependencyDeleting,allowDependencyInsert:e.allowDependencyAdding,allowTaskDelete:e.allowTaskDeleting,allowTaskInsert:e.allowTaskAdding,allowTaskUpdate:e.allowTaskUpdating,allowResourceDelete:e.allowResourceDeleting,allowResourceInsert:e.allowResourceAdding,allowResourceUpdate:e.allowResourceUpdating,allowTaskResourceUpdate:e.allowTaskResourceUpdating}}},{key:"_optionChanged",value:function(e){switch(e.name){case"width":l(_(t.prototype),"_optionChanged",this).call(this,e),this._ganttViewCore.setWidth(e.value);break;case"height":this._ganttViewCore.setHeight(e.value);break;case"tasks":case"dependencies":case"resources":case"resourceAssignments":this._update();break;case"showResources":this._ganttViewCore.setShowResources(e.value);break;case"taskTitlePosition":this._ganttViewCore.setTaskTitlePosition(this._getTaskTitlePosition(e.value));break;case"firstDayOfWeek":this._ganttViewCore.setFirstDayOfWeek(this._getFirstDayOfWeek(e.value));break;case"allowSelection":this._ganttViewCore.setAllowSelection(e.value);break;case"selectedRowKey":this._selectTask(e.value);break;case"editing":this._ganttViewCore.setEditingSettings(this._parseEditingSettings(e.value));break;case"validation":this._ganttViewCore.setValidationSettings(e.value),this._update();break;case"showRowLines":this._ganttViewCore.setRowLinesVisible(e.value);break;case"scaleType":this._ganttViewCore.setViewType(this._getViewTypeByScaleType(e.value));break;case"stripLines":this._ganttViewCore.setStripLines({stripLines:e.value});break;case"taskTooltipContentTemplate":this._ganttViewCore.setTaskTooltipContentTemplate(e.value);break;default:l(_(t.prototype),"_optionChanged",this).call(this,e)}}},{key:"getRowHeight",value:function(){return this.option("rowHeight")}},{key:"getHeaderHeight",value:function(){return this.option("headerHeight")}},{key:"getGanttTasksData",value:function(){return this.option("tasks")}},{key:"getGanttDependenciesData",value:function(){return this.option("dependencies")}},{key:"getGanttResourcesData",value:function(){return this.option("resources")}},{key:"getGanttResourceAssignmentsData",value:function(){return this.option("resourceAssignments")}},{key:"getGanttWorkTimeRules",value:function(){return{}}},{key:"getExternalTaskAreaContainer",value:function(e){return this._taskAreaContainer||(this._taskAreaContainer=new b.TaskAreaContainer(e,this)),this._taskAreaContainer}},{key:"changeGanttTaskSelection",value:function(e,t){this._onSelectionChanged({id:e,selected:t})}},{key:"onGanttScroll",value:function(e){this._onScroll({scrollTop:e})}},{key:"showDialog",value:function(e,t,n,i){this._onDialogShowing({name:e,parameters:t,callback:n,afterClosing:i})}},{key:"getModelChangesListener",value:function(){return this.option("modelChangesListener")}},{key:"showPopupMenu",value:function(e){this._onPopupMenuShowing(e)}},{key:"getMainElement",value:function(){return this.option("mainElement").get(0)}},{key:"adjustControl",value:function(){}},{key:"getRequireFirstLoadParentAutoCalc",value:function(){return this.option("validation.autoUpdateParentTasks")}},{key:"collapseAll",value:function(){this._collapseAll()}},{key:"expandAll",value:function(){this._expandAll()}},{key:"onTaskClick",value:function(e,t){return this._taskClick({key:e,event:t}),!0}},{key:"onTaskDblClick",value:function(e,t){return this._taskDblClick({key:e,event:t})}},{key:"onGanttViewContextMenu",value:function(e,t,n){return!0}},{key:"getFormattedDateText",value:function(e){var t="";if(e){var n=x.default.format(e,"shortDate"),i=x.default.format(e,"hh:mm");t=n+" "+i}return t}},{key:"destroyTemplate",value:function(e){(0,m.default)(e).empty()}},{key:"bars",get:function(){return this.option("bars")}}]),t}(v.default);t.GanttView=C},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){if(!r.default)throw a.default.Error("E1041","devexpress-gantt");return r.default}t.getGanttViewCore=o;var a=i(n(17)),r=i(n(898))},function(e,t){e.exports=window.DevExpress.Gantt},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function a(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t&&a(e.prototype,t),n&&a(e,n),e}t.TaskAreaContainer=void 0;var s=i(n(96)),l=function(){function e(t,n){o(this,e),this._element=t,this._scrollView=n._createComponent(this._element,s.default,{scrollByContent:!1,scrollByThumb:!0,showScrollbar:"onHover",direction:"both",onScroll:function(){n.updateView()}})}return r(e,[{key:"getWidth",value:function(){return this._element.offsetWidth}},{key:"getHeight",value:function(){return this._element.offsetHeight}},{key:"getElement",value:function(){return this._element}},{key:"scrollTop",get:function(){return this._scrollView.scrollTop()},set:function(e){var t=e-this._scrollView.scrollTop();0!==t&&this._scrollView.scrollBy({left:0,top:t})}},{key:"scrollLeft",get:function(){return this._scrollView.scrollLeft()},set:function(e){var t=e-this._scrollView.scrollLeft();0!==t&&this._scrollView.scrollBy({left:t,top:0})}},{key:"scrollWidth",get:function(){return this._scrollView.scrollWidth()}},{key:"scrollHeight",get:function(){return this._scrollView.scrollHeight()}},{key:"isExternal",get:function(){return!0}}]),e}();t.TaskAreaContainer=l},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t,n){return(a="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(e,t,n){var i=r(e,t);if(i){var o=Object.getOwnPropertyDescriptor(i,t);return o.get?o.get.call(n):o.value}})(e,t,n||e)}function r(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&(e=f(e),null!==e););return e}function s(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&l(e,t)}function l(e,t){return(l=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function u(e){var t=h();return function(){var n,i=f(e);if(t){var o=f(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return c(this,n)}}function c(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?d(e):t}function d(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function h(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function f(e){return(f=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function p(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function g(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function _(e,t,n){return t&&g(e.prototype,t),n&&g(e,n),e}t.GanttContextMenuBar=t.GanttToolbar=void 0;var m=o(n(2)),v=o(n(166)),y=o(n(110)),b=o(n(10)),x=n(0),w="dx-gantt-toolbar-separator",C={createTask:0,createSubTask:1,removeTask:2,removeDependency:3,taskInformation:4,taskAddContextItem:5,undo:6,redo:7,zoomIn:8,zoomOut:9,fullScreen:10,collapseAll:11,expandAll:12},k=function(){function e(t,n){p(this,e),this._element=t,this._owner=n,this._items=[],this._createControl()}return _(e,[{key:"createItems",value:function(e){this._cache=null,this._items=this._createItemsCore(e),this._menu.option("items",this._items)}},{key:"_createItemsCore",value:function(e){var t=this;return e.map(function(e){var n;return n="string"==typeof e?t._createItemByText(e):e.name?(0,x.extend)(t._createItemByText(e.name),e):(0,x.extend)(t._getDefaultItemOptions(),e),e.items&&(n.items=t._createItemsCore(e.items)),n})}},{key:"_createItemByText",value:function(e){switch(e.toLowerCase()){case"separator":return this._createSeparator();case"undo":return this._createDefaultItem(C.undo,b.default.format("dxGantt-undo"),this._getIcon("undo"));case"redo":return this._createDefaultItem(C.redo,b.default.format("dxGantt-redo"),this._getIcon("redo"));case"expandall":return this._createDefaultItem(C.expandAll,b.default.format("dxGantt-expandAll"),this._getIcon("expand"));case"collapseall":return this._createDefaultItem(C.collapseAll,b.default.format("dxGantt-collapseAll"),this._getIcon("collapse"));case"addtask":return this._createDefaultItem(C.createTask,b.default.format("dxGantt-addNewTask"),this._getIcon("add"));case"addsubtask":return this._createDefaultItem(C.createSubTask,b.default.format("dxGantt-contextMenuNewSubtask"),this._getIcon("add-sub-task"));case"deletetask":return this._createDefaultItem(C.removeTask,b.default.format("dxGantt-deleteSelectedTask"),this._getIcon("delete"));case"deletedependency":return this._createDefaultItem(C.removeDependency,b.default.format("dxGantt-contextMenuDeleteDependency"),this._getIcon("delete-dependency"));case"zoomin":return this._createDefaultItem(C.zoomIn,b.default.format("dxGantt-zoomIn"),this._getIcon("zoom-in"));case"zoomout":return this._createDefaultItem(C.zoomOut,b.default.format("dxGantt-zoomOut"),this._getIcon("zoom-out"));case"fullscreen":return this._createDefaultItem(C.fullScreen,b.default.format("dxGantt-fullScreen"),this._getIcon("full-screen"));case"taskdetails":return this._createDefaultItem(C.taskInformation,b.default.format("dxGantt-dialogTaskDetailsTitle")+"...",this._getIcon("task-details"));default:return(0,x.extend)(this._getDefaultItemOptions(),{options:{text:e}})}}},{key:"_getDefaultItemOptions",value:function(){return{}}},{key:"_getItemsCache",value:function(){return this._cache||(this._cache={},this._fillCache(this._items)),this._cache}},{key:"_fillCache",value:function(e){var t=this;e.forEach(function(e){var n=e.commandId;void 0!==n&&(t._cache[n]||(t._cache[n]=[]),t._cache[n].push(e)),e.items&&t._fillCache(e.items)})}},{key:"_getIcon",value:function(e){return"dx-gantt-i dx-gantt-i-"+e}},{key:"getCommandKeys",value:function(){var e=this._getItemsCache(),t=[];for(var n in e)t.push(parseInt(n));return t}},{key:"setItemEnabled",value:function(e,t){var n=this._getItemsCache();n[e].forEach(function(e){e.disabled=!t})}},{key:"setItemVisible",value:function(e,t){var n=this._getItemsCache();n[e].forEach(function(e){e.visible=t})}},{key:"setItemValue",value:function(e,t){}},{key:"setEnabled",value:function(e){this._menu.option("disabled",!e)}},{key:"updateItemsList",value:function(){}},{key:"isVisible",value:function(){return!0}},{key:"isContextMenu",value:function(){return!1}},{key:"completeUpdate",value:function(){}}]),e}(),S=function(e){function t(){return p(this,t),n.apply(this,arguments)}s(t,e);var n=u(t);return _(t,[{key:"_createControl",value:function(){var e=this;this._menu=this._owner._createComponent(this._element,v.default,{onItemClick:function(t){var n=t.itemData.commandId;void 0!==n&&e._owner._executeCoreCommand(t.itemData.commandId)}})}},{key:"_createDefaultItem",value:function(e,t,n){return{commandId:e,disabled:!0,widget:"dxButton",location:"before",options:{icon:n,stylingMode:"text",hint:t}}}},{key:"_createSeparator",value:function(){return{location:"before",template:function(e,t,n){(0,m.default)(n).addClass(w)}}}},{key:"_getDefaultItemOptions",value:function(){return{location:"before",widget:"dxButton"}}},{key:"completeUpdate",value:function(){this._menu.option("items",this._items)}}]),t}(k);t.GanttToolbar=S;var D=function(e){function t(){return p(this,t),n.apply(this,arguments)}s(t,e);var n=u(t);return _(t,[{key:"_createControl",value:function(){var e=this;this._menu=this._owner._createComponent(this._element,y.default,{showEvent:void 0,onItemClick:function(t){void 0!==t.itemData.commandId?e._owner._executeCoreCommand(t.itemData.commandId):void 0!==t.itemData.name&&e._owner._raiseCustomCommand(t.itemData.name)}})}},{key:"createItems",value:function(e){e&&0!==e.length||(e=this._getDefaultItems()),a(f(t.prototype),"createItems",this).call(this,e)}},{key:"_getDefaultItems",value:function(){return[{text:b.default.format("dxGantt-dialogButtonAdd"),commandId:C.taskAddContextItem,icon:this._getIcon("add"),items:[{text:b.default.format("dxGantt-contextMenuNewTask"),commandId:C.createTask,icon:this._getIcon("add-task")},{text:b.default.format("dxGantt-contextMenuNewSubtask"),commandId:C.createSubTask,icon:this._getIcon("add-sub-task")}]},{text:b.default.format("dxGantt-dialogTaskDetailsTitle")+"...",commandId:C.taskInformation,icon:this._getIcon("task-details")},{text:b.default.format("dxGantt-contextMenuDeleteTask"),commandId:C.removeTask,icon:this._getIcon("delete")},{text:b.default.format("dxGantt-contextMenuDeleteDependency"),commandId:C.removeDependency,icon:this._getIcon("delete-dependency")}]}},{key:"_createDefaultItem",value:function(e,t,n){return{commandId:e,text:t,icon:n}}},{key:"show",value:function(e,t){this._menu.option("items",t||this._items),this._menu.option("position.offset",{x:e.x,y:e.y}),this._menu.show()}},{key:"isContextMenu",value:function(){
return!0}}]),t}(k);t.GanttContextMenuBar=D},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function r(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function s(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}function l(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e){var t=f();return function(){var n,i=p(e);if(t){var o=p(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return d(this,n)}}function d(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?h(e):t}function h(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function f(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function p(e){return(p=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}t.default=void 0;var g=o(n(117)),_=o(n(94)),m=function(e){function t(e,i,o){var r;return a(this,t),r=n.call(this),r._optionName=e,r._loadPanel=i,r._dataSourceChangedCallback=o,r}l(t,e);var n=c(t);return s(t,[{key:"insert",value:function(e,t,n){var i=this;this._showLoadPanel(),this._getStore().insert(e).done(function(e){t&&t(e),i._hideLoadPanel()}).fail(function(e){n&&n(e),i._hideLoadPanel()})}},{key:"update",value:function(e,t,n,i){var o=this;this._showLoadPanel(),this._getStore().update(e,t).done(function(e,t){n&&n(e,t),o._hideLoadPanel()}).fail(function(e){i&&i(e),o._hideLoadPanel()})}},{key:"remove",value:function(e,t,n){var i=this;this._showLoadPanel(),this._getStore().remove(e).done(function(e){t&&t(e),i._hideLoadPanel()}).fail(function(e){n&&n(e),i._hideLoadPanel()})}},{key:"_dataSourceChangedHandler",value:function(e,t){this._dataSourceChangedCallback(this._optionName,e)}},{key:"_dataSourceOptions",value:function(){return{paginate:!1}}},{key:"_dataSourceLoadingChangedHandler",value:function(e){e&&!this._dataSource.isLoaded()?this._showLoadPanel():this._hideLoadPanel()}},{key:"_showLoadPanel",value:function(){this._loadPanel.show()}},{key:"_hideLoadPanel",value:function(){this._loadPanel.hide()}},{key:"_getStore",value:function(){return this._dataSource.store()}},{key:"_getItems",value:function(){return this._getStore()._array||this._dataSource.items()}}]),t}(g.default);m.include(_.default);var v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e){return e&&e.__esModule?e:{"default":e}}function a(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&r(e,t)}function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function s(e){var t=c();return function(){var n,i=d(e);if(t){var o=d(this).constructor;n=Reflect.construct(i,arguments,o)}else n=i.apply(this,arguments);return l(this,n)}}function l(e,t){return!t||"object"!==i(t)&&"function"!=typeof t?u(e):t}function u(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function c(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function h(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function f(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function p(e,t,n){return t&&f(e.prototype,t),n&&f(e,n),e}t.GanttDialog=void 0;var g=o(n(48)),_=o(n(146));n(301);var m=o(n(10)),v=function(){function e(t,n){h(this,e),this._popupInstance=t._createComponent(n,g.default),this.infoMap={TaskEdit:b,Resources:x,Confirmation:w,ConstraintViolation:C}}return p(e,[{key:"_apply",value:function(){var e=this._dialogInfo.getResult();this._callback(e),this.hide()}},{key:"show",value:function(e,t,n,i,o){this._callback=n,this._afterClosing=i,this.infoMap[e]&&(this._dialogInfo=new this.infoMap[e](t,this._apply.bind(this),this.hide.bind(this),o),this._popupInstance.option({showTitle:!!this._dialogInfo.getTitle(),title:this._dialogInfo.getTitle(),toolbarItems:this._dialogInfo.getToolbarItems(),maxWidth:this._dialogInfo.getMaxWidth(),height:this._dialogInfo.getHeight(),contentTemplate:this._dialogInfo.getContentTemplate()}),this._popupInstance.show())}},{key:"hide",value:function(){this._popupInstance.hide(),this._afterClosing&&this._afterClosing()}}]),e}();t.GanttDialog=v;var y=function(){function e(t,n,i,o){h(this,e),this._parameters=t,this._applyAction=n,this._hideAction=i,this._editingOptions=o}return p(e,[{key:"_getFormItems",value:function(){return{}}},{key:"_getFormCssClass",value:function(){return""}},{key:"_getFormData",value:function(){return this._parameters}},{key:"_updateParameters",value:function(){}},{key:"_getOkToolbarItem",value:function(){return this._getToolbarItem("OK",this._applyAction)}},{key:"_getCancelToolbarItem",value:function(){return this._getToolbarItem("Cancel",this._hideAction)}},{key:"_getYesToolbarItem",value:function(){return this._getToolbarItem("Yes",this._applyAction)}},{key:"_getNoToolbarItem",value:function(){return this._getToolbarItem("No",this._hideAction)}},{key:"_getToolbarItem",value:function(e,t){return{widget:"dxButton",toolbar:"bottom",options:{text:m.default.format(e),onClick:t}}}},{key:"getTitle",value:function(){return""}},{key:"getToolbarItems",value:function(){return this._editingOptions.enabled?[this._getOkToolbarItem(),this._getCancelToolbarItem()]:[this._getCancelToolbarItem()]}},{key:"getMaxWidth",value:function(){return 400}},{key:"getHeight",value:function(){return"auto"}},{key:"getContentTemplate",value:function(){var e=this;return function(t){return e._form=new _.default(t,{formData:e._getFormData(),items:e._getFormItems(),elementAttr:{"class":e._getFormCssClass()}}),t}}},{key:"getResult",value:function(){var e=this._form&&this._form.option("formData");return this._updateParameters(e),this._parameters}}]),e}(),b=function(e){function t(){return h(this,t),n.apply(this,arguments)}a(t,e);var n=s(t);return p(t,[{key:"getTitle",value:function(){return m.default.format("dxGantt-dialogTaskDetailsTitle")}},{key:"_getFormItems",value:function(){var e=this,t=!this._editingOptions.enabled||!this._editingOptions.allowTaskUpdating,n=t||!this._parameters.enableRangeEdit;return[{dataField:"title",editorType:"dxTextBox",label:{text:m.default.format("dxGantt-dialogTitle")},editorOptions:{readOnly:t||this._isReadOnlyField("title")},visible:!this._isHiddenField("title")},{dataField:"start",editorType:"dxDateBox",label:{text:m.default.format("dxGantt-dialogStartTitle")},editorOptions:{type:"datetime",width:"100%",readOnly:n||this._isReadOnlyField("start")},visible:!this._isHiddenField("start")},{dataField:"end",editorType:"dxDateBox",label:{text:m.default.format("dxGantt-dialogEndTitle")},editorOptions:{type:"datetime",width:"100%",readOnly:n||this._isReadOnlyField("end")},visible:!this._isHiddenField("end")},{dataField:"progress",editorType:"dxNumberBox",label:{text:m.default.format("dxGantt-dialogProgressTitle")},editorOptions:{showSpinButtons:!0,min:0,max:1,format:"#0%",step:.01,readOnly:n||this._isReadOnlyField("progress")},visible:!this._isHiddenField("progress")},{dataField:"assigned.items",editorType:"dxTagBox",label:{text:m.default.format("dxGantt-dialogResourcesTitle")},editorOptions:{readOnly:t||!this._editingOptions.allowTaskResourceUpdating,dataSource:this._parameters.resources.items,displayExpr:"text",buttons:[{name:"editResources",location:"after",options:{disabled:!this._editingOptions.allowResourceAdding&&!this._editingOptions.allowResourceDeleting,text:"...",hint:m.default.format("dxGantt-dialogEditResourceListHint"),onClick:function(){var t=function(){e._parameters.showTaskEditDialogCommand.execute()};e._parameters.showResourcesDialogCommand.execute(t)}}}]}}]}},{key:"_isReadOnlyField",value:function(e){return this._parameters.readOnlyFields.indexOf(e)>-1}},{key:"_isHiddenField",value:function(e){return this._parameters.hiddenFields.indexOf(e)>-1}},{key:"_getFormData",value:function(){var e={};for(var t in this._parameters)e[t]="progress"===t?this._parameters[t]/100:this._parameters[t];return e}},{key:"_updateParameters",value:function(e){this._parameters.title=e.title,this._parameters.start=e.start,this._parameters.end=e.end,this._parameters.progress=100*e.progress,this._parameters.assigned=e.assigned}}]),t}(y),x=function(e){function t(){return h(this,t),n.apply(this,arguments)}a(t,e);var n=s(t);return p(t,[{key:"getTitle",value:function(){return m.default.format("dxGantt-dialogResourceManagerTitle")}},{key:"_getFormItems",value:function(){var e=this;return[{label:{visible:!1},dataField:"resources.items",editorType:"dxList",editorOptions:{allowItemDeleting:this._editingOptions.enabled&&this._editingOptions.allowResourceDeleting,itemDeleteMode:"static",selectionMode:"none",items:this._parameters.resources.items,height:250,noDataText:m.default.format("dxGantt-dialogEditNoResources"),onInitialized:function(t){e.list=t.component},onItemDeleted:function(t){e._parameters.resources.remove(t.itemData)}}},{label:{visible:!1},editorType:"dxTextBox",editorOptions:{readOnly:!this._editingOptions.enabled||!this._editingOptions.allowResourceAdding,onInitialized:function(t){e.textBox=t.component},onInput:function(e){var t=e.component.getButton("addResource"),n=e.component.option("text");t.option("disabled",0===n.length)},buttons:[{name:"addResource",location:"after",options:{text:m.default.format("dxGantt-dialogButtonAdd"),disabled:!0,onClick:function(t){var n=e._parameters.resources.createItem();n.text=e.textBox.option("text"),e._parameters.resources.add(n),e.list.option("items",e._parameters.resources.items),e.list.scrollToItem(n),e.textBox.reset(),t.component.option("disabled",!0)}}}]}}]}}]),t}(y),w=function(e){function t(){return h(this,t),n.apply(this,arguments)}a(t,e);var n=s(t);return p(t,[{key:"getContentTemplate",value:function(){var e=this;return function(t){return e._getConfirmMessage()}}},{key:"_getConfirmMessage",value:function(){switch(this._parameters.type){case 0:return m.default.format("dxGantt-dialogTaskDeleteConfirmation");case 1:return m.default.format("dxGantt-dialogDependencyDeleteConfirmation");case 2:return m.default.format("dxGantt-dialogResourcesDeleteConfirmation",this._parameters.message);default:return""}}},{key:"getToolbarItems",value:function(){return[this._getYesToolbarItem(),this._getNoToolbarItem()]}}]),t}(y),C=function(e){function t(){return h(this,t),n.apply(this,arguments)}a(t,e);var n=s(t);return p(t,[{key:"_getFormItems",value:function(){var e=[];return e.push({text:m.default.format("dxGantt-dialogCancelOperationMessage"),value:0}),e.push({text:m.default.format("dxGantt-dialogDeleteDependencyMessage"),value:1}),this._parameters.validationError.critical||e.push({text:m.default.format("dxGantt-dialogMoveTaskAndKeepDependencyMessage"),value:2}),[{template:this._parameters.validationError.critical?m.default.format("dxGantt-dialogConstraintCriticalViolationMessage"):m.default.format("dxGantt-dialogConstraintViolationMessage")},{cssClass:"dx-cv-dialog-row",dataField:"option",label:{visible:!1},editorType:"dxRadioGroup",editorOptions:{items:e,valueExpr:"value",value:0}}]}},{key:"_getFormCssClass",value:function(){return"dx-cv-dialog"}},{key:"_updateParameters",value:function(e){this._parameters.option=e.option}}]),t}(y)},function(e,t,n){function i(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function a(e,t,n){return t&&o(e.prototype,t),n&&o(e,n),e}t.GanttDataCache=void 0;var r=n(0),s=function(){function e(){i(this,e),this._cache={},this._timers={}}return a(e,[{key:"saveData",value:function(e,t,n){if(t){this._clearTimer(e);var i=this._getCache(e,!0);(0,r.extendFromObject)(i,t,!0),n&&this._setExpireTimer(e,n)}}},{key:"pullDataFromCache",value:function(e,t){var n=this._getCache(e);n&&(0,r.extendFromObject)(t,n),this._onKeyExpired(e)}},{key:"hasData",value:function(e){return!!this._cache[e]}},{key:"_getCache",value:function(e,t){return!this._cache[e]&&t&&(this._cache[e]={}),this._cache[e]}},{key:"_setExpireTimer",value:function(e,t){var n=this;this._timers[e]=setTimeout(function(){t(e,n._getCache(e)),n._onKeyExpired(e)},200)}},{key:"_onKeyExpired",value:function(e){this._clearCache(e),this._clearTimer(e)}},{key:"_clearCache",value:function(e){delete this._cache[e]}},{key:"_clearTimer",value:function(e){var t=this._timers;t&&t[e]&&(clearTimeout(t[e]),delete t[e])}}]),e}();t.GanttDataCache=s},function(e,t,n){var i=n(180);n(262);var o=i.viz=n(905);o.currentTheme=n(57).currentTheme,o.registerTheme=n(57).registerTheme,o.exportFromMarkup=n(320).exportFromMarkup,o.getMarkup=n(320).getMarkup,o.exportWidgets=n(320).exportWidgets,o.currentPalette=n(201).currentPalette,o.getPalette=n(201).getPalette,o.generateColors=n(201).generateColors,o.registerPalette=n(201).registerPalette,o.refreshTheme=n(57).refreshTheme,o.dxChart=n(906),o.dxPieChart=n(942),o.dxPolarChart=n(943),o.dxLinearGauge=n(944),o.dxCircularGauge=n(950),o.dxBarGauge=n(953),o.dxRangeSelector=n(955),o.dxVectorMap=n(963),o.map={},o.map.sources={},o.map.projection=n(563).projection,o.dxSparkline=n(974),o.dxBullet=n(976),o.dxTreeMap=n(978),o.dxFunnel=n(989),o.dxSankey=n(997),o.BaseWidget=n(112),o.getTheme=n(57).getTheme,o.findTheme=n(57).getTheme,o.refreshAll=n(57).refreshTheme,o.refreshPaths=n(542).refreshPaths,o.gauges={__internals:{}},o._dashboard={},o._dashboard.Renderer=n(171).Renderer,o._dashboard.SvgElement=n(171).SvgElement,o._dashboard.patchFontOptions=n(14).patchFontOptions,e.exports=o},function(e,t,n){n(137),e.exports=DevExpress.viz=DevExpress.viz||{}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){for(var i,o=0;o<e.length;o++)if(e[o].pane===t||void 0===e[o].pane&&t===n){i=e[o].name;break}return i||(i=e[0].name),i}function a(e,t,n){var i=e.getOptions().grid,o=e.getOptions().minorGrid;i.visible=t,o&&(o.visible=n)}function r(e){var t=!1,n=[],i=e.some(function(e){var t=e.getOptions().minorGrid;return t&&t.visible}),o=e.some(function(e){var t=e.getOptions().grid;return t&&t.visible});e.length>1&&(e.forEach(function(e){var r=e.getOptions().grid;t?a(e,!1,!1):r&&r.visible&&(e.getTranslator().getBusinessRange().isEmpty()?(a(e,!1,!1),n.push(e)):(t=!0,a(e,o,i)))}),!t&&n.length&&a(n[0],o,i))}function s(e,t,n){var i,o;for(o=0;o<t.length;o++)if(t[o].name===n){i=t[o],i.priority=o;break}if(!i)for(o=0;o<e.length;o++)if(e[o].name===n){i=e[o].getOptions(),i.priority=e[o].priority;break}return i}function l(e,t,n){var i,o;for(o=0;o<n.length;o++)if(i=n[o],i.name===t&&i.pane===e)return i;if(e)return l(void 0,t,n)}function u(e,t){return e.priority-t.priority}function c(e,t){var n=!1;return(0,A.each)(e,function(e,i){if(i.name===t)return n=!0,!1}),n}function d(e,t,n,i){var o=t[e]||0,a=n[e]||0;return o+a+(o&&a?i:0)}function h(e,t,n){return f(t[e],n[e])}function f(e,t){return Math.max(e||0,t||0)}function p(e){return e.getMargins()}function g(e,t){return e.reduce(function(e,n){var i=t(n),o=e.panes[n.pane]=e.panes[n.pane]||{},a=n.getMultipleAxesSpacing();return o.top=d("top",o,i,a),o.bottom=d("bottom",o,i,a),o.left=h("left",o,i),o.right=h("right",o,i),e.top=h("top",o,e),e.bottom=h("bottom",o,e),e.left=h("left",o,e),e.right=h("right",o,e),e},{panes:{}})}function _(e){return e.reduce(function(e,t){var n=t.getMargins(),i=e.panes[t.pane]=e.panes[t.pane]||{},o=t.getMultipleAxesSpacing();return i.top=h("top",i,n),i.bottom=h("bottom",i,n),i.left=d("left",i,n,o),i.right=d("right",i,n,o),e.top=h("top",i,e),e.bottom=h("bottom",i,e),e.left=h("left",i,e),e.right=h("right",i,e),e},{panes:{}})}function m(e,t,n,i,o){e.forEach(function(e){e[t](n&&n[e.pane],i&&i[e.pane]||i,o)})}function v(e,t,n,i,o){function a(t,n,i){var o=(e?["left","right"]:["top","bottom"]).indexOf(t)===-1?n:n.panes[i]||{};return o[t]}function r(e,t,n,i){return f(a(e,t,i),a(e,n,i))}function s(e,t,a,s,u,c){t=t.sort(function(t,n){return e[n][s]-e[t][s]}),t.forEach(function(t){var n=e[t];c.forEach(function(e){n[e]=n[l(e)]+r(e,i,o,t)})});var d=e[t[0]],h=t.reduce(function(e,t){return e-=r(s,i,o,t)+r(u,i,o,t)},d[a]-d[l(u)]-e[t[t.length-1]][l(s)])-M.PANE_PADDING*(t.length-1),f=Object.keys(n).reduce(function(e,t){return e+(n[t].unit?n[t].height:0)},0);h-=f,t.reduce(function(t,l){var c=e[l],f=n[l];return t-=r(u,i,o,l),c[u]=d[a]-t,t-=f.unit?f.height:Math.floor(h*f.height),c[s]=t,t-=r(s,i,o,l)+M.PANE_PADDING},d[a]-d[l(u)]-(h<0?h:0))}var l=function(e){return"original".concat(e[0].toUpperCase()).concat(e.slice(1))},u=Object.keys(t);return e?s(t,u,"width","left","right",["top","bottom"]):s(t,u,"height","top","bottom",["left","right"]),t}function y(e,t,n,i){t&&(m(e,"createTicks",n),L.default.synchronize(e)),m(e,"draw",!t&&n,i)}function b(e,t){var n={};return function(i){if(!i.customPositionIsAvailable()||i.customPositionEqualsToPredefined()){var o=n[i.pane]=n[i.pane]||{top:0,left:0,bottom:0,right:0},a=i.getMultipleAxesSpacing(),r=i.getMargins();i.shift(o),o[e]=d(e,o,r,a),o[t]=d(t,o,r,a)}else i.shift({top:0,left:0,bottom:0,right:0})}}function x(e,t){var n,i,o=0;for(n in t.panes)i=t.panes[n],o+="height"===e?i.top+i.bottom:i.left+i.right;return o}function w(e,t,n,i){var o=0;e[t]>0&&(o=x(t,i(n,p)),m(n,"hideTitle"),e[t]-=o-x(t,i(n,p))),e[t]>0&&m(n,"hideOuterElements")}function C(e,t){var n=t.reduce(function(e,t){return e+t},0)/t.length;return e.animate&&n<=e.animationPointsLimit}function k(e,t,n){var i=[],o={},a=n.visualRange(),r=n.getTranslator(),s=j.default.getViewPortFilter(a||{});return t.forEach(function(n){var a=n.getValueAxis(),l=a.getCanvasRange(),u=a.getTranslator(),c=e.indexOf(n),d=j.default.getViewPortFilter(l||{});o[c]={},t.forEach(function(t){return o[c][e.indexOf(t)]=0});var h=[];n.getPoints().filter(function(e){return e.getOptions().visible&&s(e.argument)&&(d(e.getMinValue(!0))||d(e.getMaxValue(!0)))}).forEach(function(e){var t={seriesIndex:c,argument:e.argument,value:e.getMaxValue(!0),size:e.bubbleSize||e.getOptions().size};if(e.getMinValue(!0)!==e.getMaxValue(!0)){var n=(0,O.extend)({},t);n.value=e.getMinValue(!0),n.x=r.to(n.argument,1),n.y=u.to(n.value,1),h.push(n)}t.x=r.to(t.argument,1),t.y=u.to(t.value,1),h.push(t)}),o[c].pointsCount=h.length,o[c].total=0,o[c].continuousSeries=0,i.push({name:n.name,index:c,points:h})}),{series:i,overloadedSeries:o}}function S(e,t,n,i){var o=i.getOptions().type;t.forEach(function(t){var a=e.indexOf(t);t.autoHidePointMarkers=!1;var r=i.getTicksValues().majorTicksValues.length;if(t.autoHidePointMarkersEnabled()&&(o===X||n[a].pointsCount>r))for(var s in n[a]){var l=parseInt(s);if(!(isNaN(l)||n[a].total/n[a].continuousSeries<3))if(l===a){if(2*n[l][l]>=n[l].pointsCount){t.autoHidePointMarkers=!0;break}}else if(n[a].total>=n[a].pointsCount){t.autoHidePointMarkers=!0;break}}})}function D(e,t,n){for(var i=e.width*e.height,o=t.series,a=function(e){var a=n.filter(function(t){return t.name===o[e].name})[0],r=o[e].points,s=r.length?r[0].size:0,l=s*s*r.length;if(a.autoHidePointMarkersEnabled()&&l>=i/o.length){var u=o[e].index;a.autoHidePointMarkers=!0,o.splice(e,1),n.splice(n.indexOf(a),1),delete t.overloadedSeries[u]}},r=o.length-1;r>=0;r--)a(r)}function I(e,t){for(var n=!1,i=0;i<e.length-1;i++){var o=e[i],a=o.size;if((0,B.isDefined)(o.x)&&(0,B.isDefined)(o.y))for(var r=i+1;r<e.length;r++){var s=e[r],l=null===s||void 0===s?void 0:s.x,u=null===s||void 0===s?void 0:s.y;if(!(0,B.isDefined)(l)||Math.abs(o.x-l)>=a){n&=r!==i+1;break}var c=(0,B.isDefined)(l)&&(0,B.isDefined)(u)&&Math.sqrt(Math.pow(o.x-l,2)+Math.pow(o.y-u,2));c&&c<a&&(t[o.seriesIndex][s.seriesIndex]++,t[o.seriesIndex].total++,n||(t[o.seriesIndex].continuousSeries++,n=!0))}}}t.default=void 0;var T=n(4),O=n(0),E=n(12),A=n(3),P=i(n(9)),R=n(542),M=n(14),B=n(1),V=n(29),F=n(321),L=i(n(930)),H=n(550),z=n(939),N=n(556),j=i(n(230)),W=n(548),G=n(134),q=i(n(940)),$=i(n(941)),K=n(233),U="default",Y="VISUAL_RANGE",Z=[{name:U,border:{}}],X="discrete",Q=Array.isArray,J=H.AdvancedChart.inherit({_themeSection:"chart",_fontFields:["crosshair.label.font"],_initCore:function(){this.paneAxis={},this.callBase()},_correctAxes:function(){this._correctValueAxes(!0)},_getExtraOptions:T.noop,_createPanes:function(){var e,t=this,n=t.option("panes"),i=0;return(!n||Q(n)&&!n.length)&&(n=Z),t.callBase(),e=t.option("defaultPane"),n=(0,O.extend)(!0,[],Q(n)?n:[n]),(0,A.each)(n,function(e,t){t.name=(0,B.isDefined)(t.name)?t.name:U+i++}),(0,B.isDefined)(e)?c(n,e)||(t._incidentOccurred("W2101",[e]),e=n[n.length-1].name):e=n[n.length-1].name,t.defaultPane=e,n=t._isRotated()?n.reverse():n},_getAxisRenderingOptions:function(){return{axisType:"xyAxes",drawingType:"linear"}},_prepareAxisOptions:function(e,t,n){return{isHorizontal:"argumentAxis"===e!==n,containerColor:this._themeManager.getOptions("containerBackgroundColor")}},_checkPaneName:function(e){var t=(0,M.map)(this.panes,function(e){return e.name});return e.pane=e.pane||this.defaultPane,(0,E.inArray)(e.pane,t)!==-1},_initCustomPositioningAxes:function(){var e=this,t=e.getArgumentAxis(),n=t.getOptions().customPositionAxis,i=e._valueAxes.filter(function(e){return e.pane===t.pane&&(!n||n===e.name)})[0];e._valueAxes.forEach(function(e){t!==e.getOrthogonalAxis()&&(e.getOrthogonalAxis=function(){return t},e.customPositionIsBoundaryOrthogonalAxis=function(){return t.customPositionIsBoundary()})}),(0,B.isDefined)(i)&&i!==t.getOrthogonalAxis()?(t.getOrthogonalAxis=function(){return i},t.customPositionIsBoundaryOrthogonalAxis=function(){return e._valueAxes.some(function(e){return e.customPositionIsBoundary()})}):(0,B.isDefined)(t.getOrthogonalAxis())&&!(0,B.isDefined)(i)&&(t.getOrthogonalAxis=T.noop)},_getAllAxes:function(){return this._argumentAxes.concat(this._valueAxes)},_resetAxesAnimation:function(e){this._getAllAxes().forEach(function(t){t.resetApplyingAnimation(e)})},_cleanGroups:function(){this._getAllAxes().forEach(function(e){return e.beforeCleanGroups()}),this.callBase(),this._getAllAxes().forEach(function(e){return e.afterCleanGroups()})},_axesBoundaryPositioning:function(){var e=this,t=e._getAllAxes(),n=!1;return t.forEach(function(e){if(!e.customPositionIsAvailable())return!1;var t=e.customPositionIsBoundary();e._customBoundaryPosition=e.getCustomBoundaryPosition(),n|=t!==e.customPositionIsBoundary()}),n},_getValueAxis:function(e,t){var n,i,a=this,r=a._valueAxes,u=a.option("valueAxis")||{},c=Q(u)?u:[u],d=a._isRotated(),h=a._getCrosshairMargins();return t=t||o(r,e,a.defaultPane),i=l(e,t,r),i||(n=s(r,c,t),n||(a._incidentOccurred("W2102",[t]),n={name:t,priority:r.length}),i=a._createAxis(!1,a._populateAxesOptions("valueAxis",n,{pane:e,name:t,optionPath:Q(u)?"valueAxis[".concat(n.priority,"]"):"valueAxis",crosshairMargin:d?h.y:h.x},d)),i.applyVisualRangeSetter(a._getVisualRangeSetter()),r.push(i)),i.setPane(e),i},_correctValueAxes:function(e){var t=this,n=t._themeManager.getOptions("synchronizeMultiAxes"),i=t._valueAxes,o={};t.series.forEach(function(e){var t=e.getValueAxis();o[t.pane]=!0}),t.panes.forEach(function(a){var s=a.name;o[s]||t._getValueAxis(s),e&&n&&r(i.filter(function(e){return e.pane===s}))}),t._valueAxes=i.filter(function(e){return e.pane||e.setPane(t.defaultPane),c(t.panes,e.pane)}).sort(u);var a=this.getValueAxis();t._valueAxes.forEach(function(e){var n=e.getOptions().optionPath;if(n){var i=t._valueAxes.filter(function(e){return e.getOptions().optionPath===n});i.length>1&&(i.some(function(e){return e===a})?i.forEach(function(e){e!==a&&(e.getOptions().optionPath=null)}):i.forEach(function(e,t){0!==t&&(e.getOptions().optionPath=null)}))}})},_getSeriesForPane:function(e){var t=[];return(0,A.each)(this.series,function(n,i){i.pane===e&&t.push(i)}),t},_createPanesBorderOptions:function(){var e=this._themeManager.getOptions("commonPaneSettings").border,t={};return this.panes.forEach(function(n){return t[n.name]=(0,O.extend)(!0,{},e,n.border)}),t},_createScrollBar:function(){var e=this,t=e._themeManager.getOptions("scrollBar")||{},n=e._scrollBarGroup;t.visible?(t.rotated=e._isRotated(),e._scrollBar=(e._scrollBar||new z.ScrollBar(e._renderer,n)).update(t)):(n.linkRemove(),e._scrollBar&&e._scrollBar.dispose(),e._scrollBar=null)},_prepareToRender:function(e){var t=this._createPanesBorderOptions();return this._createPanesBackground(),this._appendAxesGroups(),this._adjustViewport(),t},_adjustViewport:function(){var e=this,t=e._getVisibleSeries(),n=e.getArgumentAxis(),i=t.some(function(e){return e.useAggregation()}),o=e._themeManager.getOptions("adjustOnZoom"),a=!n.dataVisualRangeIsReduced();(i||o)&&e._valueAxes.forEach(function(e){return e.adjust(a)})},_recreateSizeDependentObjects:function(e){var t=this,n=t._getVisibleSeries(),i=n.some(function(e){return e.useAggregation()}),o=t._isZooming();i&&(t._argumentAxes.forEach(function(e){e.updateCanvas(t._canvas,!0)}),n.forEach(function(t){t.useAggregation()&&(e||o||!t._useAllAggregatedPoints)&&t.createPoints()}),t._processSeriesFamilies())},_isZooming:function(){var e=this,t=e.getArgumentAxis();if(!t||!t.getTranslator())return!1;var n=t.getTranslator().getBusinessRange(),i=t.getViewport(),o=i?i.min:0,a=i?i.max:0;"logarithmic"===n.axisType&&(o=(0,M.getLog)(o,n.base),a=(0,M.getLog)(a,n.base));var r=n.axisType===X?(0,M.getCategoriesInfo)(n.categories,o,a).categories.length:Math.abs(a-o),s=(0,V.getPrecision)(r);s=s>1?Math.pow(10,s-2):1;var l=Math.round((e._zoomLength-r)*s)/s!==0;return e._zoomLength=r,l},_handleSeriesDataUpdated:function(){var e=this,t=new G.Range;e.series.forEach(function(e){t.addRange(e.getArgumentRange())}),e._argumentAxes.forEach(function(n){n.updateCanvas(e._canvas,!0),n.setBusinessRange(t,e._axesReinitialized)}),e.callBase()},_isLegendInside:function(){return this._legend&&"inside"===this._legend.getPosition()},_isRotated:function(){return this._themeManager.getOptions("rotated")},_getLayoutTargets:function(){return this.panes},_applyClipRects:function(e){var t=this;t._drawPanesBorders(e),t._createClipRectsForPanes(),t._applyClipRectsForAxes(),t._fillPanesBackground()},_updateLegendPosition:function(e,t){var n=this;if(e.drawLegend&&n._legend&&t){var i=n.panes,o=(0,O.extend)({},i[0].canvas),a=new W.LayoutManager;o.right=i[i.length-1].canvas.right,o.bottom=i[i.length-1].canvas.bottom,a.layoutInsideLegend(n._legend,o)}},_allowLegendInsidePosition:function(){return!0},_applyExtraSettings:function(e){var t=this,n=t._getPaneIndex(e.pane),i=t._panesClipRects,o=i.wide[n];e.setClippingParams(i.base[n].id,o&&o.id,t._getPaneBorderVisibility(n))},_updatePanesCanvases:function(e){e.recreateCanvas&&(0,M.updatePanesCanvases)(this.panes,this._canvas,this._isRotated())},_normalizePanesHeight:function(){(0,M.normalizePanesHeight)(this.panes)},_renderScaleBreaks:function(){this._valueAxes.concat(this._argumentAxes).forEach(function(e){e.drawScaleBreaks()})},_getArgFilter:function(){return j.default.getViewPortFilter(this.getArgumentAxis().visualRange()||{})},_applyPointMarkersAutoHiding:function(){var e=this,t=e.series;return e._themeManager.getOptions("autoHidePointMarkers")?void e.panes.forEach(function(n){var i=n.borderCoords,o=n.name,a=t.filter(function(e){return e.pane===o&&e.usePointsToDefineAutoHiding()}),r=e.getArgumentAxis(),s=k(t,a,r);if(D(i,s,a),s.series.length){var l=r.visualRange(),u=r.getOptions().type===X,c=u?function(e,t){return l.categories.indexOf(e.argument)-l.categories.indexOf(t.argument)}:function(e,t){return e.argument-t.argument},d=[];s.series.forEach(function(e){return d=d.concat(e.points)}),d.sort(c),I(d,s.overloadedSeries),S(t,a,s.overloadedSeries,r)}}):void t.forEach(function(e){return e.autoHidePointMarkers=!1})},_renderAxes:function(e,t){function n(e){return e.map(function(e){if(!e.getTitle)return 0;var t=e.getTitle();return t?t.bBox.width:0})}var i=this,o=i._isRotated(),a=i._themeManager.getOptions("synchronizeMultiAxes"),r=i._scrollBar?[i._scrollBar]:[],s=i._isArgumentAxisBeforeScrollBar()?i._argumentAxes.concat(r):r.concat(i._argumentAxes),l=o?i._argumentAxes:i._valueAxes,u=o?s:i._valueAxes,c=o?i._valueAxes:i._argumentAxes,d=o?i._valueAxes:s,h=l.concat(c),f=h.concat(r);i._normalizePanesHeight(),i._updatePanesCanvases(e);var x=i.panes.reduce(function(e,t){return e[t.name]=(0,O.extend)({},t.canvas),e},{}),w=i.panes.reduce(function(e,t){return e[t.name]={height:t.height,unit:t.unit},e},{}),k=(0,O.extend)(!0,{},x);i._initCustomPositioningAxes();var S=i._axesBoundaryPositioning();if(!e.adjustAxes&&!S)return y(l,!o&&a,x,t),y(c,o&&a,x,t),m(h,"prepareAnimation"),i._renderScaleBreaks(),c.forEach(function(e){return e.resolveOverlappingForCustomPositioning(l)}),l.forEach(function(e){return e.resolveOverlappingForCustomPositioning(c)}),!1;S&&h.forEach(function(e){return e.customPositionIsAvailable()&&e.shift({top:0,left:0,bottom:0,right:0})}),i._scrollBar&&i._scrollBar.setPane(i.panes);var D={panes:{}},I=g(d,function(e){return e.estimateMargins(x[e.pane])});x=v(o,x,w,D,I);var T=function(e){var n=e?c:l,i=(e?o:!o)&&a;y(n,i,x,t),e?I=g(d,p):D=_(u),x=v(o,x,w,D,I)};T(!1),T(!0),!i._changesApplying&&i._estimateTickIntervals(l,x)&&T(!1);var E=n(l),A=i._getVisibleSeries(),P=i._getPointsToAnimation(A);return m(f,"updateSize",x,C(e,P)),d.forEach(b("top","bottom")),u.forEach(b("left","right")),i._renderScaleBreaks(),i.panes.forEach(function(e){(0,O.extend)(e.canvas,x[e.name])}),i._valueAxes.forEach(function(e){e.setInitRange()}),l.forEach(function(e,t){if(e.hasWrap&&e.hasWrap()){var a=e.getTitle(),r=a?a.bBox.width:0,s=r-E[t];"right"===e.getOptions().position?D.right+=s:(D.left+=s,i.panes.forEach(function(e){var t=e.name;return D.panes[t].left+=s})),x=v(o,x,w,D,I),m(f,"updateSize",x,!1,!1),E=n(l)}}),l.some(function(e){return e.customPositionIsAvailable()&&e.getCustomPosition()!==e._axisPosition})&&m(l,"updateSize",x,!1),c.forEach(function(e){return e.resolveOverlappingForCustomPositioning(l)}),l.forEach(function(e){return e.resolveOverlappingForCustomPositioning(c)}),k},_getExtraTemplatesItems:function(){var e=this,t=(e._argumentAxes||[]).concat(e._valueAxes||[]),n=e._collectTemplatesFromItems(t);return{items:n.items,groups:n.groups,launchRequest:function(){t.forEach(function(e){e.setRenderedState(!0)})},doneRequest:function(){t.forEach(function(e){e.setRenderedState(!1)})}}},_estimateTickIntervals:function(e,t){return e.some(function(e){return e.estimateTickInterval(t[e.pane])})},checkForMoreSpaceForPanesCanvas:function(){var e=this,t=e._isRotated(),n=e.panes.filter(function(e){return e.unit}).length===e.panes.length,i=!1;if(n){var o=0,a=0;if(t){var r=e.getArgumentAxis().getMargins().right,s=Math.min.apply(Math,e.panes.map(function(e){
return e.canvas.right}));o=e._canvas.right+r-s}else{var l=e.getArgumentAxis().getMargins().bottom,u=Math.min.apply(Math,e.panes.map(function(e){return e.canvas.bottom}));a=e._canvas.bottom+l-u}if(i=(o>0||a>0)&&{width:o,height:a},0!==a){var c=e.getSize(),d=e.option("size"),h=e._$element[0],f=!!h.style.height||(0,M.checkElementHasPropertyFromStyleSheet)(h,"height");t||d&&d.height||f||(e._forceResize(c.width,c.height+a),i=!1)}}else i=e.layoutManager.needMoreSpaceForPanesCanvas(e._getLayoutTargets(),t,function(e){return{width:t&&!!e.unit,height:!t&&!!e.unit}});return i},_forceResize:function(e,t){this._renderer.resize(e,t),this._updateSize(),this._setContentSize(),this._preserveOriginalCanvas(),this._updateCanvasClipRect(this._canvas)},_shrinkAxes:function(e,t){if(e&&t){this._renderer.stopAllAnimations(!0);var n=this,i=n._isRotated(),o=n._scrollBar?[n._scrollBar]:[],a=n._isArgumentAxisBeforeScrollBar()?n._argumentAxes.concat(o):o.concat(n._argumentAxes),r=i?a:n._valueAxes,s=i?n._valueAxes:a,l=r.concat(s);if(e.width||e.height){w(e,"height",s,g),w(e,"width",r,_),m(l,"updateSize",t);var u=n.panes.reduce(function(e,t){return e[t.name]={height:t.height,unit:t.unit},e},{});t=v(i,t,u,_(r),g(s,p)),m(l,"updateSize",t),s.forEach(b("top","bottom")),r.forEach(b("left","right")),n.panes.forEach(function(e){return(0,O.extend)(e.canvas,t[e.name])})}}},_isArgumentAxisBeforeScrollBar:function(){var e=this,t=e.getArgumentAxis();if(e._scrollBar){var n,i=t.getResolvedBoundaryPosition(),o=null===(n=t.getOptions().label)||void 0===n?void 0:n.position,a=e._scrollBar.getOptions().position;return t.hasCustomPosition()||a===i&&o!==a}return!1},_getPanesParameters:function(){var e,t=this,n=t.panes,i=[];for(e=0;e<n.length;e++)t._getPaneBorderVisibility(e)&&i.push({coords:n[e].borderCoords,clipRect:t._panesClipRects.fixed[e]});return i},_createCrosshairCursor:function(){var e=this,t=e._themeManager.getOptions("crosshair")||{},n=e.getArgumentAxis(),i=e._isRotated()?[e._valueAxes,[n]]:[[n],e._valueAxes],o={canvas:e._getCommonCanvas(),panes:e._getPanesParameters(),axes:i};t&&t.enabled&&(e._crosshair?e._crosshair.update(t,o):e._crosshair=new N.Crosshair(e._renderer,t,o,e._crosshairCursorGroup),e._crosshair.render())},_getCommonCanvas:function(){var e,t,n,i=this.panes;for(e=0;e<i.length;e++)t=i[e].canvas,n?(n.right=t.right,n.bottom=t.bottom):n=(0,O.extend)({},t);return n},_createPanesBackground:function(){var e,t,n,i=this,o=i._themeManager.getOptions("commonPaneSettings").backgroundColor,a=i._renderer,r=[];for(i._panesBackgroundGroup.clear(),n=0;n<i.panes.length;n++)e=i.panes[n].backgroundColor||o,e&&"none"!==e?(t=a.rect(0,0,0,0).attr({fill:e,"stroke-width":0}).append(i._panesBackgroundGroup),r.push(t)):r.push(null);i.panesBackground=r},_fillPanesBackground:function(){var e,t=this;(0,A.each)(t.panes,function(n,i){e=i.borderCoords,null!==t.panesBackground[n]&&t.panesBackground[n].attr({x:e.left,y:e.top,width:e.width,height:e.height})})},_calcPaneBorderCoords:function(e){var t=e.canvas,n=e.borderCoords=e.borderCoords||{};n.left=t.left,n.top=t.top,n.right=t.width-t.right,n.bottom=t.height-t.bottom,n.width=Math.max(n.right-n.left,0),n.height=Math.max(n.bottom-n.top,0)},_drawPanesBorders:function(e){var t=this,n=t._isRotated();t._panesBorderGroup.linkRemove().clear(),(0,A.each)(t.panes,function(i,o){var a=e[o.name],r={fill:"none",stroke:a.color,"stroke-opacity":a.opacity,"stroke-width":a.width,dashStyle:a.dashStyle,"stroke-linecap":"square"};if(t._calcPaneBorderCoords(o,n),a.visible){var s=o.borderCoords,l=(0,R.prepareSegmentRectPoints)(s.left,s.top,s.width,s.height,a);t._renderer.path(l.points,l.pathType).attr(r).append(t._panesBorderGroup)}}),t._panesBorderGroup.linkAppend()},_createClipRect:function(e,t,n,i,o,a){var r=this,s=e[t];s?s.attr({x:n,y:i,width:o,height:a}):(s=r._renderer.clipRect(n,i,o,a),e[t]=s)},_createClipRectsForPanes:function(){var e=this,t=e._canvas;(0,A.each)(e.panes,function(n,i){var o=!1,a=i.borderCoords,r=a.left,s=a.top,l=a.width,u=a.height,c=e._panesClipRects;e._createClipRect(c.fixed,n,r,s,l,u),e._createClipRect(c.base,n,r,s,l,u),(0,A.each)(e.series,function(e,t){t.pane===i.name&&(t.isFinancialSeries()||t.areErrorBarsVisible())&&(o=!0)}),o?(e._isRotated()?(s=0,u=t.height):(r=0,l=t.width),e._createClipRect(c.wide,n,r,s,l,u)):c.wide[n]=null})},_applyClipRectsForAxes:function(){for(var e=this,t=e._getAllAxes(),n=t.filter(function(e){return e.hasCustomPosition()}),i=e._getCanvasClipRectID(),o=0;o<t.length;o++){var a=e._getElementsClipRectID(t[o].pane),r=n.indexOf(t[o])>=0?a:i;t[o].applyClipRects(a,r)}},_getPaneBorderVisibility:function(e){var t=this._themeManager.getOptions("commonPaneSettings").border.visible,n=this.panes[e]||{},i=n.border||{};return"visible"in i?i.visible:t},_getCanvasForPane:function(e){var t,n=this.panes,i=n.length;for(t=0;t<i;t++)if(n[t].name===e)return n[t].canvas},_getTrackerSettings:function(){var e=this,t=e._themeManager;return(0,O.extend)(this.callBase(),{chart:e,rotated:e._isRotated(),crosshair:e._getCrosshairOptions().enabled?e._crosshair:null,stickyHovering:t.getOptions("stickyHovering")})},_resolveLabelOverlappingStack:function(){var e=this,t=e._isRotated(),n=t?function(e,t){return{x:e.x-t,y:e.y}}:function(e,t){return{x:e.x,y:e.y-t}};(0,A.each)(e._getStackPoints(),function(i,o){(0,A.each)(o,function(i,o){F.overlapping.resolveLabelOverlappingInOneDirection(o,e._getCommonCanvas(),t,n,function(e,n){var i=t?1:0,o=e.labels[0].getFigureCenter()[i],a=n.labels[0].getFigureCenter()[i];return o-a===0?(e.value()-n.value())*(e.labels[0].getPoint().series.getValueAxis().getTranslator().isInverted()?-1:1):0})})})},_getStackPoints:function(){var e={},t=this._getVisibleSeries();return(0,A.each)(t,function(t,n){var i=n.getPoints(),o=n.getStackName()||null;(0,A.each)(i,function(t,n){var i=n.argument;e[i]||(e[i]={}),e[i][o]||(e[i][o]=[]),e[i][o].push(n)})}),e},_getCrosshairOptions:function(){return this._getOption("crosshair")},zoomArgument:function(e,t){var n=this;n._initialized&&((0,B.isDefined)(e)||(0,B.isDefined)(t))&&n.getArgumentAxis().visualRange([e,t])},resetVisualRange:function(){var e=this,t=e._argumentAxes,n=e.getArgumentAxis();t.forEach(function(t){t.resetVisualRange(n!==t),e._applyCustomVisualRangeOption(t)}),e.callBase()},getVisibleArgumentBounds:function(){var e=this._argumentAxes[0].getTranslator(),t=e.getBusinessRange(),n=t.axisType===X,i=t.categories;return{minVisible:n?t.minVisible||i[0]:t.minVisible,maxVisible:n?t.maxVisible||i[i.length-1]:t.maxVisible}},_change_FULL_RENDER:function(){this.callBase(),this._changes.has(Y)&&this._raiseZoomEndHandlers()},_getAxesForScaling:function(){return[this.getArgumentAxis()].concat(this._valueAxes)},_applyVisualRangeByVirtualAxes:function(e,t){var n=this;if(e.isArgumentAxis){if(e!==n.getArgumentAxis())return!0;n._argumentAxes.filter(function(t){return t!==e}).forEach(function(e){return e.visualRange(t,{start:!0,end:!0})})}return!1},_raiseZoomEndHandlers:function(){this._argumentAxes.forEach(function(e){return e.handleZoomEnd()}),this.callBase()},_setOptionsByReference:function(){this.callBase(),(0,O.extend)(this._optionsByReference,{"argumentAxis.visualRange":!0})},option:function e(){var e=this.callBase.apply(this,arguments),t=this._options.silent("valueAxis");if("array"===(0,B.type)(t))for(var n=0;n<t.length;n++){var i="valueAxis[".concat(n,"].visualRange");this._optionsByReference[i]=!0}return e},_notifyVisualRange:function(){var e=this,t=e._argumentAxes[0],n=(0,M.convertVisualRangeObject)(t.visualRange(),!Q(e.option("argumentAxis.visualRange")));t.skipEventRising&&(0,M.rangesAreEqual)(n,e.option("argumentAxis.visualRange"))?t.skipEventRising=null:e.option("argumentAxis.visualRange",n),e.callBase()}});J.addPlugin(q.default),J.addPlugin($.default),J.addPlugin(K.plugins.core),J.addPlugin(K.plugins.chart),(0,P.default)("dxChart",J);var ee=J;t.default=ee,e.exports=t.default},function(e,t,n){function i(e){var t,n=this,i=n._animateStep;n._progress=n._calcProgress(e);for(t in n.params){var o=i[t]||i.base;o(n.element,n.params[t],n._progress,n._easing,n._currentParams,t)}return n.options.step&&n.options.step(n._easing(n._progress,0,1),n._progress),1!==n._progress||n.stop()}function o(e){return e-this._startTime>=this.delay&&(this.tick=i),!0}function a(e){return this._startTime=e,this.tick=this.delay?o:i,!0}function r(e,t,n){var i=this;i._progress=0,i.element=e,i.params=t,i.options=n,i.duration=n.partitionDuration?n.duration*n.partitionDuration:n.duration,i.delay=n.delay&&n.duration*n.delay||0,i._animateStep=n.animateStep||d,i._easing=c[n.easing]||c.easeOutCubic,i._currentParams={},i.tick=a}function s(e){var t=this;t._animationCount=0,t._timerId=null,t._animations={},t.element=e}t.AnimationController=s,t.animationSvgStep=t.easingFunctions=void 0;var l=n(120),u=function(){},c={easeOutCubic:function(e,t,n){return 1===e?n:(1-Math.pow(1-e,3))*(n-t)+ +t},linear:function(e,t,n){return 1===e?n:e*(n-t)+ +t}};t.easingFunctions=c;var d={segments:function e(t,n,i,o,a){var r,s,l,u,c=n.from,d=n.to,e=[];for(l=0;l<c.length;l++){if(r=c[l],s=[r[0]],r.length>1)for(u=1;u<r.length;u++)s.push(o(i,r[u],d[l][u]));e.push(s)}a.segments=n.end&&1===i?n.end:e,t.attr({segments:e})},arc:function(e,t,n,i){var o=t.from,a=t.to,r={};for(var s in o)r[s]=i(n,o[s],a[s]);e.attr(r)},transform:function(e,t,n,i,o){var a=t.from,r=t.to,s={};for(var l in a)s[l]=o[l]=i(n,a[l],r[l]);e.attr(s)},base:function(e,t,n,i,o,a){var r={};r[a]=o[a]=i(n,t.from,t.to),e.attr(r)},_:u,complete:function(e,t){e.attr(t)}};t.animationSvgStep=d,r.prototype={_calcProgress:function(e){return Math.min(1,(e-this.delay-this._startTime)/this.duration)},stop:function(e){var t=this,n=t.options,i=t._animateStep;t.stop=t.tick=u,i.complete&&i.complete(t.element,t._currentParams),n.complete&&!e&&n.complete()}},s.prototype={_loop:function(){var e,t=this,n=t._animations,i=0,o=(new Date).getTime(),a=t._endAnimation;for(e in n)n[e].tick(o)||delete n[e],i++;return 0===i?(t.stop(),void(t._endAnimationTimer=a&&setTimeout(function(){0===t._animationCount&&(a(),t._endAnimation=null)}))):void(t._timerId=l.requestAnimationFrame.call(null,function(){t._loop()},t.element))},addAnimation:function(e){var t=this;t._animations[t._animationCount++]=e,clearTimeout(t._endAnimationTimer),t._timerId||(clearTimeout(t._startDelay),t._startDelay=setTimeout(function(){t._timerId=1,t._loop()},0))},animateElement:function(e,t,n){e&&t&&n&&(e.animation&&e.animation.stop(),this.addAnimation(e.animation=new r(e,t,n)))},onEndAnimation:function(e){this._animationCount?this._endAnimation=e:e()},dispose:function(){this.stop(),this.element=null},stop:function(){var e=this;e._animations={},e._animationCount=0,(0,l.cancelAnimationFrame)(e._timerId),clearTimeout(e._startDelay),clearTimeout(e._endAnimationTimer),e._timerId=null},lock:function(){var e,t,n,i=this._animations;for(e in i)t=i[e].options.unstoppable,n=n||t,t||(i[e].stop(!0),delete i[e]);!n&&this.stop()}}},function(e,t,n){var i=n(57),o="#ffffff",a="#000000",r="#d3d3d3",s="#303030",l="#2b2b2b",u="#ff0000",c="#232323",d="#767676",h="none",f="solid",p="top",g="right",_="bottom",m="left",v="center",y="inside",b="outside";(0,i.registerTheme)({name:"generic.light",isDefault:!0,font:{color:d,family:"'Segoe UI', 'Helvetica Neue', 'Trebuchet MS', Verdana, sans-serif",weight:400,size:12,cursor:"default"},redrawOnResize:!0,backgroundColor:o,primaryTitleColor:c,secondaryTitleColor:d,gridColor:r,axisColor:d,title:{backgroundColor:o,font:{size:28,family:"'Segoe UI Light', 'Helvetica Neue Light', 'Segoe UI', 'Helvetica Neue', 'Trebuchet MS', Verdana, sans-serif",weight:200},subtitle:{font:{size:16},offset:0,wordWrap:"normal",textOverflow:"ellipsis"},wordWrap:"normal",textOverflow:"ellipsis"},loadingIndicator:{text:"Loading..."},"export":{backgroundColor:o,margin:10,font:{size:14,color:c,weight:400},button:{margin:{top:8,left:10,right:10,bottom:8},"default":{color:"#333",borderColor:"#ddd",backgroundColor:o},hover:{color:"#333",borderColor:"#bebebe",backgroundColor:"#e6e6e6"},focus:{color:a,borderColor:"#9d9d9d",backgroundColor:"#e6e6e6"},active:{color:"#333",borderColor:"#9d9d9d",backgroundColor:"#d4d4d4"}},shadowColor:r},tooltip:{enabled:!1,border:{width:1,color:r,dashStyle:f,visible:!0},font:{color:c},color:o,arrowLength:10,paddingLeftRight:18,paddingTopBottom:15,textAlignment:"center",shared:!1,location:v,shadow:{opacity:.4,offsetX:0,offsetY:4,blur:2,color:a},interactive:!1},legend:{hoverMode:"includePoints",verticalAlignment:p,horizontalAlignment:g,position:b,visible:!0,margin:10,markerSize:12,border:{visible:!1,width:1,cornerRadius:0,dashStyle:f},paddingLeftRight:20,paddingTopBottom:15,columnCount:0,rowCount:0,columnItemSpacing:20,rowItemSpacing:8,title:{backgroundColor:o,margin:{left:0,bottom:9,right:0,top:0},font:{size:18,weight:200},subtitle:{offset:0,font:{size:14},wordWrap:"none",textOverflow:"ellipsis"},wordWrap:"none",textOverflow:"ellipsis"}},"chart:common":{animation:{enabled:!0,duration:1e3,easing:"easeOutCubic",maxPointCountSupported:300},commonSeriesSettings:{border:{visible:!1,width:2},showInLegend:!0,visible:!0,hoverMode:"nearestPoint",selectionMode:"includePoints",hoverStyle:{hatching:{direction:g,width:2,step:6,opacity:.75},border:{visible:!1,width:3}},selectionStyle:{hatching:{direction:g,width:2,step:6,opacity:.5},border:{visible:!1,width:3}},valueErrorBar:{displayMode:"auto",value:1,color:a,lineWidth:2,edgeLength:8},label:{visible:!1,alignment:v,rotationAngle:0,horizontalOffset:0,verticalOffset:0,radialOffset:0,showForZeroValues:!0,customizeText:void 0,maxLabelCount:void 0,position:b,font:{color:o},border:{visible:!1,width:1,color:r,dashStyle:f},connector:{visible:!1,width:1}}},seriesSelectionMode:"single",pointSelectionMode:"single",equalRowHeight:!0,dataPrepareSettings:{checkTypeForAllData:!1,convertToAxisDataType:!0,sortingMethod:!0},title:{margin:10},adaptiveLayout:{width:80,height:80,keepLabels:!0},_rtl:{legend:{itemTextPosition:m}},resolveLabelOverlapping:h},"chart:common:axis":{visible:!0,valueMarginsEnabled:!0,placeholderSize:null,logarithmBase:10,discreteAxisDivisionMode:"betweenLabels",width:1,label:{visible:!0},grid:{visible:!1,width:1},minorGrid:{visible:!1,width:1,opacity:.3},tick:{visible:!0,width:1,length:7,shift:3},minorTick:{visible:!1,width:1,opacity:.3,length:7,shift:3},stripStyle:{paddingLeftRight:10,paddingTopBottom:5},constantLineStyle:{width:1,color:a,dashStyle:f,label:{visible:!0,position:y}},marker:{label:{}}},"chart:common:annotation":{font:{color:"#333333"},tooltipEnabled:!0,border:{width:1,color:"#dddddd",dashStyle:f,visible:!0},color:o,opacity:.9,arrowLength:14,arrowWidth:14,paddingLeftRight:10,paddingTopBottom:10,shadow:{opacity:.15,offsetX:0,offsetY:1,blur:4,color:a},image:{width:30,height:30},wordWrap:"normal",textOverflow:"ellipsis",allowDragging:!1},chart:{commonSeriesSettings:{type:"line",stack:"default",aggregation:{enabled:void 0},point:{visible:!0,symbol:"circle",size:12,border:{visible:!1,width:1},hoverMode:"onlyPoint",selectionMode:"onlyPoint",hoverStyle:{border:{visible:!0,width:4}},selectionStyle:{border:{visible:!0,width:4}}},scatter:{},line:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},stackedline:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},stackedspline:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},fullstackedline:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},fullstackedspline:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},stepline:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},area:{point:{visible:!1},opacity:.5},stackedarea:{point:{visible:!1},opacity:.5},fullstackedarea:{point:{visible:!1},opacity:.5},fullstackedsplinearea:{point:{visible:!1},opacity:.5},steparea:{border:{visible:!0,width:2},point:{visible:!1},hoverStyle:{border:{visible:!0,width:3}},selectionStyle:{border:{visible:!0,width:3}},opacity:.5},spline:{width:2,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},splinearea:{point:{visible:!1},opacity:.5},stackedsplinearea:{point:{visible:!1},opacity:.5},bar:{cornerRadius:0,point:{hoverStyle:{border:{visible:!1}},selectionStyle:{border:{visible:!1}}}},stackedbar:{cornerRadius:0,point:{hoverStyle:{border:{visible:!1}},selectionStyle:{border:{visible:!1}}},label:{position:y}},fullstackedbar:{cornerRadius:0,point:{hoverStyle:{border:{visible:!1}},selectionStyle:{border:{visible:!1}}},label:{position:y}},rangebar:{cornerRadius:0,point:{hoverStyle:{border:{visible:!1}},selectionStyle:{border:{visible:!1}}}},rangearea:{point:{visible:!1},opacity:.5},rangesplinearea:{point:{visible:!1},opacity:.5},bubble:{opacity:.5,point:{hoverStyle:{border:{visible:!1}},selectionStyle:{border:{visible:!1}}}},candlestick:{width:1,reduction:{color:u},hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3},point:{border:{visible:!0}}},stock:{width:1,reduction:{color:u},hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3},point:{border:{visible:!0}}}},crosshair:{enabled:!1,color:"#f05b41",width:1,dashStyle:f,label:{visible:!1,font:{color:o,size:12}},verticalLine:{visible:!0},horizontalLine:{visible:!0}},commonAxisSettings:{multipleAxesSpacing:5,forceUserTickInterval:!1,breakStyle:{width:5,color:"#ababab",line:"waved"},label:{displayMode:"standard",overlappingBehavior:"hide",indentFromAxis:10,wordWrap:"normal",textOverflow:"none"},title:{font:{size:16},margin:6,alignment:v},constantLineStyle:{paddingLeftRight:10,paddingTopBottom:10}},horizontalAxis:{position:_,axisDivisionFactor:70,label:{rotationAngle:90,staggeringSpacing:5,alignment:v},stripStyle:{label:{horizontalAlignment:v,verticalAlignment:p}},constantLineStyle:{label:{horizontalAlignment:g,verticalAlignment:p}},constantLines:[]},verticalAxis:{position:m,axisDivisionFactor:40,label:{alignment:g},stripStyle:{label:{horizontalAlignment:m,verticalAlignment:v}},constantLineStyle:{label:{horizontalAlignment:m,verticalAlignment:p}},constantLines:[]},argumentAxis:{endOnTick:!1,workWeek:[1,2,3,4,5]},valueAxis:{grid:{visible:!0},autoBreaksEnabled:!1,maxAutoBreakCount:4},commonPaneSettings:{backgroundColor:h,border:{color:r,width:1,visible:!1,top:!0,bottom:!0,left:!0,right:!0,dashStyle:f}},scrollBar:{visible:!1,offset:5,color:"gray",width:10},adjustOnZoom:!0,autoHidePointMarkers:!0,rotated:!1,synchronizeMultiAxes:!0,stickyHovering:!0,barGroupPadding:.3,minBubbleSize:12,maxBubbleSize:.2,zoomAndPan:{dragBoxStyle:{color:"#2a2a2a",opacity:.2},panKey:"shift",allowMouseWheel:!0,allowTouchGestures:!0}},pie:{innerRadius:.5,minDiameter:.5,type:"pie",dataPrepareSettings:{_skipArgumentSorting:!0},commonSeriesSettings:{pie:{border:{visible:!1,width:2,color:o},hoverStyle:{hatching:{direction:g,width:4,step:10,opacity:.75},border:{visible:!1,width:2}},selectionStyle:{hatching:{direction:g,width:4,step:10,opacity:.5},border:{visible:!1,width:2}}},doughnut:{border:{visible:!1,width:2,color:o},hoverStyle:{hatching:{direction:g,width:4,step:10,opacity:.75},border:{visible:!1,width:2}},selectionStyle:{hatching:{direction:g,width:4,step:10,opacity:.5},border:{visible:!1,width:2}}},donut:{border:{visible:!1,width:2,color:o},hoverStyle:{hatching:{direction:g,width:4,step:10,opacity:.75},border:{visible:!1,width:2}},selectionStyle:{hatching:{direction:g,width:4,step:10,opacity:.5},border:{visible:!1,width:2}}},label:{textOverflow:"ellipsis",wordWrap:"normal"}},legend:{hoverMode:"allArgumentPoints",backgroundColor:h},adaptiveLayout:{keepLabels:!1}},gauge:{scale:{tick:{visible:!0,length:5,width:2,opacity:1},minorTick:{visible:!1,length:3,width:1,opacity:1},label:{visible:!0,alignment:v,hideFirstOrLast:"last",overlappingBehavior:"hide"},position:p,endOnTick:!1},rangeContainer:{offset:0,width:5,backgroundColor:"#808080"},valueIndicators:{_default:{color:"#c2c2c2"},rangebar:{space:2,size:10,color:"#cbc5cf",backgroundColor:h,text:{indent:0,font:{size:14,color:null}}},twocolorneedle:{secondColor:"#e18e92"},trianglemarker:{space:2,length:14,width:13,color:"#8798a5"},textcloud:{arrowLength:5,horizontalOffset:6,verticalOffset:3,color:"#679ec5",text:{font:{color:o,size:18}}}},indicator:{hasPositiveMeaning:!0,layout:{horizontalAlignment:v,verticalAlignment:_},text:{font:{size:18}}},_circular:{scale:{scaleDivisionFactor:17,orientation:b,label:{indentFromTick:10}},rangeContainer:{orientation:b},valueIndicatorType:"rectangleneedle",subvalueIndicatorType:"trianglemarker",valueIndicators:{_type:"rectangleneedle",_default:{offset:20,indentFromCenter:0,width:2,spindleSize:14,spindleGapSize:10,beginAdaptingAtRadius:50},triangleneedle:{width:4},twocolorneedle:{space:2,secondFraction:.4},rangebar:{offset:30},trianglemarker:{offset:6},textcloud:{offset:-6}}},_linear:{scale:{scaleDivisionFactor:25,horizontalOrientation:g,verticalOrientation:_,label:{indentFromTick:-10}},rangeContainer:{horizontalOrientation:g,verticalOrientation:_},valueIndicatorType:"rangebar",subvalueIndicatorType:"trianglemarker",valueIndicators:{_type:"rectangle",_default:{offset:2.5,length:15,width:15},rectangle:{width:10},rangebar:{offset:10,horizontalOrientation:g,verticalOrientation:_},trianglemarker:{offset:10,horizontalOrientation:m,verticalOrientation:p},textcloud:{offset:-1,horizontalOrientation:m,verticalOrientation:p}}}},barGauge:{backgroundColor:"#e0e0e0",relativeInnerRadius:.3,barSpacing:4,resolveLabelOverlapping:"hide",label:{indent:20,connectorWidth:2,font:{size:16}},legend:{visible:!1},indicator:{hasPositiveMeaning:!0,layout:{horizontalAlignment:v,verticalAlignment:_},text:{font:{size:18}}}},rangeSelector:{scale:{valueMarginsEnabled:!0,width:1,color:a,opacity:.1,showCustomBoundaryTicks:!0,label:{overlappingBehavior:"hide",alignment:v,visible:!0,topIndent:7,font:{size:11}},tick:{width:1,color:a,opacity:.17,visible:!0,length:12},minorTick:{width:1,color:a,opacity:.05,visible:!0,length:12},marker:{width:1,color:"#000000",opacity:.1,visible:!0,separatorHeight:33,topIndent:10,textLeftIndent:7,textTopIndent:11,label:{}},logarithmBase:10,workWeek:[1,2,3,4,5],breakStyle:{width:5,color:"#ababab",line:"waved"},endOnTick:!1},selectedRangeColor:"#606060",sliderMarker:{visible:!0,paddingTopBottom:2,paddingLeftRight:4,color:"#606060",invalidRangeColor:u,font:{color:o,size:11}},sliderHandle:{width:1,color:a,opacity:.2},shutter:{opacity:.75},background:{color:"#c0bae1",visible:!0,image:{location:"full"}},behavior:{snapToTicks:!0,animationEnabled:!0,moveSelectedRangeByClick:!0,manualRangeSelectionEnabled:!0,allowSlidersSwap:!0,callValueChanged:"onMovingComplete"},redrawOnResize:!0,chart:{barGroupPadding:.3,minBubbleSize:12,maxBubbleSize:.2,topIndent:.1,bottomIndent:0,valueAxis:{inverted:!1,logarithmBase:10},commonSeriesSettings:{type:"area",aggregation:{enabled:void 0},point:{visible:!1},scatter:{point:{visible:!0}}}}},map:{title:{margin:10},background:{borderWidth:1,borderColor:"#cacaca"},layer:{label:{enabled:!1,stroke:o,"stroke-width":1,"stroke-opacity":.7,font:{color:l,size:12}}},"layer:area":{borderWidth:1,borderColor:o,color:"#d2d2d2",hoveredBorderColor:s,selectedBorderWidth:2,selectedBorderColor:s,label:{"stroke-width":2,font:{size:16}}},"layer:line":{borderWidth:2,color:"#ba8365",hoveredColor:"#a94813",selectedBorderWidth:3,selectedColor:"#e55100",label:{"stroke-width":2,font:{size:16}}},"layer:marker":{label:{enabled:!0,"stroke-width":1,font:{size:12}}},"layer:marker:dot":{borderWidth:2,borderColor:o,size:8,selectedStep:2,backStep:18,backColor:o,backOpacity:.32,shadow:!0},"layer:marker:bubble":{minSize:20,maxSize:50,hoveredBorderWidth:1,hoveredBorderColor:s,selectedBorderWidth:2,selectedBorderColor:s},"layer:marker:pie":{size:50,hoveredBorderWidth:1,hoveredBorderColor:s,selectedBorderWidth:2,selectedBorderColor:s},"layer:marker:image":{size:20},legend:{verticalAlignment:_,horizontalAlignment:g,position:y,backgroundOpacity:.65,border:{visible:!0},paddingLeftRight:16,paddingTopBottom:12},controlBar:{borderColor:"#5d5d5d",borderWidth:3,color:o,margin:20,opacity:.3},_rtl:{legend:{itemTextPosition:m}}},treeMap:{tile:{border:{width:1,opacity:.2,color:"#000000"},color:"#5f8b95",hoverStyle:{hatching:{opacity:.75,step:6,width:2,direction:"right"},border:{}},selectionStyle:{hatching:{opacity:.5,step:6,width:2,direction:"right"},border:{opacity:1}},label:{visible:!0,paddingLeftRight:5,paddingTopBottom:4,font:{color:"#ffffff",weight:600},shadow:{opacity:.6,offsetX:0,offsetY:1,blur:2,color:"#000000"},wordWrap:"normal",textOverflow:"ellipsis"}},group:{padding:4,border:{width:1},color:"#eeeeee",hoverStyle:{hatching:{opacity:0,step:6,width:2,direction:"right"},border:{}},selectionStyle:{hatching:{opacity:0,step:6,width:2,direction:"right"},border:{}},label:{visible:!0,paddingLeftRight:5,paddingTopBottom:4,font:{color:d,weight:600},textOverflow:"ellipsis"}},title:{subtitle:{}},tooltip:{},loadingIndicator:{}},sparkline:{lineColor:"#666666",lineWidth:2,areaOpacity:.2,minColor:"#e8c267",maxColor:"#e55253",barPositiveColor:"#a9a9a9",barNegativeColor:"#d7d7d7",winColor:"#a9a9a9",lossColor:"#d7d7d7",firstLastColor:"#666666",pointSymbol:"circle",pointColor:o,pointSize:4,type:"line",argumentField:"arg",valueField:"val",winlossThreshold:0,showFirstLast:!0,showMinMax:!1,tooltip:{enabled:!0}},bullet:{color:"#e8c267",targetColor:"#666666",targetWidth:4,showTarget:!0,showZeroLevel:!0,tooltip:{enabled:!0}},polar:{commonSeriesSettings:{type:"scatter",closed:!0,point:{visible:!0,symbol:"circle",size:12,border:{visible:!1,width:1},hoverMode:"onlyPoint",selectionMode:"onlyPoint",hoverStyle:{border:{visible:!0,width:4},size:12},selectionStyle:{border:{visible:!0,width:4},size:12}},scatter:{},line:{width:2,dashStyle:f,hoverStyle:{width:3,hatching:{direction:h}},selectionStyle:{width:3}},area:{point:{visible:!1},opacity:.5},stackedline:{width:2},bar:{opacity:.8},stackedbar:{opacity:.8}},adaptiveLayout:{width:80,height:80,keepLabels:!0},barGroupPadding:.3,commonAxisSettings:{visible:!0,forceUserTickInterval:!1,label:{overlappingBehavior:"hide",indentFromAxis:5},grid:{visible:!0},minorGrid:{visible:!0},tick:{visible:!0},title:{font:{size:16},margin:10}},argumentAxis:{startAngle:0,firstPointOnStartAngle:!1,period:void 0},valueAxis:{endOnTick:!1,tick:{visible:!1}},horizontalAxis:{position:p,axisDivisionFactor:50,label:{alignment:v}},verticalAxis:{position:p,axisDivisionFactor:30,label:{alignment:g}}},funnel:{sortData:!0,valueField:"val",colorField:"color",argumentField:"arg",hoverEnabled:!0,selectionMode:"single",item:{border:{visible:!1,width:2,color:o},hoverStyle:{hatching:{opacity:.75,step:6,width:2,direction:g},border:{}},selectionStyle:{hatching:{opacity:.5,step:6,width:2,direction:g},border:{}}},title:{margin:10},adaptiveLayout:{width:80,height:80,keepLabels:!0},legend:{visible:!1},_rtl:{legend:{itemTextPosition:m}},tooltip:{customizeTooltip:function(e){return{text:e.item.argument+" "+e.valueText}}},inverted:!1,algorithm:"dynamicSlope",neckWidth:0,neckHeight:0,resolveLabelOverlapping:"shift",label:{textOverflow:"ellipsis",wordWrap:"normal",visible:!0,horizontalAlignment:g,horizontalOffset:0,verticalOffset:0,showForZeroValues:!1,customizeText:function(e){return e.item.argument+" "+e.valueText},position:"columns",font:{color:o},border:{visible:!1,width:1,color:r,dashStyle:f},connector:{visible:!0,width:1,opacity:.5}}},sankey:{sourceField:"source",targetField:"target",weightField:"weight",hoverEnabled:!0,alignment:v,adaptiveLayout:{width:80,height:80,keepLabels:!0},label:{visible:!0,horizontalOffset:8,verticalOffset:0,overlappingBehavior:"ellipsis",useNodeColors:!1,font:{color:a,weight:500},border:{visible:!1,width:2,color:o},customizeText:function(e){return e.title},shadow:{opacity:.2,offsetX:0,offsetY:1,blur:1,color:o}},title:{margin:10,font:{size:28,weight:200},subtitle:{font:{size:16}}},tooltip:{enabled:!0},node:{padding:30,width:8,opacity:1,border:{color:o,width:1,visible:!1},hoverStyle:{hatching:{opacity:.75,step:6,width:2,direction:g},border:{}}},link:{color:"#888888",colorMode:"none",opacity:.3,border:{color:o,width:1,visible:!1},hoverStyle:{opacity:.5,hatching:{opacity:.75,step:6,width:2,direction:g},border:{}}}}}),(0,i.registerTheme)({name:"generic.light.compact"},"generic.light")},function(e,t,n){var i=n(57),o="#ffffff",a="#000000",r="#2b2b2b",s="#b5b5b5",l="#303030",u="#686868",c="#c7c7c7",d="#8e8e8e",h="#b8b8b8",f="#494949";(0,i.registerTheme)({name:"generic.dark",font:{color:"#808080"},backgroundColor:"#2a2a2a",primaryTitleColor:"#dedede",secondaryTitleColor:"#a3a3a3",gridColor:"#555555",axisColor:"#a3a3a3","export":{backgroundColor:"#2a2a2a",font:{color:"#dbdbdb"},button:{"default":{color:"#dedede",borderColor:"#4d4d4d",backgroundColor:"#2e2e2e"},hover:{color:"#dedede",borderColor:"#6c6c6c",backgroundColor:"#444"},focus:{color:"#dedede",borderColor:"#8d8d8d",backgroundColor:"#444444"},active:{color:"#dedede",borderColor:"#8d8d8d",backgroundColor:"#555555"}},shadowColor:"#292929"},tooltip:{color:r,border:{color:f},font:{color:"#929292"}},"chart:common":{commonSeriesSettings:{label:{border:{color:f}},valueErrorBar:{color:o}}},"chart:common:axis":{constantLineStyle:{color:o}},"chart:common:annotation":{font:{color:"#929292"},border:{color:f},color:r,shadow:{opacity:.008,offsetY:4,blur:8}},chart:{commonPaneSettings:{border:{color:f}},commonAxisSettings:{breakStyle:{color:"#818181"}},zoomAndPan:{dragBoxStyle:{color:o}}},gauge:{rangeContainer:{backgroundColor:s},valueIndicators:{_default:{color:s},rangebar:{color:"#84788b"},twocolorneedle:{secondColor:"#ba544d"},trianglemarker:{color:"#b7918f"},textcloud:{color:"#ba544d"}}},barGauge:{backgroundColor:"#3c3c3c"},rangeSelector:{scale:{tick:{color:o,opacity:.32},minorTick:{color:o,opacity:.1},breakStyle:{color:"#818181"}},selectedRangeColor:s,sliderMarker:{color:s,font:{color:l}},sliderHandle:{color:o,opacity:.2},shutter:{color:r,opacity:.9}},map:{background:{borderColor:"#3f3f3f"},layer:{label:{stroke:a,font:{color:o}}},"layer:area":{borderColor:l,color:u,hoveredBorderColor:o,selectedBorderColor:o},"layer:line":{color:"#c77244",hoveredColor:"#ff5d04",selectedColor:"#ff784f"},"layer:marker:bubble":{hoveredBorderColor:o,selectedBorderColor:o},"layer:marker:pie":{hoveredBorderColor:o,selectedBorderColor:o},legend:{border:{color:"#3f3f3f"},font:{color:o}},controlBar:{borderColor:c,color:l}},treeMap:{group:{color:"#4c4c4c",label:{font:{color:"#a3a3a3"}}}},sparkline:{lineColor:c,firstLastColor:c,barPositiveColor:h,barNegativeColor:d,winColor:h,lossColor:d,pointColor:l},bullet:{targetColor:d},funnel:{item:{border:{color:"#2a2a2a"}}},sankey:{label:{font:{color:o},shadow:{opacity:0}},node:{border:{color:"#2a2a2a"}},link:{color:"#888888",border:{color:"#2a2a2a"},hoverStyle:{color:"#bbbbbb"}}}},"generic.light"),(0,i.registerTheme)({name:"generic.dark.compact"},"generic.dark")},function(e,t,n){var i=n(57),o="#ffffff",a="#000000",r="#cf00da",s="#f8ca00",l="#686868";(0,i.registerTheme)({name:"generic.contrast",defaultPalette:"Bright",font:{color:o},backgroundColor:a,primaryTitleColor:o,secondaryTitleColor:o,gridColor:o,axisColor:o,"export":{backgroundColor:a,font:{color:o},button:{"default":{color:o,borderColor:o,backgroundColor:a},hover:{color:o,borderColor:o,backgroundColor:"#cf00d7"},focus:{color:o,borderColor:"#cf00d7",backgroundColor:a},active:{color:a,borderColor:o,backgroundColor:o}},borderColor:o,menuButtonColor:a,activeBackgroundColor:o,activeColor:a,selectedBorderColor:r,selectedColor:r,shadowColor:"none"},tooltip:{border:{color:o},font:{color:o},color:a},"chart:common":{commonSeriesSettings:{valueErrorBar:{color:o},hoverStyle:{hatching:{opacity:.5}},selectionStyle:{hatching:{opacity:.35}},label:{font:{color:o},border:{color:o}}}},"chart:common:axis":{constantLineStyle:{color:o}},"chart:common:annotation":{font:{color:o},border:{color:o},color:a},chart:{commonSeriesSettings:{},crosshair:{color:"#cf00d7"},commonPaneSettings:{backgroundColor:a,border:{color:o}},scrollBar:{color:o},commonAxisSettings:{breakStyle:{color:"#cf00d7"}},zoomAndPan:{dragBoxStyle:{color:o,
opacity:.7}}},pie:{commonSeriesSettings:{pie:{hoverStyle:{hatching:{opacity:.5}},selectionStyle:{hatching:{opacity:.35}}},doughnut:{hoverStyle:{hatching:{opacity:.5}},selectionStyle:{hatching:{opacity:.35}}},donut:{hoverStyle:{hatching:{opacity:.5}},selectionStyle:{hatching:{opacity:.35}}}}},gauge:{rangeContainer:{backgroundColor:o},valueIndicators:{_default:{color:o},rangebar:{color:o,backgroundColor:a},twocolorneedle:{secondColor:o},trianglemarker:{color:o},textcloud:{color:o,text:{font:{color:a}}}}},barGauge:{backgroundColor:"#3c3c3c"},rangeSelector:{scale:{tick:{color:o,opacity:.4},minorTick:{color:o,opacity:.12},breakStyle:{color:"#cf00d7"}},selectedRangeColor:r,sliderMarker:{color:r},sliderHandle:{color:r,opacity:1},shutter:{opacity:.75},background:{color:a}},map:{background:{borderColor:o},layer:{label:{stroke:a,font:{color:o}}},"layer:area":{borderColor:a,color:l,hoveredBorderColor:o,selectedBorderColor:o,label:{font:{opacity:1}}},"layer:line":{color:"#267cff",hoveredColor:"#f613ff",selectedColor:o},"layer:marker:dot":{borderColor:a,color:s,backColor:a,backOpacity:.32},"layer:marker:bubble":{color:s,hoveredBorderColor:o,selectedBorderColor:o},"layer:marker:pie":{hoveredBorderColor:o,selectedBorderColor:o},controlBar:{borderColor:o,color:a,opacity:.3}},treeMap:{tile:{color:"#70c92f"},group:{color:"#797979",label:{font:{color:o}}}},sparkline:{pointColor:a},bullet:{},polar:{commonSeriesSettings:{}},funnel:{label:{connector:{opacity:1}}},sankey:{label:{font:{color:o},shadow:{opacity:0}},node:{border:{visible:!0,width:1,color:o}},link:{opacity:.5,border:{visible:!0,width:1,color:o},hoverStyle:{opacity:.9}}}},"generic.light"),(0,i.registerTheme)({name:"generic.contrast.compact"},"generic.contrast")},function(e,t,n){var i=n(57),o="#f05b41",a="#fff",r="#333",s="#8899a8",l="#707070",u="#dee1e3";(0,i.registerTheme)({name:"generic.carmine",defaultPalette:"Carmine",backgroundColor:a,primaryTitleColor:r,secondaryTitleColor:s,gridColor:u,axisColor:l,"export":{backgroundColor:a,font:{color:r},button:{"default":{color:r,borderColor:"#b1b7bd",backgroundColor:a},hover:{color:r,borderColor:"#b1b7bd",backgroundColor:"#faf2f0"},focus:{color:r,borderColor:"#6d7781",backgroundColor:"#faf2f0"},active:{color:r,borderColor:"#6d7781",backgroundColor:"#f5e7e4"}}},legend:{font:{color:l}},tooltip:{color:a,border:{color:u},font:{color:r}},"chart:common":{commonSeriesSettings:{label:{border:{color:u}}}},"chart:common:annotation":{font:{color:r},border:{color:u},color:a},chart:{commonPaneSettings:{border:{color:u}},commonAxisSettings:{breakStyle:{color:"#c1c5c7"}}},rangeSelector:{scale:{breakStyle:{color:"#c1c5c7"},tick:{opacity:.12}},selectedRangeColor:o,sliderMarker:{color:o},sliderHandle:{color:o,opacity:.5}},sparkline:{pointColor:a,minColor:"#f0ad4e",maxColor:"#f74d61"},treeMap:{group:{color:u,label:{font:{color:s}}}},bullet:{color:o},gauge:{valueIndicators:{rangebar:{color:o},textcloud:{color:o}}}},"generic.light"),(0,i.registerTheme)({name:"generic.carmine.compact"},"generic.carmine")},function(e,t,n){var i=n(57),o="#3debd3",a="#465672",r="#fff",s="#919bac",l="#c7ccd4",u="#596980";(0,i.registerTheme)({name:"generic.darkmoon",defaultPalette:"Dark Moon",backgroundColor:a,primaryTitleColor:r,secondaryTitleColor:s,gridColor:u,axisColor:l,"export":{backgroundColor:a,font:{color:r},button:{"default":{color:r,borderColor:"#7a889e",backgroundColor:a},hover:{color:r,borderColor:"#9da8b8",backgroundColor:"#596e92"},focus:{color:r,borderColor:"#c4cad4",backgroundColor:"#596e92"},active:{color:r,borderColor:"#c4cad4",backgroundColor:"#6b80a4"}}},legend:{font:{color:l}},tooltip:{color:"#62789e",border:{color:u},font:{color:r}},"chart:common":{commonSeriesSettings:{label:{border:{color:u}}}},"chart:common:annotation":{font:{color:r},border:{color:u},color:"#62789e"},chart:{commonPaneSettings:{border:{color:u}},commonAxisSettings:{breakStyle:{color:"#73869e"}}},gauge:{valueIndicators:{rangebar:{color:o},textcloud:{color:o,text:{font:{color:a}}}}},barGauge:{backgroundColor:"#526280"},funnel:{item:{border:{color:a}}},sparkline:{pointColor:a,minColor:"#f0ad4e",maxColor:"#f9517e"},treeMap:{group:{color:u,label:{font:{color:r}}}},map:{background:{borderColor:u},"layer:area":{color:"#97a3b6",borderColor:a}},rangeSelector:{shutter:{color:a},scale:{breakStyle:{color:"#73869e"},tick:{opacity:.2}},selectedRangeColor:o,sliderMarker:{color:o,font:{color:"#000"}},sliderHandle:{color:o,opacity:.5}},bullet:{color:o},sankey:{link:{border:{color:a}},node:{border:{color:a}}}},"generic.dark"),(0,i.registerTheme)({name:"generic.darkmoon.compact"},"generic.darkmoon")},function(e,t,n){var i=n(57),o="#7ab8eb",a="#fff",r="#333",s="#99a1a8",l="#707070",u="#e8eaeb";(0,i.registerTheme)({name:"generic.softblue",defaultPalette:"Soft Blue",backgroundColor:a,primaryTitleColor:r,secondaryTitleColor:s,gridColor:u,axisColor:l,"export":{backgroundColor:a,font:{color:r},button:{"default":{color:r,borderColor:"#c9d0d4",backgroundColor:a},hover:{color:r,borderColor:"#a7b2b9",backgroundColor:"#e6e6e6"},focus:{color:r,borderColor:"#82929b",backgroundColor:"#e6e6e6"},active:{color:r,borderColor:"#82929b",backgroundColor:"#d4d4d4"}}},legend:{font:{color:l}},tooltip:{color:a,border:{color:u},font:{color:r}},"chart:common":{commonSeriesSettings:{label:{border:{color:u}}}},"chart:common:annotation":{color:a,border:{color:u},font:{color:r}},chart:{commonPaneSettings:{border:{color:u}},commonAxisSettings:{breakStyle:{color:"#cfd2d3"}}},rangeSelector:{scale:{breakStyle:{color:"#cfd2d3"},tick:{opacity:.12}},selectedRangeColor:o,sliderMarker:{color:o},sliderHandle:{color:o,opacity:.5}},sparkline:{pointColor:a,minColor:"#f0ad4e",maxColor:"#d9534f"},treeMap:{group:{color:u,label:{font:{color:s}}}},bullet:{color:o},gauge:{valueIndicators:{rangebar:{color:o},textcloud:{color:o}}}},"generic.light"),(0,i.registerTheme)({name:"generic.softblue.compact"},"generic.softblue")},function(e,t,n){var i=n(57),o="#9c63ff",a="#17171f",r="#f5f6f7",s="#fff",l="#b2b2b6",u="#343840";(0,i.registerTheme)({name:"generic.darkviolet",defaultPalette:"Dark Violet",backgroundColor:a,primaryTitleColor:r,secondaryTitleColor:s,gridColor:u,axisColor:l,"export":{backgroundColor:a,font:{color:r},button:{"default":{color:r,borderColor:"#414152",backgroundColor:a},hover:{color:r,borderColor:"#5c5c74",backgroundColor:"#2d2d3c"},focus:{color:r,borderColor:"#7c7c97",backgroundColor:"#2d2d3c"},active:{color:r,borderColor:"#7c7c97",backgroundColor:"#3c3c51"}}},legend:{font:{color:l}},tooltip:{color:a,border:{color:"#414152"},font:{color:r}},"chart:common":{commonSeriesSettings:{label:{border:{color:u}}}},"chart:common:annotation":{font:{color:r},border:{color:"#414152"},color:a},chart:{commonPaneSettings:{border:{color:u}},commonAxisSettings:{breakStyle:{color:"#575e6b"}}},funnel:{item:{border:{color:a}}},sparkline:{pointColor:a,minColor:"#f0ad4e",maxColor:"#d9534f"},treeMap:{group:{color:u,label:{font:{color:s}}}},rangeSelector:{shutter:{color:a},scale:{breakStyle:{color:"#575e6b"},tick:{opacity:.2}},selectedRangeColor:o,sliderMarker:{color:o,font:{color:"#fff"}},sliderHandle:{color:o,opacity:.5}},bullet:{color:o},gauge:{valueIndicators:{rangebar:{color:o},textcloud:{color:o}}},sankey:{link:{border:{color:a}},node:{border:{color:a}}}},"generic.dark"),(0,i.registerTheme)({name:"generic.darkviolet.compact"},"generic.darkviolet")},function(e,t,n){var i=n(57),o="#3cbab2",a="#f5f5f5",r="#28484f",s="#7eb2be",l="#657c80",u="#dedede";(0,i.registerTheme)({name:"generic.greenmist",defaultPalette:"Green Mist",backgroundColor:a,primaryTitleColor:r,secondaryTitleColor:s,gridColor:u,axisColor:l,"export":{backgroundColor:a,font:{color:r},button:{"default":{color:r,borderColor:"#a2b4b8",backgroundColor:a},hover:{color:r,borderColor:"#7f989e",backgroundColor:"rgba(222, 222, 222, 0.4)"},focus:{color:r,borderColor:"#5f777c",backgroundColor:"rgba(222, 222, 222, 0.4)"},active:{color:r,borderColor:"#5f777c",backgroundColor:"rgba(222, 222, 222, 0.8)"}}},legend:{font:{color:l}},tooltip:{color:"#fff",border:{color:u},font:{color:r}},"chart:common":{commonSeriesSettings:{label:{border:{color:u}}}},"chart:common:annotation":{color:"#fff",border:{color:u},font:{color:r}},chart:{commonPaneSettings:{border:{color:u}},commonAxisSettings:{breakStyle:{color:"#c1c1c1"}}},funnel:{item:{border:{color:a}}},sparkline:{pointColor:a,minColor:"#ffc852",maxColor:"#f74a5e"},treeMap:{group:{color:u,label:{font:{color:s}}}},rangeSelector:{shutter:{color:a},scale:{breakStyle:{color:"#c1c1c1"},tick:{opacity:.12}},selectedRangeColor:o,sliderMarker:{color:o},sliderHandle:{color:o,opacity:.5}},bullet:{color:o},gauge:{valueIndicators:{rangebar:{color:o},textcloud:{color:o}}}},"generic.light"),(0,i.registerTheme)({name:"generic.greenmist.compact"},"generic.greenmist")},function(e,t,n){function i(e,t,n){(0,o.registerTheme)({name:"material."+e+"."+t,rangeSelector:{selectedRangeColor:n,sliderMarker:{color:n},sliderHandle:{color:n}},map:{"layer:marker:dot":{color:n},"layer:marker:bubble":{color:n},legend:{markerColor:n}},bullet:{color:n},gauge:{valueIndicators:{rangebar:{color:n},textcloud:{color:n}}}},"material."+t)}var o=n(57),a="'Roboto', 'RobotoFallback', 'Helvetica', 'Arial', sans-serif",r="rgba(0,0,0,0.87)",s="rgba(0,0,0,0.54)",l="rgba(255,255,255,0.87)",u="rgba(255,255,255,0.54)",c="#363640",d="#ffffff",h="#000000",f="#b5b5b5",p="#686868",g="#c7c7c7",_="#8e8e8e",m="#b8b8b8",v="#494949",y="#818181";(0,o.registerTheme)({name:"material",defaultPalette:"Material",font:{family:a},title:{margin:{top:20,bottom:20,left:0,right:0},font:{size:20,family:a,weight:500},horizontalAlignment:"left",subtitle:{font:{size:14},horizontalAlignment:"left"}},tooltip:{shadow:{opacity:0},border:{visible:!1},paddingLeftRight:8,paddingTopBottom:6,arrowLength:0,location:"edge",color:"#616161",font:{color:d},cornerRadius:4},chart:{commonAxisSettings:{minorTick:{opacity:.5},label:{font:{size:11}}},commonAnnotationSettings:{font:{color:d},border:{color:"#616161"},color:"#616161",arrowLength:14,arrowWidth:0,shadow:{opacity:.08,offsetY:4,blur:8},cornerRadius:4}},pie:{title:{horizontalAlignment:"center",subtitle:{horizontalAlignment:"center"}}},polar:{commonAxisSettings:{minorTick:{opacity:.5}},title:{horizontalAlignment:"center",subtitle:{horizontalAlignment:"center"}}},funnel:{title:{horizontalAlignment:"center",subtitle:{horizontalAlignment:"center"}}},gauge:{title:{horizontalAlignment:"center",subtitle:{horizontalAlignment:"center"}}},barGauge:{title:{horizontalAlignment:"center",subtitle:{horizontalAlignment:"center"}}},rangeSelector:{sliderHandle:{opacity:.5}},treeMap:{group:{label:{font:{weight:500}}}}},"generic.light"),(0,o.registerTheme)({name:"material.light",gridColor:"#e0e0e0",axisColor:s,primaryTitleColor:r,legend:{font:{color:s}},chart:{scrollBar:{color:"#bfbfbf",opacity:.7}},gauge:{rangeContainer:{backgroundColor:"rgba(0,0,0,0.2)"}},barGauge:{backgroundColor:"#efefef"}},"material"),(0,o.registerTheme)({name:"material.dark",gridColor:"#515159",backgroundColor:c,axisColor:u,font:{color:u},primaryTitleColor:l,secondaryTitleColor:l,tooltip:{color:"#000"},"export":{backgroundColor:c,font:{color:"#dbdbdb"},button:{"default":{color:"#dedede",borderColor:"#4d4d4d",backgroundColor:c},hover:{color:"#dedede",borderColor:"#6c6c6c",backgroundColor:"#3f3f4b"},focus:{color:"#dedede",borderColor:"#8d8d8d",backgroundColor:"#494956"},active:{color:"#dedede",borderColor:"#8d8d8d",backgroundColor:"#494956"}},shadowColor:"#292929"},"chart:common":{commonSeriesSettings:{label:{border:{color:v}},valueErrorBar:{color:d}}},"chart:common:axis":{constantLineStyle:{color:d}},"chart:common:annotation":{border:{color:"#000"},color:"#000"},chart:{commonPaneSettings:{border:{color:v}},commonAxisSettings:{breakStyle:{color:y}},zoomAndPan:{dragBoxStyle:{color:d}}},gauge:{rangeContainer:{backgroundColor:f},valueIndicators:{_default:{color:f},rangebar:{color:"#84788b"},twocolorneedle:{secondColor:"#ba544d"},trianglemarker:{color:"#b7918f"},textcloud:{color:"#ba544d"}}},barGauge:{backgroundColor:"#3c3c3c"},rangeSelector:{scale:{tick:{color:d,opacity:.32},minorTick:{color:d,opacity:.1},breakStyle:{color:y}},selectedRangeColor:f,sliderMarker:{color:f,font:{color:c}},sliderHandle:{color:d,opacity:.2},shutter:{color:d,opacity:.1}},map:{background:{borderColor:"#3f3f3f"},layer:{label:{stroke:h,font:{color:d}}},"layer:area":{borderColor:c,color:p,hoveredBorderColor:d,selectedBorderColor:d},"layer:line":{color:"#c77244",hoveredColor:"#ff5d04",selectedColor:"#ff784f"},"layer:marker:bubble":{hoveredBorderColor:d,selectedBorderColor:d},"layer:marker:pie":{hoveredBorderColor:d,selectedBorderColor:d},legend:{border:{color:"#3f3f3f"},font:{color:d}},controlBar:{borderColor:g,color:c}},treeMap:{group:{color:"#4c4c4c",label:{font:{color:"#a3a3a3"}}}},sparkline:{lineColor:g,firstLastColor:g,barPositiveColor:m,barNegativeColor:_,winColor:m,lossColor:_,pointColor:c},bullet:{targetColor:_},funnel:{item:{border:{color:c}}},sankey:{label:{font:{color:d}}}},"material");var b={blue:"#03a9f4",lime:"#cddc39",orange:"#ff5722",purple:"#9c27b0",teal:"#009688"};for(var x in b)if(Object.prototype.hasOwnProperty.call(b,x)){var w=b[x];i(x,"light",w),i(x,"dark",w),(0,o.registerTheme)({name:"material.".concat(x,".light.compact")},"material.".concat(x,".light")),(0,o.registerTheme)({name:"material.".concat(x,".dark.compact")},"material.".concat(x,".dark"))}},function(e,t,n){function i(e,t,n){var i=(0,m.normalizeEnum)(e);return i in t?t[i]:n}function o(e){var t=i(e.side,D,1),n=[i(e.horizontalAlignment,k,w),i(e.verticalAlignment,S,x)];return{side:t,primary:a(n[t]),secondary:n[1-t],weak:e.weak,priority:e.priority||0,header:e.header,position:e.position}}function a(e){return e<2?0:2}function r(e){return 1-e}function s(e,t,n,i){return I[e](t,n,i)}function l(e,t){return(e>0?-1:1)*t}function u(e,t,n){var i=e.side,o=e.element.measure([t[2]-t[0],t[3]-t[1]]),a="indside"===e.position?0:n[i],r=o[i]<t[2+i]-t[i]-a;return r&&("inside"!==e.position&&(t[e.primary+i]+=l(e.primary,o[i])),e.size=o),r}function c(e,t,n){var i=e.side,o=r(i),a=[],u=s(e.secondary,n[o],n[2+o],e.size[o]);return a[i]=b(a[2+i]=t[e.primary+i]+("inside"===e.position?l(e.primary,e.size[i]):0)),a[e.primary+i]=b(t[e.primary+i]-l(e.primary,e.size[i])),"inside"!==e.position&&(t[e.primary+i]=a[e.primary+i]),a[o]=b(u[0]),a[2+o]=b(u[1]),a}function d(e,t,n,i,o,a){var r=c(e,t,n),s=c(e,i,i);o[e.side]>0?(o[e.side]-=e.size[e.side],a[e.primary+e.side]=r[e.primary+e.side],e.element.freeSpace()):e.element.move(r,s)}function h(){this._targets=[]}function f(e){var t,n,i=e.length,a=[];for(t=0;t<i;++t)n=e[t].layoutOptions(),n&&(n=o(n),n.element=e[t],a.push(n));return a.sort(function(e,t){return t.side-e.side||e.priority-t.priority}),a=p(a)}function p(e){var t,n=e.filter(function(e){return e.weak===!0})[0];return n&&(t=e.filter(function(e){return n.primary===e.primary&&e.side===n.side&&e!==n})[0]),n&&t?[_(t,n)].concat(e.filter(function(e){return!(e===t||e===n)})):e}function g(e,t){var n=t.slice(),i=c(e,n,n);return i[e.side]=t[e.side],i[2+e.side]=t[2+e.side],i}function _(e,t){var n=e.side,i=e.primary,o=e.secondary;return{side:n,primary:i,secondary:o,priority:0,element:{measure:function(o){var a=o.slice(),r=t.element.measure(o.slice());o[i]-=r[i];var s=e.element.measure(o.slice());return a[n]=r[n]=s[n]=Math.max(s[n],r[n]),t.size=r,e.size=s,a},move:function(n,i){if(i[2]-i[0]<e.size[0]+t.size[0]-T)return void this.freeSpace();var o=g(t,i,i);i[2+t.primary]=o[t.primary];var a=g(e,i,i);i[2+t.primary]<n[2+t.primary]&&e.size[e.primary]>n[2+e.primary]-n[e.primary]&&(n[2+t.primary]=i[2+t.primary]);var r=g(e,n,n);r[2+t.primary]>i[2+t.primary]&&(n[2+t.primary]=i[2+t.primary],r=g(e,n,n)),t.element.move(o),e.element.move(r,a)},freeSpace:function(){e.element.freeSpace(),t.element.freeSpace()}}}}t.default=void 0;var m=n(14),v=Math.min,y=Math.max,b=Math.round,x=0,w=1,C=2,k={left:x,center:w,right:C},S={top:x,center:w,bottom:C},D={horizontal:0,vertical:1},I={},T=2;I[x]=function(e,t,n){return[e,v(t,e+n)]},I[w]=function(e,t,n){return[y(e,(e+t-n)/2),v(t,(e+t+n)/2)]},I[C]=function(e,t,n){return[y(e,t-n),t]},h.prototype={constructor:h,dispose:function(){this._targets=null},add:function(e){this._targets.push(e)},forward:function(e,t){var n,i=e.slice(),o=f(this._targets),a=o.length,r=[];for(n=0;n<a;++n)u(o[n],i,t)?r.push(o[n]):o[n].element.freeSpace();return this._cache=r.reverse(),i},backward:function(e,t){var n,i,o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[0,0],a=e.slice(),r=e.slice(),s=this._cache,l=0,u=s.length;for(i=0;i<u;++i)n=s[i],n.side!==l&&(a=e.slice()),d(n,a,t,r,o,e),l=n.side;return o}};var O=h;t.default=O,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){function n(n){var i=e[n];o[i.name]=t(n)}function i(e,t,n){o[e](t),n&&n()}var o={};(0,u.each)(e,function(e,t){t.name&&n(e)});var a;return i.change=function(t){var n=e[t];return n&&((a=a||{})[t]=n),!!n},i.applyChanges=function(){a&&((0,u.each)(a,function(e,t){n(t.newName||e)}),a=null)},i.dispose=function(){e=t=o=null},i}function a(e){var t,n=function(){clearTimeout(t),t=setTimeout(e,100)};return n.dispose=function(){return clearTimeout(t),this},n}t.createEventTrigger=o,t.createResizeHandler=a,t.createIncidentOccurred=void 0;var r=i(n(160)),s=n(42),l=i(n(543)),u=n(3),c=l.default.ERROR_MESSAGES,d=function(e,t){return function(n,i){t("incidentOccurred",{target:{id:n,type:"E"===n[0]?"error":"warning",args:i,text:s.format.apply(null,[c[n]].concat(i||[])),widget:e,version:r.default}})}};t.createIncidentOccurred=d},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return"none"===(0,_.normalizeEnum)(e)}function a(e,t,n){this.fullState=E,this.series=e,this.update(t,n),this._viewCounters={hover:0,selection:0},this._emptySettings={fill:null,stroke:null,dashStyle:null}}t.Point=a;var r=i(n(151)),s=i(n(152)),l=i(n(229)),u=i(n(920)),c=i(n(544)),d=i(n(545)),h=i(n(921)),f=i(n(546)),p=i(n(922)),g=n(923),_=n(14),m=n(0),v=n(1),y=n(4),b={},x=m.extend,w=r.default.states,C="symbolPoint",k="polarSymbolPoint",S="barPoint",D="polarBarPoint",I="piePoint",T=w.selectedMark,O=w.hoverMark,E=w.normalMark,A=w.hover,P=w.normal,R=w.selection,M={chart:{scatter:C,line:C,spline:C,stepline:C,stackedline:C,fullstackedline:C,stackedspline:C,fullstackedspline:C,stackedsplinearea:C,fullstackedsplinearea:C,area:C,splinearea:C,steparea:C,stackedarea:C,fullstackedarea:C,rangearea:"rangeSymbolPoint",bar:S,stackedbar:S,fullstackedbar:S,rangebar:"rangeBarPoint",bubble:"bubblePoint",stock:"stockPoint",candlestick:"candlestickPoint"},pie:{pie:I,doughnut:I,donut:I},polar:{scatter:k,line:k,area:k,bar:D,stackedbar:D}};b.symbolPoint=s.default,b.barPoint=l.default,b.bubblePoint=u.default,b.piePoint=c.default,b.rangeSymbolPoint=d.default,b.rangeBarPoint=h.default,b.candlestickPoint=f.default,b.stockPoint=p.default,b.polarSymbolPoint=g.polarSymbolPoint,b.polarBarPoint=g.polarBarPoint,a.prototype={constructor:a,getColor:function(){return this.hasValue()||this._styles.usePointCustomOptions||this.series.customizePoint(this,this._dataItem),this._styles.normal.fill||this.series.getColor()},_getStyle:function(){return this._styles[this._currentStyle||"normal"]},update:function(e,t){this.updateOptions(t),this.updateData(e)},updateData:function(e){var t=this,n=t.argument!==e.argument;t.argument=t.initialArgument=t.originalArgument=e.argument,t.tag=e.tag,t.index=e.index,t._dataItem=e,t.data=e.data,t.lowError=e.lowError,t.highError=e.highError,t.aggregationInfo=e.aggregationInfo,t._updateData(e,n),!t.hasValue()&&t.setInvisibility(),t._fillStyle(),t._updateLabelData()},deleteMarker:function(){var e=this;e.graphic&&e.graphic.dispose(),e.graphic=null},draw:function(e,t,n,i){var o=this;return(o._needDeletingOnDraw||o.series.autoHidePointMarkers&&!o.isSelected())&&(o.deleteMarker(),o._needDeletingOnDraw=!1),o._needClearingOnDraw&&(o.clearMarker(),o._needClearingOnDraw=!1),o._hasGraphic()?o._updateMarker(n,this._getStyle(),t.markers):o.getMarkerVisibility()&&!o.series.autoHidePointMarkers&&o._drawMarker(e,t.markers,n,i),o._drawLabel(),o._drawErrorBar(e,t.errorBars,n),o},_getViewStyle:function(){var e=E,t=this.fullState,n=[P,A,R,R];return this._viewCounters.hover&&(e|=O),this._viewCounters.selection&&(e|=T),o(this.getOptions().selectionMode)&&(t&=~T),o(this.getOptions().hoverMode)&&(t&=~O),e|=t,n[e]},applyView:function(e){var t=this._getViewStyle(),n=this;n._currentStyle=t,n.graphic||!n.series.autoHidePointMarkers||t!==R&&t!==A||n._drawMarker(n.series.getRenderer(),n.series.getMarkersGroup()),n.graphic&&(n.series.autoHidePointMarkers&&t!==R&&t!==A?n.deleteMarker():("normal"===t?n.clearMarker():n.graphic.toForeground(),n._updateMarker(!0,n._styles[t],void 0,e)))},setView:function(e){this._viewCounters[e]++,this.applyView()},resetView:function(e){var t=this._viewCounters;--t[e],t[e]<0&&(t[e]=0),this.applyView()},releaseHoverState:function(){var e=this;e.graphic&&!e.isSelected()&&e.graphic.toBackground()},select:function(){this.series.selectPoint(this)},clearSelection:function(){this.series.deselectPoint(this)},hover:function(){this.series.hoverPoint(this)},clearHover:function(){this.series.clearPointHover()},showTooltip:function(){this.series.showPointTooltip(this)},hideTooltip:function(){this.series.hidePointTooltip(this)},_checkLabelsChanging:function(e,t){var n=~t.indexOf("range"),i=~e.indexOf("range");return i&&!n||!i&&n},updateOptions:function(e){if(e){var t=this,n=t._options,i=e.widgetType,o=n&&n.type,a=e.type,r=M[i][a];o!==a?(t._needDeletingOnDraw=!0,t._needClearingOnDraw=!1,o&&(t._checkLabelsChanging(o,a)&&t.deleteLabel(),t._resetType(b[M[o]])),t._setType(b[r])):(t._needDeletingOnDraw=t._checkSymbol(n,e),t._needClearingOnDraw=t._checkCustomize(n,e)),t._options=e,t._fillStyle(),t._updateLabelOptions(r)}},translate:function(){this.hasValue()&&(this._translate(),this.translated=!0)},_checkCustomize:function(e,t){return e.styles.usePointCustomOptions&&!t.styles.usePointCustomOptions},_getCustomLabelVisibility:function(){return this._styles.useLabelCustomOptions?!!this._options.label.visible:null},getBoundingRect:function(){return this._getGraphicBBox()},_resetType:function(e){for(var t in e)delete this[t]},_setType:function(e){for(var t in e)this[t]=e[t]},isInVisibleArea:function(){return this.inVisibleArea},isSelected:function(){return!!(this.fullState&T)},isHovered:function(){return!!(this.fullState&O)},getOptions:function(){return this._options},animate:function(e,t,n){return this.graphic?void this.graphic.animate(t,{partitionDuration:n},e):void(e&&e())},getCoords:function(e){var t=this;return e?t._options.rotated?{x:t.minX-(t.x-t.minX?0:1),y:t.y}:{x:t.x,y:t.minY+(t.y-t.minY?0:1)}:{x:t.x,y:t.y}},getDefaultCoords:function(){var e=this;return e._options.rotated?{x:e.defaultX,y:e.y}:{x:e.x,y:e.defaultY}},setDefaultCoords:function(){var e=this.getDefaultCoords();this.x=e.x,this.y=e.y},_getVisibleArea:function(){return this.series.getVisibleArea()},_getArgTranslator:function(){return this.series.getArgumentAxis().getTranslator()},_getValTranslator:function(){return this.series.getValueAxis().getTranslator()},_calculateVisibility:function(e,t,n,i){var o=this,a=o._getVisibleArea(),r=o._options.rotated;a.minX>e+(n||0)||a.maxX<e||a.minY>t+(i||0)||a.maxY<t||r&&(0,v.isDefined)(n)&&0!==n&&(a.minX===e+n||a.maxX===e)||!r&&(0,v.isDefined)(i)&&0!==i&&(a.minY===t+i||a.maxY===t)?o.inVisibleArea=!1:o.inVisibleArea=!0},isArgumentCorrect:function(){return this.series._argumentChecker(this.argument)},isValueCorrect:function(){var e=this.series._valueChecker;return e(this.getMinValue())&&e(this.getMaxValue())},hasValue:function(){return null!==this.value&&null!==this.minValue&&this.isArgumentCorrect()&&this.isValueCorrect()},hasCoords:y.noop,correctPosition:y.noop,correctRadius:y.noop,correctLabelRadius:y.noop,getCrosshairData:y.noop,getPointRadius:y.noop,_populatePointShape:y.noop,_checkSymbol:y.noop,getMarkerCoords:y.noop,hide:y.noop,show:y.noop,hideMarker:y.noop,setInvisibility:y.noop,clearVisibility:y.noop,isVisible:y.noop,resetCorrection:y.noop,correctValue:y.noop,resetValue:y.noop,setPercentValue:y.noop,correctCoordinates:y.noop,coordsIn:y.noop,getTooltipParams:y.noop,applyWordWrap:y.noop,setLabelTrackerData:y.noop,updateLabelCoord:y.noop,drawLabel:y.noop,correctLabelPosition:y.noop,getMinValue:y.noop,getMaxValue:y.noop,_drawErrorBar:y.noop,getMarkerVisibility:y.noop,dispose:function(){var e=this;e.deleteMarker(),e.deleteLabel(),e._errorBar&&this._errorBar.dispose(),e._options=e._styles=e.series=e._errorBar=null},getTooltipFormatObject:function(e,t){var n=this,i=n._getFormatObject(e),o=[],a=[];t&&(t.forEach(function(t){if(t.isVisible()){var n=t._getFormatObject(e);a.push(n),o.push(n.seriesName+": "+n.valueText)}}),x(i,{points:a,valueText:o.join("\n"),stackName:n.series.getStackName()||null}));var r=n.aggregationInfo;if(r){var s=n.series.getArgumentAxis(),l=s.formatRange(r.intervalStart,r.intervalEnd,r.aggregationInterval);l&&(i.valueText+="\n".concat(l))}return i},setHole:function(e,t){var n=this,i=isFinite(n.minValue)?n.minValue:0;(0,v.isDefined)(e)&&("left"===t?(n.leftHole=n.value-e,n.minLeftHole=i-e):(n.rightHole=n.value-e,n.minRightHole=i-e))},resetHoles:function(){this.leftHole=null,this.minLeftHole=null,this.rightHole=null,this.minRightHole=null},getLabel:function(){return this._label},getLabels:function(){return[this._label]},getCenterCoord:function(){return{x:this.x,y:this.y}}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=i(n(152)),r=o.extend,s=20,l=r({},a.default,{correctCoordinates:function(e){this.bubbleSize=e/2},_drawMarker:function(e,t,n){var i=this,o=r({translateX:i.x,translateY:i.y},i._getStyle());i.graphic=e.circle(0,0,n?0:i.bubbleSize).smartAttr(o).data({"chart-data-point":i}).append(t)},getTooltipParams:function(e){var t=this,n=t.graphic;if(n){var i=n.getBBox().height;return{x:t.x,y:t.y,offset:i<s||"edge"===e?i/2:0}}},_getLabelFormatObject:function(){var e=a.default._getLabelFormatObject.call(this);return e.size=this.initialSize,e},_updateData:function(e){a.default._updateData.call(this,e),this.size=this.initialSize=e.size},_getGraphicBBox:function(){var e=this;return e._getSymbolBBox(e.x,e.y,e.bubbleSize)},_updateMarker:function(e,t){var n=this;e||(t=r({r:n.bubbleSize,translateX:n.x,translateY:n.y},t)),n.graphic.smartAttr(t)},_getFormatObject:function(e){var t=a.default._getFormatObject.call(this,e);return t.sizeText=e.formatValue(this.initialSize),t},_storeTrackerR:function(){return this.bubbleSize},_getLabelCoords:function(e){var t;return t="inside"===e.getLayoutOptions().position?this._getLabelCoordOfPosition(e,"inside"):a.default._getLabelCoords.call(this,e)}});t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(4),a=n(0),r=i(n(229)),s=i(n(545)),l=a.extend,u=l({},r.default,{deleteLabel:s.default.deleteLabel,_getFormatObject:s.default._getFormatObject,clearVisibility:function(){var e=this.graphic;e&&e.attr("visibility")&&e.attr({visibility:null})},setInvisibility:function(){var e=this.graphic;e&&"hidden"!==e.attr("visibility")&&e.attr({visibility:"hidden"}),this._topLabel.draw(!1),this._bottomLabel.draw(!1)},getTooltipParams:function(e){var t,n,i=this,o="edge"===e;return i._options.rotated?(t=o?i.x+i.width:i.x+i.width/2,n=i.y+i.height/2):(t=i.x+i.width/2,n=o?i.y:i.y+i.height/2),{x:t,y:n,offset:0}},_translate:function(){var e=this,t=r.default;t._translate.call(e),e._options.rotated?e.width=e.width||1:e.height=e.height||1},hasCoords:s.default.hasCoords,_updateData:s.default._updateData,_getLabelPosition:s.default._getLabelPosition,_getLabelMinFormatObject:s.default._getLabelMinFormatObject,_updateLabelData:s.default._updateLabelData,_updateLabelOptions:s.default._updateLabelOptions,getCrosshairData:s.default.getCrosshairData,_createLabel:s.default._createLabel,_checkOverlay:s.default._checkOverlay,_checkLabelsOverlay:s.default._checkLabelsOverlay,_getOverlayCorrections:s.default._getOverlayCorrections,_drawLabel:s.default._drawLabel,_getLabelCoords:s.default._getLabelCoords,getLabel:s.default.getLabel,getLabels:s.default.getLabels,getBoundingRect:o.noop,getMinValue:s.default.getMinValue,getMaxValue:s.default.getMaxValue});t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(0),a=n(1),r=i(n(546)),s=o.extend,l=a.isNumeric,u=s({},r.default,{_getPoints:function(){var e=this,t=e._options.rotated?function(e,t){return[t,e]}:function(e,t){return[e,t]},n=l(e.openY),i=l(e.closeY),o=e.x,a=e.width,r=[].concat(t(o,e.highY));return n&&(r=r.concat(t(o,e.openY))),n&&(r=r.concat(t(o-a/2,e.openY))),n&&(r=r.concat(t(o,e.openY))),i&&(r=r.concat(t(o,e.closeY))),i&&(r=r.concat(t(o+a/2,e.closeY))),i&&(r=r.concat(t(o,e.closeY))),r=r.concat(t(o,e.lowY))},_drawMarkerInGroup:function(e,t,n){this.graphic=n.path(this._getPoints(),"line").attr({"stroke-linecap":"square"}).attr(t).data({"chart-data-point":this}).sharp().append(e)},_getMinTrackerWidth:function(){var e=2+this._styles.normal["stroke-width"];return e+e%2}});t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.polarBarPoint=t.polarSymbolPoint=void 0;var o=n(0),a=i(n(152)),r=i(n(229)),s=i(n(544)),l=n(1),u=n(14),c=i(n(151)),d=o.extend,h=Math,f=h.max,p=c.default.radialLabelIndent,g=90,_="canvas_position_end",m="canvas_position_default",v=d({},a.default,{_getLabelCoords:s.default._getLabelCoords,_getElementCoords:s.default._getElementCoords,_moveLabelOnCanvas:function(e,t,n){var i=e.x,o=e.y;return t.minX>i&&(i=t.minX),t.maxX<i+n.width&&(i=t.maxX-n.width),t.minY>o&&(o=t.minY),t.maxY<o+n.height&&(o=t.maxY-n.height),{x:i,y:o}},_getLabelPosition:function(){return"outside"},_getCoords:function(e,t){var n=this.series.getValueAxis(),i=n.getAngles()[0],o=this._getArgTranslator().translate(e),a=this._getValTranslator().translate(t),r=(0,u.convertPolarToXY)(n.getCenter(),n.getAngles()[0],o,a);return r.angle=o+i-90,r.radius=a,r},_translate:function(){var e=this,t=e.series.getValueAxis().getCenter(),n=e._getCoords(e.argument,e.value),i=e._getValTranslator().translate(_),o=(0,l.isDefined)(n.radius)&&n.radius>=0?n.radius:null;e.vx=(0,u.normalizeAngle)(n.angle),e.vy=e.radiusOuter=e.radiusLabels=o,e.radiusLabels+=p,e.radius=o,e.middleAngle=-n.angle,e.angle=-n.angle,e.x=n.x,e.y=n.y,e.defaultX=e.centerX=t.x,e.defaultY=e.centerY=t.y,e._translateErrorBars(),e.inVisibleArea=e._checkRadiusForVisibleArea(o,i)},_checkRadiusForVisibleArea:function(e,t){return(0,l.isDefined)(e)&&e<=t},_translateErrorBars:function(){var e=this,t=e._options.errorBars,n=e._getValTranslator();t&&((0,l.isDefined)(e.lowError)&&(e._lowErrorCoord=e.centerY-n.translate(e.lowError)),(0,l.isDefined)(e.highError)&&(e._highErrorCoord=e.centerY-n.translate(e.highError)),e._errorBarPos=e.centerX,e._baseErrorBarPos="stdDeviation"===t.type?e._lowErrorCoord+(e._highErrorCoord-e._lowErrorCoord)/2:e.centerY-e.radius)},_getTranslates:function(e){return e?this.getDefaultCoords():{x:this.x,y:this.y}},getDefaultCoords:function(){var e=(0,u.getCosAndSin)(-this.angle),t=this._getValTranslator().translate(m),n=this.defaultX+t*e.cos,i=this.defaultY+t*e.sin;return{x:n,y:i}},_addLabelAlignmentAndOffset:function(e,t){return t},_checkLabelPosition:function(e,t){var n=this,i=n._getVisibleArea(),o=n._getGraphicBBox();return n._isPointInVisibleArea(i,o)&&(t=n._moveLabelOnCanvas(t,i,e.getBoundingRect())),t},_getErrorBarSettings:function(e,t){var n=a.default._getErrorBarSettings.call(this,e,t);return n.rotate=g-this.angle,n.rotateX=this.centerX,n.rotateY=this.centerY,n},getCoords:function(e){return e?this.getDefaultCoords():{x:this.x,y:this.y}}});t.polarSymbolPoint=v;var y=d({},r.default,{_translateErrorBars:v._translateErrorBars,_getErrorBarSettings:v._getErrorBarSettings,_moveLabelOnCanvas:v._moveLabelOnCanvas,_getLabelCoords:s.default._getLabelCoords,_getElementCoords:s.default._getElementCoords,_getLabelConnector:s.default._getLabelConnector,
getTooltipParams:s.default.getTooltipParams,_getLabelPosition:s.default._getLabelPosition,_getCoords:v._getCoords,_translate:function(){var e=this,t=e._getValTranslator(),n=t.getBusinessRange(),i=t.translate(_);e.radiusInner=t.translate(e.minValue),v._translate.call(e),null===e.radiusInner?e.radiusInner=e.radius=i:null===e.radius?e.radius=e.value>=n.minVisible?i:0:e.radius>i&&(e.radius=i),e.radiusOuter=e.radiusLabels=f(e.radiusInner,e.radius),e.radiusLabels+=p,e.radiusInner=e.defaultRadius=h.min(e.radiusInner,e.radius),e.middleAngle=e.angle=-(0,u.normalizeAngle)(e.middleAngleCorrection-e.angle)},_checkRadiusForVisibleArea:function(e){return(0,l.isDefined)(e)||this._getValTranslator().translate(this.minValue)>0},_getErrorBarBaseEdgeLength:function(){var e=this.getMarkerCoords();return h.PI*e.outerRadius*h.abs(e.startAngle-e.endAngle)/180},getMarkerCoords:function(){return{x:this.centerX,y:this.centerY,outerRadius:this.radiusOuter,innerRadius:this.defaultRadius,startAngle:this.middleAngle-this.interval/2,endAngle:this.middleAngle+this.interval/2}},_drawMarker:function(e,t,n){var i=this,o=i._getStyle(),a=i.getMarkerCoords(),r=a.innerRadius,s=a.outerRadius,l=i._getCoords(i.argument,m),u=a.x,c=a.y;n&&(r=0,s=0,u=l.x,c=l.y),i.graphic=e.arc(u,c,r,s,a.startAngle,a.endAngle).attr(o).data({"chart-data-point":i}).append(t)},_checkLabelPosition:function(e,t){var n=this,i=n._getVisibleArea(),o=(0,u.getCosAndSin)(n.middleAngle),a=n.centerX+n.defaultRadius*o.cos,r=n.centerY-n.defaultRadius*o.sin;return a>i.minX&&a<i.maxX&&r>i.minY&&r<i.maxY&&(t=n._moveLabelOnCanvas(t,i,e.getBoundingRect())),t},_addLabelAlignmentAndOffset:function(e,t){return t},correctCoordinates:function(e){this.middleAngleCorrection=e.offset,this.interval=e.width},coordsIn:function(e,t){var n=(0,u.convertXYToPolar)(this.series.getValueAxis().getCenter(),e,t),i=this.getMarkerCoords(),o=i.startAngle<i.endAngle?-n.phi>=i.startAngle&&-n.phi<=i.endAngle:-n.phi<=i.startAngle&&-n.phi>=i.endAngle;return n.r>=i.innerRadius&&n.r<=i.outerRadius&&o}});t.polarBarPoint=y},function(e,t,n){function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.chart=void 0;var o=n(0),a=n(1),r=n(14),s=n(4),l=n(132),u=n(173),c=n(202),d=o.extend,h=u.chart.bar,f=c.chart.area,p={};t.chart=p;var g={areErrorBarsVisible:s.noop,_createErrorBarGroup:s.noop,_checkData:function(e,t){var n=this.getValueFields();return l.chart._checkData.call(this,e,t,{minValue:n[0],value:n[1]})&&e.minValue===e.minValue},getValueRangeInitialValue:l.chart.getValueRangeInitialValue,_getPointDataSelector:function(e){var t=this,n=this.getValueFields(),i=n[0],o=n[1],a=this.getTagField(),r=this.getArgumentField();return function(e){return{tag:e[a],minValue:t._processEmptyValue(e[i]),value:t._processEmptyValue(e[o]),argument:e[r],data:e}}},_defaultAggregator:"range",_aggregators:{range:function(e,t){var n,o=e.intervalStart,r=e.data;if(r.length){var s=t.getValueFields(),l=s[0],u=s[1],c=r.reduce(function(e,t){var n=t[l],i=t[u];return(0,a.isDefined)(n)&&(0,a.isDefined)(i)?(e[l]=Math.min(e[l],Math.min(n,i)),e[u]=Math.max(e[u],Math.max(n,i)),e):e},(n={},i(n,l,1/0),i(n,u,-(1/0)),i(n,t.getArgumentField(),o),n));if(!isFinite(c[l])||!isFinite(c[u])){if(r.filter(function(e){return null===e[l]&&null===e[u]}).length!==r.length)return;c[l]=c[u]=null}return c}}},getValueFields:function(){return[this._options.rangeValue1Field||"val1",this._options.rangeValue2Field||"val2"]},getSeriesPairCoord:function(e,t){for(var n=null,i=this._options.rotated,o=!t&&!i||t&&i,a=o?"vy":"vx",r=i?"minX":"minY",s=o?"vx":"vy",l=this.getPoints(),u=0;u<l.length;u++){var c=l[u],d=void 0;if(t)d=c.getCenterCoord()[a[1]]===e?c[s]:void 0;else{var h=[Math.min(c[a],c[r]),Math.max(c[a],c[r])];d=e>=h[0]&&e<=h[1]?c[s]:void 0}if(this._checkAxisVisibleAreaCoord(!t,d)){n=d;break}}return n}};p.rangebar=d({},h,g),p.rangearea=d({},f,{_drawPoint:function(e){var t=e.point;t.isInVisibleArea()?(t.clearVisibility(),t.draw(this._renderer,e.groups),this._drawnPoints.push(t),t.visibleTopMarker||t.hideMarker("top"),t.visibleBottomMarker||t.hideMarker("bottom")):t.setInvisibility()},_prepareSegment:function(e,t){var n=this._processSinglePointsAreaSegment(e,t),i=(0,r.map)(n,function(e){return e.getCoords(!0)});return{line:n,bottomLine:i,area:(0,r.map)(n,function(e){return e.getCoords()}).concat(i.slice().reverse()),singlePointSegment:n!==e}},_getDefaultSegment:function(e){var t=f._getDefaultSegment.call(this,e);return t.bottomLine=t.line,t},_removeElement:function(e){f._removeElement.call(this,e),e.bottomLine&&e.bottomLine.remove()},_drawElement:function(e,t){var n=this,i=f._drawElement.call(n,e,t);return i.bottomLine=n._bordersGroup&&n._createBorderElement(e.bottomLine,{"stroke-width":n._styles.normal.border["stroke-width"]}).append(n._bordersGroup),i},_applyStyle:function(e){var t=this,n=t._elementsGroup,i=t._bordersGroup;n&&n.smartAttr(e.elements),i&&i.attr(e.border),(t._graphics||[]).forEach(function(t){t.line&&t.line.attr({"stroke-width":e.border["stroke-width"]}),t.bottomLine&&t.bottomLine.attr({"stroke-width":e.border["stroke-width"]})})},_updateElement:function(e,t,n,i){var o={points:t.bottomLine},a=e.bottomLine;f._updateElement.apply(this,arguments),a&&(n?a.animate(o):a.attr(o))}},g)},function(e,t,n){function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}t.chart=void 0;var o=n(231),a=n(132),r=n(202),s=n(173),l=n(0),u=n(3),c=n(4),d=o.chart.line,h=r.chart.area,f=s.chart.bar,p=s.polar.bar,g=l.extend,_=u.each,m=c.noop,v={};t.chart=v,v.bubble=g({},a.chart,{_calculateErrorBars:m,_getMainColor:f._getMainColor,_createPointStyles:f._createPointStyles,_updatePointsVisibility:f._updatePointsVisibility,_getOptionsForPoint:f._getOptionsForPoint,_applyMarkerClipRect:d._applyElementsClipRect,_parsePointStyle:p._parsePointStyle,_createLegendState:h._createLegendState,_setMarkerGroupSettings:p._setMarkerGroupSettings,areErrorBarsVisible:m,_createErrorBarGroup:m,_checkData:function(e,t){return a.chart._checkData.call(this,e,t,{value:this.getValueFields()[0],size:this.getSizeField()})},_getPointDataSelector:function(e,t){var n=this.getSizeField(),i=a.chart._getPointDataSelector.call(this);return function(e){var t=i(e);return t.size=e[n],t}},_aggregators:{avg:function(e,t){var n,o=e.data,a=e.intervalStart;if(o.length){var r=t.getValueFields()[0],s=t.getSizeField(),l=o.reduce(function(e,t){return e[0]+=t[r],e[1]+=t[s],e[2]++,e},[0,0,0]);return n={},i(n,r,l[0]/l[2]),i(n,s,l[1]/l[2]),i(n,t.getArgumentField(),a),n}}},getValueFields:function(){return[this._options.valueField||"val"]},getSizeField:function(){return this._options.sizeField||"size"},_animate:function(){var e=this,t=e._drawnPoints.length-1,n=e._labelsGroup,i=function(){n&&n.animate({opacity:1},{duration:e._defaultDuration})};_(e._drawnPoints||[],function(e,n){n.animate(e===t?i:void 0,{r:n.bubbleSize,translateX:n.x,translateY:n.y})})},_patchMarginOptions:function(e){return e.processBubbleSize=!0,e}})},function(e,t,n){t.donut=t.doughnut=t.pie=void 0;var i=n(4),o=n(3),a=n(132),r=n(14),s=n(0),l=n(173),u=a.chart,c=l.chart.bar,d=s.extend,h=o.each,f=i.noop,p=r.map,g=isFinite,_=Math.max,m=.7,v="inside",y=d({},c,{_setGroupsSettings:function(){u._setGroupsSettings.apply(this,arguments),this._labelsGroup.attr({"pointer-events":null})},_createErrorBarGroup:f,_drawPoint:function(e){var t=e.point,n=this._legendCallback;u._drawPoint.call(this,e),!t.isVisible()&&t.setInvisibility(),t.isSelected()&&n()},_getOldPoint:function(e,t,n){var i=(this._points||[])[n];return i&&(t[i.argument.valueOf()]=t[i.argument.valueOf()].filter(function(e){return e!==i})),i},adjustLabels:function(e){return(this._points||[]).reduce(function(t,n){if(n._label.isVisible())return n.setLabelTrackerData(),t=n.applyWordWrap(e)||t,n.updateLabelCoord(e),t},!1)},_applyElementsClipRect:f,getColor:f,areErrorBarsVisible:f,drawLabelsWOPoints:function(){var e=this;return e._options.label.position!==v&&(e._labelsGroup.append(e._extGroups.labelsGroup),(e._points||[]).forEach(function(e){e.drawLabel()}),!0)},getPointsCount:function(){var e=this;return this._data.filter(function(t){return e._checkData(t)}).length},setMaxPointsCount:function(e){this._pointsCount=e},_getCreatingPointOptions:function(e,t){return this._getPointOptions(e,t)},_updateOptions:function(e){this.labelSpace=0,this.innerRadius="pie"===this.type?0:e.innerRadius},_checkData:function(e,t){var n=c._checkData.call(this,e,t,{value:this.getValueFields()[0]});return this._options.paintNullPoints?n:n&&null!==e.value},_createGroups:u._createGroups,_setMarkerGroupSettings:function(){this._markersGroup.attr({"class":"dxc-markers"})},_getMainColor:function(e,t){var n=this.getPointsByArg(e.argument),i=t?n.indexOf(t):n.length;return this._options.mainSeriesColor(e.argument,i,this._pointsCount)},_getPointOptions:function(e){return this._parsePointOptions(this._preparePointOptions(),this._options.label,e)},_getRangeData:function(){return this._rangeData},_createPointStyles:function(e,t,n){var i=this,o=e.color||i._getMainColor(t,n);return{normal:i._parsePointStyle(e,o,o),hover:i._parsePointStyle(e.hoverStyle,o,o),selection:i._parsePointStyle(e.selectionStyle,o,o),legendStyles:{normal:i._createLegendState(e,o),hover:i._createLegendState(e.hoverStyle,o),selection:i._createLegendState(e.selectionStyle,o)}}},_getArrangeMinShownValue:function(e,t){var n=this._options.minSegmentSize,i=0,o=0;return t=t||e.length,h(e,function(e,a){a.isVisible()&&(a.normalInitialValue<n*t/360?i+=n:o+=a.normalInitialValue)}),i<360?n*o/(360-i):0},_applyArrangeCorrection:function(e,t,n){var i,o=this._options,a="anticlockwise"!==o.segmentsDirection,s=g(o.startAngle)?(0,r.normalizeAngle)(o.startAngle):0,l=o.minSegmentSize,u=0,c=0;0===n&&(n=e.filter(function(e){return e.isVisible()}).length,c=1),h(a?e:e.concat([]).reverse(),function(e,o){var a,r=o.isVisible()?c||o.normalInitialValue:0;l&&o.isVisible()&&r<t&&(a=t),i=r/n,o.correctValue(u,i,c+(a||0)),o.shiftedAngle=s,u+=a||r}),this._rangeData={val:{min:0,max:u}}},_removePoint:function(e){var t=this.getPointsByArg(e.argument);t.splice(t.indexOf(e),1),e.dispose()},arrangePoints:function(){for(var e,t=this,n=t._points||[],i=t._options.minSegmentSize,o=!0,a=0,r=n.length;a<r&&o;)o=n[a].value<=0,a++;var s=t._points=p(n,function(e){return null===e.value||!o&&e.value<0?(t._removePoint(e),null):e}),l=s.reduce(function(e,t){return _(e,Math.abs(t.initialValue))},0);s.forEach(function(e){e.normalInitialValue=e.initialValue/(0!==l?l:1)});var u=s.reduce(function(e,t){return e+(t.isVisible()?t.normalInitialValue:0)},0);i&&(e=this._getArrangeMinShownValue(s,u)),t._applyArrangeCorrection(s,e,u)},correctPosition:function(e,t){h(this._points,function(t,n){n.correctPosition(e)}),this.setVisibleArea(t)},correctRadius:function(e){this._points.forEach(function(t){t.correctRadius(e)})},correctLabelRadius:function(e){this._points.forEach(function(t){t.correctLabelRadius(e)})},setVisibleArea:function(e){this._visibleArea={minX:e.left,maxX:e.width-e.right,minY:e.top,maxY:e.height-e.bottom}},_applyVisibleArea:f,_animate:function(e){var t,n=this,i=n._points,o=i&&i.length,a=function(){n._animateComplete()};t=e?function(e,t){e.animate(t===o-1?a:void 0,m,(1-m)*t/(o-1))}:function(e,t){e.animate(t===o-1?a:void 0)},i.forEach(t)},getVisiblePoints:function(){return p(this._points,function(e){return e.isVisible()?e:null})},getPointsByKeys:function(e,t){var n=this.getPointsByArg(e);return n[t]&&[n[t]]||[]}});t.pie=y;var b=y;t.doughnut=b;var x=y;t.donut=x},function(e,t,n){t.candlestick=t.stock=void 0;var i=n(132),o=n(173),a=n(0),r=n(1),s=n(14),l=n(4),u=o.chart.bar,c=10,d=(0,a.extend)({},i.chart,{_animate:l.noop,_applyMarkerClipRect:function(e){e["clip-path"]=this._forceClipping?this._paneClipRectID:this._widePaneClipRectID},_updatePointsVisibility:u._updatePointsVisibility,_getOptionsForPoint:u._getOptionsForPoint,_createErrorBarGroup:l.noop,areErrorBarsVisible:l.noop,_createGroups:i.chart._createGroups,_setMarkerGroupSettings:function(){var e=this,t=e._markersGroup,n=e._createPointStyles(e._getMarkerGroupOptions()),i=(0,a.extend)(n.normal,{"class":"default-markers"}),o=(0,a.extend)(n.positive.normal,{"class":"default-positive-markers"}),r=(0,a.extend)(n.reduction.normal,{"class":"reduction-markers"}),s=(0,a.extend)(n.reductionPositive.normal,{"class":"reduction-positive-markers"}),l={"class":"dxc-markers"};e._applyMarkerClipRect(l),t.attr(l),e._createGroup("defaultMarkersGroup",t,t,i),e._createGroup("reductionMarkersGroup",t,t,r),e._createGroup("defaultPositiveMarkersGroup",t,t,o),e._createGroup("reductionPositiveMarkersGroup",t,t,s)},_setGroupsSettings:function(){i.chart._setGroupsSettings.call(this,!1)},_getCreatingPointOptions:function(){var e,t=this,n=t._predefinedPointOptions;return n||(e=this._getPointOptions(),t._predefinedPointOptions=n=(0,a.extend)(!0,{styles:{}},e),n.styles.normal=n.styles.positive.normal=n.styles.reduction.normal=n.styles.reductionPositive.normal={"stroke-width":e.styles&&e.styles.normal&&e.styles.normal["stroke-width"]}),n},_checkData:function(e,t){var n=this.getValueFields();return i.chart._checkData.call(this,e,t,{openValue:n[0],highValue:n[1],lowValue:n[2],closeValue:n[3]})&&e.highValue===e.highValue&&e.lowValue===e.lowValue},_getPointDataSelector:function(e,t){var n,i=this,o=this,a=o.getValueFields(),l=o.getArgumentField(),u=a[0],c=a[1],d=a[2],h=a[3];switch(o.level=o._options.reduction.level,(0,s.normalizeEnum)(o.level)){case"open":n=u;break;case"high":n=c;break;case"low":n=d;break;default:n=h,o.level="close"}var f;return function(e){var t=e[n],a=!1;return(0,r.isDefined)(t)&&((0,r.isDefined)(f)&&(a=t<f),f=t),{argument:e[l],highValue:i._processEmptyValue(e[c]),lowValue:i._processEmptyValue(e[d]),closeValue:i._processEmptyValue(e[h]),openValue:i._processEmptyValue(e[u]),reductionValue:t,tag:e[o.getTagField()],isReduction:a,data:e}}},_parsePointStyle:function(e,t,n){return{stroke:e.color||t,"stroke-width":e.width,fill:e.color||n}},_getDefaultStyle:function(e){var t=this,n=e.color||t._options.mainSeriesColor;return{normal:t._parsePointStyle(e,n,n),hover:t._parsePointStyle(e.hoverStyle,n,n),selection:t._parsePointStyle(e.selectionStyle,n,n)}},_getReductionStyle:function(e){var t=this,n=e.reduction.color;return{normal:t._parsePointStyle({color:n,width:e.width,hatching:e.hatching},n,n),hover:t._parsePointStyle(e.hoverStyle,n,n),selection:t._parsePointStyle(e.selectionStyle,n,n)}},_createPointStyles:function(e){var t=this,n=t._options.innerColor,i=t._getDefaultStyle(e),o=(0,a.extend)(!0,{},i),r=t._getReductionStyle(e),s=(0,a.extend)(!0,{},r);return o.normal.fill=o.hover.fill=o.selection.fill=n,s.normal.fill=s.hover.fill=s.selection.fill=n,i.positive=o,i.reduction=r,i.reductionPositive=s,i},_endUpdateData:function(){delete this._predefinedPointOptions},_defaultAggregator:"ohlc",_aggregators:{ohlc:function(e,t){var n=e.intervalStart,i=e.data;if(i.length){var o={},a=t.getValueFields(),r=a[1],s=a[2];return o[r]=-(1/0),o[s]=1/0,o=i.reduce(function(e,t){return null!==t[r]&&(e[r]=Math.max(e[r],t[r])),null!==t[s]&&(e[s]=Math.min(e[s],t[s])),e},o),o[a[0]]=i[0][a[0]],o[a[3]]=i[i.length-1][a[3]],isFinite(o[r])||(o[r]=null),isFinite(o[s])||(o[s]=null),o[t.getArgumentField()]=n,o}}},getValueFields:function(){var e=this._options;return[e.openValueField||"open",e.highValueField||"high",e.lowValueField||"low",e.closeValueField||"close"]},getArgumentField:function(){return this._options.argumentField||"date"},_patchMarginOptions:function(e){var t=this._getCreatingPointOptions(),n=t.styles,i=[n.normal,n.hover,n.selection].reduce(function(e,t){return Math.max(e,t["stroke-width"])},0);return e.size=c+i,e.sizePointNormalState=c,e},getSeriesPairCoord:function(e,t){for(var n=null,i=this.getVisiblePoints(),o=0;o<i.length;o++){var a=i[o],r=void 0;if(t)r=a.vx===e?(a.openY+a.closeY)/2:void 0;else{var s=[Math.min(a.lowY,a.highY),Math.max(a.lowY,a.highY)];r=e>=s[0]&&e<=s[1]?a.vx:void 0}if(this._checkAxisVisibleAreaCoord(!t,r)){n=r;break}}return n},usePointsToDefineAutoHiding:function(){return!1}});t.stock=d;var h=(0,a.extend)({},d,{_parsePointStyle:function(e,t,n){var i=e.color||n,o=d._parsePointStyle.call(this,e,t,i);return o.fill=i,o.hatching=e.hatching,o}});t.candlestick=h},function(e,t,n){function i(e,t,n,i){return e=(0,f.clone)(e),e.value=t,e.minValue=n,e.translate(),e.argument=e.argument+i,e}function o(e){for(var t,n,o=0,a=[],r=e.length;o<r;)t=e[o],n=[t],t.leftHole&&(n=[i(t,t.leftHole,t.minLeftHole,"left"),t]),t.rightHole&&n.push(i(t,t.rightHole,t.minRightHole,"right")),a.push(n),o++;return[].concat.apply([],a)}function a(e,t){for(var n;!n&&e;)n=e._segmentByArg&&e._segmentByArg[t],e=e._prevSeries;return n}t.polar=t.chart=void 0;var r=n(4),s=n(0),l=n(3),u=n(202),c=n(173),d=n(231),h=n(14),f=n(50),p=u.chart.area,g=c.chart.bar,_={_calculateErrorBars:r.noop,_updateOptions:function(e){this._stackName="axis_"+(e.axis||"default")}},m={};t.chart=m;var v={};t.polar=v,m.stackedline=(0,s.extend)({},d.chart.line,_,{}),m.stackedspline=(0,s.extend)({},d.chart.spline,_,{}),m.fullstackedline=(0,s.extend)({},d.chart.line,_,{getValueRangeInitialValue:u.chart.area.getValueRangeInitialValue}),m.fullstackedspline=(0,s.extend)({},d.chart.spline,_,{getValueRangeInitialValue:u.chart.area.getValueRangeInitialValue});var y=m.stackedbar=(0,s.extend)({},g,_,{_updateOptions:function(e){_._updateOptions.call(this,e),this._stackName=this._stackName+"_stack_"+(e.stack||"default")}});m.fullstackedbar=(0,s.extend)({},g,_,{_updateOptions:y._updateOptions}),m.stackedarea=(0,s.extend)({},p,_,{_prepareSegment:function(e,t){return p._prepareSegment.call(this,o(e),t)},_appendInGroup:function(){this._group.append(this._extGroups.seriesGroup).toBackground()}}),m.stackedsplinearea=(0,s.extend)({},u.chart.splinearea,_,{_prepareSegment:function(e,t){var n,i=this;if(e=o(e),this._prevSeries&&1!==e.length){for(var r=d.chart.spline._calculateBezierPoints(e,t),s=(0,h.map)(e,function(e){var t=e.getCoords(!0);return t.argument=e.argument,t}),c=[],f={},p=0,g=i._prevSeries._segments.length;p<g;)c=c.concat(i._prevSeries._segments[p].line),p++;(0,l.each)(c,function(e,t){if(null!==t.argument){var n=t.argument.valueOf();f[n]?f[n].push(t):f[n]=[t]}}),i._prevSeries._segmentByArg=f,s=d.chart.spline._calculateBezierPoints(s,t),(0,l.each)(s,function(e,t){var n,o=t.argument.valueOf();e%3===0&&(n=f[o]||a(i._prevSeries,o),n&&(s[e-1]&&n[0]&&(s[e-1]=n[0]),s[e+1]&&(s[e+1]=n[2]||t)))}),n={line:r,area:r.concat(s.reverse())},i._areaPointsToSplineAreaPoints(n.area)}else n=u.chart.splinearea._prepareSegment.call(this,e,t);return n},_appendInGroup:m.stackedarea._appendInGroup}),m.fullstackedarea=(0,s.extend)({},p,_,{_prepareSegment:m.stackedarea._prepareSegment,_appendInGroup:m.stackedarea._appendInGroup}),m.fullstackedsplinearea=(0,s.extend)({},u.chart.splinearea,_,{_prepareSegment:m.stackedsplinearea._prepareSegment,_appendInGroup:m.stackedarea._appendInGroup}),v.stackedbar=(0,s.extend)({},c.polar.bar,_,{})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=e.target;if("tspan"===i.tagName)return i.parentNode[t];var o=i[t];if(n&&!(0,m.isDefined)(o)){var a=function e(n){if(n.parentNode)return(0,m.isDefined)(n.parentNode[t])?n.parentNode[t]:e(n.parentNode)};return a(i)}return o}function a(e,t){return e.cancel||!t.getOptions()}function r(e){return H.indexOf(e)>-1?e:F}function s(e){var t=e.getOptions().hoverMode;return t===B?t:V}t.PieTracker=t.ChartTracker=void 0;var l=i(n(11)),u=i(n(5)),c=n(21),d=n(0),h=n(3),f=i(n(151)),p=n(14),g=i(n(25)),_=n(8),m=n(1),v=n(4),y=Math.floor,b=f.default.events,x=f.default.states,w=x.hoverMark,C=x.normalMark,k="dxChartTracker",S="."+k,D=(0,_.addNamespace)([g.default.down,g.default.move],k),I="legendClick",T="seriesClick",O="pointClick",E="chart-data-point",A="chart-data-series",P="chart-data-argument",R=100,M=300,B="none",V="allargumentpoints",F="includepoints",L="excludepoints",H=[F,L,B],z={ctor:function(e){var t=this,n={tracker:t};t._renderer=e.renderer,t._legend=e.legend,t._tooltip=e.tooltip,t._eventTrigger=e.eventTrigger,t._seriesGroup=e.seriesGroup,e.seriesGroup.off(S).on((0,_.addNamespace)(b.showPointTooltip,k),n,t._showPointTooltip).on((0,_.addNamespace)(b.hidePointTooltip,k),n,t._hidePointTooltip),t._renderer.root.off(S).on(D,n,t._pointerHandler).on((0,_.addNamespace)(g.default.up,k),function(){return clearTimeout(t._holdTimer)}).on((0,_.addNamespace)(c.name,k),n,t._clickHandler)},update:function(e){this._chart=e.chart},updateSeries:function(e,t){var n=this,i=!(null!==e&&void 0!==e&&e.some(function(e){return e===n.hoveredSeries})||n._hoveredPoint&&n._hoveredPoint.series);n._storedSeries!==e&&(n._storedSeries=e||[]),i&&(n._clean(),n._renderer.initHatching()),t&&(n.clearSelection(),i||(n._hideTooltip(n.pointAtShownTooltip),n.clearHover()))},setCanvases:function(e,t){this._mainCanvas=e,this._canvases=t},repairTooltip:function(){var e=this.pointAtShownTooltip;e&&e.series&&e.isVisible()?this._showTooltip(e):this._hideTooltip(e,!0)},_setHoveredPoint:function(e){e!==this._hoveredPoint&&(this._releaseHoveredPoint(),e.hover(),this._hoveredPoint=e)},_releaseHoveredPoint:function(){this._hoveredPoint&&this._hoveredPoint.getOptions()&&(this._hoveredPoint.clearHover(),this._hoveredPoint=null,this._tooltip.isEnabled()&&this._hideTooltip(this._hoveredPoint))},_setHoveredSeries:function(e,t){this._releaseHoveredSeries(),this._releaseHoveredPoint(),e.hover(t),this.hoveredSeries=e},_releaseHoveredSeries:function(){this.hoveredSeries&&(this.hoveredSeries.clearHover(),this.hoveredSeries=null)},clearSelection:function(){this._storedSeries.forEach(function(e){e&&(e.clearSelection(),e.getPoints().forEach(function(e){return e.clearSelection()}))})},_clean:function(){var e=this;e.hoveredPoint=e.hoveredSeries=e._hoveredArgumentPoints=null,e._hideTooltip(e.pointAtShownTooltip)},clearHover:function(){this._resetHoveredArgument(),this._releaseHoveredSeries(),this._releaseHoveredPoint()},_hideTooltip:function(e,t){var n=this;!n._tooltip||e&&n.pointAtShownTooltip!==e||(!t&&n.pointAtShownTooltip&&(n.pointAtShownTooltip=null),n._tooltip.hide())},_showTooltip:function(e){var t,n=this,i={target:e};if(null!==e&&void 0!==e&&e.getOptions()){if(t=e.getTooltipFormatObject(n._tooltip,n._tooltip.isShared()&&n._chart.getStackedPoints(e)),!(0,m.isDefined)(t.valueText)&&!t.points||!e.isVisible())return;var o=e.getTooltipParams(n._tooltip.getLocation()),a=n._renderer.getRootOffset();o.x+=a.left,o.y+=a.top;var r=function(t){t&&(n.pointAtShownTooltip=e)};r(n._tooltip.show(t,o,i,void 0,r))}},_showPointTooltip:function(e,t){var n=e.data.tracker,i=n.pointAtShownTooltip;i&&i!==t&&n._hideTooltip(i),n._showTooltip(t)},_hidePointTooltip:function(e,t){e.data.tracker._hideTooltip(t)},_enableOutHandler:function(){if(!this._outHandler){var e=this,t=function(t){var n=e._renderer.getRootOffset(),i=y(t.pageX-n.left),o=y(t.pageY-n.top);(0,p.pointInCanvas)(e._mainCanvas,i,o)||e._tooltip.isCursorOnTooltip(t.pageX,t.pageY)||(e._pointerOut(),e._disableOutHandler())};u.default.on(l.default.getDocument(),D,t),this._outHandler=t}},_disableOutHandler:function(){this._outHandler&&u.default.off(l.default.getDocument(),D,this._outHandler),this._outHandler=null},stopCurrentHandling:function(){this._pointerOut(!0)},_pointerOut:function(e){this.clearHover(),(e||this._tooltip.isEnabled())&&this._hideTooltip(this.pointAtShownTooltip)},_triggerLegendClick:function(e,t){var n=this._eventTrigger;n(I,e,function(){!a(e.event,e.target)&&n(t,e)})},_hoverLegendItem:function(e,t){var n,i=this,o=i._legend.getItemByCoord(e,t),a=r(i._legend.getOptions().hoverMode);o?(n=i._storedSeries[o.id],n.isHovered()&&n.lastHoverMode===a||i._setHoveredSeries(n,a),i._tooltip.isEnabled()&&i._hideTooltip(i.pointAtShownTooltip)):i.clearHover()},_hoverArgument:function(e,t){var n=this,i=n._getArgumentHoverMode();(0,m.isDefined)(e)&&(n._releaseHoveredPoint(),n._hoveredArgument=e,n._argumentIndex=t,n._notifySeries({action:"pointHover",notifyLegend:n._notifyLegendOnHoverArgument,target:{argument:e,fullState:w,argumentIndex:t,getOptions:function(){return{hoverMode:i}}}}))},_resetHoveredArgument:function(){var e,t=this;(0,m.isDefined)(t._hoveredArgument)&&(e=t._getArgumentHoverMode(),t._notifySeries({action:"clearPointHover",notifyLegend:t._notifyLegendOnHoverArgument,target:{fullState:C,argumentIndex:t._argumentIndex,argument:t._hoveredArgument,getOptions:function(){return{hoverMode:e}}}}),t._hoveredArgument=null)},_notifySeries:function(e){this._storedSeries.forEach(function(t){t.notify(e)})},_pointerHandler:function(e){var t,n=e.data.tracker,i=n._renderer.getRootOffset(),a=y(e.pageX-i.left),r=y(e.pageY-i.top),s=n._getCanvas(a,r),l=o(e,A),u=o(e,E)||(null===(t=l)||void 0===t?void 0:t.getPointByCoord(a,r));if(n._isHolding=!1,clearTimeout(n._holdTimer),e.type===g.default.down&&(n._holdTimer=setTimeout(function(){return n._isHolding=!0},M)),u&&!u.getMarkerVisibility()&&(u=void 0),n._enableOutHandler(),n._legend.coordsIn(a,r))return void n._hoverLegendItem(a,r);if(n.hoveredSeries&&n.hoveredSeries!==n._stuckSeries&&n._releaseHoveredSeries(),!n._hoverArgumentAxis(a,r,e)&&(n._isPointerOut(s,u)&&n._pointerOut(),s||u)){if(l&&!u){if(u=l.getNeighborPoint(a,r),n._stickyHovering||!u||u.coordsIn(a,r)||(u=null),l!==n.hoveredSeries)return void n._setTimeout(function(){n._setHoveredSeries(l),n._setStuckSeries(e,l,a,r),n._pointerComplete(u,a,r)},l)}else{if(u){if(e.type!==g.default.move&&"touch"!==e.pointerType)return;return void(n.hoveredSeries?n._setTimeout(function(){return n._pointerOnPoint(u,a,r,e)},u):n._pointerOnPoint(u,a,r,e))}if(n._setStuckSeries(e,void 0,a,r)&&n._stickyHovering){var c;l=n._stuckSeries,u=l.getNeighborPoint(a,r),n._releaseHoveredSeries(),(null===(c=u)||void 0===c?void 0:c.getMarkerVisibility())&&n._setHoveredPoint(u)}else n._stickyHovering||n._pointerOut()}n._pointerComplete(u,a,r)}},_pointerOnPoint:function(e,t,n){this._resetHoveredArgument(),this._setHoveredPoint(e),this._pointerComplete(e,t,n)},_pointerComplete:function(e){this.pointAtShownTooltip!==e&&this._tooltip.isEnabled()&&this._showTooltip(e)},_clickHandler:function(e){var t,n=e.data.tracker;if(!n._isHolding){var i=n._renderer.getRootOffset(),a=y(e.pageX-i.left),r=y(e.pageY-i.top),s=o(e,E),l=n._stuckSeries||o(e,A)||(null===(t=s)||void 0===t?void 0:t.series),u=n._argumentAxis;if(n._legend.coordsIn(a,r)){var c=n._legend.getItemByCoord(a,r);c&&n._legendClick(c,e)}else if(null!==u&&void 0!==u&&u.coordsIn(a,r)){var d=o(e,P,!0);(0,m.isDefined)(d)&&n._eventTrigger("argumentAxisClick",{argument:d,event:e})}else if(l){var h;s=s||l.getPointByCoord(a,r),null!==(h=s)&&void 0!==h&&h.getMarkerVisibility()?n._pointClick(s,e):o(e,A)&&n._eventTrigger(T,{target:l,event:e})}}},dispose:function(){var e=this;e._disableOutHandler(),e._renderer.root.off(S),e._seriesGroup.off(S)}},N=function(e){this.ctor(e)};t.ChartTracker=N,(0,d.extend)(N.prototype,z,{_pointClick:function(e,t){var n=this,i=n._eventTrigger,o=e.series;i(O,{target:e,event:t},function(){!a(t,o)&&i(T,{target:o,event:t})})},update:function(e){var t=this;z.update.call(this,e),t._argumentAxis=e.argumentAxis||{},t._axisHoverEnabled=t._argumentAxis&&(0,p.normalizeEnum)(t._argumentAxis.getOptions().hoverMode)===V,t._rotated=e.rotated,t._crosshair=e.crosshair,t._stickyHovering=e.stickyHovering},_getCanvas:function(e,t){for(var n=this,i=n._canvases||[],o=0;o<i.length;o++){var a=i[o];if((0,p.pointInCanvas)(a,e,t))return a}return null},_isPointerOut:function(e){return!e&&this._stuckSeries},_hideCrosshair:function(){var e;null===(e=this._crosshair)||void 0===e?void 0:e.hide()},_moveCrosshair:function(e,t,n){this._crosshair&&null!==e&&void 0!==e&&e.isVisible()&&this._crosshair.show({point:e,x:t,y:n})},_clean:function(){var e=this;z._clean.call(e),e._resetTimer(),e._stuckSeries=null},_getSeriesForShared:function(e,t){var n,i=this,o=[],a=null,r=1/0;return i._tooltip.isShared()&&!i.hoveredSeries&&((0,h.each)(i._storedSeries,function(n,i){var a=i.getNeighborPoint(e,t);a&&o.push(a)}),(0,h.each)(o,function(n,i){var o=i.getCrosshairData(e,t),s=(0,p.getDistance)(e,t,o.x,o.y);s<r&&(a=i,r=s)})),null===(n=a)||void 0===n?void 0:n.series},_setTimeout:function(e,t){var n=this;n._timeoutKeeper!==t&&(n._resetTimer(),n._hoverTimeout=setTimeout(function(){e(),n._timeoutKeeper=null},R),n._timeoutKeeper=t)},_resetTimer:function(){clearTimeout(this._hoverTimeout),this._timeoutKeeper=this._hoverTimeout=null},_stopEvent:function(e){(0,m.isDefined)(e.cancelable)&&!e.cancelable||(e.preventDefault(),e.stopPropagation())},_setStuckSeries:function(e,t,n,i){return"mouse"!==e.pointerType?this._stuckSeries=null:this._stuckSeries=t||this._stuckSeries||this._getSeriesForShared(n,i),!!this._stuckSeries},_pointerOut:function(){var e=this;e._stuckSeries=null,e._hideCrosshair(),e._resetTimer(),z._pointerOut.apply(e,arguments)},_hoverArgumentAxis:function(e,t,n){var i=this;if(i._resetHoveredArgument(),i._axisHoverEnabled&&i._argumentAxis.coordsIn(e,t))return i._hoverArgument(o(n,P,!0)),!0},_pointerComplete:function(e,t,n){var i=this;i.hoveredSeries&&i.hoveredSeries.updateHover(t,n),i._resetTimer(),i._moveCrosshair(e,t,n),z._pointerComplete.call(i,e)},_legendClick:function(e,t){var n=this._storedSeries[e.id];this._triggerLegendClick({target:n,event:t},T)},_hoverLegendItem:function(e,t){this._stuckSeries=null,this._hideCrosshair(),z._hoverLegendItem.call(this,e,t)},_pointerOnPoint:function(e,t,n,i){this._setStuckSeries(i,e.series,t,n),this._releaseHoveredSeries(),z._pointerOnPoint.call(this,e,t,n,i)},_notifyLegendOnHoverArgument:!1,_getArgumentHoverMode:function(){return s(this._argumentAxis)},dispose:function(){this._resetTimer(),z.dispose.call(this)}});var j=function(e){this.ctor(e)};t.PieTracker=j,(0,d.extend)(j.prototype,z,{_isPointerOut:function(e,t){return!t},_legendClick:function(e,t){var n=this,i=[];n._storedSeries.forEach(function(t){return i.push.apply(i,t.getPointsByKeys(e.argument,e.argumentIndex))}),n._eventTrigger(I,{target:e.argument,points:i,event:t})},_pointClick:function(e,t){this._eventTrigger(O,{target:e,event:t})},_hoverLegendItem:function(e,t){var n=this,i=n._legend.getItemByCoord(e,t);i&&n._hoveredArgument!==i.argument?(n._resetHoveredArgument(),n._hoverArgument(i.argument,i.argumentIndex)):i||n.clearHover()},_getArgumentHoverMode:function(){return s(this._legend)},_hoverArgumentAxis:v.noop,_setStuckSeries:v.noop,_getCanvas:v.noop,_notifyLegendOnHoverArgument:!0})},function(e,t,n){function i(e){var t={};return e.forEach(function(e){var n=e.pane;t[n]||(t[n]=[]),t[n].push(e)}),t}function o(e,t){if(e.isLogarithmic){var n,i=e.logarithmicBase,o=e.tickValues,a=[];e.minValue=t.transform(e.minValue,i),e.oldMinValue=t.transform(e.oldMinValue,i),e.maxValue=t.transform(e.maxValue,i),e.oldMaxValue=t.transform(e.oldMaxValue,i),e.tickInterval=x.round(e.tickInterval),e.tickInterval<1&&(e.tickInterval=1);var r=t.getInterval(i,e.tickInterval);for(n=t.transform(o[0],i);a.length<o.length;)a.push(n),n=t.addInterval(n,r);a.tickInterval=e.tickInterval,e.tickValues=a}}function a(e){return e.reduce(function(e,t){var n,i=t.getTicksValues(),a=i.majorTicksValues,r=t.getOptions(),s=t.getTranslator().getBusinessRange(),l=t.getVisibleArea(),u=t._tickInterval,c=r.synchronizedValue;if(a&&a.length>0&&(0,m.isNumeric)(a[0])&&"discrete"!==r.type&&!s.isEmpty()&&(!s.breaks||!s.breaks.length)&&"zoom"!==t.getViewport().action){t.applyMargins();var d=t.getTranslator().from(l[0]),h=t.getTranslator().from(l[1]),f=d<h?d:h,p=d<h?h:d;f===p&&(0,m.isDefined)(c)&&(u=k(a[0]-c)||1,f=a[0]-u,p=a[0]+u),n={axis:t,isLogarithmic:"logarithmic"===r.type,logarithmicBase:s.base,tickValues:a,minorValues:i.minorTicksValues,minorTickInterval:t._minorTickInterval,minValue:f,oldMinValue:f,maxValue:p,oldMaxValue:p,inverted:s.invert,tickInterval:u,synchronizedValue:c},o(n,S),e.push(n)}return e},[])}function r(e){var t=e.reduce(function(e,t){return C(e,t.tickValues.length)},0);e.forEach(function(e){var n,i,o=0,a=e.synchronizedValue,r=e.tickValues,s=e.tickInterval;if((0,m.isDefined)(a))e.baseTickValue=e.invertedBaseTickValue=a,
e.tickValues=[e.baseTickValue];else{if(r.length>1&&s){for(n=w((t+1)/r.length),i=n>1?w((t+1)/n):t,o=w((i-r.length)/2);o>0&&0!==r[0];)r.unshift((0,b.adjust)(r[0]-s)),o--;for(;r.length<i;)r.push((0,b.adjust)(r[r.length-1]+s));e.tickInterval=s/n}e.baseTickValue=r[0],e.invertedBaseTickValue=r[r.length-1]}})}function s(e){return e.maxValue-e.minValue||1}function l(e){for(var t=0;t<e.length;t++)if(!e[t].stubData)return e[t];return null}function u(e){var t=l(e),n=t.tickInterval;e.forEach(function(e){var i,o,a,r;e!==t&&(n&&e.tickInterval&&(e.stubData&&(0,m.isDefined)(e.synchronizedValue)&&(e.oldMinValue=e.minValue=e.baseTickValue-(t.baseTickValue-t.minValue)/n*e.tickInterval,e.oldMaxValue=e.maxValue=e.baseTickValue-(t.baseTickValue-t.maxValue)/n*e.tickInterval),i=n/s(t)/e.tickInterval*s(e),e.maxValue=e.minValue+s(e)/i),a=t.inverted&&!e.inverted||!t.inverted&&e.inverted?t.maxValue-t.invertedBaseTickValue:t.baseTickValue-t.minValue,r=s(e),o=(a/s(t)-(e.baseTickValue-e.minValue)/r)*r,e.minValue-=o,e.maxValue-=o)})}function c(e){var t,n,i=0,o=0;return e.forEach(function(e){var a=e.inverted;t=e.minValue>e.oldMinValue?(e.minValue-e.oldMinValue)/s(e):0,n=e.maxValue<e.oldMaxValue?(e.oldMaxValue-e.maxValue)/s(e):0,i=C(i,a?n:t),o=C(o,a?t:n)}),{start:i,end:o}}function d(e,t){e.forEach(function(e){var n=s(e),i=e.inverted;e.minValue=(0,b.adjust)(e.minValue-t[i?"end":"start"]*n),e.maxValue=(0,b.adjust)(e.maxValue+t[i?"start":"end"]*n)})}function h(e){var t=!1;e.forEach(function(e){t=t||(0,m.isDefined)(e.synchronizedValue)}),e.forEach(function(e){var n,i=e.tickInterval,o=e.tickValues,a=e.maxValue,r=e.minValue;if(t&&i){for(;(n=(0,b.adjust)(o[0]-i))>=r;)o.unshift(n);for(n=o[o.length-1];(n=(0,b.adjust)(n+i))<=a;)o.push(n)}for(;o[0]+i/10<r;)o.shift();for(;o[o.length-1]-i/10>a;)o.pop()})}function f(e){e.forEach(function(e){var t=e.axis,n=t.getTranslator().getBusinessRange();n.min===n.minVisible&&(n.min=e.minValue),n.max===n.maxVisible&&(n.max=e.maxValue),n.minVisible=e.minValue,n.maxVisible=e.maxValue,n.min>n.minVisible&&(n.min=n.minVisible),n.max<n.maxVisible&&(n.max=n.maxVisible),t.getTranslator().updateBusinessRange(n),t.setTicks({majorTicks:e.tickValues,minorTicks:e.minorValues})})}function p(e){var t,n=[];e.forEach(function(e){e.oldMaxValue-e.oldMinValue===0?n.push(e):(0,m.isDefined)(t)||(0,m.isDefined)(e.synchronizedValue)||(t=k((e.maxValue-e.minValue)/(e.tickValues[w(e.tickValues.length/2)]-e.minValue||e.maxValue)))}),(0,m.isDefined)(t)&&n.forEach(function(e){var n=e.tickValues[0],i=n*t;n>0?(e.maxValue=i,e.minValue=0):n<0&&(e.minValue=i,e.maxValue=0)})}function g(e){e.forEach(function(e){if(e.minorTickInterval){for(var t=[],n=e.minorTickInterval,i=e.tickInterval/n-1,o=1;o<e.tickValues.length;o++)for(var a=e.tickValues[o-1],r=0;r<i;r++)a+=n,t.push(a);e.minorValues=t}})}function _(e,t){return e.reduce(function(e,t){var n=t.inverted,i=t.axis.getCorrectedValuesToZero(t.minValue,t.maxValue),o=i.start,a=i.end;return(0,m.isDefined)(o)||(0,m.isDefined)(a)?n?{start:e.start,end:Math.min(e.end,a)}:{start:Math.min(e.start,o),end:e.end}:e},t)}t.default=void 0;var m=(n(106),n(1)),v=n(3),y=n(14),b=n(29),x=Math,w=x.floor,C=x.max,k=x.abs,S={transform:function(e,t){return(0,b.adjust)((0,y.getLog)(e,t))},addInterval:function(e,t){return(0,b.adjust)(e+t)},getInterval:function(e,t){return t}},D={transform:function(e,t){return(0,b.adjust)((0,y.raiseTo)(e,t))},addInterval:function(e,t){return(0,b.adjust)(e*t)},getInterval:function(e,t){return x.pow(e,t)}},I={synchronize:function(e){(0,v.each)(i(e),function(e,t){var n,i;if(t.length>1){if(n=a(t),n.length<2||!l(n))return;r(n),u(n),i=c(n),i=_(n,i),d(n,i),p(n),h(n),g(n),n.forEach(function(e){o(e,D)}),f(n)}})}},T=I;t.default=T,e.exports=t.default},function(e,t,n){function i(e){return e}t.default=void 0;var o=n(1),a=n(29),r=Math.round,s={translate:function(e,t){var n=this,i=n._canvasOptions,a=n._categoriesToPoints[e.valueOf()],s=n.translateSpecialCase(e),l=i.startPointIndex||0,u=n._options.stick?0:.5;if((0,o.isDefined)(s))return r(s);if(!a&&0!==a)return null;t=t||0;var c=a+u-l+.5*t;return r(n._calculateProjection(i.interval*c))},getInterval:function(){return this._canvasOptions.interval},getEventScale:function(e){var t=e.deltaScale||1;return 1-(1-t)/(.75+this.visibleCategories.length/this._categories.length)},zoom:function(e,t){var n,i=this,o=i._categories,r=i._canvasOptions,s=i._options.stick,l=r.invert,u=r.interval*t,c=e/u,d=(i.visibleCategories||[]).length,h=parseInt((r.startPointIndex||0)+c+.5),f=parseInt((0,a.adjust)(r.canvasLength/u)+(s?1:0))||1;l&&(h=parseInt((r.startPointIndex||0)+d-c+.5)-f),h<0&&(h=0),n=h+f,n>o.length&&(n=o.length,h=n-f,h<0&&(h=0));var p=o.slice(parseInt(h),parseInt(n)),g=i._getDiscreteInterval(p.length,r);return t=g/r.interval,e=i.translate(l?p[p.length-1]:p[0])*t-(r.startPoint+(s?0:g/2)),{min:p[0],max:p[p.length-1],translate:e,scale:t}},getMinScale:function(e){var t=this,n=t._canvasOptions,i=(t.visibleCategories||t._categories).length;return i+=(parseInt(.1*i)||1)*(e?-2:2),n.canvasLength/(Math.max(i,1)*n.interval)},getScale:function(e,t){var n=this,i=n._canvasOptions,a=n.getCanvasVisibleArea(),r=!n._options.stick&&1,s=(0,o.isDefined)(e)?n.translate(e,-r):null,l=(0,o.isDefined)(t)?n.translate(t,+r):null;return null===s&&(s=i.invert?a.max:a.min),null===l&&(l=i.invert?a.min:a.max),n.canvasLength/Math.abs(l-s)},isValid:function(e){return!!(0,o.isDefined)(e)&&this._categoriesToPoints[e.valueOf()]>=0},getCorrectValue:i,to:function(e,t){var n=this._canvasOptions,i=this._categoriesToPoints[e.valueOf()],o=n.startPointIndex||0,a=i+(this._options.stick?0:.5)-o+(this._businessRange.invert?-1:1)*t*.5;return r(this._calculateProjection(n.interval*a))},from:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=this._canvasOptions,i=n.startPoint,o=this.visibleCategories||this._categories,a=o.length,s=this._options.stick?.5:0,l=r((e-i)/n.interval+s-.5-.5*t);return l>=a&&(l=a-1),l<0&&(l=0),n.invert&&(l=a-l-1),o[l]},_add:function(){return NaN},_toValue:i,isValueProlonged:!0,getRangeByMinZoomValue:function(e,t){var n=this._categories,i=n.indexOf(t.minVisible),o=n.indexOf(t.maxVisible),a=i+e-1,r=o-e+1;return n[a]?[t.minVisible,n[a]]:[n[r],t.maxVisible]}};t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(1),a=i(n(20)),r=n(29),s=Math.floor,l={_intervalize:function(e,t){if((0,o.isDefined)(e))return"datetime"===this._businessRange.dataType?(e=(0,o.isNumeric)(e)?new Date(e):new Date(e.getTime()),e=a.default.correctDateWithUnitBeginning(e,t,null,this._options.firstDayOfWeek)):e=(0,r.adjust)(s((0,r.adjust)(e/t))*t,t),e},translate:function(e,t,n){var i=this,a=i.translateSpecialCase(e);return(0,o.isDefined)(a)?Math.round(a):(n=n||i._options.interval,i.isValid(e,n)?i.to(e,t,n):null)},getInterval:function(){return Math.round(this._canvasOptions.ratioOfCanvasRange*(this._businessRange.interval||Math.abs(this._canvasOptions.rangeMax-this._canvasOptions.rangeMin)))},zoom:function(){},getMinScale:function(){},getScale:function(){},_parse:function(e){return"datetime"===this._businessRange.dataType?new Date(e):Number(e)},_fromValue:function(e){return this._parse(e)},_toValue:function(e){return this._parse(e)},isValid:function(e,t){var n=this,i=n._canvasOptions,r=i.rangeMin,s=i.rangeMax;return t=t||n._options.interval,null!==e&&!isNaN(e)&&(e="datetime"===n._businessRange.dataType&&(0,o.isNumeric)(e)?new Date(e):e,t!==n._options.interval&&(r=n._intervalize(r,t),s=n._intervalize(s,t)),!(e.valueOf()<r||e.valueOf()>=a.default.addInterval(s,t)))},to:function(e,t,n){var i=this;n=n||i._options.interval;var o=i._intervalize(e,n),r=a.default.addInterval(o,n),l=i._to(o),u=i._to(r);return t?t>0&&(l=u):l=s((l+u)/2),l},_to:function(e){var t=this._canvasOptions,n=t.rangeMinVisible,i=t.rangeMaxVisible,o=e-n;return e<n?o=0:e>i&&(o=a.default.addInterval(i,this._options.interval)-n),this._conversionValue(this._calculateProjection(o*this._canvasOptions.ratioOfCanvasRange))},from:function(e,t){var n,i=this,o=i._options.interval,r=o,s=i._canvasOptions,l=s.rangeMinVisible,u=s.rangeMaxVisible;return"datetime"===i._businessRange.dataType&&(r=a.default.dateToMilliseconds(o)),n=i._calculateUnProjection((e-i._canvasOptions.startPoint)/i._canvasOptions.ratioOfCanvasRange),n=i._intervalize(a.default.addInterval(n,r/2,t>0),o),n<l?n=l:n>u&&(n=u),n},_add:function(){return NaN},isValueProlonged:!0};t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return null!==e?new Date(e):e}t.default=void 0;var a=i(n(20)),r={_fromValue:o,_toValue:o,_add:a.default.addDateInterval,convert:a.default.dateToMilliseconds};t.default=r,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(14),o=n(1),a={_fromValue:function(e){return null!==e?(0,i.getLogExt)(e,this._canvasOptions.base,this._businessRange.allowNegatives,this._businessRange.linearThreshold):e},_toValue:function(e){return null!==e?(0,i.raiseToExt)(e,this._canvasOptions.base,this._businessRange.allowNegatives,this._businessRange.linearThreshold):e},getMinBarSize:function(e){var t=this.getCanvasVisibleArea(),n=this.from(t.min+e),i=this._canvasOptions;return Math.pow(i.base,i.rangeMinVisible+this._fromValue(this.from(t.min))-this._fromValue((0,o.isDefined)(n)?n:this.from(t.max)))},checkMinBarSize:function(e,t,n){var i,a,r=this._canvasOptions,s=n-e,l=this.constructor.prototype.checkMinBarSize;return(0,o.isDefined)(t)&&s>0?(i=l(this._fromValue(n/s),this._fromValue(t)-r.rangeMinVisible),a=Math.pow(r.base,this._fromValue(s)+i)-s):a=l(e,t),a}};t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return _.filter(function(t){return!e.some(function(e){return e===t})})}function a(e){return(e+1)%7}function r(e,t){for(var n=e.start,i=e.end;n!==i;){if(n===t)return!0;n=a(n)}return!1}function s(e,t){for(var n=0;e!==t;)e=a(e),n++;return n}function l(e,t){var n=[],i=new Date(t);return i.setDate(t.getDate()+1),t>e.from&&n.push({from:e.from,to:t}),i<e.to&&n.push({from:i,to:e.to}),n}function u(e){var t=o(e);if(t.length<7)for(;a(t[t.length-1])===t[0];)t.unshift(t.pop());return t}function c(e,t,n){var i=e.getDate(),o=[],l=n.reduce(function(e,t){var n=e[1];return void 0===n.start?(n={start:t,end:a(t)},e[0].push(n),[e[0],n]):n.end===t?(n.end=a(t),e):(n={start:t,end:a(t)},e[0].push(n),[e[0],n])},[[],{}]);return l[0].forEach(function(n){var a=new Date(e);for(a=g.default.trimTime(a);a<t;){i=a.getDay();var l=a.getDate();if(r(n,i)){var u=new Date(a);a.setDate(l+s(i,n.end));var c=new Date(a);o.push({from:u,to:c})}a.setDate(a.getDate()+1)}}),o}function d(e,t){var n,i=e.slice(),o=function(e){if(e=g.default.trimTime(new Date(e)),i[n].from<=e&&i[n].to>e){var t=l(i[n],e);2===t.length?i.splice(n,1,t[0],t[1]):1===t.length?i.splice(n,1,t[0]):i.splice(n,1)}};for(n=0;n<i.length;n++)t.forEach(o);return i}function h(e,t,n,i){var o,a=function(e){return e===o},r=g.default.trimTime(e),s=g.default.trimTime(t);return s.setDate(t.getDate()+1),n.reduce(function(e,t){var n,l;return t=new Date(t),o=t.getDay(),!i.some(a)&&t>=r&&t<=s&&(n=g.default.trimTime(t),l=new Date(n),l.setDate(n.getDate()+1),e.push({from:n,to:l})),e},[])}function f(e){return e.map(function(e){return{from:e.from,to:e.to,gapSize:g.default.convertMillisecondsToDateUnits(e.to-e.from)}})}function p(e,t,n,i,o){var a=u(n),r=c(e,t,a);return r.push.apply(r,h(e,t,o||[],a)),f(d(r,i||[]))}t.generateDateBreaks=p;var g=i(n(20)),_=[0,1,2,3,4,5,6]},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t;return e=(0,a.normalizeAngle)(e),e>=315&&e<=360||e<45&&e>=0?t=1:e>=45&&e<135?t=2:e>=135&&e<225?t=3:e>=225&&e<315&&(t=4),t}t.linearSpider=t.linear=t.circularSpider=t.circular=void 0;var a=n(14),r=n(1),s=n(0),l=i(n(328)),u=i(n(554)),c=n(553),d=n(232),h=n(4),f=Math.PI,p=Math.abs,g=Math.atan,_=Math.round,m=Math.min,v=Math.max,y=u.default.linear,b=90,x={_calculateValueMargins:function(e){var t=this._getViewportRange(),n=t.minVisible,i=t.maxVisible;return e&&e.length>1&&(n=n<e[0].value?n:e[0].value,i=n>e[e.length-1].value?i:e[e.length-1].value),{minValue:n,maxValue:i}},applyMargins:function(){var e=this._calculateValueMargins(this._majorTicks),t=this._translator.getBusinessRange();t.addRange({minVisible:e.minValue,maxVisible:e.maxValue,interval:this._calculateRangeInterval(t.interval)}),this._translator.updateBusinessRange(t)},_getTranslatorOptions:function(){return{isHorizontal:!0,conversionValue:!0,addSpiderCategory:this._getSpiderCategoryOption(),stick:this._getStick()}},getCenter:function(){return this._center},getRadius:function(){return this._radius},getAngles:function(){var e=this._options;return[e.startAngle,e.endAngle]},_updateRadius:function(e){var t=m(e.width-e.left-e.right,e.height-e.top-e.bottom)/2;this._radius=t<0?0:t},_updateCenter:function(e){this._center={x:e.left+(e.width-e.right-e.left)/2,y:e.top+(e.height-e.top-e.bottom)/2}},_processCanvas:function(e){return this._updateRadius(e),this._updateCenter(e),{left:0,right:0,width:this._getScreenDelta()}},_createAxisElement:function(){return this._renderer.circle()},_updateAxisElementPosition:function(){var e=this.getCenter();this._axisElement.attr({cx:e.x,cy:e.y,r:this.getRadius()})},_boundaryTicksVisibility:{min:!0},_getSpiderCategoryOption:function(){return this._options.firstPointOnStartAngle},_validateOptions:function(e){var t=this,n=e.originValue,i=e.wholeRange={},o=e.period;(0,r.isDefined)(n)&&(n=t.validateUnit(n)),o>0&&e.argumentType===l.default.numeric&&(n=n||0,i.endValue=n+o,t._viewport=(0,a.getVizRangeObject)([n,i.endValue])),(0,r.isDefined)(n)&&(i.startValue=n)},getMargins:function(){var e=this._options.tick,t=v(e.visible?e.length/2+e.shift:0,0),n=this.getRadius(),i=this._center,o=i.x,a=i.y,r=this._majorTicks.map(function(e){return e.label&&e.label.getBBox()}).filter(function(e){return e}),l=(0,s.extend)({},this._canvas,{left:o-n,top:a-n,right:this._canvas.width-(o+n),bottom:this._canvas.height-(a+n)}),u=(0,d.calculateCanvasMargins)(r,l);return Object.keys(u).forEach(function(e){return u[e]=u[e]<t?t:u[e]}),u},updateSize:function(){var e=this;d.Axis.prototype.updateSize.apply(e,arguments),(0,d.measureLabels)(e._majorTicks),e._adjustLabelsCoord(0,0,!0),this._checkBoundedLabelsOverlapping(this._majorTicks,this._majorTicks.map(function(e){return e.labelBBox}))},_setVisualRange:h.noop,applyVisualRangeSetter:h.noop,_getStick:function(){return this._options.firstPointOnStartAngle||this._options.type!==l.default.discrete},_getTranslatedCoord:function(e,t){return this._translator.translate(e,t)-b},_getCanvasStartEnd:function(){return{start:0-b,end:360-b}},_getStripGraphicAttributes:function(e,t){var n=this.getCenter(),i=this.getAngles()[0],o=this.getRadius();return{x:n.x,y:n.y,innerRadius:0,outerRadius:o,startAngle:-t-i,endAngle:-e-i}},_createStrip:function(e){return this._renderer.arc(e.x,e.y,e.innerRadius,e.outerRadius,e.startAngle,e.endAngle)},_getStripLabelCoords:function(e,t){var n=this,i=n._getStripGraphicAttributes(e,t),o=i.startAngle+(i.endAngle-i.startAngle)/2,r=(0,a.getCosAndSin)(o),s=n.getRadius()/2,u=n.getCenter(),c=_(u.x+s*r.cos),d=_(u.y-s*r.sin);return{x:c,y:d,align:l.default.center}},_getConstantLineGraphicAttributes:function(e){var t=this.getCenter(),n=this.getRadius();return{points:[t.x,t.y,t.x+n,t.y]}},_createConstantLine:function(e,t){return this._createPathElement(this._getConstantLineGraphicAttributes(e).points,t)},_rotateConstantLine:function(e,t){var n=this.getCenter(),i=n.x,o=n.y;e.rotate(t+this.getAngles()[0],i,o)},_getConstantLineLabelsCoords:function(e){var t=this,n=(0,a.getCosAndSin)(-e-t.getAngles()[0]),i=t.getRadius()/2,o=t.getCenter(),r=_(o.x+i*n.cos),s=_(o.y-i*n.sin);return{x:r,y:s}},_checkAlignmentConstantLineLabels:h.noop,_adjustDivisionFactor:function(e){return 180*e/(this.getRadius()*f)},_getScreenDelta:function(){var e=this.getAngles();return p(e[0]-e[1])},_getTickMarkPoints:function(e,t,n){var i=n.shift,o=void 0===i?0:i,a=this.getCenter(),r={inside:-1,center:-.5,outside:0},s=this.getRadius()+t*r[this._options.tickOrientation||"center"];return[a.x+s+o,a.y,a.x+s+t+o,a.y]},_getLabelAdjustedCoord:function(e,t,n,i){var r=this,s=e.labelCoords,l=s.y,u=s.angle,c=(0,a.getCosAndSin)(u),d=c.cos,h=c.sin,f=e.labelBBox,p=f.width/2,g=f.height/2,_=r._options.label.indentFromAxis||0,m=s.x+_*d,v=l+(l-f.y-g)+_*h,y=0,b=0;switch(o(u)){case 1:y=p,b=g*h;break;case 2:y=p*d,b=g;break;case 3:y=-p,b=g*h;break;case 4:y=p*d,b=-g}if(i){var x=r._canvas,w=m-s.x+y,C=v-s.y+b;f.x+w<x.originalLeft&&(y-=f.x+w-x.originalLeft),f.x+f.width+w>x.width-x.originalRight&&(y-=f.x+f.width+w-(x.width-x.originalRight)),f.y+C<x.originalTop&&(b-=f.y+C-x.originalTop),f.y+f.height+C>x.height-x.originalBottom&&(b-=f.y+f.height+C-(x.height-x.originalBottom))}return{x:m+y,y:v+b}},_getGridLineDrawer:function(){var e=this;return function(t,n){var i=e.getCenter();return e._createPathElement(e._getGridPoints().points,n).rotate(t.coords.angle,i.x,i.y)}},_getGridPoints:function(){var e=this.getRadius(),t=this.getCenter();return{points:[t.x,t.y,t.x+e,t.y]}},_getTranslatedValue:function(e,t){var n=this.getAngles()[0],i=this._translator.translate(e,-t),o=(0,a.convertPolarToXY)(this.getCenter(),n,i,this.getRadius());return{x:o.x,y:o.y,angle:this.getTranslatedAngle(i)}},_getAdjustedStripLabelCoords:function(e){var t=e.labelBBox;return{translateY:e.label.attr("y")-t.y-t.height/2}},coordsIn:function(e,t){return(0,a.convertXYToPolar)(this.getCenter(),e,t).r>this.getRadius()},_rotateTick:function(e,t){var n=this.getCenter();e.rotate(t.angle,n.x,n.y)},_validateOverlappingMode:function(e){return l.default.validateOverlappingMode(e)},_validateDisplayMode:function(){return"standard"},_getStep:function(e){var t=this,n=t.getRadius()+(t._options.label.indentFromAxis||0),i=e.reduce(function(e,t){var n=e;return e.width<t.width&&(n.width=t.width),e.height<t.height&&(n.height=t.height),n},{width:0,height:0}),o=p(2*g(i.height/(2*n-i.width))*180/f),a=p(2*g(i.width/(2*n-i.height))*180/f);return l.default.getTicksCountInRange(t._majorTicks,"angle",v(o,a))},_checkBoundedLabelsOverlapping:function(e,t,n){var i=this._options.label;if(n=n||this._validateOverlappingMode(i.overlappingBehavior),"hide"===n){var o=e.reduce(function(e,t,n){return t.label?n:e},null);o&&l.default.areLabelsOverlap(t[0],t[o],i.minSpacing,l.default.center)&&("first"===i.hideFirstOrLast?e[0].label.remove():e[o].label.remove())}},shift:function(e){this._axisGroup.attr({translateX:e.right,translateY:e.bottom})},getTranslatedAngle:function(e){var t=this.getAngles()[0];return e+t-b}},w=x;t.circular=w;var C=(0,s.extend)({},x,{_createAxisElement:function(){return this._renderer.path([],"area")},_updateAxisElementPosition:function(){this._axisElement.attr({points:(0,a.map)(this.getSpiderTicks(),function(e){return{x:e.coords.x,y:e.coords.y}})})},_getStick:function(){return!0},_getSpiderCategoryOption:function(){return!0},getSpiderTicks:function(){var e=this,t=e.getFullTicks();return e._spiderTicks=t.map((0,c.tick)(e,e.renderer,{},{},e._getSkippedCategory(t),!0)),e._spiderTicks.forEach(function(e){e.initCoords()}),e._spiderTicks},_getStripGraphicAttributes:function(e,t){for(var n,i,o,a,r=this.getCenter(),s=this.getSpiderTicks(),l=[],u=0,c=s.length;u<c;)a=s[u].coords,a.angle>=e&&a.angle<=t&&(n||(n=(s[u-1]||s[s.length-1]).coords,l.push((a.x+n.x)/2,(a.y+n.y)/2)),l.push(a.x,a.y),o=(s[u+1]||s[0]).coords,i={x:(a.x+o.x)/2,y:(a.y+o.y)/2}),u++;return l.push(i.x,i.y),l.push(r.x,r.y),{points:l}},_createStrip:function(e){var t=e.points;return this._renderer.path(t,"area")},_getTranslatedCoord:function(e,t){return this._translator.translate(e,t)-b},_setTickOffset:function(){this._tickOffset=!1}});t.circularSpider=C;var k={_resetMargins:function(){this._reinitTranslator(this._getViewportRange())},_getStick:y._getStick,_getSpiderCategoryOption:h.noop,_getTranslatorOptions:function(){return{isHorizontal:!0,stick:this._getStick()}},getRadius:x.getRadius,getCenter:x.getCenter,getAngles:x.getAngles,_updateRadius:x._updateRadius,_updateCenter:x._updateCenter,_processCanvas:function(e){return this._updateRadius(e),this._updateCenter(e),{left:0,right:0,startPadding:e.startPadding,endPadding:e.endPadding,width:this.getRadius()}},_createAxisElement:y._createAxisElement,_updateAxisElementPosition:function(){var e=this.getCenter();this._axisElement.attr({points:[e.x,e.y,e.x+this.getRadius(),e.y]}).rotate(this.getAngles()[0]-b,e.x,e.y)},_getScreenDelta:function(){return this.getRadius()},_getTickMarkPoints:function(e,t){return[e.x-t/2,e.y,e.x+t/2,e.y]},_getLabelAdjustedCoord:function(e){var t=this,n=e.labelCoords,i=n.y,o=(0,a.getCosAndSin)(n.angle),r=t._options.label.indentFromAxis||0,s=e.labelBBox,l=n.x-p(r*o.sin)+p(s.width/2*o.cos)-s.width/2,u=i+(i-s.y)-p(s.height/2*o.sin)+p(r*o.cos);return{x:l,y:u}},_getGridLineDrawer:function(){var e=this;return function(t,n){var i=e._getGridPoints(t.coords);return e._renderer.circle(i.cx,i.cy,i.r).attr(n).sharp()}},_getGridPoints:function(e){var t=this.getCenter(),n=(0,a.getDistance)(t.x,t.y,e.x,e.y);return n>this.getRadius()?{cx:null,cy:null,r:null}:{cx:t.x,cy:t.y,r:n}},_getTranslatedValue:function(e,t){var n=this.getAngles()[0],i=(0,a.convertPolarToXY)(this.getCenter(),n,0,this._translator.translate(e,t));return{x:i.x,y:i.y,angle:n-b}},_getTranslatedCoord:function(e,t){return this._translator.translate(e,t)},_getCanvasStartEnd:function(){var e=this.getTranslator().getBusinessRange().invert,t=[0,this.getRadius()];return e&&t.reverse(),{start:t[0],end:t[1]}},_getStripGraphicAttributes:function(e,t){var n=this.getCenter();return{x:n.x,y:n.y,innerRadius:e,outerRadius:t}},_createStrip:function(e){return this._renderer.arc(e.x,e.y,e.innerRadius,e.outerRadius,0,360)},_getAdjustedStripLabelCoords:x._getAdjustedStripLabelCoords,_getStripLabelCoords:function(e,t){var n=this,i=e+(t-e)/2,o=n.getCenter(),a=_(o.y-i);return{x:o.x,y:a,align:l.default.center}},_getConstantLineGraphicAttributes:function(e){var t=this.getCenter();return{cx:t.x,cy:t.y,r:e}},_createConstantLine:function(e,t){var n=this._getConstantLineGraphicAttributes(e);return this._renderer.circle(n.cx,n.cy,n.r).attr(t).sharp()},_getConstantLineLabelsCoords:function(e){var t=this,n=t.getCenter(),i=_(n.y-e);return{x:n.x,y:i}},_checkAlignmentConstantLineLabels:h.noop,_rotateTick:function(e,t,n){!n&&e.rotate(t.angle+b,t.x,t.y)},_validateOverlappingMode:x._validateOverlappingMode,_validateDisplayMode:x._validateDisplayMode,_getStep:function(e){var t=o(this.getAngles()[0]),n=this._options.label.minSpacing,i=2===t||4===t?function(e){return e.width+n}:function(e){return e.height},a=e.reduce(function(e,t){return v(e,i(t))},0);return l.default.getTicksCountInRange(this._majorTicks,2===t||4===t?"x":"y",a)}};t.linear=k;var S=(0,s.extend)({},k,{_createPathElement:function(e,t){return this._renderer.path(e,"area").attr(t).sharp()},setSpiderTicks:function(e){this._spiderTicks=e},_getGridLineDrawer:function(){var e=this;return function(t,n){return e._createPathElement(e._getGridPoints(t.coords).points,n)}},_getGridPoints:function(e){var t=this.getCenter(),n=(0,a.getDistance)(t.x,t.y,e.x,e.y);return this._getGridPointsByRadius(n)},_getGridPointsByRadius:function(e){var t=this.getCenter();return e>this.getRadius()?{points:null}:{points:(0,a.map)(this._spiderTicks,function(n){var i=(0,a.getCosAndSin)(n.coords.angle);return{x:_(t.x+e*i.cos),y:_(t.y+e*i.sin)}})}},_getStripGraphicAttributes:function(e,t){var n=this._getGridPointsByRadius(t).points,i=this._getGridPointsByRadius(e).points;return{points:[i,n.reverse()]}},_createStrip:C._createStrip,_getConstantLineGraphicAttributes:function(e){return this._getGridPointsByRadius(e)},_createConstantLine:function(e,t){return this._createPathElement(this._getConstantLineGraphicAttributes(e).points,t)}});t.linearSpider=S},function(e,t,n){function i(e,t){var n,i,a=t.label||{},r=a.position||"inside",s=!1;e._checkAlignmentConstantLineLabels(a);var l;return{options:t,labelOptions:a,labelPosition:r,label:null,line:null,getParsedValue:function(){return s?n:(n=e.validateUnit(t.value,"E2105","constantLine"),s=!0,n)},draw:function(){if(!(0,o.isDefined)(t.value)||e._translator.getBusinessRange().isEmpty())return this;var n=e._getCanvasStartEnd(),i=this.getParsedValue();this.coord=e._getConstantLinePos(i,n.start,n.end);var s=t.displayBehindSeries?e._axisConstantLineGroups.under:e._axisConstantLineGroups.above,l=s[r];if(!l){var u=e._isHorizontal?a.verticalAlignment:a.horizontalAlignment;l=s[u]}if(!(0,o.isDefined)(this.coord))return this;var c=e._createConstantLine(this.coord,{stroke:t.color,"stroke-width":t.width,dashStyle:t.dashStyle});return this.line=c.append(s.inside),this.label=a.visible?e._drawConstantLineLabels(i,a,this.coord,l):null,this.updatePosition(),this},getContentContainer:function(){return this.label},removeLabel:function(){this.label&&this.label.remove()},updatePosition:function(t){var n=e._getCanvasStartEnd(),i=e._getConstantLinePos(this.getParsedValue(),n.start,n.end);(0,o.isDefined)(i)&&(this.coord=i,t&&l?(this.label&&this.label.attr(e._getConstantLineLabelsCoords(l,this.labelOptions)),this.line&&this.line.attr(e._getConstantLineGraphicAttributes(l)),this.label&&this.label.animate(e._getConstantLineLabelsCoords(this.coord,this.labelOptions)),this.line&&this.line.animate(e._getConstantLineGraphicAttributes(this.coord))):(this.label&&this.label.attr(e._getConstantLineLabelsCoords(this.coord,this.labelOptions)),this.line&&this.line.attr(e._getConstantLineGraphicAttributes(this.coord)),e._rotateConstantLine(this.line,this.coord)))},saveCoords:function(){i=l,l=this.coord},resetCoordinates:function(){l=i}}}t.default=i;var o=n(1);e.exports=t.default},function(e,t,n){function i(e,t){var n,i,s=t.label||{};return{options:t,label:null,rect:null,_getCoord:function(){var n=e._getCanvasStartEnd(),i=e._translator.getBusinessRange();return e._getStripPos(t.startValue,t.endValue,n.start,n.end,i)},_drawLabel:function(t){return e._renderer.text(s.text,t.x,t.y).css((0,a.patchFontOptions)((0,r.extend)({},e.getOptions().label.font,s.font))).attr({align:"center","class":s.cssClass}).append(e._axisStripLabelGroup)},draw:function(){if(!e._translator.getBusinessRange().isEmpty()&&((0,o.isDefined)(t.startValue)||(0,o.isDefined)(t.endValue))&&(0,o.isDefined)(t.color)){var n=this._getCoord();if(this.labelCoords=s.text?e._getStripLabelCoords(n.from,n.to,s):null,n.outOfCanvas||!(0,o.isDefined)(n.to)||!(0,o.isDefined)(n.from))return;this.rect=e._createStrip(e._getStripGraphicAttributes(n.from,n.to)).attr({fill:t.color}).append(e._axisStripGroup),this.label=s.text?this._drawLabel(this.labelCoords):null}},getContentContainer:function(){return this.label},removeLabel:function(){},updatePosition:function(i){var o=this._getCoord();i&&n?(this.label&&this.label.attr(e._getStripLabelCoords(n.from,n.to,t.label)),this.rect&&this.rect.attr(e._getStripGraphicAttributes(n.from,n.to)),this.label&&this.label.animate(e._getStripLabelCoords(o.from,o.to,t.label)),this.rect&&this.rect.animate(e._getStripGraphicAttributes(o.from,o.to))):(this.label&&this.label.attr(e._getStripLabelCoords(o.from,o.to,t.label)),this.rect&&this.rect.attr(e._getStripGraphicAttributes(o.from,o.to)))},saveCoords:function(){i=n,n=this._getCoord()},resetCoordinates:function(){n=i}}}t.default=i;var o=n(1),a=n(14),r=n(0);e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){var o=0;return"right"===t?o=e.width-e.right+n:"left"===t&&(o=e.left-n-i),o}function a(e,t,n,i){var o=0;return"top"===t?o=e.top-n:"bottom"===t&&(o=e.height-e.bottom+i+n),o}t.ScrollBar=void 0;var r=i(n(5)),s=n(8),l=n(0),u=n(329),c=n(1),d=n(4),h=n(80),f=Math.min,p=Math.max,g=2,_=function(e,t){this._translator=new u.Translator2D({},{},{}),this._scroll=e.rect().append(t),this._addEvents()};t.ScrollBar=_,_.prototype={_addEvents:function(){var e=this,t=this._scroll.element;r.default.on(t,h.start,function(e){(0,s.fireEvent)({type:"dxc-scroll-start",originalEvent:e,target:t})}),r.default.on(t,h.move,function(n){var i=-n.offset.x*e._scale,o=-n.offset.y*e._scale,a=e._offset-(e._layoutOptions.vertical?o:i)/e._scale;e._applyPosition(a,a+e._translator.canvasLength/e._scale),(0,s.fireEvent)({type:"dxc-scroll-move",originalEvent:n,target:t,offset:{x:i,y:o}})}),r.default.on(t,h.end,function(n){(0,s.fireEvent)({type:"dxc-scroll-end",originalEvent:n,target:t,offset:{x:-n.offset.x*e._scale,y:-n.offset.y*e._scale}})})},update:function(e){var t=this,n=e.position,i=e.rotated,o=i?"right":"top",a=i?"left":"bottom";return n!==o&&n!==a&&(n=o),t._scroll.attr({rotate:e.rotated?0:-90,rotateX:0,rotateY:0,fill:e.color,width:e.width,opacity:e.opacity}),t._layoutOptions={width:e.width,offset:e.offset,vertical:i,position:n},t},init:function(e,t){var n=this,i="discrete"===e.axisType;return n._translateWithOffset=i&&!t&&1||0,n._translator.update((0,l.extend)({},e,{minVisible:null,maxVisible:null,visibleCategories:null},i&&{min:null,max:null}||{}),n._canvas,{isHorizontal:!n._layoutOptions.vertical,stick:t}),n},getOptions:function(){return this._layoutOptions},setPane:function(e){var t,n=this._layoutOptions.position;return t="left"===n||"top"===n?e[0]:e[e.length-1],this.pane=t.name,this},updateSize:function(e){this._canvas=(0,l.extend)({},e);var t=this._layoutOptions,n=t.position,i=t.offset,r=t.width;this._scroll.attr({translateX:o(e,n,i,r),translateY:a(e,n,i,r)})},getMultipleAxesSpacing:function(){return 0},estimateMargins:function(){return this.getMargins()},getMargins:function(){var e=this._layoutOptions,t={left:0,top:0,right:0,bottom:0};return t[e.position]=e.width+e.offset,t},shift:function e(t){var n,i,o=this,a=o._layoutOptions,r=a.position,s=a.vertical,l={translateX:null!==(n=o._scroll.attr("translateX"))&&void 0!==n?n:0,translateY:null!==(i=o._scroll.attr("translateY"))&&void 0!==i?i:0},e=t[r];l[s?"translateX":"translateY"]+=("left"===r||"top"===r?-1:1)*e,o._scroll.attr(l)},hideTitle:d.noop,hideOuterElements:d.noop,setPosition:function(e,t){var n=this,i=n._translator,o=(0,c.isDefined)(e)?i.translate(e,-n._translateWithOffset):i.translate("canvas_position_start"),a=(0,c.isDefined)(t)?i.translate(t,n._translateWithOffset):i.translate("canvas_position_end");n._offset=f(o,a),n._scale=i.getScale(e,t),n._applyPosition(f(o,a),p(o,a))},customPositionIsAvailable:function(){return!1},dispose:function(){this._scroll.dispose(),this._scroll=this._translator=null},_applyPosition:function(e,t){var n=this,i=n._translator.getCanvasVisibleArea();e=p(e,i.min),e=f(e,i.max),t=f(t,i.max),t=p(t,i.min);var o=Math.abs(t-e);n._scroll.attr({y:e,height:o<g?g:o})}}},function(e,t,n){function i(e,t,n,i){var o=Math.floor(n?i.pageY-e.top:i.pageX-e.left),a=n?t.y1:t.x1,r=n?t.y2:t.x2;return o<a?o=a:o>r&&(o=r),o}function o(e,t,n){var i=n.pageX-e.left,o=n.pageY-e.top;return i>=t.x1&&i<=t.x2&&o>=t.y1&&o<=t.y2}function a(e){return function(t){var n=e.getRootOffset(),a=e.getCanvas();return o(n,a,t)?(e.rootOffset=n,e.canvas=a,e.startCoord=i(n,a,e.rotated,t),e.triggerStart(),void e.rect.attr({x:a.x1,y:a.y1,width:a.width,height:a.height}).append(e.root)):void(t.cancel=!0)}}function r(e){return function(t){var n=i(e.rootOffset,e.canvas,e.rotated,t),o={};e.curCoord=n,o[e.rotated?"y":"x"]=Math.min(e.startCoord,n),o[e.rotated?"height":"width"]=Math.abs(e.startCoord-n),e.rect.attr(o)}}function s(e){return function(t){e.triggerEnd(),e.rect.remove()}}function l(e){var t=e.chart,n=e.renderer,i=e.rotated,o=n.rect(0,0,0,0).attr(e.shutterOptions),l={rect:o,root:n.root,rotated:i,triggerStart:function(){t._eventTrigger("zoomStart")},triggerEnd:function(){var e=t._argumentAxes[0].getTranslator(),n=Math.min(this.startCoord,this.curCoord),i=Math.max(this.startCoord,this.curCoord);t._eventTrigger("zoomEnd",{rangeStart:e.from(n),rangeEnd:e.from(i)})},dispose:function(){n.root.off(c),o.dispose()},getRootOffset:function(){return n.getRootOffset()},getCanvas:function(){var e=t._canvas,n=t.panes,i=n[0].canvas,o=n[n.length-1].canvas;return{x1:i.left,y1:i.top,x2:e.width-o.right,y2:e.height-o.bottom,width:e.width-i.left-o.right,
height:e.height-i.top-o.bottom}}};return n.root.off(c).on(d,{direction:i?"vertical":"horizontal",immediate:!0},a(l)).on(h,r(l)).on(f,s(l)),l}t.default=void 0;var u=n(80),c=".shutter-zoom",d=u.start+c,h=u.move+c,f=u.end+c,p={name:"shutter_zoom",init:function(){var e=this.option("shutterZoom")||{};e.enabled&&(this._shutterZoom=l({chart:this,renderer:this._renderer,rotated:this.option("rotated"),shutterOptions:e}))},dispose:function(){this._shutterZoom&&this._shutterZoom.dispose()}};t.default=p,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return{x:e.left,y:e.top,width:e.width-e.left-e.right,height:e.height-e.top-e.bottom}}function s(e,t){var n=t.x,i=t.y;return n>=e.x&&n<=e.width+e.x&&i>=e.y&&i<=e.height+e.y}function l(e,t){return t&&(e=e.sort(function(e,n){return e===t?-1:n===t?1:0})),e}function u(e){return!e.getTranslator().getBusinessRange().isEmpty()}function c(e,t,n,i,o,a,r){var s=t&&e!==t,l=n(e),u=e.checkZoomingLowerLimitOvercome(o,a,l),c=u.stopInteraction,d=u.correctedRange,h=e.handleZooming(c?null:d,i(s),r,o);return c&&e.handleZoomEnd(),{stopInteraction:c,result:h}}t.default=void 0;var d=n(1),h=n(0),f=n(14),p=n(139),g=a(n(275)),_=n(80),m=".zoomAndPanNS",v=_.start+m,y=_.move+m,b=_.end+m,x=g.pinchstart+m,w=g.pinch+m,C=g.pinchend+m,k="dxc-scroll-start"+m,S="dxc-scroll-move"+m,D="dxc-scroll-end"+m,I=300,T=5,O=Math.min,E=Math.max,A=Math.abs,P={name:"zoom_and_pan",init:function(){function e(t){t.originalEvent&&e(t.originalEvent),t.cancelable!==!1&&(t.cancel=!0)}function t(t,n,i){var o=t.options,a=t.actionData,r=[];o.argumentAxis[n]&&r.push(M.getArgumentAxis()),o.valueAxis[n]&&(r=r.concat(a.valueAxes)),r.reduce(function(e,t){return e?e:u(t)?t.handleZooming(null,{end:!0},i,n).isPrevented:e},!1)&&e(i)}function n(e,t,n,i,o){function a(e,n,o,a,r){var s={zoomed:!1};return n&&e.filter(u).forEach(function(e){var n=e.getOptions(),l=e.visualRange(),u=e.getTranslator().getEventScale(a),c=-i(a,r,o,u);s=(0,h.extend)(!0,s,e.getTranslator().zoom(c,u,e.getZoomBounds()));var p=e.adjustRange((0,f.getVizRangeObject)([s.min,s.max])),g=e.checkZoomingLowerLimitOvercome(t,u,p),_=g.stopInteraction,m=g.correctedRange;if((0,d.isDefined)(l)&&l.startValue.valueOf()===m.startValue.valueOf()&&l.endValue.valueOf()===m.endValue.valueOf()){if("touch"===a.pointerType&&"discrete"===n.type){var v=e.isExtremePosition(!1),y=e.isExtremePosition(!0),b=u>1&&!_,x=u<1&&(!v||!y),w=1===u&&!(v&&(c<0&&!n.inverted||c>0&&n.inverted)||y&&(c>0&&!n.inverted||c<0&&n.inverted));s.enabled=b||x||w}}else e.handleZooming(_?null:m,{start:!0,end:!0},a,t),_||(s.zoomed=!0,s.deltaTranslate=c-s.translate)}),s}function r(e,t,n,i){n.zoomed&&(t.offset[i]=(e.offset?e.offset[i]:t.offset[i])+n.deltaTranslate)}function s(e,t,n,i){n.zoomed&&(t.center[i]=e[i]+n.deltaTranslate)}var l=M.option("rotated"),c=e.actionData,p=e.options,g={},_={};c.fallback||(g=a(M._argumentAxes,p.argumentAxis[t],l?"y":"x",n,c),_=a(c.valueAxes,p.valueAxis[t],l?"x":"y",n,c),M._requestChange(["VISUAL_RANGE"]),r(n,c,g,l?"y":"x"),r(n,c,_,l?"x":"y"));var m=o(n);return s(m,c,g,l?"y":"x"),s(m,c,_,l?"x":"y"),g.zoomed||_.zoomed||(c.center=m),g.zoomed||_.zoomed||c.fallback||g.enabled||_.enabled}function i(e,t,n,i){function o(e,o,a,r,s){var l=!1,u=n.scale||1,d=function(e){var t=e.getTranslator().zoom(-i(n,r,a,u),u,e.getZoomBounds());return{startValue:t.min,endValue:t.max}},h=function(e){return{start:!0,end:e}};return o&&e.forEach(function(e){l=!c(e,s,d,h,t,u,n).stopInteraction}),l}var a=M.option("rotated"),r=e.actionData,s=e.options,l=!0;if(r.fallback)l&=o(M._argumentAxes,s.argumentAxis[t],a?"y":"x",r,M.getArgumentAxis()),l|=o(r.valueAxes,s.valueAxis[t],a?"x":"y",r);else{var d=[];s.argumentAxis[t]&&d.push(M.getArgumentAxis()),s.valueAxis[t]&&(d=d.concat(r.valueAxes)),d.filter(u).forEach(function(e){e.handleZooming(null,{start:!0},n,t)}),l=l&&d.length}l&&M._requestChange(["VISUAL_RANGE"])}function o(e,t){var n=M._argumentAxes.filter(function(t){return s(r(t.getCanvas()),e)});return{fallback:M._lastRenderingTime>I,cancel:!n.length||!(0,d.isDefined)(t),action:t,curAxisRect:n.length&&r(n[0].getCanvas()),valueAxes:n.length&&M._valueAxes.filter(function(t){return s(r(t.getCanvas()),e)}),offset:{x:0,y:0},center:e,startCenter:e}}function a(e,t){var n=B.getRootOffset();return{x:O(E(t.pageX-n.left,e.x),e.width+e.x),y:O(E(t.pageY-n.top,e.y),e.height+e.y)}}function g(e){var t=B.getRootOffset(),n=e.pointers[0].pageX,i=e.pointers[1].pageX,o=e.pointers[0].pageY,a=e.pointers[1].pageY;return{x:O(n,i)+A(i-n)/2-t.left,y:O(o,a)+A(a-o)/2-t.top}}function _(e){var t=B.getRootOffset();return{x:e.pageX-t.left,y:e.pageY-t.top}}function P(e,t,n){return e.offset[n]-t.offset[n]}function R(e){e.cancelable!==!1&&(e.preventDefault(),e.stopPropagation()),M._stopCurrentHandling()}var M=this,B=this._renderer,V={dragStartHandler:function(e){var n,i=V.options,r="touch"===e.pointerType,s=i.argumentAxis.pan||i.valueAxis.pan,l=i.argumentAxis.zoom||i.valueAxis.zoom,u=(0,d.isDefined)(i.panKey)&&e[(0,f.normalizeEnum)(i.panKey)+"Key"],c=i.dragToZoom;if(e._cancelPreventDefault=!0,r){if(i.allowTouchGestures&&s){var h=!V.panningVisualRangeEnabled()||V.skipEvent;n=h?null:"pan"}}else c&&s&&u?n="pan":!c&&s?n="pan":c&&l&&(n="zoom");var p=o(_(e),n);return p.cancel?(V.skipEvent=!1,void(e.cancelable!==!1&&(e.cancel=!0))):(V.actionData=p,void("zoom"===n?(p.startCoords=a(p.curAxisRect,e),p.rect=B.rect(0,0,0,0).attr(i.dragBoxStyle).append(B.root)):t(V,"pan",e)))},dragHandler:function(e){var t=M.option("rotated"),i=V.options,o=V.actionData,r="touch"===e.pointerType;if(e._cancelPreventDefault=!0,o&&(!r||V.panningVisualRangeEnabled()))if("zoom"===o.action){R(e);var s=o.curAxisRect,l=o.startCoords,u=a(s,e),c=i.argumentAxis.zoom,d=i.valueAxis.zoom,h={x:O(l.x,u.x),y:O(l.y,u.y),width:A(l.x-u.x),height:A(l.y-u.y)};c&&d||(!c&&!t||!d&&t?(h.x=s.x,h.width=s.width):(h.y=s.y,h.height=s.height)),o.rect.attr(h)}else if("pan"===o.action){n(V,"pan",e,P,function(e){return e.offset});var f=Math.abs(e.offset.y-o.offset.y),p=Math.abs(e.offset.x-o.offset.x);if(r&&(f>T&&f>Math.abs(o.offset.x)||p>T&&p>Math.abs(o.offset.y)))return;R(e)}},dragEndHandler:function(e){var t=M.option("rotated"),n=V.options,o=V.actionData,r="touch"===e.pointerType,s=o&&"pan"===o.action&&!o.fallback&&0===o.offset.x&&0===o.offset.y;if(o&&(!r||V.panningVisualRangeEnabled())&&!s){if((!r||!V.actionData.isNative)&&R(e),"zoom"===o.action){var u=function(t,n,i,a,r,s){t=l(t,s);var u=r[i],d=a[i],h=!1,f=function(e){return{start:!!e,end:!!e}};return n&&A(u-d)>T&&t.some(function(t){var n=t.getTranslator();if(!n.getBusinessRange().isEmpty()){var i=function(){return[n.from(d),n.from(u)]},a=c(t,s,i,f,o.action,n.getMinScale(!0),e),r=a.stopInteraction,l=a.result;return h=!r,s&&l.isPrevented}}),h},d=a(o.curAxisRect,e),h=u(M._argumentAxes,n.argumentAxis.zoom,t?"y":"x",o.startCoords,d,M.getArgumentAxis()),f=u(o.valueAxes,n.valueAxis.zoom,t?"x":"y",o.startCoords,d);(f||h)&&M._requestChange(["VISUAL_RANGE"]),o.rect.dispose()}else"pan"===o.action&&i(V,"pan",e,P);V.actionData=null}},pinchStartHandler:function(n){var i=o(g(n),"zoom");return i.isNative=!V.panningVisualRangeEnabled(),i.cancel?void e(n):(V.actionData=i,void t(V,"zoom",n))},pinchHandler:function(e){if(V.actionData){var t=n(V,"zoom",e,function(e,t,n,i){return g(e)[n]-t.center[n]+(t.center[n]-t.center[n]*i)},g);V.defineTouchBehavior(!t,e),!t&&(V.actionData=null)}},pinchEndHandler:function(e){V.actionData&&(i(V,"zoom",e,function(e,t,n,i){return t.center[n]-t.startCenter[n]+(t.startCenter[n]-t.startCenter[n]*i)}),V.actionData=null)},cleanup:function(){B.root.off(m),V.actionData&&V.actionData.rect&&V.actionData.rect.dispose(),V.actionData=null,B.root.css({"touch-action":"","-ms-touch-action":""})},setup:function(o){if(V.cleanup(),o.argumentAxis.pan||B.root.on(k,e),!o.argumentAxis.none||!o.valueAxis.none){V.options=o;var a=M.option("rotated");(o.argumentAxis.zoom||o.valueAxis.zoom)&&o.allowMouseWheel&&B.root.on(p.name+m,function(e){function t(t,n,i,o){t=l(t,o);var a=!1,r=function(e){return{start:!!e,end:!!e}};return t.some(function(t){var s=t.getTranslator();if(!s.getBusinessRange().isEmpty()){var l=s.getMinScale(i>0),u=function(){var e=s.zoom(-(n-n*l),l,t.getZoomBounds());return{startValue:e.min,endValue:e.max}},d=c(t,o,u,r,"zoom",l,e),h=d.stopInteraction,f=d.result;return a=!h,o&&f.isPrevented}}),a}var n,i=_(e),u=!1;if(o.valueAxis.zoom){if(n=M._valueAxes.filter(function(e){return s(r(e.getCanvas()),i)}),0===n.length){var d=M._valueAxes.reduce(function(e,t){return!e&&t.coordsIn(i.x,i.y)&&(e=t.getCanvas()),e},null);d&&(n=M._valueAxes.filter(function(e){return s(r(e.getCanvas()),{x:d.left,y:d.top})}))}u|=t(n,a?i.x:i.y,e.delta)}if(o.argumentAxis.zoom){var h=M._argumentAxes.some(function(e){return!(!s(r(e.getCanvas()),i)&&!e.coordsIn(i.x,i.y))});u|=h&&t(M._argumentAxes,a?i.y:i.x,e.delta,M.getArgumentAxis())}u&&(M._requestChange(["VISUAL_RANGE"]),V.panningVisualRangeEnabled(n)&&R(e))}),o.allowTouchGestures&&(o.argumentAxis.zoom||o.valueAxis.zoom)&&B.root.on(x,{passive:!1},V.pinchStartHandler).on(w,{passive:!1},V.pinchHandler).on(C,V.pinchEndHandler),B.root.on(v,{immediate:!0,passive:!1},V.dragStartHandler).on(y,{immediate:!0,passive:!1},V.dragHandler).on(b,V.dragEndHandler),o.argumentAxis.pan&&B.root.on(k,function(e){V.actionData={valueAxes:[],offset:{x:0,y:0},center:{x:0,y:0}},R(e),t(V,"pan",e)}).on(S,function(e){R(e),n(V,"pan",e,P,function(e){return e.offset})}).on(D,function(e){R(e),i(V,"pan",e,P),V.actionData=null})}},defineTouchBehavior:function(e,t){V.actionData&&(V.actionData.isNative=e),e||R(t)},panningVisualRangeEnabled:function(e){if(null!==e&&void 0!==e&&e.length)return e.some(function(e){return!e.isExtremePosition(!1)||!e.isExtremePosition(!0)});var t=M._valueAxes.some(function(e){return!e.isExtremePosition(!1)||!e.isExtremePosition(!0)}),n=M._argumentAxes.some(function(e){return!e.isExtremePosition(!1)||!e.isExtremePosition(!0)});return t||n}};this._zoomAndPan=V},members:{_setupZoomAndPan:function(){this._zoomAndPan.setup(this._themeManager.getOptions("zoomAndPan"))}},dispose:function(){this._zoomAndPan.cleanup()},customize:function(e){e.addChange({code:"ZOOM_AND_PAN",handler:function(){this._setupZoomAndPan()},isThemeDependent:!0,isOptionChange:!0,option:"zoomAndPan"})}};t.default=P,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=b;return e.forEach(function(e){var n,i=null===(n=e.series)||void 0===n?void 0:n.getOptions(),o=e.fullState;"none"===(null===i||void 0===i?void 0:i.hoverMode)&&(o&=~x),"none"===(null===i||void 0===i?void 0:i.selectionMode)&&(o&=~w),t|=o}),k[t]}function a(e){return(0,u.isNumeric)(e)?e>1?e=1:e<0&&(e=0):e=void 0,e}t.default=void 0;var r=i(n(151)),s=n(14),l=n(0),u=n(1),c=n(3),d=n(134),h=i(n(9)),f=n(321),p=n(4),g=n(557),_=n(233),m=r.default.states,v=r.default.pieSeriesSpacing,y=["startAngle","innerRadius","segmentsDirection","type"],b=m.normalMark,x=m.hoverMark,w=m.selectedMark,C=5,k=[m.resetItem,m.applyHover,m.applySelected,m.applySelected],S=function(){function e(e,t){var n=t.filter(function(t){return t._isVisible()&&t.getSizeGroup()===e}),i=Math.min.apply(null,n.map(function(e){return e.getSizeGroupLayout().radius})),o=n.filter(function(e){return e.getSizeGroupLayout().radius===i});n.forEach(function(e){return e.render({force:!0,sizeGroupLayout:o.length?o[0].getSizeGroupLayout():{}})})}function t(e,t){return e.filter(function(e){return e!==t})}function n(e,n){return t(e,n).concat(n)}var i=[],o={};return{queue:function(t){var a=t.getSizeGroup();i=n(i,t),clearTimeout(o[a]),o[a]=setTimeout(function(){e(a,i)})},remove:function(e){i=t(i,e),i.length||(o={})}}}(),D=f.BaseChart.inherit({_themeSection:"pie",_layoutManagerOptions:function(){return(0,l.extend)(!0,{},this.callBase(),{piePercentage:a(this._themeManager.getOptions("diameter")),minPiePercentage:a(this._themeManager.getOptions("minDiameter"))})},_optionChangesMap:{diameter:"REINIT",minDiameter:"REINIT",sizeGroup:"REINIT"},_disposeCore:function(){S.remove(this),this.callBase()},_groupSeries:function(){var e=this.series;this._groupsData={groups:[{series:e,valueOptions:{valueType:"numeric"}}],argumentOptions:e[0]&&e[0].getOptions()}},getArgumentAxis:function(){return null},_getValueAxis:function(){var e=(new g.Translator1D).setCodomain(360,0);return{getTranslator:function(){return e},setBusinessRange:function(t){e.setDomain(t.min,t.max)}}},_populateBusinessRange:function(){this.series.map(function(e){var t=new d.Range;return t.addRange(e.getRangeData().val),e.getValueAxis().setBusinessRange(t),t})},_specialProcessSeries:function(){(0,c.each)(this.series,function(e,t){t.arrangePoints()})},_checkPaneName:function(){return!0},_processSingleSeries:function(e){this.callBase(e),e.arrangePoints()},_handleSeriesDataUpdated:function(){var e=0;this.series.forEach(function(t){e=Math.max(t.getPointsCount(),e)}),this.series.forEach(function(t){t.setMaxPointsCount(e)}),this.callBase()},_getLegendOptions:function(e){var t=this.callBase(e),n=t.legendData;return n.argument=e.argument,n.argumentIndex=e.argumentIndex,n.points=[e],t},_getLegendTargets:function(){var e=this,t={};(e.series||[]).forEach(function(e){e.getPoints().forEach(function(n){var i=n.argument.valueOf(),o=e.getPointsByArg(i).indexOf(n),a=i.valueOf().toString()+o;t[a]=t[a]||[];var r=t[a].push(n);n.index=t[a][r-2]?t[a][r-2].index:Object.keys(t).length-1,n.argumentIndex=o})});var n=[];return(0,c.each)(t,function(t,i){i.forEach(function(t,i){if(0===i)return void n.push(e._getLegendOptions(t));var o=n[n.length-1];o.legendData.points.push(t),o.visible||(o.visible=t.isVisible())})}),n},_getLayoutTargets:function(){return[{canvas:this._canvas}]},_getLayoutSeries:function(e,t){var n,i=this,o=i._canvas,a=!1;return n=i.layoutManager.applyPieChartSeriesLayout(o,e,!0),e.forEach(function(e){e.correctPosition(n,o),a=e.drawLabelsWOPoints()||a}),a&&(n=i.layoutManager.applyPieChartSeriesLayout(o,e,t.hideLayoutLabels)),e.forEach(function(e){e.hideLabels()}),i._sizeGroupLayout={x:n.centerX,y:n.centerY,radius:n.radiusOuter,drawOptions:t},n},_getLayoutSeriesForEqualPies:function(e,t){var n=this._canvas,i=this.layoutManager.applyEqualPieChartLayout(e,t);return e.forEach(function(e){e.correctPosition(i,n),e.drawLabelsWOPoints()}),this.layoutManager.correctPieLabelRadius(e,i,n),i},_updateSeriesDimensions:function(e){var t,n,i,o=this,a=o._getVisibleSeries(),r=a.length,s=e.sizeGroupLayout;r&&(i=s?o._getLayoutSeriesForEqualPies(a,s):o._getLayoutSeries(a,e),n=(i.radiusOuter-i.radiusInner-v*(r-1))/r,t=i.radiusInner,o._setGeometry(i),a.forEach(function(e){e.correctRadius({radiusInner:t,radiusOuter:t+n}),t+=n+v}))},_renderSeries:function(e,t,n){return this._calculateSeriesLayout(e,t),!e.sizeGroupLayout&&this.getSizeGroup()?(S.queue(this),void this._clearCanvas()):void this._renderSeriesElements(e,n)},_renderExtraElements:function(){var e=this,t=this.option("centerTemplate");this._centerTemplateGroup&&this._centerTemplateGroup.clear(),t&&(this._centerTemplateGroup||(this._centerTemplateGroup=this._renderer.g().attr({"class":"dxc-hole-template"}).css((0,s.patchFontOptions)(this._themeManager._font))),this._centerTemplateGroup.attr({visibility:"hidden"}).append(this._renderer.root),t=this._getTemplate(t),t.render({model:this,container:this._centerTemplateGroup.element,onRendered:function(){var t=e._centerTemplateGroup,n=t.getBBox();t.move(e._center.x-(n.x+n.width/2),e._center.y-(n.y+n.height/2)),t.attr({visibility:"visible"})}}))},getInnerRadius:function(){return this._innerRadius},_getLegendCallBack:function(){var e=this,t=this._legend,n=this._getLegendTargets().map(function(e){return e.legendData});return function(i){n.forEach(function(n){var a=[],r=t.getActionCallback({index:n.id});e.series.forEach(function(e){var t=e.getPointsByKeys(n.argument,n.argumentIndex);a.push.apply(a,t)}),i&&i.argument===n.argument&&i.argumentIndex===n.argumentIndex&&a.push(i),r(o(a))})}},_locateLabels:function(e){var t,n,i=0;do n=this._adjustSeriesLabels("shift"===e),t=this._resolveLabelOverlapping(e);while((t||n)&&++i<C)},_adjustSeriesLabels:function(e){return this.series.reduce(function(t,n){return n.adjustLabels(e)||t},!1)},_applyExtraSettings:p.noop,_resolveLabelOverlappingShift:function(){function e(e,t){return e.getVisiblePoints().reduce(function(e,t){var n=(0,s.normalizeAngle)(t.middleAngle);return(n<=90||n>=270?e.right:e.left).push(t),e},t||{left:[],right:[]})}function t(e,t){var n=!1;return a&&(e.left.reverse(),e.right.reverse()),n=f.overlapping.resolveLabelOverlappingInOneDirection(e.left,o._canvas,!1,t),f.overlapping.resolveLabelOverlappingInOneDirection(e.right,o._canvas,!1,t)||n}function n(e,t){return(0,s.getVerticallyShiftedAngularCoords)(e,-t,o._center)}function i(e,t){return{x:e.x,y:e.y-t}}var o=this,a="anticlockwise"===o.option("segmentsDirection"),r=o.series.reduce(function(e,t){return(e[t.getOptions().label.position]||e.outside).push(t),e},{inside:[],columns:[],outside:[]}),l=!1;return r.inside.length>0&&(l=t(r.inside.reduce(function(e,t){return t.getVisiblePoints().reduce(function(e,t){return e.left.push(t),e},e)},{left:[],right:[]}),i)||l),l=r.columns.reduce(function(n,o){return t(e(o),i)||n},l),r.outside.length>0&&(l=t(r.outside.reduce(function(t,n){return e(n,t)},null),n)||l),l},_setGeometry:function(e){var t=e.centerX,n=e.centerY,i=e.radiusInner;this._center={x:t,y:n},this._innerRadius=i},_disposeSeries:function(e){this.callBase.apply(this,arguments),this._abstractSeries=null},_legendDataField:"point",_legendItemTextField:"argument",_applyPointMarkersAutoHiding:p.noop,_renderTrackers:p.noop,_trackerType:"PieTracker",_createScrollBar:p.noop,_updateAxesLayout:p.noop,_applyClipRects:p.noop,_appendAdditionalSeriesGroups:p.noop,_prepareToRender:p.noop,_isLegendInside:p.noop,_renderAxes:p.noop,_shrinkAxes:p.noop,_isRotated:p.noop,_seriesPopulatedHandlerCore:p.noop,_reinitAxes:p.noop,_correctAxes:p.noop,_getExtraOptions:function(){var e=this;return{startAngle:e.option("startAngle"),innerRadius:e.option("innerRadius"),segmentsDirection:e.option("segmentsDirection"),type:e.option("type")}},getSizeGroup:function(){return this._themeManager.getOptions("sizeGroup")},getSizeGroupLayout:function(){return this._sizeGroupLayout||{}}});(0,c.each)(y,function(e,t){D.prototype._optionChangesMap[t]="REFRESH_SERIES_DATA_INIT"}),D.addPlugin(_.plugins.core),D.addPlugin(_.plugins.pieChart),(0,h.default)("dxPieChart",D);var I=D;t.default=I,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(4),a=i(n(9)),r=n(0),s=n(14),l=n(550),u=n(1),c=n(233),d="default",h=360,f=l.AdvancedChart.inherit({_themeSection:"polar",_createPanes:function(){return this.callBase(),[{name:d}]},_checkPaneName:function(){return!0},_getAxisRenderingOptions:function(e){var t="argumentAxis"===e,n=t?"circular":"linear",i=this.option("useSpiderWeb");return i&&(n+="Spider"),{axisType:"polarAxes",drawingType:n}},_prepareAxisOptions:function(e,t){var n="argumentAxis"===e,i=this._themeManager,o=this.option("argumentAxis"),a=i.getOptions("argumentAxis",o)||{},r=isFinite(a.startAngle)?(0,s.normalizeAngle)(a.startAngle):0;return{type:this.option("useSpiderWeb")&&n?"discrete":t.type,isHorizontal:!0,showCustomBoundaryTicks:n,startAngle:r,endAngle:r+360}},_optionChangesMap:{useSpiderWeb:"AXES_AND_PANES"},_getExtraOptions:function(){return{spiderWidget:this.option("useSpiderWeb")}},_prepareToRender:function(){return this._appendAxesGroups(),{}},_calcCanvas:function(){var e=(0,r.extend)({},this._canvas),t=this.getArgumentAxis(),n=t.getMargins();return Object.keys(n).forEach(function(t){return e[t]=e["original".concat(t[0].toUpperCase()).concat(t.slice(1))]+n[t]}),e},_renderAxes:function(e){var t=this,n=t._getValueAxis(),i=t.getArgumentAxis();i.draw(t._canvas),n.setSpiderTicks(i.getSpiderTicks());var o=t._calcCanvas();return i.updateSize(o),n.draw(o),o},_getValueAxis:function(){return this._valueAxes[0]},_shrinkAxes:function(e){var t=this._getValueAxis(),n=this.getArgumentAxis();if(e&&(e.width||e.height)){n.hideOuterElements();var i=this._calcCanvas();n.updateSize(i),t.updateSize(i)}},checkForMoreSpaceForPanesCanvas:function(){return this.layoutManager.needMoreSpaceForPanesCanvas([{canvas:this.getArgumentAxis().getCanvas()}],this._isRotated())},_getLayoutTargets:function(){return[{canvas:this._canvas}]},_getSeriesForPane:function(){return this.series},_applyClipRects:function(){var e=this._getCanvasClipRectID();this._createClipPathForPane(),this.getArgumentAxis().applyClipRects(this._getElementsClipRectID(),e),this._getValueAxis().applyClipRects(this._getElementsClipRectID(),e)},_createClipPathForPane:function(){var e=this,t=e._getValueAxis(),n=t.getCenter(),i=t.getRadius(),o=e._panesClipRects;n={x:Math.round(n.x),y:Math.round(n.y)},e._createClipCircle(o.fixed,n.x,n.y,i),e._createClipCircle(o.base,n.x,n.y,i),e.series.some(function(e){return e.areErrorBarsVisible()})?e._createClipCircle(o.wide,n.x,n.y,i):o.wide[0]=null},_createClipCircle:function(e,t,n,i){var o=this,a=e[0];a?a.attr({cx:t,cy:n,r:i}):(a=o._renderer.clipCircle(t,n,i),e[0]=a)},_applyExtraSettings:function(e){var t=this._panesClipRects.wide[0];e.setClippingParams(this._panesClipRects.base[0].id,t&&t.id,!1,!1)},getActualAngle:function(e){return this.getArgumentAxis().getOptions().inverted?h-e:e},getXYFromPolar:function(e,t,n,i){var o={angle:void 0,radius:void 0,x:void 0,y:void 0};if(!((0,u.isDefined)(e)||(0,u.isDefined)(t)||(0,u.isDefined)(n)||(0,u.isDefined)(i)))return o;var a,l,c=this.getArgumentAxis(),d=c.getAngles()[0];if((0,u.isDefined)(n)?a=c.getTranslator().translate(n):isFinite(e)?a=this.getActualAngle(e):(0,u.isDefined)(e)||(a=0),(0,u.isDefined)(i)?l=this.getValueAxis().getTranslator().translate(i):isFinite(t)?l=t:(0,u.isDefined)(t)||(l=c.getRadius()),(0,u.isDefined)(a)&&(0,u.isDefined)(l)){var h=(0,s.convertPolarToXY)(c.getCenter(),d,a,l);(0,r.extend)(o,h,{angle:c.getTranslatedAngle(a),radius:l})}return o},_applyPointMarkersAutoHiding:o.noop,_createScrollBar:o.noop,_isRotated:o.noop,_getCrosshairOptions:o.noop,_isLegendInside:o.noop});f.addPlugin(c.plugins.core),f.addPlugin(c.plugins.polarChart),(0,a.default)("dxPolarChart",f);var p=f;t.default=p,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(945)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){"@babel/helpers - typeof";return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return o=function(){return e},e}function a(e){if(e&&e.__esModule)return e;if(null===e||"object"!==i(e)&&"function"!=typeof e)return{"default":e};var t=o();if(t&&t.has(e))return t.get(e);var n={},a=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){var s=a?Object.getOwnPropertyDescriptor(e,r):null;s&&(s.get||s.set)?Object.defineProperty(n,r,s):n[r]=e[r]}return n.default=e,t&&t.set(e,n),n}function r(e){return e&&e.__esModule?e:{"default":e}}function s(e,t,n){var i,o=(0,c.extend)({},e);return n=n||{},t&&(o.left+=n.left||0,o.right-=n.right||0,o.top+=n.top||0,o.bottom-=n.bottom||0,t.width>0&&(i=(o.right-o.left-t.width)/2,i>0&&(o.left+=i,o.right-=i)),t.height>0&&(i=(o.bottom-o.top-t.height)/2,i>0&&(o.top+=i,o.bottom-=i))),o}t.default=void 0;var l=n(3),u=r(n(9)),c=n(0),d=n(50),h=n(203),f=n(558),p=n(14),g=a(n(948)),_=r(n(949)),m=Math.max,v=Math.min,y=Math.round,b=f.dxGauge.inherit({_rootClass:"dxg-linear-gauge",_factoryMethods:{rangeContainer:"createLinearRangeContainer",indicator:"createLinearIndicator"},_gridSpacingFactor:25,_scaleTypes:{type:"xyAxes",drawingType:"linear"},_getTicksOrientation:function(e){return e.isHorizontal?e.verticalOrientation:e.horizontalOrientation},_getThemeManagerOptions:function(){var e=this.callBase.apply(this,arguments);return e.subTheme="_linear",e},_getInvertedState:function(){return!this._area.vertical&&this.option("rtlEnabled")},_prepareScaleSettings:function(){var e=this.callBase();return e.inverted=this._getInvertedState(),e},_updateScaleTickIndent:function(e){var t,n,i,o=e.label.indentFromTick,a=e.tick.length,r=this._scale.measureLabels((0,c.extend)({},this._canvas)),s=e.isHorizontal?r.height+r.y:0,l=o>0;e.isHorizontal?(t=l?{center:.5,top:0,bottom:1}:{center:.5,top:1,bottom:0},i=a*t[e.verticalOrientation],n=r.y):(t=l?{center:.5,left:0,right:1}:{center:.5,left:1,right:0},i=a*t[e.horizontalOrientation],n=-r.width),e.label.indentFromAxis=-o+(l?-i+n:i-s),this._scale.updateOptions(e)},_shiftScale:function(e,t){var n=this,i=(0,c.extend)({},n._canvas),o=t.isHorizontal,a=n._scale;i[o?"left":"top"]=n._area[o?"startCoord":"endCoord"],i[o?"right":"bottom"]=i[o?"width":"height"]-n._area[o?"endCoord":"startCoord"],a.draw(i),a.shift({left:-e.x,top:-e.y})},_setupCodomain:function(){var e=this,t=e.option("geometry")||{},n="vertical"===(0,p.normalizeEnum)(t.orientation),i=-100,o=100;e._area={vertical:n,x:0,y:0,startCoord:i,endCoord:o},e._rangeContainer.vertical=n,e._translator.setInverted(e._getInvertedState()),e._translator.setCodomain(i,o)},_getScaleLayoutValue:function(){return this._area[this._area.vertical?"x":"y"]},_getTicksCoefficients:function(e){var t={inner:0,outer:1};return this._area.vertical?"left"===e.horizontalOrientation?(t.inner=1,t.outer=0):"center"===e.horizontalOrientation&&(t.inner=t.outer=.5):"top"===e.verticalOrientation?(t.inner=1,t.outer=0):"center"===e.verticalOrientation&&(t.inner=t.outer=.5),t},_correctScaleIndents:function(e,t,n){var i=this._area.vertical;t>=0?e.max+=t+n[i?"width":"height"]:e.min-=-t+n[i?"width":"height"],e.indent=n[i?"height":"width"]/2},_measureMainElements:function(e,t){var n=this,i=n._area.x,o=n._area.y,a=1e3,r=0,s=0,u=n._scale;return(0,l.each)(e.concat(u),function(e,n){var l=n.measure?n.measure({x:i+n.getOffset(),y:o+n.getOffset()}):t;void 0!==l.max&&(r=m(r,l.max)),void 0!==l.min&&(a=v(a,l.min)),l.indent>0&&(s=m(s,l.indent))}),{minBound:a,maxBound:r,indent:s}},_applyMainLayout:function(e,t){var n,i,o=this,a=o._measureMainElements(e,t),r=o._area;r.vertical?(n=s(o._innerRect,{width:a.maxBound-a.minBound}),i=(n.left+n.right)/2-(a.minBound+a.maxBound)/2,r.startCoord=n.bottom-a.indent,r.endCoord=n.top+a.indent,r.x=y(r.x+i)):(n=s(o._innerRect,{height:a.maxBound-a.minBound}),i=(n.top+n.bottom)/2-(a.minBound+a.maxBound)/2,r.startCoord=n.left+a.indent,r.endCoord=n.right-a.indent,r.y=y(r.y+i)),o._translator.setCodomain(r.startCoord,r.endCoord),o._innerRect=n},_getElementLayout:function(e){return{x:y(this._area.x+e),y:y(this._area.y+e)}},_getApproximateScreenRange:function(){var e=this,t=e._area,n=t.vertical?e._canvas.height:e._canvas.width;return n>t.totalSize&&(n=t.totalSize),n=.8*n},_getDefaultSize:function(){var e=this.option("geometry")||{};return"vertical"===e.orientation?{width:100,height:300}:{width:300,height:100}},_factory:(0,d.clone)(h.dxBaseGauge.prototype._factory)}),x=b.prototype._factory.indicators={};b.prototype._factory.createIndicator=(0,f.createIndicatorCreator)(x),x._default=g._default,x.rectangle=g.rectangle,x.rhombus=g.rhombus,x.circle=g.circle,x.trianglemarker=g.trianglemarker,x.textcloud=g.textcloud,x.rangebar=g.rangebar,b.prototype._factory.RangeContainer=_.default,(0,u.default)("dxLinearGauge",b);var w=b;t.default=w,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(0),o=n(322),a=i.extend,r=o.BaseThemeManager.inherit({ctor:function(e){this.callBase.apply(this,arguments),this._subTheme=e.subTheme},_initializeTheme:function(){var e,t=this;t._subTheme&&(e=a(!0,{},t._theme[t._subTheme],t._theme),a(!0,t._theme,e)),t.callBase.apply(t,arguments)}}),s={ThemeManager:r};t.default=s,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=e.data.tracker;t._x=e.pageX,t._y=e.pageY,t._showTooltip(e)}function a(e){e.data.tracker._hideTooltip(v)}function r(e){e.data.tracker._hideTooltip()}function s(e){var t=x=e.data.tracker;t._touch=!0,o(e)}function l(){x._touch=!1}function u(e){var t=x;t&&!t._touch&&(t._hideTooltip(v),x=null)}t.default=void 0;var c=i(n(5)),d=i(n(15)),h=i(n(11)),f=n(139),p=i(n(49)),g=n(8),_=i(n(25)),m="gauge-tooltip",v=100,y=p.default.add,b=d.default.inherit({ctor:function(e){var t=this;t._element=e.renderer.g().attr({"class":"dxg-tracker",stroke:"none","stroke-width":0,fill:"#000000",opacity:1e-4}).linkOn(e.container,{name:"tracker",after:"peripheral"}),t._showTooltipCallback=function(){var e=t._tooltipEvent.target,n=e["gauge-data-target"],i=e["gauge-data-info"];if(t._targetEvent=null,t._tooltipTarget!==e){var o=function(n){n&&(t._tooltipTarget=e)};o(t._callbacks["tooltip-show"](n,i,o))}},t._hideTooltipCallback=function(){t._hideTooltipTimeout=null,t._targetEvent=null,t._tooltipTarget&&(t._callbacks["tooltip-hide"](),t._tooltipTarget=null)},t._dispose=function(){clearTimeout(t._hideTooltipTimeout),t._showTooltipCallback=t._hideTooltipCallback=t._dispose=null}},dispose:function(){var e=this;return e._dispose(),e.deactivate(),e._element.off("."+m),e._element.linkOff(),e._element=e._context=e._callbacks=null,e},activate:function(){return this._element.linkAppend(),this},deactivate:function(){return this._element.linkRemove().clear(),this},attach:function(e,t,n){return e.data({"gauge-data-target":t,"gauge-data-info":n}).append(this._element),this},detach:function(e){return e.remove(),this},setTooltipState:function(e){var t=this;if(t._element.off("."+m),e){var n={tracker:t};t._element.on((0,g.addNamespace)([_.default.move],m),n,o).on((0,g.addNamespace)([_.default.out],m),n,a).on((0,g.addNamespace)([_.default.down],m),n,s).on((0,g.addNamespace)([_.default.up],m),n,l).on((0,g.addNamespace)([f.name],m),n,r)}return t},setCallbacks:function(e){return this._callbacks=e,this},_showTooltip:function(e){var t=this;clearTimeout(t._hideTooltipTimeout),t._hideTooltipTimeout=null,t._tooltipTarget!==e.target&&(t._tooltipEvent=e,t._showTooltipCallback())},_hideTooltip:function(e){var t=this;clearTimeout(t._hideTooltipTimeout),e?t._hideTooltipTimeout=setTimeout(t._hideTooltipCallback,e):t._hideTooltipCallback()}}),x=null;y(function(){c.default.subscribeGlobal(h.default.getDocument(),(0,g.addNamespace)([_.default.down],m),u)});var w=b;t.default=w,e.exports=t.default},function(e,t,n){t.textcloud=t.trianglemarker=t.circle=t.rhombus=t.rectangle=t.rangebar=t._default=void 0;var i=n(330),o=n(14),a=Number,r=i.BaseIndicator.inherit({_move:function(){var e=this,t=e._actualPosition-e._zeroPosition;e._rootElement.move(e.vertical?0:t,e.vertical?t:0),e._trackerElement&&e._trackerElement.move(e.vertical?0:t,e.vertical?t:0)},_isEnabled:function(){return this.vertical=this._options.vertical,this._options.length>0&&this._options.width>0},_isVisible:function(){return!0},_getTrackerSettings:function(){var e,t,n,i,o=this._options,a=o.width/2,r=o.length/2,s=this._zeroPosition;return a>10||(a=10),r>10||(r=10),this.vertical?(e=o.x-r,t=o.x+r,n=s+a,i=s-a):(e=s-a,t=s+a,n=o.y+r,i=o.y-r),{points:[e,n,e,i,t,i,t,n]}},_render:function(){var e=this;e._zeroPosition=e._translator.getCodomainStart()},_clear:function(){delete this._element},measure:function(e){var t=this.vertical?e.x:e.y;
return{min:t-this._options.length/2,max:t+this._options.length/2}},getTooltipParameters:function(){var e=this,t=e._options,n=e._actualPosition,i={x:n,y:n,value:e._currentValue,color:t.color,offset:t.width/2};return e.vertical?i.x=t.x:i.y=t.y,i}}),s=r.inherit({_render:function(){var e,t,n,i,o=this,a=o._options;o.callBase();var r=o._zeroPosition;o.vertical?(e=a.x-a.length/2,t=a.x+a.length/2,n=r+a.width/2,i=r-a.width/2):(e=r-a.width/2,t=r+a.width/2,n=a.y+a.length/2,i=a.y-a.length/2),o._element=o._element||o._renderer.path([],"area").append(o._rootElement),o._element.attr({points:[e,n,e,i,t,i,t,n]})}});t.rectangle=s;var l=r.inherit({_render:function(){var e,t,n,i,o=this,a=o._options;o.callBase(),o.vertical?(e=a.x,t=o._zeroPosition,n=a.length/2||0,i=a.width/2||0):(e=o._zeroPosition,t=a.y,n=a.width/2||0,i=a.length/2||0),o._element=o._element||o._renderer.path([],"area").append(o._rootElement),o._element.attr({points:[e-n,t,e,t-i,e+n,t,e,t+i]})}});t.rhombus=l;var u=r.inherit({_render:function(){var e,t,n=this,i=n._options;n.callBase(),n.vertical?(e=i.x,t=n._zeroPosition):(e=n._zeroPosition,t=i.y);var o=i.length/2||0;n._element=n._element||n._renderer.circle().append(n._rootElement),n._element.attr({cx:e,cy:t,r:o})}});t.circle=u;var c=r.inherit({_isEnabled:function(){var e=this;return e.vertical=e._options.vertical,e._inverted=e.vertical?"right"===(0,o.normalizeEnum)(e._options.horizontalOrientation):"bottom"===(0,o.normalizeEnum)(e._options.verticalOrientation),e._options.length>0&&e._options.width>0},_isVisible:function(){return!0},_render:function(){var e,t,n,i,o=this,r=o._options,s={stroke:"none","stroke-width":0,"stroke-linecap":"square"};o.callBase(),o.vertical?(e=r.x,n=o._zeroPosition,t=e+a(o._inverted?r.length:-r.length),s.points=[e,n,t,n-r.width/2,t,n+r.width/2]):(n=r.y,e=o._zeroPosition,i=n+a(o._inverted?r.length:-r.length),s.points=[e,n,e-r.width/2,i,e+r.width/2,i]),r.space>0&&(s["stroke-width"]=Math.min(r.space,r.width/4)||0,s.stroke=s["stroke-width"]>0?r.containerBackgroundColor||"none":"none"),o._element=o._element||o._renderer.path([],"area").append(o._rootElement),o._element.attr(s).sharp()},_getTrackerSettings:function(){var e,t,n,i,o,r=this,s=r._options,l=s.width/2,u=a(s.length);return l>10||(l=10),u>20||(u=20),r.vertical?(e=t=s.x,t=e+(r._inverted?u:-u),n=r._zeroPosition+l,i=r._zeroPosition-l,o=[e,n,t,n,t,i,e,i]):(n=s.y,i=n+(r._inverted?u:-u),e=r._zeroPosition-l,t=r._zeroPosition+l,o=[e,n,e,i,t,i,t,n]),{points:o}},measure:function(e){var t,n,i=this,o=a(i._options.length);return i.vertical?(t=n=e.x,i._inverted?n=t+o:t=n-o):(t=n=e.y,i._inverted?n=t+o:t=n-o),{min:t,max:n,indent:i._options.width/2}},getTooltipParameters:function(){var e=this,t=e._options,n=(e._inverted?t.length:-t.length)/2,i=e.callBase();return e.vertical?i.x+=n:i.y+=n,i.offset=t.length/2,i}});t.trianglemarker=c;var d=i.BaseTextCloudMarker.inherit({_isEnabled:function(){var e=this;return e.vertical=e._options.vertical,e._inverted=e.vertical?"right"===(0,o.normalizeEnum)(e._options.horizontalOrientation):"bottom"===(0,o.normalizeEnum)(e._options.verticalOrientation),!0},_isVisible:function(){return!0},_getTextCloudOptions:function(){var e,t=this,n=t._actualPosition,i=t._actualPosition;return t.vertical?(n=t._options.x,e=t._inverted?"top-left":"top-right"):(i=t._options.y,e=t._inverted?"right-top":"right-bottom"),{x:n,y:i,type:e}},measure:function(e){var t,n,i,o=this,r=a(o._options.arrowLength)||0;return o._measureText(),o.vertical?(i=o._textFullHeight,o._inverted?(t=e.x,n=e.x+r+o._textFullWidth):(t=e.x-r-o._textFullWidth,n=e.x)):(i=o._textFullWidth,o._inverted?(t=e.y,n=e.y+r+o._textFullHeight):(t=e.y-r-o._textFullHeight,n=e.y)),{min:t,max:n,indent:i}}});t.textcloud=d;var h=i.BaseRangeBar.inherit({_isEnabled:function(){var e=this;return e.vertical=e._options.vertical,e._inverted=e.vertical?"right"===(0,o.normalizeEnum)(e._options.horizontalOrientation):"bottom"===(0,o.normalizeEnum)(e._options.verticalOrientation),e._options.size>0},_isVisible:function(){return!0},_createBarItem:function(){return this._renderer.path([],"area").append(this._rootElement)},_createTracker:function(){return this._renderer.path([],"area")},_setBarSides:function(){var e,t,n=this,i=n._options,o=a(i.size);n.vertical?n._inverted?(e=i.x,t=i.x+o):(e=i.x-o,t=i.x):n._inverted?(e=i.y,t=i.y+o):(e=i.y-o,t=i.y),n._minSide=e,n._maxSide=t,n._minBound=e,n._maxBound=t},_getSpace:function(){var e=this._options;return e.space>0?a(e.space):0},_isTextVisible:function(){var e=this._options.text||{};return e.indent>0||e.indent<0},_getTextAlign:function(){return this.vertical?this._options.text.indent>0?"left":"right":"center"},_setTextItemsSides:function(){var e=this,t=a(e._options.text.indent);t>0?(e._lineStart=e._maxSide,e._lineEnd=e._maxSide+t,e._textPosition=e._lineEnd+(e.vertical?2:e._textHeight/2),e._maxBound=e._textPosition+(e.vertical?e._textWidth:e._textHeight/2)):t<0&&(e._lineStart=e._minSide,e._lineEnd=e._minSide+t,e._textPosition=e._lineEnd-(e.vertical?2:e._textHeight/2),e._minBound=e._textPosition-(e.vertical?e._textWidth:e._textHeight/2))},_getPositions:function(){var e,t,n,i,o=this,a=o._startPosition,r=o._endPosition,s=o._space,l=o._basePosition,u=o._actualPosition;return a<r?(l<u?(e=l,t=u):(e=u,t=l),n=e-s,i=t+s):(l>u?(e=l,t=u):(e=u,t=l),n=e+s,i=t-s),{start:a,end:r,main1:e,main2:t,back1:n,back2:i}},_buildItemSettings:function(e,t){var n=this,i=n._minSide,o=n._maxSide,a=n.vertical?[i,e,i,t,o,t,o,e]:[e,i,e,o,t,o,t,i];return{points:a}},_updateTextPosition:function(){var e=this;e._text.attr(e.vertical?{x:e._textPosition,y:e._actualPosition+e._textVerticalOffset}:{x:e._actualPosition,y:e._textPosition+e._textVerticalOffset})},_updateLinePosition:function(){var e,t,n,i=this,o=i._actualPosition;i.vertical?(i._basePosition>=o?(e=o,t=o+2):(e=o-2,t=o),n=[i._lineStart,e,i._lineStart,t,i._lineEnd,t,i._lineEnd,e]):(i._basePosition<=o?(e=o-2,t=o):(e=o,t=o+2),n=[e,i._lineStart,e,i._lineEnd,t,i._lineEnd,t,i._lineStart]),i._line.attr({points:n}).sharp()},_getTooltipPosition:function(){var e=this,t=(e._minSide+e._maxSide)/2,n=(e._basePosition+e._actualPosition)/2;return e.vertical?{x:t,y:n}:{x:n,y:t}},measure:function(e){var t,n,i,o=this,r=a(o._options.size),s=a(o._options.text.indent);return o._measureText(),o.vertical?(t=n=e.x,o._inverted?n+=r:t-=r,o._hasText&&(i=o._textHeight/2,s>0&&(n+=s+o._textWidth),s<0&&(t+=s-o._textWidth))):(t=n=e.y,o._inverted?n+=r:t-=r,o._hasText&&(i=o._textWidth/2,s>0&&(n+=s+o._textHeight),s<0&&(t+=s-o._textHeight))),{min:t,max:n,indent:i}}});t.rangebar=t._default=h},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(559)),a=n(14),r=Number,s=Math.max,l=o.default.inherit({_processOptions:function(){var e=this;if(e.vertical=e._options.vertical,e._inner=e._outer=0,e.vertical)switch((0,a.normalizeEnum)(e._options.horizontalOrientation)){case"left":e._inner=1;break;case"center":e._inner=e._outer=.5;break;default:e._outer=1}else switch((0,a.normalizeEnum)(e._options.verticalOrientation)){case"top":e._inner=1;break;case"center":e._inner=e._outer=.5;break;default:e._outer=1}},_isVisible:function(){return!0},_createRange:function(e,t){var n,i=this,o=i._inner,a=i._outer,r=i._translator.translate(e.start),s=i._translator.translate(e.end),l=t.x,u=t.y,c=e.startWidth,d=e.endWidth;return n=i.vertical?[l-c*o,r,l-d*o,s,l+d*a,s,l+c*a,r]:[r,u+c*a,r,u-c*o,s,u-d*o,s,u+d*a],i._renderer.path(n,"area")},measure:function(e){var t,n={};return n.min=n.max=e[this.vertical?"x":"y"],t=this._options.width,t=r(t)||s(r(t.start),r(t.end)),n.min-=this._inner*t,n.max+=this._outer*t,n}}),u=l;t.default=u,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(560)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){t.rangebar=t.textcloud=t.trianglemarker=t.twocolorneedle=t.triangleneedle=t.rectangleneedle=t._default=void 0;var i=n(330),o=n(14),a=Number,r=o.getCosAndSin,s=o.convertAngleToRendererSpace,l=i.BaseIndicator.inherit({_move:function(){var e=this,t=e._options,n=s(e._actualPosition);e._rootElement.rotate(n,t.x,t.y),e._trackerElement&&e._trackerElement.rotate(n,t.x,t.y)},_isEnabled:function(){return this._options.width>0},_isVisible:function(e){return e.radius-a(this._options.indentFromCenter)>0},_getTrackerSettings:function(){var e=this._options,t=this._getRadius(),n=this._getIndentFromCenter(),i=e.x,o=e.y-(t+n)/2,a=e.width/2,r=(t-n)/2;return a>10||(a=10),r>10||(r=10),{points:[i-a,o-r,i-a,o+r,i+a,o+r,i+a,o-r]}},_render:function(){var e=this;e._renderPointer()},_clearPointer:function(){delete this._element},_clear:function(){this._clearPointer()},_getIndentFromCenter:function(e){return Number(this._options.indentFromCenter)||0},_getRadius:function(){return 0},measure:function(e){var t={max:e.radius};return this._options.indentFromCenter<0&&(t.inverseHorizontalOffset=t.inverseVerticalOffset=-a(this._options.indentFromCenter)),t},getTooltipParameters:function(){var e=this._options,t=r(this._actualPosition),n=(this._getRadius()+this._getIndentFromCenter())/2;return{x:e.x+t.cos*n,y:e.y-t.sin*n,value:this._currentValue,color:e.color,offset:e.width/2}}}),u=l.inherit({_isVisible:function(e){var t=this._adjustOffset(Number(this._options.indentFromCenter),e.radius),n=this._adjustOffset(Number(this._options.offset),e.radius);return e.radius-t-n>0},getOffset:function(){return 0},_adjustOffset:function(e,t){var n=Number(this._options.beginAdaptingAtRadius),i=t/n;return i<1&&(e=Math.floor(e*i)),e||0},_getIndentFromCenter:function(e){return this._adjustOffset(Number(this._options.indentFromCenter),this._options.radius)},_getRadius:function(){var e=this._options;return e.radius-this._adjustOffset(Number(e.offset),e.radius)},_renderSpindle:function(){var e=this,t=e._options,n=t.radius,i=2*this._adjustOffset(a(t.spindleSize)/2,n),o=2*this._adjustOffset(a(t.spindleGapSize)/2,n)||0;o>0&&(o=o<=i?o:i),i>0&&(e._spindleOuter=e._spindleOuter||e._renderer.circle().append(e._rootElement),e._spindleInner=e._spindleInner||e._renderer.circle().append(e._rootElement),e._spindleOuter.attr({"class":"dxg-spindle-border",cx:t.x,cy:t.y,r:i/2}),e._spindleInner.attr({"class":"dxg-spindle-hole",cx:t.x,cy:t.y,r:o/2,fill:t.containerBackgroundColor}))},_render:function(){var e=this;e.callBase(),e._renderSpindle()},_clear:function(){this.callBase(),delete this._spindleOuter,delete this._spindleInner}}),c=u.inherit({_renderPointer:function(){var e=this,t=e._options,n=t.y-this._getRadius(),i=t.y-this._getIndentFromCenter(),o=t.x-t.width/2,r=o+a(t.width);e._element=e._element||e._renderer.path([],"area").append(e._rootElement),e._element.attr({points:[o,i,o,n,r,n,r,i]})}});t.rectangleneedle=t._default=c;var d=u.inherit({_renderPointer:function(){var e=this,t=e._options,n=t.y-this._getRadius(),i=t.y-this._getIndentFromCenter(),o=t.x-t.width/2,a=t.x+t.width/2;e._element=e._element||e._renderer.path([],"area").append(e._rootElement),e._element.attr({points:[o,i,t.x,n,a,i]})}});t.triangleneedle=d;var h=u.inherit({_renderPointer:function(){var e,t,n=this,i=n._options,o=i.x-i.width/2,r=i.x+i.width/2,s=i.y-this._getRadius(),l=i.y-this._getIndentFromCenter(),u=a(i.secondFraction)||0;u>=1?e=t=l:u<=0?e=t=s:(t=s+(l-s)*u,e=t+a(i.space)),n._firstElement=n._firstElement||n._renderer.path([],"area").append(n._rootElement),n._spaceElement=n._spaceElement||n._renderer.path([],"area").append(n._rootElement),n._secondElement=n._secondElement||n._renderer.path([],"area").append(n._rootElement),n._firstElement.attr({points:[o,l,o,e,r,e,r,l]}),n._spaceElement.attr({points:[o,e,o,t,r,t,r,e],"class":"dxg-hole",fill:i.containerBackgroundColor}),n._secondElement.attr({points:[o,t,o,s,r,s,r,t],"class":"dxg-part",fill:i.secondColor})},_clearPointer:function(){delete this._firstElement,delete this._secondElement,delete this._spaceElement}});t.twocolorneedle=h;var f=l.inherit({_isEnabled:function(){return this._options.length>0&&this._options.width>0},_isVisible:function(e){return e.radius>0},_render:function(){var e=this,t=e._options,n=t.x,i=t.y-t.radius,o=t.width/2||0,r=i-a(t.length);e._element=e._element||e._renderer.path([],"area").append(e._rootElement);var s={points:[n,i,n-o,r,n+o,r],stroke:"none","stroke-width":0,"stroke-linecap":"square"};t.space>0&&(s["stroke-width"]=Math.min(t.space,t.width/4)||0,s.stroke=s["stroke-width"]>0?t.containerBackgroundColor||"none":"none"),e._element.attr(s).sharp()},_clear:function(){delete this._element},_getTrackerSettings:function(){var e=this._options,t=e.x,n=e.y-e.radius-e.length/2,i=e.width/2,o=e.length/2;return i>10||(i=10),o>10||(o=10),{points:[t-i,n-o,t-i,n+o,t+i,n+o,t+i,n-o]}},measure:function(e){return{min:e.radius,max:e.radius+a(this._options.length)}},getTooltipParameters:function(){var e=this._options,t=r(this._actualPosition),n=e.radius+e.length/2,i=this.callBase();return i.x=e.x+t.cos*n,i.y=e.y-t.sin*n,i.offset=e.length/2,i}});t.trianglemarker=f;var p=i.BaseTextCloudMarker.inherit({_isEnabled:function(){return!0},_isVisible:function(e){return e.radius>0},_getTextCloudOptions:function(){var e=this,t=r(e._actualPosition),n=(0,o.normalizeAngle)(e._actualPosition);return{x:e._options.x+t.cos*e._options.radius,y:e._options.y-t.sin*e._options.radius,type:n>270?"left-top":n>180?"top-right":n>90?"right-bottom":"bottom-left"}},measure:function(e){var t=this,n=a(t._options.arrowLength)||0;t._measureText();var i=t._textFullHeight+n,o=t._textFullWidth+n;return{min:e.radius,max:e.radius,horizontalOffset:o,verticalOffset:i,inverseHorizontalOffset:o,inverseVerticalOffset:i}}});t.textcloud=p;var g=i.BaseRangeBar.inherit({_isEnabled:function(){return this._options.size>0},_isVisible:function(e){return e.radius-a(this._options.size)>0},_createBarItem:function(){return this._renderer.arc().attr({"stroke-linejoin":"round"}).append(this._rootElement)},_createTracker:function(){return this._renderer.arc().attr({"stroke-linejoin":"round"})},_setBarSides:function(){var e=this;e._maxSide=e._options.radius,e._minSide=e._maxSide-a(e._options.size)},_getSpace:function(){var e=this._options;return e.space>0?180*e.space/e.radius/Math.PI:0},_isTextVisible:function(){var e=this._options.text||{};return e.indent>0},_setTextItemsSides:function(){var e=this,t=e._options,n=a(t.text.indent);e._lineFrom=t.y-t.radius,e._lineTo=e._lineFrom-n,e._textRadius=t.radius+n},_getPositions:function(){var e,t,n=this,i=n._basePosition,o=n._actualPosition;return i>=o?(e=i,t=o):(e=o,t=i),{start:n._startPosition,end:n._endPosition,main1:e,main2:t,back1:Math.min(e+n._space,n._startPosition),back2:Math.max(t-n._space,n._endPosition)}},_buildItemSettings:function(e,t){var n=this;return{x:n._options.x,y:n._options.y,innerRadius:n._minSide,outerRadius:n._maxSide,startAngle:t,endAngle:e}},_updateTextPosition:function(){var e=this,t=r(e._actualPosition),n=e._options.x+e._textRadius*t.cos,i=e._options.y-e._textRadius*t.sin;n+=t.cos*e._textWidth*.6,i-=t.sin*e._textHeight*.6,e._text.attr({x:n,y:i+e._textVerticalOffset})},_updateLinePosition:function(){var e,t,n=this,i=n._options.x;n._basePosition>n._actualPosition?(e=i-2,t=i):n._basePosition<n._actualPosition?(e=i,t=i+2):(e=i-1,t=i+1),n._line.attr({points:[e,n._lineFrom,e,n._lineTo,t,n._lineTo,t,n._lineFrom]}).rotate(s(n._actualPosition),i,n._options.y).sharp()},_getTooltipPosition:function(){var e=this,t=r((e._basePosition+e._actualPosition)/2),n=(e._minSide+e._maxSide)/2;return{x:e._options.x+t.cos*n,y:e._options.y-t.sin*n}},measure:function(e){var t=this,n={min:e.radius-a(t._options.size),max:e.radius};return t._measureText(),t._hasText&&(n.max+=a(t._options.text.indent),n.horizontalOffset=t._textWidth,n.verticalOffset=t._textHeight),n}});t.rangebar=g},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(559)),a=n(14),r=Number,s=Math.max,l=o.default.inherit({_processOptions:function(){var e=this;switch(e._inner=e._outer=0,(0,a.normalizeEnum)(e._options.orientation)){case"inside":e._inner=1;break;case"center":e._inner=e._outer=.5;break;default:e._outer=1}},_isVisible:function(e){var t=this._options.width;return t=r(t)||s(r(t.start),r(t.end)),e.radius-this._inner*t>0},_createRange:function(e,t){var n=this,i=(e.startWidth+e.endWidth)/2;return n._renderer.arc(t.x,t.y,t.radius-n._inner*i,t.radius+n._outer*i,n._translator.translate(e.end),n._translator.translate(e.start)).attr({"stroke-linejoin":"round"})},measure:function(e){var t=this._options.width;return t=r(t)||s(r(t.start),r(t.end)),{min:e.radius-this._inner*t,max:e.radius+this._outer*t}}}),u=l;t.default=u,e.exports=t.default},function(e,t,n){t.default=void 0;var i=n(954),o=i.dxBarGauge;t.default=o,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){e.startAngle=t<n?t:n,e.endAngle=t<n?n:t}function a(e,t){return _(e-t)<1e-4}t.dxBarGauge=void 0;var r,s=i(n(9)),l=n(50),u=n(4),c=n(0),d=n(14),h=n(203),f=i(n(560)),p=n(228),g=Math.PI/180,_=Math.abs,m=Math.round,v=Math.floor,y=Math.min,b=Math.max,x=h.getSampleText,w=h.formatValue,C=h.compareArrays,k=Array.isArray,S=d.convertAngleToRendererSpace,D=d.getCosAndSin,I=d.patchFontOptions,T=Number,O=isFinite,E=u.noop,A=c.extend,P="values",R=h.dxBaseGauge.inherit({_rootClass:"dxbg-bar-gauge",_themeSection:"barGauge",_fontFields:["label.font","legend.font","legend.title.font","legend.title.subtitle.font"],_initCore:function(){var e=this;e.callBase.apply(e,arguments),e._barsGroup=e._renderer.g().attr({"class":"dxbg-bars"}).linkOn(e._renderer.root,"bars"),e._values=[],e._context={renderer:e._renderer,translator:e._translator,tracker:e._tracker,group:e._barsGroup},e._animateStep=function(t){var n,i,o=e._bars;for(n=0,i=o.length;n<i;++n)o[n].animate(t)},e._animateComplete=function(){e._bars.forEach(function(e){return e.endAnimation()}),e._checkOverlap()}},_disposeCore:function(){var e=this;e._barsGroup.linkOff(),e._barsGroup=e._values=e._context=e._animateStep=e._animateComplete=null,e.callBase.apply(e,arguments)},_setupDomainCore:function(){var e=this,t=e.option("startValue"),n=e.option("endValue");O(t)||(t=0),O(n)||(n=100),e._translator.setDomain(t,n),e._baseValue=e._translator.adjust(e.option("baseValue")),O(e._baseValue)||(e._baseValue=t<n?t:n)},_getDefaultSize:function(){return{width:300,height:300}},_setupCodomain:f.default.prototype._setupCodomain,_getApproximateScreenRange:function(){var e=this,t=e._area.sides,n=e._canvas.width/(t.right-t.left),i=e._canvas.height/(t.down-t.up),o=n<i?n:i;return-e._translator.getCodomainRange()*o*g},_setupAnimationSettings:function(){var e=this;e.callBase.apply(e,arguments),e._animationSettings&&(e._animationSettings.step=e._animateStep,e._animationSettings.complete=e._animateComplete)},_cleanContent:function(){var e=this;e._barsGroup.linkRemove(),e._animationSettings&&e._barsGroup.stopAnimation(),e._barsGroup.clear()},_renderContent:function(){var e,t,n=this,i=n.option("label"),o=n._context;n._barsGroup.linkAppend(),o.textEnabled=void 0===i||i&&(!("visible"in i)||i.visible),o.textEnabled&&(o.textColor=i&&i.font&&i.font.color||null,i=A(!0,{},n._themeManager.theme().label,i),o.formatOptions={format:void 0!==i.format?i.format:n._defaultFormatOptions,customizeText:i.customizeText},o.textOptions={align:"center"},o.fontStyles=I(A({},n._themeManager.theme().label.font,i.font,{color:null})),n._textIndent=i.indent>0?T(i.indent):0,o.lineWidth=i.connectorWidth>0?T(i.connectorWidth):0,o.lineColor=i.connectorColor||null,e=n._renderer.text(x(n._translator,o.formatOptions),0,0).attr(o.textOptions).css(o.fontStyles).append(n._barsGroup),t=e.getBBox(),e.remove(),o.textY=t.y,o.textWidth=t.width,o.textHeight=t.height),f.default.prototype._applyMainLayout.call(n),n._renderBars()},_measureMainElements:function(){var e={maxRadius:this._area.radius};return this._context.textEnabled&&(e.horizontalMargin=this._context.textWidth,e.verticalMargin=this._context.textHeight,e.inverseHorizontalMargin=this._context.textWidth/2,e.inverseVerticalMargin=this._context.textHeight/2),e},_renderBars:function(){var e,t=this,n=A({},t._themeManager.theme(),t.option()),i=t._area,o=n.relativeInnerRadius>0&&n.relativeInnerRadius<1?T(n.relativeInnerRadius):.1;e=i.radius,t._context.textEnabled&&(t._textIndent=m(y(t._textIndent,e/2)),e-=t._textIndent),t._outerRadius=v(e),t._innerRadius=v(e*o),t._barSpacing=n.barSpacing>0?T(n.barSpacing):0,A(t._context,{backgroundColor:n.backgroundColor,x:i.x,y:i.y,startAngle:i.startCoord,endAngle:i.endCoord,baseAngle:t._translator.translate(t._baseValue)}),t._arrangeBars()},_arrangeBars:function(){var e,t=this,n=t._outerRadius-t._innerRadius,i=t._context,o=t._bars.length;t._beginValueChanging(),i.barSize=o>0?b((n-(o-1)*t._barSpacing)/o,1):0;var a=o>1?b(y((n-o*i.barSize)/(o-1),t._barSpacing),0):0,r=y(v((n+a)/i.barSize),o);t._setBarsCount(o),n=t._outerRadius,i.textRadius=n,i.textIndent=t._textIndent,t._palette.reset();var s=i.barSize+a,l=t._palette.generateColors(r);for(e=0;e<r;++e,n-=s)t._bars[e].arrange({radius:n,color:l[e]});for(var u=r;u<o;u++)t._bars[u].hide();t._animationSettings&&!t._noAnimation?t._animateBars():t._updateBars(),t._endValueChanging()},_setBarsCount:function(){var e=this;e._bars.length>0?e._dummyBackground&&(e._dummyBackground.dispose(),e._dummyBackground=null):(e._dummyBackground||(e._dummyBackground=e._renderer.arc().attr({"stroke-linejoin":"round"})),e._dummyBackground.attr({x:e._context.x,y:e._context.y,outerRadius:e._outerRadius,innerRadius:e._innerRadius,startAngle:e._context.endAngle,endAngle:e._context.startAngle,fill:e._context.backgroundColor}).append(e._barsGroup))},_updateBars:function(){this._bars.forEach(function(e){return e.applyValue()}),this._checkOverlap()},_checkOverlap:function(){var e=this,t=e._bars,n=(0,d.normalizeEnum)(e._getOption("resolveLabelOverlapping",!0));if("none"!==n)for(var i=t.concat().sort(function(e,t){return e.getValue()-t.getValue()}),o=0,a=1;o<i.length&&a<i.length;){var r=i[o],s=i[a];r.checkIntersect(s)?(s.hideLabel(),a++):(o=a,a=o+1)}},_animateBars:function(){var e,t=this,n=t._bars.length;if(n>0){for(e=0;e<n;++e)t._bars[e].beginAnimation();t._barsGroup.animate({_:0},t._animationSettings)}},_buildNodes:function(){var e=this,t=e._options.silent();e._palette=e._themeManager.createPalette(t.palette,{useHighlight:!0,extensionMode:t.paletteExtensionMode}),e._palette.reset(),e._bars=e._bars||[],e._animationSettings&&e._barsGroup.stopAnimation();var n=e._values.filter(O),i=n.length;if(e._bars.length>i){for(var o=e._bars.length,a=i;a<o;++a)e._bars[a].dispose();e._bars.splice(i,o-i)}else if(e._bars.length<i)for(var s=e._bars.length;s<i;++s)e._bars.push(new r(s,e._context));e._bars.forEach(function(t,o){t.update({color:e._palette.getNextColor(i),value:n[o]})})},_updateValues:function(e){var t,n,i=this,o=k(e)&&e||O(e)&&[e]||[],a=o.length;for(i._values.length=a,t=0;t<a;++t)n=o[t],i._values[t]=n=T(O(n)?n:i._values[t]);i._resizing||C(i._values,i.option(P))||i.option(P,i._values.slice()),this._change(["NODES"])},values:function(e){return void 0!==e?(this._updateValues(e),this):this._values.slice(0)},_optionChangesMap:{backgroundColor:"MOSTLY_TOTAL",relativeInnerRadius:"MOSTLY_TOTAL",barSpacing:"MOSTLY_TOTAL",label:"MOSTLY_TOTAL",resolveLabelOverlapping:"MOSTLY_TOTAL",palette:"MOSTLY_TOTAL",paletteExtensionMode:"MOSTLY_TOTAL",values:"VALUES"},_change_VALUES:function(){this._updateValues(this.option(P))},_factory:(0,l.clone)(h.dxBaseGauge.prototype._factory),_optionChangesOrder:["VALUES","NODES"],_initialChanges:["VALUES"],_change_NODES:function(){this._buildNodes()},_change_MOSTLY_TOTAL:function(){this._change(["NODES"]),this.callBase()},_proxyData:[],_getLegendData:function(){var e=this,t={},n=e._options.silent(),i=(n.label||{}).format,o=(n.legend||{}).itemTextFormat;return o?t.format=o:t.format=i||e._defaultFormatOptions,(this._bars||[]).map(function(e){return{id:e.index,item:{value:e.getValue(),color:e.getColor(),index:e.index},text:w(e.getValue(),t),visible:!0,states:{normal:{fill:e.getColor()}}}})}});t.dxBarGauge=R,r=function(e,t){var n=this;n._context=t,n._tracker=t.renderer.arc().attr({"stroke-linejoin":"round"}),n.index=e},A(r.prototype,{dispose:function(){var e=this;return e._background.dispose(),e._bar.dispose(),e._context.textEnabled&&(e._line.dispose(),e._text.dispose()),e._context.tracker.detach(e._tracker),e._context=e._settings=e._background=e._bar=e._line=e._text=e._tracker=null,e},arrange:function(e){var t=this,n=t._context;return this._visible=!0,n.tracker.attach(t._tracker,t,{index:t.index}),t._background=n.renderer.arc().attr({"stroke-linejoin":"round",fill:n.backgroundColor}).append(n.group),t._settings=t._settings||{x:n.x,y:n.y,startAngle:n.baseAngle,endAngle:n.baseAngle},t._bar=n.renderer.arc().attr(A({"stroke-linejoin":"round"},t._settings)).append(n.group),n.textEnabled&&(t._line=n.renderer.path([],"line").attr({"stroke-width":n.lineWidth}).append(n.group),t._text=n.renderer.text().css(n.fontStyles).attr(n.textOptions).append(n.group)),t._angle=isFinite(t._angle)?t._angle:n.baseAngle,t._settings.outerRadius=e.radius,t._settings.innerRadius=e.radius-n.barSize,t._settings.x=n.x,t._settings.y=n.y,t._background.attr(A({},t._settings,{startAngle:n.endAngle,endAngle:n.startAngle,fill:t._context.backgroundColor})),t._bar.attr({x:n.x,y:n.y,outerRadius:t._settings.outerRadius,innerRadius:t._settings.innerRadius,fill:t._color}),t._tracker.attr(t._settings),n.textEnabled&&(t._line.attr({points:[n.x,n.y-t._settings.innerRadius,n.x,n.y-n.textRadius-n.textIndent],stroke:n.lineColor||t._color}).sharp(),t._text.css({fill:n.textColor||t._color})),t},getTooltipParameters:function(){var e=this,t=D((e._angle+e._context.baseAngle)/2);return{x:m(e._context.x+(e._settings.outerRadius+e._settings.innerRadius)/2*t.cos),y:m(e._context.y-(e._settings.outerRadius+e._settings.innerRadius)/2*t.sin),offset:0,color:e._color,value:e._value}},setAngle:function(e){var t,n=this,i=n._context,a=n._settings;if(n._angle=e,o(a,i.baseAngle,e),n._bar.attr(a),n._tracker.attr(a),i.textEnabled){t=D(e);var r=i.textIndent,s=i.textRadius+r,l=i.x+s*t.cos,u=i.y-s*t.sin,c=.5*i.textWidth,d=i.textHeight,h=i.textY;_(l-i.x)>r&&(l+=l<i.x?-c:c),u-=_(u-i.y)<=r?h+.5*d:u<i.y?h+d:h;var f=w(n._value,i.formatOptions,{index:n.index}),p=""===f?"hidden":null;n._text.attr({text:f,x:l,y:u,visibility:p}),n._line.attr({visibility:p}),n._line.rotate(S(e),i.x,i.y)}return n},hideLabel:function(){this._text.attr({visibility:"hidden"}),this._line.attr({visibility:"hidden"})},checkIntersect:function(e){var t=this.calculateLabelCoords(),n=e.calculateLabelCoords();if(!t||!n)return!1;var i=Math.max(0,Math.min(t.bottomRight.x,n.bottomRight.x)-Math.max(t.topLeft.x,n.topLeft.x)),o=Math.max(0,Math.min(t.bottomRight.y,n.bottomRight.y)-Math.max(t.topLeft.y,n.topLeft.y));return i*o!==0},calculateLabelCoords:function(){if(this._text){var e=this._text.getBBox();return{topLeft:{x:e.x,y:e.y},bottomRight:{x:e.x+e.width,y:e.y+e.height}}}},_processValue:function(e){return this._context.translator.translate(this._context.translator.adjust(e))},applyValue:function(){return this._visible?this.setAngle(this._processValue(this.getValue())):this},update:function(e){var t=e.color,n=e.value;this._color=t,this._value=n},hide:function(){this._visible=!1},getColor:function(){return this._color},getValue:function(){return this._value},beginAnimation:function(){if(!this._visible)return this;var e=this,t=this._processValue(this.getValue());a(e._angle,t)?(e.animate=E,e.setAngle(e._angle)):(e._start=e._angle,e._delta=t-e._angle,e._tracker.attr({visibility:"hidden"}),e._context.textEnabled&&(e._line.attr({visibility:"hidden"}),e._text.attr({visibility:"hidden"})))},animate:function(e){if(!this._visible)return this;var t=this;t._angle=t._start+t._delta*e,o(t._settings,t._context.baseAngle,t._angle),t._bar.attr(t._settings)},endAnimation:function(){var e=this;void 0!==e._delta?a(e._angle,e._start+e._delta)&&(e._tracker.attr({visibility:null}),e.setAngle(e._angle)):delete e.animate,delete e._start,delete e._delta}}),(0,s.default)("dxBarGauge",R),R.addPlugin(p.plugin)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(956)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){var i=void 0===t?j.consts.emptySliderMarkerText:(0,j.formatValue)(t,n),o=y(e,i,n.font);return ee(o.height)+2*n.paddingTopBottom+j.consts.pointerSize}function a(e,t,n,i){var o=(0,j.formatValue)(t,n.label,i,n.valueType,n.type,n.logarithmBase),a=y(e,o,n.label.font);return ee(a.width/2)}function r(e,t,n,i,r){var s,l,u,c,d,h,f,p=0,g=0,_="semidiscrete"===t.type?t.customTicks:r.ticks;return i=i||{},u=i.left,c=i.right,d=n.placeholderHeight,n.visible&&(s=o(e,t.startValue,n),l=o(e,t.endValue,n),void 0===d&&(d=J(s,l))),t.label.visible&&(h=(0,P.isDefined)(t.startValue)?_[0]:void 0,f=(0,P.isDefined)(t.endValue)?_[_.length-1]:void 0,p=a(e,h,t,r),g=a(e,f,t,r)),u=void 0!==u?u:p,c=(void 0!==c?c:g)||1,{left:u,right:c,top:d||0,bottom:0}}function s(e,t){var n=(0,P.type)(e),i=(0,P.type)(t),o=function(e){return n===e||i===e};return o("date")?oe:o("number")?"numeric":o(le)?le:""}function l(e){return e.valueType===oe&&e.marker.visible}function u(e,t){var n=t.minorTickInterval||t.tickInterval;"datetime"===t.valueType&&(n=V.default.dateToMilliseconds(n)),e.addRange({interval:n})}function c(e,t,n){if(e){var i=e.logarithmBase;e.type===fe&&i<=0||i&&!(0,P.isNumeric)(i)?(e.logarithmBase=t,n("E2104")):e.type!==fe&&(e.logarithmBase=void 0)}}function d(e,t,n,i){var o=t.label,a=t.marker,r=t.placeholderHeight,s="semidiscrete"===t.type?t.customTicks:i.ticks,l=(0,j.formatValue)(s[0],o);return r?r:(o.visible?o.topIndent+y(e,l,o.font).height:0)+(n?a.topIndent+a.separatorHeight:0)}function h(e,t,n){var i=V.default.getDateUnitInterval(t),o=V.default.getDateUnitInterval(e),a=V.default.dateUnitIntervals.indexOf(i);return n&&i===o&&a>0&&(i=V.default.dateUnitIntervals[a-1]),i}function f(e,t,n){return e?n?e=V.default.getNextDateUnit(e):e+=t:e=t,e}function p(e,t,n,i){var o,a,r,s=e.minorTickInterval,l=e.tickInterval,u="datetime"===e.valueType,c=e.axisDivisionFactor||{};if(!l)do{if(o=f(l,s,u),l===o)break;l=o,u&&(o=V.default.dateToMilliseconds(l)),a=ee((n-t)/o),r=te(i/(c[l]||ve))||1}while(o&&a>r);return{tickInterval:l,minorTickInterval:s,bounds:{minVisible:t,maxVisible:n},ticks:[]}}function g(e,t,n,i){var o,a,r,s=(0,P.isDefined)(i.minVisible)?i.minVisible:i.min,l=(0,P.isDefined)(i.maxVisible)?i.maxVisible:i.max,u=e._categoriesInfo,c={};return e.type===se?o=p(e,s,l,t):(a=(0,K.tickGenerator)({axisType:e.type,dataType:e.valueType,logBase:e.logarithmBase,allowNegatives:!0,linearThreshold:Math.abs(e.linearThreshold||0),axisDivisionFactor:e.axisDivisionFactor,minorAxisDivisionFactor:e.minorAxisDivisionFactor,calculateMinors:!0,allowDecimals:e.allowDecimals,endOnTick:e.endOnTick,incidentOccurred:n,rangeIsEmpty:i.isEmpty()})({min:s,max:l,categories:(0,P.isDefined)(u)?u.categories:[]},t,e.tickInterval,e.forceUserTickInterval,void 0,e.minorTickInterval,e.minorTickCount),r=a.ticks.length,c.minVisible=a.ticks[0]<s?a.ticks[0]:s,c.maxVisible=a.ticks[r-1]>l?a.ticks[r-1]:l,o={tickInterval:a.tickInterval,minorTickInterval:0===e.minorTickInterval?0:a.minorTickInterval,bounds:c,ticks:a.ticks}),o}function _(e){var t;return null===(t=e.workWeek)||void 0===t?void 0:t[0]}function m(e,t){var n,i,o,a,r,s=!1,l=t.startValue,u=t.endValue,c=e?e.getBoundRange().arg:new L.Range,d="datetime"===t.valueType,h=_(t),f=t.minRange;return t.type===re&&(r=new L.Range({minVisible:l,maxVisible:u}),r.addRange(c),c=r,o=e?e.argCategories:t.categories||!e&&l&&u&&[l,u],o=o||[],t._categoriesInfo=a=(0,B.getCategoriesInfo)(o,l,u)),t.type===se&&(l=t.startValue=C(t.startValue,d,f,h),u=t.endValue=C(t.endValue,d,f,h),c.minVisible=C(c.minVisible,d,f,h),c.maxVisible=C(c.maxVisible,d,f,h),c.min=C(c.min,d,f,h),c.max=C(c.max,d,f,h)),(0,P.isDefined)(l)&&(0,P.isDefined)(u)?(s=a?a.inverted:l>u,n=a?a.start:s?u:l,i=a?a.end:s?l:u):(0,P.isDefined)(l)||(0,P.isDefined)(u)?(n=l,i=u):a&&(n=a.start,i=a.end),c.addRange({invert:s,min:n,max:i,minVisible:n,
maxVisible:i,dataType:t.valueType}),c.addRange({categories:e?void 0:o,base:t.logarithmBase,axisType:t.type,dataType:t.valueType}),e&&c.sortCategories(o),c}function v(e,t){return!(0,P.isDefined)(e)||!(0,P.isDefined)(t)}function y(e,t,n){var i=e.text(t,me,me).css((0,B.patchFontOptions)(n)).append(e.root),o=i.getBBox();return i.remove(),o}function b(e){return function(t,n,i,o,a){return n&&t&&(!(0,P.isDefined)(a)||a.years||a.months>=6||e/ve<ee((o-i)/V.default.dateToMilliseconds("year"))+1)&&(n=!1),n}}function x(e,t,n,i,o){var a,r,s,l=e._categoriesInfo,u=e.valueType===oe;!t||t.isEmpty()||n.isEmpty()||(a=i.bounds,n.addRange(a),e.startValue=n.invert?a.maxVisible:a.minVisible,e.endValue=n.invert?a.minVisible:a.maxVisible),e.marker.visible=o(u&&e.type.indexOf(re)===-1,e.marker.visible,e.startValue,e.endValue,i.tickInterval),l&&(e.startValue=l.start,e.endValue=l.end),e.type.indexOf(re)===-1&&(r=(0,P.isDate)(e.startValue)&&(0,P.isDate)(e.endValue)&&e.startValue.getTime()===e.endValue.getTime()||e.startValue===e.endValue),e.isEmpty=v(e.startValue,e.endValue)||r,e.isEmpty?e.startValue=e.endValue=void 0:(e.minorTickInterval=i.minorTickInterval,e.tickInterval=i.tickInterval,u&&(!(0,P.isDefined)(e.label.format)||e.type===se&&e.minorTickInterval!==e.tickInterval)&&(e.type===re?e.label.format=N.default.getDateFormatByTicks(i.ticks):e.marker.visible?e.label.format=V.default.getDateFormatByTickInterval(e.tickInterval):e.label.format=N.default.getDateFormatByTickInterval(e.startValue,e.endValue,e.tickInterval))),e.type===se&&(s=k(e),e.customMinorTicks=s.altIntervals,e.customTicks=s.intervals,e.customBoundTicks=[e.customTicks[0]])}function w(e,t,n,i){var o=0,a=(0,z.correctValueType)((0,B.normalizeEnum)(e.valueType)),r=function(t,i){var a=t===ne?"start":"end";(0,P.isDefined)(e[t])&&(o=i(e[t]),(0,P.isDefined)(o)?e[t]=o:(e[t]=void 0,n("E2202",[a])))};a=t||a,a||(a=s(e.startValue,e.endValue)||"numeric"),(a===le||e.categories)&&(e.type=re,a=le),e.containerColor=i,e.valueType=a,e.dataType=a;var l=(0,z.getParser)(a);return r(ne,l),r(ie,l),c(e,xe,n),e.type||(e.type="continuous"),e.parser=l,e.type===se&&(e.minorTick.visible=!1,e.minorTickInterval=e.minRange,e.marker.visible=!1,e.maxRange=void 0),e.forceUserTickInterval|=(0,P.isDefined)(e.tickInterval)&&!(0,P.isDefined)(e.axisDivisionFactor),e.axisDivisionFactor=(0,P.isDefined)(e.axisDivisionFactor)?e.axisDivisionFactor:ye,e.minorAxisDivisionFactor=(0,P.isDefined)(e.minorAxisDivisionFactor)?e.minorAxisDivisionFactor:be,e}function C(e,t,n,i){return(0,P.isDefined)(e)&&(e=t?V.default.correctDateWithUnitBeginning(new Date(e),n,null,i):(0,F.adjust)(te((0,F.adjust)(e/n))*n)),e}function k(e){var t=e.startValue,n=e.endValue,i="datetime"===e.valueType,o=_(e),a=e.tickInterval,r={intervals:[]};return(0,P.isDefined)(t)&&(0,P.isDefined)(n)?(r.intervals=V.default.getSequenceByInterval(t,n,e.minorTickInterval),a!==e.minorTickInterval&&(r.altIntervals=r.intervals,t=C(t,i,a,o),n=C(n,i,a,o),r.intervals=V.default.getSequenceByInterval(t,n,a),r.intervals[0]=r.altIntervals[0]),r):r}function S(e,t,n){var i=Math.abs(t-e)/n,o=i-te(i);return o>0?ee(Math.abs((0,F.adjust)((0,B.getLog)(o,10)))):0}function D(e,t,n,i){return e.marker.label.font=e.label.font,e.color=e.marker.color=e.tick.color,e.opacity=e.marker.opacity=e.tick.opacity,e.width=e.marker.width=e.tick.width,e.placeholderSize=(e.placeholderHeight||0)+i,e.argumentType=e.valueType,e.visible=t,e.isHorizontal=!0,e.calculateMinors=!0,e.semiDiscreteInterval=e.minRange,t||(e.minorTick.length=e.tick.length=n),e.label.indentFromAxis=e.label.topIndent+i,e}function I(e,t,n){function i(t){var i=t.target.range,o=e.minRange?V.default.addInterval(i.startValue,e.minRange):void 0,a=e.maxRange?V.default.addInterval(i.startValue,e.maxRange):void 0;o&&o>i.endValue||a&&a<i.endValue||n(i,t)}(0,M.each)(t,function(e,t){t.on("dxpointerdown",i)})}function T(){return 1}function O(e){return e%2===1?-te(e/2):-e/2}function E(e){var t=this;t._axis=new H.Axis({renderer:e.renderer,axesContainerGroup:e.root,scaleBreaksGroup:e.scaleBreaksGroup,incidentOccurred:e.incidentOccurred,axisType:"xyAxes",drawingType:"linear",widgetClass:"dxrs",axisClass:"range-selector",isArgumentAxis:!0,getTemplate:function(){}}),t._updateSelectedRangeCallback=e.updateSelectedRange,t._axis.getAxisSharpDirection=t._axis.getSharpDirectionByCoords=T,t._axis.getTickStartPositionShift=O}t.default=void 0;var A=i(n(9)),P=n(1),R=n(0),M=n(3),B=n(14),V=i(n(20)),F=n(29),L=n(134),H=n(232),z=n(325),N=i(n(79)),j=n(234),W=n(957),G=n(960),q=n(961),$=n(962),K=n(552),U=i(n(112)),Y=n(111),Z=n(131),X=n(153),Q=n(174),J=Math.max,ee=Math.ceil,te=Math.floor,ne="startValue",ie="endValue",oe="datetime",ae="value",re="discrete",se="semidiscrete",le="string",ue=ae+"Changed",ce="containerBackgroundColor",de="sliderMarker",he="background",fe="logarithmic",pe="keep",ge="shift",_e="reset",me=-1e3,ve=50,ye=30,be=15,xe=10,we=U.default.inherit({_toggleParentsScrollSubscription:function(){},_eventsMap:{onValueChanged:{name:ue}},_rootClassPrefix:"dxrs",_rootClass:"dxrs-range-selector",_dataIsReady:function(){return this._dataIsLoaded()},_initialChanges:["DATA_SOURCE","VALUE"],_themeDependentChanges:["MOSTLY_TOTAL"],_themeSection:"rangeSelector",_fontFields:["scale.label.font","sliderMarker.font"],_initCore:function(){var e=this,t=e._renderer,n=t.root;n.css({"touch-action":"pan-y"}),e._clipRect=t.clipRect();var i=t.g().attr({"class":"dxrs-view"}).append(n),o=t.g().attr({"class":"dxrs-slidersContainer","clip-path":e._clipRect.id}).append(n),a=t.g().attr({"class":"dxrs-scale","clip-path":e._clipRect.id}).append(n),r=t.g().attr({"class":"dxrs-scale-breaks"}).append(n),s=t.g().attr({"class":"dxrs-trackers"}).append(n);e._axis=new E({renderer:t,root:a,scaleBreaksGroup:r,updateSelectedRange:function(t,n){e.setValue((0,B.convertVisualRangeObject)(t),n)},incidentOccurred:e._incidentOccurred}),e._rangeView=new q.RangeView({renderer:t,root:i,translator:e._axis.getTranslator()}),e._slidersController=new W.SlidersController({renderer:t,root:o,trackersGroup:s,updateSelectedRange:function(t,n,i){e._rangeOption||e.option(ae,(0,B.convertVisualRangeObject)(t,(0,P.isPlainObject)(e._options.silent(ae)))),e._eventTrigger(ue,{value:(0,B.convertVisualRangeObject)(t),previousValue:(0,B.convertVisualRangeObject)(n),event:i})},axis:e._axis,translator:e._axis.getTranslator()}),e._tracker=new G.Tracker({renderer:t,controller:e._slidersController})},_getDefaultSize:function(){return{width:400,height:160}},_disposeCore:function(){this._axis.dispose(),this._slidersController.dispose(),this._tracker.dispose()},_applySize:function(e){this._clientRect=e.slice(),this._change(["MOSTLY_TOTAL"])},_optionChangesMap:{scale:"SCALE",value:"VALUE",dataSource:"DATA_SOURCE"},_optionChangesOrder:["SCALE","DATA_SOURCE"],_change_SCALE:function(){this._change(["MOSTLY_TOTAL"])},_setValueByDataSource:function(){var e=this,t=e._options.silent(),n=e._axis;if(t.dataSource){var i=e.option("selectedRangeUpdateMode"),o=e.getValue(),a=(0,P.isDefined)(o[0])&&(0,P.isDefined)(o[1]);if((0,P.isDefined)(i)?(i=(0,B.normalizeEnum)(i),e.__skipAnimation=!0):a&&!e._dataSourceIsAsync&&(i=_e),"auto"===i&&a){var r=n.allScaleSelected(o);i=r.startValue&&r.endValue?_e:r.endValue?ge:pe}if(i===_e)t[ae]=null;else if(i===ge&&a){var s=e.getValue();e.__skipAnimation=!0,t[ae]={length:n.getVisualRangeLength({minVisible:s[0],maxVisible:s[1]})}}else i===pe&&(e.__skipAnimation=!0)}e._dataSourceIsAsync=void 0},_change_DATA_SOURCE:function(){this._options.silent("dataSource")&&this._updateDataSource()},_customChangesOrder:["MOSTLY_TOTAL","VALUE","SLIDER_SELECTION"],_change_MOSTLY_TOTAL:function(){this._applyMostlyTotalChange()},_change_SLIDER_SELECTION:function(){var e=this,t=e._options.silent(ae);e._slidersController.setSelectedRange(t&&(0,B.getVizRangeObject)(t))},_change_VALUE:function(){var e=this,t=e._rangeOption;e._dataSourceIsAsync=!e._dataIsReady(),t&&(e._options.silent(ae,t),e.setValue(t))},_validateRange:function(e,t){var n=this,i=n._axis.getTranslator();((0,P.isDefined)(e)&&!i.isValid(e)||(0,P.isDefined)(t)&&!i.isValid(t))&&n._incidentOccurred("E2203")},_applyChanges:function(){var e=this,t=e._options.silent(ae);e._changes.has("VALUE")&&t&&(e._rangeOption=t),e.callBase.apply(e,arguments),e._rangeOption=null,e.__isResizing=e.__skipAnimation=!1},_applyMostlyTotalChange:function(){var e,t=this,n=t._renderer,i=t._clientRect,o={left:i[0],top:i[1],width:i[2]-i[0],height:i[3]-i[1]};(t.__isResizing||t.__skipAnimation)&&(e=n.animationEnabled(),n.updateAnimationOptions({enabled:!1})),t._clipRect.attr({x:i[0],y:i[1],width:i[2]-i[0],height:i[3]-i[1]}),t._axis.getTranslator().update(new L.Range,o,{isHorizontal:!0}),t._updateContent({left:i[0],top:i[1],width:i[2]-i[0],height:i[3]-i[1]}),(t.__isResizing||t.__skipAnimation)&&n.updateAnimationOptions({enabled:e}),t._drawn()},_dataSourceChangedHandler:function(){this._setValueByDataSource(),this._requestChange(["MOSTLY_TOTAL"])},_completeSeriesDataSourceCreation:function(e,t){var n=this._clientRect,i={left:n[0],top:n[1],width:n[2]-n[0],height:n[3]-n[1]};this._axis.updateOptions((0,R.extend)({},e,{isHorizontal:!0,label:{}})),t.isShowChart()&&this._axis.setMarginOptions(t.getMarginOptions(i)),this._axis.updateCanvas(i),t.createPoints()},_updateContent:function(e){var t=this,n=t.option("chart"),i=t._createSeriesDataSource(n),o=!(i&&i.isShowChart()||t.option("background.image.url")),a=w(t._getOption("scale"),i&&i.getCalculatedValueType(),t._incidentOccurred,this._getOption("containerBackgroundColor",!0));i&&t._completeSeriesDataSourceCreation(a,i);var s=m(i,a),h=g(a,e.width,t._incidentOccurred,s),f=i&&i.isShowChart()&&i.getThemeManager();f&&c(n&&n.valueAxis,f.getOptions("valueAxis").logarithmBase,t._incidentOccurred),x(a,i,s,h,b(e.width)),u(s,a);var p=t._prepareSliderMarkersOptions(a,e.width,h,s),_=r(t._renderer,a,p,t.option("indent"),h),v={left:e.left+_.left,top:e.top+_.top,width:e.left+_.left+J(e.width-_.left-_.right,1),height:J(o?j.HEIGHT_COMPACT_MODE:e.height-_.top-_.bottom-d(t._renderer,a,l(a),h),0),right:0,bottom:0};t._axis.update(a,o,v,s,i),a.minorTickInterval=a.isEmpty?0:a.minorTickInterval,t._updateElements(a,p,o,v,i),f&&f.dispose()},_updateElements:function(e,t,n,i,o){var a=this,r=a._getOption("behavior"),s=a._getOption("shutter"),l=e.type!==se;s.color=s.color||a._getOption(ce,!0),a._rangeView.update(a.option("background"),a._themeManager.theme("background"),i,n,r.animationEnabled&&a._renderer.animationEnabled(),o),a._isUpdating=!0,a._slidersController.update([i.top,i.top+i.height],r,n,a._getOption("sliderHandle"),t,s,{minRange:l?a.option("scale.minRange"):void 0,maxRange:l?a.option("scale.maxRange"):void 0},a._axis.getFullTicks(),a._getOption("selectedRangeColor",!0)),a._requestChange(["SLIDER_SELECTION"]),a._isUpdating=!1,a._tracker.update(!a._axis.getTranslator().getBusinessRange().isEmpty(),r)},_createSeriesDataSource:function(e){var t,n=this,i=n._dataSourceItems(),o=n._getOption("scale"),a=o.valueType||s(o.startValue,o.endValue),r=new H.Axis({renderer:n._renderer,axisType:"xyAxes",drawingType:"linear"});return r.updateOptions({isHorizontal:!1,label:{},categoriesSortingMethod:n._getOption("chart").valueAxis.categoriesSortingMethod}),(i||e&&e.series)&&(e=(0,R.extend)({},e,{theme:n.option("theme")}),t=new $.SeriesDataSource({renderer:n._renderer,dataSource:i,valueType:(0,B.normalizeEnum)(a),axisType:o.type,chart:e,dataSourceField:n.option("dataSourceField"),incidentOccurred:n._incidentOccurred,categories:o.categories,argumentAxis:n._axis,valueAxis:r})),t},_prepareSliderMarkersOptions:function(e,t,n,i){var o=this,a=n.minorTickInterval,r=n.tickInterval,s=r,l=e.endValue,u=e.startValue,c=o._getOption(de),d=!o._getOption("behavior").snapToTicks,f=e.type===re,p=e.valueType===oe;return c.borderColor=o._getOption(ce,!0),c.format||i.isEmpty()||(d&&(0,P.isNumeric)(e.startValue)&&(c.format={type:"fixedPoint",precision:S(u,l,t)}),p&&!f&&((0,P.isDefined)(a)&&0!==a&&(s=h(r,a,d)),e.marker.visible?c.format=V.default.getDateFormatByTickInterval(s):(0,P.isDefined)(u)&&(0,P.isDefined)(l)&&(c.format=N.default.getDateFormatByTickInterval(u,l,s))),p&&f&&n.ticks.length&&(c.format=N.default.getDateFormatByTicks(n.ticks))),c},getValue:function(){return(0,B.convertVisualRangeObject)(this._slidersController.getSelectedRange())},setValue:function(e,t){var n=(0,B.getVizRangeObject)(e);!this._isUpdating&&e&&(this._validateRange(n.startValue,n.endValue),!(0,B.rangesAreEqual)(n,this._slidersController.getSelectedRange())&&this._slidersController.setSelectedRange(n,t))},_setContentSize:function(){this.__isResizing=2===this._changes.count(),this.callBase.apply(this,arguments)}});(0,M.each)(["selectedRangeColor","containerBackgroundColor","sliderMarker","sliderHandle","shutter",he,"behavior","chart","indent"],function(e,t){we.prototype._optionChangesMap[t]="MOSTLY_TOTAL"}),E.prototype={constructor:E,dispose:function(){this._axis.dispose()},calculateInterval:function(e,t){return this._axis.calculateInterval(e,t)},update:function(e,t,n,i,o){var a=this._axis;a.updateOptions(D(e,t,n.height,n.height/2-ee(e.width/2))),a.validate(),a.setBusinessRange(i,!0),void 0!==o&&o.isShowChart()&&a.setMarginOptions(o.getMarginOptions(n)),a.draw(n),a.shift({left:0,bottom:-n.height/2+n.top}),a.getMarkerTrackers()&&I(e,a.getMarkerTrackers(),this._updateSelectedRangeCallback),a.drawScaleBreaks({start:n.top,end:n.top+n.height})},visualRange:function(){},getViewport:function(){return{}},allScaleSelected:function(e){var t=this._axis.visualRange(),n=t.startValue,i=t.endValue;return{startValue:e[0].valueOf()===n.valueOf(),endValue:e[1].valueOf()===i.valueOf()}},getOptions:function(){return this._axis.getOptions()||{}}},["setMarginOptions","getFullTicks","updateCanvas","updateOptions","getAggregationInfo","getTranslator","getVisualRangeLength","getVisibleArea","getMarginOptions"].forEach(function(e){E.prototype[e]=function(){var t=this._axis;return t[e].apply(t,arguments)}}),(0,A.default)("dxRangeSelector",we);var Ce=we;t.default=Ce,we.addPlugin(Y.plugin),we.addPlugin(Z.plugin),we.addPlugin(X.plugin),we.addPlugin(Q.plugin),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n,i){return[e,t,n,t,n,i,e,i]}function a(e,t){return e<t}function r(e,t){return e>t}function s(e,t){for(var n,i=0,o=t?t.length-1:0,a=e;o-i>1;){if(n=i+o>>1,a=t[n],a===e)return e;e<a?o=n:i=n}return t&&(a=t[e-t[i]<=t[o]-e?i:o]),a}function l(){this._lastSelectedRange=this.getSelectedRange(),delete this._processSelectionChanged}function u(e){e.setSelectedRange=h.noop,e._processSelectionChanged===l&&e._processSelectionChanged()}function c(e){delete e.setSelectedRange}function d(e){var t=this,n={renderer:e.renderer,root:e.root,trackersGroup:e.trackersGroup,translator:e.translator};t._params=e,t._areaTracker=e.renderer.path(null,"area").attr({"class":"area-tracker",fill:"#000000",opacity:1e-4}).append(e.trackersGroup),t._selectedAreaTracker=e.renderer.path(null,"area").attr({"class":"selected-area-tracker",fill:"#000000",opacity:1e-4}).append(e.trackersGroup),t._shutter=e.renderer.path(null,"area").append(e.root),t._sliders=[new p.default(n,0),new p.default(n,1)],t._processSelectionChanged=l}t.SlidersController=d;var h=n(4),f=n(234),p=i(n(958)),g=n(14),_=n(1),m=n(29),v=f.utils.animationSettings,y=f.consts.emptySliderMarkerText;d.prototype={constructor:d,dispose:function(){this._sliders[0].dispose(),this._sliders[1].dispose()},getTrackerTargets:function(){return{area:this._areaTracker,selectedArea:this._selectedAreaTracker,sliders:this._sliders}},_processSelectionChanged:function(e){var t=this,n=t.getSelectedRange();(0,g.rangesAreEqual)(n,t._lastSelectedRange)||(t._params.updateSelectedRange(n,t._lastSelectedRange,e),t._lastSelectedRange=n)},update:function(e,t,n,i,a,r,s,l,u){var c=this,d=c._params.translator.getScreenRange();c._verticalRange=e,c._minRange=s.minRange,c._maxRange=s.maxRange,c._animationEnabled=t.animationEnabled&&c._params.renderer.animationEnabled(),c._allowSlidersSwap=t.allowSlidersSwap,c._sliders[0].update(e,i,a),c._sliders[1].update(e,i,a),c._sliders[0]._position=c._sliders[1]._position=d[0],c._values=!c._params.translator.isValueProlonged&&t.snapToTicks?l:null,c._areaTracker.attr({points:o(d[0],e[0],d[1],e[1])}),c._isCompactMode=n,c._shutterOffset=i.width/2,c._updateSelectedView(r,u),c._isOnMoving="onmoving"===(0,g.normalizeEnum)(t.callValueChanged),c._updateSelectedRange(),c._applyTotalPosition(!1)},_updateSelectedView:function(e,t){var n={fill:null,"fill-opacity":null,stroke:null,"stroke-width":null};this._isCompactMode?(n.stroke=t,n["stroke-width"]=3,n.sharp="v"):(n.fill=e.color,n["fill-opacity"]=e.opacity),this._shutter.attr(n)},_updateSelectedRange:function(){var e=this,t=e._sliders;t[0].cancelAnimation(),t[1].cancelAnimation(),e._shutter.stopAnimation(),e._params.translator.getBusinessRange().isEmpty()?(t[0]._setText(y),t[1]._setText(y),t[0]._value=t[1]._value=void 0,t[0]._position=e._params.translator.getScreenRange()[0],t[1]._position=e._params.translator.getScreenRange()[1],e._applyTotalPosition(!1),u(e)):c(e)},_applyTotalPosition:function(e){var t=this._sliders;e=this._animationEnabled&&e,t[0].applyPosition(e),t[1].applyPosition(e);var n=t[0].getCloudBorder()>t[1].getCloudBorder();t[0].setOverlapped(n),t[1].setOverlapped(n),this._applyAreaTrackersPosition(),this._applySelectedRangePosition(e)},_applyAreaTrackersPosition:function(){var e=this,t=e._sliders[0].getPosition(),n=e._sliders[1].getPosition();e._selectedAreaTracker.attr({points:o(t,e._verticalRange[0],n,e._verticalRange[1])}).css({cursor:Math.abs(e._params.translator.getScreenRange()[1]-e._params.translator.getScreenRange()[0]-n+t)<.001?"default":"pointer"})},_applySelectedRangePosition:function(e){var t,n,i=this,a=i._verticalRange,r=i._sliders[0].getPosition(),s=i._sliders[1].getPosition();i._isCompactMode?n=[r+Math.ceil(i._shutterOffset),(a[0]+a[1])/2,s-Math.floor(i._shutterOffset),(a[0]+a[1])/2]:(t=i._params.axis.getVisibleArea(),n=[o(t[0],a[0],Math.max(r-Math.floor(i._shutterOffset),t[0]),a[1]),o(t[1],a[0],Math.min(s+Math.ceil(i._shutterOffset),t[1]),a[1])]),e?i._shutter.animate({points:n},v):i._shutter.attr({points:n})},getSelectedRange:function(){return{startValue:this._sliders[0].getValue(),endValue:this._sliders[1].getValue()}},setSelectedRange:function(e,t){e=e||{};var n=this,i=n._params.translator,o=i.getBusinessRange(),a="discrete"===o.axisType?function(e,t){return e<t}:function(e,t){return e<=t},r=(0,g.adjustVisualRange)({dataType:o.dataType,axisType:o.axisType,base:o.base},{startValue:i.isValid(e.startValue)?i.getCorrectValue(e.startValue,1):void 0,endValue:i.isValid(e.endValue)?i.getCorrectValue(e.endValue,-1):void 0,length:e.length},{min:o.minVisible,max:o.maxVisible,categories:o.categories}),s=r.startValue,l=r.endValue;s=(0,_.isNumeric)(s)?(0,m.adjust)(s):s,l=(0,_.isNumeric)(l)?(0,m.adjust)(l):l;var u=a(i.to(s,-1),i.to(l,1))?[s,l]:[l,s];n._sliders[0].setDisplayValue(u[0]),n._sliders[1].setDisplayValue(u[1]),n._sliders[0]._position=i.to(u[0],-1),n._sliders[1]._position=i.to(u[1],1),n._applyTotalPosition(!0),n._processSelectionChanged(t)},beginSelectedAreaMoving:function(e){function t(e,t){e!==a&&e>a==e>(i[0].getPosition()+i[1].getPosition())/2-o&&n._moveSelectedArea(e+o,!1,t),a=e}var n=this,i=n._sliders,o=(i[0].getPosition()+i[1].getPosition())/2-e,a=e;return t.complete=function(e){n._dockSelectedArea(e)},t},_dockSelectedArea:function(e){var t=this._params.translator,n=this._sliders;n[0]._position=t.to(n[0].getValue(),-1),n[1]._position=t.to(n[1].getValue(),1),this._applyTotalPosition(!0),this._processSelectionChanged(e)},moveSelectedArea:function(e,t){this._moveSelectedArea(e,!0,t),this._dockSelectedArea(t)},_moveSelectedArea:function(e,t,n){var i=this,o=i._params.translator,a=i._sliders,r=a[1].getPosition()-a[0].getPosition(),l=e-r/2,u=e+r/2;l<o.getScreenRange()[0]&&(l=o.getScreenRange()[0],u=l+r),u>o.getScreenRange()[1]&&(u=o.getScreenRange()[1],l=u-r);var c=s(o.from(l,-1),i._values);a[0].setDisplayValue(c),a[1].setDisplayValue(s(o.from(o.to(c,-1)+r,1),i._values)),a[0]._position=l,a[1]._position=u,i._applyTotalPosition(t),i._isOnMoving&&i._processSelectionChanged(n)},placeSliderAndBeginMoving:function(e,t,n){var i,o=this,l=o._params.translator,u=o._sliders,c=e<t?0:1,d=c>0?1:-1,h=c>0?r:a,f=c>0?a:r,p=[],g=[];g[c]=l.from(e,d),g[1-c]=l.from(t,-d),p[1-c]=t,l.isValueProlonged&&(h(e,l.to(g[c],d))&&(g[c]=l.from(e,-d)),h(t,l.to(g[c],-d))&&(g[1-c]=g[c])),o._minRange&&(i=l.to(l.add(s(g[c],o._values),o._minRange,-d),-d),h(t,i)&&(g[1-c]=l.add(g[c],o._minRange,-d)),i=l.to(l.add(l.getRange()[1-c],o._minRange,d),-d),f(e,i)&&(g[1-c]=l.getRange()[1-c],g[c]=l.add(g[1-c],o._minRange,d),p[1-c]=e)),g[0]=s(g[0],o._values),g[1]=s(g[1],o._values),p[c]=l.to(g[c],d),u[0].setDisplayValue(g[0]),u[1].setDisplayValue(g[1]),u[0]._position=p[0],u[1]._position=p[1],o._applyTotalPosition(!0),o._isOnMoving&&o._processSelectionChanged(n);var _=o.beginSliderMoving(1-c,t);return u[1-c]._sliderGroup.stopAnimation(),o._shutter.stopAnimation(),_(t),_},beginSliderMoving:function(e,t){function n(e,t){var n,a,r,b,x;if(e!==f&&(g(e+y,h)&&(n=i._allowSlidersSwap,n&&!o.isValueProlonged&&i._minRange&&(n=o.isValid(o.add(l[1-d].getValue(),i._minRange,-p))),n&&(i._changeMovingSlider(d),d=1-d,p=-p,a=g,g=m,m=a,v=-p*Math.abs(v),y=-v)),m(e+v,h))){n=!0,b=l[d],x=l[1-d].getValue(),r=Math.max(Math.min(e+v,c),u),n&&o.isValueProlonged&&(n=!g(r,o.to(x,p)));var w;n&&i._minRange&&(n=!g(r,o.to(o.add(x,i._minRange,p),p)),n||(w=o.add(x,i._minRange,p))),n&&i._maxRange&&(n=!m(r,o.to(o.add(x,i._maxRange,p),p)),n||(w=o.add(x,i._maxRange,p))),b._setValid(n),b.setDisplayValue(n?s(o.from(r,p),i._values):(0,_.isDefined)(w)?w:b.getValue()),b._position=r,i._applyTotalPosition(!1),b.toForeground(),i._isOnMoving&&i._processSelectionChanged(t)}f=e}var i=this,o=i._params.translator,l=i._sliders,u=o.getScreenRange()[0],c=o.getScreenRange()[1],d=e,h=l[1-d].getPosition(),f=t,p=d>0?1:-1,g=d>0?a:r,m=d>0?r:a,v=l[d].getPosition()-t,y=g(l[d].getPosition(),t)?-v:v;return n.complete=function(e){l[d]._setValid(!0),i._dockSelectedArea(e)},n},_changeMovingSlider:function(e){var t,n=this,i=n._params.translator,o=n._sliders,a=o[1-e].getPosition(),r=e>0?1:-1;o[e].setDisplayValue(s(i.from(a,r),n._values)),t=i.from(a,-r),i.isValueProlonged?t=i.from(a,r):n._minRange&&(t=i.add(t,n._minRange,-r)),o[1-e].setDisplayValue(s(t,n._values)),o[e]._setValid(!0),o[e]._marker._update(),o[0]._position=o[1]._position=a},foregroundSlider:function(e){this._sliders[e].toForeground()}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return l.touchEvents||l.pointerEvents?d:c<e?e:c}function a(e,t){var n=this;n._translator=e.translator,n._sliderGroup=e.renderer.g().attr({"class":"slider"}).append(e.root),n._line=e.renderer.path(null,"line").append(n._sliderGroup),n._marker=new s.default(e.renderer,n._sliderGroup,1===t),n._tracker=e.renderer.rect().attr({"class":"slider-tracker",fill:"#000000",opacity:1e-4}).css({cursor:"w-resize"}).append(e.trackersGroup)}t.default=void 0;var r=n(234),s=i(n(959)),l=n(46),u=r.utils.animationSettings,c=8,d=20;a.prototype={constructor:a,cancelAnimation:function(){this._sliderGroup.stopAnimation(),this._tracker.stopAnimation()},applyPosition:function(e){var t=this,n=t._sliderGroup,i=t._tracker,o={translateX:t._position};t._marker.setPosition(t._position),e?(n.animate(o,u),i.animate(o,u)):(n.attr(o),i.attr(o))},_setValid:function(e){this._marker.setValid(e),this._line.attr({stroke:this._colors[Number(e)]})},_setText:function(e){this._marker.setText(e)},update:function(e,t,n){var i=this;i._formatOptions={format:n.format,customizeText:n.customizeText},i._marker.applyOptions(n,i._translator.getScreenRange()),i._colors=[n.invalidRangeColor,t.color],i._sliderGroup.attr({translateY:e[0]}),i._line.attr({"stroke-width":t.width,stroke:t.color,"stroke-opacity":t.opacity,sharp:"h",points:[0,0,0,e[1]-e[0]]});var a=o(t.width);i._tracker.attr({x:-a/2,y:0,width:a,height:e[1]-e[0],translateY:e[0]})},toForeground:function(){this._sliderGroup.toForeground()},getSliderTracker:function(){return this._tracker},getPosition:function(){return this._position},setDisplayValue:function(e){this._value=e,this._setText((0,r.formatValue)(e,this._formatOptions))},setOverlapped:function(e){this._marker.setOverlapped(e)},getValue:function(){return this._value},on:function(e,t){this._tracker.on(e,t),this._marker.getTracker().on(e,t)},getCloudBorder:function(){return this._marker.getBorderPosition()},dispose:function(){this._marker.dispose()}};var h=a;t.default=h,e.exports=t.default},function(e,t,n){function i(e,t,n){var i=this;i._isLeftPointer=n,i._isOverlapped=!1,i._group=e.g().attr({"class":"slider-marker"}).append(t),i._area=e.path(null,"area").append(i._group),i._label=e.text().attr({align:"left"}).append(i._group),i._tracker=e.rect().attr({"class":"slider-marker-tracker",fill:"#000000",opacity:1e-4}).css({cursor:"pointer"}).append(i._group),i._border=e.rect(0,0,1,0)}t.default=void 0;var o=n(14),a=n(234),r=a.consts.pointerSize,s=75;i.prototype={constructor:i,_getRectSize:function(e){return{width:Math.round(2*this._paddingLeftRight+e.width),height:Math.round(2*this._paddingTopBottom+e.height)}},_getTextSize:function(){var e=this._label.getBBox();return!this._textHeight&&isFinite(e.height)&&(this._textHeight=e.height),{width:e.width,height:this._textHeight,y:e.y}},_getAreaPointsInfo:function(e){var t=this,n=t._getRectSize(e),i=n.width,o=n.height,a=-i,s=0,l=r,u=0,c=-r,d=t._position,h=t._isLeftPointer,f=function(){a++,s++,l++,u++,c++},p=function(){l>s?l=s:c<a&&(c=a),h&&f()},g=d;return h?d>t._range[1]-i?(s=-d+t._range[1],a=s-i,p(),g+=a):(a=c=0,s=i):d-t._range[0]<i?(a=-(d-t._range[0]),s=a+i,p(),g+=s):(l=0,f()),t._borderPosition=g,{offset:a,isCut:(!h||u!==c)&&(h||u!==l),points:[a,0,s,0,s,o,l,o,u,o+r,c,o,a,o]}},_update:function(){function e(e,t){t=t||n._getRectSize(e),n._group.attr({translateY:-(t.height+r)});var i=n._getAreaPointsInfo(e),o=i.points,a=i.offset;n._area.attr({points:o}),n._border.attr({x:n._isLeftPointer?o[0]-1:o[2],height:i.isCut?t.height:t.height+r}),n._tracker.attr({translateX:a,width:t.width,height:t.height+r}),n._label.attr({translateX:n._paddingLeftRight+a,translateY:t.height/2-(e.y+e.height/2)})}var t,n=this;clearTimeout(n._timeout),n._label.attr({text:n._text||""});var i=n._getTextSize(),o=n._getRectSize(i);t=n._textSize||i,t=n._textSize=i.width>t.width||i.height>t.height?i:t,n._timeout=setTimeout(function(){e(i,o),n._textSize=i},s),e(t)},setText:function(e){this._text=e},setPosition:function(e){this._position=e,this._update()},applyOptions:function(e,t){var n=this;n._range=t,n._paddingLeftRight=e.paddingLeftRight,n._paddingTopBottom=e.paddingTopBottom,n._textHeight=null,n._colors=[e.invalidRangeColor,e.color],n._area.attr({fill:e.color}),n._border.attr({fill:e.borderColor}),n._label.css((0,o.patchFontOptions)(e.font)),n._update()},getTracker:function(){return this._tracker},setValid:function(e){this._area.attr({fill:this._colors[Number(e)]})},setColor:function(e){this._area.attr({fill:e})},dispose:function(){clearTimeout(this._timeout)},setOverlapped:function(e){var t=this;t._isOverlapped!==e&&(e?t._border.append(t._group):t._isOverlapped&&t._border.remove(),t._isOverlapped=e)},getBorderPosition:function(){return this._borderPosition}};var l=i;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e){var t=e||w.event,n=t.originalEvent,i=t.touches,o=!!n&&n.pointerType,a=!!n&&n.touches,r=n&&void 0!==o&&(o===(n.MSPOINTER_TYPE_TOUCH||"touch")||o===(n.MSPOINTER_TYPE_MOUSE||"mouse")&&1===n.buttons),s=i&&i.length>0||a&&a.length>0;return 1===t.which||r||s}function r(e){var t=e.originalEvent,n=e.touches,i=t&&t.touches;return n&&n.length>1||i&&i.length>1||null}function s(e){r(e)||e.preventDefault()}function l(e){r(e)||(e.stopPropagation(),e.preventDefault())}function u(e){return e&&e.type&&0===e.type.indexOf("touch")}function c(e){var t=e.originalEvent,n=0;return e.pageX?n=e.pageX:t&&t.pageX&&(n=t.pageX),t&&t.touches&&(t.touches.length>0?n=t.touches[0].pageX:t.changedTouches.length>0&&(n=t.changedTouches[0].pageX)),n}function d(e,t,n,i){function r(e){h&&(h=!1,f&&(f.complete(e),f=null))}var s,l,d,h=!1,f=null,p=(s={},o(s,_.default.move,function(t){var o,s;l===u(t)&&(a(t)||r(t),h&&(o=c(t),s=i(),f?f(o-s,t):n.manualRangeSelectionEnabled&&Math.abs(d-o)>=x&&(f=e.placeSliderAndBeginMoving(d-s,o-s,t))))}),o(s,_.default.up,function(t){var o;h&&(o=c(t),!f&&n.moveSelectedRangeByClick&&Math.abs(d-o)<x&&e.moveSelectedArea(o-i(),t),r(t))}),s);return t.on(_.default.down,function(e){n.enabled&&a(e)&&!h&&(h=!0,l=u(e),d=c(e))}),p}function h(e,t,n,i){function r(e){f&&(f=!1,p.complete(e),p=null)}var d,h,f=!1,p=null,g=(d={},o(d,_.default.move,function(e){h===u(e)&&(a(e)||r(e),f&&(s(e),p(c(e)-i(),e)))}),o(d,_.default.up,r),d);return t.on(_.default.down,function(t){n.enabled&&a(t)&&!f&&(f=!0,h=u(t),p=e.beginSelectedAreaMoving(c(t)-i()),l(t))}),g}function f(e,t,n,i){function r(e){f&&(f=!1,p.complete(e),p=null)}var d,h,f=!1,p=null,g=(d={},o(d,_.default.move,function(e){h===u(e)&&(a(e)||r(e),f&&(s(e),p(c(e)-i(),e)))}),o(d,_.default.up,r),d);return(0,y.each)(t,function(t,r){var s;r.on((s={},o(s,_.default.down,function(o){n.enabled&&a(o)&&!f&&(f=!0,h=u(o),p=e.beginSliderMoving(t,c(o)-i()),l(o))}),o(s,_.default.move,function(){p||e.foregroundSlider(t)}),s))}),g}function p(e){function t(){return e.renderer.getRootOffset().left}var n=this._state={},i=e.controller.getTrackerTargets();b.pointerEvents&&e.renderer.root.css({msTouchAction:"pinch-zoom"}),this._docEvents=[h(e.controller,i.selectedArea,n,t),d(e.controller,i.area,n,t),f(e.controller,i.sliders,n,t)],(0,y.each)(this._docEvents,function(e,t){g.default.on(v.default.getDocument(),t)})}t.Tracker=p;var g=i(n(5)),_=i(n(25)),m=n(7),v=i(n(11)),y=n(3),b=n(46),x=10,w=(0,m.getWindow)();p.prototype={constructor:p,dispose:function(){(0,y.each)(this._docEvents,function(e,t){g.default.off(v.default.getDocument(),t)})},update:function(e,t){var n=this._state;n.enabled=e,n.moveSelectedRangeByClick=t.moveSelectedRangeByClick,n.manualRangeSelectionEnabled=t.manualRangeSelectionEnabled}}},function(e,t,n){function i(e,t,n,i){var o=t.getSeries();if(o.length){var a=o[0].getValueAxis();a.updateCanvas({top:n.top,bottom:0,height:n.height+n.top}),t.adjustSeriesDimensions();var r=t.getBoundRange().val;r.sortCategories(a.getCategoriesSorter()),a.setBusinessRange(r),o.forEach(function(t){t._extGroups.seriesGroup=t._extGroups.labelsGroup=e,t.draw(i)})}}function o(e,t){return void 0!==e?e:t}function a(e){this._params=e,this._clipRect=e.renderer.clipRect(),e.root.attr({"clip-path":this._clipRect.id})}t.RangeView=a,a.prototype={constructor:a,update:function(e,t,n,a,r,s){var l,u=this._params.renderer,c=this._params.root,d=n.width-n.left;e=e||{},c.clear(),this._clipRect.attr({x:n.left,y:n.top,width:d,height:n.height}),a||(o(e.visible,t.visible)&&(e.color&&u.rect(n.left,n.top,d+1,n.height).attr({fill:o(e.color,t.color),"class":"dx-range-selector-background"}).append(c),e.image&&e.image.url&&u.image(n.left,n.top,d+1,n.height,e.image.url,o(e.image.location,t.image.location)).append(c)),s&&s.isShowChart()&&(l=u.g().attr({"class":"dxrs-series-group"}).append(c),i(l,s,n,r)))}}},function(e,t,n){t.SeriesDataSource=void 0;var i=n(326),o=n(555),a=n(1),r=n(0),s=n(12),l=n(3),u=n(14),c=n(134),d=n(324),h=n(547),f=function(e){return new h.ThemeManager({options:e,themeSection:"rangeSelector.chart",fontFields:["commonSeriesSettings.label.font"]})},p=function(e,t,n,i,a){var r=[],u=[];return(0,l.each)(e,function(e,t){(0,s.inArray)(t.type,u)===-1&&u.push(t.type)}),(0,l.each)(u,function(s,l){var u=new o.SeriesFamily({type:l,minBubbleSize:t,maxBubbleSize:n,barGroupPadding:i.barGroupPadding,barGroupWidth:i.barGroupWidth,negativesAsZeroes:a});u.add(e),u.adjustSeriesValues(),r.push(u)}),r},g=function(e){var t=this,n=t._themeManager=f(e.chart);n.setTheme(e.chart.theme);
var i=n.getOptions("topIndent"),o=n.getOptions("bottomIndent");t._indent={top:i>=0&&i<1?i:0,bottom:o>=0&&o<1?o:0},t._valueAxis=n.getOptions("valueAxisRangeSelector")||{},t._hideChart=!1,t._series=t._calculateSeries(e),t._seriesFamilies=[]};t.SeriesDataSource=g,g.prototype={constructor:g,_calculateSeries:function(e){var t,n,o,a,s,l,c,h=this,f=[],p=e.dataSource||[],g=h._themeManager,_=g.getOptions("seriesTemplate"),m=_?(0,u.processSeriesTemplate)(_,p):e.chart.series,v=h._valueAxis;for(e.dataSource&&!m&&(a=e.dataSourceField||"arg",m={argumentField:a,valueField:a},h._hideChart=!0),m=Array.isArray(m)?m:m?[m]:[],s=0;s<m.length;s++)t=(0,r.extend)(!0,{},m[s]),t.rotated=!1,n=g.getOptions("series",t,m.length),n.argumentField=n.argumentField||e.dataSourceField,n.name||(n.name="Series "+(s+1).toString()),p&&p.length>0&&(l=new i.Series({renderer:e.renderer,argumentAxis:e.argumentAxis,valueAxis:e.valueAxis,incidentOccurred:e.incidentOccurred},n),f.push(l));if(f.length)for(c={groups:[{series:f,valueAxis:e.valueAxis,valueOptions:{type:v.type,valueType:a?e.valueType:v.valueType}}],argumentOptions:{categories:e.categories,argumentType:e.valueType,type:e.axisType}},o=(0,d.validateData)(p,c,e.incidentOccurred,g.getOptions("dataPrepareSettings")),h.argCategories=c.categories,s=0;s<f.length;s++)f[s].updateData(o[f[s].getArgumentField()]);return f},createPoints:function(){if(0!==this._series.length){var e=this._series,t=new c.Range,n=e[0].getArgumentAxis(),i=this._themeManager,o=i.getOptions("negativesAsZeroes"),r=i.getOptions("negativesAsZeros");e.forEach(function(e){t.addRange(e.getArgumentRange())}),n.getTranslator().updateBusinessRange(t),e.forEach(function(e){e.createPoints()}),this._seriesFamilies=p(e,i.getOptions("minBubbleSize"),i.getOptions("maxBubbleSize"),{barGroupPadding:i.getOptions("barGroupPadding"),barGroupWidth:i.getOptions("barGroupWidth")},(0,a.isDefined)(o)?o:r)}},adjustSeriesDimensions:function(){(0,l.each)(this._seriesFamilies,function(e,t){t.adjustSeriesDimensions()})},getBoundRange:function(){var e,t,n,i,o,r=this,s=r._valueAxis,u=new c.Range({min:s.min,minVisible:s.min,max:s.max,maxVisible:s.max,axisType:s.type,base:s.logarithmBase}),d=new c.Range({});return(0,l.each)(r._series,function(t,n){e=n.getRangeData(),u.addRange(e.val),d.addRange(e.arg)}),u.isEmpty()||d.isEmpty()||(i=s.inverted?r._indent.top:r._indent.bottom,o=s.inverted?r._indent.bottom:r._indent.top,t=u.max-u.min,n=((0,a.isNumeric)(u.maxVisible)?u.maxVisible:u.max)-((0,a.isNumeric)(u.minVisible)?u.minVisible:u.min),(0,a.isDate)(u.min)?u.min=new Date(u.min.valueOf()-t*i):u.min-=t*i,(0,a.isDate)(u.max)?u.max=new Date(u.max.valueOf()+t*o):u.max+=t*o,(0,a.isNumeric)(n)&&(u.maxVisible=u.maxVisible?u.maxVisible+n*o:void 0,u.minVisible=u.minVisible?u.minVisible-n*i:void 0),u.invert=s.inverted),{arg:d,val:u}},getMarginOptions:function(e){var t=Math.min(e.width,e.height)*this._themeManager.getOptions("maxBubbleSize");return this._series.reduce(function(e,n){var i=n.getMarginOptions();return i.processBubbleSize===!0&&(i.size=t),(0,u.mergeMarginOptions)(e,i)},{})},getSeries:function(){return this._series},isEmpty:function(){return 0===this.getSeries().length},isShowChart:function(){return!this._hideChart},getCalculatedValueType:function(){var e=this._series[0];return null===e||void 0===e?void 0:e.argumentType},getThemeManager:function(){return this._themeManager}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(964)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){return t?[Math.min(t[0],t[2],e[0]),Math.min(t[1],t[3],e[3]),Math.max(t[0],t[2],e[2]),Math.max(t[1],t[3],e[1])]:e}t.default=void 0;var a=n(14),r=n(561),s=n(965),l=n(966),u=n(967),c=n(968),d=n(969),h=n(970),f=n(971),p=n(972),g=n(973);n(563);var _=i(n(112)),m=i(n(9)),v=n(111),y=n(131),b=n(133),x=n(153),w=n(233),C=800,k=400,S=/^layers/,D=/\.dataSource$/,I=_.default.inherit({_eventsMap:{onClick:{name:"click"},onCenterChanged:{name:"centerChanged"},onZoomFactorChanged:{name:"zoomFactorChanged"},onHoverChanged:{name:"hoverChanged"},onSelectionChanged:{name:"selectionChanged"}},_rootClassPrefix:"dxm",_rootClass:"dxm-vector-map",_themeSection:"map",_fontFields:["layer:area.label.font","layer:marker:dot.label.font","layer:marker:bubble.label.font","layer:marker:pie.label.font","layer:marker:image.label.font","legend.font","legend.title.font","legend.title.subtitle.font"],_initLayerCollection:function(e){var t=this;t._layerCollection=new f.MapLayerCollection({renderer:t._renderer,projection:t._projection,themeManager:t._themeManager,tracker:t._tracker,dataKey:e,eventTrigger:t._eventTrigger,dataExchanger:t._dataExchanger,tooltip:t._tooltip,notifyDirty:t._notifyDirty,notifyReady:t._notifyReady,dataReady:function(){var e;if(t.option("getBoundsFromData")&&!t.option("bounds")&&(t._preventProjectionEvents(),e=t._getBoundsFromData(),t._projection.setBounds(e),t._allowProjectionEvents()),!t.option("projection")&&(e=e||t._getBoundsFromData(),Math.ceil(e[0])<-180||Math.ceil(e[3])<-90||Math.floor(e[2])>180||Math.floor(e[1])>90)){var n=e[2]-e[0],i=e[1]-e[3];t._projection.setEngine({to:function(t){return[2*(t[0]-e[0])/n-1,2*(t[1]-e[3])/i-1]},from:function(t){return[(t[0]+1)*n/2+e[0],(t[1]+1)*i/2+e[3]]}})}}})},_getBoundsFromData:function(){var e=this._getBoundingBoxFromDataSource();if(!e){var t=(0,f.getMaxBound)(this.getLayers().map(function(e){return e.getBounds()}));t&&(e=t)}return e=e||[],e=[e[0],e[3],e[2],e[1]]},_initLegendsControl:function(){var e=this;e._legendsControl=new d.LegendsControl({renderer:e._renderer,container:e._root,widget:e,layoutControl:e._layoutControl,themeManager:e._themeManager,dataExchanger:e._dataExchanger,notifyDirty:e._notifyDirty,notifyReady:e._notifyReady})},_initControlBar:function(e){var t=this;t._controlBar=new s.ControlBar({renderer:t._renderer,container:t._root,layoutControl:t._layoutControl,projection:t._projection,tracker:t._tracker,dataKey:e})},_initElements:function(){var e,t=this,n=(0,g.generateDataKey)(),i=0;t._preventProjectionEvents=function(){e=!0},t._allowProjectionEvents=function(){e=!1},t._notifyDirty=function(){t._resetIsReady(),++i},t._notifyReady=function(){t._allowProjectionEvents(),0===--i&&t._drawn()},t._preventProjectionEvents(),t._dataExchanger=new c.DataExchanger,t._projection=new r.Projection({centerChanged:function(n){e||t._eventTrigger("centerChanged",{center:n})},zoomChanged:function(n){e||t._eventTrigger("zoomFactorChanged",{zoomFactor:n})}}),t._tracker=new u.Tracker({root:t._root,projection:t._projection,dataKey:n}),t._gestureHandler=new l.GestureHandler({projection:t._projection,renderer:t._renderer,tracker:t._tracker}),t._layoutControl=new h.LayoutControl(t),t._layoutControl.suspend(),t._initLayerCollection(n),t._createHtmlStructure(),t._initControlBar(n),t._initLegendsControl(),t._prepareExtraElements(),t._tooltipViewer=new p.TooltipViewer({tracker:t._tracker,tooltip:t._tooltip,layerCollection:t._layerCollection})},_change_RESUME_LAYOUT:function(){this._layoutControl.resume()},_initialChanges:["PROJECTION","RESUME_LAYOUT","LAYOUT_INIT","BOUNDS","MAX_ZOOM_FACTOR","ZOOM_FACTOR","CENTER"],_layoutChangesOrder:["RESUME_LAYOUT","LAYERS"],_customChangesOrder:["EXTRA_ELEMENTS"],_initCore:function(){this._root=this._renderer.root.attr({align:"center",cursor:"default"}),this._initElements()},_disposeCore:function(){var e=this;e._controlBar.dispose(),e._gestureHandler.dispose(),e._tracker.dispose(),e._legendsControl.dispose(),e._layerCollection.dispose(),e._layoutControl.dispose(),e._tooltipViewer.dispose(),e._dataExchanger.dispose(),e._projection.dispose(),e._dataExchanger=e._gestureHandler=e._projection=e._tracker=e._layoutControl=e._root=e._layerCollection=e._controlBar=e._legendsControl=null},_setupInteraction:function(){var e={centeringEnabled:!!(0,a.parseScalar)(this._getOption("panningEnabled",!0),!0),zoomingEnabled:!!(0,a.parseScalar)(this._getOption("zoomingEnabled",!0),!0)};this._gestureHandler.setInteraction(e),this._controlBar.setInteraction(e)},_getDefaultSize:function(){return{width:C,height:k}},_applySize:function(e){var t={left:e[0],top:e[1],width:e[2]-e[0],height:e[3]-e[1],right:0,bottom:0};this._projection.setSize(t),this._layoutControl.setSize(t),this._layerCollection.setRect([t.left,t.top,t.width,t.height]),this._requestChange(["EXTRA_ELEMENTS"])},_optionChanging:function(e,t,n){t&&n&&S.test(e)&&(t.dataSource&&n.dataSource&&t!==n?t.dataSource=null:D.test(e)&&this.option(e,null))},_applyChanges:function(){this._notifyDirty(),this.callBase.apply(this,arguments),this._notifyReady()},_optionChangesMap:{background:"BACKGROUND",layers:"LAYERS",extraElements:"EXTRA_ELEMENTS",controlBar:"CONTROL_BAR",legends:"LEGENDS",touchEnabled:"TRACKER",wheelEnabled:"TRACKER",panningEnabled:"INTERACTION",zoomingEnabled:"INTERACTION",projection:"PROJECTION",bounds:"BOUNDS",maxZoomFactor:"MAX_ZOOM_FACTOR",zoomFactor:"ZOOM_FACTOR",center:"CENTER"},_optionChangesOrder:["PROJECTION","BOUNDS","MAX_ZOOM_FACTOR","ZOOM_FACTOR","CENTER","BACKGROUND","CONTROL_BAR","LEGENDS","TRACKER","INTERACTION"],_change_PROJECTION:function(){this._setProjection()},_change_BOUNDS:function(){this._setBounds()},_change_MAX_ZOOM_FACTOR:function(){this._setMaxZoom()},_change_ZOOM_FACTOR:function(){this._setZoom()},_change_CENTER:function(){this._setCenter()},_change_BACKGROUND:function(){this._setBackgroundOptions()},_change_LAYERS:function(){this._setLayerCollectionOptions()},_change_CONTROL_BAR:function(){this._setControlBarOptions()},_change_EXTRA_ELEMENTS:function(){this._renderExtraElements()},_change_LEGENDS:function(){this._setLegendsOptions()},_change_TRACKER:function(){this._setTrackerOptions()},_change_INTERACTION:function(){this._setupInteraction()},_themeDependentChanges:["BACKGROUND","LAYERS","CONTROL_BAR","LEGENDS","TRACKER","INTERACTION"],_setProjection:function(){this._projection.setEngine(this.option("projection"))},_setBounds:function(){this._projection.setBounds(this.option("bounds"))},_setMaxZoom:function(){this._projection.setMaxZoom(this.option("maxZoomFactor"))},_setZoom:function(){this._projection.setZoom(this.option("zoomFactor"))},_setCenter:function(){this._projection.setCenter(this.option("center"))},_setBackgroundOptions:function(){this._layerCollection.setBackgroundOptions(this._getOption("background"))},_setLayerCollectionOptions:function(){this._layerCollection.setOptions(this.option("layers"))},_getBoundingBoxFromDataSource:function(){var e=this,t=e._layerCollection.items(),n=[1/0,-(1/0),-(1/0),1/0],i=t&&t.length?t.reduce(function(e,t){var n=t.getData(),i=n.count();if(i>0){var a=n.getBBox();if(a)e=o(e,a);else for(var r=0;r<i;r++)e=o(e,n.getBBox(r))}return e},n):void 0;return i===n?void 0:i},_setControlBarOptions:function(){this._controlBar.setOptions(this._getOption("controlBar"))},_setLegendsOptions:function(){this._legendsControl.setOptions(this.option("legends"))},_setTrackerOptions:function(){this._tracker.setOptions({touchEnabled:this._getOption("touchEnabled",!0),wheelEnabled:this._getOption("wheelEnabled",!0)})},getLayers:function(){return this._layerCollection.items().map(function(e){return e.proxy})},getLayerByIndex:function(e){var t=this._layerCollection.byIndex(e);return t?t.proxy:null},getLayerByName:function(e){var t=this._layerCollection.byName(e);return t?t.proxy:null},clearSelection:function(e){var t,n=this._layerCollection.items(),i=n.length;for(t=0;t<i;++t)n[t].clearSelection(e);return this},center:function(e){var t=this;return void 0===e?t._projection.getCenter():(t._projection.setCenter(e),t)},zoomFactor:function(e){var t=this;return void 0===e?t._projection.getZoom():(t._projection.setZoom(e),t)},viewport:function(e){var t=this;return void 0===e?t._projection.getViewport():(t._projection.setViewport(e),t)},convertCoordinates:function(e){return e=e&&e.length?e:[arguments[0],arguments[1]],this.convertToGeo(e[0],e[1])},convertToGeo:function(e,t){return this._projection.fromScreenPoint([e,t])},convertToXY:function(e,t){return this._projection.toScreenPoint([e,t])}});(0,m.default)("dxVectorMap",I);var T=I;t.default=T,I.addPlugin(v.plugin),I.addPlugin(y.plugin),I.addPlugin(b.plugin),I.addPlugin(x.plugin),I.addPlugin(w.plugins.core),I.addPlugin(w.plugins.vectorMap),e.exports=t.default},function(e,t,n){function i(e){var t=this;t._params=e,t._createElements(e.renderer,e.container,e.dataKey),e.layoutControl.addItem(t),t._subscribeToProjection(e.projection),t._subscribeToTracker(e.tracker),t._createCallbacks(e.projection)}function o(e){delete e._owner,e.update=function(){},e.finish=function(){}}function a(e,t){this._owner=e,this._command=t}function r(e,t,n){function i(){e._callbacks.move([r,s]),o=setTimeout(i,a)}this._command=t;var o=null,a=100,r=0,s=0;switch(this._command){case y:s=-10;break;case b:r=10;break;case x:s=10;break;case w:r=-10}this._stop=function(){return clearTimeout(o),e._callbacks.endMove(),this._stop=e=null,this},n=null,e._callbacks.beginMove(),i()}function s(e,t){function n(){e._adjustZoom(e._zoomFactor+a),i=setTimeout(n,o)}this._owner=e,this._command=t;var i=null,o=150,a=this._command===C?1:-1;this._stop=function(){return clearTimeout(i),this._stop=e=null,this},n()}function l(e,t,n){this._owner=e,this._zoomFactor=e._zoomFactor,this._pos=n.y}t.ControlBar=i;var u=n(14),c=Math,d=c.min,h=c.max,f=c.round,p=c.floor,g=c.sqrt,_=(0,u.enumParser)(["left","center","right"]),m=(0,u.enumParser)(["top","bottom"]),v="command-reset",y="command-move-up",b="command-move-right",x="command-move-down",w="command-move-left",C="command-zoom-in",k="command-zoom-out",S="command-zoom-drag-line",D="command-zoom-drag",I="control-bar",T=1,O=2,E={bigCircleSize:58,smallCircleSize:28,buttonSize:10,arrowButtonOffset:20,incDecButtonSize:11,incButtonOffset:66,decButtonOffset:227,sliderLineStartOffset:88.5,sliderLineEndOffset:205.5,sliderLength:20,sliderWidth:8,trackerGap:4},A=30.5,P=30.5,R=61,M=274,B={};B[v]=a,B[y]=B[b]=B[x]=B[w]=r,B[C]=B[k]=s,B[D]=l,i.prototype={constructor:i,_flags:0,dispose:function(){var e=this;e._params.layoutControl.removeItem(e),e._root.linkRemove().linkOff(),e._offProjection(),e._offTracker(),e._params=e._root=e._offProjection=e._offTracker=e._callbacks=null},_subscribeToProjection:function(e){function t(){n._adjustZoom(e.getScaledZoom())}var n=this;n._offProjection=e.on({engine:function(){n._update()},zoom:t,"max-zoom":function(){n._zoomPartition=e.getZoomScalePartition(),n._sliderUnitLength=n._sliderLineLength/n._zoomPartition,t()}})},_subscribeToTracker:function(e){var t=this,n=!1;t._offTracker=e.on({start:function(e){n=e.data.name===I,n&&t._processStart(e.data.index,e)},move:function(e){n&&t._processMove(e.data.index,e)},end:function(){n&&(t._processEnd(),n=!1)}})},_createCallbacks:function(e){var t=this;t._callbacks={reset:function(t,n){t&&e.setCenter(null),n&&e.setZoom(null)},beginMove:function(){e.beginMoveCenter()},endMove:function(){e.endMoveCenter()},move:function(t){e.moveCenter(t)},zoom:function(t){e.setScaledZoom(t)}}},_createElements:function(e,t,n){var i=this;i._root=e.g().attr({"class":"dxm-control-bar"}).linkOn(t,"control-bar");var o=i._buttonsGroup=e.g().attr({"class":"dxm-control-buttons"}).append(i._root),a=e.g().attr({stroke:"none","stroke-width":0,fill:"#000000",opacity:1e-4}).css({cursor:"pointer"}).append(i._root);i._createButtons(e,n,o),i._createTrackers(e,n,a)},_createButtons:function(e,t,n){var i=this,o=E,a=o.buttonSize/2,r=o.arrowButtonOffset-a,s=o.arrowButtonOffset,l=o.incDecButtonSize/2,u={"stroke-linecap":"square",fill:"none"},c="line";e.circle(0,0,o.bigCircleSize/2).append(n),e.circle(0,0,a).attr({fill:"none"}).append(n),e.path([-a,-r,0,-s,a,-r],c).attr(u).append(n),e.path([r,-a,s,0,r,a],c).attr(u).append(n),e.path([a,r,0,s,-a,r],c).attr(u).append(n),e.path([-r,a,-s,0,-r,-a],c).attr(u).append(n),e.circle(0,o.incButtonOffset,o.smallCircleSize/2).append(n),e.path([[-l,o.incButtonOffset,l,o.incButtonOffset],[0,o.incButtonOffset-l,0,o.incButtonOffset+l]],"area").append(n),e.circle(0,o.decButtonOffset,o.smallCircleSize/2).append(n),e.path([-l,o.decButtonOffset,l,o.decButtonOffset],"area").append(n),i._zoomLine=e.path([],"line").append(n),i._zoomDrag=e.rect(p(-o.sliderLength/2),p(o.sliderLineEndOffset-o.sliderWidth/2),o.sliderLength,o.sliderWidth).append(n),i._sliderLineLength=o.sliderLineEndOffset-o.sliderLineStartOffset},_createTrackers:function(e,t,n){var i=E,o=f((i.arrowButtonOffset-i.trackerGap)/2),a=i.arrowButtonOffset-o,r=f(g(i.bigCircleSize*i.bigCircleSize/4-o*o)),s=r-a;e.rect(-o,-o,2*o,2*o).data(t,{index:v,name:I}).append(n),e.rect(-o,-r,2*o,s).data(t,{index:y,name:I}).append(n),e.rect(a,-o,s,2*o).data(t,{index:b,name:I}).append(n),e.rect(-o,a,2*o,s).data(t,{index:x,name:I}).append(n),e.rect(-r,-o,s,2*o).data(t,{index:w,name:I}).append(n),e.circle(0,i.incButtonOffset,i.smallCircleSize/2).data(t,{index:C,name:I}).append(n),e.circle(0,i.decButtonOffset,i.smallCircleSize/2).data(t,{index:k,name:I}).append(n),e.rect(-2,i.sliderLineStartOffset-2,4,i.sliderLineEndOffset-i.sliderLineStartOffset+4).css({cursor:"default"}).data(t,{index:S,name:I}).append(n),this._zoomDragTracker=e.rect(-i.sliderLength/2,i.sliderLineEndOffset-i.sliderWidth/2,i.sliderLength,i.sliderWidth).data(t,{index:D,name:I}).append(n)},resize:function(e){this._isActive&&this._root.attr({visibility:null!==e?null:"hidden"})},getLayoutOptions:function(){return this._isActive?this._layoutOptions:null},locate:function(e,t){this._root.attr({translateX:e+this._margin+A,translateY:t+this._margin+P})},_update:function(){var e=this;e._isActive=e._isEnabled&&e._flags&&e._params.projection.isInvertible(),e._isActive?e._root.linkAppend():e._root.linkRemove(),e._processEnd(),e.updateLayout()},setInteraction:function(e){var t=this;(0,u.parseScalar)(e.centeringEnabled,!0)?t._flags|=T:t._flags&=~T,(0,u.parseScalar)(e.zoomingEnabled,!0)?t._flags|=O:t._flags&=~O,t._update()},setOptions:function(e){var t=this;t._isEnabled=!!(0,u.parseScalar)(e.enabled,!0),t._margin=e.margin||0,t._layoutOptions={width:2*t._margin+R,height:2*t._margin+M,horizontalAlignment:_(e.horizontalAlignment,"left"),verticalAlignment:m(e.verticalAlignment,"top")},t._buttonsGroup.attr({"stroke-width":e.borderWidth,stroke:e.borderColor,fill:e.color,"fill-opacity":e.opacity}),t._update()},_adjustZoom:function(e){var t=this,n=E.sliderLineStartOffset,i=E.sliderLineEndOffset,o=E.sliderWidth;t._zoomFactor=h(d(f(e),t._zoomPartition),0);var a={translateY:-f(t._zoomFactor*t._sliderUnitLength)},r=i-o/2+a.translateY;t._zoomLine.attr({points:[[0,n,0,h(n,r)],[0,d(i,r+o),0,i]]}),t._zoomDrag.attr(a),t._zoomDragTracker.attr(a)},_applyZoom:function(){this._callbacks.zoom(this._zoomFactor)},_processStart:function(e,t){var n;this._isActive&&(n=B[e],this._command=n&&n.flags&this._flags?new n(this,e,t):null)},_processMove:function(e,t){this._command&&this._command.update(e,t)},_processEnd:function(){this._command&&this._command.finish(),this._command=null}},a.flags=T|O,a.prototype.update=function(e){e!==this._command&&o(this)},a.prototype.finish=function(){var e=this._owner._flags;this._owner._callbacks.reset(!!(e&T),!!(e&O)),o(this)},r.flags=T,r.prototype.update=function(e){this._command!==e&&this.finish()},r.prototype.finish=function(){o(this._stop())},s.flags=O,s.prototype.update=function(e){this._command!==e&&this.finish()},s.prototype.finish=function(){this._owner._applyZoom(),o(this._stop())},l.flags=O,l.prototype.update=function(e,t){var n=this._owner;n._adjustZoom(this._zoomFactor+n._zoomPartition*(this._pos-t.y)/n._sliderLineLength)},l.prototype.finish=function(){this._owner._applyZoom(),o(this)}},function(e,t,n){function i(e){var t=this;t._projection=e.projection,t._renderer=e.renderer,t._x=t._y=0,t._subscribeToTracker(e.tracker)}t.GestureHandler=i;var o=Math.log,a=Math.LN2;i.prototype={constructor:i,dispose:function(){this._offTracker(),this._offTracker=null},_subscribeToTracker:function(e){var t=this,n=!1;t._offTracker=e.on({start:function(e){n="control-bar"!==e.data.name,n&&t._processStart(e)},move:function(e){n&&t._processMove(e)},end:function(){n&&t._processEnd()},zoom:function(e){t._processZoom(e)}})},setInteraction:function(e){this._processEnd(),this._centeringEnabled=e.centeringEnabled,this._zoomingEnabled=e.zoomingEnabled},_processStart:function(e){this._centeringEnabled&&(this._x=e.x,this._y=e.y,this._projection.beginMoveCenter())},_processMove:function(e){var t=this;t._centeringEnabled&&(t._renderer.root.attr({cursor:"move"}),t._projection.moveCenter([t._x-e.x,t._y-e.y]),t._x=e.x,t._y=e.y)},_processEnd:function(){this._centeringEnabled&&(this._renderer.root.attr({cursor:"default"}),this._projection.endMoveCenter())},_processZoom:function(e){var t,n,i,r=this;r._zoomingEnabled&&(e.delta?t=e.delta:e.ratio&&(t=o(e.ratio)/a),r._centeringEnabled&&(n=r._renderer.getRootOffset(),n=[e.x-n.left,e.y-n.top],i=r._projection.fromScreenPoint(n)),r._projection.changeScaledZoom(t),r._centeringEnabled&&r._projection.setCenterByPoint(i,n))}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){var t=this;t._root=e.root,t._createEventHandlers(e.dataKey),t._createProjectionHandlers(e.projection),t._initEvents(),t._focus=new p(function(e,n){t._fire(e,n)}),t._attachHandlers()}function a(e,t,n,i){return S((e-n)*(e-n)+(t-i)*(t-i))}function r(e){var t=e.originalEvent.type,n=e.originalEvent.pointerType;return/^touch/.test(t)||/^MSPointer/.test(t)&&4!==n||/^pointer/.test(t)&&"mouse"!==n}function s(e,t){for(var n,i=0,o=e.length;i<o;++i)if(e[i]){n=t[i];break}return I(n||t[i],T)}function l(){var e=[w.pointerEnabled,w.msPointerEnabled,(0,_.hasProperty)("ontouchstart")];f={start:s(e,["pointerdown","MSPointerDown","touchstart mousedown","mousedown"]),move:s(e,["pointermove","MSPointerMove","touchmove mousemove","mousemove"]),end:s(e,["pointerup","MSPointerUp","touchend mouseup","mouseup"]),wheel:I(b.name,T)}}function u(e){var t=e.originalEvent,n=t.touches&&t.touches[0]||{};return{x:n.pageX||t.pageX||e.pageX,y:n.pageY||t.pageY||e.pageY}}function c(e){return e.originalEvent.pointerId}function d(e,t){var n=e.originalEvent;return n=void 0!==n.pointerId?n.pointerId===t?n:null:n.touches[t],n?{x:n.pageX||e.pageX,y:n.pageY||e.pageY}:null}function h(e,t){if(0===e)return 0;var n=k(e),i=D(e/n);return t.dir&&i!==t.dir?0:(t.dir=i,n=n<.1?0:n<1?1:n>4?4:D(n),i*n)}t.Tracker=o;var f,p,g=i(n(5)),_=n(7),m=i(n(11)),v=n(562),y=n(8),b=n(139),x=n(14),w=(0,_.getNavigator)(),C=Math,k=C.abs,S=C.sqrt,D=C.round,I=y.addNamespace,T="dxVectorMap",O="start",E="move",A="end",P="zoom",R="hover-on",M="hover-off",B="click",V="focus-on",F="focus-move",L="focus-off",H=500,z=5,N=20,j=5,W=10,G=100,q=50,$=300;l(),o.prototype={constructor:o,dispose:function(){var e=this;e._detachHandlers(),e._disposeEvents(),e._focus.dispose(),e._root=e._focus=e._docHandlers=e._rootHandlers=null},_eventNames:[O,E,A,P,B,R,M,V,L,F],_startClick:function(e,t){if(t){var n=u(e);this._clickState={x:n.x,y:n.y,threshold:r(e)?N:z,time:Date.now()}}},_endClick:function(e,t){var n,i,o=this._clickState;o&&(t&&Date.now()-o.time<=H&&(n=o.threshold,i=u(e),k(i.x-o.x)<=n&&k(i.y-o.y)<=n&&this._fire(B,{data:t,x:i.x,y:i.y,$event:e})),this._clickState=null)},_startDrag:function(e,t){if(t){var n=u(e),i=this._dragState={x:n.x,y:n.y,data:t};this._fire(O,{x:i.x,y:i.y,data:i.data})}},_moveDrag:function(e,t){var n=this._dragState;if(n){var i=u(e),o=r(e)?W:j;(n.active||k(i.x-n.x)>o||k(i.y-n.y)>o)&&(n.x=i.x,n.y=i.y,n.active=!0,n.data=t||{},this._fire(E,{x:n.x,y:n.y,data:n.data}))}},_endDrag:function(){var e=this._dragState;e&&(this._dragState=null,this._fire(A,{x:e.x,y:e.y,data:e.data}))},_wheelZoom:function(e,t){if(t){var n=this,i=n._wheelLock,o=Date.now();if(!(o-i.time<=q)){o-i.dirTime>$&&(i.dir=0);var a=h(e.delta/120||0,i);if(0!==a){var r=u(e);n._fire(P,{delta:a,x:r.x,y:r.y}),i.time=i.dirTime=o}}}},_startZoom:function(e,t){if(r(e)&&t){var n,i,o=this._zoomState=this._zoomState||{};o.pointer1&&o.pointer2||(void 0===o.pointer1&&(o.pointer1=c(e)||0,n=d(e,o.pointer1),o.x1=o.x1_0=n.x,o.y1=o.y1_0=n.y),void 0===o.pointer2&&(i=c(e)||1,i!==o.pointer1&&(n=d(e,i),n&&(o.x2=o.x2_0=n.x,o.y2=o.y2_0=n.y,o.pointer2=i,o.ready=!0,this._endDrag()))))}},_moveZoom:function(e){var t,n=this._zoomState;n&&r(e)&&(void 0!==n.pointer1&&(t=d(e,n.pointer1),t&&(n.x1=t.x,n.y1=t.y)),void 0!==n.pointer2&&(t=d(e,n.pointer2),t&&(n.x2=t.x,n.y2=t.y)))},_endZoom:function(e){var t,n,i=this._zoomState;i&&r(e)&&(i.ready&&(t=a(i.x1_0,i.y1_0,i.x2_0,i.y2_0),n=a(i.x1,i.y1,i.x2,i.y2),this._fire(P,{ratio:n/t,x:(i.x1_0+i.x2_0)/2,y:(i.y1_0+i.y2_0)/2})),this._zoomState=null)},_startHover:function(e,t){this._doHover(e,t,!0)},_moveHover:function(e,t){this._doHover(e,t,!1)},_doHover:function(e,t,n){var i=this;return i._dragState&&i._dragState.active||i._zoomState&&i._zoomState.ready?void i._cancelHover():void(r(e)!==n||i._hoverTarget===e.target||i._hoverState&&i._hoverState.data===t||(i._cancelHover(),t&&(i._hoverState={data:t},i._fire(R,{data:t})),i._hoverTarget=e.target))},_cancelHover:function(){var e=this._hoverState;this._hoverState=this._hoverTarget=null,e&&this._fire(M,{data:e.data})},_startFocus:function(e,t){this._doFocus(e,t,!0)},_moveFocus:function(e,t){this._doFocus(e,t,!1)},_doFocus:function(e,t,n){var i=this;return i._dragState&&i._dragState.active||i._zoomState&&i._zoomState.ready?void i._cancelFocus():void(r(e)===n&&(i._focus.turnOff(),t&&i._focus.turnOn(t,u(e))))},_cancelFocus:function(){this._focus.cancel()},_createEventHandlers:function(e){function t(t){var n=t.target;return("tspan"===n.tagName?n.parentNode:n)[e]}var n=this;n._docHandlers={},n._rootHandlers={},n._rootHandlers[f.start]=n._docHandlers[f.start]=function(e){var i=r(e),o=t(e);i&&!n._isTouchEnabled||(o&&(e.preventDefault(),e.stopPropagation()),n._startClick(e,o),n._startDrag(e,o),n._startZoom(e,o),n._startHover(e,o),n._startFocus(e,o))},n._docHandlers[f.move]=function(e){var i=r(e),o=t(e);i&&!n._isTouchEnabled||(n._moveDrag(e,o),n._moveZoom(e,o),n._moveHover(e,o),n._moveFocus(e,o))},n._docHandlers[f.end]=function(e){var i=r(e),o=t(e);i&&!n._isTouchEnabled||(n._endClick(e,o),n._endDrag(e,o),n._endZoom(e,o))},n._rootHandlers[f.wheel]=function(e){if(n._cancelFocus(),n._isWheelEnabled){var i=t(e);i&&(e.preventDefault(),e.stopPropagation(),n._wheelZoom(e,i))}},n._wheelLock={dir:0}},_createProjectionHandlers:function(e){function t(){n._cancelFocus()}var n=this;e.on({center:t,zoom:t})},reset:function(){var e=this;e._clickState=null,e._endDrag(),e._cancelHover(),e._cancelFocus()},setOptions:function(e){var t=this;t.reset(),t._detachHandlers(),t._isTouchEnabled=!!(0,x.parseScalar)(e.touchEnabled,!0),t._isWheelEnabled=!!(0,x.parseScalar)(e.wheelEnabled,!0),t._attachHandlers()},_detachHandlers:function(){var e=this;e._isTouchEnabled&&e._root.css({"touch-action":"","-webkit-user-select":""}).off(I("MSHoldVisual",T)).off(I("contextmenu",T)),g.default.off(m.default.getDocument(),e._docHandlers),e._root.off(e._rootHandlers)},_attachHandlers:function(){var e=this;e._isTouchEnabled&&e._root.css({"touch-action":"none","-webkit-user-select":"none"}).on(I("MSHoldVisual",T),function(e){e.preventDefault()}).on(I("contextmenu",T),function(e){r(e)&&e.preventDefault()}),g.default.on(m.default.getDocument(),e._docHandlers),e._root.on(e._rootHandlers)}},p=function(e){var t,n,i=this,o=null,a=null,r=!1,s=null;i.dispose=function(){clearTimeout(s),i.turnOn=i.turnOff=i.cancel=i.dispose=i=e=o=a=s=null},i.turnOn=function(i,l){function u(e){r=!e,e&&(o=a,clearTimeout(s),s=null)}i===a&&r||(r=!1,a=i,o?(t=l.x,n=l.y,a===o?(e(F,{data:a,x:t,y:n}),u(!0)):e(V,{data:a,x:t,y:n,done:u})):(t=l.x,n=l.y,e(V,{data:a,x:t,y:n,done:u})))},i.turnOff=function(){a=null,o&&!r&&(s=s||setTimeout(function(){s=null,e(L,{data:o}),o=null},G))},i.cancel=function(){clearTimeout(s),o&&e(L,{data:o}),o=a=s=null}},(0,v.makeEventEmitter)(o)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(){this._store={}}t.DataExchanger=o;var a=i(n(27));o.prototype={constructor:o,dispose:function(){return this._store=null,this},_get:function(e,t){var n=this._store[e]||(this._store[e]={});return n[t]||(n[t]={callbacks:(0,a.default)()})},set:function(e,t,n){var i=this._get(e,t);return i.data=n,i.callbacks.fire(n),this},bind:function(e,t,n){var i=this._get(e,t);return i.callbacks.add(n),i.data&&n(i.data),this},unbind:function(e,t,n){var i=this._get(e,t);return i.callbacks.remove(n),this}}},function(e,t,n){function i(e,t,n){var i,o,a=t.length,r=[];for(i=0;i<a;++i)r[i]=o={start:e[i],end:e[i+1],index:i},o[n]=t[i],o.states={normal:{fill:o.color}},o.visible=!0;return r}function o(e){this._params=e,this._items=[],e.container.virtualLink("legend-base")}t.LegendsControl=o;var a=n(0),r=n(3),s=n(50),l=n(228),u=a.extend,c=r.each,d={category:"UNKNOWN",name:"UNKNOWN"},h=function(e){var t=this;t._params=e,t._root=e.renderer.g().attr({"class":"dxm-legend"}).linkOn(e.container,{name:"legend",after:"legend-base"}).enableLinks().linkAppend(),e.layoutControl.addItem(t),l.Legend.call(t,{renderer:e.renderer,widget:e.widget,group:t._root,backgroundClass:null,itemsGroupClass:null,textField:"text",getFormatObject:function(e){return e}}),t._onDataChanged=function(e){t._updateData(e)}};h.prototype=u((0,s.clone)(l.Legend.prototype),{constructor:h,dispose:function(){var e=this;return e._params.layoutControl.removeItem(e),e._unbindData(),e._root.linkRemove().linkOff(),e._params=e._root=e._onDataChanged=null,l.Legend.prototype.dispose.apply(e,arguments)},resize:function(e){this._params.notifyDirty(),null===e?this.erase():this.draw(e.width,e.height),this._params.notifyReady()},locate:l.Legend.prototype.shift,_updateData:function(e){this._options.defaultColor=e&&e.defaultColor,this.update(e?i(e.partition,e.values,this._dataName):[],this._options,this._params.themeManager.theme("legend").title),this.updateLayout()},_unbindData:function(){this._dataCategory&&this._params.dataExchanger.unbind(this._dataCategory,this._dataName,this._onDataChanged)},_bindData:function(e){this._params.dataExchanger.bind(this._dataCategory=e.category,this._dataName=e.name,this._onDataChanged)},setOptions:function(e){var t=this;t.update(t._data,e,this._params.themeManager.theme("legend").title),t._unbindData();var n=e.source;return t._bindData(n?{category:n.layer,name:n.grouping}:d),t.updateLayout(),t}}),o.prototype={constructor:o,dispose:function(){c(this._items,function(e,t){t.dispose()}),this._params=this._items=null},setOptions:function(e){var t,n=e&&e.length?e:[],i=this._items,o=n.length,a=this._params,r=a.themeManager.theme("legend");for(t=i.length;t<o;++t)i[t]=new h(a);for(t=i.length-1;t>=o;--t)i[t].dispose(),i.splice(t,1);for(a.layoutControl.suspend(),t=0;t<o;++t)i[t].setOptions(u(!0,{},r,n[t]));a.layoutControl.resume()}}},function(e,t,n){function i(e){return 3*v[e.verticalAlignment]+m[e.horizontalAlignment]}function o(e,t){var n=(e.right-e.left)/3,o=(e.bottom-e.top)/2,a=e.left,r=h(a+n),s=h(a+n+n),l=e.right,u=e.top,c=h(u+o),d=e.bottom,f=[{rect:[a,u,r,c]},{rect:[r,u,s,c],center:!0},{rect:[s,u,l,c],horInversion:!0},{rect:[a,c,r,d],verInversion:!0},{rect:[r,c,s,d],center:!0,verInversion:!0},{rect:[s,c,l,d],horInversion:!0,verInversion:!0}],p=[[],[],[],[],[],[]];return g(t,function(e,t){var n=t.getLayoutOptions();n&&p[i(n)].push({item:t,width:n.width,height:n.height})}),g(f,function(e,t){p[e].length?t.items=p[e]:(t.center?t.rect[0]=t.rect[2]=(t.rect[0]+t.rect[2])/2:t.rect[t.horInversion?0:2]=t.rect[t.horInversion?2:0],t.rect[t.verInversion?1:3]=t.rect[t.verInversion?3:1])}),f}function a(e){g([0,1,2,3,4,5],function(t,n){var i=e[n],o=e[(n+3)%6];i.items&&(o.items||(i.rect[1]=f(i.rect[1],o.rect[3]),i.rect[3]=p(i.rect[3],o.rect[1])))}),g([1,4],function(t,n){var i,o,a=e[n],r=e[n-1],s=e[n+1];a.items?r.items||s.items||(i=a.rect[0]-r.rect[2],o=s.rect[0]-a.rect[2],i>o?i/o>=2?(a.rect[0]-=i,a.right=!0):(a.rect[0]-=o,a.rect[2]+=o):o/i>=2?(a.rect[2]+=o,a.center=null):(a.rect[0]-=i,a.rect[2]+=i)):(r.items&&(r.rect[2]=(a.rect[0]+a.rect[2])/2),s.items&&(s.rect[0]=(a.rect[0]+a.rect[2])/2))})}function r(e,t){var n=!1;return a(e),g(e,function(e,i){
i.items&&(n=s(i,t)||n)}),n}function s(e,t){var n=e.rect,i=n[2]-n[0],o=n[3]-n[1],a=0,r=0,s=0,l=n[2],u=n[3],c=n[0],d=n[1],_=[],m=!1;return g(e.items,function(h,g){if(g.width>i||g.height>o)return _.push(null),m=!0,t||!1;if(a+g.width>i&&(r+=s,a=s=0),r+g.height>o)return _.push(null),m=!0,t||!1;s=p(s,g.height);var v=e.horInversion?n[2]-g.width-a:n[0]+a,y=e.verInversion?n[3]-g.height-r:n[1]+r;a+=g.width,l=f(l,v),u=f(u,y),c=p(c,v+g.width),d=p(d,y+g.height),_.push([v,y])}),!t&&m||(a=0,e.right?a=n[2]-n[0]-c+l:e.center&&(a=h((n[2]-n[0]-c+l)/2)),g(e.items,function(e,t){var n=_[e];n?t.item.locate(n[0]+a,n[1]):t.item.resize(null)}),e.rect=[l,u,c,d],e.items=null),m}function l(e,t){var n=o(e,t);r(n)&&r(n,!0)}function u(e){var t=this;t._items=[],t._suspended=0,t._widget=e,t._updateLayout=function(){t._update()}}t.LayoutControl=u;var c=n(12),d=n(3),h=Math.round,f=Math.min,p=Math.max,g=d.each,_=c.inArray,m={left:0,center:1,right:2},v={top:0,bottom:1};u.prototype={constructor:u,dispose:function(){this._items=this._updateLayout=null},setSize:function(e){this._canvas=e,this._update()},suspend:function(){++this._suspended},resume:function(){0===--this._suspended&&this._update()},addItem:function(e){this._items.push(e),e.updateLayout=this._updateLayout},removeItem:function(e){this._items.splice(_(e,this._items),1),e.updateLayout=null},_update:function(){var e;0===this._suspended&&(e=this._canvas,g(this._items,function(t,n){n.resize(e)}),this._widget.resolveItemsDeferred(this._items.filter(function(e){return e.getTemplatesGroups&&e.getTemplatesDef})),l({left:e.left,top:e.top,right:e.width+e.left,bottom:e.height+e.top},this._items))}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e){return e.reduce(function(e,t){return t?[ve(e[0],t[0]),ve(e[1],t[1]),ye(e[2],t[2]),ye(e[3],t[3])]:e},e[0])}function a(e){var t=(0,J.normalizeEnum)(e);return t=t in he?he[t]:he.single,null!==t&&(t={state:{},single:t}),t}function r(e,t){return(e[t]||{}).name}function s(){}function l(e){this.raw=e}function u(e){this.raw=e}function c(e){return fe(e.features)}function d(e){var t;return e&&(c(e)?t=u:1===e.length&&e[0]&&c(e[0])?(t=u,e=e[0]):fe(e)&&(t=l)),new(t=t||s)(e)}function h(e){return e?c(e)?[e]:e:[]}function f(e,t,n){t.call(n,e)}function p(e){e.text.attr({visibility:e.size[0]/e.spaceSize[0]<de&&e.size[1]/e.spaceSize[1]<de?null:"hidden"})}function g(e){e.text.attr({visibility:e.size[0]/e.spaceSize[0]<de||e.size[1]/e.spaceSize[1]<de?null:"hidden"})}function _(e,t){return e.attribute(t)}function m(e,t){return void 0!==e?e:t}function v(e){var t=Ce[e.type],n=e.coordinates;return t||(t="number"==typeof n[0]?re:"number"==typeof n[0][0]?ae:oe),t}function y(e,t,n,i){e[i].attr(t[i][n])}function b(e,t){return e.project(t)}function x(e,t){var n,i=[],o=i.length=t.length;for(n=0;n<o;++n)i[n]=e.project(t[n]);return i}function w(e,t){return[x(e,t)]}function C(e,t){var n,i=[],o=i.length=t.length;for(n=0;n<o;++n)i[n]=x(e,t[n]);return i}function k(e,t){var n,i=[],o=i.length=t.length;for(n=0;n<o;++n)i[n]=C(e,t[n]);return ie.apply([],i)}function S(e,t,n){var i=t.transform(n);e.root.attr({translateX:i[0],translateY:i[1]})}function D(e,t){var n,i,o=[],a=t.length,r=0;for(o.length=2*a,n=0;n<a;++n)i=e.transform(t[n]),o[r++]=i[0],o[r++]=i[1];return o}function I(e,t,n){var i,o=[],a=o.length=n.length;for(i=0;i<a;++i)o[i]=D(t,n[i]);e.root.attr({points:o})}function T(e,t,n){var i=t.transform(n[0]);e.spaceSize=t.getSquareSize(n[1]),e.text.attr({translateX:i[0],translateY:i[1]}),p(e)}function O(e,t,n){var i=t.transform(n[0]);e.spaceSize=t.getSquareSize(n[1]),e.text.attr({translateX:i[0],translateY:i[1]}),g(e)}function E(e,t,n){var i=M(e.settings,n);return A(e.grouping,t,i),void 0===n.color&&n.paletteIndex>=0&&(i.color=i._colors[n.paletteIndex]),i}function A(e,t,n){ne(e,function(e,i){var o=P(i.callback(t,i.field),i.partition);o>=0&&(n[e]=i.values[o])})}function P(e,t){var n,i=0,o=t.length-1,a=-1;if(t[i]<=e&&e<=t[o])if(e===t[o])a=o-1;else{for(;o-i>1;)n=i+o>>1,e<t[n]?o=n:i=n;a=i}return a}function R(e,t,n,i){e.params.eventTrigger(i,{target:t.proxy,state:n})}function M(e,t){var n=te({},e,t);return n.label=te({},e.label,n.label),n.label.font=te({},e.label.font,n.label.font),n}function B(e,t){var n,i,o,a=e.params.themeManager,r=e.str,s=M(te({label:{},color:r.getDefaultColor(e,t.palette)},a.theme("layer:"+r.fullType)),t);if(s.paletteSize>0){for(o=a.createDiscretePalette(s.palette,s.paletteSize),i=0,n=[];i<s.paletteSize;++i)n.push(o.getColor(i));s._colors=n}return s}function V(e,t){return e.attribute(t)}function F(e){var t=e.name,n=e.params.dataExchanger;ne(e.grouping,function(e){n.set(t,e,null)}),e.grouping={}}function L(e,t,n){return n?e|=t:e&=~t,e}function H(e,t){return!!(e&t)}function z(e,t,n){var i={index:n,name:t,getElements:function(){return e.getProxies()},clearSelection:function(t){return e.clearSelection(t),i},getDataSource:function(){return e.getDataSource()},getBounds:function(){return e.getBounds()}};return i}function N(e,t,n){var i={coordinates:function(){return t},attribute:function(e,t){return arguments.length>1?(n[e]=t,i):arguments.length>0?n[e]:n},selected:function(t,n){return arguments.length>0?(e.setSelected(t,n),i):e.isSelected()},applySettings:function(t){return e.update(t),i}};return i}function j(e){var t,n,i,o=e.length,a=e[o-1],r=0,s=0,l=0,u=1/0,c=-(1/0),d=1/0,h=-(1/0);for(t=0;t<o;++t)n=a,a=e[t],i=n[0]*a[1]-a[0]*n[1],l+=i,r+=(n[0]+a[0])*i,s+=(n[1]+a[1])*i,u=ve(u,a[0]),c=ye(c,a[0]),d=ve(d,a[1]),h=ye(h,a[1]);return{area:_e(l)/2,center:[2*r/3/l-(u+c)/2,2*s/3/l-(d+h)/2]}}function W(e){var t,n,i=e.length,o=e[0]||[],a=0,r=[0],s=o[0],l=o[0],u=o[1],c=o[1];for(t=1;t<i;++t)n=o,o=e[t],a+=be((n[0]-o[0])*(n[0]-o[0])+(n[1]-o[1])*(n[1]-o[1])),r[t]=a,s=ve(s,o[0]),l=ye(l,o[0]),u=ve(u,o[1]),c=ye(c,o[1]);t=P(a/2,r),n=e[t],o=e[t+1];var d=(a/2-r[t])/(r[t+1]-r[t]);return i?[[n[0]*(1-d)+o[0]*d,n[1]*(1-d)+o[1]*d],[l-s,c-u],a]:[]}function G(e){var t,n,i,o=e.length,a=0;for(t=0;t<o;++t)n=j(e[t]),n.area>a&&(a=n.area,i=n);return i?[i.center,[be(i.area),be(i.area)]]:[[],[]]}function q(e){var t,n,i,o=e.length,a=0;for(t=0;t<o;++t)n=W(e[t]),n[2]>a&&(a=n[2],i=n);return i||[[],[]]}function $(e){var t=this,n=e.renderer;t._params=e,t._layers=[],t._layerByName={},t._rect=[0,0,0,0],t._clip=n.clipRect(),t._background=n.rect().attr({"class":"dxm-background"}).data(e.dataKey,{name:"background"}).append(n.root),t._container=n.g().attr({"class":"dxm-layers","clip-path":t._clip.id}).append(n.root).enableLinks(),t._subscribeToTracker(e.tracker,n,e.eventTrigger),t._dataReady=e.dataReady}t.getMaxBound=o,t.MapLayerCollection=$;var K=n(4),U=n(0),Y=n(3),Z=i(n(94)),X=n(1),Q=n(6),J=n(14),ee=K.noop,te=U.extend,ne=Y.each,ie=Array.prototype.concat,oe="area",ae="line",re="marker",se=0,le=1,ue=2,ce=[0,1,2,2],de=1,he={none:null,single:-1,multiple:NaN},fe=Array.isArray,pe=Number,ge=String,_e=Math.abs,me=Math.round,ve=Math.min,ye=Math.max,be=Math.sqrt;s.prototype.count=function(){return 0},l.prototype={constructor:l,count:function(){return this.raw.length},item:function(e){return this.raw[e]},geometry:function(e){return{coordinates:e.coordinates}},attributes:function(e){return e.attributes},getBBox:function(e){return 0===arguments.length?void 0:this.raw[e].bbox}},u.prototype={constructor:u,count:function(){return this.raw.features.length},item:function(e){return this.raw.features[e]},geometry:function(e){return e.geometry},attributes:function(e){return e.properties},getBBox:function(e){return 0===arguments.length?this.raw.bbox:this.raw.features[e].bbox}};var xe,we,Ce={Point:re,MultiPoint:ae,LineString:ae,MultiLineString:ae,Polygon:oe,MultiPolygon:oe},ke={setup:ee,reset:ee,arrange:ee,updateGrouping:ee,getDefaultColor:ee},Se={},De={},Ie={},Te=function(e,t){var n,i=(0,J.normalizeEnum)(e.type),o=(0,J.normalizeEnum)(e.elementType),a=te({},ke);return t.count()>0&&(n=t.geometry(t.item(0)),i=Se[i]?i:v(n),te(a,Se[i]),a.fullType=a.type=i,De[i]&&te(a,De[i](n)),Ie[i]&&(o=Ie[i][o]?o:Ie[i]._default,te(a,Ie[i][o]),a.elementType=o,a.fullType+=":"+o)),a};Se[oe]={projectLabel:G,transform:I,transformLabel:T,draw:function(e,t,n){t.root=e.renderer.path([],"area").data(e.dataKey,n)},refresh:ee,getLabelOffset:function(e){return p(e),[0,0]},getStyles:function(e){var t=e.color||null,n=e.borderColor||null,i=m(e.borderWidth,null),o=m(e.opacity,null);return{root:[{"class":"dxm-area",stroke:n,"stroke-width":i,fill:t,opacity:o},{"class":"dxm-area dxm-area-hovered",stroke:e.hoveredBorderColor||n,"stroke-width":m(e.hoveredBorderWidth,i),fill:e.hoveredColor||t,opacity:m(e.hoveredOpacity,o)},{"class":"dxm-area dxm-area-selected",stroke:e.selectedBorderColor||n,"stroke-width":m(e.selectedBorderWidth,i),fill:e.selectedColor||t,opacity:m(e.selectedOpacity,o)}]}},setState:function(e,t,n){y(e,t,n,"root")},hasLabelsGroup:!0,updateGrouping:function(e){xe(e)},getDefaultColor:ee},Se[ae]={projectLabel:q,transform:I,transformLabel:O,draw:function(e,t,n){t.root=e.renderer.path([],"line").data(e.dataKey,n)},refresh:ee,getLabelOffset:function(e){return g(e),[0,0]},getStyles:function(e){var t=e.color||e.borderColor||null,n=m(e.borderWidth,null),i=m(e.opacity,null);return{root:[{"class":"dxm-line",stroke:t,"stroke-width":n,opacity:i},{"class":"dxm-line dxm-line-hovered",stroke:e.hoveredColor||e.hoveredBorderColor||t,"stroke-width":m(e.hoveredBorderWidth,n),opacity:m(e.hoveredOpacity,i)},{"class":"dxm-line dxm-line-selected",stroke:e.selectedColor||e.selectedBorderColor||t,"stroke-width":m(e.selectedBorderWidth,n),opacity:m(e.selectedOpacity,i)}]}},setState:function(e,t,n){y(e,t,n,"root")},hasLabelsGroup:!0,updateGrouping:function(e){xe(e)},getDefaultColor:ee},Se[re]={project:b,transform:S,draw:function(e,t,n){t.root=e.renderer.g(),this._draw(e,t,n)},refresh:ee,hasLabelsGroup:!1,getLabelOffset:function(e,t){return[me((e.size[0]+ye(t.size||0,0))/2)+2,0]},getStyles:function(e){var t={root:[{"class":"dxm-marker"},{"class":"dxm-marker dxm-marker-hovered"},{"class":"dxm-marker dxm-marker-selected"}]};return this._getStyles(t,e),t},setState:function(e,t,n){y(e,t,n,"root"),this._setState(e,t,n)},updateGrouping:function(e){xe(e),we(e)},getDefaultColor:function(e,t){return e.params.themeManager.getAccentColor(t)}},De[oe]=function(e){return{project:function(e,t){return t[0]&&t[0][0]&&t[0][0][0]&&"number"==typeof t[0][0][0][0]?k(e,t):C(e,t)}}},De[ae]=function(e){var t=e.coordinates;return{project:t[0]&&t[0][0]&&"number"==typeof t[0][0][0]?C:w}},Ie[re]={_default:"dot",dot:{setup:function(e){e.filter=e.renderer.shadowFilter("-40%","-40%","180%","200%",0,1,1,"#000000",.2)},reset:function(e){e.filter.dispose(),e.filter=null},_draw:function(e,t,n){t.back=e.renderer.circle().sharp().data(e.dataKey,n).append(t.root),t.dot=e.renderer.circle().sharp().data(e.dataKey,n).append(t.root)},refresh:function(e,t,n,i,o){t.dot.attr({filter:o.shadow?e.filter.id:null})},_getStyles:function(e,t){var n=t.size>0?pe(t.size):0,i=n,o=n+(t.selectedStep>0?pe(t.selectedStep):0),a=i+(t.backStep>0?pe(t.backStep):0),r=o+(t.backStep>0?pe(t.backStep):0),s=t.color||null,l=t.borderColor||null,u=m(t.borderWidth,null),c=m(t.opacity,null),d=t.backColor||null,h=m(t.backOpacity,null);e.dot=[{r:n/2,stroke:l,"stroke-width":u,fill:s,opacity:c},{r:i/2,stroke:t.hoveredBorderColor||l,"stroke-width":m(t.hoveredBorderWidth,u),fill:t.hoveredColor||s,opacity:m(t.hoveredOpacity,c)},{r:o/2,stroke:t.selectedBorderColor||l,"stroke-width":m(t.selectedBorderWidth,u),fill:t.selectedColor||s,opacity:m(t.selectedOpacity,c)}],e.back=[{r:n/2,stroke:"none","stroke-width":0,fill:d,opacity:h},{r:a/2,stroke:"none","stroke-width":0,fill:d,opacity:h},{r:r/2,stroke:"none","stroke-width":0,fill:d,opacity:h}]},_setState:function(e,t,n){y(e,t,n,"dot"),y(e,t,n,"back")}},bubble:{_draw:function(e,t,n){t.bubble=e.renderer.circle().sharp().data(e.dataKey,n).append(t.root)},refresh:function(e,t,n,i,o){t.bubble.attr({r:o.size/2})},_getStyles:function(e,t){var n=t.color||null,i=t.borderColor||null,o=m(t.borderWidth,null),a=m(t.opacity,null);e.bubble=[{stroke:i,"stroke-width":o,fill:n,opacity:a},{stroke:t.hoveredBorderColor||i,"stroke-width":m(t.hoveredBorderWidth,o),fill:t.hoveredColor||t.color,opacity:m(t.hoveredOpacity,a)},{stroke:t.selectedBorderColor||i,"stroke-width":m(t.selectedBorderWidth,o),fill:t.selectedColor||t.color,opacity:m(t.selectedOpacity,a)}]},_setState:function(e,t,n){y(e,t,n,"bubble")},arrange:function(e,t){var n,i=[],o=i.length=t.length,a=e.settings,r=a.dataField,s=a.minSize>0?pe(a.minSize):0,l=a.maxSize>s?pe(a.maxSize):s;if(!a.sizeGroups){for(n=0;n<o;++n)i[n]=ye(_(t[n].proxy,r)||0,0);var u=ve.apply(null,i),c=ye.apply(null,i),d=c-u||1,h=l-s;for(n=0;n<o;++n)t[n]._settings.size=s+h*(i[n]-u)/d}},updateGrouping:function(e){var t=e.settings.dataField;Se[re].updateGrouping(e),we(e,function(e){return _(e,t)})}},pie:{_draw:function(e,t,n){t.pie=e.renderer.g().append(t.root),t.border=e.renderer.circle().sharp().data(e.dataKey,n).append(t.root)},refresh:function(e,t,n,i,o){var a=_(i,e.settings.dataField)||[],r=o._colors,s=0,l=t.pie,u=e.renderer,c=e.dataKey,d=(o.size>0?pe(o.size):0)/2,h=90,f=h,p=!1;s=a.reduce(function(e,t){return e+(t||0)},0),0===s&&(p=!0,s=360/a.length),a.forEach(function(e,t){h=f,f+=p?s:(e||0)/s*360,u.arc(0,0,0,d,h,f).attr({"stroke-linejoin":"round",fill:r[t]}).data(c,n).append(l)}),t.border.attr({r:d})},_getStyles:function(e,t){var n=m(t.opacity,null),i=t.borderColor||null,o=m(t.borderWidth,null);e.pie=[{opacity:n},{opacity:m(t.hoveredOpacity,n)},{opacity:m(t.selectedOpacity,n)}],e.border=[{stroke:i,"stroke-width":o},{stroke:t.hoveredBorderColor||i,"stroke-width":m(t.hoveredBorderWidth,o)},{stroke:t.selectedBorderColor||i,"stroke-width":m(t.selectedBorderWidth,o)}]},_setState:function(e,t,n){y(e,t,n,"pie"),y(e,t,n,"border")},arrange:function(e,t){var n,i,o,a=t.length,r=e.settings.dataField,s=0;for(n=0;n<a;++n)i=_(t[n].proxy,r),i&&i.length>s&&(s=i.length);s>0&&(o=e.params.themeManager.createPalette(e.settings.palette,{useHighlight:!0,extensionMode:"alternate"}),i=o.generateColors(s),e.settings._colors=i,e.grouping.color={callback:ee,field:"",partition:[],values:[]},e.params.dataExchanger.set(e.name,"color",{partition:[],values:i}))}},image:{_draw:function(e,t,n){t.image=e.renderer.image(null,null,null,null,null,"center").attr({"pointer-events":"visible"}).data(e.dataKey,n).append(t.root)},refresh:function(e,t,n,i){t.image.attr({href:_(i,e.settings.dataField)})},_getStyles:function(e,t){var n=t.size>0?pe(t.size):0,i=n+(t.hoveredStep>0?pe(t.hoveredStep):0),o=n+(t.selectedStep>0?pe(t.selectedStep):0),a=m(t.opacity,null);e.image=[{x:-n/2,y:-n/2,width:n,height:n,opacity:a},{x:-i/2,y:-i/2,width:i,height:i,opacity:m(t.hoveredOpacity,a)},{x:-o/2,y:-o/2,width:o,height:o,opacity:m(t.selectedOpacity,a)}]},_setState:function(e,t,n){y(e,t,n,"image")}}};var Oe=function(e,t,n,i,o){var a;i&&t&&t.length>1&&(a=o(t.length-1),e.grouping[n]={callback:(0,X.isFunction)(i)?i:V,field:i,partition:t,values:a},e.params.dataExchanger.set(e.name,n,{partition:t,values:a,defaultColor:e.settings.color}))};xe=function(e){Oe(e,e.settings.colorGroups,"color",e.settings.colorGroupingField,function(t){var n,i=e.params.themeManager.createDiscretePalette(e.settings.palette,t),o=[];for(n=0;n<t;++n)o.push(i.getColor(n));return o})},we=function(e,t){var n=e.settings;Oe(e,n.sizeGroups,"size",t||n.sizeGroupingField,function(e){var t=n.minSize>0?pe(n.minSize):0,i=n.maxSize>=t?pe(n.maxSize):0,o=0,a=[];if(e>1)for(o=0;o<e;++o)a.push((t*(e-o-1)+i*o)/(e-1));else 1===e&&a.push((t+i)/2);return a})};var Ee,Ae=function(e,t,n,i){var o=this;o._params=e,o._onProjection(),o.proxy=z(o,n,i),o._context={name:n,layer:o.proxy,renderer:e.renderer,projection:e.projection,params:e,dataKey:e.dataKey,str:ke,hover:!1,selection:null,grouping:{},root:e.renderer.g().attr({"class":"dxm-layer"}).linkOn(t,n).linkAppend()},o._container=t,o._options={},o._handles=[],o._data=new s,o._dataSourceLoaded=null};Ae.prototype=te({constructor:Ae,getDataReadyCallback:function(){return this._dataSourceLoaded},_onProjection:function(){var e=this;e._removeHandlers=e._params.projection.on({engine:function(){e._project()},screen:function(){e._transform()},center:function(){e._transformCore()},zoom:function(){e._transform()}})},getData:function(){return this._data},_dataSourceLoadErrorHandler:function(){this._dataSourceChangedHandler()},_dataSourceChangedHandler:function(){var e=this;e._data=d(e._dataSource&&e._dataSource.items()),e._update(!0)},_dataSourceOptions:function(){return{paginate:!1}},_getSpecificDataSourceOption:function(){return this._specificDataSourceOption},_normalizeDataSource:function(e){var t=e.store();return"raw"===t._loadMode&&(t._loadMode=void 0),e},_offProjection:function(){this._removeHandlers(),this._removeHandlers=null},dispose:function(){var e=this;return e._disposeDataSource(),e._destroyHandles(),F(e._context),e._context.root.linkRemove().linkOff(),e._context.labelRoot&&e._context.labelRoot.linkRemove().linkOff(),e._context.str.reset(e._context),e._offProjection(),e._params=e._container=e._context=e.proxy=null,e},setOptions:function(e){var t=this;e=t._options=e||{},t._dataSourceLoaded=new Q.Deferred,"dataSource"in e&&e.dataSource!==t._options_dataSource?(t._options_dataSource=e.dataSource,t._params.notifyDirty(),t._specificDataSourceOption=h(e.dataSource),t._refreshDataSource()):t._data.count()>0&&(t._params.notifyDirty(),t._update(void 0!==e.type&&e.type!==t._context.str.type||void 0!==e.elementType&&e.elementType!==t._context.str.elementType)),t._transformCore()},_update:function(e){var t=this,n=t._context;e&&(n.str.reset(n),n.root.clear(),n.labelRoot&&n.labelRoot.clear(),t._params.tracker.reset(),t._destroyHandles(),n.str=Te(t._options,t._data),n.str.setup(n),t.proxy.type=n.str.type,t.proxy.elementType=n.str.elementType),n.settings=B(n,t._options),n.hasSeparateLabel=!(!n.settings.label.enabled||!n.str.hasLabelsGroup),n.hover=!!(0,J.parseScalar)(n.settings.hoverEnabled,!0),n.selection&&ne(n.selection.state,function(e,t){t&&t.resetSelected()}),n.selection=a(n.settings.selectionMode),n.hasSeparateLabel?n.labelRoot||(n.labelRoot=n.renderer.g().attr({"class":"dxm-layer-labels"}).linkOn(t._container,{name:n.name+"-labels",after:n.name}).linkAppend(),t._transformCore()):n.labelRoot&&(n.labelRoot.linkRemove().linkOff(),n.labelRoot=null),e&&t._createHandles(),F(n),n.str.arrange(n,t._handles),n.str.updateGrouping(n),t._updateHandles(),t._params.notifyReady(),t._dataSourceLoaded?(t._dataSourceLoaded.resolve(),t._dataSourceLoaded=null):t._params.dataReady()},getBounds:function(){return o(this._handles.map(function(e){var t=e.proxy;return t.coordinates().map(function(e){if(fe(e)){var t=fe(e[0][0])?e.reduce(function(e,t){return e.concat(t)},[]):e,n=t[0];return t.reduce(function(e,t){return[ve(e[0],t[0]),ve(e[1],t[1]),ye(e[2],t[0]),ye(e[3],t[1])]},[n[0],n[1],n[0],n[1]])}})}).map(o))},_destroyHandles:function(){this._handles.forEach(function(e){return e.dispose()}),this._context.selection&&(this._context.selection.state={}),this._handles=[]},_createHandles:function(){var e,t,n,i=this,o=i._handles=[],a=i._data,r=o.length=a.count(),s=i._context,l=a.geometry,u=a.attributes;for(e=0;e<r;++e)n=a.item(e),o[e]=new Ee(s,e,l(n),u(n));for((0,X.isFunction)(i._options.customize)&&f(i.getProxies(),i._options.customize,i._params.widget),e=0;e<r;++e)t=o[e],t.project(),t.draw(),t.transform();s.selection&&ne(s.selection.state,function(e,t){t&&t.restoreSelected()})},_updateHandles:function(){var e,t=this._handles,n=t.length;for(e=0;e<n;++e)t[e].refresh();if(this._context.settings.label.enabled){for(e=0;e<n;++e)t[e].measureLabel();for(e=0;e<n;++e)t[e].adjustLabel()}},_transformCore:function(){var e=this._params.projection.getTransform();this._context.root.attr(e),this._context.labelRoot&&this._context.labelRoot.attr(e)},_project:function(){var e,t=this._handles,n=t.length;for(e=0;e<n;++e)t[e].project()},_transform:function(){var e,t=this._handles,n=t.length;for(this._transformCore(),e=0;e<n;++e)t[e].transform()},getProxies:function(){return this._handles.map(function(e){return e.proxy})},getProxy:function(e){return this._handles[e].proxy},raiseClick:function(e,t){this._params.eventTrigger("click",{target:this._handles[e].proxy,event:t})},hoverItem:function(e,t){this._handles[e].setHovered(t)},selectItem:function(e,t,n){this._handles[e].setSelected(t,n)},clearSelection:function(){var e=this._context.selection;e&&(ne(e.state,function(e,t){t&&t.setSelected(!1)}),e.state={})}},Z.default),Ee=function(e,t,n,i){var o=this,a=o.proxy=N(o,n.coordinates,te({},i));o._ctx=e,o._index=t,o._fig=o._label=null,o._state=se,o._coordinates=n.coordinates,o._settings={label:{}},a.index=t,a.layer=e.layer,o._data={name:e.name,index:t}},Ee.prototype={constructor:Ee,dispose:function(){var e=this;return e._ctx=e.proxy=e._settings=e._fig=e._label=e.data=null,e},project:function(){var e=this._ctx;this._projection=e.str.project(e.projection,this._coordinates),e.hasSeparateLabel&&this._label&&this._projectLabel()},_projectLabel:function(){this._labelProjection=this._ctx.str.projectLabel(this._projection)},draw:function(){var e=this,t=this._ctx;t.str.draw(t,e._fig={},e._data),e._fig.root.append(t.root)},transform:function(){var e=this,t=e._ctx;t.str.transform(e._fig,t.projection,e._projection),t.hasSeparateLabel&&e._label&&e._transformLabel()},_transformLabel:function(){this._ctx.str.transformLabel(this._label,this._ctx.projection,this._labelProjection)},refresh:function(){var e=this,t=e._ctx.str,n=E(e._ctx,e.proxy,e._settings);e._styles=t.getStyles(n),t.refresh(e._ctx,e._fig,e._data,e.proxy,n),e._refreshLabel(n),e._setState()},_refreshLabel:function(e){var t=this,n=t._ctx,i=e.label,o=t._label;n.settings.label.enabled?(o||(o=t._label={root:n.labelRoot||t._fig.root,text:n.renderer.text().attr({"class":"dxm-label"}),size:[0,0]},n.hasSeparateLabel&&(t._projectLabel(),t._transformLabel())),o.value=ge(t.proxy.text||t.proxy.attribute(i.dataField)||""),o.value&&(o.text.attr({text:o.value,x:0,y:0}).css((0,J.patchFontOptions)(i.font)).attr({align:"center",stroke:i.stroke,"stroke-width":i["stroke-width"],"stroke-opacity":i["stroke-opacity"]}).data(n.dataKey,t._data).append(o.root),o.settings=e)):o&&(o.text.remove(),t._label=null)},measureLabel:function(){var e,t=this._label;t.value&&(e=t.text.getBBox(),t.size=[e.width,e.height,-e.y-e.height/2])},adjustLabel:function(){var e,t=this._label;t.value&&(e=this._ctx.str.getLabelOffset(t,t.settings),t.settings=null,t.text.attr({x:e[0],y:e[1]+t.size[2]}))},update:function(e){var t=this;t._settings=M(t._settings,e),t._fig&&(t.refresh(),t._label&&t._label.value&&(t.measureLabel(),t.adjustLabel()))},_setState:function(){this._ctx.str.setState(this._fig,this._styles,ce[this._state])},_setForeground:function(){var e=this._fig.root;this._state?e.toForeground():e.toBackground()},setHovered:function(e){var t=this,n=H(t._state,le),i=!!e;return t._ctx.hover&&n!==i&&(t._state=L(t._state,le,i),t._setState(),t._setForeground(),R(t._ctx,t,i,"hoverChanged")),t},setSelected:function(e,t){var n,i=this,o=H(i._state,ue),a=!!e,r=i._ctx.selection;r&&o!==a&&(i._state=L(i._state,ue,a),n=r.state[r.single],r.state[r.single]=null,n&&n.setSelected(!1),r.state[r.single||i._index]=e?i:null,i._fig&&(i._setState(),i._setForeground(),t||R(i._ctx,i,a,"selectionChanged")))},isSelected:function(){return H(this._state,ue)},resetSelected:function(){this._state=L(this._state,ue,!1)},restoreSelected:function(){this._fig.root.toForeground()}},$.prototype={constructor:$,dispose:function(){var e=this;e._clip.dispose(),e._layers.forEach(function(e){return e.dispose()}),e._offTracker(),e._params=e._offTracker=e._layers=e._layerByName=e._clip=e._background=e._container=null},_subscribeToTracker:function(e,t,n){var i=this;i._offTracker=e.on({click:function(e){var o=t.getRootOffset(),a=i.byName(e.data.name);e.$event.x=e.x-o.left,e.$event.y=e.y-o.top,a?a.raiseClick(e.data.index,e.$event):"background"===e.data.name&&n("click",{event:e.$event})},"hover-on":function(e){var t=i.byName(e.data.name);t&&t.hoverItem(e.data.index,!0)},"hover-off":function(e){var t=i.byName(e.data.name);t&&t.hoverItem(e.data.index,!1)}})},setOptions:function(e){var t=this,n=e?fe(e)?e:[e]:[],i=t._layerByName,o=t._layers,a=[],s=n.length!==o.length||o.some(function(e,t){var i=r(n,t);return(0,X.isDefined)(i)&&i!==e.proxy.name});if(s){t._params.tracker.reset(),t._layers.forEach(function(e){return e.dispose()}),t._layerByName=i={},t._layers=o=[];for(var l=0,u=n.length;l<u;++l){var c=r(n,l)||"map-layer-"+l,d=o[l]=new Ae(t._params,t._container,c,l);i[c]=d}}o.forEach(function(e,t){e.setOptions(n[t])}),a=o.map(function(e){return e.getDataReadyCallback()}),a.length&&Q.when.apply(void 0,a).done(t._dataReady)},_updateClip:function(){var e=this._rect,t=this._borderWidth;this._clip.attr({x:e[0]+t,y:e[1]+t,width:ye(e[2]-2*t,0),height:ye(e[3]-2*t,0)})},setBackgroundOptions:function(e){this._background.attr({stroke:e.borderColor,"stroke-width":e.borderWidth,fill:e.color}),this._borderWidth=ye(e.borderWidth,0),this._updateClip()},setRect:function(e){this._rect=e,this._background.attr({x:e[0],y:e[1],width:e[2],height:e[3]}),this._updateClip()},byIndex:function(e){return this._layers[e]},byName:function(e){return this._layerByName[e]},items:function(){return this._layers}}},function(e,t,n){function i(e){this._subscribeToTracker(e.tracker,e.tooltip,e.layerCollection)}t.TooltipViewer=i;var o=12;i.prototype={constructor:i,dispose:function(){this._offTracker(),this._offTracker=null},_subscribeToTracker:function(e,t,n){this._offTracker=e.on({"focus-on":function(e){var i,a;if(t.isEnabled()){i=n.byName(e.data.name),a=i&&i.getProxy(e.data.index);var r=function(t){t&&e.done(t)};a&&r(t.show(a,{x:e.x,y:e.y,offset:o},{target:a},void 0,r))}},"focus-move":function(e){t.move(e.x,e.y,o)},"focus-off":function(){t.hide()}})}}},function(e,t,n){function i(){return"vectormap-data-"+o++}t.generateDataKey=i;var o=1},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(975)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i,o=e[0]||{},a=o[t]||0,r=a,s=a,l=[0],u=[0],c=e.length;for(i=1;i<c;i++)n=e[i][t],n<r?(r=n,l=[i]):n===r&&l.push(i),n>s?(s=n,u=[i]):n===s&&u.push(i);return s===r&&(l=u=[]),{minIndexes:l,maxIndexes:u}}function a(e,t,n,i){return(0,f.map)(e,function(e,o){var a,r,s=null;return void 0!==e&&(s={},a=O(e),s[t]=a?A(o):e[t],r=a?e:e[n],s[n]=null===r?i?void 0:r:E(r),s=void 0!==s[t]&&void 0!==s[n]?s:null),s})}function r(e,t,n,i){var o=-1,a=0,r=1,s=1e-4;return(0,f.map)(e,function(e){var l={};return l[t]=e[t],S(e[n]-i)<s?l[n]=a:e[n]>i?l[n]=r:l[n]=o,l})}function s(e,t,n,i){return n!==i.first&&n!==i.last||(e=t.firstLastColor),(i.min||[]).indexOf(n)>=0&&(e=t.minColor),(i.max||[]).indexOf(n)>=0&&(e=t.maxColor),e}function l(e,t){return function(){var n=s(void 0,t,this.index,e);return n?{visible:!0,border:{color:n}}:{}}}function u(e,t,n){return function(){var i=this.index,o="winloss"===t.type,a=o?t.winlossThreshold:0,r=o?n[i][t.valueField]:this.value,l=o?t.winColor:t.barPositiveColor,u=o?t.lossColor:t.barNegativeColor;return{color:s(r>=a?l:u,t,i,e)}}}t.default=void 0;var c=i(n(564)),d=n(324),h=n(326),f=n(14),p=n(1),g=i(n(9)),_=n(174),m=1,v=50,y=4,b=250,x=30,w=2,C={line:!0,spline:!0,stepline:!0,area:!0,steparea:!0,splinearea:!0,bar:!0,winloss:!0},k=Math,S=k.abs,D=k.round,I=k.max,T=k.min,O=isFinite,E=Number,A=String,P=c.default.inherit({_rootClassPrefix:"dxsl",_rootClass:"dxsl-sparkline",_themeSection:"sparkline",_defaultSize:{width:b,height:x},_initCore:function(){this.callBase(),this._createSeries()},_initialChanges:["DATA_SOURCE"],_dataSourceChangedHandler:function(){this._requestChange(["UPDATE"])},_updateWidgetElements:function(){this._updateSeries(),this.callBase()},_disposeWidgetElements:function(){var e=this;e._series&&e._series.dispose(),e._series=e._seriesGroup=e._seriesLabelGroup=null},_cleanWidgetElements:function(){this._seriesGroup.remove(),this._seriesLabelGroup.remove(),this._seriesGroup.clear(),this._seriesLabelGroup.clear()},_drawWidgetElements:function(){this._dataIsLoaded()&&(this._drawSeries(),this._drawn())},_getCorrectCanvas:function(){var e=this._allOptions,t=this._canvas,n=e.pointSize&&Math.ceil(e.pointSize/2)+w,i=e.type;return"bar"!==i&&"winloss"!==i&&(e.showFirstLast||e.showMinMax)?{width:t.width,height:t.height,left:t.left+n,right:t.right+n,top:t.top+n,bottom:t.bottom+n}:t},_prepareOptions:function(){var e=this;e._allOptions=e.callBase(),e._allOptions.type=(0,f.normalizeEnum)(e._allOptions.type),C[e._allOptions.type]||(e._allOptions.type="line")},_createHtmlElements:function(){this._seriesGroup=this._renderer.g().attr({"class":"dxsl-series"}),this._seriesLabelGroup=this._renderer.g().attr({"class":"dxsl-series-labels"})},_createSeries:function(){this._series=new h.Series({renderer:this._renderer,seriesGroup:this._seriesGroup,labelsGroup:this._seriesLabelGroup,argumentAxis:this._argumentAxis,valueAxis:this._valueAxis},{widgetType:"chart",type:"line"})},_updateSeries:function(){var e=this,t=e._series;e._prepareDataSource();var n=e._prepareSeriesOptions();t.updateOptions(n);var i={groups:[{series:[t]}]};i.argumentOptions={type:"bar"===n.type?"discrete":void 0},e._simpleDataSource=(0,d.validateData)(e._simpleDataSource,i,e._incidentOccurred,{checkTypeForAllData:!1,convertToAxisDataType:!0,sortingMethod:!0})[t.getArgumentField()],n.customizePoint=e._getCustomizeFunction(),t.updateData(e._simpleDataSource),t.createPoints(),e._groupsDataCategories=i.categories},_optionChangesMap:{dataSource:"DATA_SOURCE"},_optionChangesOrder:["DATA_SOURCE"],_change_DATA_SOURCE:function(){this._updateDataSource()},_prepareDataSource:function(){var e=this,t=e._allOptions,n=t.argumentField,i=t.valueField,o=e._dataSourceItems()||[],s=a(o,n,i,e.option("ignoreEmptyPoints"));"winloss"===t.type?(e._winlossDataSource=s,e._simpleDataSource=r(s,n,i,t.winlossThreshold)):e._simpleDataSource=s},_prepareSeriesOptions:function(){var e=this,t=e._allOptions,n="winloss"===t.type?"bar":t.type;return{visible:!0,argumentField:t.argumentField,valueField:t.valueField,color:t.lineColor,width:t.lineWidth,widgetType:"chart",type:n,opacity:n.indexOf("area")!==-1?e._allOptions.areaOpacity:void 0,point:{size:t.pointSize,symbol:t.pointSymbol,border:{visible:!0,width:w},color:t.pointColor,visible:!1,hoverStyle:{border:{}},selectionStyle:{border:{}}},border:{color:t.lineColor,width:t.lineWidth,visible:"bar"!==n}}},_getCustomizeFunction:function(){var e,t=this,n=t._allOptions,i=t._winlossDataSource||t._simpleDataSource,o=t._getExtremumPointsIndexes(i);return e="winloss"===n.type||"bar"===n.type?u(o,n,t._winlossDataSource):l(o,n)},_getExtremumPointsIndexes:function(e){var t=this,n=t._allOptions,i=e.length-1,a={};return t._minMaxIndexes=o(e,n.valueField),n.showFirstLast&&(a.first=0,a.last=i),n.showMinMax&&(a.min=t._minMaxIndexes.minIndexes,a.max=t._minMaxIndexes.maxIndexes),a},_getStick:function(){return{stick:"bar"!==this._series.type}},_updateRange:function(){var e,t=this,n=t._series,i=n.type,o="bar"===i,a="winloss"===i,r=.15,s=.1,l=1,u=-1,c=n.getRangeData(),d=t._allOptions.minValue,h=(0,p.isDefined)(d)&&O(d),f=t._allOptions.maxValue,g=(0,p.isDefined)(f)&&O(f),_=(c.val.max-c.val.min)*r;o||a||"area"===i?(0!==c.val.min&&(c.val.min-=_),0!==c.val.max&&(c.val.max+=_)):(c.val.min-=_,c.val.max+=_),(h||g)&&(h&&g?(c.val.minVisible=T(d,f),c.val.maxVisible=I(d,f)):(c.val.minVisible=h?E(d):void 0,c.val.maxVisible=g?E(f):void 0),a&&(c.val.minVisible=h?I(c.val.minVisible,u):void 0,c.val.maxVisible=g?T(c.val.maxVisible,l):void 0)),n.getPoints().length>1&&o&&(e=(c.arg.max-c.arg.min)*s,c.arg.min=c.arg.min-e,c.arg.max=c.arg.max+e),c.arg.categories=t._groupsDataCategories,t._ranges=c},_getBarWidth:function(e){var t=this,n=t._canvas,i=e*y,o=n.width-n.left-n.right-i,a=D(o/e);return a<m&&(a=m),a>v&&(a=v),a},_correctPoints:function(){var e,t,n=this,i=n._allOptions.type,o=n._series.getPoints(),a=o.length;if("bar"===i||"winloss"===i)for(e=n._getBarWidth(a),t=0;t<a;t++)o[t].correctCoordinates({width:e,offset:0})},_drawSeries:function(){
var e=this;e._simpleDataSource.length>0&&(e._correctPoints(),e._series.draw(),e._seriesGroup.append(e._renderer.root))},_isTooltipEnabled:function(){return!!this._simpleDataSource.length},_getTooltipData:function(){var e=this,t=e._allOptions,n=e._winlossDataSource||e._simpleDataSource,i=e._tooltip;if(0===n.length)return{};var o=e._minMaxIndexes,a=t.valueField,r=n[0][a],s=n[n.length-1][a],l=(0,p.isDefined)(o.minIndexes[0])?n[o.minIndexes[0]][a]:r,u=(0,p.isDefined)(o.maxIndexes[0])?n[o.maxIndexes[0]][a]:r,c=i.formatValue(r),d=i.formatValue(s),h=i.formatValue(l),f=i.formatValue(u),g={firstValue:c,lastValue:d,minValue:h,maxValue:f,originalFirstValue:r,originalLastValue:s,originalMinValue:l,originalMaxValue:u,valueText:["Start:",c,"End:",d,"Min:",h,"Max:",f]};return"winloss"===t.type&&(g.originalThresholdValue=t.winlossThreshold,g.thresholdValue=i.formatValue(t.winlossThreshold)),g}});(0,f.map)(["lineColor","lineWidth","areaOpacity","minColor","maxColor","barPositiveColor","barNegativeColor","winColor","lessColor","firstLastColor","pointSymbol","pointColor","pointSize","type","argumentField","valueField","winlossThreshold","showFirstLast","showMinMax","ignoreEmptyPoints","minValue","maxValue"],function(e){P.prototype._optionChangesMap[e]="OPTIONS"}),(0,g.default)("dxSparkline",P);var R=P;t.default=R,P.addPlugin(_.plugin),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(977)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=n(3),a=i(n(564)),r=i(n(9)),s=.02,l=.98,u=.1,c=.9,d=300,h=30,f=1,p=2,g=Number,_=isFinite,m=a.default.inherit({_rootClassPrefix:"dxb",_rootClass:"dxb-bullet",_themeSection:"bullet",_defaultSize:{width:d,height:h,left:f,right:f,top:p,bottom:p},_disposeWidgetElements:function(){delete this._zeroLevelPath,delete this._targetPath,delete this._barValuePath},_cleanWidgetElements:function(){this._zeroLevelPath.remove(),this._targetPath.remove(),this._barValuePath.remove()},_drawWidgetElements:function(){this._drawBullet(),this._drawn()},_createHtmlElements:function(){var e=this._renderer;this._zeroLevelPath=e.path(void 0,"line").attr({"class":"dxb-zero-level","stroke-linecap":"square"}),this._targetPath=e.path(void 0,"line").attr({"class":"dxb-target","stroke-linecap":"square"}),this._barValuePath=e.path(void 0,"line").attr({"class":"dxb-bar-value","stroke-linecap":"square"})},_prepareOptions:function(){var e,t,n,i,o,a,r=this;r._allOptions=e=r.callBase();var s=void 0===r._allOptions.value,l=void 0===r._allOptions.target;r._tooltipEnabled=!(s&&l),s&&(r._allOptions.value=0),l&&(r._allOptions.target=0),e.value=o=g(e.value),e.target=a=g(e.target),void 0===r._allOptions.startScaleValue&&(r._allOptions.startScaleValue=a<o?a:o,r._allOptions.startScaleValue=r._allOptions.startScaleValue<0?r._allOptions.startScaleValue:0),void 0===r._allOptions.endScaleValue&&(r._allOptions.endScaleValue=a>o?a:o),e.startScaleValue=t=g(e.startScaleValue),e.endScaleValue=n=g(e.endScaleValue),n<t&&(i=n,r._allOptions.endScaleValue=t,r._allOptions.startScaleValue=i,r._allOptions.inverted=!0)},_updateRange:function(){var e=this,t=e._allOptions;e._ranges={arg:{invert:t.rtlEnabled?!t.inverted:t.inverted,min:t.startScaleValue,max:t.endScaleValue,axisType:"continuous",dataType:"numeric"},val:{min:0,max:1,axisType:"continuous",dataType:"numeric"}}},_drawBullet:function(){var e=this,t=e._allOptions,n=t.startScaleValue!==t.endScaleValue,i=_(t.startScaleValue),o=_(t.endScaleValue),a=_(t.value),r=_(t.target);n&&o&&i&&r&&a&&(this._drawBarValue(),this._drawTarget(),this._drawZeroLevel())},_getTargetParams:function(){var e=this,t=e._allOptions,n=e._valueAxis.getTranslator(),i=e._argumentAxis.getTranslator().translate(t.target);return{points:[i,n.translate(s),i,n.translate(l)],stroke:t.targetColor,"stroke-width":t.targetWidth}},_getBarValueParams:function(){var e,t,n=this,i=n._allOptions,o=n._argumentAxis.getTranslator(),a=n._valueAxis.getTranslator(),r=i.startScaleValue,s=i.endScaleValue,l=i.value,d=a.translate(u),h=a.translate(c);return l>0?(e=r<=0?0:r,t=l>=s?s:l<e?e:l):(e=s>=0?0:s,t=l<r?r:l>e?e:l),e=o.translate(e),t=o.translate(t),{points:[e,h,t,h,t,d,e,d],fill:i.color}},_getCorrectCanvas:function(){return this._canvas},_getZeroLevelParams:function(){var e=this,t=e._valueAxis.getTranslator(),n=e._argumentAxis.getTranslator().translate(0);return{points:[n,t.translate(s),n,t.translate(l)],stroke:e._allOptions.targetColor,"stroke-width":1}},_drawZeroLevel:function(){var e=this,t=e._allOptions;0>t.endScaleValue||0<t.startScaleValue||!t.showZeroLevel||e._zeroLevelPath.attr(e._getZeroLevelParams()).sharp().append(e._renderer.root)},_drawTarget:function(){var e=this,t=e._allOptions,n=t.target;n>t.endScaleValue||n<t.startScaleValue||!t.showTarget||e._targetPath.attr(e._getTargetParams()).sharp().append(e._renderer.root)},_drawBarValue:function(){this._barValuePath.attr(this._getBarValueParams()).append(this._renderer.root)},_getTooltipCoords:function(){var e=this._canvas,t=this._renderer.getRootOffset(),n=this._barValuePath.getBBox();return{x:n.x+n.width/2+t.left,y:e.height/2+t.top}},_getTooltipData:function(){var e=this,t=e._tooltip,n=e._allOptions,i=n.value,o=n.target,a=t.formatValue(i),r=t.formatValue(o);return{originalValue:i,originalTarget:o,value:a,target:r,valueText:["Actual Value:",a,"Target Value:",r]}},_isTooltipEnabled:function(){return this._tooltipEnabled}});(0,o.each)(["color","targetColor","targetWidth","showTarget","showZeroLevel","value","target","startScaleValue","endScaleValue"],function(e,t){m.prototype._optionChangesMap[t]="OPTIONS"}),(0,r.default)("dxBullet",m);var v=m;t.default=v,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(979)),a=o.default;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(113));n(565),n(980),n(981),n(982),n(567),n(983),n(984),n(176),n(568),n(985),n(570),n(986),n(987),n(988);var a=n(111),r=n(131),s=n(153),l=o.default;t.default=l,o.default.addPlugin(a.plugin),o.default.addPlugin(r.plugin),o.default.addPlugin(s.plugin),e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t,n){return((n-1)*e+t)/n}function a(e){return(0,r.default)(e,o,!0)}t.default=void 0;var r=i(n(566)),s=n(175);(0,s.addAlgorithm)("strip",a);var l=a;t.default=l,e.exports=t.default},function(e,t,n){function i(e){var t=e.items,n=(0,o.buildSidesData)(e.rect,e.directions,e.isRotated?1:0);(0,o.calculateRectangles)(t,0,e.rect,n,{sum:e.sum,count:t.length,side:n.variedSide})}t.default=void 0;var o=n(175);(0,o.addAlgorithm)("sliceanddice",i);var a=i;t.default=a,e.exports=t.default},function(e,t,n){function i(e){return e.isRotated=!e.isRotated,a.call(this,e)}var o=n(175),a=(0,o.getAlgorithm)("sliceanddice");(0,o.addAlgorithm)("rotatedsliceanddice",i)},function(e,t,n){function i(e,t){return function(n){return e(n,t)}}function o(e){return[Number(e[0])||0,e[1]-e[0]||1]}function a(e,t){var n,i,a=e.length,r=[];for(n=0;n<a;++n)i=t(e[n]),isFinite(i)&&r.push(i);return o([u.apply(null,r),c.apply(null,r)])}function r(e,t){var n={};return function(i){var o=i.parent;return e(i,n[o._id]||(n[o._id]=a(o.nodes,t)))}}function s(e,t){function n(e,t){return a.getColor((s(e)-t[0])/t[1])}var a=t.createGradientPalette(e.palette),s=(0,l.createColorCodeGetter)(e);return"range"in e?i(n,o(e.range||[])):r(n,s)}t.default=void 0;var l=n(235),u=Math.min,c=Math.max;(0,l.addColorizer)("gradient",s);var d=s;t.default=d,e.exports=t.default},function(e,t,n){function i(e,t){var n,i=0,o=t.length-1,a=-1;if(t[i]<=e&&e<=t[o])if(e===t[o])a=o-1;else{for(;o-i>1;)n=i+o>>1,e<t[n]?o=n:i=n;a=i}return a}function o(e,t){var n=e.range||[],o=t.createDiscretePalette(e.palette,n.length-1),r=(0,a.createColorCodeGetter)(e);return function(e){return o.getColor(i(r(e),n))}}t.default=void 0;var a=n(235);(0,a.addColorizer)("range",o);var r=o;t.default=r,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i,o=t.length;for(n=0;n<o;++n)i=e[t[n]],y[Number(i.isNode())](i.tile)}function a(e){e.toForeground()}function r(e){e.outer.toForeground(),e.inner.toForeground()}var s=i(n(113)),l=i(n(204)),u=n(172),c=n(331),d=n(14),h=n(12);n(176),n(569);var f=s.default.prototype,p=l.default.prototype,g=0,_=1,m=2,v=2;f._eventsMap.onSelectionChanged={name:"selectionChanged"},(0,u.expand)(f._handlers,"calculateAdditionalStates",function(e,t){e[2]=t.selectionStyle?(0,c.buildRectAppearance)(t.selectionStyle):{}}),p.statesMap[2]=p.statesMap[3]=v,p.additionalStates.push(2),(0,u.expand)(f,"_onNodesCreated",function(){this._selectionList.length=0}),(0,u.expand)(f,"_extendProxyType",function(e){var t=this;e.select=function(e){t._selectNode(this._id,!!e)},e.isSelected=function(){return(0,h.inArray)(this._id,t._selectionList)>=0},t._selectionList=[]}),s.default.addChange({code:"SELECTION_MODE",handler:function(){var e,t=this,n=(0,d.normalizeEnum)(t._getOption("selectionMode",!0)),i=t._selectionList,o="none"===n?g:"multiple"===n?m:_;o===_&&i.length>1?(e=i.pop(),t.clearSelection(),i.push(e)):o===g&&t.clearSelection(),t._selectionMode=o},isThemeDependent:!0,isOptionChange:!0,option:"selectionMode"}),(0,u.expand)(f,"_applyTilesAppearance",function(){this._selectionList.length&&o(this._nodes,this._selectionList)});var y=[a,r];f._applySelectionState=function(e,t){var n=this._nodes[e];n.setState(v,t),this._eventTrigger("selectionChanged",{node:n.proxy})},f._selectNode=function(e,t){var n,i,o,a=this;a._selectionMode!==g&&(a._context.suspend(),n=a._selectionList,i=(0,h.inArray)(e,n),t&&i===-1?(a._selectionMode===_&&n.length&&(o=n.pop(),a._applySelectionState(o,!1)),n.push(e),a._applySelectionState(e,!0)):!t&&i>=0&&(n.splice(i,1),a._applySelectionState(e,!1)),a._context.resume())},f.clearSelection=function(){var e,t=this,n=t._selectionList,i=n.length;if(t._selectionMode!==g){for(t._context.suspend(),e=0;e<i;++e)t._applySelectionState(n[e],!1);n.length=0,t._context.resume()}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(113)),a=n(332),r=n(172),s=n(14);n(176),n(568),n(570);var l="__treemap_data_",u=0,c=o.default.prototype;c._eventsMap.onClick={name:"click"};var d=function(){var e=l+u++;return e};(0,r.expand)(c,"_initCore",function(){var e=this,t=d(),n=function(t){return e._nodes[t].proxy};e._tracker=new a.Tracker({widget:e,root:e._renderer.root,getNode:function(t){var i=n(t),o=(0,s.parseScalar)(e._getOption("interactWithGroup",!0));return o&&i.isLeaf()&&i.getParent().isActive()?i.getParent():i},getData:function(e){var n=e.target;return("tspan"===n.tagName?n.parentNode:n)[t]},getProxy:n,click:function(t){e._eventTrigger("click",t)}}),e._handlers.setTrackerData=function(e,n){n.data(t,e._id)}}),(0,r.expand)(c,"_disposeCore",function(){this._tracker.dispose()})},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}var o=i(n(113)),a=n(172);n(176);var r=o.default.prototype;r._eventsMap.onDrill={name:"drill"},(0,a.expand)(r,"_extendProxyType",function(e){var t=this;e.drillDown=function(){t._drillToNode(this._id)}}),(0,a.expand)(r,"_onNodesCreated",function(){this._drilldownIndex=-1}),r._drillToNode=function(e){var t,n=this;n._drilldownIndex!==e&&(t=n._nodes[e]||n._root,t.nodes&&(n._drilldownIndex=e,n._topNode=t,n._context.suspend(),n._context.change(["MAX_DEPTH","NODES_RESET"]),n._context.resume(),n._eventTrigger("drill",{node:t.proxy})))},r.resetDrillDown=function(){return this._drillToNode(-1),this},r.drillUp=function(){return this._drillToNode(this._topNode.parent._id||-1),this},r.getCurrentNode=function(){return this._topNode.proxy}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}function o(e,t){var n,i,a,r=e.struct;for(i=0;i<t.length;i++)n=t[i],a=n[e.idField],r[a]&&(n.items=r[a].items,r[a]=null,o(e,n.items))}var a=i(n(113)),r=a.default.prototype;r._optionChangesMap.idField=r._optionChangesMap.parentField="NODES_CREATE",r._processDataSourceItems=function(e){var t,n,i,a,r,s={},l=this._getOption("idField",!0),u=this._getOption("parentField",!0),c=[];if(!l||!u||0===e.length)return{items:e,isPlain:!0};for(t=0;t<e.length;t++)n=e[t],i=n[u],i?(s[i]=s[i]||{items:[]},a=s[i].items):a=c,a.push(n);o({struct:s,idField:l},c);for(r in s)s[r]&&c.push(s[r]);return{items:c,isPlain:!0}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(571)),a=n(994),r=n(111),s=n(131),l=n(228),u=n(995),c=n(996),d=n(153);o.default.addPlugin(a.plugin),o.default.addPlugin(r.plugin),o.default.addPlugin(s.plugin),o.default.addPlugin(l.plugin),o.default.addPlugin(u.plugin),o.default.addPlugin(c.plugin),o.default.addPlugin(d.plugin);var h=o.default;t.default=h,e.exports=t.default},function(e,t,n){function i(e){return s[(0,r.normalizeEnum)(e)]||a}function o(e,t,n){s[e]=t,n&&(a=s[e])}t.getAlgorithm=i,t.addAlgorithm=o;var a,r=n(14),s={}},function(e,t,n){t.default=void 0;var i=.5,o={getFigures:function(e){var t=1/e.length;return e.map(function(e,n,o){var a=o[n+1]?o[n+1]:o[n];return[i-e/2,t*n,i+e/2,t*n,i+a/2,t*(n+1),i-a/2,t*(n+1)]})},normalizeValues:function(e){var t=e.reduce(function(e,t){return Math.max(t.value,e)},e[0]&&e[0].value||0);return e.map(function(e){return e.value/t})}};t.default=o,e.exports=t.default},function(e,t,n){t.default=void 0;var i=.5,o=0,a=1,r={getFigures:function(e,t,n){var r=0,s=0,l=0,u=0,c=t/2,d=i-c,h=1-n,f=i-c,p=i+c;return e.map(function(e){return l=u,s=r,r+=e,u=d*r/h,s<=h&&r<=h?[l,s,a-l,s,a-u,r,o+u,r]:s<=h&&r>h?[l,s,a-l,s,p,h,p,r,f,r,f,h]:[f,s,p,s,p,r,f,r]})},normalizeValues:function(e){var t=e.reduce(function(e,t){return e+t.value},0);return e.map(function(e){return e.value/t})}};t.default=r,e.exports=t.default},function(e,t,n){function i(e,t,n){var i=t.border,o=n.border,r=(0,a.isDefined)(i.visible)?i.visible:o.visible,s=(0,a.isDefined)(i.width)?i.width:o.width;return{fill:e,hatching:t.hatching,stroke:i.color||o.color,"stroke-width":r?s:0}}function o(e,t){var n=this,o=t.data;n.code=0,n.widget=e,n.figure=t.figure,n.argument=o.argument,n.value=o.value,n.data=o.dataItem,n.percent=t.percent,n.id=t.id,n.color=t.color,n.states={normal:i(t.color,t.itemOptions,t.itemOptions),hover:i(t.color,t.itemOptions.hoverStyle,t.itemOptions),selection:i(t.color,t.itemOptions.selectionStyle,t.itemOptions)}}t.default=void 0;var a=n(1),r=["normal","hover","selection","selection"];o.prototype={getState:function(){return r[this.code]},getNormalStyle:function(){return this.states.normal},setHover:function(){this.hover(!0)},hover:function(e){this.widget._getOption("hoverEnabled",!0)&&e!==this.isHovered()&&(this.widget._suspend(),e&&this.widget.clearHover(),this.setState(1,e),this.widget._eventTrigger("hoverChanged",{item:this}),this.widget._resume())},setState:function(e,t){t?this.code|=e:this.code&=~e,this.widget._applyTilesAppearance()},select:function(e){var t=this.widget._getOption("selectionMode",!0);"none"!==t&&e!==this.isSelected()&&(this.widget._suspend(),e&&"multiple"!==t&&this.widget.clearSelection(),this.setState(2,e),this.widget._eventTrigger("selectionChanged",{item:this}),this.widget._resume())},showTooltip:function(e){this.widget._showTooltip(this.id,e)},getColor:function(){return this.color},isHovered:function(){return!!(1&this.code)},isSelected:function(){return!!(2&this.code)}};var s=o;t.default=s,e.exports=t.default},function(e,t,n){function i(e,t){return r(e)||a(e,t)||u(e,t)||o()}function o(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],i=!0,o=!1,a=void 0;try{for(var r,s=e[Symbol.iterator]();!(i=(r=s.next()).done)&&(n.push(r.value),!t||n.length!==t);i=!0);}catch(e){o=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(o)throw a}}return n}}function r(e){if(Array.isArray(e))return e}function s(e){return d(e)||c(e)||u(e)||l()}function l(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function u(e,t){if(e){if("string"==typeof e)return h(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?h(e,t):void 0}}function c(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function d(e){if(Array.isArray(e))return h(e)}function h(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,i=new Array(t);n<t;n++)i[n]=e[n];return i}function f(e){return e=(0,D.normalizeEnum)(e),e===O?A:e===E?0:P}function p(e){return e=(0,D.normalizeEnum)(e),e===O||e!==E}function g(e,t,n){return n?e-t.height:e}function _(e,t,n,i){return{x:e[2]+n.horizontalOffset+A,y:g(e[3]+n.verticalOffset,t,i)}}function m(e,t,n,i){return{x:e[0]-t.width-n.horizontalOffset-A,y:g(e[1]+n.verticalOffset,t,i)}}function v(e,t,n){var i=e[2]-e[0],o=e[7]-e[1];return{x:e[0]+i/2+n.horizontalOffset-t.width/2,y:e[1]+n.verticalOffset+o/2-t.height/2}}function y(e,t,n){return function(i,o,a,r){return{x:"left"===n?t[2]+a.horizontalOffset+P:e[2]-o.width,y:g(i[3]+a.verticalOffset,o,r)}}}function b(e,t,n){return function(i,o,a,r){return{x:"left"===n?e[0]:t[0]-o.width-a.horizontalOffset-P,y:g(i[3]+a.verticalOffset,o,r)}}}function x(e,t){function n(e){return i?[e[0]+M,e[1]+a]:[e[2]-M,e[3]+a]}var i="left"===e.horizontalAlignment,o=i?R:-R,a=t?-M:0;return{isLabelInside:function(){return!p(e.position)},getFigureCenter:n,prepareLabelPoints:function(e){var t=e.x+o,n=e.y,i=t+e.width;return s(Array(e.height+1)).map(function(e,i){return[t,n+i]}).concat(s(Array(e.height+1)).map(function(e,t){return[i,n+t]}))},isHorizontal:function(){return!0},findFigurePoint:function(e){return n(e)},adjustPoints:function(e){return e.map(Math.round)}}}function w(e,t,n){var i=e||{},o=(0,I.extend)({},i.font)||{},a=i.border||{},r=i.connector||{},s={fill:i.backgroundColor||t,"stroke-width":a.visible?a.width||0:0,stroke:a.visible&&a.width?a.color:"none",dashStyle:a.dashStyle},l={stroke:r.visible&&r.width?r.color||t:"none","stroke-width":r.visible?r.width||0:0,opacity:r.opacity};return o.color="none"===i.backgroundColor&&"#ffffff"===(0,D.normalizeEnum)(o.color)&&"inside"!==i.position?t:o.color,{format:i.format,textAlignment:i.textAlignment||(p(i.position)?n:"center"),customizeText:i.customizeText,attributes:{font:o},visible:0!==o.size&&i.visible,showForZeroValues:i.showForZeroValues,horizontalOffset:i.horizontalOffset,verticalOffset:i.verticalOffset,background:s,connector:l,wordWrap:e.wordWrap,textOverflow:e.textOverflow}}function C(e,t,n){return e.x<n[0]&&(e.x=n[0]),e.x+t.width>n[2]&&(e.x=n[2]-t.width),e.y<n[1]&&(e.y=n[1]),e.y+t.height>n[3]&&(e.y=n[3]-t.height),e}function k(e,t,n){e.reduce(function(e,t,i,o){var a=o[i+1];if(e>0){var r=t.getBoundingRect(),s=a?a.getBoundingRect().y+a.getBoundingRect().height:n,l=r.y-s,u=Math.min(l,e);o.slice(0,i+1).forEach(function(e){var t=e.getBoundingRect();e.shift(t.x,t.y-u)}),e-=u}return e},t)}t.plugin=void 0;var S=n(327),D=n(14),I=n(0),T=n(4),O="outside",E="inside",A=5,P=20,R=4,M=1,B={name:"lables",init:T.noop,dispose:T.noop,extenders:{_initCore:function(){this._labelsGroup=this._renderer.g().attr({"class":this._rootClassPrefix+"-labels"}).append(this._renderer.root),this._labels=[]},_applySize:function(){var e=this._getOption("label"),t=this._getOption("adaptiveLayout"),n=this._rect,i=0,o=n[2]-n[0];if(this._labelRect=n.slice(),!this._labels.length||!p(e.position))return void((0,D.normalizeEnum)("none"!==this._getOption("resolveLabelOverlapping",!0))&&this._labels.forEach(function(e){return!e.isVisible()&&e.draw(!0)}));var a=this._labels.map(function(e){return e.resetEllipsis(),e.getBoundingRect().width}).reduce(function(e,t){return Math.max(e,t)},0);return i=a+e.horizontalOffset+f(e.position),!t.keepLabels&&o-i<t.width?void this._labels.forEach(function(e){e.draw(!1)}):(o-i<t.width&&(i=o-t.width,i=i>0?i:0),this._labels.forEach(function(e){e.draw(!0)}),void("left"===e.horizontalAlignment?n[0]+=i:n[2]-=i))},_buildNodes:function(){this._createLabels()},_change_TILING:function(){var e,t=this,n=t._getOption("label"),i=v,o=t._getOption("inverted",!0);p(n.position)&&((0,D.normalizeEnum)(n.position)===O?i="left"===n.horizontalAlignment?m:_:(e=this._defaultLabelTextAlignment(),i="left"===n.horizontalAlignment?b(this._labelRect,this._rect,e):y(this._labelRect,this._rect,e))),t._labels.forEach(function(e,a){var r=t._items[a],s=r.getNormalStyle()["stroke-width"],l=o?s/2:-s/2,u=l?r.coords.map(function(e,t){return 1===t||3===t?e-l:2===t?e-s:0===t?e+s:e}):r.coords;if(!n.showForZeroValues&&0===r.value)return void e.draw(!1);p(n.position)&&t._correctLabelWidth(e,r.coords,n);var c=e.getBoundingRect(),d=C(i(u,c,n,o),c,t._labelRect);e.setFigureToDrawConnector(u),e.shift(d.x,d.y)}),t._resolveLabelOverlapping()}},members:{_resolveLabelOverlapping:function(){var e=this,t=(0,D.normalizeEnum)(e._getOption("resolveLabelOverlapping",!0)),n=this._getOption("inverted",!0)?e._labels.slice().reverse():e._labels;if("hide"===t)n.reduce(function(e,t){return t.getBoundingRect().y<e?t.hide():e=t.getBoundingRect().y+t.getBoundingRect().height,e},0);else if("shift"===t){var o=this._labelRect[3];n.reduce(function(t,n,a,r){var s=i(t,2),l=s[0],u=s[1],c=n.getBoundingRect(),d=c.y;return c.y<l&&(n.shift(c.x,l),d=l),d-l>0&&(u+=d-l),d+c.height>o?u&&u>d+c.height-o?(k(r.slice(0,a).reverse(),d+c.height-o,e._labelRect[1]),u-=d+c.height-o,n.shift(c.x,d-(d+c.height-o)),l=d-(d+c.height-o)+c.height):n.hide():l=d+c.height,[l,u]},[this._labelRect[1],0])}},_defaultLabelTextAlignment:function(){return this._getOption("rtlEnabled",!0)?"right":"left"},_correctLabelWidth:function(e,t,n){var i="left"===n.horizontalAlignment,o=i?this._labelRect[0]:t[2],a=i?t[0]:this._labelRect[2],r=a-o;e.getBoundingRect().width>r&&e.fit(r)},_createLabels:function(){var e=this,t=e._getOption("label"),n=x(t,e._getOption("inverted",!0));this._labelsGroup.clear(),t.visible&&(this._labels=e._items.map(function(i){var o=new S.Label({renderer:e._renderer,labelsGroup:e._labelsGroup,strategy:n});return o.setOptions(w(t,i.color,e._defaultLabelTextAlignment())),o.setData({item:i,value:i.value,percent:i.percent}),o.draw(!0),o}),this._labels.length&&p(t.position)&&this._requestChange(["LAYOUT"]))}},customize:function(e){e.prototype._proxyData.push(function(e,t){var n,i=this;return i._labels.forEach(function(o,a){var r=o.getBoundingRect();if(e>=r.x&&e<=r.x+r.width&&t>=r.y&&t<=r.y+r.height){var s=p(i._getOption("label").position)?"outside":"inside";return n={id:a,type:s+"-label"},!0}}),n}),["label","resolveLabelOverlapping"].forEach(function(t){e.addChange({code:t.toUpperCase(),handler:function(){this._createLabels(),this._requestChange(["LAYOUT"])},isThemeDependent:!0,isOptionChange:!0,option:t})})},fontFields:["label.font"]};t.plugin=B},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.plugin=void 0;var o=i(n(571)),a=n(332),r=n(1),s="__funnel_data_",l=0,u=o.default.prototype;u._eventsMap.onItemClick={name:"itemClick"},u._eventsMap.onLegendClick={name:"legendClick"};var c=function(){return s+l++},d={name:"tracker",init:function(){var e=this,t=c(),n=function(t){var n=e._renderer.getRootOffset(),i=Math.floor(t.pageX-n.left),o=Math.floor(t.pageY-n.top);return e._hitTestTargets(i,o)};e._tracker=new a.Tracker({widget:e,root:e._renderer.root,getData:function(e,i){var o=e.target,a=o[t];if((0,r.isDefined)(a))return a;var s=n(e);return i&&s&&"inside-label"!==s.type?void 0:s&&s.id},getNode:function(t){return e._items[t]},click:function(t){var i=n(t.event),o=i&&i.type,a="legend"===o?"legendClick":"itemClick";e._eventTrigger(a,{item:t.node,event:t.event})}}),this._dataKey=t},dispose:function(){this._tracker.dispose()},extenders:{_change_TILING:function(){var e=this._dataKey;this._items.forEach(function(t,n){t.element.data(e,n)})}}};t.plugin=d},function(e,t,n){function i(e,t){var n=t.getRootOffset();return[(e[0]+e[2])/2+n.left,(e[1]+e[5])/2+n.top]}t.plugin=void 0;var o=n(4),a=n(133),r={name:"funnel-tooltip",init:o.noop,dispose:o.noop,extenders:{_buildNodes:function(){this.hideTooltip()},_change_TILING:function(){this._tooltipIndex>=0&&this._moveTooltip(this._items[this._tooltipIndex])}},members:{hideTooltip:function(){this._tooltipIndex>=0&&(this._tooltipIndex=-1,this._tooltip.hide())},_moveTooltip:function(e,t){var n=t||e.coords&&i(e.coords,this._renderer)||[-1e3,-1e3];this._tooltip.move(n[0],n[1],0)},_showTooltip:function(e,t){var n=this,i=n._tooltip,o=n._items[e];if(n._tooltipIndex===e)return void n._moveTooltip(o,t);var a=function(a){void 0!==a&&(a?n._moveTooltip(o,t):i.hide(),n._tooltipIndex=a?e:-1)};a(i.show({value:o.value,valueText:i.formatValue(o.value),percentText:i.formatValue(o.percent,"percent"),percent:o.percent,item:o},{x:0,y:0,offset:0},{item:o},void 0,a))}},customize:function(e){e.addPlugin(a.plugin)}};t.plugin=r},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(572)),a=n(1002),r=n(111),s=n(131),l=n(1003),u=n(133),c=n(153);o.default.addPlugin(r.plugin),o.default.addPlugin(s.plugin),o.default.addPlugin(l.plugin),o.default.addPlugin(c.plugin),o.default.addPlugin(u.plugin),(0,a.setTooltipCustomOptions)(o.default);var d=o.default;t.default=d,e.exports=t.default},function(e,t,n){function i(e,t,n){var i=t.border,o=n.border,a=(0,r.isDefined)(i.visible)?i.visible:o.visible,s=(0,r.isDefined)(i.width)?i.width:o.width,l=(0,r.isDefined)(i.opacity)?i.opacity:(0,r.isDefined)(o.opacity)?o.opacity:1,u=(0,r.isDefined)(t.opacity)?t.opacity:(0,r.isDefined)(n.opacity)?n.opacity:1;return{fill:t.color||e,"stroke-width":a?s:0,stroke:t.border.color||n.border.color,"stroke-opacity":l,opacity:u,hatching:t.hatching}}function o(e,t,n){var i=s.patchFontOptions;e.useNodeColors&&(e.font.color=n.color);var o=!!(0,r.isDefined)(e.border.visible)&&e.border.visible,a=(0,r.isDefined)(e.border.width)?e.border.width:0,l=(0,r.isDefined)(e.border.color)?e.border.color:e.font.color,u=(0,r.isDefined)(e.border.opacity)?e.border.opacity:1,c={filter:t};return o&&a&&(c.stroke=l,c["stroke-width"]=o?a:0,c["stroke-opacity"]=u),{attr:c,css:i(e.font)}}function a(e,t){var n=this,o=e._renderer.getRootOffset();n.code=0,n.widget=e,n.color=t.color,n.options=t.options,n.rect=t.rect,n.label=n.title=t.rect._name,n.coords={x:t.rect.x+t.rect.width/2+o.left,y:t.rect.y+t.rect.height/2+o.top},n.id=t.id,n.linksIn=t.linksIn,n.linksOut=t.linksOut,this.states={normal:i(this.color,n.options,n.options),hover:i(this.color,n.options.hoverStyle,n.options)}}t.default=void 0;var r=n(1),s=n(14),l=["normal","hover"];a.prototype={compileAttrs:function(){return i(this.color,this.options)},getState:function(){return l[this.code]},isHovered:function(){return!!(1&this.code)},setState:function(e,t){var n=this;t?this.code|=e:this.code&=~e,t?this.linksIn.concat(this.linksOut).forEach(function(e){n.widget._links[e.index].setAdjacentNodeHover(!0)}):(this.widget._links.forEach(function(e){e.isAdjacentNodeHovered()&&e.adjacentNodeHover(!1)}),this.hideTooltip()),this.widget._applyNodesAppearance(),this.widget._applyLinksAppearance()},hover:function(e){this.widget._getOption("hoverEnabled",!0)&&e!==this.isHovered()&&(this.widget._suspend(),e&&this.widget.clearHover(),this.setState(1,e),this.widget._eventTrigger("nodeHoverChanged",{target:this}),this.widget._resume())},setHover:function(){this.hover(!0)},showTooltip:function(e){this.widget._getOption("hoverEnabled",!0)&&this.widget._tooltip&&this.widget._tooltip.show({type:"node",info:{label:this.label,title:this.label,weightIn:this.linksIn.reduce(function(e,t){return e+t.weight},0),weightOut:this.linksOut.reduce(function(e,t){return e+t.weight},0)}},"undefined"!=typeof e?{x:e[0],y:e[1]}:this.coords)},hideTooltip:function(){this.widget._tooltip&&this.widget._tooltip.hide()},getLabelAttributes:function(e,t){return o(e,t,this)}};var u=a;t.default=u,e.exports=t.default},function(e,t,n){function i(e,t,n,i){var o=t.border,s=n.border,l=(0,r.isDefined)(o.visible)?o.visible:s.visible,u=(0,r.isDefined)(o.width)?o.width:s.width,c=(0,r.isDefined)(o.opacity)?o.opacity:(0,r.isDefined)(s.opacity)?s.opacity:1,d=(0,r.isDefined)(t.opacity)?t.opacity:(0,r.isDefined)(n.opacity)?n.opacity:1,h=t.color||e;return n.colorMode===a.COLOR_MODE_TARGET||n.colorMode===a.COLOR_MODE_SOURCE?h=e:n.colorMode===a.COLOR_MODE_GRADIENT&&i&&(0,r.isDefined)(i.id)&&(h=i.id),{fill:h,"stroke-width":l?u:0,stroke:t.border.color||n.border.color,"stroke-opacity":c,opacity:d,hatching:t.hatching}}function o(e,t){var n=this,o=e._renderer.getRootOffset();n.code=0,n.widget=e,n.color=t.color,n.connection=t.connection,n.d=t.d,n.options=t.options,n.boundingRect=t.boundingRect,n.coords={x:t.boundingRect.x+t.boundingRect.width/2+o.left,y:t.boundingRect.y+t.boundingRect.height/2+o.top},n.states={normal:i(n.color,n.options,n.options,t.gradient),adjacentNodeHover:i(n.color,{opacity:0,border:{}},n.options,t.gradient),hover:i(n.color,{opacity:0,border:{}},n.options,t.gradient)},n.overlayStates={normal:i(n.color,{opacity:0,border:{}},n.options),adjacentNodeHover:i(n.color,n.options.hoverStyle,n.options),hover:i(n.color,n.options.hoverStyle,n.options)}}t.default=void 0;var a=n(573),r=n(1),s=["normal","adjacentNodeHover","hover"];o.prototype={getState:function(){return s[this.code]},isHovered:function(){return 2===this.code},isAdjacentNodeHovered:function(){return 1===this.code},setState:function(e,t){t?this.code=e:(this.code=0,this.hideTooltip()),this.widget._applyLinksAppearance()},setHover:function(){this.hover(!0)},hover:function(e){this.widget._getOption("hoverEnabled",!0)&&e!==this.isHovered()&&(this.widget._suspend(),e&&this.widget.clearHover(),this.setState(2,e),this.widget._eventTrigger("linkHoverChanged",{target:this}),this.widget._resume())},adjacentNodeHover:function(e){this.widget._getOption("hoverEnabled",!0)&&e!==this.isAdjacentNodeHovered()&&(this.widget._suspend(),this.setState(1,e),this.widget._resume())},setAdjacentNodeHover:function(){this.adjacentNodeHover(!0)},showTooltip:function(e){this.widget._getOption("hoverEnabled",!0)&&this.widget._tooltip&&this.widget._tooltip.show({type:"link",info:{source:this.connection.source,target:this.connection.target,weight:this.connection.weight}},"undefined"!=typeof e?{x:e[0],y:e[1]}:this.coords)},hideTooltip:function(){this.widget._tooltip&&this.widget._tooltip.hide()}};var l=o;t.default=l,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(574)),a=i(n(1001)),r=.3,s="center",l="bottom",u=s,c={_weightPerPixel:null,_getCascadeIdx:function(e,t){var n=t.filter(function(t){return t.name===e})[0];return n.outgoing.length>0?n.lp:o.default.routines.maxOfArray(t.map(function(e){return e.lp}))},_getInWeightForNode:function(e,t){var n=0;return t.forEach(function(t){t[1]===e&&(n+=t[2])}),n},_getOutWeightForNode:function(e,t){var n=0;return t.forEach(function(t){t[0]===e&&(n+=t[2])}),n},_computeCascades:function(e){for(var t=this,n=o.default.struct.computeLongestPaths(e),i=o.default.routines.maxOfArray(n.map(function(e){return e.lp})),a=[],r=0;r<i+1;r++)a.push({});return e.forEach(function(e){var i=a[t._getCascadeIdx(e[0],n)];i[e[0]]||(i[e[0]]={nodeTitle:e[0]}),i=a[t._getCascadeIdx(e[1],n)],i[e[1]]||(i[e[1]]={nodeTitle:e[1]})}),a.forEach(function(n){Object.keys(n).forEach(function(i){var o=n[i];o.inWeight=t._getInWeightForNode(o.nodeTitle,e),o.outWeight=t._getOutWeightForNode(o.nodeTitle,e),o.maxWeight=Math.max(o.inWeight,o.outWeight)})}),a},_getWeightForCascade:function(e,t){var n=0,i=e[t];return Object.keys(i).forEach(function(e){n+=Math.max(i[e].inWeight,i[e].outWeight)}),n},_getMaxWeightThroughCascades:function(e){var t=[];return e.forEach(function(e){var n=0;Object.keys(e).forEach(function(t){var i=e[t];n+=Math.max(i.inWeight,i.outWeight);
}),t.push(n)}),o.default.routines.maxOfArray(t)},_computeNodes:function(e,t){var n=this,i=[],a=this._getMaxWeightThroughCascades(e),r=o.default.routines.maxOfArray(e.map(function(e){return Object.keys(e).length})),c=t.nodePadding,d=t.height-c*(r-1);d<0&&(c=0,d=t.height-c*(r-1)),this._weightPerPixel=a/d;var h=0;return e.forEach(function(o){var r=[],f=0,p=Object.keys(o).length,g=n._getWeightForCascade(e,h)/n._weightPerPixel+c*(p-1),_=u;_=Array.isArray(t.nodeAlign)?h<t.nodeAlign.length?t.nodeAlign[h]:u:t.nodeAlign,_===l?f=t.height-g:_===s&&(f=.5*(t.height-g)),f=Math.round(f),Object.keys(o).forEach(function(e){o[e].sort=n._sort&&Object.prototype.hasOwnProperty.call(n._sort,e)?n._sort[e]:1}),Object.keys(o).sort(function(e,t){return o[e].sort-o[t].sort}).forEach(function(n){var i=o[n],s=Math.floor(d*i.maxWeight/a),l=Math.round(h*t.width/(e.length-1))-(0===h?0:t.nodeWidth),u={};u._name=n,u.width=t.nodeWidth,u.height=s,u.x=l+t.x,u.y=f+t.y,f+=s+c,r.push(u)}),h++,i.push(r)}),i},_findRectByName:function(e,t){for(var n=0;n<e.length;n++)for(var i=0;i<e[n].length;i++)if(t===e[n][i]._name)return e[n][i];return null},_findIndexByName:function(e,t){for(var n=0,i=0;i<e.length;i++)for(var o=0;o<e[i].length;o++){if(t===e[i][o]._name)return n;n++}return null},_computeLinks:function(e,t,n){var i=this,o={},a=[],r=[];return n.forEach(function(e){Object.keys(e).forEach(function(e){o[e]={"in":0,out:0}})}),t.forEach(function(n){n.forEach(function(n){var r=n._name,s=i._findRectByName(t,r),l=e.filter(function(e){return e[0]===r});l.forEach(function(e){e.sort=i._findIndexByName(t,e[1])}),l.sort(function(e,t){return e.sort-t.sort}).forEach(function(e){var n=i._findRectByName(t,e[1]),r=Math.round(e[2]/i._weightPerPixel),l=o[e[0]].out,u=o[e[1]].in,c=o[e[0]].out+r>s.height?s.height-o[e[0]].out:r,d=o[e[1]].in+r>n.height?n.height-o[e[1]].in:r;a.push({from:{x:s.x,y:s.y+l,width:s.width,height:c,node:s,weight:e[2]},to:{x:n.x,y:n.y+u,width:n.width,height:d,node:n}}),o[e[0]].out+=r,o[e[1]].in+=r})})}),a.forEach(function(e){var t={d:i._spline(e.from,e.to),_boundingRect:{x:e.from.x+e.from.width,y:Math.min(e.from.y,e.to.y),width:e.to.x-(e.from.x+e.from.width),height:Math.max(e.from.x+e.from.height,e.to.y+e.to.height)-Math.min(e.from.y,e.to.y)},_weight:e.from.weight,_from:e.from.node,_to:e.to.node};r.push(t)}),this._fitAllNodesHeight(t,a),r},_fitNodeHeight:function(e,t,n){var i=this._findRectByName(t,e),o=0,a=0;n.forEach(function(t){t.from.node._name===e&&(a+=t.from.height),t.to.node._name===e&&(o+=t.to.height)}),i.height=Math.max(o,a)},_fitAllNodesHeight:function(e,t){for(var n=0;n<e.length;n++)for(var i=0;i<e[n].length;i++)this._fitNodeHeight(e[n][i]._name,e,t)},_spline:function(e,t){var n={x:e.x+e.width,y:e.y},i={x:e.x+e.width,y:e.y+e.height},o={x:t.x,y:t.y},a={x:t.x,y:t.y+t.height},s=r*(o.x-n.x),l="M ".concat(n.x," ").concat(n.y," C ").concat(n.x+s," ").concat(n.y," ").concat(o.x-s," ").concat(o.y," ").concat(o.x," ").concat(o.y," L ").concat(a.x," ").concat(a.y," C ").concat(a.x-s," ").concat(a.y," ").concat(i.x+s," ").concat(i.y," ").concat(i.x," ").concat(i.y," Z");return l},computeLayout:function(e,t,n,i){this._sort=t;var o={},r=a.default.validate(e,i);return r?o.error=r:(o.cascades=this._computeCascades(e),o.nodes=this._computeNodes(o.cascades,{width:n.availableRect.width,height:n.availableRect.height,x:n.availableRect.x,y:n.availableRect.y,nodePadding:n.nodePadding,nodeWidth:n.nodeWidth,nodeAlign:n.nodeAlign}),o.links=this._computeLinks(e,o.nodes,o.cascades)),o},overlap:function(e,t){return!(t.x>e.x+e.width||t.x+t.width<e.x||t.y>=e.y+e.height||t.y+t.height<=e.y)}},d=c;t.default=d,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(574)),a={validate:function(e,t){var n=null;return this._hasCycle(e)&&(n="E2006",t("E2006")),n},_hasCycle:function(e){return o.default.struct.hasCycle(e)}},r=a;t.default=r,e.exports=t.default},function(e,t,n){function i(e){e.prototype._setTooltipOptions=function(){var e,t,n=this._tooltip,i=n&&this._getOption("tooltip");i.linkTooltipTemplate&&(e=this._getTemplate(i.linkTooltipTemplate)),i.nodeTooltipTemplate&&(t=this._getTemplate(i.nodeTooltipTemplate)),n&&n.update((0,o.extend)({},i,{customizeTooltip:function(n){return e&&"link"===n.type||t&&"node"===n.type||(n.skipTemplate=!0),"node"===n.type?l(i.customizeNodeTooltip,s)(n.info):"link"===n.type?l(i.customizeLinkTooltip,r)(n.info):{}},contentTemplate:function(n,i){var o={model:n.info,container:i};return e&&"link"===n.type?e.render(o):t&&"node"===n.type?t.render(o):void 0},enabled:i.enabled}))},e.prototype.hideTooltip=function(){this._tooltip&&this._tooltip.hide()}}t.setTooltipCustomOptions=i;var o=n(0),a=n(1),r=function(e){return{html:"<strong>".concat(e.source," > ").concat(e.target,"</strong><br/>Weight: ").concat(e.weight)}},s=function(e){return{html:"<strong>".concat(e.label,"</strong><br/>Incoming weight: ").concat(e.weightIn,"<br/>Outgoing weight: ").concat(e.weightOut)}},l=function(e,t){return function(n){var i=(0,a.isFunction)(e)?e.call(n,n):{},r=Object.prototype.hasOwnProperty.bind(i);return r("html")||r("text")||(i=(0,o.extend)(i,t.call(n,n))),i}}},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.plugin=void 0;var o=i(n(572)),a=n(332),r=o.default.prototype,s="__sankey_data_",l=0;r._eventsMap.onNodeClick={name:"nodeClick"},r._eventsMap.onLinkClick={name:"linkClick"};var u=function(){return s+l++},c={name:"tracker",init:function(){var e=this,t=u();e._tracker=new a.Tracker({widget:e,root:e._renderer.root,getData:function(e){var n=e.target;return n[t]},getNode:function(t){return t<e._nodes.length?e._nodes[t]:e._links[t-e._nodes.length]},click:function(t){var n=this.getData(t.event)<e._nodes.length?"nodeClick":"linkClick";e._eventTrigger(n,{target:t.node,event:t.event})}}),this._dataKey=t},dispose:function(){this._tracker.dispose()},extenders:{_change_LINKS_DRAW:function(){var e=this._dataKey;this._nodes.concat(this._links).forEach(function(t,n){t.element.data(e,n)})}}};t.plugin=c},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=i(n(5)),r=n(4),s=n(21),l=n(30),u=n(18),c=n(209),d=i(n(9)),h=n(0),f=i(n(16)),p=i(n(179)),g=n(93),_=n(6),m=n(7),v=n(1010),y="dx-slideoutview",b="dx-slideoutview-wrapper",x="dx-slideoutview-menu-content",w="dx-slideoutview-content",C="dx-slideoutview-shield",k="dx-state-invisible",S="content",D=f.default.inherit({ctor:function(e,t){this.callBase(e,t),this._logDeprecatedComponentWarning("20.1","dxDrawer")},_getDefaultOptions:function(){return(0,h.extend)(this.callBase(),{menuPosition:"normal",menuVisible:!1,swipeEnabled:!0,menuTemplate:"menu",contentTemplate:"content",contentOffset:45})},_defaultOptionsRules:function(){return this.callBase().concat([{device:{android:!0},options:{contentOffset:54}},{device:function(e){return"generic"===e.platform&&"desktop"!==e.deviceType},options:{contentOffset:56}},{device:{win:!0,phone:!1},options:{contentOffset:76}}])},_init:function(){this.callBase(),this.$element().addClass(y),this._whenAnimationComplete=void 0,this._whenMenuRendered=void 0,this._initHideTopOverlayHandler()},_initHideTopOverlayHandler:function(){this._hideMenuHandler=this.hideMenu.bind(this)},_getAnonymousTemplateName:function(){return S},_initTemplates:function(){this._templateManager.addDefaultTemplates({menu:new g.EmptyTemplate,content:new g.EmptyTemplate}),this.callBase()},_initMarkup:function(){var e=this;this.callBase(),this._renderMarkup(),this._whenMenuRendered=new _.Deferred;var t=this._getTemplate(this.option("menuTemplate"));t&&t.render({container:this.menuContent(),onRendered:function(){e._whenMenuRendered.resolve()}});var n=this.option("contentTemplate"),i=this._getTemplate(n),o=this._templateManager.anonymousTemplateName===n;i&&i.render({container:this.content(),noModel:!0,transclude:o}),this._renderShield(),this._toggleMenuPositionClass()},_render:function(){var e=this;this.callBase(),this._whenMenuRendered.always(function(){e._initSwipeHandlers(),e._dimensionChanged()})},_renderMarkup:function(){var e=(0,o.default)("<div>").addClass(b);this._$menu=(0,o.default)("<div>").addClass(x),this._$container=(0,o.default)("<div>").addClass(w),e.append(this._$menu),e.append(this._$container),this.$element().append(e),a.default.on(this._$container,"MSPointerDown",r.noop)},_renderShield:function(){this._$shield=this._$shield||(0,o.default)("<div>").addClass(C),this._$shield.appendTo(this.content()),a.default.off(this._$shield,s.name),a.default.on(this._$shield,s.name,this.hideMenu.bind(this)),this._toggleShieldVisibility(this.option("menuVisible"))},_initSwipeHandlers:function(){this._createComponent((0,o.default)(this.content()),p.default,{disabled:!this.option("swipeEnabled"),elastic:!1,itemSizeFunc:this._getMenuWidth.bind(this),onStart:this._swipeStartHandler.bind(this),onUpdated:this._swipeUpdateHandler.bind(this),onEnd:this._swipeEndHandler.bind(this)})},_isRightMenuPosition:function(){var e="inverted"===this.option("menuPosition"),t=this.option("rtlEnabled");return t&&!e||!t&&e},_swipeStartHandler:function(e){v.animation.complete((0,o.default)(this.content()));var t=e.event,n=this.option("menuVisible"),i=this._isRightMenuPosition();t.maxLeftOffset=+(i?!n:n),t.maxRightOffset=+(i?n:!n),this._toggleShieldVisibility(!0)},_swipeUpdateHandler:function(e){var t=e.event,n=this.option("menuVisible")?t.offset+1*this._getRTLSignCorrection():t.offset;n*=this._getRTLSignCorrection(),this._renderPosition(n,!1)},_swipeEndHandler:function(e){var t=e.event.targetOffset*this._getRTLSignCorrection()+this.option("menuVisible"),n=0!==t;this.option("menuVisible")===n?this._renderPosition(this.option("menuVisible"),!0):this.option("menuVisible",n)},_toggleMenuPositionClass:function(){var e=y+"-left",t=y+"-right",n=this._isRightMenuPosition()?"right":"left";this._$menu.removeClass(e+" "+t),this._$menu.addClass(y+"-"+n)},_renderPosition:function(e,t){if((0,m.hasWindow)()){var n=this._calculatePixelOffset(e)*this._getRTLSignCorrection();this._toggleHideMenuCallback(e),t?(this._toggleShieldVisibility(!0),v.animation.moveTo((0,o.default)(this.content()),n,this._animationCompleteHandler.bind(this))):(0,l.move)((0,o.default)(this.content()),{left:n})}},_calculatePixelOffset:function(e){return e=e||0,e*this._getMenuWidth()},_getMenuWidth:function(){if(!this._menuWidth){var e=this.$element().width()-this.option("contentOffset"),t=(0,o.default)(this.menuContent());t.css("maxWidth",e<0?0:e);var n=t.width();this._menuWidth=Math.min(n,e)}return this._menuWidth},_animationCompleteHandler:function(){this._toggleShieldVisibility(this.option("menuVisible")),this._whenAnimationComplete&&this._whenAnimationComplete.resolveWith(this)},_toggleHideMenuCallback:function(e){e?c.hideCallback.add(this._hideMenuHandler):c.hideCallback.remove(this._hideMenuHandler)},_getRTLSignCorrection:function(){return this._isRightMenuPosition()?-1:1},_dispose:function(){v.animation.complete((0,o.default)(this.content())),this._toggleHideMenuCallback(!1),this.callBase()},_visibilityChanged:function(e){e&&this._dimensionChanged()},_dimensionChanged:function(){delete this._menuWidth,this._renderPosition(this.option("menuVisible"),!1)},_toggleShieldVisibility:function(e){this._$shield.toggleClass(k,!e)},_optionChanged:function(e){switch(e.name){case"width":this.callBase(e),this._dimensionChanged();break;case"contentOffset":this._dimensionChanged();break;case"menuVisible":this._renderPosition(e.value,!0);break;case"menuPosition":this._renderPosition(this.option("menuVisible"),!0),this._toggleMenuPositionClass();break;case"swipeEnabled":this._initSwipeHandlers();break;case"contentTemplate":case"menuTemplate":this._invalidate();break;default:this.callBase(e)}},menuContent:function(){return(0,u.getPublicElement)(this._$menu)},content:function(){return(0,u.getPublicElement)(this._$container)},showMenu:function(){return this.toggleMenuVisibility(!0)},hideMenu:function(){return this.toggleMenuVisibility(!1)},toggleMenuVisibility:function(e){return e=void 0===e?!this.option("menuVisible"):e,this._whenAnimationComplete=new _.Deferred,this.option("menuVisible",e),this._whenAnimationComplete.promise()}});(0,d.default)("dxSlideOutView",D);var I=D;t.default=I,e.exports=t.default},function(e,t,n){e.exports=n(1006)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0,n(1007),n(904);var o=i(n(137)),a=n(92),r=n(333);o.default.integration={},o.default.integration.EventsStrategy=a.EventsStrategy,o.default.integration.Options=r.Options;var s=o.default;t.default=s,e.exports=t.default},function(e,t,n){n(1008),n(686)},function(e,t,n){var i=n(575);i.dxSlideOut=n(1009),i.dxSlideOutView=n(1004)},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.default=void 0;var o=i(n(2)),a=n(18),r=n(4),s=n(1),l=i(n(9)),u=n(0),c=i(n(207)),d=i(n(1004)),h=i(n(58)),f=i(n(115)),p=n(154),g=n(93),_=i(n(334)),m="dx-slideout",v="dx-slideout-item-container",y="dx-slideout-menu",b="dx-slideout-item",x="dxSlideoutItemData",w=h.default.inherit({ctor:function(e,t){this.callBase(e,t),this._logDeprecatedComponentWarning("20.1","dxDrawer")},_getDefaultOptions:function(){return(0,u.extend)(this.callBase(),{activeStateEnabled:!1,menuItemTemplate:"menuItem",swipeEnabled:!0,menuVisible:!1,menuPosition:"normal",menuGrouped:!1,menuGroupTemplate:"menuGroup",onMenuItemRendered:null,onMenuGroupRendered:null,contentTemplate:"content",selectionMode:"single",selectedIndex:0,selectionRequired:!0})},_itemClass:function(){return b},_itemDataKey:function(){return x},_itemContainer:function(){return(0,o.default)(this._slideOutView.content())},_init:function(){this._selectedItemContentRendered=!1,this.callBase(),this.$element().addClass(m),this._initSlideOutView()},_initTemplates:function(){this.callBase(),this._templateManager.addDefaultTemplates({menuItem:new p.ChildDefaultTemplate("item"),menuGroup:new p.ChildDefaultTemplate("group"),content:new g.EmptyTemplate})},_initEditStrategy:function(){if(this.option("menuGrouped")){var e=c.default.inherit({_getPlainItems:function(){return this.callBase().reduce(function(e,t){return e.push.apply(e,t.items),e},[])}});this._editStrategy=new e(this)}else this.callBase()},_initSlideOutView:function(){this._slideOutView=this._createComponent(this.$element(),d.default,{integrationOptions:{},menuVisible:this.option("menuVisible"),swipeEnabled:this.option("swipeEnabled"),menuPosition:this.option("menuPosition"),onOptionChanged:this._slideOutViewOptionChanged.bind(this)}),this._itemContainer().addClass(v)},_slideOutViewOptionChanged:function(e){"menuVisible"===e.name&&this.option(e.name,e.value)},_initMarkup:function(){this._renderList(),this._renderContentTemplate(),this.callBase()},_render:function(){this._slideOutView._renderShield(),this.callBase()},_renderList:function(){var e=this._list&&this._list.$element()||(0,o.default)("<div>").addClass(y).appendTo((0,o.default)(this._slideOutView.menuContent()));this._renderItemClickAction(),this._list=this._createComponent(e,f.default,{itemTemplateProperty:"menuTemplate",selectionMode:this.option("selectionMode"),selectionRequired:this.option("selectionRequired"),indicateLoading:!1,onItemClick:this._listItemClickHandler.bind(this),items:this.option("items"),dataSource:this._dataSource,itemTemplate:this._getTemplateByOption("menuItemTemplate"),grouped:this.option("menuGrouped"),groupTemplate:this._getTemplateByOption("menuGroupTemplate"),onItemRendered:this.option("onMenuItemRendered"),onGroupRendered:this.option("onMenuGroupRendered"),onContentReady:this._updateSlideOutView.bind(this)}),this._list.option("selectedIndex",this.option("selectedIndex"))},_getGroupedOption:function(){return this.option("menuGrouped")},_updateSlideOutView:function(){this._slideOutView._dimensionChanged()},_renderItemClickAction:function(){this._itemClickAction=this._createActionByOption("onItemClick")},_listItemClickHandler:function(e){var t=this._list.$element().find(".dx-list-item").index(e.itemElement);this.option("selectedIndex",t),this._itemClickAction(e)},_renderContentTemplate:function(){if(!(0,s.isDefined)(this._singleContent)){var e=this._itemContainer().html().length;this._getTemplateByOption("contentTemplate").render({container:(0,a.getPublicElement)(this._itemContainer())}),this._singleContent=this._itemContainer().html().length!==e}},_itemClickHandler:r.noop,_renderContentImpl:function(){if(!this._singleContent){var e=this.option("items"),t=this.option("selectedIndex");if(e.length&&t>-1){this._selectedItemContentRendered=!0;var n=this._list.getItemByIndex(t);this._renderItems([n])}}},_renderItem:function(e,t){this._itemContainer().find("."+b).remove(),this.callBase(e,t)},_selectedItemElement:function(){return this._itemElements().eq(0)},_updateSelection:function(){this._prepareContent(),this._renderContent()},_getListWidth:function(){return this._slideOutView._getMenuWidth()},_changeMenuOption:function(e,t){this._list.option(e,t),this._updateSlideOutView()},_cleanItemContainer:function(){this._singleContent||this.callBase()},beginUpdate:function(){this.callBase(),this._list&&this._list.beginUpdate()},endUpdate:function(){this._list&&this._list.endUpdate(),this.callBase()},_optionChanged:function(e){var t=e.name,n=e.value;switch(t){case"menuVisible":case"swipeEnabled":case"rtlEnabled":case"menuPosition":this._slideOutView.option(t,n);break;case"width":this.callBase(e),this._updateSlideOutView();break;case"menuItemTemplate":this._changeMenuOption("itemTemplate",this._getTemplate(n));break;case"items":this._changeMenuOption("items",this.option("items")),this._selectedItemContentRendered||this._updateSelection();break;case"dataSource":case"selectedIndex":case"selectedItem":this._changeMenuOption(t,n),this.callBase(e);break;case"menuGrouped":this._initEditStrategy(),this._changeMenuOption("grouped",n);break;case"menuGroupTemplate":this._changeMenuOption("groupTemplate",this._getTemplate(n));break;case"onMenuItemRendered":this._changeMenuOption("onItemRendered",n);break;case"onMenuGroupRendered":this._changeMenuOption("onGroupRendered",n);break;case"onItemClick":this._renderItemClickAction();break;case"contentTemplate":this._singleContent=null,this._invalidate();break;default:this.callBase(e)}},showMenu:function(){return this._slideOutView.toggleMenuVisibility(!0)},hideMenu:function(){return this._slideOutView.toggleMenuVisibility(!1)},toggleMenuVisibility:function(e){return this._slideOutView.toggleMenuVisibility(e)}}).include(_.default);(0,l.default)("dxSlideOut",w);var C=w;t.default=C,e.exports=t.default},function(e,t,n){function i(e){return e&&e.__esModule?e:{"default":e}}t.animation=void 0;var o=i(n(43)),a=400,r={moveTo:function(e,t,n){o.default.animate(e,{type:"slide",to:{left:t},duration:a,complete:n})},complete:function(e){o.default.stop(e,!0)}};t.animation=r}]);

/*!
 * html2canvas 1.0.0-alpha.5 <https://html2canvas.hertzen.com>
 * Copyright (c) 2017 Niklas von Hertzen <https://hertzen.com>
 * Released under MIT License
 */
!function(e,t){"object"==typeof exports&&"object"==typeof module?module.exports=t():"function"==typeof define&&define.amd?define([],t):"object"==typeof exports?exports.html2canvas=t():e.html2canvas=t()}(this,function(){return function(e){function t(r){if(n[r])return n[r].exports;var a=n[r]={i:r,l:!1,exports:{}};return e[r].call(a.exports,a,a.exports,t),a.l=!0,a.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r})},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=24)}([function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),a=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=/^#([a-f0-9]{3})$/i,i=function(e){var t=e.match(o);return!!t&&[parseInt(t[1][0]+t[1][0],16),parseInt(t[1][1]+t[1][1],16),parseInt(t[1][2]+t[1][2],16),null]},u=/^#([a-f0-9]{6})$/i,l=function(e){var t=e.match(u);return!!t&&[parseInt(t[1].substring(0,2),16),parseInt(t[1].substring(2,4),16),parseInt(t[1].substring(4,6),16),null]},s=/^rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)$/,c=function(e){var t=e.match(s);return!!t&&[Number(t[1]),Number(t[2]),Number(t[3]),null]},d=/^rgba\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d?\.?\d+)\s*\)$/,f=function(e){var t=e.match(d);return!!(t&&t.length>4)&&[Number(t[1]),Number(t[2]),Number(t[3]),Number(t[4])]},h=function(e){return[Math.min(e[0],255),Math.min(e[1],255),Math.min(e[2],255),e.length>3?e[3]:null]},p=function(e){var t=E[e.toLowerCase()];return t||!1},g=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e);var n=Array.isArray(t)?h(t):i(t)||c(t)||f(t)||p(t)||l(t)||[0,0,0,null],a=r(n,4),o=a[0],u=a[1],s=a[2],d=a[3];this.r=o,this.g=u,this.b=s,this.a=d}return a(e,[{key:"isTransparent",value:function(){return 0===this.a}},{key:"toString",value:function(){return null!==this.a&&1!==this.a?"rgba("+this.r+","+this.g+","+this.b+","+this.a+")":"rgb("+this.r+","+this.g+","+this.b+")"}}]),e}();t.default=g;var E={transparent:[0,0,0,0],aliceblue:[240,248,255,null],antiquewhite:[250,235,215,null],aqua:[0,255,255,null],aquamarine:[127,255,212,null],azure:[240,255,255,null],beige:[245,245,220,null],bisque:[255,228,196,null],black:[0,0,0,null],blanchedalmond:[255,235,205,null],blue:[0,0,255,null],blueviolet:[138,43,226,null],brown:[165,42,42,null],burlywood:[222,184,135,null],cadetblue:[95,158,160,null],chartreuse:[127,255,0,null],chocolate:[210,105,30,null],coral:[255,127,80,null],cornflowerblue:[100,149,237,null],cornsilk:[255,248,220,null],crimson:[220,20,60,null],cyan:[0,255,255,null],darkblue:[0,0,139,null],darkcyan:[0,139,139,null],darkgoldenrod:[184,134,11,null],darkgray:[169,169,169,null],darkgreen:[0,100,0,null],darkgrey:[169,169,169,null],darkkhaki:[189,183,107,null],darkmagenta:[139,0,139,null],darkolivegreen:[85,107,47,null],darkorange:[255,140,0,null],darkorchid:[153,50,204,null],darkred:[139,0,0,null],darksalmon:[233,150,122,null],darkseagreen:[143,188,143,null],darkslateblue:[72,61,139,null],darkslategray:[47,79,79,null],darkslategrey:[47,79,79,null],darkturquoise:[0,206,209,null],darkviolet:[148,0,211,null],deeppink:[255,20,147,null],deepskyblue:[0,191,255,null],dimgray:[105,105,105,null],dimgrey:[105,105,105,null],dodgerblue:[30,144,255,null],firebrick:[178,34,34,null],floralwhite:[255,250,240,null],forestgreen:[34,139,34,null],fuchsia:[255,0,255,null],gainsboro:[220,220,220,null],ghostwhite:[248,248,255,null],gold:[255,215,0,null],goldenrod:[218,165,32,null],gray:[128,128,128,null],green:[0,128,0,null],greenyellow:[173,255,47,null],grey:[128,128,128,null],honeydew:[240,255,240,null],hotpink:[255,105,180,null],indianred:[205,92,92,null],indigo:[75,0,130,null],ivory:[255,255,240,null],khaki:[240,230,140,null],lavender:[230,230,250,null],lavenderblush:[255,240,245,null],lawngreen:[124,252,0,null],lemonchiffon:[255,250,205,null],lightblue:[173,216,230,null],lightcoral:[240,128,128,null],lightcyan:[224,255,255,null],lightgoldenrodyellow:[250,250,210,null],lightgray:[211,211,211,null],lightgreen:[144,238,144,null],lightgrey:[211,211,211,null],lightpink:[255,182,193,null],lightsalmon:[255,160,122,null],lightseagreen:[32,178,170,null],lightskyblue:[135,206,250,null],lightslategray:[119,136,153,null],lightslategrey:[119,136,153,null],lightsteelblue:[176,196,222,null],lightyellow:[255,255,224,null],lime:[0,255,0,null],limegreen:[50,205,50,null],linen:[250,240,230,null],magenta:[255,0,255,null],maroon:[128,0,0,null],mediumaquamarine:[102,205,170,null],mediumblue:[0,0,205,null],mediumorchid:[186,85,211,null],mediumpurple:[147,112,219,null],mediumseagreen:[60,179,113,null],mediumslateblue:[123,104,238,null],mediumspringgreen:[0,250,154,null],mediumturquoise:[72,209,204,null],mediumvioletred:[199,21,133,null],midnightblue:[25,25,112,null],mintcream:[245,255,250,null],mistyrose:[255,228,225,null],moccasin:[255,228,181,null],navajowhite:[255,222,173,null],navy:[0,0,128,null],oldlace:[253,245,230,null],olive:[128,128,0,null],olivedrab:[107,142,35,null],orange:[255,165,0,null],orangered:[255,69,0,null],orchid:[218,112,214,null],palegoldenrod:[238,232,170,null],palegreen:[152,251,152,null],paleturquoise:[175,238,238,null],palevioletred:[219,112,147,null],papayawhip:[255,239,213,null],peachpuff:[255,218,185,null],peru:[205,133,63,null],pink:[255,192,203,null],plum:[221,160,221,null],powderblue:[176,224,230,null],purple:[128,0,128,null],rebeccapurple:[102,51,153,null],red:[255,0,0,null],rosybrown:[188,143,143,null],royalblue:[65,105,225,null],saddlebrown:[139,69,19,null],salmon:[250,128,114,null],sandybrown:[244,164,96,null],seagreen:[46,139,87,null],seashell:[255,245,238,null],sienna:[160,82,45,null],silver:[192,192,192,null],skyblue:[135,206,235,null],slateblue:[106,90,205,null],slategray:[112,128,144,null],slategrey:[112,128,144,null],snow:[255,250,250,null],springgreen:[0,255,127,null],steelblue:[70,130,180,null],tan:[210,180,140,null],teal:[0,128,128,null],thistle:[216,191,216,null],tomato:[255,99,71,null],turquoise:[64,224,208,null],violet:[238,130,238,null],wheat:[245,222,179,null],white:[255,255,255,null],whitesmoke:[245,245,245,null],yellow:[255,255,0,null],yellowgreen:[154,205,50,null]};t.TRANSPARENT=new g([0,0,0,0])},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.parseBoundCurves=t.calculatePaddingBoxPath=t.calculateBorderBoxPath=t.parsePathForBorder=t.parseDocumentSize=t.calculateContentBox=t.calculatePaddingBox=t.parseBounds=t.Bounds=void 0;var a=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=r(n(7)),i=r(n(29)),u=t.Bounds=function(){function e(t,n,r,a){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.left=t,this.top=n,this.width=r,this.height=a}return a(e,null,[{key:"fromClientRect",value:function(t,n,r){return new e(t.left+n,t.top+r,t.width,t.height)}}]),e}(),l=(t.parseBounds=function(e,t,n){return u.fromClientRect(e.getBoundingClientRect(),t,n)},t.calculatePaddingBox=function(e,t){return new u(e.left+t[3].borderWidth,e.top+t[0].borderWidth,e.width-(t[1].borderWidth+t[3].borderWidth),e.height-(t[0].borderWidth+t[2].borderWidth))},t.calculateContentBox=function(e,t,n){var r=t[0].value,a=t[1].value,o=t[2].value,i=t[3].value;return new u(e.left+i+n[3].borderWidth,e.top+r+n[0].borderWidth,e.width-(n[1].borderWidth+n[3].borderWidth+i+a),e.height-(n[0].borderWidth+n[2].borderWidth+r+o))},t.parseDocumentSize=function(e){var t=e.body,n=e.documentElement;if(!t||!n)throw new Error("");var r=Math.max(Math.max(t.scrollWidth,n.scrollWidth),Math.max(t.offsetWidth,n.offsetWidth),Math.max(t.clientWidth,n.clientWidth)),a=Math.max(Math.max(t.scrollHeight,n.scrollHeight),Math.max(t.offsetHeight,n.offsetHeight),Math.max(t.clientHeight,n.clientHeight));return new u(0,0,r,a)},t.parsePathForBorder=function(e,t){switch(t){case 0:return l(e.topLeftOuter,e.topLeftInner,e.topRightOuter,e.topRightInner);case 1:return l(e.topRightOuter,e.topRightInner,e.bottomRightOuter,e.bottomRightInner);case 2:return l(e.bottomRightOuter,e.bottomRightInner,e.bottomLeftOuter,e.bottomLeftInner);case 3:default:return l(e.bottomLeftOuter,e.bottomLeftInner,e.topLeftOuter,e.topLeftInner)}},function(e,t,n,r){var a=[];return e instanceof i.default?a.push(e.subdivide(.5,!1)):a.push(e),n instanceof i.default?a.push(n.subdivide(.5,!0)):a.push(n),r instanceof i.default?a.push(r.subdivide(.5,!0).reverse()):a.push(r),t instanceof i.default?a.push(t.subdivide(.5,!1).reverse()):a.push(t),a}),s=(t.calculateBorderBoxPath=function(e){return[e.topLeftOuter,e.topRightOuter,e.bottomRightOuter,e.bottomLeftOuter]},t.calculatePaddingBoxPath=function(e){return[e.topLeftInner,e.topRightInner,e.bottomRightInner,e.bottomLeftInner]},t.parseBoundCurves=function(e,t,n){var r=e.width/2,a=e.height/2,i=n[s.TOP_LEFT][0].getAbsoluteValue(e.width)<r?n[s.TOP_LEFT][0].getAbsoluteValue(e.width):r,u=n[s.TOP_LEFT][1].getAbsoluteValue(e.height)<a?n[s.TOP_LEFT][1].getAbsoluteValue(e.height):a,l=n[s.TOP_RIGHT][0].getAbsoluteValue(e.width)<r?n[s.TOP_RIGHT][0].getAbsoluteValue(e.width):r,d=n[s.TOP_RIGHT][1].getAbsoluteValue(e.height)<a?n[s.TOP_RIGHT][1].getAbsoluteValue(e.height):a,f=n[s.BOTTOM_RIGHT][0].getAbsoluteValue(e.width)<r?n[s.BOTTOM_RIGHT][0].getAbsoluteValue(e.width):r,h=n[s.BOTTOM_RIGHT][1].getAbsoluteValue(e.height)<a?n[s.BOTTOM_RIGHT][1].getAbsoluteValue(e.height):a,p=n[s.BOTTOM_LEFT][0].getAbsoluteValue(e.width)<r?n[s.BOTTOM_LEFT][0].getAbsoluteValue(e.width):r,g=n[s.BOTTOM_LEFT][1].getAbsoluteValue(e.height)<a?n[s.BOTTOM_LEFT][1].getAbsoluteValue(e.height):a,E=e.width-l,m=e.height-h,T=e.width-f,y=e.height-g;return{topLeftOuter:i>0||u>0?c(e.left,e.top,i,u,s.TOP_LEFT):new o.default(e.left,e.top),topLeftInner:i>0||u>0?c(e.left+t[3].borderWidth,e.top+t[0].borderWidth,Math.max(0,i-t[3].borderWidth),Math.max(0,u-t[0].borderWidth),s.TOP_LEFT):new o.default(e.left+t[3].borderWidth,e.top+t[0].borderWidth),topRightOuter:l>0||d>0?c(e.left+E,e.top,l,d,s.TOP_RIGHT):new o.default(e.left+e.width,e.top),topRightInner:l>0||d>0?c(e.left+Math.min(E,e.width+t[3].borderWidth),e.top+t[0].borderWidth,E>e.width+t[3].borderWidth?0:l-t[3].borderWidth,d-t[0].borderWidth,s.TOP_RIGHT):new o.default(e.left+e.width-t[1].borderWidth,e.top+t[0].borderWidth),bottomRightOuter:f>0||h>0?c(e.left+T,e.top+m,f,h,s.BOTTOM_RIGHT):new o.default(e.left+e.width,e.top+e.height),bottomRightInner:f>0||h>0?c(e.left+Math.min(T,e.width-t[3].borderWidth),e.top+Math.min(m,e.height+t[0].borderWidth),Math.max(0,f-t[1].borderWidth),h-t[2].borderWidth,s.BOTTOM_RIGHT):new o.default(e.left+e.width-t[1].borderWidth,e.top+e.height-t[2].borderWidth),bottomLeftOuter:p>0||g>0?c(e.left,e.top+y,p,g,s.BOTTOM_LEFT):new o.default(e.left,e.top+e.height),bottomLeftInner:p>0||g>0?c(e.left+t[3].borderWidth,e.top+y,Math.max(0,p-t[3].borderWidth),g-t[2].borderWidth,s.BOTTOM_LEFT):new o.default(e.left+t[3].borderWidth,e.top+e.height-t[2].borderWidth)}},{TOP_LEFT:0,TOP_RIGHT:1,BOTTOM_RIGHT:2,BOTTOM_LEFT:3}),c=function(e,t,n,r,a){var u=(Math.sqrt(2)-1)/3*4,l=n*u,c=r*u,d=e+n,f=t+r;switch(a){case s.TOP_LEFT:return new i.default(new o.default(e,f),new o.default(e,f-c),new o.default(d-l,t),new o.default(d,t));case s.TOP_RIGHT:return new i.default(new o.default(e,t),new o.default(e+l,t),new o.default(d,f-c),new o.default(d,f));case s.BOTTOM_RIGHT:return new i.default(new o.default(d,t),new o.default(d,t+c),new o.default(e+l,f),new o.default(e,f));case s.BOTTOM_LEFT:default:return new i.default(new o.default(d,f),new o.default(d-l,f),new o.default(e,t+c),new o.default(e,t))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.calculateLengthFromValueWithUnit=t.LENGTH_TYPE=void 0;var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=(function(e){e&&e.__esModule}(n(4)),t.LENGTH_TYPE={PX:0,PERCENTAGE:1}),o=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.type="%"===t.substr(t.length-1)?a.PERCENTAGE:a.PX;var n=parseFloat(t);this.value=isNaN(n)?0:n}return r(e,[{key:"isPercentage",value:function(){return this.type===a.PERCENTAGE}},{key:"getAbsoluteValue",value:function(e){return this.isPercentage()?e*(this.value/100):this.value}}],[{key:"create",value:function(t){return new e(t)}}]),e}();t.default=o;t.calculateLengthFromValueWithUnit=function(e,t,n){switch(n){case"px":case"%":return new o(t+n);case"em":case"rem":var r=new o(t);return r.value*="em"===n?parseFloat(e.style.font.fontSize):function e(t){var n=t.parent;return n?e(n):parseFloat(t.style.font.fontSize)}(e),r;default:return new o("0")}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.contains=function(e,t){return 0!=(e&t)},t.distance=function(e,t){return Math.sqrt(e*e+t*t)},t.copyCSSStyles=function(e,t){for(var n=e.length-1;n>=0;n--){var r=e.item(n);"content"!==r&&t.style.setProperty(r,e.getPropertyValue(r))}return t},t.SMALL_IMAGE="data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=function(e){return e&&e.__esModule?e:{default:e}}(n(0)),o=n(3),i=n(5),u=n(11),l=n(30),s=n(31),c=n(32),d=n(33),f=n(34),h=n(15),p=n(35),g=n(36),E=n(14),m=n(16),T=n(10),y=n(37),v=n(17),b=n(38),I=n(39),_=n(40),w=n(1),A=n(18),O=n(21),S=["INPUT","TEXTAREA","SELECT"],L=function(){function e(t,n,r,o){var L=this;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.parent=n,this.tagName=t.tagName,this.index=o,this.childNodes=[],this.listItems=[],"number"==typeof t.start&&(this.listStart=t.start);var P=t.ownerDocument.defaultView,N=P.pageXOffset,C=P.pageYOffset,M=P.getComputedStyle(t,null),x=(0,s.parseDisplay)(M.display),D="radio"===t.type||"checkbox"===t.type,k=(0,m.parsePosition)(M.position);if(this.style={background:D?A.INPUT_BACKGROUND:(0,i.parseBackground)(M,r),border:D?A.INPUT_BORDERS:(0,u.parseBorder)(M),borderRadius:(t instanceof P.HTMLInputElement||t instanceof HTMLInputElement)&&D?(0,A.getInputBorderRadius)(t):(0,l.parseBorderRadius)(M),color:D?A.INPUT_COLOR:new a.default(M.color),display:x,float:(0,c.parseCSSFloat)(M.float),font:(0,d.parseFont)(M),letterSpacing:(0,f.parseLetterSpacing)(M.letterSpacing),listStyle:x===s.DISPLAY.LIST_ITEM?(0,h.parseListStyle)(M):null,margin:(0,p.parseMargin)(M),opacity:parseFloat(M.opacity),overflow:-1===S.indexOf(t.tagName)?(0,g.parseOverflow)(M.overflow):g.OVERFLOW.HIDDEN,padding:(0,E.parsePadding)(M),position:k,textDecoration:(0,T.parseTextDecoration)(M),textShadow:(0,y.parseTextShadow)(M.textShadow),textTransform:(0,v.parseTextTransform)(M.textTransform),transform:(0,b.parseTransform)(M),visibility:(0,I.parseVisibility)(M.visibility),zIndex:(0,_.parseZIndex)(k!==m.POSITION.STATIC?M.zIndex:"auto")},this.isTransformed()&&(t.style.transform="matrix(1,0,0,1,0,0)"),x===s.DISPLAY.LIST_ITEM){var B=(0,O.getListOwner)(this);if(B){var Y=B.listItems.length;B.listItems.push(this),this.listIndex=t.hasAttribute("value")&&"number"==typeof t.value?t.value:0===Y?"number"==typeof B.listStart?B.listStart:1:B.listItems[Y-1].listIndex+1}}"IMG"===t.tagName&&t.addEventListener("load",function(){L.bounds=(0,w.parseBounds)(t,N,C),L.curvedBounds=(0,w.parseBoundCurves)(L.bounds,L.style.border,L.style.borderRadius)}),this.image=R(t,r),this.bounds=D?(0,A.reformatInputBounds)((0,w.parseBounds)(t,N,C)):(0,w.parseBounds)(t,N,C),this.curvedBounds=(0,w.parseBoundCurves)(this.bounds,this.style.border,this.style.borderRadius)}return r(e,[{key:"getClipPaths",value:function(){var e=this.parent?this.parent.getClipPaths():[];return this.style.overflow!==g.OVERFLOW.VISIBLE?e.concat([(0,w.calculatePaddingBoxPath)(this.curvedBounds)]):e}},{key:"isInFlow",value:function(){return this.isRootElement()&&!this.isFloating()&&!this.isAbsolutelyPositioned()}},{key:"isVisible",value:function(){return!(0,o.contains)(this.style.display,s.DISPLAY.NONE)&&this.style.opacity>0&&this.style.visibility===I.VISIBILITY.VISIBLE}},{key:"isAbsolutelyPositioned",value:function(){return this.style.position!==m.POSITION.STATIC&&this.style.position!==m.POSITION.RELATIVE}},{key:"isPositioned",value:function(){return this.style.position!==m.POSITION.STATIC}},{key:"isFloating",value:function(){return this.style.float!==c.FLOAT.NONE}},{key:"isRootElement",value:function(){return null===this.parent}},{key:"isTransformed",value:function(){return null!==this.style.transform}},{key:"isPositionedWithZIndex",value:function(){return this.isPositioned()&&!this.style.zIndex.auto}},{key:"isInlineLevel",value:function(){return(0,o.contains)(this.style.display,s.DISPLAY.INLINE)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_BLOCK)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_FLEX)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_GRID)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_LIST_ITEM)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_TABLE)}},{key:"isInlineBlockOrInlineTable",value:function(){return(0,o.contains)(this.style.display,s.DISPLAY.INLINE_BLOCK)||(0,o.contains)(this.style.display,s.DISPLAY.INLINE_TABLE)}}]),e}();t.default=L;var R=function(e,t){if(e instanceof e.ownerDocument.defaultView.SVGSVGElement||e instanceof SVGSVGElement){var n=new XMLSerializer;return t.loadImage("data:image/svg+xml,"+encodeURIComponent(n.serializeToString(e)))}switch(e.tagName){case"IMG":var r=e;return t.loadImage(r.currentSrc||r.src);case"CANVAS":var a=e;return t.loadCanvas(a);case"IFRAME":var o=e.getAttribute("data-html2canvas-internal-iframe-key");if(o)return o}return null}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.parseBackgroundImage=t.parseBackground=t.calculateBackgroundRepeatPath=t.calculateBackgroundPosition=t.calculateBackgroungPositioningArea=t.calculateBackgroungPaintingArea=t.calculateGradientBackgroundSize=t.calculateBackgroundSize=t.BACKGROUND_ORIGIN=t.BACKGROUND_CLIP=t.BACKGROUND_SIZE=t.BACKGROUND_REPEAT=void 0;var a=r(n(0)),o=r(n(2)),i=r(n(28)),u=r(n(7)),l=n(1),s=n(14),c=t.BACKGROUND_REPEAT={REPEAT:0,NO_REPEAT:1,REPEAT_X:2,REPEAT_Y:3},d=t.BACKGROUND_SIZE={AUTO:0,CONTAIN:1,COVER:2,LENGTH:3},f=t.BACKGROUND_CLIP={BORDER_BOX:0,PADDING_BOX:1,CONTENT_BOX:2},h=t.BACKGROUND_ORIGIN=f,p=function e(t){switch(function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),t){case"contain":this.size=d.CONTAIN;break;case"cover":this.size=d.COVER;break;case"auto":this.size=d.AUTO;break;default:this.value=new o.default(t)}},g=(t.calculateBackgroundSize=function(e,t,n){var r=0,a=0,o=e.size;if(o[0].size===d.CONTAIN||o[0].size===d.COVER){var u=n.width/n.height,l=t.width/t.height;return u<l!=(o[0].size===d.COVER)?new i.default(n.width,n.width/l):new i.default(n.height*l,n.height)}return o[0].value&&(r=o[0].value.getAbsoluteValue(n.width)),o[0].size===d.AUTO&&o[1].size===d.AUTO?a=t.height:o[1].size===d.AUTO?a=r/t.width*t.height:o[1].value&&(a=o[1].value.getAbsoluteValue(n.height)),o[0].size===d.AUTO&&(r=a/t.height*t.width),new i.default(r,a)},t.calculateGradientBackgroundSize=function(e,t){var n=e.size,r=n[0].value?n[0].value.getAbsoluteValue(t.width):t.width,a=n[1].value?n[1].value.getAbsoluteValue(t.height):n[0].value?r:t.height;return new i.default(r,a)},new p("auto")),E=(t.calculateBackgroungPaintingArea=function(e,t){switch(t){case f.BORDER_BOX:return(0,l.calculateBorderBoxPath)(e);case f.PADDING_BOX:default:return(0,l.calculatePaddingBoxPath)(e)}},t.calculateBackgroungPositioningArea=function(e,t,n,r){var a=(0,l.calculatePaddingBox)(t,r);switch(e){case h.BORDER_BOX:return t;case h.CONTENT_BOX:var o=n[s.PADDING_SIDES.LEFT].getAbsoluteValue(t.width),i=n[s.PADDING_SIDES.RIGHT].getAbsoluteValue(t.width),u=n[s.PADDING_SIDES.TOP].getAbsoluteValue(t.width),c=n[s.PADDING_SIDES.BOTTOM].getAbsoluteValue(t.width);return new l.Bounds(a.left+o,a.top+u,a.width-o-i,a.height-u-c);case h.PADDING_BOX:default:return a}},t.calculateBackgroundPosition=function(e,t,n){return new u.default(e[0].getAbsoluteValue(n.width-t.width),e[1].getAbsoluteValue(n.height-t.height))},t.calculateBackgroundRepeatPath=function(e,t,n,r,a){switch(e.repeat){case c.REPEAT_X:return[new u.default(Math.round(a.left),Math.round(r.top+t.y)),new u.default(Math.round(a.left+a.width),Math.round(r.top+t.y)),new u.default(Math.round(a.left+a.width),Math.round(n.height+r.top+t.y)),new u.default(Math.round(a.left),Math.round(n.height+r.top+t.y))];case c.REPEAT_Y:return[new u.default(Math.round(r.left+t.x),Math.round(a.top)),new u.default(Math.round(r.left+t.x+n.width),Math.round(a.top)),new u.default(Math.round(r.left+t.x+n.width),Math.round(a.height+a.top)),new u.default(Math.round(r.left+t.x),Math.round(a.height+a.top))];case c.NO_REPEAT:return[new u.default(Math.round(r.left+t.x),Math.round(r.top+t.y)),new u.default(Math.round(r.left+t.x+n.width),Math.round(r.top+t.y)),new u.default(Math.round(r.left+t.x+n.width),Math.round(r.top+t.y+n.height)),new u.default(Math.round(r.left+t.x),Math.round(r.top+t.y+n.height))];default:return[new u.default(Math.round(a.left),Math.round(a.top)),new u.default(Math.round(a.left+a.width),Math.round(a.top)),new u.default(Math.round(a.left+a.width),Math.round(a.height+a.top)),new u.default(Math.round(a.left),Math.round(a.height+a.top))]}},t.parseBackground=function(e,t){return{backgroundColor:new a.default(e.backgroundColor),backgroundImage:T(e,t),backgroundClip:E(e.backgroundClip),backgroundOrigin:m(e.backgroundOrigin)}},function(e){switch(e){case"padding-box":return f.PADDING_BOX;case"content-box":return f.CONTENT_BOX}return f.BORDER_BOX}),m=function(e){switch(e){case"padding-box":return h.PADDING_BOX;case"content-box":return h.CONTENT_BOX}return h.BORDER_BOX},T=function(e,t){var n=b(e.backgroundImage).map(function(e){if("url"===e.method){var n=t.loadImage(e.args[0]);e.args=n?[n]:[]}return e}),r=e.backgroundPosition.split(","),a=e.backgroundRepeat.split(","),o=e.backgroundSize.split(",");return n.map(function(e,t){var n=(o[t]||"auto").trim().split(" ").map(y),i=(r[t]||"auto").trim().split(" ").map(v);return{source:e,repeat:function(e){switch(e.trim()){case"no-repeat":return c.NO_REPEAT;case"repeat-x":case"repeat no-repeat":return c.REPEAT_X;case"repeat-y":case"no-repeat repeat":return c.REPEAT_Y;case"repeat":return c.REPEAT}return c.REPEAT}("string"==typeof a[t]?a[t]:a[0]),size:n.length<2?[n[0],g]:[n[0],n[1]],position:i.length<2?[i[0],i[0]]:[i[0],i[1]]}})},y=function(e){return"auto"===e?g:new p(e)},v=function(e){switch(e){case"bottom":case"right":return new o.default("100%");case"left":case"top":return new o.default("0%");case"auto":return new o.default("0")}return new o.default(e)},b=t.parseBackgroundImage=function(e){var t=/^\s$/,n=[],r=[],a="",o=null,i="",u=0,l=0,s=function(){var e="";if(a){'"'===i.substr(0,1)&&(i=i.substr(1,i.length-2)),i&&r.push(i.trim());var t=a.indexOf("-",1)+1;"-"===a.substr(0,1)&&t>0&&(e=a.substr(0,t).toLowerCase(),a=a.substr(t)),"none"!==(a=a.toLowerCase())&&n.push({prefix:e,method:a,args:r})}r=[],a=i=""};return e.split("").forEach(function(e){if(0!==u||!t.test(e)){switch(e){case'"':o?o===e&&(o=null):o=e;break;case"(":if(o)break;if(0===u)return void(u=1);l++;break;case")":if(o)break;if(1===u){if(0===l)return u=0,void s();l--}break;case",":if(o)break;if(0===u)return void s();if(1===u&&0===l&&!a.match(/^url$/i))return r.push(i.trim()),void(i="")}0===u?a+=e:i+=e}}),s(),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.PATH={VECTOR:0,BEZIER_CURVE:1,CIRCLE:2}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(6);t.default=function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.type=r.PATH.VECTOR,this.x=t,this.y=n}},function(e,t,n){"use strict";function r(e,t,n){return e.length>0?t+n.toUpperCase():e}Object.defineProperty(t,"__esModule",{value:!0});var a=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=n(17),i=n(19),u=function(){function e(t,n,r){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.text=t,this.parent=n,this.bounds=r}return a(e,null,[{key:"fromTextNode",value:function(t,n){var r=s(t.data,n.style.textTransform);return new e(r,n,(0,i.parseTextBounds)(r,n,t))}}]),e}();t.default=u;var l=/(^|\s|:|-|\(|\))([a-z])/g,s=function(e,t){switch(t){case o.TEXT_TRANSFORM.LOWERCASE:return e.toLowerCase();case o.TEXT_TRANSFORM.CAPITALIZE:return e.replace(l,r);case o.TEXT_TRANSFORM.UPPERCASE:return e.toUpperCase();default:return e}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(20),a=function(e){return 0===e[0]&&255===e[1]&&0===e[2]&&255===e[3]},o={get SUPPORT_RANGE_BOUNDS(){var e=function(e){if(e.createRange){var t=e.createRange();if(t.getBoundingClientRect){var n=e.createElement("boundtest");n.style.height="123px",n.style.display="block",e.body.appendChild(n),t.selectNode(n);var r=t.getBoundingClientRect(),a=Math.round(r.height);if(e.body.removeChild(n),123===a)return!0}}return!1}(document);return Object.defineProperty(o,"SUPPORT_RANGE_BOUNDS",{value:e}),e},get SUPPORT_SVG_DRAWING(){var e=function(e){var t=new Image,n=e.createElement("canvas"),r=n.getContext("2d");t.src="data:image/svg+xml,<svg xmlns='http://www.w3.org/2000/svg'></svg>";try{r.drawImage(t,0,0),n.toDataURL()}catch(e){return!1}return!0}(document);return Object.defineProperty(o,"SUPPORT_SVG_DRAWING",{value:e}),e},get SUPPORT_BASE64_DRAWING(){return function(e){var t=function(e,t){var n=new Image,r=e.createElement("canvas"),a=r.getContext("2d");return new Promise(function(e){n.src=t;var o=function(){try{a.drawImage(n,0,0),r.toDataURL()}catch(t){return e(!1)}return e(!0)};n.onload=o,n.onerror=function(){return e(!1)},!0===n.complete&&setTimeout(function(){o()},500)})}(document,e);return Object.defineProperty(o,"SUPPORT_BASE64_DRAWING",{value:function(){return t}}),t}},get SUPPORT_FOREIGNOBJECT_DRAWING(){var e="function"==typeof Array.from&&"function"==typeof window.fetch?function(e){var t=e.createElement("canvas");t.width=100,t.height=100;var n=t.getContext("2d");n.fillStyle="rgb(0, 255, 0)",n.fillRect(0,0,100,100);var o=new Image,i=t.toDataURL();o.src=i;var u=(0,r.createForeignObjectSVG)(100,100,0,0,o);return n.fillStyle="red",n.fillRect(0,0,100,100),(0,r.loadSerializedSVG)(u).then(function(t){n.drawImage(t,0,0);var o=n.getImageData(0,0,100,100).data;n.fillStyle="red",n.fillRect(0,0,100,100);var u=e.createElement("div");return u.style.backgroundImage="url("+i+")",u.style.height="100px",a(o)?(0,r.loadSerializedSVG)((0,r.createForeignObjectSVG)(100,100,0,0,u)):Promise.reject(!1)}).then(function(e){return n.drawImage(e,0,0),a(n.getImageData(0,0,100,100).data)}).catch(function(e){return!1})}(document):Promise.resolve(!1);return Object.defineProperty(o,"SUPPORT_FOREIGNOBJECT_DRAWING",{value:e}),e},get SUPPORT_CORS_IMAGES(){var e=void 0!==(new Image).crossOrigin;return Object.defineProperty(o,"SUPPORT_CORS_IMAGES",{value:e}),e},get SUPPORT_RESPONSE_TYPE(){var e="string"==typeof(new XMLHttpRequest).responseType;return Object.defineProperty(o,"SUPPORT_RESPONSE_TYPE",{value:e}),e},get SUPPORT_CORS_XHR(){var e="withCredentials"in new XMLHttpRequest;return Object.defineProperty(o,"SUPPORT_CORS_XHR",{value:e}),e}};t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseTextDecoration=t.TEXT_DECORATION_LINE=t.TEXT_DECORATION=t.TEXT_DECORATION_STYLE=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(0)),a=t.TEXT_DECORATION_STYLE={SOLID:0,DOUBLE:1,DOTTED:2,DASHED:3,WAVY:4},o=t.TEXT_DECORATION={NONE:null},i=t.TEXT_DECORATION_LINE={UNDERLINE:1,OVERLINE:2,LINE_THROUGH:3,BLINK:4},u=function(e){switch(e){case"underline":return i.UNDERLINE;case"overline":return i.OVERLINE;case"line-through":return i.LINE_THROUGH}return i.BLINK};t.parseTextDecoration=function(e){var t=function(e){return"none"===e?null:e.split(" ").map(u)}(e.textDecorationLine?e.textDecorationLine:e.textDecoration);if(null===t)return o.NONE;return{textDecorationLine:t,textDecorationColor:e.textDecorationColor?new r.default(e.textDecorationColor):null,textDecorationStyle:function(e){switch(e){case"double":return a.DOUBLE;case"dotted":return a.DOTTED;case"dashed":return a.DASHED;case"wavy":return a.WAVY}return a.SOLID}(e.textDecorationStyle)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseBorder=t.BORDER_SIDES=t.BORDER_STYLE=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(0)),a=t.BORDER_STYLE={NONE:0,SOLID:1},o=t.BORDER_SIDES={TOP:0,RIGHT:1,BOTTOM:2,LEFT:3},i=Object.keys(o).map(function(e){return e.toLowerCase()});t.parseBorder=function(e){return i.map(function(t){var n=new r.default(e.getPropertyValue("border-"+t+"-color")),o=function(e){switch(e){case"none":return a.NONE}return a.SOLID}(e.getPropertyValue("border-"+t+"-style")),i=parseFloat(e.getPropertyValue("border-"+t+"-width"));return{borderColor:n,borderStyle:o,borderWidth:isNaN(i)?0:i}})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=n(6),o=n(10),i=function(e,t){var n=Math.max.apply(null,e.colorStops.map(function(e){return e.stop})),r=1/Math.max(1,n);e.colorStops.forEach(function(e){t.addColorStop(r*e.stop,e.color.toString())})},u=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.canvas=t||document.createElement("canvas")}return r(e,[{key:"render",value:function(e){this.ctx=this.canvas.getContext("2d"),this.options=e,this.canvas.width=Math.floor(e.width*e.scale),this.canvas.height=Math.floor(e.height*e.scale),this.canvas.style.width=e.width+"px",this.canvas.style.height=e.height+"px",this.ctx.scale(this.options.scale,this.options.scale),this.ctx.translate(-e.x,-e.y),this.ctx.textBaseline="bottom",e.logger.log("Canvas renderer initialized ("+e.width+"x"+e.height+" at "+e.x+","+e.y+") with scale "+this.options.scale)}},{key:"clip",value:function(e,t){var n=this;e.length&&(this.ctx.save(),e.forEach(function(e){n.path(e),n.ctx.clip()})),t(),e.length&&this.ctx.restore()}},{key:"drawImage",value:function(e,t,n){this.ctx.drawImage(e,t.left,t.top,t.width,t.height,n.left,n.top,n.width,n.height)}},{key:"drawShape",value:function(e,t){this.path(e),this.ctx.fillStyle=t.toString(),this.ctx.fill()}},{key:"fill",value:function(e){this.ctx.fillStyle=e.toString(),this.ctx.fill()}},{key:"getTarget",value:function(){return Promise.resolve(this.canvas)}},{key:"path",value:function(e){var t=this;this.ctx.beginPath(),Array.isArray(e)?e.forEach(function(e,n){var r=e.type===a.PATH.VECTOR?e:e.start;0===n?t.ctx.moveTo(r.x,r.y):t.ctx.lineTo(r.x,r.y),e.type===a.PATH.BEZIER_CURVE&&t.ctx.bezierCurveTo(e.startControl.x,e.startControl.y,e.endControl.x,e.endControl.y,e.end.x,e.end.y)}):this.ctx.arc(e.x+e.radius,e.y+e.radius,e.radius,0,2*Math.PI,!0),this.ctx.closePath()}},{key:"rectangle",value:function(e,t,n,r,a){this.ctx.fillStyle=a.toString(),this.ctx.fillRect(e,t,n,r)}},{key:"renderLinearGradient",value:function(e,t){var n=this.ctx.createLinearGradient(e.left+t.direction.x1,e.top+t.direction.y1,e.left+t.direction.x0,e.top+t.direction.y0);i(t,n),this.ctx.fillStyle=n,this.ctx.fillRect(e.left,e.top,e.width,e.height)}},{key:"renderRadialGradient",value:function(e,t){var n=this,r=e.left+t.center.x,a=e.top+t.center.y,o=this.ctx.createRadialGradient(r,a,0,r,a,t.radius.x);if(o)if(i(t,o),this.ctx.fillStyle=o,t.radius.x!==t.radius.y){var u=e.left+.5*e.width,l=e.top+.5*e.height,s=t.radius.y/t.radius.x,c=1/s;this.transform(u,l,[1,0,0,s,0,0],function(){return n.ctx.fillRect(e.left,c*(e.top-l)+l,e.width,e.height*c)})}else this.ctx.fillRect(e.left,e.top,e.width,e.height)}},{key:"renderRepeat",value:function(e,t,n,r,a){this.path(e),this.ctx.fillStyle=this.ctx.createPattern(this.resizeImage(t,n),"repeat"),this.ctx.translate(r,a),this.ctx.fill(),this.ctx.translate(-r,-a)}},{key:"renderTextNode",value:function(e,t,n,r,a){var i=this;this.ctx.font=[n.fontStyle,n.fontVariant,n.fontWeight,n.fontSize,n.fontFamily].join(" "),e.forEach(function(e){if(i.ctx.fillStyle=t.toString(),a&&e.text.trim().length?a.slice(0).reverse().forEach(function(t){i.ctx.shadowColor=t.color.toString(),i.ctx.shadowOffsetX=t.offsetX*i.options.scale,i.ctx.shadowOffsetY=t.offsetY*i.options.scale,i.ctx.shadowBlur=t.blur,i.ctx.fillText(e.text,e.bounds.left,e.bounds.top+e.bounds.height)}):i.ctx.fillText(e.text,e.bounds.left,e.bounds.top+e.bounds.height),null!==r){var u=r.textDecorationColor||t;r.textDecorationLine.forEach(function(t){switch(t){case o.TEXT_DECORATION_LINE.UNDERLINE:var r=i.options.fontMetrics.getMetrics(n).baseline;i.rectangle(e.bounds.left,Math.round(e.bounds.top+e.bounds.height-r),e.bounds.width,1,u);break;case o.TEXT_DECORATION_LINE.OVERLINE:i.rectangle(e.bounds.left,Math.round(e.bounds.top),e.bounds.width,1,u);break;case o.TEXT_DECORATION_LINE.LINE_THROUGH:var a=i.options.fontMetrics.getMetrics(n).middle;i.rectangle(e.bounds.left,Math.ceil(e.bounds.top+a),e.bounds.width,1,u)}})}})}},{key:"resizeImage",value:function(e,t){if(e.width===t.width&&e.height===t.height)return e;var n=this.canvas.ownerDocument.createElement("canvas");n.width=t.width,n.height=t.height;return n.getContext("2d").drawImage(e,0,0,e.width,e.height,0,0,t.width,t.height),n}},{key:"setOpacity",value:function(e){this.ctx.globalAlpha=e}},{key:"transform",value:function(e,t,n,r){this.ctx.save(),this.ctx.translate(e,t),this.ctx.transform(n[0],n[1],n[2],n[3],n[4],n[5]),this.ctx.translate(-e,-t),r(),this.ctx.restore()}}]),e}();t.default=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=function(){function e(t,n,r){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.enabled=t,this.start=r||Date.now(),this.id=n}return r(e,[{key:"child",value:function(t){return new e(this.enabled,t,this.start)}},{key:"log",value:function(){if(this.enabled&&window.console&&window.console.log){for(var e=arguments.length,t=Array(e),n=0;n<e;n++)t[n]=arguments[n];Function.prototype.bind.call(window.console.log,window.console).apply(window.console,[Date.now()-this.start+"ms",this.id?"html2canvas ("+this.id+"):":"html2canvas:"].concat([].slice.call(t,0)))}}},{key:"error",value:function(){if(this.enabled&&window.console&&window.console.error){for(var e=arguments.length,t=Array(e),n=0;n<e;n++)t[n]=arguments[n];Function.prototype.bind.call(window.console.error,window.console).apply(window.console,[Date.now()-this.start+"ms",this.id?"html2canvas ("+this.id+"):":"html2canvas:"].concat([].slice.call(t,0)))}}}]),e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parsePadding=t.PADDING_SIDES=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(2)),a=(t.PADDING_SIDES={TOP:0,RIGHT:1,BOTTOM:2,LEFT:3},["top","right","bottom","left"]);t.parsePadding=function(e){return a.map(function(t){return new r.default(e.getPropertyValue("padding-"+t))})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseListStyle=t.LIST_STYLE_TYPE=t.LIST_STYLE_POSITION=void 0;var r=n(5),a=t.LIST_STYLE_POSITION={INSIDE:0,OUTSIDE:1},o=t.LIST_STYLE_TYPE={NONE:-1,DISC:0,CIRCLE:1,SQUARE:2,DECIMAL:3,CJK_DECIMAL:4,DECIMAL_LEADING_ZERO:5,LOWER_ROMAN:6,UPPER_ROMAN:7,LOWER_GREEK:8,LOWER_ALPHA:9,UPPER_ALPHA:10,ARABIC_INDIC:11,ARMENIAN:12,BENGALI:13,CAMBODIAN:14,CJK_EARTHLY_BRANCH:15,CJK_HEAVENLY_STEM:16,CJK_IDEOGRAPHIC:17,DEVANAGARI:18,ETHIOPIC_NUMERIC:19,GEORGIAN:20,GUJARATI:21,GURMUKHI:22,HEBREW:22,HIRAGANA:23,HIRAGANA_IROHA:24,JAPANESE_FORMAL:25,JAPANESE_INFORMAL:26,KANNADA:27,KATAKANA:28,KATAKANA_IROHA:29,KHMER:30,KOREAN_HANGUL_FORMAL:31,KOREAN_HANJA_FORMAL:32,KOREAN_HANJA_INFORMAL:33,LAO:34,LOWER_ARMENIAN:35,MALAYALAM:36,MONGOLIAN:37,MYANMAR:38,ORIYA:39,PERSIAN:40,SIMP_CHINESE_FORMAL:41,SIMP_CHINESE_INFORMAL:42,TAMIL:43,TELUGU:44,THAI:45,TIBETAN:46,TRAD_CHINESE_FORMAL:47,TRAD_CHINESE_INFORMAL:48,UPPER_ARMENIAN:49,DISCLOSURE_OPEN:50,DISCLOSURE_CLOSED:51},i=(t.parseListStyle=function(e){var t=(0,r.parseBackgroundImage)(e.getPropertyValue("list-style-image"));return{listStyleType:function(e){switch(e){case"disc":return o.DISC;case"circle":return o.CIRCLE;case"square":return o.SQUARE;case"decimal":return o.DECIMAL;case"cjk-decimal":return o.CJK_DECIMAL;case"decimal-leading-zero":return o.DECIMAL_LEADING_ZERO;case"lower-roman":return o.LOWER_ROMAN;case"upper-roman":return o.UPPER_ROMAN;case"lower-greek":return o.LOWER_GREEK;case"lower-alpha":return o.LOWER_ALPHA;case"upper-alpha":return o.UPPER_ALPHA;case"arabic-indic":return o.ARABIC_INDIC;case"armenian":return o.ARMENIAN;case"bengali":return o.BENGALI;case"cambodian":return o.CAMBODIAN;case"cjk-earthly-branch":return o.CJK_EARTHLY_BRANCH;case"cjk-heavenly-stem":return o.CJK_HEAVENLY_STEM;case"cjk-ideographic":return o.CJK_IDEOGRAPHIC;case"devanagari":return o.DEVANAGARI;case"ethiopic-numeric":return o.ETHIOPIC_NUMERIC;case"georgian":return o.GEORGIAN;case"gujarati":return o.GUJARATI;case"gurmukhi":return o.GURMUKHI;case"hebrew":return o.HEBREW;case"hiragana":return o.HIRAGANA;case"hiragana-iroha":return o.HIRAGANA_IROHA;case"japanese-formal":return o.JAPANESE_FORMAL;case"japanese-informal":return o.JAPANESE_INFORMAL;case"kannada":return o.KANNADA;case"katakana":return o.KATAKANA;case"katakana-iroha":return o.KATAKANA_IROHA;case"khmer":return o.KHMER;case"korean-hangul-formal":return o.KOREAN_HANGUL_FORMAL;case"korean-hanja-formal":return o.KOREAN_HANJA_FORMAL;case"korean-hanja-informal":return o.KOREAN_HANJA_INFORMAL;case"lao":return o.LAO;case"lower-armenian":return o.LOWER_ARMENIAN;case"malayalam":return o.MALAYALAM;case"mongolian":return o.MONGOLIAN;case"myanmar":return o.MYANMAR;case"oriya":return o.ORIYA;case"persian":return o.PERSIAN;case"simp-chinese-formal":return o.SIMP_CHINESE_FORMAL;case"simp-chinese-informal":return o.SIMP_CHINESE_INFORMAL;case"tamil":return o.TAMIL;case"telugu":return o.TELUGU;case"thai":return o.THAI;case"tibetan":return o.TIBETAN;case"trad-chinese-formal":return o.TRAD_CHINESE_FORMAL;case"trad-chinese-informal":return o.TRAD_CHINESE_INFORMAL;case"upper-armenian":return o.UPPER_ARMENIAN;case"disclosure-open":return o.DISCLOSURE_OPEN;case"disclosure-closed":return o.DISCLOSURE_CLOSED;case"none":default:return o.NONE}}(e.getPropertyValue("list-style-type")),listStyleImage:t.length?t[0]:null,listStylePosition:i(e.getPropertyValue("list-style-position"))}},function(e){switch(e){case"inside":return a.INSIDE;case"outside":default:return a.OUTSIDE}})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.POSITION={STATIC:0,RELATIVE:1,ABSOLUTE:2,FIXED:3,STICKY:4};t.parsePosition=function(e){switch(e){case"relative":return r.RELATIVE;case"absolute":return r.ABSOLUTE;case"fixed":return r.FIXED;case"sticky":return r.STICKY}return r.STATIC}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.TEXT_TRANSFORM={NONE:0,LOWERCASE:1,UPPERCASE:2,CAPITALIZE:3};t.parseTextTransform=function(e){switch(e){case"uppercase":return r.UPPERCASE;case"lowercase":return r.LOWERCASE;case"capitalize":return r.CAPITALIZE}return r.NONE}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.reformatInputBounds=t.inlineSelectElement=t.inlineTextAreaElement=t.inlineInputElement=t.getInputBorderRadius=t.INPUT_BACKGROUND=t.INPUT_BORDERS=t.INPUT_COLOR=void 0;var a=r(n(8)),o=n(5),i=n(11),u=r(n(44)),l=r(n(7)),s=r(n(0)),c=r(n(2)),d=(n(1),n(19),n(3)),f=(t.INPUT_COLOR=new s.default([42,42,42]),new s.default([165,165,165])),h=new s.default([222,222,222]),p={borderWidth:1,borderColor:f,borderStyle:i.BORDER_STYLE.SOLID},g=(t.INPUT_BORDERS=[p,p,p,p],t.INPUT_BACKGROUND={backgroundColor:h,backgroundImage:[],backgroundClip:o.BACKGROUND_CLIP.PADDING_BOX,backgroundOrigin:o.BACKGROUND_ORIGIN.PADDING_BOX},new c.default("50%")),E=[g,g],m=[E,E,E,E],T=new c.default("3px"),y=[T,T],v=[y,y,y,y],b=(t.getInputBorderRadius=function(e){return"radio"===e.type?m:v},t.inlineInputElement=function(e,t){if("radio"===e.type||"checkbox"===e.type){if(e.checked){var n=Math.min(t.bounds.width,t.bounds.height);t.childNodes.push("checkbox"===e.type?[new l.default(t.bounds.left+.39363*n,t.bounds.top+.79*n),new l.default(t.bounds.left+.16*n,t.bounds.top+.5549*n),new l.default(t.bounds.left+.27347*n,t.bounds.top+.44071*n),new l.default(t.bounds.left+.39694*n,t.bounds.top+.5649*n),new l.default(t.bounds.left+.72983*n,t.bounds.top+.23*n),new l.default(t.bounds.left+.84*n,t.bounds.top+.34085*n),new l.default(t.bounds.left+.39363*n,t.bounds.top+.79*n)]:new u.default(t.bounds.left+n/4,t.bounds.top+n/4,n/4))}}else b(I(e),e,t,!1)},t.inlineTextAreaElement=function(e,t){b(e.value,e,t,!0)},t.inlineSelectElement=function(e,t){var n=e.options[e.selectedIndex||0];b(n?n.text||"":"",e,t,!1)},t.reformatInputBounds=function(e){return e.width>e.height?(e.left+=(e.width-e.height)/2,e.width=e.height):e.width<e.height&&(e.top+=(e.height-e.width)/2,e.height=e.width),e},function(e,t,n,r){var o=t.ownerDocument.body;if(e.length>0&&o){var i=t.ownerDocument.createElement("html2canvaswrapper");(0,d.copyCSSStyles)(t.ownerDocument.defaultView.getComputedStyle(t,null),i),i.style.position="fixed",i.style.left=n.bounds.left+"px",i.style.top=n.bounds.top+"px",r||(i.style.whiteSpace="nowrap");var u=t.ownerDocument.createTextNode(e);i.appendChild(u),o.appendChild(i),n.childNodes.push(a.default.fromTextNode(u,n)),o.removeChild(i)}}),I=function(e){var t="password"===e.type?new Array(e.value.length+1).join("•"):e.value;return 0===t.length?e.placeholder||"":t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseTextBounds=t.TextBounds=void 0;var r=n(41),a=n(1),o=n(10),i=function(e){return e&&e.__esModule?e:{default:e}}(n(9)),u=/[^\u0000-\u00ff]/,l=function(e){return r.ucs2.encode([e])},s=t.TextBounds=function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.text=t,this.bounds=n},c=(t.parseTextBounds=function(e,t,n){for(var a=r.ucs2.decode(e),h=0!==t.style.letterSpacing||function(e){return u.test(e)}(e)?a.map(l):f(a),p=h.length,g=n.parentNode?n.parentNode.ownerDocument.defaultView:null,E=g?g.pageXOffset:0,m=g?g.pageYOffset:0,T=[],y=0,v=0;v<p;v++){var b=h[v];if(t.style.textDecoration!==o.TEXT_DECORATION.NONE||b.trim().length>0)if(i.default.SUPPORT_RANGE_BOUNDS)T.push(new s(b,d(n,y,b.length,E,m)));else{var I=n.splitText(b.length);T.push(new s(b,c(n,E,m))),n=I}else i.default.SUPPORT_RANGE_BOUNDS||(n=n.splitText(b.length));y+=b.length}return T},function(e,t,n){var r=e.ownerDocument.createElement("html2canvaswrapper");r.appendChild(e.cloneNode(!0));var o=e.parentNode;if(o){o.replaceChild(r,e);var i=(0,a.parseBounds)(r,t,n);return r.firstChild&&o.replaceChild(r.firstChild,r),i}return new a.Bounds(0,0,0,0)}),d=function(e,t,n,r,o){var i=e.ownerDocument.createRange();return i.setStart(e,t),i.setEnd(e,t+n),a.Bounds.fromClientRect(i.getBoundingClientRect(),r,o)},f=function(e){for(var t=[],n=0,a=!1,o=void 0;e.length;)h(e[n])===a?((o=e.splice(0,n)).length&&t.push(r.ucs2.encode(o)),a=!a,n=0):n++,n>=e.length&&(o=e.splice(0,n)).length&&t.push(r.ucs2.encode(o));return t},h=function(e){return-1!==[32,13,10,9,45].indexOf(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.element=t}return r(e,[{key:"render",value:function(e){var t=this;this.options=e,this.canvas=document.createElement("canvas"),this.ctx=this.canvas.getContext("2d"),this.canvas.width=Math.floor(e.width)*e.scale,this.canvas.height=Math.floor(e.height)*e.scale,this.canvas.style.width=e.width+"px",this.canvas.style.height=e.height+"px",e.logger.log("ForeignObject renderer initialized ("+e.width+"x"+e.height+" at "+e.x+","+e.y+") with scale "+e.scale);var n=o(Math.max(e.windowWidth,e.width)*e.scale,Math.max(e.windowHeight,e.height)*e.scale,e.scrollX*e.scale,e.scrollY*e.scale,this.element);return i(n).then(function(n){return e.backgroundColor&&(t.ctx.fillStyle=e.backgroundColor.toString(),t.ctx.fillRect(0,0,e.width*e.scale,e.height*e.scale)),t.ctx.drawImage(n,-e.x*e.scale,-e.y*e.scale),t.canvas})}}]),e}();t.default=a;var o=t.createForeignObjectSVG=function(e,t,n,r,a){var o="http://www.w3.org/2000/svg",i=document.createElementNS(o,"svg"),u=document.createElementNS(o,"foreignObject");return i.setAttributeNS(null,"width",e),i.setAttributeNS(null,"height",t),u.setAttributeNS(null,"width","100%"),u.setAttributeNS(null,"height","100%"),u.setAttributeNS(null,"x",n),u.setAttributeNS(null,"y",r),u.setAttributeNS(null,"externalResourcesRequired","true"),i.appendChild(u),u.appendChild(a),i},i=t.loadSerializedSVG=function(e){return new Promise(function(t,n){var r=new Image;r.onload=function(){return t(r)},r.onerror=n,r.src="data:image/svg+xml;charset=utf-8,"+encodeURIComponent((new XMLSerializer).serializeToString(e))})}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.inlineListItemElement=t.getListOwner=void 0;var a=n(3),o=r(n(4)),i=r(n(8)),u=n(15),l=n(45),s=["OL","UL","MENU"],c=(t.getListOwner=function(e){var t=e.parent;if(!t)return null;do{if(-1!==s.indexOf(t.tagName))return t;t=t.parent}while(t);return e.parent},t.inlineListItemElement=function(e,t,n){var r=t.style.listStyle;if(r){var l=e.ownerDocument.defaultView.getComputedStyle(e,null),s=e.ownerDocument.createElement("html2canvaswrapper");switch((0,a.copyCSSStyles)(l,s),s.style.position="absolute",s.style.bottom="auto",s.style.display="block",s.style.letterSpacing="normal",r.listStylePosition){case u.LIST_STYLE_POSITION.OUTSIDE:s.style.left="auto",s.style.right=e.ownerDocument.defaultView.innerWidth-t.bounds.left-t.style.margin[1].getAbsoluteValue(t.bounds.width)+7+"px",s.style.textAlign="right";break;case u.LIST_STYLE_POSITION.INSIDE:s.style.left=t.bounds.left-t.style.margin[3].getAbsoluteValue(t.bounds.width)+"px",s.style.right="auto",s.style.textAlign="left"}var c=void 0,d=t.style.margin[0].getAbsoluteValue(t.bounds.width),f=r.listStyleImage;if(f)if("url"===f.method){var h=e.ownerDocument.createElement("img");h.src=f.args[0],s.style.top=t.bounds.top-d+"px",s.style.width="auto",s.style.height="auto",s.appendChild(h)}else{var p=.5*parseFloat(t.style.font.fontSize);s.style.top=t.bounds.top-d+t.bounds.height-1.5*p+"px",s.style.width=p+"px",s.style.height=p+"px",s.style.backgroundImage=l.listStyleImage}else"number"==typeof t.listIndex&&(c=e.ownerDocument.createTextNode(y(t.listIndex,r.listStyleType)),s.appendChild(c),s.style.top=t.bounds.top-d+"px");var g=e.ownerDocument.body;g.appendChild(s),c?(t.childNodes.push(i.default.fromTextNode(c,t)),g.removeChild(s)):t.childNodes.push(new o.default(s,t,n,0))}},{integers:[1e3,900,500,400,100,90,50,40,10,9,5,4,1],values:["M","CM","D","CD","C","XC","L","XL","X","IX","V","IV","I"]}),d={integers:[9e3,8e3,7e3,6e3,5e3,4e3,3e3,2e3,1e3,900,800,700,600,500,400,300,200,100,90,80,70,60,50,40,30,20,10,9,8,7,6,5,4,3,2,1],values:["Ք","Փ","Ւ","Ց","Ր","Տ","Վ","Ս","Ռ","Ջ","Պ","Չ","Ո","Շ","Ն","Յ","Մ","Ճ","Ղ","Ձ","Հ","Կ","Ծ","Խ","Լ","Ի","Ժ","Թ","Ը","Է","Զ","Ե","Դ","Գ","Բ","Ա"]},f={integers:[1e4,9e3,8e3,7e3,6e3,5e3,4e3,3e3,2e3,1e3,400,300,200,100,90,80,70,60,50,40,30,20,19,18,17,16,15,10,9,8,7,6,5,4,3,2,1],values:["י׳","ט׳","ח׳","ז׳","ו׳","ה׳","ד׳","ג׳","ב׳","א׳","ת","ש","ר","ק","צ","פ","ע","ס","נ","מ","ל","כ","יט","יח","יז","טז","טו","י","ט","ח","ז","ו","ה","ד","ג","ב","א"]},h={integers:[1e4,9e3,8e3,7e3,6e3,5e3,4e3,3e3,2e3,1e3,900,800,700,600,500,400,300,200,100,90,80,70,60,50,40,30,20,10,9,8,7,6,5,4,3,2,1],values:["ჵ","ჰ","ჯ","ჴ","ხ","ჭ","წ","ძ","ც","ჩ","შ","ყ","ღ","ქ","ფ","ჳ","ტ","ს","რ","ჟ","პ","ო","ჲ","ნ","მ","ლ","კ","ი","თ","ჱ","ზ","ვ","ე","დ","გ","ბ","ა"]},p=function(e,t,n,r,a){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:". ";return e<t||e>n?y(e,a):r.integers.reduce(function(t,n,a){for(;e>=n;)e-=n,t+=r.values[a];return t},"")+o},g=function(e,t,n,r){var a="";do{n||e--,a=r(e)+a,e/=t}while(e*t>=t);return a},E=function(e,t,n,r){var a=arguments.length>4&&void 0!==arguments[4]?arguments[4]:". ",o=n-t+1;return(e<0?"-":"")+(g(Math.abs(e),o,r,function(e){return(0,l.fromCodePoint)(Math.floor(e%o)+t)})+a)},m=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:". ",r=t.length;return g(Math.abs(e),r,!1,function(e){return t[Math.floor(e%r)]})+n},T=function(e,t,n,r,o,i){if(e<-9999||e>9999)return y(e,u.LIST_STYLE_TYPE.CJK_DECIMAL);var l=Math.abs(e),s=o;if(0===l)return t[0]+s;for(var c=0;l>0&&c<=4;c++){var d=l%10;0===d&&(0,a.contains)(i,1)&&""!==s?s=t[d]+s:d>1||1===d&&0===c||1===d&&1===c&&(0,a.contains)(i,2)||1===d&&1===c&&(0,a.contains)(i,4)&&e>100||1===d&&c>1&&(0,a.contains)(i,8)?s=t[d]+(c>0?n[c-1]:"")+s:1===d&&c>0&&(s=n[c-1]+s),l=Math.floor(l/10)}return(e<0?r:"")+s},y=function(e,t){switch(t){case u.LIST_STYLE_TYPE.DISC:return"•";case u.LIST_STYLE_TYPE.CIRCLE:return"◦";case u.LIST_STYLE_TYPE.SQUARE:return"◾";case u.LIST_STYLE_TYPE.DECIMAL_LEADING_ZERO:var n=E(e,48,57,!0);return n.length<4?"0"+n:n;case u.LIST_STYLE_TYPE.CJK_DECIMAL:return m(e,"〇一二三四五六七八九","、");case u.LIST_STYLE_TYPE.LOWER_ROMAN:return p(e,1,3999,c,u.LIST_STYLE_TYPE.DECIMAL).toLowerCase();case u.LIST_STYLE_TYPE.UPPER_ROMAN:return p(e,1,3999,c,u.LIST_STYLE_TYPE.DECIMAL);case u.LIST_STYLE_TYPE.LOWER_GREEK:return E(e,945,969,!1);case u.LIST_STYLE_TYPE.LOWER_ALPHA:return E(e,97,122,!1);case u.LIST_STYLE_TYPE.UPPER_ALPHA:return E(e,65,90,!1);case u.LIST_STYLE_TYPE.ARABIC_INDIC:return E(e,1632,1641,!0);case u.LIST_STYLE_TYPE.ARMENIAN:case u.LIST_STYLE_TYPE.UPPER_ARMENIAN:return p(e,1,9999,d,u.LIST_STYLE_TYPE.DECIMAL);case u.LIST_STYLE_TYPE.LOWER_ARMENIAN:return p(e,1,9999,d,u.LIST_STYLE_TYPE.DECIMAL).toLowerCase();case u.LIST_STYLE_TYPE.BENGALI:return E(e,2534,2543,!0);case u.LIST_STYLE_TYPE.CAMBODIAN:case u.LIST_STYLE_TYPE.KHMER:return E(e,6112,6121,!0);case u.LIST_STYLE_TYPE.CJK_EARTHLY_BRANCH:return m(e,"子丑寅卯辰巳午未申酉戌亥","、");case u.LIST_STYLE_TYPE.CJK_HEAVENLY_STEM:return m(e,"甲乙丙丁戊己庚辛壬癸","、");case u.LIST_STYLE_TYPE.CJK_IDEOGRAPHIC:case u.LIST_STYLE_TYPE.TRAD_CHINESE_INFORMAL:return T(e,"零一二三四五六七八九","十百千萬","負","、",14);case u.LIST_STYLE_TYPE.TRAD_CHINESE_FORMAL:return T(e,"零壹貳參肆伍陸柒捌玖","拾佰仟萬","負","、",15);case u.LIST_STYLE_TYPE.SIMP_CHINESE_INFORMAL:return T(e,"零一二三四五六七八九","十百千萬","负","、",14);case u.LIST_STYLE_TYPE.SIMP_CHINESE_FORMAL:return T(e,"零壹贰叁肆伍陆柒捌玖","拾佰仟萬","负","、",15);case u.LIST_STYLE_TYPE.JAPANESE_INFORMAL:return T(e,"〇一二三四五六七八九","十百千万","マイナス","、",0);case u.LIST_STYLE_TYPE.JAPANESE_FORMAL:return T(e,"零壱弐参四伍六七八九","拾百千万","マイナス","、",7);case u.LIST_STYLE_TYPE.KOREAN_HANGUL_FORMAL:return T(e,"영일이삼사오육칠팔구","십백천만","마이너스 ",", ",7);case u.LIST_STYLE_TYPE.KOREAN_HANJA_INFORMAL:return T(e,"零一二三四五六七八九","十百千萬","마이너스 ",", ",0);case u.LIST_STYLE_TYPE.KOREAN_HANJA_FORMAL:return T(e,"零壹貳參四五六七八九","拾百千","마이너스 ",", ",7);case u.LIST_STYLE_TYPE.DEVANAGARI:return E(e,2406,2415,!0);case u.LIST_STYLE_TYPE.GEORGIAN:return p(e,1,19999,h,u.LIST_STYLE_TYPE.DECIMAL);case u.LIST_STYLE_TYPE.GUJARATI:return E(e,2790,2799,!0);case u.LIST_STYLE_TYPE.GURMUKHI:return E(e,2662,2671,!0);case u.LIST_STYLE_TYPE.HEBREW:return p(e,1,10999,f,u.LIST_STYLE_TYPE.DECIMAL);case u.LIST_STYLE_TYPE.HIRAGANA:return m(e,"あいうえおかきくけこさしすせそたちつてとなにぬねのはひふへほまみむめもやゆよらりるれろわゐゑをん");case u.LIST_STYLE_TYPE.HIRAGANA_IROHA:return m(e,"いろはにほへとちりぬるをわかよたれそつねならむうゐのおくやまけふこえてあさきゆめみしゑひもせす");case u.LIST_STYLE_TYPE.KANNADA:return E(e,3302,3311,!0);case u.LIST_STYLE_TYPE.KATAKANA:return m(e,"アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヰヱヲン","、");case u.LIST_STYLE_TYPE.KATAKANA_IROHA:return m(e,"イロハニホヘトチリヌルヲワカヨタレソツネナラムウヰノオクヤマケフコエテアサキユメミシヱヒモセス","、");case u.LIST_STYLE_TYPE.LAO:return E(e,3792,3801,!0);case u.LIST_STYLE_TYPE.MONGOLIAN:return E(e,6160,6169,!0);case u.LIST_STYLE_TYPE.MYANMAR:return E(e,4160,4169,!0);case u.LIST_STYLE_TYPE.ORIYA:return E(e,2918,2927,!0);case u.LIST_STYLE_TYPE.PERSIAN:return E(e,1776,1785,!0);case u.LIST_STYLE_TYPE.TAMIL:return E(e,3046,3055,!0);case u.LIST_STYLE_TYPE.TELUGU:return E(e,3174,3183,!0);case u.LIST_STYLE_TYPE.THAI:return E(e,3664,3673,!0);case u.LIST_STYLE_TYPE.TIBETAN:return E(e,3872,3881,!0);case u.LIST_STYLE_TYPE.DECIMAL:default:return E(e,48,57,!0)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.FontMetrics=void 0;var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=n(3);t.FontMetrics=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this._data={},this._document=t}return r(e,[{key:"_parseMetrics",value:function(e){var t=this._document.createElement("div"),n=this._document.createElement("img"),r=this._document.createElement("span"),o=this._document.body;if(!o)throw new Error("");t.style.visibility="hidden",t.style.fontFamily=e.fontFamily,t.style.fontSize=e.fontSize,t.style.margin="0",t.style.padding="0",o.appendChild(t),n.src=a.SMALL_IMAGE,n.width=1,n.height=1,n.style.margin="0",n.style.padding="0",n.style.verticalAlign="baseline",r.style.fontFamily=e.fontFamily,r.style.fontSize=e.fontSize,r.style.margin="0",r.style.padding="0",r.appendChild(this._document.createTextNode("Hidden Text")),t.appendChild(r),t.appendChild(n);var i=n.offsetTop-r.offsetTop+2;t.removeChild(r),t.appendChild(this._document.createTextNode("Hidden Text")),t.style.lineHeight="normal",n.style.verticalAlign="super";var u=n.offsetTop-t.offsetTop+2;return o.removeChild(t),{baseline:i,middle:u}}},{key:"getMetrics",value:function(e){var t=e.fontFamily+" "+e.fontSize;return void 0===this._data[t]&&(this._data[t]=this._parseMetrics(e)),this._data[t]}}]),e}()},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Proxy=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(9));t.Proxy=function(e,t){if(!t.proxy)return Promise.reject(null);var n=t.proxy;return new Promise(function(a,o){var i=r.default.SUPPORT_CORS_XHR&&r.default.SUPPORT_RESPONSE_TYPE?"blob":"text",u=r.default.SUPPORT_CORS_XHR?new XMLHttpRequest:new XDomainRequest;if(u.onload=function(){if(u instanceof XMLHttpRequest)if(200===u.status)if("text"===i)a(u.response);else{var e=new FileReader;e.addEventListener("load",function(){return a(e.result)},!1),e.addEventListener("error",function(e){return o(e)},!1),e.readAsDataURL(u.response)}else o("");else a(u.responseText)},u.onerror=o,u.open("GET",n+"?url="+encodeURIComponent(e)+"&responseType="+i),"text"!==i&&u instanceof XMLHttpRequest&&(u.responseType=i),t.imageTimeout){var l=t.imageTimeout;u.timeout=l,u.ontimeout=function(){return o("")}}u.send()})}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}var a=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},i=r(n(12)),u=r(n(13)),l=n(25),s=n(1),c=function(e,t){"object"===("undefined"==typeof console?"undefined":o(console))&&"function"==typeof console.log&&console.log("html2canvas 1.0.0-alpha.5");var n=t||{},r=new u.default("boolean"!=typeof n.logging||n.logging);var c=e.ownerDocument;if(!c)return Promise.reject("Provided element is not within a Document");var d=c.defaultView,f=d.pageXOffset,h=d.pageYOffset,p="HTML"===e.tagName||"BODY"===e.tagName?(0,s.parseDocumentSize)(c):(0,s.parseBounds)(e,f,h),g=p.width,E=p.height,m=p.left,T=p.top,y={async:!0,allowTaint:!1,backgroundColor:"#ffffff",imageTimeout:15e3,logging:!0,proxy:null,removeContainer:!0,foreignObjectRendering:!1,scale:d.devicePixelRatio||1,target:new i.default(n.canvas),useCORS:!1,x:m,y:T,width:Math.ceil(g),height:Math.ceil(E),windowWidth:d.innerWidth,windowHeight:d.innerHeight,scrollX:d.pageXOffset,scrollY:d.pageYOffset},v=(0,l.renderElement)(e,a({},y,n),r);return v};c.CanvasRenderer=i.default,e.exports=c},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.renderElement=void 0;var a=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),o=(r(n(13)),n(26)),i=r(n(46)),u=r(n(20)),l=r(n(9)),s=n(1),c=n(49),d=n(22),f=n(0),h=r(f);t.renderElement=function e(t,n,r){var p=t.ownerDocument,g=new s.Bounds(n.scrollX,n.scrollY,n.windowWidth,n.windowHeight),E=p.documentElement?new h.default(getComputedStyle(p.documentElement).backgroundColor):f.TRANSPARENT,m=p.body?new h.default(getComputedStyle(p.body).backgroundColor):f.TRANSPARENT,T=t===p.documentElement?E.isTransparent()?m.isTransparent()?n.backgroundColor?new h.default(n.backgroundColor):null:m:E:n.backgroundColor?new h.default(n.backgroundColor):null;return(n.foreignObjectRendering?l.default.SUPPORT_FOREIGNOBJECT_DRAWING:Promise.resolve(!1)).then(function(l){return l?function(e){return e.inlineFonts(p).then(function(){return e.resourceLoader.ready()}).then(function(){return new u.default(e.documentElement).render({backgroundColor:T,logger:r,scale:n.scale,x:n.x,y:n.y,width:n.width,height:n.height,windowWidth:n.windowWidth,windowHeight:n.windowHeight,scrollX:n.scrollX,scrollY:n.scrollY})})}(new c.DocumentCloner(t,n,r,!0,e)):(0,c.cloneWindow)(p,g,t,n,r,e).then(function(e){var t=a(e,3),u=t[0],l=t[1],s=t[2];var c=(0,o.NodeParser)(l,s,r),h=l.ownerDocument;return T===c.container.style.background.backgroundColor&&(c.container.style.background.backgroundColor=f.TRANSPARENT),s.ready().then(function(e){var t=new d.FontMetrics(h);var a={backgroundColor:T,fontMetrics:t,imageStore:e,logger:r,scale:n.scale,x:n.x,y:n.y,width:n.width,height:n.height};if(Array.isArray(n.target))return Promise.all(n.target.map(function(e){return new i.default(e,a).render(c)}));var o=new i.default(n.target,a).render(c);return!0===n.removeContainer&&u.parentNode&&u.parentNode.removeChild(u),o})})})}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.NodeParser=void 0;var a=r(n(27)),o=r(n(4)),i=r(n(8)),u=n(18),l=n(21),s=(t.NodeParser=function(e,t,n){var r=0,i=new o.default(e,null,t,r++),u=new a.default(i,null,!0);return c(e,i,u,t,r),u},["SCRIPT","HEAD","TITLE","OBJECT","BR","OPTION"]),c=function e(t,n,r,c,h){for(var p,g=t.firstChild;g;g=p){p=g.nextSibling;var E=g.ownerDocument.defaultView;if(g instanceof E.Text||g instanceof Text||E.parent&&g instanceof E.parent.Text)g.data.trim().length>0&&n.childNodes.push(i.default.fromTextNode(g,n));else if(g instanceof E.HTMLElement||g instanceof HTMLElement||E.parent&&g instanceof E.parent.HTMLElement){if(-1===s.indexOf(g.nodeName)){var m=new o.default(g,n,c,h++);if(m.isVisible()){"INPUT"===g.tagName?(0,u.inlineInputElement)(g,m):"TEXTAREA"===g.tagName?(0,u.inlineTextAreaElement)(g,m):"SELECT"===g.tagName?(0,u.inlineSelectElement)(g,m):m.style.listStyle&&"none"!==m.style.listStyle.listStyleType&&(0,l.inlineListItemElement)(g,m,c);var T="TEXTAREA"!==g.tagName,y=d(m,g);if(y||f(m)){var v=y||m.isPositioned()?r.getRealParentStackingContext():r,b=new a.default(m,v,y);v.contexts.push(b),T&&e(g,m,b,c,h)}else r.children.push(m),T&&e(g,m,r,c,h)}}}else if(g instanceof E.SVGSVGElement||g instanceof SVGSVGElement||E.parent&&g instanceof E.parent.SVGSVGElement){var I=new o.default(g,n,c,h++),_=d(I,g);if(_||f(I)){var w=_||I.isPositioned()?r.getRealParentStackingContext():r,A=new a.default(I,w,_);w.contexts.push(A)}else r.children.push(I)}}},d=function(e,t){return e.isRootElement()||e.isPositionedWithZIndex()||e.style.opacity<1||e.isTransformed()||h(e,t)},f=function(e){return e.isPositioned()||e.isFloating()},h=function(e,t){return"BODY"===t.nodeName&&e.parent instanceof o.default&&e.parent.style.background.backgroundColor.isTransparent()}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=(function(e){e&&e.__esModule}(n(4)),n(16),function(){function e(t,n,r){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.container=t,this.parent=n,this.contexts=[],this.children=[],this.treatAsRealStackingContext=r}return r(e,[{key:"getOpacity",value:function(){return this.parent?this.container.style.opacity*this.parent.getOpacity():this.container.style.opacity}},{key:"getRealParentStackingContext",value:function(){return!this.parent||this.treatAsRealStackingContext?this:this.parent.getRealParentStackingContext()}}]),e}());t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.default=function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.width=t,this.height=n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),a=n(6),o=function(e){return e&&e.__esModule?e:{default:e}}(n(7)),i=function(e,t,n){return new o.default(e.x+(t.x-e.x)*n,e.y+(t.y-e.y)*n)},u=function(){function e(t,n,r,o){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.type=a.PATH.BEZIER_CURVE,this.start=t,this.startControl=n,this.endControl=r,this.end=o}return r(e,[{key:"subdivide",value:function(t,n){var r=i(this.start,this.startControl,t),a=i(this.startControl,this.endControl,t),o=i(this.endControl,this.end,t),u=i(r,a,t),l=i(a,o,t),s=i(u,l,t);return n?new e(this.start,r,u,s):new e(s,l,o,this.end)}},{key:"reverse",value:function(){return new e(this.end,this.endControl,this.startControl,this.start)}}]),e}();t.default=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseBorderRadius=void 0;var r=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),a=function(e){return e&&e.__esModule?e:{default:e}}(n(2)),o=["top-left","top-right","bottom-right","bottom-left"];t.parseBorderRadius=function(e){return o.map(function(t){var n=e.getPropertyValue("border-"+t+"-radius").split(" ").map(a.default.create),o=r(n,2),i=o[0],u=o[1];return void 0===u?[i,i]:[i,u]})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.DISPLAY={NONE:1,BLOCK:2,INLINE:4,RUN_IN:8,FLOW:16,FLOW_ROOT:32,TABLE:64,FLEX:128,GRID:256,RUBY:512,SUBGRID:1024,LIST_ITEM:2048,TABLE_ROW_GROUP:4096,TABLE_HEADER_GROUP:8192,TABLE_FOOTER_GROUP:16384,TABLE_ROW:32768,TABLE_CELL:65536,TABLE_COLUMN_GROUP:1<<17,TABLE_COLUMN:1<<18,TABLE_CAPTION:1<<19,RUBY_BASE:1<<20,RUBY_TEXT:1<<21,RUBY_BASE_CONTAINER:1<<22,RUBY_TEXT_CONTAINER:1<<23,CONTENTS:1<<24,INLINE_BLOCK:1<<25,INLINE_LIST_ITEM:1<<26,INLINE_TABLE:1<<27,INLINE_FLEX:1<<28,INLINE_GRID:1<<29},a=function(e,t){return e|function(e){switch(e){case"block":return r.BLOCK;case"inline":return r.INLINE;case"run-in":return r.RUN_IN;case"flow":return r.FLOW;case"flow-root":return r.FLOW_ROOT;case"table":return r.TABLE;case"flex":return r.FLEX;case"grid":return r.GRID;case"ruby":return r.RUBY;case"subgrid":return r.SUBGRID;case"list-item":return r.LIST_ITEM;case"table-row-group":return r.TABLE_ROW_GROUP;case"table-header-group":return r.TABLE_HEADER_GROUP;case"table-footer-group":return r.TABLE_FOOTER_GROUP;case"table-row":return r.TABLE_ROW;case"table-cell":return r.TABLE_CELL;case"table-column-group":return r.TABLE_COLUMN_GROUP;case"table-column":return r.TABLE_COLUMN;case"table-caption":return r.TABLE_CAPTION;case"ruby-base":return r.RUBY_BASE;case"ruby-text":return r.RUBY_TEXT;case"ruby-base-container":return r.RUBY_BASE_CONTAINER;case"ruby-text-container":return r.RUBY_TEXT_CONTAINER;case"contents":return r.CONTENTS;case"inline-block":return r.INLINE_BLOCK;case"inline-list-item":return r.INLINE_LIST_ITEM;case"inline-table":return r.INLINE_TABLE;case"inline-flex":return r.INLINE_FLEX;case"inline-grid":return r.INLINE_GRID}return r.NONE}(t)};t.parseDisplay=function(e){return e.split(" ").reduce(a,0)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.FLOAT={NONE:0,LEFT:1,RIGHT:2,INLINE_START:3,INLINE_END:4};t.parseCSSFloat=function(e){switch(e){case"left":return r.LEFT;case"right":return r.RIGHT;case"inline-start":return r.INLINE_START;case"inline-end":return r.INLINE_END}return r.NONE}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.parseFont=function(e){return{fontFamily:e.fontFamily,fontSize:e.fontSize,fontStyle:e.fontStyle,fontVariant:e.fontVariant,fontWeight:function(e){switch(e){case"normal":return 400;case"bold":return 700}var t=parseInt(e,10);return isNaN(t)?400:t}(e.fontWeight)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.parseLetterSpacing=function(e){if("normal"===e)return 0;var t=parseFloat(e);return isNaN(t)?0:t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseMargin=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(2)),a=["top","right","bottom","left"];t.parseMargin=function(e){return a.map(function(t){return new r.default(e.getPropertyValue("margin-"+t))})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.OVERFLOW={VISIBLE:0,HIDDEN:1,SCROLL:2,AUTO:3};t.parseOverflow=function(e){switch(e){case"hidden":return r.HIDDEN;case"scroll":return r.SCROLL;case"auto":return r.AUTO;case"visible":default:return r.VISIBLE}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseTextShadow=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(0)),a=/^([+-]|\d|\.)$/i;t.parseTextShadow=function(e){if("none"===e||"string"!=typeof e)return null;for(var t="",n=!1,o=[],i=[],u=0,l=null,s=function(){t.length&&(n?o.push(parseFloat(t)):l=new r.default(t)),n=!1,t=""},c=function(){o.length&&null!==l&&i.push({color:l,offsetX:o[0]||0,offsetY:o[1]||0,blur:o[2]||0}),o.splice(0,o.length),l=null},d=0;d<e.length;d++){var f=e[d];switch(f){case"(":t+=f,u++;break;case")":t+=f,u--;break;case",":0===u?(s(),c()):t+=f;break;case" ":0===u?s():t+=f;break;default:0===t.length&&a.test(f)&&(n=!0),t+=f}}return s(),c(),0===i.length?null:i}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseTransform=void 0;var r=function(e){return e&&e.__esModule?e:{default:e}}(n(2)),a=function(e){return parseFloat(e.trim())},o=/(matrix|matrix3d)\((.+)\)/,i=(t.parseTransform=function(e){var t=u(e.transform||e.webkitTransform||e.mozTransform||e.msTransform||e.oTransform);return null===t?null:{transform:t,transformOrigin:i(e.transformOrigin||e.webkitTransformOrigin||e.mozTransformOrigin||e.msTransformOrigin||e.oTransformOrigin)}},function(e){if("string"!=typeof e){var t=new r.default("0");return[t,t]}var n=e.split(" ").map(r.default.create);return[n[0],n[1]]}),u=function(e){if("none"===e||"string"!=typeof e)return null;var t=e.match(o);if(t){if("matrix"===t[1]){var n=t[2].split(",").map(a);return[n[0],n[1],n[2],n[3],n[4],n[5]]}var r=t[2].split(",").map(a);return[r[0],r[1],r[4],r[5],r[12],r[13]]}return null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=t.VISIBILITY={VISIBLE:0,HIDDEN:1,COLLAPSE:2};t.parseVisibility=function(e){switch(e){case"hidden":return r.HIDDEN;case"collapse":return r.COLLAPSE;case"visible":default:return r.VISIBLE}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.parseZIndex=function(e){var t="auto"===e;return{auto:t,order:t?0:parseInt(e,10)}}},function(e,t,n){(function(e,r){var a;!function(o){function i(e){throw new RangeError(P[e])}function u(e,t){for(var n=e.length,r=[];n--;)r[n]=t(e[n]);return r}function l(e,t){var n=e.split("@"),r="";n.length>1&&(r=n[0]+"@",e=n[1]);return r+u((e=e.replace(R,".")).split("."),t).join(".")}function s(e){for(var t,n,r=[],a=0,o=e.length;a<o;)(t=e.charCodeAt(a++))>=55296&&t<=56319&&a<o?56320==(64512&(n=e.charCodeAt(a++)))?r.push(((1023&t)<<10)+(1023&n)+65536):(r.push(t),a--):r.push(t);return r}function c(e){return u(e,function(e){var t="";return e>65535&&(t+=M((e-=65536)>>>10&1023|55296),e=56320|1023&e),t+=M(e)}).join("")}function d(e){return e-48<10?e-22:e-65<26?e-65:e-97<26?e-97:y}function f(e,t){return e+22+75*(e<26)-((0!=t)<<5)}function h(e,t,n){var r=0;for(e=n?C(e/_):e>>1,e+=C(e/t);e>N*b>>1;r+=y)e=C(e/N);return C(r+(N+1)*e/(e+I))}function p(e){var t,n,r,a,o,u,l,s,f,p,g=[],E=e.length,m=0,I=A,_=w;for((n=e.lastIndexOf(O))<0&&(n=0),r=0;r<n;++r)e.charCodeAt(r)>=128&&i("not-basic"),g.push(e.charCodeAt(r));for(a=n>0?n+1:0;a<E;){for(o=m,u=1,l=y;a>=E&&i("invalid-input"),((s=d(e.charCodeAt(a++)))>=y||s>C((T-m)/u))&&i("overflow"),m+=s*u,f=l<=_?v:l>=_+b?b:l-_,!(s<f);l+=y)u>C(T/(p=y-f))&&i("overflow"),u*=p;_=h(m-o,t=g.length+1,0==o),C(m/t)>T-I&&i("overflow"),I+=C(m/t),m%=t,g.splice(m++,0,I)}return c(g)}function g(e){var t,n,r,a,o,u,l,c,d,p,g,E,m,I,_,S=[];for(E=(e=s(e)).length,t=A,n=0,o=w,u=0;u<E;++u)(g=e[u])<128&&S.push(M(g));for(r=a=S.length,a&&S.push(O);r<E;){for(l=T,u=0;u<E;++u)(g=e[u])>=t&&g<l&&(l=g);for(l-t>C((T-n)/(m=r+1))&&i("overflow"),n+=(l-t)*m,t=l,u=0;u<E;++u)if((g=e[u])<t&&++n>T&&i("overflow"),g==t){for(c=n,d=y;p=d<=o?v:d>=o+b?b:d-o,!(c<p);d+=y)_=c-p,I=y-p,S.push(M(f(p+_%I,0))),c=C(_/I);S.push(M(f(c,0))),o=h(n,m,r==a),n=0,++r}++n,++t}return S.join("")}"object"==typeof t&&t&&t.nodeType,"object"==typeof e&&e&&e.nodeType;var E="object"==typeof r&&r;var m,T=2147483647,y=36,v=1,b=26,I=38,_=700,w=72,A=128,O="-",S=/^xn--/,L=/[^\x20-\x7E]/,R=/[\x2E\u3002\uFF0E\uFF61]/g,P={overflow:"Overflow: input needs wider integers to process","not-basic":"Illegal input >= 0x80 (not a basic code point)","invalid-input":"Invalid input"},N=y-v,C=Math.floor,M=String.fromCharCode;m={version:"1.4.1",ucs2:{decode:s,encode:c},decode:p,encode:g,toASCII:function(e){return l(e,function(e){return L.test(e)?"xn--"+g(e):e})},toUnicode:function(e){return l(e,function(e){return S.test(e)?p(e.slice(4).toLowerCase()):e})}},void 0===(a=function(){return m}.call(t,n,t,e))||(e.exports=a)}()}).call(t,n(42)(e),n(43))},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t){var n;n=function(){return this}();try{n=n||Function("return this")()||(0,eval)("this")}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(6);t.default=function e(t,n,a){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.type=r.PATH.CIRCLE,this.x=t,this.y=n,this.radius=a}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});t.fromCodePoint=function(){if(String.fromCodePoint)return String.fromCodePoint.apply(String,arguments);var e=arguments.length;if(!e)return"";for(var t=[],n=-1,r="";++n<e;){var a=arguments.length<=n?void 0:arguments[n];a<=65535?t.push(a):(a-=65536,t.push(55296+(a>>10),a%1024+56320)),(n+1===e||t.length>16384)&&(r+=String.fromCharCode.apply(String,t),t.length=0)}return r}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),a=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=n(1),i=(n(22),n(47)),u=function(e){return e&&e.__esModule?e:{default:e}}(n(8)),l=n(5),s=n(11),c=function(){function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.target=t,this.options=n,t.render(n)}return a(e,[{key:"renderNode",value:function(e){e.isVisible()&&(this.renderNodeBackgroundAndBorders(e),this.renderNodeContent(e))}},{key:"renderNodeContent",value:function(e){var t=this,n=function(){if(e.childNodes.length&&e.childNodes.forEach(function(n){if(n instanceof u.default){var r=n.parent.style;t.target.renderTextNode(n.bounds,r.color,r.font,r.textDecoration,r.textShadow)}else t.target.drawShape(n,e.style.color)}),e.image){var n=t.options.imageStore.get(e.image);if(n){var r=(0,o.calculateContentBox)(e.bounds,e.style.padding,e.style.border),a="number"==typeof n.width&&n.width>0?n.width:r.width,i="number"==typeof n.height&&n.height>0?n.height:r.height;a>0&&i>0&&t.target.clip([(0,o.calculatePaddingBoxPath)(e.curvedBounds)],function(){t.target.drawImage(n,new o.Bounds(0,0,a,i),r)})}}},r=e.getClipPaths();r.length?this.target.clip(r,n):n()}},{key:"renderNodeBackgroundAndBorders",value:function(e){var t=this,n=!e.style.background.backgroundColor.isTransparent()||e.style.background.backgroundImage.length,r=e.style.border.filter(function(e){return e.borderStyle!==s.BORDER_STYLE.NONE&&!e.borderColor.isTransparent()}),a=function(){var a=(0,l.calculateBackgroungPaintingArea)(e.curvedBounds,e.style.background.backgroundClip);n&&t.target.clip([a],function(){e.style.background.backgroundColor.isTransparent()||t.target.fill(e.style.background.backgroundColor),t.renderBackgroundImage(e)}),r.forEach(function(n,r){t.renderBorder(n,r,e.curvedBounds)})};if(n||r.length){var o=e.parent?e.parent.getClipPaths():[];o.length?this.target.clip(o,a):a()}}},{key:"renderBackgroundImage",value:function(e){var t=this;e.style.background.backgroundImage.slice(0).reverse().forEach(function(n){"url"===n.source.method&&n.source.args.length?t.renderBackgroundRepeat(e,n):/gradient/i.test(n.source.method)&&t.renderBackgroundGradient(e,n)})}},{key:"renderBackgroundRepeat",value:function(e,t){var n=this.options.imageStore.get(t.source.args[0]);if(n){var r=(0,l.calculateBackgroungPositioningArea)(e.style.background.backgroundOrigin,e.bounds,e.style.padding,e.style.border),a=(0,l.calculateBackgroundSize)(t,n,r),o=(0,l.calculateBackgroundPosition)(t.position,a,r),i=(0,l.calculateBackgroundRepeatPath)(t,o,a,r,e.bounds),u=Math.round(r.left+o.x),s=Math.round(r.top+o.y);this.target.renderRepeat(i,n,a,u,s)}}},{key:"renderBackgroundGradient",value:function(e,t){var n=(0,l.calculateBackgroungPositioningArea)(e.style.background.backgroundOrigin,e.bounds,e.style.padding,e.style.border),r=(0,l.calculateGradientBackgroundSize)(t,n),a=(0,l.calculateBackgroundPosition)(t.position,r,n),u=new o.Bounds(Math.round(n.left+a.x),Math.round(n.top+a.y),r.width,r.height),s=(0,i.parseGradient)(e,t.source,u);if(s)switch(s.type){case i.GRADIENT_TYPE.LINEAR_GRADIENT:this.target.renderLinearGradient(u,s);break;case i.GRADIENT_TYPE.RADIAL_GRADIENT:this.target.renderRadialGradient(u,s)}}},{key:"renderBorder",value:function(e,t,n){this.target.drawShape((0,o.parsePathForBorder)(n,t),e.borderColor)}},{key:"renderStack",value:function(e){var t=this;if(e.container.isVisible()){var n=e.getOpacity();n!==this._opacity&&(this.target.setOpacity(e.getOpacity()),this._opacity=n);var r=e.container.style.transform;null!==r?this.target.transform(e.container.bounds.left+r.transformOrigin[0].value,e.container.bounds.top+r.transformOrigin[1].value,r.transform,function(){return t.renderStackContent(e)}):this.renderStackContent(e)}}},{key:"renderStackContent",value:function(e){var t=f(e),n=r(t,5),a=n[0],o=n[1],i=n[2],u=n[3],l=n[4],s=d(e),c=r(s,2),p=c[0],g=c[1];this.renderNodeBackgroundAndBorders(e.container),a.sort(h).forEach(this.renderStack,this),this.renderNodeContent(e.container),g.forEach(this.renderNode,this),u.forEach(this.renderStack,this),l.forEach(this.renderStack,this),p.forEach(this.renderNode,this),o.forEach(this.renderStack,this),i.sort(h).forEach(this.renderStack,this)}},{key:"render",value:function(e){this.options.backgroundColor&&this.target.rectangle(this.options.x,this.options.y,this.options.width,this.options.height,this.options.backgroundColor),this.renderStack(e);var t=this.target.getTarget();return t}}]),e}();t.default=c;var d=function(e){for(var t=[],n=[],r=e.children.length,a=0;a<r;a++){var o=e.children[a];o.isInlineLevel()?t.push(o):n.push(o)}return[t,n]},f=function(e){for(var t=[],n=[],r=[],a=[],o=[],i=e.contexts.length,u=0;u<i;u++){var l=e.contexts[u];l.container.isPositioned()||l.container.style.opacity<1||l.container.isTransformed()?l.container.style.zIndex.order<0?t.push(l):l.container.style.zIndex.order>0?r.push(l):n.push(l):l.container.isFloating()?a.push(l):o.push(l)}return[t,n,r,a,o]},h=function(e,t){return e.container.style.zIndex.order>t.container.style.zIndex.order?1:e.container.style.zIndex.order<t.container.style.zIndex.order?-1:e.container.index>t.container.index?1:-1}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}function a(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}Object.defineProperty(t,"__esModule",{value:!0}),t.transformWebkitRadialGradientArgs=t.parseGradient=t.RadialGradient=t.LinearGradient=t.RADIAL_GRADIENT_SHAPE=t.GRADIENT_TYPE=void 0;var o=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),i=(r(n(4)),n(48)),u=r(n(0)),l=n(2),s=r(l),c=n(3),d=/^(to )?(left|top|right|bottom)( (left|top|right|bottom))?$/i,f=/^([+-]?\d*\.?\d+)% ([+-]?\d*\.?\d+)%$/i,h=/(px)|%|( 0)$/i,p=/^(from|to|color-stop)\((?:([\d.]+)(%)?,\s*)?(.+?)\)$/i,g=/^\s*(circle|ellipse)?\s*((?:([\d.]+)(px|r?em|%)\s*(?:([\d.]+)(px|r?em|%))?)|closest-side|closest-corner|farthest-side|farthest-corner)?\s*(?:at\s*(?:(left|center|right)|([\d.]+)(px|r?em|%))\s+(?:(top|center|bottom)|([\d.]+)(px|r?em|%)))?(?:\s|$)/i,E=t.GRADIENT_TYPE={LINEAR_GRADIENT:0,RADIAL_GRADIENT:1},m=t.RADIAL_GRADIENT_SHAPE={CIRCLE:0,ELLIPSE:1},T={left:new s.default("0%"),top:new s.default("0%"),center:new s.default("50%"),right:new s.default("100%"),bottom:new s.default("100%")},y=t.LinearGradient=function e(t,n){a(this,e),this.type=E.LINEAR_GRADIENT,this.colorStops=t,this.direction=n},v=t.RadialGradient=function e(t,n,r,o){a(this,e),this.type=E.RADIAL_GRADIENT,this.colorStops=t,this.shape=n,this.center=r,this.radius=o},b=(t.parseGradient=function(e,t,n){var r=t.args,a=t.method,o=t.prefix;return"linear-gradient"===a?I(r,n,!!o):"gradient"===a&&"linear"===r[0]?I(["to bottom"].concat(N(r.slice(3))),n,!!o):"radial-gradient"===a?_(e,"-webkit-"===o?P(r):r,n):"gradient"===a&&"radial"===r[0]?_(e,N(P(r.slice(1))),n):void 0},function(e,t,n){for(var r=[],a=t;a<e.length;a++){var o=e[a],i=h.test(o),l=o.lastIndexOf(" "),c=new u.default(i?o.substring(0,l):o),d=i?new s.default(o.substring(l+1)):a===t?new s.default("0%"):a===e.length-1?new s.default("100%"):null;r.push({color:c,stop:d})}for(var f=r.map(function(e){var t=e.color,r=e.stop;return{color:t,stop:0===n?0:r?r.getAbsoluteValue(n)/n:null}}),p=f[0].stop,g=0;g<f.length;g++)if(null!==p){var E=f[g].stop;if(null===E){for(var m=g;null===f[m].stop;)m++;for(var T=m-g+1,y=(f[m].stop-p)/T;g<m;g++)p=f[g].stop=p+y}else p=E}return f}),I=function(e,t,n){var r=(0,i.parseAngle)(e[0]),a=d.test(e[0]),o=a||null!==r||f.test(e[0]),u=o?null!==r?w(n?r-.5*Math.PI:r,t):a?O(e[0],t):S(e[0],t):w(Math.PI,t),l=o?1:0,s=Math.min((0,c.distance)(Math.abs(u.x0)+Math.abs(u.x1),Math.abs(u.y0)+Math.abs(u.y1)),2*t.width,2*t.height);return new y(b(e,l,s),u)},_=function(e,t,n){var r=t[0].match(g),a=r&&("circle"===r[1]||void 0!==r[3]&&void 0===r[5])?m.CIRCLE:m.ELLIPSE,o={},i={};r&&(void 0!==r[3]&&(o.x=(0,l.calculateLengthFromValueWithUnit)(e,r[3],r[4]).getAbsoluteValue(n.width)),void 0!==r[5]&&(o.y=(0,l.calculateLengthFromValueWithUnit)(e,r[5],r[6]).getAbsoluteValue(n.height)),r[7]?i.x=T[r[7].toLowerCase()]:void 0!==r[8]&&(i.x=(0,l.calculateLengthFromValueWithUnit)(e,r[8],r[9])),r[10]?i.y=T[r[10].toLowerCase()]:void 0!==r[11]&&(i.y=(0,l.calculateLengthFromValueWithUnit)(e,r[11],r[12])));var u={x:void 0===i.x?n.width/2:i.x.getAbsoluteValue(n.width),y:void 0===i.y?n.height/2:i.y.getAbsoluteValue(n.height)},s=R(r&&r[2]||"farthest-corner",a,u,o,n);return new v(b(t,r?1:0,Math.min(s.x,s.y)),a,u,s)},w=function(e,t){var n=t.width,r=t.height,a=.5*n,o=.5*r,i=(Math.abs(n*Math.sin(e))+Math.abs(r*Math.cos(e)))/2,u=a+Math.sin(e)*i,l=o-Math.cos(e)*i;return{x0:u,x1:n-u,y0:l,y1:r-l}},A=function(e){return Math.acos(e.width/2/((0,c.distance)(e.width,e.height)/2))},O=function(e,t){switch(e){case"bottom":case"to top":return w(0,t);case"left":case"to right":return w(Math.PI/2,t);case"right":case"to left":return w(3*Math.PI/2,t);case"top right":case"right top":case"to bottom left":case"to left bottom":return w(Math.PI+A(t),t);case"top left":case"left top":case"to bottom right":case"to right bottom":return w(Math.PI-A(t),t);case"bottom left":case"left bottom":case"to top right":case"to right top":return w(A(t),t);case"bottom right":case"right bottom":case"to top left":case"to left top":return w(2*Math.PI-A(t),t);case"top":case"to bottom":default:return w(Math.PI,t)}},S=function(e,t){var n=e.split(" ").map(parseFloat),r=o(n,2),a=r[0],i=r[1],u=a/100*t.width/(i/100*t.height);return w(Math.atan(isNaN(u)?1:u)+Math.PI/2,t)},L=function(e,t,n,r){return[{x:0,y:0},{x:0,y:e.height},{x:e.width,y:0},{x:e.width,y:e.height}].reduce(function(e,a){var o=(0,c.distance)(t-a.x,n-a.y);return(r?o<e.optimumDistance:o>e.optimumDistance)?{optimumCorner:a,optimumDistance:o}:e},{optimumDistance:r?1/0:-1/0,optimumCorner:null}).optimumCorner},R=function(e,t,n,r,a){var o=n.x,i=n.y,u=0,l=0;switch(e){case"closest-side":t===m.CIRCLE?u=l=Math.min(Math.abs(o),Math.abs(o-a.width),Math.abs(i),Math.abs(i-a.height)):t===m.ELLIPSE&&(u=Math.min(Math.abs(o),Math.abs(o-a.width)),l=Math.min(Math.abs(i),Math.abs(i-a.height)));break;case"closest-corner":if(t===m.CIRCLE)u=l=Math.min((0,c.distance)(o,i),(0,c.distance)(o,i-a.height),(0,c.distance)(o-a.width,i),(0,c.distance)(o-a.width,i-a.height));else if(t===m.ELLIPSE){var s=Math.min(Math.abs(i),Math.abs(i-a.height))/Math.min(Math.abs(o),Math.abs(o-a.width)),d=L(a,o,i,!0);l=s*(u=(0,c.distance)(d.x-o,(d.y-i)/s))}break;case"farthest-side":t===m.CIRCLE?u=l=Math.max(Math.abs(o),Math.abs(o-a.width),Math.abs(i),Math.abs(i-a.height)):t===m.ELLIPSE&&(u=Math.max(Math.abs(o),Math.abs(o-a.width)),l=Math.max(Math.abs(i),Math.abs(i-a.height)));break;case"farthest-corner":if(t===m.CIRCLE)u=l=Math.max((0,c.distance)(o,i),(0,c.distance)(o,i-a.height),(0,c.distance)(o-a.width,i),(0,c.distance)(o-a.width,i-a.height));else if(t===m.ELLIPSE){var f=Math.max(Math.abs(i),Math.abs(i-a.height))/Math.max(Math.abs(o),Math.abs(o-a.width)),h=L(a,o,i,!1);l=f*(u=(0,c.distance)(h.x-o,(h.y-i)/f))}break;default:u=r.x||0,l=void 0!==r.y?r.y:u}return{x:u,y:l}},P=t.transformWebkitRadialGradientArgs=function(e){var t="",n="",r="",a="",o=0,i=/^(left|center|right|\d+(?:px|r?em|%)?)(?:\s+(top|center|bottom|\d+(?:px|r?em|%)?))?$/i,u=/^\d+(px|r?em|%)?(?:\s+\d+(px|r?em|%)?)?$/i,l=e[o].match(i);l&&o++;var s=e[o].match(/^(circle|ellipse)?\s*(closest-side|closest-corner|farthest-side|farthest-corner|contain|cover)?$/i);s&&(t=s[1]||"","contain"===(r=s[2]||"")?r="closest-side":"cover"===r&&(r="farthest-corner"),o++);var c=e[o].match(u);c&&o++;var d=e[o].match(i);d&&o++;var f=e[o].match(u);f&&o++;var h=d||l;h&&h[1]&&(a=h[1]+(/^\d+$/.test(h[1])?"px":""),h[2]&&(a+=" "+h[2]+(/^\d+$/.test(h[2])?"px":"")));var p=f||c;return p&&(n=p[0],p[1]||(n+="px")),!a||t||n||r||(n=a,a=""),a&&(a="at "+a),[[t,r,n,a].filter(function(e){return!!e}).join(" ")].concat(e.slice(o))},N=function(e){return e.map(function(e){return e.match(p)}).map(function(t,n){if(!t)return e[n];switch(t[1]){case"from":return t[4]+" 0%";case"to":return t[4]+" 100%";case"color-stop":return"%"===t[3]?t[4]+" "+t[2]:t[4]+" "+100*parseFloat(t[2])+"%"}})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=/([+-]?\d*\.?\d+)(deg|grad|rad|turn)/i;t.parseAngle=function(e){var t=e.match(r);if(t){var n=parseFloat(t[1]);switch(t[2].toLowerCase()){case"deg":return Math.PI*n/180;case"grad":return Math.PI/200*n;case"rad":return n;case"turn":return 2*Math.PI*n}}return null}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0}),t.cloneWindow=t.DocumentCloner=void 0;var a=function(){return function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,a=!1,o=void 0;try{for(var i,u=e[Symbol.iterator]();!(r=(i=u.next()).done)&&(n.push(i.value),!t||n.length!==t);r=!0);}catch(e){a=!0,o=e}finally{try{!r&&u.return&&u.return()}finally{if(a)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}}(),o=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),i=n(1),u=n(23),l=r(n(50)),s=n(3),c=n(5),d=r(n(12)),f="data-html2canvas-ignore",h=t.DocumentCloner=function(){function e(t,n,r,a,o){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.referenceElement=t,this.scrolledElements=[],this.copyStyles=a,this.inlineImages=a,this.logger=r,this.options=n,this.renderer=o,this.resourceLoader=new l.default(n,r,window),this.documentElement=this.cloneNode(t.ownerDocument.documentElement)}return o(e,[{key:"inlineAllImages",value:function(e){var t=this;if(this.inlineImages&&e){var n=e.style;Promise.all((0,c.parseBackgroundImage)(n.backgroundImage).map(function(e){return"url"===e.method?t.resourceLoader.inlineImage(e.args[0]).then(function(e){return e&&"string"==typeof e.src?'url("'+e.src+'")':"none"}).catch(function(e){0}):Promise.resolve(""+e.prefix+e.method+"("+e.args.join(",")+")")})).then(function(e){e.length>1&&(n.backgroundColor=""),n.backgroundImage=e.join(",")}),e instanceof HTMLImageElement&&this.resourceLoader.inlineImage(e.src).then(function(t){if(t&&e instanceof HTMLImageElement&&e.parentNode){var n=e.parentNode,r=(0,s.copyCSSStyles)(e.style,t.cloneNode(!1));n.replaceChild(r,e)}}).catch(function(e){0})}}},{key:"inlineFonts",value:function(e){var t=this;return Promise.all(Array.from(e.styleSheets).map(function(t){return t.href?fetch(t.href).then(function(e){return e.text()}).then(function(e){return g(e,t.href)}).catch(function(e){return[]}):p(t,e)})).then(function(e){return e.reduce(function(e,t){return e.concat(t)},[])}).then(function(e){return Promise.all(e.map(function(e){return fetch(e.formats[0].src).then(function(e){return e.blob()}).then(function(e){return new Promise(function(t,n){var r=new FileReader;r.onerror=n,r.onload=function(){var e=r.result;t(e)},r.readAsDataURL(e)})}).then(function(t){return e.fontFace.setProperty("src",'url("'+t+'")'),"@font-face {"+e.fontFace.cssText+" "})}))}).then(function(n){var r=e.createElement("style");r.textContent=n.join("\n"),t.documentElement.appendChild(r)})}},{key:"createElementClone",value:function(e){var t=this;if(this.copyStyles&&e instanceof HTMLCanvasElement){var n=e.ownerDocument.createElement("img");try{return n.src=e.toDataURL(),n}catch(e){0}}if(e instanceof HTMLIFrameElement){var r=e.cloneNode(!1),a=L();r.setAttribute("data-html2canvas-internal-iframe-key",a);var o=(0,i.parseBounds)(e,0,0),u=o.width,l=o.height;return this.resourceLoader.cache[a]=P(e,this.options).then(function(e){return t.renderer(e,{async:t.options.async,allowTaint:t.options.allowTaint,backgroundColor:"#ffffff",canvas:null,imageTimeout:t.options.imageTimeout,logging:t.options.logging,proxy:t.options.proxy,removeContainer:t.options.removeContainer,scale:t.options.scale,foreignObjectRendering:t.options.foreignObjectRendering,useCORS:t.options.useCORS,target:new d.default,width:u,height:l,x:0,y:0,windowWidth:e.ownerDocument.defaultView.innerWidth,windowHeight:e.ownerDocument.defaultView.innerHeight,scrollX:e.ownerDocument.defaultView.pageXOffset,scrollY:e.ownerDocument.defaultView.pageYOffset},t.logger.child(a))}).then(function(t){return new Promise(function(n,a){var o=document.createElement("img");o.onload=function(){return n(t)},o.onerror=a,o.src=t.toDataURL(),r.parentNode&&r.parentNode.replaceChild((0,s.copyCSSStyles)(e.ownerDocument.defaultView.getComputedStyle(e),o),r)})}),r}return e.cloneNode(!1)}},{key:"cloneNode",value:function(e){var t=e.nodeType===Node.TEXT_NODE?document.createTextNode(e.nodeValue):this.createElementClone(e),n=e.ownerDocument.defaultView;this.referenceElement===e&&t instanceof n.HTMLElement&&(this.clonedReferenceElement=t),t instanceof n.HTMLBodyElement&&A(t);for(var r=e.firstChild;r;r=r.nextSibling)(r.nodeType!==Node.ELEMENT_NODE||"SCRIPT"!==r.nodeName&&!r.hasAttribute(f))&&(this.copyStyles&&"STYLE"===r.nodeName||t.appendChild(this.cloneNode(r)));if(e instanceof n.HTMLElement&&t instanceof n.HTMLElement)switch(this.inlineAllImages(m(e,t,v)),this.inlineAllImages(m(e,t,b)),!this.copyStyles||e instanceof HTMLIFrameElement||(0,s.copyCSSStyles)(e.ownerDocument.defaultView.getComputedStyle(e),t),this.inlineAllImages(t),0===e.scrollTop&&0===e.scrollLeft||this.scrolledElements.push([t,e.scrollLeft,e.scrollTop]),e.nodeName){case"CANVAS":this.copyStyles||E(e,t);break;case"TEXTAREA":case"SELECT":t.value=e.value}return t}}]),e}(),p=function(e,t){return(e.cssRules?Array.from(e.cssRules):[]).filter(function(e){return e.type===CSSRule.FONT_FACE_RULE}).map(function(e){for(var n=(0,c.parseBackgroundImage)(e.style.getPropertyValue("src")),r=[],a=0;a<n.length;a++)if("url"===n[a].method&&n[a+1]&&"format"===n[a+1].method){var o=t.createElement("a");o.href=n[a].args[0],t.body&&t.body.appendChild(o);var i={src:o.href,format:n[a+1].args[0]};r.push(i)}return{formats:r.filter(function(e){return/^woff/i.test(e.format)}),fontFace:e.style}}).filter(function(e){return e.formats.length})},g=function(e,t){var n=document.implementation.createHTMLDocument(""),r=document.createElement("base");r.href=t;var a=document.createElement("style");return a.textContent=e,n.head&&n.head.appendChild(r),n.body&&n.body.appendChild(a),a.sheet?p(a.sheet,n):[]},E=function(e,t){try{if(t){t.width=e.width,t.height=e.height;var n=e.getContext("2d"),r=t.getContext("2d");n?r.putImageData(n.getImageData(0,0,e.width,e.height),0,0):r.drawImage(e,0,0)}}catch(e){}},m=function(e,t,n){var r=e.ownerDocument.defaultView.getComputedStyle(e,n);if(r&&r.content&&"none"!==r.content&&"-moz-alt-content"!==r.content&&"none"!==r.display){var a=T(r.content),o=a.match(y),i=t.ownerDocument.createElement(o?"img":"html2canvaspseudoelement");return o?i.src=T(o[1]):i.textContent=a,(0,s.copyCSSStyles)(r,i),i.className=I+" "+_,t.className+=n===v?" "+I:" "+_,n===v?t.insertBefore(i,t.firstChild):t.appendChild(i),i}},T=function(e){var t=e.substr(0,1);return t===e.substr(e.length-1)&&t.match(/['"]/)?e.substr(1,e.length-2):e},y=/^url\((.+)\)$/i,v=":before",b=":after",I="___html2canvas___pseudoelement_before",_="___html2canvas___pseudoelement_after",w='{\n    content: "" !important;\n    display: none !important;\n}',A=function(e){O(e,"."+I+v+w+"\n         ."+_+b+w)},O=function(e,t){var n=e.ownerDocument.createElement("style");n.innerHTML=t,e.appendChild(n)},S=function(e){var t=a(e,3),n=t[0],r=t[1],o=t[2];n.scrollLeft=r,n.scrollTop=o},L=function(){return Math.ceil(Date.now()+1e7*Math.random()).toString(16)},R=/^data:text\/(.+);(base64)?,(.*)$/i,P=function(e,t){try{return Promise.resolve(e.contentWindow.document.documentElement)}catch(n){return t.proxy?(0,u.Proxy)(e.src,t).then(function(e){var t=e.match(R);return t?"base64"===t[2]?window.atob(decodeURIComponent(t[3])):decodeURIComponent(t[3]):Promise.reject()}).then(function(t){return N(e.ownerDocument,(0,i.parseBounds)(e,0,0)).then(function(e){var n=e.contentWindow.document;n.open(),n.write(t);var r=C(e).then(function(){return n.documentElement});return n.close(),r})}):Promise.reject()}},N=function(e,t){var n=e.createElement("iframe");return n.className="html2canvas-container",n.style.visibility="hidden",n.style.position="fixed",n.style.left="-10000px",n.style.top="0px",n.style.border="0",n.width=t.width.toString(),n.height=t.height.toString(),n.scrolling="no",n.setAttribute(f,"true"),e.body?(e.body.appendChild(n),Promise.resolve(n)):Promise.reject("")},C=function(e){var t=e.contentWindow,n=t.document;return new Promise(function(r,a){t.onload=e.onload=n.onreadystatechange=function(){var t=setInterval(function(){n.body.childNodes.length>0&&"complete"===n.readyState&&(clearInterval(t),r(e))},50)}})};t.cloneWindow=function(e,t,n,r,a,o){var i=new h(n,r,a,!1,o),u=e.defaultView.pageXOffset,l=e.defaultView.pageYOffset;return N(e,t).then(function(r){var a=r.contentWindow,o=a.document,s=C(r).then(function(){return i.scrolledElements.forEach(S),a.scrollTo(t.left,t.top),!/(iPad|iPhone|iPod)/g.test(navigator.userAgent)||a.scrollY===t.top&&a.scrollX===t.left||(o.documentElement.style.top=-t.top+"px",o.documentElement.style.left=-t.left+"px",o.documentElement.style.position="absolute"),i.clonedReferenceElement instanceof a.HTMLElement||i.clonedReferenceElement instanceof e.defaultView.HTMLElement||i.clonedReferenceElement instanceof HTMLElement?Promise.resolve([r,i.clonedReferenceElement,i.resourceLoader]):Promise.reject("")});return o.open(),o.write("<!DOCTYPE html><html></html>"),function(e,t,n){!e.defaultView||t===e.defaultView.pageXOffset&&n===e.defaultView.pageYOffset||e.defaultView.scrollTo(t,n)}(n.ownerDocument,u,l),o.replaceChild(o.adoptNode(i.documentElement),o.documentElement),o.close(),s})}},function(e,t,n){"use strict";function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}Object.defineProperty(t,"__esModule",{value:!0}),t.ResourceStore=void 0;var a=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=function(e){return e&&e.__esModule?e:{default:e}}(n(9)),i=n(23),u=function(){function e(t,n,a){r(this,e),this.options=t,this._window=a,this.origin=this.getOrigin(a.location.href),this.cache={},this.logger=n,this._index=0}return a(e,[{key:"loadImage",value:function(e){var t=this;if(this.hasResourceInCache(e))return e;if(!p(e)||o.default.SUPPORT_SVG_DRAWING){if(!0===this.options.allowTaint||f(e)||this.isSameOrigin(e))return this.addImage(e,e,!1);if(!this.isSameOrigin(e)){if("string"==typeof this.options.proxy)return this.cache[e]=(0,i.Proxy)(e,this.options).then(function(e){return g(e,t.options.imageTimeout||0)}),e;if(!0===this.options.useCORS&&o.default.SUPPORT_CORS_IMAGES)return this.addImage(e,e,!0)}}}},{key:"inlineImage",value:function(e){var t=this;return f(e)?g(e,this.options.imageTimeout||0):this.hasResourceInCache(e)?this.cache[e]:this.isSameOrigin(e)||"string"!=typeof this.options.proxy?this.xhrImage(e):this.cache[e]=(0,i.Proxy)(e,this.options).then(function(e){return g(e,t.options.imageTimeout||0)})}},{key:"xhrImage",value:function(e){var t=this;return this.cache[e]=new Promise(function(n,r){var a=new XMLHttpRequest;if(a.onreadystatechange=function(){if(4===a.readyState)if(200!==a.status)r("Failed to fetch image "+e.substring(0,256)+" with status code "+a.status);else{var t=new FileReader;t.addEventListener("load",function(){var e=t.result;n(e)},!1),t.addEventListener("error",function(e){return r(e)},!1),t.readAsDataURL(a.response)}},a.responseType="blob",t.options.imageTimeout){var o=t.options.imageTimeout;a.timeout=o,a.ontimeout=function(){return r("")}}a.open("GET",e,!0),a.send()}).then(function(e){return g(e,t.options.imageTimeout||0)}),this.cache[e]}},{key:"loadCanvas",value:function(e){var t=String(this._index++);return this.cache[t]=Promise.resolve(e),t}},{key:"hasResourceInCache",value:function(e){return void 0!==this.cache[e]}},{key:"addImage",value:function(e,t,n){var r=this;var a=function(e){return new Promise(function(a,o){var i=new Image;if(i.onload=function(){return a(i)},e&&!n||(i.crossOrigin="anonymous"),i.onerror=o,i.src=t,!0===i.complete&&setTimeout(function(){a(i)},500),r.options.imageTimeout){var u=r.options.imageTimeout;setTimeout(function(){return o("")},u)}})};return this.cache[e]=h(t)&&!p(t)?o.default.SUPPORT_BASE64_DRAWING(t).then(a):a(!0),e}},{key:"isSameOrigin",value:function(e){return this.getOrigin(e)===this.origin}},{key:"getOrigin",value:function(e){var t=this._link||(this._link=this._window.document.createElement("a"));return t.href=e,t.href=t.href,t.protocol+t.hostname+t.port}},{key:"ready",value:function(){var e=this,t=Object.keys(this.cache),n=t.map(function(t){return e.cache[t].catch(function(e){return null})});return Promise.all(n).then(function(e){return new l(t,e)})}}]),e}();t.default=u;var l=t.ResourceStore=function(){function e(t,n){r(this,e),this._keys=t,this._resources=n}return a(e,[{key:"get",value:function(e){var t=this._keys.indexOf(e);return-1===t?null:this._resources[t]}}]),e}(),s=/^data:image\/svg\+xml/i,c=/^data:image\/.*;base64,/i,d=/^data:image\/.*/i,f=function(e){return d.test(e)},h=function(e){return c.test(e)},p=function(e){return"svg"===e.substr(-3).toLowerCase()||s.test(e)},g=function(e,t){return new Promise(function(n,r){var a=new Image;a.onload=function(){return n(a)},a.onerror=r,a.src=e,!0===a.complete&&setTimeout(function(){n(a)},500),t&&setTimeout(function(){return r("")},t)})}}])});
// PAwidget Plugin
(function ($) {
	
  $.fn.PACalendar = function( options ) {

		var self = this;
			  
		// default options
    var settings = $.extend(true, {
			prevArrow:	 '&laquo;',
			nextArrow:	 '&raquo;',
			from: {
				date: moment(),
				element: $('[data-element="from"]')
			},	
			to:	{
				date: null,
				element: $('[data-element="to"]')
			},
			mode: 'date', // range
			rangeInterval: null, // work only for range mode, priority to "from" date
			limit_to: null,
			limit_from: null,
			locale: moment.locale(),
			format: 'YYYY-MM-DD',
			onPopulate: function(){}
		}, options );
		
		moment.locale(settings.locale);
								
		var _id = Math.random().toString(36).substring(7);
				
		// fixing values
		if (settings.from.element) {
			settings.from.date = moment(settings.from.element.val(), settings.format);
			if (!settings.from.date.isValid()) { settings.from.date = moment(); }
		}
		
		if (settings.to.element) {
		
			settings.to.date = moment(settings.to.element.val(), settings.format);
			if (!settings.to.date.isValid()) { settings.to.date = moment(); }
			settings.to.element.val(moment(settings.to.date).format(settings.format));
			if (settings.to.date.isBefore(settings.from.date)) { 
			    settings.to.element.val('');
				settings.to.date = null; 
			}
			
		    // YB : to refresh ng-model
			settings.to.element.trigger('input');
			settings.from.element.trigger('input');
		}
			
		var mode = true;
		var displayDate = moment(settings.startDate);
		var selectedDateFrom = moment(settings.startDate);
		var selectedDateTo = (settings.mode === 'range') ? moment(settings.to.date) : null;
		settings.from.element.addClass('PAactive');
		// elements
		var headerEl, headerPrevEl, headerNextEl, headerMonthEl, weekEl, monthEl;
				
		// methods
		function createStructure() {
			
			// header
			headerEl = $('<header></header>').addClass('PAheader');
			headerPrevEl = $('<span></span>').html(settings.prevArrow).addClass('PAHprev');
			headerNextEl = $('<span></span>').html(settings.nextArrow).addClass('PAHnext');
			headerMonthEl = $('<span></span>').addClass('PAHmonth');
			
		    // week
			weekEl = $('<section></section>').addClass('PAweek');
			//for (var i = 1; i < 8; i++) {
			for (var i = 0; i < 7; i++) { // YB : fix days names order + direction
			    weekEl.append($('<span></span>').text(moment().isoWeekday(i).format('dd')));
			}
			
			// month
			monthEl = $('<section></section>').addClass('PAmonth');
			
			headerEl
				.append(headerPrevEl)
				.append(headerMonthEl)
				.append(headerNextEl);
								
			self
				.addClass('PACalendar')
				.append(headerEl)	
				.append(weekEl)
				.append(monthEl)	;
				
			settings.from.element.addClass('PAcalendar_element').attr('rel', _id);
			settings.to.element.addClass('PAcalendar_element').attr('rel', _id);
			
		}

		function popolateCalendar(date) {

			var momentDate = moment(date);
			var current_month = momentDate.month();
			var current_year = momentDate.year();

			headerMonthEl.text(momentDate.format('MMMM YYYY')).attr('data-month', current_month).attr('data-year', current_year);
										
			// set start position from the first day of the week
			momentDate.set('date', 1);
			//momentDate.subtract((momentDate.isoWeekday() || 8) - 1, 'days');
			momentDate.subtract((momentDate.isoWeekday() || 8), 'days'); // YB: fix first day
			
			monthEl.find('span').remove();
			
		    for (var i = 0; i < 42; i++) {
				var dd = momentDate.date();
				var mm = momentDate.month();
				var value = moment(momentDate);
				
				var active = false;
				active = mm === current_month;
				if (settings.limit_to && active) { active = momentDate.startOf('day').isSameOrBefore(settings.limit_to.startOf('day')); }
				if (settings.limit_from && active) { active = momentDate.startOf('day').isSameOrAfter(settings.limit_from.startOf('day')); }
				
				var span = $('<span></span>')
										.text(dd)
										.attr('data-value', value.format('YYYYMMDD'))
										.toggleClass('PAactive', active);
				
				if (settings.mode === 'range') {
					span.toggleClass('PAfrom', selectedDateFrom.isSame(value));
					if (selectedDateTo && selectedDateTo.isValid()) {
						span.toggleClass('PAto', selectedDateTo.isSame(value));
					}
				}
				
				monthEl.append(span);
				momentDate.add(1, 'days');
			}
			
			cellsBetween();
			
			settings.onPopulate();
		}

		function cellsBetween() {
			
			monthEl
				.find('span')
				.removeClass('PAselected PAfrom PAto PAbetween');
			
			if (settings.mode === 'range' && selectedDateTo && selectedDateTo.isValid() && selectedDateFrom.isValid() && selectedDateFrom.isBefore(selectedDateTo)) {
				
				monthEl.find('span[data-value]')
					.each(function(i,e) {
						var d = moment($(e).attr('data-value'), 'YYYYMMDD');
						$(e).toggleClass('PAbetween', d.isAfter(selectedDateFrom) && d.isBefore(selectedDateTo));
					});
				
			}
			
			monthEl.find('span[data-value="'+ selectedDateFrom.format('YYYYMMDD') +'"]').removeClass('PAbetween').addClass('PAselected PAfrom');
			if (settings.mode === 'range' && selectedDateTo && selectedDateTo.isValid()) {
				monthEl.find('span[data-value="'+ selectedDateTo.format('YYYYMMDD') +'"]').removeClass('PAbetween').addClass('PAselected PAto');
			}
			
		}
		
		// fix TO value 
		function fixRange(fromTop) {

			fromTop = (typeof fromTop === 'undefined') ? false : true;

			if (settings.rangeInterval && settings.rangeInterval.length === 2 && settings.rangeInterval instanceof Array && settings.mode === 'range') {
				
				if (!fromTop) {
					selectedDateTo = moment(selectedDateFrom).add(settings.rangeInterval[0], settings.rangeInterval[1]).subtract(1,'d');
				} else {
					selectedDateFrom = moment(selectedDateTo).subtract(settings.rangeInterval[0], settings.rangeInterval[1]).add(1,'d');
				}
				
				if (!fromTop && settings.to.element) {
					settings.to.element.val(moment(selectedDateTo).format(settings.format));
				}
				if (fromTop && settings.from.element) {
					settings.from.element.val(moment(selectedDateFrom).format(settings.format));				
				}
				
			}

		    // YB : to refresh ng-model
			settings.to.element.trigger('input');
			settings.from.element.trigger('input');

		}
		
		fixRange();
		createStructure();
		popolateCalendar(displayDate);	
		monthEl.addClass(mode ? 'PAfrom' : 'PAto');
		
		headerNextEl
			.off('click')
			.on('click', function() {

			    displayDate.subtract(1, 'months');
			    //displayDate.add(1, 'months');
				popolateCalendar(displayDate);
				
			});
			
		headerPrevEl
			.off('click')
			.on('click', function() {

			    displayDate.add(1, 'months');
			    //displayDate.subtract(1, 'months');
				popolateCalendar(displayDate);
				
			});
		
		monthEl
			.off('click', 'span.PAactive')
			.on('click', 'span.PAactive', function() {
				
					
					var modeClass = mode ? 'PAto' : 'PAfrom';
					monthEl.find('span:not(.'+modeClass+')').removeClass('PAselected');
					$(this).addClass('PAselected');
					var selectedDate = moment($(this).attr('data-value'), 'YYYYMMDD');
					
					if (mode) {
						selectedDateFrom = selectedDate;
						if (settings.from.element) {
							settings.from.element.val(moment(selectedDate, 'YYYYMMDD').format(settings.format));
						}
					}
									
					if (settings.mode === 'range') {
						
						if (!mode) {
							selectedDateTo = selectedDate;
							if (settings.to.element) {
								settings.to.element.val(moment(selectedDate, 'YYYYMMDD').format(settings.format));
							}
						}
						
						fixRange();
						
						monthEl.find('span').removeClass(mode ? 'PAfrom' : 'PAto');
						$(this).addClass(mode ? 'PAfrom' : 'PAto');
						
						if (selectedDateTo < selectedDateFrom) {
							
							monthEl.find('span').removeClass('PAselected PAbetween PAfrom PAto');
							$(this).addClass('PAselected PAfrom');
							selectedDateFrom = selectedDate;
							if (settings.from.element) {
								settings.from.element.val(moment(selectedDate, 'YYYYMMDD').format(settings.format));
							}
							if (settings.to.element) {
								settings.to.element.val('');
							}
							mode = false;
							
						} else {
							mode = !mode;
						}
						
						if (settings.rangeInterval && settings.rangeInterval.length === 2 && settings.rangeInterval instanceof Array) {
							mode = true;	
						}
						
					}
					
					if (settings.mode === 'range' && selectedDateFrom && selectedDateTo) {
						self.trigger({
							type: 'setDate', 
							from: moment(selectedDateFrom),
							to: moment(selectedDateTo)
						});
					}
	
					if (settings.mode !== 'range' && selectedDateFrom) {
						self.trigger({
							type: 'setDate',
							from: moment(selectedDateFrom)
						});
					}
					
					
					$('.PAcalendar_element[rel="'+ _id +'"]').removeClass('PAactive');
					settings[mode ? 'from' : 'to'].element.addClass('PAactive');
									
					cellsBetween();
				
			        // YB : to refresh ng-model
					settings.to.element.trigger('input');
					settings.from.element.trigger('input');
				
			});
			
		if (settings.from.element) {
			
			settings.from.element
				.off('change keyup')
				.on('change keyup', function() {
					
					self.setDateFrom($(this).val());

				})
				.off('focus')
				.on('focus', function() {
					mode = true;
					$('.PAcalendar_element[rel="'+ _id +'"]').removeClass('PAactive');
					settings.from.element.addClass('PAactive');
				});
			
			}
			
		if (settings.to.element) {
			settings.to.element
				.off('change keyup')
				.on('change keyup', function() {
					
					self.setDateTo($(this).val());

				})
				.off('focus')
				.on('focus', function() {
					mode = (settings.rangeInterval && settings.rangeInterval.length === 2 && settings.rangeInterval instanceof Array && settings.mode === 'range') ? true: false;
					$('.PAcalendar_element[rel="'+ _id +'"]').removeClass('PAactive');
					settings.to.element.addClass('PAactive');
				});

			
		}
		
		// EVENTS
		// change mode
		self.setMode = function(newMode) {
			if ((newMode === 'range' || newMode === 'date') && settings.mode !== newMode) {
				settings.mode = newMode;
				
				if (settings.mode === 'date') { selectedDateTo = null; }
				fixRange();
				popolateCalendar(selectedDateFrom);	
				return true;
			} else {
				return false;
			}
		};
		
		// change from date
		self.setDateFrom = function(changeDate, doTrigger) {
			
			doTrigger = (typeof doTrigger == 'boolean') ? doTrigger : true;
			var date = moment(changeDate, settings.format);
			if (date.isValid()) {
				monthEl.find('span:not(.PAto)').removeClass('PAselected PAfrom');
				monthEl.find('span[data-value="'+ date.format('YYYYMMDD') +'"]').addClass('PAselected PAfrom');
				selectedDateFrom = moment(date);
				
				fixRange();
				cellsBetween();
				
				if (doTrigger) {
					if (settings.mode === 'range' && selectedDateFrom && selectedDateTo) {
						self.trigger({
							type: 'setDate', 
							from: moment(selectedDateFrom),
							to: moment(selectedDateTo)
						});
					}
	
					if (settings.mode !== 'range' && selectedDateFrom) {
						self.trigger({
							type: 'setDate',
							from: moment(selectedDateFrom)
						});
					}
				}
				
				return true;
			} else {
				return false;
			}
			
		};
		
		// change to date
		self.setDateTo = function(changeDate, doTrigger) {
			
			doTrigger = (typeof doTrigger == 'boolean') ? doTrigger : true;			
			var date = moment(changeDate, settings.format);
			if (date.isValid()) {
				monthEl.find('span:not(.PAfrom)').removeClass('PAselected PAto');
				monthEl.find('span[data-value="'+ date.format('YYYYMMDD') +'"]').addClass('PAselected PAto');
				selectedDateTo = moment(date);
				
				fixRange(true);
				cellsBetween();
				
				if (doTrigger && settings.mode === 'range' && selectedDateFrom && selectedDateTo) {
					self.trigger({
						type: 'setDate', 
						from: moment(selectedDateFrom),
						to: moment(selectedDateTo)
					});
				}
				
				return true;
			} else {
				return false;
			}
			
		};
		
		// set range interval
		self.setRangeInterval = function(interval) {
			
			settings.rangeInterval = interval;
			
			fixRange();
			cellsBetween();
			
		};
		
		// change both dates
		self.setDate = function(changeFrom, changeTo) {
			return (self.setDateFrom(changeFrom) && self.setDateTo(changeTo));
		};
		
		return self;
				
	};

}( jQuery ));

angular.module('templates-angularwizard', []);

angular.module('mgo-angular-wizard', ['templates-angularwizard']);

//wizard directive
angular.module('mgo-angular-wizard').directive('wizard', function () {
    var defaultSetting = {
        theme: "web"
    };

    var themes = {
        "web": "../libs/angular-wizard/wizard.html",
        "web-new": "../libs/angular-wizard/wizard.web-new.html",
        "mobile": "../libs/angular-wizard/wizard.mobile.html",
        "mobile-new": "../libs/angular-wizard/wizard.mobile-new.html",
        "error": "../libs/angular-wizard/wizard.error.html"
    }

    return {
        restrict: 'EA',
        replace: true,
        transclude: {
            "steps": "wzStep",
            "footerContent": "?wzFooterContent",
            "sideContent": "?wzSideContent"
        },
        scope: {
            currentStep: '=',
            onFinish: '&',
            hideIndicators: '=',
            hideFooter: '=',
            editMode: '=',
            name: '@',
            enableButton: '=?',
            showOtp: '=',
            onShowOtp: '&',
            theme: '=',
            templateUrl: '=',
            approvalText: '=?',
            hideCompeleteButtons: '=',
            /* wizard OTP not in use in london and mobile */
            showOtpImbedded: '=',
            onShowOtpAlert: '&',
            isClosePopupButton: '=?',
            isHodatTashlom: '=?',
            isActionCompeleteSuccessfuly: '=?'
        },
        templateUrl: function (element, attributes) {
            if (attributes.templateUrl) {
                return attributes.templateUrl;
            } else if (attributes.theme) {
                if (themes[attributes.theme]) {
                    return themes[attributes.theme];
                } else {
                    console.error("wizard theme '", attributes.theme, "' not found!");
                    return themes["error"];
                }
            } else {
                return themes[defaultSetting.theme];
            }

        },

        controller: ['$scope', '$rootScope', '$element', '$log', 'WizardHandler', '$q', '$compile', function ($scope, $rootScope, $element, $log, WizardHandler, $q, $compile) {
            var self = this;
            var sideContentIndex = 0;
            var footerContentIndex = 0;
            var firstRun = true;//this variable allows directive to load without having to pass any step validation

            function init() {
                $scope._id = "wizard";
                $scope.steps = [];//steps array where all the scopes of each step are added
                $scope.context = {};//access to context object for step validation

                $scope.otpStatus = { isAllowToleaveStep: false };
                $scope.displayOtpUI = true;
                $scope.isConfirmStep = false;

                this.context = $scope.context;

                if ($scope.enableButton === undefined) {
                    $scope.enableButton = true;
                }

                //creating instance of wizard, passing this as second argument allows access to functions attached to this via Service
                WizardHandler.addWizard($scope.name || WizardHandler.defaultName, self);

                $scope.$on('$destroy', function () {
                    WizardHandler.removeWizard($scope.name || WizardHandler.defaultName);
                });

                //watching changes to currentStep
                $scope.$watch('currentStep', function (step) {
                    //checking to make sure currentStep is truthy value
                    if (!step) return;

                    //setting stepTitle equal to current step title or default title
                    var stepTitle = $scope.selectedStep.wzTitle;
                    if ($scope.selectedStep && stepTitle !== $scope.currentStep) {
                        //invoking goTo() with step title as argument
                        $scope.goTo(stepByTitle($scope.currentStep));
                    }

                });

                //watching steps array length and editMode value, if edit module is undefined or null the nothing is done
                //if edit mode is truthy, then all steps are marked as completed
                $scope.$watch('[editMode, steps.length]', function () {
                    var editMode = $scope.editMode;
                    if (angular.isUndefined(editMode) || (editMode === null)) return;

                    if (editMode) {
                        angular.forEach($scope.getEnabledSteps(), function (step) {
                            step.completed = true;
                        });
                    } else {
                        var completedStepsIndex = $scope.currentStepNumber() - 1;
                        angular.forEach($scope.getEnabledSteps(), function (step, stepIndex) {
                            if (stepIndex >= completedStepsIndex) {
                                step.completed = false;
                            }
                        });
                    }
                }, true);



                /* Init OTP stuff. TBD: need refactor*/
                $scope.displayOtpUI = true;
                if ($scope.isClosePopupButton === undefined) {
                    $scope.isClosePopupButton = false;
                }

                $scope.$watch('otpStatus.isAllowToleaveStep', function (value) {
                    if (value === true) {
                        $scope.displayOtpUI = false;
                    }
                });

                $scope.$watch('showOtp', function (value) {
                    if (value === true) {
                        $scope.displayOtpUI = true;

                        /* TBD: rewrite this, should not access directly to document */

                        if ($scope.showOtpImbedded && document.getElementById("wizOtpPlaceholder")) { // going to confirm step
                            angular.element(document.getElementById("wizOtpPlaceholder")).append($compile(
                                "<div class='col-xs-12 steps' ng-if='displayOtpUI && showOtp && showOtpImbedded && isConfirmStep' data-sky-otp-directive  data-l-show-otp-imbedded='true' data-is-allow-toleave-step='otpStatus.isAllowToleaveStep'></div>"
                            )($scope));
                        } else if ($scope.showOtpImbedded && !document.getElementById("wizOtpPlaceholder")) {
                            console.error("Can't display OTP - no wizOtpPlaceholder found - fallback used !");
                            angular.element(document.getElementById("wizOtpPlaceholderFallback")).append($compile(
                                "<div class='col-xs-12 steps' ng-if='displayOtpUI && showOtp && showOtpImbedded && isConfirmStep' data-sky-otp-directive  data-l-show-otp-imbedded='true' data-is-allow-toleave-step='otpStatus.isAllowToleaveStep'></div>"
                            )($scope));
                        }
                    } else {
                        $scope.displayOtpUI = false;
                    }
                });

                $scope.$watchCollection('steps', function (value) {
                    if (value) {
                        $scope.isConfirmStep = $scope.currentStepNumber() + 1 === $scope.getEnabledSteps().length;
                    }
                });
            }

            function stepIdx(step) {
                var idx = 0;
                var res = -1;
                angular.forEach($scope.getEnabledSteps(), function (currStep) {
                    if (currStep === step) {
                        res = idx;
                    }
                    idx++;
                });

                return res;
            };

            function stepByTitle(titleToFind) {
                var foundStep = null;
                angular.forEach($scope.getEnabledSteps(), function (step) {
                    if (step.wzTitle === titleToFind) {
                        foundStep = step;
                    }
                });

                return foundStep;
            };

            function continueNextStep(step, thisStep, removeOtpFlag) {
                if($scope.inTransition){
                    return
                }

                //$log.log('steps[thisStep] Data: ', $scope.getEnabledSteps()[thisStep].canexit);
                $scope.inTransition = true;

                var canExitStepPromise = canExitStep($scope.getEnabledSteps()[thisStep], step);
                var canEnterStepPromise = canEnterStep(step);

                $q.all([canExitStepPromise, canEnterStepPromise]).then(
                    function ok(data) {
                        if (data[0] && data[1]) {
                            //deselect all steps so you can set fresh below
                            unselectAll();

                            //$log.log('value for canExit argument: ', $scope.currentStep.canexit);
                            $scope.selectedStep = step;
                            //making sure current step is not undefined
                            if (!angular.isUndefined($scope.currentStep)) {
                                $scope.currentStep = step.wzTitle;
                            }

                            if (removeOtpFlag) {
                                $scope.showOtp = false;
                            }

                            //setting selected step to argument passed into goTo()
                            step.selected = true;
                            //emit event upwards with data on goTo() invoktion
                            $scope.$emit('wizard:stepChanged', { step: step, index: stepIdx(step), length: $scope.steps.length });
                            $scope.$broadcast('wizard:stepChanged', { step: step, index: stepIdx(step), length: $scope.steps.length });
                            //$log.log('current step number: ', $scope.currentStepNumber());

                            $scope.isConfirmStep = $scope.currentStepNumber() + 1 === $scope.getEnabledSteps().length;

                            setTimeout(function () {
                                $('li.current a').focus();
                            }, 0);
                        }

                        $scope.inTransition = false;
                    },
                    function fail(err) {
                        $scope.inTransition = false;
                        console.warn("wizrd:continueNextStep fail! ", err);
                    }
                ).finally(function(){
                    $scope.inTransition = false;
                });
            }

            function closePopUpWizard() {
                $scope.$emit("close-self");
            }

            function pdfExport() {
                $scope.$emit('pdfExport');
            }

            function hodatTashlom() {
                // $scope.$emit('hodatTashlom');
                //paymentConfirmationLink = string.Format("{0}?OrderID={1}&echo={2}",
                //                           paymentConfirmationLink,
                //                           tehilaPaymentNumber,
                //                           echo);

                //$window.open(paymentConfirmationLink, '_blank', 'width=1000, height=800', 'resizable = 1', 'scrollbars = 1');
            }

            function print() {
                $rootScope.$broadcast("print-page");
                //$rootScope.$broadcast("tadpis");
            }

            function canEnterStep(step) {
                var defer,
                    canEnter;

                //If no validation function is provided, allow the user to enter the step
                if (step.canenter === undefined) {
                    return true;
                }

                //If canenter is a boolean value instead of a function, return the value
                if (typeof step.canenter === 'boolean') {
                    return step.canenter;
                }

                //Check to see if the canenter function is a promise which needs to be returned
                canEnter = step.canenter($scope.context);
                if (angular.isFunction(canEnter.then)) {
                    return canEnter;
                } else {
                    return canEnter === true;
                }
            }

            function canExitStep(step, stepTo) {
                var defer,
                    canExit;

                //Exiting the step should be allowed if no validation function was provided or if the user is moving backwards
                if (typeof (step.canexit) === 'undefined' || $scope.getStepNumber(stepTo) < $scope.currentStepNumber()) {
                    return true;
                }

                //If canexit is a boolean value instead of a function, return the value
                if (typeof step.canexit === 'boolean') {
                    return step.canexit;
                }

                //Check to see if the canexit function is a promise which needs to be returned
                canExit = step.canexit($scope.context);
                if (angular.isFunction(canExit.then)) {
                    return canExit
                } else {
                    return canExit === true;
                }
            }

            //unSelect All Steps
            function unselectAll() {
                //traverse steps array and set each "selected" property to false
                angular.forEach($scope.getEnabledSteps(), function (step) {
                    step.selected = false;
                });
                //set selectedStep variable to null
                $scope.selectedStep = null;
            }

            function getStepNumber(step) {
                return stepIdx(step) + 1;
            };

            function goTo(step) {
                //if this is the first time the wizard is loading it bi-passes step validation
                if (firstRun) {
                    //deselect all steps so you can set fresh below
                    unselectAll();
                    $scope.selectedStep = step;
                    //making sure current step is not undefined
                    if (!angular.isUndefined($scope.currentStep)) {
                        $scope.currentStep = step.wzTitle;
                    }
                    //setting selected step to argument passed into goTo()
                    step.selected = true;
                    //emit event upwards with data on goTo() invoktion
                    $scope.$emit('wizard:stepChanged', { step: step, index: stepIdx(step), length: $scope.steps.length });
                    $scope.$broadcast('wizard:stepChanged', { step: step, index: stepIdx(step), length: $scope.steps.length });

                    //setting variable to false so all other step changes must pass validation
                    firstRun = false;
                } else if ($scope.currentStepNumber() < $scope.getEnabledSteps().length) {
                    //createing variables to capture current state that goTo() was invoked from and allow booleans
                    var thisStep;
                    //getting data for step you are transitioning out of
                    if ($scope.currentStepNumber() > 0) {
                        thisStep = $scope.currentStepNumber() - 1;
                    } else if ($scope.currentStepNumber() === 0) {
                        thisStep = 0;
                    }

                    /// ----- OTP start -------///

                    var enabledSteps = $scope.getEnabledSteps();

                    if ($scope.currentStepNumber() > $scope.getStepNumber(step)) { // back
                        continueNextStep(step, thisStep);
                    } else {
                        if ($scope.currentStepNumber() + 1 === enabledSteps.length) { // confirmation step
                            if ($scope.showOtp === true) {
                                if ($scope.showOtpImbedded) {
                                    if ($scope.otpStatus.isAllowToleaveStep === true) {
                                        continueNextStep(step, thisStep, true);
                                        if (stepIdx(step) === $scope.steps.length - 1 && $scope.steps.length !== 1) {
                                            for (var i = 0; i < $scope.steps.length; i++) {
                                                if (i != stepIdx(step)) {
                                                    $scope.steps[i].canenter = false;
                                                }
                                            }
                                        }
                                    }
                                    else {
                                        if ($scope.onShowOtpAlert) {
                                            $scope.onShowOtpAlert();
                                        }
                                    }
                                }
                                else {
                                    if ($scope.onShowOtp) { // action defined
                                        $scope.onShowOtp().then(function (result) {
                                            //skyOtpService.initOtp(2).then(function (result) {
                                            if (result.isAllowToleaveStep === true) {
                                                continueNextStep(step, thisStep, true);
                                                if (stepIdx(step) === $scope.steps.length - 1 && $scope.steps.length !== 1) {
                                                    for (var i = 0; i < $scope.steps.length; i++) {
                                                        if (i != stepIdx(step)) {
                                                            $scope.steps[i].canenter = false;
                                                        }
                                                    }
                                                }
                                            }
                                            else {
                                                // nothing
                                            }
                                        }, function (error) {
                                            // nothing
                                        });
                                    }
                                }
                            }
                            else {
                                continueNextStep(step, thisStep);
                            }
                        }
                        else {
                            continueNextStep(step, thisStep);
                        }
                    }

                    /// ----- OTP end -------///
                } else {
                    console.log("mgo-angular-wizard: Can't goto step on last step!");
                }
            };

            function currentStepNumber() {
                //retreive current step number
                return stepIdx($scope.selectedStep) + 1;
            };

            function accStepNum(stepNum, sumSteps, isCurrent, title, from, step, cureentStep) {
                if (isCurrent) {
                    setTimeout(function () {
                        $('.steps-indicator .current').focus();
                    }, 100);
                    return cureentStep + ' ' + stepNum + ' ' + from + ' ' + sumSteps + ' ' + title;
                } else {
                    return step + ' ' + stepNum + ' ' + from + ' ' + sumSteps + ' ' + title;
                }
            }

            function getEnabledSteps() {
                return $scope.steps.filter(function (step) {
                    return step.disabled !== 'true';
                });
            };

            function getEnabledStepsMinusLast() {
                var enabledSteps = getEnabledSteps();
                var dupArr = enabledSteps.slice();//duplicate

                return dupArr.splice(0, enabledSteps.length - 1);
            };

            //called each time step directive is loaded
            function addStep(step) {
                //pushing the scope of directive onto step array
                $scope.steps.push(step);
                //if this is first step being pushed then goTo that first step
                if ($scope.getEnabledSteps().length === 1) {
                    $scope.goTo($scope.getEnabledSteps()[0]);
                }
            };

            //generate index for side content
            function getSideContentIndex() {
                $scope.hasSideContent = true;
                return sideContentIndex++;
            }

            //generate index for footer content
            function getFooterContentIndex() {
                return footerContentIndex++;
            }

            function currentStepTitle() {
                return $scope.selectedStep.wzTitle;
            };

            function currentStepDescription() {
                return $scope.selectedStep.description;
            };

            function currentStep() {
                return $scope.selectedStep;
            };

            function totalStepCount() {
                return $scope.getEnabledSteps().length;
            }

            //Access to enabled steps from outside
            function getEnabledSteps2() {
                return $scope.getEnabledSteps();
            };

            //Access to current step number from outside
            function currentStepNumber2() {
                return $scope.currentStepNumber();
            };

            //method used for next button within step
            function next(callback) {
                var enabledSteps = $scope.getEnabledSteps();
                //setting variable equal to step  you were on when next() was invoked
                var index = stepIdx($scope.selectedStep);
                //checking to see if callback is a function

                if (angular.isFunction(callback)) {
                    if (callback()) {
                        if (index === enabledSteps.length - 1) {
                            this.finish();
                        } else {
                            //invoking goTo() with step number next in line
                            $scope.goTo(enabledSteps[index + 1]);
                        }
                    } else {
                        return;
                    }
                }

                if (!callback) {
                    //completed property set on scope which is used to add class/remove class from progress bar
                    $scope.selectedStep.completed = true;
                }
                
                //checking to see if this is the last step.  If it is next behaves the same as finish()
                if (index === enabledSteps.length - 1) {
                    this.finish();
                } else {
                    //invoking goTo() with step number next in line
                    $scope.goTo(enabledSteps[index + 1]);
                }

                //acc : checking to see if this is the last step. focus on main title.
                if (index === enabledSteps.length - 2) {
                    setTimeout(function () {
                        $('.main-title').focus();
                    }, 0);
                }
            };

            //used to traverse to any step, step number placed as argument
            function goTo2(step) {
                var enabledSteps = $scope.getEnabledSteps();
                var stepTo;
                //checking that step is a Number
                if (angular.isNumber(step)) {
                    stepTo = enabledSteps[step];
                } else {
                    //finding the step associated with the title entered as goTo argument
                    stepTo = stepByTitle(step);
                }
                //going to step
                $scope.goTo(stepTo);
            };

            //calls finish() which calls onFinish() which is declared on an attribute and linked to controller via wizard directive.
            function finish() {
                if ($scope.onFinish) {
                    $scope.onFinish();
                }
            };

            function previous() {
                //getting index of current step
                var index = stepIdx($scope.selectedStep);
                //ensuring you aren't trying to go back from the first step or last step
                if (index === 0 || index >= $scope.getEnabledSteps().length - 1) {
                    console.log("mgo-angular-wizard: Can't go back. on first step or last step!");
                } else {
                    //go back one step from current step
                    $scope.goTo($scope.getEnabledSteps()[index - 1]);
                }
            };

            //cancel is alias for previous.
            function cancel() {
                //getting index of current step
                var index = stepIdx($scope.selectedStep);
                //ensuring you aren't trying to go back from the first step
                if (index === 0) {
                    throw new Error("Can't go back. It's already in step 0");
                } else {
                    //go back one step from current step
                    $scope.goTo($scope.getEnabledSteps()[0]);
                }
            };

            //reset
            function reset() {
                //traverse steps array and set each "completed" property to false
                angular.forEach($scope.getEnabledSteps(), function (step) {
                    step.completed = false;
                });
                //go to first step
                this.goTo(0);
            };

            /* ALL METHODS ATTACHED TO this ARE ACCESSIBLE VIA WizardHandler.wizard().methodName() */
            this.addStep = addStep;
            this.getSideContentIndex = getSideContentIndex;
            this.getFooterContentIndex = getFooterContentIndex;
            this.currentStepTitle = currentStepTitle;
            this.currentStepDescription = currentStepDescription;
            this.currentStep = currentStep;
            this.totalStepCount = totalStepCount;
            this.getEnabledSteps = getEnabledSteps2;
            this.currentStepNumber = currentStepNumber2;
            this.next = next;
            this.goTo = goTo2;
            this.finish = finish;
            this.previous = previous;
            this.cancel = cancel;
            this.reset = reset;
            //this.accStepNum = accStepNum;

            $scope.getStepNumber = getStepNumber;
            $scope.goTo = goTo;
            $scope.currentStepNumber = currentStepNumber;
            $scope.getEnabledSteps = getEnabledSteps;
            $scope.getEnabledStepsMinusLast = getEnabledStepsMinusLast;
            $scope.pdfExport = pdfExport;
            $scope.hodatTashlom = hodatTashlom;
            $scope.print = print;
            $scope.hasSideContent = false;
            $scope.stepIdx = stepIdx;
            $scope.accStepNum = accStepNum;

            $scope.closePopUpWizard = closePopUpWizard;//OTP

            $scope.inTransition = false;

            self.$onInit = init;
        }]
    };
});
angular.module('mgo-angular-wizard').factory('WizardHandler', function() {
   var service = {};
   
   var wizards = {};
   
   service.defaultName = "defaultWizard";
   
   service.addWizard = function(name, wizard) {
       wizards[name] = wizard;
   };
   
   service.removeWizard = function(name) {
       delete wizards[name];
   };
   
   service.wizard = function(name) {
       var nameToUse = name;
       if (!name) {
           nameToUse = service.defaultName;
       }
       
       return wizards[nameToUse];
   };
   
   return service;
});

angular.module('mgo-angular-wizard').directive('wzStep', function () {
    return {
        restrict: 'EA',
        replace: true,
        transclude: true,
        scope: {
            wzTitle: '@',
            wzCustomBackButtonText: '@',
            wzCustomNextButtonText: '@',
            hideNext: '=',
            hidePrev: '=',
            canenter: '=',
            canexit: '=',
            disabled: '@?wzDisabled',
            description: '@',
            wzData: '=',
            wzFormName: '@'
        },
        require: '^wizard',
        templateUrl: function (element, attributes) {
            return attributes.template || "../libs/angular-wizard/step.html";
        },
        link: function ($scope, $element, $attrs, wizard) {
            $scope._id = "step";
            //$scope.forms = {};
            $scope.title = $scope.wzTitle;
            wizard.addStep($scope);
            //$scope.$on("formSubmit", function (args) {
            //});
        }
    };
});

angular.module('mgo-angular-wizard').directive('wzSideContent', function () {
    return {
        restrict: 'EA',
        replace: true,
        transclude: true,
        scope: {
        },
        require: '^wizard',
        template: '<section ng-show="selected" class="wzSideContent" ng-transclude></section>',
        link: function ($scope, $element, $attrs, wizard) {
            $scope._id = "wzSideContent";
            $scope.index = wizard.getSideContentIndex();
            $scope.selected = false;

            $scope.$on('wizard:stepChanged', function (evt, arg) {
                $scope.selected = ($scope.index == arg.index);
            });
        }
    };
});

angular.module('mgo-angular-wizard').directive('wzFooterContent', function () {
    return {
        restrict: 'EA',
        replace: true,
        transclude: true,
        scope: {
        },
        require: '^wizard',
        template: '<section ng-show="selected" class="wzFooterContent" ng-transclude></section>',
        link: function ($scope, $element, $attrs, wizard) {
            $scope._id = "wzFooterContent";
            $scope.index = wizard.getFooterContentIndex();
            $scope.selected = false;

            $scope.$on('wizard:stepChanged', function (evt, arg) {
                $scope.selected = ($scope.index == arg.index);
            });
        }
    };
});

function wizardButtonDirective(action) {
    angular.module('mgo-angular-wizard').directive(action, function () {

        return {
            restrict: 'A',
            replace: false,
            require: '^wizard',
            link: function ($scope, $element, $attrs, wizard) {
                //$scope._id = action;
                $element.on("click", function (e) {
                    e.preventDefault();

                    var isFromStep = ($scope.$parent._id !== "wizard");

                    if (isFromStep) {//from step
                        var formScope = $scope;
                        var stepScope = $scope.$parent;
                        var formName = stepScope.wzFormName;//step.wzFormName
                        var form = formScope[formName];
                        $scope.$emit(action, { form: form });
                    } else {//from footer
                        var wizardScope = $scope.$parent;
                        var stepScope = wizardScope.selectedStep;
                        var formScope = stepScope.$$childHead;     //TBD refine   !!!!!!!!!!!!!!!!!!!!!!!!!!!
                        var formName = stepScope.wzFormName;//step.wzFormName
                        var form = formScope[formName];
                        $scope.$emit(action, { form: form });
                    }

                    if (form) {
                        form.$submitted = true;
                        $("form[name='" + formName + "']").addClass("ng-submitted");
                    }

                    $scope.$apply(function () {
                        if ((action !== 'wzNext' && action !== 'wzFinish') || !(formName) || (form && form.$valid) || !form) {
                            $scope.$eval($attrs[action]);
                            wizard[action.replace("wz", "").toLowerCase()]();
                        }
                    });
                });
            }
        };
    });
}

wizardButtonDirective('wzNext');
wizardButtonDirective('wzPrevious');
wizardButtonDirective('wzFinish');
wizardButtonDirective('wzCancel');
wizardButtonDirective('wzReset');
